<!doctype html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta
      name="viewport"
      content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=0"
    />
    <title>ubo - web-ui</title>
  </head>

  <script type="module">
    window.GRPC_ENVOY_LISTEN_PORT = "{{ GRPC_ENVOY_LISTEN_PORT }}";

    import { init } from "{{ url_for('static', filename='main.js') }}";
    init({{ state | tojson }});
  </script>

  <body>
    <div id="web-app-root"></div>
  </body>
</html>
