"""Mixin for AI providers that require setup."""

from ubo_app.engines.abstraction.engine import EngineMixin


class AIProviderMixin(EngineMixin):
    """Base class for AI providers that require setup."""
