/*
 * ATTENTION: The "eval" devtool has been used (maybe by default in mode: "development").
 * This devtool is neither made for production nor for readable output files.
 * It uses "eval()" calls to create a separate source file in the browser devtools.
 * If you are trying to read the output file, select a different devtool (https://webpack.js.org/configuration/devtool/)
 * or disable the default devtool with "devtool: false".
 * If you are looking for production-ready output files, see mode: "production" (https://webpack.js.org/configuration/mode/).
 */
/******/ var __webpack_modules__ = ({

/***/ "./node_modules/@babel/runtime/helpers/esm/assertThisInitialized.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@babel/runtime/helpers/esm/assertThisInitialized.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ _assertThisInitialized)\n/* harmony export */ });\nfunction _assertThisInitialized(e) {\n  if (void 0 === e) throw new ReferenceError(\"this hasn't been initialised - super() hasn't been called\");\n  return e;\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@babel/runtime/helpers/esm/assertThisInitialized.js?");

/***/ }),

/***/ "./node_modules/@babel/runtime/helpers/esm/extends.js":
/*!************************************************************!*\
  !*** ./node_modules/@babel/runtime/helpers/esm/extends.js ***!
  \************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ _extends)\n/* harmony export */ });\nfunction _extends() {\n  return _extends = Object.assign ? Object.assign.bind() : function (n) {\n    for (var e = 1; e < arguments.length; e++) {\n      var t = arguments[e];\n      for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]);\n    }\n    return n;\n  }, _extends.apply(null, arguments);\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@babel/runtime/helpers/esm/extends.js?");

/***/ }),

/***/ "./node_modules/@babel/runtime/helpers/esm/inheritsLoose.js":
/*!******************************************************************!*\
  !*** ./node_modules/@babel/runtime/helpers/esm/inheritsLoose.js ***!
  \******************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ _inheritsLoose)\n/* harmony export */ });\n/* harmony import */ var _setPrototypeOf_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./setPrototypeOf.js */ \"./node_modules/@babel/runtime/helpers/esm/setPrototypeOf.js\");\n\nfunction _inheritsLoose(t, o) {\n  t.prototype = Object.create(o.prototype), t.prototype.constructor = t, (0,_setPrototypeOf_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(t, o);\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@babel/runtime/helpers/esm/inheritsLoose.js?");

/***/ }),

/***/ "./node_modules/@babel/runtime/helpers/esm/objectWithoutPropertiesLoose.js":
/*!*********************************************************************************!*\
  !*** ./node_modules/@babel/runtime/helpers/esm/objectWithoutPropertiesLoose.js ***!
  \*********************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ _objectWithoutPropertiesLoose)\n/* harmony export */ });\nfunction _objectWithoutPropertiesLoose(r, e) {\n  if (null == r) return {};\n  var t = {};\n  for (var n in r) if ({}.hasOwnProperty.call(r, n)) {\n    if (-1 !== e.indexOf(n)) continue;\n    t[n] = r[n];\n  }\n  return t;\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@babel/runtime/helpers/esm/objectWithoutPropertiesLoose.js?");

/***/ }),

/***/ "./node_modules/@babel/runtime/helpers/esm/setPrototypeOf.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@babel/runtime/helpers/esm/setPrototypeOf.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ _setPrototypeOf)\n/* harmony export */ });\nfunction _setPrototypeOf(t, e) {\n  return _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function (t, e) {\n    return t.__proto__ = e, t;\n  }, _setPrototypeOf(t, e);\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@babel/runtime/helpers/esm/setPrototypeOf.js?");

/***/ }),

/***/ "./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js":
/*!***********************************************************************************!*\
  !*** ./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js ***!
  \***********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createCache)\n/* harmony export */ });\n/* harmony import */ var _emotion_sheet__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @emotion/sheet */ \"./node_modules/@emotion/sheet/dist/emotion-sheet.development.esm.js\");\n/* harmony import */ var stylis__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! stylis */ \"./node_modules/stylis/src/Tokenizer.js\");\n/* harmony import */ var stylis__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! stylis */ \"./node_modules/stylis/src/Utility.js\");\n/* harmony import */ var stylis__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! stylis */ \"./node_modules/stylis/src/Enum.js\");\n/* harmony import */ var stylis__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! stylis */ \"./node_modules/stylis/src/Serializer.js\");\n/* harmony import */ var stylis__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! stylis */ \"./node_modules/stylis/src/Middleware.js\");\n/* harmony import */ var stylis__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! stylis */ \"./node_modules/stylis/src/Parser.js\");\n/* harmony import */ var _emotion_weak_memoize__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/weak-memoize */ \"./node_modules/@emotion/weak-memoize/dist/emotion-weak-memoize.esm.js\");\n/* harmony import */ var _emotion_memoize__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/memoize */ \"./node_modules/@emotion/memoize/dist/emotion-memoize.esm.js\");\n\n\n\n\n\nvar identifierWithPointTracking = function identifierWithPointTracking(begin, points, index) {\n  var previous = 0;\n  var character = 0;\n\n  while (true) {\n    previous = character;\n    character = (0,stylis__WEBPACK_IMPORTED_MODULE_3__.peek)(); // &\\f\n\n    if (previous === 38 && character === 12) {\n      points[index] = 1;\n    }\n\n    if ((0,stylis__WEBPACK_IMPORTED_MODULE_3__.token)(character)) {\n      break;\n    }\n\n    (0,stylis__WEBPACK_IMPORTED_MODULE_3__.next)();\n  }\n\n  return (0,stylis__WEBPACK_IMPORTED_MODULE_3__.slice)(begin, stylis__WEBPACK_IMPORTED_MODULE_3__.position);\n};\n\nvar toRules = function toRules(parsed, points) {\n  // pretend we've started with a comma\n  var index = -1;\n  var character = 44;\n\n  do {\n    switch ((0,stylis__WEBPACK_IMPORTED_MODULE_3__.token)(character)) {\n      case 0:\n        // &\\f\n        if (character === 38 && (0,stylis__WEBPACK_IMPORTED_MODULE_3__.peek)() === 12) {\n          // this is not 100% correct, we don't account for literal sequences here - like for example quoted strings\n          // stylis inserts \\f after & to know when & where it should replace this sequence with the context selector\n          // and when it should just concatenate the outer and inner selectors\n          // it's very unlikely for this sequence to actually appear in a different context, so we just leverage this fact here\n          points[index] = 1;\n        }\n\n        parsed[index] += identifierWithPointTracking(stylis__WEBPACK_IMPORTED_MODULE_3__.position - 1, points, index);\n        break;\n\n      case 2:\n        parsed[index] += (0,stylis__WEBPACK_IMPORTED_MODULE_3__.delimit)(character);\n        break;\n\n      case 4:\n        // comma\n        if (character === 44) {\n          // colon\n          parsed[++index] = (0,stylis__WEBPACK_IMPORTED_MODULE_3__.peek)() === 58 ? '&\\f' : '';\n          points[index] = parsed[index].length;\n          break;\n        }\n\n      // fallthrough\n\n      default:\n        parsed[index] += (0,stylis__WEBPACK_IMPORTED_MODULE_4__.from)(character);\n    }\n  } while (character = (0,stylis__WEBPACK_IMPORTED_MODULE_3__.next)());\n\n  return parsed;\n};\n\nvar getRules = function getRules(value, points) {\n  return (0,stylis__WEBPACK_IMPORTED_MODULE_3__.dealloc)(toRules((0,stylis__WEBPACK_IMPORTED_MODULE_3__.alloc)(value), points));\n}; // WeakSet would be more appropriate, but only WeakMap is supported in IE11\n\n\nvar fixedElements = /* #__PURE__ */new WeakMap();\nvar compat = function compat(element) {\n  if (element.type !== 'rule' || !element.parent || // positive .length indicates that this rule contains pseudo\n  // negative .length indicates that this rule has been already prefixed\n  element.length < 1) {\n    return;\n  }\n\n  var value = element.value;\n  var parent = element.parent;\n  var isImplicitRule = element.column === parent.column && element.line === parent.line;\n\n  while (parent.type !== 'rule') {\n    parent = parent.parent;\n    if (!parent) return;\n  } // short-circuit for the simplest case\n\n\n  if (element.props.length === 1 && value.charCodeAt(0) !== 58\n  /* colon */\n  && !fixedElements.get(parent)) {\n    return;\n  } // if this is an implicitly inserted rule (the one eagerly inserted at the each new nested level)\n  // then the props has already been manipulated beforehand as they that array is shared between it and its \"rule parent\"\n\n\n  if (isImplicitRule) {\n    return;\n  }\n\n  fixedElements.set(element, true);\n  var points = [];\n  var rules = getRules(value, points);\n  var parentRules = parent.props;\n\n  for (var i = 0, k = 0; i < rules.length; i++) {\n    for (var j = 0; j < parentRules.length; j++, k++) {\n      element.props[k] = points[i] ? rules[i].replace(/&\\f/g, parentRules[j]) : parentRules[j] + \" \" + rules[i];\n    }\n  }\n};\nvar removeLabel = function removeLabel(element) {\n  if (element.type === 'decl') {\n    var value = element.value;\n\n    if ( // charcode for l\n    value.charCodeAt(0) === 108 && // charcode for b\n    value.charCodeAt(2) === 98) {\n      // this ignores label\n      element[\"return\"] = '';\n      element.value = '';\n    }\n  }\n};\nvar ignoreFlag = 'emotion-disable-server-rendering-unsafe-selector-warning-please-do-not-use-this-the-warning-exists-for-a-reason';\n\nvar isIgnoringComment = function isIgnoringComment(element) {\n  return element.type === 'comm' && element.children.indexOf(ignoreFlag) > -1;\n};\n\nvar createUnsafeSelectorsAlarm = function createUnsafeSelectorsAlarm(cache) {\n  return function (element, index, children) {\n    if (element.type !== 'rule' || cache.compat) return;\n    var unsafePseudoClasses = element.value.match(/(:first|:nth|:nth-last)-child/g);\n\n    if (unsafePseudoClasses) {\n      var isNested = !!element.parent; // in nested rules comments become children of the \"auto-inserted\" rule and that's always the `element.parent`\n      //\n      // considering this input:\n      // .a {\n      //   .b /* comm */ {}\n      //   color: hotpink;\n      // }\n      // we get output corresponding to this:\n      // .a {\n      //   & {\n      //     /* comm */\n      //     color: hotpink;\n      //   }\n      //   .b {}\n      // }\n\n      var commentContainer = isNested ? element.parent.children : // global rule at the root level\n      children;\n\n      for (var i = commentContainer.length - 1; i >= 0; i--) {\n        var node = commentContainer[i];\n\n        if (node.line < element.line) {\n          break;\n        } // it is quite weird but comments are *usually* put at `column: element.column - 1`\n        // so we seek *from the end* for the node that is earlier than the rule's `element` and check that\n        // this will also match inputs like this:\n        // .a {\n        //   /* comm */\n        //   .b {}\n        // }\n        //\n        // but that is fine\n        //\n        // it would be the easiest to change the placement of the comment to be the first child of the rule:\n        // .a {\n        //   .b { /* comm */ }\n        // }\n        // with such inputs we wouldn't have to search for the comment at all\n        // TODO: consider changing this comment placement in the next major version\n\n\n        if (node.column < element.column) {\n          if (isIgnoringComment(node)) {\n            return;\n          }\n\n          break;\n        }\n      }\n\n      unsafePseudoClasses.forEach(function (unsafePseudoClass) {\n        console.error(\"The pseudo class \\\"\" + unsafePseudoClass + \"\\\" is potentially unsafe when doing server-side rendering. Try changing it to \\\"\" + unsafePseudoClass.split('-child')[0] + \"-of-type\\\".\");\n      });\n    }\n  };\n};\n\nvar isImportRule = function isImportRule(element) {\n  return element.type.charCodeAt(1) === 105 && element.type.charCodeAt(0) === 64;\n};\n\nvar isPrependedWithRegularRules = function isPrependedWithRegularRules(index, children) {\n  for (var i = index - 1; i >= 0; i--) {\n    if (!isImportRule(children[i])) {\n      return true;\n    }\n  }\n\n  return false;\n}; // use this to remove incorrect elements from further processing\n// so they don't get handed to the `sheet` (or anything else)\n// as that could potentially lead to additional logs which in turn could be overhelming to the user\n\n\nvar nullifyElement = function nullifyElement(element) {\n  element.type = '';\n  element.value = '';\n  element[\"return\"] = '';\n  element.children = '';\n  element.props = '';\n};\n\nvar incorrectImportAlarm = function incorrectImportAlarm(element, index, children) {\n  if (!isImportRule(element)) {\n    return;\n  }\n\n  if (element.parent) {\n    console.error(\"`@import` rules can't be nested inside other rules. Please move it to the top level and put it before regular rules. Keep in mind that they can only be used within global styles.\");\n    nullifyElement(element);\n  } else if (isPrependedWithRegularRules(index, children)) {\n    console.error(\"`@import` rules can't be after other rules. Please put your `@import` rules before your other rules.\");\n    nullifyElement(element);\n  }\n};\n\n/* eslint-disable no-fallthrough */\n\nfunction prefix(value, length) {\n  switch ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.hash)(value, length)) {\n    // color-adjust\n    case 5103:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + 'print-' + value + value;\n    // animation, animation-(delay|direction|duration|fill-mode|iteration-count|name|play-state|timing-function)\n\n    case 5737:\n    case 4201:\n    case 3177:\n    case 3433:\n    case 1641:\n    case 4457:\n    case 2921: // text-decoration, filter, clip-path, backface-visibility, column, box-decoration-break\n\n    case 5572:\n    case 6356:\n    case 5844:\n    case 3191:\n    case 6645:\n    case 3005: // mask, mask-image, mask-(mode|clip|size), mask-(repeat|origin), mask-position, mask-composite,\n\n    case 6391:\n    case 5879:\n    case 5623:\n    case 6135:\n    case 4599:\n    case 4855: // background-clip, columns, column-(count|fill|gap|rule|rule-color|rule-style|rule-width|span|width)\n\n    case 4215:\n    case 6389:\n    case 5109:\n    case 5365:\n    case 5621:\n    case 3829:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + value;\n    // appearance, user-select, transform, hyphens, text-size-adjust\n\n    case 5349:\n    case 4246:\n    case 4810:\n    case 6968:\n    case 2756:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MOZ + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + value + value;\n    // flex, flex-direction\n\n    case 6828:\n    case 4268:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + value + value;\n    // order\n\n    case 6165:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + 'flex-' + value + value;\n    // align-items\n\n    case 5187:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(\\w+).+(:[^]+)/, stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + 'box-$1$2' + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + 'flex-$1$2') + value;\n    // align-self\n\n    case 5443:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + 'flex-item-' + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /flex-|-self/, '') + value;\n    // align-content\n\n    case 4675:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + 'flex-line-pack' + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /align-content|flex-|-self/, '') + value;\n    // flex-shrink\n\n    case 5548:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, 'shrink', 'negative') + value;\n    // flex-basis\n\n    case 5292:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, 'basis', 'preferred-size') + value;\n    // flex-grow\n\n    case 6060:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + 'box-' + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, '-grow', '') + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, 'grow', 'positive') + value;\n    // transition\n\n    case 4554:\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /([^-])(transform)/g, '$1' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$2') + value;\n    // cursor\n\n    case 6187:\n      return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)((0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)((0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(zoom-|grab)/, stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$1'), /(image-set)/, stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$1'), value, '') + value;\n    // background, background-image\n\n    case 5495:\n    case 3959:\n      return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(image-set\\([^]*)/, stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$1' + '$`$1');\n    // justify-content\n\n    case 4968:\n      return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)((0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(.+:)(flex-)?(.*)/, stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + 'box-pack:$3' + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + 'flex-pack:$3'), /s.+-b[^;]+/, 'justify') + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + value;\n    // (margin|padding)-inline-(start|end)\n\n    case 4095:\n    case 3583:\n    case 4068:\n    case 2532:\n      return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(.+)-inline(.+)/, stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$1$2') + value;\n    // (min|max)?(width|height|inline-size|block-size)\n\n    case 8116:\n    case 7059:\n    case 5753:\n    case 5535:\n    case 5445:\n    case 5701:\n    case 4933:\n    case 4677:\n    case 5533:\n    case 5789:\n    case 5021:\n    case 4765:\n      // stretch, max-content, min-content, fill-available\n      if ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.strlen)(value) - 1 - length > 6) switch ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, length + 1)) {\n        // (m)ax-content, (m)in-content\n        case 109:\n          // -\n          if ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, length + 4) !== 45) break;\n        // (f)ill-available, (f)it-content\n\n        case 102:\n          return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(.+:)(.+)-([^]+)/, '$1' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$2-$3' + '$1' + stylis__WEBPACK_IMPORTED_MODULE_5__.MOZ + ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, length + 3) == 108 ? '$3' : '$2-$3')) + value;\n        // (s)tretch\n\n        case 115:\n          return ~(0,stylis__WEBPACK_IMPORTED_MODULE_4__.indexof)(value, 'stretch') ? prefix((0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, 'stretch', 'fill-available'), length) + value : value;\n      }\n      break;\n    // position: sticky\n\n    case 4949:\n      // (s)ticky?\n      if ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, length + 1) !== 115) break;\n    // display: (flex|inline-flex)\n\n    case 6444:\n      switch ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, (0,stylis__WEBPACK_IMPORTED_MODULE_4__.strlen)(value) - 3 - (~(0,stylis__WEBPACK_IMPORTED_MODULE_4__.indexof)(value, '!important') && 10))) {\n        // stic(k)y\n        case 107:\n          return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, ':', ':' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT) + value;\n        // (inline-)?fl(e)x\n\n        case 101:\n          return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /(.+:)([^;!]+)(;|!.+)?/, '$1' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, 14) === 45 ? 'inline-' : '') + 'box$3' + '$1' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + '$2$3' + '$1' + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + '$2box$3') + value;\n      }\n\n      break;\n    // writing-mode\n\n    case 5936:\n      switch ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.charat)(value, length + 11)) {\n        // vertical-l(r)\n        case 114:\n          return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /[svh]\\w+-[tblr]{2}/, 'tb') + value;\n        // vertical-r(l)\n\n        case 108:\n          return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /[svh]\\w+-[tblr]{2}/, 'tb-rl') + value;\n        // horizontal(-)tb\n\n        case 45:\n          return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /[svh]\\w+-[tblr]{2}/, 'lr') + value;\n      }\n\n      return stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + value + stylis__WEBPACK_IMPORTED_MODULE_5__.MS + value + value;\n  }\n\n  return value;\n}\n\nvar prefixer = function prefixer(element, index, children, callback) {\n  if (element.length > -1) if (!element[\"return\"]) switch (element.type) {\n    case stylis__WEBPACK_IMPORTED_MODULE_5__.DECLARATION:\n      element[\"return\"] = prefix(element.value, element.length);\n      break;\n\n    case stylis__WEBPACK_IMPORTED_MODULE_5__.KEYFRAMES:\n      return (0,stylis__WEBPACK_IMPORTED_MODULE_6__.serialize)([(0,stylis__WEBPACK_IMPORTED_MODULE_3__.copy)(element, {\n        value: (0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(element.value, '@', '@' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT)\n      })], callback);\n\n    case stylis__WEBPACK_IMPORTED_MODULE_5__.RULESET:\n      if (element.length) return (0,stylis__WEBPACK_IMPORTED_MODULE_4__.combine)(element.props, function (value) {\n        switch ((0,stylis__WEBPACK_IMPORTED_MODULE_4__.match)(value, /(::plac\\w+|:read-\\w+)/)) {\n          // :read-(only|write)\n          case ':read-only':\n          case ':read-write':\n            return (0,stylis__WEBPACK_IMPORTED_MODULE_6__.serialize)([(0,stylis__WEBPACK_IMPORTED_MODULE_3__.copy)(element, {\n              props: [(0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /:(read-\\w+)/, ':' + stylis__WEBPACK_IMPORTED_MODULE_5__.MOZ + '$1')]\n            })], callback);\n          // :placeholder\n\n          case '::placeholder':\n            return (0,stylis__WEBPACK_IMPORTED_MODULE_6__.serialize)([(0,stylis__WEBPACK_IMPORTED_MODULE_3__.copy)(element, {\n              props: [(0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /:(plac\\w+)/, ':' + stylis__WEBPACK_IMPORTED_MODULE_5__.WEBKIT + 'input-$1')]\n            }), (0,stylis__WEBPACK_IMPORTED_MODULE_3__.copy)(element, {\n              props: [(0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /:(plac\\w+)/, ':' + stylis__WEBPACK_IMPORTED_MODULE_5__.MOZ + '$1')]\n            }), (0,stylis__WEBPACK_IMPORTED_MODULE_3__.copy)(element, {\n              props: [(0,stylis__WEBPACK_IMPORTED_MODULE_4__.replace)(value, /:(plac\\w+)/, stylis__WEBPACK_IMPORTED_MODULE_5__.MS + 'input-$1')]\n            })], callback);\n        }\n\n        return '';\n      });\n  }\n};\n\nvar defaultStylisPlugins = [prefixer];\nvar getSourceMap;\n\n{\n  var sourceMapPattern = /\\/\\*#\\ssourceMappingURL=data:application\\/json;\\S+\\s+\\*\\//g;\n\n  getSourceMap = function getSourceMap(styles) {\n    var matches = styles.match(sourceMapPattern);\n    if (!matches) return;\n    return matches[matches.length - 1];\n  };\n}\n\nvar createCache = function createCache(options) {\n  var key = options.key;\n\n  if (!key) {\n    throw new Error(\"You have to configure `key` for your cache. Please make sure it's unique (and not equal to 'css') as it's used for linking styles to your cache.\\n\" + \"If multiple caches share the same key they might \\\"fight\\\" for each other's style elements.\");\n  }\n\n  if (key === 'css') {\n    var ssrStyles = document.querySelectorAll(\"style[data-emotion]:not([data-s])\"); // get SSRed styles out of the way of React's hydration\n    // document.head is a safe place to move them to(though note document.head is not necessarily the last place they will be)\n    // note this very very intentionally targets all style elements regardless of the key to ensure\n    // that creating a cache works inside of render of a React component\n\n    Array.prototype.forEach.call(ssrStyles, function (node) {\n      // we want to only move elements which have a space in the data-emotion attribute value\n      // because that indicates that it is an Emotion 11 server-side rendered style elements\n      // while we will already ignore Emotion 11 client-side inserted styles because of the :not([data-s]) part in the selector\n      // Emotion 10 client-side inserted styles did not have data-s (but importantly did not have a space in their data-emotion attributes)\n      // so checking for the space ensures that loading Emotion 11 after Emotion 10 has inserted some styles\n      // will not result in the Emotion 10 styles being destroyed\n      var dataEmotionAttribute = node.getAttribute('data-emotion');\n\n      if (dataEmotionAttribute.indexOf(' ') === -1) {\n        return;\n      }\n\n      document.head.appendChild(node);\n      node.setAttribute('data-s', '');\n    });\n  }\n\n  var stylisPlugins = options.stylisPlugins || defaultStylisPlugins;\n\n  {\n    if (/[^a-z-]/.test(key)) {\n      throw new Error(\"Emotion key must only contain lower case alphabetical characters and - but \\\"\" + key + \"\\\" was passed\");\n    }\n  }\n\n  var inserted = {};\n  var container;\n  var nodesToHydrate = [];\n\n  {\n    container = options.container || document.head;\n    Array.prototype.forEach.call( // this means we will ignore elements which don't have a space in them which\n    // means that the style elements we're looking at are only Emotion 11 server-rendered style elements\n    document.querySelectorAll(\"style[data-emotion^=\\\"\" + key + \" \\\"]\"), function (node) {\n      var attrib = node.getAttribute(\"data-emotion\").split(' ');\n\n      for (var i = 1; i < attrib.length; i++) {\n        inserted[attrib[i]] = true;\n      }\n\n      nodesToHydrate.push(node);\n    });\n  }\n\n  var _insert;\n\n  var omnipresentPlugins = [compat, removeLabel];\n\n  {\n    omnipresentPlugins.push(createUnsafeSelectorsAlarm({\n      get compat() {\n        return cache.compat;\n      }\n\n    }), incorrectImportAlarm);\n  }\n\n  {\n    var currentSheet;\n    var finalizingPlugins = [stylis__WEBPACK_IMPORTED_MODULE_6__.stringify, function (element) {\n      if (!element.root) {\n        if (element[\"return\"]) {\n          currentSheet.insert(element[\"return\"]);\n        } else if (element.value && element.type !== stylis__WEBPACK_IMPORTED_MODULE_5__.COMMENT) {\n          // insert empty rule in non-production environments\n          // so @emotion/jest can grab `key` from the (JS)DOM for caches without any rules inserted yet\n          currentSheet.insert(element.value + \"{}\");\n        }\n      }\n    } ];\n    var serializer = (0,stylis__WEBPACK_IMPORTED_MODULE_7__.middleware)(omnipresentPlugins.concat(stylisPlugins, finalizingPlugins));\n\n    var stylis = function stylis(styles) {\n      return (0,stylis__WEBPACK_IMPORTED_MODULE_6__.serialize)((0,stylis__WEBPACK_IMPORTED_MODULE_8__.compile)(styles), serializer);\n    };\n\n    _insert = function insert(selector, serialized, sheet, shouldCache) {\n      currentSheet = sheet;\n\n      if (getSourceMap) {\n        var sourceMap = getSourceMap(serialized.styles);\n\n        if (sourceMap) {\n          currentSheet = {\n            insert: function insert(rule) {\n              sheet.insert(rule + sourceMap);\n            }\n          };\n        }\n      }\n\n      stylis(selector ? selector + \"{\" + serialized.styles + \"}\" : serialized.styles);\n\n      if (shouldCache) {\n        cache.inserted[serialized.name] = true;\n      }\n    };\n  }\n\n  var cache = {\n    key: key,\n    sheet: new _emotion_sheet__WEBPACK_IMPORTED_MODULE_0__.StyleSheet({\n      key: key,\n      container: container,\n      nonce: options.nonce,\n      speedy: options.speedy,\n      prepend: options.prepend,\n      insertionPoint: options.insertionPoint\n    }),\n    nonce: options.nonce,\n    inserted: inserted,\n    registered: {},\n    insert: _insert\n  };\n  cache.sheet.hydrate(nodesToHydrate);\n  return cache;\n};\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/hash/dist/emotion-hash.esm.js":
/*!*************************************************************!*\
  !*** ./node_modules/@emotion/hash/dist/emotion-hash.esm.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ murmur2)\n/* harmony export */ });\n/* eslint-disable */\n// Inspired by https://github.com/garycourt/murmurhash-js\n// Ported from https://github.com/aappleby/smhasher/blob/61a0530f28277f2e850bfc39600ce61d02b518de/src/MurmurHash2.cpp#L37-L86\nfunction murmur2(str) {\n  // 'm' and 'r' are mixing constants generated offline.\n  // They're not really 'magic', they just happen to work well.\n  // const m = 0x5bd1e995;\n  // const r = 24;\n  // Initialize the hash\n  var h = 0; // Mix 4 bytes at a time into the hash\n\n  var k,\n      i = 0,\n      len = str.length;\n\n  for (; len >= 4; ++i, len -= 4) {\n    k = str.charCodeAt(i) & 0xff | (str.charCodeAt(++i) & 0xff) << 8 | (str.charCodeAt(++i) & 0xff) << 16 | (str.charCodeAt(++i) & 0xff) << 24;\n    k =\n    /* Math.imul(k, m): */\n    (k & 0xffff) * 0x5bd1e995 + ((k >>> 16) * 0xe995 << 16);\n    k ^=\n    /* k >>> r: */\n    k >>> 24;\n    h =\n    /* Math.imul(k, m): */\n    (k & 0xffff) * 0x5bd1e995 + ((k >>> 16) * 0xe995 << 16) ^\n    /* Math.imul(h, m): */\n    (h & 0xffff) * 0x5bd1e995 + ((h >>> 16) * 0xe995 << 16);\n  } // Handle the last few bytes of the input array\n\n\n  switch (len) {\n    case 3:\n      h ^= (str.charCodeAt(i + 2) & 0xff) << 16;\n\n    case 2:\n      h ^= (str.charCodeAt(i + 1) & 0xff) << 8;\n\n    case 1:\n      h ^= str.charCodeAt(i) & 0xff;\n      h =\n      /* Math.imul(h, m): */\n      (h & 0xffff) * 0x5bd1e995 + ((h >>> 16) * 0xe995 << 16);\n  } // Do a few final mixes of the hash to ensure the last few\n  // bytes are well-incorporated.\n\n\n  h ^= h >>> 13;\n  h =\n  /* Math.imul(h, m): */\n  (h & 0xffff) * 0x5bd1e995 + ((h >>> 16) * 0xe995 << 16);\n  return ((h ^ h >>> 15) >>> 0).toString(36);\n}\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/hash/dist/emotion-hash.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/is-prop-valid/dist/emotion-is-prop-valid.esm.js":
/*!*******************************************************************************!*\
  !*** ./node_modules/@emotion/is-prop-valid/dist/emotion-is-prop-valid.esm.js ***!
  \*******************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ isPropValid)\n/* harmony export */ });\n/* harmony import */ var _emotion_memoize__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @emotion/memoize */ \"./node_modules/@emotion/memoize/dist/emotion-memoize.esm.js\");\n\n\n// eslint-disable-next-line no-undef\nvar reactPropsRegex = /^((children|dangerouslySetInnerHTML|key|ref|autoFocus|defaultValue|defaultChecked|innerHTML|suppressContentEditableWarning|suppressHydrationWarning|valueLink|abbr|accept|acceptCharset|accessKey|action|allow|allowUserMedia|allowPaymentRequest|allowFullScreen|allowTransparency|alt|async|autoComplete|autoPlay|capture|cellPadding|cellSpacing|challenge|charSet|checked|cite|classID|className|cols|colSpan|content|contentEditable|contextMenu|controls|controlsList|coords|crossOrigin|data|dateTime|decoding|default|defer|dir|disabled|disablePictureInPicture|disableRemotePlayback|download|draggable|encType|enterKeyHint|fetchpriority|fetchPriority|form|formAction|formEncType|formMethod|formNoValidate|formTarget|frameBorder|headers|height|hidden|high|href|hrefLang|htmlFor|httpEquiv|id|inputMode|integrity|is|keyParams|keyType|kind|label|lang|list|loading|loop|low|marginHeight|marginWidth|max|maxLength|media|mediaGroup|method|min|minLength|multiple|muted|name|nonce|noValidate|open|optimum|pattern|placeholder|playsInline|poster|preload|profile|radioGroup|readOnly|referrerPolicy|rel|required|reversed|role|rows|rowSpan|sandbox|scope|scoped|scrolling|seamless|selected|shape|size|sizes|slot|span|spellCheck|src|srcDoc|srcLang|srcSet|start|step|style|summary|tabIndex|target|title|translate|type|useMap|value|width|wmode|wrap|about|datatype|inlist|prefix|property|resource|typeof|vocab|autoCapitalize|autoCorrect|autoSave|color|incremental|fallback|inert|itemProp|itemScope|itemType|itemID|itemRef|on|option|results|security|unselectable|accentHeight|accumulate|additive|alignmentBaseline|allowReorder|alphabetic|amplitude|arabicForm|ascent|attributeName|attributeType|autoReverse|azimuth|baseFrequency|baselineShift|baseProfile|bbox|begin|bias|by|calcMode|capHeight|clip|clipPathUnits|clipPath|clipRule|colorInterpolation|colorInterpolationFilters|colorProfile|colorRendering|contentScriptType|contentStyleType|cursor|cx|cy|d|decelerate|descent|diffuseConstant|direction|display|divisor|dominantBaseline|dur|dx|dy|edgeMode|elevation|enableBackground|end|exponent|externalResourcesRequired|fill|fillOpacity|fillRule|filter|filterRes|filterUnits|floodColor|floodOpacity|focusable|fontFamily|fontSize|fontSizeAdjust|fontStretch|fontStyle|fontVariant|fontWeight|format|from|fr|fx|fy|g1|g2|glyphName|glyphOrientationHorizontal|glyphOrientationVertical|glyphRef|gradientTransform|gradientUnits|hanging|horizAdvX|horizOriginX|ideographic|imageRendering|in|in2|intercept|k|k1|k2|k3|k4|kernelMatrix|kernelUnitLength|kerning|keyPoints|keySplines|keyTimes|lengthAdjust|letterSpacing|lightingColor|limitingConeAngle|local|markerEnd|markerMid|markerStart|markerHeight|markerUnits|markerWidth|mask|maskContentUnits|maskUnits|mathematical|mode|numOctaves|offset|opacity|operator|order|orient|orientation|origin|overflow|overlinePosition|overlineThickness|panose1|paintOrder|pathLength|patternContentUnits|patternTransform|patternUnits|pointerEvents|points|pointsAtX|pointsAtY|pointsAtZ|preserveAlpha|preserveAspectRatio|primitiveUnits|r|radius|refX|refY|renderingIntent|repeatCount|repeatDur|requiredExtensions|requiredFeatures|restart|result|rotate|rx|ry|scale|seed|shapeRendering|slope|spacing|specularConstant|specularExponent|speed|spreadMethod|startOffset|stdDeviation|stemh|stemv|stitchTiles|stopColor|stopOpacity|strikethroughPosition|strikethroughThickness|string|stroke|strokeDasharray|strokeDashoffset|strokeLinecap|strokeLinejoin|strokeMiterlimit|strokeOpacity|strokeWidth|surfaceScale|systemLanguage|tableValues|targetX|targetY|textAnchor|textDecoration|textRendering|textLength|to|transform|u1|u2|underlinePosition|underlineThickness|unicode|unicodeBidi|unicodeRange|unitsPerEm|vAlphabetic|vHanging|vIdeographic|vMathematical|values|vectorEffect|version|vertAdvY|vertOriginX|vertOriginY|viewBox|viewTarget|visibility|widths|wordSpacing|writingMode|x|xHeight|x1|x2|xChannelSelector|xlinkActuate|xlinkArcrole|xlinkHref|xlinkRole|xlinkShow|xlinkTitle|xlinkType|xmlBase|xmlns|xmlnsXlink|xmlLang|xmlSpace|y|y1|y2|yChannelSelector|z|zoomAndPan|for|class|autofocus)|(([Dd][Aa][Tt][Aa]|[Aa][Rr][Ii][Aa]|x)-.*))$/; // https://esbench.com/bench/5bfee68a4cd7e6009ef61d23\n\nvar isPropValid = /* #__PURE__ */(0,_emotion_memoize__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(function (prop) {\n  return reactPropsRegex.test(prop) || prop.charCodeAt(0) === 111\n  /* o */\n  && prop.charCodeAt(1) === 110\n  /* n */\n  && prop.charCodeAt(2) < 91;\n}\n/* Z+1 */\n);\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/is-prop-valid/dist/emotion-is-prop-valid.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/memoize/dist/emotion-memoize.esm.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@emotion/memoize/dist/emotion-memoize.esm.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ memoize)\n/* harmony export */ });\nfunction memoize(fn) {\n  var cache = Object.create(null);\n  return function (arg) {\n    if (cache[arg] === undefined) cache[arg] = fn(arg);\n    return cache[arg];\n  };\n}\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/memoize/dist/emotion-memoize.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/react/_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.esm.js":
/*!*****************************************************************************************************************!*\
  !*** ./node_modules/@emotion/react/_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.esm.js ***!
  \*****************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ hoistNonReactStatics)\n/* harmony export */ });\n/* harmony import */ var hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! hoist-non-react-statics */ \"./node_modules/hoist-non-react-statics/dist/hoist-non-react-statics.cjs.js\");\n/* harmony import */ var hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_0__);\n\n\n// this file isolates this package that is not tree-shakeable\n// and if this module doesn't actually contain any logic of its own\n// then Rollup just use 'hoist-non-react-statics' directly in other chunks\n\nvar hoistNonReactStatics = (function (targetComponent, sourceComponent) {\n  return hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_0___default()(targetComponent, sourceComponent);\n});\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/react/_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js":
/*!**********************************************************************************************!*\
  !*** ./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js ***!
  \**********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   C: () => (/* binding */ CacheProvider),\n/* harmony export */   E: () => (/* binding */ Emotion$1),\n/* harmony export */   T: () => (/* binding */ ThemeContext),\n/* harmony export */   _: () => (/* binding */ __unsafe_useEmotionCache),\n/* harmony export */   a: () => (/* binding */ ThemeProvider),\n/* harmony export */   b: () => (/* binding */ withTheme),\n/* harmony export */   c: () => (/* binding */ createEmotionProps),\n/* harmony export */   h: () => (/* binding */ hasOwn),\n/* harmony export */   u: () => (/* binding */ useTheme),\n/* harmony export */   w: () => (/* binding */ withEmotionCache)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _emotion_cache__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/cache */ \"./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js\");\n/* harmony import */ var _babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @babel/runtime/helpers/esm/extends */ \"./node_modules/@babel/runtime/helpers/esm/extends.js\");\n/* harmony import */ var _emotion_weak_memoize__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @emotion/weak-memoize */ \"./node_modules/@emotion/weak-memoize/dist/emotion-weak-memoize.esm.js\");\n/* harmony import */ var _isolated_hnrs_dist_emotion_react_isolated_hnrs_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.esm.js */ \"./node_modules/@emotion/react/_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.esm.js\");\n/* harmony import */ var _emotion_utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @emotion/utils */ \"./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js\");\n/* harmony import */ var _emotion_serialize__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @emotion/serialize */ \"./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js\");\n/* harmony import */ var _emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @emotion/use-insertion-effect-with-fallbacks */ \"./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js\");\n\n\n\n\n\n\n\n\n\n\nvar EmotionCacheContext = /* #__PURE__ */react__WEBPACK_IMPORTED_MODULE_0__.createContext( // we're doing this to avoid preconstruct's dead code elimination in this one case\n// because this module is primarily intended for the browser and node\n// but it's also required in react native and similar environments sometimes\n// and we could have a special build just for that\n// but this is much easier and the native packages\n// might use a different theme context in the future anyway\ntypeof HTMLElement !== 'undefined' ? /* #__PURE__ */(0,_emotion_cache__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n  key: 'css'\n}) : null);\n\n{\n  EmotionCacheContext.displayName = 'EmotionCacheContext';\n}\n\nvar CacheProvider = EmotionCacheContext.Provider;\nvar __unsafe_useEmotionCache = function useEmotionCache() {\n  return (0,react__WEBPACK_IMPORTED_MODULE_0__.useContext)(EmotionCacheContext);\n};\n\nvar withEmotionCache = function withEmotionCache(func) {\n  return /*#__PURE__*/(0,react__WEBPACK_IMPORTED_MODULE_0__.forwardRef)(function (props, ref) {\n    // the cache will never be null in the browser\n    var cache = (0,react__WEBPACK_IMPORTED_MODULE_0__.useContext)(EmotionCacheContext);\n    return func(props, cache, ref);\n  });\n};\n\nvar ThemeContext = /* #__PURE__ */react__WEBPACK_IMPORTED_MODULE_0__.createContext({});\n\n{\n  ThemeContext.displayName = 'EmotionThemeContext';\n}\n\nvar useTheme = function useTheme() {\n  return react__WEBPACK_IMPORTED_MODULE_0__.useContext(ThemeContext);\n};\n\nvar getTheme = function getTheme(outerTheme, theme) {\n  if (typeof theme === 'function') {\n    var mergedTheme = theme(outerTheme);\n\n    if ((mergedTheme == null || typeof mergedTheme !== 'object' || Array.isArray(mergedTheme))) {\n      throw new Error('[ThemeProvider] Please return an object from your theme function, i.e. theme={() => ({})}!');\n    }\n\n    return mergedTheme;\n  }\n\n  if ((theme == null || typeof theme !== 'object' || Array.isArray(theme))) {\n    throw new Error('[ThemeProvider] Please make your theme prop a plain object');\n  }\n\n  return (0,_babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({}, outerTheme, theme);\n};\n\nvar createCacheWithTheme = /* #__PURE__ */(0,_emotion_weak_memoize__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(function (outerTheme) {\n  return (0,_emotion_weak_memoize__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(function (theme) {\n    return getTheme(outerTheme, theme);\n  });\n});\nvar ThemeProvider = function ThemeProvider(props) {\n  var theme = react__WEBPACK_IMPORTED_MODULE_0__.useContext(ThemeContext);\n\n  if (props.theme !== theme) {\n    theme = createCacheWithTheme(theme)(props.theme);\n  }\n\n  return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createElement(ThemeContext.Provider, {\n    value: theme\n  }, props.children);\n};\nfunction withTheme(Component) {\n  var componentName = Component.displayName || Component.name || 'Component';\n  var WithTheme = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function render(props, ref) {\n    var theme = react__WEBPACK_IMPORTED_MODULE_0__.useContext(ThemeContext);\n    return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createElement(Component, (0,_babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n      theme: theme,\n      ref: ref\n    }, props));\n  });\n  WithTheme.displayName = \"WithTheme(\" + componentName + \")\";\n  return (0,_isolated_hnrs_dist_emotion_react_isolated_hnrs_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(WithTheme, Component);\n}\n\nvar hasOwn = {}.hasOwnProperty;\n\nvar getLastPart = function getLastPart(functionName) {\n  // The match may be something like 'Object.createEmotionProps' or\n  // 'Loader.prototype.render'\n  var parts = functionName.split('.');\n  return parts[parts.length - 1];\n};\n\nvar getFunctionNameFromStackTraceLine = function getFunctionNameFromStackTraceLine(line) {\n  // V8\n  var match = /^\\s+at\\s+([A-Za-z0-9$.]+)\\s/.exec(line);\n  if (match) return getLastPart(match[1]); // Safari / Firefox\n\n  match = /^([A-Za-z0-9$.]+)@/.exec(line);\n  if (match) return getLastPart(match[1]);\n  return undefined;\n};\n\nvar internalReactFunctionNames = /* #__PURE__ */new Set(['renderWithHooks', 'processChild', 'finishClassComponent', 'renderToString']); // These identifiers come from error stacks, so they have to be valid JS\n// identifiers, thus we only need to replace what is a valid character for JS,\n// but not for CSS.\n\nvar sanitizeIdentifier = function sanitizeIdentifier(identifier) {\n  return identifier.replace(/\\$/g, '-');\n};\n\nvar getLabelFromStackTrace = function getLabelFromStackTrace(stackTrace) {\n  if (!stackTrace) return undefined;\n  var lines = stackTrace.split('\\n');\n\n  for (var i = 0; i < lines.length; i++) {\n    var functionName = getFunctionNameFromStackTraceLine(lines[i]); // The first line of V8 stack traces is just \"Error\"\n\n    if (!functionName) continue; // If we reach one of these, we have gone too far and should quit\n\n    if (internalReactFunctionNames.has(functionName)) break; // The component name is the first function in the stack that starts with an\n    // uppercase letter\n\n    if (/^[A-Z]/.test(functionName)) return sanitizeIdentifier(functionName);\n  }\n\n  return undefined;\n};\n\nvar typePropName = '__EMOTION_TYPE_PLEASE_DO_NOT_USE__';\nvar labelPropName = '__EMOTION_LABEL_PLEASE_DO_NOT_USE__';\nvar createEmotionProps = function createEmotionProps(type, props) {\n  if (typeof props.css === 'string' && // check if there is a css declaration\n  props.css.indexOf(':') !== -1) {\n    throw new Error(\"Strings are not allowed as css prop values, please wrap it in a css template literal from '@emotion/react' like this: css`\" + props.css + \"`\");\n  }\n\n  var newProps = {};\n\n  for (var _key in props) {\n    if (hasOwn.call(props, _key)) {\n      newProps[_key] = props[_key];\n    }\n  }\n\n  newProps[typePropName] = type; // Runtime labeling is an opt-in feature because:\n  // - It causes hydration warnings when using Safari and SSR\n  // - It can degrade performance if there are a huge number of elements\n  //\n  // Even if the flag is set, we still don't compute the label if it has already\n  // been determined by the Babel plugin.\n\n  if (typeof globalThis !== 'undefined' && !!globalThis.EMOTION_RUNTIME_AUTO_LABEL && !!props.css && (typeof props.css !== 'object' || !('name' in props.css) || typeof props.css.name !== 'string' || props.css.name.indexOf('-') === -1)) {\n    var label = getLabelFromStackTrace(new Error().stack);\n    if (label) newProps[labelPropName] = label;\n  }\n\n  return newProps;\n};\n\nvar Insertion = function Insertion(_ref) {\n  var cache = _ref.cache,\n      serialized = _ref.serialized,\n      isStringTag = _ref.isStringTag;\n  (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_4__.registerStyles)(cache, serialized, isStringTag);\n  (0,_emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_6__.useInsertionEffectAlwaysWithSyncFallback)(function () {\n    return (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_4__.insertStyles)(cache, serialized, isStringTag);\n  });\n\n  return null;\n};\n\nvar Emotion = /* #__PURE__ */withEmotionCache(function (props, cache, ref) {\n  var cssProp = props.css; // so that using `css` from `emotion` and passing the result to the css prop works\n  // not passing the registered cache to serializeStyles because it would\n  // make certain babel optimisations not possible\n\n  if (typeof cssProp === 'string' && cache.registered[cssProp] !== undefined) {\n    cssProp = cache.registered[cssProp];\n  }\n\n  var WrappedComponent = props[typePropName];\n  var registeredStyles = [cssProp];\n  var className = '';\n\n  if (typeof props.className === 'string') {\n    className = (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_4__.getRegisteredStyles)(cache.registered, registeredStyles, props.className);\n  } else if (props.className != null) {\n    className = props.className + \" \";\n  }\n\n  var serialized = (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_5__.serializeStyles)(registeredStyles, undefined, react__WEBPACK_IMPORTED_MODULE_0__.useContext(ThemeContext));\n\n  if (serialized.name.indexOf('-') === -1) {\n    var labelFromStack = props[labelPropName];\n\n    if (labelFromStack) {\n      serialized = (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_5__.serializeStyles)([serialized, 'label:' + labelFromStack + ';']);\n    }\n  }\n\n  className += cache.key + \"-\" + serialized.name;\n  var newProps = {};\n\n  for (var _key2 in props) {\n    if (hasOwn.call(props, _key2) && _key2 !== 'css' && _key2 !== typePropName && (_key2 !== labelPropName)) {\n      newProps[_key2] = props[_key2];\n    }\n  }\n\n  newProps.className = className;\n\n  if (ref) {\n    newProps.ref = ref;\n  }\n\n  return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createElement(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, null, /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createElement(Insertion, {\n    cache: cache,\n    serialized: serialized,\n    isStringTag: typeof WrappedComponent === 'string'\n  }), /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createElement(WrappedComponent, newProps));\n});\n\n{\n  Emotion.displayName = 'EmotionCssPropInternal';\n}\n\nvar Emotion$1 = Emotion;\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js":
/*!***********************************************************************************!*\
  !*** ./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js ***!
  \***********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   CacheProvider: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.C),\n/* harmony export */   ClassNames: () => (/* binding */ ClassNames),\n/* harmony export */   Global: () => (/* binding */ Global),\n/* harmony export */   ThemeContext: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.T),\n/* harmony export */   ThemeProvider: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.a),\n/* harmony export */   __unsafe_useEmotionCache: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__._),\n/* harmony export */   createElement: () => (/* binding */ jsx),\n/* harmony export */   css: () => (/* binding */ css),\n/* harmony export */   jsx: () => (/* binding */ jsx),\n/* harmony export */   keyframes: () => (/* binding */ keyframes),\n/* harmony export */   useTheme: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.u),\n/* harmony export */   withEmotionCache: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.w),\n/* harmony export */   withTheme: () => (/* reexport safe */ _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.b)\n/* harmony export */ });\n/* harmony import */ var _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./emotion-element-489459f2.browser.development.esm.js */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _emotion_utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/utils */ \"./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js\");\n/* harmony import */ var _emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @emotion/use-insertion-effect-with-fallbacks */ \"./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js\");\n/* harmony import */ var _emotion_serialize__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @emotion/serialize */ \"./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js\");\n/* harmony import */ var _emotion_cache__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @emotion/cache */ \"./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js\");\n/* harmony import */ var _babel_runtime_helpers_extends__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @babel/runtime/helpers/extends */ \"./node_modules/@babel/runtime/helpers/esm/extends.js\");\n/* harmony import */ var _emotion_weak_memoize__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @emotion/weak-memoize */ \"./node_modules/@emotion/weak-memoize/dist/emotion-weak-memoize.esm.js\");\n/* harmony import */ var hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! hoist-non-react-statics */ \"./node_modules/hoist-non-react-statics/dist/hoist-non-react-statics.cjs.js\");\n/* harmony import */ var hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(hoist_non_react_statics__WEBPACK_IMPORTED_MODULE_8__);\n\n\n\n\n\n\n\n\n\n\n\n\nvar isDevelopment = true;\n\nvar pkg = {\n\tname: \"@emotion/react\",\n\tversion: \"11.14.0\",\n\tmain: \"dist/emotion-react.cjs.js\",\n\tmodule: \"dist/emotion-react.esm.js\",\n\ttypes: \"dist/emotion-react.cjs.d.ts\",\n\texports: {\n\t\t\".\": {\n\t\t\ttypes: {\n\t\t\t\t\"import\": \"./dist/emotion-react.cjs.mjs\",\n\t\t\t\t\"default\": \"./dist/emotion-react.cjs.js\"\n\t\t\t},\n\t\t\tdevelopment: {\n\t\t\t\t\"edge-light\": {\n\t\t\t\t\tmodule: \"./dist/emotion-react.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./dist/emotion-react.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./dist/emotion-react.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworker: {\n\t\t\t\t\tmodule: \"./dist/emotion-react.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./dist/emotion-react.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./dist/emotion-react.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworkerd: {\n\t\t\t\t\tmodule: \"./dist/emotion-react.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./dist/emotion-react.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./dist/emotion-react.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tbrowser: {\n\t\t\t\t\tmodule: \"./dist/emotion-react.browser.development.esm.js\",\n\t\t\t\t\t\"import\": \"./dist/emotion-react.browser.development.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./dist/emotion-react.browser.development.cjs.js\"\n\t\t\t\t},\n\t\t\t\tmodule: \"./dist/emotion-react.development.esm.js\",\n\t\t\t\t\"import\": \"./dist/emotion-react.development.cjs.mjs\",\n\t\t\t\t\"default\": \"./dist/emotion-react.development.cjs.js\"\n\t\t\t},\n\t\t\t\"edge-light\": {\n\t\t\t\tmodule: \"./dist/emotion-react.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./dist/emotion-react.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./dist/emotion-react.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworker: {\n\t\t\t\tmodule: \"./dist/emotion-react.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./dist/emotion-react.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./dist/emotion-react.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworkerd: {\n\t\t\t\tmodule: \"./dist/emotion-react.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./dist/emotion-react.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./dist/emotion-react.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tbrowser: {\n\t\t\t\tmodule: \"./dist/emotion-react.browser.esm.js\",\n\t\t\t\t\"import\": \"./dist/emotion-react.browser.cjs.mjs\",\n\t\t\t\t\"default\": \"./dist/emotion-react.browser.cjs.js\"\n\t\t\t},\n\t\t\tmodule: \"./dist/emotion-react.esm.js\",\n\t\t\t\"import\": \"./dist/emotion-react.cjs.mjs\",\n\t\t\t\"default\": \"./dist/emotion-react.cjs.js\"\n\t\t},\n\t\t\"./jsx-runtime\": {\n\t\t\ttypes: {\n\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.cjs.js\"\n\t\t\t},\n\t\t\tdevelopment: {\n\t\t\t\t\"edge-light\": {\n\t\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworker: {\n\t\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworkerd: {\n\t\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tbrowser: {\n\t\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.browser.development.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.browser.development.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.browser.development.cjs.js\"\n\t\t\t\t},\n\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.esm.js\",\n\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.development.cjs.js\"\n\t\t\t},\n\t\t\t\"edge-light\": {\n\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworker: {\n\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworkerd: {\n\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tbrowser: {\n\t\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.browser.esm.js\",\n\t\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.browser.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.browser.cjs.js\"\n\t\t\t},\n\t\t\tmodule: \"./jsx-runtime/dist/emotion-react-jsx-runtime.esm.js\",\n\t\t\t\"import\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.cjs.mjs\",\n\t\t\t\"default\": \"./jsx-runtime/dist/emotion-react-jsx-runtime.cjs.js\"\n\t\t},\n\t\t\"./_isolated-hnrs\": {\n\t\t\ttypes: {\n\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.cjs.mjs\",\n\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.cjs.js\"\n\t\t\t},\n\t\t\tdevelopment: {\n\t\t\t\t\"edge-light\": {\n\t\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworker: {\n\t\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworkerd: {\n\t\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tbrowser: {\n\t\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.esm.js\",\n\t\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.development.cjs.js\"\n\t\t\t\t},\n\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.esm.js\",\n\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.cjs.mjs\",\n\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.development.cjs.js\"\n\t\t\t},\n\t\t\t\"edge-light\": {\n\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworker: {\n\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworkerd: {\n\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tbrowser: {\n\t\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.esm.js\",\n\t\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.cjs.mjs\",\n\t\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.cjs.js\"\n\t\t\t},\n\t\t\tmodule: \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.esm.js\",\n\t\t\t\"import\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.cjs.mjs\",\n\t\t\t\"default\": \"./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.cjs.js\"\n\t\t},\n\t\t\"./jsx-dev-runtime\": {\n\t\t\ttypes: {\n\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.cjs.js\"\n\t\t\t},\n\t\t\tdevelopment: {\n\t\t\t\t\"edge-light\": {\n\t\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworker: {\n\t\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tworkerd: {\n\t\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.edge-light.cjs.js\"\n\t\t\t\t},\n\t\t\t\tbrowser: {\n\t\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.development.esm.js\",\n\t\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.development.cjs.mjs\",\n\t\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.development.cjs.js\"\n\t\t\t\t},\n\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.esm.js\",\n\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.development.cjs.js\"\n\t\t\t},\n\t\t\t\"edge-light\": {\n\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworker: {\n\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tworkerd: {\n\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.esm.js\",\n\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.edge-light.cjs.js\"\n\t\t\t},\n\t\t\tbrowser: {\n\t\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.esm.js\",\n\t\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.cjs.mjs\",\n\t\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.cjs.js\"\n\t\t\t},\n\t\t\tmodule: \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.esm.js\",\n\t\t\t\"import\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.cjs.mjs\",\n\t\t\t\"default\": \"./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.cjs.js\"\n\t\t},\n\t\t\"./package.json\": \"./package.json\",\n\t\t\"./types/css-prop\": \"./types/css-prop.d.ts\",\n\t\t\"./macro\": {\n\t\t\ttypes: {\n\t\t\t\t\"import\": \"./macro.d.mts\",\n\t\t\t\t\"default\": \"./macro.d.ts\"\n\t\t\t},\n\t\t\t\"default\": \"./macro.js\"\n\t\t}\n\t},\n\timports: {\n\t\t\"#is-development\": {\n\t\t\tdevelopment: \"./src/conditions/true.ts\",\n\t\t\t\"default\": \"./src/conditions/false.ts\"\n\t\t},\n\t\t\"#is-browser\": {\n\t\t\t\"edge-light\": \"./src/conditions/false.ts\",\n\t\t\tworkerd: \"./src/conditions/false.ts\",\n\t\t\tworker: \"./src/conditions/false.ts\",\n\t\t\tbrowser: \"./src/conditions/true.ts\",\n\t\t\t\"default\": \"./src/conditions/is-browser.ts\"\n\t\t}\n\t},\n\tfiles: [\n\t\t\"src\",\n\t\t\"dist\",\n\t\t\"jsx-runtime\",\n\t\t\"jsx-dev-runtime\",\n\t\t\"_isolated-hnrs\",\n\t\t\"types/css-prop.d.ts\",\n\t\t\"macro.*\"\n\t],\n\tsideEffects: false,\n\tauthor: \"Emotion Contributors\",\n\tlicense: \"MIT\",\n\tscripts: {\n\t\t\"test:typescript\": \"dtslint types\"\n\t},\n\tdependencies: {\n\t\t\"@babel/runtime\": \"^7.18.3\",\n\t\t\"@emotion/babel-plugin\": \"^11.13.5\",\n\t\t\"@emotion/cache\": \"^11.14.0\",\n\t\t\"@emotion/serialize\": \"^1.3.3\",\n\t\t\"@emotion/use-insertion-effect-with-fallbacks\": \"^1.2.0\",\n\t\t\"@emotion/utils\": \"^1.4.2\",\n\t\t\"@emotion/weak-memoize\": \"^0.4.0\",\n\t\t\"hoist-non-react-statics\": \"^3.3.1\"\n\t},\n\tpeerDependencies: {\n\t\treact: \">=16.8.0\"\n\t},\n\tpeerDependenciesMeta: {\n\t\t\"@types/react\": {\n\t\t\toptional: true\n\t\t}\n\t},\n\tdevDependencies: {\n\t\t\"@definitelytyped/dtslint\": \"0.0.112\",\n\t\t\"@emotion/css\": \"11.13.5\",\n\t\t\"@emotion/css-prettifier\": \"1.2.0\",\n\t\t\"@emotion/server\": \"11.11.0\",\n\t\t\"@emotion/styled\": \"11.14.0\",\n\t\t\"@types/hoist-non-react-statics\": \"^3.3.5\",\n\t\t\"html-tag-names\": \"^1.1.2\",\n\t\treact: \"16.14.0\",\n\t\t\"svg-tag-names\": \"^1.1.1\",\n\t\ttypescript: \"^5.4.5\"\n\t},\n\trepository: \"https://github.com/emotion-js/emotion/tree/main/packages/react\",\n\tpublishConfig: {\n\t\taccess: \"public\"\n\t},\n\t\"umd:main\": \"dist/emotion-react.umd.min.js\",\n\tpreconstruct: {\n\t\tentrypoints: [\n\t\t\t\"./index.ts\",\n\t\t\t\"./jsx-runtime.ts\",\n\t\t\t\"./jsx-dev-runtime.ts\",\n\t\t\t\"./_isolated-hnrs.ts\"\n\t\t],\n\t\tumdName: \"emotionReact\",\n\t\texports: {\n\t\t\textra: {\n\t\t\t\t\"./types/css-prop\": \"./types/css-prop.d.ts\",\n\t\t\t\t\"./macro\": {\n\t\t\t\t\ttypes: {\n\t\t\t\t\t\t\"import\": \"./macro.d.mts\",\n\t\t\t\t\t\t\"default\": \"./macro.d.ts\"\n\t\t\t\t\t},\n\t\t\t\t\t\"default\": \"./macro.js\"\n\t\t\t\t}\n\t\t\t}\n\t\t}\n\t}\n};\n\nvar jsx = function jsx(type, props) {\n  // eslint-disable-next-line prefer-rest-params\n  var args = arguments;\n\n  if (props == null || !_emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.h.call(props, 'css')) {\n    return react__WEBPACK_IMPORTED_MODULE_1__.createElement.apply(undefined, args);\n  }\n\n  var argsLength = args.length;\n  var createElementArgArray = new Array(argsLength);\n  createElementArgArray[0] = _emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.E;\n  createElementArgArray[1] = (0,_emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.c)(type, props);\n\n  for (var i = 2; i < argsLength; i++) {\n    createElementArgArray[i] = args[i];\n  }\n\n  return react__WEBPACK_IMPORTED_MODULE_1__.createElement.apply(null, createElementArgArray);\n};\n\n(function (_jsx) {\n  var JSX;\n\n  (function (_JSX) {})(JSX || (JSX = _jsx.JSX || (_jsx.JSX = {})));\n})(jsx || (jsx = {}));\n\nvar warnedAboutCssPropForGlobal = false; // maintain place over rerenders.\n// initial render from browser, insertBefore context.sheet.tags[0] or if a style hasn't been inserted there yet, appendChild\n// initial client-side render from SSR, use place of hydrating tag\n\nvar Global = /* #__PURE__ */(0,_emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.w)(function (props, cache) {\n  if (!warnedAboutCssPropForGlobal && ( // check for className as well since the user is\n  // probably using the custom createElement which\n  // means it will be turned into a className prop\n  // I don't really want to add it to the type since it shouldn't be used\n  'className' in props && props.className || 'css' in props && props.css)) {\n    console.error(\"It looks like you're using the css prop on Global, did you mean to use the styles prop instead?\");\n    warnedAboutCssPropForGlobal = true;\n  }\n\n  var styles = props.styles;\n  var serialized = (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_4__.serializeStyles)([styles], undefined, react__WEBPACK_IMPORTED_MODULE_1__.useContext(_emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.T));\n  // but it is based on a constant that will never change at runtime\n  // it's effectively like having two implementations and switching them out\n  // so it's not actually breaking anything\n\n\n  var sheetRef = react__WEBPACK_IMPORTED_MODULE_1__.useRef();\n  (0,_emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_3__.useInsertionEffectWithLayoutFallback)(function () {\n    var key = cache.key + \"-global\"; // use case of https://github.com/emotion-js/emotion/issues/2675\n\n    var sheet = new cache.sheet.constructor({\n      key: key,\n      nonce: cache.sheet.nonce,\n      container: cache.sheet.container,\n      speedy: cache.sheet.isSpeedy\n    });\n    var rehydrating = false;\n    var node = document.querySelector(\"style[data-emotion=\\\"\" + key + \" \" + serialized.name + \"\\\"]\");\n\n    if (cache.sheet.tags.length) {\n      sheet.before = cache.sheet.tags[0];\n    }\n\n    if (node !== null) {\n      rehydrating = true; // clear the hash so this node won't be recognizable as rehydratable by other <Global/>s\n\n      node.setAttribute('data-emotion', key);\n      sheet.hydrate([node]);\n    }\n\n    sheetRef.current = [sheet, rehydrating];\n    return function () {\n      sheet.flush();\n    };\n  }, [cache]);\n  (0,_emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_3__.useInsertionEffectWithLayoutFallback)(function () {\n    var sheetRefCurrent = sheetRef.current;\n    var sheet = sheetRefCurrent[0],\n        rehydrating = sheetRefCurrent[1];\n\n    if (rehydrating) {\n      sheetRefCurrent[1] = false;\n      return;\n    }\n\n    if (serialized.next !== undefined) {\n      // insert keyframes\n      (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_2__.insertStyles)(cache, serialized.next, true);\n    }\n\n    if (sheet.tags.length) {\n      // if this doesn't exist then it will be null so the style element will be appended\n      var element = sheet.tags[sheet.tags.length - 1].nextElementSibling;\n      sheet.before = element;\n      sheet.flush();\n    }\n\n    cache.insert(\"\", serialized, sheet, false);\n  }, [cache, serialized.name]);\n  return null;\n});\n\n{\n  Global.displayName = 'EmotionGlobal';\n}\n\nfunction css() {\n  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {\n    args[_key] = arguments[_key];\n  }\n\n  return (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_4__.serializeStyles)(args);\n}\n\nfunction keyframes() {\n  var insertable = css.apply(void 0, arguments);\n  var name = \"animation-\" + insertable.name;\n  return {\n    name: name,\n    styles: \"@keyframes \" + name + \"{\" + insertable.styles + \"}\",\n    anim: 1,\n    toString: function toString() {\n      return \"_EMO_\" + this.name + \"_\" + this.styles + \"_EMO_\";\n    }\n  };\n}\n\nvar classnames = function classnames(args) {\n  var len = args.length;\n  var i = 0;\n  var cls = '';\n\n  for (; i < len; i++) {\n    var arg = args[i];\n    if (arg == null) continue;\n    var toAdd = void 0;\n\n    switch (typeof arg) {\n      case 'boolean':\n        break;\n\n      case 'object':\n        {\n          if (Array.isArray(arg)) {\n            toAdd = classnames(arg);\n          } else {\n            if (arg.styles !== undefined && arg.name !== undefined) {\n              console.error('You have passed styles created with `css` from `@emotion/react` package to the `cx`.\\n' + '`cx` is meant to compose class names (strings) so you should convert those styles to a class name by passing them to the `css` received from <ClassNames/> component.');\n            }\n\n            toAdd = '';\n\n            for (var k in arg) {\n              if (arg[k] && k) {\n                toAdd && (toAdd += ' ');\n                toAdd += k;\n              }\n            }\n          }\n\n          break;\n        }\n\n      default:\n        {\n          toAdd = arg;\n        }\n    }\n\n    if (toAdd) {\n      cls && (cls += ' ');\n      cls += toAdd;\n    }\n  }\n\n  return cls;\n};\n\nfunction merge(registered, css, className) {\n  var registeredStyles = [];\n  var rawClassName = (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_2__.getRegisteredStyles)(registered, registeredStyles, className);\n\n  if (registeredStyles.length < 2) {\n    return className;\n  }\n\n  return rawClassName + css(registeredStyles);\n}\n\nvar Insertion = function Insertion(_ref) {\n  var cache = _ref.cache,\n      serializedArr = _ref.serializedArr;\n  (0,_emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_3__.useInsertionEffectAlwaysWithSyncFallback)(function () {\n\n    for (var i = 0; i < serializedArr.length; i++) {\n      (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_2__.insertStyles)(cache, serializedArr[i], false);\n    }\n  });\n\n  return null;\n};\n\nvar ClassNames = /* #__PURE__ */(0,_emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.w)(function (props, cache) {\n  var hasRendered = false;\n  var serializedArr = [];\n\n  var css = function css() {\n    if (hasRendered && isDevelopment) {\n      throw new Error('css can only be used during render');\n    }\n\n    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {\n      args[_key] = arguments[_key];\n    }\n\n    var serialized = (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_4__.serializeStyles)(args, cache.registered);\n    serializedArr.push(serialized); // registration has to happen here as the result of this might get consumed by `cx`\n\n    (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_2__.registerStyles)(cache, serialized, false);\n    return cache.key + \"-\" + serialized.name;\n  };\n\n  var cx = function cx() {\n    if (hasRendered && isDevelopment) {\n      throw new Error('cx can only be used during render');\n    }\n\n    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {\n      args[_key2] = arguments[_key2];\n    }\n\n    return merge(cache.registered, css, classnames(args));\n  };\n\n  var content = {\n    css: css,\n    cx: cx,\n    theme: react__WEBPACK_IMPORTED_MODULE_1__.useContext(_emotion_element_489459f2_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__.T)\n  };\n  var ele = props.children(content);\n  hasRendered = true;\n  return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_1__.createElement(react__WEBPACK_IMPORTED_MODULE_1__.Fragment, null, /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_1__.createElement(Insertion, {\n    cache: cache,\n    serializedArr: serializedArr\n  }), ele);\n});\n\n{\n  ClassNames.displayName = 'EmotionClassNames';\n}\n\n{\n  var isBrowser = typeof document !== 'undefined'; // #1727, #2905 for some reason Jest and Vitest evaluate modules twice if some consuming module gets mocked\n\n  var isTestEnv = typeof jest !== 'undefined' || typeof vi !== 'undefined';\n\n  if (isBrowser && !isTestEnv) {\n    // globalThis has wide browser support - https://caniuse.com/?search=globalThis, Node.js 12 and later\n    var globalContext = typeof globalThis !== 'undefined' ? globalThis // eslint-disable-line no-undef\n    : isBrowser ? window : __webpack_require__.g;\n    var globalKey = \"__EMOTION_REACT_\" + pkg.version.split('.')[0] + \"__\";\n\n    if (globalContext[globalKey]) {\n      console.warn('You are loading @emotion/react when it is already loaded. Running ' + 'multiple instances may cause problems. This can happen if multiple ' + 'versions are used, or if multiple builds of the same version are ' + 'used.');\n    }\n\n    globalContext[globalKey] = true;\n  }\n}\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js":
/*!***********************************************************************************!*\
  !*** ./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js ***!
  \***********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   serializeStyles: () => (/* binding */ serializeStyles)\n/* harmony export */ });\n/* harmony import */ var _emotion_hash__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @emotion/hash */ \"./node_modules/@emotion/hash/dist/emotion-hash.esm.js\");\n/* harmony import */ var _emotion_unitless__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/unitless */ \"./node_modules/@emotion/unitless/dist/emotion-unitless.esm.js\");\n/* harmony import */ var _emotion_memoize__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/memoize */ \"./node_modules/@emotion/memoize/dist/emotion-memoize.esm.js\");\n\n\n\n\nvar isDevelopment = true;\n\nvar ILLEGAL_ESCAPE_SEQUENCE_ERROR = \"You have illegal escape sequence in your template literal, most likely inside content's property value.\\nBecause you write your CSS inside a JavaScript string you actually have to do double escaping, so for example \\\"content: '\\\\00d7';\\\" should become \\\"content: '\\\\\\\\00d7';\\\".\\nYou can read more about this here:\\nhttps://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Template_literals#ES2018_revision_of_illegal_escape_sequences\";\nvar UNDEFINED_AS_OBJECT_KEY_ERROR = \"You have passed in falsy value as style object's key (can happen when in example you pass unexported component as computed key).\";\nvar hyphenateRegex = /[A-Z]|^ms/g;\nvar animationRegex = /_EMO_([^_]+?)_([^]*?)_EMO_/g;\n\nvar isCustomProperty = function isCustomProperty(property) {\n  return property.charCodeAt(1) === 45;\n};\n\nvar isProcessableValue = function isProcessableValue(value) {\n  return value != null && typeof value !== 'boolean';\n};\n\nvar processStyleName = /* #__PURE__ */(0,_emotion_memoize__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(function (styleName) {\n  return isCustomProperty(styleName) ? styleName : styleName.replace(hyphenateRegex, '-$&').toLowerCase();\n});\n\nvar processStyleValue = function processStyleValue(key, value) {\n  switch (key) {\n    case 'animation':\n    case 'animationName':\n      {\n        if (typeof value === 'string') {\n          return value.replace(animationRegex, function (match, p1, p2) {\n            cursor = {\n              name: p1,\n              styles: p2,\n              next: cursor\n            };\n            return p1;\n          });\n        }\n      }\n  }\n\n  if (_emotion_unitless__WEBPACK_IMPORTED_MODULE_1__[\"default\"][key] !== 1 && !isCustomProperty(key) && typeof value === 'number' && value !== 0) {\n    return value + 'px';\n  }\n\n  return value;\n};\n\n{\n  var contentValuePattern = /(var|attr|counters?|url|element|(((repeating-)?(linear|radial))|conic)-gradient)\\(|(no-)?(open|close)-quote/;\n  var contentValues = ['normal', 'none', 'initial', 'inherit', 'unset'];\n  var oldProcessStyleValue = processStyleValue;\n  var msPattern = /^-ms-/;\n  var hyphenPattern = /-(.)/g;\n  var hyphenatedCache = {};\n\n  processStyleValue = function processStyleValue(key, value) {\n    if (key === 'content') {\n      if (typeof value !== 'string' || contentValues.indexOf(value) === -1 && !contentValuePattern.test(value) && (value.charAt(0) !== value.charAt(value.length - 1) || value.charAt(0) !== '\"' && value.charAt(0) !== \"'\")) {\n        throw new Error(\"You seem to be using a value for 'content' without quotes, try replacing it with `content: '\\\"\" + value + \"\\\"'`\");\n      }\n    }\n\n    var processed = oldProcessStyleValue(key, value);\n\n    if (processed !== '' && !isCustomProperty(key) && key.indexOf('-') !== -1 && hyphenatedCache[key] === undefined) {\n      hyphenatedCache[key] = true;\n      console.error(\"Using kebab-case for css properties in objects is not supported. Did you mean \" + key.replace(msPattern, 'ms-').replace(hyphenPattern, function (str, _char) {\n        return _char.toUpperCase();\n      }) + \"?\");\n    }\n\n    return processed;\n  };\n}\n\nvar noComponentSelectorMessage = 'Component selectors can only be used in conjunction with ' + '@emotion/babel-plugin, the swc Emotion plugin, or another Emotion-aware ' + 'compiler transform.';\n\nfunction handleInterpolation(mergedProps, registered, interpolation) {\n  if (interpolation == null) {\n    return '';\n  }\n\n  var componentSelector = interpolation;\n\n  if (componentSelector.__emotion_styles !== undefined) {\n    if (String(componentSelector) === 'NO_COMPONENT_SELECTOR') {\n      throw new Error(noComponentSelectorMessage);\n    }\n\n    return componentSelector;\n  }\n\n  switch (typeof interpolation) {\n    case 'boolean':\n      {\n        return '';\n      }\n\n    case 'object':\n      {\n        var keyframes = interpolation;\n\n        if (keyframes.anim === 1) {\n          cursor = {\n            name: keyframes.name,\n            styles: keyframes.styles,\n            next: cursor\n          };\n          return keyframes.name;\n        }\n\n        var serializedStyles = interpolation;\n\n        if (serializedStyles.styles !== undefined) {\n          var next = serializedStyles.next;\n\n          if (next !== undefined) {\n            // not the most efficient thing ever but this is a pretty rare case\n            // and there will be very few iterations of this generally\n            while (next !== undefined) {\n              cursor = {\n                name: next.name,\n                styles: next.styles,\n                next: cursor\n              };\n              next = next.next;\n            }\n          }\n\n          var styles = serializedStyles.styles + \";\";\n          return styles;\n        }\n\n        return createStringFromObject(mergedProps, registered, interpolation);\n      }\n\n    case 'function':\n      {\n        if (mergedProps !== undefined) {\n          var previousCursor = cursor;\n          var result = interpolation(mergedProps);\n          cursor = previousCursor;\n          return handleInterpolation(mergedProps, registered, result);\n        } else {\n          console.error('Functions that are interpolated in css calls will be stringified.\\n' + 'If you want to have a css call based on props, create a function that returns a css call like this\\n' + 'let dynamicStyle = (props) => css`color: ${props.color}`\\n' + 'It can be called directly with props or interpolated in a styled call like this\\n' + \"let SomeComponent = styled('div')`${dynamicStyle}`\");\n        }\n\n        break;\n      }\n\n    case 'string':\n      {\n        var matched = [];\n        var replaced = interpolation.replace(animationRegex, function (_match, _p1, p2) {\n          var fakeVarName = \"animation\" + matched.length;\n          matched.push(\"const \" + fakeVarName + \" = keyframes`\" + p2.replace(/^@keyframes animation-\\w+/, '') + \"`\");\n          return \"${\" + fakeVarName + \"}\";\n        });\n\n        if (matched.length) {\n          console.error(\"`keyframes` output got interpolated into plain string, please wrap it with `css`.\\n\\nInstead of doing this:\\n\\n\" + [].concat(matched, [\"`\" + replaced + \"`\"]).join('\\n') + \"\\n\\nYou should wrap it with `css` like this:\\n\\ncss`\" + replaced + \"`\");\n        }\n      }\n\n      break;\n  } // finalize string values (regular strings and functions interpolated into css calls)\n\n\n  var asString = interpolation;\n\n  if (registered == null) {\n    return asString;\n  }\n\n  var cached = registered[asString];\n  return cached !== undefined ? cached : asString;\n}\n\nfunction createStringFromObject(mergedProps, registered, obj) {\n  var string = '';\n\n  if (Array.isArray(obj)) {\n    for (var i = 0; i < obj.length; i++) {\n      string += handleInterpolation(mergedProps, registered, obj[i]) + \";\";\n    }\n  } else {\n    for (var key in obj) {\n      var value = obj[key];\n\n      if (typeof value !== 'object') {\n        var asString = value;\n\n        if (registered != null && registered[asString] !== undefined) {\n          string += key + \"{\" + registered[asString] + \"}\";\n        } else if (isProcessableValue(asString)) {\n          string += processStyleName(key) + \":\" + processStyleValue(key, asString) + \";\";\n        }\n      } else {\n        if (key === 'NO_COMPONENT_SELECTOR' && isDevelopment) {\n          throw new Error(noComponentSelectorMessage);\n        }\n\n        if (Array.isArray(value) && typeof value[0] === 'string' && (registered == null || registered[value[0]] === undefined)) {\n          for (var _i = 0; _i < value.length; _i++) {\n            if (isProcessableValue(value[_i])) {\n              string += processStyleName(key) + \":\" + processStyleValue(key, value[_i]) + \";\";\n            }\n          }\n        } else {\n          var interpolated = handleInterpolation(mergedProps, registered, value);\n\n          switch (key) {\n            case 'animation':\n            case 'animationName':\n              {\n                string += processStyleName(key) + \":\" + interpolated + \";\";\n                break;\n              }\n\n            default:\n              {\n                if (key === 'undefined') {\n                  console.error(UNDEFINED_AS_OBJECT_KEY_ERROR);\n                }\n\n                string += key + \"{\" + interpolated + \"}\";\n              }\n          }\n        }\n      }\n    }\n  }\n\n  return string;\n}\n\nvar labelPattern = /label:\\s*([^\\s;{]+)\\s*(;|$)/g; // this is the cursor for keyframes\n// keyframes are stored on the SerializedStyles object as a linked list\n\nvar cursor;\nfunction serializeStyles(args, registered, mergedProps) {\n  if (args.length === 1 && typeof args[0] === 'object' && args[0] !== null && args[0].styles !== undefined) {\n    return args[0];\n  }\n\n  var stringMode = true;\n  var styles = '';\n  cursor = undefined;\n  var strings = args[0];\n\n  if (strings == null || strings.raw === undefined) {\n    stringMode = false;\n    styles += handleInterpolation(mergedProps, registered, strings);\n  } else {\n    var asTemplateStringsArr = strings;\n\n    if (asTemplateStringsArr[0] === undefined) {\n      console.error(ILLEGAL_ESCAPE_SEQUENCE_ERROR);\n    }\n\n    styles += asTemplateStringsArr[0];\n  } // we start at 1 since we've already handled the first arg\n\n\n  for (var i = 1; i < args.length; i++) {\n    styles += handleInterpolation(mergedProps, registered, args[i]);\n\n    if (stringMode) {\n      var templateStringsArr = strings;\n\n      if (templateStringsArr[i] === undefined) {\n        console.error(ILLEGAL_ESCAPE_SEQUENCE_ERROR);\n      }\n\n      styles += templateStringsArr[i];\n    }\n  } // using a global regex with .exec is stateful so lastIndex has to be reset each time\n\n\n  labelPattern.lastIndex = 0;\n  var identifierName = '';\n  var match; // https://esbench.com/bench/5b809c2cf2949800a0f61fb5\n\n  while ((match = labelPattern.exec(styles)) !== null) {\n    identifierName += '-' + match[1];\n  }\n\n  var name = (0,_emotion_hash__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(styles) + identifierName;\n\n  {\n    var devStyles = {\n      name: name,\n      styles: styles,\n      next: cursor,\n      toString: function toString() {\n        return \"You have tried to stringify object returned from `css` function. It isn't supposed to be used directly (e.g. as value of the `className` prop), but rather handed to emotion so it can handle it (e.g. as value of `css` prop).\";\n      }\n    };\n    return devStyles;\n  }\n}\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/sheet/dist/emotion-sheet.development.esm.js":
/*!***************************************************************************!*\
  !*** ./node_modules/@emotion/sheet/dist/emotion-sheet.development.esm.js ***!
  \***************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   StyleSheet: () => (/* binding */ StyleSheet)\n/* harmony export */ });\nvar isDevelopment = true;\n\n/*\n\nBased off glamor's StyleSheet, thanks Sunil ❤️\n\nhigh performance StyleSheet for css-in-js systems\n\n- uses multiple style tags behind the scenes for millions of rules\n- uses `insertRule` for appending in production for *much* faster performance\n\n// usage\n\nimport { StyleSheet } from '@emotion/sheet'\n\nlet styleSheet = new StyleSheet({ key: '', container: document.head })\n\nstyleSheet.insert('#box { border: 1px solid red; }')\n- appends a css rule into the stylesheet\n\nstyleSheet.flush()\n- empties the stylesheet of all its contents\n\n*/\n\nfunction sheetForTag(tag) {\n  if (tag.sheet) {\n    return tag.sheet;\n  } // this weirdness brought to you by firefox\n\n  /* istanbul ignore next */\n\n\n  for (var i = 0; i < document.styleSheets.length; i++) {\n    if (document.styleSheets[i].ownerNode === tag) {\n      return document.styleSheets[i];\n    }\n  } // this function should always return with a value\n  // TS can't understand it though so we make it stop complaining here\n\n\n  return undefined;\n}\n\nfunction createStyleElement(options) {\n  var tag = document.createElement('style');\n  tag.setAttribute('data-emotion', options.key);\n\n  if (options.nonce !== undefined) {\n    tag.setAttribute('nonce', options.nonce);\n  }\n\n  tag.appendChild(document.createTextNode(''));\n  tag.setAttribute('data-s', '');\n  return tag;\n}\n\nvar StyleSheet = /*#__PURE__*/function () {\n  // Using Node instead of HTMLElement since container may be a ShadowRoot\n  function StyleSheet(options) {\n    var _this = this;\n\n    this._insertTag = function (tag) {\n      var before;\n\n      if (_this.tags.length === 0) {\n        if (_this.insertionPoint) {\n          before = _this.insertionPoint.nextSibling;\n        } else if (_this.prepend) {\n          before = _this.container.firstChild;\n        } else {\n          before = _this.before;\n        }\n      } else {\n        before = _this.tags[_this.tags.length - 1].nextSibling;\n      }\n\n      _this.container.insertBefore(tag, before);\n\n      _this.tags.push(tag);\n    };\n\n    this.isSpeedy = options.speedy === undefined ? !isDevelopment : options.speedy;\n    this.tags = [];\n    this.ctr = 0;\n    this.nonce = options.nonce; // key is the value of the data-emotion attribute, it's used to identify different sheets\n\n    this.key = options.key;\n    this.container = options.container;\n    this.prepend = options.prepend;\n    this.insertionPoint = options.insertionPoint;\n    this.before = null;\n  }\n\n  var _proto = StyleSheet.prototype;\n\n  _proto.hydrate = function hydrate(nodes) {\n    nodes.forEach(this._insertTag);\n  };\n\n  _proto.insert = function insert(rule) {\n    // the max length is how many rules we have per style tag, it's 65000 in speedy mode\n    // it's 1 in dev because we insert source maps that map a single rule to a location\n    // and you can only have one source map per style tag\n    if (this.ctr % (this.isSpeedy ? 65000 : 1) === 0) {\n      this._insertTag(createStyleElement(this));\n    }\n\n    var tag = this.tags[this.tags.length - 1];\n\n    {\n      var isImportRule = rule.charCodeAt(0) === 64 && rule.charCodeAt(1) === 105;\n\n      if (isImportRule && this._alreadyInsertedOrderInsensitiveRule) {\n        // this would only cause problem in speedy mode\n        // but we don't want enabling speedy to affect the observable behavior\n        // so we report this error at all times\n        console.error(\"You're attempting to insert the following rule:\\n\" + rule + '\\n\\n`@import` rules must be before all other types of rules in a stylesheet but other rules have already been inserted. Please ensure that `@import` rules are before all other rules.');\n      }\n\n      this._alreadyInsertedOrderInsensitiveRule = this._alreadyInsertedOrderInsensitiveRule || !isImportRule;\n    }\n\n    if (this.isSpeedy) {\n      var sheet = sheetForTag(tag);\n\n      try {\n        // this is the ultrafast version, works across browsers\n        // the big drawback is that the css won't be editable in devtools\n        sheet.insertRule(rule, sheet.cssRules.length);\n      } catch (e) {\n        if (!/:(-moz-placeholder|-moz-focus-inner|-moz-focusring|-ms-input-placeholder|-moz-read-write|-moz-read-only|-ms-clear|-ms-expand|-ms-reveal){/.test(rule)) {\n          console.error(\"There was a problem inserting the following rule: \\\"\" + rule + \"\\\"\", e);\n        }\n      }\n    } else {\n      tag.appendChild(document.createTextNode(rule));\n    }\n\n    this.ctr++;\n  };\n\n  _proto.flush = function flush() {\n    this.tags.forEach(function (tag) {\n      var _tag$parentNode;\n\n      return (_tag$parentNode = tag.parentNode) == null ? void 0 : _tag$parentNode.removeChild(tag);\n    });\n    this.tags = [];\n    this.ctr = 0;\n\n    {\n      this._alreadyInsertedOrderInsensitiveRule = false;\n    }\n  };\n\n  return StyleSheet;\n}();\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/sheet/dist/emotion-sheet.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/styled/base/dist/emotion-styled-base.browser.development.esm.js":
/*!***********************************************************************************************!*\
  !*** ./node_modules/@emotion/styled/base/dist/emotion-styled-base.browser.development.esm.js ***!
  \***********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createStyled)\n/* harmony export */ });\n/* harmony import */ var _babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @babel/runtime/helpers/esm/extends */ \"./node_modules/@babel/runtime/helpers/esm/extends.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var _emotion_serialize__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/serialize */ \"./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js\");\n/* harmony import */ var _emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/use-insertion-effect-with-fallbacks */ \"./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js\");\n/* harmony import */ var _emotion_utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @emotion/utils */ \"./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_4___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_4__);\n/* harmony import */ var _emotion_is_prop_valid__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @emotion/is-prop-valid */ \"./node_modules/@emotion/is-prop-valid/dist/emotion-is-prop-valid.esm.js\");\n\n\n\n\n\n\n\n\nvar isDevelopment = true;\n\nvar testOmitPropsOnStringTag = _emotion_is_prop_valid__WEBPACK_IMPORTED_MODULE_5__[\"default\"];\n\nvar testOmitPropsOnComponent = function testOmitPropsOnComponent(key) {\n  return key !== 'theme';\n};\n\nvar getDefaultShouldForwardProp = function getDefaultShouldForwardProp(tag) {\n  return typeof tag === 'string' && // 96 is one less than the char code\n  // for \"a\" so this is checking that\n  // it's a lowercase character\n  tag.charCodeAt(0) > 96 ? testOmitPropsOnStringTag : testOmitPropsOnComponent;\n};\nvar composeShouldForwardProps = function composeShouldForwardProps(tag, options, isReal) {\n  var shouldForwardProp;\n\n  if (options) {\n    var optionsShouldForwardProp = options.shouldForwardProp;\n    shouldForwardProp = tag.__emotion_forwardProp && optionsShouldForwardProp ? function (propName) {\n      return tag.__emotion_forwardProp(propName) && optionsShouldForwardProp(propName);\n    } : optionsShouldForwardProp;\n  }\n\n  if (typeof shouldForwardProp !== 'function' && isReal) {\n    shouldForwardProp = tag.__emotion_forwardProp;\n  }\n\n  return shouldForwardProp;\n};\n\nvar ILLEGAL_ESCAPE_SEQUENCE_ERROR = \"You have illegal escape sequence in your template literal, most likely inside content's property value.\\nBecause you write your CSS inside a JavaScript string you actually have to do double escaping, so for example \\\"content: '\\\\00d7';\\\" should become \\\"content: '\\\\\\\\00d7';\\\".\\nYou can read more about this here:\\nhttps://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Template_literals#ES2018_revision_of_illegal_escape_sequences\";\n\nvar Insertion = function Insertion(_ref) {\n  var cache = _ref.cache,\n      serialized = _ref.serialized,\n      isStringTag = _ref.isStringTag;\n  (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_3__.registerStyles)(cache, serialized, isStringTag);\n  (0,_emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_2__.useInsertionEffectAlwaysWithSyncFallback)(function () {\n    return (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_3__.insertStyles)(cache, serialized, isStringTag);\n  });\n\n  return null;\n};\n\nvar createStyled = function createStyled(tag, options) {\n  {\n    if (tag === undefined) {\n      throw new Error('You are trying to create a styled element with an undefined component.\\nYou may have forgotten to import it.');\n    }\n  }\n\n  var isReal = tag.__emotion_real === tag;\n  var baseTag = isReal && tag.__emotion_base || tag;\n  var identifierName;\n  var targetClassName;\n\n  if (options !== undefined) {\n    identifierName = options.label;\n    targetClassName = options.target;\n  }\n\n  var shouldForwardProp = composeShouldForwardProps(tag, options, isReal);\n  var defaultShouldForwardProp = shouldForwardProp || getDefaultShouldForwardProp(baseTag);\n  var shouldUseAs = !defaultShouldForwardProp('as');\n  return function () {\n    // eslint-disable-next-line prefer-rest-params\n    var args = arguments;\n    var styles = isReal && tag.__emotion_styles !== undefined ? tag.__emotion_styles.slice(0) : [];\n\n    if (identifierName !== undefined) {\n      styles.push(\"label:\" + identifierName + \";\");\n    }\n\n    if (args[0] == null || args[0].raw === undefined) {\n      // eslint-disable-next-line prefer-spread\n      styles.push.apply(styles, args);\n    } else {\n      var templateStringsArr = args[0];\n\n      if (templateStringsArr[0] === undefined) {\n        console.error(ILLEGAL_ESCAPE_SEQUENCE_ERROR);\n      }\n\n      styles.push(templateStringsArr[0]);\n      var len = args.length;\n      var i = 1;\n\n      for (; i < len; i++) {\n        if (templateStringsArr[i] === undefined) {\n          console.error(ILLEGAL_ESCAPE_SEQUENCE_ERROR);\n        }\n\n        styles.push(args[i], templateStringsArr[i]);\n      }\n    }\n\n    var Styled = (0,_emotion_react__WEBPACK_IMPORTED_MODULE_6__.w)(function (props, cache, ref) {\n      var FinalTag = shouldUseAs && props.as || baseTag;\n      var className = '';\n      var classInterpolations = [];\n      var mergedProps = props;\n\n      if (props.theme == null) {\n        mergedProps = {};\n\n        for (var key in props) {\n          mergedProps[key] = props[key];\n        }\n\n        mergedProps.theme = react__WEBPACK_IMPORTED_MODULE_4__.useContext(_emotion_react__WEBPACK_IMPORTED_MODULE_6__.T);\n      }\n\n      if (typeof props.className === 'string') {\n        className = (0,_emotion_utils__WEBPACK_IMPORTED_MODULE_3__.getRegisteredStyles)(cache.registered, classInterpolations, props.className);\n      } else if (props.className != null) {\n        className = props.className + \" \";\n      }\n\n      var serialized = (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_1__.serializeStyles)(styles.concat(classInterpolations), cache.registered, mergedProps);\n      className += cache.key + \"-\" + serialized.name;\n\n      if (targetClassName !== undefined) {\n        className += \" \" + targetClassName;\n      }\n\n      var finalShouldForwardProp = shouldUseAs && shouldForwardProp === undefined ? getDefaultShouldForwardProp(FinalTag) : defaultShouldForwardProp;\n      var newProps = {};\n\n      for (var _key in props) {\n        if (shouldUseAs && _key === 'as') continue;\n\n        if (finalShouldForwardProp(_key)) {\n          newProps[_key] = props[_key];\n        }\n      }\n\n      newProps.className = className;\n\n      if (ref) {\n        newProps.ref = ref;\n      }\n\n      return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_4__.createElement(react__WEBPACK_IMPORTED_MODULE_4__.Fragment, null, /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_4__.createElement(Insertion, {\n        cache: cache,\n        serialized: serialized,\n        isStringTag: typeof FinalTag === 'string'\n      }), /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_4__.createElement(FinalTag, newProps));\n    });\n    Styled.displayName = identifierName !== undefined ? identifierName : \"Styled(\" + (typeof baseTag === 'string' ? baseTag : baseTag.displayName || baseTag.name || 'Component') + \")\";\n    Styled.defaultProps = tag.defaultProps;\n    Styled.__emotion_real = Styled;\n    Styled.__emotion_base = baseTag;\n    Styled.__emotion_styles = styles;\n    Styled.__emotion_forwardProp = shouldForwardProp;\n    Object.defineProperty(Styled, 'toString', {\n      value: function value() {\n        if (targetClassName === undefined && isDevelopment) {\n          return 'NO_COMPONENT_SELECTOR';\n        }\n\n        return \".\" + targetClassName;\n      }\n    });\n\n    Styled.withComponent = function (nextTag, nextOptions) {\n      var newStyled = createStyled(nextTag, (0,_babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({}, options, nextOptions, {\n        shouldForwardProp: composeShouldForwardProps(Styled, nextOptions, true)\n      }));\n      return newStyled.apply(void 0, styles);\n    };\n\n    return Styled;\n  };\n};\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/styled/base/dist/emotion-styled-base.browser.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/styled/dist/emotion-styled.browser.development.esm.js":
/*!*************************************************************************************!*\
  !*** ./node_modules/@emotion/styled/dist/emotion-styled.browser.development.esm.js ***!
  \*************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ newStyled)\n/* harmony export */ });\n/* harmony import */ var _base_dist_emotion_styled_base_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../base/dist/emotion-styled-base.browser.development.esm.js */ \"./node_modules/@emotion/styled/base/dist/emotion-styled-base.browser.development.esm.js\");\n/* harmony import */ var _babel_runtime_helpers_extends__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @babel/runtime/helpers/extends */ \"./node_modules/@babel/runtime/helpers/esm/extends.js\");\n/* harmony import */ var _emotion_serialize__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/serialize */ \"./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js\");\n/* harmony import */ var _emotion_use_insertion_effect_with_fallbacks__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @emotion/use-insertion-effect-with-fallbacks */ \"./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js\");\n/* harmony import */ var _emotion_utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @emotion/utils */ \"./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _emotion_is_prop_valid__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @emotion/is-prop-valid */ \"./node_modules/@emotion/is-prop-valid/dist/emotion-is-prop-valid.esm.js\");\n\n\n\n\n\n\n\n\n\nvar tags = ['a', 'abbr', 'address', 'area', 'article', 'aside', 'audio', 'b', 'base', 'bdi', 'bdo', 'big', 'blockquote', 'body', 'br', 'button', 'canvas', 'caption', 'cite', 'code', 'col', 'colgroup', 'data', 'datalist', 'dd', 'del', 'details', 'dfn', 'dialog', 'div', 'dl', 'dt', 'em', 'embed', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'header', 'hgroup', 'hr', 'html', 'i', 'iframe', 'img', 'input', 'ins', 'kbd', 'keygen', 'label', 'legend', 'li', 'link', 'main', 'map', 'mark', 'marquee', 'menu', 'menuitem', 'meta', 'meter', 'nav', 'noscript', 'object', 'ol', 'optgroup', 'option', 'output', 'p', 'param', 'picture', 'pre', 'progress', 'q', 'rp', 'rt', 'ruby', 's', 'samp', 'script', 'section', 'select', 'small', 'source', 'span', 'strong', 'style', 'sub', 'summary', 'sup', 'table', 'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'time', 'title', 'tr', 'track', 'u', 'ul', 'var', 'video', 'wbr', // SVG\n'circle', 'clipPath', 'defs', 'ellipse', 'foreignObject', 'g', 'image', 'line', 'linearGradient', 'mask', 'path', 'pattern', 'polygon', 'polyline', 'radialGradient', 'rect', 'stop', 'svg', 'text', 'tspan'];\n\n// bind it to avoid mutating the original function\nvar newStyled = _base_dist_emotion_styled_base_browser_development_esm_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"].bind(null);\ntags.forEach(function (tagName) {\n  newStyled[tagName] = newStyled(tagName);\n});\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/styled/dist/emotion-styled.browser.development.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/unitless/dist/emotion-unitless.esm.js":
/*!*********************************************************************!*\
  !*** ./node_modules/@emotion/unitless/dist/emotion-unitless.esm.js ***!
  \*********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ unitlessKeys)\n/* harmony export */ });\nvar unitlessKeys = {\n  animationIterationCount: 1,\n  aspectRatio: 1,\n  borderImageOutset: 1,\n  borderImageSlice: 1,\n  borderImageWidth: 1,\n  boxFlex: 1,\n  boxFlexGroup: 1,\n  boxOrdinalGroup: 1,\n  columnCount: 1,\n  columns: 1,\n  flex: 1,\n  flexGrow: 1,\n  flexPositive: 1,\n  flexShrink: 1,\n  flexNegative: 1,\n  flexOrder: 1,\n  gridRow: 1,\n  gridRowEnd: 1,\n  gridRowSpan: 1,\n  gridRowStart: 1,\n  gridColumn: 1,\n  gridColumnEnd: 1,\n  gridColumnSpan: 1,\n  gridColumnStart: 1,\n  msGridRow: 1,\n  msGridRowSpan: 1,\n  msGridColumn: 1,\n  msGridColumnSpan: 1,\n  fontWeight: 1,\n  lineHeight: 1,\n  opacity: 1,\n  order: 1,\n  orphans: 1,\n  scale: 1,\n  tabSize: 1,\n  widows: 1,\n  zIndex: 1,\n  zoom: 1,\n  WebkitLineClamp: 1,\n  // SVG-related properties\n  fillOpacity: 1,\n  floodOpacity: 1,\n  stopOpacity: 1,\n  strokeDasharray: 1,\n  strokeDashoffset: 1,\n  strokeMiterlimit: 1,\n  strokeOpacity: 1,\n  strokeWidth: 1\n};\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/unitless/dist/emotion-unitless.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js":
/*!***********************************************************************************************************************************!*\
  !*** ./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js ***!
  \***********************************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   useInsertionEffectAlwaysWithSyncFallback: () => (/* binding */ useInsertionEffectAlwaysWithSyncFallback),\n/* harmony export */   useInsertionEffectWithLayoutFallback: () => (/* binding */ useInsertionEffectWithLayoutFallback)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n\n\nvar syncFallback = function syncFallback(create) {\n  return create();\n};\n\nvar useInsertionEffect = react__WEBPACK_IMPORTED_MODULE_0__['useInsertion' + 'Effect'] ? react__WEBPACK_IMPORTED_MODULE_0__['useInsertion' + 'Effect'] : false;\nvar useInsertionEffectAlwaysWithSyncFallback = useInsertionEffect || syncFallback;\nvar useInsertionEffectWithLayoutFallback = useInsertionEffect || react__WEBPACK_IMPORTED_MODULE_0__.useLayoutEffect;\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/use-insertion-effect-with-fallbacks/dist/emotion-use-insertion-effect-with-fallbacks.browser.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js":
/*!***********************************************************************!*\
  !*** ./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   getRegisteredStyles: () => (/* binding */ getRegisteredStyles),\n/* harmony export */   insertStyles: () => (/* binding */ insertStyles),\n/* harmony export */   registerStyles: () => (/* binding */ registerStyles)\n/* harmony export */ });\nvar isBrowser = true;\n\nfunction getRegisteredStyles(registered, registeredStyles, classNames) {\n  var rawClassName = '';\n  classNames.split(' ').forEach(function (className) {\n    if (registered[className] !== undefined) {\n      registeredStyles.push(registered[className] + \";\");\n    } else if (className) {\n      rawClassName += className + \" \";\n    }\n  });\n  return rawClassName;\n}\nvar registerStyles = function registerStyles(cache, serialized, isStringTag) {\n  var className = cache.key + \"-\" + serialized.name;\n\n  if ( // we only need to add the styles to the registered cache if the\n  // class name could be used further down\n  // the tree but if it's a string tag, we know it won't\n  // so we don't have to add it to registered cache.\n  // this improves memory usage since we can avoid storing the whole style string\n  (isStringTag === false || // we need to always store it if we're in compat mode and\n  // in node since emotion-server relies on whether a style is in\n  // the registered cache to know whether a style is global or not\n  // also, note that this check will be dead code eliminated in the browser\n  isBrowser === false ) && cache.registered[className] === undefined) {\n    cache.registered[className] = serialized.styles;\n  }\n};\nvar insertStyles = function insertStyles(cache, serialized, isStringTag) {\n  registerStyles(cache, serialized, isStringTag);\n  var className = cache.key + \"-\" + serialized.name;\n\n  if (cache.inserted[serialized.name] === undefined) {\n    var current = serialized;\n\n    do {\n      cache.insert(serialized === current ? \".\" + className : '', current, cache.sheet, true);\n\n      current = current.next;\n    } while (current !== undefined);\n  }\n};\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/utils/dist/emotion-utils.browser.esm.js?");

/***/ }),

/***/ "./node_modules/@emotion/weak-memoize/dist/emotion-weak-memoize.esm.js":
/*!*****************************************************************************!*\
  !*** ./node_modules/@emotion/weak-memoize/dist/emotion-weak-memoize.esm.js ***!
  \*****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ weakMemoize)\n/* harmony export */ });\nvar weakMemoize = function weakMemoize(func) {\n  var cache = new WeakMap();\n  return function (arg) {\n    if (cache.has(arg)) {\n      // Use non-null assertion because we just checked that the cache `has` it\n      // This allows us to remove `undefined` from the return value\n      return cache.get(arg);\n    }\n\n    var ret = func(arg);\n    cache.set(arg, ret);\n    return ret;\n  };\n};\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@emotion/weak-memoize/dist/emotion-weak-memoize.esm.js?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/300.css":
/*!*************************************************!*\
  !*** ./node_modules/@fontsource/roboto/300.css ***!
  \*************************************************/
/***/ ((module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../../css-loader/dist/runtime/noSourceMaps.js */ \"./node_modules/css-loader/dist/runtime/noSourceMaps.js\");\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../../css-loader/dist/runtime/api.js */ \"./node_modules/css-loader/dist/runtime/api.js\");\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../css-loader/dist/runtime/getUrl.js */ \"./node_modules/css-loader/dist/runtime/getUrl.js\");\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__);\n// Imports\n\n\n\nvar ___CSS_LOADER_URL_IMPORT_0___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_1___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_2___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_3___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_4___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_5___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_6___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_7___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_8___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_9___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_10___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_11___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_12___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_13___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_14___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_15___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_16___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-300-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_17___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-300-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_EXPORT___ = _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default()((_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default()));\nvar ___CSS_LOADER_URL_REPLACEMENT_0___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_0___);\nvar ___CSS_LOADER_URL_REPLACEMENT_1___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_1___);\nvar ___CSS_LOADER_URL_REPLACEMENT_2___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_2___);\nvar ___CSS_LOADER_URL_REPLACEMENT_3___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_3___);\nvar ___CSS_LOADER_URL_REPLACEMENT_4___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_4___);\nvar ___CSS_LOADER_URL_REPLACEMENT_5___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_5___);\nvar ___CSS_LOADER_URL_REPLACEMENT_6___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_6___);\nvar ___CSS_LOADER_URL_REPLACEMENT_7___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_7___);\nvar ___CSS_LOADER_URL_REPLACEMENT_8___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_8___);\nvar ___CSS_LOADER_URL_REPLACEMENT_9___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_9___);\nvar ___CSS_LOADER_URL_REPLACEMENT_10___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_10___);\nvar ___CSS_LOADER_URL_REPLACEMENT_11___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_11___);\nvar ___CSS_LOADER_URL_REPLACEMENT_12___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_12___);\nvar ___CSS_LOADER_URL_REPLACEMENT_13___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_13___);\nvar ___CSS_LOADER_URL_REPLACEMENT_14___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_14___);\nvar ___CSS_LOADER_URL_REPLACEMENT_15___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_15___);\nvar ___CSS_LOADER_URL_REPLACEMENT_16___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_16___);\nvar ___CSS_LOADER_URL_REPLACEMENT_17___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_17___);\n// Module\n___CSS_LOADER_EXPORT___.push([module.id, `/* roboto-cyrillic-ext-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_0___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_1___}) format('woff');\n  unicode-range: U+0460-052F,U+1C80-1C8A,U+20B4,U+2DE0-2DFF,U+A640-A69F,U+FE2E-FE2F;\n}\n\n/* roboto-cyrillic-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_2___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_3___}) format('woff');\n  unicode-range: U+0301,U+0400-045F,U+0490-0491,U+04B0-04B1,U+2116;\n}\n\n/* roboto-greek-ext-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_4___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_5___}) format('woff');\n  unicode-range: U+1F00-1FFF;\n}\n\n/* roboto-greek-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_6___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_7___}) format('woff');\n  unicode-range: U+0370-0377,U+037A-037F,U+0384-038A,U+038C,U+038E-03A1,U+03A3-03FF;\n}\n\n/* roboto-math-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_8___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_9___}) format('woff');\n  unicode-range: U+0302-0303,U+0305,U+0307-0308,U+0310,U+0312,U+0315,U+031A,U+0326-0327,U+032C,U+032F-0330,U+0332-0333,U+0338,U+033A,U+0346,U+034D,U+0391-03A1,U+03A3-03A9,U+03B1-03C9,U+03D1,U+03D5-03D6,U+03F0-03F1,U+03F4-03F5,U+2016-2017,U+2034-2038,U+203C,U+2040,U+2043,U+2047,U+2050,U+2057,U+205F,U+2070-2071,U+2074-208E,U+2090-209C,U+20D0-20DC,U+20E1,U+20E5-20EF,U+2100-2112,U+2114-2115,U+2117-2121,U+2123-214F,U+2190,U+2192,U+2194-21AE,U+21B0-21E5,U+21F1-21F2,U+21F4-2211,U+2213-2214,U+2216-22FF,U+2308-230B,U+2310,U+2319,U+231C-2321,U+2336-237A,U+237C,U+2395,U+239B-23B7,U+23D0,U+23DC-23E1,U+2474-2475,U+25AF,U+25B3,U+25B7,U+25BD,U+25C1,U+25CA,U+25CC,U+25FB,U+266D-266F,U+27C0-27FF,U+2900-2AFF,U+2B0E-2B11,U+2B30-2B4C,U+2BFE,U+3030,U+FF5B,U+FF5D,U+1D400-1D7FF,U+1EE00-1EEFF;\n}\n\n/* roboto-symbols-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_10___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_11___}) format('woff');\n  unicode-range: U+0001-000C,U+000E-001F,U+007F-009F,U+20DD-20E0,U+20E2-20E4,U+2150-218F,U+2190,U+2192,U+2194-2199,U+21AF,U+21E6-21F0,U+21F3,U+2218-2219,U+2299,U+22C4-22C6,U+2300-243F,U+2440-244A,U+2460-24FF,U+25A0-27BF,U+2800-28FF,U+2921-2922,U+2981,U+29BF,U+29EB,U+2B00-2BFF,U+4DC0-4DFF,U+FFF9-FFFB,U+10140-1018E,U+10190-1019C,U+101A0,U+101D0-101FD,U+102E0-102FB,U+10E60-10E7E,U+1D2C0-1D2D3,U+1D2E0-1D37F,U+1F000-1F0FF,U+1F100-1F1AD,U+1F1E6-1F1FF,U+1F30D-1F30F,U+1F315,U+1F31C,U+1F31E,U+1F320-1F32C,U+1F336,U+1F378,U+1F37D,U+1F382,U+1F393-1F39F,U+1F3A7-1F3A8,U+1F3AC-1F3AF,U+1F3C2,U+1F3C4-1F3C6,U+1F3CA-1F3CE,U+1F3D4-1F3E0,U+1F3ED,U+1F3F1-1F3F3,U+1F3F5-1F3F7,U+1F408,U+1F415,U+1F41F,U+1F426,U+1F43F,U+1F441-1F442,U+1F444,U+1F446-1F449,U+1F44C-1F44E,U+1F453,U+1F46A,U+1F47D,U+1F4A3,U+1F4B0,U+1F4B3,U+1F4B9,U+1F4BB,U+1F4BF,U+1F4C8-1F4CB,U+1F4D6,U+1F4DA,U+1F4DF,U+1F4E3-1F4E6,U+1F4EA-1F4ED,U+1F4F7,U+1F4F9-1F4FB,U+1F4FD-1F4FE,U+1F503,U+1F507-1F50B,U+1F50D,U+1F512-1F513,U+1F53E-1F54A,U+1F54F-1F5FA,U+1F610,U+1F650-1F67F,U+1F687,U+1F68D,U+1F691,U+1F694,U+1F698,U+1F6AD,U+1F6B2,U+1F6B9-1F6BA,U+1F6BC,U+1F6C6-1F6CF,U+1F6D3-1F6D7,U+1F6E0-1F6EA,U+1F6F0-1F6F3,U+1F6F7-1F6FC,U+1F700-1F7FF,U+1F800-1F80B,U+1F810-1F847,U+1F850-1F859,U+1F860-1F887,U+1F890-1F8AD,U+1F8B0-1F8BB,U+1F8C0-1F8C1,U+1F900-1F90B,U+1F93B,U+1F946,U+1F984,U+1F996,U+1F9E9,U+1FA00-1FA6F,U+1FA70-1FA7C,U+1FA80-1FA89,U+1FA8F-1FAC6,U+1FACE-1FADC,U+1FADF-1FAE9,U+1FAF0-1FAF8,U+1FB00-1FBFF;\n}\n\n/* roboto-vietnamese-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_12___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_13___}) format('woff');\n  unicode-range: U+0102-0103,U+0110-0111,U+0128-0129,U+0168-0169,U+01A0-01A1,U+01AF-01B0,U+0300-0301,U+0303-0304,U+0308-0309,U+0323,U+0329,U+1EA0-1EF9,U+20AB;\n}\n\n/* roboto-latin-ext-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_14___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_15___}) format('woff');\n  unicode-range: U+0100-02BA,U+02BD-02C5,U+02C7-02CC,U+02CE-02D7,U+02DD-02FF,U+0304,U+0308,U+0329,U+1D00-1DBF,U+1E00-1E9F,U+1EF2-1EFF,U+2020,U+20A0-20AB,U+20AD-20C0,U+2113,U+2C60-2C7F,U+A720-A7FF;\n}\n\n/* roboto-latin-300-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 300;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_16___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_17___}) format('woff');\n  unicode-range: U+0000-00FF,U+0131,U+0152-0153,U+02BB-02BC,U+02C6,U+02DA,U+02DC,U+0304,U+0308,U+0329,U+2000-206F,U+20AC,U+2122,U+2191,U+2193,U+2212,U+2215,U+FEFF,U+FFFD;\n}`, \"\"]);\n// Exports\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (___CSS_LOADER_EXPORT___);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/300.css?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/400.css":
/*!*************************************************!*\
  !*** ./node_modules/@fontsource/roboto/400.css ***!
  \*************************************************/
/***/ ((module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../../css-loader/dist/runtime/noSourceMaps.js */ \"./node_modules/css-loader/dist/runtime/noSourceMaps.js\");\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../../css-loader/dist/runtime/api.js */ \"./node_modules/css-loader/dist/runtime/api.js\");\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../css-loader/dist/runtime/getUrl.js */ \"./node_modules/css-loader/dist/runtime/getUrl.js\");\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__);\n// Imports\n\n\n\nvar ___CSS_LOADER_URL_IMPORT_0___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_1___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_2___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_3___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_4___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_5___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_6___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_7___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_8___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_9___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_10___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_11___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_12___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_13___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_14___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_15___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_16___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-400-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_17___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-400-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_EXPORT___ = _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default()((_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default()));\nvar ___CSS_LOADER_URL_REPLACEMENT_0___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_0___);\nvar ___CSS_LOADER_URL_REPLACEMENT_1___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_1___);\nvar ___CSS_LOADER_URL_REPLACEMENT_2___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_2___);\nvar ___CSS_LOADER_URL_REPLACEMENT_3___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_3___);\nvar ___CSS_LOADER_URL_REPLACEMENT_4___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_4___);\nvar ___CSS_LOADER_URL_REPLACEMENT_5___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_5___);\nvar ___CSS_LOADER_URL_REPLACEMENT_6___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_6___);\nvar ___CSS_LOADER_URL_REPLACEMENT_7___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_7___);\nvar ___CSS_LOADER_URL_REPLACEMENT_8___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_8___);\nvar ___CSS_LOADER_URL_REPLACEMENT_9___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_9___);\nvar ___CSS_LOADER_URL_REPLACEMENT_10___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_10___);\nvar ___CSS_LOADER_URL_REPLACEMENT_11___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_11___);\nvar ___CSS_LOADER_URL_REPLACEMENT_12___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_12___);\nvar ___CSS_LOADER_URL_REPLACEMENT_13___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_13___);\nvar ___CSS_LOADER_URL_REPLACEMENT_14___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_14___);\nvar ___CSS_LOADER_URL_REPLACEMENT_15___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_15___);\nvar ___CSS_LOADER_URL_REPLACEMENT_16___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_16___);\nvar ___CSS_LOADER_URL_REPLACEMENT_17___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_17___);\n// Module\n___CSS_LOADER_EXPORT___.push([module.id, `/* roboto-cyrillic-ext-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_0___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_1___}) format('woff');\n  unicode-range: U+0460-052F,U+1C80-1C8A,U+20B4,U+2DE0-2DFF,U+A640-A69F,U+FE2E-FE2F;\n}\n\n/* roboto-cyrillic-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_2___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_3___}) format('woff');\n  unicode-range: U+0301,U+0400-045F,U+0490-0491,U+04B0-04B1,U+2116;\n}\n\n/* roboto-greek-ext-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_4___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_5___}) format('woff');\n  unicode-range: U+1F00-1FFF;\n}\n\n/* roboto-greek-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_6___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_7___}) format('woff');\n  unicode-range: U+0370-0377,U+037A-037F,U+0384-038A,U+038C,U+038E-03A1,U+03A3-03FF;\n}\n\n/* roboto-math-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_8___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_9___}) format('woff');\n  unicode-range: U+0302-0303,U+0305,U+0307-0308,U+0310,U+0312,U+0315,U+031A,U+0326-0327,U+032C,U+032F-0330,U+0332-0333,U+0338,U+033A,U+0346,U+034D,U+0391-03A1,U+03A3-03A9,U+03B1-03C9,U+03D1,U+03D5-03D6,U+03F0-03F1,U+03F4-03F5,U+2016-2017,U+2034-2038,U+203C,U+2040,U+2043,U+2047,U+2050,U+2057,U+205F,U+2070-2071,U+2074-208E,U+2090-209C,U+20D0-20DC,U+20E1,U+20E5-20EF,U+2100-2112,U+2114-2115,U+2117-2121,U+2123-214F,U+2190,U+2192,U+2194-21AE,U+21B0-21E5,U+21F1-21F2,U+21F4-2211,U+2213-2214,U+2216-22FF,U+2308-230B,U+2310,U+2319,U+231C-2321,U+2336-237A,U+237C,U+2395,U+239B-23B7,U+23D0,U+23DC-23E1,U+2474-2475,U+25AF,U+25B3,U+25B7,U+25BD,U+25C1,U+25CA,U+25CC,U+25FB,U+266D-266F,U+27C0-27FF,U+2900-2AFF,U+2B0E-2B11,U+2B30-2B4C,U+2BFE,U+3030,U+FF5B,U+FF5D,U+1D400-1D7FF,U+1EE00-1EEFF;\n}\n\n/* roboto-symbols-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_10___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_11___}) format('woff');\n  unicode-range: U+0001-000C,U+000E-001F,U+007F-009F,U+20DD-20E0,U+20E2-20E4,U+2150-218F,U+2190,U+2192,U+2194-2199,U+21AF,U+21E6-21F0,U+21F3,U+2218-2219,U+2299,U+22C4-22C6,U+2300-243F,U+2440-244A,U+2460-24FF,U+25A0-27BF,U+2800-28FF,U+2921-2922,U+2981,U+29BF,U+29EB,U+2B00-2BFF,U+4DC0-4DFF,U+FFF9-FFFB,U+10140-1018E,U+10190-1019C,U+101A0,U+101D0-101FD,U+102E0-102FB,U+10E60-10E7E,U+1D2C0-1D2D3,U+1D2E0-1D37F,U+1F000-1F0FF,U+1F100-1F1AD,U+1F1E6-1F1FF,U+1F30D-1F30F,U+1F315,U+1F31C,U+1F31E,U+1F320-1F32C,U+1F336,U+1F378,U+1F37D,U+1F382,U+1F393-1F39F,U+1F3A7-1F3A8,U+1F3AC-1F3AF,U+1F3C2,U+1F3C4-1F3C6,U+1F3CA-1F3CE,U+1F3D4-1F3E0,U+1F3ED,U+1F3F1-1F3F3,U+1F3F5-1F3F7,U+1F408,U+1F415,U+1F41F,U+1F426,U+1F43F,U+1F441-1F442,U+1F444,U+1F446-1F449,U+1F44C-1F44E,U+1F453,U+1F46A,U+1F47D,U+1F4A3,U+1F4B0,U+1F4B3,U+1F4B9,U+1F4BB,U+1F4BF,U+1F4C8-1F4CB,U+1F4D6,U+1F4DA,U+1F4DF,U+1F4E3-1F4E6,U+1F4EA-1F4ED,U+1F4F7,U+1F4F9-1F4FB,U+1F4FD-1F4FE,U+1F503,U+1F507-1F50B,U+1F50D,U+1F512-1F513,U+1F53E-1F54A,U+1F54F-1F5FA,U+1F610,U+1F650-1F67F,U+1F687,U+1F68D,U+1F691,U+1F694,U+1F698,U+1F6AD,U+1F6B2,U+1F6B9-1F6BA,U+1F6BC,U+1F6C6-1F6CF,U+1F6D3-1F6D7,U+1F6E0-1F6EA,U+1F6F0-1F6F3,U+1F6F7-1F6FC,U+1F700-1F7FF,U+1F800-1F80B,U+1F810-1F847,U+1F850-1F859,U+1F860-1F887,U+1F890-1F8AD,U+1F8B0-1F8BB,U+1F8C0-1F8C1,U+1F900-1F90B,U+1F93B,U+1F946,U+1F984,U+1F996,U+1F9E9,U+1FA00-1FA6F,U+1FA70-1FA7C,U+1FA80-1FA89,U+1FA8F-1FAC6,U+1FACE-1FADC,U+1FADF-1FAE9,U+1FAF0-1FAF8,U+1FB00-1FBFF;\n}\n\n/* roboto-vietnamese-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_12___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_13___}) format('woff');\n  unicode-range: U+0102-0103,U+0110-0111,U+0128-0129,U+0168-0169,U+01A0-01A1,U+01AF-01B0,U+0300-0301,U+0303-0304,U+0308-0309,U+0323,U+0329,U+1EA0-1EF9,U+20AB;\n}\n\n/* roboto-latin-ext-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_14___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_15___}) format('woff');\n  unicode-range: U+0100-02BA,U+02BD-02C5,U+02C7-02CC,U+02CE-02D7,U+02DD-02FF,U+0304,U+0308,U+0329,U+1D00-1DBF,U+1E00-1E9F,U+1EF2-1EFF,U+2020,U+20A0-20AB,U+20AD-20C0,U+2113,U+2C60-2C7F,U+A720-A7FF;\n}\n\n/* roboto-latin-400-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 400;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_16___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_17___}) format('woff');\n  unicode-range: U+0000-00FF,U+0131,U+0152-0153,U+02BB-02BC,U+02C6,U+02DA,U+02DC,U+0304,U+0308,U+0329,U+2000-206F,U+20AC,U+2122,U+2191,U+2193,U+2212,U+2215,U+FEFF,U+FFFD;\n}`, \"\"]);\n// Exports\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (___CSS_LOADER_EXPORT___);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/400.css?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/500.css":
/*!*************************************************!*\
  !*** ./node_modules/@fontsource/roboto/500.css ***!
  \*************************************************/
/***/ ((module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../../css-loader/dist/runtime/noSourceMaps.js */ \"./node_modules/css-loader/dist/runtime/noSourceMaps.js\");\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../../css-loader/dist/runtime/api.js */ \"./node_modules/css-loader/dist/runtime/api.js\");\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../css-loader/dist/runtime/getUrl.js */ \"./node_modules/css-loader/dist/runtime/getUrl.js\");\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__);\n// Imports\n\n\n\nvar ___CSS_LOADER_URL_IMPORT_0___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_1___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_2___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_3___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_4___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_5___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_6___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_7___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_8___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_9___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_10___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_11___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_12___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_13___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_14___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_15___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_16___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-500-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_17___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-500-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_EXPORT___ = _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default()((_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default()));\nvar ___CSS_LOADER_URL_REPLACEMENT_0___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_0___);\nvar ___CSS_LOADER_URL_REPLACEMENT_1___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_1___);\nvar ___CSS_LOADER_URL_REPLACEMENT_2___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_2___);\nvar ___CSS_LOADER_URL_REPLACEMENT_3___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_3___);\nvar ___CSS_LOADER_URL_REPLACEMENT_4___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_4___);\nvar ___CSS_LOADER_URL_REPLACEMENT_5___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_5___);\nvar ___CSS_LOADER_URL_REPLACEMENT_6___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_6___);\nvar ___CSS_LOADER_URL_REPLACEMENT_7___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_7___);\nvar ___CSS_LOADER_URL_REPLACEMENT_8___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_8___);\nvar ___CSS_LOADER_URL_REPLACEMENT_9___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_9___);\nvar ___CSS_LOADER_URL_REPLACEMENT_10___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_10___);\nvar ___CSS_LOADER_URL_REPLACEMENT_11___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_11___);\nvar ___CSS_LOADER_URL_REPLACEMENT_12___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_12___);\nvar ___CSS_LOADER_URL_REPLACEMENT_13___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_13___);\nvar ___CSS_LOADER_URL_REPLACEMENT_14___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_14___);\nvar ___CSS_LOADER_URL_REPLACEMENT_15___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_15___);\nvar ___CSS_LOADER_URL_REPLACEMENT_16___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_16___);\nvar ___CSS_LOADER_URL_REPLACEMENT_17___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_17___);\n// Module\n___CSS_LOADER_EXPORT___.push([module.id, `/* roboto-cyrillic-ext-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_0___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_1___}) format('woff');\n  unicode-range: U+0460-052F,U+1C80-1C8A,U+20B4,U+2DE0-2DFF,U+A640-A69F,U+FE2E-FE2F;\n}\n\n/* roboto-cyrillic-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_2___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_3___}) format('woff');\n  unicode-range: U+0301,U+0400-045F,U+0490-0491,U+04B0-04B1,U+2116;\n}\n\n/* roboto-greek-ext-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_4___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_5___}) format('woff');\n  unicode-range: U+1F00-1FFF;\n}\n\n/* roboto-greek-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_6___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_7___}) format('woff');\n  unicode-range: U+0370-0377,U+037A-037F,U+0384-038A,U+038C,U+038E-03A1,U+03A3-03FF;\n}\n\n/* roboto-math-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_8___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_9___}) format('woff');\n  unicode-range: U+0302-0303,U+0305,U+0307-0308,U+0310,U+0312,U+0315,U+031A,U+0326-0327,U+032C,U+032F-0330,U+0332-0333,U+0338,U+033A,U+0346,U+034D,U+0391-03A1,U+03A3-03A9,U+03B1-03C9,U+03D1,U+03D5-03D6,U+03F0-03F1,U+03F4-03F5,U+2016-2017,U+2034-2038,U+203C,U+2040,U+2043,U+2047,U+2050,U+2057,U+205F,U+2070-2071,U+2074-208E,U+2090-209C,U+20D0-20DC,U+20E1,U+20E5-20EF,U+2100-2112,U+2114-2115,U+2117-2121,U+2123-214F,U+2190,U+2192,U+2194-21AE,U+21B0-21E5,U+21F1-21F2,U+21F4-2211,U+2213-2214,U+2216-22FF,U+2308-230B,U+2310,U+2319,U+231C-2321,U+2336-237A,U+237C,U+2395,U+239B-23B7,U+23D0,U+23DC-23E1,U+2474-2475,U+25AF,U+25B3,U+25B7,U+25BD,U+25C1,U+25CA,U+25CC,U+25FB,U+266D-266F,U+27C0-27FF,U+2900-2AFF,U+2B0E-2B11,U+2B30-2B4C,U+2BFE,U+3030,U+FF5B,U+FF5D,U+1D400-1D7FF,U+1EE00-1EEFF;\n}\n\n/* roboto-symbols-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_10___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_11___}) format('woff');\n  unicode-range: U+0001-000C,U+000E-001F,U+007F-009F,U+20DD-20E0,U+20E2-20E4,U+2150-218F,U+2190,U+2192,U+2194-2199,U+21AF,U+21E6-21F0,U+21F3,U+2218-2219,U+2299,U+22C4-22C6,U+2300-243F,U+2440-244A,U+2460-24FF,U+25A0-27BF,U+2800-28FF,U+2921-2922,U+2981,U+29BF,U+29EB,U+2B00-2BFF,U+4DC0-4DFF,U+FFF9-FFFB,U+10140-1018E,U+10190-1019C,U+101A0,U+101D0-101FD,U+102E0-102FB,U+10E60-10E7E,U+1D2C0-1D2D3,U+1D2E0-1D37F,U+1F000-1F0FF,U+1F100-1F1AD,U+1F1E6-1F1FF,U+1F30D-1F30F,U+1F315,U+1F31C,U+1F31E,U+1F320-1F32C,U+1F336,U+1F378,U+1F37D,U+1F382,U+1F393-1F39F,U+1F3A7-1F3A8,U+1F3AC-1F3AF,U+1F3C2,U+1F3C4-1F3C6,U+1F3CA-1F3CE,U+1F3D4-1F3E0,U+1F3ED,U+1F3F1-1F3F3,U+1F3F5-1F3F7,U+1F408,U+1F415,U+1F41F,U+1F426,U+1F43F,U+1F441-1F442,U+1F444,U+1F446-1F449,U+1F44C-1F44E,U+1F453,U+1F46A,U+1F47D,U+1F4A3,U+1F4B0,U+1F4B3,U+1F4B9,U+1F4BB,U+1F4BF,U+1F4C8-1F4CB,U+1F4D6,U+1F4DA,U+1F4DF,U+1F4E3-1F4E6,U+1F4EA-1F4ED,U+1F4F7,U+1F4F9-1F4FB,U+1F4FD-1F4FE,U+1F503,U+1F507-1F50B,U+1F50D,U+1F512-1F513,U+1F53E-1F54A,U+1F54F-1F5FA,U+1F610,U+1F650-1F67F,U+1F687,U+1F68D,U+1F691,U+1F694,U+1F698,U+1F6AD,U+1F6B2,U+1F6B9-1F6BA,U+1F6BC,U+1F6C6-1F6CF,U+1F6D3-1F6D7,U+1F6E0-1F6EA,U+1F6F0-1F6F3,U+1F6F7-1F6FC,U+1F700-1F7FF,U+1F800-1F80B,U+1F810-1F847,U+1F850-1F859,U+1F860-1F887,U+1F890-1F8AD,U+1F8B0-1F8BB,U+1F8C0-1F8C1,U+1F900-1F90B,U+1F93B,U+1F946,U+1F984,U+1F996,U+1F9E9,U+1FA00-1FA6F,U+1FA70-1FA7C,U+1FA80-1FA89,U+1FA8F-1FAC6,U+1FACE-1FADC,U+1FADF-1FAE9,U+1FAF0-1FAF8,U+1FB00-1FBFF;\n}\n\n/* roboto-vietnamese-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_12___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_13___}) format('woff');\n  unicode-range: U+0102-0103,U+0110-0111,U+0128-0129,U+0168-0169,U+01A0-01A1,U+01AF-01B0,U+0300-0301,U+0303-0304,U+0308-0309,U+0323,U+0329,U+1EA0-1EF9,U+20AB;\n}\n\n/* roboto-latin-ext-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_14___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_15___}) format('woff');\n  unicode-range: U+0100-02BA,U+02BD-02C5,U+02C7-02CC,U+02CE-02D7,U+02DD-02FF,U+0304,U+0308,U+0329,U+1D00-1DBF,U+1E00-1E9F,U+1EF2-1EFF,U+2020,U+20A0-20AB,U+20AD-20C0,U+2113,U+2C60-2C7F,U+A720-A7FF;\n}\n\n/* roboto-latin-500-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 500;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_16___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_17___}) format('woff');\n  unicode-range: U+0000-00FF,U+0131,U+0152-0153,U+02BB-02BC,U+02C6,U+02DA,U+02DC,U+0304,U+0308,U+0329,U+2000-206F,U+20AC,U+2122,U+2191,U+2193,U+2212,U+2215,U+FEFF,U+FFFD;\n}`, \"\"]);\n// Exports\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (___CSS_LOADER_EXPORT___);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/500.css?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/700.css":
/*!*************************************************!*\
  !*** ./node_modules/@fontsource/roboto/700.css ***!
  \*************************************************/
/***/ ((module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../../css-loader/dist/runtime/noSourceMaps.js */ \"./node_modules/css-loader/dist/runtime/noSourceMaps.js\");\n/* harmony import */ var _css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../../css-loader/dist/runtime/api.js */ \"./node_modules/css-loader/dist/runtime/api.js\");\n/* harmony import */ var _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../css-loader/dist/runtime/getUrl.js */ \"./node_modules/css-loader/dist/runtime/getUrl.js\");\n/* harmony import */ var _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2__);\n// Imports\n\n\n\nvar ___CSS_LOADER_URL_IMPORT_0___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_1___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-ext-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_2___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_3___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-cyrillic-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_4___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_5___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-ext-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_6___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_7___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-greek-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_8___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_9___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-math-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_10___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_11___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-symbols-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_12___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_13___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-vietnamese-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_14___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_15___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-ext-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_16___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-700-normal.woff2 */ \"./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff2\"), __webpack_require__.b);\nvar ___CSS_LOADER_URL_IMPORT_17___ = new URL(/* asset import */ __webpack_require__(/*! ./files/roboto-latin-700-normal.woff */ \"./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff\"), __webpack_require__.b);\nvar ___CSS_LOADER_EXPORT___ = _css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default()((_css_loader_dist_runtime_noSourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default()));\nvar ___CSS_LOADER_URL_REPLACEMENT_0___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_0___);\nvar ___CSS_LOADER_URL_REPLACEMENT_1___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_1___);\nvar ___CSS_LOADER_URL_REPLACEMENT_2___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_2___);\nvar ___CSS_LOADER_URL_REPLACEMENT_3___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_3___);\nvar ___CSS_LOADER_URL_REPLACEMENT_4___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_4___);\nvar ___CSS_LOADER_URL_REPLACEMENT_5___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_5___);\nvar ___CSS_LOADER_URL_REPLACEMENT_6___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_6___);\nvar ___CSS_LOADER_URL_REPLACEMENT_7___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_7___);\nvar ___CSS_LOADER_URL_REPLACEMENT_8___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_8___);\nvar ___CSS_LOADER_URL_REPLACEMENT_9___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_9___);\nvar ___CSS_LOADER_URL_REPLACEMENT_10___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_10___);\nvar ___CSS_LOADER_URL_REPLACEMENT_11___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_11___);\nvar ___CSS_LOADER_URL_REPLACEMENT_12___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_12___);\nvar ___CSS_LOADER_URL_REPLACEMENT_13___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_13___);\nvar ___CSS_LOADER_URL_REPLACEMENT_14___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_14___);\nvar ___CSS_LOADER_URL_REPLACEMENT_15___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_15___);\nvar ___CSS_LOADER_URL_REPLACEMENT_16___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_16___);\nvar ___CSS_LOADER_URL_REPLACEMENT_17___ = _css_loader_dist_runtime_getUrl_js__WEBPACK_IMPORTED_MODULE_2___default()(___CSS_LOADER_URL_IMPORT_17___);\n// Module\n___CSS_LOADER_EXPORT___.push([module.id, `/* roboto-cyrillic-ext-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_0___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_1___}) format('woff');\n  unicode-range: U+0460-052F,U+1C80-1C8A,U+20B4,U+2DE0-2DFF,U+A640-A69F,U+FE2E-FE2F;\n}\n\n/* roboto-cyrillic-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_2___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_3___}) format('woff');\n  unicode-range: U+0301,U+0400-045F,U+0490-0491,U+04B0-04B1,U+2116;\n}\n\n/* roboto-greek-ext-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_4___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_5___}) format('woff');\n  unicode-range: U+1F00-1FFF;\n}\n\n/* roboto-greek-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_6___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_7___}) format('woff');\n  unicode-range: U+0370-0377,U+037A-037F,U+0384-038A,U+038C,U+038E-03A1,U+03A3-03FF;\n}\n\n/* roboto-math-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_8___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_9___}) format('woff');\n  unicode-range: U+0302-0303,U+0305,U+0307-0308,U+0310,U+0312,U+0315,U+031A,U+0326-0327,U+032C,U+032F-0330,U+0332-0333,U+0338,U+033A,U+0346,U+034D,U+0391-03A1,U+03A3-03A9,U+03B1-03C9,U+03D1,U+03D5-03D6,U+03F0-03F1,U+03F4-03F5,U+2016-2017,U+2034-2038,U+203C,U+2040,U+2043,U+2047,U+2050,U+2057,U+205F,U+2070-2071,U+2074-208E,U+2090-209C,U+20D0-20DC,U+20E1,U+20E5-20EF,U+2100-2112,U+2114-2115,U+2117-2121,U+2123-214F,U+2190,U+2192,U+2194-21AE,U+21B0-21E5,U+21F1-21F2,U+21F4-2211,U+2213-2214,U+2216-22FF,U+2308-230B,U+2310,U+2319,U+231C-2321,U+2336-237A,U+237C,U+2395,U+239B-23B7,U+23D0,U+23DC-23E1,U+2474-2475,U+25AF,U+25B3,U+25B7,U+25BD,U+25C1,U+25CA,U+25CC,U+25FB,U+266D-266F,U+27C0-27FF,U+2900-2AFF,U+2B0E-2B11,U+2B30-2B4C,U+2BFE,U+3030,U+FF5B,U+FF5D,U+1D400-1D7FF,U+1EE00-1EEFF;\n}\n\n/* roboto-symbols-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_10___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_11___}) format('woff');\n  unicode-range: U+0001-000C,U+000E-001F,U+007F-009F,U+20DD-20E0,U+20E2-20E4,U+2150-218F,U+2190,U+2192,U+2194-2199,U+21AF,U+21E6-21F0,U+21F3,U+2218-2219,U+2299,U+22C4-22C6,U+2300-243F,U+2440-244A,U+2460-24FF,U+25A0-27BF,U+2800-28FF,U+2921-2922,U+2981,U+29BF,U+29EB,U+2B00-2BFF,U+4DC0-4DFF,U+FFF9-FFFB,U+10140-1018E,U+10190-1019C,U+101A0,U+101D0-101FD,U+102E0-102FB,U+10E60-10E7E,U+1D2C0-1D2D3,U+1D2E0-1D37F,U+1F000-1F0FF,U+1F100-1F1AD,U+1F1E6-1F1FF,U+1F30D-1F30F,U+1F315,U+1F31C,U+1F31E,U+1F320-1F32C,U+1F336,U+1F378,U+1F37D,U+1F382,U+1F393-1F39F,U+1F3A7-1F3A8,U+1F3AC-1F3AF,U+1F3C2,U+1F3C4-1F3C6,U+1F3CA-1F3CE,U+1F3D4-1F3E0,U+1F3ED,U+1F3F1-1F3F3,U+1F3F5-1F3F7,U+1F408,U+1F415,U+1F41F,U+1F426,U+1F43F,U+1F441-1F442,U+1F444,U+1F446-1F449,U+1F44C-1F44E,U+1F453,U+1F46A,U+1F47D,U+1F4A3,U+1F4B0,U+1F4B3,U+1F4B9,U+1F4BB,U+1F4BF,U+1F4C8-1F4CB,U+1F4D6,U+1F4DA,U+1F4DF,U+1F4E3-1F4E6,U+1F4EA-1F4ED,U+1F4F7,U+1F4F9-1F4FB,U+1F4FD-1F4FE,U+1F503,U+1F507-1F50B,U+1F50D,U+1F512-1F513,U+1F53E-1F54A,U+1F54F-1F5FA,U+1F610,U+1F650-1F67F,U+1F687,U+1F68D,U+1F691,U+1F694,U+1F698,U+1F6AD,U+1F6B2,U+1F6B9-1F6BA,U+1F6BC,U+1F6C6-1F6CF,U+1F6D3-1F6D7,U+1F6E0-1F6EA,U+1F6F0-1F6F3,U+1F6F7-1F6FC,U+1F700-1F7FF,U+1F800-1F80B,U+1F810-1F847,U+1F850-1F859,U+1F860-1F887,U+1F890-1F8AD,U+1F8B0-1F8BB,U+1F8C0-1F8C1,U+1F900-1F90B,U+1F93B,U+1F946,U+1F984,U+1F996,U+1F9E9,U+1FA00-1FA6F,U+1FA70-1FA7C,U+1FA80-1FA89,U+1FA8F-1FAC6,U+1FACE-1FADC,U+1FADF-1FAE9,U+1FAF0-1FAF8,U+1FB00-1FBFF;\n}\n\n/* roboto-vietnamese-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_12___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_13___}) format('woff');\n  unicode-range: U+0102-0103,U+0110-0111,U+0128-0129,U+0168-0169,U+01A0-01A1,U+01AF-01B0,U+0300-0301,U+0303-0304,U+0308-0309,U+0323,U+0329,U+1EA0-1EF9,U+20AB;\n}\n\n/* roboto-latin-ext-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_14___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_15___}) format('woff');\n  unicode-range: U+0100-02BA,U+02BD-02C5,U+02C7-02CC,U+02CE-02D7,U+02DD-02FF,U+0304,U+0308,U+0329,U+1D00-1DBF,U+1E00-1E9F,U+1EF2-1EFF,U+2020,U+20A0-20AB,U+20AD-20C0,U+2113,U+2C60-2C7F,U+A720-A7FF;\n}\n\n/* roboto-latin-700-normal */\n@font-face {\n  font-family: 'Roboto';\n  font-style: normal;\n  font-display: swap;\n  font-weight: 700;\n  src: url(${___CSS_LOADER_URL_REPLACEMENT_16___}) format('woff2'), url(${___CSS_LOADER_URL_REPLACEMENT_17___}) format('woff');\n  unicode-range: U+0000-00FF,U+0131,U+0152-0153,U+02BB-02BC,U+02C6,U+02DA,U+02DC,U+0304,U+0308,U+0329,U+2000-206F,U+20AC,U+2122,U+2191,U+2193,U+2212,U+2215,U+FEFF,U+FFFD;\n}`, \"\"]);\n// Exports\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (___CSS_LOADER_EXPORT___);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/700.css?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"44340549d94d10899346.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff2":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff2 ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"89d0351bce4bc857dba6.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"d67ac585bb6a05dbf71c.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff2":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff2 ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"86d5c52f4588f9f221d7.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"36f79cc7e73a69da4438.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff2":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff2 ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"1fb2c6d685bfb888cfa3.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"e00802373a2c2db6b30d.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff2":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff2 ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"fd3dfdd6cb1a9175b63d.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff":
/*!***********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff ***!
  \***********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"a80c0d0719b1acb8f731.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff2":
/*!************************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff2 ***!
  \************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"b9d87b04a9119d8d2fdf.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff":
/*!***********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff ***!
  \***********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"5e3f232f89080810567d.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff2":
/*!************************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff2 ***!
  \************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"31476620b88eec076438.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff":
/*!***********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff ***!
  \***********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"d6c661248da2fde17768.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff2":
/*!************************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff2 ***!
  \************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"634ee2238bf30f362d52.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff":
/*!***********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff ***!
  \***********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"6b08bc756cd72f5af9e8.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff2":
/*!************************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff2 ***!
  \************************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"361cdfd3a3f9c4bb09ca.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-cyrillic-ext-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"8300b541aa89b8301a6f.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"fdd1f928a606aa116a44.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"ecd8572d631f20ff5bd5.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"98a717d5a38e77c0f657.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"4fe733bc436afc295c24.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"5c8100481d4e784afbf2.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"d23e03cf87ba44e5af6f.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"d7dfd0b02cd8311e2a97.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"60729cafbded24073dfb.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"a88b77bb10633a8045e3.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"a0baf7d6726d8f751a27.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"2d5875b032a1cca91eb2.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"1964239c2800b6bd7e39.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"bef9c15c7164d6435aad.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"1aff9f4cd71608489b9a.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"eb28a447335ba6d54fcb.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-greek-ext-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"cb14f8e80cc69ddbac34.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"db56943a88e4852343ae.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"50a0a61e29c19a2f05cb.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"df1be0be92f6f19b8115.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"c320def131b39bceabd8.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"599f66a60bdf974e578e.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"bcfbe8accc968a375a8e.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff2":
/*!*****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff2 ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"c4d6cab43bec89049809.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"6ddd1cfdbc5e74bcdab8.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"948c05192b1e64d931b1.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"2bfbba2d51a85c8702dd.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"0f86a30ca7e981fcfc99.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"aecaab4c4da2bf91377a.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"8f02573e78730021ef49.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff":
/*!********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff ***!
  \********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"2d3c3ba6fe2d9c1026a5.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff2":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff2 ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"8e656eff240311c6050a.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-latin-ext-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff":
/*!***************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff ***!
  \***************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"90364ecfad5101ceb1a0.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff2":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff2 ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"acc9c7c1d1fe3a1c7d44.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff":
/*!***************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff ***!
  \***************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"3d3a272e5233c5fb1969.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff2":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff2 ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"b60d9fba1e21da7497e6.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff":
/*!***************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff ***!
  \***************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"c3014a611cd9d8fa6252.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff2":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff2 ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"41db483cb764343fca71.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff":
/*!***************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff ***!
  \***************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"a6fde3ddcb1629fd58b7.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff2":
/*!****************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff2 ***!
  \****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"f6f4b54add6ab9d60a0f.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-math-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff":
/*!******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff ***!
  \******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"52cdf8344b378f0c4580.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff2":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff2 ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"616638ec44336b3da884.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff":
/*!******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff ***!
  \******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"f4f7e3bd8264f1a640cb.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff2":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff2 ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"bb5b5d1459beb07bd3d5.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff":
/*!******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff ***!
  \******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"09b674875029289fd9a7.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff2":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff2 ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"a5457b0ec984fd4cc8da.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff":
/*!******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff ***!
  \******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"017e476ef02f62144169.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff2":
/*!*******************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff2 ***!
  \*******************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"634070e045ac99822c21.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-symbols-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"53f399e4522b647bafa7.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff2":
/*!**********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff2 ***!
  \**********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"6f0bf63e956c09377ef8.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-300-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"1cffe58e71a9109191a2.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff2":
/*!**********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff2 ***!
  \**********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"b1b8baa94fbcaa57d098.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-400-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"72dbf2a25dd55b80b137.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff2":
/*!**********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff2 ***!
  \**********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"148734d63bd96c6e964f.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-500-normal.woff2?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff":
/*!*********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff ***!
  \*********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"fa58a041a3336692af1e.woff\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff?");

/***/ }),

/***/ "./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff2":
/*!**********************************************************************************!*\
  !*** ./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff2 ***!
  \**********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("module.exports = __webpack_require__.p + \"44a103f706f3ffe6a041.woff2\";\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@fontsource/roboto/files/roboto-vietnamese-700-normal.woff2?");

/***/ }),

/***/ "./node_modules/@mui/icons-material/esm/Clear.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/icons-material/esm/Clear.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\"use client\";\n\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"path\", {\n  d: \"M19 6.41 17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z\"\n}), 'Clear'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/icons-material/esm/Clear.js?");

/***/ }),

/***/ "./node_modules/@mui/icons-material/esm/DarkModeOutlined.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/icons-material/esm/DarkModeOutlined.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\"use client\";\n\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"path\", {\n  d: \"M9.37 5.51c-.18.64-.27 1.31-.27 1.99 0 4.08 3.32 7.4 7.4 7.4.68 0 1.35-.09 1.99-.27C17.45 17.19 14.93 19 12 19c-3.86 0-7-3.14-7-7 0-2.93 1.81-5.45 4.37-6.49M12 3c-4.97 0-9 4.03-9 9s4.03 9 9 9 9-4.03 9-9c0-.46-.04-.92-.1-1.36-.98 1.37-2.58 2.26-4.4 2.26-2.98 0-5.4-2.42-5.4-5.4 0-1.81.89-3.42 2.26-4.4-.44-.06-.9-.1-1.36-.1\"\n}), 'DarkModeOutlined'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/icons-material/esm/DarkModeOutlined.js?");

/***/ }),

/***/ "./node_modules/@mui/icons-material/esm/LightMode.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/icons-material/esm/LightMode.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\"use client\";\n\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"path\", {\n  d: \"M12 7c-2.76 0-5 2.24-5 5s2.24 5 5 5 5-2.24 5-5-2.24-5-5-5M2 13h2c.55 0 1-.45 1-1s-.45-1-1-1H2c-.55 0-1 .45-1 1s.45 1 1 1m18 0h2c.55 0 1-.45 1-1s-.45-1-1-1h-2c-.55 0-1 .45-1 1s.45 1 1 1M11 2v2c0 .55.45 1 1 1s1-.45 1-1V2c0-.55-.45-1-1-1s-1 .45-1 1m0 18v2c0 .55.45 1 1 1s1-.45 1-1v-2c0-.55-.45-1-1-1s-1 .45-1 1M5.99 4.58c-.39-.39-1.03-.39-1.41 0-.39.39-.39 1.03 0 1.41l1.06 1.06c.39.39 1.03.39 1.41 0s.39-1.03 0-1.41zm12.37 12.37c-.39-.39-1.03-.39-1.41 0-.39.39-.39 1.03 0 1.41l1.06 1.06c.39.39 1.03.39 1.41 0 .39-.39.39-1.03 0-1.41zm1.06-10.96c.39-.39.39-1.03 0-1.41-.39-.39-1.03-.39-1.41 0l-1.06 1.06c-.39.39-.39 1.03 0 1.41s1.03.39 1.41 0zM7.05 18.36c.39-.39.39-1.03 0-1.41-.39-.39-1.03-.39-1.41 0l-1.06 1.06c-.39.39-.39 1.03 0 1.41s1.03.39 1.41 0z\"\n}), 'LightMode'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/icons-material/esm/LightMode.js?");

/***/ }),

/***/ "./node_modules/@mui/icons-material/esm/SettingsBrightness.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/icons-material/esm/SettingsBrightness.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\"use client\";\n\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"path\", {\n  d: \"M21 3H3c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h18c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2m0 16.01H3V4.99h18zM8 16h2.5l1.5 1.5 1.5-1.5H16v-2.5l1.5-1.5-1.5-1.5V8h-2.5L12 6.5 10.5 8H8v2.5L6.5 12 8 13.5zm4-7c1.66 0 3 1.34 3 3s-1.34 3-3 3z\"\n}), 'SettingsBrightness'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/icons-material/esm/SettingsBrightness.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Alert/Alert.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/Alert/Alert.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_19___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_19__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _Paper_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Paper/index.js */ \"./node_modules/@mui/material/Paper/Paper.js\");\n/* harmony import */ var _alertClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./alertClasses.js */ \"./node_modules/@mui/material/Alert/alertClasses.js\");\n/* harmony import */ var _IconButton_index_js__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../IconButton/index.js */ \"./node_modules/@mui/material/IconButton/IconButton.js\");\n/* harmony import */ var _internal_svg_icons_SuccessOutlined_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../internal/svg-icons/SuccessOutlined.js */ \"./node_modules/@mui/material/internal/svg-icons/SuccessOutlined.js\");\n/* harmony import */ var _internal_svg_icons_ReportProblemOutlined_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../internal/svg-icons/ReportProblemOutlined.js */ \"./node_modules/@mui/material/internal/svg-icons/ReportProblemOutlined.js\");\n/* harmony import */ var _internal_svg_icons_ErrorOutline_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../internal/svg-icons/ErrorOutline.js */ \"./node_modules/@mui/material/internal/svg-icons/ErrorOutline.js\");\n/* harmony import */ var _internal_svg_icons_InfoOutlined_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../internal/svg-icons/InfoOutlined.js */ \"./node_modules/@mui/material/internal/svg-icons/InfoOutlined.js\");\n/* harmony import */ var _internal_svg_icons_Close_js__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! ../internal/svg-icons/Close.js */ \"./node_modules/@mui/material/internal/svg-icons/Close.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    variant,\n    color,\n    severity,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color || severity)}`, `${variant}${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color || severity)}`, `${variant}`],\n    icon: ['icon'],\n    message: ['message'],\n    action: ['action']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _alertClasses_js__WEBPACK_IMPORTED_MODULE_5__.getAlertUtilityClass, classes);\n};\nconst AlertRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_Paper_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiAlert',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[ownerState.variant], styles[`${ownerState.variant}${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color || ownerState.severity)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => {\n  const getColor = theme.palette.mode === 'light' ? _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.darken : _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.lighten;\n  const getBackgroundColor = theme.palette.mode === 'light' ? _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.lighten : _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.darken;\n  return {\n    ...theme.typography.body2,\n    backgroundColor: 'transparent',\n    display: 'flex',\n    padding: '6px 16px',\n    variants: [...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(['light'])).map(([color]) => ({\n      props: {\n        colorSeverity: color,\n        variant: 'standard'\n      },\n      style: {\n        color: theme.vars ? theme.vars.palette.Alert[`${color}Color`] : getColor(theme.palette[color].light, 0.6),\n        backgroundColor: theme.vars ? theme.vars.palette.Alert[`${color}StandardBg`] : getBackgroundColor(theme.palette[color].light, 0.9),\n        [`& .${_alertClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].icon}`]: theme.vars ? {\n          color: theme.vars.palette.Alert[`${color}IconColor`]\n        } : {\n          color: theme.palette[color].main\n        }\n      }\n    })), ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(['light'])).map(([color]) => ({\n      props: {\n        colorSeverity: color,\n        variant: 'outlined'\n      },\n      style: {\n        color: theme.vars ? theme.vars.palette.Alert[`${color}Color`] : getColor(theme.palette[color].light, 0.6),\n        border: `1px solid ${(theme.vars || theme).palette[color].light}`,\n        [`& .${_alertClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].icon}`]: theme.vars ? {\n          color: theme.vars.palette.Alert[`${color}IconColor`]\n        } : {\n          color: theme.palette[color].main\n        }\n      }\n    })), ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(['dark'])).map(([color]) => ({\n      props: {\n        colorSeverity: color,\n        variant: 'filled'\n      },\n      style: {\n        fontWeight: theme.typography.fontWeightMedium,\n        ...(theme.vars ? {\n          color: theme.vars.palette.Alert[`${color}FilledColor`],\n          backgroundColor: theme.vars.palette.Alert[`${color}FilledBg`]\n        } : {\n          backgroundColor: theme.palette.mode === 'dark' ? theme.palette[color].dark : theme.palette[color].main,\n          color: theme.palette.getContrastText(theme.palette[color].main)\n        })\n      }\n    }))]\n  };\n}));\nconst AlertIcon = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('div', {\n  name: 'MuiAlert',\n  slot: 'Icon',\n  overridesResolver: (props, styles) => styles.icon\n})({\n  marginRight: 12,\n  padding: '7px 0',\n  display: 'flex',\n  fontSize: 22,\n  opacity: 0.9\n});\nconst AlertMessage = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('div', {\n  name: 'MuiAlert',\n  slot: 'Message',\n  overridesResolver: (props, styles) => styles.message\n})({\n  padding: '8px 0',\n  minWidth: 0,\n  overflow: 'auto'\n});\nconst AlertAction = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('div', {\n  name: 'MuiAlert',\n  slot: 'Action',\n  overridesResolver: (props, styles) => styles.action\n})({\n  display: 'flex',\n  alignItems: 'flex-start',\n  padding: '4px 0 0 16px',\n  marginLeft: 'auto',\n  marginRight: -8\n});\nconst defaultIconMapping = {\n  success: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_internal_svg_icons_SuccessOutlined_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"], {\n    fontSize: \"inherit\"\n  }),\n  warning: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_internal_svg_icons_ReportProblemOutlined_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"], {\n    fontSize: \"inherit\"\n  }),\n  error: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_internal_svg_icons_ErrorOutline_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"], {\n    fontSize: \"inherit\"\n  }),\n  info: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_internal_svg_icons_InfoOutlined_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"], {\n    fontSize: \"inherit\"\n  })\n};\nconst Alert = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Alert(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_15__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiAlert'\n  });\n  const {\n    action,\n    children,\n    className,\n    closeText = 'Close',\n    color,\n    components = {},\n    componentsProps = {},\n    icon,\n    iconMapping = defaultIconMapping,\n    onClose,\n    role = 'alert',\n    severity = 'success',\n    slotProps = {},\n    slots = {},\n    variant = 'standard',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    color,\n    severity,\n    variant,\n    colorSeverity: color || severity\n  };\n  const classes = useUtilityClasses(ownerState);\n  const externalForwardedProps = {\n    slots: {\n      closeButton: components.CloseButton,\n      closeIcon: components.CloseIcon,\n      ...slots\n    },\n    slotProps: {\n      ...componentsProps,\n      ...slotProps\n    }\n  };\n  const [RootSlot, rootSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])('root', {\n    ref,\n    shouldForwardComponentProp: true,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    elementType: AlertRoot,\n    externalForwardedProps: {\n      ...externalForwardedProps,\n      ...other\n    },\n    ownerState,\n    additionalProps: {\n      role,\n      elevation: 0\n    }\n  });\n  const [IconSlot, iconSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])('icon', {\n    className: classes.icon,\n    elementType: AlertIcon,\n    externalForwardedProps,\n    ownerState\n  });\n  const [MessageSlot, messageSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])('message', {\n    className: classes.message,\n    elementType: AlertMessage,\n    externalForwardedProps,\n    ownerState\n  });\n  const [ActionSlot, actionSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])('action', {\n    className: classes.action,\n    elementType: AlertAction,\n    externalForwardedProps,\n    ownerState\n  });\n  const [CloseButtonSlot, closeButtonProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])('closeButton', {\n    elementType: _IconButton_index_js__WEBPACK_IMPORTED_MODULE_17__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  const [CloseIconSlot, closeIconProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])('closeIcon', {\n    elementType: _internal_svg_icons_Close_js__WEBPACK_IMPORTED_MODULE_18__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(RootSlot, {\n    ...rootSlotProps,\n    children: [icon !== false ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(IconSlot, {\n      ...iconSlotProps,\n      children: icon || iconMapping[severity] || defaultIconMapping[severity]\n    }) : null, /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(MessageSlot, {\n      ...messageSlotProps,\n      children: children\n    }), action != null ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ActionSlot, {\n      ...actionSlotProps,\n      children: action\n    }) : null, action == null && onClose ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ActionSlot, {\n      ...actionSlotProps,\n      children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CloseButtonSlot, {\n        size: \"small\",\n        \"aria-label\": closeText,\n        title: closeText,\n        color: \"inherit\",\n        onClick: onClose,\n        ...closeButtonProps,\n        children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CloseIconSlot, {\n          fontSize: \"small\",\n          ...closeIconProps\n        })\n      })\n    }) : null]\n  });\n});\n true ? Alert.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The action to display. It renders after the message, at the end of the alert.\n   */\n  action: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string),\n  /**\n   * Override the default label for the *close popup* icon button.\n   *\n   * For localization purposes, you can use the provided [translations](https://mui.com/material-ui/guides/localization/).\n   * @default 'Close'\n   */\n  closeText: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string),\n  /**\n   * The color of the component. Unless provided, the value is taken from the `severity` prop.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOf(['error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string)]),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_19___default().shape({\n    CloseButton: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n    CloseIcon: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType)\n  }),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * @deprecated use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_19___default().shape({\n    closeButton: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object),\n    closeIcon: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)\n  }),\n  /**\n   * Override the icon displayed before the children.\n   * Unless provided, the icon is mapped to the value of the `severity` prop.\n   * Set to `false` to remove the `icon`.\n   */\n  icon: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n  /**\n   * The component maps the `severity` prop to a range of different icons,\n   * for instance success to `<SuccessOutlined>`.\n   * If you wish to change this mapping, you can provide your own.\n   * Alternatively, you can use the `icon` prop to override the icon displayed.\n   */\n  iconMapping: prop_types__WEBPACK_IMPORTED_MODULE_19___default().shape({\n    error: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n    info: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n    success: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n    warning: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node)\n  }),\n  /**\n   * Callback fired when the component requests to be closed.\n   * When provided and no `action` prop is set, a close icon button is displayed that triggers the callback when clicked.\n   * @param {React.SyntheticEvent} event The event source of the callback.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func),\n  /**\n   * The ARIA role attribute of the element.\n   * @default 'alert'\n   */\n  role: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string),\n  /**\n   * The severity of the alert. This defines the color and icon used.\n   * @default 'success'\n   */\n  severity: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOf(['error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string)]),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_19___default().shape({\n    action: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n    closeButton: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n    closeIcon: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n    icon: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n    message: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_19___default().shape({\n    action: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n    closeButton: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n    closeIcon: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n    icon: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n    message: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_19___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n  /**\n   * The variant to use.\n   * @default 'standard'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOf(['filled', 'outlined', 'standard']), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Alert);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Alert/Alert.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Alert/alertClasses.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/Alert/alertClasses.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getAlertUtilityClass: () => (/* binding */ getAlertUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getAlertUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiAlert', slot);\n}\nconst alertClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiAlert', ['root', 'action', 'icon', 'message', 'filled', 'colorSuccess', 'colorInfo', 'colorWarning', 'colorError', 'filledSuccess', 'filledInfo', 'filledWarning', 'filledError', 'outlined', 'outlinedSuccess', 'outlinedInfo', 'outlinedWarning', 'outlinedError', 'standard', 'standardSuccess', 'standardInfo', 'standardWarning', 'standardError']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (alertClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Alert/alertClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/AlertTitle/AlertTitle.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/AlertTitle/AlertTitle.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _Typography_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Typography/index.js */ \"./node_modules/@mui/material/Typography/Typography.js\");\n/* harmony import */ var _alertTitleClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./alertTitleClasses.js */ \"./node_modules/@mui/material/AlertTitle/alertTitleClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _alertTitleClasses_js__WEBPACK_IMPORTED_MODULE_4__.getAlertTitleUtilityClass, classes);\n};\nconst AlertTitleRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Typography_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n  name: 'MuiAlertTitle',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => {\n  return {\n    fontWeight: theme.typography.fontWeightMedium,\n    marginTop: -2\n  };\n}));\nconst AlertTitle = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function AlertTitle(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiAlertTitle'\n  });\n  const {\n    className,\n    ...other\n  } = props;\n  const ownerState = props;\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(AlertTitleRoot, {\n    gutterBottom: true,\n    component: \"div\",\n    ownerState: ownerState,\n    ref: ref,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ...other\n  });\n});\n true ? AlertTitle.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (AlertTitle);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/AlertTitle/AlertTitle.js?");

/***/ }),

/***/ "./node_modules/@mui/material/AlertTitle/alertTitleClasses.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/AlertTitle/alertTitleClasses.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getAlertTitleUtilityClass: () => (/* binding */ getAlertTitleUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getAlertTitleUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiAlertTitle', slot);\n}\nconst alertTitleClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiAlertTitle', ['root']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (alertTitleClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/AlertTitle/alertTitleClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/AppBar/AppBar.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/AppBar/AppBar.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _Paper_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Paper/index.js */ \"./node_modules/@mui/material/Paper/Paper.js\");\n/* harmony import */ var _appBarClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./appBarClasses.js */ \"./node_modules/@mui/material/AppBar/appBarClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    color,\n    position,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, `position${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(position)}`]\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _appBarClasses_js__WEBPACK_IMPORTED_MODULE_5__.getAppBarUtilityClass, classes);\n};\n\n// var2 is the fallback.\n// Ex. var1: 'var(--a)', var2: 'var(--b)'; return: 'var(--a, var(--b))'\nconst joinVars = (var1, var2) => var1 ? `${var1?.replace(')', '')}, ${var2})` : var2;\nconst AppBarRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_Paper_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiAppBar',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[`position${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.position)}`], styles[`color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  display: 'flex',\n  flexDirection: 'column',\n  width: '100%',\n  boxSizing: 'border-box',\n  // Prevent padding issue with the Modal and fixed positioned AppBar.\n  flexShrink: 0,\n  variants: [{\n    props: {\n      position: 'fixed'\n    },\n    style: {\n      position: 'fixed',\n      zIndex: (theme.vars || theme).zIndex.appBar,\n      top: 0,\n      left: 'auto',\n      right: 0,\n      '@media print': {\n        // Prevent the app bar to be visible on each printed page.\n        position: 'absolute'\n      }\n    }\n  }, {\n    props: {\n      position: 'absolute'\n    },\n    style: {\n      position: 'absolute',\n      zIndex: (theme.vars || theme).zIndex.appBar,\n      top: 0,\n      left: 'auto',\n      right: 0\n    }\n  }, {\n    props: {\n      position: 'sticky'\n    },\n    style: {\n      position: 'sticky',\n      zIndex: (theme.vars || theme).zIndex.appBar,\n      top: 0,\n      left: 'auto',\n      right: 0\n    }\n  }, {\n    props: {\n      position: 'static'\n    },\n    style: {\n      position: 'static'\n    }\n  }, {\n    props: {\n      position: 'relative'\n    },\n    style: {\n      position: 'relative'\n    }\n  }, {\n    props: {\n      color: 'inherit'\n    },\n    style: {\n      '--AppBar-color': 'inherit'\n    }\n  }, {\n    props: {\n      color: 'default'\n    },\n    style: {\n      '--AppBar-background': theme.vars ? theme.vars.palette.AppBar.defaultBg : theme.palette.grey[100],\n      '--AppBar-color': theme.vars ? theme.vars.palette.text.primary : theme.palette.getContrastText(theme.palette.grey[100]),\n      ...theme.applyStyles('dark', {\n        '--AppBar-background': theme.vars ? theme.vars.palette.AppBar.defaultBg : theme.palette.grey[900],\n        '--AppBar-color': theme.vars ? theme.vars.palette.text.primary : theme.palette.getContrastText(theme.palette.grey[900])\n      })\n    }\n  }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(['contrastText'])).map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      '--AppBar-background': (theme.vars ?? theme).palette[color].main,\n      '--AppBar-color': (theme.vars ?? theme).palette[color].contrastText\n    }\n  })), {\n    props: props => props.enableColorOnDark === true && !['inherit', 'transparent'].includes(props.color),\n    style: {\n      backgroundColor: 'var(--AppBar-background)',\n      color: 'var(--AppBar-color)'\n    }\n  }, {\n    props: props => props.enableColorOnDark === false && !['inherit', 'transparent'].includes(props.color),\n    style: {\n      backgroundColor: 'var(--AppBar-background)',\n      color: 'var(--AppBar-color)',\n      ...theme.applyStyles('dark', {\n        backgroundColor: theme.vars ? joinVars(theme.vars.palette.AppBar.darkBg, 'var(--AppBar-background)') : null,\n        color: theme.vars ? joinVars(theme.vars.palette.AppBar.darkColor, 'var(--AppBar-color)') : null\n      })\n    }\n  }, {\n    props: {\n      color: 'transparent'\n    },\n    style: {\n      '--AppBar-background': 'transparent',\n      '--AppBar-color': 'inherit',\n      backgroundColor: 'var(--AppBar-background)',\n      color: 'var(--AppBar-color)',\n      ...theme.applyStyles('dark', {\n        backgroundImage: 'none'\n      })\n    }\n  }]\n})));\nconst AppBar = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function AppBar(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiAppBar'\n  });\n  const {\n    className,\n    color = 'primary',\n    enableColorOnDark = false,\n    position = 'fixed',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    color,\n    position,\n    enableColorOnDark\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(AppBarRoot, {\n    square: true,\n    component: \"header\",\n    ownerState: ownerState,\n    elevation: 4,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className, position === 'fixed' && 'mui-fixed'),\n    ref: ref,\n    ...other\n  });\n});\n true ? AppBar.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'primary'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['default', 'inherit', 'primary', 'secondary', 'transparent', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * If true, the `color` prop is applied in dark mode.\n   * @default false\n   */\n  enableColorOnDark: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * The positioning type. The behavior of the different options is described\n   * [in the MDN web docs](https://developer.mozilla.org/en-US/docs/Web/CSS/position).\n   * Note: `sticky` is not universally supported and will fall back to `static` when unavailable.\n   * @default 'fixed'\n   */\n  position: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['absolute', 'fixed', 'relative', 'static', 'sticky']),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (AppBar);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/AppBar/AppBar.js?");

/***/ }),

/***/ "./node_modules/@mui/material/AppBar/appBarClasses.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/AppBar/appBarClasses.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getAppBarUtilityClass: () => (/* binding */ getAppBarUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getAppBarUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiAppBar', slot);\n}\nconst appBarClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiAppBar', ['root', 'positionFixed', 'positionAbsolute', 'positionSticky', 'positionStatic', 'positionRelative', 'colorDefault', 'colorPrimary', 'colorSecondary', 'colorInherit', 'colorTransparent', 'colorError', 'colorInfo', 'colorSuccess', 'colorWarning']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (appBarClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/AppBar/appBarClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Backdrop/Backdrop.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/Backdrop/Backdrop.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var _Fade_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Fade/index.js */ \"./node_modules/@mui/material/Fade/Fade.js\");\n/* harmony import */ var _backdropClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./backdropClasses.js */ \"./node_modules/@mui/material/Backdrop/backdropClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    invisible\n  } = ownerState;\n  const slots = {\n    root: ['root', invisible && 'invisible']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _backdropClasses_js__WEBPACK_IMPORTED_MODULE_4__.getBackdropUtilityClass, classes);\n};\nconst BackdropRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiBackdrop',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.invisible && styles.invisible];\n  }\n})({\n  position: 'fixed',\n  display: 'flex',\n  alignItems: 'center',\n  justifyContent: 'center',\n  right: 0,\n  bottom: 0,\n  top: 0,\n  left: 0,\n  backgroundColor: 'rgba(0, 0, 0, 0.5)',\n  WebkitTapHighlightColor: 'transparent',\n  variants: [{\n    props: {\n      invisible: true\n    },\n    style: {\n      backgroundColor: 'transparent'\n    }\n  }]\n});\nconst Backdrop = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Backdrop(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiBackdrop'\n  });\n  const {\n    children,\n    className,\n    component = 'div',\n    invisible = false,\n    open,\n    components = {},\n    componentsProps = {},\n    slotProps = {},\n    slots = {},\n    TransitionComponent: TransitionComponentProp,\n    transitionDuration,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    component,\n    invisible\n  };\n  const classes = useUtilityClasses(ownerState);\n  const backwardCompatibleSlots = {\n    transition: TransitionComponentProp,\n    root: components.Root,\n    ...slots\n  };\n  const backwardCompatibleSlotProps = {\n    ...componentsProps,\n    ...slotProps\n  };\n  const externalForwardedProps = {\n    slots: backwardCompatibleSlots,\n    slotProps: backwardCompatibleSlotProps\n  };\n  const [RootSlot, rootProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])('root', {\n    elementType: BackdropRoot,\n    externalForwardedProps,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState\n  });\n  const [TransitionSlot, transitionProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])('transition', {\n    elementType: _Fade_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TransitionSlot, {\n    in: open,\n    timeout: transitionDuration,\n    ...other,\n    ...transitionProps,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(RootSlot, {\n      \"aria-hidden\": true,\n      ...rootProps,\n      classes: classes,\n      ref: ref,\n      children: children\n    })\n  });\n});\n true ? Backdrop.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated Use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_9___default().shape({\n    Root: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType)\n  }),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * @deprecated Use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_9___default().shape({\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)\n  }),\n  /**\n   * If `true`, the backdrop is invisible.\n   * It can be used when rendering a popover or a custom select component.\n   * @default false\n   */\n  invisible: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * If `true`, the component is shown.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool).isRequired,\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_9___default().shape({\n    root: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)]),\n    transition: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_9___default().shape({\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType),\n    transition: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)]),\n  /**\n   * The component used for the transition.\n   * [Follow this guide](https://mui.com/material-ui/transitions/#transitioncomponent-prop) to learn more about the requirements for this component.\n   * @default Fade\n   * @deprecated Use `slots.transition` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  TransitionComponent: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType),\n  /**\n   * The duration for the transition, in milliseconds.\n   * You may specify a single timeout for all transitions, or individually with an object.\n   */\n  transitionDuration: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().number), prop_types__WEBPACK_IMPORTED_MODULE_9___default().shape({\n    appear: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().number),\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().number),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().number)\n  })])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Backdrop);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Backdrop/Backdrop.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Backdrop/backdropClasses.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/Backdrop/backdropClasses.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getBackdropUtilityClass: () => (/* binding */ getBackdropUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getBackdropUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiBackdrop', slot);\n}\nconst backdropClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiBackdrop', ['root', 'invisible']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (backdropClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Backdrop/backdropClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Box/Box.js":
/*!***********************************************!*\
  !*** ./node_modules/@mui/material/Box/Box.js ***!
  \***********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createBox/createBox.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _className_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../className/index.js */ \"./node_modules/@mui/utils/esm/ClassNameGenerator/ClassNameGenerator.js\");\n/* harmony import */ var _styles_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../styles/index.js */ \"./node_modules/@mui/material/styles/createTheme.js\");\n/* harmony import */ var _styles_identifier_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../styles/identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n/* harmony import */ var _boxClasses_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./boxClasses.js */ \"./node_modules/@mui/material/Box/boxClasses.js\");\n'use client';\n\n\n\n\n\n\n\nconst defaultTheme = (0,_styles_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\nconst Box = (0,_mui_system__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n  themeId: _styles_identifier_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"],\n  defaultTheme,\n  defaultClassName: _boxClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].root,\n  generateClassName: _className_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].generate\n});\n true ? Box.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().node),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().elementType),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_5___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_5___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_5___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_5___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Box);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Box/Box.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Box/boxClasses.js":
/*!******************************************************!*\
  !*** ./node_modules/@mui/material/Box/boxClasses.js ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n\nconst boxClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiBox', ['root']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (boxClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Box/boxClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Button/Button.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/Button/Button.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_18___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_18__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/resolveProps */ \"./node_modules/@mui/utils/esm/resolveProps/resolveProps.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _mui_material_utils__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! @mui/material/utils */ \"./node_modules/@mui/material/utils/useId.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../ButtonBase/index.js */ \"./node_modules/@mui/material/ButtonBase/ButtonBase.js\");\n/* harmony import */ var _CircularProgress_index_js__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../CircularProgress/index.js */ \"./node_modules/@mui/material/CircularProgress/CircularProgress.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./buttonClasses.js */ \"./node_modules/@mui/material/Button/buttonClasses.js\");\n/* harmony import */ var _ButtonGroup_ButtonGroupContext_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../ButtonGroup/ButtonGroupContext.js */ \"./node_modules/@mui/material/ButtonGroup/ButtonGroupContext.js\");\n/* harmony import */ var _ButtonGroup_ButtonGroupButtonContext_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../ButtonGroup/ButtonGroupButtonContext.js */ \"./node_modules/@mui/material/ButtonGroup/ButtonGroupButtonContext.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    color,\n    disableElevation,\n    fullWidth,\n    size,\n    variant,\n    loading,\n    loadingPosition,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', loading && 'loading', variant, `${variant}${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`, `${variant}Size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`, `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, disableElevation && 'disableElevation', fullWidth && 'fullWidth', loading && `loadingPosition${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(loadingPosition)}`],\n    startIcon: ['icon', 'startIcon', `iconSize${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`],\n    endIcon: ['icon', 'endIcon', `iconSize${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`],\n    loadingIndicator: ['loadingIndicator'],\n    loadingWrapper: ['loadingWrapper']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__.getButtonUtilityClass, classes);\n  return {\n    ...classes,\n    // forward the focused, disabled, etc. classes to the ButtonBase\n    ...composedClasses\n  };\n};\nconst commonIconStyles = [{\n  props: {\n    size: 'small'\n  },\n  style: {\n    '& > *:nth-of-type(1)': {\n      fontSize: 18\n    }\n  }\n}, {\n  props: {\n    size: 'medium'\n  },\n  style: {\n    '& > *:nth-of-type(1)': {\n      fontSize: 20\n    }\n  }\n}, {\n  props: {\n    size: 'large'\n  },\n  style: {\n    '& > *:nth-of-type(1)': {\n      fontSize: 22\n    }\n  }\n}];\nconst ButtonRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiButton',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[ownerState.variant], styles[`${ownerState.variant}${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color)}`], styles[`size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`], styles[`${ownerState.variant}Size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`], ownerState.color === 'inherit' && styles.colorInherit, ownerState.disableElevation && styles.disableElevation, ownerState.fullWidth && styles.fullWidth, ownerState.loading && styles.loading];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(({\n  theme\n}) => {\n  const inheritContainedBackgroundColor = theme.palette.mode === 'light' ? theme.palette.grey[300] : theme.palette.grey[800];\n  const inheritContainedHoverBackgroundColor = theme.palette.mode === 'light' ? theme.palette.grey.A100 : theme.palette.grey[700];\n  return {\n    ...theme.typography.button,\n    minWidth: 64,\n    padding: '6px 16px',\n    border: 0,\n    borderRadius: (theme.vars || theme).shape.borderRadius,\n    transition: theme.transitions.create(['background-color', 'box-shadow', 'border-color', 'color'], {\n      duration: theme.transitions.duration.short\n    }),\n    '&:hover': {\n      textDecoration: 'none'\n    },\n    [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n      color: (theme.vars || theme).palette.action.disabled\n    },\n    variants: [{\n      props: {\n        variant: 'contained'\n      },\n      style: {\n        color: `var(--variant-containedColor)`,\n        backgroundColor: `var(--variant-containedBg)`,\n        boxShadow: (theme.vars || theme).shadows[2],\n        '&:hover': {\n          boxShadow: (theme.vars || theme).shadows[4],\n          // Reset on touch devices, it doesn't add specificity\n          '@media (hover: none)': {\n            boxShadow: (theme.vars || theme).shadows[2]\n          }\n        },\n        '&:active': {\n          boxShadow: (theme.vars || theme).shadows[8]\n        },\n        [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].focusVisible}`]: {\n          boxShadow: (theme.vars || theme).shadows[6]\n        },\n        [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n          color: (theme.vars || theme).palette.action.disabled,\n          boxShadow: (theme.vars || theme).shadows[0],\n          backgroundColor: (theme.vars || theme).palette.action.disabledBackground\n        }\n      }\n    }, {\n      props: {\n        variant: 'outlined'\n      },\n      style: {\n        padding: '5px 15px',\n        border: '1px solid currentColor',\n        borderColor: `var(--variant-outlinedBorder, currentColor)`,\n        backgroundColor: `var(--variant-outlinedBg)`,\n        color: `var(--variant-outlinedColor)`,\n        [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n          border: `1px solid ${(theme.vars || theme).palette.action.disabledBackground}`\n        }\n      }\n    }, {\n      props: {\n        variant: 'text'\n      },\n      style: {\n        padding: '6px 8px',\n        color: `var(--variant-textColor)`,\n        backgroundColor: `var(--variant-textBg)`\n      }\n    }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])()).map(([color]) => ({\n      props: {\n        color\n      },\n      style: {\n        '--variant-textColor': (theme.vars || theme).palette[color].main,\n        '--variant-outlinedColor': (theme.vars || theme).palette[color].main,\n        '--variant-outlinedBorder': theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / 0.5)` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_11__.alpha)(theme.palette[color].main, 0.5),\n        '--variant-containedColor': (theme.vars || theme).palette[color].contrastText,\n        '--variant-containedBg': (theme.vars || theme).palette[color].main,\n        '@media (hover: hover)': {\n          '&:hover': {\n            '--variant-containedBg': (theme.vars || theme).palette[color].dark,\n            '--variant-textBg': theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_11__.alpha)(theme.palette[color].main, theme.palette.action.hoverOpacity),\n            '--variant-outlinedBorder': (theme.vars || theme).palette[color].main,\n            '--variant-outlinedBg': theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_11__.alpha)(theme.palette[color].main, theme.palette.action.hoverOpacity)\n          }\n        }\n      }\n    })), {\n      props: {\n        color: 'inherit'\n      },\n      style: {\n        color: 'inherit',\n        borderColor: 'currentColor',\n        '--variant-containedBg': theme.vars ? theme.vars.palette.Button.inheritContainedBg : inheritContainedBackgroundColor,\n        '@media (hover: hover)': {\n          '&:hover': {\n            '--variant-containedBg': theme.vars ? theme.vars.palette.Button.inheritContainedHoverBg : inheritContainedHoverBackgroundColor,\n            '--variant-textBg': theme.vars ? `rgba(${theme.vars.palette.text.primaryChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_11__.alpha)(theme.palette.text.primary, theme.palette.action.hoverOpacity),\n            '--variant-outlinedBg': theme.vars ? `rgba(${theme.vars.palette.text.primaryChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_11__.alpha)(theme.palette.text.primary, theme.palette.action.hoverOpacity)\n          }\n        }\n      }\n    }, {\n      props: {\n        size: 'small',\n        variant: 'text'\n      },\n      style: {\n        padding: '4px 5px',\n        fontSize: theme.typography.pxToRem(13)\n      }\n    }, {\n      props: {\n        size: 'large',\n        variant: 'text'\n      },\n      style: {\n        padding: '8px 11px',\n        fontSize: theme.typography.pxToRem(15)\n      }\n    }, {\n      props: {\n        size: 'small',\n        variant: 'outlined'\n      },\n      style: {\n        padding: '3px 9px',\n        fontSize: theme.typography.pxToRem(13)\n      }\n    }, {\n      props: {\n        size: 'large',\n        variant: 'outlined'\n      },\n      style: {\n        padding: '7px 21px',\n        fontSize: theme.typography.pxToRem(15)\n      }\n    }, {\n      props: {\n        size: 'small',\n        variant: 'contained'\n      },\n      style: {\n        padding: '4px 10px',\n        fontSize: theme.typography.pxToRem(13)\n      }\n    }, {\n      props: {\n        size: 'large',\n        variant: 'contained'\n      },\n      style: {\n        padding: '8px 22px',\n        fontSize: theme.typography.pxToRem(15)\n      }\n    }, {\n      props: {\n        disableElevation: true\n      },\n      style: {\n        boxShadow: 'none',\n        '&:hover': {\n          boxShadow: 'none'\n        },\n        [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].focusVisible}`]: {\n          boxShadow: 'none'\n        },\n        '&:active': {\n          boxShadow: 'none'\n        },\n        [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n          boxShadow: 'none'\n        }\n      }\n    }, {\n      props: {\n        fullWidth: true\n      },\n      style: {\n        width: '100%'\n      }\n    }, {\n      props: {\n        loadingPosition: 'center'\n      },\n      style: {\n        transition: theme.transitions.create(['background-color', 'box-shadow', 'border-color'], {\n          duration: theme.transitions.duration.short\n        }),\n        [`&.${_buttonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].loading}`]: {\n          color: 'transparent'\n        }\n      }\n    }]\n  };\n}));\nconst ButtonStartIcon = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiButton',\n  slot: 'StartIcon',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.startIcon, ownerState.loading && styles.startIconLoadingStart, styles[`iconSize${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`]];\n  }\n})(({\n  theme\n}) => ({\n  display: 'inherit',\n  marginRight: 8,\n  marginLeft: -4,\n  variants: [{\n    props: {\n      size: 'small'\n    },\n    style: {\n      marginLeft: -2\n    }\n  }, {\n    props: {\n      loadingPosition: 'start',\n      loading: true\n    },\n    style: {\n      transition: theme.transitions.create(['opacity'], {\n        duration: theme.transitions.duration.short\n      }),\n      opacity: 0\n    }\n  }, {\n    props: {\n      loadingPosition: 'start',\n      loading: true,\n      fullWidth: true\n    },\n    style: {\n      marginRight: -8\n    }\n  }, ...commonIconStyles]\n}));\nconst ButtonEndIcon = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiButton',\n  slot: 'EndIcon',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.endIcon, ownerState.loading && styles.endIconLoadingEnd, styles[`iconSize${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`]];\n  }\n})(({\n  theme\n}) => ({\n  display: 'inherit',\n  marginRight: -4,\n  marginLeft: 8,\n  variants: [{\n    props: {\n      size: 'small'\n    },\n    style: {\n      marginRight: -2\n    }\n  }, {\n    props: {\n      loadingPosition: 'end',\n      loading: true\n    },\n    style: {\n      transition: theme.transitions.create(['opacity'], {\n        duration: theme.transitions.duration.short\n      }),\n      opacity: 0\n    }\n  }, {\n    props: {\n      loadingPosition: 'end',\n      loading: true,\n      fullWidth: true\n    },\n    style: {\n      marginLeft: -8\n    }\n  }, ...commonIconStyles]\n}));\nconst ButtonLoadingIndicator = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiButton',\n  slot: 'LoadingIndicator',\n  overridesResolver: (props, styles) => styles.loadingIndicator\n})(({\n  theme\n}) => ({\n  display: 'none',\n  position: 'absolute',\n  visibility: 'visible',\n  variants: [{\n    props: {\n      loading: true\n    },\n    style: {\n      display: 'flex'\n    }\n  }, {\n    props: {\n      loadingPosition: 'start'\n    },\n    style: {\n      left: 14\n    }\n  }, {\n    props: {\n      loadingPosition: 'start',\n      size: 'small'\n    },\n    style: {\n      left: 10\n    }\n  }, {\n    props: {\n      variant: 'text',\n      loadingPosition: 'start'\n    },\n    style: {\n      left: 6\n    }\n  }, {\n    props: {\n      loadingPosition: 'center'\n    },\n    style: {\n      left: '50%',\n      transform: 'translate(-50%)',\n      color: (theme.vars || theme).palette.action.disabled\n    }\n  }, {\n    props: {\n      loadingPosition: 'end'\n    },\n    style: {\n      right: 14\n    }\n  }, {\n    props: {\n      loadingPosition: 'end',\n      size: 'small'\n    },\n    style: {\n      right: 10\n    }\n  }, {\n    props: {\n      variant: 'text',\n      loadingPosition: 'end'\n    },\n    style: {\n      right: 6\n    }\n  }, {\n    props: {\n      loadingPosition: 'start',\n      fullWidth: true\n    },\n    style: {\n      position: 'relative',\n      left: -10\n    }\n  }, {\n    props: {\n      loadingPosition: 'end',\n      fullWidth: true\n    },\n    style: {\n      position: 'relative',\n      right: -10\n    }\n  }]\n}));\nconst ButtonLoadingIconPlaceholder = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiButton',\n  slot: 'LoadingIconPlaceholder',\n  overridesResolver: (props, styles) => styles.loadingIconPlaceholder\n})({\n  display: 'inline-block',\n  width: '1em',\n  height: '1em'\n});\nconst Button = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Button(inProps, ref) {\n  // props priority: `inProps` > `contextProps` > `themeDefaultProps`\n  const contextProps = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_ButtonGroup_ButtonGroupContext_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"]);\n  const buttonGroupButtonContextPositionClassName = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_ButtonGroup_ButtonGroupButtonContext_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"]);\n  const resolvedProps = (0,_mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_14__[\"default\"])(contextProps, inProps);\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_15__.useDefaultProps)({\n    props: resolvedProps,\n    name: 'MuiButton'\n  });\n  const {\n    children,\n    color = 'primary',\n    component = 'button',\n    className,\n    disabled = false,\n    disableElevation = false,\n    disableFocusRipple = false,\n    endIcon: endIconProp,\n    focusVisibleClassName,\n    fullWidth = false,\n    id: idProp,\n    loading = null,\n    loadingIndicator: loadingIndicatorProp,\n    loadingPosition = 'center',\n    size = 'medium',\n    startIcon: startIconProp,\n    type,\n    variant = 'text',\n    ...other\n  } = props;\n  const loadingId = (0,_mui_material_utils__WEBPACK_IMPORTED_MODULE_16__[\"default\"])(idProp);\n  const loadingIndicator = loadingIndicatorProp ?? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_CircularProgress_index_js__WEBPACK_IMPORTED_MODULE_17__[\"default\"], {\n    \"aria-labelledby\": loadingId,\n    color: \"inherit\",\n    size: 16\n  });\n  const ownerState = {\n    ...props,\n    color,\n    component,\n    disabled,\n    disableElevation,\n    disableFocusRipple,\n    fullWidth,\n    loading,\n    loadingIndicator,\n    loadingPosition,\n    size,\n    type,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  const startIcon = (startIconProp || loading && loadingPosition === 'start') && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ButtonStartIcon, {\n    className: classes.startIcon,\n    ownerState: ownerState,\n    children: startIconProp || /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ButtonLoadingIconPlaceholder, {\n      className: classes.loadingIconPlaceholder,\n      ownerState: ownerState\n    })\n  });\n  const endIcon = (endIconProp || loading && loadingPosition === 'end') && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ButtonEndIcon, {\n    className: classes.endIcon,\n    ownerState: ownerState,\n    children: endIconProp || /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ButtonLoadingIconPlaceholder, {\n      className: classes.loadingIconPlaceholder,\n      ownerState: ownerState\n    })\n  });\n  const positionClassName = buttonGroupButtonContextPositionClassName || '';\n  const loader = typeof loading === 'boolean' ?\n  /*#__PURE__*/\n  // use plain HTML span to minimize the runtime overhead\n  (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"span\", {\n    className: classes.loadingWrapper,\n    style: {\n      display: 'contents'\n    },\n    children: loading && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ButtonLoadingIndicator, {\n      className: classes.loadingIndicator,\n      ownerState: ownerState,\n      children: loadingIndicator\n    })\n  }) : null;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(ButtonRoot, {\n    ownerState: ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(contextProps.className, classes.root, className, positionClassName),\n    component: component,\n    disabled: disabled || loading,\n    focusRipple: !disableFocusRipple,\n    focusVisibleClassName: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.focusVisible, focusVisibleClassName),\n    ref: ref,\n    type: type,\n    id: loading ? loadingId : idProp,\n    ...other,\n    classes: classes,\n    children: [startIcon, loadingPosition !== 'end' && loader, children, loadingPosition === 'end' && loader, endIcon]\n  });\n});\n true ? Button.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'primary'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['inherit', 'primary', 'secondary', 'success', 'error', 'info', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n  /**\n   * If `true`, the component is disabled.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * If `true`, no elevation is used.\n   * @default false\n   */\n  disableElevation: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * If `true`, the  keyboard focus ripple is disabled.\n   * @default false\n   */\n  disableFocusRipple: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * If `true`, the ripple effect is disabled.\n   *\n   * ⚠️ Without a ripple there is no styling for :focus-visible by default. Be sure\n   * to highlight the element by applying separate styles with the `.Mui-focusVisible` class.\n   * @default false\n   */\n  disableRipple: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * Element placed after the children.\n   */\n  endIcon: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * @ignore\n   */\n  focusVisibleClassName: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * If `true`, the button will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * The URL to link to when the button is clicked.\n   * If defined, an `a` element will be used as the root node.\n   */\n  href: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * @ignore\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * If `true`, the loading indicator is visible and the button is disabled.\n   * If `true | false`, the loading wrapper is always rendered before the children to prevent [Google Translation Crash](https://github.com/mui/material-ui/issues/27853).\n   * @default null\n   */\n  loading: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * Element placed before the children if the button is in loading state.\n   * The node should contain an element with `role=\"progressbar\"` with an accessible name.\n   * By default, it renders a `CircularProgress` that is labeled by the button itself.\n   * @default <CircularProgress color=\"inherit\" size={16} />\n   */\n  loadingIndicator: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * The loading indicator can be positioned on the start, end, or the center of the button.\n   * @default 'center'\n   */\n  loadingPosition: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['center', 'end', 'start']),\n  /**\n   * The size of the component.\n   * `small` is equivalent to the dense button styling.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['small', 'medium', 'large']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * Element placed before the children.\n   */\n  startIcon: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)]),\n  /**\n   * @ignore\n   */\n  type: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['button', 'reset', 'submit']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * The variant to use.\n   * @default 'text'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['contained', 'outlined', 'text']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Button);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Button/Button.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Button/buttonClasses.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/Button/buttonClasses.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getButtonUtilityClass: () => (/* binding */ getButtonUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getButtonUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiButton', slot);\n}\nconst buttonClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiButton', ['root', 'text', 'textInherit', 'textPrimary', 'textSecondary', 'textSuccess', 'textError', 'textInfo', 'textWarning', 'outlined', 'outlinedInherit', 'outlinedPrimary', 'outlinedSecondary', 'outlinedSuccess', 'outlinedError', 'outlinedInfo', 'outlinedWarning', 'contained', 'containedInherit', 'containedPrimary', 'containedSecondary', 'containedSuccess', 'containedError', 'containedInfo', 'containedWarning', 'disableElevation', 'focusVisible', 'disabled', 'colorInherit', 'colorPrimary', 'colorSecondary', 'colorSuccess', 'colorError', 'colorInfo', 'colorWarning', 'textSizeSmall', 'textSizeMedium', 'textSizeLarge', 'outlinedSizeSmall', 'outlinedSizeMedium', 'outlinedSizeLarge', 'containedSizeSmall', 'containedSizeMedium', 'containedSizeLarge', 'sizeMedium', 'sizeSmall', 'sizeLarge', 'fullWidth', 'startIcon', 'endIcon', 'icon', 'iconSizeSmall', 'iconSizeMedium', 'iconSizeLarge', 'loading', 'loadingWrapper', 'loadingIconPlaceholder', 'loadingIndicator', 'loadingPositionCenter', 'loadingPositionStart', 'loadingPositionEnd']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (buttonClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Button/buttonClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonBase/ButtonBase.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/ButtonBase/ButtonBase.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   ButtonBaseRoot: () => (/* binding */ ButtonBaseRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_13__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_elementTypeAcceptingRef__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/elementTypeAcceptingRef */ \"./node_modules/@mui/utils/esm/elementTypeAcceptingRef/elementTypeAcceptingRef.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_isFocusVisible__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/utils/isFocusVisible */ \"./node_modules/@mui/utils/esm/isFocusVisible/isFocusVisible.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var _utils_useEventCallback_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/useEventCallback.js */ \"./node_modules/@mui/material/utils/useEventCallback.js\");\n/* harmony import */ var _useLazyRipple_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../useLazyRipple/index.js */ \"./node_modules/@mui/material/useLazyRipple/useLazyRipple.js\");\n/* harmony import */ var _TouchRipple_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./TouchRipple.js */ \"./node_modules/@mui/material/ButtonBase/TouchRipple.js\");\n/* harmony import */ var _buttonBaseClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./buttonBaseClasses.js */ \"./node_modules/@mui/material/ButtonBase/buttonBaseClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    disabled,\n    focusVisible,\n    focusVisibleClassName,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', disabled && 'disabled', focusVisible && 'focusVisible']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _buttonBaseClasses_js__WEBPACK_IMPORTED_MODULE_4__.getButtonBaseUtilityClass, classes);\n  if (focusVisible && focusVisibleClassName) {\n    composedClasses.root += ` ${focusVisibleClassName}`;\n  }\n  return composedClasses;\n};\nconst ButtonBaseRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('button', {\n  name: 'MuiButtonBase',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({\n  display: 'inline-flex',\n  alignItems: 'center',\n  justifyContent: 'center',\n  position: 'relative',\n  boxSizing: 'border-box',\n  WebkitTapHighlightColor: 'transparent',\n  backgroundColor: 'transparent',\n  // Reset default value\n  // We disable the focus ring for mouse, touch and keyboard users.\n  outline: 0,\n  border: 0,\n  margin: 0,\n  // Remove the margin in Safari\n  borderRadius: 0,\n  padding: 0,\n  // Remove the padding in Firefox\n  cursor: 'pointer',\n  userSelect: 'none',\n  verticalAlign: 'middle',\n  MozAppearance: 'none',\n  // Reset\n  WebkitAppearance: 'none',\n  // Reset\n  textDecoration: 'none',\n  // So we take precedent over the style of a native <a /> element.\n  color: 'inherit',\n  '&::-moz-focus-inner': {\n    borderStyle: 'none' // Remove Firefox dotted outline.\n  },\n  [`&.${_buttonBaseClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].disabled}`]: {\n    pointerEvents: 'none',\n    // Disable link interactions\n    cursor: 'default'\n  },\n  '@media print': {\n    colorAdjust: 'exact'\n  }\n});\n\n/**\n * `ButtonBase` contains as few styles as possible.\n * It aims to be a simple building block for creating a button.\n * It contains a load of style reset and some focus/ripple logic.\n */\nconst ButtonBase = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function ButtonBase(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiButtonBase'\n  });\n  const {\n    action,\n    centerRipple = false,\n    children,\n    className,\n    component = 'button',\n    disabled = false,\n    disableRipple = false,\n    disableTouchRipple = false,\n    focusRipple = false,\n    focusVisibleClassName,\n    LinkComponent = 'a',\n    onBlur,\n    onClick,\n    onContextMenu,\n    onDragLeave,\n    onFocus,\n    onFocusVisible,\n    onKeyDown,\n    onKeyUp,\n    onMouseDown,\n    onMouseLeave,\n    onMouseUp,\n    onTouchEnd,\n    onTouchMove,\n    onTouchStart,\n    tabIndex = 0,\n    TouchRippleProps,\n    touchRippleRef,\n    type,\n    ...other\n  } = props;\n  const buttonRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const ripple = (0,_useLazyRipple_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])();\n  const handleRippleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(ripple.ref, touchRippleRef);\n  const [focusVisible, setFocusVisible] = react__WEBPACK_IMPORTED_MODULE_0__.useState(false);\n  if (disabled && focusVisible) {\n    setFocusVisible(false);\n  }\n  react__WEBPACK_IMPORTED_MODULE_0__.useImperativeHandle(action, () => ({\n    focusVisible: () => {\n      setFocusVisible(true);\n      buttonRef.current.focus();\n    }\n  }), []);\n  const enableTouchRipple = ripple.shouldMount && !disableRipple && !disabled;\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (focusVisible && focusRipple && !disableRipple) {\n      ripple.pulsate();\n    }\n  }, [disableRipple, focusRipple, focusVisible, ripple]);\n  const handleMouseDown = useRippleHandler(ripple, 'start', onMouseDown, disableTouchRipple);\n  const handleContextMenu = useRippleHandler(ripple, 'stop', onContextMenu, disableTouchRipple);\n  const handleDragLeave = useRippleHandler(ripple, 'stop', onDragLeave, disableTouchRipple);\n  const handleMouseUp = useRippleHandler(ripple, 'stop', onMouseUp, disableTouchRipple);\n  const handleMouseLeave = useRippleHandler(ripple, 'stop', event => {\n    if (focusVisible) {\n      event.preventDefault();\n    }\n    if (onMouseLeave) {\n      onMouseLeave(event);\n    }\n  }, disableTouchRipple);\n  const handleTouchStart = useRippleHandler(ripple, 'start', onTouchStart, disableTouchRipple);\n  const handleTouchEnd = useRippleHandler(ripple, 'stop', onTouchEnd, disableTouchRipple);\n  const handleTouchMove = useRippleHandler(ripple, 'stop', onTouchMove, disableTouchRipple);\n  const handleBlur = useRippleHandler(ripple, 'stop', event => {\n    if (!(0,_mui_utils_isFocusVisible__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(event.target)) {\n      setFocusVisible(false);\n    }\n    if (onBlur) {\n      onBlur(event);\n    }\n  }, false);\n  const handleFocus = (0,_utils_useEventCallback_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(event => {\n    // Fix for https://github.com/facebook/react/issues/7769\n    if (!buttonRef.current) {\n      buttonRef.current = event.currentTarget;\n    }\n    if ((0,_mui_utils_isFocusVisible__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(event.target)) {\n      setFocusVisible(true);\n      if (onFocusVisible) {\n        onFocusVisible(event);\n      }\n    }\n    if (onFocus) {\n      onFocus(event);\n    }\n  });\n  const isNonNativeButton = () => {\n    const button = buttonRef.current;\n    return component && component !== 'button' && !(button.tagName === 'A' && button.href);\n  };\n  const handleKeyDown = (0,_utils_useEventCallback_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(event => {\n    // Check if key is already down to avoid repeats being counted as multiple activations\n    if (focusRipple && !event.repeat && focusVisible && event.key === ' ') {\n      ripple.stop(event, () => {\n        ripple.start(event);\n      });\n    }\n    if (event.target === event.currentTarget && isNonNativeButton() && event.key === ' ') {\n      event.preventDefault();\n    }\n    if (onKeyDown) {\n      onKeyDown(event);\n    }\n\n    // Keyboard accessibility for non interactive elements\n    if (event.target === event.currentTarget && isNonNativeButton() && event.key === 'Enter' && !disabled) {\n      event.preventDefault();\n      if (onClick) {\n        onClick(event);\n      }\n    }\n  });\n  const handleKeyUp = (0,_utils_useEventCallback_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(event => {\n    // calling preventDefault in keyUp on a <button> will not dispatch a click event if Space is pressed\n    // https://codesandbox.io/p/sandbox/button-keyup-preventdefault-dn7f0\n    if (focusRipple && event.key === ' ' && focusVisible && !event.defaultPrevented) {\n      ripple.stop(event, () => {\n        ripple.pulsate(event);\n      });\n    }\n    if (onKeyUp) {\n      onKeyUp(event);\n    }\n\n    // Keyboard accessibility for non interactive elements\n    if (onClick && event.target === event.currentTarget && isNonNativeButton() && event.key === ' ' && !event.defaultPrevented) {\n      onClick(event);\n    }\n  });\n  let ComponentProp = component;\n  if (ComponentProp === 'button' && (other.href || other.to)) {\n    ComponentProp = LinkComponent;\n  }\n  const buttonProps = {};\n  if (ComponentProp === 'button') {\n    buttonProps.type = type === undefined ? 'button' : type;\n    buttonProps.disabled = disabled;\n  } else {\n    if (!other.href && !other.to) {\n      buttonProps.role = 'button';\n    }\n    if (disabled) {\n      buttonProps['aria-disabled'] = disabled;\n    }\n  }\n  const handleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(ref, buttonRef);\n  const ownerState = {\n    ...props,\n    centerRipple,\n    component,\n    disabled,\n    disableRipple,\n    disableTouchRipple,\n    focusRipple,\n    tabIndex,\n    focusVisible\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(ButtonBaseRoot, {\n    as: ComponentProp,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState: ownerState,\n    onBlur: handleBlur,\n    onClick: onClick,\n    onContextMenu: handleContextMenu,\n    onFocus: handleFocus,\n    onKeyDown: handleKeyDown,\n    onKeyUp: handleKeyUp,\n    onMouseDown: handleMouseDown,\n    onMouseLeave: handleMouseLeave,\n    onMouseUp: handleMouseUp,\n    onDragLeave: handleDragLeave,\n    onTouchEnd: handleTouchEnd,\n    onTouchMove: handleTouchMove,\n    onTouchStart: handleTouchStart,\n    ref: handleRef,\n    tabIndex: disabled ? -1 : tabIndex,\n    type: type,\n    ...buttonProps,\n    ...other,\n    children: [children, enableTouchRipple ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_TouchRipple_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"], {\n      ref: handleRippleRef,\n      center: centerRipple,\n      ...TouchRippleProps\n    }) : null]\n  });\n});\nfunction useRippleHandler(ripple, rippleAction, eventCallback, skipRippleAction = false) {\n  return (0,_utils_useEventCallback_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(event => {\n    if (eventCallback) {\n      eventCallback(event);\n    }\n    if (!skipRippleAction) {\n      ripple[rippleAction](event);\n    }\n    return true;\n  });\n}\n true ? ButtonBase.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * A ref for imperative actions.\n   * It currently only supports `focusVisible()` action.\n   */\n  action: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_12__[\"default\"],\n  /**\n   * If `true`, the ripples are centered.\n   * They won't start at the cursor interaction position.\n   * @default false\n   */\n  centerRipple: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: _mui_utils_elementTypeAcceptingRef__WEBPACK_IMPORTED_MODULE_14__[\"default\"],\n  /**\n   * If `true`, the component is disabled.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the ripple effect is disabled.\n   *\n   * ⚠️ Without a ripple there is no styling for :focus-visible by default. Be sure\n   * to highlight the element by applying separate styles with the `.Mui-focusVisible` class.\n   * @default false\n   */\n  disableRipple: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the touch ripple effect is disabled.\n   * @default false\n   */\n  disableTouchRipple: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the base button will have a keyboard focus ripple.\n   * @default false\n   */\n  focusRipple: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * This prop can help identify which element has keyboard focus.\n   * The class name will be applied when the element gains the focus through keyboard interaction.\n   * It's a polyfill for the [CSS :focus-visible selector](https://drafts.csswg.org/selectors-4/#the-focus-visible-pseudo).\n   * The rationale for using this feature [is explained here](https://github.com/WICG/focus-visible/blob/HEAD/explainer.md).\n   * A [polyfill can be used](https://github.com/WICG/focus-visible) to apply a `focus-visible` class to other components\n   * if needed.\n   */\n  focusVisibleClassName: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * @ignore\n   */\n  href: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().any),\n  /**\n   * The component used to render a link when the `href` prop is provided.\n   * @default 'a'\n   */\n  LinkComponent: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n  /**\n   * @ignore\n   */\n  onBlur: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onClick: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onContextMenu: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onDragLeave: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onFocus: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * Callback fired when the component is focused with a keyboard.\n   * We trigger a `onFocus` callback too.\n   */\n  onFocusVisible: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onKeyDown: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onKeyUp: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onMouseDown: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onMouseLeave: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onMouseUp: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onTouchEnd: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onTouchMove: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * @ignore\n   */\n  onTouchStart: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n  /**\n   * @default 0\n   */\n  tabIndex: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().number),\n  /**\n   * Props applied to the `TouchRipple` element.\n   */\n  TouchRippleProps: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * A ref that points to the `TouchRipple` element.\n   */\n  touchRippleRef: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    current: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n      pulsate: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func).isRequired,\n      start: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func).isRequired,\n      stop: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func).isRequired\n    })\n  })]),\n  /**\n   * @ignore\n   */\n  type: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['button', 'reset', 'submit']), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ButtonBase);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonBase/ButtonBase.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonBase/Ripple.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/ButtonBase/Ripple.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_3__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n/**\n * @ignore - internal component.\n */\n\nfunction Ripple(props) {\n  const {\n    className,\n    classes,\n    pulsate = false,\n    rippleX,\n    rippleY,\n    rippleSize,\n    in: inProp,\n    onExited,\n    timeout\n  } = props;\n  const [leaving, setLeaving] = react__WEBPACK_IMPORTED_MODULE_0__.useState(false);\n  const rippleClassName = (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(className, classes.ripple, classes.rippleVisible, pulsate && classes.ripplePulsate);\n  const rippleStyles = {\n    width: rippleSize,\n    height: rippleSize,\n    top: -(rippleSize / 2) + rippleY,\n    left: -(rippleSize / 2) + rippleX\n  };\n  const childClassName = (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.child, leaving && classes.childLeaving, pulsate && classes.childPulsate);\n  if (!inProp && !leaving) {\n    setLeaving(true);\n  }\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (!inProp && onExited != null) {\n      // react-transition-group#onExited\n      const timeoutId = setTimeout(onExited, timeout);\n      return () => {\n        clearTimeout(timeoutId);\n      };\n    }\n    return undefined;\n  }, [onExited, inProp, timeout]);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"span\", {\n    className: rippleClassName,\n    style: rippleStyles,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"span\", {\n      className: childClassName\n    })\n  });\n}\n true ? Ripple.propTypes /* remove-proptypes */ = {\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object).isRequired,\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().string),\n  /**\n   * @ignore - injected from TransitionGroup\n   */\n  in: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().bool),\n  /**\n   * @ignore - injected from TransitionGroup\n   */\n  onExited: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().func),\n  /**\n   * If `true`, the ripple pulsates, typically indicating the keyboard focus state of an element.\n   */\n  pulsate: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().bool),\n  /**\n   * Diameter of the ripple.\n   */\n  rippleSize: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().number),\n  /**\n   * Horizontal position of the ripple center.\n   */\n  rippleX: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().number),\n  /**\n   * Vertical position of the ripple center.\n   */\n  rippleY: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().number),\n  /**\n   * exit delay\n   */\n  timeout: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().number).isRequired\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Ripple);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonBase/Ripple.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonBase/TouchRipple.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/ButtonBase/TouchRipple.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   DELAY_RIPPLE: () => (/* binding */ DELAY_RIPPLE),\n/* harmony export */   TouchRippleRipple: () => (/* binding */ TouchRippleRipple),\n/* harmony export */   TouchRippleRoot: () => (/* binding */ TouchRippleRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_10___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_10__);\n/* harmony import */ var react_transition_group__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! react-transition-group */ \"./node_modules/react-transition-group/esm/TransitionGroup.js\");\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_useTimeout__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/useTimeout */ \"./node_modules/@mui/utils/esm/useTimeout/useTimeout.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _Ripple_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./Ripple.js */ \"./node_modules/@mui/material/ButtonBase/Ripple.js\");\n/* harmony import */ var _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./touchRippleClasses.js */ \"./node_modules/@mui/material/ButtonBase/touchRippleClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst DURATION = 550;\nconst DELAY_RIPPLE = 80;\nconst enterKeyframe = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.keyframes)`\n  0% {\n    transform: scale(0);\n    opacity: 0.1;\n  }\n\n  100% {\n    transform: scale(1);\n    opacity: 0.3;\n  }\n`;\nconst exitKeyframe = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.keyframes)`\n  0% {\n    opacity: 1;\n  }\n\n  100% {\n    opacity: 0;\n  }\n`;\nconst pulsateKeyframe = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.keyframes)`\n  0% {\n    transform: scale(1);\n  }\n\n  50% {\n    transform: scale(0.92);\n  }\n\n  100% {\n    transform: scale(1);\n  }\n`;\nconst TouchRippleRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])('span', {\n  name: 'MuiTouchRipple',\n  slot: 'Root'\n})({\n  overflow: 'hidden',\n  pointerEvents: 'none',\n  position: 'absolute',\n  zIndex: 0,\n  top: 0,\n  right: 0,\n  bottom: 0,\n  left: 0,\n  borderRadius: 'inherit'\n});\n\n// This `styled()` function invokes keyframes. `styled-components` only supports keyframes\n// in string templates. Do not convert these styles in JS object as it will break.\nconst TouchRippleRipple = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])((0,_Ripple_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"]), {\n  name: 'MuiTouchRipple',\n  slot: 'Ripple'\n})`\n  opacity: 0;\n  position: absolute;\n\n  &.${_touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].rippleVisible} {\n    opacity: 0.3;\n    transform: scale(1);\n    animation-name: ${enterKeyframe};\n    animation-duration: ${DURATION}ms;\n    animation-timing-function: ${({\n  theme\n}) => theme.transitions.easing.easeInOut};\n  }\n\n  &.${_touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].ripplePulsate} {\n    animation-duration: ${({\n  theme\n}) => theme.transitions.duration.shorter}ms;\n  }\n\n  & .${_touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].child} {\n    opacity: 1;\n    display: block;\n    width: 100%;\n    height: 100%;\n    border-radius: 50%;\n    background-color: currentColor;\n  }\n\n  & .${_touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].childLeaving} {\n    opacity: 0;\n    animation-name: ${exitKeyframe};\n    animation-duration: ${DURATION}ms;\n    animation-timing-function: ${({\n  theme\n}) => theme.transitions.easing.easeInOut};\n  }\n\n  & .${_touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].childPulsate} {\n    position: absolute;\n    /* @noflip */\n    left: 0px;\n    top: 0;\n    animation-name: ${pulsateKeyframe};\n    animation-duration: 2500ms;\n    animation-timing-function: ${({\n  theme\n}) => theme.transitions.easing.easeInOut};\n    animation-iteration-count: infinite;\n    animation-delay: 200ms;\n  }\n`;\n\n/**\n * @ignore - internal component.\n *\n * TODO v5: Make private\n */\nconst TouchRipple = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function TouchRipple(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiTouchRipple'\n  });\n  const {\n    center: centerProp = false,\n    classes = {},\n    className,\n    ...other\n  } = props;\n  const [ripples, setRipples] = react__WEBPACK_IMPORTED_MODULE_0__.useState([]);\n  const nextKey = react__WEBPACK_IMPORTED_MODULE_0__.useRef(0);\n  const rippleCallback = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (rippleCallback.current) {\n      rippleCallback.current();\n      rippleCallback.current = null;\n    }\n  }, [ripples]);\n\n  // Used to filter out mouse emulated events on mobile.\n  const ignoringMouseDown = react__WEBPACK_IMPORTED_MODULE_0__.useRef(false);\n  // We use a timer in order to only show the ripples for touch \"click\" like events.\n  // We don't want to display the ripple for touch scroll events.\n  const startTimer = (0,_mui_utils_useTimeout__WEBPACK_IMPORTED_MODULE_8__[\"default\"])();\n\n  // This is the hook called once the previous timeout is ready.\n  const startTimerCommit = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const container = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const startCommit = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(params => {\n    const {\n      pulsate,\n      rippleX,\n      rippleY,\n      rippleSize,\n      cb\n    } = params;\n    setRipples(oldRipples => [...oldRipples, /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TouchRippleRipple, {\n      classes: {\n        ripple: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.ripple, _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].ripple),\n        rippleVisible: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.rippleVisible, _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].rippleVisible),\n        ripplePulsate: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.ripplePulsate, _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].ripplePulsate),\n        child: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.child, _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].child),\n        childLeaving: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.childLeaving, _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].childLeaving),\n        childPulsate: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.childPulsate, _touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].childPulsate)\n      },\n      timeout: DURATION,\n      pulsate: pulsate,\n      rippleX: rippleX,\n      rippleY: rippleY,\n      rippleSize: rippleSize\n    }, nextKey.current)]);\n    nextKey.current += 1;\n    rippleCallback.current = cb;\n  }, [classes]);\n  const start = react__WEBPACK_IMPORTED_MODULE_0__.useCallback((event = {}, options = {}, cb = () => {}) => {\n    const {\n      pulsate = false,\n      center = centerProp || options.pulsate,\n      fakeElement = false // For test purposes\n    } = options;\n    if (event?.type === 'mousedown' && ignoringMouseDown.current) {\n      ignoringMouseDown.current = false;\n      return;\n    }\n    if (event?.type === 'touchstart') {\n      ignoringMouseDown.current = true;\n    }\n    const element = fakeElement ? null : container.current;\n    const rect = element ? element.getBoundingClientRect() : {\n      width: 0,\n      height: 0,\n      left: 0,\n      top: 0\n    };\n\n    // Get the size of the ripple\n    let rippleX;\n    let rippleY;\n    let rippleSize;\n    if (center || event === undefined || event.clientX === 0 && event.clientY === 0 || !event.clientX && !event.touches) {\n      rippleX = Math.round(rect.width / 2);\n      rippleY = Math.round(rect.height / 2);\n    } else {\n      const {\n        clientX,\n        clientY\n      } = event.touches && event.touches.length > 0 ? event.touches[0] : event;\n      rippleX = Math.round(clientX - rect.left);\n      rippleY = Math.round(clientY - rect.top);\n    }\n    if (center) {\n      rippleSize = Math.sqrt((2 * rect.width ** 2 + rect.height ** 2) / 3);\n\n      // For some reason the animation is broken on Mobile Chrome if the size is even.\n      if (rippleSize % 2 === 0) {\n        rippleSize += 1;\n      }\n    } else {\n      const sizeX = Math.max(Math.abs((element ? element.clientWidth : 0) - rippleX), rippleX) * 2 + 2;\n      const sizeY = Math.max(Math.abs((element ? element.clientHeight : 0) - rippleY), rippleY) * 2 + 2;\n      rippleSize = Math.sqrt(sizeX ** 2 + sizeY ** 2);\n    }\n\n    // Touche devices\n    if (event?.touches) {\n      // check that this isn't another touchstart due to multitouch\n      // otherwise we will only clear a single timer when unmounting while two\n      // are running\n      if (startTimerCommit.current === null) {\n        // Prepare the ripple effect.\n        startTimerCommit.current = () => {\n          startCommit({\n            pulsate,\n            rippleX,\n            rippleY,\n            rippleSize,\n            cb\n          });\n        };\n        // Delay the execution of the ripple effect.\n        // We have to make a tradeoff with this delay value.\n        startTimer.start(DELAY_RIPPLE, () => {\n          if (startTimerCommit.current) {\n            startTimerCommit.current();\n            startTimerCommit.current = null;\n          }\n        });\n      }\n    } else {\n      startCommit({\n        pulsate,\n        rippleX,\n        rippleY,\n        rippleSize,\n        cb\n      });\n    }\n  }, [centerProp, startCommit, startTimer]);\n  const pulsate = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    start({}, {\n      pulsate: true\n    });\n  }, [start]);\n  const stop = react__WEBPACK_IMPORTED_MODULE_0__.useCallback((event, cb) => {\n    startTimer.clear();\n\n    // The touch interaction occurs too quickly.\n    // We still want to show ripple effect.\n    if (event?.type === 'touchend' && startTimerCommit.current) {\n      startTimerCommit.current();\n      startTimerCommit.current = null;\n      startTimer.start(0, () => {\n        stop(event, cb);\n      });\n      return;\n    }\n    startTimerCommit.current = null;\n    setRipples(oldRipples => {\n      if (oldRipples.length > 0) {\n        return oldRipples.slice(1);\n      }\n      return oldRipples;\n    });\n    rippleCallback.current = cb;\n  }, [startTimer]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useImperativeHandle(ref, () => ({\n    pulsate,\n    start,\n    stop\n  }), [pulsate, start, stop]);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TouchRippleRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(_touchRippleClasses_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"].root, classes.root, className),\n    ref: container,\n    ...other,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(react_transition_group__WEBPACK_IMPORTED_MODULE_9__[\"default\"], {\n      component: null,\n      exit: true,\n      children: ripples\n    })\n  });\n});\n true ? TouchRipple.propTypes /* remove-proptypes */ = {\n  /**\n   * If `true`, the ripple starts at the center of the component\n   * rather than at the point of interaction.\n   */\n  center: (prop_types__WEBPACK_IMPORTED_MODULE_10___default().bool),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_10___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_10___default().string)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (TouchRipple);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonBase/TouchRipple.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonBase/buttonBaseClasses.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/ButtonBase/buttonBaseClasses.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getButtonBaseUtilityClass: () => (/* binding */ getButtonBaseUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getButtonBaseUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiButtonBase', slot);\n}\nconst buttonBaseClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiButtonBase', ['root', 'disabled', 'focusVisible']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (buttonBaseClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonBase/buttonBaseClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonBase/touchRippleClasses.js":
/*!*********************************************************************!*\
  !*** ./node_modules/@mui/material/ButtonBase/touchRippleClasses.js ***!
  \*********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getTouchRippleUtilityClass: () => (/* binding */ getTouchRippleUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getTouchRippleUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiTouchRipple', slot);\n}\nconst touchRippleClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiTouchRipple', ['root', 'ripple', 'rippleVisible', 'ripplePulsate', 'child', 'childLeaving', 'childPulsate']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (touchRippleClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonBase/touchRippleClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonGroup/ButtonGroupButtonContext.js":
/*!****************************************************************************!*\
  !*** ./node_modules/@mui/material/ButtonGroup/ButtonGroupButtonContext.js ***!
  \****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n/**\n * @ignore - internal component.\n */\nconst ButtonGroupButtonContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext(undefined);\nif (true) {\n  ButtonGroupButtonContext.displayName = 'ButtonGroupButtonContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ButtonGroupButtonContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonGroup/ButtonGroupButtonContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ButtonGroup/ButtonGroupContext.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/ButtonGroup/ButtonGroupContext.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n/**\n * @ignore - internal component.\n */\nconst ButtonGroupContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext({});\nif (true) {\n  ButtonGroupContext.displayName = 'ButtonGroupContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ButtonGroupContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ButtonGroup/ButtonGroupContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Card/Card.js":
/*!*************************************************!*\
  !*** ./node_modules/@mui/material/Card/Card.js ***!
  \*************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/utils/chainPropTypes */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _Paper_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Paper/index.js */ \"./node_modules/@mui/material/Paper/Paper.js\");\n/* harmony import */ var _cardClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./cardClasses.js */ \"./node_modules/@mui/material/Card/cardClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _cardClasses_js__WEBPACK_IMPORTED_MODULE_4__.getCardUtilityClass, classes);\n};\nconst CardRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Paper_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n  name: 'MuiCard',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({\n  overflow: 'hidden'\n});\nconst Card = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Card(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiCard'\n  });\n  const {\n    className,\n    raised = false,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    raised\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CardRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    elevation: raised ? 8 : undefined,\n    ref: ref,\n    ownerState: ownerState,\n    ...other\n  });\n});\n true ? Card.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n  /**\n   * If `true`, the card will use raised styling.\n   * @default false\n   */\n  raised: (0,_mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_9__[\"default\"])((prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool), props => {\n    if (props.raised && props.variant === 'outlined') {\n      return new Error('MUI: Combining `raised={true}` with `variant=\"outlined\"` has no effect.');\n    }\n    return null;\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_8___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_8___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Card);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Card/Card.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Card/cardClasses.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/Card/cardClasses.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getCardUtilityClass: () => (/* binding */ getCardUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getCardUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiCard', slot);\n}\nconst cardClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiCard', ['root']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (cardClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Card/cardClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CardActions/CardActions.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/CardActions/CardActions.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_7__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _cardActionsClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./cardActionsClasses.js */ \"./node_modules/@mui/material/CardActions/cardActionsClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    disableSpacing\n  } = ownerState;\n  const slots = {\n    root: ['root', !disableSpacing && 'spacing']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _cardActionsClasses_js__WEBPACK_IMPORTED_MODULE_4__.getCardActionsUtilityClass, classes);\n};\nconst CardActionsRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiCardActions',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, !ownerState.disableSpacing && styles.spacing];\n  }\n})({\n  display: 'flex',\n  alignItems: 'center',\n  padding: 8,\n  variants: [{\n    props: {\n      disableSpacing: false\n    },\n    style: {\n      '& > :not(style) ~ :not(style)': {\n        marginLeft: 8\n      }\n    }\n  }]\n});\nconst CardActions = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function CardActions(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiCardActions'\n  });\n  const {\n    disableSpacing = false,\n    className,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    disableSpacing\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CardActionsRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState: ownerState,\n    ref: ref,\n    ...other\n  });\n});\n true ? CardActions.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string),\n  /**\n   * If `true`, the actions do not have additional margin.\n   * @default false\n   */\n  disableSpacing: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_7___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_7___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (CardActions);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CardActions/CardActions.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CardActions/cardActionsClasses.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/CardActions/cardActionsClasses.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getCardActionsUtilityClass: () => (/* binding */ getCardActionsUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getCardActionsUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiCardActions', slot);\n}\nconst cardActionsClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiCardActions', ['root', 'spacing']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (cardActionsClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CardActions/cardActionsClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CardContent/CardContent.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/CardContent/CardContent.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_7__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _cardContentClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./cardContentClasses.js */ \"./node_modules/@mui/material/CardContent/cardContentClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _cardContentClasses_js__WEBPACK_IMPORTED_MODULE_4__.getCardContentUtilityClass, classes);\n};\nconst CardContentRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiCardContent',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({\n  padding: 16,\n  '&:last-child': {\n    paddingBottom: 24\n  }\n});\nconst CardContent = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function CardContent(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiCardContent'\n  });\n  const {\n    className,\n    component = 'div',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    component\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CardContentRoot, {\n    as: component,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState: ownerState,\n    ref: ref,\n    ...other\n  });\n});\n true ? CardContent.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().elementType),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_7___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_7___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (CardContent);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CardContent/CardContent.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CardContent/cardContentClasses.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/CardContent/cardContentClasses.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getCardContentUtilityClass: () => (/* binding */ getCardContentUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getCardContentUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiCardContent', slot);\n}\nconst cardContentClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiCardContent', ['root']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (cardContentClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CardContent/cardContentClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CircularProgress/CircularProgress.js":
/*!*************************************************************************!*\
  !*** ./node_modules/@mui/material/CircularProgress/CircularProgress.js ***!
  \*************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/utils/chainPropTypes */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _circularProgressClasses_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./circularProgressClasses.js */ \"./node_modules/@mui/material/CircularProgress/circularProgressClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst SIZE = 44;\nconst circularRotateKeyframe = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.keyframes)`\n  0% {\n    transform: rotate(0deg);\n  }\n\n  100% {\n    transform: rotate(360deg);\n  }\n`;\nconst circularDashKeyframe = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.keyframes)`\n  0% {\n    stroke-dasharray: 1px, 200px;\n    stroke-dashoffset: 0;\n  }\n\n  50% {\n    stroke-dasharray: 100px, 200px;\n    stroke-dashoffset: -15px;\n  }\n\n  100% {\n    stroke-dasharray: 1px, 200px;\n    stroke-dashoffset: -126px;\n  }\n`;\n\n// This implementation is for supporting both Styled-components v4+ and Pigment CSS.\n// A global animation has to be created here for Styled-components v4+ (https://github.com/styled-components/styled-components/blob/main/packages/styled-components/src/utils/errors.md#12).\n// which can be done by checking typeof indeterminate1Keyframe !== 'string' (at runtime, Pigment CSS transform keyframes`` to a string).\nconst rotateAnimation = typeof circularRotateKeyframe !== 'string' ? (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.css)`\n        animation: ${circularRotateKeyframe} 1.4s linear infinite;\n      ` : null;\nconst dashAnimation = typeof circularDashKeyframe !== 'string' ? (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.css)`\n        animation: ${circularDashKeyframe} 1.4s ease-in-out infinite;\n      ` : null;\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    variant,\n    color,\n    disableShrink\n  } = ownerState;\n  const slots = {\n    root: ['root', variant, `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(color)}`],\n    svg: ['svg'],\n    circle: ['circle', `circle${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(variant)}`, disableShrink && 'circleDisableShrink']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(slots, _circularProgressClasses_js__WEBPACK_IMPORTED_MODULE_6__.getCircularProgressUtilityClass, classes);\n};\nconst CircularProgressRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])('span', {\n  name: 'MuiCircularProgress',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[ownerState.variant], styles[`color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(ownerState.color)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  display: 'inline-block',\n  variants: [{\n    props: {\n      variant: 'determinate'\n    },\n    style: {\n      transition: theme.transitions.create('transform')\n    }\n  }, {\n    props: {\n      variant: 'indeterminate'\n    },\n    style: rotateAnimation || {\n      animation: `${circularRotateKeyframe} 1.4s linear infinite`\n    }\n  }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])()).map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      color: (theme.vars || theme).palette[color].main\n    }\n  }))]\n})));\nconst CircularProgressSVG = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])('svg', {\n  name: 'MuiCircularProgress',\n  slot: 'Svg',\n  overridesResolver: (props, styles) => styles.svg\n})({\n  display: 'block' // Keeps the progress centered\n});\nconst CircularProgressCircle = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])('circle', {\n  name: 'MuiCircularProgress',\n  slot: 'Circle',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.circle, styles[`circle${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(ownerState.variant)}`], ownerState.disableShrink && styles.circleDisableShrink];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  stroke: 'currentColor',\n  variants: [{\n    props: {\n      variant: 'determinate'\n    },\n    style: {\n      transition: theme.transitions.create('stroke-dashoffset')\n    }\n  }, {\n    props: {\n      variant: 'indeterminate'\n    },\n    style: {\n      // Some default value that looks fine waiting for the animation to kicks in.\n      strokeDasharray: '80px, 200px',\n      strokeDashoffset: 0 // Add the unit to fix a Edge 16 and below bug.\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.variant === 'indeterminate' && !ownerState.disableShrink,\n    style: dashAnimation || {\n      // At runtime for Pigment CSS, `bufferAnimation` will be null and the generated keyframe will be used.\n      animation: `${circularDashKeyframe} 1.4s ease-in-out infinite`\n    }\n  }]\n})));\n\n/**\n * ## ARIA\n *\n * If the progress bar is describing the loading progress of a particular region of a page,\n * you should use `aria-describedby` to point to the progress bar, and set the `aria-busy`\n * attribute to `true` on that region until it has finished loading.\n */\nconst CircularProgress = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function CircularProgress(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiCircularProgress'\n  });\n  const {\n    className,\n    color = 'primary',\n    disableShrink = false,\n    size = 40,\n    style,\n    thickness = 3.6,\n    value = 0,\n    variant = 'indeterminate',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    color,\n    disableShrink,\n    size,\n    thickness,\n    value,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  const circleStyle = {};\n  const rootStyle = {};\n  const rootProps = {};\n  if (variant === 'determinate') {\n    const circumference = 2 * Math.PI * ((SIZE - thickness) / 2);\n    circleStyle.strokeDasharray = circumference.toFixed(3);\n    rootProps['aria-valuenow'] = Math.round(value);\n    circleStyle.strokeDashoffset = `${((100 - value) / 100 * circumference).toFixed(3)}px`;\n    rootStyle.transform = 'rotate(-90deg)';\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CircularProgressRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    style: {\n      width: size,\n      height: size,\n      ...rootStyle,\n      ...style\n    },\n    ownerState: ownerState,\n    ref: ref,\n    role: \"progressbar\",\n    ...rootProps,\n    ...other,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CircularProgressSVG, {\n      className: classes.svg,\n      ownerState: ownerState,\n      viewBox: `${SIZE / 2} ${SIZE / 2} ${SIZE} ${SIZE}`,\n      children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(CircularProgressCircle, {\n        className: classes.circle,\n        style: circleStyle,\n        ownerState: ownerState,\n        cx: SIZE,\n        cy: SIZE,\n        r: (SIZE - thickness) / 2,\n        fill: \"none\",\n        strokeWidth: thickness\n      })\n    })\n  });\n});\n true ? CircularProgress.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'primary'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['inherit', 'primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * If `true`, the shrink animation is disabled.\n   * This only works if variant is `indeterminate`.\n   * @default false\n   */\n  disableShrink: (0,_mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_12__[\"default\"])((prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool), props => {\n    if (props.disableShrink && props.variant && props.variant !== 'indeterminate') {\n      return new Error('MUI: You have provided the `disableShrink` prop ' + 'with a variant other than `indeterminate`. This will have no effect.');\n    }\n    return null;\n  }),\n  /**\n   * The size of the component.\n   * If using a number, the pixel unit is assumed.\n   * If using a string, you need to provide the CSS unit, for example '3rem'.\n   * @default 40\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n  /**\n   * The thickness of the circle.\n   * @default 3.6\n   */\n  thickness: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().number),\n  /**\n   * The value of the progress indicator for the determinate variant.\n   * Value between 0 and 100.\n   * @default 0\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().number),\n  /**\n   * The variant to use.\n   * Use indeterminate when there is no progress value.\n   * @default 'indeterminate'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['determinate', 'indeterminate'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (CircularProgress);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CircularProgress/CircularProgress.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CircularProgress/circularProgressClasses.js":
/*!********************************************************************************!*\
  !*** ./node_modules/@mui/material/CircularProgress/circularProgressClasses.js ***!
  \********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getCircularProgressUtilityClass: () => (/* binding */ getCircularProgressUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getCircularProgressUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiCircularProgress', slot);\n}\nconst circularProgressClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiCircularProgress', ['root', 'determinate', 'indeterminate', 'colorPrimary', 'colorSecondary', 'svg', 'circle', 'circleDeterminate', 'circleIndeterminate', 'circleDisableShrink']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (circularProgressClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CircularProgress/circularProgressClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Container/Container.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/Container/Container.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_4___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_4__);\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/Container/createContainer.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _styles_styled_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../styles/styled.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n'use client';\n\n\n\n\n\n\nconst Container = (0,_mui_system__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  createStyledComponent: (0,_styles_styled_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('div', {\n    name: 'MuiContainer',\n    slot: 'Root',\n    overridesResolver: (props, styles) => {\n      const {\n        ownerState\n      } = props;\n      return [styles.root, styles[`maxWidth${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(String(ownerState.maxWidth))}`], ownerState.fixed && styles.fixed, ownerState.disableGutters && styles.disableGutters];\n    }\n  }),\n  useThemeProps: inProps => (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_3__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiContainer'\n  })\n});\n true ? Container.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().object),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().elementType),\n  /**\n   * If `true`, the left and right padding is removed.\n   * @default false\n   */\n  disableGutters: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().bool),\n  /**\n   * Set the max-width to match the min-width of the current breakpoint.\n   * This is useful if you'd prefer to design for a fixed set of sizes\n   * instead of trying to accommodate a fully fluid viewport.\n   * It's fluid by default.\n   * @default false\n   */\n  fixed: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().bool),\n  /**\n   * Determine the max-width of the container.\n   * The container width grows with the size of the screen.\n   * Set to `false` to disable `maxWidth`.\n   * @default 'lg'\n   */\n  maxWidth: prop_types__WEBPACK_IMPORTED_MODULE_4___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_4___default().oneOf(['xs', 'sm', 'md', 'lg', 'xl', false]), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().string)]),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_4___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_4___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_4___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_4___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Container);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Container/Container.js?");

/***/ }),

/***/ "./node_modules/@mui/material/CssBaseline/CssBaseline.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/CssBaseline/CssBaseline.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   body: () => (/* binding */ body),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   html: () => (/* binding */ html),\n/* harmony export */   styles: () => (/* binding */ styles)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_4___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_4__);\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/zero-styled/index.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n// to determine if the global styles are static or dynamic\n\nconst isDynamicSupport = typeof (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__.globalCss)({}) === 'function';\nconst html = (theme, enableColorScheme) => ({\n  WebkitFontSmoothing: 'antialiased',\n  // Antialiasing.\n  MozOsxFontSmoothing: 'grayscale',\n  // Antialiasing.\n  // Change from `box-sizing: content-box` so that `width`\n  // is not affected by `padding` or `border`.\n  boxSizing: 'border-box',\n  // Fix font resize problem in iOS\n  WebkitTextSizeAdjust: '100%',\n  // When used under CssVarsProvider, colorScheme should not be applied dynamically because it will generate the stylesheet twice for server-rendered applications.\n  ...(enableColorScheme && !theme.vars && {\n    colorScheme: theme.palette.mode\n  })\n});\nconst body = theme => ({\n  color: (theme.vars || theme).palette.text.primary,\n  ...theme.typography.body1,\n  backgroundColor: (theme.vars || theme).palette.background.default,\n  '@media print': {\n    // Save printer ink.\n    backgroundColor: (theme.vars || theme).palette.common.white\n  }\n});\nconst styles = (theme, enableColorScheme = false) => {\n  const colorSchemeStyles = {};\n  if (enableColorScheme && theme.colorSchemes && typeof theme.getColorSchemeSelector === 'function') {\n    Object.entries(theme.colorSchemes).forEach(([key, scheme]) => {\n      const selector = theme.getColorSchemeSelector(key);\n      if (selector.startsWith('@')) {\n        // for @media (prefers-color-scheme), we need to target :root\n        colorSchemeStyles[selector] = {\n          ':root': {\n            colorScheme: scheme.palette?.mode\n          }\n        };\n      } else {\n        // else, it's likely that the selector already target an element with a class or data attribute\n        colorSchemeStyles[selector.replace(/\\s*&/, '')] = {\n          colorScheme: scheme.palette?.mode\n        };\n      }\n    });\n  }\n  let defaultStyles = {\n    html: html(theme, enableColorScheme),\n    '*, *::before, *::after': {\n      boxSizing: 'inherit'\n    },\n    'strong, b': {\n      fontWeight: theme.typography.fontWeightBold\n    },\n    body: {\n      margin: 0,\n      // Remove the margin in all browsers.\n      ...body(theme),\n      // Add support for document.body.requestFullScreen().\n      // Other elements, if background transparent, are not supported.\n      '&::backdrop': {\n        backgroundColor: (theme.vars || theme).palette.background.default\n      }\n    },\n    ...colorSchemeStyles\n  };\n  const themeOverrides = theme.components?.MuiCssBaseline?.styleOverrides;\n  if (themeOverrides) {\n    defaultStyles = [defaultStyles, themeOverrides];\n  }\n  return defaultStyles;\n};\n\n// `ecs` stands for enableColorScheme. This is internal logic to make it work with Pigment CSS, so shorter is better.\nconst SELECTOR = 'mui-ecs';\nconst staticStyles = theme => {\n  const result = styles(theme, false);\n  const baseStyles = Array.isArray(result) ? result[0] : result;\n  if (!theme.vars && baseStyles) {\n    baseStyles.html[`:root:has(${SELECTOR})`] = {\n      colorScheme: theme.palette.mode\n    };\n  }\n  if (theme.colorSchemes) {\n    Object.entries(theme.colorSchemes).forEach(([key, scheme]) => {\n      const selector = theme.getColorSchemeSelector(key);\n      if (selector.startsWith('@')) {\n        // for @media (prefers-color-scheme), we need to target :root\n        baseStyles[selector] = {\n          [`:root:not(:has(.${SELECTOR}))`]: {\n            colorScheme: scheme.palette?.mode\n          }\n        };\n      } else {\n        // else, it's likely that the selector already target an element with a class or data attribute\n        baseStyles[selector.replace(/\\s*&/, '')] = {\n          [`&:not(:has(.${SELECTOR}))`]: {\n            colorScheme: scheme.palette?.mode\n          }\n        };\n      }\n    });\n  }\n  return result;\n};\nconst GlobalStyles = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__.globalCss)(isDynamicSupport ? ({\n  theme,\n  enableColorScheme\n}) => styles(theme, enableColorScheme) : ({\n  theme\n}) => staticStyles(theme));\n\n/**\n * Kickstart an elegant, consistent, and simple baseline to build upon.\n */\nfunction CssBaseline(inProps) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_3__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiCssBaseline'\n  });\n  const {\n    children,\n    enableColorScheme = false\n  } = props;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: [isDynamicSupport && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(GlobalStyles, {\n      enableColorScheme: enableColorScheme\n    }), !isDynamicSupport && !enableColorScheme && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"span\", {\n      className: SELECTOR,\n      style: {\n        display: 'none'\n      }\n    }), children]\n  });\n}\n true ? CssBaseline.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * You can wrap a node.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().node),\n  /**\n   * Enable `color-scheme` CSS property to use `theme.palette.mode`.\n   * For more details, check out https://developer.mozilla.org/en-US/docs/Web/CSS/color-scheme\n   * For browser support, check out https://caniuse.com/?search=color-scheme\n   * @default false\n   */\n  enableColorScheme: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().bool)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (CssBaseline);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/CssBaseline/CssBaseline.js?");

/***/ }),

/***/ "./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js":
/*!*********************************************************************************!*\
  !*** ./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js ***!
  \*********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   useDefaultProps: () => (/* binding */ useDefaultProps)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_3__);\n/* harmony import */ var _mui_system_DefaultPropsProvider__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system/DefaultPropsProvider */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\nfunction DefaultPropsProvider(props) {\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_system_DefaultPropsProvider__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n    ...props\n  });\n}\n true ? DefaultPropsProvider.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().node),\n  /**\n   * @ignore\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object).isRequired\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (DefaultPropsProvider);\nfunction useDefaultProps(params) {\n  return (0,_mui_system_DefaultPropsProvider__WEBPACK_IMPORTED_MODULE_2__.useDefaultProps)(params);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Dialog/Dialog.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/Dialog/Dialog.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_17___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_17__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_useId__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/useId */ \"./node_modules/@mui/utils/esm/useId/useId.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _Modal_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Modal/index.js */ \"./node_modules/@mui/material/Modal/Modal.js\");\n/* harmony import */ var _Fade_index_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../Fade/index.js */ \"./node_modules/@mui/material/Fade/Fade.js\");\n/* harmony import */ var _Paper_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../Paper/index.js */ \"./node_modules/@mui/material/Paper/Paper.js\");\n/* harmony import */ var _dialogClasses_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./dialogClasses.js */ \"./node_modules/@mui/material/Dialog/dialogClasses.js\");\n/* harmony import */ var _DialogContext_js__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ./DialogContext.js */ \"./node_modules/@mui/material/Dialog/DialogContext.js\");\n/* harmony import */ var _Backdrop_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../Backdrop/index.js */ \"./node_modules/@mui/material/Backdrop/Backdrop.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/useTheme.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst DialogBackdrop = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(_Backdrop_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"], {\n  name: 'MuiDialog',\n  slot: 'Backdrop',\n  overrides: (props, styles) => styles.backdrop\n})({\n  // Improve scrollable dialog support.\n  zIndex: -1\n});\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    scroll,\n    maxWidth,\n    fullWidth,\n    fullScreen\n  } = ownerState;\n  const slots = {\n    root: ['root'],\n    container: ['container', `scroll${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(scroll)}`],\n    paper: ['paper', `paperScroll${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(scroll)}`, `paperWidth${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(String(maxWidth))}`, fullWidth && 'paperFullWidth', fullScreen && 'paperFullScreen']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(slots, _dialogClasses_js__WEBPACK_IMPORTED_MODULE_7__.getDialogUtilityClass, classes);\n};\nconst DialogRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(_Modal_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"], {\n  name: 'MuiDialog',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({\n  '@media print': {\n    // Use !important to override the Modal inline-style.\n    position: 'absolute !important'\n  }\n});\nconst DialogContainer = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])('div', {\n  name: 'MuiDialog',\n  slot: 'Container',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.container, styles[`scroll${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(ownerState.scroll)}`]];\n  }\n})({\n  height: '100%',\n  '@media print': {\n    height: 'auto'\n  },\n  // We disable the focus ring for mouse, touch and keyboard users.\n  outline: 0,\n  variants: [{\n    props: {\n      scroll: 'paper'\n    },\n    style: {\n      display: 'flex',\n      justifyContent: 'center',\n      alignItems: 'center'\n    }\n  }, {\n    props: {\n      scroll: 'body'\n    },\n    style: {\n      overflowY: 'auto',\n      overflowX: 'hidden',\n      textAlign: 'center',\n      '&::after': {\n        content: '\"\"',\n        display: 'inline-block',\n        verticalAlign: 'middle',\n        height: '100%',\n        width: '0'\n      }\n    }\n  }]\n});\nconst DialogPaper = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(_Paper_index_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"], {\n  name: 'MuiDialog',\n  slot: 'Paper',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.paper, styles[`scrollPaper${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(ownerState.scroll)}`], styles[`paperWidth${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(String(ownerState.maxWidth))}`], ownerState.fullWidth && styles.paperFullWidth, ownerState.fullScreen && styles.paperFullScreen];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(({\n  theme\n}) => ({\n  margin: 32,\n  position: 'relative',\n  overflowY: 'auto',\n  '@media print': {\n    overflowY: 'visible',\n    boxShadow: 'none'\n  },\n  variants: [{\n    props: {\n      scroll: 'paper'\n    },\n    style: {\n      display: 'flex',\n      flexDirection: 'column',\n      maxHeight: 'calc(100% - 64px)'\n    }\n  }, {\n    props: {\n      scroll: 'body'\n    },\n    style: {\n      display: 'inline-block',\n      verticalAlign: 'middle',\n      textAlign: 'initial'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => !ownerState.maxWidth,\n    style: {\n      maxWidth: 'calc(100% - 64px)'\n    }\n  }, {\n    props: {\n      maxWidth: 'xs'\n    },\n    style: {\n      maxWidth: theme.breakpoints.unit === 'px' ? Math.max(theme.breakpoints.values.xs, 444) : `max(${theme.breakpoints.values.xs}${theme.breakpoints.unit}, 444px)`,\n      [`&.${_dialogClasses_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"].paperScrollBody}`]: {\n        [theme.breakpoints.down(Math.max(theme.breakpoints.values.xs, 444) + 32 * 2)]: {\n          maxWidth: 'calc(100% - 64px)'\n        }\n      }\n    }\n  }, ...Object.keys(theme.breakpoints.values).filter(maxWidth => maxWidth !== 'xs').map(maxWidth => ({\n    props: {\n      maxWidth\n    },\n    style: {\n      maxWidth: `${theme.breakpoints.values[maxWidth]}${theme.breakpoints.unit}`,\n      [`&.${_dialogClasses_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"].paperScrollBody}`]: {\n        [theme.breakpoints.down(theme.breakpoints.values[maxWidth] + 32 * 2)]: {\n          maxWidth: 'calc(100% - 64px)'\n        }\n      }\n    }\n  })), {\n    props: ({\n      ownerState\n    }) => ownerState.fullWidth,\n    style: {\n      width: 'calc(100% - 64px)'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.fullScreen,\n    style: {\n      margin: 0,\n      width: '100%',\n      maxWidth: '100%',\n      height: '100%',\n      maxHeight: 'none',\n      borderRadius: 0,\n      [`&.${_dialogClasses_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"].paperScrollBody}`]: {\n        margin: 0,\n        maxWidth: '100%'\n      }\n    }\n  }]\n})));\n\n/**\n * Dialogs are overlaid modal paper based components with a backdrop.\n */\nconst Dialog = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Dialog(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiDialog'\n  });\n  const theme = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])();\n  const defaultTransitionDuration = {\n    enter: theme.transitions.duration.enteringScreen,\n    exit: theme.transitions.duration.leavingScreen\n  };\n  const {\n    'aria-describedby': ariaDescribedby,\n    'aria-labelledby': ariaLabelledbyProp,\n    'aria-modal': ariaModal = true,\n    BackdropComponent,\n    BackdropProps,\n    children,\n    className,\n    disableEscapeKeyDown = false,\n    fullScreen = false,\n    fullWidth = false,\n    maxWidth = 'sm',\n    onBackdropClick,\n    onClick,\n    onClose,\n    open,\n    PaperComponent = _Paper_index_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"],\n    PaperProps = {},\n    scroll = 'paper',\n    slots = {},\n    slotProps = {},\n    TransitionComponent = _Fade_index_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"],\n    transitionDuration = defaultTransitionDuration,\n    TransitionProps,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    disableEscapeKeyDown,\n    fullScreen,\n    fullWidth,\n    maxWidth,\n    scroll\n  };\n  const classes = useUtilityClasses(ownerState);\n  const backdropClick = react__WEBPACK_IMPORTED_MODULE_0__.useRef();\n  const handleMouseDown = event => {\n    // We don't want to close the dialog when clicking the dialog content.\n    // Make sure the event starts and ends on the same DOM element.\n    backdropClick.current = event.target === event.currentTarget;\n  };\n  const handleBackdropClick = event => {\n    if (onClick) {\n      onClick(event);\n    }\n\n    // Ignore the events not coming from the \"backdrop\".\n    if (!backdropClick.current) {\n      return;\n    }\n    backdropClick.current = null;\n    if (onBackdropClick) {\n      onBackdropClick(event);\n    }\n    if (onClose) {\n      onClose(event, 'backdropClick');\n    }\n  };\n  const ariaLabelledby = (0,_mui_utils_useId__WEBPACK_IMPORTED_MODULE_14__[\"default\"])(ariaLabelledbyProp);\n  const dialogContextValue = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    return {\n      titleId: ariaLabelledby\n    };\n  }, [ariaLabelledby]);\n  const backwardCompatibleSlots = {\n    transition: TransitionComponent,\n    ...slots\n  };\n  const backwardCompatibleSlotProps = {\n    transition: TransitionProps,\n    paper: PaperProps,\n    backdrop: BackdropProps,\n    ...slotProps\n  };\n  const externalForwardedProps = {\n    slots: backwardCompatibleSlots,\n    slotProps: backwardCompatibleSlotProps\n  };\n  const [RootSlot, rootSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])('root', {\n    elementType: DialogRoot,\n    shouldForwardComponentProp: true,\n    externalForwardedProps,\n    ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ref\n  });\n  const [BackdropSlot, backdropSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])('backdrop', {\n    elementType: DialogBackdrop,\n    shouldForwardComponentProp: true,\n    externalForwardedProps,\n    ownerState\n  });\n  const [PaperSlot, paperSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])('paper', {\n    elementType: DialogPaper,\n    shouldForwardComponentProp: true,\n    externalForwardedProps,\n    ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.paper, PaperProps.className)\n  });\n  const [ContainerSlot, containerSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])('container', {\n    elementType: DialogContainer,\n    externalForwardedProps,\n    ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.container)\n  });\n  const [TransitionSlot, transitionSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])('transition', {\n    elementType: _Fade_index_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"],\n    externalForwardedProps,\n    ownerState,\n    additionalProps: {\n      appear: true,\n      in: open,\n      timeout: transitionDuration,\n      role: 'presentation'\n    }\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(RootSlot, {\n    closeAfterTransition: true,\n    slots: {\n      backdrop: BackdropSlot\n    },\n    slotProps: {\n      backdrop: {\n        transitionDuration,\n        as: BackdropComponent,\n        ...backdropSlotProps\n      }\n    },\n    disableEscapeKeyDown: disableEscapeKeyDown,\n    onClose: onClose,\n    open: open,\n    onClick: handleBackdropClick,\n    ...rootSlotProps,\n    ...other,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TransitionSlot, {\n      ...transitionSlotProps,\n      children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ContainerSlot, {\n        onMouseDown: handleMouseDown,\n        ...containerSlotProps,\n        children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(PaperSlot, {\n          as: PaperComponent,\n          elevation: 24,\n          role: \"dialog\",\n          \"aria-describedby\": ariaDescribedby,\n          \"aria-labelledby\": ariaLabelledby,\n          \"aria-modal\": ariaModal,\n          ...paperSlotProps,\n          children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_DialogContext_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"].Provider, {\n            value: dialogContextValue,\n            children: children\n          })\n        })\n      })\n    })\n  });\n});\n true ? Dialog.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The id(s) of the element(s) that describe the dialog.\n   */\n  'aria-describedby': (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string),\n  /**\n   * The id(s) of the element(s) that label the dialog.\n   */\n  'aria-labelledby': (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string),\n  /**\n   * Informs assistive technologies that the element is modal.\n   * It's added on the element with role=\"dialog\".\n   * @default true\n   */\n  'aria-modal': prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOf(['false', 'true']), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool)]),\n  /**\n   * A backdrop component. This prop enables custom backdrop rendering.\n   * @deprecated Use `slots.backdrop` instead. While this prop currently works, it will be removed in the next major version.\n   * Use the `slots.backdrop` prop to make your application ready for the next version of Material UI.\n   * @default styled(Backdrop, {\n   *   name: 'MuiModal',\n   *   slot: 'Backdrop',\n   *   overridesResolver: (props, styles) => {\n   *     return styles.backdrop;\n   *   },\n   * })({\n   *   zIndex: -1,\n   * })\n   */\n  BackdropComponent: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n  /**\n   * @ignore\n   */\n  BackdropProps: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object),\n  /**\n   * Dialog children, usually the included sub-components.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string),\n  /**\n   * If `true`, hitting escape will not fire the `onClose` callback.\n   * @default false\n   */\n  disableEscapeKeyDown: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * If `true`, the dialog is full-screen.\n   * @default false\n   */\n  fullScreen: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * If `true`, the dialog stretches to `maxWidth`.\n   *\n   * Notice that the dialog width grow is limited by the default margin.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * Determine the max-width of the dialog.\n   * The dialog width grows with the size of the screen.\n   * Set to `false` to disable `maxWidth`.\n   * @default 'sm'\n   */\n  maxWidth: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOf(['xs', 'sm', 'md', 'lg', 'xl', false]), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string)]),\n  /**\n   * Callback fired when the backdrop is clicked.\n   * @deprecated Use the `onClose` prop with the `reason` argument to handle the `backdropClick` events.\n   */\n  onBackdropClick: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().func),\n  /**\n   * @ignore\n   */\n  onClick: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().func),\n  /**\n   * Callback fired when the component requests to be closed.\n   *\n   * @param {object} event The event source of the callback.\n   * @param {string} reason Can be: `\"escapeKeyDown\"`, `\"backdropClick\"`.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().func),\n  /**\n   * If `true`, the component is shown.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool).isRequired,\n  /**\n   * The component used to render the body of the dialog.\n   * @default Paper\n   */\n  PaperComponent: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n  /**\n   * Props applied to the [`Paper`](https://mui.com/material-ui/api/paper/) element.\n   * @default {}\n   * @deprecated Use `slotProps.paper` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  PaperProps: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object),\n  /**\n   * Determine the container for scrolling the dialog.\n   * @default 'paper'\n   */\n  scroll: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOf(['body', 'paper']),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_17___default().shape({\n    backdrop: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)]),\n    container: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)]),\n    paper: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)]),\n    transition: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_17___default().shape({\n    backdrop: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n    container: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n    paper: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n    transition: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_17___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)]),\n  /**\n   * The component used for the transition.\n   * [Follow this guide](https://mui.com/material-ui/transitions/#transitioncomponent-prop) to learn more about the requirements for this component.\n   * @default Fade\n   * @deprecated Use `slots.transition` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  TransitionComponent: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n  /**\n   * The duration for the transition, in milliseconds.\n   * You may specify a single timeout for all transitions, or individually with an object.\n   * @default {\n   *   enter: theme.transitions.duration.enteringScreen,\n   *   exit: theme.transitions.duration.leavingScreen,\n   * }\n   */\n  transitionDuration: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().number), prop_types__WEBPACK_IMPORTED_MODULE_17___default().shape({\n    appear: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().number),\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().number),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().number)\n  })]),\n  /**\n   * Props applied to the transition element.\n   * By default, the element is based on this [`Transition`](https://reactcommunity.org/react-transition-group/transition/) component.\n   * @deprecated Use `slotProps.transition` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  TransitionProps: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Dialog);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Dialog/Dialog.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Dialog/DialogContext.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/Dialog/DialogContext.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\nconst DialogContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext({});\nif (true) {\n  DialogContext.displayName = 'DialogContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (DialogContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Dialog/DialogContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Dialog/dialogClasses.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/Dialog/dialogClasses.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getDialogUtilityClass: () => (/* binding */ getDialogUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getDialogUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiDialog', slot);\n}\nconst dialogClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiDialog', ['root', 'scrollPaper', 'scrollBody', 'container', 'paper', 'paperScrollPaper', 'paperScrollBody', 'paperWidthFalse', 'paperWidthXs', 'paperWidthSm', 'paperWidthMd', 'paperWidthLg', 'paperWidthXl', 'paperFullWidth', 'paperFullScreen']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (dialogClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Dialog/dialogClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/DialogContent/DialogContent.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@mui/material/DialogContent/DialogContent.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _dialogContentClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./dialogContentClasses.js */ \"./node_modules/@mui/material/DialogContent/dialogContentClasses.js\");\n/* harmony import */ var _DialogTitle_dialogTitleClasses_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DialogTitle/dialogTitleClasses.js */ \"./node_modules/@mui/material/DialogTitle/dialogTitleClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    dividers\n  } = ownerState;\n  const slots = {\n    root: ['root', dividers && 'dividers']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _dialogContentClasses_js__WEBPACK_IMPORTED_MODULE_4__.getDialogContentUtilityClass, classes);\n};\nconst DialogContentRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiDialogContent',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.dividers && styles.dividers];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(({\n  theme\n}) => ({\n  flex: '1 1 auto',\n  // Add iOS momentum scrolling for iOS < 13.0\n  WebkitOverflowScrolling: 'touch',\n  overflowY: 'auto',\n  padding: '20px 24px',\n  variants: [{\n    props: ({\n      ownerState\n    }) => ownerState.dividers,\n    style: {\n      padding: '16px 24px',\n      borderTop: `1px solid ${(theme.vars || theme).palette.divider}`,\n      borderBottom: `1px solid ${(theme.vars || theme).palette.divider}`\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => !ownerState.dividers,\n    style: {\n      [`.${_DialogTitle_dialogTitleClasses_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"].root} + &`]: {\n        paddingTop: 0\n      }\n    }\n  }]\n})));\nconst DialogContent = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function DialogContent(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiDialogContent'\n  });\n  const {\n    className,\n    dividers = false,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    dividers\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(DialogContentRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState: ownerState,\n    ref: ref,\n    ...other\n  });\n});\n true ? DialogContent.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * Display the top and bottom dividers.\n   * @default false\n   */\n  dividers: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (DialogContent);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/DialogContent/DialogContent.js?");

/***/ }),

/***/ "./node_modules/@mui/material/DialogContent/dialogContentClasses.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/material/DialogContent/dialogContentClasses.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getDialogContentUtilityClass: () => (/* binding */ getDialogContentUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getDialogContentUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiDialogContent', slot);\n}\nconst dialogContentClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiDialogContent', ['root', 'dividers']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (dialogContentClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/DialogContent/dialogContentClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/DialogTitle/DialogTitle.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/DialogTitle/DialogTitle.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _Typography_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Typography/index.js */ \"./node_modules/@mui/material/Typography/Typography.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _dialogTitleClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./dialogTitleClasses.js */ \"./node_modules/@mui/material/DialogTitle/dialogTitleClasses.js\");\n/* harmony import */ var _Dialog_DialogContext_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../Dialog/DialogContext.js */ \"./node_modules/@mui/material/Dialog/DialogContext.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _dialogTitleClasses_js__WEBPACK_IMPORTED_MODULE_4__.getDialogTitleUtilityClass, classes);\n};\nconst DialogTitleRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Typography_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n  name: 'MuiDialogTitle',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({\n  padding: '16px 24px',\n  flex: '0 0 auto'\n});\nconst DialogTitle = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function DialogTitle(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiDialogTitle'\n  });\n  const {\n    className,\n    id: idProp,\n    ...other\n  } = props;\n  const ownerState = props;\n  const classes = useUtilityClasses(ownerState);\n  const {\n    titleId = idProp\n  } = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_Dialog_DialogContext_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"]);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(DialogTitleRoot, {\n    component: \"h2\",\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState: ownerState,\n    ref: ref,\n    variant: \"h6\",\n    id: idProp ?? titleId,\n    ...other\n  });\n});\n true ? DialogTitle.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * @ignore\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (DialogTitle);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/DialogTitle/DialogTitle.js?");

/***/ }),

/***/ "./node_modules/@mui/material/DialogTitle/dialogTitleClasses.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/DialogTitle/dialogTitleClasses.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getDialogTitleUtilityClass: () => (/* binding */ getDialogTitleUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getDialogTitleUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiDialogTitle', slot);\n}\nconst dialogTitleClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiDialogTitle', ['root']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (dialogTitleClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/DialogTitle/dialogTitleClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Divider/Divider.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/material/Divider/Divider.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _dividerClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./dividerClasses.js */ \"./node_modules/@mui/material/Divider/dividerClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    absolute,\n    children,\n    classes,\n    flexItem,\n    light,\n    orientation,\n    textAlign,\n    variant\n  } = ownerState;\n  const slots = {\n    root: ['root', absolute && 'absolute', variant, light && 'light', orientation === 'vertical' && 'vertical', flexItem && 'flexItem', children && 'withChildren', children && orientation === 'vertical' && 'withChildrenVertical', textAlign === 'right' && orientation !== 'vertical' && 'textAlignRight', textAlign === 'left' && orientation !== 'vertical' && 'textAlignLeft'],\n    wrapper: ['wrapper', orientation === 'vertical' && 'wrapperVertical']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _dividerClasses_js__WEBPACK_IMPORTED_MODULE_4__.getDividerUtilityClass, classes);\n};\nconst DividerRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiDivider',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.absolute && styles.absolute, styles[ownerState.variant], ownerState.light && styles.light, ownerState.orientation === 'vertical' && styles.vertical, ownerState.flexItem && styles.flexItem, ownerState.children && styles.withChildren, ownerState.children && ownerState.orientation === 'vertical' && styles.withChildrenVertical, ownerState.textAlign === 'right' && ownerState.orientation !== 'vertical' && styles.textAlignRight, ownerState.textAlign === 'left' && ownerState.orientation !== 'vertical' && styles.textAlignLeft];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(({\n  theme\n}) => ({\n  margin: 0,\n  // Reset browser default style.\n  flexShrink: 0,\n  borderWidth: 0,\n  borderStyle: 'solid',\n  borderColor: (theme.vars || theme).palette.divider,\n  borderBottomWidth: 'thin',\n  variants: [{\n    props: {\n      absolute: true\n    },\n    style: {\n      position: 'absolute',\n      bottom: 0,\n      left: 0,\n      width: '100%'\n    }\n  }, {\n    props: {\n      light: true\n    },\n    style: {\n      borderColor: theme.vars ? `rgba(${theme.vars.palette.dividerChannel} / 0.08)` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_7__.alpha)(theme.palette.divider, 0.08)\n    }\n  }, {\n    props: {\n      variant: 'inset'\n    },\n    style: {\n      marginLeft: 72\n    }\n  }, {\n    props: {\n      variant: 'middle',\n      orientation: 'horizontal'\n    },\n    style: {\n      marginLeft: theme.spacing(2),\n      marginRight: theme.spacing(2)\n    }\n  }, {\n    props: {\n      variant: 'middle',\n      orientation: 'vertical'\n    },\n    style: {\n      marginTop: theme.spacing(1),\n      marginBottom: theme.spacing(1)\n    }\n  }, {\n    props: {\n      orientation: 'vertical'\n    },\n    style: {\n      height: '100%',\n      borderBottomWidth: 0,\n      borderRightWidth: 'thin'\n    }\n  }, {\n    props: {\n      flexItem: true\n    },\n    style: {\n      alignSelf: 'stretch',\n      height: 'auto'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => !!ownerState.children,\n    style: {\n      display: 'flex',\n      textAlign: 'center',\n      border: 0,\n      borderTopStyle: 'solid',\n      borderLeftStyle: 'solid',\n      '&::before, &::after': {\n        content: '\"\"',\n        alignSelf: 'center'\n      }\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.children && ownerState.orientation !== 'vertical',\n    style: {\n      '&::before, &::after': {\n        width: '100%',\n        borderTop: `thin solid ${(theme.vars || theme).palette.divider}`,\n        borderTopStyle: 'inherit'\n      }\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.orientation === 'vertical' && ownerState.children,\n    style: {\n      flexDirection: 'column',\n      '&::before, &::after': {\n        height: '100%',\n        borderLeft: `thin solid ${(theme.vars || theme).palette.divider}`,\n        borderLeftStyle: 'inherit'\n      }\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.textAlign === 'right' && ownerState.orientation !== 'vertical',\n    style: {\n      '&::before': {\n        width: '90%'\n      },\n      '&::after': {\n        width: '10%'\n      }\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.textAlign === 'left' && ownerState.orientation !== 'vertical',\n    style: {\n      '&::before': {\n        width: '10%'\n      },\n      '&::after': {\n        width: '90%'\n      }\n    }\n  }]\n})));\nconst DividerWrapper = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('span', {\n  name: 'MuiDivider',\n  slot: 'Wrapper',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.wrapper, ownerState.orientation === 'vertical' && styles.wrapperVertical];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(({\n  theme\n}) => ({\n  display: 'inline-block',\n  paddingLeft: `calc(${theme.spacing(1)} * 1.2)`,\n  paddingRight: `calc(${theme.spacing(1)} * 1.2)`,\n  whiteSpace: 'nowrap',\n  variants: [{\n    props: {\n      orientation: 'vertical'\n    },\n    style: {\n      paddingTop: `calc(${theme.spacing(1)} * 1.2)`,\n      paddingBottom: `calc(${theme.spacing(1)} * 1.2)`\n    }\n  }]\n})));\nconst Divider = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Divider(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiDivider'\n  });\n  const {\n    absolute = false,\n    children,\n    className,\n    orientation = 'horizontal',\n    component = children || orientation === 'vertical' ? 'div' : 'hr',\n    flexItem = false,\n    light = false,\n    role = component !== 'hr' ? 'separator' : undefined,\n    textAlign = 'center',\n    variant = 'fullWidth',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    absolute,\n    component,\n    flexItem,\n    light,\n    orientation,\n    role,\n    textAlign,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(DividerRoot, {\n    as: component,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    role: role,\n    ref: ref,\n    ownerState: ownerState,\n    \"aria-orientation\": role === 'separator' && (component !== 'hr' || orientation === 'vertical') ? orientation : undefined,\n    ...other,\n    children: children ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(DividerWrapper, {\n      className: classes.wrapper,\n      ownerState: ownerState,\n      children: children\n    }) : null\n  });\n});\n\n/**\n * The following flag is used to ensure that this component isn't tabbable i.e.\n * does not get highlight/focus inside of MUI List.\n */\nif (Divider) {\n  Divider.muiSkipListHighlight = true;\n}\n true ? Divider.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Absolutely position the element.\n   * @default false\n   */\n  absolute: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType),\n  /**\n   * If `true`, a vertical divider will have the correct height when used in flex container.\n   * (By default, a vertical divider will have a calculated height of `0px` if it is the child of a flex container.)\n   * @default false\n   */\n  flexItem: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * If `true`, the divider will have a lighter color.\n   * @default false\n   * @deprecated Use <Divider sx={{ opacity: 0.6 }} /> (or any opacity or color) instead. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  light: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * The component orientation.\n   * @default 'horizontal'\n   */\n  orientation: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['horizontal', 'vertical']),\n  /**\n   * @ignore\n   */\n  role: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)]),\n  /**\n   * The text alignment.\n   * @default 'center'\n   */\n  textAlign: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['center', 'left', 'right']),\n  /**\n   * The variant to use.\n   * @default 'fullWidth'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['fullWidth', 'inset', 'middle']), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Divider);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Divider/Divider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Divider/dividerClasses.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/Divider/dividerClasses.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getDividerUtilityClass: () => (/* binding */ getDividerUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getDividerUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiDivider', slot);\n}\nconst dividerClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiDivider', ['root', 'absolute', 'fullWidth', 'inset', 'middle', 'flexItem', 'light', 'vertical', 'withChildren', 'withChildrenVertical', 'textAlignRight', 'textAlignLeft', 'wrapper', 'wrapperVertical']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (dividerClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Divider/dividerClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Fade/Fade.js":
/*!*************************************************!*\
  !*** ./node_modules/@mui/material/Fade/Fade.js ***!
  \*************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_7__);\n/* harmony import */ var react_transition_group__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! react-transition-group */ \"./node_modules/react-transition-group/esm/Transition.js\");\n/* harmony import */ var _mui_utils_elementAcceptingRef__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/elementAcceptingRef */ \"./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js\");\n/* harmony import */ var _mui_utils_getReactElementRef__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/getReactElementRef */ \"./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/useTheme.js\");\n/* harmony import */ var _transitions_utils_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../transitions/utils.js */ \"./node_modules/@mui/material/transitions/utils.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\nconst styles = {\n  entering: {\n    opacity: 1\n  },\n  entered: {\n    opacity: 1\n  }\n};\n\n/**\n * The Fade transition is used by the [Modal](/material-ui/react-modal/) component.\n * It uses [react-transition-group](https://github.com/reactjs/react-transition-group) internally.\n */\nconst Fade = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Fade(props, ref) {\n  const theme = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])();\n  const defaultTimeout = {\n    enter: theme.transitions.duration.enteringScreen,\n    exit: theme.transitions.duration.leavingScreen\n  };\n  const {\n    addEndListener,\n    appear = true,\n    children,\n    easing,\n    in: inProp,\n    onEnter,\n    onEntered,\n    onEntering,\n    onExit,\n    onExited,\n    onExiting,\n    style,\n    timeout = defaultTimeout,\n    // eslint-disable-next-line react/prop-types\n    TransitionComponent = react_transition_group__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n    ...other\n  } = props;\n  const enableStrictModeCompat = true;\n  const nodeRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const handleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(nodeRef, (0,_mui_utils_getReactElementRef__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(children), ref);\n  const normalizedTransitionCallback = callback => maybeIsAppearing => {\n    if (callback) {\n      const node = nodeRef.current;\n\n      // onEnterXxx and onExitXxx callbacks have a different arguments.length value.\n      if (maybeIsAppearing === undefined) {\n        callback(node);\n      } else {\n        callback(node, maybeIsAppearing);\n      }\n    }\n  };\n  const handleEntering = normalizedTransitionCallback(onEntering);\n  const handleEnter = normalizedTransitionCallback((node, isAppearing) => {\n    (0,_transitions_utils_js__WEBPACK_IMPORTED_MODULE_6__.reflow)(node); // So the animation always start from the start.\n\n    const transitionProps = (0,_transitions_utils_js__WEBPACK_IMPORTED_MODULE_6__.getTransitionProps)({\n      style,\n      timeout,\n      easing\n    }, {\n      mode: 'enter'\n    });\n    node.style.webkitTransition = theme.transitions.create('opacity', transitionProps);\n    node.style.transition = theme.transitions.create('opacity', transitionProps);\n    if (onEnter) {\n      onEnter(node, isAppearing);\n    }\n  });\n  const handleEntered = normalizedTransitionCallback(onEntered);\n  const handleExiting = normalizedTransitionCallback(onExiting);\n  const handleExit = normalizedTransitionCallback(node => {\n    const transitionProps = (0,_transitions_utils_js__WEBPACK_IMPORTED_MODULE_6__.getTransitionProps)({\n      style,\n      timeout,\n      easing\n    }, {\n      mode: 'exit'\n    });\n    node.style.webkitTransition = theme.transitions.create('opacity', transitionProps);\n    node.style.transition = theme.transitions.create('opacity', transitionProps);\n    if (onExit) {\n      onExit(node);\n    }\n  });\n  const handleExited = normalizedTransitionCallback(onExited);\n  const handleAddEndListener = next => {\n    if (addEndListener) {\n      // Old call signature before `react-transition-group` implemented `nodeRef`\n      addEndListener(nodeRef.current, next);\n    }\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(TransitionComponent, {\n    appear: appear,\n    in: inProp,\n    nodeRef: enableStrictModeCompat ? nodeRef : undefined,\n    onEnter: handleEnter,\n    onEntered: handleEntered,\n    onEntering: handleEntering,\n    onExit: handleExit,\n    onExited: handleExited,\n    onExiting: handleExiting,\n    addEndListener: handleAddEndListener,\n    timeout: timeout,\n    ...other,\n    children: (state, {\n      ownerState,\n      ...restChildProps\n    }) => {\n      return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(children, {\n        style: {\n          opacity: 0,\n          visibility: state === 'exited' && !inProp ? 'hidden' : undefined,\n          ...styles[state],\n          ...style,\n          ...children.props.style\n        },\n        ref: handleRef,\n        ...restChildProps\n      });\n    }\n  });\n});\n true ? Fade.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Add a custom transition end trigger. Called with the transitioning DOM\n   * node and a done callback. Allows for more fine grained transition end\n   * logic. Note: Timeouts are still used as a fallback if provided.\n   */\n  addEndListener: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * Perform the enter transition when it first mounts if `in` is also `true`.\n   * Set this to `false` to disable this behavior.\n   * @default true\n   */\n  appear: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n  /**\n   * A single child content element.\n   */\n  children: _mui_utils_elementAcceptingRef__WEBPACK_IMPORTED_MODULE_8__[\"default\"].isRequired,\n  /**\n   * The transition timing function.\n   * You may specify a single easing or a object containing enter and exit values.\n   */\n  easing: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_7___default().shape({\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string)\n  }), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string)]),\n  /**\n   * If `true`, the component will transition in.\n   */\n  in: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n  /**\n   * @ignore\n   */\n  onEnter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  onEntered: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  onEntering: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  onExit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  onExited: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  onExiting: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object),\n  /**\n   * The duration for the transition, in milliseconds.\n   * You may specify a single timeout for all transitions, or individually with an object.\n   * @default {\n   *   enter: theme.transitions.duration.enteringScreen,\n   *   exit: theme.transitions.duration.leavingScreen,\n   * }\n   */\n  timeout: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_7___default().number), prop_types__WEBPACK_IMPORTED_MODULE_7___default().shape({\n    appear: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().number),\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().number),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().number)\n  })])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Fade);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Fade/Fade.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FilledInput/FilledInput.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/FilledInput/FilledInput.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_12___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_12__);\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../InputBase/InputBase.js */ \"./node_modules/@mui/material/InputBase/InputBase.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./filledInputClasses.js */ \"./node_modules/@mui/material/FilledInput/filledInputClasses.js\");\n/* harmony import */ var _utils_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/index.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    disableUnderline,\n    startAdornment,\n    endAdornment,\n    size,\n    hiddenLabel,\n    multiline\n  } = ownerState;\n  const slots = {\n    root: ['root', !disableUnderline && 'underline', startAdornment && 'adornedStart', endAdornment && 'adornedEnd', size === 'small' && `size${(0,_utils_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(size)}`, hiddenLabel && 'hiddenLabel', multiline && 'multiline'],\n    input: ['input']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__.getFilledInputUtilityClass, classes);\n  return {\n    ...classes,\n    // forward classes to the InputBase\n    ...composedClasses\n  };\n};\nconst FilledInputRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_6__.InputBaseRoot, {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiFilledInput',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [...(0,_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_6__.rootOverridesResolver)(props, styles), !ownerState.disableUnderline && styles.underline];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => {\n  const light = theme.palette.mode === 'light';\n  const bottomLineColor = light ? 'rgba(0, 0, 0, 0.42)' : 'rgba(255, 255, 255, 0.7)';\n  const backgroundColor = light ? 'rgba(0, 0, 0, 0.06)' : 'rgba(255, 255, 255, 0.09)';\n  const hoverBackground = light ? 'rgba(0, 0, 0, 0.09)' : 'rgba(255, 255, 255, 0.13)';\n  const disabledBackground = light ? 'rgba(0, 0, 0, 0.12)' : 'rgba(255, 255, 255, 0.12)';\n  return {\n    position: 'relative',\n    backgroundColor: theme.vars ? theme.vars.palette.FilledInput.bg : backgroundColor,\n    borderTopLeftRadius: (theme.vars || theme).shape.borderRadius,\n    borderTopRightRadius: (theme.vars || theme).shape.borderRadius,\n    transition: theme.transitions.create('background-color', {\n      duration: theme.transitions.duration.shorter,\n      easing: theme.transitions.easing.easeOut\n    }),\n    '&:hover': {\n      backgroundColor: theme.vars ? theme.vars.palette.FilledInput.hoverBg : hoverBackground,\n      // Reset on touch devices, it doesn't add specificity\n      '@media (hover: none)': {\n        backgroundColor: theme.vars ? theme.vars.palette.FilledInput.bg : backgroundColor\n      }\n    },\n    [`&.${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].focused}`]: {\n      backgroundColor: theme.vars ? theme.vars.palette.FilledInput.bg : backgroundColor\n    },\n    [`&.${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].disabled}`]: {\n      backgroundColor: theme.vars ? theme.vars.palette.FilledInput.disabledBg : disabledBackground\n    },\n    variants: [{\n      props: ({\n        ownerState\n      }) => !ownerState.disableUnderline,\n      style: {\n        '&::after': {\n          left: 0,\n          bottom: 0,\n          content: '\"\"',\n          position: 'absolute',\n          right: 0,\n          transform: 'scaleX(0)',\n          transition: theme.transitions.create('transform', {\n            duration: theme.transitions.duration.shorter,\n            easing: theme.transitions.easing.easeOut\n          }),\n          pointerEvents: 'none' // Transparent to the hover style.\n        },\n        [`&.${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].focused}:after`]: {\n          // translateX(0) is a workaround for Safari transform scale bug\n          // See https://github.com/mui/material-ui/issues/31766\n          transform: 'scaleX(1) translateX(0)'\n        },\n        [`&.${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].error}`]: {\n          '&::before, &::after': {\n            borderBottomColor: (theme.vars || theme).palette.error.main\n          }\n        },\n        '&::before': {\n          borderBottom: `1px solid ${theme.vars ? `rgba(${theme.vars.palette.common.onBackgroundChannel} / ${theme.vars.opacity.inputUnderline})` : bottomLineColor}`,\n          left: 0,\n          bottom: 0,\n          content: '\"\\\\00a0\"',\n          position: 'absolute',\n          right: 0,\n          transition: theme.transitions.create('border-bottom-color', {\n            duration: theme.transitions.duration.shorter\n          }),\n          pointerEvents: 'none' // Transparent to the hover style.\n        },\n        [`&:hover:not(.${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].disabled}, .${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].error}):before`]: {\n          borderBottom: `1px solid ${(theme.vars || theme).palette.text.primary}`\n        },\n        [`&.${_filledInputClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].disabled}:before`]: {\n          borderBottomStyle: 'dotted'\n        }\n      }\n    }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])()) // check all the used fields in the style below\n    .map(([color]) => ({\n      props: {\n        disableUnderline: false,\n        color\n      },\n      style: {\n        '&::after': {\n          borderBottom: `2px solid ${(theme.vars || theme).palette[color]?.main}`\n        }\n      }\n    })), {\n      props: ({\n        ownerState\n      }) => ownerState.startAdornment,\n      style: {\n        paddingLeft: 12\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.endAdornment,\n      style: {\n        paddingRight: 12\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.multiline,\n      style: {\n        padding: '25px 12px 8px'\n      }\n    }, {\n      props: ({\n        ownerState,\n        size\n      }) => ownerState.multiline && size === 'small',\n      style: {\n        paddingTop: 21,\n        paddingBottom: 4\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.multiline && ownerState.hiddenLabel,\n      style: {\n        paddingTop: 16,\n        paddingBottom: 17\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.multiline && ownerState.hiddenLabel && ownerState.size === 'small',\n      style: {\n        paddingTop: 8,\n        paddingBottom: 9\n      }\n    }]\n  };\n}));\nconst FilledInputInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_6__.InputBaseInput, {\n  name: 'MuiFilledInput',\n  slot: 'Input',\n  overridesResolver: _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_6__.inputOverridesResolver\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  paddingTop: 25,\n  paddingRight: 12,\n  paddingBottom: 8,\n  paddingLeft: 12,\n  ...(!theme.vars && {\n    '&:-webkit-autofill': {\n      WebkitBoxShadow: theme.palette.mode === 'light' ? null : '0 0 0 100px #266798 inset',\n      WebkitTextFillColor: theme.palette.mode === 'light' ? null : '#fff',\n      caretColor: theme.palette.mode === 'light' ? null : '#fff',\n      borderTopLeftRadius: 'inherit',\n      borderTopRightRadius: 'inherit'\n    }\n  }),\n  ...(theme.vars && {\n    '&:-webkit-autofill': {\n      borderTopLeftRadius: 'inherit',\n      borderTopRightRadius: 'inherit'\n    },\n    [theme.getColorSchemeSelector('dark')]: {\n      '&:-webkit-autofill': {\n        WebkitBoxShadow: '0 0 0 100px #266798 inset',\n        WebkitTextFillColor: '#fff',\n        caretColor: '#fff'\n      }\n    }\n  }),\n  variants: [{\n    props: {\n      size: 'small'\n    },\n    style: {\n      paddingTop: 21,\n      paddingBottom: 4\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.hiddenLabel,\n    style: {\n      paddingTop: 16,\n      paddingBottom: 17\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.startAdornment,\n    style: {\n      paddingLeft: 0\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.endAdornment,\n    style: {\n      paddingRight: 0\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.hiddenLabel && ownerState.size === 'small',\n    style: {\n      paddingTop: 8,\n      paddingBottom: 9\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.multiline,\n    style: {\n      paddingTop: 0,\n      paddingBottom: 0,\n      paddingLeft: 0,\n      paddingRight: 0\n    }\n  }]\n})));\nconst FilledInput = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function FilledInput(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiFilledInput'\n  });\n  const {\n    disableUnderline = false,\n    components = {},\n    componentsProps: componentsPropsProp,\n    fullWidth = false,\n    hiddenLabel,\n    // declare here to prevent spreading to DOM\n    inputComponent = 'input',\n    multiline = false,\n    slotProps,\n    slots = {},\n    type = 'text',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    disableUnderline,\n    fullWidth,\n    inputComponent,\n    multiline,\n    type\n  };\n  const classes = useUtilityClasses(props);\n  const filledInputComponentsProps = {\n    root: {\n      ownerState\n    },\n    input: {\n      ownerState\n    }\n  };\n  const componentsProps = slotProps ?? componentsPropsProp ? (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_11__[\"default\"])(filledInputComponentsProps, slotProps ?? componentsPropsProp) : filledInputComponentsProps;\n  const RootSlot = slots.root ?? components.Root ?? FilledInputRoot;\n  const InputSlot = slots.input ?? components.Input ?? FilledInputInput;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n    slots: {\n      root: RootSlot,\n      input: InputSlot\n    },\n    slotProps: componentsProps,\n    fullWidth: fullWidth,\n    inputComponent: inputComponent,\n    multiline: multiline,\n    ref: ref,\n    type: type,\n    ...other,\n    classes: classes\n  });\n});\n true ? FilledInput.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * This prop helps users to fill forms faster, especially on mobile devices.\n   * The name can be confusing, as it's more like an autofill.\n   * You can learn more about it [following the specification](https://html.spec.whatwg.org/multipage/form-control-infrastructure.html#autofill).\n   */\n  autoComplete: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string),\n  /**\n   * If `true`, the `input` element is focused during the first mount.\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * The prop defaults to the value (`'primary'`) inherited from the parent FormControl component.\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOf(['primary', 'secondary']), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)]),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_12___default().shape({\n    Input: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().elementType),\n    Root: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().elementType)\n  }),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * @deprecated use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_12___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object)\n  }),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().any),\n  /**\n   * If `true`, the component is disabled.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the input will not have an underline.\n   * @default false\n   */\n  disableUnderline: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * End `InputAdornment` for this component.\n   */\n  endAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().node),\n  /**\n   * If `true`, the `input` will indicate an error.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the `input` will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the label is hidden.\n   * This is used to increase density for a `FilledInput`.\n   * Be sure to add `aria-label` to the `input` element.\n   * @default false\n   */\n  hiddenLabel: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * The id of the `input` element.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string),\n  /**\n   * The component used for the `input` element.\n   * Either a string to use a HTML element or a component.\n   * @default 'input'\n   */\n  inputComponent: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().elementType),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * @default {}\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_13__[\"default\"],\n  /**\n   * If `dense`, will adjust vertical spacing. This is normally obtained via context from\n   * FormControl.\n   * The prop defaults to the value (`'none'`) inherited from the parent FormControl component.\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOf(['dense', 'none']),\n  /**\n   * Maximum number of rows to display when multiline option is set to true.\n   */\n  maxRows: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)]),\n  /**\n   * Minimum number of rows to display when multiline option is set to true.\n   */\n  minRows: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)]),\n  /**\n   * If `true`, a [TextareaAutosize](https://mui.com/material-ui/react-textarea-autosize/) element is rendered.\n   * @default false\n   */\n  multiline: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * Name attribute of the `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string),\n  /**\n   * Callback fired when the value is changed.\n   *\n   * @param {React.ChangeEvent<HTMLTextAreaElement | HTMLInputElement>} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().func),\n  /**\n   * The short hint displayed in the `input` before the user enters a value.\n   */\n  placeholder: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string),\n  /**\n   * It prevents the user from changing the value of the field\n   * (not from interacting with the field).\n   */\n  readOnly: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the `input` element is required.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * Number of rows to display when multiline option is set to true.\n   */\n  rows: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)]),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * This prop is an alias for the `componentsProps` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_12___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object)\n  }),\n  /**\n   * The components used for each slot inside.\n   *\n   * This prop is an alias for the `components` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_12___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().elementType)\n  }),\n  /**\n   * Start `InputAdornment` for this component.\n   */\n  startAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().node),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object)]),\n  /**\n   * Type of the `input` element. It should be [a valid HTML5 input type](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Form_%3Cinput%3E_types).\n   * @default 'text'\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string),\n  /**\n   * The value of the `input` element, required for a controlled component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().any)\n} : 0;\nFilledInput.muiName = 'Input';\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FilledInput);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FilledInput/FilledInput.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FilledInput/filledInputClasses.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/FilledInput/filledInputClasses.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getFilledInputUtilityClass: () => (/* binding */ getFilledInputUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n/* harmony import */ var _InputBase_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../InputBase/index.js */ \"./node_modules/@mui/material/InputBase/inputBaseClasses.js\");\n\n\n\nfunction getFilledInputUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiFilledInput', slot);\n}\nconst filledInputClasses = {\n  ..._InputBase_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"],\n  ...(0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_2__[\"default\"])('MuiFilledInput', ['root', 'underline', 'input', 'adornedStart', 'adornedEnd', 'sizeSmall', 'multiline', 'hiddenLabel'])\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (filledInputClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FilledInput/filledInputClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControl/FormControl.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/FormControl/FormControl.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _InputBase_utils_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../InputBase/utils.js */ \"./node_modules/@mui/material/InputBase/utils.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_isMuiElement_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/isMuiElement.js */ \"./node_modules/@mui/material/utils/isMuiElement.js\");\n/* harmony import */ var _FormControlContext_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./FormControlContext.js */ \"./node_modules/@mui/material/FormControl/FormControlContext.js\");\n/* harmony import */ var _formControlClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./formControlClasses.js */ \"./node_modules/@mui/material/FormControl/formControlClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    margin,\n    fullWidth\n  } = ownerState;\n  const slots = {\n    root: ['root', margin !== 'none' && `margin${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(margin)}`, fullWidth && 'fullWidth']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _formControlClasses_js__WEBPACK_IMPORTED_MODULE_5__.getFormControlUtilityClasses, classes);\n};\nconst FormControlRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('div', {\n  name: 'MuiFormControl',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[`margin${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.margin)}`], ownerState.fullWidth && styles.fullWidth];\n  }\n})({\n  display: 'inline-flex',\n  flexDirection: 'column',\n  position: 'relative',\n  // Reset fieldset default style.\n  minWidth: 0,\n  padding: 0,\n  margin: 0,\n  border: 0,\n  verticalAlign: 'top',\n  // Fix alignment issue on Safari.\n  variants: [{\n    props: {\n      margin: 'normal'\n    },\n    style: {\n      marginTop: 16,\n      marginBottom: 8\n    }\n  }, {\n    props: {\n      margin: 'dense'\n    },\n    style: {\n      marginTop: 8,\n      marginBottom: 4\n    }\n  }, {\n    props: {\n      fullWidth: true\n    },\n    style: {\n      width: '100%'\n    }\n  }]\n});\n\n/**\n * Provides context such as filled/focused/error/required for form inputs.\n * Relying on the context provides high flexibility and ensures that the state always stays\n * consistent across the children of the `FormControl`.\n * This context is used by the following components:\n *\n *  - FormLabel\n *  - FormHelperText\n *  - Input\n *  - InputLabel\n *\n * You can find one composition example below and more going to [the demos](/material-ui/react-text-field/#components).\n *\n * ```jsx\n * <FormControl>\n *   <InputLabel htmlFor=\"my-input\">Email address</InputLabel>\n *   <Input id=\"my-input\" aria-describedby=\"my-helper-text\" />\n *   <FormHelperText id=\"my-helper-text\">We'll never share your email.</FormHelperText>\n * </FormControl>\n * ```\n *\n * ⚠️ Only one `InputBase` can be used within a FormControl because it creates visual inconsistencies.\n * For instance, only one input can be focused at the same time, the state shouldn't be shared.\n */\nconst FormControl = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function FormControl(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiFormControl'\n  });\n  const {\n    children,\n    className,\n    color = 'primary',\n    component = 'div',\n    disabled = false,\n    error = false,\n    focused: visuallyFocused,\n    fullWidth = false,\n    hiddenLabel = false,\n    margin = 'none',\n    required = false,\n    size = 'medium',\n    variant = 'outlined',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    color,\n    component,\n    disabled,\n    error,\n    fullWidth,\n    hiddenLabel,\n    margin,\n    required,\n    size,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  const [adornedStart, setAdornedStart] = react__WEBPACK_IMPORTED_MODULE_0__.useState(() => {\n    // We need to iterate through the children and find the Input in order\n    // to fully support server-side rendering.\n    let initialAdornedStart = false;\n    if (children) {\n      react__WEBPACK_IMPORTED_MODULE_0__.Children.forEach(children, child => {\n        if (!(0,_utils_isMuiElement_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(child, ['Input', 'Select'])) {\n          return;\n        }\n        const input = (0,_utils_isMuiElement_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(child, ['Select']) ? child.props.input : child;\n        if (input && (0,_InputBase_utils_js__WEBPACK_IMPORTED_MODULE_9__.isAdornedStart)(input.props)) {\n          initialAdornedStart = true;\n        }\n      });\n    }\n    return initialAdornedStart;\n  });\n  const [filled, setFilled] = react__WEBPACK_IMPORTED_MODULE_0__.useState(() => {\n    // We need to iterate through the children and find the Input in order\n    // to fully support server-side rendering.\n    let initialFilled = false;\n    if (children) {\n      react__WEBPACK_IMPORTED_MODULE_0__.Children.forEach(children, child => {\n        if (!(0,_utils_isMuiElement_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(child, ['Input', 'Select'])) {\n          return;\n        }\n        if ((0,_InputBase_utils_js__WEBPACK_IMPORTED_MODULE_9__.isFilled)(child.props, true) || (0,_InputBase_utils_js__WEBPACK_IMPORTED_MODULE_9__.isFilled)(child.props.inputProps, true)) {\n          initialFilled = true;\n        }\n      });\n    }\n    return initialFilled;\n  });\n  const [focusedState, setFocused] = react__WEBPACK_IMPORTED_MODULE_0__.useState(false);\n  if (disabled && focusedState) {\n    setFocused(false);\n  }\n  const focused = visuallyFocused !== undefined && !disabled ? visuallyFocused : focusedState;\n  let registerEffect;\n  const registeredInput = react__WEBPACK_IMPORTED_MODULE_0__.useRef(false);\n  if (true) {\n    registerEffect = () => {\n      if (registeredInput.current) {\n        console.error(['MUI: There are multiple `InputBase` components inside a FormControl.', 'This creates visual inconsistencies, only use one `InputBase`.'].join('\\n'));\n      }\n      registeredInput.current = true;\n      return () => {\n        registeredInput.current = false;\n      };\n    };\n  }\n  const onFilled = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    setFilled(true);\n  }, []);\n  const onEmpty = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    setFilled(false);\n  }, []);\n  const childContext = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    return {\n      adornedStart,\n      setAdornedStart,\n      color,\n      disabled,\n      error,\n      filled,\n      focused,\n      fullWidth,\n      hiddenLabel,\n      size,\n      onBlur: () => {\n        setFocused(false);\n      },\n      onFocus: () => {\n        setFocused(true);\n      },\n      onEmpty,\n      onFilled,\n      registerEffect,\n      required,\n      variant\n    };\n  }, [adornedStart, color, disabled, error, filled, focused, fullWidth, hiddenLabel, registerEffect, onEmpty, onFilled, required, size, variant]);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_FormControlContext_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"].Provider, {\n    value: childContext,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(FormControlRoot, {\n      as: component,\n      ownerState: ownerState,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n      ref: ref,\n      ...other,\n      children: children\n    })\n  });\n});\n true ? FormControl.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'primary'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n  /**\n   * If `true`, the label, input and helper text should be displayed in a disabled state.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the label is displayed in an error state.\n   * @default false\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the component is displayed in focused state.\n   */\n  focused: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the component will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the label is hidden.\n   * This is used to increase density for a `FilledInput`.\n   * Be sure to add `aria-label` to the `input` element.\n   * @default false\n   */\n  hiddenLabel: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `dense` or `normal`, will adjust vertical spacing of this and contained components.\n   * @default 'none'\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['dense', 'none', 'normal']),\n  /**\n   * If `true`, the label will indicate that the `input` is required.\n   * @default false\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * The size of the component.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['medium', 'small']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n  /**\n   * The variant to use.\n   * @default 'outlined'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['filled', 'outlined', 'standard'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FormControl);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControl/FormControl.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControl/FormControlContext.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/FormControl/FormControlContext.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n/**\n * @ignore - internal component.\n */\nconst FormControlContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext(undefined);\nif (true) {\n  FormControlContext.displayName = 'FormControlContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FormControlContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControl/FormControlContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControl/formControlClasses.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/FormControl/formControlClasses.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getFormControlUtilityClasses: () => (/* binding */ getFormControlUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getFormControlUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiFormControl', slot);\n}\nconst formControlClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiFormControl', ['root', 'marginNone', 'marginNormal', 'marginDense', 'fullWidth', 'disabled']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (formControlClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControl/formControlClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControl/formControlState.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/FormControl/formControlState.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ formControlState)\n/* harmony export */ });\nfunction formControlState({\n  props,\n  states,\n  muiFormControl\n}) {\n  return states.reduce((acc, state) => {\n    acc[state] = props[state];\n    if (muiFormControl) {\n      if (typeof props[state] === 'undefined') {\n        acc[state] = muiFormControl[state];\n      }\n    }\n    return acc;\n  }, {});\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControl/formControlState.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControl/useFormControl.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/FormControl/useFormControl.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useFormControl)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _FormControlContext_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./FormControlContext.js */ \"./node_modules/@mui/material/FormControl/FormControlContext.js\");\n'use client';\n\n\n\nfunction useFormControl() {\n  return react__WEBPACK_IMPORTED_MODULE_0__.useContext(_FormControlContext_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"]);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControl/useFormControl.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControlLabel/FormControlLabel.js":
/*!*************************************************************************!*\
  !*** ./node_modules/@mui/material/FormControlLabel/FormControlLabel.js ***!
  \*************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   FormControlLabelRoot: () => (/* binding */ FormControlLabelRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_13__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _FormControl_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../FormControl/index.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _Typography_index_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../Typography/index.js */ \"./node_modules/@mui/material/Typography/Typography.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./formControlLabelClasses.js */ \"./node_modules/@mui/material/FormControlLabel/formControlLabelClasses.js\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    disabled,\n    labelPlacement,\n    error,\n    required\n  } = ownerState;\n  const slots = {\n    root: ['root', disabled && 'disabled', `labelPlacement${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(labelPlacement)}`, error && 'error', required && 'required'],\n    label: ['label', disabled && 'disabled'],\n    asterisk: ['asterisk', error && 'error']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__.getFormControlLabelUtilityClasses, classes);\n};\nconst FormControlLabelRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('label', {\n  name: 'MuiFormControlLabel',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [{\n      [`& .${_formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].label}`]: styles.label\n    }, styles.root, styles[`labelPlacement${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.labelPlacement)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  display: 'inline-flex',\n  alignItems: 'center',\n  cursor: 'pointer',\n  // For correct alignment with the text.\n  verticalAlign: 'middle',\n  WebkitTapHighlightColor: 'transparent',\n  marginLeft: -11,\n  marginRight: 16,\n  // used for row presentation of radio/checkbox\n  [`&.${_formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    cursor: 'default'\n  },\n  [`& .${_formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].label}`]: {\n    [`&.${_formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n      color: (theme.vars || theme).palette.text.disabled\n    }\n  },\n  variants: [{\n    props: {\n      labelPlacement: 'start'\n    },\n    style: {\n      flexDirection: 'row-reverse',\n      marginRight: -11\n    }\n  }, {\n    props: {\n      labelPlacement: 'top'\n    },\n    style: {\n      flexDirection: 'column-reverse'\n    }\n  }, {\n    props: {\n      labelPlacement: 'bottom'\n    },\n    style: {\n      flexDirection: 'column'\n    }\n  }, {\n    props: ({\n      labelPlacement\n    }) => labelPlacement === 'start' || labelPlacement === 'top' || labelPlacement === 'bottom',\n    style: {\n      marginLeft: 16 // used for row presentation of radio/checkbox\n    }\n  }]\n})));\nconst AsteriskComponent = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiFormControlLabel',\n  slot: 'Asterisk',\n  overridesResolver: (props, styles) => styles.asterisk\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  [`&.${_formControlLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].error}`]: {\n    color: (theme.vars || theme).palette.error.main\n  }\n})));\n\n/**\n * Drop-in replacement of the `Radio`, `Switch` and `Checkbox` component.\n * Use this component if you want to display an extra label.\n */\nconst FormControlLabel = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function FormControlLabel(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiFormControlLabel'\n  });\n  const {\n    checked,\n    className,\n    componentsProps = {},\n    control,\n    disabled: disabledProp,\n    disableTypography,\n    inputRef,\n    label: labelProp,\n    labelPlacement = 'end',\n    name,\n    onChange,\n    required: requiredProp,\n    slots = {},\n    slotProps = {},\n    value,\n    ...other\n  } = props;\n  const muiFormControl = (0,_FormControl_index_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])();\n  const disabled = disabledProp ?? control.props.disabled ?? muiFormControl?.disabled;\n  const required = requiredProp ?? control.props.required;\n  const controlProps = {\n    disabled,\n    required\n  };\n  ['checked', 'name', 'onChange', 'value', 'inputRef'].forEach(key => {\n    if (typeof control.props[key] === 'undefined' && typeof props[key] !== 'undefined') {\n      controlProps[key] = props[key];\n    }\n  });\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['error']\n  });\n  const ownerState = {\n    ...props,\n    disabled,\n    labelPlacement,\n    required,\n    error: fcs.error\n  };\n  const classes = useUtilityClasses(ownerState);\n  const externalForwardedProps = {\n    slots,\n    slotProps: {\n      ...componentsProps,\n      ...slotProps\n    }\n  };\n  const [TypographySlot, typographySlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])('typography', {\n    elementType: _Typography_index_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  let label = labelProp;\n  if (label != null && label.type !== _Typography_index_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"] && !disableTypography) {\n    label = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TypographySlot, {\n      component: \"span\",\n      ...typographySlotProps,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.label, typographySlotProps?.className),\n      children: label\n    });\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(FormControlLabelRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ownerState: ownerState,\n    ref: ref,\n    ...other,\n    children: [/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(control, controlProps), required ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(\"div\", {\n      children: [label, /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(AsteriskComponent, {\n        ownerState: ownerState,\n        \"aria-hidden\": true,\n        className: classes.asterisk,\n        children: [\"\\u2009\", '*']\n      })]\n    }) : label]\n  });\n});\n true ? FormControlLabel.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * If `true`, the component appears selected.\n   */\n  checked: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   * @deprecated use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    typography: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)\n  }),\n  /**\n   * A control element. For instance, it can be a `Radio`, a `Switch` or a `Checkbox`.\n   */\n  control: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().element).isRequired,\n  /**\n   * If `true`, the control is disabled.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the label is rendered as it is passed without an additional typography node.\n   */\n  disableTypography: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_14__[\"default\"],\n  /**\n   * A text or an element to be used in an enclosing label element.\n   */\n  label: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * The position of the label.\n   * @default 'end'\n   */\n  labelPlacement: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['bottom', 'end', 'start', 'top']),\n  /**\n   * @ignore\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * Callback fired when the state is changed.\n   *\n   * @param {React.SyntheticEvent} event The event source of the callback.\n   * You can pull out the new checked state by accessing `event.target.checked` (boolean).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * If `true`, the label will indicate that the `input` is required.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    typography: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    typography: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n  /**\n   * The value of the component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().any)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FormControlLabel);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControlLabel/FormControlLabel.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormControlLabel/formControlLabelClasses.js":
/*!********************************************************************************!*\
  !*** ./node_modules/@mui/material/FormControlLabel/formControlLabelClasses.js ***!
  \********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getFormControlLabelUtilityClasses: () => (/* binding */ getFormControlLabelUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getFormControlLabelUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiFormControlLabel', slot);\n}\nconst formControlLabelClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiFormControlLabel', ['root', 'labelPlacementStart', 'labelPlacementTop', 'labelPlacementBottom', 'disabled', 'label', 'error', 'required', 'asterisk']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (formControlLabelClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormControlLabel/formControlLabelClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormHelperText/FormHelperText.js":
/*!*********************************************************************!*\
  !*** ./node_modules/@mui/material/FormHelperText/FormHelperText.js ***!
  \*********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _formHelperTextClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./formHelperTextClasses.js */ \"./node_modules/@mui/material/FormHelperText/formHelperTextClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\nvar _span;\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    contained,\n    size,\n    disabled,\n    error,\n    filled,\n    focused,\n    required\n  } = ownerState;\n  const slots = {\n    root: ['root', disabled && 'disabled', error && 'error', size && `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`, contained && 'contained', focused && 'focused', filled && 'filled', required && 'required']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _formHelperTextClasses_js__WEBPACK_IMPORTED_MODULE_5__.getFormHelperTextUtilityClasses, classes);\n};\nconst FormHelperTextRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('p', {\n  name: 'MuiFormHelperText',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.size && styles[`size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`], ownerState.contained && styles.contained, ownerState.filled && styles.filled];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  color: (theme.vars || theme).palette.text.secondary,\n  ...theme.typography.caption,\n  textAlign: 'left',\n  marginTop: 3,\n  marginRight: 0,\n  marginBottom: 0,\n  marginLeft: 0,\n  [`&.${_formHelperTextClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    color: (theme.vars || theme).palette.text.disabled\n  },\n  [`&.${_formHelperTextClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].error}`]: {\n    color: (theme.vars || theme).palette.error.main\n  },\n  variants: [{\n    props: {\n      size: 'small'\n    },\n    style: {\n      marginTop: 4\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.contained,\n    style: {\n      marginLeft: 14,\n      marginRight: 14\n    }\n  }]\n})));\nconst FormHelperText = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function FormHelperText(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiFormHelperText'\n  });\n  const {\n    children,\n    className,\n    component = 'p',\n    disabled,\n    error,\n    filled,\n    focused,\n    margin,\n    required,\n    variant,\n    ...other\n  } = props;\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])();\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['variant', 'size', 'disabled', 'error', 'filled', 'focused', 'required']\n  });\n  const ownerState = {\n    ...props,\n    component,\n    contained: fcs.variant === 'filled' || fcs.variant === 'outlined',\n    variant: fcs.variant,\n    size: fcs.size,\n    disabled: fcs.disabled,\n    error: fcs.error,\n    filled: fcs.filled,\n    focused: fcs.focused,\n    required: fcs.required\n  };\n\n  // This issue explains why this is required: https://github.com/mui/material-ui/issues/42184\n  delete ownerState.ownerState;\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(FormHelperTextRoot, {\n    as: component,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ref: ref,\n    ...other,\n    ownerState: ownerState,\n    children: children === ' ' ? // notranslate needed while Google Translate will not fix zero-width space issue\n    _span || (_span = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"span\", {\n      className: \"notranslate\",\n      \"aria-hidden\": true,\n      children: \"\\u200B\"\n    })) : children\n  });\n});\n true ? FormHelperText.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   *\n   * If `' '` is provided, the component reserves one line height for displaying a future message.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n  /**\n   * If `true`, the helper text should be displayed in a disabled state.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, helper text should be displayed in an error state.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the helper text should use filled classes key.\n   */\n  filled: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the helper text should use focused classes key.\n   */\n  focused: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `dense`, will adjust vertical spacing. This is normally obtained via context from\n   * FormControl.\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['dense']),\n  /**\n   * If `true`, the helper text should use required classes key.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n  /**\n   * The variant to use.\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['filled', 'outlined', 'standard']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FormHelperText);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormHelperText/FormHelperText.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormHelperText/formHelperTextClasses.js":
/*!****************************************************************************!*\
  !*** ./node_modules/@mui/material/FormHelperText/formHelperTextClasses.js ***!
  \****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getFormHelperTextUtilityClasses: () => (/* binding */ getFormHelperTextUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getFormHelperTextUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiFormHelperText', slot);\n}\nconst formHelperTextClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiFormHelperText', ['root', 'error', 'disabled', 'sizeSmall', 'sizeMedium', 'contained', 'focused', 'filled', 'required']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (formHelperTextClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormHelperText/formHelperTextClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormLabel/FormLabel.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/FormLabel/FormLabel.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   FormLabelRoot: () => (/* binding */ FormLabelRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_12___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_12__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _formLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./formLabelClasses.js */ \"./node_modules/@mui/material/FormLabel/formLabelClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    color,\n    focused,\n    disabled,\n    error,\n    filled,\n    required\n  } = ownerState;\n  const slots = {\n    root: ['root', `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, disabled && 'disabled', error && 'error', filled && 'filled', focused && 'focused', required && 'required'],\n    asterisk: ['asterisk', error && 'error']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _formLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__.getFormLabelUtilityClasses, classes);\n};\nconst FormLabelRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('label', {\n  name: 'MuiFormLabel',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.color === 'secondary' && styles.colorSecondary, ownerState.filled && styles.filled];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  color: (theme.vars || theme).palette.text.secondary,\n  ...theme.typography.body1,\n  lineHeight: '1.4375em',\n  padding: 0,\n  position: 'relative',\n  variants: [...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])()).map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      [`&.${_formLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].focused}`]: {\n        color: (theme.vars || theme).palette[color].main\n      }\n    }\n  })), {\n    props: {},\n    style: {\n      [`&.${_formLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n        color: (theme.vars || theme).palette.text.disabled\n      },\n      [`&.${_formLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].error}`]: {\n        color: (theme.vars || theme).palette.error.main\n      }\n    }\n  }]\n})));\nconst AsteriskComponent = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiFormLabel',\n  slot: 'Asterisk',\n  overridesResolver: (props, styles) => styles.asterisk\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  [`&.${_formLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].error}`]: {\n    color: (theme.vars || theme).palette.error.main\n  }\n})));\nconst FormLabel = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function FormLabel(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiFormLabel'\n  });\n  const {\n    children,\n    className,\n    color,\n    component = 'label',\n    disabled,\n    error,\n    filled,\n    focused,\n    required,\n    ...other\n  } = props;\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])();\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['color', 'required', 'focused', 'disabled', 'error', 'filled']\n  });\n  const ownerState = {\n    ...props,\n    color: fcs.color || 'primary',\n    component,\n    disabled: fcs.disabled,\n    error: fcs.error,\n    filled: fcs.filled,\n    focused: fcs.focused,\n    required: fcs.required\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(FormLabelRoot, {\n    as: component,\n    ownerState: ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ref: ref,\n    ...other,\n    children: [children, fcs.required && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(AsteriskComponent, {\n      ownerState: ownerState,\n      \"aria-hidden\": true,\n      className: classes.asterisk,\n      children: [\"\\u2009\", '*']\n    })]\n  });\n});\n true ? FormLabel.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOf(['error', 'info', 'primary', 'secondary', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)]),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().elementType),\n  /**\n   * If `true`, the label should be displayed in a disabled state.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the label is displayed in an error state.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the label should use filled classes key.\n   */\n  filled: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the input of this label is focused (used by `FormGroup` components).\n   */\n  focused: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * If `true`, the label will indicate that the `input` is required.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FormLabel);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormLabel/FormLabel.js?");

/***/ }),

/***/ "./node_modules/@mui/material/FormLabel/formLabelClasses.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/FormLabel/formLabelClasses.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getFormLabelUtilityClasses: () => (/* binding */ getFormLabelUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getFormLabelUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiFormLabel', slot);\n}\nconst formLabelClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiFormLabel', ['root', 'colorSecondary', 'focused', 'disabled', 'error', 'filled', 'required', 'asterisk']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (formLabelClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/FormLabel/formLabelClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/GlobalStyles/GlobalStyles.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@mui/material/GlobalStyles/GlobalStyles.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/GlobalStyles/GlobalStyles.js\");\n/* harmony import */ var _styles_defaultTheme_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../styles/defaultTheme.js */ \"./node_modules/@mui/material/styles/defaultTheme.js\");\n/* harmony import */ var _styles_identifier_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../styles/identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\nfunction GlobalStyles(props) {\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_system__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n    ...props,\n    defaultTheme: _styles_defaultTheme_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n    themeId: _styles_identifier_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"]\n  });\n}\n true ? GlobalStyles.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The styles you want to apply globally.\n   */\n  styles: prop_types__WEBPACK_IMPORTED_MODULE_5___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_5___default().array), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().string), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().bool)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (GlobalStyles);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/GlobalStyles/GlobalStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Grow/Grow.js":
/*!*************************************************!*\
  !*** ./node_modules/@mui/material/Grow/Grow.js ***!
  \*************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var _mui_utils_useTimeout__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/useTimeout */ \"./node_modules/@mui/utils/esm/useTimeout/useTimeout.js\");\n/* harmony import */ var _mui_utils_elementAcceptingRef__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/utils/elementAcceptingRef */ \"./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js\");\n/* harmony import */ var _mui_utils_getReactElementRef__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils/getReactElementRef */ \"./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js\");\n/* harmony import */ var react_transition_group__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react-transition-group */ \"./node_modules/react-transition-group/esm/Transition.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/useTheme.js\");\n/* harmony import */ var _transitions_utils_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../transitions/utils.js */ \"./node_modules/@mui/material/transitions/utils.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nfunction getScale(value) {\n  return `scale(${value}, ${value ** 2})`;\n}\nconst styles = {\n  entering: {\n    opacity: 1,\n    transform: getScale(1)\n  },\n  entered: {\n    opacity: 1,\n    transform: 'none'\n  }\n};\n\n/*\n TODO v6: remove\n Conditionally apply a workaround for the CSS transition bug in Safari 15.4 / WebKit browsers.\n */\nconst isWebKit154 = typeof navigator !== 'undefined' && /^((?!chrome|android).)*(safari|mobile)/i.test(navigator.userAgent) && /(os |version\\/)15(.|_)4/i.test(navigator.userAgent);\n\n/**\n * The Grow transition is used by the [Tooltip](/material-ui/react-tooltip/) and\n * [Popover](/material-ui/react-popover/) components.\n * It uses [react-transition-group](https://github.com/reactjs/react-transition-group) internally.\n */\nconst Grow = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Grow(props, ref) {\n  const {\n    addEndListener,\n    appear = true,\n    children,\n    easing,\n    in: inProp,\n    onEnter,\n    onEntered,\n    onEntering,\n    onExit,\n    onExited,\n    onExiting,\n    style,\n    timeout = 'auto',\n    // eslint-disable-next-line react/prop-types\n    TransitionComponent = react_transition_group__WEBPACK_IMPORTED_MODULE_2__[\"default\"],\n    ...other\n  } = props;\n  const timer = (0,_mui_utils_useTimeout__WEBPACK_IMPORTED_MODULE_3__[\"default\"])();\n  const autoTimeout = react__WEBPACK_IMPORTED_MODULE_0__.useRef();\n  const theme = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])();\n  const nodeRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const handleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(nodeRef, (0,_mui_utils_getReactElementRef__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(children), ref);\n  const normalizedTransitionCallback = callback => maybeIsAppearing => {\n    if (callback) {\n      const node = nodeRef.current;\n\n      // onEnterXxx and onExitXxx callbacks have a different arguments.length value.\n      if (maybeIsAppearing === undefined) {\n        callback(node);\n      } else {\n        callback(node, maybeIsAppearing);\n      }\n    }\n  };\n  const handleEntering = normalizedTransitionCallback(onEntering);\n  const handleEnter = normalizedTransitionCallback((node, isAppearing) => {\n    (0,_transitions_utils_js__WEBPACK_IMPORTED_MODULE_7__.reflow)(node); // So the animation always start from the start.\n\n    const {\n      duration: transitionDuration,\n      delay,\n      easing: transitionTimingFunction\n    } = (0,_transitions_utils_js__WEBPACK_IMPORTED_MODULE_7__.getTransitionProps)({\n      style,\n      timeout,\n      easing\n    }, {\n      mode: 'enter'\n    });\n    let duration;\n    if (timeout === 'auto') {\n      duration = theme.transitions.getAutoHeightDuration(node.clientHeight);\n      autoTimeout.current = duration;\n    } else {\n      duration = transitionDuration;\n    }\n    node.style.transition = [theme.transitions.create('opacity', {\n      duration,\n      delay\n    }), theme.transitions.create('transform', {\n      duration: isWebKit154 ? duration : duration * 0.666,\n      delay,\n      easing: transitionTimingFunction\n    })].join(',');\n    if (onEnter) {\n      onEnter(node, isAppearing);\n    }\n  });\n  const handleEntered = normalizedTransitionCallback(onEntered);\n  const handleExiting = normalizedTransitionCallback(onExiting);\n  const handleExit = normalizedTransitionCallback(node => {\n    const {\n      duration: transitionDuration,\n      delay,\n      easing: transitionTimingFunction\n    } = (0,_transitions_utils_js__WEBPACK_IMPORTED_MODULE_7__.getTransitionProps)({\n      style,\n      timeout,\n      easing\n    }, {\n      mode: 'exit'\n    });\n    let duration;\n    if (timeout === 'auto') {\n      duration = theme.transitions.getAutoHeightDuration(node.clientHeight);\n      autoTimeout.current = duration;\n    } else {\n      duration = transitionDuration;\n    }\n    node.style.transition = [theme.transitions.create('opacity', {\n      duration,\n      delay\n    }), theme.transitions.create('transform', {\n      duration: isWebKit154 ? duration : duration * 0.666,\n      delay: isWebKit154 ? delay : delay || duration * 0.333,\n      easing: transitionTimingFunction\n    })].join(',');\n    node.style.opacity = 0;\n    node.style.transform = getScale(0.75);\n    if (onExit) {\n      onExit(node);\n    }\n  });\n  const handleExited = normalizedTransitionCallback(onExited);\n  const handleAddEndListener = next => {\n    if (timeout === 'auto') {\n      timer.start(autoTimeout.current || 0, next);\n    }\n    if (addEndListener) {\n      // Old call signature before `react-transition-group` implemented `nodeRef`\n      addEndListener(nodeRef.current, next);\n    }\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(TransitionComponent, {\n    appear: appear,\n    in: inProp,\n    nodeRef: nodeRef,\n    onEnter: handleEnter,\n    onEntered: handleEntered,\n    onEntering: handleEntering,\n    onExit: handleExit,\n    onExited: handleExited,\n    onExiting: handleExiting,\n    addEndListener: handleAddEndListener,\n    timeout: timeout === 'auto' ? null : timeout,\n    ...other,\n    children: (state, {\n      ownerState,\n      ...restChildProps\n    }) => {\n      return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(children, {\n        style: {\n          opacity: 0,\n          transform: getScale(0.75),\n          visibility: state === 'exited' && !inProp ? 'hidden' : undefined,\n          ...styles[state],\n          ...style,\n          ...children.props.style\n        },\n        ref: handleRef,\n        ...restChildProps\n      });\n    }\n  });\n});\n true ? Grow.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Add a custom transition end trigger. Called with the transitioning DOM\n   * node and a done callback. Allows for more fine grained transition end\n   * logic. Note: Timeouts are still used as a fallback if provided.\n   */\n  addEndListener: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * Perform the enter transition when it first mounts if `in` is also `true`.\n   * Set this to `false` to disable this behavior.\n   * @default true\n   */\n  appear: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * A single child content element.\n   */\n  children: _mui_utils_elementAcceptingRef__WEBPACK_IMPORTED_MODULE_9__[\"default\"].isRequired,\n  /**\n   * The transition timing function.\n   * You may specify a single easing or a object containing enter and exit values.\n   */\n  easing: prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_8___default().shape({\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string)\n  }), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string)]),\n  /**\n   * If `true`, the component will transition in.\n   */\n  in: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * @ignore\n   */\n  onEnter: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * @ignore\n   */\n  onEntered: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * @ignore\n   */\n  onEntering: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * @ignore\n   */\n  onExit: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * @ignore\n   */\n  onExited: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * @ignore\n   */\n  onExiting: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object),\n  /**\n   * The duration for the transition, in milliseconds.\n   * You may specify a single timeout for all transitions, or individually with an object.\n   *\n   * Set to 'auto' to automatically calculate transition time based on height.\n   * @default 'auto'\n   */\n  timeout: prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOf(['auto']), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().number), prop_types__WEBPACK_IMPORTED_MODULE_8___default().shape({\n    appear: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().number),\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().number),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().number)\n  })])\n} : 0;\nif (Grow) {\n  Grow.muiSupportAuto = true;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Grow);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Grow/Grow.js?");

/***/ }),

/***/ "./node_modules/@mui/material/IconButton/IconButton.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/IconButton/IconButton.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_15___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_15__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/chainPropTypes */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_material_utils__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/material/utils */ \"./node_modules/@mui/material/utils/useId.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../ButtonBase/index.js */ \"./node_modules/@mui/material/ButtonBase/ButtonBase.js\");\n/* harmony import */ var _CircularProgress_index_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../CircularProgress/index.js */ \"./node_modules/@mui/material/CircularProgress/CircularProgress.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _iconButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./iconButtonClasses.js */ \"./node_modules/@mui/material/IconButton/iconButtonClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    disabled,\n    color,\n    edge,\n    size,\n    loading\n  } = ownerState;\n  const slots = {\n    root: ['root', loading && 'loading', disabled && 'disabled', color !== 'default' && `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, edge && `edge${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(edge)}`, `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`],\n    loadingIndicator: ['loadingIndicator'],\n    loadingWrapper: ['loadingWrapper']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _iconButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__.getIconButtonUtilityClass, classes);\n};\nconst IconButtonRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiIconButton',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.loading && styles.loading, ownerState.color !== 'default' && styles[`color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color)}`], ownerState.edge && styles[`edge${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.edge)}`], styles[`size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  textAlign: 'center',\n  flex: '0 0 auto',\n  fontSize: theme.typography.pxToRem(24),\n  padding: 8,\n  borderRadius: '50%',\n  color: (theme.vars || theme).palette.action.active,\n  transition: theme.transitions.create('background-color', {\n    duration: theme.transitions.duration.shortest\n  }),\n  variants: [{\n    props: props => !props.disableRipple,\n    style: {\n      '--IconButton-hoverBg': theme.vars ? `rgba(${theme.vars.palette.action.activeChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.action.active, theme.palette.action.hoverOpacity),\n      '&:hover': {\n        backgroundColor: 'var(--IconButton-hoverBg)',\n        // Reset on touch devices, it doesn't add specificity\n        '@media (hover: none)': {\n          backgroundColor: 'transparent'\n        }\n      }\n    }\n  }, {\n    props: {\n      edge: 'start'\n    },\n    style: {\n      marginLeft: -12\n    }\n  }, {\n    props: {\n      edge: 'start',\n      size: 'small'\n    },\n    style: {\n      marginLeft: -3\n    }\n  }, {\n    props: {\n      edge: 'end'\n    },\n    style: {\n      marginRight: -12\n    }\n  }, {\n    props: {\n      edge: 'end',\n      size: 'small'\n    },\n    style: {\n      marginRight: -3\n    }\n  }]\n})), (0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  variants: [{\n    props: {\n      color: 'inherit'\n    },\n    style: {\n      color: 'inherit'\n    }\n  }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])()) // check all the used fields in the style below\n  .map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      color: (theme.vars || theme).palette[color].main\n    }\n  })), ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])()) // check all the used fields in the style below\n  .map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      '--IconButton-hoverBg': theme.vars ? `rgba(${(theme.vars || theme).palette[color].mainChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)((theme.vars || theme).palette[color].main, theme.palette.action.hoverOpacity)\n    }\n  })), {\n    props: {\n      size: 'small'\n    },\n    style: {\n      padding: 5,\n      fontSize: theme.typography.pxToRem(18)\n    }\n  }, {\n    props: {\n      size: 'large'\n    },\n    style: {\n      padding: 12,\n      fontSize: theme.typography.pxToRem(28)\n    }\n  }],\n  [`&.${_iconButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    backgroundColor: 'transparent',\n    color: (theme.vars || theme).palette.action.disabled\n  },\n  [`&.${_iconButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].loading}`]: {\n    color: 'transparent'\n  }\n})));\nconst IconButtonLoadingIndicator = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiIconButton',\n  slot: 'LoadingIndicator',\n  overridesResolver: (props, styles) => styles.loadingIndicator\n})(({\n  theme\n}) => ({\n  display: 'none',\n  position: 'absolute',\n  visibility: 'visible',\n  top: '50%',\n  left: '50%',\n  transform: 'translate(-50%, -50%)',\n  color: (theme.vars || theme).palette.action.disabled,\n  variants: [{\n    props: {\n      loading: true\n    },\n    style: {\n      display: 'flex'\n    }\n  }]\n}));\n\n/**\n * Refer to the [Icons](/material-ui/icons/) section of the documentation\n * regarding the available icon options.\n */\nconst IconButton = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function IconButton(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiIconButton'\n  });\n  const {\n    edge = false,\n    children,\n    className,\n    color = 'default',\n    disabled = false,\n    disableFocusRipple = false,\n    size = 'medium',\n    id: idProp,\n    loading = null,\n    loadingIndicator: loadingIndicatorProp,\n    ...other\n  } = props;\n  const loadingId = (0,_mui_material_utils__WEBPACK_IMPORTED_MODULE_12__[\"default\"])(idProp);\n  const loadingIndicator = loadingIndicatorProp ?? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_CircularProgress_index_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"], {\n    \"aria-labelledby\": loadingId,\n    color: \"inherit\",\n    size: 16\n  });\n  const ownerState = {\n    ...props,\n    edge,\n    color,\n    disabled,\n    disableFocusRipple,\n    loading,\n    loadingIndicator,\n    size\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(IconButtonRoot, {\n    id: loading ? loadingId : idProp,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    centerRipple: true,\n    focusRipple: !disableFocusRipple,\n    disabled: disabled || loading,\n    ref: ref,\n    ...other,\n    ownerState: ownerState,\n    children: [typeof loading === 'boolean' &&\n    /*#__PURE__*/\n    // use plain HTML span to minimize the runtime overhead\n    (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"span\", {\n      className: classes.loadingWrapper,\n      style: {\n        display: 'contents'\n      },\n      children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(IconButtonLoadingIndicator, {\n        className: classes.loadingIndicator,\n        ownerState: ownerState,\n        children: loading && loadingIndicator\n      })\n    }), children]\n  });\n});\n true ? IconButton.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The icon to display.\n   */\n  children: (0,_mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_14__[\"default\"])((prop_types__WEBPACK_IMPORTED_MODULE_15___default().node), props => {\n    const found = react__WEBPACK_IMPORTED_MODULE_0__.Children.toArray(props.children).some(child => /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(child) && child.props.onClick);\n    if (found) {\n      return new Error(['MUI: You are providing an onClick event listener to a child of a button element.', 'Prefer applying it to the IconButton directly.', 'This guarantees that the whole <button> will be responsive to click events.'].join('\\n'));\n    }\n    return null;\n  }),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'default'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOf(['inherit', 'default', 'primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_15___default().string)]),\n  /**\n   * If `true`, the component is disabled.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().bool),\n  /**\n   * If `true`, the  keyboard focus ripple is disabled.\n   * @default false\n   */\n  disableFocusRipple: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().bool),\n  /**\n   * If `true`, the ripple effect is disabled.\n   *\n   * ⚠️ Without a ripple there is no styling for :focus-visible by default. Be sure\n   * to highlight the element by applying separate styles with the `.Mui-focusVisible` class.\n   * @default false\n   */\n  disableRipple: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().bool),\n  /**\n   * If given, uses a negative margin to counteract the padding on one\n   * side (this is often helpful for aligning the left or right\n   * side of the icon with content above or below, without ruining the border\n   * size and shape).\n   * @default false\n   */\n  edge: prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOf(['end', 'start', false]),\n  /**\n   * @ignore\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().string),\n  /**\n   * If `true`, the loading indicator is visible and the button is disabled.\n   * If `true | false`, the loading wrapper is always rendered before the children to prevent [Google Translation Crash](https://github.com/mui/material-ui/issues/27853).\n   * @default null\n   */\n  loading: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().bool),\n  /**\n   * Element placed before the children if the button is in loading state.\n   * The node should contain an element with `role=\"progressbar\"` with an accessible name.\n   * By default, it renders a `CircularProgress` that is labeled by the button itself.\n   * @default <CircularProgress color=\"inherit\" size={16} />\n   */\n  loadingIndicator: (prop_types__WEBPACK_IMPORTED_MODULE_15___default().node),\n  /**\n   * The size of the component.\n   * `small` is equivalent to the dense button styling.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOf(['small', 'medium', 'large']), (prop_types__WEBPACK_IMPORTED_MODULE_15___default().string)]),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_15___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_15___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_15___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_15___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_15___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_15___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_15___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (IconButton);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/IconButton/IconButton.js?");

/***/ }),

/***/ "./node_modules/@mui/material/IconButton/iconButtonClasses.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/IconButton/iconButtonClasses.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getIconButtonUtilityClass: () => (/* binding */ getIconButtonUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getIconButtonUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiIconButton', slot);\n}\nconst iconButtonClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiIconButton', ['root', 'disabled', 'colorInherit', 'colorPrimary', 'colorSecondary', 'colorError', 'colorInfo', 'colorSuccess', 'colorWarning', 'edgeStart', 'edgeEnd', 'sizeSmall', 'sizeMedium', 'sizeLarge', 'loading', 'loadingIndicator', 'loadingWrapper']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (iconButtonClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/IconButton/iconButtonClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/InitColorSchemeScript/InitColorSchemeScript.js":
/*!***********************************************************************************!*\
  !*** ./node_modules/@mui/material/InitColorSchemeScript/InitColorSchemeScript.js ***!
  \***********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   defaultConfig: () => (/* binding */ defaultConfig)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_system_InitColorSchemeScript__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system/InitColorSchemeScript */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/InitColorSchemeScript/InitColorSchemeScript.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\n\n\nconst defaultConfig = {\n  attribute: 'data-mui-color-scheme',\n  colorSchemeStorageKey: 'mui-color-scheme',\n  defaultLightColorScheme: 'light',\n  defaultDarkColorScheme: 'dark',\n  modeStorageKey: 'mui-mode'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (function InitColorSchemeScript(props) {\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_system_InitColorSchemeScript__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n    ...defaultConfig,\n    ...props\n  });\n});\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/InitColorSchemeScript/InitColorSchemeScript.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Input/Input.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/Input/Input.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../InputBase/InputBase.js */ \"./node_modules/@mui/material/InputBase/InputBase.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _inputClasses_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./inputClasses.js */ \"./node_modules/@mui/material/Input/inputClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    disableUnderline\n  } = ownerState;\n  const slots = {\n    root: ['root', !disableUnderline && 'underline'],\n    input: ['input']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(slots, _inputClasses_js__WEBPACK_IMPORTED_MODULE_3__.getInputUtilityClass, classes);\n  return {\n    ...classes,\n    // forward classes to the InputBase\n    ...composedClasses\n  };\n};\nconst InputRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.InputBaseRoot, {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiInput',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [...(0,_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.rootOverridesResolver)(props, styles), !ownerState.disableUnderline && styles.underline];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => {\n  const light = theme.palette.mode === 'light';\n  let bottomLineColor = light ? 'rgba(0, 0, 0, 0.42)' : 'rgba(255, 255, 255, 0.7)';\n  if (theme.vars) {\n    bottomLineColor = `rgba(${theme.vars.palette.common.onBackgroundChannel} / ${theme.vars.opacity.inputUnderline})`;\n  }\n  return {\n    position: 'relative',\n    variants: [{\n      props: ({\n        ownerState\n      }) => ownerState.formControl,\n      style: {\n        'label + &': {\n          marginTop: 16\n        }\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => !ownerState.disableUnderline,\n      style: {\n        '&::after': {\n          left: 0,\n          bottom: 0,\n          content: '\"\"',\n          position: 'absolute',\n          right: 0,\n          transform: 'scaleX(0)',\n          transition: theme.transitions.create('transform', {\n            duration: theme.transitions.duration.shorter,\n            easing: theme.transitions.easing.easeOut\n          }),\n          pointerEvents: 'none' // Transparent to the hover style.\n        },\n        [`&.${_inputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].focused}:after`]: {\n          // translateX(0) is a workaround for Safari transform scale bug\n          // See https://github.com/mui/material-ui/issues/31766\n          transform: 'scaleX(1) translateX(0)'\n        },\n        [`&.${_inputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].error}`]: {\n          '&::before, &::after': {\n            borderBottomColor: (theme.vars || theme).palette.error.main\n          }\n        },\n        '&::before': {\n          borderBottom: `1px solid ${bottomLineColor}`,\n          left: 0,\n          bottom: 0,\n          content: '\"\\\\00a0\"',\n          position: 'absolute',\n          right: 0,\n          transition: theme.transitions.create('border-bottom-color', {\n            duration: theme.transitions.duration.shorter\n          }),\n          pointerEvents: 'none' // Transparent to the hover style.\n        },\n        [`&:hover:not(.${_inputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].disabled}, .${_inputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].error}):before`]: {\n          borderBottom: `2px solid ${(theme.vars || theme).palette.text.primary}`,\n          // Reset on touch devices, it doesn't add specificity\n          '@media (hover: none)': {\n            borderBottom: `1px solid ${bottomLineColor}`\n          }\n        },\n        [`&.${_inputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].disabled}:before`]: {\n          borderBottomStyle: 'dotted'\n        }\n      }\n    }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])()).map(([color]) => ({\n      props: {\n        color,\n        disableUnderline: false\n      },\n      style: {\n        '&::after': {\n          borderBottom: `2px solid ${(theme.vars || theme).palette[color].main}`\n        }\n      }\n    }))]\n  };\n}));\nconst InputInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.InputBaseInput, {\n  name: 'MuiInput',\n  slot: 'Input',\n  overridesResolver: _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.inputOverridesResolver\n})({});\nconst Input = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Input(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiInput'\n  });\n  const {\n    disableUnderline = false,\n    components = {},\n    componentsProps: componentsPropsProp,\n    fullWidth = false,\n    inputComponent = 'input',\n    multiline = false,\n    slotProps,\n    slots = {},\n    type = 'text',\n    ...other\n  } = props;\n  const classes = useUtilityClasses(props);\n  const ownerState = {\n    disableUnderline\n  };\n  const inputComponentsProps = {\n    root: {\n      ownerState\n    }\n  };\n  const componentsProps = slotProps ?? componentsPropsProp ? (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(slotProps ?? componentsPropsProp, inputComponentsProps) : inputComponentsProps;\n  const RootSlot = slots.root ?? components.Root ?? InputRoot;\n  const InputSlot = slots.input ?? components.Input ?? InputInput;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"], {\n    slots: {\n      root: RootSlot,\n      input: InputSlot\n    },\n    slotProps: componentsProps,\n    fullWidth: fullWidth,\n    inputComponent: inputComponent,\n    multiline: multiline,\n    ref: ref,\n    type: type,\n    ...other,\n    classes: classes\n  });\n});\n true ? Input.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * This prop helps users to fill forms faster, especially on mobile devices.\n   * The name can be confusing, as it's more like an autofill.\n   * You can learn more about it [following the specification](https://html.spec.whatwg.org/multipage/form-control-infrastructure.html#autofill).\n   */\n  autoComplete: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * If `true`, the `input` element is focused during the first mount.\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * The prop defaults to the value (`'primary'`) inherited from the parent FormControl component.\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['primary', 'secondary']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_11___default().shape({\n    Input: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n    Root: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType)\n  }),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * @deprecated use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_11___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)\n  }),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().any),\n  /**\n   * If `true`, the component is disabled.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the `input` will not have an underline.\n   * @default false\n   */\n  disableUnderline: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * End `InputAdornment` for this component.\n   */\n  endAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * If `true`, the `input` will indicate an error.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the `input` will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * The id of the `input` element.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The component used for the `input` element.\n   * Either a string to use a HTML element or a component.\n   * @default 'input'\n   */\n  inputComponent: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * @default {}\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_12__[\"default\"],\n  /**\n   * If `dense`, will adjust vertical spacing. This is normally obtained via context from\n   * FormControl.\n   * The prop defaults to the value (`'none'`) inherited from the parent FormControl component.\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['dense', 'none']),\n  /**\n   * Maximum number of rows to display when multiline option is set to true.\n   */\n  maxRows: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * Minimum number of rows to display when multiline option is set to true.\n   */\n  minRows: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * If `true`, a [TextareaAutosize](https://mui.com/material-ui/react-textarea-autosize/) element is rendered.\n   * @default false\n   */\n  multiline: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * Name attribute of the `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * Callback fired when the value is changed.\n   *\n   * @param {React.ChangeEvent<HTMLTextAreaElement | HTMLInputElement>} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func),\n  /**\n   * The short hint displayed in the `input` before the user enters a value.\n   */\n  placeholder: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * It prevents the user from changing the value of the field\n   * (not from interacting with the field).\n   */\n  readOnly: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the `input` element is required.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * Number of rows to display when multiline option is set to true.\n   */\n  rows: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * This prop is an alias for the `componentsProps` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_11___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)\n  }),\n  /**\n   * The components used for each slot inside.\n   *\n   * This prop is an alias for the `components` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_11___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType)\n  }),\n  /**\n   * Start `InputAdornment` for this component.\n   */\n  startAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n  /**\n   * Type of the `input` element. It should be [a valid HTML5 input type](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Form_%3Cinput%3E_types).\n   * @default 'text'\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The value of the `input` element, required for a controlled component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().any)\n} : 0;\nInput.muiName = 'Input';\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Input);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Input/Input.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Input/inputClasses.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/Input/inputClasses.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getInputUtilityClass: () => (/* binding */ getInputUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n/* harmony import */ var _InputBase_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../InputBase/index.js */ \"./node_modules/@mui/material/InputBase/inputBaseClasses.js\");\n\n\n\nfunction getInputUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiInput', slot);\n}\nconst inputClasses = {\n  ..._InputBase_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"],\n  ...(0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_2__[\"default\"])('MuiInput', ['root', 'underline', 'input'])\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (inputClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Input/inputClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/InputBase/InputBase.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/InputBase/InputBase.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   InputBaseInput: () => (/* binding */ InputBaseInput),\n/* harmony export */   InputBaseRoot: () => (/* binding */ InputBaseRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   inputOverridesResolver: () => (/* binding */ inputOverridesResolver),\n/* harmony export */   rootOverridesResolver: () => (/* binding */ rootOverridesResolver)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_18___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_18__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_elementTypeAcceptingRef__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! @mui/utils/elementTypeAcceptingRef */ \"./node_modules/@mui/utils/esm/elementTypeAcceptingRef/elementTypeAcceptingRef.js\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _TextareaAutosize_index_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../TextareaAutosize/index.js */ \"./node_modules/@mui/material/TextareaAutosize/TextareaAutosize.js\");\n/* harmony import */ var _utils_isHostComponent_js__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../utils/isHostComponent.js */ \"./node_modules/@mui/material/utils/isHostComponent.js\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _FormControl_FormControlContext_js__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! ../FormControl/FormControlContext.js */ \"./node_modules/@mui/material/FormControl/FormControlContext.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/zero-styled/index.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var _utils_useEnhancedEffect_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../utils/useEnhancedEffect.js */ \"./node_modules/@mui/material/utils/useEnhancedEffect.js\");\n/* harmony import */ var _utils_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./utils.js */ \"./node_modules/@mui/material/InputBase/utils.js\");\n/* harmony import */ var _inputBaseClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./inputBaseClasses.js */ \"./node_modules/@mui/material/InputBase/inputBaseClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\nvar _InputGlobalStyles;\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst rootOverridesResolver = (props, styles) => {\n  const {\n    ownerState\n  } = props;\n  return [styles.root, ownerState.formControl && styles.formControl, ownerState.startAdornment && styles.adornedStart, ownerState.endAdornment && styles.adornedEnd, ownerState.error && styles.error, ownerState.size === 'small' && styles.sizeSmall, ownerState.multiline && styles.multiline, ownerState.color && styles[`color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color)}`], ownerState.fullWidth && styles.fullWidth, ownerState.hiddenLabel && styles.hiddenLabel];\n};\nconst inputOverridesResolver = (props, styles) => {\n  const {\n    ownerState\n  } = props;\n  return [styles.input, ownerState.size === 'small' && styles.inputSizeSmall, ownerState.multiline && styles.inputMultiline, ownerState.type === 'search' && styles.inputTypeSearch, ownerState.startAdornment && styles.inputAdornedStart, ownerState.endAdornment && styles.inputAdornedEnd, ownerState.hiddenLabel && styles.inputHiddenLabel];\n};\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    color,\n    disabled,\n    error,\n    endAdornment,\n    focused,\n    formControl,\n    fullWidth,\n    hiddenLabel,\n    multiline,\n    readOnly,\n    size,\n    startAdornment,\n    type\n  } = ownerState;\n  const slots = {\n    root: ['root', `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, disabled && 'disabled', error && 'error', fullWidth && 'fullWidth', focused && 'focused', formControl && 'formControl', size && size !== 'medium' && `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`, multiline && 'multiline', startAdornment && 'adornedStart', endAdornment && 'adornedEnd', hiddenLabel && 'hiddenLabel', readOnly && 'readOnly'],\n    input: ['input', disabled && 'disabled', type === 'search' && 'inputTypeSearch', multiline && 'inputMultiline', size === 'small' && 'inputSizeSmall', hiddenLabel && 'inputHiddenLabel', startAdornment && 'inputAdornedStart', endAdornment && 'inputAdornedEnd', readOnly && 'readOnly']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _inputBaseClasses_js__WEBPACK_IMPORTED_MODULE_5__.getInputBaseUtilityClass, classes);\n};\nconst InputBaseRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('div', {\n  name: 'MuiInputBase',\n  slot: 'Root',\n  overridesResolver: rootOverridesResolver\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  ...theme.typography.body1,\n  color: (theme.vars || theme).palette.text.primary,\n  lineHeight: '1.4375em',\n  // 23px\n  boxSizing: 'border-box',\n  // Prevent padding issue with fullWidth.\n  position: 'relative',\n  cursor: 'text',\n  display: 'inline-flex',\n  alignItems: 'center',\n  [`&.${_inputBaseClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    color: (theme.vars || theme).palette.text.disabled,\n    cursor: 'default'\n  },\n  variants: [{\n    props: ({\n      ownerState\n    }) => ownerState.multiline,\n    style: {\n      padding: '4px 0 5px'\n    }\n  }, {\n    props: ({\n      ownerState,\n      size\n    }) => ownerState.multiline && size === 'small',\n    style: {\n      paddingTop: 1\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.fullWidth,\n    style: {\n      width: '100%'\n    }\n  }]\n})));\nconst InputBaseInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('input', {\n  name: 'MuiInputBase',\n  slot: 'Input',\n  overridesResolver: inputOverridesResolver\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => {\n  const light = theme.palette.mode === 'light';\n  const placeholder = {\n    color: 'currentColor',\n    ...(theme.vars ? {\n      opacity: theme.vars.opacity.inputPlaceholder\n    } : {\n      opacity: light ? 0.42 : 0.5\n    }),\n    transition: theme.transitions.create('opacity', {\n      duration: theme.transitions.duration.shorter\n    })\n  };\n  const placeholderHidden = {\n    opacity: '0 !important'\n  };\n  const placeholderVisible = theme.vars ? {\n    opacity: theme.vars.opacity.inputPlaceholder\n  } : {\n    opacity: light ? 0.42 : 0.5\n  };\n  return {\n    font: 'inherit',\n    letterSpacing: 'inherit',\n    color: 'currentColor',\n    padding: '4px 0 5px',\n    border: 0,\n    boxSizing: 'content-box',\n    background: 'none',\n    height: '1.4375em',\n    // Reset 23pxthe native input line-height\n    margin: 0,\n    // Reset for Safari\n    WebkitTapHighlightColor: 'transparent',\n    display: 'block',\n    // Make the flex item shrink with Firefox\n    minWidth: 0,\n    width: '100%',\n    '&::-webkit-input-placeholder': placeholder,\n    '&::-moz-placeholder': placeholder,\n    // Firefox 19+\n    '&::-ms-input-placeholder': placeholder,\n    // Edge\n    '&:focus': {\n      outline: 0\n    },\n    // Reset Firefox invalid required input style\n    '&:invalid': {\n      boxShadow: 'none'\n    },\n    '&::-webkit-search-decoration': {\n      // Remove the padding when type=search.\n      WebkitAppearance: 'none'\n    },\n    // Show and hide the placeholder logic\n    [`label[data-shrink=false] + .${_inputBaseClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].formControl} &`]: {\n      '&::-webkit-input-placeholder': placeholderHidden,\n      '&::-moz-placeholder': placeholderHidden,\n      // Firefox 19+\n      '&::-ms-input-placeholder': placeholderHidden,\n      // Edge\n      '&:focus::-webkit-input-placeholder': placeholderVisible,\n      '&:focus::-moz-placeholder': placeholderVisible,\n      // Firefox 19+\n      '&:focus::-ms-input-placeholder': placeholderVisible // Edge\n    },\n    [`&.${_inputBaseClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n      opacity: 1,\n      // Reset iOS opacity\n      WebkitTextFillColor: (theme.vars || theme).palette.text.disabled // Fix opacity Safari bug\n    },\n    variants: [{\n      props: ({\n        ownerState\n      }) => !ownerState.disableInjectingGlobalStyles,\n      style: {\n        animationName: 'mui-auto-fill-cancel',\n        animationDuration: '10ms',\n        '&:-webkit-autofill': {\n          animationDuration: '5000s',\n          animationName: 'mui-auto-fill'\n        }\n      }\n    }, {\n      props: {\n        size: 'small'\n      },\n      style: {\n        paddingTop: 1\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.multiline,\n      style: {\n        height: 'auto',\n        resize: 'none',\n        padding: 0,\n        paddingTop: 0\n      }\n    }, {\n      props: {\n        type: 'search'\n      },\n      style: {\n        MozAppearance: 'textfield' // Improve type search style.\n      }\n    }]\n  };\n}));\nconst InputGlobalStyles = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_8__.globalCss)({\n  '@keyframes mui-auto-fill': {\n    from: {\n      display: 'block'\n    }\n  },\n  '@keyframes mui-auto-fill-cancel': {\n    from: {\n      display: 'block'\n    }\n  }\n});\n\n/**\n * `InputBase` contains as few styles as possible.\n * It aims to be a simple building block for creating an input.\n * It contains a load of style reset and some state logic.\n */\nconst InputBase = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function InputBase(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiInputBase'\n  });\n  const {\n    'aria-describedby': ariaDescribedby,\n    autoComplete,\n    autoFocus,\n    className,\n    color,\n    components = {},\n    componentsProps = {},\n    defaultValue,\n    disabled,\n    disableInjectingGlobalStyles,\n    endAdornment,\n    error,\n    fullWidth = false,\n    id,\n    inputComponent = 'input',\n    inputProps: inputPropsProp = {},\n    inputRef: inputRefProp,\n    margin,\n    maxRows,\n    minRows,\n    multiline = false,\n    name,\n    onBlur,\n    onChange,\n    onClick,\n    onFocus,\n    onKeyDown,\n    onKeyUp,\n    placeholder,\n    readOnly,\n    renderSuffix,\n    rows,\n    size,\n    slotProps = {},\n    slots = {},\n    startAdornment,\n    type = 'text',\n    value: valueProp,\n    ...other\n  } = props;\n  const value = inputPropsProp.value != null ? inputPropsProp.value : valueProp;\n  const {\n    current: isControlled\n  } = react__WEBPACK_IMPORTED_MODULE_0__.useRef(value != null);\n  const inputRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef();\n  const handleInputRefWarning = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(instance => {\n    if (true) {\n      if (instance && instance.nodeName !== 'INPUT' && !instance.focus) {\n        console.error(['MUI: You have provided a `inputComponent` to the input component', 'that does not correctly handle the `ref` prop.', 'Make sure the `ref` prop is called with a HTMLInputElement.'].join('\\n'));\n      }\n    }\n  }, []);\n  const handleInputRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(inputRef, inputRefProp, inputPropsProp.ref, handleInputRefWarning);\n  const [focused, setFocused] = react__WEBPACK_IMPORTED_MODULE_0__.useState(false);\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])();\n  if (true) {\n    // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler\n    // eslint-disable-next-line react-hooks/rules-of-hooks\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n      if (muiFormControl) {\n        return muiFormControl.registerEffect();\n      }\n      return undefined;\n    }, [muiFormControl]);\n  }\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['color', 'disabled', 'error', 'hiddenLabel', 'size', 'required', 'filled']\n  });\n  fcs.focused = muiFormControl ? muiFormControl.focused : focused;\n\n  // The blur won't fire when the disabled state is set on a focused input.\n  // We need to book keep the focused state manually.\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (!muiFormControl && disabled && focused) {\n      setFocused(false);\n      if (onBlur) {\n        onBlur();\n      }\n    }\n  }, [muiFormControl, disabled, focused, onBlur]);\n  const onFilled = muiFormControl && muiFormControl.onFilled;\n  const onEmpty = muiFormControl && muiFormControl.onEmpty;\n  const checkDirty = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(obj => {\n    if ((0,_utils_js__WEBPACK_IMPORTED_MODULE_13__.isFilled)(obj)) {\n      if (onFilled) {\n        onFilled();\n      }\n    } else if (onEmpty) {\n      onEmpty();\n    }\n  }, [onFilled, onEmpty]);\n  (0,_utils_useEnhancedEffect_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"])(() => {\n    if (isControlled) {\n      checkDirty({\n        value\n      });\n    }\n  }, [value, checkDirty, isControlled]);\n  const handleFocus = event => {\n    if (onFocus) {\n      onFocus(event);\n    }\n    if (inputPropsProp.onFocus) {\n      inputPropsProp.onFocus(event);\n    }\n    if (muiFormControl && muiFormControl.onFocus) {\n      muiFormControl.onFocus(event);\n    } else {\n      setFocused(true);\n    }\n  };\n  const handleBlur = event => {\n    if (onBlur) {\n      onBlur(event);\n    }\n    if (inputPropsProp.onBlur) {\n      inputPropsProp.onBlur(event);\n    }\n    if (muiFormControl && muiFormControl.onBlur) {\n      muiFormControl.onBlur(event);\n    } else {\n      setFocused(false);\n    }\n  };\n  const handleChange = (event, ...args) => {\n    if (!isControlled) {\n      const element = event.target || inputRef.current;\n      if (element == null) {\n        throw new Error( true ? 'MUI: Expected valid input target. ' + 'Did you use a custom `inputComponent` and forget to forward refs? ' + 'See https://mui.com/r/input-component-ref-interface for more info.' : 0);\n      }\n      checkDirty({\n        value: element.value\n      });\n    }\n    if (inputPropsProp.onChange) {\n      inputPropsProp.onChange(event, ...args);\n    }\n\n    // Perform in the willUpdate\n    if (onChange) {\n      onChange(event, ...args);\n    }\n  };\n\n  // Check the input state on mount, in case it was filled by the user\n  // or auto filled by the browser before the hydration (for SSR).\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    checkDirty(inputRef.current);\n    // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler\n    // eslint-disable-next-line react-hooks/exhaustive-deps\n  }, []);\n  const handleClick = event => {\n    if (inputRef.current && event.currentTarget === event.target) {\n      inputRef.current.focus();\n    }\n    if (onClick) {\n      onClick(event);\n    }\n  };\n  let InputComponent = inputComponent;\n  let inputProps = inputPropsProp;\n  if (multiline && InputComponent === 'input') {\n    if (rows) {\n      if (true) {\n        if (minRows || maxRows) {\n          console.warn('MUI: You can not use the `minRows` or `maxRows` props when the input `rows` prop is set.');\n        }\n      }\n      inputProps = {\n        type: undefined,\n        minRows: rows,\n        maxRows: rows,\n        ...inputProps\n      };\n    } else {\n      inputProps = {\n        type: undefined,\n        maxRows,\n        minRows,\n        ...inputProps\n      };\n    }\n    InputComponent = _TextareaAutosize_index_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"];\n  }\n  const handleAutoFill = event => {\n    // Provide a fake value as Chrome might not let you access it for security reasons.\n    checkDirty(event.animationName === 'mui-auto-fill-cancel' ? inputRef.current : {\n      value: 'x'\n    });\n  };\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (muiFormControl) {\n      muiFormControl.setAdornedStart(Boolean(startAdornment));\n    }\n  }, [muiFormControl, startAdornment]);\n  const ownerState = {\n    ...props,\n    color: fcs.color || 'primary',\n    disabled: fcs.disabled,\n    endAdornment,\n    error: fcs.error,\n    focused: fcs.focused,\n    formControl: muiFormControl,\n    fullWidth,\n    hiddenLabel: fcs.hiddenLabel,\n    multiline,\n    size: fcs.size,\n    startAdornment,\n    type\n  };\n  const classes = useUtilityClasses(ownerState);\n  const Root = slots.root || components.Root || InputBaseRoot;\n  const rootProps = slotProps.root || componentsProps.root || {};\n  const Input = slots.input || components.Input || InputBaseInput;\n  inputProps = {\n    ...inputProps,\n    ...(slotProps.input ?? componentsProps.input)\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: [!disableInjectingGlobalStyles && typeof InputGlobalStyles === 'function' && (// For Emotion/Styled-components, InputGlobalStyles will be a function\n    // For Pigment CSS, this has no effect because the InputGlobalStyles will be null.\n    _InputGlobalStyles || (_InputGlobalStyles = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(InputGlobalStyles, {}))), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(Root, {\n      ...rootProps,\n      ref: ref,\n      onClick: handleClick,\n      ...other,\n      ...(!(0,_utils_isHostComponent_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])(Root) && {\n        ownerState: {\n          ...ownerState,\n          ...rootProps.ownerState\n        }\n      }),\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, rootProps.className, className, readOnly && 'MuiInputBase-readOnly'),\n      children: [startAdornment, /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_FormControl_FormControlContext_js__WEBPACK_IMPORTED_MODULE_17__[\"default\"].Provider, {\n        value: null,\n        children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(Input, {\n          \"aria-invalid\": fcs.error,\n          \"aria-describedby\": ariaDescribedby,\n          autoComplete: autoComplete,\n          autoFocus: autoFocus,\n          defaultValue: defaultValue,\n          disabled: fcs.disabled,\n          id: id,\n          onAnimationStart: handleAutoFill,\n          name: name,\n          placeholder: placeholder,\n          readOnly: readOnly,\n          required: fcs.required,\n          rows: rows,\n          value: value,\n          onKeyDown: onKeyDown,\n          onKeyUp: onKeyUp,\n          type: type,\n          ...inputProps,\n          ...(!(0,_utils_isHostComponent_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])(Input) && {\n            as: InputComponent,\n            ownerState: {\n              ...ownerState,\n              ...inputProps.ownerState\n            }\n          }),\n          ref: handleInputRef,\n          className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.input, inputProps.className, readOnly && 'MuiInputBase-readOnly'),\n          onBlur: handleBlur,\n          onChange: handleChange,\n          onFocus: handleFocus\n        })\n      }), endAdornment, renderSuffix ? renderSuffix({\n        ...fcs,\n        startAdornment\n      }) : null]\n    })]\n  });\n});\n true ? InputBase.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  'aria-describedby': (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * This prop helps users to fill forms faster, especially on mobile devices.\n   * The name can be confusing, as it's more like an autofill.\n   * You can learn more about it [following the specification](https://html.spec.whatwg.org/multipage/form-control-infrastructure.html#autofill).\n   */\n  autoComplete: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * If `true`, the `input` element is focused during the first mount.\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * The prop defaults to the value (`'primary'`) inherited from the parent FormControl component.\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    Input: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n    Root: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType)\n  }),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * @deprecated use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)\n  }),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().any),\n  /**\n   * If `true`, the component is disabled.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * If `true`, GlobalStyles for the auto-fill keyframes will not be injected/removed on mount/unmount. Make sure to inject them at the top of your application.\n   * This option is intended to help with boosting the initial rendering performance if you are loading a big amount of Input components at once.\n   * @default false\n   */\n  disableInjectingGlobalStyles: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * End `InputAdornment` for this component.\n   */\n  endAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * If `true`, the `input` will indicate an error.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * If `true`, the `input` will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * The id of the `input` element.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * The component used for the `input` element.\n   * Either a string to use a HTML element or a component.\n   * @default 'input'\n   */\n  inputComponent: _mui_utils_elementTypeAcceptingRef__WEBPACK_IMPORTED_MODULE_19__[\"default\"],\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * @default {}\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_20__[\"default\"],\n  /**\n   * If `dense`, will adjust vertical spacing. This is normally obtained via context from\n   * FormControl.\n   * The prop defaults to the value (`'none'`) inherited from the parent FormControl component.\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['dense', 'none']),\n  /**\n   * Maximum number of rows to display when multiline option is set to true.\n   */\n  maxRows: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * Minimum number of rows to display when multiline option is set to true.\n   */\n  minRows: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * If `true`, a [TextareaAutosize](https://mui.com/material-ui/react-textarea-autosize/) element is rendered.\n   * @default false\n   */\n  multiline: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * Name attribute of the `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * Callback fired when the `input` is blurred.\n   *\n   * Notice that the first argument (event) might be undefined.\n   */\n  onBlur: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * Callback fired when the value is changed.\n   *\n   * @param {React.ChangeEvent<HTMLTextAreaElement | HTMLInputElement>} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * @ignore\n   */\n  onClick: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * @ignore\n   */\n  onFocus: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * Callback fired when the `input` doesn't satisfy its constraints.\n   */\n  onInvalid: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * @ignore\n   */\n  onKeyDown: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * @ignore\n   */\n  onKeyUp: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * The short hint displayed in the `input` before the user enters a value.\n   */\n  placeholder: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * It prevents the user from changing the value of the field\n   * (not from interacting with the field).\n   */\n  readOnly: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * @ignore\n   */\n  renderSuffix: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * If `true`, the `input` element is required.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * Number of rows to display when multiline option is set to true.\n   */\n  rows: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * The size of the component.\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['medium', 'small']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string)]),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * This prop is an alias for the `componentsProps` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)\n  }),\n  /**\n   * The components used for each slot inside.\n   *\n   * This prop is an alias for the `components` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType)\n  }),\n  /**\n   * Start `InputAdornment` for this component.\n   */\n  startAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)]),\n  /**\n   * Type of the `input` element. It should be [a valid HTML5 input type](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Form_%3Cinput%3E_types).\n   * @default 'text'\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * The value of the `input` element, required for a controlled component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().any)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (InputBase);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/InputBase/InputBase.js?");

/***/ }),

/***/ "./node_modules/@mui/material/InputBase/inputBaseClasses.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/InputBase/inputBaseClasses.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getInputBaseUtilityClass: () => (/* binding */ getInputBaseUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getInputBaseUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiInputBase', slot);\n}\nconst inputBaseClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiInputBase', ['root', 'formControl', 'focused', 'disabled', 'adornedStart', 'adornedEnd', 'error', 'sizeSmall', 'multiline', 'colorSecondary', 'fullWidth', 'hiddenLabel', 'readOnly', 'input', 'inputSizeSmall', 'inputMultiline', 'inputTypeSearch', 'inputAdornedStart', 'inputAdornedEnd', 'inputHiddenLabel']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (inputBaseClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/InputBase/inputBaseClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/InputBase/utils.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/material/InputBase/utils.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   hasValue: () => (/* binding */ hasValue),\n/* harmony export */   isAdornedStart: () => (/* binding */ isAdornedStart),\n/* harmony export */   isFilled: () => (/* binding */ isFilled)\n/* harmony export */ });\n// Supports determination of isControlled().\n// Controlled input accepts its current value as a prop.\n//\n// @see https://facebook.github.io/react/docs/forms.html#controlled-components\n// @param value\n// @returns {boolean} true if string (including '') or number (including zero)\nfunction hasValue(value) {\n  return value != null && !(Array.isArray(value) && value.length === 0);\n}\n\n// Determine if field is empty or filled.\n// Response determines if label is presented above field or as placeholder.\n//\n// @param obj\n// @param SSR\n// @returns {boolean} False when not present or empty string.\n//                    True when any number or string with length.\nfunction isFilled(obj, SSR = false) {\n  return obj && (hasValue(obj.value) && obj.value !== '' || SSR && hasValue(obj.defaultValue) && obj.defaultValue !== '');\n}\n\n// Determine if an Input is adorned on start.\n// It's corresponding to the left with LTR.\n//\n// @param obj\n// @returns {boolean} False when no adornments.\n//                    True when adorned at the start.\nfunction isAdornedStart(obj) {\n  return obj.startAdornment;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/InputBase/utils.js?");

/***/ }),

/***/ "./node_modules/@mui/material/InputLabel/InputLabel.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/InputLabel/InputLabel.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_14___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_14__);\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _FormLabel_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../FormLabel/index.js */ \"./node_modules/@mui/material/FormLabel/FormLabel.js\");\n/* harmony import */ var _FormLabel_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../FormLabel/index.js */ \"./node_modules/@mui/material/FormLabel/formLabelClasses.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _inputLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./inputLabelClasses.js */ \"./node_modules/@mui/material/InputLabel/inputLabelClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    formControl,\n    size,\n    shrink,\n    disableAnimation,\n    variant,\n    required\n  } = ownerState;\n  const slots = {\n    root: ['root', formControl && 'formControl', !disableAnimation && 'animated', shrink && 'shrink', size && size !== 'normal' && `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`, variant],\n    asterisk: [required && 'asterisk']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _inputLabelClasses_js__WEBPACK_IMPORTED_MODULE_5__.getInputLabelUtilityClasses, classes);\n  return {\n    ...classes,\n    // forward the focused, disabled, etc. classes to the FormLabel\n    ...composedClasses\n  };\n};\nconst InputLabelRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_FormLabel_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiInputLabel',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [{\n      [`& .${_FormLabel_index_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"].asterisk}`]: styles.asterisk\n    }, styles.root, ownerState.formControl && styles.formControl, ownerState.size === 'small' && styles.sizeSmall, ownerState.shrink && styles.shrink, !ownerState.disableAnimation && styles.animated, ownerState.focused && styles.focused, styles[ownerState.variant]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(({\n  theme\n}) => ({\n  display: 'block',\n  transformOrigin: 'top left',\n  whiteSpace: 'nowrap',\n  overflow: 'hidden',\n  textOverflow: 'ellipsis',\n  maxWidth: '100%',\n  variants: [{\n    props: ({\n      ownerState\n    }) => ownerState.formControl,\n    style: {\n      position: 'absolute',\n      left: 0,\n      top: 0,\n      // slight alteration to spec spacing to match visual spec result\n      transform: 'translate(0, 20px) scale(1)'\n    }\n  }, {\n    props: {\n      size: 'small'\n    },\n    style: {\n      // Compensation for the `Input.inputSizeSmall` style.\n      transform: 'translate(0, 17px) scale(1)'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.shrink,\n    style: {\n      transform: 'translate(0, -1.5px) scale(0.75)',\n      transformOrigin: 'top left',\n      maxWidth: '133%'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => !ownerState.disableAnimation,\n    style: {\n      transition: theme.transitions.create(['color', 'transform', 'max-width'], {\n        duration: theme.transitions.duration.shorter,\n        easing: theme.transitions.easing.easeOut\n      })\n    }\n  }, {\n    props: {\n      variant: 'filled'\n    },\n    style: {\n      // Chrome's autofill feature gives the input field a yellow background.\n      // Since the input field is behind the label in the HTML tree,\n      // the input field is drawn last and hides the label with an opaque background color.\n      // zIndex: 1 will raise the label above opaque background-colors of input.\n      zIndex: 1,\n      pointerEvents: 'none',\n      transform: 'translate(12px, 16px) scale(1)',\n      maxWidth: 'calc(100% - 24px)'\n    }\n  }, {\n    props: {\n      variant: 'filled',\n      size: 'small'\n    },\n    style: {\n      transform: 'translate(12px, 13px) scale(1)'\n    }\n  }, {\n    props: ({\n      variant,\n      ownerState\n    }) => variant === 'filled' && ownerState.shrink,\n    style: {\n      userSelect: 'none',\n      pointerEvents: 'auto',\n      transform: 'translate(12px, 7px) scale(0.75)',\n      maxWidth: 'calc(133% - 24px)'\n    }\n  }, {\n    props: ({\n      variant,\n      ownerState,\n      size\n    }) => variant === 'filled' && ownerState.shrink && size === 'small',\n    style: {\n      transform: 'translate(12px, 4px) scale(0.75)'\n    }\n  }, {\n    props: {\n      variant: 'outlined'\n    },\n    style: {\n      // see comment above on filled.zIndex\n      zIndex: 1,\n      pointerEvents: 'none',\n      transform: 'translate(14px, 16px) scale(1)',\n      maxWidth: 'calc(100% - 24px)'\n    }\n  }, {\n    props: {\n      variant: 'outlined',\n      size: 'small'\n    },\n    style: {\n      transform: 'translate(14px, 9px) scale(1)'\n    }\n  }, {\n    props: ({\n      variant,\n      ownerState\n    }) => variant === 'outlined' && ownerState.shrink,\n    style: {\n      userSelect: 'none',\n      pointerEvents: 'auto',\n      // Theoretically, we should have (8+5)*2/0.75 = 34px\n      // but it feels a better when it bleeds a bit on the left, so 32px.\n      maxWidth: 'calc(133% - 32px)',\n      transform: 'translate(14px, -9px) scale(0.75)'\n    }\n  }]\n})));\nconst InputLabel = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function InputLabel(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__.useDefaultProps)({\n    name: 'MuiInputLabel',\n    props: inProps\n  });\n  const {\n    disableAnimation = false,\n    margin,\n    shrink: shrinkProp,\n    variant,\n    className,\n    ...other\n  } = props;\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])();\n  let shrink = shrinkProp;\n  if (typeof shrink === 'undefined' && muiFormControl) {\n    shrink = muiFormControl.filled || muiFormControl.focused || muiFormControl.adornedStart;\n  }\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['size', 'variant', 'required', 'focused']\n  });\n  const ownerState = {\n    ...props,\n    disableAnimation,\n    formControl: muiFormControl,\n    shrink,\n    size: fcs.size,\n    variant: fcs.variant,\n    required: fcs.required,\n    focused: fcs.focused\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(InputLabelRoot, {\n    \"data-shrink\": shrink,\n    ref: ref,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ...other,\n    ownerState: ownerState,\n    classes: classes\n  });\n});\n true ? InputLabel.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['error', 'info', 'primary', 'secondary', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string)]),\n  /**\n   * If `true`, the transition animation is disabled.\n   * @default false\n   */\n  disableAnimation: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `true`, the component is disabled.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `true`, the label is displayed in an error state.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `true`, the `input` of this label is focused.\n   */\n  focused: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `dense`, will adjust vertical spacing. This is normally obtained via context from\n   * FormControl.\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['dense']),\n  /**\n   * if `true`, the label will indicate that the `input` is required.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `true`, the label is shrunk.\n   */\n  shrink: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * The size of the component.\n   * @default 'normal'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['normal', 'small']), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string)]),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n  /**\n   * The variant to use.\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['filled', 'outlined', 'standard'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (InputLabel);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/InputLabel/InputLabel.js?");

/***/ }),

/***/ "./node_modules/@mui/material/InputLabel/inputLabelClasses.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/InputLabel/inputLabelClasses.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getInputLabelUtilityClasses: () => (/* binding */ getInputLabelUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getInputLabelUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiInputLabel', slot);\n}\nconst inputLabelClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiInputLabel', ['root', 'focused', 'disabled', 'error', 'required', 'asterisk', 'formControl', 'sizeSmall', 'shrink', 'animated', 'standard', 'filled', 'outlined']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (inputLabelClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/InputLabel/inputLabelClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/List/List.js":
/*!*************************************************!*\
  !*** ./node_modules/@mui/material/List/List.js ***!
  \*************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _ListContext_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./ListContext.js */ \"./node_modules/@mui/material/List/ListContext.js\");\n/* harmony import */ var _listClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./listClasses.js */ \"./node_modules/@mui/material/List/listClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    disablePadding,\n    dense,\n    subheader\n  } = ownerState;\n  const slots = {\n    root: ['root', !disablePadding && 'padding', dense && 'dense', subheader && 'subheader']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _listClasses_js__WEBPACK_IMPORTED_MODULE_4__.getListUtilityClass, classes);\n};\nconst ListRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('ul', {\n  name: 'MuiList',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, !ownerState.disablePadding && styles.padding, ownerState.dense && styles.dense, ownerState.subheader && styles.subheader];\n  }\n})({\n  listStyle: 'none',\n  margin: 0,\n  padding: 0,\n  position: 'relative',\n  variants: [{\n    props: ({\n      ownerState\n    }) => !ownerState.disablePadding,\n    style: {\n      paddingTop: 8,\n      paddingBottom: 8\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.subheader,\n    style: {\n      paddingTop: 0\n    }\n  }]\n});\nconst List = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function List(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_6__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiList'\n  });\n  const {\n    children,\n    className,\n    component = 'ul',\n    dense = false,\n    disablePadding = false,\n    subheader,\n    ...other\n  } = props;\n  const context = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => ({\n    dense\n  }), [dense]);\n  const ownerState = {\n    ...props,\n    component,\n    dense,\n    disablePadding\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_ListContext_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"].Provider, {\n    value: context,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(ListRoot, {\n      as: component,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n      ref: ref,\n      ownerState: ownerState,\n      ...other,\n      children: [subheader, children]\n    })\n  });\n});\n true ? List.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().elementType),\n  /**\n   * If `true`, compact vertical padding designed for keyboard and mouse input is used for\n   * the list and list items.\n   * The prop is available to descendant components as the `dense` context.\n   * @default false\n   */\n  dense: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * If `true`, vertical padding is removed from the list.\n   * @default false\n   */\n  disablePadding: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * The content of the subheader, normally `ListSubheader`.\n   */\n  subheader: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().node),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_8___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_8___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (List);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/List/List.js?");

/***/ }),

/***/ "./node_modules/@mui/material/List/ListContext.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/List/ListContext.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n\n/**\n * @ignore - internal component.\n */\nconst ListContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext({});\nif (true) {\n  ListContext.displayName = 'ListContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ListContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/List/ListContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/List/listClasses.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/List/listClasses.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getListUtilityClass: () => (/* binding */ getListUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getListUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiList', slot);\n}\nconst listClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiList', ['root', 'padding', 'dense', 'subheader']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (listClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/List/listClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ListItemIcon/listItemIconClasses.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/material/ListItemIcon/listItemIconClasses.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getListItemIconUtilityClass: () => (/* binding */ getListItemIconUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getListItemIconUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiListItemIcon', slot);\n}\nconst listItemIconClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiListItemIcon', ['root', 'alignItemsFlexStart']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (listItemIconClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ListItemIcon/listItemIconClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ListItemText/listItemTextClasses.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/material/ListItemText/listItemTextClasses.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getListItemTextUtilityClass: () => (/* binding */ getListItemTextUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getListItemTextUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiListItemText', slot);\n}\nconst listItemTextClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiListItemText', ['root', 'multiline', 'dense', 'inset', 'primary', 'secondary']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (listItemTextClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ListItemText/listItemTextClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Menu/Menu.js":
/*!*************************************************!*\
  !*** ./node_modules/@mui/material/Menu/Menu.js ***!
  \*************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   MenuPaper: () => (/* binding */ MenuPaper),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_14___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_14__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! @mui/utils/HTMLElementType */ \"./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js\");\n/* harmony import */ var _mui_system_RtlProvider__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/system/RtlProvider */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/RtlProvider/index.js\");\n/* harmony import */ var _mui_utils_useSlotProps__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/utils/useSlotProps */ \"./node_modules/@mui/utils/esm/useSlotProps/useSlotProps.js\");\n/* harmony import */ var _MenuList_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../MenuList/index.js */ \"./node_modules/@mui/material/MenuList/MenuList.js\");\n/* harmony import */ var _Popover_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Popover/index.js */ \"./node_modules/@mui/material/Popover/Popover.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _menuClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./menuClasses.js */ \"./node_modules/@mui/material/Menu/menuClasses.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst RTL_ORIGIN = {\n  vertical: 'top',\n  horizontal: 'right'\n};\nconst LTR_ORIGIN = {\n  vertical: 'top',\n  horizontal: 'left'\n};\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root'],\n    paper: ['paper'],\n    list: ['list']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _menuClasses_js__WEBPACK_IMPORTED_MODULE_4__.getMenuUtilityClass, classes);\n};\nconst MenuRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Popover_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiMenu',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({});\nconst MenuPaper = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Popover_index_js__WEBPACK_IMPORTED_MODULE_6__.PopoverPaper, {\n  name: 'MuiMenu',\n  slot: 'Paper',\n  overridesResolver: (props, styles) => styles.paper\n})({\n  // specZ: The maximum height of a simple menu should be one or more rows less than the view\n  // height. This ensures a tappable area outside of the simple menu with which to dismiss\n  // the menu.\n  maxHeight: 'calc(100% - 96px)',\n  // Add iOS momentum scrolling for iOS < 13.0\n  WebkitOverflowScrolling: 'touch'\n});\nconst MenuMenuList = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_MenuList_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"], {\n  name: 'MuiMenu',\n  slot: 'List',\n  overridesResolver: (props, styles) => styles.list\n})({\n  // We disable the focus ring for mouse, touch and keyboard users.\n  outline: 0\n});\nconst Menu = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Menu(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiMenu'\n  });\n  const {\n    autoFocus = true,\n    children,\n    className,\n    disableAutoFocusItem = false,\n    MenuListProps = {},\n    onClose,\n    open,\n    PaperProps = {},\n    PopoverClasses,\n    transitionDuration = 'auto',\n    TransitionProps: {\n      onEntering,\n      ...TransitionProps\n    } = {},\n    variant = 'selectedMenu',\n    slots = {},\n    slotProps = {},\n    ...other\n  } = props;\n  const isRtl = (0,_mui_system_RtlProvider__WEBPACK_IMPORTED_MODULE_10__.useRtl)();\n  const ownerState = {\n    ...props,\n    autoFocus,\n    disableAutoFocusItem,\n    MenuListProps,\n    onEntering,\n    PaperProps,\n    transitionDuration,\n    TransitionProps,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  const autoFocusItem = autoFocus && !disableAutoFocusItem && open;\n  const menuListActionsRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const handleEntering = (element, isAppearing) => {\n    if (menuListActionsRef.current) {\n      menuListActionsRef.current.adjustStyleForScrollbar(element, {\n        direction: isRtl ? 'rtl' : 'ltr'\n      });\n    }\n    if (onEntering) {\n      onEntering(element, isAppearing);\n    }\n  };\n  const handleListKeyDown = event => {\n    if (event.key === 'Tab') {\n      event.preventDefault();\n      if (onClose) {\n        onClose(event, 'tabKeyDown');\n      }\n    }\n  };\n\n  /**\n   * the index of the item should receive focus\n   * in a `variant=\"selectedMenu\"` it's the first `selected` item\n   * otherwise it's the very first item.\n   */\n  let activeItemIndex = -1;\n  // since we inject focus related props into children we have to do a lookahead\n  // to check if there is a `selected` item. We're looking for the last `selected`\n  // item and use the first valid item as a fallback\n  react__WEBPACK_IMPORTED_MODULE_0__.Children.map(children, (child, index) => {\n    if (! /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(child)) {\n      return;\n    }\n    if (true) {\n      if ((0,react_is__WEBPACK_IMPORTED_MODULE_11__.isFragment)(child)) {\n        console.error([\"MUI: The Menu component doesn't accept a Fragment as a child.\", 'Consider providing an array instead.'].join('\\n'));\n      }\n    }\n    if (!child.props.disabled) {\n      if (variant === 'selectedMenu' && child.props.selected) {\n        activeItemIndex = index;\n      } else if (activeItemIndex === -1) {\n        activeItemIndex = index;\n      }\n    }\n  });\n  const externalForwardedProps = {\n    slots,\n    slotProps: {\n      list: MenuListProps,\n      transition: TransitionProps,\n      paper: PaperProps,\n      ...slotProps\n    }\n  };\n  const rootSlotProps = (0,_mui_utils_useSlotProps__WEBPACK_IMPORTED_MODULE_12__[\"default\"])({\n    elementType: slots.root,\n    externalSlotProps: slotProps.root,\n    ownerState,\n    className: [classes.root, className]\n  });\n  const [PaperSlot, paperSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"])('paper', {\n    className: classes.paper,\n    elementType: MenuPaper,\n    externalForwardedProps,\n    shouldForwardComponentProp: true,\n    ownerState\n  });\n  const [ListSlot, listSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"])('list', {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.list, MenuListProps.className),\n    elementType: MenuMenuList,\n    shouldForwardComponentProp: true,\n    externalForwardedProps,\n    getSlotProps: handlers => ({\n      ...handlers,\n      onKeyDown: event => {\n        handleListKeyDown(event);\n        handlers.onKeyDown?.(event);\n      }\n    }),\n    ownerState\n  });\n  const resolvedTransitionProps = typeof externalForwardedProps.slotProps.transition === 'function' ? externalForwardedProps.slotProps.transition(ownerState) : externalForwardedProps.slotProps.transition;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(MenuRoot, {\n    onClose: onClose,\n    anchorOrigin: {\n      vertical: 'bottom',\n      horizontal: isRtl ? 'right' : 'left'\n    },\n    transformOrigin: isRtl ? RTL_ORIGIN : LTR_ORIGIN,\n    slots: {\n      root: slots.root,\n      paper: PaperSlot,\n      backdrop: slots.backdrop,\n      ...(slots.transition && {\n        // TODO: pass `slots.transition` directly once `TransitionComponent` is removed from Popover\n        transition: slots.transition\n      })\n    },\n    slotProps: {\n      root: rootSlotProps,\n      paper: paperSlotProps,\n      backdrop: typeof slotProps.backdrop === 'function' ? slotProps.backdrop(ownerState) : slotProps.backdrop,\n      transition: {\n        ...resolvedTransitionProps,\n        onEntering: (...args) => {\n          handleEntering(...args);\n          resolvedTransitionProps?.onEntering?.(...args);\n        }\n      }\n    },\n    open: open,\n    ref: ref,\n    transitionDuration: transitionDuration,\n    ownerState: ownerState,\n    ...other,\n    classes: PopoverClasses,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ListSlot, {\n      actions: menuListActionsRef,\n      autoFocus: autoFocus && (activeItemIndex === -1 || disableAutoFocusItem),\n      autoFocusItem: autoFocusItem,\n      variant: variant,\n      ...listSlotProps,\n      children: children\n    })\n  });\n});\n true ? Menu.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * An HTML element, or a function that returns one.\n   * It's used to set the position of the menu.\n   */\n  anchorEl: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([_mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_15__[\"default\"], (prop_types__WEBPACK_IMPORTED_MODULE_14___default().func)]),\n  /**\n   * If `true` (Default) will focus the `[role=\"menu\"]` if no focusable child is found. Disabled\n   * children are not focusable. If you set this prop to `false` focus will be placed\n   * on the parent modal container. This has severe accessibility implications\n   * and should only be considered if you manage focus otherwise.\n   * @default true\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * Menu contents, normally `MenuItem`s.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string),\n  /**\n   * When opening the menu will not focus the active item but the `[role=\"menu\"]`\n   * unless `autoFocus` is also set to `false`. Not using the default means not\n   * following WAI-ARIA authoring practices. Please be considerate about possible\n   * accessibility implications.\n   * @default false\n   */\n  disableAutoFocusItem: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * Props applied to the [`MenuList`](https://mui.com/material-ui/api/menu-list/) element.\n   * @deprecated use the `slotProps.list` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   * @default {}\n   */\n  MenuListProps: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * Callback fired when the component requests to be closed.\n   *\n   * @param {object} event The event source of the callback.\n   * @param {string} reason Can be: `\"escapeKeyDown\"`, `\"backdropClick\"`, `\"tabKeyDown\"`.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().func),\n  /**\n   * If `true`, the component is shown.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool).isRequired,\n  /**\n   * @ignore\n   */\n  PaperProps: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * `classes` prop applied to the [`Popover`](https://mui.com/material-ui/api/popover/) element.\n   */\n  PopoverClasses: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_14___default().shape({\n    backdrop: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n    list: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n    paper: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n    transition: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_14___default().shape({\n    backdrop: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().elementType),\n    list: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().elementType),\n    paper: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().elementType),\n    transition: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n  /**\n   * The length of the transition in `ms`, or 'auto'\n   * @default 'auto'\n   */\n  transitionDuration: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['auto']), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().number), prop_types__WEBPACK_IMPORTED_MODULE_14___default().shape({\n    appear: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().number),\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().number),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().number)\n  })]),\n  /**\n   * Props applied to the transition element.\n   * By default, the element is based on this [`Transition`](https://reactcommunity.org/react-transition-group/transition/) component.\n   * @deprecated use the `slotProps.transition` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   * @default {}\n   */\n  TransitionProps: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * The variant to use. Use `menu` to prevent selected items from impacting the initial focus.\n   * @default 'selectedMenu'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['menu', 'selectedMenu'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Menu);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Menu/Menu.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Menu/menuClasses.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/Menu/menuClasses.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getMenuUtilityClass: () => (/* binding */ getMenuUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getMenuUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiMenu', slot);\n}\nconst menuClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiMenu', ['root', 'paper', 'list']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (menuClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Menu/menuClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/MenuItem/MenuItem.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/MenuItem/MenuItem.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   overridesResolver: () => (/* binding */ overridesResolver)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_17___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_17__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _List_ListContext_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../List/ListContext.js */ \"./node_modules/@mui/material/List/ListContext.js\");\n/* harmony import */ var _ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../ButtonBase/index.js */ \"./node_modules/@mui/material/ButtonBase/ButtonBase.js\");\n/* harmony import */ var _utils_useEnhancedEffect_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../utils/useEnhancedEffect.js */ \"./node_modules/@mui/material/utils/useEnhancedEffect.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var _Divider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../Divider/index.js */ \"./node_modules/@mui/material/Divider/dividerClasses.js\");\n/* harmony import */ var _ListItemIcon_index_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../ListItemIcon/index.js */ \"./node_modules/@mui/material/ListItemIcon/listItemIconClasses.js\");\n/* harmony import */ var _ListItemText_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../ListItemText/index.js */ \"./node_modules/@mui/material/ListItemText/listItemTextClasses.js\");\n/* harmony import */ var _menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./menuItemClasses.js */ \"./node_modules/@mui/material/MenuItem/menuItemClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst overridesResolver = (props, styles) => {\n  const {\n    ownerState\n  } = props;\n  return [styles.root, ownerState.dense && styles.dense, ownerState.divider && styles.divider, !ownerState.disableGutters && styles.gutters];\n};\nconst useUtilityClasses = ownerState => {\n  const {\n    disabled,\n    dense,\n    divider,\n    disableGutters,\n    selected,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', dense && 'dense', disabled && 'disabled', !disableGutters && 'gutters', divider && 'divider', selected && 'selected']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__.getMenuItemUtilityClass, classes);\n  return {\n    ...classes,\n    ...composedClasses\n  };\n};\nconst MenuItemRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiMenuItem',\n  slot: 'Root',\n  overridesResolver\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  ...theme.typography.body1,\n  display: 'flex',\n  justifyContent: 'flex-start',\n  alignItems: 'center',\n  position: 'relative',\n  textDecoration: 'none',\n  minHeight: 48,\n  paddingTop: 6,\n  paddingBottom: 6,\n  boxSizing: 'border-box',\n  whiteSpace: 'nowrap',\n  '&:hover': {\n    textDecoration: 'none',\n    backgroundColor: (theme.vars || theme).palette.action.hover,\n    // Reset on touch devices, it doesn't add specificity\n    '@media (hover: none)': {\n      backgroundColor: 'transparent'\n    }\n  },\n  [`&.${_menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].selected}`]: {\n    backgroundColor: theme.vars ? `rgba(${theme.vars.palette.primary.mainChannel} / ${theme.vars.palette.action.selectedOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.primary.main, theme.palette.action.selectedOpacity),\n    [`&.${_menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].focusVisible}`]: {\n      backgroundColor: theme.vars ? `rgba(${theme.vars.palette.primary.mainChannel} / calc(${theme.vars.palette.action.selectedOpacity} + ${theme.vars.palette.action.focusOpacity}))` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.primary.main, theme.palette.action.selectedOpacity + theme.palette.action.focusOpacity)\n    }\n  },\n  [`&.${_menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].selected}:hover`]: {\n    backgroundColor: theme.vars ? `rgba(${theme.vars.palette.primary.mainChannel} / calc(${theme.vars.palette.action.selectedOpacity} + ${theme.vars.palette.action.hoverOpacity}))` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.primary.main, theme.palette.action.selectedOpacity + theme.palette.action.hoverOpacity),\n    // Reset on touch devices, it doesn't add specificity\n    '@media (hover: none)': {\n      backgroundColor: theme.vars ? `rgba(${theme.vars.palette.primary.mainChannel} / ${theme.vars.palette.action.selectedOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.primary.main, theme.palette.action.selectedOpacity)\n    }\n  },\n  [`&.${_menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].focusVisible}`]: {\n    backgroundColor: (theme.vars || theme).palette.action.focus\n  },\n  [`&.${_menuItemClasses_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].disabled}`]: {\n    opacity: (theme.vars || theme).palette.action.disabledOpacity\n  },\n  [`& + .${_Divider_index_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"].root}`]: {\n    marginTop: theme.spacing(1),\n    marginBottom: theme.spacing(1)\n  },\n  [`& + .${_Divider_index_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"].inset}`]: {\n    marginLeft: 52\n  },\n  [`& .${_ListItemText_index_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"].root}`]: {\n    marginTop: 0,\n    marginBottom: 0\n  },\n  [`& .${_ListItemText_index_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"].inset}`]: {\n    paddingLeft: 36\n  },\n  [`& .${_ListItemIcon_index_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"].root}`]: {\n    minWidth: 36\n  },\n  variants: [{\n    props: ({\n      ownerState\n    }) => !ownerState.disableGutters,\n    style: {\n      paddingLeft: 16,\n      paddingRight: 16\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.divider,\n    style: {\n      borderBottom: `1px solid ${(theme.vars || theme).palette.divider}`,\n      backgroundClip: 'padding-box'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => !ownerState.dense,\n    style: {\n      [theme.breakpoints.up('sm')]: {\n        minHeight: 'auto'\n      }\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.dense,\n    style: {\n      minHeight: 32,\n      // https://m2.material.io/components/menus#specs > Dense\n      paddingTop: 4,\n      paddingBottom: 4,\n      ...theme.typography.body2,\n      [`& .${_ListItemIcon_index_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"].root} svg`]: {\n        fontSize: '1.25rem'\n      }\n    }\n  }]\n})));\nconst MenuItem = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function MenuItem(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_13__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiMenuItem'\n  });\n  const {\n    autoFocus = false,\n    component = 'li',\n    dense = false,\n    divider = false,\n    disableGutters = false,\n    focusVisibleClassName,\n    role = 'menuitem',\n    tabIndex: tabIndexProp,\n    className,\n    ...other\n  } = props;\n  const context = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_List_ListContext_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"]);\n  const childContext = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => ({\n    dense: dense || context.dense || false,\n    disableGutters\n  }), [context.dense, dense, disableGutters]);\n  const menuItemRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  (0,_utils_useEnhancedEffect_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])(() => {\n    if (autoFocus) {\n      if (menuItemRef.current) {\n        menuItemRef.current.focus();\n      } else if (true) {\n        console.error('MUI: Unable to set focus to a MenuItem whose component has not been rendered.');\n      }\n    }\n  }, [autoFocus]);\n  const ownerState = {\n    ...props,\n    dense: childContext.dense,\n    divider,\n    disableGutters\n  };\n  const classes = useUtilityClasses(props);\n  const handleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])(menuItemRef, ref);\n  let tabIndex;\n  if (!props.disabled) {\n    tabIndex = tabIndexProp !== undefined ? tabIndexProp : -1;\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_List_ListContext_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"].Provider, {\n    value: childContext,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(MenuItemRoot, {\n      ref: handleRef,\n      role: role,\n      tabIndex: tabIndex,\n      component: component,\n      focusVisibleClassName: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.focusVisible, focusVisibleClassName),\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n      ...other,\n      ownerState: ownerState,\n      classes: classes\n    })\n  });\n});\n true ? MenuItem.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * If `true`, the list item is focused during the first mount.\n   * Focus will also be triggered if the value changes from false to true.\n   * @default false\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().elementType),\n  /**\n   * If `true`, compact vertical padding designed for keyboard and mouse input is used.\n   * The prop defaults to the value inherited from the parent Menu component.\n   * @default false\n   */\n  dense: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * @ignore\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * If `true`, the left and right padding is removed.\n   * @default false\n   */\n  disableGutters: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * If `true`, a 1px light border is added to the bottom of the menu item.\n   * @default false\n   */\n  divider: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * This prop can help identify which element has keyboard focus.\n   * The class name will be applied when the element gains the focus through keyboard interaction.\n   * It's a polyfill for the [CSS :focus-visible selector](https://drafts.csswg.org/selectors-4/#the-focus-visible-pseudo).\n   * The rationale for using this feature [is explained here](https://github.com/WICG/focus-visible/blob/HEAD/explainer.md).\n   * A [polyfill can be used](https://github.com/WICG/focus-visible) to apply a `focus-visible` class to other components\n   * if needed.\n   */\n  focusVisibleClassName: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string),\n  /**\n   * @ignore\n   */\n  role: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().string),\n  /**\n   * If `true`, the component is selected.\n   * @default false\n   */\n  selected: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_17___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_17___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_17___default().object)]),\n  /**\n   * @default 0\n   */\n  tabIndex: (prop_types__WEBPACK_IMPORTED_MODULE_17___default().number)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (MenuItem);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/MenuItem/MenuItem.js?");

/***/ }),

/***/ "./node_modules/@mui/material/MenuItem/menuItemClasses.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/MenuItem/menuItemClasses.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getMenuItemUtilityClass: () => (/* binding */ getMenuItemUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getMenuItemUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiMenuItem', slot);\n}\nconst menuItemClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiMenuItem', ['root', 'focusVisible', 'dense', 'disabled', 'divider', 'gutters', 'selected']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (menuItemClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/MenuItem/menuItemClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/MenuList/MenuList.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/MenuList/MenuList.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var _utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../utils/ownerDocument.js */ \"./node_modules/@mui/material/utils/ownerDocument.js\");\n/* harmony import */ var _List_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../List/index.js */ \"./node_modules/@mui/material/List/List.js\");\n/* harmony import */ var _utils_getScrollbarSize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/getScrollbarSize.js */ \"./node_modules/@mui/material/utils/getScrollbarSize.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var _utils_useEnhancedEffect_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/useEnhancedEffect.js */ \"./node_modules/@mui/material/utils/useEnhancedEffect.js\");\n/* harmony import */ var _utils_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/index.js */ \"./node_modules/@mui/material/utils/ownerWindow.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nfunction nextItem(list, item, disableListWrap) {\n  if (list === item) {\n    return list.firstChild;\n  }\n  if (item && item.nextElementSibling) {\n    return item.nextElementSibling;\n  }\n  return disableListWrap ? null : list.firstChild;\n}\nfunction previousItem(list, item, disableListWrap) {\n  if (list === item) {\n    return disableListWrap ? list.firstChild : list.lastChild;\n  }\n  if (item && item.previousElementSibling) {\n    return item.previousElementSibling;\n  }\n  return disableListWrap ? null : list.lastChild;\n}\nfunction textCriteriaMatches(nextFocus, textCriteria) {\n  if (textCriteria === undefined) {\n    return true;\n  }\n  let text = nextFocus.innerText;\n  if (text === undefined) {\n    // jsdom doesn't support innerText\n    text = nextFocus.textContent;\n  }\n  text = text.trim().toLowerCase();\n  if (text.length === 0) {\n    return false;\n  }\n  if (textCriteria.repeating) {\n    return text[0] === textCriteria.keys[0];\n  }\n  return text.startsWith(textCriteria.keys.join(''));\n}\nfunction moveFocus(list, currentFocus, disableListWrap, disabledItemsFocusable, traversalFunction, textCriteria) {\n  let wrappedOnce = false;\n  let nextFocus = traversalFunction(list, currentFocus, currentFocus ? disableListWrap : false);\n  while (nextFocus) {\n    // Prevent infinite loop.\n    if (nextFocus === list.firstChild) {\n      if (wrappedOnce) {\n        return false;\n      }\n      wrappedOnce = true;\n    }\n\n    // Same logic as useAutocomplete.js\n    const nextFocusDisabled = disabledItemsFocusable ? false : nextFocus.disabled || nextFocus.getAttribute('aria-disabled') === 'true';\n    if (!nextFocus.hasAttribute('tabindex') || !textCriteriaMatches(nextFocus, textCriteria) || nextFocusDisabled) {\n      // Move to the next element.\n      nextFocus = traversalFunction(list, nextFocus, disableListWrap);\n    } else {\n      nextFocus.focus();\n      return true;\n    }\n  }\n  return false;\n}\n\n/**\n * A permanently displayed menu following https://www.w3.org/WAI/ARIA/apg/patterns/menu-button/.\n * It's exposed to help customization of the [`Menu`](/material-ui/api/menu/) component if you\n * use it separately you need to move focus into the component manually. Once\n * the focus is placed inside the component it is fully keyboard accessible.\n */\nconst MenuList = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function MenuList(props, ref) {\n  const {\n    // private\n    // eslint-disable-next-line react/prop-types\n    actions,\n    autoFocus = false,\n    autoFocusItem = false,\n    children,\n    className,\n    disabledItemsFocusable = false,\n    disableListWrap = false,\n    onKeyDown,\n    variant = 'selectedMenu',\n    ...other\n  } = props;\n  const listRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const textCriteriaRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef({\n    keys: [],\n    repeating: true,\n    previousKeyMatched: true,\n    lastTime: null\n  });\n  (0,_utils_useEnhancedEffect_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(() => {\n    if (autoFocus) {\n      listRef.current.focus();\n    }\n  }, [autoFocus]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useImperativeHandle(actions, () => ({\n    adjustStyleForScrollbar: (containerElement, {\n      direction\n    }) => {\n      // Let's ignore that piece of logic if users are already overriding the width\n      // of the menu.\n      const noExplicitWidth = !listRef.current.style.width;\n      if (containerElement.clientHeight < listRef.current.clientHeight && noExplicitWidth) {\n        const scrollbarSize = `${(0,_utils_getScrollbarSize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])((0,_utils_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(containerElement))}px`;\n        listRef.current.style[direction === 'rtl' ? 'paddingLeft' : 'paddingRight'] = scrollbarSize;\n        listRef.current.style.width = `calc(100% + ${scrollbarSize})`;\n      }\n      return listRef.current;\n    }\n  }), []);\n  const handleKeyDown = event => {\n    const list = listRef.current;\n    const key = event.key;\n    const isModifierKeyPressed = event.ctrlKey || event.metaKey || event.altKey;\n    if (isModifierKeyPressed) {\n      if (onKeyDown) {\n        onKeyDown(event);\n      }\n      return;\n    }\n\n    /**\n     * @type {Element} - will always be defined since we are in a keydown handler\n     * attached to an element. A keydown event is either dispatched to the activeElement\n     * or document.body or document.documentElement. Only the first case will\n     * trigger this specific handler.\n     */\n    const currentFocus = (0,_utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(list).activeElement;\n    if (key === 'ArrowDown') {\n      // Prevent scroll of the page\n      event.preventDefault();\n      moveFocus(list, currentFocus, disableListWrap, disabledItemsFocusable, nextItem);\n    } else if (key === 'ArrowUp') {\n      event.preventDefault();\n      moveFocus(list, currentFocus, disableListWrap, disabledItemsFocusable, previousItem);\n    } else if (key === 'Home') {\n      event.preventDefault();\n      moveFocus(list, null, disableListWrap, disabledItemsFocusable, nextItem);\n    } else if (key === 'End') {\n      event.preventDefault();\n      moveFocus(list, null, disableListWrap, disabledItemsFocusable, previousItem);\n    } else if (key.length === 1) {\n      const criteria = textCriteriaRef.current;\n      const lowerKey = key.toLowerCase();\n      const currTime = performance.now();\n      if (criteria.keys.length > 0) {\n        // Reset\n        if (currTime - criteria.lastTime > 500) {\n          criteria.keys = [];\n          criteria.repeating = true;\n          criteria.previousKeyMatched = true;\n        } else if (criteria.repeating && lowerKey !== criteria.keys[0]) {\n          criteria.repeating = false;\n        }\n      }\n      criteria.lastTime = currTime;\n      criteria.keys.push(lowerKey);\n      const keepFocusOnCurrent = currentFocus && !criteria.repeating && textCriteriaMatches(currentFocus, criteria);\n      if (criteria.previousKeyMatched && (keepFocusOnCurrent || moveFocus(list, currentFocus, false, disabledItemsFocusable, nextItem, criteria))) {\n        event.preventDefault();\n      } else {\n        criteria.previousKeyMatched = false;\n      }\n    }\n    if (onKeyDown) {\n      onKeyDown(event);\n    }\n  };\n  const handleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(listRef, ref);\n\n  /**\n   * the index of the item should receive focus\n   * in a `variant=\"selectedMenu\"` it's the first `selected` item\n   * otherwise it's the very first item.\n   */\n  let activeItemIndex = -1;\n  // since we inject focus related props into children we have to do a lookahead\n  // to check if there is a `selected` item. We're looking for the last `selected`\n  // item and use the first valid item as a fallback\n  react__WEBPACK_IMPORTED_MODULE_0__.Children.forEach(children, (child, index) => {\n    if (! /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(child)) {\n      if (activeItemIndex === index) {\n        activeItemIndex += 1;\n        if (activeItemIndex >= children.length) {\n          // there are no focusable items within the list.\n          activeItemIndex = -1;\n        }\n      }\n      return;\n    }\n    if (true) {\n      if ((0,react_is__WEBPACK_IMPORTED_MODULE_7__.isFragment)(child)) {\n        console.error([\"MUI: The Menu component doesn't accept a Fragment as a child.\", 'Consider providing an array instead.'].join('\\n'));\n      }\n    }\n    if (!child.props.disabled) {\n      if (variant === 'selectedMenu' && child.props.selected) {\n        activeItemIndex = index;\n      } else if (activeItemIndex === -1) {\n        activeItemIndex = index;\n      }\n    }\n    if (activeItemIndex === index && (child.props.disabled || child.props.muiSkipListHighlight || child.type.muiSkipListHighlight)) {\n      activeItemIndex += 1;\n      if (activeItemIndex >= children.length) {\n        // there are no focusable items within the list.\n        activeItemIndex = -1;\n      }\n    }\n  });\n  const items = react__WEBPACK_IMPORTED_MODULE_0__.Children.map(children, (child, index) => {\n    if (index === activeItemIndex) {\n      const newChildProps = {};\n      if (autoFocusItem) {\n        newChildProps.autoFocus = true;\n      }\n      if (child.props.tabIndex === undefined && variant === 'selectedMenu') {\n        newChildProps.tabIndex = 0;\n      }\n      return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(child, newChildProps);\n    }\n    return child;\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_List_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"], {\n    role: \"menu\",\n    ref: handleRef,\n    className: className,\n    onKeyDown: handleKeyDown,\n    tabIndex: autoFocus ? 0 : -1,\n    ...other,\n    children: items\n  });\n});\n true ? MenuList.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * If `true`, will focus the `[role=\"menu\"]` container and move into tab order.\n   * @default false\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * If `true`, will focus the first menuitem if `variant=\"menu\"` or selected item\n   * if `variant=\"selectedMenu\"`.\n   * @default false\n   */\n  autoFocusItem: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * MenuList contents, normally `MenuItem`s.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * If `true`, will allow focus on disabled items.\n   * @default false\n   */\n  disabledItemsFocusable: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * If `true`, the menu items will not wrap focus.\n   * @default false\n   */\n  disableListWrap: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * @ignore\n   */\n  onKeyDown: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func),\n  /**\n   * The variant to use. Use `menu` to prevent selected items from impacting the initial focus\n   * and the vertical alignment relative to the anchor element.\n   * @default 'selectedMenu'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['menu', 'selectedMenu'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (MenuList);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/MenuList/MenuList.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Modal/Modal.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/Modal/Modal.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_13__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! @mui/utils/HTMLElementType */ \"./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js\");\n/* harmony import */ var _mui_utils_elementAcceptingRef__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/elementAcceptingRef */ \"./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _Unstable_TrapFocus_index_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../Unstable_TrapFocus/index.js */ \"./node_modules/@mui/material/Unstable_TrapFocus/FocusTrap.js\");\n/* harmony import */ var _Portal_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../Portal/index.js */ \"./node_modules/@mui/material/Portal/Portal.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _Backdrop_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Backdrop/index.js */ \"./node_modules/@mui/material/Backdrop/Backdrop.js\");\n/* harmony import */ var _useModal_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./useModal.js */ \"./node_modules/@mui/material/Modal/useModal.js\");\n/* harmony import */ var _modalClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./modalClasses.js */ \"./node_modules/@mui/material/Modal/modalClasses.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    open,\n    exited,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', !open && exited && 'hidden'],\n    backdrop: ['backdrop']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _modalClasses_js__WEBPACK_IMPORTED_MODULE_4__.getModalUtilityClass, classes);\n};\nconst ModalRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiModal',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, !ownerState.open && ownerState.exited && styles.hidden];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(({\n  theme\n}) => ({\n  position: 'fixed',\n  zIndex: (theme.vars || theme).zIndex.modal,\n  right: 0,\n  bottom: 0,\n  top: 0,\n  left: 0,\n  variants: [{\n    props: ({\n      ownerState\n    }) => !ownerState.open && ownerState.exited,\n    style: {\n      visibility: 'hidden'\n    }\n  }]\n})));\nconst ModalBackdrop = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Backdrop_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiModal',\n  slot: 'Backdrop',\n  overridesResolver: (props, styles) => {\n    return styles.backdrop;\n  }\n})({\n  zIndex: -1\n});\n\n/**\n * Modal is a lower-level construct that is leveraged by the following components:\n *\n * - [Dialog](/material-ui/api/dialog/)\n * - [Drawer](/material-ui/api/drawer/)\n * - [Menu](/material-ui/api/menu/)\n * - [Popover](/material-ui/api/popover/)\n *\n * If you are creating a modal dialog, you probably want to use the [Dialog](/material-ui/api/dialog/) component\n * rather than directly using Modal.\n *\n * This component shares many concepts with [react-overlays](https://react-bootstrap.github.io/react-overlays/#modals).\n */\nconst Modal = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Modal(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    name: 'MuiModal',\n    props: inProps\n  });\n  const {\n    BackdropComponent = ModalBackdrop,\n    BackdropProps,\n    classes: classesProp,\n    className,\n    closeAfterTransition = false,\n    children,\n    container,\n    component,\n    components = {},\n    componentsProps = {},\n    disableAutoFocus = false,\n    disableEnforceFocus = false,\n    disableEscapeKeyDown = false,\n    disablePortal = false,\n    disableRestoreFocus = false,\n    disableScrollLock = false,\n    hideBackdrop = false,\n    keepMounted = false,\n    onBackdropClick,\n    onClose,\n    onTransitionEnter,\n    onTransitionExited,\n    open,\n    slotProps = {},\n    slots = {},\n    // eslint-disable-next-line react/prop-types\n    theme,\n    ...other\n  } = props;\n  const propsWithDefaults = {\n    ...props,\n    closeAfterTransition,\n    disableAutoFocus,\n    disableEnforceFocus,\n    disableEscapeKeyDown,\n    disablePortal,\n    disableRestoreFocus,\n    disableScrollLock,\n    hideBackdrop,\n    keepMounted\n  };\n  const {\n    getRootProps,\n    getBackdropProps,\n    getTransitionProps,\n    portalRef,\n    isTopModal,\n    exited,\n    hasTransition\n  } = (0,_useModal_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])({\n    ...propsWithDefaults,\n    rootRef: ref\n  });\n  const ownerState = {\n    ...propsWithDefaults,\n    exited\n  };\n  const classes = useUtilityClasses(ownerState);\n  const childProps = {};\n  if (children.props.tabIndex === undefined) {\n    childProps.tabIndex = '-1';\n  }\n\n  // It's a Transition like component\n  if (hasTransition) {\n    const {\n      onEnter,\n      onExited\n    } = getTransitionProps();\n    childProps.onEnter = onEnter;\n    childProps.onExited = onExited;\n  }\n  const externalForwardedProps = {\n    slots: {\n      root: components.Root,\n      backdrop: components.Backdrop,\n      ...slots\n    },\n    slotProps: {\n      ...componentsProps,\n      ...slotProps\n    }\n  };\n  const [RootSlot, rootProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])('root', {\n    ref,\n    elementType: ModalRoot,\n    externalForwardedProps: {\n      ...externalForwardedProps,\n      ...other,\n      component\n    },\n    getSlotProps: getRootProps,\n    ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(className, classes?.root, !ownerState.open && ownerState.exited && classes?.hidden)\n  });\n  const [BackdropSlot, backdropProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])('backdrop', {\n    ref: BackdropProps?.ref,\n    elementType: BackdropComponent,\n    externalForwardedProps,\n    shouldForwardComponentProp: true,\n    additionalProps: BackdropProps,\n    getSlotProps: otherHandlers => {\n      return getBackdropProps({\n        ...otherHandlers,\n        onClick: event => {\n          if (onBackdropClick) {\n            onBackdropClick(event);\n          }\n          if (otherHandlers?.onClick) {\n            otherHandlers.onClick(event);\n          }\n        }\n      });\n    },\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(BackdropProps?.className, classes?.backdrop),\n    ownerState\n  });\n  if (!keepMounted && !open && (!hasTransition || exited)) {\n    return null;\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_Portal_index_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"], {\n    ref: portalRef,\n    container: container,\n    disablePortal: disablePortal,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(RootSlot, {\n      ...rootProps,\n      children: [!hideBackdrop && BackdropComponent ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(BackdropSlot, {\n        ...backdropProps\n      }) : null, /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_Unstable_TrapFocus_index_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"], {\n        disableEnforceFocus: disableEnforceFocus,\n        disableAutoFocus: disableAutoFocus,\n        disableRestoreFocus: disableRestoreFocus,\n        isEnabled: isTopModal,\n        open: open,\n        children: /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(children, childProps)\n      })]\n    })\n  });\n});\n true ? Modal.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * A backdrop component. This prop enables custom backdrop rendering.\n   * @deprecated Use `slots.backdrop` instead. While this prop currently works, it will be removed in the next major version.\n   * Use the `slots.backdrop` prop to make your application ready for the next version of Material UI.\n   * @default styled(Backdrop, {\n   *   name: 'MuiModal',\n   *   slot: 'Backdrop',\n   *   overridesResolver: (props, styles) => {\n   *     return styles.backdrop;\n   *   },\n   * })({\n   *   zIndex: -1,\n   * })\n   */\n  BackdropComponent: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n  /**\n   * Props applied to the [`Backdrop`](https://mui.com/material-ui/api/backdrop/) element.\n   * @deprecated Use `slotProps.backdrop` instead.\n   */\n  BackdropProps: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * A single child content element.\n   */\n  children: _mui_utils_elementAcceptingRef__WEBPACK_IMPORTED_MODULE_14__[\"default\"].isRequired,\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * When set to true the Modal waits until a nested Transition is completed before closing.\n   * @default false\n   */\n  closeAfterTransition: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated Use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    Backdrop: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    Root: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType)\n  }),\n  /**\n   * The extra props for the slot components.\n   * You can override the existing props or add new ones.\n   *\n   * @deprecated Use the `slotProps` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  componentsProps: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    backdrop: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)])\n  }),\n  /**\n   * An HTML element or function that returns one.\n   * The `container` will have the portal children appended to it.\n   *\n   * You can also provide a callback, which is called in a React layout effect.\n   * This lets you set the container from a ref, and also makes server-side rendering possible.\n   *\n   * By default, it uses the body of the top-level document object,\n   * so it's simply `document.body` most of the time.\n   */\n  container: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([_mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_15__[\"default\"], (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func)]),\n  /**\n   * If `true`, the modal will not automatically shift focus to itself when it opens, and\n   * replace it to the last focused element when it closes.\n   * This also works correctly with any modal children that have the `disableAutoFocus` prop.\n   *\n   * Generally this should never be set to `true` as it makes the modal less\n   * accessible to assistive technologies, like screen readers.\n   * @default false\n   */\n  disableAutoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the modal will not prevent focus from leaving the modal while open.\n   *\n   * Generally this should never be set to `true` as it makes the modal less\n   * accessible to assistive technologies, like screen readers.\n   * @default false\n   */\n  disableEnforceFocus: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, hitting escape will not fire the `onClose` callback.\n   * @default false\n   */\n  disableEscapeKeyDown: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The `children` will be under the DOM hierarchy of the parent component.\n   * @default false\n   */\n  disablePortal: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the modal will not restore focus to previously focused element once\n   * modal is hidden or unmounted.\n   * @default false\n   */\n  disableRestoreFocus: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Disable the scroll lock behavior.\n   * @default false\n   */\n  disableScrollLock: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the backdrop is not rendered.\n   * @default false\n   */\n  hideBackdrop: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Always keep the children in the DOM.\n   * This prop can be useful in SEO situation or\n   * when you want to maximize the responsiveness of the Modal.\n   * @default false\n   */\n  keepMounted: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Callback fired when the backdrop is clicked.\n   * @deprecated Use the `onClose` prop with the `reason` argument to handle the `backdropClick` events.\n   */\n  onBackdropClick: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * Callback fired when the component requests to be closed.\n   * The `reason` parameter can optionally be used to control the response to `onClose`.\n   *\n   * @param {object} event The event source of the callback.\n   * @param {string} reason Can be: `\"escapeKeyDown\"`, `\"backdropClick\"`.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * A function called when a transition enters.\n   */\n  onTransitionEnter: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * A function called when a transition has exited.\n   */\n  onTransitionExited: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * If `true`, the component is shown.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool).isRequired,\n  /**\n   * The props used for each slot inside the Modal.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    backdrop: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)])\n  }),\n  /**\n   * The components used for each slot inside the Modal.\n   * Either a string to use a HTML element or a component.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    backdrop: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Modal);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Modal/Modal.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Modal/ModalManager.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/Modal/ModalManager.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   ModalManager: () => (/* binding */ ModalManager),\n/* harmony export */   ariaHidden: () => (/* binding */ ariaHidden)\n/* harmony export */ });\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/ownerWindow/ownerWindow.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/getScrollbarSize/getScrollbarSize.js\");\n\n// Is a vertical scrollbar displayed?\nfunction isOverflowing(container) {\n  const doc = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(container);\n  if (doc.body === container) {\n    return (0,_mui_utils__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(container).innerWidth > doc.documentElement.clientWidth;\n  }\n  return container.scrollHeight > container.clientHeight;\n}\nfunction ariaHidden(element, hide) {\n  if (hide) {\n    element.setAttribute('aria-hidden', 'true');\n  } else {\n    element.removeAttribute('aria-hidden');\n  }\n}\nfunction getPaddingRight(element) {\n  return parseInt((0,_mui_utils__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(element).getComputedStyle(element).paddingRight, 10) || 0;\n}\nfunction isAriaHiddenForbiddenOnElement(element) {\n  // The forbidden HTML tags are the ones from ARIA specification that\n  // can be children of body and can't have aria-hidden attribute.\n  // cf. https://www.w3.org/TR/html-aria/#docconformance\n  const forbiddenTagNames = ['TEMPLATE', 'SCRIPT', 'STYLE', 'LINK', 'MAP', 'META', 'NOSCRIPT', 'PICTURE', 'COL', 'COLGROUP', 'PARAM', 'SLOT', 'SOURCE', 'TRACK'];\n  const isForbiddenTagName = forbiddenTagNames.includes(element.tagName);\n  const isInputHidden = element.tagName === 'INPUT' && element.getAttribute('type') === 'hidden';\n  return isForbiddenTagName || isInputHidden;\n}\nfunction ariaHiddenSiblings(container, mountElement, currentElement, elementsToExclude, hide) {\n  const blacklist = [mountElement, currentElement, ...elementsToExclude];\n  [].forEach.call(container.children, element => {\n    const isNotExcludedElement = !blacklist.includes(element);\n    const isNotForbiddenElement = !isAriaHiddenForbiddenOnElement(element);\n    if (isNotExcludedElement && isNotForbiddenElement) {\n      ariaHidden(element, hide);\n    }\n  });\n}\nfunction findIndexOf(items, callback) {\n  let idx = -1;\n  items.some((item, index) => {\n    if (callback(item)) {\n      idx = index;\n      return true;\n    }\n    return false;\n  });\n  return idx;\n}\nfunction handleContainer(containerInfo, props) {\n  const restoreStyle = [];\n  const container = containerInfo.container;\n  if (!props.disableScrollLock) {\n    if (isOverflowing(container)) {\n      // Compute the size before applying overflow hidden to avoid any scroll jumps.\n      const scrollbarSize = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_2__[\"default\"])((0,_mui_utils__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(container));\n      restoreStyle.push({\n        value: container.style.paddingRight,\n        property: 'padding-right',\n        el: container\n      });\n      // Use computed style, here to get the real padding to add our scrollbar width.\n      container.style.paddingRight = `${getPaddingRight(container) + scrollbarSize}px`;\n\n      // .mui-fixed is a global helper.\n      const fixedElements = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(container).querySelectorAll('.mui-fixed');\n      [].forEach.call(fixedElements, element => {\n        restoreStyle.push({\n          value: element.style.paddingRight,\n          property: 'padding-right',\n          el: element\n        });\n        element.style.paddingRight = `${getPaddingRight(element) + scrollbarSize}px`;\n      });\n    }\n    let scrollContainer;\n    if (container.parentNode instanceof DocumentFragment) {\n      scrollContainer = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(container).body;\n    } else {\n      // Support html overflow-y: auto for scroll stability between pages\n      // https://css-tricks.com/snippets/css/force-vertical-scrollbar/\n      const parent = container.parentElement;\n      const containerWindow = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(container);\n      scrollContainer = parent?.nodeName === 'HTML' && containerWindow.getComputedStyle(parent).overflowY === 'scroll' ? parent : container;\n    }\n\n    // Block the scroll even if no scrollbar is visible to account for mobile keyboard\n    // screensize shrink.\n    restoreStyle.push({\n      value: scrollContainer.style.overflow,\n      property: 'overflow',\n      el: scrollContainer\n    }, {\n      value: scrollContainer.style.overflowX,\n      property: 'overflow-x',\n      el: scrollContainer\n    }, {\n      value: scrollContainer.style.overflowY,\n      property: 'overflow-y',\n      el: scrollContainer\n    });\n    scrollContainer.style.overflow = 'hidden';\n  }\n  const restore = () => {\n    restoreStyle.forEach(({\n      value,\n      el,\n      property\n    }) => {\n      if (value) {\n        el.style.setProperty(property, value);\n      } else {\n        el.style.removeProperty(property);\n      }\n    });\n  };\n  return restore;\n}\nfunction getHiddenSiblings(container) {\n  const hiddenSiblings = [];\n  [].forEach.call(container.children, element => {\n    if (element.getAttribute('aria-hidden') === 'true') {\n      hiddenSiblings.push(element);\n    }\n  });\n  return hiddenSiblings;\n}\n/**\n * @ignore - do not document.\n *\n * Proper state management for containers and the modals in those containers.\n * Simplified, but inspired by react-overlay's ModalManager class.\n * Used by the Modal to ensure proper styling of containers.\n */\nclass ModalManager {\n  constructor() {\n    this.modals = [];\n    this.containers = [];\n  }\n  add(modal, container) {\n    let modalIndex = this.modals.indexOf(modal);\n    if (modalIndex !== -1) {\n      return modalIndex;\n    }\n    modalIndex = this.modals.length;\n    this.modals.push(modal);\n\n    // If the modal we are adding is already in the DOM.\n    if (modal.modalRef) {\n      ariaHidden(modal.modalRef, false);\n    }\n    const hiddenSiblings = getHiddenSiblings(container);\n    ariaHiddenSiblings(container, modal.mount, modal.modalRef, hiddenSiblings, true);\n    const containerIndex = findIndexOf(this.containers, item => item.container === container);\n    if (containerIndex !== -1) {\n      this.containers[containerIndex].modals.push(modal);\n      return modalIndex;\n    }\n    this.containers.push({\n      modals: [modal],\n      container,\n      restore: null,\n      hiddenSiblings\n    });\n    return modalIndex;\n  }\n  mount(modal, props) {\n    const containerIndex = findIndexOf(this.containers, item => item.modals.includes(modal));\n    const containerInfo = this.containers[containerIndex];\n    if (!containerInfo.restore) {\n      containerInfo.restore = handleContainer(containerInfo, props);\n    }\n  }\n  remove(modal, ariaHiddenState = true) {\n    const modalIndex = this.modals.indexOf(modal);\n    if (modalIndex === -1) {\n      return modalIndex;\n    }\n    const containerIndex = findIndexOf(this.containers, item => item.modals.includes(modal));\n    const containerInfo = this.containers[containerIndex];\n    containerInfo.modals.splice(containerInfo.modals.indexOf(modal), 1);\n    this.modals.splice(modalIndex, 1);\n\n    // If that was the last modal in a container, clean up the container.\n    if (containerInfo.modals.length === 0) {\n      // The modal might be closed before it had the chance to be mounted in the DOM.\n      if (containerInfo.restore) {\n        containerInfo.restore();\n      }\n      if (modal.modalRef) {\n        // In case the modal wasn't in the DOM yet.\n        ariaHidden(modal.modalRef, ariaHiddenState);\n      }\n      ariaHiddenSiblings(containerInfo.container, modal.mount, modal.modalRef, containerInfo.hiddenSiblings, false);\n      this.containers.splice(containerIndex, 1);\n    } else {\n      // Otherwise make sure the next top modal is visible to a screen reader.\n      const nextTop = containerInfo.modals[containerInfo.modals.length - 1];\n      // as soon as a modal is adding its modalRef is undefined. it can't set\n      // aria-hidden because the dom element doesn't exist either\n      // when modal was unmounted before modalRef gets null\n      if (nextTop.modalRef) {\n        ariaHidden(nextTop.modalRef, false);\n      }\n    }\n    return modalIndex;\n  }\n  isTopModal(modal) {\n    return this.modals.length > 0 && this.modals[this.modals.length - 1] === modal;\n  }\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Modal/ModalManager.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Modal/modalClasses.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/Modal/modalClasses.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getModalUtilityClass: () => (/* binding */ getModalUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getModalUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiModal', slot);\n}\nconst modalClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiModal', ['root', 'hidden', 'backdrop']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (modalClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Modal/modalClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Modal/useModal.js":
/*!******************************************************!*\
  !*** ./node_modules/@mui/material/Modal/useModal.js ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useEventCallback/useEventCallback.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/createChainedFunction/createChainedFunction.js\");\n/* harmony import */ var _mui_utils_extractEventHandlers__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/extractEventHandlers */ \"./node_modules/@mui/utils/esm/extractEventHandlers/extractEventHandlers.js\");\n/* harmony import */ var _ModalManager_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./ModalManager.js */ \"./node_modules/@mui/material/Modal/ModalManager.js\");\n'use client';\n\n\n\n\n\nfunction getContainer(container) {\n  return typeof container === 'function' ? container() : container;\n}\nfunction getHasTransition(children) {\n  return children ? children.props.hasOwnProperty('in') : false;\n}\nconst noop = () => {};\n\n// A modal manager used to track and manage the state of open Modals.\n// Modals don't open on the server so this won't conflict with concurrent requests.\nconst manager = new _ModalManager_js__WEBPACK_IMPORTED_MODULE_1__.ModalManager();\n/**\n *\n * Demos:\n *\n * - [Modal](https://mui.com/base-ui/react-modal/#hook)\n *\n * API:\n *\n * - [useModal API](https://mui.com/base-ui/react-modal/hooks-api/#use-modal)\n */\nfunction useModal(parameters) {\n  const {\n    container,\n    disableEscapeKeyDown = false,\n    disableScrollLock = false,\n    closeAfterTransition = false,\n    onTransitionEnter,\n    onTransitionExited,\n    children,\n    onClose,\n    open,\n    rootRef\n  } = parameters;\n\n  // @ts-ignore internal logic\n  const modal = react__WEBPACK_IMPORTED_MODULE_0__.useRef({});\n  const mountNodeRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const modalRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const handleRef = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(modalRef, rootRef);\n  const [exited, setExited] = react__WEBPACK_IMPORTED_MODULE_0__.useState(!open);\n  const hasTransition = getHasTransition(children);\n  let ariaHiddenProp = true;\n  if (parameters['aria-hidden'] === 'false' || parameters['aria-hidden'] === false) {\n    ariaHiddenProp = false;\n  }\n  const getDoc = () => (0,_mui_utils__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(mountNodeRef.current);\n  const getModal = () => {\n    modal.current.modalRef = modalRef.current;\n    modal.current.mount = mountNodeRef.current;\n    return modal.current;\n  };\n  const handleMounted = () => {\n    manager.mount(getModal(), {\n      disableScrollLock\n    });\n\n    // Fix a bug on Chrome where the scroll isn't initially 0.\n    if (modalRef.current) {\n      modalRef.current.scrollTop = 0;\n    }\n  };\n  const handleOpen = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(() => {\n    const resolvedContainer = getContainer(container) || getDoc().body;\n    manager.add(getModal(), resolvedContainer);\n\n    // The element was already mounted.\n    if (modalRef.current) {\n      handleMounted();\n    }\n  });\n  const isTopModal = () => manager.isTopModal(getModal());\n  const handlePortalRef = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(node => {\n    mountNodeRef.current = node;\n    if (!node) {\n      return;\n    }\n    if (open && isTopModal()) {\n      handleMounted();\n    } else if (modalRef.current) {\n      (0,_ModalManager_js__WEBPACK_IMPORTED_MODULE_1__.ariaHidden)(modalRef.current, ariaHiddenProp);\n    }\n  });\n  const handleClose = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    manager.remove(getModal(), ariaHiddenProp);\n  }, [ariaHiddenProp]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    return () => {\n      handleClose();\n    };\n  }, [handleClose]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (open) {\n      handleOpen();\n    } else if (!hasTransition || !closeAfterTransition) {\n      handleClose();\n    }\n  }, [open, handleClose, hasTransition, closeAfterTransition, handleOpen]);\n  const createHandleKeyDown = otherHandlers => event => {\n    otherHandlers.onKeyDown?.(event);\n\n    // The handler doesn't take event.defaultPrevented into account:\n    //\n    // event.preventDefault() is meant to stop default behaviors like\n    // clicking a checkbox to check it, hitting a button to submit a form,\n    // and hitting left arrow to move the cursor in a text input etc.\n    // Only special HTML elements have these default behaviors.\n    if (event.key !== 'Escape' || event.which === 229 ||\n    // Wait until IME is settled.\n    !isTopModal()) {\n      return;\n    }\n    if (!disableEscapeKeyDown) {\n      // Swallow the event, in case someone is listening for the escape key on the body.\n      event.stopPropagation();\n      if (onClose) {\n        onClose(event, 'escapeKeyDown');\n      }\n    }\n  };\n  const createHandleBackdropClick = otherHandlers => event => {\n    otherHandlers.onClick?.(event);\n    if (event.target !== event.currentTarget) {\n      return;\n    }\n    if (onClose) {\n      onClose(event, 'backdropClick');\n    }\n  };\n  const getRootProps = (otherHandlers = {}) => {\n    const propsEventHandlers = (0,_mui_utils_extractEventHandlers__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(parameters);\n\n    // The custom event handlers shouldn't be spread on the root element\n    delete propsEventHandlers.onTransitionEnter;\n    delete propsEventHandlers.onTransitionExited;\n    const externalEventHandlers = {\n      ...propsEventHandlers,\n      ...otherHandlers\n    };\n    return {\n      /*\n       * Marking an element with the role presentation indicates to assistive technology\n       * that this element should be ignored; it exists to support the web application and\n       * is not meant for humans to interact with directly.\n       * https://github.com/evcohen/eslint-plugin-jsx-a11y/blob/master/docs/rules/no-static-element-interactions.md\n       */\n      role: 'presentation',\n      ...externalEventHandlers,\n      onKeyDown: createHandleKeyDown(externalEventHandlers),\n      ref: handleRef\n    };\n  };\n  const getBackdropProps = (otherHandlers = {}) => {\n    const externalEventHandlers = otherHandlers;\n    return {\n      'aria-hidden': true,\n      ...externalEventHandlers,\n      onClick: createHandleBackdropClick(externalEventHandlers),\n      open\n    };\n  };\n  const getTransitionProps = () => {\n    const handleEnter = () => {\n      setExited(false);\n      if (onTransitionEnter) {\n        onTransitionEnter();\n      }\n    };\n    const handleExited = () => {\n      setExited(true);\n      if (onTransitionExited) {\n        onTransitionExited();\n      }\n      if (closeAfterTransition) {\n        handleClose();\n      }\n    };\n    return {\n      onEnter: (0,_mui_utils__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(handleEnter, children?.props.onEnter ?? noop),\n      onExited: (0,_mui_utils__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(handleExited, children?.props.onExited ?? noop)\n    };\n  };\n  return {\n    getRootProps,\n    getBackdropProps,\n    getTransitionProps,\n    rootRef: handleRef,\n    portalRef: handlePortalRef,\n    isTopModal,\n    exited,\n    hasTransition\n  };\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (useModal);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Modal/useModal.js?");

/***/ }),

/***/ "./node_modules/@mui/material/NativeSelect/NativeSelectInput.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/NativeSelect/NativeSelectInput.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   StyledSelectIcon: () => (/* binding */ StyledSelectIcon),\n/* harmony export */   StyledSelectSelect: () => (/* binding */ StyledSelectSelect),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _nativeSelectClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./nativeSelectClasses.js */ \"./node_modules/@mui/material/NativeSelect/nativeSelectClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    variant,\n    disabled,\n    multiple,\n    open,\n    error\n  } = ownerState;\n  const slots = {\n    select: ['select', variant, disabled && 'disabled', multiple && 'multiple', error && 'error'],\n    icon: ['icon', `icon${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(variant)}`, open && 'iconOpen', disabled && 'disabled']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _nativeSelectClasses_js__WEBPACK_IMPORTED_MODULE_5__.getNativeSelectUtilityClasses, classes);\n};\nconst StyledSelectSelect = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('select')(({\n  theme\n}) => ({\n  // Reset\n  MozAppearance: 'none',\n  // Reset\n  WebkitAppearance: 'none',\n  // When interacting quickly, the text can end up selected.\n  // Native select can't be selected either.\n  userSelect: 'none',\n  // Reset\n  borderRadius: 0,\n  cursor: 'pointer',\n  '&:focus': {\n    // Reset Chrome style\n    borderRadius: 0\n  },\n  [`&.${_nativeSelectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    cursor: 'default'\n  },\n  '&[multiple]': {\n    height: 'auto'\n  },\n  '&:not([multiple]) option, &:not([multiple]) optgroup': {\n    backgroundColor: (theme.vars || theme).palette.background.paper\n  },\n  variants: [{\n    props: ({\n      ownerState\n    }) => ownerState.variant !== 'filled' && ownerState.variant !== 'outlined',\n    style: {\n      // Bump specificity to allow extending custom inputs\n      '&&&': {\n        paddingRight: 24,\n        minWidth: 16 // So it doesn't collapse.\n      }\n    }\n  }, {\n    props: {\n      variant: 'filled'\n    },\n    style: {\n      '&&&': {\n        paddingRight: 32\n      }\n    }\n  }, {\n    props: {\n      variant: 'outlined'\n    },\n    style: {\n      borderRadius: (theme.vars || theme).shape.borderRadius,\n      '&:focus': {\n        borderRadius: (theme.vars || theme).shape.borderRadius // Reset the reset for Chrome style\n      },\n      '&&&': {\n        paddingRight: 32\n      }\n    }\n  }]\n}));\nconst NativeSelectSelect = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(StyledSelectSelect, {\n  name: 'MuiNativeSelect',\n  slot: 'Select',\n  shouldForwardProp: _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"],\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.select, styles[ownerState.variant], ownerState.error && styles.error, {\n      [`&.${_nativeSelectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].multiple}`]: styles.multiple\n    }];\n  }\n})({});\nconst StyledSelectIcon = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('svg')(({\n  theme\n}) => ({\n  // We use a position absolute over a flexbox in order to forward the pointer events\n  // to the input and to support wrapping tags..\n  position: 'absolute',\n  right: 0,\n  // Center vertically, height is 1em\n  top: 'calc(50% - .5em)',\n  // Don't block pointer events on the select under the icon.\n  pointerEvents: 'none',\n  color: (theme.vars || theme).palette.action.active,\n  [`&.${_nativeSelectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    color: (theme.vars || theme).palette.action.disabled\n  },\n  variants: [{\n    props: ({\n      ownerState\n    }) => ownerState.open,\n    style: {\n      transform: 'rotate(180deg)'\n    }\n  }, {\n    props: {\n      variant: 'filled'\n    },\n    style: {\n      right: 7\n    }\n  }, {\n    props: {\n      variant: 'outlined'\n    },\n    style: {\n      right: 7\n    }\n  }]\n}));\nconst NativeSelectIcon = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(StyledSelectIcon, {\n  name: 'MuiNativeSelect',\n  slot: 'Icon',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.icon, ownerState.variant && styles[`icon${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.variant)}`], ownerState.open && styles.iconOpen];\n  }\n})({});\n\n/**\n * @ignore - internal component.\n */\nconst NativeSelectInput = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function NativeSelectInput(props, ref) {\n  const {\n    className,\n    disabled,\n    error,\n    IconComponent,\n    inputRef,\n    variant = 'standard',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    disabled,\n    variant,\n    error\n  };\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(NativeSelectSelect, {\n      ownerState: ownerState,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.select, className),\n      disabled: disabled,\n      ref: inputRef || ref,\n      ...other\n    }), props.multiple ? null : /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(NativeSelectIcon, {\n      as: IconComponent,\n      ownerState: ownerState,\n      className: classes.icon\n    })]\n  });\n});\n true ? NativeSelectInput.propTypes = {\n  /**\n   * The option elements to populate the select with.\n   * Can be some `<option>` elements.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object),\n  /**\n   * The CSS class name of the select element.\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n  /**\n   * If `true`, the select is disabled.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * If `true`, the `select input` will indicate an error.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * The icon that displays the arrow.\n   */\n  IconComponent: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().elementType).isRequired,\n  /**\n   * Use that prop to pass a ref to the native select element.\n   * @deprecated\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_9__[\"default\"],\n  /**\n   * @ignore\n   */\n  multiple: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n  /**\n   * Name attribute of the `select` or hidden `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n  /**\n   * Callback fired when a menu item is selected.\n   *\n   * @param {object} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n  /**\n   * The input value.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().any),\n  /**\n   * The variant to use.\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOf(['standard', 'outlined', 'filled'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (NativeSelectInput);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/NativeSelect/NativeSelectInput.js?");

/***/ }),

/***/ "./node_modules/@mui/material/NativeSelect/nativeSelectClasses.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/material/NativeSelect/nativeSelectClasses.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getNativeSelectUtilityClasses: () => (/* binding */ getNativeSelectUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getNativeSelectUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiNativeSelect', slot);\n}\nconst nativeSelectClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiNativeSelect', ['root', 'select', 'multiple', 'filled', 'outlined', 'standard', 'disabled', 'icon', 'iconOpen', 'iconFilled', 'iconOutlined', 'iconStandard', 'nativeInput', 'error']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (nativeSelectClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/NativeSelect/nativeSelectClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/OutlinedInput/NotchedOutline.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/OutlinedInput/NotchedOutline.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ NotchedOutline)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\nvar _span;\n\n\n\n\n\n\nconst NotchedOutlineRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])('fieldset', {\n  shouldForwardProp: _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]\n})({\n  textAlign: 'left',\n  position: 'absolute',\n  bottom: 0,\n  right: 0,\n  top: -5,\n  left: 0,\n  margin: 0,\n  padding: '0 8px',\n  pointerEvents: 'none',\n  borderRadius: 'inherit',\n  borderStyle: 'solid',\n  borderWidth: 1,\n  overflow: 'hidden',\n  minWidth: '0%'\n});\nconst NotchedOutlineLegend = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])('legend', {\n  shouldForwardProp: _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(({\n  theme\n}) => ({\n  float: 'unset',\n  // Fix conflict with bootstrap\n  width: 'auto',\n  // Fix conflict with bootstrap\n  overflow: 'hidden',\n  // Fix Horizontal scroll when label too long\n  variants: [{\n    props: ({\n      ownerState\n    }) => !ownerState.withLabel,\n    style: {\n      padding: 0,\n      lineHeight: '11px',\n      // sync with `height` in `legend` styles\n      transition: theme.transitions.create('width', {\n        duration: 150,\n        easing: theme.transitions.easing.easeOut\n      })\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.withLabel,\n    style: {\n      display: 'block',\n      // Fix conflict with normalize.css and sanitize.css\n      padding: 0,\n      height: 11,\n      // sync with `lineHeight` in `legend` styles\n      fontSize: '0.75em',\n      visibility: 'hidden',\n      maxWidth: 0.01,\n      transition: theme.transitions.create('max-width', {\n        duration: 50,\n        easing: theme.transitions.easing.easeOut\n      }),\n      whiteSpace: 'nowrap',\n      '& > span': {\n        paddingLeft: 5,\n        paddingRight: 5,\n        display: 'inline-block',\n        opacity: 0,\n        visibility: 'visible'\n      }\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.withLabel && ownerState.notched,\n    style: {\n      maxWidth: '100%',\n      transition: theme.transitions.create('max-width', {\n        duration: 100,\n        easing: theme.transitions.easing.easeOut,\n        delay: 50\n      })\n    }\n  }]\n})));\n\n/**\n * @ignore - internal component.\n */\nfunction NotchedOutline(props) {\n  const {\n    children,\n    classes,\n    className,\n    label,\n    notched,\n    ...other\n  } = props;\n  const withLabel = label != null && label !== '';\n  const ownerState = {\n    ...props,\n    notched,\n    withLabel\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(NotchedOutlineRoot, {\n    \"aria-hidden\": true,\n    className: className,\n    ownerState: ownerState,\n    ...other,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(NotchedOutlineLegend, {\n      ownerState: ownerState,\n      children: withLabel ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"span\", {\n        children: label\n      }) : // notranslate needed while Google Translate will not fix zero-width space issue\n      _span || (_span = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"span\", {\n        className: \"notranslate\",\n        \"aria-hidden\": true,\n        children: \"\\u200B\"\n      }))\n    })\n  });\n}\n true ? NotchedOutline.propTypes /* remove-proptypes */ = {\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().string),\n  /**\n   * The label.\n   */\n  label: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().node),\n  /**\n   * If `true`, the outline is notched to accommodate the label.\n   */\n  notched: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().bool).isRequired,\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().object)\n} : 0;\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/OutlinedInput/NotchedOutline.js?");

/***/ }),

/***/ "./node_modules/@mui/material/OutlinedInput/OutlinedInput.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@mui/material/OutlinedInput/OutlinedInput.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_13__);\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _NotchedOutline_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./NotchedOutline.js */ \"./node_modules/@mui/material/OutlinedInput/NotchedOutline.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./outlinedInputClasses.js */ \"./node_modules/@mui/material/OutlinedInput/outlinedInputClasses.js\");\n/* harmony import */ var _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../InputBase/InputBase.js */ \"./node_modules/@mui/material/InputBase/InputBase.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root'],\n    notchedOutline: ['notchedOutline'],\n    input: ['input']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(slots, _outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__.getOutlinedInputUtilityClass, classes);\n  return {\n    ...classes,\n    // forward classes to the InputBase\n    ...composedClasses\n  };\n};\nconst OutlinedInputRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.InputBaseRoot, {\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(prop) || prop === 'classes',\n  name: 'MuiOutlinedInput',\n  slot: 'Root',\n  overridesResolver: _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.rootOverridesResolver\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => {\n  const borderColor = theme.palette.mode === 'light' ? 'rgba(0, 0, 0, 0.23)' : 'rgba(255, 255, 255, 0.23)';\n  return {\n    position: 'relative',\n    borderRadius: (theme.vars || theme).shape.borderRadius,\n    [`&:hover .${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].notchedOutline}`]: {\n      borderColor: (theme.vars || theme).palette.text.primary\n    },\n    // Reset on touch devices, it doesn't add specificity\n    '@media (hover: none)': {\n      [`&:hover .${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].notchedOutline}`]: {\n        borderColor: theme.vars ? `rgba(${theme.vars.palette.common.onBackgroundChannel} / 0.23)` : borderColor\n      }\n    },\n    [`&.${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].focused} .${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].notchedOutline}`]: {\n      borderWidth: 2\n    },\n    variants: [...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])()).map(([color]) => ({\n      props: {\n        color\n      },\n      style: {\n        [`&.${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].focused} .${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].notchedOutline}`]: {\n          borderColor: (theme.vars || theme).palette[color].main\n        }\n      }\n    })), {\n      props: {},\n      // to overide the above style\n      style: {\n        [`&.${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].error} .${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].notchedOutline}`]: {\n          borderColor: (theme.vars || theme).palette.error.main\n        },\n        [`&.${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].disabled} .${_outlinedInputClasses_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"].notchedOutline}`]: {\n          borderColor: (theme.vars || theme).palette.action.disabled\n        }\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.startAdornment,\n      style: {\n        paddingLeft: 14\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.endAdornment,\n      style: {\n        paddingRight: 14\n      }\n    }, {\n      props: ({\n        ownerState\n      }) => ownerState.multiline,\n      style: {\n        padding: '16.5px 14px'\n      }\n    }, {\n      props: ({\n        ownerState,\n        size\n      }) => ownerState.multiline && size === 'small',\n      style: {\n        padding: '8.5px 14px'\n      }\n    }]\n  };\n}));\nconst NotchedOutlineRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(_NotchedOutline_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"], {\n  name: 'MuiOutlinedInput',\n  slot: 'NotchedOutline',\n  overridesResolver: (props, styles) => styles.notchedOutline\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => {\n  const borderColor = theme.palette.mode === 'light' ? 'rgba(0, 0, 0, 0.23)' : 'rgba(255, 255, 255, 0.23)';\n  return {\n    borderColor: theme.vars ? `rgba(${theme.vars.palette.common.onBackgroundChannel} / 0.23)` : borderColor\n  };\n}));\nconst OutlinedInputInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.InputBaseInput, {\n  name: 'MuiOutlinedInput',\n  slot: 'Input',\n  overridesResolver: _InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__.inputOverridesResolver\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  padding: '16.5px 14px',\n  ...(!theme.vars && {\n    '&:-webkit-autofill': {\n      WebkitBoxShadow: theme.palette.mode === 'light' ? null : '0 0 0 100px #266798 inset',\n      WebkitTextFillColor: theme.palette.mode === 'light' ? null : '#fff',\n      caretColor: theme.palette.mode === 'light' ? null : '#fff',\n      borderRadius: 'inherit'\n    }\n  }),\n  ...(theme.vars && {\n    '&:-webkit-autofill': {\n      borderRadius: 'inherit'\n    },\n    [theme.getColorSchemeSelector('dark')]: {\n      '&:-webkit-autofill': {\n        WebkitBoxShadow: '0 0 0 100px #266798 inset',\n        WebkitTextFillColor: '#fff',\n        caretColor: '#fff'\n      }\n    }\n  }),\n  variants: [{\n    props: {\n      size: 'small'\n    },\n    style: {\n      padding: '8.5px 14px'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.multiline,\n    style: {\n      padding: 0\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.startAdornment,\n    style: {\n      paddingLeft: 0\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.endAdornment,\n    style: {\n      paddingRight: 0\n    }\n  }]\n})));\nconst OutlinedInput = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function OutlinedInput(inProps, ref) {\n  var _React$Fragment;\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiOutlinedInput'\n  });\n  const {\n    components = {},\n    fullWidth = false,\n    inputComponent = 'input',\n    label,\n    multiline = false,\n    notched,\n    slots = {},\n    type = 'text',\n    ...other\n  } = props;\n  const classes = useUtilityClasses(props);\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])();\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['color', 'disabled', 'error', 'focused', 'hiddenLabel', 'size', 'required']\n  });\n  const ownerState = {\n    ...props,\n    color: fcs.color || 'primary',\n    disabled: fcs.disabled,\n    error: fcs.error,\n    focused: fcs.focused,\n    formControl: muiFormControl,\n    fullWidth,\n    hiddenLabel: fcs.hiddenLabel,\n    multiline,\n    size: fcs.size,\n    type\n  };\n  const RootSlot = slots.root ?? components.Root ?? OutlinedInputRoot;\n  const InputSlot = slots.input ?? components.Input ?? OutlinedInputInput;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_InputBase_InputBase_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"], {\n    slots: {\n      root: RootSlot,\n      input: InputSlot\n    },\n    renderSuffix: state => /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(NotchedOutlineRoot, {\n      ownerState: ownerState,\n      className: classes.notchedOutline,\n      label: label != null && label !== '' && fcs.required ? _React$Fragment || (_React$Fragment = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n        children: [label, \"\\u2009\", '*']\n      })) : label,\n      notched: typeof notched !== 'undefined' ? notched : Boolean(state.startAdornment || state.filled || state.focused)\n    }),\n    fullWidth: fullWidth,\n    inputComponent: inputComponent,\n    multiline: multiline,\n    ref: ref,\n    type: type,\n    ...other,\n    classes: {\n      ...classes,\n      notchedOutline: null\n    }\n  });\n});\n true ? OutlinedInput.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * This prop helps users to fill forms faster, especially on mobile devices.\n   * The name can be confusing, as it's more like an autofill.\n   * You can learn more about it [following the specification](https://html.spec.whatwg.org/multipage/form-control-infrastructure.html#autofill).\n   */\n  autoComplete: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * If `true`, the `input` element is focused during the first mount.\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * The prop defaults to the value (`'primary'`) inherited from the parent FormControl component.\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['primary', 'secondary']), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)]),\n  /**\n   * The components used for each slot inside.\n   *\n   * @deprecated use the `slots` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   *\n   * @default {}\n   */\n  components: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    Input: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    Root: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType)\n  }),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().any),\n  /**\n   * If `true`, the component is disabled.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * End `InputAdornment` for this component.\n   */\n  endAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * If `true`, the `input` will indicate an error.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the `input` will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The id of the `input` element.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * The component used for the `input` element.\n   * Either a string to use a HTML element or a component.\n   * @default 'input'\n   */\n  inputComponent: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * @default {}\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_14__[\"default\"],\n  /**\n   * The label of the `input`. It is only used for layout. The actual labelling\n   * is handled by `InputLabel`.\n   */\n  label: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * If `dense`, will adjust vertical spacing. This is normally obtained via context from\n   * FormControl.\n   * The prop defaults to the value (`'none'`) inherited from the parent FormControl component.\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['dense', 'none']),\n  /**\n   * Maximum number of rows to display when multiline option is set to true.\n   */\n  maxRows: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)]),\n  /**\n   * Minimum number of rows to display when multiline option is set to true.\n   */\n  minRows: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)]),\n  /**\n   * If `true`, a [TextareaAutosize](https://mui.com/material-ui/react-textarea-autosize/) element is rendered.\n   * @default false\n   */\n  multiline: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Name attribute of the `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * If `true`, the outline is notched to accommodate the label.\n   */\n  notched: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Callback fired when the value is changed.\n   *\n   * @param {React.ChangeEvent<HTMLTextAreaElement | HTMLInputElement>} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * The short hint displayed in the `input` before the user enters a value.\n   */\n  placeholder: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * It prevents the user from changing the value of the field\n   * (not from interacting with the field).\n   */\n  readOnly: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the `input` element is required.\n   * The prop defaults to the value (`false`) inherited from the parent FormControl component.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * Number of rows to display when multiline option is set to true.\n   */\n  rows: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)]),\n  /**\n   * The components used for each slot inside.\n   *\n   * This prop is an alias for the `components` prop, which will be deprecated in the future.\n   *\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType)\n  }),\n  /**\n   * Start `InputAdornment` for this component.\n   */\n  startAdornment: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n  /**\n   * Type of the `input` element. It should be [a valid HTML5 input type](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Form_%3Cinput%3E_types).\n   * @default 'text'\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * The value of the `input` element, required for a controlled component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().any)\n} : 0;\nOutlinedInput.muiName = 'Input';\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (OutlinedInput);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/OutlinedInput/OutlinedInput.js?");

/***/ }),

/***/ "./node_modules/@mui/material/OutlinedInput/outlinedInputClasses.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/material/OutlinedInput/outlinedInputClasses.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getOutlinedInputUtilityClass: () => (/* binding */ getOutlinedInputUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n/* harmony import */ var _InputBase_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../InputBase/index.js */ \"./node_modules/@mui/material/InputBase/inputBaseClasses.js\");\n\n\n\nfunction getOutlinedInputUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiOutlinedInput', slot);\n}\nconst outlinedInputClasses = {\n  ..._InputBase_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"],\n  ...(0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_2__[\"default\"])('MuiOutlinedInput', ['root', 'notchedOutline', 'input'])\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (outlinedInputClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/OutlinedInput/outlinedInputClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Paper/Paper.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/Paper/Paper.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_integerPropType__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! @mui/utils/integerPropType */ \"./node_modules/@mui/utils/esm/integerPropType/integerPropType.js\");\n/* harmony import */ var _mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/utils/chainPropTypes */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/useTheme.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _styles_getOverlayAlpha_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../styles/getOverlayAlpha.js */ \"./node_modules/@mui/material/styles/getOverlayAlpha.js\");\n/* harmony import */ var _paperClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./paperClasses.js */ \"./node_modules/@mui/material/Paper/paperClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    square,\n    elevation,\n    variant,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', variant, !square && 'rounded', variant === 'elevation' && `elevation${elevation}`]\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _paperClasses_js__WEBPACK_IMPORTED_MODULE_4__.getPaperUtilityClass, classes);\n};\nconst PaperRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('div', {\n  name: 'MuiPaper',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[ownerState.variant], !ownerState.square && styles.rounded, ownerState.variant === 'elevation' && styles[`elevation${ownerState.elevation}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(({\n  theme\n}) => ({\n  backgroundColor: (theme.vars || theme).palette.background.paper,\n  color: (theme.vars || theme).palette.text.primary,\n  transition: theme.transitions.create('box-shadow'),\n  variants: [{\n    props: ({\n      ownerState\n    }) => !ownerState.square,\n    style: {\n      borderRadius: theme.shape.borderRadius\n    }\n  }, {\n    props: {\n      variant: 'outlined'\n    },\n    style: {\n      border: `1px solid ${(theme.vars || theme).palette.divider}`\n    }\n  }, {\n    props: {\n      variant: 'elevation'\n    },\n    style: {\n      boxShadow: 'var(--Paper-shadow)',\n      backgroundImage: 'var(--Paper-overlay)'\n    }\n  }]\n})));\nconst Paper = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Paper(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiPaper'\n  });\n  const theme = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])();\n  const {\n    className,\n    component = 'div',\n    elevation = 1,\n    square = false,\n    variant = 'elevation',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    component,\n    elevation,\n    square,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  if (true) {\n    if (theme.shadows[elevation] === undefined) {\n      console.error([`MUI: The elevation provided <Paper elevation={${elevation}}> is not available in the theme.`, `Please make sure that \\`theme.shadows[${elevation}]\\` is defined.`].join('\\n'));\n    }\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(PaperRoot, {\n    as: component,\n    ownerState: ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ref: ref,\n    ...other,\n    style: {\n      ...(variant === 'elevation' && {\n        '--Paper-shadow': (theme.vars || theme).shadows[elevation],\n        ...(theme.vars && {\n          '--Paper-overlay': theme.vars.overlays?.[elevation]\n        }),\n        ...(!theme.vars && theme.palette.mode === 'dark' && {\n          '--Paper-overlay': `linear-gradient(${(0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)('#fff', (0,_styles_getOverlayAlpha_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(elevation))}, ${(0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)('#fff', (0,_styles_getOverlayAlpha_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(elevation))})`\n        })\n      }),\n      ...other.style\n    }\n  });\n});\n true ? Paper.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n  /**\n   * Shadow depth, corresponds to `dp` in the spec.\n   * It accepts values between 0 and 24 inclusive.\n   * @default 1\n   */\n  elevation: (0,_mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_12__[\"default\"])(_mui_utils_integerPropType__WEBPACK_IMPORTED_MODULE_13__[\"default\"], props => {\n    const {\n      elevation,\n      variant\n    } = props;\n    if (elevation > 0 && variant === 'outlined') {\n      return new Error(`MUI: Combining \\`elevation={${elevation}}\\` with \\`variant=\"${variant}\"\\` has no effect. Either use \\`elevation={0}\\` or use a different \\`variant\\`.`);\n    }\n    return null;\n  }),\n  /**\n   * If `true`, rounded corners are disabled.\n   * @default false\n   */\n  square: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n  /**\n   * The variant to use.\n   * @default 'elevation'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['elevation', 'outlined']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Paper);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Paper/Paper.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Paper/paperClasses.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/Paper/paperClasses.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getPaperUtilityClass: () => (/* binding */ getPaperUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getPaperUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiPaper', slot);\n}\nconst paperClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiPaper', ['root', 'rounded', 'outlined', 'elevation', 'elevation0', 'elevation1', 'elevation2', 'elevation3', 'elevation4', 'elevation5', 'elevation6', 'elevation7', 'elevation8', 'elevation9', 'elevation10', 'elevation11', 'elevation12', 'elevation13', 'elevation14', 'elevation15', 'elevation16', 'elevation17', 'elevation18', 'elevation19', 'elevation20', 'elevation21', 'elevation22', 'elevation23', 'elevation24']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (paperClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Paper/paperClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Popover/Popover.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/material/Popover/Popover.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   PopoverPaper: () => (/* binding */ PopoverPaper),\n/* harmony export */   PopoverRoot: () => (/* binding */ PopoverRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getOffsetLeft: () => (/* binding */ getOffsetLeft),\n/* harmony export */   getOffsetTop: () => (/* binding */ getOffsetTop)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_18___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_18__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! @mui/utils/HTMLElementType */ \"./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_elementTypeAcceptingRef__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! @mui/utils/elementTypeAcceptingRef */ \"./node_modules/@mui/utils/esm/elementTypeAcceptingRef/elementTypeAcceptingRef.js\");\n/* harmony import */ var _mui_utils_integerPropType__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! @mui/utils/integerPropType */ \"./node_modules/@mui/utils/esm/integerPropType/integerPropType.js\");\n/* harmony import */ var _mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! @mui/utils/chainPropTypes */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n/* harmony import */ var _utils_isHostComponent_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../utils/isHostComponent.js */ \"./node_modules/@mui/material/utils/isHostComponent.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_debounce_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../utils/debounce.js */ \"./node_modules/@mui/material/utils/debounce.js\");\n/* harmony import */ var _utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/ownerDocument.js */ \"./node_modules/@mui/material/utils/ownerDocument.js\");\n/* harmony import */ var _utils_ownerWindow_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/ownerWindow.js */ \"./node_modules/@mui/material/utils/ownerWindow.js\");\n/* harmony import */ var _Grow_index_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../Grow/index.js */ \"./node_modules/@mui/material/Grow/Grow.js\");\n/* harmony import */ var _Modal_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../Modal/index.js */ \"./node_modules/@mui/material/Modal/Modal.js\");\n/* harmony import */ var _Paper_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Paper/index.js */ \"./node_modules/@mui/material/Paper/Paper.js\");\n/* harmony import */ var _popoverClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./popoverClasses.js */ \"./node_modules/@mui/material/Popover/popoverClasses.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var _utils_index_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../utils/index.js */ \"./node_modules/@mui/material/utils/mergeSlotProps.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nfunction getOffsetTop(rect, vertical) {\n  let offset = 0;\n  if (typeof vertical === 'number') {\n    offset = vertical;\n  } else if (vertical === 'center') {\n    offset = rect.height / 2;\n  } else if (vertical === 'bottom') {\n    offset = rect.height;\n  }\n  return offset;\n}\nfunction getOffsetLeft(rect, horizontal) {\n  let offset = 0;\n  if (typeof horizontal === 'number') {\n    offset = horizontal;\n  } else if (horizontal === 'center') {\n    offset = rect.width / 2;\n  } else if (horizontal === 'right') {\n    offset = rect.width;\n  }\n  return offset;\n}\nfunction getTransformOriginValue(transformOrigin) {\n  return [transformOrigin.horizontal, transformOrigin.vertical].map(n => typeof n === 'number' ? `${n}px` : n).join(' ');\n}\nfunction resolveAnchorEl(anchorEl) {\n  return typeof anchorEl === 'function' ? anchorEl() : anchorEl;\n}\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root'],\n    paper: ['paper']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _popoverClasses_js__WEBPACK_IMPORTED_MODULE_4__.getPopoverUtilityClass, classes);\n};\nconst PopoverRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Modal_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n  name: 'MuiPopover',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({});\nconst PopoverPaper = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_Paper_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiPopover',\n  slot: 'Paper',\n  overridesResolver: (props, styles) => styles.paper\n})({\n  position: 'absolute',\n  overflowY: 'auto',\n  overflowX: 'hidden',\n  // So we see the popover when it's empty.\n  // It's most likely on issue on userland.\n  minWidth: 16,\n  minHeight: 16,\n  maxWidth: 'calc(100% - 32px)',\n  maxHeight: 'calc(100% - 32px)',\n  // We disable the focus ring for mouse, touch and keyboard users.\n  outline: 0\n});\nconst Popover = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Popover(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiPopover'\n  });\n  const {\n    action,\n    anchorEl,\n    anchorOrigin = {\n      vertical: 'top',\n      horizontal: 'left'\n    },\n    anchorPosition,\n    anchorReference = 'anchorEl',\n    children,\n    className,\n    container: containerProp,\n    elevation = 8,\n    marginThreshold = 16,\n    open,\n    PaperProps: PaperPropsProp = {},\n    // TODO: remove in v7\n    slots = {},\n    slotProps = {},\n    transformOrigin = {\n      vertical: 'top',\n      horizontal: 'left'\n    },\n    TransitionComponent,\n    // TODO: remove in v7\n    transitionDuration: transitionDurationProp = 'auto',\n    TransitionProps = {},\n    // TODO: remove in v7\n    disableScrollLock = false,\n    ...other\n  } = props;\n  const paperRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef();\n  const ownerState = {\n    ...props,\n    anchorOrigin,\n    anchorReference,\n    elevation,\n    marginThreshold,\n    transformOrigin,\n    TransitionComponent,\n    transitionDuration: transitionDurationProp,\n    TransitionProps\n  };\n  const classes = useUtilityClasses(ownerState);\n\n  // Returns the top/left offset of the position\n  // to attach to on the anchor element (or body if none is provided)\n  const getAnchorOffset = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    if (anchorReference === 'anchorPosition') {\n      if (true) {\n        if (!anchorPosition) {\n          console.error('MUI: You need to provide a `anchorPosition` prop when using ' + '<Popover anchorReference=\"anchorPosition\" />.');\n        }\n      }\n      return anchorPosition;\n    }\n    const resolvedAnchorEl = resolveAnchorEl(anchorEl);\n\n    // If an anchor element wasn't provided, just use the parent body element of this Popover\n    const anchorElement = resolvedAnchorEl && resolvedAnchorEl.nodeType === 1 ? resolvedAnchorEl : (0,_utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(paperRef.current).body;\n    const anchorRect = anchorElement.getBoundingClientRect();\n    if (true) {\n      const box = anchorElement.getBoundingClientRect();\n      if ( true && box.top === 0 && box.left === 0 && box.right === 0 && box.bottom === 0) {\n        console.warn(['MUI: The `anchorEl` prop provided to the component is invalid.', 'The anchor element should be part of the document layout.', \"Make sure the element is present in the document or that it's not display none.\"].join('\\n'));\n      }\n    }\n    return {\n      top: anchorRect.top + getOffsetTop(anchorRect, anchorOrigin.vertical),\n      left: anchorRect.left + getOffsetLeft(anchorRect, anchorOrigin.horizontal)\n    };\n  }, [anchorEl, anchorOrigin.horizontal, anchorOrigin.vertical, anchorPosition, anchorReference]);\n\n  // Returns the base transform origin using the element\n  const getTransformOrigin = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(elemRect => {\n    return {\n      vertical: getOffsetTop(elemRect, transformOrigin.vertical),\n      horizontal: getOffsetLeft(elemRect, transformOrigin.horizontal)\n    };\n  }, [transformOrigin.horizontal, transformOrigin.vertical]);\n  const getPositioningStyle = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(element => {\n    const elemRect = {\n      width: element.offsetWidth,\n      height: element.offsetHeight\n    };\n\n    // Get the transform origin point on the element itself\n    const elemTransformOrigin = getTransformOrigin(elemRect);\n    if (anchorReference === 'none') {\n      return {\n        top: null,\n        left: null,\n        transformOrigin: getTransformOriginValue(elemTransformOrigin)\n      };\n    }\n\n    // Get the offset of the anchoring element\n    const anchorOffset = getAnchorOffset();\n\n    // Calculate element positioning\n    let top = anchorOffset.top - elemTransformOrigin.vertical;\n    let left = anchorOffset.left - elemTransformOrigin.horizontal;\n    const bottom = top + elemRect.height;\n    const right = left + elemRect.width;\n\n    // Use the parent window of the anchorEl if provided\n    const containerWindow = (0,_utils_ownerWindow_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(resolveAnchorEl(anchorEl));\n\n    // Window thresholds taking required margin into account\n    const heightThreshold = containerWindow.innerHeight - marginThreshold;\n    const widthThreshold = containerWindow.innerWidth - marginThreshold;\n\n    // Check if the vertical axis needs shifting\n    if (marginThreshold !== null && top < marginThreshold) {\n      const diff = top - marginThreshold;\n      top -= diff;\n      elemTransformOrigin.vertical += diff;\n    } else if (marginThreshold !== null && bottom > heightThreshold) {\n      const diff = bottom - heightThreshold;\n      top -= diff;\n      elemTransformOrigin.vertical += diff;\n    }\n    if (true) {\n      if (elemRect.height > heightThreshold && elemRect.height && heightThreshold) {\n        console.error(['MUI: The popover component is too tall.', `Some part of it can not be seen on the screen (${elemRect.height - heightThreshold}px).`, 'Please consider adding a `max-height` to improve the user-experience.'].join('\\n'));\n      }\n    }\n\n    // Check if the horizontal axis needs shifting\n    if (marginThreshold !== null && left < marginThreshold) {\n      const diff = left - marginThreshold;\n      left -= diff;\n      elemTransformOrigin.horizontal += diff;\n    } else if (right > widthThreshold) {\n      const diff = right - widthThreshold;\n      left -= diff;\n      elemTransformOrigin.horizontal += diff;\n    }\n    return {\n      top: `${Math.round(top)}px`,\n      left: `${Math.round(left)}px`,\n      transformOrigin: getTransformOriginValue(elemTransformOrigin)\n    };\n  }, [anchorEl, anchorReference, getAnchorOffset, getTransformOrigin, marginThreshold]);\n  const [isPositioned, setIsPositioned] = react__WEBPACK_IMPORTED_MODULE_0__.useState(open);\n  const setPositioningStyles = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    const element = paperRef.current;\n    if (!element) {\n      return;\n    }\n    const positioning = getPositioningStyle(element);\n    if (positioning.top !== null) {\n      element.style.setProperty('top', positioning.top);\n    }\n    if (positioning.left !== null) {\n      element.style.left = positioning.left;\n    }\n    element.style.transformOrigin = positioning.transformOrigin;\n    setIsPositioned(true);\n  }, [getPositioningStyle]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (disableScrollLock) {\n      window.addEventListener('scroll', setPositioningStyles);\n    }\n    return () => window.removeEventListener('scroll', setPositioningStyles);\n  }, [anchorEl, disableScrollLock, setPositioningStyles]);\n  const handleEntering = () => {\n    setPositioningStyles();\n  };\n  const handleExited = () => {\n    setIsPositioned(false);\n  };\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (open) {\n      setPositioningStyles();\n    }\n  });\n  react__WEBPACK_IMPORTED_MODULE_0__.useImperativeHandle(action, () => open ? {\n    updatePosition: () => {\n      setPositioningStyles();\n    }\n  } : null, [open, setPositioningStyles]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (!open) {\n      return undefined;\n    }\n    const handleResize = (0,_utils_debounce_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])(() => {\n      setPositioningStyles();\n    });\n    const containerWindow = (0,_utils_ownerWindow_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(resolveAnchorEl(anchorEl));\n    containerWindow.addEventListener('resize', handleResize);\n    return () => {\n      handleResize.clear();\n      containerWindow.removeEventListener('resize', handleResize);\n    };\n  }, [anchorEl, open, setPositioningStyles]);\n  let transitionDuration = transitionDurationProp;\n  const externalForwardedProps = {\n    slots: {\n      transition: TransitionComponent,\n      ...slots\n    },\n    slotProps: {\n      transition: TransitionProps,\n      paper: PaperPropsProp,\n      ...slotProps\n    }\n  };\n  const [TransitionSlot, transitionSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('transition', {\n    elementType: _Grow_index_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"],\n    externalForwardedProps,\n    ownerState,\n    getSlotProps: handlers => ({\n      ...handlers,\n      onEntering: (element, isAppearing) => {\n        handlers.onEntering?.(element, isAppearing);\n        handleEntering();\n      },\n      onExited: element => {\n        handlers.onExited?.(element);\n        handleExited();\n      }\n    }),\n    additionalProps: {\n      appear: true,\n      in: open\n    }\n  });\n  if (transitionDurationProp === 'auto' && !TransitionSlot.muiSupportAuto) {\n    transitionDuration = undefined;\n  }\n\n  // If the container prop is provided, use that\n  // If the anchorEl prop is provided, use its parent body element as the container\n  // If neither are provided let the Modal take care of choosing the container\n  const container = containerProp || (anchorEl ? (0,_utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(resolveAnchorEl(anchorEl)).body : undefined);\n  const [RootSlot, {\n    slots: rootSlotsProp,\n    slotProps: rootSlotPropsProp,\n    ...rootProps\n  }] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('root', {\n    ref,\n    elementType: PopoverRoot,\n    externalForwardedProps: {\n      ...externalForwardedProps,\n      ...other\n    },\n    shouldForwardComponentProp: true,\n    additionalProps: {\n      slots: {\n        backdrop: slots.backdrop\n      },\n      slotProps: {\n        backdrop: (0,_utils_index_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"])(typeof slotProps.backdrop === 'function' ? slotProps.backdrop(ownerState) : slotProps.backdrop, {\n          invisible: true\n        })\n      },\n      container,\n      open\n    },\n    ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className)\n  });\n  const [PaperSlot, paperProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('paper', {\n    ref: paperRef,\n    className: classes.paper,\n    elementType: PopoverPaper,\n    externalForwardedProps,\n    shouldForwardComponentProp: true,\n    additionalProps: {\n      elevation,\n      style: isPositioned ? undefined : {\n        opacity: 0\n      }\n    },\n    ownerState\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(RootSlot, {\n    ...rootProps,\n    ...(!(0,_utils_isHostComponent_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])(RootSlot) && {\n      slots: rootSlotsProp,\n      slotProps: rootSlotPropsProp,\n      disableScrollLock\n    }),\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TransitionSlot, {\n      ...transitionSlotProps,\n      timeout: transitionDuration,\n      children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(PaperSlot, {\n        ...paperProps,\n        children: children\n      })\n    })\n  });\n});\n true ? Popover.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * A ref for imperative actions.\n   * It currently only supports updatePosition() action.\n   */\n  action: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_16__[\"default\"],\n  /**\n   * An HTML element, [PopoverVirtualElement](https://mui.com/material-ui/react-popover/#virtual-element),\n   * or a function that returns either.\n   * It's used to set the position of the popover.\n   */\n  anchorEl: (0,_mui_utils_chainPropTypes__WEBPACK_IMPORTED_MODULE_17__[\"default\"])(prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([_mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_19__[\"default\"], (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func)]), props => {\n    if (props.open && (!props.anchorReference || props.anchorReference === 'anchorEl')) {\n      const resolvedAnchorEl = resolveAnchorEl(props.anchorEl);\n      if (resolvedAnchorEl && resolvedAnchorEl.nodeType === 1) {\n        const box = resolvedAnchorEl.getBoundingClientRect();\n        if ( true && box.top === 0 && box.left === 0 && box.right === 0 && box.bottom === 0) {\n          return new Error(['MUI: The `anchorEl` prop provided to the component is invalid.', 'The anchor element should be part of the document layout.', \"Make sure the element is present in the document or that it's not display none.\"].join('\\n'));\n        }\n      } else {\n        return new Error(['MUI: The `anchorEl` prop provided to the component is invalid.', `It should be an Element or PopoverVirtualElement instance but it's \\`${resolvedAnchorEl}\\` instead.`].join('\\n'));\n      }\n    }\n    return null;\n  }),\n  /**\n   * This is the point on the anchor where the popover's\n   * `anchorEl` will attach to. This is not used when the\n   * anchorReference is 'anchorPosition'.\n   *\n   * Options:\n   * vertical: [top, center, bottom];\n   * horizontal: [left, center, right].\n   * @default {\n   *   vertical: 'top',\n   *   horizontal: 'left',\n   * }\n   */\n  anchorOrigin: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    horizontal: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['center', 'left', 'right']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number)]).isRequired,\n    vertical: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['bottom', 'center', 'top']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number)]).isRequired\n  }),\n  /**\n   * This is the position that may be used to set the position of the popover.\n   * The coordinates are relative to the application's client area.\n   */\n  anchorPosition: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    left: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number).isRequired,\n    top: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number).isRequired\n  }),\n  /**\n   * This determines which anchor prop to refer to when setting\n   * the position of the popover.\n   * @default 'anchorEl'\n   */\n  anchorReference: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['anchorEl', 'anchorPosition', 'none']),\n  /**\n   * A backdrop component. This prop enables custom backdrop rendering.\n   * @deprecated Use `slots.backdrop` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   * @default styled(Backdrop, {\n   *   name: 'MuiModal',\n   *   slot: 'Backdrop',\n   *   overridesResolver: (props, styles) => {\n   *     return styles.backdrop;\n   *   },\n   * })({\n   *   zIndex: -1,\n   * })\n   */\n  BackdropComponent: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n  /**\n   * Props applied to the [`Backdrop`](/material-ui/api/backdrop/) element.\n   * @deprecated Use `slotProps.backdrop` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  BackdropProps: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().string),\n  /**\n   * An HTML element, component instance, or function that returns either.\n   * The `container` will passed to the Modal component.\n   *\n   * By default, it uses the body of the anchorEl's top-level document object,\n   * so it's simply `document.body` most of the time.\n   */\n  container: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([_mui_utils_HTMLElementType__WEBPACK_IMPORTED_MODULE_19__[\"default\"], (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func)]),\n  /**\n   * Disable the scroll lock behavior.\n   * @default false\n   */\n  disableScrollLock: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool),\n  /**\n   * The elevation of the popover.\n   * @default 8\n   */\n  elevation: _mui_utils_integerPropType__WEBPACK_IMPORTED_MODULE_20__[\"default\"],\n  /**\n   * Specifies how close to the edge of the window the popover can appear.\n   * If null, the popover will not be constrained by the window.\n   * @default 16\n   */\n  marginThreshold: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number),\n  /**\n   * Callback fired when the component requests to be closed.\n   * The `reason` parameter can optionally be used to control the response to `onClose`.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func),\n  /**\n   * If `true`, the component is shown.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool).isRequired,\n  /**\n   * Props applied to the [`Paper`](https://mui.com/material-ui/api/paper/) element.\n   *\n   * This prop is an alias for `slotProps.paper` and will be overriden by it if both are used.\n   * @deprecated Use `slotProps.paper` instead.\n   *\n   * @default {}\n   */\n  PaperProps: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    component: _mui_utils_elementTypeAcceptingRef__WEBPACK_IMPORTED_MODULE_21__[\"default\"]\n  }),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    backdrop: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)]),\n    paper: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)]),\n    transition: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    backdrop: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n    paper: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n    transition: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)]),\n  /**\n   * This is the point on the popover which\n   * will attach to the anchor's origin.\n   *\n   * Options:\n   * vertical: [top, center, bottom, x(px)];\n   * horizontal: [left, center, right, x(px)].\n   * @default {\n   *   vertical: 'top',\n   *   horizontal: 'left',\n   * }\n   */\n  transformOrigin: prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    horizontal: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['center', 'left', 'right']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number)]).isRequired,\n    vertical: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['bottom', 'center', 'top']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number)]).isRequired\n  }),\n  /**\n   * The component used for the transition.\n   * [Follow this guide](https://mui.com/material-ui/transitions/#transitioncomponent-prop) to learn more about the requirements for this component.\n   * @deprecated use the `slots.transition` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   * @default Grow\n   */\n  TransitionComponent: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().elementType),\n  /**\n   * Set to 'auto' to automatically calculate transition time based on height.\n   * @default 'auto'\n   */\n  transitionDuration: prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_18___default().oneOf(['auto']), (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number), prop_types__WEBPACK_IMPORTED_MODULE_18___default().shape({\n    appear: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number),\n    enter: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number),\n    exit: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().number)\n  })]),\n  /**\n   * Props applied to the transition element.\n   * By default, the element is based on this [`Transition`](https://reactcommunity.org/react-transition-group/transition/) component.\n   * @deprecated use the `slotProps.transition` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   * @default {}\n   */\n  TransitionProps: (prop_types__WEBPACK_IMPORTED_MODULE_18___default().object)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Popover);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Popover/Popover.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Popover/popoverClasses.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/Popover/popoverClasses.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getPopoverUtilityClass: () => (/* binding */ getPopoverUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getPopoverUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiPopover', slot);\n}\nconst popoverClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiPopover', ['root', 'paper']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (popoverClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Popover/popoverClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Portal/Portal.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/Portal/Portal.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_dom__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react-dom */ \"./node_modules/react-dom/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_6___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_6__);\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/setRef/setRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/exactProp/exactProp.js\");\n'use client';\n\n\n\n\n\nfunction getContainer(container) {\n  return typeof container === 'function' ? container() : container;\n}\n\n/**\n * Portals provide a first-class way to render children into a DOM node\n * that exists outside the DOM hierarchy of the parent component.\n *\n * Demos:\n *\n * - [Portal](https://v6.mui.com/material-ui/react-portal/)\n *\n * API:\n *\n * - [Portal API](https://v6.mui.com/material-ui/api/portal/)\n */\nconst Portal = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Portal(props, forwardedRef) {\n  const {\n    children,\n    container,\n    disablePortal = false\n  } = props;\n  const [mountNode, setMountNode] = react__WEBPACK_IMPORTED_MODULE_0__.useState(null);\n  const handleRef = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(children) ? (0,_mui_utils__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(children) : null, forwardedRef);\n  (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(() => {\n    if (!disablePortal) {\n      setMountNode(getContainer(container) || document.body);\n    }\n  }, [container, disablePortal]);\n  (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(() => {\n    if (mountNode && !disablePortal) {\n      (0,_mui_utils__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(forwardedRef, mountNode);\n      return () => {\n        (0,_mui_utils__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(forwardedRef, null);\n      };\n    }\n    return undefined;\n  }, [forwardedRef, mountNode, disablePortal]);\n  if (disablePortal) {\n    if (/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(children)) {\n      const newProps = {\n        ref: handleRef\n      };\n      return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(children, newProps);\n    }\n    return children;\n  }\n  return mountNode ? /*#__PURE__*/react_dom__WEBPACK_IMPORTED_MODULE_1__.createPortal(children, mountNode) : mountNode;\n});\n true ? Portal.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The children to render into the `container`.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().node),\n  /**\n   * An HTML element or function that returns one.\n   * The `container` will have the portal children appended to it.\n   *\n   * You can also provide a callback, which is called in a React layout effect.\n   * This lets you set the container from a ref, and also makes server-side rendering possible.\n   *\n   * By default, it uses the body of the top-level document object,\n   * so it's simply `document.body` most of the time.\n   */\n  container: prop_types__WEBPACK_IMPORTED_MODULE_6___default().oneOfType([_mui_utils__WEBPACK_IMPORTED_MODULE_7__[\"default\"], (prop_types__WEBPACK_IMPORTED_MODULE_6___default().func)]),\n  /**\n   * The `children` will be under the DOM hierarchy of the parent component.\n   * @default false\n   */\n  disablePortal: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().bool)\n} : 0;\nif (true) {\n  // eslint-disable-next-line\n  Portal['propTypes' + ''] = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(Portal.propTypes);\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Portal);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Portal/Portal.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Select/Select.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/Select/Select.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_19___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_19__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_getReactElementRef__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! @mui/utils/getReactElementRef */ \"./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js\");\n/* harmony import */ var _SelectInput_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./SelectInput.js */ \"./node_modules/@mui/material/Select/SelectInput.js\");\n/* harmony import */ var _FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../FormControl/formControlState.js */ \"./node_modules/@mui/material/FormControl/formControlState.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _internal_svg_icons_ArrowDropDown_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../internal/svg-icons/ArrowDropDown.js */ \"./node_modules/@mui/material/internal/svg-icons/ArrowDropDown.js\");\n/* harmony import */ var _Input_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../Input/index.js */ \"./node_modules/@mui/material/Input/Input.js\");\n/* harmony import */ var _NativeSelect_NativeSelectInput_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../NativeSelect/NativeSelectInput.js */ \"./node_modules/@mui/material/NativeSelect/NativeSelectInput.js\");\n/* harmony import */ var _FilledInput_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../FilledInput/index.js */ \"./node_modules/@mui/material/FilledInput/FilledInput.js\");\n/* harmony import */ var _OutlinedInput_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../OutlinedInput/index.js */ \"./node_modules/@mui/material/OutlinedInput/OutlinedInput.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _selectClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./selectClasses.js */ \"./node_modules/@mui/material/Select/selectClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _selectClasses_js__WEBPACK_IMPORTED_MODULE_4__.getSelectUtilityClasses, classes);\n  return {\n    ...classes,\n    ...composedClasses\n  };\n};\nconst styledRootConfig = {\n  name: 'MuiSelect',\n  overridesResolver: (props, styles) => styles.root,\n  shouldForwardProp: prop => (0,_styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(prop) && prop !== 'variant',\n  slot: 'Root'\n};\nconst StyledInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_Input_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], styledRootConfig)('');\nconst StyledOutlinedInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_OutlinedInput_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"], styledRootConfig)('');\nconst StyledFilledInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_FilledInput_index_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"], styledRootConfig)('');\nconst Select = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Select(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    name: 'MuiSelect',\n    props: inProps\n  });\n  const {\n    autoWidth = false,\n    children,\n    classes: classesProp = {},\n    className,\n    defaultOpen = false,\n    displayEmpty = false,\n    IconComponent = _internal_svg_icons_ArrowDropDown_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"],\n    id,\n    input,\n    inputProps,\n    label,\n    labelId,\n    MenuProps,\n    multiple = false,\n    native = false,\n    onClose,\n    onOpen,\n    open,\n    renderValue,\n    SelectDisplayProps,\n    variant: variantProp = 'outlined',\n    ...other\n  } = props;\n  const inputComponent = native ? _NativeSelect_NativeSelectInput_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"] : _SelectInput_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"];\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"])();\n  const fcs = (0,_FormControl_formControlState_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"])({\n    props,\n    muiFormControl,\n    states: ['variant', 'error']\n  });\n  const variant = fcs.variant || variantProp;\n  const ownerState = {\n    ...props,\n    variant,\n    classes: classesProp\n  };\n  const classes = useUtilityClasses(ownerState);\n  const {\n    root,\n    ...restOfClasses\n  } = classes;\n  const InputComponent = input || {\n    standard: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(StyledInput, {\n      ownerState: ownerState\n    }),\n    outlined: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(StyledOutlinedInput, {\n      label: label,\n      ownerState: ownerState\n    }),\n    filled: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(StyledFilledInput, {\n      ownerState: ownerState\n    })\n  }[variant];\n  const inputComponentRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_16__[\"default\"])(ref, (0,_mui_utils_getReactElementRef__WEBPACK_IMPORTED_MODULE_17__[\"default\"])(InputComponent));\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(InputComponent, {\n      // Most of the logic is implemented in `SelectInput`.\n      // The `Select` component is a simple API wrapper to expose something better to play with.\n      inputComponent,\n      inputProps: {\n        children,\n        error: fcs.error,\n        IconComponent,\n        variant,\n        type: undefined,\n        // We render a select. We can ignore the type provided by the `Input`.\n        multiple,\n        ...(native ? {\n          id\n        } : {\n          autoWidth,\n          defaultOpen,\n          displayEmpty,\n          labelId,\n          MenuProps,\n          onClose,\n          onOpen,\n          open,\n          renderValue,\n          SelectDisplayProps: {\n            id,\n            ...SelectDisplayProps\n          }\n        }),\n        ...inputProps,\n        classes: inputProps ? (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_18__[\"default\"])(restOfClasses, inputProps.classes) : restOfClasses,\n        ...(input ? input.props.inputProps : {})\n      },\n      ...((multiple && native || displayEmpty) && variant === 'outlined' ? {\n        notched: true\n      } : {}),\n      ref: inputComponentRef,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(InputComponent.props.className, className, classes.root),\n      // If a custom input is provided via 'input' prop, do not allow 'variant' to be propagated to it's root element. See https://github.com/mui/material-ui/issues/33894.\n      ...(!input && {\n        variant\n      }),\n      ...other\n    })\n  });\n});\n true ? Select.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * If `true`, the width of the popover will automatically be set according to the items inside the\n   * menu, otherwise it will be at least the width of the select input.\n   * @default false\n   */\n  autoWidth: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool),\n  /**\n   * The option elements to populate the select with.\n   * Can be some `MenuItem` when `native` is false and `option` when `native` is true.\n   *\n   * ⚠️The `MenuItem` elements **must** be direct descendants when `native` is false.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   * @default {}\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string),\n  /**\n   * If `true`, the component is initially open. Use when the component open state is not controlled (i.e. the `open` prop is not defined).\n   * You can only use it when the `native` prop is `false` (default).\n   * @default false\n   */\n  defaultOpen: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().any),\n  /**\n   * If `true`, a value is displayed even if no items are selected.\n   *\n   * In order to display a meaningful value, a function can be passed to the `renderValue` prop which\n   * returns the value to be displayed when no items are selected.\n   *\n   * ⚠️ When using this prop, make sure the label doesn't overlap with the empty displayed value.\n   * The label should either be hidden or forced to a shrunk state.\n   * @default false\n   */\n  displayEmpty: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool),\n  /**\n   * The icon that displays the arrow.\n   * @default ArrowDropDownIcon\n   */\n  IconComponent: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().elementType),\n  /**\n   * The `id` of the wrapper element or the `select` element when `native`.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string),\n  /**\n   * An `Input` element; does not have to be a material-ui specific `Input`.\n   */\n  input: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().element),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * When `native` is `true`, the attributes are applied on the `select` element.\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object),\n  /**\n   * See [OutlinedInput#label](https://mui.com/material-ui/api/outlined-input/#props)\n   */\n  label: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().node),\n  /**\n   * The ID of an element that acts as an additional label. The Select will\n   * be labelled by the additional label and the selected value.\n   */\n  labelId: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().string),\n  /**\n   * Props applied to the [`Menu`](https://mui.com/material-ui/api/menu/) element.\n   */\n  MenuProps: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object),\n  /**\n   * If `true`, `value` must be an array and the menu will support multiple selections.\n   * @default false\n   */\n  multiple: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool),\n  /**\n   * If `true`, the component uses a native `select` element.\n   * @default false\n   */\n  native: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool),\n  /**\n   * Callback fired when a menu item is selected.\n   *\n   * @param {SelectChangeEvent<Value>} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (any).\n   * **Warning**: This is a generic event, not a change event, unless the change event is caused by browser autofill.\n   * @param {object} [child] The react element that was selected when `native` is `false` (default).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func),\n  /**\n   * Callback fired when the component requests to be closed.\n   * Use it in either controlled (see the `open` prop), or uncontrolled mode (to detect when the Select collapses).\n   *\n   * @param {object} event The event source of the callback.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func),\n  /**\n   * Callback fired when the component requests to be opened.\n   * Use it in either controlled (see the `open` prop), or uncontrolled mode (to detect when the Select expands).\n   *\n   * @param {object} event The event source of the callback.\n   */\n  onOpen: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func),\n  /**\n   * If `true`, the component is shown.\n   * You can only use it when the `native` prop is `false` (default).\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool),\n  /**\n   * Render the selected value.\n   * You can only use it when the `native` prop is `false` (default).\n   *\n   * @param {any} value The `value` provided to the component.\n   * @returns {ReactNode}\n   */\n  renderValue: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func),\n  /**\n   * Props applied to the clickable div element.\n   */\n  SelectDisplayProps: (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_19___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().object)]),\n  /**\n   * The `input` value. Providing an empty string will select no options.\n   * Set to an empty string `''` if you don't want any of the available options to be selected.\n   *\n   * If the value is an object it must have reference equality with the option in order to be selected.\n   * If the value is not an object, the string representation must match with the string representation of the option in order to be selected.\n   */\n  value: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOf(['']), (prop_types__WEBPACK_IMPORTED_MODULE_19___default().any)]),\n  /**\n   * The variant to use.\n   * @default 'outlined'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_19___default().oneOf(['filled', 'outlined', 'standard'])\n} : 0;\nSelect.muiName = 'Select';\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Select);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Select/Select.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Select/SelectInput.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/Select/SelectInput.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_16___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_16__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_useId__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/useId */ \"./node_modules/@mui/utils/esm/useId/useId.js\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../utils/ownerDocument.js */ \"./node_modules/@mui/material/utils/ownerDocument.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _Menu_Menu_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../Menu/Menu.js */ \"./node_modules/@mui/material/Menu/Menu.js\");\n/* harmony import */ var _NativeSelect_NativeSelectInput_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../NativeSelect/NativeSelectInput.js */ \"./node_modules/@mui/material/NativeSelect/NativeSelectInput.js\");\n/* harmony import */ var _InputBase_utils_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../InputBase/utils.js */ \"./node_modules/@mui/material/InputBase/utils.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _styles_slotShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styles/slotShouldForwardProp.js */ \"./node_modules/@mui/material/styles/slotShouldForwardProp.js\");\n/* harmony import */ var _utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/useForkRef.js */ \"./node_modules/@mui/material/utils/useForkRef.js\");\n/* harmony import */ var _utils_useControlled_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/useControlled.js */ \"./node_modules/@mui/material/utils/useControlled.js\");\n/* harmony import */ var _selectClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./selectClasses.js */ \"./node_modules/@mui/material/Select/selectClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\nvar _span;\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst SelectSelect = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(_NativeSelect_NativeSelectInput_js__WEBPACK_IMPORTED_MODULE_4__.StyledSelectSelect, {\n  name: 'MuiSelect',\n  slot: 'Select',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [\n    // Win specificity over the input base\n    {\n      [`&.${_selectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].select}`]: styles.select\n    }, {\n      [`&.${_selectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].select}`]: styles[ownerState.variant]\n    }, {\n      [`&.${_selectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].error}`]: styles.error\n    }, {\n      [`&.${_selectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].multiple}`]: styles.multiple\n    }];\n  }\n})({\n  // Win specificity over the input base\n  [`&.${_selectClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].select}`]: {\n    height: 'auto',\n    // Resets for multiple select with chips\n    minHeight: '1.4375em',\n    // Required for select\\text-field height consistency\n    textOverflow: 'ellipsis',\n    whiteSpace: 'nowrap',\n    overflow: 'hidden'\n  }\n});\nconst SelectIcon = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(_NativeSelect_NativeSelectInput_js__WEBPACK_IMPORTED_MODULE_4__.StyledSelectIcon, {\n  name: 'MuiSelect',\n  slot: 'Icon',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.icon, ownerState.variant && styles[`icon${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(ownerState.variant)}`], ownerState.open && styles.iconOpen];\n  }\n})({});\nconst SelectNativeInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])('input', {\n  shouldForwardProp: prop => (0,_styles_slotShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(prop) && prop !== 'classes',\n  name: 'MuiSelect',\n  slot: 'NativeInput',\n  overridesResolver: (props, styles) => styles.nativeInput\n})({\n  bottom: 0,\n  left: 0,\n  position: 'absolute',\n  opacity: 0,\n  pointerEvents: 'none',\n  width: '100%',\n  boxSizing: 'border-box'\n});\nfunction areEqualValues(a, b) {\n  if (typeof b === 'object' && b !== null) {\n    return a === b;\n  }\n\n  // The value could be a number, the DOM will stringify it anyway.\n  return String(a) === String(b);\n}\nfunction isEmpty(display) {\n  return display == null || typeof display === 'string' && !display.trim();\n}\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    variant,\n    disabled,\n    multiple,\n    open,\n    error\n  } = ownerState;\n  const slots = {\n    select: ['select', variant, disabled && 'disabled', multiple && 'multiple', error && 'error'],\n    icon: ['icon', `icon${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(variant)}`, open && 'iconOpen', disabled && 'disabled'],\n    nativeInput: ['nativeInput']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(slots, _selectClasses_js__WEBPACK_IMPORTED_MODULE_5__.getSelectUtilityClasses, classes);\n};\n\n/**\n * @ignore - internal component.\n */\nconst SelectInput = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function SelectInput(props, ref) {\n  const {\n    'aria-describedby': ariaDescribedby,\n    'aria-label': ariaLabel,\n    autoFocus,\n    autoWidth,\n    children,\n    className,\n    defaultOpen,\n    defaultValue,\n    disabled,\n    displayEmpty,\n    error = false,\n    IconComponent,\n    inputRef: inputRefProp,\n    labelId,\n    MenuProps = {},\n    multiple,\n    name,\n    onBlur,\n    onChange,\n    onClose,\n    onFocus,\n    onOpen,\n    open: openProp,\n    readOnly,\n    renderValue,\n    required,\n    SelectDisplayProps = {},\n    tabIndex: tabIndexProp,\n    // catching `type` from Input which makes no sense for SelectInput\n    type,\n    value: valueProp,\n    variant = 'standard',\n    ...other\n  } = props;\n  const [value, setValueState] = (0,_utils_useControlled_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])({\n    controlled: valueProp,\n    default: defaultValue,\n    name: 'Select'\n  });\n  const [openState, setOpenState] = (0,_utils_useControlled_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])({\n    controlled: openProp,\n    default: defaultOpen,\n    name: 'Select'\n  });\n  const inputRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const displayRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const [displayNode, setDisplayNode] = react__WEBPACK_IMPORTED_MODULE_0__.useState(null);\n  const {\n    current: isOpenControlled\n  } = react__WEBPACK_IMPORTED_MODULE_0__.useRef(openProp != null);\n  const [menuMinWidthState, setMenuMinWidthState] = react__WEBPACK_IMPORTED_MODULE_0__.useState();\n  const handleRef = (0,_utils_useForkRef_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(ref, inputRefProp);\n  const handleDisplayRef = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(node => {\n    displayRef.current = node;\n    if (node) {\n      setDisplayNode(node);\n    }\n  }, []);\n  const anchorElement = displayNode?.parentNode;\n  react__WEBPACK_IMPORTED_MODULE_0__.useImperativeHandle(handleRef, () => ({\n    focus: () => {\n      displayRef.current.focus();\n    },\n    node: inputRef.current,\n    value\n  }), [value]);\n\n  // Resize menu on `defaultOpen` automatic toggle.\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (defaultOpen && openState && displayNode && !isOpenControlled) {\n      setMenuMinWidthState(autoWidth ? null : anchorElement.clientWidth);\n      displayRef.current.focus();\n    }\n    // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler\n    // eslint-disable-next-line react-hooks/exhaustive-deps\n  }, [displayNode, autoWidth]);\n  // `isOpenControlled` is ignored because the component should never switch between controlled and uncontrolled modes.\n  // `defaultOpen` and `openState` are ignored to avoid unnecessary callbacks.\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (autoFocus) {\n      displayRef.current.focus();\n    }\n  }, [autoFocus]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (!labelId) {\n      return undefined;\n    }\n    const label = (0,_utils_ownerDocument_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])(displayRef.current).getElementById(labelId);\n    if (label) {\n      const handler = () => {\n        if (getSelection().isCollapsed) {\n          displayRef.current.focus();\n        }\n      };\n      label.addEventListener('click', handler);\n      return () => {\n        label.removeEventListener('click', handler);\n      };\n    }\n    return undefined;\n  }, [labelId]);\n  const update = (open, event) => {\n    if (open) {\n      if (onOpen) {\n        onOpen(event);\n      }\n    } else if (onClose) {\n      onClose(event);\n    }\n    if (!isOpenControlled) {\n      setMenuMinWidthState(autoWidth ? null : anchorElement.clientWidth);\n      setOpenState(open);\n    }\n  };\n  const handleMouseDown = event => {\n    // Ignore everything but left-click\n    if (event.button !== 0) {\n      return;\n    }\n    // Hijack the default focus behavior.\n    event.preventDefault();\n    displayRef.current.focus();\n    update(true, event);\n  };\n  const handleClose = event => {\n    update(false, event);\n  };\n  const childrenArray = react__WEBPACK_IMPORTED_MODULE_0__.Children.toArray(children);\n\n  // Support autofill.\n  const handleChange = event => {\n    const child = childrenArray.find(childItem => childItem.props.value === event.target.value);\n    if (child === undefined) {\n      return;\n    }\n    setValueState(child.props.value);\n    if (onChange) {\n      onChange(event, child);\n    }\n  };\n  const handleItemClick = child => event => {\n    let newValue;\n\n    // We use the tabindex attribute to signal the available options.\n    if (!event.currentTarget.hasAttribute('tabindex')) {\n      return;\n    }\n    if (multiple) {\n      newValue = Array.isArray(value) ? value.slice() : [];\n      const itemIndex = value.indexOf(child.props.value);\n      if (itemIndex === -1) {\n        newValue.push(child.props.value);\n      } else {\n        newValue.splice(itemIndex, 1);\n      }\n    } else {\n      newValue = child.props.value;\n    }\n    if (child.props.onClick) {\n      child.props.onClick(event);\n    }\n    if (value !== newValue) {\n      setValueState(newValue);\n      if (onChange) {\n        // Redefine target to allow name and value to be read.\n        // This allows seamless integration with the most popular form libraries.\n        // https://github.com/mui/material-ui/issues/13485#issuecomment-676048492\n        // Clone the event to not override `target` of the original event.\n        const nativeEvent = event.nativeEvent || event;\n        const clonedEvent = new nativeEvent.constructor(nativeEvent.type, nativeEvent);\n        Object.defineProperty(clonedEvent, 'target', {\n          writable: true,\n          value: {\n            value: newValue,\n            name\n          }\n        });\n        onChange(clonedEvent, child);\n      }\n    }\n    if (!multiple) {\n      update(false, event);\n    }\n  };\n  const handleKeyDown = event => {\n    if (!readOnly) {\n      const validKeys = [' ', 'ArrowUp', 'ArrowDown',\n      // The native select doesn't respond to enter on macOS, but it's recommended by\n      // https://www.w3.org/WAI/ARIA/apg/patterns/combobox/examples/combobox-select-only/\n      'Enter'];\n      if (validKeys.includes(event.key)) {\n        event.preventDefault();\n        update(true, event);\n      }\n    }\n  };\n  const open = displayNode !== null && openState;\n  const handleBlur = event => {\n    // if open event.stopImmediatePropagation\n    if (!open && onBlur) {\n      // Preact support, target is read only property on a native event.\n      Object.defineProperty(event, 'target', {\n        writable: true,\n        value: {\n          value,\n          name\n        }\n      });\n      onBlur(event);\n    }\n  };\n  delete other['aria-invalid'];\n  let display;\n  let displaySingle;\n  const displayMultiple = [];\n  let computeDisplay = false;\n  let foundMatch = false;\n\n  // No need to display any value if the field is empty.\n  if ((0,_InputBase_utils_js__WEBPACK_IMPORTED_MODULE_12__.isFilled)({\n    value\n  }) || displayEmpty) {\n    if (renderValue) {\n      display = renderValue(value);\n    } else {\n      computeDisplay = true;\n    }\n  }\n  const items = childrenArray.map(child => {\n    if (! /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(child)) {\n      return null;\n    }\n    if (true) {\n      if ((0,react_is__WEBPACK_IMPORTED_MODULE_13__.isFragment)(child)) {\n        console.error([\"MUI: The Select component doesn't accept a Fragment as a child.\", 'Consider providing an array instead.'].join('\\n'));\n      }\n    }\n    let selected;\n    if (multiple) {\n      if (!Array.isArray(value)) {\n        throw new Error( true ? 'MUI: The `value` prop must be an array ' + 'when using the `Select` component with `multiple`.' : 0);\n      }\n      selected = value.some(v => areEqualValues(v, child.props.value));\n      if (selected && computeDisplay) {\n        displayMultiple.push(child.props.children);\n      }\n    } else {\n      selected = areEqualValues(value, child.props.value);\n      if (selected && computeDisplay) {\n        displaySingle = child.props.children;\n      }\n    }\n    if (selected) {\n      foundMatch = true;\n    }\n    return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(child, {\n      'aria-selected': selected ? 'true' : 'false',\n      onClick: handleItemClick(child),\n      onKeyUp: event => {\n        if (event.key === ' ') {\n          // otherwise our MenuItems dispatches a click event\n          // it's not behavior of the native <option> and causes\n          // the select to close immediately since we open on space keydown\n          event.preventDefault();\n        }\n        if (child.props.onKeyUp) {\n          child.props.onKeyUp(event);\n        }\n      },\n      role: 'option',\n      selected,\n      value: undefined,\n      // The value is most likely not a valid HTML attribute.\n      'data-value': child.props.value // Instead, we provide it as a data attribute.\n    });\n  });\n  if (true) {\n    // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler\n    // eslint-disable-next-line react-hooks/rules-of-hooks\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n      if (!foundMatch && !multiple && value !== '') {\n        const values = childrenArray.map(child => child.props.value);\n        console.warn([`MUI: You have provided an out-of-range value \\`${value}\\` for the select ${name ? `(name=\"${name}\") ` : ''}component.`, \"Consider providing a value that matches one of the available options or ''.\", `The available values are ${values.filter(x => x != null).map(x => `\\`${x}\\``).join(', ') || '\"\"'}.`].join('\\n'));\n      }\n    }, [foundMatch, childrenArray, multiple, name, value]);\n  }\n  if (computeDisplay) {\n    if (multiple) {\n      if (displayMultiple.length === 0) {\n        display = null;\n      } else {\n        display = displayMultiple.reduce((output, child, index) => {\n          output.push(child);\n          if (index < displayMultiple.length - 1) {\n            output.push(', ');\n          }\n          return output;\n        }, []);\n      }\n    } else {\n      display = displaySingle;\n    }\n  }\n\n  // Avoid performing a layout computation in the render method.\n  let menuMinWidth = menuMinWidthState;\n  if (!autoWidth && isOpenControlled && displayNode) {\n    menuMinWidth = anchorElement.clientWidth;\n  }\n  let tabIndex;\n  if (typeof tabIndexProp !== 'undefined') {\n    tabIndex = tabIndexProp;\n  } else {\n    tabIndex = disabled ? null : 0;\n  }\n  const buttonId = SelectDisplayProps.id || (name ? `mui-component-select-${name}` : undefined);\n  const ownerState = {\n    ...props,\n    variant,\n    value,\n    open,\n    error\n  };\n  const classes = useUtilityClasses(ownerState);\n  const paperProps = {\n    ...MenuProps.PaperProps,\n    ...MenuProps.slotProps?.paper\n  };\n  const listboxId = (0,_mui_utils_useId__WEBPACK_IMPORTED_MODULE_14__[\"default\"])();\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(SelectSelect, {\n      as: \"div\",\n      ref: handleDisplayRef,\n      tabIndex: tabIndex,\n      role: \"combobox\",\n      \"aria-controls\": open ? listboxId : undefined,\n      \"aria-disabled\": disabled ? 'true' : undefined,\n      \"aria-expanded\": open ? 'true' : 'false',\n      \"aria-haspopup\": \"listbox\",\n      \"aria-label\": ariaLabel,\n      \"aria-labelledby\": [labelId, buttonId].filter(Boolean).join(' ') || undefined,\n      \"aria-describedby\": ariaDescribedby,\n      \"aria-required\": required ? 'true' : undefined,\n      \"aria-invalid\": error ? 'true' : undefined,\n      onKeyDown: handleKeyDown,\n      onMouseDown: disabled || readOnly ? null : handleMouseDown,\n      onBlur: handleBlur,\n      onFocus: onFocus,\n      ...SelectDisplayProps,\n      ownerState: ownerState,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(SelectDisplayProps.className, classes.select, className)\n      // The id is required for proper a11y\n      ,\n      id: buttonId,\n      children: isEmpty(display) ? // notranslate needed while Google Translate will not fix zero-width space issue\n      _span || (_span = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"span\", {\n        className: \"notranslate\",\n        \"aria-hidden\": true,\n        children: \"\\u200B\"\n      })) : display\n    }), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(SelectNativeInput, {\n      \"aria-invalid\": error,\n      value: Array.isArray(value) ? value.join(',') : value,\n      name: name,\n      ref: inputRef,\n      \"aria-hidden\": true,\n      onChange: handleChange,\n      tabIndex: -1,\n      disabled: disabled,\n      className: classes.nativeInput,\n      autoFocus: autoFocus,\n      required: required,\n      ...other,\n      ownerState: ownerState\n    }), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(SelectIcon, {\n      as: IconComponent,\n      className: classes.icon,\n      ownerState: ownerState\n    }), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_Menu_Menu_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"], {\n      id: `menu-${name || ''}`,\n      anchorEl: anchorElement,\n      open: open,\n      onClose: handleClose,\n      anchorOrigin: {\n        vertical: 'bottom',\n        horizontal: 'center'\n      },\n      transformOrigin: {\n        vertical: 'top',\n        horizontal: 'center'\n      },\n      ...MenuProps,\n      slotProps: {\n        ...MenuProps.slotProps,\n        list: {\n          'aria-labelledby': labelId,\n          role: 'listbox',\n          'aria-multiselectable': multiple ? 'true' : undefined,\n          disableListWrap: true,\n          id: listboxId,\n          ...MenuProps.MenuListProps\n        },\n        paper: {\n          ...paperProps,\n          style: {\n            minWidth: menuMinWidth,\n            ...(paperProps != null ? paperProps.style : null)\n          }\n        }\n      },\n      children: items\n    })]\n  });\n});\n true ? SelectInput.propTypes = {\n  /**\n   * @ignore\n   */\n  'aria-describedby': (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * @ignore\n   */\n  'aria-label': (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * @ignore\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the width of the popover will automatically be set according to the items inside the\n   * menu, otherwise it will be at least the width of the select input.\n   */\n  autoWidth: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * The option elements to populate the select with.\n   * Can be some `<MenuItem>` elements.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * The CSS class name of the select element.\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * If `true`, the component is toggled on mount. Use when the component open state is not controlled.\n   * You can only use it when the `native` prop is `false` (default).\n   */\n  defaultOpen: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().any),\n  /**\n   * If `true`, the select is disabled.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the selected item is displayed even if its value is empty.\n   */\n  displayEmpty: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the `select input` will indicate an error.\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * The icon that displays the arrow.\n   */\n  IconComponent: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType).isRequired,\n  /**\n   * Imperative handle implementing `{ value: T, node: HTMLElement, focus(): void }`\n   * Equivalent to `ref`\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_17__[\"default\"],\n  /**\n   * The ID of an element that acts as an additional label. The Select will\n   * be labelled by the additional label and the selected value.\n   */\n  labelId: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * Props applied to the [`Menu`](/material-ui/api/menu/) element.\n   */\n  MenuProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * If `true`, `value` must be an array and the menu will support multiple selections.\n   */\n  multiple: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Name attribute of the `select` or hidden `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * @ignore\n   */\n  onBlur: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * Callback fired when a menu item is selected.\n   *\n   * @param {object} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (any).\n   * @param {object} [child] The react element that was selected.\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * Callback fired when the component requests to be closed.\n   * Use in controlled mode (see open).\n   *\n   * @param {object} event The event source of the callback.\n   */\n  onClose: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * @ignore\n   */\n  onFocus: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * Callback fired when the component requests to be opened.\n   * Use in controlled mode (see open).\n   *\n   * @param {object} event The event source of the callback.\n   */\n  onOpen: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * If `true`, the component is shown.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * @ignore\n   */\n  readOnly: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Render the selected value.\n   *\n   * @param {any} value The `value` provided to the component.\n   * @returns {ReactNode}\n   */\n  renderValue: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * If `true`, the component is required.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Props applied to the clickable div element.\n   */\n  SelectDisplayProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * @ignore\n   */\n  tabIndex: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * @ignore\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().any),\n  /**\n   * The input value.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().any),\n  /**\n   * The variant to use.\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['standard', 'outlined', 'filled'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (SelectInput);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Select/SelectInput.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Select/selectClasses.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/Select/selectClasses.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getSelectUtilityClasses: () => (/* binding */ getSelectUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getSelectUtilityClasses(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiSelect', slot);\n}\nconst selectClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiSelect', ['root', 'select', 'multiple', 'filled', 'outlined', 'standard', 'disabled', 'focused', 'icon', 'iconOpen', 'iconFilled', 'iconOutlined', 'iconStandard', 'nativeInput', 'error']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (selectClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Select/selectClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Stack/Stack.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/Stack/Stack.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_3__);\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/Stack/createStack.js\");\n/* harmony import */ var _styles_styled_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../styles/styled.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n'use client';\n\n\n\n\n\nconst Stack = (0,_mui_system__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  createStyledComponent: (0,_styles_styled_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('div', {\n    name: 'MuiStack',\n    slot: 'Root',\n    overridesResolver: (props, styles) => styles.root\n  }),\n  useThemeProps: inProps => (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_2__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiStack'\n  })\n});\n true ? Stack.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().node),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().elementType),\n  /**\n   * Defines the `flex-direction` style property.\n   * It is applied for all screen sizes.\n   * @default 'column'\n   */\n  direction: prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOf(['column-reverse', 'column', 'row-reverse', 'row']), prop_types__WEBPACK_IMPORTED_MODULE_3___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOf(['column-reverse', 'column', 'row-reverse', 'row'])), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object)]),\n  /**\n   * Add an element between each child.\n   */\n  divider: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().node),\n  /**\n   * Defines the space between immediate children.\n   * @default 0\n   */\n  spacing: prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_3___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_3___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().string)])), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().string)]),\n  /**\n   * The system prop, which allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_3___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_3___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object)]),\n  /**\n   * If `true`, the CSS flexbox `gap` is used instead of applying `margin` to children.\n   *\n   * While CSS `gap` removes the [known limitations](https://mui.com/joy-ui/react-stack/#limitations),\n   * it is not fully supported in some browsers. We recommend checking https://caniuse.com/?search=flex%20gap before using this flag.\n   *\n   * To enable this flag globally, follow the [theme's default props](https://mui.com/material-ui/customization/theme-components/#default-props) configuration.\n   * @default false\n   */\n  useFlexGap: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().bool)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Stack);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Stack/Stack.js?");

/***/ }),

/***/ "./node_modules/@mui/material/SvgIcon/SvgIcon.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/material/SvgIcon/SvgIcon.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _svgIconClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./svgIconClasses.js */ \"./node_modules/@mui/material/SvgIcon/svgIconClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    color,\n    fontSize,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', color !== 'inherit' && `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, `fontSize${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(fontSize)}`]\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _svgIconClasses_js__WEBPACK_IMPORTED_MODULE_5__.getSvgIconUtilityClass, classes);\n};\nconst SvgIconRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('svg', {\n  name: 'MuiSvgIcon',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.color !== 'inherit' && styles[`color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color)}`], styles[`fontSize${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.fontSize)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  userSelect: 'none',\n  width: '1em',\n  height: '1em',\n  display: 'inline-block',\n  flexShrink: 0,\n  transition: theme.transitions?.create?.('fill', {\n    duration: (theme.vars ?? theme).transitions?.duration?.shorter\n  }),\n  variants: [{\n    props: props => !props.hasSvgAsChild,\n    style: {\n      // the <svg> will define the property that has `currentColor`\n      // for example heroicons uses fill=\"none\" and stroke=\"currentColor\"\n      fill: 'currentColor'\n    }\n  }, {\n    props: {\n      fontSize: 'inherit'\n    },\n    style: {\n      fontSize: 'inherit'\n    }\n  }, {\n    props: {\n      fontSize: 'small'\n    },\n    style: {\n      fontSize: theme.typography?.pxToRem?.(20) || '1.25rem'\n    }\n  }, {\n    props: {\n      fontSize: 'medium'\n    },\n    style: {\n      fontSize: theme.typography?.pxToRem?.(24) || '1.5rem'\n    }\n  }, {\n    props: {\n      fontSize: 'large'\n    },\n    style: {\n      fontSize: theme.typography?.pxToRem?.(35) || '2.1875rem'\n    }\n  },\n  // TODO v5 deprecate color prop, v6 remove for sx\n  ...Object.entries((theme.vars ?? theme).palette).filter(([, value]) => value && value.main).map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      color: (theme.vars ?? theme).palette?.[color]?.main\n    }\n  })), {\n    props: {\n      color: 'action'\n    },\n    style: {\n      color: (theme.vars ?? theme).palette?.action?.active\n    }\n  }, {\n    props: {\n      color: 'disabled'\n    },\n    style: {\n      color: (theme.vars ?? theme).palette?.action?.disabled\n    }\n  }, {\n    props: {\n      color: 'inherit'\n    },\n    style: {\n      color: undefined\n    }\n  }]\n})));\nconst SvgIcon = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function SvgIcon(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_8__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiSvgIcon'\n  });\n  const {\n    children,\n    className,\n    color = 'inherit',\n    component = 'svg',\n    fontSize = 'medium',\n    htmlColor,\n    inheritViewBox = false,\n    titleAccess,\n    viewBox = '0 0 24 24',\n    ...other\n  } = props;\n  const hasSvgAsChild = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(children) && children.type === 'svg';\n  const ownerState = {\n    ...props,\n    color,\n    component,\n    fontSize,\n    instanceFontSize: inProps.fontSize,\n    inheritViewBox,\n    viewBox,\n    hasSvgAsChild\n  };\n  const more = {};\n  if (!inheritViewBox) {\n    more.viewBox = viewBox;\n  }\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(SvgIconRoot, {\n    as: component,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    focusable: \"false\",\n    color: htmlColor,\n    \"aria-hidden\": titleAccess ? undefined : true,\n    role: titleAccess ? 'img' : undefined,\n    ref: ref,\n    ...more,\n    ...other,\n    ...(hasSvgAsChild && children.props),\n    ownerState: ownerState,\n    children: [hasSvgAsChild ? children.props.children : children, titleAccess ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(\"title\", {\n      children: titleAccess\n    }) : null]\n  });\n});\n true ? SvgIcon.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Node passed into the SVG element.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * You can use the `htmlColor` prop to apply a color attribute to the SVG element.\n   * @default 'inherit'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['inherit', 'action', 'disabled', 'primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string)]),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType),\n  /**\n   * The fontSize applied to the icon. Defaults to 24px, but can be configure to inherit font size.\n   * @default 'medium'\n   */\n  fontSize: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['inherit', 'large', 'medium', 'small']), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string)]),\n  /**\n   * Applies a color attribute to the SVG element.\n   */\n  htmlColor: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * If `true`, the root node will inherit the custom `component`'s viewBox and the `viewBox`\n   * prop will be ignored.\n   * Useful when you want to reference a custom `component` and have `SvgIcon` pass that\n   * `component`'s viewBox to the root node.\n   * @default false\n   */\n  inheritViewBox: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n  /**\n   * The shape-rendering attribute. The behavior of the different options is described on the\n   * [MDN Web Docs](https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/shape-rendering).\n   * If you are having issues with blurry icons you should investigate this prop.\n   */\n  shapeRendering: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)]),\n  /**\n   * Provides a human-readable title for the element that contains it.\n   * https://www.w3.org/TR/SVG-access/#Equivalent\n   */\n  titleAccess: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n  /**\n   * Allows you to redefine what the coordinates without units mean inside an SVG element.\n   * For example, if the SVG element is 500 (width) by 200 (height),\n   * and you pass viewBox=\"0 0 50 20\",\n   * this means that the coordinates inside the SVG will go from the top left corner (0,0)\n   * to bottom right (50,20) and each unit will be worth 10px.\n   * @default '0 0 24 24'\n   */\n  viewBox: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string)\n} : 0;\nSvgIcon.muiName = 'SvgIcon';\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (SvgIcon);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/SvgIcon/SvgIcon.js?");

/***/ }),

/***/ "./node_modules/@mui/material/SvgIcon/svgIconClasses.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/SvgIcon/svgIconClasses.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getSvgIconUtilityClass: () => (/* binding */ getSvgIconUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getSvgIconUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiSvgIcon', slot);\n}\nconst svgIconClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiSvgIcon', ['root', 'colorPrimary', 'colorSecondary', 'colorAction', 'colorError', 'colorDisabled', 'fontSizeInherit', 'fontSizeSmall', 'fontSizeMedium', 'fontSizeLarge']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (svgIconClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/SvgIcon/svgIconClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Switch/Switch.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/Switch/Switch.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_13___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_13__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _internal_SwitchBase_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../internal/SwitchBase.js */ \"./node_modules/@mui/material/internal/SwitchBase.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _switchClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./switchClasses.js */ \"./node_modules/@mui/material/Switch/switchClasses.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n// @inheritedComponent IconButton\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    edge,\n    size,\n    color,\n    checked,\n    disabled\n  } = ownerState;\n  const slots = {\n    root: ['root', edge && `edge${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(edge)}`, `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`],\n    switchBase: ['switchBase', `color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(color)}`, checked && 'checked', disabled && 'disabled'],\n    thumb: ['thumb'],\n    track: ['track'],\n    input: ['input']\n  };\n  const composedClasses = (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _switchClasses_js__WEBPACK_IMPORTED_MODULE_5__.getSwitchUtilityClass, classes);\n  return {\n    ...classes,\n    // forward the disabled and checked classes to the SwitchBase\n    ...composedClasses\n  };\n};\nconst SwitchRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiSwitch',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.edge && styles[`edge${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.edge)}`], styles[`size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`]];\n  }\n})({\n  display: 'inline-flex',\n  width: 34 + 12 * 2,\n  height: 14 + 12 * 2,\n  overflow: 'hidden',\n  padding: 12,\n  boxSizing: 'border-box',\n  position: 'relative',\n  flexShrink: 0,\n  zIndex: 0,\n  // Reset the stacking context.\n  verticalAlign: 'middle',\n  // For correct alignment with the text.\n  '@media print': {\n    colorAdjust: 'exact'\n  },\n  variants: [{\n    props: {\n      edge: 'start'\n    },\n    style: {\n      marginLeft: -8\n    }\n  }, {\n    props: {\n      edge: 'end'\n    },\n    style: {\n      marginRight: -8\n    }\n  }, {\n    props: {\n      size: 'small'\n    },\n    style: {\n      width: 40,\n      height: 24,\n      padding: 7,\n      [`& .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].thumb}`]: {\n        width: 16,\n        height: 16\n      },\n      [`& .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].switchBase}`]: {\n        padding: 4,\n        [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].checked}`]: {\n          transform: 'translateX(16px)'\n        }\n      }\n    }\n  }]\n});\nconst SwitchSwitchBase = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_internal_SwitchBase_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiSwitch',\n  slot: 'SwitchBase',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.switchBase, {\n      [`& .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].input}`]: styles.input\n    }, ownerState.color !== 'default' && styles[`color${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.color)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  position: 'absolute',\n  top: 0,\n  left: 0,\n  zIndex: 1,\n  // Render above the focus ripple.\n  color: theme.vars ? theme.vars.palette.Switch.defaultColor : `${theme.palette.mode === 'light' ? theme.palette.common.white : theme.palette.grey[300]}`,\n  transition: theme.transitions.create(['left', 'transform'], {\n    duration: theme.transitions.duration.shortest\n  }),\n  [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].checked}`]: {\n    transform: 'translateX(20px)'\n  },\n  [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    color: theme.vars ? theme.vars.palette.Switch.defaultDisabledColor : `${theme.palette.mode === 'light' ? theme.palette.grey[100] : theme.palette.grey[600]}`\n  },\n  [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].checked} + .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].track}`]: {\n    opacity: 0.5\n  },\n  [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled} + .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].track}`]: {\n    opacity: theme.vars ? theme.vars.opacity.switchTrackDisabled : `${theme.palette.mode === 'light' ? 0.12 : 0.2}`\n  },\n  [`& .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].input}`]: {\n    left: '-100%',\n    width: '300%'\n  }\n})), (0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  '&:hover': {\n    backgroundColor: theme.vars ? `rgba(${theme.vars.palette.action.activeChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.action.active, theme.palette.action.hoverOpacity),\n    // Reset on touch devices, it doesn't add specificity\n    '@media (hover: none)': {\n      backgroundColor: 'transparent'\n    }\n  },\n  variants: [...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(['light'])) // check all the used fields in the style below\n  .map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].checked}`]: {\n        color: (theme.vars || theme).palette[color].main,\n        '&:hover': {\n          backgroundColor: theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette[color].main, theme.palette.action.hoverOpacity),\n          '@media (hover: none)': {\n            backgroundColor: 'transparent'\n          }\n        },\n        [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n          color: theme.vars ? theme.vars.palette.Switch[`${color}DisabledColor`] : `${theme.palette.mode === 'light' ? (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.lighten)(theme.palette[color].main, 0.62) : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.darken)(theme.palette[color].main, 0.55)}`\n        }\n      },\n      [`&.${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].checked} + .${_switchClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].track}`]: {\n        backgroundColor: (theme.vars || theme).palette[color].main\n      }\n    }\n  }))]\n})));\nconst SwitchTrack = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiSwitch',\n  slot: 'Track',\n  overridesResolver: (props, styles) => styles.track\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  height: '100%',\n  width: '100%',\n  borderRadius: 14 / 2,\n  zIndex: -1,\n  transition: theme.transitions.create(['opacity', 'background-color'], {\n    duration: theme.transitions.duration.shortest\n  }),\n  backgroundColor: theme.vars ? theme.vars.palette.common.onBackground : `${theme.palette.mode === 'light' ? theme.palette.common.black : theme.palette.common.white}`,\n  opacity: theme.vars ? theme.vars.opacity.switchTrack : `${theme.palette.mode === 'light' ? 0.38 : 0.3}`\n})));\nconst SwitchThumb = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('span', {\n  name: 'MuiSwitch',\n  slot: 'Thumb',\n  overridesResolver: (props, styles) => styles.thumb\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  boxShadow: (theme.vars || theme).shadows[1],\n  backgroundColor: 'currentColor',\n  width: 20,\n  height: 20,\n  borderRadius: '50%'\n})));\nconst Switch = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Switch(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_11__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiSwitch'\n  });\n  const {\n    className,\n    color = 'primary',\n    edge = false,\n    size = 'medium',\n    sx,\n    slots = {},\n    slotProps = {},\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    color,\n    edge,\n    size\n  };\n  const classes = useUtilityClasses(ownerState);\n  const externalForwardedProps = {\n    slots,\n    slotProps\n  };\n  const [RootSlot, rootSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('root', {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    elementType: SwitchRoot,\n    externalForwardedProps,\n    ownerState,\n    additionalProps: {\n      sx\n    }\n  });\n  const [ThumbSlot, thumbSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('thumb', {\n    className: classes.thumb,\n    elementType: SwitchThumb,\n    externalForwardedProps,\n    ownerState\n  });\n  const icon = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ThumbSlot, {\n    ...thumbSlotProps\n  });\n  const [TrackSlot, trackSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('track', {\n    className: classes.track,\n    elementType: SwitchTrack,\n    externalForwardedProps,\n    ownerState\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(RootSlot, {\n    ...rootSlotProps,\n    children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(SwitchSwitchBase, {\n      type: \"checkbox\",\n      icon: icon,\n      checkedIcon: icon,\n      ref: ref,\n      ownerState: ownerState,\n      ...other,\n      classes: {\n        ...classes,\n        root: classes.switchBase\n      },\n      slots: {\n        ...(slots.switchBase && {\n          root: slots.switchBase\n        }),\n        ...(slots.input && {\n          input: slots.input\n        })\n      },\n      slotProps: {\n        ...(slotProps.switchBase && {\n          root: typeof slotProps.switchBase === 'function' ? slotProps.switchBase(ownerState) : slotProps.switchBase\n        }),\n        ...(slotProps.input && {\n          input: typeof slotProps.input === 'function' ? slotProps.input(ownerState) : slotProps.input\n        })\n      }\n    }), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TrackSlot, {\n      ...trackSlotProps\n    })]\n  });\n});\n true ? Switch.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * If `true`, the component is checked.\n   */\n  checked: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The icon to display when the component is checked.\n   */\n  checkedIcon: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'primary'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['default', 'primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)]),\n  /**\n   * The default checked state. Use when the component is not controlled.\n   */\n  defaultChecked: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the component is disabled.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If `true`, the ripple effect is disabled.\n   * @default false\n   */\n  disableRipple: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * If given, uses a negative margin to counteract the padding on one\n   * side (this is often helpful for aligning the left or right\n   * side of the icon with content above or below, without ruining the border\n   * size and shape).\n   * @default false\n   */\n  edge: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['end', 'start', false]),\n  /**\n   * The icon to display when the component is unchecked.\n   */\n  icon: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().node),\n  /**\n   * The id of the `input` element.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * @deprecated Use `slotProps.input` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   * @deprecated Use `slotProps.input.ref` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_14__[\"default\"],\n  /**\n   * Callback fired when the state is changed.\n   *\n   * @param {React.ChangeEvent<HTMLInputElement>} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   * You can pull out the new checked state by accessing `event.target.checked` (boolean).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func),\n  /**\n   * If `true`, the `input` element is required.\n   * @default false\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool),\n  /**\n   * The size of the component.\n   * `small` is equivalent to the dense switch styling.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOf(['medium', 'small']), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().string)]),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    input: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n    switchBase: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n    thumb: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n    track: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_13___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    switchBase: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    thumb: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType),\n    track: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_13___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_13___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_13___default().object)]),\n  /**\n   * The value of the component. The DOM API casts this to a string.\n   * The browser uses \"on\" as the default value.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_13___default().any)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Switch);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Switch/Switch.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Switch/switchClasses.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/Switch/switchClasses.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getSwitchUtilityClass: () => (/* binding */ getSwitchUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getSwitchUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiSwitch', slot);\n}\nconst switchClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiSwitch', ['root', 'edgeStart', 'edgeEnd', 'switchBase', 'colorPrimary', 'colorSecondary', 'sizeSmall', 'sizeMedium', 'checked', 'disabled', 'input', 'thumb', 'track']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (switchClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Switch/switchClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/TextField/TextField.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/TextField/TextField.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_16___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_16__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_useId__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! @mui/utils/useId */ \"./node_modules/@mui/utils/esm/useId/useId.js\");\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _Input_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../Input/index.js */ \"./node_modules/@mui/material/Input/Input.js\");\n/* harmony import */ var _FilledInput_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../FilledInput/index.js */ \"./node_modules/@mui/material/FilledInput/FilledInput.js\");\n/* harmony import */ var _OutlinedInput_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../OutlinedInput/index.js */ \"./node_modules/@mui/material/OutlinedInput/OutlinedInput.js\");\n/* harmony import */ var _InputLabel_index_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ../InputLabel/index.js */ \"./node_modules/@mui/material/InputLabel/InputLabel.js\");\n/* harmony import */ var _FormControl_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../FormControl/index.js */ \"./node_modules/@mui/material/FormControl/FormControl.js\");\n/* harmony import */ var _FormHelperText_index_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../FormHelperText/index.js */ \"./node_modules/@mui/material/FormHelperText/FormHelperText.js\");\n/* harmony import */ var _Select_index_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../Select/index.js */ \"./node_modules/@mui/material/Select/Select.js\");\n/* harmony import */ var _textFieldClasses_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./textFieldClasses.js */ \"./node_modules/@mui/material/TextField/textFieldClasses.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst variantComponent = {\n  standard: _Input_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n  filled: _FilledInput_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"],\n  outlined: _OutlinedInput_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"]\n};\nconst useUtilityClasses = ownerState => {\n  const {\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(slots, _textFieldClasses_js__WEBPACK_IMPORTED_MODULE_7__.getTextFieldUtilityClass, classes);\n};\nconst TextFieldRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(_FormControl_index_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"], {\n  name: 'MuiTextField',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n})({});\n\n/**\n * The `TextField` is a convenience wrapper for the most common cases (80%).\n * It cannot be all things to all people, otherwise the API would grow out of control.\n *\n * ## Advanced Configuration\n *\n * It's important to understand that the text field is a simple abstraction\n * on top of the following components:\n *\n * - [FormControl](/material-ui/api/form-control/)\n * - [InputLabel](/material-ui/api/input-label/)\n * - [FilledInput](/material-ui/api/filled-input/)\n * - [OutlinedInput](/material-ui/api/outlined-input/)\n * - [Input](/material-ui/api/input/)\n * - [FormHelperText](/material-ui/api/form-helper-text/)\n *\n * If you wish to alter the props applied to the `input` element, you can do so as follows:\n *\n * ```jsx\n * const inputProps = {\n *   step: 300,\n * };\n *\n * return <TextField id=\"time\" type=\"time\" inputProps={inputProps} />;\n * ```\n *\n * For advanced cases, please look at the source of TextField by clicking on the\n * \"Edit this page\" button above. Consider either:\n *\n * - using the upper case props for passing values directly to the components\n * - using the underlying components directly as shown in the demos\n */\nconst TextField = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function TextField(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiTextField'\n  });\n  const {\n    autoComplete,\n    autoFocus = false,\n    children,\n    className,\n    color = 'primary',\n    defaultValue,\n    disabled = false,\n    error = false,\n    FormHelperTextProps: FormHelperTextPropsProp,\n    fullWidth = false,\n    helperText,\n    id: idOverride,\n    InputLabelProps: InputLabelPropsProp,\n    inputProps: inputPropsProp,\n    InputProps: InputPropsProp,\n    inputRef,\n    label,\n    maxRows,\n    minRows,\n    multiline = false,\n    name,\n    onBlur,\n    onChange,\n    onFocus,\n    placeholder,\n    required = false,\n    rows,\n    select = false,\n    SelectProps: SelectPropsProp,\n    slots = {},\n    slotProps = {},\n    type,\n    value,\n    variant = 'outlined',\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    autoFocus,\n    color,\n    disabled,\n    error,\n    fullWidth,\n    multiline,\n    required,\n    select,\n    variant\n  };\n  const classes = useUtilityClasses(ownerState);\n  if (true) {\n    if (select && !children) {\n      console.error('MUI: `children` must be passed when using the `TextField` component with `select`.');\n    }\n  }\n  const id = (0,_mui_utils_useId__WEBPACK_IMPORTED_MODULE_11__[\"default\"])(idOverride);\n  const helperTextId = helperText && id ? `${id}-helper-text` : undefined;\n  const inputLabelId = label && id ? `${id}-label` : undefined;\n  const InputComponent = variantComponent[variant];\n  const externalForwardedProps = {\n    slots,\n    slotProps: {\n      input: InputPropsProp,\n      inputLabel: InputLabelPropsProp,\n      htmlInput: inputPropsProp,\n      formHelperText: FormHelperTextPropsProp,\n      select: SelectPropsProp,\n      ...slotProps\n    }\n  };\n  const inputAdditionalProps = {};\n  const inputLabelSlotProps = externalForwardedProps.slotProps.inputLabel;\n  if (variant === 'outlined') {\n    if (inputLabelSlotProps && typeof inputLabelSlotProps.shrink !== 'undefined') {\n      inputAdditionalProps.notched = inputLabelSlotProps.shrink;\n    }\n    inputAdditionalProps.label = label;\n  }\n  if (select) {\n    // unset defaults from textbox inputs\n    if (!SelectPropsProp || !SelectPropsProp.native) {\n      inputAdditionalProps.id = undefined;\n    }\n    inputAdditionalProps['aria-describedby'] = undefined;\n  }\n  const [RootSlot, rootProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('root', {\n    elementType: TextFieldRoot,\n    shouldForwardComponentProp: true,\n    externalForwardedProps: {\n      ...externalForwardedProps,\n      ...other\n    },\n    ownerState,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ref,\n    additionalProps: {\n      disabled,\n      error,\n      fullWidth,\n      required,\n      color,\n      variant\n    }\n  });\n  const [InputSlot, inputProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('input', {\n    elementType: InputComponent,\n    externalForwardedProps,\n    additionalProps: inputAdditionalProps,\n    ownerState\n  });\n  const [InputLabelSlot, inputLabelProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('inputLabel', {\n    elementType: _InputLabel_index_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  const [HtmlInputSlot, htmlInputProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('htmlInput', {\n    elementType: 'input',\n    externalForwardedProps,\n    ownerState\n  });\n  const [FormHelperTextSlot, formHelperTextProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('formHelperText', {\n    elementType: _FormHelperText_index_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  const [SelectSlot, selectProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"])('select', {\n    elementType: _Select_index_js__WEBPACK_IMPORTED_MODULE_15__[\"default\"],\n    externalForwardedProps,\n    ownerState\n  });\n  const InputElement = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(InputSlot, {\n    \"aria-describedby\": helperTextId,\n    autoComplete: autoComplete,\n    autoFocus: autoFocus,\n    defaultValue: defaultValue,\n    fullWidth: fullWidth,\n    multiline: multiline,\n    name: name,\n    rows: rows,\n    maxRows: maxRows,\n    minRows: minRows,\n    type: type,\n    value: value,\n    id: id,\n    inputRef: inputRef,\n    onBlur: onBlur,\n    onChange: onChange,\n    onFocus: onFocus,\n    placeholder: placeholder,\n    inputProps: htmlInputProps,\n    slots: {\n      input: slots.htmlInput ? HtmlInputSlot : undefined\n    },\n    ...inputProps\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsxs)(RootSlot, {\n    ...rootProps,\n    children: [label != null && label !== '' && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(InputLabelSlot, {\n      htmlFor: id,\n      id: inputLabelId,\n      ...inputLabelProps,\n      children: label\n    }), select ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(SelectSlot, {\n      \"aria-describedby\": helperTextId,\n      id: id,\n      labelId: inputLabelId,\n      value: value,\n      input: InputElement,\n      ...selectProps,\n      children: children\n    }) : InputElement, helperText && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(FormHelperTextSlot, {\n      id: helperTextId,\n      ...formHelperTextProps,\n      children: helperText\n    })]\n  });\n});\n true ? TextField.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * This prop helps users to fill forms faster, especially on mobile devices.\n   * The name can be confusing, as it's more like an autofill.\n   * You can learn more about it [following the specification](https://html.spec.whatwg.org/multipage/form-control-infrastructure.html#autofill).\n   */\n  autoComplete: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * If `true`, the `input` element is focused during the first mount.\n   * @default false\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * @ignore\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'primary'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * The default value. Use when the component is not controlled.\n   */\n  defaultValue: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().any),\n  /**\n   * If `true`, the component is disabled.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the label is displayed in an error state.\n   * @default false\n   */\n  error: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Props applied to the [`FormHelperText`](https://mui.com/material-ui/api/form-helper-text/) element.\n   * @deprecated Use `slotProps.formHelperText` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  FormHelperTextProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * If `true`, the input will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * The helper text content.\n   */\n  helperText: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().node),\n  /**\n   * The id of the `input` element.\n   * Use this prop to make `label` and `helperText` accessible for screen readers.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * Props applied to the [`InputLabel`](https://mui.com/material-ui/api/input-label/) element.\n   * Pointer events like `onClick` are enabled if and only if `shrink` is `true`.\n   * @deprecated Use `slotProps.inputLabel` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  InputLabelProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   * @deprecated Use `slotProps.htmlInput` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * Props applied to the Input element.\n   * It will be a [`FilledInput`](https://mui.com/material-ui/api/filled-input/),\n   * [`OutlinedInput`](https://mui.com/material-ui/api/outlined-input/) or [`Input`](https://mui.com/material-ui/api/input/)\n   * component depending on the `variant` prop value.\n   * @deprecated Use `slotProps.input` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  InputProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_17__[\"default\"],\n  /**\n   * The label content.\n   */\n  label: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().node),\n  /**\n   * If `dense` or `normal`, will adjust vertical spacing of this and contained components.\n   * @default 'none'\n   */\n  margin: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['dense', 'none', 'normal']),\n  /**\n   * Maximum number of rows to display when multiline option is set to true.\n   */\n  maxRows: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * Minimum number of rows to display when multiline option is set to true.\n   */\n  minRows: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * If `true`, a `textarea` element is rendered instead of an input.\n   * @default false\n   */\n  multiline: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Name attribute of the `input` element.\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * @ignore\n   */\n  onBlur: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * Callback fired when the value is changed.\n   *\n   * @param {object} event The event source of the callback.\n   * You can pull out the new value by accessing `event.target.value` (string).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * @ignore\n   */\n  onFocus: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * The short hint displayed in the `input` before the user enters a value.\n   */\n  placeholder: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * If `true`, the label is displayed as required and the `input` element is required.\n   * @default false\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Number of rows to display when multiline option is set to true.\n   */\n  rows: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * Render a [`Select`](https://mui.com/material-ui/api/select/) element while passing the Input element to `Select` as `input` parameter.\n   * If this option is set you must pass the options of the select as children.\n   * @default false\n   */\n  select: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Props applied to the [`Select`](https://mui.com/material-ui/api/select/) element.\n   * @deprecated Use `slotProps.select` instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  SelectProps: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * The size of the component.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['medium', 'small']), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_16___default().shape({\n    formHelperText: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)]),\n    htmlInput: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)]),\n    input: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)]),\n    inputLabel: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)]),\n    select: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_16___default().shape({\n    formHelperText: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType),\n    htmlInput: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType),\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType),\n    inputLabel: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType),\n    select: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_16___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)]),\n  /**\n   * Type of the `input` element. It should be [a valid HTML5 input type](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Form_%3Cinput%3E_types).\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * The value of the `input` element, required for a controlled component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().any),\n  /**\n   * The variant to use.\n   * @default 'outlined'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['filled', 'outlined', 'standard'])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (TextField);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/TextField/TextField.js?");

/***/ }),

/***/ "./node_modules/@mui/material/TextField/textFieldClasses.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/TextField/textFieldClasses.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getTextFieldUtilityClass: () => (/* binding */ getTextFieldUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getTextFieldUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiTextField', slot);\n}\nconst textFieldClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiTextField', ['root']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (textFieldClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/TextField/textFieldClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/TextareaAutosize/TextareaAutosize.js":
/*!*************************************************************************!*\
  !*** ./node_modules/@mui/material/TextareaAutosize/TextareaAutosize.js ***!
  \*************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_7__);\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/ownerWindow/ownerWindow.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useEventCallback/useEventCallback.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/debounce/debounce.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\nfunction getStyleValue(value) {\n  return parseInt(value, 10) || 0;\n}\nconst styles = {\n  shadow: {\n    // Visibility needed to hide the extra text area on iPads\n    visibility: 'hidden',\n    // Remove from the content flow\n    position: 'absolute',\n    // Ignore the scrollbar width\n    overflow: 'hidden',\n    height: 0,\n    top: 0,\n    left: 0,\n    // Create a new layer, increase the isolation of the computed values\n    transform: 'translateZ(0)'\n  }\n};\nfunction isObjectEmpty(object) {\n  // eslint-disable-next-line\n  for (const _ in object) {\n    return false;\n  }\n  return true;\n}\nfunction isEmpty(obj) {\n  return isObjectEmpty(obj) || obj.outerHeightStyle === 0 && !obj.overflowing;\n}\n\n/**\n *\n * Demos:\n *\n * - [Textarea Autosize](https://v6.mui.com/material-ui/react-textarea-autosize/)\n *\n * API:\n *\n * - [TextareaAutosize API](https://v6.mui.com/material-ui/api/textarea-autosize/)\n */\nconst TextareaAutosize = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function TextareaAutosize(props, forwardedRef) {\n  const {\n    onChange,\n    maxRows,\n    minRows = 1,\n    style,\n    value,\n    ...other\n  } = props;\n  const {\n    current: isControlled\n  } = react__WEBPACK_IMPORTED_MODULE_0__.useRef(value != null);\n  const textareaRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const handleRef = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(forwardedRef, textareaRef);\n  const heightRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const hiddenTextareaRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const calculateTextareaStyles = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    const textarea = textareaRef.current;\n    const hiddenTextarea = hiddenTextareaRef.current;\n    if (!textarea || !hiddenTextarea) {\n      return undefined;\n    }\n    const containerWindow = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(textarea);\n    const computedStyle = containerWindow.getComputedStyle(textarea);\n\n    // If input's width is shrunk and it's not visible, don't sync height.\n    if (computedStyle.width === '0px') {\n      return {\n        outerHeightStyle: 0,\n        overflowing: false\n      };\n    }\n    hiddenTextarea.style.width = computedStyle.width;\n    hiddenTextarea.value = textarea.value || props.placeholder || 'x';\n    if (hiddenTextarea.value.slice(-1) === '\\n') {\n      // Certain fonts which overflow the line height will cause the textarea\n      // to report a different scrollHeight depending on whether the last line\n      // is empty. Make it non-empty to avoid this issue.\n      hiddenTextarea.value += ' ';\n    }\n    const boxSizing = computedStyle.boxSizing;\n    const padding = getStyleValue(computedStyle.paddingBottom) + getStyleValue(computedStyle.paddingTop);\n    const border = getStyleValue(computedStyle.borderBottomWidth) + getStyleValue(computedStyle.borderTopWidth);\n\n    // The height of the inner content\n    const innerHeight = hiddenTextarea.scrollHeight;\n\n    // Measure height of a textarea with a single row\n    hiddenTextarea.value = 'x';\n    const singleRowHeight = hiddenTextarea.scrollHeight;\n\n    // The height of the outer content\n    let outerHeight = innerHeight;\n    if (minRows) {\n      outerHeight = Math.max(Number(minRows) * singleRowHeight, outerHeight);\n    }\n    if (maxRows) {\n      outerHeight = Math.min(Number(maxRows) * singleRowHeight, outerHeight);\n    }\n    outerHeight = Math.max(outerHeight, singleRowHeight);\n\n    // Take the box sizing into account for applying this value as a style.\n    const outerHeightStyle = outerHeight + (boxSizing === 'border-box' ? padding + border : 0);\n    const overflowing = Math.abs(outerHeight - innerHeight) <= 1;\n    return {\n      outerHeightStyle,\n      overflowing\n    };\n  }, [maxRows, minRows, props.placeholder]);\n  const didHeightChange = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(() => {\n    const textarea = textareaRef.current;\n    const textareaStyles = calculateTextareaStyles();\n    if (!textarea || !textareaStyles || isEmpty(textareaStyles)) {\n      return false;\n    }\n    const outerHeightStyle = textareaStyles.outerHeightStyle;\n    return heightRef.current != null && heightRef.current !== outerHeightStyle;\n  });\n  const syncHeight = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(() => {\n    const textarea = textareaRef.current;\n    const textareaStyles = calculateTextareaStyles();\n    if (!textarea || !textareaStyles || isEmpty(textareaStyles)) {\n      return;\n    }\n    const outerHeightStyle = textareaStyles.outerHeightStyle;\n    if (heightRef.current !== outerHeightStyle) {\n      heightRef.current = outerHeightStyle;\n      textarea.style.height = `${outerHeightStyle}px`;\n    }\n    textarea.style.overflow = textareaStyles.overflowing ? 'hidden' : '';\n  }, [calculateTextareaStyles]);\n  const frameRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(-1);\n  (0,_mui_utils__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(() => {\n    const debouncedHandleResize = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(syncHeight);\n    const textarea = textareaRef?.current;\n    if (!textarea) {\n      return undefined;\n    }\n    const containerWindow = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(textarea);\n    containerWindow.addEventListener('resize', debouncedHandleResize);\n    let resizeObserver;\n    if (typeof ResizeObserver !== 'undefined') {\n      resizeObserver = new ResizeObserver(() => {\n        if (didHeightChange()) {\n          // avoid \"ResizeObserver loop completed with undelivered notifications\" error\n          // by temporarily unobserving the textarea element while manipulating the height\n          // and reobserving one frame later\n          resizeObserver.unobserve(textarea);\n          cancelAnimationFrame(frameRef.current);\n          syncHeight();\n          frameRef.current = requestAnimationFrame(() => {\n            resizeObserver.observe(textarea);\n          });\n        }\n      });\n      resizeObserver.observe(textarea);\n    }\n    return () => {\n      debouncedHandleResize.clear();\n      cancelAnimationFrame(frameRef.current);\n      containerWindow.removeEventListener('resize', debouncedHandleResize);\n      if (resizeObserver) {\n        resizeObserver.disconnect();\n      }\n    };\n  }, [calculateTextareaStyles, syncHeight, didHeightChange]);\n  (0,_mui_utils__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(() => {\n    syncHeight();\n  });\n  const handleChange = event => {\n    if (!isControlled) {\n      syncHeight();\n    }\n    if (onChange) {\n      onChange(event);\n    }\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"textarea\", {\n      value: value,\n      onChange: handleChange,\n      ref: handleRef\n      // Apply the rows prop to get a \"correct\" first SSR paint\n      ,\n      rows: minRows,\n      style: style,\n      ...other\n    }), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"textarea\", {\n      \"aria-hidden\": true,\n      className: props.className,\n      readOnly: true,\n      ref: hiddenTextareaRef,\n      tabIndex: -1,\n      style: {\n        ...styles.shadow,\n        ...style,\n        paddingTop: 0,\n        paddingBottom: 0\n      }\n    })]\n  });\n});\n true ? TextareaAutosize.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string),\n  /**\n   * Maximum number of rows to display.\n   */\n  maxRows: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_7___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string)]),\n  /**\n   * Minimum number of rows to display.\n   * @default 1\n   */\n  minRows: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_7___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string)]),\n  /**\n   * @ignore\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n  /**\n   * @ignore\n   */\n  placeholder: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string),\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().object),\n  /**\n   * @ignore\n   */\n  value: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_7___default().arrayOf((prop_types__WEBPACK_IMPORTED_MODULE_7___default().string)), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_7___default().string)])\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (TextareaAutosize);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/TextareaAutosize/TextareaAutosize.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButton/ToggleButton.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButton/ToggleButton.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_16___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_16__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! @mui/utils/resolveProps */ \"./node_modules/@mui/utils/esm/resolveProps/resolveProps.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../ButtonBase/index.js */ \"./node_modules/@mui/material/ButtonBase/ButtonBase.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./toggleButtonClasses.js */ \"./node_modules/@mui/material/ToggleButton/toggleButtonClasses.js\");\n/* harmony import */ var _ToggleButtonGroup_ToggleButtonGroupContext_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../ToggleButtonGroup/ToggleButtonGroupContext.js */ \"./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupContext.js\");\n/* harmony import */ var _ToggleButtonGroup_ToggleButtonGroupButtonContext_js__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! ../ToggleButtonGroup/ToggleButtonGroupButtonContext.js */ \"./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupButtonContext.js\");\n/* harmony import */ var _ToggleButtonGroup_isValueSelected_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ../ToggleButtonGroup/isValueSelected.js */ \"./node_modules/@mui/material/ToggleButtonGroup/isValueSelected.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n// @inheritedComponent ButtonBase\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    fullWidth,\n    selected,\n    disabled,\n    size,\n    color\n  } = ownerState;\n  const slots = {\n    root: ['root', selected && 'selected', disabled && 'disabled', fullWidth && 'fullWidth', `size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(size)}`, color]\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__.getToggleButtonUtilityClass, classes);\n};\nconst ToggleButtonRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(_ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n  name: 'MuiToggleButton',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[`size${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.size)}`]];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  ...theme.typography.button,\n  borderRadius: (theme.vars || theme).shape.borderRadius,\n  padding: 11,\n  border: `1px solid ${(theme.vars || theme).palette.divider}`,\n  color: (theme.vars || theme).palette.action.active,\n  [`&.${_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled}`]: {\n    color: (theme.vars || theme).palette.action.disabled,\n    border: `1px solid ${(theme.vars || theme).palette.action.disabledBackground}`\n  },\n  '&:hover': {\n    textDecoration: 'none',\n    // Reset on mouse devices\n    backgroundColor: theme.vars ? `rgba(${theme.vars.palette.text.primaryChannel} / ${theme.vars.palette.action.hoverOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.text.primary, theme.palette.action.hoverOpacity),\n    '@media (hover: none)': {\n      backgroundColor: 'transparent'\n    }\n  },\n  variants: [{\n    props: {\n      color: 'standard'\n    },\n    style: {\n      [`&.${_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].selected}`]: {\n        color: (theme.vars || theme).palette.text.primary,\n        backgroundColor: theme.vars ? `rgba(${theme.vars.palette.text.primaryChannel} / ${theme.vars.palette.action.selectedOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.text.primary, theme.palette.action.selectedOpacity),\n        '&:hover': {\n          backgroundColor: theme.vars ? `rgba(${theme.vars.palette.text.primaryChannel} / calc(${theme.vars.palette.action.selectedOpacity} + ${theme.vars.palette.action.hoverOpacity}))` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.text.primary, theme.palette.action.selectedOpacity + theme.palette.action.hoverOpacity),\n          // Reset on touch devices, it doesn't add specificity\n          '@media (hover: none)': {\n            backgroundColor: theme.vars ? `rgba(${theme.vars.palette.text.primaryChannel} / ${theme.vars.palette.action.selectedOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette.text.primary, theme.palette.action.selectedOpacity)\n          }\n        }\n      }\n    }\n  }, ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])()).map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      [`&.${_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].selected}`]: {\n        color: (theme.vars || theme).palette[color].main,\n        backgroundColor: theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / ${theme.vars.palette.action.selectedOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette[color].main, theme.palette.action.selectedOpacity),\n        '&:hover': {\n          backgroundColor: theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / calc(${theme.vars.palette.action.selectedOpacity} + ${theme.vars.palette.action.hoverOpacity}))` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette[color].main, theme.palette.action.selectedOpacity + theme.palette.action.hoverOpacity),\n          // Reset on touch devices, it doesn't add specificity\n          '@media (hover: none)': {\n            backgroundColor: theme.vars ? `rgba(${theme.vars.palette[color].mainChannel} / ${theme.vars.palette.action.selectedOpacity})` : (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_9__.alpha)(theme.palette[color].main, theme.palette.action.selectedOpacity)\n          }\n        }\n      }\n    }\n  })), {\n    props: {\n      fullWidth: true\n    },\n    style: {\n      width: '100%'\n    }\n  }, {\n    props: {\n      size: 'small'\n    },\n    style: {\n      padding: 7,\n      fontSize: theme.typography.pxToRem(13)\n    }\n  }, {\n    props: {\n      size: 'large'\n    },\n    style: {\n      padding: 15,\n      fontSize: theme.typography.pxToRem(15)\n    }\n  }]\n})));\nconst ToggleButton = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function ToggleButton(inProps, ref) {\n  // props priority: `inProps` > `contextProps` > `themeDefaultProps`\n  const {\n    value: contextValue,\n    ...contextProps\n  } = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_ToggleButtonGroup_ToggleButtonGroupContext_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"]);\n  const toggleButtonGroupButtonContextPositionClassName = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_ToggleButtonGroup_ToggleButtonGroupButtonContext_js__WEBPACK_IMPORTED_MODULE_12__[\"default\"]);\n  const resolvedProps = (0,_mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_13__[\"default\"])({\n    ...contextProps,\n    selected: (0,_ToggleButtonGroup_isValueSelected_js__WEBPACK_IMPORTED_MODULE_14__[\"default\"])(inProps.value, contextValue)\n  }, inProps);\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_15__.useDefaultProps)({\n    props: resolvedProps,\n    name: 'MuiToggleButton'\n  });\n  const {\n    children,\n    className,\n    color = 'standard',\n    disabled = false,\n    disableFocusRipple = false,\n    fullWidth = false,\n    onChange,\n    onClick,\n    selected,\n    size = 'medium',\n    value,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    color,\n    disabled,\n    disableFocusRipple,\n    fullWidth,\n    size\n  };\n  const classes = useUtilityClasses(ownerState);\n  const handleChange = event => {\n    if (onClick) {\n      onClick(event, value);\n      if (event.defaultPrevented) {\n        return;\n      }\n    }\n    if (onChange) {\n      onChange(event, value);\n    }\n  };\n  const positionClassName = toggleButtonGroupButtonContextPositionClassName || '';\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ToggleButtonRoot, {\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(contextProps.className, classes.root, className, positionClassName),\n    disabled: disabled,\n    focusRipple: !disableFocusRipple,\n    ref: ref,\n    onClick: handleChange,\n    onChange: onChange,\n    value: value,\n    ownerState: ownerState,\n    \"aria-pressed\": selected,\n    ...other,\n    children: children\n  });\n});\n true ? ToggleButton.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string),\n  /**\n   * The color of the button when it is in an active state.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'standard'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['standard', 'primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * If `true`, the component is disabled.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the  keyboard focus ripple is disabled.\n   * @default false\n   */\n  disableFocusRipple: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the ripple effect is disabled.\n   *\n   * ⚠️ Without a ripple there is no styling for :focus-visible by default. Be sure\n   * to highlight the element by applying separate styles with the `.Mui-focusVisible` class.\n   * @default false\n   */\n  disableRipple: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * If `true`, the button will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * Callback fired when the state changes.\n   *\n   * @param {React.MouseEvent<HTMLElement>} event The event source of the callback.\n   * @param {any} value of the selected button.\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * Callback fired when the button is clicked.\n   *\n   * @param {React.MouseEvent<HTMLElement>} event The event source of the callback.\n   * @param {any} value of the selected button.\n   */\n  onClick: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func),\n  /**\n   * If `true`, the button is rendered in an active state.\n   */\n  selected: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool),\n  /**\n   * The size of the component.\n   * The prop defaults to the value inherited from the parent ToggleButtonGroup component.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOf(['small', 'medium', 'large']), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().string)]),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_16___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_16___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_16___default().object)]),\n  /**\n   * The value to associate with the button when selected in a\n   * ToggleButtonGroup.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_16___default().any).isRequired\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ToggleButton);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButton/ToggleButton.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButton/toggleButtonClasses.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButton/toggleButtonClasses.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getToggleButtonUtilityClass: () => (/* binding */ getToggleButtonUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getToggleButtonUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiToggleButton', slot);\n}\nconst toggleButtonClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiToggleButton', ['root', 'disabled', 'selected', 'standard', 'primary', 'secondary', 'sizeSmall', 'sizeMedium', 'sizeLarge', 'fullWidth']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (toggleButtonClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButton/toggleButtonClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroup.js":
/*!***************************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroup.js ***!
  \***************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_14___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_14__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_getValidReactChildren__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/utils/getValidReactChildren */ \"./node_modules/@mui/utils/esm/getValidReactChildren/getValidReactChildren.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./toggleButtonGroupClasses.js */ \"./node_modules/@mui/material/ToggleButtonGroup/toggleButtonGroupClasses.js\");\n/* harmony import */ var _ToggleButtonGroupContext_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./ToggleButtonGroupContext.js */ \"./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupContext.js\");\n/* harmony import */ var _ToggleButtonGroupButtonContext_js__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! ./ToggleButtonGroupButtonContext.js */ \"./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupButtonContext.js\");\n/* harmony import */ var _ToggleButton_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../ToggleButton/toggleButtonClasses.js */ \"./node_modules/@mui/material/ToggleButton/toggleButtonClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    orientation,\n    fullWidth,\n    disabled\n  } = ownerState;\n  const slots = {\n    root: ['root', orientation, fullWidth && 'fullWidth'],\n    grouped: ['grouped', `grouped${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(orientation)}`, disabled && 'disabled'],\n    firstButton: ['firstButton'],\n    lastButton: ['lastButton'],\n    middleButton: ['middleButton']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(slots, _toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__.getToggleButtonGroupUtilityClass, classes);\n};\nconst ToggleButtonGroupRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])('div', {\n  name: 'MuiToggleButtonGroup',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [{\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].grouped}`]: styles.grouped\n    }, {\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].grouped}`]: styles[`grouped${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(ownerState.orientation)}`]\n    }, {\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].firstButton}`]: styles.firstButton\n    }, {\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].lastButton}`]: styles.lastButton\n    }, {\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}`]: styles.middleButton\n    }, styles.root, ownerState.orientation === 'vertical' && styles.vertical, ownerState.fullWidth && styles.fullWidth];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(({\n  theme\n}) => ({\n  display: 'inline-flex',\n  borderRadius: (theme.vars || theme).shape.borderRadius,\n  variants: [{\n    props: {\n      orientation: 'vertical'\n    },\n    style: {\n      flexDirection: 'column',\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].grouped}`]: {\n        [`&.${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].selected} + .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].grouped}.${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].selected}`]: {\n          borderTop: 0,\n          marginTop: 0\n        }\n      },\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].firstButton},& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}`]: {\n        borderBottomLeftRadius: 0,\n        borderBottomRightRadius: 0\n      },\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].lastButton},& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}`]: {\n        marginTop: -1,\n        borderTop: '1px solid transparent',\n        borderTopLeftRadius: 0,\n        borderTopRightRadius: 0\n      },\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].lastButton}.${_ToggleButton_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"].disabled},& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}.${_ToggleButton_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"].disabled}`]: {\n        borderTop: '1px solid transparent'\n      }\n    }\n  }, {\n    props: {\n      fullWidth: true\n    },\n    style: {\n      width: '100%'\n    }\n  }, {\n    props: {\n      orientation: 'horizontal'\n    },\n    style: {\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].grouped}`]: {\n        [`&.${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].selected} + .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].grouped}.${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].selected}`]: {\n          borderLeft: 0,\n          marginLeft: 0\n        }\n      },\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].firstButton},& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}`]: {\n        borderTopRightRadius: 0,\n        borderBottomRightRadius: 0\n      },\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].lastButton},& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}`]: {\n        marginLeft: -1,\n        borderLeft: '1px solid transparent',\n        borderTopLeftRadius: 0,\n        borderBottomLeftRadius: 0\n      },\n      [`& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].lastButton}.${_ToggleButton_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"].disabled},& .${_toggleButtonGroupClasses_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"].middleButton}.${_ToggleButton_toggleButtonClasses_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"].disabled}`]: {\n        borderLeft: '1px solid transparent'\n      }\n    }\n  }]\n})));\nconst ToggleButtonGroup = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function ToggleButtonGroup(inProps, ref) {\n  const props = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_9__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiToggleButtonGroup'\n  });\n  const {\n    children,\n    className,\n    color = 'standard',\n    disabled = false,\n    exclusive = false,\n    fullWidth = false,\n    onChange,\n    orientation = 'horizontal',\n    size = 'medium',\n    value,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    disabled,\n    fullWidth,\n    orientation,\n    size\n  };\n  const classes = useUtilityClasses(ownerState);\n  const handleChange = react__WEBPACK_IMPORTED_MODULE_0__.useCallback((event, buttonValue) => {\n    if (!onChange) {\n      return;\n    }\n    const index = value && value.indexOf(buttonValue);\n    let newValue;\n    if (value && index >= 0) {\n      newValue = value.slice();\n      newValue.splice(index, 1);\n    } else {\n      newValue = value ? value.concat(buttonValue) : [buttonValue];\n    }\n    onChange(event, newValue);\n  }, [onChange, value]);\n  const handleExclusiveChange = react__WEBPACK_IMPORTED_MODULE_0__.useCallback((event, buttonValue) => {\n    if (!onChange) {\n      return;\n    }\n    onChange(event, value === buttonValue ? null : buttonValue);\n  }, [onChange, value]);\n  const context = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => ({\n    className: classes.grouped,\n    onChange: exclusive ? handleExclusiveChange : handleChange,\n    value,\n    size,\n    fullWidth,\n    color,\n    disabled\n  }), [classes.grouped, exclusive, handleExclusiveChange, handleChange, value, size, fullWidth, color, disabled]);\n  const validChildren = (0,_mui_utils_getValidReactChildren__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(children);\n  const childrenCount = validChildren.length;\n  const getButtonPositionClassName = index => {\n    const isFirstButton = index === 0;\n    const isLastButton = index === childrenCount - 1;\n    if (isFirstButton && isLastButton) {\n      return '';\n    }\n    if (isFirstButton) {\n      return classes.firstButton;\n    }\n    if (isLastButton) {\n      return classes.lastButton;\n    }\n    return classes.middleButton;\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ToggleButtonGroupRoot, {\n    role: \"group\",\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ref: ref,\n    ownerState: ownerState,\n    ...other,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_ToggleButtonGroupContext_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"].Provider, {\n      value: context,\n      children: validChildren.map((child, index) => {\n        if (true) {\n          if ((0,react_is__WEBPACK_IMPORTED_MODULE_12__.isFragment)(child)) {\n            console.error([\"MUI: The ToggleButtonGroup component doesn't accept a Fragment as a child.\", 'Consider providing an array instead.'].join('\\n'));\n          }\n        }\n        return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(_ToggleButtonGroupButtonContext_js__WEBPACK_IMPORTED_MODULE_13__[\"default\"].Provider, {\n          value: getButtonPositionClassName(index),\n          children: child\n        }, index);\n      })\n    })\n  });\n});\n true ? ToggleButtonGroup.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string),\n  /**\n   * The color of the button when it is selected.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   * @default 'standard'\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['standard', 'primary', 'secondary', 'error', 'info', 'success', 'warning']), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string)]),\n  /**\n   * If `true`, the component is disabled. This implies that all ToggleButton children will be disabled.\n   * @default false\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `true`, only allow one of the child ToggleButton values to be selected.\n   * @default false\n   */\n  exclusive: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * If `true`, the button group will take up the full width of its container.\n   * @default false\n   */\n  fullWidth: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool),\n  /**\n   * Callback fired when the value changes.\n   *\n   * @param {React.MouseEvent<HTMLElement>} event The event source of the callback.\n   * @param {any} value of the selected buttons. When `exclusive` is true\n   * this is a single value; when false an array of selected values. If no value\n   * is selected and `exclusive` is true the value is null; when false an empty array.\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().func),\n  /**\n   * The component orientation (layout flow direction).\n   * @default 'horizontal'\n   */\n  orientation: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['horizontal', 'vertical']),\n  /**\n   * The size of the component.\n   * @default 'medium'\n   */\n  size: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOf(['small', 'medium', 'large']), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().string)]),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_14___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_14___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_14___default().object)]),\n  /**\n   * The currently selected value within the group or an array of selected\n   * values when `exclusive` is false.\n   *\n   * The value must have reference equality with the option in order to be selected.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_14___default().any)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ToggleButtonGroup);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroup.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupButtonContext.js":
/*!****************************************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupButtonContext.js ***!
  \****************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n/**\n * @ignore - internal component.\n */\nconst ToggleButtonGroupButtonContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext(undefined);\nif (true) {\n  ToggleButtonGroupButtonContext.displayName = 'ToggleButtonGroupButtonContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ToggleButtonGroupButtonContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupButtonContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupContext.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupContext.js ***!
  \**********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n/**\n * @ignore - internal component.\n */\nconst ToggleButtonGroupContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext({});\nif (true) {\n  ToggleButtonGroupContext.displayName = 'ToggleButtonGroupContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ToggleButtonGroupContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroupContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButtonGroup/isValueSelected.js":
/*!*************************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButtonGroup/isValueSelected.js ***!
  \*************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ isValueSelected)\n/* harmony export */ });\n// Determine if the toggle button value matches, or is contained in, the\n// candidate group value.\nfunction isValueSelected(value, candidate) {\n  if (candidate === undefined || value === undefined) {\n    return false;\n  }\n  if (Array.isArray(candidate)) {\n    return candidate.includes(value);\n  }\n  return value === candidate;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButtonGroup/isValueSelected.js?");

/***/ }),

/***/ "./node_modules/@mui/material/ToggleButtonGroup/toggleButtonGroupClasses.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/@mui/material/ToggleButtonGroup/toggleButtonGroupClasses.js ***!
  \**********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getToggleButtonGroupUtilityClass: () => (/* binding */ getToggleButtonGroupUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getToggleButtonGroupUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiToggleButtonGroup', slot);\n}\nconst toggleButtonGroupClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiToggleButtonGroup', ['root', 'selected', 'horizontal', 'vertical', 'disabled', 'grouped', 'groupedHorizontal', 'groupedVertical', 'fullWidth', 'firstButton', 'lastButton', 'middleButton']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (toggleButtonGroupClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/ToggleButtonGroup/toggleButtonGroupClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Typography/Typography.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/Typography/Typography.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   TypographyRoot: () => (/* binding */ TypographyRoot),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/zero-styled/index.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/memoTheme.js */ \"./node_modules/@mui/material/utils/memoTheme.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../utils/createSimplePaletteValueFilter.js */ \"./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js\");\n/* harmony import */ var _typographyClasses_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./typographyClasses.js */ \"./node_modules/@mui/material/Typography/typographyClasses.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\nconst v6Colors = {\n  primary: true,\n  secondary: true,\n  error: true,\n  info: true,\n  success: true,\n  warning: true,\n  textPrimary: true,\n  textSecondary: true,\n  textDisabled: true\n};\nconst extendSxProp = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_3__.internal_createExtendSxProp)();\nconst useUtilityClasses = ownerState => {\n  const {\n    align,\n    gutterBottom,\n    noWrap,\n    paragraph,\n    variant,\n    classes\n  } = ownerState;\n  const slots = {\n    root: ['root', variant, ownerState.align !== 'inherit' && `align${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(align)}`, gutterBottom && 'gutterBottom', noWrap && 'noWrap', paragraph && 'paragraph']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(slots, _typographyClasses_js__WEBPACK_IMPORTED_MODULE_6__.getTypographyUtilityClass, classes);\n};\nconst TypographyRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])('span', {\n  name: 'MuiTypography',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, ownerState.variant && styles[ownerState.variant], ownerState.align !== 'inherit' && styles[`align${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(ownerState.align)}`], ownerState.noWrap && styles.noWrap, ownerState.gutterBottom && styles.gutterBottom, ownerState.paragraph && styles.paragraph];\n  }\n})((0,_utils_memoTheme_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(({\n  theme\n}) => ({\n  margin: 0,\n  variants: [{\n    props: {\n      variant: 'inherit'\n    },\n    style: {\n      // Some elements, like <button> on Chrome have default font that doesn't inherit, reset this.\n      font: 'inherit',\n      lineHeight: 'inherit',\n      letterSpacing: 'inherit'\n    }\n  }, ...Object.entries(theme.typography).filter(([variant, value]) => variant !== 'inherit' && value && typeof value === 'object').map(([variant, value]) => ({\n    props: {\n      variant\n    },\n    style: value\n  })), ...Object.entries(theme.palette).filter((0,_utils_createSimplePaletteValueFilter_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])()).map(([color]) => ({\n    props: {\n      color\n    },\n    style: {\n      color: (theme.vars || theme).palette[color].main\n    }\n  })), ...Object.entries(theme.palette?.text || {}).filter(([, value]) => typeof value === 'string').map(([color]) => ({\n    props: {\n      color: `text${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(color)}`\n    },\n    style: {\n      color: (theme.vars || theme).palette.text[color]\n    }\n  })), {\n    props: ({\n      ownerState\n    }) => ownerState.align !== 'inherit',\n    style: {\n      textAlign: 'var(--Typography-textAlign)'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.noWrap,\n    style: {\n      overflow: 'hidden',\n      textOverflow: 'ellipsis',\n      whiteSpace: 'nowrap'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.gutterBottom,\n    style: {\n      marginBottom: '0.35em'\n    }\n  }, {\n    props: ({\n      ownerState\n    }) => ownerState.paragraph,\n    style: {\n      marginBottom: 16\n    }\n  }]\n})));\nconst defaultVariantMapping = {\n  h1: 'h1',\n  h2: 'h2',\n  h3: 'h3',\n  h4: 'h4',\n  h5: 'h5',\n  h6: 'h6',\n  subtitle1: 'h6',\n  subtitle2: 'h6',\n  body1: 'p',\n  body2: 'p',\n  inherit: 'p'\n};\nconst Typography = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Typography(inProps, ref) {\n  const {\n    color,\n    ...themeProps\n  } = (0,_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_10__.useDefaultProps)({\n    props: inProps,\n    name: 'MuiTypography'\n  });\n  const isSxColor = !v6Colors[color];\n  // TODO: Remove `extendSxProp` in v7\n  const props = extendSxProp({\n    ...themeProps,\n    ...(isSxColor && {\n      color\n    })\n  });\n  const {\n    align = 'inherit',\n    className,\n    component,\n    gutterBottom = false,\n    noWrap = false,\n    paragraph = false,\n    variant = 'body1',\n    variantMapping = defaultVariantMapping,\n    ...other\n  } = props;\n  const ownerState = {\n    ...props,\n    align,\n    color,\n    className,\n    component,\n    gutterBottom,\n    noWrap,\n    paragraph,\n    variant,\n    variantMapping\n  };\n  const Component = component || (paragraph ? 'p' : variantMapping[variant] || defaultVariantMapping[variant]) || 'span';\n  const classes = useUtilityClasses(ownerState);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(TypographyRoot, {\n    as: Component,\n    ref: ref,\n    className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n    ...other,\n    ownerState: ownerState,\n    style: {\n      ...(align !== 'inherit' && {\n        '--Typography-textAlign': align\n      }),\n      ...other.style\n    }\n  });\n});\n true ? Typography.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Set the text-align on the component.\n   * @default 'inherit'\n   */\n  align: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['center', 'inherit', 'justify', 'left', 'right']),\n  /**\n   * The content of the component.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node),\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * The color of the component.\n   * It supports both default and custom theme colors, which can be added as shown in the\n   * [palette customization guide](https://mui.com/material-ui/customization/palette/#custom-colors).\n   */\n  color: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['primary', 'secondary', 'success', 'error', 'info', 'warning', 'textPrimary', 'textSecondary', 'textDisabled']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The component used for the root node.\n   * Either a string to use a HTML element or a component.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n  /**\n   * If `true`, the text will have a bottom margin.\n   * @default false\n   */\n  gutterBottom: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the text will not wrap, but instead will truncate with a text overflow ellipsis.\n   *\n   * Note that text overflow can only happen with block or inline-block level elements\n   * (the element needs to have a width in order to overflow).\n   * @default false\n   */\n  noWrap: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the element will be a paragraph element.\n   * @default false\n   * @deprecated Use the `component` prop instead. This prop will be removed in v7. See [Migrating from deprecated APIs](https://mui.com/material-ui/migration/migrating-from-deprecated-apis/) for more details.\n   */\n  paragraph: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * @ignore\n   */\n  style: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n  /**\n   * Applies the theme typography styles.\n   * @default 'body1'\n   */\n  variant: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['body1', 'body2', 'button', 'caption', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'inherit', 'overline', 'subtitle1', 'subtitle2']), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The component maps the variant prop to a range of different HTML element types.\n   * For instance, subtitle1 to `<h6>`.\n   * If you wish to change that mapping, you can provide your own.\n   * Alternatively, you can use the `component` prop.\n   * @default {\n   *   h1: 'h1',\n   *   h2: 'h2',\n   *   h3: 'h3',\n   *   h4: 'h4',\n   *   h5: 'h5',\n   *   h6: 'h6',\n   *   subtitle1: 'h6',\n   *   subtitle2: 'h6',\n   *   body1: 'p',\n   *   body2: 'p',\n   *   inherit: 'p',\n   * }\n   */\n  variantMapping: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Typography);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Typography/Typography.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Typography/typographyClasses.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/Typography/typographyClasses.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getTypographyUtilityClass: () => (/* binding */ getTypographyUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getTypographyUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('MuiTypography', slot);\n}\nconst typographyClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('MuiTypography', ['root', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'subtitle1', 'subtitle2', 'body1', 'body2', 'inherit', 'button', 'caption', 'overline', 'alignLeft', 'alignRight', 'alignCenter', 'alignJustify', 'noWrap', 'gutterBottom', 'paragraph']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (typographyClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Typography/typographyClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/Unstable_TrapFocus/FocusTrap.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/Unstable_TrapFocus/FocusTrap.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_6___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_6__);\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js\");\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/exactProp/exactProp.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n/* eslint-disable consistent-return, jsx-a11y/no-noninteractive-tabindex */\n\n\n\n\n// Inspired by https://github.com/focus-trap/tabbable\nconst candidatesSelector = ['input', 'select', 'textarea', 'a[href]', 'button', '[tabindex]', 'audio[controls]', 'video[controls]', '[contenteditable]:not([contenteditable=\"false\"])'].join(',');\nfunction getTabIndex(node) {\n  const tabindexAttr = parseInt(node.getAttribute('tabindex') || '', 10);\n  if (!Number.isNaN(tabindexAttr)) {\n    return tabindexAttr;\n  }\n\n  // Browsers do not return `tabIndex` correctly for contentEditable nodes;\n  // https://issues.chromium.org/issues/41283952\n  // so if they don't have a tabindex attribute specifically set, assume it's 0.\n  // in Chrome, <details/>, <audio controls/> and <video controls/> elements get a default\n  //  `tabIndex` of -1 when the 'tabindex' attribute isn't specified in the DOM,\n  //  yet they are still part of the regular tab order; in FF, they get a default\n  //  `tabIndex` of 0; since Chrome still puts those elements in the regular tab\n  //  order, consider their tab index to be 0.\n  if (node.contentEditable === 'true' || (node.nodeName === 'AUDIO' || node.nodeName === 'VIDEO' || node.nodeName === 'DETAILS') && node.getAttribute('tabindex') === null) {\n    return 0;\n  }\n  return node.tabIndex;\n}\nfunction isNonTabbableRadio(node) {\n  if (node.tagName !== 'INPUT' || node.type !== 'radio') {\n    return false;\n  }\n  if (!node.name) {\n    return false;\n  }\n  const getRadio = selector => node.ownerDocument.querySelector(`input[type=\"radio\"]${selector}`);\n  let roving = getRadio(`[name=\"${node.name}\"]:checked`);\n  if (!roving) {\n    roving = getRadio(`[name=\"${node.name}\"]`);\n  }\n  return roving !== node;\n}\nfunction isNodeMatchingSelectorFocusable(node) {\n  if (node.disabled || node.tagName === 'INPUT' && node.type === 'hidden' || isNonTabbableRadio(node)) {\n    return false;\n  }\n  return true;\n}\nfunction defaultGetTabbable(root) {\n  const regularTabNodes = [];\n  const orderedTabNodes = [];\n  Array.from(root.querySelectorAll(candidatesSelector)).forEach((node, i) => {\n    const nodeTabIndex = getTabIndex(node);\n    if (nodeTabIndex === -1 || !isNodeMatchingSelectorFocusable(node)) {\n      return;\n    }\n    if (nodeTabIndex === 0) {\n      regularTabNodes.push(node);\n    } else {\n      orderedTabNodes.push({\n        documentOrder: i,\n        tabIndex: nodeTabIndex,\n        node: node\n      });\n    }\n  });\n  return orderedTabNodes.sort((a, b) => a.tabIndex === b.tabIndex ? a.documentOrder - b.documentOrder : a.tabIndex - b.tabIndex).map(a => a.node).concat(regularTabNodes);\n}\nfunction defaultIsEnabled() {\n  return true;\n}\n\n/**\n * @ignore - internal component.\n */\nfunction FocusTrap(props) {\n  const {\n    children,\n    disableAutoFocus = false,\n    disableEnforceFocus = false,\n    disableRestoreFocus = false,\n    getTabbable = defaultGetTabbable,\n    isEnabled = defaultIsEnabled,\n    open\n  } = props;\n  const ignoreNextEnforceFocus = react__WEBPACK_IMPORTED_MODULE_0__.useRef(false);\n  const sentinelStart = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const sentinelEnd = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const nodeToRestore = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const reactFocusEventTarget = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  // This variable is useful when disableAutoFocus is true.\n  // It waits for the active element to move into the component to activate.\n  const activated = react__WEBPACK_IMPORTED_MODULE_0__.useRef(false);\n  const rootRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  const handleRef = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_2__[\"default\"])((0,_mui_utils__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(children), rootRef);\n  const lastKeydown = react__WEBPACK_IMPORTED_MODULE_0__.useRef(null);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    // We might render an empty child.\n    if (!open || !rootRef.current) {\n      return;\n    }\n    activated.current = !disableAutoFocus;\n  }, [disableAutoFocus, open]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    // We might render an empty child.\n    if (!open || !rootRef.current) {\n      return;\n    }\n    const doc = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(rootRef.current);\n    if (!rootRef.current.contains(doc.activeElement)) {\n      if (!rootRef.current.hasAttribute('tabIndex')) {\n        if (true) {\n          console.error(['MUI: The modal content node does not accept focus.', 'For the benefit of assistive technologies, ' + 'the tabIndex of the node is being set to \"-1\".'].join('\\n'));\n        }\n        rootRef.current.setAttribute('tabIndex', '-1');\n      }\n      if (activated.current) {\n        rootRef.current.focus();\n      }\n    }\n    return () => {\n      // restoreLastFocus()\n      if (!disableRestoreFocus) {\n        // In IE11 it is possible for document.activeElement to be null resulting\n        // in nodeToRestore.current being null.\n        // Not all elements in IE11 have a focus method.\n        // Once IE11 support is dropped the focus() call can be unconditional.\n        if (nodeToRestore.current && nodeToRestore.current.focus) {\n          ignoreNextEnforceFocus.current = true;\n          nodeToRestore.current.focus();\n        }\n        nodeToRestore.current = null;\n      }\n    };\n    // Missing `disableRestoreFocus` which is fine.\n    // We don't support changing that prop on an open FocusTrap\n    // eslint-disable-next-line react-hooks/exhaustive-deps\n  }, [open]);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    // We might render an empty child.\n    if (!open || !rootRef.current) {\n      return;\n    }\n    const doc = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(rootRef.current);\n    const loopFocus = nativeEvent => {\n      lastKeydown.current = nativeEvent;\n      if (disableEnforceFocus || !isEnabled() || nativeEvent.key !== 'Tab') {\n        return;\n      }\n\n      // Make sure the next tab starts from the right place.\n      // doc.activeElement refers to the origin.\n      if (doc.activeElement === rootRef.current && nativeEvent.shiftKey) {\n        // We need to ignore the next contain as\n        // it will try to move the focus back to the rootRef element.\n        ignoreNextEnforceFocus.current = true;\n        if (sentinelEnd.current) {\n          sentinelEnd.current.focus();\n        }\n      }\n    };\n    const contain = () => {\n      const rootElement = rootRef.current;\n\n      // Cleanup functions are executed lazily in React 17.\n      // Contain can be called between the component being unmounted and its cleanup function being run.\n      if (rootElement === null) {\n        return;\n      }\n      if (!doc.hasFocus() || !isEnabled() || ignoreNextEnforceFocus.current) {\n        ignoreNextEnforceFocus.current = false;\n        return;\n      }\n\n      // The focus is already inside\n      if (rootElement.contains(doc.activeElement)) {\n        return;\n      }\n\n      // The disableEnforceFocus is set and the focus is outside of the focus trap (and sentinel nodes)\n      if (disableEnforceFocus && doc.activeElement !== sentinelStart.current && doc.activeElement !== sentinelEnd.current) {\n        return;\n      }\n\n      // if the focus event is not coming from inside the children's react tree, reset the refs\n      if (doc.activeElement !== reactFocusEventTarget.current) {\n        reactFocusEventTarget.current = null;\n      } else if (reactFocusEventTarget.current !== null) {\n        return;\n      }\n      if (!activated.current) {\n        return;\n      }\n      let tabbable = [];\n      if (doc.activeElement === sentinelStart.current || doc.activeElement === sentinelEnd.current) {\n        tabbable = getTabbable(rootRef.current);\n      }\n\n      // one of the sentinel nodes was focused, so move the focus\n      // to the first/last tabbable element inside the focus trap\n      if (tabbable.length > 0) {\n        const isShiftTab = Boolean(lastKeydown.current?.shiftKey && lastKeydown.current?.key === 'Tab');\n        const focusNext = tabbable[0];\n        const focusPrevious = tabbable[tabbable.length - 1];\n        if (typeof focusNext !== 'string' && typeof focusPrevious !== 'string') {\n          if (isShiftTab) {\n            focusPrevious.focus();\n          } else {\n            focusNext.focus();\n          }\n        }\n        // no tabbable elements in the trap focus or the focus was outside of the focus trap\n      } else {\n        rootElement.focus();\n      }\n    };\n    doc.addEventListener('focusin', contain);\n    doc.addEventListener('keydown', loopFocus, true);\n\n    // With Edge, Safari and Firefox, no focus related events are fired when the focused area stops being a focused area.\n    // for example https://bugzilla.mozilla.org/show_bug.cgi?id=559561.\n    // Instead, we can look if the active element was restored on the BODY element.\n    //\n    // The whatwg spec defines how the browser should behave but does not explicitly mention any events:\n    // https://html.spec.whatwg.org/multipage/interaction.html#focus-fixup-rule.\n    const interval = setInterval(() => {\n      if (doc.activeElement && doc.activeElement.tagName === 'BODY') {\n        contain();\n      }\n    }, 50);\n    return () => {\n      clearInterval(interval);\n      doc.removeEventListener('focusin', contain);\n      doc.removeEventListener('keydown', loopFocus, true);\n    };\n  }, [disableAutoFocus, disableEnforceFocus, disableRestoreFocus, isEnabled, open, getTabbable]);\n  const onFocus = event => {\n    if (nodeToRestore.current === null) {\n      nodeToRestore.current = event.relatedTarget;\n    }\n    activated.current = true;\n    reactFocusEventTarget.current = event.target;\n    const childrenPropsHandler = children.props.onFocus;\n    if (childrenPropsHandler) {\n      childrenPropsHandler(event);\n    }\n  };\n  const handleFocusSentinel = event => {\n    if (nodeToRestore.current === null) {\n      nodeToRestore.current = event.relatedTarget;\n    }\n    activated.current = true;\n  };\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n    children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"div\", {\n      tabIndex: open ? 0 : -1,\n      onFocus: handleFocusSentinel,\n      ref: sentinelStart,\n      \"data-testid\": \"sentinelStart\"\n    }), /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(children, {\n      ref: handleRef,\n      onFocus\n    }), /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"div\", {\n      tabIndex: open ? 0 : -1,\n      onFocus: handleFocusSentinel,\n      ref: sentinelEnd,\n      \"data-testid\": \"sentinelEnd\"\n    })]\n  });\n}\n true ? FocusTrap.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * A single child content element.\n   */\n  children: _mui_utils__WEBPACK_IMPORTED_MODULE_5__[\"default\"],\n  /**\n   * If `true`, the focus trap will not automatically shift focus to itself when it opens, and\n   * replace it to the last focused element when it closes.\n   * This also works correctly with any focus trap children that have the `disableAutoFocus` prop.\n   *\n   * Generally this should never be set to `true` as it makes the focus trap less\n   * accessible to assistive technologies, like screen readers.\n   * @default false\n   */\n  disableAutoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().bool),\n  /**\n   * If `true`, the focus trap will not prevent focus from leaving the focus trap while open.\n   *\n   * Generally this should never be set to `true` as it makes the focus trap less\n   * accessible to assistive technologies, like screen readers.\n   * @default false\n   */\n  disableEnforceFocus: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().bool),\n  /**\n   * If `true`, the focus trap will not restore focus to previously focused element once\n   * focus trap is hidden or unmounted.\n   * @default false\n   */\n  disableRestoreFocus: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().bool),\n  /**\n   * Returns an array of ordered tabbable nodes (i.e. in tab order) within the root.\n   * For instance, you can provide the \"tabbable\" npm dependency.\n   * @param {HTMLElement} root\n   */\n  getTabbable: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().func),\n  /**\n   * This prop extends the `open` prop.\n   * It allows to toggle the open state without having to wait for a rerender when changing the `open` prop.\n   * This prop should be memoized.\n   * It can be used to support multiple focus trap mounted at the same time.\n   * @default function defaultIsEnabled(): boolean {\n   *   return true;\n   * }\n   */\n  isEnabled: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().func),\n  /**\n   * If `true`, focus is locked.\n   */\n  open: (prop_types__WEBPACK_IMPORTED_MODULE_6___default().bool).isRequired\n} : 0;\nif (true) {\n  // eslint-disable-next-line\n  FocusTrap['propTypes' + ''] = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(FocusTrap.propTypes);\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (FocusTrap);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/Unstable_TrapFocus/FocusTrap.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/blue.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/colors/blue.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst blue = {\n  50: '#e3f2fd',\n  100: '#bbdefb',\n  200: '#90caf9',\n  300: '#64b5f6',\n  400: '#42a5f5',\n  500: '#2196f3',\n  600: '#1e88e5',\n  700: '#1976d2',\n  800: '#1565c0',\n  900: '#0d47a1',\n  A100: '#82b1ff',\n  A200: '#448aff',\n  A400: '#2979ff',\n  A700: '#2962ff'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (blue);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/blue.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/common.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/colors/common.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst common = {\n  black: '#000',\n  white: '#fff'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (common);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/common.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/green.js":
/*!****************************************************!*\
  !*** ./node_modules/@mui/material/colors/green.js ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst green = {\n  50: '#e8f5e9',\n  100: '#c8e6c9',\n  200: '#a5d6a7',\n  300: '#81c784',\n  400: '#66bb6a',\n  500: '#4caf50',\n  600: '#43a047',\n  700: '#388e3c',\n  800: '#2e7d32',\n  900: '#1b5e20',\n  A100: '#b9f6ca',\n  A200: '#69f0ae',\n  A400: '#00e676',\n  A700: '#00c853'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (green);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/green.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/grey.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/colors/grey.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst grey = {\n  50: '#fafafa',\n  100: '#f5f5f5',\n  200: '#eeeeee',\n  300: '#e0e0e0',\n  400: '#bdbdbd',\n  500: '#9e9e9e',\n  600: '#757575',\n  700: '#616161',\n  800: '#424242',\n  900: '#212121',\n  A100: '#f5f5f5',\n  A200: '#eeeeee',\n  A400: '#bdbdbd',\n  A700: '#616161'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (grey);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/grey.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/lightBlue.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/colors/lightBlue.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst lightBlue = {\n  50: '#e1f5fe',\n  100: '#b3e5fc',\n  200: '#81d4fa',\n  300: '#4fc3f7',\n  400: '#29b6f6',\n  500: '#03a9f4',\n  600: '#039be5',\n  700: '#0288d1',\n  800: '#0277bd',\n  900: '#01579b',\n  A100: '#80d8ff',\n  A200: '#40c4ff',\n  A400: '#00b0ff',\n  A700: '#0091ea'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (lightBlue);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/lightBlue.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/orange.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/colors/orange.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst orange = {\n  50: '#fff3e0',\n  100: '#ffe0b2',\n  200: '#ffcc80',\n  300: '#ffb74d',\n  400: '#ffa726',\n  500: '#ff9800',\n  600: '#fb8c00',\n  700: '#f57c00',\n  800: '#ef6c00',\n  900: '#e65100',\n  A100: '#ffd180',\n  A200: '#ffab40',\n  A400: '#ff9100',\n  A700: '#ff6d00'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (orange);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/orange.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/purple.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/colors/purple.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst purple = {\n  50: '#f3e5f5',\n  100: '#e1bee7',\n  200: '#ce93d8',\n  300: '#ba68c8',\n  400: '#ab47bc',\n  500: '#9c27b0',\n  600: '#8e24aa',\n  700: '#7b1fa2',\n  800: '#6a1b9a',\n  900: '#4a148c',\n  A100: '#ea80fc',\n  A200: '#e040fb',\n  A400: '#d500f9',\n  A700: '#aa00ff'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (purple);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/purple.js?");

/***/ }),

/***/ "./node_modules/@mui/material/colors/red.js":
/*!**************************************************!*\
  !*** ./node_modules/@mui/material/colors/red.js ***!
  \**************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst red = {\n  50: '#ffebee',\n  100: '#ffcdd2',\n  200: '#ef9a9a',\n  300: '#e57373',\n  400: '#ef5350',\n  500: '#f44336',\n  600: '#e53935',\n  700: '#d32f2f',\n  800: '#c62828',\n  900: '#b71c1c',\n  A100: '#ff8a80',\n  A200: '#ff5252',\n  A400: '#ff1744',\n  A700: '#d50000'\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (red);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/colors/red.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/SwitchBase.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/internal/SwitchBase.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_11___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_11__);\n/* harmony import */ var _mui_utils_refType__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/utils/refType */ \"./node_modules/@mui/utils/esm/refType/refType.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _utils_capitalize_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../utils/capitalize.js */ \"./node_modules/@mui/material/utils/capitalize.js\");\n/* harmony import */ var _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styles/rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../zero-styled/index.js */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _utils_useControlled_js__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ../utils/useControlled.js */ \"./node_modules/@mui/material/utils/useControlled.js\");\n/* harmony import */ var _FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../FormControl/useFormControl.js */ \"./node_modules/@mui/material/FormControl/useFormControl.js\");\n/* harmony import */ var _ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../ButtonBase/index.js */ \"./node_modules/@mui/material/ButtonBase/ButtonBase.js\");\n/* harmony import */ var _switchBaseClasses_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./switchBaseClasses.js */ \"./node_modules/@mui/material/internal/switchBaseClasses.js\");\n/* harmony import */ var _utils_useSlot_js__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ../utils/useSlot.js */ \"./node_modules/@mui/material/utils/useSlot.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst useUtilityClasses = ownerState => {\n  const {\n    classes,\n    checked,\n    disabled,\n    edge\n  } = ownerState;\n  const slots = {\n    root: ['root', checked && 'checked', disabled && 'disabled', edge && `edge${(0,_utils_capitalize_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(edge)}`],\n    input: ['input']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(slots, _switchBaseClasses_js__WEBPACK_IMPORTED_MODULE_4__.getSwitchBaseUtilityClass, classes);\n};\nconst SwitchBaseRoot = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(_ButtonBase_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])({\n  padding: 9,\n  borderRadius: '50%',\n  variants: [{\n    props: {\n      edge: 'start',\n      size: 'small'\n    },\n    style: {\n      marginLeft: -3\n    }\n  }, {\n    props: ({\n      edge,\n      ownerState\n    }) => edge === 'start' && ownerState.size !== 'small',\n    style: {\n      marginLeft: -12\n    }\n  }, {\n    props: {\n      edge: 'end',\n      size: 'small'\n    },\n    style: {\n      marginRight: -3\n    }\n  }, {\n    props: ({\n      edge,\n      ownerState\n    }) => edge === 'end' && ownerState.size !== 'small',\n    style: {\n      marginRight: -12\n    }\n  }]\n});\nconst SwitchBaseInput = (0,_zero_styled_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])('input', {\n  shouldForwardProp: _styles_rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"]\n})({\n  cursor: 'inherit',\n  position: 'absolute',\n  opacity: 0,\n  width: '100%',\n  height: '100%',\n  top: 0,\n  left: 0,\n  margin: 0,\n  padding: 0,\n  zIndex: 1\n});\n\n/**\n * @ignore - internal component.\n */\nconst SwitchBase = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function SwitchBase(props, ref) {\n  const {\n    autoFocus,\n    checked: checkedProp,\n    checkedIcon,\n    defaultChecked,\n    disabled: disabledProp,\n    disableFocusRipple = false,\n    edge = false,\n    icon,\n    id,\n    inputProps,\n    inputRef,\n    name,\n    onBlur,\n    onChange,\n    onFocus,\n    readOnly,\n    required = false,\n    tabIndex,\n    type,\n    value,\n    slots = {},\n    slotProps = {},\n    ...other\n  } = props;\n  const [checked, setCheckedState] = (0,_utils_useControlled_js__WEBPACK_IMPORTED_MODULE_8__[\"default\"])({\n    controlled: checkedProp,\n    default: Boolean(defaultChecked),\n    name: 'SwitchBase',\n    state: 'checked'\n  });\n  const muiFormControl = (0,_FormControl_useFormControl_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"])();\n  const handleFocus = event => {\n    if (onFocus) {\n      onFocus(event);\n    }\n    if (muiFormControl && muiFormControl.onFocus) {\n      muiFormControl.onFocus(event);\n    }\n  };\n  const handleBlur = event => {\n    if (onBlur) {\n      onBlur(event);\n    }\n    if (muiFormControl && muiFormControl.onBlur) {\n      muiFormControl.onBlur(event);\n    }\n  };\n  const handleInputChange = event => {\n    // Workaround for https://github.com/facebook/react/issues/9023\n    if (event.nativeEvent.defaultPrevented) {\n      return;\n    }\n    const newChecked = event.target.checked;\n    setCheckedState(newChecked);\n    if (onChange) {\n      // TODO v6: remove the second argument.\n      onChange(event, newChecked);\n    }\n  };\n  let disabled = disabledProp;\n  if (muiFormControl) {\n    if (typeof disabled === 'undefined') {\n      disabled = muiFormControl.disabled;\n    }\n  }\n  const hasLabelFor = type === 'checkbox' || type === 'radio';\n  const ownerState = {\n    ...props,\n    checked,\n    disabled,\n    disableFocusRipple,\n    edge\n  };\n  const classes = useUtilityClasses(ownerState);\n  const externalForwardedProps = {\n    slots,\n    slotProps: {\n      input: inputProps,\n      ...slotProps\n    }\n  };\n  const [RootSlot, rootSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])('root', {\n    ref,\n    elementType: SwitchBaseRoot,\n    className: classes.root,\n    shouldForwardComponentProp: true,\n    externalForwardedProps: {\n      ...externalForwardedProps,\n      component: 'span',\n      ...other\n    },\n    getSlotProps: handlers => ({\n      ...handlers,\n      onFocus: event => {\n        handlers.onFocus?.(event);\n        handleFocus(event);\n      },\n      onBlur: event => {\n        handlers.onBlur?.(event);\n        handleBlur(event);\n      }\n    }),\n    ownerState,\n    additionalProps: {\n      centerRipple: true,\n      focusRipple: !disableFocusRipple,\n      disabled,\n      role: undefined,\n      tabIndex: null\n    }\n  });\n  const [InputSlot, inputSlotProps] = (0,_utils_useSlot_js__WEBPACK_IMPORTED_MODULE_10__[\"default\"])('input', {\n    ref: inputRef,\n    elementType: SwitchBaseInput,\n    className: classes.input,\n    externalForwardedProps,\n    getSlotProps: handlers => ({\n      onChange: event => {\n        handlers.onChange?.(event);\n        handleInputChange(event);\n      }\n    }),\n    ownerState,\n    additionalProps: {\n      autoFocus,\n      checked: checkedProp,\n      defaultChecked,\n      disabled,\n      id: hasLabelFor ? id : undefined,\n      name,\n      readOnly,\n      required,\n      tabIndex,\n      type,\n      ...(type === 'checkbox' && value === undefined ? {} : {\n        value\n      })\n    }\n  });\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsxs)(RootSlot, {\n    ...rootSlotProps,\n    children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(InputSlot, {\n      ...inputSlotProps\n    }), checked ? checkedIcon : icon]\n  });\n});\n\n// NB: If changed, please update Checkbox, Switch and Radio\n// so that the API documentation is updated.\n true ? SwitchBase.propTypes = {\n  /**\n   * If `true`, the `input` element is focused during the first mount.\n   */\n  autoFocus: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the component is checked.\n   */\n  checked: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * The icon to display when the component is checked.\n   */\n  checkedIcon: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node).isRequired,\n  /**\n   * Override or extend the styles applied to the component.\n   */\n  classes: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  className: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * @ignore\n   */\n  defaultChecked: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the component is disabled.\n   */\n  disabled: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the  keyboard focus ripple is disabled.\n   * @default false\n   */\n  disableFocusRipple: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If given, uses a negative margin to counteract the padding on one\n   * side (this is often helpful for aligning the left or right\n   * side of the icon with content above or below, without ruining the border\n   * size and shape).\n   * @default false\n   */\n  edge: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOf(['end', 'start', false]),\n  /**\n   * The icon to display when the component is unchecked.\n   */\n  icon: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().node).isRequired,\n  /**\n   * The id of the `input` element.\n   */\n  id: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * [Attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#Attributes) applied to the `input` element.\n   */\n  inputProps: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * Pass a ref to the `input` element.\n   */\n  inputRef: _mui_utils_refType__WEBPACK_IMPORTED_MODULE_12__[\"default\"],\n  /*\n   * @ignore\n   */\n  name: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string),\n  /**\n   * @ignore\n   */\n  onBlur: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func),\n  /**\n   * Callback fired when the state is changed.\n   *\n   * @param {object} event The event source of the callback.\n   * You can pull out the new checked state by accessing `event.target.checked` (boolean).\n   */\n  onChange: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func),\n  /**\n   * @ignore\n   */\n  onFocus: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().func),\n  /**\n   * It prevents the user from changing the value of the field\n   * (not from interacting with the field).\n   */\n  readOnly: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * If `true`, the `input` element is required.\n   */\n  required: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().bool),\n  /**\n   * The props used for each slot inside.\n   * @default {}\n   */\n  slotProps: prop_types__WEBPACK_IMPORTED_MODULE_11___default().shape({\n    input: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)]),\n    root: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object)])\n  }),\n  /**\n   * The components used for each slot inside.\n   * @default {}\n   */\n  slots: prop_types__WEBPACK_IMPORTED_MODULE_11___default().shape({\n    input: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType),\n    root: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().elementType)\n  }),\n  /**\n   * The system prop that allows defining system overrides as well as additional CSS styles.\n   */\n  sx: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().object),\n  /**\n   * @ignore\n   */\n  tabIndex: prop_types__WEBPACK_IMPORTED_MODULE_11___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_11___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string)]),\n  /**\n   * The input component prop `type`.\n   */\n  type: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().string).isRequired,\n  /**\n   * The value of the component.\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_11___default().any)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (SwitchBase);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/SwitchBase.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/svg-icons/ArrowDropDown.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/material/internal/svg-icons/ArrowDropDown.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * @ignore - internal component.\n */\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"path\", {\n  d: \"M7 10l5 5 5-5z\"\n}), 'ArrowDropDown'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/svg-icons/ArrowDropDown.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/svg-icons/Close.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/internal/svg-icons/Close.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * @ignore - internal component.\n *\n * Alias to `Clear`.\n */\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"path\", {\n  d: \"M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z\"\n}), 'Close'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/svg-icons/Close.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/svg-icons/ErrorOutline.js":
/*!***********************************************************************!*\
  !*** ./node_modules/@mui/material/internal/svg-icons/ErrorOutline.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * @ignore - internal component.\n */\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"path\", {\n  d: \"M11 15h2v2h-2zm0-8h2v6h-2zm.99-5C6.47 2 2 6.48 2 12s4.47 10 9.99 10C17.52 22 22 17.52 22 12S17.52 2 11.99 2zM12 20c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8z\"\n}), 'ErrorOutline'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/svg-icons/ErrorOutline.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/svg-icons/InfoOutlined.js":
/*!***********************************************************************!*\
  !*** ./node_modules/@mui/material/internal/svg-icons/InfoOutlined.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * @ignore - internal component.\n */\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"path\", {\n  d: \"M11,9H13V7H11M12,20C7.59,20 4,16.41 4,12C4,7.59 7.59,4 12,4C16.41,4 20,7.59 20, 12C20,16.41 16.41,20 12,20M12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10, 10 0 0,0 12,2M11,17H13V11H11V17Z\"\n}), 'InfoOutlined'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/svg-icons/InfoOutlined.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/svg-icons/ReportProblemOutlined.js":
/*!********************************************************************************!*\
  !*** ./node_modules/@mui/material/internal/svg-icons/ReportProblemOutlined.js ***!
  \********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * @ignore - internal component.\n */\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"path\", {\n  d: \"M12 5.99L19.53 19H4.47L12 5.99M12 2L1 21h22L12 2zm1 14h-2v2h2v-2zm0-6h-2v4h2v-4z\"\n}), 'ReportProblemOutlined'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/svg-icons/ReportProblemOutlined.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/svg-icons/SuccessOutlined.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/material/internal/svg-icons/SuccessOutlined.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../../utils/createSvgIcon.js */ \"./node_modules/@mui/material/utils/createSvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * @ignore - internal component.\n */\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_utils_createSvgIcon_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"path\", {\n  d: \"M20,12A8,8 0 0,1 12,20A8,8 0 0,1 4,12A8,8 0 0,1 12,4C12.76,4 13.5,4.11 14.2, 4.31L15.77,2.74C14.61,2.26 13.34,2 12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0, 0 22,12M7.91,10.08L6.5,11.5L11,16L21,6L19.59,4.58L11,13.17L7.91,10.08Z\"\n}), 'SuccessOutlined'));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/svg-icons/SuccessOutlined.js?");

/***/ }),

/***/ "./node_modules/@mui/material/internal/switchBaseClasses.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/internal/switchBaseClasses.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getSwitchBaseUtilityClass: () => (/* binding */ getSwitchBaseUtilityClass)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/generateUtilityClasses */ \"./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\n\nfunction getSwitchBaseUtilityClass(slot) {\n  return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_0__[\"default\"])('PrivateSwitchBase', slot);\n}\nconst switchBaseClasses = (0,_mui_utils_generateUtilityClasses__WEBPACK_IMPORTED_MODULE_1__[\"default\"])('PrivateSwitchBase', ['root', 'checked', 'disabled', 'input', 'edgeStart', 'edgeEnd']);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (switchBaseClasses);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/internal/switchBaseClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/ThemeProvider.js":
/*!*****************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/ThemeProvider.js ***!
  \*****************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _mui_utils__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils */ \"./node_modules/@mui/utils/esm/exactProp/exactProp.js\");\n/* harmony import */ var _useTheme_ThemeContext_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../useTheme/ThemeContext.js */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/ThemeContext.js\");\n/* harmony import */ var _useTheme_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../useTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/useTheme.js\");\n/* harmony import */ var _nested_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./nested.js */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/nested.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\n\n\n\n\n\n\n// To support composition of theme.\n\nfunction mergeOuterLocalTheme(outerTheme, localTheme) {\n  if (typeof localTheme === 'function') {\n    const mergedTheme = localTheme(outerTheme);\n    if (true) {\n      if (!mergedTheme) {\n        console.error(['MUI: You should return an object from your theme function, i.e.', '<ThemeProvider theme={() => ({})} />'].join('\\n'));\n      }\n    }\n    return mergedTheme;\n  }\n  return {\n    ...outerTheme,\n    ...localTheme\n  };\n}\n\n/**\n * This component takes a `theme` prop.\n * It makes the `theme` available down the React tree thanks to React context.\n * This component should preferably be used at **the root of your component tree**.\n */\nfunction ThemeProvider(props) {\n  const {\n    children,\n    theme: localTheme\n  } = props;\n  const outerTheme = (0,_useTheme_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])();\n  if (true) {\n    if (outerTheme === null && typeof localTheme === 'function') {\n      console.error(['MUI: You are providing a theme function prop to the ThemeProvider component:', '<ThemeProvider theme={outerTheme => outerTheme} />', '', 'However, no outer theme is present.', 'Make sure a theme is already injected higher in the React tree ' + 'or provide a theme object.'].join('\\n'));\n    }\n  }\n  const theme = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    const output = outerTheme === null ? {\n      ...localTheme\n    } : mergeOuterLocalTheme(outerTheme, localTheme);\n    if (output != null) {\n      output[_nested_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]] = outerTheme !== null;\n    }\n    return output;\n  }, [localTheme, outerTheme]);\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_useTheme_ThemeContext_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].Provider, {\n    value: theme,\n    children: children\n  });\n}\n true ? ThemeProvider.propTypes = {\n  /**\n   * Your component tree.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().node),\n  /**\n   * A theme object. You can provide a function to extend the outer theme.\n   */\n  theme: prop_types__WEBPACK_IMPORTED_MODULE_5___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_5___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_5___default().func)]).isRequired\n} : 0;\nif (true) {\n   true ? ThemeProvider.propTypes = (0,_mui_utils__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(ThemeProvider.propTypes) : 0;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ThemeProvider);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/ThemeProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/nested.js":
/*!**********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/nested.js ***!
  \**********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst hasSymbol = typeof Symbol === 'function' && Symbol.for;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (hasSymbol ? Symbol.for('mui.nested') : '__THEME_NESTED__');\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/nested.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/ThemeContext.js":
/*!***********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/ThemeContext.js ***!
  \***********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\nconst ThemeContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext(null);\nif (true) {\n  ThemeContext.displayName = 'ThemeContext';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ThemeContext);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/ThemeContext.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/useTheme.js":
/*!*******************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/useTheme.js ***!
  \*******************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useTheme)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _ThemeContext_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./ThemeContext.js */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/ThemeContext.js\");\n\n\nfunction useTheme() {\n  const theme = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_ThemeContext_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"]);\n  if (true) {\n    // TODO: uncomment once we enable eslint-plugin-react-compiler eslint-disable-next-line react-compiler/react-compiler\n    // eslint-disable-next-line react-hooks/rules-of-hooks -- It's not required to run React.useDebugValue in production\n    react__WEBPACK_IMPORTED_MODULE_0__.useDebugValue(theme);\n  }\n  return theme;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/useTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/styled-engine/GlobalStyles/GlobalStyles.js":
/*!*************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/styled-engine/GlobalStyles/GlobalStyles.js ***!
  \*************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ GlobalStyles)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_3__);\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\nfunction isEmpty(obj) {\n  return obj === undefined || obj === null || Object.keys(obj).length === 0;\n}\nfunction GlobalStyles(props) {\n  const {\n    styles,\n    defaultTheme = {}\n  } = props;\n  const globalStyles = typeof styles === 'function' ? themeInput => styles(isEmpty(themeInput) ? defaultTheme : themeInput) : styles;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_emotion_react__WEBPACK_IMPORTED_MODULE_2__.Global, {\n    styles: globalStyles\n  });\n}\n true ? GlobalStyles.propTypes = {\n  defaultTheme: (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object),\n  styles: prop_types__WEBPACK_IMPORTED_MODULE_3___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_3___default().array), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().string), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_3___default().func)])\n} : 0;\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/styled-engine/GlobalStyles/GlobalStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/styled-engine/StyledEngineProvider/StyledEngineProvider.js":
/*!*****************************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/styled-engine/StyledEngineProvider/StyledEngineProvider.js ***!
  \*****************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   TEST_INTERNALS_DO_NOT_USE: () => (/* binding */ TEST_INTERNALS_DO_NOT_USE),\n/* harmony export */   \"default\": () => (/* binding */ StyledEngineProvider)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var _emotion_cache__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/cache */ \"./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js\");\n/* harmony import */ var _emotion_sheet__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/sheet */ \"./node_modules/@emotion/sheet/dist/emotion-sheet.development.esm.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n// To fix [Jest performance](https://github.com/mui/material-ui/issues/45638).\n\nconst cacheMap = new Map();\n\n// Need to add a private variable to test the generated CSS from Emotion, this is the simplest way to do it.\n// We can't test the CSS from `style` tag easily because the `speedy: true` (produce empty text content) is enabled by Emotion.\n// Even if we disable it, JSDOM needs extra configuration to be able to parse `@layer` CSS.\nconst TEST_INTERNALS_DO_NOT_USE = {\n  /**\n   * to intercept the generated CSS before inserting to the style tag, so that we can check the generated CSS.\n   *\n   * let rule;\n   * TEST_INTERNALS_DO_NOT_USE.insert = (...args) => {\n   *    rule = args[0];\n   * };\n   *\n   * expect(rule).to.equal(...);\n   */\n  insert: undefined\n};\n\n// We might be able to remove this when this issue is fixed:\n// https://github.com/emotion-js/emotion/issues/2790\nconst createEmotionCache = (options, CustomSheet) => {\n  const cache = (0,_emotion_cache__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(options);\n\n  // Do the same as https://github.com/emotion-js/emotion/blob/main/packages/cache/src/index.js#L238-L245\n  cache.sheet = new CustomSheet({\n    key: cache.key,\n    nonce: cache.sheet.nonce,\n    container: cache.sheet.container,\n    speedy: cache.sheet.isSpeedy,\n    prepend: cache.sheet.prepend,\n    insertionPoint: cache.sheet.insertionPoint\n  });\n  return cache;\n};\nlet insertionPoint;\nif (typeof document === 'object') {\n  // Use `insertionPoint` over `prepend`(deprecated) because it can be controlled for GlobalStyles injection order\n  // For more information, see https://github.com/mui/material-ui/issues/44597\n  insertionPoint = document.querySelector('[name=\"emotion-insertion-point\"]');\n  if (!insertionPoint) {\n    insertionPoint = document.createElement('meta');\n    insertionPoint.setAttribute('name', 'emotion-insertion-point');\n    insertionPoint.setAttribute('content', '');\n    const head = document.querySelector('head');\n    if (head) {\n      head.prepend(insertionPoint);\n    }\n  }\n}\nfunction getCache(injectFirst, enableCssLayer) {\n  if (injectFirst || enableCssLayer) {\n    /**\n     * This is for client-side apps only.\n     * A custom sheet is required to make the GlobalStyles API injected above the insertion point.\n     * This is because the [sheet](https://github.com/emotion-js/emotion/blob/main/packages/react/src/global.js#L94-L99) does not consume the options.\n     */\n    class MyStyleSheet extends _emotion_sheet__WEBPACK_IMPORTED_MODULE_2__.StyleSheet {\n      insert(rule, options) {\n        if (TEST_INTERNALS_DO_NOT_USE.insert) {\n          return TEST_INTERNALS_DO_NOT_USE.insert(rule, options);\n        }\n        if (this.key && this.key.endsWith('global')) {\n          this.before = insertionPoint;\n        }\n        return super.insert(rule, options);\n      }\n    }\n    const emotionCache = createEmotionCache({\n      key: 'css',\n      insertionPoint: injectFirst ? insertionPoint : undefined\n    }, MyStyleSheet);\n    if (enableCssLayer) {\n      const prevInsert = emotionCache.insert;\n      emotionCache.insert = (...args) => {\n        if (!args[1].styles.startsWith('@layer')) {\n          // avoid nested @layer\n          args[1].styles = `@layer mui {${args[1].styles}}`;\n        }\n        return prevInsert(...args);\n      };\n    }\n    return emotionCache;\n  }\n  return undefined;\n}\nfunction StyledEngineProvider(props) {\n  const {\n    injectFirst,\n    enableCssLayer,\n    children\n  } = props;\n  const cache = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    const cacheKey = `${injectFirst}-${enableCssLayer}`;\n    if (cacheMap.has(cacheKey)) {\n      return cacheMap.get(cacheKey);\n    }\n    const fresh = getCache(injectFirst, enableCssLayer);\n    cacheMap.set(cacheKey, fresh);\n    return fresh;\n  }, [injectFirst, enableCssLayer]);\n  return cache ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_3__.jsx)(_emotion_react__WEBPACK_IMPORTED_MODULE_4__.C, {\n    value: cache,\n    children: children\n  }) : children;\n}\n true ? StyledEngineProvider.propTypes = {\n  /**\n   * Your component tree.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().node),\n  /**\n   * If `true`, the styles are wrapped in `@layer mui`.\n   * Learn more about [Cascade layers](https://developer.mozilla.org/en-US/docs/Learn_web_development/Core/Styling_basics/Cascade_layers).\n   */\n  enableCssLayer: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().bool),\n  /**\n   * By default, the styles are injected last in the <head> element of the page.\n   * As a result, they gain more specificity than any other style sheet.\n   * If you want to override MUI's styles, set this prop.\n   */\n  injectFirst: (prop_types__WEBPACK_IMPORTED_MODULE_5___default().bool)\n} : 0;\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/styled-engine/StyledEngineProvider/StyledEngineProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/styled-engine/index.js":
/*!*****************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/styled-engine/index.js ***!
  \*****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   GlobalStyles: () => (/* reexport safe */ _GlobalStyles_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"]),\n/* harmony export */   StyledEngineProvider: () => (/* reexport safe */ _StyledEngineProvider_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"]),\n/* harmony export */   ThemeContext: () => (/* reexport safe */ _emotion_react__WEBPACK_IMPORTED_MODULE_2__.T),\n/* harmony export */   css: () => (/* reexport safe */ _emotion_react__WEBPACK_IMPORTED_MODULE_3__.css),\n/* harmony export */   \"default\": () => (/* binding */ styled),\n/* harmony export */   internal_mutateStyles: () => (/* binding */ internal_mutateStyles),\n/* harmony export */   internal_serializeStyles: () => (/* binding */ internal_serializeStyles),\n/* harmony export */   keyframes: () => (/* reexport safe */ _emotion_react__WEBPACK_IMPORTED_MODULE_3__.keyframes)\n/* harmony export */ });\n/* harmony import */ var _emotion_styled__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @emotion/styled */ \"./node_modules/@emotion/styled/dist/emotion-styled.browser.development.esm.js\");\n/* harmony import */ var _emotion_serialize__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/serialize */ \"./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var _StyledEngineProvider_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./StyledEngineProvider/index.js */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/StyledEngineProvider/StyledEngineProvider.js\");\n/* harmony import */ var _GlobalStyles_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./GlobalStyles/index.js */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/GlobalStyles/GlobalStyles.js\");\n/**\n * @mui/styled-engine v6.4.11\n *\n * @license MIT\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n/* eslint-disable no-underscore-dangle */\n\n\nfunction styled(tag, options) {\n  const stylesFactory = (0,_emotion_styled__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(tag, options);\n  if (true) {\n    return (...styles) => {\n      const component = typeof tag === 'string' ? `\"${tag}\"` : 'component';\n      if (styles.length === 0) {\n        console.error([`MUI: Seems like you called \\`styled(${component})()\\` without a \\`style\\` argument.`, 'You must provide a `styles` argument: `styled(\"div\")(styleYouForgotToPass)`.'].join('\\n'));\n      } else if (styles.some(style => style === undefined)) {\n        console.error(`MUI: the styled(${component})(...args) API requires all its args to be defined.`);\n      }\n      return stylesFactory(...styles);\n    };\n  }\n  // removed by dead control flow\n{}\n}\n\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction internal_mutateStyles(tag, processor) {\n  // Emotion attaches all the styles as `__emotion_styles`.\n  // Ref: https://github.com/emotion-js/emotion/blob/16d971d0da229596d6bcc39d282ba9753c9ee7cf/packages/styled/src/base.js#L186\n  if (Array.isArray(tag.__emotion_styles)) {\n    tag.__emotion_styles = processor(tag.__emotion_styles);\n  }\n}\n\n// Emotion only accepts an array, but we want to avoid allocations\nconst wrapper = [];\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction internal_serializeStyles(styles) {\n  wrapper[0] = styles;\n  return (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_1__.serializeStyles)(wrapper);\n}\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/styled-engine/index.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/Container/createContainer.js":
/*!**********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/Container/createContainer.js ***!
  \**********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createContainer)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_9___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_9__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _useThemeProps_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../useThemeProps/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/useThemeProps.js\");\n/* harmony import */ var _styled_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../styled/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styled/styled.js\");\n/* harmony import */ var _createTheme_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../createTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\n\n\n\n\n\n\n\n\n\nconst defaultTheme = (0,_createTheme_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])();\nconst defaultCreateStyledComponent = (0,_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])('div', {\n  name: 'MuiContainer',\n  slot: 'Root',\n  overridesResolver: (props, styles) => {\n    const {\n      ownerState\n    } = props;\n    return [styles.root, styles[`maxWidth${(0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(String(ownerState.maxWidth))}`], ownerState.fixed && styles.fixed, ownerState.disableGutters && styles.disableGutters];\n  }\n});\nconst useThemePropsDefault = inProps => (0,_useThemeProps_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])({\n  props: inProps,\n  name: 'MuiContainer',\n  defaultTheme\n});\nconst useUtilityClasses = (ownerState, componentName) => {\n  const getContainerUtilityClass = slot => {\n    return (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(componentName, slot);\n  };\n  const {\n    classes,\n    fixed,\n    disableGutters,\n    maxWidth\n  } = ownerState;\n  const slots = {\n    root: ['root', maxWidth && `maxWidth${(0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(String(maxWidth))}`, fixed && 'fixed', disableGutters && 'disableGutters']\n  };\n  return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(slots, getContainerUtilityClass, classes);\n};\nfunction createContainer(options = {}) {\n  const {\n    // This will allow adding custom styled fn (for example for custom sx style function)\n    createStyledComponent = defaultCreateStyledComponent,\n    useThemeProps = useThemePropsDefault,\n    componentName = 'MuiContainer'\n  } = options;\n  const ContainerRoot = createStyledComponent(({\n    theme,\n    ownerState\n  }) => ({\n    width: '100%',\n    marginLeft: 'auto',\n    boxSizing: 'border-box',\n    marginRight: 'auto',\n    ...(!ownerState.disableGutters && {\n      paddingLeft: theme.spacing(2),\n      paddingRight: theme.spacing(2),\n      // @ts-ignore module augmentation fails if custom breakpoints are used\n      [theme.breakpoints.up('sm')]: {\n        paddingLeft: theme.spacing(3),\n        paddingRight: theme.spacing(3)\n      }\n    })\n  }), ({\n    theme,\n    ownerState\n  }) => ownerState.fixed && Object.keys(theme.breakpoints.values).reduce((acc, breakpointValueKey) => {\n    const breakpoint = breakpointValueKey;\n    const value = theme.breakpoints.values[breakpoint];\n    if (value !== 0) {\n      // @ts-ignore\n      acc[theme.breakpoints.up(breakpoint)] = {\n        maxWidth: `${value}${theme.breakpoints.unit}`\n      };\n    }\n    return acc;\n  }, {}), ({\n    theme,\n    ownerState\n  }) => ({\n    // @ts-ignore module augmentation fails if custom breakpoints are used\n    ...(ownerState.maxWidth === 'xs' && {\n      // @ts-ignore module augmentation fails if custom breakpoints are used\n      [theme.breakpoints.up('xs')]: {\n        // @ts-ignore module augmentation fails if custom breakpoints are used\n        maxWidth: Math.max(theme.breakpoints.values.xs, 444)\n      }\n    }),\n    ...(ownerState.maxWidth &&\n    // @ts-ignore module augmentation fails if custom breakpoints are used\n    ownerState.maxWidth !== 'xs' && {\n      // @ts-ignore module augmentation fails if custom breakpoints are used\n      [theme.breakpoints.up(ownerState.maxWidth)]: {\n        // @ts-ignore module augmentation fails if custom breakpoints are used\n        maxWidth: `${theme.breakpoints.values[ownerState.maxWidth]}${theme.breakpoints.unit}`\n      }\n    })\n  }));\n  const Container = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Container(inProps, ref) {\n    const props = useThemeProps(inProps);\n    const {\n      className,\n      component = 'div',\n      disableGutters = false,\n      fixed = false,\n      maxWidth = 'lg',\n      classes: classesProp,\n      ...other\n    } = props;\n    const ownerState = {\n      ...props,\n      component,\n      disableGutters,\n      fixed,\n      maxWidth\n    };\n\n    // @ts-ignore module augmentation fails if custom breakpoints are used\n    const classes = useUtilityClasses(ownerState, componentName);\n    return (\n      /*#__PURE__*/\n      // @ts-ignore theme is injected by the styled util\n      (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(ContainerRoot, {\n        as: component\n        // @ts-ignore module augmentation fails if custom breakpoints are used\n        ,\n        ownerState: ownerState,\n        className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n        ref: ref,\n        ...other\n      })\n    );\n  });\n   true ? Container.propTypes /* remove-proptypes */ = {\n    children: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().node),\n    classes: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object),\n    className: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string),\n    component: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().elementType),\n    disableGutters: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n    fixed: (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool),\n    maxWidth: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOf(['xs', 'sm', 'md', 'lg', 'xl', false]), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().string)]),\n    sx: prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_9___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_9___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_9___default().object)])\n  } : 0;\n  return Container;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/Container/createContainer.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/DefaultPropsProvider/DefaultPropsProvider.js":
/*!**************************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/DefaultPropsProvider/DefaultPropsProvider.js ***!
  \**************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   useDefaultProps: () => (/* binding */ useDefaultProps)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_2__);\n/* harmony import */ var _mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/resolveProps */ \"./node_modules/@mui/utils/esm/resolveProps/resolveProps.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\nconst PropsContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext(undefined);\nfunction DefaultPropsProvider({\n  value,\n  children\n}) {\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(PropsContext.Provider, {\n    value: value,\n    children: children\n  });\n}\n true ? DefaultPropsProvider.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().node),\n  /**\n   * @ignore\n   */\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().object)\n} : 0;\nfunction getThemeProps(params) {\n  const {\n    theme,\n    name,\n    props\n  } = params;\n  if (!theme || !theme.components || !theme.components[name]) {\n    return props;\n  }\n  const config = theme.components[name];\n  if (config.defaultProps) {\n    // compatible with v5 signature\n    return (0,_mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(config.defaultProps, props);\n  }\n  if (!config.styleOverrides && !config.variants) {\n    // v6 signature, no property 'defaultProps'\n    return (0,_mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(config, props);\n  }\n  return props;\n}\nfunction useDefaultProps({\n  props,\n  name\n}) {\n  const ctx = react__WEBPACK_IMPORTED_MODULE_0__.useContext(PropsContext);\n  return getThemeProps({\n    props,\n    name,\n    theme: {\n      components: ctx\n    }\n  });\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (DefaultPropsProvider);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/DefaultPropsProvider/DefaultPropsProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/GlobalStyles/GlobalStyles.js":
/*!**********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/GlobalStyles/GlobalStyles.js ***!
  \**********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_4___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_4__);\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/GlobalStyles/GlobalStyles.js\");\n/* harmony import */ var _useTheme_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../useTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\nfunction GlobalStyles({\n  styles,\n  themeId,\n  defaultTheme = {}\n}) {\n  const upperTheme = (0,_useTheme_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(defaultTheme);\n  const globalStyles = typeof styles === 'function' ? styles(themeId ? upperTheme[themeId] || upperTheme : upperTheme) : styles;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_styled_engine__WEBPACK_IMPORTED_MODULE_3__[\"default\"], {\n    styles: globalStyles\n  });\n}\n true ? GlobalStyles.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * @ignore\n   */\n  defaultTheme: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().object),\n  /**\n   * @ignore\n   */\n  styles: prop_types__WEBPACK_IMPORTED_MODULE_4___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_4___default().array), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().string), (prop_types__WEBPACK_IMPORTED_MODULE_4___default().bool)]),\n  /**\n   * @ignore\n   */\n  themeId: (prop_types__WEBPACK_IMPORTED_MODULE_4___default().string)\n} : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (GlobalStyles);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/GlobalStyles/GlobalStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/InitColorSchemeScript/InitColorSchemeScript.js":
/*!****************************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/InitColorSchemeScript/InitColorSchemeScript.js ***!
  \****************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   DEFAULT_ATTRIBUTE: () => (/* binding */ DEFAULT_ATTRIBUTE),\n/* harmony export */   DEFAULT_COLOR_SCHEME_STORAGE_KEY: () => (/* binding */ DEFAULT_COLOR_SCHEME_STORAGE_KEY),\n/* harmony export */   DEFAULT_MODE_STORAGE_KEY: () => (/* binding */ DEFAULT_MODE_STORAGE_KEY),\n/* harmony export */   \"default\": () => (/* binding */ InitColorSchemeScript)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/**\n * Split this component for RSC import\n */\n\n\nconst DEFAULT_MODE_STORAGE_KEY = 'mode';\nconst DEFAULT_COLOR_SCHEME_STORAGE_KEY = 'color-scheme';\nconst DEFAULT_ATTRIBUTE = 'data-color-scheme';\nfunction InitColorSchemeScript(options) {\n  const {\n    defaultMode = 'system',\n    defaultLightColorScheme = 'light',\n    defaultDarkColorScheme = 'dark',\n    modeStorageKey = DEFAULT_MODE_STORAGE_KEY,\n    colorSchemeStorageKey = DEFAULT_COLOR_SCHEME_STORAGE_KEY,\n    attribute: initialAttribute = DEFAULT_ATTRIBUTE,\n    colorSchemeNode = 'document.documentElement',\n    nonce\n  } = options || {};\n  let setter = '';\n  let attribute = initialAttribute;\n  if (initialAttribute === 'class') {\n    attribute = '.%s';\n  }\n  if (initialAttribute === 'data') {\n    attribute = '[data-%s]';\n  }\n  if (attribute.startsWith('.')) {\n    const selector = attribute.substring(1);\n    setter += `${colorSchemeNode}.classList.remove('${selector}'.replace('%s', light), '${selector}'.replace('%s', dark));\n      ${colorSchemeNode}.classList.add('${selector}'.replace('%s', colorScheme));`;\n  }\n  const matches = attribute.match(/\\[([^\\]]+)\\]/); // case [data-color-scheme=%s] or [data-color-scheme]\n  if (matches) {\n    const [attr, value] = matches[1].split('=');\n    if (!value) {\n      setter += `${colorSchemeNode}.removeAttribute('${attr}'.replace('%s', light));\n      ${colorSchemeNode}.removeAttribute('${attr}'.replace('%s', dark));`;\n    }\n    setter += `\n      ${colorSchemeNode}.setAttribute('${attr}'.replace('%s', colorScheme), ${value ? `${value}.replace('%s', colorScheme)` : '\"\"'});`;\n  } else {\n    setter += `${colorSchemeNode}.setAttribute('${attribute}', colorScheme);`;\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(\"script\", {\n    suppressHydrationWarning: true,\n    nonce: typeof window === 'undefined' ? nonce : ''\n    // eslint-disable-next-line react/no-danger\n    ,\n    dangerouslySetInnerHTML: {\n      __html: `(function() {\ntry {\n  let colorScheme = '';\n  const mode = localStorage.getItem('${modeStorageKey}') || '${defaultMode}';\n  const dark = localStorage.getItem('${colorSchemeStorageKey}-dark') || '${defaultDarkColorScheme}';\n  const light = localStorage.getItem('${colorSchemeStorageKey}-light') || '${defaultLightColorScheme}';\n  if (mode === 'system') {\n    // handle system mode\n    const mql = window.matchMedia('(prefers-color-scheme: dark)');\n    if (mql.matches) {\n      colorScheme = dark\n    } else {\n      colorScheme = light\n    }\n  }\n  if (mode === 'light') {\n    colorScheme = light;\n  }\n  if (mode === 'dark') {\n    colorScheme = dark;\n  }\n  if (colorScheme) {\n    ${setter}\n  }\n} catch(e){}})();`\n    }\n  }, \"mui-color-scheme-init\");\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/InitColorSchemeScript/InitColorSchemeScript.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/RtlProvider/index.js":
/*!**************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/RtlProvider/index.js ***!
  \**************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   useRtl: () => (/* binding */ useRtl)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_2__);\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\nconst RtlContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext();\nfunction RtlProvider({\n  value,\n  ...props\n}) {\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(RtlContext.Provider, {\n    value: value ?? true,\n    ...props\n  });\n}\n true ? RtlProvider.propTypes = {\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().node),\n  value: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().bool)\n} : 0;\nconst useRtl = () => {\n  const value = react__WEBPACK_IMPORTED_MODULE_0__.useContext(RtlContext);\n  return value ?? false;\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (RtlProvider);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/RtlProvider/index.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/Stack/createStack.js":
/*!**************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/Stack/createStack.js ***!
  \**************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createStack),\n/* harmony export */   style: () => (/* binding */ style)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_12___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_12__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n/* harmony import */ var _mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/utils/composeClasses */ \"./node_modules/@mui/utils/esm/composeClasses/composeClasses.js\");\n/* harmony import */ var _styled_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../styled/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styled/styled.js\");\n/* harmony import */ var _useThemeProps_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../useThemeProps/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/useThemeProps.js\");\n/* harmony import */ var _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ../styleFunctionSx/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/extendSxProp.js\");\n/* harmony import */ var _createTheme_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../createTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n\n\n\n\n\n\n\n\n\n\n\n\n\nconst defaultTheme = (0,_createTheme_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])();\n// widening Theme to any so that the consumer can own the theme structure.\nconst defaultCreateStyledComponent = (0,_styled_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])('div', {\n  name: 'MuiStack',\n  slot: 'Root',\n  overridesResolver: (props, styles) => styles.root\n});\nfunction useThemePropsDefault(props) {\n  return (0,_useThemeProps_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])({\n    props,\n    name: 'MuiStack',\n    defaultTheme\n  });\n}\n\n/**\n * Return an array with the separator React element interspersed between\n * each React node of the input children.\n *\n * > joinChildren([1,2,3], 0)\n * [1,0,2,0,3]\n */\nfunction joinChildren(children, separator) {\n  const childrenArray = react__WEBPACK_IMPORTED_MODULE_0__.Children.toArray(children).filter(Boolean);\n  return childrenArray.reduce((output, child, index) => {\n    output.push(child);\n    if (index < childrenArray.length - 1) {\n      output.push(/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.cloneElement(separator, {\n        key: `separator-${index}`\n      }));\n    }\n    return output;\n  }, []);\n}\nconst getSideFromDirection = direction => {\n  return {\n    row: 'Left',\n    'row-reverse': 'Right',\n    column: 'Top',\n    'column-reverse': 'Bottom'\n  }[direction];\n};\nconst style = ({\n  ownerState,\n  theme\n}) => {\n  let styles = {\n    display: 'flex',\n    flexDirection: 'column',\n    ...(0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__.handleBreakpoints)({\n      theme\n    }, (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__.resolveBreakpointValues)({\n      values: ownerState.direction,\n      breakpoints: theme.breakpoints.values\n    }), propValue => ({\n      flexDirection: propValue\n    }))\n  };\n  if (ownerState.spacing) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_7__.createUnarySpacing)(theme);\n    const base = Object.keys(theme.breakpoints.values).reduce((acc, breakpoint) => {\n      if (typeof ownerState.spacing === 'object' && ownerState.spacing[breakpoint] != null || typeof ownerState.direction === 'object' && ownerState.direction[breakpoint] != null) {\n        acc[breakpoint] = true;\n      }\n      return acc;\n    }, {});\n    const directionValues = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__.resolveBreakpointValues)({\n      values: ownerState.direction,\n      base\n    });\n    const spacingValues = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__.resolveBreakpointValues)({\n      values: ownerState.spacing,\n      base\n    });\n    if (typeof directionValues === 'object') {\n      Object.keys(directionValues).forEach((breakpoint, index, breakpoints) => {\n        const directionValue = directionValues[breakpoint];\n        if (!directionValue) {\n          const previousDirectionValue = index > 0 ? directionValues[breakpoints[index - 1]] : 'column';\n          directionValues[breakpoint] = previousDirectionValue;\n        }\n      });\n    }\n    const styleFromPropValue = (propValue, breakpoint) => {\n      if (ownerState.useFlexGap) {\n        return {\n          gap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_7__.getValue)(transformer, propValue)\n        };\n      }\n      return {\n        // The useFlexGap={false} implement relies on each child to give up control of the margin.\n        // We need to reset the margin to avoid double spacing.\n        '& > :not(style):not(style)': {\n          margin: 0\n        },\n        '& > :not(style) ~ :not(style)': {\n          [`margin${getSideFromDirection(breakpoint ? directionValues[breakpoint] : ownerState.direction)}`]: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_7__.getValue)(transformer, propValue)\n        }\n      };\n    };\n    styles = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(styles, (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__.handleBreakpoints)({\n      theme\n    }, spacingValues, styleFromPropValue));\n  }\n  styles = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_6__.mergeBreakpointsInOrder)(theme.breakpoints, styles);\n  return styles;\n};\nfunction createStack(options = {}) {\n  const {\n    // This will allow adding custom styled fn (for example for custom sx style function)\n    createStyledComponent = defaultCreateStyledComponent,\n    useThemeProps = useThemePropsDefault,\n    componentName = 'MuiStack'\n  } = options;\n  const useUtilityClasses = () => {\n    const slots = {\n      root: ['root']\n    };\n    return (0,_mui_utils_composeClasses__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(slots, slot => (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_10__[\"default\"])(componentName, slot), {});\n  };\n  const StackRoot = createStyledComponent(style);\n  const Stack = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Grid(inProps, ref) {\n    const themeProps = useThemeProps(inProps);\n    const props = (0,_styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_11__[\"default\"])(themeProps); // `color` type conflicts with html color attribute.\n    const {\n      component = 'div',\n      direction = 'column',\n      spacing = 0,\n      divider,\n      children,\n      className,\n      useFlexGap = false,\n      ...other\n    } = props;\n    const ownerState = {\n      direction,\n      spacing,\n      useFlexGap\n    };\n    const classes = useUtilityClasses();\n    return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(StackRoot, {\n      as: component,\n      ownerState: ownerState,\n      ref: ref,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(classes.root, className),\n      ...other,\n      children: divider ? joinChildren(children, divider) : children\n    });\n  });\n   true ? Stack.propTypes /* remove-proptypes */ = {\n    children: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().node),\n    direction: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOf(['column-reverse', 'column', 'row-reverse', 'row']), prop_types__WEBPACK_IMPORTED_MODULE_12___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOf(['column-reverse', 'column', 'row-reverse', 'row'])), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object)]),\n    divider: (prop_types__WEBPACK_IMPORTED_MODULE_12___default().node),\n    spacing: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)])), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().string)]),\n    sx: prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_12___default().arrayOf(prop_types__WEBPACK_IMPORTED_MODULE_12___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_12___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().bool)])), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_12___default().object)])\n  } : 0;\n  return Stack;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/Stack/createStack.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/ThemeProvider/ThemeProvider.js":
/*!************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/ThemeProvider/ThemeProvider.js ***!
  \************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var _mui_private_theming__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/private-theming */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/useTheme.js\");\n/* harmony import */ var _mui_private_theming__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/private-theming */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/ThemeProvider/ThemeProvider.js\");\n/* harmony import */ var _mui_utils_exactProp__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/utils/exactProp */ \"./node_modules/@mui/utils/esm/exactProp/exactProp.js\");\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var _useThemeWithoutDefault_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../useThemeWithoutDefault/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeWithoutDefault/useThemeWithoutDefault.js\");\n/* harmony import */ var _RtlProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../RtlProvider/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/RtlProvider/index.js\");\n/* harmony import */ var _DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../DefaultPropsProvider/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/DefaultPropsProvider/DefaultPropsProvider.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\nconst EMPTY_THEME = {};\nfunction useThemeScoping(themeId, upperTheme, localTheme, isPrivate = false) {\n  return react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    const resolvedTheme = themeId ? upperTheme[themeId] || upperTheme : upperTheme;\n    if (typeof localTheme === 'function') {\n      const mergedTheme = localTheme(resolvedTheme);\n      const result = themeId ? {\n        ...upperTheme,\n        [themeId]: mergedTheme\n      } : mergedTheme;\n      // must return a function for the private theme to NOT merge with the upper theme.\n      // see the test case \"use provided theme from a callback\" in ThemeProvider.test.js\n      if (isPrivate) {\n        return () => result;\n      }\n      return result;\n    }\n    return themeId ? {\n      ...upperTheme,\n      [themeId]: localTheme\n    } : {\n      ...upperTheme,\n      ...localTheme\n    };\n  }, [themeId, upperTheme, localTheme, isPrivate]);\n}\n\n/**\n * This component makes the `theme` available down the React tree.\n * It should preferably be used at **the root of your component tree**.\n *\n * <ThemeProvider theme={theme}> // existing use case\n * <ThemeProvider theme={{ id: theme }}> // theme scoping\n */\nfunction ThemeProvider(props) {\n  const {\n    children,\n    theme: localTheme,\n    themeId\n  } = props;\n  const upperTheme = (0,_useThemeWithoutDefault_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(EMPTY_THEME);\n  const upperPrivateTheme = (0,_mui_private_theming__WEBPACK_IMPORTED_MODULE_3__[\"default\"])() || EMPTY_THEME;\n  if (true) {\n    if (upperTheme === null && typeof localTheme === 'function' || themeId && upperTheme && !upperTheme[themeId] && typeof localTheme === 'function') {\n      console.error(['MUI: You are providing a theme function prop to the ThemeProvider component:', '<ThemeProvider theme={outerTheme => outerTheme} />', '', 'However, no outer theme is present.', 'Make sure a theme is already injected higher in the React tree ' + 'or provide a theme object.'].join('\\n'));\n    }\n  }\n  const engineTheme = useThemeScoping(themeId, upperTheme, localTheme);\n  const privateTheme = useThemeScoping(themeId, upperPrivateTheme, localTheme, true);\n  const rtlValue = (themeId ? engineTheme[themeId] : engineTheme).direction === 'rtl';\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_private_theming__WEBPACK_IMPORTED_MODULE_4__[\"default\"], {\n    theme: privateTheme,\n    children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_styled_engine__WEBPACK_IMPORTED_MODULE_5__.T.Provider, {\n      value: engineTheme,\n      children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_RtlProvider_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n        value: rtlValue,\n        children: /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_DefaultPropsProvider_index_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n          value: themeId ? engineTheme[themeId].components : engineTheme.components,\n          children: children\n        })\n      })\n    })\n  });\n}\n true ? ThemeProvider.propTypes /* remove-proptypes */ = {\n  // ┌────────────────────────────── Warning ──────────────────────────────┐\n  // │ These PropTypes are generated from the TypeScript type definitions. │\n  // │    To update them, edit the d.ts file and run `pnpm proptypes`.     │\n  // └─────────────────────────────────────────────────────────────────────┘\n  /**\n   * Your component tree.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().node),\n  /**\n   * A theme object. You can provide a function to extend the outer theme.\n   */\n  theme: prop_types__WEBPACK_IMPORTED_MODULE_8___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_8___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object)]).isRequired,\n  /**\n   * The design system's unique id for getting the corresponded theme when there are multiple design systems.\n   */\n  themeId: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string)\n} : 0;\nif (true) {\n   true ? ThemeProvider.propTypes = (0,_mui_utils_exactProp__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(ThemeProvider.propTypes) : 0;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ThemeProvider);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/ThemeProvider/ThemeProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/borders/borders.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/borders/borders.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   border: () => (/* binding */ border),\n/* harmony export */   borderBottom: () => (/* binding */ borderBottom),\n/* harmony export */   borderBottomColor: () => (/* binding */ borderBottomColor),\n/* harmony export */   borderColor: () => (/* binding */ borderColor),\n/* harmony export */   borderLeft: () => (/* binding */ borderLeft),\n/* harmony export */   borderLeftColor: () => (/* binding */ borderLeftColor),\n/* harmony export */   borderRadius: () => (/* binding */ borderRadius),\n/* harmony export */   borderRight: () => (/* binding */ borderRight),\n/* harmony export */   borderRightColor: () => (/* binding */ borderRightColor),\n/* harmony export */   borderTop: () => (/* binding */ borderTop),\n/* harmony export */   borderTopColor: () => (/* binding */ borderTopColor),\n/* harmony export */   borderTransform: () => (/* binding */ borderTransform),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   outline: () => (/* binding */ outline),\n/* harmony export */   outlineColor: () => (/* binding */ outlineColor)\n/* harmony export */ });\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js\");\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n\n\n\n\n\nfunction borderTransform(value) {\n  if (typeof value !== 'number') {\n    return value;\n  }\n  return `${value}px solid`;\n}\nfunction createBorderStyle(prop, transform) {\n  return (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n    prop,\n    themeKey: 'borders',\n    transform\n  });\n}\nconst border = createBorderStyle('border', borderTransform);\nconst borderTop = createBorderStyle('borderTop', borderTransform);\nconst borderRight = createBorderStyle('borderRight', borderTransform);\nconst borderBottom = createBorderStyle('borderBottom', borderTransform);\nconst borderLeft = createBorderStyle('borderLeft', borderTransform);\nconst borderColor = createBorderStyle('borderColor');\nconst borderTopColor = createBorderStyle('borderTopColor');\nconst borderRightColor = createBorderStyle('borderRightColor');\nconst borderBottomColor = createBorderStyle('borderBottomColor');\nconst borderLeftColor = createBorderStyle('borderLeftColor');\nconst outline = createBorderStyle('outline', borderTransform);\nconst outlineColor = createBorderStyle('outlineColor');\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst borderRadius = props => {\n  if (props.borderRadius !== undefined && props.borderRadius !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_1__.createUnaryUnit)(props.theme, 'shape.borderRadius', 4, 'borderRadius');\n    const styleFromPropValue = propValue => ({\n      borderRadius: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_1__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)(props, props.borderRadius, styleFromPropValue);\n  }\n  return null;\n};\nborderRadius.propTypes =  true ? {\n  borderRadius: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]\n} : 0;\nborderRadius.filterProps = ['borderRadius'];\nconst borders = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(border, borderTop, borderRight, borderBottom, borderLeft, borderColor, borderTopColor, borderRightColor, borderBottomColor, borderLeftColor, borderRadius, outline, outlineColor);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (borders);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/borders/borders.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js":
/*!********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js ***!
  \********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   computeBreakpointsBase: () => (/* binding */ computeBreakpointsBase),\n/* harmony export */   createEmptyBreakpointObject: () => (/* binding */ createEmptyBreakpointObject),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   handleBreakpoints: () => (/* binding */ handleBreakpoints),\n/* harmony export */   mergeBreakpointsInOrder: () => (/* binding */ mergeBreakpointsInOrder),\n/* harmony export */   removeUnusedBreakpoints: () => (/* binding */ removeUnusedBreakpoints),\n/* harmony export */   resolveBreakpointValues: () => (/* binding */ resolveBreakpointValues),\n/* harmony export */   values: () => (/* binding */ values)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_2__);\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js\");\n/* harmony import */ var _cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../cssContainerQueries/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js\");\n\n\n\n\n\n// The breakpoint **start** at this value.\n// For instance with the first breakpoint xs: [xs, sm[.\nconst values = {\n  xs: 0,\n  // phone\n  sm: 600,\n  // tablet\n  md: 900,\n  // small laptop\n  lg: 1200,\n  // desktop\n  xl: 1536 // large screen\n};\nconst defaultBreakpoints = {\n  // Sorted ASC by size. That's important.\n  // It can't be configured as it's used statically for propTypes.\n  keys: ['xs', 'sm', 'md', 'lg', 'xl'],\n  up: key => `@media (min-width:${values[key]}px)`\n};\nconst defaultContainerQueries = {\n  containerQueries: containerName => ({\n    up: key => {\n      let result = typeof key === 'number' ? key : values[key] || key;\n      if (typeof result === 'number') {\n        result = `${result}px`;\n      }\n      return containerName ? `@container ${containerName} (min-width:${result})` : `@container (min-width:${result})`;\n    }\n  })\n};\nfunction handleBreakpoints(props, propValue, styleFromPropValue) {\n  const theme = props.theme || {};\n  if (Array.isArray(propValue)) {\n    const themeBreakpoints = theme.breakpoints || defaultBreakpoints;\n    return propValue.reduce((acc, item, index) => {\n      acc[themeBreakpoints.up(themeBreakpoints.keys[index])] = styleFromPropValue(propValue[index]);\n      return acc;\n    }, {});\n  }\n  if (typeof propValue === 'object') {\n    const themeBreakpoints = theme.breakpoints || defaultBreakpoints;\n    return Object.keys(propValue).reduce((acc, breakpoint) => {\n      if ((0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_0__.isCqShorthand)(themeBreakpoints.keys, breakpoint)) {\n        const containerKey = (0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_0__.getContainerQuery)(theme.containerQueries ? theme : defaultContainerQueries, breakpoint);\n        if (containerKey) {\n          acc[containerKey] = styleFromPropValue(propValue[breakpoint], breakpoint);\n        }\n      }\n      // key is breakpoint\n      else if (Object.keys(themeBreakpoints.values || values).includes(breakpoint)) {\n        const mediaKey = themeBreakpoints.up(breakpoint);\n        acc[mediaKey] = styleFromPropValue(propValue[breakpoint], breakpoint);\n      } else {\n        const cssKey = breakpoint;\n        acc[cssKey] = propValue[cssKey];\n      }\n      return acc;\n    }, {});\n  }\n  const output = styleFromPropValue(propValue);\n  return output;\n}\nfunction breakpoints(styleFunction) {\n  // false positive\n  // eslint-disable-next-line react/function-component-definition\n  const newStyleFunction = props => {\n    const theme = props.theme || {};\n    const base = styleFunction(props);\n    const themeBreakpoints = theme.breakpoints || defaultBreakpoints;\n    const extended = themeBreakpoints.keys.reduce((acc, key) => {\n      if (props[key]) {\n        acc = acc || {};\n        acc[themeBreakpoints.up(key)] = styleFunction({\n          theme,\n          ...props[key]\n        });\n      }\n      return acc;\n    }, null);\n    return (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(base, extended);\n  };\n  newStyleFunction.propTypes =  true ? {\n    ...styleFunction.propTypes,\n    xs: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().object),\n    sm: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().object),\n    md: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().object),\n    lg: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().object),\n    xl: (prop_types__WEBPACK_IMPORTED_MODULE_2___default().object)\n  } : 0;\n  newStyleFunction.filterProps = ['xs', 'sm', 'md', 'lg', 'xl', ...styleFunction.filterProps];\n  return newStyleFunction;\n}\nfunction createEmptyBreakpointObject(breakpointsInput = {}) {\n  const breakpointsInOrder = breakpointsInput.keys?.reduce((acc, key) => {\n    const breakpointStyleKey = breakpointsInput.up(key);\n    acc[breakpointStyleKey] = {};\n    return acc;\n  }, {});\n  return breakpointsInOrder || {};\n}\nfunction removeUnusedBreakpoints(breakpointKeys, style) {\n  return breakpointKeys.reduce((acc, key) => {\n    const breakpointOutput = acc[key];\n    const isBreakpointUnused = !breakpointOutput || Object.keys(breakpointOutput).length === 0;\n    if (isBreakpointUnused) {\n      delete acc[key];\n    }\n    return acc;\n  }, style);\n}\nfunction mergeBreakpointsInOrder(breakpointsInput, ...styles) {\n  const emptyBreakpoints = createEmptyBreakpointObject(breakpointsInput);\n  const mergedOutput = [emptyBreakpoints, ...styles].reduce((prev, next) => (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(prev, next), {});\n  return removeUnusedBreakpoints(Object.keys(emptyBreakpoints), mergedOutput);\n}\n\n// compute base for responsive values; e.g.,\n// [1,2,3] => {xs: true, sm: true, md: true}\n// {xs: 1, sm: 2, md: 3} => {xs: true, sm: true, md: true}\nfunction computeBreakpointsBase(breakpointValues, themeBreakpoints) {\n  // fixed value\n  if (typeof breakpointValues !== 'object') {\n    return {};\n  }\n  const base = {};\n  const breakpointsKeys = Object.keys(themeBreakpoints);\n  if (Array.isArray(breakpointValues)) {\n    breakpointsKeys.forEach((breakpoint, i) => {\n      if (i < breakpointValues.length) {\n        base[breakpoint] = true;\n      }\n    });\n  } else {\n    breakpointsKeys.forEach(breakpoint => {\n      if (breakpointValues[breakpoint] != null) {\n        base[breakpoint] = true;\n      }\n    });\n  }\n  return base;\n}\nfunction resolveBreakpointValues({\n  values: breakpointValues,\n  breakpoints: themeBreakpoints,\n  base: customBase\n}) {\n  const base = customBase || computeBreakpointsBase(breakpointValues, themeBreakpoints);\n  const keys = Object.keys(base);\n  if (keys.length === 0) {\n    return breakpointValues;\n  }\n  let previous;\n  return keys.reduce((acc, breakpoint, i) => {\n    if (Array.isArray(breakpointValues)) {\n      acc[breakpoint] = breakpointValues[i] != null ? breakpointValues[i] : breakpointValues[previous];\n      previous = i;\n    } else if (typeof breakpointValues === 'object') {\n      acc[breakpoint] = breakpointValues[breakpoint] != null ? breakpointValues[breakpoint] : breakpointValues[previous];\n      previous = breakpoint;\n    } else {\n      acc[breakpoint] = breakpointValues;\n    }\n    return acc;\n  }, {});\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (breakpoints);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js":
/*!******************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js ***!
  \******************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   alpha: () => (/* binding */ alpha),\n/* harmony export */   blend: () => (/* binding */ blend),\n/* harmony export */   colorChannel: () => (/* binding */ colorChannel),\n/* harmony export */   darken: () => (/* binding */ darken),\n/* harmony export */   decomposeColor: () => (/* binding */ decomposeColor),\n/* harmony export */   emphasize: () => (/* binding */ emphasize),\n/* harmony export */   getContrastRatio: () => (/* binding */ getContrastRatio),\n/* harmony export */   getLuminance: () => (/* binding */ getLuminance),\n/* harmony export */   hexToRgb: () => (/* binding */ hexToRgb),\n/* harmony export */   hslToRgb: () => (/* binding */ hslToRgb),\n/* harmony export */   lighten: () => (/* binding */ lighten),\n/* harmony export */   private_safeAlpha: () => (/* binding */ private_safeAlpha),\n/* harmony export */   private_safeColorChannel: () => (/* binding */ private_safeColorChannel),\n/* harmony export */   private_safeDarken: () => (/* binding */ private_safeDarken),\n/* harmony export */   private_safeEmphasize: () => (/* binding */ private_safeEmphasize),\n/* harmony export */   private_safeLighten: () => (/* binding */ private_safeLighten),\n/* harmony export */   recomposeColor: () => (/* binding */ recomposeColor),\n/* harmony export */   rgbToHex: () => (/* binding */ rgbToHex)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_clamp__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/clamp */ \"./node_modules/@mui/utils/esm/clamp/clamp.js\");\n\n/* eslint-disable @typescript-eslint/naming-convention */\n\n\n/**\n * Returns a number whose value is limited to the given range.\n * @param {number} value The value to be clamped\n * @param {number} min The lower boundary of the output range\n * @param {number} max The upper boundary of the output range\n * @returns {number} A number in the range [min, max]\n */\nfunction clampWrapper(value, min = 0, max = 1) {\n  if (true) {\n    if (value < min || value > max) {\n      console.error(`MUI: The value provided ${value} is out of range [${min}, ${max}].`);\n    }\n  }\n  return (0,_mui_utils_clamp__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(value, min, max);\n}\n\n/**\n * Converts a color from CSS hex format to CSS rgb format.\n * @param {string} color - Hex color, i.e. #nnn or #nnnnnn\n * @returns {string} A CSS rgb color string\n */\nfunction hexToRgb(color) {\n  color = color.slice(1);\n  const re = new RegExp(`.{1,${color.length >= 6 ? 2 : 1}}`, 'g');\n  let colors = color.match(re);\n  if (colors && colors[0].length === 1) {\n    colors = colors.map(n => n + n);\n  }\n  if (true) {\n    if (color.length !== color.trim().length) {\n      console.error(`MUI: The color: \"${color}\" is invalid. Make sure the color input doesn't contain leading/trailing space.`);\n    }\n  }\n  return colors ? `rgb${colors.length === 4 ? 'a' : ''}(${colors.map((n, index) => {\n    return index < 3 ? parseInt(n, 16) : Math.round(parseInt(n, 16) / 255 * 1000) / 1000;\n  }).join(', ')})` : '';\n}\nfunction intToHex(int) {\n  const hex = int.toString(16);\n  return hex.length === 1 ? `0${hex}` : hex;\n}\n\n/**\n * Returns an object with the type and values of a color.\n *\n * Note: Does not support rgb % values.\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @returns {object} - A MUI color object: {type: string, values: number[]}\n */\nfunction decomposeColor(color) {\n  // Idempotent\n  if (color.type) {\n    return color;\n  }\n  if (color.charAt(0) === '#') {\n    return decomposeColor(hexToRgb(color));\n  }\n  const marker = color.indexOf('(');\n  const type = color.substring(0, marker);\n  if (!['rgb', 'rgba', 'hsl', 'hsla', 'color'].includes(type)) {\n    throw new Error( true ? `MUI: Unsupported \\`${color}\\` color.\\n` + 'The following formats are supported: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color().' : 0);\n  }\n  let values = color.substring(marker + 1, color.length - 1);\n  let colorSpace;\n  if (type === 'color') {\n    values = values.split(' ');\n    colorSpace = values.shift();\n    if (values.length === 4 && values[3].charAt(0) === '/') {\n      values[3] = values[3].slice(1);\n    }\n    if (!['srgb', 'display-p3', 'a98-rgb', 'prophoto-rgb', 'rec-2020'].includes(colorSpace)) {\n      throw new Error( true ? `MUI: unsupported \\`${colorSpace}\\` color space.\\n` + 'The following color spaces are supported: srgb, display-p3, a98-rgb, prophoto-rgb, rec-2020.' : 0);\n    }\n  } else {\n    values = values.split(',');\n  }\n  values = values.map(value => parseFloat(value));\n  return {\n    type,\n    values,\n    colorSpace\n  };\n}\n\n/**\n * Returns a channel created from the input color.\n *\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @returns {string} - The channel for the color, that can be used in rgba or hsla colors\n */\nconst colorChannel = color => {\n  const decomposedColor = decomposeColor(color);\n  return decomposedColor.values.slice(0, 3).map((val, idx) => decomposedColor.type.includes('hsl') && idx !== 0 ? `${val}%` : val).join(' ');\n};\nconst private_safeColorChannel = (color, warning) => {\n  try {\n    return colorChannel(color);\n  } catch (error) {\n    if (warning && \"development\" !== 'production') {\n      console.warn(warning);\n    }\n    return color;\n  }\n};\n\n/**\n * Converts a color object with type and values to a string.\n * @param {object} color - Decomposed color\n * @param {string} color.type - One of: 'rgb', 'rgba', 'hsl', 'hsla', 'color'\n * @param {array} color.values - [n,n,n] or [n,n,n,n]\n * @returns {string} A CSS color string\n */\nfunction recomposeColor(color) {\n  const {\n    type,\n    colorSpace\n  } = color;\n  let {\n    values\n  } = color;\n  if (type.includes('rgb')) {\n    // Only convert the first 3 values to int (i.e. not alpha)\n    values = values.map((n, i) => i < 3 ? parseInt(n, 10) : n);\n  } else if (type.includes('hsl')) {\n    values[1] = `${values[1]}%`;\n    values[2] = `${values[2]}%`;\n  }\n  if (type.includes('color')) {\n    values = `${colorSpace} ${values.join(' ')}`;\n  } else {\n    values = `${values.join(', ')}`;\n  }\n  return `${type}(${values})`;\n}\n\n/**\n * Converts a color from CSS rgb format to CSS hex format.\n * @param {string} color - RGB color, i.e. rgb(n, n, n)\n * @returns {string} A CSS rgb color string, i.e. #nnnnnn\n */\nfunction rgbToHex(color) {\n  // Idempotent\n  if (color.startsWith('#')) {\n    return color;\n  }\n  const {\n    values\n  } = decomposeColor(color);\n  return `#${values.map((n, i) => intToHex(i === 3 ? Math.round(255 * n) : n)).join('')}`;\n}\n\n/**\n * Converts a color from hsl format to rgb format.\n * @param {string} color - HSL color values\n * @returns {string} rgb color values\n */\nfunction hslToRgb(color) {\n  color = decomposeColor(color);\n  const {\n    values\n  } = color;\n  const h = values[0];\n  const s = values[1] / 100;\n  const l = values[2] / 100;\n  const a = s * Math.min(l, 1 - l);\n  const f = (n, k = (n + h / 30) % 12) => l - a * Math.max(Math.min(k - 3, 9 - k, 1), -1);\n  let type = 'rgb';\n  const rgb = [Math.round(f(0) * 255), Math.round(f(8) * 255), Math.round(f(4) * 255)];\n  if (color.type === 'hsla') {\n    type += 'a';\n    rgb.push(values[3]);\n  }\n  return recomposeColor({\n    type,\n    values: rgb\n  });\n}\n/**\n * The relative brightness of any point in a color space,\n * normalized to 0 for darkest black and 1 for lightest white.\n *\n * Formula: https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @returns {number} The relative brightness of the color in the range 0 - 1\n */\nfunction getLuminance(color) {\n  color = decomposeColor(color);\n  let rgb = color.type === 'hsl' || color.type === 'hsla' ? decomposeColor(hslToRgb(color)).values : color.values;\n  rgb = rgb.map(val => {\n    if (color.type !== 'color') {\n      val /= 255; // normalized\n    }\n    return val <= 0.03928 ? val / 12.92 : ((val + 0.055) / 1.055) ** 2.4;\n  });\n\n  // Truncate at 3 digits\n  return Number((0.2126 * rgb[0] + 0.7152 * rgb[1] + 0.0722 * rgb[2]).toFixed(3));\n}\n\n/**\n * Calculates the contrast ratio between two colors.\n *\n * Formula: https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests\n * @param {string} foreground - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla()\n * @param {string} background - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla()\n * @returns {number} A contrast ratio value in the range 0 - 21.\n */\nfunction getContrastRatio(foreground, background) {\n  const lumA = getLuminance(foreground);\n  const lumB = getLuminance(background);\n  return (Math.max(lumA, lumB) + 0.05) / (Math.min(lumA, lumB) + 0.05);\n}\n\n/**\n * Sets the absolute transparency of a color.\n * Any existing alpha values are overwritten.\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @param {number} value - value to set the alpha channel to in the range 0 - 1\n * @returns {string} A CSS color string. Hex input values are returned as rgb\n */\nfunction alpha(color, value) {\n  color = decomposeColor(color);\n  value = clampWrapper(value);\n  if (color.type === 'rgb' || color.type === 'hsl') {\n    color.type += 'a';\n  }\n  if (color.type === 'color') {\n    color.values[3] = `/${value}`;\n  } else {\n    color.values[3] = value;\n  }\n  return recomposeColor(color);\n}\nfunction private_safeAlpha(color, value, warning) {\n  try {\n    return alpha(color, value);\n  } catch (error) {\n    if (warning && \"development\" !== 'production') {\n      console.warn(warning);\n    }\n    return color;\n  }\n}\n\n/**\n * Darkens a color.\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @param {number} coefficient - multiplier in the range 0 - 1\n * @returns {string} A CSS color string. Hex input values are returned as rgb\n */\nfunction darken(color, coefficient) {\n  color = decomposeColor(color);\n  coefficient = clampWrapper(coefficient);\n  if (color.type.includes('hsl')) {\n    color.values[2] *= 1 - coefficient;\n  } else if (color.type.includes('rgb') || color.type.includes('color')) {\n    for (let i = 0; i < 3; i += 1) {\n      color.values[i] *= 1 - coefficient;\n    }\n  }\n  return recomposeColor(color);\n}\nfunction private_safeDarken(color, coefficient, warning) {\n  try {\n    return darken(color, coefficient);\n  } catch (error) {\n    if (warning && \"development\" !== 'production') {\n      console.warn(warning);\n    }\n    return color;\n  }\n}\n\n/**\n * Lightens a color.\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @param {number} coefficient - multiplier in the range 0 - 1\n * @returns {string} A CSS color string. Hex input values are returned as rgb\n */\nfunction lighten(color, coefficient) {\n  color = decomposeColor(color);\n  coefficient = clampWrapper(coefficient);\n  if (color.type.includes('hsl')) {\n    color.values[2] += (100 - color.values[2]) * coefficient;\n  } else if (color.type.includes('rgb')) {\n    for (let i = 0; i < 3; i += 1) {\n      color.values[i] += (255 - color.values[i]) * coefficient;\n    }\n  } else if (color.type.includes('color')) {\n    for (let i = 0; i < 3; i += 1) {\n      color.values[i] += (1 - color.values[i]) * coefficient;\n    }\n  }\n  return recomposeColor(color);\n}\nfunction private_safeLighten(color, coefficient, warning) {\n  try {\n    return lighten(color, coefficient);\n  } catch (error) {\n    if (warning && \"development\" !== 'production') {\n      console.warn(warning);\n    }\n    return color;\n  }\n}\n\n/**\n * Darken or lighten a color, depending on its luminance.\n * Light colors are darkened, dark colors are lightened.\n * @param {string} color - CSS color, i.e. one of: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color()\n * @param {number} coefficient=0.15 - multiplier in the range 0 - 1\n * @returns {string} A CSS color string. Hex input values are returned as rgb\n */\nfunction emphasize(color, coefficient = 0.15) {\n  return getLuminance(color) > 0.5 ? darken(color, coefficient) : lighten(color, coefficient);\n}\nfunction private_safeEmphasize(color, coefficient, warning) {\n  try {\n    return emphasize(color, coefficient);\n  } catch (error) {\n    if (warning && \"development\" !== 'production') {\n      console.warn(warning);\n    }\n    return color;\n  }\n}\n\n/**\n * Blend a transparent overlay color with a background color, resulting in a single\n * RGB color.\n * @param {string} background - CSS color\n * @param {string} overlay - CSS color\n * @param {number} opacity - Opacity multiplier in the range 0 - 1\n * @param {number} [gamma=1.0] - Gamma correction factor. For gamma-correct blending, 2.2 is usual.\n */\nfunction blend(background, overlay, opacity, gamma = 1.0) {\n  const blendChannel = (b, o) => Math.round((b ** (1 / gamma) * (1 - opacity) + o ** (1 / gamma) * opacity) ** gamma);\n  const backgroundColor = decomposeColor(background);\n  const overlayColor = decomposeColor(overlay);\n  const rgb = [blendChannel(backgroundColor.values[0], overlayColor.values[0]), blendChannel(backgroundColor.values[1], overlayColor.values[1]), blendChannel(backgroundColor.values[2], overlayColor.values[2])];\n  return recomposeColor({\n    type: 'rgb',\n    values: rgb\n  });\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js\");\n\nfunction compose(...styles) {\n  const handlers = styles.reduce((acc, style) => {\n    style.filterProps.forEach(prop => {\n      acc[prop] = style;\n    });\n    return acc;\n  }, {});\n\n  // false positive\n  // eslint-disable-next-line react/function-component-definition\n  const fn = props => {\n    return Object.keys(props).reduce((acc, prop) => {\n      if (handlers[prop]) {\n        return (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(acc, handlers[prop](props));\n      }\n      return acc;\n    }, {});\n  };\n  fn.propTypes =  true ? styles.reduce((acc, style) => Object.assign(acc, style.propTypes), {}) : 0;\n  fn.filterProps = styles.reduce((acc, style) => acc.concat(style.filterProps), []);\n  return fn;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (compose);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createBox/createBox.js":
/*!****************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createBox/createBox.js ***!
  \****************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createBox)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/index.js\");\n/* harmony import */ var _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../styleFunctionSx/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../styleFunctionSx/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/extendSxProp.js\");\n/* harmony import */ var _useTheme_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../useTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\nfunction createBox(options = {}) {\n  const {\n    themeId,\n    defaultTheme,\n    defaultClassName = 'MuiBox-root',\n    generateClassName\n  } = options;\n  const BoxRoot = (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_3__[\"default\"])('div', {\n    shouldForwardProp: prop => prop !== 'theme' && prop !== 'sx' && prop !== 'as'\n  })(_styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"]);\n  const Box = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(function Box(inProps, ref) {\n    const theme = (0,_useTheme_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(defaultTheme);\n    const {\n      className,\n      component = 'div',\n      ...other\n    } = (0,_styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(inProps);\n    return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_2__.jsx)(BoxRoot, {\n      as: component,\n      ref: ref,\n      className: (0,clsx__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(className, generateClassName ? generateClassName(defaultClassName) : defaultClassName),\n      theme: themeId ? theme[themeId] || theme : theme,\n      ...other\n    });\n  });\n  return Box;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createBox/createBox.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js":
/*!********************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js ***!
  \********************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   breakpointKeys: () => (/* binding */ breakpointKeys),\n/* harmony export */   \"default\": () => (/* binding */ createBreakpoints)\n/* harmony export */ });\n// Sorted ASC by size. That's important.\n// It can't be configured as it's used statically for propTypes.\nconst breakpointKeys = ['xs', 'sm', 'md', 'lg', 'xl'];\nconst sortBreakpointsValues = values => {\n  const breakpointsAsArray = Object.keys(values).map(key => ({\n    key,\n    val: values[key]\n  })) || [];\n  // Sort in ascending order\n  breakpointsAsArray.sort((breakpoint1, breakpoint2) => breakpoint1.val - breakpoint2.val);\n  return breakpointsAsArray.reduce((acc, obj) => {\n    return {\n      ...acc,\n      [obj.key]: obj.val\n    };\n  }, {});\n};\n\n// Keep in mind that @media is inclusive by the CSS specification.\nfunction createBreakpoints(breakpoints) {\n  const {\n    // The breakpoint **start** at this value.\n    // For instance with the first breakpoint xs: [xs, sm).\n    values = {\n      xs: 0,\n      // phone\n      sm: 600,\n      // tablet\n      md: 900,\n      // small laptop\n      lg: 1200,\n      // desktop\n      xl: 1536 // large screen\n    },\n    unit = 'px',\n    step = 5,\n    ...other\n  } = breakpoints;\n  const sortedValues = sortBreakpointsValues(values);\n  const keys = Object.keys(sortedValues);\n  function up(key) {\n    const value = typeof values[key] === 'number' ? values[key] : key;\n    return `@media (min-width:${value}${unit})`;\n  }\n  function down(key) {\n    const value = typeof values[key] === 'number' ? values[key] : key;\n    return `@media (max-width:${value - step / 100}${unit})`;\n  }\n  function between(start, end) {\n    const endIndex = keys.indexOf(end);\n    return `@media (min-width:${typeof values[start] === 'number' ? values[start] : start}${unit}) and ` + `(max-width:${(endIndex !== -1 && typeof values[keys[endIndex]] === 'number' ? values[keys[endIndex]] : end) - step / 100}${unit})`;\n  }\n  function only(key) {\n    if (keys.indexOf(key) + 1 < keys.length) {\n      return between(key, keys[keys.indexOf(key) + 1]);\n    }\n    return up(key);\n  }\n  function not(key) {\n    // handle first and last key separately, for better readability\n    const keyIndex = keys.indexOf(key);\n    if (keyIndex === 0) {\n      return up(keys[1]);\n    }\n    if (keyIndex === keys.length - 1) {\n      return down(keys[keyIndex]);\n    }\n    return between(key, keys[keys.indexOf(key) + 1]).replace('@media', '@media not all and');\n  }\n  return {\n    keys,\n    values: sortedValues,\n    up,\n    down,\n    between,\n    only,\n    not,\n    unit,\n    ...other\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createStyled/createStyled.js":
/*!**********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createStyled/createStyled.js ***!
  \**********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createStyled),\n/* harmony export */   shouldForwardProp: () => (/* binding */ shouldForwardProp),\n/* harmony export */   systemDefaultTheme: () => (/* binding */ systemDefaultTheme)\n/* harmony export */ });\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/index.js\");\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _mui_utils_getDisplayName__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils/getDisplayName */ \"./node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js\");\n/* harmony import */ var _createTheme_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js\");\n/* harmony import */ var _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../styleFunctionSx/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _preprocessStyles_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../preprocessStyles.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/preprocessStyles.js\");\n\n\n\n\n\n\n\n\n/* eslint-disable no-underscore-dangle */\n/* eslint-disable no-labels */\n/* eslint-disable no-lone-blocks */\n\nconst systemDefaultTheme = (0,_createTheme_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\n\n// Update /system/styled/#api in case if this changes\nfunction shouldForwardProp(prop) {\n  return prop !== 'ownerState' && prop !== 'theme' && prop !== 'sx' && prop !== 'as';\n}\nfunction defaultOverridesResolver(slot) {\n  if (!slot) {\n    return null;\n  }\n  return (_props, styles) => styles[slot];\n}\nfunction attachTheme(props, themeId, defaultTheme) {\n  props.theme = isObjectEmpty(props.theme) ? defaultTheme : props.theme[themeId] || props.theme;\n}\nfunction processStyle(props, style) {\n  /*\n   * Style types:\n   *  - null/undefined\n   *  - string\n   *  - CSS style object: { [cssKey]: [cssValue], variants }\n   *  - Processed style object: { style, variants, isProcessed: true }\n   *  - Array of any of the above\n   */\n\n  const resolvedStyle = typeof style === 'function' ? style(props) : style;\n  if (Array.isArray(resolvedStyle)) {\n    return resolvedStyle.flatMap(subStyle => processStyle(props, subStyle));\n  }\n  if (Array.isArray(resolvedStyle?.variants)) {\n    let rootStyle;\n    if (resolvedStyle.isProcessed) {\n      rootStyle = resolvedStyle.style;\n    } else {\n      const {\n        variants,\n        ...otherStyles\n      } = resolvedStyle;\n      rootStyle = otherStyles;\n    }\n    return processStyleVariants(props, resolvedStyle.variants, [rootStyle]);\n  }\n  if (resolvedStyle?.isProcessed) {\n    return resolvedStyle.style;\n  }\n  return resolvedStyle;\n}\nfunction processStyleVariants(props, variants, results = []) {\n  let mergedState; // We might not need it, initialized lazily\n\n  variantLoop: for (let i = 0; i < variants.length; i += 1) {\n    const variant = variants[i];\n    if (typeof variant.props === 'function') {\n      mergedState ??= {\n        ...props,\n        ...props.ownerState,\n        ownerState: props.ownerState\n      };\n      if (!variant.props(mergedState)) {\n        continue;\n      }\n    } else {\n      for (const key in variant.props) {\n        if (props[key] !== variant.props[key] && props.ownerState?.[key] !== variant.props[key]) {\n          continue variantLoop;\n        }\n      }\n    }\n    if (typeof variant.style === 'function') {\n      mergedState ??= {\n        ...props,\n        ...props.ownerState,\n        ownerState: props.ownerState\n      };\n      results.push(variant.style(mergedState));\n    } else {\n      results.push(variant.style);\n    }\n  }\n  return results;\n}\nfunction createStyled(input = {}) {\n  const {\n    themeId,\n    defaultTheme = systemDefaultTheme,\n    rootShouldForwardProp = shouldForwardProp,\n    slotShouldForwardProp = shouldForwardProp\n  } = input;\n  function styleAttachTheme(props) {\n    attachTheme(props, themeId, defaultTheme);\n  }\n  const styled = (tag, inputOptions = {}) => {\n    // If `tag` is already a styled component, filter out the `sx` style function\n    // to prevent unnecessary styles generated by the composite components.\n    (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__.internal_mutateStyles)(tag, styles => styles.filter(style => style !== _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]));\n    const {\n      name: componentName,\n      slot: componentSlot,\n      skipVariantsResolver: inputSkipVariantsResolver,\n      skipSx: inputSkipSx,\n      // TODO v6: remove `lowercaseFirstLetter()` in the next major release\n      // For more details: https://github.com/mui/material-ui/pull/37908\n      overridesResolver = defaultOverridesResolver(lowercaseFirstLetter(componentSlot)),\n      ...options\n    } = inputOptions;\n\n    // if skipVariantsResolver option is defined, take the value, otherwise, true for root and false for other slots.\n    const skipVariantsResolver = inputSkipVariantsResolver !== undefined ? inputSkipVariantsResolver :\n    // TODO v6: remove `Root` in the next major release\n    // For more details: https://github.com/mui/material-ui/pull/37908\n    componentSlot && componentSlot !== 'Root' && componentSlot !== 'root' || false;\n    const skipSx = inputSkipSx || false;\n    let shouldForwardPropOption = shouldForwardProp;\n\n    // TODO v6: remove `Root` in the next major release\n    // For more details: https://github.com/mui/material-ui/pull/37908\n    if (componentSlot === 'Root' || componentSlot === 'root') {\n      shouldForwardPropOption = rootShouldForwardProp;\n    } else if (componentSlot) {\n      // any other slot specified\n      shouldForwardPropOption = slotShouldForwardProp;\n    } else if (isStringTag(tag)) {\n      // for string (html) tag, preserve the behavior in emotion & styled-components.\n      shouldForwardPropOption = undefined;\n    }\n    const defaultStyledResolver = (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(tag, {\n      shouldForwardProp: shouldForwardPropOption,\n      label: generateStyledLabel(componentName, componentSlot),\n      ...options\n    });\n    const transformStyle = style => {\n      // On the server Emotion doesn't use React.forwardRef for creating components, so the created\n      // component stays as a function. This condition makes sure that we do not interpolate functions\n      // which are basically components used as a selectors.\n      if (typeof style === 'function' && style.__emotion_real !== style) {\n        return function styleFunctionProcessor(props) {\n          return processStyle(props, style);\n        };\n      }\n      if ((0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__.isPlainObject)(style)) {\n        const serialized = (0,_preprocessStyles_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(style);\n        if (!serialized.variants) {\n          return serialized.style;\n        }\n        return function styleObjectProcessor(props) {\n          return processStyle(props, serialized);\n        };\n      }\n      return style;\n    };\n    const muiStyledResolver = (...expressionsInput) => {\n      const expressionsHead = [];\n      const expressionsBody = expressionsInput.map(transformStyle);\n      const expressionsTail = [];\n\n      // Preprocess `props` to set the scoped theme value.\n      // This must run before any other expression.\n      expressionsHead.push(styleAttachTheme);\n      if (componentName && overridesResolver) {\n        expressionsTail.push(function styleThemeOverrides(props) {\n          const theme = props.theme;\n          const styleOverrides = theme.components?.[componentName]?.styleOverrides;\n          if (!styleOverrides) {\n            return null;\n          }\n          const resolvedStyleOverrides = {};\n\n          // TODO: v7 remove iteration and use `resolveStyleArg(styleOverrides[slot])` directly\n          // eslint-disable-next-line guard-for-in\n          for (const slotKey in styleOverrides) {\n            resolvedStyleOverrides[slotKey] = processStyle(props, styleOverrides[slotKey]);\n          }\n          return overridesResolver(props, resolvedStyleOverrides);\n        });\n      }\n      if (componentName && !skipVariantsResolver) {\n        expressionsTail.push(function styleThemeVariants(props) {\n          const theme = props.theme;\n          const themeVariants = theme?.components?.[componentName]?.variants;\n          if (!themeVariants) {\n            return null;\n          }\n          return processStyleVariants(props, themeVariants);\n        });\n      }\n      if (!skipSx) {\n        expressionsTail.push(_styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]);\n      }\n\n      // This function can be called as a tagged template, so the first argument would contain\n      // CSS `string[]` values.\n      if (Array.isArray(expressionsBody[0])) {\n        const inputStrings = expressionsBody.shift();\n\n        // We need to add placeholders in the tagged template for the custom functions we have\n        // possibly added (attachTheme, overrides, variants, and sx).\n        const placeholdersHead = new Array(expressionsHead.length).fill('');\n        const placeholdersTail = new Array(expressionsTail.length).fill('');\n        let outputStrings;\n        // prettier-ignore\n        {\n          outputStrings = [...placeholdersHead, ...inputStrings, ...placeholdersTail];\n          outputStrings.raw = [...placeholdersHead, ...inputStrings.raw, ...placeholdersTail];\n        }\n\n        // The only case where we put something before `attachTheme`\n        expressionsHead.unshift(outputStrings);\n      }\n      const expressions = [...expressionsHead, ...expressionsBody, ...expressionsTail];\n      const Component = defaultStyledResolver(...expressions);\n      if (tag.muiName) {\n        Component.muiName = tag.muiName;\n      }\n      if (true) {\n        Component.displayName = generateDisplayName(componentName, componentSlot, tag);\n      }\n      return Component;\n    };\n    if (defaultStyledResolver.withConfig) {\n      muiStyledResolver.withConfig = defaultStyledResolver.withConfig;\n    }\n    return muiStyledResolver;\n  };\n  return styled;\n}\nfunction generateDisplayName(componentName, componentSlot, tag) {\n  if (componentName) {\n    return `${componentName}${(0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(componentSlot || '')}`;\n  }\n  return `Styled(${(0,_mui_utils_getDisplayName__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(tag)})`;\n}\nfunction generateStyledLabel(componentName, componentSlot) {\n  let label;\n  if (true) {\n    if (componentName) {\n      // TODO v6: remove `lowercaseFirstLetter()` in the next major release\n      // For more details: https://github.com/mui/material-ui/pull/37908\n      label = `${componentName}-${lowercaseFirstLetter(componentSlot || 'Root')}`;\n    }\n  }\n  return label;\n}\nfunction isObjectEmpty(object) {\n  // eslint-disable-next-line\n  for (const _ in object) {\n    return false;\n  }\n  return true;\n}\n\n// https://github.com/emotion-js/emotion/blob/26ded6109fcd8ca9875cc2ce4564fee678a3f3c5/packages/styled/src/utils.js#L40\nfunction isStringTag(tag) {\n  return typeof tag === 'string' &&\n  // 96 is one less than the char code\n  // for \"a\" so this is checking that\n  // it's a lowercase character\n  tag.charCodeAt(0) > 96;\n}\nfunction lowercaseFirstLetter(string) {\n  if (!string) {\n    return string;\n  }\n  return string.charAt(0).toLowerCase() + string.slice(1);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createStyled/createStyled.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/applyStyles.js":
/*!********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/applyStyles.js ***!
  \********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ applyStyles)\n/* harmony export */ });\n/**\n * A universal utility to style components with multiple color modes. Always use it from the theme object.\n * It works with:\n *  - [Basic theme](https://mui.com/material-ui/customization/dark-mode/)\n *  - [CSS theme variables](https://mui.com/material-ui/customization/css-theme-variables/overview/)\n *  - Zero-runtime engine\n *\n * Tips: Use an array over object spread and place `theme.applyStyles()` last.\n *\n * With the styled function:\n * ✅ [{ background: '#e5e5e5' }, theme.applyStyles('dark', { background: '#1c1c1c' })]\n * 🚫 { background: '#e5e5e5', ...theme.applyStyles('dark', { background: '#1c1c1c' })}\n *\n * With the sx prop:\n * ✅ [{ background: '#e5e5e5' }, theme => theme.applyStyles('dark', { background: '#1c1c1c' })]\n * 🚫 { background: '#e5e5e5', ...theme => theme.applyStyles('dark', { background: '#1c1c1c' })}\n *\n * @example\n * 1. using with `styled`:\n * ```jsx\n *   const Component = styled('div')(({ theme }) => [\n *     { background: '#e5e5e5' },\n *     theme.applyStyles('dark', {\n *       background: '#1c1c1c',\n *       color: '#fff',\n *     }),\n *   ]);\n * ```\n *\n * @example\n * 2. using with `sx` prop:\n * ```jsx\n *   <Box sx={[\n *     { background: '#e5e5e5' },\n *     theme => theme.applyStyles('dark', {\n *        background: '#1c1c1c',\n *        color: '#fff',\n *      }),\n *     ]}\n *   />\n * ```\n *\n * @example\n * 3. theming a component:\n * ```jsx\n *   extendTheme({\n *     components: {\n *       MuiButton: {\n *         styleOverrides: {\n *           root: ({ theme }) => [\n *             { background: '#e5e5e5' },\n *             theme.applyStyles('dark', {\n *               background: '#1c1c1c',\n *               color: '#fff',\n *             }),\n *           ],\n *         },\n *       }\n *     }\n *   })\n *```\n */\nfunction applyStyles(key, styles) {\n  // @ts-expect-error this is 'any' type\n  const theme = this;\n  if (theme.vars) {\n    if (!theme.colorSchemes?.[key] || typeof theme.getColorSchemeSelector !== 'function') {\n      return {};\n    }\n    // If CssVarsProvider is used as a provider, returns '*:where({selector}) &'\n    let selector = theme.getColorSchemeSelector(key);\n    if (selector === '&') {\n      return styles;\n    }\n    if (selector.includes('data-') || selector.includes('.')) {\n      // '*' is required as a workaround for Emotion issue (https://github.com/emotion-js/emotion/issues/2836)\n      selector = `*:where(${selector.replace(/\\s*&$/, '')}) &`;\n    }\n    return {\n      [selector]: styles\n    };\n  }\n  if (theme.palette.mode === key) {\n    return styles;\n  }\n  return {};\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/applyStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createSpacing.js":
/*!**********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createSpacing.js ***!
  \**********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createSpacing)\n/* harmony export */ });\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js\");\n\n\n// The different signatures imply different meaning for their arguments that can't be expressed structurally.\n// We express the difference with variable names.\n\nfunction createSpacing(spacingInput = 8,\n// Material Design layouts are visually balanced. Most measurements align to an 8dp grid, which aligns both spacing and the overall layout.\n// Smaller components, such as icons, can align to a 4dp grid.\n// https://m2.material.io/design/layout/understanding-layout.html\ntransform = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnarySpacing)({\n  spacing: spacingInput\n})) {\n  // Already transformed.\n  if (spacingInput.mui) {\n    return spacingInput;\n  }\n  const spacing = (...argsInput) => {\n    if (true) {\n      if (!(argsInput.length <= 4)) {\n        console.error(`MUI: Too many arguments provided, expected between 0 and 4, got ${argsInput.length}`);\n      }\n    }\n    const args = argsInput.length === 0 ? [1] : argsInput;\n    return args.map(argument => {\n      const output = transform(argument);\n      return typeof output === 'number' ? `${output}px` : output;\n    }).join(' ');\n  };\n  spacing.mui = true;\n  return spacing;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createSpacing.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js":
/*!********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js ***!
  \********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _createBreakpoints_createBreakpoints_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createBreakpoints/createBreakpoints.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js\");\n/* harmony import */ var _cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../cssContainerQueries/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js\");\n/* harmony import */ var _shape_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./shape.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/shape.js\");\n/* harmony import */ var _createSpacing_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./createSpacing.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createSpacing.js\");\n/* harmony import */ var _styleFunctionSx_styleFunctionSx_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styleFunctionSx/styleFunctionSx.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _styleFunctionSx_defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../styleFunctionSx/defaultSxConfig.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n/* harmony import */ var _applyStyles_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./applyStyles.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/applyStyles.js\");\n\n\n\n\n\n\n\n\nfunction createTheme(options = {}, ...args) {\n  const {\n    breakpoints: breakpointsInput = {},\n    palette: paletteInput = {},\n    spacing: spacingInput,\n    shape: shapeInput = {},\n    ...other\n  } = options;\n  const breakpoints = (0,_createBreakpoints_createBreakpoints_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(breakpointsInput);\n  const spacing = (0,_createSpacing_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(spacingInput);\n  let muiTheme = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n    breakpoints,\n    direction: 'ltr',\n    components: {},\n    // Inject component definitions.\n    palette: {\n      mode: 'light',\n      ...paletteInput\n    },\n    spacing,\n    shape: {\n      ..._shape_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n      ...shapeInput\n    }\n  }, other);\n  muiTheme = (0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(muiTheme);\n  muiTheme.applyStyles = _applyStyles_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"];\n  muiTheme = args.reduce((acc, argument) => (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(acc, argument), muiTheme);\n  muiTheme.unstable_sxConfig = {\n    ..._styleFunctionSx_defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"],\n    ...other?.unstable_sxConfig\n  };\n  muiTheme.unstable_sx = function sx(props) {\n    return (0,_styleFunctionSx_styleFunctionSx_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])({\n      sx: props,\n      theme: this\n    });\n  };\n  return muiTheme;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (createTheme);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/shape.js":
/*!**************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/shape.js ***!
  \**************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst shape = {\n  borderRadius: 4\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (shape);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/shape.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js":
/*!************************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js ***!
  \************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ cssContainerQueries),\n/* harmony export */   getContainerQuery: () => (/* binding */ getContainerQuery),\n/* harmony export */   isCqShorthand: () => (/* binding */ isCqShorthand),\n/* harmony export */   sortContainerQueries: () => (/* binding */ sortContainerQueries)\n/* harmony export */ });\n\n/**\n * For using in `sx` prop to sort the breakpoint from low to high.\n * Note: this function does not work and will not support multiple units.\n *       e.g. input: { '@container (min-width:300px)': '1rem', '@container (min-width:40rem)': '2rem' }\n *            output: { '@container (min-width:40rem)': '2rem', '@container (min-width:300px)': '1rem' } // since 40 < 300 eventhough 40rem > 300px\n */\nfunction sortContainerQueries(theme, css) {\n  if (!theme.containerQueries) {\n    return css;\n  }\n  const sorted = Object.keys(css).filter(key => key.startsWith('@container')).sort((a, b) => {\n    const regex = /min-width:\\s*([0-9.]+)/;\n    return +(a.match(regex)?.[1] || 0) - +(b.match(regex)?.[1] || 0);\n  });\n  if (!sorted.length) {\n    return css;\n  }\n  return sorted.reduce((acc, key) => {\n    const value = css[key];\n    delete acc[key];\n    acc[key] = value;\n    return acc;\n  }, {\n    ...css\n  });\n}\nfunction isCqShorthand(breakpointKeys, value) {\n  return value === '@' || value.startsWith('@') && (breakpointKeys.some(key => value.startsWith(`@${key}`)) || !!value.match(/^@\\d/));\n}\nfunction getContainerQuery(theme, shorthand) {\n  const matches = shorthand.match(/^@([^/]+)?\\/?(.+)?$/);\n  if (!matches) {\n    if (true) {\n      throw new Error( true ? `MUI: The provided shorthand ${`(${shorthand})`} is invalid. The format should be \\`@<breakpoint | number>\\` or \\`@<breakpoint | number>/<container>\\`.\\n` + 'For example, `@sm` or `@600` or `@40rem/sidebar`.' : 0);\n    }\n    // removed by dead control flow\n{}\n  }\n  const [, containerQuery, containerName] = matches;\n  const value = Number.isNaN(+containerQuery) ? containerQuery || 0 : +containerQuery;\n  return theme.containerQueries(containerName).up(value);\n}\nfunction cssContainerQueries(themeInput) {\n  const toContainerQuery = (mediaQuery, name) => mediaQuery.replace('@media', name ? `@container ${name}` : '@container');\n  function attachCq(node, name) {\n    node.up = (...args) => toContainerQuery(themeInput.breakpoints.up(...args), name);\n    node.down = (...args) => toContainerQuery(themeInput.breakpoints.down(...args), name);\n    node.between = (...args) => toContainerQuery(themeInput.breakpoints.between(...args), name);\n    node.only = (...args) => toContainerQuery(themeInput.breakpoints.only(...args), name);\n    node.not = (...args) => {\n      const result = toContainerQuery(themeInput.breakpoints.not(...args), name);\n      if (result.includes('not all and')) {\n        // `@container` does not work with `not all and`, so need to invert the logic\n        return result.replace('not all and ', '').replace('min-width:', 'width<').replace('max-width:', 'width>').replace('and', 'or');\n      }\n      return result;\n    };\n  }\n  const node = {};\n  const containerQueries = name => {\n    attachCq(node, name);\n    return node;\n  };\n  attachCq(containerQueries);\n  return {\n    ...themeInput,\n    containerQueries\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssGrid/cssGrid.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssGrid/cssGrid.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   columnGap: () => (/* binding */ columnGap),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   gap: () => (/* binding */ gap),\n/* harmony export */   gridArea: () => (/* binding */ gridArea),\n/* harmony export */   gridAutoColumns: () => (/* binding */ gridAutoColumns),\n/* harmony export */   gridAutoFlow: () => (/* binding */ gridAutoFlow),\n/* harmony export */   gridAutoRows: () => (/* binding */ gridAutoRows),\n/* harmony export */   gridColumn: () => (/* binding */ gridColumn),\n/* harmony export */   gridRow: () => (/* binding */ gridRow),\n/* harmony export */   gridTemplateAreas: () => (/* binding */ gridTemplateAreas),\n/* harmony export */   gridTemplateColumns: () => (/* binding */ gridTemplateColumns),\n/* harmony export */   gridTemplateRows: () => (/* binding */ gridTemplateRows),\n/* harmony export */   rowGap: () => (/* binding */ rowGap)\n/* harmony export */ });\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js\");\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n\n\n\n\n\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst gap = props => {\n  if (props.gap !== undefined && props.gap !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnaryUnit)(props.theme, 'spacing', 8, 'gap');\n    const styleFromPropValue = propValue => ({\n      gap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.gap, styleFromPropValue);\n  }\n  return null;\n};\ngap.propTypes =  true ? {\n  gap: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n} : 0;\ngap.filterProps = ['gap'];\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst columnGap = props => {\n  if (props.columnGap !== undefined && props.columnGap !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnaryUnit)(props.theme, 'spacing', 8, 'columnGap');\n    const styleFromPropValue = propValue => ({\n      columnGap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.columnGap, styleFromPropValue);\n  }\n  return null;\n};\ncolumnGap.propTypes =  true ? {\n  columnGap: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n} : 0;\ncolumnGap.filterProps = ['columnGap'];\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst rowGap = props => {\n  if (props.rowGap !== undefined && props.rowGap !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnaryUnit)(props.theme, 'spacing', 8, 'rowGap');\n    const styleFromPropValue = propValue => ({\n      rowGap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.rowGap, styleFromPropValue);\n  }\n  return null;\n};\nrowGap.propTypes =  true ? {\n  rowGap: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n} : 0;\nrowGap.filterProps = ['rowGap'];\nconst gridColumn = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridColumn'\n});\nconst gridRow = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridRow'\n});\nconst gridAutoFlow = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridAutoFlow'\n});\nconst gridAutoColumns = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridAutoColumns'\n});\nconst gridAutoRows = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridAutoRows'\n});\nconst gridTemplateColumns = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridTemplateColumns'\n});\nconst gridTemplateRows = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridTemplateRows'\n});\nconst gridTemplateAreas = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridTemplateAreas'\n});\nconst gridArea = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridArea'\n});\nconst grid = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(gap, columnGap, rowGap, gridColumn, gridRow, gridAutoFlow, gridAutoColumns, gridAutoRows, gridTemplateColumns, gridTemplateRows, gridTemplateAreas, gridArea);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (grid);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssGrid/cssGrid.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createCssVarsProvider.js":
/*!**************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createCssVarsProvider.js ***!
  \**************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   DISABLE_CSS_TRANSITION: () => (/* binding */ DISABLE_CSS_TRANSITION),\n/* harmony export */   \"default\": () => (/* binding */ createCssVarsProvider)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/GlobalStyles/GlobalStyles.js\");\n/* harmony import */ var _mui_private_theming__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/private-theming */ \"./node_modules/@mui/material/node_modules/@mui/private-theming/useTheme/useTheme.js\");\n/* harmony import */ var _mui_utils_useEnhancedEffect__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/useEnhancedEffect */ \"./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js\");\n/* harmony import */ var _ThemeProvider_index_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../ThemeProvider/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/ThemeProvider/ThemeProvider.js\");\n/* harmony import */ var _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../InitColorSchemeScript/InitColorSchemeScript.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/InitColorSchemeScript/InitColorSchemeScript.js\");\n/* harmony import */ var _useCurrentColorScheme_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./useCurrentColorScheme.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/useCurrentColorScheme.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\n\nconst DISABLE_CSS_TRANSITION = '*{-webkit-transition:none!important;-moz-transition:none!important;-o-transition:none!important;-ms-transition:none!important;transition:none!important}';\nfunction createCssVarsProvider(options) {\n  const {\n    themeId,\n    /**\n     * This `theme` object needs to follow a certain structure to\n     * be used correctly by the finel `CssVarsProvider`. It should have a\n     * `colorSchemes` key with the light and dark (and any other) palette.\n     * It should also ideally have a vars object created using `prepareCssVars`.\n     */\n    theme: defaultTheme = {},\n    modeStorageKey: defaultModeStorageKey = _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_2__.DEFAULT_MODE_STORAGE_KEY,\n    colorSchemeStorageKey: defaultColorSchemeStorageKey = _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_2__.DEFAULT_COLOR_SCHEME_STORAGE_KEY,\n    disableTransitionOnChange: designSystemTransitionOnChange = false,\n    defaultColorScheme,\n    resolveTheme\n  } = options;\n  const defaultContext = {\n    allColorSchemes: [],\n    colorScheme: undefined,\n    darkColorScheme: undefined,\n    lightColorScheme: undefined,\n    mode: undefined,\n    setColorScheme: () => {},\n    setMode: () => {},\n    systemMode: undefined\n  };\n  const ColorSchemeContext = /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.createContext(undefined);\n  if (true) {\n    ColorSchemeContext.displayName = 'ColorSchemeContext';\n  }\n  const useColorScheme = () => react__WEBPACK_IMPORTED_MODULE_0__.useContext(ColorSchemeContext) || defaultContext;\n  const defaultColorSchemes = {};\n  const defaultComponents = {};\n  function CssVarsProvider(props) {\n    const {\n      children,\n      theme: themeProp,\n      modeStorageKey = defaultModeStorageKey,\n      colorSchemeStorageKey = defaultColorSchemeStorageKey,\n      disableTransitionOnChange = designSystemTransitionOnChange,\n      storageManager,\n      storageWindow = typeof window === 'undefined' ? undefined : window,\n      documentNode = typeof document === 'undefined' ? undefined : document,\n      colorSchemeNode = typeof document === 'undefined' ? undefined : document.documentElement,\n      disableNestedContext = false,\n      disableStyleSheetGeneration = false,\n      defaultMode: initialMode = 'system',\n      noSsr\n    } = props;\n    const hasMounted = react__WEBPACK_IMPORTED_MODULE_0__.useRef(false);\n    const upperTheme = (0,_mui_private_theming__WEBPACK_IMPORTED_MODULE_3__[\"default\"])();\n    const ctx = react__WEBPACK_IMPORTED_MODULE_0__.useContext(ColorSchemeContext);\n    const nested = !!ctx && !disableNestedContext;\n    const initialTheme = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n      if (themeProp) {\n        return themeProp;\n      }\n      return typeof defaultTheme === 'function' ? defaultTheme() : defaultTheme;\n    }, [themeProp]);\n    const scopedTheme = initialTheme[themeId];\n    const restThemeProp = scopedTheme || initialTheme;\n    const {\n      colorSchemes = defaultColorSchemes,\n      components = defaultComponents,\n      cssVarPrefix\n    } = restThemeProp;\n    const joinedColorSchemes = Object.keys(colorSchemes).filter(k => !!colorSchemes[k]).join(',');\n    const allColorSchemes = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => joinedColorSchemes.split(','), [joinedColorSchemes]);\n    const defaultLightColorScheme = typeof defaultColorScheme === 'string' ? defaultColorScheme : defaultColorScheme.light;\n    const defaultDarkColorScheme = typeof defaultColorScheme === 'string' ? defaultColorScheme : defaultColorScheme.dark;\n    const defaultMode = colorSchemes[defaultLightColorScheme] && colorSchemes[defaultDarkColorScheme] ? initialMode : colorSchemes[restThemeProp.defaultColorScheme]?.palette?.mode || restThemeProp.palette?.mode;\n\n    // 1. Get the data about the `mode`, `colorScheme`, and setter functions.\n    const {\n      mode: stateMode,\n      setMode,\n      systemMode,\n      lightColorScheme,\n      darkColorScheme,\n      colorScheme: stateColorScheme,\n      setColorScheme\n    } = (0,_useCurrentColorScheme_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])({\n      supportedColorSchemes: allColorSchemes,\n      defaultLightColorScheme,\n      defaultDarkColorScheme,\n      modeStorageKey,\n      colorSchemeStorageKey,\n      defaultMode,\n      storageManager,\n      storageWindow,\n      noSsr\n    });\n    let mode = stateMode;\n    let colorScheme = stateColorScheme;\n    if (nested) {\n      mode = ctx.mode;\n      colorScheme = ctx.colorScheme;\n    }\n    const memoTheme = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n      // `colorScheme` is undefined on the server and hydration phase\n      const calculatedColorScheme = colorScheme || restThemeProp.defaultColorScheme;\n\n      // 2. get the `vars` object that refers to the CSS custom properties\n      const themeVars = restThemeProp.generateThemeVars?.() || restThemeProp.vars;\n\n      // 3. Start composing the theme object\n      const theme = {\n        ...restThemeProp,\n        components,\n        colorSchemes,\n        cssVarPrefix,\n        vars: themeVars\n      };\n      if (typeof theme.generateSpacing === 'function') {\n        theme.spacing = theme.generateSpacing();\n      }\n\n      // 4. Resolve the color scheme and merge it to the theme\n      if (calculatedColorScheme) {\n        const scheme = colorSchemes[calculatedColorScheme];\n        if (scheme && typeof scheme === 'object') {\n          // 4.1 Merge the selected color scheme to the theme\n          Object.keys(scheme).forEach(schemeKey => {\n            if (scheme[schemeKey] && typeof scheme[schemeKey] === 'object') {\n              // shallow merge the 1st level structure of the theme.\n              theme[schemeKey] = {\n                ...theme[schemeKey],\n                ...scheme[schemeKey]\n              };\n            } else {\n              theme[schemeKey] = scheme[schemeKey];\n            }\n          });\n        }\n      }\n      return resolveTheme ? resolveTheme(theme) : theme;\n    }, [restThemeProp, colorScheme, components, colorSchemes, cssVarPrefix]);\n\n    // 5. Declaring effects\n    // 5.1 Updates the selector value to use the current color scheme which tells CSS to use the proper stylesheet.\n    const colorSchemeSelector = restThemeProp.colorSchemeSelector;\n    (0,_mui_utils_useEnhancedEffect__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(() => {\n      if (colorScheme && colorSchemeNode && colorSchemeSelector && colorSchemeSelector !== 'media') {\n        const selector = colorSchemeSelector;\n        let rule = colorSchemeSelector;\n        if (selector === 'class') {\n          rule = `.%s`;\n        }\n        if (selector === 'data') {\n          rule = `[data-%s]`;\n        }\n        if (selector?.startsWith('data-') && !selector.includes('%s')) {\n          // 'data-mui-color-scheme' -> '[data-mui-color-scheme=\"%s\"]'\n          rule = `[${selector}=\"%s\"]`;\n        }\n        if (rule.startsWith('.')) {\n          colorSchemeNode.classList.remove(...allColorSchemes.map(scheme => rule.substring(1).replace('%s', scheme)));\n          colorSchemeNode.classList.add(rule.substring(1).replace('%s', colorScheme));\n        } else {\n          const matches = rule.replace('%s', colorScheme).match(/\\[([^\\]]+)\\]/);\n          if (matches) {\n            const [attr, value] = matches[1].split('=');\n            if (!value) {\n              // for attributes like `data-theme-dark`, `data-theme-light`\n              // remove all the existing data attributes before setting the new one\n              allColorSchemes.forEach(scheme => {\n                colorSchemeNode.removeAttribute(attr.replace(colorScheme, scheme));\n              });\n            }\n            colorSchemeNode.setAttribute(attr, value ? value.replace(/\"|'/g, '') : '');\n          } else {\n            colorSchemeNode.setAttribute(rule, colorScheme);\n          }\n        }\n      }\n    }, [colorScheme, colorSchemeSelector, colorSchemeNode, allColorSchemes]);\n\n    // 5.2 Remove the CSS transition when color scheme changes to create instant experience.\n    // credit: https://github.com/pacocoursey/next-themes/blob/b5c2bad50de2d61ad7b52a9c5cdc801a78507d7a/index.tsx#L313\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n      let timer;\n      if (disableTransitionOnChange && hasMounted.current && documentNode) {\n        const css = documentNode.createElement('style');\n        css.appendChild(documentNode.createTextNode(DISABLE_CSS_TRANSITION));\n        documentNode.head.appendChild(css);\n\n        // Force browser repaint\n        (() => window.getComputedStyle(documentNode.body))();\n        timer = setTimeout(() => {\n          documentNode.head.removeChild(css);\n        }, 1);\n      }\n      return () => {\n        clearTimeout(timer);\n      };\n    }, [colorScheme, disableTransitionOnChange, documentNode]);\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n      hasMounted.current = true;\n      return () => {\n        hasMounted.current = false;\n      };\n    }, []);\n    const contextValue = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => ({\n      allColorSchemes,\n      colorScheme,\n      darkColorScheme,\n      lightColorScheme,\n      mode,\n      setColorScheme,\n      setMode:  false ? 0 : newMode => {\n        if (memoTheme.colorSchemeSelector === 'media') {\n          console.error(['MUI: The `setMode` function has no effect if `colorSchemeSelector` is `media` (`media` is the default value).', 'To toggle the mode manually, please configure `colorSchemeSelector` to use a class or data attribute.', 'To learn more, visit https://mui.com/material-ui/customization/css-theme-variables/configuration/#toggling-dark-mode-manually'].join('\\n'));\n        }\n        setMode(newMode);\n      },\n      systemMode\n    }), [allColorSchemes, colorScheme, darkColorScheme, lightColorScheme, mode, setColorScheme, setMode, systemMode, memoTheme.colorSchemeSelector]);\n    let shouldGenerateStyleSheet = true;\n    if (disableStyleSheetGeneration || restThemeProp.cssVariables === false || nested && upperTheme?.cssVarPrefix === cssVarPrefix) {\n      shouldGenerateStyleSheet = false;\n    }\n    const element = /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsxs)(react__WEBPACK_IMPORTED_MODULE_0__.Fragment, {\n      children: [/*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_ThemeProvider_index_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"], {\n        themeId: scopedTheme ? themeId : undefined,\n        theme: memoTheme,\n        children: children\n      }), shouldGenerateStyleSheet && /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_styled_engine__WEBPACK_IMPORTED_MODULE_7__[\"default\"], {\n        styles: memoTheme.generateStyleSheets?.() || []\n      })]\n    });\n    if (nested) {\n      return element;\n    }\n    return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(ColorSchemeContext.Provider, {\n      value: contextValue,\n      children: element\n    });\n  }\n   true ? CssVarsProvider.propTypes = {\n    /**\n     * The component tree.\n     */\n    children: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().node),\n    /**\n     * The node used to attach the color-scheme attribute\n     */\n    colorSchemeNode: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().any),\n    /**\n     * localStorage key used to store `colorScheme`\n     */\n    colorSchemeStorageKey: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n    /**\n     * The default mode when the storage is empty,\n     * require the theme to have `colorSchemes` with light and dark.\n     */\n    defaultMode: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n    /**\n     * If `true`, the provider creates its own context and generate stylesheet as if it is a root `CssVarsProvider`.\n     */\n    disableNestedContext: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n    /**\n     * If `true`, the style sheet won't be generated.\n     *\n     * This is useful for controlling nested CssVarsProvider behavior.\n     */\n    disableStyleSheetGeneration: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n    /**\n     * Disable CSS transitions when switching between modes or color schemes.\n     */\n    disableTransitionOnChange: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n    /**\n     * The document to attach the attribute to.\n     */\n    documentNode: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().any),\n    /**\n     * The key in the local storage used to store current color scheme.\n     */\n    modeStorageKey: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().string),\n    /**\n     * If `true`, the mode will be the same value as the storage without an extra rerendering after the hydration.\n     * You should use this option in conjuction with `InitColorSchemeScript` component.\n     */\n    noSsr: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().bool),\n    /**\n     * The storage manager to be used for storing the mode and color scheme\n     * @default using `window.localStorage`\n     */\n    storageManager: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().func),\n    /**\n     * The window that attaches the 'storage' event listener.\n     * @default window\n     */\n    storageWindow: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().any),\n    /**\n     * The calculated theme object that will be passed through context.\n     */\n    theme: (prop_types__WEBPACK_IMPORTED_MODULE_8___default().object)\n  } : 0;\n  const defaultLightColorScheme = typeof defaultColorScheme === 'string' ? defaultColorScheme : defaultColorScheme.light;\n  const defaultDarkColorScheme = typeof defaultColorScheme === 'string' ? defaultColorScheme : defaultColorScheme.dark;\n  const getInitColorSchemeScript = params => (0,_InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n    colorSchemeStorageKey: defaultColorSchemeStorageKey,\n    defaultLightColorScheme,\n    defaultDarkColorScheme,\n    modeStorageKey: defaultModeStorageKey,\n    ...params\n  });\n  return {\n    CssVarsProvider,\n    useColorScheme,\n    getInitColorSchemeScript\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createCssVarsProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createGetCssVar.js":
/*!********************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createGetCssVar.js ***!
  \********************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createGetCssVar)\n/* harmony export */ });\n/**\n * The benefit of this function is to help developers get CSS var from theme without specifying the whole variable\n * and they does not need to remember the prefix (defined once).\n */\nfunction createGetCssVar(prefix = '') {\n  function appendVar(...vars) {\n    if (!vars.length) {\n      return '';\n    }\n    const value = vars[0];\n    if (typeof value === 'string' && !value.match(/(#|\\(|\\)|(-?(\\d*\\.)?\\d+)(px|em|%|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc))|^(-?(\\d*\\.)?\\d+)$|(\\d+ \\d+ \\d+)/)) {\n      return `, var(--${prefix ? `${prefix}-` : ''}${value}${appendVar(...vars.slice(1))})`;\n    }\n    return `, ${value}`;\n  }\n\n  // AdditionalVars makes `getCssVar` less strict, so it can be use like this `getCssVar('non-mui-variable')` without type error.\n  const getCssVar = (field, ...fallbacks) => {\n    return `var(--${prefix ? `${prefix}-` : ''}${field}${appendVar(...fallbacks)})`;\n  };\n  return getCssVar;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createGetCssVar.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/cssVarsParser.js":
/*!******************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/cssVarsParser.js ***!
  \******************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   assignNestedKeys: () => (/* binding */ assignNestedKeys),\n/* harmony export */   \"default\": () => (/* binding */ cssVarsParser),\n/* harmony export */   walkObjectDeep: () => (/* binding */ walkObjectDeep)\n/* harmony export */ });\n/**\n * This function create an object from keys, value and then assign to target\n *\n * @param {Object} obj : the target object to be assigned\n * @param {string[]} keys\n * @param {string | number} value\n *\n * @example\n * const source = {}\n * assignNestedKeys(source, ['palette', 'primary'], 'var(--palette-primary)')\n * console.log(source) // { palette: { primary: 'var(--palette-primary)' } }\n *\n * @example\n * const source = { palette: { primary: 'var(--palette-primary)' } }\n * assignNestedKeys(source, ['palette', 'secondary'], 'var(--palette-secondary)')\n * console.log(source) // { palette: { primary: 'var(--palette-primary)', secondary: 'var(--palette-secondary)' } }\n */\nconst assignNestedKeys = (obj, keys, value, arrayKeys = []) => {\n  let temp = obj;\n  keys.forEach((k, index) => {\n    if (index === keys.length - 1) {\n      if (Array.isArray(temp)) {\n        temp[Number(k)] = value;\n      } else if (temp && typeof temp === 'object') {\n        temp[k] = value;\n      }\n    } else if (temp && typeof temp === 'object') {\n      if (!temp[k]) {\n        temp[k] = arrayKeys.includes(k) ? [] : {};\n      }\n      temp = temp[k];\n    }\n  });\n};\n\n/**\n *\n * @param {Object} obj : source object\n * @param {Function} callback : a function that will be called when\n *                   - the deepest key in source object is reached\n *                   - the value of the deepest key is NOT `undefined` | `null`\n *\n * @example\n * walkObjectDeep({ palette: { primary: { main: '#000000' } } }, console.log)\n * // ['palette', 'primary', 'main'] '#000000'\n */\nconst walkObjectDeep = (obj, callback, shouldSkipPaths) => {\n  function recurse(object, parentKeys = [], arrayKeys = []) {\n    Object.entries(object).forEach(([key, value]) => {\n      if (!shouldSkipPaths || shouldSkipPaths && !shouldSkipPaths([...parentKeys, key])) {\n        if (value !== undefined && value !== null) {\n          if (typeof value === 'object' && Object.keys(value).length > 0) {\n            recurse(value, [...parentKeys, key], Array.isArray(value) ? [...arrayKeys, key] : arrayKeys);\n          } else {\n            callback([...parentKeys, key], value, arrayKeys);\n          }\n        }\n      }\n    });\n  }\n  recurse(obj);\n};\nconst getCssValue = (keys, value) => {\n  if (typeof value === 'number') {\n    if (['lineHeight', 'fontWeight', 'opacity', 'zIndex'].some(prop => keys.includes(prop))) {\n      // CSS property that are unitless\n      return value;\n    }\n    const lastKey = keys[keys.length - 1];\n    if (lastKey.toLowerCase().includes('opacity')) {\n      // opacity values are unitless\n      return value;\n    }\n    return `${value}px`;\n  }\n  return value;\n};\n\n/**\n * a function that parse theme and return { css, vars }\n *\n * @param {Object} theme\n * @param {{\n *  prefix?: string,\n *  shouldSkipGeneratingVar?: (objectPathKeys: Array<string>, value: string | number) => boolean\n * }} options.\n *  `prefix`: The prefix of the generated CSS variables. This function does not change the value.\n *\n * @returns {{ css: Object, vars: Object }} `css` is the stylesheet, `vars` is an object to get css variable (same structure as theme).\n *\n * @example\n * const { css, vars } = parser({\n *   fontSize: 12,\n *   lineHeight: 1.2,\n *   palette: { primary: { 500: 'var(--color)' } }\n * }, { prefix: 'foo' })\n *\n * console.log(css) // { '--foo-fontSize': '12px', '--foo-lineHeight': 1.2, '--foo-palette-primary-500': 'var(--color)' }\n * console.log(vars) // { fontSize: 'var(--foo-fontSize)', lineHeight: 'var(--foo-lineHeight)', palette: { primary: { 500: 'var(--foo-palette-primary-500)' } } }\n */\nfunction cssVarsParser(theme, options) {\n  const {\n    prefix,\n    shouldSkipGeneratingVar\n  } = options || {};\n  const css = {};\n  const vars = {};\n  const varsWithDefaults = {};\n  walkObjectDeep(theme, (keys, value, arrayKeys) => {\n    if (typeof value === 'string' || typeof value === 'number') {\n      if (!shouldSkipGeneratingVar || !shouldSkipGeneratingVar(keys, value)) {\n        // only create css & var if `shouldSkipGeneratingVar` return false\n        const cssVar = `--${prefix ? `${prefix}-` : ''}${keys.join('-')}`;\n        const resolvedValue = getCssValue(keys, value);\n        Object.assign(css, {\n          [cssVar]: resolvedValue\n        });\n        assignNestedKeys(vars, keys, `var(${cssVar})`, arrayKeys);\n        assignNestedKeys(varsWithDefaults, keys, `var(${cssVar}, ${resolvedValue})`, arrayKeys);\n      }\n    }\n  }, keys => keys[0] === 'vars' // skip 'vars/*' paths\n  );\n  return {\n    css,\n    vars,\n    varsWithDefaults\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/cssVarsParser.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/getColorSchemeSelector.js":
/*!***************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/getColorSchemeSelector.js ***!
  \***************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   createGetColorSchemeSelector: () => (/* binding */ createGetColorSchemeSelector)\n/* harmony export */ });\n/* eslint-disable import/prefer-default-export */\nfunction createGetColorSchemeSelector(selector) {\n  return function getColorSchemeSelector(colorScheme) {\n    if (selector === 'media') {\n      if (true) {\n        if (colorScheme !== 'light' && colorScheme !== 'dark') {\n          console.error(`MUI: @media (prefers-color-scheme) supports only 'light' or 'dark', but receive '${colorScheme}'.`);\n        }\n      }\n      return `@media (prefers-color-scheme: ${colorScheme})`;\n    }\n    if (selector) {\n      if (selector.startsWith('data-') && !selector.includes('%s')) {\n        return `[${selector}=\"${colorScheme}\"] &`;\n      }\n      if (selector === 'class') {\n        return `.${colorScheme} &`;\n      }\n      if (selector === 'data') {\n        return `[data-${colorScheme}] &`;\n      }\n      return `${selector.replace('%s', colorScheme)} &`;\n    }\n    return '&';\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/getColorSchemeSelector.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/localStorageManager.js":
/*!************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/localStorageManager.js ***!
  \************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nfunction noop() {}\nconst localStorageManager = ({\n  key,\n  storageWindow\n}) => {\n  if (!storageWindow && typeof window !== 'undefined') {\n    storageWindow = window;\n  }\n  return {\n    get(defaultValue) {\n      if (typeof window === 'undefined') {\n        return undefined;\n      }\n      if (!storageWindow) {\n        return defaultValue;\n      }\n      let value;\n      try {\n        value = storageWindow.localStorage.getItem(key);\n      } catch {\n        // Unsupported\n      }\n      return value || defaultValue;\n    },\n    set: value => {\n      if (storageWindow) {\n        try {\n          storageWindow.localStorage.setItem(key, value);\n        } catch {\n          // Unsupported\n        }\n      }\n    },\n    subscribe: handler => {\n      if (!storageWindow) {\n        return noop;\n      }\n      const listener = event => {\n        const value = event.newValue;\n        if (event.key === key) {\n          handler(value);\n        }\n      };\n      storageWindow.addEventListener('storage', listener);\n      return () => {\n        storageWindow.removeEventListener('storage', listener);\n      };\n    }\n  };\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (localStorageManager);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/localStorageManager.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareCssVars.js":
/*!*******************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareCssVars.js ***!
  \*******************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _cssVarsParser_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./cssVarsParser.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/cssVarsParser.js\");\n\n\nfunction prepareCssVars(theme, parserConfig = {}) {\n  const {\n    getSelector = defaultGetSelector,\n    disableCssColorScheme,\n    colorSchemeSelector: selector\n  } = parserConfig;\n  // @ts-ignore - ignore components do not exist\n  const {\n    colorSchemes = {},\n    components,\n    defaultColorScheme = 'light',\n    ...otherTheme\n  } = theme;\n  const {\n    vars: rootVars,\n    css: rootCss,\n    varsWithDefaults: rootVarsWithDefaults\n  } = (0,_cssVarsParser_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(otherTheme, parserConfig);\n  let themeVars = rootVarsWithDefaults;\n  const colorSchemesMap = {};\n  const {\n    [defaultColorScheme]: defaultScheme,\n    ...otherColorSchemes\n  } = colorSchemes;\n  Object.entries(otherColorSchemes || {}).forEach(([key, scheme]) => {\n    const {\n      vars,\n      css,\n      varsWithDefaults\n    } = (0,_cssVarsParser_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(scheme, parserConfig);\n    themeVars = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(themeVars, varsWithDefaults);\n    colorSchemesMap[key] = {\n      css,\n      vars\n    };\n  });\n  if (defaultScheme) {\n    // default color scheme vars should be merged last to set as default\n    const {\n      css,\n      vars,\n      varsWithDefaults\n    } = (0,_cssVarsParser_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(defaultScheme, parserConfig);\n    themeVars = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(themeVars, varsWithDefaults);\n    colorSchemesMap[defaultColorScheme] = {\n      css,\n      vars\n    };\n  }\n  function defaultGetSelector(colorScheme, cssObject) {\n    let rule = selector;\n    if (selector === 'class') {\n      rule = '.%s';\n    }\n    if (selector === 'data') {\n      rule = '[data-%s]';\n    }\n    if (selector?.startsWith('data-') && !selector.includes('%s')) {\n      // 'data-joy-color-scheme' -> '[data-joy-color-scheme=\"%s\"]'\n      rule = `[${selector}=\"%s\"]`;\n    }\n    if (colorScheme) {\n      if (rule === 'media') {\n        if (theme.defaultColorScheme === colorScheme) {\n          return ':root';\n        }\n        const mode = colorSchemes[colorScheme]?.palette?.mode || colorScheme;\n        return {\n          [`@media (prefers-color-scheme: ${mode})`]: {\n            ':root': cssObject\n          }\n        };\n      }\n      if (rule) {\n        if (theme.defaultColorScheme === colorScheme) {\n          return `:root, ${rule.replace('%s', String(colorScheme))}`;\n        }\n        return rule.replace('%s', String(colorScheme));\n      }\n    }\n    return ':root';\n  }\n  const generateThemeVars = () => {\n    let vars = {\n      ...rootVars\n    };\n    Object.entries(colorSchemesMap).forEach(([, {\n      vars: schemeVars\n    }]) => {\n      vars = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(vars, schemeVars);\n    });\n    return vars;\n  };\n  const generateStyleSheets = () => {\n    const stylesheets = [];\n    const colorScheme = theme.defaultColorScheme || 'light';\n    function insertStyleSheet(key, css) {\n      if (Object.keys(css).length) {\n        stylesheets.push(typeof key === 'string' ? {\n          [key]: {\n            ...css\n          }\n        } : key);\n      }\n    }\n    insertStyleSheet(getSelector(undefined, {\n      ...rootCss\n    }), rootCss);\n    const {\n      [colorScheme]: defaultSchemeVal,\n      ...other\n    } = colorSchemesMap;\n    if (defaultSchemeVal) {\n      // default color scheme has to come before other color schemes\n      const {\n        css\n      } = defaultSchemeVal;\n      const cssColorSheme = colorSchemes[colorScheme]?.palette?.mode;\n      const finalCss = !disableCssColorScheme && cssColorSheme ? {\n        colorScheme: cssColorSheme,\n        ...css\n      } : {\n        ...css\n      };\n      insertStyleSheet(getSelector(colorScheme, {\n        ...finalCss\n      }), finalCss);\n    }\n    Object.entries(other).forEach(([key, {\n      css\n    }]) => {\n      const cssColorSheme = colorSchemes[key]?.palette?.mode;\n      const finalCss = !disableCssColorScheme && cssColorSheme ? {\n        colorScheme: cssColorSheme,\n        ...css\n      } : {\n        ...css\n      };\n      insertStyleSheet(getSelector(key, {\n        ...finalCss\n      }), finalCss);\n    });\n    return stylesheets;\n  };\n  return {\n    vars: themeVars,\n    generateThemeVars,\n    generateStyleSheets\n  };\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (prepareCssVars);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareCssVars.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareTypographyVars.js":
/*!**************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareTypographyVars.js ***!
  \**************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ prepareTypographyVars)\n/* harmony export */ });\nfunction prepareTypographyVars(typography) {\n  const vars = {};\n  const entries = Object.entries(typography);\n  entries.forEach(entry => {\n    const [key, value] = entry;\n    if (typeof value === 'object') {\n      vars[key] = `${value.fontStyle ? `${value.fontStyle} ` : ''}${value.fontVariant ? `${value.fontVariant} ` : ''}${value.fontWeight ? `${value.fontWeight} ` : ''}${value.fontStretch ? `${value.fontStretch} ` : ''}${value.fontSize || ''}${value.lineHeight ? `/${value.lineHeight} ` : ''}${value.fontFamily || ''}`;\n    }\n  });\n  return vars;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareTypographyVars.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/useCurrentColorScheme.js":
/*!**************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/useCurrentColorScheme.js ***!
  \**************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useCurrentColorScheme),\n/* harmony export */   getColorScheme: () => (/* binding */ getColorScheme),\n/* harmony export */   getSystemMode: () => (/* binding */ getSystemMode)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../InitColorSchemeScript/InitColorSchemeScript.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/InitColorSchemeScript/InitColorSchemeScript.js\");\n/* harmony import */ var _localStorageManager_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./localStorageManager.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/localStorageManager.js\");\n'use client';\n\n\n\n\nfunction noop() {}\nfunction getSystemMode(mode) {\n  if (typeof window !== 'undefined' && typeof window.matchMedia === 'function' && mode === 'system') {\n    const mql = window.matchMedia('(prefers-color-scheme: dark)');\n    if (mql.matches) {\n      return 'dark';\n    }\n    return 'light';\n  }\n  return undefined;\n}\nfunction processState(state, callback) {\n  if (state.mode === 'light' || state.mode === 'system' && state.systemMode === 'light') {\n    return callback('light');\n  }\n  if (state.mode === 'dark' || state.mode === 'system' && state.systemMode === 'dark') {\n    return callback('dark');\n  }\n  return undefined;\n}\nfunction getColorScheme(state) {\n  return processState(state, mode => {\n    if (mode === 'light') {\n      return state.lightColorScheme;\n    }\n    if (mode === 'dark') {\n      return state.darkColorScheme;\n    }\n    return undefined;\n  });\n}\nfunction useCurrentColorScheme(options) {\n  const {\n    defaultMode = 'light',\n    defaultLightColorScheme,\n    defaultDarkColorScheme,\n    supportedColorSchemes = [],\n    modeStorageKey = _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_1__.DEFAULT_MODE_STORAGE_KEY,\n    colorSchemeStorageKey = _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_1__.DEFAULT_COLOR_SCHEME_STORAGE_KEY,\n    storageWindow = typeof window === 'undefined' ? undefined : window,\n    storageManager = _localStorageManager_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"],\n    noSsr = false\n  } = options;\n  const joinedColorSchemes = supportedColorSchemes.join(',');\n  const isMultiSchemes = supportedColorSchemes.length > 1;\n  const modeStorage = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => storageManager?.({\n    key: modeStorageKey,\n    storageWindow\n  }), [storageManager, modeStorageKey, storageWindow]);\n  const lightStorage = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => storageManager?.({\n    key: `${colorSchemeStorageKey}-light`,\n    storageWindow\n  }), [storageManager, colorSchemeStorageKey, storageWindow]);\n  const darkStorage = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => storageManager?.({\n    key: `${colorSchemeStorageKey}-dark`,\n    storageWindow\n  }), [storageManager, colorSchemeStorageKey, storageWindow]);\n  const [state, setState] = react__WEBPACK_IMPORTED_MODULE_0__.useState(() => {\n    const initialMode = modeStorage?.get(defaultMode) || defaultMode;\n    const lightColorScheme = lightStorage?.get(defaultLightColorScheme) || defaultLightColorScheme;\n    const darkColorScheme = darkStorage?.get(defaultDarkColorScheme) || defaultDarkColorScheme;\n    return {\n      mode: initialMode,\n      systemMode: getSystemMode(initialMode),\n      lightColorScheme,\n      darkColorScheme\n    };\n  });\n  const [isClient, setIsClient] = react__WEBPACK_IMPORTED_MODULE_0__.useState(noSsr || !isMultiSchemes);\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    setIsClient(true); // to rerender the component after hydration\n  }, []);\n  const colorScheme = getColorScheme(state);\n  const setMode = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(mode => {\n    setState(currentState => {\n      if (mode === currentState.mode) {\n        // do nothing if mode does not change\n        return currentState;\n      }\n      const newMode = mode ?? defaultMode;\n      modeStorage?.set(newMode);\n      return {\n        ...currentState,\n        mode: newMode,\n        systemMode: getSystemMode(newMode)\n      };\n    });\n  }, [modeStorage, defaultMode]);\n  const setColorScheme = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(value => {\n    if (!value) {\n      setState(currentState => {\n        lightStorage?.set(defaultLightColorScheme);\n        darkStorage?.set(defaultDarkColorScheme);\n        return {\n          ...currentState,\n          lightColorScheme: defaultLightColorScheme,\n          darkColorScheme: defaultDarkColorScheme\n        };\n      });\n    } else if (typeof value === 'string') {\n      if (value && !joinedColorSchemes.includes(value)) {\n        console.error(`\\`${value}\\` does not exist in \\`theme.colorSchemes\\`.`);\n      } else {\n        setState(currentState => {\n          const newState = {\n            ...currentState\n          };\n          processState(currentState, mode => {\n            if (mode === 'light') {\n              lightStorage?.set(value);\n              newState.lightColorScheme = value;\n            }\n            if (mode === 'dark') {\n              darkStorage?.set(value);\n              newState.darkColorScheme = value;\n            }\n          });\n          return newState;\n        });\n      }\n    } else {\n      setState(currentState => {\n        const newState = {\n          ...currentState\n        };\n        const newLightColorScheme = value.light === null ? defaultLightColorScheme : value.light;\n        const newDarkColorScheme = value.dark === null ? defaultDarkColorScheme : value.dark;\n        if (newLightColorScheme) {\n          if (!joinedColorSchemes.includes(newLightColorScheme)) {\n            console.error(`\\`${newLightColorScheme}\\` does not exist in \\`theme.colorSchemes\\`.`);\n          } else {\n            newState.lightColorScheme = newLightColorScheme;\n            lightStorage?.set(newLightColorScheme);\n          }\n        }\n        if (newDarkColorScheme) {\n          if (!joinedColorSchemes.includes(newDarkColorScheme)) {\n            console.error(`\\`${newDarkColorScheme}\\` does not exist in \\`theme.colorSchemes\\`.`);\n          } else {\n            newState.darkColorScheme = newDarkColorScheme;\n            darkStorage?.set(newDarkColorScheme);\n          }\n        }\n        return newState;\n      });\n    }\n  }, [joinedColorSchemes, lightStorage, darkStorage, defaultLightColorScheme, defaultDarkColorScheme]);\n  const handleMediaQuery = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(event => {\n    if (state.mode === 'system') {\n      setState(currentState => {\n        const systemMode = event?.matches ? 'dark' : 'light';\n\n        // Early exit, nothing changed.\n        if (currentState.systemMode === systemMode) {\n          return currentState;\n        }\n        return {\n          ...currentState,\n          systemMode\n        };\n      });\n    }\n  }, [state.mode]);\n\n  // Ref hack to avoid adding handleMediaQuery as a dep\n  const mediaListener = react__WEBPACK_IMPORTED_MODULE_0__.useRef(handleMediaQuery);\n  mediaListener.current = handleMediaQuery;\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (typeof window.matchMedia !== 'function' || !isMultiSchemes) {\n      return undefined;\n    }\n    const handler = (...args) => mediaListener.current(...args);\n\n    // Always listen to System preference\n    const media = window.matchMedia('(prefers-color-scheme: dark)');\n\n    // Intentionally use deprecated listener methods to support iOS & old browsers\n    media.addListener(handler);\n    handler(media);\n    return () => {\n      media.removeListener(handler);\n    };\n  }, [isMultiSchemes]);\n\n  // Handle when localStorage has changed\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (isMultiSchemes) {\n      const unsubscribeMode = modeStorage?.subscribe(value => {\n        if (!value || ['light', 'dark', 'system'].includes(value)) {\n          setMode(value || defaultMode);\n        }\n      }) || noop;\n      const unsubscribeLight = lightStorage?.subscribe(value => {\n        if (!value || joinedColorSchemes.match(value)) {\n          setColorScheme({\n            light: value\n          });\n        }\n      }) || noop;\n      const unsubscribeDark = darkStorage?.subscribe(value => {\n        if (!value || joinedColorSchemes.match(value)) {\n          setColorScheme({\n            dark: value\n          });\n        }\n      }) || noop;\n      return () => {\n        unsubscribeMode();\n        unsubscribeLight();\n        unsubscribeDark();\n      };\n    }\n    return undefined;\n  }, [setColorScheme, setMode, joinedColorSchemes, defaultMode, storageWindow, isMultiSchemes, modeStorage, lightStorage, darkStorage]);\n  return {\n    ...state,\n    mode: isClient ? state.mode : undefined,\n    systemMode: isClient ? state.systemMode : undefined,\n    colorScheme: isClient ? colorScheme : undefined,\n    setMode,\n    setColorScheme\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/useCurrentColorScheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/memoTheme.js":
/*!******************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/memoTheme.js ***!
  \******************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ unstable_memoTheme)\n/* harmony export */ });\n/* harmony import */ var _preprocessStyles_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./preprocessStyles.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/preprocessStyles.js\");\n\n\n/* eslint-disable @typescript-eslint/naming-convention */\n\n// We need to pass an argument as `{ theme }` for PigmentCSS, but we don't want to\n// allocate more objects.\nconst arg = {\n  theme: undefined\n};\n\n/**\n * Memoize style function on theme.\n * Intended to be used in styled() calls that only need access to the theme.\n */\nfunction unstable_memoTheme(styleFn) {\n  let lastValue;\n  let lastTheme;\n  return function styleMemoized(props) {\n    let value = lastValue;\n    if (value === undefined || props.theme !== lastTheme) {\n      arg.theme = props.theme;\n      value = (0,_preprocessStyles_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(styleFn(arg));\n      lastValue = value;\n      lastTheme = props.theme;\n    }\n    return value;\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/memoTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/memoize/memoize.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/memoize/memoize.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ memoize)\n/* harmony export */ });\nfunction memoize(fn) {\n  const cache = {};\n  return arg => {\n    if (cache[arg] === undefined) {\n      cache[arg] = fn(arg);\n    }\n    return cache[arg];\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/memoize/memoize.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js":
/*!********************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js ***!
  \********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n\nfunction merge(acc, item) {\n  if (!item) {\n    return acc;\n  }\n  return (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(acc, item, {\n    clone: false // No need to clone deep, it's way faster.\n  });\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (merge);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/palette/palette.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/palette/palette.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   backgroundColor: () => (/* binding */ backgroundColor),\n/* harmony export */   bgcolor: () => (/* binding */ bgcolor),\n/* harmony export */   color: () => (/* binding */ color),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   paletteTransform: () => (/* binding */ paletteTransform)\n/* harmony export */ });\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js\");\n\n\nfunction paletteTransform(value, userValue) {\n  if (userValue === 'grey') {\n    return userValue;\n  }\n  return value;\n}\nconst color = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'color',\n  themeKey: 'palette',\n  transform: paletteTransform\n});\nconst bgcolor = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'bgcolor',\n  cssProperty: 'backgroundColor',\n  themeKey: 'palette',\n  transform: paletteTransform\n});\nconst backgroundColor = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'backgroundColor',\n  themeKey: 'palette',\n  transform: paletteTransform\n});\nconst palette = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(color, bgcolor, backgroundColor);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (palette);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/palette/palette.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/preprocessStyles.js":
/*!*************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/preprocessStyles.js ***!
  \*************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ preprocessStyles)\n/* harmony export */ });\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/material/node_modules/@mui/styled-engine/index.js\");\n\nfunction preprocessStyles(input) {\n  const {\n    variants,\n    ...style\n  } = input;\n  const result = {\n    variants,\n    style: (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_0__.internal_serializeStyles)(style),\n    isProcessed: true\n  };\n\n  // Not supported on styled-components\n  if (result.style === style) {\n    return result;\n  }\n  if (variants) {\n    variants.forEach(variant => {\n      if (typeof variant.style !== 'function') {\n        variant.style = (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_0__.internal_serializeStyles)(variant.style);\n      }\n    });\n  }\n  return result;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/preprocessStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js":
/*!**********************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js ***!
  \**********************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_0__);\n\nconst responsivePropType =  true ? prop_types__WEBPACK_IMPORTED_MODULE_0___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_0___default().number), (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string), (prop_types__WEBPACK_IMPORTED_MODULE_0___default().object), (prop_types__WEBPACK_IMPORTED_MODULE_0___default().array)]) : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (responsivePropType);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/sizing/sizing.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/sizing/sizing.js ***!
  \**********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   boxSizing: () => (/* binding */ boxSizing),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   height: () => (/* binding */ height),\n/* harmony export */   maxHeight: () => (/* binding */ maxHeight),\n/* harmony export */   maxWidth: () => (/* binding */ maxWidth),\n/* harmony export */   minHeight: () => (/* binding */ minHeight),\n/* harmony export */   minWidth: () => (/* binding */ minWidth),\n/* harmony export */   sizeHeight: () => (/* binding */ sizeHeight),\n/* harmony export */   sizeWidth: () => (/* binding */ sizeWidth),\n/* harmony export */   sizingTransform: () => (/* binding */ sizingTransform),\n/* harmony export */   width: () => (/* binding */ width)\n/* harmony export */ });\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/compose/compose.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n\n\n\nfunction sizingTransform(value) {\n  return value <= 1 && value !== 0 ? `${value * 100}%` : value;\n}\nconst width = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'width',\n  transform: sizingTransform\n});\nconst maxWidth = props => {\n  if (props.maxWidth !== undefined && props.maxWidth !== null) {\n    const styleFromPropValue = propValue => {\n      const breakpoint = props.theme?.breakpoints?.values?.[propValue] || _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.values[propValue];\n      if (!breakpoint) {\n        return {\n          maxWidth: sizingTransform(propValue)\n        };\n      }\n      if (props.theme?.breakpoints?.unit !== 'px') {\n        return {\n          maxWidth: `${breakpoint}${props.theme.breakpoints.unit}`\n        };\n      }\n      return {\n        maxWidth: breakpoint\n      };\n    };\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.maxWidth, styleFromPropValue);\n  }\n  return null;\n};\nmaxWidth.filterProps = ['maxWidth'];\nconst minWidth = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'minWidth',\n  transform: sizingTransform\n});\nconst height = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'height',\n  transform: sizingTransform\n});\nconst maxHeight = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'maxHeight',\n  transform: sizingTransform\n});\nconst minHeight = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'minHeight',\n  transform: sizingTransform\n});\nconst sizeWidth = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'size',\n  cssProperty: 'width',\n  transform: sizingTransform\n});\nconst sizeHeight = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'size',\n  cssProperty: 'height',\n  transform: sizingTransform\n});\nconst boxSizing = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'boxSizing'\n});\nconst sizing = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(width, maxWidth, minWidth, height, maxHeight, minHeight, boxSizing);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (sizing);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/sizing/sizing.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   createUnarySpacing: () => (/* binding */ createUnarySpacing),\n/* harmony export */   createUnaryUnit: () => (/* binding */ createUnaryUnit),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getStyleFromPropValue: () => (/* binding */ getStyleFromPropValue),\n/* harmony export */   getValue: () => (/* binding */ getValue),\n/* harmony export */   margin: () => (/* binding */ margin),\n/* harmony export */   marginKeys: () => (/* binding */ marginKeys),\n/* harmony export */   padding: () => (/* binding */ padding),\n/* harmony export */   paddingKeys: () => (/* binding */ paddingKeys)\n/* harmony export */ });\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js\");\n/* harmony import */ var _memoize_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../memoize/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/memoize/memoize.js\");\n\n\n\n\n\nconst properties = {\n  m: 'margin',\n  p: 'padding'\n};\nconst directions = {\n  t: 'Top',\n  r: 'Right',\n  b: 'Bottom',\n  l: 'Left',\n  x: ['Left', 'Right'],\n  y: ['Top', 'Bottom']\n};\nconst aliases = {\n  marginX: 'mx',\n  marginY: 'my',\n  paddingX: 'px',\n  paddingY: 'py'\n};\n\n// memoize() impact:\n// From 300,000 ops/sec\n// To 350,000 ops/sec\nconst getCssProperties = (0,_memoize_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(prop => {\n  // It's not a shorthand notation.\n  if (prop.length > 2) {\n    if (aliases[prop]) {\n      prop = aliases[prop];\n    } else {\n      return [prop];\n    }\n  }\n  const [a, b] = prop.split('');\n  const property = properties[a];\n  const direction = directions[b] || '';\n  return Array.isArray(direction) ? direction.map(dir => property + dir) : [property + direction];\n});\nconst marginKeys = ['m', 'mt', 'mr', 'mb', 'ml', 'mx', 'my', 'margin', 'marginTop', 'marginRight', 'marginBottom', 'marginLeft', 'marginX', 'marginY', 'marginInline', 'marginInlineStart', 'marginInlineEnd', 'marginBlock', 'marginBlockStart', 'marginBlockEnd'];\nconst paddingKeys = ['p', 'pt', 'pr', 'pb', 'pl', 'px', 'py', 'padding', 'paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft', 'paddingX', 'paddingY', 'paddingInline', 'paddingInlineStart', 'paddingInlineEnd', 'paddingBlock', 'paddingBlockStart', 'paddingBlockEnd'];\nconst spacingKeys = [...marginKeys, ...paddingKeys];\nfunction createUnaryUnit(theme, themeKey, defaultValue, propName) {\n  const themeSpacing = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_1__.getPath)(theme, themeKey, true) ?? defaultValue;\n  if (typeof themeSpacing === 'number' || typeof themeSpacing === 'string') {\n    return val => {\n      if (typeof val === 'string') {\n        return val;\n      }\n      if (true) {\n        if (typeof val !== 'number') {\n          console.error(`MUI: Expected ${propName} argument to be a number or a string, got ${val}.`);\n        }\n      }\n      if (typeof themeSpacing === 'string') {\n        return `calc(${val} * ${themeSpacing})`;\n      }\n      return themeSpacing * val;\n    };\n  }\n  if (Array.isArray(themeSpacing)) {\n    return val => {\n      if (typeof val === 'string') {\n        return val;\n      }\n      const abs = Math.abs(val);\n      if (true) {\n        if (!Number.isInteger(abs)) {\n          console.error([`MUI: The \\`theme.${themeKey}\\` array type cannot be combined with non integer values.` + `You should either use an integer value that can be used as index, or define the \\`theme.${themeKey}\\` as a number.`].join('\\n'));\n        } else if (abs > themeSpacing.length - 1) {\n          console.error([`MUI: The value provided (${abs}) overflows.`, `The supported values are: ${JSON.stringify(themeSpacing)}.`, `${abs} > ${themeSpacing.length - 1}, you need to add the missing values.`].join('\\n'));\n        }\n      }\n      const transformed = themeSpacing[abs];\n      if (val >= 0) {\n        return transformed;\n      }\n      if (typeof transformed === 'number') {\n        return -transformed;\n      }\n      return `-${transformed}`;\n    };\n  }\n  if (typeof themeSpacing === 'function') {\n    return themeSpacing;\n  }\n  if (true) {\n    console.error([`MUI: The \\`theme.${themeKey}\\` value (${themeSpacing}) is invalid.`, 'It should be a number, an array or a function.'].join('\\n'));\n  }\n  return () => undefined;\n}\nfunction createUnarySpacing(theme) {\n  return createUnaryUnit(theme, 'spacing', 8, 'spacing');\n}\nfunction getValue(transformer, propValue) {\n  if (typeof propValue === 'string' || propValue == null) {\n    return propValue;\n  }\n  return transformer(propValue);\n}\nfunction getStyleFromPropValue(cssProperties, transformer) {\n  return propValue => cssProperties.reduce((acc, cssProperty) => {\n    acc[cssProperty] = getValue(transformer, propValue);\n    return acc;\n  }, {});\n}\nfunction resolveCssProperty(props, keys, prop, transformer) {\n  // Using a hash computation over an array iteration could be faster, but with only 28 items,\n  // it's doesn't worth the bundle size.\n  if (!keys.includes(prop)) {\n    return null;\n  }\n  const cssProperties = getCssProperties(prop);\n  const styleFromPropValue = getStyleFromPropValue(cssProperties, transformer);\n  const propValue = props[prop];\n  return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)(props, propValue, styleFromPropValue);\n}\nfunction style(props, keys) {\n  const transformer = createUnarySpacing(props.theme);\n  return Object.keys(props).map(prop => resolveCssProperty(props, keys, prop, transformer)).reduce(_merge_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"], {});\n}\nfunction margin(props) {\n  return style(props, marginKeys);\n}\nmargin.propTypes =  true ? marginKeys.reduce((obj, key) => {\n  obj[key] = _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"];\n  return obj;\n}, {}) : 0;\nmargin.filterProps = marginKeys;\nfunction padding(props) {\n  return style(props, paddingKeys);\n}\npadding.propTypes =  true ? paddingKeys.reduce((obj, key) => {\n  obj[key] = _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"];\n  return obj;\n}, {}) : 0;\npadding.filterProps = paddingKeys;\nfunction spacing(props) {\n  return style(props, spacingKeys);\n}\nspacing.propTypes =  true ? spacingKeys.reduce((obj, key) => {\n  obj[key] = _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"];\n  return obj;\n}, {}) : 0;\nspacing.filterProps = spacingKeys;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (spacing);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js":
/*!********************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js ***!
  \********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getPath: () => (/* binding */ getPath),\n/* harmony export */   getStyleValue: () => (/* binding */ getStyleValue)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n\n\n\nfunction getPath(obj, path, checkVars = true) {\n  if (!path || typeof path !== 'string') {\n    return null;\n  }\n\n  // Check if CSS variables are used\n  if (obj && obj.vars && checkVars) {\n    const val = `vars.${path}`.split('.').reduce((acc, item) => acc && acc[item] ? acc[item] : null, obj);\n    if (val != null) {\n      return val;\n    }\n  }\n  return path.split('.').reduce((acc, item) => {\n    if (acc && acc[item] != null) {\n      return acc[item];\n    }\n    return null;\n  }, obj);\n}\nfunction getStyleValue(themeMapping, transform, propValueFinal, userValue = propValueFinal) {\n  let value;\n  if (typeof themeMapping === 'function') {\n    value = themeMapping(propValueFinal);\n  } else if (Array.isArray(themeMapping)) {\n    value = themeMapping[propValueFinal] || userValue;\n  } else {\n    value = getPath(themeMapping, propValueFinal) || userValue;\n  }\n  if (transform) {\n    value = transform(value, userValue, themeMapping);\n  }\n  return value;\n}\nfunction style(options) {\n  const {\n    prop,\n    cssProperty = options.prop,\n    themeKey,\n    transform\n  } = options;\n\n  // false positive\n  // eslint-disable-next-line react/function-component-definition\n  const fn = props => {\n    if (props[prop] == null) {\n      return null;\n    }\n    const propValue = props[prop];\n    const theme = props.theme;\n    const themeMapping = getPath(theme, themeKey) || {};\n    const styleFromPropValue = propValueFinal => {\n      let value = getStyleValue(themeMapping, transform, propValueFinal);\n      if (propValueFinal === value && typeof propValueFinal === 'string') {\n        // Haven't found value\n        value = getStyleValue(themeMapping, transform, `${prop}${propValueFinal === 'default' ? '' : (0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(propValueFinal)}`, propValueFinal);\n      }\n      if (cssProperty === false) {\n        return value;\n      }\n      return {\n        [cssProperty]: value\n      };\n    };\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, propValue, styleFromPropValue);\n  };\n  fn.propTypes =  true ? {\n    [prop]: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n  } : 0;\n  fn.filterProps = [prop];\n  return fn;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (style);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js":
/*!****************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js ***!
  \****************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _borders_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../borders/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/borders/borders.js\");\n/* harmony import */ var _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../cssGrid/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssGrid/cssGrid.js\");\n/* harmony import */ var _palette_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../palette/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/palette/palette.js\");\n/* harmony import */ var _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../sizing/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/sizing/sizing.js\");\n\n\n\n\n\nconst defaultSxConfig = {\n  // borders\n  border: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderTop: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderRight: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderBottom: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderLeft: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderColor: {\n    themeKey: 'palette'\n  },\n  borderTopColor: {\n    themeKey: 'palette'\n  },\n  borderRightColor: {\n    themeKey: 'palette'\n  },\n  borderBottomColor: {\n    themeKey: 'palette'\n  },\n  borderLeftColor: {\n    themeKey: 'palette'\n  },\n  outline: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  outlineColor: {\n    themeKey: 'palette'\n  },\n  borderRadius: {\n    themeKey: 'shape.borderRadius',\n    style: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderRadius\n  },\n  // palette\n  color: {\n    themeKey: 'palette',\n    transform: _palette_index_js__WEBPACK_IMPORTED_MODULE_1__.paletteTransform\n  },\n  bgcolor: {\n    themeKey: 'palette',\n    cssProperty: 'backgroundColor',\n    transform: _palette_index_js__WEBPACK_IMPORTED_MODULE_1__.paletteTransform\n  },\n  backgroundColor: {\n    themeKey: 'palette',\n    transform: _palette_index_js__WEBPACK_IMPORTED_MODULE_1__.paletteTransform\n  },\n  // spacing\n  p: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pt: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pr: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pb: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pl: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  px: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  py: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  padding: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingTop: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingRight: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBottom: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingLeft: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingX: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingY: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingInline: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingInlineStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingInlineEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBlock: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBlockStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBlockEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  m: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mt: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mr: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mb: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  ml: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mx: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  my: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  margin: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginTop: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginRight: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBottom: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginLeft: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginX: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginY: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginInline: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginInlineStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginInlineEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBlock: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBlockStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBlockEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  // display\n  displayPrint: {\n    cssProperty: false,\n    transform: value => ({\n      '@media print': {\n        display: value\n      }\n    })\n  },\n  display: {},\n  overflow: {},\n  textOverflow: {},\n  visibility: {},\n  whiteSpace: {},\n  // flexbox\n  flexBasis: {},\n  flexDirection: {},\n  flexWrap: {},\n  justifyContent: {},\n  alignItems: {},\n  alignContent: {},\n  order: {},\n  flex: {},\n  flexGrow: {},\n  flexShrink: {},\n  alignSelf: {},\n  justifyItems: {},\n  justifySelf: {},\n  // grid\n  gap: {\n    style: _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__.gap\n  },\n  rowGap: {\n    style: _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__.rowGap\n  },\n  columnGap: {\n    style: _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__.columnGap\n  },\n  gridColumn: {},\n  gridRow: {},\n  gridAutoFlow: {},\n  gridAutoColumns: {},\n  gridAutoRows: {},\n  gridTemplateColumns: {},\n  gridTemplateRows: {},\n  gridTemplateAreas: {},\n  gridArea: {},\n  // positions\n  position: {},\n  zIndex: {\n    themeKey: 'zIndex'\n  },\n  top: {},\n  right: {},\n  bottom: {},\n  left: {},\n  // shadows\n  boxShadow: {\n    themeKey: 'shadows'\n  },\n  // sizing\n  width: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  maxWidth: {\n    style: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.maxWidth\n  },\n  minWidth: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  height: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  maxHeight: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  minHeight: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  boxSizing: {},\n  // typography\n  font: {\n    themeKey: 'font'\n  },\n  fontFamily: {\n    themeKey: 'typography'\n  },\n  fontSize: {\n    themeKey: 'typography'\n  },\n  fontStyle: {\n    themeKey: 'typography'\n  },\n  fontWeight: {\n    themeKey: 'typography'\n  },\n  letterSpacing: {},\n  textTransform: {},\n  lineHeight: {},\n  textAlign: {},\n  typography: {\n    cssProperty: false,\n    themeKey: 'typography'\n  }\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (defaultSxConfig);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/extendSxProp.js":
/*!*************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/extendSxProp.js ***!
  \*************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ extendSxProp)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./defaultSxConfig.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n\n\nconst splitProps = props => {\n  const result = {\n    systemProps: {},\n    otherProps: {}\n  };\n  const config = props?.theme?.unstable_sxConfig ?? _defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"];\n  Object.keys(props).forEach(prop => {\n    if (config[prop]) {\n      result.systemProps[prop] = props[prop];\n    } else {\n      result.otherProps[prop] = props[prop];\n    }\n  });\n  return result;\n};\nfunction extendSxProp(props) {\n  const {\n    sx: inSx,\n    ...other\n  } = props;\n  const {\n    systemProps,\n    otherProps\n  } = splitProps(other);\n  let finalSx;\n  if (Array.isArray(inSx)) {\n    finalSx = [systemProps, ...inSx];\n  } else if (typeof inSx === 'function') {\n    finalSx = (...args) => {\n      const result = inSx(...args);\n      if (!(0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_1__.isPlainObject)(result)) {\n        return systemProps;\n      }\n      return {\n        ...systemProps,\n        ...result\n      };\n    };\n  } else {\n    finalSx = {\n      ...systemProps,\n      ...inSx\n    };\n  }\n  return {\n    ...otherProps,\n    sx: finalSx\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/extendSxProp.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js":
/*!****************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js ***!
  \****************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   unstable_createStyleFunctionSx: () => (/* binding */ unstable_createStyleFunctionSx)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/merge/merge.js\");\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../cssContainerQueries/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js\");\n/* harmony import */ var _defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./defaultSxConfig.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n\n\n\n\n\n\nfunction objectsHaveSameKeys(...objects) {\n  const allKeys = objects.reduce((keys, object) => keys.concat(Object.keys(object)), []);\n  const union = new Set(allKeys);\n  return objects.every(object => union.size === Object.keys(object).length);\n}\nfunction callIfFn(maybeFn, arg) {\n  return typeof maybeFn === 'function' ? maybeFn(arg) : maybeFn;\n}\n\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction unstable_createStyleFunctionSx() {\n  function getThemeValue(prop, val, theme, config) {\n    const props = {\n      [prop]: val,\n      theme\n    };\n    const options = config[prop];\n    if (!options) {\n      return {\n        [prop]: val\n      };\n    }\n    const {\n      cssProperty = prop,\n      themeKey,\n      transform,\n      style\n    } = options;\n    if (val == null) {\n      return null;\n    }\n\n    // TODO v6: remove, see https://github.com/mui/material-ui/pull/38123\n    if (themeKey === 'typography' && val === 'inherit') {\n      return {\n        [prop]: val\n      };\n    }\n    const themeMapping = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__.getPath)(theme, themeKey) || {};\n    if (style) {\n      return style(props);\n    }\n    const styleFromPropValue = propValueFinal => {\n      let value = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__.getStyleValue)(themeMapping, transform, propValueFinal);\n      if (propValueFinal === value && typeof propValueFinal === 'string') {\n        // Haven't found value\n        value = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__.getStyleValue)(themeMapping, transform, `${prop}${propValueFinal === 'default' ? '' : (0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(propValueFinal)}`, propValueFinal);\n      }\n      if (cssProperty === false) {\n        return value;\n      }\n      return {\n        [cssProperty]: value\n      };\n    };\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)(props, val, styleFromPropValue);\n  }\n  function styleFunctionSx(props) {\n    const {\n      sx,\n      theme = {}\n    } = props || {};\n    if (!sx) {\n      return null; // Emotion & styled-components will neglect null\n    }\n    const config = theme.unstable_sxConfig ?? _defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"];\n\n    /*\n     * Receive `sxInput` as object or callback\n     * and then recursively check keys & values to create media query object styles.\n     * (the result will be used in `styled`)\n     */\n    function traverse(sxInput) {\n      let sxObject = sxInput;\n      if (typeof sxInput === 'function') {\n        sxObject = sxInput(theme);\n      } else if (typeof sxInput !== 'object') {\n        // value\n        return sxInput;\n      }\n      if (!sxObject) {\n        return null;\n      }\n      const emptyBreakpoints = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.createEmptyBreakpointObject)(theme.breakpoints);\n      const breakpointsKeys = Object.keys(emptyBreakpoints);\n      let css = emptyBreakpoints;\n      Object.keys(sxObject).forEach(styleKey => {\n        const value = callIfFn(sxObject[styleKey], theme);\n        if (value !== null && value !== undefined) {\n          if (typeof value === 'object') {\n            if (config[styleKey]) {\n              css = (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(css, getThemeValue(styleKey, value, theme, config));\n            } else {\n              const breakpointsValues = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)({\n                theme\n              }, value, x => ({\n                [styleKey]: x\n              }));\n              if (objectsHaveSameKeys(breakpointsValues, value)) {\n                css[styleKey] = styleFunctionSx({\n                  sx: value,\n                  theme\n                });\n              } else {\n                css = (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(css, breakpointsValues);\n              }\n            }\n          } else {\n            css = (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(css, getThemeValue(styleKey, value, theme, config));\n          }\n        }\n      });\n      return (0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_5__.sortContainerQueries)(theme, (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.removeUnusedBreakpoints)(breakpointsKeys, css));\n    }\n    return Array.isArray(sx) ? sx.map(traverse) : traverse(sx);\n  }\n  return styleFunctionSx;\n}\nconst styleFunctionSx = unstable_createStyleFunctionSx();\nstyleFunctionSx.filterProps = ['sx'];\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (styleFunctionSx);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/styled/styled.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/styled/styled.js ***!
  \**********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _createStyled_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createStyled/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createStyled/createStyled.js\");\n\nconst styled = (0,_createStyled_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (styled);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/styled/styled.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js":
/*!**************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js ***!
  \**************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   systemDefaultTheme: () => (/* binding */ systemDefaultTheme)\n/* harmony export */ });\n/* harmony import */ var _createTheme_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js\");\n/* harmony import */ var _useThemeWithoutDefault_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../useThemeWithoutDefault/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeWithoutDefault/useThemeWithoutDefault.js\");\n'use client';\n\n\n\nconst systemDefaultTheme = (0,_createTheme_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\nfunction useTheme(defaultTheme = systemDefaultTheme) {\n  return (0,_useThemeWithoutDefault_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(defaultTheme);\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (useTheme);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/getThemeProps.js":
/*!************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/getThemeProps.js ***!
  \************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getThemeProps)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/resolveProps */ \"./node_modules/@mui/utils/esm/resolveProps/resolveProps.js\");\n\nfunction getThemeProps(params) {\n  const {\n    theme,\n    name,\n    props\n  } = params;\n  if (!theme || !theme.components || !theme.components[name] || !theme.components[name].defaultProps) {\n    return props;\n  }\n  return (0,_mui_utils_resolveProps__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(theme.components[name].defaultProps, props);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/getThemeProps.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/useThemeProps.js":
/*!************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/useThemeProps.js ***!
  \************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useThemeProps)\n/* harmony export */ });\n/* harmony import */ var _getThemeProps_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./getThemeProps.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/getThemeProps.js\");\n/* harmony import */ var _useTheme_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../useTheme/index.js */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js\");\n'use client';\n\n\n\nfunction useThemeProps({\n  props,\n  name,\n  defaultTheme,\n  themeId\n}) {\n  let theme = (0,_useTheme_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(defaultTheme);\n  if (themeId) {\n    theme = theme[themeId] || theme;\n  }\n  return (0,_getThemeProps_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n    theme,\n    name,\n    props\n  });\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeProps/useThemeProps.js?");

/***/ }),

/***/ "./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeWithoutDefault/useThemeWithoutDefault.js":
/*!******************************************************************************************************************!*\
  !*** ./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeWithoutDefault/useThemeWithoutDefault.js ***!
  \******************************************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n'use client';\n\n\n\nfunction isObjectEmpty(obj) {\n  return Object.keys(obj).length === 0;\n}\nfunction useTheme(defaultTheme = null) {\n  const contextTheme = react__WEBPACK_IMPORTED_MODULE_0__.useContext(_mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__.T);\n  return !contextTheme || isObjectEmpty(contextTheme) ? defaultTheme : contextTheme;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (useTheme);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/node_modules/@mui/system/esm/useThemeWithoutDefault/useThemeWithoutDefault.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/ThemeProvider.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/styles/ThemeProvider.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ ThemeProvider)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _ThemeProviderNoVars_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./ThemeProviderNoVars.js */ \"./node_modules/@mui/material/styles/ThemeProviderNoVars.js\");\n/* harmony import */ var _ThemeProviderWithVars_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./ThemeProviderWithVars.js */ \"./node_modules/@mui/material/styles/ThemeProviderWithVars.js\");\n/* harmony import */ var _identifier_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\nfunction ThemeProvider({\n  theme,\n  ...props\n}) {\n  if (typeof theme === 'function') {\n    return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_ThemeProviderNoVars_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n      theme: theme,\n      ...props\n    });\n  }\n  const muiTheme = _identifier_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"] in theme ? theme[_identifier_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]] : theme;\n  if (!('colorSchemes' in muiTheme)) {\n    if (!('vars' in muiTheme)) {\n      // For non-CSS variables themes, set `vars` to null to prevent theme inheritance from the upper theme.\n      // The example use case is the docs demo that uses ThemeProvider to customize the theme while the upper theme is using CSS variables.\n      return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_ThemeProviderNoVars_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n        theme: {\n          ...theme,\n          vars: null\n        },\n        ...props\n      });\n    }\n    return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_ThemeProviderNoVars_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n      theme: theme,\n      ...props\n    });\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_ThemeProviderWithVars_js__WEBPACK_IMPORTED_MODULE_4__.CssVarsProvider, {\n    theme: theme,\n    ...props\n  });\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/ThemeProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/ThemeProviderNoVars.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/styles/ThemeProviderNoVars.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ ThemeProviderNoVars)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/ThemeProvider/ThemeProvider.js\");\n/* harmony import */ var _identifier_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\nfunction ThemeProviderNoVars({\n  theme: themeInput,\n  ...props\n}) {\n  const scopedTheme = _identifier_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"] in themeInput ? themeInput[_identifier_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]] : undefined;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_mui_system__WEBPACK_IMPORTED_MODULE_3__[\"default\"], {\n    ...props,\n    themeId: scopedTheme ? _identifier_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"] : undefined,\n    theme: scopedTheme || themeInput\n  });\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/ThemeProviderNoVars.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/ThemeProviderWithVars.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/styles/ThemeProviderWithVars.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   CssVarsProvider: () => (/* binding */ CssVarsProvider),\n/* harmony export */   Experimental_CssVarsProvider: () => (/* binding */ Experimental_CssVarsProvider),\n/* harmony export */   getInitColorSchemeScript: () => (/* binding */ getInitColorSchemeScript),\n/* harmony export */   useColorScheme: () => (/* binding */ useColorScheme)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/system/styleFunctionSx */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createCssVarsProvider.js\");\n/* harmony import */ var _createTheme_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./createTheme.js */ \"./node_modules/@mui/material/styles/createTheme.js\");\n/* harmony import */ var _createTypography_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./createTypography.js */ \"./node_modules/@mui/material/styles/createTypography.js\");\n/* harmony import */ var _identifier_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n/* harmony import */ var _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../InitColorSchemeScript/InitColorSchemeScript.js */ \"./node_modules/@mui/material/InitColorSchemeScript/InitColorSchemeScript.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n\n\nconst {\n  CssVarsProvider: InternalCssVarsProvider,\n  useColorScheme,\n  getInitColorSchemeScript: deprecatedGetInitColorSchemeScript\n} = (0,_mui_system__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n  themeId: _identifier_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n  // @ts-ignore ignore module augmentation tests\n  theme: () => (0,_createTheme_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])({\n    cssVariables: true\n  }),\n  colorSchemeStorageKey: _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_5__.defaultConfig.colorSchemeStorageKey,\n  modeStorageKey: _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_5__.defaultConfig.modeStorageKey,\n  defaultColorScheme: {\n    light: _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_5__.defaultConfig.defaultLightColorScheme,\n    dark: _InitColorSchemeScript_InitColorSchemeScript_js__WEBPACK_IMPORTED_MODULE_5__.defaultConfig.defaultDarkColorScheme\n  },\n  resolveTheme: theme => {\n    const newTheme = {\n      ...theme,\n      typography: (0,_createTypography_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(theme.palette, theme.typography)\n    };\n    newTheme.unstable_sx = function sx(props) {\n      return (0,_mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_7__[\"default\"])({\n        sx: props,\n        theme: this\n      });\n    };\n    return newTheme;\n  }\n});\nlet warnedOnce = false;\n\n// TODO: remove in v7\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction Experimental_CssVarsProvider(props) {\n  if (true) {\n    if (!warnedOnce) {\n      console.warn(['MUI: The Experimental_CssVarsProvider component has been ported into ThemeProvider.', '', \"You should use `import { ThemeProvider } from '@mui/material/styles'` instead.\", 'For more details, check out https://mui.com/material-ui/customization/css-theme-variables/usage/'].join('\\n'));\n      warnedOnce = true;\n    }\n  }\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(InternalCssVarsProvider, {\n    ...props\n  });\n}\nlet warnedInitScriptOnce = false;\n\n// TODO: remove in v7\nconst getInitColorSchemeScript = params => {\n  if (!warnedInitScriptOnce) {\n    console.warn(['MUI: The getInitColorSchemeScript function has been deprecated.', '', \"You should use `import InitColorSchemeScript from '@mui/material/InitColorSchemeScript'`\", 'and replace the function call with `<InitColorSchemeScript />` instead.'].join('\\n'));\n    warnedInitScriptOnce = true;\n  }\n  return deprecatedGetInitColorSchemeScript(params);\n};\n\n/**\n * TODO: remove this export in v7\n * @deprecated\n * The `CssVarsProvider` component has been deprecated and ported into `ThemeProvider`.\n *\n * You should use `ThemeProvider` and `createTheme()` instead:\n *\n * ```diff\n * - import { CssVarsProvider, extendTheme } from '@mui/material/styles';\n * + import { ThemeProvider, createTheme } from '@mui/material/styles';\n *\n * - const theme = extendTheme();\n * + const theme = createTheme({\n * +   cssVariables: true,\n * +   colorSchemes: { light: true, dark: true },\n * + });\n *\n * - <CssVarsProvider theme={theme}>\n * + <ThemeProvider theme={theme}>\n * ```\n *\n * To see the full documentation, check out https://mui.com/material-ui/customization/css-theme-variables/usage/.\n */\nconst CssVarsProvider = InternalCssVarsProvider;\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/ThemeProviderWithVars.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createColorScheme.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createColorScheme.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createColorScheme),\n/* harmony export */   getOpacity: () => (/* binding */ getOpacity),\n/* harmony export */   getOverlays: () => (/* binding */ getOverlays)\n/* harmony export */ });\n/* harmony import */ var _createPalette_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./createPalette.js */ \"./node_modules/@mui/material/styles/createPalette.js\");\n/* harmony import */ var _getOverlayAlpha_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./getOverlayAlpha.js */ \"./node_modules/@mui/material/styles/getOverlayAlpha.js\");\n\n\nconst defaultDarkOverlays = [...Array(25)].map((_, index) => {\n  if (index === 0) {\n    return 'none';\n  }\n  const overlay = (0,_getOverlayAlpha_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(index);\n  return `linear-gradient(rgba(255 255 255 / ${overlay}), rgba(255 255 255 / ${overlay}))`;\n});\nfunction getOpacity(mode) {\n  return {\n    inputPlaceholder: mode === 'dark' ? 0.5 : 0.42,\n    inputUnderline: mode === 'dark' ? 0.7 : 0.42,\n    switchTrackDisabled: mode === 'dark' ? 0.2 : 0.12,\n    switchTrack: mode === 'dark' ? 0.3 : 0.38\n  };\n}\nfunction getOverlays(mode) {\n  return mode === 'dark' ? defaultDarkOverlays : [];\n}\nfunction createColorScheme(options) {\n  const {\n    palette: paletteInput = {\n      mode: 'light'\n    },\n    // need to cast to avoid module augmentation test\n    opacity,\n    overlays,\n    ...rest\n  } = options;\n  const palette = (0,_createPalette_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(paletteInput);\n  return {\n    palette,\n    opacity: {\n      ...getOpacity(palette.mode),\n      ...opacity\n    },\n    overlays: overlays || getOverlays(palette.mode),\n    ...rest\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createColorScheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createGetSelector.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createGetSelector.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _excludeVariablesFromRoot_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./excludeVariablesFromRoot.js */ \"./node_modules/@mui/material/styles/excludeVariablesFromRoot.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (theme => (colorScheme, css) => {\n  const root = theme.rootSelector || ':root';\n  const selector = theme.colorSchemeSelector;\n  let rule = selector;\n  if (selector === 'class') {\n    rule = '.%s';\n  }\n  if (selector === 'data') {\n    rule = '[data-%s]';\n  }\n  if (selector?.startsWith('data-') && !selector.includes('%s')) {\n    // 'data-mui-color-scheme' -> '[data-mui-color-scheme=\"%s\"]'\n    rule = `[${selector}=\"%s\"]`;\n  }\n  if (theme.defaultColorScheme === colorScheme) {\n    if (colorScheme === 'dark') {\n      const excludedVariables = {};\n      (0,_excludeVariablesFromRoot_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(theme.cssVarPrefix).forEach(cssVar => {\n        excludedVariables[cssVar] = css[cssVar];\n        delete css[cssVar];\n      });\n      if (rule === 'media') {\n        return {\n          [root]: css,\n          [`@media (prefers-color-scheme: dark)`]: {\n            [root]: excludedVariables\n          }\n        };\n      }\n      if (rule) {\n        return {\n          [rule.replace('%s', colorScheme)]: excludedVariables,\n          [`${root}, ${rule.replace('%s', colorScheme)}`]: css\n        };\n      }\n      return {\n        [root]: {\n          ...css,\n          ...excludedVariables\n        }\n      };\n    }\n    if (rule && rule !== 'media') {\n      return `${root}, ${rule.replace('%s', String(colorScheme))}`;\n    }\n  } else if (colorScheme) {\n    if (rule === 'media') {\n      return {\n        [`@media (prefers-color-scheme: ${String(colorScheme)})`]: {\n          [root]: css\n        }\n      };\n    }\n    if (rule) {\n      return rule.replace('%s', String(colorScheme));\n    }\n  }\n  return root;\n});\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createGetSelector.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createMixins.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/styles/createMixins.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createMixins)\n/* harmony export */ });\nfunction createMixins(breakpoints, mixins) {\n  return {\n    toolbar: {\n      minHeight: 56,\n      [breakpoints.up('xs')]: {\n        '@media (orientation: landscape)': {\n          minHeight: 48\n        }\n      },\n      [breakpoints.up('sm')]: {\n        minHeight: 64\n      }\n    },\n    ...mixins\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createMixins.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createPalette.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createPalette.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   dark: () => (/* binding */ dark),\n/* harmony export */   \"default\": () => (/* binding */ createPalette),\n/* harmony export */   light: () => (/* binding */ light)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _colors_common_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../colors/common.js */ \"./node_modules/@mui/material/colors/common.js\");\n/* harmony import */ var _colors_grey_js__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ../colors/grey.js */ \"./node_modules/@mui/material/colors/grey.js\");\n/* harmony import */ var _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../colors/purple.js */ \"./node_modules/@mui/material/colors/purple.js\");\n/* harmony import */ var _colors_red_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../colors/red.js */ \"./node_modules/@mui/material/colors/red.js\");\n/* harmony import */ var _colors_orange_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../colors/orange.js */ \"./node_modules/@mui/material/colors/orange.js\");\n/* harmony import */ var _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../colors/blue.js */ \"./node_modules/@mui/material/colors/blue.js\");\n/* harmony import */ var _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../colors/lightBlue.js */ \"./node_modules/@mui/material/colors/lightBlue.js\");\n/* harmony import */ var _colors_green_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../colors/green.js */ \"./node_modules/@mui/material/colors/green.js\");\n\n\n\n\n\n\n\n\n\n\n\nfunction getLight() {\n  return {\n    // The colors used to style the text.\n    text: {\n      // The most important text.\n      primary: 'rgba(0, 0, 0, 0.87)',\n      // Secondary text.\n      secondary: 'rgba(0, 0, 0, 0.6)',\n      // Disabled text have even lower visual prominence.\n      disabled: 'rgba(0, 0, 0, 0.38)'\n    },\n    // The color used to divide different elements.\n    divider: 'rgba(0, 0, 0, 0.12)',\n    // The background colors used to style the surfaces.\n    // Consistency between these values is important.\n    background: {\n      paper: _colors_common_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"].white,\n      default: _colors_common_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"].white\n    },\n    // The colors used to style the action elements.\n    action: {\n      // The color of an active action like an icon button.\n      active: 'rgba(0, 0, 0, 0.54)',\n      // The color of an hovered action.\n      hover: 'rgba(0, 0, 0, 0.04)',\n      hoverOpacity: 0.04,\n      // The color of a selected action.\n      selected: 'rgba(0, 0, 0, 0.08)',\n      selectedOpacity: 0.08,\n      // The color of a disabled action.\n      disabled: 'rgba(0, 0, 0, 0.26)',\n      // The background color of a disabled action.\n      disabledBackground: 'rgba(0, 0, 0, 0.12)',\n      disabledOpacity: 0.38,\n      focus: 'rgba(0, 0, 0, 0.12)',\n      focusOpacity: 0.12,\n      activatedOpacity: 0.12\n    }\n  };\n}\nconst light = getLight();\nfunction getDark() {\n  return {\n    text: {\n      primary: _colors_common_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"].white,\n      secondary: 'rgba(255, 255, 255, 0.7)',\n      disabled: 'rgba(255, 255, 255, 0.5)',\n      icon: 'rgba(255, 255, 255, 0.5)'\n    },\n    divider: 'rgba(255, 255, 255, 0.12)',\n    background: {\n      paper: '#121212',\n      default: '#121212'\n    },\n    action: {\n      active: _colors_common_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"].white,\n      hover: 'rgba(255, 255, 255, 0.08)',\n      hoverOpacity: 0.08,\n      selected: 'rgba(255, 255, 255, 0.16)',\n      selectedOpacity: 0.16,\n      disabled: 'rgba(255, 255, 255, 0.3)',\n      disabledBackground: 'rgba(255, 255, 255, 0.12)',\n      disabledOpacity: 0.38,\n      focus: 'rgba(255, 255, 255, 0.12)',\n      focusOpacity: 0.12,\n      activatedOpacity: 0.24\n    }\n  };\n}\nconst dark = getDark();\nfunction addLightOrDark(intent, direction, shade, tonalOffset) {\n  const tonalOffsetLight = tonalOffset.light || tonalOffset;\n  const tonalOffsetDark = tonalOffset.dark || tonalOffset * 1.5;\n  if (!intent[direction]) {\n    if (intent.hasOwnProperty(shade)) {\n      intent[direction] = intent[shade];\n    } else if (direction === 'light') {\n      intent.light = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_1__.lighten)(intent.main, tonalOffsetLight);\n    } else if (direction === 'dark') {\n      intent.dark = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_1__.darken)(intent.main, tonalOffsetDark);\n    }\n  }\n}\nfunction getDefaultPrimary(mode = 'light') {\n  if (mode === 'dark') {\n    return {\n      main: _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"][200],\n      light: _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"][50],\n      dark: _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"][400]\n    };\n  }\n  return {\n    main: _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"][700],\n    light: _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"][400],\n    dark: _colors_blue_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"][800]\n  };\n}\nfunction getDefaultSecondary(mode = 'light') {\n  if (mode === 'dark') {\n    return {\n      main: _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"][200],\n      light: _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"][50],\n      dark: _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"][400]\n    };\n  }\n  return {\n    main: _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"][500],\n    light: _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"][300],\n    dark: _colors_purple_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"][700]\n  };\n}\nfunction getDefaultError(mode = 'light') {\n  if (mode === 'dark') {\n    return {\n      main: _colors_red_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"][500],\n      light: _colors_red_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"][300],\n      dark: _colors_red_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"][700]\n    };\n  }\n  return {\n    main: _colors_red_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"][700],\n    light: _colors_red_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"][400],\n    dark: _colors_red_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"][800]\n  };\n}\nfunction getDefaultInfo(mode = 'light') {\n  if (mode === 'dark') {\n    return {\n      main: _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"][400],\n      light: _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"][300],\n      dark: _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"][700]\n    };\n  }\n  return {\n    main: _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"][700],\n    light: _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"][500],\n    dark: _colors_lightBlue_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"][900]\n  };\n}\nfunction getDefaultSuccess(mode = 'light') {\n  if (mode === 'dark') {\n    return {\n      main: _colors_green_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"][400],\n      light: _colors_green_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"][300],\n      dark: _colors_green_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"][700]\n    };\n  }\n  return {\n    main: _colors_green_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"][800],\n    light: _colors_green_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"][500],\n    dark: _colors_green_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"][900]\n  };\n}\nfunction getDefaultWarning(mode = 'light') {\n  if (mode === 'dark') {\n    return {\n      main: _colors_orange_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"][400],\n      light: _colors_orange_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"][300],\n      dark: _colors_orange_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"][700]\n    };\n  }\n  return {\n    main: '#ed6c02',\n    // closest to orange[800] that pass 3:1.\n    light: _colors_orange_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"][500],\n    dark: _colors_orange_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"][900]\n  };\n}\nfunction createPalette(palette) {\n  const {\n    mode = 'light',\n    contrastThreshold = 3,\n    tonalOffset = 0.2,\n    ...other\n  } = palette;\n  const primary = palette.primary || getDefaultPrimary(mode);\n  const secondary = palette.secondary || getDefaultSecondary(mode);\n  const error = palette.error || getDefaultError(mode);\n  const info = palette.info || getDefaultInfo(mode);\n  const success = palette.success || getDefaultSuccess(mode);\n  const warning = palette.warning || getDefaultWarning(mode);\n\n  // Use the same logic as\n  // Bootstrap: https://github.com/twbs/bootstrap/blob/1d6e3710dd447de1a200f29e8fa521f8a0908f70/scss/_functions.scss#L59\n  // and material-components-web https://github.com/material-components/material-components-web/blob/ac46b8863c4dab9fc22c4c662dc6bd1b65dd652f/packages/mdc-theme/_functions.scss#L54\n  function getContrastText(background) {\n    const contrastText = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_1__.getContrastRatio)(background, dark.text.primary) >= contrastThreshold ? dark.text.primary : light.text.primary;\n    if (true) {\n      const contrast = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_1__.getContrastRatio)(background, contrastText);\n      if (contrast < 3) {\n        console.error([`MUI: The contrast ratio of ${contrast}:1 for ${contrastText} on ${background}`, 'falls below the WCAG recommended absolute minimum contrast ratio of 3:1.', 'https://www.w3.org/TR/2008/REC-WCAG20-20081211/#visual-audio-contrast-contrast'].join('\\n'));\n      }\n    }\n    return contrastText;\n  }\n  const augmentColor = ({\n    color,\n    name,\n    mainShade = 500,\n    lightShade = 300,\n    darkShade = 700\n  }) => {\n    color = {\n      ...color\n    };\n    if (!color.main && color[mainShade]) {\n      color.main = color[mainShade];\n    }\n    if (!color.hasOwnProperty('main')) {\n      throw new Error( true ? `MUI: The color${name ? ` (${name})` : ''} provided to augmentColor(color) is invalid.\\n` + `The color object needs to have a \\`main\\` property or a \\`${mainShade}\\` property.` : 0);\n    }\n    if (typeof color.main !== 'string') {\n      throw new Error( true ? `MUI: The color${name ? ` (${name})` : ''} provided to augmentColor(color) is invalid.\\n` + `\\`color.main\\` should be a string, but \\`${JSON.stringify(color.main)}\\` was provided instead.\\n` + '\\n' + 'Did you intend to use one of the following approaches?\\n' + '\\n' + 'import { green } from \"@mui/material/colors\";\\n' + '\\n' + 'const theme1 = createTheme({ palette: {\\n' + '  primary: green,\\n' + '} });\\n' + '\\n' + 'const theme2 = createTheme({ palette: {\\n' + '  primary: { main: green[500] },\\n' + '} });' : 0);\n    }\n    addLightOrDark(color, 'light', lightShade, tonalOffset);\n    addLightOrDark(color, 'dark', darkShade, tonalOffset);\n    if (!color.contrastText) {\n      color.contrastText = getContrastText(color.main);\n    }\n    return color;\n  };\n  let modeHydrated;\n  if (mode === 'light') {\n    modeHydrated = getLight();\n  } else if (mode === 'dark') {\n    modeHydrated = getDark();\n  }\n  if (true) {\n    if (!modeHydrated) {\n      console.error(`MUI: The palette mode \\`${mode}\\` is not supported.`);\n    }\n  }\n  const paletteOutput = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_8__[\"default\"])({\n    // A collection of common colors.\n    common: {\n      ..._colors_common_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"]\n    },\n    // prevent mutable object.\n    // The palette mode, can be light or dark.\n    mode,\n    // The colors used to represent primary interface elements for a user.\n    primary: augmentColor({\n      color: primary,\n      name: 'primary'\n    }),\n    // The colors used to represent secondary interface elements for a user.\n    secondary: augmentColor({\n      color: secondary,\n      name: 'secondary',\n      mainShade: 'A400',\n      lightShade: 'A200',\n      darkShade: 'A700'\n    }),\n    // The colors used to represent interface elements that the user should be made aware of.\n    error: augmentColor({\n      color: error,\n      name: 'error'\n    }),\n    // The colors used to represent potentially dangerous actions or important messages.\n    warning: augmentColor({\n      color: warning,\n      name: 'warning'\n    }),\n    // The colors used to present information to the user that is neutral and not necessarily important.\n    info: augmentColor({\n      color: info,\n      name: 'info'\n    }),\n    // The colors used to indicate the successful completion of an action that user triggered.\n    success: augmentColor({\n      color: success,\n      name: 'success'\n    }),\n    // The grey colors.\n    grey: _colors_grey_js__WEBPACK_IMPORTED_MODULE_9__[\"default\"],\n    // Used by `getContrastText()` to maximize the contrast between\n    // the background and the text.\n    contrastThreshold,\n    // Takes a background color and returns the text color that maximizes the contrast.\n    getContrastText,\n    // Generate a rich color object.\n    augmentColor,\n    // Used by the functions below to shift a color's luminance by approximately\n    // two indexes within its tonal palette.\n    // E.g., shift from Red 500 to Red 300 or Red 700.\n    tonalOffset,\n    // The light and dark mode object.\n    ...modeHydrated\n  }, other);\n  return paletteOutput;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createPalette.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createTheme.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/styles/createTheme.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   createMuiTheme: () => (/* reexport safe */ _createThemeNoVars_js__WEBPACK_IMPORTED_MODULE_0__.createMuiTheme),\n/* harmony export */   \"default\": () => (/* binding */ createTheme)\n/* harmony export */ });\n/* harmony import */ var _createPalette_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./createPalette.js */ \"./node_modules/@mui/material/styles/createPalette.js\");\n/* harmony import */ var _createThemeWithVars_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./createThemeWithVars.js */ \"./node_modules/@mui/material/styles/createThemeWithVars.js\");\n/* harmony import */ var _createThemeNoVars_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./createThemeNoVars.js */ \"./node_modules/@mui/material/styles/createThemeNoVars.js\");\n\n\n\n\n// eslint-disable-next-line consistent-return\nfunction attachColorScheme(theme, scheme, colorScheme) {\n  if (!theme.colorSchemes) {\n    return undefined;\n  }\n  if (colorScheme) {\n    theme.colorSchemes[scheme] = {\n      ...(colorScheme !== true && colorScheme),\n      palette: (0,_createPalette_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n        ...(colorScheme === true ? {} : colorScheme.palette),\n        mode: scheme\n      }) // cast type to skip module augmentation test\n    };\n  }\n}\n\n/**\n * Generate a theme base on the options received.\n * @param options Takes an incomplete theme object and adds the missing parts.\n * @param args Deep merge the arguments with the about to be returned theme.\n * @returns A complete, ready-to-use theme object.\n */\nfunction createTheme(options = {},\n// cast type to skip module augmentation test\n...args) {\n  const {\n    palette,\n    cssVariables = false,\n    colorSchemes: initialColorSchemes = !palette ? {\n      light: true\n    } : undefined,\n    defaultColorScheme: initialDefaultColorScheme = palette?.mode,\n    ...rest\n  } = options;\n  const defaultColorSchemeInput = initialDefaultColorScheme || 'light';\n  const defaultScheme = initialColorSchemes?.[defaultColorSchemeInput];\n  const colorSchemesInput = {\n    ...initialColorSchemes,\n    ...(palette ? {\n      [defaultColorSchemeInput]: {\n        ...(typeof defaultScheme !== 'boolean' && defaultScheme),\n        palette\n      }\n    } : undefined)\n  };\n  if (cssVariables === false) {\n    if (!('colorSchemes' in options)) {\n      // Behaves exactly as v5\n      return (0,_createThemeNoVars_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(options, ...args);\n    }\n    let paletteOptions = palette;\n    if (!('palette' in options)) {\n      if (colorSchemesInput[defaultColorSchemeInput]) {\n        if (colorSchemesInput[defaultColorSchemeInput] !== true) {\n          paletteOptions = colorSchemesInput[defaultColorSchemeInput].palette;\n        } else if (defaultColorSchemeInput === 'dark') {\n          // @ts-ignore to prevent the module augmentation test from failing\n          paletteOptions = {\n            mode: 'dark'\n          };\n        }\n      }\n    }\n    const theme = (0,_createThemeNoVars_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n      ...options,\n      palette: paletteOptions\n    }, ...args);\n    theme.defaultColorScheme = defaultColorSchemeInput;\n    theme.colorSchemes = colorSchemesInput;\n    if (theme.palette.mode === 'light') {\n      theme.colorSchemes.light = {\n        ...(colorSchemesInput.light !== true && colorSchemesInput.light),\n        palette: theme.palette\n      };\n      attachColorScheme(theme, 'dark', colorSchemesInput.dark);\n    }\n    if (theme.palette.mode === 'dark') {\n      theme.colorSchemes.dark = {\n        ...(colorSchemesInput.dark !== true && colorSchemesInput.dark),\n        palette: theme.palette\n      };\n      attachColorScheme(theme, 'light', colorSchemesInput.light);\n    }\n    return theme;\n  }\n  if (!palette && !('light' in colorSchemesInput) && defaultColorSchemeInput === 'light') {\n    colorSchemesInput.light = true;\n  }\n  return (0,_createThemeWithVars_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n    ...rest,\n    colorSchemes: colorSchemesInput,\n    defaultColorScheme: defaultColorSchemeInput,\n    ...(typeof cssVariables !== 'boolean' && cssVariables)\n  }, ...args);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createThemeNoVars.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createThemeNoVars.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   createMuiTheme: () => (/* binding */ createMuiTheme),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system/styleFunctionSx */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n/* harmony import */ var _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/system/styleFunctionSx */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _mui_system_createTheme__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/system/createTheme */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createTheme.js\");\n/* harmony import */ var _mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/utils/generateUtilityClass */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n/* harmony import */ var _createMixins_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./createMixins.js */ \"./node_modules/@mui/material/styles/createMixins.js\");\n/* harmony import */ var _createPalette_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./createPalette.js */ \"./node_modules/@mui/material/styles/createPalette.js\");\n/* harmony import */ var _createTypography_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./createTypography.js */ \"./node_modules/@mui/material/styles/createTypography.js\");\n/* harmony import */ var _shadows_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./shadows.js */ \"./node_modules/@mui/material/styles/shadows.js\");\n/* harmony import */ var _createTransitions_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./createTransitions.js */ \"./node_modules/@mui/material/styles/createTransitions.js\");\n/* harmony import */ var _zIndex_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./zIndex.js */ \"./node_modules/@mui/material/styles/zIndex.js\");\n/* harmony import */ var _stringifyTheme_js__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./stringifyTheme.js */ \"./node_modules/@mui/material/styles/stringifyTheme.js\");\n\n\n\n\n\n\n\n\n\n\n\n\nfunction createThemeNoVars(options = {}, ...args) {\n  const {\n    breakpoints: breakpointsInput,\n    mixins: mixinsInput = {},\n    spacing: spacingInput,\n    palette: paletteInput = {},\n    transitions: transitionsInput = {},\n    typography: typographyInput = {},\n    shape: shapeInput,\n    ...other\n  } = options;\n  if (options.vars &&\n  // The error should throw only for the root theme creation because user is not allowed to use a custom node `vars`.\n  // `generateThemeVars` is the closest identifier for checking that the `options` is a result of `createTheme` with CSS variables so that user can create new theme for nested ThemeProvider.\n  options.generateThemeVars === undefined) {\n    throw new Error( true ? 'MUI: `vars` is a private field used for CSS variables support.\\n' + 'Please use another name or follow the [docs](https://mui.com/material-ui/customization/css-theme-variables/usage/) to enable the feature.' : 0);\n  }\n  const palette = (0,_createPalette_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(paletteInput);\n  const systemTheme = (0,_mui_system_createTheme__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(options);\n  let muiTheme = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(systemTheme, {\n    mixins: (0,_createMixins_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(systemTheme.breakpoints, mixinsInput),\n    palette,\n    // Don't use [...shadows] until you've verified its transpiled code is not invoking the iterator protocol.\n    shadows: _shadows_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"].slice(),\n    typography: (0,_createTypography_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(palette, typographyInput),\n    transitions: (0,_createTransitions_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(transitionsInput),\n    zIndex: {\n      ..._zIndex_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"]\n    }\n  });\n  muiTheme = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(muiTheme, other);\n  muiTheme = args.reduce((acc, argument) => (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(acc, argument), muiTheme);\n  if (true) {\n    // TODO v6: Refactor to use globalStateClassesMapping from @mui/utils once `readOnly` state class is used in Rating component.\n    const stateClasses = ['active', 'checked', 'completed', 'disabled', 'error', 'expanded', 'focused', 'focusVisible', 'required', 'selected'];\n    const traverse = (node, component) => {\n      let key;\n\n      // eslint-disable-next-line guard-for-in\n      for (key in node) {\n        const child = node[key];\n        if (stateClasses.includes(key) && Object.keys(child).length > 0) {\n          if (true) {\n            const stateClass = (0,_mui_utils_generateUtilityClass__WEBPACK_IMPORTED_MODULE_8__[\"default\"])('', key);\n            console.error([`MUI: The \\`${component}\\` component increases ` + `the CSS specificity of the \\`${key}\\` internal state.`, 'You can not override it like this: ', JSON.stringify(node, null, 2), '', `Instead, you need to use the '&.${stateClass}' syntax:`, JSON.stringify({\n              root: {\n                [`&.${stateClass}`]: child\n              }\n            }, null, 2), '', 'https://mui.com/r/state-classes-guide'].join('\\n'));\n          }\n          // Remove the style to prevent global conflicts.\n          node[key] = {};\n        }\n      }\n    };\n    Object.keys(muiTheme.components).forEach(component => {\n      const styleOverrides = muiTheme.components[component].styleOverrides;\n      if (styleOverrides && component.startsWith('Mui')) {\n        traverse(styleOverrides, component);\n      }\n    });\n  }\n  muiTheme.unstable_sxConfig = {\n    ..._mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_9__[\"default\"],\n    ...other?.unstable_sxConfig\n  };\n  muiTheme.unstable_sx = function sx(props) {\n    return (0,_mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_10__[\"default\"])({\n      sx: props,\n      theme: this\n    });\n  };\n  muiTheme.toRuntimeSource = _stringifyTheme_js__WEBPACK_IMPORTED_MODULE_11__.stringifyTheme; // for Pigment CSS integration\n\n  return muiTheme;\n}\nlet warnedOnce = false;\nfunction createMuiTheme(...args) {\n  if (true) {\n    if (!warnedOnce) {\n      warnedOnce = true;\n      console.error(['MUI: the createMuiTheme function was renamed to createTheme.', '', \"You should use `import { createTheme } from '@mui/material/styles'`\"].join('\\n'));\n    }\n  }\n  return createThemeNoVars(...args);\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (createThemeNoVars);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createThemeNoVars.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createThemeWithVars.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createThemeWithVars.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   createGetCssVar: () => (/* binding */ createGetCssVar),\n/* harmony export */   \"default\": () => (/* binding */ createThemeWithVars)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/createGetCssVar.js\");\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createTheme/createSpacing.js\");\n/* harmony import */ var _mui_system_spacing__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/system/spacing */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _mui_system_cssVars__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/system/cssVars */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareTypographyVars.js\");\n/* harmony import */ var _mui_system_cssVars__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/system/cssVars */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/prepareCssVars.js\");\n/* harmony import */ var _mui_system_cssVars__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! @mui/system/cssVars */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/cssVars/getColorSchemeSelector.js\");\n/* harmony import */ var _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/system/styleFunctionSx */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n/* harmony import */ var _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! @mui/system/styleFunctionSx */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/system/colorManipulator */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/colorManipulator/colorManipulator.js\");\n/* harmony import */ var _createThemeNoVars_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./createThemeNoVars.js */ \"./node_modules/@mui/material/styles/createThemeNoVars.js\");\n/* harmony import */ var _createColorScheme_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./createColorScheme.js */ \"./node_modules/@mui/material/styles/createColorScheme.js\");\n/* harmony import */ var _shouldSkipGeneratingVar_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./shouldSkipGeneratingVar.js */ \"./node_modules/@mui/material/styles/shouldSkipGeneratingVar.js\");\n/* harmony import */ var _createGetSelector_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./createGetSelector.js */ \"./node_modules/@mui/material/styles/createGetSelector.js\");\n/* harmony import */ var _stringifyTheme_js__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! ./stringifyTheme.js */ \"./node_modules/@mui/material/styles/stringifyTheme.js\");\n\n\n\n\n\n\n\n\n\n\n\n\nfunction assignNode(obj, keys) {\n  keys.forEach(k => {\n    if (!obj[k]) {\n      obj[k] = {};\n    }\n  });\n}\nfunction setColor(obj, key, defaultValue) {\n  if (!obj[key] && defaultValue) {\n    obj[key] = defaultValue;\n  }\n}\nfunction toRgb(color) {\n  if (typeof color !== 'string' || !color.startsWith('hsl')) {\n    return color;\n  }\n  return (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.hslToRgb)(color);\n}\nfunction setColorChannel(obj, key) {\n  if (!(`${key}Channel` in obj)) {\n    // custom channel token is not provided, generate one.\n    // if channel token can't be generated, show a warning.\n    obj[`${key}Channel`] = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeColorChannel)(toRgb(obj[key]), `MUI: Can't create \\`palette.${key}Channel\\` because \\`palette.${key}\\` is not one of these formats: #nnn, #nnnnnn, rgb(), rgba(), hsl(), hsla(), color().` + '\\n' + `To suppress this warning, you need to explicitly provide the \\`palette.${key}Channel\\` as a string (in rgb format, for example \"12 12 12\") or undefined if you want to remove the channel token.`);\n  }\n}\nfunction getSpacingVal(spacingInput) {\n  if (typeof spacingInput === 'number') {\n    return `${spacingInput}px`;\n  }\n  if (typeof spacingInput === 'string' || typeof spacingInput === 'function' || Array.isArray(spacingInput)) {\n    return spacingInput;\n  }\n  return '8px';\n}\nconst silent = fn => {\n  try {\n    return fn();\n  } catch (error) {\n    // ignore error\n  }\n  return undefined;\n};\nconst createGetCssVar = (cssVarPrefix = 'mui') => (0,_mui_system__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(cssVarPrefix);\nfunction attachColorScheme(colorSchemes, scheme, restTheme, colorScheme) {\n  if (!scheme) {\n    return undefined;\n  }\n  scheme = scheme === true ? {} : scheme;\n  const mode = colorScheme === 'dark' ? 'dark' : 'light';\n  if (!restTheme) {\n    colorSchemes[colorScheme] = (0,_createColorScheme_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n      ...scheme,\n      palette: {\n        mode,\n        ...scheme?.palette\n      }\n    });\n    return undefined;\n  }\n  const {\n    palette,\n    ...muiTheme\n  } = (0,_createThemeNoVars_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n    ...restTheme,\n    palette: {\n      mode,\n      ...scheme?.palette\n    }\n  });\n  colorSchemes[colorScheme] = {\n    ...scheme,\n    palette,\n    opacity: {\n      ...(0,_createColorScheme_js__WEBPACK_IMPORTED_MODULE_2__.getOpacity)(mode),\n      ...scheme?.opacity\n    },\n    overlays: scheme?.overlays || (0,_createColorScheme_js__WEBPACK_IMPORTED_MODULE_2__.getOverlays)(mode)\n  };\n  return muiTheme;\n}\n\n/**\n * A default `createThemeWithVars` comes with a single color scheme, either `light` or `dark` based on the `defaultColorScheme`.\n * This is better suited for apps that only need a single color scheme.\n *\n * To enable built-in `light` and `dark` color schemes, either:\n * 1. provide a `colorSchemeSelector` to define how the color schemes will change.\n * 2. provide `colorSchemes.dark` will set `colorSchemeSelector: 'media'` by default.\n */\nfunction createThemeWithVars(options = {}, ...args) {\n  const {\n    colorSchemes: colorSchemesInput = {\n      light: true\n    },\n    defaultColorScheme: defaultColorSchemeInput,\n    disableCssColorScheme = false,\n    cssVarPrefix = 'mui',\n    shouldSkipGeneratingVar = _shouldSkipGeneratingVar_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"],\n    colorSchemeSelector: selector = colorSchemesInput.light && colorSchemesInput.dark ? 'media' : undefined,\n    rootSelector = ':root',\n    ...input\n  } = options;\n  const firstColorScheme = Object.keys(colorSchemesInput)[0];\n  const defaultColorScheme = defaultColorSchemeInput || (colorSchemesInput.light && firstColorScheme !== 'light' ? 'light' : firstColorScheme);\n  const getCssVar = createGetCssVar(cssVarPrefix);\n  const {\n    [defaultColorScheme]: defaultSchemeInput,\n    light: builtInLight,\n    dark: builtInDark,\n    ...customColorSchemes\n  } = colorSchemesInput;\n  const colorSchemes = {\n    ...customColorSchemes\n  };\n  let defaultScheme = defaultSchemeInput;\n\n  // For built-in light and dark color schemes, ensure that the value is valid if they are the default color scheme.\n  if (defaultColorScheme === 'dark' && !('dark' in colorSchemesInput) || defaultColorScheme === 'light' && !('light' in colorSchemesInput)) {\n    defaultScheme = true;\n  }\n  if (!defaultScheme) {\n    throw new Error( true ? `MUI: The \\`colorSchemes.${defaultColorScheme}\\` option is either missing or invalid.` : 0);\n  }\n\n  // Create the palette for the default color scheme, either `light`, `dark`, or custom color scheme.\n  const muiTheme = attachColorScheme(colorSchemes, defaultScheme, input, defaultColorScheme);\n  if (builtInLight && !colorSchemes.light) {\n    attachColorScheme(colorSchemes, builtInLight, undefined, 'light');\n  }\n  if (builtInDark && !colorSchemes.dark) {\n    attachColorScheme(colorSchemes, builtInDark, undefined, 'dark');\n  }\n  let theme = {\n    defaultColorScheme,\n    ...muiTheme,\n    cssVarPrefix,\n    colorSchemeSelector: selector,\n    rootSelector,\n    getCssVar,\n    colorSchemes,\n    font: {\n      ...(0,_mui_system_cssVars__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(muiTheme.typography),\n      ...muiTheme.font\n    },\n    spacing: getSpacingVal(input.spacing)\n  };\n  Object.keys(theme.colorSchemes).forEach(key => {\n    const palette = theme.colorSchemes[key].palette;\n    const setCssVarColor = cssVar => {\n      const tokens = cssVar.split('-');\n      const color = tokens[1];\n      const colorToken = tokens[2];\n      return getCssVar(cssVar, palette[color][colorToken]);\n    };\n\n    // attach black & white channels to common node\n    if (palette.mode === 'light') {\n      setColor(palette.common, 'background', '#fff');\n      setColor(palette.common, 'onBackground', '#000');\n    }\n    if (palette.mode === 'dark') {\n      setColor(palette.common, 'background', '#000');\n      setColor(palette.common, 'onBackground', '#fff');\n    }\n\n    // assign component variables\n    assignNode(palette, ['Alert', 'AppBar', 'Avatar', 'Button', 'Chip', 'FilledInput', 'LinearProgress', 'Skeleton', 'Slider', 'SnackbarContent', 'SpeedDialAction', 'StepConnector', 'StepContent', 'Switch', 'TableCell', 'Tooltip']);\n    if (palette.mode === 'light') {\n      setColor(palette.Alert, 'errorColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.error.light, 0.6));\n      setColor(palette.Alert, 'infoColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.info.light, 0.6));\n      setColor(palette.Alert, 'successColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.success.light, 0.6));\n      setColor(palette.Alert, 'warningColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.warning.light, 0.6));\n      setColor(palette.Alert, 'errorFilledBg', setCssVarColor('palette-error-main'));\n      setColor(palette.Alert, 'infoFilledBg', setCssVarColor('palette-info-main'));\n      setColor(palette.Alert, 'successFilledBg', setCssVarColor('palette-success-main'));\n      setColor(palette.Alert, 'warningFilledBg', setCssVarColor('palette-warning-main'));\n      setColor(palette.Alert, 'errorFilledColor', silent(() => palette.getContrastText(palette.error.main)));\n      setColor(palette.Alert, 'infoFilledColor', silent(() => palette.getContrastText(palette.info.main)));\n      setColor(palette.Alert, 'successFilledColor', silent(() => palette.getContrastText(palette.success.main)));\n      setColor(palette.Alert, 'warningFilledColor', silent(() => palette.getContrastText(palette.warning.main)));\n      setColor(palette.Alert, 'errorStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.error.light, 0.9));\n      setColor(palette.Alert, 'infoStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.info.light, 0.9));\n      setColor(palette.Alert, 'successStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.success.light, 0.9));\n      setColor(palette.Alert, 'warningStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.warning.light, 0.9));\n      setColor(palette.Alert, 'errorIconColor', setCssVarColor('palette-error-main'));\n      setColor(palette.Alert, 'infoIconColor', setCssVarColor('palette-info-main'));\n      setColor(palette.Alert, 'successIconColor', setCssVarColor('palette-success-main'));\n      setColor(palette.Alert, 'warningIconColor', setCssVarColor('palette-warning-main'));\n      setColor(palette.AppBar, 'defaultBg', setCssVarColor('palette-grey-100'));\n      setColor(palette.Avatar, 'defaultBg', setCssVarColor('palette-grey-400'));\n      setColor(palette.Button, 'inheritContainedBg', setCssVarColor('palette-grey-300'));\n      setColor(palette.Button, 'inheritContainedHoverBg', setCssVarColor('palette-grey-A100'));\n      setColor(palette.Chip, 'defaultBorder', setCssVarColor('palette-grey-400'));\n      setColor(palette.Chip, 'defaultAvatarColor', setCssVarColor('palette-grey-700'));\n      setColor(palette.Chip, 'defaultIconColor', setCssVarColor('palette-grey-700'));\n      setColor(palette.FilledInput, 'bg', 'rgba(0, 0, 0, 0.06)');\n      setColor(palette.FilledInput, 'hoverBg', 'rgba(0, 0, 0, 0.09)');\n      setColor(palette.FilledInput, 'disabledBg', 'rgba(0, 0, 0, 0.12)');\n      setColor(palette.LinearProgress, 'primaryBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.primary.main, 0.62));\n      setColor(palette.LinearProgress, 'secondaryBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.secondary.main, 0.62));\n      setColor(palette.LinearProgress, 'errorBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.error.main, 0.62));\n      setColor(palette.LinearProgress, 'infoBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.info.main, 0.62));\n      setColor(palette.LinearProgress, 'successBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.success.main, 0.62));\n      setColor(palette.LinearProgress, 'warningBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.warning.main, 0.62));\n      setColor(palette.Skeleton, 'bg', `rgba(${setCssVarColor('palette-text-primaryChannel')} / 0.11)`);\n      setColor(palette.Slider, 'primaryTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.primary.main, 0.62));\n      setColor(palette.Slider, 'secondaryTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.secondary.main, 0.62));\n      setColor(palette.Slider, 'errorTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.error.main, 0.62));\n      setColor(palette.Slider, 'infoTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.info.main, 0.62));\n      setColor(palette.Slider, 'successTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.success.main, 0.62));\n      setColor(palette.Slider, 'warningTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.warning.main, 0.62));\n      const snackbarContentBackground = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeEmphasize)(palette.background.default, 0.8);\n      setColor(palette.SnackbarContent, 'bg', snackbarContentBackground);\n      setColor(palette.SnackbarContent, 'color', silent(() => palette.getContrastText(snackbarContentBackground)));\n      setColor(palette.SpeedDialAction, 'fabHoverBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeEmphasize)(palette.background.paper, 0.15));\n      setColor(palette.StepConnector, 'border', setCssVarColor('palette-grey-400'));\n      setColor(palette.StepContent, 'border', setCssVarColor('palette-grey-400'));\n      setColor(palette.Switch, 'defaultColor', setCssVarColor('palette-common-white'));\n      setColor(palette.Switch, 'defaultDisabledColor', setCssVarColor('palette-grey-100'));\n      setColor(palette.Switch, 'primaryDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.primary.main, 0.62));\n      setColor(palette.Switch, 'secondaryDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.secondary.main, 0.62));\n      setColor(palette.Switch, 'errorDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.error.main, 0.62));\n      setColor(palette.Switch, 'infoDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.info.main, 0.62));\n      setColor(palette.Switch, 'successDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.success.main, 0.62));\n      setColor(palette.Switch, 'warningDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.warning.main, 0.62));\n      setColor(palette.TableCell, 'border', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)((0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeAlpha)(palette.divider, 1), 0.88));\n      setColor(palette.Tooltip, 'bg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeAlpha)(palette.grey[700], 0.92));\n    }\n    if (palette.mode === 'dark') {\n      setColor(palette.Alert, 'errorColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.error.light, 0.6));\n      setColor(palette.Alert, 'infoColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.info.light, 0.6));\n      setColor(palette.Alert, 'successColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.success.light, 0.6));\n      setColor(palette.Alert, 'warningColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeLighten)(palette.warning.light, 0.6));\n      setColor(palette.Alert, 'errorFilledBg', setCssVarColor('palette-error-dark'));\n      setColor(palette.Alert, 'infoFilledBg', setCssVarColor('palette-info-dark'));\n      setColor(palette.Alert, 'successFilledBg', setCssVarColor('palette-success-dark'));\n      setColor(palette.Alert, 'warningFilledBg', setCssVarColor('palette-warning-dark'));\n      setColor(palette.Alert, 'errorFilledColor', silent(() => palette.getContrastText(palette.error.dark)));\n      setColor(palette.Alert, 'infoFilledColor', silent(() => palette.getContrastText(palette.info.dark)));\n      setColor(palette.Alert, 'successFilledColor', silent(() => palette.getContrastText(palette.success.dark)));\n      setColor(palette.Alert, 'warningFilledColor', silent(() => palette.getContrastText(palette.warning.dark)));\n      setColor(palette.Alert, 'errorStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.error.light, 0.9));\n      setColor(palette.Alert, 'infoStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.info.light, 0.9));\n      setColor(palette.Alert, 'successStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.success.light, 0.9));\n      setColor(palette.Alert, 'warningStandardBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.warning.light, 0.9));\n      setColor(palette.Alert, 'errorIconColor', setCssVarColor('palette-error-main'));\n      setColor(palette.Alert, 'infoIconColor', setCssVarColor('palette-info-main'));\n      setColor(palette.Alert, 'successIconColor', setCssVarColor('palette-success-main'));\n      setColor(palette.Alert, 'warningIconColor', setCssVarColor('palette-warning-main'));\n      setColor(palette.AppBar, 'defaultBg', setCssVarColor('palette-grey-900'));\n      setColor(palette.AppBar, 'darkBg', setCssVarColor('palette-background-paper')); // specific for dark mode\n      setColor(palette.AppBar, 'darkColor', setCssVarColor('palette-text-primary')); // specific for dark mode\n      setColor(palette.Avatar, 'defaultBg', setCssVarColor('palette-grey-600'));\n      setColor(palette.Button, 'inheritContainedBg', setCssVarColor('palette-grey-800'));\n      setColor(palette.Button, 'inheritContainedHoverBg', setCssVarColor('palette-grey-700'));\n      setColor(palette.Chip, 'defaultBorder', setCssVarColor('palette-grey-700'));\n      setColor(palette.Chip, 'defaultAvatarColor', setCssVarColor('palette-grey-300'));\n      setColor(palette.Chip, 'defaultIconColor', setCssVarColor('palette-grey-300'));\n      setColor(palette.FilledInput, 'bg', 'rgba(255, 255, 255, 0.09)');\n      setColor(palette.FilledInput, 'hoverBg', 'rgba(255, 255, 255, 0.13)');\n      setColor(palette.FilledInput, 'disabledBg', 'rgba(255, 255, 255, 0.12)');\n      setColor(palette.LinearProgress, 'primaryBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.primary.main, 0.5));\n      setColor(palette.LinearProgress, 'secondaryBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.secondary.main, 0.5));\n      setColor(palette.LinearProgress, 'errorBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.error.main, 0.5));\n      setColor(palette.LinearProgress, 'infoBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.info.main, 0.5));\n      setColor(palette.LinearProgress, 'successBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.success.main, 0.5));\n      setColor(palette.LinearProgress, 'warningBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.warning.main, 0.5));\n      setColor(palette.Skeleton, 'bg', `rgba(${setCssVarColor('palette-text-primaryChannel')} / 0.13)`);\n      setColor(palette.Slider, 'primaryTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.primary.main, 0.5));\n      setColor(palette.Slider, 'secondaryTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.secondary.main, 0.5));\n      setColor(palette.Slider, 'errorTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.error.main, 0.5));\n      setColor(palette.Slider, 'infoTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.info.main, 0.5));\n      setColor(palette.Slider, 'successTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.success.main, 0.5));\n      setColor(palette.Slider, 'warningTrack', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.warning.main, 0.5));\n      const snackbarContentBackground = (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeEmphasize)(palette.background.default, 0.98);\n      setColor(palette.SnackbarContent, 'bg', snackbarContentBackground);\n      setColor(palette.SnackbarContent, 'color', silent(() => palette.getContrastText(snackbarContentBackground)));\n      setColor(palette.SpeedDialAction, 'fabHoverBg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeEmphasize)(palette.background.paper, 0.15));\n      setColor(palette.StepConnector, 'border', setCssVarColor('palette-grey-600'));\n      setColor(palette.StepContent, 'border', setCssVarColor('palette-grey-600'));\n      setColor(palette.Switch, 'defaultColor', setCssVarColor('palette-grey-300'));\n      setColor(palette.Switch, 'defaultDisabledColor', setCssVarColor('palette-grey-600'));\n      setColor(palette.Switch, 'primaryDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.primary.main, 0.55));\n      setColor(palette.Switch, 'secondaryDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.secondary.main, 0.55));\n      setColor(palette.Switch, 'errorDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.error.main, 0.55));\n      setColor(palette.Switch, 'infoDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.info.main, 0.55));\n      setColor(palette.Switch, 'successDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.success.main, 0.55));\n      setColor(palette.Switch, 'warningDisabledColor', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)(palette.warning.main, 0.55));\n      setColor(palette.TableCell, 'border', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeDarken)((0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeAlpha)(palette.divider, 1), 0.68));\n      setColor(palette.Tooltip, 'bg', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeAlpha)(palette.grey[700], 0.92));\n    }\n\n    // MUI X - DataGrid needs this token.\n    setColorChannel(palette.background, 'default');\n\n    // added for consistency with the `background.default` token\n    setColorChannel(palette.background, 'paper');\n    setColorChannel(palette.common, 'background');\n    setColorChannel(palette.common, 'onBackground');\n    setColorChannel(palette, 'divider');\n    Object.keys(palette).forEach(color => {\n      const colors = palette[color];\n\n      // The default palettes (primary, secondary, error, info, success, and warning) errors are handled by the above `createTheme(...)`.\n\n      if (color !== 'tonalOffset' && colors && typeof colors === 'object') {\n        // Silent the error for custom palettes.\n        if (colors.main) {\n          setColor(palette[color], 'mainChannel', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeColorChannel)(toRgb(colors.main)));\n        }\n        if (colors.light) {\n          setColor(palette[color], 'lightChannel', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeColorChannel)(toRgb(colors.light)));\n        }\n        if (colors.dark) {\n          setColor(palette[color], 'darkChannel', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeColorChannel)(toRgb(colors.dark)));\n        }\n        if (colors.contrastText) {\n          setColor(palette[color], 'contrastTextChannel', (0,_mui_system_colorManipulator__WEBPACK_IMPORTED_MODULE_0__.private_safeColorChannel)(toRgb(colors.contrastText)));\n        }\n        if (color === 'text') {\n          // Text colors: text.primary, text.secondary\n          setColorChannel(palette[color], 'primary');\n          setColorChannel(palette[color], 'secondary');\n        }\n        if (color === 'action') {\n          // Action colors: action.active, action.selected\n          if (colors.active) {\n            setColorChannel(palette[color], 'active');\n          }\n          if (colors.selected) {\n            setColorChannel(palette[color], 'selected');\n          }\n        }\n      }\n    });\n  });\n  theme = args.reduce((acc, argument) => (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(acc, argument), theme);\n  const parserConfig = {\n    prefix: cssVarPrefix,\n    disableCssColorScheme,\n    shouldSkipGeneratingVar,\n    getSelector: (0,_createGetSelector_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])(theme)\n  };\n  const {\n    vars,\n    generateThemeVars,\n    generateStyleSheets\n  } = (0,_mui_system_cssVars__WEBPACK_IMPORTED_MODULE_8__[\"default\"])(theme, parserConfig);\n  theme.vars = vars;\n  Object.entries(theme.colorSchemes[theme.defaultColorScheme]).forEach(([key, value]) => {\n    theme[key] = value;\n  });\n  theme.generateThemeVars = generateThemeVars;\n  theme.generateStyleSheets = generateStyleSheets;\n  theme.generateSpacing = function generateSpacing() {\n    return (0,_mui_system__WEBPACK_IMPORTED_MODULE_9__[\"default\"])(input.spacing, (0,_mui_system_spacing__WEBPACK_IMPORTED_MODULE_10__.createUnarySpacing)(this));\n  };\n  theme.getColorSchemeSelector = (0,_mui_system_cssVars__WEBPACK_IMPORTED_MODULE_11__.createGetColorSchemeSelector)(selector);\n  theme.spacing = theme.generateSpacing();\n  theme.shouldSkipGeneratingVar = shouldSkipGeneratingVar;\n  theme.unstable_sxConfig = {\n    ..._mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_12__[\"default\"],\n    ...input?.unstable_sxConfig\n  };\n  theme.unstable_sx = function sx(props) {\n    return (0,_mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_13__[\"default\"])({\n      sx: props,\n      theme: this\n    });\n  };\n  theme.toRuntimeSource = _stringifyTheme_js__WEBPACK_IMPORTED_MODULE_14__.stringifyTheme; // for Pigment CSS integration\n\n  return theme;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createThemeWithVars.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createTransitions.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createTransitions.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createTransitions),\n/* harmony export */   duration: () => (/* binding */ duration),\n/* harmony export */   easing: () => (/* binding */ easing)\n/* harmony export */ });\n// Follow https://material.google.com/motion/duration-easing.html#duration-easing-natural-easing-curves\n// to learn the context in which each easing should be used.\nconst easing = {\n  // This is the most common easing curve.\n  easeInOut: 'cubic-bezier(0.4, 0, 0.2, 1)',\n  // Objects enter the screen at full velocity from off-screen and\n  // slowly decelerate to a resting point.\n  easeOut: 'cubic-bezier(0.0, 0, 0.2, 1)',\n  // Objects leave the screen at full velocity. They do not decelerate when off-screen.\n  easeIn: 'cubic-bezier(0.4, 0, 1, 1)',\n  // The sharp curve is used by objects that may return to the screen at any time.\n  sharp: 'cubic-bezier(0.4, 0, 0.6, 1)'\n};\n\n// Follow https://m2.material.io/guidelines/motion/duration-easing.html#duration-easing-common-durations\n// to learn when use what timing\nconst duration = {\n  shortest: 150,\n  shorter: 200,\n  short: 250,\n  // most basic recommended timing\n  standard: 300,\n  // this is to be used in complex animations\n  complex: 375,\n  // recommended when something is entering screen\n  enteringScreen: 225,\n  // recommended when something is leaving screen\n  leavingScreen: 195\n};\nfunction formatMs(milliseconds) {\n  return `${Math.round(milliseconds)}ms`;\n}\nfunction getAutoHeightDuration(height) {\n  if (!height) {\n    return 0;\n  }\n  const constant = height / 36;\n\n  // https://www.desmos.com/calculator/vbrp3ggqet\n  return Math.min(Math.round((4 + 15 * constant ** 0.25 + constant / 5) * 10), 3000);\n}\nfunction createTransitions(inputTransitions) {\n  const mergedEasing = {\n    ...easing,\n    ...inputTransitions.easing\n  };\n  const mergedDuration = {\n    ...duration,\n    ...inputTransitions.duration\n  };\n  const create = (props = ['all'], options = {}) => {\n    const {\n      duration: durationOption = mergedDuration.standard,\n      easing: easingOption = mergedEasing.easeInOut,\n      delay = 0,\n      ...other\n    } = options;\n    if (true) {\n      const isString = value => typeof value === 'string';\n      const isNumber = value => !Number.isNaN(parseFloat(value));\n      if (!isString(props) && !Array.isArray(props)) {\n        console.error('MUI: Argument \"props\" must be a string or Array.');\n      }\n      if (!isNumber(durationOption) && !isString(durationOption)) {\n        console.error(`MUI: Argument \"duration\" must be a number or a string but found ${durationOption}.`);\n      }\n      if (!isString(easingOption)) {\n        console.error('MUI: Argument \"easing\" must be a string.');\n      }\n      if (!isNumber(delay) && !isString(delay)) {\n        console.error('MUI: Argument \"delay\" must be a number or a string.');\n      }\n      if (typeof options !== 'object') {\n        console.error(['MUI: Secong argument of transition.create must be an object.', \"Arguments should be either `create('prop1', options)` or `create(['prop1', 'prop2'], options)`\"].join('\\n'));\n      }\n      if (Object.keys(other).length !== 0) {\n        console.error(`MUI: Unrecognized argument(s) [${Object.keys(other).join(',')}].`);\n      }\n    }\n    return (Array.isArray(props) ? props : [props]).map(animatedProp => `${animatedProp} ${typeof durationOption === 'string' ? durationOption : formatMs(durationOption)} ${easingOption} ${typeof delay === 'string' ? delay : formatMs(delay)}`).join(',');\n  };\n  return {\n    getAutoHeightDuration,\n    create,\n    ...inputTransitions,\n    easing: mergedEasing,\n    duration: mergedDuration\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createTransitions.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/createTypography.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/styles/createTypography.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createTypography)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n\nfunction round(value) {\n  return Math.round(value * 1e5) / 1e5;\n}\nconst caseAllCaps = {\n  textTransform: 'uppercase'\n};\nconst defaultFontFamily = '\"Roboto\", \"Helvetica\", \"Arial\", sans-serif';\n\n/**\n * @see @link{https://m2.material.io/design/typography/the-type-system.html}\n * @see @link{https://m2.material.io/design/typography/understanding-typography.html}\n */\nfunction createTypography(palette, typography) {\n  const {\n    fontFamily = defaultFontFamily,\n    // The default font size of the Material Specification.\n    fontSize = 14,\n    // px\n    fontWeightLight = 300,\n    fontWeightRegular = 400,\n    fontWeightMedium = 500,\n    fontWeightBold = 700,\n    // Tell MUI what's the font-size on the html element.\n    // 16px is the default font-size used by browsers.\n    htmlFontSize = 16,\n    // Apply the CSS properties to all the variants.\n    allVariants,\n    pxToRem: pxToRem2,\n    ...other\n  } = typeof typography === 'function' ? typography(palette) : typography;\n  if (true) {\n    if (typeof fontSize !== 'number') {\n      console.error('MUI: `fontSize` is required to be a number.');\n    }\n    if (typeof htmlFontSize !== 'number') {\n      console.error('MUI: `htmlFontSize` is required to be a number.');\n    }\n  }\n  const coef = fontSize / 14;\n  const pxToRem = pxToRem2 || (size => `${size / htmlFontSize * coef}rem`);\n  const buildVariant = (fontWeight, size, lineHeight, letterSpacing, casing) => ({\n    fontFamily,\n    fontWeight,\n    fontSize: pxToRem(size),\n    // Unitless following https://meyerweb.com/eric/thoughts/2006/02/08/unitless-line-heights/\n    lineHeight,\n    // The letter spacing was designed for the Roboto font-family. Using the same letter-spacing\n    // across font-families can cause issues with the kerning.\n    ...(fontFamily === defaultFontFamily ? {\n      letterSpacing: `${round(letterSpacing / size)}em`\n    } : {}),\n    ...casing,\n    ...allVariants\n  });\n  const variants = {\n    h1: buildVariant(fontWeightLight, 96, 1.167, -1.5),\n    h2: buildVariant(fontWeightLight, 60, 1.2, -0.5),\n    h3: buildVariant(fontWeightRegular, 48, 1.167, 0),\n    h4: buildVariant(fontWeightRegular, 34, 1.235, 0.25),\n    h5: buildVariant(fontWeightRegular, 24, 1.334, 0),\n    h6: buildVariant(fontWeightMedium, 20, 1.6, 0.15),\n    subtitle1: buildVariant(fontWeightRegular, 16, 1.75, 0.15),\n    subtitle2: buildVariant(fontWeightMedium, 14, 1.57, 0.1),\n    body1: buildVariant(fontWeightRegular, 16, 1.5, 0.15),\n    body2: buildVariant(fontWeightRegular, 14, 1.43, 0.15),\n    button: buildVariant(fontWeightMedium, 14, 1.75, 0.4, caseAllCaps),\n    caption: buildVariant(fontWeightRegular, 12, 1.66, 0.4),\n    overline: buildVariant(fontWeightRegular, 12, 2.66, 1, caseAllCaps),\n    // TODO v6: Remove handling of 'inherit' variant from the theme as it is already handled in Material UI's Typography component. Also, remember to remove the associated types.\n    inherit: {\n      fontFamily: 'inherit',\n      fontWeight: 'inherit',\n      fontSize: 'inherit',\n      lineHeight: 'inherit',\n      letterSpacing: 'inherit'\n    }\n  };\n  return (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n    htmlFontSize,\n    pxToRem,\n    fontFamily,\n    fontSize,\n    fontWeightLight,\n    fontWeightRegular,\n    fontWeightMedium,\n    fontWeightBold,\n    ...variants\n  }, other, {\n    clone: false // No need to clone deep\n  });\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/createTypography.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/defaultTheme.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/styles/defaultTheme.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _createTheme_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./createTheme.js */ \"./node_modules/@mui/material/styles/createTheme.js\");\n'use client';\n\n\nconst defaultTheme = (0,_createTheme_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (defaultTheme);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/defaultTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/excludeVariablesFromRoot.js":
/*!***********************************************************************!*\
  !*** ./node_modules/@mui/material/styles/excludeVariablesFromRoot.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/**\n * @internal These variables should not appear in the :root stylesheet when the `defaultColorScheme=\"dark\"`\n */\nconst excludeVariablesFromRoot = cssVarPrefix => [...[...Array(25)].map((_, index) => `--${cssVarPrefix ? `${cssVarPrefix}-` : ''}overlays-${index}`), `--${cssVarPrefix ? `${cssVarPrefix}-` : ''}palette-AppBar-darkBg`, `--${cssVarPrefix ? `${cssVarPrefix}-` : ''}palette-AppBar-darkColor`];\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (excludeVariablesFromRoot);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/excludeVariablesFromRoot.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/getOverlayAlpha.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/styles/getOverlayAlpha.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getOverlayAlpha)\n/* harmony export */ });\n// Inspired by https://github.com/material-components/material-components-ios/blob/bca36107405594d5b7b16265a5b0ed698f85a5ee/components/Elevation/src/UIColor%2BMaterialElevation.m#L61\nfunction getOverlayAlpha(elevation) {\n  let alphaValue;\n  if (elevation < 1) {\n    alphaValue = 5.11916 * elevation ** 2;\n  } else {\n    alphaValue = 4.5 * Math.log(elevation + 1) + 2;\n  }\n  return Math.round(alphaValue * 10) / 1000;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/getOverlayAlpha.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/identifier.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/styles/identifier.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ('$$material');\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/identifier.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/rootShouldForwardProp.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/styles/rootShouldForwardProp.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _slotShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./slotShouldForwardProp.js */ \"./node_modules/@mui/material/styles/slotShouldForwardProp.js\");\n\nconst rootShouldForwardProp = prop => (0,_slotShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(prop) && prop !== 'classes';\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (rootShouldForwardProp);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/rootShouldForwardProp.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/shadows.js":
/*!******************************************************!*\
  !*** ./node_modules/@mui/material/styles/shadows.js ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst shadowKeyUmbraOpacity = 0.2;\nconst shadowKeyPenumbraOpacity = 0.14;\nconst shadowAmbientShadowOpacity = 0.12;\nfunction createShadow(...px) {\n  return [`${px[0]}px ${px[1]}px ${px[2]}px ${px[3]}px rgba(0,0,0,${shadowKeyUmbraOpacity})`, `${px[4]}px ${px[5]}px ${px[6]}px ${px[7]}px rgba(0,0,0,${shadowKeyPenumbraOpacity})`, `${px[8]}px ${px[9]}px ${px[10]}px ${px[11]}px rgba(0,0,0,${shadowAmbientShadowOpacity})`].join(',');\n}\n\n// Values from https://github.com/material-components/material-components-web/blob/be8747f94574669cb5e7add1a7c54fa41a89cec7/packages/mdc-elevation/_variables.scss\nconst shadows = ['none', createShadow(0, 2, 1, -1, 0, 1, 1, 0, 0, 1, 3, 0), createShadow(0, 3, 1, -2, 0, 2, 2, 0, 0, 1, 5, 0), createShadow(0, 3, 3, -2, 0, 3, 4, 0, 0, 1, 8, 0), createShadow(0, 2, 4, -1, 0, 4, 5, 0, 0, 1, 10, 0), createShadow(0, 3, 5, -1, 0, 5, 8, 0, 0, 1, 14, 0), createShadow(0, 3, 5, -1, 0, 6, 10, 0, 0, 1, 18, 0), createShadow(0, 4, 5, -2, 0, 7, 10, 1, 0, 2, 16, 1), createShadow(0, 5, 5, -3, 0, 8, 10, 1, 0, 3, 14, 2), createShadow(0, 5, 6, -3, 0, 9, 12, 1, 0, 3, 16, 2), createShadow(0, 6, 6, -3, 0, 10, 14, 1, 0, 4, 18, 3), createShadow(0, 6, 7, -4, 0, 11, 15, 1, 0, 4, 20, 3), createShadow(0, 7, 8, -4, 0, 12, 17, 2, 0, 5, 22, 4), createShadow(0, 7, 8, -4, 0, 13, 19, 2, 0, 5, 24, 4), createShadow(0, 7, 9, -4, 0, 14, 21, 2, 0, 5, 26, 4), createShadow(0, 8, 9, -5, 0, 15, 22, 2, 0, 6, 28, 5), createShadow(0, 8, 10, -5, 0, 16, 24, 2, 0, 6, 30, 5), createShadow(0, 8, 11, -5, 0, 17, 26, 2, 0, 6, 32, 5), createShadow(0, 9, 11, -5, 0, 18, 28, 2, 0, 7, 34, 6), createShadow(0, 9, 12, -6, 0, 19, 29, 2, 0, 7, 36, 6), createShadow(0, 10, 13, -6, 0, 20, 31, 3, 0, 8, 38, 7), createShadow(0, 10, 13, -6, 0, 21, 33, 3, 0, 8, 40, 7), createShadow(0, 10, 14, -6, 0, 22, 35, 3, 0, 8, 42, 7), createShadow(0, 11, 14, -7, 0, 23, 36, 3, 0, 9, 44, 8), createShadow(0, 11, 15, -7, 0, 24, 38, 3, 0, 9, 46, 8)];\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (shadows);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/shadows.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/shouldSkipGeneratingVar.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/material/styles/shouldSkipGeneratingVar.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ shouldSkipGeneratingVar)\n/* harmony export */ });\nfunction shouldSkipGeneratingVar(keys) {\n  return !!keys[0].match(/(cssVarPrefix|colorSchemeSelector|rootSelector|typography|mixins|breakpoints|direction|transitions)/) || !!keys[0].match(/sxConfig$/) ||\n  // ends with sxConfig\n  keys[0] === 'palette' && !!keys[1]?.match(/(mode|contrastThreshold|tonalOffset)/);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/shouldSkipGeneratingVar.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/slotShouldForwardProp.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/material/styles/slotShouldForwardProp.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n// copied from @mui/system/createStyled\nfunction slotShouldForwardProp(prop) {\n  return prop !== 'ownerState' && prop !== 'theme' && prop !== 'sx' && prop !== 'as';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (slotShouldForwardProp);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/slotShouldForwardProp.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/stringifyTheme.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/styles/stringifyTheme.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   stringifyTheme: () => (/* binding */ stringifyTheme)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* eslint-disable import/prefer-default-export */\n\nfunction isSerializable(val) {\n  return (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__.isPlainObject)(val) || typeof val === 'undefined' || typeof val === 'string' || typeof val === 'boolean' || typeof val === 'number' || Array.isArray(val);\n}\n\n/**\n * `baseTheme` usually comes from `createTheme()` or `extendTheme()`.\n *\n * This function is intended to be used with zero-runtime CSS-in-JS like Pigment CSS\n * For example, in a Next.js project:\n *\n * ```js\n * // next.config.js\n * const { extendTheme } = require('@mui/material/styles');\n *\n * const theme = extendTheme();\n * // `.toRuntimeSource` is Pigment CSS specific to create a theme that is available at runtime.\n * theme.toRuntimeSource = stringifyTheme;\n *\n * module.exports = withPigment({\n *  theme,\n * });\n * ```\n */\nfunction stringifyTheme(baseTheme = {}) {\n  const serializableTheme = {\n    ...baseTheme\n  };\n  function serializeTheme(object) {\n    const array = Object.entries(object);\n    // eslint-disable-next-line no-plusplus\n    for (let index = 0; index < array.length; index++) {\n      const [key, value] = array[index];\n      if (!isSerializable(value) || key.startsWith('unstable_')) {\n        delete object[key];\n      } else if ((0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__.isPlainObject)(value)) {\n        object[key] = {\n          ...value\n        };\n        serializeTheme(object[key]);\n      }\n    }\n  }\n  serializeTheme(serializableTheme);\n  return `import { unstable_createBreakpoints as createBreakpoints, createTransitions } from '@mui/material/styles';\n\nconst theme = ${JSON.stringify(serializableTheme, null, 2)};\n\ntheme.breakpoints = createBreakpoints(theme.breakpoints || {});\ntheme.transitions = createTransitions(theme.transitions || {});\n\nexport default theme;`;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/stringifyTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/styled.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/styles/styled.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   rootShouldForwardProp: () => (/* reexport safe */ _rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"]),\n/* harmony export */   slotShouldForwardProp: () => (/* reexport safe */ _slotShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])\n/* harmony export */ });\n/* harmony import */ var _mui_system_createStyled__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system/createStyled */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/createStyled/createStyled.js\");\n/* harmony import */ var _defaultTheme_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./defaultTheme.js */ \"./node_modules/@mui/material/styles/defaultTheme.js\");\n/* harmony import */ var _identifier_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n/* harmony import */ var _rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./rootShouldForwardProp.js */ \"./node_modules/@mui/material/styles/rootShouldForwardProp.js\");\n/* harmony import */ var _slotShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./slotShouldForwardProp.js */ \"./node_modules/@mui/material/styles/slotShouldForwardProp.js\");\n'use client';\n\n\n\n\n\n\n\nconst styled = (0,_mui_system_createStyled__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n  themeId: _identifier_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n  defaultTheme: _defaultTheme_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"],\n  rootShouldForwardProp: _rootShouldForwardProp_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"]\n});\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (styled);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/styled.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/useTheme.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/material/styles/useTheme.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useTheme)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/useTheme/useTheme.js\");\n/* harmony import */ var _defaultTheme_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./defaultTheme.js */ \"./node_modules/@mui/material/styles/defaultTheme.js\");\n/* harmony import */ var _identifier_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./identifier.js */ \"./node_modules/@mui/material/styles/identifier.js\");\n'use client';\n\n\n\n\n\nfunction useTheme() {\n  const theme = (0,_mui_system__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(_defaultTheme_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]);\n  if (true) {\n    // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler\n    // eslint-disable-next-line react-hooks/rules-of-hooks\n    react__WEBPACK_IMPORTED_MODULE_0__.useDebugValue(theme);\n  }\n  return theme[_identifier_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]] || theme;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/useTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/styles/zIndex.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/styles/zIndex.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n// We need to centralize the zIndex definitions as they work\n// like global values in the browser.\nconst zIndex = {\n  mobileStepper: 1000,\n  fab: 1050,\n  speedDial: 1050,\n  appBar: 1100,\n  drawer: 1200,\n  modal: 1300,\n  snackbar: 1400,\n  tooltip: 1500\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (zIndex);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/styles/zIndex.js?");

/***/ }),

/***/ "./node_modules/@mui/material/transitions/utils.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/transitions/utils.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   getTransitionProps: () => (/* binding */ getTransitionProps),\n/* harmony export */   reflow: () => (/* binding */ reflow)\n/* harmony export */ });\nconst reflow = node => node.scrollTop;\nfunction getTransitionProps(props, options) {\n  const {\n    timeout,\n    easing,\n    style = {}\n  } = props;\n  return {\n    duration: style.transitionDuration ?? (typeof timeout === 'number' ? timeout : timeout[options.mode] || 0),\n    easing: style.transitionTimingFunction ?? (typeof easing === 'object' ? easing[options.mode] : easing),\n    delay: style.transitionDelay\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/transitions/utils.js?");

/***/ }),

/***/ "./node_modules/@mui/material/useLazyRipple/useLazyRipple.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@mui/material/useLazyRipple/useLazyRipple.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   LazyRipple: () => (/* binding */ LazyRipple),\n/* harmony export */   \"default\": () => (/* binding */ useLazyRipple)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_utils_useLazyRef__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/useLazyRef */ \"./node_modules/@mui/utils/esm/useLazyRef/useLazyRef.js\");\n'use client';\n\n\n\n/**\n * Lazy initialization container for the Ripple instance. This improves\n * performance by delaying mounting the ripple until it's needed.\n */\nclass LazyRipple {\n  /** React ref to the ripple instance */\n\n  /** If the ripple component should be mounted */\n\n  /** Promise that resolves when the ripple component is mounted */\n\n  /** If the ripple component has been mounted */\n\n  /** React state hook setter */\n\n  static create() {\n    return new LazyRipple();\n  }\n  static use() {\n    /* eslint-disable */\n    const ripple = (0,_mui_utils_useLazyRef__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(LazyRipple.create).current;\n    const [shouldMount, setShouldMount] = react__WEBPACK_IMPORTED_MODULE_0__.useState(false);\n    ripple.shouldMount = shouldMount;\n    ripple.setShouldMount = setShouldMount;\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(ripple.mountEffect, [shouldMount]);\n    /* eslint-enable */\n\n    return ripple;\n  }\n  constructor() {\n    this.ref = {\n      current: null\n    };\n    this.mounted = null;\n    this.didMount = false;\n    this.shouldMount = false;\n    this.setShouldMount = null;\n  }\n  mount() {\n    if (!this.mounted) {\n      this.mounted = createControlledPromise();\n      this.shouldMount = true;\n      this.setShouldMount(this.shouldMount);\n    }\n    return this.mounted;\n  }\n  mountEffect = () => {\n    if (this.shouldMount && !this.didMount) {\n      if (this.ref.current !== null) {\n        this.didMount = true;\n        this.mounted.resolve();\n      }\n    }\n  };\n\n  /* Ripple API */\n\n  start(...args) {\n    this.mount().then(() => this.ref.current?.start(...args));\n  }\n  stop(...args) {\n    this.mount().then(() => this.ref.current?.stop(...args));\n  }\n  pulsate(...args) {\n    this.mount().then(() => this.ref.current?.pulsate(...args));\n  }\n}\nfunction useLazyRipple() {\n  return LazyRipple.use();\n}\nfunction createControlledPromise() {\n  let resolve;\n  let reject;\n  const p = new Promise((resolveFn, rejectFn) => {\n    resolve = resolveFn;\n    reject = rejectFn;\n  });\n  p.resolve = resolve;\n  p.reject = reject;\n  return p;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/useLazyRipple/useLazyRipple.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/capitalize.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/utils/capitalize.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/capitalize.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js":
/*!****************************************************************************!*\
  !*** ./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js ***!
  \****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createSimplePaletteValueFilter)\n/* harmony export */ });\n/**\n * Type guard to check if the object has a \"main\" property of type string.\n *\n * @param obj - the object to check\n * @returns boolean\n */\nfunction hasCorrectMainProperty(obj) {\n  return typeof obj.main === 'string';\n}\n/**\n * Checks if the object conforms to the SimplePaletteColorOptions type.\n * The minimum requirement is that the object has a \"main\" property of type string, this is always checked.\n * Optionally, you can pass additional properties to check.\n *\n * @param obj - The object to check\n * @param additionalPropertiesToCheck - Array containing \"light\", \"dark\", and/or \"contrastText\"\n * @returns boolean\n */\nfunction checkSimplePaletteColorValues(obj, additionalPropertiesToCheck = []) {\n  if (!hasCorrectMainProperty(obj)) {\n    return false;\n  }\n  for (const value of additionalPropertiesToCheck) {\n    if (!obj.hasOwnProperty(value) || typeof obj[value] !== 'string') {\n      return false;\n    }\n  }\n  return true;\n}\n\n/**\n * Creates a filter function used to filter simple palette color options.\n * The minimum requirement is that the object has a \"main\" property of type string, this is always checked.\n * Optionally, you can pass additional properties to check.\n *\n * @param additionalPropertiesToCheck - Array containing \"light\", \"dark\", and/or \"contrastText\"\n * @returns ([, value]: [any, PaletteColorOptions]) => boolean\n */\nfunction createSimplePaletteValueFilter(additionalPropertiesToCheck = []) {\n  return ([, value]) => value && checkSimplePaletteColorValues(value, additionalPropertiesToCheck);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/createSimplePaletteValueFilter.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/createSvgIcon.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/utils/createSvgIcon.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createSvgIcon)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _SvgIcon_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../SvgIcon/index.js */ \"./node_modules/@mui/material/SvgIcon/SvgIcon.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n/**\n * Private module reserved for @mui packages.\n */\n\nfunction createSvgIcon(path, displayName) {\n  function Component(props, ref) {\n    return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_SvgIcon_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"], {\n      \"data-testid\": `${displayName}Icon`,\n      ref: ref,\n      ...props,\n      children: path\n    });\n  }\n  if (true) {\n    // Need to set `displayName` on the inner component for React.memo.\n    // React prior to 16.14 ignores `displayName` on the wrapper.\n    Component.displayName = `${displayName}Icon`;\n  }\n  Component.muiName = _SvgIcon_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"].muiName;\n  return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.memo(/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.forwardRef(Component));\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/createSvgIcon.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/debounce.js":
/*!******************************************************!*\
  !*** ./node_modules/@mui/material/utils/debounce.js ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_debounce__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/debounce */ \"./node_modules/@mui/utils/esm/debounce/debounce.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_debounce__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/debounce.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/getScrollbarSize.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/utils/getScrollbarSize.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_getScrollbarSize__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/getScrollbarSize */ \"./node_modules/@mui/utils/esm/getScrollbarSize/getScrollbarSize.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_getScrollbarSize__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/getScrollbarSize.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/isHostComponent.js":
/*!*************************************************************!*\
  !*** ./node_modules/@mui/material/utils/isHostComponent.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/**\n * Determines if a given element is a DOM element name (i.e. not a React component).\n */\nfunction isHostComponent(element) {\n  return typeof element === 'string';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (isHostComponent);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/isHostComponent.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/isMuiElement.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/material/utils/isMuiElement.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_isMuiElement__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/isMuiElement */ \"./node_modules/@mui/utils/esm/isMuiElement/isMuiElement.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_isMuiElement__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/isMuiElement.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/memoTheme.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/material/utils/memoTheme.js ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/memoTheme.js\");\n\nconst memoTheme = _mui_system__WEBPACK_IMPORTED_MODULE_0__[\"default\"];\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (memoTheme);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/memoTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/mergeSlotProps.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/material/utils/mergeSlotProps.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ mergeSlotProps)\n/* harmony export */ });\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n\nfunction mergeSlotProps(externalSlotProps, defaultSlotProps) {\n  if (!externalSlotProps) {\n    return defaultSlotProps;\n  }\n  if (typeof externalSlotProps === 'function' || typeof defaultSlotProps === 'function') {\n    return ownerState => {\n      const defaultSlotPropsValue = typeof defaultSlotProps === 'function' ? defaultSlotProps(ownerState) : defaultSlotProps;\n      const externalSlotPropsValue = typeof externalSlotProps === 'function' ? externalSlotProps({\n        ...ownerState,\n        ...defaultSlotPropsValue\n      }) : externalSlotProps;\n      const className = (0,clsx__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(ownerState?.className, defaultSlotPropsValue?.className, externalSlotPropsValue?.className);\n      return {\n        ...defaultSlotPropsValue,\n        ...externalSlotPropsValue,\n        ...(!!className && {\n          className\n        }),\n        ...(defaultSlotPropsValue?.style && externalSlotPropsValue?.style && {\n          style: {\n            ...defaultSlotPropsValue.style,\n            ...externalSlotPropsValue.style\n          }\n        }),\n        ...(defaultSlotPropsValue?.sx && externalSlotPropsValue?.sx && {\n          sx: [...(Array.isArray(defaultSlotPropsValue.sx) ? defaultSlotPropsValue.sx : [defaultSlotPropsValue.sx]), ...(Array.isArray(externalSlotPropsValue.sx) ? externalSlotPropsValue.sx : [externalSlotPropsValue.sx])]\n        })\n      };\n    };\n  }\n  const typedDefaultSlotProps = defaultSlotProps;\n  const className = (0,clsx__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(typedDefaultSlotProps?.className, externalSlotProps?.className);\n  return {\n    ...defaultSlotProps,\n    ...externalSlotProps,\n    ...(!!className && {\n      className\n    }),\n    ...(typedDefaultSlotProps?.style && externalSlotProps?.style && {\n      style: {\n        ...typedDefaultSlotProps.style,\n        ...externalSlotProps.style\n      }\n    }),\n    ...(typedDefaultSlotProps?.sx && externalSlotProps?.sx && {\n      sx: [...(Array.isArray(typedDefaultSlotProps.sx) ? typedDefaultSlotProps.sx : [typedDefaultSlotProps.sx]), ...(Array.isArray(externalSlotProps.sx) ? externalSlotProps.sx : [externalSlotProps.sx])]\n    })\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/mergeSlotProps.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/ownerDocument.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/utils/ownerDocument.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_ownerDocument__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/ownerDocument */ \"./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_ownerDocument__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/ownerDocument.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/ownerWindow.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/utils/ownerWindow.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_ownerWindow__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/ownerWindow */ \"./node_modules/@mui/utils/esm/ownerWindow/ownerWindow.js\");\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_ownerWindow__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/ownerWindow.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/useControlled.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/material/utils/useControlled.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_useControlled__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/useControlled */ \"./node_modules/@mui/utils/esm/useControlled/useControlled.js\");\n'use client';\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_useControlled__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/useControlled.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/useEnhancedEffect.js":
/*!***************************************************************!*\
  !*** ./node_modules/@mui/material/utils/useEnhancedEffect.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_useEnhancedEffect__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/useEnhancedEffect */ \"./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js\");\n'use client';\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_useEnhancedEffect__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/useEnhancedEffect.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/useEventCallback.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/material/utils/useEventCallback.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_useEventCallback__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/useEventCallback */ \"./node_modules/@mui/utils/esm/useEventCallback/useEventCallback.js\");\n'use client';\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_useEventCallback__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/useEventCallback.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/useForkRef.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/material/utils/useForkRef.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_useForkRef__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/useForkRef */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n'use client';\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_useForkRef__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/useForkRef.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/useId.js":
/*!***************************************************!*\
  !*** ./node_modules/@mui/material/utils/useId.js ***!
  \***************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_useId__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/useId */ \"./node_modules/@mui/utils/esm/useId/useId.js\");\n'use client';\n\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_mui_utils_useId__WEBPACK_IMPORTED_MODULE_0__[\"default\"]);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/useId.js?");

/***/ }),

/***/ "./node_modules/@mui/material/utils/useSlot.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/material/utils/useSlot.js ***!
  \*****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useSlot)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_useForkRef__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/useForkRef */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n/* harmony import */ var _mui_utils_appendOwnerState__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/appendOwnerState */ \"./node_modules/@mui/utils/esm/appendOwnerState/appendOwnerState.js\");\n/* harmony import */ var _mui_utils_resolveComponentProps__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/resolveComponentProps */ \"./node_modules/@mui/utils/esm/resolveComponentProps/resolveComponentProps.js\");\n/* harmony import */ var _mui_utils_mergeSlotProps__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/mergeSlotProps */ \"./node_modules/@mui/utils/esm/mergeSlotProps/mergeSlotProps.js\");\n'use client';\n\n\n\n\n\n/**\n * An internal function to create a Material UI slot.\n *\n * This is an advanced version of Base UI `useSlotProps` because Material UI allows leaf component to be customized via `component` prop\n * while Base UI does not need to support leaf component customization.\n *\n * @param {string} name: name of the slot\n * @param {object} parameters\n * @returns {[Slot, slotProps]} The slot's React component and the slot's props\n *\n * Note: the returned slot's props\n * - will never contain `component` prop.\n * - might contain `as` prop.\n */\nfunction useSlot(\n/**\n * The slot's name. All Material UI components should have `root` slot.\n *\n * If the name is `root`, the logic behaves differently from other slots,\n * e.g. the `externalForwardedProps` are spread to `root` slot but not other slots.\n */\nname, parameters) {\n  const {\n    className,\n    elementType: initialElementType,\n    ownerState,\n    externalForwardedProps,\n    internalForwardedProps,\n    shouldForwardComponentProp = false,\n    ...useSlotPropsParams\n  } = parameters;\n  const {\n    component: rootComponent,\n    slots = {\n      [name]: undefined\n    },\n    slotProps = {\n      [name]: undefined\n    },\n    ...other\n  } = externalForwardedProps;\n  const elementType = slots[name] || initialElementType;\n\n  // `slotProps[name]` can be a callback that receives the component's ownerState.\n  // `resolvedComponentsProps` is always a plain object.\n  const resolvedComponentsProps = (0,_mui_utils_resolveComponentProps__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(slotProps[name], ownerState);\n  const {\n    props: {\n      component: slotComponent,\n      ...mergedProps\n    },\n    internalRef\n  } = (0,_mui_utils_mergeSlotProps__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n    className,\n    ...useSlotPropsParams,\n    externalForwardedProps: name === 'root' ? other : undefined,\n    externalSlotProps: resolvedComponentsProps\n  });\n  const ref = (0,_mui_utils_useForkRef__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(internalRef, resolvedComponentsProps?.ref, parameters.ref);\n  const LeafComponent = name === 'root' ? slotComponent || rootComponent : slotComponent;\n  const props = (0,_mui_utils_appendOwnerState__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(elementType, {\n    ...(name === 'root' && !rootComponent && !slots[name] && internalForwardedProps),\n    ...(name !== 'root' && !slots[name] && internalForwardedProps),\n    ...mergedProps,\n    ...(LeafComponent && !shouldForwardComponentProp && {\n      as: LeafComponent\n    }),\n    ...(LeafComponent && shouldForwardComponentProp && {\n      component: LeafComponent\n    }),\n    ref\n  }, ownerState);\n  return [elementType, props];\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/utils/useSlot.js?");

/***/ }),

/***/ "./node_modules/@mui/material/zero-styled/index.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/material/zero-styled/index.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   css: () => (/* reexport safe */ _mui_system__WEBPACK_IMPORTED_MODULE_2__.css),\n/* harmony export */   globalCss: () => (/* binding */ globalCss),\n/* harmony export */   internal_createExtendSxProp: () => (/* binding */ internal_createExtendSxProp),\n/* harmony export */   keyframes: () => (/* reexport safe */ _mui_system__WEBPACK_IMPORTED_MODULE_2__.keyframes),\n/* harmony export */   styled: () => (/* reexport safe */ _styles_styled_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]),\n/* harmony export */   useTheme: () => (/* reexport safe */ _styles_useTheme_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"])\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/system/styleFunctionSx */ \"./node_modules/@mui/material/node_modules/@mui/system/esm/styleFunctionSx/extendSxProp.js\");\n/* harmony import */ var _styles_useTheme_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../styles/useTheme.js */ \"./node_modules/@mui/material/styles/useTheme.js\");\n/* harmony import */ var _GlobalStyles_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../GlobalStyles/index.js */ \"./node_modules/@mui/material/GlobalStyles/GlobalStyles.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var _styles_styled_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../styles/styled.js */ \"./node_modules/@mui/material/styles/styled.js\");\n\n\n\n\n\n\n\nfunction globalCss(styles) {\n  return function GlobalStylesWrapper(props) {\n    return (\n      /*#__PURE__*/\n      // Pigment CSS `globalCss` support callback with theme inside an object but `GlobalStyles` support theme as a callback value.\n      (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_GlobalStyles_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"], {\n        styles: typeof styles === 'function' ? theme => styles({\n          theme,\n          ...props\n        }) : styles\n      })\n    );\n  };\n}\n\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction internal_createExtendSxProp() {\n  return _mui_system_styleFunctionSx__WEBPACK_IMPORTED_MODULE_5__[\"default\"];\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/material/zero-styled/index.js?");

/***/ }),

/***/ "./node_modules/@mui/styled-engine/esm/GlobalStyles/GlobalStyles.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/styled-engine/esm/GlobalStyles/GlobalStyles.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ GlobalStyles)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\nfunction isEmpty(obj) {\n  return obj === undefined || obj === null || Object.keys(obj).length === 0;\n}\nfunction GlobalStyles(props) {\n  const {\n    styles,\n    defaultTheme = {}\n  } = props;\n  const globalStyles = typeof styles === 'function' ? themeInput => styles(isEmpty(themeInput) ? defaultTheme : themeInput) : styles;\n  return /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_1__.jsx)(_emotion_react__WEBPACK_IMPORTED_MODULE_2__.Global, {\n    styles: globalStyles\n  });\n}\n true ? GlobalStyles.propTypes = {\n  defaultTheme: prop_types__WEBPACK_IMPORTED_MODULE_3__.object,\n  styles: prop_types__WEBPACK_IMPORTED_MODULE_3__.oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_3__.array, prop_types__WEBPACK_IMPORTED_MODULE_3__.string, prop_types__WEBPACK_IMPORTED_MODULE_3__.object, prop_types__WEBPACK_IMPORTED_MODULE_3__.func])\n} : 0;\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/styled-engine/esm/GlobalStyles/GlobalStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/styled-engine/esm/StyledEngineProvider/StyledEngineProvider.js":
/*!******************************************************************************************!*\
  !*** ./node_modules/@mui/styled-engine/esm/StyledEngineProvider/StyledEngineProvider.js ***!
  \******************************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   TEST_INTERNALS_DO_NOT_USE: () => (/* binding */ TEST_INTERNALS_DO_NOT_USE),\n/* harmony export */   \"default\": () => (/* binding */ StyledEngineProvider)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var _emotion_cache__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/cache */ \"./node_modules/@emotion/cache/dist/emotion-cache.browser.development.esm.js\");\n/* harmony import */ var _emotion_sheet__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/sheet */ \"./node_modules/@emotion/sheet/dist/emotion-sheet.development.esm.js\");\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n'use client';\n\n\n\n\n\n\n\n// To fix [Jest performance](https://github.com/mui/material-ui/issues/45638).\n\nconst cacheMap = new Map();\n\n// Need to add a private variable to test the generated CSS from Emotion, this is the simplest way to do it.\n// We can't test the CSS from `style` tag easily because the `speedy: true` (produce empty text content) is enabled by Emotion.\n// Even if we disable it, JSDOM needs extra configuration to be able to parse `@layer` CSS.\nconst TEST_INTERNALS_DO_NOT_USE = {\n  /**\n   * to intercept the generated CSS before inserting to the style tag, so that we can check the generated CSS.\n   *\n   * let rule;\n   * TEST_INTERNALS_DO_NOT_USE.insert = (...args) => {\n   *    rule = args[0];\n   * };\n   *\n   * expect(rule).to.equal(...);\n   */\n  insert: undefined\n};\n\n// We might be able to remove this when this issue is fixed:\n// https://github.com/emotion-js/emotion/issues/2790\nconst createEmotionCache = (options, CustomSheet) => {\n  const cache = (0,_emotion_cache__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(options);\n\n  // Do the same as https://github.com/emotion-js/emotion/blob/main/packages/cache/src/index.js#L238-L245\n  cache.sheet = new CustomSheet({\n    key: cache.key,\n    nonce: cache.sheet.nonce,\n    container: cache.sheet.container,\n    speedy: cache.sheet.isSpeedy,\n    prepend: cache.sheet.prepend,\n    insertionPoint: cache.sheet.insertionPoint\n  });\n  return cache;\n};\nlet insertionPoint;\nif (typeof document === 'object') {\n  // Use `insertionPoint` over `prepend`(deprecated) because it can be controlled for GlobalStyles injection order\n  // For more information, see https://github.com/mui/material-ui/issues/44597\n  insertionPoint = document.querySelector('[name=\"emotion-insertion-point\"]');\n  if (!insertionPoint) {\n    insertionPoint = document.createElement('meta');\n    insertionPoint.setAttribute('name', 'emotion-insertion-point');\n    insertionPoint.setAttribute('content', '');\n    const head = document.querySelector('head');\n    if (head) {\n      head.prepend(insertionPoint);\n    }\n  }\n}\nfunction getCache(injectFirst, enableCssLayer) {\n  if (injectFirst || enableCssLayer) {\n    /**\n     * This is for client-side apps only.\n     * A custom sheet is required to make the GlobalStyles API injected above the insertion point.\n     * This is because the [sheet](https://github.com/emotion-js/emotion/blob/main/packages/react/src/global.js#L94-L99) does not consume the options.\n     */\n    class MyStyleSheet extends _emotion_sheet__WEBPACK_IMPORTED_MODULE_2__.StyleSheet {\n      insert(rule, options) {\n        if (TEST_INTERNALS_DO_NOT_USE.insert) {\n          return TEST_INTERNALS_DO_NOT_USE.insert(rule, options);\n        }\n        if (this.key && this.key.endsWith('global')) {\n          this.before = insertionPoint;\n        }\n        return super.insert(rule, options);\n      }\n    }\n    const emotionCache = createEmotionCache({\n      key: 'css',\n      insertionPoint: injectFirst ? insertionPoint : undefined\n    }, MyStyleSheet);\n    if (enableCssLayer) {\n      const prevInsert = emotionCache.insert;\n      emotionCache.insert = (...args) => {\n        if (!args[1].styles.startsWith('@layer')) {\n          // avoid nested @layer\n          args[1].styles = `@layer mui {${args[1].styles}}`;\n        }\n        return prevInsert(...args);\n      };\n    }\n    return emotionCache;\n  }\n  return undefined;\n}\nfunction StyledEngineProvider(props) {\n  const {\n    injectFirst,\n    enableCssLayer,\n    children\n  } = props;\n  const cache = react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    const cacheKey = `${injectFirst}-${enableCssLayer}`;\n    if (cacheMap.has(cacheKey)) {\n      return cacheMap.get(cacheKey);\n    }\n    const fresh = getCache(injectFirst, enableCssLayer);\n    cacheMap.set(cacheKey, fresh);\n    return fresh;\n  }, [injectFirst, enableCssLayer]);\n  return cache ? /*#__PURE__*/(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_3__.jsx)(_emotion_react__WEBPACK_IMPORTED_MODULE_4__.C, {\n    value: cache,\n    children: children\n  }) : children;\n}\n true ? StyledEngineProvider.propTypes = {\n  /**\n   * Your component tree.\n   */\n  children: prop_types__WEBPACK_IMPORTED_MODULE_5__.node,\n  /**\n   * If `true`, the styles are wrapped in `@layer mui`.\n   * Learn more about [Cascade layers](https://developer.mozilla.org/en-US/docs/Learn_web_development/Core/Styling_basics/Cascade_layers).\n   */\n  enableCssLayer: prop_types__WEBPACK_IMPORTED_MODULE_5__.bool,\n  /**\n   * By default, the styles are injected last in the <head> element of the page.\n   * As a result, they gain more specificity than any other style sheet.\n   * If you want to override MUI's styles, set this prop.\n   */\n  injectFirst: prop_types__WEBPACK_IMPORTED_MODULE_5__.bool\n} : 0;\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/styled-engine/esm/StyledEngineProvider/StyledEngineProvider.js?");

/***/ }),

/***/ "./node_modules/@mui/styled-engine/esm/index.js":
/*!******************************************************!*\
  !*** ./node_modules/@mui/styled-engine/esm/index.js ***!
  \******************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   GlobalStyles: () => (/* reexport safe */ _GlobalStyles_index_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"]),\n/* harmony export */   StyledEngineProvider: () => (/* reexport safe */ _StyledEngineProvider_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"]),\n/* harmony export */   ThemeContext: () => (/* reexport safe */ _emotion_react__WEBPACK_IMPORTED_MODULE_2__.T),\n/* harmony export */   css: () => (/* reexport safe */ _emotion_react__WEBPACK_IMPORTED_MODULE_3__.css),\n/* harmony export */   \"default\": () => (/* binding */ styled),\n/* harmony export */   internal_mutateStyles: () => (/* binding */ internal_mutateStyles),\n/* harmony export */   internal_serializeStyles: () => (/* binding */ internal_serializeStyles),\n/* harmony export */   keyframes: () => (/* reexport safe */ _emotion_react__WEBPACK_IMPORTED_MODULE_3__.keyframes)\n/* harmony export */ });\n/* harmony import */ var _emotion_styled__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @emotion/styled */ \"./node_modules/@emotion/styled/dist/emotion-styled.browser.development.esm.js\");\n/* harmony import */ var _emotion_serialize__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @emotion/serialize */ \"./node_modules/@emotion/serialize/dist/emotion-serialize.development.esm.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-element-489459f2.browser.development.esm.js\");\n/* harmony import */ var _emotion_react__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @emotion/react */ \"./node_modules/@emotion/react/dist/emotion-react.browser.development.esm.js\");\n/* harmony import */ var _StyledEngineProvider_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./StyledEngineProvider/index.js */ \"./node_modules/@mui/styled-engine/esm/StyledEngineProvider/StyledEngineProvider.js\");\n/* harmony import */ var _GlobalStyles_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./GlobalStyles/index.js */ \"./node_modules/@mui/styled-engine/esm/GlobalStyles/GlobalStyles.js\");\n/**\n * @mui/styled-engine v7.1.0\n *\n * @license MIT\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n'use client';\n\n/* eslint-disable no-underscore-dangle */\n\n\nfunction styled(tag, options) {\n  const stylesFactory = (0,_emotion_styled__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(tag, options);\n  if (true) {\n    return (...styles) => {\n      const component = typeof tag === 'string' ? `\"${tag}\"` : 'component';\n      if (styles.length === 0) {\n        console.error([`MUI: Seems like you called \\`styled(${component})()\\` without a \\`style\\` argument.`, 'You must provide a `styles` argument: `styled(\"div\")(styleYouForgotToPass)`.'].join('\\n'));\n      } else if (styles.some(style => style === undefined)) {\n        console.error(`MUI: the styled(${component})(...args) API requires all its args to be defined.`);\n      }\n      return stylesFactory(...styles);\n    };\n  }\n  // removed by dead control flow\n{}\n}\n\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction internal_mutateStyles(tag, processor) {\n  // Emotion attaches all the styles as `__emotion_styles`.\n  // Ref: https://github.com/emotion-js/emotion/blob/16d971d0da229596d6bcc39d282ba9753c9ee7cf/packages/styled/src/base.js#L186\n  if (Array.isArray(tag.__emotion_styles)) {\n    tag.__emotion_styles = processor(tag.__emotion_styles);\n  }\n}\n\n// Emotion only accepts an array, but we want to avoid allocations\nconst wrapper = [];\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction internal_serializeStyles(styles) {\n  wrapper[0] = styles;\n  return (0,_emotion_serialize__WEBPACK_IMPORTED_MODULE_1__.serializeStyles)(wrapper);\n}\n\n\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/styled-engine/esm/index.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/borders/borders.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/system/esm/borders/borders.js ***!
  \*********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   border: () => (/* binding */ border),\n/* harmony export */   borderBottom: () => (/* binding */ borderBottom),\n/* harmony export */   borderBottomColor: () => (/* binding */ borderBottomColor),\n/* harmony export */   borderColor: () => (/* binding */ borderColor),\n/* harmony export */   borderLeft: () => (/* binding */ borderLeft),\n/* harmony export */   borderLeftColor: () => (/* binding */ borderLeftColor),\n/* harmony export */   borderRadius: () => (/* binding */ borderRadius),\n/* harmony export */   borderRight: () => (/* binding */ borderRight),\n/* harmony export */   borderRightColor: () => (/* binding */ borderRightColor),\n/* harmony export */   borderTop: () => (/* binding */ borderTop),\n/* harmony export */   borderTopColor: () => (/* binding */ borderTopColor),\n/* harmony export */   borderTransform: () => (/* binding */ borderTransform),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   outline: () => (/* binding */ outline),\n/* harmony export */   outlineColor: () => (/* binding */ outlineColor)\n/* harmony export */ });\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/system/esm/compose/compose.js\");\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n\n\n\n\n\nfunction borderTransform(value) {\n  if (typeof value !== 'number') {\n    return value;\n  }\n  return `${value}px solid`;\n}\nfunction createBorderStyle(prop, transform) {\n  return (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n    prop,\n    themeKey: 'borders',\n    transform\n  });\n}\nconst border = createBorderStyle('border', borderTransform);\nconst borderTop = createBorderStyle('borderTop', borderTransform);\nconst borderRight = createBorderStyle('borderRight', borderTransform);\nconst borderBottom = createBorderStyle('borderBottom', borderTransform);\nconst borderLeft = createBorderStyle('borderLeft', borderTransform);\nconst borderColor = createBorderStyle('borderColor');\nconst borderTopColor = createBorderStyle('borderTopColor');\nconst borderRightColor = createBorderStyle('borderRightColor');\nconst borderBottomColor = createBorderStyle('borderBottomColor');\nconst borderLeftColor = createBorderStyle('borderLeftColor');\nconst outline = createBorderStyle('outline', borderTransform);\nconst outlineColor = createBorderStyle('outlineColor');\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst borderRadius = props => {\n  if (props.borderRadius !== undefined && props.borderRadius !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_1__.createUnaryUnit)(props.theme, 'shape.borderRadius', 4, 'borderRadius');\n    const styleFromPropValue = propValue => ({\n      borderRadius: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_1__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)(props, props.borderRadius, styleFromPropValue);\n  }\n  return null;\n};\nborderRadius.propTypes =  true ? {\n  borderRadius: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"]\n} : 0;\nborderRadius.filterProps = ['borderRadius'];\nconst borders = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(border, borderTop, borderRight, borderBottom, borderLeft, borderColor, borderTopColor, borderRightColor, borderBottomColor, borderLeftColor, borderRadius, outline, outlineColor);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (borders);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/borders/borders.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/breakpoints/breakpoints.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@mui/system/esm/breakpoints/breakpoints.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   computeBreakpointsBase: () => (/* binding */ computeBreakpointsBase),\n/* harmony export */   createEmptyBreakpointObject: () => (/* binding */ createEmptyBreakpointObject),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   handleBreakpoints: () => (/* binding */ handleBreakpoints),\n/* harmony export */   mergeBreakpointsInOrder: () => (/* binding */ mergeBreakpointsInOrder),\n/* harmony export */   removeUnusedBreakpoints: () => (/* binding */ removeUnusedBreakpoints),\n/* harmony export */   resolveBreakpointValues: () => (/* binding */ resolveBreakpointValues),\n/* harmony export */   values: () => (/* binding */ values)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/system/esm/merge/merge.js\");\n/* harmony import */ var _cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../cssContainerQueries/index.js */ \"./node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js\");\n\n\n\n\n\n// The breakpoint **start** at this value.\n// For instance with the first breakpoint xs: [xs, sm[.\nconst values = {\n  xs: 0,\n  // phone\n  sm: 600,\n  // tablet\n  md: 900,\n  // small laptop\n  lg: 1200,\n  // desktop\n  xl: 1536 // large screen\n};\nconst defaultBreakpoints = {\n  // Sorted ASC by size. That's important.\n  // It can't be configured as it's used statically for propTypes.\n  keys: ['xs', 'sm', 'md', 'lg', 'xl'],\n  up: key => `@media (min-width:${values[key]}px)`\n};\nconst defaultContainerQueries = {\n  containerQueries: containerName => ({\n    up: key => {\n      let result = typeof key === 'number' ? key : values[key] || key;\n      if (typeof result === 'number') {\n        result = `${result}px`;\n      }\n      return containerName ? `@container ${containerName} (min-width:${result})` : `@container (min-width:${result})`;\n    }\n  })\n};\nfunction handleBreakpoints(props, propValue, styleFromPropValue) {\n  const theme = props.theme || {};\n  if (Array.isArray(propValue)) {\n    const themeBreakpoints = theme.breakpoints || defaultBreakpoints;\n    return propValue.reduce((acc, item, index) => {\n      acc[themeBreakpoints.up(themeBreakpoints.keys[index])] = styleFromPropValue(propValue[index]);\n      return acc;\n    }, {});\n  }\n  if (typeof propValue === 'object') {\n    const themeBreakpoints = theme.breakpoints || defaultBreakpoints;\n    return Object.keys(propValue).reduce((acc, breakpoint) => {\n      if ((0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_0__.isCqShorthand)(themeBreakpoints.keys, breakpoint)) {\n        const containerKey = (0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_0__.getContainerQuery)(theme.containerQueries ? theme : defaultContainerQueries, breakpoint);\n        if (containerKey) {\n          acc[containerKey] = styleFromPropValue(propValue[breakpoint], breakpoint);\n        }\n      }\n      // key is breakpoint\n      else if (Object.keys(themeBreakpoints.values || values).includes(breakpoint)) {\n        const mediaKey = themeBreakpoints.up(breakpoint);\n        acc[mediaKey] = styleFromPropValue(propValue[breakpoint], breakpoint);\n      } else {\n        const cssKey = breakpoint;\n        acc[cssKey] = propValue[cssKey];\n      }\n      return acc;\n    }, {});\n  }\n  const output = styleFromPropValue(propValue);\n  return output;\n}\nfunction breakpoints(styleFunction) {\n  // false positive\n  // eslint-disable-next-line react/function-component-definition\n  const newStyleFunction = props => {\n    const theme = props.theme || {};\n    const base = styleFunction(props);\n    const themeBreakpoints = theme.breakpoints || defaultBreakpoints;\n    const extended = themeBreakpoints.keys.reduce((acc, key) => {\n      if (props[key]) {\n        acc = acc || {};\n        acc[themeBreakpoints.up(key)] = styleFunction({\n          theme,\n          ...props[key]\n        });\n      }\n      return acc;\n    }, null);\n    return (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(base, extended);\n  };\n  newStyleFunction.propTypes =  true ? {\n    ...styleFunction.propTypes,\n    xs: prop_types__WEBPACK_IMPORTED_MODULE_2__.object,\n    sm: prop_types__WEBPACK_IMPORTED_MODULE_2__.object,\n    md: prop_types__WEBPACK_IMPORTED_MODULE_2__.object,\n    lg: prop_types__WEBPACK_IMPORTED_MODULE_2__.object,\n    xl: prop_types__WEBPACK_IMPORTED_MODULE_2__.object\n  } : 0;\n  newStyleFunction.filterProps = ['xs', 'sm', 'md', 'lg', 'xl', ...styleFunction.filterProps];\n  return newStyleFunction;\n}\nfunction createEmptyBreakpointObject(breakpointsInput = {}) {\n  const breakpointsInOrder = breakpointsInput.keys?.reduce((acc, key) => {\n    const breakpointStyleKey = breakpointsInput.up(key);\n    acc[breakpointStyleKey] = {};\n    return acc;\n  }, {});\n  return breakpointsInOrder || {};\n}\nfunction removeUnusedBreakpoints(breakpointKeys, style) {\n  return breakpointKeys.reduce((acc, key) => {\n    const breakpointOutput = acc[key];\n    const isBreakpointUnused = !breakpointOutput || Object.keys(breakpointOutput).length === 0;\n    if (isBreakpointUnused) {\n      delete acc[key];\n    }\n    return acc;\n  }, style);\n}\nfunction mergeBreakpointsInOrder(breakpointsInput, ...styles) {\n  const emptyBreakpoints = createEmptyBreakpointObject(breakpointsInput);\n  const mergedOutput = [emptyBreakpoints, ...styles].reduce((prev, next) => (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(prev, next), {});\n  return removeUnusedBreakpoints(Object.keys(emptyBreakpoints), mergedOutput);\n}\n\n// compute base for responsive values; e.g.,\n// [1,2,3] => {xs: true, sm: true, md: true}\n// {xs: 1, sm: 2, md: 3} => {xs: true, sm: true, md: true}\nfunction computeBreakpointsBase(breakpointValues, themeBreakpoints) {\n  // fixed value\n  if (typeof breakpointValues !== 'object') {\n    return {};\n  }\n  const base = {};\n  const breakpointsKeys = Object.keys(themeBreakpoints);\n  if (Array.isArray(breakpointValues)) {\n    breakpointsKeys.forEach((breakpoint, i) => {\n      if (i < breakpointValues.length) {\n        base[breakpoint] = true;\n      }\n    });\n  } else {\n    breakpointsKeys.forEach(breakpoint => {\n      if (breakpointValues[breakpoint] != null) {\n        base[breakpoint] = true;\n      }\n    });\n  }\n  return base;\n}\nfunction resolveBreakpointValues({\n  values: breakpointValues,\n  breakpoints: themeBreakpoints,\n  base: customBase\n}) {\n  const base = customBase || computeBreakpointsBase(breakpointValues, themeBreakpoints);\n  const keys = Object.keys(base);\n  if (keys.length === 0) {\n    return breakpointValues;\n  }\n  let previous;\n  return keys.reduce((acc, breakpoint, i) => {\n    if (Array.isArray(breakpointValues)) {\n      acc[breakpoint] = breakpointValues[i] != null ? breakpointValues[i] : breakpointValues[previous];\n      previous = i;\n    } else if (typeof breakpointValues === 'object') {\n      acc[breakpoint] = breakpointValues[breakpoint] != null ? breakpointValues[breakpoint] : breakpointValues[previous];\n      previous = breakpoint;\n    } else {\n      acc[breakpoint] = breakpointValues;\n    }\n    return acc;\n  }, {});\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (breakpoints);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/breakpoints/breakpoints.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/compose/compose.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/system/esm/compose/compose.js ***!
  \*********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/system/esm/merge/merge.js\");\n\nfunction compose(...styles) {\n  const handlers = styles.reduce((acc, style) => {\n    style.filterProps.forEach(prop => {\n      acc[prop] = style;\n    });\n    return acc;\n  }, {});\n\n  // false positive\n  // eslint-disable-next-line react/function-component-definition\n  const fn = props => {\n    return Object.keys(props).reduce((acc, prop) => {\n      if (handlers[prop]) {\n        return (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(acc, handlers[prop](props));\n      }\n      return acc;\n    }, {});\n  };\n  fn.propTypes =  true ? styles.reduce((acc, style) => Object.assign(acc, style.propTypes), {}) : 0;\n  fn.filterProps = styles.reduce((acc, style) => acc.concat(style.filterProps), []);\n  return fn;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (compose);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/compose/compose.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js":
/*!*****************************************************************************!*\
  !*** ./node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js ***!
  \*****************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   breakpointKeys: () => (/* binding */ breakpointKeys),\n/* harmony export */   \"default\": () => (/* binding */ createBreakpoints)\n/* harmony export */ });\n// Sorted ASC by size. That's important.\n// It can't be configured as it's used statically for propTypes.\nconst breakpointKeys = ['xs', 'sm', 'md', 'lg', 'xl'];\nconst sortBreakpointsValues = values => {\n  const breakpointsAsArray = Object.keys(values).map(key => ({\n    key,\n    val: values[key]\n  })) || [];\n  // Sort in ascending order\n  breakpointsAsArray.sort((breakpoint1, breakpoint2) => breakpoint1.val - breakpoint2.val);\n  return breakpointsAsArray.reduce((acc, obj) => {\n    return {\n      ...acc,\n      [obj.key]: obj.val\n    };\n  }, {});\n};\n\n// Keep in mind that @media is inclusive by the CSS specification.\nfunction createBreakpoints(breakpoints) {\n  const {\n    // The breakpoint **start** at this value.\n    // For instance with the first breakpoint xs: [xs, sm).\n    values = {\n      xs: 0,\n      // phone\n      sm: 600,\n      // tablet\n      md: 900,\n      // small laptop\n      lg: 1200,\n      // desktop\n      xl: 1536 // large screen\n    },\n    unit = 'px',\n    step = 5,\n    ...other\n  } = breakpoints;\n  const sortedValues = sortBreakpointsValues(values);\n  const keys = Object.keys(sortedValues);\n  function up(key) {\n    const value = typeof values[key] === 'number' ? values[key] : key;\n    return `@media (min-width:${value}${unit})`;\n  }\n  function down(key) {\n    const value = typeof values[key] === 'number' ? values[key] : key;\n    return `@media (max-width:${value - step / 100}${unit})`;\n  }\n  function between(start, end) {\n    const endIndex = keys.indexOf(end);\n    return `@media (min-width:${typeof values[start] === 'number' ? values[start] : start}${unit}) and ` + `(max-width:${(endIndex !== -1 && typeof values[keys[endIndex]] === 'number' ? values[keys[endIndex]] : end) - step / 100}${unit})`;\n  }\n  function only(key) {\n    if (keys.indexOf(key) + 1 < keys.length) {\n      return between(key, keys[keys.indexOf(key) + 1]);\n    }\n    return up(key);\n  }\n  function not(key) {\n    // handle first and last key separately, for better readability\n    const keyIndex = keys.indexOf(key);\n    if (keyIndex === 0) {\n      return up(keys[1]);\n    }\n    if (keyIndex === keys.length - 1) {\n      return down(keys[keyIndex]);\n    }\n    return between(key, keys[keys.indexOf(key) + 1]).replace('@media', '@media not all and');\n  }\n  return {\n    keys,\n    values: sortedValues,\n    up,\n    down,\n    between,\n    only,\n    not,\n    unit,\n    ...other\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/createStyled/createStyled.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@mui/system/esm/createStyled/createStyled.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createStyled),\n/* harmony export */   shouldForwardProp: () => (/* binding */ shouldForwardProp),\n/* harmony export */   systemDefaultTheme: () => (/* binding */ systemDefaultTheme)\n/* harmony export */ });\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/styled-engine/esm/index.js\");\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _mui_utils_getDisplayName__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/utils/getDisplayName */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js\");\n/* harmony import */ var _createTheme_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createTheme/index.js */ \"./node_modules/@mui/system/esm/createTheme/createTheme.js\");\n/* harmony import */ var _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../styleFunctionSx/index.js */ \"./node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _preprocessStyles_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../preprocessStyles.js */ \"./node_modules/@mui/system/esm/preprocessStyles.js\");\n\n\n\n\n\n\n\n\n/* eslint-disable no-underscore-dangle */\n/* eslint-disable no-labels */\n/* eslint-disable no-lone-blocks */\n\nconst systemDefaultTheme = (0,_createTheme_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\n\n// Update /system/styled/#api in case if this changes\nfunction shouldForwardProp(prop) {\n  return prop !== 'ownerState' && prop !== 'theme' && prop !== 'sx' && prop !== 'as';\n}\nfunction defaultOverridesResolver(slot) {\n  if (!slot) {\n    return null;\n  }\n  return (_props, styles) => styles[slot];\n}\nfunction attachTheme(props, themeId, defaultTheme) {\n  props.theme = isObjectEmpty(props.theme) ? defaultTheme : props.theme[themeId] || props.theme;\n}\nfunction processStyle(props, style) {\n  /*\n   * Style types:\n   *  - null/undefined\n   *  - string\n   *  - CSS style object: { [cssKey]: [cssValue], variants }\n   *  - Processed style object: { style, variants, isProcessed: true }\n   *  - Array of any of the above\n   */\n\n  const resolvedStyle = typeof style === 'function' ? style(props) : style;\n  if (Array.isArray(resolvedStyle)) {\n    return resolvedStyle.flatMap(subStyle => processStyle(props, subStyle));\n  }\n  if (Array.isArray(resolvedStyle?.variants)) {\n    let rootStyle;\n    if (resolvedStyle.isProcessed) {\n      rootStyle = resolvedStyle.style;\n    } else {\n      const {\n        variants,\n        ...otherStyles\n      } = resolvedStyle;\n      rootStyle = otherStyles;\n    }\n    return processStyleVariants(props, resolvedStyle.variants, [rootStyle]);\n  }\n  if (resolvedStyle?.isProcessed) {\n    return resolvedStyle.style;\n  }\n  return resolvedStyle;\n}\nfunction processStyleVariants(props, variants, results = []) {\n  let mergedState; // We might not need it, initialized lazily\n\n  variantLoop: for (let i = 0; i < variants.length; i += 1) {\n    const variant = variants[i];\n    if (typeof variant.props === 'function') {\n      mergedState ??= {\n        ...props,\n        ...props.ownerState,\n        ownerState: props.ownerState\n      };\n      if (!variant.props(mergedState)) {\n        continue;\n      }\n    } else {\n      for (const key in variant.props) {\n        if (props[key] !== variant.props[key] && props.ownerState?.[key] !== variant.props[key]) {\n          continue variantLoop;\n        }\n      }\n    }\n    if (typeof variant.style === 'function') {\n      mergedState ??= {\n        ...props,\n        ...props.ownerState,\n        ownerState: props.ownerState\n      };\n      results.push(variant.style(mergedState));\n    } else {\n      results.push(variant.style);\n    }\n  }\n  return results;\n}\nfunction createStyled(input = {}) {\n  const {\n    themeId,\n    defaultTheme = systemDefaultTheme,\n    rootShouldForwardProp = shouldForwardProp,\n    slotShouldForwardProp = shouldForwardProp\n  } = input;\n  function styleAttachTheme(props) {\n    attachTheme(props, themeId, defaultTheme);\n  }\n  const styled = (tag, inputOptions = {}) => {\n    // If `tag` is already a styled component, filter out the `sx` style function\n    // to prevent unnecessary styles generated by the composite components.\n    (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__.internal_mutateStyles)(tag, styles => styles.filter(style => style !== _styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]));\n    const {\n      name: componentName,\n      slot: componentSlot,\n      skipVariantsResolver: inputSkipVariantsResolver,\n      skipSx: inputSkipSx,\n      // TODO v6: remove `lowercaseFirstLetter()` in the next major release\n      // For more details: https://github.com/mui/material-ui/pull/37908\n      overridesResolver = defaultOverridesResolver(lowercaseFirstLetter(componentSlot)),\n      ...options\n    } = inputOptions;\n\n    // if skipVariantsResolver option is defined, take the value, otherwise, true for root and false for other slots.\n    const skipVariantsResolver = inputSkipVariantsResolver !== undefined ? inputSkipVariantsResolver :\n    // TODO v6: remove `Root` in the next major release\n    // For more details: https://github.com/mui/material-ui/pull/37908\n    componentSlot && componentSlot !== 'Root' && componentSlot !== 'root' || false;\n    const skipSx = inputSkipSx || false;\n    let shouldForwardPropOption = shouldForwardProp;\n\n    // TODO v6: remove `Root` in the next major release\n    // For more details: https://github.com/mui/material-ui/pull/37908\n    if (componentSlot === 'Root' || componentSlot === 'root') {\n      shouldForwardPropOption = rootShouldForwardProp;\n    } else if (componentSlot) {\n      // any other slot specified\n      shouldForwardPropOption = slotShouldForwardProp;\n    } else if (isStringTag(tag)) {\n      // for string (html) tag, preserve the behavior in emotion & styled-components.\n      shouldForwardPropOption = undefined;\n    }\n    const defaultStyledResolver = (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(tag, {\n      shouldForwardProp: shouldForwardPropOption,\n      label: generateStyledLabel(componentName, componentSlot),\n      ...options\n    });\n    const transformStyle = style => {\n      // On the server Emotion doesn't use React.forwardRef for creating components, so the created\n      // component stays as a function. This condition makes sure that we do not interpolate functions\n      // which are basically components used as a selectors.\n      if (typeof style === 'function' && style.__emotion_real !== style) {\n        return function styleFunctionProcessor(props) {\n          return processStyle(props, style);\n        };\n      }\n      if ((0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_3__.isPlainObject)(style)) {\n        const serialized = (0,_preprocessStyles_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(style);\n        if (!serialized.variants) {\n          return serialized.style;\n        }\n        return function styleObjectProcessor(props) {\n          return processStyle(props, serialized);\n        };\n      }\n      return style;\n    };\n    const muiStyledResolver = (...expressionsInput) => {\n      const expressionsHead = [];\n      const expressionsBody = expressionsInput.map(transformStyle);\n      const expressionsTail = [];\n\n      // Preprocess `props` to set the scoped theme value.\n      // This must run before any other expression.\n      expressionsHead.push(styleAttachTheme);\n      if (componentName && overridesResolver) {\n        expressionsTail.push(function styleThemeOverrides(props) {\n          const theme = props.theme;\n          const styleOverrides = theme.components?.[componentName]?.styleOverrides;\n          if (!styleOverrides) {\n            return null;\n          }\n          const resolvedStyleOverrides = {};\n\n          // TODO: v7 remove iteration and use `resolveStyleArg(styleOverrides[slot])` directly\n          // eslint-disable-next-line guard-for-in\n          for (const slotKey in styleOverrides) {\n            resolvedStyleOverrides[slotKey] = processStyle(props, styleOverrides[slotKey]);\n          }\n          return overridesResolver(props, resolvedStyleOverrides);\n        });\n      }\n      if (componentName && !skipVariantsResolver) {\n        expressionsTail.push(function styleThemeVariants(props) {\n          const theme = props.theme;\n          const themeVariants = theme?.components?.[componentName]?.variants;\n          if (!themeVariants) {\n            return null;\n          }\n          return processStyleVariants(props, themeVariants);\n        });\n      }\n      if (!skipSx) {\n        expressionsTail.push(_styleFunctionSx_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]);\n      }\n\n      // This function can be called as a tagged template, so the first argument would contain\n      // CSS `string[]` values.\n      if (Array.isArray(expressionsBody[0])) {\n        const inputStrings = expressionsBody.shift();\n\n        // We need to add placeholders in the tagged template for the custom functions we have\n        // possibly added (attachTheme, overrides, variants, and sx).\n        const placeholdersHead = new Array(expressionsHead.length).fill('');\n        const placeholdersTail = new Array(expressionsTail.length).fill('');\n        let outputStrings;\n        // prettier-ignore\n        {\n          outputStrings = [...placeholdersHead, ...inputStrings, ...placeholdersTail];\n          outputStrings.raw = [...placeholdersHead, ...inputStrings.raw, ...placeholdersTail];\n        }\n\n        // The only case where we put something before `attachTheme`\n        expressionsHead.unshift(outputStrings);\n      }\n      const expressions = [...expressionsHead, ...expressionsBody, ...expressionsTail];\n      const Component = defaultStyledResolver(...expressions);\n      if (tag.muiName) {\n        Component.muiName = tag.muiName;\n      }\n      if (true) {\n        Component.displayName = generateDisplayName(componentName, componentSlot, tag);\n      }\n      return Component;\n    };\n    if (defaultStyledResolver.withConfig) {\n      muiStyledResolver.withConfig = defaultStyledResolver.withConfig;\n    }\n    return muiStyledResolver;\n  };\n  return styled;\n}\nfunction generateDisplayName(componentName, componentSlot, tag) {\n  if (componentName) {\n    return `${componentName}${(0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_5__[\"default\"])(componentSlot || '')}`;\n  }\n  return `Styled(${(0,_mui_utils_getDisplayName__WEBPACK_IMPORTED_MODULE_6__[\"default\"])(tag)})`;\n}\nfunction generateStyledLabel(componentName, componentSlot) {\n  let label;\n  if (true) {\n    if (componentName) {\n      // TODO v6: remove `lowercaseFirstLetter()` in the next major release\n      // For more details: https://github.com/mui/material-ui/pull/37908\n      label = `${componentName}-${lowercaseFirstLetter(componentSlot || 'Root')}`;\n    }\n  }\n  return label;\n}\nfunction isObjectEmpty(object) {\n  // eslint-disable-next-line\n  for (const _ in object) {\n    return false;\n  }\n  return true;\n}\n\n// https://github.com/emotion-js/emotion/blob/26ded6109fcd8ca9875cc2ce4564fee678a3f3c5/packages/styled/src/utils.js#L40\nfunction isStringTag(tag) {\n  return typeof tag === 'string' &&\n  // 96 is one less than the char code\n  // for \"a\" so this is checking that\n  // it's a lowercase character\n  tag.charCodeAt(0) > 96;\n}\nfunction lowercaseFirstLetter(string) {\n  if (!string) {\n    return string;\n  }\n  return string.charAt(0).toLowerCase() + string.slice(1);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/createStyled/createStyled.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/createTheme/applyStyles.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@mui/system/esm/createTheme/applyStyles.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ applyStyles)\n/* harmony export */ });\n/**\n * A universal utility to style components with multiple color modes. Always use it from the theme object.\n * It works with:\n *  - [Basic theme](https://mui.com/material-ui/customization/dark-mode/)\n *  - [CSS theme variables](https://mui.com/material-ui/customization/css-theme-variables/overview/)\n *  - Zero-runtime engine\n *\n * Tips: Use an array over object spread and place `theme.applyStyles()` last.\n *\n * With the styled function:\n * ✅ [{ background: '#e5e5e5' }, theme.applyStyles('dark', { background: '#1c1c1c' })]\n * 🚫 { background: '#e5e5e5', ...theme.applyStyles('dark', { background: '#1c1c1c' })}\n *\n * With the sx prop:\n * ✅ [{ background: '#e5e5e5' }, theme => theme.applyStyles('dark', { background: '#1c1c1c' })]\n * 🚫 { background: '#e5e5e5', ...theme => theme.applyStyles('dark', { background: '#1c1c1c' })}\n *\n * @example\n * 1. using with `styled`:\n * ```jsx\n *   const Component = styled('div')(({ theme }) => [\n *     { background: '#e5e5e5' },\n *     theme.applyStyles('dark', {\n *       background: '#1c1c1c',\n *       color: '#fff',\n *     }),\n *   ]);\n * ```\n *\n * @example\n * 2. using with `sx` prop:\n * ```jsx\n *   <Box sx={[\n *     { background: '#e5e5e5' },\n *     theme => theme.applyStyles('dark', {\n *        background: '#1c1c1c',\n *        color: '#fff',\n *      }),\n *     ]}\n *   />\n * ```\n *\n * @example\n * 3. theming a component:\n * ```jsx\n *   extendTheme({\n *     components: {\n *       MuiButton: {\n *         styleOverrides: {\n *           root: ({ theme }) => [\n *             { background: '#e5e5e5' },\n *             theme.applyStyles('dark', {\n *               background: '#1c1c1c',\n *               color: '#fff',\n *             }),\n *           ],\n *         },\n *       }\n *     }\n *   })\n *```\n */\nfunction applyStyles(key, styles) {\n  // @ts-expect-error this is 'any' type\n  const theme = this;\n  if (theme.vars) {\n    if (!theme.colorSchemes?.[key] || typeof theme.getColorSchemeSelector !== 'function') {\n      return {};\n    }\n    // If CssVarsProvider is used as a provider, returns '*:where({selector}) &'\n    let selector = theme.getColorSchemeSelector(key);\n    if (selector === '&') {\n      return styles;\n    }\n    if (selector.includes('data-') || selector.includes('.')) {\n      // '*' is required as a workaround for Emotion issue (https://github.com/emotion-js/emotion/issues/2836)\n      selector = `*:where(${selector.replace(/\\s*&$/, '')}) &`;\n    }\n    return {\n      [selector]: styles\n    };\n  }\n  if (theme.palette.mode === key) {\n    return styles;\n  }\n  return {};\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/createTheme/applyStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/createTheme/createSpacing.js":
/*!*******************************************************************!*\
  !*** ./node_modules/@mui/system/esm/createTheme/createSpacing.js ***!
  \*******************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createSpacing)\n/* harmony export */ });\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/system/esm/spacing/spacing.js\");\n\n\n// The different signatures imply different meaning for their arguments that can't be expressed structurally.\n// We express the difference with variable names.\n\nfunction createSpacing(spacingInput = 8,\n// Material Design layouts are visually balanced. Most measurements align to an 8dp grid, which aligns both spacing and the overall layout.\n// Smaller components, such as icons, can align to a 4dp grid.\n// https://m2.material.io/design/layout/understanding-layout.html\ntransform = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnarySpacing)({\n  spacing: spacingInput\n})) {\n  // Already transformed.\n  if (spacingInput.mui) {\n    return spacingInput;\n  }\n  const spacing = (...argsInput) => {\n    if (true) {\n      if (!(argsInput.length <= 4)) {\n        console.error(`MUI: Too many arguments provided, expected between 0 and 4, got ${argsInput.length}`);\n      }\n    }\n    const args = argsInput.length === 0 ? [1] : argsInput;\n    return args.map(argument => {\n      const output = transform(argument);\n      return typeof output === 'number' ? `${output}px` : output;\n    }).join(' ');\n  };\n  spacing.mui = true;\n  return spacing;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/createTheme/createSpacing.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/createTheme/createTheme.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@mui/system/esm/createTheme/createTheme.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n/* harmony import */ var _createBreakpoints_createBreakpoints_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createBreakpoints/createBreakpoints.js */ \"./node_modules/@mui/system/esm/createBreakpoints/createBreakpoints.js\");\n/* harmony import */ var _cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../cssContainerQueries/index.js */ \"./node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js\");\n/* harmony import */ var _shape_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./shape.js */ \"./node_modules/@mui/system/esm/createTheme/shape.js\");\n/* harmony import */ var _createSpacing_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./createSpacing.js */ \"./node_modules/@mui/system/esm/createTheme/createSpacing.js\");\n/* harmony import */ var _styleFunctionSx_styleFunctionSx_js__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ../styleFunctionSx/styleFunctionSx.js */ \"./node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _styleFunctionSx_defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../styleFunctionSx/defaultSxConfig.js */ \"./node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n/* harmony import */ var _applyStyles_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./applyStyles.js */ \"./node_modules/@mui/system/esm/createTheme/applyStyles.js\");\n\n\n\n\n\n\n\n\nfunction createTheme(options = {}, ...args) {\n  const {\n    breakpoints: breakpointsInput = {},\n    palette: paletteInput = {},\n    spacing: spacingInput,\n    shape: shapeInput = {},\n    ...other\n  } = options;\n  const breakpoints = (0,_createBreakpoints_createBreakpoints_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(breakpointsInput);\n  const spacing = (0,_createSpacing_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(spacingInput);\n  let muiTheme = (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n    breakpoints,\n    direction: 'ltr',\n    components: {},\n    // Inject component definitions.\n    palette: {\n      mode: 'light',\n      ...paletteInput\n    },\n    spacing,\n    shape: {\n      ..._shape_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"],\n      ...shapeInput\n    }\n  }, other);\n  muiTheme = (0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(muiTheme);\n  muiTheme.applyStyles = _applyStyles_js__WEBPACK_IMPORTED_MODULE_5__[\"default\"];\n  muiTheme = args.reduce((acc, argument) => (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(acc, argument), muiTheme);\n  muiTheme.unstable_sxConfig = {\n    ..._styleFunctionSx_defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_6__[\"default\"],\n    ...other?.unstable_sxConfig\n  };\n  muiTheme.unstable_sx = function sx(props) {\n    return (0,_styleFunctionSx_styleFunctionSx_js__WEBPACK_IMPORTED_MODULE_7__[\"default\"])({\n      sx: props,\n      theme: this\n    });\n  };\n  return muiTheme;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (createTheme);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/createTheme/createTheme.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/createTheme/shape.js":
/*!***********************************************************!*\
  !*** ./node_modules/@mui/system/esm/createTheme/shape.js ***!
  \***********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst shape = {\n  borderRadius: 4\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (shape);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/createTheme/shape.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js":
/*!*********************************************************************************!*\
  !*** ./node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js ***!
  \*********************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ cssContainerQueries),\n/* harmony export */   getContainerQuery: () => (/* binding */ getContainerQuery),\n/* harmony export */   isCqShorthand: () => (/* binding */ isCqShorthand),\n/* harmony export */   sortContainerQueries: () => (/* binding */ sortContainerQueries)\n/* harmony export */ });\n\n/**\n * For using in `sx` prop to sort the breakpoint from low to high.\n * Note: this function does not work and will not support multiple units.\n *       e.g. input: { '@container (min-width:300px)': '1rem', '@container (min-width:40rem)': '2rem' }\n *            output: { '@container (min-width:40rem)': '2rem', '@container (min-width:300px)': '1rem' } // since 40 < 300 eventhough 40rem > 300px\n */\nfunction sortContainerQueries(theme, css) {\n  if (!theme.containerQueries) {\n    return css;\n  }\n  const sorted = Object.keys(css).filter(key => key.startsWith('@container')).sort((a, b) => {\n    const regex = /min-width:\\s*([0-9.]+)/;\n    return +(a.match(regex)?.[1] || 0) - +(b.match(regex)?.[1] || 0);\n  });\n  if (!sorted.length) {\n    return css;\n  }\n  return sorted.reduce((acc, key) => {\n    const value = css[key];\n    delete acc[key];\n    acc[key] = value;\n    return acc;\n  }, {\n    ...css\n  });\n}\nfunction isCqShorthand(breakpointKeys, value) {\n  return value === '@' || value.startsWith('@') && (breakpointKeys.some(key => value.startsWith(`@${key}`)) || !!value.match(/^@\\d/));\n}\nfunction getContainerQuery(theme, shorthand) {\n  const matches = shorthand.match(/^@([^/]+)?\\/?(.+)?$/);\n  if (!matches) {\n    if (true) {\n      throw new Error( true ? `MUI: The provided shorthand ${`(${shorthand})`} is invalid. The format should be \\`@<breakpoint | number>\\` or \\`@<breakpoint | number>/<container>\\`.\\n` + 'For example, `@sm` or `@600` or `@40rem/sidebar`.' : 0);\n    }\n    // removed by dead control flow\n{}\n  }\n  const [, containerQuery, containerName] = matches;\n  const value = Number.isNaN(+containerQuery) ? containerQuery || 0 : +containerQuery;\n  return theme.containerQueries(containerName).up(value);\n}\nfunction cssContainerQueries(themeInput) {\n  const toContainerQuery = (mediaQuery, name) => mediaQuery.replace('@media', name ? `@container ${name}` : '@container');\n  function attachCq(node, name) {\n    node.up = (...args) => toContainerQuery(themeInput.breakpoints.up(...args), name);\n    node.down = (...args) => toContainerQuery(themeInput.breakpoints.down(...args), name);\n    node.between = (...args) => toContainerQuery(themeInput.breakpoints.between(...args), name);\n    node.only = (...args) => toContainerQuery(themeInput.breakpoints.only(...args), name);\n    node.not = (...args) => {\n      const result = toContainerQuery(themeInput.breakpoints.not(...args), name);\n      if (result.includes('not all and')) {\n        // `@container` does not work with `not all and`, so need to invert the logic\n        return result.replace('not all and ', '').replace('min-width:', 'width<').replace('max-width:', 'width>').replace('and', 'or');\n      }\n      return result;\n    };\n  }\n  const node = {};\n  const containerQueries = name => {\n    attachCq(node, name);\n    return node;\n  };\n  attachCq(containerQueries);\n  return {\n    ...themeInput,\n    containerQueries\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/cssGrid/cssGrid.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/system/esm/cssGrid/cssGrid.js ***!
  \*********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   columnGap: () => (/* binding */ columnGap),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   gap: () => (/* binding */ gap),\n/* harmony export */   gridArea: () => (/* binding */ gridArea),\n/* harmony export */   gridAutoColumns: () => (/* binding */ gridAutoColumns),\n/* harmony export */   gridAutoFlow: () => (/* binding */ gridAutoFlow),\n/* harmony export */   gridAutoRows: () => (/* binding */ gridAutoRows),\n/* harmony export */   gridColumn: () => (/* binding */ gridColumn),\n/* harmony export */   gridRow: () => (/* binding */ gridRow),\n/* harmony export */   gridTemplateAreas: () => (/* binding */ gridTemplateAreas),\n/* harmony export */   gridTemplateColumns: () => (/* binding */ gridTemplateColumns),\n/* harmony export */   gridTemplateRows: () => (/* binding */ gridTemplateRows),\n/* harmony export */   rowGap: () => (/* binding */ rowGap)\n/* harmony export */ });\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/system/esm/compose/compose.js\");\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n\n\n\n\n\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst gap = props => {\n  if (props.gap !== undefined && props.gap !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnaryUnit)(props.theme, 'spacing', 8, 'gap');\n    const styleFromPropValue = propValue => ({\n      gap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.gap, styleFromPropValue);\n  }\n  return null;\n};\ngap.propTypes =  true ? {\n  gap: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n} : 0;\ngap.filterProps = ['gap'];\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst columnGap = props => {\n  if (props.columnGap !== undefined && props.columnGap !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnaryUnit)(props.theme, 'spacing', 8, 'columnGap');\n    const styleFromPropValue = propValue => ({\n      columnGap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.columnGap, styleFromPropValue);\n  }\n  return null;\n};\ncolumnGap.propTypes =  true ? {\n  columnGap: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n} : 0;\ncolumnGap.filterProps = ['columnGap'];\n\n// false positive\n// eslint-disable-next-line react/function-component-definition\nconst rowGap = props => {\n  if (props.rowGap !== undefined && props.rowGap !== null) {\n    const transformer = (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.createUnaryUnit)(props.theme, 'spacing', 8, 'rowGap');\n    const styleFromPropValue = propValue => ({\n      rowGap: (0,_spacing_index_js__WEBPACK_IMPORTED_MODULE_0__.getValue)(transformer, propValue)\n    });\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.rowGap, styleFromPropValue);\n  }\n  return null;\n};\nrowGap.propTypes =  true ? {\n  rowGap: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n} : 0;\nrowGap.filterProps = ['rowGap'];\nconst gridColumn = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridColumn'\n});\nconst gridRow = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridRow'\n});\nconst gridAutoFlow = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridAutoFlow'\n});\nconst gridAutoColumns = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridAutoColumns'\n});\nconst gridAutoRows = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridAutoRows'\n});\nconst gridTemplateColumns = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridTemplateColumns'\n});\nconst gridTemplateRows = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridTemplateRows'\n});\nconst gridTemplateAreas = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridTemplateAreas'\n});\nconst gridArea = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])({\n  prop: 'gridArea'\n});\nconst grid = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(gap, columnGap, rowGap, gridColumn, gridRow, gridAutoFlow, gridAutoColumns, gridAutoRows, gridTemplateColumns, gridTemplateRows, gridTemplateAreas, gridArea);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (grid);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/cssGrid/cssGrid.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/memoize/memoize.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/system/esm/memoize/memoize.js ***!
  \*********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ memoize)\n/* harmony export */ });\nfunction memoize(fn) {\n  const cache = {};\n  return arg => {\n    if (cache[arg] === undefined) {\n      cache[arg] = fn(arg);\n    }\n    return cache[arg];\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/memoize/memoize.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/merge/merge.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/system/esm/merge/merge.js ***!
  \*****************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/deepmerge */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js\");\n\nfunction merge(acc, item) {\n  if (!item) {\n    return acc;\n  }\n  return (0,_mui_utils_deepmerge__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(acc, item, {\n    clone: false // No need to clone deep, it's way faster.\n  });\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (merge);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/merge/merge.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/palette/palette.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/system/esm/palette/palette.js ***!
  \*********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   backgroundColor: () => (/* binding */ backgroundColor),\n/* harmony export */   bgcolor: () => (/* binding */ bgcolor),\n/* harmony export */   color: () => (/* binding */ color),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   paletteTransform: () => (/* binding */ paletteTransform)\n/* harmony export */ });\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/system/esm/compose/compose.js\");\n\n\nfunction paletteTransform(value, userValue) {\n  if (userValue === 'grey') {\n    return userValue;\n  }\n  return value;\n}\nconst color = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'color',\n  themeKey: 'palette',\n  transform: paletteTransform\n});\nconst bgcolor = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'bgcolor',\n  cssProperty: 'backgroundColor',\n  themeKey: 'palette',\n  transform: paletteTransform\n});\nconst backgroundColor = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'backgroundColor',\n  themeKey: 'palette',\n  transform: paletteTransform\n});\nconst palette = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(color, bgcolor, backgroundColor);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (palette);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/palette/palette.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/preprocessStyles.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/system/esm/preprocessStyles.js ***!
  \**********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ preprocessStyles)\n/* harmony export */ });\n/* harmony import */ var _mui_styled_engine__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/styled-engine */ \"./node_modules/@mui/styled-engine/esm/index.js\");\n\nfunction preprocessStyles(input) {\n  const {\n    variants,\n    ...style\n  } = input;\n  const result = {\n    variants,\n    style: (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_0__.internal_serializeStyles)(style),\n    isProcessed: true\n  };\n\n  // Not supported on styled-components\n  if (result.style === style) {\n    return result;\n  }\n  if (variants) {\n    variants.forEach(variant => {\n      if (typeof variant.style !== 'function') {\n        variant.style = (0,_mui_styled_engine__WEBPACK_IMPORTED_MODULE_0__.internal_serializeStyles)(variant.style);\n      }\n    });\n  }\n  return result;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/preprocessStyles.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js":
/*!*******************************************************************************!*\
  !*** ./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js ***!
  \*******************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n\nconst responsivePropType =  true ? prop_types__WEBPACK_IMPORTED_MODULE_0__.oneOfType([prop_types__WEBPACK_IMPORTED_MODULE_0__.number, prop_types__WEBPACK_IMPORTED_MODULE_0__.string, prop_types__WEBPACK_IMPORTED_MODULE_0__.object, prop_types__WEBPACK_IMPORTED_MODULE_0__.array]) : 0;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (responsivePropType);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/sizing/sizing.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/system/esm/sizing/sizing.js ***!
  \*******************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   boxSizing: () => (/* binding */ boxSizing),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   height: () => (/* binding */ height),\n/* harmony export */   maxHeight: () => (/* binding */ maxHeight),\n/* harmony export */   maxWidth: () => (/* binding */ maxWidth),\n/* harmony export */   minHeight: () => (/* binding */ minHeight),\n/* harmony export */   minWidth: () => (/* binding */ minWidth),\n/* harmony export */   sizeHeight: () => (/* binding */ sizeHeight),\n/* harmony export */   sizeWidth: () => (/* binding */ sizeWidth),\n/* harmony export */   sizingTransform: () => (/* binding */ sizingTransform),\n/* harmony export */   width: () => (/* binding */ width)\n/* harmony export */ });\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _compose_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../compose/index.js */ \"./node_modules/@mui/system/esm/compose/compose.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n\n\n\nfunction sizingTransform(value) {\n  return value <= 1 && value !== 0 ? `${value * 100}%` : value;\n}\nconst width = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'width',\n  transform: sizingTransform\n});\nconst maxWidth = props => {\n  if (props.maxWidth !== undefined && props.maxWidth !== null) {\n    const styleFromPropValue = propValue => {\n      const breakpoint = props.theme?.breakpoints?.values?.[propValue] || _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.values[propValue];\n      if (!breakpoint) {\n        return {\n          maxWidth: sizingTransform(propValue)\n        };\n      }\n      if (props.theme?.breakpoints?.unit !== 'px') {\n        return {\n          maxWidth: `${breakpoint}${props.theme.breakpoints.unit}`\n        };\n      }\n      return {\n        maxWidth: breakpoint\n      };\n    };\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, props.maxWidth, styleFromPropValue);\n  }\n  return null;\n};\nmaxWidth.filterProps = ['maxWidth'];\nconst minWidth = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'minWidth',\n  transform: sizingTransform\n});\nconst height = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'height',\n  transform: sizingTransform\n});\nconst maxHeight = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'maxHeight',\n  transform: sizingTransform\n});\nconst minHeight = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'minHeight',\n  transform: sizingTransform\n});\nconst sizeWidth = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'size',\n  cssProperty: 'width',\n  transform: sizingTransform\n});\nconst sizeHeight = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'size',\n  cssProperty: 'height',\n  transform: sizingTransform\n});\nconst boxSizing = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])({\n  prop: 'boxSizing'\n});\nconst sizing = (0,_compose_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(width, maxWidth, minWidth, height, maxHeight, minHeight, boxSizing);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (sizing);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/sizing/sizing.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/spacing/spacing.js":
/*!*********************************************************!*\
  !*** ./node_modules/@mui/system/esm/spacing/spacing.js ***!
  \*********************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   createUnarySpacing: () => (/* binding */ createUnarySpacing),\n/* harmony export */   createUnaryUnit: () => (/* binding */ createUnaryUnit),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getStyleFromPropValue: () => (/* binding */ getStyleFromPropValue),\n/* harmony export */   getValue: () => (/* binding */ getValue),\n/* harmony export */   margin: () => (/* binding */ margin),\n/* harmony export */   marginKeys: () => (/* binding */ marginKeys),\n/* harmony export */   padding: () => (/* binding */ padding),\n/* harmony export */   paddingKeys: () => (/* binding */ paddingKeys)\n/* harmony export */ });\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/system/esm/merge/merge.js\");\n/* harmony import */ var _memoize_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../memoize/index.js */ \"./node_modules/@mui/system/esm/memoize/memoize.js\");\n\n\n\n\n\nconst properties = {\n  m: 'margin',\n  p: 'padding'\n};\nconst directions = {\n  t: 'Top',\n  r: 'Right',\n  b: 'Bottom',\n  l: 'Left',\n  x: ['Left', 'Right'],\n  y: ['Top', 'Bottom']\n};\nconst aliases = {\n  marginX: 'mx',\n  marginY: 'my',\n  paddingX: 'px',\n  paddingY: 'py'\n};\n\n// memoize() impact:\n// From 300,000 ops/sec\n// To 350,000 ops/sec\nconst getCssProperties = (0,_memoize_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(prop => {\n  // It's not a shorthand notation.\n  if (prop.length > 2) {\n    if (aliases[prop]) {\n      prop = aliases[prop];\n    } else {\n      return [prop];\n    }\n  }\n  const [a, b] = prop.split('');\n  const property = properties[a];\n  const direction = directions[b] || '';\n  return Array.isArray(direction) ? direction.map(dir => property + dir) : [property + direction];\n});\nconst marginKeys = ['m', 'mt', 'mr', 'mb', 'ml', 'mx', 'my', 'margin', 'marginTop', 'marginRight', 'marginBottom', 'marginLeft', 'marginX', 'marginY', 'marginInline', 'marginInlineStart', 'marginInlineEnd', 'marginBlock', 'marginBlockStart', 'marginBlockEnd'];\nconst paddingKeys = ['p', 'pt', 'pr', 'pb', 'pl', 'px', 'py', 'padding', 'paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft', 'paddingX', 'paddingY', 'paddingInline', 'paddingInlineStart', 'paddingInlineEnd', 'paddingBlock', 'paddingBlockStart', 'paddingBlockEnd'];\nconst spacingKeys = [...marginKeys, ...paddingKeys];\nfunction createUnaryUnit(theme, themeKey, defaultValue, propName) {\n  const themeSpacing = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_1__.getPath)(theme, themeKey, true) ?? defaultValue;\n  if (typeof themeSpacing === 'number' || typeof themeSpacing === 'string') {\n    return val => {\n      if (typeof val === 'string') {\n        return val;\n      }\n      if (true) {\n        if (typeof val !== 'number') {\n          console.error(`MUI: Expected ${propName} argument to be a number or a string, got ${val}.`);\n        }\n      }\n      if (typeof themeSpacing === 'string') {\n        if (themeSpacing.startsWith('var(') && val === 0) {\n          return 0;\n        }\n        if (themeSpacing.startsWith('var(') && val === 1) {\n          return themeSpacing;\n        }\n        return `calc(${val} * ${themeSpacing})`;\n      }\n      return themeSpacing * val;\n    };\n  }\n  if (Array.isArray(themeSpacing)) {\n    return val => {\n      if (typeof val === 'string') {\n        return val;\n      }\n      const abs = Math.abs(val);\n      if (true) {\n        if (!Number.isInteger(abs)) {\n          console.error([`MUI: The \\`theme.${themeKey}\\` array type cannot be combined with non integer values.` + `You should either use an integer value that can be used as index, or define the \\`theme.${themeKey}\\` as a number.`].join('\\n'));\n        } else if (abs > themeSpacing.length - 1) {\n          console.error([`MUI: The value provided (${abs}) overflows.`, `The supported values are: ${JSON.stringify(themeSpacing)}.`, `${abs} > ${themeSpacing.length - 1}, you need to add the missing values.`].join('\\n'));\n        }\n      }\n      const transformed = themeSpacing[abs];\n      if (val >= 0) {\n        return transformed;\n      }\n      if (typeof transformed === 'number') {\n        return -transformed;\n      }\n      if (typeof transformed === 'string' && transformed.startsWith('var(')) {\n        return `calc(-1 * ${transformed})`;\n      }\n      return `-${transformed}`;\n    };\n  }\n  if (typeof themeSpacing === 'function') {\n    return themeSpacing;\n  }\n  if (true) {\n    console.error([`MUI: The \\`theme.${themeKey}\\` value (${themeSpacing}) is invalid.`, 'It should be a number, an array or a function.'].join('\\n'));\n  }\n  return () => undefined;\n}\nfunction createUnarySpacing(theme) {\n  return createUnaryUnit(theme, 'spacing', 8, 'spacing');\n}\nfunction getValue(transformer, propValue) {\n  if (typeof propValue === 'string' || propValue == null) {\n    return propValue;\n  }\n  return transformer(propValue);\n}\nfunction getStyleFromPropValue(cssProperties, transformer) {\n  return propValue => cssProperties.reduce((acc, cssProperty) => {\n    acc[cssProperty] = getValue(transformer, propValue);\n    return acc;\n  }, {});\n}\nfunction resolveCssProperty(props, keys, prop, transformer) {\n  // Using a hash computation over an array iteration could be faster, but with only 28 items,\n  // it's doesn't worth the bundle size.\n  if (!keys.includes(prop)) {\n    return null;\n  }\n  const cssProperties = getCssProperties(prop);\n  const styleFromPropValue = getStyleFromPropValue(cssProperties, transformer);\n  const propValue = props[prop];\n  return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)(props, propValue, styleFromPropValue);\n}\nfunction style(props, keys) {\n  const transformer = createUnarySpacing(props.theme);\n  return Object.keys(props).map(prop => resolveCssProperty(props, keys, prop, transformer)).reduce(_merge_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"], {});\n}\nfunction margin(props) {\n  return style(props, marginKeys);\n}\nmargin.propTypes =  true ? marginKeys.reduce((obj, key) => {\n  obj[key] = _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"];\n  return obj;\n}, {}) : 0;\nmargin.filterProps = marginKeys;\nfunction padding(props) {\n  return style(props, paddingKeys);\n}\npadding.propTypes =  true ? paddingKeys.reduce((obj, key) => {\n  obj[key] = _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"];\n  return obj;\n}, {}) : 0;\npadding.filterProps = paddingKeys;\nfunction spacing(props) {\n  return style(props, spacingKeys);\n}\nspacing.propTypes =  true ? spacingKeys.reduce((obj, key) => {\n  obj[key] = _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"];\n  return obj;\n}, {}) : 0;\nspacing.filterProps = spacingKeys;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (spacing);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/spacing/spacing.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/style/style.js":
/*!*****************************************************!*\
  !*** ./node_modules/@mui/system/esm/style/style.js ***!
  \*****************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getPath: () => (/* binding */ getPath),\n/* harmony export */   getStyleValue: () => (/* binding */ getStyleValue)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../responsivePropType/index.js */ \"./node_modules/@mui/system/esm/responsivePropType/responsivePropType.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n\n\n\nfunction getPath(obj, path, checkVars = true) {\n  if (!path || typeof path !== 'string') {\n    return null;\n  }\n\n  // Check if CSS variables are used\n  if (obj && obj.vars && checkVars) {\n    const val = `vars.${path}`.split('.').reduce((acc, item) => acc && acc[item] ? acc[item] : null, obj);\n    if (val != null) {\n      return val;\n    }\n  }\n  return path.split('.').reduce((acc, item) => {\n    if (acc && acc[item] != null) {\n      return acc[item];\n    }\n    return null;\n  }, obj);\n}\nfunction getStyleValue(themeMapping, transform, propValueFinal, userValue = propValueFinal) {\n  let value;\n  if (typeof themeMapping === 'function') {\n    value = themeMapping(propValueFinal);\n  } else if (Array.isArray(themeMapping)) {\n    value = themeMapping[propValueFinal] || userValue;\n  } else {\n    value = getPath(themeMapping, propValueFinal) || userValue;\n  }\n  if (transform) {\n    value = transform(value, userValue, themeMapping);\n  }\n  return value;\n}\nfunction style(options) {\n  const {\n    prop,\n    cssProperty = options.prop,\n    themeKey,\n    transform\n  } = options;\n\n  // false positive\n  // eslint-disable-next-line react/function-component-definition\n  const fn = props => {\n    if (props[prop] == null) {\n      return null;\n    }\n    const propValue = props[prop];\n    const theme = props.theme;\n    const themeMapping = getPath(theme, themeKey) || {};\n    const styleFromPropValue = propValueFinal => {\n      let value = getStyleValue(themeMapping, transform, propValueFinal);\n      if (propValueFinal === value && typeof propValueFinal === 'string') {\n        // Haven't found value\n        value = getStyleValue(themeMapping, transform, `${prop}${propValueFinal === 'default' ? '' : (0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(propValueFinal)}`, propValueFinal);\n      }\n      if (cssProperty === false) {\n        return value;\n      }\n      return {\n        [cssProperty]: value\n      };\n    };\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_1__.handleBreakpoints)(props, propValue, styleFromPropValue);\n  };\n  fn.propTypes =  true ? {\n    [prop]: _responsivePropType_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"]\n  } : 0;\n  fn.filterProps = [prop];\n  return fn;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (style);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/style/style.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js":
/*!*************************************************************************!*\
  !*** ./node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js ***!
  \*************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../spacing/index.js */ \"./node_modules/@mui/system/esm/spacing/spacing.js\");\n/* harmony import */ var _borders_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../borders/index.js */ \"./node_modules/@mui/system/esm/borders/borders.js\");\n/* harmony import */ var _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../cssGrid/index.js */ \"./node_modules/@mui/system/esm/cssGrid/cssGrid.js\");\n/* harmony import */ var _palette_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../palette/index.js */ \"./node_modules/@mui/system/esm/palette/palette.js\");\n/* harmony import */ var _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../sizing/index.js */ \"./node_modules/@mui/system/esm/sizing/sizing.js\");\n\n\n\n\n\nconst defaultSxConfig = {\n  // borders\n  border: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderTop: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderRight: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderBottom: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderLeft: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  borderColor: {\n    themeKey: 'palette'\n  },\n  borderTopColor: {\n    themeKey: 'palette'\n  },\n  borderRightColor: {\n    themeKey: 'palette'\n  },\n  borderBottomColor: {\n    themeKey: 'palette'\n  },\n  borderLeftColor: {\n    themeKey: 'palette'\n  },\n  outline: {\n    themeKey: 'borders',\n    transform: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderTransform\n  },\n  outlineColor: {\n    themeKey: 'palette'\n  },\n  borderRadius: {\n    themeKey: 'shape.borderRadius',\n    style: _borders_index_js__WEBPACK_IMPORTED_MODULE_0__.borderRadius\n  },\n  // palette\n  color: {\n    themeKey: 'palette',\n    transform: _palette_index_js__WEBPACK_IMPORTED_MODULE_1__.paletteTransform\n  },\n  bgcolor: {\n    themeKey: 'palette',\n    cssProperty: 'backgroundColor',\n    transform: _palette_index_js__WEBPACK_IMPORTED_MODULE_1__.paletteTransform\n  },\n  backgroundColor: {\n    themeKey: 'palette',\n    transform: _palette_index_js__WEBPACK_IMPORTED_MODULE_1__.paletteTransform\n  },\n  // spacing\n  p: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pt: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pr: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pb: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  pl: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  px: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  py: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  padding: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingTop: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingRight: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBottom: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingLeft: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingX: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingY: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingInline: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingInlineStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingInlineEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBlock: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBlockStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  paddingBlockEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.padding\n  },\n  m: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mt: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mr: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mb: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  ml: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  mx: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  my: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  margin: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginTop: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginRight: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBottom: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginLeft: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginX: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginY: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginInline: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginInlineStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginInlineEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBlock: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBlockStart: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  marginBlockEnd: {\n    style: _spacing_index_js__WEBPACK_IMPORTED_MODULE_2__.margin\n  },\n  // display\n  displayPrint: {\n    cssProperty: false,\n    transform: value => ({\n      '@media print': {\n        display: value\n      }\n    })\n  },\n  display: {},\n  overflow: {},\n  textOverflow: {},\n  visibility: {},\n  whiteSpace: {},\n  // flexbox\n  flexBasis: {},\n  flexDirection: {},\n  flexWrap: {},\n  justifyContent: {},\n  alignItems: {},\n  alignContent: {},\n  order: {},\n  flex: {},\n  flexGrow: {},\n  flexShrink: {},\n  alignSelf: {},\n  justifyItems: {},\n  justifySelf: {},\n  // grid\n  gap: {\n    style: _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__.gap\n  },\n  rowGap: {\n    style: _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__.rowGap\n  },\n  columnGap: {\n    style: _cssGrid_index_js__WEBPACK_IMPORTED_MODULE_3__.columnGap\n  },\n  gridColumn: {},\n  gridRow: {},\n  gridAutoFlow: {},\n  gridAutoColumns: {},\n  gridAutoRows: {},\n  gridTemplateColumns: {},\n  gridTemplateRows: {},\n  gridTemplateAreas: {},\n  gridArea: {},\n  // positions\n  position: {},\n  zIndex: {\n    themeKey: 'zIndex'\n  },\n  top: {},\n  right: {},\n  bottom: {},\n  left: {},\n  // shadows\n  boxShadow: {\n    themeKey: 'shadows'\n  },\n  // sizing\n  width: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  maxWidth: {\n    style: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.maxWidth\n  },\n  minWidth: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  height: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  maxHeight: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  minHeight: {\n    transform: _sizing_index_js__WEBPACK_IMPORTED_MODULE_4__.sizingTransform\n  },\n  boxSizing: {},\n  // typography\n  font: {\n    themeKey: 'font'\n  },\n  fontFamily: {\n    themeKey: 'typography'\n  },\n  fontSize: {\n    themeKey: 'typography'\n  },\n  fontStyle: {\n    themeKey: 'typography'\n  },\n  fontWeight: {\n    themeKey: 'typography'\n  },\n  letterSpacing: {},\n  textTransform: {},\n  lineHeight: {},\n  textAlign: {},\n  typography: {\n    cssProperty: false,\n    themeKey: 'typography'\n  }\n};\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (defaultSxConfig);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js":
/*!*************************************************************************!*\
  !*** ./node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js ***!
  \*************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   unstable_createStyleFunctionSx: () => (/* binding */ unstable_createStyleFunctionSx)\n/* harmony export */ });\n/* harmony import */ var _mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/utils/capitalize */ \"./node_modules/@mui/system/node_modules/@mui/utils/esm/capitalize/capitalize.js\");\n/* harmony import */ var _merge_index_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ../merge/index.js */ \"./node_modules/@mui/system/esm/merge/merge.js\");\n/* harmony import */ var _style_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../style/index.js */ \"./node_modules/@mui/system/esm/style/style.js\");\n/* harmony import */ var _breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../breakpoints/index.js */ \"./node_modules/@mui/system/esm/breakpoints/breakpoints.js\");\n/* harmony import */ var _cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../cssContainerQueries/index.js */ \"./node_modules/@mui/system/esm/cssContainerQueries/cssContainerQueries.js\");\n/* harmony import */ var _defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./defaultSxConfig.js */ \"./node_modules/@mui/system/esm/styleFunctionSx/defaultSxConfig.js\");\n\n\n\n\n\n\nfunction objectsHaveSameKeys(...objects) {\n  const allKeys = objects.reduce((keys, object) => keys.concat(Object.keys(object)), []);\n  const union = new Set(allKeys);\n  return objects.every(object => union.size === Object.keys(object).length);\n}\nfunction callIfFn(maybeFn, arg) {\n  return typeof maybeFn === 'function' ? maybeFn(arg) : maybeFn;\n}\n\n// eslint-disable-next-line @typescript-eslint/naming-convention\nfunction unstable_createStyleFunctionSx() {\n  function getThemeValue(prop, val, theme, config) {\n    const props = {\n      [prop]: val,\n      theme\n    };\n    const options = config[prop];\n    if (!options) {\n      return {\n        [prop]: val\n      };\n    }\n    const {\n      cssProperty = prop,\n      themeKey,\n      transform,\n      style\n    } = options;\n    if (val == null) {\n      return null;\n    }\n\n    // TODO v6: remove, see https://github.com/mui/material-ui/pull/38123\n    if (themeKey === 'typography' && val === 'inherit') {\n      return {\n        [prop]: val\n      };\n    }\n    const themeMapping = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__.getPath)(theme, themeKey) || {};\n    if (style) {\n      return style(props);\n    }\n    const styleFromPropValue = propValueFinal => {\n      let value = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__.getStyleValue)(themeMapping, transform, propValueFinal);\n      if (propValueFinal === value && typeof propValueFinal === 'string') {\n        // Haven't found value\n        value = (0,_style_index_js__WEBPACK_IMPORTED_MODULE_0__.getStyleValue)(themeMapping, transform, `${prop}${propValueFinal === 'default' ? '' : (0,_mui_utils_capitalize__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(propValueFinal)}`, propValueFinal);\n      }\n      if (cssProperty === false) {\n        return value;\n      }\n      return {\n        [cssProperty]: value\n      };\n    };\n    return (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)(props, val, styleFromPropValue);\n  }\n  function styleFunctionSx(props) {\n    const {\n      sx,\n      theme = {}\n    } = props || {};\n    if (!sx) {\n      return null; // Emotion & styled-components will neglect null\n    }\n    const config = theme.unstable_sxConfig ?? _defaultSxConfig_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"];\n\n    /*\n     * Receive `sxInput` as object or callback\n     * and then recursively check keys & values to create media query object styles.\n     * (the result will be used in `styled`)\n     */\n    function traverse(sxInput) {\n      let sxObject = sxInput;\n      if (typeof sxInput === 'function') {\n        sxObject = sxInput(theme);\n      } else if (typeof sxInput !== 'object') {\n        // value\n        return sxInput;\n      }\n      if (!sxObject) {\n        return null;\n      }\n      const emptyBreakpoints = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.createEmptyBreakpointObject)(theme.breakpoints);\n      const breakpointsKeys = Object.keys(emptyBreakpoints);\n      let css = emptyBreakpoints;\n      Object.keys(sxObject).forEach(styleKey => {\n        const value = callIfFn(sxObject[styleKey], theme);\n        if (value !== null && value !== undefined) {\n          if (typeof value === 'object') {\n            if (config[styleKey]) {\n              css = (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(css, getThemeValue(styleKey, value, theme, config));\n            } else {\n              const breakpointsValues = (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.handleBreakpoints)({\n                theme\n              }, value, x => ({\n                [styleKey]: x\n              }));\n              if (objectsHaveSameKeys(breakpointsValues, value)) {\n                css[styleKey] = styleFunctionSx({\n                  sx: value,\n                  theme\n                });\n              } else {\n                css = (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(css, breakpointsValues);\n              }\n            }\n          } else {\n            css = (0,_merge_index_js__WEBPACK_IMPORTED_MODULE_4__[\"default\"])(css, getThemeValue(styleKey, value, theme, config));\n          }\n        }\n      });\n      return (0,_cssContainerQueries_index_js__WEBPACK_IMPORTED_MODULE_5__.sortContainerQueries)(theme, (0,_breakpoints_index_js__WEBPACK_IMPORTED_MODULE_2__.removeUnusedBreakpoints)(breakpointsKeys, css));\n    }\n    return Array.isArray(sx) ? sx.map(traverse) : traverse(sx);\n  }\n  return styleFunctionSx;\n}\nconst styleFunctionSx = unstable_createStyleFunctionSx();\nstyleFunctionSx.filterProps = ['sx'];\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (styleFunctionSx);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js?");

/***/ }),

/***/ "./node_modules/@mui/system/esm/styled/styled.js":
/*!*******************************************************!*\
  !*** ./node_modules/@mui/system/esm/styled/styled.js ***!
  \*******************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _createStyled_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../createStyled/index.js */ \"./node_modules/@mui/system/esm/createStyled/createStyled.js\");\n\nconst styled = (0,_createStyled_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])();\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (styled);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/esm/styled/styled.js?");

/***/ }),

/***/ "./node_modules/@mui/system/node_modules/@mui/utils/esm/capitalize/capitalize.js":
/*!***************************************************************************************!*\
  !*** ./node_modules/@mui/system/node_modules/@mui/utils/esm/capitalize/capitalize.js ***!
  \***************************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ capitalize)\n/* harmony export */ });\n\n// It should to be noted that this function isn't equivalent to `text-transform: capitalize`.\n//\n// A strict capitalization should uppercase the first letter of each word in the sentence.\n// We only handle the first word.\nfunction capitalize(string) {\n  if (typeof string !== 'string') {\n    throw new Error( true ? 'MUI: `capitalize(string)` expects a string argument.' : 0);\n  }\n  return string.charAt(0).toUpperCase() + string.slice(1);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/node_modules/@mui/utils/esm/capitalize/capitalize.js?");

/***/ }),

/***/ "./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js":
/*!*************************************************************************************!*\
  !*** ./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js ***!
  \*************************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ deepmerge),\n/* harmony export */   isPlainObject: () => (/* binding */ isPlainObject)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n\n\n\n// https://github.com/sindresorhus/is-plain-obj/blob/main/index.js\nfunction isPlainObject(item) {\n  if (typeof item !== 'object' || item === null) {\n    return false;\n  }\n  const prototype = Object.getPrototypeOf(item);\n  return (prototype === null || prototype === Object.prototype || Object.getPrototypeOf(prototype) === null) && !(Symbol.toStringTag in item) && !(Symbol.iterator in item);\n}\nfunction deepClone(source) {\n  if (/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(source) || (0,react_is__WEBPACK_IMPORTED_MODULE_1__.isValidElementType)(source) || !isPlainObject(source)) {\n    return source;\n  }\n  const output = {};\n  Object.keys(source).forEach(key => {\n    output[key] = deepClone(source[key]);\n  });\n  return output;\n}\n\n/**\n * Merge objects deeply.\n * It will shallow copy React elements.\n *\n * If `options.clone` is set to `false` the source object will be merged directly into the target object.\n *\n * @example\n * ```ts\n * deepmerge({ a: { b: 1 }, d: 2 }, { a: { c: 2 }, d: 4 });\n * // => { a: { b: 1, c: 2 }, d: 4 }\n * ````\n *\n * @param target The target object.\n * @param source The source object.\n * @param options The merge options.\n * @param options.clone Set to `false` to merge the source object directly into the target object.\n * @returns The merged object.\n */\nfunction deepmerge(target, source, options = {\n  clone: true\n}) {\n  const output = options.clone ? {\n    ...target\n  } : target;\n  if (isPlainObject(target) && isPlainObject(source)) {\n    Object.keys(source).forEach(key => {\n      if (/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(source[key]) || (0,react_is__WEBPACK_IMPORTED_MODULE_1__.isValidElementType)(source[key])) {\n        output[key] = source[key];\n      } else if (isPlainObject(source[key]) &&\n      // Avoid prototype pollution\n      Object.prototype.hasOwnProperty.call(target, key) && isPlainObject(target[key])) {\n        // Since `output` is a clone of `target` and we have narrowed `target` in this block we can cast to the same type.\n        output[key] = deepmerge(target[key], source[key], options);\n      } else if (options.clone) {\n        output[key] = isPlainObject(source[key]) ? deepClone(source[key]) : source[key];\n      } else {\n        output[key] = source[key];\n      }\n    });\n  }\n  return output;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/node_modules/@mui/utils/esm/deepmerge/deepmerge.js?");

/***/ }),

/***/ "./node_modules/@mui/system/node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js":
/*!***********************************************************************************************!*\
  !*** ./node_modules/@mui/system/node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js ***!
  \***********************************************************************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getDisplayName)\n/* harmony export */ });\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n\nfunction getFunctionComponentName(Component, fallback = '') {\n  return Component.displayName || Component.name || fallback;\n}\nfunction getWrappedName(outerType, innerType, wrapperName) {\n  const functionName = getFunctionComponentName(innerType);\n  return outerType.displayName || (functionName !== '' ? `${wrapperName}(${functionName})` : wrapperName);\n}\n\n/**\n * cherry-pick from\n * https://github.com/facebook/react/blob/769b1f270e1251d9dbdce0fcbd9e92e502d059b8/packages/shared/getComponentName.js\n * originally forked from recompose/getDisplayName\n */\nfunction getDisplayName(Component) {\n  if (Component == null) {\n    return undefined;\n  }\n  if (typeof Component === 'string') {\n    return Component;\n  }\n  if (typeof Component === 'function') {\n    return getFunctionComponentName(Component, 'Component');\n  }\n\n  // TypeScript can't have components as objects but they exist in the form of `memo` or `Suspense`\n  if (typeof Component === 'object') {\n    switch (Component.$$typeof) {\n      case react_is__WEBPACK_IMPORTED_MODULE_0__.ForwardRef:\n        return getWrappedName(Component, Component.render, 'ForwardRef');\n      case react_is__WEBPACK_IMPORTED_MODULE_0__.Memo:\n        return getWrappedName(Component, Component.type, 'memo');\n      default:\n        return undefined;\n    }\n  }\n  return undefined;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/system/node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/ClassNameGenerator/ClassNameGenerator.js":
/*!******************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/ClassNameGenerator/ClassNameGenerator.js ***!
  \******************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nconst defaultGenerator = componentName => componentName;\nconst createClassNameGenerator = () => {\n  let generate = defaultGenerator;\n  return {\n    configure(generator) {\n      generate = generator;\n    },\n    generate(componentName) {\n      return generate(componentName);\n    },\n    reset() {\n      generate = defaultGenerator;\n    }\n  };\n};\nconst ClassNameGenerator = createClassNameGenerator();\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (ClassNameGenerator);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/ClassNameGenerator/ClassNameGenerator.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ HTMLElementType)\n/* harmony export */ });\nfunction HTMLElementType(props, propName, componentName, location, propFullName) {\n  if (false) // removed by dead control flow\n{}\n  const propValue = props[propName];\n  const safePropName = propFullName || propName;\n  if (propValue == null) {\n    return null;\n  }\n  if (propValue && propValue.nodeType !== 1) {\n    return new Error(`Invalid ${location} \\`${safePropName}\\` supplied to \\`${componentName}\\`. ` + `Expected an HTMLElement.`);\n  }\n  return null;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/HTMLElementType/HTMLElementType.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/appendOwnerState/appendOwnerState.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/appendOwnerState/appendOwnerState.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _isHostComponent_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../isHostComponent/index.js */ \"./node_modules/@mui/utils/esm/isHostComponent/isHostComponent.js\");\n\n\n/**\n * Type of the ownerState based on the type of an element it applies to.\n * This resolves to the provided OwnerState for React components and `undefined` for host components.\n * Falls back to `OwnerState | undefined` when the exact type can't be determined in development time.\n */\n\n/**\n * Appends the ownerState object to the props, merging with the existing one if necessary.\n *\n * @param elementType Type of the element that owns the `existingProps`. If the element is a DOM node or undefined, `ownerState` is not applied.\n * @param otherProps Props of the element.\n * @param ownerState\n */\nfunction appendOwnerState(elementType, otherProps, ownerState) {\n  if (elementType === undefined || (0,_isHostComponent_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(elementType)) {\n    return otherProps;\n  }\n  return {\n    ...otherProps,\n    ownerState: {\n      ...otherProps.ownerState,\n      ...ownerState\n    }\n  };\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (appendOwnerState);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/appendOwnerState/appendOwnerState.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/capitalize/capitalize.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/capitalize/capitalize.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ capitalize)\n/* harmony export */ });\n\n// It should to be noted that this function isn't equivalent to `text-transform: capitalize`.\n//\n// A strict capitalization should uppercase the first letter of each word in the sentence.\n// We only handle the first word.\nfunction capitalize(string) {\n  if (typeof string !== 'string') {\n    throw new Error( true ? 'MUI: `capitalize(string)` expects a string argument.' : 0);\n  }\n  return string.charAt(0).toUpperCase() + string.slice(1);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/capitalize/capitalize.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ chainPropTypes)\n/* harmony export */ });\nfunction chainPropTypes(propType1, propType2) {\n  if (false) // removed by dead control flow\n{}\n  return function validate(...args) {\n    return propType1(...args) || propType2(...args);\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/clamp/clamp.js":
/*!****************************************************!*\
  !*** ./node_modules/@mui/utils/esm/clamp/clamp.js ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nfunction clamp(val, min = Number.MIN_SAFE_INTEGER, max = Number.MAX_SAFE_INTEGER) {\n  return Math.max(min, Math.min(val, max));\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (clamp);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/clamp/clamp.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/composeClasses/composeClasses.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/composeClasses/composeClasses.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ composeClasses)\n/* harmony export */ });\n/* eslint no-restricted-syntax: 0, prefer-template: 0, guard-for-in: 0\n   ---\n   These rules are preventing the performance optimizations below.\n */\n\n/**\n * Compose classes from multiple sources.\n *\n * @example\n * ```tsx\n * const slots = {\n *  root: ['root', 'primary'],\n *  label: ['label'],\n * };\n *\n * const getUtilityClass = (slot) => `MuiButton-${slot}`;\n *\n * const classes = {\n *   root: 'my-root-class',\n * };\n *\n * const output = composeClasses(slots, getUtilityClass, classes);\n * // {\n * //   root: 'MuiButton-root MuiButton-primary my-root-class',\n * //   label: 'MuiButton-label',\n * // }\n * ```\n *\n * @param slots a list of classes for each possible slot\n * @param getUtilityClass a function to resolve the class based on the slot name\n * @param classes the input classes from props\n * @returns the resolved classes for all slots\n */\nfunction composeClasses(slots, getUtilityClass, classes = undefined) {\n  const output = {};\n  for (const slotName in slots) {\n    const slot = slots[slotName];\n    let buffer = '';\n    let start = true;\n    for (let i = 0; i < slot.length; i += 1) {\n      const value = slot[i];\n      if (value) {\n        buffer += (start === true ? '' : ' ') + getUtilityClass(value);\n        start = false;\n        if (classes && classes[value]) {\n          buffer += ' ' + classes[value];\n        }\n      }\n    }\n    output[slotName] = buffer;\n  }\n  return output;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/composeClasses/composeClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/createChainedFunction/createChainedFunction.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/createChainedFunction/createChainedFunction.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ createChainedFunction)\n/* harmony export */ });\n/**\n * Safe chained function.\n *\n * Will only create a new function if needed,\n * otherwise will pass back existing functions or null.\n */\nfunction createChainedFunction(...funcs) {\n  return funcs.reduce((acc, func) => {\n    if (func == null) {\n      return acc;\n    }\n    return function chainedFunction(...args) {\n      acc.apply(this, args);\n      func.apply(this, args);\n    };\n  }, () => {});\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/createChainedFunction/createChainedFunction.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/debounce/debounce.js":
/*!**********************************************************!*\
  !*** ./node_modules/@mui/utils/esm/debounce/debounce.js ***!
  \**********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ debounce)\n/* harmony export */ });\n// Corresponds to 10 frames at 60 Hz.\n// A few bytes payload overhead when lodash/debounce is ~3 kB and debounce ~300 B.\nfunction debounce(func, wait = 166) {\n  let timeout;\n  function debounced(...args) {\n    const later = () => {\n      // @ts-ignore\n      func.apply(this, args);\n    };\n    clearTimeout(timeout);\n    timeout = setTimeout(later, wait);\n  }\n  debounced.clear = () => {\n    clearTimeout(timeout);\n  };\n  return debounced;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/debounce/debounce.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/deepmerge/deepmerge.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/deepmerge/deepmerge.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ deepmerge),\n/* harmony export */   isPlainObject: () => (/* binding */ isPlainObject)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n\n\n\n// https://github.com/sindresorhus/is-plain-obj/blob/main/index.js\nfunction isPlainObject(item) {\n  if (typeof item !== 'object' || item === null) {\n    return false;\n  }\n  const prototype = Object.getPrototypeOf(item);\n  return (prototype === null || prototype === Object.prototype || Object.getPrototypeOf(prototype) === null) && !(Symbol.toStringTag in item) && !(Symbol.iterator in item);\n}\nfunction deepClone(source) {\n  if (/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(source) || (0,react_is__WEBPACK_IMPORTED_MODULE_1__.isValidElementType)(source) || !isPlainObject(source)) {\n    return source;\n  }\n  const output = {};\n  Object.keys(source).forEach(key => {\n    output[key] = deepClone(source[key]);\n  });\n  return output;\n}\n\n/**\n * Merge objects deeply.\n * It will shallow copy React elements.\n *\n * If `options.clone` is set to `false` the source object will be merged directly into the target object.\n *\n * @example\n * ```ts\n * deepmerge({ a: { b: 1 }, d: 2 }, { a: { c: 2 }, d: 4 });\n * // => { a: { b: 1, c: 2 }, d: 4 }\n * ````\n *\n * @param target The target object.\n * @param source The source object.\n * @param options The merge options.\n * @param options.clone Set to `false` to merge the source object directly into the target object.\n * @returns The merged object.\n */\nfunction deepmerge(target, source, options = {\n  clone: true\n}) {\n  const output = options.clone ? {\n    ...target\n  } : target;\n  if (isPlainObject(target) && isPlainObject(source)) {\n    Object.keys(source).forEach(key => {\n      if (/*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(source[key]) || (0,react_is__WEBPACK_IMPORTED_MODULE_1__.isValidElementType)(source[key])) {\n        output[key] = source[key];\n      } else if (isPlainObject(source[key]) &&\n      // Avoid prototype pollution\n      Object.prototype.hasOwnProperty.call(target, key) && isPlainObject(target[key])) {\n        // Since `output` is a clone of `target` and we have narrowed `target` in this block we can cast to the same type.\n        output[key] = deepmerge(target[key], source[key], options);\n      } else if (options.clone) {\n        output[key] = isPlainObject(source[key]) ? deepClone(source[key]) : source[key];\n      } else {\n        output[key] = source[key];\n      }\n    });\n  }\n  return output;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/deepmerge/deepmerge.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js":
/*!********************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js ***!
  \********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _chainPropTypes_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../chainPropTypes/index.js */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n\n\nfunction isClassComponent(elementType) {\n  // elementType.prototype?.isReactComponent\n  const {\n    prototype = {}\n  } = elementType;\n  return Boolean(prototype.isReactComponent);\n}\nfunction acceptingRef(props, propName, componentName, location, propFullName) {\n  const element = props[propName];\n  const safePropName = propFullName || propName;\n  if (element == null ||\n  // When server-side rendering React doesn't warn either.\n  // This is not an accurate check for SSR.\n  // This is only in place for Emotion compat.\n  // TODO: Revisit once https://github.com/facebook/react/issues/20047 is resolved.\n  typeof window === 'undefined') {\n    return null;\n  }\n  let warningHint;\n  const elementType = element.type;\n  /**\n   * Blacklisting instead of whitelisting\n   *\n   * Blacklisting will miss some components, such as React.Fragment. Those will at least\n   * trigger a warning in React.\n   * We can't whitelist because there is no safe way to detect React.forwardRef\n   * or class components. \"Safe\" means there's no public API.\n   *\n   */\n  if (typeof elementType === 'function' && !isClassComponent(elementType)) {\n    warningHint = 'Did you accidentally use a plain function component for an element instead?';\n  }\n  if (warningHint !== undefined) {\n    return new Error(`Invalid ${location} \\`${safePropName}\\` supplied to \\`${componentName}\\`. ` + `Expected an element that can hold a ref. ${warningHint} ` + 'For more information see https://mui.com/r/caveat-with-refs-guide');\n  }\n  return null;\n}\nconst elementAcceptingRef = (0,_chainPropTypes_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])((prop_types__WEBPACK_IMPORTED_MODULE_1___default().element), acceptingRef);\nelementAcceptingRef.isRequired = (0,_chainPropTypes_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])((prop_types__WEBPACK_IMPORTED_MODULE_1___default().element).isRequired, acceptingRef);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (elementAcceptingRef);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/elementAcceptingRef/elementAcceptingRef.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/elementTypeAcceptingRef/elementTypeAcceptingRef.js":
/*!****************************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/elementTypeAcceptingRef/elementTypeAcceptingRef.js ***!
  \****************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _chainPropTypes_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../chainPropTypes/index.js */ \"./node_modules/@mui/utils/esm/chainPropTypes/chainPropTypes.js\");\n\n\nfunction isClassComponent(elementType) {\n  // elementType.prototype?.isReactComponent\n  const {\n    prototype = {}\n  } = elementType;\n  return Boolean(prototype.isReactComponent);\n}\nfunction elementTypeAcceptingRef(props, propName, componentName, location, propFullName) {\n  const propValue = props[propName];\n  const safePropName = propFullName || propName;\n  if (propValue == null ||\n  // When server-side rendering React doesn't warn either.\n  // This is not an accurate check for SSR.\n  // This is only in place for emotion compat.\n  // TODO: Revisit once https://github.com/facebook/react/issues/20047 is resolved.\n  typeof window === 'undefined') {\n    return null;\n  }\n  let warningHint;\n\n  /**\n   * Blacklisting instead of whitelisting\n   *\n   * Blacklisting will miss some components, such as React.Fragment. Those will at least\n   * trigger a warning in React.\n   * We can't whitelist because there is no safe way to detect React.forwardRef\n   * or class components. \"Safe\" means there's no public API.\n   *\n   */\n  if (typeof propValue === 'function' && !isClassComponent(propValue)) {\n    warningHint = 'Did you accidentally provide a plain function component instead?';\n  }\n  if (warningHint !== undefined) {\n    return new Error(`Invalid ${location} \\`${safePropName}\\` supplied to \\`${componentName}\\`. ` + `Expected an element type that can hold a ref. ${warningHint} ` + 'For more information see https://mui.com/r/caveat-with-refs-guide');\n  }\n  return null;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ((0,_chainPropTypes_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])((prop_types__WEBPACK_IMPORTED_MODULE_1___default().elementType), elementTypeAcceptingRef));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/elementTypeAcceptingRef/elementTypeAcceptingRef.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/exactProp/exactProp.js":
/*!************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/exactProp/exactProp.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ exactProp)\n/* harmony export */ });\n// This module is based on https://github.com/airbnb/prop-types-exact repository.\n// However, in order to reduce the number of dependencies and to remove some extra safe checks\n// the module was forked.\n\nconst specialProperty = 'exact-prop: \\u200b';\nfunction exactProp(propTypes) {\n  if (false) // removed by dead control flow\n{}\n  return {\n    ...propTypes,\n    [specialProperty]: props => {\n      const unsupportedProps = Object.keys(props).filter(prop => !propTypes.hasOwnProperty(prop));\n      if (unsupportedProps.length > 0) {\n        return new Error(`The following props are not supported: ${unsupportedProps.map(prop => `\\`${prop}\\``).join(', ')}. Please remove them.`);\n      }\n      return null;\n    }\n  };\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/exactProp/exactProp.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/extractEventHandlers/extractEventHandlers.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/extractEventHandlers/extractEventHandlers.js ***!
  \**********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/**\n * Extracts event handlers from a given object.\n * A prop is considered an event handler if it is a function and its name starts with `on`.\n *\n * @param object An object to extract event handlers from.\n * @param excludeKeys An array of keys to exclude from the returned object.\n */\nfunction extractEventHandlers(object, excludeKeys = []) {\n  if (object === undefined) {\n    return {};\n  }\n  const result = {};\n  Object.keys(object).filter(prop => prop.match(/^on[A-Z]/) && typeof object[prop] === 'function' && !excludeKeys.includes(prop)).forEach(prop => {\n    result[prop] = object[prop];\n  });\n  return result;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (extractEventHandlers);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/extractEventHandlers/extractEventHandlers.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js ***!
  \**********************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ generateUtilityClass),\n/* harmony export */   globalStateClasses: () => (/* binding */ globalStateClasses),\n/* harmony export */   isGlobalState: () => (/* binding */ isGlobalState)\n/* harmony export */ });\n/* harmony import */ var _ClassNameGenerator_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../ClassNameGenerator/index.js */ \"./node_modules/@mui/utils/esm/ClassNameGenerator/ClassNameGenerator.js\");\n\nconst globalStateClasses = {\n  active: 'active',\n  checked: 'checked',\n  completed: 'completed',\n  disabled: 'disabled',\n  error: 'error',\n  expanded: 'expanded',\n  focused: 'focused',\n  focusVisible: 'focusVisible',\n  open: 'open',\n  readOnly: 'readOnly',\n  required: 'required',\n  selected: 'selected'\n};\nfunction generateUtilityClass(componentName, slot, globalStatePrefix = 'Mui') {\n  const globalStateClass = globalStateClasses[slot];\n  return globalStateClass ? `${globalStatePrefix}-${globalStateClass}` : `${_ClassNameGenerator_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"].generate(componentName)}-${slot}`;\n}\nfunction isGlobalState(slot) {\n  return globalStateClasses[slot] !== undefined;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js":
/*!**************************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js ***!
  \**************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ generateUtilityClasses)\n/* harmony export */ });\n/* harmony import */ var _generateUtilityClass_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../generateUtilityClass/index.js */ \"./node_modules/@mui/utils/esm/generateUtilityClass/generateUtilityClass.js\");\n\nfunction generateUtilityClasses(componentName, slots, globalStatePrefix = 'Mui') {\n  const result = {};\n  slots.forEach(slot => {\n    result[slot] = (0,_generateUtilityClass_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(componentName, slot, globalStatePrefix);\n  });\n  return result;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/generateUtilityClasses/generateUtilityClasses.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getDisplayName)\n/* harmony export */ });\n/* harmony import */ var react_is__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react-is */ \"./node_modules/react-is/cjs/react-is.development.js\");\n\nfunction getFunctionComponentName(Component, fallback = '') {\n  return Component.displayName || Component.name || fallback;\n}\nfunction getWrappedName(outerType, innerType, wrapperName) {\n  const functionName = getFunctionComponentName(innerType);\n  return outerType.displayName || (functionName !== '' ? `${wrapperName}(${functionName})` : wrapperName);\n}\n\n/**\n * cherry-pick from\n * https://github.com/facebook/react/blob/769b1f270e1251d9dbdce0fcbd9e92e502d059b8/packages/shared/getComponentName.js\n * originally forked from recompose/getDisplayName\n */\nfunction getDisplayName(Component) {\n  if (Component == null) {\n    return undefined;\n  }\n  if (typeof Component === 'string') {\n    return Component;\n  }\n  if (typeof Component === 'function') {\n    return getFunctionComponentName(Component, 'Component');\n  }\n\n  // TypeScript can't have components as objects but they exist in the form of `memo` or `Suspense`\n  if (typeof Component === 'object') {\n    switch (Component.$$typeof) {\n      case react_is__WEBPACK_IMPORTED_MODULE_0__.ForwardRef:\n        return getWrappedName(Component, Component.render, 'ForwardRef');\n      case react_is__WEBPACK_IMPORTED_MODULE_0__.Memo:\n        return getWrappedName(Component, Component.type, 'memo');\n      default:\n        return undefined;\n    }\n  }\n  return undefined;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/getDisplayName/getDisplayName.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js":
/*!******************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js ***!
  \******************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getReactElementRef)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n\n\n/**\n * Returns the ref of a React element handling differences between React 19 and older versions.\n * It will throw runtime error if the element is not a valid React element.\n *\n * @param element React.ReactElement\n * @returns React.Ref<any> | null\n */\nfunction getReactElementRef(element) {\n  // 'ref' is passed as prop in React 19, whereas 'ref' is directly attached to children in older versions\n  if (parseInt(react__WEBPACK_IMPORTED_MODULE_0__.version, 10) >= 19) {\n    return element?.props?.ref || null;\n  }\n  // @ts-expect-error element.ref is not included in the ReactElement type\n  // https://github.com/DefinitelyTyped/DefinitelyTyped/discussions/70189\n  return element?.ref || null;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/getReactElementRef/getReactElementRef.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/getScrollbarSize/getScrollbarSize.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/getScrollbarSize/getScrollbarSize.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getScrollbarSize)\n/* harmony export */ });\n// A change of the browser zoom change the scrollbar size.\n// Credit https://github.com/twbs/bootstrap/blob/488fd8afc535ca3a6ad4dc581f5e89217b6a36ac/js/src/util/scrollbar.js#L14-L18\nfunction getScrollbarSize(win = window) {\n  // https://developer.mozilla.org/en-US/docs/Web/API/Window/innerWidth#usage_notes\n  const documentWidth = win.document.documentElement.clientWidth;\n  return win.innerWidth - documentWidth;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/getScrollbarSize/getScrollbarSize.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/getValidReactChildren/getValidReactChildren.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/getValidReactChildren/getValidReactChildren.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ getValidReactChildren)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n\n\n/**\n * Gets only the valid children of a component,\n * and ignores any nullish or falsy child.\n *\n * @param children the children\n */\nfunction getValidReactChildren(children) {\n  return react__WEBPACK_IMPORTED_MODULE_0__.Children.toArray(children).filter(child => /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(child));\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/getValidReactChildren/getValidReactChildren.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/integerPropType/integerPropType.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/integerPropType/integerPropType.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__),\n/* harmony export */   getTypeByValue: () => (/* binding */ getTypeByValue)\n/* harmony export */ });\nfunction getTypeByValue(value) {\n  const valueType = typeof value;\n  switch (valueType) {\n    case 'number':\n      if (Number.isNaN(value)) {\n        return 'NaN';\n      }\n      if (!Number.isFinite(value)) {\n        return 'Infinity';\n      }\n      if (value !== Math.floor(value)) {\n        return 'float';\n      }\n      return 'number';\n    case 'object':\n      if (value === null) {\n        return 'null';\n      }\n      return value.constructor.name;\n    default:\n      return valueType;\n  }\n}\nfunction requiredInteger(props, propName, componentName, location) {\n  const propValue = props[propName];\n  if (propValue == null || !Number.isInteger(propValue)) {\n    const propType = getTypeByValue(propValue);\n    return new RangeError(`Invalid ${location} \\`${propName}\\` of type \\`${propType}\\` supplied to \\`${componentName}\\`, expected \\`integer\\`.`);\n  }\n  return null;\n}\nfunction validator(props, propName, ...other) {\n  const propValue = props[propName];\n  if (propValue === undefined) {\n    return null;\n  }\n  return requiredInteger(props, propName, ...other);\n}\nfunction validatorNoop() {\n  return null;\n}\nvalidator.isRequired = requiredInteger;\nvalidatorNoop.isRequired = validatorNoop;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ( false ? 0 : validator);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/integerPropType/integerPropType.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/isFocusVisible/isFocusVisible.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/isFocusVisible/isFocusVisible.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ isFocusVisible)\n/* harmony export */ });\n/**\n * Returns a boolean indicating if the event's target has :focus-visible\n */\nfunction isFocusVisible(element) {\n  try {\n    return element.matches(':focus-visible');\n  } catch (error) {\n    // Do not warn on jsdom tests, otherwise all tests that rely on focus have to be skipped\n    // Tests that rely on `:focus-visible` will still have to be skipped in jsdom\n    if ( true && !/jsdom/.test(window.navigator.userAgent)) {\n      console.warn(['MUI: The `:focus-visible` pseudo class is not supported in this browser.', 'Some components rely on this feature to work properly.'].join('\\n'));\n    }\n  }\n  return false;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/isFocusVisible/isFocusVisible.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/isHostComponent/isHostComponent.js":
/*!************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/isHostComponent/isHostComponent.js ***!
  \************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/**\n * Determines if a given element is a DOM element name (i.e. not a React component).\n */\nfunction isHostComponent(element) {\n  return typeof element === 'string';\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (isHostComponent);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/isHostComponent/isHostComponent.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/isMuiElement/isMuiElement.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/isMuiElement/isMuiElement.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ isMuiElement)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n\nfunction isMuiElement(element, muiNames) {\n  return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_0__.isValidElement(element) && muiNames.indexOf(\n  // For server components `muiName` is avaialble in element.type._payload.value.muiName\n  // relevant info - https://github.com/facebook/react/blob/2807d781a08db8e9873687fccc25c0f12b4fb3d4/packages/react/src/ReactLazy.js#L45\n  // eslint-disable-next-line no-underscore-dangle\n  element.type.muiName ?? element.type?._payload?.value?.muiName) !== -1;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/isMuiElement/isMuiElement.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/mergeSlotProps/mergeSlotProps.js":
/*!**********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/mergeSlotProps/mergeSlotProps.js ***!
  \**********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var clsx__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! clsx */ \"./node_modules/clsx/dist/clsx.mjs\");\n/* harmony import */ var _extractEventHandlers_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../extractEventHandlers/index.js */ \"./node_modules/@mui/utils/esm/extractEventHandlers/extractEventHandlers.js\");\n/* harmony import */ var _omitEventHandlers_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../omitEventHandlers/index.js */ \"./node_modules/@mui/utils/esm/omitEventHandlers/omitEventHandlers.js\");\n\n\n\n/**\n * Merges the slot component internal props (usually coming from a hook)\n * with the externally provided ones.\n *\n * The merge order is (the latter overrides the former):\n * 1. The internal props (specified as a getter function to work with get*Props hook result)\n * 2. Additional props (specified internally on a Base UI component)\n * 3. External props specified on the owner component. These should only be used on a root slot.\n * 4. External props specified in the `slotProps.*` prop.\n * 5. The `className` prop - combined from all the above.\n * @param parameters\n * @returns\n */\nfunction mergeSlotProps(parameters) {\n  const {\n    getSlotProps,\n    additionalProps,\n    externalSlotProps,\n    externalForwardedProps,\n    className\n  } = parameters;\n  if (!getSlotProps) {\n    // The simpler case - getSlotProps is not defined, so no internal event handlers are defined,\n    // so we can simply merge all the props without having to worry about extracting event handlers.\n    const joinedClasses = (0,clsx__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(additionalProps?.className, className, externalForwardedProps?.className, externalSlotProps?.className);\n    const mergedStyle = {\n      ...additionalProps?.style,\n      ...externalForwardedProps?.style,\n      ...externalSlotProps?.style\n    };\n    const props = {\n      ...additionalProps,\n      ...externalForwardedProps,\n      ...externalSlotProps\n    };\n    if (joinedClasses.length > 0) {\n      props.className = joinedClasses;\n    }\n    if (Object.keys(mergedStyle).length > 0) {\n      props.style = mergedStyle;\n    }\n    return {\n      props,\n      internalRef: undefined\n    };\n  }\n\n  // In this case, getSlotProps is responsible for calling the external event handlers.\n  // We don't need to include them in the merged props because of this.\n\n  const eventHandlers = (0,_extractEventHandlers_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n    ...externalForwardedProps,\n    ...externalSlotProps\n  });\n  const componentsPropsWithoutEventHandlers = (0,_omitEventHandlers_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(externalSlotProps);\n  const otherPropsWithoutEventHandlers = (0,_omitEventHandlers_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(externalForwardedProps);\n  const internalSlotProps = getSlotProps(eventHandlers);\n\n  // The order of classes is important here.\n  // Emotion (that we use in libraries consuming Base UI) depends on this order\n  // to properly override style. It requires the most important classes to be last\n  // (see https://github.com/mui/material-ui/pull/33205) for the related discussion.\n  const joinedClasses = (0,clsx__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(internalSlotProps?.className, additionalProps?.className, className, externalForwardedProps?.className, externalSlotProps?.className);\n  const mergedStyle = {\n    ...internalSlotProps?.style,\n    ...additionalProps?.style,\n    ...externalForwardedProps?.style,\n    ...externalSlotProps?.style\n  };\n  const props = {\n    ...internalSlotProps,\n    ...additionalProps,\n    ...otherPropsWithoutEventHandlers,\n    ...componentsPropsWithoutEventHandlers\n  };\n  if (joinedClasses.length > 0) {\n    props.className = joinedClasses;\n  }\n  if (Object.keys(mergedStyle).length > 0) {\n    props.style = mergedStyle;\n  }\n  return {\n    props,\n    internalRef: internalSlotProps.ref\n  };\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (mergeSlotProps);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/mergeSlotProps/mergeSlotProps.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/omitEventHandlers/omitEventHandlers.js":
/*!****************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/omitEventHandlers/omitEventHandlers.js ***!
  \****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/**\n * Removes event handlers from the given object.\n * A field is considered an event handler if it is a function with a name beginning with `on`.\n *\n * @param object Object to remove event handlers from.\n * @returns Object with event handlers removed.\n */\nfunction omitEventHandlers(object) {\n  if (object === undefined) {\n    return {};\n  }\n  const result = {};\n  Object.keys(object).filter(prop => !(prop.match(/^on[A-Z]/) && typeof object[prop] === 'function')).forEach(prop => {\n    result[prop] = object[prop];\n  });\n  return result;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (omitEventHandlers);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/omitEventHandlers/omitEventHandlers.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ ownerDocument)\n/* harmony export */ });\nfunction ownerDocument(node) {\n  return node && node.ownerDocument || document;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/ownerWindow/ownerWindow.js":
/*!****************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/ownerWindow/ownerWindow.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ ownerWindow)\n/* harmony export */ });\n/* harmony import */ var _ownerDocument_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../ownerDocument/index.js */ \"./node_modules/@mui/utils/esm/ownerDocument/ownerDocument.js\");\n\nfunction ownerWindow(node) {\n  const doc = (0,_ownerDocument_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(node);\n  return doc.defaultView || window;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/ownerWindow/ownerWindow.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/refType/refType.js":
/*!********************************************************!*\
  !*** ./node_modules/@mui/utils/esm/refType/refType.js ***!
  \********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_0__);\n\nconst refType = prop_types__WEBPACK_IMPORTED_MODULE_0___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_0___default().func), (prop_types__WEBPACK_IMPORTED_MODULE_0___default().object)]);\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (refType);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/refType/refType.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/resolveComponentProps/resolveComponentProps.js":
/*!************************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/resolveComponentProps/resolveComponentProps.js ***!
  \************************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/**\n * If `componentProps` is a function, calls it with the provided `ownerState`.\n * Otherwise, just returns `componentProps`.\n */\nfunction resolveComponentProps(componentProps, ownerState, slotState) {\n  if (typeof componentProps === 'function') {\n    return componentProps(ownerState, slotState);\n  }\n  return componentProps;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (resolveComponentProps);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/resolveComponentProps/resolveComponentProps.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/resolveProps/resolveProps.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/resolveProps/resolveProps.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ resolveProps)\n/* harmony export */ });\n/**\n * Add keys, values of `defaultProps` that does not exist in `props`\n * @param defaultProps\n * @param props\n * @returns resolved props\n */\nfunction resolveProps(defaultProps, props) {\n  const output = {\n    ...props\n  };\n  for (const key in defaultProps) {\n    if (Object.prototype.hasOwnProperty.call(defaultProps, key)) {\n      const propName = key;\n      if (propName === 'components' || propName === 'slots') {\n        output[propName] = {\n          ...defaultProps[propName],\n          ...output[propName]\n        };\n      } else if (propName === 'componentsProps' || propName === 'slotProps') {\n        const defaultSlotProps = defaultProps[propName];\n        const slotProps = props[propName];\n        if (!slotProps) {\n          output[propName] = defaultSlotProps || {};\n        } else if (!defaultSlotProps) {\n          output[propName] = slotProps;\n        } else {\n          output[propName] = {\n            ...slotProps\n          };\n          for (const slotKey in defaultSlotProps) {\n            if (Object.prototype.hasOwnProperty.call(defaultSlotProps, slotKey)) {\n              const slotPropName = slotKey;\n              output[propName][slotPropName] = resolveProps(defaultSlotProps[slotPropName], slotProps[slotPropName]);\n            }\n          }\n        }\n      } else if (output[propName] === undefined) {\n        output[propName] = defaultProps[propName];\n      }\n    }\n  }\n  return output;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/resolveProps/resolveProps.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/setRef/setRef.js":
/*!******************************************************!*\
  !*** ./node_modules/@mui/utils/esm/setRef/setRef.js ***!
  \******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ setRef)\n/* harmony export */ });\n/**\n * TODO v5: consider making it private\n *\n * passes {value} to {ref}\n *\n * WARNING: Be sure to only call this inside a callback that is passed as a ref.\n * Otherwise, make sure to cleanup the previous {ref} if it changes. See\n * https://github.com/mui/material-ui/issues/13539\n *\n * Useful if you want to expose the ref of an inner component to the public API\n * while still using it inside the component.\n * @param ref A ref callback or ref object. If anything falsy, this is a no-op.\n */\nfunction setRef(ref, value) {\n  if (typeof ref === 'function') {\n    ref(value);\n  } else if (ref) {\n    ref.current = value;\n  }\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/setRef/setRef.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useControlled/useControlled.js":
/*!********************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useControlled/useControlled.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useControlled)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n// TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler -- process.env never changes, dependency arrays are intentionally ignored\n/* eslint-disable react-hooks/rules-of-hooks, react-hooks/exhaustive-deps */\n\nfunction useControlled({\n  controlled,\n  default: defaultProp,\n  name,\n  state = 'value'\n}) {\n  // isControlled is ignored in the hook dependency lists as it should never change.\n  const {\n    current: isControlled\n  } = react__WEBPACK_IMPORTED_MODULE_0__.useRef(controlled !== undefined);\n  const [valueState, setValue] = react__WEBPACK_IMPORTED_MODULE_0__.useState(defaultProp);\n  const value = isControlled ? controlled : valueState;\n  if (true) {\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n      if (isControlled !== (controlled !== undefined)) {\n        console.error([`MUI: A component is changing the ${isControlled ? '' : 'un'}controlled ${state} state of ${name} to be ${isControlled ? 'un' : ''}controlled.`, 'Elements should not switch from uncontrolled to controlled (or vice versa).', `Decide between using a controlled or uncontrolled ${name} ` + 'element for the lifetime of the component.', \"The nature of the state is determined during the first render. It's considered controlled if the value is not `undefined`.\", 'More info: https://fb.me/react-controlled-components'].join('\\n'));\n      }\n    }, [state, name, controlled]);\n    const {\n      current: defaultValue\n    } = react__WEBPACK_IMPORTED_MODULE_0__.useRef(defaultProp);\n    react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n      // Object.is() is not equivalent to the === operator.\n      // See https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is for more details.\n      if (!isControlled && !Object.is(defaultValue, defaultProp)) {\n        console.error([`MUI: A component is changing the default ${state} state of an uncontrolled ${name} after being initialized. ` + `To suppress this warning opt to use a controlled ${name}.`].join('\\n'));\n      }\n    }, [JSON.stringify(defaultProp)]);\n  }\n  const setValueIfUncontrolled = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(newValue => {\n    if (!isControlled) {\n      setValue(newValue);\n    }\n  }, []);\n  return [value, setValueIfUncontrolled];\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useControlled/useControlled.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js":
/*!****************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js ***!
  \****************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n\n/**\n * A version of `React.useLayoutEffect` that does not show a warning when server-side rendering.\n * This is useful for effects that are only needed for client-side rendering but not for SSR.\n *\n * Before you use this hook, make sure to read https://gist.github.com/gaearon/e7d97cdf38a2907924ea12e4ebdf3c85\n * and confirm it doesn't apply to your use-case.\n */\nconst useEnhancedEffect = typeof window !== 'undefined' ? react__WEBPACK_IMPORTED_MODULE_0__.useLayoutEffect : react__WEBPACK_IMPORTED_MODULE_0__.useEffect;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (useEnhancedEffect);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useEventCallback/useEventCallback.js":
/*!**************************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useEventCallback/useEventCallback.js ***!
  \**************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _useEnhancedEffect_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../useEnhancedEffect/index.js */ \"./node_modules/@mui/utils/esm/useEnhancedEffect/useEnhancedEffect.js\");\n'use client';\n\n\n\n\n/**\n * Inspired by https://github.com/facebook/react/issues/14099#issuecomment-440013892\n * See RFC in https://github.com/reactjs/rfcs/pull/220\n */\n\nfunction useEventCallback(fn) {\n  const ref = react__WEBPACK_IMPORTED_MODULE_0__.useRef(fn);\n  (0,_useEnhancedEffect_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(() => {\n    ref.current = fn;\n  });\n  return react__WEBPACK_IMPORTED_MODULE_0__.useRef((...args) =>\n  // @ts-expect-error hide `this`\n  (0, ref.current)(...args)).current;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (useEventCallback);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useEventCallback/useEventCallback.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useForkRef/useForkRef.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useForkRef/useForkRef.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useForkRef)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\n\n/**\n * Merges refs into a single memoized callback ref or `null`.\n *\n * ```tsx\n * const rootRef = React.useRef<Instance>(null);\n * const refFork = useForkRef(rootRef, props.ref);\n *\n * return (\n *   <Root {...props} ref={refFork} />\n * );\n * ```\n *\n * @param {Array<React.Ref<Instance> | undefined>} refs The ref array.\n * @returns {React.RefCallback<Instance> | null} The new ref callback.\n */\nfunction useForkRef(...refs) {\n  const cleanupRef = react__WEBPACK_IMPORTED_MODULE_0__.useRef(undefined);\n  const refEffect = react__WEBPACK_IMPORTED_MODULE_0__.useCallback(instance => {\n    const cleanups = refs.map(ref => {\n      if (ref == null) {\n        return null;\n      }\n      if (typeof ref === 'function') {\n        const refCallback = ref;\n        const refCleanup = refCallback(instance);\n        return typeof refCleanup === 'function' ? refCleanup : () => {\n          refCallback(null);\n        };\n      }\n      ref.current = instance;\n      return () => {\n        ref.current = null;\n      };\n    });\n    return () => {\n      cleanups.forEach(refCleanup => refCleanup?.());\n    };\n    // eslint-disable-next-line react-hooks/exhaustive-deps\n  }, refs);\n  return react__WEBPACK_IMPORTED_MODULE_0__.useMemo(() => {\n    if (refs.every(ref => ref == null)) {\n      return null;\n    }\n    return value => {\n      if (cleanupRef.current) {\n        cleanupRef.current();\n        cleanupRef.current = undefined;\n      }\n      if (value != null) {\n        cleanupRef.current = refEffect(value);\n      }\n    };\n    // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler -- intentionally ignoring that the dependency array must be an array literal\n    // eslint-disable-next-line react-hooks/exhaustive-deps\n  }, refs);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useForkRef/useForkRef.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useId/useId.js":
/*!****************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useId/useId.js ***!
  \****************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useId)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\nlet globalId = 0;\n\n// TODO React 17: Remove `useGlobalId` once React 17 support is removed\nfunction useGlobalId(idOverride) {\n  const [defaultId, setDefaultId] = react__WEBPACK_IMPORTED_MODULE_0__.useState(idOverride);\n  const id = idOverride || defaultId;\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(() => {\n    if (defaultId == null) {\n      // Fallback to this default id when possible.\n      // Use the incrementing value for client-side rendering only.\n      // We can't use it server-side.\n      // If you want to use random values please consider the Birthday Problem: https://en.wikipedia.org/wiki/Birthday_problem\n      globalId += 1;\n      setDefaultId(`mui-${globalId}`);\n    }\n  }, [defaultId]);\n  return id;\n}\n\n// See https://github.com/mui/material-ui/issues/41190#issuecomment-2040873379 for why\nconst safeReact = {\n  ...react__WEBPACK_IMPORTED_MODULE_0__\n};\nconst maybeReactUseId = safeReact.useId;\n\n/**\n *\n * @example <div id={useId()} />\n * @param idOverride\n * @returns {string}\n */\nfunction useId(idOverride) {\n  // React.useId() is only available from React 17.0.0.\n  if (maybeReactUseId !== undefined) {\n    const reactId = maybeReactUseId();\n    return idOverride ?? reactId;\n  }\n\n  // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler\n  // eslint-disable-next-line react-hooks/rules-of-hooks -- `React.useId` is invariant at runtime.\n  return useGlobalId(idOverride);\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useId/useId.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useLazyRef/useLazyRef.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useLazyRef/useLazyRef.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useLazyRef)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\nconst UNINITIALIZED = {};\n\n/**\n * A React.useRef() that is initialized lazily with a function. Note that it accepts an optional\n * initialization argument, so the initialization function doesn't need to be an inline closure.\n *\n * @usage\n *   const ref = useLazyRef(sortColumns, columns)\n */\nfunction useLazyRef(init, initArg) {\n  const ref = react__WEBPACK_IMPORTED_MODULE_0__.useRef(UNINITIALIZED);\n  if (ref.current === UNINITIALIZED) {\n    ref.current = init(initArg);\n  }\n  return ref;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useLazyRef/useLazyRef.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useOnMount/useOnMount.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useOnMount/useOnMount.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (/* binding */ useOnMount)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n'use client';\n\n\nconst EMPTY = [];\n\n/**\n * A React.useEffect equivalent that runs once, when the component is mounted.\n */\nfunction useOnMount(fn) {\n  // TODO: uncomment once we enable eslint-plugin-react-compiler // eslint-disable-next-line react-compiler/react-compiler -- no need to put `fn` in the dependency array\n  /* eslint-disable react-hooks/exhaustive-deps */\n  react__WEBPACK_IMPORTED_MODULE_0__.useEffect(fn, EMPTY);\n  /* eslint-enable react-hooks/exhaustive-deps */\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useOnMount/useOnMount.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useSlotProps/useSlotProps.js":
/*!******************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useSlotProps/useSlotProps.js ***!
  \******************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _useForkRef_index_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ../useForkRef/index.js */ \"./node_modules/@mui/utils/esm/useForkRef/useForkRef.js\");\n/* harmony import */ var _appendOwnerState_index_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ../appendOwnerState/index.js */ \"./node_modules/@mui/utils/esm/appendOwnerState/appendOwnerState.js\");\n/* harmony import */ var _mergeSlotProps_index_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../mergeSlotProps/index.js */ \"./node_modules/@mui/utils/esm/mergeSlotProps/mergeSlotProps.js\");\n/* harmony import */ var _resolveComponentProps_index_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../resolveComponentProps/index.js */ \"./node_modules/@mui/utils/esm/resolveComponentProps/resolveComponentProps.js\");\n'use client';\n\n\n\n\n\n/**\n * @ignore - do not document.\n * Builds the props to be passed into the slot of an unstyled component.\n * It merges the internal props of the component with the ones supplied by the user, allowing to customize the behavior.\n * If the slot component is not a host component, it also merges in the `ownerState`.\n *\n * @param parameters.getSlotProps - A function that returns the props to be passed to the slot component.\n */\nfunction useSlotProps(parameters) {\n  const {\n    elementType,\n    externalSlotProps,\n    ownerState,\n    skipResolvingSlotProps = false,\n    ...other\n  } = parameters;\n  const resolvedComponentsProps = skipResolvingSlotProps ? {} : (0,_resolveComponentProps_index_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(externalSlotProps, ownerState);\n  const {\n    props: mergedProps,\n    internalRef\n  } = (0,_mergeSlotProps_index_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({\n    ...other,\n    externalSlotProps: resolvedComponentsProps\n  });\n  const ref = (0,_useForkRef_index_js__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(internalRef, resolvedComponentsProps?.ref, parameters.additionalProps?.ref);\n  const props = (0,_appendOwnerState_index_js__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(elementType, {\n    ...mergedProps,\n    ref\n  }, ownerState);\n  return props;\n}\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (useSlotProps);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useSlotProps/useSlotProps.js?");

/***/ }),

/***/ "./node_modules/@mui/utils/esm/useTimeout/useTimeout.js":
/*!**************************************************************!*\
  !*** ./node_modules/@mui/utils/esm/useTimeout/useTimeout.js ***!
  \**************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   Timeout: () => (/* binding */ Timeout),\n/* harmony export */   \"default\": () => (/* binding */ useTimeout)\n/* harmony export */ });\n/* harmony import */ var _useLazyRef_useLazyRef_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../useLazyRef/useLazyRef.js */ \"./node_modules/@mui/utils/esm/useLazyRef/useLazyRef.js\");\n/* harmony import */ var _useOnMount_useOnMount_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../useOnMount/useOnMount.js */ \"./node_modules/@mui/utils/esm/useOnMount/useOnMount.js\");\n'use client';\n\n\n\nclass Timeout {\n  static create() {\n    return new Timeout();\n  }\n  currentId = null;\n\n  /**\n   * Executes `fn` after `delay`, clearing any previously scheduled call.\n   */\n  start(delay, fn) {\n    this.clear();\n    this.currentId = setTimeout(() => {\n      this.currentId = null;\n      fn();\n    }, delay);\n  }\n  clear = () => {\n    if (this.currentId !== null) {\n      clearTimeout(this.currentId);\n      this.currentId = null;\n    }\n  };\n  disposeEffect = () => {\n    return this.clear;\n  };\n}\nfunction useTimeout() {\n  const timeout = (0,_useLazyRef_useLazyRef_js__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(Timeout.create).current;\n  (0,_useOnMount_useOnMount_js__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(timeout.disposeEffect);\n  return timeout;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/@mui/utils/esm/useTimeout/useTimeout.js?");

/***/ }),

/***/ "./node_modules/clsx/dist/clsx.mjs":
/*!*****************************************!*\
  !*** ./node_modules/clsx/dist/clsx.mjs ***!
  \*****************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   clsx: () => (/* binding */ clsx),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\nfunction r(e){var t,f,n=\"\";if(\"string\"==typeof e||\"number\"==typeof e)n+=e;else if(\"object\"==typeof e)if(Array.isArray(e)){var o=e.length;for(t=0;t<o;t++)e[t]&&(f=r(e[t]))&&(n&&(n+=\" \"),n+=f)}else for(f in e)e[f]&&(n&&(n+=\" \"),n+=f);return n}function clsx(){for(var e,t,f=0,n=\"\",o=arguments.length;f<o;f++)(e=arguments[f])&&(t=r(e))&&(n&&(n+=\" \"),n+=t);return n}/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (clsx);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/clsx/dist/clsx.mjs?");

/***/ }),

/***/ "./node_modules/css-loader/dist/runtime/api.js":
/*!*****************************************************!*\
  !*** ./node_modules/css-loader/dist/runtime/api.js ***!
  \*****************************************************/
/***/ ((module) => {

eval("\n\n/*\n  MIT License http://www.opensource.org/licenses/mit-license.php\n  Author Tobias Koppers @sokra\n*/\nmodule.exports = function (cssWithMappingToString) {\n  var list = [];\n\n  // return the list of modules as css string\n  list.toString = function toString() {\n    return this.map(function (item) {\n      var content = \"\";\n      var needLayer = typeof item[5] !== \"undefined\";\n      if (item[4]) {\n        content += \"@supports (\".concat(item[4], \") {\");\n      }\n      if (item[2]) {\n        content += \"@media \".concat(item[2], \" {\");\n      }\n      if (needLayer) {\n        content += \"@layer\".concat(item[5].length > 0 ? \" \".concat(item[5]) : \"\", \" {\");\n      }\n      content += cssWithMappingToString(item);\n      if (needLayer) {\n        content += \"}\";\n      }\n      if (item[2]) {\n        content += \"}\";\n      }\n      if (item[4]) {\n        content += \"}\";\n      }\n      return content;\n    }).join(\"\");\n  };\n\n  // import a list of modules into the list\n  list.i = function i(modules, media, dedupe, supports, layer) {\n    if (typeof modules === \"string\") {\n      modules = [[null, modules, undefined]];\n    }\n    var alreadyImportedModules = {};\n    if (dedupe) {\n      for (var k = 0; k < this.length; k++) {\n        var id = this[k][0];\n        if (id != null) {\n          alreadyImportedModules[id] = true;\n        }\n      }\n    }\n    for (var _k = 0; _k < modules.length; _k++) {\n      var item = [].concat(modules[_k]);\n      if (dedupe && alreadyImportedModules[item[0]]) {\n        continue;\n      }\n      if (typeof layer !== \"undefined\") {\n        if (typeof item[5] === \"undefined\") {\n          item[5] = layer;\n        } else {\n          item[1] = \"@layer\".concat(item[5].length > 0 ? \" \".concat(item[5]) : \"\", \" {\").concat(item[1], \"}\");\n          item[5] = layer;\n        }\n      }\n      if (media) {\n        if (!item[2]) {\n          item[2] = media;\n        } else {\n          item[1] = \"@media \".concat(item[2], \" {\").concat(item[1], \"}\");\n          item[2] = media;\n        }\n      }\n      if (supports) {\n        if (!item[4]) {\n          item[4] = \"\".concat(supports);\n        } else {\n          item[1] = \"@supports (\".concat(item[4], \") {\").concat(item[1], \"}\");\n          item[4] = supports;\n        }\n      }\n      list.push(item);\n    }\n  };\n  return list;\n};\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/css-loader/dist/runtime/api.js?");

/***/ }),

/***/ "./node_modules/css-loader/dist/runtime/getUrl.js":
/*!********************************************************!*\
  !*** ./node_modules/css-loader/dist/runtime/getUrl.js ***!
  \********************************************************/
/***/ ((module) => {

eval("\n\nmodule.exports = function (url, options) {\n  if (!options) {\n    options = {};\n  }\n  if (!url) {\n    return url;\n  }\n  url = String(url.__esModule ? url.default : url);\n\n  // If url is already wrapped in quotes, remove them\n  if (/^['\"].*['\"]$/.test(url)) {\n    url = url.slice(1, -1);\n  }\n  if (options.hash) {\n    url += options.hash;\n  }\n\n  // Should url be wrapped?\n  // See https://drafts.csswg.org/css-values-3/#urls\n  if (/[\"'() \\t\\n]|(%20)/.test(url) || options.needQuotes) {\n    return \"\\\"\".concat(url.replace(/\"/g, '\\\\\"').replace(/\\n/g, \"\\\\n\"), \"\\\"\");\n  }\n  return url;\n};\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/css-loader/dist/runtime/getUrl.js?");

/***/ }),

/***/ "./node_modules/css-loader/dist/runtime/noSourceMaps.js":
/*!**************************************************************!*\
  !*** ./node_modules/css-loader/dist/runtime/noSourceMaps.js ***!
  \**************************************************************/
/***/ ((module) => {

eval("\n\nmodule.exports = function (i) {\n  return i[1];\n};\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/css-loader/dist/runtime/noSourceMaps.js?");

/***/ }),

/***/ "./node_modules/fflate/esm/browser.js":
/*!********************************************!*\
  !*** ./node_modules/fflate/esm/browser.js ***!
  \********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   AsyncCompress: () => (/* binding */ AsyncGzip),\n/* harmony export */   AsyncDecompress: () => (/* binding */ AsyncDecompress),\n/* harmony export */   AsyncDeflate: () => (/* binding */ AsyncDeflate),\n/* harmony export */   AsyncGunzip: () => (/* binding */ AsyncGunzip),\n/* harmony export */   AsyncGzip: () => (/* binding */ AsyncGzip),\n/* harmony export */   AsyncInflate: () => (/* binding */ AsyncInflate),\n/* harmony export */   AsyncUnzipInflate: () => (/* binding */ AsyncUnzipInflate),\n/* harmony export */   AsyncUnzlib: () => (/* binding */ AsyncUnzlib),\n/* harmony export */   AsyncZipDeflate: () => (/* binding */ AsyncZipDeflate),\n/* harmony export */   AsyncZlib: () => (/* binding */ AsyncZlib),\n/* harmony export */   Compress: () => (/* binding */ Gzip),\n/* harmony export */   DecodeUTF8: () => (/* binding */ DecodeUTF8),\n/* harmony export */   Decompress: () => (/* binding */ Decompress),\n/* harmony export */   Deflate: () => (/* binding */ Deflate),\n/* harmony export */   EncodeUTF8: () => (/* binding */ EncodeUTF8),\n/* harmony export */   FlateErrorCode: () => (/* binding */ FlateErrorCode),\n/* harmony export */   Gunzip: () => (/* binding */ Gunzip),\n/* harmony export */   Gzip: () => (/* binding */ Gzip),\n/* harmony export */   Inflate: () => (/* binding */ Inflate),\n/* harmony export */   Unzip: () => (/* binding */ Unzip),\n/* harmony export */   UnzipInflate: () => (/* binding */ UnzipInflate),\n/* harmony export */   UnzipPassThrough: () => (/* binding */ UnzipPassThrough),\n/* harmony export */   Unzlib: () => (/* binding */ Unzlib),\n/* harmony export */   Zip: () => (/* binding */ Zip),\n/* harmony export */   ZipDeflate: () => (/* binding */ ZipDeflate),\n/* harmony export */   ZipPassThrough: () => (/* binding */ ZipPassThrough),\n/* harmony export */   Zlib: () => (/* binding */ Zlib),\n/* harmony export */   compress: () => (/* binding */ gzip),\n/* harmony export */   compressSync: () => (/* binding */ gzipSync),\n/* harmony export */   decompress: () => (/* binding */ decompress),\n/* harmony export */   decompressSync: () => (/* binding */ decompressSync),\n/* harmony export */   deflate: () => (/* binding */ deflate),\n/* harmony export */   deflateSync: () => (/* binding */ deflateSync),\n/* harmony export */   gunzip: () => (/* binding */ gunzip),\n/* harmony export */   gunzipSync: () => (/* binding */ gunzipSync),\n/* harmony export */   gzip: () => (/* binding */ gzip),\n/* harmony export */   gzipSync: () => (/* binding */ gzipSync),\n/* harmony export */   inflate: () => (/* binding */ inflate),\n/* harmony export */   inflateSync: () => (/* binding */ inflateSync),\n/* harmony export */   strFromU8: () => (/* binding */ strFromU8),\n/* harmony export */   strToU8: () => (/* binding */ strToU8),\n/* harmony export */   unzip: () => (/* binding */ unzip),\n/* harmony export */   unzipSync: () => (/* binding */ unzipSync),\n/* harmony export */   unzlib: () => (/* binding */ unzlib),\n/* harmony export */   unzlibSync: () => (/* binding */ unzlibSync),\n/* harmony export */   zip: () => (/* binding */ zip),\n/* harmony export */   zipSync: () => (/* binding */ zipSync),\n/* harmony export */   zlib: () => (/* binding */ zlib),\n/* harmony export */   zlibSync: () => (/* binding */ zlibSync)\n/* harmony export */ });\n// DEFLATE is a complex format; to read this code, you should probably check the RFC first:\n// https://tools.ietf.org/html/rfc1951\n// You may also wish to take a look at the guide I made about this program:\n// https://gist.github.com/101arrowz/253f31eb5abc3d9275ab943003ffecad\n// Some of the following code is similar to that of UZIP.js:\n// https://github.com/photopea/UZIP.js\n// However, the vast majority of the codebase has diverged from UZIP.js to increase performance and reduce bundle size.\n// Sometimes 0 will appear where -1 would be more appropriate. This is because using a uint\n// is better for memory in most engines (I *think*).\nvar ch2 = {};\nvar wk = (function (c, id, msg, transfer, cb) {\n    var w = new Worker(ch2[id] || (ch2[id] = URL.createObjectURL(new Blob([\n        c + ';addEventListener(\"error\",function(e){e=e.error;postMessage({$e$:[e.message,e.code,e.stack]})})'\n    ], { type: 'text/javascript' }))));\n    w.onmessage = function (e) {\n        var d = e.data, ed = d.$e$;\n        if (ed) {\n            var err = new Error(ed[0]);\n            err['code'] = ed[1];\n            err.stack = ed[2];\n            cb(err, null);\n        }\n        else\n            cb(null, d);\n    };\n    w.postMessage(msg, transfer);\n    return w;\n});\n\n// aliases for shorter compressed code (most minifers don't do this)\nvar u8 = Uint8Array, u16 = Uint16Array, i32 = Int32Array;\n// fixed length extra bits\nvar fleb = new u8([0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, /* unused */ 0, 0, /* impossible */ 0]);\n// fixed distance extra bits\nvar fdeb = new u8([0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13, /* unused */ 0, 0]);\n// code length index map\nvar clim = new u8([16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15]);\n// get base, reverse index map from extra bits\nvar freb = function (eb, start) {\n    var b = new u16(31);\n    for (var i = 0; i < 31; ++i) {\n        b[i] = start += 1 << eb[i - 1];\n    }\n    // numbers here are at max 18 bits\n    var r = new i32(b[30]);\n    for (var i = 1; i < 30; ++i) {\n        for (var j = b[i]; j < b[i + 1]; ++j) {\n            r[j] = ((j - b[i]) << 5) | i;\n        }\n    }\n    return { b: b, r: r };\n};\nvar _a = freb(fleb, 2), fl = _a.b, revfl = _a.r;\n// we can ignore the fact that the other numbers are wrong; they never happen anyway\nfl[28] = 258, revfl[258] = 28;\nvar _b = freb(fdeb, 0), fd = _b.b, revfd = _b.r;\n// map of value to reverse (assuming 16 bits)\nvar rev = new u16(32768);\nfor (var i = 0; i < 32768; ++i) {\n    // reverse table algorithm from SO\n    var x = ((i & 0xAAAA) >> 1) | ((i & 0x5555) << 1);\n    x = ((x & 0xCCCC) >> 2) | ((x & 0x3333) << 2);\n    x = ((x & 0xF0F0) >> 4) | ((x & 0x0F0F) << 4);\n    rev[i] = (((x & 0xFF00) >> 8) | ((x & 0x00FF) << 8)) >> 1;\n}\n// create huffman tree from u8 \"map\": index -> code length for code index\n// mb (max bits) must be at most 15\n// TODO: optimize/split up?\nvar hMap = (function (cd, mb, r) {\n    var s = cd.length;\n    // index\n    var i = 0;\n    // u16 \"map\": index -> # of codes with bit length = index\n    var l = new u16(mb);\n    // length of cd must be 288 (total # of codes)\n    for (; i < s; ++i) {\n        if (cd[i])\n            ++l[cd[i] - 1];\n    }\n    // u16 \"map\": index -> minimum code for bit length = index\n    var le = new u16(mb);\n    for (i = 1; i < mb; ++i) {\n        le[i] = (le[i - 1] + l[i - 1]) << 1;\n    }\n    var co;\n    if (r) {\n        // u16 \"map\": index -> number of actual bits, symbol for code\n        co = new u16(1 << mb);\n        // bits to remove for reverser\n        var rvb = 15 - mb;\n        for (i = 0; i < s; ++i) {\n            // ignore 0 lengths\n            if (cd[i]) {\n                // num encoding both symbol and bits read\n                var sv = (i << 4) | cd[i];\n                // free bits\n                var r_1 = mb - cd[i];\n                // start value\n                var v = le[cd[i] - 1]++ << r_1;\n                // m is end value\n                for (var m = v | ((1 << r_1) - 1); v <= m; ++v) {\n                    // every 16 bit value starting with the code yields the same result\n                    co[rev[v] >> rvb] = sv;\n                }\n            }\n        }\n    }\n    else {\n        co = new u16(s);\n        for (i = 0; i < s; ++i) {\n            if (cd[i]) {\n                co[i] = rev[le[cd[i] - 1]++] >> (15 - cd[i]);\n            }\n        }\n    }\n    return co;\n});\n// fixed length tree\nvar flt = new u8(288);\nfor (var i = 0; i < 144; ++i)\n    flt[i] = 8;\nfor (var i = 144; i < 256; ++i)\n    flt[i] = 9;\nfor (var i = 256; i < 280; ++i)\n    flt[i] = 7;\nfor (var i = 280; i < 288; ++i)\n    flt[i] = 8;\n// fixed distance tree\nvar fdt = new u8(32);\nfor (var i = 0; i < 32; ++i)\n    fdt[i] = 5;\n// fixed length map\nvar flm = /*#__PURE__*/ hMap(flt, 9, 0), flrm = /*#__PURE__*/ hMap(flt, 9, 1);\n// fixed distance map\nvar fdm = /*#__PURE__*/ hMap(fdt, 5, 0), fdrm = /*#__PURE__*/ hMap(fdt, 5, 1);\n// find max of array\nvar max = function (a) {\n    var m = a[0];\n    for (var i = 1; i < a.length; ++i) {\n        if (a[i] > m)\n            m = a[i];\n    }\n    return m;\n};\n// read d, starting at bit p and mask with m\nvar bits = function (d, p, m) {\n    var o = (p / 8) | 0;\n    return ((d[o] | (d[o + 1] << 8)) >> (p & 7)) & m;\n};\n// read d, starting at bit p continuing for at least 16 bits\nvar bits16 = function (d, p) {\n    var o = (p / 8) | 0;\n    return ((d[o] | (d[o + 1] << 8) | (d[o + 2] << 16)) >> (p & 7));\n};\n// get end of byte\nvar shft = function (p) { return ((p + 7) / 8) | 0; };\n// typed array slice - allows garbage collector to free original reference,\n// while being more compatible than .slice\nvar slc = function (v, s, e) {\n    if (s == null || s < 0)\n        s = 0;\n    if (e == null || e > v.length)\n        e = v.length;\n    // can't use .constructor in case user-supplied\n    return new u8(v.subarray(s, e));\n};\n/**\n * Codes for errors generated within this library\n */\nvar FlateErrorCode = {\n    UnexpectedEOF: 0,\n    InvalidBlockType: 1,\n    InvalidLengthLiteral: 2,\n    InvalidDistance: 3,\n    StreamFinished: 4,\n    NoStreamHandler: 5,\n    InvalidHeader: 6,\n    NoCallback: 7,\n    InvalidUTF8: 8,\n    ExtraFieldTooLong: 9,\n    InvalidDate: 10,\n    FilenameTooLong: 11,\n    StreamFinishing: 12,\n    InvalidZipData: 13,\n    UnknownCompressionMethod: 14\n};\n// error codes\nvar ec = [\n    'unexpected EOF',\n    'invalid block type',\n    'invalid length/literal',\n    'invalid distance',\n    'stream finished',\n    'no stream handler',\n    ,\n    'no callback',\n    'invalid UTF-8 data',\n    'extra field too long',\n    'date not in range 1980-2099',\n    'filename too long',\n    'stream finishing',\n    'invalid zip data'\n    // determined by unknown compression method\n];\n;\nvar err = function (ind, msg, nt) {\n    var e = new Error(msg || ec[ind]);\n    e.code = ind;\n    if (Error.captureStackTrace)\n        Error.captureStackTrace(e, err);\n    if (!nt)\n        throw e;\n    return e;\n};\n// expands raw DEFLATE data\nvar inflt = function (dat, st, buf, dict) {\n    // source length       dict length\n    var sl = dat.length, dl = dict ? dict.length : 0;\n    if (!sl || st.f && !st.l)\n        return buf || new u8(0);\n    var noBuf = !buf;\n    // have to estimate size\n    var resize = noBuf || st.i != 2;\n    // no state\n    var noSt = st.i;\n    // Assumes roughly 33% compression ratio average\n    if (noBuf)\n        buf = new u8(sl * 3);\n    // ensure buffer can fit at least l elements\n    var cbuf = function (l) {\n        var bl = buf.length;\n        // need to increase size to fit\n        if (l > bl) {\n            // Double or set to necessary, whichever is greater\n            var nbuf = new u8(Math.max(bl * 2, l));\n            nbuf.set(buf);\n            buf = nbuf;\n        }\n    };\n    //  last chunk         bitpos           bytes\n    var final = st.f || 0, pos = st.p || 0, bt = st.b || 0, lm = st.l, dm = st.d, lbt = st.m, dbt = st.n;\n    // total bits\n    var tbts = sl * 8;\n    do {\n        if (!lm) {\n            // BFINAL - this is only 1 when last chunk is next\n            final = bits(dat, pos, 1);\n            // type: 0 = no compression, 1 = fixed huffman, 2 = dynamic huffman\n            var type = bits(dat, pos + 1, 3);\n            pos += 3;\n            if (!type) {\n                // go to end of byte boundary\n                var s = shft(pos) + 4, l = dat[s - 4] | (dat[s - 3] << 8), t = s + l;\n                if (t > sl) {\n                    if (noSt)\n                        err(0);\n                    break;\n                }\n                // ensure size\n                if (resize)\n                    cbuf(bt + l);\n                // Copy over uncompressed data\n                buf.set(dat.subarray(s, t), bt);\n                // Get new bitpos, update byte count\n                st.b = bt += l, st.p = pos = t * 8, st.f = final;\n                continue;\n            }\n            else if (type == 1)\n                lm = flrm, dm = fdrm, lbt = 9, dbt = 5;\n            else if (type == 2) {\n                //  literal                            lengths\n                var hLit = bits(dat, pos, 31) + 257, hcLen = bits(dat, pos + 10, 15) + 4;\n                var tl = hLit + bits(dat, pos + 5, 31) + 1;\n                pos += 14;\n                // length+distance tree\n                var ldt = new u8(tl);\n                // code length tree\n                var clt = new u8(19);\n                for (var i = 0; i < hcLen; ++i) {\n                    // use index map to get real code\n                    clt[clim[i]] = bits(dat, pos + i * 3, 7);\n                }\n                pos += hcLen * 3;\n                // code lengths bits\n                var clb = max(clt), clbmsk = (1 << clb) - 1;\n                // code lengths map\n                var clm = hMap(clt, clb, 1);\n                for (var i = 0; i < tl;) {\n                    var r = clm[bits(dat, pos, clbmsk)];\n                    // bits read\n                    pos += r & 15;\n                    // symbol\n                    var s = r >> 4;\n                    // code length to copy\n                    if (s < 16) {\n                        ldt[i++] = s;\n                    }\n                    else {\n                        //  copy   count\n                        var c = 0, n = 0;\n                        if (s == 16)\n                            n = 3 + bits(dat, pos, 3), pos += 2, c = ldt[i - 1];\n                        else if (s == 17)\n                            n = 3 + bits(dat, pos, 7), pos += 3;\n                        else if (s == 18)\n                            n = 11 + bits(dat, pos, 127), pos += 7;\n                        while (n--)\n                            ldt[i++] = c;\n                    }\n                }\n                //    length tree                 distance tree\n                var lt = ldt.subarray(0, hLit), dt = ldt.subarray(hLit);\n                // max length bits\n                lbt = max(lt);\n                // max dist bits\n                dbt = max(dt);\n                lm = hMap(lt, lbt, 1);\n                dm = hMap(dt, dbt, 1);\n            }\n            else\n                err(1);\n            if (pos > tbts) {\n                if (noSt)\n                    err(0);\n                break;\n            }\n        }\n        // Make sure the buffer can hold this + the largest possible addition\n        // Maximum chunk size (practically, theoretically infinite) is 2^17\n        if (resize)\n            cbuf(bt + 131072);\n        var lms = (1 << lbt) - 1, dms = (1 << dbt) - 1;\n        var lpos = pos;\n        for (;; lpos = pos) {\n            // bits read, code\n            var c = lm[bits16(dat, pos) & lms], sym = c >> 4;\n            pos += c & 15;\n            if (pos > tbts) {\n                if (noSt)\n                    err(0);\n                break;\n            }\n            if (!c)\n                err(2);\n            if (sym < 256)\n                buf[bt++] = sym;\n            else if (sym == 256) {\n                lpos = pos, lm = null;\n                break;\n            }\n            else {\n                var add = sym - 254;\n                // no extra bits needed if less\n                if (sym > 264) {\n                    // index\n                    var i = sym - 257, b = fleb[i];\n                    add = bits(dat, pos, (1 << b) - 1) + fl[i];\n                    pos += b;\n                }\n                // dist\n                var d = dm[bits16(dat, pos) & dms], dsym = d >> 4;\n                if (!d)\n                    err(3);\n                pos += d & 15;\n                var dt = fd[dsym];\n                if (dsym > 3) {\n                    var b = fdeb[dsym];\n                    dt += bits16(dat, pos) & (1 << b) - 1, pos += b;\n                }\n                if (pos > tbts) {\n                    if (noSt)\n                        err(0);\n                    break;\n                }\n                if (resize)\n                    cbuf(bt + 131072);\n                var end = bt + add;\n                if (bt < dt) {\n                    var shift = dl - dt, dend = Math.min(dt, end);\n                    if (shift + bt < 0)\n                        err(3);\n                    for (; bt < dend; ++bt)\n                        buf[bt] = dict[shift + bt];\n                }\n                for (; bt < end; ++bt)\n                    buf[bt] = buf[bt - dt];\n            }\n        }\n        st.l = lm, st.p = lpos, st.b = bt, st.f = final;\n        if (lm)\n            final = 1, st.m = lbt, st.d = dm, st.n = dbt;\n    } while (!final);\n    // don't reallocate for streams or user buffers\n    return bt != buf.length && noBuf ? slc(buf, 0, bt) : buf.subarray(0, bt);\n};\n// starting at p, write the minimum number of bits that can hold v to d\nvar wbits = function (d, p, v) {\n    v <<= p & 7;\n    var o = (p / 8) | 0;\n    d[o] |= v;\n    d[o + 1] |= v >> 8;\n};\n// starting at p, write the minimum number of bits (>8) that can hold v to d\nvar wbits16 = function (d, p, v) {\n    v <<= p & 7;\n    var o = (p / 8) | 0;\n    d[o] |= v;\n    d[o + 1] |= v >> 8;\n    d[o + 2] |= v >> 16;\n};\n// creates code lengths from a frequency table\nvar hTree = function (d, mb) {\n    // Need extra info to make a tree\n    var t = [];\n    for (var i = 0; i < d.length; ++i) {\n        if (d[i])\n            t.push({ s: i, f: d[i] });\n    }\n    var s = t.length;\n    var t2 = t.slice();\n    if (!s)\n        return { t: et, l: 0 };\n    if (s == 1) {\n        var v = new u8(t[0].s + 1);\n        v[t[0].s] = 1;\n        return { t: v, l: 1 };\n    }\n    t.sort(function (a, b) { return a.f - b.f; });\n    // after i2 reaches last ind, will be stopped\n    // freq must be greater than largest possible number of symbols\n    t.push({ s: -1, f: 25001 });\n    var l = t[0], r = t[1], i0 = 0, i1 = 1, i2 = 2;\n    t[0] = { s: -1, f: l.f + r.f, l: l, r: r };\n    // efficient algorithm from UZIP.js\n    // i0 is lookbehind, i2 is lookahead - after processing two low-freq\n    // symbols that combined have high freq, will start processing i2 (high-freq,\n    // non-composite) symbols instead\n    // see https://reddit.com/r/photopea/comments/ikekht/uzipjs_questions/\n    while (i1 != s - 1) {\n        l = t[t[i0].f < t[i2].f ? i0++ : i2++];\n        r = t[i0 != i1 && t[i0].f < t[i2].f ? i0++ : i2++];\n        t[i1++] = { s: -1, f: l.f + r.f, l: l, r: r };\n    }\n    var maxSym = t2[0].s;\n    for (var i = 1; i < s; ++i) {\n        if (t2[i].s > maxSym)\n            maxSym = t2[i].s;\n    }\n    // code lengths\n    var tr = new u16(maxSym + 1);\n    // max bits in tree\n    var mbt = ln(t[i1 - 1], tr, 0);\n    if (mbt > mb) {\n        // more algorithms from UZIP.js\n        // TODO: find out how this code works (debt)\n        //  ind    debt\n        var i = 0, dt = 0;\n        //    left            cost\n        var lft = mbt - mb, cst = 1 << lft;\n        t2.sort(function (a, b) { return tr[b.s] - tr[a.s] || a.f - b.f; });\n        for (; i < s; ++i) {\n            var i2_1 = t2[i].s;\n            if (tr[i2_1] > mb) {\n                dt += cst - (1 << (mbt - tr[i2_1]));\n                tr[i2_1] = mb;\n            }\n            else\n                break;\n        }\n        dt >>= lft;\n        while (dt > 0) {\n            var i2_2 = t2[i].s;\n            if (tr[i2_2] < mb)\n                dt -= 1 << (mb - tr[i2_2]++ - 1);\n            else\n                ++i;\n        }\n        for (; i >= 0 && dt; --i) {\n            var i2_3 = t2[i].s;\n            if (tr[i2_3] == mb) {\n                --tr[i2_3];\n                ++dt;\n            }\n        }\n        mbt = mb;\n    }\n    return { t: new u8(tr), l: mbt };\n};\n// get the max length and assign length codes\nvar ln = function (n, l, d) {\n    return n.s == -1\n        ? Math.max(ln(n.l, l, d + 1), ln(n.r, l, d + 1))\n        : (l[n.s] = d);\n};\n// length codes generation\nvar lc = function (c) {\n    var s = c.length;\n    // Note that the semicolon was intentional\n    while (s && !c[--s])\n        ;\n    var cl = new u16(++s);\n    //  ind      num         streak\n    var cli = 0, cln = c[0], cls = 1;\n    var w = function (v) { cl[cli++] = v; };\n    for (var i = 1; i <= s; ++i) {\n        if (c[i] == cln && i != s)\n            ++cls;\n        else {\n            if (!cln && cls > 2) {\n                for (; cls > 138; cls -= 138)\n                    w(32754);\n                if (cls > 2) {\n                    w(cls > 10 ? ((cls - 11) << 5) | 28690 : ((cls - 3) << 5) | 12305);\n                    cls = 0;\n                }\n            }\n            else if (cls > 3) {\n                w(cln), --cls;\n                for (; cls > 6; cls -= 6)\n                    w(8304);\n                if (cls > 2)\n                    w(((cls - 3) << 5) | 8208), cls = 0;\n            }\n            while (cls--)\n                w(cln);\n            cls = 1;\n            cln = c[i];\n        }\n    }\n    return { c: cl.subarray(0, cli), n: s };\n};\n// calculate the length of output from tree, code lengths\nvar clen = function (cf, cl) {\n    var l = 0;\n    for (var i = 0; i < cl.length; ++i)\n        l += cf[i] * cl[i];\n    return l;\n};\n// writes a fixed block\n// returns the new bit pos\nvar wfblk = function (out, pos, dat) {\n    // no need to write 00 as type: TypedArray defaults to 0\n    var s = dat.length;\n    var o = shft(pos + 2);\n    out[o] = s & 255;\n    out[o + 1] = s >> 8;\n    out[o + 2] = out[o] ^ 255;\n    out[o + 3] = out[o + 1] ^ 255;\n    for (var i = 0; i < s; ++i)\n        out[o + i + 4] = dat[i];\n    return (o + 4 + s) * 8;\n};\n// writes a block\nvar wblk = function (dat, out, final, syms, lf, df, eb, li, bs, bl, p) {\n    wbits(out, p++, final);\n    ++lf[256];\n    var _a = hTree(lf, 15), dlt = _a.t, mlb = _a.l;\n    var _b = hTree(df, 15), ddt = _b.t, mdb = _b.l;\n    var _c = lc(dlt), lclt = _c.c, nlc = _c.n;\n    var _d = lc(ddt), lcdt = _d.c, ndc = _d.n;\n    var lcfreq = new u16(19);\n    for (var i = 0; i < lclt.length; ++i)\n        ++lcfreq[lclt[i] & 31];\n    for (var i = 0; i < lcdt.length; ++i)\n        ++lcfreq[lcdt[i] & 31];\n    var _e = hTree(lcfreq, 7), lct = _e.t, mlcb = _e.l;\n    var nlcc = 19;\n    for (; nlcc > 4 && !lct[clim[nlcc - 1]]; --nlcc)\n        ;\n    var flen = (bl + 5) << 3;\n    var ftlen = clen(lf, flt) + clen(df, fdt) + eb;\n    var dtlen = clen(lf, dlt) + clen(df, ddt) + eb + 14 + 3 * nlcc + clen(lcfreq, lct) + 2 * lcfreq[16] + 3 * lcfreq[17] + 7 * lcfreq[18];\n    if (bs >= 0 && flen <= ftlen && flen <= dtlen)\n        return wfblk(out, p, dat.subarray(bs, bs + bl));\n    var lm, ll, dm, dl;\n    wbits(out, p, 1 + (dtlen < ftlen)), p += 2;\n    if (dtlen < ftlen) {\n        lm = hMap(dlt, mlb, 0), ll = dlt, dm = hMap(ddt, mdb, 0), dl = ddt;\n        var llm = hMap(lct, mlcb, 0);\n        wbits(out, p, nlc - 257);\n        wbits(out, p + 5, ndc - 1);\n        wbits(out, p + 10, nlcc - 4);\n        p += 14;\n        for (var i = 0; i < nlcc; ++i)\n            wbits(out, p + 3 * i, lct[clim[i]]);\n        p += 3 * nlcc;\n        var lcts = [lclt, lcdt];\n        for (var it = 0; it < 2; ++it) {\n            var clct = lcts[it];\n            for (var i = 0; i < clct.length; ++i) {\n                var len = clct[i] & 31;\n                wbits(out, p, llm[len]), p += lct[len];\n                if (len > 15)\n                    wbits(out, p, (clct[i] >> 5) & 127), p += clct[i] >> 12;\n            }\n        }\n    }\n    else {\n        lm = flm, ll = flt, dm = fdm, dl = fdt;\n    }\n    for (var i = 0; i < li; ++i) {\n        var sym = syms[i];\n        if (sym > 255) {\n            var len = (sym >> 18) & 31;\n            wbits16(out, p, lm[len + 257]), p += ll[len + 257];\n            if (len > 7)\n                wbits(out, p, (sym >> 23) & 31), p += fleb[len];\n            var dst = sym & 31;\n            wbits16(out, p, dm[dst]), p += dl[dst];\n            if (dst > 3)\n                wbits16(out, p, (sym >> 5) & 8191), p += fdeb[dst];\n        }\n        else {\n            wbits16(out, p, lm[sym]), p += ll[sym];\n        }\n    }\n    wbits16(out, p, lm[256]);\n    return p + ll[256];\n};\n// deflate options (nice << 13) | chain\nvar deo = /*#__PURE__*/ new i32([65540, 131080, 131088, 131104, 262176, 1048704, 1048832, 2114560, 2117632]);\n// empty\nvar et = /*#__PURE__*/ new u8(0);\n// compresses data into a raw DEFLATE buffer\nvar dflt = function (dat, lvl, plvl, pre, post, st) {\n    var s = st.z || dat.length;\n    var o = new u8(pre + s + 5 * (1 + Math.ceil(s / 7000)) + post);\n    // writing to this writes to the output buffer\n    var w = o.subarray(pre, o.length - post);\n    var lst = st.l;\n    var pos = (st.r || 0) & 7;\n    if (lvl) {\n        if (pos)\n            w[0] = st.r >> 3;\n        var opt = deo[lvl - 1];\n        var n = opt >> 13, c = opt & 8191;\n        var msk_1 = (1 << plvl) - 1;\n        //    prev 2-byte val map    curr 2-byte val map\n        var prev = st.p || new u16(32768), head = st.h || new u16(msk_1 + 1);\n        var bs1_1 = Math.ceil(plvl / 3), bs2_1 = 2 * bs1_1;\n        var hsh = function (i) { return (dat[i] ^ (dat[i + 1] << bs1_1) ^ (dat[i + 2] << bs2_1)) & msk_1; };\n        // 24576 is an arbitrary number of maximum symbols per block\n        // 424 buffer for last block\n        var syms = new i32(25000);\n        // length/literal freq   distance freq\n        var lf = new u16(288), df = new u16(32);\n        //  l/lcnt  exbits  index          l/lind  waitdx          blkpos\n        var lc_1 = 0, eb = 0, i = st.i || 0, li = 0, wi = st.w || 0, bs = 0;\n        for (; i + 2 < s; ++i) {\n            // hash value\n            var hv = hsh(i);\n            // index mod 32768    previous index mod\n            var imod = i & 32767, pimod = head[hv];\n            prev[imod] = pimod;\n            head[hv] = imod;\n            // We always should modify head and prev, but only add symbols if\n            // this data is not yet processed (\"wait\" for wait index)\n            if (wi <= i) {\n                // bytes remaining\n                var rem = s - i;\n                if ((lc_1 > 7000 || li > 24576) && (rem > 423 || !lst)) {\n                    pos = wblk(dat, w, 0, syms, lf, df, eb, li, bs, i - bs, pos);\n                    li = lc_1 = eb = 0, bs = i;\n                    for (var j = 0; j < 286; ++j)\n                        lf[j] = 0;\n                    for (var j = 0; j < 30; ++j)\n                        df[j] = 0;\n                }\n                //  len    dist   chain\n                var l = 2, d = 0, ch_1 = c, dif = imod - pimod & 32767;\n                if (rem > 2 && hv == hsh(i - dif)) {\n                    var maxn = Math.min(n, rem) - 1;\n                    var maxd = Math.min(32767, i);\n                    // max possible length\n                    // not capped at dif because decompressors implement \"rolling\" index population\n                    var ml = Math.min(258, rem);\n                    while (dif <= maxd && --ch_1 && imod != pimod) {\n                        if (dat[i + l] == dat[i + l - dif]) {\n                            var nl = 0;\n                            for (; nl < ml && dat[i + nl] == dat[i + nl - dif]; ++nl)\n                                ;\n                            if (nl > l) {\n                                l = nl, d = dif;\n                                // break out early when we reach \"nice\" (we are satisfied enough)\n                                if (nl > maxn)\n                                    break;\n                                // now, find the rarest 2-byte sequence within this\n                                // length of literals and search for that instead.\n                                // Much faster than just using the start\n                                var mmd = Math.min(dif, nl - 2);\n                                var md = 0;\n                                for (var j = 0; j < mmd; ++j) {\n                                    var ti = i - dif + j & 32767;\n                                    var pti = prev[ti];\n                                    var cd = ti - pti & 32767;\n                                    if (cd > md)\n                                        md = cd, pimod = ti;\n                                }\n                            }\n                        }\n                        // check the previous match\n                        imod = pimod, pimod = prev[imod];\n                        dif += imod - pimod & 32767;\n                    }\n                }\n                // d will be nonzero only when a match was found\n                if (d) {\n                    // store both dist and len data in one int32\n                    // Make sure this is recognized as a len/dist with 28th bit (2^28)\n                    syms[li++] = 268435456 | (revfl[l] << 18) | revfd[d];\n                    var lin = revfl[l] & 31, din = revfd[d] & 31;\n                    eb += fleb[lin] + fdeb[din];\n                    ++lf[257 + lin];\n                    ++df[din];\n                    wi = i + l;\n                    ++lc_1;\n                }\n                else {\n                    syms[li++] = dat[i];\n                    ++lf[dat[i]];\n                }\n            }\n        }\n        for (i = Math.max(i, wi); i < s; ++i) {\n            syms[li++] = dat[i];\n            ++lf[dat[i]];\n        }\n        pos = wblk(dat, w, lst, syms, lf, df, eb, li, bs, i - bs, pos);\n        if (!lst) {\n            st.r = (pos & 7) | w[(pos / 8) | 0] << 3;\n            // shft(pos) now 1 less if pos & 7 != 0\n            pos -= 7;\n            st.h = head, st.p = prev, st.i = i, st.w = wi;\n        }\n    }\n    else {\n        for (var i = st.w || 0; i < s + lst; i += 65535) {\n            // end\n            var e = i + 65535;\n            if (e >= s) {\n                // write final block\n                w[(pos / 8) | 0] = lst;\n                e = s;\n            }\n            pos = wfblk(w, pos + 1, dat.subarray(i, e));\n        }\n        st.i = s;\n    }\n    return slc(o, 0, pre + shft(pos) + post);\n};\n// CRC32 table\nvar crct = /*#__PURE__*/ (function () {\n    var t = new Int32Array(256);\n    for (var i = 0; i < 256; ++i) {\n        var c = i, k = 9;\n        while (--k)\n            c = ((c & 1) && -306674912) ^ (c >>> 1);\n        t[i] = c;\n    }\n    return t;\n})();\n// CRC32\nvar crc = function () {\n    var c = -1;\n    return {\n        p: function (d) {\n            // closures have awful performance\n            var cr = c;\n            for (var i = 0; i < d.length; ++i)\n                cr = crct[(cr & 255) ^ d[i]] ^ (cr >>> 8);\n            c = cr;\n        },\n        d: function () { return ~c; }\n    };\n};\n// Adler32\nvar adler = function () {\n    var a = 1, b = 0;\n    return {\n        p: function (d) {\n            // closures have awful performance\n            var n = a, m = b;\n            var l = d.length | 0;\n            for (var i = 0; i != l;) {\n                var e = Math.min(i + 2655, l);\n                for (; i < e; ++i)\n                    m += n += d[i];\n                n = (n & 65535) + 15 * (n >> 16), m = (m & 65535) + 15 * (m >> 16);\n            }\n            a = n, b = m;\n        },\n        d: function () {\n            a %= 65521, b %= 65521;\n            return (a & 255) << 24 | (a & 0xFF00) << 8 | (b & 255) << 8 | (b >> 8);\n        }\n    };\n};\n;\n// deflate with opts\nvar dopt = function (dat, opt, pre, post, st) {\n    if (!st) {\n        st = { l: 1 };\n        if (opt.dictionary) {\n            var dict = opt.dictionary.subarray(-32768);\n            var newDat = new u8(dict.length + dat.length);\n            newDat.set(dict);\n            newDat.set(dat, dict.length);\n            dat = newDat;\n            st.w = dict.length;\n        }\n    }\n    return dflt(dat, opt.level == null ? 6 : opt.level, opt.mem == null ? (st.l ? Math.ceil(Math.max(8, Math.min(13, Math.log(dat.length))) * 1.5) : 20) : (12 + opt.mem), pre, post, st);\n};\n// Walmart object spread\nvar mrg = function (a, b) {\n    var o = {};\n    for (var k in a)\n        o[k] = a[k];\n    for (var k in b)\n        o[k] = b[k];\n    return o;\n};\n// worker clone\n// This is possibly the craziest part of the entire codebase, despite how simple it may seem.\n// The only parameter to this function is a closure that returns an array of variables outside of the function scope.\n// We're going to try to figure out the variable names used in the closure as strings because that is crucial for workerization.\n// We will return an object mapping of true variable name to value (basically, the current scope as a JS object).\n// The reason we can't just use the original variable names is minifiers mangling the toplevel scope.\n// This took me three weeks to figure out how to do.\nvar wcln = function (fn, fnStr, td) {\n    var dt = fn();\n    var st = fn.toString();\n    var ks = st.slice(st.indexOf('[') + 1, st.lastIndexOf(']')).replace(/\\s+/g, '').split(',');\n    for (var i = 0; i < dt.length; ++i) {\n        var v = dt[i], k = ks[i];\n        if (typeof v == 'function') {\n            fnStr += ';' + k + '=';\n            var st_1 = v.toString();\n            if (v.prototype) {\n                // for global objects\n                if (st_1.indexOf('[native code]') != -1) {\n                    var spInd = st_1.indexOf(' ', 8) + 1;\n                    fnStr += st_1.slice(spInd, st_1.indexOf('(', spInd));\n                }\n                else {\n                    fnStr += st_1;\n                    for (var t in v.prototype)\n                        fnStr += ';' + k + '.prototype.' + t + '=' + v.prototype[t].toString();\n                }\n            }\n            else\n                fnStr += st_1;\n        }\n        else\n            td[k] = v;\n    }\n    return fnStr;\n};\nvar ch = [];\n// clone bufs\nvar cbfs = function (v) {\n    var tl = [];\n    for (var k in v) {\n        if (v[k].buffer) {\n            tl.push((v[k] = new v[k].constructor(v[k])).buffer);\n        }\n    }\n    return tl;\n};\n// use a worker to execute code\nvar wrkr = function (fns, init, id, cb) {\n    if (!ch[id]) {\n        var fnStr = '', td_1 = {}, m = fns.length - 1;\n        for (var i = 0; i < m; ++i)\n            fnStr = wcln(fns[i], fnStr, td_1);\n        ch[id] = { c: wcln(fns[m], fnStr, td_1), e: td_1 };\n    }\n    var td = mrg({}, ch[id].e);\n    return wk(ch[id].c + ';onmessage=function(e){for(var k in e.data)self[k]=e.data[k];onmessage=' + init.toString() + '}', id, td, cbfs(td), cb);\n};\n// base async inflate fn\nvar bInflt = function () { return [u8, u16, i32, fleb, fdeb, clim, fl, fd, flrm, fdrm, rev, ec, hMap, max, bits, bits16, shft, slc, err, inflt, inflateSync, pbf, gopt]; };\nvar bDflt = function () { return [u8, u16, i32, fleb, fdeb, clim, revfl, revfd, flm, flt, fdm, fdt, rev, deo, et, hMap, wbits, wbits16, hTree, ln, lc, clen, wfblk, wblk, shft, slc, dflt, dopt, deflateSync, pbf]; };\n// gzip extra\nvar gze = function () { return [gzh, gzhl, wbytes, crc, crct]; };\n// gunzip extra\nvar guze = function () { return [gzs, gzl]; };\n// zlib extra\nvar zle = function () { return [zlh, wbytes, adler]; };\n// unzlib extra\nvar zule = function () { return [zls]; };\n// post buf\nvar pbf = function (msg) { return postMessage(msg, [msg.buffer]); };\n// get opts\nvar gopt = function (o) { return o && {\n    out: o.size && new u8(o.size),\n    dictionary: o.dictionary\n}; };\n// async helper\nvar cbify = function (dat, opts, fns, init, id, cb) {\n    var w = wrkr(fns, init, id, function (err, dat) {\n        w.terminate();\n        cb(err, dat);\n    });\n    w.postMessage([dat, opts], opts.consume ? [dat.buffer] : []);\n    return function () { w.terminate(); };\n};\n// auto stream\nvar astrm = function (strm) {\n    strm.ondata = function (dat, final) { return postMessage([dat, final], [dat.buffer]); };\n    return function (ev) {\n        if (ev.data.length) {\n            strm.push(ev.data[0], ev.data[1]);\n            postMessage([ev.data[0].length]);\n        }\n        else\n            strm.flush();\n    };\n};\n// async stream attach\nvar astrmify = function (fns, strm, opts, init, id, flush, ext) {\n    var t;\n    var w = wrkr(fns, init, id, function (err, dat) {\n        if (err)\n            w.terminate(), strm.ondata.call(strm, err);\n        else if (!Array.isArray(dat))\n            ext(dat);\n        else if (dat.length == 1) {\n            strm.queuedSize -= dat[0];\n            if (strm.ondrain)\n                strm.ondrain(dat[0]);\n        }\n        else {\n            if (dat[1])\n                w.terminate();\n            strm.ondata.call(strm, err, dat[0], dat[1]);\n        }\n    });\n    w.postMessage(opts);\n    strm.queuedSize = 0;\n    strm.push = function (d, f) {\n        if (!strm.ondata)\n            err(5);\n        if (t)\n            strm.ondata(err(4, 0, 1), null, !!f);\n        strm.queuedSize += d.length;\n        w.postMessage([d, t = f], [d.buffer]);\n    };\n    strm.terminate = function () { w.terminate(); };\n    if (flush) {\n        strm.flush = function () { w.postMessage([]); };\n    }\n};\n// read 2 bytes\nvar b2 = function (d, b) { return d[b] | (d[b + 1] << 8); };\n// read 4 bytes\nvar b4 = function (d, b) { return (d[b] | (d[b + 1] << 8) | (d[b + 2] << 16) | (d[b + 3] << 24)) >>> 0; };\nvar b8 = function (d, b) { return b4(d, b) + (b4(d, b + 4) * 4294967296); };\n// write bytes\nvar wbytes = function (d, b, v) {\n    for (; v; ++b)\n        d[b] = v, v >>>= 8;\n};\n// gzip header\nvar gzh = function (c, o) {\n    var fn = o.filename;\n    c[0] = 31, c[1] = 139, c[2] = 8, c[8] = o.level < 2 ? 4 : o.level == 9 ? 2 : 0, c[9] = 3; // assume Unix\n    if (o.mtime != 0)\n        wbytes(c, 4, Math.floor(new Date(o.mtime || Date.now()) / 1000));\n    if (fn) {\n        c[3] = 8;\n        for (var i = 0; i <= fn.length; ++i)\n            c[i + 10] = fn.charCodeAt(i);\n    }\n};\n// gzip footer: -8 to -4 = CRC, -4 to -0 is length\n// gzip start\nvar gzs = function (d) {\n    if (d[0] != 31 || d[1] != 139 || d[2] != 8)\n        err(6, 'invalid gzip data');\n    var flg = d[3];\n    var st = 10;\n    if (flg & 4)\n        st += (d[10] | d[11] << 8) + 2;\n    for (var zs = (flg >> 3 & 1) + (flg >> 4 & 1); zs > 0; zs -= !d[st++])\n        ;\n    return st + (flg & 2);\n};\n// gzip length\nvar gzl = function (d) {\n    var l = d.length;\n    return (d[l - 4] | d[l - 3] << 8 | d[l - 2] << 16 | d[l - 1] << 24) >>> 0;\n};\n// gzip header length\nvar gzhl = function (o) { return 10 + (o.filename ? o.filename.length + 1 : 0); };\n// zlib header\nvar zlh = function (c, o) {\n    var lv = o.level, fl = lv == 0 ? 0 : lv < 6 ? 1 : lv == 9 ? 3 : 2;\n    c[0] = 120, c[1] = (fl << 6) | (o.dictionary && 32);\n    c[1] |= 31 - ((c[0] << 8) | c[1]) % 31;\n    if (o.dictionary) {\n        var h = adler();\n        h.p(o.dictionary);\n        wbytes(c, 2, h.d());\n    }\n};\n// zlib start\nvar zls = function (d, dict) {\n    if ((d[0] & 15) != 8 || (d[0] >> 4) > 7 || ((d[0] << 8 | d[1]) % 31))\n        err(6, 'invalid zlib data');\n    if ((d[1] >> 5 & 1) == +!dict)\n        err(6, 'invalid zlib data: ' + (d[1] & 32 ? 'need' : 'unexpected') + ' dictionary');\n    return (d[1] >> 3 & 4) + 2;\n};\nfunction StrmOpt(opts, cb) {\n    if (typeof opts == 'function')\n        cb = opts, opts = {};\n    this.ondata = cb;\n    return opts;\n}\n/**\n * Streaming DEFLATE compression\n */\nvar Deflate = /*#__PURE__*/ (function () {\n    function Deflate(opts, cb) {\n        if (typeof opts == 'function')\n            cb = opts, opts = {};\n        this.ondata = cb;\n        this.o = opts || {};\n        this.s = { l: 0, i: 32768, w: 32768, z: 32768 };\n        // Buffer length must always be 0 mod 32768 for index calculations to be correct when modifying head and prev\n        // 98304 = 32768 (lookback) + 65536 (common chunk size)\n        this.b = new u8(98304);\n        if (this.o.dictionary) {\n            var dict = this.o.dictionary.subarray(-32768);\n            this.b.set(dict, 32768 - dict.length);\n            this.s.i = 32768 - dict.length;\n        }\n    }\n    Deflate.prototype.p = function (c, f) {\n        this.ondata(dopt(c, this.o, 0, 0, this.s), f);\n    };\n    /**\n     * Pushes a chunk to be deflated\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Deflate.prototype.push = function (chunk, final) {\n        if (!this.ondata)\n            err(5);\n        if (this.s.l)\n            err(4);\n        var endLen = chunk.length + this.s.z;\n        if (endLen > this.b.length) {\n            if (endLen > 2 * this.b.length - 32768) {\n                var newBuf = new u8(endLen & -32768);\n                newBuf.set(this.b.subarray(0, this.s.z));\n                this.b = newBuf;\n            }\n            var split = this.b.length - this.s.z;\n            this.b.set(chunk.subarray(0, split), this.s.z);\n            this.s.z = this.b.length;\n            this.p(this.b, false);\n            this.b.set(this.b.subarray(-32768));\n            this.b.set(chunk.subarray(split), 32768);\n            this.s.z = chunk.length - split + 32768;\n            this.s.i = 32766, this.s.w = 32768;\n        }\n        else {\n            this.b.set(chunk, this.s.z);\n            this.s.z += chunk.length;\n        }\n        this.s.l = final & 1;\n        if (this.s.z > this.s.w + 8191 || final) {\n            this.p(this.b, final || false);\n            this.s.w = this.s.i, this.s.i -= 2;\n        }\n    };\n    /**\n     * Flushes buffered uncompressed data. Useful to immediately retrieve the\n     * deflated output for small inputs.\n     */\n    Deflate.prototype.flush = function () {\n        if (!this.ondata)\n            err(5);\n        if (this.s.l)\n            err(4);\n        this.p(this.b, false);\n        this.s.w = this.s.i, this.s.i -= 2;\n    };\n    return Deflate;\n}());\n\n/**\n * Asynchronous streaming DEFLATE compression\n */\nvar AsyncDeflate = /*#__PURE__*/ (function () {\n    function AsyncDeflate(opts, cb) {\n        astrmify([\n            bDflt,\n            function () { return [astrm, Deflate]; }\n        ], this, StrmOpt.call(this, opts, cb), function (ev) {\n            var strm = new Deflate(ev.data);\n            onmessage = astrm(strm);\n        }, 6, 1);\n    }\n    return AsyncDeflate;\n}());\n\nfunction deflate(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return cbify(data, opts, [\n        bDflt,\n    ], function (ev) { return pbf(deflateSync(ev.data[0], ev.data[1])); }, 0, cb);\n}\n/**\n * Compresses data with DEFLATE without any wrapper\n * @param data The data to compress\n * @param opts The compression options\n * @returns The deflated version of the data\n */\nfunction deflateSync(data, opts) {\n    return dopt(data, opts || {}, 0, 0);\n}\n/**\n * Streaming DEFLATE decompression\n */\nvar Inflate = /*#__PURE__*/ (function () {\n    function Inflate(opts, cb) {\n        // no StrmOpt here to avoid adding to workerizer\n        if (typeof opts == 'function')\n            cb = opts, opts = {};\n        this.ondata = cb;\n        var dict = opts && opts.dictionary && opts.dictionary.subarray(-32768);\n        this.s = { i: 0, b: dict ? dict.length : 0 };\n        this.o = new u8(32768);\n        this.p = new u8(0);\n        if (dict)\n            this.o.set(dict);\n    }\n    Inflate.prototype.e = function (c) {\n        if (!this.ondata)\n            err(5);\n        if (this.d)\n            err(4);\n        if (!this.p.length)\n            this.p = c;\n        else if (c.length) {\n            var n = new u8(this.p.length + c.length);\n            n.set(this.p), n.set(c, this.p.length), this.p = n;\n        }\n    };\n    Inflate.prototype.c = function (final) {\n        this.s.i = +(this.d = final || false);\n        var bts = this.s.b;\n        var dt = inflt(this.p, this.s, this.o);\n        this.ondata(slc(dt, bts, this.s.b), this.d);\n        this.o = slc(dt, this.s.b - 32768), this.s.b = this.o.length;\n        this.p = slc(this.p, (this.s.p / 8) | 0), this.s.p &= 7;\n    };\n    /**\n     * Pushes a chunk to be inflated\n     * @param chunk The chunk to push\n     * @param final Whether this is the final chunk\n     */\n    Inflate.prototype.push = function (chunk, final) {\n        this.e(chunk), this.c(final);\n    };\n    return Inflate;\n}());\n\n/**\n * Asynchronous streaming DEFLATE decompression\n */\nvar AsyncInflate = /*#__PURE__*/ (function () {\n    function AsyncInflate(opts, cb) {\n        astrmify([\n            bInflt,\n            function () { return [astrm, Inflate]; }\n        ], this, StrmOpt.call(this, opts, cb), function (ev) {\n            var strm = new Inflate(ev.data);\n            onmessage = astrm(strm);\n        }, 7, 0);\n    }\n    return AsyncInflate;\n}());\n\nfunction inflate(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return cbify(data, opts, [\n        bInflt\n    ], function (ev) { return pbf(inflateSync(ev.data[0], gopt(ev.data[1]))); }, 1, cb);\n}\n/**\n * Expands DEFLATE data with no wrapper\n * @param data The data to decompress\n * @param opts The decompression options\n * @returns The decompressed version of the data\n */\nfunction inflateSync(data, opts) {\n    return inflt(data, { i: 2 }, opts && opts.out, opts && opts.dictionary);\n}\n// before you yell at me for not just using extends, my reason is that TS inheritance is hard to workerize.\n/**\n * Streaming GZIP compression\n */\nvar Gzip = /*#__PURE__*/ (function () {\n    function Gzip(opts, cb) {\n        this.c = crc();\n        this.l = 0;\n        this.v = 1;\n        Deflate.call(this, opts, cb);\n    }\n    /**\n     * Pushes a chunk to be GZIPped\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Gzip.prototype.push = function (chunk, final) {\n        this.c.p(chunk);\n        this.l += chunk.length;\n        Deflate.prototype.push.call(this, chunk, final);\n    };\n    Gzip.prototype.p = function (c, f) {\n        var raw = dopt(c, this.o, this.v && gzhl(this.o), f && 8, this.s);\n        if (this.v)\n            gzh(raw, this.o), this.v = 0;\n        if (f)\n            wbytes(raw, raw.length - 8, this.c.d()), wbytes(raw, raw.length - 4, this.l);\n        this.ondata(raw, f);\n    };\n    /**\n     * Flushes buffered uncompressed data. Useful to immediately retrieve the\n     * GZIPped output for small inputs.\n     */\n    Gzip.prototype.flush = function () {\n        Deflate.prototype.flush.call(this);\n    };\n    return Gzip;\n}());\n\n/**\n * Asynchronous streaming GZIP compression\n */\nvar AsyncGzip = /*#__PURE__*/ (function () {\n    function AsyncGzip(opts, cb) {\n        astrmify([\n            bDflt,\n            gze,\n            function () { return [astrm, Deflate, Gzip]; }\n        ], this, StrmOpt.call(this, opts, cb), function (ev) {\n            var strm = new Gzip(ev.data);\n            onmessage = astrm(strm);\n        }, 8, 1);\n    }\n    return AsyncGzip;\n}());\n\nfunction gzip(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return cbify(data, opts, [\n        bDflt,\n        gze,\n        function () { return [gzipSync]; }\n    ], function (ev) { return pbf(gzipSync(ev.data[0], ev.data[1])); }, 2, cb);\n}\n/**\n * Compresses data with GZIP\n * @param data The data to compress\n * @param opts The compression options\n * @returns The gzipped version of the data\n */\nfunction gzipSync(data, opts) {\n    if (!opts)\n        opts = {};\n    var c = crc(), l = data.length;\n    c.p(data);\n    var d = dopt(data, opts, gzhl(opts), 8), s = d.length;\n    return gzh(d, opts), wbytes(d, s - 8, c.d()), wbytes(d, s - 4, l), d;\n}\n/**\n * Streaming single or multi-member GZIP decompression\n */\nvar Gunzip = /*#__PURE__*/ (function () {\n    function Gunzip(opts, cb) {\n        this.v = 1;\n        this.r = 0;\n        Inflate.call(this, opts, cb);\n    }\n    /**\n     * Pushes a chunk to be GUNZIPped\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Gunzip.prototype.push = function (chunk, final) {\n        Inflate.prototype.e.call(this, chunk);\n        this.r += chunk.length;\n        if (this.v) {\n            var p = this.p.subarray(this.v - 1);\n            var s = p.length > 3 ? gzs(p) : 4;\n            if (s > p.length) {\n                if (!final)\n                    return;\n            }\n            else if (this.v > 1 && this.onmember) {\n                this.onmember(this.r - p.length);\n            }\n            this.p = p.subarray(s), this.v = 0;\n        }\n        // necessary to prevent TS from using the closure value\n        // This allows for workerization to function correctly\n        Inflate.prototype.c.call(this, final);\n        // process concatenated GZIP\n        if (this.s.f && !this.s.l && !final) {\n            this.v = shft(this.s.p) + 9;\n            this.s = { i: 0 };\n            this.o = new u8(0);\n            this.push(new u8(0), final);\n        }\n    };\n    return Gunzip;\n}());\n\n/**\n * Asynchronous streaming single or multi-member GZIP decompression\n */\nvar AsyncGunzip = /*#__PURE__*/ (function () {\n    function AsyncGunzip(opts, cb) {\n        var _this = this;\n        astrmify([\n            bInflt,\n            guze,\n            function () { return [astrm, Inflate, Gunzip]; }\n        ], this, StrmOpt.call(this, opts, cb), function (ev) {\n            var strm = new Gunzip(ev.data);\n            strm.onmember = function (offset) { return postMessage(offset); };\n            onmessage = astrm(strm);\n        }, 9, 0, function (offset) { return _this.onmember && _this.onmember(offset); });\n    }\n    return AsyncGunzip;\n}());\n\nfunction gunzip(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return cbify(data, opts, [\n        bInflt,\n        guze,\n        function () { return [gunzipSync]; }\n    ], function (ev) { return pbf(gunzipSync(ev.data[0], ev.data[1])); }, 3, cb);\n}\n/**\n * Expands GZIP data\n * @param data The data to decompress\n * @param opts The decompression options\n * @returns The decompressed version of the data\n */\nfunction gunzipSync(data, opts) {\n    var st = gzs(data);\n    if (st + 8 > data.length)\n        err(6, 'invalid gzip data');\n    return inflt(data.subarray(st, -8), { i: 2 }, opts && opts.out || new u8(gzl(data)), opts && opts.dictionary);\n}\n/**\n * Streaming Zlib compression\n */\nvar Zlib = /*#__PURE__*/ (function () {\n    function Zlib(opts, cb) {\n        this.c = adler();\n        this.v = 1;\n        Deflate.call(this, opts, cb);\n    }\n    /**\n     * Pushes a chunk to be zlibbed\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Zlib.prototype.push = function (chunk, final) {\n        this.c.p(chunk);\n        Deflate.prototype.push.call(this, chunk, final);\n    };\n    Zlib.prototype.p = function (c, f) {\n        var raw = dopt(c, this.o, this.v && (this.o.dictionary ? 6 : 2), f && 4, this.s);\n        if (this.v)\n            zlh(raw, this.o), this.v = 0;\n        if (f)\n            wbytes(raw, raw.length - 4, this.c.d());\n        this.ondata(raw, f);\n    };\n    /**\n     * Flushes buffered uncompressed data. Useful to immediately retrieve the\n     * zlibbed output for small inputs.\n     */\n    Zlib.prototype.flush = function () {\n        Deflate.prototype.flush.call(this);\n    };\n    return Zlib;\n}());\n\n/**\n * Asynchronous streaming Zlib compression\n */\nvar AsyncZlib = /*#__PURE__*/ (function () {\n    function AsyncZlib(opts, cb) {\n        astrmify([\n            bDflt,\n            zle,\n            function () { return [astrm, Deflate, Zlib]; }\n        ], this, StrmOpt.call(this, opts, cb), function (ev) {\n            var strm = new Zlib(ev.data);\n            onmessage = astrm(strm);\n        }, 10, 1);\n    }\n    return AsyncZlib;\n}());\n\nfunction zlib(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return cbify(data, opts, [\n        bDflt,\n        zle,\n        function () { return [zlibSync]; }\n    ], function (ev) { return pbf(zlibSync(ev.data[0], ev.data[1])); }, 4, cb);\n}\n/**\n * Compress data with Zlib\n * @param data The data to compress\n * @param opts The compression options\n * @returns The zlib-compressed version of the data\n */\nfunction zlibSync(data, opts) {\n    if (!opts)\n        opts = {};\n    var a = adler();\n    a.p(data);\n    var d = dopt(data, opts, opts.dictionary ? 6 : 2, 4);\n    return zlh(d, opts), wbytes(d, d.length - 4, a.d()), d;\n}\n/**\n * Streaming Zlib decompression\n */\nvar Unzlib = /*#__PURE__*/ (function () {\n    function Unzlib(opts, cb) {\n        Inflate.call(this, opts, cb);\n        this.v = opts && opts.dictionary ? 2 : 1;\n    }\n    /**\n     * Pushes a chunk to be unzlibbed\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Unzlib.prototype.push = function (chunk, final) {\n        Inflate.prototype.e.call(this, chunk);\n        if (this.v) {\n            if (this.p.length < 6 && !final)\n                return;\n            this.p = this.p.subarray(zls(this.p, this.v - 1)), this.v = 0;\n        }\n        if (final) {\n            if (this.p.length < 4)\n                err(6, 'invalid zlib data');\n            this.p = this.p.subarray(0, -4);\n        }\n        // necessary to prevent TS from using the closure value\n        // This allows for workerization to function correctly\n        Inflate.prototype.c.call(this, final);\n    };\n    return Unzlib;\n}());\n\n/**\n * Asynchronous streaming Zlib decompression\n */\nvar AsyncUnzlib = /*#__PURE__*/ (function () {\n    function AsyncUnzlib(opts, cb) {\n        astrmify([\n            bInflt,\n            zule,\n            function () { return [astrm, Inflate, Unzlib]; }\n        ], this, StrmOpt.call(this, opts, cb), function (ev) {\n            var strm = new Unzlib(ev.data);\n            onmessage = astrm(strm);\n        }, 11, 0);\n    }\n    return AsyncUnzlib;\n}());\n\nfunction unzlib(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return cbify(data, opts, [\n        bInflt,\n        zule,\n        function () { return [unzlibSync]; }\n    ], function (ev) { return pbf(unzlibSync(ev.data[0], gopt(ev.data[1]))); }, 5, cb);\n}\n/**\n * Expands Zlib data\n * @param data The data to decompress\n * @param opts The decompression options\n * @returns The decompressed version of the data\n */\nfunction unzlibSync(data, opts) {\n    return inflt(data.subarray(zls(data, opts && opts.dictionary), -4), { i: 2 }, opts && opts.out, opts && opts.dictionary);\n}\n// Default algorithm for compression (used because having a known output size allows faster decompression)\n\n\n/**\n * Streaming GZIP, Zlib, or raw DEFLATE decompression\n */\nvar Decompress = /*#__PURE__*/ (function () {\n    function Decompress(opts, cb) {\n        this.o = StrmOpt.call(this, opts, cb) || {};\n        this.G = Gunzip;\n        this.I = Inflate;\n        this.Z = Unzlib;\n    }\n    // init substream\n    // overriden by AsyncDecompress\n    Decompress.prototype.i = function () {\n        var _this = this;\n        this.s.ondata = function (dat, final) {\n            _this.ondata(dat, final);\n        };\n    };\n    /**\n     * Pushes a chunk to be decompressed\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Decompress.prototype.push = function (chunk, final) {\n        if (!this.ondata)\n            err(5);\n        if (!this.s) {\n            if (this.p && this.p.length) {\n                var n = new u8(this.p.length + chunk.length);\n                n.set(this.p), n.set(chunk, this.p.length);\n            }\n            else\n                this.p = chunk;\n            if (this.p.length > 2) {\n                this.s = (this.p[0] == 31 && this.p[1] == 139 && this.p[2] == 8)\n                    ? new this.G(this.o)\n                    : ((this.p[0] & 15) != 8 || (this.p[0] >> 4) > 7 || ((this.p[0] << 8 | this.p[1]) % 31))\n                        ? new this.I(this.o)\n                        : new this.Z(this.o);\n                this.i();\n                this.s.push(this.p, final);\n                this.p = null;\n            }\n        }\n        else\n            this.s.push(chunk, final);\n    };\n    return Decompress;\n}());\n\n/**\n * Asynchronous streaming GZIP, Zlib, or raw DEFLATE decompression\n */\nvar AsyncDecompress = /*#__PURE__*/ (function () {\n    function AsyncDecompress(opts, cb) {\n        Decompress.call(this, opts, cb);\n        this.queuedSize = 0;\n        this.G = AsyncGunzip;\n        this.I = AsyncInflate;\n        this.Z = AsyncUnzlib;\n    }\n    AsyncDecompress.prototype.i = function () {\n        var _this = this;\n        this.s.ondata = function (err, dat, final) {\n            _this.ondata(err, dat, final);\n        };\n        this.s.ondrain = function (size) {\n            _this.queuedSize -= size;\n            if (_this.ondrain)\n                _this.ondrain(size);\n        };\n    };\n    /**\n     * Pushes a chunk to be decompressed\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    AsyncDecompress.prototype.push = function (chunk, final) {\n        this.queuedSize += chunk.length;\n        Decompress.prototype.push.call(this, chunk, final);\n    };\n    return AsyncDecompress;\n}());\n\nfunction decompress(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    return (data[0] == 31 && data[1] == 139 && data[2] == 8)\n        ? gunzip(data, opts, cb)\n        : ((data[0] & 15) != 8 || (data[0] >> 4) > 7 || ((data[0] << 8 | data[1]) % 31))\n            ? inflate(data, opts, cb)\n            : unzlib(data, opts, cb);\n}\n/**\n * Expands compressed GZIP, Zlib, or raw DEFLATE data, automatically detecting the format\n * @param data The data to decompress\n * @param opts The decompression options\n * @returns The decompressed version of the data\n */\nfunction decompressSync(data, opts) {\n    return (data[0] == 31 && data[1] == 139 && data[2] == 8)\n        ? gunzipSync(data, opts)\n        : ((data[0] & 15) != 8 || (data[0] >> 4) > 7 || ((data[0] << 8 | data[1]) % 31))\n            ? inflateSync(data, opts)\n            : unzlibSync(data, opts);\n}\n// flatten a directory structure\nvar fltn = function (d, p, t, o) {\n    for (var k in d) {\n        var val = d[k], n = p + k, op = o;\n        if (Array.isArray(val))\n            op = mrg(o, val[1]), val = val[0];\n        if (val instanceof u8)\n            t[n] = [val, op];\n        else {\n            t[n += '/'] = [new u8(0), op];\n            fltn(val, n, t, o);\n        }\n    }\n};\n// text encoder\nvar te = typeof TextEncoder != 'undefined' && /*#__PURE__*/ new TextEncoder();\n// text decoder\nvar td = typeof TextDecoder != 'undefined' && /*#__PURE__*/ new TextDecoder();\n// text decoder stream\nvar tds = 0;\ntry {\n    td.decode(et, { stream: true });\n    tds = 1;\n}\ncatch (e) { }\n// decode UTF8\nvar dutf8 = function (d) {\n    for (var r = '', i = 0;;) {\n        var c = d[i++];\n        var eb = (c > 127) + (c > 223) + (c > 239);\n        if (i + eb > d.length)\n            return { s: r, r: slc(d, i - 1) };\n        if (!eb)\n            r += String.fromCharCode(c);\n        else if (eb == 3) {\n            c = ((c & 15) << 18 | (d[i++] & 63) << 12 | (d[i++] & 63) << 6 | (d[i++] & 63)) - 65536,\n                r += String.fromCharCode(55296 | (c >> 10), 56320 | (c & 1023));\n        }\n        else if (eb & 1)\n            r += String.fromCharCode((c & 31) << 6 | (d[i++] & 63));\n        else\n            r += String.fromCharCode((c & 15) << 12 | (d[i++] & 63) << 6 | (d[i++] & 63));\n    }\n};\n/**\n * Streaming UTF-8 decoding\n */\nvar DecodeUTF8 = /*#__PURE__*/ (function () {\n    /**\n     * Creates a UTF-8 decoding stream\n     * @param cb The callback to call whenever data is decoded\n     */\n    function DecodeUTF8(cb) {\n        this.ondata = cb;\n        if (tds)\n            this.t = new TextDecoder();\n        else\n            this.p = et;\n    }\n    /**\n     * Pushes a chunk to be decoded from UTF-8 binary\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    DecodeUTF8.prototype.push = function (chunk, final) {\n        if (!this.ondata)\n            err(5);\n        final = !!final;\n        if (this.t) {\n            this.ondata(this.t.decode(chunk, { stream: true }), final);\n            if (final) {\n                if (this.t.decode().length)\n                    err(8);\n                this.t = null;\n            }\n            return;\n        }\n        if (!this.p)\n            err(4);\n        var dat = new u8(this.p.length + chunk.length);\n        dat.set(this.p);\n        dat.set(chunk, this.p.length);\n        var _a = dutf8(dat), s = _a.s, r = _a.r;\n        if (final) {\n            if (r.length)\n                err(8);\n            this.p = null;\n        }\n        else\n            this.p = r;\n        this.ondata(s, final);\n    };\n    return DecodeUTF8;\n}());\n\n/**\n * Streaming UTF-8 encoding\n */\nvar EncodeUTF8 = /*#__PURE__*/ (function () {\n    /**\n     * Creates a UTF-8 decoding stream\n     * @param cb The callback to call whenever data is encoded\n     */\n    function EncodeUTF8(cb) {\n        this.ondata = cb;\n    }\n    /**\n     * Pushes a chunk to be encoded to UTF-8\n     * @param chunk The string data to push\n     * @param final Whether this is the last chunk\n     */\n    EncodeUTF8.prototype.push = function (chunk, final) {\n        if (!this.ondata)\n            err(5);\n        if (this.d)\n            err(4);\n        this.ondata(strToU8(chunk), this.d = final || false);\n    };\n    return EncodeUTF8;\n}());\n\n/**\n * Converts a string into a Uint8Array for use with compression/decompression methods\n * @param str The string to encode\n * @param latin1 Whether or not to interpret the data as Latin-1. This should\n *               not need to be true unless decoding a binary string.\n * @returns The string encoded in UTF-8/Latin-1 binary\n */\nfunction strToU8(str, latin1) {\n    if (latin1) {\n        var ar_1 = new u8(str.length);\n        for (var i = 0; i < str.length; ++i)\n            ar_1[i] = str.charCodeAt(i);\n        return ar_1;\n    }\n    if (te)\n        return te.encode(str);\n    var l = str.length;\n    var ar = new u8(str.length + (str.length >> 1));\n    var ai = 0;\n    var w = function (v) { ar[ai++] = v; };\n    for (var i = 0; i < l; ++i) {\n        if (ai + 5 > ar.length) {\n            var n = new u8(ai + 8 + ((l - i) << 1));\n            n.set(ar);\n            ar = n;\n        }\n        var c = str.charCodeAt(i);\n        if (c < 128 || latin1)\n            w(c);\n        else if (c < 2048)\n            w(192 | (c >> 6)), w(128 | (c & 63));\n        else if (c > 55295 && c < 57344)\n            c = 65536 + (c & 1023 << 10) | (str.charCodeAt(++i) & 1023),\n                w(240 | (c >> 18)), w(128 | ((c >> 12) & 63)), w(128 | ((c >> 6) & 63)), w(128 | (c & 63));\n        else\n            w(224 | (c >> 12)), w(128 | ((c >> 6) & 63)), w(128 | (c & 63));\n    }\n    return slc(ar, 0, ai);\n}\n/**\n * Converts a Uint8Array to a string\n * @param dat The data to decode to string\n * @param latin1 Whether or not to interpret the data as Latin-1. This should\n *               not need to be true unless encoding to binary string.\n * @returns The original UTF-8/Latin-1 string\n */\nfunction strFromU8(dat, latin1) {\n    if (latin1) {\n        var r = '';\n        for (var i = 0; i < dat.length; i += 16384)\n            r += String.fromCharCode.apply(null, dat.subarray(i, i + 16384));\n        return r;\n    }\n    else if (td) {\n        return td.decode(dat);\n    }\n    else {\n        var _a = dutf8(dat), s = _a.s, r = _a.r;\n        if (r.length)\n            err(8);\n        return s;\n    }\n}\n;\n// deflate bit flag\nvar dbf = function (l) { return l == 1 ? 3 : l < 6 ? 2 : l == 9 ? 1 : 0; };\n// skip local zip header\nvar slzh = function (d, b) { return b + 30 + b2(d, b + 26) + b2(d, b + 28); };\n// read zip header\nvar zh = function (d, b, z) {\n    var fnl = b2(d, b + 28), fn = strFromU8(d.subarray(b + 46, b + 46 + fnl), !(b2(d, b + 8) & 2048)), es = b + 46 + fnl, bs = b4(d, b + 20);\n    var _a = z && bs == 4294967295 ? z64e(d, es) : [bs, b4(d, b + 24), b4(d, b + 42)], sc = _a[0], su = _a[1], off = _a[2];\n    return [b2(d, b + 10), sc, su, fn, es + b2(d, b + 30) + b2(d, b + 32), off];\n};\n// read zip64 extra field\nvar z64e = function (d, b) {\n    for (; b2(d, b) != 1; b += 4 + b2(d, b + 2))\n        ;\n    return [b8(d, b + 12), b8(d, b + 4), b8(d, b + 20)];\n};\n// extra field length\nvar exfl = function (ex) {\n    var le = 0;\n    if (ex) {\n        for (var k in ex) {\n            var l = ex[k].length;\n            if (l > 65535)\n                err(9);\n            le += l + 4;\n        }\n    }\n    return le;\n};\n// write zip header\nvar wzh = function (d, b, f, fn, u, c, ce, co) {\n    var fl = fn.length, ex = f.extra, col = co && co.length;\n    var exl = exfl(ex);\n    wbytes(d, b, ce != null ? 0x2014B50 : 0x4034B50), b += 4;\n    if (ce != null)\n        d[b++] = 20, d[b++] = f.os;\n    d[b] = 20, b += 2; // spec compliance? what's that?\n    d[b++] = (f.flag << 1) | (c < 0 && 8), d[b++] = u && 8;\n    d[b++] = f.compression & 255, d[b++] = f.compression >> 8;\n    var dt = new Date(f.mtime == null ? Date.now() : f.mtime), y = dt.getFullYear() - 1980;\n    if (y < 0 || y > 119)\n        err(10);\n    wbytes(d, b, (y << 25) | ((dt.getMonth() + 1) << 21) | (dt.getDate() << 16) | (dt.getHours() << 11) | (dt.getMinutes() << 5) | (dt.getSeconds() >> 1)), b += 4;\n    if (c != -1) {\n        wbytes(d, b, f.crc);\n        wbytes(d, b + 4, c < 0 ? -c - 2 : c);\n        wbytes(d, b + 8, f.size);\n    }\n    wbytes(d, b + 12, fl);\n    wbytes(d, b + 14, exl), b += 16;\n    if (ce != null) {\n        wbytes(d, b, col);\n        wbytes(d, b + 6, f.attrs);\n        wbytes(d, b + 10, ce), b += 14;\n    }\n    d.set(fn, b);\n    b += fl;\n    if (exl) {\n        for (var k in ex) {\n            var exf = ex[k], l = exf.length;\n            wbytes(d, b, +k);\n            wbytes(d, b + 2, l);\n            d.set(exf, b + 4), b += 4 + l;\n        }\n    }\n    if (col)\n        d.set(co, b), b += col;\n    return b;\n};\n// write zip footer (end of central directory)\nvar wzf = function (o, b, c, d, e) {\n    wbytes(o, b, 0x6054B50); // skip disk\n    wbytes(o, b + 8, c);\n    wbytes(o, b + 10, c);\n    wbytes(o, b + 12, d);\n    wbytes(o, b + 16, e);\n};\n/**\n * A pass-through stream to keep data uncompressed in a ZIP archive.\n */\nvar ZipPassThrough = /*#__PURE__*/ (function () {\n    /**\n     * Creates a pass-through stream that can be added to ZIP archives\n     * @param filename The filename to associate with this data stream\n     */\n    function ZipPassThrough(filename) {\n        this.filename = filename;\n        this.c = crc();\n        this.size = 0;\n        this.compression = 0;\n    }\n    /**\n     * Processes a chunk and pushes to the output stream. You can override this\n     * method in a subclass for custom behavior, but by default this passes\n     * the data through. You must call this.ondata(err, chunk, final) at some\n     * point in this method.\n     * @param chunk The chunk to process\n     * @param final Whether this is the last chunk\n     */\n    ZipPassThrough.prototype.process = function (chunk, final) {\n        this.ondata(null, chunk, final);\n    };\n    /**\n     * Pushes a chunk to be added. If you are subclassing this with a custom\n     * compression algorithm, note that you must push data from the source\n     * file only, pre-compression.\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    ZipPassThrough.prototype.push = function (chunk, final) {\n        if (!this.ondata)\n            err(5);\n        this.c.p(chunk);\n        this.size += chunk.length;\n        if (final)\n            this.crc = this.c.d();\n        this.process(chunk, final || false);\n    };\n    return ZipPassThrough;\n}());\n\n// I don't extend because TypeScript extension adds 1kB of runtime bloat\n/**\n * Streaming DEFLATE compression for ZIP archives. Prefer using AsyncZipDeflate\n * for better performance\n */\nvar ZipDeflate = /*#__PURE__*/ (function () {\n    /**\n     * Creates a DEFLATE stream that can be added to ZIP archives\n     * @param filename The filename to associate with this data stream\n     * @param opts The compression options\n     */\n    function ZipDeflate(filename, opts) {\n        var _this = this;\n        if (!opts)\n            opts = {};\n        ZipPassThrough.call(this, filename);\n        this.d = new Deflate(opts, function (dat, final) {\n            _this.ondata(null, dat, final);\n        });\n        this.compression = 8;\n        this.flag = dbf(opts.level);\n    }\n    ZipDeflate.prototype.process = function (chunk, final) {\n        try {\n            this.d.push(chunk, final);\n        }\n        catch (e) {\n            this.ondata(e, null, final);\n        }\n    };\n    /**\n     * Pushes a chunk to be deflated\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    ZipDeflate.prototype.push = function (chunk, final) {\n        ZipPassThrough.prototype.push.call(this, chunk, final);\n    };\n    return ZipDeflate;\n}());\n\n/**\n * Asynchronous streaming DEFLATE compression for ZIP archives\n */\nvar AsyncZipDeflate = /*#__PURE__*/ (function () {\n    /**\n     * Creates an asynchronous DEFLATE stream that can be added to ZIP archives\n     * @param filename The filename to associate with this data stream\n     * @param opts The compression options\n     */\n    function AsyncZipDeflate(filename, opts) {\n        var _this = this;\n        if (!opts)\n            opts = {};\n        ZipPassThrough.call(this, filename);\n        this.d = new AsyncDeflate(opts, function (err, dat, final) {\n            _this.ondata(err, dat, final);\n        });\n        this.compression = 8;\n        this.flag = dbf(opts.level);\n        this.terminate = this.d.terminate;\n    }\n    AsyncZipDeflate.prototype.process = function (chunk, final) {\n        this.d.push(chunk, final);\n    };\n    /**\n     * Pushes a chunk to be deflated\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    AsyncZipDeflate.prototype.push = function (chunk, final) {\n        ZipPassThrough.prototype.push.call(this, chunk, final);\n    };\n    return AsyncZipDeflate;\n}());\n\n// TODO: Better tree shaking\n/**\n * A zippable archive to which files can incrementally be added\n */\nvar Zip = /*#__PURE__*/ (function () {\n    /**\n     * Creates an empty ZIP archive to which files can be added\n     * @param cb The callback to call whenever data for the generated ZIP archive\n     *           is available\n     */\n    function Zip(cb) {\n        this.ondata = cb;\n        this.u = [];\n        this.d = 1;\n    }\n    /**\n     * Adds a file to the ZIP archive\n     * @param file The file stream to add\n     */\n    Zip.prototype.add = function (file) {\n        var _this = this;\n        if (!this.ondata)\n            err(5);\n        // finishing or finished\n        if (this.d & 2)\n            this.ondata(err(4 + (this.d & 1) * 8, 0, 1), null, false);\n        else {\n            var f = strToU8(file.filename), fl_1 = f.length;\n            var com = file.comment, o = com && strToU8(com);\n            var u = fl_1 != file.filename.length || (o && (com.length != o.length));\n            var hl_1 = fl_1 + exfl(file.extra) + 30;\n            if (fl_1 > 65535)\n                this.ondata(err(11, 0, 1), null, false);\n            var header = new u8(hl_1);\n            wzh(header, 0, file, f, u, -1);\n            var chks_1 = [header];\n            var pAll_1 = function () {\n                for (var _i = 0, chks_2 = chks_1; _i < chks_2.length; _i++) {\n                    var chk = chks_2[_i];\n                    _this.ondata(null, chk, false);\n                }\n                chks_1 = [];\n            };\n            var tr_1 = this.d;\n            this.d = 0;\n            var ind_1 = this.u.length;\n            var uf_1 = mrg(file, {\n                f: f,\n                u: u,\n                o: o,\n                t: function () {\n                    if (file.terminate)\n                        file.terminate();\n                },\n                r: function () {\n                    pAll_1();\n                    if (tr_1) {\n                        var nxt = _this.u[ind_1 + 1];\n                        if (nxt)\n                            nxt.r();\n                        else\n                            _this.d = 1;\n                    }\n                    tr_1 = 1;\n                }\n            });\n            var cl_1 = 0;\n            file.ondata = function (err, dat, final) {\n                if (err) {\n                    _this.ondata(err, dat, final);\n                    _this.terminate();\n                }\n                else {\n                    cl_1 += dat.length;\n                    chks_1.push(dat);\n                    if (final) {\n                        var dd = new u8(16);\n                        wbytes(dd, 0, 0x8074B50);\n                        wbytes(dd, 4, file.crc);\n                        wbytes(dd, 8, cl_1);\n                        wbytes(dd, 12, file.size);\n                        chks_1.push(dd);\n                        uf_1.c = cl_1, uf_1.b = hl_1 + cl_1 + 16, uf_1.crc = file.crc, uf_1.size = file.size;\n                        if (tr_1)\n                            uf_1.r();\n                        tr_1 = 1;\n                    }\n                    else if (tr_1)\n                        pAll_1();\n                }\n            };\n            this.u.push(uf_1);\n        }\n    };\n    /**\n     * Ends the process of adding files and prepares to emit the final chunks.\n     * This *must* be called after adding all desired files for the resulting\n     * ZIP file to work properly.\n     */\n    Zip.prototype.end = function () {\n        var _this = this;\n        if (this.d & 2) {\n            this.ondata(err(4 + (this.d & 1) * 8, 0, 1), null, true);\n            return;\n        }\n        if (this.d)\n            this.e();\n        else\n            this.u.push({\n                r: function () {\n                    if (!(_this.d & 1))\n                        return;\n                    _this.u.splice(-1, 1);\n                    _this.e();\n                },\n                t: function () { }\n            });\n        this.d = 3;\n    };\n    Zip.prototype.e = function () {\n        var bt = 0, l = 0, tl = 0;\n        for (var _i = 0, _a = this.u; _i < _a.length; _i++) {\n            var f = _a[_i];\n            tl += 46 + f.f.length + exfl(f.extra) + (f.o ? f.o.length : 0);\n        }\n        var out = new u8(tl + 22);\n        for (var _b = 0, _c = this.u; _b < _c.length; _b++) {\n            var f = _c[_b];\n            wzh(out, bt, f, f.f, f.u, -f.c - 2, l, f.o);\n            bt += 46 + f.f.length + exfl(f.extra) + (f.o ? f.o.length : 0), l += f.b;\n        }\n        wzf(out, bt, this.u.length, tl, l);\n        this.ondata(null, out, true);\n        this.d = 2;\n    };\n    /**\n     * A method to terminate any internal workers used by the stream. Subsequent\n     * calls to add() will fail.\n     */\n    Zip.prototype.terminate = function () {\n        for (var _i = 0, _a = this.u; _i < _a.length; _i++) {\n            var f = _a[_i];\n            f.t();\n        }\n        this.d = 2;\n    };\n    return Zip;\n}());\n\nfunction zip(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    var r = {};\n    fltn(data, '', r, opts);\n    var k = Object.keys(r);\n    var lft = k.length, o = 0, tot = 0;\n    var slft = lft, files = new Array(lft);\n    var term = [];\n    var tAll = function () {\n        for (var i = 0; i < term.length; ++i)\n            term[i]();\n    };\n    var cbd = function (a, b) {\n        mt(function () { cb(a, b); });\n    };\n    mt(function () { cbd = cb; });\n    var cbf = function () {\n        var out = new u8(tot + 22), oe = o, cdl = tot - o;\n        tot = 0;\n        for (var i = 0; i < slft; ++i) {\n            var f = files[i];\n            try {\n                var l = f.c.length;\n                wzh(out, tot, f, f.f, f.u, l);\n                var badd = 30 + f.f.length + exfl(f.extra);\n                var loc = tot + badd;\n                out.set(f.c, loc);\n                wzh(out, o, f, f.f, f.u, l, tot, f.m), o += 16 + badd + (f.m ? f.m.length : 0), tot = loc + l;\n            }\n            catch (e) {\n                return cbd(e, null);\n            }\n        }\n        wzf(out, o, files.length, cdl, oe);\n        cbd(null, out);\n    };\n    if (!lft)\n        cbf();\n    var _loop_1 = function (i) {\n        var fn = k[i];\n        var _a = r[fn], file = _a[0], p = _a[1];\n        var c = crc(), size = file.length;\n        c.p(file);\n        var f = strToU8(fn), s = f.length;\n        var com = p.comment, m = com && strToU8(com), ms = m && m.length;\n        var exl = exfl(p.extra);\n        var compression = p.level == 0 ? 0 : 8;\n        var cbl = function (e, d) {\n            if (e) {\n                tAll();\n                cbd(e, null);\n            }\n            else {\n                var l = d.length;\n                files[i] = mrg(p, {\n                    size: size,\n                    crc: c.d(),\n                    c: d,\n                    f: f,\n                    m: m,\n                    u: s != fn.length || (m && (com.length != ms)),\n                    compression: compression\n                });\n                o += 30 + s + exl + l;\n                tot += 76 + 2 * (s + exl) + (ms || 0) + l;\n                if (!--lft)\n                    cbf();\n            }\n        };\n        if (s > 65535)\n            cbl(err(11, 0, 1), null);\n        if (!compression)\n            cbl(null, file);\n        else if (size < 160000) {\n            try {\n                cbl(null, deflateSync(file, p));\n            }\n            catch (e) {\n                cbl(e, null);\n            }\n        }\n        else\n            term.push(deflate(file, p, cbl));\n    };\n    // Cannot use lft because it can decrease\n    for (var i = 0; i < slft; ++i) {\n        _loop_1(i);\n    }\n    return tAll;\n}\n/**\n * Synchronously creates a ZIP file. Prefer using `zip` for better performance\n * with more than one file.\n * @param data The directory structure for the ZIP archive\n * @param opts The main options, merged with per-file options\n * @returns The generated ZIP archive\n */\nfunction zipSync(data, opts) {\n    if (!opts)\n        opts = {};\n    var r = {};\n    var files = [];\n    fltn(data, '', r, opts);\n    var o = 0;\n    var tot = 0;\n    for (var fn in r) {\n        var _a = r[fn], file = _a[0], p = _a[1];\n        var compression = p.level == 0 ? 0 : 8;\n        var f = strToU8(fn), s = f.length;\n        var com = p.comment, m = com && strToU8(com), ms = m && m.length;\n        var exl = exfl(p.extra);\n        if (s > 65535)\n            err(11);\n        var d = compression ? deflateSync(file, p) : file, l = d.length;\n        var c = crc();\n        c.p(file);\n        files.push(mrg(p, {\n            size: file.length,\n            crc: c.d(),\n            c: d,\n            f: f,\n            m: m,\n            u: s != fn.length || (m && (com.length != ms)),\n            o: o,\n            compression: compression\n        }));\n        o += 30 + s + exl + l;\n        tot += 76 + 2 * (s + exl) + (ms || 0) + l;\n    }\n    var out = new u8(tot + 22), oe = o, cdl = tot - o;\n    for (var i = 0; i < files.length; ++i) {\n        var f = files[i];\n        wzh(out, f.o, f, f.f, f.u, f.c.length);\n        var badd = 30 + f.f.length + exfl(f.extra);\n        out.set(f.c, f.o + badd);\n        wzh(out, o, f, f.f, f.u, f.c.length, f.o, f.m), o += 16 + badd + (f.m ? f.m.length : 0);\n    }\n    wzf(out, o, files.length, cdl, oe);\n    return out;\n}\n/**\n * Streaming pass-through decompression for ZIP archives\n */\nvar UnzipPassThrough = /*#__PURE__*/ (function () {\n    function UnzipPassThrough() {\n    }\n    UnzipPassThrough.prototype.push = function (data, final) {\n        this.ondata(null, data, final);\n    };\n    UnzipPassThrough.compression = 0;\n    return UnzipPassThrough;\n}());\n\n/**\n * Streaming DEFLATE decompression for ZIP archives. Prefer AsyncZipInflate for\n * better performance.\n */\nvar UnzipInflate = /*#__PURE__*/ (function () {\n    /**\n     * Creates a DEFLATE decompression that can be used in ZIP archives\n     */\n    function UnzipInflate() {\n        var _this = this;\n        this.i = new Inflate(function (dat, final) {\n            _this.ondata(null, dat, final);\n        });\n    }\n    UnzipInflate.prototype.push = function (data, final) {\n        try {\n            this.i.push(data, final);\n        }\n        catch (e) {\n            this.ondata(e, null, final);\n        }\n    };\n    UnzipInflate.compression = 8;\n    return UnzipInflate;\n}());\n\n/**\n * Asynchronous streaming DEFLATE decompression for ZIP archives\n */\nvar AsyncUnzipInflate = /*#__PURE__*/ (function () {\n    /**\n     * Creates a DEFLATE decompression that can be used in ZIP archives\n     */\n    function AsyncUnzipInflate(_, sz) {\n        var _this = this;\n        if (sz < 320000) {\n            this.i = new Inflate(function (dat, final) {\n                _this.ondata(null, dat, final);\n            });\n        }\n        else {\n            this.i = new AsyncInflate(function (err, dat, final) {\n                _this.ondata(err, dat, final);\n            });\n            this.terminate = this.i.terminate;\n        }\n    }\n    AsyncUnzipInflate.prototype.push = function (data, final) {\n        if (this.i.terminate)\n            data = slc(data, 0);\n        this.i.push(data, final);\n    };\n    AsyncUnzipInflate.compression = 8;\n    return AsyncUnzipInflate;\n}());\n\n/**\n * A ZIP archive decompression stream that emits files as they are discovered\n */\nvar Unzip = /*#__PURE__*/ (function () {\n    /**\n     * Creates a ZIP decompression stream\n     * @param cb The callback to call whenever a file in the ZIP archive is found\n     */\n    function Unzip(cb) {\n        this.onfile = cb;\n        this.k = [];\n        this.o = {\n            0: UnzipPassThrough\n        };\n        this.p = et;\n    }\n    /**\n     * Pushes a chunk to be unzipped\n     * @param chunk The chunk to push\n     * @param final Whether this is the last chunk\n     */\n    Unzip.prototype.push = function (chunk, final) {\n        var _this = this;\n        if (!this.onfile)\n            err(5);\n        if (!this.p)\n            err(4);\n        if (this.c > 0) {\n            var len = Math.min(this.c, chunk.length);\n            var toAdd = chunk.subarray(0, len);\n            this.c -= len;\n            if (this.d)\n                this.d.push(toAdd, !this.c);\n            else\n                this.k[0].push(toAdd);\n            chunk = chunk.subarray(len);\n            if (chunk.length)\n                return this.push(chunk, final);\n        }\n        else {\n            var f = 0, i = 0, is = void 0, buf = void 0;\n            if (!this.p.length)\n                buf = chunk;\n            else if (!chunk.length)\n                buf = this.p;\n            else {\n                buf = new u8(this.p.length + chunk.length);\n                buf.set(this.p), buf.set(chunk, this.p.length);\n            }\n            var l = buf.length, oc = this.c, add = oc && this.d;\n            var _loop_2 = function () {\n                var _a;\n                var sig = b4(buf, i);\n                if (sig == 0x4034B50) {\n                    f = 1, is = i;\n                    this_1.d = null;\n                    this_1.c = 0;\n                    var bf = b2(buf, i + 6), cmp_1 = b2(buf, i + 8), u = bf & 2048, dd = bf & 8, fnl = b2(buf, i + 26), es = b2(buf, i + 28);\n                    if (l > i + 30 + fnl + es) {\n                        var chks_3 = [];\n                        this_1.k.unshift(chks_3);\n                        f = 2;\n                        var sc_1 = b4(buf, i + 18), su_1 = b4(buf, i + 22);\n                        var fn_1 = strFromU8(buf.subarray(i + 30, i += 30 + fnl), !u);\n                        if (sc_1 == 4294967295) {\n                            _a = dd ? [-2] : z64e(buf, i), sc_1 = _a[0], su_1 = _a[1];\n                        }\n                        else if (dd)\n                            sc_1 = -1;\n                        i += es;\n                        this_1.c = sc_1;\n                        var d_1;\n                        var file_1 = {\n                            name: fn_1,\n                            compression: cmp_1,\n                            start: function () {\n                                if (!file_1.ondata)\n                                    err(5);\n                                if (!sc_1)\n                                    file_1.ondata(null, et, true);\n                                else {\n                                    var ctr = _this.o[cmp_1];\n                                    if (!ctr)\n                                        file_1.ondata(err(14, 'unknown compression type ' + cmp_1, 1), null, false);\n                                    d_1 = sc_1 < 0 ? new ctr(fn_1) : new ctr(fn_1, sc_1, su_1);\n                                    d_1.ondata = function (err, dat, final) { file_1.ondata(err, dat, final); };\n                                    for (var _i = 0, chks_4 = chks_3; _i < chks_4.length; _i++) {\n                                        var dat = chks_4[_i];\n                                        d_1.push(dat, false);\n                                    }\n                                    if (_this.k[0] == chks_3 && _this.c)\n                                        _this.d = d_1;\n                                    else\n                                        d_1.push(et, true);\n                                }\n                            },\n                            terminate: function () {\n                                if (d_1 && d_1.terminate)\n                                    d_1.terminate();\n                            }\n                        };\n                        if (sc_1 >= 0)\n                            file_1.size = sc_1, file_1.originalSize = su_1;\n                        this_1.onfile(file_1);\n                    }\n                    return \"break\";\n                }\n                else if (oc) {\n                    if (sig == 0x8074B50) {\n                        is = i += 12 + (oc == -2 && 8), f = 3, this_1.c = 0;\n                        return \"break\";\n                    }\n                    else if (sig == 0x2014B50) {\n                        is = i -= 4, f = 3, this_1.c = 0;\n                        return \"break\";\n                    }\n                }\n            };\n            var this_1 = this;\n            for (; i < l - 4; ++i) {\n                var state_1 = _loop_2();\n                if (state_1 === \"break\")\n                    break;\n            }\n            this.p = et;\n            if (oc < 0) {\n                var dat = f ? buf.subarray(0, is - 12 - (oc == -2 && 8) - (b4(buf, is - 16) == 0x8074B50 && 4)) : buf.subarray(0, i);\n                if (add)\n                    add.push(dat, !!f);\n                else\n                    this.k[+(f == 2)].push(dat);\n            }\n            if (f & 2)\n                return this.push(buf.subarray(i), final);\n            this.p = buf.subarray(i);\n        }\n        if (final) {\n            if (this.c)\n                err(13);\n            this.p = null;\n        }\n    };\n    /**\n     * Registers a decoder with the stream, allowing for files compressed with\n     * the compression type provided to be expanded correctly\n     * @param decoder The decoder constructor\n     */\n    Unzip.prototype.register = function (decoder) {\n        this.o[decoder.compression] = decoder;\n    };\n    return Unzip;\n}());\n\nvar mt = typeof queueMicrotask == 'function' ? queueMicrotask : typeof setTimeout == 'function' ? setTimeout : function (fn) { fn(); };\nfunction unzip(data, opts, cb) {\n    if (!cb)\n        cb = opts, opts = {};\n    if (typeof cb != 'function')\n        err(7);\n    var term = [];\n    var tAll = function () {\n        for (var i = 0; i < term.length; ++i)\n            term[i]();\n    };\n    var files = {};\n    var cbd = function (a, b) {\n        mt(function () { cb(a, b); });\n    };\n    mt(function () { cbd = cb; });\n    var e = data.length - 22;\n    for (; b4(data, e) != 0x6054B50; --e) {\n        if (!e || data.length - e > 65558) {\n            cbd(err(13, 0, 1), null);\n            return tAll;\n        }\n    }\n    ;\n    var lft = b2(data, e + 8);\n    if (lft) {\n        var c = lft;\n        var o = b4(data, e + 16);\n        var z = o == 4294967295 || c == 65535;\n        if (z) {\n            var ze = b4(data, e - 12);\n            z = b4(data, ze) == 0x6064B50;\n            if (z) {\n                c = lft = b4(data, ze + 32);\n                o = b4(data, ze + 48);\n            }\n        }\n        var fltr = opts && opts.filter;\n        var _loop_3 = function (i) {\n            var _a = zh(data, o, z), c_1 = _a[0], sc = _a[1], su = _a[2], fn = _a[3], no = _a[4], off = _a[5], b = slzh(data, off);\n            o = no;\n            var cbl = function (e, d) {\n                if (e) {\n                    tAll();\n                    cbd(e, null);\n                }\n                else {\n                    if (d)\n                        files[fn] = d;\n                    if (!--lft)\n                        cbd(null, files);\n                }\n            };\n            if (!fltr || fltr({\n                name: fn,\n                size: sc,\n                originalSize: su,\n                compression: c_1\n            })) {\n                if (!c_1)\n                    cbl(null, slc(data, b, b + sc));\n                else if (c_1 == 8) {\n                    var infl = data.subarray(b, b + sc);\n                    // Synchronously decompress under 512KB, or barely-compressed data\n                    if (su < 524288 || sc > 0.8 * su) {\n                        try {\n                            cbl(null, inflateSync(infl, { out: new u8(su) }));\n                        }\n                        catch (e) {\n                            cbl(e, null);\n                        }\n                    }\n                    else\n                        term.push(inflate(infl, { size: su }, cbl));\n                }\n                else\n                    cbl(err(14, 'unknown compression type ' + c_1, 1), null);\n            }\n            else\n                cbl(null, null);\n        };\n        for (var i = 0; i < c; ++i) {\n            _loop_3(i);\n        }\n    }\n    else\n        cbd(null, {});\n    return tAll;\n}\n/**\n * Synchronously decompresses a ZIP archive. Prefer using `unzip` for better\n * performance with more than one file.\n * @param data The raw compressed ZIP file\n * @param opts The ZIP extraction options\n * @returns The decompressed files\n */\nfunction unzipSync(data, opts) {\n    var files = {};\n    var e = data.length - 22;\n    for (; b4(data, e) != 0x6054B50; --e) {\n        if (!e || data.length - e > 65558)\n            err(13);\n    }\n    ;\n    var c = b2(data, e + 8);\n    if (!c)\n        return {};\n    var o = b4(data, e + 16);\n    var z = o == 4294967295 || c == 65535;\n    if (z) {\n        var ze = b4(data, e - 12);\n        z = b4(data, ze) == 0x6064B50;\n        if (z) {\n            c = b4(data, ze + 32);\n            o = b4(data, ze + 48);\n        }\n    }\n    var fltr = opts && opts.filter;\n    for (var i = 0; i < c; ++i) {\n        var _a = zh(data, o, z), c_2 = _a[0], sc = _a[1], su = _a[2], fn = _a[3], no = _a[4], off = _a[5], b = slzh(data, off);\n        o = no;\n        if (!fltr || fltr({\n            name: fn,\n            size: sc,\n            originalSize: su,\n            compression: c_2\n        })) {\n            if (!c_2)\n                files[fn] = slc(data, b, b + sc);\n            else if (c_2 == 8)\n                files[fn] = inflateSync(data.subarray(b, b + sc), { out: new u8(su) });\n            else\n                err(14, 'unknown compression type ' + c_2);\n        }\n    }\n    return files;\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/fflate/esm/browser.js?");

/***/ }),

/***/ "./node_modules/google-protobuf/google-protobuf.js":
/*!*********************************************************!*\
  !*** ./node_modules/google-protobuf/google-protobuf.js ***!
  \*********************************************************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {

eval("/*\n\n Copyright The Closure Library Authors.\n SPDX-License-Identifier: Apache-2.0\n*/\nvar aa=\"function\"==typeof Object.defineProperties?Object.defineProperty:function(a,b,c){a!=Array.prototype&&a!=Object.prototype&&(a[b]=c.value)},e=\"undefined\"!=typeof window&&window===this?this:\"undefined\"!=typeof __webpack_require__.g&&null!=__webpack_require__.g?__webpack_require__.g:this;function ba(a,b){if(b){var c=e;a=a.split(\".\");for(var d=0;d<a.length-1;d++){var f=a[d];f in c||(c[f]={});c=c[f]}a=a[a.length-1];d=c[a];b=b(d);b!=d&&null!=b&&aa(c,a,{configurable:!0,writable:!0,value:b})}}\nfunction ca(a){var b=0;return function(){return b<a.length?{done:!1,value:a[b++]}:{done:!0}}}function da(){da=function(){};e.Symbol||(e.Symbol=ea)}function fa(a,b){this.a=a;aa(this,\"description\",{configurable:!0,writable:!0,value:b})}fa.prototype.toString=function(){return this.a};var ea=function(){function a(c){if(this instanceof a)throw new TypeError(\"Symbol is not a constructor\");return new fa(\"jscomp_symbol_\"+(c||\"\")+\"_\"+b++,c)}var b=0;return a}();\nfunction ha(){da();var a=e.Symbol.iterator;a||(a=e.Symbol.iterator=e.Symbol(\"Symbol.iterator\"));\"function\"!=typeof Array.prototype[a]&&aa(Array.prototype,a,{configurable:!0,writable:!0,value:function(){return ia(ca(this))}});ha=function(){}}function ia(a){ha();a={next:a};a[e.Symbol.iterator]=function(){return this};return a}\nfunction ja(a,b){ha();a instanceof String&&(a+=\"\");var c=0,d={next:function(){if(c<a.length){var f=c++;return{value:b(f,a[f]),done:!1}}d.next=function(){return{done:!0,value:void 0}};return d.next()}};d[Symbol.iterator]=function(){return d};return d}ba(\"Array.prototype.entries\",function(a){return a?a:function(){return ja(this,function(b,c){return[b,c]})}});var ka=this||self;\nfunction g(a,b,c){a=a.split(\".\");c=c||ka;a[0]in c||\"undefined\"==typeof c.execScript||c.execScript(\"var \"+a[0]);for(var d;a.length&&(d=a.shift());)a.length||void 0===b?c[d]&&c[d]!==Object.prototype[d]?c=c[d]:c=c[d]={}:c[d]=b}\nfunction k(a){var b=typeof a;if(\"object\"==b)if(a){if(a instanceof Array)return\"array\";if(a instanceof Object)return b;var c=Object.prototype.toString.call(a);if(\"[object Window]\"==c)return\"object\";if(\"[object Array]\"==c||\"number\"==typeof a.length&&\"undefined\"!=typeof a.splice&&\"undefined\"!=typeof a.propertyIsEnumerable&&!a.propertyIsEnumerable(\"splice\"))return\"array\";if(\"[object Function]\"==c||\"undefined\"!=typeof a.call&&\"undefined\"!=typeof a.propertyIsEnumerable&&!a.propertyIsEnumerable(\"call\"))return\"function\"}else return\"null\";\nelse if(\"function\"==b&&\"undefined\"==typeof a.call)return\"object\";return b}function la(a){var b=typeof a;return\"object\"==b&&null!=a||\"function\"==b}function ma(a,b,c){g(a,b,c)}function na(a,b){function c(){}c.prototype=b.prototype;a.prototype=new c;a.prototype.constructor=a};var oa=\"constructor hasOwnProperty isPrototypeOf propertyIsEnumerable toLocaleString toString valueOf\".split(\" \");function pa(a,b){for(var c,d,f=1;f<arguments.length;f++){d=arguments[f];for(c in d)a[c]=d[c];for(var h=0;h<oa.length;h++)c=oa[h],Object.prototype.hasOwnProperty.call(d,c)&&(a[c]=d[c])}};var qa=Array.prototype.forEach?function(a,b){Array.prototype.forEach.call(a,b,void 0)}:function(a,b){for(var c=a.length,d=\"string\"===typeof a?a.split(\"\"):a,f=0;f<c;f++)f in d&&b.call(void 0,d[f],f,a)},l=Array.prototype.map?function(a,b){return Array.prototype.map.call(a,b,void 0)}:function(a,b){for(var c=a.length,d=Array(c),f=\"string\"===typeof a?a.split(\"\"):a,h=0;h<c;h++)h in f&&(d[h]=b.call(void 0,f[h],h,a));return d};\nfunction ra(a,b,c){return 2>=arguments.length?Array.prototype.slice.call(a,b):Array.prototype.slice.call(a,b,c)};function sa(a,b,c,d){var f=\"Assertion failed\";if(c){f+=\": \"+c;var h=d}else a&&(f+=\": \"+a,h=b);throw Error(f,h||[]);}function n(a,b,c){for(var d=[],f=2;f<arguments.length;++f)d[f-2]=arguments[f];a||sa(\"\",null,b,d);return a}function ta(a,b,c){for(var d=[],f=2;f<arguments.length;++f)d[f-2]=arguments[f];\"string\"!==typeof a&&sa(\"Expected string but got %s: %s.\",[k(a),a],b,d)}\nfunction ua(a,b,c){for(var d=[],f=2;f<arguments.length;++f)d[f-2]=arguments[f];Array.isArray(a)||sa(\"Expected array but got %s: %s.\",[k(a),a],b,d)}function p(a,b){for(var c=[],d=1;d<arguments.length;++d)c[d-1]=arguments[d];throw Error(\"Failure\"+(a?\": \"+a:\"\"),c);}function q(a,b,c,d){for(var f=[],h=3;h<arguments.length;++h)f[h-3]=arguments[h];a instanceof b||sa(\"Expected instanceof %s but got %s.\",[va(b),va(a)],c,f)}\nfunction va(a){return a instanceof Function?a.displayName||a.name||\"unknown type name\":a instanceof Object?a.constructor.displayName||a.constructor.name||Object.prototype.toString.call(a):null===a?\"null\":typeof a};function r(a,b){this.c=a;this.b=b;this.a={};this.arrClean=!0;if(0<this.c.length){for(a=0;a<this.c.length;a++){b=this.c[a];var c=b[0];this.a[c.toString()]=new wa(c,b[1])}this.arrClean=!0}}g(\"jspb.Map\",r,void 0);\nr.prototype.g=function(){if(this.arrClean){if(this.b){var a=this.a,b;for(b in a)if(Object.prototype.hasOwnProperty.call(a,b)){var c=a[b].a;c&&c.g()}}}else{this.c.length=0;a=u(this);a.sort();for(b=0;b<a.length;b++){var d=this.a[a[b]];(c=d.a)&&c.g();this.c.push([d.key,d.value])}this.arrClean=!0}return this.c};r.prototype.toArray=r.prototype.g;\nr.prototype.Mc=function(a,b){for(var c=this.g(),d=[],f=0;f<c.length;f++){var h=this.a[c[f][0].toString()];v(this,h);var m=h.a;m?(n(b),d.push([h.key,b(a,m)])):d.push([h.key,h.value])}return d};r.prototype.toObject=r.prototype.Mc;r.fromObject=function(a,b,c){b=new r([],b);for(var d=0;d<a.length;d++){var f=a[d][0],h=c(a[d][1]);b.set(f,h)}return b};function w(a){this.a=0;this.b=a}w.prototype.next=function(){return this.a<this.b.length?{done:!1,value:this.b[this.a++]}:{done:!0,value:void 0}};\n\"undefined\"!=typeof Symbol&&(w.prototype[Symbol.iterator]=function(){return this});r.prototype.Jb=function(){return u(this).length};r.prototype.getLength=r.prototype.Jb;r.prototype.clear=function(){this.a={};this.arrClean=!1};r.prototype.clear=r.prototype.clear;r.prototype.Cb=function(a){a=a.toString();var b=this.a.hasOwnProperty(a);delete this.a[a];this.arrClean=!1;return b};r.prototype.del=r.prototype.Cb;\nr.prototype.Eb=function(){var a=[],b=u(this);b.sort();for(var c=0;c<b.length;c++){var d=this.a[b[c]];a.push([d.key,d.value])}return a};r.prototype.getEntryList=r.prototype.Eb;r.prototype.entries=function(){var a=[],b=u(this);b.sort();for(var c=0;c<b.length;c++){var d=this.a[b[c]];a.push([d.key,v(this,d)])}return new w(a)};r.prototype.entries=r.prototype.entries;r.prototype.keys=function(){var a=[],b=u(this);b.sort();for(var c=0;c<b.length;c++)a.push(this.a[b[c]].key);return new w(a)};\nr.prototype.keys=r.prototype.keys;r.prototype.values=function(){var a=[],b=u(this);b.sort();for(var c=0;c<b.length;c++)a.push(v(this,this.a[b[c]]));return new w(a)};r.prototype.values=r.prototype.values;r.prototype.forEach=function(a,b){var c=u(this);c.sort();for(var d=0;d<c.length;d++){var f=this.a[c[d]];a.call(b,v(this,f),f.key,this)}};r.prototype.forEach=r.prototype.forEach;\nr.prototype.set=function(a,b){var c=new wa(a);this.b?(c.a=b,c.value=b.g()):c.value=b;this.a[a.toString()]=c;this.arrClean=!1;return this};r.prototype.set=r.prototype.set;function v(a,b){return a.b?(b.a||(b.a=new a.b(b.value)),b.a):b.value}r.prototype.get=function(a){if(a=this.a[a.toString()])return v(this,a)};r.prototype.get=r.prototype.get;r.prototype.has=function(a){return a.toString()in this.a};r.prototype.has=r.prototype.has;\nr.prototype.Jc=function(a,b,c,d,f){var h=u(this);h.sort();for(var m=0;m<h.length;m++){var t=this.a[h[m]];b.Va(a);c.call(b,1,t.key);this.b?d.call(b,2,v(this,t),f):d.call(b,2,t.value);b.Ya()}};r.prototype.serializeBinary=r.prototype.Jc;r.deserializeBinary=function(a,b,c,d,f,h,m){for(;b.oa()&&!b.bb();){var t=b.c;1==t?h=c.call(b):2==t&&(a.b?(n(f),m||(m=new a.b),d.call(b,m,f)):m=d.call(b))}n(void 0!=h);n(void 0!=m);a.set(h,m)};\nfunction u(a){a=a.a;var b=[],c;for(c in a)Object.prototype.hasOwnProperty.call(a,c)&&b.push(c);return b}function wa(a,b){this.key=a;this.value=b;this.a=void 0};function xa(a){if(8192>=a.length)return String.fromCharCode.apply(null,a);for(var b=\"\",c=0;c<a.length;c+=8192)b+=String.fromCharCode.apply(null,ra(a,c,c+8192));return b};var ya={\"\\x00\":\"\\\\0\",\"\\b\":\"\\\\b\",\"\\f\":\"\\\\f\",\"\\n\":\"\\\\n\",\"\\r\":\"\\\\r\",\"\\t\":\"\\\\t\",\"\\x0B\":\"\\\\x0B\",'\"':'\\\\\"',\"\\\\\":\"\\\\\\\\\",\"<\":\"\\\\u003C\"},za={\"'\":\"\\\\'\"};var Aa={},x=null;function Ba(a,b){void 0===b&&(b=0);Ca();b=Aa[b];for(var c=[],d=0;d<a.length;d+=3){var f=a[d],h=d+1<a.length,m=h?a[d+1]:0,t=d+2<a.length,B=t?a[d+2]:0,M=f>>2;f=(f&3)<<4|m>>4;m=(m&15)<<2|B>>6;B&=63;t||(B=64,h||(m=64));c.push(b[M],b[f],b[m]||\"\",b[B]||\"\")}return c.join(\"\")}function Da(a){var b=a.length,c=3*b/4;c%3?c=Math.floor(c):-1!=\"=.\".indexOf(a[b-1])&&(c=-1!=\"=.\".indexOf(a[b-2])?c-2:c-1);var d=new Uint8Array(c),f=0;Ea(a,function(h){d[f++]=h});return d.subarray(0,f)}\nfunction Ea(a,b){function c(B){for(;d<a.length;){var M=a.charAt(d++),La=x[M];if(null!=La)return La;if(!/^[\\s\\xa0]*$/.test(M))throw Error(\"Unknown base64 encoding at char: \"+M);}return B}Ca();for(var d=0;;){var f=c(-1),h=c(0),m=c(64),t=c(64);if(64===t&&-1===f)break;b(f<<2|h>>4);64!=m&&(b(h<<4&240|m>>2),64!=t&&b(m<<6&192|t))}}\nfunction Ca(){if(!x){x={};for(var a=\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\".split(\"\"),b=[\"+/=\",\"+/\",\"-_=\",\"-_.\",\"-_\"],c=0;5>c;c++){var d=a.concat(b[c].split(\"\"));Aa[c]=d;for(var f=0;f<d.length;f++){var h=d[f];void 0===x[h]&&(x[h]=f)}}}};g(\"jspb.ConstBinaryMessage\",function(){},void 0);g(\"jspb.BinaryMessage\",function(){},void 0);g(\"jspb.BinaryConstants.FieldType\",{yb:-1,ee:1,FLOAT:2,ke:3,te:4,je:5,xb:6,wb:7,BOOL:8,re:9,ie:10,le:11,ce:12,se:13,ge:14,me:15,ne:16,oe:17,pe:18,he:30,ve:31},void 0);g(\"jspb.BinaryConstants.WireType\",{yb:-1,ue:0,xb:1,de:2,qe:3,fe:4,wb:5},void 0);\ng(\"jspb.BinaryConstants.FieldTypeToWireType\",function(a){switch(a){case 5:case 3:case 13:case 4:case 17:case 18:case 8:case 14:case 31:return 0;case 1:case 6:case 16:case 30:return 1;case 9:case 11:case 12:return 2;case 2:case 7:case 15:return 5;default:return-1}},void 0);g(\"jspb.BinaryConstants.INVALID_FIELD_NUMBER\",-1,void 0);g(\"jspb.BinaryConstants.FLOAT32_EPS\",1.401298464324817E-45,void 0);g(\"jspb.BinaryConstants.FLOAT32_MIN\",1.1754943508222875E-38,void 0);\ng(\"jspb.BinaryConstants.FLOAT32_MAX\",3.4028234663852886E38,void 0);g(\"jspb.BinaryConstants.FLOAT64_EPS\",4.9E-324,void 0);g(\"jspb.BinaryConstants.FLOAT64_MIN\",2.2250738585072014E-308,void 0);g(\"jspb.BinaryConstants.FLOAT64_MAX\",1.7976931348623157E308,void 0);g(\"jspb.BinaryConstants.TWO_TO_20\",1048576,void 0);g(\"jspb.BinaryConstants.TWO_TO_23\",8388608,void 0);g(\"jspb.BinaryConstants.TWO_TO_31\",2147483648,void 0);g(\"jspb.BinaryConstants.TWO_TO_32\",4294967296,void 0);\ng(\"jspb.BinaryConstants.TWO_TO_52\",4503599627370496,void 0);g(\"jspb.BinaryConstants.TWO_TO_63\",0x7fffffffffffffff,void 0);g(\"jspb.BinaryConstants.TWO_TO_64\",1.8446744073709552E19,void 0);g(\"jspb.BinaryConstants.ZERO_HASH\",\"\\x00\\x00\\x00\\x00\\x00\\x00\\x00\\x00\",void 0);var y=0,z=0;g(\"jspb.utils.getSplit64Low\",function(){return y},void 0);g(\"jspb.utils.getSplit64High\",function(){return z},void 0);function Fa(a){var b=a>>>0;a=Math.floor((a-b)/4294967296)>>>0;y=b;z=a}g(\"jspb.utils.splitUint64\",Fa,void 0);function A(a){var b=0>a;a=Math.abs(a);var c=a>>>0;a=Math.floor((a-c)/4294967296);a>>>=0;b&&(a=~a>>>0,c=(~c>>>0)+1,4294967295<c&&(c=0,a++,4294967295<a&&(a=0)));y=c;z=a}g(\"jspb.utils.splitInt64\",A,void 0);\nfunction Ga(a){var b=0>a;a=2*Math.abs(a);Fa(a);a=y;var c=z;b&&(0==a?0==c?c=a=4294967295:(c--,a=4294967295):a--);y=a;z=c}g(\"jspb.utils.splitZigzag64\",Ga,void 0);\nfunction Ha(a){var b=0>a?1:0;a=b?-a:a;if(0===a)0<1/a?y=z=0:(z=0,y=2147483648);else if(isNaN(a))z=0,y=2147483647;else if(3.4028234663852886E38<a)z=0,y=(b<<31|2139095040)>>>0;else if(1.1754943508222875E-38>a)a=Math.round(a/Math.pow(2,-149)),z=0,y=(b<<31|a)>>>0;else{var c=Math.floor(Math.log(a)/Math.LN2);a*=Math.pow(2,-c);a=Math.round(8388608*a);16777216<=a&&++c;z=0;y=(b<<31|c+127<<23|a&8388607)>>>0}}g(\"jspb.utils.splitFloat32\",Ha,void 0);\nfunction Ia(a){var b=0>a?1:0;a=b?-a:a;if(0===a)z=0<1/a?0:2147483648,y=0;else if(isNaN(a))z=2147483647,y=4294967295;else if(1.7976931348623157E308<a)z=(b<<31|2146435072)>>>0,y=0;else if(2.2250738585072014E-308>a)a/=Math.pow(2,-1074),z=(b<<31|a/4294967296)>>>0,y=a>>>0;else{var c=a,d=0;if(2<=c)for(;2<=c&&1023>d;)d++,c/=2;else for(;1>c&&-1022<d;)c*=2,d--;a*=Math.pow(2,-d);z=(b<<31|d+1023<<20|1048576*a&1048575)>>>0;y=4503599627370496*a>>>0}}g(\"jspb.utils.splitFloat64\",Ia,void 0);\nfunction C(a){var b=a.charCodeAt(4),c=a.charCodeAt(5),d=a.charCodeAt(6),f=a.charCodeAt(7);y=a.charCodeAt(0)+(a.charCodeAt(1)<<8)+(a.charCodeAt(2)<<16)+(a.charCodeAt(3)<<24)>>>0;z=b+(c<<8)+(d<<16)+(f<<24)>>>0}g(\"jspb.utils.splitHash64\",C,void 0);function D(a,b){return 4294967296*b+(a>>>0)}g(\"jspb.utils.joinUint64\",D,void 0);function E(a,b){var c=b&2147483648;c&&(a=~a+1>>>0,b=~b>>>0,0==a&&(b=b+1>>>0));a=D(a,b);return c?-a:a}g(\"jspb.utils.joinInt64\",E,void 0);\nfunction Ja(a,b,c){var d=b>>31;return c(a<<1^d,(b<<1|a>>>31)^d)}g(\"jspb.utils.toZigzag64\",Ja,void 0);function Ka(a,b){return Ma(a,b,E)}g(\"jspb.utils.joinZigzag64\",Ka,void 0);function Ma(a,b,c){var d=-(a&1);return c((a>>>1|b<<31)^d,b>>>1^d)}g(\"jspb.utils.fromZigzag64\",Ma,void 0);function Na(a){var b=2*(a>>31)+1,c=a>>>23&255;a&=8388607;return 255==c?a?NaN:Infinity*b:0==c?b*Math.pow(2,-149)*a:b*Math.pow(2,c-150)*(a+Math.pow(2,23))}g(\"jspb.utils.joinFloat32\",Na,void 0);\nfunction Oa(a,b){var c=2*(b>>31)+1,d=b>>>20&2047;a=4294967296*(b&1048575)+a;return 2047==d?a?NaN:Infinity*c:0==d?c*Math.pow(2,-1074)*a:c*Math.pow(2,d-1075)*(a+4503599627370496)}g(\"jspb.utils.joinFloat64\",Oa,void 0);function Pa(a,b){return String.fromCharCode(a>>>0&255,a>>>8&255,a>>>16&255,a>>>24&255,b>>>0&255,b>>>8&255,b>>>16&255,b>>>24&255)}g(\"jspb.utils.joinHash64\",Pa,void 0);g(\"jspb.utils.DIGITS\",\"0123456789abcdef\".split(\"\"),void 0);\nfunction F(a,b){function c(f,h){f=f?String(f):\"\";return h?\"0000000\".slice(f.length)+f:f}if(2097151>=b)return\"\"+D(a,b);var d=(a>>>24|b<<8)>>>0&16777215;b=b>>16&65535;a=(a&16777215)+6777216*d+6710656*b;d+=8147497*b;b*=2;1E7<=a&&(d+=Math.floor(a/1E7),a%=1E7);1E7<=d&&(b+=Math.floor(d/1E7),d%=1E7);return c(b,0)+c(d,b)+c(a,1)}g(\"jspb.utils.joinUnsignedDecimalString\",F,void 0);function G(a,b){var c=b&2147483648;c&&(a=~a+1>>>0,b=~b+(0==a?1:0)>>>0);a=F(a,b);return c?\"-\"+a:a}\ng(\"jspb.utils.joinSignedDecimalString\",G,void 0);function Qa(a,b){C(a);a=y;var c=z;return b?G(a,c):F(a,c)}g(\"jspb.utils.hash64ToDecimalString\",Qa,void 0);g(\"jspb.utils.hash64ArrayToDecimalStrings\",function(a,b){for(var c=Array(a.length),d=0;d<a.length;d++)c[d]=Qa(a[d],b);return c},void 0);\nfunction H(a){function b(m,t){for(var B=0;8>B&&(1!==m||0<t);B++)t=m*f[B]+t,f[B]=t&255,t>>>=8}function c(){for(var m=0;8>m;m++)f[m]=~f[m]&255}n(0<a.length);var d=!1;\"-\"===a[0]&&(d=!0,a=a.slice(1));for(var f=[0,0,0,0,0,0,0,0],h=0;h<a.length;h++)b(10,a.charCodeAt(h)-48);d&&(c(),b(1,1));return xa(f)}g(\"jspb.utils.decimalStringToHash64\",H,void 0);g(\"jspb.utils.splitDecimalString\",function(a){C(H(a))},void 0);function Ra(a){return String.fromCharCode(10>a?48+a:87+a)}\nfunction Sa(a){return 97<=a?a-97+10:a-48}g(\"jspb.utils.hash64ToHexString\",function(a){var b=Array(18);b[0]=\"0\";b[1]=\"x\";for(var c=0;8>c;c++){var d=a.charCodeAt(7-c);b[2*c+2]=Ra(d>>4);b[2*c+3]=Ra(d&15)}return b.join(\"\")},void 0);g(\"jspb.utils.hexStringToHash64\",function(a){a=a.toLowerCase();n(18==a.length);n(\"0\"==a[0]);n(\"x\"==a[1]);for(var b=\"\",c=0;8>c;c++)b=String.fromCharCode(16*Sa(a.charCodeAt(2*c+2))+Sa(a.charCodeAt(2*c+3)))+b;return b},void 0);\ng(\"jspb.utils.hash64ToNumber\",function(a,b){C(a);a=y;var c=z;return b?E(a,c):D(a,c)},void 0);g(\"jspb.utils.numberToHash64\",function(a){A(a);return Pa(y,z)},void 0);g(\"jspb.utils.countVarints\",function(a,b,c){for(var d=0,f=b;f<c;f++)d+=a[f]>>7;return c-b-d},void 0);\ng(\"jspb.utils.countVarintFields\",function(a,b,c,d){var f=0;d*=8;if(128>d)for(;b<c&&a[b++]==d;)for(f++;;){var h=a[b++];if(0==(h&128))break}else for(;b<c;){for(h=d;128<h;){if(a[b]!=(h&127|128))return f;b++;h>>=7}if(a[b++]!=h)break;for(f++;h=a[b++],0!=(h&128););}return f},void 0);function Ta(a,b,c,d,f){var h=0;if(128>d)for(;b<c&&a[b++]==d;)h++,b+=f;else for(;b<c;){for(var m=d;128<m;){if(a[b++]!=(m&127|128))return h;m>>=7}if(a[b++]!=m)break;h++;b+=f}return h}\ng(\"jspb.utils.countFixed32Fields\",function(a,b,c,d){return Ta(a,b,c,8*d+5,4)},void 0);g(\"jspb.utils.countFixed64Fields\",function(a,b,c,d){return Ta(a,b,c,8*d+1,8)},void 0);g(\"jspb.utils.countDelimitedFields\",function(a,b,c,d){var f=0;for(d=8*d+2;b<c;){for(var h=d;128<h;){if(a[b++]!=(h&127|128))return f;h>>=7}if(a[b++]!=h)break;f++;for(var m=0,t=1;h=a[b++],m+=(h&127)*t,t*=128,0!=(h&128););b+=m}return f},void 0);\ng(\"jspb.utils.debugBytesToTextFormat\",function(a){var b='\"';if(a){a=Ua(a);for(var c=0;c<a.length;c++)b+=\"\\\\x\",16>a[c]&&(b+=\"0\"),b+=a[c].toString(16)}return b+'\"'},void 0);\ng(\"jspb.utils.debugScalarToTextFormat\",function(a){if(\"string\"===typeof a){a=String(a);for(var b=['\"'],c=0;c<a.length;c++){var d=a.charAt(c),f=d.charCodeAt(0),h=c+1,m;if(!(m=ya[d])){if(!(31<f&&127>f))if(f=d,f in za)d=za[f];else if(f in ya)d=za[f]=ya[f];else{m=f.charCodeAt(0);if(31<m&&127>m)d=f;else{if(256>m){if(d=\"\\\\x\",16>m||256<m)d+=\"0\"}else d=\"\\\\u\",4096>m&&(d+=\"0\");d+=m.toString(16).toUpperCase()}d=za[f]=d}m=d}b[h]=m}b.push('\"');a=b.join(\"\")}else a=a.toString();return a},void 0);\ng(\"jspb.utils.stringToByteArray\",function(a){for(var b=new Uint8Array(a.length),c=0;c<a.length;c++){var d=a.charCodeAt(c);if(255<d)throw Error(\"Conversion error: string contains codepoint outside of byte range\");b[c]=d}return b},void 0);\nfunction Ua(a){if(a.constructor===Uint8Array)return a;if(a.constructor===ArrayBuffer)return new Uint8Array(a);if(a.constructor===Array)return new Uint8Array(a);if(a.constructor===String)return Da(a);if(a instanceof Uint8Array)return new Uint8Array(a.buffer,a.byteOffset,a.byteLength);p(\"Type not convertible to Uint8Array.\");return new Uint8Array(0)}g(\"jspb.utils.byteSourceToUint8Array\",Ua,void 0);function I(a,b,c){this.b=null;this.a=this.c=this.h=0;this.v=!1;a&&this.H(a,b,c)}g(\"jspb.BinaryDecoder\",I,void 0);var Va=[];I.getInstanceCacheLength=function(){return Va.length};function Wa(a,b,c){if(Va.length){var d=Va.pop();a&&d.H(a,b,c);return d}return new I(a,b,c)}I.alloc=Wa;I.prototype.Ca=function(){this.clear();100>Va.length&&Va.push(this)};I.prototype.free=I.prototype.Ca;I.prototype.clone=function(){return Wa(this.b,this.h,this.c-this.h)};I.prototype.clone=I.prototype.clone;\nI.prototype.clear=function(){this.b=null;this.a=this.c=this.h=0;this.v=!1};I.prototype.clear=I.prototype.clear;I.prototype.Y=function(){return this.b};I.prototype.getBuffer=I.prototype.Y;I.prototype.H=function(a,b,c){this.b=Ua(a);this.h=void 0!==b?b:0;this.c=void 0!==c?this.h+c:this.b.length;this.a=this.h};I.prototype.setBlock=I.prototype.H;I.prototype.Db=function(){return this.c};I.prototype.getEnd=I.prototype.Db;I.prototype.setEnd=function(a){this.c=a};I.prototype.setEnd=I.prototype.setEnd;\nI.prototype.reset=function(){this.a=this.h};I.prototype.reset=I.prototype.reset;I.prototype.B=function(){return this.a};I.prototype.getCursor=I.prototype.B;I.prototype.Ma=function(a){this.a=a};I.prototype.setCursor=I.prototype.Ma;I.prototype.advance=function(a){this.a+=a;n(this.a<=this.c)};I.prototype.advance=I.prototype.advance;I.prototype.ya=function(){return this.a==this.c};I.prototype.atEnd=I.prototype.ya;I.prototype.Qb=function(){return this.a>this.c};I.prototype.pastEnd=I.prototype.Qb;\nI.prototype.getError=function(){return this.v||0>this.a||this.a>this.c};I.prototype.getError=I.prototype.getError;I.prototype.w=function(a){for(var b=128,c=0,d=0,f=0;4>f&&128<=b;f++)b=this.b[this.a++],c|=(b&127)<<7*f;128<=b&&(b=this.b[this.a++],c|=(b&127)<<28,d|=(b&127)>>4);if(128<=b)for(f=0;5>f&&128<=b;f++)b=this.b[this.a++],d|=(b&127)<<7*f+3;if(128>b)return a(c>>>0,d>>>0);p(\"Failed to read varint, encoding is invalid.\");this.v=!0};I.prototype.readSplitVarint64=I.prototype.w;\nI.prototype.ea=function(a){return this.w(function(b,c){return Ma(b,c,a)})};I.prototype.readSplitZigzagVarint64=I.prototype.ea;I.prototype.ta=function(a){var b=this.b,c=this.a;this.a+=8;for(var d=0,f=0,h=c+7;h>=c;h--)d=d<<8|b[h],f=f<<8|b[h+4];return a(d,f)};I.prototype.readSplitFixed64=I.prototype.ta;I.prototype.kb=function(){for(;this.b[this.a]&128;)this.a++;this.a++};I.prototype.skipVarint=I.prototype.kb;I.prototype.mb=function(a){for(;128<a;)this.a--,a>>>=7;this.a--};I.prototype.unskipVarint=I.prototype.mb;\nI.prototype.o=function(){var a=this.b;var b=a[this.a];var c=b&127;if(128>b)return this.a+=1,n(this.a<=this.c),c;b=a[this.a+1];c|=(b&127)<<7;if(128>b)return this.a+=2,n(this.a<=this.c),c;b=a[this.a+2];c|=(b&127)<<14;if(128>b)return this.a+=3,n(this.a<=this.c),c;b=a[this.a+3];c|=(b&127)<<21;if(128>b)return this.a+=4,n(this.a<=this.c),c;b=a[this.a+4];c|=(b&15)<<28;if(128>b)return this.a+=5,n(this.a<=this.c),c>>>0;this.a+=5;128<=a[this.a++]&&128<=a[this.a++]&&128<=a[this.a++]&&128<=a[this.a++]&&128<=\na[this.a++]&&n(!1);n(this.a<=this.c);return c};I.prototype.readUnsignedVarint32=I.prototype.o;I.prototype.da=function(){return~~this.o()};I.prototype.readSignedVarint32=I.prototype.da;I.prototype.O=function(){return this.o().toString()};I.prototype.Ea=function(){return this.da().toString()};I.prototype.readSignedVarint32String=I.prototype.Ea;I.prototype.Ia=function(){var a=this.o();return a>>>1^-(a&1)};I.prototype.readZigzagVarint32=I.prototype.Ia;I.prototype.Ga=function(){return this.w(D)};\nI.prototype.readUnsignedVarint64=I.prototype.Ga;I.prototype.Ha=function(){return this.w(F)};I.prototype.readUnsignedVarint64String=I.prototype.Ha;I.prototype.sa=function(){return this.w(E)};I.prototype.readSignedVarint64=I.prototype.sa;I.prototype.Fa=function(){return this.w(G)};I.prototype.readSignedVarint64String=I.prototype.Fa;I.prototype.Ja=function(){return this.w(Ka)};I.prototype.readZigzagVarint64=I.prototype.Ja;I.prototype.fb=function(){return this.ea(Pa)};\nI.prototype.readZigzagVarintHash64=I.prototype.fb;I.prototype.Ka=function(){return this.ea(G)};I.prototype.readZigzagVarint64String=I.prototype.Ka;I.prototype.Gc=function(){var a=this.b[this.a];this.a+=1;n(this.a<=this.c);return a};I.prototype.readUint8=I.prototype.Gc;I.prototype.Ec=function(){var a=this.b[this.a],b=this.b[this.a+1];this.a+=2;n(this.a<=this.c);return a<<0|b<<8};I.prototype.readUint16=I.prototype.Ec;\nI.prototype.m=function(){var a=this.b[this.a],b=this.b[this.a+1],c=this.b[this.a+2],d=this.b[this.a+3];this.a+=4;n(this.a<=this.c);return(a<<0|b<<8|c<<16|d<<24)>>>0};I.prototype.readUint32=I.prototype.m;I.prototype.ga=function(){var a=this.m(),b=this.m();return D(a,b)};I.prototype.readUint64=I.prototype.ga;I.prototype.ha=function(){var a=this.m(),b=this.m();return F(a,b)};I.prototype.readUint64String=I.prototype.ha;\nI.prototype.Xb=function(){var a=this.b[this.a];this.a+=1;n(this.a<=this.c);return a<<24>>24};I.prototype.readInt8=I.prototype.Xb;I.prototype.Vb=function(){var a=this.b[this.a],b=this.b[this.a+1];this.a+=2;n(this.a<=this.c);return(a<<0|b<<8)<<16>>16};I.prototype.readInt16=I.prototype.Vb;I.prototype.P=function(){var a=this.b[this.a],b=this.b[this.a+1],c=this.b[this.a+2],d=this.b[this.a+3];this.a+=4;n(this.a<=this.c);return a<<0|b<<8|c<<16|d<<24};I.prototype.readInt32=I.prototype.P;\nI.prototype.ba=function(){var a=this.m(),b=this.m();return E(a,b)};I.prototype.readInt64=I.prototype.ba;I.prototype.ca=function(){var a=this.m(),b=this.m();return G(a,b)};I.prototype.readInt64String=I.prototype.ca;I.prototype.aa=function(){var a=this.m();return Na(a,0)};I.prototype.readFloat=I.prototype.aa;I.prototype.Z=function(){var a=this.m(),b=this.m();return Oa(a,b)};I.prototype.readDouble=I.prototype.Z;I.prototype.pa=function(){return!!this.b[this.a++]};I.prototype.readBool=I.prototype.pa;\nI.prototype.ra=function(){return this.da()};I.prototype.readEnum=I.prototype.ra;\nI.prototype.fa=function(a){var b=this.b,c=this.a;a=c+a;for(var d=[],f=\"\";c<a;){var h=b[c++];if(128>h)d.push(h);else if(192>h)continue;else if(224>h){var m=b[c++];d.push((h&31)<<6|m&63)}else if(240>h){m=b[c++];var t=b[c++];d.push((h&15)<<12|(m&63)<<6|t&63)}else if(248>h){m=b[c++];t=b[c++];var B=b[c++];h=(h&7)<<18|(m&63)<<12|(t&63)<<6|B&63;h-=65536;d.push((h>>10&1023)+55296,(h&1023)+56320)}8192<=d.length&&(f+=String.fromCharCode.apply(null,d),d.length=0)}f+=xa(d);this.a=c;return f};\nI.prototype.readString=I.prototype.fa;I.prototype.Dc=function(){var a=this.o();return this.fa(a)};I.prototype.readStringWithLength=I.prototype.Dc;I.prototype.qa=function(a){if(0>a||this.a+a>this.b.length)return this.v=!0,p(\"Invalid byte length!\"),new Uint8Array(0);var b=this.b.subarray(this.a,this.a+a);this.a+=a;n(this.a<=this.c);return b};I.prototype.readBytes=I.prototype.qa;I.prototype.ia=function(){return this.w(Pa)};I.prototype.readVarintHash64=I.prototype.ia;\nI.prototype.$=function(){var a=this.b,b=this.a,c=a[b],d=a[b+1],f=a[b+2],h=a[b+3],m=a[b+4],t=a[b+5],B=a[b+6];a=a[b+7];this.a+=8;return String.fromCharCode(c,d,f,h,m,t,B,a)};I.prototype.readFixedHash64=I.prototype.$;function J(a,b,c){this.a=Wa(a,b,c);this.O=this.a.B();this.b=this.c=-1;this.h=!1;this.v=null}g(\"jspb.BinaryReader\",J,void 0);var K=[];J.clearInstanceCache=function(){K=[]};J.getInstanceCacheLength=function(){return K.length};function Xa(a,b,c){if(K.length){var d=K.pop();a&&d.a.H(a,b,c);return d}return new J(a,b,c)}J.alloc=Xa;J.prototype.zb=Xa;J.prototype.alloc=J.prototype.zb;J.prototype.Ca=function(){this.a.clear();this.b=this.c=-1;this.h=!1;this.v=null;100>K.length&&K.push(this)};\nJ.prototype.free=J.prototype.Ca;J.prototype.Fb=function(){return this.O};J.prototype.getFieldCursor=J.prototype.Fb;J.prototype.B=function(){return this.a.B()};J.prototype.getCursor=J.prototype.B;J.prototype.Y=function(){return this.a.Y()};J.prototype.getBuffer=J.prototype.Y;J.prototype.Hb=function(){return this.c};J.prototype.getFieldNumber=J.prototype.Hb;J.prototype.Lb=function(){return this.b};J.prototype.getWireType=J.prototype.Lb;J.prototype.Mb=function(){return 2==this.b};\nJ.prototype.isDelimited=J.prototype.Mb;J.prototype.bb=function(){return 4==this.b};J.prototype.isEndGroup=J.prototype.bb;J.prototype.getError=function(){return this.h||this.a.getError()};J.prototype.getError=J.prototype.getError;J.prototype.H=function(a,b,c){this.a.H(a,b,c);this.b=this.c=-1};J.prototype.setBlock=J.prototype.H;J.prototype.reset=function(){this.a.reset();this.b=this.c=-1};J.prototype.reset=J.prototype.reset;J.prototype.advance=function(a){this.a.advance(a)};J.prototype.advance=J.prototype.advance;\nJ.prototype.oa=function(){if(this.a.ya())return!1;if(this.getError())return p(\"Decoder hit an error\"),!1;this.O=this.a.B();var a=this.a.o(),b=a>>>3;a&=7;if(0!=a&&5!=a&&1!=a&&2!=a&&3!=a&&4!=a)return p(\"Invalid wire type: %s (at position %s)\",a,this.O),this.h=!0,!1;this.c=b;this.b=a;return!0};J.prototype.nextField=J.prototype.oa;J.prototype.Oa=function(){this.a.mb(this.c<<3|this.b)};J.prototype.unskipHeader=J.prototype.Oa;\nJ.prototype.Lc=function(){var a=this.c;for(this.Oa();this.oa()&&this.c==a;)this.C();this.a.ya()||this.Oa()};J.prototype.skipMatchingFields=J.prototype.Lc;J.prototype.lb=function(){0!=this.b?(p(\"Invalid wire type for skipVarintField\"),this.C()):this.a.kb()};J.prototype.skipVarintField=J.prototype.lb;J.prototype.gb=function(){if(2!=this.b)p(\"Invalid wire type for skipDelimitedField\"),this.C();else{var a=this.a.o();this.a.advance(a)}};J.prototype.skipDelimitedField=J.prototype.gb;\nJ.prototype.hb=function(){5!=this.b?(p(\"Invalid wire type for skipFixed32Field\"),this.C()):this.a.advance(4)};J.prototype.skipFixed32Field=J.prototype.hb;J.prototype.ib=function(){1!=this.b?(p(\"Invalid wire type for skipFixed64Field\"),this.C()):this.a.advance(8)};J.prototype.skipFixed64Field=J.prototype.ib;J.prototype.jb=function(){var a=this.c;do{if(!this.oa()){p(\"Unmatched start-group tag: stream EOF\");this.h=!0;break}if(4==this.b){this.c!=a&&(p(\"Unmatched end-group tag\"),this.h=!0);break}this.C()}while(1)};\nJ.prototype.skipGroup=J.prototype.jb;J.prototype.C=function(){switch(this.b){case 0:this.lb();break;case 1:this.ib();break;case 2:this.gb();break;case 5:this.hb();break;case 3:this.jb();break;default:p(\"Invalid wire encoding for field.\")}};J.prototype.skipField=J.prototype.C;J.prototype.Hc=function(a,b){null===this.v&&(this.v={});n(!this.v[a]);this.v[a]=b};J.prototype.registerReadCallback=J.prototype.Hc;J.prototype.Ic=function(a){n(null!==this.v);a=this.v[a];n(a);return a(this)};\nJ.prototype.runReadCallback=J.prototype.Ic;J.prototype.Yb=function(a,b){n(2==this.b);var c=this.a.c,d=this.a.o();d=this.a.B()+d;this.a.setEnd(d);b(a,this);this.a.Ma(d);this.a.setEnd(c)};J.prototype.readMessage=J.prototype.Yb;J.prototype.Ub=function(a,b,c){n(3==this.b);n(this.c==a);c(b,this);this.h||4==this.b||(p(\"Group submessage did not end with an END_GROUP tag\"),this.h=!0)};J.prototype.readGroup=J.prototype.Ub;\nJ.prototype.Gb=function(){n(2==this.b);var a=this.a.o(),b=this.a.B(),c=b+a;a=Wa(this.a.Y(),b,a);this.a.Ma(c);return a};J.prototype.getFieldDecoder=J.prototype.Gb;J.prototype.P=function(){n(0==this.b);return this.a.da()};J.prototype.readInt32=J.prototype.P;J.prototype.Wb=function(){n(0==this.b);return this.a.Ea()};J.prototype.readInt32String=J.prototype.Wb;J.prototype.ba=function(){n(0==this.b);return this.a.sa()};J.prototype.readInt64=J.prototype.ba;J.prototype.ca=function(){n(0==this.b);return this.a.Fa()};\nJ.prototype.readInt64String=J.prototype.ca;J.prototype.m=function(){n(0==this.b);return this.a.o()};J.prototype.readUint32=J.prototype.m;J.prototype.Fc=function(){n(0==this.b);return this.a.O()};J.prototype.readUint32String=J.prototype.Fc;J.prototype.ga=function(){n(0==this.b);return this.a.Ga()};J.prototype.readUint64=J.prototype.ga;J.prototype.ha=function(){n(0==this.b);return this.a.Ha()};J.prototype.readUint64String=J.prototype.ha;J.prototype.zc=function(){n(0==this.b);return this.a.Ia()};\nJ.prototype.readSint32=J.prototype.zc;J.prototype.Ac=function(){n(0==this.b);return this.a.Ja()};J.prototype.readSint64=J.prototype.Ac;J.prototype.Bc=function(){n(0==this.b);return this.a.Ka()};J.prototype.readSint64String=J.prototype.Bc;J.prototype.Rb=function(){n(5==this.b);return this.a.m()};J.prototype.readFixed32=J.prototype.Rb;J.prototype.Sb=function(){n(1==this.b);return this.a.ga()};J.prototype.readFixed64=J.prototype.Sb;J.prototype.Tb=function(){n(1==this.b);return this.a.ha()};\nJ.prototype.readFixed64String=J.prototype.Tb;J.prototype.vc=function(){n(5==this.b);return this.a.P()};J.prototype.readSfixed32=J.prototype.vc;J.prototype.wc=function(){n(5==this.b);return this.a.P().toString()};J.prototype.readSfixed32String=J.prototype.wc;J.prototype.xc=function(){n(1==this.b);return this.a.ba()};J.prototype.readSfixed64=J.prototype.xc;J.prototype.yc=function(){n(1==this.b);return this.a.ca()};J.prototype.readSfixed64String=J.prototype.yc;\nJ.prototype.aa=function(){n(5==this.b);return this.a.aa()};J.prototype.readFloat=J.prototype.aa;J.prototype.Z=function(){n(1==this.b);return this.a.Z()};J.prototype.readDouble=J.prototype.Z;J.prototype.pa=function(){n(0==this.b);return!!this.a.o()};J.prototype.readBool=J.prototype.pa;J.prototype.ra=function(){n(0==this.b);return this.a.sa()};J.prototype.readEnum=J.prototype.ra;J.prototype.fa=function(){n(2==this.b);var a=this.a.o();return this.a.fa(a)};J.prototype.readString=J.prototype.fa;\nJ.prototype.qa=function(){n(2==this.b);var a=this.a.o();return this.a.qa(a)};J.prototype.readBytes=J.prototype.qa;J.prototype.ia=function(){n(0==this.b);return this.a.ia()};J.prototype.readVarintHash64=J.prototype.ia;J.prototype.Cc=function(){n(0==this.b);return this.a.fb()};J.prototype.readSintHash64=J.prototype.Cc;J.prototype.w=function(a){n(0==this.b);return this.a.w(a)};J.prototype.readSplitVarint64=J.prototype.w;\nJ.prototype.ea=function(a){n(0==this.b);return this.a.w(function(b,c){return Ma(b,c,a)})};J.prototype.readSplitZigzagVarint64=J.prototype.ea;J.prototype.$=function(){n(1==this.b);return this.a.$()};J.prototype.readFixedHash64=J.prototype.$;J.prototype.ta=function(a){n(1==this.b);return this.a.ta(a)};J.prototype.readSplitFixed64=J.prototype.ta;function L(a,b){n(2==a.b);var c=a.a.o();c=a.a.B()+c;for(var d=[];a.a.B()<c;)d.push(b.call(a.a));return d}J.prototype.gc=function(){return L(this,this.a.da)};\nJ.prototype.readPackedInt32=J.prototype.gc;J.prototype.hc=function(){return L(this,this.a.Ea)};J.prototype.readPackedInt32String=J.prototype.hc;J.prototype.ic=function(){return L(this,this.a.sa)};J.prototype.readPackedInt64=J.prototype.ic;J.prototype.jc=function(){return L(this,this.a.Fa)};J.prototype.readPackedInt64String=J.prototype.jc;J.prototype.qc=function(){return L(this,this.a.o)};J.prototype.readPackedUint32=J.prototype.qc;J.prototype.rc=function(){return L(this,this.a.O)};\nJ.prototype.readPackedUint32String=J.prototype.rc;J.prototype.sc=function(){return L(this,this.a.Ga)};J.prototype.readPackedUint64=J.prototype.sc;J.prototype.tc=function(){return L(this,this.a.Ha)};J.prototype.readPackedUint64String=J.prototype.tc;J.prototype.nc=function(){return L(this,this.a.Ia)};J.prototype.readPackedSint32=J.prototype.nc;J.prototype.oc=function(){return L(this,this.a.Ja)};J.prototype.readPackedSint64=J.prototype.oc;J.prototype.pc=function(){return L(this,this.a.Ka)};\nJ.prototype.readPackedSint64String=J.prototype.pc;J.prototype.bc=function(){return L(this,this.a.m)};J.prototype.readPackedFixed32=J.prototype.bc;J.prototype.cc=function(){return L(this,this.a.ga)};J.prototype.readPackedFixed64=J.prototype.cc;J.prototype.dc=function(){return L(this,this.a.ha)};J.prototype.readPackedFixed64String=J.prototype.dc;J.prototype.kc=function(){return L(this,this.a.P)};J.prototype.readPackedSfixed32=J.prototype.kc;J.prototype.lc=function(){return L(this,this.a.ba)};\nJ.prototype.readPackedSfixed64=J.prototype.lc;J.prototype.mc=function(){return L(this,this.a.ca)};J.prototype.readPackedSfixed64String=J.prototype.mc;J.prototype.fc=function(){return L(this,this.a.aa)};J.prototype.readPackedFloat=J.prototype.fc;J.prototype.$b=function(){return L(this,this.a.Z)};J.prototype.readPackedDouble=J.prototype.$b;J.prototype.Zb=function(){return L(this,this.a.pa)};J.prototype.readPackedBool=J.prototype.Zb;J.prototype.ac=function(){return L(this,this.a.ra)};\nJ.prototype.readPackedEnum=J.prototype.ac;J.prototype.uc=function(){return L(this,this.a.ia)};J.prototype.readPackedVarintHash64=J.prototype.uc;J.prototype.ec=function(){return L(this,this.a.$)};J.prototype.readPackedFixedHash64=J.prototype.ec;function Ya(a,b,c,d,f){this.ma=a;this.Ba=b;this.la=c;this.Na=d;this.na=f}g(\"jspb.ExtensionFieldInfo\",Ya,void 0);function Za(a,b,c,d,f,h){this.Za=a;this.za=b;this.Aa=c;this.Wa=d;this.Ab=f;this.Nb=h}g(\"jspb.ExtensionFieldBinaryInfo\",Za,void 0);Ya.prototype.F=function(){return!!this.la};Ya.prototype.isMessageType=Ya.prototype.F;function N(){}g(\"jspb.Message\",N,void 0);N.GENERATE_TO_OBJECT=!0;N.GENERATE_FROM_OBJECT=!0;var $a=\"function\"==typeof Uint8Array;N.prototype.Ib=function(){return this.b};\nN.prototype.getJsPbMessageId=N.prototype.Ib;\nN.initialize=function(a,b,c,d,f,h){a.f=null;b||(b=c?[c]:[]);a.b=c?String(c):void 0;a.D=0===c?-1:0;a.u=b;a:{c=a.u.length;b=-1;if(c&&(b=c-1,c=a.u[b],!(null===c||\"object\"!=typeof c||Array.isArray(c)||$a&&c instanceof Uint8Array))){a.G=b-a.D;a.i=c;break a}-1<d?(a.G=Math.max(d,b+1-a.D),a.i=null):a.G=Number.MAX_VALUE}a.a={};if(f)for(d=0;d<f.length;d++)b=f[d],b<a.G?(b+=a.D,a.u[b]=a.u[b]||ab):(bb(a),a.i[b]=a.i[b]||ab);if(h&&h.length)for(d=0;d<h.length;d++)cb(a,h[d])};\nvar ab=Object.freeze?Object.freeze([]):[];function bb(a){var b=a.G+a.D;a.u[b]||(a.i=a.u[b]={})}function db(a,b,c){for(var d=[],f=0;f<a.length;f++)d[f]=b.call(a[f],c,a[f]);return d}N.toObjectList=db;N.toObjectExtension=function(a,b,c,d,f){for(var h in c){var m=c[h],t=d.call(a,m);if(null!=t){for(var B in m.Ba)if(m.Ba.hasOwnProperty(B))break;b[B]=m.Na?m.na?db(t,m.Na,f):m.Na(f,t):t}}};\nN.serializeBinaryExtensions=function(a,b,c,d){for(var f in c){var h=c[f],m=h.Za;if(!h.Aa)throw Error(\"Message extension present that was generated without binary serialization support\");var t=d.call(a,m);if(null!=t)if(m.F())if(h.Wa)h.Aa.call(b,m.ma,t,h.Wa);else throw Error(\"Message extension present holding submessage without binary support enabled, and message is being serialized to binary format\");else h.Aa.call(b,m.ma,t)}};\nN.readBinaryExtension=function(a,b,c,d,f){var h=c[b.c];if(h){c=h.Za;if(!h.za)throw Error(\"Deserializing extension whose generated code does not support binary format\");if(c.F()){var m=new c.la;h.za.call(b,m,h.Ab)}else m=h.za.call(b);c.na&&!h.Nb?(b=d.call(a,c))?b.push(m):f.call(a,c,[m]):f.call(a,c,m)}else b.C()};function O(a,b){if(b<a.G){b+=a.D;var c=a.u[b];return c===ab?a.u[b]=[]:c}if(a.i)return c=a.i[b],c===ab?a.i[b]=[]:c}N.getField=O;N.getRepeatedField=function(a,b){return O(a,b)};\nfunction eb(a,b){a=O(a,b);return null==a?a:+a}N.getOptionalFloatingPointField=eb;function fb(a,b){a=O(a,b);return null==a?a:!!a}N.getBooleanField=fb;N.getRepeatedFloatingPointField=function(a,b){var c=O(a,b);a.a||(a.a={});if(!a.a[b]){for(var d=0;d<c.length;d++)c[d]=+c[d];a.a[b]=!0}return c};N.getRepeatedBooleanField=function(a,b){var c=O(a,b);a.a||(a.a={});if(!a.a[b]){for(var d=0;d<c.length;d++)c[d]=!!c[d];a.a[b]=!0}return c};\nfunction gb(a){if(null==a||\"string\"===typeof a)return a;if($a&&a instanceof Uint8Array)return Ba(a);p(\"Cannot coerce to b64 string: \"+k(a));return null}N.bytesAsB64=gb;function hb(a){if(null==a||a instanceof Uint8Array)return a;if(\"string\"===typeof a)return Da(a);p(\"Cannot coerce to Uint8Array: \"+k(a));return null}N.bytesAsU8=hb;N.bytesListAsB64=function(a){ib(a);return a.length&&\"string\"!==typeof a[0]?l(a,gb):a};N.bytesListAsU8=function(a){ib(a);return!a.length||a[0]instanceof Uint8Array?a:l(a,hb)};\nfunction ib(a){if(a&&1<a.length){var b=k(a[0]);qa(a,function(c){k(c)!=b&&p(\"Inconsistent type in JSPB repeated field array. Got \"+k(c)+\" expected \"+b)})}}function jb(a,b,c){a=O(a,b);return null==a?c:a}N.getFieldWithDefault=jb;N.getBooleanFieldWithDefault=function(a,b,c){a=fb(a,b);return null==a?c:a};N.getFloatingPointFieldWithDefault=function(a,b,c){a=eb(a,b);return null==a?c:a};N.getFieldProto3=jb;\nN.getMapField=function(a,b,c,d){a.f||(a.f={});if(b in a.f)return a.f[b];var f=O(a,b);if(!f){if(c)return;f=[];P(a,b,f)}return a.f[b]=new r(f,d)};function P(a,b,c){q(a,N);b<a.G?a.u[b+a.D]=c:(bb(a),a.i[b]=c);return a}N.setField=P;N.setProto3IntField=function(a,b,c){return Q(a,b,c,0)};N.setProto3FloatField=function(a,b,c){return Q(a,b,c,0)};N.setProto3BooleanField=function(a,b,c){return Q(a,b,c,!1)};N.setProto3StringField=function(a,b,c){return Q(a,b,c,\"\")};\nN.setProto3BytesField=function(a,b,c){return Q(a,b,c,\"\")};N.setProto3EnumField=function(a,b,c){return Q(a,b,c,0)};N.setProto3StringIntField=function(a,b,c){return Q(a,b,c,\"0\")};function Q(a,b,c,d){q(a,N);c!==d?P(a,b,c):b<a.G?a.u[b+a.D]=null:(bb(a),delete a.i[b]);return a}N.addToRepeatedField=function(a,b,c,d){q(a,N);b=O(a,b);void 0!=d?b.splice(d,0,c):b.push(c);return a};function kb(a,b,c,d){q(a,N);(c=cb(a,c))&&c!==b&&void 0!==d&&(a.f&&c in a.f&&(a.f[c]=void 0),P(a,c,void 0));return P(a,b,d)}\nN.setOneofField=kb;function cb(a,b){for(var c,d,f=0;f<b.length;f++){var h=b[f],m=O(a,h);null!=m&&(c=h,d=m,P(a,h,void 0))}return c?(P(a,c,d),c):0}N.computeOneofCase=cb;N.getWrapperField=function(a,b,c,d){a.f||(a.f={});if(!a.f[c]){var f=O(a,c);if(d||f)a.f[c]=new b(f)}return a.f[c]};N.getRepeatedWrapperField=function(a,b,c){lb(a,b,c);b=a.f[c];b==ab&&(b=a.f[c]=[]);return b};function lb(a,b,c){a.f||(a.f={});if(!a.f[c]){for(var d=O(a,c),f=[],h=0;h<d.length;h++)f[h]=new b(d[h]);a.f[c]=f}}\nN.setWrapperField=function(a,b,c){q(a,N);a.f||(a.f={});var d=c?c.g():c;a.f[b]=c;return P(a,b,d)};N.setOneofWrapperField=function(a,b,c,d){q(a,N);a.f||(a.f={});var f=d?d.g():d;a.f[b]=d;return kb(a,b,c,f)};N.setRepeatedWrapperField=function(a,b,c){q(a,N);a.f||(a.f={});c=c||[];for(var d=[],f=0;f<c.length;f++)d[f]=c[f].g();a.f[b]=c;return P(a,b,d)};\nN.addToRepeatedWrapperField=function(a,b,c,d,f){lb(a,d,b);var h=a.f[b];h||(h=a.f[b]=[]);c=c?c:new d;a=O(a,b);void 0!=f?(h.splice(f,0,c),a.splice(f,0,c.g())):(h.push(c),a.push(c.g()));return c};N.toMap=function(a,b,c,d){for(var f={},h=0;h<a.length;h++)f[b.call(a[h])]=c?c.call(a[h],d,a[h]):a[h];return f};function mb(a){if(a.f)for(var b in a.f){var c=a.f[b];if(Array.isArray(c))for(var d=0;d<c.length;d++)c[d]&&c[d].g();else c&&c.g()}}N.prototype.g=function(){mb(this);return this.u};\nN.prototype.toArray=N.prototype.g;N.prototype.toString=function(){mb(this);return this.u.toString()};N.prototype.getExtension=function(a){if(this.i){this.f||(this.f={});var b=a.ma;if(a.na){if(a.F())return this.f[b]||(this.f[b]=l(this.i[b]||[],function(c){return new a.la(c)})),this.f[b]}else if(a.F())return!this.f[b]&&this.i[b]&&(this.f[b]=new a.la(this.i[b])),this.f[b];return this.i[b]}};N.prototype.getExtension=N.prototype.getExtension;\nN.prototype.Kc=function(a,b){this.f||(this.f={});bb(this);var c=a.ma;a.na?(b=b||[],a.F()?(this.f[c]=b,this.i[c]=l(b,function(d){return d.g()})):this.i[c]=b):a.F()?(this.f[c]=b,this.i[c]=b?b.g():b):this.i[c]=b;return this};N.prototype.setExtension=N.prototype.Kc;N.difference=function(a,b){if(!(a instanceof b.constructor))throw Error(\"Messages have different types.\");var c=a.g();b=b.g();var d=[],f=0,h=c.length>b.length?c.length:b.length;a.b&&(d[0]=a.b,f=1);for(;f<h;f++)nb(c[f],b[f])||(d[f]=b[f]);return new a.constructor(d)};\nN.equals=function(a,b){return a==b||!(!a||!b)&&a instanceof b.constructor&&nb(a.g(),b.g())};function ob(a,b){a=a||{};b=b||{};var c={},d;for(d in a)c[d]=0;for(d in b)c[d]=0;for(d in c)if(!nb(a[d],b[d]))return!1;return!0}N.compareExtensions=ob;\nfunction nb(a,b){if(a==b)return!0;if(!la(a)||!la(b))return\"number\"===typeof a&&isNaN(a)||\"number\"===typeof b&&isNaN(b)?String(a)==String(b):!1;if(a.constructor!=b.constructor)return!1;if($a&&a.constructor===Uint8Array){if(a.length!=b.length)return!1;for(var c=0;c<a.length;c++)if(a[c]!=b[c])return!1;return!0}if(a.constructor===Array){var d=void 0,f=void 0,h=Math.max(a.length,b.length);for(c=0;c<h;c++){var m=a[c],t=b[c];m&&m.constructor==Object&&(n(void 0===d),n(c===a.length-1),d=m,m=void 0);t&&t.constructor==\nObject&&(n(void 0===f),n(c===b.length-1),f=t,t=void 0);if(!nb(m,t))return!1}return d||f?(d=d||{},f=f||{},ob(d,f)):!0}if(a.constructor===Object)return ob(a,b);throw Error(\"Invalid type in JSPB array\");}N.compareFields=nb;N.prototype.Bb=function(){return pb(this)};N.prototype.cloneMessage=N.prototype.Bb;N.prototype.clone=function(){return pb(this)};N.prototype.clone=N.prototype.clone;N.clone=function(a){return pb(a)};function pb(a){return new a.constructor(qb(a.g()))}\nN.copyInto=function(a,b){q(a,N);q(b,N);n(a.constructor==b.constructor,\"Copy source and target message should have the same type.\");a=pb(a);for(var c=b.g(),d=a.g(),f=c.length=0;f<d.length;f++)c[f]=d[f];b.f=a.f;b.i=a.i};function qb(a){if(Array.isArray(a)){for(var b=Array(a.length),c=0;c<a.length;c++){var d=a[c];null!=d&&(b[c]=\"object\"==typeof d?qb(n(d)):d)}return b}if($a&&a instanceof Uint8Array)return new Uint8Array(a);b={};for(c in a)d=a[c],null!=d&&(b[c]=\"object\"==typeof d?qb(n(d)):d);return b}\nN.registerMessageType=function(a,b){b.we=a};var R={dump:function(a){q(a,N,\"jspb.Message instance expected\");n(a.getExtension,\"Only unobfuscated and unoptimized compilation modes supported.\");return R.X(a)}};g(\"jspb.debug.dump\",R.dump,void 0);\nR.X=function(a){var b=k(a);if(\"number\"==b||\"string\"==b||\"boolean\"==b||\"null\"==b||\"undefined\"==b||\"undefined\"!==typeof Uint8Array&&a instanceof Uint8Array)return a;if(\"array\"==b)return ua(a),l(a,R.X);if(a instanceof r){var c={};a=a.entries();for(var d=a.next();!d.done;d=a.next())c[d.value[0]]=R.X(d.value[1]);return c}q(a,N,\"Only messages expected: \"+a);b=a.constructor;var f={$name:b.name||b.displayName};for(t in b.prototype){var h=/^get([A-Z]\\w*)/.exec(t);if(h&&\"getExtension\"!=t&&\"getJsPbMessageId\"!=\nt){var m=\"has\"+h[1];if(!a[m]||a[m]())m=a[t](),f[R.$a(h[1])]=R.X(m)}}if(a.extensionObject_)return f.$extensions=\"Recursive dumping of extensions not supported in compiled code. Switch to uncompiled or dump extension object directly\",f;for(d in b.extensions)if(/^\\d+$/.test(d)){m=b.extensions[d];var t=a.getExtension(m);h=void 0;m=m.Ba;var B=[],M=0;for(h in m)B[M++]=h;h=B[0];null!=t&&(c||(c=f.$extensions={}),c[R.$a(h)]=R.X(t))}return f};R.$a=function(a){return a.replace(/^[A-Z]/,function(b){return b.toLowerCase()})};function S(){this.a=[]}g(\"jspb.BinaryEncoder\",S,void 0);S.prototype.length=function(){return this.a.length};S.prototype.length=S.prototype.length;S.prototype.end=function(){var a=this.a;this.a=[];return a};S.prototype.end=S.prototype.end;S.prototype.l=function(a,b){n(a==Math.floor(a));n(b==Math.floor(b));n(0<=a&&4294967296>a);for(n(0<=b&&4294967296>b);0<b||127<a;)this.a.push(a&127|128),a=(a>>>7|b<<25)>>>0,b>>>=7;this.a.push(a)};S.prototype.writeSplitVarint64=S.prototype.l;\nS.prototype.A=function(a,b){n(a==Math.floor(a));n(b==Math.floor(b));n(0<=a&&4294967296>a);n(0<=b&&4294967296>b);this.s(a);this.s(b)};S.prototype.writeSplitFixed64=S.prototype.A;S.prototype.j=function(a){n(a==Math.floor(a));for(n(0<=a&&4294967296>a);127<a;)this.a.push(a&127|128),a>>>=7;this.a.push(a)};S.prototype.writeUnsignedVarint32=S.prototype.j;S.prototype.M=function(a){n(a==Math.floor(a));n(-2147483648<=a&&2147483648>a);if(0<=a)this.j(a);else{for(var b=0;9>b;b++)this.a.push(a&127|128),a>>=7;this.a.push(1)}};\nS.prototype.writeSignedVarint32=S.prototype.M;S.prototype.va=function(a){n(a==Math.floor(a));n(0<=a&&1.8446744073709552E19>a);A(a);this.l(y,z)};S.prototype.writeUnsignedVarint64=S.prototype.va;S.prototype.ua=function(a){n(a==Math.floor(a));n(-9223372036854775808<=a&&0x7fffffffffffffff>a);A(a);this.l(y,z)};S.prototype.writeSignedVarint64=S.prototype.ua;S.prototype.wa=function(a){n(a==Math.floor(a));n(-2147483648<=a&&2147483648>a);this.j((a<<1^a>>31)>>>0)};S.prototype.writeZigzagVarint32=S.prototype.wa;\nS.prototype.xa=function(a){n(a==Math.floor(a));n(-9223372036854775808<=a&&0x7fffffffffffffff>a);Ga(a);this.l(y,z)};S.prototype.writeZigzagVarint64=S.prototype.xa;S.prototype.Ta=function(a){this.W(H(a))};S.prototype.writeZigzagVarint64String=S.prototype.Ta;S.prototype.W=function(a){var b=this;C(a);Ja(y,z,function(c,d){b.l(c>>>0,d>>>0)})};S.prototype.writeZigzagVarintHash64=S.prototype.W;S.prototype.be=function(a){n(a==Math.floor(a));n(0<=a&&256>a);this.a.push(a>>>0&255)};S.prototype.writeUint8=S.prototype.be;\nS.prototype.ae=function(a){n(a==Math.floor(a));n(0<=a&&65536>a);this.a.push(a>>>0&255);this.a.push(a>>>8&255)};S.prototype.writeUint16=S.prototype.ae;S.prototype.s=function(a){n(a==Math.floor(a));n(0<=a&&4294967296>a);this.a.push(a>>>0&255);this.a.push(a>>>8&255);this.a.push(a>>>16&255);this.a.push(a>>>24&255)};S.prototype.writeUint32=S.prototype.s;S.prototype.V=function(a){n(a==Math.floor(a));n(0<=a&&1.8446744073709552E19>a);Fa(a);this.s(y);this.s(z)};S.prototype.writeUint64=S.prototype.V;\nS.prototype.Qc=function(a){n(a==Math.floor(a));n(-128<=a&&128>a);this.a.push(a>>>0&255)};S.prototype.writeInt8=S.prototype.Qc;S.prototype.Pc=function(a){n(a==Math.floor(a));n(-32768<=a&&32768>a);this.a.push(a>>>0&255);this.a.push(a>>>8&255)};S.prototype.writeInt16=S.prototype.Pc;S.prototype.S=function(a){n(a==Math.floor(a));n(-2147483648<=a&&2147483648>a);this.a.push(a>>>0&255);this.a.push(a>>>8&255);this.a.push(a>>>16&255);this.a.push(a>>>24&255)};S.prototype.writeInt32=S.prototype.S;\nS.prototype.T=function(a){n(a==Math.floor(a));n(-9223372036854775808<=a&&0x7fffffffffffffff>a);A(a);this.A(y,z)};S.prototype.writeInt64=S.prototype.T;S.prototype.ka=function(a){n(a==Math.floor(a));n(-9223372036854775808<=+a&&0x7fffffffffffffff>+a);C(H(a));this.A(y,z)};S.prototype.writeInt64String=S.prototype.ka;S.prototype.L=function(a){n(Infinity===a||-Infinity===a||isNaN(a)||-3.4028234663852886E38<=a&&3.4028234663852886E38>=a);Ha(a);this.s(y)};S.prototype.writeFloat=S.prototype.L;\nS.prototype.J=function(a){n(Infinity===a||-Infinity===a||isNaN(a)||-1.7976931348623157E308<=a&&1.7976931348623157E308>=a);Ia(a);this.s(y);this.s(z)};S.prototype.writeDouble=S.prototype.J;S.prototype.I=function(a){n(\"boolean\"===typeof a||\"number\"===typeof a);this.a.push(a?1:0)};S.prototype.writeBool=S.prototype.I;S.prototype.R=function(a){n(a==Math.floor(a));n(-2147483648<=a&&2147483648>a);this.M(a)};S.prototype.writeEnum=S.prototype.R;S.prototype.ja=function(a){this.a.push.apply(this.a,a)};\nS.prototype.writeBytes=S.prototype.ja;S.prototype.N=function(a){C(a);this.l(y,z)};S.prototype.writeVarintHash64=S.prototype.N;S.prototype.K=function(a){C(a);this.s(y);this.s(z)};S.prototype.writeFixedHash64=S.prototype.K;\nS.prototype.U=function(a){var b=this.a.length;ta(a);for(var c=0;c<a.length;c++){var d=a.charCodeAt(c);if(128>d)this.a.push(d);else if(2048>d)this.a.push(d>>6|192),this.a.push(d&63|128);else if(65536>d)if(55296<=d&&56319>=d&&c+1<a.length){var f=a.charCodeAt(c+1);56320<=f&&57343>=f&&(d=1024*(d-55296)+f-56320+65536,this.a.push(d>>18|240),this.a.push(d>>12&63|128),this.a.push(d>>6&63|128),this.a.push(d&63|128),c++)}else this.a.push(d>>12|224),this.a.push(d>>6&63|128),this.a.push(d&63|128)}return this.a.length-\nb};S.prototype.writeString=S.prototype.U;function T(a,b){this.lo=a;this.hi=b}g(\"jspb.arith.UInt64\",T,void 0);T.prototype.cmp=function(a){return this.hi<a.hi||this.hi==a.hi&&this.lo<a.lo?-1:this.hi==a.hi&&this.lo==a.lo?0:1};T.prototype.cmp=T.prototype.cmp;T.prototype.La=function(){return new T((this.lo>>>1|(this.hi&1)<<31)>>>0,this.hi>>>1>>>0)};T.prototype.rightShift=T.prototype.La;T.prototype.Da=function(){return new T(this.lo<<1>>>0,(this.hi<<1|this.lo>>>31)>>>0)};T.prototype.leftShift=T.prototype.Da;\nT.prototype.cb=function(){return!!(this.hi&2147483648)};T.prototype.msb=T.prototype.cb;T.prototype.Ob=function(){return!!(this.lo&1)};T.prototype.lsb=T.prototype.Ob;T.prototype.Ua=function(){return 0==this.lo&&0==this.hi};T.prototype.zero=T.prototype.Ua;T.prototype.add=function(a){return new T((this.lo+a.lo&4294967295)>>>0>>>0,((this.hi+a.hi&4294967295)>>>0)+(4294967296<=this.lo+a.lo?1:0)>>>0)};T.prototype.add=T.prototype.add;\nT.prototype.sub=function(a){return new T((this.lo-a.lo&4294967295)>>>0>>>0,((this.hi-a.hi&4294967295)>>>0)-(0>this.lo-a.lo?1:0)>>>0)};T.prototype.sub=T.prototype.sub;function rb(a,b){var c=a&65535;a>>>=16;var d=b&65535,f=b>>>16;b=c*d+65536*(c*f&65535)+65536*(a*d&65535);for(c=a*f+(c*f>>>16)+(a*d>>>16);4294967296<=b;)b-=4294967296,c+=1;return new T(b>>>0,c>>>0)}T.mul32x32=rb;T.prototype.eb=function(a){var b=rb(this.lo,a);a=rb(this.hi,a);a.hi=a.lo;a.lo=0;return b.add(a)};T.prototype.mul=T.prototype.eb;\nT.prototype.Xa=function(a){if(0==a)return[];var b=new T(0,0),c=new T(this.lo,this.hi);a=new T(a,0);for(var d=new T(1,0);!a.cb();)a=a.Da(),d=d.Da();for(;!d.Ua();)0>=a.cmp(c)&&(b=b.add(d),c=c.sub(a)),a=a.La(),d=d.La();return[b,c]};T.prototype.div=T.prototype.Xa;T.prototype.toString=function(){for(var a=\"\",b=this;!b.Ua();){b=b.Xa(10);var c=b[0];a=b[1].lo+a;b=c}\"\"==a&&(a=\"0\");return a};T.prototype.toString=T.prototype.toString;\nfunction U(a){for(var b=new T(0,0),c=new T(0,0),d=0;d<a.length;d++){if(\"0\">a[d]||\"9\"<a[d])return null;c.lo=parseInt(a[d],10);b=b.eb(10).add(c)}return b}T.fromString=U;T.prototype.clone=function(){return new T(this.lo,this.hi)};T.prototype.clone=T.prototype.clone;function V(a,b){this.lo=a;this.hi=b}g(\"jspb.arith.Int64\",V,void 0);V.prototype.add=function(a){return new V((this.lo+a.lo&4294967295)>>>0>>>0,((this.hi+a.hi&4294967295)>>>0)+(4294967296<=this.lo+a.lo?1:0)>>>0)};V.prototype.add=V.prototype.add;\nV.prototype.sub=function(a){return new V((this.lo-a.lo&4294967295)>>>0>>>0,((this.hi-a.hi&4294967295)>>>0)-(0>this.lo-a.lo?1:0)>>>0)};V.prototype.sub=V.prototype.sub;V.prototype.clone=function(){return new V(this.lo,this.hi)};V.prototype.clone=V.prototype.clone;V.prototype.toString=function(){var a=0!=(this.hi&2147483648),b=new T(this.lo,this.hi);a&&(b=(new T(0,0)).sub(b));return(a?\"-\":\"\")+b.toString()};V.prototype.toString=V.prototype.toString;\nfunction sb(a){var b=0<a.length&&\"-\"==a[0];b&&(a=a.substring(1));a=U(a);if(null===a)return null;b&&(a=(new T(0,0)).sub(a));return new V(a.lo,a.hi)}V.fromString=sb;function W(){this.c=[];this.b=0;this.a=new S;this.h=[]}g(\"jspb.BinaryWriter\",W,void 0);function tb(a,b){var c=a.a.end();a.c.push(c);a.c.push(b);a.b+=c.length+b.length}function X(a,b){Y(a,b,2);b=a.a.end();a.c.push(b);a.b+=b.length;b.push(a.b);return b}function Z(a,b){var c=b.pop();c=a.b+a.a.length()-c;for(n(0<=c);127<c;)b.push(c&127|128),c>>>=7,a.b++;b.push(c);a.b++}W.prototype.pb=function(a,b,c){tb(this,a.subarray(b,c))};W.prototype.writeSerializedMessage=W.prototype.pb;\nW.prototype.Pb=function(a,b,c){null!=a&&null!=b&&null!=c&&this.pb(a,b,c)};W.prototype.maybeWriteSerializedMessage=W.prototype.Pb;W.prototype.reset=function(){this.c=[];this.a.end();this.b=0;this.h=[]};W.prototype.reset=W.prototype.reset;W.prototype.ab=function(){n(0==this.h.length);for(var a=new Uint8Array(this.b+this.a.length()),b=this.c,c=b.length,d=0,f=0;f<c;f++){var h=b[f];a.set(h,d);d+=h.length}b=this.a.end();a.set(b,d);d+=b.length;n(d==a.length);this.c=[a];return a};\nW.prototype.getResultBuffer=W.prototype.ab;W.prototype.Kb=function(a){return Ba(this.ab(),a)};W.prototype.getResultBase64String=W.prototype.Kb;W.prototype.Va=function(a){this.h.push(X(this,a))};W.prototype.beginSubMessage=W.prototype.Va;W.prototype.Ya=function(){n(0<=this.h.length);Z(this,this.h.pop())};W.prototype.endSubMessage=W.prototype.Ya;function Y(a,b,c){n(1<=b&&b==Math.floor(b));a.a.j(8*b+c)}\nW.prototype.Nc=function(a,b,c){switch(a){case 1:this.J(b,c);break;case 2:this.L(b,c);break;case 3:this.T(b,c);break;case 4:this.V(b,c);break;case 5:this.S(b,c);break;case 6:this.Qa(b,c);break;case 7:this.Pa(b,c);break;case 8:this.I(b,c);break;case 9:this.U(b,c);break;case 10:p(\"Group field type not supported in writeAny()\");break;case 11:p(\"Message field type not supported in writeAny()\");break;case 12:this.ja(b,c);break;case 13:this.s(b,c);break;case 14:this.R(b,c);break;case 15:this.Ra(b,c);break;\ncase 16:this.Sa(b,c);break;case 17:this.rb(b,c);break;case 18:this.sb(b,c);break;case 30:this.K(b,c);break;case 31:this.N(b,c);break;default:p(\"Invalid field type in writeAny()\")}};W.prototype.writeAny=W.prototype.Nc;function ub(a,b,c){null!=c&&(Y(a,b,0),a.a.j(c))}function vb(a,b,c){null!=c&&(Y(a,b,0),a.a.M(c))}W.prototype.S=function(a,b){null!=b&&(n(-2147483648<=b&&2147483648>b),vb(this,a,b))};W.prototype.writeInt32=W.prototype.S;\nW.prototype.ob=function(a,b){null!=b&&(b=parseInt(b,10),n(-2147483648<=b&&2147483648>b),vb(this,a,b))};W.prototype.writeInt32String=W.prototype.ob;W.prototype.T=function(a,b){null!=b&&(n(-9223372036854775808<=b&&0x7fffffffffffffff>b),null!=b&&(Y(this,a,0),this.a.ua(b)))};W.prototype.writeInt64=W.prototype.T;W.prototype.ka=function(a,b){null!=b&&(b=sb(b),Y(this,a,0),this.a.l(b.lo,b.hi))};W.prototype.writeInt64String=W.prototype.ka;\nW.prototype.s=function(a,b){null!=b&&(n(0<=b&&4294967296>b),ub(this,a,b))};W.prototype.writeUint32=W.prototype.s;W.prototype.ub=function(a,b){null!=b&&(b=parseInt(b,10),n(0<=b&&4294967296>b),ub(this,a,b))};W.prototype.writeUint32String=W.prototype.ub;W.prototype.V=function(a,b){null!=b&&(n(0<=b&&1.8446744073709552E19>b),null!=b&&(Y(this,a,0),this.a.va(b)))};W.prototype.writeUint64=W.prototype.V;W.prototype.vb=function(a,b){null!=b&&(b=U(b),Y(this,a,0),this.a.l(b.lo,b.hi))};\nW.prototype.writeUint64String=W.prototype.vb;W.prototype.rb=function(a,b){null!=b&&(n(-2147483648<=b&&2147483648>b),null!=b&&(Y(this,a,0),this.a.wa(b)))};W.prototype.writeSint32=W.prototype.rb;W.prototype.sb=function(a,b){null!=b&&(n(-9223372036854775808<=b&&0x7fffffffffffffff>b),null!=b&&(Y(this,a,0),this.a.xa(b)))};W.prototype.writeSint64=W.prototype.sb;W.prototype.$d=function(a,b){null!=b&&null!=b&&(Y(this,a,0),this.a.W(b))};W.prototype.writeSintHash64=W.prototype.$d;\nW.prototype.Zd=function(a,b){null!=b&&null!=b&&(Y(this,a,0),this.a.Ta(b))};W.prototype.writeSint64String=W.prototype.Zd;W.prototype.Pa=function(a,b){null!=b&&(n(0<=b&&4294967296>b),Y(this,a,5),this.a.s(b))};W.prototype.writeFixed32=W.prototype.Pa;W.prototype.Qa=function(a,b){null!=b&&(n(0<=b&&1.8446744073709552E19>b),Y(this,a,1),this.a.V(b))};W.prototype.writeFixed64=W.prototype.Qa;W.prototype.nb=function(a,b){null!=b&&(b=U(b),Y(this,a,1),this.a.A(b.lo,b.hi))};W.prototype.writeFixed64String=W.prototype.nb;\nW.prototype.Ra=function(a,b){null!=b&&(n(-2147483648<=b&&2147483648>b),Y(this,a,5),this.a.S(b))};W.prototype.writeSfixed32=W.prototype.Ra;W.prototype.Sa=function(a,b){null!=b&&(n(-9223372036854775808<=b&&0x7fffffffffffffff>b),Y(this,a,1),this.a.T(b))};W.prototype.writeSfixed64=W.prototype.Sa;W.prototype.qb=function(a,b){null!=b&&(b=sb(b),Y(this,a,1),this.a.A(b.lo,b.hi))};W.prototype.writeSfixed64String=W.prototype.qb;W.prototype.L=function(a,b){null!=b&&(Y(this,a,5),this.a.L(b))};\nW.prototype.writeFloat=W.prototype.L;W.prototype.J=function(a,b){null!=b&&(Y(this,a,1),this.a.J(b))};W.prototype.writeDouble=W.prototype.J;W.prototype.I=function(a,b){null!=b&&(n(\"boolean\"===typeof b||\"number\"===typeof b),Y(this,a,0),this.a.I(b))};W.prototype.writeBool=W.prototype.I;W.prototype.R=function(a,b){null!=b&&(n(-2147483648<=b&&2147483648>b),Y(this,a,0),this.a.M(b))};W.prototype.writeEnum=W.prototype.R;W.prototype.U=function(a,b){null!=b&&(a=X(this,a),this.a.U(b),Z(this,a))};\nW.prototype.writeString=W.prototype.U;W.prototype.ja=function(a,b){null!=b&&(b=Ua(b),Y(this,a,2),this.a.j(b.length),tb(this,b))};W.prototype.writeBytes=W.prototype.ja;W.prototype.Rc=function(a,b,c){null!=b&&(a=X(this,a),c(b,this),Z(this,a))};W.prototype.writeMessage=W.prototype.Rc;W.prototype.Sc=function(a,b,c){null!=b&&(Y(this,1,3),Y(this,2,0),this.a.M(a),a=X(this,3),c(b,this),Z(this,a),Y(this,1,4))};W.prototype.writeMessageSet=W.prototype.Sc;\nW.prototype.Oc=function(a,b,c){null!=b&&(Y(this,a,3),c(b,this),Y(this,a,4))};W.prototype.writeGroup=W.prototype.Oc;W.prototype.K=function(a,b){null!=b&&(n(8==b.length),Y(this,a,1),this.a.K(b))};W.prototype.writeFixedHash64=W.prototype.K;W.prototype.N=function(a,b){null!=b&&(n(8==b.length),Y(this,a,0),this.a.N(b))};W.prototype.writeVarintHash64=W.prototype.N;W.prototype.A=function(a,b,c){Y(this,a,1);this.a.A(b,c)};W.prototype.writeSplitFixed64=W.prototype.A;\nW.prototype.l=function(a,b,c){Y(this,a,0);this.a.l(b,c)};W.prototype.writeSplitVarint64=W.prototype.l;W.prototype.tb=function(a,b,c){Y(this,a,0);var d=this.a;Ja(b,c,function(f,h){d.l(f>>>0,h>>>0)})};W.prototype.writeSplitZigzagVarint64=W.prototype.tb;W.prototype.Ed=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)vb(this,a,b[c])};W.prototype.writeRepeatedInt32=W.prototype.Ed;W.prototype.Fd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.ob(a,b[c])};\nW.prototype.writeRepeatedInt32String=W.prototype.Fd;W.prototype.Gd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++){var d=b[c];null!=d&&(Y(this,a,0),this.a.ua(d))}};W.prototype.writeRepeatedInt64=W.prototype.Gd;W.prototype.Qd=function(a,b,c,d){if(null!=b)for(var f=0;f<b.length;f++)this.A(a,c(b[f]),d(b[f]))};W.prototype.writeRepeatedSplitFixed64=W.prototype.Qd;W.prototype.Rd=function(a,b,c,d){if(null!=b)for(var f=0;f<b.length;f++)this.l(a,c(b[f]),d(b[f]))};\nW.prototype.writeRepeatedSplitVarint64=W.prototype.Rd;W.prototype.Sd=function(a,b,c,d){if(null!=b)for(var f=0;f<b.length;f++)this.tb(a,c(b[f]),d(b[f]))};W.prototype.writeRepeatedSplitZigzagVarint64=W.prototype.Sd;W.prototype.Hd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.ka(a,b[c])};W.prototype.writeRepeatedInt64String=W.prototype.Hd;W.prototype.Ud=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)ub(this,a,b[c])};W.prototype.writeRepeatedUint32=W.prototype.Ud;\nW.prototype.Vd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.ub(a,b[c])};W.prototype.writeRepeatedUint32String=W.prototype.Vd;W.prototype.Wd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++){var d=b[c];null!=d&&(Y(this,a,0),this.a.va(d))}};W.prototype.writeRepeatedUint64=W.prototype.Wd;W.prototype.Xd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.vb(a,b[c])};W.prototype.writeRepeatedUint64String=W.prototype.Xd;\nW.prototype.Md=function(a,b){if(null!=b)for(var c=0;c<b.length;c++){var d=b[c];null!=d&&(Y(this,a,0),this.a.wa(d))}};W.prototype.writeRepeatedSint32=W.prototype.Md;W.prototype.Nd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++){var d=b[c];null!=d&&(Y(this,a,0),this.a.xa(d))}};W.prototype.writeRepeatedSint64=W.prototype.Nd;W.prototype.Od=function(a,b){if(null!=b)for(var c=0;c<b.length;c++){var d=b[c];null!=d&&(Y(this,a,0),this.a.Ta(d))}};W.prototype.writeRepeatedSint64String=W.prototype.Od;\nW.prototype.Pd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++){var d=b[c];null!=d&&(Y(this,a,0),this.a.W(d))}};W.prototype.writeRepeatedSintHash64=W.prototype.Pd;W.prototype.yd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.Pa(a,b[c])};W.prototype.writeRepeatedFixed32=W.prototype.yd;W.prototype.zd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.Qa(a,b[c])};W.prototype.writeRepeatedFixed64=W.prototype.zd;\nW.prototype.Ad=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.nb(a,b[c])};W.prototype.writeRepeatedFixed64String=W.prototype.Ad;W.prototype.Jd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.Ra(a,b[c])};W.prototype.writeRepeatedSfixed32=W.prototype.Jd;W.prototype.Kd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.Sa(a,b[c])};W.prototype.writeRepeatedSfixed64=W.prototype.Kd;W.prototype.Ld=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.qb(a,b[c])};\nW.prototype.writeRepeatedSfixed64String=W.prototype.Ld;W.prototype.Cd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.L(a,b[c])};W.prototype.writeRepeatedFloat=W.prototype.Cd;W.prototype.wd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.J(a,b[c])};W.prototype.writeRepeatedDouble=W.prototype.wd;W.prototype.ud=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.I(a,b[c])};W.prototype.writeRepeatedBool=W.prototype.ud;\nW.prototype.xd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.R(a,b[c])};W.prototype.writeRepeatedEnum=W.prototype.xd;W.prototype.Td=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.U(a,b[c])};W.prototype.writeRepeatedString=W.prototype.Td;W.prototype.vd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.ja(a,b[c])};W.prototype.writeRepeatedBytes=W.prototype.vd;W.prototype.Id=function(a,b,c){if(null!=b)for(var d=0;d<b.length;d++){var f=X(this,a);c(b[d],this);Z(this,f)}};\nW.prototype.writeRepeatedMessage=W.prototype.Id;W.prototype.Dd=function(a,b,c){if(null!=b)for(var d=0;d<b.length;d++)Y(this,a,3),c(b[d],this),Y(this,a,4)};W.prototype.writeRepeatedGroup=W.prototype.Dd;W.prototype.Bd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.K(a,b[c])};W.prototype.writeRepeatedFixedHash64=W.prototype.Bd;W.prototype.Yd=function(a,b){if(null!=b)for(var c=0;c<b.length;c++)this.N(a,b[c])};W.prototype.writeRepeatedVarintHash64=W.prototype.Yd;\nW.prototype.ad=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.M(b[c]);Z(this,a)}};W.prototype.writePackedInt32=W.prototype.ad;W.prototype.bd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.M(parseInt(b[c],10));Z(this,a)}};W.prototype.writePackedInt32String=W.prototype.bd;W.prototype.cd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.ua(b[c]);Z(this,a)}};W.prototype.writePackedInt64=W.prototype.cd;\nW.prototype.md=function(a,b,c,d){if(null!=b){a=X(this,a);for(var f=0;f<b.length;f++)this.a.A(c(b[f]),d(b[f]));Z(this,a)}};W.prototype.writePackedSplitFixed64=W.prototype.md;W.prototype.nd=function(a,b,c,d){if(null!=b){a=X(this,a);for(var f=0;f<b.length;f++)this.a.l(c(b[f]),d(b[f]));Z(this,a)}};W.prototype.writePackedSplitVarint64=W.prototype.nd;W.prototype.od=function(a,b,c,d){if(null!=b){a=X(this,a);for(var f=this.a,h=0;h<b.length;h++)Ja(c(b[h]),d(b[h]),function(m,t){f.l(m>>>0,t>>>0)});Z(this,a)}};\nW.prototype.writePackedSplitZigzagVarint64=W.prototype.od;W.prototype.dd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++){var d=sb(b[c]);this.a.l(d.lo,d.hi)}Z(this,a)}};W.prototype.writePackedInt64String=W.prototype.dd;W.prototype.pd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.j(b[c]);Z(this,a)}};W.prototype.writePackedUint32=W.prototype.pd;\nW.prototype.qd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.j(parseInt(b[c],10));Z(this,a)}};W.prototype.writePackedUint32String=W.prototype.qd;W.prototype.rd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.va(b[c]);Z(this,a)}};W.prototype.writePackedUint64=W.prototype.rd;W.prototype.sd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++){var d=U(b[c]);this.a.l(d.lo,d.hi)}Z(this,a)}};\nW.prototype.writePackedUint64String=W.prototype.sd;W.prototype.hd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.wa(b[c]);Z(this,a)}};W.prototype.writePackedSint32=W.prototype.hd;W.prototype.jd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.xa(b[c]);Z(this,a)}};W.prototype.writePackedSint64=W.prototype.jd;W.prototype.kd=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.W(H(b[c]));Z(this,a)}};\nW.prototype.writePackedSint64String=W.prototype.kd;W.prototype.ld=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.W(b[c]);Z(this,a)}};W.prototype.writePackedSintHash64=W.prototype.ld;W.prototype.Wc=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(4*b.length),a=0;a<b.length;a++)this.a.s(b[a])};W.prototype.writePackedFixed32=W.prototype.Wc;W.prototype.Xc=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(8*b.length),a=0;a<b.length;a++)this.a.V(b[a])};\nW.prototype.writePackedFixed64=W.prototype.Xc;W.prototype.Yc=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(8*b.length),a=0;a<b.length;a++){var c=U(b[a]);this.a.A(c.lo,c.hi)}};W.prototype.writePackedFixed64String=W.prototype.Yc;W.prototype.ed=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(4*b.length),a=0;a<b.length;a++)this.a.S(b[a])};W.prototype.writePackedSfixed32=W.prototype.ed;\nW.prototype.fd=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(8*b.length),a=0;a<b.length;a++)this.a.T(b[a])};W.prototype.writePackedSfixed64=W.prototype.fd;W.prototype.gd=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(8*b.length),a=0;a<b.length;a++)this.a.ka(b[a])};W.prototype.writePackedSfixed64String=W.prototype.gd;W.prototype.$c=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(4*b.length),a=0;a<b.length;a++)this.a.L(b[a])};W.prototype.writePackedFloat=W.prototype.$c;\nW.prototype.Uc=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(8*b.length),a=0;a<b.length;a++)this.a.J(b[a])};W.prototype.writePackedDouble=W.prototype.Uc;W.prototype.Tc=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(b.length),a=0;a<b.length;a++)this.a.I(b[a])};W.prototype.writePackedBool=W.prototype.Tc;W.prototype.Vc=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.R(b[c]);Z(this,a)}};W.prototype.writePackedEnum=W.prototype.Vc;\nW.prototype.Zc=function(a,b){if(null!=b&&b.length)for(Y(this,a,2),this.a.j(8*b.length),a=0;a<b.length;a++)this.a.K(b[a])};W.prototype.writePackedFixedHash64=W.prototype.Zc;W.prototype.td=function(a,b){if(null!=b&&b.length){a=X(this,a);for(var c=0;c<b.length;c++)this.a.N(b[c]);Z(this,a)}};W.prototype.writePackedVarintHash64=W.prototype.td; true&&(exports.debug=R,exports.Map=r,exports.Message=N,exports.BinaryReader=J,exports.BinaryWriter=W,exports.ExtensionFieldInfo=Ya,exports.ExtensionFieldBinaryInfo=Za,exports.exportSymbol=ma,exports.inherits=na,exports.object={extend:pa},exports.typeOf=k);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/google-protobuf/google-protobuf.js?");

/***/ }),

/***/ "./node_modules/google-protobuf/google/protobuf/any_pb.js":
/*!****************************************************************!*\
  !*** ./node_modules/google-protobuf/google/protobuf/any_pb.js ***!
  \****************************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("// source: google/protobuf/any.proto\n/**\n * @fileoverview\n * @enhanceable\n * @suppress {missingRequire} reports error on implicit type usages.\n * @suppress {messageConventions} JS Compiler reports an error if a variable or\n *     field starts with 'MSG_' and isn't a translatable message.\n * @public\n */\n// GENERATED CODE -- DO NOT EDIT!\n/* eslint-disable */\n// @ts-nocheck\n\nvar jspb = __webpack_require__(/*! google-protobuf */ \"./node_modules/google-protobuf/google-protobuf.js\");\nvar goog = jspb;\nvar global =\n    (typeof globalThis !== 'undefined' && globalThis) ||\n    (typeof window !== 'undefined' && window) ||\n    (typeof global !== 'undefined' && global) ||\n    (typeof self !== 'undefined' && self) ||\n    (function () { return this; }).call(null) ||\n    Function('return this')();\n\ngoog.exportSymbol('proto.google.protobuf.Any', null, global);\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.Any = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.Any, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.Any.displayName = 'proto.google.protobuf.Any';\n}\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.Any.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.Any.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.Any} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.Any.toObject = function(includeInstance, msg) {\n  var f, obj = {\ntypeUrl: jspb.Message.getFieldWithDefault(msg, 1, \"\"),\nvalue: msg.getValue_asB64()\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.Any}\n */\nproto.google.protobuf.Any.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.Any;\n  return proto.google.protobuf.Any.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.Any} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.Any}\n */\nproto.google.protobuf.Any.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTypeUrl(value);\n      break;\n    case 2:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setValue(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.Any.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.Any.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.Any} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.Any.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getTypeUrl();\n  if (f.length > 0) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = message.getValue_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string type_url = 1;\n * @return {string}\n */\nproto.google.protobuf.Any.prototype.getTypeUrl = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.Any} returns this\n */\nproto.google.protobuf.Any.prototype.setTypeUrl = function(value) {\n  return jspb.Message.setProto3StringField(this, 1, value);\n};\n\n\n/**\n * optional bytes value = 2;\n * @return {!(string|Uint8Array)}\n */\nproto.google.protobuf.Any.prototype.getValue = function() {\n  return /** @type {!(string|Uint8Array)} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * optional bytes value = 2;\n * This is a type-conversion wrapper around `getValue()`\n * @return {string}\n */\nproto.google.protobuf.Any.prototype.getValue_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getValue()));\n};\n\n\n/**\n * optional bytes value = 2;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getValue()`\n * @return {!Uint8Array}\n */\nproto.google.protobuf.Any.prototype.getValue_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getValue()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.google.protobuf.Any} returns this\n */\nproto.google.protobuf.Any.prototype.setValue = function(value) {\n  return jspb.Message.setProto3BytesField(this, 2, value);\n};\n\n\ngoog.object.extend(exports, proto.google.protobuf);\n/* This code will be inserted into generated code for\n * google/protobuf/any.proto. */\n\n/**\n * Returns the type name contained in this instance, if any.\n * @return {string|undefined}\n */\nproto.google.protobuf.Any.prototype.getTypeName = function() {\n  return this.getTypeUrl().split('/').pop();\n};\n\n\n/**\n * Packs the given message instance into this Any.\n * For binary format usage only.\n * @param {!Uint8Array} serialized The serialized data to pack.\n * @param {string} name The type name of this message object.\n * @param {string=} opt_typeUrlPrefix the type URL prefix.\n */\nproto.google.protobuf.Any.prototype.pack = function(serialized, name,\n                                                    opt_typeUrlPrefix) {\n  if (!opt_typeUrlPrefix) {\n    opt_typeUrlPrefix = 'type.googleapis.com/';\n  }\n\n  if (opt_typeUrlPrefix.substr(-1) != '/') {\n    this.setTypeUrl(opt_typeUrlPrefix + '/' + name);\n  } else {\n    this.setTypeUrl(opt_typeUrlPrefix + name);\n  }\n\n  this.setValue(serialized);\n};\n\n\n/**\n * @template T\n * Unpacks this Any into the given message object.\n * @param {function(Uint8Array):T} deserialize Function that will deserialize\n *     the binary data properly.\n * @param {string} name The expected type name of this message object.\n * @return {?T} If the name matched the expected name, returns the deserialized\n *     object, otherwise returns null.\n */\nproto.google.protobuf.Any.prototype.unpack = function(deserialize, name) {\n  if (this.getTypeName() == name) {\n    return deserialize(this.getValue_asU8());\n  } else {\n    return null;\n  }\n};\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/google-protobuf/google/protobuf/any_pb.js?");

/***/ }),

/***/ "./node_modules/google-protobuf/google/protobuf/descriptor_pb.js":
/*!***********************************************************************!*\
  !*** ./node_modules/google-protobuf/google/protobuf/descriptor_pb.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("// source: google/protobuf/descriptor.proto\n/**\n * @fileoverview\n * @enhanceable\n * @suppress {missingRequire} reports error on implicit type usages.\n * @suppress {messageConventions} JS Compiler reports an error if a variable or\n *     field starts with 'MSG_' and isn't a translatable message.\n * @public\n */\n// GENERATED CODE -- DO NOT EDIT!\n/* eslint-disable */\n// @ts-nocheck\n\nvar jspb = __webpack_require__(/*! google-protobuf */ \"./node_modules/google-protobuf/google-protobuf.js\");\nvar goog = jspb;\nvar global =\n    (typeof globalThis !== 'undefined' && globalThis) ||\n    (typeof window !== 'undefined' && window) ||\n    (typeof global !== 'undefined' && global) ||\n    (typeof self !== 'undefined' && self) ||\n    (function () { return this; }).call(null) ||\n    Function('return this')();\n\ngoog.exportSymbol('proto.google.protobuf.DescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.DescriptorProto.ExtensionRange', null, global);\ngoog.exportSymbol('proto.google.protobuf.DescriptorProto.ReservedRange', null, global);\ngoog.exportSymbol('proto.google.protobuf.Edition', null, global);\ngoog.exportSymbol('proto.google.protobuf.EnumDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.EnumDescriptorProto.EnumReservedRange', null, global);\ngoog.exportSymbol('proto.google.protobuf.EnumOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.EnumValueDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.EnumValueOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.ExtensionRangeOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.ExtensionRangeOptions.Declaration', null, global);\ngoog.exportSymbol('proto.google.protobuf.ExtensionRangeOptions.VerificationState', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet.EnumType', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet.FieldPresence', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet.JsonFormat', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet.MessageEncoding', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet.RepeatedFieldEncoding', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSet.Utf8Validation', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSetDefaults', null, global);\ngoog.exportSymbol('proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldDescriptorProto.Label', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldDescriptorProto.Type', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions.CType', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions.EditionDefault', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions.FeatureSupport', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions.JSType', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions.OptionRetention', null, global);\ngoog.exportSymbol('proto.google.protobuf.FieldOptions.OptionTargetType', null, global);\ngoog.exportSymbol('proto.google.protobuf.FileDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.FileDescriptorSet', null, global);\ngoog.exportSymbol('proto.google.protobuf.FileOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.FileOptions.OptimizeMode', null, global);\ngoog.exportSymbol('proto.google.protobuf.GeneratedCodeInfo', null, global);\ngoog.exportSymbol('proto.google.protobuf.GeneratedCodeInfo.Annotation', null, global);\ngoog.exportSymbol('proto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic', null, global);\ngoog.exportSymbol('proto.google.protobuf.MessageOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.MethodDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.MethodOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.MethodOptions.IdempotencyLevel', null, global);\ngoog.exportSymbol('proto.google.protobuf.OneofDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.OneofOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.ServiceDescriptorProto', null, global);\ngoog.exportSymbol('proto.google.protobuf.ServiceOptions', null, global);\ngoog.exportSymbol('proto.google.protobuf.SourceCodeInfo', null, global);\ngoog.exportSymbol('proto.google.protobuf.SourceCodeInfo.Location', null, global);\ngoog.exportSymbol('proto.google.protobuf.UninterpretedOption', null, global);\ngoog.exportSymbol('proto.google.protobuf.UninterpretedOption.NamePart', null, global);\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FileDescriptorSet = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.FileDescriptorSet.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.FileDescriptorSet, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FileDescriptorSet.displayName = 'proto.google.protobuf.FileDescriptorSet';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FileDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.FileDescriptorProto.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.FileDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FileDescriptorProto.displayName = 'proto.google.protobuf.FileDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.DescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.DescriptorProto.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.DescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.DescriptorProto.displayName = 'proto.google.protobuf.DescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.DescriptorProto.ExtensionRange, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.DescriptorProto.ExtensionRange.displayName = 'proto.google.protobuf.DescriptorProto.ExtensionRange';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.DescriptorProto.ReservedRange = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.DescriptorProto.ReservedRange, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.DescriptorProto.ReservedRange.displayName = 'proto.google.protobuf.DescriptorProto.ReservedRange';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.ExtensionRangeOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.ExtensionRangeOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.ExtensionRangeOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.ExtensionRangeOptions.displayName = 'proto.google.protobuf.ExtensionRangeOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.ExtensionRangeOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.ExtensionRangeOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.ExtensionRangeOptions.Declaration, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.ExtensionRangeOptions.Declaration.displayName = 'proto.google.protobuf.ExtensionRangeOptions.Declaration';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FieldDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.FieldDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FieldDescriptorProto.displayName = 'proto.google.protobuf.FieldDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.OneofDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.OneofDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.OneofDescriptorProto.displayName = 'proto.google.protobuf.OneofDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.EnumDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.EnumDescriptorProto.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.EnumDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.EnumDescriptorProto.displayName = 'proto.google.protobuf.EnumDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.EnumDescriptorProto.EnumReservedRange, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.displayName = 'proto.google.protobuf.EnumDescriptorProto.EnumReservedRange';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.EnumValueDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.EnumValueDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.EnumValueDescriptorProto.displayName = 'proto.google.protobuf.EnumValueDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.ServiceDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.ServiceDescriptorProto.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.ServiceDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.ServiceDescriptorProto.displayName = 'proto.google.protobuf.ServiceDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.MethodDescriptorProto = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.MethodDescriptorProto, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.MethodDescriptorProto.displayName = 'proto.google.protobuf.MethodDescriptorProto';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FileOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.FileOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.FileOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FileOptions.displayName = 'proto.google.protobuf.FileOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.FileOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.FileOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.MessageOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.MessageOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.MessageOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.MessageOptions.displayName = 'proto.google.protobuf.MessageOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.MessageOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.MessageOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FieldOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.FieldOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.FieldOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FieldOptions.displayName = 'proto.google.protobuf.FieldOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.FieldOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.FieldOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FieldOptions.EditionDefault = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.FieldOptions.EditionDefault, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FieldOptions.EditionDefault.displayName = 'proto.google.protobuf.FieldOptions.EditionDefault';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FieldOptions.FeatureSupport = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.FieldOptions.FeatureSupport, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FieldOptions.FeatureSupport.displayName = 'proto.google.protobuf.FieldOptions.FeatureSupport';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.OneofOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.OneofOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.OneofOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.OneofOptions.displayName = 'proto.google.protobuf.OneofOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.OneofOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.OneofOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.EnumOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.EnumOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.EnumOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.EnumOptions.displayName = 'proto.google.protobuf.EnumOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.EnumOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.EnumOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.EnumValueOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.EnumValueOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.EnumValueOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.EnumValueOptions.displayName = 'proto.google.protobuf.EnumValueOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.EnumValueOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.EnumValueOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.ServiceOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.ServiceOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.ServiceOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.ServiceOptions.displayName = 'proto.google.protobuf.ServiceOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.ServiceOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.ServiceOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.MethodOptions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.google.protobuf.MethodOptions.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.MethodOptions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.MethodOptions.displayName = 'proto.google.protobuf.MethodOptions';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.MethodOptions.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.MethodOptions.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.UninterpretedOption = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.UninterpretedOption.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.UninterpretedOption, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.UninterpretedOption.displayName = 'proto.google.protobuf.UninterpretedOption';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.UninterpretedOption.NamePart = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.UninterpretedOption.NamePart, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.UninterpretedOption.NamePart.displayName = 'proto.google.protobuf.UninterpretedOption.NamePart';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FeatureSet = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 7, null, null);\n};\ngoog.inherits(proto.google.protobuf.FeatureSet, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FeatureSet.displayName = 'proto.google.protobuf.FeatureSet';\n}\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldInfo>}\n */\nproto.google.protobuf.FeatureSet.extensions = {};\n\n\n/**\n * The extensions registered with this message class. This is a map of\n * extension field number to fieldInfo object.\n *\n * For example:\n *     { 123: {fieldIndex: 123, fieldName: {my_field_name: 0}, ctor: proto.example.MyMessage} }\n *\n * fieldName contains the JsCompiler renamed field name property so that it\n * works in OPTIMIZED mode.\n *\n * @type {!Object<number, jspb.ExtensionFieldBinaryInfo>}\n */\nproto.google.protobuf.FeatureSet.extensionsBinary = {};\n\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FeatureSetDefaults = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.FeatureSetDefaults.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.FeatureSetDefaults, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FeatureSetDefaults.displayName = 'proto.google.protobuf.FeatureSetDefaults';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.displayName = 'proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.SourceCodeInfo = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.SourceCodeInfo.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.SourceCodeInfo, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.SourceCodeInfo.displayName = 'proto.google.protobuf.SourceCodeInfo';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.SourceCodeInfo.Location = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.SourceCodeInfo.Location.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.SourceCodeInfo.Location, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.SourceCodeInfo.Location.displayName = 'proto.google.protobuf.SourceCodeInfo.Location';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.GeneratedCodeInfo = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.GeneratedCodeInfo.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.GeneratedCodeInfo, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.GeneratedCodeInfo.displayName = 'proto.google.protobuf.GeneratedCodeInfo';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.google.protobuf.GeneratedCodeInfo.Annotation.repeatedFields_, null);\n};\ngoog.inherits(proto.google.protobuf.GeneratedCodeInfo.Annotation, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.google.protobuf.GeneratedCodeInfo.Annotation.displayName = 'proto.google.protobuf.GeneratedCodeInfo.Annotation';\n}\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.FileDescriptorSet.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FileDescriptorSet.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FileDescriptorSet.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FileDescriptorSet} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FileDescriptorSet.toObject = function(includeInstance, msg) {\n  var f, obj = {\nfileList: jspb.Message.toObjectList(msg.getFileList(),\n    proto.google.protobuf.FileDescriptorProto.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FileDescriptorSet}\n */\nproto.google.protobuf.FileDescriptorSet.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FileDescriptorSet;\n  return proto.google.protobuf.FileDescriptorSet.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FileDescriptorSet} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FileDescriptorSet}\n */\nproto.google.protobuf.FileDescriptorSet.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.google.protobuf.FileDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.FileDescriptorProto.deserializeBinaryFromReader);\n      msg.addFile(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FileDescriptorSet.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FileDescriptorSet.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FileDescriptorSet} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FileDescriptorSet.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getFileList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.google.protobuf.FileDescriptorProto.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated FileDescriptorProto file = 1;\n * @return {!Array<!proto.google.protobuf.FileDescriptorProto>}\n */\nproto.google.protobuf.FileDescriptorSet.prototype.getFileList = function() {\n  return /** @type{!Array<!proto.google.protobuf.FileDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.FileDescriptorProto, 1));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FileDescriptorProto>} value\n * @return {!proto.google.protobuf.FileDescriptorSet} returns this\n*/\nproto.google.protobuf.FileDescriptorSet.prototype.setFileList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FileDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FileDescriptorProto}\n */\nproto.google.protobuf.FileDescriptorSet.prototype.addFile = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.google.protobuf.FileDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorSet} returns this\n */\nproto.google.protobuf.FileDescriptorSet.prototype.clearFileList = function() {\n  return this.setFileList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.FileDescriptorProto.repeatedFields_ = [3,10,11,4,5,6,7];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FileDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FileDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FileDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\npb_package: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\ndependencyList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f,\npublicDependencyList: (f = jspb.Message.getRepeatedField(msg, 10)) == null ? undefined : f,\nweakDependencyList: (f = jspb.Message.getRepeatedField(msg, 11)) == null ? undefined : f,\nmessageTypeList: jspb.Message.toObjectList(msg.getMessageTypeList(),\n    proto.google.protobuf.DescriptorProto.toObject, includeInstance),\nenumTypeList: jspb.Message.toObjectList(msg.getEnumTypeList(),\n    proto.google.protobuf.EnumDescriptorProto.toObject, includeInstance),\nserviceList: jspb.Message.toObjectList(msg.getServiceList(),\n    proto.google.protobuf.ServiceDescriptorProto.toObject, includeInstance),\nextensionList: jspb.Message.toObjectList(msg.getExtensionList(),\n    proto.google.protobuf.FieldDescriptorProto.toObject, includeInstance),\noptions: (f = msg.getOptions()) && proto.google.protobuf.FileOptions.toObject(includeInstance, f),\nsourceCodeInfo: (f = msg.getSourceCodeInfo()) && proto.google.protobuf.SourceCodeInfo.toObject(includeInstance, f),\nsyntax: (f = jspb.Message.getField(msg, 12)) == null ? undefined : f,\nedition: (f = jspb.Message.getField(msg, 14)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FileDescriptorProto}\n */\nproto.google.protobuf.FileDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FileDescriptorProto;\n  return proto.google.protobuf.FileDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FileDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FileDescriptorProto}\n */\nproto.google.protobuf.FileDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPackage(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addDependency(value);\n      break;\n    case 10:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt32() : [reader.readInt32()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addPublicDependency(values[i]);\n      }\n      break;\n    case 11:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt32() : [reader.readInt32()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addWeakDependency(values[i]);\n      }\n      break;\n    case 4:\n      var value = new proto.google.protobuf.DescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.DescriptorProto.deserializeBinaryFromReader);\n      msg.addMessageType(value);\n      break;\n    case 5:\n      var value = new proto.google.protobuf.EnumDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.EnumDescriptorProto.deserializeBinaryFromReader);\n      msg.addEnumType(value);\n      break;\n    case 6:\n      var value = new proto.google.protobuf.ServiceDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.ServiceDescriptorProto.deserializeBinaryFromReader);\n      msg.addService(value);\n      break;\n    case 7:\n      var value = new proto.google.protobuf.FieldDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.FieldDescriptorProto.deserializeBinaryFromReader);\n      msg.addExtension$(value);\n      break;\n    case 8:\n      var value = new proto.google.protobuf.FileOptions;\n      reader.readMessage(value,proto.google.protobuf.FileOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    case 9:\n      var value = new proto.google.protobuf.SourceCodeInfo;\n      reader.readMessage(value,proto.google.protobuf.SourceCodeInfo.deserializeBinaryFromReader);\n      msg.setSourceCodeInfo(value);\n      break;\n    case 12:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSyntax(value);\n      break;\n    case 14:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setEdition(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FileDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FileDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FileDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getDependencyList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      3,\n      f\n    );\n  }\n  f = message.getPublicDependencyList();\n  if (f.length > 0) {\n    writer.writeRepeatedInt32(\n      10,\n      f\n    );\n  }\n  f = message.getWeakDependencyList();\n  if (f.length > 0) {\n    writer.writeRepeatedInt32(\n      11,\n      f\n    );\n  }\n  f = message.getMessageTypeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      4,\n      f,\n      proto.google.protobuf.DescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getEnumTypeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      5,\n      f,\n      proto.google.protobuf.EnumDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getServiceList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      6,\n      f,\n      proto.google.protobuf.ServiceDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getExtensionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      7,\n      f,\n      proto.google.protobuf.FieldDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.google.protobuf.FileOptions.serializeBinaryToWriter\n    );\n  }\n  f = message.getSourceCodeInfo();\n  if (f != null) {\n    writer.writeMessage(\n      9,\n      f,\n      proto.google.protobuf.SourceCodeInfo.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 12));\n  if (f != null) {\n    writer.writeString(\n      12,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 14));\n  if (f != null) {\n    writer.writeEnum(\n      14,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional string package = 2;\n * @return {string}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getPackage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setPackage = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearPackage = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.hasPackage = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * repeated string dependency = 3;\n * @return {!Array<string>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getDependencyList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 3));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setDependencyList = function(value) {\n  return jspb.Message.setField(this, 3, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addDependency = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearDependencyList = function() {\n  return this.setDependencyList([]);\n};\n\n\n/**\n * repeated int32 public_dependency = 10;\n * @return {!Array<number>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getPublicDependencyList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 10));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setPublicDependencyList = function(value) {\n  return jspb.Message.setField(this, 10, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addPublicDependency = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 10, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearPublicDependencyList = function() {\n  return this.setPublicDependencyList([]);\n};\n\n\n/**\n * repeated int32 weak_dependency = 11;\n * @return {!Array<number>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getWeakDependencyList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 11));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setWeakDependencyList = function(value) {\n  return jspb.Message.setField(this, 11, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addWeakDependency = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 11, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearWeakDependencyList = function() {\n  return this.setWeakDependencyList([]);\n};\n\n\n/**\n * repeated DescriptorProto message_type = 4;\n * @return {!Array<!proto.google.protobuf.DescriptorProto>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getMessageTypeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.DescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.DescriptorProto, 4));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.DescriptorProto>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n*/\nproto.google.protobuf.FileDescriptorProto.prototype.setMessageTypeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 4, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.DescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.DescriptorProto}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addMessageType = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.google.protobuf.DescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearMessageTypeList = function() {\n  return this.setMessageTypeList([]);\n};\n\n\n/**\n * repeated EnumDescriptorProto enum_type = 5;\n * @return {!Array<!proto.google.protobuf.EnumDescriptorProto>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getEnumTypeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.EnumDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.EnumDescriptorProto, 5));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.EnumDescriptorProto>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n*/\nproto.google.protobuf.FileDescriptorProto.prototype.setEnumTypeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 5, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.EnumDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.EnumDescriptorProto}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addEnumType = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.google.protobuf.EnumDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearEnumTypeList = function() {\n  return this.setEnumTypeList([]);\n};\n\n\n/**\n * repeated ServiceDescriptorProto service = 6;\n * @return {!Array<!proto.google.protobuf.ServiceDescriptorProto>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getServiceList = function() {\n  return /** @type{!Array<!proto.google.protobuf.ServiceDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.ServiceDescriptorProto, 6));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.ServiceDescriptorProto>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n*/\nproto.google.protobuf.FileDescriptorProto.prototype.setServiceList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 6, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.ServiceDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.ServiceDescriptorProto}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addService = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 6, opt_value, proto.google.protobuf.ServiceDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearServiceList = function() {\n  return this.setServiceList([]);\n};\n\n\n/**\n * repeated FieldDescriptorProto extension = 7;\n * @return {!Array<!proto.google.protobuf.FieldDescriptorProto>}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getExtensionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.FieldDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.FieldDescriptorProto, 7));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FieldDescriptorProto>} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n*/\nproto.google.protobuf.FileDescriptorProto.prototype.setExtensionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 7, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FieldDescriptorProto}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.addExtension$ = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 7, opt_value, proto.google.protobuf.FieldDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearExtensionList = function() {\n  return this.setExtensionList([]);\n};\n\n\n/**\n * optional FileOptions options = 8;\n * @return {?proto.google.protobuf.FileOptions}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.FileOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FileOptions, 8));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FileOptions|undefined} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n*/\nproto.google.protobuf.FileDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 8, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional SourceCodeInfo source_code_info = 9;\n * @return {?proto.google.protobuf.SourceCodeInfo}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getSourceCodeInfo = function() {\n  return /** @type{?proto.google.protobuf.SourceCodeInfo} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.SourceCodeInfo, 9));\n};\n\n\n/**\n * @param {?proto.google.protobuf.SourceCodeInfo|undefined} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n*/\nproto.google.protobuf.FileDescriptorProto.prototype.setSourceCodeInfo = function(value) {\n  return jspb.Message.setWrapperField(this, 9, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearSourceCodeInfo = function() {\n  return this.setSourceCodeInfo(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.hasSourceCodeInfo = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional string syntax = 12;\n * @return {string}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getSyntax = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setSyntax = function(value) {\n  return jspb.Message.setField(this, 12, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearSyntax = function() {\n  return jspb.Message.setField(this, 12, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.hasSyntax = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional Edition edition = 14;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.getEdition = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 14, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.setEdition = function(value) {\n  return jspb.Message.setField(this, 14, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileDescriptorProto} returns this\n */\nproto.google.protobuf.FileDescriptorProto.prototype.clearEdition = function() {\n  return jspb.Message.setField(this, 14, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileDescriptorProto.prototype.hasEdition = function() {\n  return jspb.Message.getField(this, 14) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.DescriptorProto.repeatedFields_ = [2,6,3,4,5,8,9,10];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.DescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.DescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.DescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.DescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nfieldList: jspb.Message.toObjectList(msg.getFieldList(),\n    proto.google.protobuf.FieldDescriptorProto.toObject, includeInstance),\nextensionList: jspb.Message.toObjectList(msg.getExtensionList(),\n    proto.google.protobuf.FieldDescriptorProto.toObject, includeInstance),\nnestedTypeList: jspb.Message.toObjectList(msg.getNestedTypeList(),\n    proto.google.protobuf.DescriptorProto.toObject, includeInstance),\nenumTypeList: jspb.Message.toObjectList(msg.getEnumTypeList(),\n    proto.google.protobuf.EnumDescriptorProto.toObject, includeInstance),\nextensionRangeList: jspb.Message.toObjectList(msg.getExtensionRangeList(),\n    proto.google.protobuf.DescriptorProto.ExtensionRange.toObject, includeInstance),\noneofDeclList: jspb.Message.toObjectList(msg.getOneofDeclList(),\n    proto.google.protobuf.OneofDescriptorProto.toObject, includeInstance),\noptions: (f = msg.getOptions()) && proto.google.protobuf.MessageOptions.toObject(includeInstance, f),\nreservedRangeList: jspb.Message.toObjectList(msg.getReservedRangeList(),\n    proto.google.protobuf.DescriptorProto.ReservedRange.toObject, includeInstance),\nreservedNameList: (f = jspb.Message.getRepeatedField(msg, 10)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.DescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.DescriptorProto;\n  return proto.google.protobuf.DescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.DescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.DescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = new proto.google.protobuf.FieldDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.FieldDescriptorProto.deserializeBinaryFromReader);\n      msg.addField(value);\n      break;\n    case 6:\n      var value = new proto.google.protobuf.FieldDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.FieldDescriptorProto.deserializeBinaryFromReader);\n      msg.addExtension$(value);\n      break;\n    case 3:\n      var value = new proto.google.protobuf.DescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.DescriptorProto.deserializeBinaryFromReader);\n      msg.addNestedType(value);\n      break;\n    case 4:\n      var value = new proto.google.protobuf.EnumDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.EnumDescriptorProto.deserializeBinaryFromReader);\n      msg.addEnumType(value);\n      break;\n    case 5:\n      var value = new proto.google.protobuf.DescriptorProto.ExtensionRange;\n      reader.readMessage(value,proto.google.protobuf.DescriptorProto.ExtensionRange.deserializeBinaryFromReader);\n      msg.addExtensionRange(value);\n      break;\n    case 8:\n      var value = new proto.google.protobuf.OneofDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.OneofDescriptorProto.deserializeBinaryFromReader);\n      msg.addOneofDecl(value);\n      break;\n    case 7:\n      var value = new proto.google.protobuf.MessageOptions;\n      reader.readMessage(value,proto.google.protobuf.MessageOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    case 9:\n      var value = new proto.google.protobuf.DescriptorProto.ReservedRange;\n      reader.readMessage(value,proto.google.protobuf.DescriptorProto.ReservedRange.deserializeBinaryFromReader);\n      msg.addReservedRange(value);\n      break;\n    case 10:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addReservedName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.DescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.DescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.DescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.DescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = message.getFieldList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.google.protobuf.FieldDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getExtensionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      6,\n      f,\n      proto.google.protobuf.FieldDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getNestedTypeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      3,\n      f,\n      proto.google.protobuf.DescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getEnumTypeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      4,\n      f,\n      proto.google.protobuf.EnumDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getExtensionRangeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      5,\n      f,\n      proto.google.protobuf.DescriptorProto.ExtensionRange.serializeBinaryToWriter\n    );\n  }\n  f = message.getOneofDeclList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      8,\n      f,\n      proto.google.protobuf.OneofDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.google.protobuf.MessageOptions.serializeBinaryToWriter\n    );\n  }\n  f = message.getReservedRangeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      9,\n      f,\n      proto.google.protobuf.DescriptorProto.ReservedRange.serializeBinaryToWriter\n    );\n  }\n  f = message.getReservedNameList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      10,\n      f\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.DescriptorProto.ExtensionRange.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.DescriptorProto.ExtensionRange} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.toObject = function(includeInstance, msg) {\n  var f, obj = {\nstart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nend: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\noptions: (f = msg.getOptions()) && proto.google.protobuf.ExtensionRangeOptions.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.DescriptorProto.ExtensionRange;\n  return proto.google.protobuf.DescriptorProto.ExtensionRange.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.DescriptorProto.ExtensionRange} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setStart(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setEnd(value);\n      break;\n    case 3:\n      var value = new proto.google.protobuf.ExtensionRangeOptions;\n      reader.readMessage(value,proto.google.protobuf.ExtensionRangeOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.DescriptorProto.ExtensionRange.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.DescriptorProto.ExtensionRange} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {number} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeInt32(\n      1,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt32(\n      2,\n      f\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.google.protobuf.ExtensionRangeOptions.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional int32 start = 1;\n * @return {number}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.getStart = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.setStart = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.clearStart = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.hasStart = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional int32 end = 2;\n * @return {number}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.getEnd = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.setEnd = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.clearEnd = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.hasEnd = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional ExtensionRangeOptions options = 3;\n * @return {?proto.google.protobuf.ExtensionRangeOptions}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.ExtensionRangeOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.ExtensionRangeOptions, 3));\n};\n\n\n/**\n * @param {?proto.google.protobuf.ExtensionRangeOptions|undefined} value\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange} returns this\n*/\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.ExtensionRange.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.DescriptorProto.ReservedRange.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.DescriptorProto.ReservedRange} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.toObject = function(includeInstance, msg) {\n  var f, obj = {\nstart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nend: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.DescriptorProto.ReservedRange;\n  return proto.google.protobuf.DescriptorProto.ReservedRange.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.DescriptorProto.ReservedRange} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setStart(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setEnd(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.DescriptorProto.ReservedRange.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.DescriptorProto.ReservedRange} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {number} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeInt32(\n      1,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt32(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional int32 start = 1;\n * @return {number}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.getStart = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.setStart = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.clearStart = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.hasStart = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional int32 end = 2;\n * @return {number}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.getEnd = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.setEnd = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange} returns this\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.clearEnd = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.ReservedRange.prototype.hasEnd = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.DescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * repeated FieldDescriptorProto field = 2;\n * @return {!Array<!proto.google.protobuf.FieldDescriptorProto>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getFieldList = function() {\n  return /** @type{!Array<!proto.google.protobuf.FieldDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.FieldDescriptorProto, 2));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FieldDescriptorProto>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setFieldList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FieldDescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.prototype.addField = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.google.protobuf.FieldDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearFieldList = function() {\n  return this.setFieldList([]);\n};\n\n\n/**\n * repeated FieldDescriptorProto extension = 6;\n * @return {!Array<!proto.google.protobuf.FieldDescriptorProto>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getExtensionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.FieldDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.FieldDescriptorProto, 6));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FieldDescriptorProto>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setExtensionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 6, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FieldDescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.prototype.addExtension$ = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 6, opt_value, proto.google.protobuf.FieldDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearExtensionList = function() {\n  return this.setExtensionList([]);\n};\n\n\n/**\n * repeated DescriptorProto nested_type = 3;\n * @return {!Array<!proto.google.protobuf.DescriptorProto>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getNestedTypeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.DescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.DescriptorProto, 3));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.DescriptorProto>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setNestedTypeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 3, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.DescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.DescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.prototype.addNestedType = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.google.protobuf.DescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearNestedTypeList = function() {\n  return this.setNestedTypeList([]);\n};\n\n\n/**\n * repeated EnumDescriptorProto enum_type = 4;\n * @return {!Array<!proto.google.protobuf.EnumDescriptorProto>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getEnumTypeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.EnumDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.EnumDescriptorProto, 4));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.EnumDescriptorProto>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setEnumTypeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 4, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.EnumDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.EnumDescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.prototype.addEnumType = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.google.protobuf.EnumDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearEnumTypeList = function() {\n  return this.setEnumTypeList([]);\n};\n\n\n/**\n * repeated ExtensionRange extension_range = 5;\n * @return {!Array<!proto.google.protobuf.DescriptorProto.ExtensionRange>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getExtensionRangeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.DescriptorProto.ExtensionRange>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.DescriptorProto.ExtensionRange, 5));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.DescriptorProto.ExtensionRange>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setExtensionRangeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 5, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.DescriptorProto.ExtensionRange=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.DescriptorProto.ExtensionRange}\n */\nproto.google.protobuf.DescriptorProto.prototype.addExtensionRange = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.google.protobuf.DescriptorProto.ExtensionRange, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearExtensionRangeList = function() {\n  return this.setExtensionRangeList([]);\n};\n\n\n/**\n * repeated OneofDescriptorProto oneof_decl = 8;\n * @return {!Array<!proto.google.protobuf.OneofDescriptorProto>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getOneofDeclList = function() {\n  return /** @type{!Array<!proto.google.protobuf.OneofDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.OneofDescriptorProto, 8));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.OneofDescriptorProto>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setOneofDeclList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 8, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.OneofDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.OneofDescriptorProto}\n */\nproto.google.protobuf.DescriptorProto.prototype.addOneofDecl = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 8, opt_value, proto.google.protobuf.OneofDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearOneofDeclList = function() {\n  return this.setOneofDeclList([]);\n};\n\n\n/**\n * optional MessageOptions options = 7;\n * @return {?proto.google.protobuf.MessageOptions}\n */\nproto.google.protobuf.DescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.MessageOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.MessageOptions, 7));\n};\n\n\n/**\n * @param {?proto.google.protobuf.MessageOptions|undefined} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 7, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.DescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * repeated ReservedRange reserved_range = 9;\n * @return {!Array<!proto.google.protobuf.DescriptorProto.ReservedRange>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getReservedRangeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.DescriptorProto.ReservedRange>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.DescriptorProto.ReservedRange, 9));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.DescriptorProto.ReservedRange>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n*/\nproto.google.protobuf.DescriptorProto.prototype.setReservedRangeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 9, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.DescriptorProto.ReservedRange=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.DescriptorProto.ReservedRange}\n */\nproto.google.protobuf.DescriptorProto.prototype.addReservedRange = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 9, opt_value, proto.google.protobuf.DescriptorProto.ReservedRange, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearReservedRangeList = function() {\n  return this.setReservedRangeList([]);\n};\n\n\n/**\n * repeated string reserved_name = 10;\n * @return {!Array<string>}\n */\nproto.google.protobuf.DescriptorProto.prototype.getReservedNameList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 10));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.setReservedNameList = function(value) {\n  return jspb.Message.setField(this, 10, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.addReservedName = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 10, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.DescriptorProto} returns this\n */\nproto.google.protobuf.DescriptorProto.prototype.clearReservedNameList = function() {\n  return this.setReservedNameList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.ExtensionRangeOptions.repeatedFields_ = [999,2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.ExtensionRangeOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.ExtensionRangeOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ExtensionRangeOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance),\ndeclarationList: jspb.Message.toObjectList(msg.getDeclarationList(),\n    proto.google.protobuf.ExtensionRangeOptions.Declaration.toObject, includeInstance),\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nverification: jspb.Message.getFieldWithDefault(msg, 3, 1)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.ExtensionRangeOptions.extensions, proto.google.protobuf.ExtensionRangeOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.ExtensionRangeOptions}\n */\nproto.google.protobuf.ExtensionRangeOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.ExtensionRangeOptions;\n  return proto.google.protobuf.ExtensionRangeOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.ExtensionRangeOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.ExtensionRangeOptions}\n */\nproto.google.protobuf.ExtensionRangeOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    case 2:\n      var value = new proto.google.protobuf.ExtensionRangeOptions.Declaration;\n      reader.readMessage(value,proto.google.protobuf.ExtensionRangeOptions.Declaration.deserializeBinaryFromReader);\n      msg.addDeclaration(value);\n      break;\n    case 50:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.google.protobuf.ExtensionRangeOptions.VerificationState} */ (reader.readEnum());\n      msg.setVerification(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.ExtensionRangeOptions.extensionsBinary,\n        proto.google.protobuf.ExtensionRangeOptions.prototype.getExtension,\n        proto.google.protobuf.ExtensionRangeOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.ExtensionRangeOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.ExtensionRangeOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ExtensionRangeOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  f = message.getDeclarationList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.google.protobuf.ExtensionRangeOptions.Declaration.serializeBinaryToWriter\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      50,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {!proto.google.protobuf.ExtensionRangeOptions.VerificationState} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.ExtensionRangeOptions.extensionsBinary, proto.google.protobuf.ExtensionRangeOptions.prototype.getExtension);\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.ExtensionRangeOptions.VerificationState = {\n  DECLARATION: 0,\n  UNVERIFIED: 1\n};\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.ExtensionRangeOptions.Declaration.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.ExtensionRangeOptions.Declaration} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.toObject = function(includeInstance, msg) {\n  var f, obj = {\nnumber: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nfullName: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\ntype: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\nreserved: (f = jspb.Message.getBooleanField(msg, 5)) == null ? undefined : f,\nrepeated: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.ExtensionRangeOptions.Declaration;\n  return proto.google.protobuf.ExtensionRangeOptions.Declaration.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.ExtensionRangeOptions.Declaration} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setNumber(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setFullName(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setType(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setReserved(value);\n      break;\n    case 6:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setRepeated(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.ExtensionRangeOptions.Declaration.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.ExtensionRangeOptions.Declaration} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {number} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeInt32(\n      1,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeBool(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional int32 number = 1;\n * @return {number}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.getNumber = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.setNumber = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.clearNumber = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.hasNumber = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional string full_name = 2;\n * @return {string}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.getFullName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.setFullName = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.clearFullName = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.hasFullName = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string type = 3;\n * @return {string}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.getType = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.setType = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.clearType = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.hasType = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool reserved = 5;\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.getReserved = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.setReserved = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.clearReserved = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.hasReserved = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional bool repeated = 6;\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.getRepeated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.setRepeated = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.clearRepeated = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.Declaration.prototype.hasRepeated = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n*/\nproto.google.protobuf.ExtensionRangeOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n/**\n * repeated Declaration declaration = 2;\n * @return {!Array<!proto.google.protobuf.ExtensionRangeOptions.Declaration>}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.getDeclarationList = function() {\n  return /** @type{!Array<!proto.google.protobuf.ExtensionRangeOptions.Declaration>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.ExtensionRangeOptions.Declaration, 2));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.ExtensionRangeOptions.Declaration>} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n*/\nproto.google.protobuf.ExtensionRangeOptions.prototype.setDeclarationList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.ExtensionRangeOptions.Declaration=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.ExtensionRangeOptions.Declaration}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.addDeclaration = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.google.protobuf.ExtensionRangeOptions.Declaration, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.clearDeclarationList = function() {\n  return this.setDeclarationList([]);\n};\n\n\n/**\n * optional FeatureSet features = 50;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 50));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n*/\nproto.google.protobuf.ExtensionRangeOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 50, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 50) != null;\n};\n\n\n/**\n * optional VerificationState verification = 3;\n * @return {!proto.google.protobuf.ExtensionRangeOptions.VerificationState}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.getVerification = function() {\n  return /** @type {!proto.google.protobuf.ExtensionRangeOptions.VerificationState} */ (jspb.Message.getFieldWithDefault(this, 3, 1));\n};\n\n\n/**\n * @param {!proto.google.protobuf.ExtensionRangeOptions.VerificationState} value\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.setVerification = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ExtensionRangeOptions} returns this\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.clearVerification = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ExtensionRangeOptions.prototype.hasVerification = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FieldDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FieldDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nnumber: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\nlabel: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,\ntype: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,\ntypeName: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f,\nextendee: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\ndefaultValue: (f = jspb.Message.getField(msg, 7)) == null ? undefined : f,\noneofIndex: (f = jspb.Message.getField(msg, 9)) == null ? undefined : f,\njsonName: (f = jspb.Message.getField(msg, 10)) == null ? undefined : f,\noptions: (f = msg.getOptions()) && proto.google.protobuf.FieldOptions.toObject(includeInstance, f),\nproto3Optional: (f = jspb.Message.getBooleanField(msg, 17)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FieldDescriptorProto}\n */\nproto.google.protobuf.FieldDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FieldDescriptorProto;\n  return proto.google.protobuf.FieldDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FieldDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FieldDescriptorProto}\n */\nproto.google.protobuf.FieldDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setNumber(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.google.protobuf.FieldDescriptorProto.Label} */ (reader.readEnum());\n      msg.setLabel(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.google.protobuf.FieldDescriptorProto.Type} */ (reader.readEnum());\n      msg.setType(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTypeName(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setExtendee(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDefaultValue(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setOneofIndex(value);\n      break;\n    case 10:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setJsonName(value);\n      break;\n    case 8:\n      var value = new proto.google.protobuf.FieldOptions;\n      reader.readMessage(value,proto.google.protobuf.FieldOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    case 17:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setProto3Optional(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FieldDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FieldDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt32(\n      3,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FieldDescriptorProto.Label} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FieldDescriptorProto.Type} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeInt32(\n      9,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeString(\n      10,\n      f\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.google.protobuf.FieldOptions.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 17));\n  if (f != null) {\n    writer.writeBool(\n      17,\n      f\n    );\n  }\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FieldDescriptorProto.Type = {\n  TYPE_DOUBLE: 1,\n  TYPE_FLOAT: 2,\n  TYPE_INT64: 3,\n  TYPE_UINT64: 4,\n  TYPE_INT32: 5,\n  TYPE_FIXED64: 6,\n  TYPE_FIXED32: 7,\n  TYPE_BOOL: 8,\n  TYPE_STRING: 9,\n  TYPE_GROUP: 10,\n  TYPE_MESSAGE: 11,\n  TYPE_BYTES: 12,\n  TYPE_UINT32: 13,\n  TYPE_ENUM: 14,\n  TYPE_SFIXED32: 15,\n  TYPE_SFIXED64: 16,\n  TYPE_SINT32: 17,\n  TYPE_SINT64: 18\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FieldDescriptorProto.Label = {\n  LABEL_OPTIONAL: 1,\n  LABEL_REPEATED: 3,\n  LABEL_REQUIRED: 2\n};\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional int32 number = 3;\n * @return {number}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getNumber = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setNumber = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearNumber = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasNumber = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional Label label = 4;\n * @return {!proto.google.protobuf.FieldDescriptorProto.Label}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getLabel = function() {\n  return /** @type {!proto.google.protobuf.FieldDescriptorProto.Label} */ (jspb.Message.getFieldWithDefault(this, 4, 1));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldDescriptorProto.Label} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional Type type = 5;\n * @return {!proto.google.protobuf.FieldDescriptorProto.Type}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getType = function() {\n  return /** @type {!proto.google.protobuf.FieldDescriptorProto.Type} */ (jspb.Message.getFieldWithDefault(this, 5, 1));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldDescriptorProto.Type} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setType = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearType = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasType = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string type_name = 6;\n * @return {string}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getTypeName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setTypeName = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearTypeName = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasTypeName = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string extendee = 2;\n * @return {string}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getExtendee = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setExtendee = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearExtendee = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasExtendee = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string default_value = 7;\n * @return {string}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getDefaultValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setDefaultValue = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearDefaultValue = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasDefaultValue = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional int32 oneof_index = 9;\n * @return {number}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getOneofIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 9, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setOneofIndex = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearOneofIndex = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasOneofIndex = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional string json_name = 10;\n * @return {string}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getJsonName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setJsonName = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearJsonName = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasJsonName = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional FieldOptions options = 8;\n * @return {?proto.google.protobuf.FieldOptions}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.FieldOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FieldOptions, 8));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FieldOptions|undefined} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n*/\nproto.google.protobuf.FieldDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 8, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional bool proto3_optional = 17;\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.getProto3Optional = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 17, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.setProto3Optional = function(value) {\n  return jspb.Message.setField(this, 17, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldDescriptorProto} returns this\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.clearProto3Optional = function() {\n  return jspb.Message.setField(this, 17, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldDescriptorProto.prototype.hasProto3Optional = function() {\n  return jspb.Message.getField(this, 17) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.OneofDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.OneofDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.OneofDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\noptions: (f = msg.getOptions()) && proto.google.protobuf.OneofOptions.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.OneofDescriptorProto}\n */\nproto.google.protobuf.OneofDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.OneofDescriptorProto;\n  return proto.google.protobuf.OneofDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.OneofDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.OneofDescriptorProto}\n */\nproto.google.protobuf.OneofDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = new proto.google.protobuf.OneofOptions;\n      reader.readMessage(value,proto.google.protobuf.OneofOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.OneofDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.OneofDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.OneofDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.google.protobuf.OneofOptions.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.OneofDescriptorProto} returns this\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.OneofDescriptorProto} returns this\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional OneofOptions options = 2;\n * @return {?proto.google.protobuf.OneofOptions}\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.OneofOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.OneofOptions, 2));\n};\n\n\n/**\n * @param {?proto.google.protobuf.OneofOptions|undefined} value\n * @return {!proto.google.protobuf.OneofDescriptorProto} returns this\n*/\nproto.google.protobuf.OneofDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.OneofDescriptorProto} returns this\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.OneofDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.EnumDescriptorProto.repeatedFields_ = [2,4,5];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.EnumDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.EnumDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nvalueList: jspb.Message.toObjectList(msg.getValueList(),\n    proto.google.protobuf.EnumValueDescriptorProto.toObject, includeInstance),\noptions: (f = msg.getOptions()) && proto.google.protobuf.EnumOptions.toObject(includeInstance, f),\nreservedRangeList: jspb.Message.toObjectList(msg.getReservedRangeList(),\n    proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.toObject, includeInstance),\nreservedNameList: (f = jspb.Message.getRepeatedField(msg, 5)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.EnumDescriptorProto}\n */\nproto.google.protobuf.EnumDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.EnumDescriptorProto;\n  return proto.google.protobuf.EnumDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.EnumDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.EnumDescriptorProto}\n */\nproto.google.protobuf.EnumDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = new proto.google.protobuf.EnumValueDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.EnumValueDescriptorProto.deserializeBinaryFromReader);\n      msg.addValue(value);\n      break;\n    case 3:\n      var value = new proto.google.protobuf.EnumOptions;\n      reader.readMessage(value,proto.google.protobuf.EnumOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    case 4:\n      var value = new proto.google.protobuf.EnumDescriptorProto.EnumReservedRange;\n      reader.readMessage(value,proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.deserializeBinaryFromReader);\n      msg.addReservedRange(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addReservedName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.EnumDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.EnumDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = message.getValueList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.google.protobuf.EnumValueDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.google.protobuf.EnumOptions.serializeBinaryToWriter\n    );\n  }\n  f = message.getReservedRangeList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      4,\n      f,\n      proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.serializeBinaryToWriter\n    );\n  }\n  f = message.getReservedNameList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      5,\n      f\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.toObject = function(includeInstance, msg) {\n  var f, obj = {\nstart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nend: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.EnumDescriptorProto.EnumReservedRange;\n  return proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setStart(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setEnd(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.EnumDescriptorProto.EnumReservedRange.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {number} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeInt32(\n      1,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt32(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional int32 start = 1;\n * @return {number}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.getStart = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.setStart = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.clearStart = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.hasStart = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional int32 end = 2;\n * @return {number}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.getEnd = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.setEnd = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.clearEnd = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumDescriptorProto.EnumReservedRange.prototype.hasEnd = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * repeated EnumValueDescriptorProto value = 2;\n * @return {!Array<!proto.google.protobuf.EnumValueDescriptorProto>}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.getValueList = function() {\n  return /** @type{!Array<!proto.google.protobuf.EnumValueDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.EnumValueDescriptorProto, 2));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.EnumValueDescriptorProto>} value\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n*/\nproto.google.protobuf.EnumDescriptorProto.prototype.setValueList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.EnumValueDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.EnumValueDescriptorProto}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.addValue = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.google.protobuf.EnumValueDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.clearValueList = function() {\n  return this.setValueList([]);\n};\n\n\n/**\n * optional EnumOptions options = 3;\n * @return {?proto.google.protobuf.EnumOptions}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.EnumOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.EnumOptions, 3));\n};\n\n\n/**\n * @param {?proto.google.protobuf.EnumOptions|undefined} value\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n*/\nproto.google.protobuf.EnumDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * repeated EnumReservedRange reserved_range = 4;\n * @return {!Array<!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange>}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.getReservedRangeList = function() {\n  return /** @type{!Array<!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.EnumDescriptorProto.EnumReservedRange, 4));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange>} value\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n*/\nproto.google.protobuf.EnumDescriptorProto.prototype.setReservedRangeList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 4, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.EnumDescriptorProto.EnumReservedRange}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.addReservedRange = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 4, opt_value, proto.google.protobuf.EnumDescriptorProto.EnumReservedRange, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.clearReservedRangeList = function() {\n  return this.setReservedRangeList([]);\n};\n\n\n/**\n * repeated string reserved_name = 5;\n * @return {!Array<string>}\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.getReservedNameList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 5));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.setReservedNameList = function(value) {\n  return jspb.Message.setField(this, 5, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.addReservedName = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 5, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.EnumDescriptorProto} returns this\n */\nproto.google.protobuf.EnumDescriptorProto.prototype.clearReservedNameList = function() {\n  return this.setReservedNameList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.EnumValueDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.EnumValueDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumValueDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nnumber: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\noptions: (f = msg.getOptions()) && proto.google.protobuf.EnumValueOptions.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.EnumValueDescriptorProto}\n */\nproto.google.protobuf.EnumValueDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.EnumValueDescriptorProto;\n  return proto.google.protobuf.EnumValueDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.EnumValueDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.EnumValueDescriptorProto}\n */\nproto.google.protobuf.EnumValueDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setNumber(value);\n      break;\n    case 3:\n      var value = new proto.google.protobuf.EnumValueOptions;\n      reader.readMessage(value,proto.google.protobuf.EnumValueOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.EnumValueDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.EnumValueDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumValueDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt32(\n      2,\n      f\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.google.protobuf.EnumValueOptions.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.EnumValueDescriptorProto} returns this\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumValueDescriptorProto} returns this\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional int32 number = 2;\n * @return {number}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.getNumber = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.EnumValueDescriptorProto} returns this\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.setNumber = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumValueDescriptorProto} returns this\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.clearNumber = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.hasNumber = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional EnumValueOptions options = 3;\n * @return {?proto.google.protobuf.EnumValueOptions}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.EnumValueOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.EnumValueOptions, 3));\n};\n\n\n/**\n * @param {?proto.google.protobuf.EnumValueOptions|undefined} value\n * @return {!proto.google.protobuf.EnumValueDescriptorProto} returns this\n*/\nproto.google.protobuf.EnumValueDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.EnumValueDescriptorProto} returns this\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.ServiceDescriptorProto.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.ServiceDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.ServiceDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ServiceDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nmethodList: jspb.Message.toObjectList(msg.getMethodList(),\n    proto.google.protobuf.MethodDescriptorProto.toObject, includeInstance),\noptions: (f = msg.getOptions()) && proto.google.protobuf.ServiceOptions.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.ServiceDescriptorProto}\n */\nproto.google.protobuf.ServiceDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.ServiceDescriptorProto;\n  return proto.google.protobuf.ServiceDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.ServiceDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.ServiceDescriptorProto}\n */\nproto.google.protobuf.ServiceDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = new proto.google.protobuf.MethodDescriptorProto;\n      reader.readMessage(value,proto.google.protobuf.MethodDescriptorProto.deserializeBinaryFromReader);\n      msg.addMethod(value);\n      break;\n    case 3:\n      var value = new proto.google.protobuf.ServiceOptions;\n      reader.readMessage(value,proto.google.protobuf.ServiceOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.ServiceDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.ServiceDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ServiceDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = message.getMethodList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.google.protobuf.MethodDescriptorProto.serializeBinaryToWriter\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.google.protobuf.ServiceOptions.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.ServiceDescriptorProto} returns this\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ServiceDescriptorProto} returns this\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * repeated MethodDescriptorProto method = 2;\n * @return {!Array<!proto.google.protobuf.MethodDescriptorProto>}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.getMethodList = function() {\n  return /** @type{!Array<!proto.google.protobuf.MethodDescriptorProto>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.MethodDescriptorProto, 2));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.MethodDescriptorProto>} value\n * @return {!proto.google.protobuf.ServiceDescriptorProto} returns this\n*/\nproto.google.protobuf.ServiceDescriptorProto.prototype.setMethodList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.MethodDescriptorProto=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.MethodDescriptorProto}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.addMethod = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.google.protobuf.MethodDescriptorProto, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.ServiceDescriptorProto} returns this\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.clearMethodList = function() {\n  return this.setMethodList([]);\n};\n\n\n/**\n * optional ServiceOptions options = 3;\n * @return {?proto.google.protobuf.ServiceOptions}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.ServiceOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.ServiceOptions, 3));\n};\n\n\n/**\n * @param {?proto.google.protobuf.ServiceOptions|undefined} value\n * @return {!proto.google.protobuf.ServiceDescriptorProto} returns this\n*/\nproto.google.protobuf.ServiceDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.ServiceDescriptorProto} returns this\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ServiceDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.MethodDescriptorProto.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.MethodDescriptorProto} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.MethodDescriptorProto.toObject = function(includeInstance, msg) {\n  var f, obj = {\nname: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\ninputType: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\noutputType: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\noptions: (f = msg.getOptions()) && proto.google.protobuf.MethodOptions.toObject(includeInstance, f),\nclientStreaming: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\nserverStreaming: jspb.Message.getBooleanFieldWithDefault(msg, 6, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.MethodDescriptorProto}\n */\nproto.google.protobuf.MethodDescriptorProto.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.MethodDescriptorProto;\n  return proto.google.protobuf.MethodDescriptorProto.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.MethodDescriptorProto} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.MethodDescriptorProto}\n */\nproto.google.protobuf.MethodDescriptorProto.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setInputType(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setOutputType(value);\n      break;\n    case 4:\n      var value = new proto.google.protobuf.MethodOptions;\n      reader.readMessage(value,proto.google.protobuf.MethodOptions.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setClientStreaming(value);\n      break;\n    case 6:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setServerStreaming(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.MethodDescriptorProto.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.MethodDescriptorProto} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.MethodDescriptorProto.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.google.protobuf.MethodOptions.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeBool(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string name = 1;\n * @return {string}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.clearName = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.hasName = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional string input_type = 2;\n * @return {string}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.getInputType = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.setInputType = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.clearInputType = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.hasInputType = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string output_type = 3;\n * @return {string}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.getOutputType = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.setOutputType = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.clearOutputType = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.hasOutputType = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional MethodOptions options = 4;\n * @return {?proto.google.protobuf.MethodOptions}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.getOptions = function() {\n  return /** @type{?proto.google.protobuf.MethodOptions} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.MethodOptions, 4));\n};\n\n\n/**\n * @param {?proto.google.protobuf.MethodOptions|undefined} value\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n*/\nproto.google.protobuf.MethodDescriptorProto.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool client_streaming = 5;\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.getClientStreaming = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.setClientStreaming = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.clearClientStreaming = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.hasClientStreaming = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional bool server_streaming = 6;\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.getServerStreaming = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.setServerStreaming = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodDescriptorProto} returns this\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.clearServerStreaming = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodDescriptorProto.prototype.hasServerStreaming = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.FileOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FileOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FileOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FileOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FileOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\njavaPackage: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\njavaOuterClassname: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f,\njavaMultipleFiles: jspb.Message.getBooleanFieldWithDefault(msg, 10, false),\njavaGenerateEqualsAndHash: (f = jspb.Message.getBooleanField(msg, 20)) == null ? undefined : f,\njavaStringCheckUtf8: jspb.Message.getBooleanFieldWithDefault(msg, 27, false),\noptimizeFor: jspb.Message.getFieldWithDefault(msg, 9, 1),\ngoPackage: (f = jspb.Message.getField(msg, 11)) == null ? undefined : f,\nccGenericServices: jspb.Message.getBooleanFieldWithDefault(msg, 16, false),\njavaGenericServices: jspb.Message.getBooleanFieldWithDefault(msg, 17, false),\npyGenericServices: jspb.Message.getBooleanFieldWithDefault(msg, 18, false),\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 23, false),\nccEnableArenas: jspb.Message.getBooleanFieldWithDefault(msg, 31, true),\nobjcClassPrefix: (f = jspb.Message.getField(msg, 36)) == null ? undefined : f,\ncsharpNamespace: (f = jspb.Message.getField(msg, 37)) == null ? undefined : f,\nswiftPrefix: (f = jspb.Message.getField(msg, 39)) == null ? undefined : f,\nphpClassPrefix: (f = jspb.Message.getField(msg, 40)) == null ? undefined : f,\nphpNamespace: (f = jspb.Message.getField(msg, 41)) == null ? undefined : f,\nphpMetadataNamespace: (f = jspb.Message.getField(msg, 44)) == null ? undefined : f,\nrubyPackage: (f = jspb.Message.getField(msg, 45)) == null ? undefined : f,\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.FileOptions.extensions, proto.google.protobuf.FileOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FileOptions}\n */\nproto.google.protobuf.FileOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FileOptions;\n  return proto.google.protobuf.FileOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FileOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FileOptions}\n */\nproto.google.protobuf.FileOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setJavaPackage(value);\n      break;\n    case 8:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setJavaOuterClassname(value);\n      break;\n    case 10:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setJavaMultipleFiles(value);\n      break;\n    case 20:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setJavaGenerateEqualsAndHash(value);\n      break;\n    case 27:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setJavaStringCheckUtf8(value);\n      break;\n    case 9:\n      var value = /** @type {!proto.google.protobuf.FileOptions.OptimizeMode} */ (reader.readEnum());\n      msg.setOptimizeFor(value);\n      break;\n    case 11:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setGoPackage(value);\n      break;\n    case 16:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setCcGenericServices(value);\n      break;\n    case 17:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setJavaGenericServices(value);\n      break;\n    case 18:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setPyGenericServices(value);\n      break;\n    case 23:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 31:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setCcEnableArenas(value);\n      break;\n    case 36:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setObjcClassPrefix(value);\n      break;\n    case 37:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setCsharpNamespace(value);\n      break;\n    case 39:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSwiftPrefix(value);\n      break;\n    case 40:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPhpClassPrefix(value);\n      break;\n    case 41:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPhpNamespace(value);\n      break;\n    case 44:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPhpMetadataNamespace(value);\n      break;\n    case 45:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setRubyPackage(value);\n      break;\n    case 50:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.FileOptions.extensionsBinary,\n        proto.google.protobuf.FileOptions.prototype.getExtension,\n        proto.google.protobuf.FileOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FileOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FileOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FileOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FileOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeString(\n      8,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeBool(\n      10,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 20));\n  if (f != null) {\n    writer.writeBool(\n      20,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 27));\n  if (f != null) {\n    writer.writeBool(\n      27,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FileOptions.OptimizeMode} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeEnum(\n      9,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeString(\n      11,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 16));\n  if (f != null) {\n    writer.writeBool(\n      16,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 17));\n  if (f != null) {\n    writer.writeBool(\n      17,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 18));\n  if (f != null) {\n    writer.writeBool(\n      18,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 23));\n  if (f != null) {\n    writer.writeBool(\n      23,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 31));\n  if (f != null) {\n    writer.writeBool(\n      31,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 36));\n  if (f != null) {\n    writer.writeString(\n      36,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 37));\n  if (f != null) {\n    writer.writeString(\n      37,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 39));\n  if (f != null) {\n    writer.writeString(\n      39,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 40));\n  if (f != null) {\n    writer.writeString(\n      40,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 41));\n  if (f != null) {\n    writer.writeString(\n      41,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 44));\n  if (f != null) {\n    writer.writeString(\n      44,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 45));\n  if (f != null) {\n    writer.writeString(\n      45,\n      f\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      50,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.FileOptions.extensionsBinary, proto.google.protobuf.FileOptions.prototype.getExtension);\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FileOptions.OptimizeMode = {\n  SPEED: 1,\n  CODE_SIZE: 2,\n  LITE_RUNTIME: 3\n};\n\n/**\n * optional string java_package = 1;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getJavaPackage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setJavaPackage = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearJavaPackage = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasJavaPackage = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional string java_outer_classname = 8;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getJavaOuterClassname = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setJavaOuterClassname = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearJavaOuterClassname = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasJavaOuterClassname = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional bool java_multiple_files = 10;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getJavaMultipleFiles = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 10, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setJavaMultipleFiles = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearJavaMultipleFiles = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasJavaMultipleFiles = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional bool java_generate_equals_and_hash = 20;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getJavaGenerateEqualsAndHash = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 20, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setJavaGenerateEqualsAndHash = function(value) {\n  return jspb.Message.setField(this, 20, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearJavaGenerateEqualsAndHash = function() {\n  return jspb.Message.setField(this, 20, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasJavaGenerateEqualsAndHash = function() {\n  return jspb.Message.getField(this, 20) != null;\n};\n\n\n/**\n * optional bool java_string_check_utf8 = 27;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getJavaStringCheckUtf8 = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 27, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setJavaStringCheckUtf8 = function(value) {\n  return jspb.Message.setField(this, 27, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearJavaStringCheckUtf8 = function() {\n  return jspb.Message.setField(this, 27, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasJavaStringCheckUtf8 = function() {\n  return jspb.Message.getField(this, 27) != null;\n};\n\n\n/**\n * optional OptimizeMode optimize_for = 9;\n * @return {!proto.google.protobuf.FileOptions.OptimizeMode}\n */\nproto.google.protobuf.FileOptions.prototype.getOptimizeFor = function() {\n  return /** @type {!proto.google.protobuf.FileOptions.OptimizeMode} */ (jspb.Message.getFieldWithDefault(this, 9, 1));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FileOptions.OptimizeMode} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setOptimizeFor = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearOptimizeFor = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasOptimizeFor = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional string go_package = 11;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getGoPackage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 11, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setGoPackage = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearGoPackage = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasGoPackage = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional bool cc_generic_services = 16;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getCcGenericServices = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 16, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setCcGenericServices = function(value) {\n  return jspb.Message.setField(this, 16, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearCcGenericServices = function() {\n  return jspb.Message.setField(this, 16, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasCcGenericServices = function() {\n  return jspb.Message.getField(this, 16) != null;\n};\n\n\n/**\n * optional bool java_generic_services = 17;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getJavaGenericServices = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 17, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setJavaGenericServices = function(value) {\n  return jspb.Message.setField(this, 17, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearJavaGenericServices = function() {\n  return jspb.Message.setField(this, 17, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasJavaGenericServices = function() {\n  return jspb.Message.getField(this, 17) != null;\n};\n\n\n/**\n * optional bool py_generic_services = 18;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getPyGenericServices = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 18, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setPyGenericServices = function(value) {\n  return jspb.Message.setField(this, 18, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearPyGenericServices = function() {\n  return jspb.Message.setField(this, 18, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasPyGenericServices = function() {\n  return jspb.Message.getField(this, 18) != null;\n};\n\n\n/**\n * optional bool deprecated = 23;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 23, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 23, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 23, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 23) != null;\n};\n\n\n/**\n * optional bool cc_enable_arenas = 31;\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.getCcEnableArenas = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 31, true));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setCcEnableArenas = function(value) {\n  return jspb.Message.setField(this, 31, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearCcEnableArenas = function() {\n  return jspb.Message.setField(this, 31, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasCcEnableArenas = function() {\n  return jspb.Message.getField(this, 31) != null;\n};\n\n\n/**\n * optional string objc_class_prefix = 36;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getObjcClassPrefix = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 36, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setObjcClassPrefix = function(value) {\n  return jspb.Message.setField(this, 36, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearObjcClassPrefix = function() {\n  return jspb.Message.setField(this, 36, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasObjcClassPrefix = function() {\n  return jspb.Message.getField(this, 36) != null;\n};\n\n\n/**\n * optional string csharp_namespace = 37;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getCsharpNamespace = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 37, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setCsharpNamespace = function(value) {\n  return jspb.Message.setField(this, 37, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearCsharpNamespace = function() {\n  return jspb.Message.setField(this, 37, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasCsharpNamespace = function() {\n  return jspb.Message.getField(this, 37) != null;\n};\n\n\n/**\n * optional string swift_prefix = 39;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getSwiftPrefix = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 39, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setSwiftPrefix = function(value) {\n  return jspb.Message.setField(this, 39, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearSwiftPrefix = function() {\n  return jspb.Message.setField(this, 39, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasSwiftPrefix = function() {\n  return jspb.Message.getField(this, 39) != null;\n};\n\n\n/**\n * optional string php_class_prefix = 40;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getPhpClassPrefix = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 40, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setPhpClassPrefix = function(value) {\n  return jspb.Message.setField(this, 40, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearPhpClassPrefix = function() {\n  return jspb.Message.setField(this, 40, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasPhpClassPrefix = function() {\n  return jspb.Message.getField(this, 40) != null;\n};\n\n\n/**\n * optional string php_namespace = 41;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getPhpNamespace = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 41, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setPhpNamespace = function(value) {\n  return jspb.Message.setField(this, 41, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearPhpNamespace = function() {\n  return jspb.Message.setField(this, 41, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasPhpNamespace = function() {\n  return jspb.Message.getField(this, 41) != null;\n};\n\n\n/**\n * optional string php_metadata_namespace = 44;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getPhpMetadataNamespace = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 44, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setPhpMetadataNamespace = function(value) {\n  return jspb.Message.setField(this, 44, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearPhpMetadataNamespace = function() {\n  return jspb.Message.setField(this, 44, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasPhpMetadataNamespace = function() {\n  return jspb.Message.getField(this, 44) != null;\n};\n\n\n/**\n * optional string ruby_package = 45;\n * @return {string}\n */\nproto.google.protobuf.FileOptions.prototype.getRubyPackage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 45, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.setRubyPackage = function(value) {\n  return jspb.Message.setField(this, 45, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearRubyPackage = function() {\n  return jspb.Message.setField(this, 45, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasRubyPackage = function() {\n  return jspb.Message.getField(this, 45) != null;\n};\n\n\n/**\n * optional FeatureSet features = 50;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.FileOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 50));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n*/\nproto.google.protobuf.FileOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 50, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FileOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 50) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.FileOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.FileOptions} returns this\n*/\nproto.google.protobuf.FileOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.FileOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FileOptions} returns this\n */\nproto.google.protobuf.FileOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.MessageOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.MessageOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.MessageOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.MessageOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.MessageOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\nmessageSetWireFormat: jspb.Message.getBooleanFieldWithDefault(msg, 1, false),\nnoStandardDescriptorAccessor: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\nmapEntry: (f = jspb.Message.getBooleanField(msg, 7)) == null ? undefined : f,\ndeprecatedLegacyJsonFieldConflicts: (f = jspb.Message.getBooleanField(msg, 11)) == null ? undefined : f,\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.MessageOptions.extensions, proto.google.protobuf.MessageOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.MessageOptions}\n */\nproto.google.protobuf.MessageOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.MessageOptions;\n  return proto.google.protobuf.MessageOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.MessageOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.MessageOptions}\n */\nproto.google.protobuf.MessageOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setMessageSetWireFormat(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setNoStandardDescriptorAccessor(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 7:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setMapEntry(value);\n      break;\n    case 11:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecatedLegacyJsonFieldConflicts(value);\n      break;\n    case 12:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.MessageOptions.extensionsBinary,\n        proto.google.protobuf.MessageOptions.prototype.getExtension,\n        proto.google.protobuf.MessageOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.MessageOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.MessageOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.MessageOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.MessageOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeBool(\n      1,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeBool(\n      7,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeBool(\n      11,\n      f\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      12,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.MessageOptions.extensionsBinary, proto.google.protobuf.MessageOptions.prototype.getExtension);\n};\n\n\n/**\n * optional bool message_set_wire_format = 1;\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.getMessageSetWireFormat = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 1, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.setMessageSetWireFormat = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearMessageSetWireFormat = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.hasMessageSetWireFormat = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional bool no_standard_descriptor_accessor = 2;\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.getNoStandardDescriptorAccessor = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.setNoStandardDescriptorAccessor = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearNoStandardDescriptorAccessor = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.hasNoStandardDescriptorAccessor = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool deprecated = 3;\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool map_entry = 7;\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.getMapEntry = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.setMapEntry = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearMapEntry = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.hasMapEntry = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional bool deprecated_legacy_json_field_conflicts = 11;\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.getDeprecatedLegacyJsonFieldConflicts = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.setDeprecatedLegacyJsonFieldConflicts = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearDeprecatedLegacyJsonFieldConflicts = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.hasDeprecatedLegacyJsonFieldConflicts = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional FeatureSet features = 12;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.MessageOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 12));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n*/\nproto.google.protobuf.MessageOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 12, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MessageOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.MessageOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.MessageOptions} returns this\n*/\nproto.google.protobuf.MessageOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.MessageOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.MessageOptions} returns this\n */\nproto.google.protobuf.MessageOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.FieldOptions.repeatedFields_ = [19,20,999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FieldOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FieldOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FieldOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\nctype: jspb.Message.getFieldWithDefault(msg, 1, 0),\npacked: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f,\njstype: jspb.Message.getFieldWithDefault(msg, 6, 0),\nlazy: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\nunverifiedLazy: jspb.Message.getBooleanFieldWithDefault(msg, 15, false),\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\nweak: jspb.Message.getBooleanFieldWithDefault(msg, 10, false),\ndebugRedact: jspb.Message.getBooleanFieldWithDefault(msg, 16, false),\nretention: (f = jspb.Message.getField(msg, 17)) == null ? undefined : f,\ntargetsList: (f = jspb.Message.getRepeatedField(msg, 19)) == null ? undefined : f,\neditionDefaultsList: jspb.Message.toObjectList(msg.getEditionDefaultsList(),\n    proto.google.protobuf.FieldOptions.EditionDefault.toObject, includeInstance),\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nfeatureSupport: (f = msg.getFeatureSupport()) && proto.google.protobuf.FieldOptions.FeatureSupport.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.FieldOptions.extensions, proto.google.protobuf.FieldOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FieldOptions}\n */\nproto.google.protobuf.FieldOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FieldOptions;\n  return proto.google.protobuf.FieldOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FieldOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FieldOptions}\n */\nproto.google.protobuf.FieldOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {!proto.google.protobuf.FieldOptions.CType} */ (reader.readEnum());\n      msg.setCtype(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setPacked(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.google.protobuf.FieldOptions.JSType} */ (reader.readEnum());\n      msg.setJstype(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setLazy(value);\n      break;\n    case 15:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setUnverifiedLazy(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 10:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setWeak(value);\n      break;\n    case 16:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDebugRedact(value);\n      break;\n    case 17:\n      var value = /** @type {!proto.google.protobuf.FieldOptions.OptionRetention} */ (reader.readEnum());\n      msg.setRetention(value);\n      break;\n    case 19:\n      var values = /** @type {!Array<!proto.google.protobuf.FieldOptions.OptionTargetType>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addTargets(values[i]);\n      }\n      break;\n    case 20:\n      var value = new proto.google.protobuf.FieldOptions.EditionDefault;\n      reader.readMessage(value,proto.google.protobuf.FieldOptions.EditionDefault.deserializeBinaryFromReader);\n      msg.addEditionDefaults(value);\n      break;\n    case 21:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 22:\n      var value = new proto.google.protobuf.FieldOptions.FeatureSupport;\n      reader.readMessage(value,proto.google.protobuf.FieldOptions.FeatureSupport.deserializeBinaryFromReader);\n      msg.setFeatureSupport(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.FieldOptions.extensionsBinary,\n        proto.google.protobuf.FieldOptions.prototype.getExtension,\n        proto.google.protobuf.FieldOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FieldOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FieldOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FieldOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {!proto.google.protobuf.FieldOptions.CType} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeEnum(\n      1,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FieldOptions.JSType} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 15));\n  if (f != null) {\n    writer.writeBool(\n      15,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeBool(\n      10,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 16));\n  if (f != null) {\n    writer.writeBool(\n      16,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FieldOptions.OptionRetention} */ (jspb.Message.getField(message, 17));\n  if (f != null) {\n    writer.writeEnum(\n      17,\n      f\n    );\n  }\n  f = message.getTargetsList();\n  if (f.length > 0) {\n    writer.writeRepeatedEnum(\n      19,\n      f\n    );\n  }\n  f = message.getEditionDefaultsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      20,\n      f,\n      proto.google.protobuf.FieldOptions.EditionDefault.serializeBinaryToWriter\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      21,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getFeatureSupport();\n  if (f != null) {\n    writer.writeMessage(\n      22,\n      f,\n      proto.google.protobuf.FieldOptions.FeatureSupport.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.FieldOptions.extensionsBinary, proto.google.protobuf.FieldOptions.prototype.getExtension);\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FieldOptions.CType = {\n  STRING: 0,\n  CORD: 1,\n  STRING_PIECE: 2\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FieldOptions.JSType = {\n  JS_NORMAL: 0,\n  JS_STRING: 1,\n  JS_NUMBER: 2\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FieldOptions.OptionRetention = {\n  RETENTION_UNKNOWN: 0,\n  RETENTION_RUNTIME: 1,\n  RETENTION_SOURCE: 2\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FieldOptions.OptionTargetType = {\n  TARGET_TYPE_UNKNOWN: 0,\n  TARGET_TYPE_FILE: 1,\n  TARGET_TYPE_EXTENSION_RANGE: 2,\n  TARGET_TYPE_MESSAGE: 3,\n  TARGET_TYPE_FIELD: 4,\n  TARGET_TYPE_ONEOF: 5,\n  TARGET_TYPE_ENUM: 6,\n  TARGET_TYPE_ENUM_ENTRY: 7,\n  TARGET_TYPE_SERVICE: 8,\n  TARGET_TYPE_METHOD: 9\n};\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FieldOptions.EditionDefault.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FieldOptions.EditionDefault} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldOptions.EditionDefault.toObject = function(includeInstance, msg) {\n  var f, obj = {\nedition: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\nvalue: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FieldOptions.EditionDefault;\n  return proto.google.protobuf.FieldOptions.EditionDefault.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FieldOptions.EditionDefault} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 3:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setEdition(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setValue(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FieldOptions.EditionDefault.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FieldOptions.EditionDefault} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldOptions.EditionDefault.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional Edition edition = 3;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.getEdition = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault} returns this\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.setEdition = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault} returns this\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.clearEdition = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.hasEdition = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string value = 2;\n * @return {string}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.getValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault} returns this\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.setValue = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault} returns this\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.clearValue = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.EditionDefault.prototype.hasValue = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FieldOptions.FeatureSupport.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FieldOptions.FeatureSupport} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.toObject = function(includeInstance, msg) {\n  var f, obj = {\neditionIntroduced: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\neditionDeprecated: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\ndeprecationWarning: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\neditionRemoved: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FieldOptions.FeatureSupport;\n  return proto.google.protobuf.FieldOptions.FeatureSupport.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FieldOptions.FeatureSupport} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setEditionIntroduced(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setEditionDeprecated(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDeprecationWarning(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setEditionRemoved(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FieldOptions.FeatureSupport.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FieldOptions.FeatureSupport} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeEnum(\n      1,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional Edition edition_introduced = 1;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.getEditionIntroduced = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.setEditionIntroduced = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.clearEditionIntroduced = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.hasEditionIntroduced = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional Edition edition_deprecated = 2;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.getEditionDeprecated = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.setEditionDeprecated = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.clearEditionDeprecated = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.hasEditionDeprecated = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string deprecation_warning = 3;\n * @return {string}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.getDeprecationWarning = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.setDeprecationWarning = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.clearDeprecationWarning = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.hasDeprecationWarning = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional Edition edition_removed = 4;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.getEditionRemoved = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.setEditionRemoved = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions.FeatureSupport} returns this\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.clearEditionRemoved = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.FeatureSupport.prototype.hasEditionRemoved = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional CType ctype = 1;\n * @return {!proto.google.protobuf.FieldOptions.CType}\n */\nproto.google.protobuf.FieldOptions.prototype.getCtype = function() {\n  return /** @type {!proto.google.protobuf.FieldOptions.CType} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldOptions.CType} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setCtype = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearCtype = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasCtype = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional bool packed = 2;\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.getPacked = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setPacked = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearPacked = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasPacked = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional JSType jstype = 6;\n * @return {!proto.google.protobuf.FieldOptions.JSType}\n */\nproto.google.protobuf.FieldOptions.prototype.getJstype = function() {\n  return /** @type {!proto.google.protobuf.FieldOptions.JSType} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldOptions.JSType} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setJstype = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearJstype = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasJstype = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional bool lazy = 5;\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.getLazy = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setLazy = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearLazy = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasLazy = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional bool unverified_lazy = 15;\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.getUnverifiedLazy = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 15, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setUnverifiedLazy = function(value) {\n  return jspb.Message.setField(this, 15, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearUnverifiedLazy = function() {\n  return jspb.Message.setField(this, 15, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasUnverifiedLazy = function() {\n  return jspb.Message.getField(this, 15) != null;\n};\n\n\n/**\n * optional bool deprecated = 3;\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool weak = 10;\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.getWeak = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 10, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setWeak = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearWeak = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasWeak = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional bool debug_redact = 16;\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.getDebugRedact = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 16, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setDebugRedact = function(value) {\n  return jspb.Message.setField(this, 16, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearDebugRedact = function() {\n  return jspb.Message.setField(this, 16, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasDebugRedact = function() {\n  return jspb.Message.getField(this, 16) != null;\n};\n\n\n/**\n * optional OptionRetention retention = 17;\n * @return {!proto.google.protobuf.FieldOptions.OptionRetention}\n */\nproto.google.protobuf.FieldOptions.prototype.getRetention = function() {\n  return /** @type {!proto.google.protobuf.FieldOptions.OptionRetention} */ (jspb.Message.getFieldWithDefault(this, 17, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldOptions.OptionRetention} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setRetention = function(value) {\n  return jspb.Message.setField(this, 17, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearRetention = function() {\n  return jspb.Message.setField(this, 17, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasRetention = function() {\n  return jspb.Message.getField(this, 17) != null;\n};\n\n\n/**\n * repeated OptionTargetType targets = 19;\n * @return {!Array<!proto.google.protobuf.FieldOptions.OptionTargetType>}\n */\nproto.google.protobuf.FieldOptions.prototype.getTargetsList = function() {\n  return /** @type {!Array<!proto.google.protobuf.FieldOptions.OptionTargetType>} */ (jspb.Message.getRepeatedField(this, 19));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FieldOptions.OptionTargetType>} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.setTargetsList = function(value) {\n  return jspb.Message.setField(this, 19, value || []);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldOptions.OptionTargetType} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.addTargets = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 19, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearTargetsList = function() {\n  return this.setTargetsList([]);\n};\n\n\n/**\n * repeated EditionDefault edition_defaults = 20;\n * @return {!Array<!proto.google.protobuf.FieldOptions.EditionDefault>}\n */\nproto.google.protobuf.FieldOptions.prototype.getEditionDefaultsList = function() {\n  return /** @type{!Array<!proto.google.protobuf.FieldOptions.EditionDefault>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.FieldOptions.EditionDefault, 20));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FieldOptions.EditionDefault>} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n*/\nproto.google.protobuf.FieldOptions.prototype.setEditionDefaultsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 20, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FieldOptions.EditionDefault=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FieldOptions.EditionDefault}\n */\nproto.google.protobuf.FieldOptions.prototype.addEditionDefaults = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 20, opt_value, proto.google.protobuf.FieldOptions.EditionDefault, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearEditionDefaultsList = function() {\n  return this.setEditionDefaultsList([]);\n};\n\n\n/**\n * optional FeatureSet features = 21;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.FieldOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 21));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n*/\nproto.google.protobuf.FieldOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 21, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 21) != null;\n};\n\n\n/**\n * optional FeatureSupport feature_support = 22;\n * @return {?proto.google.protobuf.FieldOptions.FeatureSupport}\n */\nproto.google.protobuf.FieldOptions.prototype.getFeatureSupport = function() {\n  return /** @type{?proto.google.protobuf.FieldOptions.FeatureSupport} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FieldOptions.FeatureSupport, 22));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FieldOptions.FeatureSupport|undefined} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n*/\nproto.google.protobuf.FieldOptions.prototype.setFeatureSupport = function(value) {\n  return jspb.Message.setWrapperField(this, 22, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearFeatureSupport = function() {\n  return this.setFeatureSupport(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FieldOptions.prototype.hasFeatureSupport = function() {\n  return jspb.Message.getField(this, 22) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.FieldOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.FieldOptions} returns this\n*/\nproto.google.protobuf.FieldOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.FieldOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FieldOptions} returns this\n */\nproto.google.protobuf.FieldOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.OneofOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.OneofOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.OneofOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.OneofOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.OneofOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.OneofOptions.extensions, proto.google.protobuf.OneofOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.OneofOptions}\n */\nproto.google.protobuf.OneofOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.OneofOptions;\n  return proto.google.protobuf.OneofOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.OneofOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.OneofOptions}\n */\nproto.google.protobuf.OneofOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.OneofOptions.extensionsBinary,\n        proto.google.protobuf.OneofOptions.prototype.getExtension,\n        proto.google.protobuf.OneofOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.OneofOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.OneofOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.OneofOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.OneofOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.OneofOptions.extensionsBinary, proto.google.protobuf.OneofOptions.prototype.getExtension);\n};\n\n\n/**\n * optional FeatureSet features = 1;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.OneofOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 1));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.OneofOptions} returns this\n*/\nproto.google.protobuf.OneofOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 1, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.OneofOptions} returns this\n */\nproto.google.protobuf.OneofOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.OneofOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.OneofOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.OneofOptions} returns this\n*/\nproto.google.protobuf.OneofOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.OneofOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.OneofOptions} returns this\n */\nproto.google.protobuf.OneofOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.EnumOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.EnumOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.EnumOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.EnumOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\nallowAlias: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f,\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\ndeprecatedLegacyJsonFieldConflicts: (f = jspb.Message.getBooleanField(msg, 6)) == null ? undefined : f,\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.EnumOptions.extensions, proto.google.protobuf.EnumOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.EnumOptions}\n */\nproto.google.protobuf.EnumOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.EnumOptions;\n  return proto.google.protobuf.EnumOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.EnumOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.EnumOptions}\n */\nproto.google.protobuf.EnumOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setAllowAlias(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 6:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecatedLegacyJsonFieldConflicts(value);\n      break;\n    case 7:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.EnumOptions.extensionsBinary,\n        proto.google.protobuf.EnumOptions.prototype.getExtension,\n        proto.google.protobuf.EnumOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.EnumOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.EnumOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.EnumOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeBool(\n      6,\n      f\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.EnumOptions.extensionsBinary, proto.google.protobuf.EnumOptions.prototype.getExtension);\n};\n\n\n/**\n * optional bool allow_alias = 2;\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.getAllowAlias = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.setAllowAlias = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.clearAllowAlias = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.hasAllowAlias = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool deprecated = 3;\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool deprecated_legacy_json_field_conflicts = 6;\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.getDeprecatedLegacyJsonFieldConflicts = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.setDeprecatedLegacyJsonFieldConflicts = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.clearDeprecatedLegacyJsonFieldConflicts = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.hasDeprecatedLegacyJsonFieldConflicts = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional FeatureSet features = 7;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.EnumOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 7));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.EnumOptions} returns this\n*/\nproto.google.protobuf.EnumOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 7, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.EnumOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.EnumOptions} returns this\n*/\nproto.google.protobuf.EnumOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.EnumOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.EnumOptions} returns this\n */\nproto.google.protobuf.EnumOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.EnumValueOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.EnumValueOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.EnumValueOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.EnumValueOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumValueOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 1, false),\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\ndebugRedact: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\nfeatureSupport: (f = msg.getFeatureSupport()) && proto.google.protobuf.FieldOptions.FeatureSupport.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.EnumValueOptions.extensions, proto.google.protobuf.EnumValueOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.EnumValueOptions}\n */\nproto.google.protobuf.EnumValueOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.EnumValueOptions;\n  return proto.google.protobuf.EnumValueOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.EnumValueOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.EnumValueOptions}\n */\nproto.google.protobuf.EnumValueOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 2:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDebugRedact(value);\n      break;\n    case 4:\n      var value = new proto.google.protobuf.FieldOptions.FeatureSupport;\n      reader.readMessage(value,proto.google.protobuf.FieldOptions.FeatureSupport.deserializeBinaryFromReader);\n      msg.setFeatureSupport(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.EnumValueOptions.extensionsBinary,\n        proto.google.protobuf.EnumValueOptions.prototype.getExtension,\n        proto.google.protobuf.EnumValueOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.EnumValueOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.EnumValueOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.EnumValueOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.EnumValueOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeBool(\n      1,\n      f\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = message.getFeatureSupport();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.google.protobuf.FieldOptions.FeatureSupport.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.EnumValueOptions.extensionsBinary, proto.google.protobuf.EnumValueOptions.prototype.getExtension);\n};\n\n\n/**\n * optional bool deprecated = 1;\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 1, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional FeatureSet features = 2;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.EnumValueOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 2));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n*/\nproto.google.protobuf.EnumValueOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool debug_redact = 3;\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueOptions.prototype.getDebugRedact = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.setDebugRedact = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.clearDebugRedact = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueOptions.prototype.hasDebugRedact = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional FieldOptions.FeatureSupport feature_support = 4;\n * @return {?proto.google.protobuf.FieldOptions.FeatureSupport}\n */\nproto.google.protobuf.EnumValueOptions.prototype.getFeatureSupport = function() {\n  return /** @type{?proto.google.protobuf.FieldOptions.FeatureSupport} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FieldOptions.FeatureSupport, 4));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FieldOptions.FeatureSupport|undefined} value\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n*/\nproto.google.protobuf.EnumValueOptions.prototype.setFeatureSupport = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.clearFeatureSupport = function() {\n  return this.setFeatureSupport(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.EnumValueOptions.prototype.hasFeatureSupport = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.EnumValueOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n*/\nproto.google.protobuf.EnumValueOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.EnumValueOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.EnumValueOptions} returns this\n */\nproto.google.protobuf.EnumValueOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.ServiceOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.ServiceOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.ServiceOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.ServiceOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ServiceOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 33, false),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.ServiceOptions.extensions, proto.google.protobuf.ServiceOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.ServiceOptions}\n */\nproto.google.protobuf.ServiceOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.ServiceOptions;\n  return proto.google.protobuf.ServiceOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.ServiceOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.ServiceOptions}\n */\nproto.google.protobuf.ServiceOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 34:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 33:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.ServiceOptions.extensionsBinary,\n        proto.google.protobuf.ServiceOptions.prototype.getExtension,\n        proto.google.protobuf.ServiceOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.ServiceOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.ServiceOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.ServiceOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.ServiceOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      34,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 33));\n  if (f != null) {\n    writer.writeBool(\n      33,\n      f\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.ServiceOptions.extensionsBinary, proto.google.protobuf.ServiceOptions.prototype.getExtension);\n};\n\n\n/**\n * optional FeatureSet features = 34;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.ServiceOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 34));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.ServiceOptions} returns this\n*/\nproto.google.protobuf.ServiceOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 34, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.ServiceOptions} returns this\n */\nproto.google.protobuf.ServiceOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ServiceOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 34) != null;\n};\n\n\n/**\n * optional bool deprecated = 33;\n * @return {boolean}\n */\nproto.google.protobuf.ServiceOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 33, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.ServiceOptions} returns this\n */\nproto.google.protobuf.ServiceOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 33, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.ServiceOptions} returns this\n */\nproto.google.protobuf.ServiceOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 33, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.ServiceOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 33) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.ServiceOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.ServiceOptions} returns this\n*/\nproto.google.protobuf.ServiceOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.ServiceOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.ServiceOptions} returns this\n */\nproto.google.protobuf.ServiceOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.MethodOptions.repeatedFields_ = [999];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.MethodOptions.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.MethodOptions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.MethodOptions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.MethodOptions.toObject = function(includeInstance, msg) {\n  var f, obj = {\ndeprecated: jspb.Message.getBooleanFieldWithDefault(msg, 33, false),\nidempotencyLevel: jspb.Message.getFieldWithDefault(msg, 34, 0),\nfeatures: (f = msg.getFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nuninterpretedOptionList: jspb.Message.toObjectList(msg.getUninterpretedOptionList(),\n    proto.google.protobuf.UninterpretedOption.toObject, includeInstance)\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.MethodOptions.extensions, proto.google.protobuf.MethodOptions.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.MethodOptions}\n */\nproto.google.protobuf.MethodOptions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.MethodOptions;\n  return proto.google.protobuf.MethodOptions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.MethodOptions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.MethodOptions}\n */\nproto.google.protobuf.MethodOptions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 33:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDeprecated(value);\n      break;\n    case 34:\n      var value = /** @type {!proto.google.protobuf.MethodOptions.IdempotencyLevel} */ (reader.readEnum());\n      msg.setIdempotencyLevel(value);\n      break;\n    case 35:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFeatures(value);\n      break;\n    case 999:\n      var value = new proto.google.protobuf.UninterpretedOption;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader);\n      msg.addUninterpretedOption(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.MethodOptions.extensionsBinary,\n        proto.google.protobuf.MethodOptions.prototype.getExtension,\n        proto.google.protobuf.MethodOptions.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.MethodOptions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.MethodOptions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.MethodOptions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.MethodOptions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 33));\n  if (f != null) {\n    writer.writeBool(\n      33,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.MethodOptions.IdempotencyLevel} */ (jspb.Message.getField(message, 34));\n  if (f != null) {\n    writer.writeEnum(\n      34,\n      f\n    );\n  }\n  f = message.getFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      35,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getUninterpretedOptionList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      999,\n      f,\n      proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.MethodOptions.extensionsBinary, proto.google.protobuf.MethodOptions.prototype.getExtension);\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.MethodOptions.IdempotencyLevel = {\n  IDEMPOTENCY_UNKNOWN: 0,\n  NO_SIDE_EFFECTS: 1,\n  IDEMPOTENT: 2\n};\n\n/**\n * optional bool deprecated = 33;\n * @return {boolean}\n */\nproto.google.protobuf.MethodOptions.prototype.getDeprecated = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 33, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.MethodOptions} returns this\n */\nproto.google.protobuf.MethodOptions.prototype.setDeprecated = function(value) {\n  return jspb.Message.setField(this, 33, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodOptions} returns this\n */\nproto.google.protobuf.MethodOptions.prototype.clearDeprecated = function() {\n  return jspb.Message.setField(this, 33, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodOptions.prototype.hasDeprecated = function() {\n  return jspb.Message.getField(this, 33) != null;\n};\n\n\n/**\n * optional IdempotencyLevel idempotency_level = 34;\n * @return {!proto.google.protobuf.MethodOptions.IdempotencyLevel}\n */\nproto.google.protobuf.MethodOptions.prototype.getIdempotencyLevel = function() {\n  return /** @type {!proto.google.protobuf.MethodOptions.IdempotencyLevel} */ (jspb.Message.getFieldWithDefault(this, 34, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.MethodOptions.IdempotencyLevel} value\n * @return {!proto.google.protobuf.MethodOptions} returns this\n */\nproto.google.protobuf.MethodOptions.prototype.setIdempotencyLevel = function(value) {\n  return jspb.Message.setField(this, 34, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.MethodOptions} returns this\n */\nproto.google.protobuf.MethodOptions.prototype.clearIdempotencyLevel = function() {\n  return jspb.Message.setField(this, 34, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodOptions.prototype.hasIdempotencyLevel = function() {\n  return jspb.Message.getField(this, 34) != null;\n};\n\n\n/**\n * optional FeatureSet features = 35;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.MethodOptions.prototype.getFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 35));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.MethodOptions} returns this\n*/\nproto.google.protobuf.MethodOptions.prototype.setFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 35, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.MethodOptions} returns this\n */\nproto.google.protobuf.MethodOptions.prototype.clearFeatures = function() {\n  return this.setFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.MethodOptions.prototype.hasFeatures = function() {\n  return jspb.Message.getField(this, 35) != null;\n};\n\n\n/**\n * repeated UninterpretedOption uninterpreted_option = 999;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption>}\n */\nproto.google.protobuf.MethodOptions.prototype.getUninterpretedOptionList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption, 999));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption>} value\n * @return {!proto.google.protobuf.MethodOptions} returns this\n*/\nproto.google.protobuf.MethodOptions.prototype.setUninterpretedOptionList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 999, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.MethodOptions.prototype.addUninterpretedOption = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 999, opt_value, proto.google.protobuf.UninterpretedOption, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.MethodOptions} returns this\n */\nproto.google.protobuf.MethodOptions.prototype.clearUninterpretedOptionList = function() {\n  return this.setUninterpretedOptionList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.UninterpretedOption.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.UninterpretedOption.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.UninterpretedOption.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.UninterpretedOption} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.UninterpretedOption.toObject = function(includeInstance, msg) {\n  var f, obj = {\nnameList: jspb.Message.toObjectList(msg.getNameList(),\n    proto.google.protobuf.UninterpretedOption.NamePart.toObject, includeInstance),\nidentifierValue: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\npositiveIntValue: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,\nnegativeIntValue: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,\ndoubleValue: (f = jspb.Message.getOptionalFloatingPointField(msg, 6)) == null ? undefined : f,\nstringValue: msg.getStringValue_asB64(),\naggregateValue: (f = jspb.Message.getField(msg, 8)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.UninterpretedOption.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.UninterpretedOption;\n  return proto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.UninterpretedOption} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.UninterpretedOption}\n */\nproto.google.protobuf.UninterpretedOption.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 2:\n      var value = new proto.google.protobuf.UninterpretedOption.NamePart;\n      reader.readMessage(value,proto.google.protobuf.UninterpretedOption.NamePart.deserializeBinaryFromReader);\n      msg.addName(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIdentifierValue(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readUint64());\n      msg.setPositiveIntValue(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setNegativeIntValue(value);\n      break;\n    case 6:\n      var value = /** @type {number} */ (reader.readDouble());\n      msg.setDoubleValue(value);\n      break;\n    case 7:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setStringValue(value);\n      break;\n    case 8:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setAggregateValue(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.UninterpretedOption.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.UninterpretedOption.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.UninterpretedOption} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.UninterpretedOption.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getNameList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.google.protobuf.UninterpretedOption.NamePart.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeUint64(\n      4,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeInt64(\n      5,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeDouble(\n      6,\n      f\n    );\n  }\n  f = /** @type {!(string|Uint8Array)} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeBytes(\n      7,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeString(\n      8,\n      f\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.UninterpretedOption.NamePart.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.UninterpretedOption.NamePart} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.UninterpretedOption.NamePart.toObject = function(includeInstance, msg) {\n  var f, obj = {\nnamePart: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nisExtension: (f = jspb.Message.getBooleanField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.UninterpretedOption.NamePart;\n  return proto.google.protobuf.UninterpretedOption.NamePart.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.UninterpretedOption.NamePart} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setNamePart(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsExtension(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.UninterpretedOption.NamePart.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.UninterpretedOption.NamePart} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.UninterpretedOption.NamePart.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * required string name_part = 1;\n * @return {string}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.getNamePart = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart} returns this\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.setNamePart = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart} returns this\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.clearNamePart = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.hasNamePart = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * required bool is_extension = 2;\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.getIsExtension = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart} returns this\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.setIsExtension = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart} returns this\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.clearIsExtension = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.NamePart.prototype.hasIsExtension = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * repeated NamePart name = 2;\n * @return {!Array<!proto.google.protobuf.UninterpretedOption.NamePart>}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getNameList = function() {\n  return /** @type{!Array<!proto.google.protobuf.UninterpretedOption.NamePart>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.UninterpretedOption.NamePart, 2));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.UninterpretedOption.NamePart>} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n*/\nproto.google.protobuf.UninterpretedOption.prototype.setNameList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.UninterpretedOption.NamePart=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.UninterpretedOption.NamePart}\n */\nproto.google.protobuf.UninterpretedOption.prototype.addName = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.google.protobuf.UninterpretedOption.NamePart, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearNameList = function() {\n  return this.setNameList([]);\n};\n\n\n/**\n * optional string identifier_value = 3;\n * @return {string}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getIdentifierValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.setIdentifierValue = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearIdentifierValue = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.prototype.hasIdentifierValue = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional uint64 positive_int_value = 4;\n * @return {number}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getPositiveIntValue = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.setPositiveIntValue = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearPositiveIntValue = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.prototype.hasPositiveIntValue = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional int64 negative_int_value = 5;\n * @return {number}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getNegativeIntValue = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.setNegativeIntValue = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearNegativeIntValue = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.prototype.hasNegativeIntValue = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional double double_value = 6;\n * @return {number}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getDoubleValue = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 6, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.setDoubleValue = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearDoubleValue = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.prototype.hasDoubleValue = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional bytes string_value = 7;\n * @return {!(string|Uint8Array)}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getStringValue = function() {\n  return /** @type {!(string|Uint8Array)} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * optional bytes string_value = 7;\n * This is a type-conversion wrapper around `getStringValue()`\n * @return {string}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getStringValue_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getStringValue()));\n};\n\n\n/**\n * optional bytes string_value = 7;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getStringValue()`\n * @return {!Uint8Array}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getStringValue_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getStringValue()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.setStringValue = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearStringValue = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.prototype.hasStringValue = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional string aggregate_value = 8;\n * @return {string}\n */\nproto.google.protobuf.UninterpretedOption.prototype.getAggregateValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.setAggregateValue = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.UninterpretedOption} returns this\n */\nproto.google.protobuf.UninterpretedOption.prototype.clearAggregateValue = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.UninterpretedOption.prototype.hasAggregateValue = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FeatureSet.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FeatureSet.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FeatureSet} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FeatureSet.toObject = function(includeInstance, msg) {\n  var f, obj = {\nfieldPresence: (f = jspb.Message.getField(msg, 1)) == null ? undefined : f,\nenumType: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\nrepeatedFieldEncoding: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\nutf8Validation: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,\nmessageEncoding: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f,\njsonFormat: (f = jspb.Message.getField(msg, 6)) == null ? undefined : f\n  };\n\n  jspb.Message.toObjectExtension(/** @type {!jspb.Message} */ (msg), obj,\n      proto.google.protobuf.FeatureSet.extensions, proto.google.protobuf.FeatureSet.prototype.getExtension,\n      includeInstance);\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.FeatureSet.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FeatureSet;\n  return proto.google.protobuf.FeatureSet.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FeatureSet} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.FeatureSet.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {!proto.google.protobuf.FeatureSet.FieldPresence} */ (reader.readEnum());\n      msg.setFieldPresence(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.google.protobuf.FeatureSet.EnumType} */ (reader.readEnum());\n      msg.setEnumType(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.google.protobuf.FeatureSet.RepeatedFieldEncoding} */ (reader.readEnum());\n      msg.setRepeatedFieldEncoding(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.google.protobuf.FeatureSet.Utf8Validation} */ (reader.readEnum());\n      msg.setUtf8Validation(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.google.protobuf.FeatureSet.MessageEncoding} */ (reader.readEnum());\n      msg.setMessageEncoding(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.google.protobuf.FeatureSet.JsonFormat} */ (reader.readEnum());\n      msg.setJsonFormat(value);\n      break;\n    default:\n      jspb.Message.readBinaryExtension(msg, reader,\n        proto.google.protobuf.FeatureSet.extensionsBinary,\n        proto.google.protobuf.FeatureSet.prototype.getExtension,\n        proto.google.protobuf.FeatureSet.prototype.setExtension);\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FeatureSet.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FeatureSet.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FeatureSet} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FeatureSet.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {!proto.google.protobuf.FeatureSet.FieldPresence} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeEnum(\n      1,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FeatureSet.EnumType} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FeatureSet.RepeatedFieldEncoding} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FeatureSet.Utf8Validation} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FeatureSet.MessageEncoding} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.FeatureSet.JsonFormat} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n  jspb.Message.serializeBinaryExtensions(message, writer,\n    proto.google.protobuf.FeatureSet.extensionsBinary, proto.google.protobuf.FeatureSet.prototype.getExtension);\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FeatureSet.FieldPresence = {\n  FIELD_PRESENCE_UNKNOWN: 0,\n  EXPLICIT: 1,\n  IMPLICIT: 2,\n  LEGACY_REQUIRED: 3\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FeatureSet.EnumType = {\n  ENUM_TYPE_UNKNOWN: 0,\n  OPEN: 1,\n  CLOSED: 2\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FeatureSet.RepeatedFieldEncoding = {\n  REPEATED_FIELD_ENCODING_UNKNOWN: 0,\n  PACKED: 1,\n  EXPANDED: 2\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FeatureSet.Utf8Validation = {\n  UTF8_VALIDATION_UNKNOWN: 0,\n  VERIFY: 2,\n  NONE: 3\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FeatureSet.MessageEncoding = {\n  MESSAGE_ENCODING_UNKNOWN: 0,\n  LENGTH_PREFIXED: 1,\n  DELIMITED: 2\n};\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.FeatureSet.JsonFormat = {\n  JSON_FORMAT_UNKNOWN: 0,\n  ALLOW: 1,\n  LEGACY_BEST_EFFORT: 2\n};\n\n/**\n * optional FieldPresence field_presence = 1;\n * @return {!proto.google.protobuf.FeatureSet.FieldPresence}\n */\nproto.google.protobuf.FeatureSet.prototype.getFieldPresence = function() {\n  return /** @type {!proto.google.protobuf.FeatureSet.FieldPresence} */ (jspb.Message.getFieldWithDefault(this, 1, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSet.FieldPresence} value\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.setFieldPresence = function(value) {\n  return jspb.Message.setField(this, 1, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.clearFieldPresence = function() {\n  return jspb.Message.setField(this, 1, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSet.prototype.hasFieldPresence = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional EnumType enum_type = 2;\n * @return {!proto.google.protobuf.FeatureSet.EnumType}\n */\nproto.google.protobuf.FeatureSet.prototype.getEnumType = function() {\n  return /** @type {!proto.google.protobuf.FeatureSet.EnumType} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSet.EnumType} value\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.setEnumType = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.clearEnumType = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSet.prototype.hasEnumType = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional RepeatedFieldEncoding repeated_field_encoding = 3;\n * @return {!proto.google.protobuf.FeatureSet.RepeatedFieldEncoding}\n */\nproto.google.protobuf.FeatureSet.prototype.getRepeatedFieldEncoding = function() {\n  return /** @type {!proto.google.protobuf.FeatureSet.RepeatedFieldEncoding} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSet.RepeatedFieldEncoding} value\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.setRepeatedFieldEncoding = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.clearRepeatedFieldEncoding = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSet.prototype.hasRepeatedFieldEncoding = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional Utf8Validation utf8_validation = 4;\n * @return {!proto.google.protobuf.FeatureSet.Utf8Validation}\n */\nproto.google.protobuf.FeatureSet.prototype.getUtf8Validation = function() {\n  return /** @type {!proto.google.protobuf.FeatureSet.Utf8Validation} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSet.Utf8Validation} value\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.setUtf8Validation = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.clearUtf8Validation = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSet.prototype.hasUtf8Validation = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional MessageEncoding message_encoding = 5;\n * @return {!proto.google.protobuf.FeatureSet.MessageEncoding}\n */\nproto.google.protobuf.FeatureSet.prototype.getMessageEncoding = function() {\n  return /** @type {!proto.google.protobuf.FeatureSet.MessageEncoding} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSet.MessageEncoding} value\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.setMessageEncoding = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.clearMessageEncoding = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSet.prototype.hasMessageEncoding = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional JsonFormat json_format = 6;\n * @return {!proto.google.protobuf.FeatureSet.JsonFormat}\n */\nproto.google.protobuf.FeatureSet.prototype.getJsonFormat = function() {\n  return /** @type {!proto.google.protobuf.FeatureSet.JsonFormat} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSet.JsonFormat} value\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.setJsonFormat = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSet} returns this\n */\nproto.google.protobuf.FeatureSet.prototype.clearJsonFormat = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSet.prototype.hasJsonFormat = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.FeatureSetDefaults.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FeatureSetDefaults.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FeatureSetDefaults} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FeatureSetDefaults.toObject = function(includeInstance, msg) {\n  var f, obj = {\ndefaultsList: jspb.Message.toObjectList(msg.getDefaultsList(),\n    proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.toObject, includeInstance),\nminimumEdition: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,\nmaximumEdition: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FeatureSetDefaults}\n */\nproto.google.protobuf.FeatureSetDefaults.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FeatureSetDefaults;\n  return proto.google.protobuf.FeatureSetDefaults.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FeatureSetDefaults} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FeatureSetDefaults}\n */\nproto.google.protobuf.FeatureSetDefaults.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault;\n      reader.readMessage(value,proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.deserializeBinaryFromReader);\n      msg.addDefaults(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setMinimumEdition(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setMaximumEdition(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FeatureSetDefaults.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FeatureSetDefaults} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FeatureSetDefaults.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getDefaultsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.toObject = function(includeInstance, msg) {\n  var f, obj = {\nedition: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\noverridableFeatures: (f = msg.getOverridableFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f),\nfixedFeatures: (f = msg.getFixedFeatures()) && proto.google.protobuf.FeatureSet.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault;\n  return proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 3:\n      var value = /** @type {!proto.google.protobuf.Edition} */ (reader.readEnum());\n      msg.setEdition(value);\n      break;\n    case 4:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setOverridableFeatures(value);\n      break;\n    case 5:\n      var value = new proto.google.protobuf.FeatureSet;\n      reader.readMessage(value,proto.google.protobuf.FeatureSet.deserializeBinaryFromReader);\n      msg.setFixedFeatures(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = message.getOverridableFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n  f = message.getFixedFeatures();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.google.protobuf.FeatureSet.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional Edition edition = 3;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.getEdition = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.setEdition = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.clearEdition = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.hasEdition = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional FeatureSet overridable_features = 4;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.getOverridableFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 4));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} returns this\n*/\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.setOverridableFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.clearOverridableFeatures = function() {\n  return this.setOverridableFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.hasOverridableFeatures = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional FeatureSet fixed_features = 5;\n * @return {?proto.google.protobuf.FeatureSet}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.getFixedFeatures = function() {\n  return /** @type{?proto.google.protobuf.FeatureSet} */ (\n    jspb.Message.getWrapperField(this, proto.google.protobuf.FeatureSet, 5));\n};\n\n\n/**\n * @param {?proto.google.protobuf.FeatureSet|undefined} value\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} returns this\n*/\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.setFixedFeatures = function(value) {\n  return jspb.Message.setWrapperField(this, 5, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.clearFixedFeatures = function() {\n  return this.setFixedFeatures(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.prototype.hasFixedFeatures = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * repeated FeatureSetEditionDefault defaults = 1;\n * @return {!Array<!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault>}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.getDefaultsList = function() {\n  return /** @type{!Array<!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault, 1));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault>} value\n * @return {!proto.google.protobuf.FeatureSetDefaults} returns this\n*/\nproto.google.protobuf.FeatureSetDefaults.prototype.setDefaultsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.addDefaults = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.FeatureSetDefaults} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.clearDefaultsList = function() {\n  return this.setDefaultsList([]);\n};\n\n\n/**\n * optional Edition minimum_edition = 4;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.getMinimumEdition = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FeatureSetDefaults} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.setMinimumEdition = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSetDefaults} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.clearMinimumEdition = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.hasMinimumEdition = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional Edition maximum_edition = 5;\n * @return {!proto.google.protobuf.Edition}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.getMaximumEdition = function() {\n  return /** @type {!proto.google.protobuf.Edition} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.Edition} value\n * @return {!proto.google.protobuf.FeatureSetDefaults} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.setMaximumEdition = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.FeatureSetDefaults} returns this\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.clearMaximumEdition = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.FeatureSetDefaults.prototype.hasMaximumEdition = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.SourceCodeInfo.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.SourceCodeInfo.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.SourceCodeInfo.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.SourceCodeInfo} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.SourceCodeInfo.toObject = function(includeInstance, msg) {\n  var f, obj = {\nlocationList: jspb.Message.toObjectList(msg.getLocationList(),\n    proto.google.protobuf.SourceCodeInfo.Location.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.SourceCodeInfo}\n */\nproto.google.protobuf.SourceCodeInfo.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.SourceCodeInfo;\n  return proto.google.protobuf.SourceCodeInfo.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.SourceCodeInfo} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.SourceCodeInfo}\n */\nproto.google.protobuf.SourceCodeInfo.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.google.protobuf.SourceCodeInfo.Location;\n      reader.readMessage(value,proto.google.protobuf.SourceCodeInfo.Location.deserializeBinaryFromReader);\n      msg.addLocation(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.SourceCodeInfo.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.SourceCodeInfo.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.SourceCodeInfo} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.SourceCodeInfo.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getLocationList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.google.protobuf.SourceCodeInfo.Location.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.SourceCodeInfo.Location.repeatedFields_ = [1,2,6];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.SourceCodeInfo.Location.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.SourceCodeInfo.Location} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.SourceCodeInfo.Location.toObject = function(includeInstance, msg) {\n  var f, obj = {\npathList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f,\nspanList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,\nleadingComments: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\ntrailingComments: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,\nleadingDetachedCommentsList: (f = jspb.Message.getRepeatedField(msg, 6)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location}\n */\nproto.google.protobuf.SourceCodeInfo.Location.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.SourceCodeInfo.Location;\n  return proto.google.protobuf.SourceCodeInfo.Location.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.SourceCodeInfo.Location} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location}\n */\nproto.google.protobuf.SourceCodeInfo.Location.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt32() : [reader.readInt32()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addPath(values[i]);\n      }\n      break;\n    case 2:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt32() : [reader.readInt32()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addSpan(values[i]);\n      }\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLeadingComments(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTrailingComments(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addLeadingDetachedComments(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.SourceCodeInfo.Location.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.SourceCodeInfo.Location} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.SourceCodeInfo.Location.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getPathList();\n  if (f.length > 0) {\n    writer.writePackedInt32(\n      1,\n      f\n    );\n  }\n  f = message.getSpanList();\n  if (f.length > 0) {\n    writer.writePackedInt32(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getLeadingDetachedCommentsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated int32 path = 1;\n * @return {!Array<number>}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.getPathList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.setPathList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.addPath = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.clearPathList = function() {\n  return this.setPathList([]);\n};\n\n\n/**\n * repeated int32 span = 2;\n * @return {!Array<number>}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.getSpanList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.setSpanList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.addSpan = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.clearSpanList = function() {\n  return this.setSpanList([]);\n};\n\n\n/**\n * optional string leading_comments = 3;\n * @return {string}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.getLeadingComments = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.setLeadingComments = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.clearLeadingComments = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.hasLeadingComments = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string trailing_comments = 4;\n * @return {string}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.getTrailingComments = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.setTrailingComments = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.clearTrailingComments = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.hasTrailingComments = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * repeated string leading_detached_comments = 6;\n * @return {!Array<string>}\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.getLeadingDetachedCommentsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 6));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.setLeadingDetachedCommentsList = function(value) {\n  return jspb.Message.setField(this, 6, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.addLeadingDetachedComments = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 6, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.SourceCodeInfo.Location} returns this\n */\nproto.google.protobuf.SourceCodeInfo.Location.prototype.clearLeadingDetachedCommentsList = function() {\n  return this.setLeadingDetachedCommentsList([]);\n};\n\n\n/**\n * repeated Location location = 1;\n * @return {!Array<!proto.google.protobuf.SourceCodeInfo.Location>}\n */\nproto.google.protobuf.SourceCodeInfo.prototype.getLocationList = function() {\n  return /** @type{!Array<!proto.google.protobuf.SourceCodeInfo.Location>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.SourceCodeInfo.Location, 1));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.SourceCodeInfo.Location>} value\n * @return {!proto.google.protobuf.SourceCodeInfo} returns this\n*/\nproto.google.protobuf.SourceCodeInfo.prototype.setLocationList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.SourceCodeInfo.Location=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.SourceCodeInfo.Location}\n */\nproto.google.protobuf.SourceCodeInfo.prototype.addLocation = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.google.protobuf.SourceCodeInfo.Location, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.SourceCodeInfo} returns this\n */\nproto.google.protobuf.SourceCodeInfo.prototype.clearLocationList = function() {\n  return this.setLocationList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.GeneratedCodeInfo.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.GeneratedCodeInfo.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.GeneratedCodeInfo.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.GeneratedCodeInfo} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.GeneratedCodeInfo.toObject = function(includeInstance, msg) {\n  var f, obj = {\nannotationList: jspb.Message.toObjectList(msg.getAnnotationList(),\n    proto.google.protobuf.GeneratedCodeInfo.Annotation.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.GeneratedCodeInfo}\n */\nproto.google.protobuf.GeneratedCodeInfo.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.GeneratedCodeInfo;\n  return proto.google.protobuf.GeneratedCodeInfo.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.GeneratedCodeInfo} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.GeneratedCodeInfo}\n */\nproto.google.protobuf.GeneratedCodeInfo.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.google.protobuf.GeneratedCodeInfo.Annotation;\n      reader.readMessage(value,proto.google.protobuf.GeneratedCodeInfo.Annotation.deserializeBinaryFromReader);\n      msg.addAnnotation(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.GeneratedCodeInfo.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.GeneratedCodeInfo.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.GeneratedCodeInfo} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.GeneratedCodeInfo.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getAnnotationList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.google.protobuf.GeneratedCodeInfo.Annotation.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.toObject = function(opt_includeInstance) {\n  return proto.google.protobuf.GeneratedCodeInfo.Annotation.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.google.protobuf.GeneratedCodeInfo.Annotation} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.toObject = function(includeInstance, msg) {\n  var f, obj = {\npathList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f,\nsourceFile: (f = jspb.Message.getField(msg, 2)) == null ? undefined : f,\nbegin: (f = jspb.Message.getField(msg, 3)) == null ? undefined : f,\nend: (f = jspb.Message.getField(msg, 4)) == null ? undefined : f,\nsemantic: (f = jspb.Message.getField(msg, 5)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.google.protobuf.GeneratedCodeInfo.Annotation;\n  return proto.google.protobuf.GeneratedCodeInfo.Annotation.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.google.protobuf.GeneratedCodeInfo.Annotation} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt32() : [reader.readInt32()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addPath(values[i]);\n      }\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSourceFile(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setBegin(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt32());\n      msg.setEnd(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic} */ (reader.readEnum());\n      msg.setSemantic(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.google.protobuf.GeneratedCodeInfo.Annotation.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.google.protobuf.GeneratedCodeInfo.Annotation} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getPathList();\n  if (f.length > 0) {\n    writer.writePackedInt32(\n      1,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt32(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt32(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic = {\n  NONE: 0,\n  SET: 1,\n  ALIAS: 2\n};\n\n/**\n * repeated int32 path = 1;\n * @return {!Array<number>}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.getPathList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.setPathList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.addPath = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.clearPathList = function() {\n  return this.setPathList([]);\n};\n\n\n/**\n * optional string source_file = 2;\n * @return {string}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.getSourceFile = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.setSourceFile = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.clearSourceFile = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.hasSourceFile = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int32 begin = 3;\n * @return {number}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.getBegin = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.setBegin = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.clearBegin = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.hasBegin = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int32 end = 4;\n * @return {number}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.getEnd = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.setEnd = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.clearEnd = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.hasEnd = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional Semantic semantic = 5;\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.getSemantic = function() {\n  return /** @type {!proto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.google.protobuf.GeneratedCodeInfo.Annotation.Semantic} value\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.setSemantic = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.clearSemantic = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.google.protobuf.GeneratedCodeInfo.Annotation.prototype.hasSemantic = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * repeated Annotation annotation = 1;\n * @return {!Array<!proto.google.protobuf.GeneratedCodeInfo.Annotation>}\n */\nproto.google.protobuf.GeneratedCodeInfo.prototype.getAnnotationList = function() {\n  return /** @type{!Array<!proto.google.protobuf.GeneratedCodeInfo.Annotation>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.google.protobuf.GeneratedCodeInfo.Annotation, 1));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.GeneratedCodeInfo.Annotation>} value\n * @return {!proto.google.protobuf.GeneratedCodeInfo} returns this\n*/\nproto.google.protobuf.GeneratedCodeInfo.prototype.setAnnotationList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.GeneratedCodeInfo.Annotation=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.GeneratedCodeInfo.Annotation}\n */\nproto.google.protobuf.GeneratedCodeInfo.prototype.addAnnotation = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.google.protobuf.GeneratedCodeInfo.Annotation, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.google.protobuf.GeneratedCodeInfo} returns this\n */\nproto.google.protobuf.GeneratedCodeInfo.prototype.clearAnnotationList = function() {\n  return this.setAnnotationList([]);\n};\n\n\n/**\n * @enum {number}\n */\nproto.google.protobuf.Edition = {\n  EDITION_UNKNOWN: 0,\n  EDITION_LEGACY: 900,\n  EDITION_PROTO2: 998,\n  EDITION_PROTO3: 999,\n  EDITION_2023: 1000,\n  EDITION_2024: 1001,\n  EDITION_1_TEST_ONLY: 1,\n  EDITION_2_TEST_ONLY: 2,\n  EDITION_99997_TEST_ONLY: 99997,\n  EDITION_99998_TEST_ONLY: 99998,\n  EDITION_99999_TEST_ONLY: 99999,\n  EDITION_MAX: 2147483647\n};\n\ngoog.object.extend(exports, proto.google.protobuf);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/google-protobuf/google/protobuf/descriptor_pb.js?");

/***/ }),

/***/ "./node_modules/grpc-web/index.js":
/*!****************************************!*\
  !*** ./node_modules/grpc-web/index.js ***!
  \****************************************/
/***/ (function(module, __unused_webpack_exports, __webpack_require__) {

eval("/*\n\n Copyright The Closure Library Authors.\n SPDX-License-Identifier: Apache-2.0\n*/\nvar n;function aa(a){var b=0;return function(){return b<a.length?{done:!1,value:a[b++]}:{done:!0}}}var ba=\"function\"==typeof Object.defineProperties?Object.defineProperty:function(a,b,c){a!=Array.prototype&&a!=Object.prototype&&(a[b]=c.value)};function ca(a){a=[\"object\"==typeof window&&window,\"object\"==typeof self&&self,\"object\"==typeof __webpack_require__.g&&__webpack_require__.g,a];for(var b=0;b<a.length;++b){var c=a[b];if(c&&c.Math==Math)return c}throw Error(\"Cannot find global object\");}var r=ca(this);\nfunction t(){t=function(){};r.Symbol||(r.Symbol=da)}function ea(a,b){this.a=a;ba(this,\"description\",{configurable:!0,writable:!0,value:b})}ea.prototype.toString=function(){return this.a};var da=function(){function a(c){if(this instanceof a)throw new TypeError(\"Symbol is not a constructor\");return new ea(\"jscomp_symbol_\"+(c||\"\")+\"_\"+b++,c)}var b=0;return a}();\nfunction u(){t();var a=r.Symbol.iterator;a||(a=r.Symbol.iterator=r.Symbol(\"Symbol.iterator\"));\"function\"!=typeof Array.prototype[a]&&ba(Array.prototype,a,{configurable:!0,writable:!0,value:function(){return fa(aa(this))}});u=function(){}}function fa(a){u();a={next:a};a[r.Symbol.iterator]=function(){return this};return a}function ha(a){var b=\"undefined\"!=typeof Symbol&&Symbol.iterator&&a[Symbol.iterator];return b?b.call(a):{next:aa(a)}}\nvar ia=\"function\"==typeof Object.create?Object.create:function(a){function b(){}b.prototype=a;return new b},ja;if(\"function\"==typeof Object.setPrototypeOf)ja=Object.setPrototypeOf;else{var ka;a:{var la={V:!0},ma={};try{ma.__proto__=la;ka=ma.V;break a}catch(a){}ka=!1}ja=ka?function(a,b){a.__proto__=b;if(a.__proto__!==b)throw new TypeError(a+\" is not extensible\");return a}:null}var na=ja;\nfunction oa(a,b){a.prototype=ia(b.prototype);a.prototype.constructor=a;if(na)na(a,b);else for(var c in b)if(\"prototype\"!=c)if(Object.defineProperties){var d=Object.getOwnPropertyDescriptor(b,c);d&&Object.defineProperty(a,c,d)}else a[c]=b[c];a.O=b.prototype}\nfunction pa(a,b){u();a instanceof String&&(a+=\"\");var c=0,d={next:function(){if(c<a.length){var f=c++;return{value:b(f,a[f]),done:!1}}d.next=function(){return{done:!0,value:void 0}};return d.next()}};d[Symbol.iterator]=function(){return d};return d}function v(a,b){if(b){var c=r;a=a.split(\".\");for(var d=0;d<a.length-1;d++){var f=a[d];f in c||(c[f]={});c=c[f]}a=a[a.length-1];d=c[a];b=b(d);b!=d&&null!=b&&ba(c,a,{configurable:!0,writable:!0,value:b})}}\nv(\"Array.prototype.keys\",function(a){return a?a:function(){return pa(this,function(b){return b})}});v(\"Array.prototype.find\",function(a){return a?a:function(b,c){a:{var d=this;d instanceof String&&(d=String(d));for(var f=d.length,g=0;g<f;g++){var e=d[g];if(b.call(c,e,g,d)){b=e;break a}}b=void 0}return b}});v(\"Object.is\",function(a){return a?a:function(b,c){return b===c?0!==b||1/b===1/c:b!==b&&c!==c}});\nv(\"Array.prototype.includes\",function(a){return a?a:function(b,c){var d=this;d instanceof String&&(d=String(d));var f=d.length;c=c||0;for(0>c&&(c=Math.max(c+f,0));c<f;c++){var g=d[c];if(g===b||Object.is(g,b))return!0}return!1}});\nv(\"Promise\",function(a){function b(e){this.b=0;this.c=void 0;this.a=[];var h=this.f();try{e(h.resolve,h.reject)}catch(k){h.reject(k)}}function c(){this.a=null}function d(e){return e instanceof b?e:new b(function(h){h(e)})}if(a)return a;c.prototype.b=function(e){if(null==this.a){this.a=[];var h=this;this.c(function(){h.g()})}this.a.push(e)};var f=r.setTimeout;c.prototype.c=function(e){f(e,0)};c.prototype.g=function(){for(;this.a&&this.a.length;){var e=this.a;this.a=[];for(var h=0;h<e.length;++h){var k=\ne[h];e[h]=null;try{k()}catch(l){this.f(l)}}}this.a=null};c.prototype.f=function(e){this.c(function(){throw e;})};b.prototype.f=function(){function e(l){return function(m){k||(k=!0,l.call(h,m))}}var h=this,k=!1;return{resolve:e(this.s),reject:e(this.g)}};b.prototype.s=function(e){if(e===this)this.g(new TypeError(\"A Promise cannot resolve to itself\"));else if(e instanceof b)this.v(e);else{a:switch(typeof e){case \"object\":var h=null!=e;break a;case \"function\":h=!0;break a;default:h=!1}h?this.m(e):this.h(e)}};\nb.prototype.m=function(e){var h=void 0;try{h=e.then}catch(k){this.g(k);return}\"function\"==typeof h?this.w(h,e):this.h(e)};b.prototype.g=function(e){this.i(2,e)};b.prototype.h=function(e){this.i(1,e)};b.prototype.i=function(e,h){if(0!=this.b)throw Error(\"Cannot settle(\"+e+\", \"+h+\"): Promise already settled in state\"+this.b);this.b=e;this.c=h;this.l()};b.prototype.l=function(){if(null!=this.a){for(var e=0;e<this.a.length;++e)g.b(this.a[e]);this.a=null}};var g=new c;b.prototype.v=function(e){var h=this.f();\ne.F(h.resolve,h.reject)};b.prototype.w=function(e,h){var k=this.f();try{e.call(h,k.resolve,k.reject)}catch(l){k.reject(l)}};b.prototype.then=function(e,h){function k(q,w){return\"function\"==typeof q?function(A){try{l(q(A))}catch(L){m(L)}}:w}var l,m,p=new b(function(q,w){l=q;m=w});this.F(k(e,l),k(h,m));return p};b.prototype.catch=function(e){return this.then(void 0,e)};b.prototype.F=function(e,h){function k(){switch(l.b){case 1:e(l.c);break;case 2:h(l.c);break;default:throw Error(\"Unexpected state: \"+\nl.b);}}var l=this;null==this.a?g.b(k):this.a.push(k)};b.resolve=d;b.reject=function(e){return new b(function(h,k){k(e)})};b.race=function(e){return new b(function(h,k){for(var l=ha(e),m=l.next();!m.done;m=l.next())d(m.value).F(h,k)})};b.all=function(e){var h=ha(e),k=h.next();return k.done?d([]):new b(function(l,m){function p(A){return function(L){q[A]=L;w--;0==w&&l(q)}}var q=[],w=0;do q.push(void 0),w++,d(k.value).F(p(q.length-1),m),k=h.next();while(!k.done)})};return b});var qa=qa||{},x=this||self;\nfunction y(a,b){a=a.split(\".\");b=b||x;for(var c=0;c<a.length;c++)if(b=b[a[c]],null==b)return null;return b}function ra(){}function sa(a){var b=typeof a;return\"object\"==b&&null!=a||\"function\"==b}var ta=\"closure_uid_\"+(1E9*Math.random()>>>0),ua=0;function va(a,b,c){return a.call.apply(a.bind,arguments)}\nfunction wa(a,b,c){if(!a)throw Error();if(2<arguments.length){var d=Array.prototype.slice.call(arguments,2);return function(){var f=Array.prototype.slice.call(arguments);Array.prototype.unshift.apply(f,d);return a.apply(b,f)}}return function(){return a.apply(b,arguments)}}function z(a,b,c){Function.prototype.bind&&-1!=Function.prototype.bind.toString().indexOf(\"native code\")?z=va:z=wa;return z.apply(null,arguments)}\nfunction B(a,b){function c(){}c.prototype=b.prototype;a.O=b.prototype;a.prototype=new c;a.prototype.constructor=a};function xa(a){this.a=a||{}}xa.prototype.get=function(a){return this.a[a]};xa.prototype.G=function(){return Object.keys(this.a)};function C(a,b,c,d){this.f=a;this.c=b;this.b=c;this.a=d}C.prototype.getRequestMessage=function(){return this.f};C.prototype.getMethodDescriptor=function(){return this.c};C.prototype.getMetadata=function(){return this.b};C.prototype.getCallOptions=function(){return this.a};function D(a,b,c,d){c=void 0===c?{}:c;this.c=a;this.a=c;this.b=b;this.f=void 0===d?null:d}D.prototype.getResponseMessage=function(){return this.c};D.prototype.getMetadata=function(){return this.a};D.prototype.getMethodDescriptor=function(){return this.b};D.prototype.getStatus=function(){return this.f};function ya(a,b,c,d,f,g){this.name=a;this.a=f;this.b=g}function za(a,b,c){c=void 0===c?{}:c;var d=void 0===d?new xa:d;return new C(b,a,c,d)}ya.prototype.getName=function(){return this.name};ya.prototype.getName=ya.prototype.getName;function Aa(a){switch(a){case 200:return 0;case 400:return 3;case 401:return 16;case 403:return 7;case 404:return 5;case 409:return 10;case 412:return 9;case 429:return 8;case 499:return 1;case 500:return 2;case 501:return 12;case 503:return 14;case 504:return 4;default:return 2}}\nfunction Ba(a){switch(a){case 0:return\"OK\";case 1:return\"CANCELLED\";case 2:return\"UNKNOWN\";case 3:return\"INVALID_ARGUMENT\";case 4:return\"DEADLINE_EXCEEDED\";case 5:return\"NOT_FOUND\";case 6:return\"ALREADY_EXISTS\";case 7:return\"PERMISSION_DENIED\";case 16:return\"UNAUTHENTICATED\";case 8:return\"RESOURCE_EXHAUSTED\";case 9:return\"FAILED_PRECONDITION\";case 10:return\"ABORTED\";case 11:return\"OUT_OF_RANGE\";case 12:return\"UNIMPLEMENTED\";case 13:return\"INTERNAL\";case 14:return\"UNAVAILABLE\";case 15:return\"DATA_LOSS\";\ndefault:return\"\"}};function E(a,b,c){c=void 0===c?{}:c;b=Error.call(this,b);this.message=b.message;\"stack\"in b&&(this.stack=b.stack);this.code=a;this.metadata=c}oa(E,Error);E.prototype.toString=function(){var a=\"RpcError(\"+(Ba(this.code)||String(this.code))+\")\";this.message&&(a+=\": \"+this.message);return a};E.prototype.name=\"RpcError\";function Ca(a){this.a=a}Ca.prototype.on=function(a,b){return\"data\"==a||\"error\"==a?this:this.a.on(a,b)};Ca.prototype.removeListener=function(a,b){return this.a.removeListener(a,b)};Ca.prototype.cancel=function(){this.a.cancel()};function Da(a){switch(a){case 0:return\"No Error\";case 1:return\"Access denied to content document\";case 2:return\"File not found\";case 3:return\"Firefox silently errored\";case 4:return\"Application custom error\";case 5:return\"An exception occurred\";case 6:return\"Http response at 400 or 500 level\";case 7:return\"Request was aborted\";case 8:return\"Request timed out\";case 9:return\"The resource is not available offline\";default:return\"Unrecognized error code\"}};function F(a){if(Error.captureStackTrace)Error.captureStackTrace(this,F);else{var b=Error().stack;b&&(this.stack=b)}a&&(this.message=String(a))}B(F,Error);F.prototype.name=\"CustomError\";function Ea(a,b){a=a.split(\"%s\");for(var c=\"\",d=a.length-1,f=0;f<d;f++)c+=a[f]+(f<b.length?b[f]:\"%s\");F.call(this,c+a[d])}B(Ea,F);Ea.prototype.name=\"AssertionError\";function Fa(a,b){throw new Ea(\"Failure\"+(a?\": \"+a:\"\"),Array.prototype.slice.call(arguments,1));};function Ga(){this.l=null;this.i=[];this.m=0;this.b=Ha;this.f=this.a=this.h=0;this.c=null;this.g=0}\nfunction Ia(a,b){function c(l){l==Ja?e.h=l:l==G?e.h=l:Ka(e,h,k,\"invalid frame byte\");e.b=La;e.a=0;e.f=0}function d(l){e.f++;e.a=(e.a<<8)+l;4==e.f&&(e.b=Ma,e.g=0,\"undefined\"!==typeof Uint8Array?e.c=new Uint8Array(e.a):e.c=Array(e.a),0==e.a&&g())}function f(l){e.c[e.g++]=l;e.g==e.a&&g()}function g(){var l={};l[e.h]=e.c;e.i.push(l);e.b=Ha}var e=a,h,k=0;for(b instanceof Uint8Array||b instanceof Array?h=b:h=new Uint8Array(b);k<h.length;){switch(e.b){case Na:Ka(e,h,k,\"stream already broken\");break;case Ha:c(h[k]);\nbreak;case La:d(h[k]);break;case Ma:f(h[k]);break;default:throw Error(\"unexpected parser state: \"+e.b);}e.m++;k++}a=e.i;e.i=[];return 0<a.length?a:null}var Ha=0,La=1,Ma=2,Na=3,Ja=0,G=128;function Ka(a,b,c,d){a.b=Na;a.l=\"The stream is broken @\"+a.m+\"/\"+c+\". Error: \"+d+\". With input:\\n\"+b;throw Error(a.l);};var Oa=Array.prototype.indexOf?function(a,b){return Array.prototype.indexOf.call(a,b,void 0)}:function(a,b){if(\"string\"===typeof a)return\"string\"!==typeof b||1!=b.length?-1:a.indexOf(b,0);for(var c=0;c<a.length;c++)if(c in a&&a[c]===b)return c;return-1};var Pa=String.prototype.trim?function(a){return a.trim()}:function(a){return/^[\\s\\xa0]*([\\s\\S]*?)[\\s\\xa0]*$/.exec(a)[1]};function H(a,b){return-1!=a.indexOf(b)}function Qa(a,b){return a<b?-1:a>b?1:0};var I;a:{var Ra=x.navigator;if(Ra){var Sa=Ra.userAgent;if(Sa){I=Sa;break a}}I=\"\"};function Ta(a,b){for(var c in a)b.call(void 0,a[c],c,a)}function Ua(a,b){var c={},d;for(d in a)c[d]=b.call(void 0,a[d],d,a);return c}var Va=\"constructor hasOwnProperty isPrototypeOf propertyIsEnumerable toLocaleString toString valueOf\".split(\" \");function Wa(a,b){for(var c,d,f=1;f<arguments.length;f++){d=arguments[f];for(c in d)a[c]=d[c];for(var g=0;g<Va.length;g++)c=Va[g],Object.prototype.hasOwnProperty.call(d,c)&&(a[c]=d[c])}};function Xa(a){var b=1;a=a.split(\":\");for(var c=[];0<b&&a.length;)c.push(a.shift()),b--;a.length&&c.push(a.join(\":\"));return c};function Ya(a){Ya[\" \"](a);return a}Ya[\" \"]=ra;function Za(a){var b=$a;return Object.prototype.hasOwnProperty.call(b,9)?b[9]:b[9]=a(9)};var ab=H(I,\"Opera\"),bb=H(I,\"Trident\")||H(I,\"MSIE\"),cb=H(I,\"Edge\"),db=H(I,\"Gecko\")&&!(H(I.toLowerCase(),\"webkit\")&&!H(I,\"Edge\"))&&!(H(I,\"Trident\")||H(I,\"MSIE\"))&&!H(I,\"Edge\"),eb=H(I.toLowerCase(),\"webkit\")&&!H(I,\"Edge\"),fb;\na:{var gb=\"\",hb=function(){var a=I;if(db)return/rv:([^\\);]+)(\\)|;)/.exec(a);if(cb)return/Edge\\/([\\d\\.]+)/.exec(a);if(bb)return/\\b(?:MSIE|rv)[: ]([^\\);]+)(\\)|;)/.exec(a);if(eb)return/WebKit\\/(\\S+)/.exec(a);if(ab)return/(?:Version)[ \\/]?(\\S+)/.exec(a)}();hb&&(gb=hb?hb[1]:\"\");if(bb){var ib,jb=x.document;ib=jb?jb.documentMode:void 0;if(null!=ib&&ib>parseFloat(gb)){fb=String(ib);break a}}fb=gb}var $a={};\nfunction kb(){return Za(function(){for(var a=0,b=Pa(String(fb)).split(\".\"),c=Pa(\"9\").split(\".\"),d=Math.max(b.length,c.length),f=0;0==a&&f<d;f++){var g=b[f]||\"\",e=c[f]||\"\";do{g=/(\\d*)(\\D*)(.*)/.exec(g)||[\"\",\"\",\"\",\"\"];e=/(\\d*)(\\D*)(.*)/.exec(e)||[\"\",\"\",\"\",\"\"];if(0==g[0].length&&0==e[0].length)break;a=Qa(0==g[1].length?0:parseInt(g[1],10),0==e[1].length?0:parseInt(e[1],10))||Qa(0==g[2].length,0==e[2].length)||Qa(g[2],e[2]);g=g[3];e=e[3]}while(0==a)}return 0<=a})};function lb(){0!=mb&&(Object.prototype.hasOwnProperty.call(this,ta)&&this[ta]||(this[ta]=++ua));this.K=this.K}var mb=0;lb.prototype.K=!1;var nb=Object.freeze||function(a){return a};function J(a,b){this.type=a;this.a=this.target=b;this.defaultPrevented=!1}J.prototype.b=function(){this.defaultPrevented=!0};var ob=function(){if(!x.addEventListener||!Object.defineProperty)return!1;var a=!1,b=Object.defineProperty({},\"passive\",{get:function(){a=!0}});try{x.addEventListener(\"test\",ra,b),x.removeEventListener(\"test\",ra,b)}catch(c){}return a}();function K(a,b){J.call(this,a?a.type:\"\");this.relatedTarget=this.a=this.target=null;this.button=this.screenY=this.screenX=this.clientY=this.clientX=0;this.key=\"\";this.metaKey=this.shiftKey=this.altKey=this.ctrlKey=!1;this.pointerId=0;this.pointerType=\"\";this.c=null;if(a){var c=this.type=a.type,d=a.changedTouches&&a.changedTouches.length?a.changedTouches[0]:null;this.target=a.target||a.srcElement;this.a=b;if(b=a.relatedTarget){if(db){a:{try{Ya(b.nodeName);var f=!0;break a}catch(g){}f=!1}f||(b=null)}}else\"mouseover\"==\nc?b=a.fromElement:\"mouseout\"==c&&(b=a.toElement);this.relatedTarget=b;d?(this.clientX=void 0!==d.clientX?d.clientX:d.pageX,this.clientY=void 0!==d.clientY?d.clientY:d.pageY,this.screenX=d.screenX||0,this.screenY=d.screenY||0):(this.clientX=void 0!==a.clientX?a.clientX:a.pageX,this.clientY=void 0!==a.clientY?a.clientY:a.pageY,this.screenX=a.screenX||0,this.screenY=a.screenY||0);this.button=a.button;this.key=a.key||\"\";this.ctrlKey=a.ctrlKey;this.altKey=a.altKey;this.shiftKey=a.shiftKey;this.metaKey=\na.metaKey;this.pointerId=a.pointerId||0;this.pointerType=\"string\"===typeof a.pointerType?a.pointerType:pb[a.pointerType]||\"\";this.c=a;a.defaultPrevented&&K.O.b.call(this)}}B(K,J);var pb=nb({2:\"touch\",3:\"pen\",4:\"mouse\"});K.prototype.b=function(){K.O.b.call(this);var a=this.c;a.preventDefault?a.preventDefault():a.returnValue=!1};var M=\"closure_listenable_\"+(1E6*Math.random()|0);var qb=0;function rb(a,b,c,d,f){this.listener=a;this.proxy=null;this.src=b;this.type=c;this.capture=!!d;this.H=f;this.key=++qb;this.A=this.D=!1}function sb(a){a.A=!0;a.listener=null;a.proxy=null;a.src=null;a.H=null};function tb(a){this.src=a;this.a={};this.b=0}tb.prototype.add=function(a,b,c,d,f){var g=a.toString();a=this.a[g];a||(a=this.a[g]=[],this.b++);var e=ub(a,b,d,f);-1<e?(b=a[e],c||(b.D=!1)):(b=new rb(b,this.src,g,!!d,f),b.D=c,a.push(b));return b};tb.prototype.remove=function(a,b,c,d){a=a.toString();if(!(a in this.a))return!1;var f=this.a[a];b=ub(f,b,c,d);return-1<b?(sb(f[b]),Array.prototype.splice.call(f,b,1),0==f.length&&(delete this.a[a],this.b--),!0):!1};\nfunction vb(a,b){var c=b.type;if(c in a.a){var d=a.a[c],f=Oa(d,b),g;(g=0<=f)&&Array.prototype.splice.call(d,f,1);g&&(sb(b),0==a.a[c].length&&(delete a.a[c],a.b--))}}function ub(a,b,c,d){for(var f=0;f<a.length;++f){var g=a[f];if(!g.A&&g.listener==b&&g.capture==!!c&&g.H==d)return f}return-1};var wb=\"closure_lm_\"+(1E6*Math.random()|0),xb={},yb=0;function zb(a,b,c,d,f){if(d&&d.once)Ab(a,b,c,d,f);else if(Array.isArray(b))for(var g=0;g<b.length;g++)zb(a,b[g],c,d,f);else c=Bb(c),a&&a[M]?a.f.add(String(b),c,!1,sa(d)?!!d.capture:!!d,f):Cb(a,b,c,!1,d,f)}\nfunction Cb(a,b,c,d,f,g){if(!b)throw Error(\"Invalid event type\");var e=sa(f)?!!f.capture:!!f,h=Db(a);h||(a[wb]=h=new tb(a));c=h.add(b,c,d,e,g);if(!c.proxy){d=Eb();c.proxy=d;d.src=a;d.listener=c;if(a.addEventListener)ob||(f=e),void 0===f&&(f=!1),a.addEventListener(b.toString(),d,f);else if(a.attachEvent)a.attachEvent(Fb(b.toString()),d);else if(a.addListener&&a.removeListener)a.addListener(d);else throw Error(\"addEventListener and attachEvent are unavailable.\");yb++}}\nfunction Eb(){function a(c){return b.call(a.src,a.listener,c)}var b=Gb;return a}function Ab(a,b,c,d,f){if(Array.isArray(b))for(var g=0;g<b.length;g++)Ab(a,b[g],c,d,f);else c=Bb(c),a&&a[M]?a.f.add(String(b),c,!0,sa(d)?!!d.capture:!!d,f):Cb(a,b,c,!0,d,f)}function Hb(a,b,c,d,f){if(Array.isArray(b))for(var g=0;g<b.length;g++)Hb(a,b[g],c,d,f);else(d=sa(d)?!!d.capture:!!d,c=Bb(c),a&&a[M])?a.f.remove(String(b),c,d,f):a&&(a=Db(a))&&(b=a.a[b.toString()],a=-1,b&&(a=ub(b,c,d,f)),(c=-1<a?b[a]:null)&&Ib(c))}\nfunction Ib(a){if(\"number\"!==typeof a&&a&&!a.A){var b=a.src;if(b&&b[M])vb(b.f,a);else{var c=a.type,d=a.proxy;b.removeEventListener?b.removeEventListener(c,d,a.capture):b.detachEvent?b.detachEvent(Fb(c),d):b.addListener&&b.removeListener&&b.removeListener(d);yb--;(c=Db(b))?(vb(c,a),0==c.b&&(c.src=null,b[wb]=null)):sb(a)}}}function Fb(a){return a in xb?xb[a]:xb[a]=\"on\"+a}function Gb(a,b){if(a.A)a=!0;else{b=new K(b,this);var c=a.listener,d=a.H||a.src;a.D&&Ib(a);a=c.call(d,b)}return a}\nfunction Db(a){a=a[wb];return a instanceof tb?a:null}var Jb=\"__closure_events_fn_\"+(1E9*Math.random()>>>0);function Bb(a){if(\"function\"===typeof a)return a;a[Jb]||(a[Jb]=function(b){return a.handleEvent(b)});return a[Jb]};function N(){lb.call(this);this.f=new tb(this);this.U=this}B(N,lb);N.prototype[M]=!0;N.prototype.addEventListener=function(a,b,c,d){zb(this,a,b,c,d)};N.prototype.removeEventListener=function(a,b,c,d){Hb(this,a,b,c,d)};function O(a,b){a=a.U;var c=b.type||b;if(\"string\"===typeof b)b=new J(b,a);else if(b instanceof J)b.target=b.target||a;else{var d=b;b=new J(c,a);Wa(b,d)}a=b.a=a;Kb(a,c,!0,b);Kb(a,c,!1,b)}\nfunction Kb(a,b,c,d){if(b=a.f.a[String(b)]){b=b.concat();for(var f=!0,g=0;g<b.length;++g){var e=b[g];if(e&&!e.A&&e.capture==c){var h=e.listener,k=e.H||e.src;e.D&&vb(a.f,e);f=!1!==h.call(k,d)&&f}}}};var Lb=x;function Mb(a,b,c){if(\"function\"===typeof a)c&&(a=z(a,c));else if(a&&\"function\"==typeof a.handleEvent)a=z(a.handleEvent,a);else throw Error(\"Invalid listener argument\");return 2147483647<Number(b)?-1:Lb.setTimeout(a,b||0)};function Nb(a,b){this.name=a;this.value=b}Nb.prototype.toString=function(){return this.name};var Ob=new Nb(\"OFF\",Infinity),Pb=new Nb(\"SEVERE\",1E3),Qb=new Nb(\"CONFIG\",700),Rb=new Nb(\"FINE\",500);function Tb(){this.clear()}var Ub;Tb.prototype.clear=function(){};function Vb(a,b,c){this.reset(a||Ob,b,c,void 0,void 0)}Vb.prototype.reset=function(){};function Wb(a,b){this.a=null;this.f=[];this.b=(void 0===b?null:b)||null;this.c=[];this.g={getName:function(){return a}}}\nfunction Xb(a){if(a.a)return a.a;if(a.b)return Xb(a.b);Fa(\"Root logger has no level set.\");return Ob}function Yb(a,b){for(;a;)a.f.forEach(function(c){c(b)}),a=a.b}function Zb(){this.entries={};var a=new Wb(\"\");a.a=Qb;this.entries[\"\"]=a}var $b;function ac(a,b,c){var d=a.entries[b];if(d)return void 0!==c&&(d.a=c),d;d=ac(a,b.substr(0,b.lastIndexOf(\".\")));var f=new Wb(b,d);a.entries[b]=f;d.c.push(f);void 0!==c&&(f.a=c);return f}function bc(){$b||($b=new Zb);return $b}\nfunction cc(a,b,c){var d;if(d=a)if(d=a&&b){d=b.value;var f=a?Xb(ac(bc(),a.getName())):Ob;d=d>=f.value}d&&(b=b||Ob,d=ac(bc(),a.getName()),\"function\"===typeof c&&(c=c()),Ub||(Ub=new Tb),a=a.getName(),a=new Vb(b,c,a),Yb(d,a))}function P(a,b){a&&cc(a,Rb,b)};function dc(){}dc.prototype.a=null;function ec(a){var b;(b=a.a)||(b={},fc(a)&&(b[0]=!0,b[1]=!0),b=a.a=b);return b};var gc;function hc(){}B(hc,dc);function ic(a){return(a=fc(a))?new ActiveXObject(a):new XMLHttpRequest}function fc(a){if(!a.b&&\"undefined\"==typeof XMLHttpRequest&&\"undefined\"!=typeof ActiveXObject){for(var b=[\"MSXML2.XMLHTTP.6.0\",\"MSXML2.XMLHTTP.3.0\",\"MSXML2.XMLHTTP\",\"Microsoft.XMLHTTP\"],c=0;c<b.length;c++){var d=b[c];try{return new ActiveXObject(d),a.b=d}catch(f){}}throw Error(\"Could not create ActiveXObject. ActiveX might be disabled, or MSXML might not be installed\");}return a.b}gc=new hc;t();u();function jc(a,b){this.b=a[x.Symbol.iterator]();this.c=b;this.f=0}jc.prototype[Symbol.iterator]=function(){return this};jc.prototype.next=function(){var a=this.b.next();return{value:a.done?void 0:this.c.call(void 0,a.value,this.f++),done:a.done}};function kc(a,b){return new jc(a,b)}t();u();t();u();var lc=\"StopIteration\"in x?x.StopIteration:{message:\"StopIteration\",stack:\"\"};function Q(){}Q.prototype.next=function(){return Q.prototype.a.call(this)};Q.prototype.a=function(){throw lc;};Q.prototype.u=function(){return this};function mc(a){if(a instanceof R||a instanceof S||a instanceof T)return a;if(\"function\"==typeof a.next)return new R(function(){return nc(a)});t();u();if(\"function\"==typeof a[Symbol.iterator])return t(),u(),new R(function(){return a[Symbol.iterator]()});if(\"function\"==typeof a.u)return new R(function(){return nc(a.u())});throw Error(\"Not an iterator or iterable.\");}\nfunction nc(a){if(!(a instanceof Q))return a;var b=!1;return{next:function(){for(var c;!b;)try{c=a.a();break}catch(d){if(d!==lc)throw d;b=!0}return{value:c,done:b}}}}t();u();function R(a){this.b=a}R.prototype.u=function(){return new S(this.b())};R.prototype[Symbol.iterator]=function(){return new T(this.b())};R.prototype.c=function(){return new T(this.b())};t();u();function S(a){this.b=a}oa(S,Q);S.prototype.a=function(){var a=this.b.next();if(a.done)throw lc;return a.value};S.prototype.next=function(){return S.prototype.a.call(this)};\nS.prototype[Symbol.iterator]=function(){return new T(this.b)};S.prototype.c=function(){return new T(this.b)};function T(a){R.call(this,function(){return a});this.f=a}oa(T,R);T.prototype.next=function(){return this.f.next()};function oc(a,b){this.o={};this.j=[];this.B=this.size=0;var c=arguments.length;if(1<c){if(c%2)throw Error(\"Uneven number of arguments\");for(var d=0;d<c;d+=2)this.set(arguments[d],arguments[d+1])}else a&&this.addAll(a)}n=oc.prototype;n.G=function(){pc(this);return this.j.concat()};n.has=function(a){return U(this.o,a)};n.clear=function(){this.o={};this.B=this.size=this.j.length=0};n.remove=function(a){return this.delete(a)};\nn.delete=function(a){return U(this.o,a)?(delete this.o[a],--this.size,this.B++,this.j.length>2*this.size&&pc(this),!0):!1};function pc(a){if(a.size!=a.j.length){for(var b=0,c=0;b<a.j.length;){var d=a.j[b];U(a.o,d)&&(a.j[c++]=d);b++}a.j.length=c}if(a.size!=a.j.length){var f={};for(c=b=0;b<a.j.length;)d=a.j[b],U(f,d)||(a.j[c++]=d,f[d]=1),b++;a.j.length=c}}n.get=function(a,b){return U(this.o,a)?this.o[a]:b};n.set=function(a,b){U(this.o,a)||(this.size+=1,this.j.push(a),this.B++);this.o[a]=b};\nn.addAll=function(a){if(a instanceof oc)for(var b=a.G(),c=0;c<b.length;c++)this.set(b[c],a.get(b[c]));else for(b in a)this.set(b,a[b])};n.forEach=function(a,b){for(var c=this.G(),d=0;d<c.length;d++){var f=c[d],g=this.get(f);a.call(b,g,f,this)}};n.clone=function(){return new oc(this)};n.keys=function(){return mc(this.u(!0)).c()};n.values=function(){return mc(this.u(!1)).c()};n.entries=function(){var a=this;return kc(this.keys(),function(b){return[b,a.get(b)]})};\nn.u=function(a){pc(this);var b=0,c=this.B,d=this,f=new Q;f.a=function(){if(c!=d.B)throw Error(\"The map has changed since the iterator was created\");if(b>=d.j.length)throw lc;var g=d.j[b++];return a?g:d.o[g]};f.next=f.a.bind(f);return f};function U(a,b){return Object.prototype.hasOwnProperty.call(a,b)};var qc=/^(?:([^:/?#.]+):)?(?:\\/\\/(?:([^\\\\/?#]*)@)?([^\\\\/?#]*?)(?::([0-9]+))?(?=[\\\\/?#]|$))?([^?#]+)?(?:\\?([^#]*))?(?:#([\\s\\S]*))?$/;function rc(a){N.call(this);this.headers=new oc;this.C=a||null;this.c=!1;this.J=this.a=null;this.P=this.v=\"\";this.g=0;this.l=\"\";this.i=this.N=this.s=this.L=!1;this.h=0;this.w=null;this.m=sc;this.I=this.M=!1}B(rc,N);var sc=\"\";rc.prototype.b=ac(bc(),\"goog.net.XhrIo\",void 0).g;var tc=/^https?$/i,uc=[\"POST\",\"PUT\"];\nfunction vc(a,b,c){if(a.a)throw Error(\"[goog.net.XhrIo] Object is active with another request=\"+a.v+\"; newUri=\"+b);a.v=b;a.l=\"\";a.g=0;a.P=\"POST\";a.L=!1;a.c=!0;a.a=a.C?ic(a.C):ic(gc);a.J=a.C?ec(a.C):ec(gc);a.a.onreadystatechange=z(a.R,a);try{P(a.b,V(a,\"Opening Xhr\")),a.N=!0,a.a.open(\"POST\",String(b),!0),a.N=!1}catch(g){P(a.b,V(a,\"Error opening Xhr: \"+g.message));wc(a,g);return}b=c||\"\";c=a.headers.clone();var d=c.G().find(function(g){return\"content-type\"==g.toLowerCase()}),f=x.FormData&&b instanceof\nx.FormData;!(0<=Oa(uc,\"POST\"))||d||f||c.set(\"Content-Type\",\"application/x-www-form-urlencoded;charset=utf-8\");c.forEach(function(g,e){this.a.setRequestHeader(e,g)},a);a.m&&(a.a.responseType=a.m);\"withCredentials\"in a.a&&a.a.withCredentials!==a.M&&(a.a.withCredentials=a.M);try{xc(a),0<a.h&&(a.I=yc(a.a),P(a.b,V(a,\"Will abort after \"+a.h+\"ms if incomplete, xhr2 \"+a.I)),a.I?(a.a.timeout=a.h,a.a.ontimeout=z(a.T,a)):a.w=Mb(a.T,a.h,a)),P(a.b,V(a,\"Sending request\")),a.s=!0,a.a.send(b),a.s=!1}catch(g){P(a.b,\nV(a,\"Send error: \"+g.message)),wc(a,g)}}function yc(a){return bb&&kb()&&\"number\"===typeof a.timeout&&void 0!==a.ontimeout}n=rc.prototype;n.T=function(){\"undefined\"!=typeof qa&&this.a&&(this.l=\"Timed out after \"+this.h+\"ms, aborting\",this.g=8,P(this.b,V(this,this.l)),O(this,\"timeout\"),this.abort(8))};function wc(a,b){a.c=!1;a.a&&(a.i=!0,a.a.abort(),a.i=!1);a.l=b;a.g=5;zc(a);Ac(a)}function zc(a){a.L||(a.L=!0,O(a,\"complete\"),O(a,\"error\"))}\nn.abort=function(a){this.a&&this.c&&(P(this.b,V(this,\"Aborting\")),this.c=!1,this.i=!0,this.a.abort(),this.i=!1,this.g=a||7,O(this,\"complete\"),O(this,\"abort\"),Ac(this))};n.R=function(){this.K||(this.N||this.s||this.i?Bc(this):this.W())};n.W=function(){Bc(this)};\nfunction Bc(a){if(a.c&&\"undefined\"!=typeof qa)if(a.J[1]&&4==W(a)&&2==a.getStatus())P(a.b,V(a,\"Local request error detected and ignored\"));else if(a.s&&4==W(a))Mb(a.R,0,a);else if(O(a,\"readystatechange\"),4==W(a)){P(a.b,V(a,\"Request complete\"));a.c=!1;try{var b=a.getStatus();a:switch(b){case 200:case 201:case 202:case 204:case 206:case 304:case 1223:var c=!0;break a;default:c=!1}var d;if(!(d=c)){var f;if(f=0===b){var g=String(a.v).match(qc)[1]||null;if(!g&&x.self&&x.self.location){var e=x.self.location.protocol;\ng=e.substr(0,e.length-1)}f=!tc.test(g?g.toLowerCase():\"\")}d=f}if(d)O(a,\"complete\"),O(a,\"success\");else{a.g=6;try{var h=2<W(a)?a.a.statusText:\"\"}catch(k){P(a.b,\"Can not get status: \"+k.message),h=\"\"}a.l=h+\" [\"+a.getStatus()+\"]\";zc(a)}}finally{Ac(a)}}}function Ac(a){if(a.a){xc(a);var b=a.a,c=a.J[0]?ra:null;a.a=null;a.J=null;O(a,\"ready\");try{b.onreadystatechange=c}catch(d){(a=a.b)&&cc(a,Pb,\"Problem encountered resetting onreadystatechange: \"+d.message)}}}\nfunction xc(a){a.a&&a.I&&(a.a.ontimeout=null);a.w&&(Lb.clearTimeout(a.w),a.w=null)}function W(a){return a.a?a.a.readyState:0}n.getStatus=function(){try{return 2<W(this)?this.a.status:-1}catch(a){return-1}};\nfunction Cc(a){try{if(!a.a)return null;if(\"response\"in a.a)return a.a.response;switch(a.m){case sc:case \"text\":return a.a.responseText;case \"arraybuffer\":if(\"mozResponseArrayBuffer\"in a.a)return a.a.mozResponseArrayBuffer}var b=a.b;b&&cc(b,Pb,\"Response type \"+a.m+\" is not supported on this browser\");return null}catch(c){return P(a.b,\"Can not get response: \"+c.message),null}}\nfunction Dc(a){var b={};a=(a.a&&4==W(a)?a.a.getAllResponseHeaders()||\"\":\"\").split(\"\\r\\n\");for(var c=0;c<a.length;c++)if(!/^[\\s\\xa0]*$/.test(a[c])){var d=Xa(a[c]),f=d[0];d=d[1];if(\"string\"===typeof d){d=d.trim();var g=b[f]||[];b[f]=g;g.push(d)}}return Ua(b,function(e){return e.join(\", \")})}function V(a,b){return b+\" [\"+a.P+\" \"+a.v+\" \"+a.getStatus()+\"]\"};var Ec={},Fc=null;function Gc(a){var b=a.length,c=3*b/4;c%3?c=Math.floor(c):H(\"=.\",a[b-1])&&(c=H(\"=.\",a[b-2])?c-2:c-1);var d=new Uint8Array(c),f=0;Hc(a,function(g){d[f++]=g});return d.subarray(0,f)}\nfunction Hc(a,b){function c(k){for(;d<a.length;){var l=a.charAt(d++),m=Fc[l];if(null!=m)return m;if(!/^[\\s\\xa0]*$/.test(l))throw Error(\"Unknown base64 encoding at char: \"+l);}return k}Ic();for(var d=0;;){var f=c(-1),g=c(0),e=c(64),h=c(64);if(64===h&&-1===f)break;b(f<<2|g>>4);64!=e&&(b(g<<4&240|e>>2),64!=h&&b(e<<6&192|h))}}\nfunction Ic(){if(!Fc){Fc={};for(var a=\"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\".split(\"\"),b=[\"+/=\",\"+/\",\"-_=\",\"-_.\",\"-_\"],c=0;5>c;c++){var d=a.concat(b[c].split(\"\"));Ec[c]=d;for(var f=0;f<d.length;f++){var g=d[f];void 0===Fc[g]&&(Fc[g]=f)}}}};var Jc=[\"content-type\",\"grpc-status\",\"grpc-message\"];\nfunction X(a){this.a=a.Z;this.m=null;this.b=[];this.h=[];this.g=[];this.f=[];this.c=[];this.l=!1;this.i=0;this.s=new Ga;var b=this;zb(this.a,\"readystatechange\",function(){var c=b.a;if(c=c.a?c.a.getResponseHeader(\"Content-Type\"):null){c=c.toLowerCase();if(0==c.lastIndexOf(\"application/grpc-web-text\",0)){c=b.a;try{var d=c.a?c.a.responseText:\"\"}catch(k){P(c.b,\"Can not get responseText: \"+k.message),d=\"\"}c=d||\"\";d=c.length-c.length%4;c=c.substr(b.i,d-b.i);if(0==c.length)return;b.i=d;c=Gc(c)}else if(0==\nc.lastIndexOf(\"application/grpc\",0))c=new Uint8Array(Cc(b.a));else{Y(b,new E(2,\"Unknown Content-type received.\"));return}d=null;try{d=Ia(b.s,c)}catch(k){Y(b,new E(2,\"Error in parsing response body\"))}if(d)for(c=0;c<d.length;c++){if(Ja in d[c]){var f=d[c][Ja];if(f){var g=!1,e=void 0;try{e=b.m(f),g=!0}catch(k){Y(b,new E(13,\"Error when deserializing response data; error: \"+k+(\", response: \"+e)))}if(g)for(f=e,g=0;g<b.b.length;g++)b.b[g](f)}}if(G in d[c]&&0<d[c][G].length){f=\"\";for(g=0;g<d[c][G].length;g++)f+=\nString.fromCharCode(d[c][G][g]);f=f.trim().split(\"\\r\\n\");g={};for(e=0;e<f.length;e++){var h=f[e].indexOf(\":\");g[f[e].substring(0,h).trim()]=f[e].substring(h+1).trim()}f=g;g=0;e=\"\";\"grpc-status\"in f&&(g=Number(f[\"grpc-status\"]),delete f[\"grpc-status\"]);\"grpc-message\"in f&&(e=f[\"grpc-message\"],delete f[\"grpc-message\"]);Y(b,new E(g,e,f))}}}});zb(this.a,\"complete\",function(){var c=b.a.g,d=2,f=\"\",g={};d=Dc(b.a);var e={};for(h in d)d.hasOwnProperty(h)&&(e[h.toLowerCase()]=d[h]);Object.keys(e).forEach(function(k){Jc.includes(k)||\n(g[k]=e[k])});Kc(b,g);var h=-1;if(0!=c){switch(c){case 7:d=10;break;case 8:d=4;break;case 6:h=b.a.getStatus();d=Aa(h);break;default:d=14}10==d&&b.l||(f=Da(c),-1!=h&&(f+=\", http status code: \"+h),Y(b,new E(d,f)))}else c=!1,\"grpc-status\"in e&&(d=Number(e[\"grpc-status\"]),\"grpc-message\"in e&&(f=e[\"grpc-message\"]),0!=d&&(Y(b,new E(d,f||\"\",e)),c=!0)),c||Lc(b)})}\nX.prototype.on=function(a,b){\"data\"==a?this.b.push(b):\"status\"==a?this.h.push(b):\"metadata\"==a?this.g.push(b):\"end\"==a?this.c.push(b):\"error\"==a&&this.f.push(b);return this};function Mc(a,b){b=a.indexOf(b);-1<b&&a.splice(b,1)}X.prototype.removeListener=function(a,b){\"data\"==a?Mc(this.b,b):\"status\"==a?Mc(this.h,b):\"metadata\"==a?Mc(this.g,b):\"end\"==a?Mc(this.c,b):\"error\"==a&&Mc(this.f,b);return this};X.prototype.cancel=function(){this.l=!0;this.a.abort()};\nfunction Y(a,b){if(0!=b.code)for(var c=new E(b.code,decodeURIComponent(b.message||\"\"),b.metadata),d=0;d<a.f.length;d++)a.f[d](c);b={code:b.code,details:decodeURIComponent(b.message||\"\"),metadata:b.metadata};for(c=0;c<a.h.length;c++)a.h[c](b)}function Kc(a,b){for(var c=0;c<a.g.length;c++)a.g[c](b)}function Lc(a){for(var b=0;b<a.c.length;b++)a.c[b]()}X.prototype.cancel=X.prototype.cancel;X.prototype.removeListener=X.prototype.removeListener;X.prototype.on=X.prototype.on;function Nc(a){var b=\"\";Ta(a,function(c,d){b+=d;b+=\":\";b+=c;b+=\"\\r\\n\"});return b};function Z(a,b){a=void 0===a?{}:a;this.a=a.format||y(\"format\",a)||\"text\";this.g=a.aa||y(\"suppressCorsPreflight\",a)||!1;this.f=a.withCredentials||y(\"withCredentials\",a)||!1;this.b=a.$||y(\"streamInterceptors\",a)||[];this.h=a.ba||y(\"unaryInterceptors\",a)||[];this.c=b||null}Z.prototype.X=function(a,b,c,d,f){var g=this,e=a.substr(0,a.length-d.name.length);a=Oc(function(h){return Pc(g,h,e)},this.b).call(this,za(d,b,c));Qc(a,f,!1);return new Ca(a)};\nZ.prototype.S=function(a,b,c,d){var f=this,g=a.substr(0,a.length-d.name.length);return Oc(function(e){return new Promise(function(h,k){var l=Pc(f,e,g),m,p,q;Qc(l,function(w,A,L,Sb,Rc){w?k(w):Rc?q=A:L?p=L:Sb?m=Sb:(w=e.getMethodDescriptor(),A=m,A=void 0===A?{}:A,h(new D(q,w,A,void 0===p?null:p)))},!0)})},this.h).call(this,za(d,b,c)).then(function(e){return e.getResponseMessage()})};Z.prototype.unaryCall=function(a,b,c,d){return this.S(a,b,c,d)};\nZ.prototype.Y=function(a,b,c,d){var f=this,g=a.substr(0,a.length-d.name.length);return Oc(function(e){return Pc(f,e,g)},this.b).call(this,za(d,b,c))};\nfunction Pc(a,b,c){var d=b.getMethodDescriptor(),f=c+d.getName();c=a.c?a.c:new rc;c.M=a.f;var g=new X({Z:c});g.m=d.b;var e=b.getMetadata();for(h in e)c.headers.set(h,e[h]);\"text\"==a.a?(c.headers.set(\"Content-Type\",\"application/grpc-web-text\"),c.headers.set(\"Accept\",\"application/grpc-web-text\")):c.headers.set(\"Content-Type\",\"application/grpc-web+proto\");c.headers.set(\"X-User-Agent\",\"grpc-web-javascript/0.1\");c.headers.set(\"X-Grpc-Web\",\"1\");if(c.headers.has(\"deadline\")){var h=Number(c.headers.get(\"deadline\"));\nh=Math.ceil(h-(new Date).getTime());c.headers.delete(\"deadline\");Infinity===h&&(h=0);0<h&&(c.headers.set(\"grpc-timeout\",h+\"m\"),c.h=Math.max(0,Math.max(1E3,Math.ceil(1.1*h))))}if(a.g){e=c.headers;h={};for(var k=ha(e.keys()),l=k.next();!l.done;l=k.next())l=l.value,h[l]=e.get(l);c.headers.clear();b:{for(m in h){var m=!1;break b}m=!0}if(!m)if(h=Nc(h),\"string\"===typeof f){if(m=encodeURIComponent(\"$httpHeaders\"),h=null!=h?\"=\"+encodeURIComponent(String(h)):\"\",m+=h)h=f.indexOf(\"#\"),0>h&&(h=f.length),e=f.indexOf(\"?\"),\n0>e||e>h?(e=h,k=\"\"):k=f.substring(e+1,h),f=[f.substr(0,e),k,f.substr(h)],h=f[1],f[1]=m?h?h+\"&\"+m:m:h,f=f[0]+(f[1]?\"?\"+f[1]:\"\")+f[2]}else f.a(\"$httpHeaders\",h)}b=(0,d.a)(b.getRequestMessage());d=b.length;m=[0,0,0,0];h=new Uint8Array(5+d);for(e=3;0<=e;e--)m[e]=d%256,d>>>=8;h.set(new Uint8Array(m),1);h.set(b,5);b=h;if(\"text\"==a.a){a=b;var p;void 0===p&&(p=0);Ic();p=Ec[p];b=Array(Math.floor(a.length/3));d=p[64]||\"\";for(m=h=0;h<a.length-2;h+=3){l=a[h];var q=a[h+1];k=a[h+2];e=p[l>>2];l=p[(l&3)<<4|q>>4];\nq=p[(q&15)<<2|k>>6];k=p[k&63];b[m++]=e+l+q+k}e=0;k=d;switch(a.length-h){case 2:e=a[h+1],k=p[(e&15)<<2]||d;case 1:a=a[h],b[m]=p[a>>2]+p[(a&3)<<4|e>>4]+k+d}b=b.join(\"\")}else\"binary\"==a.a&&(c.m=\"arraybuffer\");vc(c,f,b);return g}\nfunction Qc(a,b,c){var d=!1,f=null,g=!1;a.on(\"data\",function(e){d=!0;f=e});a.on(\"error\",function(e){0==e.code||g||(g=!0,b(e,null))});a.on(\"status\",function(e){0==e.code||g?c&&b(null,null,e):(g=!0,b({code:e.code,message:e.details,metadata:e.metadata},null))});if(c)a.on(\"metadata\",function(e){b(null,null,null,e)});a.on(\"end\",function(){g||(d?c?b(null,f,null,null,!0):b(null,f):b({code:2,message:\"Incomplete response\"}));c&&b(null,null)})}\nfunction Oc(a,b){var c=a;b.forEach(function(d){var f=c;c=function(g){return d.intercept(g,f)}});return c}Z.prototype.serverStreaming=Z.prototype.Y;Z.prototype.unaryCall=Z.prototype.unaryCall;Z.prototype.thenableCall=Z.prototype.S;Z.prototype.rpcCall=Z.prototype.X;module.exports.CallOptions=xa;module.exports.MethodDescriptor=ya;module.exports.GrpcWebClientBase=Z;module.exports.RpcError=E;module.exports.StatusCode={OK:0,CANCELLED:1,UNKNOWN:2,INVALID_ARGUMENT:3,DEADLINE_EXCEEDED:4,NOT_FOUND:5,ALREADY_EXISTS:6,PERMISSION_DENIED:7,UNAUTHENTICATED:16,RESOURCE_EXHAUSTED:8,FAILED_PRECONDITION:9,ABORTED:10,OUT_OF_RANGE:11,UNIMPLEMENTED:12,INTERNAL:13,UNAVAILABLE:14,DATA_LOSS:15};module.exports.MethodType={UNARY:\"unary\",SERVER_STREAMING:\"server_streaming\",BIDI_STREAMING:\"bidi_streaming\"};\nLb=\"undefined\"!==typeof globalThis&&globalThis||self;\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/grpc-web/index.js?");

/***/ }),

/***/ "./node_modules/hoist-non-react-statics/dist/hoist-non-react-statics.cjs.js":
/*!**********************************************************************************!*\
  !*** ./node_modules/hoist-non-react-statics/dist/hoist-non-react-statics.cjs.js ***!
  \**********************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nvar reactIs = __webpack_require__(/*! react-is */ \"./node_modules/hoist-non-react-statics/node_modules/react-is/index.js\");\n\n/**\n * Copyright 2015, Yahoo! Inc.\n * Copyrights licensed under the New BSD License. See the accompanying LICENSE file for terms.\n */\nvar REACT_STATICS = {\n  childContextTypes: true,\n  contextType: true,\n  contextTypes: true,\n  defaultProps: true,\n  displayName: true,\n  getDefaultProps: true,\n  getDerivedStateFromError: true,\n  getDerivedStateFromProps: true,\n  mixins: true,\n  propTypes: true,\n  type: true\n};\nvar KNOWN_STATICS = {\n  name: true,\n  length: true,\n  prototype: true,\n  caller: true,\n  callee: true,\n  arguments: true,\n  arity: true\n};\nvar FORWARD_REF_STATICS = {\n  '$$typeof': true,\n  render: true,\n  defaultProps: true,\n  displayName: true,\n  propTypes: true\n};\nvar MEMO_STATICS = {\n  '$$typeof': true,\n  compare: true,\n  defaultProps: true,\n  displayName: true,\n  propTypes: true,\n  type: true\n};\nvar TYPE_STATICS = {};\nTYPE_STATICS[reactIs.ForwardRef] = FORWARD_REF_STATICS;\nTYPE_STATICS[reactIs.Memo] = MEMO_STATICS;\n\nfunction getStatics(component) {\n  // React v16.11 and below\n  if (reactIs.isMemo(component)) {\n    return MEMO_STATICS;\n  } // React v16.12 and above\n\n\n  return TYPE_STATICS[component['$$typeof']] || REACT_STATICS;\n}\n\nvar defineProperty = Object.defineProperty;\nvar getOwnPropertyNames = Object.getOwnPropertyNames;\nvar getOwnPropertySymbols = Object.getOwnPropertySymbols;\nvar getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;\nvar getPrototypeOf = Object.getPrototypeOf;\nvar objectPrototype = Object.prototype;\nfunction hoistNonReactStatics(targetComponent, sourceComponent, blacklist) {\n  if (typeof sourceComponent !== 'string') {\n    // don't hoist over string (html) components\n    if (objectPrototype) {\n      var inheritedComponent = getPrototypeOf(sourceComponent);\n\n      if (inheritedComponent && inheritedComponent !== objectPrototype) {\n        hoistNonReactStatics(targetComponent, inheritedComponent, blacklist);\n      }\n    }\n\n    var keys = getOwnPropertyNames(sourceComponent);\n\n    if (getOwnPropertySymbols) {\n      keys = keys.concat(getOwnPropertySymbols(sourceComponent));\n    }\n\n    var targetStatics = getStatics(targetComponent);\n    var sourceStatics = getStatics(sourceComponent);\n\n    for (var i = 0; i < keys.length; ++i) {\n      var key = keys[i];\n\n      if (!KNOWN_STATICS[key] && !(blacklist && blacklist[key]) && !(sourceStatics && sourceStatics[key]) && !(targetStatics && targetStatics[key])) {\n        var descriptor = getOwnPropertyDescriptor(sourceComponent, key);\n\n        try {\n          // Avoid failures from read-only properties\n          defineProperty(targetComponent, key, descriptor);\n        } catch (e) {}\n      }\n    }\n  }\n\n  return targetComponent;\n}\n\nmodule.exports = hoistNonReactStatics;\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/hoist-non-react-statics/dist/hoist-non-react-statics.cjs.js?");

/***/ }),

/***/ "./node_modules/hoist-non-react-statics/node_modules/react-is/cjs/react-is.development.js":
/*!************************************************************************************************!*\
  !*** ./node_modules/hoist-non-react-statics/node_modules/react-is/cjs/react-is.development.js ***!
  \************************************************************************************************/
/***/ ((__unused_webpack_module, exports) => {

eval("/** @license React v16.13.1\n * react-is.development.js\n *\n * Copyright (c) Facebook, Inc. and its affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n\n\n\nif (true) {\n  (function() {\n'use strict';\n\n// The Symbol used to tag the ReactElement-like types. If there is no native Symbol\n// nor polyfill, then a plain number is used for performance.\nvar hasSymbol = typeof Symbol === 'function' && Symbol.for;\nvar REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;\nvar REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;\nvar REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;\nvar REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;\nvar REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;\nvar REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;\nvar REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace; // TODO: We don't use AsyncMode or ConcurrentMode anymore. They were temporary\n// (unstable) APIs that have been removed. Can we remove the symbols?\n\nvar REACT_ASYNC_MODE_TYPE = hasSymbol ? Symbol.for('react.async_mode') : 0xeacf;\nvar REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;\nvar REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;\nvar REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;\nvar REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for('react.suspense_list') : 0xead8;\nvar REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;\nvar REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;\nvar REACT_BLOCK_TYPE = hasSymbol ? Symbol.for('react.block') : 0xead9;\nvar REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for('react.fundamental') : 0xead5;\nvar REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for('react.responder') : 0xead6;\nvar REACT_SCOPE_TYPE = hasSymbol ? Symbol.for('react.scope') : 0xead7;\n\nfunction isValidElementType(type) {\n  return typeof type === 'string' || typeof type === 'function' || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.\n  type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);\n}\n\nfunction typeOf(object) {\n  if (typeof object === 'object' && object !== null) {\n    var $$typeof = object.$$typeof;\n\n    switch ($$typeof) {\n      case REACT_ELEMENT_TYPE:\n        var type = object.type;\n\n        switch (type) {\n          case REACT_ASYNC_MODE_TYPE:\n          case REACT_CONCURRENT_MODE_TYPE:\n          case REACT_FRAGMENT_TYPE:\n          case REACT_PROFILER_TYPE:\n          case REACT_STRICT_MODE_TYPE:\n          case REACT_SUSPENSE_TYPE:\n            return type;\n\n          default:\n            var $$typeofType = type && type.$$typeof;\n\n            switch ($$typeofType) {\n              case REACT_CONTEXT_TYPE:\n              case REACT_FORWARD_REF_TYPE:\n              case REACT_LAZY_TYPE:\n              case REACT_MEMO_TYPE:\n              case REACT_PROVIDER_TYPE:\n                return $$typeofType;\n\n              default:\n                return $$typeof;\n            }\n\n        }\n\n      case REACT_PORTAL_TYPE:\n        return $$typeof;\n    }\n  }\n\n  return undefined;\n} // AsyncMode is deprecated along with isAsyncMode\n\nvar AsyncMode = REACT_ASYNC_MODE_TYPE;\nvar ConcurrentMode = REACT_CONCURRENT_MODE_TYPE;\nvar ContextConsumer = REACT_CONTEXT_TYPE;\nvar ContextProvider = REACT_PROVIDER_TYPE;\nvar Element = REACT_ELEMENT_TYPE;\nvar ForwardRef = REACT_FORWARD_REF_TYPE;\nvar Fragment = REACT_FRAGMENT_TYPE;\nvar Lazy = REACT_LAZY_TYPE;\nvar Memo = REACT_MEMO_TYPE;\nvar Portal = REACT_PORTAL_TYPE;\nvar Profiler = REACT_PROFILER_TYPE;\nvar StrictMode = REACT_STRICT_MODE_TYPE;\nvar Suspense = REACT_SUSPENSE_TYPE;\nvar hasWarnedAboutDeprecatedIsAsyncMode = false; // AsyncMode should be deprecated\n\nfunction isAsyncMode(object) {\n  {\n    if (!hasWarnedAboutDeprecatedIsAsyncMode) {\n      hasWarnedAboutDeprecatedIsAsyncMode = true; // Using console['warn'] to evade Babel and ESLint\n\n      console['warn']('The ReactIs.isAsyncMode() alias has been deprecated, ' + 'and will be removed in React 17+. Update your code to use ' + 'ReactIs.isConcurrentMode() instead. It has the exact same API.');\n    }\n  }\n\n  return isConcurrentMode(object) || typeOf(object) === REACT_ASYNC_MODE_TYPE;\n}\nfunction isConcurrentMode(object) {\n  return typeOf(object) === REACT_CONCURRENT_MODE_TYPE;\n}\nfunction isContextConsumer(object) {\n  return typeOf(object) === REACT_CONTEXT_TYPE;\n}\nfunction isContextProvider(object) {\n  return typeOf(object) === REACT_PROVIDER_TYPE;\n}\nfunction isElement(object) {\n  return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;\n}\nfunction isForwardRef(object) {\n  return typeOf(object) === REACT_FORWARD_REF_TYPE;\n}\nfunction isFragment(object) {\n  return typeOf(object) === REACT_FRAGMENT_TYPE;\n}\nfunction isLazy(object) {\n  return typeOf(object) === REACT_LAZY_TYPE;\n}\nfunction isMemo(object) {\n  return typeOf(object) === REACT_MEMO_TYPE;\n}\nfunction isPortal(object) {\n  return typeOf(object) === REACT_PORTAL_TYPE;\n}\nfunction isProfiler(object) {\n  return typeOf(object) === REACT_PROFILER_TYPE;\n}\nfunction isStrictMode(object) {\n  return typeOf(object) === REACT_STRICT_MODE_TYPE;\n}\nfunction isSuspense(object) {\n  return typeOf(object) === REACT_SUSPENSE_TYPE;\n}\n\nexports.AsyncMode = AsyncMode;\nexports.ConcurrentMode = ConcurrentMode;\nexports.ContextConsumer = ContextConsumer;\nexports.ContextProvider = ContextProvider;\nexports.Element = Element;\nexports.ForwardRef = ForwardRef;\nexports.Fragment = Fragment;\nexports.Lazy = Lazy;\nexports.Memo = Memo;\nexports.Portal = Portal;\nexports.Profiler = Profiler;\nexports.StrictMode = StrictMode;\nexports.Suspense = Suspense;\nexports.isAsyncMode = isAsyncMode;\nexports.isConcurrentMode = isConcurrentMode;\nexports.isContextConsumer = isContextConsumer;\nexports.isContextProvider = isContextProvider;\nexports.isElement = isElement;\nexports.isForwardRef = isForwardRef;\nexports.isFragment = isFragment;\nexports.isLazy = isLazy;\nexports.isMemo = isMemo;\nexports.isPortal = isPortal;\nexports.isProfiler = isProfiler;\nexports.isStrictMode = isStrictMode;\nexports.isSuspense = isSuspense;\nexports.isValidElementType = isValidElementType;\nexports.typeOf = typeOf;\n  })();\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/hoist-non-react-statics/node_modules/react-is/cjs/react-is.development.js?");

/***/ }),

/***/ "./node_modules/hoist-non-react-statics/node_modules/react-is/index.js":
/*!*****************************************************************************!*\
  !*** ./node_modules/hoist-non-react-statics/node_modules/react-is/index.js ***!
  \*****************************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/react-is.development.js */ \"./node_modules/hoist-non-react-statics/node_modules/react-is/cjs/react-is.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/hoist-non-react-statics/node_modules/react-is/index.js?");

/***/ }),

/***/ "./node_modules/object-assign/index.js":
/*!*********************************************!*\
  !*** ./node_modules/object-assign/index.js ***!
  \*********************************************/
/***/ ((module) => {

eval("/*\nobject-assign\n(c) Sindre Sorhus\n@license MIT\n*/\n\n\n/* eslint-disable no-unused-vars */\nvar getOwnPropertySymbols = Object.getOwnPropertySymbols;\nvar hasOwnProperty = Object.prototype.hasOwnProperty;\nvar propIsEnumerable = Object.prototype.propertyIsEnumerable;\n\nfunction toObject(val) {\n\tif (val === null || val === undefined) {\n\t\tthrow new TypeError('Object.assign cannot be called with null or undefined');\n\t}\n\n\treturn Object(val);\n}\n\nfunction shouldUseNative() {\n\ttry {\n\t\tif (!Object.assign) {\n\t\t\treturn false;\n\t\t}\n\n\t\t// Detect buggy property enumeration order in older V8 versions.\n\n\t\t// https://bugs.chromium.org/p/v8/issues/detail?id=4118\n\t\tvar test1 = new String('abc');  // eslint-disable-line no-new-wrappers\n\t\ttest1[5] = 'de';\n\t\tif (Object.getOwnPropertyNames(test1)[0] === '5') {\n\t\t\treturn false;\n\t\t}\n\n\t\t// https://bugs.chromium.org/p/v8/issues/detail?id=3056\n\t\tvar test2 = {};\n\t\tfor (var i = 0; i < 10; i++) {\n\t\t\ttest2['_' + String.fromCharCode(i)] = i;\n\t\t}\n\t\tvar order2 = Object.getOwnPropertyNames(test2).map(function (n) {\n\t\t\treturn test2[n];\n\t\t});\n\t\tif (order2.join('') !== '0123456789') {\n\t\t\treturn false;\n\t\t}\n\n\t\t// https://bugs.chromium.org/p/v8/issues/detail?id=3056\n\t\tvar test3 = {};\n\t\t'abcdefghijklmnopqrst'.split('').forEach(function (letter) {\n\t\t\ttest3[letter] = letter;\n\t\t});\n\t\tif (Object.keys(Object.assign({}, test3)).join('') !==\n\t\t\t\t'abcdefghijklmnopqrst') {\n\t\t\treturn false;\n\t\t}\n\n\t\treturn true;\n\t} catch (err) {\n\t\t// We don't expect any of the above to throw, but better to be safe.\n\t\treturn false;\n\t}\n}\n\nmodule.exports = shouldUseNative() ? Object.assign : function (target, source) {\n\tvar from;\n\tvar to = toObject(target);\n\tvar symbols;\n\n\tfor (var s = 1; s < arguments.length; s++) {\n\t\tfrom = Object(arguments[s]);\n\n\t\tfor (var key in from) {\n\t\t\tif (hasOwnProperty.call(from, key)) {\n\t\t\t\tto[key] = from[key];\n\t\t\t}\n\t\t}\n\n\t\tif (getOwnPropertySymbols) {\n\t\t\tsymbols = getOwnPropertySymbols(from);\n\t\t\tfor (var i = 0; i < symbols.length; i++) {\n\t\t\t\tif (propIsEnumerable.call(from, symbols[i])) {\n\t\t\t\t\tto[symbols[i]] = from[symbols[i]];\n\t\t\t\t}\n\t\t\t}\n\t\t}\n\t}\n\n\treturn to;\n};\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/object-assign/index.js?");

/***/ }),

/***/ "./node_modules/prop-types/checkPropTypes.js":
/*!***************************************************!*\
  !*** ./node_modules/prop-types/checkPropTypes.js ***!
  \***************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("/**\n * Copyright (c) 2013-present, Facebook, Inc.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n\nvar printWarning = function() {};\n\nif (true) {\n  var ReactPropTypesSecret = __webpack_require__(/*! ./lib/ReactPropTypesSecret */ \"./node_modules/prop-types/lib/ReactPropTypesSecret.js\");\n  var loggedTypeFailures = {};\n  var has = __webpack_require__(/*! ./lib/has */ \"./node_modules/prop-types/lib/has.js\");\n\n  printWarning = function(text) {\n    var message = 'Warning: ' + text;\n    if (typeof console !== 'undefined') {\n      console.error(message);\n    }\n    try {\n      // --- Welcome to debugging React ---\n      // This error was thrown as a convenience so that you can use this stack\n      // to find the callsite that caused this warning to fire.\n      throw new Error(message);\n    } catch (x) { /**/ }\n  };\n}\n\n/**\n * Assert that the values match with the type specs.\n * Error messages are memorized and will only be shown once.\n *\n * @param {object} typeSpecs Map of name to a ReactPropType\n * @param {object} values Runtime values that need to be type-checked\n * @param {string} location e.g. \"prop\", \"context\", \"child context\"\n * @param {string} componentName Name of the component for error messages.\n * @param {?Function} getStack Returns the component stack.\n * @private\n */\nfunction checkPropTypes(typeSpecs, values, location, componentName, getStack) {\n  if (true) {\n    for (var typeSpecName in typeSpecs) {\n      if (has(typeSpecs, typeSpecName)) {\n        var error;\n        // Prop type validation may throw. In case they do, we don't want to\n        // fail the render phase where it didn't fail before. So we log it.\n        // After these have been cleaned up, we'll let them throw.\n        try {\n          // This is intentionally an invariant that gets caught. It's the same\n          // behavior as without this statement except with a better message.\n          if (typeof typeSpecs[typeSpecName] !== 'function') {\n            var err = Error(\n              (componentName || 'React class') + ': ' + location + ' type `' + typeSpecName + '` is invalid; ' +\n              'it must be a function, usually from the `prop-types` package, but received `' + typeof typeSpecs[typeSpecName] + '`.' +\n              'This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`.'\n            );\n            err.name = 'Invariant Violation';\n            throw err;\n          }\n          error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret);\n        } catch (ex) {\n          error = ex;\n        }\n        if (error && !(error instanceof Error)) {\n          printWarning(\n            (componentName || 'React class') + ': type specification of ' +\n            location + ' `' + typeSpecName + '` is invalid; the type checker ' +\n            'function must return `null` or an `Error` but returned a ' + typeof error + '. ' +\n            'You may have forgotten to pass an argument to the type checker ' +\n            'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' +\n            'shape all require an argument).'\n          );\n        }\n        if (error instanceof Error && !(error.message in loggedTypeFailures)) {\n          // Only monitor this failure once because there tends to be a lot of the\n          // same error.\n          loggedTypeFailures[error.message] = true;\n\n          var stack = getStack ? getStack() : '';\n\n          printWarning(\n            'Failed ' + location + ' type: ' + error.message + (stack != null ? stack : '')\n          );\n        }\n      }\n    }\n  }\n}\n\n/**\n * Resets warning cache when testing.\n *\n * @private\n */\ncheckPropTypes.resetWarningCache = function() {\n  if (true) {\n    loggedTypeFailures = {};\n  }\n}\n\nmodule.exports = checkPropTypes;\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/checkPropTypes.js?");

/***/ }),

/***/ "./node_modules/prop-types/factoryWithTypeCheckers.js":
/*!************************************************************!*\
  !*** ./node_modules/prop-types/factoryWithTypeCheckers.js ***!
  \************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("/**\n * Copyright (c) 2013-present, Facebook, Inc.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n\nvar ReactIs = __webpack_require__(/*! react-is */ \"./node_modules/prop-types/node_modules/react-is/index.js\");\nvar assign = __webpack_require__(/*! object-assign */ \"./node_modules/object-assign/index.js\");\n\nvar ReactPropTypesSecret = __webpack_require__(/*! ./lib/ReactPropTypesSecret */ \"./node_modules/prop-types/lib/ReactPropTypesSecret.js\");\nvar has = __webpack_require__(/*! ./lib/has */ \"./node_modules/prop-types/lib/has.js\");\nvar checkPropTypes = __webpack_require__(/*! ./checkPropTypes */ \"./node_modules/prop-types/checkPropTypes.js\");\n\nvar printWarning = function() {};\n\nif (true) {\n  printWarning = function(text) {\n    var message = 'Warning: ' + text;\n    if (typeof console !== 'undefined') {\n      console.error(message);\n    }\n    try {\n      // --- Welcome to debugging React ---\n      // This error was thrown as a convenience so that you can use this stack\n      // to find the callsite that caused this warning to fire.\n      throw new Error(message);\n    } catch (x) {}\n  };\n}\n\nfunction emptyFunctionThatReturnsNull() {\n  return null;\n}\n\nmodule.exports = function(isValidElement, throwOnDirectAccess) {\n  /* global Symbol */\n  var ITERATOR_SYMBOL = typeof Symbol === 'function' && Symbol.iterator;\n  var FAUX_ITERATOR_SYMBOL = '@@iterator'; // Before Symbol spec.\n\n  /**\n   * Returns the iterator method function contained on the iterable object.\n   *\n   * Be sure to invoke the function with the iterable as context:\n   *\n   *     var iteratorFn = getIteratorFn(myIterable);\n   *     if (iteratorFn) {\n   *       var iterator = iteratorFn.call(myIterable);\n   *       ...\n   *     }\n   *\n   * @param {?object} maybeIterable\n   * @return {?function}\n   */\n  function getIteratorFn(maybeIterable) {\n    var iteratorFn = maybeIterable && (ITERATOR_SYMBOL && maybeIterable[ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL]);\n    if (typeof iteratorFn === 'function') {\n      return iteratorFn;\n    }\n  }\n\n  /**\n   * Collection of methods that allow declaration and validation of props that are\n   * supplied to React components. Example usage:\n   *\n   *   var Props = require('ReactPropTypes');\n   *   var MyArticle = React.createClass({\n   *     propTypes: {\n   *       // An optional string prop named \"description\".\n   *       description: Props.string,\n   *\n   *       // A required enum prop named \"category\".\n   *       category: Props.oneOf(['News','Photos']).isRequired,\n   *\n   *       // A prop named \"dialog\" that requires an instance of Dialog.\n   *       dialog: Props.instanceOf(Dialog).isRequired\n   *     },\n   *     render: function() { ... }\n   *   });\n   *\n   * A more formal specification of how these methods are used:\n   *\n   *   type := array|bool|func|object|number|string|oneOf([...])|instanceOf(...)\n   *   decl := ReactPropTypes.{type}(.isRequired)?\n   *\n   * Each and every declaration produces a function with the same signature. This\n   * allows the creation of custom validation functions. For example:\n   *\n   *  var MyLink = React.createClass({\n   *    propTypes: {\n   *      // An optional string or URI prop named \"href\".\n   *      href: function(props, propName, componentName) {\n   *        var propValue = props[propName];\n   *        if (propValue != null && typeof propValue !== 'string' &&\n   *            !(propValue instanceof URI)) {\n   *          return new Error(\n   *            'Expected a string or an URI for ' + propName + ' in ' +\n   *            componentName\n   *          );\n   *        }\n   *      }\n   *    },\n   *    render: function() {...}\n   *  });\n   *\n   * @internal\n   */\n\n  var ANONYMOUS = '<<anonymous>>';\n\n  // Important!\n  // Keep this list in sync with production version in `./factoryWithThrowingShims.js`.\n  var ReactPropTypes = {\n    array: createPrimitiveTypeChecker('array'),\n    bigint: createPrimitiveTypeChecker('bigint'),\n    bool: createPrimitiveTypeChecker('boolean'),\n    func: createPrimitiveTypeChecker('function'),\n    number: createPrimitiveTypeChecker('number'),\n    object: createPrimitiveTypeChecker('object'),\n    string: createPrimitiveTypeChecker('string'),\n    symbol: createPrimitiveTypeChecker('symbol'),\n\n    any: createAnyTypeChecker(),\n    arrayOf: createArrayOfTypeChecker,\n    element: createElementTypeChecker(),\n    elementType: createElementTypeTypeChecker(),\n    instanceOf: createInstanceTypeChecker,\n    node: createNodeChecker(),\n    objectOf: createObjectOfTypeChecker,\n    oneOf: createEnumTypeChecker,\n    oneOfType: createUnionTypeChecker,\n    shape: createShapeTypeChecker,\n    exact: createStrictShapeTypeChecker,\n  };\n\n  /**\n   * inlined Object.is polyfill to avoid requiring consumers ship their own\n   * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is\n   */\n  /*eslint-disable no-self-compare*/\n  function is(x, y) {\n    // SameValue algorithm\n    if (x === y) {\n      // Steps 1-5, 7-10\n      // Steps 6.b-6.e: +0 != -0\n      return x !== 0 || 1 / x === 1 / y;\n    } else {\n      // Step 6.a: NaN == NaN\n      return x !== x && y !== y;\n    }\n  }\n  /*eslint-enable no-self-compare*/\n\n  /**\n   * We use an Error-like object for backward compatibility as people may call\n   * PropTypes directly and inspect their output. However, we don't use real\n   * Errors anymore. We don't inspect their stack anyway, and creating them\n   * is prohibitively expensive if they are created too often, such as what\n   * happens in oneOfType() for any type before the one that matched.\n   */\n  function PropTypeError(message, data) {\n    this.message = message;\n    this.data = data && typeof data === 'object' ? data: {};\n    this.stack = '';\n  }\n  // Make `instanceof Error` still work for returned errors.\n  PropTypeError.prototype = Error.prototype;\n\n  function createChainableTypeChecker(validate) {\n    if (true) {\n      var manualPropTypeCallCache = {};\n      var manualPropTypeWarningCount = 0;\n    }\n    function checkType(isRequired, props, propName, componentName, location, propFullName, secret) {\n      componentName = componentName || ANONYMOUS;\n      propFullName = propFullName || propName;\n\n      if (secret !== ReactPropTypesSecret) {\n        if (throwOnDirectAccess) {\n          // New behavior only for users of `prop-types` package\n          var err = new Error(\n            'Calling PropTypes validators directly is not supported by the `prop-types` package. ' +\n            'Use `PropTypes.checkPropTypes()` to call them. ' +\n            'Read more at http://fb.me/use-check-prop-types'\n          );\n          err.name = 'Invariant Violation';\n          throw err;\n        } else if ( true && typeof console !== 'undefined') {\n          // Old behavior for people using React.PropTypes\n          var cacheKey = componentName + ':' + propName;\n          if (\n            !manualPropTypeCallCache[cacheKey] &&\n            // Avoid spamming the console because they are often not actionable except for lib authors\n            manualPropTypeWarningCount < 3\n          ) {\n            printWarning(\n              'You are manually calling a React.PropTypes validation ' +\n              'function for the `' + propFullName + '` prop on `' + componentName + '`. This is deprecated ' +\n              'and will throw in the standalone `prop-types` package. ' +\n              'You may be seeing this warning due to a third-party PropTypes ' +\n              'library. See https://fb.me/react-warning-dont-call-proptypes ' + 'for details.'\n            );\n            manualPropTypeCallCache[cacheKey] = true;\n            manualPropTypeWarningCount++;\n          }\n        }\n      }\n      if (props[propName] == null) {\n        if (isRequired) {\n          if (props[propName] === null) {\n            return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required ' + ('in `' + componentName + '`, but its value is `null`.'));\n          }\n          return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required in ' + ('`' + componentName + '`, but its value is `undefined`.'));\n        }\n        return null;\n      } else {\n        return validate(props, propName, componentName, location, propFullName);\n      }\n    }\n\n    var chainedCheckType = checkType.bind(null, false);\n    chainedCheckType.isRequired = checkType.bind(null, true);\n\n    return chainedCheckType;\n  }\n\n  function createPrimitiveTypeChecker(expectedType) {\n    function validate(props, propName, componentName, location, propFullName, secret) {\n      var propValue = props[propName];\n      var propType = getPropType(propValue);\n      if (propType !== expectedType) {\n        // `propValue` being instance of, say, date/regexp, pass the 'object'\n        // check, but we can offer a more precise error message here rather than\n        // 'of type `object`'.\n        var preciseType = getPreciseType(propValue);\n\n        return new PropTypeError(\n          'Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + preciseType + '` supplied to `' + componentName + '`, expected ') + ('`' + expectedType + '`.'),\n          {expectedType: expectedType}\n        );\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createAnyTypeChecker() {\n    return createChainableTypeChecker(emptyFunctionThatReturnsNull);\n  }\n\n  function createArrayOfTypeChecker(typeChecker) {\n    function validate(props, propName, componentName, location, propFullName) {\n      if (typeof typeChecker !== 'function') {\n        return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside arrayOf.');\n      }\n      var propValue = props[propName];\n      if (!Array.isArray(propValue)) {\n        var propType = getPropType(propValue);\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an array.'));\n      }\n      for (var i = 0; i < propValue.length; i++) {\n        var error = typeChecker(propValue, i, componentName, location, propFullName + '[' + i + ']', ReactPropTypesSecret);\n        if (error instanceof Error) {\n          return error;\n        }\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createElementTypeChecker() {\n    function validate(props, propName, componentName, location, propFullName) {\n      var propValue = props[propName];\n      if (!isValidElement(propValue)) {\n        var propType = getPropType(propValue);\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement.'));\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createElementTypeTypeChecker() {\n    function validate(props, propName, componentName, location, propFullName) {\n      var propValue = props[propName];\n      if (!ReactIs.isValidElementType(propValue)) {\n        var propType = getPropType(propValue);\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement type.'));\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createInstanceTypeChecker(expectedClass) {\n    function validate(props, propName, componentName, location, propFullName) {\n      if (!(props[propName] instanceof expectedClass)) {\n        var expectedClassName = expectedClass.name || ANONYMOUS;\n        var actualClassName = getClassName(props[propName]);\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + actualClassName + '` supplied to `' + componentName + '`, expected ') + ('instance of `' + expectedClassName + '`.'));\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createEnumTypeChecker(expectedValues) {\n    if (!Array.isArray(expectedValues)) {\n      if (true) {\n        if (arguments.length > 1) {\n          printWarning(\n            'Invalid arguments supplied to oneOf, expected an array, got ' + arguments.length + ' arguments. ' +\n            'A common mistake is to write oneOf(x, y, z) instead of oneOf([x, y, z]).'\n          );\n        } else {\n          printWarning('Invalid argument supplied to oneOf, expected an array.');\n        }\n      }\n      return emptyFunctionThatReturnsNull;\n    }\n\n    function validate(props, propName, componentName, location, propFullName) {\n      var propValue = props[propName];\n      for (var i = 0; i < expectedValues.length; i++) {\n        if (is(propValue, expectedValues[i])) {\n          return null;\n        }\n      }\n\n      var valuesString = JSON.stringify(expectedValues, function replacer(key, value) {\n        var type = getPreciseType(value);\n        if (type === 'symbol') {\n          return String(value);\n        }\n        return value;\n      });\n      return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of value `' + String(propValue) + '` ' + ('supplied to `' + componentName + '`, expected one of ' + valuesString + '.'));\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createObjectOfTypeChecker(typeChecker) {\n    function validate(props, propName, componentName, location, propFullName) {\n      if (typeof typeChecker !== 'function') {\n        return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside objectOf.');\n      }\n      var propValue = props[propName];\n      var propType = getPropType(propValue);\n      if (propType !== 'object') {\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an object.'));\n      }\n      for (var key in propValue) {\n        if (has(propValue, key)) {\n          var error = typeChecker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret);\n          if (error instanceof Error) {\n            return error;\n          }\n        }\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createUnionTypeChecker(arrayOfTypeCheckers) {\n    if (!Array.isArray(arrayOfTypeCheckers)) {\n       true ? printWarning('Invalid argument supplied to oneOfType, expected an instance of array.') : 0;\n      return emptyFunctionThatReturnsNull;\n    }\n\n    for (var i = 0; i < arrayOfTypeCheckers.length; i++) {\n      var checker = arrayOfTypeCheckers[i];\n      if (typeof checker !== 'function') {\n        printWarning(\n          'Invalid argument supplied to oneOfType. Expected an array of check functions, but ' +\n          'received ' + getPostfixForTypeWarning(checker) + ' at index ' + i + '.'\n        );\n        return emptyFunctionThatReturnsNull;\n      }\n    }\n\n    function validate(props, propName, componentName, location, propFullName) {\n      var expectedTypes = [];\n      for (var i = 0; i < arrayOfTypeCheckers.length; i++) {\n        var checker = arrayOfTypeCheckers[i];\n        var checkerResult = checker(props, propName, componentName, location, propFullName, ReactPropTypesSecret);\n        if (checkerResult == null) {\n          return null;\n        }\n        if (checkerResult.data && has(checkerResult.data, 'expectedType')) {\n          expectedTypes.push(checkerResult.data.expectedType);\n        }\n      }\n      var expectedTypesMessage = (expectedTypes.length > 0) ? ', expected one of type [' + expectedTypes.join(', ') + ']': '';\n      return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`' + expectedTypesMessage + '.'));\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createNodeChecker() {\n    function validate(props, propName, componentName, location, propFullName) {\n      if (!isNode(props[propName])) {\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`, expected a ReactNode.'));\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function invalidValidatorError(componentName, location, propFullName, key, type) {\n    return new PropTypeError(\n      (componentName || 'React class') + ': ' + location + ' type `' + propFullName + '.' + key + '` is invalid; ' +\n      'it must be a function, usually from the `prop-types` package, but received `' + type + '`.'\n    );\n  }\n\n  function createShapeTypeChecker(shapeTypes) {\n    function validate(props, propName, componentName, location, propFullName) {\n      var propValue = props[propName];\n      var propType = getPropType(propValue);\n      if (propType !== 'object') {\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));\n      }\n      for (var key in shapeTypes) {\n        var checker = shapeTypes[key];\n        if (typeof checker !== 'function') {\n          return invalidValidatorError(componentName, location, propFullName, key, getPreciseType(checker));\n        }\n        var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret);\n        if (error) {\n          return error;\n        }\n      }\n      return null;\n    }\n    return createChainableTypeChecker(validate);\n  }\n\n  function createStrictShapeTypeChecker(shapeTypes) {\n    function validate(props, propName, componentName, location, propFullName) {\n      var propValue = props[propName];\n      var propType = getPropType(propValue);\n      if (propType !== 'object') {\n        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));\n      }\n      // We need to check all keys in case some are required but missing from props.\n      var allKeys = assign({}, props[propName], shapeTypes);\n      for (var key in allKeys) {\n        var checker = shapeTypes[key];\n        if (has(shapeTypes, key) && typeof checker !== 'function') {\n          return invalidValidatorError(componentName, location, propFullName, key, getPreciseType(checker));\n        }\n        if (!checker) {\n          return new PropTypeError(\n            'Invalid ' + location + ' `' + propFullName + '` key `' + key + '` supplied to `' + componentName + '`.' +\n            '\\nBad object: ' + JSON.stringify(props[propName], null, '  ') +\n            '\\nValid keys: ' + JSON.stringify(Object.keys(shapeTypes), null, '  ')\n          );\n        }\n        var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret);\n        if (error) {\n          return error;\n        }\n      }\n      return null;\n    }\n\n    return createChainableTypeChecker(validate);\n  }\n\n  function isNode(propValue) {\n    switch (typeof propValue) {\n      case 'number':\n      case 'string':\n      case 'undefined':\n        return true;\n      case 'boolean':\n        return !propValue;\n      case 'object':\n        if (Array.isArray(propValue)) {\n          return propValue.every(isNode);\n        }\n        if (propValue === null || isValidElement(propValue)) {\n          return true;\n        }\n\n        var iteratorFn = getIteratorFn(propValue);\n        if (iteratorFn) {\n          var iterator = iteratorFn.call(propValue);\n          var step;\n          if (iteratorFn !== propValue.entries) {\n            while (!(step = iterator.next()).done) {\n              if (!isNode(step.value)) {\n                return false;\n              }\n            }\n          } else {\n            // Iterator will provide entry [k,v] tuples rather than values.\n            while (!(step = iterator.next()).done) {\n              var entry = step.value;\n              if (entry) {\n                if (!isNode(entry[1])) {\n                  return false;\n                }\n              }\n            }\n          }\n        } else {\n          return false;\n        }\n\n        return true;\n      default:\n        return false;\n    }\n  }\n\n  function isSymbol(propType, propValue) {\n    // Native Symbol.\n    if (propType === 'symbol') {\n      return true;\n    }\n\n    // falsy value can't be a Symbol\n    if (!propValue) {\n      return false;\n    }\n\n    // 19.4.3.5 Symbol.prototype[@@toStringTag] === 'Symbol'\n    if (propValue['@@toStringTag'] === 'Symbol') {\n      return true;\n    }\n\n    // Fallback for non-spec compliant Symbols which are polyfilled.\n    if (typeof Symbol === 'function' && propValue instanceof Symbol) {\n      return true;\n    }\n\n    return false;\n  }\n\n  // Equivalent of `typeof` but with special handling for array and regexp.\n  function getPropType(propValue) {\n    var propType = typeof propValue;\n    if (Array.isArray(propValue)) {\n      return 'array';\n    }\n    if (propValue instanceof RegExp) {\n      // Old webkits (at least until Android 4.0) return 'function' rather than\n      // 'object' for typeof a RegExp. We'll normalize this here so that /bla/\n      // passes PropTypes.object.\n      return 'object';\n    }\n    if (isSymbol(propType, propValue)) {\n      return 'symbol';\n    }\n    return propType;\n  }\n\n  // This handles more types than `getPropType`. Only used for error messages.\n  // See `createPrimitiveTypeChecker`.\n  function getPreciseType(propValue) {\n    if (typeof propValue === 'undefined' || propValue === null) {\n      return '' + propValue;\n    }\n    var propType = getPropType(propValue);\n    if (propType === 'object') {\n      if (propValue instanceof Date) {\n        return 'date';\n      } else if (propValue instanceof RegExp) {\n        return 'regexp';\n      }\n    }\n    return propType;\n  }\n\n  // Returns a string that is postfixed to a warning about an invalid type.\n  // For example, \"undefined\" or \"of type array\"\n  function getPostfixForTypeWarning(value) {\n    var type = getPreciseType(value);\n    switch (type) {\n      case 'array':\n      case 'object':\n        return 'an ' + type;\n      case 'boolean':\n      case 'date':\n      case 'regexp':\n        return 'a ' + type;\n      default:\n        return type;\n    }\n  }\n\n  // Returns class name of the object, if any.\n  function getClassName(propValue) {\n    if (!propValue.constructor || !propValue.constructor.name) {\n      return ANONYMOUS;\n    }\n    return propValue.constructor.name;\n  }\n\n  ReactPropTypes.checkPropTypes = checkPropTypes;\n  ReactPropTypes.resetWarningCache = checkPropTypes.resetWarningCache;\n  ReactPropTypes.PropTypes = ReactPropTypes;\n\n  return ReactPropTypes;\n};\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/factoryWithTypeCheckers.js?");

/***/ }),

/***/ "./node_modules/prop-types/index.js":
/*!******************************************!*\
  !*** ./node_modules/prop-types/index.js ***!
  \******************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("/**\n * Copyright (c) 2013-present, Facebook, Inc.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\nif (true) {\n  var ReactIs = __webpack_require__(/*! react-is */ \"./node_modules/prop-types/node_modules/react-is/index.js\");\n\n  // By explicitly using `prop-types` you are opting into new development behavior.\n  // http://fb.me/prop-types-in-prod\n  var throwOnDirectAccess = true;\n  module.exports = __webpack_require__(/*! ./factoryWithTypeCheckers */ \"./node_modules/prop-types/factoryWithTypeCheckers.js\")(ReactIs.isElement, throwOnDirectAccess);\n} else // removed by dead control flow\n{}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/index.js?");

/***/ }),

/***/ "./node_modules/prop-types/lib/ReactPropTypesSecret.js":
/*!*************************************************************!*\
  !*** ./node_modules/prop-types/lib/ReactPropTypesSecret.js ***!
  \*************************************************************/
/***/ ((module) => {

eval("/**\n * Copyright (c) 2013-present, Facebook, Inc.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n\nvar ReactPropTypesSecret = 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED';\n\nmodule.exports = ReactPropTypesSecret;\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/lib/ReactPropTypesSecret.js?");

/***/ }),

/***/ "./node_modules/prop-types/lib/has.js":
/*!********************************************!*\
  !*** ./node_modules/prop-types/lib/has.js ***!
  \********************************************/
/***/ ((module) => {

eval("module.exports = Function.call.bind(Object.prototype.hasOwnProperty);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/lib/has.js?");

/***/ }),

/***/ "./node_modules/prop-types/node_modules/react-is/cjs/react-is.development.js":
/*!***********************************************************************************!*\
  !*** ./node_modules/prop-types/node_modules/react-is/cjs/react-is.development.js ***!
  \***********************************************************************************/
/***/ ((__unused_webpack_module, exports) => {

eval("/** @license React v16.13.1\n * react-is.development.js\n *\n * Copyright (c) Facebook, Inc. and its affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n\n\n\nif (true) {\n  (function() {\n'use strict';\n\n// The Symbol used to tag the ReactElement-like types. If there is no native Symbol\n// nor polyfill, then a plain number is used for performance.\nvar hasSymbol = typeof Symbol === 'function' && Symbol.for;\nvar REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;\nvar REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;\nvar REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;\nvar REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;\nvar REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;\nvar REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;\nvar REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace; // TODO: We don't use AsyncMode or ConcurrentMode anymore. They were temporary\n// (unstable) APIs that have been removed. Can we remove the symbols?\n\nvar REACT_ASYNC_MODE_TYPE = hasSymbol ? Symbol.for('react.async_mode') : 0xeacf;\nvar REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;\nvar REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;\nvar REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;\nvar REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for('react.suspense_list') : 0xead8;\nvar REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;\nvar REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;\nvar REACT_BLOCK_TYPE = hasSymbol ? Symbol.for('react.block') : 0xead9;\nvar REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for('react.fundamental') : 0xead5;\nvar REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for('react.responder') : 0xead6;\nvar REACT_SCOPE_TYPE = hasSymbol ? Symbol.for('react.scope') : 0xead7;\n\nfunction isValidElementType(type) {\n  return typeof type === 'string' || typeof type === 'function' || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.\n  type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);\n}\n\nfunction typeOf(object) {\n  if (typeof object === 'object' && object !== null) {\n    var $$typeof = object.$$typeof;\n\n    switch ($$typeof) {\n      case REACT_ELEMENT_TYPE:\n        var type = object.type;\n\n        switch (type) {\n          case REACT_ASYNC_MODE_TYPE:\n          case REACT_CONCURRENT_MODE_TYPE:\n          case REACT_FRAGMENT_TYPE:\n          case REACT_PROFILER_TYPE:\n          case REACT_STRICT_MODE_TYPE:\n          case REACT_SUSPENSE_TYPE:\n            return type;\n\n          default:\n            var $$typeofType = type && type.$$typeof;\n\n            switch ($$typeofType) {\n              case REACT_CONTEXT_TYPE:\n              case REACT_FORWARD_REF_TYPE:\n              case REACT_LAZY_TYPE:\n              case REACT_MEMO_TYPE:\n              case REACT_PROVIDER_TYPE:\n                return $$typeofType;\n\n              default:\n                return $$typeof;\n            }\n\n        }\n\n      case REACT_PORTAL_TYPE:\n        return $$typeof;\n    }\n  }\n\n  return undefined;\n} // AsyncMode is deprecated along with isAsyncMode\n\nvar AsyncMode = REACT_ASYNC_MODE_TYPE;\nvar ConcurrentMode = REACT_CONCURRENT_MODE_TYPE;\nvar ContextConsumer = REACT_CONTEXT_TYPE;\nvar ContextProvider = REACT_PROVIDER_TYPE;\nvar Element = REACT_ELEMENT_TYPE;\nvar ForwardRef = REACT_FORWARD_REF_TYPE;\nvar Fragment = REACT_FRAGMENT_TYPE;\nvar Lazy = REACT_LAZY_TYPE;\nvar Memo = REACT_MEMO_TYPE;\nvar Portal = REACT_PORTAL_TYPE;\nvar Profiler = REACT_PROFILER_TYPE;\nvar StrictMode = REACT_STRICT_MODE_TYPE;\nvar Suspense = REACT_SUSPENSE_TYPE;\nvar hasWarnedAboutDeprecatedIsAsyncMode = false; // AsyncMode should be deprecated\n\nfunction isAsyncMode(object) {\n  {\n    if (!hasWarnedAboutDeprecatedIsAsyncMode) {\n      hasWarnedAboutDeprecatedIsAsyncMode = true; // Using console['warn'] to evade Babel and ESLint\n\n      console['warn']('The ReactIs.isAsyncMode() alias has been deprecated, ' + 'and will be removed in React 17+. Update your code to use ' + 'ReactIs.isConcurrentMode() instead. It has the exact same API.');\n    }\n  }\n\n  return isConcurrentMode(object) || typeOf(object) === REACT_ASYNC_MODE_TYPE;\n}\nfunction isConcurrentMode(object) {\n  return typeOf(object) === REACT_CONCURRENT_MODE_TYPE;\n}\nfunction isContextConsumer(object) {\n  return typeOf(object) === REACT_CONTEXT_TYPE;\n}\nfunction isContextProvider(object) {\n  return typeOf(object) === REACT_PROVIDER_TYPE;\n}\nfunction isElement(object) {\n  return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;\n}\nfunction isForwardRef(object) {\n  return typeOf(object) === REACT_FORWARD_REF_TYPE;\n}\nfunction isFragment(object) {\n  return typeOf(object) === REACT_FRAGMENT_TYPE;\n}\nfunction isLazy(object) {\n  return typeOf(object) === REACT_LAZY_TYPE;\n}\nfunction isMemo(object) {\n  return typeOf(object) === REACT_MEMO_TYPE;\n}\nfunction isPortal(object) {\n  return typeOf(object) === REACT_PORTAL_TYPE;\n}\nfunction isProfiler(object) {\n  return typeOf(object) === REACT_PROFILER_TYPE;\n}\nfunction isStrictMode(object) {\n  return typeOf(object) === REACT_STRICT_MODE_TYPE;\n}\nfunction isSuspense(object) {\n  return typeOf(object) === REACT_SUSPENSE_TYPE;\n}\n\nexports.AsyncMode = AsyncMode;\nexports.ConcurrentMode = ConcurrentMode;\nexports.ContextConsumer = ContextConsumer;\nexports.ContextProvider = ContextProvider;\nexports.Element = Element;\nexports.ForwardRef = ForwardRef;\nexports.Fragment = Fragment;\nexports.Lazy = Lazy;\nexports.Memo = Memo;\nexports.Portal = Portal;\nexports.Profiler = Profiler;\nexports.StrictMode = StrictMode;\nexports.Suspense = Suspense;\nexports.isAsyncMode = isAsyncMode;\nexports.isConcurrentMode = isConcurrentMode;\nexports.isContextConsumer = isContextConsumer;\nexports.isContextProvider = isContextProvider;\nexports.isElement = isElement;\nexports.isForwardRef = isForwardRef;\nexports.isFragment = isFragment;\nexports.isLazy = isLazy;\nexports.isMemo = isMemo;\nexports.isPortal = isPortal;\nexports.isProfiler = isProfiler;\nexports.isStrictMode = isStrictMode;\nexports.isSuspense = isSuspense;\nexports.isValidElementType = isValidElementType;\nexports.typeOf = typeOf;\n  })();\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/node_modules/react-is/cjs/react-is.development.js?");

/***/ }),

/***/ "./node_modules/prop-types/node_modules/react-is/index.js":
/*!****************************************************************!*\
  !*** ./node_modules/prop-types/node_modules/react-is/index.js ***!
  \****************************************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/react-is.development.js */ \"./node_modules/prop-types/node_modules/react-is/cjs/react-is.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/prop-types/node_modules/react-is/index.js?");

/***/ }),

/***/ "./node_modules/react-dom/cjs/react-dom-client.development.js":
/*!********************************************************************!*\
  !*** ./node_modules/react-dom/cjs/react-dom-client.development.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("/**\n * @license React\n * react-dom-client.development.js\n *\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n/*\n Modernizr 3.0.0pre (Custom Build) | MIT\n*/\n\n true &&\n  (function () {\n    function findHook(fiber, id) {\n      for (fiber = fiber.memoizedState; null !== fiber && 0 < id; )\n        (fiber = fiber.next), id--;\n      return fiber;\n    }\n    function copyWithSetImpl(obj, path, index, value) {\n      if (index >= path.length) return value;\n      var key = path[index],\n        updated = isArrayImpl(obj) ? obj.slice() : assign({}, obj);\n      updated[key] = copyWithSetImpl(obj[key], path, index + 1, value);\n      return updated;\n    }\n    function copyWithRename(obj, oldPath, newPath) {\n      if (oldPath.length !== newPath.length)\n        console.warn(\"copyWithRename() expects paths of the same length\");\n      else {\n        for (var i = 0; i < newPath.length - 1; i++)\n          if (oldPath[i] !== newPath[i]) {\n            console.warn(\n              \"copyWithRename() expects paths to be the same except for the deepest key\"\n            );\n            return;\n          }\n        return copyWithRenameImpl(obj, oldPath, newPath, 0);\n      }\n    }\n    function copyWithRenameImpl(obj, oldPath, newPath, index) {\n      var oldKey = oldPath[index],\n        updated = isArrayImpl(obj) ? obj.slice() : assign({}, obj);\n      index + 1 === oldPath.length\n        ? ((updated[newPath[index]] = updated[oldKey]),\n          isArrayImpl(updated)\n            ? updated.splice(oldKey, 1)\n            : delete updated[oldKey])\n        : (updated[oldKey] = copyWithRenameImpl(\n            obj[oldKey],\n            oldPath,\n            newPath,\n            index + 1\n          ));\n      return updated;\n    }\n    function copyWithDeleteImpl(obj, path, index) {\n      var key = path[index],\n        updated = isArrayImpl(obj) ? obj.slice() : assign({}, obj);\n      if (index + 1 === path.length)\n        return (\n          isArrayImpl(updated) ? updated.splice(key, 1) : delete updated[key],\n          updated\n        );\n      updated[key] = copyWithDeleteImpl(obj[key], path, index + 1);\n      return updated;\n    }\n    function shouldSuspendImpl() {\n      return !1;\n    }\n    function shouldErrorImpl() {\n      return null;\n    }\n    function warnForMissingKey() {}\n    function warnInvalidHookAccess() {\n      console.error(\n        \"Do not call Hooks inside useEffect(...), useMemo(...), or other built-in Hooks. You can only call Hooks at the top level of your React function. For more information, see https://react.dev/link/rules-of-hooks\"\n      );\n    }\n    function warnInvalidContextAccess() {\n      console.error(\n        \"Context can only be read while React is rendering. In classes, you can read it in the render method or getDerivedStateFromProps. In function components, you can read it directly in the function body, but not inside Hooks like useReducer() or useMemo().\"\n      );\n    }\n    function noop$2() {}\n    function setToSortedString(set) {\n      var array = [];\n      set.forEach(function (value) {\n        array.push(value);\n      });\n      return array.sort().join(\", \");\n    }\n    function createFiber(tag, pendingProps, key, mode) {\n      return new FiberNode(tag, pendingProps, key, mode);\n    }\n    function scheduleRoot(root, element) {\n      root.context === emptyContextObject &&\n        (updateContainerImpl(root.current, 2, element, root, null, null),\n        flushSyncWork$1());\n    }\n    function scheduleRefresh(root, update) {\n      if (null !== resolveFamily) {\n        var staleFamilies = update.staleFamilies;\n        update = update.updatedFamilies;\n        flushPendingEffects();\n        scheduleFibersWithFamiliesRecursively(\n          root.current,\n          update,\n          staleFamilies\n        );\n        flushSyncWork$1();\n      }\n    }\n    function setRefreshHandler(handler) {\n      resolveFamily = handler;\n    }\n    function isValidContainer(node) {\n      return !(\n        !node ||\n        (1 !== node.nodeType && 9 !== node.nodeType && 11 !== node.nodeType)\n      );\n    }\n    function getNearestMountedFiber(fiber) {\n      var node = fiber,\n        nearestMounted = fiber;\n      if (fiber.alternate) for (; node.return; ) node = node.return;\n      else {\n        fiber = node;\n        do\n          (node = fiber),\n            0 !== (node.flags & 4098) && (nearestMounted = node.return),\n            (fiber = node.return);\n        while (fiber);\n      }\n      return 3 === node.tag ? nearestMounted : null;\n    }\n    function getSuspenseInstanceFromFiber(fiber) {\n      if (13 === fiber.tag) {\n        var suspenseState = fiber.memoizedState;\n        null === suspenseState &&\n          ((fiber = fiber.alternate),\n          null !== fiber && (suspenseState = fiber.memoizedState));\n        if (null !== suspenseState) return suspenseState.dehydrated;\n      }\n      return null;\n    }\n    function assertIsMounted(fiber) {\n      if (getNearestMountedFiber(fiber) !== fiber)\n        throw Error(\"Unable to find node on an unmounted component.\");\n    }\n    function findCurrentFiberUsingSlowPath(fiber) {\n      var alternate = fiber.alternate;\n      if (!alternate) {\n        alternate = getNearestMountedFiber(fiber);\n        if (null === alternate)\n          throw Error(\"Unable to find node on an unmounted component.\");\n        return alternate !== fiber ? null : fiber;\n      }\n      for (var a = fiber, b = alternate; ; ) {\n        var parentA = a.return;\n        if (null === parentA) break;\n        var parentB = parentA.alternate;\n        if (null === parentB) {\n          b = parentA.return;\n          if (null !== b) {\n            a = b;\n            continue;\n          }\n          break;\n        }\n        if (parentA.child === parentB.child) {\n          for (parentB = parentA.child; parentB; ) {\n            if (parentB === a) return assertIsMounted(parentA), fiber;\n            if (parentB === b) return assertIsMounted(parentA), alternate;\n            parentB = parentB.sibling;\n          }\n          throw Error(\"Unable to find node on an unmounted component.\");\n        }\n        if (a.return !== b.return) (a = parentA), (b = parentB);\n        else {\n          for (var didFindChild = !1, _child = parentA.child; _child; ) {\n            if (_child === a) {\n              didFindChild = !0;\n              a = parentA;\n              b = parentB;\n              break;\n            }\n            if (_child === b) {\n              didFindChild = !0;\n              b = parentA;\n              a = parentB;\n              break;\n            }\n            _child = _child.sibling;\n          }\n          if (!didFindChild) {\n            for (_child = parentB.child; _child; ) {\n              if (_child === a) {\n                didFindChild = !0;\n                a = parentB;\n                b = parentA;\n                break;\n              }\n              if (_child === b) {\n                didFindChild = !0;\n                b = parentB;\n                a = parentA;\n                break;\n              }\n              _child = _child.sibling;\n            }\n            if (!didFindChild)\n              throw Error(\n                \"Child was not found in either parent set. This indicates a bug in React related to the return pointer. Please file an issue.\"\n              );\n          }\n        }\n        if (a.alternate !== b)\n          throw Error(\n            \"Return fibers should always be each others' alternates. This error is likely caused by a bug in React. Please file an issue.\"\n          );\n      }\n      if (3 !== a.tag)\n        throw Error(\"Unable to find node on an unmounted component.\");\n      return a.stateNode.current === a ? fiber : alternate;\n    }\n    function findCurrentHostFiberImpl(node) {\n      var tag = node.tag;\n      if (5 === tag || 26 === tag || 27 === tag || 6 === tag) return node;\n      for (node = node.child; null !== node; ) {\n        tag = findCurrentHostFiberImpl(node);\n        if (null !== tag) return tag;\n        node = node.sibling;\n      }\n      return null;\n    }\n    function getIteratorFn(maybeIterable) {\n      if (null === maybeIterable || \"object\" !== typeof maybeIterable)\n        return null;\n      maybeIterable =\n        (MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL]) ||\n        maybeIterable[\"@@iterator\"];\n      return \"function\" === typeof maybeIterable ? maybeIterable : null;\n    }\n    function getComponentNameFromType(type) {\n      if (null == type) return null;\n      if (\"function\" === typeof type)\n        return type.$$typeof === REACT_CLIENT_REFERENCE\n          ? null\n          : type.displayName || type.name || null;\n      if (\"string\" === typeof type) return type;\n      switch (type) {\n        case REACT_FRAGMENT_TYPE:\n          return \"Fragment\";\n        case REACT_PROFILER_TYPE:\n          return \"Profiler\";\n        case REACT_STRICT_MODE_TYPE:\n          return \"StrictMode\";\n        case REACT_SUSPENSE_TYPE:\n          return \"Suspense\";\n        case REACT_SUSPENSE_LIST_TYPE:\n          return \"SuspenseList\";\n        case REACT_ACTIVITY_TYPE:\n          return \"Activity\";\n      }\n      if (\"object\" === typeof type)\n        switch (\n          (\"number\" === typeof type.tag &&\n            console.error(\n              \"Received an unexpected object in getComponentNameFromType(). This is likely a bug in React. Please file an issue.\"\n            ),\n          type.$$typeof)\n        ) {\n          case REACT_PORTAL_TYPE:\n            return \"Portal\";\n          case REACT_CONTEXT_TYPE:\n            return (type.displayName || \"Context\") + \".Provider\";\n          case REACT_CONSUMER_TYPE:\n            return (type._context.displayName || \"Context\") + \".Consumer\";\n          case REACT_FORWARD_REF_TYPE:\n            var innerType = type.render;\n            type = type.displayName;\n            type ||\n              ((type = innerType.displayName || innerType.name || \"\"),\n              (type = \"\" !== type ? \"ForwardRef(\" + type + \")\" : \"ForwardRef\"));\n            return type;\n          case REACT_MEMO_TYPE:\n            return (\n              (innerType = type.displayName || null),\n              null !== innerType\n                ? innerType\n                : getComponentNameFromType(type.type) || \"Memo\"\n            );\n          case REACT_LAZY_TYPE:\n            innerType = type._payload;\n            type = type._init;\n            try {\n              return getComponentNameFromType(type(innerType));\n            } catch (x) {}\n        }\n      return null;\n    }\n    function getComponentNameFromOwner(owner) {\n      return \"number\" === typeof owner.tag\n        ? getComponentNameFromFiber(owner)\n        : \"string\" === typeof owner.name\n          ? owner.name\n          : null;\n    }\n    function getComponentNameFromFiber(fiber) {\n      var type = fiber.type;\n      switch (fiber.tag) {\n        case 31:\n          return \"Activity\";\n        case 24:\n          return \"Cache\";\n        case 9:\n          return (type._context.displayName || \"Context\") + \".Consumer\";\n        case 10:\n          return (type.displayName || \"Context\") + \".Provider\";\n        case 18:\n          return \"DehydratedFragment\";\n        case 11:\n          return (\n            (fiber = type.render),\n            (fiber = fiber.displayName || fiber.name || \"\"),\n            type.displayName ||\n              (\"\" !== fiber ? \"ForwardRef(\" + fiber + \")\" : \"ForwardRef\")\n          );\n        case 7:\n          return \"Fragment\";\n        case 26:\n        case 27:\n        case 5:\n          return type;\n        case 4:\n          return \"Portal\";\n        case 3:\n          return \"Root\";\n        case 6:\n          return \"Text\";\n        case 16:\n          return getComponentNameFromType(type);\n        case 8:\n          return type === REACT_STRICT_MODE_TYPE ? \"StrictMode\" : \"Mode\";\n        case 22:\n          return \"Offscreen\";\n        case 12:\n          return \"Profiler\";\n        case 21:\n          return \"Scope\";\n        case 13:\n          return \"Suspense\";\n        case 19:\n          return \"SuspenseList\";\n        case 25:\n          return \"TracingMarker\";\n        case 1:\n        case 0:\n        case 14:\n        case 15:\n          if (\"function\" === typeof type)\n            return type.displayName || type.name || null;\n          if (\"string\" === typeof type) return type;\n          break;\n        case 29:\n          type = fiber._debugInfo;\n          if (null != type)\n            for (var i = type.length - 1; 0 <= i; i--)\n              if (\"string\" === typeof type[i].name) return type[i].name;\n          if (null !== fiber.return)\n            return getComponentNameFromFiber(fiber.return);\n      }\n      return null;\n    }\n    function createCursor(defaultValue) {\n      return { current: defaultValue };\n    }\n    function pop(cursor, fiber) {\n      0 > index$jscomp$0\n        ? console.error(\"Unexpected pop.\")\n        : (fiber !== fiberStack[index$jscomp$0] &&\n            console.error(\"Unexpected Fiber popped.\"),\n          (cursor.current = valueStack[index$jscomp$0]),\n          (valueStack[index$jscomp$0] = null),\n          (fiberStack[index$jscomp$0] = null),\n          index$jscomp$0--);\n    }\n    function push(cursor, value, fiber) {\n      index$jscomp$0++;\n      valueStack[index$jscomp$0] = cursor.current;\n      fiberStack[index$jscomp$0] = fiber;\n      cursor.current = value;\n    }\n    function requiredContext(c) {\n      null === c &&\n        console.error(\n          \"Expected host context to exist. This error is likely caused by a bug in React. Please file an issue.\"\n        );\n      return c;\n    }\n    function pushHostContainer(fiber, nextRootInstance) {\n      push(rootInstanceStackCursor, nextRootInstance, fiber);\n      push(contextFiberStackCursor, fiber, fiber);\n      push(contextStackCursor, null, fiber);\n      var nextRootContext = nextRootInstance.nodeType;\n      switch (nextRootContext) {\n        case 9:\n        case 11:\n          nextRootContext = 9 === nextRootContext ? \"#document\" : \"#fragment\";\n          nextRootInstance = (nextRootInstance =\n            nextRootInstance.documentElement)\n            ? (nextRootInstance = nextRootInstance.namespaceURI)\n              ? getOwnHostContext(nextRootInstance)\n              : HostContextNamespaceNone\n            : HostContextNamespaceNone;\n          break;\n        default:\n          if (\n            ((nextRootContext = nextRootInstance.tagName),\n            (nextRootInstance = nextRootInstance.namespaceURI))\n          )\n            (nextRootInstance = getOwnHostContext(nextRootInstance)),\n              (nextRootInstance = getChildHostContextProd(\n                nextRootInstance,\n                nextRootContext\n              ));\n          else\n            switch (nextRootContext) {\n              case \"svg\":\n                nextRootInstance = HostContextNamespaceSvg;\n                break;\n              case \"math\":\n                nextRootInstance = HostContextNamespaceMath;\n                break;\n              default:\n                nextRootInstance = HostContextNamespaceNone;\n            }\n      }\n      nextRootContext = nextRootContext.toLowerCase();\n      nextRootContext = updatedAncestorInfoDev(null, nextRootContext);\n      nextRootContext = {\n        context: nextRootInstance,\n        ancestorInfo: nextRootContext\n      };\n      pop(contextStackCursor, fiber);\n      push(contextStackCursor, nextRootContext, fiber);\n    }\n    function popHostContainer(fiber) {\n      pop(contextStackCursor, fiber);\n      pop(contextFiberStackCursor, fiber);\n      pop(rootInstanceStackCursor, fiber);\n    }\n    function getHostContext() {\n      return requiredContext(contextStackCursor.current);\n    }\n    function pushHostContext(fiber) {\n      null !== fiber.memoizedState &&\n        push(hostTransitionProviderCursor, fiber, fiber);\n      var context = requiredContext(contextStackCursor.current);\n      var type = fiber.type;\n      var nextContext = getChildHostContextProd(context.context, type);\n      type = updatedAncestorInfoDev(context.ancestorInfo, type);\n      nextContext = { context: nextContext, ancestorInfo: type };\n      context !== nextContext &&\n        (push(contextFiberStackCursor, fiber, fiber),\n        push(contextStackCursor, nextContext, fiber));\n    }\n    function popHostContext(fiber) {\n      contextFiberStackCursor.current === fiber &&\n        (pop(contextStackCursor, fiber), pop(contextFiberStackCursor, fiber));\n      hostTransitionProviderCursor.current === fiber &&\n        (pop(hostTransitionProviderCursor, fiber),\n        (HostTransitionContext._currentValue = NotPendingTransition));\n    }\n    function typeName(value) {\n      return (\n        (\"function\" === typeof Symbol &&\n          Symbol.toStringTag &&\n          value[Symbol.toStringTag]) ||\n        value.constructor.name ||\n        \"Object\"\n      );\n    }\n    function willCoercionThrow(value) {\n      try {\n        return testStringCoercion(value), !1;\n      } catch (e) {\n        return !0;\n      }\n    }\n    function testStringCoercion(value) {\n      return \"\" + value;\n    }\n    function checkAttributeStringCoercion(value, attributeName) {\n      if (willCoercionThrow(value))\n        return (\n          console.error(\n            \"The provided `%s` attribute is an unsupported type %s. This value must be coerced to a string before using it here.\",\n            attributeName,\n            typeName(value)\n          ),\n          testStringCoercion(value)\n        );\n    }\n    function checkCSSPropertyStringCoercion(value, propName) {\n      if (willCoercionThrow(value))\n        return (\n          console.error(\n            \"The provided `%s` CSS property is an unsupported type %s. This value must be coerced to a string before using it here.\",\n            propName,\n            typeName(value)\n          ),\n          testStringCoercion(value)\n        );\n    }\n    function checkFormFieldValueStringCoercion(value) {\n      if (willCoercionThrow(value))\n        return (\n          console.error(\n            \"Form field values (value, checked, defaultValue, or defaultChecked props) must be strings, not %s. This value must be coerced to a string before using it here.\",\n            typeName(value)\n          ),\n          testStringCoercion(value)\n        );\n    }\n    function injectInternals(internals) {\n      if (\"undefined\" === typeof __REACT_DEVTOOLS_GLOBAL_HOOK__) return !1;\n      var hook = __REACT_DEVTOOLS_GLOBAL_HOOK__;\n      if (hook.isDisabled) return !0;\n      if (!hook.supportsFiber)\n        return (\n          console.error(\n            \"The installed version of React DevTools is too old and will not work with the current version of React. Please update React DevTools. https://react.dev/link/react-devtools\"\n          ),\n          !0\n        );\n      try {\n        (rendererID = hook.inject(internals)), (injectedHook = hook);\n      } catch (err) {\n        console.error(\"React instrumentation encountered an error: %s.\", err);\n      }\n      return hook.checkDCE ? !0 : !1;\n    }\n    function setIsStrictModeForDevtools(newIsStrictMode) {\n      \"function\" === typeof log$1 &&\n        unstable_setDisableYieldValue(newIsStrictMode);\n      if (injectedHook && \"function\" === typeof injectedHook.setStrictMode)\n        try {\n          injectedHook.setStrictMode(rendererID, newIsStrictMode);\n        } catch (err) {\n          hasLoggedError ||\n            ((hasLoggedError = !0),\n            console.error(\n              \"React instrumentation encountered an error: %s\",\n              err\n            ));\n        }\n    }\n    function injectProfilingHooks(profilingHooks) {\n      injectedProfilingHooks = profilingHooks;\n    }\n    function markCommitStopped() {\n      null !== injectedProfilingHooks &&\n        \"function\" === typeof injectedProfilingHooks.markCommitStopped &&\n        injectedProfilingHooks.markCommitStopped();\n    }\n    function markComponentRenderStarted(fiber) {\n      null !== injectedProfilingHooks &&\n        \"function\" ===\n          typeof injectedProfilingHooks.markComponentRenderStarted &&\n        injectedProfilingHooks.markComponentRenderStarted(fiber);\n    }\n    function markComponentRenderStopped() {\n      null !== injectedProfilingHooks &&\n        \"function\" ===\n          typeof injectedProfilingHooks.markComponentRenderStopped &&\n        injectedProfilingHooks.markComponentRenderStopped();\n    }\n    function markRenderStarted(lanes) {\n      null !== injectedProfilingHooks &&\n        \"function\" === typeof injectedProfilingHooks.markRenderStarted &&\n        injectedProfilingHooks.markRenderStarted(lanes);\n    }\n    function markRenderStopped() {\n      null !== injectedProfilingHooks &&\n        \"function\" === typeof injectedProfilingHooks.markRenderStopped &&\n        injectedProfilingHooks.markRenderStopped();\n    }\n    function markStateUpdateScheduled(fiber, lane) {\n      null !== injectedProfilingHooks &&\n        \"function\" === typeof injectedProfilingHooks.markStateUpdateScheduled &&\n        injectedProfilingHooks.markStateUpdateScheduled(fiber, lane);\n    }\n    function clz32Fallback(x) {\n      x >>>= 0;\n      return 0 === x ? 32 : (31 - ((log(x) / LN2) | 0)) | 0;\n    }\n    function getLabelForLane(lane) {\n      if (lane & 1) return \"SyncHydrationLane\";\n      if (lane & 2) return \"Sync\";\n      if (lane & 4) return \"InputContinuousHydration\";\n      if (lane & 8) return \"InputContinuous\";\n      if (lane & 16) return \"DefaultHydration\";\n      if (lane & 32) return \"Default\";\n      if (lane & 128) return \"TransitionHydration\";\n      if (lane & 4194048) return \"Transition\";\n      if (lane & 62914560) return \"Retry\";\n      if (lane & 67108864) return \"SelectiveHydration\";\n      if (lane & 134217728) return \"IdleHydration\";\n      if (lane & 268435456) return \"Idle\";\n      if (lane & 536870912) return \"Offscreen\";\n      if (lane & 1073741824) return \"Deferred\";\n    }\n    function getHighestPriorityLanes(lanes) {\n      var pendingSyncLanes = lanes & 42;\n      if (0 !== pendingSyncLanes) return pendingSyncLanes;\n      switch (lanes & -lanes) {\n        case 1:\n          return 1;\n        case 2:\n          return 2;\n        case 4:\n          return 4;\n        case 8:\n          return 8;\n        case 16:\n          return 16;\n        case 32:\n          return 32;\n        case 64:\n          return 64;\n        case 128:\n          return 128;\n        case 256:\n        case 512:\n        case 1024:\n        case 2048:\n        case 4096:\n        case 8192:\n        case 16384:\n        case 32768:\n        case 65536:\n        case 131072:\n        case 262144:\n        case 524288:\n        case 1048576:\n        case 2097152:\n          return lanes & 4194048;\n        case 4194304:\n        case 8388608:\n        case 16777216:\n        case 33554432:\n          return lanes & 62914560;\n        case 67108864:\n          return 67108864;\n        case 134217728:\n          return 134217728;\n        case 268435456:\n          return 268435456;\n        case 536870912:\n          return 536870912;\n        case 1073741824:\n          return 0;\n        default:\n          return (\n            console.error(\n              \"Should have found matching lanes. This is a bug in React.\"\n            ),\n            lanes\n          );\n      }\n    }\n    function getNextLanes(root, wipLanes, rootHasPendingCommit) {\n      var pendingLanes = root.pendingLanes;\n      if (0 === pendingLanes) return 0;\n      var nextLanes = 0,\n        suspendedLanes = root.suspendedLanes,\n        pingedLanes = root.pingedLanes;\n      root = root.warmLanes;\n      var nonIdlePendingLanes = pendingLanes & 134217727;\n      0 !== nonIdlePendingLanes\n        ? ((pendingLanes = nonIdlePendingLanes & ~suspendedLanes),\n          0 !== pendingLanes\n            ? (nextLanes = getHighestPriorityLanes(pendingLanes))\n            : ((pingedLanes &= nonIdlePendingLanes),\n              0 !== pingedLanes\n                ? (nextLanes = getHighestPriorityLanes(pingedLanes))\n                : rootHasPendingCommit ||\n                  ((rootHasPendingCommit = nonIdlePendingLanes & ~root),\n                  0 !== rootHasPendingCommit &&\n                    (nextLanes =\n                      getHighestPriorityLanes(rootHasPendingCommit)))))\n        : ((nonIdlePendingLanes = pendingLanes & ~suspendedLanes),\n          0 !== nonIdlePendingLanes\n            ? (nextLanes = getHighestPriorityLanes(nonIdlePendingLanes))\n            : 0 !== pingedLanes\n              ? (nextLanes = getHighestPriorityLanes(pingedLanes))\n              : rootHasPendingCommit ||\n                ((rootHasPendingCommit = pendingLanes & ~root),\n                0 !== rootHasPendingCommit &&\n                  (nextLanes = getHighestPriorityLanes(rootHasPendingCommit))));\n      return 0 === nextLanes\n        ? 0\n        : 0 !== wipLanes &&\n            wipLanes !== nextLanes &&\n            0 === (wipLanes & suspendedLanes) &&\n            ((suspendedLanes = nextLanes & -nextLanes),\n            (rootHasPendingCommit = wipLanes & -wipLanes),\n            suspendedLanes >= rootHasPendingCommit ||\n              (32 === suspendedLanes && 0 !== (rootHasPendingCommit & 4194048)))\n          ? wipLanes\n          : nextLanes;\n    }\n    function checkIfRootIsPrerendering(root, renderLanes) {\n      return (\n        0 ===\n        (root.pendingLanes &\n          ~(root.suspendedLanes & ~root.pingedLanes) &\n          renderLanes)\n      );\n    }\n    function computeExpirationTime(lane, currentTime) {\n      switch (lane) {\n        case 1:\n        case 2:\n        case 4:\n        case 8:\n        case 64:\n          return currentTime + 250;\n        case 16:\n        case 32:\n        case 128:\n        case 256:\n        case 512:\n        case 1024:\n        case 2048:\n        case 4096:\n        case 8192:\n        case 16384:\n        case 32768:\n        case 65536:\n        case 131072:\n        case 262144:\n        case 524288:\n        case 1048576:\n        case 2097152:\n          return currentTime + 5e3;\n        case 4194304:\n        case 8388608:\n        case 16777216:\n        case 33554432:\n          return -1;\n        case 67108864:\n        case 134217728:\n        case 268435456:\n        case 536870912:\n        case 1073741824:\n          return -1;\n        default:\n          return (\n            console.error(\n              \"Should have found matching lanes. This is a bug in React.\"\n            ),\n            -1\n          );\n      }\n    }\n    function claimNextTransitionLane() {\n      var lane = nextTransitionLane;\n      nextTransitionLane <<= 1;\n      0 === (nextTransitionLane & 4194048) && (nextTransitionLane = 256);\n      return lane;\n    }\n    function claimNextRetryLane() {\n      var lane = nextRetryLane;\n      nextRetryLane <<= 1;\n      0 === (nextRetryLane & 62914560) && (nextRetryLane = 4194304);\n      return lane;\n    }\n    function createLaneMap(initial) {\n      for (var laneMap = [], i = 0; 31 > i; i++) laneMap.push(initial);\n      return laneMap;\n    }\n    function markRootUpdated$1(root, updateLane) {\n      root.pendingLanes |= updateLane;\n      268435456 !== updateLane &&\n        ((root.suspendedLanes = 0),\n        (root.pingedLanes = 0),\n        (root.warmLanes = 0));\n    }\n    function markRootFinished(\n      root,\n      finishedLanes,\n      remainingLanes,\n      spawnedLane,\n      updatedLanes,\n      suspendedRetryLanes\n    ) {\n      var previouslyPendingLanes = root.pendingLanes;\n      root.pendingLanes = remainingLanes;\n      root.suspendedLanes = 0;\n      root.pingedLanes = 0;\n      root.warmLanes = 0;\n      root.expiredLanes &= remainingLanes;\n      root.entangledLanes &= remainingLanes;\n      root.errorRecoveryDisabledLanes &= remainingLanes;\n      root.shellSuspendCounter = 0;\n      var entanglements = root.entanglements,\n        expirationTimes = root.expirationTimes,\n        hiddenUpdates = root.hiddenUpdates;\n      for (\n        remainingLanes = previouslyPendingLanes & ~remainingLanes;\n        0 < remainingLanes;\n\n      ) {\n        var index = 31 - clz32(remainingLanes),\n          lane = 1 << index;\n        entanglements[index] = 0;\n        expirationTimes[index] = -1;\n        var hiddenUpdatesForLane = hiddenUpdates[index];\n        if (null !== hiddenUpdatesForLane)\n          for (\n            hiddenUpdates[index] = null, index = 0;\n            index < hiddenUpdatesForLane.length;\n            index++\n          ) {\n            var update = hiddenUpdatesForLane[index];\n            null !== update && (update.lane &= -536870913);\n          }\n        remainingLanes &= ~lane;\n      }\n      0 !== spawnedLane && markSpawnedDeferredLane(root, spawnedLane, 0);\n      0 !== suspendedRetryLanes &&\n        0 === updatedLanes &&\n        0 !== root.tag &&\n        (root.suspendedLanes |=\n          suspendedRetryLanes & ~(previouslyPendingLanes & ~finishedLanes));\n    }\n    function markSpawnedDeferredLane(root, spawnedLane, entangledLanes) {\n      root.pendingLanes |= spawnedLane;\n      root.suspendedLanes &= ~spawnedLane;\n      var spawnedLaneIndex = 31 - clz32(spawnedLane);\n      root.entangledLanes |= spawnedLane;\n      root.entanglements[spawnedLaneIndex] =\n        root.entanglements[spawnedLaneIndex] |\n        1073741824 |\n        (entangledLanes & 4194090);\n    }\n    function markRootEntangled(root, entangledLanes) {\n      var rootEntangledLanes = (root.entangledLanes |= entangledLanes);\n      for (root = root.entanglements; rootEntangledLanes; ) {\n        var index = 31 - clz32(rootEntangledLanes),\n          lane = 1 << index;\n        (lane & entangledLanes) | (root[index] & entangledLanes) &&\n          (root[index] |= entangledLanes);\n        rootEntangledLanes &= ~lane;\n      }\n    }\n    function getBumpedLaneForHydrationByLane(lane) {\n      switch (lane) {\n        case 2:\n          lane = 1;\n          break;\n        case 8:\n          lane = 4;\n          break;\n        case 32:\n          lane = 16;\n          break;\n        case 256:\n        case 512:\n        case 1024:\n        case 2048:\n        case 4096:\n        case 8192:\n        case 16384:\n        case 32768:\n        case 65536:\n        case 131072:\n        case 262144:\n        case 524288:\n        case 1048576:\n        case 2097152:\n        case 4194304:\n        case 8388608:\n        case 16777216:\n        case 33554432:\n          lane = 128;\n          break;\n        case 268435456:\n          lane = 134217728;\n          break;\n        default:\n          lane = 0;\n      }\n      return lane;\n    }\n    function addFiberToLanesMap(root, fiber, lanes) {\n      if (isDevToolsPresent)\n        for (root = root.pendingUpdatersLaneMap; 0 < lanes; ) {\n          var index = 31 - clz32(lanes),\n            lane = 1 << index;\n          root[index].add(fiber);\n          lanes &= ~lane;\n        }\n    }\n    function movePendingFibersToMemoized(root, lanes) {\n      if (isDevToolsPresent)\n        for (\n          var pendingUpdatersLaneMap = root.pendingUpdatersLaneMap,\n            memoizedUpdaters = root.memoizedUpdaters;\n          0 < lanes;\n\n        ) {\n          var index = 31 - clz32(lanes);\n          root = 1 << index;\n          index = pendingUpdatersLaneMap[index];\n          0 < index.size &&\n            (index.forEach(function (fiber) {\n              var alternate = fiber.alternate;\n              (null !== alternate && memoizedUpdaters.has(alternate)) ||\n                memoizedUpdaters.add(fiber);\n            }),\n            index.clear());\n          lanes &= ~root;\n        }\n    }\n    function lanesToEventPriority(lanes) {\n      lanes &= -lanes;\n      return 0 !== DiscreteEventPriority && DiscreteEventPriority < lanes\n        ? 0 !== ContinuousEventPriority && ContinuousEventPriority < lanes\n          ? 0 !== (lanes & 134217727)\n            ? DefaultEventPriority\n            : IdleEventPriority\n          : ContinuousEventPriority\n        : DiscreteEventPriority;\n    }\n    function resolveUpdatePriority() {\n      var updatePriority = ReactDOMSharedInternals.p;\n      if (0 !== updatePriority) return updatePriority;\n      updatePriority = window.event;\n      return void 0 === updatePriority\n        ? DefaultEventPriority\n        : getEventPriority(updatePriority.type);\n    }\n    function runWithPriority(priority, fn) {\n      var previousPriority = ReactDOMSharedInternals.p;\n      try {\n        return (ReactDOMSharedInternals.p = priority), fn();\n      } finally {\n        ReactDOMSharedInternals.p = previousPriority;\n      }\n    }\n    function detachDeletedInstance(node) {\n      delete node[internalInstanceKey];\n      delete node[internalPropsKey];\n      delete node[internalEventHandlersKey];\n      delete node[internalEventHandlerListenersKey];\n      delete node[internalEventHandlesSetKey];\n    }\n    function getClosestInstanceFromNode(targetNode) {\n      var targetInst = targetNode[internalInstanceKey];\n      if (targetInst) return targetInst;\n      for (var parentNode = targetNode.parentNode; parentNode; ) {\n        if (\n          (targetInst =\n            parentNode[internalContainerInstanceKey] ||\n            parentNode[internalInstanceKey])\n        ) {\n          parentNode = targetInst.alternate;\n          if (\n            null !== targetInst.child ||\n            (null !== parentNode && null !== parentNode.child)\n          )\n            for (\n              targetNode = getParentSuspenseInstance(targetNode);\n              null !== targetNode;\n\n            ) {\n              if ((parentNode = targetNode[internalInstanceKey]))\n                return parentNode;\n              targetNode = getParentSuspenseInstance(targetNode);\n            }\n          return targetInst;\n        }\n        targetNode = parentNode;\n        parentNode = targetNode.parentNode;\n      }\n      return null;\n    }\n    function getInstanceFromNode(node) {\n      if (\n        (node = node[internalInstanceKey] || node[internalContainerInstanceKey])\n      ) {\n        var tag = node.tag;\n        if (\n          5 === tag ||\n          6 === tag ||\n          13 === tag ||\n          26 === tag ||\n          27 === tag ||\n          3 === tag\n        )\n          return node;\n      }\n      return null;\n    }\n    function getNodeFromInstance(inst) {\n      var tag = inst.tag;\n      if (5 === tag || 26 === tag || 27 === tag || 6 === tag)\n        return inst.stateNode;\n      throw Error(\"getNodeFromInstance: Invalid argument.\");\n    }\n    function getResourcesFromRoot(root) {\n      var resources = root[internalRootNodeResourcesKey];\n      resources ||\n        (resources = root[internalRootNodeResourcesKey] =\n          { hoistableStyles: new Map(), hoistableScripts: new Map() });\n      return resources;\n    }\n    function markNodeAsHoistable(node) {\n      node[internalHoistableMarker] = !0;\n    }\n    function registerTwoPhaseEvent(registrationName, dependencies) {\n      registerDirectEvent(registrationName, dependencies);\n      registerDirectEvent(registrationName + \"Capture\", dependencies);\n    }\n    function registerDirectEvent(registrationName, dependencies) {\n      registrationNameDependencies[registrationName] &&\n        console.error(\n          \"EventRegistry: More than one plugin attempted to publish the same registration name, `%s`.\",\n          registrationName\n        );\n      registrationNameDependencies[registrationName] = dependencies;\n      var lowerCasedName = registrationName.toLowerCase();\n      possibleRegistrationNames[lowerCasedName] = registrationName;\n      \"onDoubleClick\" === registrationName &&\n        (possibleRegistrationNames.ondblclick = registrationName);\n      for (\n        registrationName = 0;\n        registrationName < dependencies.length;\n        registrationName++\n      )\n        allNativeEvents.add(dependencies[registrationName]);\n    }\n    function checkControlledValueProps(tagName, props) {\n      hasReadOnlyValue[props.type] ||\n        props.onChange ||\n        props.onInput ||\n        props.readOnly ||\n        props.disabled ||\n        null == props.value ||\n        (\"select\" === tagName\n          ? console.error(\n              \"You provided a `value` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultValue`. Otherwise, set `onChange`.\"\n            )\n          : console.error(\n              \"You provided a `value` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultValue`. Otherwise, set either `onChange` or `readOnly`.\"\n            ));\n      props.onChange ||\n        props.readOnly ||\n        props.disabled ||\n        null == props.checked ||\n        console.error(\n          \"You provided a `checked` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultChecked`. Otherwise, set either `onChange` or `readOnly`.\"\n        );\n    }\n    function isAttributeNameSafe(attributeName) {\n      if (hasOwnProperty.call(validatedAttributeNameCache, attributeName))\n        return !0;\n      if (hasOwnProperty.call(illegalAttributeNameCache, attributeName))\n        return !1;\n      if (VALID_ATTRIBUTE_NAME_REGEX.test(attributeName))\n        return (validatedAttributeNameCache[attributeName] = !0);\n      illegalAttributeNameCache[attributeName] = !0;\n      console.error(\"Invalid attribute name: `%s`\", attributeName);\n      return !1;\n    }\n    function getValueForAttributeOnCustomComponent(node, name, expected) {\n      if (isAttributeNameSafe(name)) {\n        if (!node.hasAttribute(name)) {\n          switch (typeof expected) {\n            case \"symbol\":\n            case \"object\":\n              return expected;\n            case \"function\":\n              return expected;\n            case \"boolean\":\n              if (!1 === expected) return expected;\n          }\n          return void 0 === expected ? void 0 : null;\n        }\n        node = node.getAttribute(name);\n        if (\"\" === node && !0 === expected) return !0;\n        checkAttributeStringCoercion(expected, name);\n        return node === \"\" + expected ? expected : node;\n      }\n    }\n    function setValueForAttribute(node, name, value) {\n      if (isAttributeNameSafe(name))\n        if (null === value) node.removeAttribute(name);\n        else {\n          switch (typeof value) {\n            case \"undefined\":\n            case \"function\":\n            case \"symbol\":\n              node.removeAttribute(name);\n              return;\n            case \"boolean\":\n              var prefix = name.toLowerCase().slice(0, 5);\n              if (\"data-\" !== prefix && \"aria-\" !== prefix) {\n                node.removeAttribute(name);\n                return;\n              }\n          }\n          checkAttributeStringCoercion(value, name);\n          node.setAttribute(name, \"\" + value);\n        }\n    }\n    function setValueForKnownAttribute(node, name, value) {\n      if (null === value) node.removeAttribute(name);\n      else {\n        switch (typeof value) {\n          case \"undefined\":\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            node.removeAttribute(name);\n            return;\n        }\n        checkAttributeStringCoercion(value, name);\n        node.setAttribute(name, \"\" + value);\n      }\n    }\n    function setValueForNamespacedAttribute(node, namespace, name, value) {\n      if (null === value) node.removeAttribute(name);\n      else {\n        switch (typeof value) {\n          case \"undefined\":\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            node.removeAttribute(name);\n            return;\n        }\n        checkAttributeStringCoercion(value, name);\n        node.setAttributeNS(namespace, name, \"\" + value);\n      }\n    }\n    function disabledLog() {}\n    function disableLogs() {\n      if (0 === disabledDepth) {\n        prevLog = console.log;\n        prevInfo = console.info;\n        prevWarn = console.warn;\n        prevError = console.error;\n        prevGroup = console.group;\n        prevGroupCollapsed = console.groupCollapsed;\n        prevGroupEnd = console.groupEnd;\n        var props = {\n          configurable: !0,\n          enumerable: !0,\n          value: disabledLog,\n          writable: !0\n        };\n        Object.defineProperties(console, {\n          info: props,\n          log: props,\n          warn: props,\n          error: props,\n          group: props,\n          groupCollapsed: props,\n          groupEnd: props\n        });\n      }\n      disabledDepth++;\n    }\n    function reenableLogs() {\n      disabledDepth--;\n      if (0 === disabledDepth) {\n        var props = { configurable: !0, enumerable: !0, writable: !0 };\n        Object.defineProperties(console, {\n          log: assign({}, props, { value: prevLog }),\n          info: assign({}, props, { value: prevInfo }),\n          warn: assign({}, props, { value: prevWarn }),\n          error: assign({}, props, { value: prevError }),\n          group: assign({}, props, { value: prevGroup }),\n          groupCollapsed: assign({}, props, { value: prevGroupCollapsed }),\n          groupEnd: assign({}, props, { value: prevGroupEnd })\n        });\n      }\n      0 > disabledDepth &&\n        console.error(\n          \"disabledDepth fell below zero. This is a bug in React. Please file an issue.\"\n        );\n    }\n    function describeBuiltInComponentFrame(name) {\n      if (void 0 === prefix)\n        try {\n          throw Error();\n        } catch (x) {\n          var match = x.stack.trim().match(/\\n( *(at )?)/);\n          prefix = (match && match[1]) || \"\";\n          suffix =\n            -1 < x.stack.indexOf(\"\\n    at\")\n              ? \" (<anonymous>)\"\n              : -1 < x.stack.indexOf(\"@\")\n                ? \"@unknown:0:0\"\n                : \"\";\n        }\n      return \"\\n\" + prefix + name + suffix;\n    }\n    function describeNativeComponentFrame(fn, construct) {\n      if (!fn || reentry) return \"\";\n      var frame = componentFrameCache.get(fn);\n      if (void 0 !== frame) return frame;\n      reentry = !0;\n      frame = Error.prepareStackTrace;\n      Error.prepareStackTrace = void 0;\n      var previousDispatcher = null;\n      previousDispatcher = ReactSharedInternals.H;\n      ReactSharedInternals.H = null;\n      disableLogs();\n      try {\n        var RunInRootFrame = {\n          DetermineComponentFrameRoot: function () {\n            try {\n              if (construct) {\n                var Fake = function () {\n                  throw Error();\n                };\n                Object.defineProperty(Fake.prototype, \"props\", {\n                  set: function () {\n                    throw Error();\n                  }\n                });\n                if (\"object\" === typeof Reflect && Reflect.construct) {\n                  try {\n                    Reflect.construct(Fake, []);\n                  } catch (x) {\n                    var control = x;\n                  }\n                  Reflect.construct(fn, [], Fake);\n                } else {\n                  try {\n                    Fake.call();\n                  } catch (x$0) {\n                    control = x$0;\n                  }\n                  fn.call(Fake.prototype);\n                }\n              } else {\n                try {\n                  throw Error();\n                } catch (x$1) {\n                  control = x$1;\n                }\n                (Fake = fn()) &&\n                  \"function\" === typeof Fake.catch &&\n                  Fake.catch(function () {});\n              }\n            } catch (sample) {\n              if (sample && control && \"string\" === typeof sample.stack)\n                return [sample.stack, control.stack];\n            }\n            return [null, null];\n          }\n        };\n        RunInRootFrame.DetermineComponentFrameRoot.displayName =\n          \"DetermineComponentFrameRoot\";\n        var namePropDescriptor = Object.getOwnPropertyDescriptor(\n          RunInRootFrame.DetermineComponentFrameRoot,\n          \"name\"\n        );\n        namePropDescriptor &&\n          namePropDescriptor.configurable &&\n          Object.defineProperty(\n            RunInRootFrame.DetermineComponentFrameRoot,\n            \"name\",\n            { value: \"DetermineComponentFrameRoot\" }\n          );\n        var _RunInRootFrame$Deter =\n            RunInRootFrame.DetermineComponentFrameRoot(),\n          sampleStack = _RunInRootFrame$Deter[0],\n          controlStack = _RunInRootFrame$Deter[1];\n        if (sampleStack && controlStack) {\n          var sampleLines = sampleStack.split(\"\\n\"),\n            controlLines = controlStack.split(\"\\n\");\n          for (\n            _RunInRootFrame$Deter = namePropDescriptor = 0;\n            namePropDescriptor < sampleLines.length &&\n            !sampleLines[namePropDescriptor].includes(\n              \"DetermineComponentFrameRoot\"\n            );\n\n          )\n            namePropDescriptor++;\n          for (\n            ;\n            _RunInRootFrame$Deter < controlLines.length &&\n            !controlLines[_RunInRootFrame$Deter].includes(\n              \"DetermineComponentFrameRoot\"\n            );\n\n          )\n            _RunInRootFrame$Deter++;\n          if (\n            namePropDescriptor === sampleLines.length ||\n            _RunInRootFrame$Deter === controlLines.length\n          )\n            for (\n              namePropDescriptor = sampleLines.length - 1,\n                _RunInRootFrame$Deter = controlLines.length - 1;\n              1 <= namePropDescriptor &&\n              0 <= _RunInRootFrame$Deter &&\n              sampleLines[namePropDescriptor] !==\n                controlLines[_RunInRootFrame$Deter];\n\n            )\n              _RunInRootFrame$Deter--;\n          for (\n            ;\n            1 <= namePropDescriptor && 0 <= _RunInRootFrame$Deter;\n            namePropDescriptor--, _RunInRootFrame$Deter--\n          )\n            if (\n              sampleLines[namePropDescriptor] !==\n              controlLines[_RunInRootFrame$Deter]\n            ) {\n              if (1 !== namePropDescriptor || 1 !== _RunInRootFrame$Deter) {\n                do\n                  if (\n                    (namePropDescriptor--,\n                    _RunInRootFrame$Deter--,\n                    0 > _RunInRootFrame$Deter ||\n                      sampleLines[namePropDescriptor] !==\n                        controlLines[_RunInRootFrame$Deter])\n                  ) {\n                    var _frame =\n                      \"\\n\" +\n                      sampleLines[namePropDescriptor].replace(\n                        \" at new \",\n                        \" at \"\n                      );\n                    fn.displayName &&\n                      _frame.includes(\"<anonymous>\") &&\n                      (_frame = _frame.replace(\"<anonymous>\", fn.displayName));\n                    \"function\" === typeof fn &&\n                      componentFrameCache.set(fn, _frame);\n                    return _frame;\n                  }\n                while (1 <= namePropDescriptor && 0 <= _RunInRootFrame$Deter);\n              }\n              break;\n            }\n        }\n      } finally {\n        (reentry = !1),\n          (ReactSharedInternals.H = previousDispatcher),\n          reenableLogs(),\n          (Error.prepareStackTrace = frame);\n      }\n      sampleLines = (sampleLines = fn ? fn.displayName || fn.name : \"\")\n        ? describeBuiltInComponentFrame(sampleLines)\n        : \"\";\n      \"function\" === typeof fn && componentFrameCache.set(fn, sampleLines);\n      return sampleLines;\n    }\n    function formatOwnerStack(error) {\n      var prevPrepareStackTrace = Error.prepareStackTrace;\n      Error.prepareStackTrace = void 0;\n      error = error.stack;\n      Error.prepareStackTrace = prevPrepareStackTrace;\n      error.startsWith(\"Error: react-stack-top-frame\\n\") &&\n        (error = error.slice(29));\n      prevPrepareStackTrace = error.indexOf(\"\\n\");\n      -1 !== prevPrepareStackTrace &&\n        (error = error.slice(prevPrepareStackTrace + 1));\n      prevPrepareStackTrace = error.indexOf(\"react-stack-bottom-frame\");\n      -1 !== prevPrepareStackTrace &&\n        (prevPrepareStackTrace = error.lastIndexOf(\n          \"\\n\",\n          prevPrepareStackTrace\n        ));\n      if (-1 !== prevPrepareStackTrace)\n        error = error.slice(0, prevPrepareStackTrace);\n      else return \"\";\n      return error;\n    }\n    function describeFiber(fiber) {\n      switch (fiber.tag) {\n        case 26:\n        case 27:\n        case 5:\n          return describeBuiltInComponentFrame(fiber.type);\n        case 16:\n          return describeBuiltInComponentFrame(\"Lazy\");\n        case 13:\n          return describeBuiltInComponentFrame(\"Suspense\");\n        case 19:\n          return describeBuiltInComponentFrame(\"SuspenseList\");\n        case 0:\n        case 15:\n          return describeNativeComponentFrame(fiber.type, !1);\n        case 11:\n          return describeNativeComponentFrame(fiber.type.render, !1);\n        case 1:\n          return describeNativeComponentFrame(fiber.type, !0);\n        case 31:\n          return describeBuiltInComponentFrame(\"Activity\");\n        default:\n          return \"\";\n      }\n    }\n    function getStackByFiberInDevAndProd(workInProgress) {\n      try {\n        var info = \"\";\n        do {\n          info += describeFiber(workInProgress);\n          var debugInfo = workInProgress._debugInfo;\n          if (debugInfo)\n            for (var i = debugInfo.length - 1; 0 <= i; i--) {\n              var entry = debugInfo[i];\n              if (\"string\" === typeof entry.name) {\n                var JSCompiler_temp_const = info,\n                  env = entry.env;\n                var JSCompiler_inline_result = describeBuiltInComponentFrame(\n                  entry.name + (env ? \" [\" + env + \"]\" : \"\")\n                );\n                info = JSCompiler_temp_const + JSCompiler_inline_result;\n              }\n            }\n          workInProgress = workInProgress.return;\n        } while (workInProgress);\n        return info;\n      } catch (x) {\n        return \"\\nError generating stack: \" + x.message + \"\\n\" + x.stack;\n      }\n    }\n    function describeFunctionComponentFrameWithoutLineNumber(fn) {\n      return (fn = fn ? fn.displayName || fn.name : \"\")\n        ? describeBuiltInComponentFrame(fn)\n        : \"\";\n    }\n    function getCurrentFiberOwnerNameInDevOrNull() {\n      if (null === current) return null;\n      var owner = current._debugOwner;\n      return null != owner ? getComponentNameFromOwner(owner) : null;\n    }\n    function getCurrentFiberStackInDev() {\n      if (null === current) return \"\";\n      var workInProgress = current;\n      try {\n        var info = \"\";\n        6 === workInProgress.tag && (workInProgress = workInProgress.return);\n        switch (workInProgress.tag) {\n          case 26:\n          case 27:\n          case 5:\n            info += describeBuiltInComponentFrame(workInProgress.type);\n            break;\n          case 13:\n            info += describeBuiltInComponentFrame(\"Suspense\");\n            break;\n          case 19:\n            info += describeBuiltInComponentFrame(\"SuspenseList\");\n            break;\n          case 31:\n            info += describeBuiltInComponentFrame(\"Activity\");\n            break;\n          case 30:\n          case 0:\n          case 15:\n          case 1:\n            workInProgress._debugOwner ||\n              \"\" !== info ||\n              (info += describeFunctionComponentFrameWithoutLineNumber(\n                workInProgress.type\n              ));\n            break;\n          case 11:\n            workInProgress._debugOwner ||\n              \"\" !== info ||\n              (info += describeFunctionComponentFrameWithoutLineNumber(\n                workInProgress.type.render\n              ));\n        }\n        for (; workInProgress; )\n          if (\"number\" === typeof workInProgress.tag) {\n            var fiber = workInProgress;\n            workInProgress = fiber._debugOwner;\n            var debugStack = fiber._debugStack;\n            workInProgress &&\n              debugStack &&\n              (\"string\" !== typeof debugStack &&\n                (fiber._debugStack = debugStack = formatOwnerStack(debugStack)),\n              \"\" !== debugStack && (info += \"\\n\" + debugStack));\n          } else if (null != workInProgress.debugStack) {\n            var ownerStack = workInProgress.debugStack;\n            (workInProgress = workInProgress.owner) &&\n              ownerStack &&\n              (info += \"\\n\" + formatOwnerStack(ownerStack));\n          } else break;\n        var JSCompiler_inline_result = info;\n      } catch (x) {\n        JSCompiler_inline_result =\n          \"\\nError generating stack: \" + x.message + \"\\n\" + x.stack;\n      }\n      return JSCompiler_inline_result;\n    }\n    function runWithFiberInDEV(fiber, callback, arg0, arg1, arg2, arg3, arg4) {\n      var previousFiber = current;\n      setCurrentFiber(fiber);\n      try {\n        return null !== fiber && fiber._debugTask\n          ? fiber._debugTask.run(\n              callback.bind(null, arg0, arg1, arg2, arg3, arg4)\n            )\n          : callback(arg0, arg1, arg2, arg3, arg4);\n      } finally {\n        setCurrentFiber(previousFiber);\n      }\n      // removed by dead control flow\n{}\n    }\n    function setCurrentFiber(fiber) {\n      ReactSharedInternals.getCurrentStack =\n        null === fiber ? null : getCurrentFiberStackInDev;\n      isRendering = !1;\n      current = fiber;\n    }\n    function getToStringValue(value) {\n      switch (typeof value) {\n        case \"bigint\":\n        case \"boolean\":\n        case \"number\":\n        case \"string\":\n        case \"undefined\":\n          return value;\n        case \"object\":\n          return checkFormFieldValueStringCoercion(value), value;\n        default:\n          return \"\";\n      }\n    }\n    function isCheckable(elem) {\n      var type = elem.type;\n      return (\n        (elem = elem.nodeName) &&\n        \"input\" === elem.toLowerCase() &&\n        (\"checkbox\" === type || \"radio\" === type)\n      );\n    }\n    function trackValueOnNode(node) {\n      var valueField = isCheckable(node) ? \"checked\" : \"value\",\n        descriptor = Object.getOwnPropertyDescriptor(\n          node.constructor.prototype,\n          valueField\n        );\n      checkFormFieldValueStringCoercion(node[valueField]);\n      var currentValue = \"\" + node[valueField];\n      if (\n        !node.hasOwnProperty(valueField) &&\n        \"undefined\" !== typeof descriptor &&\n        \"function\" === typeof descriptor.get &&\n        \"function\" === typeof descriptor.set\n      ) {\n        var get = descriptor.get,\n          set = descriptor.set;\n        Object.defineProperty(node, valueField, {\n          configurable: !0,\n          get: function () {\n            return get.call(this);\n          },\n          set: function (value) {\n            checkFormFieldValueStringCoercion(value);\n            currentValue = \"\" + value;\n            set.call(this, value);\n          }\n        });\n        Object.defineProperty(node, valueField, {\n          enumerable: descriptor.enumerable\n        });\n        return {\n          getValue: function () {\n            return currentValue;\n          },\n          setValue: function (value) {\n            checkFormFieldValueStringCoercion(value);\n            currentValue = \"\" + value;\n          },\n          stopTracking: function () {\n            node._valueTracker = null;\n            delete node[valueField];\n          }\n        };\n      }\n    }\n    function track(node) {\n      node._valueTracker || (node._valueTracker = trackValueOnNode(node));\n    }\n    function updateValueIfChanged(node) {\n      if (!node) return !1;\n      var tracker = node._valueTracker;\n      if (!tracker) return !0;\n      var lastValue = tracker.getValue();\n      var value = \"\";\n      node &&\n        (value = isCheckable(node)\n          ? node.checked\n            ? \"true\"\n            : \"false\"\n          : node.value);\n      node = value;\n      return node !== lastValue ? (tracker.setValue(node), !0) : !1;\n    }\n    function getActiveElement(doc) {\n      doc = doc || (\"undefined\" !== typeof document ? document : void 0);\n      if (\"undefined\" === typeof doc) return null;\n      try {\n        return doc.activeElement || doc.body;\n      } catch (e) {\n        return doc.body;\n      }\n    }\n    function escapeSelectorAttributeValueInsideDoubleQuotes(value) {\n      return value.replace(\n        escapeSelectorAttributeValueInsideDoubleQuotesRegex,\n        function (ch) {\n          return \"\\\\\" + ch.charCodeAt(0).toString(16) + \" \";\n        }\n      );\n    }\n    function validateInputProps(element, props) {\n      void 0 === props.checked ||\n        void 0 === props.defaultChecked ||\n        didWarnCheckedDefaultChecked ||\n        (console.error(\n          \"%s contains an input of type %s with both checked and defaultChecked props. Input elements must be either controlled or uncontrolled (specify either the checked prop, or the defaultChecked prop, but not both). Decide between using a controlled or uncontrolled input element and remove one of these props. More info: https://react.dev/link/controlled-components\",\n          getCurrentFiberOwnerNameInDevOrNull() || \"A component\",\n          props.type\n        ),\n        (didWarnCheckedDefaultChecked = !0));\n      void 0 === props.value ||\n        void 0 === props.defaultValue ||\n        didWarnValueDefaultValue$1 ||\n        (console.error(\n          \"%s contains an input of type %s with both value and defaultValue props. Input elements must be either controlled or uncontrolled (specify either the value prop, or the defaultValue prop, but not both). Decide between using a controlled or uncontrolled input element and remove one of these props. More info: https://react.dev/link/controlled-components\",\n          getCurrentFiberOwnerNameInDevOrNull() || \"A component\",\n          props.type\n        ),\n        (didWarnValueDefaultValue$1 = !0));\n    }\n    function updateInput(\n      element,\n      value,\n      defaultValue,\n      lastDefaultValue,\n      checked,\n      defaultChecked,\n      type,\n      name\n    ) {\n      element.name = \"\";\n      null != type &&\n      \"function\" !== typeof type &&\n      \"symbol\" !== typeof type &&\n      \"boolean\" !== typeof type\n        ? (checkAttributeStringCoercion(type, \"type\"), (element.type = type))\n        : element.removeAttribute(\"type\");\n      if (null != value)\n        if (\"number\" === type) {\n          if ((0 === value && \"\" === element.value) || element.value != value)\n            element.value = \"\" + getToStringValue(value);\n        } else\n          element.value !== \"\" + getToStringValue(value) &&\n            (element.value = \"\" + getToStringValue(value));\n      else\n        (\"submit\" !== type && \"reset\" !== type) ||\n          element.removeAttribute(\"value\");\n      null != value\n        ? setDefaultValue(element, type, getToStringValue(value))\n        : null != defaultValue\n          ? setDefaultValue(element, type, getToStringValue(defaultValue))\n          : null != lastDefaultValue && element.removeAttribute(\"value\");\n      null == checked &&\n        null != defaultChecked &&\n        (element.defaultChecked = !!defaultChecked);\n      null != checked &&\n        (element.checked =\n          checked &&\n          \"function\" !== typeof checked &&\n          \"symbol\" !== typeof checked);\n      null != name &&\n      \"function\" !== typeof name &&\n      \"symbol\" !== typeof name &&\n      \"boolean\" !== typeof name\n        ? (checkAttributeStringCoercion(name, \"name\"),\n          (element.name = \"\" + getToStringValue(name)))\n        : element.removeAttribute(\"name\");\n    }\n    function initInput(\n      element,\n      value,\n      defaultValue,\n      checked,\n      defaultChecked,\n      type,\n      name,\n      isHydrating\n    ) {\n      null != type &&\n        \"function\" !== typeof type &&\n        \"symbol\" !== typeof type &&\n        \"boolean\" !== typeof type &&\n        (checkAttributeStringCoercion(type, \"type\"), (element.type = type));\n      if (null != value || null != defaultValue) {\n        if (\n          !(\n            (\"submit\" !== type && \"reset\" !== type) ||\n            (void 0 !== value && null !== value)\n          )\n        )\n          return;\n        defaultValue =\n          null != defaultValue ? \"\" + getToStringValue(defaultValue) : \"\";\n        value = null != value ? \"\" + getToStringValue(value) : defaultValue;\n        isHydrating || value === element.value || (element.value = value);\n        element.defaultValue = value;\n      }\n      checked = null != checked ? checked : defaultChecked;\n      checked =\n        \"function\" !== typeof checked &&\n        \"symbol\" !== typeof checked &&\n        !!checked;\n      element.checked = isHydrating ? element.checked : !!checked;\n      element.defaultChecked = !!checked;\n      null != name &&\n        \"function\" !== typeof name &&\n        \"symbol\" !== typeof name &&\n        \"boolean\" !== typeof name &&\n        (checkAttributeStringCoercion(name, \"name\"), (element.name = name));\n    }\n    function setDefaultValue(node, type, value) {\n      (\"number\" === type && getActiveElement(node.ownerDocument) === node) ||\n        node.defaultValue === \"\" + value ||\n        (node.defaultValue = \"\" + value);\n    }\n    function validateOptionProps(element, props) {\n      null == props.value &&\n        (\"object\" === typeof props.children && null !== props.children\n          ? React.Children.forEach(props.children, function (child) {\n              null == child ||\n                \"string\" === typeof child ||\n                \"number\" === typeof child ||\n                \"bigint\" === typeof child ||\n                didWarnInvalidChild ||\n                ((didWarnInvalidChild = !0),\n                console.error(\n                  \"Cannot infer the option value of complex children. Pass a `value` prop or use a plain string as children to <option>.\"\n                ));\n            })\n          : null == props.dangerouslySetInnerHTML ||\n            didWarnInvalidInnerHTML ||\n            ((didWarnInvalidInnerHTML = !0),\n            console.error(\n              \"Pass a `value` prop if you set dangerouslyInnerHTML so React knows which value should be selected.\"\n            )));\n      null == props.selected ||\n        didWarnSelectedSetOnOption ||\n        (console.error(\n          \"Use the `defaultValue` or `value` props on <select> instead of setting `selected` on <option>.\"\n        ),\n        (didWarnSelectedSetOnOption = !0));\n    }\n    function getDeclarationErrorAddendum() {\n      var ownerName = getCurrentFiberOwnerNameInDevOrNull();\n      return ownerName\n        ? \"\\n\\nCheck the render method of `\" + ownerName + \"`.\"\n        : \"\";\n    }\n    function updateOptions(node, multiple, propValue, setDefaultSelected) {\n      node = node.options;\n      if (multiple) {\n        multiple = {};\n        for (var i = 0; i < propValue.length; i++)\n          multiple[\"$\" + propValue[i]] = !0;\n        for (propValue = 0; propValue < node.length; propValue++)\n          (i = multiple.hasOwnProperty(\"$\" + node[propValue].value)),\n            node[propValue].selected !== i && (node[propValue].selected = i),\n            i && setDefaultSelected && (node[propValue].defaultSelected = !0);\n      } else {\n        propValue = \"\" + getToStringValue(propValue);\n        multiple = null;\n        for (i = 0; i < node.length; i++) {\n          if (node[i].value === propValue) {\n            node[i].selected = !0;\n            setDefaultSelected && (node[i].defaultSelected = !0);\n            return;\n          }\n          null !== multiple || node[i].disabled || (multiple = node[i]);\n        }\n        null !== multiple && (multiple.selected = !0);\n      }\n    }\n    function validateSelectProps(element, props) {\n      for (element = 0; element < valuePropNames.length; element++) {\n        var propName = valuePropNames[element];\n        if (null != props[propName]) {\n          var propNameIsArray = isArrayImpl(props[propName]);\n          props.multiple && !propNameIsArray\n            ? console.error(\n                \"The `%s` prop supplied to <select> must be an array if `multiple` is true.%s\",\n                propName,\n                getDeclarationErrorAddendum()\n              )\n            : !props.multiple &&\n              propNameIsArray &&\n              console.error(\n                \"The `%s` prop supplied to <select> must be a scalar value if `multiple` is false.%s\",\n                propName,\n                getDeclarationErrorAddendum()\n              );\n        }\n      }\n      void 0 === props.value ||\n        void 0 === props.defaultValue ||\n        didWarnValueDefaultValue ||\n        (console.error(\n          \"Select elements must be either controlled or uncontrolled (specify either the value prop, or the defaultValue prop, but not both). Decide between using a controlled or uncontrolled select element and remove one of these props. More info: https://react.dev/link/controlled-components\"\n        ),\n        (didWarnValueDefaultValue = !0));\n    }\n    function validateTextareaProps(element, props) {\n      void 0 === props.value ||\n        void 0 === props.defaultValue ||\n        didWarnValDefaultVal ||\n        (console.error(\n          \"%s contains a textarea with both value and defaultValue props. Textarea elements must be either controlled or uncontrolled (specify either the value prop, or the defaultValue prop, but not both). Decide between using a controlled or uncontrolled textarea and remove one of these props. More info: https://react.dev/link/controlled-components\",\n          getCurrentFiberOwnerNameInDevOrNull() || \"A component\"\n        ),\n        (didWarnValDefaultVal = !0));\n      null != props.children &&\n        null == props.value &&\n        console.error(\n          \"Use the `defaultValue` or `value` props instead of setting children on <textarea>.\"\n        );\n    }\n    function updateTextarea(element, value, defaultValue) {\n      if (\n        null != value &&\n        ((value = \"\" + getToStringValue(value)),\n        value !== element.value && (element.value = value),\n        null == defaultValue)\n      ) {\n        element.defaultValue !== value && (element.defaultValue = value);\n        return;\n      }\n      element.defaultValue =\n        null != defaultValue ? \"\" + getToStringValue(defaultValue) : \"\";\n    }\n    function initTextarea(element, value, defaultValue, children) {\n      if (null == value) {\n        if (null != children) {\n          if (null != defaultValue)\n            throw Error(\n              \"If you supply `defaultValue` on a <textarea>, do not pass children.\"\n            );\n          if (isArrayImpl(children)) {\n            if (1 < children.length)\n              throw Error(\"<textarea> can only have at most one child.\");\n            children = children[0];\n          }\n          defaultValue = children;\n        }\n        null == defaultValue && (defaultValue = \"\");\n        value = defaultValue;\n      }\n      defaultValue = getToStringValue(value);\n      element.defaultValue = defaultValue;\n      children = element.textContent;\n      children === defaultValue &&\n        \"\" !== children &&\n        null !== children &&\n        (element.value = children);\n    }\n    function findNotableNode(node, indent) {\n      return void 0 === node.serverProps &&\n        0 === node.serverTail.length &&\n        1 === node.children.length &&\n        3 < node.distanceFromLeaf &&\n        node.distanceFromLeaf > 15 - indent\n        ? findNotableNode(node.children[0], indent)\n        : node;\n    }\n    function indentation(indent) {\n      return \"  \" + \"  \".repeat(indent);\n    }\n    function added(indent) {\n      return \"+ \" + \"  \".repeat(indent);\n    }\n    function removed(indent) {\n      return \"- \" + \"  \".repeat(indent);\n    }\n    function describeFiberType(fiber) {\n      switch (fiber.tag) {\n        case 26:\n        case 27:\n        case 5:\n          return fiber.type;\n        case 16:\n          return \"Lazy\";\n        case 13:\n          return \"Suspense\";\n        case 19:\n          return \"SuspenseList\";\n        case 0:\n        case 15:\n          return (fiber = fiber.type), fiber.displayName || fiber.name || null;\n        case 11:\n          return (\n            (fiber = fiber.type.render), fiber.displayName || fiber.name || null\n          );\n        case 1:\n          return (fiber = fiber.type), fiber.displayName || fiber.name || null;\n        default:\n          return null;\n      }\n    }\n    function describeTextNode(content, maxLength) {\n      return needsEscaping.test(content)\n        ? ((content = JSON.stringify(content)),\n          content.length > maxLength - 2\n            ? 8 > maxLength\n              ? '{\"...\"}'\n              : \"{\" + content.slice(0, maxLength - 7) + '...\"}'\n            : \"{\" + content + \"}\")\n        : content.length > maxLength\n          ? 5 > maxLength\n            ? '{\"...\"}'\n            : content.slice(0, maxLength - 3) + \"...\"\n          : content;\n    }\n    function describeTextDiff(clientText, serverProps, indent) {\n      var maxLength = 120 - 2 * indent;\n      if (null === serverProps)\n        return added(indent) + describeTextNode(clientText, maxLength) + \"\\n\";\n      if (\"string\" === typeof serverProps) {\n        for (\n          var firstDiff = 0;\n          firstDiff < serverProps.length &&\n          firstDiff < clientText.length &&\n          serverProps.charCodeAt(firstDiff) ===\n            clientText.charCodeAt(firstDiff);\n          firstDiff++\n        );\n        firstDiff > maxLength - 8 &&\n          10 < firstDiff &&\n          ((clientText = \"...\" + clientText.slice(firstDiff - 8)),\n          (serverProps = \"...\" + serverProps.slice(firstDiff - 8)));\n        return (\n          added(indent) +\n          describeTextNode(clientText, maxLength) +\n          \"\\n\" +\n          removed(indent) +\n          describeTextNode(serverProps, maxLength) +\n          \"\\n\"\n        );\n      }\n      return (\n        indentation(indent) + describeTextNode(clientText, maxLength) + \"\\n\"\n      );\n    }\n    function objectName(object) {\n      return Object.prototype.toString\n        .call(object)\n        .replace(/^\\[object (.*)\\]$/, function (m, p0) {\n          return p0;\n        });\n    }\n    function describeValue(value, maxLength) {\n      switch (typeof value) {\n        case \"string\":\n          return (\n            (value = JSON.stringify(value)),\n            value.length > maxLength\n              ? 5 > maxLength\n                ? '\"...\"'\n                : value.slice(0, maxLength - 4) + '...\"'\n              : value\n          );\n        case \"object\":\n          if (null === value) return \"null\";\n          if (isArrayImpl(value)) return \"[...]\";\n          if (value.$$typeof === REACT_ELEMENT_TYPE)\n            return (maxLength = getComponentNameFromType(value.type))\n              ? \"<\" + maxLength + \">\"\n              : \"<...>\";\n          var name = objectName(value);\n          if (\"Object\" === name) {\n            name = \"\";\n            maxLength -= 2;\n            for (var propName in value)\n              if (value.hasOwnProperty(propName)) {\n                var jsonPropName = JSON.stringify(propName);\n                jsonPropName !== '\"' + propName + '\"' &&\n                  (propName = jsonPropName);\n                maxLength -= propName.length - 2;\n                jsonPropName = describeValue(\n                  value[propName],\n                  15 > maxLength ? maxLength : 15\n                );\n                maxLength -= jsonPropName.length;\n                if (0 > maxLength) {\n                  name += \"\" === name ? \"...\" : \", ...\";\n                  break;\n                }\n                name +=\n                  (\"\" === name ? \"\" : \",\") + propName + \":\" + jsonPropName;\n              }\n            return \"{\" + name + \"}\";\n          }\n          return name;\n        case \"function\":\n          return (maxLength = value.displayName || value.name)\n            ? \"function \" + maxLength\n            : \"function\";\n        default:\n          return String(value);\n      }\n    }\n    function describePropValue(value, maxLength) {\n      return \"string\" !== typeof value || needsEscaping.test(value)\n        ? \"{\" + describeValue(value, maxLength - 2) + \"}\"\n        : value.length > maxLength - 2\n          ? 5 > maxLength\n            ? '\"...\"'\n            : '\"' + value.slice(0, maxLength - 5) + '...\"'\n          : '\"' + value + '\"';\n    }\n    function describeExpandedElement(type, props, rowPrefix) {\n      var remainingRowLength = 120 - rowPrefix.length - type.length,\n        properties = [],\n        propName;\n      for (propName in props)\n        if (props.hasOwnProperty(propName) && \"children\" !== propName) {\n          var propValue = describePropValue(\n            props[propName],\n            120 - rowPrefix.length - propName.length - 1\n          );\n          remainingRowLength -= propName.length + propValue.length + 2;\n          properties.push(propName + \"=\" + propValue);\n        }\n      return 0 === properties.length\n        ? rowPrefix + \"<\" + type + \">\\n\"\n        : 0 < remainingRowLength\n          ? rowPrefix + \"<\" + type + \" \" + properties.join(\" \") + \">\\n\"\n          : rowPrefix +\n            \"<\" +\n            type +\n            \"\\n\" +\n            rowPrefix +\n            \"  \" +\n            properties.join(\"\\n\" + rowPrefix + \"  \") +\n            \"\\n\" +\n            rowPrefix +\n            \">\\n\";\n    }\n    function describePropertiesDiff(clientObject, serverObject, indent) {\n      var properties = \"\",\n        remainingServerProperties = assign({}, serverObject),\n        propName;\n      for (propName in clientObject)\n        if (clientObject.hasOwnProperty(propName)) {\n          delete remainingServerProperties[propName];\n          var maxLength = 120 - 2 * indent - propName.length - 2,\n            clientPropValue = describeValue(clientObject[propName], maxLength);\n          serverObject.hasOwnProperty(propName)\n            ? ((maxLength = describeValue(serverObject[propName], maxLength)),\n              (properties +=\n                added(indent) + propName + \": \" + clientPropValue + \"\\n\"),\n              (properties +=\n                removed(indent) + propName + \": \" + maxLength + \"\\n\"))\n            : (properties +=\n                added(indent) + propName + \": \" + clientPropValue + \"\\n\");\n        }\n      for (var _propName in remainingServerProperties)\n        remainingServerProperties.hasOwnProperty(_propName) &&\n          ((clientObject = describeValue(\n            remainingServerProperties[_propName],\n            120 - 2 * indent - _propName.length - 2\n          )),\n          (properties +=\n            removed(indent) + _propName + \": \" + clientObject + \"\\n\"));\n      return properties;\n    }\n    function describeElementDiff(type, clientProps, serverProps, indent) {\n      var content = \"\",\n        serverPropNames = new Map();\n      for (propName$jscomp$0 in serverProps)\n        serverProps.hasOwnProperty(propName$jscomp$0) &&\n          serverPropNames.set(\n            propName$jscomp$0.toLowerCase(),\n            propName$jscomp$0\n          );\n      if (1 === serverPropNames.size && serverPropNames.has(\"children\"))\n        content += describeExpandedElement(\n          type,\n          clientProps,\n          indentation(indent)\n        );\n      else {\n        for (var _propName2 in clientProps)\n          if (\n            clientProps.hasOwnProperty(_propName2) &&\n            \"children\" !== _propName2\n          ) {\n            var maxLength$jscomp$0 =\n                120 - 2 * (indent + 1) - _propName2.length - 1,\n              serverPropName = serverPropNames.get(_propName2.toLowerCase());\n            if (void 0 !== serverPropName) {\n              serverPropNames.delete(_propName2.toLowerCase());\n              var propName$jscomp$0 = clientProps[_propName2];\n              serverPropName = serverProps[serverPropName];\n              var clientPropValue = describePropValue(\n                propName$jscomp$0,\n                maxLength$jscomp$0\n              );\n              maxLength$jscomp$0 = describePropValue(\n                serverPropName,\n                maxLength$jscomp$0\n              );\n              \"object\" === typeof propName$jscomp$0 &&\n              null !== propName$jscomp$0 &&\n              \"object\" === typeof serverPropName &&\n              null !== serverPropName &&\n              \"Object\" === objectName(propName$jscomp$0) &&\n              \"Object\" === objectName(serverPropName) &&\n              (2 < Object.keys(propName$jscomp$0).length ||\n                2 < Object.keys(serverPropName).length ||\n                -1 < clientPropValue.indexOf(\"...\") ||\n                -1 < maxLength$jscomp$0.indexOf(\"...\"))\n                ? (content +=\n                    indentation(indent + 1) +\n                    _propName2 +\n                    \"={{\\n\" +\n                    describePropertiesDiff(\n                      propName$jscomp$0,\n                      serverPropName,\n                      indent + 2\n                    ) +\n                    indentation(indent + 1) +\n                    \"}}\\n\")\n                : ((content +=\n                    added(indent + 1) +\n                    _propName2 +\n                    \"=\" +\n                    clientPropValue +\n                    \"\\n\"),\n                  (content +=\n                    removed(indent + 1) +\n                    _propName2 +\n                    \"=\" +\n                    maxLength$jscomp$0 +\n                    \"\\n\"));\n            } else\n              content +=\n                indentation(indent + 1) +\n                _propName2 +\n                \"=\" +\n                describePropValue(clientProps[_propName2], maxLength$jscomp$0) +\n                \"\\n\";\n          }\n        serverPropNames.forEach(function (propName) {\n          if (\"children\" !== propName) {\n            var maxLength = 120 - 2 * (indent + 1) - propName.length - 1;\n            content +=\n              removed(indent + 1) +\n              propName +\n              \"=\" +\n              describePropValue(serverProps[propName], maxLength) +\n              \"\\n\";\n          }\n        });\n        content =\n          \"\" === content\n            ? indentation(indent) + \"<\" + type + \">\\n\"\n            : indentation(indent) +\n              \"<\" +\n              type +\n              \"\\n\" +\n              content +\n              indentation(indent) +\n              \">\\n\";\n      }\n      type = serverProps.children;\n      clientProps = clientProps.children;\n      if (\n        \"string\" === typeof type ||\n        \"number\" === typeof type ||\n        \"bigint\" === typeof type\n      ) {\n        serverPropNames = \"\";\n        if (\n          \"string\" === typeof clientProps ||\n          \"number\" === typeof clientProps ||\n          \"bigint\" === typeof clientProps\n        )\n          serverPropNames = \"\" + clientProps;\n        content += describeTextDiff(serverPropNames, \"\" + type, indent + 1);\n      } else if (\n        \"string\" === typeof clientProps ||\n        \"number\" === typeof clientProps ||\n        \"bigint\" === typeof clientProps\n      )\n        content =\n          null == type\n            ? content + describeTextDiff(\"\" + clientProps, null, indent + 1)\n            : content + describeTextDiff(\"\" + clientProps, void 0, indent + 1);\n      return content;\n    }\n    function describeSiblingFiber(fiber, indent) {\n      var type = describeFiberType(fiber);\n      if (null === type) {\n        type = \"\";\n        for (fiber = fiber.child; fiber; )\n          (type += describeSiblingFiber(fiber, indent)),\n            (fiber = fiber.sibling);\n        return type;\n      }\n      return indentation(indent) + \"<\" + type + \">\\n\";\n    }\n    function describeNode(node, indent) {\n      var skipToNode = findNotableNode(node, indent);\n      if (\n        skipToNode !== node &&\n        (1 !== node.children.length || node.children[0] !== skipToNode)\n      )\n        return (\n          indentation(indent) + \"...\\n\" + describeNode(skipToNode, indent + 1)\n        );\n      skipToNode = \"\";\n      var debugInfo = node.fiber._debugInfo;\n      if (debugInfo)\n        for (var i = 0; i < debugInfo.length; i++) {\n          var serverComponentName = debugInfo[i].name;\n          \"string\" === typeof serverComponentName &&\n            ((skipToNode +=\n              indentation(indent) + \"<\" + serverComponentName + \">\\n\"),\n            indent++);\n        }\n      debugInfo = \"\";\n      i = node.fiber.pendingProps;\n      if (6 === node.fiber.tag)\n        (debugInfo = describeTextDiff(i, node.serverProps, indent)), indent++;\n      else if (\n        ((serverComponentName = describeFiberType(node.fiber)),\n        null !== serverComponentName)\n      )\n        if (void 0 === node.serverProps) {\n          debugInfo = indent;\n          var maxLength = 120 - 2 * debugInfo - serverComponentName.length - 2,\n            content = \"\";\n          for (propName in i)\n            if (i.hasOwnProperty(propName) && \"children\" !== propName) {\n              var propValue = describePropValue(i[propName], 15);\n              maxLength -= propName.length + propValue.length + 2;\n              if (0 > maxLength) {\n                content += \" ...\";\n                break;\n              }\n              content += \" \" + propName + \"=\" + propValue;\n            }\n          debugInfo =\n            indentation(debugInfo) +\n            \"<\" +\n            serverComponentName +\n            content +\n            \">\\n\";\n          indent++;\n        } else\n          null === node.serverProps\n            ? ((debugInfo = describeExpandedElement(\n                serverComponentName,\n                i,\n                added(indent)\n              )),\n              indent++)\n            : \"string\" === typeof node.serverProps\n              ? console.error(\n                  \"Should not have matched a non HostText fiber to a Text node. This is a bug in React.\"\n                )\n              : ((debugInfo = describeElementDiff(\n                  serverComponentName,\n                  i,\n                  node.serverProps,\n                  indent\n                )),\n                indent++);\n      var propName = \"\";\n      i = node.fiber.child;\n      for (\n        serverComponentName = 0;\n        i && serverComponentName < node.children.length;\n\n      )\n        (maxLength = node.children[serverComponentName]),\n          maxLength.fiber === i\n            ? ((propName += describeNode(maxLength, indent)),\n              serverComponentName++)\n            : (propName += describeSiblingFiber(i, indent)),\n          (i = i.sibling);\n      i &&\n        0 < node.children.length &&\n        (propName += indentation(indent) + \"...\\n\");\n      i = node.serverTail;\n      null === node.serverProps && indent--;\n      for (node = 0; node < i.length; node++)\n        (serverComponentName = i[node]),\n          (propName =\n            \"string\" === typeof serverComponentName\n              ? propName +\n                (removed(indent) +\n                  describeTextNode(serverComponentName, 120 - 2 * indent) +\n                  \"\\n\")\n              : propName +\n                describeExpandedElement(\n                  serverComponentName.type,\n                  serverComponentName.props,\n                  removed(indent)\n                ));\n      return skipToNode + debugInfo + propName;\n    }\n    function describeDiff(rootNode) {\n      try {\n        return \"\\n\\n\" + describeNode(rootNode, 0);\n      } catch (x) {\n        return \"\";\n      }\n    }\n    function describeAncestors(ancestor, child, props) {\n      for (var fiber = child, node = null, distanceFromLeaf = 0; fiber; )\n        fiber === ancestor && (distanceFromLeaf = 0),\n          (node = {\n            fiber: fiber,\n            children: null !== node ? [node] : [],\n            serverProps:\n              fiber === child ? props : fiber === ancestor ? null : void 0,\n            serverTail: [],\n            distanceFromLeaf: distanceFromLeaf\n          }),\n          distanceFromLeaf++,\n          (fiber = fiber.return);\n      return null !== node ? describeDiff(node).replaceAll(/^[+-]/gm, \">\") : \"\";\n    }\n    function updatedAncestorInfoDev(oldInfo, tag) {\n      var ancestorInfo = assign({}, oldInfo || emptyAncestorInfoDev),\n        info = { tag: tag };\n      -1 !== inScopeTags.indexOf(tag) &&\n        ((ancestorInfo.aTagInScope = null),\n        (ancestorInfo.buttonTagInScope = null),\n        (ancestorInfo.nobrTagInScope = null));\n      -1 !== buttonScopeTags.indexOf(tag) &&\n        (ancestorInfo.pTagInButtonScope = null);\n      -1 !== specialTags.indexOf(tag) &&\n        \"address\" !== tag &&\n        \"div\" !== tag &&\n        \"p\" !== tag &&\n        ((ancestorInfo.listItemTagAutoclosing = null),\n        (ancestorInfo.dlItemTagAutoclosing = null));\n      ancestorInfo.current = info;\n      \"form\" === tag && (ancestorInfo.formTag = info);\n      \"a\" === tag && (ancestorInfo.aTagInScope = info);\n      \"button\" === tag && (ancestorInfo.buttonTagInScope = info);\n      \"nobr\" === tag && (ancestorInfo.nobrTagInScope = info);\n      \"p\" === tag && (ancestorInfo.pTagInButtonScope = info);\n      \"li\" === tag && (ancestorInfo.listItemTagAutoclosing = info);\n      if (\"dd\" === tag || \"dt\" === tag)\n        ancestorInfo.dlItemTagAutoclosing = info;\n      \"#document\" === tag || \"html\" === tag\n        ? (ancestorInfo.containerTagInScope = null)\n        : ancestorInfo.containerTagInScope ||\n          (ancestorInfo.containerTagInScope = info);\n      null !== oldInfo ||\n      (\"#document\" !== tag && \"html\" !== tag && \"body\" !== tag)\n        ? !0 === ancestorInfo.implicitRootScope &&\n          (ancestorInfo.implicitRootScope = !1)\n        : (ancestorInfo.implicitRootScope = !0);\n      return ancestorInfo;\n    }\n    function isTagValidWithParent(tag, parentTag, implicitRootScope) {\n      switch (parentTag) {\n        case \"select\":\n          return (\n            \"hr\" === tag ||\n            \"option\" === tag ||\n            \"optgroup\" === tag ||\n            \"script\" === tag ||\n            \"template\" === tag ||\n            \"#text\" === tag\n          );\n        case \"optgroup\":\n          return \"option\" === tag || \"#text\" === tag;\n        case \"option\":\n          return \"#text\" === tag;\n        case \"tr\":\n          return (\n            \"th\" === tag ||\n            \"td\" === tag ||\n            \"style\" === tag ||\n            \"script\" === tag ||\n            \"template\" === tag\n          );\n        case \"tbody\":\n        case \"thead\":\n        case \"tfoot\":\n          return (\n            \"tr\" === tag ||\n            \"style\" === tag ||\n            \"script\" === tag ||\n            \"template\" === tag\n          );\n        case \"colgroup\":\n          return \"col\" === tag || \"template\" === tag;\n        case \"table\":\n          return (\n            \"caption\" === tag ||\n            \"colgroup\" === tag ||\n            \"tbody\" === tag ||\n            \"tfoot\" === tag ||\n            \"thead\" === tag ||\n            \"style\" === tag ||\n            \"script\" === tag ||\n            \"template\" === tag\n          );\n        case \"head\":\n          return (\n            \"base\" === tag ||\n            \"basefont\" === tag ||\n            \"bgsound\" === tag ||\n            \"link\" === tag ||\n            \"meta\" === tag ||\n            \"title\" === tag ||\n            \"noscript\" === tag ||\n            \"noframes\" === tag ||\n            \"style\" === tag ||\n            \"script\" === tag ||\n            \"template\" === tag\n          );\n        case \"html\":\n          if (implicitRootScope) break;\n          return \"head\" === tag || \"body\" === tag || \"frameset\" === tag;\n        case \"frameset\":\n          return \"frame\" === tag;\n        case \"#document\":\n          if (!implicitRootScope) return \"html\" === tag;\n      }\n      switch (tag) {\n        case \"h1\":\n        case \"h2\":\n        case \"h3\":\n        case \"h4\":\n        case \"h5\":\n        case \"h6\":\n          return (\n            \"h1\" !== parentTag &&\n            \"h2\" !== parentTag &&\n            \"h3\" !== parentTag &&\n            \"h4\" !== parentTag &&\n            \"h5\" !== parentTag &&\n            \"h6\" !== parentTag\n          );\n        case \"rp\":\n        case \"rt\":\n          return -1 === impliedEndTags.indexOf(parentTag);\n        case \"caption\":\n        case \"col\":\n        case \"colgroup\":\n        case \"frameset\":\n        case \"frame\":\n        case \"tbody\":\n        case \"td\":\n        case \"tfoot\":\n        case \"th\":\n        case \"thead\":\n        case \"tr\":\n          return null == parentTag;\n        case \"head\":\n          return implicitRootScope || null === parentTag;\n        case \"html\":\n          return (\n            (implicitRootScope && \"#document\" === parentTag) ||\n            null === parentTag\n          );\n        case \"body\":\n          return (\n            (implicitRootScope &&\n              (\"#document\" === parentTag || \"html\" === parentTag)) ||\n            null === parentTag\n          );\n      }\n      return !0;\n    }\n    function findInvalidAncestorForTag(tag, ancestorInfo) {\n      switch (tag) {\n        case \"address\":\n        case \"article\":\n        case \"aside\":\n        case \"blockquote\":\n        case \"center\":\n        case \"details\":\n        case \"dialog\":\n        case \"dir\":\n        case \"div\":\n        case \"dl\":\n        case \"fieldset\":\n        case \"figcaption\":\n        case \"figure\":\n        case \"footer\":\n        case \"header\":\n        case \"hgroup\":\n        case \"main\":\n        case \"menu\":\n        case \"nav\":\n        case \"ol\":\n        case \"p\":\n        case \"section\":\n        case \"summary\":\n        case \"ul\":\n        case \"pre\":\n        case \"listing\":\n        case \"table\":\n        case \"hr\":\n        case \"xmp\":\n        case \"h1\":\n        case \"h2\":\n        case \"h3\":\n        case \"h4\":\n        case \"h5\":\n        case \"h6\":\n          return ancestorInfo.pTagInButtonScope;\n        case \"form\":\n          return ancestorInfo.formTag || ancestorInfo.pTagInButtonScope;\n        case \"li\":\n          return ancestorInfo.listItemTagAutoclosing;\n        case \"dd\":\n        case \"dt\":\n          return ancestorInfo.dlItemTagAutoclosing;\n        case \"button\":\n          return ancestorInfo.buttonTagInScope;\n        case \"a\":\n          return ancestorInfo.aTagInScope;\n        case \"nobr\":\n          return ancestorInfo.nobrTagInScope;\n      }\n      return null;\n    }\n    function findAncestor(parent, tagName) {\n      for (; parent; ) {\n        switch (parent.tag) {\n          case 5:\n          case 26:\n          case 27:\n            if (parent.type === tagName) return parent;\n        }\n        parent = parent.return;\n      }\n      return null;\n    }\n    function validateDOMNesting(childTag, ancestorInfo) {\n      ancestorInfo = ancestorInfo || emptyAncestorInfoDev;\n      var parentInfo = ancestorInfo.current;\n      ancestorInfo = (parentInfo = isTagValidWithParent(\n        childTag,\n        parentInfo && parentInfo.tag,\n        ancestorInfo.implicitRootScope\n      )\n        ? null\n        : parentInfo)\n        ? null\n        : findInvalidAncestorForTag(childTag, ancestorInfo);\n      ancestorInfo = parentInfo || ancestorInfo;\n      if (!ancestorInfo) return !0;\n      var ancestorTag = ancestorInfo.tag;\n      ancestorInfo = String(!!parentInfo) + \"|\" + childTag + \"|\" + ancestorTag;\n      if (didWarn[ancestorInfo]) return !1;\n      didWarn[ancestorInfo] = !0;\n      var ancestor = (ancestorInfo = current)\n          ? findAncestor(ancestorInfo.return, ancestorTag)\n          : null,\n        ancestorDescription =\n          null !== ancestorInfo && null !== ancestor\n            ? describeAncestors(ancestor, ancestorInfo, null)\n            : \"\",\n        tagDisplayName = \"<\" + childTag + \">\";\n      parentInfo\n        ? ((parentInfo = \"\"),\n          \"table\" === ancestorTag &&\n            \"tr\" === childTag &&\n            (parentInfo +=\n              \" Add a <tbody>, <thead> or <tfoot> to your code to match the DOM tree generated by the browser.\"),\n          console.error(\n            \"In HTML, %s cannot be a child of <%s>.%s\\nThis will cause a hydration error.%s\",\n            tagDisplayName,\n            ancestorTag,\n            parentInfo,\n            ancestorDescription\n          ))\n        : console.error(\n            \"In HTML, %s cannot be a descendant of <%s>.\\nThis will cause a hydration error.%s\",\n            tagDisplayName,\n            ancestorTag,\n            ancestorDescription\n          );\n      ancestorInfo &&\n        ((childTag = ancestorInfo.return),\n        null === ancestor ||\n          null === childTag ||\n          (ancestor === childTag &&\n            childTag._debugOwner === ancestorInfo._debugOwner) ||\n          runWithFiberInDEV(ancestor, function () {\n            console.error(\n              \"<%s> cannot contain a nested %s.\\nSee this log for the ancestor stack trace.\",\n              ancestorTag,\n              tagDisplayName\n            );\n          }));\n      return !1;\n    }\n    function validateTextNesting(childText, parentTag, implicitRootScope) {\n      if (implicitRootScope || isTagValidWithParent(\"#text\", parentTag, !1))\n        return !0;\n      implicitRootScope = \"#text|\" + parentTag;\n      if (didWarn[implicitRootScope]) return !1;\n      didWarn[implicitRootScope] = !0;\n      var ancestor = (implicitRootScope = current)\n        ? findAncestor(implicitRootScope, parentTag)\n        : null;\n      implicitRootScope =\n        null !== implicitRootScope && null !== ancestor\n          ? describeAncestors(\n              ancestor,\n              implicitRootScope,\n              6 !== implicitRootScope.tag ? { children: null } : null\n            )\n          : \"\";\n      /\\S/.test(childText)\n        ? console.error(\n            \"In HTML, text nodes cannot be a child of <%s>.\\nThis will cause a hydration error.%s\",\n            parentTag,\n            implicitRootScope\n          )\n        : console.error(\n            \"In HTML, whitespace text nodes cannot be a child of <%s>. Make sure you don't have any extra whitespace between tags on each line of your source code.\\nThis will cause a hydration error.%s\",\n            parentTag,\n            implicitRootScope\n          );\n      return !1;\n    }\n    function setTextContent(node, text) {\n      if (text) {\n        var firstChild = node.firstChild;\n        if (\n          firstChild &&\n          firstChild === node.lastChild &&\n          3 === firstChild.nodeType\n        ) {\n          firstChild.nodeValue = text;\n          return;\n        }\n      }\n      node.textContent = text;\n    }\n    function camelize(string) {\n      return string.replace(hyphenPattern, function (_, character) {\n        return character.toUpperCase();\n      });\n    }\n    function setValueForStyle(style, styleName, value) {\n      var isCustomProperty = 0 === styleName.indexOf(\"--\");\n      isCustomProperty ||\n        (-1 < styleName.indexOf(\"-\")\n          ? (warnedStyleNames.hasOwnProperty(styleName) &&\n              warnedStyleNames[styleName]) ||\n            ((warnedStyleNames[styleName] = !0),\n            console.error(\n              \"Unsupported style property %s. Did you mean %s?\",\n              styleName,\n              camelize(styleName.replace(msPattern, \"ms-\"))\n            ))\n          : badVendoredStyleNamePattern.test(styleName)\n            ? (warnedStyleNames.hasOwnProperty(styleName) &&\n                warnedStyleNames[styleName]) ||\n              ((warnedStyleNames[styleName] = !0),\n              console.error(\n                \"Unsupported vendor-prefixed style property %s. Did you mean %s?\",\n                styleName,\n                styleName.charAt(0).toUpperCase() + styleName.slice(1)\n              ))\n            : !badStyleValueWithSemicolonPattern.test(value) ||\n              (warnedStyleValues.hasOwnProperty(value) &&\n                warnedStyleValues[value]) ||\n              ((warnedStyleValues[value] = !0),\n              console.error(\n                'Style property values shouldn\\'t contain a semicolon. Try \"%s: %s\" instead.',\n                styleName,\n                value.replace(badStyleValueWithSemicolonPattern, \"\")\n              )),\n        \"number\" === typeof value &&\n          (isNaN(value)\n            ? warnedForNaNValue ||\n              ((warnedForNaNValue = !0),\n              console.error(\n                \"`NaN` is an invalid value for the `%s` css style property.\",\n                styleName\n              ))\n            : isFinite(value) ||\n              warnedForInfinityValue ||\n              ((warnedForInfinityValue = !0),\n              console.error(\n                \"`Infinity` is an invalid value for the `%s` css style property.\",\n                styleName\n              ))));\n      null == value || \"boolean\" === typeof value || \"\" === value\n        ? isCustomProperty\n          ? style.setProperty(styleName, \"\")\n          : \"float\" === styleName\n            ? (style.cssFloat = \"\")\n            : (style[styleName] = \"\")\n        : isCustomProperty\n          ? style.setProperty(styleName, value)\n          : \"number\" !== typeof value ||\n              0 === value ||\n              unitlessNumbers.has(styleName)\n            ? \"float\" === styleName\n              ? (style.cssFloat = value)\n              : (checkCSSPropertyStringCoercion(value, styleName),\n                (style[styleName] = (\"\" + value).trim()))\n            : (style[styleName] = value + \"px\");\n    }\n    function setValueForStyles(node, styles, prevStyles) {\n      if (null != styles && \"object\" !== typeof styles)\n        throw Error(\n          \"The `style` prop expects a mapping from style properties to values, not a string. For example, style={{marginRight: spacing + 'em'}} when using JSX.\"\n        );\n      styles && Object.freeze(styles);\n      node = node.style;\n      if (null != prevStyles) {\n        if (styles) {\n          var expandedUpdates = {};\n          if (prevStyles)\n            for (var key in prevStyles)\n              if (prevStyles.hasOwnProperty(key) && !styles.hasOwnProperty(key))\n                for (\n                  var longhands = shorthandToLonghand[key] || [key], i = 0;\n                  i < longhands.length;\n                  i++\n                )\n                  expandedUpdates[longhands[i]] = key;\n          for (var _key in styles)\n            if (\n              styles.hasOwnProperty(_key) &&\n              (!prevStyles || prevStyles[_key] !== styles[_key])\n            )\n              for (\n                key = shorthandToLonghand[_key] || [_key], longhands = 0;\n                longhands < key.length;\n                longhands++\n              )\n                expandedUpdates[key[longhands]] = _key;\n          _key = {};\n          for (var key$jscomp$0 in styles)\n            for (\n              key = shorthandToLonghand[key$jscomp$0] || [key$jscomp$0],\n                longhands = 0;\n              longhands < key.length;\n              longhands++\n            )\n              _key[key[longhands]] = key$jscomp$0;\n          key$jscomp$0 = {};\n          for (var _key2 in expandedUpdates)\n            if (\n              ((key = expandedUpdates[_key2]),\n              (longhands = _key[_key2]) &&\n                key !== longhands &&\n                ((i = key + \",\" + longhands), !key$jscomp$0[i]))\n            ) {\n              key$jscomp$0[i] = !0;\n              i = console;\n              var value = styles[key];\n              i.error.call(\n                i,\n                \"%s a style property during rerender (%s) when a conflicting property is set (%s) can lead to styling bugs. To avoid this, don't mix shorthand and non-shorthand properties for the same value; instead, replace the shorthand with separate values.\",\n                null == value || \"boolean\" === typeof value || \"\" === value\n                  ? \"Removing\"\n                  : \"Updating\",\n                key,\n                longhands\n              );\n            }\n        }\n        for (var styleName in prevStyles)\n          !prevStyles.hasOwnProperty(styleName) ||\n            (null != styles && styles.hasOwnProperty(styleName)) ||\n            (0 === styleName.indexOf(\"--\")\n              ? node.setProperty(styleName, \"\")\n              : \"float\" === styleName\n                ? (node.cssFloat = \"\")\n                : (node[styleName] = \"\"));\n        for (var _styleName in styles)\n          (_key2 = styles[_styleName]),\n            styles.hasOwnProperty(_styleName) &&\n              prevStyles[_styleName] !== _key2 &&\n              setValueForStyle(node, _styleName, _key2);\n      } else\n        for (expandedUpdates in styles)\n          styles.hasOwnProperty(expandedUpdates) &&\n            setValueForStyle(node, expandedUpdates, styles[expandedUpdates]);\n    }\n    function isCustomElement(tagName) {\n      if (-1 === tagName.indexOf(\"-\")) return !1;\n      switch (tagName) {\n        case \"annotation-xml\":\n        case \"color-profile\":\n        case \"font-face\":\n        case \"font-face-src\":\n        case \"font-face-uri\":\n        case \"font-face-format\":\n        case \"font-face-name\":\n        case \"missing-glyph\":\n          return !1;\n        default:\n          return !0;\n      }\n    }\n    function getAttributeAlias(name) {\n      return aliases.get(name) || name;\n    }\n    function validateProperty$1(tagName, name) {\n      if (\n        hasOwnProperty.call(warnedProperties$1, name) &&\n        warnedProperties$1[name]\n      )\n        return !0;\n      if (rARIACamel$1.test(name)) {\n        tagName = \"aria-\" + name.slice(4).toLowerCase();\n        tagName = ariaProperties.hasOwnProperty(tagName) ? tagName : null;\n        if (null == tagName)\n          return (\n            console.error(\n              \"Invalid ARIA attribute `%s`. ARIA attributes follow the pattern aria-* and must be lowercase.\",\n              name\n            ),\n            (warnedProperties$1[name] = !0)\n          );\n        if (name !== tagName)\n          return (\n            console.error(\n              \"Invalid ARIA attribute `%s`. Did you mean `%s`?\",\n              name,\n              tagName\n            ),\n            (warnedProperties$1[name] = !0)\n          );\n      }\n      if (rARIA$1.test(name)) {\n        tagName = name.toLowerCase();\n        tagName = ariaProperties.hasOwnProperty(tagName) ? tagName : null;\n        if (null == tagName) return (warnedProperties$1[name] = !0), !1;\n        name !== tagName &&\n          (console.error(\n            \"Unknown ARIA attribute `%s`. Did you mean `%s`?\",\n            name,\n            tagName\n          ),\n          (warnedProperties$1[name] = !0));\n      }\n      return !0;\n    }\n    function validateProperties$2(type, props) {\n      var invalidProps = [],\n        key;\n      for (key in props)\n        validateProperty$1(type, key) || invalidProps.push(key);\n      props = invalidProps\n        .map(function (prop) {\n          return \"`\" + prop + \"`\";\n        })\n        .join(\", \");\n      1 === invalidProps.length\n        ? console.error(\n            \"Invalid aria prop %s on <%s> tag. For details, see https://react.dev/link/invalid-aria-props\",\n            props,\n            type\n          )\n        : 1 < invalidProps.length &&\n          console.error(\n            \"Invalid aria props %s on <%s> tag. For details, see https://react.dev/link/invalid-aria-props\",\n            props,\n            type\n          );\n    }\n    function validateProperty(tagName, name, value, eventRegistry) {\n      if (hasOwnProperty.call(warnedProperties, name) && warnedProperties[name])\n        return !0;\n      var lowerCasedName = name.toLowerCase();\n      if (\"onfocusin\" === lowerCasedName || \"onfocusout\" === lowerCasedName)\n        return (\n          console.error(\n            \"React uses onFocus and onBlur instead of onFocusIn and onFocusOut. All React events are normalized to bubble, so onFocusIn and onFocusOut are not needed/supported by React.\"\n          ),\n          (warnedProperties[name] = !0)\n        );\n      if (\n        \"function\" === typeof value &&\n        ((\"form\" === tagName && \"action\" === name) ||\n          (\"input\" === tagName && \"formAction\" === name) ||\n          (\"button\" === tagName && \"formAction\" === name))\n      )\n        return !0;\n      if (null != eventRegistry) {\n        tagName = eventRegistry.possibleRegistrationNames;\n        if (eventRegistry.registrationNameDependencies.hasOwnProperty(name))\n          return !0;\n        eventRegistry = tagName.hasOwnProperty(lowerCasedName)\n          ? tagName[lowerCasedName]\n          : null;\n        if (null != eventRegistry)\n          return (\n            console.error(\n              \"Invalid event handler property `%s`. Did you mean `%s`?\",\n              name,\n              eventRegistry\n            ),\n            (warnedProperties[name] = !0)\n          );\n        if (EVENT_NAME_REGEX.test(name))\n          return (\n            console.error(\n              \"Unknown event handler property `%s`. It will be ignored.\",\n              name\n            ),\n            (warnedProperties[name] = !0)\n          );\n      } else if (EVENT_NAME_REGEX.test(name))\n        return (\n          INVALID_EVENT_NAME_REGEX.test(name) &&\n            console.error(\n              \"Invalid event handler property `%s`. React events use the camelCase naming convention, for example `onClick`.\",\n              name\n            ),\n          (warnedProperties[name] = !0)\n        );\n      if (rARIA.test(name) || rARIACamel.test(name)) return !0;\n      if (\"innerhtml\" === lowerCasedName)\n        return (\n          console.error(\n            \"Directly setting property `innerHTML` is not permitted. For more information, lookup documentation on `dangerouslySetInnerHTML`.\"\n          ),\n          (warnedProperties[name] = !0)\n        );\n      if (\"aria\" === lowerCasedName)\n        return (\n          console.error(\n            \"The `aria` attribute is reserved for future use in React. Pass individual `aria-` attributes instead.\"\n          ),\n          (warnedProperties[name] = !0)\n        );\n      if (\n        \"is\" === lowerCasedName &&\n        null !== value &&\n        void 0 !== value &&\n        \"string\" !== typeof value\n      )\n        return (\n          console.error(\n            \"Received a `%s` for a string attribute `is`. If this is expected, cast the value to a string.\",\n            typeof value\n          ),\n          (warnedProperties[name] = !0)\n        );\n      if (\"number\" === typeof value && isNaN(value))\n        return (\n          console.error(\n            \"Received NaN for the `%s` attribute. If this is expected, cast the value to a string.\",\n            name\n          ),\n          (warnedProperties[name] = !0)\n        );\n      if (possibleStandardNames.hasOwnProperty(lowerCasedName)) {\n        if (\n          ((lowerCasedName = possibleStandardNames[lowerCasedName]),\n          lowerCasedName !== name)\n        )\n          return (\n            console.error(\n              \"Invalid DOM property `%s`. Did you mean `%s`?\",\n              name,\n              lowerCasedName\n            ),\n            (warnedProperties[name] = !0)\n          );\n      } else if (name !== lowerCasedName)\n        return (\n          console.error(\n            \"React does not recognize the `%s` prop on a DOM element. If you intentionally want it to appear in the DOM as a custom attribute, spell it as lowercase `%s` instead. If you accidentally passed it from a parent component, remove it from the DOM element.\",\n            name,\n            lowerCasedName\n          ),\n          (warnedProperties[name] = !0)\n        );\n      switch (name) {\n        case \"dangerouslySetInnerHTML\":\n        case \"children\":\n        case \"style\":\n        case \"suppressContentEditableWarning\":\n        case \"suppressHydrationWarning\":\n        case \"defaultValue\":\n        case \"defaultChecked\":\n        case \"innerHTML\":\n        case \"ref\":\n          return !0;\n        case \"innerText\":\n        case \"textContent\":\n          return !0;\n      }\n      switch (typeof value) {\n        case \"boolean\":\n          switch (name) {\n            case \"autoFocus\":\n            case \"checked\":\n            case \"multiple\":\n            case \"muted\":\n            case \"selected\":\n            case \"contentEditable\":\n            case \"spellCheck\":\n            case \"draggable\":\n            case \"value\":\n            case \"autoReverse\":\n            case \"externalResourcesRequired\":\n            case \"focusable\":\n            case \"preserveAlpha\":\n            case \"allowFullScreen\":\n            case \"async\":\n            case \"autoPlay\":\n            case \"controls\":\n            case \"default\":\n            case \"defer\":\n            case \"disabled\":\n            case \"disablePictureInPicture\":\n            case \"disableRemotePlayback\":\n            case \"formNoValidate\":\n            case \"hidden\":\n            case \"loop\":\n            case \"noModule\":\n            case \"noValidate\":\n            case \"open\":\n            case \"playsInline\":\n            case \"readOnly\":\n            case \"required\":\n            case \"reversed\":\n            case \"scoped\":\n            case \"seamless\":\n            case \"itemScope\":\n            case \"capture\":\n            case \"download\":\n            case \"inert\":\n              return !0;\n            default:\n              lowerCasedName = name.toLowerCase().slice(0, 5);\n              if (\"data-\" === lowerCasedName || \"aria-\" === lowerCasedName)\n                return !0;\n              value\n                ? console.error(\n                    'Received `%s` for a non-boolean attribute `%s`.\\n\\nIf you want to write it to the DOM, pass a string instead: %s=\"%s\" or %s={value.toString()}.',\n                    value,\n                    name,\n                    name,\n                    value,\n                    name\n                  )\n                : console.error(\n                    'Received `%s` for a non-boolean attribute `%s`.\\n\\nIf you want to write it to the DOM, pass a string instead: %s=\"%s\" or %s={value.toString()}.\\n\\nIf you used to conditionally omit it with %s={condition && value}, pass %s={condition ? value : undefined} instead.',\n                    value,\n                    name,\n                    name,\n                    value,\n                    name,\n                    name,\n                    name\n                  );\n              return (warnedProperties[name] = !0);\n          }\n        case \"function\":\n        case \"symbol\":\n          return (warnedProperties[name] = !0), !1;\n        case \"string\":\n          if (\"false\" === value || \"true\" === value) {\n            switch (name) {\n              case \"checked\":\n              case \"selected\":\n              case \"multiple\":\n              case \"muted\":\n              case \"allowFullScreen\":\n              case \"async\":\n              case \"autoPlay\":\n              case \"controls\":\n              case \"default\":\n              case \"defer\":\n              case \"disabled\":\n              case \"disablePictureInPicture\":\n              case \"disableRemotePlayback\":\n              case \"formNoValidate\":\n              case \"hidden\":\n              case \"loop\":\n              case \"noModule\":\n              case \"noValidate\":\n              case \"open\":\n              case \"playsInline\":\n              case \"readOnly\":\n              case \"required\":\n              case \"reversed\":\n              case \"scoped\":\n              case \"seamless\":\n              case \"itemScope\":\n              case \"inert\":\n                break;\n              default:\n                return !0;\n            }\n            console.error(\n              \"Received the string `%s` for the boolean attribute `%s`. %s Did you mean %s={%s}?\",\n              value,\n              name,\n              \"false\" === value\n                ? \"The browser will interpret it as a truthy value.\"\n                : 'Although this works, it will not work as expected if you pass the string \"false\".',\n              name,\n              value\n            );\n            warnedProperties[name] = !0;\n          }\n      }\n      return !0;\n    }\n    function warnUnknownProperties(type, props, eventRegistry) {\n      var unknownProps = [],\n        key;\n      for (key in props)\n        validateProperty(type, key, props[key], eventRegistry) ||\n          unknownProps.push(key);\n      props = unknownProps\n        .map(function (prop) {\n          return \"`\" + prop + \"`\";\n        })\n        .join(\", \");\n      1 === unknownProps.length\n        ? console.error(\n            \"Invalid value for prop %s on <%s> tag. Either remove it from the element, or pass a string or number value to keep it in the DOM. For details, see https://react.dev/link/attribute-behavior \",\n            props,\n            type\n          )\n        : 1 < unknownProps.length &&\n          console.error(\n            \"Invalid values for props %s on <%s> tag. Either remove them from the element, or pass a string or number value to keep them in the DOM. For details, see https://react.dev/link/attribute-behavior \",\n            props,\n            type\n          );\n    }\n    function sanitizeURL(url) {\n      return isJavaScriptProtocol.test(\"\" + url)\n        ? \"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')\"\n        : url;\n    }\n    function getEventTarget(nativeEvent) {\n      nativeEvent = nativeEvent.target || nativeEvent.srcElement || window;\n      nativeEvent.correspondingUseElement &&\n        (nativeEvent = nativeEvent.correspondingUseElement);\n      return 3 === nativeEvent.nodeType ? nativeEvent.parentNode : nativeEvent;\n    }\n    function restoreStateOfTarget(target) {\n      var internalInstance = getInstanceFromNode(target);\n      if (internalInstance && (target = internalInstance.stateNode)) {\n        var props = target[internalPropsKey] || null;\n        a: switch (\n          ((target = internalInstance.stateNode), internalInstance.type)\n        ) {\n          case \"input\":\n            updateInput(\n              target,\n              props.value,\n              props.defaultValue,\n              props.defaultValue,\n              props.checked,\n              props.defaultChecked,\n              props.type,\n              props.name\n            );\n            internalInstance = props.name;\n            if (\"radio\" === props.type && null != internalInstance) {\n              for (props = target; props.parentNode; ) props = props.parentNode;\n              checkAttributeStringCoercion(internalInstance, \"name\");\n              props = props.querySelectorAll(\n                'input[name=\"' +\n                  escapeSelectorAttributeValueInsideDoubleQuotes(\n                    \"\" + internalInstance\n                  ) +\n                  '\"][type=\"radio\"]'\n              );\n              for (\n                internalInstance = 0;\n                internalInstance < props.length;\n                internalInstance++\n              ) {\n                var otherNode = props[internalInstance];\n                if (otherNode !== target && otherNode.form === target.form) {\n                  var otherProps = otherNode[internalPropsKey] || null;\n                  if (!otherProps)\n                    throw Error(\n                      \"ReactDOMInput: Mixing React and non-React radio inputs with the same `name` is not supported.\"\n                    );\n                  updateInput(\n                    otherNode,\n                    otherProps.value,\n                    otherProps.defaultValue,\n                    otherProps.defaultValue,\n                    otherProps.checked,\n                    otherProps.defaultChecked,\n                    otherProps.type,\n                    otherProps.name\n                  );\n                }\n              }\n              for (\n                internalInstance = 0;\n                internalInstance < props.length;\n                internalInstance++\n              )\n                (otherNode = props[internalInstance]),\n                  otherNode.form === target.form &&\n                    updateValueIfChanged(otherNode);\n            }\n            break a;\n          case \"textarea\":\n            updateTextarea(target, props.value, props.defaultValue);\n            break a;\n          case \"select\":\n            (internalInstance = props.value),\n              null != internalInstance &&\n                updateOptions(target, !!props.multiple, internalInstance, !1);\n        }\n      }\n    }\n    function batchedUpdates$1(fn, a, b) {\n      if (isInsideEventHandler) return fn(a, b);\n      isInsideEventHandler = !0;\n      try {\n        var JSCompiler_inline_result = fn(a);\n        return JSCompiler_inline_result;\n      } finally {\n        if (\n          ((isInsideEventHandler = !1),\n          null !== restoreTarget || null !== restoreQueue)\n        )\n          if (\n            (flushSyncWork$1(),\n            restoreTarget &&\n              ((a = restoreTarget),\n              (fn = restoreQueue),\n              (restoreQueue = restoreTarget = null),\n              restoreStateOfTarget(a),\n              fn))\n          )\n            for (a = 0; a < fn.length; a++) restoreStateOfTarget(fn[a]);\n      }\n    }\n    function getListener(inst, registrationName) {\n      var stateNode = inst.stateNode;\n      if (null === stateNode) return null;\n      var props = stateNode[internalPropsKey] || null;\n      if (null === props) return null;\n      stateNode = props[registrationName];\n      a: switch (registrationName) {\n        case \"onClick\":\n        case \"onClickCapture\":\n        case \"onDoubleClick\":\n        case \"onDoubleClickCapture\":\n        case \"onMouseDown\":\n        case \"onMouseDownCapture\":\n        case \"onMouseMove\":\n        case \"onMouseMoveCapture\":\n        case \"onMouseUp\":\n        case \"onMouseUpCapture\":\n        case \"onMouseEnter\":\n          (props = !props.disabled) ||\n            ((inst = inst.type),\n            (props = !(\n              \"button\" === inst ||\n              \"input\" === inst ||\n              \"select\" === inst ||\n              \"textarea\" === inst\n            )));\n          inst = !props;\n          break a;\n        default:\n          inst = !1;\n      }\n      if (inst) return null;\n      if (stateNode && \"function\" !== typeof stateNode)\n        throw Error(\n          \"Expected `\" +\n            registrationName +\n            \"` listener to be a function, instead got a value of `\" +\n            typeof stateNode +\n            \"` type.\"\n        );\n      return stateNode;\n    }\n    function getData() {\n      if (fallbackText) return fallbackText;\n      var start,\n        startValue = startText,\n        startLength = startValue.length,\n        end,\n        endValue = \"value\" in root ? root.value : root.textContent,\n        endLength = endValue.length;\n      for (\n        start = 0;\n        start < startLength && startValue[start] === endValue[start];\n        start++\n      );\n      var minEnd = startLength - start;\n      for (\n        end = 1;\n        end <= minEnd &&\n        startValue[startLength - end] === endValue[endLength - end];\n        end++\n      );\n      return (fallbackText = endValue.slice(start, 1 < end ? 1 - end : void 0));\n    }\n    function getEventCharCode(nativeEvent) {\n      var keyCode = nativeEvent.keyCode;\n      \"charCode\" in nativeEvent\n        ? ((nativeEvent = nativeEvent.charCode),\n          0 === nativeEvent && 13 === keyCode && (nativeEvent = 13))\n        : (nativeEvent = keyCode);\n      10 === nativeEvent && (nativeEvent = 13);\n      return 32 <= nativeEvent || 13 === nativeEvent ? nativeEvent : 0;\n    }\n    function functionThatReturnsTrue() {\n      return !0;\n    }\n    function functionThatReturnsFalse() {\n      return !1;\n    }\n    function createSyntheticEvent(Interface) {\n      function SyntheticBaseEvent(\n        reactName,\n        reactEventType,\n        targetInst,\n        nativeEvent,\n        nativeEventTarget\n      ) {\n        this._reactName = reactName;\n        this._targetInst = targetInst;\n        this.type = reactEventType;\n        this.nativeEvent = nativeEvent;\n        this.target = nativeEventTarget;\n        this.currentTarget = null;\n        for (var propName in Interface)\n          Interface.hasOwnProperty(propName) &&\n            ((reactName = Interface[propName]),\n            (this[propName] = reactName\n              ? reactName(nativeEvent)\n              : nativeEvent[propName]));\n        this.isDefaultPrevented = (\n          null != nativeEvent.defaultPrevented\n            ? nativeEvent.defaultPrevented\n            : !1 === nativeEvent.returnValue\n        )\n          ? functionThatReturnsTrue\n          : functionThatReturnsFalse;\n        this.isPropagationStopped = functionThatReturnsFalse;\n        return this;\n      }\n      assign(SyntheticBaseEvent.prototype, {\n        preventDefault: function () {\n          this.defaultPrevented = !0;\n          var event = this.nativeEvent;\n          event &&\n            (event.preventDefault\n              ? event.preventDefault()\n              : \"unknown\" !== typeof event.returnValue &&\n                (event.returnValue = !1),\n            (this.isDefaultPrevented = functionThatReturnsTrue));\n        },\n        stopPropagation: function () {\n          var event = this.nativeEvent;\n          event &&\n            (event.stopPropagation\n              ? event.stopPropagation()\n              : \"unknown\" !== typeof event.cancelBubble &&\n                (event.cancelBubble = !0),\n            (this.isPropagationStopped = functionThatReturnsTrue));\n        },\n        persist: function () {},\n        isPersistent: functionThatReturnsTrue\n      });\n      return SyntheticBaseEvent;\n    }\n    function modifierStateGetter(keyArg) {\n      var nativeEvent = this.nativeEvent;\n      return nativeEvent.getModifierState\n        ? nativeEvent.getModifierState(keyArg)\n        : (keyArg = modifierKeyToProp[keyArg])\n          ? !!nativeEvent[keyArg]\n          : !1;\n    }\n    function getEventModifierState() {\n      return modifierStateGetter;\n    }\n    function isFallbackCompositionEnd(domEventName, nativeEvent) {\n      switch (domEventName) {\n        case \"keyup\":\n          return -1 !== END_KEYCODES.indexOf(nativeEvent.keyCode);\n        case \"keydown\":\n          return nativeEvent.keyCode !== START_KEYCODE;\n        case \"keypress\":\n        case \"mousedown\":\n        case \"focusout\":\n          return !0;\n        default:\n          return !1;\n      }\n    }\n    function getDataFromCustomEvent(nativeEvent) {\n      nativeEvent = nativeEvent.detail;\n      return \"object\" === typeof nativeEvent && \"data\" in nativeEvent\n        ? nativeEvent.data\n        : null;\n    }\n    function getNativeBeforeInputChars(domEventName, nativeEvent) {\n      switch (domEventName) {\n        case \"compositionend\":\n          return getDataFromCustomEvent(nativeEvent);\n        case \"keypress\":\n          if (nativeEvent.which !== SPACEBAR_CODE) return null;\n          hasSpaceKeypress = !0;\n          return SPACEBAR_CHAR;\n        case \"textInput\":\n          return (\n            (domEventName = nativeEvent.data),\n            domEventName === SPACEBAR_CHAR && hasSpaceKeypress\n              ? null\n              : domEventName\n          );\n        default:\n          return null;\n      }\n    }\n    function getFallbackBeforeInputChars(domEventName, nativeEvent) {\n      if (isComposing)\n        return \"compositionend\" === domEventName ||\n          (!canUseCompositionEvent &&\n            isFallbackCompositionEnd(domEventName, nativeEvent))\n          ? ((domEventName = getData()),\n            (fallbackText = startText = root = null),\n            (isComposing = !1),\n            domEventName)\n          : null;\n      switch (domEventName) {\n        case \"paste\":\n          return null;\n        case \"keypress\":\n          if (\n            !(\n              nativeEvent.ctrlKey ||\n              nativeEvent.altKey ||\n              nativeEvent.metaKey\n            ) ||\n            (nativeEvent.ctrlKey && nativeEvent.altKey)\n          ) {\n            if (nativeEvent.char && 1 < nativeEvent.char.length)\n              return nativeEvent.char;\n            if (nativeEvent.which)\n              return String.fromCharCode(nativeEvent.which);\n          }\n          return null;\n        case \"compositionend\":\n          return useFallbackCompositionData && \"ko\" !== nativeEvent.locale\n            ? null\n            : nativeEvent.data;\n        default:\n          return null;\n      }\n    }\n    function isTextInputElement(elem) {\n      var nodeName = elem && elem.nodeName && elem.nodeName.toLowerCase();\n      return \"input\" === nodeName\n        ? !!supportedInputTypes[elem.type]\n        : \"textarea\" === nodeName\n          ? !0\n          : !1;\n    }\n    function isEventSupported(eventNameSuffix) {\n      if (!canUseDOM) return !1;\n      eventNameSuffix = \"on\" + eventNameSuffix;\n      var isSupported = eventNameSuffix in document;\n      isSupported ||\n        ((isSupported = document.createElement(\"div\")),\n        isSupported.setAttribute(eventNameSuffix, \"return;\"),\n        (isSupported = \"function\" === typeof isSupported[eventNameSuffix]));\n      return isSupported;\n    }\n    function createAndAccumulateChangeEvent(\n      dispatchQueue,\n      inst,\n      nativeEvent,\n      target\n    ) {\n      restoreTarget\n        ? restoreQueue\n          ? restoreQueue.push(target)\n          : (restoreQueue = [target])\n        : (restoreTarget = target);\n      inst = accumulateTwoPhaseListeners(inst, \"onChange\");\n      0 < inst.length &&\n        ((nativeEvent = new SyntheticEvent(\n          \"onChange\",\n          \"change\",\n          null,\n          nativeEvent,\n          target\n        )),\n        dispatchQueue.push({ event: nativeEvent, listeners: inst }));\n    }\n    function runEventInBatch(dispatchQueue) {\n      processDispatchQueue(dispatchQueue, 0);\n    }\n    function getInstIfValueChanged(targetInst) {\n      var targetNode = getNodeFromInstance(targetInst);\n      if (updateValueIfChanged(targetNode)) return targetInst;\n    }\n    function getTargetInstForChangeEvent(domEventName, targetInst) {\n      if (\"change\" === domEventName) return targetInst;\n    }\n    function stopWatchingForValueChange() {\n      activeElement$1 &&\n        (activeElement$1.detachEvent(\"onpropertychange\", handlePropertyChange),\n        (activeElementInst$1 = activeElement$1 = null));\n    }\n    function handlePropertyChange(nativeEvent) {\n      if (\n        \"value\" === nativeEvent.propertyName &&\n        getInstIfValueChanged(activeElementInst$1)\n      ) {\n        var dispatchQueue = [];\n        createAndAccumulateChangeEvent(\n          dispatchQueue,\n          activeElementInst$1,\n          nativeEvent,\n          getEventTarget(nativeEvent)\n        );\n        batchedUpdates$1(runEventInBatch, dispatchQueue);\n      }\n    }\n    function handleEventsForInputEventPolyfill(\n      domEventName,\n      target,\n      targetInst\n    ) {\n      \"focusin\" === domEventName\n        ? (stopWatchingForValueChange(),\n          (activeElement$1 = target),\n          (activeElementInst$1 = targetInst),\n          activeElement$1.attachEvent(\"onpropertychange\", handlePropertyChange))\n        : \"focusout\" === domEventName && stopWatchingForValueChange();\n    }\n    function getTargetInstForInputEventPolyfill(domEventName) {\n      if (\n        \"selectionchange\" === domEventName ||\n        \"keyup\" === domEventName ||\n        \"keydown\" === domEventName\n      )\n        return getInstIfValueChanged(activeElementInst$1);\n    }\n    function getTargetInstForClickEvent(domEventName, targetInst) {\n      if (\"click\" === domEventName) return getInstIfValueChanged(targetInst);\n    }\n    function getTargetInstForInputOrChangeEvent(domEventName, targetInst) {\n      if (\"input\" === domEventName || \"change\" === domEventName)\n        return getInstIfValueChanged(targetInst);\n    }\n    function is(x, y) {\n      return (x === y && (0 !== x || 1 / x === 1 / y)) || (x !== x && y !== y);\n    }\n    function shallowEqual(objA, objB) {\n      if (objectIs(objA, objB)) return !0;\n      if (\n        \"object\" !== typeof objA ||\n        null === objA ||\n        \"object\" !== typeof objB ||\n        null === objB\n      )\n        return !1;\n      var keysA = Object.keys(objA),\n        keysB = Object.keys(objB);\n      if (keysA.length !== keysB.length) return !1;\n      for (keysB = 0; keysB < keysA.length; keysB++) {\n        var currentKey = keysA[keysB];\n        if (\n          !hasOwnProperty.call(objB, currentKey) ||\n          !objectIs(objA[currentKey], objB[currentKey])\n        )\n          return !1;\n      }\n      return !0;\n    }\n    function getLeafNode(node) {\n      for (; node && node.firstChild; ) node = node.firstChild;\n      return node;\n    }\n    function getNodeForCharacterOffset(root, offset) {\n      var node = getLeafNode(root);\n      root = 0;\n      for (var nodeEnd; node; ) {\n        if (3 === node.nodeType) {\n          nodeEnd = root + node.textContent.length;\n          if (root <= offset && nodeEnd >= offset)\n            return { node: node, offset: offset - root };\n          root = nodeEnd;\n        }\n        a: {\n          for (; node; ) {\n            if (node.nextSibling) {\n              node = node.nextSibling;\n              break a;\n            }\n            node = node.parentNode;\n          }\n          node = void 0;\n        }\n        node = getLeafNode(node);\n      }\n    }\n    function containsNode(outerNode, innerNode) {\n      return outerNode && innerNode\n        ? outerNode === innerNode\n          ? !0\n          : outerNode && 3 === outerNode.nodeType\n            ? !1\n            : innerNode && 3 === innerNode.nodeType\n              ? containsNode(outerNode, innerNode.parentNode)\n              : \"contains\" in outerNode\n                ? outerNode.contains(innerNode)\n                : outerNode.compareDocumentPosition\n                  ? !!(outerNode.compareDocumentPosition(innerNode) & 16)\n                  : !1\n        : !1;\n    }\n    function getActiveElementDeep(containerInfo) {\n      containerInfo =\n        null != containerInfo &&\n        null != containerInfo.ownerDocument &&\n        null != containerInfo.ownerDocument.defaultView\n          ? containerInfo.ownerDocument.defaultView\n          : window;\n      for (\n        var element = getActiveElement(containerInfo.document);\n        element instanceof containerInfo.HTMLIFrameElement;\n\n      ) {\n        try {\n          var JSCompiler_inline_result =\n            \"string\" === typeof element.contentWindow.location.href;\n        } catch (err) {\n          JSCompiler_inline_result = !1;\n        }\n        if (JSCompiler_inline_result) containerInfo = element.contentWindow;\n        else break;\n        element = getActiveElement(containerInfo.document);\n      }\n      return element;\n    }\n    function hasSelectionCapabilities(elem) {\n      var nodeName = elem && elem.nodeName && elem.nodeName.toLowerCase();\n      return (\n        nodeName &&\n        ((\"input\" === nodeName &&\n          (\"text\" === elem.type ||\n            \"search\" === elem.type ||\n            \"tel\" === elem.type ||\n            \"url\" === elem.type ||\n            \"password\" === elem.type)) ||\n          \"textarea\" === nodeName ||\n          \"true\" === elem.contentEditable)\n      );\n    }\n    function constructSelectEvent(\n      dispatchQueue,\n      nativeEvent,\n      nativeEventTarget\n    ) {\n      var doc =\n        nativeEventTarget.window === nativeEventTarget\n          ? nativeEventTarget.document\n          : 9 === nativeEventTarget.nodeType\n            ? nativeEventTarget\n            : nativeEventTarget.ownerDocument;\n      mouseDown ||\n        null == activeElement ||\n        activeElement !== getActiveElement(doc) ||\n        ((doc = activeElement),\n        \"selectionStart\" in doc && hasSelectionCapabilities(doc)\n          ? (doc = { start: doc.selectionStart, end: doc.selectionEnd })\n          : ((doc = (\n              (doc.ownerDocument && doc.ownerDocument.defaultView) ||\n              window\n            ).getSelection()),\n            (doc = {\n              anchorNode: doc.anchorNode,\n              anchorOffset: doc.anchorOffset,\n              focusNode: doc.focusNode,\n              focusOffset: doc.focusOffset\n            })),\n        (lastSelection && shallowEqual(lastSelection, doc)) ||\n          ((lastSelection = doc),\n          (doc = accumulateTwoPhaseListeners(activeElementInst, \"onSelect\")),\n          0 < doc.length &&\n            ((nativeEvent = new SyntheticEvent(\n              \"onSelect\",\n              \"select\",\n              null,\n              nativeEvent,\n              nativeEventTarget\n            )),\n            dispatchQueue.push({ event: nativeEvent, listeners: doc }),\n            (nativeEvent.target = activeElement))));\n    }\n    function makePrefixMap(styleProp, eventName) {\n      var prefixes = {};\n      prefixes[styleProp.toLowerCase()] = eventName.toLowerCase();\n      prefixes[\"Webkit\" + styleProp] = \"webkit\" + eventName;\n      prefixes[\"Moz\" + styleProp] = \"moz\" + eventName;\n      return prefixes;\n    }\n    function getVendorPrefixedEventName(eventName) {\n      if (prefixedEventNames[eventName]) return prefixedEventNames[eventName];\n      if (!vendorPrefixes[eventName]) return eventName;\n      var prefixMap = vendorPrefixes[eventName],\n        styleProp;\n      for (styleProp in prefixMap)\n        if (prefixMap.hasOwnProperty(styleProp) && styleProp in style)\n          return (prefixedEventNames[eventName] = prefixMap[styleProp]);\n      return eventName;\n    }\n    function registerSimpleEvent(domEventName, reactName) {\n      topLevelEventsToReactNames.set(domEventName, reactName);\n      registerTwoPhaseEvent(reactName, [domEventName]);\n    }\n    function createCapturedValueAtFiber(value, source) {\n      if (\"object\" === typeof value && null !== value) {\n        var existing = CapturedStacks.get(value);\n        if (void 0 !== existing) return existing;\n        source = {\n          value: value,\n          source: source,\n          stack: getStackByFiberInDevAndProd(source)\n        };\n        CapturedStacks.set(value, source);\n        return source;\n      }\n      return {\n        value: value,\n        source: source,\n        stack: getStackByFiberInDevAndProd(source)\n      };\n    }\n    function finishQueueingConcurrentUpdates() {\n      for (\n        var endIndex = concurrentQueuesIndex,\n          i = (concurrentlyUpdatedLanes = concurrentQueuesIndex = 0);\n        i < endIndex;\n\n      ) {\n        var fiber = concurrentQueues[i];\n        concurrentQueues[i++] = null;\n        var queue = concurrentQueues[i];\n        concurrentQueues[i++] = null;\n        var update = concurrentQueues[i];\n        concurrentQueues[i++] = null;\n        var lane = concurrentQueues[i];\n        concurrentQueues[i++] = null;\n        if (null !== queue && null !== update) {\n          var pending = queue.pending;\n          null === pending\n            ? (update.next = update)\n            : ((update.next = pending.next), (pending.next = update));\n          queue.pending = update;\n        }\n        0 !== lane && markUpdateLaneFromFiberToRoot(fiber, update, lane);\n      }\n    }\n    function enqueueUpdate$1(fiber, queue, update, lane) {\n      concurrentQueues[concurrentQueuesIndex++] = fiber;\n      concurrentQueues[concurrentQueuesIndex++] = queue;\n      concurrentQueues[concurrentQueuesIndex++] = update;\n      concurrentQueues[concurrentQueuesIndex++] = lane;\n      concurrentlyUpdatedLanes |= lane;\n      fiber.lanes |= lane;\n      fiber = fiber.alternate;\n      null !== fiber && (fiber.lanes |= lane);\n    }\n    function enqueueConcurrentHookUpdate(fiber, queue, update, lane) {\n      enqueueUpdate$1(fiber, queue, update, lane);\n      return getRootForUpdatedFiber(fiber);\n    }\n    function enqueueConcurrentRenderForLane(fiber, lane) {\n      enqueueUpdate$1(fiber, null, null, lane);\n      return getRootForUpdatedFiber(fiber);\n    }\n    function markUpdateLaneFromFiberToRoot(sourceFiber, update, lane) {\n      sourceFiber.lanes |= lane;\n      var alternate = sourceFiber.alternate;\n      null !== alternate && (alternate.lanes |= lane);\n      for (var isHidden = !1, parent = sourceFiber.return; null !== parent; )\n        (parent.childLanes |= lane),\n          (alternate = parent.alternate),\n          null !== alternate && (alternate.childLanes |= lane),\n          22 === parent.tag &&\n            ((sourceFiber = parent.stateNode),\n            null === sourceFiber ||\n              sourceFiber._visibility & OffscreenVisible ||\n              (isHidden = !0)),\n          (sourceFiber = parent),\n          (parent = parent.return);\n      return 3 === sourceFiber.tag\n        ? ((parent = sourceFiber.stateNode),\n          isHidden &&\n            null !== update &&\n            ((isHidden = 31 - clz32(lane)),\n            (sourceFiber = parent.hiddenUpdates),\n            (alternate = sourceFiber[isHidden]),\n            null === alternate\n              ? (sourceFiber[isHidden] = [update])\n              : alternate.push(update),\n            (update.lane = lane | 536870912)),\n          parent)\n        : null;\n    }\n    function getRootForUpdatedFiber(sourceFiber) {\n      if (nestedUpdateCount > NESTED_UPDATE_LIMIT)\n        throw (\n          ((nestedPassiveUpdateCount = nestedUpdateCount = 0),\n          (rootWithPassiveNestedUpdates = rootWithNestedUpdates = null),\n          Error(\n            \"Maximum update depth exceeded. This can happen when a component repeatedly calls setState inside componentWillUpdate or componentDidUpdate. React limits the number of nested updates to prevent infinite loops.\"\n          ))\n        );\n      nestedPassiveUpdateCount > NESTED_PASSIVE_UPDATE_LIMIT &&\n        ((nestedPassiveUpdateCount = 0),\n        (rootWithPassiveNestedUpdates = null),\n        console.error(\n          \"Maximum update depth exceeded. This can happen when a component calls setState inside useEffect, but useEffect either doesn't have a dependency array, or one of the dependencies changes on every render.\"\n        ));\n      null === sourceFiber.alternate &&\n        0 !== (sourceFiber.flags & 4098) &&\n        warnAboutUpdateOnNotYetMountedFiberInDEV(sourceFiber);\n      for (var node = sourceFiber, parent = node.return; null !== parent; )\n        null === node.alternate &&\n          0 !== (node.flags & 4098) &&\n          warnAboutUpdateOnNotYetMountedFiberInDEV(sourceFiber),\n          (node = parent),\n          (parent = node.return);\n      return 3 === node.tag ? node.stateNode : null;\n    }\n    function resolveFunctionForHotReloading(type) {\n      if (null === resolveFamily) return type;\n      var family = resolveFamily(type);\n      return void 0 === family ? type : family.current;\n    }\n    function resolveForwardRefForHotReloading(type) {\n      if (null === resolveFamily) return type;\n      var family = resolveFamily(type);\n      return void 0 === family\n        ? null !== type &&\n          void 0 !== type &&\n          \"function\" === typeof type.render &&\n          ((family = resolveFunctionForHotReloading(type.render)),\n          type.render !== family)\n          ? ((family = { $$typeof: REACT_FORWARD_REF_TYPE, render: family }),\n            void 0 !== type.displayName &&\n              (family.displayName = type.displayName),\n            family)\n          : type\n        : family.current;\n    }\n    function isCompatibleFamilyForHotReloading(fiber, element) {\n      if (null === resolveFamily) return !1;\n      var prevType = fiber.elementType;\n      element = element.type;\n      var needsCompareFamilies = !1,\n        $$typeofNextType =\n          \"object\" === typeof element && null !== element\n            ? element.$$typeof\n            : null;\n      switch (fiber.tag) {\n        case 1:\n          \"function\" === typeof element && (needsCompareFamilies = !0);\n          break;\n        case 0:\n          \"function\" === typeof element\n            ? (needsCompareFamilies = !0)\n            : $$typeofNextType === REACT_LAZY_TYPE &&\n              (needsCompareFamilies = !0);\n          break;\n        case 11:\n          $$typeofNextType === REACT_FORWARD_REF_TYPE\n            ? (needsCompareFamilies = !0)\n            : $$typeofNextType === REACT_LAZY_TYPE &&\n              (needsCompareFamilies = !0);\n          break;\n        case 14:\n        case 15:\n          $$typeofNextType === REACT_MEMO_TYPE\n            ? (needsCompareFamilies = !0)\n            : $$typeofNextType === REACT_LAZY_TYPE &&\n              (needsCompareFamilies = !0);\n          break;\n        default:\n          return !1;\n      }\n      return needsCompareFamilies &&\n        ((fiber = resolveFamily(prevType)),\n        void 0 !== fiber && fiber === resolveFamily(element))\n        ? !0\n        : !1;\n    }\n    function markFailedErrorBoundaryForHotReloading(fiber) {\n      null !== resolveFamily &&\n        \"function\" === typeof WeakSet &&\n        (null === failedBoundaries && (failedBoundaries = new WeakSet()),\n        failedBoundaries.add(fiber));\n    }\n    function scheduleFibersWithFamiliesRecursively(\n      fiber,\n      updatedFamilies,\n      staleFamilies\n    ) {\n      var alternate = fiber.alternate,\n        child = fiber.child,\n        sibling = fiber.sibling,\n        tag = fiber.tag,\n        type = fiber.type,\n        candidateType = null;\n      switch (tag) {\n        case 0:\n        case 15:\n        case 1:\n          candidateType = type;\n          break;\n        case 11:\n          candidateType = type.render;\n      }\n      if (null === resolveFamily)\n        throw Error(\"Expected resolveFamily to be set during hot reload.\");\n      var needsRender = !1;\n      type = !1;\n      null !== candidateType &&\n        ((candidateType = resolveFamily(candidateType)),\n        void 0 !== candidateType &&\n          (staleFamilies.has(candidateType)\n            ? (type = !0)\n            : updatedFamilies.has(candidateType) &&\n              (1 === tag ? (type = !0) : (needsRender = !0))));\n      null !== failedBoundaries &&\n        (failedBoundaries.has(fiber) ||\n          (null !== alternate && failedBoundaries.has(alternate))) &&\n        (type = !0);\n      type && (fiber._debugNeedsRemount = !0);\n      if (type || needsRender)\n        (alternate = enqueueConcurrentRenderForLane(fiber, 2)),\n          null !== alternate && scheduleUpdateOnFiber(alternate, fiber, 2);\n      null === child ||\n        type ||\n        scheduleFibersWithFamiliesRecursively(\n          child,\n          updatedFamilies,\n          staleFamilies\n        );\n      null !== sibling &&\n        scheduleFibersWithFamiliesRecursively(\n          sibling,\n          updatedFamilies,\n          staleFamilies\n        );\n    }\n    function FiberNode(tag, pendingProps, key, mode) {\n      this.tag = tag;\n      this.key = key;\n      this.sibling =\n        this.child =\n        this.return =\n        this.stateNode =\n        this.type =\n        this.elementType =\n          null;\n      this.index = 0;\n      this.refCleanup = this.ref = null;\n      this.pendingProps = pendingProps;\n      this.dependencies =\n        this.memoizedState =\n        this.updateQueue =\n        this.memoizedProps =\n          null;\n      this.mode = mode;\n      this.subtreeFlags = this.flags = 0;\n      this.deletions = null;\n      this.childLanes = this.lanes = 0;\n      this.alternate = null;\n      this.actualDuration = -0;\n      this.actualStartTime = -1.1;\n      this.treeBaseDuration = this.selfBaseDuration = -0;\n      this._debugTask =\n        this._debugStack =\n        this._debugOwner =\n        this._debugInfo =\n          null;\n      this._debugNeedsRemount = !1;\n      this._debugHookTypes = null;\n      hasBadMapPolyfill ||\n        \"function\" !== typeof Object.preventExtensions ||\n        Object.preventExtensions(this);\n    }\n    function shouldConstruct(Component) {\n      Component = Component.prototype;\n      return !(!Component || !Component.isReactComponent);\n    }\n    function createWorkInProgress(current, pendingProps) {\n      var workInProgress = current.alternate;\n      null === workInProgress\n        ? ((workInProgress = createFiber(\n            current.tag,\n            pendingProps,\n            current.key,\n            current.mode\n          )),\n          (workInProgress.elementType = current.elementType),\n          (workInProgress.type = current.type),\n          (workInProgress.stateNode = current.stateNode),\n          (workInProgress._debugOwner = current._debugOwner),\n          (workInProgress._debugStack = current._debugStack),\n          (workInProgress._debugTask = current._debugTask),\n          (workInProgress._debugHookTypes = current._debugHookTypes),\n          (workInProgress.alternate = current),\n          (current.alternate = workInProgress))\n        : ((workInProgress.pendingProps = pendingProps),\n          (workInProgress.type = current.type),\n          (workInProgress.flags = 0),\n          (workInProgress.subtreeFlags = 0),\n          (workInProgress.deletions = null),\n          (workInProgress.actualDuration = -0),\n          (workInProgress.actualStartTime = -1.1));\n      workInProgress.flags = current.flags & 65011712;\n      workInProgress.childLanes = current.childLanes;\n      workInProgress.lanes = current.lanes;\n      workInProgress.child = current.child;\n      workInProgress.memoizedProps = current.memoizedProps;\n      workInProgress.memoizedState = current.memoizedState;\n      workInProgress.updateQueue = current.updateQueue;\n      pendingProps = current.dependencies;\n      workInProgress.dependencies =\n        null === pendingProps\n          ? null\n          : {\n              lanes: pendingProps.lanes,\n              firstContext: pendingProps.firstContext,\n              _debugThenableState: pendingProps._debugThenableState\n            };\n      workInProgress.sibling = current.sibling;\n      workInProgress.index = current.index;\n      workInProgress.ref = current.ref;\n      workInProgress.refCleanup = current.refCleanup;\n      workInProgress.selfBaseDuration = current.selfBaseDuration;\n      workInProgress.treeBaseDuration = current.treeBaseDuration;\n      workInProgress._debugInfo = current._debugInfo;\n      workInProgress._debugNeedsRemount = current._debugNeedsRemount;\n      switch (workInProgress.tag) {\n        case 0:\n        case 15:\n          workInProgress.type = resolveFunctionForHotReloading(current.type);\n          break;\n        case 1:\n          workInProgress.type = resolveFunctionForHotReloading(current.type);\n          break;\n        case 11:\n          workInProgress.type = resolveForwardRefForHotReloading(current.type);\n      }\n      return workInProgress;\n    }\n    function resetWorkInProgress(workInProgress, renderLanes) {\n      workInProgress.flags &= 65011714;\n      var current = workInProgress.alternate;\n      null === current\n        ? ((workInProgress.childLanes = 0),\n          (workInProgress.lanes = renderLanes),\n          (workInProgress.child = null),\n          (workInProgress.subtreeFlags = 0),\n          (workInProgress.memoizedProps = null),\n          (workInProgress.memoizedState = null),\n          (workInProgress.updateQueue = null),\n          (workInProgress.dependencies = null),\n          (workInProgress.stateNode = null),\n          (workInProgress.selfBaseDuration = 0),\n          (workInProgress.treeBaseDuration = 0))\n        : ((workInProgress.childLanes = current.childLanes),\n          (workInProgress.lanes = current.lanes),\n          (workInProgress.child = current.child),\n          (workInProgress.subtreeFlags = 0),\n          (workInProgress.deletions = null),\n          (workInProgress.memoizedProps = current.memoizedProps),\n          (workInProgress.memoizedState = current.memoizedState),\n          (workInProgress.updateQueue = current.updateQueue),\n          (workInProgress.type = current.type),\n          (renderLanes = current.dependencies),\n          (workInProgress.dependencies =\n            null === renderLanes\n              ? null\n              : {\n                  lanes: renderLanes.lanes,\n                  firstContext: renderLanes.firstContext,\n                  _debugThenableState: renderLanes._debugThenableState\n                }),\n          (workInProgress.selfBaseDuration = current.selfBaseDuration),\n          (workInProgress.treeBaseDuration = current.treeBaseDuration));\n      return workInProgress;\n    }\n    function createFiberFromTypeAndProps(\n      type,\n      key,\n      pendingProps,\n      owner,\n      mode,\n      lanes\n    ) {\n      var fiberTag = 0,\n        resolvedType = type;\n      if (\"function\" === typeof type)\n        shouldConstruct(type) && (fiberTag = 1),\n          (resolvedType = resolveFunctionForHotReloading(resolvedType));\n      else if (\"string\" === typeof type)\n        (fiberTag = getHostContext()),\n          (fiberTag = isHostHoistableType(type, pendingProps, fiberTag)\n            ? 26\n            : \"html\" === type || \"head\" === type || \"body\" === type\n              ? 27\n              : 5);\n      else\n        a: switch (type) {\n          case REACT_ACTIVITY_TYPE:\n            return (\n              (key = createFiber(31, pendingProps, key, mode)),\n              (key.elementType = REACT_ACTIVITY_TYPE),\n              (key.lanes = lanes),\n              key\n            );\n          case REACT_FRAGMENT_TYPE:\n            return createFiberFromFragment(\n              pendingProps.children,\n              mode,\n              lanes,\n              key\n            );\n          case REACT_STRICT_MODE_TYPE:\n            fiberTag = 8;\n            mode |= StrictLegacyMode;\n            mode |= StrictEffectsMode;\n            break;\n          case REACT_PROFILER_TYPE:\n            return (\n              (type = pendingProps),\n              (owner = mode),\n              \"string\" !== typeof type.id &&\n                console.error(\n                  'Profiler must specify an \"id\" of type `string` as a prop. Received the type `%s` instead.',\n                  typeof type.id\n                ),\n              (key = createFiber(12, type, key, owner | ProfileMode)),\n              (key.elementType = REACT_PROFILER_TYPE),\n              (key.lanes = lanes),\n              (key.stateNode = { effectDuration: 0, passiveEffectDuration: 0 }),\n              key\n            );\n          case REACT_SUSPENSE_TYPE:\n            return (\n              (key = createFiber(13, pendingProps, key, mode)),\n              (key.elementType = REACT_SUSPENSE_TYPE),\n              (key.lanes = lanes),\n              key\n            );\n          case REACT_SUSPENSE_LIST_TYPE:\n            return (\n              (key = createFiber(19, pendingProps, key, mode)),\n              (key.elementType = REACT_SUSPENSE_LIST_TYPE),\n              (key.lanes = lanes),\n              key\n            );\n          default:\n            if (\"object\" === typeof type && null !== type)\n              switch (type.$$typeof) {\n                case REACT_PROVIDER_TYPE:\n                case REACT_CONTEXT_TYPE:\n                  fiberTag = 10;\n                  break a;\n                case REACT_CONSUMER_TYPE:\n                  fiberTag = 9;\n                  break a;\n                case REACT_FORWARD_REF_TYPE:\n                  fiberTag = 11;\n                  resolvedType = resolveForwardRefForHotReloading(resolvedType);\n                  break a;\n                case REACT_MEMO_TYPE:\n                  fiberTag = 14;\n                  break a;\n                case REACT_LAZY_TYPE:\n                  fiberTag = 16;\n                  resolvedType = null;\n                  break a;\n              }\n            resolvedType = \"\";\n            if (\n              void 0 === type ||\n              (\"object\" === typeof type &&\n                null !== type &&\n                0 === Object.keys(type).length)\n            )\n              resolvedType +=\n                \" You likely forgot to export your component from the file it's defined in, or you might have mixed up default and named imports.\";\n            null === type\n              ? (pendingProps = \"null\")\n              : isArrayImpl(type)\n                ? (pendingProps = \"array\")\n                : void 0 !== type && type.$$typeof === REACT_ELEMENT_TYPE\n                  ? ((pendingProps =\n                      \"<\" +\n                      (getComponentNameFromType(type.type) || \"Unknown\") +\n                      \" />\"),\n                    (resolvedType =\n                      \" Did you accidentally export a JSX literal instead of a component?\"))\n                  : (pendingProps = typeof type);\n            (fiberTag = owner ? getComponentNameFromOwner(owner) : null) &&\n              (resolvedType +=\n                \"\\n\\nCheck the render method of `\" + fiberTag + \"`.\");\n            fiberTag = 29;\n            pendingProps = Error(\n              \"Element type is invalid: expected a string (for built-in components) or a class/function (for composite components) but got: \" +\n                (pendingProps + \".\" + resolvedType)\n            );\n            resolvedType = null;\n        }\n      key = createFiber(fiberTag, pendingProps, key, mode);\n      key.elementType = type;\n      key.type = resolvedType;\n      key.lanes = lanes;\n      key._debugOwner = owner;\n      return key;\n    }\n    function createFiberFromElement(element, mode, lanes) {\n      mode = createFiberFromTypeAndProps(\n        element.type,\n        element.key,\n        element.props,\n        element._owner,\n        mode,\n        lanes\n      );\n      mode._debugOwner = element._owner;\n      mode._debugStack = element._debugStack;\n      mode._debugTask = element._debugTask;\n      return mode;\n    }\n    function createFiberFromFragment(elements, mode, lanes, key) {\n      elements = createFiber(7, elements, key, mode);\n      elements.lanes = lanes;\n      return elements;\n    }\n    function createFiberFromText(content, mode, lanes) {\n      content = createFiber(6, content, null, mode);\n      content.lanes = lanes;\n      return content;\n    }\n    function createFiberFromPortal(portal, mode, lanes) {\n      mode = createFiber(\n        4,\n        null !== portal.children ? portal.children : [],\n        portal.key,\n        mode\n      );\n      mode.lanes = lanes;\n      mode.stateNode = {\n        containerInfo: portal.containerInfo,\n        pendingChildren: null,\n        implementation: portal.implementation\n      };\n      return mode;\n    }\n    function pushTreeFork(workInProgress, totalChildren) {\n      warnIfNotHydrating();\n      forkStack[forkStackIndex++] = treeForkCount;\n      forkStack[forkStackIndex++] = treeForkProvider;\n      treeForkProvider = workInProgress;\n      treeForkCount = totalChildren;\n    }\n    function pushTreeId(workInProgress, totalChildren, index) {\n      warnIfNotHydrating();\n      idStack[idStackIndex++] = treeContextId;\n      idStack[idStackIndex++] = treeContextOverflow;\n      idStack[idStackIndex++] = treeContextProvider;\n      treeContextProvider = workInProgress;\n      var baseIdWithLeadingBit = treeContextId;\n      workInProgress = treeContextOverflow;\n      var baseLength = 32 - clz32(baseIdWithLeadingBit) - 1;\n      baseIdWithLeadingBit &= ~(1 << baseLength);\n      index += 1;\n      var length = 32 - clz32(totalChildren) + baseLength;\n      if (30 < length) {\n        var numberOfOverflowBits = baseLength - (baseLength % 5);\n        length = (\n          baseIdWithLeadingBit &\n          ((1 << numberOfOverflowBits) - 1)\n        ).toString(32);\n        baseIdWithLeadingBit >>= numberOfOverflowBits;\n        baseLength -= numberOfOverflowBits;\n        treeContextId =\n          (1 << (32 - clz32(totalChildren) + baseLength)) |\n          (index << baseLength) |\n          baseIdWithLeadingBit;\n        treeContextOverflow = length + workInProgress;\n      } else\n        (treeContextId =\n          (1 << length) | (index << baseLength) | baseIdWithLeadingBit),\n          (treeContextOverflow = workInProgress);\n    }\n    function pushMaterializedTreeId(workInProgress) {\n      warnIfNotHydrating();\n      null !== workInProgress.return &&\n        (pushTreeFork(workInProgress, 1), pushTreeId(workInProgress, 1, 0));\n    }\n    function popTreeContext(workInProgress) {\n      for (; workInProgress === treeForkProvider; )\n        (treeForkProvider = forkStack[--forkStackIndex]),\n          (forkStack[forkStackIndex] = null),\n          (treeForkCount = forkStack[--forkStackIndex]),\n          (forkStack[forkStackIndex] = null);\n      for (; workInProgress === treeContextProvider; )\n        (treeContextProvider = idStack[--idStackIndex]),\n          (idStack[idStackIndex] = null),\n          (treeContextOverflow = idStack[--idStackIndex]),\n          (idStack[idStackIndex] = null),\n          (treeContextId = idStack[--idStackIndex]),\n          (idStack[idStackIndex] = null);\n    }\n    function warnIfNotHydrating() {\n      isHydrating ||\n        console.error(\n          \"Expected to be hydrating. This is a bug in React. Please file an issue.\"\n        );\n    }\n    function buildHydrationDiffNode(fiber, distanceFromLeaf) {\n      if (null === fiber.return) {\n        if (null === hydrationDiffRootDEV)\n          hydrationDiffRootDEV = {\n            fiber: fiber,\n            children: [],\n            serverProps: void 0,\n            serverTail: [],\n            distanceFromLeaf: distanceFromLeaf\n          };\n        else {\n          if (hydrationDiffRootDEV.fiber !== fiber)\n            throw Error(\n              \"Saw multiple hydration diff roots in a pass. This is a bug in React.\"\n            );\n          hydrationDiffRootDEV.distanceFromLeaf > distanceFromLeaf &&\n            (hydrationDiffRootDEV.distanceFromLeaf = distanceFromLeaf);\n        }\n        return hydrationDiffRootDEV;\n      }\n      var siblings = buildHydrationDiffNode(\n        fiber.return,\n        distanceFromLeaf + 1\n      ).children;\n      if (0 < siblings.length && siblings[siblings.length - 1].fiber === fiber)\n        return (\n          (siblings = siblings[siblings.length - 1]),\n          siblings.distanceFromLeaf > distanceFromLeaf &&\n            (siblings.distanceFromLeaf = distanceFromLeaf),\n          siblings\n        );\n      distanceFromLeaf = {\n        fiber: fiber,\n        children: [],\n        serverProps: void 0,\n        serverTail: [],\n        distanceFromLeaf: distanceFromLeaf\n      };\n      siblings.push(distanceFromLeaf);\n      return distanceFromLeaf;\n    }\n    function warnNonHydratedInstance(fiber, rejectedCandidate) {\n      didSuspendOrErrorDEV ||\n        ((fiber = buildHydrationDiffNode(fiber, 0)),\n        (fiber.serverProps = null),\n        null !== rejectedCandidate &&\n          ((rejectedCandidate =\n            describeHydratableInstanceForDevWarnings(rejectedCandidate)),\n          fiber.serverTail.push(rejectedCandidate)));\n    }\n    function throwOnHydrationMismatch(fiber) {\n      var diff = \"\",\n        diffRoot = hydrationDiffRootDEV;\n      null !== diffRoot &&\n        ((hydrationDiffRootDEV = null), (diff = describeDiff(diffRoot)));\n      queueHydrationError(\n        createCapturedValueAtFiber(\n          Error(\n            \"Hydration failed because the server rendered HTML didn't match the client. As a result this tree will be regenerated on the client. This can happen if a SSR-ed Client Component used:\\n\\n- A server/client branch `if (typeof window !== 'undefined')`.\\n- Variable input such as `Date.now()` or `Math.random()` which changes each time it's called.\\n- Date formatting in a user's locale which doesn't match the server.\\n- External changing data without sending a snapshot of it along with the HTML.\\n- Invalid HTML tag nesting.\\n\\nIt can also happen if the client has a browser extension installed which messes with the HTML before React loaded.\\n\\nhttps://react.dev/link/hydration-mismatch\" +\n              diff\n          ),\n          fiber\n        )\n      );\n      throw HydrationMismatchException;\n    }\n    function prepareToHydrateHostInstance(fiber) {\n      var didHydrate = fiber.stateNode;\n      var type = fiber.type,\n        props = fiber.memoizedProps;\n      didHydrate[internalInstanceKey] = fiber;\n      didHydrate[internalPropsKey] = props;\n      validatePropertiesInDevelopment(type, props);\n      switch (type) {\n        case \"dialog\":\n          listenToNonDelegatedEvent(\"cancel\", didHydrate);\n          listenToNonDelegatedEvent(\"close\", didHydrate);\n          break;\n        case \"iframe\":\n        case \"object\":\n        case \"embed\":\n          listenToNonDelegatedEvent(\"load\", didHydrate);\n          break;\n        case \"video\":\n        case \"audio\":\n          for (type = 0; type < mediaEventTypes.length; type++)\n            listenToNonDelegatedEvent(mediaEventTypes[type], didHydrate);\n          break;\n        case \"source\":\n          listenToNonDelegatedEvent(\"error\", didHydrate);\n          break;\n        case \"img\":\n        case \"image\":\n        case \"link\":\n          listenToNonDelegatedEvent(\"error\", didHydrate);\n          listenToNonDelegatedEvent(\"load\", didHydrate);\n          break;\n        case \"details\":\n          listenToNonDelegatedEvent(\"toggle\", didHydrate);\n          break;\n        case \"input\":\n          checkControlledValueProps(\"input\", props);\n          listenToNonDelegatedEvent(\"invalid\", didHydrate);\n          validateInputProps(didHydrate, props);\n          initInput(\n            didHydrate,\n            props.value,\n            props.defaultValue,\n            props.checked,\n            props.defaultChecked,\n            props.type,\n            props.name,\n            !0\n          );\n          track(didHydrate);\n          break;\n        case \"option\":\n          validateOptionProps(didHydrate, props);\n          break;\n        case \"select\":\n          checkControlledValueProps(\"select\", props);\n          listenToNonDelegatedEvent(\"invalid\", didHydrate);\n          validateSelectProps(didHydrate, props);\n          break;\n        case \"textarea\":\n          checkControlledValueProps(\"textarea\", props),\n            listenToNonDelegatedEvent(\"invalid\", didHydrate),\n            validateTextareaProps(didHydrate, props),\n            initTextarea(\n              didHydrate,\n              props.value,\n              props.defaultValue,\n              props.children\n            ),\n            track(didHydrate);\n      }\n      type = props.children;\n      (\"string\" !== typeof type &&\n        \"number\" !== typeof type &&\n        \"bigint\" !== typeof type) ||\n      didHydrate.textContent === \"\" + type ||\n      !0 === props.suppressHydrationWarning ||\n      checkForUnmatchedText(didHydrate.textContent, type)\n        ? (null != props.popover &&\n            (listenToNonDelegatedEvent(\"beforetoggle\", didHydrate),\n            listenToNonDelegatedEvent(\"toggle\", didHydrate)),\n          null != props.onScroll &&\n            listenToNonDelegatedEvent(\"scroll\", didHydrate),\n          null != props.onScrollEnd &&\n            listenToNonDelegatedEvent(\"scrollend\", didHydrate),\n          null != props.onClick && (didHydrate.onclick = noop$1),\n          (didHydrate = !0))\n        : (didHydrate = !1);\n      didHydrate || throwOnHydrationMismatch(fiber);\n    }\n    function popToNextHostParent(fiber) {\n      for (hydrationParentFiber = fiber.return; hydrationParentFiber; )\n        switch (hydrationParentFiber.tag) {\n          case 5:\n          case 13:\n            rootOrSingletonContext = !1;\n            return;\n          case 27:\n          case 3:\n            rootOrSingletonContext = !0;\n            return;\n          default:\n            hydrationParentFiber = hydrationParentFiber.return;\n        }\n    }\n    function popHydrationState(fiber) {\n      if (fiber !== hydrationParentFiber) return !1;\n      if (!isHydrating)\n        return popToNextHostParent(fiber), (isHydrating = !0), !1;\n      var tag = fiber.tag,\n        JSCompiler_temp;\n      if ((JSCompiler_temp = 3 !== tag && 27 !== tag)) {\n        if ((JSCompiler_temp = 5 === tag))\n          (JSCompiler_temp = fiber.type),\n            (JSCompiler_temp =\n              !(\"form\" !== JSCompiler_temp && \"button\" !== JSCompiler_temp) ||\n              shouldSetTextContent(fiber.type, fiber.memoizedProps));\n        JSCompiler_temp = !JSCompiler_temp;\n      }\n      if (JSCompiler_temp && nextHydratableInstance) {\n        for (JSCompiler_temp = nextHydratableInstance; JSCompiler_temp; ) {\n          var diffNode = buildHydrationDiffNode(fiber, 0),\n            description =\n              describeHydratableInstanceForDevWarnings(JSCompiler_temp);\n          diffNode.serverTail.push(description);\n          JSCompiler_temp =\n            \"Suspense\" === description.type\n              ? getNextHydratableInstanceAfterSuspenseInstance(JSCompiler_temp)\n              : getNextHydratable(JSCompiler_temp.nextSibling);\n        }\n        throwOnHydrationMismatch(fiber);\n      }\n      popToNextHostParent(fiber);\n      if (13 === tag) {\n        fiber = fiber.memoizedState;\n        fiber = null !== fiber ? fiber.dehydrated : null;\n        if (!fiber)\n          throw Error(\n            \"Expected to have a hydrated suspense instance. This error is likely caused by a bug in React. Please file an issue.\"\n          );\n        nextHydratableInstance =\n          getNextHydratableInstanceAfterSuspenseInstance(fiber);\n      } else\n        27 === tag\n          ? ((tag = nextHydratableInstance),\n            isSingletonScope(fiber.type)\n              ? ((fiber = previousHydratableOnEnteringScopedSingleton),\n                (previousHydratableOnEnteringScopedSingleton = null),\n                (nextHydratableInstance = fiber))\n              : (nextHydratableInstance = tag))\n          : (nextHydratableInstance = hydrationParentFiber\n              ? getNextHydratable(fiber.stateNode.nextSibling)\n              : null);\n      return !0;\n    }\n    function resetHydrationState() {\n      nextHydratableInstance = hydrationParentFiber = null;\n      didSuspendOrErrorDEV = isHydrating = !1;\n    }\n    function upgradeHydrationErrorsToRecoverable() {\n      var queuedErrors = hydrationErrors;\n      null !== queuedErrors &&\n        (null === workInProgressRootRecoverableErrors\n          ? (workInProgressRootRecoverableErrors = queuedErrors)\n          : workInProgressRootRecoverableErrors.push.apply(\n              workInProgressRootRecoverableErrors,\n              queuedErrors\n            ),\n        (hydrationErrors = null));\n      return queuedErrors;\n    }\n    function queueHydrationError(error) {\n      null === hydrationErrors\n        ? (hydrationErrors = [error])\n        : hydrationErrors.push(error);\n    }\n    function emitPendingHydrationWarnings() {\n      var diffRoot = hydrationDiffRootDEV;\n      if (null !== diffRoot) {\n        hydrationDiffRootDEV = null;\n        for (var diff = describeDiff(diffRoot); 0 < diffRoot.children.length; )\n          diffRoot = diffRoot.children[0];\n        runWithFiberInDEV(diffRoot.fiber, function () {\n          console.error(\n            \"A tree hydrated but some attributes of the server rendered HTML didn't match the client properties. This won't be patched up. This can happen if a SSR-ed Client Component used:\\n\\n- A server/client branch `if (typeof window !== 'undefined')`.\\n- Variable input such as `Date.now()` or `Math.random()` which changes each time it's called.\\n- Date formatting in a user's locale which doesn't match the server.\\n- External changing data without sending a snapshot of it along with the HTML.\\n- Invalid HTML tag nesting.\\n\\nIt can also happen if the client has a browser extension installed which messes with the HTML before React loaded.\\n\\n%s%s\",\n            \"https://react.dev/link/hydration-mismatch\",\n            diff\n          );\n        });\n      }\n    }\n    function resetContextDependencies() {\n      lastContextDependency = currentlyRenderingFiber$1 = null;\n      isDisallowedContextReadInDEV = !1;\n    }\n    function pushProvider(providerFiber, context, nextValue) {\n      push(valueCursor, context._currentValue, providerFiber);\n      context._currentValue = nextValue;\n      push(rendererCursorDEV, context._currentRenderer, providerFiber);\n      void 0 !== context._currentRenderer &&\n        null !== context._currentRenderer &&\n        context._currentRenderer !== rendererSigil &&\n        console.error(\n          \"Detected multiple renderers concurrently rendering the same context provider. This is currently unsupported.\"\n        );\n      context._currentRenderer = rendererSigil;\n    }\n    function popProvider(context, providerFiber) {\n      context._currentValue = valueCursor.current;\n      var currentRenderer = rendererCursorDEV.current;\n      pop(rendererCursorDEV, providerFiber);\n      context._currentRenderer = currentRenderer;\n      pop(valueCursor, providerFiber);\n    }\n    function scheduleContextWorkOnParentPath(\n      parent,\n      renderLanes,\n      propagationRoot\n    ) {\n      for (; null !== parent; ) {\n        var alternate = parent.alternate;\n        (parent.childLanes & renderLanes) !== renderLanes\n          ? ((parent.childLanes |= renderLanes),\n            null !== alternate && (alternate.childLanes |= renderLanes))\n          : null !== alternate &&\n            (alternate.childLanes & renderLanes) !== renderLanes &&\n            (alternate.childLanes |= renderLanes);\n        if (parent === propagationRoot) break;\n        parent = parent.return;\n      }\n      parent !== propagationRoot &&\n        console.error(\n          \"Expected to find the propagation root when scheduling context work. This error is likely caused by a bug in React. Please file an issue.\"\n        );\n    }\n    function propagateContextChanges(\n      workInProgress,\n      contexts,\n      renderLanes,\n      forcePropagateEntireTree\n    ) {\n      var fiber = workInProgress.child;\n      null !== fiber && (fiber.return = workInProgress);\n      for (; null !== fiber; ) {\n        var list = fiber.dependencies;\n        if (null !== list) {\n          var nextFiber = fiber.child;\n          list = list.firstContext;\n          a: for (; null !== list; ) {\n            var dependency = list;\n            list = fiber;\n            for (var i = 0; i < contexts.length; i++)\n              if (dependency.context === contexts[i]) {\n                list.lanes |= renderLanes;\n                dependency = list.alternate;\n                null !== dependency && (dependency.lanes |= renderLanes);\n                scheduleContextWorkOnParentPath(\n                  list.return,\n                  renderLanes,\n                  workInProgress\n                );\n                forcePropagateEntireTree || (nextFiber = null);\n                break a;\n              }\n            list = dependency.next;\n          }\n        } else if (18 === fiber.tag) {\n          nextFiber = fiber.return;\n          if (null === nextFiber)\n            throw Error(\n              \"We just came from a parent so we must have had a parent. This is a bug in React.\"\n            );\n          nextFiber.lanes |= renderLanes;\n          list = nextFiber.alternate;\n          null !== list && (list.lanes |= renderLanes);\n          scheduleContextWorkOnParentPath(\n            nextFiber,\n            renderLanes,\n            workInProgress\n          );\n          nextFiber = null;\n        } else nextFiber = fiber.child;\n        if (null !== nextFiber) nextFiber.return = fiber;\n        else\n          for (nextFiber = fiber; null !== nextFiber; ) {\n            if (nextFiber === workInProgress) {\n              nextFiber = null;\n              break;\n            }\n            fiber = nextFiber.sibling;\n            if (null !== fiber) {\n              fiber.return = nextFiber.return;\n              nextFiber = fiber;\n              break;\n            }\n            nextFiber = nextFiber.return;\n          }\n        fiber = nextFiber;\n      }\n    }\n    function propagateParentContextChanges(\n      current,\n      workInProgress,\n      renderLanes,\n      forcePropagateEntireTree\n    ) {\n      current = null;\n      for (\n        var parent = workInProgress, isInsidePropagationBailout = !1;\n        null !== parent;\n\n      ) {\n        if (!isInsidePropagationBailout)\n          if (0 !== (parent.flags & 524288)) isInsidePropagationBailout = !0;\n          else if (0 !== (parent.flags & 262144)) break;\n        if (10 === parent.tag) {\n          var currentParent = parent.alternate;\n          if (null === currentParent)\n            throw Error(\"Should have a current fiber. This is a bug in React.\");\n          currentParent = currentParent.memoizedProps;\n          if (null !== currentParent) {\n            var context = parent.type;\n            objectIs(parent.pendingProps.value, currentParent.value) ||\n              (null !== current\n                ? current.push(context)\n                : (current = [context]));\n          }\n        } else if (parent === hostTransitionProviderCursor.current) {\n          currentParent = parent.alternate;\n          if (null === currentParent)\n            throw Error(\"Should have a current fiber. This is a bug in React.\");\n          currentParent.memoizedState.memoizedState !==\n            parent.memoizedState.memoizedState &&\n            (null !== current\n              ? current.push(HostTransitionContext)\n              : (current = [HostTransitionContext]));\n        }\n        parent = parent.return;\n      }\n      null !== current &&\n        propagateContextChanges(\n          workInProgress,\n          current,\n          renderLanes,\n          forcePropagateEntireTree\n        );\n      workInProgress.flags |= 262144;\n    }\n    function checkIfContextChanged(currentDependencies) {\n      for (\n        currentDependencies = currentDependencies.firstContext;\n        null !== currentDependencies;\n\n      ) {\n        if (\n          !objectIs(\n            currentDependencies.context._currentValue,\n            currentDependencies.memoizedValue\n          )\n        )\n          return !0;\n        currentDependencies = currentDependencies.next;\n      }\n      return !1;\n    }\n    function prepareToReadContext(workInProgress) {\n      currentlyRenderingFiber$1 = workInProgress;\n      lastContextDependency = null;\n      workInProgress = workInProgress.dependencies;\n      null !== workInProgress && (workInProgress.firstContext = null);\n    }\n    function readContext(context) {\n      isDisallowedContextReadInDEV &&\n        console.error(\n          \"Context can only be read while React is rendering. In classes, you can read it in the render method or getDerivedStateFromProps. In function components, you can read it directly in the function body, but not inside Hooks like useReducer() or useMemo().\"\n        );\n      return readContextForConsumer(currentlyRenderingFiber$1, context);\n    }\n    function readContextDuringReconciliation(consumer, context) {\n      null === currentlyRenderingFiber$1 && prepareToReadContext(consumer);\n      return readContextForConsumer(consumer, context);\n    }\n    function readContextForConsumer(consumer, context) {\n      var value = context._currentValue;\n      context = { context: context, memoizedValue: value, next: null };\n      if (null === lastContextDependency) {\n        if (null === consumer)\n          throw Error(\n            \"Context can only be read while React is rendering. In classes, you can read it in the render method or getDerivedStateFromProps. In function components, you can read it directly in the function body, but not inside Hooks like useReducer() or useMemo().\"\n          );\n        lastContextDependency = context;\n        consumer.dependencies = {\n          lanes: 0,\n          firstContext: context,\n          _debugThenableState: null\n        };\n        consumer.flags |= 524288;\n      } else lastContextDependency = lastContextDependency.next = context;\n      return value;\n    }\n    function createCache() {\n      return {\n        controller: new AbortControllerLocal(),\n        data: new Map(),\n        refCount: 0\n      };\n    }\n    function retainCache(cache) {\n      cache.controller.signal.aborted &&\n        console.warn(\n          \"A cache instance was retained after it was already freed. This likely indicates a bug in React.\"\n        );\n      cache.refCount++;\n    }\n    function releaseCache(cache) {\n      cache.refCount--;\n      0 > cache.refCount &&\n        console.warn(\n          \"A cache instance was released after it was already freed. This likely indicates a bug in React.\"\n        );\n      0 === cache.refCount &&\n        scheduleCallback$2(NormalPriority, function () {\n          cache.controller.abort();\n        });\n    }\n    function pushNestedEffectDurations() {\n      var prevEffectDuration = profilerEffectDuration;\n      profilerEffectDuration = 0;\n      return prevEffectDuration;\n    }\n    function popNestedEffectDurations(prevEffectDuration) {\n      var elapsedTime = profilerEffectDuration;\n      profilerEffectDuration = prevEffectDuration;\n      return elapsedTime;\n    }\n    function bubbleNestedEffectDurations(prevEffectDuration) {\n      var elapsedTime = profilerEffectDuration;\n      profilerEffectDuration += prevEffectDuration;\n      return elapsedTime;\n    }\n    function startProfilerTimer(fiber) {\n      profilerStartTime = now();\n      0 > fiber.actualStartTime && (fiber.actualStartTime = profilerStartTime);\n    }\n    function stopProfilerTimerIfRunningAndRecordDuration(fiber) {\n      if (0 <= profilerStartTime) {\n        var elapsedTime = now() - profilerStartTime;\n        fiber.actualDuration += elapsedTime;\n        fiber.selfBaseDuration = elapsedTime;\n        profilerStartTime = -1;\n      }\n    }\n    function stopProfilerTimerIfRunningAndRecordIncompleteDuration(fiber) {\n      if (0 <= profilerStartTime) {\n        var elapsedTime = now() - profilerStartTime;\n        fiber.actualDuration += elapsedTime;\n        profilerStartTime = -1;\n      }\n    }\n    function recordEffectDuration() {\n      if (0 <= profilerStartTime) {\n        var elapsedTime = now() - profilerStartTime;\n        profilerStartTime = -1;\n        profilerEffectDuration += elapsedTime;\n      }\n    }\n    function startEffectTimer() {\n      profilerStartTime = now();\n    }\n    function transferActualDuration(fiber) {\n      for (var child = fiber.child; child; )\n        (fiber.actualDuration += child.actualDuration), (child = child.sibling);\n    }\n    function entangleAsyncAction(transition, thenable) {\n      if (null === currentEntangledListeners) {\n        var entangledListeners = (currentEntangledListeners = []);\n        currentEntangledPendingCount = 0;\n        currentEntangledLane = requestTransitionLane();\n        currentEntangledActionThenable = {\n          status: \"pending\",\n          value: void 0,\n          then: function (resolve) {\n            entangledListeners.push(resolve);\n          }\n        };\n      }\n      currentEntangledPendingCount++;\n      thenable.then(pingEngtangledActionScope, pingEngtangledActionScope);\n      return thenable;\n    }\n    function pingEngtangledActionScope() {\n      if (\n        0 === --currentEntangledPendingCount &&\n        null !== currentEntangledListeners\n      ) {\n        null !== currentEntangledActionThenable &&\n          (currentEntangledActionThenable.status = \"fulfilled\");\n        var listeners = currentEntangledListeners;\n        currentEntangledListeners = null;\n        currentEntangledLane = 0;\n        currentEntangledActionThenable = null;\n        for (var i = 0; i < listeners.length; i++) (0, listeners[i])();\n      }\n    }\n    function chainThenableValue(thenable, result) {\n      var listeners = [],\n        thenableWithOverride = {\n          status: \"pending\",\n          value: null,\n          reason: null,\n          then: function (resolve) {\n            listeners.push(resolve);\n          }\n        };\n      thenable.then(\n        function () {\n          thenableWithOverride.status = \"fulfilled\";\n          thenableWithOverride.value = result;\n          for (var i = 0; i < listeners.length; i++) (0, listeners[i])(result);\n        },\n        function (error) {\n          thenableWithOverride.status = \"rejected\";\n          thenableWithOverride.reason = error;\n          for (error = 0; error < listeners.length; error++)\n            (0, listeners[error])(void 0);\n        }\n      );\n      return thenableWithOverride;\n    }\n    function peekCacheFromPool() {\n      var cacheResumedFromPreviousRender = resumedCache.current;\n      return null !== cacheResumedFromPreviousRender\n        ? cacheResumedFromPreviousRender\n        : workInProgressRoot.pooledCache;\n    }\n    function pushTransition(offscreenWorkInProgress, prevCachePool) {\n      null === prevCachePool\n        ? push(resumedCache, resumedCache.current, offscreenWorkInProgress)\n        : push(resumedCache, prevCachePool.pool, offscreenWorkInProgress);\n    }\n    function getSuspendedCache() {\n      var cacheFromPool = peekCacheFromPool();\n      return null === cacheFromPool\n        ? null\n        : { parent: CacheContext._currentValue, pool: cacheFromPool };\n    }\n    function createThenableState() {\n      return { didWarnAboutUncachedPromise: !1, thenables: [] };\n    }\n    function isThenableResolved(thenable) {\n      thenable = thenable.status;\n      return \"fulfilled\" === thenable || \"rejected\" === thenable;\n    }\n    function noop$3() {}\n    function trackUsedThenable(thenableState, thenable, index) {\n      null !== ReactSharedInternals.actQueue &&\n        (ReactSharedInternals.didUsePromise = !0);\n      var trackedThenables = thenableState.thenables;\n      index = trackedThenables[index];\n      void 0 === index\n        ? trackedThenables.push(thenable)\n        : index !== thenable &&\n          (thenableState.didWarnAboutUncachedPromise ||\n            ((thenableState.didWarnAboutUncachedPromise = !0),\n            console.error(\n              \"A component was suspended by an uncached promise. Creating promises inside a Client Component or hook is not yet supported, except via a Suspense-compatible library or framework.\"\n            )),\n          thenable.then(noop$3, noop$3),\n          (thenable = index));\n      switch (thenable.status) {\n        case \"fulfilled\":\n          return thenable.value;\n        case \"rejected\":\n          throw (\n            ((thenableState = thenable.reason),\n            checkIfUseWrappedInAsyncCatch(thenableState),\n            thenableState)\n          );\n        default:\n          if (\"string\" === typeof thenable.status)\n            thenable.then(noop$3, noop$3);\n          else {\n            thenableState = workInProgressRoot;\n            if (\n              null !== thenableState &&\n              100 < thenableState.shellSuspendCounter\n            )\n              throw Error(\n                \"An unknown Component is an async Client Component. Only Server Components can be async at the moment. This error is often caused by accidentally adding `'use client'` to a module that was originally written for the server.\"\n              );\n            thenableState = thenable;\n            thenableState.status = \"pending\";\n            thenableState.then(\n              function (fulfilledValue) {\n                if (\"pending\" === thenable.status) {\n                  var fulfilledThenable = thenable;\n                  fulfilledThenable.status = \"fulfilled\";\n                  fulfilledThenable.value = fulfilledValue;\n                }\n              },\n              function (error) {\n                if (\"pending\" === thenable.status) {\n                  var rejectedThenable = thenable;\n                  rejectedThenable.status = \"rejected\";\n                  rejectedThenable.reason = error;\n                }\n              }\n            );\n          }\n          switch (thenable.status) {\n            case \"fulfilled\":\n              return thenable.value;\n            case \"rejected\":\n              throw (\n                ((thenableState = thenable.reason),\n                checkIfUseWrappedInAsyncCatch(thenableState),\n                thenableState)\n              );\n          }\n          suspendedThenable = thenable;\n          needsToResetSuspendedThenableDEV = !0;\n          throw SuspenseException;\n      }\n    }\n    function getSuspendedThenable() {\n      if (null === suspendedThenable)\n        throw Error(\n          \"Expected a suspended thenable. This is a bug in React. Please file an issue.\"\n        );\n      var thenable = suspendedThenable;\n      suspendedThenable = null;\n      needsToResetSuspendedThenableDEV = !1;\n      return thenable;\n    }\n    function checkIfUseWrappedInAsyncCatch(rejectedReason) {\n      if (\n        rejectedReason === SuspenseException ||\n        rejectedReason === SuspenseActionException\n      )\n        throw Error(\n          \"Hooks are not supported inside an async component. This error is often caused by accidentally adding `'use client'` to a module that was originally written for the server.\"\n        );\n    }\n    function initializeUpdateQueue(fiber) {\n      fiber.updateQueue = {\n        baseState: fiber.memoizedState,\n        firstBaseUpdate: null,\n        lastBaseUpdate: null,\n        shared: { pending: null, lanes: 0, hiddenCallbacks: null },\n        callbacks: null\n      };\n    }\n    function cloneUpdateQueue(current, workInProgress) {\n      current = current.updateQueue;\n      workInProgress.updateQueue === current &&\n        (workInProgress.updateQueue = {\n          baseState: current.baseState,\n          firstBaseUpdate: current.firstBaseUpdate,\n          lastBaseUpdate: current.lastBaseUpdate,\n          shared: current.shared,\n          callbacks: null\n        });\n    }\n    function createUpdate(lane) {\n      return {\n        lane: lane,\n        tag: UpdateState,\n        payload: null,\n        callback: null,\n        next: null\n      };\n    }\n    function enqueueUpdate(fiber, update, lane) {\n      var updateQueue = fiber.updateQueue;\n      if (null === updateQueue) return null;\n      updateQueue = updateQueue.shared;\n      if (\n        currentlyProcessingQueue === updateQueue &&\n        !didWarnUpdateInsideUpdate\n      ) {\n        var componentName = getComponentNameFromFiber(fiber);\n        console.error(\n          \"An update (setState, replaceState, or forceUpdate) was scheduled from inside an update function. Update functions should be pure, with zero side-effects. Consider using componentDidUpdate or a callback.\\n\\nPlease update the following component: %s\",\n          componentName\n        );\n        didWarnUpdateInsideUpdate = !0;\n      }\n      if ((executionContext & RenderContext) !== NoContext)\n        return (\n          (componentName = updateQueue.pending),\n          null === componentName\n            ? (update.next = update)\n            : ((update.next = componentName.next),\n              (componentName.next = update)),\n          (updateQueue.pending = update),\n          (update = getRootForUpdatedFiber(fiber)),\n          markUpdateLaneFromFiberToRoot(fiber, null, lane),\n          update\n        );\n      enqueueUpdate$1(fiber, updateQueue, update, lane);\n      return getRootForUpdatedFiber(fiber);\n    }\n    function entangleTransitions(root, fiber, lane) {\n      fiber = fiber.updateQueue;\n      if (null !== fiber && ((fiber = fiber.shared), 0 !== (lane & 4194048))) {\n        var queueLanes = fiber.lanes;\n        queueLanes &= root.pendingLanes;\n        lane |= queueLanes;\n        fiber.lanes = lane;\n        markRootEntangled(root, lane);\n      }\n    }\n    function enqueueCapturedUpdate(workInProgress, capturedUpdate) {\n      var queue = workInProgress.updateQueue,\n        current = workInProgress.alternate;\n      if (\n        null !== current &&\n        ((current = current.updateQueue), queue === current)\n      ) {\n        var newFirst = null,\n          newLast = null;\n        queue = queue.firstBaseUpdate;\n        if (null !== queue) {\n          do {\n            var clone = {\n              lane: queue.lane,\n              tag: queue.tag,\n              payload: queue.payload,\n              callback: null,\n              next: null\n            };\n            null === newLast\n              ? (newFirst = newLast = clone)\n              : (newLast = newLast.next = clone);\n            queue = queue.next;\n          } while (null !== queue);\n          null === newLast\n            ? (newFirst = newLast = capturedUpdate)\n            : (newLast = newLast.next = capturedUpdate);\n        } else newFirst = newLast = capturedUpdate;\n        queue = {\n          baseState: current.baseState,\n          firstBaseUpdate: newFirst,\n          lastBaseUpdate: newLast,\n          shared: current.shared,\n          callbacks: current.callbacks\n        };\n        workInProgress.updateQueue = queue;\n        return;\n      }\n      workInProgress = queue.lastBaseUpdate;\n      null === workInProgress\n        ? (queue.firstBaseUpdate = capturedUpdate)\n        : (workInProgress.next = capturedUpdate);\n      queue.lastBaseUpdate = capturedUpdate;\n    }\n    function suspendIfUpdateReadFromEntangledAsyncAction() {\n      if (didReadFromEntangledAsyncAction) {\n        var entangledActionThenable = currentEntangledActionThenable;\n        if (null !== entangledActionThenable) throw entangledActionThenable;\n      }\n    }\n    function processUpdateQueue(\n      workInProgress,\n      props,\n      instance$jscomp$0,\n      renderLanes\n    ) {\n      didReadFromEntangledAsyncAction = !1;\n      var queue = workInProgress.updateQueue;\n      hasForceUpdate = !1;\n      currentlyProcessingQueue = queue.shared;\n      var firstBaseUpdate = queue.firstBaseUpdate,\n        lastBaseUpdate = queue.lastBaseUpdate,\n        pendingQueue = queue.shared.pending;\n      if (null !== pendingQueue) {\n        queue.shared.pending = null;\n        var lastPendingUpdate = pendingQueue,\n          firstPendingUpdate = lastPendingUpdate.next;\n        lastPendingUpdate.next = null;\n        null === lastBaseUpdate\n          ? (firstBaseUpdate = firstPendingUpdate)\n          : (lastBaseUpdate.next = firstPendingUpdate);\n        lastBaseUpdate = lastPendingUpdate;\n        var current = workInProgress.alternate;\n        null !== current &&\n          ((current = current.updateQueue),\n          (pendingQueue = current.lastBaseUpdate),\n          pendingQueue !== lastBaseUpdate &&\n            (null === pendingQueue\n              ? (current.firstBaseUpdate = firstPendingUpdate)\n              : (pendingQueue.next = firstPendingUpdate),\n            (current.lastBaseUpdate = lastPendingUpdate)));\n      }\n      if (null !== firstBaseUpdate) {\n        var newState = queue.baseState;\n        lastBaseUpdate = 0;\n        current = firstPendingUpdate = lastPendingUpdate = null;\n        pendingQueue = firstBaseUpdate;\n        do {\n          var updateLane = pendingQueue.lane & -536870913,\n            isHiddenUpdate = updateLane !== pendingQueue.lane;\n          if (\n            isHiddenUpdate\n              ? (workInProgressRootRenderLanes & updateLane) === updateLane\n              : (renderLanes & updateLane) === updateLane\n          ) {\n            0 !== updateLane &&\n              updateLane === currentEntangledLane &&\n              (didReadFromEntangledAsyncAction = !0);\n            null !== current &&\n              (current = current.next =\n                {\n                  lane: 0,\n                  tag: pendingQueue.tag,\n                  payload: pendingQueue.payload,\n                  callback: null,\n                  next: null\n                });\n            a: {\n              updateLane = workInProgress;\n              var partialState = pendingQueue;\n              var nextProps = props,\n                instance = instance$jscomp$0;\n              switch (partialState.tag) {\n                case ReplaceState:\n                  partialState = partialState.payload;\n                  if (\"function\" === typeof partialState) {\n                    isDisallowedContextReadInDEV = !0;\n                    var nextState = partialState.call(\n                      instance,\n                      newState,\n                      nextProps\n                    );\n                    if (updateLane.mode & StrictLegacyMode) {\n                      setIsStrictModeForDevtools(!0);\n                      try {\n                        partialState.call(instance, newState, nextProps);\n                      } finally {\n                        setIsStrictModeForDevtools(!1);\n                      }\n                    }\n                    isDisallowedContextReadInDEV = !1;\n                    newState = nextState;\n                    break a;\n                  }\n                  newState = partialState;\n                  break a;\n                case CaptureUpdate:\n                  updateLane.flags = (updateLane.flags & -65537) | 128;\n                case UpdateState:\n                  nextState = partialState.payload;\n                  if (\"function\" === typeof nextState) {\n                    isDisallowedContextReadInDEV = !0;\n                    partialState = nextState.call(\n                      instance,\n                      newState,\n                      nextProps\n                    );\n                    if (updateLane.mode & StrictLegacyMode) {\n                      setIsStrictModeForDevtools(!0);\n                      try {\n                        nextState.call(instance, newState, nextProps);\n                      } finally {\n                        setIsStrictModeForDevtools(!1);\n                      }\n                    }\n                    isDisallowedContextReadInDEV = !1;\n                  } else partialState = nextState;\n                  if (null === partialState || void 0 === partialState) break a;\n                  newState = assign({}, newState, partialState);\n                  break a;\n                case ForceUpdate:\n                  hasForceUpdate = !0;\n              }\n            }\n            updateLane = pendingQueue.callback;\n            null !== updateLane &&\n              ((workInProgress.flags |= 64),\n              isHiddenUpdate && (workInProgress.flags |= 8192),\n              (isHiddenUpdate = queue.callbacks),\n              null === isHiddenUpdate\n                ? (queue.callbacks = [updateLane])\n                : isHiddenUpdate.push(updateLane));\n          } else\n            (isHiddenUpdate = {\n              lane: updateLane,\n              tag: pendingQueue.tag,\n              payload: pendingQueue.payload,\n              callback: pendingQueue.callback,\n              next: null\n            }),\n              null === current\n                ? ((firstPendingUpdate = current = isHiddenUpdate),\n                  (lastPendingUpdate = newState))\n                : (current = current.next = isHiddenUpdate),\n              (lastBaseUpdate |= updateLane);\n          pendingQueue = pendingQueue.next;\n          if (null === pendingQueue)\n            if (((pendingQueue = queue.shared.pending), null === pendingQueue))\n              break;\n            else\n              (isHiddenUpdate = pendingQueue),\n                (pendingQueue = isHiddenUpdate.next),\n                (isHiddenUpdate.next = null),\n                (queue.lastBaseUpdate = isHiddenUpdate),\n                (queue.shared.pending = null);\n        } while (1);\n        null === current && (lastPendingUpdate = newState);\n        queue.baseState = lastPendingUpdate;\n        queue.firstBaseUpdate = firstPendingUpdate;\n        queue.lastBaseUpdate = current;\n        null === firstBaseUpdate && (queue.shared.lanes = 0);\n        workInProgressRootSkippedLanes |= lastBaseUpdate;\n        workInProgress.lanes = lastBaseUpdate;\n        workInProgress.memoizedState = newState;\n      }\n      currentlyProcessingQueue = null;\n    }\n    function callCallback(callback, context) {\n      if (\"function\" !== typeof callback)\n        throw Error(\n          \"Invalid argument passed as callback. Expected a function. Instead received: \" +\n            callback\n        );\n      callback.call(context);\n    }\n    function commitHiddenCallbacks(updateQueue, context) {\n      var hiddenCallbacks = updateQueue.shared.hiddenCallbacks;\n      if (null !== hiddenCallbacks)\n        for (\n          updateQueue.shared.hiddenCallbacks = null, updateQueue = 0;\n          updateQueue < hiddenCallbacks.length;\n          updateQueue++\n        )\n          callCallback(hiddenCallbacks[updateQueue], context);\n    }\n    function commitCallbacks(updateQueue, context) {\n      var callbacks = updateQueue.callbacks;\n      if (null !== callbacks)\n        for (\n          updateQueue.callbacks = null, updateQueue = 0;\n          updateQueue < callbacks.length;\n          updateQueue++\n        )\n          callCallback(callbacks[updateQueue], context);\n    }\n    function pushHiddenContext(fiber, context) {\n      var prevEntangledRenderLanes = entangledRenderLanes;\n      push(prevEntangledRenderLanesCursor, prevEntangledRenderLanes, fiber);\n      push(currentTreeHiddenStackCursor, context, fiber);\n      entangledRenderLanes = prevEntangledRenderLanes | context.baseLanes;\n    }\n    function reuseHiddenContextOnStack(fiber) {\n      push(prevEntangledRenderLanesCursor, entangledRenderLanes, fiber);\n      push(\n        currentTreeHiddenStackCursor,\n        currentTreeHiddenStackCursor.current,\n        fiber\n      );\n    }\n    function popHiddenContext(fiber) {\n      entangledRenderLanes = prevEntangledRenderLanesCursor.current;\n      pop(currentTreeHiddenStackCursor, fiber);\n      pop(prevEntangledRenderLanesCursor, fiber);\n    }\n    function mountHookTypesDev() {\n      var hookName = currentHookNameInDev;\n      null === hookTypesDev\n        ? (hookTypesDev = [hookName])\n        : hookTypesDev.push(hookName);\n    }\n    function updateHookTypesDev() {\n      var hookName = currentHookNameInDev;\n      if (\n        null !== hookTypesDev &&\n        (hookTypesUpdateIndexDev++,\n        hookTypesDev[hookTypesUpdateIndexDev] !== hookName)\n      ) {\n        var componentName = getComponentNameFromFiber(currentlyRenderingFiber);\n        if (\n          !didWarnAboutMismatchedHooksForComponent.has(componentName) &&\n          (didWarnAboutMismatchedHooksForComponent.add(componentName),\n          null !== hookTypesDev)\n        ) {\n          for (var table = \"\", i = 0; i <= hookTypesUpdateIndexDev; i++) {\n            var oldHookName = hookTypesDev[i],\n              newHookName =\n                i === hookTypesUpdateIndexDev ? hookName : oldHookName;\n            for (\n              oldHookName = i + 1 + \". \" + oldHookName;\n              30 > oldHookName.length;\n\n            )\n              oldHookName += \" \";\n            oldHookName += newHookName + \"\\n\";\n            table += oldHookName;\n          }\n          console.error(\n            \"React has detected a change in the order of Hooks called by %s. This will lead to bugs and errors if not fixed. For more information, read the Rules of Hooks: https://react.dev/link/rules-of-hooks\\n\\n   Previous render            Next render\\n   ------------------------------------------------------\\n%s   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\\n\",\n            componentName,\n            table\n          );\n        }\n      }\n    }\n    function checkDepsAreArrayDev(deps) {\n      void 0 === deps ||\n        null === deps ||\n        isArrayImpl(deps) ||\n        console.error(\n          \"%s received a final argument that is not an array (instead, received `%s`). When specified, the final argument must be an array.\",\n          currentHookNameInDev,\n          typeof deps\n        );\n    }\n    function warnOnUseFormStateInDev() {\n      var componentName = getComponentNameFromFiber(currentlyRenderingFiber);\n      didWarnAboutUseFormState.has(componentName) ||\n        (didWarnAboutUseFormState.add(componentName),\n        console.error(\n          \"ReactDOM.useFormState has been renamed to React.useActionState. Please update %s to use React.useActionState.\",\n          componentName\n        ));\n    }\n    function throwInvalidHookError() {\n      throw Error(\n        \"Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:\\n1. You might have mismatching versions of React and the renderer (such as React DOM)\\n2. You might be breaking the Rules of Hooks\\n3. You might have more than one copy of React in the same app\\nSee https://react.dev/link/invalid-hook-call for tips about how to debug and fix this problem.\"\n      );\n    }\n    function areHookInputsEqual(nextDeps, prevDeps) {\n      if (ignorePreviousDependencies) return !1;\n      if (null === prevDeps)\n        return (\n          console.error(\n            \"%s received a final argument during this render, but not during the previous render. Even though the final argument is optional, its type cannot change between renders.\",\n            currentHookNameInDev\n          ),\n          !1\n        );\n      nextDeps.length !== prevDeps.length &&\n        console.error(\n          \"The final argument passed to %s changed size between renders. The order and size of this array must remain constant.\\n\\nPrevious: %s\\nIncoming: %s\",\n          currentHookNameInDev,\n          \"[\" + prevDeps.join(\", \") + \"]\",\n          \"[\" + nextDeps.join(\", \") + \"]\"\n        );\n      for (var i = 0; i < prevDeps.length && i < nextDeps.length; i++)\n        if (!objectIs(nextDeps[i], prevDeps[i])) return !1;\n      return !0;\n    }\n    function renderWithHooks(\n      current,\n      workInProgress,\n      Component,\n      props,\n      secondArg,\n      nextRenderLanes\n    ) {\n      renderLanes = nextRenderLanes;\n      currentlyRenderingFiber = workInProgress;\n      hookTypesDev = null !== current ? current._debugHookTypes : null;\n      hookTypesUpdateIndexDev = -1;\n      ignorePreviousDependencies =\n        null !== current && current.type !== workInProgress.type;\n      if (\n        \"[object AsyncFunction]\" ===\n          Object.prototype.toString.call(Component) ||\n        \"[object AsyncGeneratorFunction]\" ===\n          Object.prototype.toString.call(Component)\n      )\n        (nextRenderLanes = getComponentNameFromFiber(currentlyRenderingFiber)),\n          didWarnAboutAsyncClientComponent.has(nextRenderLanes) ||\n            (didWarnAboutAsyncClientComponent.add(nextRenderLanes),\n            console.error(\n              \"%s is an async Client Component. Only Server Components can be async at the moment. This error is often caused by accidentally adding `'use client'` to a module that was originally written for the server.\",\n              null === nextRenderLanes\n                ? \"An unknown Component\"\n                : \"<\" + nextRenderLanes + \">\"\n            ));\n      workInProgress.memoizedState = null;\n      workInProgress.updateQueue = null;\n      workInProgress.lanes = 0;\n      ReactSharedInternals.H =\n        null !== current && null !== current.memoizedState\n          ? HooksDispatcherOnUpdateInDEV\n          : null !== hookTypesDev\n            ? HooksDispatcherOnMountWithHookTypesInDEV\n            : HooksDispatcherOnMountInDEV;\n      shouldDoubleInvokeUserFnsInHooksDEV = nextRenderLanes =\n        (workInProgress.mode & StrictLegacyMode) !== NoMode;\n      var children = callComponentInDEV(Component, props, secondArg);\n      shouldDoubleInvokeUserFnsInHooksDEV = !1;\n      didScheduleRenderPhaseUpdateDuringThisPass &&\n        (children = renderWithHooksAgain(\n          workInProgress,\n          Component,\n          props,\n          secondArg\n        ));\n      if (nextRenderLanes) {\n        setIsStrictModeForDevtools(!0);\n        try {\n          children = renderWithHooksAgain(\n            workInProgress,\n            Component,\n            props,\n            secondArg\n          );\n        } finally {\n          setIsStrictModeForDevtools(!1);\n        }\n      }\n      finishRenderingHooks(current, workInProgress);\n      return children;\n    }\n    function finishRenderingHooks(current, workInProgress) {\n      workInProgress._debugHookTypes = hookTypesDev;\n      null === workInProgress.dependencies\n        ? null !== thenableState$1 &&\n          (workInProgress.dependencies = {\n            lanes: 0,\n            firstContext: null,\n            _debugThenableState: thenableState$1\n          })\n        : (workInProgress.dependencies._debugThenableState = thenableState$1);\n      ReactSharedInternals.H = ContextOnlyDispatcher;\n      var didRenderTooFewHooks =\n        null !== currentHook && null !== currentHook.next;\n      renderLanes = 0;\n      hookTypesDev =\n        currentHookNameInDev =\n        workInProgressHook =\n        currentHook =\n        currentlyRenderingFiber =\n          null;\n      hookTypesUpdateIndexDev = -1;\n      null !== current &&\n        (current.flags & 65011712) !== (workInProgress.flags & 65011712) &&\n        console.error(\n          \"Internal React error: Expected static flag was missing. Please notify the React team.\"\n        );\n      didScheduleRenderPhaseUpdate = !1;\n      thenableIndexCounter$1 = 0;\n      thenableState$1 = null;\n      if (didRenderTooFewHooks)\n        throw Error(\n          \"Rendered fewer hooks than expected. This may be caused by an accidental early return statement.\"\n        );\n      null === current ||\n        didReceiveUpdate ||\n        ((current = current.dependencies),\n        null !== current &&\n          checkIfContextChanged(current) &&\n          (didReceiveUpdate = !0));\n      needsToResetSuspendedThenableDEV\n        ? ((needsToResetSuspendedThenableDEV = !1), (current = !0))\n        : (current = !1);\n      current &&\n        ((workInProgress =\n          getComponentNameFromFiber(workInProgress) || \"Unknown\"),\n        didWarnAboutUseWrappedInTryCatch.has(workInProgress) ||\n          didWarnAboutAsyncClientComponent.has(workInProgress) ||\n          (didWarnAboutUseWrappedInTryCatch.add(workInProgress),\n          console.error(\n            \"`use` was called from inside a try/catch block. This is not allowed and can lead to unexpected behavior. To handle errors triggered by `use`, wrap your component in a error boundary.\"\n          )));\n    }\n    function renderWithHooksAgain(workInProgress, Component, props, secondArg) {\n      currentlyRenderingFiber = workInProgress;\n      var numberOfReRenders = 0;\n      do {\n        didScheduleRenderPhaseUpdateDuringThisPass && (thenableState$1 = null);\n        thenableIndexCounter$1 = 0;\n        didScheduleRenderPhaseUpdateDuringThisPass = !1;\n        if (numberOfReRenders >= RE_RENDER_LIMIT)\n          throw Error(\n            \"Too many re-renders. React limits the number of renders to prevent an infinite loop.\"\n          );\n        numberOfReRenders += 1;\n        ignorePreviousDependencies = !1;\n        workInProgressHook = currentHook = null;\n        if (null != workInProgress.updateQueue) {\n          var children = workInProgress.updateQueue;\n          children.lastEffect = null;\n          children.events = null;\n          children.stores = null;\n          null != children.memoCache && (children.memoCache.index = 0);\n        }\n        hookTypesUpdateIndexDev = -1;\n        ReactSharedInternals.H = HooksDispatcherOnRerenderInDEV;\n        children = callComponentInDEV(Component, props, secondArg);\n      } while (didScheduleRenderPhaseUpdateDuringThisPass);\n      return children;\n    }\n    function TransitionAwareHostComponent() {\n      var dispatcher = ReactSharedInternals.H,\n        maybeThenable = dispatcher.useState()[0];\n      maybeThenable =\n        \"function\" === typeof maybeThenable.then\n          ? useThenable(maybeThenable)\n          : maybeThenable;\n      dispatcher = dispatcher.useState()[0];\n      (null !== currentHook ? currentHook.memoizedState : null) !==\n        dispatcher && (currentlyRenderingFiber.flags |= 1024);\n      return maybeThenable;\n    }\n    function checkDidRenderIdHook() {\n      var didRenderIdHook = 0 !== localIdCounter;\n      localIdCounter = 0;\n      return didRenderIdHook;\n    }\n    function bailoutHooks(current, workInProgress, lanes) {\n      workInProgress.updateQueue = current.updateQueue;\n      workInProgress.flags =\n        (workInProgress.mode & StrictEffectsMode) !== NoMode\n          ? workInProgress.flags & -402655237\n          : workInProgress.flags & -2053;\n      current.lanes &= ~lanes;\n    }\n    function resetHooksOnUnwind(workInProgress) {\n      if (didScheduleRenderPhaseUpdate) {\n        for (\n          workInProgress = workInProgress.memoizedState;\n          null !== workInProgress;\n\n        ) {\n          var queue = workInProgress.queue;\n          null !== queue && (queue.pending = null);\n          workInProgress = workInProgress.next;\n        }\n        didScheduleRenderPhaseUpdate = !1;\n      }\n      renderLanes = 0;\n      hookTypesDev =\n        workInProgressHook =\n        currentHook =\n        currentlyRenderingFiber =\n          null;\n      hookTypesUpdateIndexDev = -1;\n      currentHookNameInDev = null;\n      didScheduleRenderPhaseUpdateDuringThisPass = !1;\n      thenableIndexCounter$1 = localIdCounter = 0;\n      thenableState$1 = null;\n    }\n    function mountWorkInProgressHook() {\n      var hook = {\n        memoizedState: null,\n        baseState: null,\n        baseQueue: null,\n        queue: null,\n        next: null\n      };\n      null === workInProgressHook\n        ? (currentlyRenderingFiber.memoizedState = workInProgressHook = hook)\n        : (workInProgressHook = workInProgressHook.next = hook);\n      return workInProgressHook;\n    }\n    function updateWorkInProgressHook() {\n      if (null === currentHook) {\n        var nextCurrentHook = currentlyRenderingFiber.alternate;\n        nextCurrentHook =\n          null !== nextCurrentHook ? nextCurrentHook.memoizedState : null;\n      } else nextCurrentHook = currentHook.next;\n      var nextWorkInProgressHook =\n        null === workInProgressHook\n          ? currentlyRenderingFiber.memoizedState\n          : workInProgressHook.next;\n      if (null !== nextWorkInProgressHook)\n        (workInProgressHook = nextWorkInProgressHook),\n          (currentHook = nextCurrentHook);\n      else {\n        if (null === nextCurrentHook) {\n          if (null === currentlyRenderingFiber.alternate)\n            throw Error(\n              \"Update hook called on initial render. This is likely a bug in React. Please file an issue.\"\n            );\n          throw Error(\"Rendered more hooks than during the previous render.\");\n        }\n        currentHook = nextCurrentHook;\n        nextCurrentHook = {\n          memoizedState: currentHook.memoizedState,\n          baseState: currentHook.baseState,\n          baseQueue: currentHook.baseQueue,\n          queue: currentHook.queue,\n          next: null\n        };\n        null === workInProgressHook\n          ? (currentlyRenderingFiber.memoizedState = workInProgressHook =\n              nextCurrentHook)\n          : (workInProgressHook = workInProgressHook.next = nextCurrentHook);\n      }\n      return workInProgressHook;\n    }\n    function createFunctionComponentUpdateQueue() {\n      return { lastEffect: null, events: null, stores: null, memoCache: null };\n    }\n    function useThenable(thenable) {\n      var index = thenableIndexCounter$1;\n      thenableIndexCounter$1 += 1;\n      null === thenableState$1 && (thenableState$1 = createThenableState());\n      thenable = trackUsedThenable(thenableState$1, thenable, index);\n      index = currentlyRenderingFiber;\n      null ===\n        (null === workInProgressHook\n          ? index.memoizedState\n          : workInProgressHook.next) &&\n        ((index = index.alternate),\n        (ReactSharedInternals.H =\n          null !== index && null !== index.memoizedState\n            ? HooksDispatcherOnUpdateInDEV\n            : HooksDispatcherOnMountInDEV));\n      return thenable;\n    }\n    function use(usable) {\n      if (null !== usable && \"object\" === typeof usable) {\n        if (\"function\" === typeof usable.then) return useThenable(usable);\n        if (usable.$$typeof === REACT_CONTEXT_TYPE) return readContext(usable);\n      }\n      throw Error(\"An unsupported type was passed to use(): \" + String(usable));\n    }\n    function useMemoCache(size) {\n      var memoCache = null,\n        updateQueue = currentlyRenderingFiber.updateQueue;\n      null !== updateQueue && (memoCache = updateQueue.memoCache);\n      if (null == memoCache) {\n        var current = currentlyRenderingFiber.alternate;\n        null !== current &&\n          ((current = current.updateQueue),\n          null !== current &&\n            ((current = current.memoCache),\n            null != current &&\n              (memoCache = {\n                data: current.data.map(function (array) {\n                  return array.slice();\n                }),\n                index: 0\n              })));\n      }\n      null == memoCache && (memoCache = { data: [], index: 0 });\n      null === updateQueue &&\n        ((updateQueue = createFunctionComponentUpdateQueue()),\n        (currentlyRenderingFiber.updateQueue = updateQueue));\n      updateQueue.memoCache = memoCache;\n      updateQueue = memoCache.data[memoCache.index];\n      if (void 0 === updateQueue || ignorePreviousDependencies)\n        for (\n          updateQueue = memoCache.data[memoCache.index] = Array(size),\n            current = 0;\n          current < size;\n          current++\n        )\n          updateQueue[current] = REACT_MEMO_CACHE_SENTINEL;\n      else\n        updateQueue.length !== size &&\n          console.error(\n            \"Expected a constant size argument for each invocation of useMemoCache. The previous cache was allocated with size %s but size %s was requested.\",\n            updateQueue.length,\n            size\n          );\n      memoCache.index++;\n      return updateQueue;\n    }\n    function basicStateReducer(state, action) {\n      return \"function\" === typeof action ? action(state) : action;\n    }\n    function mountReducer(reducer, initialArg, init) {\n      var hook = mountWorkInProgressHook();\n      if (void 0 !== init) {\n        var initialState = init(initialArg);\n        if (shouldDoubleInvokeUserFnsInHooksDEV) {\n          setIsStrictModeForDevtools(!0);\n          try {\n            init(initialArg);\n          } finally {\n            setIsStrictModeForDevtools(!1);\n          }\n        }\n      } else initialState = initialArg;\n      hook.memoizedState = hook.baseState = initialState;\n      reducer = {\n        pending: null,\n        lanes: 0,\n        dispatch: null,\n        lastRenderedReducer: reducer,\n        lastRenderedState: initialState\n      };\n      hook.queue = reducer;\n      reducer = reducer.dispatch = dispatchReducerAction.bind(\n        null,\n        currentlyRenderingFiber,\n        reducer\n      );\n      return [hook.memoizedState, reducer];\n    }\n    function updateReducer(reducer) {\n      var hook = updateWorkInProgressHook();\n      return updateReducerImpl(hook, currentHook, reducer);\n    }\n    function updateReducerImpl(hook, current, reducer) {\n      var queue = hook.queue;\n      if (null === queue)\n        throw Error(\n          \"Should have a queue. You are likely calling Hooks conditionally, which is not allowed. (https://react.dev/link/invalid-hook-call)\"\n        );\n      queue.lastRenderedReducer = reducer;\n      var baseQueue = hook.baseQueue,\n        pendingQueue = queue.pending;\n      if (null !== pendingQueue) {\n        if (null !== baseQueue) {\n          var baseFirst = baseQueue.next;\n          baseQueue.next = pendingQueue.next;\n          pendingQueue.next = baseFirst;\n        }\n        current.baseQueue !== baseQueue &&\n          console.error(\n            \"Internal error: Expected work-in-progress queue to be a clone. This is a bug in React.\"\n          );\n        current.baseQueue = baseQueue = pendingQueue;\n        queue.pending = null;\n      }\n      pendingQueue = hook.baseState;\n      if (null === baseQueue) hook.memoizedState = pendingQueue;\n      else {\n        current = baseQueue.next;\n        var newBaseQueueFirst = (baseFirst = null),\n          newBaseQueueLast = null,\n          update = current,\n          didReadFromEntangledAsyncAction = !1;\n        do {\n          var updateLane = update.lane & -536870913;\n          if (\n            updateLane !== update.lane\n              ? (workInProgressRootRenderLanes & updateLane) === updateLane\n              : (renderLanes & updateLane) === updateLane\n          ) {\n            var revertLane = update.revertLane;\n            if (0 === revertLane)\n              null !== newBaseQueueLast &&\n                (newBaseQueueLast = newBaseQueueLast.next =\n                  {\n                    lane: 0,\n                    revertLane: 0,\n                    action: update.action,\n                    hasEagerState: update.hasEagerState,\n                    eagerState: update.eagerState,\n                    next: null\n                  }),\n                updateLane === currentEntangledLane &&\n                  (didReadFromEntangledAsyncAction = !0);\n            else if ((renderLanes & revertLane) === revertLane) {\n              update = update.next;\n              revertLane === currentEntangledLane &&\n                (didReadFromEntangledAsyncAction = !0);\n              continue;\n            } else\n              (updateLane = {\n                lane: 0,\n                revertLane: update.revertLane,\n                action: update.action,\n                hasEagerState: update.hasEagerState,\n                eagerState: update.eagerState,\n                next: null\n              }),\n                null === newBaseQueueLast\n                  ? ((newBaseQueueFirst = newBaseQueueLast = updateLane),\n                    (baseFirst = pendingQueue))\n                  : (newBaseQueueLast = newBaseQueueLast.next = updateLane),\n                (currentlyRenderingFiber.lanes |= revertLane),\n                (workInProgressRootSkippedLanes |= revertLane);\n            updateLane = update.action;\n            shouldDoubleInvokeUserFnsInHooksDEV &&\n              reducer(pendingQueue, updateLane);\n            pendingQueue = update.hasEagerState\n              ? update.eagerState\n              : reducer(pendingQueue, updateLane);\n          } else\n            (revertLane = {\n              lane: updateLane,\n              revertLane: update.revertLane,\n              action: update.action,\n              hasEagerState: update.hasEagerState,\n              eagerState: update.eagerState,\n              next: null\n            }),\n              null === newBaseQueueLast\n                ? ((newBaseQueueFirst = newBaseQueueLast = revertLane),\n                  (baseFirst = pendingQueue))\n                : (newBaseQueueLast = newBaseQueueLast.next = revertLane),\n              (currentlyRenderingFiber.lanes |= updateLane),\n              (workInProgressRootSkippedLanes |= updateLane);\n          update = update.next;\n        } while (null !== update && update !== current);\n        null === newBaseQueueLast\n          ? (baseFirst = pendingQueue)\n          : (newBaseQueueLast.next = newBaseQueueFirst);\n        if (\n          !objectIs(pendingQueue, hook.memoizedState) &&\n          ((didReceiveUpdate = !0),\n          didReadFromEntangledAsyncAction &&\n            ((reducer = currentEntangledActionThenable), null !== reducer))\n        )\n          throw reducer;\n        hook.memoizedState = pendingQueue;\n        hook.baseState = baseFirst;\n        hook.baseQueue = newBaseQueueLast;\n        queue.lastRenderedState = pendingQueue;\n      }\n      null === baseQueue && (queue.lanes = 0);\n      return [hook.memoizedState, queue.dispatch];\n    }\n    function rerenderReducer(reducer) {\n      var hook = updateWorkInProgressHook(),\n        queue = hook.queue;\n      if (null === queue)\n        throw Error(\n          \"Should have a queue. You are likely calling Hooks conditionally, which is not allowed. (https://react.dev/link/invalid-hook-call)\"\n        );\n      queue.lastRenderedReducer = reducer;\n      var dispatch = queue.dispatch,\n        lastRenderPhaseUpdate = queue.pending,\n        newState = hook.memoizedState;\n      if (null !== lastRenderPhaseUpdate) {\n        queue.pending = null;\n        var update = (lastRenderPhaseUpdate = lastRenderPhaseUpdate.next);\n        do\n          (newState = reducer(newState, update.action)), (update = update.next);\n        while (update !== lastRenderPhaseUpdate);\n        objectIs(newState, hook.memoizedState) || (didReceiveUpdate = !0);\n        hook.memoizedState = newState;\n        null === hook.baseQueue && (hook.baseState = newState);\n        queue.lastRenderedState = newState;\n      }\n      return [newState, dispatch];\n    }\n    function mountSyncExternalStore(subscribe, getSnapshot, getServerSnapshot) {\n      var fiber = currentlyRenderingFiber,\n        hook = mountWorkInProgressHook();\n      if (isHydrating) {\n        if (void 0 === getServerSnapshot)\n          throw Error(\n            \"Missing getServerSnapshot, which is required for server-rendered content. Will revert to client rendering.\"\n          );\n        var nextSnapshot = getServerSnapshot();\n        didWarnUncachedGetSnapshot ||\n          nextSnapshot === getServerSnapshot() ||\n          (console.error(\n            \"The result of getServerSnapshot should be cached to avoid an infinite loop\"\n          ),\n          (didWarnUncachedGetSnapshot = !0));\n      } else {\n        nextSnapshot = getSnapshot();\n        didWarnUncachedGetSnapshot ||\n          ((getServerSnapshot = getSnapshot()),\n          objectIs(nextSnapshot, getServerSnapshot) ||\n            (console.error(\n              \"The result of getSnapshot should be cached to avoid an infinite loop\"\n            ),\n            (didWarnUncachedGetSnapshot = !0)));\n        if (null === workInProgressRoot)\n          throw Error(\n            \"Expected a work-in-progress root. This is a bug in React. Please file an issue.\"\n          );\n        0 !== (workInProgressRootRenderLanes & 124) ||\n          pushStoreConsistencyCheck(fiber, getSnapshot, nextSnapshot);\n      }\n      hook.memoizedState = nextSnapshot;\n      getServerSnapshot = { value: nextSnapshot, getSnapshot: getSnapshot };\n      hook.queue = getServerSnapshot;\n      mountEffect(\n        subscribeToStore.bind(null, fiber, getServerSnapshot, subscribe),\n        [subscribe]\n      );\n      fiber.flags |= 2048;\n      pushSimpleEffect(\n        HasEffect | Passive,\n        createEffectInstance(),\n        updateStoreInstance.bind(\n          null,\n          fiber,\n          getServerSnapshot,\n          nextSnapshot,\n          getSnapshot\n        ),\n        null\n      );\n      return nextSnapshot;\n    }\n    function updateSyncExternalStore(\n      subscribe,\n      getSnapshot,\n      getServerSnapshot\n    ) {\n      var fiber = currentlyRenderingFiber,\n        hook = updateWorkInProgressHook(),\n        isHydrating$jscomp$0 = isHydrating;\n      if (isHydrating$jscomp$0) {\n        if (void 0 === getServerSnapshot)\n          throw Error(\n            \"Missing getServerSnapshot, which is required for server-rendered content. Will revert to client rendering.\"\n          );\n        getServerSnapshot = getServerSnapshot();\n      } else if (\n        ((getServerSnapshot = getSnapshot()), !didWarnUncachedGetSnapshot)\n      ) {\n        var cachedSnapshot = getSnapshot();\n        objectIs(getServerSnapshot, cachedSnapshot) ||\n          (console.error(\n            \"The result of getSnapshot should be cached to avoid an infinite loop\"\n          ),\n          (didWarnUncachedGetSnapshot = !0));\n      }\n      if (\n        (cachedSnapshot = !objectIs(\n          (currentHook || hook).memoizedState,\n          getServerSnapshot\n        ))\n      )\n        (hook.memoizedState = getServerSnapshot), (didReceiveUpdate = !0);\n      hook = hook.queue;\n      var create = subscribeToStore.bind(null, fiber, hook, subscribe);\n      updateEffectImpl(2048, Passive, create, [subscribe]);\n      if (\n        hook.getSnapshot !== getSnapshot ||\n        cachedSnapshot ||\n        (null !== workInProgressHook &&\n          workInProgressHook.memoizedState.tag & HasEffect)\n      ) {\n        fiber.flags |= 2048;\n        pushSimpleEffect(\n          HasEffect | Passive,\n          createEffectInstance(),\n          updateStoreInstance.bind(\n            null,\n            fiber,\n            hook,\n            getServerSnapshot,\n            getSnapshot\n          ),\n          null\n        );\n        if (null === workInProgressRoot)\n          throw Error(\n            \"Expected a work-in-progress root. This is a bug in React. Please file an issue.\"\n          );\n        isHydrating$jscomp$0 ||\n          0 !== (renderLanes & 124) ||\n          pushStoreConsistencyCheck(fiber, getSnapshot, getServerSnapshot);\n      }\n      return getServerSnapshot;\n    }\n    function pushStoreConsistencyCheck(fiber, getSnapshot, renderedSnapshot) {\n      fiber.flags |= 16384;\n      fiber = { getSnapshot: getSnapshot, value: renderedSnapshot };\n      getSnapshot = currentlyRenderingFiber.updateQueue;\n      null === getSnapshot\n        ? ((getSnapshot = createFunctionComponentUpdateQueue()),\n          (currentlyRenderingFiber.updateQueue = getSnapshot),\n          (getSnapshot.stores = [fiber]))\n        : ((renderedSnapshot = getSnapshot.stores),\n          null === renderedSnapshot\n            ? (getSnapshot.stores = [fiber])\n            : renderedSnapshot.push(fiber));\n    }\n    function updateStoreInstance(fiber, inst, nextSnapshot, getSnapshot) {\n      inst.value = nextSnapshot;\n      inst.getSnapshot = getSnapshot;\n      checkIfSnapshotChanged(inst) && forceStoreRerender(fiber);\n    }\n    function subscribeToStore(fiber, inst, subscribe) {\n      return subscribe(function () {\n        checkIfSnapshotChanged(inst) && forceStoreRerender(fiber);\n      });\n    }\n    function checkIfSnapshotChanged(inst) {\n      var latestGetSnapshot = inst.getSnapshot;\n      inst = inst.value;\n      try {\n        var nextValue = latestGetSnapshot();\n        return !objectIs(inst, nextValue);\n      } catch (error) {\n        return !0;\n      }\n    }\n    function forceStoreRerender(fiber) {\n      var root = enqueueConcurrentRenderForLane(fiber, 2);\n      null !== root && scheduleUpdateOnFiber(root, fiber, 2);\n    }\n    function mountStateImpl(initialState) {\n      var hook = mountWorkInProgressHook();\n      if (\"function\" === typeof initialState) {\n        var initialStateInitializer = initialState;\n        initialState = initialStateInitializer();\n        if (shouldDoubleInvokeUserFnsInHooksDEV) {\n          setIsStrictModeForDevtools(!0);\n          try {\n            initialStateInitializer();\n          } finally {\n            setIsStrictModeForDevtools(!1);\n          }\n        }\n      }\n      hook.memoizedState = hook.baseState = initialState;\n      hook.queue = {\n        pending: null,\n        lanes: 0,\n        dispatch: null,\n        lastRenderedReducer: basicStateReducer,\n        lastRenderedState: initialState\n      };\n      return hook;\n    }\n    function mountState(initialState) {\n      initialState = mountStateImpl(initialState);\n      var queue = initialState.queue,\n        dispatch = dispatchSetState.bind(null, currentlyRenderingFiber, queue);\n      queue.dispatch = dispatch;\n      return [initialState.memoizedState, dispatch];\n    }\n    function mountOptimistic(passthrough) {\n      var hook = mountWorkInProgressHook();\n      hook.memoizedState = hook.baseState = passthrough;\n      var queue = {\n        pending: null,\n        lanes: 0,\n        dispatch: null,\n        lastRenderedReducer: null,\n        lastRenderedState: null\n      };\n      hook.queue = queue;\n      hook = dispatchOptimisticSetState.bind(\n        null,\n        currentlyRenderingFiber,\n        !0,\n        queue\n      );\n      queue.dispatch = hook;\n      return [passthrough, hook];\n    }\n    function updateOptimistic(passthrough, reducer) {\n      var hook = updateWorkInProgressHook();\n      return updateOptimisticImpl(hook, currentHook, passthrough, reducer);\n    }\n    function updateOptimisticImpl(hook, current, passthrough, reducer) {\n      hook.baseState = passthrough;\n      return updateReducerImpl(\n        hook,\n        currentHook,\n        \"function\" === typeof reducer ? reducer : basicStateReducer\n      );\n    }\n    function rerenderOptimistic(passthrough, reducer) {\n      var hook = updateWorkInProgressHook();\n      if (null !== currentHook)\n        return updateOptimisticImpl(hook, currentHook, passthrough, reducer);\n      hook.baseState = passthrough;\n      return [passthrough, hook.queue.dispatch];\n    }\n    function dispatchActionState(\n      fiber,\n      actionQueue,\n      setPendingState,\n      setState,\n      payload\n    ) {\n      if (isRenderPhaseUpdate(fiber))\n        throw Error(\"Cannot update form state while rendering.\");\n      fiber = actionQueue.action;\n      if (null !== fiber) {\n        var actionNode = {\n          payload: payload,\n          action: fiber,\n          next: null,\n          isTransition: !0,\n          status: \"pending\",\n          value: null,\n          reason: null,\n          listeners: [],\n          then: function (listener) {\n            actionNode.listeners.push(listener);\n          }\n        };\n        null !== ReactSharedInternals.T\n          ? setPendingState(!0)\n          : (actionNode.isTransition = !1);\n        setState(actionNode);\n        setPendingState = actionQueue.pending;\n        null === setPendingState\n          ? ((actionNode.next = actionQueue.pending = actionNode),\n            runActionStateAction(actionQueue, actionNode))\n          : ((actionNode.next = setPendingState.next),\n            (actionQueue.pending = setPendingState.next = actionNode));\n      }\n    }\n    function runActionStateAction(actionQueue, node) {\n      var action = node.action,\n        payload = node.payload,\n        prevState = actionQueue.state;\n      if (node.isTransition) {\n        var prevTransition = ReactSharedInternals.T,\n          currentTransition = {};\n        ReactSharedInternals.T = currentTransition;\n        ReactSharedInternals.T._updatedFibers = new Set();\n        try {\n          var returnValue = action(prevState, payload),\n            onStartTransitionFinish = ReactSharedInternals.S;\n          null !== onStartTransitionFinish &&\n            onStartTransitionFinish(currentTransition, returnValue);\n          handleActionReturnValue(actionQueue, node, returnValue);\n        } catch (error) {\n          onActionError(actionQueue, node, error);\n        } finally {\n          (ReactSharedInternals.T = prevTransition),\n            null === prevTransition &&\n              currentTransition._updatedFibers &&\n              ((actionQueue = currentTransition._updatedFibers.size),\n              currentTransition._updatedFibers.clear(),\n              10 < actionQueue &&\n                console.warn(\n                  \"Detected a large number of updates inside startTransition. If this is due to a subscription please re-write it to use React provided hooks. Otherwise concurrent mode guarantees are off the table.\"\n                ));\n        }\n      } else\n        try {\n          (currentTransition = action(prevState, payload)),\n            handleActionReturnValue(actionQueue, node, currentTransition);\n        } catch (error$4) {\n          onActionError(actionQueue, node, error$4);\n        }\n    }\n    function handleActionReturnValue(actionQueue, node, returnValue) {\n      null !== returnValue &&\n      \"object\" === typeof returnValue &&\n      \"function\" === typeof returnValue.then\n        ? (returnValue.then(\n            function (nextState) {\n              onActionSuccess(actionQueue, node, nextState);\n            },\n            function (error) {\n              return onActionError(actionQueue, node, error);\n            }\n          ),\n          node.isTransition ||\n            console.error(\n              \"An async function with useActionState was called outside of a transition. This is likely not what you intended (for example, isPending will not update correctly). Either call the returned function inside startTransition, or pass it to an `action` or `formAction` prop.\"\n            ))\n        : onActionSuccess(actionQueue, node, returnValue);\n    }\n    function onActionSuccess(actionQueue, actionNode, nextState) {\n      actionNode.status = \"fulfilled\";\n      actionNode.value = nextState;\n      notifyActionListeners(actionNode);\n      actionQueue.state = nextState;\n      actionNode = actionQueue.pending;\n      null !== actionNode &&\n        ((nextState = actionNode.next),\n        nextState === actionNode\n          ? (actionQueue.pending = null)\n          : ((nextState = nextState.next),\n            (actionNode.next = nextState),\n            runActionStateAction(actionQueue, nextState)));\n    }\n    function onActionError(actionQueue, actionNode, error) {\n      var last = actionQueue.pending;\n      actionQueue.pending = null;\n      if (null !== last) {\n        last = last.next;\n        do\n          (actionNode.status = \"rejected\"),\n            (actionNode.reason = error),\n            notifyActionListeners(actionNode),\n            (actionNode = actionNode.next);\n        while (actionNode !== last);\n      }\n      actionQueue.action = null;\n    }\n    function notifyActionListeners(actionNode) {\n      actionNode = actionNode.listeners;\n      for (var i = 0; i < actionNode.length; i++) (0, actionNode[i])();\n    }\n    function actionStateReducer(oldState, newState) {\n      return newState;\n    }\n    function mountActionState(action, initialStateProp) {\n      if (isHydrating) {\n        var ssrFormState = workInProgressRoot.formState;\n        if (null !== ssrFormState) {\n          a: {\n            var isMatching = currentlyRenderingFiber;\n            if (isHydrating) {\n              if (nextHydratableInstance) {\n                b: {\n                  var markerInstance = nextHydratableInstance;\n                  for (\n                    var inRootOrSingleton = rootOrSingletonContext;\n                    8 !== markerInstance.nodeType;\n\n                  ) {\n                    if (!inRootOrSingleton) {\n                      markerInstance = null;\n                      break b;\n                    }\n                    markerInstance = getNextHydratable(\n                      markerInstance.nextSibling\n                    );\n                    if (null === markerInstance) {\n                      markerInstance = null;\n                      break b;\n                    }\n                  }\n                  inRootOrSingleton = markerInstance.data;\n                  markerInstance =\n                    inRootOrSingleton === FORM_STATE_IS_MATCHING ||\n                    inRootOrSingleton === FORM_STATE_IS_NOT_MATCHING\n                      ? markerInstance\n                      : null;\n                }\n                if (markerInstance) {\n                  nextHydratableInstance = getNextHydratable(\n                    markerInstance.nextSibling\n                  );\n                  isMatching = markerInstance.data === FORM_STATE_IS_MATCHING;\n                  break a;\n                }\n              }\n              throwOnHydrationMismatch(isMatching);\n            }\n            isMatching = !1;\n          }\n          isMatching && (initialStateProp = ssrFormState[0]);\n        }\n      }\n      ssrFormState = mountWorkInProgressHook();\n      ssrFormState.memoizedState = ssrFormState.baseState = initialStateProp;\n      isMatching = {\n        pending: null,\n        lanes: 0,\n        dispatch: null,\n        lastRenderedReducer: actionStateReducer,\n        lastRenderedState: initialStateProp\n      };\n      ssrFormState.queue = isMatching;\n      ssrFormState = dispatchSetState.bind(\n        null,\n        currentlyRenderingFiber,\n        isMatching\n      );\n      isMatching.dispatch = ssrFormState;\n      isMatching = mountStateImpl(!1);\n      inRootOrSingleton = dispatchOptimisticSetState.bind(\n        null,\n        currentlyRenderingFiber,\n        !1,\n        isMatching.queue\n      );\n      isMatching = mountWorkInProgressHook();\n      markerInstance = {\n        state: initialStateProp,\n        dispatch: null,\n        action: action,\n        pending: null\n      };\n      isMatching.queue = markerInstance;\n      ssrFormState = dispatchActionState.bind(\n        null,\n        currentlyRenderingFiber,\n        markerInstance,\n        inRootOrSingleton,\n        ssrFormState\n      );\n      markerInstance.dispatch = ssrFormState;\n      isMatching.memoizedState = action;\n      return [initialStateProp, ssrFormState, !1];\n    }\n    function updateActionState(action) {\n      var stateHook = updateWorkInProgressHook();\n      return updateActionStateImpl(stateHook, currentHook, action);\n    }\n    function updateActionStateImpl(stateHook, currentStateHook, action) {\n      currentStateHook = updateReducerImpl(\n        stateHook,\n        currentStateHook,\n        actionStateReducer\n      )[0];\n      stateHook = updateReducer(basicStateReducer)[0];\n      if (\n        \"object\" === typeof currentStateHook &&\n        null !== currentStateHook &&\n        \"function\" === typeof currentStateHook.then\n      )\n        try {\n          var state = useThenable(currentStateHook);\n        } catch (x) {\n          if (x === SuspenseException) throw SuspenseActionException;\n          throw x;\n        }\n      else state = currentStateHook;\n      currentStateHook = updateWorkInProgressHook();\n      var actionQueue = currentStateHook.queue,\n        dispatch = actionQueue.dispatch;\n      action !== currentStateHook.memoizedState &&\n        ((currentlyRenderingFiber.flags |= 2048),\n        pushSimpleEffect(\n          HasEffect | Passive,\n          createEffectInstance(),\n          actionStateActionEffect.bind(null, actionQueue, action),\n          null\n        ));\n      return [state, dispatch, stateHook];\n    }\n    function actionStateActionEffect(actionQueue, action) {\n      actionQueue.action = action;\n    }\n    function rerenderActionState(action) {\n      var stateHook = updateWorkInProgressHook(),\n        currentStateHook = currentHook;\n      if (null !== currentStateHook)\n        return updateActionStateImpl(stateHook, currentStateHook, action);\n      updateWorkInProgressHook();\n      stateHook = stateHook.memoizedState;\n      currentStateHook = updateWorkInProgressHook();\n      var dispatch = currentStateHook.queue.dispatch;\n      currentStateHook.memoizedState = action;\n      return [stateHook, dispatch, !1];\n    }\n    function pushSimpleEffect(tag, inst, create, createDeps) {\n      tag = {\n        tag: tag,\n        create: create,\n        deps: createDeps,\n        inst: inst,\n        next: null\n      };\n      inst = currentlyRenderingFiber.updateQueue;\n      null === inst &&\n        ((inst = createFunctionComponentUpdateQueue()),\n        (currentlyRenderingFiber.updateQueue = inst));\n      create = inst.lastEffect;\n      null === create\n        ? (inst.lastEffect = tag.next = tag)\n        : ((createDeps = create.next),\n          (create.next = tag),\n          (tag.next = createDeps),\n          (inst.lastEffect = tag));\n      return tag;\n    }\n    function createEffectInstance() {\n      return { destroy: void 0, resource: void 0 };\n    }\n    function mountRef(initialValue) {\n      var hook = mountWorkInProgressHook();\n      initialValue = { current: initialValue };\n      return (hook.memoizedState = initialValue);\n    }\n    function mountEffectImpl(fiberFlags, hookFlags, create, createDeps) {\n      var hook = mountWorkInProgressHook();\n      createDeps = void 0 === createDeps ? null : createDeps;\n      currentlyRenderingFiber.flags |= fiberFlags;\n      hook.memoizedState = pushSimpleEffect(\n        HasEffect | hookFlags,\n        createEffectInstance(),\n        create,\n        createDeps\n      );\n    }\n    function updateEffectImpl(fiberFlags, hookFlags, create, deps) {\n      var hook = updateWorkInProgressHook();\n      deps = void 0 === deps ? null : deps;\n      var inst = hook.memoizedState.inst;\n      null !== currentHook &&\n      null !== deps &&\n      areHookInputsEqual(deps, currentHook.memoizedState.deps)\n        ? (hook.memoizedState = pushSimpleEffect(hookFlags, inst, create, deps))\n        : ((currentlyRenderingFiber.flags |= fiberFlags),\n          (hook.memoizedState = pushSimpleEffect(\n            HasEffect | hookFlags,\n            inst,\n            create,\n            deps\n          )));\n    }\n    function mountEffect(create, createDeps) {\n      (currentlyRenderingFiber.mode & StrictEffectsMode) !== NoMode &&\n      (currentlyRenderingFiber.mode & NoStrictPassiveEffectsMode) === NoMode\n        ? mountEffectImpl(276826112, Passive, create, createDeps)\n        : mountEffectImpl(8390656, Passive, create, createDeps);\n    }\n    function mountLayoutEffect(create, deps) {\n      var fiberFlags = 4194308;\n      (currentlyRenderingFiber.mode & StrictEffectsMode) !== NoMode &&\n        (fiberFlags |= 134217728);\n      return mountEffectImpl(fiberFlags, Layout, create, deps);\n    }\n    function imperativeHandleEffect(create, ref) {\n      if (\"function\" === typeof ref) {\n        create = create();\n        var refCleanup = ref(create);\n        return function () {\n          \"function\" === typeof refCleanup ? refCleanup() : ref(null);\n        };\n      }\n      if (null !== ref && void 0 !== ref)\n        return (\n          ref.hasOwnProperty(\"current\") ||\n            console.error(\n              \"Expected useImperativeHandle() first argument to either be a ref callback or React.createRef() object. Instead received: %s.\",\n              \"an object with keys {\" + Object.keys(ref).join(\", \") + \"}\"\n            ),\n          (create = create()),\n          (ref.current = create),\n          function () {\n            ref.current = null;\n          }\n        );\n    }\n    function mountImperativeHandle(ref, create, deps) {\n      \"function\" !== typeof create &&\n        console.error(\n          \"Expected useImperativeHandle() second argument to be a function that creates a handle. Instead received: %s.\",\n          null !== create ? typeof create : \"null\"\n        );\n      deps = null !== deps && void 0 !== deps ? deps.concat([ref]) : null;\n      var fiberFlags = 4194308;\n      (currentlyRenderingFiber.mode & StrictEffectsMode) !== NoMode &&\n        (fiberFlags |= 134217728);\n      mountEffectImpl(\n        fiberFlags,\n        Layout,\n        imperativeHandleEffect.bind(null, create, ref),\n        deps\n      );\n    }\n    function updateImperativeHandle(ref, create, deps) {\n      \"function\" !== typeof create &&\n        console.error(\n          \"Expected useImperativeHandle() second argument to be a function that creates a handle. Instead received: %s.\",\n          null !== create ? typeof create : \"null\"\n        );\n      deps = null !== deps && void 0 !== deps ? deps.concat([ref]) : null;\n      updateEffectImpl(\n        4,\n        Layout,\n        imperativeHandleEffect.bind(null, create, ref),\n        deps\n      );\n    }\n    function mountCallback(callback, deps) {\n      mountWorkInProgressHook().memoizedState = [\n        callback,\n        void 0 === deps ? null : deps\n      ];\n      return callback;\n    }\n    function updateCallback(callback, deps) {\n      var hook = updateWorkInProgressHook();\n      deps = void 0 === deps ? null : deps;\n      var prevState = hook.memoizedState;\n      if (null !== deps && areHookInputsEqual(deps, prevState[1]))\n        return prevState[0];\n      hook.memoizedState = [callback, deps];\n      return callback;\n    }\n    function mountMemo(nextCreate, deps) {\n      var hook = mountWorkInProgressHook();\n      deps = void 0 === deps ? null : deps;\n      var nextValue = nextCreate();\n      if (shouldDoubleInvokeUserFnsInHooksDEV) {\n        setIsStrictModeForDevtools(!0);\n        try {\n          nextCreate();\n        } finally {\n          setIsStrictModeForDevtools(!1);\n        }\n      }\n      hook.memoizedState = [nextValue, deps];\n      return nextValue;\n    }\n    function updateMemo(nextCreate, deps) {\n      var hook = updateWorkInProgressHook();\n      deps = void 0 === deps ? null : deps;\n      var prevState = hook.memoizedState;\n      if (null !== deps && areHookInputsEqual(deps, prevState[1]))\n        return prevState[0];\n      prevState = nextCreate();\n      if (shouldDoubleInvokeUserFnsInHooksDEV) {\n        setIsStrictModeForDevtools(!0);\n        try {\n          nextCreate();\n        } finally {\n          setIsStrictModeForDevtools(!1);\n        }\n      }\n      hook.memoizedState = [prevState, deps];\n      return prevState;\n    }\n    function mountDeferredValue(value, initialValue) {\n      var hook = mountWorkInProgressHook();\n      return mountDeferredValueImpl(hook, value, initialValue);\n    }\n    function updateDeferredValue(value, initialValue) {\n      var hook = updateWorkInProgressHook();\n      return updateDeferredValueImpl(\n        hook,\n        currentHook.memoizedState,\n        value,\n        initialValue\n      );\n    }\n    function rerenderDeferredValue(value, initialValue) {\n      var hook = updateWorkInProgressHook();\n      return null === currentHook\n        ? mountDeferredValueImpl(hook, value, initialValue)\n        : updateDeferredValueImpl(\n            hook,\n            currentHook.memoizedState,\n            value,\n            initialValue\n          );\n    }\n    function mountDeferredValueImpl(hook, value, initialValue) {\n      if (void 0 === initialValue || 0 !== (renderLanes & 1073741824))\n        return (hook.memoizedState = value);\n      hook.memoizedState = initialValue;\n      hook = requestDeferredLane();\n      currentlyRenderingFiber.lanes |= hook;\n      workInProgressRootSkippedLanes |= hook;\n      return initialValue;\n    }\n    function updateDeferredValueImpl(hook, prevValue, value, initialValue) {\n      if (objectIs(value, prevValue)) return value;\n      if (null !== currentTreeHiddenStackCursor.current)\n        return (\n          (hook = mountDeferredValueImpl(hook, value, initialValue)),\n          objectIs(hook, prevValue) || (didReceiveUpdate = !0),\n          hook\n        );\n      if (0 === (renderLanes & 42))\n        return (didReceiveUpdate = !0), (hook.memoizedState = value);\n      hook = requestDeferredLane();\n      currentlyRenderingFiber.lanes |= hook;\n      workInProgressRootSkippedLanes |= hook;\n      return prevValue;\n    }\n    function startTransition(\n      fiber,\n      queue,\n      pendingState,\n      finishedState,\n      callback\n    ) {\n      var previousPriority = ReactDOMSharedInternals.p;\n      ReactDOMSharedInternals.p =\n        0 !== previousPriority && previousPriority < ContinuousEventPriority\n          ? previousPriority\n          : ContinuousEventPriority;\n      var prevTransition = ReactSharedInternals.T,\n        currentTransition = {};\n      ReactSharedInternals.T = currentTransition;\n      dispatchOptimisticSetState(fiber, !1, queue, pendingState);\n      currentTransition._updatedFibers = new Set();\n      try {\n        var returnValue = callback(),\n          onStartTransitionFinish = ReactSharedInternals.S;\n        null !== onStartTransitionFinish &&\n          onStartTransitionFinish(currentTransition, returnValue);\n        if (\n          null !== returnValue &&\n          \"object\" === typeof returnValue &&\n          \"function\" === typeof returnValue.then\n        ) {\n          var thenableForFinishedState = chainThenableValue(\n            returnValue,\n            finishedState\n          );\n          dispatchSetStateInternal(\n            fiber,\n            queue,\n            thenableForFinishedState,\n            requestUpdateLane(fiber)\n          );\n        } else\n          dispatchSetStateInternal(\n            fiber,\n            queue,\n            finishedState,\n            requestUpdateLane(fiber)\n          );\n      } catch (error) {\n        dispatchSetStateInternal(\n          fiber,\n          queue,\n          { then: function () {}, status: \"rejected\", reason: error },\n          requestUpdateLane(fiber)\n        );\n      } finally {\n        (ReactDOMSharedInternals.p = previousPriority),\n          (ReactSharedInternals.T = prevTransition),\n          null === prevTransition &&\n            currentTransition._updatedFibers &&\n            ((fiber = currentTransition._updatedFibers.size),\n            currentTransition._updatedFibers.clear(),\n            10 < fiber &&\n              console.warn(\n                \"Detected a large number of updates inside startTransition. If this is due to a subscription please re-write it to use React provided hooks. Otherwise concurrent mode guarantees are off the table.\"\n              ));\n      }\n    }\n    function startHostTransition(formFiber, pendingState, action, formData) {\n      if (5 !== formFiber.tag)\n        throw Error(\n          \"Expected the form instance to be a HostComponent. This is a bug in React.\"\n        );\n      var queue = ensureFormComponentIsStateful(formFiber).queue;\n      startTransition(\n        formFiber,\n        queue,\n        pendingState,\n        NotPendingTransition,\n        null === action\n          ? noop$2\n          : function () {\n              requestFormReset$1(formFiber);\n              return action(formData);\n            }\n      );\n    }\n    function ensureFormComponentIsStateful(formFiber) {\n      var existingStateHook = formFiber.memoizedState;\n      if (null !== existingStateHook) return existingStateHook;\n      existingStateHook = {\n        memoizedState: NotPendingTransition,\n        baseState: NotPendingTransition,\n        baseQueue: null,\n        queue: {\n          pending: null,\n          lanes: 0,\n          dispatch: null,\n          lastRenderedReducer: basicStateReducer,\n          lastRenderedState: NotPendingTransition\n        },\n        next: null\n      };\n      var initialResetState = {};\n      existingStateHook.next = {\n        memoizedState: initialResetState,\n        baseState: initialResetState,\n        baseQueue: null,\n        queue: {\n          pending: null,\n          lanes: 0,\n          dispatch: null,\n          lastRenderedReducer: basicStateReducer,\n          lastRenderedState: initialResetState\n        },\n        next: null\n      };\n      formFiber.memoizedState = existingStateHook;\n      formFiber = formFiber.alternate;\n      null !== formFiber && (formFiber.memoizedState = existingStateHook);\n      return existingStateHook;\n    }\n    function requestFormReset$1(formFiber) {\n      null === ReactSharedInternals.T &&\n        console.error(\n          \"requestFormReset was called outside a transition or action. To fix, move to an action, or wrap with startTransition.\"\n        );\n      var resetStateQueue = ensureFormComponentIsStateful(formFiber).next.queue;\n      dispatchSetStateInternal(\n        formFiber,\n        resetStateQueue,\n        {},\n        requestUpdateLane(formFiber)\n      );\n    }\n    function mountTransition() {\n      var stateHook = mountStateImpl(!1);\n      stateHook = startTransition.bind(\n        null,\n        currentlyRenderingFiber,\n        stateHook.queue,\n        !0,\n        !1\n      );\n      mountWorkInProgressHook().memoizedState = stateHook;\n      return [!1, stateHook];\n    }\n    function updateTransition() {\n      var booleanOrThenable = updateReducer(basicStateReducer)[0],\n        start = updateWorkInProgressHook().memoizedState;\n      return [\n        \"boolean\" === typeof booleanOrThenable\n          ? booleanOrThenable\n          : useThenable(booleanOrThenable),\n        start\n      ];\n    }\n    function rerenderTransition() {\n      var booleanOrThenable = rerenderReducer(basicStateReducer)[0],\n        start = updateWorkInProgressHook().memoizedState;\n      return [\n        \"boolean\" === typeof booleanOrThenable\n          ? booleanOrThenable\n          : useThenable(booleanOrThenable),\n        start\n      ];\n    }\n    function useHostTransitionStatus() {\n      return readContext(HostTransitionContext);\n    }\n    function mountId() {\n      var hook = mountWorkInProgressHook(),\n        identifierPrefix = workInProgressRoot.identifierPrefix;\n      if (isHydrating) {\n        var treeId = treeContextOverflow;\n        var idWithLeadingBit = treeContextId;\n        treeId =\n          (\n            idWithLeadingBit & ~(1 << (32 - clz32(idWithLeadingBit) - 1))\n          ).toString(32) + treeId;\n        identifierPrefix = \"\\u00ab\" + identifierPrefix + \"R\" + treeId;\n        treeId = localIdCounter++;\n        0 < treeId && (identifierPrefix += \"H\" + treeId.toString(32));\n        identifierPrefix += \"\\u00bb\";\n      } else\n        (treeId = globalClientIdCounter++),\n          (identifierPrefix =\n            \"\\u00ab\" + identifierPrefix + \"r\" + treeId.toString(32) + \"\\u00bb\");\n      return (hook.memoizedState = identifierPrefix);\n    }\n    function mountRefresh() {\n      return (mountWorkInProgressHook().memoizedState = refreshCache.bind(\n        null,\n        currentlyRenderingFiber\n      ));\n    }\n    function refreshCache(fiber, seedKey) {\n      for (var provider = fiber.return; null !== provider; ) {\n        switch (provider.tag) {\n          case 24:\n          case 3:\n            var lane = requestUpdateLane(provider);\n            fiber = createUpdate(lane);\n            var root = enqueueUpdate(provider, fiber, lane);\n            null !== root &&\n              (scheduleUpdateOnFiber(root, provider, lane),\n              entangleTransitions(root, provider, lane));\n            provider = createCache();\n            null !== seedKey &&\n              void 0 !== seedKey &&\n              null !== root &&\n              console.error(\n                \"The seed argument is not enabled outside experimental channels.\"\n              );\n            fiber.payload = { cache: provider };\n            return;\n        }\n        provider = provider.return;\n      }\n    }\n    function dispatchReducerAction(fiber, queue, action) {\n      var args = arguments;\n      \"function\" === typeof args[3] &&\n        console.error(\n          \"State updates from the useState() and useReducer() Hooks don't support the second callback argument. To execute a side effect after rendering, declare it in the component body with useEffect().\"\n        );\n      args = requestUpdateLane(fiber);\n      var update = {\n        lane: args,\n        revertLane: 0,\n        action: action,\n        hasEagerState: !1,\n        eagerState: null,\n        next: null\n      };\n      isRenderPhaseUpdate(fiber)\n        ? enqueueRenderPhaseUpdate(queue, update)\n        : ((update = enqueueConcurrentHookUpdate(fiber, queue, update, args)),\n          null !== update &&\n            (scheduleUpdateOnFiber(update, fiber, args),\n            entangleTransitionUpdate(update, queue, args)));\n      markStateUpdateScheduled(fiber, args);\n    }\n    function dispatchSetState(fiber, queue, action) {\n      var args = arguments;\n      \"function\" === typeof args[3] &&\n        console.error(\n          \"State updates from the useState() and useReducer() Hooks don't support the second callback argument. To execute a side effect after rendering, declare it in the component body with useEffect().\"\n        );\n      args = requestUpdateLane(fiber);\n      dispatchSetStateInternal(fiber, queue, action, args);\n      markStateUpdateScheduled(fiber, args);\n    }\n    function dispatchSetStateInternal(fiber, queue, action, lane) {\n      var update = {\n        lane: lane,\n        revertLane: 0,\n        action: action,\n        hasEagerState: !1,\n        eagerState: null,\n        next: null\n      };\n      if (isRenderPhaseUpdate(fiber)) enqueueRenderPhaseUpdate(queue, update);\n      else {\n        var alternate = fiber.alternate;\n        if (\n          0 === fiber.lanes &&\n          (null === alternate || 0 === alternate.lanes) &&\n          ((alternate = queue.lastRenderedReducer), null !== alternate)\n        ) {\n          var prevDispatcher = ReactSharedInternals.H;\n          ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n          try {\n            var currentState = queue.lastRenderedState,\n              eagerState = alternate(currentState, action);\n            update.hasEagerState = !0;\n            update.eagerState = eagerState;\n            if (objectIs(eagerState, currentState))\n              return (\n                enqueueUpdate$1(fiber, queue, update, 0),\n                null === workInProgressRoot &&\n                  finishQueueingConcurrentUpdates(),\n                !1\n              );\n          } catch (error) {\n          } finally {\n            ReactSharedInternals.H = prevDispatcher;\n          }\n        }\n        action = enqueueConcurrentHookUpdate(fiber, queue, update, lane);\n        if (null !== action)\n          return (\n            scheduleUpdateOnFiber(action, fiber, lane),\n            entangleTransitionUpdate(action, queue, lane),\n            !0\n          );\n      }\n      return !1;\n    }\n    function dispatchOptimisticSetState(\n      fiber,\n      throwIfDuringRender,\n      queue,\n      action\n    ) {\n      null === ReactSharedInternals.T &&\n        0 === currentEntangledLane &&\n        console.error(\n          \"An optimistic state update occurred outside a transition or action. To fix, move the update to an action, or wrap with startTransition.\"\n        );\n      action = {\n        lane: 2,\n        revertLane: requestTransitionLane(),\n        action: action,\n        hasEagerState: !1,\n        eagerState: null,\n        next: null\n      };\n      if (isRenderPhaseUpdate(fiber)) {\n        if (throwIfDuringRender)\n          throw Error(\"Cannot update optimistic state while rendering.\");\n        console.error(\"Cannot call startTransition while rendering.\");\n      } else\n        (throwIfDuringRender = enqueueConcurrentHookUpdate(\n          fiber,\n          queue,\n          action,\n          2\n        )),\n          null !== throwIfDuringRender &&\n            scheduleUpdateOnFiber(throwIfDuringRender, fiber, 2);\n      markStateUpdateScheduled(fiber, 2);\n    }\n    function isRenderPhaseUpdate(fiber) {\n      var alternate = fiber.alternate;\n      return (\n        fiber === currentlyRenderingFiber ||\n        (null !== alternate && alternate === currentlyRenderingFiber)\n      );\n    }\n    function enqueueRenderPhaseUpdate(queue, update) {\n      didScheduleRenderPhaseUpdateDuringThisPass =\n        didScheduleRenderPhaseUpdate = !0;\n      var pending = queue.pending;\n      null === pending\n        ? (update.next = update)\n        : ((update.next = pending.next), (pending.next = update));\n      queue.pending = update;\n    }\n    function entangleTransitionUpdate(root, queue, lane) {\n      if (0 !== (lane & 4194048)) {\n        var queueLanes = queue.lanes;\n        queueLanes &= root.pendingLanes;\n        lane |= queueLanes;\n        queue.lanes = lane;\n        markRootEntangled(root, lane);\n      }\n    }\n    function pushDebugInfo(debugInfo) {\n      var previousDebugInfo = currentDebugInfo;\n      null != debugInfo &&\n        (currentDebugInfo =\n          null === previousDebugInfo\n            ? debugInfo\n            : previousDebugInfo.concat(debugInfo));\n      return previousDebugInfo;\n    }\n    function validateFragmentProps(element, fiber, returnFiber) {\n      for (var keys = Object.keys(element.props), i = 0; i < keys.length; i++) {\n        var key = keys[i];\n        if (\"children\" !== key && \"key\" !== key) {\n          null === fiber &&\n            ((fiber = createFiberFromElement(element, returnFiber.mode, 0)),\n            (fiber._debugInfo = currentDebugInfo),\n            (fiber.return = returnFiber));\n          runWithFiberInDEV(\n            fiber,\n            function (erroredKey) {\n              console.error(\n                \"Invalid prop `%s` supplied to `React.Fragment`. React.Fragment can only have `key` and `children` props.\",\n                erroredKey\n              );\n            },\n            key\n          );\n          break;\n        }\n      }\n    }\n    function unwrapThenable(thenable) {\n      var index = thenableIndexCounter;\n      thenableIndexCounter += 1;\n      null === thenableState && (thenableState = createThenableState());\n      return trackUsedThenable(thenableState, thenable, index);\n    }\n    function coerceRef(workInProgress, element) {\n      element = element.props.ref;\n      workInProgress.ref = void 0 !== element ? element : null;\n    }\n    function throwOnInvalidObjectType(returnFiber, newChild) {\n      if (newChild.$$typeof === REACT_LEGACY_ELEMENT_TYPE)\n        throw Error(\n          'A React Element from an older version of React was rendered. This is not supported. It can happen if:\\n- Multiple copies of the \"react\" package is used.\\n- A library pre-bundled an old copy of \"react\" or \"react/jsx-runtime\".\\n- A compiler tries to \"inline\" JSX instead of using the runtime.'\n        );\n      returnFiber = Object.prototype.toString.call(newChild);\n      throw Error(\n        \"Objects are not valid as a React child (found: \" +\n          (\"[object Object]\" === returnFiber\n            ? \"object with keys {\" + Object.keys(newChild).join(\", \") + \"}\"\n            : returnFiber) +\n          \"). If you meant to render a collection of children, use an array instead.\"\n      );\n    }\n    function warnOnFunctionType(returnFiber, invalidChild) {\n      var parentName = getComponentNameFromFiber(returnFiber) || \"Component\";\n      ownerHasFunctionTypeWarning[parentName] ||\n        ((ownerHasFunctionTypeWarning[parentName] = !0),\n        (invalidChild =\n          invalidChild.displayName || invalidChild.name || \"Component\"),\n        3 === returnFiber.tag\n          ? console.error(\n              \"Functions are not valid as a React child. This may happen if you return %s instead of <%s /> from render. Or maybe you meant to call this function rather than return it.\\n  root.render(%s)\",\n              invalidChild,\n              invalidChild,\n              invalidChild\n            )\n          : console.error(\n              \"Functions are not valid as a React child. This may happen if you return %s instead of <%s /> from render. Or maybe you meant to call this function rather than return it.\\n  <%s>{%s}</%s>\",\n              invalidChild,\n              invalidChild,\n              parentName,\n              invalidChild,\n              parentName\n            ));\n    }\n    function warnOnSymbolType(returnFiber, invalidChild) {\n      var parentName = getComponentNameFromFiber(returnFiber) || \"Component\";\n      ownerHasSymbolTypeWarning[parentName] ||\n        ((ownerHasSymbolTypeWarning[parentName] = !0),\n        (invalidChild = String(invalidChild)),\n        3 === returnFiber.tag\n          ? console.error(\n              \"Symbols are not valid as a React child.\\n  root.render(%s)\",\n              invalidChild\n            )\n          : console.error(\n              \"Symbols are not valid as a React child.\\n  <%s>%s</%s>\",\n              parentName,\n              invalidChild,\n              parentName\n            ));\n    }\n    function createChildReconciler(shouldTrackSideEffects) {\n      function deleteChild(returnFiber, childToDelete) {\n        if (shouldTrackSideEffects) {\n          var deletions = returnFiber.deletions;\n          null === deletions\n            ? ((returnFiber.deletions = [childToDelete]),\n              (returnFiber.flags |= 16))\n            : deletions.push(childToDelete);\n        }\n      }\n      function deleteRemainingChildren(returnFiber, currentFirstChild) {\n        if (!shouldTrackSideEffects) return null;\n        for (; null !== currentFirstChild; )\n          deleteChild(returnFiber, currentFirstChild),\n            (currentFirstChild = currentFirstChild.sibling);\n        return null;\n      }\n      function mapRemainingChildren(currentFirstChild) {\n        for (var existingChildren = new Map(); null !== currentFirstChild; )\n          null !== currentFirstChild.key\n            ? existingChildren.set(currentFirstChild.key, currentFirstChild)\n            : existingChildren.set(currentFirstChild.index, currentFirstChild),\n            (currentFirstChild = currentFirstChild.sibling);\n        return existingChildren;\n      }\n      function useFiber(fiber, pendingProps) {\n        fiber = createWorkInProgress(fiber, pendingProps);\n        fiber.index = 0;\n        fiber.sibling = null;\n        return fiber;\n      }\n      function placeChild(newFiber, lastPlacedIndex, newIndex) {\n        newFiber.index = newIndex;\n        if (!shouldTrackSideEffects)\n          return (newFiber.flags |= 1048576), lastPlacedIndex;\n        newIndex = newFiber.alternate;\n        if (null !== newIndex)\n          return (\n            (newIndex = newIndex.index),\n            newIndex < lastPlacedIndex\n              ? ((newFiber.flags |= 67108866), lastPlacedIndex)\n              : newIndex\n          );\n        newFiber.flags |= 67108866;\n        return lastPlacedIndex;\n      }\n      function placeSingleChild(newFiber) {\n        shouldTrackSideEffects &&\n          null === newFiber.alternate &&\n          (newFiber.flags |= 67108866);\n        return newFiber;\n      }\n      function updateTextNode(returnFiber, current, textContent, lanes) {\n        if (null === current || 6 !== current.tag)\n          return (\n            (current = createFiberFromText(\n              textContent,\n              returnFiber.mode,\n              lanes\n            )),\n            (current.return = returnFiber),\n            (current._debugOwner = returnFiber),\n            (current._debugTask = returnFiber._debugTask),\n            (current._debugInfo = currentDebugInfo),\n            current\n          );\n        current = useFiber(current, textContent);\n        current.return = returnFiber;\n        current._debugInfo = currentDebugInfo;\n        return current;\n      }\n      function updateElement(returnFiber, current, element, lanes) {\n        var elementType = element.type;\n        if (elementType === REACT_FRAGMENT_TYPE)\n          return (\n            (current = updateFragment(\n              returnFiber,\n              current,\n              element.props.children,\n              lanes,\n              element.key\n            )),\n            validateFragmentProps(element, current, returnFiber),\n            current\n          );\n        if (\n          null !== current &&\n          (current.elementType === elementType ||\n            isCompatibleFamilyForHotReloading(current, element) ||\n            (\"object\" === typeof elementType &&\n              null !== elementType &&\n              elementType.$$typeof === REACT_LAZY_TYPE &&\n              callLazyInitInDEV(elementType) === current.type))\n        )\n          return (\n            (current = useFiber(current, element.props)),\n            coerceRef(current, element),\n            (current.return = returnFiber),\n            (current._debugOwner = element._owner),\n            (current._debugInfo = currentDebugInfo),\n            current\n          );\n        current = createFiberFromElement(element, returnFiber.mode, lanes);\n        coerceRef(current, element);\n        current.return = returnFiber;\n        current._debugInfo = currentDebugInfo;\n        return current;\n      }\n      function updatePortal(returnFiber, current, portal, lanes) {\n        if (\n          null === current ||\n          4 !== current.tag ||\n          current.stateNode.containerInfo !== portal.containerInfo ||\n          current.stateNode.implementation !== portal.implementation\n        )\n          return (\n            (current = createFiberFromPortal(portal, returnFiber.mode, lanes)),\n            (current.return = returnFiber),\n            (current._debugInfo = currentDebugInfo),\n            current\n          );\n        current = useFiber(current, portal.children || []);\n        current.return = returnFiber;\n        current._debugInfo = currentDebugInfo;\n        return current;\n      }\n      function updateFragment(returnFiber, current, fragment, lanes, key) {\n        if (null === current || 7 !== current.tag)\n          return (\n            (current = createFiberFromFragment(\n              fragment,\n              returnFiber.mode,\n              lanes,\n              key\n            )),\n            (current.return = returnFiber),\n            (current._debugOwner = returnFiber),\n            (current._debugTask = returnFiber._debugTask),\n            (current._debugInfo = currentDebugInfo),\n            current\n          );\n        current = useFiber(current, fragment);\n        current.return = returnFiber;\n        current._debugInfo = currentDebugInfo;\n        return current;\n      }\n      function createChild(returnFiber, newChild, lanes) {\n        if (\n          (\"string\" === typeof newChild && \"\" !== newChild) ||\n          \"number\" === typeof newChild ||\n          \"bigint\" === typeof newChild\n        )\n          return (\n            (newChild = createFiberFromText(\n              \"\" + newChild,\n              returnFiber.mode,\n              lanes\n            )),\n            (newChild.return = returnFiber),\n            (newChild._debugOwner = returnFiber),\n            (newChild._debugTask = returnFiber._debugTask),\n            (newChild._debugInfo = currentDebugInfo),\n            newChild\n          );\n        if (\"object\" === typeof newChild && null !== newChild) {\n          switch (newChild.$$typeof) {\n            case REACT_ELEMENT_TYPE:\n              return (\n                (lanes = createFiberFromElement(\n                  newChild,\n                  returnFiber.mode,\n                  lanes\n                )),\n                coerceRef(lanes, newChild),\n                (lanes.return = returnFiber),\n                (returnFiber = pushDebugInfo(newChild._debugInfo)),\n                (lanes._debugInfo = currentDebugInfo),\n                (currentDebugInfo = returnFiber),\n                lanes\n              );\n            case REACT_PORTAL_TYPE:\n              return (\n                (newChild = createFiberFromPortal(\n                  newChild,\n                  returnFiber.mode,\n                  lanes\n                )),\n                (newChild.return = returnFiber),\n                (newChild._debugInfo = currentDebugInfo),\n                newChild\n              );\n            case REACT_LAZY_TYPE:\n              var _prevDebugInfo = pushDebugInfo(newChild._debugInfo);\n              newChild = callLazyInitInDEV(newChild);\n              returnFiber = createChild(returnFiber, newChild, lanes);\n              currentDebugInfo = _prevDebugInfo;\n              return returnFiber;\n          }\n          if (isArrayImpl(newChild) || getIteratorFn(newChild))\n            return (\n              (lanes = createFiberFromFragment(\n                newChild,\n                returnFiber.mode,\n                lanes,\n                null\n              )),\n              (lanes.return = returnFiber),\n              (lanes._debugOwner = returnFiber),\n              (lanes._debugTask = returnFiber._debugTask),\n              (returnFiber = pushDebugInfo(newChild._debugInfo)),\n              (lanes._debugInfo = currentDebugInfo),\n              (currentDebugInfo = returnFiber),\n              lanes\n            );\n          if (\"function\" === typeof newChild.then)\n            return (\n              (_prevDebugInfo = pushDebugInfo(newChild._debugInfo)),\n              (returnFiber = createChild(\n                returnFiber,\n                unwrapThenable(newChild),\n                lanes\n              )),\n              (currentDebugInfo = _prevDebugInfo),\n              returnFiber\n            );\n          if (newChild.$$typeof === REACT_CONTEXT_TYPE)\n            return createChild(\n              returnFiber,\n              readContextDuringReconciliation(returnFiber, newChild),\n              lanes\n            );\n          throwOnInvalidObjectType(returnFiber, newChild);\n        }\n        \"function\" === typeof newChild &&\n          warnOnFunctionType(returnFiber, newChild);\n        \"symbol\" === typeof newChild && warnOnSymbolType(returnFiber, newChild);\n        return null;\n      }\n      function updateSlot(returnFiber, oldFiber, newChild, lanes) {\n        var key = null !== oldFiber ? oldFiber.key : null;\n        if (\n          (\"string\" === typeof newChild && \"\" !== newChild) ||\n          \"number\" === typeof newChild ||\n          \"bigint\" === typeof newChild\n        )\n          return null !== key\n            ? null\n            : updateTextNode(returnFiber, oldFiber, \"\" + newChild, lanes);\n        if (\"object\" === typeof newChild && null !== newChild) {\n          switch (newChild.$$typeof) {\n            case REACT_ELEMENT_TYPE:\n              return newChild.key === key\n                ? ((key = pushDebugInfo(newChild._debugInfo)),\n                  (returnFiber = updateElement(\n                    returnFiber,\n                    oldFiber,\n                    newChild,\n                    lanes\n                  )),\n                  (currentDebugInfo = key),\n                  returnFiber)\n                : null;\n            case REACT_PORTAL_TYPE:\n              return newChild.key === key\n                ? updatePortal(returnFiber, oldFiber, newChild, lanes)\n                : null;\n            case REACT_LAZY_TYPE:\n              return (\n                (key = pushDebugInfo(newChild._debugInfo)),\n                (newChild = callLazyInitInDEV(newChild)),\n                (returnFiber = updateSlot(\n                  returnFiber,\n                  oldFiber,\n                  newChild,\n                  lanes\n                )),\n                (currentDebugInfo = key),\n                returnFiber\n              );\n          }\n          if (isArrayImpl(newChild) || getIteratorFn(newChild)) {\n            if (null !== key) return null;\n            key = pushDebugInfo(newChild._debugInfo);\n            returnFiber = updateFragment(\n              returnFiber,\n              oldFiber,\n              newChild,\n              lanes,\n              null\n            );\n            currentDebugInfo = key;\n            return returnFiber;\n          }\n          if (\"function\" === typeof newChild.then)\n            return (\n              (key = pushDebugInfo(newChild._debugInfo)),\n              (returnFiber = updateSlot(\n                returnFiber,\n                oldFiber,\n                unwrapThenable(newChild),\n                lanes\n              )),\n              (currentDebugInfo = key),\n              returnFiber\n            );\n          if (newChild.$$typeof === REACT_CONTEXT_TYPE)\n            return updateSlot(\n              returnFiber,\n              oldFiber,\n              readContextDuringReconciliation(returnFiber, newChild),\n              lanes\n            );\n          throwOnInvalidObjectType(returnFiber, newChild);\n        }\n        \"function\" === typeof newChild &&\n          warnOnFunctionType(returnFiber, newChild);\n        \"symbol\" === typeof newChild && warnOnSymbolType(returnFiber, newChild);\n        return null;\n      }\n      function updateFromMap(\n        existingChildren,\n        returnFiber,\n        newIdx,\n        newChild,\n        lanes\n      ) {\n        if (\n          (\"string\" === typeof newChild && \"\" !== newChild) ||\n          \"number\" === typeof newChild ||\n          \"bigint\" === typeof newChild\n        )\n          return (\n            (existingChildren = existingChildren.get(newIdx) || null),\n            updateTextNode(returnFiber, existingChildren, \"\" + newChild, lanes)\n          );\n        if (\"object\" === typeof newChild && null !== newChild) {\n          switch (newChild.$$typeof) {\n            case REACT_ELEMENT_TYPE:\n              return (\n                (newIdx =\n                  existingChildren.get(\n                    null === newChild.key ? newIdx : newChild.key\n                  ) || null),\n                (existingChildren = pushDebugInfo(newChild._debugInfo)),\n                (returnFiber = updateElement(\n                  returnFiber,\n                  newIdx,\n                  newChild,\n                  lanes\n                )),\n                (currentDebugInfo = existingChildren),\n                returnFiber\n              );\n            case REACT_PORTAL_TYPE:\n              return (\n                (existingChildren =\n                  existingChildren.get(\n                    null === newChild.key ? newIdx : newChild.key\n                  ) || null),\n                updatePortal(returnFiber, existingChildren, newChild, lanes)\n              );\n            case REACT_LAZY_TYPE:\n              var _prevDebugInfo7 = pushDebugInfo(newChild._debugInfo);\n              newChild = callLazyInitInDEV(newChild);\n              returnFiber = updateFromMap(\n                existingChildren,\n                returnFiber,\n                newIdx,\n                newChild,\n                lanes\n              );\n              currentDebugInfo = _prevDebugInfo7;\n              return returnFiber;\n          }\n          if (isArrayImpl(newChild) || getIteratorFn(newChild))\n            return (\n              (newIdx = existingChildren.get(newIdx) || null),\n              (existingChildren = pushDebugInfo(newChild._debugInfo)),\n              (returnFiber = updateFragment(\n                returnFiber,\n                newIdx,\n                newChild,\n                lanes,\n                null\n              )),\n              (currentDebugInfo = existingChildren),\n              returnFiber\n            );\n          if (\"function\" === typeof newChild.then)\n            return (\n              (_prevDebugInfo7 = pushDebugInfo(newChild._debugInfo)),\n              (returnFiber = updateFromMap(\n                existingChildren,\n                returnFiber,\n                newIdx,\n                unwrapThenable(newChild),\n                lanes\n              )),\n              (currentDebugInfo = _prevDebugInfo7),\n              returnFiber\n            );\n          if (newChild.$$typeof === REACT_CONTEXT_TYPE)\n            return updateFromMap(\n              existingChildren,\n              returnFiber,\n              newIdx,\n              readContextDuringReconciliation(returnFiber, newChild),\n              lanes\n            );\n          throwOnInvalidObjectType(returnFiber, newChild);\n        }\n        \"function\" === typeof newChild &&\n          warnOnFunctionType(returnFiber, newChild);\n        \"symbol\" === typeof newChild && warnOnSymbolType(returnFiber, newChild);\n        return null;\n      }\n      function warnOnInvalidKey(returnFiber, workInProgress, child, knownKeys) {\n        if (\"object\" !== typeof child || null === child) return knownKeys;\n        switch (child.$$typeof) {\n          case REACT_ELEMENT_TYPE:\n          case REACT_PORTAL_TYPE:\n            warnForMissingKey(returnFiber, workInProgress, child);\n            var key = child.key;\n            if (\"string\" !== typeof key) break;\n            if (null === knownKeys) {\n              knownKeys = new Set();\n              knownKeys.add(key);\n              break;\n            }\n            if (!knownKeys.has(key)) {\n              knownKeys.add(key);\n              break;\n            }\n            runWithFiberInDEV(workInProgress, function () {\n              console.error(\n                \"Encountered two children with the same key, `%s`. Keys should be unique so that components maintain their identity across updates. Non-unique keys may cause children to be duplicated and/or omitted \\u2014 the behavior is unsupported and could change in a future version.\",\n                key\n              );\n            });\n            break;\n          case REACT_LAZY_TYPE:\n            (child = callLazyInitInDEV(child)),\n              warnOnInvalidKey(returnFiber, workInProgress, child, knownKeys);\n        }\n        return knownKeys;\n      }\n      function reconcileChildrenArray(\n        returnFiber,\n        currentFirstChild,\n        newChildren,\n        lanes\n      ) {\n        for (\n          var knownKeys = null,\n            resultingFirstChild = null,\n            previousNewFiber = null,\n            oldFiber = currentFirstChild,\n            newIdx = (currentFirstChild = 0),\n            nextOldFiber = null;\n          null !== oldFiber && newIdx < newChildren.length;\n          newIdx++\n        ) {\n          oldFiber.index > newIdx\n            ? ((nextOldFiber = oldFiber), (oldFiber = null))\n            : (nextOldFiber = oldFiber.sibling);\n          var newFiber = updateSlot(\n            returnFiber,\n            oldFiber,\n            newChildren[newIdx],\n            lanes\n          );\n          if (null === newFiber) {\n            null === oldFiber && (oldFiber = nextOldFiber);\n            break;\n          }\n          knownKeys = warnOnInvalidKey(\n            returnFiber,\n            newFiber,\n            newChildren[newIdx],\n            knownKeys\n          );\n          shouldTrackSideEffects &&\n            oldFiber &&\n            null === newFiber.alternate &&\n            deleteChild(returnFiber, oldFiber);\n          currentFirstChild = placeChild(newFiber, currentFirstChild, newIdx);\n          null === previousNewFiber\n            ? (resultingFirstChild = newFiber)\n            : (previousNewFiber.sibling = newFiber);\n          previousNewFiber = newFiber;\n          oldFiber = nextOldFiber;\n        }\n        if (newIdx === newChildren.length)\n          return (\n            deleteRemainingChildren(returnFiber, oldFiber),\n            isHydrating && pushTreeFork(returnFiber, newIdx),\n            resultingFirstChild\n          );\n        if (null === oldFiber) {\n          for (; newIdx < newChildren.length; newIdx++)\n            (oldFiber = createChild(returnFiber, newChildren[newIdx], lanes)),\n              null !== oldFiber &&\n                ((knownKeys = warnOnInvalidKey(\n                  returnFiber,\n                  oldFiber,\n                  newChildren[newIdx],\n                  knownKeys\n                )),\n                (currentFirstChild = placeChild(\n                  oldFiber,\n                  currentFirstChild,\n                  newIdx\n                )),\n                null === previousNewFiber\n                  ? (resultingFirstChild = oldFiber)\n                  : (previousNewFiber.sibling = oldFiber),\n                (previousNewFiber = oldFiber));\n          isHydrating && pushTreeFork(returnFiber, newIdx);\n          return resultingFirstChild;\n        }\n        for (\n          oldFiber = mapRemainingChildren(oldFiber);\n          newIdx < newChildren.length;\n          newIdx++\n        )\n          (nextOldFiber = updateFromMap(\n            oldFiber,\n            returnFiber,\n            newIdx,\n            newChildren[newIdx],\n            lanes\n          )),\n            null !== nextOldFiber &&\n              ((knownKeys = warnOnInvalidKey(\n                returnFiber,\n                nextOldFiber,\n                newChildren[newIdx],\n                knownKeys\n              )),\n              shouldTrackSideEffects &&\n                null !== nextOldFiber.alternate &&\n                oldFiber.delete(\n                  null === nextOldFiber.key ? newIdx : nextOldFiber.key\n                ),\n              (currentFirstChild = placeChild(\n                nextOldFiber,\n                currentFirstChild,\n                newIdx\n              )),\n              null === previousNewFiber\n                ? (resultingFirstChild = nextOldFiber)\n                : (previousNewFiber.sibling = nextOldFiber),\n              (previousNewFiber = nextOldFiber));\n        shouldTrackSideEffects &&\n          oldFiber.forEach(function (child) {\n            return deleteChild(returnFiber, child);\n          });\n        isHydrating && pushTreeFork(returnFiber, newIdx);\n        return resultingFirstChild;\n      }\n      function reconcileChildrenIterator(\n        returnFiber,\n        currentFirstChild,\n        newChildren,\n        lanes\n      ) {\n        if (null == newChildren)\n          throw Error(\"An iterable object provided no iterator.\");\n        for (\n          var resultingFirstChild = null,\n            previousNewFiber = null,\n            oldFiber = currentFirstChild,\n            newIdx = (currentFirstChild = 0),\n            nextOldFiber = null,\n            knownKeys = null,\n            step = newChildren.next();\n          null !== oldFiber && !step.done;\n          newIdx++, step = newChildren.next()\n        ) {\n          oldFiber.index > newIdx\n            ? ((nextOldFiber = oldFiber), (oldFiber = null))\n            : (nextOldFiber = oldFiber.sibling);\n          var newFiber = updateSlot(returnFiber, oldFiber, step.value, lanes);\n          if (null === newFiber) {\n            null === oldFiber && (oldFiber = nextOldFiber);\n            break;\n          }\n          knownKeys = warnOnInvalidKey(\n            returnFiber,\n            newFiber,\n            step.value,\n            knownKeys\n          );\n          shouldTrackSideEffects &&\n            oldFiber &&\n            null === newFiber.alternate &&\n            deleteChild(returnFiber, oldFiber);\n          currentFirstChild = placeChild(newFiber, currentFirstChild, newIdx);\n          null === previousNewFiber\n            ? (resultingFirstChild = newFiber)\n            : (previousNewFiber.sibling = newFiber);\n          previousNewFiber = newFiber;\n          oldFiber = nextOldFiber;\n        }\n        if (step.done)\n          return (\n            deleteRemainingChildren(returnFiber, oldFiber),\n            isHydrating && pushTreeFork(returnFiber, newIdx),\n            resultingFirstChild\n          );\n        if (null === oldFiber) {\n          for (; !step.done; newIdx++, step = newChildren.next())\n            (oldFiber = createChild(returnFiber, step.value, lanes)),\n              null !== oldFiber &&\n                ((knownKeys = warnOnInvalidKey(\n                  returnFiber,\n                  oldFiber,\n                  step.value,\n                  knownKeys\n                )),\n                (currentFirstChild = placeChild(\n                  oldFiber,\n                  currentFirstChild,\n                  newIdx\n                )),\n                null === previousNewFiber\n                  ? (resultingFirstChild = oldFiber)\n                  : (previousNewFiber.sibling = oldFiber),\n                (previousNewFiber = oldFiber));\n          isHydrating && pushTreeFork(returnFiber, newIdx);\n          return resultingFirstChild;\n        }\n        for (\n          oldFiber = mapRemainingChildren(oldFiber);\n          !step.done;\n          newIdx++, step = newChildren.next()\n        )\n          (nextOldFiber = updateFromMap(\n            oldFiber,\n            returnFiber,\n            newIdx,\n            step.value,\n            lanes\n          )),\n            null !== nextOldFiber &&\n              ((knownKeys = warnOnInvalidKey(\n                returnFiber,\n                nextOldFiber,\n                step.value,\n                knownKeys\n              )),\n              shouldTrackSideEffects &&\n                null !== nextOldFiber.alternate &&\n                oldFiber.delete(\n                  null === nextOldFiber.key ? newIdx : nextOldFiber.key\n                ),\n              (currentFirstChild = placeChild(\n                nextOldFiber,\n                currentFirstChild,\n                newIdx\n              )),\n              null === previousNewFiber\n                ? (resultingFirstChild = nextOldFiber)\n                : (previousNewFiber.sibling = nextOldFiber),\n              (previousNewFiber = nextOldFiber));\n        shouldTrackSideEffects &&\n          oldFiber.forEach(function (child) {\n            return deleteChild(returnFiber, child);\n          });\n        isHydrating && pushTreeFork(returnFiber, newIdx);\n        return resultingFirstChild;\n      }\n      function reconcileChildFibersImpl(\n        returnFiber,\n        currentFirstChild,\n        newChild,\n        lanes\n      ) {\n        \"object\" === typeof newChild &&\n          null !== newChild &&\n          newChild.type === REACT_FRAGMENT_TYPE &&\n          null === newChild.key &&\n          (validateFragmentProps(newChild, null, returnFiber),\n          (newChild = newChild.props.children));\n        if (\"object\" === typeof newChild && null !== newChild) {\n          switch (newChild.$$typeof) {\n            case REACT_ELEMENT_TYPE:\n              var prevDebugInfo = pushDebugInfo(newChild._debugInfo);\n              a: {\n                for (var key = newChild.key; null !== currentFirstChild; ) {\n                  if (currentFirstChild.key === key) {\n                    key = newChild.type;\n                    if (key === REACT_FRAGMENT_TYPE) {\n                      if (7 === currentFirstChild.tag) {\n                        deleteRemainingChildren(\n                          returnFiber,\n                          currentFirstChild.sibling\n                        );\n                        lanes = useFiber(\n                          currentFirstChild,\n                          newChild.props.children\n                        );\n                        lanes.return = returnFiber;\n                        lanes._debugOwner = newChild._owner;\n                        lanes._debugInfo = currentDebugInfo;\n                        validateFragmentProps(newChild, lanes, returnFiber);\n                        returnFiber = lanes;\n                        break a;\n                      }\n                    } else if (\n                      currentFirstChild.elementType === key ||\n                      isCompatibleFamilyForHotReloading(\n                        currentFirstChild,\n                        newChild\n                      ) ||\n                      (\"object\" === typeof key &&\n                        null !== key &&\n                        key.$$typeof === REACT_LAZY_TYPE &&\n                        callLazyInitInDEV(key) === currentFirstChild.type)\n                    ) {\n                      deleteRemainingChildren(\n                        returnFiber,\n                        currentFirstChild.sibling\n                      );\n                      lanes = useFiber(currentFirstChild, newChild.props);\n                      coerceRef(lanes, newChild);\n                      lanes.return = returnFiber;\n                      lanes._debugOwner = newChild._owner;\n                      lanes._debugInfo = currentDebugInfo;\n                      returnFiber = lanes;\n                      break a;\n                    }\n                    deleteRemainingChildren(returnFiber, currentFirstChild);\n                    break;\n                  } else deleteChild(returnFiber, currentFirstChild);\n                  currentFirstChild = currentFirstChild.sibling;\n                }\n                newChild.type === REACT_FRAGMENT_TYPE\n                  ? ((lanes = createFiberFromFragment(\n                      newChild.props.children,\n                      returnFiber.mode,\n                      lanes,\n                      newChild.key\n                    )),\n                    (lanes.return = returnFiber),\n                    (lanes._debugOwner = returnFiber),\n                    (lanes._debugTask = returnFiber._debugTask),\n                    (lanes._debugInfo = currentDebugInfo),\n                    validateFragmentProps(newChild, lanes, returnFiber),\n                    (returnFiber = lanes))\n                  : ((lanes = createFiberFromElement(\n                      newChild,\n                      returnFiber.mode,\n                      lanes\n                    )),\n                    coerceRef(lanes, newChild),\n                    (lanes.return = returnFiber),\n                    (lanes._debugInfo = currentDebugInfo),\n                    (returnFiber = lanes));\n              }\n              returnFiber = placeSingleChild(returnFiber);\n              currentDebugInfo = prevDebugInfo;\n              return returnFiber;\n            case REACT_PORTAL_TYPE:\n              a: {\n                prevDebugInfo = newChild;\n                for (\n                  newChild = prevDebugInfo.key;\n                  null !== currentFirstChild;\n\n                ) {\n                  if (currentFirstChild.key === newChild)\n                    if (\n                      4 === currentFirstChild.tag &&\n                      currentFirstChild.stateNode.containerInfo ===\n                        prevDebugInfo.containerInfo &&\n                      currentFirstChild.stateNode.implementation ===\n                        prevDebugInfo.implementation\n                    ) {\n                      deleteRemainingChildren(\n                        returnFiber,\n                        currentFirstChild.sibling\n                      );\n                      lanes = useFiber(\n                        currentFirstChild,\n                        prevDebugInfo.children || []\n                      );\n                      lanes.return = returnFiber;\n                      returnFiber = lanes;\n                      break a;\n                    } else {\n                      deleteRemainingChildren(returnFiber, currentFirstChild);\n                      break;\n                    }\n                  else deleteChild(returnFiber, currentFirstChild);\n                  currentFirstChild = currentFirstChild.sibling;\n                }\n                lanes = createFiberFromPortal(\n                  prevDebugInfo,\n                  returnFiber.mode,\n                  lanes\n                );\n                lanes.return = returnFiber;\n                returnFiber = lanes;\n              }\n              return placeSingleChild(returnFiber);\n            case REACT_LAZY_TYPE:\n              return (\n                (prevDebugInfo = pushDebugInfo(newChild._debugInfo)),\n                (newChild = callLazyInitInDEV(newChild)),\n                (returnFiber = reconcileChildFibersImpl(\n                  returnFiber,\n                  currentFirstChild,\n                  newChild,\n                  lanes\n                )),\n                (currentDebugInfo = prevDebugInfo),\n                returnFiber\n              );\n          }\n          if (isArrayImpl(newChild))\n            return (\n              (prevDebugInfo = pushDebugInfo(newChild._debugInfo)),\n              (returnFiber = reconcileChildrenArray(\n                returnFiber,\n                currentFirstChild,\n                newChild,\n                lanes\n              )),\n              (currentDebugInfo = prevDebugInfo),\n              returnFiber\n            );\n          if (getIteratorFn(newChild)) {\n            prevDebugInfo = pushDebugInfo(newChild._debugInfo);\n            key = getIteratorFn(newChild);\n            if (\"function\" !== typeof key)\n              throw Error(\n                \"An object is not an iterable. This error is likely caused by a bug in React. Please file an issue.\"\n              );\n            var newChildren = key.call(newChild);\n            if (newChildren === newChild) {\n              if (\n                0 !== returnFiber.tag ||\n                \"[object GeneratorFunction]\" !==\n                  Object.prototype.toString.call(returnFiber.type) ||\n                \"[object Generator]\" !==\n                  Object.prototype.toString.call(newChildren)\n              )\n                didWarnAboutGenerators ||\n                  console.error(\n                    \"Using Iterators as children is unsupported and will likely yield unexpected results because enumerating a generator mutates it. You may convert it to an array with `Array.from()` or the `[...spread]` operator before rendering. You can also use an Iterable that can iterate multiple times over the same items.\"\n                  ),\n                  (didWarnAboutGenerators = !0);\n            } else\n              newChild.entries !== key ||\n                didWarnAboutMaps ||\n                (console.error(\n                  \"Using Maps as children is not supported. Use an array of keyed ReactElements instead.\"\n                ),\n                (didWarnAboutMaps = !0));\n            returnFiber = reconcileChildrenIterator(\n              returnFiber,\n              currentFirstChild,\n              newChildren,\n              lanes\n            );\n            currentDebugInfo = prevDebugInfo;\n            return returnFiber;\n          }\n          if (\"function\" === typeof newChild.then)\n            return (\n              (prevDebugInfo = pushDebugInfo(newChild._debugInfo)),\n              (returnFiber = reconcileChildFibersImpl(\n                returnFiber,\n                currentFirstChild,\n                unwrapThenable(newChild),\n                lanes\n              )),\n              (currentDebugInfo = prevDebugInfo),\n              returnFiber\n            );\n          if (newChild.$$typeof === REACT_CONTEXT_TYPE)\n            return reconcileChildFibersImpl(\n              returnFiber,\n              currentFirstChild,\n              readContextDuringReconciliation(returnFiber, newChild),\n              lanes\n            );\n          throwOnInvalidObjectType(returnFiber, newChild);\n        }\n        if (\n          (\"string\" === typeof newChild && \"\" !== newChild) ||\n          \"number\" === typeof newChild ||\n          \"bigint\" === typeof newChild\n        )\n          return (\n            (prevDebugInfo = \"\" + newChild),\n            null !== currentFirstChild && 6 === currentFirstChild.tag\n              ? (deleteRemainingChildren(\n                  returnFiber,\n                  currentFirstChild.sibling\n                ),\n                (lanes = useFiber(currentFirstChild, prevDebugInfo)),\n                (lanes.return = returnFiber),\n                (returnFiber = lanes))\n              : (deleteRemainingChildren(returnFiber, currentFirstChild),\n                (lanes = createFiberFromText(\n                  prevDebugInfo,\n                  returnFiber.mode,\n                  lanes\n                )),\n                (lanes.return = returnFiber),\n                (lanes._debugOwner = returnFiber),\n                (lanes._debugTask = returnFiber._debugTask),\n                (lanes._debugInfo = currentDebugInfo),\n                (returnFiber = lanes)),\n            placeSingleChild(returnFiber)\n          );\n        \"function\" === typeof newChild &&\n          warnOnFunctionType(returnFiber, newChild);\n        \"symbol\" === typeof newChild && warnOnSymbolType(returnFiber, newChild);\n        return deleteRemainingChildren(returnFiber, currentFirstChild);\n      }\n      return function (returnFiber, currentFirstChild, newChild, lanes) {\n        var prevDebugInfo = currentDebugInfo;\n        currentDebugInfo = null;\n        try {\n          thenableIndexCounter = 0;\n          var firstChildFiber = reconcileChildFibersImpl(\n            returnFiber,\n            currentFirstChild,\n            newChild,\n            lanes\n          );\n          thenableState = null;\n          return firstChildFiber;\n        } catch (x) {\n          if (x === SuspenseException || x === SuspenseActionException) throw x;\n          var fiber = createFiber(29, x, null, returnFiber.mode);\n          fiber.lanes = lanes;\n          fiber.return = returnFiber;\n          var debugInfo = (fiber._debugInfo = currentDebugInfo);\n          fiber._debugOwner = returnFiber._debugOwner;\n          fiber._debugTask = returnFiber._debugTask;\n          if (null != debugInfo)\n            for (var i = debugInfo.length - 1; 0 <= i; i--)\n              if (\"string\" === typeof debugInfo[i].stack) {\n                fiber._debugOwner = debugInfo[i];\n                fiber._debugTask = debugInfo[i].debugTask;\n                break;\n              }\n          return fiber;\n        } finally {\n          currentDebugInfo = prevDebugInfo;\n        }\n      };\n    }\n    function pushPrimaryTreeSuspenseHandler(handler) {\n      var current = handler.alternate;\n      push(\n        suspenseStackCursor,\n        suspenseStackCursor.current & SubtreeSuspenseContextMask,\n        handler\n      );\n      push(suspenseHandlerStackCursor, handler, handler);\n      null === shellBoundary &&\n        (null === current || null !== currentTreeHiddenStackCursor.current\n          ? (shellBoundary = handler)\n          : null !== current.memoizedState && (shellBoundary = handler));\n    }\n    function pushOffscreenSuspenseHandler(fiber) {\n      if (22 === fiber.tag) {\n        if (\n          (push(suspenseStackCursor, suspenseStackCursor.current, fiber),\n          push(suspenseHandlerStackCursor, fiber, fiber),\n          null === shellBoundary)\n        ) {\n          var current = fiber.alternate;\n          null !== current &&\n            null !== current.memoizedState &&\n            (shellBoundary = fiber);\n        }\n      } else reuseSuspenseHandlerOnStack(fiber);\n    }\n    function reuseSuspenseHandlerOnStack(fiber) {\n      push(suspenseStackCursor, suspenseStackCursor.current, fiber);\n      push(\n        suspenseHandlerStackCursor,\n        suspenseHandlerStackCursor.current,\n        fiber\n      );\n    }\n    function popSuspenseHandler(fiber) {\n      pop(suspenseHandlerStackCursor, fiber);\n      shellBoundary === fiber && (shellBoundary = null);\n      pop(suspenseStackCursor, fiber);\n    }\n    function findFirstSuspended(row) {\n      for (var node = row; null !== node; ) {\n        if (13 === node.tag) {\n          var state = node.memoizedState;\n          if (\n            null !== state &&\n            ((state = state.dehydrated),\n            null === state ||\n              state.data === SUSPENSE_PENDING_START_DATA ||\n              isSuspenseInstanceFallback(state))\n          )\n            return node;\n        } else if (\n          19 === node.tag &&\n          void 0 !== node.memoizedProps.revealOrder\n        ) {\n          if (0 !== (node.flags & 128)) return node;\n        } else if (null !== node.child) {\n          node.child.return = node;\n          node = node.child;\n          continue;\n        }\n        if (node === row) break;\n        for (; null === node.sibling; ) {\n          if (null === node.return || node.return === row) return null;\n          node = node.return;\n        }\n        node.sibling.return = node.return;\n        node = node.sibling;\n      }\n      return null;\n    }\n    function warnOnInvalidCallback(callback) {\n      if (null !== callback && \"function\" !== typeof callback) {\n        var key = String(callback);\n        didWarnOnInvalidCallback.has(key) ||\n          (didWarnOnInvalidCallback.add(key),\n          console.error(\n            \"Expected the last optional `callback` argument to be a function. Instead received: %s.\",\n            callback\n          ));\n      }\n    }\n    function applyDerivedStateFromProps(\n      workInProgress,\n      ctor,\n      getDerivedStateFromProps,\n      nextProps\n    ) {\n      var prevState = workInProgress.memoizedState,\n        partialState = getDerivedStateFromProps(nextProps, prevState);\n      if (workInProgress.mode & StrictLegacyMode) {\n        setIsStrictModeForDevtools(!0);\n        try {\n          partialState = getDerivedStateFromProps(nextProps, prevState);\n        } finally {\n          setIsStrictModeForDevtools(!1);\n        }\n      }\n      void 0 === partialState &&\n        ((ctor = getComponentNameFromType(ctor) || \"Component\"),\n        didWarnAboutUndefinedDerivedState.has(ctor) ||\n          (didWarnAboutUndefinedDerivedState.add(ctor),\n          console.error(\n            \"%s.getDerivedStateFromProps(): A valid state object (or null) must be returned. You have returned undefined.\",\n            ctor\n          )));\n      prevState =\n        null === partialState || void 0 === partialState\n          ? prevState\n          : assign({}, prevState, partialState);\n      workInProgress.memoizedState = prevState;\n      0 === workInProgress.lanes &&\n        (workInProgress.updateQueue.baseState = prevState);\n    }\n    function checkShouldComponentUpdate(\n      workInProgress,\n      ctor,\n      oldProps,\n      newProps,\n      oldState,\n      newState,\n      nextContext\n    ) {\n      var instance = workInProgress.stateNode;\n      if (\"function\" === typeof instance.shouldComponentUpdate) {\n        oldProps = instance.shouldComponentUpdate(\n          newProps,\n          newState,\n          nextContext\n        );\n        if (workInProgress.mode & StrictLegacyMode) {\n          setIsStrictModeForDevtools(!0);\n          try {\n            oldProps = instance.shouldComponentUpdate(\n              newProps,\n              newState,\n              nextContext\n            );\n          } finally {\n            setIsStrictModeForDevtools(!1);\n          }\n        }\n        void 0 === oldProps &&\n          console.error(\n            \"%s.shouldComponentUpdate(): Returned undefined instead of a boolean value. Make sure to return true or false.\",\n            getComponentNameFromType(ctor) || \"Component\"\n          );\n        return oldProps;\n      }\n      return ctor.prototype && ctor.prototype.isPureReactComponent\n        ? !shallowEqual(oldProps, newProps) || !shallowEqual(oldState, newState)\n        : !0;\n    }\n    function callComponentWillReceiveProps(\n      workInProgress,\n      instance,\n      newProps,\n      nextContext\n    ) {\n      var oldState = instance.state;\n      \"function\" === typeof instance.componentWillReceiveProps &&\n        instance.componentWillReceiveProps(newProps, nextContext);\n      \"function\" === typeof instance.UNSAFE_componentWillReceiveProps &&\n        instance.UNSAFE_componentWillReceiveProps(newProps, nextContext);\n      instance.state !== oldState &&\n        ((workInProgress =\n          getComponentNameFromFiber(workInProgress) || \"Component\"),\n        didWarnAboutStateAssignmentForComponent.has(workInProgress) ||\n          (didWarnAboutStateAssignmentForComponent.add(workInProgress),\n          console.error(\n            \"%s.componentWillReceiveProps(): Assigning directly to this.state is deprecated (except inside a component's constructor). Use setState instead.\",\n            workInProgress\n          )),\n        classComponentUpdater.enqueueReplaceState(\n          instance,\n          instance.state,\n          null\n        ));\n    }\n    function resolveClassComponentProps(Component, baseProps) {\n      var newProps = baseProps;\n      if (\"ref\" in baseProps) {\n        newProps = {};\n        for (var propName in baseProps)\n          \"ref\" !== propName && (newProps[propName] = baseProps[propName]);\n      }\n      if ((Component = Component.defaultProps)) {\n        newProps === baseProps && (newProps = assign({}, newProps));\n        for (var _propName in Component)\n          void 0 === newProps[_propName] &&\n            (newProps[_propName] = Component[_propName]);\n      }\n      return newProps;\n    }\n    function defaultOnUncaughtError(error) {\n      reportGlobalError(error);\n      console.warn(\n        \"%s\\n\\n%s\\n\",\n        componentName\n          ? \"An error occurred in the <\" + componentName + \"> component.\"\n          : \"An error occurred in one of your React components.\",\n        \"Consider adding an error boundary to your tree to customize error handling behavior.\\nVisit https://react.dev/link/error-boundaries to learn more about error boundaries.\"\n      );\n    }\n    function defaultOnCaughtError(error) {\n      var componentNameMessage = componentName\n          ? \"The above error occurred in the <\" + componentName + \"> component.\"\n          : \"The above error occurred in one of your React components.\",\n        recreateMessage =\n          \"React will try to recreate this component tree from scratch using the error boundary you provided, \" +\n          ((errorBoundaryName || \"Anonymous\") + \".\");\n      if (\n        \"object\" === typeof error &&\n        null !== error &&\n        \"string\" === typeof error.environmentName\n      ) {\n        var JSCompiler_inline_result = error.environmentName;\n        error = [\n          \"%o\\n\\n%s\\n\\n%s\\n\",\n          error,\n          componentNameMessage,\n          recreateMessage\n        ].slice(0);\n        \"string\" === typeof error[0]\n          ? error.splice(\n              0,\n              1,\n              badgeFormat + error[0],\n              badgeStyle,\n              pad + JSCompiler_inline_result + pad,\n              resetStyle\n            )\n          : error.splice(\n              0,\n              0,\n              badgeFormat,\n              badgeStyle,\n              pad + JSCompiler_inline_result + pad,\n              resetStyle\n            );\n        error.unshift(console);\n        JSCompiler_inline_result = bind.apply(console.error, error);\n        JSCompiler_inline_result();\n      } else\n        console.error(\n          \"%o\\n\\n%s\\n\\n%s\\n\",\n          error,\n          componentNameMessage,\n          recreateMessage\n        );\n    }\n    function defaultOnRecoverableError(error) {\n      reportGlobalError(error);\n    }\n    function logUncaughtError(root, errorInfo) {\n      try {\n        componentName = errorInfo.source\n          ? getComponentNameFromFiber(errorInfo.source)\n          : null;\n        errorBoundaryName = null;\n        var error = errorInfo.value;\n        if (null !== ReactSharedInternals.actQueue)\n          ReactSharedInternals.thrownErrors.push(error);\n        else {\n          var onUncaughtError = root.onUncaughtError;\n          onUncaughtError(error, { componentStack: errorInfo.stack });\n        }\n      } catch (e$5) {\n        setTimeout(function () {\n          throw e$5;\n        });\n      }\n    }\n    function logCaughtError(root, boundary, errorInfo) {\n      try {\n        componentName = errorInfo.source\n          ? getComponentNameFromFiber(errorInfo.source)\n          : null;\n        errorBoundaryName = getComponentNameFromFiber(boundary);\n        var onCaughtError = root.onCaughtError;\n        onCaughtError(errorInfo.value, {\n          componentStack: errorInfo.stack,\n          errorBoundary: 1 === boundary.tag ? boundary.stateNode : null\n        });\n      } catch (e$6) {\n        setTimeout(function () {\n          throw e$6;\n        });\n      }\n    }\n    function createRootErrorUpdate(root, errorInfo, lane) {\n      lane = createUpdate(lane);\n      lane.tag = CaptureUpdate;\n      lane.payload = { element: null };\n      lane.callback = function () {\n        runWithFiberInDEV(errorInfo.source, logUncaughtError, root, errorInfo);\n      };\n      return lane;\n    }\n    function createClassErrorUpdate(lane) {\n      lane = createUpdate(lane);\n      lane.tag = CaptureUpdate;\n      return lane;\n    }\n    function initializeClassErrorUpdate(update, root, fiber, errorInfo) {\n      var getDerivedStateFromError = fiber.type.getDerivedStateFromError;\n      if (\"function\" === typeof getDerivedStateFromError) {\n        var error = errorInfo.value;\n        update.payload = function () {\n          return getDerivedStateFromError(error);\n        };\n        update.callback = function () {\n          markFailedErrorBoundaryForHotReloading(fiber);\n          runWithFiberInDEV(\n            errorInfo.source,\n            logCaughtError,\n            root,\n            fiber,\n            errorInfo\n          );\n        };\n      }\n      var inst = fiber.stateNode;\n      null !== inst &&\n        \"function\" === typeof inst.componentDidCatch &&\n        (update.callback = function () {\n          markFailedErrorBoundaryForHotReloading(fiber);\n          runWithFiberInDEV(\n            errorInfo.source,\n            logCaughtError,\n            root,\n            fiber,\n            errorInfo\n          );\n          \"function\" !== typeof getDerivedStateFromError &&\n            (null === legacyErrorBoundariesThatAlreadyFailed\n              ? (legacyErrorBoundariesThatAlreadyFailed = new Set([this]))\n              : legacyErrorBoundariesThatAlreadyFailed.add(this));\n          callComponentDidCatchInDEV(this, errorInfo);\n          \"function\" === typeof getDerivedStateFromError ||\n            (0 === (fiber.lanes & 2) &&\n              console.error(\n                \"%s: Error boundaries should implement getDerivedStateFromError(). In that method, return a state update to display an error message or fallback UI.\",\n                getComponentNameFromFiber(fiber) || \"Unknown\"\n              ));\n        });\n    }\n    function throwException(\n      root,\n      returnFiber,\n      sourceFiber,\n      value,\n      rootRenderLanes\n    ) {\n      sourceFiber.flags |= 32768;\n      isDevToolsPresent && restorePendingUpdaters(root, rootRenderLanes);\n      if (\n        null !== value &&\n        \"object\" === typeof value &&\n        \"function\" === typeof value.then\n      ) {\n        returnFiber = sourceFiber.alternate;\n        null !== returnFiber &&\n          propagateParentContextChanges(\n            returnFiber,\n            sourceFiber,\n            rootRenderLanes,\n            !0\n          );\n        isHydrating && (didSuspendOrErrorDEV = !0);\n        sourceFiber = suspenseHandlerStackCursor.current;\n        if (null !== sourceFiber) {\n          switch (sourceFiber.tag) {\n            case 13:\n              return (\n                null === shellBoundary\n                  ? renderDidSuspendDelayIfPossible()\n                  : null === sourceFiber.alternate &&\n                    workInProgressRootExitStatus === RootInProgress &&\n                    (workInProgressRootExitStatus = RootSuspended),\n                (sourceFiber.flags &= -257),\n                (sourceFiber.flags |= 65536),\n                (sourceFiber.lanes = rootRenderLanes),\n                value === noopSuspenseyCommitThenable\n                  ? (sourceFiber.flags |= 16384)\n                  : ((returnFiber = sourceFiber.updateQueue),\n                    null === returnFiber\n                      ? (sourceFiber.updateQueue = new Set([value]))\n                      : returnFiber.add(value),\n                    attachPingListener(root, value, rootRenderLanes)),\n                !1\n              );\n            case 22:\n              return (\n                (sourceFiber.flags |= 65536),\n                value === noopSuspenseyCommitThenable\n                  ? (sourceFiber.flags |= 16384)\n                  : ((returnFiber = sourceFiber.updateQueue),\n                    null === returnFiber\n                      ? ((returnFiber = {\n                          transitions: null,\n                          markerInstances: null,\n                          retryQueue: new Set([value])\n                        }),\n                        (sourceFiber.updateQueue = returnFiber))\n                      : ((sourceFiber = returnFiber.retryQueue),\n                        null === sourceFiber\n                          ? (returnFiber.retryQueue = new Set([value]))\n                          : sourceFiber.add(value)),\n                    attachPingListener(root, value, rootRenderLanes)),\n                !1\n              );\n          }\n          throw Error(\n            \"Unexpected Suspense handler tag (\" +\n              sourceFiber.tag +\n              \"). This is a bug in React.\"\n          );\n        }\n        attachPingListener(root, value, rootRenderLanes);\n        renderDidSuspendDelayIfPossible();\n        return !1;\n      }\n      if (isHydrating)\n        return (\n          (didSuspendOrErrorDEV = !0),\n          (returnFiber = suspenseHandlerStackCursor.current),\n          null !== returnFiber\n            ? (0 === (returnFiber.flags & 65536) && (returnFiber.flags |= 256),\n              (returnFiber.flags |= 65536),\n              (returnFiber.lanes = rootRenderLanes),\n              value !== HydrationMismatchException &&\n                queueHydrationError(\n                  createCapturedValueAtFiber(\n                    Error(\n                      \"There was an error while hydrating but React was able to recover by instead client rendering from the nearest Suspense boundary.\",\n                      { cause: value }\n                    ),\n                    sourceFiber\n                  )\n                ))\n            : (value !== HydrationMismatchException &&\n                queueHydrationError(\n                  createCapturedValueAtFiber(\n                    Error(\n                      \"There was an error while hydrating but React was able to recover by instead client rendering the entire root.\",\n                      { cause: value }\n                    ),\n                    sourceFiber\n                  )\n                ),\n              (root = root.current.alternate),\n              (root.flags |= 65536),\n              (rootRenderLanes &= -rootRenderLanes),\n              (root.lanes |= rootRenderLanes),\n              (value = createCapturedValueAtFiber(value, sourceFiber)),\n              (rootRenderLanes = createRootErrorUpdate(\n                root.stateNode,\n                value,\n                rootRenderLanes\n              )),\n              enqueueCapturedUpdate(root, rootRenderLanes),\n              workInProgressRootExitStatus !== RootSuspendedWithDelay &&\n                (workInProgressRootExitStatus = RootErrored)),\n          !1\n        );\n      var error = createCapturedValueAtFiber(\n        Error(\n          \"There was an error during concurrent rendering but React was able to recover by instead synchronously rendering the entire root.\",\n          { cause: value }\n        ),\n        sourceFiber\n      );\n      null === workInProgressRootConcurrentErrors\n        ? (workInProgressRootConcurrentErrors = [error])\n        : workInProgressRootConcurrentErrors.push(error);\n      workInProgressRootExitStatus !== RootSuspendedWithDelay &&\n        (workInProgressRootExitStatus = RootErrored);\n      if (null === returnFiber) return !0;\n      value = createCapturedValueAtFiber(value, sourceFiber);\n      sourceFiber = returnFiber;\n      do {\n        switch (sourceFiber.tag) {\n          case 3:\n            return (\n              (sourceFiber.flags |= 65536),\n              (root = rootRenderLanes & -rootRenderLanes),\n              (sourceFiber.lanes |= root),\n              (root = createRootErrorUpdate(\n                sourceFiber.stateNode,\n                value,\n                root\n              )),\n              enqueueCapturedUpdate(sourceFiber, root),\n              !1\n            );\n          case 1:\n            if (\n              ((returnFiber = sourceFiber.type),\n              (error = sourceFiber.stateNode),\n              0 === (sourceFiber.flags & 128) &&\n                (\"function\" === typeof returnFiber.getDerivedStateFromError ||\n                  (null !== error &&\n                    \"function\" === typeof error.componentDidCatch &&\n                    (null === legacyErrorBoundariesThatAlreadyFailed ||\n                      !legacyErrorBoundariesThatAlreadyFailed.has(error)))))\n            )\n              return (\n                (sourceFiber.flags |= 65536),\n                (rootRenderLanes &= -rootRenderLanes),\n                (sourceFiber.lanes |= rootRenderLanes),\n                (rootRenderLanes = createClassErrorUpdate(rootRenderLanes)),\n                initializeClassErrorUpdate(\n                  rootRenderLanes,\n                  root,\n                  sourceFiber,\n                  value\n                ),\n                enqueueCapturedUpdate(sourceFiber, rootRenderLanes),\n                !1\n              );\n        }\n        sourceFiber = sourceFiber.return;\n      } while (null !== sourceFiber);\n      return !1;\n    }\n    function reconcileChildren(\n      current,\n      workInProgress,\n      nextChildren,\n      renderLanes\n    ) {\n      workInProgress.child =\n        null === current\n          ? mountChildFibers(workInProgress, null, nextChildren, renderLanes)\n          : reconcileChildFibers(\n              workInProgress,\n              current.child,\n              nextChildren,\n              renderLanes\n            );\n    }\n    function updateForwardRef(\n      current,\n      workInProgress,\n      Component,\n      nextProps,\n      renderLanes\n    ) {\n      Component = Component.render;\n      var ref = workInProgress.ref;\n      if (\"ref\" in nextProps) {\n        var propsWithoutRef = {};\n        for (var key in nextProps)\n          \"ref\" !== key && (propsWithoutRef[key] = nextProps[key]);\n      } else propsWithoutRef = nextProps;\n      prepareToReadContext(workInProgress);\n      markComponentRenderStarted(workInProgress);\n      nextProps = renderWithHooks(\n        current,\n        workInProgress,\n        Component,\n        propsWithoutRef,\n        ref,\n        renderLanes\n      );\n      key = checkDidRenderIdHook();\n      markComponentRenderStopped();\n      if (null !== current && !didReceiveUpdate)\n        return (\n          bailoutHooks(current, workInProgress, renderLanes),\n          bailoutOnAlreadyFinishedWork(current, workInProgress, renderLanes)\n        );\n      isHydrating && key && pushMaterializedTreeId(workInProgress);\n      workInProgress.flags |= 1;\n      reconcileChildren(current, workInProgress, nextProps, renderLanes);\n      return workInProgress.child;\n    }\n    function updateMemoComponent(\n      current,\n      workInProgress,\n      Component,\n      nextProps,\n      renderLanes\n    ) {\n      if (null === current) {\n        var type = Component.type;\n        if (\n          \"function\" === typeof type &&\n          !shouldConstruct(type) &&\n          void 0 === type.defaultProps &&\n          null === Component.compare\n        )\n          return (\n            (Component = resolveFunctionForHotReloading(type)),\n            (workInProgress.tag = 15),\n            (workInProgress.type = Component),\n            validateFunctionComponentInDev(workInProgress, type),\n            updateSimpleMemoComponent(\n              current,\n              workInProgress,\n              Component,\n              nextProps,\n              renderLanes\n            )\n          );\n        current = createFiberFromTypeAndProps(\n          Component.type,\n          null,\n          nextProps,\n          workInProgress,\n          workInProgress.mode,\n          renderLanes\n        );\n        current.ref = workInProgress.ref;\n        current.return = workInProgress;\n        return (workInProgress.child = current);\n      }\n      type = current.child;\n      if (!checkScheduledUpdateOrContext(current, renderLanes)) {\n        var prevProps = type.memoizedProps;\n        Component = Component.compare;\n        Component = null !== Component ? Component : shallowEqual;\n        if (\n          Component(prevProps, nextProps) &&\n          current.ref === workInProgress.ref\n        )\n          return bailoutOnAlreadyFinishedWork(\n            current,\n            workInProgress,\n            renderLanes\n          );\n      }\n      workInProgress.flags |= 1;\n      current = createWorkInProgress(type, nextProps);\n      current.ref = workInProgress.ref;\n      current.return = workInProgress;\n      return (workInProgress.child = current);\n    }\n    function updateSimpleMemoComponent(\n      current,\n      workInProgress,\n      Component,\n      nextProps,\n      renderLanes\n    ) {\n      if (null !== current) {\n        var prevProps = current.memoizedProps;\n        if (\n          shallowEqual(prevProps, nextProps) &&\n          current.ref === workInProgress.ref &&\n          workInProgress.type === current.type\n        )\n          if (\n            ((didReceiveUpdate = !1),\n            (workInProgress.pendingProps = nextProps = prevProps),\n            checkScheduledUpdateOrContext(current, renderLanes))\n          )\n            0 !== (current.flags & 131072) && (didReceiveUpdate = !0);\n          else\n            return (\n              (workInProgress.lanes = current.lanes),\n              bailoutOnAlreadyFinishedWork(current, workInProgress, renderLanes)\n            );\n      }\n      return updateFunctionComponent(\n        current,\n        workInProgress,\n        Component,\n        nextProps,\n        renderLanes\n      );\n    }\n    function updateOffscreenComponent(current, workInProgress, renderLanes) {\n      var nextProps = workInProgress.pendingProps,\n        nextChildren = nextProps.children,\n        prevState = null !== current ? current.memoizedState : null;\n      if (\"hidden\" === nextProps.mode) {\n        if (0 !== (workInProgress.flags & 128)) {\n          nextProps =\n            null !== prevState\n              ? prevState.baseLanes | renderLanes\n              : renderLanes;\n          if (null !== current) {\n            nextChildren = workInProgress.child = current.child;\n            for (prevState = 0; null !== nextChildren; )\n              (prevState =\n                prevState | nextChildren.lanes | nextChildren.childLanes),\n                (nextChildren = nextChildren.sibling);\n            workInProgress.childLanes = prevState & ~nextProps;\n          } else (workInProgress.childLanes = 0), (workInProgress.child = null);\n          return deferHiddenOffscreenComponent(\n            current,\n            workInProgress,\n            nextProps,\n            renderLanes\n          );\n        }\n        if (0 !== (renderLanes & 536870912))\n          (workInProgress.memoizedState = { baseLanes: 0, cachePool: null }),\n            null !== current &&\n              pushTransition(\n                workInProgress,\n                null !== prevState ? prevState.cachePool : null\n              ),\n            null !== prevState\n              ? pushHiddenContext(workInProgress, prevState)\n              : reuseHiddenContextOnStack(workInProgress),\n            pushOffscreenSuspenseHandler(workInProgress);\n        else\n          return (\n            (workInProgress.lanes = workInProgress.childLanes = 536870912),\n            deferHiddenOffscreenComponent(\n              current,\n              workInProgress,\n              null !== prevState\n                ? prevState.baseLanes | renderLanes\n                : renderLanes,\n              renderLanes\n            )\n          );\n      } else\n        null !== prevState\n          ? (pushTransition(workInProgress, prevState.cachePool),\n            pushHiddenContext(workInProgress, prevState),\n            reuseSuspenseHandlerOnStack(workInProgress),\n            (workInProgress.memoizedState = null))\n          : (null !== current && pushTransition(workInProgress, null),\n            reuseHiddenContextOnStack(workInProgress),\n            reuseSuspenseHandlerOnStack(workInProgress));\n      reconcileChildren(current, workInProgress, nextChildren, renderLanes);\n      return workInProgress.child;\n    }\n    function deferHiddenOffscreenComponent(\n      current,\n      workInProgress,\n      nextBaseLanes,\n      renderLanes\n    ) {\n      var JSCompiler_inline_result = peekCacheFromPool();\n      JSCompiler_inline_result =\n        null === JSCompiler_inline_result\n          ? null\n          : {\n              parent: CacheContext._currentValue,\n              pool: JSCompiler_inline_result\n            };\n      workInProgress.memoizedState = {\n        baseLanes: nextBaseLanes,\n        cachePool: JSCompiler_inline_result\n      };\n      null !== current && pushTransition(workInProgress, null);\n      reuseHiddenContextOnStack(workInProgress);\n      pushOffscreenSuspenseHandler(workInProgress);\n      null !== current &&\n        propagateParentContextChanges(current, workInProgress, renderLanes, !0);\n      return null;\n    }\n    function markRef(current, workInProgress) {\n      var ref = workInProgress.ref;\n      if (null === ref)\n        null !== current &&\n          null !== current.ref &&\n          (workInProgress.flags |= 4194816);\n      else {\n        if (\"function\" !== typeof ref && \"object\" !== typeof ref)\n          throw Error(\n            \"Expected ref to be a function, an object returned by React.createRef(), or undefined/null.\"\n          );\n        if (null === current || current.ref !== ref)\n          workInProgress.flags |= 4194816;\n      }\n    }\n    function updateFunctionComponent(\n      current,\n      workInProgress,\n      Component,\n      nextProps,\n      renderLanes\n    ) {\n      if (\n        Component.prototype &&\n        \"function\" === typeof Component.prototype.render\n      ) {\n        var componentName = getComponentNameFromType(Component) || \"Unknown\";\n        didWarnAboutBadClass[componentName] ||\n          (console.error(\n            \"The <%s /> component appears to have a render method, but doesn't extend React.Component. This is likely to cause errors. Change %s to extend React.Component instead.\",\n            componentName,\n            componentName\n          ),\n          (didWarnAboutBadClass[componentName] = !0));\n      }\n      workInProgress.mode & StrictLegacyMode &&\n        ReactStrictModeWarnings.recordLegacyContextWarning(\n          workInProgress,\n          null\n        );\n      null === current &&\n        (validateFunctionComponentInDev(workInProgress, workInProgress.type),\n        Component.contextTypes &&\n          ((componentName = getComponentNameFromType(Component) || \"Unknown\"),\n          didWarnAboutContextTypes[componentName] ||\n            ((didWarnAboutContextTypes[componentName] = !0),\n            console.error(\n              \"%s uses the legacy contextTypes API which was removed in React 19. Use React.createContext() with React.useContext() instead. (https://react.dev/link/legacy-context)\",\n              componentName\n            ))));\n      prepareToReadContext(workInProgress);\n      markComponentRenderStarted(workInProgress);\n      Component = renderWithHooks(\n        current,\n        workInProgress,\n        Component,\n        nextProps,\n        void 0,\n        renderLanes\n      );\n      nextProps = checkDidRenderIdHook();\n      markComponentRenderStopped();\n      if (null !== current && !didReceiveUpdate)\n        return (\n          bailoutHooks(current, workInProgress, renderLanes),\n          bailoutOnAlreadyFinishedWork(current, workInProgress, renderLanes)\n        );\n      isHydrating && nextProps && pushMaterializedTreeId(workInProgress);\n      workInProgress.flags |= 1;\n      reconcileChildren(current, workInProgress, Component, renderLanes);\n      return workInProgress.child;\n    }\n    function replayFunctionComponent(\n      current,\n      workInProgress,\n      nextProps,\n      Component,\n      secondArg,\n      renderLanes\n    ) {\n      prepareToReadContext(workInProgress);\n      markComponentRenderStarted(workInProgress);\n      hookTypesUpdateIndexDev = -1;\n      ignorePreviousDependencies =\n        null !== current && current.type !== workInProgress.type;\n      workInProgress.updateQueue = null;\n      nextProps = renderWithHooksAgain(\n        workInProgress,\n        Component,\n        nextProps,\n        secondArg\n      );\n      finishRenderingHooks(current, workInProgress);\n      Component = checkDidRenderIdHook();\n      markComponentRenderStopped();\n      if (null !== current && !didReceiveUpdate)\n        return (\n          bailoutHooks(current, workInProgress, renderLanes),\n          bailoutOnAlreadyFinishedWork(current, workInProgress, renderLanes)\n        );\n      isHydrating && Component && pushMaterializedTreeId(workInProgress);\n      workInProgress.flags |= 1;\n      reconcileChildren(current, workInProgress, nextProps, renderLanes);\n      return workInProgress.child;\n    }\n    function updateClassComponent(\n      current,\n      workInProgress,\n      Component,\n      nextProps,\n      renderLanes\n    ) {\n      switch (shouldErrorImpl(workInProgress)) {\n        case !1:\n          var _instance = workInProgress.stateNode,\n            state = new workInProgress.type(\n              workInProgress.memoizedProps,\n              _instance.context\n            ).state;\n          _instance.updater.enqueueSetState(_instance, state, null);\n          break;\n        case !0:\n          workInProgress.flags |= 128;\n          workInProgress.flags |= 65536;\n          _instance = Error(\"Simulated error coming from DevTools\");\n          var lane = renderLanes & -renderLanes;\n          workInProgress.lanes |= lane;\n          state = workInProgressRoot;\n          if (null === state)\n            throw Error(\n              \"Expected a work-in-progress root. This is a bug in React. Please file an issue.\"\n            );\n          lane = createClassErrorUpdate(lane);\n          initializeClassErrorUpdate(\n            lane,\n            state,\n            workInProgress,\n            createCapturedValueAtFiber(_instance, workInProgress)\n          );\n          enqueueCapturedUpdate(workInProgress, lane);\n      }\n      prepareToReadContext(workInProgress);\n      if (null === workInProgress.stateNode) {\n        state = emptyContextObject;\n        _instance = Component.contextType;\n        \"contextType\" in Component &&\n          null !== _instance &&\n          (void 0 === _instance || _instance.$$typeof !== REACT_CONTEXT_TYPE) &&\n          !didWarnAboutInvalidateContextType.has(Component) &&\n          (didWarnAboutInvalidateContextType.add(Component),\n          (lane =\n            void 0 === _instance\n              ? \" However, it is set to undefined. This can be caused by a typo or by mixing up named and default imports. This can also happen due to a circular dependency, so try moving the createContext() call to a separate file.\"\n              : \"object\" !== typeof _instance\n                ? \" However, it is set to a \" + typeof _instance + \".\"\n                : _instance.$$typeof === REACT_CONSUMER_TYPE\n                  ? \" Did you accidentally pass the Context.Consumer instead?\"\n                  : \" However, it is set to an object with keys {\" +\n                    Object.keys(_instance).join(\", \") +\n                    \"}.\"),\n          console.error(\n            \"%s defines an invalid contextType. contextType should point to the Context object returned by React.createContext().%s\",\n            getComponentNameFromType(Component) || \"Component\",\n            lane\n          ));\n        \"object\" === typeof _instance &&\n          null !== _instance &&\n          (state = readContext(_instance));\n        _instance = new Component(nextProps, state);\n        if (workInProgress.mode & StrictLegacyMode) {\n          setIsStrictModeForDevtools(!0);\n          try {\n            _instance = new Component(nextProps, state);\n          } finally {\n            setIsStrictModeForDevtools(!1);\n          }\n        }\n        state = workInProgress.memoizedState =\n          null !== _instance.state && void 0 !== _instance.state\n            ? _instance.state\n            : null;\n        _instance.updater = classComponentUpdater;\n        workInProgress.stateNode = _instance;\n        _instance._reactInternals = workInProgress;\n        _instance._reactInternalInstance = fakeInternalInstance;\n        \"function\" === typeof Component.getDerivedStateFromProps &&\n          null === state &&\n          ((state = getComponentNameFromType(Component) || \"Component\"),\n          didWarnAboutUninitializedState.has(state) ||\n            (didWarnAboutUninitializedState.add(state),\n            console.error(\n              \"`%s` uses `getDerivedStateFromProps` but its initial state is %s. This is not recommended. Instead, define the initial state by assigning an object to `this.state` in the constructor of `%s`. This ensures that `getDerivedStateFromProps` arguments have a consistent shape.\",\n              state,\n              null === _instance.state ? \"null\" : \"undefined\",\n              state\n            )));\n        if (\n          \"function\" === typeof Component.getDerivedStateFromProps ||\n          \"function\" === typeof _instance.getSnapshotBeforeUpdate\n        ) {\n          var foundWillUpdateName = (lane = state = null);\n          \"function\" === typeof _instance.componentWillMount &&\n          !0 !== _instance.componentWillMount.__suppressDeprecationWarning\n            ? (state = \"componentWillMount\")\n            : \"function\" === typeof _instance.UNSAFE_componentWillMount &&\n              (state = \"UNSAFE_componentWillMount\");\n          \"function\" === typeof _instance.componentWillReceiveProps &&\n          !0 !==\n            _instance.componentWillReceiveProps.__suppressDeprecationWarning\n            ? (lane = \"componentWillReceiveProps\")\n            : \"function\" ===\n                typeof _instance.UNSAFE_componentWillReceiveProps &&\n              (lane = \"UNSAFE_componentWillReceiveProps\");\n          \"function\" === typeof _instance.componentWillUpdate &&\n          !0 !== _instance.componentWillUpdate.__suppressDeprecationWarning\n            ? (foundWillUpdateName = \"componentWillUpdate\")\n            : \"function\" === typeof _instance.UNSAFE_componentWillUpdate &&\n              (foundWillUpdateName = \"UNSAFE_componentWillUpdate\");\n          if (null !== state || null !== lane || null !== foundWillUpdateName) {\n            _instance = getComponentNameFromType(Component) || \"Component\";\n            var newApiName =\n              \"function\" === typeof Component.getDerivedStateFromProps\n                ? \"getDerivedStateFromProps()\"\n                : \"getSnapshotBeforeUpdate()\";\n            didWarnAboutLegacyLifecyclesAndDerivedState.has(_instance) ||\n              (didWarnAboutLegacyLifecyclesAndDerivedState.add(_instance),\n              console.error(\n                \"Unsafe legacy lifecycles will not be called for components using new component APIs.\\n\\n%s uses %s but also contains the following legacy lifecycles:%s%s%s\\n\\nThe above lifecycles should be removed. Learn more about this warning here:\\nhttps://react.dev/link/unsafe-component-lifecycles\",\n                _instance,\n                newApiName,\n                null !== state ? \"\\n  \" + state : \"\",\n                null !== lane ? \"\\n  \" + lane : \"\",\n                null !== foundWillUpdateName ? \"\\n  \" + foundWillUpdateName : \"\"\n              ));\n          }\n        }\n        _instance = workInProgress.stateNode;\n        state = getComponentNameFromType(Component) || \"Component\";\n        _instance.render ||\n          (Component.prototype &&\n          \"function\" === typeof Component.prototype.render\n            ? console.error(\n                \"No `render` method found on the %s instance: did you accidentally return an object from the constructor?\",\n                state\n              )\n            : console.error(\n                \"No `render` method found on the %s instance: you may have forgotten to define `render`.\",\n                state\n              ));\n        !_instance.getInitialState ||\n          _instance.getInitialState.isReactClassApproved ||\n          _instance.state ||\n          console.error(\n            \"getInitialState was defined on %s, a plain JavaScript class. This is only supported for classes created using React.createClass. Did you mean to define a state property instead?\",\n            state\n          );\n        _instance.getDefaultProps &&\n          !_instance.getDefaultProps.isReactClassApproved &&\n          console.error(\n            \"getDefaultProps was defined on %s, a plain JavaScript class. This is only supported for classes created using React.createClass. Use a static property to define defaultProps instead.\",\n            state\n          );\n        _instance.contextType &&\n          console.error(\n            \"contextType was defined as an instance property on %s. Use a static property to define contextType instead.\",\n            state\n          );\n        Component.childContextTypes &&\n          !didWarnAboutChildContextTypes.has(Component) &&\n          (didWarnAboutChildContextTypes.add(Component),\n          console.error(\n            \"%s uses the legacy childContextTypes API which was removed in React 19. Use React.createContext() instead. (https://react.dev/link/legacy-context)\",\n            state\n          ));\n        Component.contextTypes &&\n          !didWarnAboutContextTypes$1.has(Component) &&\n          (didWarnAboutContextTypes$1.add(Component),\n          console.error(\n            \"%s uses the legacy contextTypes API which was removed in React 19. Use React.createContext() with static contextType instead. (https://react.dev/link/legacy-context)\",\n            state\n          ));\n        \"function\" === typeof _instance.componentShouldUpdate &&\n          console.error(\n            \"%s has a method called componentShouldUpdate(). Did you mean shouldComponentUpdate()? The name is phrased as a question because the function is expected to return a value.\",\n            state\n          );\n        Component.prototype &&\n          Component.prototype.isPureReactComponent &&\n          \"undefined\" !== typeof _instance.shouldComponentUpdate &&\n          console.error(\n            \"%s has a method called shouldComponentUpdate(). shouldComponentUpdate should not be used when extending React.PureComponent. Please extend React.Component if shouldComponentUpdate is used.\",\n            getComponentNameFromType(Component) || \"A pure component\"\n          );\n        \"function\" === typeof _instance.componentDidUnmount &&\n          console.error(\n            \"%s has a method called componentDidUnmount(). But there is no such lifecycle method. Did you mean componentWillUnmount()?\",\n            state\n          );\n        \"function\" === typeof _instance.componentDidReceiveProps &&\n          console.error(\n            \"%s has a method called componentDidReceiveProps(). But there is no such lifecycle method. If you meant to update the state in response to changing props, use componentWillReceiveProps(). If you meant to fetch data or run side-effects or mutations after React has updated the UI, use componentDidUpdate().\",\n            state\n          );\n        \"function\" === typeof _instance.componentWillRecieveProps &&\n          console.error(\n            \"%s has a method called componentWillRecieveProps(). Did you mean componentWillReceiveProps()?\",\n            state\n          );\n        \"function\" === typeof _instance.UNSAFE_componentWillRecieveProps &&\n          console.error(\n            \"%s has a method called UNSAFE_componentWillRecieveProps(). Did you mean UNSAFE_componentWillReceiveProps()?\",\n            state\n          );\n        lane = _instance.props !== nextProps;\n        void 0 !== _instance.props &&\n          lane &&\n          console.error(\n            \"When calling super() in `%s`, make sure to pass up the same props that your component's constructor was passed.\",\n            state\n          );\n        _instance.defaultProps &&\n          console.error(\n            \"Setting defaultProps as an instance property on %s is not supported and will be ignored. Instead, define defaultProps as a static property on %s.\",\n            state,\n            state\n          );\n        \"function\" !== typeof _instance.getSnapshotBeforeUpdate ||\n          \"function\" === typeof _instance.componentDidUpdate ||\n          didWarnAboutGetSnapshotBeforeUpdateWithoutDidUpdate.has(Component) ||\n          (didWarnAboutGetSnapshotBeforeUpdateWithoutDidUpdate.add(Component),\n          console.error(\n            \"%s: getSnapshotBeforeUpdate() should be used with componentDidUpdate(). This component defines getSnapshotBeforeUpdate() only.\",\n            getComponentNameFromType(Component)\n          ));\n        \"function\" === typeof _instance.getDerivedStateFromProps &&\n          console.error(\n            \"%s: getDerivedStateFromProps() is defined as an instance method and will be ignored. Instead, declare it as a static method.\",\n            state\n          );\n        \"function\" === typeof _instance.getDerivedStateFromError &&\n          console.error(\n            \"%s: getDerivedStateFromError() is defined as an instance method and will be ignored. Instead, declare it as a static method.\",\n            state\n          );\n        \"function\" === typeof Component.getSnapshotBeforeUpdate &&\n          console.error(\n            \"%s: getSnapshotBeforeUpdate() is defined as a static method and will be ignored. Instead, declare it as an instance method.\",\n            state\n          );\n        (lane = _instance.state) &&\n          (\"object\" !== typeof lane || isArrayImpl(lane)) &&\n          console.error(\"%s.state: must be set to an object or null\", state);\n        \"function\" === typeof _instance.getChildContext &&\n          \"object\" !== typeof Component.childContextTypes &&\n          console.error(\n            \"%s.getChildContext(): childContextTypes must be defined in order to use getChildContext().\",\n            state\n          );\n        _instance = workInProgress.stateNode;\n        _instance.props = nextProps;\n        _instance.state = workInProgress.memoizedState;\n        _instance.refs = {};\n        initializeUpdateQueue(workInProgress);\n        state = Component.contextType;\n        _instance.context =\n          \"object\" === typeof state && null !== state\n            ? readContext(state)\n            : emptyContextObject;\n        _instance.state === nextProps &&\n          ((state = getComponentNameFromType(Component) || \"Component\"),\n          didWarnAboutDirectlyAssigningPropsToState.has(state) ||\n            (didWarnAboutDirectlyAssigningPropsToState.add(state),\n            console.error(\n              \"%s: It is not recommended to assign props directly to state because updates to props won't be reflected in state. In most cases, it is better to use props directly.\",\n              state\n            )));\n        workInProgress.mode & StrictLegacyMode &&\n          ReactStrictModeWarnings.recordLegacyContextWarning(\n            workInProgress,\n            _instance\n          );\n        ReactStrictModeWarnings.recordUnsafeLifecycleWarnings(\n          workInProgress,\n          _instance\n        );\n        _instance.state = workInProgress.memoizedState;\n        state = Component.getDerivedStateFromProps;\n        \"function\" === typeof state &&\n          (applyDerivedStateFromProps(\n            workInProgress,\n            Component,\n            state,\n            nextProps\n          ),\n          (_instance.state = workInProgress.memoizedState));\n        \"function\" === typeof Component.getDerivedStateFromProps ||\n          \"function\" === typeof _instance.getSnapshotBeforeUpdate ||\n          (\"function\" !== typeof _instance.UNSAFE_componentWillMount &&\n            \"function\" !== typeof _instance.componentWillMount) ||\n          ((state = _instance.state),\n          \"function\" === typeof _instance.componentWillMount &&\n            _instance.componentWillMount(),\n          \"function\" === typeof _instance.UNSAFE_componentWillMount &&\n            _instance.UNSAFE_componentWillMount(),\n          state !== _instance.state &&\n            (console.error(\n              \"%s.componentWillMount(): Assigning directly to this.state is deprecated (except inside a component's constructor). Use setState instead.\",\n              getComponentNameFromFiber(workInProgress) || \"Component\"\n            ),\n            classComponentUpdater.enqueueReplaceState(\n              _instance,\n              _instance.state,\n              null\n            )),\n          processUpdateQueue(workInProgress, nextProps, _instance, renderLanes),\n          suspendIfUpdateReadFromEntangledAsyncAction(),\n          (_instance.state = workInProgress.memoizedState));\n        \"function\" === typeof _instance.componentDidMount &&\n          (workInProgress.flags |= 4194308);\n        (workInProgress.mode & StrictEffectsMode) !== NoMode &&\n          (workInProgress.flags |= 134217728);\n        _instance = !0;\n      } else if (null === current) {\n        _instance = workInProgress.stateNode;\n        var unresolvedOldProps = workInProgress.memoizedProps;\n        lane = resolveClassComponentProps(Component, unresolvedOldProps);\n        _instance.props = lane;\n        var oldContext = _instance.context;\n        foundWillUpdateName = Component.contextType;\n        state = emptyContextObject;\n        \"object\" === typeof foundWillUpdateName &&\n          null !== foundWillUpdateName &&\n          (state = readContext(foundWillUpdateName));\n        newApiName = Component.getDerivedStateFromProps;\n        foundWillUpdateName =\n          \"function\" === typeof newApiName ||\n          \"function\" === typeof _instance.getSnapshotBeforeUpdate;\n        unresolvedOldProps = workInProgress.pendingProps !== unresolvedOldProps;\n        foundWillUpdateName ||\n          (\"function\" !== typeof _instance.UNSAFE_componentWillReceiveProps &&\n            \"function\" !== typeof _instance.componentWillReceiveProps) ||\n          ((unresolvedOldProps || oldContext !== state) &&\n            callComponentWillReceiveProps(\n              workInProgress,\n              _instance,\n              nextProps,\n              state\n            ));\n        hasForceUpdate = !1;\n        var oldState = workInProgress.memoizedState;\n        _instance.state = oldState;\n        processUpdateQueue(workInProgress, nextProps, _instance, renderLanes);\n        suspendIfUpdateReadFromEntangledAsyncAction();\n        oldContext = workInProgress.memoizedState;\n        unresolvedOldProps || oldState !== oldContext || hasForceUpdate\n          ? (\"function\" === typeof newApiName &&\n              (applyDerivedStateFromProps(\n                workInProgress,\n                Component,\n                newApiName,\n                nextProps\n              ),\n              (oldContext = workInProgress.memoizedState)),\n            (lane =\n              hasForceUpdate ||\n              checkShouldComponentUpdate(\n                workInProgress,\n                Component,\n                lane,\n                nextProps,\n                oldState,\n                oldContext,\n                state\n              ))\n              ? (foundWillUpdateName ||\n                  (\"function\" !== typeof _instance.UNSAFE_componentWillMount &&\n                    \"function\" !== typeof _instance.componentWillMount) ||\n                  (\"function\" === typeof _instance.componentWillMount &&\n                    _instance.componentWillMount(),\n                  \"function\" === typeof _instance.UNSAFE_componentWillMount &&\n                    _instance.UNSAFE_componentWillMount()),\n                \"function\" === typeof _instance.componentDidMount &&\n                  (workInProgress.flags |= 4194308),\n                (workInProgress.mode & StrictEffectsMode) !== NoMode &&\n                  (workInProgress.flags |= 134217728))\n              : (\"function\" === typeof _instance.componentDidMount &&\n                  (workInProgress.flags |= 4194308),\n                (workInProgress.mode & StrictEffectsMode) !== NoMode &&\n                  (workInProgress.flags |= 134217728),\n                (workInProgress.memoizedProps = nextProps),\n                (workInProgress.memoizedState = oldContext)),\n            (_instance.props = nextProps),\n            (_instance.state = oldContext),\n            (_instance.context = state),\n            (_instance = lane))\n          : (\"function\" === typeof _instance.componentDidMount &&\n              (workInProgress.flags |= 4194308),\n            (workInProgress.mode & StrictEffectsMode) !== NoMode &&\n              (workInProgress.flags |= 134217728),\n            (_instance = !1));\n      } else {\n        _instance = workInProgress.stateNode;\n        cloneUpdateQueue(current, workInProgress);\n        state = workInProgress.memoizedProps;\n        foundWillUpdateName = resolveClassComponentProps(Component, state);\n        _instance.props = foundWillUpdateName;\n        newApiName = workInProgress.pendingProps;\n        oldState = _instance.context;\n        oldContext = Component.contextType;\n        lane = emptyContextObject;\n        \"object\" === typeof oldContext &&\n          null !== oldContext &&\n          (lane = readContext(oldContext));\n        unresolvedOldProps = Component.getDerivedStateFromProps;\n        (oldContext =\n          \"function\" === typeof unresolvedOldProps ||\n          \"function\" === typeof _instance.getSnapshotBeforeUpdate) ||\n          (\"function\" !== typeof _instance.UNSAFE_componentWillReceiveProps &&\n            \"function\" !== typeof _instance.componentWillReceiveProps) ||\n          ((state !== newApiName || oldState !== lane) &&\n            callComponentWillReceiveProps(\n              workInProgress,\n              _instance,\n              nextProps,\n              lane\n            ));\n        hasForceUpdate = !1;\n        oldState = workInProgress.memoizedState;\n        _instance.state = oldState;\n        processUpdateQueue(workInProgress, nextProps, _instance, renderLanes);\n        suspendIfUpdateReadFromEntangledAsyncAction();\n        var newState = workInProgress.memoizedState;\n        state !== newApiName ||\n        oldState !== newState ||\n        hasForceUpdate ||\n        (null !== current &&\n          null !== current.dependencies &&\n          checkIfContextChanged(current.dependencies))\n          ? (\"function\" === typeof unresolvedOldProps &&\n              (applyDerivedStateFromProps(\n                workInProgress,\n                Component,\n                unresolvedOldProps,\n                nextProps\n              ),\n              (newState = workInProgress.memoizedState)),\n            (foundWillUpdateName =\n              hasForceUpdate ||\n              checkShouldComponentUpdate(\n                workInProgress,\n                Component,\n                foundWillUpdateName,\n                nextProps,\n                oldState,\n                newState,\n                lane\n              ) ||\n              (null !== current &&\n                null !== current.dependencies &&\n                checkIfContextChanged(current.dependencies)))\n              ? (oldContext ||\n                  (\"function\" !== typeof _instance.UNSAFE_componentWillUpdate &&\n                    \"function\" !== typeof _instance.componentWillUpdate) ||\n                  (\"function\" === typeof _instance.componentWillUpdate &&\n                    _instance.componentWillUpdate(nextProps, newState, lane),\n                  \"function\" === typeof _instance.UNSAFE_componentWillUpdate &&\n                    _instance.UNSAFE_componentWillUpdate(\n                      nextProps,\n                      newState,\n                      lane\n                    )),\n                \"function\" === typeof _instance.componentDidUpdate &&\n                  (workInProgress.flags |= 4),\n                \"function\" === typeof _instance.getSnapshotBeforeUpdate &&\n                  (workInProgress.flags |= 1024))\n              : (\"function\" !== typeof _instance.componentDidUpdate ||\n                  (state === current.memoizedProps &&\n                    oldState === current.memoizedState) ||\n                  (workInProgress.flags |= 4),\n                \"function\" !== typeof _instance.getSnapshotBeforeUpdate ||\n                  (state === current.memoizedProps &&\n                    oldState === current.memoizedState) ||\n                  (workInProgress.flags |= 1024),\n                (workInProgress.memoizedProps = nextProps),\n                (workInProgress.memoizedState = newState)),\n            (_instance.props = nextProps),\n            (_instance.state = newState),\n            (_instance.context = lane),\n            (_instance = foundWillUpdateName))\n          : (\"function\" !== typeof _instance.componentDidUpdate ||\n              (state === current.memoizedProps &&\n                oldState === current.memoizedState) ||\n              (workInProgress.flags |= 4),\n            \"function\" !== typeof _instance.getSnapshotBeforeUpdate ||\n              (state === current.memoizedProps &&\n                oldState === current.memoizedState) ||\n              (workInProgress.flags |= 1024),\n            (_instance = !1));\n      }\n      lane = _instance;\n      markRef(current, workInProgress);\n      state = 0 !== (workInProgress.flags & 128);\n      if (lane || state) {\n        lane = workInProgress.stateNode;\n        setCurrentFiber(workInProgress);\n        if (state && \"function\" !== typeof Component.getDerivedStateFromError)\n          (Component = null), (profilerStartTime = -1);\n        else {\n          markComponentRenderStarted(workInProgress);\n          Component = callRenderInDEV(lane);\n          if (workInProgress.mode & StrictLegacyMode) {\n            setIsStrictModeForDevtools(!0);\n            try {\n              callRenderInDEV(lane);\n            } finally {\n              setIsStrictModeForDevtools(!1);\n            }\n          }\n          markComponentRenderStopped();\n        }\n        workInProgress.flags |= 1;\n        null !== current && state\n          ? ((workInProgress.child = reconcileChildFibers(\n              workInProgress,\n              current.child,\n              null,\n              renderLanes\n            )),\n            (workInProgress.child = reconcileChildFibers(\n              workInProgress,\n              null,\n              Component,\n              renderLanes\n            )))\n          : reconcileChildren(current, workInProgress, Component, renderLanes);\n        workInProgress.memoizedState = lane.state;\n        current = workInProgress.child;\n      } else\n        current = bailoutOnAlreadyFinishedWork(\n          current,\n          workInProgress,\n          renderLanes\n        );\n      renderLanes = workInProgress.stateNode;\n      _instance &&\n        renderLanes.props !== nextProps &&\n        (didWarnAboutReassigningProps ||\n          console.error(\n            \"It looks like %s is reassigning its own `this.props` while rendering. This is not supported and can lead to confusing bugs.\",\n            getComponentNameFromFiber(workInProgress) || \"a component\"\n          ),\n        (didWarnAboutReassigningProps = !0));\n      return current;\n    }\n    function mountHostRootWithoutHydrating(\n      current,\n      workInProgress,\n      nextChildren,\n      renderLanes\n    ) {\n      resetHydrationState();\n      workInProgress.flags |= 256;\n      reconcileChildren(current, workInProgress, nextChildren, renderLanes);\n      return workInProgress.child;\n    }\n    function validateFunctionComponentInDev(workInProgress, Component) {\n      Component &&\n        Component.childContextTypes &&\n        console.error(\n          \"childContextTypes cannot be defined on a function component.\\n  %s.childContextTypes = ...\",\n          Component.displayName || Component.name || \"Component\"\n        );\n      \"function\" === typeof Component.getDerivedStateFromProps &&\n        ((workInProgress = getComponentNameFromType(Component) || \"Unknown\"),\n        didWarnAboutGetDerivedStateOnFunctionComponent[workInProgress] ||\n          (console.error(\n            \"%s: Function components do not support getDerivedStateFromProps.\",\n            workInProgress\n          ),\n          (didWarnAboutGetDerivedStateOnFunctionComponent[workInProgress] =\n            !0)));\n      \"object\" === typeof Component.contextType &&\n        null !== Component.contextType &&\n        ((Component = getComponentNameFromType(Component) || \"Unknown\"),\n        didWarnAboutContextTypeOnFunctionComponent[Component] ||\n          (console.error(\n            \"%s: Function components do not support contextType.\",\n            Component\n          ),\n          (didWarnAboutContextTypeOnFunctionComponent[Component] = !0)));\n    }\n    function mountSuspenseOffscreenState(renderLanes) {\n      return { baseLanes: renderLanes, cachePool: getSuspendedCache() };\n    }\n    function getRemainingWorkInPrimaryTree(\n      current,\n      primaryTreeDidDefer,\n      renderLanes\n    ) {\n      current = null !== current ? current.childLanes & ~renderLanes : 0;\n      primaryTreeDidDefer && (current |= workInProgressDeferredLane);\n      return current;\n    }\n    function updateSuspenseComponent(current, workInProgress, renderLanes) {\n      var JSCompiler_object_inline_digest_2451;\n      var JSCompiler_object_inline_stack_2452 = workInProgress.pendingProps;\n      shouldSuspendImpl(workInProgress) && (workInProgress.flags |= 128);\n      var JSCompiler_object_inline_componentStack_2453 = !1;\n      var didSuspend = 0 !== (workInProgress.flags & 128);\n      (JSCompiler_object_inline_digest_2451 = didSuspend) ||\n        (JSCompiler_object_inline_digest_2451 =\n          null !== current && null === current.memoizedState\n            ? !1\n            : 0 !== (suspenseStackCursor.current & ForceSuspenseFallback));\n      JSCompiler_object_inline_digest_2451 &&\n        ((JSCompiler_object_inline_componentStack_2453 = !0),\n        (workInProgress.flags &= -129));\n      JSCompiler_object_inline_digest_2451 = 0 !== (workInProgress.flags & 32);\n      workInProgress.flags &= -33;\n      if (null === current) {\n        if (isHydrating) {\n          JSCompiler_object_inline_componentStack_2453\n            ? pushPrimaryTreeSuspenseHandler(workInProgress)\n            : reuseSuspenseHandlerOnStack(workInProgress);\n          if (isHydrating) {\n            var JSCompiler_object_inline_message_2450 = nextHydratableInstance;\n            var JSCompiler_temp;\n            if (!(JSCompiler_temp = !JSCompiler_object_inline_message_2450)) {\n              c: {\n                var instance = JSCompiler_object_inline_message_2450;\n                for (\n                  JSCompiler_temp = rootOrSingletonContext;\n                  8 !== instance.nodeType;\n\n                ) {\n                  if (!JSCompiler_temp) {\n                    JSCompiler_temp = null;\n                    break c;\n                  }\n                  instance = getNextHydratable(instance.nextSibling);\n                  if (null === instance) {\n                    JSCompiler_temp = null;\n                    break c;\n                  }\n                }\n                JSCompiler_temp = instance;\n              }\n              null !== JSCompiler_temp\n                ? (warnIfNotHydrating(),\n                  (workInProgress.memoizedState = {\n                    dehydrated: JSCompiler_temp,\n                    treeContext:\n                      null !== treeContextProvider\n                        ? { id: treeContextId, overflow: treeContextOverflow }\n                        : null,\n                    retryLane: 536870912,\n                    hydrationErrors: null\n                  }),\n                  (instance = createFiber(18, null, null, NoMode)),\n                  (instance.stateNode = JSCompiler_temp),\n                  (instance.return = workInProgress),\n                  (workInProgress.child = instance),\n                  (hydrationParentFiber = workInProgress),\n                  (nextHydratableInstance = null),\n                  (JSCompiler_temp = !0))\n                : (JSCompiler_temp = !1);\n              JSCompiler_temp = !JSCompiler_temp;\n            }\n            JSCompiler_temp &&\n              (warnNonHydratedInstance(\n                workInProgress,\n                JSCompiler_object_inline_message_2450\n              ),\n              throwOnHydrationMismatch(workInProgress));\n          }\n          JSCompiler_object_inline_message_2450 = workInProgress.memoizedState;\n          if (\n            null !== JSCompiler_object_inline_message_2450 &&\n            ((JSCompiler_object_inline_message_2450 =\n              JSCompiler_object_inline_message_2450.dehydrated),\n            null !== JSCompiler_object_inline_message_2450)\n          )\n            return (\n              isSuspenseInstanceFallback(JSCompiler_object_inline_message_2450)\n                ? (workInProgress.lanes = 32)\n                : (workInProgress.lanes = 536870912),\n              null\n            );\n          popSuspenseHandler(workInProgress);\n        }\n        JSCompiler_object_inline_message_2450 =\n          JSCompiler_object_inline_stack_2452.children;\n        JSCompiler_object_inline_stack_2452 =\n          JSCompiler_object_inline_stack_2452.fallback;\n        if (JSCompiler_object_inline_componentStack_2453)\n          return (\n            reuseSuspenseHandlerOnStack(workInProgress),\n            (JSCompiler_object_inline_componentStack_2453 =\n              workInProgress.mode),\n            (JSCompiler_object_inline_message_2450 =\n              mountWorkInProgressOffscreenFiber(\n                {\n                  mode: \"hidden\",\n                  children: JSCompiler_object_inline_message_2450\n                },\n                JSCompiler_object_inline_componentStack_2453\n              )),\n            (JSCompiler_object_inline_stack_2452 = createFiberFromFragment(\n              JSCompiler_object_inline_stack_2452,\n              JSCompiler_object_inline_componentStack_2453,\n              renderLanes,\n              null\n            )),\n            (JSCompiler_object_inline_message_2450.return = workInProgress),\n            (JSCompiler_object_inline_stack_2452.return = workInProgress),\n            (JSCompiler_object_inline_message_2450.sibling =\n              JSCompiler_object_inline_stack_2452),\n            (workInProgress.child = JSCompiler_object_inline_message_2450),\n            (JSCompiler_object_inline_componentStack_2453 =\n              workInProgress.child),\n            (JSCompiler_object_inline_componentStack_2453.memoizedState =\n              mountSuspenseOffscreenState(renderLanes)),\n            (JSCompiler_object_inline_componentStack_2453.childLanes =\n              getRemainingWorkInPrimaryTree(\n                current,\n                JSCompiler_object_inline_digest_2451,\n                renderLanes\n              )),\n            (workInProgress.memoizedState = SUSPENDED_MARKER),\n            JSCompiler_object_inline_stack_2452\n          );\n        pushPrimaryTreeSuspenseHandler(workInProgress);\n        return mountSuspensePrimaryChildren(\n          workInProgress,\n          JSCompiler_object_inline_message_2450\n        );\n      }\n      var prevState = current.memoizedState;\n      if (\n        null !== prevState &&\n        ((JSCompiler_object_inline_message_2450 = prevState.dehydrated),\n        null !== JSCompiler_object_inline_message_2450)\n      ) {\n        if (didSuspend)\n          workInProgress.flags & 256\n            ? (pushPrimaryTreeSuspenseHandler(workInProgress),\n              (workInProgress.flags &= -257),\n              (workInProgress = retrySuspenseComponentWithoutHydrating(\n                current,\n                workInProgress,\n                renderLanes\n              )))\n            : null !== workInProgress.memoizedState\n              ? (reuseSuspenseHandlerOnStack(workInProgress),\n                (workInProgress.child = current.child),\n                (workInProgress.flags |= 128),\n                (workInProgress = null))\n              : (reuseSuspenseHandlerOnStack(workInProgress),\n                (JSCompiler_object_inline_componentStack_2453 =\n                  JSCompiler_object_inline_stack_2452.fallback),\n                (JSCompiler_object_inline_message_2450 = workInProgress.mode),\n                (JSCompiler_object_inline_stack_2452 =\n                  mountWorkInProgressOffscreenFiber(\n                    {\n                      mode: \"visible\",\n                      children: JSCompiler_object_inline_stack_2452.children\n                    },\n                    JSCompiler_object_inline_message_2450\n                  )),\n                (JSCompiler_object_inline_componentStack_2453 =\n                  createFiberFromFragment(\n                    JSCompiler_object_inline_componentStack_2453,\n                    JSCompiler_object_inline_message_2450,\n                    renderLanes,\n                    null\n                  )),\n                (JSCompiler_object_inline_componentStack_2453.flags |= 2),\n                (JSCompiler_object_inline_stack_2452.return = workInProgress),\n                (JSCompiler_object_inline_componentStack_2453.return =\n                  workInProgress),\n                (JSCompiler_object_inline_stack_2452.sibling =\n                  JSCompiler_object_inline_componentStack_2453),\n                (workInProgress.child = JSCompiler_object_inline_stack_2452),\n                reconcileChildFibers(\n                  workInProgress,\n                  current.child,\n                  null,\n                  renderLanes\n                ),\n                (JSCompiler_object_inline_stack_2452 = workInProgress.child),\n                (JSCompiler_object_inline_stack_2452.memoizedState =\n                  mountSuspenseOffscreenState(renderLanes)),\n                (JSCompiler_object_inline_stack_2452.childLanes =\n                  getRemainingWorkInPrimaryTree(\n                    current,\n                    JSCompiler_object_inline_digest_2451,\n                    renderLanes\n                  )),\n                (workInProgress.memoizedState = SUSPENDED_MARKER),\n                (workInProgress =\n                  JSCompiler_object_inline_componentStack_2453));\n        else if (\n          (pushPrimaryTreeSuspenseHandler(workInProgress),\n          isHydrating &&\n            console.error(\n              \"We should not be hydrating here. This is a bug in React. Please file a bug.\"\n            ),\n          isSuspenseInstanceFallback(JSCompiler_object_inline_message_2450))\n        ) {\n          JSCompiler_object_inline_digest_2451 =\n            JSCompiler_object_inline_message_2450.nextSibling &&\n            JSCompiler_object_inline_message_2450.nextSibling.dataset;\n          if (JSCompiler_object_inline_digest_2451) {\n            JSCompiler_temp = JSCompiler_object_inline_digest_2451.dgst;\n            var message = JSCompiler_object_inline_digest_2451.msg;\n            instance = JSCompiler_object_inline_digest_2451.stck;\n            var componentStack = JSCompiler_object_inline_digest_2451.cstck;\n          }\n          JSCompiler_object_inline_message_2450 = message;\n          JSCompiler_object_inline_digest_2451 = JSCompiler_temp;\n          JSCompiler_object_inline_stack_2452 = instance;\n          JSCompiler_temp = JSCompiler_object_inline_componentStack_2453 =\n            componentStack;\n          JSCompiler_object_inline_componentStack_2453 =\n            JSCompiler_object_inline_message_2450\n              ? Error(JSCompiler_object_inline_message_2450)\n              : Error(\n                  \"The server could not finish this Suspense boundary, likely due to an error during server rendering. Switched to client rendering.\"\n                );\n          JSCompiler_object_inline_componentStack_2453.stack =\n            JSCompiler_object_inline_stack_2452 || \"\";\n          JSCompiler_object_inline_componentStack_2453.digest =\n            JSCompiler_object_inline_digest_2451;\n          JSCompiler_object_inline_digest_2451 =\n            void 0 === JSCompiler_temp ? null : JSCompiler_temp;\n          JSCompiler_object_inline_stack_2452 = {\n            value: JSCompiler_object_inline_componentStack_2453,\n            source: null,\n            stack: JSCompiler_object_inline_digest_2451\n          };\n          \"string\" === typeof JSCompiler_object_inline_digest_2451 &&\n            CapturedStacks.set(\n              JSCompiler_object_inline_componentStack_2453,\n              JSCompiler_object_inline_stack_2452\n            );\n          queueHydrationError(JSCompiler_object_inline_stack_2452);\n          workInProgress = retrySuspenseComponentWithoutHydrating(\n            current,\n            workInProgress,\n            renderLanes\n          );\n        } else if (\n          (didReceiveUpdate ||\n            propagateParentContextChanges(\n              current,\n              workInProgress,\n              renderLanes,\n              !1\n            ),\n          (JSCompiler_object_inline_digest_2451 =\n            0 !== (renderLanes & current.childLanes)),\n          didReceiveUpdate || JSCompiler_object_inline_digest_2451)\n        ) {\n          JSCompiler_object_inline_digest_2451 = workInProgressRoot;\n          if (\n            null !== JSCompiler_object_inline_digest_2451 &&\n            ((JSCompiler_object_inline_stack_2452 = renderLanes & -renderLanes),\n            (JSCompiler_object_inline_stack_2452 =\n              0 !== (JSCompiler_object_inline_stack_2452 & 42)\n                ? 1\n                : getBumpedLaneForHydrationByLane(\n                    JSCompiler_object_inline_stack_2452\n                  )),\n            (JSCompiler_object_inline_stack_2452 =\n              0 !==\n              (JSCompiler_object_inline_stack_2452 &\n                (JSCompiler_object_inline_digest_2451.suspendedLanes |\n                  renderLanes))\n                ? 0\n                : JSCompiler_object_inline_stack_2452),\n            0 !== JSCompiler_object_inline_stack_2452 &&\n              JSCompiler_object_inline_stack_2452 !== prevState.retryLane)\n          )\n            throw (\n              ((prevState.retryLane = JSCompiler_object_inline_stack_2452),\n              enqueueConcurrentRenderForLane(\n                current,\n                JSCompiler_object_inline_stack_2452\n              ),\n              scheduleUpdateOnFiber(\n                JSCompiler_object_inline_digest_2451,\n                current,\n                JSCompiler_object_inline_stack_2452\n              ),\n              SelectiveHydrationException)\n            );\n          JSCompiler_object_inline_message_2450.data ===\n            SUSPENSE_PENDING_START_DATA || renderDidSuspendDelayIfPossible();\n          workInProgress = retrySuspenseComponentWithoutHydrating(\n            current,\n            workInProgress,\n            renderLanes\n          );\n        } else\n          JSCompiler_object_inline_message_2450.data ===\n          SUSPENSE_PENDING_START_DATA\n            ? ((workInProgress.flags |= 192),\n              (workInProgress.child = current.child),\n              (workInProgress = null))\n            : ((current = prevState.treeContext),\n              (nextHydratableInstance = getNextHydratable(\n                JSCompiler_object_inline_message_2450.nextSibling\n              )),\n              (hydrationParentFiber = workInProgress),\n              (isHydrating = !0),\n              (hydrationErrors = null),\n              (didSuspendOrErrorDEV = !1),\n              (hydrationDiffRootDEV = null),\n              (rootOrSingletonContext = !1),\n              null !== current &&\n                (warnIfNotHydrating(),\n                (idStack[idStackIndex++] = treeContextId),\n                (idStack[idStackIndex++] = treeContextOverflow),\n                (idStack[idStackIndex++] = treeContextProvider),\n                (treeContextId = current.id),\n                (treeContextOverflow = current.overflow),\n                (treeContextProvider = workInProgress)),\n              (workInProgress = mountSuspensePrimaryChildren(\n                workInProgress,\n                JSCompiler_object_inline_stack_2452.children\n              )),\n              (workInProgress.flags |= 4096));\n        return workInProgress;\n      }\n      if (JSCompiler_object_inline_componentStack_2453)\n        return (\n          reuseSuspenseHandlerOnStack(workInProgress),\n          (JSCompiler_object_inline_componentStack_2453 =\n            JSCompiler_object_inline_stack_2452.fallback),\n          (JSCompiler_object_inline_message_2450 = workInProgress.mode),\n          (JSCompiler_temp = current.child),\n          (instance = JSCompiler_temp.sibling),\n          (JSCompiler_object_inline_stack_2452 = createWorkInProgress(\n            JSCompiler_temp,\n            {\n              mode: \"hidden\",\n              children: JSCompiler_object_inline_stack_2452.children\n            }\n          )),\n          (JSCompiler_object_inline_stack_2452.subtreeFlags =\n            JSCompiler_temp.subtreeFlags & 65011712),\n          null !== instance\n            ? (JSCompiler_object_inline_componentStack_2453 =\n                createWorkInProgress(\n                  instance,\n                  JSCompiler_object_inline_componentStack_2453\n                ))\n            : ((JSCompiler_object_inline_componentStack_2453 =\n                createFiberFromFragment(\n                  JSCompiler_object_inline_componentStack_2453,\n                  JSCompiler_object_inline_message_2450,\n                  renderLanes,\n                  null\n                )),\n              (JSCompiler_object_inline_componentStack_2453.flags |= 2)),\n          (JSCompiler_object_inline_componentStack_2453.return =\n            workInProgress),\n          (JSCompiler_object_inline_stack_2452.return = workInProgress),\n          (JSCompiler_object_inline_stack_2452.sibling =\n            JSCompiler_object_inline_componentStack_2453),\n          (workInProgress.child = JSCompiler_object_inline_stack_2452),\n          (JSCompiler_object_inline_stack_2452 =\n            JSCompiler_object_inline_componentStack_2453),\n          (JSCompiler_object_inline_componentStack_2453 = workInProgress.child),\n          (JSCompiler_object_inline_message_2450 = current.child.memoizedState),\n          null === JSCompiler_object_inline_message_2450\n            ? (JSCompiler_object_inline_message_2450 =\n                mountSuspenseOffscreenState(renderLanes))\n            : ((JSCompiler_temp =\n                JSCompiler_object_inline_message_2450.cachePool),\n              null !== JSCompiler_temp\n                ? ((instance = CacheContext._currentValue),\n                  (JSCompiler_temp =\n                    JSCompiler_temp.parent !== instance\n                      ? { parent: instance, pool: instance }\n                      : JSCompiler_temp))\n                : (JSCompiler_temp = getSuspendedCache()),\n              (JSCompiler_object_inline_message_2450 = {\n                baseLanes:\n                  JSCompiler_object_inline_message_2450.baseLanes | renderLanes,\n                cachePool: JSCompiler_temp\n              })),\n          (JSCompiler_object_inline_componentStack_2453.memoizedState =\n            JSCompiler_object_inline_message_2450),\n          (JSCompiler_object_inline_componentStack_2453.childLanes =\n            getRemainingWorkInPrimaryTree(\n              current,\n              JSCompiler_object_inline_digest_2451,\n              renderLanes\n            )),\n          (workInProgress.memoizedState = SUSPENDED_MARKER),\n          JSCompiler_object_inline_stack_2452\n        );\n      pushPrimaryTreeSuspenseHandler(workInProgress);\n      renderLanes = current.child;\n      current = renderLanes.sibling;\n      renderLanes = createWorkInProgress(renderLanes, {\n        mode: \"visible\",\n        children: JSCompiler_object_inline_stack_2452.children\n      });\n      renderLanes.return = workInProgress;\n      renderLanes.sibling = null;\n      null !== current &&\n        ((JSCompiler_object_inline_digest_2451 = workInProgress.deletions),\n        null === JSCompiler_object_inline_digest_2451\n          ? ((workInProgress.deletions = [current]),\n            (workInProgress.flags |= 16))\n          : JSCompiler_object_inline_digest_2451.push(current));\n      workInProgress.child = renderLanes;\n      workInProgress.memoizedState = null;\n      return renderLanes;\n    }\n    function mountSuspensePrimaryChildren(workInProgress, primaryChildren) {\n      primaryChildren = mountWorkInProgressOffscreenFiber(\n        { mode: \"visible\", children: primaryChildren },\n        workInProgress.mode\n      );\n      primaryChildren.return = workInProgress;\n      return (workInProgress.child = primaryChildren);\n    }\n    function mountWorkInProgressOffscreenFiber(offscreenProps, mode) {\n      offscreenProps = createFiber(22, offscreenProps, null, mode);\n      offscreenProps.lanes = 0;\n      offscreenProps.stateNode = {\n        _visibility: OffscreenVisible,\n        _pendingMarkers: null,\n        _retryCache: null,\n        _transitions: null\n      };\n      return offscreenProps;\n    }\n    function retrySuspenseComponentWithoutHydrating(\n      current,\n      workInProgress,\n      renderLanes\n    ) {\n      reconcileChildFibers(workInProgress, current.child, null, renderLanes);\n      current = mountSuspensePrimaryChildren(\n        workInProgress,\n        workInProgress.pendingProps.children\n      );\n      current.flags |= 2;\n      workInProgress.memoizedState = null;\n      return current;\n    }\n    function scheduleSuspenseWorkOnFiber(fiber, renderLanes, propagationRoot) {\n      fiber.lanes |= renderLanes;\n      var alternate = fiber.alternate;\n      null !== alternate && (alternate.lanes |= renderLanes);\n      scheduleContextWorkOnParentPath(\n        fiber.return,\n        renderLanes,\n        propagationRoot\n      );\n    }\n    function validateSuspenseListNestedChild(childSlot, index) {\n      var isAnArray = isArrayImpl(childSlot);\n      childSlot = !isAnArray && \"function\" === typeof getIteratorFn(childSlot);\n      return isAnArray || childSlot\n        ? ((isAnArray = isAnArray ? \"array\" : \"iterable\"),\n          console.error(\n            \"A nested %s was passed to row #%s in <SuspenseList />. Wrap it in an additional SuspenseList to configure its revealOrder: <SuspenseList revealOrder=...> ... <SuspenseList revealOrder=...>{%s}</SuspenseList> ... </SuspenseList>\",\n            isAnArray,\n            index,\n            isAnArray\n          ),\n          !1)\n        : !0;\n    }\n    function initSuspenseListRenderState(\n      workInProgress,\n      isBackwards,\n      tail,\n      lastContentRow,\n      tailMode\n    ) {\n      var renderState = workInProgress.memoizedState;\n      null === renderState\n        ? (workInProgress.memoizedState = {\n            isBackwards: isBackwards,\n            rendering: null,\n            renderingStartTime: 0,\n            last: lastContentRow,\n            tail: tail,\n            tailMode: tailMode\n          })\n        : ((renderState.isBackwards = isBackwards),\n          (renderState.rendering = null),\n          (renderState.renderingStartTime = 0),\n          (renderState.last = lastContentRow),\n          (renderState.tail = tail),\n          (renderState.tailMode = tailMode));\n    }\n    function updateSuspenseListComponent(current, workInProgress, renderLanes) {\n      var nextProps = workInProgress.pendingProps,\n        revealOrder = nextProps.revealOrder,\n        tailMode = nextProps.tail;\n      nextProps = nextProps.children;\n      if (\n        void 0 !== revealOrder &&\n        \"forwards\" !== revealOrder &&\n        \"backwards\" !== revealOrder &&\n        \"together\" !== revealOrder &&\n        !didWarnAboutRevealOrder[revealOrder]\n      )\n        if (\n          ((didWarnAboutRevealOrder[revealOrder] = !0),\n          \"string\" === typeof revealOrder)\n        )\n          switch (revealOrder.toLowerCase()) {\n            case \"together\":\n            case \"forwards\":\n            case \"backwards\":\n              console.error(\n                '\"%s\" is not a valid value for revealOrder on <SuspenseList />. Use lowercase \"%s\" instead.',\n                revealOrder,\n                revealOrder.toLowerCase()\n              );\n              break;\n            case \"forward\":\n            case \"backward\":\n              console.error(\n                '\"%s\" is not a valid value for revealOrder on <SuspenseList />. React uses the -s suffix in the spelling. Use \"%ss\" instead.',\n                revealOrder,\n                revealOrder.toLowerCase()\n              );\n              break;\n            default:\n              console.error(\n                '\"%s\" is not a supported revealOrder on <SuspenseList />. Did you mean \"together\", \"forwards\" or \"backwards\"?',\n                revealOrder\n              );\n          }\n        else\n          console.error(\n            '%s is not a supported value for revealOrder on <SuspenseList />. Did you mean \"together\", \"forwards\" or \"backwards\"?',\n            revealOrder\n          );\n      void 0 === tailMode ||\n        didWarnAboutTailOptions[tailMode] ||\n        (\"collapsed\" !== tailMode && \"hidden\" !== tailMode\n          ? ((didWarnAboutTailOptions[tailMode] = !0),\n            console.error(\n              '\"%s\" is not a supported value for tail on <SuspenseList />. Did you mean \"collapsed\" or \"hidden\"?',\n              tailMode\n            ))\n          : \"forwards\" !== revealOrder &&\n            \"backwards\" !== revealOrder &&\n            ((didWarnAboutTailOptions[tailMode] = !0),\n            console.error(\n              '<SuspenseList tail=\"%s\" /> is only valid if revealOrder is \"forwards\" or \"backwards\". Did you mean to specify revealOrder=\"forwards\"?',\n              tailMode\n            )));\n      a: if (\n        (\"forwards\" === revealOrder || \"backwards\" === revealOrder) &&\n        void 0 !== nextProps &&\n        null !== nextProps &&\n        !1 !== nextProps\n      )\n        if (isArrayImpl(nextProps))\n          for (var i = 0; i < nextProps.length; i++) {\n            if (!validateSuspenseListNestedChild(nextProps[i], i)) break a;\n          }\n        else if (((i = getIteratorFn(nextProps)), \"function\" === typeof i)) {\n          if ((i = i.call(nextProps)))\n            for (var step = i.next(), _i = 0; !step.done; step = i.next()) {\n              if (!validateSuspenseListNestedChild(step.value, _i)) break a;\n              _i++;\n            }\n        } else\n          console.error(\n            'A single row was passed to a <SuspenseList revealOrder=\"%s\" />. This is not useful since it needs multiple rows. Did you mean to pass multiple children or an array?',\n            revealOrder\n          );\n      reconcileChildren(current, workInProgress, nextProps, renderLanes);\n      nextProps = suspenseStackCursor.current;\n      if (0 !== (nextProps & ForceSuspenseFallback))\n        (nextProps =\n          (nextProps & SubtreeSuspenseContextMask) | ForceSuspenseFallback),\n          (workInProgress.flags |= 128);\n      else {\n        if (null !== current && 0 !== (current.flags & 128))\n          a: for (current = workInProgress.child; null !== current; ) {\n            if (13 === current.tag)\n              null !== current.memoizedState &&\n                scheduleSuspenseWorkOnFiber(\n                  current,\n                  renderLanes,\n                  workInProgress\n                );\n            else if (19 === current.tag)\n              scheduleSuspenseWorkOnFiber(current, renderLanes, workInProgress);\n            else if (null !== current.child) {\n              current.child.return = current;\n              current = current.child;\n              continue;\n            }\n            if (current === workInProgress) break a;\n            for (; null === current.sibling; ) {\n              if (null === current.return || current.return === workInProgress)\n                break a;\n              current = current.return;\n            }\n            current.sibling.return = current.return;\n            current = current.sibling;\n          }\n        nextProps &= SubtreeSuspenseContextMask;\n      }\n      push(suspenseStackCursor, nextProps, workInProgress);\n      switch (revealOrder) {\n        case \"forwards\":\n          renderLanes = workInProgress.child;\n          for (revealOrder = null; null !== renderLanes; )\n            (current = renderLanes.alternate),\n              null !== current &&\n                null === findFirstSuspended(current) &&\n                (revealOrder = renderLanes),\n              (renderLanes = renderLanes.sibling);\n          renderLanes = revealOrder;\n          null === renderLanes\n            ? ((revealOrder = workInProgress.child),\n              (workInProgress.child = null))\n            : ((revealOrder = renderLanes.sibling),\n              (renderLanes.sibling = null));\n          initSuspenseListRenderState(\n            workInProgress,\n            !1,\n            revealOrder,\n            renderLanes,\n            tailMode\n          );\n          break;\n        case \"backwards\":\n          renderLanes = null;\n          revealOrder = workInProgress.child;\n          for (workInProgress.child = null; null !== revealOrder; ) {\n            current = revealOrder.alternate;\n            if (null !== current && null === findFirstSuspended(current)) {\n              workInProgress.child = revealOrder;\n              break;\n            }\n            current = revealOrder.sibling;\n            revealOrder.sibling = renderLanes;\n            renderLanes = revealOrder;\n            revealOrder = current;\n          }\n          initSuspenseListRenderState(\n            workInProgress,\n            !0,\n            renderLanes,\n            null,\n            tailMode\n          );\n          break;\n        case \"together\":\n          initSuspenseListRenderState(workInProgress, !1, null, null, void 0);\n          break;\n        default:\n          workInProgress.memoizedState = null;\n      }\n      return workInProgress.child;\n    }\n    function bailoutOnAlreadyFinishedWork(\n      current,\n      workInProgress,\n      renderLanes\n    ) {\n      null !== current && (workInProgress.dependencies = current.dependencies);\n      profilerStartTime = -1;\n      workInProgressRootSkippedLanes |= workInProgress.lanes;\n      if (0 === (renderLanes & workInProgress.childLanes))\n        if (null !== current) {\n          if (\n            (propagateParentContextChanges(\n              current,\n              workInProgress,\n              renderLanes,\n              !1\n            ),\n            0 === (renderLanes & workInProgress.childLanes))\n          )\n            return null;\n        } else return null;\n      if (null !== current && workInProgress.child !== current.child)\n        throw Error(\"Resuming work not yet implemented.\");\n      if (null !== workInProgress.child) {\n        current = workInProgress.child;\n        renderLanes = createWorkInProgress(current, current.pendingProps);\n        workInProgress.child = renderLanes;\n        for (renderLanes.return = workInProgress; null !== current.sibling; )\n          (current = current.sibling),\n            (renderLanes = renderLanes.sibling =\n              createWorkInProgress(current, current.pendingProps)),\n            (renderLanes.return = workInProgress);\n        renderLanes.sibling = null;\n      }\n      return workInProgress.child;\n    }\n    function checkScheduledUpdateOrContext(current, renderLanes) {\n      if (0 !== (current.lanes & renderLanes)) return !0;\n      current = current.dependencies;\n      return null !== current && checkIfContextChanged(current) ? !0 : !1;\n    }\n    function attemptEarlyBailoutIfNoScheduledUpdate(\n      current,\n      workInProgress,\n      renderLanes\n    ) {\n      switch (workInProgress.tag) {\n        case 3:\n          pushHostContainer(\n            workInProgress,\n            workInProgress.stateNode.containerInfo\n          );\n          pushProvider(\n            workInProgress,\n            CacheContext,\n            current.memoizedState.cache\n          );\n          resetHydrationState();\n          break;\n        case 27:\n        case 5:\n          pushHostContext(workInProgress);\n          break;\n        case 4:\n          pushHostContainer(\n            workInProgress,\n            workInProgress.stateNode.containerInfo\n          );\n          break;\n        case 10:\n          pushProvider(\n            workInProgress,\n            workInProgress.type,\n            workInProgress.memoizedProps.value\n          );\n          break;\n        case 12:\n          0 !== (renderLanes & workInProgress.childLanes) &&\n            (workInProgress.flags |= 4);\n          workInProgress.flags |= 2048;\n          var stateNode = workInProgress.stateNode;\n          stateNode.effectDuration = -0;\n          stateNode.passiveEffectDuration = -0;\n          break;\n        case 13:\n          stateNode = workInProgress.memoizedState;\n          if (null !== stateNode) {\n            if (null !== stateNode.dehydrated)\n              return (\n                pushPrimaryTreeSuspenseHandler(workInProgress),\n                (workInProgress.flags |= 128),\n                null\n              );\n            if (0 !== (renderLanes & workInProgress.child.childLanes))\n              return updateSuspenseComponent(\n                current,\n                workInProgress,\n                renderLanes\n              );\n            pushPrimaryTreeSuspenseHandler(workInProgress);\n            current = bailoutOnAlreadyFinishedWork(\n              current,\n              workInProgress,\n              renderLanes\n            );\n            return null !== current ? current.sibling : null;\n          }\n          pushPrimaryTreeSuspenseHandler(workInProgress);\n          break;\n        case 19:\n          var didSuspendBefore = 0 !== (current.flags & 128);\n          stateNode = 0 !== (renderLanes & workInProgress.childLanes);\n          stateNode ||\n            (propagateParentContextChanges(\n              current,\n              workInProgress,\n              renderLanes,\n              !1\n            ),\n            (stateNode = 0 !== (renderLanes & workInProgress.childLanes)));\n          if (didSuspendBefore) {\n            if (stateNode)\n              return updateSuspenseListComponent(\n                current,\n                workInProgress,\n                renderLanes\n              );\n            workInProgress.flags |= 128;\n          }\n          didSuspendBefore = workInProgress.memoizedState;\n          null !== didSuspendBefore &&\n            ((didSuspendBefore.rendering = null),\n            (didSuspendBefore.tail = null),\n            (didSuspendBefore.lastEffect = null));\n          push(\n            suspenseStackCursor,\n            suspenseStackCursor.current,\n            workInProgress\n          );\n          if (stateNode) break;\n          else return null;\n        case 22:\n        case 23:\n          return (\n            (workInProgress.lanes = 0),\n            updateOffscreenComponent(current, workInProgress, renderLanes)\n          );\n        case 24:\n          pushProvider(\n            workInProgress,\n            CacheContext,\n            current.memoizedState.cache\n          );\n      }\n      return bailoutOnAlreadyFinishedWork(current, workInProgress, renderLanes);\n    }\n    function beginWork(current, workInProgress, renderLanes) {\n      if (workInProgress._debugNeedsRemount && null !== current) {\n        renderLanes = createFiberFromTypeAndProps(\n          workInProgress.type,\n          workInProgress.key,\n          workInProgress.pendingProps,\n          workInProgress._debugOwner || null,\n          workInProgress.mode,\n          workInProgress.lanes\n        );\n        renderLanes._debugStack = workInProgress._debugStack;\n        renderLanes._debugTask = workInProgress._debugTask;\n        var returnFiber = workInProgress.return;\n        if (null === returnFiber) throw Error(\"Cannot swap the root fiber.\");\n        current.alternate = null;\n        workInProgress.alternate = null;\n        renderLanes.index = workInProgress.index;\n        renderLanes.sibling = workInProgress.sibling;\n        renderLanes.return = workInProgress.return;\n        renderLanes.ref = workInProgress.ref;\n        renderLanes._debugInfo = workInProgress._debugInfo;\n        if (workInProgress === returnFiber.child)\n          returnFiber.child = renderLanes;\n        else {\n          var prevSibling = returnFiber.child;\n          if (null === prevSibling)\n            throw Error(\"Expected parent to have a child.\");\n          for (; prevSibling.sibling !== workInProgress; )\n            if (((prevSibling = prevSibling.sibling), null === prevSibling))\n              throw Error(\"Expected to find the previous sibling.\");\n          prevSibling.sibling = renderLanes;\n        }\n        workInProgress = returnFiber.deletions;\n        null === workInProgress\n          ? ((returnFiber.deletions = [current]), (returnFiber.flags |= 16))\n          : workInProgress.push(current);\n        renderLanes.flags |= 2;\n        return renderLanes;\n      }\n      if (null !== current)\n        if (\n          current.memoizedProps !== workInProgress.pendingProps ||\n          workInProgress.type !== current.type\n        )\n          didReceiveUpdate = !0;\n        else {\n          if (\n            !checkScheduledUpdateOrContext(current, renderLanes) &&\n            0 === (workInProgress.flags & 128)\n          )\n            return (\n              (didReceiveUpdate = !1),\n              attemptEarlyBailoutIfNoScheduledUpdate(\n                current,\n                workInProgress,\n                renderLanes\n              )\n            );\n          didReceiveUpdate = 0 !== (current.flags & 131072) ? !0 : !1;\n        }\n      else {\n        didReceiveUpdate = !1;\n        if ((returnFiber = isHydrating))\n          warnIfNotHydrating(),\n            (returnFiber = 0 !== (workInProgress.flags & 1048576));\n        returnFiber &&\n          ((returnFiber = workInProgress.index),\n          warnIfNotHydrating(),\n          pushTreeId(workInProgress, treeForkCount, returnFiber));\n      }\n      workInProgress.lanes = 0;\n      switch (workInProgress.tag) {\n        case 16:\n          a: if (\n            ((returnFiber = workInProgress.pendingProps),\n            (current = callLazyInitInDEV(workInProgress.elementType)),\n            (workInProgress.type = current),\n            \"function\" === typeof current)\n          )\n            shouldConstruct(current)\n              ? ((returnFiber = resolveClassComponentProps(\n                  current,\n                  returnFiber\n                )),\n                (workInProgress.tag = 1),\n                (workInProgress.type = current =\n                  resolveFunctionForHotReloading(current)),\n                (workInProgress = updateClassComponent(\n                  null,\n                  workInProgress,\n                  current,\n                  returnFiber,\n                  renderLanes\n                )))\n              : ((workInProgress.tag = 0),\n                validateFunctionComponentInDev(workInProgress, current),\n                (workInProgress.type = current =\n                  resolveFunctionForHotReloading(current)),\n                (workInProgress = updateFunctionComponent(\n                  null,\n                  workInProgress,\n                  current,\n                  returnFiber,\n                  renderLanes\n                )));\n          else {\n            if (void 0 !== current && null !== current)\n              if (\n                ((prevSibling = current.$$typeof),\n                prevSibling === REACT_FORWARD_REF_TYPE)\n              ) {\n                workInProgress.tag = 11;\n                workInProgress.type = current =\n                  resolveForwardRefForHotReloading(current);\n                workInProgress = updateForwardRef(\n                  null,\n                  workInProgress,\n                  current,\n                  returnFiber,\n                  renderLanes\n                );\n                break a;\n              } else if (prevSibling === REACT_MEMO_TYPE) {\n                workInProgress.tag = 14;\n                workInProgress = updateMemoComponent(\n                  null,\n                  workInProgress,\n                  current,\n                  returnFiber,\n                  renderLanes\n                );\n                break a;\n              }\n            workInProgress = \"\";\n            null !== current &&\n              \"object\" === typeof current &&\n              current.$$typeof === REACT_LAZY_TYPE &&\n              (workInProgress =\n                \" Did you wrap a component in React.lazy() more than once?\");\n            current = getComponentNameFromType(current) || current;\n            throw Error(\n              \"Element type is invalid. Received a promise that resolves to: \" +\n                current +\n                \". Lazy element type must resolve to a class or function.\" +\n                workInProgress\n            );\n          }\n          return workInProgress;\n        case 0:\n          return updateFunctionComponent(\n            current,\n            workInProgress,\n            workInProgress.type,\n            workInProgress.pendingProps,\n            renderLanes\n          );\n        case 1:\n          return (\n            (returnFiber = workInProgress.type),\n            (prevSibling = resolveClassComponentProps(\n              returnFiber,\n              workInProgress.pendingProps\n            )),\n            updateClassComponent(\n              current,\n              workInProgress,\n              returnFiber,\n              prevSibling,\n              renderLanes\n            )\n          );\n        case 3:\n          a: {\n            pushHostContainer(\n              workInProgress,\n              workInProgress.stateNode.containerInfo\n            );\n            if (null === current)\n              throw Error(\n                \"Should have a current fiber. This is a bug in React.\"\n              );\n            returnFiber = workInProgress.pendingProps;\n            var prevState = workInProgress.memoizedState;\n            prevSibling = prevState.element;\n            cloneUpdateQueue(current, workInProgress);\n            processUpdateQueue(workInProgress, returnFiber, null, renderLanes);\n            var nextState = workInProgress.memoizedState;\n            returnFiber = nextState.cache;\n            pushProvider(workInProgress, CacheContext, returnFiber);\n            returnFiber !== prevState.cache &&\n              propagateContextChanges(\n                workInProgress,\n                [CacheContext],\n                renderLanes,\n                !0\n              );\n            suspendIfUpdateReadFromEntangledAsyncAction();\n            returnFiber = nextState.element;\n            if (prevState.isDehydrated)\n              if (\n                ((prevState = {\n                  element: returnFiber,\n                  isDehydrated: !1,\n                  cache: nextState.cache\n                }),\n                (workInProgress.updateQueue.baseState = prevState),\n                (workInProgress.memoizedState = prevState),\n                workInProgress.flags & 256)\n              ) {\n                workInProgress = mountHostRootWithoutHydrating(\n                  current,\n                  workInProgress,\n                  returnFiber,\n                  renderLanes\n                );\n                break a;\n              } else if (returnFiber !== prevSibling) {\n                prevSibling = createCapturedValueAtFiber(\n                  Error(\n                    \"This root received an early update, before anything was able hydrate. Switched the entire root to client rendering.\"\n                  ),\n                  workInProgress\n                );\n                queueHydrationError(prevSibling);\n                workInProgress = mountHostRootWithoutHydrating(\n                  current,\n                  workInProgress,\n                  returnFiber,\n                  renderLanes\n                );\n                break a;\n              } else {\n                current = workInProgress.stateNode.containerInfo;\n                switch (current.nodeType) {\n                  case 9:\n                    current = current.body;\n                    break;\n                  default:\n                    current =\n                      \"HTML\" === current.nodeName\n                        ? current.ownerDocument.body\n                        : current;\n                }\n                nextHydratableInstance = getNextHydratable(current.firstChild);\n                hydrationParentFiber = workInProgress;\n                isHydrating = !0;\n                hydrationErrors = null;\n                didSuspendOrErrorDEV = !1;\n                hydrationDiffRootDEV = null;\n                rootOrSingletonContext = !0;\n                current = mountChildFibers(\n                  workInProgress,\n                  null,\n                  returnFiber,\n                  renderLanes\n                );\n                for (workInProgress.child = current; current; )\n                  (current.flags = (current.flags & -3) | 4096),\n                    (current = current.sibling);\n              }\n            else {\n              resetHydrationState();\n              if (returnFiber === prevSibling) {\n                workInProgress = bailoutOnAlreadyFinishedWork(\n                  current,\n                  workInProgress,\n                  renderLanes\n                );\n                break a;\n              }\n              reconcileChildren(\n                current,\n                workInProgress,\n                returnFiber,\n                renderLanes\n              );\n            }\n            workInProgress = workInProgress.child;\n          }\n          return workInProgress;\n        case 26:\n          return (\n            markRef(current, workInProgress),\n            null === current\n              ? (current = getResource(\n                  workInProgress.type,\n                  null,\n                  workInProgress.pendingProps,\n                  null\n                ))\n                ? (workInProgress.memoizedState = current)\n                : isHydrating ||\n                  ((current = workInProgress.type),\n                  (renderLanes = workInProgress.pendingProps),\n                  (returnFiber = requiredContext(\n                    rootInstanceStackCursor.current\n                  )),\n                  (returnFiber =\n                    getOwnerDocumentFromRootContainer(\n                      returnFiber\n                    ).createElement(current)),\n                  (returnFiber[internalInstanceKey] = workInProgress),\n                  (returnFiber[internalPropsKey] = renderLanes),\n                  setInitialProperties(returnFiber, current, renderLanes),\n                  markNodeAsHoistable(returnFiber),\n                  (workInProgress.stateNode = returnFiber))\n              : (workInProgress.memoizedState = getResource(\n                  workInProgress.type,\n                  current.memoizedProps,\n                  workInProgress.pendingProps,\n                  current.memoizedState\n                )),\n            null\n          );\n        case 27:\n          return (\n            pushHostContext(workInProgress),\n            null === current &&\n              isHydrating &&\n              ((returnFiber = requiredContext(rootInstanceStackCursor.current)),\n              (prevSibling = getHostContext()),\n              (returnFiber = workInProgress.stateNode =\n                resolveSingletonInstance(\n                  workInProgress.type,\n                  workInProgress.pendingProps,\n                  returnFiber,\n                  prevSibling,\n                  !1\n                )),\n              didSuspendOrErrorDEV ||\n                ((prevSibling = diffHydratedProperties(\n                  returnFiber,\n                  workInProgress.type,\n                  workInProgress.pendingProps,\n                  prevSibling\n                )),\n                null !== prevSibling &&\n                  (buildHydrationDiffNode(workInProgress, 0).serverProps =\n                    prevSibling)),\n              (hydrationParentFiber = workInProgress),\n              (rootOrSingletonContext = !0),\n              (prevSibling = nextHydratableInstance),\n              isSingletonScope(workInProgress.type)\n                ? ((previousHydratableOnEnteringScopedSingleton = prevSibling),\n                  (nextHydratableInstance = getNextHydratable(\n                    returnFiber.firstChild\n                  )))\n                : (nextHydratableInstance = prevSibling)),\n            reconcileChildren(\n              current,\n              workInProgress,\n              workInProgress.pendingProps.children,\n              renderLanes\n            ),\n            markRef(current, workInProgress),\n            null === current && (workInProgress.flags |= 4194304),\n            workInProgress.child\n          );\n        case 5:\n          return (\n            null === current &&\n              isHydrating &&\n              ((prevState = getHostContext()),\n              (returnFiber = validateDOMNesting(\n                workInProgress.type,\n                prevState.ancestorInfo\n              )),\n              (prevSibling = nextHydratableInstance),\n              (nextState = !prevSibling) ||\n                ((nextState = canHydrateInstance(\n                  prevSibling,\n                  workInProgress.type,\n                  workInProgress.pendingProps,\n                  rootOrSingletonContext\n                )),\n                null !== nextState\n                  ? ((workInProgress.stateNode = nextState),\n                    didSuspendOrErrorDEV ||\n                      ((prevState = diffHydratedProperties(\n                        nextState,\n                        workInProgress.type,\n                        workInProgress.pendingProps,\n                        prevState\n                      )),\n                      null !== prevState &&\n                        (buildHydrationDiffNode(workInProgress, 0).serverProps =\n                          prevState)),\n                    (hydrationParentFiber = workInProgress),\n                    (nextHydratableInstance = getNextHydratable(\n                      nextState.firstChild\n                    )),\n                    (rootOrSingletonContext = !1),\n                    (prevState = !0))\n                  : (prevState = !1),\n                (nextState = !prevState)),\n              nextState &&\n                (returnFiber &&\n                  warnNonHydratedInstance(workInProgress, prevSibling),\n                throwOnHydrationMismatch(workInProgress))),\n            pushHostContext(workInProgress),\n            (prevSibling = workInProgress.type),\n            (prevState = workInProgress.pendingProps),\n            (nextState = null !== current ? current.memoizedProps : null),\n            (returnFiber = prevState.children),\n            shouldSetTextContent(prevSibling, prevState)\n              ? (returnFiber = null)\n              : null !== nextState &&\n                shouldSetTextContent(prevSibling, nextState) &&\n                (workInProgress.flags |= 32),\n            null !== workInProgress.memoizedState &&\n              ((prevSibling = renderWithHooks(\n                current,\n                workInProgress,\n                TransitionAwareHostComponent,\n                null,\n                null,\n                renderLanes\n              )),\n              (HostTransitionContext._currentValue = prevSibling)),\n            markRef(current, workInProgress),\n            reconcileChildren(\n              current,\n              workInProgress,\n              returnFiber,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 6:\n          return (\n            null === current &&\n              isHydrating &&\n              ((current = workInProgress.pendingProps),\n              (renderLanes = getHostContext()),\n              (returnFiber = renderLanes.ancestorInfo.current),\n              (current =\n                null != returnFiber\n                  ? validateTextNesting(\n                      current,\n                      returnFiber.tag,\n                      renderLanes.ancestorInfo.implicitRootScope\n                    )\n                  : !0),\n              (renderLanes = nextHydratableInstance),\n              (returnFiber = !renderLanes) ||\n                ((returnFiber = canHydrateTextInstance(\n                  renderLanes,\n                  workInProgress.pendingProps,\n                  rootOrSingletonContext\n                )),\n                null !== returnFiber\n                  ? ((workInProgress.stateNode = returnFiber),\n                    (hydrationParentFiber = workInProgress),\n                    (nextHydratableInstance = null),\n                    (returnFiber = !0))\n                  : (returnFiber = !1),\n                (returnFiber = !returnFiber)),\n              returnFiber &&\n                (current &&\n                  warnNonHydratedInstance(workInProgress, renderLanes),\n                throwOnHydrationMismatch(workInProgress))),\n            null\n          );\n        case 13:\n          return updateSuspenseComponent(current, workInProgress, renderLanes);\n        case 4:\n          return (\n            pushHostContainer(\n              workInProgress,\n              workInProgress.stateNode.containerInfo\n            ),\n            (returnFiber = workInProgress.pendingProps),\n            null === current\n              ? (workInProgress.child = reconcileChildFibers(\n                  workInProgress,\n                  null,\n                  returnFiber,\n                  renderLanes\n                ))\n              : reconcileChildren(\n                  current,\n                  workInProgress,\n                  returnFiber,\n                  renderLanes\n                ),\n            workInProgress.child\n          );\n        case 11:\n          return updateForwardRef(\n            current,\n            workInProgress,\n            workInProgress.type,\n            workInProgress.pendingProps,\n            renderLanes\n          );\n        case 7:\n          return (\n            reconcileChildren(\n              current,\n              workInProgress,\n              workInProgress.pendingProps,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 8:\n          return (\n            reconcileChildren(\n              current,\n              workInProgress,\n              workInProgress.pendingProps.children,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 12:\n          return (\n            (workInProgress.flags |= 4),\n            (workInProgress.flags |= 2048),\n            (returnFiber = workInProgress.stateNode),\n            (returnFiber.effectDuration = -0),\n            (returnFiber.passiveEffectDuration = -0),\n            reconcileChildren(\n              current,\n              workInProgress,\n              workInProgress.pendingProps.children,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 10:\n          return (\n            (returnFiber = workInProgress.type),\n            (prevSibling = workInProgress.pendingProps),\n            (prevState = prevSibling.value),\n            \"value\" in prevSibling ||\n              hasWarnedAboutUsingNoValuePropOnContextProvider ||\n              ((hasWarnedAboutUsingNoValuePropOnContextProvider = !0),\n              console.error(\n                \"The `value` prop is required for the `<Context.Provider>`. Did you misspell it or forget to pass it?\"\n              )),\n            pushProvider(workInProgress, returnFiber, prevState),\n            reconcileChildren(\n              current,\n              workInProgress,\n              prevSibling.children,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 9:\n          return (\n            (prevSibling = workInProgress.type._context),\n            (returnFiber = workInProgress.pendingProps.children),\n            \"function\" !== typeof returnFiber &&\n              console.error(\n                \"A context consumer was rendered with multiple children, or a child that isn't a function. A context consumer expects a single child that is a function. If you did pass a function, make sure there is no trailing or leading whitespace around it.\"\n              ),\n            prepareToReadContext(workInProgress),\n            (prevSibling = readContext(prevSibling)),\n            markComponentRenderStarted(workInProgress),\n            (returnFiber = callComponentInDEV(\n              returnFiber,\n              prevSibling,\n              void 0\n            )),\n            markComponentRenderStopped(),\n            (workInProgress.flags |= 1),\n            reconcileChildren(\n              current,\n              workInProgress,\n              returnFiber,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 14:\n          return updateMemoComponent(\n            current,\n            workInProgress,\n            workInProgress.type,\n            workInProgress.pendingProps,\n            renderLanes\n          );\n        case 15:\n          return updateSimpleMemoComponent(\n            current,\n            workInProgress,\n            workInProgress.type,\n            workInProgress.pendingProps,\n            renderLanes\n          );\n        case 19:\n          return updateSuspenseListComponent(\n            current,\n            workInProgress,\n            renderLanes\n          );\n        case 31:\n          return (\n            (returnFiber = workInProgress.pendingProps),\n            (renderLanes = workInProgress.mode),\n            (returnFiber = {\n              mode: returnFiber.mode,\n              children: returnFiber.children\n            }),\n            null === current\n              ? ((current = mountWorkInProgressOffscreenFiber(\n                  returnFiber,\n                  renderLanes\n                )),\n                (current.ref = workInProgress.ref),\n                (workInProgress.child = current),\n                (current.return = workInProgress),\n                (workInProgress = current))\n              : ((current = createWorkInProgress(current.child, returnFiber)),\n                (current.ref = workInProgress.ref),\n                (workInProgress.child = current),\n                (current.return = workInProgress),\n                (workInProgress = current)),\n            workInProgress\n          );\n        case 22:\n          return updateOffscreenComponent(current, workInProgress, renderLanes);\n        case 24:\n          return (\n            prepareToReadContext(workInProgress),\n            (returnFiber = readContext(CacheContext)),\n            null === current\n              ? ((prevSibling = peekCacheFromPool()),\n                null === prevSibling &&\n                  ((prevSibling = workInProgressRoot),\n                  (prevState = createCache()),\n                  (prevSibling.pooledCache = prevState),\n                  retainCache(prevState),\n                  null !== prevState &&\n                    (prevSibling.pooledCacheLanes |= renderLanes),\n                  (prevSibling = prevState)),\n                (workInProgress.memoizedState = {\n                  parent: returnFiber,\n                  cache: prevSibling\n                }),\n                initializeUpdateQueue(workInProgress),\n                pushProvider(workInProgress, CacheContext, prevSibling))\n              : (0 !== (current.lanes & renderLanes) &&\n                  (cloneUpdateQueue(current, workInProgress),\n                  processUpdateQueue(workInProgress, null, null, renderLanes),\n                  suspendIfUpdateReadFromEntangledAsyncAction()),\n                (prevSibling = current.memoizedState),\n                (prevState = workInProgress.memoizedState),\n                prevSibling.parent !== returnFiber\n                  ? ((prevSibling = {\n                      parent: returnFiber,\n                      cache: returnFiber\n                    }),\n                    (workInProgress.memoizedState = prevSibling),\n                    0 === workInProgress.lanes &&\n                      (workInProgress.memoizedState =\n                        workInProgress.updateQueue.baseState =\n                          prevSibling),\n                    pushProvider(workInProgress, CacheContext, returnFiber))\n                  : ((returnFiber = prevState.cache),\n                    pushProvider(workInProgress, CacheContext, returnFiber),\n                    returnFiber !== prevSibling.cache &&\n                      propagateContextChanges(\n                        workInProgress,\n                        [CacheContext],\n                        renderLanes,\n                        !0\n                      ))),\n            reconcileChildren(\n              current,\n              workInProgress,\n              workInProgress.pendingProps.children,\n              renderLanes\n            ),\n            workInProgress.child\n          );\n        case 29:\n          throw workInProgress.pendingProps;\n      }\n      throw Error(\n        \"Unknown unit of work tag (\" +\n          workInProgress.tag +\n          \"). This error is likely caused by a bug in React. Please file an issue.\"\n      );\n    }\n    function markUpdate(workInProgress) {\n      workInProgress.flags |= 4;\n    }\n    function preloadResourceAndSuspendIfNeeded(workInProgress, resource) {\n      if (\n        \"stylesheet\" !== resource.type ||\n        (resource.state.loading & Inserted) !== NotLoaded\n      )\n        workInProgress.flags &= -16777217;\n      else if (\n        ((workInProgress.flags |= 16777216), !preloadResource(resource))\n      ) {\n        resource = suspenseHandlerStackCursor.current;\n        if (\n          null !== resource &&\n          ((workInProgressRootRenderLanes & 4194048) ===\n          workInProgressRootRenderLanes\n            ? null !== shellBoundary\n            : ((workInProgressRootRenderLanes & 62914560) !==\n                workInProgressRootRenderLanes &&\n                0 === (workInProgressRootRenderLanes & 536870912)) ||\n              resource !== shellBoundary)\n        )\n          throw (\n            ((suspendedThenable = noopSuspenseyCommitThenable),\n            SuspenseyCommitException)\n          );\n        workInProgress.flags |= 8192;\n      }\n    }\n    function scheduleRetryEffect(workInProgress, retryQueue) {\n      null !== retryQueue && (workInProgress.flags |= 4);\n      workInProgress.flags & 16384 &&\n        ((retryQueue =\n          22 !== workInProgress.tag ? claimNextRetryLane() : 536870912),\n        (workInProgress.lanes |= retryQueue),\n        (workInProgressSuspendedRetryLanes |= retryQueue));\n    }\n    function cutOffTailIfNeeded(renderState, hasRenderedATailFallback) {\n      if (!isHydrating)\n        switch (renderState.tailMode) {\n          case \"hidden\":\n            hasRenderedATailFallback = renderState.tail;\n            for (var lastTailNode = null; null !== hasRenderedATailFallback; )\n              null !== hasRenderedATailFallback.alternate &&\n                (lastTailNode = hasRenderedATailFallback),\n                (hasRenderedATailFallback = hasRenderedATailFallback.sibling);\n            null === lastTailNode\n              ? (renderState.tail = null)\n              : (lastTailNode.sibling = null);\n            break;\n          case \"collapsed\":\n            lastTailNode = renderState.tail;\n            for (var _lastTailNode = null; null !== lastTailNode; )\n              null !== lastTailNode.alternate && (_lastTailNode = lastTailNode),\n                (lastTailNode = lastTailNode.sibling);\n            null === _lastTailNode\n              ? hasRenderedATailFallback || null === renderState.tail\n                ? (renderState.tail = null)\n                : (renderState.tail.sibling = null)\n              : (_lastTailNode.sibling = null);\n        }\n    }\n    function bubbleProperties(completedWork) {\n      var didBailout =\n          null !== completedWork.alternate &&\n          completedWork.alternate.child === completedWork.child,\n        newChildLanes = 0,\n        subtreeFlags = 0;\n      if (didBailout)\n        if ((completedWork.mode & ProfileMode) !== NoMode) {\n          for (\n            var _treeBaseDuration = completedWork.selfBaseDuration,\n              _child2 = completedWork.child;\n            null !== _child2;\n\n          )\n            (newChildLanes |= _child2.lanes | _child2.childLanes),\n              (subtreeFlags |= _child2.subtreeFlags & 65011712),\n              (subtreeFlags |= _child2.flags & 65011712),\n              (_treeBaseDuration += _child2.treeBaseDuration),\n              (_child2 = _child2.sibling);\n          completedWork.treeBaseDuration = _treeBaseDuration;\n        } else\n          for (\n            _treeBaseDuration = completedWork.child;\n            null !== _treeBaseDuration;\n\n          )\n            (newChildLanes |=\n              _treeBaseDuration.lanes | _treeBaseDuration.childLanes),\n              (subtreeFlags |= _treeBaseDuration.subtreeFlags & 65011712),\n              (subtreeFlags |= _treeBaseDuration.flags & 65011712),\n              (_treeBaseDuration.return = completedWork),\n              (_treeBaseDuration = _treeBaseDuration.sibling);\n      else if ((completedWork.mode & ProfileMode) !== NoMode) {\n        _treeBaseDuration = completedWork.actualDuration;\n        _child2 = completedWork.selfBaseDuration;\n        for (var child = completedWork.child; null !== child; )\n          (newChildLanes |= child.lanes | child.childLanes),\n            (subtreeFlags |= child.subtreeFlags),\n            (subtreeFlags |= child.flags),\n            (_treeBaseDuration += child.actualDuration),\n            (_child2 += child.treeBaseDuration),\n            (child = child.sibling);\n        completedWork.actualDuration = _treeBaseDuration;\n        completedWork.treeBaseDuration = _child2;\n      } else\n        for (\n          _treeBaseDuration = completedWork.child;\n          null !== _treeBaseDuration;\n\n        )\n          (newChildLanes |=\n            _treeBaseDuration.lanes | _treeBaseDuration.childLanes),\n            (subtreeFlags |= _treeBaseDuration.subtreeFlags),\n            (subtreeFlags |= _treeBaseDuration.flags),\n            (_treeBaseDuration.return = completedWork),\n            (_treeBaseDuration = _treeBaseDuration.sibling);\n      completedWork.subtreeFlags |= subtreeFlags;\n      completedWork.childLanes = newChildLanes;\n      return didBailout;\n    }\n    function completeWork(current, workInProgress, renderLanes) {\n      var newProps = workInProgress.pendingProps;\n      popTreeContext(workInProgress);\n      switch (workInProgress.tag) {\n        case 31:\n        case 16:\n        case 15:\n        case 0:\n        case 11:\n        case 7:\n        case 8:\n        case 12:\n        case 9:\n        case 14:\n          return bubbleProperties(workInProgress), null;\n        case 1:\n          return bubbleProperties(workInProgress), null;\n        case 3:\n          renderLanes = workInProgress.stateNode;\n          newProps = null;\n          null !== current && (newProps = current.memoizedState.cache);\n          workInProgress.memoizedState.cache !== newProps &&\n            (workInProgress.flags |= 2048);\n          popProvider(CacheContext, workInProgress);\n          popHostContainer(workInProgress);\n          renderLanes.pendingContext &&\n            ((renderLanes.context = renderLanes.pendingContext),\n            (renderLanes.pendingContext = null));\n          if (null === current || null === current.child)\n            popHydrationState(workInProgress)\n              ? (emitPendingHydrationWarnings(), markUpdate(workInProgress))\n              : null === current ||\n                (current.memoizedState.isDehydrated &&\n                  0 === (workInProgress.flags & 256)) ||\n                ((workInProgress.flags |= 1024),\n                upgradeHydrationErrorsToRecoverable());\n          bubbleProperties(workInProgress);\n          return null;\n        case 26:\n          return (\n            (renderLanes = workInProgress.memoizedState),\n            null === current\n              ? (markUpdate(workInProgress),\n                null !== renderLanes\n                  ? (bubbleProperties(workInProgress),\n                    preloadResourceAndSuspendIfNeeded(\n                      workInProgress,\n                      renderLanes\n                    ))\n                  : (bubbleProperties(workInProgress),\n                    (workInProgress.flags &= -16777217)))\n              : renderLanes\n                ? renderLanes !== current.memoizedState\n                  ? (markUpdate(workInProgress),\n                    bubbleProperties(workInProgress),\n                    preloadResourceAndSuspendIfNeeded(\n                      workInProgress,\n                      renderLanes\n                    ))\n                  : (bubbleProperties(workInProgress),\n                    (workInProgress.flags &= -16777217))\n                : (current.memoizedProps !== newProps &&\n                    markUpdate(workInProgress),\n                  bubbleProperties(workInProgress),\n                  (workInProgress.flags &= -16777217)),\n            null\n          );\n        case 27:\n          popHostContext(workInProgress);\n          renderLanes = requiredContext(rootInstanceStackCursor.current);\n          var _type = workInProgress.type;\n          if (null !== current && null != workInProgress.stateNode)\n            current.memoizedProps !== newProps && markUpdate(workInProgress);\n          else {\n            if (!newProps) {\n              if (null === workInProgress.stateNode)\n                throw Error(\n                  \"We must have new props for new mounts. This error is likely caused by a bug in React. Please file an issue.\"\n                );\n              bubbleProperties(workInProgress);\n              return null;\n            }\n            current = getHostContext();\n            popHydrationState(workInProgress)\n              ? prepareToHydrateHostInstance(workInProgress, current)\n              : ((current = resolveSingletonInstance(\n                  _type,\n                  newProps,\n                  renderLanes,\n                  current,\n                  !0\n                )),\n                (workInProgress.stateNode = current),\n                markUpdate(workInProgress));\n          }\n          bubbleProperties(workInProgress);\n          return null;\n        case 5:\n          popHostContext(workInProgress);\n          renderLanes = workInProgress.type;\n          if (null !== current && null != workInProgress.stateNode)\n            current.memoizedProps !== newProps && markUpdate(workInProgress);\n          else {\n            if (!newProps) {\n              if (null === workInProgress.stateNode)\n                throw Error(\n                  \"We must have new props for new mounts. This error is likely caused by a bug in React. Please file an issue.\"\n                );\n              bubbleProperties(workInProgress);\n              return null;\n            }\n            _type = getHostContext();\n            if (popHydrationState(workInProgress))\n              prepareToHydrateHostInstance(workInProgress, _type);\n            else {\n              current = requiredContext(rootInstanceStackCursor.current);\n              validateDOMNesting(renderLanes, _type.ancestorInfo);\n              _type = _type.context;\n              current = getOwnerDocumentFromRootContainer(current);\n              switch (_type) {\n                case HostContextNamespaceSvg:\n                  current = current.createElementNS(SVG_NAMESPACE, renderLanes);\n                  break;\n                case HostContextNamespaceMath:\n                  current = current.createElementNS(\n                    MATH_NAMESPACE,\n                    renderLanes\n                  );\n                  break;\n                default:\n                  switch (renderLanes) {\n                    case \"svg\":\n                      current = current.createElementNS(\n                        SVG_NAMESPACE,\n                        renderLanes\n                      );\n                      break;\n                    case \"math\":\n                      current = current.createElementNS(\n                        MATH_NAMESPACE,\n                        renderLanes\n                      );\n                      break;\n                    case \"script\":\n                      current = current.createElement(\"div\");\n                      current.innerHTML = \"<script>\\x3c/script>\";\n                      current = current.removeChild(current.firstChild);\n                      break;\n                    case \"select\":\n                      current =\n                        \"string\" === typeof newProps.is\n                          ? current.createElement(\"select\", { is: newProps.is })\n                          : current.createElement(\"select\");\n                      newProps.multiple\n                        ? (current.multiple = !0)\n                        : newProps.size && (current.size = newProps.size);\n                      break;\n                    default:\n                      (current =\n                        \"string\" === typeof newProps.is\n                          ? current.createElement(renderLanes, {\n                              is: newProps.is\n                            })\n                          : current.createElement(renderLanes)),\n                        -1 === renderLanes.indexOf(\"-\") &&\n                          (renderLanes !== renderLanes.toLowerCase() &&\n                            console.error(\n                              \"<%s /> is using incorrect casing. Use PascalCase for React components, or lowercase for HTML elements.\",\n                              renderLanes\n                            ),\n                          \"[object HTMLUnknownElement]\" !==\n                            Object.prototype.toString.call(current) ||\n                            hasOwnProperty.call(\n                              warnedUnknownTags,\n                              renderLanes\n                            ) ||\n                            ((warnedUnknownTags[renderLanes] = !0),\n                            console.error(\n                              \"The tag <%s> is unrecognized in this browser. If you meant to render a React component, start its name with an uppercase letter.\",\n                              renderLanes\n                            )));\n                  }\n              }\n              current[internalInstanceKey] = workInProgress;\n              current[internalPropsKey] = newProps;\n              a: for (_type = workInProgress.child; null !== _type; ) {\n                if (5 === _type.tag || 6 === _type.tag)\n                  current.appendChild(_type.stateNode);\n                else if (\n                  4 !== _type.tag &&\n                  27 !== _type.tag &&\n                  null !== _type.child\n                ) {\n                  _type.child.return = _type;\n                  _type = _type.child;\n                  continue;\n                }\n                if (_type === workInProgress) break a;\n                for (; null === _type.sibling; ) {\n                  if (null === _type.return || _type.return === workInProgress)\n                    break a;\n                  _type = _type.return;\n                }\n                _type.sibling.return = _type.return;\n                _type = _type.sibling;\n              }\n              workInProgress.stateNode = current;\n              a: switch (\n                (setInitialProperties(current, renderLanes, newProps),\n                renderLanes)\n              ) {\n                case \"button\":\n                case \"input\":\n                case \"select\":\n                case \"textarea\":\n                  current = !!newProps.autoFocus;\n                  break a;\n                case \"img\":\n                  current = !0;\n                  break a;\n                default:\n                  current = !1;\n              }\n              current && markUpdate(workInProgress);\n            }\n          }\n          bubbleProperties(workInProgress);\n          workInProgress.flags &= -16777217;\n          return null;\n        case 6:\n          if (current && null != workInProgress.stateNode)\n            current.memoizedProps !== newProps && markUpdate(workInProgress);\n          else {\n            if (\n              \"string\" !== typeof newProps &&\n              null === workInProgress.stateNode\n            )\n              throw Error(\n                \"We must have new props for new mounts. This error is likely caused by a bug in React. Please file an issue.\"\n              );\n            current = requiredContext(rootInstanceStackCursor.current);\n            renderLanes = getHostContext();\n            if (popHydrationState(workInProgress)) {\n              current = workInProgress.stateNode;\n              renderLanes = workInProgress.memoizedProps;\n              _type = !didSuspendOrErrorDEV;\n              newProps = null;\n              var returnFiber = hydrationParentFiber;\n              if (null !== returnFiber)\n                switch (returnFiber.tag) {\n                  case 3:\n                    _type &&\n                      ((_type = diffHydratedTextForDevWarnings(\n                        current,\n                        renderLanes,\n                        newProps\n                      )),\n                      null !== _type &&\n                        (buildHydrationDiffNode(workInProgress, 0).serverProps =\n                          _type));\n                    break;\n                  case 27:\n                  case 5:\n                    (newProps = returnFiber.memoizedProps),\n                      _type &&\n                        ((_type = diffHydratedTextForDevWarnings(\n                          current,\n                          renderLanes,\n                          newProps\n                        )),\n                        null !== _type &&\n                          (buildHydrationDiffNode(\n                            workInProgress,\n                            0\n                          ).serverProps = _type));\n                }\n              current[internalInstanceKey] = workInProgress;\n              current =\n                current.nodeValue === renderLanes ||\n                (null !== newProps &&\n                  !0 === newProps.suppressHydrationWarning) ||\n                checkForUnmatchedText(current.nodeValue, renderLanes)\n                  ? !0\n                  : !1;\n              current || throwOnHydrationMismatch(workInProgress);\n            } else\n              (_type = renderLanes.ancestorInfo.current),\n                null != _type &&\n                  validateTextNesting(\n                    newProps,\n                    _type.tag,\n                    renderLanes.ancestorInfo.implicitRootScope\n                  ),\n                (current =\n                  getOwnerDocumentFromRootContainer(current).createTextNode(\n                    newProps\n                  )),\n                (current[internalInstanceKey] = workInProgress),\n                (workInProgress.stateNode = current);\n          }\n          bubbleProperties(workInProgress);\n          return null;\n        case 13:\n          newProps = workInProgress.memoizedState;\n          if (\n            null === current ||\n            (null !== current.memoizedState &&\n              null !== current.memoizedState.dehydrated)\n          ) {\n            _type = popHydrationState(workInProgress);\n            if (null !== newProps && null !== newProps.dehydrated) {\n              if (null === current) {\n                if (!_type)\n                  throw Error(\n                    \"A dehydrated suspense component was completed without a hydrated node. This is probably a bug in React.\"\n                  );\n                _type = workInProgress.memoizedState;\n                _type = null !== _type ? _type.dehydrated : null;\n                if (!_type)\n                  throw Error(\n                    \"Expected to have a hydrated suspense instance. This error is likely caused by a bug in React. Please file an issue.\"\n                  );\n                _type[internalInstanceKey] = workInProgress;\n                bubbleProperties(workInProgress);\n                (workInProgress.mode & ProfileMode) !== NoMode &&\n                  null !== newProps &&\n                  ((_type = workInProgress.child),\n                  null !== _type &&\n                    (workInProgress.treeBaseDuration -=\n                      _type.treeBaseDuration));\n              } else\n                emitPendingHydrationWarnings(),\n                  resetHydrationState(),\n                  0 === (workInProgress.flags & 128) &&\n                    (workInProgress.memoizedState = null),\n                  (workInProgress.flags |= 4),\n                  bubbleProperties(workInProgress),\n                  (workInProgress.mode & ProfileMode) !== NoMode &&\n                    null !== newProps &&\n                    ((_type = workInProgress.child),\n                    null !== _type &&\n                      (workInProgress.treeBaseDuration -=\n                        _type.treeBaseDuration));\n              _type = !1;\n            } else\n              (_type = upgradeHydrationErrorsToRecoverable()),\n                null !== current &&\n                  null !== current.memoizedState &&\n                  (current.memoizedState.hydrationErrors = _type),\n                (_type = !0);\n            if (!_type) {\n              if (workInProgress.flags & 256)\n                return popSuspenseHandler(workInProgress), workInProgress;\n              popSuspenseHandler(workInProgress);\n              return null;\n            }\n          }\n          popSuspenseHandler(workInProgress);\n          if (0 !== (workInProgress.flags & 128))\n            return (\n              (workInProgress.lanes = renderLanes),\n              (workInProgress.mode & ProfileMode) !== NoMode &&\n                transferActualDuration(workInProgress),\n              workInProgress\n            );\n          renderLanes = null !== newProps;\n          current = null !== current && null !== current.memoizedState;\n          renderLanes &&\n            ((newProps = workInProgress.child),\n            (_type = null),\n            null !== newProps.alternate &&\n              null !== newProps.alternate.memoizedState &&\n              null !== newProps.alternate.memoizedState.cachePool &&\n              (_type = newProps.alternate.memoizedState.cachePool.pool),\n            (returnFiber = null),\n            null !== newProps.memoizedState &&\n              null !== newProps.memoizedState.cachePool &&\n              (returnFiber = newProps.memoizedState.cachePool.pool),\n            returnFiber !== _type && (newProps.flags |= 2048));\n          renderLanes !== current &&\n            renderLanes &&\n            (workInProgress.child.flags |= 8192);\n          scheduleRetryEffect(workInProgress, workInProgress.updateQueue);\n          bubbleProperties(workInProgress);\n          (workInProgress.mode & ProfileMode) !== NoMode &&\n            renderLanes &&\n            ((current = workInProgress.child),\n            null !== current &&\n              (workInProgress.treeBaseDuration -= current.treeBaseDuration));\n          return null;\n        case 4:\n          return (\n            popHostContainer(workInProgress),\n            null === current &&\n              listenToAllSupportedEvents(\n                workInProgress.stateNode.containerInfo\n              ),\n            bubbleProperties(workInProgress),\n            null\n          );\n        case 10:\n          return (\n            popProvider(workInProgress.type, workInProgress),\n            bubbleProperties(workInProgress),\n            null\n          );\n        case 19:\n          pop(suspenseStackCursor, workInProgress);\n          _type = workInProgress.memoizedState;\n          if (null === _type) return bubbleProperties(workInProgress), null;\n          newProps = 0 !== (workInProgress.flags & 128);\n          returnFiber = _type.rendering;\n          if (null === returnFiber)\n            if (newProps) cutOffTailIfNeeded(_type, !1);\n            else {\n              if (\n                workInProgressRootExitStatus !== RootInProgress ||\n                (null !== current && 0 !== (current.flags & 128))\n              )\n                for (current = workInProgress.child; null !== current; ) {\n                  returnFiber = findFirstSuspended(current);\n                  if (null !== returnFiber) {\n                    workInProgress.flags |= 128;\n                    cutOffTailIfNeeded(_type, !1);\n                    current = returnFiber.updateQueue;\n                    workInProgress.updateQueue = current;\n                    scheduleRetryEffect(workInProgress, current);\n                    workInProgress.subtreeFlags = 0;\n                    current = renderLanes;\n                    for (\n                      renderLanes = workInProgress.child;\n                      null !== renderLanes;\n\n                    )\n                      resetWorkInProgress(renderLanes, current),\n                        (renderLanes = renderLanes.sibling);\n                    push(\n                      suspenseStackCursor,\n                      (suspenseStackCursor.current &\n                        SubtreeSuspenseContextMask) |\n                        ForceSuspenseFallback,\n                      workInProgress\n                    );\n                    return workInProgress.child;\n                  }\n                  current = current.sibling;\n                }\n              null !== _type.tail &&\n                now$1() > workInProgressRootRenderTargetTime &&\n                ((workInProgress.flags |= 128),\n                (newProps = !0),\n                cutOffTailIfNeeded(_type, !1),\n                (workInProgress.lanes = 4194304));\n            }\n          else {\n            if (!newProps)\n              if (\n                ((current = findFirstSuspended(returnFiber)), null !== current)\n              ) {\n                if (\n                  ((workInProgress.flags |= 128),\n                  (newProps = !0),\n                  (current = current.updateQueue),\n                  (workInProgress.updateQueue = current),\n                  scheduleRetryEffect(workInProgress, current),\n                  cutOffTailIfNeeded(_type, !0),\n                  null === _type.tail &&\n                    \"hidden\" === _type.tailMode &&\n                    !returnFiber.alternate &&\n                    !isHydrating)\n                )\n                  return bubbleProperties(workInProgress), null;\n              } else\n                2 * now$1() - _type.renderingStartTime >\n                  workInProgressRootRenderTargetTime &&\n                  536870912 !== renderLanes &&\n                  ((workInProgress.flags |= 128),\n                  (newProps = !0),\n                  cutOffTailIfNeeded(_type, !1),\n                  (workInProgress.lanes = 4194304));\n            _type.isBackwards\n              ? ((returnFiber.sibling = workInProgress.child),\n                (workInProgress.child = returnFiber))\n              : ((current = _type.last),\n                null !== current\n                  ? (current.sibling = returnFiber)\n                  : (workInProgress.child = returnFiber),\n                (_type.last = returnFiber));\n          }\n          if (null !== _type.tail)\n            return (\n              (current = _type.tail),\n              (_type.rendering = current),\n              (_type.tail = current.sibling),\n              (_type.renderingStartTime = now$1()),\n              (current.sibling = null),\n              (renderLanes = suspenseStackCursor.current),\n              (renderLanes = newProps\n                ? (renderLanes & SubtreeSuspenseContextMask) |\n                  ForceSuspenseFallback\n                : renderLanes & SubtreeSuspenseContextMask),\n              push(suspenseStackCursor, renderLanes, workInProgress),\n              current\n            );\n          bubbleProperties(workInProgress);\n          return null;\n        case 22:\n        case 23:\n          return (\n            popSuspenseHandler(workInProgress),\n            popHiddenContext(workInProgress),\n            (newProps = null !== workInProgress.memoizedState),\n            null !== current\n              ? (null !== current.memoizedState) !== newProps &&\n                (workInProgress.flags |= 8192)\n              : newProps && (workInProgress.flags |= 8192),\n            newProps\n              ? 0 !== (renderLanes & 536870912) &&\n                0 === (workInProgress.flags & 128) &&\n                (bubbleProperties(workInProgress),\n                workInProgress.subtreeFlags & 6 &&\n                  (workInProgress.flags |= 8192))\n              : bubbleProperties(workInProgress),\n            (renderLanes = workInProgress.updateQueue),\n            null !== renderLanes &&\n              scheduleRetryEffect(workInProgress, renderLanes.retryQueue),\n            (renderLanes = null),\n            null !== current &&\n              null !== current.memoizedState &&\n              null !== current.memoizedState.cachePool &&\n              (renderLanes = current.memoizedState.cachePool.pool),\n            (newProps = null),\n            null !== workInProgress.memoizedState &&\n              null !== workInProgress.memoizedState.cachePool &&\n              (newProps = workInProgress.memoizedState.cachePool.pool),\n            newProps !== renderLanes && (workInProgress.flags |= 2048),\n            null !== current && pop(resumedCache, workInProgress),\n            null\n          );\n        case 24:\n          return (\n            (renderLanes = null),\n            null !== current && (renderLanes = current.memoizedState.cache),\n            workInProgress.memoizedState.cache !== renderLanes &&\n              (workInProgress.flags |= 2048),\n            popProvider(CacheContext, workInProgress),\n            bubbleProperties(workInProgress),\n            null\n          );\n        case 25:\n          return null;\n        case 30:\n          return null;\n      }\n      throw Error(\n        \"Unknown unit of work tag (\" +\n          workInProgress.tag +\n          \"). This error is likely caused by a bug in React. Please file an issue.\"\n      );\n    }\n    function unwindWork(current, workInProgress) {\n      popTreeContext(workInProgress);\n      switch (workInProgress.tag) {\n        case 1:\n          return (\n            (current = workInProgress.flags),\n            current & 65536\n              ? ((workInProgress.flags = (current & -65537) | 128),\n                (workInProgress.mode & ProfileMode) !== NoMode &&\n                  transferActualDuration(workInProgress),\n                workInProgress)\n              : null\n          );\n        case 3:\n          return (\n            popProvider(CacheContext, workInProgress),\n            popHostContainer(workInProgress),\n            (current = workInProgress.flags),\n            0 !== (current & 65536) && 0 === (current & 128)\n              ? ((workInProgress.flags = (current & -65537) | 128),\n                workInProgress)\n              : null\n          );\n        case 26:\n        case 27:\n        case 5:\n          return popHostContext(workInProgress), null;\n        case 13:\n          popSuspenseHandler(workInProgress);\n          current = workInProgress.memoizedState;\n          if (null !== current && null !== current.dehydrated) {\n            if (null === workInProgress.alternate)\n              throw Error(\n                \"Threw in newly mounted dehydrated component. This is likely a bug in React. Please file an issue.\"\n              );\n            resetHydrationState();\n          }\n          current = workInProgress.flags;\n          return current & 65536\n            ? ((workInProgress.flags = (current & -65537) | 128),\n              (workInProgress.mode & ProfileMode) !== NoMode &&\n                transferActualDuration(workInProgress),\n              workInProgress)\n            : null;\n        case 19:\n          return pop(suspenseStackCursor, workInProgress), null;\n        case 4:\n          return popHostContainer(workInProgress), null;\n        case 10:\n          return popProvider(workInProgress.type, workInProgress), null;\n        case 22:\n        case 23:\n          return (\n            popSuspenseHandler(workInProgress),\n            popHiddenContext(workInProgress),\n            null !== current && pop(resumedCache, workInProgress),\n            (current = workInProgress.flags),\n            current & 65536\n              ? ((workInProgress.flags = (current & -65537) | 128),\n                (workInProgress.mode & ProfileMode) !== NoMode &&\n                  transferActualDuration(workInProgress),\n                workInProgress)\n              : null\n          );\n        case 24:\n          return popProvider(CacheContext, workInProgress), null;\n        case 25:\n          return null;\n        default:\n          return null;\n      }\n    }\n    function unwindInterruptedWork(current, interruptedWork) {\n      popTreeContext(interruptedWork);\n      switch (interruptedWork.tag) {\n        case 3:\n          popProvider(CacheContext, interruptedWork);\n          popHostContainer(interruptedWork);\n          break;\n        case 26:\n        case 27:\n        case 5:\n          popHostContext(interruptedWork);\n          break;\n        case 4:\n          popHostContainer(interruptedWork);\n          break;\n        case 13:\n          popSuspenseHandler(interruptedWork);\n          break;\n        case 19:\n          pop(suspenseStackCursor, interruptedWork);\n          break;\n        case 10:\n          popProvider(interruptedWork.type, interruptedWork);\n          break;\n        case 22:\n        case 23:\n          popSuspenseHandler(interruptedWork);\n          popHiddenContext(interruptedWork);\n          null !== current && pop(resumedCache, interruptedWork);\n          break;\n        case 24:\n          popProvider(CacheContext, interruptedWork);\n      }\n    }\n    function shouldProfile(current) {\n      return (current.mode & ProfileMode) !== NoMode;\n    }\n    function commitHookLayoutEffects(finishedWork, hookFlags) {\n      shouldProfile(finishedWork)\n        ? (startEffectTimer(),\n          commitHookEffectListMount(hookFlags, finishedWork),\n          recordEffectDuration())\n        : commitHookEffectListMount(hookFlags, finishedWork);\n    }\n    function commitHookLayoutUnmountEffects(\n      finishedWork,\n      nearestMountedAncestor,\n      hookFlags\n    ) {\n      shouldProfile(finishedWork)\n        ? (startEffectTimer(),\n          commitHookEffectListUnmount(\n            hookFlags,\n            finishedWork,\n            nearestMountedAncestor\n          ),\n          recordEffectDuration())\n        : commitHookEffectListUnmount(\n            hookFlags,\n            finishedWork,\n            nearestMountedAncestor\n          );\n    }\n    function commitHookEffectListMount(flags, finishedWork) {\n      try {\n        var updateQueue = finishedWork.updateQueue,\n          lastEffect = null !== updateQueue ? updateQueue.lastEffect : null;\n        if (null !== lastEffect) {\n          var firstEffect = lastEffect.next;\n          updateQueue = firstEffect;\n          do {\n            if (\n              (updateQueue.tag & flags) === flags &&\n              ((flags & Passive) !== NoFlags\n                ? null !== injectedProfilingHooks &&\n                  \"function\" ===\n                    typeof injectedProfilingHooks.markComponentPassiveEffectMountStarted &&\n                  injectedProfilingHooks.markComponentPassiveEffectMountStarted(\n                    finishedWork\n                  )\n                : (flags & Layout) !== NoFlags &&\n                  null !== injectedProfilingHooks &&\n                  \"function\" ===\n                    typeof injectedProfilingHooks.markComponentLayoutEffectMountStarted &&\n                  injectedProfilingHooks.markComponentLayoutEffectMountStarted(\n                    finishedWork\n                  ),\n              (lastEffect = void 0),\n              (flags & Insertion) !== NoFlags &&\n                (isRunningInsertionEffect = !0),\n              (lastEffect = runWithFiberInDEV(\n                finishedWork,\n                callCreateInDEV,\n                updateQueue\n              )),\n              (flags & Insertion) !== NoFlags &&\n                (isRunningInsertionEffect = !1),\n              (flags & Passive) !== NoFlags\n                ? null !== injectedProfilingHooks &&\n                  \"function\" ===\n                    typeof injectedProfilingHooks.markComponentPassiveEffectMountStopped &&\n                  injectedProfilingHooks.markComponentPassiveEffectMountStopped()\n                : (flags & Layout) !== NoFlags &&\n                  null !== injectedProfilingHooks &&\n                  \"function\" ===\n                    typeof injectedProfilingHooks.markComponentLayoutEffectMountStopped &&\n                  injectedProfilingHooks.markComponentLayoutEffectMountStopped(),\n              void 0 !== lastEffect && \"function\" !== typeof lastEffect)\n            ) {\n              var hookName = void 0;\n              hookName =\n                0 !== (updateQueue.tag & Layout)\n                  ? \"useLayoutEffect\"\n                  : 0 !== (updateQueue.tag & Insertion)\n                    ? \"useInsertionEffect\"\n                    : \"useEffect\";\n              var addendum = void 0;\n              addendum =\n                null === lastEffect\n                  ? \" You returned null. If your effect does not require clean up, return undefined (or nothing).\"\n                  : \"function\" === typeof lastEffect.then\n                    ? \"\\n\\nIt looks like you wrote \" +\n                      hookName +\n                      \"(async () => ...) or returned a Promise. Instead, write the async function inside your effect and call it immediately:\\n\\n\" +\n                      hookName +\n                      \"(() => {\\n  async function fetchData() {\\n    // You can await here\\n    const response = await MyAPI.getData(someId);\\n    // ...\\n  }\\n  fetchData();\\n}, [someId]); // Or [] if effect doesn't need props or state\\n\\nLearn more about data fetching with Hooks: https://react.dev/link/hooks-data-fetching\"\n                    : \" You returned: \" + lastEffect;\n              runWithFiberInDEV(\n                finishedWork,\n                function (n, a) {\n                  console.error(\n                    \"%s must not return anything besides a function, which is used for clean-up.%s\",\n                    n,\n                    a\n                  );\n                },\n                hookName,\n                addendum\n              );\n            }\n            updateQueue = updateQueue.next;\n          } while (updateQueue !== firstEffect);\n        }\n      } catch (error) {\n        captureCommitPhaseError(finishedWork, finishedWork.return, error);\n      }\n    }\n    function commitHookEffectListUnmount(\n      flags,\n      finishedWork,\n      nearestMountedAncestor\n    ) {\n      try {\n        var updateQueue = finishedWork.updateQueue,\n          lastEffect = null !== updateQueue ? updateQueue.lastEffect : null;\n        if (null !== lastEffect) {\n          var firstEffect = lastEffect.next;\n          updateQueue = firstEffect;\n          do {\n            if ((updateQueue.tag & flags) === flags) {\n              var inst = updateQueue.inst,\n                destroy = inst.destroy;\n              void 0 !== destroy &&\n                ((inst.destroy = void 0),\n                (flags & Passive) !== NoFlags\n                  ? null !== injectedProfilingHooks &&\n                    \"function\" ===\n                      typeof injectedProfilingHooks.markComponentPassiveEffectUnmountStarted &&\n                    injectedProfilingHooks.markComponentPassiveEffectUnmountStarted(\n                      finishedWork\n                    )\n                  : (flags & Layout) !== NoFlags &&\n                    null !== injectedProfilingHooks &&\n                    \"function\" ===\n                      typeof injectedProfilingHooks.markComponentLayoutEffectUnmountStarted &&\n                    injectedProfilingHooks.markComponentLayoutEffectUnmountStarted(\n                      finishedWork\n                    ),\n                (flags & Insertion) !== NoFlags &&\n                  (isRunningInsertionEffect = !0),\n                (lastEffect = finishedWork),\n                runWithFiberInDEV(\n                  lastEffect,\n                  callDestroyInDEV,\n                  lastEffect,\n                  nearestMountedAncestor,\n                  destroy\n                ),\n                (flags & Insertion) !== NoFlags &&\n                  (isRunningInsertionEffect = !1),\n                (flags & Passive) !== NoFlags\n                  ? null !== injectedProfilingHooks &&\n                    \"function\" ===\n                      typeof injectedProfilingHooks.markComponentPassiveEffectUnmountStopped &&\n                    injectedProfilingHooks.markComponentPassiveEffectUnmountStopped()\n                  : (flags & Layout) !== NoFlags &&\n                    null !== injectedProfilingHooks &&\n                    \"function\" ===\n                      typeof injectedProfilingHooks.markComponentLayoutEffectUnmountStopped &&\n                    injectedProfilingHooks.markComponentLayoutEffectUnmountStopped());\n            }\n            updateQueue = updateQueue.next;\n          } while (updateQueue !== firstEffect);\n        }\n      } catch (error) {\n        captureCommitPhaseError(finishedWork, finishedWork.return, error);\n      }\n    }\n    function commitHookPassiveMountEffects(finishedWork, hookFlags) {\n      shouldProfile(finishedWork)\n        ? (startEffectTimer(),\n          commitHookEffectListMount(hookFlags, finishedWork),\n          recordEffectDuration())\n        : commitHookEffectListMount(hookFlags, finishedWork);\n    }\n    function commitHookPassiveUnmountEffects(\n      finishedWork,\n      nearestMountedAncestor,\n      hookFlags\n    ) {\n      shouldProfile(finishedWork)\n        ? (startEffectTimer(),\n          commitHookEffectListUnmount(\n            hookFlags,\n            finishedWork,\n            nearestMountedAncestor\n          ),\n          recordEffectDuration())\n        : commitHookEffectListUnmount(\n            hookFlags,\n            finishedWork,\n            nearestMountedAncestor\n          );\n    }\n    function commitClassCallbacks(finishedWork) {\n      var updateQueue = finishedWork.updateQueue;\n      if (null !== updateQueue) {\n        var instance = finishedWork.stateNode;\n        finishedWork.type.defaultProps ||\n          \"ref\" in finishedWork.memoizedProps ||\n          didWarnAboutReassigningProps ||\n          (instance.props !== finishedWork.memoizedProps &&\n            console.error(\n              \"Expected %s props to match memoized props before processing the update queue. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.\",\n              getComponentNameFromFiber(finishedWork) || \"instance\"\n            ),\n          instance.state !== finishedWork.memoizedState &&\n            console.error(\n              \"Expected %s state to match memoized state before processing the update queue. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.\",\n              getComponentNameFromFiber(finishedWork) || \"instance\"\n            ));\n        try {\n          runWithFiberInDEV(\n            finishedWork,\n            commitCallbacks,\n            updateQueue,\n            instance\n          );\n        } catch (error) {\n          captureCommitPhaseError(finishedWork, finishedWork.return, error);\n        }\n      }\n    }\n    function callGetSnapshotBeforeUpdates(instance, prevProps, prevState) {\n      return instance.getSnapshotBeforeUpdate(prevProps, prevState);\n    }\n    function commitClassSnapshot(finishedWork, current) {\n      var prevProps = current.memoizedProps,\n        prevState = current.memoizedState;\n      current = finishedWork.stateNode;\n      finishedWork.type.defaultProps ||\n        \"ref\" in finishedWork.memoizedProps ||\n        didWarnAboutReassigningProps ||\n        (current.props !== finishedWork.memoizedProps &&\n          console.error(\n            \"Expected %s props to match memoized props before getSnapshotBeforeUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.\",\n            getComponentNameFromFiber(finishedWork) || \"instance\"\n          ),\n        current.state !== finishedWork.memoizedState &&\n          console.error(\n            \"Expected %s state to match memoized state before getSnapshotBeforeUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.\",\n            getComponentNameFromFiber(finishedWork) || \"instance\"\n          ));\n      try {\n        var resolvedPrevProps = resolveClassComponentProps(\n          finishedWork.type,\n          prevProps,\n          finishedWork.elementType === finishedWork.type\n        );\n        var snapshot = runWithFiberInDEV(\n          finishedWork,\n          callGetSnapshotBeforeUpdates,\n          current,\n          resolvedPrevProps,\n          prevState\n        );\n        prevProps = didWarnAboutUndefinedSnapshotBeforeUpdate;\n        void 0 !== snapshot ||\n          prevProps.has(finishedWork.type) ||\n          (prevProps.add(finishedWork.type),\n          runWithFiberInDEV(finishedWork, function () {\n            console.error(\n              \"%s.getSnapshotBeforeUpdate(): A snapshot value (or null) must be returned. You have returned undefined.\",\n              getComponentNameFromFiber(finishedWork)\n            );\n          }));\n        current.__reactInternalSnapshotBeforeUpdate = snapshot;\n      } catch (error) {\n        captureCommitPhaseError(finishedWork, finishedWork.return, error);\n      }\n    }\n    function safelyCallComponentWillUnmount(\n      current,\n      nearestMountedAncestor,\n      instance\n    ) {\n      instance.props = resolveClassComponentProps(\n        current.type,\n        current.memoizedProps\n      );\n      instance.state = current.memoizedState;\n      shouldProfile(current)\n        ? (startEffectTimer(),\n          runWithFiberInDEV(\n            current,\n            callComponentWillUnmountInDEV,\n            current,\n            nearestMountedAncestor,\n            instance\n          ),\n          recordEffectDuration())\n        : runWithFiberInDEV(\n            current,\n            callComponentWillUnmountInDEV,\n            current,\n            nearestMountedAncestor,\n            instance\n          );\n    }\n    function commitAttachRef(finishedWork) {\n      var ref = finishedWork.ref;\n      if (null !== ref) {\n        switch (finishedWork.tag) {\n          case 26:\n          case 27:\n          case 5:\n            var instanceToUse = finishedWork.stateNode;\n            break;\n          case 30:\n            instanceToUse = finishedWork.stateNode;\n            break;\n          default:\n            instanceToUse = finishedWork.stateNode;\n        }\n        if (\"function\" === typeof ref)\n          if (shouldProfile(finishedWork))\n            try {\n              startEffectTimer(),\n                (finishedWork.refCleanup = ref(instanceToUse));\n            } finally {\n              recordEffectDuration();\n            }\n          else finishedWork.refCleanup = ref(instanceToUse);\n        else\n          \"string\" === typeof ref\n            ? console.error(\"String refs are no longer supported.\")\n            : ref.hasOwnProperty(\"current\") ||\n              console.error(\n                \"Unexpected ref object provided for %s. Use either a ref-setter function or React.createRef().\",\n                getComponentNameFromFiber(finishedWork)\n              ),\n            (ref.current = instanceToUse);\n      }\n    }\n    function safelyAttachRef(current, nearestMountedAncestor) {\n      try {\n        runWithFiberInDEV(current, commitAttachRef, current);\n      } catch (error) {\n        captureCommitPhaseError(current, nearestMountedAncestor, error);\n      }\n    }\n    function safelyDetachRef(current, nearestMountedAncestor) {\n      var ref = current.ref,\n        refCleanup = current.refCleanup;\n      if (null !== ref)\n        if (\"function\" === typeof refCleanup)\n          try {\n            if (shouldProfile(current))\n              try {\n                startEffectTimer(), runWithFiberInDEV(current, refCleanup);\n              } finally {\n                recordEffectDuration(current);\n              }\n            else runWithFiberInDEV(current, refCleanup);\n          } catch (error) {\n            captureCommitPhaseError(current, nearestMountedAncestor, error);\n          } finally {\n            (current.refCleanup = null),\n              (current = current.alternate),\n              null != current && (current.refCleanup = null);\n          }\n        else if (\"function\" === typeof ref)\n          try {\n            if (shouldProfile(current))\n              try {\n                startEffectTimer(), runWithFiberInDEV(current, ref, null);\n              } finally {\n                recordEffectDuration(current);\n              }\n            else runWithFiberInDEV(current, ref, null);\n          } catch (error$7) {\n            captureCommitPhaseError(current, nearestMountedAncestor, error$7);\n          }\n        else ref.current = null;\n    }\n    function commitProfiler(\n      finishedWork,\n      current,\n      commitStartTime,\n      effectDuration\n    ) {\n      var _finishedWork$memoize = finishedWork.memoizedProps,\n        id = _finishedWork$memoize.id,\n        onCommit = _finishedWork$memoize.onCommit;\n      _finishedWork$memoize = _finishedWork$memoize.onRender;\n      current = null === current ? \"mount\" : \"update\";\n      currentUpdateIsNested && (current = \"nested-update\");\n      \"function\" === typeof _finishedWork$memoize &&\n        _finishedWork$memoize(\n          id,\n          current,\n          finishedWork.actualDuration,\n          finishedWork.treeBaseDuration,\n          finishedWork.actualStartTime,\n          commitStartTime\n        );\n      \"function\" === typeof onCommit &&\n        onCommit(\n          finishedWork.memoizedProps.id,\n          current,\n          effectDuration,\n          commitStartTime\n        );\n    }\n    function commitProfilerPostCommitImpl(\n      finishedWork,\n      current,\n      commitStartTime,\n      passiveEffectDuration\n    ) {\n      var _finishedWork$memoize2 = finishedWork.memoizedProps;\n      finishedWork = _finishedWork$memoize2.id;\n      _finishedWork$memoize2 = _finishedWork$memoize2.onPostCommit;\n      current = null === current ? \"mount\" : \"update\";\n      currentUpdateIsNested && (current = \"nested-update\");\n      \"function\" === typeof _finishedWork$memoize2 &&\n        _finishedWork$memoize2(\n          finishedWork,\n          current,\n          passiveEffectDuration,\n          commitStartTime\n        );\n    }\n    function commitHostMount(finishedWork) {\n      var type = finishedWork.type,\n        props = finishedWork.memoizedProps,\n        instance = finishedWork.stateNode;\n      try {\n        runWithFiberInDEV(\n          finishedWork,\n          commitMount,\n          instance,\n          type,\n          props,\n          finishedWork\n        );\n      } catch (error) {\n        captureCommitPhaseError(finishedWork, finishedWork.return, error);\n      }\n    }\n    function commitHostUpdate(finishedWork, newProps, oldProps) {\n      try {\n        runWithFiberInDEV(\n          finishedWork,\n          commitUpdate,\n          finishedWork.stateNode,\n          finishedWork.type,\n          oldProps,\n          newProps,\n          finishedWork\n        );\n      } catch (error) {\n        captureCommitPhaseError(finishedWork, finishedWork.return, error);\n      }\n    }\n    function isHostParent(fiber) {\n      return (\n        5 === fiber.tag ||\n        3 === fiber.tag ||\n        26 === fiber.tag ||\n        (27 === fiber.tag && isSingletonScope(fiber.type)) ||\n        4 === fiber.tag\n      );\n    }\n    function getHostSibling(fiber) {\n      a: for (;;) {\n        for (; null === fiber.sibling; ) {\n          if (null === fiber.return || isHostParent(fiber.return)) return null;\n          fiber = fiber.return;\n        }\n        fiber.sibling.return = fiber.return;\n        for (\n          fiber = fiber.sibling;\n          5 !== fiber.tag && 6 !== fiber.tag && 18 !== fiber.tag;\n\n        ) {\n          if (27 === fiber.tag && isSingletonScope(fiber.type)) continue a;\n          if (fiber.flags & 2) continue a;\n          if (null === fiber.child || 4 === fiber.tag) continue a;\n          else (fiber.child.return = fiber), (fiber = fiber.child);\n        }\n        if (!(fiber.flags & 2)) return fiber.stateNode;\n      }\n    }\n    function insertOrAppendPlacementNodeIntoContainer(node, before, parent) {\n      var tag = node.tag;\n      if (5 === tag || 6 === tag)\n        (node = node.stateNode),\n          before\n            ? (9 === parent.nodeType\n                ? parent.body\n                : \"HTML\" === parent.nodeName\n                  ? parent.ownerDocument.body\n                  : parent\n              ).insertBefore(node, before)\n            : ((before =\n                9 === parent.nodeType\n                  ? parent.body\n                  : \"HTML\" === parent.nodeName\n                    ? parent.ownerDocument.body\n                    : parent),\n              before.appendChild(node),\n              (parent = parent._reactRootContainer),\n              (null !== parent && void 0 !== parent) ||\n                null !== before.onclick ||\n                (before.onclick = noop$1));\n      else if (\n        4 !== tag &&\n        (27 === tag &&\n          isSingletonScope(node.type) &&\n          ((parent = node.stateNode), (before = null)),\n        (node = node.child),\n        null !== node)\n      )\n        for (\n          insertOrAppendPlacementNodeIntoContainer(node, before, parent),\n            node = node.sibling;\n          null !== node;\n\n        )\n          insertOrAppendPlacementNodeIntoContainer(node, before, parent),\n            (node = node.sibling);\n    }\n    function insertOrAppendPlacementNode(node, before, parent) {\n      var tag = node.tag;\n      if (5 === tag || 6 === tag)\n        (node = node.stateNode),\n          before ? parent.insertBefore(node, before) : parent.appendChild(node);\n      else if (\n        4 !== tag &&\n        (27 === tag && isSingletonScope(node.type) && (parent = node.stateNode),\n        (node = node.child),\n        null !== node)\n      )\n        for (\n          insertOrAppendPlacementNode(node, before, parent),\n            node = node.sibling;\n          null !== node;\n\n        )\n          insertOrAppendPlacementNode(node, before, parent),\n            (node = node.sibling);\n    }\n    function commitPlacement(finishedWork) {\n      for (\n        var hostParentFiber, parentFiber = finishedWork.return;\n        null !== parentFiber;\n\n      ) {\n        if (isHostParent(parentFiber)) {\n          hostParentFiber = parentFiber;\n          break;\n        }\n        parentFiber = parentFiber.return;\n      }\n      if (null == hostParentFiber)\n        throw Error(\n          \"Expected to find a host parent. This error is likely caused by a bug in React. Please file an issue.\"\n        );\n      switch (hostParentFiber.tag) {\n        case 27:\n          hostParentFiber = hostParentFiber.stateNode;\n          parentFiber = getHostSibling(finishedWork);\n          insertOrAppendPlacementNode(\n            finishedWork,\n            parentFiber,\n            hostParentFiber\n          );\n          break;\n        case 5:\n          parentFiber = hostParentFiber.stateNode;\n          hostParentFiber.flags & 32 &&\n            (resetTextContent(parentFiber), (hostParentFiber.flags &= -33));\n          hostParentFiber = getHostSibling(finishedWork);\n          insertOrAppendPlacementNode(\n            finishedWork,\n            hostParentFiber,\n            parentFiber\n          );\n          break;\n        case 3:\n        case 4:\n          hostParentFiber = hostParentFiber.stateNode.containerInfo;\n          parentFiber = getHostSibling(finishedWork);\n          insertOrAppendPlacementNodeIntoContainer(\n            finishedWork,\n            parentFiber,\n            hostParentFiber\n          );\n          break;\n        default:\n          throw Error(\n            \"Invalid host parent fiber. This error is likely caused by a bug in React. Please file an issue.\"\n          );\n      }\n    }\n    function commitHostSingletonAcquisition(finishedWork) {\n      var singleton = finishedWork.stateNode,\n        props = finishedWork.memoizedProps;\n      try {\n        runWithFiberInDEV(\n          finishedWork,\n          acquireSingletonInstance,\n          finishedWork.type,\n          props,\n          singleton,\n          finishedWork\n        );\n      } catch (error) {\n        captureCommitPhaseError(finishedWork, finishedWork.return, error);\n      }\n    }\n    function commitBeforeMutationEffects(root, firstChild) {\n      root = root.containerInfo;\n      eventsEnabled = _enabled;\n      root = getActiveElementDeep(root);\n      if (hasSelectionCapabilities(root)) {\n        if (\"selectionStart\" in root)\n          var JSCompiler_temp = {\n            start: root.selectionStart,\n            end: root.selectionEnd\n          };\n        else\n          a: {\n            JSCompiler_temp =\n              ((JSCompiler_temp = root.ownerDocument) &&\n                JSCompiler_temp.defaultView) ||\n              window;\n            var selection =\n              JSCompiler_temp.getSelection && JSCompiler_temp.getSelection();\n            if (selection && 0 !== selection.rangeCount) {\n              JSCompiler_temp = selection.anchorNode;\n              var anchorOffset = selection.anchorOffset,\n                focusNode = selection.focusNode;\n              selection = selection.focusOffset;\n              try {\n                JSCompiler_temp.nodeType, focusNode.nodeType;\n              } catch (e$2) {\n                JSCompiler_temp = null;\n                break a;\n              }\n              var length = 0,\n                start = -1,\n                end = -1,\n                indexWithinAnchor = 0,\n                indexWithinFocus = 0,\n                node = root,\n                parentNode = null;\n              b: for (;;) {\n                for (var next; ; ) {\n                  node !== JSCompiler_temp ||\n                    (0 !== anchorOffset && 3 !== node.nodeType) ||\n                    (start = length + anchorOffset);\n                  node !== focusNode ||\n                    (0 !== selection && 3 !== node.nodeType) ||\n                    (end = length + selection);\n                  3 === node.nodeType && (length += node.nodeValue.length);\n                  if (null === (next = node.firstChild)) break;\n                  parentNode = node;\n                  node = next;\n                }\n                for (;;) {\n                  if (node === root) break b;\n                  parentNode === JSCompiler_temp &&\n                    ++indexWithinAnchor === anchorOffset &&\n                    (start = length);\n                  parentNode === focusNode &&\n                    ++indexWithinFocus === selection &&\n                    (end = length);\n                  if (null !== (next = node.nextSibling)) break;\n                  node = parentNode;\n                  parentNode = node.parentNode;\n                }\n                node = next;\n              }\n              JSCompiler_temp =\n                -1 === start || -1 === end ? null : { start: start, end: end };\n            } else JSCompiler_temp = null;\n          }\n        JSCompiler_temp = JSCompiler_temp || { start: 0, end: 0 };\n      } else JSCompiler_temp = null;\n      selectionInformation = {\n        focusedElem: root,\n        selectionRange: JSCompiler_temp\n      };\n      _enabled = !1;\n      for (nextEffect = firstChild; null !== nextEffect; )\n        if (\n          ((firstChild = nextEffect),\n          (root = firstChild.child),\n          0 !== (firstChild.subtreeFlags & 1024) && null !== root)\n        )\n          (root.return = firstChild), (nextEffect = root);\n        else\n          for (; null !== nextEffect; ) {\n            root = firstChild = nextEffect;\n            JSCompiler_temp = root.alternate;\n            anchorOffset = root.flags;\n            switch (root.tag) {\n              case 0:\n                break;\n              case 11:\n              case 15:\n                break;\n              case 1:\n                0 !== (anchorOffset & 1024) &&\n                  null !== JSCompiler_temp &&\n                  commitClassSnapshot(root, JSCompiler_temp);\n                break;\n              case 3:\n                if (0 !== (anchorOffset & 1024))\n                  if (\n                    ((root = root.stateNode.containerInfo),\n                    (JSCompiler_temp = root.nodeType),\n                    9 === JSCompiler_temp)\n                  )\n                    clearContainerSparingly(root);\n                  else if (1 === JSCompiler_temp)\n                    switch (root.nodeName) {\n                      case \"HEAD\":\n                      case \"HTML\":\n                      case \"BODY\":\n                        clearContainerSparingly(root);\n                        break;\n                      default:\n                        root.textContent = \"\";\n                    }\n                break;\n              case 5:\n              case 26:\n              case 27:\n              case 6:\n              case 4:\n              case 17:\n                break;\n              default:\n                if (0 !== (anchorOffset & 1024))\n                  throw Error(\n                    \"This unit of work tag should not have side-effects. This error is likely caused by a bug in React. Please file an issue.\"\n                  );\n            }\n            root = firstChild.sibling;\n            if (null !== root) {\n              root.return = firstChild.return;\n              nextEffect = root;\n              break;\n            }\n            nextEffect = firstChild.return;\n          }\n    }\n    function commitLayoutEffectOnFiber(finishedRoot, current, finishedWork) {\n      var flags = finishedWork.flags;\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 15:\n          recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n          flags & 4 &&\n            commitHookLayoutEffects(finishedWork, Layout | HasEffect);\n          break;\n        case 1:\n          recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n          if (flags & 4)\n            if (((finishedRoot = finishedWork.stateNode), null === current))\n              finishedWork.type.defaultProps ||\n                \"ref\" in finishedWork.memoizedProps ||\n                didWarnAboutReassigningProps ||\n                (finishedRoot.props !== finishedWork.memoizedProps &&\n                  console.error(\n                    \"Expected %s props to match memoized props before componentDidMount. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.\",\n                    getComponentNameFromFiber(finishedWork) || \"instance\"\n                  ),\n                finishedRoot.state !== finishedWork.memoizedState &&\n                  console.error(\n                    \"Expected %s state to match memoized state before componentDidMount. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.\",\n                    getComponentNameFromFiber(finishedWork) || \"instance\"\n                  )),\n                shouldProfile(finishedWork)\n                  ? (startEffectTimer(),\n                    runWithFiberInDEV(\n                      finishedWork,\n                      callComponentDidMountInDEV,\n                      finishedWork,\n                      finishedRoot\n                    ),\n                    recordEffectDuration())\n                  : runWithFiberInDEV(\n                      finishedWork,\n                      callComponentDidMountInDEV,\n                      finishedWork,\n                      finishedRoot\n                    );\n            else {\n              var prevProps = resolveClassComponentProps(\n                finishedWork.type,\n                current.memoizedProps\n              );\n              current = current.memoizedState;\n              finishedWork.type.defaultProps ||\n                \"ref\" in finishedWork.memoizedProps ||\n                didWarnAboutReassigningProps ||\n                (finishedRoot.props !== finishedWork.memoizedProps &&\n                  console.error(\n                    \"Expected %s props to match memoized props before componentDidUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.\",\n                    getComponentNameFromFiber(finishedWork) || \"instance\"\n                  ),\n                finishedRoot.state !== finishedWork.memoizedState &&\n                  console.error(\n                    \"Expected %s state to match memoized state before componentDidUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.\",\n                    getComponentNameFromFiber(finishedWork) || \"instance\"\n                  ));\n              shouldProfile(finishedWork)\n                ? (startEffectTimer(),\n                  runWithFiberInDEV(\n                    finishedWork,\n                    callComponentDidUpdateInDEV,\n                    finishedWork,\n                    finishedRoot,\n                    prevProps,\n                    current,\n                    finishedRoot.__reactInternalSnapshotBeforeUpdate\n                  ),\n                  recordEffectDuration())\n                : runWithFiberInDEV(\n                    finishedWork,\n                    callComponentDidUpdateInDEV,\n                    finishedWork,\n                    finishedRoot,\n                    prevProps,\n                    current,\n                    finishedRoot.__reactInternalSnapshotBeforeUpdate\n                  );\n            }\n          flags & 64 && commitClassCallbacks(finishedWork);\n          flags & 512 && safelyAttachRef(finishedWork, finishedWork.return);\n          break;\n        case 3:\n          current = pushNestedEffectDurations();\n          recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n          if (\n            flags & 64 &&\n            ((flags = finishedWork.updateQueue), null !== flags)\n          ) {\n            prevProps = null;\n            if (null !== finishedWork.child)\n              switch (finishedWork.child.tag) {\n                case 27:\n                case 5:\n                  prevProps = finishedWork.child.stateNode;\n                  break;\n                case 1:\n                  prevProps = finishedWork.child.stateNode;\n              }\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitCallbacks,\n                flags,\n                prevProps\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          }\n          finishedRoot.effectDuration += popNestedEffectDurations(current);\n          break;\n        case 27:\n          null === current &&\n            flags & 4 &&\n            commitHostSingletonAcquisition(finishedWork);\n        case 26:\n        case 5:\n          recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n          null === current && flags & 4 && commitHostMount(finishedWork);\n          flags & 512 && safelyAttachRef(finishedWork, finishedWork.return);\n          break;\n        case 12:\n          if (flags & 4) {\n            flags = pushNestedEffectDurations();\n            recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n            finishedRoot = finishedWork.stateNode;\n            finishedRoot.effectDuration += bubbleNestedEffectDurations(flags);\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitProfiler,\n                finishedWork,\n                current,\n                commitStartTime,\n                finishedRoot.effectDuration\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          } else recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n          break;\n        case 13:\n          recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n          flags & 4 &&\n            commitSuspenseHydrationCallbacks(finishedRoot, finishedWork);\n          flags & 64 &&\n            ((finishedRoot = finishedWork.memoizedState),\n            null !== finishedRoot &&\n              ((finishedRoot = finishedRoot.dehydrated),\n              null !== finishedRoot &&\n                ((finishedWork = retryDehydratedSuspenseBoundary.bind(\n                  null,\n                  finishedWork\n                )),\n                registerSuspenseInstanceRetry(finishedRoot, finishedWork))));\n          break;\n        case 22:\n          flags =\n            null !== finishedWork.memoizedState || offscreenSubtreeIsHidden;\n          if (!flags) {\n            current =\n              (null !== current && null !== current.memoizedState) ||\n              offscreenSubtreeWasHidden;\n            prevProps = offscreenSubtreeIsHidden;\n            var prevOffscreenSubtreeWasHidden = offscreenSubtreeWasHidden;\n            offscreenSubtreeIsHidden = flags;\n            (offscreenSubtreeWasHidden = current) &&\n            !prevOffscreenSubtreeWasHidden\n              ? recursivelyTraverseReappearLayoutEffects(\n                  finishedRoot,\n                  finishedWork,\n                  0 !== (finishedWork.subtreeFlags & 8772)\n                )\n              : recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n            offscreenSubtreeIsHidden = prevProps;\n            offscreenSubtreeWasHidden = prevOffscreenSubtreeWasHidden;\n          }\n          break;\n        case 30:\n          break;\n        default:\n          recursivelyTraverseLayoutEffects(finishedRoot, finishedWork);\n      }\n    }\n    function detachFiberAfterEffects(fiber) {\n      var alternate = fiber.alternate;\n      null !== alternate &&\n        ((fiber.alternate = null), detachFiberAfterEffects(alternate));\n      fiber.child = null;\n      fiber.deletions = null;\n      fiber.sibling = null;\n      5 === fiber.tag &&\n        ((alternate = fiber.stateNode),\n        null !== alternate && detachDeletedInstance(alternate));\n      fiber.stateNode = null;\n      fiber._debugOwner = null;\n      fiber.return = null;\n      fiber.dependencies = null;\n      fiber.memoizedProps = null;\n      fiber.memoizedState = null;\n      fiber.pendingProps = null;\n      fiber.stateNode = null;\n      fiber.updateQueue = null;\n    }\n    function recursivelyTraverseDeletionEffects(\n      finishedRoot,\n      nearestMountedAncestor,\n      parent\n    ) {\n      for (parent = parent.child; null !== parent; )\n        commitDeletionEffectsOnFiber(\n          finishedRoot,\n          nearestMountedAncestor,\n          parent\n        ),\n          (parent = parent.sibling);\n    }\n    function commitDeletionEffectsOnFiber(\n      finishedRoot,\n      nearestMountedAncestor,\n      deletedFiber\n    ) {\n      if (\n        injectedHook &&\n        \"function\" === typeof injectedHook.onCommitFiberUnmount\n      )\n        try {\n          injectedHook.onCommitFiberUnmount(rendererID, deletedFiber);\n        } catch (err) {\n          hasLoggedError ||\n            ((hasLoggedError = !0),\n            console.error(\n              \"React instrumentation encountered an error: %s\",\n              err\n            ));\n        }\n      switch (deletedFiber.tag) {\n        case 26:\n          offscreenSubtreeWasHidden ||\n            safelyDetachRef(deletedFiber, nearestMountedAncestor);\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          deletedFiber.memoizedState\n            ? deletedFiber.memoizedState.count--\n            : deletedFiber.stateNode &&\n              ((deletedFiber = deletedFiber.stateNode),\n              deletedFiber.parentNode.removeChild(deletedFiber));\n          break;\n        case 27:\n          offscreenSubtreeWasHidden ||\n            safelyDetachRef(deletedFiber, nearestMountedAncestor);\n          var prevHostParent = hostParent,\n            prevHostParentIsContainer = hostParentIsContainer;\n          isSingletonScope(deletedFiber.type) &&\n            ((hostParent = deletedFiber.stateNode),\n            (hostParentIsContainer = !1));\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          runWithFiberInDEV(\n            deletedFiber,\n            releaseSingletonInstance,\n            deletedFiber.stateNode\n          );\n          hostParent = prevHostParent;\n          hostParentIsContainer = prevHostParentIsContainer;\n          break;\n        case 5:\n          offscreenSubtreeWasHidden ||\n            safelyDetachRef(deletedFiber, nearestMountedAncestor);\n        case 6:\n          prevHostParent = hostParent;\n          prevHostParentIsContainer = hostParentIsContainer;\n          hostParent = null;\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          hostParent = prevHostParent;\n          hostParentIsContainer = prevHostParentIsContainer;\n          if (null !== hostParent)\n            if (hostParentIsContainer)\n              try {\n                runWithFiberInDEV(\n                  deletedFiber,\n                  removeChildFromContainer,\n                  hostParent,\n                  deletedFiber.stateNode\n                );\n              } catch (error) {\n                captureCommitPhaseError(\n                  deletedFiber,\n                  nearestMountedAncestor,\n                  error\n                );\n              }\n            else\n              try {\n                runWithFiberInDEV(\n                  deletedFiber,\n                  removeChild,\n                  hostParent,\n                  deletedFiber.stateNode\n                );\n              } catch (error) {\n                captureCommitPhaseError(\n                  deletedFiber,\n                  nearestMountedAncestor,\n                  error\n                );\n              }\n          break;\n        case 18:\n          null !== hostParent &&\n            (hostParentIsContainer\n              ? ((finishedRoot = hostParent),\n                clearSuspenseBoundary(\n                  9 === finishedRoot.nodeType\n                    ? finishedRoot.body\n                    : \"HTML\" === finishedRoot.nodeName\n                      ? finishedRoot.ownerDocument.body\n                      : finishedRoot,\n                  deletedFiber.stateNode\n                ),\n                retryIfBlockedOn(finishedRoot))\n              : clearSuspenseBoundary(hostParent, deletedFiber.stateNode));\n          break;\n        case 4:\n          prevHostParent = hostParent;\n          prevHostParentIsContainer = hostParentIsContainer;\n          hostParent = deletedFiber.stateNode.containerInfo;\n          hostParentIsContainer = !0;\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          hostParent = prevHostParent;\n          hostParentIsContainer = prevHostParentIsContainer;\n          break;\n        case 0:\n        case 11:\n        case 14:\n        case 15:\n          offscreenSubtreeWasHidden ||\n            commitHookEffectListUnmount(\n              Insertion,\n              deletedFiber,\n              nearestMountedAncestor\n            );\n          offscreenSubtreeWasHidden ||\n            commitHookLayoutUnmountEffects(\n              deletedFiber,\n              nearestMountedAncestor,\n              Layout\n            );\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          break;\n        case 1:\n          offscreenSubtreeWasHidden ||\n            (safelyDetachRef(deletedFiber, nearestMountedAncestor),\n            (prevHostParent = deletedFiber.stateNode),\n            \"function\" === typeof prevHostParent.componentWillUnmount &&\n              safelyCallComponentWillUnmount(\n                deletedFiber,\n                nearestMountedAncestor,\n                prevHostParent\n              ));\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          break;\n        case 21:\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          break;\n        case 22:\n          offscreenSubtreeWasHidden =\n            (prevHostParent = offscreenSubtreeWasHidden) ||\n            null !== deletedFiber.memoizedState;\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n          offscreenSubtreeWasHidden = prevHostParent;\n          break;\n        default:\n          recursivelyTraverseDeletionEffects(\n            finishedRoot,\n            nearestMountedAncestor,\n            deletedFiber\n          );\n      }\n    }\n    function commitSuspenseHydrationCallbacks(finishedRoot, finishedWork) {\n      if (\n        null === finishedWork.memoizedState &&\n        ((finishedRoot = finishedWork.alternate),\n        null !== finishedRoot &&\n          ((finishedRoot = finishedRoot.memoizedState),\n          null !== finishedRoot &&\n            ((finishedRoot = finishedRoot.dehydrated), null !== finishedRoot)))\n      )\n        try {\n          runWithFiberInDEV(\n            finishedWork,\n            commitHydratedSuspenseInstance,\n            finishedRoot\n          );\n        } catch (error) {\n          captureCommitPhaseError(finishedWork, finishedWork.return, error);\n        }\n    }\n    function getRetryCache(finishedWork) {\n      switch (finishedWork.tag) {\n        case 13:\n        case 19:\n          var retryCache = finishedWork.stateNode;\n          null === retryCache &&\n            (retryCache = finishedWork.stateNode = new PossiblyWeakSet());\n          return retryCache;\n        case 22:\n          return (\n            (finishedWork = finishedWork.stateNode),\n            (retryCache = finishedWork._retryCache),\n            null === retryCache &&\n              (retryCache = finishedWork._retryCache = new PossiblyWeakSet()),\n            retryCache\n          );\n        default:\n          throw Error(\n            \"Unexpected Suspense handler tag (\" +\n              finishedWork.tag +\n              \"). This is a bug in React.\"\n          );\n      }\n    }\n    function attachSuspenseRetryListeners(finishedWork, wakeables) {\n      var retryCache = getRetryCache(finishedWork);\n      wakeables.forEach(function (wakeable) {\n        var retry = resolveRetryWakeable.bind(null, finishedWork, wakeable);\n        if (!retryCache.has(wakeable)) {\n          retryCache.add(wakeable);\n          if (isDevToolsPresent)\n            if (null !== inProgressLanes && null !== inProgressRoot)\n              restorePendingUpdaters(inProgressRoot, inProgressLanes);\n            else\n              throw Error(\n                \"Expected finished root and lanes to be set. This is a bug in React.\"\n              );\n          wakeable.then(retry, retry);\n        }\n      });\n    }\n    function recursivelyTraverseMutationEffects(root$jscomp$0, parentFiber) {\n      var deletions = parentFiber.deletions;\n      if (null !== deletions)\n        for (var i = 0; i < deletions.length; i++) {\n          var root = root$jscomp$0,\n            returnFiber = parentFiber,\n            deletedFiber = deletions[i],\n            parent = returnFiber;\n          a: for (; null !== parent; ) {\n            switch (parent.tag) {\n              case 27:\n                if (isSingletonScope(parent.type)) {\n                  hostParent = parent.stateNode;\n                  hostParentIsContainer = !1;\n                  break a;\n                }\n                break;\n              case 5:\n                hostParent = parent.stateNode;\n                hostParentIsContainer = !1;\n                break a;\n              case 3:\n              case 4:\n                hostParent = parent.stateNode.containerInfo;\n                hostParentIsContainer = !0;\n                break a;\n            }\n            parent = parent.return;\n          }\n          if (null === hostParent)\n            throw Error(\n              \"Expected to find a host parent. This error is likely caused by a bug in React. Please file an issue.\"\n            );\n          commitDeletionEffectsOnFiber(root, returnFiber, deletedFiber);\n          hostParent = null;\n          hostParentIsContainer = !1;\n          root = deletedFiber;\n          returnFiber = root.alternate;\n          null !== returnFiber && (returnFiber.return = null);\n          root.return = null;\n        }\n      if (parentFiber.subtreeFlags & 13878)\n        for (parentFiber = parentFiber.child; null !== parentFiber; )\n          commitMutationEffectsOnFiber(parentFiber, root$jscomp$0),\n            (parentFiber = parentFiber.sibling);\n    }\n    function commitMutationEffectsOnFiber(finishedWork, root) {\n      var current = finishedWork.alternate,\n        flags = finishedWork.flags;\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 14:\n        case 15:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          flags & 4 &&\n            (commitHookEffectListUnmount(\n              Insertion | HasEffect,\n              finishedWork,\n              finishedWork.return\n            ),\n            commitHookEffectListMount(Insertion | HasEffect, finishedWork),\n            commitHookLayoutUnmountEffects(\n              finishedWork,\n              finishedWork.return,\n              Layout | HasEffect\n            ));\n          break;\n        case 1:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          flags & 512 &&\n            (offscreenSubtreeWasHidden ||\n              null === current ||\n              safelyDetachRef(current, current.return));\n          flags & 64 &&\n            offscreenSubtreeIsHidden &&\n            ((finishedWork = finishedWork.updateQueue),\n            null !== finishedWork &&\n              ((flags = finishedWork.callbacks),\n              null !== flags &&\n                ((current = finishedWork.shared.hiddenCallbacks),\n                (finishedWork.shared.hiddenCallbacks =\n                  null === current ? flags : current.concat(flags)))));\n          break;\n        case 26:\n          var hoistableRoot = currentHoistableRoot;\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          flags & 512 &&\n            (offscreenSubtreeWasHidden ||\n              null === current ||\n              safelyDetachRef(current, current.return));\n          if (flags & 4)\n            if (\n              ((root = null !== current ? current.memoizedState : null),\n              (flags = finishedWork.memoizedState),\n              null === current)\n            )\n              if (null === flags)\n                if (null === finishedWork.stateNode) {\n                  a: {\n                    flags = finishedWork.type;\n                    current = finishedWork.memoizedProps;\n                    root = hoistableRoot.ownerDocument || hoistableRoot;\n                    b: switch (flags) {\n                      case \"title\":\n                        hoistableRoot = root.getElementsByTagName(\"title\")[0];\n                        if (\n                          !hoistableRoot ||\n                          hoistableRoot[internalHoistableMarker] ||\n                          hoistableRoot[internalInstanceKey] ||\n                          hoistableRoot.namespaceURI === SVG_NAMESPACE ||\n                          hoistableRoot.hasAttribute(\"itemprop\")\n                        )\n                          (hoistableRoot = root.createElement(flags)),\n                            root.head.insertBefore(\n                              hoistableRoot,\n                              root.querySelector(\"head > title\")\n                            );\n                        setInitialProperties(hoistableRoot, flags, current);\n                        hoistableRoot[internalInstanceKey] = finishedWork;\n                        markNodeAsHoistable(hoistableRoot);\n                        flags = hoistableRoot;\n                        break a;\n                      case \"link\":\n                        var maybeNodes = getHydratableHoistableCache(\n                          \"link\",\n                          \"href\",\n                          root\n                        ).get(flags + (current.href || \"\"));\n                        if (maybeNodes)\n                          for (var i = 0; i < maybeNodes.length; i++)\n                            if (\n                              ((hoistableRoot = maybeNodes[i]),\n                              hoistableRoot.getAttribute(\"href\") ===\n                                (null == current.href || \"\" === current.href\n                                  ? null\n                                  : current.href) &&\n                                hoistableRoot.getAttribute(\"rel\") ===\n                                  (null == current.rel ? null : current.rel) &&\n                                hoistableRoot.getAttribute(\"title\") ===\n                                  (null == current.title\n                                    ? null\n                                    : current.title) &&\n                                hoistableRoot.getAttribute(\"crossorigin\") ===\n                                  (null == current.crossOrigin\n                                    ? null\n                                    : current.crossOrigin))\n                            ) {\n                              maybeNodes.splice(i, 1);\n                              break b;\n                            }\n                        hoistableRoot = root.createElement(flags);\n                        setInitialProperties(hoistableRoot, flags, current);\n                        root.head.appendChild(hoistableRoot);\n                        break;\n                      case \"meta\":\n                        if (\n                          (maybeNodes = getHydratableHoistableCache(\n                            \"meta\",\n                            \"content\",\n                            root\n                          ).get(flags + (current.content || \"\")))\n                        )\n                          for (i = 0; i < maybeNodes.length; i++)\n                            if (\n                              ((hoistableRoot = maybeNodes[i]),\n                              checkAttributeStringCoercion(\n                                current.content,\n                                \"content\"\n                              ),\n                              hoistableRoot.getAttribute(\"content\") ===\n                                (null == current.content\n                                  ? null\n                                  : \"\" + current.content) &&\n                                hoistableRoot.getAttribute(\"name\") ===\n                                  (null == current.name\n                                    ? null\n                                    : current.name) &&\n                                hoistableRoot.getAttribute(\"property\") ===\n                                  (null == current.property\n                                    ? null\n                                    : current.property) &&\n                                hoistableRoot.getAttribute(\"http-equiv\") ===\n                                  (null == current.httpEquiv\n                                    ? null\n                                    : current.httpEquiv) &&\n                                hoistableRoot.getAttribute(\"charset\") ===\n                                  (null == current.charSet\n                                    ? null\n                                    : current.charSet))\n                            ) {\n                              maybeNodes.splice(i, 1);\n                              break b;\n                            }\n                        hoistableRoot = root.createElement(flags);\n                        setInitialProperties(hoistableRoot, flags, current);\n                        root.head.appendChild(hoistableRoot);\n                        break;\n                      default:\n                        throw Error(\n                          'getNodesForType encountered a type it did not expect: \"' +\n                            flags +\n                            '\". This is a bug in React.'\n                        );\n                    }\n                    hoistableRoot[internalInstanceKey] = finishedWork;\n                    markNodeAsHoistable(hoistableRoot);\n                    flags = hoistableRoot;\n                  }\n                  finishedWork.stateNode = flags;\n                } else\n                  mountHoistable(\n                    hoistableRoot,\n                    finishedWork.type,\n                    finishedWork.stateNode\n                  );\n              else\n                finishedWork.stateNode = acquireResource(\n                  hoistableRoot,\n                  flags,\n                  finishedWork.memoizedProps\n                );\n            else\n              root !== flags\n                ? (null === root\n                    ? null !== current.stateNode &&\n                      ((current = current.stateNode),\n                      current.parentNode.removeChild(current))\n                    : root.count--,\n                  null === flags\n                    ? mountHoistable(\n                        hoistableRoot,\n                        finishedWork.type,\n                        finishedWork.stateNode\n                      )\n                    : acquireResource(\n                        hoistableRoot,\n                        flags,\n                        finishedWork.memoizedProps\n                      ))\n                : null === flags &&\n                  null !== finishedWork.stateNode &&\n                  commitHostUpdate(\n                    finishedWork,\n                    finishedWork.memoizedProps,\n                    current.memoizedProps\n                  );\n          break;\n        case 27:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          flags & 512 &&\n            (offscreenSubtreeWasHidden ||\n              null === current ||\n              safelyDetachRef(current, current.return));\n          null !== current &&\n            flags & 4 &&\n            commitHostUpdate(\n              finishedWork,\n              finishedWork.memoizedProps,\n              current.memoizedProps\n            );\n          break;\n        case 5:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          flags & 512 &&\n            (offscreenSubtreeWasHidden ||\n              null === current ||\n              safelyDetachRef(current, current.return));\n          if (finishedWork.flags & 32) {\n            root = finishedWork.stateNode;\n            try {\n              runWithFiberInDEV(finishedWork, resetTextContent, root);\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          }\n          flags & 4 &&\n            null != finishedWork.stateNode &&\n            ((root = finishedWork.memoizedProps),\n            commitHostUpdate(\n              finishedWork,\n              root,\n              null !== current ? current.memoizedProps : root\n            ));\n          flags & 1024 &&\n            ((needsFormReset = !0),\n            \"form\" !== finishedWork.type &&\n              console.error(\n                \"Unexpected host component type. Expected a form. This is a bug in React.\"\n              ));\n          break;\n        case 6:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          if (flags & 4) {\n            if (null === finishedWork.stateNode)\n              throw Error(\n                \"This should have a text node initialized. This error is likely caused by a bug in React. Please file an issue.\"\n              );\n            flags = finishedWork.memoizedProps;\n            current = null !== current ? current.memoizedProps : flags;\n            root = finishedWork.stateNode;\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitTextUpdate,\n                root,\n                current,\n                flags\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          }\n          break;\n        case 3:\n          hoistableRoot = pushNestedEffectDurations();\n          tagCaches = null;\n          maybeNodes = currentHoistableRoot;\n          currentHoistableRoot = getHoistableRoot(root.containerInfo);\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          currentHoistableRoot = maybeNodes;\n          commitReconciliationEffects(finishedWork);\n          if (\n            flags & 4 &&\n            null !== current &&\n            current.memoizedState.isDehydrated\n          )\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitHydratedContainer,\n                root.containerInfo\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          needsFormReset &&\n            ((needsFormReset = !1), recursivelyResetForms(finishedWork));\n          root.effectDuration += popNestedEffectDurations(hoistableRoot);\n          break;\n        case 4:\n          flags = currentHoistableRoot;\n          currentHoistableRoot = getHoistableRoot(\n            finishedWork.stateNode.containerInfo\n          );\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          currentHoistableRoot = flags;\n          break;\n        case 12:\n          flags = pushNestedEffectDurations();\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          finishedWork.stateNode.effectDuration +=\n            bubbleNestedEffectDurations(flags);\n          break;\n        case 13:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          finishedWork.child.flags & 8192 &&\n            (null !== finishedWork.memoizedState) !==\n              (null !== current && null !== current.memoizedState) &&\n            (globalMostRecentFallbackTime = now$1());\n          flags & 4 &&\n            ((flags = finishedWork.updateQueue),\n            null !== flags &&\n              ((finishedWork.updateQueue = null),\n              attachSuspenseRetryListeners(finishedWork, flags)));\n          break;\n        case 22:\n          hoistableRoot = null !== finishedWork.memoizedState;\n          var wasHidden = null !== current && null !== current.memoizedState,\n            prevOffscreenSubtreeIsHidden = offscreenSubtreeIsHidden,\n            prevOffscreenSubtreeWasHidden = offscreenSubtreeWasHidden;\n          offscreenSubtreeIsHidden =\n            prevOffscreenSubtreeIsHidden || hoistableRoot;\n          offscreenSubtreeWasHidden =\n            prevOffscreenSubtreeWasHidden || wasHidden;\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          offscreenSubtreeWasHidden = prevOffscreenSubtreeWasHidden;\n          offscreenSubtreeIsHidden = prevOffscreenSubtreeIsHidden;\n          commitReconciliationEffects(finishedWork);\n          if (flags & 8192)\n            a: for (\n              root = finishedWork.stateNode,\n                root._visibility = hoistableRoot\n                  ? root._visibility & ~OffscreenVisible\n                  : root._visibility | OffscreenVisible,\n                hoistableRoot &&\n                  (null === current ||\n                    wasHidden ||\n                    offscreenSubtreeIsHidden ||\n                    offscreenSubtreeWasHidden ||\n                    recursivelyTraverseDisappearLayoutEffects(finishedWork)),\n                current = null,\n                root = finishedWork;\n              ;\n\n            ) {\n              if (5 === root.tag || 26 === root.tag) {\n                if (null === current) {\n                  wasHidden = current = root;\n                  try {\n                    (maybeNodes = wasHidden.stateNode),\n                      hoistableRoot\n                        ? runWithFiberInDEV(wasHidden, hideInstance, maybeNodes)\n                        : runWithFiberInDEV(\n                            wasHidden,\n                            unhideInstance,\n                            wasHidden.stateNode,\n                            wasHidden.memoizedProps\n                          );\n                  } catch (error) {\n                    captureCommitPhaseError(wasHidden, wasHidden.return, error);\n                  }\n                }\n              } else if (6 === root.tag) {\n                if (null === current) {\n                  wasHidden = root;\n                  try {\n                    (i = wasHidden.stateNode),\n                      hoistableRoot\n                        ? runWithFiberInDEV(wasHidden, hideTextInstance, i)\n                        : runWithFiberInDEV(\n                            wasHidden,\n                            unhideTextInstance,\n                            i,\n                            wasHidden.memoizedProps\n                          );\n                  } catch (error) {\n                    captureCommitPhaseError(wasHidden, wasHidden.return, error);\n                  }\n                }\n              } else if (\n                ((22 !== root.tag && 23 !== root.tag) ||\n                  null === root.memoizedState ||\n                  root === finishedWork) &&\n                null !== root.child\n              ) {\n                root.child.return = root;\n                root = root.child;\n                continue;\n              }\n              if (root === finishedWork) break a;\n              for (; null === root.sibling; ) {\n                if (null === root.return || root.return === finishedWork)\n                  break a;\n                current === root && (current = null);\n                root = root.return;\n              }\n              current === root && (current = null);\n              root.sibling.return = root.return;\n              root = root.sibling;\n            }\n          flags & 4 &&\n            ((flags = finishedWork.updateQueue),\n            null !== flags &&\n              ((current = flags.retryQueue),\n              null !== current &&\n                ((flags.retryQueue = null),\n                attachSuspenseRetryListeners(finishedWork, current))));\n          break;\n        case 19:\n          recursivelyTraverseMutationEffects(root, finishedWork);\n          commitReconciliationEffects(finishedWork);\n          flags & 4 &&\n            ((flags = finishedWork.updateQueue),\n            null !== flags &&\n              ((finishedWork.updateQueue = null),\n              attachSuspenseRetryListeners(finishedWork, flags)));\n          break;\n        case 30:\n          break;\n        case 21:\n          break;\n        default:\n          recursivelyTraverseMutationEffects(root, finishedWork),\n            commitReconciliationEffects(finishedWork);\n      }\n    }\n    function commitReconciliationEffects(finishedWork) {\n      var flags = finishedWork.flags;\n      if (flags & 2) {\n        try {\n          runWithFiberInDEV(finishedWork, commitPlacement, finishedWork);\n        } catch (error) {\n          captureCommitPhaseError(finishedWork, finishedWork.return, error);\n        }\n        finishedWork.flags &= -3;\n      }\n      flags & 4096 && (finishedWork.flags &= -4097);\n    }\n    function recursivelyResetForms(parentFiber) {\n      if (parentFiber.subtreeFlags & 1024)\n        for (parentFiber = parentFiber.child; null !== parentFiber; ) {\n          var fiber = parentFiber;\n          recursivelyResetForms(fiber);\n          5 === fiber.tag && fiber.flags & 1024 && fiber.stateNode.reset();\n          parentFiber = parentFiber.sibling;\n        }\n    }\n    function recursivelyTraverseLayoutEffects(root, parentFiber) {\n      if (parentFiber.subtreeFlags & 8772)\n        for (parentFiber = parentFiber.child; null !== parentFiber; )\n          commitLayoutEffectOnFiber(root, parentFiber.alternate, parentFiber),\n            (parentFiber = parentFiber.sibling);\n    }\n    function disappearLayoutEffects(finishedWork) {\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 14:\n        case 15:\n          commitHookLayoutUnmountEffects(\n            finishedWork,\n            finishedWork.return,\n            Layout\n          );\n          recursivelyTraverseDisappearLayoutEffects(finishedWork);\n          break;\n        case 1:\n          safelyDetachRef(finishedWork, finishedWork.return);\n          var instance = finishedWork.stateNode;\n          \"function\" === typeof instance.componentWillUnmount &&\n            safelyCallComponentWillUnmount(\n              finishedWork,\n              finishedWork.return,\n              instance\n            );\n          recursivelyTraverseDisappearLayoutEffects(finishedWork);\n          break;\n        case 27:\n          runWithFiberInDEV(\n            finishedWork,\n            releaseSingletonInstance,\n            finishedWork.stateNode\n          );\n        case 26:\n        case 5:\n          safelyDetachRef(finishedWork, finishedWork.return);\n          recursivelyTraverseDisappearLayoutEffects(finishedWork);\n          break;\n        case 22:\n          null === finishedWork.memoizedState &&\n            recursivelyTraverseDisappearLayoutEffects(finishedWork);\n          break;\n        case 30:\n          recursivelyTraverseDisappearLayoutEffects(finishedWork);\n          break;\n        default:\n          recursivelyTraverseDisappearLayoutEffects(finishedWork);\n      }\n    }\n    function recursivelyTraverseDisappearLayoutEffects(parentFiber) {\n      for (parentFiber = parentFiber.child; null !== parentFiber; )\n        disappearLayoutEffects(parentFiber),\n          (parentFiber = parentFiber.sibling);\n    }\n    function reappearLayoutEffects(\n      finishedRoot,\n      current,\n      finishedWork,\n      includeWorkInProgressEffects\n    ) {\n      var flags = finishedWork.flags;\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 15:\n          recursivelyTraverseReappearLayoutEffects(\n            finishedRoot,\n            finishedWork,\n            includeWorkInProgressEffects\n          );\n          commitHookLayoutEffects(finishedWork, Layout);\n          break;\n        case 1:\n          recursivelyTraverseReappearLayoutEffects(\n            finishedRoot,\n            finishedWork,\n            includeWorkInProgressEffects\n          );\n          current = finishedWork.stateNode;\n          \"function\" === typeof current.componentDidMount &&\n            runWithFiberInDEV(\n              finishedWork,\n              callComponentDidMountInDEV,\n              finishedWork,\n              current\n            );\n          current = finishedWork.updateQueue;\n          if (null !== current) {\n            finishedRoot = finishedWork.stateNode;\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitHiddenCallbacks,\n                current,\n                finishedRoot\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          }\n          includeWorkInProgressEffects &&\n            flags & 64 &&\n            commitClassCallbacks(finishedWork);\n          safelyAttachRef(finishedWork, finishedWork.return);\n          break;\n        case 27:\n          commitHostSingletonAcquisition(finishedWork);\n        case 26:\n        case 5:\n          recursivelyTraverseReappearLayoutEffects(\n            finishedRoot,\n            finishedWork,\n            includeWorkInProgressEffects\n          );\n          includeWorkInProgressEffects &&\n            null === current &&\n            flags & 4 &&\n            commitHostMount(finishedWork);\n          safelyAttachRef(finishedWork, finishedWork.return);\n          break;\n        case 12:\n          if (includeWorkInProgressEffects && flags & 4) {\n            flags = pushNestedEffectDurations();\n            recursivelyTraverseReappearLayoutEffects(\n              finishedRoot,\n              finishedWork,\n              includeWorkInProgressEffects\n            );\n            includeWorkInProgressEffects = finishedWork.stateNode;\n            includeWorkInProgressEffects.effectDuration +=\n              bubbleNestedEffectDurations(flags);\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitProfiler,\n                finishedWork,\n                current,\n                commitStartTime,\n                includeWorkInProgressEffects.effectDuration\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          } else\n            recursivelyTraverseReappearLayoutEffects(\n              finishedRoot,\n              finishedWork,\n              includeWorkInProgressEffects\n            );\n          break;\n        case 13:\n          recursivelyTraverseReappearLayoutEffects(\n            finishedRoot,\n            finishedWork,\n            includeWorkInProgressEffects\n          );\n          includeWorkInProgressEffects &&\n            flags & 4 &&\n            commitSuspenseHydrationCallbacks(finishedRoot, finishedWork);\n          break;\n        case 22:\n          null === finishedWork.memoizedState &&\n            recursivelyTraverseReappearLayoutEffects(\n              finishedRoot,\n              finishedWork,\n              includeWorkInProgressEffects\n            );\n          safelyAttachRef(finishedWork, finishedWork.return);\n          break;\n        case 30:\n          break;\n        default:\n          recursivelyTraverseReappearLayoutEffects(\n            finishedRoot,\n            finishedWork,\n            includeWorkInProgressEffects\n          );\n      }\n    }\n    function recursivelyTraverseReappearLayoutEffects(\n      finishedRoot,\n      parentFiber,\n      includeWorkInProgressEffects\n    ) {\n      includeWorkInProgressEffects =\n        includeWorkInProgressEffects && 0 !== (parentFiber.subtreeFlags & 8772);\n      for (parentFiber = parentFiber.child; null !== parentFiber; )\n        reappearLayoutEffects(\n          finishedRoot,\n          parentFiber.alternate,\n          parentFiber,\n          includeWorkInProgressEffects\n        ),\n          (parentFiber = parentFiber.sibling);\n    }\n    function commitOffscreenPassiveMountEffects(current, finishedWork) {\n      var previousCache = null;\n      null !== current &&\n        null !== current.memoizedState &&\n        null !== current.memoizedState.cachePool &&\n        (previousCache = current.memoizedState.cachePool.pool);\n      current = null;\n      null !== finishedWork.memoizedState &&\n        null !== finishedWork.memoizedState.cachePool &&\n        (current = finishedWork.memoizedState.cachePool.pool);\n      current !== previousCache &&\n        (null != current && retainCache(current),\n        null != previousCache && releaseCache(previousCache));\n    }\n    function commitCachePassiveMountEffect(current, finishedWork) {\n      current = null;\n      null !== finishedWork.alternate &&\n        (current = finishedWork.alternate.memoizedState.cache);\n      finishedWork = finishedWork.memoizedState.cache;\n      finishedWork !== current &&\n        (retainCache(finishedWork), null != current && releaseCache(current));\n    }\n    function recursivelyTraversePassiveMountEffects(\n      root,\n      parentFiber,\n      committedLanes,\n      committedTransitions\n    ) {\n      if (parentFiber.subtreeFlags & 10256)\n        for (parentFiber = parentFiber.child; null !== parentFiber; )\n          commitPassiveMountOnFiber(\n            root,\n            parentFiber,\n            committedLanes,\n            committedTransitions\n          ),\n            (parentFiber = parentFiber.sibling);\n    }\n    function commitPassiveMountOnFiber(\n      finishedRoot,\n      finishedWork,\n      committedLanes,\n      committedTransitions\n    ) {\n      var flags = finishedWork.flags;\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 15:\n          recursivelyTraversePassiveMountEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions\n          );\n          flags & 2048 &&\n            commitHookPassiveMountEffects(finishedWork, Passive | HasEffect);\n          break;\n        case 1:\n          recursivelyTraversePassiveMountEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions\n          );\n          break;\n        case 3:\n          var prevEffectDuration = pushNestedEffectDurations();\n          recursivelyTraversePassiveMountEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions\n          );\n          flags & 2048 &&\n            ((committedLanes = null),\n            null !== finishedWork.alternate &&\n              (committedLanes = finishedWork.alternate.memoizedState.cache),\n            (finishedWork = finishedWork.memoizedState.cache),\n            finishedWork !== committedLanes &&\n              (retainCache(finishedWork),\n              null != committedLanes && releaseCache(committedLanes)));\n          finishedRoot.passiveEffectDuration +=\n            popNestedEffectDurations(prevEffectDuration);\n          break;\n        case 12:\n          if (flags & 2048) {\n            flags = pushNestedEffectDurations();\n            recursivelyTraversePassiveMountEffects(\n              finishedRoot,\n              finishedWork,\n              committedLanes,\n              committedTransitions\n            );\n            finishedRoot = finishedWork.stateNode;\n            finishedRoot.passiveEffectDuration +=\n              bubbleNestedEffectDurations(flags);\n            try {\n              runWithFiberInDEV(\n                finishedWork,\n                commitProfilerPostCommitImpl,\n                finishedWork,\n                finishedWork.alternate,\n                commitStartTime,\n                finishedRoot.passiveEffectDuration\n              );\n            } catch (error) {\n              captureCommitPhaseError(finishedWork, finishedWork.return, error);\n            }\n          } else\n            recursivelyTraversePassiveMountEffects(\n              finishedRoot,\n              finishedWork,\n              committedLanes,\n              committedTransitions\n            );\n          break;\n        case 13:\n          recursivelyTraversePassiveMountEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions\n          );\n          break;\n        case 23:\n          break;\n        case 22:\n          prevEffectDuration = finishedWork.stateNode;\n          var _current = finishedWork.alternate;\n          null !== finishedWork.memoizedState\n            ? prevEffectDuration._visibility & OffscreenPassiveEffectsConnected\n              ? recursivelyTraversePassiveMountEffects(\n                  finishedRoot,\n                  finishedWork,\n                  committedLanes,\n                  committedTransitions\n                )\n              : recursivelyTraverseAtomicPassiveEffects(\n                  finishedRoot,\n                  finishedWork\n                )\n            : prevEffectDuration._visibility & OffscreenPassiveEffectsConnected\n              ? recursivelyTraversePassiveMountEffects(\n                  finishedRoot,\n                  finishedWork,\n                  committedLanes,\n                  committedTransitions\n                )\n              : ((prevEffectDuration._visibility |=\n                  OffscreenPassiveEffectsConnected),\n                recursivelyTraverseReconnectPassiveEffects(\n                  finishedRoot,\n                  finishedWork,\n                  committedLanes,\n                  committedTransitions,\n                  0 !== (finishedWork.subtreeFlags & 10256)\n                ));\n          flags & 2048 &&\n            commitOffscreenPassiveMountEffects(_current, finishedWork);\n          break;\n        case 24:\n          recursivelyTraversePassiveMountEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions\n          );\n          flags & 2048 &&\n            commitCachePassiveMountEffect(finishedWork.alternate, finishedWork);\n          break;\n        default:\n          recursivelyTraversePassiveMountEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions\n          );\n      }\n    }\n    function recursivelyTraverseReconnectPassiveEffects(\n      finishedRoot,\n      parentFiber,\n      committedLanes,\n      committedTransitions,\n      includeWorkInProgressEffects\n    ) {\n      includeWorkInProgressEffects =\n        includeWorkInProgressEffects &&\n        0 !== (parentFiber.subtreeFlags & 10256);\n      for (parentFiber = parentFiber.child; null !== parentFiber; )\n        reconnectPassiveEffects(\n          finishedRoot,\n          parentFiber,\n          committedLanes,\n          committedTransitions,\n          includeWorkInProgressEffects\n        ),\n          (parentFiber = parentFiber.sibling);\n    }\n    function reconnectPassiveEffects(\n      finishedRoot,\n      finishedWork,\n      committedLanes,\n      committedTransitions,\n      includeWorkInProgressEffects\n    ) {\n      var flags = finishedWork.flags;\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 15:\n          recursivelyTraverseReconnectPassiveEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions,\n            includeWorkInProgressEffects\n          );\n          commitHookPassiveMountEffects(finishedWork, Passive);\n          break;\n        case 23:\n          break;\n        case 22:\n          var _instance2 = finishedWork.stateNode;\n          null !== finishedWork.memoizedState\n            ? _instance2._visibility & OffscreenPassiveEffectsConnected\n              ? recursivelyTraverseReconnectPassiveEffects(\n                  finishedRoot,\n                  finishedWork,\n                  committedLanes,\n                  committedTransitions,\n                  includeWorkInProgressEffects\n                )\n              : recursivelyTraverseAtomicPassiveEffects(\n                  finishedRoot,\n                  finishedWork\n                )\n            : ((_instance2._visibility |= OffscreenPassiveEffectsConnected),\n              recursivelyTraverseReconnectPassiveEffects(\n                finishedRoot,\n                finishedWork,\n                committedLanes,\n                committedTransitions,\n                includeWorkInProgressEffects\n              ));\n          includeWorkInProgressEffects &&\n            flags & 2048 &&\n            commitOffscreenPassiveMountEffects(\n              finishedWork.alternate,\n              finishedWork\n            );\n          break;\n        case 24:\n          recursivelyTraverseReconnectPassiveEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions,\n            includeWorkInProgressEffects\n          );\n          includeWorkInProgressEffects &&\n            flags & 2048 &&\n            commitCachePassiveMountEffect(finishedWork.alternate, finishedWork);\n          break;\n        default:\n          recursivelyTraverseReconnectPassiveEffects(\n            finishedRoot,\n            finishedWork,\n            committedLanes,\n            committedTransitions,\n            includeWorkInProgressEffects\n          );\n      }\n    }\n    function recursivelyTraverseAtomicPassiveEffects(\n      finishedRoot$jscomp$0,\n      parentFiber\n    ) {\n      if (parentFiber.subtreeFlags & 10256)\n        for (parentFiber = parentFiber.child; null !== parentFiber; ) {\n          var finishedRoot = finishedRoot$jscomp$0,\n            finishedWork = parentFiber,\n            flags = finishedWork.flags;\n          switch (finishedWork.tag) {\n            case 22:\n              recursivelyTraverseAtomicPassiveEffects(\n                finishedRoot,\n                finishedWork\n              );\n              flags & 2048 &&\n                commitOffscreenPassiveMountEffects(\n                  finishedWork.alternate,\n                  finishedWork\n                );\n              break;\n            case 24:\n              recursivelyTraverseAtomicPassiveEffects(\n                finishedRoot,\n                finishedWork\n              );\n              flags & 2048 &&\n                commitCachePassiveMountEffect(\n                  finishedWork.alternate,\n                  finishedWork\n                );\n              break;\n            default:\n              recursivelyTraverseAtomicPassiveEffects(\n                finishedRoot,\n                finishedWork\n              );\n          }\n          parentFiber = parentFiber.sibling;\n        }\n    }\n    function recursivelyAccumulateSuspenseyCommit(parentFiber) {\n      if (parentFiber.subtreeFlags & suspenseyCommitFlag)\n        for (parentFiber = parentFiber.child; null !== parentFiber; )\n          accumulateSuspenseyCommitOnFiber(parentFiber),\n            (parentFiber = parentFiber.sibling);\n    }\n    function accumulateSuspenseyCommitOnFiber(fiber) {\n      switch (fiber.tag) {\n        case 26:\n          recursivelyAccumulateSuspenseyCommit(fiber);\n          fiber.flags & suspenseyCommitFlag &&\n            null !== fiber.memoizedState &&\n            suspendResource(\n              currentHoistableRoot,\n              fiber.memoizedState,\n              fiber.memoizedProps\n            );\n          break;\n        case 5:\n          recursivelyAccumulateSuspenseyCommit(fiber);\n          break;\n        case 3:\n        case 4:\n          var previousHoistableRoot = currentHoistableRoot;\n          currentHoistableRoot = getHoistableRoot(\n            fiber.stateNode.containerInfo\n          );\n          recursivelyAccumulateSuspenseyCommit(fiber);\n          currentHoistableRoot = previousHoistableRoot;\n          break;\n        case 22:\n          null === fiber.memoizedState &&\n            ((previousHoistableRoot = fiber.alternate),\n            null !== previousHoistableRoot &&\n            null !== previousHoistableRoot.memoizedState\n              ? ((previousHoistableRoot = suspenseyCommitFlag),\n                (suspenseyCommitFlag = 16777216),\n                recursivelyAccumulateSuspenseyCommit(fiber),\n                (suspenseyCommitFlag = previousHoistableRoot))\n              : recursivelyAccumulateSuspenseyCommit(fiber));\n          break;\n        default:\n          recursivelyAccumulateSuspenseyCommit(fiber);\n      }\n    }\n    function detachAlternateSiblings(parentFiber) {\n      var previousFiber = parentFiber.alternate;\n      if (\n        null !== previousFiber &&\n        ((parentFiber = previousFiber.child), null !== parentFiber)\n      ) {\n        previousFiber.child = null;\n        do\n          (previousFiber = parentFiber.sibling),\n            (parentFiber.sibling = null),\n            (parentFiber = previousFiber);\n        while (null !== parentFiber);\n      }\n    }\n    function recursivelyTraversePassiveUnmountEffects(parentFiber) {\n      var deletions = parentFiber.deletions;\n      if (0 !== (parentFiber.flags & 16)) {\n        if (null !== deletions)\n          for (var i = 0; i < deletions.length; i++) {\n            var childToDelete = deletions[i];\n            nextEffect = childToDelete;\n            commitPassiveUnmountEffectsInsideOfDeletedTree_begin(\n              childToDelete,\n              parentFiber\n            );\n          }\n        detachAlternateSiblings(parentFiber);\n      }\n      if (parentFiber.subtreeFlags & 10256)\n        for (parentFiber = parentFiber.child; null !== parentFiber; )\n          commitPassiveUnmountOnFiber(parentFiber),\n            (parentFiber = parentFiber.sibling);\n    }\n    function commitPassiveUnmountOnFiber(finishedWork) {\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 15:\n          recursivelyTraversePassiveUnmountEffects(finishedWork);\n          finishedWork.flags & 2048 &&\n            commitHookPassiveUnmountEffects(\n              finishedWork,\n              finishedWork.return,\n              Passive | HasEffect\n            );\n          break;\n        case 3:\n          var prevEffectDuration = pushNestedEffectDurations();\n          recursivelyTraversePassiveUnmountEffects(finishedWork);\n          finishedWork.stateNode.passiveEffectDuration +=\n            popNestedEffectDurations(prevEffectDuration);\n          break;\n        case 12:\n          prevEffectDuration = pushNestedEffectDurations();\n          recursivelyTraversePassiveUnmountEffects(finishedWork);\n          finishedWork.stateNode.passiveEffectDuration +=\n            bubbleNestedEffectDurations(prevEffectDuration);\n          break;\n        case 22:\n          prevEffectDuration = finishedWork.stateNode;\n          null !== finishedWork.memoizedState &&\n          prevEffectDuration._visibility & OffscreenPassiveEffectsConnected &&\n          (null === finishedWork.return || 13 !== finishedWork.return.tag)\n            ? ((prevEffectDuration._visibility &=\n                ~OffscreenPassiveEffectsConnected),\n              recursivelyTraverseDisconnectPassiveEffects(finishedWork))\n            : recursivelyTraversePassiveUnmountEffects(finishedWork);\n          break;\n        default:\n          recursivelyTraversePassiveUnmountEffects(finishedWork);\n      }\n    }\n    function recursivelyTraverseDisconnectPassiveEffects(parentFiber) {\n      var deletions = parentFiber.deletions;\n      if (0 !== (parentFiber.flags & 16)) {\n        if (null !== deletions)\n          for (var i = 0; i < deletions.length; i++) {\n            var childToDelete = deletions[i];\n            nextEffect = childToDelete;\n            commitPassiveUnmountEffectsInsideOfDeletedTree_begin(\n              childToDelete,\n              parentFiber\n            );\n          }\n        detachAlternateSiblings(parentFiber);\n      }\n      for (parentFiber = parentFiber.child; null !== parentFiber; )\n        disconnectPassiveEffect(parentFiber),\n          (parentFiber = parentFiber.sibling);\n    }\n    function disconnectPassiveEffect(finishedWork) {\n      switch (finishedWork.tag) {\n        case 0:\n        case 11:\n        case 15:\n          commitHookPassiveUnmountEffects(\n            finishedWork,\n            finishedWork.return,\n            Passive\n          );\n          recursivelyTraverseDisconnectPassiveEffects(finishedWork);\n          break;\n        case 22:\n          var instance = finishedWork.stateNode;\n          instance._visibility & OffscreenPassiveEffectsConnected &&\n            ((instance._visibility &= ~OffscreenPassiveEffectsConnected),\n            recursivelyTraverseDisconnectPassiveEffects(finishedWork));\n          break;\n        default:\n          recursivelyTraverseDisconnectPassiveEffects(finishedWork);\n      }\n    }\n    function commitPassiveUnmountEffectsInsideOfDeletedTree_begin(\n      deletedSubtreeRoot,\n      nearestMountedAncestor\n    ) {\n      for (; null !== nextEffect; ) {\n        var fiber = nextEffect,\n          current = fiber;\n        switch (current.tag) {\n          case 0:\n          case 11:\n          case 15:\n            commitHookPassiveUnmountEffects(\n              current,\n              nearestMountedAncestor,\n              Passive\n            );\n            break;\n          case 23:\n          case 22:\n            null !== current.memoizedState &&\n              null !== current.memoizedState.cachePool &&\n              ((current = current.memoizedState.cachePool.pool),\n              null != current && retainCache(current));\n            break;\n          case 24:\n            releaseCache(current.memoizedState.cache);\n        }\n        current = fiber.child;\n        if (null !== current) (current.return = fiber), (nextEffect = current);\n        else\n          a: for (fiber = deletedSubtreeRoot; null !== nextEffect; ) {\n            current = nextEffect;\n            var sibling = current.sibling,\n              returnFiber = current.return;\n            detachFiberAfterEffects(current);\n            if (current === fiber) {\n              nextEffect = null;\n              break a;\n            }\n            if (null !== sibling) {\n              sibling.return = returnFiber;\n              nextEffect = sibling;\n              break a;\n            }\n            nextEffect = returnFiber;\n          }\n      }\n    }\n    function onCommitRoot() {\n      commitHooks.forEach(function (commitHook) {\n        return commitHook();\n      });\n    }\n    function isConcurrentActEnvironment() {\n      var isReactActEnvironmentGlobal =\n        \"undefined\" !== typeof IS_REACT_ACT_ENVIRONMENT\n          ? IS_REACT_ACT_ENVIRONMENT\n          : void 0;\n      isReactActEnvironmentGlobal ||\n        null === ReactSharedInternals.actQueue ||\n        console.error(\n          \"The current testing environment is not configured to support act(...)\"\n        );\n      return isReactActEnvironmentGlobal;\n    }\n    function requestUpdateLane(fiber) {\n      if (\n        (executionContext & RenderContext) !== NoContext &&\n        0 !== workInProgressRootRenderLanes\n      )\n        return workInProgressRootRenderLanes & -workInProgressRootRenderLanes;\n      var transition = ReactSharedInternals.T;\n      return null !== transition\n        ? (transition._updatedFibers || (transition._updatedFibers = new Set()),\n          transition._updatedFibers.add(fiber),\n          (fiber = currentEntangledLane),\n          0 !== fiber ? fiber : requestTransitionLane())\n        : resolveUpdatePriority();\n    }\n    function requestDeferredLane() {\n      0 === workInProgressDeferredLane &&\n        (workInProgressDeferredLane =\n          0 === (workInProgressRootRenderLanes & 536870912) || isHydrating\n            ? claimNextTransitionLane()\n            : 536870912);\n      var suspenseHandler = suspenseHandlerStackCursor.current;\n      null !== suspenseHandler && (suspenseHandler.flags |= 32);\n      return workInProgressDeferredLane;\n    }\n    function scheduleUpdateOnFiber(root, fiber, lane) {\n      isRunningInsertionEffect &&\n        console.error(\"useInsertionEffect must not schedule updates.\");\n      isFlushingPassiveEffects && (didScheduleUpdateDuringPassiveEffects = !0);\n      if (\n        (root === workInProgressRoot &&\n          (workInProgressSuspendedReason === SuspendedOnData ||\n            workInProgressSuspendedReason === SuspendedOnAction)) ||\n        null !== root.cancelPendingCommit\n      )\n        prepareFreshStack(root, 0),\n          markRootSuspended(\n            root,\n            workInProgressRootRenderLanes,\n            workInProgressDeferredLane,\n            !1\n          );\n      markRootUpdated$1(root, lane);\n      if (\n        0 !== (executionContext & RenderContext) &&\n        root === workInProgressRoot\n      ) {\n        if (isRendering)\n          switch (fiber.tag) {\n            case 0:\n            case 11:\n            case 15:\n              root =\n                (workInProgress && getComponentNameFromFiber(workInProgress)) ||\n                \"Unknown\";\n              didWarnAboutUpdateInRenderForAnotherComponent.has(root) ||\n                (didWarnAboutUpdateInRenderForAnotherComponent.add(root),\n                (fiber = getComponentNameFromFiber(fiber) || \"Unknown\"),\n                console.error(\n                  \"Cannot update a component (`%s`) while rendering a different component (`%s`). To locate the bad setState() call inside `%s`, follow the stack trace as described in https://react.dev/link/setstate-in-render\",\n                  fiber,\n                  root,\n                  root\n                ));\n              break;\n            case 1:\n              didWarnAboutUpdateInRender ||\n                (console.error(\n                  \"Cannot update during an existing state transition (such as within `render`). Render methods should be a pure function of props and state.\"\n                ),\n                (didWarnAboutUpdateInRender = !0));\n          }\n      } else\n        isDevToolsPresent && addFiberToLanesMap(root, fiber, lane),\n          warnIfUpdatesNotWrappedWithActDEV(fiber),\n          root === workInProgressRoot &&\n            ((executionContext & RenderContext) === NoContext &&\n              (workInProgressRootInterleavedUpdatedLanes |= lane),\n            workInProgressRootExitStatus === RootSuspendedWithDelay &&\n              markRootSuspended(\n                root,\n                workInProgressRootRenderLanes,\n                workInProgressDeferredLane,\n                !1\n              )),\n          ensureRootIsScheduled(root);\n    }\n    function performWorkOnRoot(root, lanes, forceSync) {\n      if ((executionContext & (RenderContext | CommitContext)) !== NoContext)\n        throw Error(\"Should not already be working.\");\n      var shouldTimeSlice =\n          (!forceSync &&\n            0 === (lanes & 124) &&\n            0 === (lanes & root.expiredLanes)) ||\n          checkIfRootIsPrerendering(root, lanes),\n        exitStatus = shouldTimeSlice\n          ? renderRootConcurrent(root, lanes)\n          : renderRootSync(root, lanes, !0),\n        renderWasConcurrent = shouldTimeSlice;\n      do {\n        if (exitStatus === RootInProgress) {\n          workInProgressRootIsPrerendering &&\n            !shouldTimeSlice &&\n            markRootSuspended(root, lanes, 0, !1);\n          break;\n        } else {\n          forceSync = root.current.alternate;\n          if (\n            renderWasConcurrent &&\n            !isRenderConsistentWithExternalStores(forceSync)\n          ) {\n            exitStatus = renderRootSync(root, lanes, !1);\n            renderWasConcurrent = !1;\n            continue;\n          }\n          if (exitStatus === RootErrored) {\n            renderWasConcurrent = lanes;\n            if (root.errorRecoveryDisabledLanes & renderWasConcurrent)\n              var errorRetryLanes = 0;\n            else\n              (errorRetryLanes = root.pendingLanes & -536870913),\n                (errorRetryLanes =\n                  0 !== errorRetryLanes\n                    ? errorRetryLanes\n                    : errorRetryLanes & 536870912\n                      ? 536870912\n                      : 0);\n            if (0 !== errorRetryLanes) {\n              lanes = errorRetryLanes;\n              a: {\n                exitStatus = root;\n                var errorRetryLanes$jscomp$0 = errorRetryLanes;\n                errorRetryLanes = workInProgressRootConcurrentErrors;\n                var wasRootDehydrated =\n                  exitStatus.current.memoizedState.isDehydrated;\n                wasRootDehydrated &&\n                  (prepareFreshStack(\n                    exitStatus,\n                    errorRetryLanes$jscomp$0\n                  ).flags |= 256);\n                errorRetryLanes$jscomp$0 = renderRootSync(\n                  exitStatus,\n                  errorRetryLanes$jscomp$0,\n                  !1\n                );\n                if (errorRetryLanes$jscomp$0 !== RootErrored) {\n                  if (\n                    workInProgressRootDidAttachPingListener &&\n                    !wasRootDehydrated\n                  ) {\n                    exitStatus.errorRecoveryDisabledLanes |=\n                      renderWasConcurrent;\n                    workInProgressRootInterleavedUpdatedLanes |=\n                      renderWasConcurrent;\n                    exitStatus = RootSuspendedWithDelay;\n                    break a;\n                  }\n                  exitStatus = workInProgressRootRecoverableErrors;\n                  workInProgressRootRecoverableErrors = errorRetryLanes;\n                  null !== exitStatus &&\n                    (null === workInProgressRootRecoverableErrors\n                      ? (workInProgressRootRecoverableErrors = exitStatus)\n                      : workInProgressRootRecoverableErrors.push.apply(\n                          workInProgressRootRecoverableErrors,\n                          exitStatus\n                        ));\n                }\n                exitStatus = errorRetryLanes$jscomp$0;\n              }\n              renderWasConcurrent = !1;\n              if (exitStatus !== RootErrored) continue;\n            }\n          }\n          if (exitStatus === RootFatalErrored) {\n            prepareFreshStack(root, 0);\n            markRootSuspended(root, lanes, 0, !0);\n            break;\n          }\n          a: {\n            shouldTimeSlice = root;\n            switch (exitStatus) {\n              case RootInProgress:\n              case RootFatalErrored:\n                throw Error(\"Root did not complete. This is a bug in React.\");\n              case RootSuspendedWithDelay:\n                if ((lanes & 4194048) !== lanes) break;\n              case RootSuspendedAtTheShell:\n                markRootSuspended(\n                  shouldTimeSlice,\n                  lanes,\n                  workInProgressDeferredLane,\n                  !workInProgressRootDidSkipSuspendedSiblings\n                );\n                break a;\n              case RootErrored:\n                workInProgressRootRecoverableErrors = null;\n                break;\n              case RootSuspended:\n              case RootCompleted:\n                break;\n              default:\n                throw Error(\"Unknown root exit status.\");\n            }\n            if (null !== ReactSharedInternals.actQueue)\n              commitRoot(\n                shouldTimeSlice,\n                forceSync,\n                lanes,\n                workInProgressRootRecoverableErrors,\n                workInProgressTransitions,\n                workInProgressRootDidIncludeRecursiveRenderUpdate,\n                workInProgressDeferredLane,\n                workInProgressRootInterleavedUpdatedLanes,\n                workInProgressSuspendedRetryLanes\n              );\n            else {\n              if (\n                (lanes & 62914560) === lanes &&\n                ((renderWasConcurrent =\n                  globalMostRecentFallbackTime +\n                  FALLBACK_THROTTLE_MS -\n                  now$1()),\n                10 < renderWasConcurrent)\n              ) {\n                markRootSuspended(\n                  shouldTimeSlice,\n                  lanes,\n                  workInProgressDeferredLane,\n                  !workInProgressRootDidSkipSuspendedSiblings\n                );\n                if (0 !== getNextLanes(shouldTimeSlice, 0, !0)) break a;\n                shouldTimeSlice.timeoutHandle = scheduleTimeout(\n                  commitRootWhenReady.bind(\n                    null,\n                    shouldTimeSlice,\n                    forceSync,\n                    workInProgressRootRecoverableErrors,\n                    workInProgressTransitions,\n                    workInProgressRootDidIncludeRecursiveRenderUpdate,\n                    lanes,\n                    workInProgressDeferredLane,\n                    workInProgressRootInterleavedUpdatedLanes,\n                    workInProgressSuspendedRetryLanes,\n                    workInProgressRootDidSkipSuspendedSiblings,\n                    exitStatus,\n                    THROTTLED_COMMIT,\n                    renderStartTime,\n                    0\n                  ),\n                  renderWasConcurrent\n                );\n                break a;\n              }\n              commitRootWhenReady(\n                shouldTimeSlice,\n                forceSync,\n                workInProgressRootRecoverableErrors,\n                workInProgressTransitions,\n                workInProgressRootDidIncludeRecursiveRenderUpdate,\n                lanes,\n                workInProgressDeferredLane,\n                workInProgressRootInterleavedUpdatedLanes,\n                workInProgressSuspendedRetryLanes,\n                workInProgressRootDidSkipSuspendedSiblings,\n                exitStatus,\n                IMMEDIATE_COMMIT,\n                renderStartTime,\n                0\n              );\n            }\n          }\n        }\n        break;\n      } while (1);\n      ensureRootIsScheduled(root);\n    }\n    function commitRootWhenReady(\n      root,\n      finishedWork,\n      recoverableErrors,\n      transitions,\n      didIncludeRenderPhaseUpdate,\n      lanes,\n      spawnedLane,\n      updatedLanes,\n      suspendedRetryLanes,\n      didSkipSuspendedSiblings,\n      exitStatus,\n      suspendedCommitReason,\n      completedRenderStartTime,\n      completedRenderEndTime\n    ) {\n      root.timeoutHandle = noTimeout;\n      suspendedCommitReason = finishedWork.subtreeFlags;\n      if (\n        suspendedCommitReason & 8192 ||\n        16785408 === (suspendedCommitReason & 16785408)\n      )\n        if (\n          ((suspendedState = { stylesheets: null, count: 0, unsuspend: noop }),\n          accumulateSuspenseyCommitOnFiber(finishedWork),\n          (suspendedCommitReason = waitForCommitToBeReady()),\n          null !== suspendedCommitReason)\n        ) {\n          root.cancelPendingCommit = suspendedCommitReason(\n            commitRoot.bind(\n              null,\n              root,\n              finishedWork,\n              lanes,\n              recoverableErrors,\n              transitions,\n              didIncludeRenderPhaseUpdate,\n              spawnedLane,\n              updatedLanes,\n              suspendedRetryLanes,\n              exitStatus,\n              SUSPENDED_COMMIT,\n              completedRenderStartTime,\n              completedRenderEndTime\n            )\n          );\n          markRootSuspended(\n            root,\n            lanes,\n            spawnedLane,\n            !didSkipSuspendedSiblings\n          );\n          return;\n        }\n      commitRoot(\n        root,\n        finishedWork,\n        lanes,\n        recoverableErrors,\n        transitions,\n        didIncludeRenderPhaseUpdate,\n        spawnedLane,\n        updatedLanes,\n        suspendedRetryLanes\n      );\n    }\n    function isRenderConsistentWithExternalStores(finishedWork) {\n      for (var node = finishedWork; ; ) {\n        var tag = node.tag;\n        if (\n          (0 === tag || 11 === tag || 15 === tag) &&\n          node.flags & 16384 &&\n          ((tag = node.updateQueue),\n          null !== tag && ((tag = tag.stores), null !== tag))\n        )\n          for (var i = 0; i < tag.length; i++) {\n            var check = tag[i],\n              getSnapshot = check.getSnapshot;\n            check = check.value;\n            try {\n              if (!objectIs(getSnapshot(), check)) return !1;\n            } catch (error) {\n              return !1;\n            }\n          }\n        tag = node.child;\n        if (node.subtreeFlags & 16384 && null !== tag)\n          (tag.return = node), (node = tag);\n        else {\n          if (node === finishedWork) break;\n          for (; null === node.sibling; ) {\n            if (null === node.return || node.return === finishedWork) return !0;\n            node = node.return;\n          }\n          node.sibling.return = node.return;\n          node = node.sibling;\n        }\n      }\n      return !0;\n    }\n    function markRootSuspended(\n      root,\n      suspendedLanes,\n      spawnedLane,\n      didAttemptEntireTree\n    ) {\n      suspendedLanes &= ~workInProgressRootPingedLanes;\n      suspendedLanes &= ~workInProgressRootInterleavedUpdatedLanes;\n      root.suspendedLanes |= suspendedLanes;\n      root.pingedLanes &= ~suspendedLanes;\n      didAttemptEntireTree && (root.warmLanes |= suspendedLanes);\n      didAttemptEntireTree = root.expirationTimes;\n      for (var lanes = suspendedLanes; 0 < lanes; ) {\n        var index = 31 - clz32(lanes),\n          lane = 1 << index;\n        didAttemptEntireTree[index] = -1;\n        lanes &= ~lane;\n      }\n      0 !== spawnedLane &&\n        markSpawnedDeferredLane(root, spawnedLane, suspendedLanes);\n    }\n    function flushSyncWork$1() {\n      return (executionContext & (RenderContext | CommitContext)) === NoContext\n        ? (flushSyncWorkAcrossRoots_impl(0, !1), !1)\n        : !0;\n    }\n    function resetWorkInProgressStack() {\n      if (null !== workInProgress) {\n        if (workInProgressSuspendedReason === NotSuspended)\n          var interruptedWork = workInProgress.return;\n        else\n          (interruptedWork = workInProgress),\n            resetContextDependencies(),\n            resetHooksOnUnwind(interruptedWork),\n            (thenableState = null),\n            (thenableIndexCounter = 0),\n            (interruptedWork = workInProgress);\n        for (; null !== interruptedWork; )\n          unwindInterruptedWork(interruptedWork.alternate, interruptedWork),\n            (interruptedWork = interruptedWork.return);\n        workInProgress = null;\n      }\n    }\n    function prepareFreshStack(root, lanes) {\n      var timeoutHandle = root.timeoutHandle;\n      timeoutHandle !== noTimeout &&\n        ((root.timeoutHandle = noTimeout), cancelTimeout(timeoutHandle));\n      timeoutHandle = root.cancelPendingCommit;\n      null !== timeoutHandle &&\n        ((root.cancelPendingCommit = null), timeoutHandle());\n      resetWorkInProgressStack();\n      workInProgressRoot = root;\n      workInProgress = timeoutHandle = createWorkInProgress(root.current, null);\n      workInProgressRootRenderLanes = lanes;\n      workInProgressSuspendedReason = NotSuspended;\n      workInProgressThrownValue = null;\n      workInProgressRootDidSkipSuspendedSiblings = !1;\n      workInProgressRootIsPrerendering = checkIfRootIsPrerendering(root, lanes);\n      workInProgressRootDidAttachPingListener = !1;\n      workInProgressRootExitStatus = RootInProgress;\n      workInProgressSuspendedRetryLanes =\n        workInProgressDeferredLane =\n        workInProgressRootPingedLanes =\n        workInProgressRootInterleavedUpdatedLanes =\n        workInProgressRootSkippedLanes =\n          0;\n      workInProgressRootRecoverableErrors = workInProgressRootConcurrentErrors =\n        null;\n      workInProgressRootDidIncludeRecursiveRenderUpdate = !1;\n      0 !== (lanes & 8) && (lanes |= lanes & 32);\n      var allEntangledLanes = root.entangledLanes;\n      if (0 !== allEntangledLanes)\n        for (\n          root = root.entanglements, allEntangledLanes &= lanes;\n          0 < allEntangledLanes;\n\n        ) {\n          var index = 31 - clz32(allEntangledLanes),\n            lane = 1 << index;\n          lanes |= root[index];\n          allEntangledLanes &= ~lane;\n        }\n      entangledRenderLanes = lanes;\n      finishQueueingConcurrentUpdates();\n      lanes = getCurrentTime();\n      1e3 < lanes - lastResetTime &&\n        ((ReactSharedInternals.recentlyCreatedOwnerStacks = 0),\n        (lastResetTime = lanes));\n      ReactStrictModeWarnings.discardPendingWarnings();\n      return timeoutHandle;\n    }\n    function handleThrow(root, thrownValue) {\n      currentlyRenderingFiber = null;\n      ReactSharedInternals.H = ContextOnlyDispatcher;\n      ReactSharedInternals.getCurrentStack = null;\n      isRendering = !1;\n      current = null;\n      thrownValue === SuspenseException ||\n      thrownValue === SuspenseActionException\n        ? ((thrownValue = getSuspendedThenable()),\n          (workInProgressSuspendedReason = SuspendedOnImmediate))\n        : thrownValue === SuspenseyCommitException\n          ? ((thrownValue = getSuspendedThenable()),\n            (workInProgressSuspendedReason = SuspendedOnInstance))\n          : (workInProgressSuspendedReason =\n              thrownValue === SelectiveHydrationException\n                ? SuspendedOnHydration\n                : null !== thrownValue &&\n                    \"object\" === typeof thrownValue &&\n                    \"function\" === typeof thrownValue.then\n                  ? SuspendedOnDeprecatedThrowPromise\n                  : SuspendedOnError);\n      workInProgressThrownValue = thrownValue;\n      var erroredWork = workInProgress;\n      if (null === erroredWork)\n        (workInProgressRootExitStatus = RootFatalErrored),\n          logUncaughtError(\n            root,\n            createCapturedValueAtFiber(thrownValue, root.current)\n          );\n      else\n        switch (\n          (erroredWork.mode & ProfileMode &&\n            stopProfilerTimerIfRunningAndRecordDuration(erroredWork),\n          markComponentRenderStopped(),\n          workInProgressSuspendedReason)\n        ) {\n          case SuspendedOnError:\n            null !== injectedProfilingHooks &&\n              \"function\" ===\n                typeof injectedProfilingHooks.markComponentErrored &&\n              injectedProfilingHooks.markComponentErrored(\n                erroredWork,\n                thrownValue,\n                workInProgressRootRenderLanes\n              );\n            break;\n          case SuspendedOnData:\n          case SuspendedOnAction:\n          case SuspendedOnImmediate:\n          case SuspendedOnDeprecatedThrowPromise:\n          case SuspendedAndReadyToContinue:\n            null !== injectedProfilingHooks &&\n              \"function\" ===\n                typeof injectedProfilingHooks.markComponentSuspended &&\n              injectedProfilingHooks.markComponentSuspended(\n                erroredWork,\n                thrownValue,\n                workInProgressRootRenderLanes\n              );\n        }\n    }\n    function pushDispatcher() {\n      var prevDispatcher = ReactSharedInternals.H;\n      ReactSharedInternals.H = ContextOnlyDispatcher;\n      return null === prevDispatcher ? ContextOnlyDispatcher : prevDispatcher;\n    }\n    function pushAsyncDispatcher() {\n      var prevAsyncDispatcher = ReactSharedInternals.A;\n      ReactSharedInternals.A = DefaultAsyncDispatcher;\n      return prevAsyncDispatcher;\n    }\n    function renderDidSuspendDelayIfPossible() {\n      workInProgressRootExitStatus = RootSuspendedWithDelay;\n      workInProgressRootDidSkipSuspendedSiblings ||\n        ((workInProgressRootRenderLanes & 4194048) !==\n          workInProgressRootRenderLanes &&\n          null !== suspenseHandlerStackCursor.current) ||\n        (workInProgressRootIsPrerendering = !0);\n      (0 === (workInProgressRootSkippedLanes & 134217727) &&\n        0 === (workInProgressRootInterleavedUpdatedLanes & 134217727)) ||\n        null === workInProgressRoot ||\n        markRootSuspended(\n          workInProgressRoot,\n          workInProgressRootRenderLanes,\n          workInProgressDeferredLane,\n          !1\n        );\n    }\n    function renderRootSync(root, lanes, shouldYieldForPrerendering) {\n      var prevExecutionContext = executionContext;\n      executionContext |= RenderContext;\n      var prevDispatcher = pushDispatcher(),\n        prevAsyncDispatcher = pushAsyncDispatcher();\n      if (\n        workInProgressRoot !== root ||\n        workInProgressRootRenderLanes !== lanes\n      ) {\n        if (isDevToolsPresent) {\n          var memoizedUpdaters = root.memoizedUpdaters;\n          0 < memoizedUpdaters.size &&\n            (restorePendingUpdaters(root, workInProgressRootRenderLanes),\n            memoizedUpdaters.clear());\n          movePendingFibersToMemoized(root, lanes);\n        }\n        workInProgressTransitions = null;\n        prepareFreshStack(root, lanes);\n      }\n      markRenderStarted(lanes);\n      lanes = !1;\n      memoizedUpdaters = workInProgressRootExitStatus;\n      a: do\n        try {\n          if (\n            workInProgressSuspendedReason !== NotSuspended &&\n            null !== workInProgress\n          ) {\n            var unitOfWork = workInProgress,\n              thrownValue = workInProgressThrownValue;\n            switch (workInProgressSuspendedReason) {\n              case SuspendedOnHydration:\n                resetWorkInProgressStack();\n                memoizedUpdaters = RootSuspendedAtTheShell;\n                break a;\n              case SuspendedOnImmediate:\n              case SuspendedOnData:\n              case SuspendedOnAction:\n              case SuspendedOnDeprecatedThrowPromise:\n                null === suspenseHandlerStackCursor.current && (lanes = !0);\n                var reason = workInProgressSuspendedReason;\n                workInProgressSuspendedReason = NotSuspended;\n                workInProgressThrownValue = null;\n                throwAndUnwindWorkLoop(root, unitOfWork, thrownValue, reason);\n                if (\n                  shouldYieldForPrerendering &&\n                  workInProgressRootIsPrerendering\n                ) {\n                  memoizedUpdaters = RootInProgress;\n                  break a;\n                }\n                break;\n              default:\n                (reason = workInProgressSuspendedReason),\n                  (workInProgressSuspendedReason = NotSuspended),\n                  (workInProgressThrownValue = null),\n                  throwAndUnwindWorkLoop(root, unitOfWork, thrownValue, reason);\n            }\n          }\n          workLoopSync();\n          memoizedUpdaters = workInProgressRootExitStatus;\n          break;\n        } catch (thrownValue$8) {\n          handleThrow(root, thrownValue$8);\n        }\n      while (1);\n      lanes && root.shellSuspendCounter++;\n      resetContextDependencies();\n      executionContext = prevExecutionContext;\n      ReactSharedInternals.H = prevDispatcher;\n      ReactSharedInternals.A = prevAsyncDispatcher;\n      markRenderStopped();\n      null === workInProgress &&\n        ((workInProgressRoot = null),\n        (workInProgressRootRenderLanes = 0),\n        finishQueueingConcurrentUpdates());\n      return memoizedUpdaters;\n    }\n    function workLoopSync() {\n      for (; null !== workInProgress; ) performUnitOfWork(workInProgress);\n    }\n    function renderRootConcurrent(root, lanes) {\n      var prevExecutionContext = executionContext;\n      executionContext |= RenderContext;\n      var prevDispatcher = pushDispatcher(),\n        prevAsyncDispatcher = pushAsyncDispatcher();\n      if (\n        workInProgressRoot !== root ||\n        workInProgressRootRenderLanes !== lanes\n      ) {\n        if (isDevToolsPresent) {\n          var memoizedUpdaters = root.memoizedUpdaters;\n          0 < memoizedUpdaters.size &&\n            (restorePendingUpdaters(root, workInProgressRootRenderLanes),\n            memoizedUpdaters.clear());\n          movePendingFibersToMemoized(root, lanes);\n        }\n        workInProgressTransitions = null;\n        workInProgressRootRenderTargetTime = now$1() + RENDER_TIMEOUT_MS;\n        prepareFreshStack(root, lanes);\n      } else\n        workInProgressRootIsPrerendering = checkIfRootIsPrerendering(\n          root,\n          lanes\n        );\n      markRenderStarted(lanes);\n      a: do\n        try {\n          if (\n            workInProgressSuspendedReason !== NotSuspended &&\n            null !== workInProgress\n          )\n            b: switch (\n              ((lanes = workInProgress),\n              (memoizedUpdaters = workInProgressThrownValue),\n              workInProgressSuspendedReason)\n            ) {\n              case SuspendedOnError:\n                workInProgressSuspendedReason = NotSuspended;\n                workInProgressThrownValue = null;\n                throwAndUnwindWorkLoop(\n                  root,\n                  lanes,\n                  memoizedUpdaters,\n                  SuspendedOnError\n                );\n                break;\n              case SuspendedOnData:\n              case SuspendedOnAction:\n                if (isThenableResolved(memoizedUpdaters)) {\n                  workInProgressSuspendedReason = NotSuspended;\n                  workInProgressThrownValue = null;\n                  replaySuspendedUnitOfWork(lanes);\n                  break;\n                }\n                lanes = function () {\n                  (workInProgressSuspendedReason !== SuspendedOnData &&\n                    workInProgressSuspendedReason !== SuspendedOnAction) ||\n                    workInProgressRoot !== root ||\n                    (workInProgressSuspendedReason =\n                      SuspendedAndReadyToContinue);\n                  ensureRootIsScheduled(root);\n                };\n                memoizedUpdaters.then(lanes, lanes);\n                break a;\n              case SuspendedOnImmediate:\n                workInProgressSuspendedReason = SuspendedAndReadyToContinue;\n                break a;\n              case SuspendedOnInstance:\n                workInProgressSuspendedReason =\n                  SuspendedOnInstanceAndReadyToContinue;\n                break a;\n              case SuspendedAndReadyToContinue:\n                isThenableResolved(memoizedUpdaters)\n                  ? ((workInProgressSuspendedReason = NotSuspended),\n                    (workInProgressThrownValue = null),\n                    replaySuspendedUnitOfWork(lanes))\n                  : ((workInProgressSuspendedReason = NotSuspended),\n                    (workInProgressThrownValue = null),\n                    throwAndUnwindWorkLoop(\n                      root,\n                      lanes,\n                      memoizedUpdaters,\n                      SuspendedAndReadyToContinue\n                    ));\n                break;\n              case SuspendedOnInstanceAndReadyToContinue:\n                var resource = null;\n                switch (workInProgress.tag) {\n                  case 26:\n                    resource = workInProgress.memoizedState;\n                  case 5:\n                  case 27:\n                    var hostFiber = workInProgress;\n                    if (resource ? preloadResource(resource) : 1) {\n                      workInProgressSuspendedReason = NotSuspended;\n                      workInProgressThrownValue = null;\n                      var sibling = hostFiber.sibling;\n                      if (null !== sibling) workInProgress = sibling;\n                      else {\n                        var returnFiber = hostFiber.return;\n                        null !== returnFiber\n                          ? ((workInProgress = returnFiber),\n                            completeUnitOfWork(returnFiber))\n                          : (workInProgress = null);\n                      }\n                      break b;\n                    }\n                    break;\n                  default:\n                    console.error(\n                      \"Unexpected type of fiber triggered a suspensey commit. This is a bug in React.\"\n                    );\n                }\n                workInProgressSuspendedReason = NotSuspended;\n                workInProgressThrownValue = null;\n                throwAndUnwindWorkLoop(\n                  root,\n                  lanes,\n                  memoizedUpdaters,\n                  SuspendedOnInstanceAndReadyToContinue\n                );\n                break;\n              case SuspendedOnDeprecatedThrowPromise:\n                workInProgressSuspendedReason = NotSuspended;\n                workInProgressThrownValue = null;\n                throwAndUnwindWorkLoop(\n                  root,\n                  lanes,\n                  memoizedUpdaters,\n                  SuspendedOnDeprecatedThrowPromise\n                );\n                break;\n              case SuspendedOnHydration:\n                resetWorkInProgressStack();\n                workInProgressRootExitStatus = RootSuspendedAtTheShell;\n                break a;\n              default:\n                throw Error(\n                  \"Unexpected SuspendedReason. This is a bug in React.\"\n                );\n            }\n          null !== ReactSharedInternals.actQueue\n            ? workLoopSync()\n            : workLoopConcurrentByScheduler();\n          break;\n        } catch (thrownValue$9) {\n          handleThrow(root, thrownValue$9);\n        }\n      while (1);\n      resetContextDependencies();\n      ReactSharedInternals.H = prevDispatcher;\n      ReactSharedInternals.A = prevAsyncDispatcher;\n      executionContext = prevExecutionContext;\n      if (null !== workInProgress)\n        return (\n          null !== injectedProfilingHooks &&\n            \"function\" === typeof injectedProfilingHooks.markRenderYielded &&\n            injectedProfilingHooks.markRenderYielded(),\n          RootInProgress\n        );\n      markRenderStopped();\n      workInProgressRoot = null;\n      workInProgressRootRenderLanes = 0;\n      finishQueueingConcurrentUpdates();\n      return workInProgressRootExitStatus;\n    }\n    function workLoopConcurrentByScheduler() {\n      for (; null !== workInProgress && !shouldYield(); )\n        performUnitOfWork(workInProgress);\n    }\n    function performUnitOfWork(unitOfWork) {\n      var current = unitOfWork.alternate;\n      (unitOfWork.mode & ProfileMode) !== NoMode\n        ? (startProfilerTimer(unitOfWork),\n          (current = runWithFiberInDEV(\n            unitOfWork,\n            beginWork,\n            current,\n            unitOfWork,\n            entangledRenderLanes\n          )),\n          stopProfilerTimerIfRunningAndRecordDuration(unitOfWork))\n        : (current = runWithFiberInDEV(\n            unitOfWork,\n            beginWork,\n            current,\n            unitOfWork,\n            entangledRenderLanes\n          ));\n      unitOfWork.memoizedProps = unitOfWork.pendingProps;\n      null === current\n        ? completeUnitOfWork(unitOfWork)\n        : (workInProgress = current);\n    }\n    function replaySuspendedUnitOfWork(unitOfWork) {\n      var next = runWithFiberInDEV(unitOfWork, replayBeginWork, unitOfWork);\n      unitOfWork.memoizedProps = unitOfWork.pendingProps;\n      null === next ? completeUnitOfWork(unitOfWork) : (workInProgress = next);\n    }\n    function replayBeginWork(unitOfWork) {\n      var current = unitOfWork.alternate,\n        isProfilingMode = (unitOfWork.mode & ProfileMode) !== NoMode;\n      isProfilingMode && startProfilerTimer(unitOfWork);\n      switch (unitOfWork.tag) {\n        case 15:\n        case 0:\n          current = replayFunctionComponent(\n            current,\n            unitOfWork,\n            unitOfWork.pendingProps,\n            unitOfWork.type,\n            void 0,\n            workInProgressRootRenderLanes\n          );\n          break;\n        case 11:\n          current = replayFunctionComponent(\n            current,\n            unitOfWork,\n            unitOfWork.pendingProps,\n            unitOfWork.type.render,\n            unitOfWork.ref,\n            workInProgressRootRenderLanes\n          );\n          break;\n        case 5:\n          resetHooksOnUnwind(unitOfWork);\n        default:\n          unwindInterruptedWork(current, unitOfWork),\n            (unitOfWork = workInProgress =\n              resetWorkInProgress(unitOfWork, entangledRenderLanes)),\n            (current = beginWork(current, unitOfWork, entangledRenderLanes));\n      }\n      isProfilingMode &&\n        stopProfilerTimerIfRunningAndRecordDuration(unitOfWork);\n      return current;\n    }\n    function throwAndUnwindWorkLoop(\n      root,\n      unitOfWork,\n      thrownValue,\n      suspendedReason\n    ) {\n      resetContextDependencies();\n      resetHooksOnUnwind(unitOfWork);\n      thenableState = null;\n      thenableIndexCounter = 0;\n      var returnFiber = unitOfWork.return;\n      try {\n        if (\n          throwException(\n            root,\n            returnFiber,\n            unitOfWork,\n            thrownValue,\n            workInProgressRootRenderLanes\n          )\n        ) {\n          workInProgressRootExitStatus = RootFatalErrored;\n          logUncaughtError(\n            root,\n            createCapturedValueAtFiber(thrownValue, root.current)\n          );\n          workInProgress = null;\n          return;\n        }\n      } catch (error) {\n        if (null !== returnFiber) throw ((workInProgress = returnFiber), error);\n        workInProgressRootExitStatus = RootFatalErrored;\n        logUncaughtError(\n          root,\n          createCapturedValueAtFiber(thrownValue, root.current)\n        );\n        workInProgress = null;\n        return;\n      }\n      if (unitOfWork.flags & 32768) {\n        if (isHydrating || suspendedReason === SuspendedOnError) root = !0;\n        else if (\n          workInProgressRootIsPrerendering ||\n          0 !== (workInProgressRootRenderLanes & 536870912)\n        )\n          root = !1;\n        else if (\n          ((workInProgressRootDidSkipSuspendedSiblings = root = !0),\n          suspendedReason === SuspendedOnData ||\n            suspendedReason === SuspendedOnAction ||\n            suspendedReason === SuspendedOnImmediate ||\n            suspendedReason === SuspendedOnDeprecatedThrowPromise)\n        )\n          (suspendedReason = suspenseHandlerStackCursor.current),\n            null !== suspendedReason &&\n              13 === suspendedReason.tag &&\n              (suspendedReason.flags |= 16384);\n        unwindUnitOfWork(unitOfWork, root);\n      } else completeUnitOfWork(unitOfWork);\n    }\n    function completeUnitOfWork(unitOfWork) {\n      var completedWork = unitOfWork;\n      do {\n        if (0 !== (completedWork.flags & 32768)) {\n          unwindUnitOfWork(\n            completedWork,\n            workInProgressRootDidSkipSuspendedSiblings\n          );\n          return;\n        }\n        var current = completedWork.alternate;\n        unitOfWork = completedWork.return;\n        startProfilerTimer(completedWork);\n        current = runWithFiberInDEV(\n          completedWork,\n          completeWork,\n          current,\n          completedWork,\n          entangledRenderLanes\n        );\n        (completedWork.mode & ProfileMode) !== NoMode &&\n          stopProfilerTimerIfRunningAndRecordIncompleteDuration(completedWork);\n        if (null !== current) {\n          workInProgress = current;\n          return;\n        }\n        completedWork = completedWork.sibling;\n        if (null !== completedWork) {\n          workInProgress = completedWork;\n          return;\n        }\n        workInProgress = completedWork = unitOfWork;\n      } while (null !== completedWork);\n      workInProgressRootExitStatus === RootInProgress &&\n        (workInProgressRootExitStatus = RootCompleted);\n    }\n    function unwindUnitOfWork(unitOfWork, skipSiblings) {\n      do {\n        var next = unwindWork(unitOfWork.alternate, unitOfWork);\n        if (null !== next) {\n          next.flags &= 32767;\n          workInProgress = next;\n          return;\n        }\n        if ((unitOfWork.mode & ProfileMode) !== NoMode) {\n          stopProfilerTimerIfRunningAndRecordIncompleteDuration(unitOfWork);\n          next = unitOfWork.actualDuration;\n          for (var child = unitOfWork.child; null !== child; )\n            (next += child.actualDuration), (child = child.sibling);\n          unitOfWork.actualDuration = next;\n        }\n        next = unitOfWork.return;\n        null !== next &&\n          ((next.flags |= 32768),\n          (next.subtreeFlags = 0),\n          (next.deletions = null));\n        if (\n          !skipSiblings &&\n          ((unitOfWork = unitOfWork.sibling), null !== unitOfWork)\n        ) {\n          workInProgress = unitOfWork;\n          return;\n        }\n        workInProgress = unitOfWork = next;\n      } while (null !== unitOfWork);\n      workInProgressRootExitStatus = RootSuspendedAtTheShell;\n      workInProgress = null;\n    }\n    function commitRoot(\n      root,\n      finishedWork,\n      lanes,\n      recoverableErrors,\n      transitions,\n      didIncludeRenderPhaseUpdate,\n      spawnedLane,\n      updatedLanes,\n      suspendedRetryLanes\n    ) {\n      root.cancelPendingCommit = null;\n      do flushPendingEffects();\n      while (pendingEffectsStatus !== NO_PENDING_EFFECTS);\n      ReactStrictModeWarnings.flushLegacyContextWarning();\n      ReactStrictModeWarnings.flushPendingUnsafeLifecycleWarnings();\n      if ((executionContext & (RenderContext | CommitContext)) !== NoContext)\n        throw Error(\"Should not already be working.\");\n      null !== injectedProfilingHooks &&\n        \"function\" === typeof injectedProfilingHooks.markCommitStarted &&\n        injectedProfilingHooks.markCommitStarted(lanes);\n      if (null === finishedWork) markCommitStopped();\n      else {\n        0 === lanes &&\n          console.error(\n            \"finishedLanes should not be empty during a commit. This is a bug in React.\"\n          );\n        if (finishedWork === root.current)\n          throw Error(\n            \"Cannot commit the same tree as before. This error is likely caused by a bug in React. Please file an issue.\"\n          );\n        didIncludeRenderPhaseUpdate =\n          finishedWork.lanes | finishedWork.childLanes;\n        didIncludeRenderPhaseUpdate |= concurrentlyUpdatedLanes;\n        markRootFinished(\n          root,\n          lanes,\n          didIncludeRenderPhaseUpdate,\n          spawnedLane,\n          updatedLanes,\n          suspendedRetryLanes\n        );\n        root === workInProgressRoot &&\n          ((workInProgress = workInProgressRoot = null),\n          (workInProgressRootRenderLanes = 0));\n        pendingFinishedWork = finishedWork;\n        pendingEffectsRoot = root;\n        pendingEffectsLanes = lanes;\n        pendingEffectsRemainingLanes = didIncludeRenderPhaseUpdate;\n        pendingPassiveTransitions = transitions;\n        pendingRecoverableErrors = recoverableErrors;\n        0 !== (finishedWork.subtreeFlags & 10256) ||\n        0 !== (finishedWork.flags & 10256)\n          ? ((root.callbackNode = null),\n            (root.callbackPriority = 0),\n            scheduleCallback$1(NormalPriority$1, function () {\n              flushPassiveEffects(!0);\n              return null;\n            }))\n          : ((root.callbackNode = null), (root.callbackPriority = 0));\n        commitStartTime = now();\n        recoverableErrors = 0 !== (finishedWork.flags & 13878);\n        if (0 !== (finishedWork.subtreeFlags & 13878) || recoverableErrors) {\n          recoverableErrors = ReactSharedInternals.T;\n          ReactSharedInternals.T = null;\n          transitions = ReactDOMSharedInternals.p;\n          ReactDOMSharedInternals.p = DiscreteEventPriority;\n          spawnedLane = executionContext;\n          executionContext |= CommitContext;\n          try {\n            commitBeforeMutationEffects(root, finishedWork, lanes);\n          } finally {\n            (executionContext = spawnedLane),\n              (ReactDOMSharedInternals.p = transitions),\n              (ReactSharedInternals.T = recoverableErrors);\n          }\n        }\n        pendingEffectsStatus = PENDING_MUTATION_PHASE;\n        flushMutationEffects();\n        flushLayoutEffects();\n        flushSpawnedWork();\n      }\n    }\n    function flushMutationEffects() {\n      if (pendingEffectsStatus === PENDING_MUTATION_PHASE) {\n        pendingEffectsStatus = NO_PENDING_EFFECTS;\n        var root = pendingEffectsRoot,\n          finishedWork = pendingFinishedWork,\n          lanes = pendingEffectsLanes,\n          rootMutationHasEffect = 0 !== (finishedWork.flags & 13878);\n        if (\n          0 !== (finishedWork.subtreeFlags & 13878) ||\n          rootMutationHasEffect\n        ) {\n          rootMutationHasEffect = ReactSharedInternals.T;\n          ReactSharedInternals.T = null;\n          var previousPriority = ReactDOMSharedInternals.p;\n          ReactDOMSharedInternals.p = DiscreteEventPriority;\n          var prevExecutionContext = executionContext;\n          executionContext |= CommitContext;\n          try {\n            inProgressLanes = lanes;\n            inProgressRoot = root;\n            commitMutationEffectsOnFiber(finishedWork, root);\n            inProgressRoot = inProgressLanes = null;\n            lanes = selectionInformation;\n            var curFocusedElem = getActiveElementDeep(root.containerInfo),\n              priorFocusedElem = lanes.focusedElem,\n              priorSelectionRange = lanes.selectionRange;\n            if (\n              curFocusedElem !== priorFocusedElem &&\n              priorFocusedElem &&\n              priorFocusedElem.ownerDocument &&\n              containsNode(\n                priorFocusedElem.ownerDocument.documentElement,\n                priorFocusedElem\n              )\n            ) {\n              if (\n                null !== priorSelectionRange &&\n                hasSelectionCapabilities(priorFocusedElem)\n              ) {\n                var start = priorSelectionRange.start,\n                  end = priorSelectionRange.end;\n                void 0 === end && (end = start);\n                if (\"selectionStart\" in priorFocusedElem)\n                  (priorFocusedElem.selectionStart = start),\n                    (priorFocusedElem.selectionEnd = Math.min(\n                      end,\n                      priorFocusedElem.value.length\n                    ));\n                else {\n                  var doc = priorFocusedElem.ownerDocument || document,\n                    win = (doc && doc.defaultView) || window;\n                  if (win.getSelection) {\n                    var selection = win.getSelection(),\n                      length = priorFocusedElem.textContent.length,\n                      start$jscomp$0 = Math.min(\n                        priorSelectionRange.start,\n                        length\n                      ),\n                      end$jscomp$0 =\n                        void 0 === priorSelectionRange.end\n                          ? start$jscomp$0\n                          : Math.min(priorSelectionRange.end, length);\n                    !selection.extend &&\n                      start$jscomp$0 > end$jscomp$0 &&\n                      ((curFocusedElem = end$jscomp$0),\n                      (end$jscomp$0 = start$jscomp$0),\n                      (start$jscomp$0 = curFocusedElem));\n                    var startMarker = getNodeForCharacterOffset(\n                        priorFocusedElem,\n                        start$jscomp$0\n                      ),\n                      endMarker = getNodeForCharacterOffset(\n                        priorFocusedElem,\n                        end$jscomp$0\n                      );\n                    if (\n                      startMarker &&\n                      endMarker &&\n                      (1 !== selection.rangeCount ||\n                        selection.anchorNode !== startMarker.node ||\n                        selection.anchorOffset !== startMarker.offset ||\n                        selection.focusNode !== endMarker.node ||\n                        selection.focusOffset !== endMarker.offset)\n                    ) {\n                      var range = doc.createRange();\n                      range.setStart(startMarker.node, startMarker.offset);\n                      selection.removeAllRanges();\n                      start$jscomp$0 > end$jscomp$0\n                        ? (selection.addRange(range),\n                          selection.extend(endMarker.node, endMarker.offset))\n                        : (range.setEnd(endMarker.node, endMarker.offset),\n                          selection.addRange(range));\n                    }\n                  }\n                }\n              }\n              doc = [];\n              for (\n                selection = priorFocusedElem;\n                (selection = selection.parentNode);\n\n              )\n                1 === selection.nodeType &&\n                  doc.push({\n                    element: selection,\n                    left: selection.scrollLeft,\n                    top: selection.scrollTop\n                  });\n              \"function\" === typeof priorFocusedElem.focus &&\n                priorFocusedElem.focus();\n              for (\n                priorFocusedElem = 0;\n                priorFocusedElem < doc.length;\n                priorFocusedElem++\n              ) {\n                var info = doc[priorFocusedElem];\n                info.element.scrollLeft = info.left;\n                info.element.scrollTop = info.top;\n              }\n            }\n            _enabled = !!eventsEnabled;\n            selectionInformation = eventsEnabled = null;\n          } finally {\n            (executionContext = prevExecutionContext),\n              (ReactDOMSharedInternals.p = previousPriority),\n              (ReactSharedInternals.T = rootMutationHasEffect);\n          }\n        }\n        root.current = finishedWork;\n        pendingEffectsStatus = PENDING_LAYOUT_PHASE;\n      }\n    }\n    function flushLayoutEffects() {\n      if (pendingEffectsStatus === PENDING_LAYOUT_PHASE) {\n        pendingEffectsStatus = NO_PENDING_EFFECTS;\n        var root = pendingEffectsRoot,\n          finishedWork = pendingFinishedWork,\n          lanes = pendingEffectsLanes,\n          rootHasLayoutEffect = 0 !== (finishedWork.flags & 8772);\n        if (0 !== (finishedWork.subtreeFlags & 8772) || rootHasLayoutEffect) {\n          rootHasLayoutEffect = ReactSharedInternals.T;\n          ReactSharedInternals.T = null;\n          var previousPriority = ReactDOMSharedInternals.p;\n          ReactDOMSharedInternals.p = DiscreteEventPriority;\n          var prevExecutionContext = executionContext;\n          executionContext |= CommitContext;\n          try {\n            null !== injectedProfilingHooks &&\n              \"function\" ===\n                typeof injectedProfilingHooks.markLayoutEffectsStarted &&\n              injectedProfilingHooks.markLayoutEffectsStarted(lanes),\n              (inProgressLanes = lanes),\n              (inProgressRoot = root),\n              commitLayoutEffectOnFiber(\n                root,\n                finishedWork.alternate,\n                finishedWork\n              ),\n              (inProgressRoot = inProgressLanes = null),\n              null !== injectedProfilingHooks &&\n                \"function\" ===\n                  typeof injectedProfilingHooks.markLayoutEffectsStopped &&\n                injectedProfilingHooks.markLayoutEffectsStopped();\n          } finally {\n            (executionContext = prevExecutionContext),\n              (ReactDOMSharedInternals.p = previousPriority),\n              (ReactSharedInternals.T = rootHasLayoutEffect);\n          }\n        }\n        pendingEffectsStatus = PENDING_AFTER_MUTATION_PHASE;\n      }\n    }\n    function flushSpawnedWork() {\n      if (\n        pendingEffectsStatus === PENDING_SPAWNED_WORK ||\n        pendingEffectsStatus === PENDING_AFTER_MUTATION_PHASE\n      ) {\n        pendingEffectsStatus = NO_PENDING_EFFECTS;\n        requestPaint();\n        var root = pendingEffectsRoot,\n          finishedWork = pendingFinishedWork,\n          lanes = pendingEffectsLanes,\n          recoverableErrors = pendingRecoverableErrors,\n          rootDidHavePassiveEffects =\n            0 !== (finishedWork.subtreeFlags & 10256) ||\n            0 !== (finishedWork.flags & 10256);\n        rootDidHavePassiveEffects\n          ? (pendingEffectsStatus = PENDING_PASSIVE_PHASE)\n          : ((pendingEffectsStatus = NO_PENDING_EFFECTS),\n            (pendingFinishedWork = pendingEffectsRoot = null),\n            releaseRootPooledCache(root, root.pendingLanes),\n            (nestedPassiveUpdateCount = 0),\n            (rootWithPassiveNestedUpdates = null));\n        var remainingLanes = root.pendingLanes;\n        0 === remainingLanes && (legacyErrorBoundariesThatAlreadyFailed = null);\n        rootDidHavePassiveEffects || commitDoubleInvokeEffectsInDEV(root);\n        rootDidHavePassiveEffects = lanesToEventPriority(lanes);\n        finishedWork = finishedWork.stateNode;\n        if (\n          injectedHook &&\n          \"function\" === typeof injectedHook.onCommitFiberRoot\n        )\n          try {\n            var didError = 128 === (finishedWork.current.flags & 128);\n            switch (rootDidHavePassiveEffects) {\n              case DiscreteEventPriority:\n                var schedulerPriority = ImmediatePriority;\n                break;\n              case ContinuousEventPriority:\n                schedulerPriority = UserBlockingPriority;\n                break;\n              case DefaultEventPriority:\n                schedulerPriority = NormalPriority$1;\n                break;\n              case IdleEventPriority:\n                schedulerPriority = IdlePriority;\n                break;\n              default:\n                schedulerPriority = NormalPriority$1;\n            }\n            injectedHook.onCommitFiberRoot(\n              rendererID,\n              finishedWork,\n              schedulerPriority,\n              didError\n            );\n          } catch (err) {\n            hasLoggedError ||\n              ((hasLoggedError = !0),\n              console.error(\n                \"React instrumentation encountered an error: %s\",\n                err\n              ));\n          }\n        isDevToolsPresent && root.memoizedUpdaters.clear();\n        onCommitRoot();\n        if (null !== recoverableErrors) {\n          didError = ReactSharedInternals.T;\n          schedulerPriority = ReactDOMSharedInternals.p;\n          ReactDOMSharedInternals.p = DiscreteEventPriority;\n          ReactSharedInternals.T = null;\n          try {\n            var onRecoverableError = root.onRecoverableError;\n            for (\n              finishedWork = 0;\n              finishedWork < recoverableErrors.length;\n              finishedWork++\n            ) {\n              var recoverableError = recoverableErrors[finishedWork],\n                errorInfo = makeErrorInfo(recoverableError.stack);\n              runWithFiberInDEV(\n                recoverableError.source,\n                onRecoverableError,\n                recoverableError.value,\n                errorInfo\n              );\n            }\n          } finally {\n            (ReactSharedInternals.T = didError),\n              (ReactDOMSharedInternals.p = schedulerPriority);\n          }\n        }\n        0 !== (pendingEffectsLanes & 3) && flushPendingEffects();\n        ensureRootIsScheduled(root);\n        remainingLanes = root.pendingLanes;\n        0 !== (lanes & 4194090) && 0 !== (remainingLanes & 42)\n          ? ((nestedUpdateScheduled = !0),\n            root === rootWithNestedUpdates\n              ? nestedUpdateCount++\n              : ((nestedUpdateCount = 0), (rootWithNestedUpdates = root)))\n          : (nestedUpdateCount = 0);\n        flushSyncWorkAcrossRoots_impl(0, !1);\n        markCommitStopped();\n      }\n    }\n    function makeErrorInfo(componentStack) {\n      componentStack = { componentStack: componentStack };\n      Object.defineProperty(componentStack, \"digest\", {\n        get: function () {\n          console.error(\n            'You are accessing \"digest\" from the errorInfo object passed to onRecoverableError. This property is no longer provided as part of errorInfo but can be accessed as a property of the Error instance itself.'\n          );\n        }\n      });\n      return componentStack;\n    }\n    function releaseRootPooledCache(root, remainingLanes) {\n      0 === (root.pooledCacheLanes &= remainingLanes) &&\n        ((remainingLanes = root.pooledCache),\n        null != remainingLanes &&\n          ((root.pooledCache = null), releaseCache(remainingLanes)));\n    }\n    function flushPendingEffects(wasDelayedCommit) {\n      flushMutationEffects();\n      flushLayoutEffects();\n      flushSpawnedWork();\n      return flushPassiveEffects(wasDelayedCommit);\n    }\n    function flushPassiveEffects() {\n      if (pendingEffectsStatus !== PENDING_PASSIVE_PHASE) return !1;\n      var root = pendingEffectsRoot,\n        remainingLanes = pendingEffectsRemainingLanes;\n      pendingEffectsRemainingLanes = 0;\n      var renderPriority = lanesToEventPriority(pendingEffectsLanes),\n        priority =\n          0 === DefaultEventPriority || DefaultEventPriority > renderPriority\n            ? DefaultEventPriority\n            : renderPriority;\n      renderPriority = ReactSharedInternals.T;\n      var previousPriority = ReactDOMSharedInternals.p;\n      try {\n        ReactDOMSharedInternals.p = priority;\n        ReactSharedInternals.T = null;\n        priority = pendingPassiveTransitions;\n        pendingPassiveTransitions = null;\n        var root$jscomp$0 = pendingEffectsRoot,\n          lanes = pendingEffectsLanes;\n        pendingEffectsStatus = NO_PENDING_EFFECTS;\n        pendingFinishedWork = pendingEffectsRoot = null;\n        pendingEffectsLanes = 0;\n        if ((executionContext & (RenderContext | CommitContext)) !== NoContext)\n          throw Error(\"Cannot flush passive effects while already rendering.\");\n        isFlushingPassiveEffects = !0;\n        didScheduleUpdateDuringPassiveEffects = !1;\n        null !== injectedProfilingHooks &&\n          \"function\" ===\n            typeof injectedProfilingHooks.markPassiveEffectsStarted &&\n          injectedProfilingHooks.markPassiveEffectsStarted(lanes);\n        var prevExecutionContext = executionContext;\n        executionContext |= CommitContext;\n        commitPassiveUnmountOnFiber(root$jscomp$0.current);\n        commitPassiveMountOnFiber(\n          root$jscomp$0,\n          root$jscomp$0.current,\n          lanes,\n          priority\n        );\n        null !== injectedProfilingHooks &&\n          \"function\" ===\n            typeof injectedProfilingHooks.markPassiveEffectsStopped &&\n          injectedProfilingHooks.markPassiveEffectsStopped();\n        commitDoubleInvokeEffectsInDEV(root$jscomp$0);\n        executionContext = prevExecutionContext;\n        flushSyncWorkAcrossRoots_impl(0, !1);\n        didScheduleUpdateDuringPassiveEffects\n          ? root$jscomp$0 === rootWithPassiveNestedUpdates\n            ? nestedPassiveUpdateCount++\n            : ((nestedPassiveUpdateCount = 0),\n              (rootWithPassiveNestedUpdates = root$jscomp$0))\n          : (nestedPassiveUpdateCount = 0);\n        didScheduleUpdateDuringPassiveEffects = isFlushingPassiveEffects = !1;\n        if (\n          injectedHook &&\n          \"function\" === typeof injectedHook.onPostCommitFiberRoot\n        )\n          try {\n            injectedHook.onPostCommitFiberRoot(rendererID, root$jscomp$0);\n          } catch (err) {\n            hasLoggedError ||\n              ((hasLoggedError = !0),\n              console.error(\n                \"React instrumentation encountered an error: %s\",\n                err\n              ));\n          }\n        var stateNode = root$jscomp$0.current.stateNode;\n        stateNode.effectDuration = 0;\n        stateNode.passiveEffectDuration = 0;\n        return !0;\n      } finally {\n        (ReactDOMSharedInternals.p = previousPriority),\n          (ReactSharedInternals.T = renderPriority),\n          releaseRootPooledCache(root, remainingLanes);\n      }\n    }\n    function captureCommitPhaseErrorOnRoot(rootFiber, sourceFiber, error) {\n      sourceFiber = createCapturedValueAtFiber(error, sourceFiber);\n      sourceFiber = createRootErrorUpdate(rootFiber.stateNode, sourceFiber, 2);\n      rootFiber = enqueueUpdate(rootFiber, sourceFiber, 2);\n      null !== rootFiber &&\n        (markRootUpdated$1(rootFiber, 2), ensureRootIsScheduled(rootFiber));\n    }\n    function captureCommitPhaseError(\n      sourceFiber,\n      nearestMountedAncestor,\n      error\n    ) {\n      isRunningInsertionEffect = !1;\n      if (3 === sourceFiber.tag)\n        captureCommitPhaseErrorOnRoot(sourceFiber, sourceFiber, error);\n      else {\n        for (; null !== nearestMountedAncestor; ) {\n          if (3 === nearestMountedAncestor.tag) {\n            captureCommitPhaseErrorOnRoot(\n              nearestMountedAncestor,\n              sourceFiber,\n              error\n            );\n            return;\n          }\n          if (1 === nearestMountedAncestor.tag) {\n            var instance = nearestMountedAncestor.stateNode;\n            if (\n              \"function\" ===\n                typeof nearestMountedAncestor.type.getDerivedStateFromError ||\n              (\"function\" === typeof instance.componentDidCatch &&\n                (null === legacyErrorBoundariesThatAlreadyFailed ||\n                  !legacyErrorBoundariesThatAlreadyFailed.has(instance)))\n            ) {\n              sourceFiber = createCapturedValueAtFiber(error, sourceFiber);\n              error = createClassErrorUpdate(2);\n              instance = enqueueUpdate(nearestMountedAncestor, error, 2);\n              null !== instance &&\n                (initializeClassErrorUpdate(\n                  error,\n                  instance,\n                  nearestMountedAncestor,\n                  sourceFiber\n                ),\n                markRootUpdated$1(instance, 2),\n                ensureRootIsScheduled(instance));\n              return;\n            }\n          }\n          nearestMountedAncestor = nearestMountedAncestor.return;\n        }\n        console.error(\n          \"Internal React error: Attempted to capture a commit phase error inside a detached tree. This indicates a bug in React. Potential causes include deleting the same fiber more than once, committing an already-finished tree, or an inconsistent return pointer.\\n\\nError message:\\n\\n%s\",\n          error\n        );\n      }\n    }\n    function attachPingListener(root, wakeable, lanes) {\n      var pingCache = root.pingCache;\n      if (null === pingCache) {\n        pingCache = root.pingCache = new PossiblyWeakMap();\n        var threadIDs = new Set();\n        pingCache.set(wakeable, threadIDs);\n      } else\n        (threadIDs = pingCache.get(wakeable)),\n          void 0 === threadIDs &&\n            ((threadIDs = new Set()), pingCache.set(wakeable, threadIDs));\n      threadIDs.has(lanes) ||\n        ((workInProgressRootDidAttachPingListener = !0),\n        threadIDs.add(lanes),\n        (pingCache = pingSuspendedRoot.bind(null, root, wakeable, lanes)),\n        isDevToolsPresent && restorePendingUpdaters(root, lanes),\n        wakeable.then(pingCache, pingCache));\n    }\n    function pingSuspendedRoot(root, wakeable, pingedLanes) {\n      var pingCache = root.pingCache;\n      null !== pingCache && pingCache.delete(wakeable);\n      root.pingedLanes |= root.suspendedLanes & pingedLanes;\n      root.warmLanes &= ~pingedLanes;\n      isConcurrentActEnvironment() &&\n        null === ReactSharedInternals.actQueue &&\n        console.error(\n          \"A suspended resource finished loading inside a test, but the event was not wrapped in act(...).\\n\\nWhen testing, code that resolves suspended data should be wrapped into act(...):\\n\\nact(() => {\\n  /* finish loading suspended data */\\n});\\n/* assert on the output */\\n\\nThis ensures that you're testing the behavior the user would see in the browser. Learn more at https://react.dev/link/wrap-tests-with-act\"\n        );\n      workInProgressRoot === root &&\n        (workInProgressRootRenderLanes & pingedLanes) === pingedLanes &&\n        (workInProgressRootExitStatus === RootSuspendedWithDelay ||\n        (workInProgressRootExitStatus === RootSuspended &&\n          (workInProgressRootRenderLanes & 62914560) ===\n            workInProgressRootRenderLanes &&\n          now$1() - globalMostRecentFallbackTime < FALLBACK_THROTTLE_MS)\n          ? (executionContext & RenderContext) === NoContext &&\n            prepareFreshStack(root, 0)\n          : (workInProgressRootPingedLanes |= pingedLanes),\n        workInProgressSuspendedRetryLanes === workInProgressRootRenderLanes &&\n          (workInProgressSuspendedRetryLanes = 0));\n      ensureRootIsScheduled(root);\n    }\n    function retryTimedOutBoundary(boundaryFiber, retryLane) {\n      0 === retryLane && (retryLane = claimNextRetryLane());\n      boundaryFiber = enqueueConcurrentRenderForLane(boundaryFiber, retryLane);\n      null !== boundaryFiber &&\n        (markRootUpdated$1(boundaryFiber, retryLane),\n        ensureRootIsScheduled(boundaryFiber));\n    }\n    function retryDehydratedSuspenseBoundary(boundaryFiber) {\n      var suspenseState = boundaryFiber.memoizedState,\n        retryLane = 0;\n      null !== suspenseState && (retryLane = suspenseState.retryLane);\n      retryTimedOutBoundary(boundaryFiber, retryLane);\n    }\n    function resolveRetryWakeable(boundaryFiber, wakeable) {\n      var retryLane = 0;\n      switch (boundaryFiber.tag) {\n        case 13:\n          var retryCache = boundaryFiber.stateNode;\n          var suspenseState = boundaryFiber.memoizedState;\n          null !== suspenseState && (retryLane = suspenseState.retryLane);\n          break;\n        case 19:\n          retryCache = boundaryFiber.stateNode;\n          break;\n        case 22:\n          retryCache = boundaryFiber.stateNode._retryCache;\n          break;\n        default:\n          throw Error(\n            \"Pinged unknown suspense boundary type. This is probably a bug in React.\"\n          );\n      }\n      null !== retryCache && retryCache.delete(wakeable);\n      retryTimedOutBoundary(boundaryFiber, retryLane);\n    }\n    function recursivelyTraverseAndDoubleInvokeEffectsInDEV(\n      root$jscomp$0,\n      parentFiber,\n      isInStrictMode\n    ) {\n      if (0 !== (parentFiber.subtreeFlags & 67117056))\n        for (parentFiber = parentFiber.child; null !== parentFiber; ) {\n          var root = root$jscomp$0,\n            fiber = parentFiber,\n            isStrictModeFiber = fiber.type === REACT_STRICT_MODE_TYPE;\n          isStrictModeFiber = isInStrictMode || isStrictModeFiber;\n          22 !== fiber.tag\n            ? fiber.flags & 67108864\n              ? isStrictModeFiber &&\n                runWithFiberInDEV(\n                  fiber,\n                  doubleInvokeEffectsOnFiber,\n                  root,\n                  fiber,\n                  (fiber.mode & NoStrictPassiveEffectsMode) === NoMode\n                )\n              : recursivelyTraverseAndDoubleInvokeEffectsInDEV(\n                  root,\n                  fiber,\n                  isStrictModeFiber\n                )\n            : null === fiber.memoizedState &&\n              (isStrictModeFiber && fiber.flags & 8192\n                ? runWithFiberInDEV(\n                    fiber,\n                    doubleInvokeEffectsOnFiber,\n                    root,\n                    fiber\n                  )\n                : fiber.subtreeFlags & 67108864 &&\n                  runWithFiberInDEV(\n                    fiber,\n                    recursivelyTraverseAndDoubleInvokeEffectsInDEV,\n                    root,\n                    fiber,\n                    isStrictModeFiber\n                  ));\n          parentFiber = parentFiber.sibling;\n        }\n    }\n    function doubleInvokeEffectsOnFiber(root, fiber) {\n      var shouldDoubleInvokePassiveEffects =\n        2 < arguments.length && void 0 !== arguments[2] ? arguments[2] : !0;\n      setIsStrictModeForDevtools(!0);\n      try {\n        disappearLayoutEffects(fiber),\n          shouldDoubleInvokePassiveEffects && disconnectPassiveEffect(fiber),\n          reappearLayoutEffects(root, fiber.alternate, fiber, !1),\n          shouldDoubleInvokePassiveEffects &&\n            reconnectPassiveEffects(root, fiber, 0, null, !1, 0);\n      } finally {\n        setIsStrictModeForDevtools(!1);\n      }\n    }\n    function commitDoubleInvokeEffectsInDEV(root) {\n      var doubleInvokeEffects = !0;\n      root.current.mode & (StrictLegacyMode | StrictEffectsMode) ||\n        (doubleInvokeEffects = !1);\n      recursivelyTraverseAndDoubleInvokeEffectsInDEV(\n        root,\n        root.current,\n        doubleInvokeEffects\n      );\n    }\n    function warnAboutUpdateOnNotYetMountedFiberInDEV(fiber) {\n      if ((executionContext & RenderContext) === NoContext) {\n        var tag = fiber.tag;\n        if (\n          3 === tag ||\n          1 === tag ||\n          0 === tag ||\n          11 === tag ||\n          14 === tag ||\n          15 === tag\n        ) {\n          tag = getComponentNameFromFiber(fiber) || \"ReactComponent\";\n          if (null !== didWarnStateUpdateForNotYetMountedComponent) {\n            if (didWarnStateUpdateForNotYetMountedComponent.has(tag)) return;\n            didWarnStateUpdateForNotYetMountedComponent.add(tag);\n          } else didWarnStateUpdateForNotYetMountedComponent = new Set([tag]);\n          runWithFiberInDEV(fiber, function () {\n            console.error(\n              \"Can't perform a React state update on a component that hasn't mounted yet. This indicates that you have a side-effect in your render function that asynchronously later calls tries to update the component. Move this work to useEffect instead.\"\n            );\n          });\n        }\n      }\n    }\n    function restorePendingUpdaters(root, lanes) {\n      isDevToolsPresent &&\n        root.memoizedUpdaters.forEach(function (schedulingFiber) {\n          addFiberToLanesMap(root, schedulingFiber, lanes);\n        });\n    }\n    function scheduleCallback$1(priorityLevel, callback) {\n      var actQueue = ReactSharedInternals.actQueue;\n      return null !== actQueue\n        ? (actQueue.push(callback), fakeActCallbackNode$1)\n        : scheduleCallback$3(priorityLevel, callback);\n    }\n    function warnIfUpdatesNotWrappedWithActDEV(fiber) {\n      isConcurrentActEnvironment() &&\n        null === ReactSharedInternals.actQueue &&\n        runWithFiberInDEV(fiber, function () {\n          console.error(\n            \"An update to %s inside a test was not wrapped in act(...).\\n\\nWhen testing, code that causes React state updates should be wrapped into act(...):\\n\\nact(() => {\\n  /* fire events that update state */\\n});\\n/* assert on the output */\\n\\nThis ensures that you're testing the behavior the user would see in the browser. Learn more at https://react.dev/link/wrap-tests-with-act\",\n            getComponentNameFromFiber(fiber)\n          );\n        });\n    }\n    function ensureRootIsScheduled(root) {\n      root !== lastScheduledRoot &&\n        null === root.next &&\n        (null === lastScheduledRoot\n          ? (firstScheduledRoot = lastScheduledRoot = root)\n          : (lastScheduledRoot = lastScheduledRoot.next = root));\n      mightHavePendingSyncWork = !0;\n      null !== ReactSharedInternals.actQueue\n        ? didScheduleMicrotask_act ||\n          ((didScheduleMicrotask_act = !0), scheduleImmediateRootScheduleTask())\n        : didScheduleMicrotask ||\n          ((didScheduleMicrotask = !0), scheduleImmediateRootScheduleTask());\n    }\n    function flushSyncWorkAcrossRoots_impl(syncTransitionLanes, onlyLegacy) {\n      if (!isFlushingWork && mightHavePendingSyncWork) {\n        isFlushingWork = !0;\n        do {\n          var didPerformSomeWork = !1;\n          for (var root = firstScheduledRoot; null !== root; ) {\n            if (!onlyLegacy)\n              if (0 !== syncTransitionLanes) {\n                var pendingLanes = root.pendingLanes;\n                if (0 === pendingLanes) var nextLanes = 0;\n                else {\n                  var suspendedLanes = root.suspendedLanes,\n                    pingedLanes = root.pingedLanes;\n                  nextLanes =\n                    (1 << (31 - clz32(42 | syncTransitionLanes) + 1)) - 1;\n                  nextLanes &= pendingLanes & ~(suspendedLanes & ~pingedLanes);\n                  nextLanes =\n                    nextLanes & 201326741\n                      ? (nextLanes & 201326741) | 1\n                      : nextLanes\n                        ? nextLanes | 2\n                        : 0;\n                }\n                0 !== nextLanes &&\n                  ((didPerformSomeWork = !0),\n                  performSyncWorkOnRoot(root, nextLanes));\n              } else\n                (nextLanes = workInProgressRootRenderLanes),\n                  (nextLanes = getNextLanes(\n                    root,\n                    root === workInProgressRoot ? nextLanes : 0,\n                    null !== root.cancelPendingCommit ||\n                      root.timeoutHandle !== noTimeout\n                  )),\n                  0 === (nextLanes & 3) ||\n                    checkIfRootIsPrerendering(root, nextLanes) ||\n                    ((didPerformSomeWork = !0),\n                    performSyncWorkOnRoot(root, nextLanes));\n            root = root.next;\n          }\n        } while (didPerformSomeWork);\n        isFlushingWork = !1;\n      }\n    }\n    function processRootScheduleInImmediateTask() {\n      processRootScheduleInMicrotask();\n    }\n    function processRootScheduleInMicrotask() {\n      mightHavePendingSyncWork =\n        didScheduleMicrotask_act =\n        didScheduleMicrotask =\n          !1;\n      var syncTransitionLanes = 0;\n      0 !== currentEventTransitionLane &&\n        (shouldAttemptEagerTransition() &&\n          (syncTransitionLanes = currentEventTransitionLane),\n        (currentEventTransitionLane = 0));\n      for (\n        var currentTime = now$1(), prev = null, root = firstScheduledRoot;\n        null !== root;\n\n      ) {\n        var next = root.next,\n          nextLanes = scheduleTaskForRootDuringMicrotask(root, currentTime);\n        if (0 === nextLanes)\n          (root.next = null),\n            null === prev ? (firstScheduledRoot = next) : (prev.next = next),\n            null === next && (lastScheduledRoot = prev);\n        else if (\n          ((prev = root), 0 !== syncTransitionLanes || 0 !== (nextLanes & 3))\n        )\n          mightHavePendingSyncWork = !0;\n        root = next;\n      }\n      flushSyncWorkAcrossRoots_impl(syncTransitionLanes, !1);\n    }\n    function scheduleTaskForRootDuringMicrotask(root, currentTime) {\n      for (\n        var suspendedLanes = root.suspendedLanes,\n          pingedLanes = root.pingedLanes,\n          expirationTimes = root.expirationTimes,\n          lanes = root.pendingLanes & -62914561;\n        0 < lanes;\n\n      ) {\n        var index = 31 - clz32(lanes),\n          lane = 1 << index,\n          expirationTime = expirationTimes[index];\n        if (-1 === expirationTime) {\n          if (0 === (lane & suspendedLanes) || 0 !== (lane & pingedLanes))\n            expirationTimes[index] = computeExpirationTime(lane, currentTime);\n        } else expirationTime <= currentTime && (root.expiredLanes |= lane);\n        lanes &= ~lane;\n      }\n      currentTime = workInProgressRoot;\n      suspendedLanes = workInProgressRootRenderLanes;\n      suspendedLanes = getNextLanes(\n        root,\n        root === currentTime ? suspendedLanes : 0,\n        null !== root.cancelPendingCommit || root.timeoutHandle !== noTimeout\n      );\n      pingedLanes = root.callbackNode;\n      if (\n        0 === suspendedLanes ||\n        (root === currentTime &&\n          (workInProgressSuspendedReason === SuspendedOnData ||\n            workInProgressSuspendedReason === SuspendedOnAction)) ||\n        null !== root.cancelPendingCommit\n      )\n        return (\n          null !== pingedLanes && cancelCallback(pingedLanes),\n          (root.callbackNode = null),\n          (root.callbackPriority = 0)\n        );\n      if (\n        0 === (suspendedLanes & 3) ||\n        checkIfRootIsPrerendering(root, suspendedLanes)\n      ) {\n        currentTime = suspendedLanes & -suspendedLanes;\n        if (\n          currentTime !== root.callbackPriority ||\n          (null !== ReactSharedInternals.actQueue &&\n            pingedLanes !== fakeActCallbackNode)\n        )\n          cancelCallback(pingedLanes);\n        else return currentTime;\n        switch (lanesToEventPriority(suspendedLanes)) {\n          case DiscreteEventPriority:\n          case ContinuousEventPriority:\n            suspendedLanes = UserBlockingPriority;\n            break;\n          case DefaultEventPriority:\n            suspendedLanes = NormalPriority$1;\n            break;\n          case IdleEventPriority:\n            suspendedLanes = IdlePriority;\n            break;\n          default:\n            suspendedLanes = NormalPriority$1;\n        }\n        pingedLanes = performWorkOnRootViaSchedulerTask.bind(null, root);\n        null !== ReactSharedInternals.actQueue\n          ? (ReactSharedInternals.actQueue.push(pingedLanes),\n            (suspendedLanes = fakeActCallbackNode))\n          : (suspendedLanes = scheduleCallback$3(suspendedLanes, pingedLanes));\n        root.callbackPriority = currentTime;\n        root.callbackNode = suspendedLanes;\n        return currentTime;\n      }\n      null !== pingedLanes && cancelCallback(pingedLanes);\n      root.callbackPriority = 2;\n      root.callbackNode = null;\n      return 2;\n    }\n    function performWorkOnRootViaSchedulerTask(root, didTimeout) {\n      nestedUpdateScheduled = currentUpdateIsNested = !1;\n      if (\n        pendingEffectsStatus !== NO_PENDING_EFFECTS &&\n        pendingEffectsStatus !== PENDING_PASSIVE_PHASE\n      )\n        return (root.callbackNode = null), (root.callbackPriority = 0), null;\n      var originalCallbackNode = root.callbackNode;\n      if (flushPendingEffects(!0) && root.callbackNode !== originalCallbackNode)\n        return null;\n      var workInProgressRootRenderLanes$jscomp$0 =\n        workInProgressRootRenderLanes;\n      workInProgressRootRenderLanes$jscomp$0 = getNextLanes(\n        root,\n        root === workInProgressRoot\n          ? workInProgressRootRenderLanes$jscomp$0\n          : 0,\n        null !== root.cancelPendingCommit || root.timeoutHandle !== noTimeout\n      );\n      if (0 === workInProgressRootRenderLanes$jscomp$0) return null;\n      performWorkOnRoot(\n        root,\n        workInProgressRootRenderLanes$jscomp$0,\n        didTimeout\n      );\n      scheduleTaskForRootDuringMicrotask(root, now$1());\n      return null != root.callbackNode &&\n        root.callbackNode === originalCallbackNode\n        ? performWorkOnRootViaSchedulerTask.bind(null, root)\n        : null;\n    }\n    function performSyncWorkOnRoot(root, lanes) {\n      if (flushPendingEffects()) return null;\n      currentUpdateIsNested = nestedUpdateScheduled;\n      nestedUpdateScheduled = !1;\n      performWorkOnRoot(root, lanes, !0);\n    }\n    function cancelCallback(callbackNode) {\n      callbackNode !== fakeActCallbackNode &&\n        null !== callbackNode &&\n        cancelCallback$1(callbackNode);\n    }\n    function scheduleImmediateRootScheduleTask() {\n      null !== ReactSharedInternals.actQueue &&\n        ReactSharedInternals.actQueue.push(function () {\n          processRootScheduleInMicrotask();\n          return null;\n        });\n      scheduleMicrotask(function () {\n        (executionContext & (RenderContext | CommitContext)) !== NoContext\n          ? scheduleCallback$3(\n              ImmediatePriority,\n              processRootScheduleInImmediateTask\n            )\n          : processRootScheduleInMicrotask();\n      });\n    }\n    function requestTransitionLane() {\n      0 === currentEventTransitionLane &&\n        (currentEventTransitionLane = claimNextTransitionLane());\n      return currentEventTransitionLane;\n    }\n    function coerceFormActionProp(actionProp) {\n      if (\n        null == actionProp ||\n        \"symbol\" === typeof actionProp ||\n        \"boolean\" === typeof actionProp\n      )\n        return null;\n      if (\"function\" === typeof actionProp) return actionProp;\n      checkAttributeStringCoercion(actionProp, \"action\");\n      return sanitizeURL(\"\" + actionProp);\n    }\n    function createFormDataWithSubmitter(form, submitter) {\n      var temp = submitter.ownerDocument.createElement(\"input\");\n      temp.name = submitter.name;\n      temp.value = submitter.value;\n      form.id && temp.setAttribute(\"form\", form.id);\n      submitter.parentNode.insertBefore(temp, submitter);\n      form = new FormData(form);\n      temp.parentNode.removeChild(temp);\n      return form;\n    }\n    function extractEvents$1(\n      dispatchQueue,\n      domEventName,\n      maybeTargetInst,\n      nativeEvent,\n      nativeEventTarget\n    ) {\n      if (\n        \"submit\" === domEventName &&\n        maybeTargetInst &&\n        maybeTargetInst.stateNode === nativeEventTarget\n      ) {\n        var action = coerceFormActionProp(\n            (nativeEventTarget[internalPropsKey] || null).action\n          ),\n          submitter = nativeEvent.submitter;\n        submitter &&\n          ((domEventName = (domEventName = submitter[internalPropsKey] || null)\n            ? coerceFormActionProp(domEventName.formAction)\n            : submitter.getAttribute(\"formAction\")),\n          null !== domEventName &&\n            ((action = domEventName), (submitter = null)));\n        var event = new SyntheticEvent(\n          \"action\",\n          \"action\",\n          null,\n          nativeEvent,\n          nativeEventTarget\n        );\n        dispatchQueue.push({\n          event: event,\n          listeners: [\n            {\n              instance: null,\n              listener: function () {\n                if (nativeEvent.defaultPrevented) {\n                  if (0 !== currentEventTransitionLane) {\n                    var formData = submitter\n                        ? createFormDataWithSubmitter(\n                            nativeEventTarget,\n                            submitter\n                          )\n                        : new FormData(nativeEventTarget),\n                      pendingState = {\n                        pending: !0,\n                        data: formData,\n                        method: nativeEventTarget.method,\n                        action: action\n                      };\n                    Object.freeze(pendingState);\n                    startHostTransition(\n                      maybeTargetInst,\n                      pendingState,\n                      null,\n                      formData\n                    );\n                  }\n                } else\n                  \"function\" === typeof action &&\n                    (event.preventDefault(),\n                    (formData = submitter\n                      ? createFormDataWithSubmitter(\n                          nativeEventTarget,\n                          submitter\n                        )\n                      : new FormData(nativeEventTarget)),\n                    (pendingState = {\n                      pending: !0,\n                      data: formData,\n                      method: nativeEventTarget.method,\n                      action: action\n                    }),\n                    Object.freeze(pendingState),\n                    startHostTransition(\n                      maybeTargetInst,\n                      pendingState,\n                      action,\n                      formData\n                    ));\n              },\n              currentTarget: nativeEventTarget\n            }\n          ]\n        });\n      }\n    }\n    function executeDispatch(event, listener, currentTarget) {\n      event.currentTarget = currentTarget;\n      try {\n        listener(event);\n      } catch (error) {\n        reportGlobalError(error);\n      }\n      event.currentTarget = null;\n    }\n    function processDispatchQueue(dispatchQueue, eventSystemFlags) {\n      eventSystemFlags = 0 !== (eventSystemFlags & 4);\n      for (var i = 0; i < dispatchQueue.length; i++) {\n        var _dispatchQueue$i = dispatchQueue[i];\n        a: {\n          var previousInstance = void 0,\n            event = _dispatchQueue$i.event;\n          _dispatchQueue$i = _dispatchQueue$i.listeners;\n          if (eventSystemFlags)\n            for (\n              var i$jscomp$0 = _dispatchQueue$i.length - 1;\n              0 <= i$jscomp$0;\n              i$jscomp$0--\n            ) {\n              var _dispatchListeners$i = _dispatchQueue$i[i$jscomp$0],\n                instance = _dispatchListeners$i.instance,\n                currentTarget = _dispatchListeners$i.currentTarget;\n              _dispatchListeners$i = _dispatchListeners$i.listener;\n              if (instance !== previousInstance && event.isPropagationStopped())\n                break a;\n              null !== instance\n                ? runWithFiberInDEV(\n                    instance,\n                    executeDispatch,\n                    event,\n                    _dispatchListeners$i,\n                    currentTarget\n                  )\n                : executeDispatch(event, _dispatchListeners$i, currentTarget);\n              previousInstance = instance;\n            }\n          else\n            for (\n              i$jscomp$0 = 0;\n              i$jscomp$0 < _dispatchQueue$i.length;\n              i$jscomp$0++\n            ) {\n              _dispatchListeners$i = _dispatchQueue$i[i$jscomp$0];\n              instance = _dispatchListeners$i.instance;\n              currentTarget = _dispatchListeners$i.currentTarget;\n              _dispatchListeners$i = _dispatchListeners$i.listener;\n              if (instance !== previousInstance && event.isPropagationStopped())\n                break a;\n              null !== instance\n                ? runWithFiberInDEV(\n                    instance,\n                    executeDispatch,\n                    event,\n                    _dispatchListeners$i,\n                    currentTarget\n                  )\n                : executeDispatch(event, _dispatchListeners$i, currentTarget);\n              previousInstance = instance;\n            }\n        }\n      }\n    }\n    function listenToNonDelegatedEvent(domEventName, targetElement) {\n      nonDelegatedEvents.has(domEventName) ||\n        console.error(\n          'Did not expect a listenToNonDelegatedEvent() call for \"%s\". This is a bug in React. Please file an issue.',\n          domEventName\n        );\n      var listenerSet = targetElement[internalEventHandlersKey];\n      void 0 === listenerSet &&\n        (listenerSet = targetElement[internalEventHandlersKey] = new Set());\n      var listenerSetKey = domEventName + \"__bubble\";\n      listenerSet.has(listenerSetKey) ||\n        (addTrappedEventListener(targetElement, domEventName, 2, !1),\n        listenerSet.add(listenerSetKey));\n    }\n    function listenToNativeEvent(domEventName, isCapturePhaseListener, target) {\n      nonDelegatedEvents.has(domEventName) &&\n        !isCapturePhaseListener &&\n        console.error(\n          'Did not expect a listenToNativeEvent() call for \"%s\" in the bubble phase. This is a bug in React. Please file an issue.',\n          domEventName\n        );\n      var eventSystemFlags = 0;\n      isCapturePhaseListener && (eventSystemFlags |= 4);\n      addTrappedEventListener(\n        target,\n        domEventName,\n        eventSystemFlags,\n        isCapturePhaseListener\n      );\n    }\n    function listenToAllSupportedEvents(rootContainerElement) {\n      if (!rootContainerElement[listeningMarker]) {\n        rootContainerElement[listeningMarker] = !0;\n        allNativeEvents.forEach(function (domEventName) {\n          \"selectionchange\" !== domEventName &&\n            (nonDelegatedEvents.has(domEventName) ||\n              listenToNativeEvent(domEventName, !1, rootContainerElement),\n            listenToNativeEvent(domEventName, !0, rootContainerElement));\n        });\n        var ownerDocument =\n          9 === rootContainerElement.nodeType\n            ? rootContainerElement\n            : rootContainerElement.ownerDocument;\n        null === ownerDocument ||\n          ownerDocument[listeningMarker] ||\n          ((ownerDocument[listeningMarker] = !0),\n          listenToNativeEvent(\"selectionchange\", !1, ownerDocument));\n      }\n    }\n    function addTrappedEventListener(\n      targetContainer,\n      domEventName,\n      eventSystemFlags,\n      isCapturePhaseListener\n    ) {\n      switch (getEventPriority(domEventName)) {\n        case DiscreteEventPriority:\n          var listenerWrapper = dispatchDiscreteEvent;\n          break;\n        case ContinuousEventPriority:\n          listenerWrapper = dispatchContinuousEvent;\n          break;\n        default:\n          listenerWrapper = dispatchEvent;\n      }\n      eventSystemFlags = listenerWrapper.bind(\n        null,\n        domEventName,\n        eventSystemFlags,\n        targetContainer\n      );\n      listenerWrapper = void 0;\n      !passiveBrowserEventsSupported ||\n        (\"touchstart\" !== domEventName &&\n          \"touchmove\" !== domEventName &&\n          \"wheel\" !== domEventName) ||\n        (listenerWrapper = !0);\n      isCapturePhaseListener\n        ? void 0 !== listenerWrapper\n          ? targetContainer.addEventListener(domEventName, eventSystemFlags, {\n              capture: !0,\n              passive: listenerWrapper\n            })\n          : targetContainer.addEventListener(domEventName, eventSystemFlags, !0)\n        : void 0 !== listenerWrapper\n          ? targetContainer.addEventListener(domEventName, eventSystemFlags, {\n              passive: listenerWrapper\n            })\n          : targetContainer.addEventListener(\n              domEventName,\n              eventSystemFlags,\n              !1\n            );\n    }\n    function dispatchEventForPluginEventSystem(\n      domEventName,\n      eventSystemFlags,\n      nativeEvent,\n      targetInst$jscomp$0,\n      targetContainer\n    ) {\n      var ancestorInst = targetInst$jscomp$0;\n      if (\n        0 === (eventSystemFlags & 1) &&\n        0 === (eventSystemFlags & 2) &&\n        null !== targetInst$jscomp$0\n      )\n        a: for (;;) {\n          if (null === targetInst$jscomp$0) return;\n          var nodeTag = targetInst$jscomp$0.tag;\n          if (3 === nodeTag || 4 === nodeTag) {\n            var container = targetInst$jscomp$0.stateNode.containerInfo;\n            if (container === targetContainer) break;\n            if (4 === nodeTag)\n              for (nodeTag = targetInst$jscomp$0.return; null !== nodeTag; ) {\n                var grandTag = nodeTag.tag;\n                if (\n                  (3 === grandTag || 4 === grandTag) &&\n                  nodeTag.stateNode.containerInfo === targetContainer\n                )\n                  return;\n                nodeTag = nodeTag.return;\n              }\n            for (; null !== container; ) {\n              nodeTag = getClosestInstanceFromNode(container);\n              if (null === nodeTag) return;\n              grandTag = nodeTag.tag;\n              if (\n                5 === grandTag ||\n                6 === grandTag ||\n                26 === grandTag ||\n                27 === grandTag\n              ) {\n                targetInst$jscomp$0 = ancestorInst = nodeTag;\n                continue a;\n              }\n              container = container.parentNode;\n            }\n          }\n          targetInst$jscomp$0 = targetInst$jscomp$0.return;\n        }\n      batchedUpdates$1(function () {\n        var targetInst = ancestorInst,\n          nativeEventTarget = getEventTarget(nativeEvent),\n          dispatchQueue = [];\n        a: {\n          var reactName = topLevelEventsToReactNames.get(domEventName);\n          if (void 0 !== reactName) {\n            var SyntheticEventCtor = SyntheticEvent,\n              reactEventType = domEventName;\n            switch (domEventName) {\n              case \"keypress\":\n                if (0 === getEventCharCode(nativeEvent)) break a;\n              case \"keydown\":\n              case \"keyup\":\n                SyntheticEventCtor = SyntheticKeyboardEvent;\n                break;\n              case \"focusin\":\n                reactEventType = \"focus\";\n                SyntheticEventCtor = SyntheticFocusEvent;\n                break;\n              case \"focusout\":\n                reactEventType = \"blur\";\n                SyntheticEventCtor = SyntheticFocusEvent;\n                break;\n              case \"beforeblur\":\n              case \"afterblur\":\n                SyntheticEventCtor = SyntheticFocusEvent;\n                break;\n              case \"click\":\n                if (2 === nativeEvent.button) break a;\n              case \"auxclick\":\n              case \"dblclick\":\n              case \"mousedown\":\n              case \"mousemove\":\n              case \"mouseup\":\n              case \"mouseout\":\n              case \"mouseover\":\n              case \"contextmenu\":\n                SyntheticEventCtor = SyntheticMouseEvent;\n                break;\n              case \"drag\":\n              case \"dragend\":\n              case \"dragenter\":\n              case \"dragexit\":\n              case \"dragleave\":\n              case \"dragover\":\n              case \"dragstart\":\n              case \"drop\":\n                SyntheticEventCtor = SyntheticDragEvent;\n                break;\n              case \"touchcancel\":\n              case \"touchend\":\n              case \"touchmove\":\n              case \"touchstart\":\n                SyntheticEventCtor = SyntheticTouchEvent;\n                break;\n              case ANIMATION_END:\n              case ANIMATION_ITERATION:\n              case ANIMATION_START:\n                SyntheticEventCtor = SyntheticAnimationEvent;\n                break;\n              case TRANSITION_END:\n                SyntheticEventCtor = SyntheticTransitionEvent;\n                break;\n              case \"scroll\":\n              case \"scrollend\":\n                SyntheticEventCtor = SyntheticUIEvent;\n                break;\n              case \"wheel\":\n                SyntheticEventCtor = SyntheticWheelEvent;\n                break;\n              case \"copy\":\n              case \"cut\":\n              case \"paste\":\n                SyntheticEventCtor = SyntheticClipboardEvent;\n                break;\n              case \"gotpointercapture\":\n              case \"lostpointercapture\":\n              case \"pointercancel\":\n              case \"pointerdown\":\n              case \"pointermove\":\n              case \"pointerout\":\n              case \"pointerover\":\n              case \"pointerup\":\n                SyntheticEventCtor = SyntheticPointerEvent;\n                break;\n              case \"toggle\":\n              case \"beforetoggle\":\n                SyntheticEventCtor = SyntheticToggleEvent;\n            }\n            var inCapturePhase = 0 !== (eventSystemFlags & 4),\n              accumulateTargetOnly =\n                !inCapturePhase &&\n                (\"scroll\" === domEventName || \"scrollend\" === domEventName),\n              reactEventName = inCapturePhase\n                ? null !== reactName\n                  ? reactName + \"Capture\"\n                  : null\n                : reactName;\n            inCapturePhase = [];\n            for (\n              var instance = targetInst, lastHostComponent;\n              null !== instance;\n\n            ) {\n              var _instance2 = instance;\n              lastHostComponent = _instance2.stateNode;\n              _instance2 = _instance2.tag;\n              (5 !== _instance2 && 26 !== _instance2 && 27 !== _instance2) ||\n                null === lastHostComponent ||\n                null === reactEventName ||\n                ((_instance2 = getListener(instance, reactEventName)),\n                null != _instance2 &&\n                  inCapturePhase.push(\n                    createDispatchListener(\n                      instance,\n                      _instance2,\n                      lastHostComponent\n                    )\n                  ));\n              if (accumulateTargetOnly) break;\n              instance = instance.return;\n            }\n            0 < inCapturePhase.length &&\n              ((reactName = new SyntheticEventCtor(\n                reactName,\n                reactEventType,\n                null,\n                nativeEvent,\n                nativeEventTarget\n              )),\n              dispatchQueue.push({\n                event: reactName,\n                listeners: inCapturePhase\n              }));\n          }\n        }\n        if (0 === (eventSystemFlags & 7)) {\n          a: {\n            reactName =\n              \"mouseover\" === domEventName || \"pointerover\" === domEventName;\n            SyntheticEventCtor =\n              \"mouseout\" === domEventName || \"pointerout\" === domEventName;\n            if (\n              reactName &&\n              nativeEvent !== currentReplayingEvent &&\n              (reactEventType =\n                nativeEvent.relatedTarget || nativeEvent.fromElement) &&\n              (getClosestInstanceFromNode(reactEventType) ||\n                reactEventType[internalContainerInstanceKey])\n            )\n              break a;\n            if (SyntheticEventCtor || reactName) {\n              reactName =\n                nativeEventTarget.window === nativeEventTarget\n                  ? nativeEventTarget\n                  : (reactName = nativeEventTarget.ownerDocument)\n                    ? reactName.defaultView || reactName.parentWindow\n                    : window;\n              if (SyntheticEventCtor) {\n                if (\n                  ((reactEventType =\n                    nativeEvent.relatedTarget || nativeEvent.toElement),\n                  (SyntheticEventCtor = targetInst),\n                  (reactEventType = reactEventType\n                    ? getClosestInstanceFromNode(reactEventType)\n                    : null),\n                  null !== reactEventType &&\n                    ((accumulateTargetOnly =\n                      getNearestMountedFiber(reactEventType)),\n                    (inCapturePhase = reactEventType.tag),\n                    reactEventType !== accumulateTargetOnly ||\n                      (5 !== inCapturePhase &&\n                        27 !== inCapturePhase &&\n                        6 !== inCapturePhase)))\n                )\n                  reactEventType = null;\n              } else (SyntheticEventCtor = null), (reactEventType = targetInst);\n              if (SyntheticEventCtor !== reactEventType) {\n                inCapturePhase = SyntheticMouseEvent;\n                _instance2 = \"onMouseLeave\";\n                reactEventName = \"onMouseEnter\";\n                instance = \"mouse\";\n                if (\n                  \"pointerout\" === domEventName ||\n                  \"pointerover\" === domEventName\n                )\n                  (inCapturePhase = SyntheticPointerEvent),\n                    (_instance2 = \"onPointerLeave\"),\n                    (reactEventName = \"onPointerEnter\"),\n                    (instance = \"pointer\");\n                accumulateTargetOnly =\n                  null == SyntheticEventCtor\n                    ? reactName\n                    : getNodeFromInstance(SyntheticEventCtor);\n                lastHostComponent =\n                  null == reactEventType\n                    ? reactName\n                    : getNodeFromInstance(reactEventType);\n                reactName = new inCapturePhase(\n                  _instance2,\n                  instance + \"leave\",\n                  SyntheticEventCtor,\n                  nativeEvent,\n                  nativeEventTarget\n                );\n                reactName.target = accumulateTargetOnly;\n                reactName.relatedTarget = lastHostComponent;\n                _instance2 = null;\n                getClosestInstanceFromNode(nativeEventTarget) === targetInst &&\n                  ((inCapturePhase = new inCapturePhase(\n                    reactEventName,\n                    instance + \"enter\",\n                    reactEventType,\n                    nativeEvent,\n                    nativeEventTarget\n                  )),\n                  (inCapturePhase.target = lastHostComponent),\n                  (inCapturePhase.relatedTarget = accumulateTargetOnly),\n                  (_instance2 = inCapturePhase));\n                accumulateTargetOnly = _instance2;\n                if (SyntheticEventCtor && reactEventType)\n                  b: {\n                    inCapturePhase = SyntheticEventCtor;\n                    reactEventName = reactEventType;\n                    instance = 0;\n                    for (\n                      lastHostComponent = inCapturePhase;\n                      lastHostComponent;\n                      lastHostComponent = getParent(lastHostComponent)\n                    )\n                      instance++;\n                    lastHostComponent = 0;\n                    for (\n                      _instance2 = reactEventName;\n                      _instance2;\n                      _instance2 = getParent(_instance2)\n                    )\n                      lastHostComponent++;\n                    for (; 0 < instance - lastHostComponent; )\n                      (inCapturePhase = getParent(inCapturePhase)), instance--;\n                    for (; 0 < lastHostComponent - instance; )\n                      (reactEventName = getParent(reactEventName)),\n                        lastHostComponent--;\n                    for (; instance--; ) {\n                      if (\n                        inCapturePhase === reactEventName ||\n                        (null !== reactEventName &&\n                          inCapturePhase === reactEventName.alternate)\n                      )\n                        break b;\n                      inCapturePhase = getParent(inCapturePhase);\n                      reactEventName = getParent(reactEventName);\n                    }\n                    inCapturePhase = null;\n                  }\n                else inCapturePhase = null;\n                null !== SyntheticEventCtor &&\n                  accumulateEnterLeaveListenersForEvent(\n                    dispatchQueue,\n                    reactName,\n                    SyntheticEventCtor,\n                    inCapturePhase,\n                    !1\n                  );\n                null !== reactEventType &&\n                  null !== accumulateTargetOnly &&\n                  accumulateEnterLeaveListenersForEvent(\n                    dispatchQueue,\n                    accumulateTargetOnly,\n                    reactEventType,\n                    inCapturePhase,\n                    !0\n                  );\n              }\n            }\n          }\n          a: {\n            reactName = targetInst ? getNodeFromInstance(targetInst) : window;\n            SyntheticEventCtor =\n              reactName.nodeName && reactName.nodeName.toLowerCase();\n            if (\n              \"select\" === SyntheticEventCtor ||\n              (\"input\" === SyntheticEventCtor && \"file\" === reactName.type)\n            )\n              var getTargetInstFunc = getTargetInstForChangeEvent;\n            else if (isTextInputElement(reactName))\n              if (isInputEventSupported)\n                getTargetInstFunc = getTargetInstForInputOrChangeEvent;\n              else {\n                getTargetInstFunc = getTargetInstForInputEventPolyfill;\n                var handleEventFunc = handleEventsForInputEventPolyfill;\n              }\n            else\n              (SyntheticEventCtor = reactName.nodeName),\n                !SyntheticEventCtor ||\n                \"input\" !== SyntheticEventCtor.toLowerCase() ||\n                (\"checkbox\" !== reactName.type && \"radio\" !== reactName.type)\n                  ? targetInst &&\n                    isCustomElement(targetInst.elementType) &&\n                    (getTargetInstFunc = getTargetInstForChangeEvent)\n                  : (getTargetInstFunc = getTargetInstForClickEvent);\n            if (\n              getTargetInstFunc &&\n              (getTargetInstFunc = getTargetInstFunc(domEventName, targetInst))\n            ) {\n              createAndAccumulateChangeEvent(\n                dispatchQueue,\n                getTargetInstFunc,\n                nativeEvent,\n                nativeEventTarget\n              );\n              break a;\n            }\n            handleEventFunc &&\n              handleEventFunc(domEventName, reactName, targetInst);\n            \"focusout\" === domEventName &&\n              targetInst &&\n              \"number\" === reactName.type &&\n              null != targetInst.memoizedProps.value &&\n              setDefaultValue(reactName, \"number\", reactName.value);\n          }\n          handleEventFunc = targetInst\n            ? getNodeFromInstance(targetInst)\n            : window;\n          switch (domEventName) {\n            case \"focusin\":\n              if (\n                isTextInputElement(handleEventFunc) ||\n                \"true\" === handleEventFunc.contentEditable\n              )\n                (activeElement = handleEventFunc),\n                  (activeElementInst = targetInst),\n                  (lastSelection = null);\n              break;\n            case \"focusout\":\n              lastSelection = activeElementInst = activeElement = null;\n              break;\n            case \"mousedown\":\n              mouseDown = !0;\n              break;\n            case \"contextmenu\":\n            case \"mouseup\":\n            case \"dragend\":\n              mouseDown = !1;\n              constructSelectEvent(\n                dispatchQueue,\n                nativeEvent,\n                nativeEventTarget\n              );\n              break;\n            case \"selectionchange\":\n              if (skipSelectionChangeEvent) break;\n            case \"keydown\":\n            case \"keyup\":\n              constructSelectEvent(\n                dispatchQueue,\n                nativeEvent,\n                nativeEventTarget\n              );\n          }\n          var fallbackData;\n          if (canUseCompositionEvent)\n            b: {\n              switch (domEventName) {\n                case \"compositionstart\":\n                  var eventType = \"onCompositionStart\";\n                  break b;\n                case \"compositionend\":\n                  eventType = \"onCompositionEnd\";\n                  break b;\n                case \"compositionupdate\":\n                  eventType = \"onCompositionUpdate\";\n                  break b;\n              }\n              eventType = void 0;\n            }\n          else\n            isComposing\n              ? isFallbackCompositionEnd(domEventName, nativeEvent) &&\n                (eventType = \"onCompositionEnd\")\n              : \"keydown\" === domEventName &&\n                nativeEvent.keyCode === START_KEYCODE &&\n                (eventType = \"onCompositionStart\");\n          eventType &&\n            (useFallbackCompositionData &&\n              \"ko\" !== nativeEvent.locale &&\n              (isComposing || \"onCompositionStart\" !== eventType\n                ? \"onCompositionEnd\" === eventType &&\n                  isComposing &&\n                  (fallbackData = getData())\n                : ((root = nativeEventTarget),\n                  (startText = \"value\" in root ? root.value : root.textContent),\n                  (isComposing = !0))),\n            (handleEventFunc = accumulateTwoPhaseListeners(\n              targetInst,\n              eventType\n            )),\n            0 < handleEventFunc.length &&\n              ((eventType = new SyntheticCompositionEvent(\n                eventType,\n                domEventName,\n                null,\n                nativeEvent,\n                nativeEventTarget\n              )),\n              dispatchQueue.push({\n                event: eventType,\n                listeners: handleEventFunc\n              }),\n              fallbackData\n                ? (eventType.data = fallbackData)\n                : ((fallbackData = getDataFromCustomEvent(nativeEvent)),\n                  null !== fallbackData && (eventType.data = fallbackData))));\n          if (\n            (fallbackData = canUseTextInputEvent\n              ? getNativeBeforeInputChars(domEventName, nativeEvent)\n              : getFallbackBeforeInputChars(domEventName, nativeEvent))\n          )\n            (eventType = accumulateTwoPhaseListeners(\n              targetInst,\n              \"onBeforeInput\"\n            )),\n              0 < eventType.length &&\n                ((handleEventFunc = new SyntheticInputEvent(\n                  \"onBeforeInput\",\n                  \"beforeinput\",\n                  null,\n                  nativeEvent,\n                  nativeEventTarget\n                )),\n                dispatchQueue.push({\n                  event: handleEventFunc,\n                  listeners: eventType\n                }),\n                (handleEventFunc.data = fallbackData));\n          extractEvents$1(\n            dispatchQueue,\n            domEventName,\n            targetInst,\n            nativeEvent,\n            nativeEventTarget\n          );\n        }\n        processDispatchQueue(dispatchQueue, eventSystemFlags);\n      });\n    }\n    function createDispatchListener(instance, listener, currentTarget) {\n      return {\n        instance: instance,\n        listener: listener,\n        currentTarget: currentTarget\n      };\n    }\n    function accumulateTwoPhaseListeners(targetFiber, reactName) {\n      for (\n        var captureName = reactName + \"Capture\", listeners = [];\n        null !== targetFiber;\n\n      ) {\n        var _instance3 = targetFiber,\n          stateNode = _instance3.stateNode;\n        _instance3 = _instance3.tag;\n        (5 !== _instance3 && 26 !== _instance3 && 27 !== _instance3) ||\n          null === stateNode ||\n          ((_instance3 = getListener(targetFiber, captureName)),\n          null != _instance3 &&\n            listeners.unshift(\n              createDispatchListener(targetFiber, _instance3, stateNode)\n            ),\n          (_instance3 = getListener(targetFiber, reactName)),\n          null != _instance3 &&\n            listeners.push(\n              createDispatchListener(targetFiber, _instance3, stateNode)\n            ));\n        if (3 === targetFiber.tag) return listeners;\n        targetFiber = targetFiber.return;\n      }\n      return [];\n    }\n    function getParent(inst) {\n      if (null === inst) return null;\n      do inst = inst.return;\n      while (inst && 5 !== inst.tag && 27 !== inst.tag);\n      return inst ? inst : null;\n    }\n    function accumulateEnterLeaveListenersForEvent(\n      dispatchQueue,\n      event,\n      target,\n      common,\n      inCapturePhase\n    ) {\n      for (\n        var registrationName = event._reactName, listeners = [];\n        null !== target && target !== common;\n\n      ) {\n        var _instance4 = target,\n          alternate = _instance4.alternate,\n          stateNode = _instance4.stateNode;\n        _instance4 = _instance4.tag;\n        if (null !== alternate && alternate === common) break;\n        (5 !== _instance4 && 26 !== _instance4 && 27 !== _instance4) ||\n          null === stateNode ||\n          ((alternate = stateNode),\n          inCapturePhase\n            ? ((stateNode = getListener(target, registrationName)),\n              null != stateNode &&\n                listeners.unshift(\n                  createDispatchListener(target, stateNode, alternate)\n                ))\n            : inCapturePhase ||\n              ((stateNode = getListener(target, registrationName)),\n              null != stateNode &&\n                listeners.push(\n                  createDispatchListener(target, stateNode, alternate)\n                )));\n        target = target.return;\n      }\n      0 !== listeners.length &&\n        dispatchQueue.push({ event: event, listeners: listeners });\n    }\n    function validatePropertiesInDevelopment(type, props) {\n      validateProperties$2(type, props);\n      (\"input\" !== type && \"textarea\" !== type && \"select\" !== type) ||\n        null == props ||\n        null !== props.value ||\n        didWarnValueNull ||\n        ((didWarnValueNull = !0),\n        \"select\" === type && props.multiple\n          ? console.error(\n              \"`value` prop on `%s` should not be null. Consider using an empty array when `multiple` is set to `true` to clear the component or `undefined` for uncontrolled components.\",\n              type\n            )\n          : console.error(\n              \"`value` prop on `%s` should not be null. Consider using an empty string to clear the component or `undefined` for uncontrolled components.\",\n              type\n            ));\n      var eventRegistry = {\n        registrationNameDependencies: registrationNameDependencies,\n        possibleRegistrationNames: possibleRegistrationNames\n      };\n      isCustomElement(type) ||\n        \"string\" === typeof props.is ||\n        warnUnknownProperties(type, props, eventRegistry);\n      props.contentEditable &&\n        !props.suppressContentEditableWarning &&\n        null != props.children &&\n        console.error(\n          \"A component is `contentEditable` and contains `children` managed by React. It is now your responsibility to guarantee that none of those nodes are unexpectedly modified or duplicated. This is probably not intentional.\"\n        );\n    }\n    function warnForPropDifference(\n      propName,\n      serverValue,\n      clientValue,\n      serverDifferences\n    ) {\n      serverValue !== clientValue &&\n        ((clientValue = normalizeMarkupForTextOrAttribute(clientValue)),\n        normalizeMarkupForTextOrAttribute(serverValue) !== clientValue &&\n          (serverDifferences[propName] = serverValue));\n    }\n    function warnForExtraAttributes(\n      domElement,\n      attributeNames,\n      serverDifferences\n    ) {\n      attributeNames.forEach(function (attributeName) {\n        serverDifferences[getPropNameFromAttributeName(attributeName)] =\n          \"style\" === attributeName\n            ? getStylesObjectFromElement(domElement)\n            : domElement.getAttribute(attributeName);\n      });\n    }\n    function warnForInvalidEventListener(registrationName, listener) {\n      !1 === listener\n        ? console.error(\n            \"Expected `%s` listener to be a function, instead got `false`.\\n\\nIf you used to conditionally omit it with %s={condition && value}, pass %s={condition ? value : undefined} instead.\",\n            registrationName,\n            registrationName,\n            registrationName\n          )\n        : console.error(\n            \"Expected `%s` listener to be a function, instead got a value of `%s` type.\",\n            registrationName,\n            typeof listener\n          );\n    }\n    function normalizeHTML(parent, html) {\n      parent =\n        parent.namespaceURI === MATH_NAMESPACE ||\n        parent.namespaceURI === SVG_NAMESPACE\n          ? parent.ownerDocument.createElementNS(\n              parent.namespaceURI,\n              parent.tagName\n            )\n          : parent.ownerDocument.createElement(parent.tagName);\n      parent.innerHTML = html;\n      return parent.innerHTML;\n    }\n    function normalizeMarkupForTextOrAttribute(markup) {\n      willCoercionThrow(markup) &&\n        (console.error(\n          \"The provided HTML markup uses a value of unsupported type %s. This value must be coerced to a string before using it here.\",\n          typeName(markup)\n        ),\n        testStringCoercion(markup));\n      return (\"string\" === typeof markup ? markup : \"\" + markup)\n        .replace(NORMALIZE_NEWLINES_REGEX, \"\\n\")\n        .replace(NORMALIZE_NULL_AND_REPLACEMENT_REGEX, \"\");\n    }\n    function checkForUnmatchedText(serverText, clientText) {\n      clientText = normalizeMarkupForTextOrAttribute(clientText);\n      return normalizeMarkupForTextOrAttribute(serverText) === clientText\n        ? !0\n        : !1;\n    }\n    function noop$1() {}\n    function setProp(domElement, tag, key, value, props, prevValue) {\n      switch (key) {\n        case \"children\":\n          if (\"string\" === typeof value)\n            validateTextNesting(value, tag, !1),\n              \"body\" === tag ||\n                (\"textarea\" === tag && \"\" === value) ||\n                setTextContent(domElement, value);\n          else if (\"number\" === typeof value || \"bigint\" === typeof value)\n            validateTextNesting(\"\" + value, tag, !1),\n              \"body\" !== tag && setTextContent(domElement, \"\" + value);\n          break;\n        case \"className\":\n          setValueForKnownAttribute(domElement, \"class\", value);\n          break;\n        case \"tabIndex\":\n          setValueForKnownAttribute(domElement, \"tabindex\", value);\n          break;\n        case \"dir\":\n        case \"role\":\n        case \"viewBox\":\n        case \"width\":\n        case \"height\":\n          setValueForKnownAttribute(domElement, key, value);\n          break;\n        case \"style\":\n          setValueForStyles(domElement, value, prevValue);\n          break;\n        case \"data\":\n          if (\"object\" !== tag) {\n            setValueForKnownAttribute(domElement, \"data\", value);\n            break;\n          }\n        case \"src\":\n        case \"href\":\n          if (\"\" === value && (\"a\" !== tag || \"href\" !== key)) {\n            \"src\" === key\n              ? console.error(\n                  'An empty string (\"\") was passed to the %s attribute. This may cause the browser to download the whole page again over the network. To fix this, either do not render the element at all or pass null to %s instead of an empty string.',\n                  key,\n                  key\n                )\n              : console.error(\n                  'An empty string (\"\") was passed to the %s attribute. To fix this, either do not render the element at all or pass null to %s instead of an empty string.',\n                  key,\n                  key\n                );\n            domElement.removeAttribute(key);\n            break;\n          }\n          if (\n            null == value ||\n            \"function\" === typeof value ||\n            \"symbol\" === typeof value ||\n            \"boolean\" === typeof value\n          ) {\n            domElement.removeAttribute(key);\n            break;\n          }\n          checkAttributeStringCoercion(value, key);\n          value = sanitizeURL(\"\" + value);\n          domElement.setAttribute(key, value);\n          break;\n        case \"action\":\n        case \"formAction\":\n          null != value &&\n            (\"form\" === tag\n              ? \"formAction\" === key\n                ? console.error(\n                    \"You can only pass the formAction prop to <input> or <button>. Use the action prop on <form>.\"\n                  )\n                : \"function\" === typeof value &&\n                  ((null == props.encType && null == props.method) ||\n                    didWarnFormActionMethod ||\n                    ((didWarnFormActionMethod = !0),\n                    console.error(\n                      \"Cannot specify a encType or method for a form that specifies a function as the action. React provides those automatically. They will get overridden.\"\n                    )),\n                  null == props.target ||\n                    didWarnFormActionTarget ||\n                    ((didWarnFormActionTarget = !0),\n                    console.error(\n                      \"Cannot specify a target for a form that specifies a function as the action. The function will always be executed in the same window.\"\n                    )))\n              : \"input\" === tag || \"button\" === tag\n                ? \"action\" === key\n                  ? console.error(\n                      \"You can only pass the action prop to <form>. Use the formAction prop on <input> or <button>.\"\n                    )\n                  : \"input\" !== tag ||\n                      \"submit\" === props.type ||\n                      \"image\" === props.type ||\n                      didWarnFormActionType\n                    ? \"button\" !== tag ||\n                      null == props.type ||\n                      \"submit\" === props.type ||\n                      didWarnFormActionType\n                      ? \"function\" === typeof value &&\n                        (null == props.name ||\n                          didWarnFormActionName ||\n                          ((didWarnFormActionName = !0),\n                          console.error(\n                            'Cannot specify a \"name\" prop for a button that specifies a function as a formAction. React needs it to encode which action should be invoked. It will get overridden.'\n                          )),\n                        (null == props.formEncType &&\n                          null == props.formMethod) ||\n                          didWarnFormActionMethod ||\n                          ((didWarnFormActionMethod = !0),\n                          console.error(\n                            \"Cannot specify a formEncType or formMethod for a button that specifies a function as a formAction. React provides those automatically. They will get overridden.\"\n                          )),\n                        null == props.formTarget ||\n                          didWarnFormActionTarget ||\n                          ((didWarnFormActionTarget = !0),\n                          console.error(\n                            \"Cannot specify a formTarget for a button that specifies a function as a formAction. The function will always be executed in the same window.\"\n                          )))\n                      : ((didWarnFormActionType = !0),\n                        console.error(\n                          'A button can only specify a formAction along with type=\"submit\" or no type.'\n                        ))\n                    : ((didWarnFormActionType = !0),\n                      console.error(\n                        'An input can only specify a formAction along with type=\"submit\" or type=\"image\".'\n                      ))\n                : \"action\" === key\n                  ? console.error(\n                      \"You can only pass the action prop to <form>.\"\n                    )\n                  : console.error(\n                      \"You can only pass the formAction prop to <input> or <button>.\"\n                    ));\n          if (\"function\" === typeof value) {\n            domElement.setAttribute(\n              key,\n              \"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')\"\n            );\n            break;\n          } else\n            \"function\" === typeof prevValue &&\n              (\"formAction\" === key\n                ? (\"input\" !== tag &&\n                    setProp(domElement, tag, \"name\", props.name, props, null),\n                  setProp(\n                    domElement,\n                    tag,\n                    \"formEncType\",\n                    props.formEncType,\n                    props,\n                    null\n                  ),\n                  setProp(\n                    domElement,\n                    tag,\n                    \"formMethod\",\n                    props.formMethod,\n                    props,\n                    null\n                  ),\n                  setProp(\n                    domElement,\n                    tag,\n                    \"formTarget\",\n                    props.formTarget,\n                    props,\n                    null\n                  ))\n                : (setProp(\n                    domElement,\n                    tag,\n                    \"encType\",\n                    props.encType,\n                    props,\n                    null\n                  ),\n                  setProp(domElement, tag, \"method\", props.method, props, null),\n                  setProp(\n                    domElement,\n                    tag,\n                    \"target\",\n                    props.target,\n                    props,\n                    null\n                  )));\n          if (\n            null == value ||\n            \"symbol\" === typeof value ||\n            \"boolean\" === typeof value\n          ) {\n            domElement.removeAttribute(key);\n            break;\n          }\n          checkAttributeStringCoercion(value, key);\n          value = sanitizeURL(\"\" + value);\n          domElement.setAttribute(key, value);\n          break;\n        case \"onClick\":\n          null != value &&\n            (\"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value),\n            (domElement.onclick = noop$1));\n          break;\n        case \"onScroll\":\n          null != value &&\n            (\"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value),\n            listenToNonDelegatedEvent(\"scroll\", domElement));\n          break;\n        case \"onScrollEnd\":\n          null != value &&\n            (\"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value),\n            listenToNonDelegatedEvent(\"scrollend\", domElement));\n          break;\n        case \"dangerouslySetInnerHTML\":\n          if (null != value) {\n            if (\"object\" !== typeof value || !(\"__html\" in value))\n              throw Error(\n                \"`props.dangerouslySetInnerHTML` must be in the form `{__html: ...}`. Please visit https://react.dev/link/dangerously-set-inner-html for more information.\"\n              );\n            key = value.__html;\n            if (null != key) {\n              if (null != props.children)\n                throw Error(\n                  \"Can only set one of `children` or `props.dangerouslySetInnerHTML`.\"\n                );\n              domElement.innerHTML = key;\n            }\n          }\n          break;\n        case \"multiple\":\n          domElement.multiple =\n            value && \"function\" !== typeof value && \"symbol\" !== typeof value;\n          break;\n        case \"muted\":\n          domElement.muted =\n            value && \"function\" !== typeof value && \"symbol\" !== typeof value;\n          break;\n        case \"suppressContentEditableWarning\":\n        case \"suppressHydrationWarning\":\n        case \"defaultValue\":\n        case \"defaultChecked\":\n        case \"innerHTML\":\n        case \"ref\":\n          break;\n        case \"autoFocus\":\n          break;\n        case \"xlinkHref\":\n          if (\n            null == value ||\n            \"function\" === typeof value ||\n            \"boolean\" === typeof value ||\n            \"symbol\" === typeof value\n          ) {\n            domElement.removeAttribute(\"xlink:href\");\n            break;\n          }\n          checkAttributeStringCoercion(value, key);\n          key = sanitizeURL(\"\" + value);\n          domElement.setAttributeNS(xlinkNamespace, \"xlink:href\", key);\n          break;\n        case \"contentEditable\":\n        case \"spellCheck\":\n        case \"draggable\":\n        case \"value\":\n        case \"autoReverse\":\n        case \"externalResourcesRequired\":\n        case \"focusable\":\n        case \"preserveAlpha\":\n          null != value &&\n          \"function\" !== typeof value &&\n          \"symbol\" !== typeof value\n            ? (checkAttributeStringCoercion(value, key),\n              domElement.setAttribute(key, \"\" + value))\n            : domElement.removeAttribute(key);\n          break;\n        case \"inert\":\n          \"\" !== value ||\n            didWarnForNewBooleanPropsWithEmptyValue[key] ||\n            ((didWarnForNewBooleanPropsWithEmptyValue[key] = !0),\n            console.error(\n              \"Received an empty string for a boolean attribute `%s`. This will treat the attribute as if it were false. Either pass `false` to silence this warning, or pass `true` if you used an empty string in earlier versions of React to indicate this attribute is true.\",\n              key\n            ));\n        case \"allowFullScreen\":\n        case \"async\":\n        case \"autoPlay\":\n        case \"controls\":\n        case \"default\":\n        case \"defer\":\n        case \"disabled\":\n        case \"disablePictureInPicture\":\n        case \"disableRemotePlayback\":\n        case \"formNoValidate\":\n        case \"hidden\":\n        case \"loop\":\n        case \"noModule\":\n        case \"noValidate\":\n        case \"open\":\n        case \"playsInline\":\n        case \"readOnly\":\n        case \"required\":\n        case \"reversed\":\n        case \"scoped\":\n        case \"seamless\":\n        case \"itemScope\":\n          value && \"function\" !== typeof value && \"symbol\" !== typeof value\n            ? domElement.setAttribute(key, \"\")\n            : domElement.removeAttribute(key);\n          break;\n        case \"capture\":\n        case \"download\":\n          !0 === value\n            ? domElement.setAttribute(key, \"\")\n            : !1 !== value &&\n                null != value &&\n                \"function\" !== typeof value &&\n                \"symbol\" !== typeof value\n              ? (checkAttributeStringCoercion(value, key),\n                domElement.setAttribute(key, value))\n              : domElement.removeAttribute(key);\n          break;\n        case \"cols\":\n        case \"rows\":\n        case \"size\":\n        case \"span\":\n          null != value &&\n          \"function\" !== typeof value &&\n          \"symbol\" !== typeof value &&\n          !isNaN(value) &&\n          1 <= value\n            ? (checkAttributeStringCoercion(value, key),\n              domElement.setAttribute(key, value))\n            : domElement.removeAttribute(key);\n          break;\n        case \"rowSpan\":\n        case \"start\":\n          null == value ||\n          \"function\" === typeof value ||\n          \"symbol\" === typeof value ||\n          isNaN(value)\n            ? domElement.removeAttribute(key)\n            : (checkAttributeStringCoercion(value, key),\n              domElement.setAttribute(key, value));\n          break;\n        case \"popover\":\n          listenToNonDelegatedEvent(\"beforetoggle\", domElement);\n          listenToNonDelegatedEvent(\"toggle\", domElement);\n          setValueForAttribute(domElement, \"popover\", value);\n          break;\n        case \"xlinkActuate\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xlinkNamespace,\n            \"xlink:actuate\",\n            value\n          );\n          break;\n        case \"xlinkArcrole\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xlinkNamespace,\n            \"xlink:arcrole\",\n            value\n          );\n          break;\n        case \"xlinkRole\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xlinkNamespace,\n            \"xlink:role\",\n            value\n          );\n          break;\n        case \"xlinkShow\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xlinkNamespace,\n            \"xlink:show\",\n            value\n          );\n          break;\n        case \"xlinkTitle\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xlinkNamespace,\n            \"xlink:title\",\n            value\n          );\n          break;\n        case \"xlinkType\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xlinkNamespace,\n            \"xlink:type\",\n            value\n          );\n          break;\n        case \"xmlBase\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xmlNamespace,\n            \"xml:base\",\n            value\n          );\n          break;\n        case \"xmlLang\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xmlNamespace,\n            \"xml:lang\",\n            value\n          );\n          break;\n        case \"xmlSpace\":\n          setValueForNamespacedAttribute(\n            domElement,\n            xmlNamespace,\n            \"xml:space\",\n            value\n          );\n          break;\n        case \"is\":\n          null != prevValue &&\n            console.error(\n              'Cannot update the \"is\" prop after it has been initialized.'\n            );\n          setValueForAttribute(domElement, \"is\", value);\n          break;\n        case \"innerText\":\n        case \"textContent\":\n          break;\n        case \"popoverTarget\":\n          didWarnPopoverTargetObject ||\n            null == value ||\n            \"object\" !== typeof value ||\n            ((didWarnPopoverTargetObject = !0),\n            console.error(\n              \"The `popoverTarget` prop expects the ID of an Element as a string. Received %s instead.\",\n              value\n            ));\n        default:\n          !(2 < key.length) ||\n          (\"o\" !== key[0] && \"O\" !== key[0]) ||\n          (\"n\" !== key[1] && \"N\" !== key[1])\n            ? ((key = getAttributeAlias(key)),\n              setValueForAttribute(domElement, key, value))\n            : registrationNameDependencies.hasOwnProperty(key) &&\n              null != value &&\n              \"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value);\n      }\n    }\n    function setPropOnCustomElement(\n      domElement,\n      tag,\n      key,\n      value,\n      props,\n      prevValue\n    ) {\n      switch (key) {\n        case \"style\":\n          setValueForStyles(domElement, value, prevValue);\n          break;\n        case \"dangerouslySetInnerHTML\":\n          if (null != value) {\n            if (\"object\" !== typeof value || !(\"__html\" in value))\n              throw Error(\n                \"`props.dangerouslySetInnerHTML` must be in the form `{__html: ...}`. Please visit https://react.dev/link/dangerously-set-inner-html for more information.\"\n              );\n            key = value.__html;\n            if (null != key) {\n              if (null != props.children)\n                throw Error(\n                  \"Can only set one of `children` or `props.dangerouslySetInnerHTML`.\"\n                );\n              domElement.innerHTML = key;\n            }\n          }\n          break;\n        case \"children\":\n          \"string\" === typeof value\n            ? setTextContent(domElement, value)\n            : (\"number\" === typeof value || \"bigint\" === typeof value) &&\n              setTextContent(domElement, \"\" + value);\n          break;\n        case \"onScroll\":\n          null != value &&\n            (\"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value),\n            listenToNonDelegatedEvent(\"scroll\", domElement));\n          break;\n        case \"onScrollEnd\":\n          null != value &&\n            (\"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value),\n            listenToNonDelegatedEvent(\"scrollend\", domElement));\n          break;\n        case \"onClick\":\n          null != value &&\n            (\"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value),\n            (domElement.onclick = noop$1));\n          break;\n        case \"suppressContentEditableWarning\":\n        case \"suppressHydrationWarning\":\n        case \"innerHTML\":\n        case \"ref\":\n          break;\n        case \"innerText\":\n        case \"textContent\":\n          break;\n        default:\n          if (registrationNameDependencies.hasOwnProperty(key))\n            null != value &&\n              \"function\" !== typeof value &&\n              warnForInvalidEventListener(key, value);\n          else\n            a: {\n              if (\n                \"o\" === key[0] &&\n                \"n\" === key[1] &&\n                ((props = key.endsWith(\"Capture\")),\n                (tag = key.slice(2, props ? key.length - 7 : void 0)),\n                (prevValue = domElement[internalPropsKey] || null),\n                (prevValue = null != prevValue ? prevValue[key] : null),\n                \"function\" === typeof prevValue &&\n                  domElement.removeEventListener(tag, prevValue, props),\n                \"function\" === typeof value)\n              ) {\n                \"function\" !== typeof prevValue &&\n                  null !== prevValue &&\n                  (key in domElement\n                    ? (domElement[key] = null)\n                    : domElement.hasAttribute(key) &&\n                      domElement.removeAttribute(key));\n                domElement.addEventListener(tag, value, props);\n                break a;\n              }\n              key in domElement\n                ? (domElement[key] = value)\n                : !0 === value\n                  ? domElement.setAttribute(key, \"\")\n                  : setValueForAttribute(domElement, key, value);\n            }\n      }\n    }\n    function setInitialProperties(domElement, tag, props) {\n      validatePropertiesInDevelopment(tag, props);\n      switch (tag) {\n        case \"div\":\n        case \"span\":\n        case \"svg\":\n        case \"path\":\n        case \"a\":\n        case \"g\":\n        case \"p\":\n        case \"li\":\n          break;\n        case \"img\":\n          listenToNonDelegatedEvent(\"error\", domElement);\n          listenToNonDelegatedEvent(\"load\", domElement);\n          var hasSrc = !1,\n            hasSrcSet = !1,\n            propKey;\n          for (propKey in props)\n            if (props.hasOwnProperty(propKey)) {\n              var propValue = props[propKey];\n              if (null != propValue)\n                switch (propKey) {\n                  case \"src\":\n                    hasSrc = !0;\n                    break;\n                  case \"srcSet\":\n                    hasSrcSet = !0;\n                    break;\n                  case \"children\":\n                  case \"dangerouslySetInnerHTML\":\n                    throw Error(\n                      tag +\n                        \" is a void element tag and must neither have `children` nor use `dangerouslySetInnerHTML`.\"\n                    );\n                  default:\n                    setProp(domElement, tag, propKey, propValue, props, null);\n                }\n            }\n          hasSrcSet &&\n            setProp(domElement, tag, \"srcSet\", props.srcSet, props, null);\n          hasSrc && setProp(domElement, tag, \"src\", props.src, props, null);\n          return;\n        case \"input\":\n          checkControlledValueProps(\"input\", props);\n          listenToNonDelegatedEvent(\"invalid\", domElement);\n          var defaultValue = (propKey = propValue = hasSrcSet = null),\n            checked = null,\n            defaultChecked = null;\n          for (hasSrc in props)\n            if (props.hasOwnProperty(hasSrc)) {\n              var _propValue = props[hasSrc];\n              if (null != _propValue)\n                switch (hasSrc) {\n                  case \"name\":\n                    hasSrcSet = _propValue;\n                    break;\n                  case \"type\":\n                    propValue = _propValue;\n                    break;\n                  case \"checked\":\n                    checked = _propValue;\n                    break;\n                  case \"defaultChecked\":\n                    defaultChecked = _propValue;\n                    break;\n                  case \"value\":\n                    propKey = _propValue;\n                    break;\n                  case \"defaultValue\":\n                    defaultValue = _propValue;\n                    break;\n                  case \"children\":\n                  case \"dangerouslySetInnerHTML\":\n                    if (null != _propValue)\n                      throw Error(\n                        tag +\n                          \" is a void element tag and must neither have `children` nor use `dangerouslySetInnerHTML`.\"\n                      );\n                    break;\n                  default:\n                    setProp(domElement, tag, hasSrc, _propValue, props, null);\n                }\n            }\n          validateInputProps(domElement, props);\n          initInput(\n            domElement,\n            propKey,\n            defaultValue,\n            checked,\n            defaultChecked,\n            propValue,\n            hasSrcSet,\n            !1\n          );\n          track(domElement);\n          return;\n        case \"select\":\n          checkControlledValueProps(\"select\", props);\n          listenToNonDelegatedEvent(\"invalid\", domElement);\n          hasSrc = propValue = propKey = null;\n          for (hasSrcSet in props)\n            if (\n              props.hasOwnProperty(hasSrcSet) &&\n              ((defaultValue = props[hasSrcSet]), null != defaultValue)\n            )\n              switch (hasSrcSet) {\n                case \"value\":\n                  propKey = defaultValue;\n                  break;\n                case \"defaultValue\":\n                  propValue = defaultValue;\n                  break;\n                case \"multiple\":\n                  hasSrc = defaultValue;\n                default:\n                  setProp(\n                    domElement,\n                    tag,\n                    hasSrcSet,\n                    defaultValue,\n                    props,\n                    null\n                  );\n              }\n          validateSelectProps(domElement, props);\n          tag = propKey;\n          props = propValue;\n          domElement.multiple = !!hasSrc;\n          null != tag\n            ? updateOptions(domElement, !!hasSrc, tag, !1)\n            : null != props && updateOptions(domElement, !!hasSrc, props, !0);\n          return;\n        case \"textarea\":\n          checkControlledValueProps(\"textarea\", props);\n          listenToNonDelegatedEvent(\"invalid\", domElement);\n          propKey = hasSrcSet = hasSrc = null;\n          for (propValue in props)\n            if (\n              props.hasOwnProperty(propValue) &&\n              ((defaultValue = props[propValue]), null != defaultValue)\n            )\n              switch (propValue) {\n                case \"value\":\n                  hasSrc = defaultValue;\n                  break;\n                case \"defaultValue\":\n                  hasSrcSet = defaultValue;\n                  break;\n                case \"children\":\n                  propKey = defaultValue;\n                  break;\n                case \"dangerouslySetInnerHTML\":\n                  if (null != defaultValue)\n                    throw Error(\n                      \"`dangerouslySetInnerHTML` does not make sense on <textarea>.\"\n                    );\n                  break;\n                default:\n                  setProp(\n                    domElement,\n                    tag,\n                    propValue,\n                    defaultValue,\n                    props,\n                    null\n                  );\n              }\n          validateTextareaProps(domElement, props);\n          initTextarea(domElement, hasSrc, hasSrcSet, propKey);\n          track(domElement);\n          return;\n        case \"option\":\n          validateOptionProps(domElement, props);\n          for (checked in props)\n            if (\n              props.hasOwnProperty(checked) &&\n              ((hasSrc = props[checked]), null != hasSrc)\n            )\n              switch (checked) {\n                case \"selected\":\n                  domElement.selected =\n                    hasSrc &&\n                    \"function\" !== typeof hasSrc &&\n                    \"symbol\" !== typeof hasSrc;\n                  break;\n                default:\n                  setProp(domElement, tag, checked, hasSrc, props, null);\n              }\n          return;\n        case \"dialog\":\n          listenToNonDelegatedEvent(\"beforetoggle\", domElement);\n          listenToNonDelegatedEvent(\"toggle\", domElement);\n          listenToNonDelegatedEvent(\"cancel\", domElement);\n          listenToNonDelegatedEvent(\"close\", domElement);\n          break;\n        case \"iframe\":\n        case \"object\":\n          listenToNonDelegatedEvent(\"load\", domElement);\n          break;\n        case \"video\":\n        case \"audio\":\n          for (hasSrc = 0; hasSrc < mediaEventTypes.length; hasSrc++)\n            listenToNonDelegatedEvent(mediaEventTypes[hasSrc], domElement);\n          break;\n        case \"image\":\n          listenToNonDelegatedEvent(\"error\", domElement);\n          listenToNonDelegatedEvent(\"load\", domElement);\n          break;\n        case \"details\":\n          listenToNonDelegatedEvent(\"toggle\", domElement);\n          break;\n        case \"embed\":\n        case \"source\":\n        case \"link\":\n          listenToNonDelegatedEvent(\"error\", domElement),\n            listenToNonDelegatedEvent(\"load\", domElement);\n        case \"area\":\n        case \"base\":\n        case \"br\":\n        case \"col\":\n        case \"hr\":\n        case \"keygen\":\n        case \"meta\":\n        case \"param\":\n        case \"track\":\n        case \"wbr\":\n        case \"menuitem\":\n          for (defaultChecked in props)\n            if (\n              props.hasOwnProperty(defaultChecked) &&\n              ((hasSrc = props[defaultChecked]), null != hasSrc)\n            )\n              switch (defaultChecked) {\n                case \"children\":\n                case \"dangerouslySetInnerHTML\":\n                  throw Error(\n                    tag +\n                      \" is a void element tag and must neither have `children` nor use `dangerouslySetInnerHTML`.\"\n                  );\n                default:\n                  setProp(domElement, tag, defaultChecked, hasSrc, props, null);\n              }\n          return;\n        default:\n          if (isCustomElement(tag)) {\n            for (_propValue in props)\n              props.hasOwnProperty(_propValue) &&\n                ((hasSrc = props[_propValue]),\n                void 0 !== hasSrc &&\n                  setPropOnCustomElement(\n                    domElement,\n                    tag,\n                    _propValue,\n                    hasSrc,\n                    props,\n                    void 0\n                  ));\n            return;\n          }\n      }\n      for (defaultValue in props)\n        props.hasOwnProperty(defaultValue) &&\n          ((hasSrc = props[defaultValue]),\n          null != hasSrc &&\n            setProp(domElement, tag, defaultValue, hasSrc, props, null));\n    }\n    function updateProperties(domElement, tag, lastProps, nextProps) {\n      validatePropertiesInDevelopment(tag, nextProps);\n      switch (tag) {\n        case \"div\":\n        case \"span\":\n        case \"svg\":\n        case \"path\":\n        case \"a\":\n        case \"g\":\n        case \"p\":\n        case \"li\":\n          break;\n        case \"input\":\n          var name = null,\n            type = null,\n            value = null,\n            defaultValue = null,\n            lastDefaultValue = null,\n            checked = null,\n            defaultChecked = null;\n          for (propKey in lastProps) {\n            var lastProp = lastProps[propKey];\n            if (lastProps.hasOwnProperty(propKey) && null != lastProp)\n              switch (propKey) {\n                case \"checked\":\n                  break;\n                case \"value\":\n                  break;\n                case \"defaultValue\":\n                  lastDefaultValue = lastProp;\n                default:\n                  nextProps.hasOwnProperty(propKey) ||\n                    setProp(\n                      domElement,\n                      tag,\n                      propKey,\n                      null,\n                      nextProps,\n                      lastProp\n                    );\n              }\n          }\n          for (var _propKey8 in nextProps) {\n            var propKey = nextProps[_propKey8];\n            lastProp = lastProps[_propKey8];\n            if (\n              nextProps.hasOwnProperty(_propKey8) &&\n              (null != propKey || null != lastProp)\n            )\n              switch (_propKey8) {\n                case \"type\":\n                  type = propKey;\n                  break;\n                case \"name\":\n                  name = propKey;\n                  break;\n                case \"checked\":\n                  checked = propKey;\n                  break;\n                case \"defaultChecked\":\n                  defaultChecked = propKey;\n                  break;\n                case \"value\":\n                  value = propKey;\n                  break;\n                case \"defaultValue\":\n                  defaultValue = propKey;\n                  break;\n                case \"children\":\n                case \"dangerouslySetInnerHTML\":\n                  if (null != propKey)\n                    throw Error(\n                      tag +\n                        \" is a void element tag and must neither have `children` nor use `dangerouslySetInnerHTML`.\"\n                    );\n                  break;\n                default:\n                  propKey !== lastProp &&\n                    setProp(\n                      domElement,\n                      tag,\n                      _propKey8,\n                      propKey,\n                      nextProps,\n                      lastProp\n                    );\n              }\n          }\n          tag =\n            \"checkbox\" === lastProps.type || \"radio\" === lastProps.type\n              ? null != lastProps.checked\n              : null != lastProps.value;\n          nextProps =\n            \"checkbox\" === nextProps.type || \"radio\" === nextProps.type\n              ? null != nextProps.checked\n              : null != nextProps.value;\n          tag ||\n            !nextProps ||\n            didWarnUncontrolledToControlled ||\n            (console.error(\n              \"A component is changing an uncontrolled input to be controlled. This is likely caused by the value changing from undefined to a defined value, which should not happen. Decide between using a controlled or uncontrolled input element for the lifetime of the component. More info: https://react.dev/link/controlled-components\"\n            ),\n            (didWarnUncontrolledToControlled = !0));\n          !tag ||\n            nextProps ||\n            didWarnControlledToUncontrolled ||\n            (console.error(\n              \"A component is changing a controlled input to be uncontrolled. This is likely caused by the value changing from a defined to undefined, which should not happen. Decide between using a controlled or uncontrolled input element for the lifetime of the component. More info: https://react.dev/link/controlled-components\"\n            ),\n            (didWarnControlledToUncontrolled = !0));\n          updateInput(\n            domElement,\n            value,\n            defaultValue,\n            lastDefaultValue,\n            checked,\n            defaultChecked,\n            type,\n            name\n          );\n          return;\n        case \"select\":\n          propKey = value = defaultValue = _propKey8 = null;\n          for (type in lastProps)\n            if (\n              ((lastDefaultValue = lastProps[type]),\n              lastProps.hasOwnProperty(type) && null != lastDefaultValue)\n            )\n              switch (type) {\n                case \"value\":\n                  break;\n                case \"multiple\":\n                  propKey = lastDefaultValue;\n                default:\n                  nextProps.hasOwnProperty(type) ||\n                    setProp(\n                      domElement,\n                      tag,\n                      type,\n                      null,\n                      nextProps,\n                      lastDefaultValue\n                    );\n              }\n          for (name in nextProps)\n            if (\n              ((type = nextProps[name]),\n              (lastDefaultValue = lastProps[name]),\n              nextProps.hasOwnProperty(name) &&\n                (null != type || null != lastDefaultValue))\n            )\n              switch (name) {\n                case \"value\":\n                  _propKey8 = type;\n                  break;\n                case \"defaultValue\":\n                  defaultValue = type;\n                  break;\n                case \"multiple\":\n                  value = type;\n                default:\n                  type !== lastDefaultValue &&\n                    setProp(\n                      domElement,\n                      tag,\n                      name,\n                      type,\n                      nextProps,\n                      lastDefaultValue\n                    );\n              }\n          nextProps = defaultValue;\n          tag = value;\n          lastProps = propKey;\n          null != _propKey8\n            ? updateOptions(domElement, !!tag, _propKey8, !1)\n            : !!lastProps !== !!tag &&\n              (null != nextProps\n                ? updateOptions(domElement, !!tag, nextProps, !0)\n                : updateOptions(domElement, !!tag, tag ? [] : \"\", !1));\n          return;\n        case \"textarea\":\n          propKey = _propKey8 = null;\n          for (defaultValue in lastProps)\n            if (\n              ((name = lastProps[defaultValue]),\n              lastProps.hasOwnProperty(defaultValue) &&\n                null != name &&\n                !nextProps.hasOwnProperty(defaultValue))\n            )\n              switch (defaultValue) {\n                case \"value\":\n                  break;\n                case \"children\":\n                  break;\n                default:\n                  setProp(domElement, tag, defaultValue, null, nextProps, name);\n              }\n          for (value in nextProps)\n            if (\n              ((name = nextProps[value]),\n              (type = lastProps[value]),\n              nextProps.hasOwnProperty(value) && (null != name || null != type))\n            )\n              switch (value) {\n                case \"value\":\n                  _propKey8 = name;\n                  break;\n                case \"defaultValue\":\n                  propKey = name;\n                  break;\n                case \"children\":\n                  break;\n                case \"dangerouslySetInnerHTML\":\n                  if (null != name)\n                    throw Error(\n                      \"`dangerouslySetInnerHTML` does not make sense on <textarea>.\"\n                    );\n                  break;\n                default:\n                  name !== type &&\n                    setProp(domElement, tag, value, name, nextProps, type);\n              }\n          updateTextarea(domElement, _propKey8, propKey);\n          return;\n        case \"option\":\n          for (var _propKey13 in lastProps)\n            if (\n              ((_propKey8 = lastProps[_propKey13]),\n              lastProps.hasOwnProperty(_propKey13) &&\n                null != _propKey8 &&\n                !nextProps.hasOwnProperty(_propKey13))\n            )\n              switch (_propKey13) {\n                case \"selected\":\n                  domElement.selected = !1;\n                  break;\n                default:\n                  setProp(\n                    domElement,\n                    tag,\n                    _propKey13,\n                    null,\n                    nextProps,\n                    _propKey8\n                  );\n              }\n          for (lastDefaultValue in nextProps)\n            if (\n              ((_propKey8 = nextProps[lastDefaultValue]),\n              (propKey = lastProps[lastDefaultValue]),\n              nextProps.hasOwnProperty(lastDefaultValue) &&\n                _propKey8 !== propKey &&\n                (null != _propKey8 || null != propKey))\n            )\n              switch (lastDefaultValue) {\n                case \"selected\":\n                  domElement.selected =\n                    _propKey8 &&\n                    \"function\" !== typeof _propKey8 &&\n                    \"symbol\" !== typeof _propKey8;\n                  break;\n                default:\n                  setProp(\n                    domElement,\n                    tag,\n                    lastDefaultValue,\n                    _propKey8,\n                    nextProps,\n                    propKey\n                  );\n              }\n          return;\n        case \"img\":\n        case \"link\":\n        case \"area\":\n        case \"base\":\n        case \"br\":\n        case \"col\":\n        case \"embed\":\n        case \"hr\":\n        case \"keygen\":\n        case \"meta\":\n        case \"param\":\n        case \"source\":\n        case \"track\":\n        case \"wbr\":\n        case \"menuitem\":\n          for (var _propKey15 in lastProps)\n            (_propKey8 = lastProps[_propKey15]),\n              lastProps.hasOwnProperty(_propKey15) &&\n                null != _propKey8 &&\n                !nextProps.hasOwnProperty(_propKey15) &&\n                setProp(\n                  domElement,\n                  tag,\n                  _propKey15,\n                  null,\n                  nextProps,\n                  _propKey8\n                );\n          for (checked in nextProps)\n            if (\n              ((_propKey8 = nextProps[checked]),\n              (propKey = lastProps[checked]),\n              nextProps.hasOwnProperty(checked) &&\n                _propKey8 !== propKey &&\n                (null != _propKey8 || null != propKey))\n            )\n              switch (checked) {\n                case \"children\":\n                case \"dangerouslySetInnerHTML\":\n                  if (null != _propKey8)\n                    throw Error(\n                      tag +\n                        \" is a void element tag and must neither have `children` nor use `dangerouslySetInnerHTML`.\"\n                    );\n                  break;\n                default:\n                  setProp(\n                    domElement,\n                    tag,\n                    checked,\n                    _propKey8,\n                    nextProps,\n                    propKey\n                  );\n              }\n          return;\n        default:\n          if (isCustomElement(tag)) {\n            for (var _propKey17 in lastProps)\n              (_propKey8 = lastProps[_propKey17]),\n                lastProps.hasOwnProperty(_propKey17) &&\n                  void 0 !== _propKey8 &&\n                  !nextProps.hasOwnProperty(_propKey17) &&\n                  setPropOnCustomElement(\n                    domElement,\n                    tag,\n                    _propKey17,\n                    void 0,\n                    nextProps,\n                    _propKey8\n                  );\n            for (defaultChecked in nextProps)\n              (_propKey8 = nextProps[defaultChecked]),\n                (propKey = lastProps[defaultChecked]),\n                !nextProps.hasOwnProperty(defaultChecked) ||\n                  _propKey8 === propKey ||\n                  (void 0 === _propKey8 && void 0 === propKey) ||\n                  setPropOnCustomElement(\n                    domElement,\n                    tag,\n                    defaultChecked,\n                    _propKey8,\n                    nextProps,\n                    propKey\n                  );\n            return;\n          }\n      }\n      for (var _propKey19 in lastProps)\n        (_propKey8 = lastProps[_propKey19]),\n          lastProps.hasOwnProperty(_propKey19) &&\n            null != _propKey8 &&\n            !nextProps.hasOwnProperty(_propKey19) &&\n            setProp(domElement, tag, _propKey19, null, nextProps, _propKey8);\n      for (lastProp in nextProps)\n        (_propKey8 = nextProps[lastProp]),\n          (propKey = lastProps[lastProp]),\n          !nextProps.hasOwnProperty(lastProp) ||\n            _propKey8 === propKey ||\n            (null == _propKey8 && null == propKey) ||\n            setProp(domElement, tag, lastProp, _propKey8, nextProps, propKey);\n    }\n    function getPropNameFromAttributeName(attrName) {\n      switch (attrName) {\n        case \"class\":\n          return \"className\";\n        case \"for\":\n          return \"htmlFor\";\n        default:\n          return attrName;\n      }\n    }\n    function getStylesObjectFromElement(domElement) {\n      var serverValueInObjectForm = {};\n      domElement = domElement.style;\n      for (var i = 0; i < domElement.length; i++) {\n        var styleName = domElement[i];\n        serverValueInObjectForm[styleName] =\n          domElement.getPropertyValue(styleName);\n      }\n      return serverValueInObjectForm;\n    }\n    function diffHydratedStyles(domElement, value$jscomp$0, serverDifferences) {\n      if (null != value$jscomp$0 && \"object\" !== typeof value$jscomp$0)\n        console.error(\n          \"The `style` prop expects a mapping from style properties to values, not a string. For example, style={{marginRight: spacing + 'em'}} when using JSX.\"\n        );\n      else {\n        var clientValue;\n        var delimiter = (clientValue = \"\"),\n          styleName;\n        for (styleName in value$jscomp$0)\n          if (value$jscomp$0.hasOwnProperty(styleName)) {\n            var value = value$jscomp$0[styleName];\n            null != value &&\n              \"boolean\" !== typeof value &&\n              \"\" !== value &&\n              (0 === styleName.indexOf(\"--\")\n                ? (checkCSSPropertyStringCoercion(value, styleName),\n                  (clientValue +=\n                    delimiter + styleName + \":\" + (\"\" + value).trim()))\n                : \"number\" !== typeof value ||\n                    0 === value ||\n                    unitlessNumbers.has(styleName)\n                  ? (checkCSSPropertyStringCoercion(value, styleName),\n                    (clientValue +=\n                      delimiter +\n                      styleName\n                        .replace(uppercasePattern, \"-$1\")\n                        .toLowerCase()\n                        .replace(msPattern$1, \"-ms-\") +\n                      \":\" +\n                      (\"\" + value).trim()))\n                  : (clientValue +=\n                      delimiter +\n                      styleName\n                        .replace(uppercasePattern, \"-$1\")\n                        .toLowerCase()\n                        .replace(msPattern$1, \"-ms-\") +\n                      \":\" +\n                      value +\n                      \"px\"),\n              (delimiter = \";\"));\n          }\n        clientValue = clientValue || null;\n        value$jscomp$0 = domElement.getAttribute(\"style\");\n        value$jscomp$0 !== clientValue &&\n          ((clientValue = normalizeMarkupForTextOrAttribute(clientValue)),\n          normalizeMarkupForTextOrAttribute(value$jscomp$0) !== clientValue &&\n            (serverDifferences.style = getStylesObjectFromElement(domElement)));\n      }\n    }\n    function hydrateAttribute(\n      domElement,\n      propKey,\n      attributeName,\n      value,\n      extraAttributes,\n      serverDifferences\n    ) {\n      extraAttributes.delete(attributeName);\n      domElement = domElement.getAttribute(attributeName);\n      if (null === domElement)\n        switch (typeof value) {\n          case \"undefined\":\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            return;\n        }\n      else if (null != value)\n        switch (typeof value) {\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            break;\n          default:\n            if (\n              (checkAttributeStringCoercion(value, propKey),\n              domElement === \"\" + value)\n            )\n              return;\n        }\n      warnForPropDifference(propKey, domElement, value, serverDifferences);\n    }\n    function hydrateBooleanAttribute(\n      domElement,\n      propKey,\n      attributeName,\n      value,\n      extraAttributes,\n      serverDifferences\n    ) {\n      extraAttributes.delete(attributeName);\n      domElement = domElement.getAttribute(attributeName);\n      if (null === domElement) {\n        switch (typeof value) {\n          case \"function\":\n          case \"symbol\":\n            return;\n        }\n        if (!value) return;\n      } else\n        switch (typeof value) {\n          case \"function\":\n          case \"symbol\":\n            break;\n          default:\n            if (value) return;\n        }\n      warnForPropDifference(propKey, domElement, value, serverDifferences);\n    }\n    function hydrateBooleanishAttribute(\n      domElement,\n      propKey,\n      attributeName,\n      value,\n      extraAttributes,\n      serverDifferences\n    ) {\n      extraAttributes.delete(attributeName);\n      domElement = domElement.getAttribute(attributeName);\n      if (null === domElement)\n        switch (typeof value) {\n          case \"undefined\":\n          case \"function\":\n          case \"symbol\":\n            return;\n        }\n      else if (null != value)\n        switch (typeof value) {\n          case \"function\":\n          case \"symbol\":\n            break;\n          default:\n            if (\n              (checkAttributeStringCoercion(value, attributeName),\n              domElement === \"\" + value)\n            )\n              return;\n        }\n      warnForPropDifference(propKey, domElement, value, serverDifferences);\n    }\n    function hydrateNumericAttribute(\n      domElement,\n      propKey,\n      attributeName,\n      value,\n      extraAttributes,\n      serverDifferences\n    ) {\n      extraAttributes.delete(attributeName);\n      domElement = domElement.getAttribute(attributeName);\n      if (null === domElement)\n        switch (typeof value) {\n          case \"undefined\":\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            return;\n          default:\n            if (isNaN(value)) return;\n        }\n      else if (null != value)\n        switch (typeof value) {\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            break;\n          default:\n            if (\n              !isNaN(value) &&\n              (checkAttributeStringCoercion(value, propKey),\n              domElement === \"\" + value)\n            )\n              return;\n        }\n      warnForPropDifference(propKey, domElement, value, serverDifferences);\n    }\n    function hydrateSanitizedAttribute(\n      domElement,\n      propKey,\n      attributeName,\n      value,\n      extraAttributes,\n      serverDifferences\n    ) {\n      extraAttributes.delete(attributeName);\n      domElement = domElement.getAttribute(attributeName);\n      if (null === domElement)\n        switch (typeof value) {\n          case \"undefined\":\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            return;\n        }\n      else if (null != value)\n        switch (typeof value) {\n          case \"function\":\n          case \"symbol\":\n          case \"boolean\":\n            break;\n          default:\n            if (\n              (checkAttributeStringCoercion(value, propKey),\n              (attributeName = sanitizeURL(\"\" + value)),\n              domElement === attributeName)\n            )\n              return;\n        }\n      warnForPropDifference(propKey, domElement, value, serverDifferences);\n    }\n    function diffHydratedProperties(domElement, tag, props, hostContext) {\n      for (\n        var serverDifferences = {},\n          extraAttributes = new Set(),\n          attributes = domElement.attributes,\n          i = 0;\n        i < attributes.length;\n        i++\n      )\n        switch (attributes[i].name.toLowerCase()) {\n          case \"value\":\n            break;\n          case \"checked\":\n            break;\n          case \"selected\":\n            break;\n          default:\n            extraAttributes.add(attributes[i].name);\n        }\n      if (isCustomElement(tag))\n        for (var propKey in props) {\n          if (props.hasOwnProperty(propKey)) {\n            var value = props[propKey];\n            if (null != value)\n              if (registrationNameDependencies.hasOwnProperty(propKey))\n                \"function\" !== typeof value &&\n                  warnForInvalidEventListener(propKey, value);\n              else if (!0 !== props.suppressHydrationWarning)\n                switch (propKey) {\n                  case \"children\":\n                    (\"string\" !== typeof value && \"number\" !== typeof value) ||\n                      warnForPropDifference(\n                        \"children\",\n                        domElement.textContent,\n                        value,\n                        serverDifferences\n                      );\n                    continue;\n                  case \"suppressContentEditableWarning\":\n                  case \"suppressHydrationWarning\":\n                  case \"defaultValue\":\n                  case \"defaultChecked\":\n                  case \"innerHTML\":\n                  case \"ref\":\n                    continue;\n                  case \"dangerouslySetInnerHTML\":\n                    attributes = domElement.innerHTML;\n                    value = value ? value.__html : void 0;\n                    null != value &&\n                      ((value = normalizeHTML(domElement, value)),\n                      warnForPropDifference(\n                        propKey,\n                        attributes,\n                        value,\n                        serverDifferences\n                      ));\n                    continue;\n                  case \"style\":\n                    extraAttributes.delete(propKey);\n                    diffHydratedStyles(domElement, value, serverDifferences);\n                    continue;\n                  case \"offsetParent\":\n                  case \"offsetTop\":\n                  case \"offsetLeft\":\n                  case \"offsetWidth\":\n                  case \"offsetHeight\":\n                  case \"isContentEditable\":\n                  case \"outerText\":\n                  case \"outerHTML\":\n                    extraAttributes.delete(propKey.toLowerCase());\n                    console.error(\n                      \"Assignment to read-only property will result in a no-op: `%s`\",\n                      propKey\n                    );\n                    continue;\n                  case \"className\":\n                    extraAttributes.delete(\"class\");\n                    attributes = getValueForAttributeOnCustomComponent(\n                      domElement,\n                      \"class\",\n                      value\n                    );\n                    warnForPropDifference(\n                      \"className\",\n                      attributes,\n                      value,\n                      serverDifferences\n                    );\n                    continue;\n                  default:\n                    hostContext.context === HostContextNamespaceNone &&\n                    \"svg\" !== tag &&\n                    \"math\" !== tag\n                      ? extraAttributes.delete(propKey.toLowerCase())\n                      : extraAttributes.delete(propKey),\n                      (attributes = getValueForAttributeOnCustomComponent(\n                        domElement,\n                        propKey,\n                        value\n                      )),\n                      warnForPropDifference(\n                        propKey,\n                        attributes,\n                        value,\n                        serverDifferences\n                      );\n                }\n          }\n        }\n      else\n        for (value in props)\n          if (\n            props.hasOwnProperty(value) &&\n            ((propKey = props[value]), null != propKey)\n          )\n            if (registrationNameDependencies.hasOwnProperty(value))\n              \"function\" !== typeof propKey &&\n                warnForInvalidEventListener(value, propKey);\n            else if (!0 !== props.suppressHydrationWarning)\n              switch (value) {\n                case \"children\":\n                  (\"string\" !== typeof propKey &&\n                    \"number\" !== typeof propKey) ||\n                    warnForPropDifference(\n                      \"children\",\n                      domElement.textContent,\n                      propKey,\n                      serverDifferences\n                    );\n                  continue;\n                case \"suppressContentEditableWarning\":\n                case \"suppressHydrationWarning\":\n                case \"value\":\n                case \"checked\":\n                case \"selected\":\n                case \"defaultValue\":\n                case \"defaultChecked\":\n                case \"innerHTML\":\n                case \"ref\":\n                  continue;\n                case \"dangerouslySetInnerHTML\":\n                  attributes = domElement.innerHTML;\n                  propKey = propKey ? propKey.__html : void 0;\n                  null != propKey &&\n                    ((propKey = normalizeHTML(domElement, propKey)),\n                    attributes !== propKey &&\n                      (serverDifferences[value] = { __html: attributes }));\n                  continue;\n                case \"className\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"class\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"tabIndex\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"tabindex\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"style\":\n                  extraAttributes.delete(value);\n                  diffHydratedStyles(domElement, propKey, serverDifferences);\n                  continue;\n                case \"multiple\":\n                  extraAttributes.delete(value);\n                  warnForPropDifference(\n                    value,\n                    domElement.multiple,\n                    propKey,\n                    serverDifferences\n                  );\n                  continue;\n                case \"muted\":\n                  extraAttributes.delete(value);\n                  warnForPropDifference(\n                    value,\n                    domElement.muted,\n                    propKey,\n                    serverDifferences\n                  );\n                  continue;\n                case \"autoFocus\":\n                  extraAttributes.delete(\"autofocus\");\n                  warnForPropDifference(\n                    value,\n                    domElement.autofocus,\n                    propKey,\n                    serverDifferences\n                  );\n                  continue;\n                case \"data\":\n                  if (\"object\" !== tag) {\n                    extraAttributes.delete(value);\n                    attributes = domElement.getAttribute(\"data\");\n                    warnForPropDifference(\n                      value,\n                      attributes,\n                      propKey,\n                      serverDifferences\n                    );\n                    continue;\n                  }\n                case \"src\":\n                case \"href\":\n                  if (\n                    !(\n                      \"\" !== propKey ||\n                      (\"a\" === tag && \"href\" === value) ||\n                      (\"object\" === tag && \"data\" === value)\n                    )\n                  ) {\n                    \"src\" === value\n                      ? console.error(\n                          'An empty string (\"\") was passed to the %s attribute. This may cause the browser to download the whole page again over the network. To fix this, either do not render the element at all or pass null to %s instead of an empty string.',\n                          value,\n                          value\n                        )\n                      : console.error(\n                          'An empty string (\"\") was passed to the %s attribute. To fix this, either do not render the element at all or pass null to %s instead of an empty string.',\n                          value,\n                          value\n                        );\n                    continue;\n                  }\n                  hydrateSanitizedAttribute(\n                    domElement,\n                    value,\n                    value,\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"action\":\n                case \"formAction\":\n                  attributes = domElement.getAttribute(value);\n                  if (\"function\" === typeof propKey) {\n                    extraAttributes.delete(value.toLowerCase());\n                    \"formAction\" === value\n                      ? (extraAttributes.delete(\"name\"),\n                        extraAttributes.delete(\"formenctype\"),\n                        extraAttributes.delete(\"formmethod\"),\n                        extraAttributes.delete(\"formtarget\"))\n                      : (extraAttributes.delete(\"enctype\"),\n                        extraAttributes.delete(\"method\"),\n                        extraAttributes.delete(\"target\"));\n                    continue;\n                  } else if (attributes === EXPECTED_FORM_ACTION_URL) {\n                    extraAttributes.delete(value.toLowerCase());\n                    warnForPropDifference(\n                      value,\n                      \"function\",\n                      propKey,\n                      serverDifferences\n                    );\n                    continue;\n                  }\n                  hydrateSanitizedAttribute(\n                    domElement,\n                    value,\n                    value.toLowerCase(),\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkHref\":\n                  hydrateSanitizedAttribute(\n                    domElement,\n                    value,\n                    \"xlink:href\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"contentEditable\":\n                  hydrateBooleanishAttribute(\n                    domElement,\n                    value,\n                    \"contenteditable\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"spellCheck\":\n                  hydrateBooleanishAttribute(\n                    domElement,\n                    value,\n                    \"spellcheck\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"draggable\":\n                case \"autoReverse\":\n                case \"externalResourcesRequired\":\n                case \"focusable\":\n                case \"preserveAlpha\":\n                  hydrateBooleanishAttribute(\n                    domElement,\n                    value,\n                    value,\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"allowFullScreen\":\n                case \"async\":\n                case \"autoPlay\":\n                case \"controls\":\n                case \"default\":\n                case \"defer\":\n                case \"disabled\":\n                case \"disablePictureInPicture\":\n                case \"disableRemotePlayback\":\n                case \"formNoValidate\":\n                case \"hidden\":\n                case \"loop\":\n                case \"noModule\":\n                case \"noValidate\":\n                case \"open\":\n                case \"playsInline\":\n                case \"readOnly\":\n                case \"required\":\n                case \"reversed\":\n                case \"scoped\":\n                case \"seamless\":\n                case \"itemScope\":\n                  hydrateBooleanAttribute(\n                    domElement,\n                    value,\n                    value.toLowerCase(),\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"capture\":\n                case \"download\":\n                  a: {\n                    i = domElement;\n                    var attributeName = (attributes = value),\n                      serverDifferences$jscomp$0 = serverDifferences;\n                    extraAttributes.delete(attributeName);\n                    i = i.getAttribute(attributeName);\n                    if (null === i)\n                      switch (typeof propKey) {\n                        case \"undefined\":\n                        case \"function\":\n                        case \"symbol\":\n                          break a;\n                        default:\n                          if (!1 === propKey) break a;\n                      }\n                    else if (null != propKey)\n                      switch (typeof propKey) {\n                        case \"function\":\n                        case \"symbol\":\n                          break;\n                        case \"boolean\":\n                          if (!0 === propKey && \"\" === i) break a;\n                          break;\n                        default:\n                          if (\n                            (checkAttributeStringCoercion(propKey, attributes),\n                            i === \"\" + propKey)\n                          )\n                            break a;\n                      }\n                    warnForPropDifference(\n                      attributes,\n                      i,\n                      propKey,\n                      serverDifferences$jscomp$0\n                    );\n                  }\n                  continue;\n                case \"cols\":\n                case \"rows\":\n                case \"size\":\n                case \"span\":\n                  a: {\n                    i = domElement;\n                    attributeName = attributes = value;\n                    serverDifferences$jscomp$0 = serverDifferences;\n                    extraAttributes.delete(attributeName);\n                    i = i.getAttribute(attributeName);\n                    if (null === i)\n                      switch (typeof propKey) {\n                        case \"undefined\":\n                        case \"function\":\n                        case \"symbol\":\n                        case \"boolean\":\n                          break a;\n                        default:\n                          if (isNaN(propKey) || 1 > propKey) break a;\n                      }\n                    else if (null != propKey)\n                      switch (typeof propKey) {\n                        case \"function\":\n                        case \"symbol\":\n                        case \"boolean\":\n                          break;\n                        default:\n                          if (\n                            !(isNaN(propKey) || 1 > propKey) &&\n                            (checkAttributeStringCoercion(propKey, attributes),\n                            i === \"\" + propKey)\n                          )\n                            break a;\n                      }\n                    warnForPropDifference(\n                      attributes,\n                      i,\n                      propKey,\n                      serverDifferences$jscomp$0\n                    );\n                  }\n                  continue;\n                case \"rowSpan\":\n                  hydrateNumericAttribute(\n                    domElement,\n                    value,\n                    \"rowspan\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"start\":\n                  hydrateNumericAttribute(\n                    domElement,\n                    value,\n                    value,\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xHeight\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"x-height\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkActuate\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xlink:actuate\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkArcrole\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xlink:arcrole\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkRole\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xlink:role\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkShow\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xlink:show\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkTitle\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xlink:title\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xlinkType\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xlink:type\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xmlBase\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xml:base\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xmlLang\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xml:lang\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"xmlSpace\":\n                  hydrateAttribute(\n                    domElement,\n                    value,\n                    \"xml:space\",\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                case \"inert\":\n                  \"\" !== propKey ||\n                    didWarnForNewBooleanPropsWithEmptyValue[value] ||\n                    ((didWarnForNewBooleanPropsWithEmptyValue[value] = !0),\n                    console.error(\n                      \"Received an empty string for a boolean attribute `%s`. This will treat the attribute as if it were false. Either pass `false` to silence this warning, or pass `true` if you used an empty string in earlier versions of React to indicate this attribute is true.\",\n                      value\n                    ));\n                  hydrateBooleanAttribute(\n                    domElement,\n                    value,\n                    value,\n                    propKey,\n                    extraAttributes,\n                    serverDifferences\n                  );\n                  continue;\n                default:\n                  if (\n                    !(2 < value.length) ||\n                    (\"o\" !== value[0] && \"O\" !== value[0]) ||\n                    (\"n\" !== value[1] && \"N\" !== value[1])\n                  ) {\n                    i = getAttributeAlias(value);\n                    attributes = !1;\n                    hostContext.context === HostContextNamespaceNone &&\n                    \"svg\" !== tag &&\n                    \"math\" !== tag\n                      ? extraAttributes.delete(i.toLowerCase())\n                      : ((attributeName = value.toLowerCase()),\n                        (attributeName = possibleStandardNames.hasOwnProperty(\n                          attributeName\n                        )\n                          ? possibleStandardNames[attributeName] || null\n                          : null),\n                        null !== attributeName &&\n                          attributeName !== value &&\n                          ((attributes = !0),\n                          extraAttributes.delete(attributeName)),\n                        extraAttributes.delete(i));\n                    a: if (\n                      ((attributeName = domElement),\n                      (serverDifferences$jscomp$0 = i),\n                      (i = propKey),\n                      isAttributeNameSafe(serverDifferences$jscomp$0))\n                    )\n                      if (\n                        attributeName.hasAttribute(serverDifferences$jscomp$0)\n                      )\n                        (attributeName = attributeName.getAttribute(\n                          serverDifferences$jscomp$0\n                        )),\n                          checkAttributeStringCoercion(\n                            i,\n                            serverDifferences$jscomp$0\n                          ),\n                          (i = attributeName === \"\" + i ? i : attributeName);\n                      else {\n                        switch (typeof i) {\n                          case \"function\":\n                          case \"symbol\":\n                            break a;\n                          case \"boolean\":\n                            if (\n                              ((attributeName = serverDifferences$jscomp$0\n                                .toLowerCase()\n                                .slice(0, 5)),\n                              \"data-\" !== attributeName &&\n                                \"aria-\" !== attributeName)\n                            )\n                              break a;\n                        }\n                        i = void 0 === i ? void 0 : null;\n                      }\n                    else i = void 0;\n                    attributes ||\n                      warnForPropDifference(\n                        value,\n                        i,\n                        propKey,\n                        serverDifferences\n                      );\n                  }\n              }\n      0 < extraAttributes.size &&\n        !0 !== props.suppressHydrationWarning &&\n        warnForExtraAttributes(domElement, extraAttributes, serverDifferences);\n      return 0 === Object.keys(serverDifferences).length\n        ? null\n        : serverDifferences;\n    }\n    function propNamesListJoin(list, combinator) {\n      switch (list.length) {\n        case 0:\n          return \"\";\n        case 1:\n          return list[0];\n        case 2:\n          return list[0] + \" \" + combinator + \" \" + list[1];\n        default:\n          return (\n            list.slice(0, -1).join(\", \") +\n            \", \" +\n            combinator +\n            \" \" +\n            list[list.length - 1]\n          );\n      }\n    }\n    function getOwnerDocumentFromRootContainer(rootContainerElement) {\n      return 9 === rootContainerElement.nodeType\n        ? rootContainerElement\n        : rootContainerElement.ownerDocument;\n    }\n    function getOwnHostContext(namespaceURI) {\n      switch (namespaceURI) {\n        case SVG_NAMESPACE:\n          return HostContextNamespaceSvg;\n        case MATH_NAMESPACE:\n          return HostContextNamespaceMath;\n        default:\n          return HostContextNamespaceNone;\n      }\n    }\n    function getChildHostContextProd(parentNamespace, type) {\n      if (parentNamespace === HostContextNamespaceNone)\n        switch (type) {\n          case \"svg\":\n            return HostContextNamespaceSvg;\n          case \"math\":\n            return HostContextNamespaceMath;\n          default:\n            return HostContextNamespaceNone;\n        }\n      return parentNamespace === HostContextNamespaceSvg &&\n        \"foreignObject\" === type\n        ? HostContextNamespaceNone\n        : parentNamespace;\n    }\n    function shouldSetTextContent(type, props) {\n      return (\n        \"textarea\" === type ||\n        \"noscript\" === type ||\n        \"string\" === typeof props.children ||\n        \"number\" === typeof props.children ||\n        \"bigint\" === typeof props.children ||\n        (\"object\" === typeof props.dangerouslySetInnerHTML &&\n          null !== props.dangerouslySetInnerHTML &&\n          null != props.dangerouslySetInnerHTML.__html)\n      );\n    }\n    function shouldAttemptEagerTransition() {\n      var event = window.event;\n      if (event && \"popstate\" === event.type) {\n        if (event === currentPopstateTransitionEvent) return !1;\n        currentPopstateTransitionEvent = event;\n        return !0;\n      }\n      currentPopstateTransitionEvent = null;\n      return !1;\n    }\n    function handleErrorInNextTick(error) {\n      setTimeout(function () {\n        throw error;\n      });\n    }\n    function commitMount(domElement, type, newProps) {\n      switch (type) {\n        case \"button\":\n        case \"input\":\n        case \"select\":\n        case \"textarea\":\n          newProps.autoFocus && domElement.focus();\n          break;\n        case \"img\":\n          newProps.src\n            ? (domElement.src = newProps.src)\n            : newProps.srcSet && (domElement.srcset = newProps.srcSet);\n      }\n    }\n    function commitUpdate(domElement, type, oldProps, newProps) {\n      updateProperties(domElement, type, oldProps, newProps);\n      domElement[internalPropsKey] = newProps;\n    }\n    function resetTextContent(domElement) {\n      setTextContent(domElement, \"\");\n    }\n    function commitTextUpdate(textInstance, oldText, newText) {\n      textInstance.nodeValue = newText;\n    }\n    function isSingletonScope(type) {\n      return \"head\" === type;\n    }\n    function removeChild(parentInstance, child) {\n      parentInstance.removeChild(child);\n    }\n    function removeChildFromContainer(container, child) {\n      (9 === container.nodeType\n        ? container.body\n        : \"HTML\" === container.nodeName\n          ? container.ownerDocument.body\n          : container\n      ).removeChild(child);\n    }\n    function clearSuspenseBoundary(parentInstance, suspenseInstance) {\n      var node = suspenseInstance,\n        possiblePreambleContribution = 0,\n        depth = 0;\n      do {\n        var nextNode = node.nextSibling;\n        parentInstance.removeChild(node);\n        if (nextNode && 8 === nextNode.nodeType)\n          if (((node = nextNode.data), node === SUSPENSE_END_DATA)) {\n            if (\n              0 < possiblePreambleContribution &&\n              8 > possiblePreambleContribution\n            ) {\n              node = possiblePreambleContribution;\n              var ownerDocument = parentInstance.ownerDocument;\n              node & PREAMBLE_CONTRIBUTION_HTML &&\n                releaseSingletonInstance(ownerDocument.documentElement);\n              node & PREAMBLE_CONTRIBUTION_BODY &&\n                releaseSingletonInstance(ownerDocument.body);\n              if (node & PREAMBLE_CONTRIBUTION_HEAD)\n                for (\n                  node = ownerDocument.head,\n                    releaseSingletonInstance(node),\n                    ownerDocument = node.firstChild;\n                  ownerDocument;\n\n                ) {\n                  var nextNode$jscomp$0 = ownerDocument.nextSibling,\n                    nodeName = ownerDocument.nodeName;\n                  ownerDocument[internalHoistableMarker] ||\n                    \"SCRIPT\" === nodeName ||\n                    \"STYLE\" === nodeName ||\n                    (\"LINK\" === nodeName &&\n                      \"stylesheet\" === ownerDocument.rel.toLowerCase()) ||\n                    node.removeChild(ownerDocument);\n                  ownerDocument = nextNode$jscomp$0;\n                }\n            }\n            if (0 === depth) {\n              parentInstance.removeChild(nextNode);\n              retryIfBlockedOn(suspenseInstance);\n              return;\n            }\n            depth--;\n          } else\n            node === SUSPENSE_START_DATA ||\n            node === SUSPENSE_PENDING_START_DATA ||\n            node === SUSPENSE_FALLBACK_START_DATA\n              ? depth++\n              : (possiblePreambleContribution = node.charCodeAt(0) - 48);\n        else possiblePreambleContribution = 0;\n        node = nextNode;\n      } while (node);\n      retryIfBlockedOn(suspenseInstance);\n    }\n    function hideInstance(instance) {\n      instance = instance.style;\n      \"function\" === typeof instance.setProperty\n        ? instance.setProperty(\"display\", \"none\", \"important\")\n        : (instance.display = \"none\");\n    }\n    function hideTextInstance(textInstance) {\n      textInstance.nodeValue = \"\";\n    }\n    function unhideInstance(instance, props) {\n      props = props[STYLE];\n      props =\n        void 0 !== props && null !== props && props.hasOwnProperty(\"display\")\n          ? props.display\n          : null;\n      instance.style.display =\n        null == props || \"boolean\" === typeof props ? \"\" : (\"\" + props).trim();\n    }\n    function unhideTextInstance(textInstance, text) {\n      textInstance.nodeValue = text;\n    }\n    function clearContainerSparingly(container) {\n      var nextNode = container.firstChild;\n      nextNode && 10 === nextNode.nodeType && (nextNode = nextNode.nextSibling);\n      for (; nextNode; ) {\n        var node = nextNode;\n        nextNode = nextNode.nextSibling;\n        switch (node.nodeName) {\n          case \"HTML\":\n          case \"HEAD\":\n          case \"BODY\":\n            clearContainerSparingly(node);\n            detachDeletedInstance(node);\n            continue;\n          case \"SCRIPT\":\n          case \"STYLE\":\n            continue;\n          case \"LINK\":\n            if (\"stylesheet\" === node.rel.toLowerCase()) continue;\n        }\n        container.removeChild(node);\n      }\n    }\n    function canHydrateInstance(instance, type, props, inRootOrSingleton) {\n      for (; 1 === instance.nodeType; ) {\n        var anyProps = props;\n        if (instance.nodeName.toLowerCase() !== type.toLowerCase()) {\n          if (\n            !inRootOrSingleton &&\n            (\"INPUT\" !== instance.nodeName || \"hidden\" !== instance.type)\n          )\n            break;\n        } else if (!inRootOrSingleton)\n          if (\"input\" === type && \"hidden\" === instance.type) {\n            checkAttributeStringCoercion(anyProps.name, \"name\");\n            var name = null == anyProps.name ? null : \"\" + anyProps.name;\n            if (\n              \"hidden\" === anyProps.type &&\n              instance.getAttribute(\"name\") === name\n            )\n              return instance;\n          } else return instance;\n        else if (!instance[internalHoistableMarker])\n          switch (type) {\n            case \"meta\":\n              if (!instance.hasAttribute(\"itemprop\")) break;\n              return instance;\n            case \"link\":\n              name = instance.getAttribute(\"rel\");\n              if (\n                \"stylesheet\" === name &&\n                instance.hasAttribute(\"data-precedence\")\n              )\n                break;\n              else if (\n                name !== anyProps.rel ||\n                instance.getAttribute(\"href\") !==\n                  (null == anyProps.href || \"\" === anyProps.href\n                    ? null\n                    : anyProps.href) ||\n                instance.getAttribute(\"crossorigin\") !==\n                  (null == anyProps.crossOrigin\n                    ? null\n                    : anyProps.crossOrigin) ||\n                instance.getAttribute(\"title\") !==\n                  (null == anyProps.title ? null : anyProps.title)\n              )\n                break;\n              return instance;\n            case \"style\":\n              if (instance.hasAttribute(\"data-precedence\")) break;\n              return instance;\n            case \"script\":\n              name = instance.getAttribute(\"src\");\n              if (\n                (name !== (null == anyProps.src ? null : anyProps.src) ||\n                  instance.getAttribute(\"type\") !==\n                    (null == anyProps.type ? null : anyProps.type) ||\n                  instance.getAttribute(\"crossorigin\") !==\n                    (null == anyProps.crossOrigin\n                      ? null\n                      : anyProps.crossOrigin)) &&\n                name &&\n                instance.hasAttribute(\"async\") &&\n                !instance.hasAttribute(\"itemprop\")\n              )\n                break;\n              return instance;\n            default:\n              return instance;\n          }\n        instance = getNextHydratable(instance.nextSibling);\n        if (null === instance) break;\n      }\n      return null;\n    }\n    function canHydrateTextInstance(instance, text, inRootOrSingleton) {\n      if (\"\" === text) return null;\n      for (; 3 !== instance.nodeType; ) {\n        if (\n          (1 !== instance.nodeType ||\n            \"INPUT\" !== instance.nodeName ||\n            \"hidden\" !== instance.type) &&\n          !inRootOrSingleton\n        )\n          return null;\n        instance = getNextHydratable(instance.nextSibling);\n        if (null === instance) return null;\n      }\n      return instance;\n    }\n    function isSuspenseInstanceFallback(instance) {\n      return (\n        instance.data === SUSPENSE_FALLBACK_START_DATA ||\n        (instance.data === SUSPENSE_PENDING_START_DATA &&\n          instance.ownerDocument.readyState === DOCUMENT_READY_STATE_COMPLETE)\n      );\n    }\n    function registerSuspenseInstanceRetry(instance, callback) {\n      var ownerDocument = instance.ownerDocument;\n      if (\n        instance.data !== SUSPENSE_PENDING_START_DATA ||\n        ownerDocument.readyState === DOCUMENT_READY_STATE_COMPLETE\n      )\n        callback();\n      else {\n        var listener = function () {\n          callback();\n          ownerDocument.removeEventListener(\"DOMContentLoaded\", listener);\n        };\n        ownerDocument.addEventListener(\"DOMContentLoaded\", listener);\n        instance._reactRetry = listener;\n      }\n    }\n    function getNextHydratable(node) {\n      for (; null != node; node = node.nextSibling) {\n        var nodeType = node.nodeType;\n        if (1 === nodeType || 3 === nodeType) break;\n        if (8 === nodeType) {\n          nodeType = node.data;\n          if (\n            nodeType === SUSPENSE_START_DATA ||\n            nodeType === SUSPENSE_FALLBACK_START_DATA ||\n            nodeType === SUSPENSE_PENDING_START_DATA ||\n            nodeType === FORM_STATE_IS_MATCHING ||\n            nodeType === FORM_STATE_IS_NOT_MATCHING\n          )\n            break;\n          if (nodeType === SUSPENSE_END_DATA) return null;\n        }\n      }\n      return node;\n    }\n    function describeHydratableInstanceForDevWarnings(instance) {\n      if (1 === instance.nodeType) {\n        for (\n          var JSCompiler_temp_const = instance.nodeName.toLowerCase(),\n            serverDifferences = {},\n            attributes = instance.attributes,\n            i = 0;\n          i < attributes.length;\n          i++\n        ) {\n          var attr = attributes[i];\n          serverDifferences[getPropNameFromAttributeName(attr.name)] =\n            \"style\" === attr.name.toLowerCase()\n              ? getStylesObjectFromElement(instance)\n              : attr.value;\n        }\n        return { type: JSCompiler_temp_const, props: serverDifferences };\n      }\n      return 8 === instance.nodeType\n        ? { type: \"Suspense\", props: {} }\n        : instance.nodeValue;\n    }\n    function diffHydratedTextForDevWarnings(textInstance, text, parentProps) {\n      return null === parentProps ||\n        !0 !== parentProps[SUPPRESS_HYDRATION_WARNING]\n        ? (textInstance.nodeValue === text\n            ? (textInstance = null)\n            : ((text = normalizeMarkupForTextOrAttribute(text)),\n              (textInstance =\n                normalizeMarkupForTextOrAttribute(textInstance.nodeValue) ===\n                text\n                  ? null\n                  : textInstance.nodeValue)),\n          textInstance)\n        : null;\n    }\n    function getNextHydratableInstanceAfterSuspenseInstance(suspenseInstance) {\n      suspenseInstance = suspenseInstance.nextSibling;\n      for (var depth = 0; suspenseInstance; ) {\n        if (8 === suspenseInstance.nodeType) {\n          var data = suspenseInstance.data;\n          if (data === SUSPENSE_END_DATA) {\n            if (0 === depth)\n              return getNextHydratable(suspenseInstance.nextSibling);\n            depth--;\n          } else\n            (data !== SUSPENSE_START_DATA &&\n              data !== SUSPENSE_FALLBACK_START_DATA &&\n              data !== SUSPENSE_PENDING_START_DATA) ||\n              depth++;\n        }\n        suspenseInstance = suspenseInstance.nextSibling;\n      }\n      return null;\n    }\n    function getParentSuspenseInstance(targetInstance) {\n      targetInstance = targetInstance.previousSibling;\n      for (var depth = 0; targetInstance; ) {\n        if (8 === targetInstance.nodeType) {\n          var data = targetInstance.data;\n          if (\n            data === SUSPENSE_START_DATA ||\n            data === SUSPENSE_FALLBACK_START_DATA ||\n            data === SUSPENSE_PENDING_START_DATA\n          ) {\n            if (0 === depth) return targetInstance;\n            depth--;\n          } else data === SUSPENSE_END_DATA && depth++;\n        }\n        targetInstance = targetInstance.previousSibling;\n      }\n      return null;\n    }\n    function commitHydratedContainer(container) {\n      retryIfBlockedOn(container);\n    }\n    function commitHydratedSuspenseInstance(suspenseInstance) {\n      retryIfBlockedOn(suspenseInstance);\n    }\n    function resolveSingletonInstance(\n      type,\n      props,\n      rootContainerInstance,\n      hostContext,\n      validateDOMNestingDev\n    ) {\n      validateDOMNestingDev &&\n        validateDOMNesting(type, hostContext.ancestorInfo);\n      props = getOwnerDocumentFromRootContainer(rootContainerInstance);\n      switch (type) {\n        case \"html\":\n          type = props.documentElement;\n          if (!type)\n            throw Error(\n              \"React expected an <html> element (document.documentElement) to exist in the Document but one was not found. React never removes the documentElement for any Document it renders into so the cause is likely in some other script running on this page.\"\n            );\n          return type;\n        case \"head\":\n          type = props.head;\n          if (!type)\n            throw Error(\n              \"React expected a <head> element (document.head) to exist in the Document but one was not found. React never removes the head for any Document it renders into so the cause is likely in some other script running on this page.\"\n            );\n          return type;\n        case \"body\":\n          type = props.body;\n          if (!type)\n            throw Error(\n              \"React expected a <body> element (document.body) to exist in the Document but one was not found. React never removes the body for any Document it renders into so the cause is likely in some other script running on this page.\"\n            );\n          return type;\n        default:\n          throw Error(\n            \"resolveSingletonInstance was called with an element type that is not supported. This is a bug in React.\"\n          );\n      }\n    }\n    function acquireSingletonInstance(\n      type,\n      props,\n      instance,\n      internalInstanceHandle\n    ) {\n      if (\n        !instance[internalContainerInstanceKey] &&\n        getInstanceFromNode(instance)\n      ) {\n        var tagName = instance.tagName.toLowerCase();\n        console.error(\n          \"You are mounting a new %s component when a previous one has not first unmounted. It is an error to render more than one %s component at a time and attributes and children of these components will likely fail in unpredictable ways. Please only render a single instance of <%s> and if you need to mount a new one, ensure any previous ones have unmounted first.\",\n          tagName,\n          tagName,\n          tagName\n        );\n      }\n      switch (type) {\n        case \"html\":\n        case \"head\":\n        case \"body\":\n          break;\n        default:\n          console.error(\n            \"acquireSingletonInstance was called with an element type that is not supported. This is a bug in React.\"\n          );\n      }\n      for (tagName = instance.attributes; tagName.length; )\n        instance.removeAttributeNode(tagName[0]);\n      setInitialProperties(instance, type, props);\n      instance[internalInstanceKey] = internalInstanceHandle;\n      instance[internalPropsKey] = props;\n    }\n    function releaseSingletonInstance(instance) {\n      for (var attributes = instance.attributes; attributes.length; )\n        instance.removeAttributeNode(attributes[0]);\n      detachDeletedInstance(instance);\n    }\n    function getHoistableRoot(container) {\n      return \"function\" === typeof container.getRootNode\n        ? container.getRootNode()\n        : 9 === container.nodeType\n          ? container\n          : container.ownerDocument;\n    }\n    function preconnectAs(rel, href, crossOrigin) {\n      var ownerDocument = globalDocument;\n      if (ownerDocument && \"string\" === typeof href && href) {\n        var limitedEscapedHref =\n          escapeSelectorAttributeValueInsideDoubleQuotes(href);\n        limitedEscapedHref =\n          'link[rel=\"' + rel + '\"][href=\"' + limitedEscapedHref + '\"]';\n        \"string\" === typeof crossOrigin &&\n          (limitedEscapedHref += '[crossorigin=\"' + crossOrigin + '\"]');\n        preconnectsSet.has(limitedEscapedHref) ||\n          (preconnectsSet.add(limitedEscapedHref),\n          (rel = { rel: rel, crossOrigin: crossOrigin, href: href }),\n          null === ownerDocument.querySelector(limitedEscapedHref) &&\n            ((href = ownerDocument.createElement(\"link\")),\n            setInitialProperties(href, \"link\", rel),\n            markNodeAsHoistable(href),\n            ownerDocument.head.appendChild(href)));\n      }\n    }\n    function getResource(type, currentProps, pendingProps, currentResource) {\n      var resourceRoot = (resourceRoot = rootInstanceStackCursor.current)\n        ? getHoistableRoot(resourceRoot)\n        : null;\n      if (!resourceRoot)\n        throw Error(\n          '\"resourceRoot\" was expected to exist. This is a bug in React.'\n        );\n      switch (type) {\n        case \"meta\":\n        case \"title\":\n          return null;\n        case \"style\":\n          return \"string\" === typeof pendingProps.precedence &&\n            \"string\" === typeof pendingProps.href\n            ? ((pendingProps = getStyleKey(pendingProps.href)),\n              (currentProps =\n                getResourcesFromRoot(resourceRoot).hoistableStyles),\n              (currentResource = currentProps.get(pendingProps)),\n              currentResource ||\n                ((currentResource = {\n                  type: \"style\",\n                  instance: null,\n                  count: 0,\n                  state: null\n                }),\n                currentProps.set(pendingProps, currentResource)),\n              currentResource)\n            : { type: \"void\", instance: null, count: 0, state: null };\n        case \"link\":\n          if (\n            \"stylesheet\" === pendingProps.rel &&\n            \"string\" === typeof pendingProps.href &&\n            \"string\" === typeof pendingProps.precedence\n          ) {\n            type = getStyleKey(pendingProps.href);\n            var _styles = getResourcesFromRoot(resourceRoot).hoistableStyles,\n              _resource = _styles.get(type);\n            if (\n              !_resource &&\n              ((resourceRoot = resourceRoot.ownerDocument || resourceRoot),\n              (_resource = {\n                type: \"stylesheet\",\n                instance: null,\n                count: 0,\n                state: { loading: NotLoaded, preload: null }\n              }),\n              _styles.set(type, _resource),\n              (_styles = resourceRoot.querySelector(\n                getStylesheetSelectorFromKey(type)\n              )) &&\n                !_styles._p &&\n                ((_resource.instance = _styles),\n                (_resource.state.loading = Loaded | Inserted)),\n              !preloadPropsMap.has(type))\n            ) {\n              var preloadProps = {\n                rel: \"preload\",\n                as: \"style\",\n                href: pendingProps.href,\n                crossOrigin: pendingProps.crossOrigin,\n                integrity: pendingProps.integrity,\n                media: pendingProps.media,\n                hrefLang: pendingProps.hrefLang,\n                referrerPolicy: pendingProps.referrerPolicy\n              };\n              preloadPropsMap.set(type, preloadProps);\n              _styles ||\n                preloadStylesheet(\n                  resourceRoot,\n                  type,\n                  preloadProps,\n                  _resource.state\n                );\n            }\n            if (currentProps && null === currentResource)\n              throw (\n                ((pendingProps =\n                  \"\\n\\n  - \" +\n                  describeLinkForResourceErrorDEV(currentProps) +\n                  \"\\n  + \" +\n                  describeLinkForResourceErrorDEV(pendingProps)),\n                Error(\n                  \"Expected <link> not to update to be updated to a stylesheet with precedence. Check the `rel`, `href`, and `precedence` props of this component. Alternatively, check whether two different <link> components render in the same slot or share the same key.\" +\n                    pendingProps\n                ))\n              );\n            return _resource;\n          }\n          if (currentProps && null !== currentResource)\n            throw (\n              ((pendingProps =\n                \"\\n\\n  - \" +\n                describeLinkForResourceErrorDEV(currentProps) +\n                \"\\n  + \" +\n                describeLinkForResourceErrorDEV(pendingProps)),\n              Error(\n                \"Expected stylesheet with precedence to not be updated to a different kind of <link>. Check the `rel`, `href`, and `precedence` props of this component. Alternatively, check whether two different <link> components render in the same slot or share the same key.\" +\n                  pendingProps\n              ))\n            );\n          return null;\n        case \"script\":\n          return (\n            (currentProps = pendingProps.async),\n            (pendingProps = pendingProps.src),\n            \"string\" === typeof pendingProps &&\n            currentProps &&\n            \"function\" !== typeof currentProps &&\n            \"symbol\" !== typeof currentProps\n              ? ((pendingProps = getScriptKey(pendingProps)),\n                (currentProps =\n                  getResourcesFromRoot(resourceRoot).hoistableScripts),\n                (currentResource = currentProps.get(pendingProps)),\n                currentResource ||\n                  ((currentResource = {\n                    type: \"script\",\n                    instance: null,\n                    count: 0,\n                    state: null\n                  }),\n                  currentProps.set(pendingProps, currentResource)),\n                currentResource)\n              : { type: \"void\", instance: null, count: 0, state: null }\n          );\n        default:\n          throw Error(\n            'getResource encountered a type it did not expect: \"' +\n              type +\n              '\". this is a bug in React.'\n          );\n      }\n    }\n    function describeLinkForResourceErrorDEV(props) {\n      var describedProps = 0,\n        description = \"<link\";\n      \"string\" === typeof props.rel\n        ? (describedProps++, (description += ' rel=\"' + props.rel + '\"'))\n        : hasOwnProperty.call(props, \"rel\") &&\n          (describedProps++,\n          (description +=\n            ' rel=\"' +\n            (null === props.rel ? \"null\" : \"invalid type \" + typeof props.rel) +\n            '\"'));\n      \"string\" === typeof props.href\n        ? (describedProps++, (description += ' href=\"' + props.href + '\"'))\n        : hasOwnProperty.call(props, \"href\") &&\n          (describedProps++,\n          (description +=\n            ' href=\"' +\n            (null === props.href\n              ? \"null\"\n              : \"invalid type \" + typeof props.href) +\n            '\"'));\n      \"string\" === typeof props.precedence\n        ? (describedProps++,\n          (description += ' precedence=\"' + props.precedence + '\"'))\n        : hasOwnProperty.call(props, \"precedence\") &&\n          (describedProps++,\n          (description +=\n            \" precedence={\" +\n            (null === props.precedence\n              ? \"null\"\n              : \"invalid type \" + typeof props.precedence) +\n            \"}\"));\n      Object.getOwnPropertyNames(props).length > describedProps &&\n        (description += \" ...\");\n      return description + \" />\";\n    }\n    function getStyleKey(href) {\n      return (\n        'href=\"' + escapeSelectorAttributeValueInsideDoubleQuotes(href) + '\"'\n      );\n    }\n    function getStylesheetSelectorFromKey(key) {\n      return 'link[rel=\"stylesheet\"][' + key + \"]\";\n    }\n    function stylesheetPropsFromRawProps(rawProps) {\n      return assign({}, rawProps, {\n        \"data-precedence\": rawProps.precedence,\n        precedence: null\n      });\n    }\n    function preloadStylesheet(ownerDocument, key, preloadProps, state) {\n      ownerDocument.querySelector(\n        'link[rel=\"preload\"][as=\"style\"][' + key + \"]\"\n      )\n        ? (state.loading = Loaded)\n        : ((key = ownerDocument.createElement(\"link\")),\n          (state.preload = key),\n          key.addEventListener(\"load\", function () {\n            return (state.loading |= Loaded);\n          }),\n          key.addEventListener(\"error\", function () {\n            return (state.loading |= Errored);\n          }),\n          setInitialProperties(key, \"link\", preloadProps),\n          markNodeAsHoistable(key),\n          ownerDocument.head.appendChild(key));\n    }\n    function getScriptKey(src) {\n      return (\n        '[src=\"' + escapeSelectorAttributeValueInsideDoubleQuotes(src) + '\"]'\n      );\n    }\n    function getScriptSelectorFromKey(key) {\n      return \"script[async]\" + key;\n    }\n    function acquireResource(hoistableRoot, resource, props) {\n      resource.count++;\n      if (null === resource.instance)\n        switch (resource.type) {\n          case \"style\":\n            var instance = hoistableRoot.querySelector(\n              'style[data-href~=\"' +\n                escapeSelectorAttributeValueInsideDoubleQuotes(props.href) +\n                '\"]'\n            );\n            if (instance)\n              return (\n                (resource.instance = instance),\n                markNodeAsHoistable(instance),\n                instance\n              );\n            var styleProps = assign({}, props, {\n              \"data-href\": props.href,\n              \"data-precedence\": props.precedence,\n              href: null,\n              precedence: null\n            });\n            instance = (\n              hoistableRoot.ownerDocument || hoistableRoot\n            ).createElement(\"style\");\n            markNodeAsHoistable(instance);\n            setInitialProperties(instance, \"style\", styleProps);\n            insertStylesheet(instance, props.precedence, hoistableRoot);\n            return (resource.instance = instance);\n          case \"stylesheet\":\n            styleProps = getStyleKey(props.href);\n            var _instance = hoistableRoot.querySelector(\n              getStylesheetSelectorFromKey(styleProps)\n            );\n            if (_instance)\n              return (\n                (resource.state.loading |= Inserted),\n                (resource.instance = _instance),\n                markNodeAsHoistable(_instance),\n                _instance\n              );\n            instance = stylesheetPropsFromRawProps(props);\n            (styleProps = preloadPropsMap.get(styleProps)) &&\n              adoptPreloadPropsForStylesheet(instance, styleProps);\n            _instance = (\n              hoistableRoot.ownerDocument || hoistableRoot\n            ).createElement(\"link\");\n            markNodeAsHoistable(_instance);\n            var linkInstance = _instance;\n            linkInstance._p = new Promise(function (resolve, reject) {\n              linkInstance.onload = resolve;\n              linkInstance.onerror = reject;\n            });\n            setInitialProperties(_instance, \"link\", instance);\n            resource.state.loading |= Inserted;\n            insertStylesheet(_instance, props.precedence, hoistableRoot);\n            return (resource.instance = _instance);\n          case \"script\":\n            _instance = getScriptKey(props.src);\n            if (\n              (styleProps = hoistableRoot.querySelector(\n                getScriptSelectorFromKey(_instance)\n              ))\n            )\n              return (\n                (resource.instance = styleProps),\n                markNodeAsHoistable(styleProps),\n                styleProps\n              );\n            instance = props;\n            if ((styleProps = preloadPropsMap.get(_instance)))\n              (instance = assign({}, props)),\n                adoptPreloadPropsForScript(instance, styleProps);\n            hoistableRoot = hoistableRoot.ownerDocument || hoistableRoot;\n            styleProps = hoistableRoot.createElement(\"script\");\n            markNodeAsHoistable(styleProps);\n            setInitialProperties(styleProps, \"link\", instance);\n            hoistableRoot.head.appendChild(styleProps);\n            return (resource.instance = styleProps);\n          case \"void\":\n            return null;\n          default:\n            throw Error(\n              'acquireResource encountered a resource type it did not expect: \"' +\n                resource.type +\n                '\". this is a bug in React.'\n            );\n        }\n      else\n        \"stylesheet\" === resource.type &&\n          (resource.state.loading & Inserted) === NotLoaded &&\n          ((instance = resource.instance),\n          (resource.state.loading |= Inserted),\n          insertStylesheet(instance, props.precedence, hoistableRoot));\n      return resource.instance;\n    }\n    function insertStylesheet(instance, precedence, root) {\n      for (\n        var nodes = root.querySelectorAll(\n            'link[rel=\"stylesheet\"][data-precedence],style[data-precedence]'\n          ),\n          last = nodes.length ? nodes[nodes.length - 1] : null,\n          prior = last,\n          i = 0;\n        i < nodes.length;\n        i++\n      ) {\n        var node = nodes[i];\n        if (node.dataset.precedence === precedence) prior = node;\n        else if (prior !== last) break;\n      }\n      prior\n        ? prior.parentNode.insertBefore(instance, prior.nextSibling)\n        : ((precedence = 9 === root.nodeType ? root.head : root),\n          precedence.insertBefore(instance, precedence.firstChild));\n    }\n    function adoptPreloadPropsForStylesheet(stylesheetProps, preloadProps) {\n      null == stylesheetProps.crossOrigin &&\n        (stylesheetProps.crossOrigin = preloadProps.crossOrigin);\n      null == stylesheetProps.referrerPolicy &&\n        (stylesheetProps.referrerPolicy = preloadProps.referrerPolicy);\n      null == stylesheetProps.title &&\n        (stylesheetProps.title = preloadProps.title);\n    }\n    function adoptPreloadPropsForScript(scriptProps, preloadProps) {\n      null == scriptProps.crossOrigin &&\n        (scriptProps.crossOrigin = preloadProps.crossOrigin);\n      null == scriptProps.referrerPolicy &&\n        (scriptProps.referrerPolicy = preloadProps.referrerPolicy);\n      null == scriptProps.integrity &&\n        (scriptProps.integrity = preloadProps.integrity);\n    }\n    function getHydratableHoistableCache(type, keyAttribute, ownerDocument) {\n      if (null === tagCaches) {\n        var cache = new Map();\n        var caches = (tagCaches = new Map());\n        caches.set(ownerDocument, cache);\n      } else\n        (caches = tagCaches),\n          (cache = caches.get(ownerDocument)),\n          cache || ((cache = new Map()), caches.set(ownerDocument, cache));\n      if (cache.has(type)) return cache;\n      cache.set(type, null);\n      ownerDocument = ownerDocument.getElementsByTagName(type);\n      for (caches = 0; caches < ownerDocument.length; caches++) {\n        var node = ownerDocument[caches];\n        if (\n          !(\n            node[internalHoistableMarker] ||\n            node[internalInstanceKey] ||\n            (\"link\" === type && \"stylesheet\" === node.getAttribute(\"rel\"))\n          ) &&\n          node.namespaceURI !== SVG_NAMESPACE\n        ) {\n          var nodeKey = node.getAttribute(keyAttribute) || \"\";\n          nodeKey = type + nodeKey;\n          var existing = cache.get(nodeKey);\n          existing ? existing.push(node) : cache.set(nodeKey, [node]);\n        }\n      }\n      return cache;\n    }\n    function mountHoistable(hoistableRoot, type, instance) {\n      hoistableRoot = hoistableRoot.ownerDocument || hoistableRoot;\n      hoistableRoot.head.insertBefore(\n        instance,\n        \"title\" === type ? hoistableRoot.querySelector(\"head > title\") : null\n      );\n    }\n    function isHostHoistableType(type, props, hostContext) {\n      var outsideHostContainerContext =\n        !hostContext.ancestorInfo.containerTagInScope;\n      if (\n        hostContext.context === HostContextNamespaceSvg ||\n        null != props.itemProp\n      )\n        return (\n          !outsideHostContainerContext ||\n            null == props.itemProp ||\n            (\"meta\" !== type &&\n              \"title\" !== type &&\n              \"style\" !== type &&\n              \"link\" !== type &&\n              \"script\" !== type) ||\n            console.error(\n              \"Cannot render a <%s> outside the main document if it has an `itemProp` prop. `itemProp` suggests the tag belongs to an `itemScope` which can appear anywhere in the DOM. If you were intending for React to hoist this <%s> remove the `itemProp` prop. Otherwise, try moving this tag into the <head> or <body> of the Document.\",\n              type,\n              type\n            ),\n          !1\n        );\n      switch (type) {\n        case \"meta\":\n        case \"title\":\n          return !0;\n        case \"style\":\n          if (\n            \"string\" !== typeof props.precedence ||\n            \"string\" !== typeof props.href ||\n            \"\" === props.href\n          ) {\n            outsideHostContainerContext &&\n              console.error(\n                'Cannot render a <style> outside the main document without knowing its precedence and a unique href key. React can hoist and deduplicate <style> tags if you provide a `precedence` prop along with an `href` prop that does not conflict with the `href` values used in any other hoisted <style> or <link rel=\"stylesheet\" ...> tags.  Note that hoisting <style> tags is considered an advanced feature that most will not use directly. Consider moving the <style> tag to the <head> or consider adding a `precedence=\"default\"` and `href=\"some unique resource identifier\"`.'\n              );\n            break;\n          }\n          return !0;\n        case \"link\":\n          if (\n            \"string\" !== typeof props.rel ||\n            \"string\" !== typeof props.href ||\n            \"\" === props.href ||\n            props.onLoad ||\n            props.onError\n          ) {\n            if (\n              \"stylesheet\" === props.rel &&\n              \"string\" === typeof props.precedence\n            ) {\n              type = props.href;\n              var onError = props.onError,\n                disabled = props.disabled;\n              hostContext = [];\n              props.onLoad && hostContext.push(\"`onLoad`\");\n              onError && hostContext.push(\"`onError`\");\n              null != disabled && hostContext.push(\"`disabled`\");\n              onError = propNamesListJoin(hostContext, \"and\");\n              onError += 1 === hostContext.length ? \" prop\" : \" props\";\n              disabled =\n                1 === hostContext.length ? \"an \" + onError : \"the \" + onError;\n              hostContext.length &&\n                console.error(\n                  'React encountered a <link rel=\"stylesheet\" href=\"%s\" ... /> with a `precedence` prop that also included %s. The presence of loading and error handlers indicates an intent to manage the stylesheet loading state from your from your Component code and React will not hoist or deduplicate this stylesheet. If your intent was to have React hoist and deduplciate this stylesheet using the `precedence` prop remove the %s, otherwise remove the `precedence` prop.',\n                  type,\n                  disabled,\n                  onError\n                );\n            }\n            outsideHostContainerContext &&\n              (\"string\" !== typeof props.rel ||\n              \"string\" !== typeof props.href ||\n              \"\" === props.href\n                ? console.error(\n                    \"Cannot render a <link> outside the main document without a `rel` and `href` prop. Try adding a `rel` and/or `href` prop to this <link> or moving the link into the <head> tag\"\n                  )\n                : (props.onError || props.onLoad) &&\n                  console.error(\n                    \"Cannot render a <link> with onLoad or onError listeners outside the main document. Try removing onLoad={...} and onError={...} or moving it into the root <head> tag or somewhere in the <body>.\"\n                  ));\n            break;\n          }\n          switch (props.rel) {\n            case \"stylesheet\":\n              return (\n                (type = props.precedence),\n                (props = props.disabled),\n                \"string\" !== typeof type &&\n                  outsideHostContainerContext &&\n                  console.error(\n                    'Cannot render a <link rel=\"stylesheet\" /> outside the main document without knowing its precedence. Consider adding precedence=\"default\" or moving it into the root <head> tag.'\n                  ),\n                \"string\" === typeof type && null == props\n              );\n            default:\n              return !0;\n          }\n        case \"script\":\n          type =\n            props.async &&\n            \"function\" !== typeof props.async &&\n            \"symbol\" !== typeof props.async;\n          if (\n            !type ||\n            props.onLoad ||\n            props.onError ||\n            !props.src ||\n            \"string\" !== typeof props.src\n          ) {\n            outsideHostContainerContext &&\n              (type\n                ? props.onLoad || props.onError\n                  ? console.error(\n                      \"Cannot render a <script> with onLoad or onError listeners outside the main document. Try removing onLoad={...} and onError={...} or moving it into the root <head> tag or somewhere in the <body>.\"\n                    )\n                  : console.error(\n                      \"Cannot render a <script> outside the main document without `async={true}` and a non-empty `src` prop. Ensure there is a valid `src` and either make the script async or move it into the root <head> tag or somewhere in the <body>.\"\n                    )\n                : console.error(\n                    'Cannot render a sync or defer <script> outside the main document without knowing its order. Try adding async=\"\" or moving it into the root <head> tag.'\n                  ));\n            break;\n          }\n          return !0;\n        case \"noscript\":\n        case \"template\":\n          outsideHostContainerContext &&\n            console.error(\n              \"Cannot render <%s> outside the main document. Try moving it into the root <head> tag.\",\n              type\n            );\n      }\n      return !1;\n    }\n    function preloadResource(resource) {\n      return \"stylesheet\" === resource.type &&\n        (resource.state.loading & Settled) === NotLoaded\n        ? !1\n        : !0;\n    }\n    function noop() {}\n    function suspendResource(hoistableRoot, resource, props) {\n      if (null === suspendedState)\n        throw Error(\n          \"Internal React Error: suspendedState null when it was expected to exists. Please report this as a React bug.\"\n        );\n      var state = suspendedState;\n      if (\n        \"stylesheet\" === resource.type &&\n        (\"string\" !== typeof props.media ||\n          !1 !== matchMedia(props.media).matches) &&\n        (resource.state.loading & Inserted) === NotLoaded\n      ) {\n        if (null === resource.instance) {\n          var key = getStyleKey(props.href),\n            instance = hoistableRoot.querySelector(\n              getStylesheetSelectorFromKey(key)\n            );\n          if (instance) {\n            hoistableRoot = instance._p;\n            null !== hoistableRoot &&\n              \"object\" === typeof hoistableRoot &&\n              \"function\" === typeof hoistableRoot.then &&\n              (state.count++,\n              (state = onUnsuspend.bind(state)),\n              hoistableRoot.then(state, state));\n            resource.state.loading |= Inserted;\n            resource.instance = instance;\n            markNodeAsHoistable(instance);\n            return;\n          }\n          instance = hoistableRoot.ownerDocument || hoistableRoot;\n          props = stylesheetPropsFromRawProps(props);\n          (key = preloadPropsMap.get(key)) &&\n            adoptPreloadPropsForStylesheet(props, key);\n          instance = instance.createElement(\"link\");\n          markNodeAsHoistable(instance);\n          var linkInstance = instance;\n          linkInstance._p = new Promise(function (resolve, reject) {\n            linkInstance.onload = resolve;\n            linkInstance.onerror = reject;\n          });\n          setInitialProperties(instance, \"link\", props);\n          resource.instance = instance;\n        }\n        null === state.stylesheets && (state.stylesheets = new Map());\n        state.stylesheets.set(resource, hoistableRoot);\n        (hoistableRoot = resource.state.preload) &&\n          (resource.state.loading & Settled) === NotLoaded &&\n          (state.count++,\n          (resource = onUnsuspend.bind(state)),\n          hoistableRoot.addEventListener(\"load\", resource),\n          hoistableRoot.addEventListener(\"error\", resource));\n      }\n    }\n    function waitForCommitToBeReady() {\n      if (null === suspendedState)\n        throw Error(\n          \"Internal React Error: suspendedState null when it was expected to exists. Please report this as a React bug.\"\n        );\n      var state = suspendedState;\n      state.stylesheets &&\n        0 === state.count &&\n        insertSuspendedStylesheets(state, state.stylesheets);\n      return 0 < state.count\n        ? function (commit) {\n            var stylesheetTimer = setTimeout(function () {\n              state.stylesheets &&\n                insertSuspendedStylesheets(state, state.stylesheets);\n              if (state.unsuspend) {\n                var unsuspend = state.unsuspend;\n                state.unsuspend = null;\n                unsuspend();\n              }\n            }, 6e4);\n            state.unsuspend = commit;\n            return function () {\n              state.unsuspend = null;\n              clearTimeout(stylesheetTimer);\n            };\n          }\n        : null;\n    }\n    function onUnsuspend() {\n      this.count--;\n      if (0 === this.count)\n        if (this.stylesheets)\n          insertSuspendedStylesheets(this, this.stylesheets);\n        else if (this.unsuspend) {\n          var unsuspend = this.unsuspend;\n          this.unsuspend = null;\n          unsuspend();\n        }\n    }\n    function insertSuspendedStylesheets(state, resources) {\n      state.stylesheets = null;\n      null !== state.unsuspend &&\n        (state.count++,\n        (precedencesByRoot = new Map()),\n        resources.forEach(insertStylesheetIntoRoot, state),\n        (precedencesByRoot = null),\n        onUnsuspend.call(state));\n    }\n    function insertStylesheetIntoRoot(root, resource) {\n      if (!(resource.state.loading & Inserted)) {\n        var precedences = precedencesByRoot.get(root);\n        if (precedences) var last = precedences.get(LAST_PRECEDENCE);\n        else {\n          precedences = new Map();\n          precedencesByRoot.set(root, precedences);\n          for (\n            var nodes = root.querySelectorAll(\n                \"link[data-precedence],style[data-precedence]\"\n              ),\n              i = 0;\n            i < nodes.length;\n            i++\n          ) {\n            var node = nodes[i];\n            if (\n              \"LINK\" === node.nodeName ||\n              \"not all\" !== node.getAttribute(\"media\")\n            )\n              precedences.set(node.dataset.precedence, node), (last = node);\n          }\n          last && precedences.set(LAST_PRECEDENCE, last);\n        }\n        nodes = resource.instance;\n        node = nodes.getAttribute(\"data-precedence\");\n        i = precedences.get(node) || last;\n        i === last && precedences.set(LAST_PRECEDENCE, nodes);\n        precedences.set(node, nodes);\n        this.count++;\n        last = onUnsuspend.bind(this);\n        nodes.addEventListener(\"load\", last);\n        nodes.addEventListener(\"error\", last);\n        i\n          ? i.parentNode.insertBefore(nodes, i.nextSibling)\n          : ((root = 9 === root.nodeType ? root.head : root),\n            root.insertBefore(nodes, root.firstChild));\n        resource.state.loading |= Inserted;\n      }\n    }\n    function FiberRootNode(\n      containerInfo,\n      tag,\n      hydrate,\n      identifierPrefix,\n      onUncaughtError,\n      onCaughtError,\n      onRecoverableError,\n      formState\n    ) {\n      this.tag = 1;\n      this.containerInfo = containerInfo;\n      this.pingCache = this.current = this.pendingChildren = null;\n      this.timeoutHandle = noTimeout;\n      this.callbackNode =\n        this.next =\n        this.pendingContext =\n        this.context =\n        this.cancelPendingCommit =\n          null;\n      this.callbackPriority = 0;\n      this.expirationTimes = createLaneMap(-1);\n      this.entangledLanes =\n        this.shellSuspendCounter =\n        this.errorRecoveryDisabledLanes =\n        this.expiredLanes =\n        this.warmLanes =\n        this.pingedLanes =\n        this.suspendedLanes =\n        this.pendingLanes =\n          0;\n      this.entanglements = createLaneMap(0);\n      this.hiddenUpdates = createLaneMap(null);\n      this.identifierPrefix = identifierPrefix;\n      this.onUncaughtError = onUncaughtError;\n      this.onCaughtError = onCaughtError;\n      this.onRecoverableError = onRecoverableError;\n      this.pooledCache = null;\n      this.pooledCacheLanes = 0;\n      this.formState = formState;\n      this.incompleteTransitions = new Map();\n      this.passiveEffectDuration = this.effectDuration = -0;\n      this.memoizedUpdaters = new Set();\n      containerInfo = this.pendingUpdatersLaneMap = [];\n      for (tag = 0; 31 > tag; tag++) containerInfo.push(new Set());\n      this._debugRootType = hydrate ? \"hydrateRoot()\" : \"createRoot()\";\n    }\n    function createFiberRoot(\n      containerInfo,\n      tag,\n      hydrate,\n      initialChildren,\n      hydrationCallbacks,\n      isStrictMode,\n      identifierPrefix,\n      onUncaughtError,\n      onCaughtError,\n      onRecoverableError,\n      transitionCallbacks,\n      formState\n    ) {\n      containerInfo = new FiberRootNode(\n        containerInfo,\n        tag,\n        hydrate,\n        identifierPrefix,\n        onUncaughtError,\n        onCaughtError,\n        onRecoverableError,\n        formState\n      );\n      tag = ConcurrentMode;\n      !0 === isStrictMode && (tag |= StrictLegacyMode | StrictEffectsMode);\n      isDevToolsPresent && (tag |= ProfileMode);\n      isStrictMode = createFiber(3, null, null, tag);\n      containerInfo.current = isStrictMode;\n      isStrictMode.stateNode = containerInfo;\n      tag = createCache();\n      retainCache(tag);\n      containerInfo.pooledCache = tag;\n      retainCache(tag);\n      isStrictMode.memoizedState = {\n        element: initialChildren,\n        isDehydrated: hydrate,\n        cache: tag\n      };\n      initializeUpdateQueue(isStrictMode);\n      return containerInfo;\n    }\n    function getContextForSubtree(parentComponent) {\n      if (!parentComponent) return emptyContextObject;\n      parentComponent = emptyContextObject;\n      return parentComponent;\n    }\n    function updateContainerImpl(\n      rootFiber,\n      lane,\n      element,\n      container,\n      parentComponent,\n      callback\n    ) {\n      if (\n        injectedHook &&\n        \"function\" === typeof injectedHook.onScheduleFiberRoot\n      )\n        try {\n          injectedHook.onScheduleFiberRoot(rendererID, container, element);\n        } catch (err) {\n          hasLoggedError ||\n            ((hasLoggedError = !0),\n            console.error(\n              \"React instrumentation encountered an error: %s\",\n              err\n            ));\n        }\n      null !== injectedProfilingHooks &&\n        \"function\" === typeof injectedProfilingHooks.markRenderScheduled &&\n        injectedProfilingHooks.markRenderScheduled(lane);\n      parentComponent = getContextForSubtree(parentComponent);\n      null === container.context\n        ? (container.context = parentComponent)\n        : (container.pendingContext = parentComponent);\n      isRendering &&\n        null !== current &&\n        !didWarnAboutNestedUpdates &&\n        ((didWarnAboutNestedUpdates = !0),\n        console.error(\n          \"Render methods should be a pure function of props and state; triggering nested component updates from render is not allowed. If necessary, trigger nested updates in componentDidUpdate.\\n\\nCheck the render method of %s.\",\n          getComponentNameFromFiber(current) || \"Unknown\"\n        ));\n      container = createUpdate(lane);\n      container.payload = { element: element };\n      callback = void 0 === callback ? null : callback;\n      null !== callback &&\n        (\"function\" !== typeof callback &&\n          console.error(\n            \"Expected the last optional `callback` argument to be a function. Instead received: %s.\",\n            callback\n          ),\n        (container.callback = callback));\n      element = enqueueUpdate(rootFiber, container, lane);\n      null !== element &&\n        (scheduleUpdateOnFiber(element, rootFiber, lane),\n        entangleTransitions(element, rootFiber, lane));\n    }\n    function markRetryLaneImpl(fiber, retryLane) {\n      fiber = fiber.memoizedState;\n      if (null !== fiber && null !== fiber.dehydrated) {\n        var a = fiber.retryLane;\n        fiber.retryLane = 0 !== a && a < retryLane ? a : retryLane;\n      }\n    }\n    function markRetryLaneIfNotHydrated(fiber, retryLane) {\n      markRetryLaneImpl(fiber, retryLane);\n      (fiber = fiber.alternate) && markRetryLaneImpl(fiber, retryLane);\n    }\n    function attemptContinuousHydration(fiber) {\n      if (13 === fiber.tag) {\n        var root = enqueueConcurrentRenderForLane(fiber, 67108864);\n        null !== root && scheduleUpdateOnFiber(root, fiber, 67108864);\n        markRetryLaneIfNotHydrated(fiber, 67108864);\n      }\n    }\n    function getCurrentFiberForDevTools() {\n      return current;\n    }\n    function getLaneLabelMap() {\n      for (var map = new Map(), lane = 1, index = 0; 31 > index; index++) {\n        var label = getLabelForLane(lane);\n        map.set(lane, label);\n        lane *= 2;\n      }\n      return map;\n    }\n    function dispatchDiscreteEvent(\n      domEventName,\n      eventSystemFlags,\n      container,\n      nativeEvent\n    ) {\n      var prevTransition = ReactSharedInternals.T;\n      ReactSharedInternals.T = null;\n      var previousPriority = ReactDOMSharedInternals.p;\n      try {\n        (ReactDOMSharedInternals.p = DiscreteEventPriority),\n          dispatchEvent(domEventName, eventSystemFlags, container, nativeEvent);\n      } finally {\n        (ReactDOMSharedInternals.p = previousPriority),\n          (ReactSharedInternals.T = prevTransition);\n      }\n    }\n    function dispatchContinuousEvent(\n      domEventName,\n      eventSystemFlags,\n      container,\n      nativeEvent\n    ) {\n      var prevTransition = ReactSharedInternals.T;\n      ReactSharedInternals.T = null;\n      var previousPriority = ReactDOMSharedInternals.p;\n      try {\n        (ReactDOMSharedInternals.p = ContinuousEventPriority),\n          dispatchEvent(domEventName, eventSystemFlags, container, nativeEvent);\n      } finally {\n        (ReactDOMSharedInternals.p = previousPriority),\n          (ReactSharedInternals.T = prevTransition);\n      }\n    }\n    function dispatchEvent(\n      domEventName,\n      eventSystemFlags,\n      targetContainer,\n      nativeEvent\n    ) {\n      if (_enabled) {\n        var blockedOn = findInstanceBlockingEvent(nativeEvent);\n        if (null === blockedOn)\n          dispatchEventForPluginEventSystem(\n            domEventName,\n            eventSystemFlags,\n            nativeEvent,\n            return_targetInst,\n            targetContainer\n          ),\n            clearIfContinuousEvent(domEventName, nativeEvent);\n        else if (\n          queueIfContinuousEvent(\n            blockedOn,\n            domEventName,\n            eventSystemFlags,\n            targetContainer,\n            nativeEvent\n          )\n        )\n          nativeEvent.stopPropagation();\n        else if (\n          (clearIfContinuousEvent(domEventName, nativeEvent),\n          eventSystemFlags & 4 &&\n            -1 < discreteReplayableEvents.indexOf(domEventName))\n        ) {\n          for (; null !== blockedOn; ) {\n            var fiber = getInstanceFromNode(blockedOn);\n            if (null !== fiber)\n              switch (fiber.tag) {\n                case 3:\n                  fiber = fiber.stateNode;\n                  if (fiber.current.memoizedState.isDehydrated) {\n                    var lanes = getHighestPriorityLanes(fiber.pendingLanes);\n                    if (0 !== lanes) {\n                      var root = fiber;\n                      root.pendingLanes |= 2;\n                      for (root.entangledLanes |= 2; lanes; ) {\n                        var lane = 1 << (31 - clz32(lanes));\n                        root.entanglements[1] |= lane;\n                        lanes &= ~lane;\n                      }\n                      ensureRootIsScheduled(fiber);\n                      (executionContext & (RenderContext | CommitContext)) ===\n                        NoContext &&\n                        ((workInProgressRootRenderTargetTime =\n                          now$1() + RENDER_TIMEOUT_MS),\n                        flushSyncWorkAcrossRoots_impl(0, !1));\n                    }\n                  }\n                  break;\n                case 13:\n                  (root = enqueueConcurrentRenderForLane(fiber, 2)),\n                    null !== root && scheduleUpdateOnFiber(root, fiber, 2),\n                    flushSyncWork$1(),\n                    markRetryLaneIfNotHydrated(fiber, 2);\n              }\n            fiber = findInstanceBlockingEvent(nativeEvent);\n            null === fiber &&\n              dispatchEventForPluginEventSystem(\n                domEventName,\n                eventSystemFlags,\n                nativeEvent,\n                return_targetInst,\n                targetContainer\n              );\n            if (fiber === blockedOn) break;\n            blockedOn = fiber;\n          }\n          null !== blockedOn && nativeEvent.stopPropagation();\n        } else\n          dispatchEventForPluginEventSystem(\n            domEventName,\n            eventSystemFlags,\n            nativeEvent,\n            null,\n            targetContainer\n          );\n      }\n    }\n    function findInstanceBlockingEvent(nativeEvent) {\n      nativeEvent = getEventTarget(nativeEvent);\n      return findInstanceBlockingTarget(nativeEvent);\n    }\n    function findInstanceBlockingTarget(targetNode) {\n      return_targetInst = null;\n      targetNode = getClosestInstanceFromNode(targetNode);\n      if (null !== targetNode) {\n        var nearestMounted = getNearestMountedFiber(targetNode);\n        if (null === nearestMounted) targetNode = null;\n        else {\n          var tag = nearestMounted.tag;\n          if (13 === tag) {\n            targetNode = getSuspenseInstanceFromFiber(nearestMounted);\n            if (null !== targetNode) return targetNode;\n            targetNode = null;\n          } else if (3 === tag) {\n            if (nearestMounted.stateNode.current.memoizedState.isDehydrated)\n              return 3 === nearestMounted.tag\n                ? nearestMounted.stateNode.containerInfo\n                : null;\n            targetNode = null;\n          } else nearestMounted !== targetNode && (targetNode = null);\n        }\n      }\n      return_targetInst = targetNode;\n      return null;\n    }\n    function getEventPriority(domEventName) {\n      switch (domEventName) {\n        case \"beforetoggle\":\n        case \"cancel\":\n        case \"click\":\n        case \"close\":\n        case \"contextmenu\":\n        case \"copy\":\n        case \"cut\":\n        case \"auxclick\":\n        case \"dblclick\":\n        case \"dragend\":\n        case \"dragstart\":\n        case \"drop\":\n        case \"focusin\":\n        case \"focusout\":\n        case \"input\":\n        case \"invalid\":\n        case \"keydown\":\n        case \"keypress\":\n        case \"keyup\":\n        case \"mousedown\":\n        case \"mouseup\":\n        case \"paste\":\n        case \"pause\":\n        case \"play\":\n        case \"pointercancel\":\n        case \"pointerdown\":\n        case \"pointerup\":\n        case \"ratechange\":\n        case \"reset\":\n        case \"resize\":\n        case \"seeked\":\n        case \"submit\":\n        case \"toggle\":\n        case \"touchcancel\":\n        case \"touchend\":\n        case \"touchstart\":\n        case \"volumechange\":\n        case \"change\":\n        case \"selectionchange\":\n        case \"textInput\":\n        case \"compositionstart\":\n        case \"compositionend\":\n        case \"compositionupdate\":\n        case \"beforeblur\":\n        case \"afterblur\":\n        case \"beforeinput\":\n        case \"blur\":\n        case \"fullscreenchange\":\n        case \"focus\":\n        case \"hashchange\":\n        case \"popstate\":\n        case \"select\":\n        case \"selectstart\":\n          return DiscreteEventPriority;\n        case \"drag\":\n        case \"dragenter\":\n        case \"dragexit\":\n        case \"dragleave\":\n        case \"dragover\":\n        case \"mousemove\":\n        case \"mouseout\":\n        case \"mouseover\":\n        case \"pointermove\":\n        case \"pointerout\":\n        case \"pointerover\":\n        case \"scroll\":\n        case \"touchmove\":\n        case \"wheel\":\n        case \"mouseenter\":\n        case \"mouseleave\":\n        case \"pointerenter\":\n        case \"pointerleave\":\n          return ContinuousEventPriority;\n        case \"message\":\n          switch (getCurrentPriorityLevel()) {\n            case ImmediatePriority:\n              return DiscreteEventPriority;\n            case UserBlockingPriority:\n              return ContinuousEventPriority;\n            case NormalPriority$1:\n            case LowPriority:\n              return DefaultEventPriority;\n            case IdlePriority:\n              return IdleEventPriority;\n            default:\n              return DefaultEventPriority;\n          }\n        default:\n          return DefaultEventPriority;\n      }\n    }\n    function clearIfContinuousEvent(domEventName, nativeEvent) {\n      switch (domEventName) {\n        case \"focusin\":\n        case \"focusout\":\n          queuedFocus = null;\n          break;\n        case \"dragenter\":\n        case \"dragleave\":\n          queuedDrag = null;\n          break;\n        case \"mouseover\":\n        case \"mouseout\":\n          queuedMouse = null;\n          break;\n        case \"pointerover\":\n        case \"pointerout\":\n          queuedPointers.delete(nativeEvent.pointerId);\n          break;\n        case \"gotpointercapture\":\n        case \"lostpointercapture\":\n          queuedPointerCaptures.delete(nativeEvent.pointerId);\n      }\n    }\n    function accumulateOrCreateContinuousQueuedReplayableEvent(\n      existingQueuedEvent,\n      blockedOn,\n      domEventName,\n      eventSystemFlags,\n      targetContainer,\n      nativeEvent\n    ) {\n      if (\n        null === existingQueuedEvent ||\n        existingQueuedEvent.nativeEvent !== nativeEvent\n      )\n        return (\n          (existingQueuedEvent = {\n            blockedOn: blockedOn,\n            domEventName: domEventName,\n            eventSystemFlags: eventSystemFlags,\n            nativeEvent: nativeEvent,\n            targetContainers: [targetContainer]\n          }),\n          null !== blockedOn &&\n            ((blockedOn = getInstanceFromNode(blockedOn)),\n            null !== blockedOn && attemptContinuousHydration(blockedOn)),\n          existingQueuedEvent\n        );\n      existingQueuedEvent.eventSystemFlags |= eventSystemFlags;\n      blockedOn = existingQueuedEvent.targetContainers;\n      null !== targetContainer &&\n        -1 === blockedOn.indexOf(targetContainer) &&\n        blockedOn.push(targetContainer);\n      return existingQueuedEvent;\n    }\n    function queueIfContinuousEvent(\n      blockedOn,\n      domEventName,\n      eventSystemFlags,\n      targetContainer,\n      nativeEvent\n    ) {\n      switch (domEventName) {\n        case \"focusin\":\n          return (\n            (queuedFocus = accumulateOrCreateContinuousQueuedReplayableEvent(\n              queuedFocus,\n              blockedOn,\n              domEventName,\n              eventSystemFlags,\n              targetContainer,\n              nativeEvent\n            )),\n            !0\n          );\n        case \"dragenter\":\n          return (\n            (queuedDrag = accumulateOrCreateContinuousQueuedReplayableEvent(\n              queuedDrag,\n              blockedOn,\n              domEventName,\n              eventSystemFlags,\n              targetContainer,\n              nativeEvent\n            )),\n            !0\n          );\n        case \"mouseover\":\n          return (\n            (queuedMouse = accumulateOrCreateContinuousQueuedReplayableEvent(\n              queuedMouse,\n              blockedOn,\n              domEventName,\n              eventSystemFlags,\n              targetContainer,\n              nativeEvent\n            )),\n            !0\n          );\n        case \"pointerover\":\n          var pointerId = nativeEvent.pointerId;\n          queuedPointers.set(\n            pointerId,\n            accumulateOrCreateContinuousQueuedReplayableEvent(\n              queuedPointers.get(pointerId) || null,\n              blockedOn,\n              domEventName,\n              eventSystemFlags,\n              targetContainer,\n              nativeEvent\n            )\n          );\n          return !0;\n        case \"gotpointercapture\":\n          return (\n            (pointerId = nativeEvent.pointerId),\n            queuedPointerCaptures.set(\n              pointerId,\n              accumulateOrCreateContinuousQueuedReplayableEvent(\n                queuedPointerCaptures.get(pointerId) || null,\n                blockedOn,\n                domEventName,\n                eventSystemFlags,\n                targetContainer,\n                nativeEvent\n              )\n            ),\n            !0\n          );\n      }\n      return !1;\n    }\n    function attemptExplicitHydrationTarget(queuedTarget) {\n      var targetInst = getClosestInstanceFromNode(queuedTarget.target);\n      if (null !== targetInst) {\n        var nearestMounted = getNearestMountedFiber(targetInst);\n        if (null !== nearestMounted)\n          if (((targetInst = nearestMounted.tag), 13 === targetInst)) {\n            if (\n              ((targetInst = getSuspenseInstanceFromFiber(nearestMounted)),\n              null !== targetInst)\n            ) {\n              queuedTarget.blockedOn = targetInst;\n              runWithPriority(queuedTarget.priority, function () {\n                if (13 === nearestMounted.tag) {\n                  var lane = requestUpdateLane(nearestMounted);\n                  lane = getBumpedLaneForHydrationByLane(lane);\n                  var root = enqueueConcurrentRenderForLane(\n                    nearestMounted,\n                    lane\n                  );\n                  null !== root &&\n                    scheduleUpdateOnFiber(root, nearestMounted, lane);\n                  markRetryLaneIfNotHydrated(nearestMounted, lane);\n                }\n              });\n              return;\n            }\n          } else if (\n            3 === targetInst &&\n            nearestMounted.stateNode.current.memoizedState.isDehydrated\n          ) {\n            queuedTarget.blockedOn =\n              3 === nearestMounted.tag\n                ? nearestMounted.stateNode.containerInfo\n                : null;\n            return;\n          }\n      }\n      queuedTarget.blockedOn = null;\n    }\n    function attemptReplayContinuousQueuedEvent(queuedEvent) {\n      if (null !== queuedEvent.blockedOn) return !1;\n      for (\n        var targetContainers = queuedEvent.targetContainers;\n        0 < targetContainers.length;\n\n      ) {\n        var nextBlockedOn = findInstanceBlockingEvent(queuedEvent.nativeEvent);\n        if (null === nextBlockedOn) {\n          nextBlockedOn = queuedEvent.nativeEvent;\n          var nativeEventClone = new nextBlockedOn.constructor(\n              nextBlockedOn.type,\n              nextBlockedOn\n            ),\n            event = nativeEventClone;\n          null !== currentReplayingEvent &&\n            console.error(\n              \"Expected currently replaying event to be null. This error is likely caused by a bug in React. Please file an issue.\"\n            );\n          currentReplayingEvent = event;\n          nextBlockedOn.target.dispatchEvent(nativeEventClone);\n          null === currentReplayingEvent &&\n            console.error(\n              \"Expected currently replaying event to not be null. This error is likely caused by a bug in React. Please file an issue.\"\n            );\n          currentReplayingEvent = null;\n        } else\n          return (\n            (targetContainers = getInstanceFromNode(nextBlockedOn)),\n            null !== targetContainers &&\n              attemptContinuousHydration(targetContainers),\n            (queuedEvent.blockedOn = nextBlockedOn),\n            !1\n          );\n        targetContainers.shift();\n      }\n      return !0;\n    }\n    function attemptReplayContinuousQueuedEventInMap(queuedEvent, key, map) {\n      attemptReplayContinuousQueuedEvent(queuedEvent) && map.delete(key);\n    }\n    function replayUnblockedEvents() {\n      hasScheduledReplayAttempt = !1;\n      null !== queuedFocus &&\n        attemptReplayContinuousQueuedEvent(queuedFocus) &&\n        (queuedFocus = null);\n      null !== queuedDrag &&\n        attemptReplayContinuousQueuedEvent(queuedDrag) &&\n        (queuedDrag = null);\n      null !== queuedMouse &&\n        attemptReplayContinuousQueuedEvent(queuedMouse) &&\n        (queuedMouse = null);\n      queuedPointers.forEach(attemptReplayContinuousQueuedEventInMap);\n      queuedPointerCaptures.forEach(attemptReplayContinuousQueuedEventInMap);\n    }\n    function scheduleCallbackIfUnblocked(queuedEvent, unblocked) {\n      queuedEvent.blockedOn === unblocked &&\n        ((queuedEvent.blockedOn = null),\n        hasScheduledReplayAttempt ||\n          ((hasScheduledReplayAttempt = !0),\n          Scheduler.unstable_scheduleCallback(\n            Scheduler.unstable_NormalPriority,\n            replayUnblockedEvents\n          )));\n    }\n    function scheduleReplayQueueIfNeeded(formReplayingQueue) {\n      lastScheduledReplayQueue !== formReplayingQueue &&\n        ((lastScheduledReplayQueue = formReplayingQueue),\n        Scheduler.unstable_scheduleCallback(\n          Scheduler.unstable_NormalPriority,\n          function () {\n            lastScheduledReplayQueue === formReplayingQueue &&\n              (lastScheduledReplayQueue = null);\n            for (var i = 0; i < formReplayingQueue.length; i += 3) {\n              var form = formReplayingQueue[i],\n                submitterOrAction = formReplayingQueue[i + 1],\n                formData = formReplayingQueue[i + 2];\n              if (\"function\" !== typeof submitterOrAction)\n                if (\n                  null === findInstanceBlockingTarget(submitterOrAction || form)\n                )\n                  continue;\n                else break;\n              var formInst = getInstanceFromNode(form);\n              null !== formInst &&\n                (formReplayingQueue.splice(i, 3),\n                (i -= 3),\n                (form = {\n                  pending: !0,\n                  data: formData,\n                  method: form.method,\n                  action: submitterOrAction\n                }),\n                Object.freeze(form),\n                startHostTransition(\n                  formInst,\n                  form,\n                  submitterOrAction,\n                  formData\n                ));\n            }\n          }\n        ));\n    }\n    function retryIfBlockedOn(unblocked) {\n      function unblock(queuedEvent) {\n        return scheduleCallbackIfUnblocked(queuedEvent, unblocked);\n      }\n      null !== queuedFocus &&\n        scheduleCallbackIfUnblocked(queuedFocus, unblocked);\n      null !== queuedDrag && scheduleCallbackIfUnblocked(queuedDrag, unblocked);\n      null !== queuedMouse &&\n        scheduleCallbackIfUnblocked(queuedMouse, unblocked);\n      queuedPointers.forEach(unblock);\n      queuedPointerCaptures.forEach(unblock);\n      for (var i = 0; i < queuedExplicitHydrationTargets.length; i++) {\n        var queuedTarget = queuedExplicitHydrationTargets[i];\n        queuedTarget.blockedOn === unblocked && (queuedTarget.blockedOn = null);\n      }\n      for (\n        ;\n        0 < queuedExplicitHydrationTargets.length &&\n        ((i = queuedExplicitHydrationTargets[0]), null === i.blockedOn);\n\n      )\n        attemptExplicitHydrationTarget(i),\n          null === i.blockedOn && queuedExplicitHydrationTargets.shift();\n      i = (unblocked.ownerDocument || unblocked).$$reactFormReplay;\n      if (null != i)\n        for (queuedTarget = 0; queuedTarget < i.length; queuedTarget += 3) {\n          var form = i[queuedTarget],\n            submitterOrAction = i[queuedTarget + 1],\n            formProps = form[internalPropsKey] || null;\n          if (\"function\" === typeof submitterOrAction)\n            formProps || scheduleReplayQueueIfNeeded(i);\n          else if (formProps) {\n            var action = null;\n            if (\n              submitterOrAction &&\n              submitterOrAction.hasAttribute(\"formAction\")\n            )\n              if (\n                ((form = submitterOrAction),\n                (formProps = submitterOrAction[internalPropsKey] || null))\n              )\n                action = formProps.formAction;\n              else {\n                if (null !== findInstanceBlockingTarget(form)) continue;\n              }\n            else action = formProps.action;\n            \"function\" === typeof action\n              ? (i[queuedTarget + 1] = action)\n              : (i.splice(queuedTarget, 3), (queuedTarget -= 3));\n            scheduleReplayQueueIfNeeded(i);\n          }\n        }\n    }\n    function ReactDOMRoot(internalRoot) {\n      this._internalRoot = internalRoot;\n    }\n    function ReactDOMHydrationRoot(internalRoot) {\n      this._internalRoot = internalRoot;\n    }\n    function warnIfReactDOMContainerInDEV(container) {\n      container[internalContainerInstanceKey] &&\n        (container._reactRootContainer\n          ? console.error(\n              \"You are calling ReactDOMClient.createRoot() on a container that was previously passed to ReactDOM.render(). This is not supported.\"\n            )\n          : console.error(\n              \"You are calling ReactDOMClient.createRoot() on a container that has already been passed to createRoot() before. Instead, call root.render() on the existing root instead if you want to update it.\"\n            ));\n    }\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(Error());\n    var Scheduler = __webpack_require__(/*! scheduler */ \"./node_modules/scheduler/index.js\"),\n      React = __webpack_require__(/*! react */ \"./node_modules/react/index.js\"),\n      ReactDOM = __webpack_require__(/*! react-dom */ \"./node_modules/react-dom/index.js\"),\n      assign = Object.assign,\n      REACT_LEGACY_ELEMENT_TYPE = Symbol.for(\"react.element\"),\n      REACT_ELEMENT_TYPE = Symbol.for(\"react.transitional.element\"),\n      REACT_PORTAL_TYPE = Symbol.for(\"react.portal\"),\n      REACT_FRAGMENT_TYPE = Symbol.for(\"react.fragment\"),\n      REACT_STRICT_MODE_TYPE = Symbol.for(\"react.strict_mode\"),\n      REACT_PROFILER_TYPE = Symbol.for(\"react.profiler\"),\n      REACT_PROVIDER_TYPE = Symbol.for(\"react.provider\"),\n      REACT_CONSUMER_TYPE = Symbol.for(\"react.consumer\"),\n      REACT_CONTEXT_TYPE = Symbol.for(\"react.context\"),\n      REACT_FORWARD_REF_TYPE = Symbol.for(\"react.forward_ref\"),\n      REACT_SUSPENSE_TYPE = Symbol.for(\"react.suspense\"),\n      REACT_SUSPENSE_LIST_TYPE = Symbol.for(\"react.suspense_list\"),\n      REACT_MEMO_TYPE = Symbol.for(\"react.memo\"),\n      REACT_LAZY_TYPE = Symbol.for(\"react.lazy\");\n    Symbol.for(\"react.scope\");\n    var REACT_ACTIVITY_TYPE = Symbol.for(\"react.activity\");\n    Symbol.for(\"react.legacy_hidden\");\n    Symbol.for(\"react.tracing_marker\");\n    var REACT_MEMO_CACHE_SENTINEL = Symbol.for(\"react.memo_cache_sentinel\");\n    Symbol.for(\"react.view_transition\");\n    var MAYBE_ITERATOR_SYMBOL = Symbol.iterator,\n      REACT_CLIENT_REFERENCE = Symbol.for(\"react.client.reference\"),\n      isArrayImpl = Array.isArray,\n      ReactSharedInternals =\n        React.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,\n      ReactDOMSharedInternals =\n        ReactDOM.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,\n      NotPending = Object.freeze({\n        pending: !1,\n        data: null,\n        method: null,\n        action: null\n      }),\n      valueStack = [];\n    var fiberStack = [];\n    var index$jscomp$0 = -1,\n      contextStackCursor = createCursor(null),\n      contextFiberStackCursor = createCursor(null),\n      rootInstanceStackCursor = createCursor(null),\n      hostTransitionProviderCursor = createCursor(null),\n      hasOwnProperty = Object.prototype.hasOwnProperty,\n      scheduleCallback$3 = Scheduler.unstable_scheduleCallback,\n      cancelCallback$1 = Scheduler.unstable_cancelCallback,\n      shouldYield = Scheduler.unstable_shouldYield,\n      requestPaint = Scheduler.unstable_requestPaint,\n      now$1 = Scheduler.unstable_now,\n      getCurrentPriorityLevel = Scheduler.unstable_getCurrentPriorityLevel,\n      ImmediatePriority = Scheduler.unstable_ImmediatePriority,\n      UserBlockingPriority = Scheduler.unstable_UserBlockingPriority,\n      NormalPriority$1 = Scheduler.unstable_NormalPriority,\n      LowPriority = Scheduler.unstable_LowPriority,\n      IdlePriority = Scheduler.unstable_IdlePriority,\n      log$1 = Scheduler.log,\n      unstable_setDisableYieldValue = Scheduler.unstable_setDisableYieldValue,\n      rendererID = null,\n      injectedHook = null,\n      injectedProfilingHooks = null,\n      hasLoggedError = !1,\n      isDevToolsPresent = \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__,\n      clz32 = Math.clz32 ? Math.clz32 : clz32Fallback,\n      log = Math.log,\n      LN2 = Math.LN2,\n      nextTransitionLane = 256,\n      nextRetryLane = 4194304,\n      DiscreteEventPriority = 2,\n      ContinuousEventPriority = 8,\n      DefaultEventPriority = 32,\n      IdleEventPriority = 268435456,\n      randomKey = Math.random().toString(36).slice(2),\n      internalInstanceKey = \"__reactFiber$\" + randomKey,\n      internalPropsKey = \"__reactProps$\" + randomKey,\n      internalContainerInstanceKey = \"__reactContainer$\" + randomKey,\n      internalEventHandlersKey = \"__reactEvents$\" + randomKey,\n      internalEventHandlerListenersKey = \"__reactListeners$\" + randomKey,\n      internalEventHandlesSetKey = \"__reactHandles$\" + randomKey,\n      internalRootNodeResourcesKey = \"__reactResources$\" + randomKey,\n      internalHoistableMarker = \"__reactMarker$\" + randomKey,\n      allNativeEvents = new Set(),\n      registrationNameDependencies = {},\n      possibleRegistrationNames = {},\n      hasReadOnlyValue = {\n        button: !0,\n        checkbox: !0,\n        image: !0,\n        hidden: !0,\n        radio: !0,\n        reset: !0,\n        submit: !0\n      },\n      VALID_ATTRIBUTE_NAME_REGEX = RegExp(\n        \"^[:A-Z_a-z\\\\u00C0-\\\\u00D6\\\\u00D8-\\\\u00F6\\\\u00F8-\\\\u02FF\\\\u0370-\\\\u037D\\\\u037F-\\\\u1FFF\\\\u200C-\\\\u200D\\\\u2070-\\\\u218F\\\\u2C00-\\\\u2FEF\\\\u3001-\\\\uD7FF\\\\uF900-\\\\uFDCF\\\\uFDF0-\\\\uFFFD][:A-Z_a-z\\\\u00C0-\\\\u00D6\\\\u00D8-\\\\u00F6\\\\u00F8-\\\\u02FF\\\\u0370-\\\\u037D\\\\u037F-\\\\u1FFF\\\\u200C-\\\\u200D\\\\u2070-\\\\u218F\\\\u2C00-\\\\u2FEF\\\\u3001-\\\\uD7FF\\\\uF900-\\\\uFDCF\\\\uFDF0-\\\\uFFFD\\\\-.0-9\\\\u00B7\\\\u0300-\\\\u036F\\\\u203F-\\\\u2040]*$\"\n      ),\n      illegalAttributeNameCache = {},\n      validatedAttributeNameCache = {},\n      disabledDepth = 0,\n      prevLog,\n      prevInfo,\n      prevWarn,\n      prevError,\n      prevGroup,\n      prevGroupCollapsed,\n      prevGroupEnd;\n    disabledLog.__reactDisabledLog = !0;\n    var prefix,\n      suffix,\n      reentry = !1;\n    var componentFrameCache = new (\n      \"function\" === typeof WeakMap ? WeakMap : Map\n    )();\n    var current = null,\n      isRendering = !1,\n      escapeSelectorAttributeValueInsideDoubleQuotesRegex = /[\\n\"\\\\]/g,\n      didWarnValueDefaultValue$1 = !1,\n      didWarnCheckedDefaultChecked = !1,\n      didWarnSelectedSetOnOption = !1,\n      didWarnInvalidChild = !1,\n      didWarnInvalidInnerHTML = !1;\n    var didWarnValueDefaultValue = !1;\n    var valuePropNames = [\"value\", \"defaultValue\"],\n      didWarnValDefaultVal = !1,\n      needsEscaping = /[\"'&<>\\n\\t]|^\\s|\\s$/,\n      specialTags =\n        \"address applet area article aside base basefont bgsound blockquote body br button caption center col colgroup dd details dir div dl dt embed fieldset figcaption figure footer form frame frameset h1 h2 h3 h4 h5 h6 head header hgroup hr html iframe img input isindex li link listing main marquee menu menuitem meta nav noembed noframes noscript object ol p param plaintext pre script section select source style summary table tbody td template textarea tfoot th thead title tr track ul wbr xmp\".split(\n          \" \"\n        ),\n      inScopeTags =\n        \"applet caption html table td th marquee object template foreignObject desc title\".split(\n          \" \"\n        ),\n      buttonScopeTags = inScopeTags.concat([\"button\"]),\n      impliedEndTags = \"dd dt li option optgroup p rp rt\".split(\" \"),\n      emptyAncestorInfoDev = {\n        current: null,\n        formTag: null,\n        aTagInScope: null,\n        buttonTagInScope: null,\n        nobrTagInScope: null,\n        pTagInButtonScope: null,\n        listItemTagAutoclosing: null,\n        dlItemTagAutoclosing: null,\n        containerTagInScope: null,\n        implicitRootScope: !1\n      },\n      didWarn = {},\n      shorthandToLonghand = {\n        animation:\n          \"animationDelay animationDirection animationDuration animationFillMode animationIterationCount animationName animationPlayState animationTimingFunction\".split(\n            \" \"\n          ),\n        background:\n          \"backgroundAttachment backgroundClip backgroundColor backgroundImage backgroundOrigin backgroundPositionX backgroundPositionY backgroundRepeat backgroundSize\".split(\n            \" \"\n          ),\n        backgroundPosition: [\"backgroundPositionX\", \"backgroundPositionY\"],\n        border:\n          \"borderBottomColor borderBottomStyle borderBottomWidth borderImageOutset borderImageRepeat borderImageSlice borderImageSource borderImageWidth borderLeftColor borderLeftStyle borderLeftWidth borderRightColor borderRightStyle borderRightWidth borderTopColor borderTopStyle borderTopWidth\".split(\n            \" \"\n          ),\n        borderBlockEnd: [\n          \"borderBlockEndColor\",\n          \"borderBlockEndStyle\",\n          \"borderBlockEndWidth\"\n        ],\n        borderBlockStart: [\n          \"borderBlockStartColor\",\n          \"borderBlockStartStyle\",\n          \"borderBlockStartWidth\"\n        ],\n        borderBottom: [\n          \"borderBottomColor\",\n          \"borderBottomStyle\",\n          \"borderBottomWidth\"\n        ],\n        borderColor: [\n          \"borderBottomColor\",\n          \"borderLeftColor\",\n          \"borderRightColor\",\n          \"borderTopColor\"\n        ],\n        borderImage: [\n          \"borderImageOutset\",\n          \"borderImageRepeat\",\n          \"borderImageSlice\",\n          \"borderImageSource\",\n          \"borderImageWidth\"\n        ],\n        borderInlineEnd: [\n          \"borderInlineEndColor\",\n          \"borderInlineEndStyle\",\n          \"borderInlineEndWidth\"\n        ],\n        borderInlineStart: [\n          \"borderInlineStartColor\",\n          \"borderInlineStartStyle\",\n          \"borderInlineStartWidth\"\n        ],\n        borderLeft: [\"borderLeftColor\", \"borderLeftStyle\", \"borderLeftWidth\"],\n        borderRadius: [\n          \"borderBottomLeftRadius\",\n          \"borderBottomRightRadius\",\n          \"borderTopLeftRadius\",\n          \"borderTopRightRadius\"\n        ],\n        borderRight: [\n          \"borderRightColor\",\n          \"borderRightStyle\",\n          \"borderRightWidth\"\n        ],\n        borderStyle: [\n          \"borderBottomStyle\",\n          \"borderLeftStyle\",\n          \"borderRightStyle\",\n          \"borderTopStyle\"\n        ],\n        borderTop: [\"borderTopColor\", \"borderTopStyle\", \"borderTopWidth\"],\n        borderWidth: [\n          \"borderBottomWidth\",\n          \"borderLeftWidth\",\n          \"borderRightWidth\",\n          \"borderTopWidth\"\n        ],\n        columnRule: [\"columnRuleColor\", \"columnRuleStyle\", \"columnRuleWidth\"],\n        columns: [\"columnCount\", \"columnWidth\"],\n        flex: [\"flexBasis\", \"flexGrow\", \"flexShrink\"],\n        flexFlow: [\"flexDirection\", \"flexWrap\"],\n        font: \"fontFamily fontFeatureSettings fontKerning fontLanguageOverride fontSize fontSizeAdjust fontStretch fontStyle fontVariant fontVariantAlternates fontVariantCaps fontVariantEastAsian fontVariantLigatures fontVariantNumeric fontVariantPosition fontWeight lineHeight\".split(\n          \" \"\n        ),\n        fontVariant:\n          \"fontVariantAlternates fontVariantCaps fontVariantEastAsian fontVariantLigatures fontVariantNumeric fontVariantPosition\".split(\n            \" \"\n          ),\n        gap: [\"columnGap\", \"rowGap\"],\n        grid: \"gridAutoColumns gridAutoFlow gridAutoRows gridTemplateAreas gridTemplateColumns gridTemplateRows\".split(\n          \" \"\n        ),\n        gridArea: [\n          \"gridColumnEnd\",\n          \"gridColumnStart\",\n          \"gridRowEnd\",\n          \"gridRowStart\"\n        ],\n        gridColumn: [\"gridColumnEnd\", \"gridColumnStart\"],\n        gridColumnGap: [\"columnGap\"],\n        gridGap: [\"columnGap\", \"rowGap\"],\n        gridRow: [\"gridRowEnd\", \"gridRowStart\"],\n        gridRowGap: [\"rowGap\"],\n        gridTemplate: [\n          \"gridTemplateAreas\",\n          \"gridTemplateColumns\",\n          \"gridTemplateRows\"\n        ],\n        listStyle: [\"listStyleImage\", \"listStylePosition\", \"listStyleType\"],\n        margin: [\"marginBottom\", \"marginLeft\", \"marginRight\", \"marginTop\"],\n        marker: [\"markerEnd\", \"markerMid\", \"markerStart\"],\n        mask: \"maskClip maskComposite maskImage maskMode maskOrigin maskPositionX maskPositionY maskRepeat maskSize\".split(\n          \" \"\n        ),\n        maskPosition: [\"maskPositionX\", \"maskPositionY\"],\n        outline: [\"outlineColor\", \"outlineStyle\", \"outlineWidth\"],\n        overflow: [\"overflowX\", \"overflowY\"],\n        padding: [\"paddingBottom\", \"paddingLeft\", \"paddingRight\", \"paddingTop\"],\n        placeContent: [\"alignContent\", \"justifyContent\"],\n        placeItems: [\"alignItems\", \"justifyItems\"],\n        placeSelf: [\"alignSelf\", \"justifySelf\"],\n        textDecoration: [\n          \"textDecorationColor\",\n          \"textDecorationLine\",\n          \"textDecorationStyle\"\n        ],\n        textEmphasis: [\"textEmphasisColor\", \"textEmphasisStyle\"],\n        transition: [\n          \"transitionDelay\",\n          \"transitionDuration\",\n          \"transitionProperty\",\n          \"transitionTimingFunction\"\n        ],\n        wordWrap: [\"overflowWrap\"]\n      },\n      uppercasePattern = /([A-Z])/g,\n      msPattern$1 = /^ms-/,\n      badVendoredStyleNamePattern = /^(?:webkit|moz|o)[A-Z]/,\n      msPattern = /^-ms-/,\n      hyphenPattern = /-(.)/g,\n      badStyleValueWithSemicolonPattern = /;\\s*$/,\n      warnedStyleNames = {},\n      warnedStyleValues = {},\n      warnedForNaNValue = !1,\n      warnedForInfinityValue = !1,\n      unitlessNumbers = new Set(\n        \"animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp\".split(\n          \" \"\n        )\n      ),\n      MATH_NAMESPACE = \"http://www.w3.org/1998/Math/MathML\",\n      SVG_NAMESPACE = \"http://www.w3.org/2000/svg\",\n      aliases = new Map([\n        [\"acceptCharset\", \"accept-charset\"],\n        [\"htmlFor\", \"for\"],\n        [\"httpEquiv\", \"http-equiv\"],\n        [\"crossOrigin\", \"crossorigin\"],\n        [\"accentHeight\", \"accent-height\"],\n        [\"alignmentBaseline\", \"alignment-baseline\"],\n        [\"arabicForm\", \"arabic-form\"],\n        [\"baselineShift\", \"baseline-shift\"],\n        [\"capHeight\", \"cap-height\"],\n        [\"clipPath\", \"clip-path\"],\n        [\"clipRule\", \"clip-rule\"],\n        [\"colorInterpolation\", \"color-interpolation\"],\n        [\"colorInterpolationFilters\", \"color-interpolation-filters\"],\n        [\"colorProfile\", \"color-profile\"],\n        [\"colorRendering\", \"color-rendering\"],\n        [\"dominantBaseline\", \"dominant-baseline\"],\n        [\"enableBackground\", \"enable-background\"],\n        [\"fillOpacity\", \"fill-opacity\"],\n        [\"fillRule\", \"fill-rule\"],\n        [\"floodColor\", \"flood-color\"],\n        [\"floodOpacity\", \"flood-opacity\"],\n        [\"fontFamily\", \"font-family\"],\n        [\"fontSize\", \"font-size\"],\n        [\"fontSizeAdjust\", \"font-size-adjust\"],\n        [\"fontStretch\", \"font-stretch\"],\n        [\"fontStyle\", \"font-style\"],\n        [\"fontVariant\", \"font-variant\"],\n        [\"fontWeight\", \"font-weight\"],\n        [\"glyphName\", \"glyph-name\"],\n        [\"glyphOrientationHorizontal\", \"glyph-orientation-horizontal\"],\n        [\"glyphOrientationVertical\", \"glyph-orientation-vertical\"],\n        [\"horizAdvX\", \"horiz-adv-x\"],\n        [\"horizOriginX\", \"horiz-origin-x\"],\n        [\"imageRendering\", \"image-rendering\"],\n        [\"letterSpacing\", \"letter-spacing\"],\n        [\"lightingColor\", \"lighting-color\"],\n        [\"markerEnd\", \"marker-end\"],\n        [\"markerMid\", \"marker-mid\"],\n        [\"markerStart\", \"marker-start\"],\n        [\"overlinePosition\", \"overline-position\"],\n        [\"overlineThickness\", \"overline-thickness\"],\n        [\"paintOrder\", \"paint-order\"],\n        [\"panose-1\", \"panose-1\"],\n        [\"pointerEvents\", \"pointer-events\"],\n        [\"renderingIntent\", \"rendering-intent\"],\n        [\"shapeRendering\", \"shape-rendering\"],\n        [\"stopColor\", \"stop-color\"],\n        [\"stopOpacity\", \"stop-opacity\"],\n        [\"strikethroughPosition\", \"strikethrough-position\"],\n        [\"strikethroughThickness\", \"strikethrough-thickness\"],\n        [\"strokeDasharray\", \"stroke-dasharray\"],\n        [\"strokeDashoffset\", \"stroke-dashoffset\"],\n        [\"strokeLinecap\", \"stroke-linecap\"],\n        [\"strokeLinejoin\", \"stroke-linejoin\"],\n        [\"strokeMiterlimit\", \"stroke-miterlimit\"],\n        [\"strokeOpacity\", \"stroke-opacity\"],\n        [\"strokeWidth\", \"stroke-width\"],\n        [\"textAnchor\", \"text-anchor\"],\n        [\"textDecoration\", \"text-decoration\"],\n        [\"textRendering\", \"text-rendering\"],\n        [\"transformOrigin\", \"transform-origin\"],\n        [\"underlinePosition\", \"underline-position\"],\n        [\"underlineThickness\", \"underline-thickness\"],\n        [\"unicodeBidi\", \"unicode-bidi\"],\n        [\"unicodeRange\", \"unicode-range\"],\n        [\"unitsPerEm\", \"units-per-em\"],\n        [\"vAlphabetic\", \"v-alphabetic\"],\n        [\"vHanging\", \"v-hanging\"],\n        [\"vIdeographic\", \"v-ideographic\"],\n        [\"vMathematical\", \"v-mathematical\"],\n        [\"vectorEffect\", \"vector-effect\"],\n        [\"vertAdvY\", \"vert-adv-y\"],\n        [\"vertOriginX\", \"vert-origin-x\"],\n        [\"vertOriginY\", \"vert-origin-y\"],\n        [\"wordSpacing\", \"word-spacing\"],\n        [\"writingMode\", \"writing-mode\"],\n        [\"xmlnsXlink\", \"xmlns:xlink\"],\n        [\"xHeight\", \"x-height\"]\n      ]),\n      possibleStandardNames = {\n        accept: \"accept\",\n        acceptcharset: \"acceptCharset\",\n        \"accept-charset\": \"acceptCharset\",\n        accesskey: \"accessKey\",\n        action: \"action\",\n        allowfullscreen: \"allowFullScreen\",\n        alt: \"alt\",\n        as: \"as\",\n        async: \"async\",\n        autocapitalize: \"autoCapitalize\",\n        autocomplete: \"autoComplete\",\n        autocorrect: \"autoCorrect\",\n        autofocus: \"autoFocus\",\n        autoplay: \"autoPlay\",\n        autosave: \"autoSave\",\n        capture: \"capture\",\n        cellpadding: \"cellPadding\",\n        cellspacing: \"cellSpacing\",\n        challenge: \"challenge\",\n        charset: \"charSet\",\n        checked: \"checked\",\n        children: \"children\",\n        cite: \"cite\",\n        class: \"className\",\n        classid: \"classID\",\n        classname: \"className\",\n        cols: \"cols\",\n        colspan: \"colSpan\",\n        content: \"content\",\n        contenteditable: \"contentEditable\",\n        contextmenu: \"contextMenu\",\n        controls: \"controls\",\n        controlslist: \"controlsList\",\n        coords: \"coords\",\n        crossorigin: \"crossOrigin\",\n        dangerouslysetinnerhtml: \"dangerouslySetInnerHTML\",\n        data: \"data\",\n        datetime: \"dateTime\",\n        default: \"default\",\n        defaultchecked: \"defaultChecked\",\n        defaultvalue: \"defaultValue\",\n        defer: \"defer\",\n        dir: \"dir\",\n        disabled: \"disabled\",\n        disablepictureinpicture: \"disablePictureInPicture\",\n        disableremoteplayback: \"disableRemotePlayback\",\n        download: \"download\",\n        draggable: \"draggable\",\n        enctype: \"encType\",\n        enterkeyhint: \"enterKeyHint\",\n        fetchpriority: \"fetchPriority\",\n        for: \"htmlFor\",\n        form: \"form\",\n        formmethod: \"formMethod\",\n        formaction: \"formAction\",\n        formenctype: \"formEncType\",\n        formnovalidate: \"formNoValidate\",\n        formtarget: \"formTarget\",\n        frameborder: \"frameBorder\",\n        headers: \"headers\",\n        height: \"height\",\n        hidden: \"hidden\",\n        high: \"high\",\n        href: \"href\",\n        hreflang: \"hrefLang\",\n        htmlfor: \"htmlFor\",\n        httpequiv: \"httpEquiv\",\n        \"http-equiv\": \"httpEquiv\",\n        icon: \"icon\",\n        id: \"id\",\n        imagesizes: \"imageSizes\",\n        imagesrcset: \"imageSrcSet\",\n        inert: \"inert\",\n        innerhtml: \"innerHTML\",\n        inputmode: \"inputMode\",\n        integrity: \"integrity\",\n        is: \"is\",\n        itemid: \"itemID\",\n        itemprop: \"itemProp\",\n        itemref: \"itemRef\",\n        itemscope: \"itemScope\",\n        itemtype: \"itemType\",\n        keyparams: \"keyParams\",\n        keytype: \"keyType\",\n        kind: \"kind\",\n        label: \"label\",\n        lang: \"lang\",\n        list: \"list\",\n        loop: \"loop\",\n        low: \"low\",\n        manifest: \"manifest\",\n        marginwidth: \"marginWidth\",\n        marginheight: \"marginHeight\",\n        max: \"max\",\n        maxlength: \"maxLength\",\n        media: \"media\",\n        mediagroup: \"mediaGroup\",\n        method: \"method\",\n        min: \"min\",\n        minlength: \"minLength\",\n        multiple: \"multiple\",\n        muted: \"muted\",\n        name: \"name\",\n        nomodule: \"noModule\",\n        nonce: \"nonce\",\n        novalidate: \"noValidate\",\n        open: \"open\",\n        optimum: \"optimum\",\n        pattern: \"pattern\",\n        placeholder: \"placeholder\",\n        playsinline: \"playsInline\",\n        poster: \"poster\",\n        preload: \"preload\",\n        profile: \"profile\",\n        radiogroup: \"radioGroup\",\n        readonly: \"readOnly\",\n        referrerpolicy: \"referrerPolicy\",\n        rel: \"rel\",\n        required: \"required\",\n        reversed: \"reversed\",\n        role: \"role\",\n        rows: \"rows\",\n        rowspan: \"rowSpan\",\n        sandbox: \"sandbox\",\n        scope: \"scope\",\n        scoped: \"scoped\",\n        scrolling: \"scrolling\",\n        seamless: \"seamless\",\n        selected: \"selected\",\n        shape: \"shape\",\n        size: \"size\",\n        sizes: \"sizes\",\n        span: \"span\",\n        spellcheck: \"spellCheck\",\n        src: \"src\",\n        srcdoc: \"srcDoc\",\n        srclang: \"srcLang\",\n        srcset: \"srcSet\",\n        start: \"start\",\n        step: \"step\",\n        style: \"style\",\n        summary: \"summary\",\n        tabindex: \"tabIndex\",\n        target: \"target\",\n        title: \"title\",\n        type: \"type\",\n        usemap: \"useMap\",\n        value: \"value\",\n        width: \"width\",\n        wmode: \"wmode\",\n        wrap: \"wrap\",\n        about: \"about\",\n        accentheight: \"accentHeight\",\n        \"accent-height\": \"accentHeight\",\n        accumulate: \"accumulate\",\n        additive: \"additive\",\n        alignmentbaseline: \"alignmentBaseline\",\n        \"alignment-baseline\": \"alignmentBaseline\",\n        allowreorder: \"allowReorder\",\n        alphabetic: \"alphabetic\",\n        amplitude: \"amplitude\",\n        arabicform: \"arabicForm\",\n        \"arabic-form\": \"arabicForm\",\n        ascent: \"ascent\",\n        attributename: \"attributeName\",\n        attributetype: \"attributeType\",\n        autoreverse: \"autoReverse\",\n        azimuth: \"azimuth\",\n        basefrequency: \"baseFrequency\",\n        baselineshift: \"baselineShift\",\n        \"baseline-shift\": \"baselineShift\",\n        baseprofile: \"baseProfile\",\n        bbox: \"bbox\",\n        begin: \"begin\",\n        bias: \"bias\",\n        by: \"by\",\n        calcmode: \"calcMode\",\n        capheight: \"capHeight\",\n        \"cap-height\": \"capHeight\",\n        clip: \"clip\",\n        clippath: \"clipPath\",\n        \"clip-path\": \"clipPath\",\n        clippathunits: \"clipPathUnits\",\n        cliprule: \"clipRule\",\n        \"clip-rule\": \"clipRule\",\n        color: \"color\",\n        colorinterpolation: \"colorInterpolation\",\n        \"color-interpolation\": \"colorInterpolation\",\n        colorinterpolationfilters: \"colorInterpolationFilters\",\n        \"color-interpolation-filters\": \"colorInterpolationFilters\",\n        colorprofile: \"colorProfile\",\n        \"color-profile\": \"colorProfile\",\n        colorrendering: \"colorRendering\",\n        \"color-rendering\": \"colorRendering\",\n        contentscripttype: \"contentScriptType\",\n        contentstyletype: \"contentStyleType\",\n        cursor: \"cursor\",\n        cx: \"cx\",\n        cy: \"cy\",\n        d: \"d\",\n        datatype: \"datatype\",\n        decelerate: \"decelerate\",\n        descent: \"descent\",\n        diffuseconstant: \"diffuseConstant\",\n        direction: \"direction\",\n        display: \"display\",\n        divisor: \"divisor\",\n        dominantbaseline: \"dominantBaseline\",\n        \"dominant-baseline\": \"dominantBaseline\",\n        dur: \"dur\",\n        dx: \"dx\",\n        dy: \"dy\",\n        edgemode: \"edgeMode\",\n        elevation: \"elevation\",\n        enablebackground: \"enableBackground\",\n        \"enable-background\": \"enableBackground\",\n        end: \"end\",\n        exponent: \"exponent\",\n        externalresourcesrequired: \"externalResourcesRequired\",\n        fill: \"fill\",\n        fillopacity: \"fillOpacity\",\n        \"fill-opacity\": \"fillOpacity\",\n        fillrule: \"fillRule\",\n        \"fill-rule\": \"fillRule\",\n        filter: \"filter\",\n        filterres: \"filterRes\",\n        filterunits: \"filterUnits\",\n        floodopacity: \"floodOpacity\",\n        \"flood-opacity\": \"floodOpacity\",\n        floodcolor: \"floodColor\",\n        \"flood-color\": \"floodColor\",\n        focusable: \"focusable\",\n        fontfamily: \"fontFamily\",\n        \"font-family\": \"fontFamily\",\n        fontsize: \"fontSize\",\n        \"font-size\": \"fontSize\",\n        fontsizeadjust: \"fontSizeAdjust\",\n        \"font-size-adjust\": \"fontSizeAdjust\",\n        fontstretch: \"fontStretch\",\n        \"font-stretch\": \"fontStretch\",\n        fontstyle: \"fontStyle\",\n        \"font-style\": \"fontStyle\",\n        fontvariant: \"fontVariant\",\n        \"font-variant\": \"fontVariant\",\n        fontweight: \"fontWeight\",\n        \"font-weight\": \"fontWeight\",\n        format: \"format\",\n        from: \"from\",\n        fx: \"fx\",\n        fy: \"fy\",\n        g1: \"g1\",\n        g2: \"g2\",\n        glyphname: \"glyphName\",\n        \"glyph-name\": \"glyphName\",\n        glyphorientationhorizontal: \"glyphOrientationHorizontal\",\n        \"glyph-orientation-horizontal\": \"glyphOrientationHorizontal\",\n        glyphorientationvertical: \"glyphOrientationVertical\",\n        \"glyph-orientation-vertical\": \"glyphOrientationVertical\",\n        glyphref: \"glyphRef\",\n        gradienttransform: \"gradientTransform\",\n        gradientunits: \"gradientUnits\",\n        hanging: \"hanging\",\n        horizadvx: \"horizAdvX\",\n        \"horiz-adv-x\": \"horizAdvX\",\n        horizoriginx: \"horizOriginX\",\n        \"horiz-origin-x\": \"horizOriginX\",\n        ideographic: \"ideographic\",\n        imagerendering: \"imageRendering\",\n        \"image-rendering\": \"imageRendering\",\n        in2: \"in2\",\n        in: \"in\",\n        inlist: \"inlist\",\n        intercept: \"intercept\",\n        k1: \"k1\",\n        k2: \"k2\",\n        k3: \"k3\",\n        k4: \"k4\",\n        k: \"k\",\n        kernelmatrix: \"kernelMatrix\",\n        kernelunitlength: \"kernelUnitLength\",\n        kerning: \"kerning\",\n        keypoints: \"keyPoints\",\n        keysplines: \"keySplines\",\n        keytimes: \"keyTimes\",\n        lengthadjust: \"lengthAdjust\",\n        letterspacing: \"letterSpacing\",\n        \"letter-spacing\": \"letterSpacing\",\n        lightingcolor: \"lightingColor\",\n        \"lighting-color\": \"lightingColor\",\n        limitingconeangle: \"limitingConeAngle\",\n        local: \"local\",\n        markerend: \"markerEnd\",\n        \"marker-end\": \"markerEnd\",\n        markerheight: \"markerHeight\",\n        markermid: \"markerMid\",\n        \"marker-mid\": \"markerMid\",\n        markerstart: \"markerStart\",\n        \"marker-start\": \"markerStart\",\n        markerunits: \"markerUnits\",\n        markerwidth: \"markerWidth\",\n        mask: \"mask\",\n        maskcontentunits: \"maskContentUnits\",\n        maskunits: \"maskUnits\",\n        mathematical: \"mathematical\",\n        mode: \"mode\",\n        numoctaves: \"numOctaves\",\n        offset: \"offset\",\n        opacity: \"opacity\",\n        operator: \"operator\",\n        order: \"order\",\n        orient: \"orient\",\n        orientation: \"orientation\",\n        origin: \"origin\",\n        overflow: \"overflow\",\n        overlineposition: \"overlinePosition\",\n        \"overline-position\": \"overlinePosition\",\n        overlinethickness: \"overlineThickness\",\n        \"overline-thickness\": \"overlineThickness\",\n        paintorder: \"paintOrder\",\n        \"paint-order\": \"paintOrder\",\n        panose1: \"panose1\",\n        \"panose-1\": \"panose1\",\n        pathlength: \"pathLength\",\n        patterncontentunits: \"patternContentUnits\",\n        patterntransform: \"patternTransform\",\n        patternunits: \"patternUnits\",\n        pointerevents: \"pointerEvents\",\n        \"pointer-events\": \"pointerEvents\",\n        points: \"points\",\n        pointsatx: \"pointsAtX\",\n        pointsaty: \"pointsAtY\",\n        pointsatz: \"pointsAtZ\",\n        popover: \"popover\",\n        popovertarget: \"popoverTarget\",\n        popovertargetaction: \"popoverTargetAction\",\n        prefix: \"prefix\",\n        preservealpha: \"preserveAlpha\",\n        preserveaspectratio: \"preserveAspectRatio\",\n        primitiveunits: \"primitiveUnits\",\n        property: \"property\",\n        r: \"r\",\n        radius: \"radius\",\n        refx: \"refX\",\n        refy: \"refY\",\n        renderingintent: \"renderingIntent\",\n        \"rendering-intent\": \"renderingIntent\",\n        repeatcount: \"repeatCount\",\n        repeatdur: \"repeatDur\",\n        requiredextensions: \"requiredExtensions\",\n        requiredfeatures: \"requiredFeatures\",\n        resource: \"resource\",\n        restart: \"restart\",\n        result: \"result\",\n        results: \"results\",\n        rotate: \"rotate\",\n        rx: \"rx\",\n        ry: \"ry\",\n        scale: \"scale\",\n        security: \"security\",\n        seed: \"seed\",\n        shaperendering: \"shapeRendering\",\n        \"shape-rendering\": \"shapeRendering\",\n        slope: \"slope\",\n        spacing: \"spacing\",\n        specularconstant: \"specularConstant\",\n        specularexponent: \"specularExponent\",\n        speed: \"speed\",\n        spreadmethod: \"spreadMethod\",\n        startoffset: \"startOffset\",\n        stddeviation: \"stdDeviation\",\n        stemh: \"stemh\",\n        stemv: \"stemv\",\n        stitchtiles: \"stitchTiles\",\n        stopcolor: \"stopColor\",\n        \"stop-color\": \"stopColor\",\n        stopopacity: \"stopOpacity\",\n        \"stop-opacity\": \"stopOpacity\",\n        strikethroughposition: \"strikethroughPosition\",\n        \"strikethrough-position\": \"strikethroughPosition\",\n        strikethroughthickness: \"strikethroughThickness\",\n        \"strikethrough-thickness\": \"strikethroughThickness\",\n        string: \"string\",\n        stroke: \"stroke\",\n        strokedasharray: \"strokeDasharray\",\n        \"stroke-dasharray\": \"strokeDasharray\",\n        strokedashoffset: \"strokeDashoffset\",\n        \"stroke-dashoffset\": \"strokeDashoffset\",\n        strokelinecap: \"strokeLinecap\",\n        \"stroke-linecap\": \"strokeLinecap\",\n        strokelinejoin: \"strokeLinejoin\",\n        \"stroke-linejoin\": \"strokeLinejoin\",\n        strokemiterlimit: \"strokeMiterlimit\",\n        \"stroke-miterlimit\": \"strokeMiterlimit\",\n        strokewidth: \"strokeWidth\",\n        \"stroke-width\": \"strokeWidth\",\n        strokeopacity: \"strokeOpacity\",\n        \"stroke-opacity\": \"strokeOpacity\",\n        suppresscontenteditablewarning: \"suppressContentEditableWarning\",\n        suppresshydrationwarning: \"suppressHydrationWarning\",\n        surfacescale: \"surfaceScale\",\n        systemlanguage: \"systemLanguage\",\n        tablevalues: \"tableValues\",\n        targetx: \"targetX\",\n        targety: \"targetY\",\n        textanchor: \"textAnchor\",\n        \"text-anchor\": \"textAnchor\",\n        textdecoration: \"textDecoration\",\n        \"text-decoration\": \"textDecoration\",\n        textlength: \"textLength\",\n        textrendering: \"textRendering\",\n        \"text-rendering\": \"textRendering\",\n        to: \"to\",\n        transform: \"transform\",\n        transformorigin: \"transformOrigin\",\n        \"transform-origin\": \"transformOrigin\",\n        typeof: \"typeof\",\n        u1: \"u1\",\n        u2: \"u2\",\n        underlineposition: \"underlinePosition\",\n        \"underline-position\": \"underlinePosition\",\n        underlinethickness: \"underlineThickness\",\n        \"underline-thickness\": \"underlineThickness\",\n        unicode: \"unicode\",\n        unicodebidi: \"unicodeBidi\",\n        \"unicode-bidi\": \"unicodeBidi\",\n        unicoderange: \"unicodeRange\",\n        \"unicode-range\": \"unicodeRange\",\n        unitsperem: \"unitsPerEm\",\n        \"units-per-em\": \"unitsPerEm\",\n        unselectable: \"unselectable\",\n        valphabetic: \"vAlphabetic\",\n        \"v-alphabetic\": \"vAlphabetic\",\n        values: \"values\",\n        vectoreffect: \"vectorEffect\",\n        \"vector-effect\": \"vectorEffect\",\n        version: \"version\",\n        vertadvy: \"vertAdvY\",\n        \"vert-adv-y\": \"vertAdvY\",\n        vertoriginx: \"vertOriginX\",\n        \"vert-origin-x\": \"vertOriginX\",\n        vertoriginy: \"vertOriginY\",\n        \"vert-origin-y\": \"vertOriginY\",\n        vhanging: \"vHanging\",\n        \"v-hanging\": \"vHanging\",\n        videographic: \"vIdeographic\",\n        \"v-ideographic\": \"vIdeographic\",\n        viewbox: \"viewBox\",\n        viewtarget: \"viewTarget\",\n        visibility: \"visibility\",\n        vmathematical: \"vMathematical\",\n        \"v-mathematical\": \"vMathematical\",\n        vocab: \"vocab\",\n        widths: \"widths\",\n        wordspacing: \"wordSpacing\",\n        \"word-spacing\": \"wordSpacing\",\n        writingmode: \"writingMode\",\n        \"writing-mode\": \"writingMode\",\n        x1: \"x1\",\n        x2: \"x2\",\n        x: \"x\",\n        xchannelselector: \"xChannelSelector\",\n        xheight: \"xHeight\",\n        \"x-height\": \"xHeight\",\n        xlinkactuate: \"xlinkActuate\",\n        \"xlink:actuate\": \"xlinkActuate\",\n        xlinkarcrole: \"xlinkArcrole\",\n        \"xlink:arcrole\": \"xlinkArcrole\",\n        xlinkhref: \"xlinkHref\",\n        \"xlink:href\": \"xlinkHref\",\n        xlinkrole: \"xlinkRole\",\n        \"xlink:role\": \"xlinkRole\",\n        xlinkshow: \"xlinkShow\",\n        \"xlink:show\": \"xlinkShow\",\n        xlinktitle: \"xlinkTitle\",\n        \"xlink:title\": \"xlinkTitle\",\n        xlinktype: \"xlinkType\",\n        \"xlink:type\": \"xlinkType\",\n        xmlbase: \"xmlBase\",\n        \"xml:base\": \"xmlBase\",\n        xmllang: \"xmlLang\",\n        \"xml:lang\": \"xmlLang\",\n        xmlns: \"xmlns\",\n        \"xml:space\": \"xmlSpace\",\n        xmlnsxlink: \"xmlnsXlink\",\n        \"xmlns:xlink\": \"xmlnsXlink\",\n        xmlspace: \"xmlSpace\",\n        y1: \"y1\",\n        y2: \"y2\",\n        y: \"y\",\n        ychannelselector: \"yChannelSelector\",\n        z: \"z\",\n        zoomandpan: \"zoomAndPan\"\n      },\n      ariaProperties = {\n        \"aria-current\": 0,\n        \"aria-description\": 0,\n        \"aria-details\": 0,\n        \"aria-disabled\": 0,\n        \"aria-hidden\": 0,\n        \"aria-invalid\": 0,\n        \"aria-keyshortcuts\": 0,\n        \"aria-label\": 0,\n        \"aria-roledescription\": 0,\n        \"aria-autocomplete\": 0,\n        \"aria-checked\": 0,\n        \"aria-expanded\": 0,\n        \"aria-haspopup\": 0,\n        \"aria-level\": 0,\n        \"aria-modal\": 0,\n        \"aria-multiline\": 0,\n        \"aria-multiselectable\": 0,\n        \"aria-orientation\": 0,\n        \"aria-placeholder\": 0,\n        \"aria-pressed\": 0,\n        \"aria-readonly\": 0,\n        \"aria-required\": 0,\n        \"aria-selected\": 0,\n        \"aria-sort\": 0,\n        \"aria-valuemax\": 0,\n        \"aria-valuemin\": 0,\n        \"aria-valuenow\": 0,\n        \"aria-valuetext\": 0,\n        \"aria-atomic\": 0,\n        \"aria-busy\": 0,\n        \"aria-live\": 0,\n        \"aria-relevant\": 0,\n        \"aria-dropeffect\": 0,\n        \"aria-grabbed\": 0,\n        \"aria-activedescendant\": 0,\n        \"aria-colcount\": 0,\n        \"aria-colindex\": 0,\n        \"aria-colspan\": 0,\n        \"aria-controls\": 0,\n        \"aria-describedby\": 0,\n        \"aria-errormessage\": 0,\n        \"aria-flowto\": 0,\n        \"aria-labelledby\": 0,\n        \"aria-owns\": 0,\n        \"aria-posinset\": 0,\n        \"aria-rowcount\": 0,\n        \"aria-rowindex\": 0,\n        \"aria-rowspan\": 0,\n        \"aria-setsize\": 0\n      },\n      warnedProperties$1 = {},\n      rARIA$1 = RegExp(\n        \"^(aria)-[:A-Z_a-z\\\\u00C0-\\\\u00D6\\\\u00D8-\\\\u00F6\\\\u00F8-\\\\u02FF\\\\u0370-\\\\u037D\\\\u037F-\\\\u1FFF\\\\u200C-\\\\u200D\\\\u2070-\\\\u218F\\\\u2C00-\\\\u2FEF\\\\u3001-\\\\uD7FF\\\\uF900-\\\\uFDCF\\\\uFDF0-\\\\uFFFD\\\\-.0-9\\\\u00B7\\\\u0300-\\\\u036F\\\\u203F-\\\\u2040]*$\"\n      ),\n      rARIACamel$1 = RegExp(\n        \"^(aria)[A-Z][:A-Z_a-z\\\\u00C0-\\\\u00D6\\\\u00D8-\\\\u00F6\\\\u00F8-\\\\u02FF\\\\u0370-\\\\u037D\\\\u037F-\\\\u1FFF\\\\u200C-\\\\u200D\\\\u2070-\\\\u218F\\\\u2C00-\\\\u2FEF\\\\u3001-\\\\uD7FF\\\\uF900-\\\\uFDCF\\\\uFDF0-\\\\uFFFD\\\\-.0-9\\\\u00B7\\\\u0300-\\\\u036F\\\\u203F-\\\\u2040]*$\"\n      ),\n      didWarnValueNull = !1,\n      warnedProperties = {},\n      EVENT_NAME_REGEX = /^on./,\n      INVALID_EVENT_NAME_REGEX = /^on[^A-Z]/,\n      rARIA = RegExp(\n        \"^(aria)-[:A-Z_a-z\\\\u00C0-\\\\u00D6\\\\u00D8-\\\\u00F6\\\\u00F8-\\\\u02FF\\\\u0370-\\\\u037D\\\\u037F-\\\\u1FFF\\\\u200C-\\\\u200D\\\\u2070-\\\\u218F\\\\u2C00-\\\\u2FEF\\\\u3001-\\\\uD7FF\\\\uF900-\\\\uFDCF\\\\uFDF0-\\\\uFFFD\\\\-.0-9\\\\u00B7\\\\u0300-\\\\u036F\\\\u203F-\\\\u2040]*$\"\n      ),\n      rARIACamel = RegExp(\n        \"^(aria)[A-Z][:A-Z_a-z\\\\u00C0-\\\\u00D6\\\\u00D8-\\\\u00F6\\\\u00F8-\\\\u02FF\\\\u0370-\\\\u037D\\\\u037F-\\\\u1FFF\\\\u200C-\\\\u200D\\\\u2070-\\\\u218F\\\\u2C00-\\\\u2FEF\\\\u3001-\\\\uD7FF\\\\uF900-\\\\uFDCF\\\\uFDF0-\\\\uFFFD\\\\-.0-9\\\\u00B7\\\\u0300-\\\\u036F\\\\u203F-\\\\u2040]*$\"\n      ),\n      isJavaScriptProtocol =\n        /^[\\u0000-\\u001F ]*j[\\r\\n\\t]*a[\\r\\n\\t]*v[\\r\\n\\t]*a[\\r\\n\\t]*s[\\r\\n\\t]*c[\\r\\n\\t]*r[\\r\\n\\t]*i[\\r\\n\\t]*p[\\r\\n\\t]*t[\\r\\n\\t]*:/i,\n      currentReplayingEvent = null,\n      restoreTarget = null,\n      restoreQueue = null,\n      isInsideEventHandler = !1,\n      canUseDOM = !(\n        \"undefined\" === typeof window ||\n        \"undefined\" === typeof window.document ||\n        \"undefined\" === typeof window.document.createElement\n      ),\n      passiveBrowserEventsSupported = !1;\n    if (canUseDOM)\n      try {\n        var options$jscomp$0 = {};\n        Object.defineProperty(options$jscomp$0, \"passive\", {\n          get: function () {\n            passiveBrowserEventsSupported = !0;\n          }\n        });\n        window.addEventListener(\"test\", options$jscomp$0, options$jscomp$0);\n        window.removeEventListener(\"test\", options$jscomp$0, options$jscomp$0);\n      } catch (e) {\n        passiveBrowserEventsSupported = !1;\n      }\n    var root = null,\n      startText = null,\n      fallbackText = null,\n      EventInterface = {\n        eventPhase: 0,\n        bubbles: 0,\n        cancelable: 0,\n        timeStamp: function (event) {\n          return event.timeStamp || Date.now();\n        },\n        defaultPrevented: 0,\n        isTrusted: 0\n      },\n      SyntheticEvent = createSyntheticEvent(EventInterface),\n      UIEventInterface = assign({}, EventInterface, { view: 0, detail: 0 }),\n      SyntheticUIEvent = createSyntheticEvent(UIEventInterface),\n      lastMovementX,\n      lastMovementY,\n      lastMouseEvent,\n      MouseEventInterface = assign({}, UIEventInterface, {\n        screenX: 0,\n        screenY: 0,\n        clientX: 0,\n        clientY: 0,\n        pageX: 0,\n        pageY: 0,\n        ctrlKey: 0,\n        shiftKey: 0,\n        altKey: 0,\n        metaKey: 0,\n        getModifierState: getEventModifierState,\n        button: 0,\n        buttons: 0,\n        relatedTarget: function (event) {\n          return void 0 === event.relatedTarget\n            ? event.fromElement === event.srcElement\n              ? event.toElement\n              : event.fromElement\n            : event.relatedTarget;\n        },\n        movementX: function (event) {\n          if (\"movementX\" in event) return event.movementX;\n          event !== lastMouseEvent &&\n            (lastMouseEvent && \"mousemove\" === event.type\n              ? ((lastMovementX = event.screenX - lastMouseEvent.screenX),\n                (lastMovementY = event.screenY - lastMouseEvent.screenY))\n              : (lastMovementY = lastMovementX = 0),\n            (lastMouseEvent = event));\n          return lastMovementX;\n        },\n        movementY: function (event) {\n          return \"movementY\" in event ? event.movementY : lastMovementY;\n        }\n      }),\n      SyntheticMouseEvent = createSyntheticEvent(MouseEventInterface),\n      DragEventInterface = assign({}, MouseEventInterface, { dataTransfer: 0 }),\n      SyntheticDragEvent = createSyntheticEvent(DragEventInterface),\n      FocusEventInterface = assign({}, UIEventInterface, { relatedTarget: 0 }),\n      SyntheticFocusEvent = createSyntheticEvent(FocusEventInterface),\n      AnimationEventInterface = assign({}, EventInterface, {\n        animationName: 0,\n        elapsedTime: 0,\n        pseudoElement: 0\n      }),\n      SyntheticAnimationEvent = createSyntheticEvent(AnimationEventInterface),\n      ClipboardEventInterface = assign({}, EventInterface, {\n        clipboardData: function (event) {\n          return \"clipboardData\" in event\n            ? event.clipboardData\n            : window.clipboardData;\n        }\n      }),\n      SyntheticClipboardEvent = createSyntheticEvent(ClipboardEventInterface),\n      CompositionEventInterface = assign({}, EventInterface, { data: 0 }),\n      SyntheticCompositionEvent = createSyntheticEvent(\n        CompositionEventInterface\n      ),\n      SyntheticInputEvent = SyntheticCompositionEvent,\n      normalizeKey = {\n        Esc: \"Escape\",\n        Spacebar: \" \",\n        Left: \"ArrowLeft\",\n        Up: \"ArrowUp\",\n        Right: \"ArrowRight\",\n        Down: \"ArrowDown\",\n        Del: \"Delete\",\n        Win: \"OS\",\n        Menu: \"ContextMenu\",\n        Apps: \"ContextMenu\",\n        Scroll: \"ScrollLock\",\n        MozPrintableKey: \"Unidentified\"\n      },\n      translateToKey = {\n        8: \"Backspace\",\n        9: \"Tab\",\n        12: \"Clear\",\n        13: \"Enter\",\n        16: \"Shift\",\n        17: \"Control\",\n        18: \"Alt\",\n        19: \"Pause\",\n        20: \"CapsLock\",\n        27: \"Escape\",\n        32: \" \",\n        33: \"PageUp\",\n        34: \"PageDown\",\n        35: \"End\",\n        36: \"Home\",\n        37: \"ArrowLeft\",\n        38: \"ArrowUp\",\n        39: \"ArrowRight\",\n        40: \"ArrowDown\",\n        45: \"Insert\",\n        46: \"Delete\",\n        112: \"F1\",\n        113: \"F2\",\n        114: \"F3\",\n        115: \"F4\",\n        116: \"F5\",\n        117: \"F6\",\n        118: \"F7\",\n        119: \"F8\",\n        120: \"F9\",\n        121: \"F10\",\n        122: \"F11\",\n        123: \"F12\",\n        144: \"NumLock\",\n        145: \"ScrollLock\",\n        224: \"Meta\"\n      },\n      modifierKeyToProp = {\n        Alt: \"altKey\",\n        Control: \"ctrlKey\",\n        Meta: \"metaKey\",\n        Shift: \"shiftKey\"\n      },\n      KeyboardEventInterface = assign({}, UIEventInterface, {\n        key: function (nativeEvent) {\n          if (nativeEvent.key) {\n            var key = normalizeKey[nativeEvent.key] || nativeEvent.key;\n            if (\"Unidentified\" !== key) return key;\n          }\n          return \"keypress\" === nativeEvent.type\n            ? ((nativeEvent = getEventCharCode(nativeEvent)),\n              13 === nativeEvent ? \"Enter\" : String.fromCharCode(nativeEvent))\n            : \"keydown\" === nativeEvent.type || \"keyup\" === nativeEvent.type\n              ? translateToKey[nativeEvent.keyCode] || \"Unidentified\"\n              : \"\";\n        },\n        code: 0,\n        location: 0,\n        ctrlKey: 0,\n        shiftKey: 0,\n        altKey: 0,\n        metaKey: 0,\n        repeat: 0,\n        locale: 0,\n        getModifierState: getEventModifierState,\n        charCode: function (event) {\n          return \"keypress\" === event.type ? getEventCharCode(event) : 0;\n        },\n        keyCode: function (event) {\n          return \"keydown\" === event.type || \"keyup\" === event.type\n            ? event.keyCode\n            : 0;\n        },\n        which: function (event) {\n          return \"keypress\" === event.type\n            ? getEventCharCode(event)\n            : \"keydown\" === event.type || \"keyup\" === event.type\n              ? event.keyCode\n              : 0;\n        }\n      }),\n      SyntheticKeyboardEvent = createSyntheticEvent(KeyboardEventInterface),\n      PointerEventInterface = assign({}, MouseEventInterface, {\n        pointerId: 0,\n        width: 0,\n        height: 0,\n        pressure: 0,\n        tangentialPressure: 0,\n        tiltX: 0,\n        tiltY: 0,\n        twist: 0,\n        pointerType: 0,\n        isPrimary: 0\n      }),\n      SyntheticPointerEvent = createSyntheticEvent(PointerEventInterface),\n      TouchEventInterface = assign({}, UIEventInterface, {\n        touches: 0,\n        targetTouches: 0,\n        changedTouches: 0,\n        altKey: 0,\n        metaKey: 0,\n        ctrlKey: 0,\n        shiftKey: 0,\n        getModifierState: getEventModifierState\n      }),\n      SyntheticTouchEvent = createSyntheticEvent(TouchEventInterface),\n      TransitionEventInterface = assign({}, EventInterface, {\n        propertyName: 0,\n        elapsedTime: 0,\n        pseudoElement: 0\n      }),\n      SyntheticTransitionEvent = createSyntheticEvent(TransitionEventInterface),\n      WheelEventInterface = assign({}, MouseEventInterface, {\n        deltaX: function (event) {\n          return \"deltaX\" in event\n            ? event.deltaX\n            : \"wheelDeltaX\" in event\n              ? -event.wheelDeltaX\n              : 0;\n        },\n        deltaY: function (event) {\n          return \"deltaY\" in event\n            ? event.deltaY\n            : \"wheelDeltaY\" in event\n              ? -event.wheelDeltaY\n              : \"wheelDelta\" in event\n                ? -event.wheelDelta\n                : 0;\n        },\n        deltaZ: 0,\n        deltaMode: 0\n      }),\n      SyntheticWheelEvent = createSyntheticEvent(WheelEventInterface),\n      ToggleEventInterface = assign({}, EventInterface, {\n        newState: 0,\n        oldState: 0\n      }),\n      SyntheticToggleEvent = createSyntheticEvent(ToggleEventInterface),\n      END_KEYCODES = [9, 13, 27, 32],\n      START_KEYCODE = 229,\n      canUseCompositionEvent = canUseDOM && \"CompositionEvent\" in window,\n      documentMode = null;\n    canUseDOM &&\n      \"documentMode\" in document &&\n      (documentMode = document.documentMode);\n    var canUseTextInputEvent =\n        canUseDOM && \"TextEvent\" in window && !documentMode,\n      useFallbackCompositionData =\n        canUseDOM &&\n        (!canUseCompositionEvent ||\n          (documentMode && 8 < documentMode && 11 >= documentMode)),\n      SPACEBAR_CODE = 32,\n      SPACEBAR_CHAR = String.fromCharCode(SPACEBAR_CODE),\n      hasSpaceKeypress = !1,\n      isComposing = !1,\n      supportedInputTypes = {\n        color: !0,\n        date: !0,\n        datetime: !0,\n        \"datetime-local\": !0,\n        email: !0,\n        month: !0,\n        number: !0,\n        password: !0,\n        range: !0,\n        search: !0,\n        tel: !0,\n        text: !0,\n        time: !0,\n        url: !0,\n        week: !0\n      },\n      activeElement$1 = null,\n      activeElementInst$1 = null,\n      isInputEventSupported = !1;\n    canUseDOM &&\n      (isInputEventSupported =\n        isEventSupported(\"input\") &&\n        (!document.documentMode || 9 < document.documentMode));\n    var objectIs = \"function\" === typeof Object.is ? Object.is : is,\n      skipSelectionChangeEvent =\n        canUseDOM && \"documentMode\" in document && 11 >= document.documentMode,\n      activeElement = null,\n      activeElementInst = null,\n      lastSelection = null,\n      mouseDown = !1,\n      vendorPrefixes = {\n        animationend: makePrefixMap(\"Animation\", \"AnimationEnd\"),\n        animationiteration: makePrefixMap(\"Animation\", \"AnimationIteration\"),\n        animationstart: makePrefixMap(\"Animation\", \"AnimationStart\"),\n        transitionrun: makePrefixMap(\"Transition\", \"TransitionRun\"),\n        transitionstart: makePrefixMap(\"Transition\", \"TransitionStart\"),\n        transitioncancel: makePrefixMap(\"Transition\", \"TransitionCancel\"),\n        transitionend: makePrefixMap(\"Transition\", \"TransitionEnd\")\n      },\n      prefixedEventNames = {},\n      style = {};\n    canUseDOM &&\n      ((style = document.createElement(\"div\").style),\n      \"AnimationEvent\" in window ||\n        (delete vendorPrefixes.animationend.animation,\n        delete vendorPrefixes.animationiteration.animation,\n        delete vendorPrefixes.animationstart.animation),\n      \"TransitionEvent\" in window ||\n        delete vendorPrefixes.transitionend.transition);\n    var ANIMATION_END = getVendorPrefixedEventName(\"animationend\"),\n      ANIMATION_ITERATION = getVendorPrefixedEventName(\"animationiteration\"),\n      ANIMATION_START = getVendorPrefixedEventName(\"animationstart\"),\n      TRANSITION_RUN = getVendorPrefixedEventName(\"transitionrun\"),\n      TRANSITION_START = getVendorPrefixedEventName(\"transitionstart\"),\n      TRANSITION_CANCEL = getVendorPrefixedEventName(\"transitioncancel\"),\n      TRANSITION_END = getVendorPrefixedEventName(\"transitionend\"),\n      topLevelEventsToReactNames = new Map(),\n      simpleEventPluginEvents =\n        \"abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel\".split(\n          \" \"\n        );\n    simpleEventPluginEvents.push(\"scrollEnd\");\n    var CapturedStacks = new WeakMap(),\n      OffscreenVisible = 1,\n      OffscreenPassiveEffectsConnected = 2,\n      concurrentQueues = [],\n      concurrentQueuesIndex = 0,\n      concurrentlyUpdatedLanes = 0,\n      emptyContextObject = {};\n    Object.freeze(emptyContextObject);\n    var resolveFamily = null,\n      failedBoundaries = null,\n      NoMode = 0,\n      ConcurrentMode = 1,\n      ProfileMode = 2,\n      StrictLegacyMode = 8,\n      StrictEffectsMode = 16,\n      NoStrictPassiveEffectsMode = 64;\n    var hasBadMapPolyfill = !1;\n    try {\n      var nonExtensibleObject = Object.preventExtensions({});\n      new Map([[nonExtensibleObject, null]]);\n      new Set([nonExtensibleObject]);\n    } catch (e$3) {\n      hasBadMapPolyfill = !0;\n    }\n    var forkStack = [],\n      forkStackIndex = 0,\n      treeForkProvider = null,\n      treeForkCount = 0,\n      idStack = [],\n      idStackIndex = 0,\n      treeContextProvider = null,\n      treeContextId = 1,\n      treeContextOverflow = \"\",\n      hydrationParentFiber = null,\n      nextHydratableInstance = null,\n      isHydrating = !1,\n      didSuspendOrErrorDEV = !1,\n      hydrationDiffRootDEV = null,\n      hydrationErrors = null,\n      rootOrSingletonContext = !1,\n      HydrationMismatchException = Error(\n        \"Hydration Mismatch Exception: This is not a real error, and should not leak into userspace. If you're seeing this, it's likely a bug in React.\"\n      ),\n      lastResetTime = 0;\n    if (\n      \"object\" === typeof performance &&\n      \"function\" === typeof performance.now\n    ) {\n      var localPerformance = performance;\n      var getCurrentTime = function () {\n        return localPerformance.now();\n      };\n    } else {\n      var localDate = Date;\n      getCurrentTime = function () {\n        return localDate.now();\n      };\n    }\n    var valueCursor = createCursor(null);\n    var rendererCursorDEV = createCursor(null);\n    var rendererSigil = {};\n    var currentlyRenderingFiber$1 = null,\n      lastContextDependency = null,\n      isDisallowedContextReadInDEV = !1,\n      AbortControllerLocal =\n        \"undefined\" !== typeof AbortController\n          ? AbortController\n          : function () {\n              var listeners = [],\n                signal = (this.signal = {\n                  aborted: !1,\n                  addEventListener: function (type, listener) {\n                    listeners.push(listener);\n                  }\n                });\n              this.abort = function () {\n                signal.aborted = !0;\n                listeners.forEach(function (listener) {\n                  return listener();\n                });\n              };\n            },\n      scheduleCallback$2 = Scheduler.unstable_scheduleCallback,\n      NormalPriority = Scheduler.unstable_NormalPriority,\n      CacheContext = {\n        $$typeof: REACT_CONTEXT_TYPE,\n        Consumer: null,\n        Provider: null,\n        _currentValue: null,\n        _currentValue2: null,\n        _threadCount: 0,\n        _currentRenderer: null,\n        _currentRenderer2: null\n      },\n      now = Scheduler.unstable_now,\n      renderStartTime = -0,\n      commitStartTime = -0,\n      profilerStartTime = -1.1,\n      profilerEffectDuration = -0,\n      currentUpdateIsNested = !1,\n      nestedUpdateScheduled = !1,\n      currentEntangledListeners = null,\n      currentEntangledPendingCount = 0,\n      currentEntangledLane = 0,\n      currentEntangledActionThenable = null,\n      prevOnStartTransitionFinish = ReactSharedInternals.S;\n    ReactSharedInternals.S = function (transition, returnValue) {\n      \"object\" === typeof returnValue &&\n        null !== returnValue &&\n        \"function\" === typeof returnValue.then &&\n        entangleAsyncAction(transition, returnValue);\n      null !== prevOnStartTransitionFinish &&\n        prevOnStartTransitionFinish(transition, returnValue);\n    };\n    var resumedCache = createCursor(null),\n      ReactStrictModeWarnings = {\n        recordUnsafeLifecycleWarnings: function () {},\n        flushPendingUnsafeLifecycleWarnings: function () {},\n        recordLegacyContextWarning: function () {},\n        flushLegacyContextWarning: function () {},\n        discardPendingWarnings: function () {}\n      },\n      pendingComponentWillMountWarnings = [],\n      pendingUNSAFE_ComponentWillMountWarnings = [],\n      pendingComponentWillReceivePropsWarnings = [],\n      pendingUNSAFE_ComponentWillReceivePropsWarnings = [],\n      pendingComponentWillUpdateWarnings = [],\n      pendingUNSAFE_ComponentWillUpdateWarnings = [],\n      didWarnAboutUnsafeLifecycles = new Set();\n    ReactStrictModeWarnings.recordUnsafeLifecycleWarnings = function (\n      fiber,\n      instance\n    ) {\n      didWarnAboutUnsafeLifecycles.has(fiber.type) ||\n        (\"function\" === typeof instance.componentWillMount &&\n          !0 !== instance.componentWillMount.__suppressDeprecationWarning &&\n          pendingComponentWillMountWarnings.push(fiber),\n        fiber.mode & StrictLegacyMode &&\n          \"function\" === typeof instance.UNSAFE_componentWillMount &&\n          pendingUNSAFE_ComponentWillMountWarnings.push(fiber),\n        \"function\" === typeof instance.componentWillReceiveProps &&\n          !0 !==\n            instance.componentWillReceiveProps.__suppressDeprecationWarning &&\n          pendingComponentWillReceivePropsWarnings.push(fiber),\n        fiber.mode & StrictLegacyMode &&\n          \"function\" === typeof instance.UNSAFE_componentWillReceiveProps &&\n          pendingUNSAFE_ComponentWillReceivePropsWarnings.push(fiber),\n        \"function\" === typeof instance.componentWillUpdate &&\n          !0 !== instance.componentWillUpdate.__suppressDeprecationWarning &&\n          pendingComponentWillUpdateWarnings.push(fiber),\n        fiber.mode & StrictLegacyMode &&\n          \"function\" === typeof instance.UNSAFE_componentWillUpdate &&\n          pendingUNSAFE_ComponentWillUpdateWarnings.push(fiber));\n    };\n    ReactStrictModeWarnings.flushPendingUnsafeLifecycleWarnings = function () {\n      var componentWillMountUniqueNames = new Set();\n      0 < pendingComponentWillMountWarnings.length &&\n        (pendingComponentWillMountWarnings.forEach(function (fiber) {\n          componentWillMountUniqueNames.add(\n            getComponentNameFromFiber(fiber) || \"Component\"\n          );\n          didWarnAboutUnsafeLifecycles.add(fiber.type);\n        }),\n        (pendingComponentWillMountWarnings = []));\n      var UNSAFE_componentWillMountUniqueNames = new Set();\n      0 < pendingUNSAFE_ComponentWillMountWarnings.length &&\n        (pendingUNSAFE_ComponentWillMountWarnings.forEach(function (fiber) {\n          UNSAFE_componentWillMountUniqueNames.add(\n            getComponentNameFromFiber(fiber) || \"Component\"\n          );\n          didWarnAboutUnsafeLifecycles.add(fiber.type);\n        }),\n        (pendingUNSAFE_ComponentWillMountWarnings = []));\n      var componentWillReceivePropsUniqueNames = new Set();\n      0 < pendingComponentWillReceivePropsWarnings.length &&\n        (pendingComponentWillReceivePropsWarnings.forEach(function (fiber) {\n          componentWillReceivePropsUniqueNames.add(\n            getComponentNameFromFiber(fiber) || \"Component\"\n          );\n          didWarnAboutUnsafeLifecycles.add(fiber.type);\n        }),\n        (pendingComponentWillReceivePropsWarnings = []));\n      var UNSAFE_componentWillReceivePropsUniqueNames = new Set();\n      0 < pendingUNSAFE_ComponentWillReceivePropsWarnings.length &&\n        (pendingUNSAFE_ComponentWillReceivePropsWarnings.forEach(\n          function (fiber) {\n            UNSAFE_componentWillReceivePropsUniqueNames.add(\n              getComponentNameFromFiber(fiber) || \"Component\"\n            );\n            didWarnAboutUnsafeLifecycles.add(fiber.type);\n          }\n        ),\n        (pendingUNSAFE_ComponentWillReceivePropsWarnings = []));\n      var componentWillUpdateUniqueNames = new Set();\n      0 < pendingComponentWillUpdateWarnings.length &&\n        (pendingComponentWillUpdateWarnings.forEach(function (fiber) {\n          componentWillUpdateUniqueNames.add(\n            getComponentNameFromFiber(fiber) || \"Component\"\n          );\n          didWarnAboutUnsafeLifecycles.add(fiber.type);\n        }),\n        (pendingComponentWillUpdateWarnings = []));\n      var UNSAFE_componentWillUpdateUniqueNames = new Set();\n      0 < pendingUNSAFE_ComponentWillUpdateWarnings.length &&\n        (pendingUNSAFE_ComponentWillUpdateWarnings.forEach(function (fiber) {\n          UNSAFE_componentWillUpdateUniqueNames.add(\n            getComponentNameFromFiber(fiber) || \"Component\"\n          );\n          didWarnAboutUnsafeLifecycles.add(fiber.type);\n        }),\n        (pendingUNSAFE_ComponentWillUpdateWarnings = []));\n      if (0 < UNSAFE_componentWillMountUniqueNames.size) {\n        var sortedNames = setToSortedString(\n          UNSAFE_componentWillMountUniqueNames\n        );\n        console.error(\n          \"Using UNSAFE_componentWillMount in strict mode is not recommended and may indicate bugs in your code. See https://react.dev/link/unsafe-component-lifecycles for details.\\n\\n* Move code with side effects to componentDidMount, and set initial state in the constructor.\\n\\nPlease update the following components: %s\",\n          sortedNames\n        );\n      }\n      0 < UNSAFE_componentWillReceivePropsUniqueNames.size &&\n        ((sortedNames = setToSortedString(\n          UNSAFE_componentWillReceivePropsUniqueNames\n        )),\n        console.error(\n          \"Using UNSAFE_componentWillReceiveProps in strict mode is not recommended and may indicate bugs in your code. See https://react.dev/link/unsafe-component-lifecycles for details.\\n\\n* Move data fetching code or side effects to componentDidUpdate.\\n* If you're updating state whenever props change, refactor your code to use memoization techniques or move it to static getDerivedStateFromProps. Learn more at: https://react.dev/link/derived-state\\n\\nPlease update the following components: %s\",\n          sortedNames\n        ));\n      0 < UNSAFE_componentWillUpdateUniqueNames.size &&\n        ((sortedNames = setToSortedString(\n          UNSAFE_componentWillUpdateUniqueNames\n        )),\n        console.error(\n          \"Using UNSAFE_componentWillUpdate in strict mode is not recommended and may indicate bugs in your code. See https://react.dev/link/unsafe-component-lifecycles for details.\\n\\n* Move data fetching code or side effects to componentDidUpdate.\\n\\nPlease update the following components: %s\",\n          sortedNames\n        ));\n      0 < componentWillMountUniqueNames.size &&\n        ((sortedNames = setToSortedString(componentWillMountUniqueNames)),\n        console.warn(\n          \"componentWillMount has been renamed, and is not recommended for use. See https://react.dev/link/unsafe-component-lifecycles for details.\\n\\n* Move code with side effects to componentDidMount, and set initial state in the constructor.\\n* Rename componentWillMount to UNSAFE_componentWillMount to suppress this warning in non-strict mode. In React 18.x, only the UNSAFE_ name will work. To rename all deprecated lifecycles to their new names, you can run `npx react-codemod rename-unsafe-lifecycles` in your project source folder.\\n\\nPlease update the following components: %s\",\n          sortedNames\n        ));\n      0 < componentWillReceivePropsUniqueNames.size &&\n        ((sortedNames = setToSortedString(\n          componentWillReceivePropsUniqueNames\n        )),\n        console.warn(\n          \"componentWillReceiveProps has been renamed, and is not recommended for use. See https://react.dev/link/unsafe-component-lifecycles for details.\\n\\n* Move data fetching code or side effects to componentDidUpdate.\\n* If you're updating state whenever props change, refactor your code to use memoization techniques or move it to static getDerivedStateFromProps. Learn more at: https://react.dev/link/derived-state\\n* Rename componentWillReceiveProps to UNSAFE_componentWillReceiveProps to suppress this warning in non-strict mode. In React 18.x, only the UNSAFE_ name will work. To rename all deprecated lifecycles to their new names, you can run `npx react-codemod rename-unsafe-lifecycles` in your project source folder.\\n\\nPlease update the following components: %s\",\n          sortedNames\n        ));\n      0 < componentWillUpdateUniqueNames.size &&\n        ((sortedNames = setToSortedString(componentWillUpdateUniqueNames)),\n        console.warn(\n          \"componentWillUpdate has been renamed, and is not recommended for use. See https://react.dev/link/unsafe-component-lifecycles for details.\\n\\n* Move data fetching code or side effects to componentDidUpdate.\\n* Rename componentWillUpdate to UNSAFE_componentWillUpdate to suppress this warning in non-strict mode. In React 18.x, only the UNSAFE_ name will work. To rename all deprecated lifecycles to their new names, you can run `npx react-codemod rename-unsafe-lifecycles` in your project source folder.\\n\\nPlease update the following components: %s\",\n          sortedNames\n        ));\n    };\n    var pendingLegacyContextWarning = new Map(),\n      didWarnAboutLegacyContext = new Set();\n    ReactStrictModeWarnings.recordLegacyContextWarning = function (\n      fiber,\n      instance\n    ) {\n      var strictRoot = null;\n      for (var node = fiber; null !== node; )\n        node.mode & StrictLegacyMode && (strictRoot = node),\n          (node = node.return);\n      null === strictRoot\n        ? console.error(\n            \"Expected to find a StrictMode component in a strict mode tree. This error is likely caused by a bug in React. Please file an issue.\"\n          )\n        : !didWarnAboutLegacyContext.has(fiber.type) &&\n          ((node = pendingLegacyContextWarning.get(strictRoot)),\n          null != fiber.type.contextTypes ||\n            null != fiber.type.childContextTypes ||\n            (null !== instance &&\n              \"function\" === typeof instance.getChildContext)) &&\n          (void 0 === node &&\n            ((node = []), pendingLegacyContextWarning.set(strictRoot, node)),\n          node.push(fiber));\n    };\n    ReactStrictModeWarnings.flushLegacyContextWarning = function () {\n      pendingLegacyContextWarning.forEach(function (fiberArray) {\n        if (0 !== fiberArray.length) {\n          var firstFiber = fiberArray[0],\n            uniqueNames = new Set();\n          fiberArray.forEach(function (fiber) {\n            uniqueNames.add(getComponentNameFromFiber(fiber) || \"Component\");\n            didWarnAboutLegacyContext.add(fiber.type);\n          });\n          var sortedNames = setToSortedString(uniqueNames);\n          runWithFiberInDEV(firstFiber, function () {\n            console.error(\n              \"Legacy context API has been detected within a strict-mode tree.\\n\\nThe old API will be supported in all 16.x releases, but applications using it should migrate to the new version.\\n\\nPlease update the following components: %s\\n\\nLearn more about this warning here: https://react.dev/link/legacy-context\",\n              sortedNames\n            );\n          });\n        }\n      });\n    };\n    ReactStrictModeWarnings.discardPendingWarnings = function () {\n      pendingComponentWillMountWarnings = [];\n      pendingUNSAFE_ComponentWillMountWarnings = [];\n      pendingComponentWillReceivePropsWarnings = [];\n      pendingUNSAFE_ComponentWillReceivePropsWarnings = [];\n      pendingComponentWillUpdateWarnings = [];\n      pendingUNSAFE_ComponentWillUpdateWarnings = [];\n      pendingLegacyContextWarning = new Map();\n    };\n    var SuspenseException = Error(\n        \"Suspense Exception: This is not a real error! It's an implementation detail of `use` to interrupt the current render. You must either rethrow it immediately, or move the `use` call outside of the `try/catch` block. Capturing without rethrowing will lead to unexpected behavior.\\n\\nTo handle async errors, wrap your component in an error boundary, or call the promise's `.catch` method and pass the result to `use`.\"\n      ),\n      SuspenseyCommitException = Error(\n        \"Suspense Exception: This is not a real error, and should not leak into userspace. If you're seeing this, it's likely a bug in React.\"\n      ),\n      SuspenseActionException = Error(\n        \"Suspense Exception: This is not a real error! It's an implementation detail of `useActionState` to interrupt the current render. You must either rethrow it immediately, or move the `useActionState` call outside of the `try/catch` block. Capturing without rethrowing will lead to unexpected behavior.\\n\\nTo handle async errors, wrap your component in an error boundary.\"\n      ),\n      noopSuspenseyCommitThenable = {\n        then: function () {\n          console.error(\n            'Internal React error: A listener was unexpectedly attached to a \"noop\" thenable. This is a bug in React. Please file an issue.'\n          );\n        }\n      },\n      suspendedThenable = null,\n      needsToResetSuspendedThenableDEV = !1,\n      NoFlags = 0,\n      HasEffect = 1,\n      Insertion = 2,\n      Layout = 4,\n      Passive = 8,\n      UpdateState = 0,\n      ReplaceState = 1,\n      ForceUpdate = 2,\n      CaptureUpdate = 3,\n      hasForceUpdate = !1;\n    var didWarnUpdateInsideUpdate = !1;\n    var currentlyProcessingQueue = null;\n    var didReadFromEntangledAsyncAction = !1,\n      currentTreeHiddenStackCursor = createCursor(null),\n      prevEntangledRenderLanesCursor = createCursor(0),\n      didWarnUncachedGetSnapshot;\n    var didWarnAboutMismatchedHooksForComponent = new Set();\n    var didWarnAboutUseWrappedInTryCatch = new Set();\n    var didWarnAboutAsyncClientComponent = new Set();\n    var didWarnAboutUseFormState = new Set();\n    var renderLanes = 0,\n      currentlyRenderingFiber = null,\n      currentHook = null,\n      workInProgressHook = null,\n      didScheduleRenderPhaseUpdate = !1,\n      didScheduleRenderPhaseUpdateDuringThisPass = !1,\n      shouldDoubleInvokeUserFnsInHooksDEV = !1,\n      localIdCounter = 0,\n      thenableIndexCounter$1 = 0,\n      thenableState$1 = null,\n      globalClientIdCounter = 0,\n      RE_RENDER_LIMIT = 25,\n      currentHookNameInDev = null,\n      hookTypesDev = null,\n      hookTypesUpdateIndexDev = -1,\n      ignorePreviousDependencies = !1,\n      ContextOnlyDispatcher = {\n        readContext: readContext,\n        use: use,\n        useCallback: throwInvalidHookError,\n        useContext: throwInvalidHookError,\n        useEffect: throwInvalidHookError,\n        useImperativeHandle: throwInvalidHookError,\n        useLayoutEffect: throwInvalidHookError,\n        useInsertionEffect: throwInvalidHookError,\n        useMemo: throwInvalidHookError,\n        useReducer: throwInvalidHookError,\n        useRef: throwInvalidHookError,\n        useState: throwInvalidHookError,\n        useDebugValue: throwInvalidHookError,\n        useDeferredValue: throwInvalidHookError,\n        useTransition: throwInvalidHookError,\n        useSyncExternalStore: throwInvalidHookError,\n        useId: throwInvalidHookError,\n        useHostTransitionStatus: throwInvalidHookError,\n        useFormState: throwInvalidHookError,\n        useActionState: throwInvalidHookError,\n        useOptimistic: throwInvalidHookError,\n        useMemoCache: throwInvalidHookError,\n        useCacheRefresh: throwInvalidHookError\n      },\n      HooksDispatcherOnMountInDEV = null,\n      HooksDispatcherOnMountWithHookTypesInDEV = null,\n      HooksDispatcherOnUpdateInDEV = null,\n      HooksDispatcherOnRerenderInDEV = null,\n      InvalidNestedHooksDispatcherOnMountInDEV = null,\n      InvalidNestedHooksDispatcherOnUpdateInDEV = null,\n      InvalidNestedHooksDispatcherOnRerenderInDEV = null;\n    HooksDispatcherOnMountInDEV = {\n      readContext: function (context) {\n        return readContext(context);\n      },\n      use: use,\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        mountHookTypesDev();\n        checkDepsAreArrayDev(deps);\n        return mountCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        mountHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        mountHookTypesDev();\n        checkDepsAreArrayDev(createDeps);\n        return mountEffect(create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        mountHookTypesDev();\n        checkDepsAreArrayDev(deps);\n        return mountImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        mountHookTypesDev();\n        checkDepsAreArrayDev(deps);\n        mountEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        mountHookTypesDev();\n        checkDepsAreArrayDev(deps);\n        return mountLayoutEffect(create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        mountHookTypesDev();\n        checkDepsAreArrayDev(deps);\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        mountHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function (initialValue) {\n        currentHookNameInDev = \"useRef\";\n        mountHookTypesDev();\n        return mountRef(initialValue);\n      },\n      useState: function (initialState) {\n        currentHookNameInDev = \"useState\";\n        mountHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountState(initialState);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        mountHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        mountHookTypesDev();\n        return mountDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        mountHookTypesDev();\n        return mountTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        mountHookTypesDev();\n        return mountSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        mountHookTypesDev();\n        return mountId();\n      },\n      useFormState: function (action, initialState) {\n        currentHookNameInDev = \"useFormState\";\n        mountHookTypesDev();\n        warnOnUseFormStateInDev();\n        return mountActionState(action, initialState);\n      },\n      useActionState: function (action, initialState) {\n        currentHookNameInDev = \"useActionState\";\n        mountHookTypesDev();\n        return mountActionState(action, initialState);\n      },\n      useOptimistic: function (passthrough) {\n        currentHookNameInDev = \"useOptimistic\";\n        mountHookTypesDev();\n        return mountOptimistic(passthrough);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useMemoCache: useMemoCache,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        mountHookTypesDev();\n        return mountRefresh();\n      }\n    };\n    HooksDispatcherOnMountWithHookTypesInDEV = {\n      readContext: function (context) {\n        return readContext(context);\n      },\n      use: use,\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        updateHookTypesDev();\n        return mountCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        updateHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        updateHookTypesDev();\n        return mountEffect(create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        updateHookTypesDev();\n        return mountImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        updateHookTypesDev();\n        mountEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        updateHookTypesDev();\n        return mountLayoutEffect(create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function (initialValue) {\n        currentHookNameInDev = \"useRef\";\n        updateHookTypesDev();\n        return mountRef(initialValue);\n      },\n      useState: function (initialState) {\n        currentHookNameInDev = \"useState\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountState(initialState);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        updateHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        updateHookTypesDev();\n        return mountDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        updateHookTypesDev();\n        return mountTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        updateHookTypesDev();\n        return mountSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        updateHookTypesDev();\n        return mountId();\n      },\n      useActionState: function (action, initialState) {\n        currentHookNameInDev = \"useActionState\";\n        updateHookTypesDev();\n        return mountActionState(action, initialState);\n      },\n      useFormState: function (action, initialState) {\n        currentHookNameInDev = \"useFormState\";\n        updateHookTypesDev();\n        warnOnUseFormStateInDev();\n        return mountActionState(action, initialState);\n      },\n      useOptimistic: function (passthrough) {\n        currentHookNameInDev = \"useOptimistic\";\n        updateHookTypesDev();\n        return mountOptimistic(passthrough);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useMemoCache: useMemoCache,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        updateHookTypesDev();\n        return mountRefresh();\n      }\n    };\n    HooksDispatcherOnUpdateInDEV = {\n      readContext: function (context) {\n        return readContext(context);\n      },\n      use: use,\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        updateHookTypesDev();\n        return updateCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        updateHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        updateHookTypesDev();\n        updateEffectImpl(2048, Passive, create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        updateHookTypesDev();\n        return updateImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        updateHookTypesDev();\n        return updateEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        updateHookTypesDev();\n        return updateEffectImpl(4, Layout, create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function () {\n        currentHookNameInDev = \"useRef\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useState: function () {\n        currentHookNameInDev = \"useState\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateReducer(basicStateReducer);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        updateHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        updateHookTypesDev();\n        return updateDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        updateHookTypesDev();\n        return updateTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        updateHookTypesDev();\n        return updateSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useFormState: function (action) {\n        currentHookNameInDev = \"useFormState\";\n        updateHookTypesDev();\n        warnOnUseFormStateInDev();\n        return updateActionState(action);\n      },\n      useActionState: function (action) {\n        currentHookNameInDev = \"useActionState\";\n        updateHookTypesDev();\n        return updateActionState(action);\n      },\n      useOptimistic: function (passthrough, reducer) {\n        currentHookNameInDev = \"useOptimistic\";\n        updateHookTypesDev();\n        return updateOptimistic(passthrough, reducer);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useMemoCache: useMemoCache,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      }\n    };\n    HooksDispatcherOnRerenderInDEV = {\n      readContext: function (context) {\n        return readContext(context);\n      },\n      use: use,\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        updateHookTypesDev();\n        return updateCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        updateHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        updateHookTypesDev();\n        updateEffectImpl(2048, Passive, create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        updateHookTypesDev();\n        return updateImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        updateHookTypesDev();\n        return updateEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        updateHookTypesDev();\n        return updateEffectImpl(4, Layout, create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnRerenderInDEV;\n        try {\n          return updateMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnRerenderInDEV;\n        try {\n          return rerenderReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function () {\n        currentHookNameInDev = \"useRef\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useState: function () {\n        currentHookNameInDev = \"useState\";\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnRerenderInDEV;\n        try {\n          return rerenderReducer(basicStateReducer);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        updateHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        updateHookTypesDev();\n        return rerenderDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        updateHookTypesDev();\n        return rerenderTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        updateHookTypesDev();\n        return updateSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useFormState: function (action) {\n        currentHookNameInDev = \"useFormState\";\n        updateHookTypesDev();\n        warnOnUseFormStateInDev();\n        return rerenderActionState(action);\n      },\n      useActionState: function (action) {\n        currentHookNameInDev = \"useActionState\";\n        updateHookTypesDev();\n        return rerenderActionState(action);\n      },\n      useOptimistic: function (passthrough, reducer) {\n        currentHookNameInDev = \"useOptimistic\";\n        updateHookTypesDev();\n        return rerenderOptimistic(passthrough, reducer);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useMemoCache: useMemoCache,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      }\n    };\n    InvalidNestedHooksDispatcherOnMountInDEV = {\n      readContext: function (context) {\n        warnInvalidContextAccess();\n        return readContext(context);\n      },\n      use: function (usable) {\n        warnInvalidHookAccess();\n        return use(usable);\n      },\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountEffect(create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        mountEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountLayoutEffect(create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function (initialValue) {\n        currentHookNameInDev = \"useRef\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountRef(initialValue);\n      },\n      useState: function (initialState) {\n        currentHookNameInDev = \"useState\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnMountInDEV;\n        try {\n          return mountState(initialState);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountId();\n      },\n      useFormState: function (action, initialState) {\n        currentHookNameInDev = \"useFormState\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountActionState(action, initialState);\n      },\n      useActionState: function (action, initialState) {\n        currentHookNameInDev = \"useActionState\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountActionState(action, initialState);\n      },\n      useOptimistic: function (passthrough) {\n        currentHookNameInDev = \"useOptimistic\";\n        warnInvalidHookAccess();\n        mountHookTypesDev();\n        return mountOptimistic(passthrough);\n      },\n      useMemoCache: function (size) {\n        warnInvalidHookAccess();\n        return useMemoCache(size);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        mountHookTypesDev();\n        return mountRefresh();\n      }\n    };\n    InvalidNestedHooksDispatcherOnUpdateInDEV = {\n      readContext: function (context) {\n        warnInvalidContextAccess();\n        return readContext(context);\n      },\n      use: function (usable) {\n        warnInvalidHookAccess();\n        return use(usable);\n      },\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        updateEffectImpl(2048, Passive, create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateEffectImpl(4, Layout, create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function () {\n        currentHookNameInDev = \"useRef\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useState: function () {\n        currentHookNameInDev = \"useState\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateReducer(basicStateReducer);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useFormState: function (action) {\n        currentHookNameInDev = \"useFormState\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateActionState(action);\n      },\n      useActionState: function (action) {\n        currentHookNameInDev = \"useActionState\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateActionState(action);\n      },\n      useOptimistic: function (passthrough, reducer) {\n        currentHookNameInDev = \"useOptimistic\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateOptimistic(passthrough, reducer);\n      },\n      useMemoCache: function (size) {\n        warnInvalidHookAccess();\n        return useMemoCache(size);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      }\n    };\n    InvalidNestedHooksDispatcherOnRerenderInDEV = {\n      readContext: function (context) {\n        warnInvalidContextAccess();\n        return readContext(context);\n      },\n      use: function (usable) {\n        warnInvalidHookAccess();\n        return use(usable);\n      },\n      useCallback: function (callback, deps) {\n        currentHookNameInDev = \"useCallback\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateCallback(callback, deps);\n      },\n      useContext: function (context) {\n        currentHookNameInDev = \"useContext\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return readContext(context);\n      },\n      useEffect: function (create, createDeps) {\n        currentHookNameInDev = \"useEffect\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        updateEffectImpl(2048, Passive, create, createDeps);\n      },\n      useImperativeHandle: function (ref, create, deps) {\n        currentHookNameInDev = \"useImperativeHandle\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateImperativeHandle(ref, create, deps);\n      },\n      useInsertionEffect: function (create, deps) {\n        currentHookNameInDev = \"useInsertionEffect\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateEffectImpl(4, Insertion, create, deps);\n      },\n      useLayoutEffect: function (create, deps) {\n        currentHookNameInDev = \"useLayoutEffect\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateEffectImpl(4, Layout, create, deps);\n      },\n      useMemo: function (create, deps) {\n        currentHookNameInDev = \"useMemo\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return updateMemo(create, deps);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useReducer: function (reducer, initialArg, init) {\n        currentHookNameInDev = \"useReducer\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return rerenderReducer(reducer, initialArg, init);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useRef: function () {\n        currentHookNameInDev = \"useRef\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useState: function () {\n        currentHookNameInDev = \"useState\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        var prevDispatcher = ReactSharedInternals.H;\n        ReactSharedInternals.H = InvalidNestedHooksDispatcherOnUpdateInDEV;\n        try {\n          return rerenderReducer(basicStateReducer);\n        } finally {\n          ReactSharedInternals.H = prevDispatcher;\n        }\n      },\n      useDebugValue: function () {\n        currentHookNameInDev = \"useDebugValue\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n      },\n      useDeferredValue: function (value, initialValue) {\n        currentHookNameInDev = \"useDeferredValue\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return rerenderDeferredValue(value, initialValue);\n      },\n      useTransition: function () {\n        currentHookNameInDev = \"useTransition\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return rerenderTransition();\n      },\n      useSyncExternalStore: function (\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      ) {\n        currentHookNameInDev = \"useSyncExternalStore\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateSyncExternalStore(\n          subscribe,\n          getSnapshot,\n          getServerSnapshot\n        );\n      },\n      useId: function () {\n        currentHookNameInDev = \"useId\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      },\n      useFormState: function (action) {\n        currentHookNameInDev = \"useFormState\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return rerenderActionState(action);\n      },\n      useActionState: function (action) {\n        currentHookNameInDev = \"useActionState\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return rerenderActionState(action);\n      },\n      useOptimistic: function (passthrough, reducer) {\n        currentHookNameInDev = \"useOptimistic\";\n        warnInvalidHookAccess();\n        updateHookTypesDev();\n        return rerenderOptimistic(passthrough, reducer);\n      },\n      useMemoCache: function (size) {\n        warnInvalidHookAccess();\n        return useMemoCache(size);\n      },\n      useHostTransitionStatus: useHostTransitionStatus,\n      useCacheRefresh: function () {\n        currentHookNameInDev = \"useCacheRefresh\";\n        updateHookTypesDev();\n        return updateWorkInProgressHook().memoizedState;\n      }\n    };\n    var callComponent = {\n        \"react-stack-bottom-frame\": function (Component, props, secondArg) {\n          var wasRendering = isRendering;\n          isRendering = !0;\n          try {\n            return Component(props, secondArg);\n          } finally {\n            isRendering = wasRendering;\n          }\n        }\n      },\n      callComponentInDEV =\n        callComponent[\"react-stack-bottom-frame\"].bind(callComponent),\n      callRender = {\n        \"react-stack-bottom-frame\": function (instance) {\n          var wasRendering = isRendering;\n          isRendering = !0;\n          try {\n            return instance.render();\n          } finally {\n            isRendering = wasRendering;\n          }\n        }\n      },\n      callRenderInDEV = callRender[\"react-stack-bottom-frame\"].bind(callRender),\n      callComponentDidMount = {\n        \"react-stack-bottom-frame\": function (finishedWork, instance) {\n          try {\n            instance.componentDidMount();\n          } catch (error) {\n            captureCommitPhaseError(finishedWork, finishedWork.return, error);\n          }\n        }\n      },\n      callComponentDidMountInDEV = callComponentDidMount[\n        \"react-stack-bottom-frame\"\n      ].bind(callComponentDidMount),\n      callComponentDidUpdate = {\n        \"react-stack-bottom-frame\": function (\n          finishedWork,\n          instance,\n          prevProps,\n          prevState,\n          snapshot\n        ) {\n          try {\n            instance.componentDidUpdate(prevProps, prevState, snapshot);\n          } catch (error) {\n            captureCommitPhaseError(finishedWork, finishedWork.return, error);\n          }\n        }\n      },\n      callComponentDidUpdateInDEV = callComponentDidUpdate[\n        \"react-stack-bottom-frame\"\n      ].bind(callComponentDidUpdate),\n      callComponentDidCatch = {\n        \"react-stack-bottom-frame\": function (instance, errorInfo) {\n          var stack = errorInfo.stack;\n          instance.componentDidCatch(errorInfo.value, {\n            componentStack: null !== stack ? stack : \"\"\n          });\n        }\n      },\n      callComponentDidCatchInDEV = callComponentDidCatch[\n        \"react-stack-bottom-frame\"\n      ].bind(callComponentDidCatch),\n      callComponentWillUnmount = {\n        \"react-stack-bottom-frame\": function (\n          current,\n          nearestMountedAncestor,\n          instance\n        ) {\n          try {\n            instance.componentWillUnmount();\n          } catch (error) {\n            captureCommitPhaseError(current, nearestMountedAncestor, error);\n          }\n        }\n      },\n      callComponentWillUnmountInDEV = callComponentWillUnmount[\n        \"react-stack-bottom-frame\"\n      ].bind(callComponentWillUnmount),\n      callCreate = {\n        \"react-stack-bottom-frame\": function (effect) {\n          null != effect.resourceKind &&\n            console.error(\n              \"Expected only SimpleEffects when enableUseEffectCRUDOverload is disabled, got %s\",\n              effect.resourceKind\n            );\n          var create = effect.create;\n          effect = effect.inst;\n          create = create();\n          return (effect.destroy = create);\n        }\n      },\n      callCreateInDEV = callCreate[\"react-stack-bottom-frame\"].bind(callCreate),\n      callDestroy = {\n        \"react-stack-bottom-frame\": function (\n          current,\n          nearestMountedAncestor,\n          destroy\n        ) {\n          try {\n            destroy();\n          } catch (error) {\n            captureCommitPhaseError(current, nearestMountedAncestor, error);\n          }\n        }\n      },\n      callDestroyInDEV =\n        callDestroy[\"react-stack-bottom-frame\"].bind(callDestroy),\n      callLazyInit = {\n        \"react-stack-bottom-frame\": function (lazy) {\n          var init = lazy._init;\n          return init(lazy._payload);\n        }\n      },\n      callLazyInitInDEV =\n        callLazyInit[\"react-stack-bottom-frame\"].bind(callLazyInit),\n      thenableState = null,\n      thenableIndexCounter = 0,\n      currentDebugInfo = null,\n      didWarnAboutMaps;\n    var didWarnAboutGenerators = (didWarnAboutMaps = !1);\n    var ownerHasKeyUseWarning = {};\n    var ownerHasFunctionTypeWarning = {};\n    var ownerHasSymbolTypeWarning = {};\n    warnForMissingKey = function (returnFiber, workInProgress, child) {\n      if (\n        null !== child &&\n        \"object\" === typeof child &&\n        child._store &&\n        ((!child._store.validated && null == child.key) ||\n          2 === child._store.validated)\n      ) {\n        if (\"object\" !== typeof child._store)\n          throw Error(\n            \"React Component in warnForMissingKey should have a _store. This error is likely caused by a bug in React. Please file an issue.\"\n          );\n        child._store.validated = 1;\n        var componentName = getComponentNameFromFiber(returnFiber),\n          componentKey = componentName || \"null\";\n        if (!ownerHasKeyUseWarning[componentKey]) {\n          ownerHasKeyUseWarning[componentKey] = !0;\n          child = child._owner;\n          returnFiber = returnFiber._debugOwner;\n          var currentComponentErrorInfo = \"\";\n          returnFiber &&\n            \"number\" === typeof returnFiber.tag &&\n            (componentKey = getComponentNameFromFiber(returnFiber)) &&\n            (currentComponentErrorInfo =\n              \"\\n\\nCheck the render method of `\" + componentKey + \"`.\");\n          currentComponentErrorInfo ||\n            (componentName &&\n              (currentComponentErrorInfo =\n                \"\\n\\nCheck the top-level render call using <\" +\n                componentName +\n                \">.\"));\n          var childOwnerAppendix = \"\";\n          null != child &&\n            returnFiber !== child &&\n            ((componentName = null),\n            \"number\" === typeof child.tag\n              ? (componentName = getComponentNameFromFiber(child))\n              : \"string\" === typeof child.name && (componentName = child.name),\n            componentName &&\n              (childOwnerAppendix =\n                \" It was passed a child from \" + componentName + \".\"));\n          runWithFiberInDEV(workInProgress, function () {\n            console.error(\n              'Each child in a list should have a unique \"key\" prop.%s%s See https://react.dev/link/warning-keys for more information.',\n              currentComponentErrorInfo,\n              childOwnerAppendix\n            );\n          });\n        }\n      }\n    };\n    var reconcileChildFibers = createChildReconciler(!0),\n      mountChildFibers = createChildReconciler(!1),\n      suspenseHandlerStackCursor = createCursor(null),\n      shellBoundary = null,\n      SubtreeSuspenseContextMask = 1,\n      ForceSuspenseFallback = 2,\n      suspenseStackCursor = createCursor(0),\n      fakeInternalInstance = {};\n    var didWarnAboutStateAssignmentForComponent = new Set();\n    var didWarnAboutUninitializedState = new Set();\n    var didWarnAboutGetSnapshotBeforeUpdateWithoutDidUpdate = new Set();\n    var didWarnAboutLegacyLifecyclesAndDerivedState = new Set();\n    var didWarnAboutDirectlyAssigningPropsToState = new Set();\n    var didWarnAboutUndefinedDerivedState = new Set();\n    var didWarnAboutContextTypes$1 = new Set();\n    var didWarnAboutChildContextTypes = new Set();\n    var didWarnAboutInvalidateContextType = new Set();\n    var didWarnOnInvalidCallback = new Set();\n    Object.freeze(fakeInternalInstance);\n    var classComponentUpdater = {\n        enqueueSetState: function (inst, payload, callback) {\n          inst = inst._reactInternals;\n          var lane = requestUpdateLane(inst),\n            update = createUpdate(lane);\n          update.payload = payload;\n          void 0 !== callback &&\n            null !== callback &&\n            (warnOnInvalidCallback(callback), (update.callback = callback));\n          payload = enqueueUpdate(inst, update, lane);\n          null !== payload &&\n            (scheduleUpdateOnFiber(payload, inst, lane),\n            entangleTransitions(payload, inst, lane));\n          markStateUpdateScheduled(inst, lane);\n        },\n        enqueueReplaceState: function (inst, payload, callback) {\n          inst = inst._reactInternals;\n          var lane = requestUpdateLane(inst),\n            update = createUpdate(lane);\n          update.tag = ReplaceState;\n          update.payload = payload;\n          void 0 !== callback &&\n            null !== callback &&\n            (warnOnInvalidCallback(callback), (update.callback = callback));\n          payload = enqueueUpdate(inst, update, lane);\n          null !== payload &&\n            (scheduleUpdateOnFiber(payload, inst, lane),\n            entangleTransitions(payload, inst, lane));\n          markStateUpdateScheduled(inst, lane);\n        },\n        enqueueForceUpdate: function (inst, callback) {\n          inst = inst._reactInternals;\n          var lane = requestUpdateLane(inst),\n            update = createUpdate(lane);\n          update.tag = ForceUpdate;\n          void 0 !== callback &&\n            null !== callback &&\n            (warnOnInvalidCallback(callback), (update.callback = callback));\n          callback = enqueueUpdate(inst, update, lane);\n          null !== callback &&\n            (scheduleUpdateOnFiber(callback, inst, lane),\n            entangleTransitions(callback, inst, lane));\n          null !== injectedProfilingHooks &&\n            \"function\" ===\n              typeof injectedProfilingHooks.markForceUpdateScheduled &&\n            injectedProfilingHooks.markForceUpdateScheduled(inst, lane);\n        }\n      },\n      reportGlobalError =\n        \"function\" === typeof reportError\n          ? reportError\n          : function (error) {\n              if (\n                \"object\" === typeof window &&\n                \"function\" === typeof window.ErrorEvent\n              ) {\n                var event = new window.ErrorEvent(\"error\", {\n                  bubbles: !0,\n                  cancelable: !0,\n                  message:\n                    \"object\" === typeof error &&\n                    null !== error &&\n                    \"string\" === typeof error.message\n                      ? String(error.message)\n                      : String(error),\n                  error: error\n                });\n                if (!window.dispatchEvent(event)) return;\n              } else if (\n                \"object\" === typeof process &&\n                \"function\" === typeof process.emit\n              ) {\n                process.emit(\"uncaughtException\", error);\n                return;\n              }\n              console.error(error);\n            },\n      componentName = null,\n      errorBoundaryName = null,\n      SelectiveHydrationException = Error(\n        \"This is not a real error. It's an implementation detail of React's selective hydration feature. If this leaks into userspace, it's a bug in React. Please file an issue.\"\n      ),\n      didReceiveUpdate = !1;\n    var didWarnAboutBadClass = {};\n    var didWarnAboutContextTypeOnFunctionComponent = {};\n    var didWarnAboutContextTypes = {};\n    var didWarnAboutGetDerivedStateOnFunctionComponent = {};\n    var didWarnAboutReassigningProps = !1;\n    var didWarnAboutRevealOrder = {};\n    var didWarnAboutTailOptions = {};\n    var SUSPENDED_MARKER = {\n        dehydrated: null,\n        treeContext: null,\n        retryLane: 0,\n        hydrationErrors: null\n      },\n      hasWarnedAboutUsingNoValuePropOnContextProvider = !1,\n      didWarnAboutUndefinedSnapshotBeforeUpdate = null;\n    didWarnAboutUndefinedSnapshotBeforeUpdate = new Set();\n    var offscreenSubtreeIsHidden = !1,\n      offscreenSubtreeWasHidden = !1,\n      needsFormReset = !1,\n      PossiblyWeakSet = \"function\" === typeof WeakSet ? WeakSet : Set,\n      nextEffect = null,\n      inProgressLanes = null,\n      inProgressRoot = null,\n      hostParent = null,\n      hostParentIsContainer = !1,\n      currentHoistableRoot = null,\n      suspenseyCommitFlag = 8192,\n      DefaultAsyncDispatcher = {\n        getCacheForType: function (resourceType) {\n          var cache = readContext(CacheContext),\n            cacheForType = cache.data.get(resourceType);\n          void 0 === cacheForType &&\n            ((cacheForType = resourceType()),\n            cache.data.set(resourceType, cacheForType));\n          return cacheForType;\n        },\n        getOwner: function () {\n          return current;\n        }\n      };\n    if (\"function\" === typeof Symbol && Symbol.for) {\n      var symbolFor = Symbol.for;\n      symbolFor(\"selector.component\");\n      symbolFor(\"selector.has_pseudo_class\");\n      symbolFor(\"selector.role\");\n      symbolFor(\"selector.test_id\");\n      symbolFor(\"selector.text\");\n    }\n    var commitHooks = [],\n      PossiblyWeakMap = \"function\" === typeof WeakMap ? WeakMap : Map,\n      NoContext = 0,\n      RenderContext = 2,\n      CommitContext = 4,\n      RootInProgress = 0,\n      RootFatalErrored = 1,\n      RootErrored = 2,\n      RootSuspended = 3,\n      RootSuspendedWithDelay = 4,\n      RootSuspendedAtTheShell = 6,\n      RootCompleted = 5,\n      executionContext = NoContext,\n      workInProgressRoot = null,\n      workInProgress = null,\n      workInProgressRootRenderLanes = 0,\n      NotSuspended = 0,\n      SuspendedOnError = 1,\n      SuspendedOnData = 2,\n      SuspendedOnImmediate = 3,\n      SuspendedOnInstance = 4,\n      SuspendedOnInstanceAndReadyToContinue = 5,\n      SuspendedOnDeprecatedThrowPromise = 6,\n      SuspendedAndReadyToContinue = 7,\n      SuspendedOnHydration = 8,\n      SuspendedOnAction = 9,\n      workInProgressSuspendedReason = NotSuspended,\n      workInProgressThrownValue = null,\n      workInProgressRootDidSkipSuspendedSiblings = !1,\n      workInProgressRootIsPrerendering = !1,\n      workInProgressRootDidAttachPingListener = !1,\n      entangledRenderLanes = 0,\n      workInProgressRootExitStatus = RootInProgress,\n      workInProgressRootSkippedLanes = 0,\n      workInProgressRootInterleavedUpdatedLanes = 0,\n      workInProgressRootPingedLanes = 0,\n      workInProgressDeferredLane = 0,\n      workInProgressSuspendedRetryLanes = 0,\n      workInProgressRootConcurrentErrors = null,\n      workInProgressRootRecoverableErrors = null,\n      workInProgressRootDidIncludeRecursiveRenderUpdate = !1,\n      globalMostRecentFallbackTime = 0,\n      FALLBACK_THROTTLE_MS = 300,\n      workInProgressRootRenderTargetTime = Infinity,\n      RENDER_TIMEOUT_MS = 500,\n      workInProgressTransitions = null,\n      legacyErrorBoundariesThatAlreadyFailed = null,\n      IMMEDIATE_COMMIT = 0,\n      SUSPENDED_COMMIT = 1,\n      THROTTLED_COMMIT = 2,\n      NO_PENDING_EFFECTS = 0,\n      PENDING_MUTATION_PHASE = 1,\n      PENDING_LAYOUT_PHASE = 2,\n      PENDING_AFTER_MUTATION_PHASE = 3,\n      PENDING_SPAWNED_WORK = 4,\n      PENDING_PASSIVE_PHASE = 5,\n      pendingEffectsStatus = 0,\n      pendingEffectsRoot = null,\n      pendingFinishedWork = null,\n      pendingEffectsLanes = 0,\n      pendingEffectsRemainingLanes = 0,\n      pendingPassiveTransitions = null,\n      pendingRecoverableErrors = null,\n      NESTED_UPDATE_LIMIT = 50,\n      nestedUpdateCount = 0,\n      rootWithNestedUpdates = null,\n      isFlushingPassiveEffects = !1,\n      didScheduleUpdateDuringPassiveEffects = !1,\n      NESTED_PASSIVE_UPDATE_LIMIT = 50,\n      nestedPassiveUpdateCount = 0,\n      rootWithPassiveNestedUpdates = null,\n      isRunningInsertionEffect = !1,\n      didWarnStateUpdateForNotYetMountedComponent = null,\n      didWarnAboutUpdateInRender = !1;\n    var didWarnAboutUpdateInRenderForAnotherComponent = new Set();\n    var fakeActCallbackNode$1 = {},\n      firstScheduledRoot = null,\n      lastScheduledRoot = null,\n      didScheduleMicrotask = !1,\n      didScheduleMicrotask_act = !1,\n      mightHavePendingSyncWork = !1,\n      isFlushingWork = !1,\n      currentEventTransitionLane = 0,\n      fakeActCallbackNode = {};\n    (function () {\n      for (var i = 0; i < simpleEventPluginEvents.length; i++) {\n        var eventName = simpleEventPluginEvents[i],\n          domEventName = eventName.toLowerCase();\n        eventName = eventName[0].toUpperCase() + eventName.slice(1);\n        registerSimpleEvent(domEventName, \"on\" + eventName);\n      }\n      registerSimpleEvent(ANIMATION_END, \"onAnimationEnd\");\n      registerSimpleEvent(ANIMATION_ITERATION, \"onAnimationIteration\");\n      registerSimpleEvent(ANIMATION_START, \"onAnimationStart\");\n      registerSimpleEvent(\"dblclick\", \"onDoubleClick\");\n      registerSimpleEvent(\"focusin\", \"onFocus\");\n      registerSimpleEvent(\"focusout\", \"onBlur\");\n      registerSimpleEvent(TRANSITION_RUN, \"onTransitionRun\");\n      registerSimpleEvent(TRANSITION_START, \"onTransitionStart\");\n      registerSimpleEvent(TRANSITION_CANCEL, \"onTransitionCancel\");\n      registerSimpleEvent(TRANSITION_END, \"onTransitionEnd\");\n    })();\n    registerDirectEvent(\"onMouseEnter\", [\"mouseout\", \"mouseover\"]);\n    registerDirectEvent(\"onMouseLeave\", [\"mouseout\", \"mouseover\"]);\n    registerDirectEvent(\"onPointerEnter\", [\"pointerout\", \"pointerover\"]);\n    registerDirectEvent(\"onPointerLeave\", [\"pointerout\", \"pointerover\"]);\n    registerTwoPhaseEvent(\n      \"onChange\",\n      \"change click focusin focusout input keydown keyup selectionchange\".split(\n        \" \"\n      )\n    );\n    registerTwoPhaseEvent(\n      \"onSelect\",\n      \"focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange\".split(\n        \" \"\n      )\n    );\n    registerTwoPhaseEvent(\"onBeforeInput\", [\n      \"compositionend\",\n      \"keypress\",\n      \"textInput\",\n      \"paste\"\n    ]);\n    registerTwoPhaseEvent(\n      \"onCompositionEnd\",\n      \"compositionend focusout keydown keypress keyup mousedown\".split(\" \")\n    );\n    registerTwoPhaseEvent(\n      \"onCompositionStart\",\n      \"compositionstart focusout keydown keypress keyup mousedown\".split(\" \")\n    );\n    registerTwoPhaseEvent(\n      \"onCompositionUpdate\",\n      \"compositionupdate focusout keydown keypress keyup mousedown\".split(\" \")\n    );\n    var mediaEventTypes =\n        \"abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting\".split(\n          \" \"\n        ),\n      nonDelegatedEvents = new Set(\n        \"beforetoggle cancel close invalid load scroll scrollend toggle\"\n          .split(\" \")\n          .concat(mediaEventTypes)\n      ),\n      listeningMarker = \"_reactListening\" + Math.random().toString(36).slice(2),\n      didWarnControlledToUncontrolled = !1,\n      didWarnUncontrolledToControlled = !1,\n      didWarnFormActionType = !1,\n      didWarnFormActionName = !1,\n      didWarnFormActionTarget = !1,\n      didWarnFormActionMethod = !1,\n      didWarnPopoverTargetObject = !1;\n    var didWarnForNewBooleanPropsWithEmptyValue = {};\n    var NORMALIZE_NEWLINES_REGEX = /\\r\\n?/g,\n      NORMALIZE_NULL_AND_REPLACEMENT_REGEX = /\\u0000|\\uFFFD/g,\n      xlinkNamespace = \"http://www.w3.org/1999/xlink\",\n      xmlNamespace = \"http://www.w3.org/XML/1998/namespace\",\n      EXPECTED_FORM_ACTION_URL =\n        \"javascript:throw new Error('React form unexpectedly submitted.')\",\n      SUPPRESS_HYDRATION_WARNING = \"suppressHydrationWarning\",\n      SUSPENSE_START_DATA = \"$\",\n      SUSPENSE_END_DATA = \"/$\",\n      SUSPENSE_PENDING_START_DATA = \"$?\",\n      SUSPENSE_FALLBACK_START_DATA = \"$!\",\n      PREAMBLE_CONTRIBUTION_HTML = 1,\n      PREAMBLE_CONTRIBUTION_BODY = 2,\n      PREAMBLE_CONTRIBUTION_HEAD = 4,\n      FORM_STATE_IS_MATCHING = \"F!\",\n      FORM_STATE_IS_NOT_MATCHING = \"F\",\n      DOCUMENT_READY_STATE_COMPLETE = \"complete\",\n      STYLE = \"style\",\n      HostContextNamespaceNone = 0,\n      HostContextNamespaceSvg = 1,\n      HostContextNamespaceMath = 2,\n      eventsEnabled = null,\n      selectionInformation = null,\n      warnedUnknownTags = { dialog: !0, webview: !0 },\n      currentPopstateTransitionEvent = null,\n      scheduleTimeout = \"function\" === typeof setTimeout ? setTimeout : void 0,\n      cancelTimeout =\n        \"function\" === typeof clearTimeout ? clearTimeout : void 0,\n      noTimeout = -1,\n      localPromise = \"function\" === typeof Promise ? Promise : void 0,\n      scheduleMicrotask =\n        \"function\" === typeof queueMicrotask\n          ? queueMicrotask\n          : \"undefined\" !== typeof localPromise\n            ? function (callback) {\n                return localPromise\n                  .resolve(null)\n                  .then(callback)\n                  .catch(handleErrorInNextTick);\n              }\n            : scheduleTimeout,\n      previousHydratableOnEnteringScopedSingleton = null,\n      NotLoaded = 0,\n      Loaded = 1,\n      Errored = 2,\n      Settled = 3,\n      Inserted = 4,\n      preloadPropsMap = new Map(),\n      preconnectsSet = new Set(),\n      previousDispatcher = ReactDOMSharedInternals.d;\n    ReactDOMSharedInternals.d = {\n      f: function () {\n        var previousWasRendering = previousDispatcher.f(),\n          wasRendering = flushSyncWork$1();\n        return previousWasRendering || wasRendering;\n      },\n      r: function (form) {\n        var formInst = getInstanceFromNode(form);\n        null !== formInst && 5 === formInst.tag && \"form\" === formInst.type\n          ? requestFormReset$1(formInst)\n          : previousDispatcher.r(form);\n      },\n      D: function (href) {\n        previousDispatcher.D(href);\n        preconnectAs(\"dns-prefetch\", href, null);\n      },\n      C: function (href, crossOrigin) {\n        previousDispatcher.C(href, crossOrigin);\n        preconnectAs(\"preconnect\", href, crossOrigin);\n      },\n      L: function (href, as, options) {\n        previousDispatcher.L(href, as, options);\n        var ownerDocument = globalDocument;\n        if (ownerDocument && href && as) {\n          var preloadSelector =\n            'link[rel=\"preload\"][as=\"' +\n            escapeSelectorAttributeValueInsideDoubleQuotes(as) +\n            '\"]';\n          \"image\" === as\n            ? options && options.imageSrcSet\n              ? ((preloadSelector +=\n                  '[imagesrcset=\"' +\n                  escapeSelectorAttributeValueInsideDoubleQuotes(\n                    options.imageSrcSet\n                  ) +\n                  '\"]'),\n                \"string\" === typeof options.imageSizes &&\n                  (preloadSelector +=\n                    '[imagesizes=\"' +\n                    escapeSelectorAttributeValueInsideDoubleQuotes(\n                      options.imageSizes\n                    ) +\n                    '\"]'))\n              : (preloadSelector +=\n                  '[href=\"' +\n                  escapeSelectorAttributeValueInsideDoubleQuotes(href) +\n                  '\"]')\n            : (preloadSelector +=\n                '[href=\"' +\n                escapeSelectorAttributeValueInsideDoubleQuotes(href) +\n                '\"]');\n          var key = preloadSelector;\n          switch (as) {\n            case \"style\":\n              key = getStyleKey(href);\n              break;\n            case \"script\":\n              key = getScriptKey(href);\n          }\n          preloadPropsMap.has(key) ||\n            ((href = assign(\n              {\n                rel: \"preload\",\n                href:\n                  \"image\" === as && options && options.imageSrcSet\n                    ? void 0\n                    : href,\n                as: as\n              },\n              options\n            )),\n            preloadPropsMap.set(key, href),\n            null !== ownerDocument.querySelector(preloadSelector) ||\n              (\"style\" === as &&\n                ownerDocument.querySelector(\n                  getStylesheetSelectorFromKey(key)\n                )) ||\n              (\"script\" === as &&\n                ownerDocument.querySelector(getScriptSelectorFromKey(key))) ||\n              ((as = ownerDocument.createElement(\"link\")),\n              setInitialProperties(as, \"link\", href),\n              markNodeAsHoistable(as),\n              ownerDocument.head.appendChild(as)));\n        }\n      },\n      m: function (href, options) {\n        previousDispatcher.m(href, options);\n        var ownerDocument = globalDocument;\n        if (ownerDocument && href) {\n          var as =\n              options && \"string\" === typeof options.as ? options.as : \"script\",\n            preloadSelector =\n              'link[rel=\"modulepreload\"][as=\"' +\n              escapeSelectorAttributeValueInsideDoubleQuotes(as) +\n              '\"][href=\"' +\n              escapeSelectorAttributeValueInsideDoubleQuotes(href) +\n              '\"]',\n            key = preloadSelector;\n          switch (as) {\n            case \"audioworklet\":\n            case \"paintworklet\":\n            case \"serviceworker\":\n            case \"sharedworker\":\n            case \"worker\":\n            case \"script\":\n              key = getScriptKey(href);\n          }\n          if (\n            !preloadPropsMap.has(key) &&\n            ((href = assign({ rel: \"modulepreload\", href: href }, options)),\n            preloadPropsMap.set(key, href),\n            null === ownerDocument.querySelector(preloadSelector))\n          ) {\n            switch (as) {\n              case \"audioworklet\":\n              case \"paintworklet\":\n              case \"serviceworker\":\n              case \"sharedworker\":\n              case \"worker\":\n              case \"script\":\n                if (ownerDocument.querySelector(getScriptSelectorFromKey(key)))\n                  return;\n            }\n            as = ownerDocument.createElement(\"link\");\n            setInitialProperties(as, \"link\", href);\n            markNodeAsHoistable(as);\n            ownerDocument.head.appendChild(as);\n          }\n        }\n      },\n      X: function (src, options) {\n        previousDispatcher.X(src, options);\n        var ownerDocument = globalDocument;\n        if (ownerDocument && src) {\n          var scripts = getResourcesFromRoot(ownerDocument).hoistableScripts,\n            key = getScriptKey(src),\n            resource = scripts.get(key);\n          resource ||\n            ((resource = ownerDocument.querySelector(\n              getScriptSelectorFromKey(key)\n            )),\n            resource ||\n              ((src = assign({ src: src, async: !0 }, options)),\n              (options = preloadPropsMap.get(key)) &&\n                adoptPreloadPropsForScript(src, options),\n              (resource = ownerDocument.createElement(\"script\")),\n              markNodeAsHoistable(resource),\n              setInitialProperties(resource, \"link\", src),\n              ownerDocument.head.appendChild(resource)),\n            (resource = {\n              type: \"script\",\n              instance: resource,\n              count: 1,\n              state: null\n            }),\n            scripts.set(key, resource));\n        }\n      },\n      S: function (href, precedence, options) {\n        previousDispatcher.S(href, precedence, options);\n        var ownerDocument = globalDocument;\n        if (ownerDocument && href) {\n          var styles = getResourcesFromRoot(ownerDocument).hoistableStyles,\n            key = getStyleKey(href);\n          precedence = precedence || \"default\";\n          var resource = styles.get(key);\n          if (!resource) {\n            var state = { loading: NotLoaded, preload: null };\n            if (\n              (resource = ownerDocument.querySelector(\n                getStylesheetSelectorFromKey(key)\n              ))\n            )\n              state.loading = Loaded | Inserted;\n            else {\n              href = assign(\n                {\n                  rel: \"stylesheet\",\n                  href: href,\n                  \"data-precedence\": precedence\n                },\n                options\n              );\n              (options = preloadPropsMap.get(key)) &&\n                adoptPreloadPropsForStylesheet(href, options);\n              var link = (resource = ownerDocument.createElement(\"link\"));\n              markNodeAsHoistable(link);\n              setInitialProperties(link, \"link\", href);\n              link._p = new Promise(function (resolve, reject) {\n                link.onload = resolve;\n                link.onerror = reject;\n              });\n              link.addEventListener(\"load\", function () {\n                state.loading |= Loaded;\n              });\n              link.addEventListener(\"error\", function () {\n                state.loading |= Errored;\n              });\n              state.loading |= Inserted;\n              insertStylesheet(resource, precedence, ownerDocument);\n            }\n            resource = {\n              type: \"stylesheet\",\n              instance: resource,\n              count: 1,\n              state: state\n            };\n            styles.set(key, resource);\n          }\n        }\n      },\n      M: function (src, options) {\n        previousDispatcher.M(src, options);\n        var ownerDocument = globalDocument;\n        if (ownerDocument && src) {\n          var scripts = getResourcesFromRoot(ownerDocument).hoistableScripts,\n            key = getScriptKey(src),\n            resource = scripts.get(key);\n          resource ||\n            ((resource = ownerDocument.querySelector(\n              getScriptSelectorFromKey(key)\n            )),\n            resource ||\n              ((src = assign({ src: src, async: !0, type: \"module\" }, options)),\n              (options = preloadPropsMap.get(key)) &&\n                adoptPreloadPropsForScript(src, options),\n              (resource = ownerDocument.createElement(\"script\")),\n              markNodeAsHoistable(resource),\n              setInitialProperties(resource, \"link\", src),\n              ownerDocument.head.appendChild(resource)),\n            (resource = {\n              type: \"script\",\n              instance: resource,\n              count: 1,\n              state: null\n            }),\n            scripts.set(key, resource));\n        }\n      }\n    };\n    var globalDocument = \"undefined\" === typeof document ? null : document,\n      tagCaches = null,\n      suspendedState = null,\n      LAST_PRECEDENCE = null,\n      precedencesByRoot = null,\n      NotPendingTransition = NotPending,\n      HostTransitionContext = {\n        $$typeof: REACT_CONTEXT_TYPE,\n        Provider: null,\n        Consumer: null,\n        _currentValue: NotPendingTransition,\n        _currentValue2: NotPendingTransition,\n        _threadCount: 0\n      },\n      badgeFormat = \"%c%s%c \",\n      badgeStyle =\n        \"background: #e6e6e6;background: light-dark(rgba(0,0,0,0.1), rgba(255,255,255,0.25));color: #000000;color: light-dark(#000000, #ffffff);border-radius: 2px\",\n      resetStyle = \"\",\n      pad = \" \",\n      bind = Function.prototype.bind;\n    var didWarnAboutNestedUpdates = !1;\n    var overrideHookState = null,\n      overrideHookStateDeletePath = null,\n      overrideHookStateRenamePath = null,\n      overrideProps = null,\n      overridePropsDeletePath = null,\n      overridePropsRenamePath = null,\n      scheduleUpdate = null,\n      setErrorHandler = null,\n      setSuspenseHandler = null;\n    overrideHookState = function (fiber, id, path, value) {\n      id = findHook(fiber, id);\n      null !== id &&\n        ((path = copyWithSetImpl(id.memoizedState, path, 0, value)),\n        (id.memoizedState = path),\n        (id.baseState = path),\n        (fiber.memoizedProps = assign({}, fiber.memoizedProps)),\n        (path = enqueueConcurrentRenderForLane(fiber, 2)),\n        null !== path && scheduleUpdateOnFiber(path, fiber, 2));\n    };\n    overrideHookStateDeletePath = function (fiber, id, path) {\n      id = findHook(fiber, id);\n      null !== id &&\n        ((path = copyWithDeleteImpl(id.memoizedState, path, 0)),\n        (id.memoizedState = path),\n        (id.baseState = path),\n        (fiber.memoizedProps = assign({}, fiber.memoizedProps)),\n        (path = enqueueConcurrentRenderForLane(fiber, 2)),\n        null !== path && scheduleUpdateOnFiber(path, fiber, 2));\n    };\n    overrideHookStateRenamePath = function (fiber, id, oldPath, newPath) {\n      id = findHook(fiber, id);\n      null !== id &&\n        ((oldPath = copyWithRename(id.memoizedState, oldPath, newPath)),\n        (id.memoizedState = oldPath),\n        (id.baseState = oldPath),\n        (fiber.memoizedProps = assign({}, fiber.memoizedProps)),\n        (oldPath = enqueueConcurrentRenderForLane(fiber, 2)),\n        null !== oldPath && scheduleUpdateOnFiber(oldPath, fiber, 2));\n    };\n    overrideProps = function (fiber, path, value) {\n      fiber.pendingProps = copyWithSetImpl(fiber.memoizedProps, path, 0, value);\n      fiber.alternate && (fiber.alternate.pendingProps = fiber.pendingProps);\n      path = enqueueConcurrentRenderForLane(fiber, 2);\n      null !== path && scheduleUpdateOnFiber(path, fiber, 2);\n    };\n    overridePropsDeletePath = function (fiber, path) {\n      fiber.pendingProps = copyWithDeleteImpl(fiber.memoizedProps, path, 0);\n      fiber.alternate && (fiber.alternate.pendingProps = fiber.pendingProps);\n      path = enqueueConcurrentRenderForLane(fiber, 2);\n      null !== path && scheduleUpdateOnFiber(path, fiber, 2);\n    };\n    overridePropsRenamePath = function (fiber, oldPath, newPath) {\n      fiber.pendingProps = copyWithRename(\n        fiber.memoizedProps,\n        oldPath,\n        newPath\n      );\n      fiber.alternate && (fiber.alternate.pendingProps = fiber.pendingProps);\n      oldPath = enqueueConcurrentRenderForLane(fiber, 2);\n      null !== oldPath && scheduleUpdateOnFiber(oldPath, fiber, 2);\n    };\n    scheduleUpdate = function (fiber) {\n      var root = enqueueConcurrentRenderForLane(fiber, 2);\n      null !== root && scheduleUpdateOnFiber(root, fiber, 2);\n    };\n    setErrorHandler = function (newShouldErrorImpl) {\n      shouldErrorImpl = newShouldErrorImpl;\n    };\n    setSuspenseHandler = function (newShouldSuspendImpl) {\n      shouldSuspendImpl = newShouldSuspendImpl;\n    };\n    var _enabled = !0,\n      return_targetInst = null,\n      hasScheduledReplayAttempt = !1,\n      queuedFocus = null,\n      queuedDrag = null,\n      queuedMouse = null,\n      queuedPointers = new Map(),\n      queuedPointerCaptures = new Map(),\n      queuedExplicitHydrationTargets = [],\n      discreteReplayableEvents =\n        \"mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset\".split(\n          \" \"\n        ),\n      lastScheduledReplayQueue = null;\n    ReactDOMHydrationRoot.prototype.render = ReactDOMRoot.prototype.render =\n      function (children) {\n        var root = this._internalRoot;\n        if (null === root) throw Error(\"Cannot update an unmounted root.\");\n        var args = arguments;\n        \"function\" === typeof args[1]\n          ? console.error(\n              \"does not support the second callback argument. To execute a side effect after rendering, declare it in a component body with useEffect().\"\n            )\n          : isValidContainer(args[1])\n            ? console.error(\n                \"You passed a container to the second argument of root.render(...). You don't need to pass it again since you already passed it to create the root.\"\n              )\n            : \"undefined\" !== typeof args[1] &&\n              console.error(\n                \"You passed a second argument to root.render(...) but it only accepts one argument.\"\n              );\n        args = children;\n        var current = root.current,\n          lane = requestUpdateLane(current);\n        updateContainerImpl(current, lane, args, root, null, null);\n      };\n    ReactDOMHydrationRoot.prototype.unmount = ReactDOMRoot.prototype.unmount =\n      function () {\n        var args = arguments;\n        \"function\" === typeof args[0] &&\n          console.error(\n            \"does not support a callback argument. To execute a side effect after rendering, declare it in a component body with useEffect().\"\n          );\n        args = this._internalRoot;\n        if (null !== args) {\n          this._internalRoot = null;\n          var container = args.containerInfo;\n          (executionContext & (RenderContext | CommitContext)) !== NoContext &&\n            console.error(\n              \"Attempted to synchronously unmount a root while React was already rendering. React cannot finish unmounting the root until the current render has completed, which may lead to a race condition.\"\n            );\n          updateContainerImpl(args.current, 2, null, args, null, null);\n          flushSyncWork$1();\n          container[internalContainerInstanceKey] = null;\n        }\n      };\n    ReactDOMHydrationRoot.prototype.unstable_scheduleHydration = function (\n      target\n    ) {\n      if (target) {\n        var updatePriority = resolveUpdatePriority();\n        target = { blockedOn: null, target: target, priority: updatePriority };\n        for (\n          var i = 0;\n          i < queuedExplicitHydrationTargets.length &&\n          0 !== updatePriority &&\n          updatePriority < queuedExplicitHydrationTargets[i].priority;\n          i++\n        );\n        queuedExplicitHydrationTargets.splice(i, 0, target);\n        0 === i && attemptExplicitHydrationTarget(target);\n      }\n    };\n    (function () {\n      var isomorphicReactPackageVersion = React.version;\n      if (\"19.1.0\" !== isomorphicReactPackageVersion)\n        throw Error(\n          'Incompatible React versions: The \"react\" and \"react-dom\" packages must have the exact same version. Instead got:\\n  - react:      ' +\n            (isomorphicReactPackageVersion +\n              \"\\n  - react-dom:  19.1.0\\nLearn more: https://react.dev/warnings/version-mismatch\")\n        );\n    })();\n    (\"function\" === typeof Map &&\n      null != Map.prototype &&\n      \"function\" === typeof Map.prototype.forEach &&\n      \"function\" === typeof Set &&\n      null != Set.prototype &&\n      \"function\" === typeof Set.prototype.clear &&\n      \"function\" === typeof Set.prototype.forEach) ||\n      console.error(\n        \"React depends on Map and Set built-in types. Make sure that you load a polyfill in older browsers. https://react.dev/link/react-polyfills\"\n      );\n    ReactDOMSharedInternals.findDOMNode = function (componentOrElement) {\n      var fiber = componentOrElement._reactInternals;\n      if (void 0 === fiber) {\n        if (\"function\" === typeof componentOrElement.render)\n          throw Error(\"Unable to find node on an unmounted component.\");\n        componentOrElement = Object.keys(componentOrElement).join(\",\");\n        throw Error(\n          \"Argument appears to not be a ReactComponent. Keys: \" +\n            componentOrElement\n        );\n      }\n      componentOrElement = findCurrentFiberUsingSlowPath(fiber);\n      componentOrElement =\n        null !== componentOrElement\n          ? findCurrentHostFiberImpl(componentOrElement)\n          : null;\n      componentOrElement =\n        null === componentOrElement ? null : componentOrElement.stateNode;\n      return componentOrElement;\n    };\n    if (\n      !(function () {\n        var internals = {\n          bundleType: 1,\n          version: \"19.1.0\",\n          rendererPackageName: \"react-dom\",\n          currentDispatcherRef: ReactSharedInternals,\n          reconcilerVersion: \"19.1.0\"\n        };\n        internals.overrideHookState = overrideHookState;\n        internals.overrideHookStateDeletePath = overrideHookStateDeletePath;\n        internals.overrideHookStateRenamePath = overrideHookStateRenamePath;\n        internals.overrideProps = overrideProps;\n        internals.overridePropsDeletePath = overridePropsDeletePath;\n        internals.overridePropsRenamePath = overridePropsRenamePath;\n        internals.scheduleUpdate = scheduleUpdate;\n        internals.setErrorHandler = setErrorHandler;\n        internals.setSuspenseHandler = setSuspenseHandler;\n        internals.scheduleRefresh = scheduleRefresh;\n        internals.scheduleRoot = scheduleRoot;\n        internals.setRefreshHandler = setRefreshHandler;\n        internals.getCurrentFiber = getCurrentFiberForDevTools;\n        internals.getLaneLabelMap = getLaneLabelMap;\n        internals.injectProfilingHooks = injectProfilingHooks;\n        return injectInternals(internals);\n      })() &&\n      canUseDOM &&\n      window.top === window.self &&\n      ((-1 < navigator.userAgent.indexOf(\"Chrome\") &&\n        -1 === navigator.userAgent.indexOf(\"Edge\")) ||\n        -1 < navigator.userAgent.indexOf(\"Firefox\"))\n    ) {\n      var protocol = window.location.protocol;\n      /^(https?|file):$/.test(protocol) &&\n        console.info(\n          \"%cDownload the React DevTools for a better development experience: https://react.dev/link/react-devtools\" +\n            (\"file:\" === protocol\n              ? \"\\nYou might need to use a local HTTP server (instead of file://): https://react.dev/link/react-devtools-faq\"\n              : \"\"),\n          \"font-weight:bold\"\n        );\n    }\n    exports.createRoot = function (container, options) {\n      if (!isValidContainer(container))\n        throw Error(\"Target container is not a DOM element.\");\n      warnIfReactDOMContainerInDEV(container);\n      var isStrictMode = !1,\n        identifierPrefix = \"\",\n        onUncaughtError = defaultOnUncaughtError,\n        onCaughtError = defaultOnCaughtError,\n        onRecoverableError = defaultOnRecoverableError,\n        transitionCallbacks = null;\n      null !== options &&\n        void 0 !== options &&\n        (options.hydrate\n          ? console.warn(\n              \"hydrate through createRoot is deprecated. Use ReactDOMClient.hydrateRoot(container, <App />) instead.\"\n            )\n          : \"object\" === typeof options &&\n            null !== options &&\n            options.$$typeof === REACT_ELEMENT_TYPE &&\n            console.error(\n              \"You passed a JSX element to createRoot. You probably meant to call root.render instead. Example usage:\\n\\n  let root = createRoot(domContainer);\\n  root.render(<App />);\"\n            ),\n        !0 === options.unstable_strictMode && (isStrictMode = !0),\n        void 0 !== options.identifierPrefix &&\n          (identifierPrefix = options.identifierPrefix),\n        void 0 !== options.onUncaughtError &&\n          (onUncaughtError = options.onUncaughtError),\n        void 0 !== options.onCaughtError &&\n          (onCaughtError = options.onCaughtError),\n        void 0 !== options.onRecoverableError &&\n          (onRecoverableError = options.onRecoverableError),\n        void 0 !== options.unstable_transitionCallbacks &&\n          (transitionCallbacks = options.unstable_transitionCallbacks));\n      options = createFiberRoot(\n        container,\n        1,\n        !1,\n        null,\n        null,\n        isStrictMode,\n        identifierPrefix,\n        onUncaughtError,\n        onCaughtError,\n        onRecoverableError,\n        transitionCallbacks,\n        null\n      );\n      container[internalContainerInstanceKey] = options.current;\n      listenToAllSupportedEvents(container);\n      return new ReactDOMRoot(options);\n    };\n    exports.hydrateRoot = function (container, initialChildren, options) {\n      if (!isValidContainer(container))\n        throw Error(\"Target container is not a DOM element.\");\n      warnIfReactDOMContainerInDEV(container);\n      void 0 === initialChildren &&\n        console.error(\n          \"Must provide initial children as second argument to hydrateRoot. Example usage: hydrateRoot(domContainer, <App />)\"\n        );\n      var isStrictMode = !1,\n        identifierPrefix = \"\",\n        onUncaughtError = defaultOnUncaughtError,\n        onCaughtError = defaultOnCaughtError,\n        onRecoverableError = defaultOnRecoverableError,\n        transitionCallbacks = null,\n        formState = null;\n      null !== options &&\n        void 0 !== options &&\n        (!0 === options.unstable_strictMode && (isStrictMode = !0),\n        void 0 !== options.identifierPrefix &&\n          (identifierPrefix = options.identifierPrefix),\n        void 0 !== options.onUncaughtError &&\n          (onUncaughtError = options.onUncaughtError),\n        void 0 !== options.onCaughtError &&\n          (onCaughtError = options.onCaughtError),\n        void 0 !== options.onRecoverableError &&\n          (onRecoverableError = options.onRecoverableError),\n        void 0 !== options.unstable_transitionCallbacks &&\n          (transitionCallbacks = options.unstable_transitionCallbacks),\n        void 0 !== options.formState && (formState = options.formState));\n      initialChildren = createFiberRoot(\n        container,\n        1,\n        !0,\n        initialChildren,\n        null != options ? options : null,\n        isStrictMode,\n        identifierPrefix,\n        onUncaughtError,\n        onCaughtError,\n        onRecoverableError,\n        transitionCallbacks,\n        formState\n      );\n      initialChildren.context = getContextForSubtree(null);\n      options = initialChildren.current;\n      isStrictMode = requestUpdateLane(options);\n      isStrictMode = getBumpedLaneForHydrationByLane(isStrictMode);\n      identifierPrefix = createUpdate(isStrictMode);\n      identifierPrefix.callback = null;\n      enqueueUpdate(options, identifierPrefix, isStrictMode);\n      options = isStrictMode;\n      initialChildren.current.lanes = options;\n      markRootUpdated$1(initialChildren, options);\n      ensureRootIsScheduled(initialChildren);\n      container[internalContainerInstanceKey] = initialChildren.current;\n      listenToAllSupportedEvents(container);\n      return new ReactDOMHydrationRoot(initialChildren);\n    };\n    exports.version = \"19.1.0\";\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(Error());\n  })();\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-dom/cjs/react-dom-client.development.js?");

/***/ }),

/***/ "./node_modules/react-dom/cjs/react-dom.development.js":
/*!*************************************************************!*\
  !*** ./node_modules/react-dom/cjs/react-dom.development.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("/**\n * @license React\n * react-dom.development.js\n *\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n true &&\n  (function () {\n    function noop() {}\n    function testStringCoercion(value) {\n      return \"\" + value;\n    }\n    function createPortal$1(children, containerInfo, implementation) {\n      var key =\n        3 < arguments.length && void 0 !== arguments[3] ? arguments[3] : null;\n      try {\n        testStringCoercion(key);\n        var JSCompiler_inline_result = !1;\n      } catch (e) {\n        JSCompiler_inline_result = !0;\n      }\n      JSCompiler_inline_result &&\n        (console.error(\n          \"The provided key is an unsupported type %s. This value must be coerced to a string before using it here.\",\n          (\"function\" === typeof Symbol &&\n            Symbol.toStringTag &&\n            key[Symbol.toStringTag]) ||\n            key.constructor.name ||\n            \"Object\"\n        ),\n        testStringCoercion(key));\n      return {\n        $$typeof: REACT_PORTAL_TYPE,\n        key: null == key ? null : \"\" + key,\n        children: children,\n        containerInfo: containerInfo,\n        implementation: implementation\n      };\n    }\n    function getCrossOriginStringAs(as, input) {\n      if (\"font\" === as) return \"\";\n      if (\"string\" === typeof input)\n        return \"use-credentials\" === input ? input : \"\";\n    }\n    function getValueDescriptorExpectingObjectForWarning(thing) {\n      return null === thing\n        ? \"`null`\"\n        : void 0 === thing\n          ? \"`undefined`\"\n          : \"\" === thing\n            ? \"an empty string\"\n            : 'something with type \"' + typeof thing + '\"';\n    }\n    function getValueDescriptorExpectingEnumForWarning(thing) {\n      return null === thing\n        ? \"`null`\"\n        : void 0 === thing\n          ? \"`undefined`\"\n          : \"\" === thing\n            ? \"an empty string\"\n            : \"string\" === typeof thing\n              ? JSON.stringify(thing)\n              : \"number\" === typeof thing\n                ? \"`\" + thing + \"`\"\n                : 'something with type \"' + typeof thing + '\"';\n    }\n    function resolveDispatcher() {\n      var dispatcher = ReactSharedInternals.H;\n      null === dispatcher &&\n        console.error(\n          \"Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:\\n1. You might have mismatching versions of React and the renderer (such as React DOM)\\n2. You might be breaking the Rules of Hooks\\n3. You might have more than one copy of React in the same app\\nSee https://react.dev/link/invalid-hook-call for tips about how to debug and fix this problem.\"\n        );\n      return dispatcher;\n    }\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(Error());\n    var React = __webpack_require__(/*! react */ \"./node_modules/react/index.js\"),\n      Internals = {\n        d: {\n          f: noop,\n          r: function () {\n            throw Error(\n              \"Invalid form element. requestFormReset must be passed a form that was rendered by React.\"\n            );\n          },\n          D: noop,\n          C: noop,\n          L: noop,\n          m: noop,\n          X: noop,\n          S: noop,\n          M: noop\n        },\n        p: 0,\n        findDOMNode: null\n      },\n      REACT_PORTAL_TYPE = Symbol.for(\"react.portal\"),\n      ReactSharedInternals =\n        React.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;\n    (\"function\" === typeof Map &&\n      null != Map.prototype &&\n      \"function\" === typeof Map.prototype.forEach &&\n      \"function\" === typeof Set &&\n      null != Set.prototype &&\n      \"function\" === typeof Set.prototype.clear &&\n      \"function\" === typeof Set.prototype.forEach) ||\n      console.error(\n        \"React depends on Map and Set built-in types. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills\"\n      );\n    exports.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE =\n      Internals;\n    exports.createPortal = function (children, container) {\n      var key =\n        2 < arguments.length && void 0 !== arguments[2] ? arguments[2] : null;\n      if (\n        !container ||\n        (1 !== container.nodeType &&\n          9 !== container.nodeType &&\n          11 !== container.nodeType)\n      )\n        throw Error(\"Target container is not a DOM element.\");\n      return createPortal$1(children, container, null, key);\n    };\n    exports.flushSync = function (fn) {\n      var previousTransition = ReactSharedInternals.T,\n        previousUpdatePriority = Internals.p;\n      try {\n        if (((ReactSharedInternals.T = null), (Internals.p = 2), fn))\n          return fn();\n      } finally {\n        (ReactSharedInternals.T = previousTransition),\n          (Internals.p = previousUpdatePriority),\n          Internals.d.f() &&\n            console.error(\n              \"flushSync was called from inside a lifecycle method. React cannot flush when React is already rendering. Consider moving this call to a scheduler task or micro task.\"\n            );\n      }\n    };\n    exports.preconnect = function (href, options) {\n      \"string\" === typeof href && href\n        ? null != options && \"object\" !== typeof options\n          ? console.error(\n              \"ReactDOM.preconnect(): Expected the `options` argument (second) to be an object but encountered %s instead. The only supported option at this time is `crossOrigin` which accepts a string.\",\n              getValueDescriptorExpectingEnumForWarning(options)\n            )\n          : null != options &&\n            \"string\" !== typeof options.crossOrigin &&\n            console.error(\n              \"ReactDOM.preconnect(): Expected the `crossOrigin` option (second argument) to be a string but encountered %s instead. Try removing this option or passing a string value instead.\",\n              getValueDescriptorExpectingObjectForWarning(options.crossOrigin)\n            )\n        : console.error(\n            \"ReactDOM.preconnect(): Expected the `href` argument (first) to be a non-empty string but encountered %s instead.\",\n            getValueDescriptorExpectingObjectForWarning(href)\n          );\n      \"string\" === typeof href &&\n        (options\n          ? ((options = options.crossOrigin),\n            (options =\n              \"string\" === typeof options\n                ? \"use-credentials\" === options\n                  ? options\n                  : \"\"\n                : void 0))\n          : (options = null),\n        Internals.d.C(href, options));\n    };\n    exports.prefetchDNS = function (href) {\n      if (\"string\" !== typeof href || !href)\n        console.error(\n          \"ReactDOM.prefetchDNS(): Expected the `href` argument (first) to be a non-empty string but encountered %s instead.\",\n          getValueDescriptorExpectingObjectForWarning(href)\n        );\n      else if (1 < arguments.length) {\n        var options = arguments[1];\n        \"object\" === typeof options && options.hasOwnProperty(\"crossOrigin\")\n          ? console.error(\n              \"ReactDOM.prefetchDNS(): Expected only one argument, `href`, but encountered %s as a second argument instead. This argument is reserved for future options and is currently disallowed. It looks like the you are attempting to set a crossOrigin property for this DNS lookup hint. Browsers do not perform DNS queries using CORS and setting this attribute on the resource hint has no effect. Try calling ReactDOM.prefetchDNS() with just a single string argument, `href`.\",\n              getValueDescriptorExpectingEnumForWarning(options)\n            )\n          : console.error(\n              \"ReactDOM.prefetchDNS(): Expected only one argument, `href`, but encountered %s as a second argument instead. This argument is reserved for future options and is currently disallowed. Try calling ReactDOM.prefetchDNS() with just a single string argument, `href`.\",\n              getValueDescriptorExpectingEnumForWarning(options)\n            );\n      }\n      \"string\" === typeof href && Internals.d.D(href);\n    };\n    exports.preinit = function (href, options) {\n      \"string\" === typeof href && href\n        ? null == options || \"object\" !== typeof options\n          ? console.error(\n              \"ReactDOM.preinit(): Expected the `options` argument (second) to be an object with an `as` property describing the type of resource to be preinitialized but encountered %s instead.\",\n              getValueDescriptorExpectingEnumForWarning(options)\n            )\n          : \"style\" !== options.as &&\n            \"script\" !== options.as &&\n            console.error(\n              'ReactDOM.preinit(): Expected the `as` property in the `options` argument (second) to contain a valid value describing the type of resource to be preinitialized but encountered %s instead. Valid values for `as` are \"style\" and \"script\".',\n              getValueDescriptorExpectingEnumForWarning(options.as)\n            )\n        : console.error(\n            \"ReactDOM.preinit(): Expected the `href` argument (first) to be a non-empty string but encountered %s instead.\",\n            getValueDescriptorExpectingObjectForWarning(href)\n          );\n      if (\n        \"string\" === typeof href &&\n        options &&\n        \"string\" === typeof options.as\n      ) {\n        var as = options.as,\n          crossOrigin = getCrossOriginStringAs(as, options.crossOrigin),\n          integrity =\n            \"string\" === typeof options.integrity ? options.integrity : void 0,\n          fetchPriority =\n            \"string\" === typeof options.fetchPriority\n              ? options.fetchPriority\n              : void 0;\n        \"style\" === as\n          ? Internals.d.S(\n              href,\n              \"string\" === typeof options.precedence\n                ? options.precedence\n                : void 0,\n              {\n                crossOrigin: crossOrigin,\n                integrity: integrity,\n                fetchPriority: fetchPriority\n              }\n            )\n          : \"script\" === as &&\n            Internals.d.X(href, {\n              crossOrigin: crossOrigin,\n              integrity: integrity,\n              fetchPriority: fetchPriority,\n              nonce: \"string\" === typeof options.nonce ? options.nonce : void 0\n            });\n      }\n    };\n    exports.preinitModule = function (href, options) {\n      var encountered = \"\";\n      (\"string\" === typeof href && href) ||\n        (encountered +=\n          \" The `href` argument encountered was \" +\n          getValueDescriptorExpectingObjectForWarning(href) +\n          \".\");\n      void 0 !== options && \"object\" !== typeof options\n        ? (encountered +=\n            \" The `options` argument encountered was \" +\n            getValueDescriptorExpectingObjectForWarning(options) +\n            \".\")\n        : options &&\n          \"as\" in options &&\n          \"script\" !== options.as &&\n          (encountered +=\n            \" The `as` option encountered was \" +\n            getValueDescriptorExpectingEnumForWarning(options.as) +\n            \".\");\n      if (encountered)\n        console.error(\n          \"ReactDOM.preinitModule(): Expected up to two arguments, a non-empty `href` string and, optionally, an `options` object with a valid `as` property.%s\",\n          encountered\n        );\n      else\n        switch (\n          ((encountered =\n            options && \"string\" === typeof options.as ? options.as : \"script\"),\n          encountered)\n        ) {\n          case \"script\":\n            break;\n          default:\n            (encountered =\n              getValueDescriptorExpectingEnumForWarning(encountered)),\n              console.error(\n                'ReactDOM.preinitModule(): Currently the only supported \"as\" type for this function is \"script\" but received \"%s\" instead. This warning was generated for `href` \"%s\". In the future other module types will be supported, aligning with the import-attributes proposal. Learn more here: (https://github.com/tc39/proposal-import-attributes)',\n                encountered,\n                href\n              );\n        }\n      if (\"string\" === typeof href)\n        if (\"object\" === typeof options && null !== options) {\n          if (null == options.as || \"script\" === options.as)\n            (encountered = getCrossOriginStringAs(\n              options.as,\n              options.crossOrigin\n            )),\n              Internals.d.M(href, {\n                crossOrigin: encountered,\n                integrity:\n                  \"string\" === typeof options.integrity\n                    ? options.integrity\n                    : void 0,\n                nonce:\n                  \"string\" === typeof options.nonce ? options.nonce : void 0\n              });\n        } else null == options && Internals.d.M(href);\n    };\n    exports.preload = function (href, options) {\n      var encountered = \"\";\n      (\"string\" === typeof href && href) ||\n        (encountered +=\n          \" The `href` argument encountered was \" +\n          getValueDescriptorExpectingObjectForWarning(href) +\n          \".\");\n      null == options || \"object\" !== typeof options\n        ? (encountered +=\n            \" The `options` argument encountered was \" +\n            getValueDescriptorExpectingObjectForWarning(options) +\n            \".\")\n        : (\"string\" === typeof options.as && options.as) ||\n          (encountered +=\n            \" The `as` option encountered was \" +\n            getValueDescriptorExpectingObjectForWarning(options.as) +\n            \".\");\n      encountered &&\n        console.error(\n          'ReactDOM.preload(): Expected two arguments, a non-empty `href` string and an `options` object with an `as` property valid for a `<link rel=\"preload\" as=\"...\" />` tag.%s',\n          encountered\n        );\n      if (\n        \"string\" === typeof href &&\n        \"object\" === typeof options &&\n        null !== options &&\n        \"string\" === typeof options.as\n      ) {\n        encountered = options.as;\n        var crossOrigin = getCrossOriginStringAs(\n          encountered,\n          options.crossOrigin\n        );\n        Internals.d.L(href, encountered, {\n          crossOrigin: crossOrigin,\n          integrity:\n            \"string\" === typeof options.integrity ? options.integrity : void 0,\n          nonce: \"string\" === typeof options.nonce ? options.nonce : void 0,\n          type: \"string\" === typeof options.type ? options.type : void 0,\n          fetchPriority:\n            \"string\" === typeof options.fetchPriority\n              ? options.fetchPriority\n              : void 0,\n          referrerPolicy:\n            \"string\" === typeof options.referrerPolicy\n              ? options.referrerPolicy\n              : void 0,\n          imageSrcSet:\n            \"string\" === typeof options.imageSrcSet\n              ? options.imageSrcSet\n              : void 0,\n          imageSizes:\n            \"string\" === typeof options.imageSizes\n              ? options.imageSizes\n              : void 0,\n          media: \"string\" === typeof options.media ? options.media : void 0\n        });\n      }\n    };\n    exports.preloadModule = function (href, options) {\n      var encountered = \"\";\n      (\"string\" === typeof href && href) ||\n        (encountered +=\n          \" The `href` argument encountered was \" +\n          getValueDescriptorExpectingObjectForWarning(href) +\n          \".\");\n      void 0 !== options && \"object\" !== typeof options\n        ? (encountered +=\n            \" The `options` argument encountered was \" +\n            getValueDescriptorExpectingObjectForWarning(options) +\n            \".\")\n        : options &&\n          \"as\" in options &&\n          \"string\" !== typeof options.as &&\n          (encountered +=\n            \" The `as` option encountered was \" +\n            getValueDescriptorExpectingObjectForWarning(options.as) +\n            \".\");\n      encountered &&\n        console.error(\n          'ReactDOM.preloadModule(): Expected two arguments, a non-empty `href` string and, optionally, an `options` object with an `as` property valid for a `<link rel=\"modulepreload\" as=\"...\" />` tag.%s',\n          encountered\n        );\n      \"string\" === typeof href &&\n        (options\n          ? ((encountered = getCrossOriginStringAs(\n              options.as,\n              options.crossOrigin\n            )),\n            Internals.d.m(href, {\n              as:\n                \"string\" === typeof options.as && \"script\" !== options.as\n                  ? options.as\n                  : void 0,\n              crossOrigin: encountered,\n              integrity:\n                \"string\" === typeof options.integrity\n                  ? options.integrity\n                  : void 0\n            }))\n          : Internals.d.m(href));\n    };\n    exports.requestFormReset = function (form) {\n      Internals.d.r(form);\n    };\n    exports.unstable_batchedUpdates = function (fn, a) {\n      return fn(a);\n    };\n    exports.useFormState = function (action, initialState, permalink) {\n      return resolveDispatcher().useFormState(action, initialState, permalink);\n    };\n    exports.useFormStatus = function () {\n      return resolveDispatcher().useHostTransitionStatus();\n    };\n    exports.version = \"19.1.0\";\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(Error());\n  })();\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-dom/cjs/react-dom.development.js?");

/***/ }),

/***/ "./node_modules/react-dom/client.js":
/*!******************************************!*\
  !*** ./node_modules/react-dom/client.js ***!
  \******************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nfunction checkDCE() {\n  /* global __REACT_DEVTOOLS_GLOBAL_HOOK__ */\n  if (\n    typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ === 'undefined' ||\n    typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE !== 'function'\n  ) {\n    return;\n  }\n  if (true) {\n    // This branch is unreachable because this function is only called\n    // in production, but the condition is true only in development.\n    // Therefore if the branch is still here, dead code elimination wasn't\n    // properly applied.\n    // Don't change the message. React DevTools relies on it. Also make sure\n    // this message doesn't occur elsewhere in this function, or it will cause\n    // a false positive.\n    throw new Error('^_^');\n  }\n  // removed by dead control flow\n{}\n}\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/react-dom-client.development.js */ \"./node_modules/react-dom/cjs/react-dom-client.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-dom/client.js?");

/***/ }),

/***/ "./node_modules/react-dom/index.js":
/*!*****************************************!*\
  !*** ./node_modules/react-dom/index.js ***!
  \*****************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nfunction checkDCE() {\n  /* global __REACT_DEVTOOLS_GLOBAL_HOOK__ */\n  if (\n    typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ === 'undefined' ||\n    typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE !== 'function'\n  ) {\n    return;\n  }\n  if (true) {\n    // This branch is unreachable because this function is only called\n    // in production, but the condition is true only in development.\n    // Therefore if the branch is still here, dead code elimination wasn't\n    // properly applied.\n    // Don't change the message. React DevTools relies on it. Also make sure\n    // this message doesn't occur elsewhere in this function, or it will cause\n    // a false positive.\n    throw new Error('^_^');\n  }\n  // removed by dead control flow\n{}\n}\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/react-dom.development.js */ \"./node_modules/react-dom/cjs/react-dom.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-dom/index.js?");

/***/ }),

/***/ "./node_modules/react-is/cjs/react-is.development.js":
/*!***********************************************************!*\
  !*** ./node_modules/react-is/cjs/react-is.development.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, exports) => {

eval("/**\n * @license React\n * react-is.development.js\n *\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n true &&\n  (function () {\n    function typeOf(object) {\n      if (\"object\" === typeof object && null !== object) {\n        var $$typeof = object.$$typeof;\n        switch ($$typeof) {\n          case REACT_ELEMENT_TYPE:\n            switch (((object = object.type), object)) {\n              case REACT_FRAGMENT_TYPE:\n              case REACT_PROFILER_TYPE:\n              case REACT_STRICT_MODE_TYPE:\n              case REACT_SUSPENSE_TYPE:\n              case REACT_SUSPENSE_LIST_TYPE:\n              case REACT_VIEW_TRANSITION_TYPE:\n                return object;\n              default:\n                switch (((object = object && object.$$typeof), object)) {\n                  case REACT_CONTEXT_TYPE:\n                  case REACT_FORWARD_REF_TYPE:\n                  case REACT_LAZY_TYPE:\n                  case REACT_MEMO_TYPE:\n                    return object;\n                  case REACT_CONSUMER_TYPE:\n                    return object;\n                  default:\n                    return $$typeof;\n                }\n            }\n          case REACT_PORTAL_TYPE:\n            return $$typeof;\n        }\n      }\n    }\n    var REACT_ELEMENT_TYPE = Symbol.for(\"react.transitional.element\"),\n      REACT_PORTAL_TYPE = Symbol.for(\"react.portal\"),\n      REACT_FRAGMENT_TYPE = Symbol.for(\"react.fragment\"),\n      REACT_STRICT_MODE_TYPE = Symbol.for(\"react.strict_mode\"),\n      REACT_PROFILER_TYPE = Symbol.for(\"react.profiler\");\n    Symbol.for(\"react.provider\");\n    var REACT_CONSUMER_TYPE = Symbol.for(\"react.consumer\"),\n      REACT_CONTEXT_TYPE = Symbol.for(\"react.context\"),\n      REACT_FORWARD_REF_TYPE = Symbol.for(\"react.forward_ref\"),\n      REACT_SUSPENSE_TYPE = Symbol.for(\"react.suspense\"),\n      REACT_SUSPENSE_LIST_TYPE = Symbol.for(\"react.suspense_list\"),\n      REACT_MEMO_TYPE = Symbol.for(\"react.memo\"),\n      REACT_LAZY_TYPE = Symbol.for(\"react.lazy\"),\n      REACT_VIEW_TRANSITION_TYPE = Symbol.for(\"react.view_transition\"),\n      REACT_CLIENT_REFERENCE = Symbol.for(\"react.client.reference\");\n    exports.ContextConsumer = REACT_CONSUMER_TYPE;\n    exports.ContextProvider = REACT_CONTEXT_TYPE;\n    exports.Element = REACT_ELEMENT_TYPE;\n    exports.ForwardRef = REACT_FORWARD_REF_TYPE;\n    exports.Fragment = REACT_FRAGMENT_TYPE;\n    exports.Lazy = REACT_LAZY_TYPE;\n    exports.Memo = REACT_MEMO_TYPE;\n    exports.Portal = REACT_PORTAL_TYPE;\n    exports.Profiler = REACT_PROFILER_TYPE;\n    exports.StrictMode = REACT_STRICT_MODE_TYPE;\n    exports.Suspense = REACT_SUSPENSE_TYPE;\n    exports.SuspenseList = REACT_SUSPENSE_LIST_TYPE;\n    exports.isContextConsumer = function (object) {\n      return typeOf(object) === REACT_CONSUMER_TYPE;\n    };\n    exports.isContextProvider = function (object) {\n      return typeOf(object) === REACT_CONTEXT_TYPE;\n    };\n    exports.isElement = function (object) {\n      return (\n        \"object\" === typeof object &&\n        null !== object &&\n        object.$$typeof === REACT_ELEMENT_TYPE\n      );\n    };\n    exports.isForwardRef = function (object) {\n      return typeOf(object) === REACT_FORWARD_REF_TYPE;\n    };\n    exports.isFragment = function (object) {\n      return typeOf(object) === REACT_FRAGMENT_TYPE;\n    };\n    exports.isLazy = function (object) {\n      return typeOf(object) === REACT_LAZY_TYPE;\n    };\n    exports.isMemo = function (object) {\n      return typeOf(object) === REACT_MEMO_TYPE;\n    };\n    exports.isPortal = function (object) {\n      return typeOf(object) === REACT_PORTAL_TYPE;\n    };\n    exports.isProfiler = function (object) {\n      return typeOf(object) === REACT_PROFILER_TYPE;\n    };\n    exports.isStrictMode = function (object) {\n      return typeOf(object) === REACT_STRICT_MODE_TYPE;\n    };\n    exports.isSuspense = function (object) {\n      return typeOf(object) === REACT_SUSPENSE_TYPE;\n    };\n    exports.isSuspenseList = function (object) {\n      return typeOf(object) === REACT_SUSPENSE_LIST_TYPE;\n    };\n    exports.isValidElementType = function (type) {\n      return \"string\" === typeof type ||\n        \"function\" === typeof type ||\n        type === REACT_FRAGMENT_TYPE ||\n        type === REACT_PROFILER_TYPE ||\n        type === REACT_STRICT_MODE_TYPE ||\n        type === REACT_SUSPENSE_TYPE ||\n        type === REACT_SUSPENSE_LIST_TYPE ||\n        (\"object\" === typeof type &&\n          null !== type &&\n          (type.$$typeof === REACT_LAZY_TYPE ||\n            type.$$typeof === REACT_MEMO_TYPE ||\n            type.$$typeof === REACT_CONTEXT_TYPE ||\n            type.$$typeof === REACT_CONSUMER_TYPE ||\n            type.$$typeof === REACT_FORWARD_REF_TYPE ||\n            type.$$typeof === REACT_CLIENT_REFERENCE ||\n            void 0 !== type.getModuleId))\n        ? !0\n        : !1;\n    };\n    exports.typeOf = typeOf;\n  })();\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-is/cjs/react-is.development.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/Transition.js":
/*!***************************************************************!*\
  !*** ./node_modules/react-transition-group/esm/Transition.js ***!
  \***************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   ENTERED: () => (/* binding */ ENTERED),\n/* harmony export */   ENTERING: () => (/* binding */ ENTERING),\n/* harmony export */   EXITED: () => (/* binding */ EXITED),\n/* harmony export */   EXITING: () => (/* binding */ EXITING),\n/* harmony export */   UNMOUNTED: () => (/* binding */ UNMOUNTED),\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _babel_runtime_helpers_esm_objectWithoutPropertiesLoose__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @babel/runtime/helpers/esm/objectWithoutPropertiesLoose */ \"./node_modules/@babel/runtime/helpers/esm/objectWithoutPropertiesLoose.js\");\n/* harmony import */ var _babel_runtime_helpers_esm_inheritsLoose__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @babel/runtime/helpers/esm/inheritsLoose */ \"./node_modules/@babel/runtime/helpers/esm/inheritsLoose.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_7__);\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_2__);\n/* harmony import */ var react_dom__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! react-dom */ \"./node_modules/react-dom/index.js\");\n/* harmony import */ var _config__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./config */ \"./node_modules/react-transition-group/esm/config.js\");\n/* harmony import */ var _utils_PropTypes__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./utils/PropTypes */ \"./node_modules/react-transition-group/esm/utils/PropTypes.js\");\n/* harmony import */ var _TransitionGroupContext__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./TransitionGroupContext */ \"./node_modules/react-transition-group/esm/TransitionGroupContext.js\");\n/* harmony import */ var _utils_reflow__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./utils/reflow */ \"./node_modules/react-transition-group/esm/utils/reflow.js\");\n\n\n\n\n\n\n\n\n\nvar UNMOUNTED = 'unmounted';\nvar EXITED = 'exited';\nvar ENTERING = 'entering';\nvar ENTERED = 'entered';\nvar EXITING = 'exiting';\n/**\n * The Transition component lets you describe a transition from one component\n * state to another _over time_ with a simple declarative API. Most commonly\n * it's used to animate the mounting and unmounting of a component, but can also\n * be used to describe in-place transition states as well.\n *\n * ---\n *\n * **Note**: `Transition` is a platform-agnostic base component. If you're using\n * transitions in CSS, you'll probably want to use\n * [`CSSTransition`](https://reactcommunity.org/react-transition-group/css-transition)\n * instead. It inherits all the features of `Transition`, but contains\n * additional features necessary to play nice with CSS transitions (hence the\n * name of the component).\n *\n * ---\n *\n * By default the `Transition` component does not alter the behavior of the\n * component it renders, it only tracks \"enter\" and \"exit\" states for the\n * components. It's up to you to give meaning and effect to those states. For\n * example we can add styles to a component when it enters or exits:\n *\n * ```jsx\n * import { Transition } from 'react-transition-group';\n *\n * const duration = 300;\n *\n * const defaultStyle = {\n *   transition: `opacity ${duration}ms ease-in-out`,\n *   opacity: 0,\n * }\n *\n * const transitionStyles = {\n *   entering: { opacity: 1 },\n *   entered:  { opacity: 1 },\n *   exiting:  { opacity: 0 },\n *   exited:  { opacity: 0 },\n * };\n *\n * const Fade = ({ in: inProp }) => (\n *   <Transition in={inProp} timeout={duration}>\n *     {state => (\n *       <div style={{\n *         ...defaultStyle,\n *         ...transitionStyles[state]\n *       }}>\n *         I'm a fade Transition!\n *       </div>\n *     )}\n *   </Transition>\n * );\n * ```\n *\n * There are 4 main states a Transition can be in:\n *  - `'entering'`\n *  - `'entered'`\n *  - `'exiting'`\n *  - `'exited'`\n *\n * Transition state is toggled via the `in` prop. When `true` the component\n * begins the \"Enter\" stage. During this stage, the component will shift from\n * its current transition state, to `'entering'` for the duration of the\n * transition and then to the `'entered'` stage once it's complete. Let's take\n * the following example (we'll use the\n * [useState](https://reactjs.org/docs/hooks-reference.html#usestate) hook):\n *\n * ```jsx\n * function App() {\n *   const [inProp, setInProp] = useState(false);\n *   return (\n *     <div>\n *       <Transition in={inProp} timeout={500}>\n *         {state => (\n *           // ...\n *         )}\n *       </Transition>\n *       <button onClick={() => setInProp(true)}>\n *         Click to Enter\n *       </button>\n *     </div>\n *   );\n * }\n * ```\n *\n * When the button is clicked the component will shift to the `'entering'` state\n * and stay there for 500ms (the value of `timeout`) before it finally switches\n * to `'entered'`.\n *\n * When `in` is `false` the same thing happens except the state moves from\n * `'exiting'` to `'exited'`.\n */\n\nvar Transition = /*#__PURE__*/function (_React$Component) {\n  (0,_babel_runtime_helpers_esm_inheritsLoose__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(Transition, _React$Component);\n\n  function Transition(props, context) {\n    var _this;\n\n    _this = _React$Component.call(this, props, context) || this;\n    var parentGroup = context; // In the context of a TransitionGroup all enters are really appears\n\n    var appear = parentGroup && !parentGroup.isMounting ? props.enter : props.appear;\n    var initialStatus;\n    _this.appearStatus = null;\n\n    if (props.in) {\n      if (appear) {\n        initialStatus = EXITED;\n        _this.appearStatus = ENTERING;\n      } else {\n        initialStatus = ENTERED;\n      }\n    } else {\n      if (props.unmountOnExit || props.mountOnEnter) {\n        initialStatus = UNMOUNTED;\n      } else {\n        initialStatus = EXITED;\n      }\n    }\n\n    _this.state = {\n      status: initialStatus\n    };\n    _this.nextCallback = null;\n    return _this;\n  }\n\n  Transition.getDerivedStateFromProps = function getDerivedStateFromProps(_ref, prevState) {\n    var nextIn = _ref.in;\n\n    if (nextIn && prevState.status === UNMOUNTED) {\n      return {\n        status: EXITED\n      };\n    }\n\n    return null;\n  } // getSnapshotBeforeUpdate(prevProps) {\n  //   let nextStatus = null\n  //   if (prevProps !== this.props) {\n  //     const { status } = this.state\n  //     if (this.props.in) {\n  //       if (status !== ENTERING && status !== ENTERED) {\n  //         nextStatus = ENTERING\n  //       }\n  //     } else {\n  //       if (status === ENTERING || status === ENTERED) {\n  //         nextStatus = EXITING\n  //       }\n  //     }\n  //   }\n  //   return { nextStatus }\n  // }\n  ;\n\n  var _proto = Transition.prototype;\n\n  _proto.componentDidMount = function componentDidMount() {\n    this.updateStatus(true, this.appearStatus);\n  };\n\n  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {\n    var nextStatus = null;\n\n    if (prevProps !== this.props) {\n      var status = this.state.status;\n\n      if (this.props.in) {\n        if (status !== ENTERING && status !== ENTERED) {\n          nextStatus = ENTERING;\n        }\n      } else {\n        if (status === ENTERING || status === ENTERED) {\n          nextStatus = EXITING;\n        }\n      }\n    }\n\n    this.updateStatus(false, nextStatus);\n  };\n\n  _proto.componentWillUnmount = function componentWillUnmount() {\n    this.cancelNextCallback();\n  };\n\n  _proto.getTimeouts = function getTimeouts() {\n    var timeout = this.props.timeout;\n    var exit, enter, appear;\n    exit = enter = appear = timeout;\n\n    if (timeout != null && typeof timeout !== 'number') {\n      exit = timeout.exit;\n      enter = timeout.enter; // TODO: remove fallback for next major\n\n      appear = timeout.appear !== undefined ? timeout.appear : enter;\n    }\n\n    return {\n      exit: exit,\n      enter: enter,\n      appear: appear\n    };\n  };\n\n  _proto.updateStatus = function updateStatus(mounting, nextStatus) {\n    if (mounting === void 0) {\n      mounting = false;\n    }\n\n    if (nextStatus !== null) {\n      // nextStatus will always be ENTERING or EXITING.\n      this.cancelNextCallback();\n\n      if (nextStatus === ENTERING) {\n        if (this.props.unmountOnExit || this.props.mountOnEnter) {\n          var node = this.props.nodeRef ? this.props.nodeRef.current : react_dom__WEBPACK_IMPORTED_MODULE_3__.findDOMNode(this); // https://github.com/reactjs/react-transition-group/pull/749\n          // With unmountOnExit or mountOnEnter, the enter animation should happen at the transition between `exited` and `entering`.\n          // To make the animation happen,  we have to separate each rendering and avoid being processed as batched.\n\n          if (node) (0,_utils_reflow__WEBPACK_IMPORTED_MODULE_4__.forceReflow)(node);\n        }\n\n        this.performEnter(mounting);\n      } else {\n        this.performExit();\n      }\n    } else if (this.props.unmountOnExit && this.state.status === EXITED) {\n      this.setState({\n        status: UNMOUNTED\n      });\n    }\n  };\n\n  _proto.performEnter = function performEnter(mounting) {\n    var _this2 = this;\n\n    var enter = this.props.enter;\n    var appearing = this.context ? this.context.isMounting : mounting;\n\n    var _ref2 = this.props.nodeRef ? [appearing] : [react_dom__WEBPACK_IMPORTED_MODULE_3__.findDOMNode(this), appearing],\n        maybeNode = _ref2[0],\n        maybeAppearing = _ref2[1];\n\n    var timeouts = this.getTimeouts();\n    var enterTimeout = appearing ? timeouts.appear : timeouts.enter; // no enter animation skip right to ENTERED\n    // if we are mounting and running this it means appear _must_ be set\n\n    if (!mounting && !enter || _config__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled) {\n      this.safeSetState({\n        status: ENTERED\n      }, function () {\n        _this2.props.onEntered(maybeNode);\n      });\n      return;\n    }\n\n    this.props.onEnter(maybeNode, maybeAppearing);\n    this.safeSetState({\n      status: ENTERING\n    }, function () {\n      _this2.props.onEntering(maybeNode, maybeAppearing);\n\n      _this2.onTransitionEnd(enterTimeout, function () {\n        _this2.safeSetState({\n          status: ENTERED\n        }, function () {\n          _this2.props.onEntered(maybeNode, maybeAppearing);\n        });\n      });\n    });\n  };\n\n  _proto.performExit = function performExit() {\n    var _this3 = this;\n\n    var exit = this.props.exit;\n    var timeouts = this.getTimeouts();\n    var maybeNode = this.props.nodeRef ? undefined : react_dom__WEBPACK_IMPORTED_MODULE_3__.findDOMNode(this); // no exit animation skip right to EXITED\n\n    if (!exit || _config__WEBPACK_IMPORTED_MODULE_5__[\"default\"].disabled) {\n      this.safeSetState({\n        status: EXITED\n      }, function () {\n        _this3.props.onExited(maybeNode);\n      });\n      return;\n    }\n\n    this.props.onExit(maybeNode);\n    this.safeSetState({\n      status: EXITING\n    }, function () {\n      _this3.props.onExiting(maybeNode);\n\n      _this3.onTransitionEnd(timeouts.exit, function () {\n        _this3.safeSetState({\n          status: EXITED\n        }, function () {\n          _this3.props.onExited(maybeNode);\n        });\n      });\n    });\n  };\n\n  _proto.cancelNextCallback = function cancelNextCallback() {\n    if (this.nextCallback !== null) {\n      this.nextCallback.cancel();\n      this.nextCallback = null;\n    }\n  };\n\n  _proto.safeSetState = function safeSetState(nextState, callback) {\n    // This shouldn't be necessary, but there are weird race conditions with\n    // setState callbacks and unmounting in testing, so always make sure that\n    // we can cancel any pending setState callbacks after we unmount.\n    callback = this.setNextCallback(callback);\n    this.setState(nextState, callback);\n  };\n\n  _proto.setNextCallback = function setNextCallback(callback) {\n    var _this4 = this;\n\n    var active = true;\n\n    this.nextCallback = function (event) {\n      if (active) {\n        active = false;\n        _this4.nextCallback = null;\n        callback(event);\n      }\n    };\n\n    this.nextCallback.cancel = function () {\n      active = false;\n    };\n\n    return this.nextCallback;\n  };\n\n  _proto.onTransitionEnd = function onTransitionEnd(timeout, handler) {\n    this.setNextCallback(handler);\n    var node = this.props.nodeRef ? this.props.nodeRef.current : react_dom__WEBPACK_IMPORTED_MODULE_3__.findDOMNode(this);\n    var doesNotHaveTimeoutOrListener = timeout == null && !this.props.addEndListener;\n\n    if (!node || doesNotHaveTimeoutOrListener) {\n      setTimeout(this.nextCallback, 0);\n      return;\n    }\n\n    if (this.props.addEndListener) {\n      var _ref3 = this.props.nodeRef ? [this.nextCallback] : [node, this.nextCallback],\n          maybeNode = _ref3[0],\n          maybeNextCallback = _ref3[1];\n\n      this.props.addEndListener(maybeNode, maybeNextCallback);\n    }\n\n    if (timeout != null) {\n      setTimeout(this.nextCallback, timeout);\n    }\n  };\n\n  _proto.render = function render() {\n    var status = this.state.status;\n\n    if (status === UNMOUNTED) {\n      return null;\n    }\n\n    var _this$props = this.props,\n        children = _this$props.children,\n        _in = _this$props.in,\n        _mountOnEnter = _this$props.mountOnEnter,\n        _unmountOnExit = _this$props.unmountOnExit,\n        _appear = _this$props.appear,\n        _enter = _this$props.enter,\n        _exit = _this$props.exit,\n        _timeout = _this$props.timeout,\n        _addEndListener = _this$props.addEndListener,\n        _onEnter = _this$props.onEnter,\n        _onEntering = _this$props.onEntering,\n        _onEntered = _this$props.onEntered,\n        _onExit = _this$props.onExit,\n        _onExiting = _this$props.onExiting,\n        _onExited = _this$props.onExited,\n        _nodeRef = _this$props.nodeRef,\n        childProps = (0,_babel_runtime_helpers_esm_objectWithoutPropertiesLoose__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(_this$props, [\"children\", \"in\", \"mountOnEnter\", \"unmountOnExit\", \"appear\", \"enter\", \"exit\", \"timeout\", \"addEndListener\", \"onEnter\", \"onEntering\", \"onEntered\", \"onExit\", \"onExiting\", \"onExited\", \"nodeRef\"]);\n\n    return (\n      /*#__PURE__*/\n      // allows for nested Transitions\n      react__WEBPACK_IMPORTED_MODULE_2___default().createElement(_TransitionGroupContext__WEBPACK_IMPORTED_MODULE_6__[\"default\"].Provider, {\n        value: null\n      }, typeof children === 'function' ? children(status, childProps) : react__WEBPACK_IMPORTED_MODULE_2___default().cloneElement(react__WEBPACK_IMPORTED_MODULE_2___default().Children.only(children), childProps))\n    );\n  };\n\n  return Transition;\n}((react__WEBPACK_IMPORTED_MODULE_2___default().Component));\n\nTransition.contextType = _TransitionGroupContext__WEBPACK_IMPORTED_MODULE_6__[\"default\"];\nTransition.propTypes =  true ? {\n  /**\n   * A React reference to DOM element that need to transition:\n   * https://stackoverflow.com/a/51127130/4671932\n   *\n   *   - When `nodeRef` prop is used, `node` is not passed to callback functions\n   *      (e.g. `onEnter`) because user already has direct access to the node.\n   *   - When changing `key` prop of `Transition` in a `TransitionGroup` a new\n   *     `nodeRef` need to be provided to `Transition` with changed `key` prop\n   *     (see\n   *     [test/CSSTransition-test.js](https://github.com/reactjs/react-transition-group/blob/13435f897b3ab71f6e19d724f145596f5910581c/test/CSSTransition-test.js#L362-L437)).\n   */\n  nodeRef: prop_types__WEBPACK_IMPORTED_MODULE_7___default().shape({\n    current: typeof Element === 'undefined' ? (prop_types__WEBPACK_IMPORTED_MODULE_7___default().any) : function (propValue, key, componentName, location, propFullName, secret) {\n      var value = propValue[key];\n      return prop_types__WEBPACK_IMPORTED_MODULE_7___default().instanceOf(value && 'ownerDocument' in value ? value.ownerDocument.defaultView.Element : Element)(propValue, key, componentName, location, propFullName, secret);\n    }\n  }),\n\n  /**\n   * A `function` child can be used instead of a React element. This function is\n   * called with the current transition status (`'entering'`, `'entered'`,\n   * `'exiting'`, `'exited'`), which can be used to apply context\n   * specific props to a component.\n   *\n   * ```jsx\n   * <Transition in={this.state.in} timeout={150}>\n   *   {state => (\n   *     <MyComponent className={`fade fade-${state}`} />\n   *   )}\n   * </Transition>\n   * ```\n   */\n  children: prop_types__WEBPACK_IMPORTED_MODULE_7___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_7___default().func).isRequired, (prop_types__WEBPACK_IMPORTED_MODULE_7___default().element).isRequired]).isRequired,\n\n  /**\n   * Show the component; triggers the enter or exit states\n   */\n  in: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * By default the child component is mounted immediately along with\n   * the parent `Transition` component. If you want to \"lazy mount\" the component on the\n   * first `in={true}` you can set `mountOnEnter`. After the first enter transition the component will stay\n   * mounted, even on \"exited\", unless you also specify `unmountOnExit`.\n   */\n  mountOnEnter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * By default the child component stays mounted after it reaches the `'exited'` state.\n   * Set `unmountOnExit` if you'd prefer to unmount the component after it finishes exiting.\n   */\n  unmountOnExit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * By default the child component does not perform the enter transition when\n   * it first mounts, regardless of the value of `in`. If you want this\n   * behavior, set both `appear` and `in` to `true`.\n   *\n   * > **Note**: there are no special appear states like `appearing`/`appeared`, this prop\n   * > only adds an additional enter transition. However, in the\n   * > `<CSSTransition>` component that first enter transition does result in\n   * > additional `.appear-*` classes, that way you can choose to style it\n   * > differently.\n   */\n  appear: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * Enable or disable enter transitions.\n   */\n  enter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * Enable or disable exit transitions.\n   */\n  exit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * The duration of the transition, in milliseconds.\n   * Required unless `addEndListener` is provided.\n   *\n   * You may specify a single timeout for all transitions:\n   *\n   * ```jsx\n   * timeout={500}\n   * ```\n   *\n   * or individually:\n   *\n   * ```jsx\n   * timeout={{\n   *  appear: 500,\n   *  enter: 300,\n   *  exit: 500,\n   * }}\n   * ```\n   *\n   * - `appear` defaults to the value of `enter`\n   * - `enter` defaults to `0`\n   * - `exit` defaults to `0`\n   *\n   * @type {number | { enter?: number, exit?: number, appear?: number }}\n   */\n  timeout: function timeout(props) {\n    var pt = _utils_PropTypes__WEBPACK_IMPORTED_MODULE_8__.timeoutsShape;\n    if (!props.addEndListener) pt = pt.isRequired;\n\n    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {\n      args[_key - 1] = arguments[_key];\n    }\n\n    return pt.apply(void 0, [props].concat(args));\n  },\n\n  /**\n   * Add a custom transition end trigger. Called with the transitioning\n   * DOM node and a `done` callback. Allows for more fine grained transition end\n   * logic. Timeouts are still used as a fallback if provided.\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed.\n   *\n   * ```jsx\n   * addEndListener={(node, done) => {\n   *   // use the css transitionend event to mark the finish of a transition\n   *   node.addEventListener('transitionend', done, false);\n   * }}\n   * ```\n   */\n  addEndListener: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n\n  /**\n   * Callback fired before the \"entering\" status is applied. An extra parameter\n   * `isAppearing` is supplied to indicate if the enter stage is occurring on the initial mount\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed.\n   *\n   * @type Function(node: HtmlElement, isAppearing: bool) -> void\n   */\n  onEnter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n\n  /**\n   * Callback fired after the \"entering\" status is applied. An extra parameter\n   * `isAppearing` is supplied to indicate if the enter stage is occurring on the initial mount\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed.\n   *\n   * @type Function(node: HtmlElement, isAppearing: bool)\n   */\n  onEntering: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n\n  /**\n   * Callback fired after the \"entered\" status is applied. An extra parameter\n   * `isAppearing` is supplied to indicate if the enter stage is occurring on the initial mount\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed.\n   *\n   * @type Function(node: HtmlElement, isAppearing: bool) -> void\n   */\n  onEntered: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n\n  /**\n   * Callback fired before the \"exiting\" status is applied.\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed.\n   *\n   * @type Function(node: HtmlElement) -> void\n   */\n  onExit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n\n  /**\n   * Callback fired after the \"exiting\" status is applied.\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed.\n   *\n   * @type Function(node: HtmlElement) -> void\n   */\n  onExiting: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func),\n\n  /**\n   * Callback fired after the \"exited\" status is applied.\n   *\n   * **Note**: when `nodeRef` prop is passed, `node` is not passed\n   *\n   * @type Function(node: HtmlElement) -> void\n   */\n  onExited: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func)\n} : 0; // Name the function so it is clearer in the documentation\n\nfunction noop() {}\n\nTransition.defaultProps = {\n  in: false,\n  mountOnEnter: false,\n  unmountOnExit: false,\n  appear: false,\n  enter: true,\n  exit: true,\n  onEnter: noop,\n  onEntering: noop,\n  onEntered: noop,\n  onExit: noop,\n  onExiting: noop,\n  onExited: noop\n};\nTransition.UNMOUNTED = UNMOUNTED;\nTransition.EXITED = EXITED;\nTransition.ENTERING = ENTERING;\nTransition.ENTERED = ENTERED;\nTransition.EXITING = EXITING;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (Transition);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/Transition.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/TransitionGroup.js":
/*!********************************************************************!*\
  !*** ./node_modules/react-transition-group/esm/TransitionGroup.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var _babel_runtime_helpers_esm_objectWithoutPropertiesLoose__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! @babel/runtime/helpers/esm/objectWithoutPropertiesLoose */ \"./node_modules/@babel/runtime/helpers/esm/objectWithoutPropertiesLoose.js\");\n/* harmony import */ var _babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @babel/runtime/helpers/esm/extends */ \"./node_modules/@babel/runtime/helpers/esm/extends.js\");\n/* harmony import */ var _babel_runtime_helpers_esm_assertThisInitialized__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @babel/runtime/helpers/esm/assertThisInitialized */ \"./node_modules/@babel/runtime/helpers/esm/assertThisInitialized.js\");\n/* harmony import */ var _babel_runtime_helpers_esm_inheritsLoose__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @babel/runtime/helpers/esm/inheritsLoose */ \"./node_modules/@babel/runtime/helpers/esm/inheritsLoose.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_7___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_7__);\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_4___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_4__);\n/* harmony import */ var _TransitionGroupContext__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ./TransitionGroupContext */ \"./node_modules/react-transition-group/esm/TransitionGroupContext.js\");\n/* harmony import */ var _utils_ChildMapping__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ./utils/ChildMapping */ \"./node_modules/react-transition-group/esm/utils/ChildMapping.js\");\n\n\n\n\n\n\n\n\n\nvar values = Object.values || function (obj) {\n  return Object.keys(obj).map(function (k) {\n    return obj[k];\n  });\n};\n\nvar defaultProps = {\n  component: 'div',\n  childFactory: function childFactory(child) {\n    return child;\n  }\n};\n/**\n * The `<TransitionGroup>` component manages a set of transition components\n * (`<Transition>` and `<CSSTransition>`) in a list. Like with the transition\n * components, `<TransitionGroup>` is a state machine for managing the mounting\n * and unmounting of components over time.\n *\n * Consider the example below. As items are removed or added to the TodoList the\n * `in` prop is toggled automatically by the `<TransitionGroup>`.\n *\n * Note that `<TransitionGroup>`  does not define any animation behavior!\n * Exactly _how_ a list item animates is up to the individual transition\n * component. This means you can mix and match animations across different list\n * items.\n */\n\nvar TransitionGroup = /*#__PURE__*/function (_React$Component) {\n  (0,_babel_runtime_helpers_esm_inheritsLoose__WEBPACK_IMPORTED_MODULE_3__[\"default\"])(TransitionGroup, _React$Component);\n\n  function TransitionGroup(props, context) {\n    var _this;\n\n    _this = _React$Component.call(this, props, context) || this;\n\n    var handleExited = _this.handleExited.bind((0,_babel_runtime_helpers_esm_assertThisInitialized__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(_this)); // Initial children should all be entering, dependent on appear\n\n\n    _this.state = {\n      contextValue: {\n        isMounting: true\n      },\n      handleExited: handleExited,\n      firstRender: true\n    };\n    return _this;\n  }\n\n  var _proto = TransitionGroup.prototype;\n\n  _proto.componentDidMount = function componentDidMount() {\n    this.mounted = true;\n    this.setState({\n      contextValue: {\n        isMounting: false\n      }\n    });\n  };\n\n  _proto.componentWillUnmount = function componentWillUnmount() {\n    this.mounted = false;\n  };\n\n  TransitionGroup.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps, _ref) {\n    var prevChildMapping = _ref.children,\n        handleExited = _ref.handleExited,\n        firstRender = _ref.firstRender;\n    return {\n      children: firstRender ? (0,_utils_ChildMapping__WEBPACK_IMPORTED_MODULE_5__.getInitialChildMapping)(nextProps, handleExited) : (0,_utils_ChildMapping__WEBPACK_IMPORTED_MODULE_5__.getNextChildMapping)(nextProps, prevChildMapping, handleExited),\n      firstRender: false\n    };\n  } // node is `undefined` when user provided `nodeRef` prop\n  ;\n\n  _proto.handleExited = function handleExited(child, node) {\n    var currentChildMapping = (0,_utils_ChildMapping__WEBPACK_IMPORTED_MODULE_5__.getChildMapping)(this.props.children);\n    if (child.key in currentChildMapping) return;\n\n    if (child.props.onExited) {\n      child.props.onExited(node);\n    }\n\n    if (this.mounted) {\n      this.setState(function (state) {\n        var children = (0,_babel_runtime_helpers_esm_extends__WEBPACK_IMPORTED_MODULE_1__[\"default\"])({}, state.children);\n\n        delete children[child.key];\n        return {\n          children: children\n        };\n      });\n    }\n  };\n\n  _proto.render = function render() {\n    var _this$props = this.props,\n        Component = _this$props.component,\n        childFactory = _this$props.childFactory,\n        props = (0,_babel_runtime_helpers_esm_objectWithoutPropertiesLoose__WEBPACK_IMPORTED_MODULE_0__[\"default\"])(_this$props, [\"component\", \"childFactory\"]);\n\n    var contextValue = this.state.contextValue;\n    var children = values(this.state.children).map(childFactory);\n    delete props.appear;\n    delete props.enter;\n    delete props.exit;\n\n    if (Component === null) {\n      return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_4___default().createElement(_TransitionGroupContext__WEBPACK_IMPORTED_MODULE_6__[\"default\"].Provider, {\n        value: contextValue\n      }, children);\n    }\n\n    return /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_4___default().createElement(_TransitionGroupContext__WEBPACK_IMPORTED_MODULE_6__[\"default\"].Provider, {\n      value: contextValue\n    }, /*#__PURE__*/react__WEBPACK_IMPORTED_MODULE_4___default().createElement(Component, props, children));\n  };\n\n  return TransitionGroup;\n}((react__WEBPACK_IMPORTED_MODULE_4___default().Component));\n\nTransitionGroup.propTypes =  true ? {\n  /**\n   * `<TransitionGroup>` renders a `<div>` by default. You can change this\n   * behavior by providing a `component` prop.\n   * If you use React v16+ and would like to avoid a wrapping `<div>` element\n   * you can pass in `component={null}`. This is useful if the wrapping div\n   * borks your css styles.\n   */\n  component: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().any),\n\n  /**\n   * A set of `<Transition>` components, that are toggled `in` and out as they\n   * leave. the `<TransitionGroup>` will inject specific transition props, so\n   * remember to spread them through if you are wrapping the `<Transition>` as\n   * with our `<Fade>` example.\n   *\n   * While this component is meant for multiple `Transition` or `CSSTransition`\n   * children, sometimes you may want to have a single transition child with\n   * content that you want to be transitioned out and in when you change it\n   * (e.g. routes, images etc.) In that case you can change the `key` prop of\n   * the transition child as you change its content, this will cause\n   * `TransitionGroup` to transition the child out and back in.\n   */\n  children: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().node),\n\n  /**\n   * A convenience prop that enables or disables appear animations\n   * for all children. Note that specifying this will override any defaults set\n   * on individual children Transitions.\n   */\n  appear: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * A convenience prop that enables or disables enter animations\n   * for all children. Note that specifying this will override any defaults set\n   * on individual children Transitions.\n   */\n  enter: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * A convenience prop that enables or disables exit animations\n   * for all children. Note that specifying this will override any defaults set\n   * on individual children Transitions.\n   */\n  exit: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().bool),\n\n  /**\n   * You may need to apply reactive updates to a child as it is exiting.\n   * This is generally done by using `cloneElement` however in the case of an exiting\n   * child the element has already been removed and not accessible to the consumer.\n   *\n   * If you do need to update a child as it leaves you can provide a `childFactory`\n   * to wrap every child, even the ones that are leaving.\n   *\n   * @type Function(child: ReactElement) -> ReactElement\n   */\n  childFactory: (prop_types__WEBPACK_IMPORTED_MODULE_7___default().func)\n} : 0;\nTransitionGroup.defaultProps = defaultProps;\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (TransitionGroup);\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/TransitionGroup.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/TransitionGroupContext.js":
/*!***************************************************************************!*\
  !*** ./node_modules/react-transition-group/esm/TransitionGroupContext.js ***!
  \***************************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (react__WEBPACK_IMPORTED_MODULE_0___default().createContext(null));\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/TransitionGroupContext.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/config.js":
/*!***********************************************************!*\
  !*** ./node_modules/react-transition-group/esm/config.js ***!
  \***********************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   \"default\": () => (__WEBPACK_DEFAULT_EXPORT__)\n/* harmony export */ });\n/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = ({\n  disabled: false\n});\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/config.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/utils/ChildMapping.js":
/*!***********************************************************************!*\
  !*** ./node_modules/react-transition-group/esm/utils/ChildMapping.js ***!
  \***********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   getChildMapping: () => (/* binding */ getChildMapping),\n/* harmony export */   getInitialChildMapping: () => (/* binding */ getInitialChildMapping),\n/* harmony export */   getNextChildMapping: () => (/* binding */ getNextChildMapping),\n/* harmony export */   mergeChildMappings: () => (/* binding */ mergeChildMappings)\n/* harmony export */ });\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_0__);\n\n/**\n * Given `this.props.children`, return an object mapping key to child.\n *\n * @param {*} children `this.props.children`\n * @return {object} Mapping of key to child\n */\n\nfunction getChildMapping(children, mapFn) {\n  var mapper = function mapper(child) {\n    return mapFn && (0,react__WEBPACK_IMPORTED_MODULE_0__.isValidElement)(child) ? mapFn(child) : child;\n  };\n\n  var result = Object.create(null);\n  if (children) react__WEBPACK_IMPORTED_MODULE_0__.Children.map(children, function (c) {\n    return c;\n  }).forEach(function (child) {\n    // run the map function here instead so that the key is the computed one\n    result[child.key] = mapper(child);\n  });\n  return result;\n}\n/**\n * When you're adding or removing children some may be added or removed in the\n * same render pass. We want to show *both* since we want to simultaneously\n * animate elements in and out. This function takes a previous set of keys\n * and a new set of keys and merges them with its best guess of the correct\n * ordering. In the future we may expose some of the utilities in\n * ReactMultiChild to make this easy, but for now React itself does not\n * directly have this concept of the union of prevChildren and nextChildren\n * so we implement it here.\n *\n * @param {object} prev prev children as returned from\n * `ReactTransitionChildMapping.getChildMapping()`.\n * @param {object} next next children as returned from\n * `ReactTransitionChildMapping.getChildMapping()`.\n * @return {object} a key set that contains all keys in `prev` and all keys\n * in `next` in a reasonable order.\n */\n\nfunction mergeChildMappings(prev, next) {\n  prev = prev || {};\n  next = next || {};\n\n  function getValueForKey(key) {\n    return key in next ? next[key] : prev[key];\n  } // For each key of `next`, the list of keys to insert before that key in\n  // the combined list\n\n\n  var nextKeysPending = Object.create(null);\n  var pendingKeys = [];\n\n  for (var prevKey in prev) {\n    if (prevKey in next) {\n      if (pendingKeys.length) {\n        nextKeysPending[prevKey] = pendingKeys;\n        pendingKeys = [];\n      }\n    } else {\n      pendingKeys.push(prevKey);\n    }\n  }\n\n  var i;\n  var childMapping = {};\n\n  for (var nextKey in next) {\n    if (nextKeysPending[nextKey]) {\n      for (i = 0; i < nextKeysPending[nextKey].length; i++) {\n        var pendingNextKey = nextKeysPending[nextKey][i];\n        childMapping[nextKeysPending[nextKey][i]] = getValueForKey(pendingNextKey);\n      }\n    }\n\n    childMapping[nextKey] = getValueForKey(nextKey);\n  } // Finally, add the keys which didn't appear before any key in `next`\n\n\n  for (i = 0; i < pendingKeys.length; i++) {\n    childMapping[pendingKeys[i]] = getValueForKey(pendingKeys[i]);\n  }\n\n  return childMapping;\n}\n\nfunction getProp(child, prop, props) {\n  return props[prop] != null ? props[prop] : child.props[prop];\n}\n\nfunction getInitialChildMapping(props, onExited) {\n  return getChildMapping(props.children, function (child) {\n    return (0,react__WEBPACK_IMPORTED_MODULE_0__.cloneElement)(child, {\n      onExited: onExited.bind(null, child),\n      in: true,\n      appear: getProp(child, 'appear', props),\n      enter: getProp(child, 'enter', props),\n      exit: getProp(child, 'exit', props)\n    });\n  });\n}\nfunction getNextChildMapping(nextProps, prevChildMapping, onExited) {\n  var nextChildMapping = getChildMapping(nextProps.children);\n  var children = mergeChildMappings(prevChildMapping, nextChildMapping);\n  Object.keys(children).forEach(function (key) {\n    var child = children[key];\n    if (!(0,react__WEBPACK_IMPORTED_MODULE_0__.isValidElement)(child)) return;\n    var hasPrev = (key in prevChildMapping);\n    var hasNext = (key in nextChildMapping);\n    var prevChild = prevChildMapping[key];\n    var isLeaving = (0,react__WEBPACK_IMPORTED_MODULE_0__.isValidElement)(prevChild) && !prevChild.props.in; // item is new (entering)\n\n    if (hasNext && (!hasPrev || isLeaving)) {\n      // console.log('entering', key)\n      children[key] = (0,react__WEBPACK_IMPORTED_MODULE_0__.cloneElement)(child, {\n        onExited: onExited.bind(null, child),\n        in: true,\n        exit: getProp(child, 'exit', nextProps),\n        enter: getProp(child, 'enter', nextProps)\n      });\n    } else if (!hasNext && hasPrev && !isLeaving) {\n      // item is old (exiting)\n      // console.log('leaving', key)\n      children[key] = (0,react__WEBPACK_IMPORTED_MODULE_0__.cloneElement)(child, {\n        in: false\n      });\n    } else if (hasNext && hasPrev && (0,react__WEBPACK_IMPORTED_MODULE_0__.isValidElement)(prevChild)) {\n      // item hasn't changed transition states\n      // copy over the last transition props;\n      // console.log('unchanged', key)\n      children[key] = (0,react__WEBPACK_IMPORTED_MODULE_0__.cloneElement)(child, {\n        onExited: onExited.bind(null, child),\n        in: prevChild.props.in,\n        exit: getProp(child, 'exit', nextProps),\n        enter: getProp(child, 'enter', nextProps)\n      });\n    }\n  });\n  return children;\n}\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/utils/ChildMapping.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/utils/PropTypes.js":
/*!********************************************************************!*\
  !*** ./node_modules/react-transition-group/esm/utils/PropTypes.js ***!
  \********************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   classNamesShape: () => (/* binding */ classNamesShape),\n/* harmony export */   timeoutsShape: () => (/* binding */ timeoutsShape)\n/* harmony export */ });\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! prop-types */ \"./node_modules/prop-types/index.js\");\n/* harmony import */ var prop_types__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(prop_types__WEBPACK_IMPORTED_MODULE_0__);\n\nvar timeoutsShape =  true ? prop_types__WEBPACK_IMPORTED_MODULE_0___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_0___default().number), prop_types__WEBPACK_IMPORTED_MODULE_0___default().shape({\n  enter: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().number),\n  exit: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().number),\n  appear: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().number)\n}).isRequired]) : 0;\nvar classNamesShape =  true ? prop_types__WEBPACK_IMPORTED_MODULE_0___default().oneOfType([(prop_types__WEBPACK_IMPORTED_MODULE_0___default().string), prop_types__WEBPACK_IMPORTED_MODULE_0___default().shape({\n  enter: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  exit: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  active: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string)\n}), prop_types__WEBPACK_IMPORTED_MODULE_0___default().shape({\n  enter: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  enterDone: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  enterActive: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  exit: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  exitDone: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string),\n  exitActive: (prop_types__WEBPACK_IMPORTED_MODULE_0___default().string)\n})]) : 0;\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/utils/PropTypes.js?");

/***/ }),

/***/ "./node_modules/react-transition-group/esm/utils/reflow.js":
/*!*****************************************************************!*\
  !*** ./node_modules/react-transition-group/esm/utils/reflow.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   forceReflow: () => (/* binding */ forceReflow)\n/* harmony export */ });\nvar forceReflow = function forceReflow(node) {\n  return node.scrollTop;\n};\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react-transition-group/esm/utils/reflow.js?");

/***/ }),

/***/ "./node_modules/react/cjs/react-jsx-runtime.development.js":
/*!*****************************************************************!*\
  !*** ./node_modules/react/cjs/react-jsx-runtime.development.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("/**\n * @license React\n * react-jsx-runtime.development.js\n *\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n true &&\n  (function () {\n    function getComponentNameFromType(type) {\n      if (null == type) return null;\n      if (\"function\" === typeof type)\n        return type.$$typeof === REACT_CLIENT_REFERENCE\n          ? null\n          : type.displayName || type.name || null;\n      if (\"string\" === typeof type) return type;\n      switch (type) {\n        case REACT_FRAGMENT_TYPE:\n          return \"Fragment\";\n        case REACT_PROFILER_TYPE:\n          return \"Profiler\";\n        case REACT_STRICT_MODE_TYPE:\n          return \"StrictMode\";\n        case REACT_SUSPENSE_TYPE:\n          return \"Suspense\";\n        case REACT_SUSPENSE_LIST_TYPE:\n          return \"SuspenseList\";\n        case REACT_ACTIVITY_TYPE:\n          return \"Activity\";\n      }\n      if (\"object\" === typeof type)\n        switch (\n          (\"number\" === typeof type.tag &&\n            console.error(\n              \"Received an unexpected object in getComponentNameFromType(). This is likely a bug in React. Please file an issue.\"\n            ),\n          type.$$typeof)\n        ) {\n          case REACT_PORTAL_TYPE:\n            return \"Portal\";\n          case REACT_CONTEXT_TYPE:\n            return (type.displayName || \"Context\") + \".Provider\";\n          case REACT_CONSUMER_TYPE:\n            return (type._context.displayName || \"Context\") + \".Consumer\";\n          case REACT_FORWARD_REF_TYPE:\n            var innerType = type.render;\n            type = type.displayName;\n            type ||\n              ((type = innerType.displayName || innerType.name || \"\"),\n              (type = \"\" !== type ? \"ForwardRef(\" + type + \")\" : \"ForwardRef\"));\n            return type;\n          case REACT_MEMO_TYPE:\n            return (\n              (innerType = type.displayName || null),\n              null !== innerType\n                ? innerType\n                : getComponentNameFromType(type.type) || \"Memo\"\n            );\n          case REACT_LAZY_TYPE:\n            innerType = type._payload;\n            type = type._init;\n            try {\n              return getComponentNameFromType(type(innerType));\n            } catch (x) {}\n        }\n      return null;\n    }\n    function testStringCoercion(value) {\n      return \"\" + value;\n    }\n    function checkKeyStringCoercion(value) {\n      try {\n        testStringCoercion(value);\n        var JSCompiler_inline_result = !1;\n      } catch (e) {\n        JSCompiler_inline_result = !0;\n      }\n      if (JSCompiler_inline_result) {\n        JSCompiler_inline_result = console;\n        var JSCompiler_temp_const = JSCompiler_inline_result.error;\n        var JSCompiler_inline_result$jscomp$0 =\n          (\"function\" === typeof Symbol &&\n            Symbol.toStringTag &&\n            value[Symbol.toStringTag]) ||\n          value.constructor.name ||\n          \"Object\";\n        JSCompiler_temp_const.call(\n          JSCompiler_inline_result,\n          \"The provided key is an unsupported type %s. This value must be coerced to a string before using it here.\",\n          JSCompiler_inline_result$jscomp$0\n        );\n        return testStringCoercion(value);\n      }\n    }\n    function getTaskName(type) {\n      if (type === REACT_FRAGMENT_TYPE) return \"<>\";\n      if (\n        \"object\" === typeof type &&\n        null !== type &&\n        type.$$typeof === REACT_LAZY_TYPE\n      )\n        return \"<...>\";\n      try {\n        var name = getComponentNameFromType(type);\n        return name ? \"<\" + name + \">\" : \"<...>\";\n      } catch (x) {\n        return \"<...>\";\n      }\n    }\n    function getOwner() {\n      var dispatcher = ReactSharedInternals.A;\n      return null === dispatcher ? null : dispatcher.getOwner();\n    }\n    function UnknownOwner() {\n      return Error(\"react-stack-top-frame\");\n    }\n    function hasValidKey(config) {\n      if (hasOwnProperty.call(config, \"key\")) {\n        var getter = Object.getOwnPropertyDescriptor(config, \"key\").get;\n        if (getter && getter.isReactWarning) return !1;\n      }\n      return void 0 !== config.key;\n    }\n    function defineKeyPropWarningGetter(props, displayName) {\n      function warnAboutAccessingKey() {\n        specialPropKeyWarningShown ||\n          ((specialPropKeyWarningShown = !0),\n          console.error(\n            \"%s: `key` is not a prop. Trying to access it will result in `undefined` being returned. If you need to access the same value within the child component, you should pass it as a different prop. (https://react.dev/link/special-props)\",\n            displayName\n          ));\n      }\n      warnAboutAccessingKey.isReactWarning = !0;\n      Object.defineProperty(props, \"key\", {\n        get: warnAboutAccessingKey,\n        configurable: !0\n      });\n    }\n    function elementRefGetterWithDeprecationWarning() {\n      var componentName = getComponentNameFromType(this.type);\n      didWarnAboutElementRef[componentName] ||\n        ((didWarnAboutElementRef[componentName] = !0),\n        console.error(\n          \"Accessing element.ref was removed in React 19. ref is now a regular prop. It will be removed from the JSX Element type in a future release.\"\n        ));\n      componentName = this.props.ref;\n      return void 0 !== componentName ? componentName : null;\n    }\n    function ReactElement(\n      type,\n      key,\n      self,\n      source,\n      owner,\n      props,\n      debugStack,\n      debugTask\n    ) {\n      self = props.ref;\n      type = {\n        $$typeof: REACT_ELEMENT_TYPE,\n        type: type,\n        key: key,\n        props: props,\n        _owner: owner\n      };\n      null !== (void 0 !== self ? self : null)\n        ? Object.defineProperty(type, \"ref\", {\n            enumerable: !1,\n            get: elementRefGetterWithDeprecationWarning\n          })\n        : Object.defineProperty(type, \"ref\", { enumerable: !1, value: null });\n      type._store = {};\n      Object.defineProperty(type._store, \"validated\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: 0\n      });\n      Object.defineProperty(type, \"_debugInfo\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: null\n      });\n      Object.defineProperty(type, \"_debugStack\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: debugStack\n      });\n      Object.defineProperty(type, \"_debugTask\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: debugTask\n      });\n      Object.freeze && (Object.freeze(type.props), Object.freeze(type));\n      return type;\n    }\n    function jsxDEVImpl(\n      type,\n      config,\n      maybeKey,\n      isStaticChildren,\n      source,\n      self,\n      debugStack,\n      debugTask\n    ) {\n      var children = config.children;\n      if (void 0 !== children)\n        if (isStaticChildren)\n          if (isArrayImpl(children)) {\n            for (\n              isStaticChildren = 0;\n              isStaticChildren < children.length;\n              isStaticChildren++\n            )\n              validateChildKeys(children[isStaticChildren]);\n            Object.freeze && Object.freeze(children);\n          } else\n            console.error(\n              \"React.jsx: Static children should always be an array. You are likely explicitly calling React.jsxs or React.jsxDEV. Use the Babel transform instead.\"\n            );\n        else validateChildKeys(children);\n      if (hasOwnProperty.call(config, \"key\")) {\n        children = getComponentNameFromType(type);\n        var keys = Object.keys(config).filter(function (k) {\n          return \"key\" !== k;\n        });\n        isStaticChildren =\n          0 < keys.length\n            ? \"{key: someKey, \" + keys.join(\": ..., \") + \": ...}\"\n            : \"{key: someKey}\";\n        didWarnAboutKeySpread[children + isStaticChildren] ||\n          ((keys =\n            0 < keys.length ? \"{\" + keys.join(\": ..., \") + \": ...}\" : \"{}\"),\n          console.error(\n            'A props object containing a \"key\" prop is being spread into JSX:\\n  let props = %s;\\n  <%s {...props} />\\nReact keys must be passed directly to JSX without using spread:\\n  let props = %s;\\n  <%s key={someKey} {...props} />',\n            isStaticChildren,\n            children,\n            keys,\n            children\n          ),\n          (didWarnAboutKeySpread[children + isStaticChildren] = !0));\n      }\n      children = null;\n      void 0 !== maybeKey &&\n        (checkKeyStringCoercion(maybeKey), (children = \"\" + maybeKey));\n      hasValidKey(config) &&\n        (checkKeyStringCoercion(config.key), (children = \"\" + config.key));\n      if (\"key\" in config) {\n        maybeKey = {};\n        for (var propName in config)\n          \"key\" !== propName && (maybeKey[propName] = config[propName]);\n      } else maybeKey = config;\n      children &&\n        defineKeyPropWarningGetter(\n          maybeKey,\n          \"function\" === typeof type\n            ? type.displayName || type.name || \"Unknown\"\n            : type\n        );\n      return ReactElement(\n        type,\n        children,\n        self,\n        source,\n        getOwner(),\n        maybeKey,\n        debugStack,\n        debugTask\n      );\n    }\n    function validateChildKeys(node) {\n      \"object\" === typeof node &&\n        null !== node &&\n        node.$$typeof === REACT_ELEMENT_TYPE &&\n        node._store &&\n        (node._store.validated = 1);\n    }\n    var React = __webpack_require__(/*! react */ \"./node_modules/react/index.js\"),\n      REACT_ELEMENT_TYPE = Symbol.for(\"react.transitional.element\"),\n      REACT_PORTAL_TYPE = Symbol.for(\"react.portal\"),\n      REACT_FRAGMENT_TYPE = Symbol.for(\"react.fragment\"),\n      REACT_STRICT_MODE_TYPE = Symbol.for(\"react.strict_mode\"),\n      REACT_PROFILER_TYPE = Symbol.for(\"react.profiler\");\n    Symbol.for(\"react.provider\");\n    var REACT_CONSUMER_TYPE = Symbol.for(\"react.consumer\"),\n      REACT_CONTEXT_TYPE = Symbol.for(\"react.context\"),\n      REACT_FORWARD_REF_TYPE = Symbol.for(\"react.forward_ref\"),\n      REACT_SUSPENSE_TYPE = Symbol.for(\"react.suspense\"),\n      REACT_SUSPENSE_LIST_TYPE = Symbol.for(\"react.suspense_list\"),\n      REACT_MEMO_TYPE = Symbol.for(\"react.memo\"),\n      REACT_LAZY_TYPE = Symbol.for(\"react.lazy\"),\n      REACT_ACTIVITY_TYPE = Symbol.for(\"react.activity\"),\n      REACT_CLIENT_REFERENCE = Symbol.for(\"react.client.reference\"),\n      ReactSharedInternals =\n        React.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,\n      hasOwnProperty = Object.prototype.hasOwnProperty,\n      isArrayImpl = Array.isArray,\n      createTask = console.createTask\n        ? console.createTask\n        : function () {\n            return null;\n          };\n    React = {\n      \"react-stack-bottom-frame\": function (callStackForError) {\n        return callStackForError();\n      }\n    };\n    var specialPropKeyWarningShown;\n    var didWarnAboutElementRef = {};\n    var unknownOwnerDebugStack = React[\"react-stack-bottom-frame\"].bind(\n      React,\n      UnknownOwner\n    )();\n    var unknownOwnerDebugTask = createTask(getTaskName(UnknownOwner));\n    var didWarnAboutKeySpread = {};\n    exports.Fragment = REACT_FRAGMENT_TYPE;\n    exports.jsx = function (type, config, maybeKey, source, self) {\n      var trackActualOwner =\n        1e4 > ReactSharedInternals.recentlyCreatedOwnerStacks++;\n      return jsxDEVImpl(\n        type,\n        config,\n        maybeKey,\n        !1,\n        source,\n        self,\n        trackActualOwner\n          ? Error(\"react-stack-top-frame\")\n          : unknownOwnerDebugStack,\n        trackActualOwner ? createTask(getTaskName(type)) : unknownOwnerDebugTask\n      );\n    };\n    exports.jsxs = function (type, config, maybeKey, source, self) {\n      var trackActualOwner =\n        1e4 > ReactSharedInternals.recentlyCreatedOwnerStacks++;\n      return jsxDEVImpl(\n        type,\n        config,\n        maybeKey,\n        !0,\n        source,\n        self,\n        trackActualOwner\n          ? Error(\"react-stack-top-frame\")\n          : unknownOwnerDebugStack,\n        trackActualOwner ? createTask(getTaskName(type)) : unknownOwnerDebugTask\n      );\n    };\n  })();\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react/cjs/react-jsx-runtime.development.js?");

/***/ }),

/***/ "./node_modules/react/cjs/react.development.js":
/*!*****************************************************!*\
  !*** ./node_modules/react/cjs/react.development.js ***!
  \*****************************************************/
/***/ ((module, exports, __webpack_require__) => {

eval("/* module decorator */ module = __webpack_require__.nmd(module);\n/**\n * @license React\n * react.development.js\n *\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n true &&\n  (function () {\n    function defineDeprecationWarning(methodName, info) {\n      Object.defineProperty(Component.prototype, methodName, {\n        get: function () {\n          console.warn(\n            \"%s(...) is deprecated in plain JavaScript React classes. %s\",\n            info[0],\n            info[1]\n          );\n        }\n      });\n    }\n    function getIteratorFn(maybeIterable) {\n      if (null === maybeIterable || \"object\" !== typeof maybeIterable)\n        return null;\n      maybeIterable =\n        (MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL]) ||\n        maybeIterable[\"@@iterator\"];\n      return \"function\" === typeof maybeIterable ? maybeIterable : null;\n    }\n    function warnNoop(publicInstance, callerName) {\n      publicInstance =\n        ((publicInstance = publicInstance.constructor) &&\n          (publicInstance.displayName || publicInstance.name)) ||\n        \"ReactClass\";\n      var warningKey = publicInstance + \".\" + callerName;\n      didWarnStateUpdateForUnmountedComponent[warningKey] ||\n        (console.error(\n          \"Can't call %s on a component that is not yet mounted. This is a no-op, but it might indicate a bug in your application. Instead, assign to `this.state` directly or define a `state = {};` class property with the desired state in the %s component.\",\n          callerName,\n          publicInstance\n        ),\n        (didWarnStateUpdateForUnmountedComponent[warningKey] = !0));\n    }\n    function Component(props, context, updater) {\n      this.props = props;\n      this.context = context;\n      this.refs = emptyObject;\n      this.updater = updater || ReactNoopUpdateQueue;\n    }\n    function ComponentDummy() {}\n    function PureComponent(props, context, updater) {\n      this.props = props;\n      this.context = context;\n      this.refs = emptyObject;\n      this.updater = updater || ReactNoopUpdateQueue;\n    }\n    function testStringCoercion(value) {\n      return \"\" + value;\n    }\n    function checkKeyStringCoercion(value) {\n      try {\n        testStringCoercion(value);\n        var JSCompiler_inline_result = !1;\n      } catch (e) {\n        JSCompiler_inline_result = !0;\n      }\n      if (JSCompiler_inline_result) {\n        JSCompiler_inline_result = console;\n        var JSCompiler_temp_const = JSCompiler_inline_result.error;\n        var JSCompiler_inline_result$jscomp$0 =\n          (\"function\" === typeof Symbol &&\n            Symbol.toStringTag &&\n            value[Symbol.toStringTag]) ||\n          value.constructor.name ||\n          \"Object\";\n        JSCompiler_temp_const.call(\n          JSCompiler_inline_result,\n          \"The provided key is an unsupported type %s. This value must be coerced to a string before using it here.\",\n          JSCompiler_inline_result$jscomp$0\n        );\n        return testStringCoercion(value);\n      }\n    }\n    function getComponentNameFromType(type) {\n      if (null == type) return null;\n      if (\"function\" === typeof type)\n        return type.$$typeof === REACT_CLIENT_REFERENCE\n          ? null\n          : type.displayName || type.name || null;\n      if (\"string\" === typeof type) return type;\n      switch (type) {\n        case REACT_FRAGMENT_TYPE:\n          return \"Fragment\";\n        case REACT_PROFILER_TYPE:\n          return \"Profiler\";\n        case REACT_STRICT_MODE_TYPE:\n          return \"StrictMode\";\n        case REACT_SUSPENSE_TYPE:\n          return \"Suspense\";\n        case REACT_SUSPENSE_LIST_TYPE:\n          return \"SuspenseList\";\n        case REACT_ACTIVITY_TYPE:\n          return \"Activity\";\n      }\n      if (\"object\" === typeof type)\n        switch (\n          (\"number\" === typeof type.tag &&\n            console.error(\n              \"Received an unexpected object in getComponentNameFromType(). This is likely a bug in React. Please file an issue.\"\n            ),\n          type.$$typeof)\n        ) {\n          case REACT_PORTAL_TYPE:\n            return \"Portal\";\n          case REACT_CONTEXT_TYPE:\n            return (type.displayName || \"Context\") + \".Provider\";\n          case REACT_CONSUMER_TYPE:\n            return (type._context.displayName || \"Context\") + \".Consumer\";\n          case REACT_FORWARD_REF_TYPE:\n            var innerType = type.render;\n            type = type.displayName;\n            type ||\n              ((type = innerType.displayName || innerType.name || \"\"),\n              (type = \"\" !== type ? \"ForwardRef(\" + type + \")\" : \"ForwardRef\"));\n            return type;\n          case REACT_MEMO_TYPE:\n            return (\n              (innerType = type.displayName || null),\n              null !== innerType\n                ? innerType\n                : getComponentNameFromType(type.type) || \"Memo\"\n            );\n          case REACT_LAZY_TYPE:\n            innerType = type._payload;\n            type = type._init;\n            try {\n              return getComponentNameFromType(type(innerType));\n            } catch (x) {}\n        }\n      return null;\n    }\n    function getTaskName(type) {\n      if (type === REACT_FRAGMENT_TYPE) return \"<>\";\n      if (\n        \"object\" === typeof type &&\n        null !== type &&\n        type.$$typeof === REACT_LAZY_TYPE\n      )\n        return \"<...>\";\n      try {\n        var name = getComponentNameFromType(type);\n        return name ? \"<\" + name + \">\" : \"<...>\";\n      } catch (x) {\n        return \"<...>\";\n      }\n    }\n    function getOwner() {\n      var dispatcher = ReactSharedInternals.A;\n      return null === dispatcher ? null : dispatcher.getOwner();\n    }\n    function UnknownOwner() {\n      return Error(\"react-stack-top-frame\");\n    }\n    function hasValidKey(config) {\n      if (hasOwnProperty.call(config, \"key\")) {\n        var getter = Object.getOwnPropertyDescriptor(config, \"key\").get;\n        if (getter && getter.isReactWarning) return !1;\n      }\n      return void 0 !== config.key;\n    }\n    function defineKeyPropWarningGetter(props, displayName) {\n      function warnAboutAccessingKey() {\n        specialPropKeyWarningShown ||\n          ((specialPropKeyWarningShown = !0),\n          console.error(\n            \"%s: `key` is not a prop. Trying to access it will result in `undefined` being returned. If you need to access the same value within the child component, you should pass it as a different prop. (https://react.dev/link/special-props)\",\n            displayName\n          ));\n      }\n      warnAboutAccessingKey.isReactWarning = !0;\n      Object.defineProperty(props, \"key\", {\n        get: warnAboutAccessingKey,\n        configurable: !0\n      });\n    }\n    function elementRefGetterWithDeprecationWarning() {\n      var componentName = getComponentNameFromType(this.type);\n      didWarnAboutElementRef[componentName] ||\n        ((didWarnAboutElementRef[componentName] = !0),\n        console.error(\n          \"Accessing element.ref was removed in React 19. ref is now a regular prop. It will be removed from the JSX Element type in a future release.\"\n        ));\n      componentName = this.props.ref;\n      return void 0 !== componentName ? componentName : null;\n    }\n    function ReactElement(\n      type,\n      key,\n      self,\n      source,\n      owner,\n      props,\n      debugStack,\n      debugTask\n    ) {\n      self = props.ref;\n      type = {\n        $$typeof: REACT_ELEMENT_TYPE,\n        type: type,\n        key: key,\n        props: props,\n        _owner: owner\n      };\n      null !== (void 0 !== self ? self : null)\n        ? Object.defineProperty(type, \"ref\", {\n            enumerable: !1,\n            get: elementRefGetterWithDeprecationWarning\n          })\n        : Object.defineProperty(type, \"ref\", { enumerable: !1, value: null });\n      type._store = {};\n      Object.defineProperty(type._store, \"validated\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: 0\n      });\n      Object.defineProperty(type, \"_debugInfo\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: null\n      });\n      Object.defineProperty(type, \"_debugStack\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: debugStack\n      });\n      Object.defineProperty(type, \"_debugTask\", {\n        configurable: !1,\n        enumerable: !1,\n        writable: !0,\n        value: debugTask\n      });\n      Object.freeze && (Object.freeze(type.props), Object.freeze(type));\n      return type;\n    }\n    function cloneAndReplaceKey(oldElement, newKey) {\n      newKey = ReactElement(\n        oldElement.type,\n        newKey,\n        void 0,\n        void 0,\n        oldElement._owner,\n        oldElement.props,\n        oldElement._debugStack,\n        oldElement._debugTask\n      );\n      oldElement._store &&\n        (newKey._store.validated = oldElement._store.validated);\n      return newKey;\n    }\n    function isValidElement(object) {\n      return (\n        \"object\" === typeof object &&\n        null !== object &&\n        object.$$typeof === REACT_ELEMENT_TYPE\n      );\n    }\n    function escape(key) {\n      var escaperLookup = { \"=\": \"=0\", \":\": \"=2\" };\n      return (\n        \"$\" +\n        key.replace(/[=:]/g, function (match) {\n          return escaperLookup[match];\n        })\n      );\n    }\n    function getElementKey(element, index) {\n      return \"object\" === typeof element &&\n        null !== element &&\n        null != element.key\n        ? (checkKeyStringCoercion(element.key), escape(\"\" + element.key))\n        : index.toString(36);\n    }\n    function noop$1() {}\n    function resolveThenable(thenable) {\n      switch (thenable.status) {\n        case \"fulfilled\":\n          return thenable.value;\n        case \"rejected\":\n          throw thenable.reason;\n        default:\n          switch (\n            (\"string\" === typeof thenable.status\n              ? thenable.then(noop$1, noop$1)\n              : ((thenable.status = \"pending\"),\n                thenable.then(\n                  function (fulfilledValue) {\n                    \"pending\" === thenable.status &&\n                      ((thenable.status = \"fulfilled\"),\n                      (thenable.value = fulfilledValue));\n                  },\n                  function (error) {\n                    \"pending\" === thenable.status &&\n                      ((thenable.status = \"rejected\"),\n                      (thenable.reason = error));\n                  }\n                )),\n            thenable.status)\n          ) {\n            case \"fulfilled\":\n              return thenable.value;\n            case \"rejected\":\n              throw thenable.reason;\n          }\n      }\n      throw thenable;\n    }\n    function mapIntoArray(children, array, escapedPrefix, nameSoFar, callback) {\n      var type = typeof children;\n      if (\"undefined\" === type || \"boolean\" === type) children = null;\n      var invokeCallback = !1;\n      if (null === children) invokeCallback = !0;\n      else\n        switch (type) {\n          case \"bigint\":\n          case \"string\":\n          case \"number\":\n            invokeCallback = !0;\n            break;\n          case \"object\":\n            switch (children.$$typeof) {\n              case REACT_ELEMENT_TYPE:\n              case REACT_PORTAL_TYPE:\n                invokeCallback = !0;\n                break;\n              case REACT_LAZY_TYPE:\n                return (\n                  (invokeCallback = children._init),\n                  mapIntoArray(\n                    invokeCallback(children._payload),\n                    array,\n                    escapedPrefix,\n                    nameSoFar,\n                    callback\n                  )\n                );\n            }\n        }\n      if (invokeCallback) {\n        invokeCallback = children;\n        callback = callback(invokeCallback);\n        var childKey =\n          \"\" === nameSoFar ? \".\" + getElementKey(invokeCallback, 0) : nameSoFar;\n        isArrayImpl(callback)\n          ? ((escapedPrefix = \"\"),\n            null != childKey &&\n              (escapedPrefix =\n                childKey.replace(userProvidedKeyEscapeRegex, \"$&/\") + \"/\"),\n            mapIntoArray(callback, array, escapedPrefix, \"\", function (c) {\n              return c;\n            }))\n          : null != callback &&\n            (isValidElement(callback) &&\n              (null != callback.key &&\n                ((invokeCallback && invokeCallback.key === callback.key) ||\n                  checkKeyStringCoercion(callback.key)),\n              (escapedPrefix = cloneAndReplaceKey(\n                callback,\n                escapedPrefix +\n                  (null == callback.key ||\n                  (invokeCallback && invokeCallback.key === callback.key)\n                    ? \"\"\n                    : (\"\" + callback.key).replace(\n                        userProvidedKeyEscapeRegex,\n                        \"$&/\"\n                      ) + \"/\") +\n                  childKey\n              )),\n              \"\" !== nameSoFar &&\n                null != invokeCallback &&\n                isValidElement(invokeCallback) &&\n                null == invokeCallback.key &&\n                invokeCallback._store &&\n                !invokeCallback._store.validated &&\n                (escapedPrefix._store.validated = 2),\n              (callback = escapedPrefix)),\n            array.push(callback));\n        return 1;\n      }\n      invokeCallback = 0;\n      childKey = \"\" === nameSoFar ? \".\" : nameSoFar + \":\";\n      if (isArrayImpl(children))\n        for (var i = 0; i < children.length; i++)\n          (nameSoFar = children[i]),\n            (type = childKey + getElementKey(nameSoFar, i)),\n            (invokeCallback += mapIntoArray(\n              nameSoFar,\n              array,\n              escapedPrefix,\n              type,\n              callback\n            ));\n      else if (((i = getIteratorFn(children)), \"function\" === typeof i))\n        for (\n          i === children.entries &&\n            (didWarnAboutMaps ||\n              console.warn(\n                \"Using Maps as children is not supported. Use an array of keyed ReactElements instead.\"\n              ),\n            (didWarnAboutMaps = !0)),\n            children = i.call(children),\n            i = 0;\n          !(nameSoFar = children.next()).done;\n\n        )\n          (nameSoFar = nameSoFar.value),\n            (type = childKey + getElementKey(nameSoFar, i++)),\n            (invokeCallback += mapIntoArray(\n              nameSoFar,\n              array,\n              escapedPrefix,\n              type,\n              callback\n            ));\n      else if (\"object\" === type) {\n        if (\"function\" === typeof children.then)\n          return mapIntoArray(\n            resolveThenable(children),\n            array,\n            escapedPrefix,\n            nameSoFar,\n            callback\n          );\n        array = String(children);\n        throw Error(\n          \"Objects are not valid as a React child (found: \" +\n            (\"[object Object]\" === array\n              ? \"object with keys {\" + Object.keys(children).join(\", \") + \"}\"\n              : array) +\n            \"). If you meant to render a collection of children, use an array instead.\"\n        );\n      }\n      return invokeCallback;\n    }\n    function mapChildren(children, func, context) {\n      if (null == children) return children;\n      var result = [],\n        count = 0;\n      mapIntoArray(children, result, \"\", \"\", function (child) {\n        return func.call(context, child, count++);\n      });\n      return result;\n    }\n    function lazyInitializer(payload) {\n      if (-1 === payload._status) {\n        var ctor = payload._result;\n        ctor = ctor();\n        ctor.then(\n          function (moduleObject) {\n            if (0 === payload._status || -1 === payload._status)\n              (payload._status = 1), (payload._result = moduleObject);\n          },\n          function (error) {\n            if (0 === payload._status || -1 === payload._status)\n              (payload._status = 2), (payload._result = error);\n          }\n        );\n        -1 === payload._status &&\n          ((payload._status = 0), (payload._result = ctor));\n      }\n      if (1 === payload._status)\n        return (\n          (ctor = payload._result),\n          void 0 === ctor &&\n            console.error(\n              \"lazy: Expected the result of a dynamic import() call. Instead received: %s\\n\\nYour code should look like: \\n  const MyComponent = lazy(() => import('./MyComponent'))\\n\\nDid you accidentally put curly braces around the import?\",\n              ctor\n            ),\n          \"default\" in ctor ||\n            console.error(\n              \"lazy: Expected the result of a dynamic import() call. Instead received: %s\\n\\nYour code should look like: \\n  const MyComponent = lazy(() => import('./MyComponent'))\",\n              ctor\n            ),\n          ctor.default\n        );\n      throw payload._result;\n    }\n    function resolveDispatcher() {\n      var dispatcher = ReactSharedInternals.H;\n      null === dispatcher &&\n        console.error(\n          \"Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:\\n1. You might have mismatching versions of React and the renderer (such as React DOM)\\n2. You might be breaking the Rules of Hooks\\n3. You might have more than one copy of React in the same app\\nSee https://react.dev/link/invalid-hook-call for tips about how to debug and fix this problem.\"\n        );\n      return dispatcher;\n    }\n    function noop() {}\n    function enqueueTask(task) {\n      if (null === enqueueTaskImpl)\n        try {\n          var requireString = (\"require\" + Math.random()).slice(0, 7);\n          enqueueTaskImpl = (module && module[requireString]).call(\n            module,\n            \"timers\"\n          ).setImmediate;\n        } catch (_err) {\n          enqueueTaskImpl = function (callback) {\n            !1 === didWarnAboutMessageChannel &&\n              ((didWarnAboutMessageChannel = !0),\n              \"undefined\" === typeof MessageChannel &&\n                console.error(\n                  \"This browser does not have a MessageChannel implementation, so enqueuing tasks via await act(async () => ...) will fail. Please file an issue at https://github.com/facebook/react/issues if you encounter this warning.\"\n                ));\n            var channel = new MessageChannel();\n            channel.port1.onmessage = callback;\n            channel.port2.postMessage(void 0);\n          };\n        }\n      return enqueueTaskImpl(task);\n    }\n    function aggregateErrors(errors) {\n      return 1 < errors.length && \"function\" === typeof AggregateError\n        ? new AggregateError(errors)\n        : errors[0];\n    }\n    function popActScope(prevActQueue, prevActScopeDepth) {\n      prevActScopeDepth !== actScopeDepth - 1 &&\n        console.error(\n          \"You seem to have overlapping act() calls, this is not supported. Be sure to await previous act() calls before making a new one. \"\n        );\n      actScopeDepth = prevActScopeDepth;\n    }\n    function recursivelyFlushAsyncActWork(returnValue, resolve, reject) {\n      var queue = ReactSharedInternals.actQueue;\n      if (null !== queue)\n        if (0 !== queue.length)\n          try {\n            flushActQueue(queue);\n            enqueueTask(function () {\n              return recursivelyFlushAsyncActWork(returnValue, resolve, reject);\n            });\n            return;\n          } catch (error) {\n            ReactSharedInternals.thrownErrors.push(error);\n          }\n        else ReactSharedInternals.actQueue = null;\n      0 < ReactSharedInternals.thrownErrors.length\n        ? ((queue = aggregateErrors(ReactSharedInternals.thrownErrors)),\n          (ReactSharedInternals.thrownErrors.length = 0),\n          reject(queue))\n        : resolve(returnValue);\n    }\n    function flushActQueue(queue) {\n      if (!isFlushing) {\n        isFlushing = !0;\n        var i = 0;\n        try {\n          for (; i < queue.length; i++) {\n            var callback = queue[i];\n            do {\n              ReactSharedInternals.didUsePromise = !1;\n              var continuation = callback(!1);\n              if (null !== continuation) {\n                if (ReactSharedInternals.didUsePromise) {\n                  queue[i] = callback;\n                  queue.splice(0, i);\n                  return;\n                }\n                callback = continuation;\n              } else break;\n            } while (1);\n          }\n          queue.length = 0;\n        } catch (error) {\n          queue.splice(0, i + 1), ReactSharedInternals.thrownErrors.push(error);\n        } finally {\n          isFlushing = !1;\n        }\n      }\n    }\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(Error());\n    var REACT_ELEMENT_TYPE = Symbol.for(\"react.transitional.element\"),\n      REACT_PORTAL_TYPE = Symbol.for(\"react.portal\"),\n      REACT_FRAGMENT_TYPE = Symbol.for(\"react.fragment\"),\n      REACT_STRICT_MODE_TYPE = Symbol.for(\"react.strict_mode\"),\n      REACT_PROFILER_TYPE = Symbol.for(\"react.profiler\");\n    Symbol.for(\"react.provider\");\n    var REACT_CONSUMER_TYPE = Symbol.for(\"react.consumer\"),\n      REACT_CONTEXT_TYPE = Symbol.for(\"react.context\"),\n      REACT_FORWARD_REF_TYPE = Symbol.for(\"react.forward_ref\"),\n      REACT_SUSPENSE_TYPE = Symbol.for(\"react.suspense\"),\n      REACT_SUSPENSE_LIST_TYPE = Symbol.for(\"react.suspense_list\"),\n      REACT_MEMO_TYPE = Symbol.for(\"react.memo\"),\n      REACT_LAZY_TYPE = Symbol.for(\"react.lazy\"),\n      REACT_ACTIVITY_TYPE = Symbol.for(\"react.activity\"),\n      MAYBE_ITERATOR_SYMBOL = Symbol.iterator,\n      didWarnStateUpdateForUnmountedComponent = {},\n      ReactNoopUpdateQueue = {\n        isMounted: function () {\n          return !1;\n        },\n        enqueueForceUpdate: function (publicInstance) {\n          warnNoop(publicInstance, \"forceUpdate\");\n        },\n        enqueueReplaceState: function (publicInstance) {\n          warnNoop(publicInstance, \"replaceState\");\n        },\n        enqueueSetState: function (publicInstance) {\n          warnNoop(publicInstance, \"setState\");\n        }\n      },\n      assign = Object.assign,\n      emptyObject = {};\n    Object.freeze(emptyObject);\n    Component.prototype.isReactComponent = {};\n    Component.prototype.setState = function (partialState, callback) {\n      if (\n        \"object\" !== typeof partialState &&\n        \"function\" !== typeof partialState &&\n        null != partialState\n      )\n        throw Error(\n          \"takes an object of state variables to update or a function which returns an object of state variables.\"\n        );\n      this.updater.enqueueSetState(this, partialState, callback, \"setState\");\n    };\n    Component.prototype.forceUpdate = function (callback) {\n      this.updater.enqueueForceUpdate(this, callback, \"forceUpdate\");\n    };\n    var deprecatedAPIs = {\n        isMounted: [\n          \"isMounted\",\n          \"Instead, make sure to clean up subscriptions and pending requests in componentWillUnmount to prevent memory leaks.\"\n        ],\n        replaceState: [\n          \"replaceState\",\n          \"Refactor your code to use setState instead (see https://github.com/facebook/react/issues/3236).\"\n        ]\n      },\n      fnName;\n    for (fnName in deprecatedAPIs)\n      deprecatedAPIs.hasOwnProperty(fnName) &&\n        defineDeprecationWarning(fnName, deprecatedAPIs[fnName]);\n    ComponentDummy.prototype = Component.prototype;\n    deprecatedAPIs = PureComponent.prototype = new ComponentDummy();\n    deprecatedAPIs.constructor = PureComponent;\n    assign(deprecatedAPIs, Component.prototype);\n    deprecatedAPIs.isPureReactComponent = !0;\n    var isArrayImpl = Array.isArray,\n      REACT_CLIENT_REFERENCE = Symbol.for(\"react.client.reference\"),\n      ReactSharedInternals = {\n        H: null,\n        A: null,\n        T: null,\n        S: null,\n        V: null,\n        actQueue: null,\n        isBatchingLegacy: !1,\n        didScheduleLegacyUpdate: !1,\n        didUsePromise: !1,\n        thrownErrors: [],\n        getCurrentStack: null,\n        recentlyCreatedOwnerStacks: 0\n      },\n      hasOwnProperty = Object.prototype.hasOwnProperty,\n      createTask = console.createTask\n        ? console.createTask\n        : function () {\n            return null;\n          };\n    deprecatedAPIs = {\n      \"react-stack-bottom-frame\": function (callStackForError) {\n        return callStackForError();\n      }\n    };\n    var specialPropKeyWarningShown, didWarnAboutOldJSXRuntime;\n    var didWarnAboutElementRef = {};\n    var unknownOwnerDebugStack = deprecatedAPIs[\n      \"react-stack-bottom-frame\"\n    ].bind(deprecatedAPIs, UnknownOwner)();\n    var unknownOwnerDebugTask = createTask(getTaskName(UnknownOwner));\n    var didWarnAboutMaps = !1,\n      userProvidedKeyEscapeRegex = /\\/+/g,\n      reportGlobalError =\n        \"function\" === typeof reportError\n          ? reportError\n          : function (error) {\n              if (\n                \"object\" === typeof window &&\n                \"function\" === typeof window.ErrorEvent\n              ) {\n                var event = new window.ErrorEvent(\"error\", {\n                  bubbles: !0,\n                  cancelable: !0,\n                  message:\n                    \"object\" === typeof error &&\n                    null !== error &&\n                    \"string\" === typeof error.message\n                      ? String(error.message)\n                      : String(error),\n                  error: error\n                });\n                if (!window.dispatchEvent(event)) return;\n              } else if (\n                \"object\" === typeof process &&\n                \"function\" === typeof process.emit\n              ) {\n                process.emit(\"uncaughtException\", error);\n                return;\n              }\n              console.error(error);\n            },\n      didWarnAboutMessageChannel = !1,\n      enqueueTaskImpl = null,\n      actScopeDepth = 0,\n      didWarnNoAwaitAct = !1,\n      isFlushing = !1,\n      queueSeveralMicrotasks =\n        \"function\" === typeof queueMicrotask\n          ? function (callback) {\n              queueMicrotask(function () {\n                return queueMicrotask(callback);\n              });\n            }\n          : enqueueTask;\n    deprecatedAPIs = Object.freeze({\n      __proto__: null,\n      c: function (size) {\n        return resolveDispatcher().useMemoCache(size);\n      }\n    });\n    exports.Children = {\n      map: mapChildren,\n      forEach: function (children, forEachFunc, forEachContext) {\n        mapChildren(\n          children,\n          function () {\n            forEachFunc.apply(this, arguments);\n          },\n          forEachContext\n        );\n      },\n      count: function (children) {\n        var n = 0;\n        mapChildren(children, function () {\n          n++;\n        });\n        return n;\n      },\n      toArray: function (children) {\n        return (\n          mapChildren(children, function (child) {\n            return child;\n          }) || []\n        );\n      },\n      only: function (children) {\n        if (!isValidElement(children))\n          throw Error(\n            \"React.Children.only expected to receive a single React element child.\"\n          );\n        return children;\n      }\n    };\n    exports.Component = Component;\n    exports.Fragment = REACT_FRAGMENT_TYPE;\n    exports.Profiler = REACT_PROFILER_TYPE;\n    exports.PureComponent = PureComponent;\n    exports.StrictMode = REACT_STRICT_MODE_TYPE;\n    exports.Suspense = REACT_SUSPENSE_TYPE;\n    exports.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE =\n      ReactSharedInternals;\n    exports.__COMPILER_RUNTIME = deprecatedAPIs;\n    exports.act = function (callback) {\n      var prevActQueue = ReactSharedInternals.actQueue,\n        prevActScopeDepth = actScopeDepth;\n      actScopeDepth++;\n      var queue = (ReactSharedInternals.actQueue =\n          null !== prevActQueue ? prevActQueue : []),\n        didAwaitActCall = !1;\n      try {\n        var result = callback();\n      } catch (error) {\n        ReactSharedInternals.thrownErrors.push(error);\n      }\n      if (0 < ReactSharedInternals.thrownErrors.length)\n        throw (\n          (popActScope(prevActQueue, prevActScopeDepth),\n          (callback = aggregateErrors(ReactSharedInternals.thrownErrors)),\n          (ReactSharedInternals.thrownErrors.length = 0),\n          callback)\n        );\n      if (\n        null !== result &&\n        \"object\" === typeof result &&\n        \"function\" === typeof result.then\n      ) {\n        var thenable = result;\n        queueSeveralMicrotasks(function () {\n          didAwaitActCall ||\n            didWarnNoAwaitAct ||\n            ((didWarnNoAwaitAct = !0),\n            console.error(\n              \"You called act(async () => ...) without await. This could lead to unexpected testing behaviour, interleaving multiple act calls and mixing their scopes. You should - await act(async () => ...);\"\n            ));\n        });\n        return {\n          then: function (resolve, reject) {\n            didAwaitActCall = !0;\n            thenable.then(\n              function (returnValue) {\n                popActScope(prevActQueue, prevActScopeDepth);\n                if (0 === prevActScopeDepth) {\n                  try {\n                    flushActQueue(queue),\n                      enqueueTask(function () {\n                        return recursivelyFlushAsyncActWork(\n                          returnValue,\n                          resolve,\n                          reject\n                        );\n                      });\n                  } catch (error$0) {\n                    ReactSharedInternals.thrownErrors.push(error$0);\n                  }\n                  if (0 < ReactSharedInternals.thrownErrors.length) {\n                    var _thrownError = aggregateErrors(\n                      ReactSharedInternals.thrownErrors\n                    );\n                    ReactSharedInternals.thrownErrors.length = 0;\n                    reject(_thrownError);\n                  }\n                } else resolve(returnValue);\n              },\n              function (error) {\n                popActScope(prevActQueue, prevActScopeDepth);\n                0 < ReactSharedInternals.thrownErrors.length\n                  ? ((error = aggregateErrors(\n                      ReactSharedInternals.thrownErrors\n                    )),\n                    (ReactSharedInternals.thrownErrors.length = 0),\n                    reject(error))\n                  : reject(error);\n              }\n            );\n          }\n        };\n      }\n      var returnValue$jscomp$0 = result;\n      popActScope(prevActQueue, prevActScopeDepth);\n      0 === prevActScopeDepth &&\n        (flushActQueue(queue),\n        0 !== queue.length &&\n          queueSeveralMicrotasks(function () {\n            didAwaitActCall ||\n              didWarnNoAwaitAct ||\n              ((didWarnNoAwaitAct = !0),\n              console.error(\n                \"A component suspended inside an `act` scope, but the `act` call was not awaited. When testing React components that depend on asynchronous data, you must await the result:\\n\\nawait act(() => ...)\"\n              ));\n          }),\n        (ReactSharedInternals.actQueue = null));\n      if (0 < ReactSharedInternals.thrownErrors.length)\n        throw (\n          ((callback = aggregateErrors(ReactSharedInternals.thrownErrors)),\n          (ReactSharedInternals.thrownErrors.length = 0),\n          callback)\n        );\n      return {\n        then: function (resolve, reject) {\n          didAwaitActCall = !0;\n          0 === prevActScopeDepth\n            ? ((ReactSharedInternals.actQueue = queue),\n              enqueueTask(function () {\n                return recursivelyFlushAsyncActWork(\n                  returnValue$jscomp$0,\n                  resolve,\n                  reject\n                );\n              }))\n            : resolve(returnValue$jscomp$0);\n        }\n      };\n    };\n    exports.cache = function (fn) {\n      return function () {\n        return fn.apply(null, arguments);\n      };\n    };\n    exports.captureOwnerStack = function () {\n      var getCurrentStack = ReactSharedInternals.getCurrentStack;\n      return null === getCurrentStack ? null : getCurrentStack();\n    };\n    exports.cloneElement = function (element, config, children) {\n      if (null === element || void 0 === element)\n        throw Error(\n          \"The argument must be a React element, but you passed \" +\n            element +\n            \".\"\n        );\n      var props = assign({}, element.props),\n        key = element.key,\n        owner = element._owner;\n      if (null != config) {\n        var JSCompiler_inline_result;\n        a: {\n          if (\n            hasOwnProperty.call(config, \"ref\") &&\n            (JSCompiler_inline_result = Object.getOwnPropertyDescriptor(\n              config,\n              \"ref\"\n            ).get) &&\n            JSCompiler_inline_result.isReactWarning\n          ) {\n            JSCompiler_inline_result = !1;\n            break a;\n          }\n          JSCompiler_inline_result = void 0 !== config.ref;\n        }\n        JSCompiler_inline_result && (owner = getOwner());\n        hasValidKey(config) &&\n          (checkKeyStringCoercion(config.key), (key = \"\" + config.key));\n        for (propName in config)\n          !hasOwnProperty.call(config, propName) ||\n            \"key\" === propName ||\n            \"__self\" === propName ||\n            \"__source\" === propName ||\n            (\"ref\" === propName && void 0 === config.ref) ||\n            (props[propName] = config[propName]);\n      }\n      var propName = arguments.length - 2;\n      if (1 === propName) props.children = children;\n      else if (1 < propName) {\n        JSCompiler_inline_result = Array(propName);\n        for (var i = 0; i < propName; i++)\n          JSCompiler_inline_result[i] = arguments[i + 2];\n        props.children = JSCompiler_inline_result;\n      }\n      props = ReactElement(\n        element.type,\n        key,\n        void 0,\n        void 0,\n        owner,\n        props,\n        element._debugStack,\n        element._debugTask\n      );\n      for (key = 2; key < arguments.length; key++)\n        (owner = arguments[key]),\n          isValidElement(owner) && owner._store && (owner._store.validated = 1);\n      return props;\n    };\n    exports.createContext = function (defaultValue) {\n      defaultValue = {\n        $$typeof: REACT_CONTEXT_TYPE,\n        _currentValue: defaultValue,\n        _currentValue2: defaultValue,\n        _threadCount: 0,\n        Provider: null,\n        Consumer: null\n      };\n      defaultValue.Provider = defaultValue;\n      defaultValue.Consumer = {\n        $$typeof: REACT_CONSUMER_TYPE,\n        _context: defaultValue\n      };\n      defaultValue._currentRenderer = null;\n      defaultValue._currentRenderer2 = null;\n      return defaultValue;\n    };\n    exports.createElement = function (type, config, children) {\n      for (var i = 2; i < arguments.length; i++) {\n        var node = arguments[i];\n        isValidElement(node) && node._store && (node._store.validated = 1);\n      }\n      i = {};\n      node = null;\n      if (null != config)\n        for (propName in (didWarnAboutOldJSXRuntime ||\n          !(\"__self\" in config) ||\n          \"key\" in config ||\n          ((didWarnAboutOldJSXRuntime = !0),\n          console.warn(\n            \"Your app (or one of its dependencies) is using an outdated JSX transform. Update to the modern JSX transform for faster performance: https://react.dev/link/new-jsx-transform\"\n          )),\n        hasValidKey(config) &&\n          (checkKeyStringCoercion(config.key), (node = \"\" + config.key)),\n        config))\n          hasOwnProperty.call(config, propName) &&\n            \"key\" !== propName &&\n            \"__self\" !== propName &&\n            \"__source\" !== propName &&\n            (i[propName] = config[propName]);\n      var childrenLength = arguments.length - 2;\n      if (1 === childrenLength) i.children = children;\n      else if (1 < childrenLength) {\n        for (\n          var childArray = Array(childrenLength), _i = 0;\n          _i < childrenLength;\n          _i++\n        )\n          childArray[_i] = arguments[_i + 2];\n        Object.freeze && Object.freeze(childArray);\n        i.children = childArray;\n      }\n      if (type && type.defaultProps)\n        for (propName in ((childrenLength = type.defaultProps), childrenLength))\n          void 0 === i[propName] && (i[propName] = childrenLength[propName]);\n      node &&\n        defineKeyPropWarningGetter(\n          i,\n          \"function\" === typeof type\n            ? type.displayName || type.name || \"Unknown\"\n            : type\n        );\n      var propName = 1e4 > ReactSharedInternals.recentlyCreatedOwnerStacks++;\n      return ReactElement(\n        type,\n        node,\n        void 0,\n        void 0,\n        getOwner(),\n        i,\n        propName ? Error(\"react-stack-top-frame\") : unknownOwnerDebugStack,\n        propName ? createTask(getTaskName(type)) : unknownOwnerDebugTask\n      );\n    };\n    exports.createRef = function () {\n      var refObject = { current: null };\n      Object.seal(refObject);\n      return refObject;\n    };\n    exports.forwardRef = function (render) {\n      null != render && render.$$typeof === REACT_MEMO_TYPE\n        ? console.error(\n            \"forwardRef requires a render function but received a `memo` component. Instead of forwardRef(memo(...)), use memo(forwardRef(...)).\"\n          )\n        : \"function\" !== typeof render\n          ? console.error(\n              \"forwardRef requires a render function but was given %s.\",\n              null === render ? \"null\" : typeof render\n            )\n          : 0 !== render.length &&\n            2 !== render.length &&\n            console.error(\n              \"forwardRef render functions accept exactly two parameters: props and ref. %s\",\n              1 === render.length\n                ? \"Did you forget to use the ref parameter?\"\n                : \"Any additional parameter will be undefined.\"\n            );\n      null != render &&\n        null != render.defaultProps &&\n        console.error(\n          \"forwardRef render functions do not support defaultProps. Did you accidentally pass a React component?\"\n        );\n      var elementType = { $$typeof: REACT_FORWARD_REF_TYPE, render: render },\n        ownName;\n      Object.defineProperty(elementType, \"displayName\", {\n        enumerable: !1,\n        configurable: !0,\n        get: function () {\n          return ownName;\n        },\n        set: function (name) {\n          ownName = name;\n          render.name ||\n            render.displayName ||\n            (Object.defineProperty(render, \"name\", { value: name }),\n            (render.displayName = name));\n        }\n      });\n      return elementType;\n    };\n    exports.isValidElement = isValidElement;\n    exports.lazy = function (ctor) {\n      return {\n        $$typeof: REACT_LAZY_TYPE,\n        _payload: { _status: -1, _result: ctor },\n        _init: lazyInitializer\n      };\n    };\n    exports.memo = function (type, compare) {\n      null == type &&\n        console.error(\n          \"memo: The first argument must be a component. Instead received: %s\",\n          null === type ? \"null\" : typeof type\n        );\n      compare = {\n        $$typeof: REACT_MEMO_TYPE,\n        type: type,\n        compare: void 0 === compare ? null : compare\n      };\n      var ownName;\n      Object.defineProperty(compare, \"displayName\", {\n        enumerable: !1,\n        configurable: !0,\n        get: function () {\n          return ownName;\n        },\n        set: function (name) {\n          ownName = name;\n          type.name ||\n            type.displayName ||\n            (Object.defineProperty(type, \"name\", { value: name }),\n            (type.displayName = name));\n        }\n      });\n      return compare;\n    };\n    exports.startTransition = function (scope) {\n      var prevTransition = ReactSharedInternals.T,\n        currentTransition = {};\n      ReactSharedInternals.T = currentTransition;\n      currentTransition._updatedFibers = new Set();\n      try {\n        var returnValue = scope(),\n          onStartTransitionFinish = ReactSharedInternals.S;\n        null !== onStartTransitionFinish &&\n          onStartTransitionFinish(currentTransition, returnValue);\n        \"object\" === typeof returnValue &&\n          null !== returnValue &&\n          \"function\" === typeof returnValue.then &&\n          returnValue.then(noop, reportGlobalError);\n      } catch (error) {\n        reportGlobalError(error);\n      } finally {\n        null === prevTransition &&\n          currentTransition._updatedFibers &&\n          ((scope = currentTransition._updatedFibers.size),\n          currentTransition._updatedFibers.clear(),\n          10 < scope &&\n            console.warn(\n              \"Detected a large number of updates inside startTransition. If this is due to a subscription please re-write it to use React provided hooks. Otherwise concurrent mode guarantees are off the table.\"\n            )),\n          (ReactSharedInternals.T = prevTransition);\n      }\n    };\n    exports.unstable_useCacheRefresh = function () {\n      return resolveDispatcher().useCacheRefresh();\n    };\n    exports.use = function (usable) {\n      return resolveDispatcher().use(usable);\n    };\n    exports.useActionState = function (action, initialState, permalink) {\n      return resolveDispatcher().useActionState(\n        action,\n        initialState,\n        permalink\n      );\n    };\n    exports.useCallback = function (callback, deps) {\n      return resolveDispatcher().useCallback(callback, deps);\n    };\n    exports.useContext = function (Context) {\n      var dispatcher = resolveDispatcher();\n      Context.$$typeof === REACT_CONSUMER_TYPE &&\n        console.error(\n          \"Calling useContext(Context.Consumer) is not supported and will cause bugs. Did you mean to call useContext(Context) instead?\"\n        );\n      return dispatcher.useContext(Context);\n    };\n    exports.useDebugValue = function (value, formatterFn) {\n      return resolveDispatcher().useDebugValue(value, formatterFn);\n    };\n    exports.useDeferredValue = function (value, initialValue) {\n      return resolveDispatcher().useDeferredValue(value, initialValue);\n    };\n    exports.useEffect = function (create, createDeps, update) {\n      null == create &&\n        console.warn(\n          \"React Hook useEffect requires an effect callback. Did you forget to pass a callback to the hook?\"\n        );\n      var dispatcher = resolveDispatcher();\n      if (\"function\" === typeof update)\n        throw Error(\n          \"useEffect CRUD overload is not enabled in this build of React.\"\n        );\n      return dispatcher.useEffect(create, createDeps);\n    };\n    exports.useId = function () {\n      return resolveDispatcher().useId();\n    };\n    exports.useImperativeHandle = function (ref, create, deps) {\n      return resolveDispatcher().useImperativeHandle(ref, create, deps);\n    };\n    exports.useInsertionEffect = function (create, deps) {\n      null == create &&\n        console.warn(\n          \"React Hook useInsertionEffect requires an effect callback. Did you forget to pass a callback to the hook?\"\n        );\n      return resolveDispatcher().useInsertionEffect(create, deps);\n    };\n    exports.useLayoutEffect = function (create, deps) {\n      null == create &&\n        console.warn(\n          \"React Hook useLayoutEffect requires an effect callback. Did you forget to pass a callback to the hook?\"\n        );\n      return resolveDispatcher().useLayoutEffect(create, deps);\n    };\n    exports.useMemo = function (create, deps) {\n      return resolveDispatcher().useMemo(create, deps);\n    };\n    exports.useOptimistic = function (passthrough, reducer) {\n      return resolveDispatcher().useOptimistic(passthrough, reducer);\n    };\n    exports.useReducer = function (reducer, initialArg, init) {\n      return resolveDispatcher().useReducer(reducer, initialArg, init);\n    };\n    exports.useRef = function (initialValue) {\n      return resolveDispatcher().useRef(initialValue);\n    };\n    exports.useState = function (initialState) {\n      return resolveDispatcher().useState(initialState);\n    };\n    exports.useSyncExternalStore = function (\n      subscribe,\n      getSnapshot,\n      getServerSnapshot\n    ) {\n      return resolveDispatcher().useSyncExternalStore(\n        subscribe,\n        getSnapshot,\n        getServerSnapshot\n      );\n    };\n    exports.useTransition = function () {\n      return resolveDispatcher().useTransition();\n    };\n    exports.version = \"19.1.0\";\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(Error());\n  })();\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react/cjs/react.development.js?");

/***/ }),

/***/ "./node_modules/react/index.js":
/*!*************************************!*\
  !*** ./node_modules/react/index.js ***!
  \*************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/react.development.js */ \"./node_modules/react/cjs/react.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react/index.js?");

/***/ }),

/***/ "./node_modules/react/jsx-runtime.js":
/*!*******************************************!*\
  !*** ./node_modules/react/jsx-runtime.js ***!
  \*******************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/react-jsx-runtime.development.js */ \"./node_modules/react/cjs/react-jsx-runtime.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/react/jsx-runtime.js?");

/***/ }),

/***/ "./node_modules/scheduler/cjs/scheduler.development.js":
/*!*************************************************************!*\
  !*** ./node_modules/scheduler/cjs/scheduler.development.js ***!
  \*************************************************************/
/***/ ((__unused_webpack_module, exports) => {

eval("/**\n * @license React\n * scheduler.development.js\n *\n * Copyright (c) Meta Platforms, Inc. and affiliates.\n *\n * This source code is licensed under the MIT license found in the\n * LICENSE file in the root directory of this source tree.\n */\n\n\n true &&\n  (function () {\n    function performWorkUntilDeadline() {\n      needsPaint = !1;\n      if (isMessageLoopRunning) {\n        var currentTime = exports.unstable_now();\n        startTime = currentTime;\n        var hasMoreWork = !0;\n        try {\n          a: {\n            isHostCallbackScheduled = !1;\n            isHostTimeoutScheduled &&\n              ((isHostTimeoutScheduled = !1),\n              localClearTimeout(taskTimeoutID),\n              (taskTimeoutID = -1));\n            isPerformingWork = !0;\n            var previousPriorityLevel = currentPriorityLevel;\n            try {\n              b: {\n                advanceTimers(currentTime);\n                for (\n                  currentTask = peek(taskQueue);\n                  null !== currentTask &&\n                  !(\n                    currentTask.expirationTime > currentTime &&\n                    shouldYieldToHost()\n                  );\n\n                ) {\n                  var callback = currentTask.callback;\n                  if (\"function\" === typeof callback) {\n                    currentTask.callback = null;\n                    currentPriorityLevel = currentTask.priorityLevel;\n                    var continuationCallback = callback(\n                      currentTask.expirationTime <= currentTime\n                    );\n                    currentTime = exports.unstable_now();\n                    if (\"function\" === typeof continuationCallback) {\n                      currentTask.callback = continuationCallback;\n                      advanceTimers(currentTime);\n                      hasMoreWork = !0;\n                      break b;\n                    }\n                    currentTask === peek(taskQueue) && pop(taskQueue);\n                    advanceTimers(currentTime);\n                  } else pop(taskQueue);\n                  currentTask = peek(taskQueue);\n                }\n                if (null !== currentTask) hasMoreWork = !0;\n                else {\n                  var firstTimer = peek(timerQueue);\n                  null !== firstTimer &&\n                    requestHostTimeout(\n                      handleTimeout,\n                      firstTimer.startTime - currentTime\n                    );\n                  hasMoreWork = !1;\n                }\n              }\n              break a;\n            } finally {\n              (currentTask = null),\n                (currentPriorityLevel = previousPriorityLevel),\n                (isPerformingWork = !1);\n            }\n            hasMoreWork = void 0;\n          }\n        } finally {\n          hasMoreWork\n            ? schedulePerformWorkUntilDeadline()\n            : (isMessageLoopRunning = !1);\n        }\n      }\n    }\n    function push(heap, node) {\n      var index = heap.length;\n      heap.push(node);\n      a: for (; 0 < index; ) {\n        var parentIndex = (index - 1) >>> 1,\n          parent = heap[parentIndex];\n        if (0 < compare(parent, node))\n          (heap[parentIndex] = node),\n            (heap[index] = parent),\n            (index = parentIndex);\n        else break a;\n      }\n    }\n    function peek(heap) {\n      return 0 === heap.length ? null : heap[0];\n    }\n    function pop(heap) {\n      if (0 === heap.length) return null;\n      var first = heap[0],\n        last = heap.pop();\n      if (last !== first) {\n        heap[0] = last;\n        a: for (\n          var index = 0, length = heap.length, halfLength = length >>> 1;\n          index < halfLength;\n\n        ) {\n          var leftIndex = 2 * (index + 1) - 1,\n            left = heap[leftIndex],\n            rightIndex = leftIndex + 1,\n            right = heap[rightIndex];\n          if (0 > compare(left, last))\n            rightIndex < length && 0 > compare(right, left)\n              ? ((heap[index] = right),\n                (heap[rightIndex] = last),\n                (index = rightIndex))\n              : ((heap[index] = left),\n                (heap[leftIndex] = last),\n                (index = leftIndex));\n          else if (rightIndex < length && 0 > compare(right, last))\n            (heap[index] = right),\n              (heap[rightIndex] = last),\n              (index = rightIndex);\n          else break a;\n        }\n      }\n      return first;\n    }\n    function compare(a, b) {\n      var diff = a.sortIndex - b.sortIndex;\n      return 0 !== diff ? diff : a.id - b.id;\n    }\n    function advanceTimers(currentTime) {\n      for (var timer = peek(timerQueue); null !== timer; ) {\n        if (null === timer.callback) pop(timerQueue);\n        else if (timer.startTime <= currentTime)\n          pop(timerQueue),\n            (timer.sortIndex = timer.expirationTime),\n            push(taskQueue, timer);\n        else break;\n        timer = peek(timerQueue);\n      }\n    }\n    function handleTimeout(currentTime) {\n      isHostTimeoutScheduled = !1;\n      advanceTimers(currentTime);\n      if (!isHostCallbackScheduled)\n        if (null !== peek(taskQueue))\n          (isHostCallbackScheduled = !0),\n            isMessageLoopRunning ||\n              ((isMessageLoopRunning = !0), schedulePerformWorkUntilDeadline());\n        else {\n          var firstTimer = peek(timerQueue);\n          null !== firstTimer &&\n            requestHostTimeout(\n              handleTimeout,\n              firstTimer.startTime - currentTime\n            );\n        }\n    }\n    function shouldYieldToHost() {\n      return needsPaint\n        ? !0\n        : exports.unstable_now() - startTime < frameInterval\n          ? !1\n          : !0;\n    }\n    function requestHostTimeout(callback, ms) {\n      taskTimeoutID = localSetTimeout(function () {\n        callback(exports.unstable_now());\n      }, ms);\n    }\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(Error());\n    exports.unstable_now = void 0;\n    if (\n      \"object\" === typeof performance &&\n      \"function\" === typeof performance.now\n    ) {\n      var localPerformance = performance;\n      exports.unstable_now = function () {\n        return localPerformance.now();\n      };\n    } else {\n      var localDate = Date,\n        initialTime = localDate.now();\n      exports.unstable_now = function () {\n        return localDate.now() - initialTime;\n      };\n    }\n    var taskQueue = [],\n      timerQueue = [],\n      taskIdCounter = 1,\n      currentTask = null,\n      currentPriorityLevel = 3,\n      isPerformingWork = !1,\n      isHostCallbackScheduled = !1,\n      isHostTimeoutScheduled = !1,\n      needsPaint = !1,\n      localSetTimeout = \"function\" === typeof setTimeout ? setTimeout : null,\n      localClearTimeout =\n        \"function\" === typeof clearTimeout ? clearTimeout : null,\n      localSetImmediate =\n        \"undefined\" !== typeof setImmediate ? setImmediate : null,\n      isMessageLoopRunning = !1,\n      taskTimeoutID = -1,\n      frameInterval = 5,\n      startTime = -1;\n    if (\"function\" === typeof localSetImmediate)\n      var schedulePerformWorkUntilDeadline = function () {\n        localSetImmediate(performWorkUntilDeadline);\n      };\n    else if (\"undefined\" !== typeof MessageChannel) {\n      var channel = new MessageChannel(),\n        port = channel.port2;\n      channel.port1.onmessage = performWorkUntilDeadline;\n      schedulePerformWorkUntilDeadline = function () {\n        port.postMessage(null);\n      };\n    } else\n      schedulePerformWorkUntilDeadline = function () {\n        localSetTimeout(performWorkUntilDeadline, 0);\n      };\n    exports.unstable_IdlePriority = 5;\n    exports.unstable_ImmediatePriority = 1;\n    exports.unstable_LowPriority = 4;\n    exports.unstable_NormalPriority = 3;\n    exports.unstable_Profiling = null;\n    exports.unstable_UserBlockingPriority = 2;\n    exports.unstable_cancelCallback = function (task) {\n      task.callback = null;\n    };\n    exports.unstable_forceFrameRate = function (fps) {\n      0 > fps || 125 < fps\n        ? console.error(\n            \"forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported\"\n          )\n        : (frameInterval = 0 < fps ? Math.floor(1e3 / fps) : 5);\n    };\n    exports.unstable_getCurrentPriorityLevel = function () {\n      return currentPriorityLevel;\n    };\n    exports.unstable_next = function (eventHandler) {\n      switch (currentPriorityLevel) {\n        case 1:\n        case 2:\n        case 3:\n          var priorityLevel = 3;\n          break;\n        default:\n          priorityLevel = currentPriorityLevel;\n      }\n      var previousPriorityLevel = currentPriorityLevel;\n      currentPriorityLevel = priorityLevel;\n      try {\n        return eventHandler();\n      } finally {\n        currentPriorityLevel = previousPriorityLevel;\n      }\n    };\n    exports.unstable_requestPaint = function () {\n      needsPaint = !0;\n    };\n    exports.unstable_runWithPriority = function (priorityLevel, eventHandler) {\n      switch (priorityLevel) {\n        case 1:\n        case 2:\n        case 3:\n        case 4:\n        case 5:\n          break;\n        default:\n          priorityLevel = 3;\n      }\n      var previousPriorityLevel = currentPriorityLevel;\n      currentPriorityLevel = priorityLevel;\n      try {\n        return eventHandler();\n      } finally {\n        currentPriorityLevel = previousPriorityLevel;\n      }\n    };\n    exports.unstable_scheduleCallback = function (\n      priorityLevel,\n      callback,\n      options\n    ) {\n      var currentTime = exports.unstable_now();\n      \"object\" === typeof options && null !== options\n        ? ((options = options.delay),\n          (options =\n            \"number\" === typeof options && 0 < options\n              ? currentTime + options\n              : currentTime))\n        : (options = currentTime);\n      switch (priorityLevel) {\n        case 1:\n          var timeout = -1;\n          break;\n        case 2:\n          timeout = 250;\n          break;\n        case 5:\n          timeout = 1073741823;\n          break;\n        case 4:\n          timeout = 1e4;\n          break;\n        default:\n          timeout = 5e3;\n      }\n      timeout = options + timeout;\n      priorityLevel = {\n        id: taskIdCounter++,\n        callback: callback,\n        priorityLevel: priorityLevel,\n        startTime: options,\n        expirationTime: timeout,\n        sortIndex: -1\n      };\n      options > currentTime\n        ? ((priorityLevel.sortIndex = options),\n          push(timerQueue, priorityLevel),\n          null === peek(taskQueue) &&\n            priorityLevel === peek(timerQueue) &&\n            (isHostTimeoutScheduled\n              ? (localClearTimeout(taskTimeoutID), (taskTimeoutID = -1))\n              : (isHostTimeoutScheduled = !0),\n            requestHostTimeout(handleTimeout, options - currentTime)))\n        : ((priorityLevel.sortIndex = timeout),\n          push(taskQueue, priorityLevel),\n          isHostCallbackScheduled ||\n            isPerformingWork ||\n            ((isHostCallbackScheduled = !0),\n            isMessageLoopRunning ||\n              ((isMessageLoopRunning = !0),\n              schedulePerformWorkUntilDeadline())));\n      return priorityLevel;\n    };\n    exports.unstable_shouldYield = shouldYieldToHost;\n    exports.unstable_wrapCallback = function (callback) {\n      var parentPriorityLevel = currentPriorityLevel;\n      return function () {\n        var previousPriorityLevel = currentPriorityLevel;\n        currentPriorityLevel = parentPriorityLevel;\n        try {\n          return callback.apply(this, arguments);\n        } finally {\n          currentPriorityLevel = previousPriorityLevel;\n        }\n      };\n    };\n    \"undefined\" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ &&\n      \"function\" ===\n        typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop &&\n      __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(Error());\n  })();\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/scheduler/cjs/scheduler.development.js?");

/***/ }),

/***/ "./node_modules/scheduler/index.js":
/*!*****************************************!*\
  !*** ./node_modules/scheduler/index.js ***!
  \*****************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

eval("\n\nif (false) // removed by dead control flow\n{} else {\n  module.exports = __webpack_require__(/*! ./cjs/scheduler.development.js */ \"./node_modules/scheduler/cjs/scheduler.development.js\");\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/scheduler/index.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Enum.js":
/*!*****************************************!*\
  !*** ./node_modules/stylis/src/Enum.js ***!
  \*****************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   CHARSET: () => (/* binding */ CHARSET),\n/* harmony export */   COMMENT: () => (/* binding */ COMMENT),\n/* harmony export */   COUNTER_STYLE: () => (/* binding */ COUNTER_STYLE),\n/* harmony export */   DECLARATION: () => (/* binding */ DECLARATION),\n/* harmony export */   DOCUMENT: () => (/* binding */ DOCUMENT),\n/* harmony export */   FONT_FACE: () => (/* binding */ FONT_FACE),\n/* harmony export */   FONT_FEATURE_VALUES: () => (/* binding */ FONT_FEATURE_VALUES),\n/* harmony export */   IMPORT: () => (/* binding */ IMPORT),\n/* harmony export */   KEYFRAMES: () => (/* binding */ KEYFRAMES),\n/* harmony export */   LAYER: () => (/* binding */ LAYER),\n/* harmony export */   MEDIA: () => (/* binding */ MEDIA),\n/* harmony export */   MOZ: () => (/* binding */ MOZ),\n/* harmony export */   MS: () => (/* binding */ MS),\n/* harmony export */   NAMESPACE: () => (/* binding */ NAMESPACE),\n/* harmony export */   PAGE: () => (/* binding */ PAGE),\n/* harmony export */   RULESET: () => (/* binding */ RULESET),\n/* harmony export */   SUPPORTS: () => (/* binding */ SUPPORTS),\n/* harmony export */   VIEWPORT: () => (/* binding */ VIEWPORT),\n/* harmony export */   WEBKIT: () => (/* binding */ WEBKIT)\n/* harmony export */ });\nvar MS = '-ms-'\nvar MOZ = '-moz-'\nvar WEBKIT = '-webkit-'\n\nvar COMMENT = 'comm'\nvar RULESET = 'rule'\nvar DECLARATION = 'decl'\n\nvar PAGE = '@page'\nvar MEDIA = '@media'\nvar IMPORT = '@import'\nvar CHARSET = '@charset'\nvar VIEWPORT = '@viewport'\nvar SUPPORTS = '@supports'\nvar DOCUMENT = '@document'\nvar NAMESPACE = '@namespace'\nvar KEYFRAMES = '@keyframes'\nvar FONT_FACE = '@font-face'\nvar COUNTER_STYLE = '@counter-style'\nvar FONT_FEATURE_VALUES = '@font-feature-values'\nvar LAYER = '@layer'\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Enum.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Middleware.js":
/*!***********************************************!*\
  !*** ./node_modules/stylis/src/Middleware.js ***!
  \***********************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   middleware: () => (/* binding */ middleware),\n/* harmony export */   namespace: () => (/* binding */ namespace),\n/* harmony export */   prefixer: () => (/* binding */ prefixer),\n/* harmony export */   rulesheet: () => (/* binding */ rulesheet)\n/* harmony export */ });\n/* harmony import */ var _Enum_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Enum.js */ \"./node_modules/stylis/src/Enum.js\");\n/* harmony import */ var _Utility_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Utility.js */ \"./node_modules/stylis/src/Utility.js\");\n/* harmony import */ var _Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./Tokenizer.js */ \"./node_modules/stylis/src/Tokenizer.js\");\n/* harmony import */ var _Serializer_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./Serializer.js */ \"./node_modules/stylis/src/Serializer.js\");\n/* harmony import */ var _Prefixer_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Prefixer.js */ \"./node_modules/stylis/src/Prefixer.js\");\n\n\n\n\n\n\n/**\n * @param {function[]} collection\n * @return {function}\n */\nfunction middleware (collection) {\n\tvar length = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.sizeof)(collection)\n\n\treturn function (element, index, children, callback) {\n\t\tvar output = ''\n\n\t\tfor (var i = 0; i < length; i++)\n\t\t\toutput += collection[i](element, index, children, callback) || ''\n\n\t\treturn output\n\t}\n}\n\n/**\n * @param {function} callback\n * @return {function}\n */\nfunction rulesheet (callback) {\n\treturn function (element) {\n\t\tif (!element.root)\n\t\t\tif (element = element.return)\n\t\t\t\tcallback(element)\n\t}\n}\n\n/**\n * @param {object} element\n * @param {number} index\n * @param {object[]} children\n * @param {function} callback\n */\nfunction prefixer (element, index, children, callback) {\n\tif (element.length > -1)\n\t\tif (!element.return)\n\t\t\tswitch (element.type) {\n\t\t\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.DECLARATION: element.return = (0,_Prefixer_js__WEBPACK_IMPORTED_MODULE_2__.prefix)(element.value, element.length, children)\n\t\t\t\t\treturn\n\t\t\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.KEYFRAMES:\n\t\t\t\t\treturn (0,_Serializer_js__WEBPACK_IMPORTED_MODULE_3__.serialize)([(0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__.copy)(element, {value: (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(element.value, '@', '@' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT)})], callback)\n\t\t\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.RULESET:\n\t\t\t\t\tif (element.length)\n\t\t\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.combine)(element.props, function (value) {\n\t\t\t\t\t\t\tswitch ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(value, /(::plac\\w+|:read-\\w+)/)) {\n\t\t\t\t\t\t\t\t// :read-(only|write)\n\t\t\t\t\t\t\t\tcase ':read-only': case ':read-write':\n\t\t\t\t\t\t\t\t\treturn (0,_Serializer_js__WEBPACK_IMPORTED_MODULE_3__.serialize)([(0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__.copy)(element, {props: [(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /:(read-\\w+)/, ':' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MOZ + '$1')]})], callback)\n\t\t\t\t\t\t\t\t// :placeholder\n\t\t\t\t\t\t\t\tcase '::placeholder':\n\t\t\t\t\t\t\t\t\treturn (0,_Serializer_js__WEBPACK_IMPORTED_MODULE_3__.serialize)([\n\t\t\t\t\t\t\t\t\t\t(0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__.copy)(element, {props: [(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /:(plac\\w+)/, ':' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + 'input-$1')]}),\n\t\t\t\t\t\t\t\t\t\t(0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__.copy)(element, {props: [(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /:(plac\\w+)/, ':' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MOZ + '$1')]}),\n\t\t\t\t\t\t\t\t\t\t(0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__.copy)(element, {props: [(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /:(plac\\w+)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'input-$1')]})\n\t\t\t\t\t\t\t\t\t], callback)\n\t\t\t\t\t\t\t}\n\n\t\t\t\t\t\t\treturn ''\n\t\t\t\t\t\t})\n\t\t\t}\n}\n\n/**\n * @param {object} element\n * @param {number} index\n * @param {object[]} children\n */\nfunction namespace (element) {\n\tswitch (element.type) {\n\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.RULESET:\n\t\t\telement.props = element.props.map(function (value) {\n\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.combine)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_4__.tokenize)(value), function (value, index, children) {\n\t\t\t\t\tswitch ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, 0)) {\n\t\t\t\t\t\t// \\f\n\t\t\t\t\t\tcase 12:\n\t\t\t\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.substr)(value, 1, (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.strlen)(value))\n\t\t\t\t\t\t// \\0 ( + > ~\n\t\t\t\t\t\tcase 0: case 40: case 43: case 62: case 126:\n\t\t\t\t\t\t\treturn value\n\t\t\t\t\t\t// :\n\t\t\t\t\t\tcase 58:\n\t\t\t\t\t\t\tif (children[++index] === 'global')\n\t\t\t\t\t\t\t\tchildren[index] = '', children[++index] = '\\f' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.substr)(children[index], index = 1, -1)\n\t\t\t\t\t\t// \\s\n\t\t\t\t\t\tcase 32:\n\t\t\t\t\t\t\treturn index === 1 ? '' : value\n\t\t\t\t\t\tdefault:\n\t\t\t\t\t\t\tswitch (index) {\n\t\t\t\t\t\t\t\tcase 0: element = value\n\t\t\t\t\t\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.sizeof)(children) > 1 ? '' : value\n\t\t\t\t\t\t\t\tcase index = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.sizeof)(children) - 1: case 2:\n\t\t\t\t\t\t\t\t\treturn index === 2 ? value + element + element : value + element\n\t\t\t\t\t\t\t\tdefault:\n\t\t\t\t\t\t\t\t\treturn value\n\t\t\t\t\t\t\t}\n\t\t\t\t\t}\n\t\t\t\t})\n\t\t\t})\n\t}\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Middleware.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Parser.js":
/*!*******************************************!*\
  !*** ./node_modules/stylis/src/Parser.js ***!
  \*******************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   comment: () => (/* binding */ comment),\n/* harmony export */   compile: () => (/* binding */ compile),\n/* harmony export */   declaration: () => (/* binding */ declaration),\n/* harmony export */   parse: () => (/* binding */ parse),\n/* harmony export */   ruleset: () => (/* binding */ ruleset)\n/* harmony export */ });\n/* harmony import */ var _Enum_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./Enum.js */ \"./node_modules/stylis/src/Enum.js\");\n/* harmony import */ var _Utility_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Utility.js */ \"./node_modules/stylis/src/Utility.js\");\n/* harmony import */ var _Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Tokenizer.js */ \"./node_modules/stylis/src/Tokenizer.js\");\n\n\n\n\n/**\n * @param {string} value\n * @return {object[]}\n */\nfunction compile (value) {\n\treturn (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.dealloc)(parse('', null, null, null, [''], value = (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.alloc)(value), 0, [0], value))\n}\n\n/**\n * @param {string} value\n * @param {object} root\n * @param {object?} parent\n * @param {string[]} rule\n * @param {string[]} rules\n * @param {string[]} rulesets\n * @param {number[]} pseudo\n * @param {number[]} points\n * @param {string[]} declarations\n * @return {object}\n */\nfunction parse (value, root, parent, rule, rules, rulesets, pseudo, points, declarations) {\n\tvar index = 0\n\tvar offset = 0\n\tvar length = pseudo\n\tvar atrule = 0\n\tvar property = 0\n\tvar previous = 0\n\tvar variable = 1\n\tvar scanning = 1\n\tvar ampersand = 1\n\tvar character = 0\n\tvar type = ''\n\tvar props = rules\n\tvar children = rulesets\n\tvar reference = rule\n\tvar characters = type\n\n\twhile (scanning)\n\t\tswitch (previous = character, character = (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.next)()) {\n\t\t\t// (\n\t\t\tcase 40:\n\t\t\t\tif (previous != 108 && (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.charat)(characters, length - 1) == 58) {\n\t\t\t\t\tif ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.indexof)(characters += (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.replace)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.delimit)(character), '&', '&\\f'), '&\\f') != -1)\n\t\t\t\t\t\tampersand = -1\n\t\t\t\t\tbreak\n\t\t\t\t}\n\t\t\t// \" ' [\n\t\t\tcase 34: case 39: case 91:\n\t\t\t\tcharacters += (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.delimit)(character)\n\t\t\t\tbreak\n\t\t\t// \\t \\n \\r \\s\n\t\t\tcase 9: case 10: case 13: case 32:\n\t\t\t\tcharacters += (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.whitespace)(previous)\n\t\t\t\tbreak\n\t\t\t// \\\n\t\t\tcase 92:\n\t\t\t\tcharacters += (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.escaping)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.caret)() - 1, 7)\n\t\t\t\tcontinue\n\t\t\t// /\n\t\t\tcase 47:\n\t\t\t\tswitch ((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.peek)()) {\n\t\t\t\t\tcase 42: case 47:\n\t\t\t\t\t\t;(0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.append)(comment((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.commenter)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.next)(), (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.caret)()), root, parent), declarations)\n\t\t\t\t\t\tbreak\n\t\t\t\t\tdefault:\n\t\t\t\t\t\tcharacters += '/'\n\t\t\t\t}\n\t\t\t\tbreak\n\t\t\t// {\n\t\t\tcase 123 * variable:\n\t\t\t\tpoints[index++] = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.strlen)(characters) * ampersand\n\t\t\t// } ; \\0\n\t\t\tcase 125 * variable: case 59: case 0:\n\t\t\t\tswitch (character) {\n\t\t\t\t\t// \\0 }\n\t\t\t\t\tcase 0: case 125: scanning = 0\n\t\t\t\t\t// ;\n\t\t\t\t\tcase 59 + offset: if (ampersand == -1) characters = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.replace)(characters, /\\f/g, '')\n\t\t\t\t\t\tif (property > 0 && ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.strlen)(characters) - length))\n\t\t\t\t\t\t\t(0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.append)(property > 32 ? declaration(characters + ';', rule, parent, length - 1) : declaration((0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.replace)(characters, ' ', '') + ';', rule, parent, length - 2), declarations)\n\t\t\t\t\t\tbreak\n\t\t\t\t\t// @ ;\n\t\t\t\t\tcase 59: characters += ';'\n\t\t\t\t\t// { rule/at-rule\n\t\t\t\t\tdefault:\n\t\t\t\t\t\t;(0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.append)(reference = ruleset(characters, root, parent, index, offset, rules, points, type, props = [], children = [], length), rulesets)\n\n\t\t\t\t\t\tif (character === 123)\n\t\t\t\t\t\t\tif (offset === 0)\n\t\t\t\t\t\t\t\tparse(characters, root, reference, reference, props, rulesets, length, points, children)\n\t\t\t\t\t\t\telse\n\t\t\t\t\t\t\t\tswitch (atrule === 99 && (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.charat)(characters, 3) === 110 ? 100 : atrule) {\n\t\t\t\t\t\t\t\t\t// d l m s\n\t\t\t\t\t\t\t\t\tcase 100: case 108: case 109: case 115:\n\t\t\t\t\t\t\t\t\t\tparse(value, reference, reference, rule && (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.append)(ruleset(value, reference, reference, 0, 0, rules, points, type, rules, props = [], length), children), rules, children, length, points, rule ? props : children)\n\t\t\t\t\t\t\t\t\t\tbreak\n\t\t\t\t\t\t\t\t\tdefault:\n\t\t\t\t\t\t\t\t\t\tparse(characters, reference, reference, reference, [''], children, 0, points, children)\n\t\t\t\t\t\t\t\t}\n\t\t\t\t}\n\n\t\t\t\tindex = offset = property = 0, variable = ampersand = 1, type = characters = '', length = pseudo\n\t\t\t\tbreak\n\t\t\t// :\n\t\t\tcase 58:\n\t\t\t\tlength = 1 + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.strlen)(characters), property = previous\n\t\t\tdefault:\n\t\t\t\tif (variable < 1)\n\t\t\t\t\tif (character == 123)\n\t\t\t\t\t\t--variable\n\t\t\t\t\telse if (character == 125 && variable++ == 0 && (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.prev)() == 125)\n\t\t\t\t\t\tcontinue\n\n\t\t\t\tswitch (characters += (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.from)(character), character * variable) {\n\t\t\t\t\t// &\n\t\t\t\t\tcase 38:\n\t\t\t\t\t\tampersand = offset > 0 ? 1 : (characters += '\\f', -1)\n\t\t\t\t\t\tbreak\n\t\t\t\t\t// ,\n\t\t\t\t\tcase 44:\n\t\t\t\t\t\tpoints[index++] = ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.strlen)(characters) - 1) * ampersand, ampersand = 1\n\t\t\t\t\t\tbreak\n\t\t\t\t\t// @\n\t\t\t\t\tcase 64:\n\t\t\t\t\t\t// -\n\t\t\t\t\t\tif ((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.peek)() === 45)\n\t\t\t\t\t\t\tcharacters += (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.delimit)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.next)())\n\n\t\t\t\t\t\tatrule = (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.peek)(), offset = length = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.strlen)(type = characters += (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.identifier)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.caret)())), character++\n\t\t\t\t\t\tbreak\n\t\t\t\t\t// -\n\t\t\t\t\tcase 45:\n\t\t\t\t\t\tif (previous === 45 && (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.strlen)(characters) == 2)\n\t\t\t\t\t\t\tvariable = 0\n\t\t\t\t}\n\t\t}\n\n\treturn rulesets\n}\n\n/**\n * @param {string} value\n * @param {object} root\n * @param {object?} parent\n * @param {number} index\n * @param {number} offset\n * @param {string[]} rules\n * @param {number[]} points\n * @param {string} type\n * @param {string[]} props\n * @param {string[]} children\n * @param {number} length\n * @return {object}\n */\nfunction ruleset (value, root, parent, index, offset, rules, points, type, props, children, length) {\n\tvar post = offset - 1\n\tvar rule = offset === 0 ? rules : ['']\n\tvar size = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.sizeof)(rule)\n\n\tfor (var i = 0, j = 0, k = 0; i < index; ++i)\n\t\tfor (var x = 0, y = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.substr)(value, post + 1, post = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.abs)(j = points[i])), z = value; x < size; ++x)\n\t\t\tif (z = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.trim)(j > 0 ? rule[x] + ' ' + y : (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.replace)(y, /&\\f/g, rule[x])))\n\t\t\t\tprops[k++] = z\n\n\treturn (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.node)(value, root, parent, offset === 0 ? _Enum_js__WEBPACK_IMPORTED_MODULE_2__.RULESET : type, props, children, length)\n}\n\n/**\n * @param {number} value\n * @param {object} root\n * @param {object?} parent\n * @return {object}\n */\nfunction comment (value, root, parent) {\n\treturn (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.node)(value, root, parent, _Enum_js__WEBPACK_IMPORTED_MODULE_2__.COMMENT, (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.from)((0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.char)()), (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.substr)(value, 2, -2), 0)\n}\n\n/**\n * @param {string} value\n * @param {object} root\n * @param {object?} parent\n * @param {number} length\n * @return {object}\n */\nfunction declaration (value, root, parent, length) {\n\treturn (0,_Tokenizer_js__WEBPACK_IMPORTED_MODULE_0__.node)(value, root, parent, _Enum_js__WEBPACK_IMPORTED_MODULE_2__.DECLARATION, (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.substr)(value, 0, length), (0,_Utility_js__WEBPACK_IMPORTED_MODULE_1__.substr)(value, length + 1, -1), length)\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Parser.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Prefixer.js":
/*!*********************************************!*\
  !*** ./node_modules/stylis/src/Prefixer.js ***!
  \*********************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   prefix: () => (/* binding */ prefix)\n/* harmony export */ });\n/* harmony import */ var _Enum_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Enum.js */ \"./node_modules/stylis/src/Enum.js\");\n/* harmony import */ var _Utility_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Utility.js */ \"./node_modules/stylis/src/Utility.js\");\n\n\n\n/**\n * @param {string} value\n * @param {number} length\n * @param {object[]} children\n * @return {string}\n */\nfunction prefix (value, length, children) {\n\tswitch ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.hash)(value, length)) {\n\t\t// color-adjust\n\t\tcase 5103:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + 'print-' + value + value\n\t\t// animation, animation-(delay|direction|duration|fill-mode|iteration-count|name|play-state|timing-function)\n\t\tcase 5737: case 4201: case 3177: case 3433: case 1641: case 4457: case 2921:\n\t\t// text-decoration, filter, clip-path, backface-visibility, column, box-decoration-break\n\t\tcase 5572: case 6356: case 5844: case 3191: case 6645: case 3005:\n\t\t// mask, mask-image, mask-(mode|clip|size), mask-(repeat|origin), mask-position, mask-composite,\n\t\tcase 6391: case 5879: case 5623: case 6135: case 4599: case 4855:\n\t\t// background-clip, columns, column-(count|fill|gap|rule|rule-color|rule-style|rule-width|span|width)\n\t\tcase 4215: case 6389: case 5109: case 5365: case 5621: case 3829:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + value\n\t\t// tab-size\n\t\tcase 4789:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MOZ + value + value\n\t\t// appearance, user-select, transform, hyphens, text-size-adjust\n\t\tcase 5349: case 4246: case 4810: case 6968: case 2756:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MOZ + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + value + value\n\t\t// writing-mode\n\t\tcase 5936:\n\t\t\tswitch ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, length + 11)) {\n\t\t\t\t// vertical-l(r)\n\t\t\t\tcase 114:\n\t\t\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /[svh]\\w+-[tblr]{2}/, 'tb') + value\n\t\t\t\t// vertical-r(l)\n\t\t\t\tcase 108:\n\t\t\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /[svh]\\w+-[tblr]{2}/, 'tb-rl') + value\n\t\t\t\t// horizontal(-)tb\n\t\t\t\tcase 45:\n\t\t\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /[svh]\\w+-[tblr]{2}/, 'lr') + value\n\t\t\t\t// default: fallthrough to below\n\t\t\t}\n\t\t// flex, flex-direction, scroll-snap-type, writing-mode\n\t\tcase 6828: case 4268: case 2903:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + value + value\n\t\t// order\n\t\tcase 6165:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'flex-' + value + value\n\t\t// align-items\n\t\tcase 5187:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(\\w+).+(:[^]+)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + 'box-$1$2' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'flex-$1$2') + value\n\t\t// align-self\n\t\tcase 5443:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'flex-item-' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /flex-|-self/g, '') + (!(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(value, /flex-|baseline/) ? _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'grid-row-' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /flex-|-self/g, '') : '') + value\n\t\t// align-content\n\t\tcase 4675:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'flex-line-pack' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /align-content|flex-|-self/g, '') + value\n\t\t// flex-shrink\n\t\tcase 5548:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, 'shrink', 'negative') + value\n\t\t// flex-basis\n\t\tcase 5292:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, 'basis', 'preferred-size') + value\n\t\t// flex-grow\n\t\tcase 6060:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + 'box-' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, '-grow', '') + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, 'grow', 'positive') + value\n\t\t// transition\n\t\tcase 4554:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /([^-])(transform)/g, '$1' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$2') + value\n\t\t// cursor\n\t\tcase 6187:\n\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(zoom-|grab)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$1'), /(image-set)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$1'), value, '') + value\n\t\t// background, background-image\n\t\tcase 5495: case 3959:\n\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(image-set\\([^]*)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$1' + '$`$1')\n\t\t// justify-content\n\t\tcase 4968:\n\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(.+:)(flex-)?(.*)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + 'box-pack:$3' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'flex-pack:$3'), /s.+-b[^;]+/, 'justify') + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + value + value\n\t\t// justify-self\n\t\tcase 4200:\n\t\t\tif (!(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(value, /flex-|baseline/)) return _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'grid-column-align' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.substr)(value, length) + value\n\t\t\tbreak\n\t\t// grid-template-(columns|rows)\n\t\tcase 2592: case 3360:\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, 'template-', '') + value\n\t\t// grid-(row|column)-start\n\t\tcase 4384: case 3616:\n\t\t\tif (children && children.some(function (element, index) { return length = index, (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(element.props, /grid-\\w+-end/) })) {\n\t\t\t\treturn ~(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.indexof)(value + (children = children[length].value), 'span') ? value : (_Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, '-start', '') + value + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + 'grid-row-span:' + (~(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.indexof)(children, 'span') ? (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(children, /\\d+/) : +(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(children, /\\d+/) - +(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(value, /\\d+/)) + ';')\n\t\t\t}\n\t\t\treturn _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, '-start', '') + value\n\t\t// grid-(row|column)-end\n\t\tcase 4896: case 4128:\n\t\t\treturn (children && children.some(function (element) { return (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.match)(element.props, /grid-\\w+-start/) })) ? value : _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, '-end', '-span'), 'span ', '') + value\n\t\t// (margin|padding)-inline-(start|end)\n\t\tcase 4095: case 3583: case 4068: case 2532:\n\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(.+)-inline(.+)/, _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$1$2') + value\n\t\t// (min|max)?(width|height|inline-size|block-size)\n\t\tcase 8116: case 7059: case 5753: case 5535:\n\t\tcase 5445: case 5701: case 4933: case 4677:\n\t\tcase 5533: case 5789: case 5021: case 4765:\n\t\t\t// stretch, max-content, min-content, fill-available\n\t\t\tif ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.strlen)(value) - 1 - length > 6)\n\t\t\t\tswitch ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, length + 1)) {\n\t\t\t\t\t// (m)ax-content, (m)in-content\n\t\t\t\t\tcase 109:\n\t\t\t\t\t\t// -\n\t\t\t\t\t\tif ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, length + 4) !== 45)\n\t\t\t\t\t\t\tbreak\n\t\t\t\t\t// (f)ill-available, (f)it-content\n\t\t\t\t\tcase 102:\n\t\t\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(.+:)(.+)-([^]+)/, '$1' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$2-$3' + '$1' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MOZ + ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, length + 3) == 108 ? '$3' : '$2-$3')) + value\n\t\t\t\t\t// (s)tretch\n\t\t\t\t\tcase 115:\n\t\t\t\t\t\treturn ~(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.indexof)(value, 'stretch') ? prefix((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, 'stretch', 'fill-available'), length, children) + value : value\n\t\t\t\t}\n\t\t\tbreak\n\t\t// grid-(column|row)\n\t\tcase 5152: case 5920:\n\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(.+?):(\\d+)(\\s*\\/\\s*(span)?\\s*(\\d+))?(.*)/, function (_, a, b, c, d, e, f) { return (_Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + a + ':' + b + f) + (c ? (_Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + a + '-span:' + (d ? e : +e - +b)) + f : '') + value })\n\t\t// position: sticky\n\t\tcase 4949:\n\t\t\t// stick(y)?\n\t\t\tif ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, length + 6) === 121)\n\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, ':', ':' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT) + value\n\t\t\tbreak\n\t\t// display: (flex|inline-flex|grid|inline-grid)\n\t\tcase 6444:\n\t\t\tswitch ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, 14) === 45 ? 18 : 11)) {\n\t\t\t\t// (inline-)?fle(x)\n\t\t\t\tcase 120:\n\t\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, /(.+:)([^;\\s!]+)(;|(\\s+)?!.+)?/, '$1' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + ((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(value, 14) === 45 ? 'inline-' : '') + 'box$3' + '$1' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.WEBKIT + '$2$3' + '$1' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS + '$2box$3') + value\n\t\t\t\t// (inline-)?gri(d)\n\t\t\t\tcase 100:\n\t\t\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, ':', ':' + _Enum_js__WEBPACK_IMPORTED_MODULE_1__.MS) + value\n\t\t\t}\n\t\t\tbreak\n\t\t// scroll-margin, scroll-margin-(top|right|bottom|left)\n\t\tcase 5719: case 2647: case 2135: case 3927: case 2391:\n\t\t\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.replace)(value, 'scroll-', 'scroll-snap-') + value\n\t}\n\n\treturn value\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Prefixer.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Serializer.js":
/*!***********************************************!*\
  !*** ./node_modules/stylis/src/Serializer.js ***!
  \***********************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   serialize: () => (/* binding */ serialize),\n/* harmony export */   stringify: () => (/* binding */ stringify)\n/* harmony export */ });\n/* harmony import */ var _Enum_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./Enum.js */ \"./node_modules/stylis/src/Enum.js\");\n/* harmony import */ var _Utility_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Utility.js */ \"./node_modules/stylis/src/Utility.js\");\n\n\n\n/**\n * @param {object[]} children\n * @param {function} callback\n * @return {string}\n */\nfunction serialize (children, callback) {\n\tvar output = ''\n\tvar length = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.sizeof)(children)\n\n\tfor (var i = 0; i < length; i++)\n\t\toutput += callback(children[i], i, children, callback) || ''\n\n\treturn output\n}\n\n/**\n * @param {object} element\n * @param {number} index\n * @param {object[]} children\n * @param {function} callback\n * @return {string}\n */\nfunction stringify (element, index, children, callback) {\n\tswitch (element.type) {\n\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.LAYER: if (element.children.length) break\n\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.IMPORT: case _Enum_js__WEBPACK_IMPORTED_MODULE_1__.DECLARATION: return element.return = element.return || element.value\n\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.COMMENT: return ''\n\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.KEYFRAMES: return element.return = element.value + '{' + serialize(element.children, callback) + '}'\n\t\tcase _Enum_js__WEBPACK_IMPORTED_MODULE_1__.RULESET: element.value = element.props.join(',')\n\t}\n\n\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.strlen)(children = serialize(element.children, callback)) ? element.return = element.value + '{' + children + '}' : ''\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Serializer.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Tokenizer.js":
/*!**********************************************!*\
  !*** ./node_modules/stylis/src/Tokenizer.js ***!
  \**********************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   alloc: () => (/* binding */ alloc),\n/* harmony export */   caret: () => (/* binding */ caret),\n/* harmony export */   char: () => (/* binding */ char),\n/* harmony export */   character: () => (/* binding */ character),\n/* harmony export */   characters: () => (/* binding */ characters),\n/* harmony export */   column: () => (/* binding */ column),\n/* harmony export */   commenter: () => (/* binding */ commenter),\n/* harmony export */   copy: () => (/* binding */ copy),\n/* harmony export */   dealloc: () => (/* binding */ dealloc),\n/* harmony export */   delimit: () => (/* binding */ delimit),\n/* harmony export */   delimiter: () => (/* binding */ delimiter),\n/* harmony export */   escaping: () => (/* binding */ escaping),\n/* harmony export */   identifier: () => (/* binding */ identifier),\n/* harmony export */   length: () => (/* binding */ length),\n/* harmony export */   line: () => (/* binding */ line),\n/* harmony export */   next: () => (/* binding */ next),\n/* harmony export */   node: () => (/* binding */ node),\n/* harmony export */   peek: () => (/* binding */ peek),\n/* harmony export */   position: () => (/* binding */ position),\n/* harmony export */   prev: () => (/* binding */ prev),\n/* harmony export */   slice: () => (/* binding */ slice),\n/* harmony export */   token: () => (/* binding */ token),\n/* harmony export */   tokenize: () => (/* binding */ tokenize),\n/* harmony export */   tokenizer: () => (/* binding */ tokenizer),\n/* harmony export */   whitespace: () => (/* binding */ whitespace)\n/* harmony export */ });\n/* harmony import */ var _Utility_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./Utility.js */ \"./node_modules/stylis/src/Utility.js\");\n\n\nvar line = 1\nvar column = 1\nvar length = 0\nvar position = 0\nvar character = 0\nvar characters = ''\n\n/**\n * @param {string} value\n * @param {object | null} root\n * @param {object | null} parent\n * @param {string} type\n * @param {string[] | string} props\n * @param {object[] | string} children\n * @param {number} length\n */\nfunction node (value, root, parent, type, props, children, length) {\n\treturn {value: value, root: root, parent: parent, type: type, props: props, children: children, line: line, column: column, length: length, return: ''}\n}\n\n/**\n * @param {object} root\n * @param {object} props\n * @return {object}\n */\nfunction copy (root, props) {\n\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.assign)(node('', null, null, '', null, null, 0), root, {length: -root.length}, props)\n}\n\n/**\n * @return {number}\n */\nfunction char () {\n\treturn character\n}\n\n/**\n * @return {number}\n */\nfunction prev () {\n\tcharacter = position > 0 ? (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(characters, --position) : 0\n\n\tif (column--, character === 10)\n\t\tcolumn = 1, line--\n\n\treturn character\n}\n\n/**\n * @return {number}\n */\nfunction next () {\n\tcharacter = position < length ? (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(characters, position++) : 0\n\n\tif (column++, character === 10)\n\t\tcolumn = 1, line++\n\n\treturn character\n}\n\n/**\n * @return {number}\n */\nfunction peek () {\n\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.charat)(characters, position)\n}\n\n/**\n * @return {number}\n */\nfunction caret () {\n\treturn position\n}\n\n/**\n * @param {number} begin\n * @param {number} end\n * @return {string}\n */\nfunction slice (begin, end) {\n\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.substr)(characters, begin, end)\n}\n\n/**\n * @param {number} type\n * @return {number}\n */\nfunction token (type) {\n\tswitch (type) {\n\t\t// \\0 \\t \\n \\r \\s whitespace token\n\t\tcase 0: case 9: case 10: case 13: case 32:\n\t\t\treturn 5\n\t\t// ! + , / > @ ~ isolate token\n\t\tcase 33: case 43: case 44: case 47: case 62: case 64: case 126:\n\t\t// ; { } breakpoint token\n\t\tcase 59: case 123: case 125:\n\t\t\treturn 4\n\t\t// : accompanied token\n\t\tcase 58:\n\t\t\treturn 3\n\t\t// \" ' ( [ opening delimit token\n\t\tcase 34: case 39: case 40: case 91:\n\t\t\treturn 2\n\t\t// ) ] closing delimit token\n\t\tcase 41: case 93:\n\t\t\treturn 1\n\t}\n\n\treturn 0\n}\n\n/**\n * @param {string} value\n * @return {any[]}\n */\nfunction alloc (value) {\n\treturn line = column = 1, length = (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.strlen)(characters = value), position = 0, []\n}\n\n/**\n * @param {any} value\n * @return {any}\n */\nfunction dealloc (value) {\n\treturn characters = '', value\n}\n\n/**\n * @param {number} type\n * @return {string}\n */\nfunction delimit (type) {\n\treturn (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.trim)(slice(position - 1, delimiter(type === 91 ? type + 2 : type === 40 ? type + 1 : type)))\n}\n\n/**\n * @param {string} value\n * @return {string[]}\n */\nfunction tokenize (value) {\n\treturn dealloc(tokenizer(alloc(value)))\n}\n\n/**\n * @param {number} type\n * @return {string}\n */\nfunction whitespace (type) {\n\twhile (character = peek())\n\t\tif (character < 33)\n\t\t\tnext()\n\t\telse\n\t\t\tbreak\n\n\treturn token(type) > 2 || token(character) > 3 ? '' : ' '\n}\n\n/**\n * @param {string[]} children\n * @return {string[]}\n */\nfunction tokenizer (children) {\n\twhile (next())\n\t\tswitch (token(character)) {\n\t\t\tcase 0: (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.append)(identifier(position - 1), children)\n\t\t\t\tbreak\n\t\t\tcase 2: ;(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.append)(delimit(character), children)\n\t\t\t\tbreak\n\t\t\tdefault: ;(0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.append)((0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.from)(character), children)\n\t\t}\n\n\treturn children\n}\n\n/**\n * @param {number} index\n * @param {number} count\n * @return {string}\n */\nfunction escaping (index, count) {\n\twhile (--count && next())\n\t\t// not 0-9 A-F a-f\n\t\tif (character < 48 || character > 102 || (character > 57 && character < 65) || (character > 70 && character < 97))\n\t\t\tbreak\n\n\treturn slice(index, caret() + (count < 6 && peek() == 32 && next() == 32))\n}\n\n/**\n * @param {number} type\n * @return {number}\n */\nfunction delimiter (type) {\n\twhile (next())\n\t\tswitch (character) {\n\t\t\t// ] ) \" '\n\t\t\tcase type:\n\t\t\t\treturn position\n\t\t\t// \" '\n\t\t\tcase 34: case 39:\n\t\t\t\tif (type !== 34 && type !== 39)\n\t\t\t\t\tdelimiter(character)\n\t\t\t\tbreak\n\t\t\t// (\n\t\t\tcase 40:\n\t\t\t\tif (type === 41)\n\t\t\t\t\tdelimiter(type)\n\t\t\t\tbreak\n\t\t\t// \\\n\t\t\tcase 92:\n\t\t\t\tnext()\n\t\t\t\tbreak\n\t\t}\n\n\treturn position\n}\n\n/**\n * @param {number} type\n * @param {number} index\n * @return {number}\n */\nfunction commenter (type, index) {\n\twhile (next())\n\t\t// //\n\t\tif (type + character === 47 + 10)\n\t\t\tbreak\n\t\t// /*\n\t\telse if (type + character === 42 + 42 && peek() === 47)\n\t\t\tbreak\n\n\treturn '/*' + slice(index, position - 1) + '*' + (0,_Utility_js__WEBPACK_IMPORTED_MODULE_0__.from)(type === 47 ? type : next())\n}\n\n/**\n * @param {number} index\n * @return {string}\n */\nfunction identifier (index) {\n\twhile (!token(peek()))\n\t\tnext()\n\n\treturn slice(index, position)\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Tokenizer.js?");

/***/ }),

/***/ "./node_modules/stylis/src/Utility.js":
/*!********************************************!*\
  !*** ./node_modules/stylis/src/Utility.js ***!
  \********************************************/
/***/ ((__unused_webpack___webpack_module__, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   abs: () => (/* binding */ abs),\n/* harmony export */   append: () => (/* binding */ append),\n/* harmony export */   assign: () => (/* binding */ assign),\n/* harmony export */   charat: () => (/* binding */ charat),\n/* harmony export */   combine: () => (/* binding */ combine),\n/* harmony export */   from: () => (/* binding */ from),\n/* harmony export */   hash: () => (/* binding */ hash),\n/* harmony export */   indexof: () => (/* binding */ indexof),\n/* harmony export */   match: () => (/* binding */ match),\n/* harmony export */   replace: () => (/* binding */ replace),\n/* harmony export */   sizeof: () => (/* binding */ sizeof),\n/* harmony export */   strlen: () => (/* binding */ strlen),\n/* harmony export */   substr: () => (/* binding */ substr),\n/* harmony export */   trim: () => (/* binding */ trim)\n/* harmony export */ });\n/**\n * @param {number}\n * @return {number}\n */\nvar abs = Math.abs\n\n/**\n * @param {number}\n * @return {string}\n */\nvar from = String.fromCharCode\n\n/**\n * @param {object}\n * @return {object}\n */\nvar assign = Object.assign\n\n/**\n * @param {string} value\n * @param {number} length\n * @return {number}\n */\nfunction hash (value, length) {\n\treturn charat(value, 0) ^ 45 ? (((((((length << 2) ^ charat(value, 0)) << 2) ^ charat(value, 1)) << 2) ^ charat(value, 2)) << 2) ^ charat(value, 3) : 0\n}\n\n/**\n * @param {string} value\n * @return {string}\n */\nfunction trim (value) {\n\treturn value.trim()\n}\n\n/**\n * @param {string} value\n * @param {RegExp} pattern\n * @return {string?}\n */\nfunction match (value, pattern) {\n\treturn (value = pattern.exec(value)) ? value[0] : value\n}\n\n/**\n * @param {string} value\n * @param {(string|RegExp)} pattern\n * @param {string} replacement\n * @return {string}\n */\nfunction replace (value, pattern, replacement) {\n\treturn value.replace(pattern, replacement)\n}\n\n/**\n * @param {string} value\n * @param {string} search\n * @return {number}\n */\nfunction indexof (value, search) {\n\treturn value.indexOf(search)\n}\n\n/**\n * @param {string} value\n * @param {number} index\n * @return {number}\n */\nfunction charat (value, index) {\n\treturn value.charCodeAt(index) | 0\n}\n\n/**\n * @param {string} value\n * @param {number} begin\n * @param {number} end\n * @return {string}\n */\nfunction substr (value, begin, end) {\n\treturn value.slice(begin, end)\n}\n\n/**\n * @param {string} value\n * @return {number}\n */\nfunction strlen (value) {\n\treturn value.length\n}\n\n/**\n * @param {any[]} value\n * @return {number}\n */\nfunction sizeof (value) {\n\treturn value.length\n}\n\n/**\n * @param {any} value\n * @param {any[]} array\n * @return {any}\n */\nfunction append (value, array) {\n\treturn array.push(value), value\n}\n\n/**\n * @param {string[]} array\n * @param {function} callback\n * @return {string}\n */\nfunction combine (array, callback) {\n\treturn array.map(callback).join('')\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./node_modules/stylis/src/Utility.js?");

/***/ }),

/***/ "./src/bindings/package_info/v1/package_info_pb.js":
/*!*********************************************************!*\
  !*** ./src/bindings/package_info/v1/package_info_pb.js ***!
  \*********************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("// source: package_info/v1/package_info.proto\n/**\n * @fileoverview\n * @enhanceable\n * @suppress {missingRequire} reports error on implicit type usages.\n * @suppress {messageConventions} JS Compiler reports an error if a variable or\n *     field starts with 'MSG_' and isn't a translatable message.\n * @public\n */\n// GENERATED CODE -- DO NOT EDIT!\n/* eslint-disable */\n// @ts-nocheck\n\nvar jspb = __webpack_require__(/*! google-protobuf */ \"./node_modules/google-protobuf/google-protobuf.js\");\nvar goog = jspb;\nvar global = (function() {\n  if (this) { return this; }\n  if (typeof window !== 'undefined') { return window; }\n  if (typeof global !== 'undefined') { return global; }\n  if (typeof self !== 'undefined') { return self; }\n  return Function('return this')();\n}.call(null));\n\nvar google_protobuf_descriptor_pb = __webpack_require__(/*! google-protobuf/google/protobuf/descriptor_pb.js */ \"./node_modules/google-protobuf/google/protobuf/descriptor_pb.js\");\ngoog.object.extend(proto, google_protobuf_descriptor_pb);\ngoog.exportSymbol('proto.package_info.v1.packageName', null, global);\n\n/**\n * A tuple of {field number, class constructor} for the extension\n * field named `packageName`.\n * @type {!jspb.ExtensionFieldInfo<string>}\n */\nproto.package_info.v1.packageName = new jspb.ExtensionFieldInfo(\n    51234,\n    {packageName: 0},\n    null,\n     /** @type {?function((boolean|undefined),!jspb.Message=): !Object} */ (\n         null),\n    0);\n\ngoogle_protobuf_descriptor_pb.MessageOptions.extensionsBinary[51234] = new jspb.ExtensionFieldBinaryInfo(\n    proto.package_info.v1.packageName,\n    jspb.BinaryReader.prototype.readString,\n    jspb.BinaryWriter.prototype.writeString,\n    undefined,\n    undefined,\n    false);\n// This registers the extension field with the extended class, so that\n// toObject() will function correctly.\ngoogle_protobuf_descriptor_pb.MessageOptions.extensions[51234] = proto.package_info.v1.packageName;\n\ngoog.object.extend(exports, proto.package_info.v1);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/bindings/package_info/v1/package_info_pb.js?");

/***/ }),

/***/ "./src/bindings/store/v1/StoreServiceClientPb.ts":
/*!*******************************************************!*\
  !*** ./src/bindings/store/v1/StoreServiceClientPb.ts ***!
  \*******************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   StoreServiceClient: () => (/* binding */ StoreServiceClient)\n/* harmony export */ });\n/* harmony import */ var grpc_web__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! grpc-web */ \"./node_modules/grpc-web/index.js\");\n/* harmony import */ var grpc_web__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(grpc_web__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../../store/v1/store_pb */ \"./src/bindings/store/v1/store_pb.js\");\n/* harmony import */ var _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__);\n/**\n * @fileoverview gRPC-Web generated client stub for store.v1\n * @enhanceable\n * @public\n */\n// Code generated by protoc-gen-grpc-web. DO NOT EDIT.\n// versions:\n// \tprotoc-gen-grpc-web v1.5.0\n// \tprotoc              v3.19.1\n// source: store/v1/store.proto\n/* eslint-disable */\n// @ts-nocheck\n\n // proto import: \"store/v1/store.proto\"\nclass StoreServiceClient {\n    client_;\n    hostname_;\n    credentials_;\n    options_;\n    constructor(hostname, credentials, options) {\n        if (!options)\n            options = {};\n        if (!credentials)\n            credentials = {};\n        options['format'] = 'text';\n        this.client_ = new grpc_web__WEBPACK_IMPORTED_MODULE_0__.GrpcWebClientBase(options);\n        this.hostname_ = hostname.replace(/\\/+$/, '');\n        this.credentials_ = credentials;\n        this.options_ = options;\n    }\n    methodDescriptorDispatchAction = new grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodDescriptor('/store.v1.StoreService/DispatchAction', grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodType.UNARY, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.DispatchActionRequest, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.DispatchActionResponse, (request) => {\n        return request.serializeBinary();\n    }, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.DispatchActionResponse.deserializeBinary);\n    dispatchAction(request, metadata, callback) {\n        if (callback !== undefined) {\n            return this.client_.rpcCall(this.hostname_ +\n                '/store.v1.StoreService/DispatchAction', request, metadata || {}, this.methodDescriptorDispatchAction, callback);\n        }\n        return this.client_.unaryCall(this.hostname_ +\n            '/store.v1.StoreService/DispatchAction', request, metadata || {}, this.methodDescriptorDispatchAction);\n    }\n    methodDescriptorDispatchEvent = new grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodDescriptor('/store.v1.StoreService/DispatchEvent', grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodType.UNARY, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.DispatchEventRequest, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.DispatchEventResponse, (request) => {\n        return request.serializeBinary();\n    }, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.DispatchEventResponse.deserializeBinary);\n    dispatchEvent(request, metadata, callback) {\n        if (callback !== undefined) {\n            return this.client_.rpcCall(this.hostname_ +\n                '/store.v1.StoreService/DispatchEvent', request, metadata || {}, this.methodDescriptorDispatchEvent, callback);\n        }\n        return this.client_.unaryCall(this.hostname_ +\n            '/store.v1.StoreService/DispatchEvent', request, metadata || {}, this.methodDescriptorDispatchEvent);\n    }\n    methodDescriptorSubscribeEvent = new grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodDescriptor('/store.v1.StoreService/SubscribeEvent', grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodType.SERVER_STREAMING, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.SubscribeEventRequest, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.SubscribeEventResponse, (request) => {\n        return request.serializeBinary();\n    }, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.SubscribeEventResponse.deserializeBinary);\n    subscribeEvent(request, metadata) {\n        return this.client_.serverStreaming(this.hostname_ +\n            '/store.v1.StoreService/SubscribeEvent', request, metadata || {}, this.methodDescriptorSubscribeEvent);\n    }\n    methodDescriptorSubscribeStore = new grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodDescriptor('/store.v1.StoreService/SubscribeStore', grpc_web__WEBPACK_IMPORTED_MODULE_0__.MethodType.SERVER_STREAMING, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.SubscribeStoreRequest, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.SubscribeStoreResponse, (request) => {\n        return request.serializeBinary();\n    }, _store_v1_store_pb__WEBPACK_IMPORTED_MODULE_1__.SubscribeStoreResponse.deserializeBinary);\n    subscribeStore(request, metadata) {\n        return this.client_.serverStreaming(this.hostname_ +\n            '/store.v1.StoreService/SubscribeStore', request, metadata || {}, this.methodDescriptorSubscribeStore);\n    }\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/bindings/store/v1/StoreServiceClientPb.ts?");

/***/ }),

/***/ "./src/bindings/store/v1/store_pb.js":
/*!*******************************************!*\
  !*** ./src/bindings/store/v1/store_pb.js ***!
  \*******************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("// source: store/v1/store.proto\n/**\n * @fileoverview\n * @enhanceable\n * @suppress {missingRequire} reports error on implicit type usages.\n * @suppress {messageConventions} JS Compiler reports an error if a variable or\n *     field starts with 'MSG_' and isn't a translatable message.\n * @public\n */\n// GENERATED CODE -- DO NOT EDIT!\n/* eslint-disable */\n// @ts-nocheck\n\nvar jspb = __webpack_require__(/*! google-protobuf */ \"./node_modules/google-protobuf/google-protobuf.js\");\nvar goog = jspb;\nvar global = (function() {\n  if (this) { return this; }\n  if (typeof window !== 'undefined') { return window; }\n  if (typeof global !== 'undefined') { return global; }\n  if (typeof self !== 'undefined') { return self; }\n  return Function('return this')();\n}.call(null));\n\nvar google_protobuf_any_pb = __webpack_require__(/*! google-protobuf/google/protobuf/any_pb.js */ \"./node_modules/google-protobuf/google/protobuf/any_pb.js\");\ngoog.object.extend(proto, google_protobuf_any_pb);\nvar ubo_v1_ubo_pb = __webpack_require__(/*! ../../ubo/v1/ubo_pb.js */ \"./src/bindings/ubo/v1/ubo_pb.js\");\ngoog.object.extend(proto, ubo_v1_ubo_pb);\ngoog.exportSymbol('proto.store.v1.DispatchActionRequest', null, global);\ngoog.exportSymbol('proto.store.v1.DispatchActionResponse', null, global);\ngoog.exportSymbol('proto.store.v1.DispatchEventRequest', null, global);\ngoog.exportSymbol('proto.store.v1.DispatchEventResponse', null, global);\ngoog.exportSymbol('proto.store.v1.SubscribeEventRequest', null, global);\ngoog.exportSymbol('proto.store.v1.SubscribeEventResponse', null, global);\ngoog.exportSymbol('proto.store.v1.SubscribeStoreRequest', null, global);\ngoog.exportSymbol('proto.store.v1.SubscribeStoreResponse', null, global);\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.DispatchActionRequest = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.store.v1.DispatchActionRequest, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.DispatchActionRequest.displayName = 'proto.store.v1.DispatchActionRequest';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.DispatchEventRequest = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.store.v1.DispatchEventRequest, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.DispatchEventRequest.displayName = 'proto.store.v1.DispatchEventRequest';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.DispatchActionResponse = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.store.v1.DispatchActionResponse, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.DispatchActionResponse.displayName = 'proto.store.v1.DispatchActionResponse';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.DispatchEventResponse = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.store.v1.DispatchEventResponse, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.DispatchEventResponse.displayName = 'proto.store.v1.DispatchEventResponse';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.SubscribeEventRequest = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.store.v1.SubscribeEventRequest, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.SubscribeEventRequest.displayName = 'proto.store.v1.SubscribeEventRequest';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.SubscribeEventResponse = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.store.v1.SubscribeEventResponse, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.SubscribeEventResponse.displayName = 'proto.store.v1.SubscribeEventResponse';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.SubscribeStoreRequest = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.store.v1.SubscribeStoreRequest.repeatedFields_, null);\n};\ngoog.inherits(proto.store.v1.SubscribeStoreRequest, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.SubscribeStoreRequest.displayName = 'proto.store.v1.SubscribeStoreRequest';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.store.v1.SubscribeStoreResponse = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.store.v1.SubscribeStoreResponse.repeatedFields_, null);\n};\ngoog.inherits(proto.store.v1.SubscribeStoreResponse, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.store.v1.SubscribeStoreResponse.displayName = 'proto.store.v1.SubscribeStoreResponse';\n}\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.DispatchActionRequest.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.DispatchActionRequest.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.DispatchActionRequest} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchActionRequest.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    action: (f = msg.getAction()) && ubo_v1_ubo_pb.Action.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.DispatchActionRequest}\n */\nproto.store.v1.DispatchActionRequest.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.DispatchActionRequest;\n  return proto.store.v1.DispatchActionRequest.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.DispatchActionRequest} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.DispatchActionRequest}\n */\nproto.store.v1.DispatchActionRequest.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new ubo_v1_ubo_pb.Action;\n      reader.readMessage(value,ubo_v1_ubo_pb.Action.deserializeBinaryFromReader);\n      msg.setAction(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.DispatchActionRequest.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.DispatchActionRequest.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.DispatchActionRequest} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchActionRequest.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getAction();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      ubo_v1_ubo_pb.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional ubo.v1.Action action = 1;\n * @return {?proto.ubo.v1.Action}\n */\nproto.store.v1.DispatchActionRequest.prototype.getAction = function() {\n  return /** @type{?proto.ubo.v1.Action} */ (\n    jspb.Message.getWrapperField(this, ubo_v1_ubo_pb.Action, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Action|undefined} value\n * @return {!proto.store.v1.DispatchActionRequest} returns this\n*/\nproto.store.v1.DispatchActionRequest.prototype.setAction = function(value) {\n  return jspb.Message.setWrapperField(this, 1, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.store.v1.DispatchActionRequest} returns this\n */\nproto.store.v1.DispatchActionRequest.prototype.clearAction = function() {\n  return this.setAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.store.v1.DispatchActionRequest.prototype.hasAction = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.DispatchEventRequest.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.DispatchEventRequest.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.DispatchEventRequest} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchEventRequest.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    event: (f = msg.getEvent()) && ubo_v1_ubo_pb.Event.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.DispatchEventRequest}\n */\nproto.store.v1.DispatchEventRequest.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.DispatchEventRequest;\n  return proto.store.v1.DispatchEventRequest.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.DispatchEventRequest} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.DispatchEventRequest}\n */\nproto.store.v1.DispatchEventRequest.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new ubo_v1_ubo_pb.Event;\n      reader.readMessage(value,ubo_v1_ubo_pb.Event.deserializeBinaryFromReader);\n      msg.setEvent(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.DispatchEventRequest.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.DispatchEventRequest.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.DispatchEventRequest} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchEventRequest.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getEvent();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      ubo_v1_ubo_pb.Event.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional ubo.v1.Event event = 1;\n * @return {?proto.ubo.v1.Event}\n */\nproto.store.v1.DispatchEventRequest.prototype.getEvent = function() {\n  return /** @type{?proto.ubo.v1.Event} */ (\n    jspb.Message.getWrapperField(this, ubo_v1_ubo_pb.Event, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Event|undefined} value\n * @return {!proto.store.v1.DispatchEventRequest} returns this\n*/\nproto.store.v1.DispatchEventRequest.prototype.setEvent = function(value) {\n  return jspb.Message.setWrapperField(this, 1, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.store.v1.DispatchEventRequest} returns this\n */\nproto.store.v1.DispatchEventRequest.prototype.clearEvent = function() {\n  return this.setEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.store.v1.DispatchEventRequest.prototype.hasEvent = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.DispatchActionResponse.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.DispatchActionResponse.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.DispatchActionResponse} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchActionResponse.toObject = function(includeInstance, msg) {\n  var f, obj = {\n\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.DispatchActionResponse}\n */\nproto.store.v1.DispatchActionResponse.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.DispatchActionResponse;\n  return proto.store.v1.DispatchActionResponse.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.DispatchActionResponse} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.DispatchActionResponse}\n */\nproto.store.v1.DispatchActionResponse.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.DispatchActionResponse.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.DispatchActionResponse.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.DispatchActionResponse} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchActionResponse.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.DispatchEventResponse.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.DispatchEventResponse.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.DispatchEventResponse} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchEventResponse.toObject = function(includeInstance, msg) {\n  var f, obj = {\n\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.DispatchEventResponse}\n */\nproto.store.v1.DispatchEventResponse.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.DispatchEventResponse;\n  return proto.store.v1.DispatchEventResponse.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.DispatchEventResponse} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.DispatchEventResponse}\n */\nproto.store.v1.DispatchEventResponse.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.DispatchEventResponse.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.DispatchEventResponse.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.DispatchEventResponse} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.DispatchEventResponse.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.SubscribeEventRequest.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.SubscribeEventRequest.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.SubscribeEventRequest} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeEventRequest.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    event: (f = msg.getEvent()) && ubo_v1_ubo_pb.Event.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.SubscribeEventRequest}\n */\nproto.store.v1.SubscribeEventRequest.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.SubscribeEventRequest;\n  return proto.store.v1.SubscribeEventRequest.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.SubscribeEventRequest} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.SubscribeEventRequest}\n */\nproto.store.v1.SubscribeEventRequest.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new ubo_v1_ubo_pb.Event;\n      reader.readMessage(value,ubo_v1_ubo_pb.Event.deserializeBinaryFromReader);\n      msg.setEvent(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.SubscribeEventRequest.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.SubscribeEventRequest.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.SubscribeEventRequest} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeEventRequest.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getEvent();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      ubo_v1_ubo_pb.Event.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional ubo.v1.Event event = 1;\n * @return {?proto.ubo.v1.Event}\n */\nproto.store.v1.SubscribeEventRequest.prototype.getEvent = function() {\n  return /** @type{?proto.ubo.v1.Event} */ (\n    jspb.Message.getWrapperField(this, ubo_v1_ubo_pb.Event, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Event|undefined} value\n * @return {!proto.store.v1.SubscribeEventRequest} returns this\n*/\nproto.store.v1.SubscribeEventRequest.prototype.setEvent = function(value) {\n  return jspb.Message.setWrapperField(this, 1, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.store.v1.SubscribeEventRequest} returns this\n */\nproto.store.v1.SubscribeEventRequest.prototype.clearEvent = function() {\n  return this.setEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.store.v1.SubscribeEventRequest.prototype.hasEvent = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.SubscribeEventResponse.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.SubscribeEventResponse.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.SubscribeEventResponse} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeEventResponse.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    event: (f = msg.getEvent()) && ubo_v1_ubo_pb.Event.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.SubscribeEventResponse}\n */\nproto.store.v1.SubscribeEventResponse.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.SubscribeEventResponse;\n  return proto.store.v1.SubscribeEventResponse.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.SubscribeEventResponse} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.SubscribeEventResponse}\n */\nproto.store.v1.SubscribeEventResponse.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new ubo_v1_ubo_pb.Event;\n      reader.readMessage(value,ubo_v1_ubo_pb.Event.deserializeBinaryFromReader);\n      msg.setEvent(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.SubscribeEventResponse.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.SubscribeEventResponse.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.SubscribeEventResponse} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeEventResponse.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getEvent();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      ubo_v1_ubo_pb.Event.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional ubo.v1.Event event = 1;\n * @return {?proto.ubo.v1.Event}\n */\nproto.store.v1.SubscribeEventResponse.prototype.getEvent = function() {\n  return /** @type{?proto.ubo.v1.Event} */ (\n    jspb.Message.getWrapperField(this, ubo_v1_ubo_pb.Event, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Event|undefined} value\n * @return {!proto.store.v1.SubscribeEventResponse} returns this\n*/\nproto.store.v1.SubscribeEventResponse.prototype.setEvent = function(value) {\n  return jspb.Message.setWrapperField(this, 1, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.store.v1.SubscribeEventResponse} returns this\n */\nproto.store.v1.SubscribeEventResponse.prototype.clearEvent = function() {\n  return this.setEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.store.v1.SubscribeEventResponse.prototype.hasEvent = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.store.v1.SubscribeStoreRequest.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.SubscribeStoreRequest.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.SubscribeStoreRequest.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.SubscribeStoreRequest} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeStoreRequest.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    selectorsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.SubscribeStoreRequest}\n */\nproto.store.v1.SubscribeStoreRequest.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.SubscribeStoreRequest;\n  return proto.store.v1.SubscribeStoreRequest.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.SubscribeStoreRequest} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.SubscribeStoreRequest}\n */\nproto.store.v1.SubscribeStoreRequest.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addSelectors(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.SubscribeStoreRequest.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.SubscribeStoreRequest.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.SubscribeStoreRequest} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeStoreRequest.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getSelectorsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string selectors = 1;\n * @return {!Array<string>}\n */\nproto.store.v1.SubscribeStoreRequest.prototype.getSelectorsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.store.v1.SubscribeStoreRequest} returns this\n */\nproto.store.v1.SubscribeStoreRequest.prototype.setSelectorsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.store.v1.SubscribeStoreRequest} returns this\n */\nproto.store.v1.SubscribeStoreRequest.prototype.addSelectors = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.store.v1.SubscribeStoreRequest} returns this\n */\nproto.store.v1.SubscribeStoreRequest.prototype.clearSelectorsList = function() {\n  return this.setSelectorsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.store.v1.SubscribeStoreResponse.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.store.v1.SubscribeStoreResponse.prototype.toObject = function(opt_includeInstance) {\n  return proto.store.v1.SubscribeStoreResponse.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.store.v1.SubscribeStoreResponse} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeStoreResponse.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    resultsList: jspb.Message.toObjectList(msg.getResultsList(),\n    google_protobuf_any_pb.Any.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.store.v1.SubscribeStoreResponse}\n */\nproto.store.v1.SubscribeStoreResponse.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.store.v1.SubscribeStoreResponse;\n  return proto.store.v1.SubscribeStoreResponse.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.store.v1.SubscribeStoreResponse} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.store.v1.SubscribeStoreResponse}\n */\nproto.store.v1.SubscribeStoreResponse.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new google_protobuf_any_pb.Any;\n      reader.readMessage(value,google_protobuf_any_pb.Any.deserializeBinaryFromReader);\n      msg.addResults(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.store.v1.SubscribeStoreResponse.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.store.v1.SubscribeStoreResponse.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.store.v1.SubscribeStoreResponse} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.store.v1.SubscribeStoreResponse.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getResultsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      google_protobuf_any_pb.Any.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated google.protobuf.Any results = 1;\n * @return {!Array<!proto.google.protobuf.Any>}\n */\nproto.store.v1.SubscribeStoreResponse.prototype.getResultsList = function() {\n  return /** @type{!Array<!proto.google.protobuf.Any>} */ (\n    jspb.Message.getRepeatedWrapperField(this, google_protobuf_any_pb.Any, 1));\n};\n\n\n/**\n * @param {!Array<!proto.google.protobuf.Any>} value\n * @return {!proto.store.v1.SubscribeStoreResponse} returns this\n*/\nproto.store.v1.SubscribeStoreResponse.prototype.setResultsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.google.protobuf.Any=} opt_value\n * @param {number=} opt_index\n * @return {!proto.google.protobuf.Any}\n */\nproto.store.v1.SubscribeStoreResponse.prototype.addResults = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.google.protobuf.Any, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.store.v1.SubscribeStoreResponse} returns this\n */\nproto.store.v1.SubscribeStoreResponse.prototype.clearResultsList = function() {\n  return this.setResultsList([]);\n};\n\n\ngoog.object.extend(exports, proto.store.v1);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/bindings/store/v1/store_pb.js?");

/***/ }),

/***/ "./src/bindings/ubo/v1/ubo_pb.js":
/*!***************************************!*\
  !*** ./src/bindings/ubo/v1/ubo_pb.js ***!
  \***************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

eval("// source: ubo/v1/ubo.proto\n/**\n * @fileoverview\n * @enhanceable\n * @suppress {missingRequire} reports error on implicit type usages.\n * @suppress {messageConventions} JS Compiler reports an error if a variable or\n *     field starts with 'MSG_' and isn't a translatable message.\n * @public\n */\n// GENERATED CODE -- DO NOT EDIT!\n/* eslint-disable */\n// @ts-nocheck\n\nvar jspb = __webpack_require__(/*! google-protobuf */ \"./node_modules/google-protobuf/google-protobuf.js\");\nvar goog = jspb;\nvar global = (function() {\n  if (this) { return this; }\n  if (typeof window !== 'undefined') { return window; }\n  if (typeof global !== 'undefined') { return global; }\n  if (typeof self !== 'undefined') { return self; }\n  return Function('return this')();\n}.call(null));\n\nvar package_info_v1_package_info_pb = __webpack_require__(/*! ../../package_info/v1/package_info_pb.js */ \"./src/bindings/package_info/v1/package_info_pb.js\");\ngoog.object.extend(proto, package_info_v1_package_info_pb);\ngoog.exportSymbol('proto.ubo.v1.AcceptableAssistanceFrame', null, global);\ngoog.exportSymbol('proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.Action', null, global);\ngoog.exportSymbol('proto.ubo.v1.Action.ActionCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.ActionItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.ApplicationItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.ApplicationItem.Application', null, global);\ngoog.exportSymbol('proto.ubo.v1.ApplicationItem.Application.ApplicationCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistanceAudioFrame', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistanceFrame', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistanceImageFrame', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistanceTextFrame', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantAddMcpServerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantAddMcpServerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantDeleteMcpServerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantDeleteMcpServerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantDownloadOllamaModelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantDownloadOllamaModelEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantHandleReportEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantImageGeneratorName', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantLLMName', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantReportAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSTTName', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSetIsActiveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSetSelectedImageGeneratorAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSetSelectedLLMAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSetSelectedModelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSetSelectedSTTAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSetSelectedTTSAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantStartListeningAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantState', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantState.EnabledMcpServers', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantState.McpServersDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantState.ProviderSetupStatusDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantState.SelectedModelsDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantStopListeningAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantSyncMcpServersAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantTTSName', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantToggleListeningAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantToggleMcpServerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantUpdateProvidersAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AssistantUpdateProvidersEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioChangeVolumeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioDevice', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioInstallDriverAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioInstallDriverEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayAudioSampleAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayAudioSampleEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayAudioSequenceAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayAudioSequenceEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayChimeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayChimeEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlayRecordingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlaybackDoneAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioPlaybackDoneEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioReportSampleAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioReportSampleEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioSample', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioSetMuteStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioSetVolumeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioStartRecordingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioState', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioStopRecordingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioToggleMuteStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AudioToggleRecordingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.AvailableInputDescription', null, global);\ngoog.exportSymbol('proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.BaseMenu', null, global);\ngoog.exportSymbol('proto.ubo.v1.BasicType', null, global);\ngoog.exportSymbol('proto.ubo.v1.BasicTypeOptional', null, global);\ngoog.exportSymbol('proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraDetectAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraDetectEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraDetectedEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraInstallDriverAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraInstallDriverEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraReinitializeEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraReportBarcodeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraReportImageEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraRestoreDefaultAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraRestoreDefaultEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraSetAvailableCamerasAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraSetIndexAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraStartViewfinderAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraStartViewfinderEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraState', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraState.AvailableCameras', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraStopViewfinderEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.CameraType', null, global);\ngoog.exportSymbol('proto.ubo.v1.Chime', null, global);\ngoog.exportSymbol('proto.ubo.v1.CloseApplicationAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.CloseApplicationEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.ConnectionState', null, global);\ngoog.exportSymbol('proto.ubo.v1.DeregisterRegularAppAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayBlankAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayBlankEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayBlankTimeout', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayCompressedRenderEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayPauseAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayRedrawAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayRedrawEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayRenderEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayResumeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplaySetBlankTimeoutAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayState', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayUnblankAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayUnblankEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DisplayUpdateActivityAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageFetchAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageFetchCompositionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageFetchEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRegisterAppEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageReleaseAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageReleaseCompositionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRemoveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRemoveCompositionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRemoveContainerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRemoveContainerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRemoveEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRunAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRunCompositionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageRunContainerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageSetDockerIdAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageSetStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageSetStatusAction.Ports', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageStopAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageStopCompositionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageStopContainerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerImageUpdateMetadataAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerInstallAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerInstallEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerItemStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerRemoveUsernameAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerServiceState', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerServiceState.UsernamesDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerSetStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerStartAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerStartEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerState', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerStopAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerStopEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.DockerStoreUsernameAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.ErrorReport', null, global);\ngoog.exportSymbol('proto.ubo.v1.Event', null, global);\ngoog.exportSymbol('proto.ubo.v1.Event.EventCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemCopyAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemCopyEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemMoveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemMoveEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemRemoveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemRemoveEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemReportSelectionAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemSelectEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.FileSystemState', null, global);\ngoog.exportSymbol('proto.ubo.v1.HeadedMenu', null, global);\ngoog.exportSymbol('proto.ubo.v1.HeadlessMenu', null, global);\ngoog.exportSymbol('proto.ubo.v1.IconState', null, global);\ngoog.exportSymbol('proto.ubo.v1.ImageState', null, global);\ngoog.exportSymbol('proto.ubo.v1.ImageState.Ports', null, global);\ngoog.exportSymbol('proto.ubo.v1.Importance', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredHandleReceivedCodeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredSendCodeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredSendCodeEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredSetShouldPropagateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredSetShouldReceiveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InfraredState', null, global);\ngoog.exportSymbol('proto.ubo.v1.InitEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputCancelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputCancelEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputDemandAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputDescription', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputFieldDescription', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputFieldDescription.Options', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputFieldType', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputMethod', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputProvideAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputProvideEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputResolveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputResolveEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.InputResult', null, global);\ngoog.exportSymbol('proto.ubo.v1.IpAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.IpEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.IpNetworkInterface', null, global);\ngoog.exportSymbol('proto.ubo.v1.IpSetIsConnectedAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.IpState', null, global);\ngoog.exportSymbol('proto.ubo.v1.IpUpdateInterfacesAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.Item', null, global);\ngoog.exportSymbol('proto.ubo.v1.Key', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadAction.HeldKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadAction.PressedKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyHoldAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyPressAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyReleaseAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyUnholdAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType', null, global);\ngoog.exportSymbol('proto.ubo.v1.KeypadState', null, global);\ngoog.exportSymbol('proto.ubo.v1.LightDMAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.LightDMClearEnabledStateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.LightDMState', null, global);\ngoog.exportSymbol('proto.ubo.v1.LightDMUpdateStateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainState', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainState.AppsItemsPrioritiesDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainState.Path', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainState.RecordedSequence', null, global);\ngoog.exportSymbol('proto.ubo.v1.MainState.SettingsItemsPrioritiesDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.McpServerMetadata', null, global);\ngoog.exportSymbol('proto.ubo.v1.McpServerType', null, global);\ngoog.exportSymbol('proto.ubo.v1.Menu', null, global);\ngoog.exportSymbol('proto.ubo.v1.Menu.MenuCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuChooseByIconAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuChooseByIconEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuChooseByIndexAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuChooseByIndexEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuChooseByLabelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuChooseByLabelEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuGoBackAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuGoBackEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuGoHomeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuGoHomeEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuScrollAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuScrollDirection', null, global);\ngoog.exportSymbol('proto.ubo.v1.MenuScrollEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.NetState', null, global);\ngoog.exportSymbol('proto.ubo.v1.Notification', null, global);\ngoog.exportSymbol('proto.ubo.v1.Notification.Actions', null, global);\ngoog.exportSymbol('proto.ubo.v1.Notification.ActionsItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.Notification.ActionsItem.ActionsItemCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.Notification.OnClose', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationActionItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationApplicationItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.Application', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.InitializationArgs', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationDispatchItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationDispatchItem.StoreAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationDispatchItem.StoreAction2', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationDisplayType', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsAddAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsClearAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsClearAllAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsClearByIdAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsClearEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsDisplayAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsDisplayEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.NotificationsState', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationArgs', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationArgs', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2', null, global);\ngoog.exportSymbol('proto.ubo.v1.PathInputDescription', null, global);\ngoog.exportSymbol('proto.ubo.v1.PathSelectorConfig', null, global);\ngoog.exportSymbol('proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes', null, global);\ngoog.exportSymbol('proto.ubo.v1.PowerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.PowerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.PowerOffAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.PowerOffEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.QRCodeInputDescription', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectDoneDownloadingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectLoginEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectSetPendingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectSetStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectStartDownloadingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectState', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.RPiConnectUpdateServiceStateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.ReadableInformation', null, global);\ngoog.exportSymbol('proto.ubo.v1.RebootAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RebootEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.RegisterAppAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RegisterRegularAppAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RegisterSettingAppAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.ReplayRecordedSequenceAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.ReplayRecordedSequenceEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.ReportReplayingDoneAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbColor', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbColorElement', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbColorElement.RgbColorElementCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingBlankAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingBlinkAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingColorfulCommandAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingCommandAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingCommandEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingFillDownfromAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingFillUptoAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingProgressWheelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingProgressWheelStepAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingPulseAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingRainbowAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingSequenceAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingSetAllAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingSetBrightnessAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingSetEnabledAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingSetIsBusyAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingSpinningWheelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingState', null, global);\ngoog.exportSymbol('proto.ubo.v1.RgbRingWaitableCommandAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SSHAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SSHClearEnabledStateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SSHState', null, global);\ngoog.exportSymbol('proto.ubo.v1.SSHUpdateStateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.ScreenshotEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.Sensor', null, global);\ngoog.exportSymbol('proto.ubo.v1.SensorState', null, global);\ngoog.exportSymbol('proto.ubo.v1.SensorsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SensorsReportReadingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SensorsState', null, global);\ngoog.exportSymbol('proto.ubo.v1.ServiceState', null, global);\ngoog.exportSymbol('proto.ubo.v1.ServiceState.Errors', null, global);\ngoog.exportSymbol('proto.ubo.v1.ServicesStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.SetAreEnclosuresVisibleAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SetMenuPathAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsCategory', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsClearServiceErrorsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsReportServiceErrorAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsServiceAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsServiceEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsServiceSetIsEnabledAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsServiceSetLogLevelAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsServiceSetShouldRestartAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsServiceSetStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsSetServicesAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsStartServiceAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsStartServiceEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsState', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsState.ServicesDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsStopServiceAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsStopServiceEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsToggleBetaVersionsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsTogglePdbSignalAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SettingsToggleVisualDebugAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SnapshotEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionEngineName', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Action', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Action2', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Phrase', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Phrase1', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportSpeechAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportTextEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionState', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechRecognitionStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisEngineName', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisReadTextAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisState', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.StatusIconsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.StatusIconsRegisterAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.StatusIconsState', null, global);\ngoog.exportSymbol('proto.ubo.v1.StoreRecordedSequenceEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.SubMenuItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.ToggleRecordingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboApplicationItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboApplicationItem.Application', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboApplicationItem.Application.ApplicationCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboApplicationItem.InitializationArgs', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboApplicationItem.InitializationKwargsDict', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboDispatchItem', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboDispatchItem.StoreAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase', null, global);\ngoog.exportSymbol('proto.ubo.v1.UboDispatchItem.StoreAction2', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateLightDMState', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerCheckEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerReportFailedCheckAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerRequestCheckAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerRequestUpdateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerSetVersionsAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerState', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerState.RecentVersions', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateManagerUpdateEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UpdateStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.UserState', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersCreateUserAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersCreateUserEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersDeleteUserAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersDeleteUserEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersResetPasswordAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersResetPasswordEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersSetUsersAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersState', null, global);\ngoog.exportSymbol('proto.ubo.v1.UsersState.Users', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeDoneDownloadingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeLoginEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeRestartEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeSetPendingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeSetStatusAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeStartDownloadingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeState', null, global);\ngoog.exportSymbol('proto.ubo.v1.VSCodeStatus', null, global);\ngoog.exportSymbol('proto.ubo.v1.WebUIEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.WebUIInitializeEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.WebUIInputDescription', null, global);\ngoog.exportSymbol('proto.ubo.v1.WebUIInputDescription.Fields', null, global);\ngoog.exportSymbol('proto.ubo.v1.WebUIState', null, global);\ngoog.exportSymbol('proto.ubo.v1.WebUIStopEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiConnection', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiInputConnectionAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiInputConnectionEvent', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiSetHasVisitedOnboardingAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiState', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiState.Connections', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiType', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiUpdateAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiUpdateRequestAction', null, global);\ngoog.exportSymbol('proto.ubo.v1.WiFiUpdateRequestEvent', null, global);\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.BaseMenu = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.BaseMenu.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.BaseMenu, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.BaseMenu.displayName = 'proto.ubo.v1.BaseMenu';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.HeadedMenu = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.HeadedMenu.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.HeadedMenu, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.HeadedMenu.displayName = 'proto.ubo.v1.HeadedMenu';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.HeadlessMenu = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.HeadlessMenu.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.HeadlessMenu, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.HeadlessMenu.displayName = 'proto.ubo.v1.HeadlessMenu';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Item = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.Item, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Item.displayName = 'proto.ubo.v1.Item';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ActionItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ActionItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ActionItem.displayName = 'proto.ubo.v1.ActionItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ApplicationItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ApplicationItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ApplicationItem.displayName = 'proto.ubo.v1.ApplicationItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ApplicationItem.Application = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.ApplicationItem.Application.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.ApplicationItem.Application, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ApplicationItem.Application.displayName = 'proto.ubo.v1.ApplicationItem.Application';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SubMenuItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SubMenuItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SubMenuItem.displayName = 'proto.ubo.v1.SubMenuItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Menu = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Menu.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.Menu, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Menu.displayName = 'proto.ubo.v1.Menu';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MainAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainAction.displayName = 'proto.ubo.v1.MainAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateLightDMState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateLightDMState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateLightDMState.displayName = 'proto.ubo.v1.UpdateLightDMState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RegisterAppAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RegisterAppAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RegisterAppAction.displayName = 'proto.ubo.v1.RegisterAppAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RegisterRegularAppAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RegisterRegularAppAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RegisterRegularAppAction.displayName = 'proto.ubo.v1.RegisterRegularAppAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DeregisterRegularAppAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DeregisterRegularAppAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DeregisterRegularAppAction.displayName = 'proto.ubo.v1.DeregisterRegularAppAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RegisterSettingAppAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RegisterSettingAppAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RegisterSettingAppAction.displayName = 'proto.ubo.v1.RegisterSettingAppAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PowerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.PowerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PowerAction.displayName = 'proto.ubo.v1.PowerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PowerOffAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.PowerOffAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PowerOffAction.displayName = 'proto.ubo.v1.PowerOffAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RebootAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RebootAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RebootAction.displayName = 'proto.ubo.v1.RebootAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SetMenuPathAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.SetMenuPathAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.SetMenuPathAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SetMenuPathAction.displayName = 'proto.ubo.v1.SetMenuPathAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SetAreEnclosuresVisibleAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SetAreEnclosuresVisibleAction.displayName = 'proto.ubo.v1.SetAreEnclosuresVisibleAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuAction.displayName = 'proto.ubo.v1.MenuAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuGoBackAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuGoBackAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuGoBackAction.displayName = 'proto.ubo.v1.MenuGoBackAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuGoHomeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuGoHomeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuGoHomeAction.displayName = 'proto.ubo.v1.MenuGoHomeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuChooseByIconAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuChooseByIconAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuChooseByIconAction.displayName = 'proto.ubo.v1.MenuChooseByIconAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuChooseByLabelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuChooseByLabelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuChooseByLabelAction.displayName = 'proto.ubo.v1.MenuChooseByLabelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuChooseByIndexAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuChooseByIndexAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuChooseByIndexAction.displayName = 'proto.ubo.v1.MenuChooseByIndexAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuScrollAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuScrollAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuScrollAction.displayName = 'proto.ubo.v1.MenuScrollAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationAction.displayName = 'proto.ubo.v1.OpenApplicationAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationAction.InitializationArgs.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationArgs, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationAction.InitializationArgs.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationArgs';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2 = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CloseApplicationAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CloseApplicationAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CloseApplicationAction.displayName = 'proto.ubo.v1.CloseApplicationAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MainEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainEvent.displayName = 'proto.ubo.v1.MainEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InitEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InitEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InitEvent.displayName = 'proto.ubo.v1.InitEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuEvent.displayName = 'proto.ubo.v1.MenuEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuGoBackEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuGoBackEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuGoBackEvent.displayName = 'proto.ubo.v1.MenuGoBackEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuGoHomeEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuGoHomeEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuGoHomeEvent.displayName = 'proto.ubo.v1.MenuGoHomeEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuChooseByIconEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuChooseByIconEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuChooseByIconEvent.displayName = 'proto.ubo.v1.MenuChooseByIconEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuChooseByLabelEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuChooseByLabelEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuChooseByLabelEvent.displayName = 'proto.ubo.v1.MenuChooseByLabelEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuChooseByIndexEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuChooseByIndexEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuChooseByIndexEvent.displayName = 'proto.ubo.v1.MenuChooseByIndexEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MenuScrollEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MenuScrollEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MenuScrollEvent.displayName = 'proto.ubo.v1.MenuScrollEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationEvent.displayName = 'proto.ubo.v1.OpenApplicationEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationEvent.InitializationArgs.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationArgs, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationEvent.InitializationArgs.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationArgs';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2 = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CloseApplicationEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CloseApplicationEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CloseApplicationEvent.displayName = 'proto.ubo.v1.CloseApplicationEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PowerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.PowerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PowerEvent.displayName = 'proto.ubo.v1.PowerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PowerOffEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.PowerOffEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PowerOffEvent.displayName = 'proto.ubo.v1.PowerOffEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RebootEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RebootEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RebootEvent.displayName = 'proto.ubo.v1.RebootEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ScreenshotEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ScreenshotEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ScreenshotEvent.displayName = 'proto.ubo.v1.ScreenshotEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SnapshotEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SnapshotEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SnapshotEvent.displayName = 'proto.ubo.v1.SnapshotEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ToggleRecordingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ToggleRecordingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ToggleRecordingAction.displayName = 'proto.ubo.v1.ToggleRecordingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.StoreRecordedSequenceEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.StoreRecordedSequenceEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.StoreRecordedSequenceEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.StoreRecordedSequenceEvent.displayName = 'proto.ubo.v1.StoreRecordedSequenceEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ReplayRecordedSequenceAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ReplayRecordedSequenceAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ReplayRecordedSequenceAction.displayName = 'proto.ubo.v1.ReplayRecordedSequenceAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ReplayRecordedSequenceEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ReplayRecordedSequenceEvent.displayName = 'proto.ubo.v1.ReplayRecordedSequenceEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ReportReplayingDoneAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ReportReplayingDoneAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ReportReplayingDoneAction.displayName = 'proto.ubo.v1.ReportReplayingDoneAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.MainState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainState.displayName = 'proto.ubo.v1.MainState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainState.Path = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.MainState.Path.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.MainState.Path, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainState.Path.displayName = 'proto.ubo.v1.MainState.Path';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.MainState.AppsItemsPrioritiesDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainState.AppsItemsPrioritiesDict.displayName = 'proto.ubo.v1.MainState.AppsItemsPrioritiesDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.MainState.SettingsItemsPrioritiesDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.displayName = 'proto.ubo.v1.MainState.SettingsItemsPrioritiesDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.MainState.RecordedSequence = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.MainState.RecordedSequence.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.MainState.RecordedSequence, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.MainState.RecordedSequence.displayName = 'proto.ubo.v1.MainState.RecordedSequence';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboDispatchItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UboDispatchItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboDispatchItem.displayName = 'proto.ubo.v1.UboDispatchItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2 = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UboDispatchItem.StoreAction2.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.UboDispatchItem.StoreAction2, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboDispatchItem.StoreAction2.displayName = 'proto.ubo.v1.UboDispatchItem.StoreAction2';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboDispatchItem.StoreAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.UboDispatchItem.StoreAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboDispatchItem.StoreAction.displayName = 'proto.ubo.v1.UboDispatchItem.StoreAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboApplicationItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UboApplicationItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboApplicationItem.displayName = 'proto.ubo.v1.UboApplicationItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboApplicationItem.Application = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.UboApplicationItem.Application, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboApplicationItem.Application.displayName = 'proto.ubo.v1.UboApplicationItem.Application';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UboApplicationItem.InitializationArgs.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.UboApplicationItem.InitializationArgs, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboApplicationItem.InitializationArgs.displayName = 'proto.ubo.v1.UboApplicationItem.InitializationArgs';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.UboApplicationItem.InitializationKwargsDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.displayName = 'proto.ubo.v1.UboApplicationItem.InitializationKwargsDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.BasicTypeOptional = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.BasicTypeOptional.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.BasicTypeOptional, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.BasicTypeOptional.displayName = 'proto.ubo.v1.BasicTypeOptional';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.BasicType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.BasicType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.BasicType.displayName = 'proto.ubo.v1.BasicType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputResult = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputResult, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputResult.displayName = 'proto.ubo.v1.InputResult';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputFieldDescription = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputFieldDescription, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputFieldDescription.displayName = 'proto.ubo.v1.InputFieldDescription';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputFieldDescription.Options = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.InputFieldDescription.Options.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.InputFieldDescription.Options, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputFieldDescription.Options.displayName = 'proto.ubo.v1.InputFieldDescription.Options';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputDescription = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputDescription, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputDescription.displayName = 'proto.ubo.v1.InputDescription';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WebUIInputDescription = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WebUIInputDescription, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WebUIInputDescription.displayName = 'proto.ubo.v1.WebUIInputDescription';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WebUIInputDescription.Fields = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.WebUIInputDescription.Fields.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.WebUIInputDescription.Fields, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WebUIInputDescription.Fields.displayName = 'proto.ubo.v1.WebUIInputDescription.Fields';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.QRCodeInputDescription = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.QRCodeInputDescription, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.QRCodeInputDescription.displayName = 'proto.ubo.v1.QRCodeInputDescription';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PathInputDescription = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.PathInputDescription, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PathInputDescription.displayName = 'proto.ubo.v1.PathInputDescription';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputAction.displayName = 'proto.ubo.v1.InputAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputDemandAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputDemandAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputDemandAction.displayName = 'proto.ubo.v1.InputDemandAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputResolveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputResolveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputResolveAction.displayName = 'proto.ubo.v1.InputResolveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputCancelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputCancelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputCancelAction.displayName = 'proto.ubo.v1.InputCancelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputProvideAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputProvideAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputProvideAction.displayName = 'proto.ubo.v1.InputProvideAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputResolveEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputResolveEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputResolveEvent.displayName = 'proto.ubo.v1.InputResolveEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputCancelEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputCancelEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputCancelEvent.displayName = 'proto.ubo.v1.InputCancelEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InputProvideEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InputProvideEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InputProvideEvent.displayName = 'proto.ubo.v1.InputProvideEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AvailableInputDescription = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.AvailableInputDescription.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.AvailableInputDescription, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AvailableInputDescription.displayName = 'proto.ubo.v1.AvailableInputDescription';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsAction.displayName = 'proto.ubo.v1.SettingsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsTogglePdbSignalAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsTogglePdbSignalAction.displayName = 'proto.ubo.v1.SettingsTogglePdbSignalAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsToggleVisualDebugAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsToggleVisualDebugAction.displayName = 'proto.ubo.v1.SettingsToggleVisualDebugAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsToggleBetaVersionsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsToggleBetaVersionsAction.displayName = 'proto.ubo.v1.SettingsToggleBetaVersionsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsSetServicesAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsSetServicesAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsSetServicesAction.displayName = 'proto.ubo.v1.SettingsSetServicesAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsServiceAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsServiceAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsServiceAction.displayName = 'proto.ubo.v1.SettingsServiceAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsStartServiceAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsStartServiceAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsStartServiceAction.displayName = 'proto.ubo.v1.SettingsStartServiceAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsStopServiceAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsStopServiceAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsStopServiceAction.displayName = 'proto.ubo.v1.SettingsStopServiceAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsServiceSetStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsServiceSetStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsServiceSetStatusAction.displayName = 'proto.ubo.v1.SettingsServiceSetStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsServiceSetIsEnabledAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsServiceSetIsEnabledAction.displayName = 'proto.ubo.v1.SettingsServiceSetIsEnabledAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsServiceSetLogLevelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsServiceSetLogLevelAction.displayName = 'proto.ubo.v1.SettingsServiceSetLogLevelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsServiceSetShouldRestartAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsServiceSetShouldRestartAction.displayName = 'proto.ubo.v1.SettingsServiceSetShouldRestartAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsReportServiceErrorAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsReportServiceErrorAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsReportServiceErrorAction.displayName = 'proto.ubo.v1.SettingsReportServiceErrorAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsClearServiceErrorsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsClearServiceErrorsAction.displayName = 'proto.ubo.v1.SettingsClearServiceErrorsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsEvent.displayName = 'proto.ubo.v1.SettingsEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsServiceEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsServiceEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsServiceEvent.displayName = 'proto.ubo.v1.SettingsServiceEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsStartServiceEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsStartServiceEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsStartServiceEvent.displayName = 'proto.ubo.v1.SettingsStartServiceEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsStopServiceEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsStopServiceEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsStopServiceEvent.displayName = 'proto.ubo.v1.SettingsStopServiceEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ErrorReport = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ErrorReport, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ErrorReport.displayName = 'proto.ubo.v1.ErrorReport';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ServiceState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ServiceState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ServiceState.displayName = 'proto.ubo.v1.ServiceState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ServiceState.Errors = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.ServiceState.Errors.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.ServiceState.Errors, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ServiceState.Errors.displayName = 'proto.ubo.v1.ServiceState.Errors';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsState.displayName = 'proto.ubo.v1.SettingsState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SettingsState.ServicesDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.SettingsState.ServicesDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SettingsState.ServicesDict.displayName = 'proto.ubo.v1.SettingsState.ServicesDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IconState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.IconState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IconState.displayName = 'proto.ubo.v1.IconState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.StatusIconsState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.StatusIconsState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.StatusIconsState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.StatusIconsState.displayName = 'proto.ubo.v1.StatusIconsState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.StatusIconsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.StatusIconsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.StatusIconsAction.displayName = 'proto.ubo.v1.StatusIconsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.StatusIconsRegisterAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.StatusIconsRegisterAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.StatusIconsRegisterAction.displayName = 'proto.ubo.v1.StatusIconsRegisterAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerAction.displayName = 'proto.ubo.v1.UpdateManagerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerSetVersionsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerSetVersionsAction.displayName = 'proto.ubo.v1.UpdateManagerSetVersionsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.displayName = 'proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerRequestCheckAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerRequestCheckAction.displayName = 'proto.ubo.v1.UpdateManagerRequestCheckAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerReportFailedCheckAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerReportFailedCheckAction.displayName = 'proto.ubo.v1.UpdateManagerReportFailedCheckAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerRequestUpdateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerRequestUpdateAction.displayName = 'proto.ubo.v1.UpdateManagerRequestUpdateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerEvent.displayName = 'proto.ubo.v1.UpdateManagerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerCheckEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerCheckEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerCheckEvent.displayName = 'proto.ubo.v1.UpdateManagerCheckEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerUpdateEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerUpdateEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerUpdateEvent.displayName = 'proto.ubo.v1.UpdateManagerUpdateEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerState.displayName = 'proto.ubo.v1.UpdateManagerState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UpdateManagerState.RecentVersions.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.UpdateManagerState.RecentVersions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UpdateManagerState.RecentVersions.displayName = 'proto.ubo.v1.UpdateManagerState.RecentVersions';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.McpServerMetadata = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.McpServerMetadata, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.McpServerMetadata.displayName = 'proto.ubo.v1.McpServerMetadata';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantAction.displayName = 'proto.ubo.v1.AssistantAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSetIsActiveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSetIsActiveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSetIsActiveAction.displayName = 'proto.ubo.v1.AssistantSetIsActiveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSetSelectedSTTAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSetSelectedSTTAction.displayName = 'proto.ubo.v1.AssistantSetSelectedSTTAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSetSelectedLLMAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSetSelectedLLMAction.displayName = 'proto.ubo.v1.AssistantSetSelectedLLMAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSetSelectedTTSAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSetSelectedTTSAction.displayName = 'proto.ubo.v1.AssistantSetSelectedTTSAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSetSelectedImageGeneratorAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.displayName = 'proto.ubo.v1.AssistantSetSelectedImageGeneratorAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSetSelectedModelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSetSelectedModelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSetSelectedModelAction.displayName = 'proto.ubo.v1.AssistantSetSelectedModelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantDownloadOllamaModelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantDownloadOllamaModelAction.displayName = 'proto.ubo.v1.AssistantDownloadOllamaModelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistanceFrame = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistanceFrame, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistanceFrame.displayName = 'proto.ubo.v1.AssistanceFrame';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistanceTextFrame = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistanceTextFrame, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistanceTextFrame.displayName = 'proto.ubo.v1.AssistanceTextFrame';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistanceAudioFrame = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistanceAudioFrame, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistanceAudioFrame.displayName = 'proto.ubo.v1.AssistanceAudioFrame';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistanceImageFrame = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistanceImageFrame, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistanceImageFrame.displayName = 'proto.ubo.v1.AssistanceImageFrame';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantReportAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantReportAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantReportAction.displayName = 'proto.ubo.v1.AssistantReportAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantStartListeningAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantStartListeningAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantStartListeningAction.displayName = 'proto.ubo.v1.AssistantStartListeningAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantStopListeningAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantStopListeningAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantStopListeningAction.displayName = 'proto.ubo.v1.AssistantStopListeningAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantToggleListeningAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantToggleListeningAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantToggleListeningAction.displayName = 'proto.ubo.v1.AssistantToggleListeningAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantUpdateProvidersAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantUpdateProvidersAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantUpdateProvidersAction.displayName = 'proto.ubo.v1.AssistantUpdateProvidersAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantAddMcpServerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantAddMcpServerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantAddMcpServerAction.displayName = 'proto.ubo.v1.AssistantAddMcpServerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantToggleMcpServerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantToggleMcpServerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantToggleMcpServerAction.displayName = 'proto.ubo.v1.AssistantToggleMcpServerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantDeleteMcpServerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantDeleteMcpServerAction.displayName = 'proto.ubo.v1.AssistantDeleteMcpServerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantSyncMcpServersAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantSyncMcpServersAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantSyncMcpServersAction.displayName = 'proto.ubo.v1.AssistantSyncMcpServersAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantEvent.displayName = 'proto.ubo.v1.AssistantEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantDownloadOllamaModelEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantDownloadOllamaModelEvent.displayName = 'proto.ubo.v1.AssistantDownloadOllamaModelEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantHandleReportEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantHandleReportEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantHandleReportEvent.displayName = 'proto.ubo.v1.AssistantHandleReportEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantUpdateProvidersEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantUpdateProvidersEvent.displayName = 'proto.ubo.v1.AssistantUpdateProvidersEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantAddMcpServerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantAddMcpServerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantAddMcpServerEvent.displayName = 'proto.ubo.v1.AssistantAddMcpServerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantDeleteMcpServerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantDeleteMcpServerEvent.displayName = 'proto.ubo.v1.AssistantDeleteMcpServerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantState.displayName = 'proto.ubo.v1.AssistantState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantState.SelectedModelsDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantState.SelectedModelsDict.displayName = 'proto.ubo.v1.AssistantState.SelectedModelsDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantState.McpServersDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantState.McpServersDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantState.McpServersDict.displayName = 'proto.ubo.v1.AssistantState.McpServersDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.AssistantState.EnabledMcpServers.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantState.EnabledMcpServers, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantState.EnabledMcpServers.displayName = 'proto.ubo.v1.AssistantState.EnabledMcpServers';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.AssistantState.ProviderSetupStatusDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AssistantState.ProviderSetupStatusDict.displayName = 'proto.ubo.v1.AssistantState.ProviderSetupStatusDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AcceptableAssistanceFrame = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.AcceptableAssistanceFrame, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AcceptableAssistanceFrame.displayName = 'proto.ubo.v1.AcceptableAssistanceFrame';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioAction.displayName = 'proto.ubo.v1.AudioAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioInstallDriverAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioInstallDriverAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioInstallDriverAction.displayName = 'proto.ubo.v1.AudioInstallDriverAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioSetVolumeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioSetVolumeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioSetVolumeAction.displayName = 'proto.ubo.v1.AudioSetVolumeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioChangeVolumeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioChangeVolumeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioChangeVolumeAction.displayName = 'proto.ubo.v1.AudioChangeVolumeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioSetMuteStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioSetMuteStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioSetMuteStatusAction.displayName = 'proto.ubo.v1.AudioSetMuteStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioToggleMuteStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioToggleMuteStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioToggleMuteStatusAction.displayName = 'proto.ubo.v1.AudioToggleMuteStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayChimeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayChimeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayChimeAction.displayName = 'proto.ubo.v1.AudioPlayChimeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayAudioSampleAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayAudioSampleAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayAudioSampleAction.displayName = 'proto.ubo.v1.AudioPlayAudioSampleAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayAudioSequenceAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayAudioSequenceAction.displayName = 'proto.ubo.v1.AudioPlayAudioSequenceAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioSample = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioSample, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioSample.displayName = 'proto.ubo.v1.AudioSample';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioReportSampleAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioReportSampleAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioReportSampleAction.displayName = 'proto.ubo.v1.AudioReportSampleAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlaybackDoneAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlaybackDoneAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlaybackDoneAction.displayName = 'proto.ubo.v1.AudioPlaybackDoneAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioStartRecordingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioStartRecordingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioStartRecordingAction.displayName = 'proto.ubo.v1.AudioStartRecordingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioStopRecordingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioStopRecordingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioStopRecordingAction.displayName = 'proto.ubo.v1.AudioStopRecordingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioToggleRecordingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioToggleRecordingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioToggleRecordingAction.displayName = 'proto.ubo.v1.AudioToggleRecordingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayRecordingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayRecordingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayRecordingAction.displayName = 'proto.ubo.v1.AudioPlayRecordingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioEvent.displayName = 'proto.ubo.v1.AudioEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioReportSampleEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioReportSampleEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioReportSampleEvent.displayName = 'proto.ubo.v1.AudioReportSampleEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioInstallDriverEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioInstallDriverEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioInstallDriverEvent.displayName = 'proto.ubo.v1.AudioInstallDriverEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayChimeEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayChimeEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayChimeEvent.displayName = 'proto.ubo.v1.AudioPlayChimeEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayAudioSampleEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayAudioSampleEvent.displayName = 'proto.ubo.v1.AudioPlayAudioSampleEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlayAudioSequenceEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlayAudioSequenceEvent.displayName = 'proto.ubo.v1.AudioPlayAudioSequenceEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioPlaybackDoneEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioPlaybackDoneEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioPlaybackDoneEvent.displayName = 'proto.ubo.v1.AudioPlaybackDoneEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.AudioState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.AudioState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.AudioState.displayName = 'proto.ubo.v1.AudioState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraAction.displayName = 'proto.ubo.v1.CameraAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraStartViewfinderAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraStartViewfinderAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraStartViewfinderAction.displayName = 'proto.ubo.v1.CameraStartViewfinderAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraReportBarcodeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraReportBarcodeAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.CameraReportBarcodeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraReportBarcodeAction.displayName = 'proto.ubo.v1.CameraReportBarcodeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraInstallDriverAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraInstallDriverAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraInstallDriverAction.displayName = 'proto.ubo.v1.CameraInstallDriverAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraEvent.displayName = 'proto.ubo.v1.CameraEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraInstallDriverEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraInstallDriverEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraInstallDriverEvent.displayName = 'proto.ubo.v1.CameraInstallDriverEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraRestoreDefaultAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraRestoreDefaultAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraRestoreDefaultAction.displayName = 'proto.ubo.v1.CameraRestoreDefaultAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraRestoreDefaultEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraRestoreDefaultEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraRestoreDefaultEvent.displayName = 'proto.ubo.v1.CameraRestoreDefaultEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraStartViewfinderEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraStartViewfinderEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraStartViewfinderEvent.displayName = 'proto.ubo.v1.CameraStartViewfinderEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraReportImageEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraReportImageEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraReportImageEvent.displayName = 'proto.ubo.v1.CameraReportImageEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraStopViewfinderEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraStopViewfinderEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraStopViewfinderEvent.displayName = 'proto.ubo.v1.CameraStopViewfinderEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraSetIndexAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraSetIndexAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraSetIndexAction.displayName = 'proto.ubo.v1.CameraSetIndexAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraDetectAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraDetectAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraDetectAction.displayName = 'proto.ubo.v1.CameraDetectAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraSetAvailableCamerasAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.CameraSetAvailableCamerasAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraSetAvailableCamerasAction.displayName = 'proto.ubo.v1.CameraSetAvailableCamerasAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraDetectEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraDetectEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraDetectEvent.displayName = 'proto.ubo.v1.CameraDetectEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraDetectedEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraDetectedEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.CameraDetectedEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraDetectedEvent.displayName = 'proto.ubo.v1.CameraDetectedEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraReinitializeEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.CameraReinitializeEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraReinitializeEvent.displayName = 'proto.ubo.v1.CameraReinitializeEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.CameraState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraState.displayName = 'proto.ubo.v1.CameraState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.CameraState.AvailableCameras = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.CameraState.AvailableCameras.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.CameraState.AvailableCameras, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.CameraState.AvailableCameras.displayName = 'proto.ubo.v1.CameraState.AvailableCameras';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayAction.displayName = 'proto.ubo.v1.DisplayAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayEvent.displayName = 'proto.ubo.v1.DisplayEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayPauseAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayPauseAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayPauseAction.displayName = 'proto.ubo.v1.DisplayPauseAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayResumeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayResumeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayResumeAction.displayName = 'proto.ubo.v1.DisplayResumeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayRedrawAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayRedrawAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayRedrawAction.displayName = 'proto.ubo.v1.DisplayRedrawAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayBlankAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayBlankAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayBlankAction.displayName = 'proto.ubo.v1.DisplayBlankAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayUnblankAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayUnblankAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayUnblankAction.displayName = 'proto.ubo.v1.DisplayUnblankAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayUpdateActivityAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayUpdateActivityAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayUpdateActivityAction.displayName = 'proto.ubo.v1.DisplayUpdateActivityAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplaySetBlankTimeoutAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplaySetBlankTimeoutAction.displayName = 'proto.ubo.v1.DisplaySetBlankTimeoutAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayRedrawEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayRedrawEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayRedrawEvent.displayName = 'proto.ubo.v1.DisplayRedrawEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayBlankEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayBlankEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayBlankEvent.displayName = 'proto.ubo.v1.DisplayBlankEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayUnblankEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayUnblankEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayUnblankEvent.displayName = 'proto.ubo.v1.DisplayUnblankEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayRenderEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.DisplayRenderEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayRenderEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayRenderEvent.displayName = 'proto.ubo.v1.DisplayRenderEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayCompressedRenderEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.DisplayCompressedRenderEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayCompressedRenderEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayCompressedRenderEvent.displayName = 'proto.ubo.v1.DisplayCompressedRenderEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DisplayState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DisplayState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DisplayState.displayName = 'proto.ubo.v1.DisplayState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerAction.displayName = 'proto.ubo.v1.DockerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerInstallAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerInstallAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerInstallAction.displayName = 'proto.ubo.v1.DockerInstallAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerStartAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerStartAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerStartAction.displayName = 'proto.ubo.v1.DockerStartAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerStopAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerStopAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerStopAction.displayName = 'proto.ubo.v1.DockerStopAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerSetStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerSetStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerSetStatusAction.displayName = 'proto.ubo.v1.DockerSetStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerStoreUsernameAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerStoreUsernameAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerStoreUsernameAction.displayName = 'proto.ubo.v1.DockerStoreUsernameAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerRemoveUsernameAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerRemoveUsernameAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerRemoveUsernameAction.displayName = 'proto.ubo.v1.DockerRemoveUsernameAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageAction.displayName = 'proto.ubo.v1.DockerImageAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageSetStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageSetStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageSetStatusAction.displayName = 'proto.ubo.v1.DockerImageSetStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.DockerImageSetStatusAction.Ports.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageSetStatusAction.Ports, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageSetStatusAction.Ports.displayName = 'proto.ubo.v1.DockerImageSetStatusAction.Ports';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageSetDockerIdAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageSetDockerIdAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageSetDockerIdAction.displayName = 'proto.ubo.v1.DockerImageSetDockerIdAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageUpdateMetadataAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageUpdateMetadataAction.displayName = 'proto.ubo.v1.DockerImageUpdateMetadataAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageFetchAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageFetchAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageFetchAction.displayName = 'proto.ubo.v1.DockerImageFetchAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRemoveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRemoveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRemoveAction.displayName = 'proto.ubo.v1.DockerImageRemoveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRunAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRunAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRunAction.displayName = 'proto.ubo.v1.DockerImageRunAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageStopAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageStopAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageStopAction.displayName = 'proto.ubo.v1.DockerImageStopAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageReleaseAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageReleaseAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageReleaseAction.displayName = 'proto.ubo.v1.DockerImageReleaseAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRemoveContainerAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRemoveContainerAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRemoveContainerAction.displayName = 'proto.ubo.v1.DockerImageRemoveContainerAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerEvent.displayName = 'proto.ubo.v1.DockerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerInstallEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerInstallEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerInstallEvent.displayName = 'proto.ubo.v1.DockerInstallEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerStartEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerStartEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerStartEvent.displayName = 'proto.ubo.v1.DockerStartEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerStopEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerStopEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerStopEvent.displayName = 'proto.ubo.v1.DockerStopEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerServiceState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerServiceState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerServiceState.displayName = 'proto.ubo.v1.DockerServiceState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerServiceState.UsernamesDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerServiceState.UsernamesDict.displayName = 'proto.ubo.v1.DockerServiceState.UsernamesDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageEvent.displayName = 'proto.ubo.v1.DockerImageEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRegisterAppEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRegisterAppEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRegisterAppEvent.displayName = 'proto.ubo.v1.DockerImageRegisterAppEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageFetchCompositionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageFetchCompositionEvent.displayName = 'proto.ubo.v1.DockerImageFetchCompositionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageFetchEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageFetchEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageFetchEvent.displayName = 'proto.ubo.v1.DockerImageFetchEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRemoveCompositionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRemoveCompositionEvent.displayName = 'proto.ubo.v1.DockerImageRemoveCompositionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRemoveEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRemoveEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRemoveEvent.displayName = 'proto.ubo.v1.DockerImageRemoveEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRunCompositionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRunCompositionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRunCompositionEvent.displayName = 'proto.ubo.v1.DockerImageRunCompositionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRunContainerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRunContainerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRunContainerEvent.displayName = 'proto.ubo.v1.DockerImageRunContainerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageStopCompositionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageStopCompositionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageStopCompositionEvent.displayName = 'proto.ubo.v1.DockerImageStopCompositionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageStopContainerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageStopContainerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageStopContainerEvent.displayName = 'proto.ubo.v1.DockerImageStopContainerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageReleaseCompositionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageReleaseCompositionEvent.displayName = 'proto.ubo.v1.DockerImageReleaseCompositionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerImageRemoveContainerEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerImageRemoveContainerEvent.displayName = 'proto.ubo.v1.DockerImageRemoveContainerEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ImageState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ImageState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ImageState.displayName = 'proto.ubo.v1.ImageState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ImageState.Ports = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.ImageState.Ports.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.ImageState.Ports, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ImageState.Ports.displayName = 'proto.ubo.v1.ImageState.Ports';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.DockerState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.DockerState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.DockerState.displayName = 'proto.ubo.v1.DockerState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PathSelectorConfig = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.PathSelectorConfig, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PathSelectorConfig.displayName = 'proto.ubo.v1.PathSelectorConfig';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.displayName = 'proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemAction.displayName = 'proto.ubo.v1.FileSystemAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemReportSelectionAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemReportSelectionAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemReportSelectionAction.displayName = 'proto.ubo.v1.FileSystemReportSelectionAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemCopyAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemCopyAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemCopyAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemCopyAction.displayName = 'proto.ubo.v1.FileSystemCopyAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemMoveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemMoveAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemMoveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemMoveAction.displayName = 'proto.ubo.v1.FileSystemMoveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemRemoveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemRemoveAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemRemoveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemRemoveAction.displayName = 'proto.ubo.v1.FileSystemRemoveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemEvent.displayName = 'proto.ubo.v1.FileSystemEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemSelectEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemSelectEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemSelectEvent.displayName = 'proto.ubo.v1.FileSystemSelectEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemCopyEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemCopyEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemCopyEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemCopyEvent.displayName = 'proto.ubo.v1.FileSystemCopyEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemMoveEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemMoveEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemMoveEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemMoveEvent.displayName = 'proto.ubo.v1.FileSystemMoveEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemRemoveEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemRemoveEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemRemoveEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemRemoveEvent.displayName = 'proto.ubo.v1.FileSystemRemoveEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.FileSystemState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.FileSystemState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.FileSystemState.displayName = 'proto.ubo.v1.FileSystemState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredAction.displayName = 'proto.ubo.v1.InfraredAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredEvent.displayName = 'proto.ubo.v1.InfraredEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredHandleReceivedCodeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredHandleReceivedCodeAction.displayName = 'proto.ubo.v1.InfraredHandleReceivedCodeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredSendCodeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredSendCodeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredSendCodeAction.displayName = 'proto.ubo.v1.InfraredSendCodeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredSetShouldPropagateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredSetShouldPropagateAction.displayName = 'proto.ubo.v1.InfraredSetShouldPropagateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredSetShouldReceiveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredSetShouldReceiveAction.displayName = 'proto.ubo.v1.InfraredSetShouldReceiveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredSendCodeEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredSendCodeEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredSendCodeEvent.displayName = 'proto.ubo.v1.InfraredSendCodeEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.InfraredState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.InfraredState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.InfraredState.displayName = 'proto.ubo.v1.InfraredState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IpAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.IpAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IpAction.displayName = 'proto.ubo.v1.IpAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IpEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.IpEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IpEvent.displayName = 'proto.ubo.v1.IpEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IpUpdateInterfacesAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.IpUpdateInterfacesAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.IpUpdateInterfacesAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IpUpdateInterfacesAction.displayName = 'proto.ubo.v1.IpUpdateInterfacesAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IpSetIsConnectedAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.IpSetIsConnectedAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IpSetIsConnectedAction.displayName = 'proto.ubo.v1.IpSetIsConnectedAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IpNetworkInterface = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.IpNetworkInterface.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.IpNetworkInterface, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IpNetworkInterface.displayName = 'proto.ubo.v1.IpNetworkInterface';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.IpState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.IpState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.IpState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.IpState.displayName = 'proto.ubo.v1.IpState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadAction.displayName = 'proto.ubo.v1.KeypadAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadAction.PressedKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadAction.PressedKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadAction.PressedKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadAction.HeldKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadAction.HeldKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadAction.HeldKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyPressAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyPressAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyPressAction.displayName = 'proto.ubo.v1.KeypadKeyPressAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyHoldAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyHoldAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyHoldAction.displayName = 'proto.ubo.v1.KeypadKeyHoldAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyUnholdAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyUnholdAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyUnholdAction.displayName = 'proto.ubo.v1.KeypadKeyUnholdAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyReleaseAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyReleaseAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyReleaseAction.displayName = 'proto.ubo.v1.KeypadKeyReleaseAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.KeypadState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.KeypadState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.KeypadState.displayName = 'proto.ubo.v1.KeypadState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.LightDMAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.LightDMAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.LightDMAction.displayName = 'proto.ubo.v1.LightDMAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.LightDMUpdateStateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.LightDMUpdateStateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.LightDMUpdateStateAction.displayName = 'proto.ubo.v1.LightDMUpdateStateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.LightDMClearEnabledStateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.LightDMClearEnabledStateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.LightDMClearEnabledStateAction.displayName = 'proto.ubo.v1.LightDMClearEnabledStateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.LightDMState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.LightDMState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.LightDMState.displayName = 'proto.ubo.v1.LightDMState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationActionItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationActionItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationActionItem.displayName = 'proto.ubo.v1.NotificationActionItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationApplicationItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationApplicationItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationApplicationItem.displayName = 'proto.ubo.v1.NotificationApplicationItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationApplicationItem.Application = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.NotificationApplicationItem.Application, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationApplicationItem.Application.displayName = 'proto.ubo.v1.NotificationApplicationItem.Application';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.NotificationApplicationItem.InitializationArgs.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationApplicationItem.InitializationArgs, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationApplicationItem.InitializationArgs.displayName = 'proto.ubo.v1.NotificationApplicationItem.InitializationArgs';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.displayName = 'proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationDispatchItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationDispatchItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationDispatchItem.displayName = 'proto.ubo.v1.NotificationDispatchItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2 = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.NotificationDispatchItem.StoreAction2.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationDispatchItem.StoreAction2, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationDispatchItem.StoreAction2.displayName = 'proto.ubo.v1.NotificationDispatchItem.StoreAction2';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.NotificationDispatchItem.StoreAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationDispatchItem.StoreAction.displayName = 'proto.ubo.v1.NotificationDispatchItem.StoreAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Notification = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.Notification, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Notification.displayName = 'proto.ubo.v1.Notification';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Notification.ActionsItem = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Notification.ActionsItem.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.Notification.ActionsItem, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Notification.ActionsItem.displayName = 'proto.ubo.v1.Notification.ActionsItem';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Notification.Actions = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.Notification.Actions.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.Notification.Actions, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Notification.Actions.displayName = 'proto.ubo.v1.Notification.Actions';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Notification.OnClose = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, null);\n};\ngoog.inherits(proto.ubo.v1.Notification.OnClose, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Notification.OnClose.displayName = 'proto.ubo.v1.Notification.OnClose';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsAction.displayName = 'proto.ubo.v1.NotificationsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsAddAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsAddAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsAddAction.displayName = 'proto.ubo.v1.NotificationsAddAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsDisplayAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsDisplayAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsDisplayAction.displayName = 'proto.ubo.v1.NotificationsDisplayAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsClearAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsClearAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsClearAction.displayName = 'proto.ubo.v1.NotificationsClearAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsClearByIdAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsClearByIdAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsClearByIdAction.displayName = 'proto.ubo.v1.NotificationsClearByIdAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsClearAllAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsClearAllAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsClearAllAction.displayName = 'proto.ubo.v1.NotificationsClearAllAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsEvent.displayName = 'proto.ubo.v1.NotificationsEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsClearEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsClearEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsClearEvent.displayName = 'proto.ubo.v1.NotificationsClearEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsDisplayEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsDisplayEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsDisplayEvent.displayName = 'proto.ubo.v1.NotificationsDisplayEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.NotificationsState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.NotificationsState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.NotificationsState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.NotificationsState.displayName = 'proto.ubo.v1.NotificationsState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingAction.displayName = 'proto.ubo.v1.RgbRingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingEvent.displayName = 'proto.ubo.v1.RgbRingEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingSetIsBusyAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingSetIsBusyAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingSetIsBusyAction.displayName = 'proto.ubo.v1.RgbRingSetIsBusyAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingCommandAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingCommandAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingCommandAction.displayName = 'proto.ubo.v1.RgbRingCommandAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingWaitableCommandAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingWaitableCommandAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingWaitableCommandAction.displayName = 'proto.ubo.v1.RgbRingWaitableCommandAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingColorfulCommandAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingColorfulCommandAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingColorfulCommandAction.displayName = 'proto.ubo.v1.RgbRingColorfulCommandAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingSetEnabledAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingSetEnabledAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingSetEnabledAction.displayName = 'proto.ubo.v1.RgbRingSetEnabledAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingSetAllAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingSetAllAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingSetAllAction.displayName = 'proto.ubo.v1.RgbRingSetAllAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingSetBrightnessAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingSetBrightnessAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingSetBrightnessAction.displayName = 'proto.ubo.v1.RgbRingSetBrightnessAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingBlankAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingBlankAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingBlankAction.displayName = 'proto.ubo.v1.RgbRingBlankAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingRainbowAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingRainbowAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingRainbowAction.displayName = 'proto.ubo.v1.RgbRingRainbowAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingProgressWheelStepAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingProgressWheelStepAction.displayName = 'proto.ubo.v1.RgbRingProgressWheelStepAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingPulseAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingPulseAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingPulseAction.displayName = 'proto.ubo.v1.RgbRingPulseAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingBlinkAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingBlinkAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingBlinkAction.displayName = 'proto.ubo.v1.RgbRingBlinkAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingSpinningWheelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingSpinningWheelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingSpinningWheelAction.displayName = 'proto.ubo.v1.RgbRingSpinningWheelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingProgressWheelAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingProgressWheelAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingProgressWheelAction.displayName = 'proto.ubo.v1.RgbRingProgressWheelAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingFillUptoAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingFillUptoAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingFillUptoAction.displayName = 'proto.ubo.v1.RgbRingFillUptoAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingFillDownfromAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingFillDownfromAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingFillDownfromAction.displayName = 'proto.ubo.v1.RgbRingFillDownfromAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingSequenceAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.RgbRingSequenceAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingSequenceAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingSequenceAction.displayName = 'proto.ubo.v1.RgbRingSequenceAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingCommandEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.RgbRingCommandEvent.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingCommandEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingCommandEvent.displayName = 'proto.ubo.v1.RgbRingCommandEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbRingState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RgbRingState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbRingState.displayName = 'proto.ubo.v1.RgbRingState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbColorElement = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.RgbColorElement.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.RgbColorElement, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbColorElement.displayName = 'proto.ubo.v1.RgbColorElement';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RgbColor = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.RgbColor.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.RgbColor, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RgbColor.displayName = 'proto.ubo.v1.RgbColor';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectAction.displayName = 'proto.ubo.v1.RPiConnectAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectEvent.displayName = 'proto.ubo.v1.RPiConnectEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectStartDownloadingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectStartDownloadingAction.displayName = 'proto.ubo.v1.RPiConnectStartDownloadingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectDoneDownloadingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectDoneDownloadingAction.displayName = 'proto.ubo.v1.RPiConnectDoneDownloadingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectSetPendingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectSetPendingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectSetPendingAction.displayName = 'proto.ubo.v1.RPiConnectSetPendingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectStatus = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectStatus, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectStatus.displayName = 'proto.ubo.v1.RPiConnectStatus';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectSetStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectSetStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectSetStatusAction.displayName = 'proto.ubo.v1.RPiConnectSetStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectLoginEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectLoginEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectLoginEvent.displayName = 'proto.ubo.v1.RPiConnectLoginEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectUpdateServiceStateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectUpdateServiceStateAction.displayName = 'proto.ubo.v1.RPiConnectUpdateServiceStateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.RPiConnectState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.RPiConnectState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.RPiConnectState.displayName = 'proto.ubo.v1.RPiConnectState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SensorsAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SensorsAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SensorsAction.displayName = 'proto.ubo.v1.SensorsAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SensorsReportReadingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SensorsReportReadingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SensorsReportReadingAction.displayName = 'proto.ubo.v1.SensorsReportReadingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SensorState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SensorState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SensorState.displayName = 'proto.ubo.v1.SensorState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SensorsState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SensorsState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SensorsState.displayName = 'proto.ubo.v1.SensorsState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionAction.displayName = 'proto.ubo.v1.SpeechRecognitionAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.displayName = 'proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.displayName = 'proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.displayName = 'proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionIntent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionIntent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionIntent.displayName = 'proto.ubo.v1.SpeechRecognitionIntent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1 = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.SpeechRecognitionIntent.Phrase1.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Phrase1, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionIntent.Phrase1.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Phrase1';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Phrase, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionIntent.Phrase.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Phrase';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2 = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.SpeechRecognitionIntent.Action2.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Action2, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionIntent.Action2.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Action2';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Action, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionIntent.Action.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Action';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.displayName = 'proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.displayName = 'proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionReportSpeechAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionReportSpeechAction.displayName = 'proto.ubo.v1.SpeechRecognitionReportSpeechAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionEvent.displayName = 'proto.ubo.v1.SpeechRecognitionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionReportTextEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionReportTextEvent.displayName = 'proto.ubo.v1.SpeechRecognitionReportTextEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechRecognitionState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.SpeechRecognitionState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechRecognitionState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechRecognitionState.displayName = 'proto.ubo.v1.SpeechRecognitionState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisAction.displayName = 'proto.ubo.v1.SpeechSynthesisAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisEvent.displayName = 'proto.ubo.v1.SpeechSynthesisEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.displayName = 'proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.displayName = 'proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.ReadableInformation = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.ReadableInformation, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.ReadableInformation.displayName = 'proto.ubo.v1.ReadableInformation';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisReadTextAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisReadTextAction.displayName = 'proto.ubo.v1.SpeechSynthesisReadTextAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.displayName = 'proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SpeechSynthesisState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SpeechSynthesisState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SpeechSynthesisState.displayName = 'proto.ubo.v1.SpeechSynthesisState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SSHAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SSHAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SSHAction.displayName = 'proto.ubo.v1.SSHAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SSHUpdateStateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SSHUpdateStateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SSHUpdateStateAction.displayName = 'proto.ubo.v1.SSHUpdateStateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SSHClearEnabledStateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SSHClearEnabledStateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SSHClearEnabledStateAction.displayName = 'proto.ubo.v1.SSHClearEnabledStateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.SSHState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.SSHState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.SSHState.displayName = 'proto.ubo.v1.SSHState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersAction.displayName = 'proto.ubo.v1.UsersAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersSetUsersAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.UsersSetUsersAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.UsersSetUsersAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersSetUsersAction.displayName = 'proto.ubo.v1.UsersSetUsersAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersCreateUserAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersCreateUserAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersCreateUserAction.displayName = 'proto.ubo.v1.UsersCreateUserAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersDeleteUserAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersDeleteUserAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersDeleteUserAction.displayName = 'proto.ubo.v1.UsersDeleteUserAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersResetPasswordAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersResetPasswordAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersResetPasswordAction.displayName = 'proto.ubo.v1.UsersResetPasswordAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersEvent.displayName = 'proto.ubo.v1.UsersEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersCreateUserEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersCreateUserEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersCreateUserEvent.displayName = 'proto.ubo.v1.UsersCreateUserEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersDeleteUserEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersDeleteUserEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersDeleteUserEvent.displayName = 'proto.ubo.v1.UsersDeleteUserEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersResetPasswordEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersResetPasswordEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersResetPasswordEvent.displayName = 'proto.ubo.v1.UsersResetPasswordEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UserState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UserState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UserState.displayName = 'proto.ubo.v1.UserState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.UsersState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersState.displayName = 'proto.ubo.v1.UsersState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.UsersState.Users = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UsersState.Users.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.UsersState.Users, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.UsersState.Users.displayName = 'proto.ubo.v1.UsersState.Users';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeAction.displayName = 'proto.ubo.v1.VSCodeAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeEvent.displayName = 'proto.ubo.v1.VSCodeEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeStartDownloadingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeStartDownloadingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeStartDownloadingAction.displayName = 'proto.ubo.v1.VSCodeStartDownloadingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeDoneDownloadingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeDoneDownloadingAction.displayName = 'proto.ubo.v1.VSCodeDoneDownloadingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeSetPendingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeSetPendingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeSetPendingAction.displayName = 'proto.ubo.v1.VSCodeSetPendingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeStatus = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeStatus, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeStatus.displayName = 'proto.ubo.v1.VSCodeStatus';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeSetStatusAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeSetStatusAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeSetStatusAction.displayName = 'proto.ubo.v1.VSCodeSetStatusAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeLoginEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeLoginEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeLoginEvent.displayName = 'proto.ubo.v1.VSCodeLoginEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeRestartEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeRestartEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeRestartEvent.displayName = 'proto.ubo.v1.VSCodeRestartEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.VSCodeState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.VSCodeState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.VSCodeState.displayName = 'proto.ubo.v1.VSCodeState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WebUIEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WebUIEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WebUIEvent.displayName = 'proto.ubo.v1.WebUIEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WebUIInitializeEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WebUIInitializeEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WebUIInitializeEvent.displayName = 'proto.ubo.v1.WebUIInitializeEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WebUIStopEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WebUIStopEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WebUIStopEvent.displayName = 'proto.ubo.v1.WebUIStopEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WebUIState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.WebUIState.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.WebUIState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WebUIState.displayName = 'proto.ubo.v1.WebUIState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiConnection = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiConnection, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiConnection.displayName = 'proto.ubo.v1.WiFiConnection';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiAction.displayName = 'proto.ubo.v1.WiFiAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiInputConnectionAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiInputConnectionAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiInputConnectionAction.displayName = 'proto.ubo.v1.WiFiInputConnectionAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiSetHasVisitedOnboardingAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.displayName = 'proto.ubo.v1.WiFiSetHasVisitedOnboardingAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiUpdateAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.WiFiUpdateAction.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiUpdateAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiUpdateAction.displayName = 'proto.ubo.v1.WiFiUpdateAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiUpdateRequestAction = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiUpdateRequestAction, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiUpdateRequestAction.displayName = 'proto.ubo.v1.WiFiUpdateRequestAction';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiEvent.displayName = 'proto.ubo.v1.WiFiEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiInputConnectionEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiInputConnectionEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiInputConnectionEvent.displayName = 'proto.ubo.v1.WiFiInputConnectionEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiUpdateRequestEvent = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiUpdateRequestEvent, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiUpdateRequestEvent.displayName = 'proto.ubo.v1.WiFiUpdateRequestEvent';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiState = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, 500, null, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiState, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiState.displayName = 'proto.ubo.v1.WiFiState';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.WiFiState.Connections = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.WiFiState.Connections.repeatedFields_, null);\n};\ngoog.inherits(proto.ubo.v1.WiFiState.Connections, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.WiFiState.Connections.displayName = 'proto.ubo.v1.WiFiState.Connections';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Action = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Action.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.Action, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Action.displayName = 'proto.ubo.v1.Action';\n}\n/**\n * Generated by JsPbCodeGenerator.\n * @param {Array=} opt_data Optional initial data array, typically from a\n * server response, or constructed directly in Javascript. The array is used\n * in place and becomes part of the constructed object. It is not cloned.\n * If no data is provided, the constructed object will be empty, but still\n * valid.\n * @extends {jspb.Message}\n * @constructor\n */\nproto.ubo.v1.Event = function(opt_data) {\n  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Event.oneofGroups_);\n};\ngoog.inherits(proto.ubo.v1.Event, jspb.Message);\nif (goog.DEBUG && !COMPILED) {\n  /**\n   * @public\n   * @override\n   */\n  proto.ubo.v1.Event.displayName = 'proto.ubo.v1.Event';\n}\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.BaseMenu.repeatedFields_ = [3];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.BaseMenu.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.BaseMenu.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.BaseMenu} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.BaseMenu.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Item.toObject, includeInstance),\n    placeholder: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.BaseMenu}\n */\nproto.ubo.v1.BaseMenu.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.BaseMenu;\n  return proto.ubo.v1.BaseMenu.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.BaseMenu} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.BaseMenu}\n */\nproto.ubo.v1.BaseMenu.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.Item;\n      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPlaceholder(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.BaseMenu.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.BaseMenu.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.BaseMenu} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.BaseMenu.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTitle();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      3,\n      f,\n      proto.ubo.v1.Item.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.BaseMenu.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.BaseMenu} returns this\n */\nproto.ubo.v1.BaseMenu.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BaseMenu} returns this\n */\nproto.ubo.v1.BaseMenu.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BaseMenu.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string title = 2;\n * @return {string}\n */\nproto.ubo.v1.BaseMenu.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.BaseMenu} returns this\n */\nproto.ubo.v1.BaseMenu.prototype.setTitle = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * repeated Item items = 3;\n * @return {!Array<!proto.ubo.v1.Item>}\n */\nproto.ubo.v1.BaseMenu.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Item>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Item, 3));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Item>} value\n * @return {!proto.ubo.v1.BaseMenu} returns this\n*/\nproto.ubo.v1.BaseMenu.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 3, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Item=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Item}\n */\nproto.ubo.v1.BaseMenu.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.Item, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.BaseMenu} returns this\n */\nproto.ubo.v1.BaseMenu.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string placeholder = 4;\n * @return {string}\n */\nproto.ubo.v1.BaseMenu.prototype.getPlaceholder = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.BaseMenu} returns this\n */\nproto.ubo.v1.BaseMenu.prototype.setPlaceholder = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BaseMenu} returns this\n */\nproto.ubo.v1.BaseMenu.prototype.clearPlaceholder = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BaseMenu.prototype.hasPlaceholder = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.HeadedMenu.repeatedFields_ = [5];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.HeadedMenu.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.HeadedMenu.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.HeadedMenu} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.HeadedMenu.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    heading: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    subHeading: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Item.toObject, includeInstance),\n    placeholder: jspb.Message.getFieldWithDefault(msg, 6, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.HeadedMenu}\n */\nproto.ubo.v1.HeadedMenu.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.HeadedMenu;\n  return proto.ubo.v1.HeadedMenu.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.HeadedMenu} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.HeadedMenu}\n */\nproto.ubo.v1.HeadedMenu.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setHeading(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSubHeading(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.Item;\n      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPlaceholder(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.HeadedMenu.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.HeadedMenu.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.HeadedMenu} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.HeadedMenu.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getHeading();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getSubHeading();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getTitle();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      5,\n      f,\n      proto.ubo.v1.Item.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.HeadedMenu.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.HeadedMenu.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string heading = 2;\n * @return {string}\n */\nproto.ubo.v1.HeadedMenu.prototype.getHeading = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.setHeading = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string sub_heading = 3;\n * @return {string}\n */\nproto.ubo.v1.HeadedMenu.prototype.getSubHeading = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.setSubHeading = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional string title = 4;\n * @return {string}\n */\nproto.ubo.v1.HeadedMenu.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.setTitle = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n/**\n * repeated Item items = 5;\n * @return {!Array<!proto.ubo.v1.Item>}\n */\nproto.ubo.v1.HeadedMenu.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Item>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Item, 5));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Item>} value\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n*/\nproto.ubo.v1.HeadedMenu.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 5, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Item=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Item}\n */\nproto.ubo.v1.HeadedMenu.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.ubo.v1.Item, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string placeholder = 6;\n * @return {string}\n */\nproto.ubo.v1.HeadedMenu.prototype.getPlaceholder = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.setPlaceholder = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.HeadedMenu} returns this\n */\nproto.ubo.v1.HeadedMenu.prototype.clearPlaceholder = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.HeadedMenu.prototype.hasPlaceholder = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.HeadlessMenu.repeatedFields_ = [3];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.HeadlessMenu.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.HeadlessMenu.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.HeadlessMenu} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.HeadlessMenu.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Item.toObject, includeInstance),\n    placeholder: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.HeadlessMenu}\n */\nproto.ubo.v1.HeadlessMenu.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.HeadlessMenu;\n  return proto.ubo.v1.HeadlessMenu.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.HeadlessMenu} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.HeadlessMenu}\n */\nproto.ubo.v1.HeadlessMenu.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.Item;\n      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPlaceholder(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.HeadlessMenu.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.HeadlessMenu.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.HeadlessMenu} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.HeadlessMenu.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTitle();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      3,\n      f,\n      proto.ubo.v1.Item.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.HeadlessMenu.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n */\nproto.ubo.v1.HeadlessMenu.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n */\nproto.ubo.v1.HeadlessMenu.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.HeadlessMenu.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string title = 2;\n * @return {string}\n */\nproto.ubo.v1.HeadlessMenu.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n */\nproto.ubo.v1.HeadlessMenu.prototype.setTitle = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * repeated Item items = 3;\n * @return {!Array<!proto.ubo.v1.Item>}\n */\nproto.ubo.v1.HeadlessMenu.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Item>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Item, 3));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Item>} value\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n*/\nproto.ubo.v1.HeadlessMenu.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 3, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Item=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Item}\n */\nproto.ubo.v1.HeadlessMenu.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.Item, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n */\nproto.ubo.v1.HeadlessMenu.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string placeholder = 4;\n * @return {string}\n */\nproto.ubo.v1.HeadlessMenu.prototype.getPlaceholder = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n */\nproto.ubo.v1.HeadlessMenu.prototype.setPlaceholder = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.HeadlessMenu} returns this\n */\nproto.ubo.v1.HeadlessMenu.prototype.clearPlaceholder = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.HeadlessMenu.prototype.hasPlaceholder = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Item.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Item.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Item} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Item.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Item}\n */\nproto.ubo.v1.Item.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Item;\n  return proto.ubo.v1.Item.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Item} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Item}\n */\nproto.ubo.v1.Item.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 7:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 8:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Item.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Item.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Item} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Item.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeBool(\n      7,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeFloat(\n      8,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeFloat(\n      9,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.Item.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string key = 2;\n * @return {string}\n */\nproto.ubo.v1.Item.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string label = 3;\n * @return {string}\n */\nproto.ubo.v1.Item.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string color = 4;\n * @return {string}\n */\nproto.ubo.v1.Item.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string background_color = 5;\n * @return {string}\n */\nproto.ubo.v1.Item.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string icon = 6;\n * @return {string}\n */\nproto.ubo.v1.Item.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional bool is_short = 7;\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional float opacity = 8;\n * @return {number}\n */\nproto.ubo.v1.Item.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional float progress = 9;\n * @return {number}\n */\nproto.ubo.v1.Item.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Item} returns this\n */\nproto.ubo.v1.Item.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Item.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ActionItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ActionItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ActionItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ActionItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ActionItem}\n */\nproto.ubo.v1.ActionItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ActionItem;\n  return proto.ubo.v1.ActionItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ActionItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ActionItem}\n */\nproto.ubo.v1.ActionItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 7:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 8:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ActionItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ActionItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ActionItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ActionItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeBool(\n      7,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeFloat(\n      8,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeFloat(\n      9,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ActionItem.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string key = 2;\n * @return {string}\n */\nproto.ubo.v1.ActionItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string label = 3;\n * @return {string}\n */\nproto.ubo.v1.ActionItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string color = 4;\n * @return {string}\n */\nproto.ubo.v1.ActionItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string background_color = 5;\n * @return {string}\n */\nproto.ubo.v1.ActionItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string icon = 6;\n * @return {string}\n */\nproto.ubo.v1.ActionItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional bool is_short = 7;\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional float opacity = 8;\n * @return {number}\n */\nproto.ubo.v1.ActionItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional float progress = 9;\n * @return {number}\n */\nproto.ubo.v1.ActionItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ActionItem} returns this\n */\nproto.ubo.v1.ActionItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ActionItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ApplicationItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ApplicationItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ApplicationItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ApplicationItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    application: (f = msg.getApplication()) && proto.ubo.v1.ApplicationItem.Application.toObject(includeInstance, f),\n    key: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ApplicationItem}\n */\nproto.ubo.v1.ApplicationItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ApplicationItem;\n  return proto.ubo.v1.ApplicationItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ApplicationItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ApplicationItem}\n */\nproto.ubo.v1.ApplicationItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.ApplicationItem.Application;\n      reader.readMessage(value,proto.ubo.v1.ApplicationItem.Application.deserializeBinaryFromReader);\n      msg.setApplication(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 8:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 10:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ApplicationItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ApplicationItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ApplicationItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ApplicationItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getApplication();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.ApplicationItem.Application.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeBool(\n      8,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeFloat(\n      9,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeFloat(\n      10,\n      f\n    );\n  }\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.ApplicationItem.Application.oneofGroups_ = [[1]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.ApplicationItem.Application.ApplicationCase = {\n  APPLICATION_NOT_SET: 0,\n  STRING: 1\n};\n\n/**\n * @return {proto.ubo.v1.ApplicationItem.Application.ApplicationCase}\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.getApplicationCase = function() {\n  return /** @type {proto.ubo.v1.ApplicationItem.Application.ApplicationCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.ApplicationItem.Application.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ApplicationItem.Application.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ApplicationItem.Application} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ApplicationItem.Application.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    string: jspb.Message.getFieldWithDefault(msg, 1, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ApplicationItem.Application}\n */\nproto.ubo.v1.ApplicationItem.Application.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ApplicationItem.Application;\n  return proto.ubo.v1.ApplicationItem.Application.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ApplicationItem.Application} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ApplicationItem.Application}\n */\nproto.ubo.v1.ApplicationItem.Application.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setString(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ApplicationItem.Application.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ApplicationItem.Application} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ApplicationItem.Application.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string string = 1;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.getString = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem.Application} returns this\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.setString = function(value) {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.ApplicationItem.Application.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem.Application} returns this\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.clearString = function() {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.ApplicationItem.Application.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.Application.prototype.hasString = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Application application = 2;\n * @return {?proto.ubo.v1.ApplicationItem.Application}\n */\nproto.ubo.v1.ApplicationItem.prototype.getApplication = function() {\n  return /** @type{?proto.ubo.v1.ApplicationItem.Application} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ApplicationItem.Application, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ApplicationItem.Application|undefined} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n*/\nproto.ubo.v1.ApplicationItem.prototype.setApplication = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearApplication = function() {\n  return this.setApplication(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasApplication = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string key = 3;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string label = 4;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string color = 5;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string background_color = 6;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string icon = 7;\n * @return {string}\n */\nproto.ubo.v1.ApplicationItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional bool is_short = 8;\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional float opacity = 9;\n * @return {number}\n */\nproto.ubo.v1.ApplicationItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional float progress = 10;\n * @return {number}\n */\nproto.ubo.v1.ApplicationItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ApplicationItem} returns this\n */\nproto.ubo.v1.ApplicationItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ApplicationItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SubMenuItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SubMenuItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SubMenuItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SubMenuItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    subMenu: (f = msg.getSubMenu()) && proto.ubo.v1.Menu.toObject(includeInstance, f),\n    key: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SubMenuItem}\n */\nproto.ubo.v1.SubMenuItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SubMenuItem;\n  return proto.ubo.v1.SubMenuItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SubMenuItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SubMenuItem}\n */\nproto.ubo.v1.SubMenuItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Menu;\n      reader.readMessage(value,proto.ubo.v1.Menu.deserializeBinaryFromReader);\n      msg.setSubMenu(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 8:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 10:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SubMenuItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SubMenuItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SubMenuItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SubMenuItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSubMenu();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Menu.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeBool(\n      8,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeFloat(\n      9,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeFloat(\n      10,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SubMenuItem.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Menu sub_menu = 2;\n * @return {?proto.ubo.v1.Menu}\n */\nproto.ubo.v1.SubMenuItem.prototype.getSubMenu = function() {\n  return /** @type{?proto.ubo.v1.Menu} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Menu, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Menu|undefined} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n*/\nproto.ubo.v1.SubMenuItem.prototype.setSubMenu = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearSubMenu = function() {\n  return this.setSubMenu(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasSubMenu = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string key = 3;\n * @return {string}\n */\nproto.ubo.v1.SubMenuItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string label = 4;\n * @return {string}\n */\nproto.ubo.v1.SubMenuItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string color = 5;\n * @return {string}\n */\nproto.ubo.v1.SubMenuItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string background_color = 6;\n * @return {string}\n */\nproto.ubo.v1.SubMenuItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string icon = 7;\n * @return {string}\n */\nproto.ubo.v1.SubMenuItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional bool is_short = 8;\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional float opacity = 9;\n * @return {number}\n */\nproto.ubo.v1.SubMenuItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional float progress = 10;\n * @return {number}\n */\nproto.ubo.v1.SubMenuItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SubMenuItem} returns this\n */\nproto.ubo.v1.SubMenuItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SubMenuItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.Menu.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Menu.MenuCase = {\n  MENU_NOT_SET: 0,\n  HEADED_MENU: 1,\n  HEADLESS_MENU: 2\n};\n\n/**\n * @return {proto.ubo.v1.Menu.MenuCase}\n */\nproto.ubo.v1.Menu.prototype.getMenuCase = function() {\n  return /** @type {proto.ubo.v1.Menu.MenuCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Menu.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Menu.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Menu.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Menu} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Menu.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    headedMenu: (f = msg.getHeadedMenu()) && proto.ubo.v1.HeadedMenu.toObject(includeInstance, f),\n    headlessMenu: (f = msg.getHeadlessMenu()) && proto.ubo.v1.HeadlessMenu.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Menu}\n */\nproto.ubo.v1.Menu.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Menu;\n  return proto.ubo.v1.Menu.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Menu} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Menu}\n */\nproto.ubo.v1.Menu.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.HeadedMenu;\n      reader.readMessage(value,proto.ubo.v1.HeadedMenu.deserializeBinaryFromReader);\n      msg.setHeadedMenu(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.HeadlessMenu;\n      reader.readMessage(value,proto.ubo.v1.HeadlessMenu.deserializeBinaryFromReader);\n      msg.setHeadlessMenu(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Menu.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Menu.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Menu} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Menu.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getHeadedMenu();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.HeadedMenu.serializeBinaryToWriter\n    );\n  }\n  f = message.getHeadlessMenu();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.HeadlessMenu.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional HeadedMenu headed_menu = 1;\n * @return {?proto.ubo.v1.HeadedMenu}\n */\nproto.ubo.v1.Menu.prototype.getHeadedMenu = function() {\n  return /** @type{?proto.ubo.v1.HeadedMenu} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.HeadedMenu, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.HeadedMenu|undefined} value\n * @return {!proto.ubo.v1.Menu} returns this\n*/\nproto.ubo.v1.Menu.prototype.setHeadedMenu = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Menu.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Menu} returns this\n */\nproto.ubo.v1.Menu.prototype.clearHeadedMenu = function() {\n  return this.setHeadedMenu(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Menu.prototype.hasHeadedMenu = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional HeadlessMenu headless_menu = 2;\n * @return {?proto.ubo.v1.HeadlessMenu}\n */\nproto.ubo.v1.Menu.prototype.getHeadlessMenu = function() {\n  return /** @type{?proto.ubo.v1.HeadlessMenu} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.HeadlessMenu, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.HeadlessMenu|undefined} value\n * @return {!proto.ubo.v1.Menu} returns this\n*/\nproto.ubo.v1.Menu.prototype.setHeadlessMenu = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Menu.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Menu} returns this\n */\nproto.ubo.v1.Menu.prototype.clearHeadlessMenu = function() {\n  return this.setHeadlessMenu(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Menu.prototype.hasHeadlessMenu = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainAction}\n */\nproto.ubo.v1.MainAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainAction;\n  return proto.ubo.v1.MainAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainAction}\n */\nproto.ubo.v1.MainAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MainAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MainAction} returns this\n */\nproto.ubo.v1.MainAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainAction} returns this\n */\nproto.ubo.v1.MainAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateLightDMState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateLightDMState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateLightDMState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateLightDMState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isEnable: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateLightDMState}\n */\nproto.ubo.v1.UpdateLightDMState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateLightDMState;\n  return proto.ubo.v1.UpdateLightDMState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateLightDMState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateLightDMState}\n */\nproto.ubo.v1.UpdateLightDMState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnable(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateLightDMState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateLightDMState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateLightDMState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateLightDMState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsActive();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getIsEnable();\n  if (f) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateLightDMState.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateLightDMState} returns this\n */\nproto.ubo.v1.UpdateLightDMState.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateLightDMState} returns this\n */\nproto.ubo.v1.UpdateLightDMState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateLightDMState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.UpdateLightDMState.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.UpdateLightDMState} returns this\n */\nproto.ubo.v1.UpdateLightDMState.prototype.setIsActive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional bool is_enable = 3;\n * @return {boolean}\n */\nproto.ubo.v1.UpdateLightDMState.prototype.getIsEnable = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.UpdateLightDMState} returns this\n */\nproto.ubo.v1.UpdateLightDMState.prototype.setIsEnable = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RegisterAppAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RegisterAppAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RegisterAppAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RegisterAppAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    menuItem: (f = msg.getMenuItem()) && proto.ubo.v1.Item.toObject(includeInstance, f),\n    service: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RegisterAppAction}\n */\nproto.ubo.v1.RegisterAppAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RegisterAppAction;\n  return proto.ubo.v1.RegisterAppAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RegisterAppAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RegisterAppAction}\n */\nproto.ubo.v1.RegisterAppAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Item;\n      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);\n      msg.setMenuItem(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setService(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RegisterAppAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RegisterAppAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RegisterAppAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RegisterAppAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getMenuItem();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Item.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.RegisterAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Item menu_item = 2;\n * @return {?proto.ubo.v1.Item}\n */\nproto.ubo.v1.RegisterAppAction.prototype.getMenuItem = function() {\n  return /** @type{?proto.ubo.v1.Item} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Item, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Item|undefined} value\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n*/\nproto.ubo.v1.RegisterAppAction.prototype.setMenuItem = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.clearMenuItem = function() {\n  return this.setMenuItem(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterAppAction.prototype.hasMenuItem = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string service = 3;\n * @return {string}\n */\nproto.ubo.v1.RegisterAppAction.prototype.getService = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.setService = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.clearService = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterAppAction.prototype.hasService = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string key = 4;\n * @return {string}\n */\nproto.ubo.v1.RegisterAppAction.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterAppAction} returns this\n */\nproto.ubo.v1.RegisterAppAction.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterAppAction.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RegisterRegularAppAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RegisterRegularAppAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RegisterRegularAppAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    priority: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    menuItem: (f = msg.getMenuItem()) && proto.ubo.v1.Item.toObject(includeInstance, f),\n    service: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 5, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RegisterRegularAppAction}\n */\nproto.ubo.v1.RegisterRegularAppAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RegisterRegularAppAction;\n  return proto.ubo.v1.RegisterRegularAppAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RegisterRegularAppAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RegisterRegularAppAction}\n */\nproto.ubo.v1.RegisterRegularAppAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setPriority(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.Item;\n      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);\n      msg.setMenuItem(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setService(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RegisterRegularAppAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RegisterRegularAppAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RegisterRegularAppAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = message.getMenuItem();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.Item.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 priority = 2;\n * @return {number}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.getPriority = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.setPriority = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.clearPriority = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.hasPriority = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional Item menu_item = 3;\n * @return {?proto.ubo.v1.Item}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.getMenuItem = function() {\n  return /** @type{?proto.ubo.v1.Item} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Item, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Item|undefined} value\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n*/\nproto.ubo.v1.RegisterRegularAppAction.prototype.setMenuItem = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.clearMenuItem = function() {\n  return this.setMenuItem(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.hasMenuItem = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string service = 4;\n * @return {string}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.getService = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.setService = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.clearService = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.hasService = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string key = 5;\n * @return {string}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterRegularAppAction.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DeregisterRegularAppAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DeregisterRegularAppAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DeregisterRegularAppAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    service: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DeregisterRegularAppAction}\n */\nproto.ubo.v1.DeregisterRegularAppAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DeregisterRegularAppAction;\n  return proto.ubo.v1.DeregisterRegularAppAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DeregisterRegularAppAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DeregisterRegularAppAction}\n */\nproto.ubo.v1.DeregisterRegularAppAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setService(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DeregisterRegularAppAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DeregisterRegularAppAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DeregisterRegularAppAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service = 2;\n * @return {string}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.getService = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.setService = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.clearService = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.hasService = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string key = 3;\n * @return {string}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DeregisterRegularAppAction.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RegisterSettingAppAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RegisterSettingAppAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RegisterSettingAppAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    category: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    priority: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    menuItem: (f = msg.getMenuItem()) && proto.ubo.v1.Item.toObject(includeInstance, f),\n    service: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 6, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RegisterSettingAppAction}\n */\nproto.ubo.v1.RegisterSettingAppAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RegisterSettingAppAction;\n  return proto.ubo.v1.RegisterSettingAppAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RegisterSettingAppAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RegisterSettingAppAction}\n */\nproto.ubo.v1.RegisterSettingAppAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.SettingsCategory} */ (reader.readEnum());\n      msg.setCategory(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setPriority(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.Item;\n      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);\n      msg.setMenuItem(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setService(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RegisterSettingAppAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RegisterSettingAppAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RegisterSettingAppAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getCategory();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getMenuItem();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.Item.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SettingsCategory category = 2;\n * @return {!proto.ubo.v1.SettingsCategory}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.getCategory = function() {\n  return /** @type {!proto.ubo.v1.SettingsCategory} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SettingsCategory} value\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.setCategory = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional int64 priority = 3;\n * @return {number}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.getPriority = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.setPriority = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.clearPriority = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.hasPriority = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional Item menu_item = 4;\n * @return {?proto.ubo.v1.Item}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.getMenuItem = function() {\n  return /** @type{?proto.ubo.v1.Item} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Item, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Item|undefined} value\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n*/\nproto.ubo.v1.RegisterSettingAppAction.prototype.setMenuItem = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.clearMenuItem = function() {\n  return this.setMenuItem(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.hasMenuItem = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string service = 5;\n * @return {string}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.getService = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.setService = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.clearService = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.hasService = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string key = 6;\n * @return {string}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RegisterSettingAppAction.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PowerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PowerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PowerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PowerAction}\n */\nproto.ubo.v1.PowerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PowerAction;\n  return proto.ubo.v1.PowerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PowerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PowerAction}\n */\nproto.ubo.v1.PowerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PowerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PowerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PowerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.PowerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PowerAction} returns this\n */\nproto.ubo.v1.PowerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PowerAction} returns this\n */\nproto.ubo.v1.PowerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PowerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PowerOffAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PowerOffAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PowerOffAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerOffAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PowerOffAction}\n */\nproto.ubo.v1.PowerOffAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PowerOffAction;\n  return proto.ubo.v1.PowerOffAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PowerOffAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PowerOffAction}\n */\nproto.ubo.v1.PowerOffAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PowerOffAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PowerOffAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PowerOffAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerOffAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.PowerOffAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PowerOffAction} returns this\n */\nproto.ubo.v1.PowerOffAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PowerOffAction} returns this\n */\nproto.ubo.v1.PowerOffAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PowerOffAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RebootAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RebootAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RebootAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RebootAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RebootAction}\n */\nproto.ubo.v1.RebootAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RebootAction;\n  return proto.ubo.v1.RebootAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RebootAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RebootAction}\n */\nproto.ubo.v1.RebootAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RebootAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RebootAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RebootAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RebootAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.RebootAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RebootAction} returns this\n */\nproto.ubo.v1.RebootAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RebootAction} returns this\n */\nproto.ubo.v1.RebootAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RebootAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.SetMenuPathAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SetMenuPathAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SetMenuPathAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SetMenuPathAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SetMenuPathAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    pathList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,\n    depth: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SetMenuPathAction}\n */\nproto.ubo.v1.SetMenuPathAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SetMenuPathAction;\n  return proto.ubo.v1.SetMenuPathAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SetMenuPathAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SetMenuPathAction}\n */\nproto.ubo.v1.SetMenuPathAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addPath(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setDepth(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SetMenuPathAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SetMenuPathAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SetMenuPathAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SetMenuPathAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getPathList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n  f = message.getDepth();\n  if (f !== 0) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SetMenuPathAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SetMenuPathAction} returns this\n */\nproto.ubo.v1.SetMenuPathAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SetMenuPathAction} returns this\n */\nproto.ubo.v1.SetMenuPathAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SetMenuPathAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string path = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.SetMenuPathAction.prototype.getPathList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.SetMenuPathAction} returns this\n */\nproto.ubo.v1.SetMenuPathAction.prototype.setPathList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.SetMenuPathAction} returns this\n */\nproto.ubo.v1.SetMenuPathAction.prototype.addPath = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.SetMenuPathAction} returns this\n */\nproto.ubo.v1.SetMenuPathAction.prototype.clearPathList = function() {\n  return this.setPathList([]);\n};\n\n\n/**\n * optional int64 depth = 3;\n * @return {number}\n */\nproto.ubo.v1.SetMenuPathAction.prototype.getDepth = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SetMenuPathAction} returns this\n */\nproto.ubo.v1.SetMenuPathAction.prototype.setDepth = function(value) {\n  return jspb.Message.setProto3IntField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SetAreEnclosuresVisibleAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SetAreEnclosuresVisibleAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isHeaderVisible: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isFooterVisible: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SetAreEnclosuresVisibleAction;\n  return proto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SetAreEnclosuresVisibleAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsHeaderVisible(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsFooterVisible(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SetAreEnclosuresVisibleAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SetAreEnclosuresVisibleAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_header_visible = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.getIsHeaderVisible = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.setIsHeaderVisible = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.clearIsHeaderVisible = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.hasIsHeaderVisible = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_footer_visible = 3;\n * @return {boolean}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.getIsFooterVisible = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.setIsFooterVisible = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.clearIsFooterVisible = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.hasIsFooterVisible = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuAction}\n */\nproto.ubo.v1.MenuAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuAction;\n  return proto.ubo.v1.MenuAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuAction}\n */\nproto.ubo.v1.MenuAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuAction} returns this\n */\nproto.ubo.v1.MenuAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuAction} returns this\n */\nproto.ubo.v1.MenuAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuGoBackAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuGoBackAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuGoBackAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoBackAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuGoBackAction}\n */\nproto.ubo.v1.MenuGoBackAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuGoBackAction;\n  return proto.ubo.v1.MenuGoBackAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuGoBackAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuGoBackAction}\n */\nproto.ubo.v1.MenuGoBackAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuGoBackAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuGoBackAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuGoBackAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoBackAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuGoBackAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuGoBackAction} returns this\n */\nproto.ubo.v1.MenuGoBackAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuGoBackAction} returns this\n */\nproto.ubo.v1.MenuGoBackAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuGoBackAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuGoHomeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuGoHomeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuGoHomeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoHomeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuGoHomeAction}\n */\nproto.ubo.v1.MenuGoHomeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuGoHomeAction;\n  return proto.ubo.v1.MenuGoHomeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuGoHomeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuGoHomeAction}\n */\nproto.ubo.v1.MenuGoHomeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuGoHomeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuGoHomeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuGoHomeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoHomeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuGoHomeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuGoHomeAction} returns this\n */\nproto.ubo.v1.MenuGoHomeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuGoHomeAction} returns this\n */\nproto.ubo.v1.MenuGoHomeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuGoHomeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuChooseByIconAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuChooseByIconAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIconAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuChooseByIconAction}\n */\nproto.ubo.v1.MenuChooseByIconAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuChooseByIconAction;\n  return proto.ubo.v1.MenuChooseByIconAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuChooseByIconAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuChooseByIconAction}\n */\nproto.ubo.v1.MenuChooseByIconAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuChooseByIconAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuChooseByIconAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIconAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIcon();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByIconAction} returns this\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuChooseByIconAction} returns this\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string icon = 2;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByIconAction} returns this\n */\nproto.ubo.v1.MenuChooseByIconAction.prototype.setIcon = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuChooseByLabelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuChooseByLabelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByLabelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuChooseByLabelAction}\n */\nproto.ubo.v1.MenuChooseByLabelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuChooseByLabelAction;\n  return proto.ubo.v1.MenuChooseByLabelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuChooseByLabelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuChooseByLabelAction}\n */\nproto.ubo.v1.MenuChooseByLabelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuChooseByLabelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuChooseByLabelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByLabelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getLabel();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByLabelAction} returns this\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuChooseByLabelAction} returns this\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string label = 2;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByLabelAction} returns this\n */\nproto.ubo.v1.MenuChooseByLabelAction.prototype.setLabel = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuChooseByIndexAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuChooseByIndexAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIndexAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuChooseByIndexAction}\n */\nproto.ubo.v1.MenuChooseByIndexAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuChooseByIndexAction;\n  return proto.ubo.v1.MenuChooseByIndexAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuChooseByIndexAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuChooseByIndexAction}\n */\nproto.ubo.v1.MenuChooseByIndexAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuChooseByIndexAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuChooseByIndexAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIndexAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByIndexAction} returns this\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuChooseByIndexAction} returns this\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 index = 2;\n * @return {number}\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.MenuChooseByIndexAction} returns this\n */\nproto.ubo.v1.MenuChooseByIndexAction.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuScrollAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuScrollAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuScrollAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuScrollAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    direction: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuScrollAction}\n */\nproto.ubo.v1.MenuScrollAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuScrollAction;\n  return proto.ubo.v1.MenuScrollAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuScrollAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuScrollAction}\n */\nproto.ubo.v1.MenuScrollAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.MenuScrollDirection} */ (reader.readEnum());\n      msg.setDirection(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuScrollAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuScrollAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuScrollAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuScrollAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDirection();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuScrollAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuScrollAction} returns this\n */\nproto.ubo.v1.MenuScrollAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuScrollAction} returns this\n */\nproto.ubo.v1.MenuScrollAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuScrollAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional MenuScrollDirection direction = 2;\n * @return {!proto.ubo.v1.MenuScrollDirection}\n */\nproto.ubo.v1.MenuScrollAction.prototype.getDirection = function() {\n  return /** @type {!proto.ubo.v1.MenuScrollDirection} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.MenuScrollDirection} value\n * @return {!proto.ubo.v1.MenuScrollAction} returns this\n */\nproto.ubo.v1.MenuScrollAction.prototype.setDirection = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    applicationId: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.OpenApplicationAction.InitializationArgs.toObject(includeInstance, f),\n    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationAction}\n */\nproto.ubo.v1.OpenApplicationAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationAction;\n  return proto.ubo.v1.OpenApplicationAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationAction}\n */\nproto.ubo.v1.OpenApplicationAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setApplicationId(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.OpenApplicationAction.InitializationArgs;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinaryFromReader);\n      msg.setInitializationArgs(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinaryFromReader);\n      msg.setInitializationKwargs(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getApplicationId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getInitializationArgs();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.OpenApplicationAction.InitializationArgs.serializeBinaryToWriter\n    );\n  }\n  f = message.getInitializationKwargs();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationAction.InitializationArgs.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.BasicType.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationArgs;\n  return proto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationAction.InitializationArgs.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated BasicType items = 1;\n * @return {!Array<!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.BasicType>} value\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} returns this\n*/\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.BasicType=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} returns this\n */\nproto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.BasicType.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2;\n  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated BasicType items = 1;\n * @return {!Array<!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.BasicType>} value\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} returns this\n*/\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.BasicType=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} returns this\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase = {\n  INITIALIZATION_KWARGS_VALUE_NOT_SET: 0,\n  BASIC_TYPE: 1,\n  LIST: 2\n};\n\n/**\n * @return {proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.getInitializationKwargsValueCase = function() {\n  return /** @type {proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    basicType: (f = msg.getBasicType()) && proto.ubo.v1.BasicType.toObject(includeInstance, f),\n    list: (f = msg.getList()) && proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue;\n  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.setBasicType(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinaryFromReader);\n      msg.setList(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getBasicType();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n  f = message.getList();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional BasicType basic_type = 1;\n * @return {?proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.getBasicType = function() {\n  return /** @type{?proto.ubo.v1.BasicType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.BasicType|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this\n*/\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.setBasicType = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.clearBasicType = function() {\n  return this.setBasicType(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.hasBasicType = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional InitializationKwargsValue2 list = 2;\n * @return {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.getList = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this\n*/\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.setList = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.clearList = function() {\n  return this.setList(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.hasList = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.toObject) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict;\n  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinaryFromReader, \"\", new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue());\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.serializeBinaryToWriter);\n  }\n};\n\n\n/**\n * map<string, InitializationKwargsValue> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue>}\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} returns this\n */\nproto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n */\nproto.ubo.v1.OpenApplicationAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n */\nproto.ubo.v1.OpenApplicationAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string application_id = 2;\n * @return {string}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.getApplicationId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n */\nproto.ubo.v1.OpenApplicationAction.prototype.setApplicationId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional InitializationArgs initialization_args = 3;\n * @return {?proto.ubo.v1.OpenApplicationAction.InitializationArgs}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.getInitializationArgs = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationAction.InitializationArgs} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction.InitializationArgs, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationAction.InitializationArgs|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n*/\nproto.ubo.v1.OpenApplicationAction.prototype.setInitializationArgs = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n */\nproto.ubo.v1.OpenApplicationAction.prototype.clearInitializationArgs = function() {\n  return this.setInitializationArgs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.hasInitializationArgs = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional InitializationKwargsDict initialization_kwargs = 4;\n * @return {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.getInitializationKwargs = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n*/\nproto.ubo.v1.OpenApplicationAction.prototype.setInitializationKwargs = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationAction} returns this\n */\nproto.ubo.v1.OpenApplicationAction.prototype.clearInitializationKwargs = function() {\n  return this.setInitializationKwargs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationAction.prototype.hasInitializationKwargs = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CloseApplicationAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CloseApplicationAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CloseApplicationAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CloseApplicationAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    applicationInstanceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CloseApplicationAction}\n */\nproto.ubo.v1.CloseApplicationAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CloseApplicationAction;\n  return proto.ubo.v1.CloseApplicationAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CloseApplicationAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CloseApplicationAction}\n */\nproto.ubo.v1.CloseApplicationAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setApplicationInstanceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CloseApplicationAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CloseApplicationAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CloseApplicationAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CloseApplicationAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getApplicationInstanceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.CloseApplicationAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CloseApplicationAction} returns this\n */\nproto.ubo.v1.CloseApplicationAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CloseApplicationAction} returns this\n */\nproto.ubo.v1.CloseApplicationAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CloseApplicationAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string application_instance_id = 2;\n * @return {string}\n */\nproto.ubo.v1.CloseApplicationAction.prototype.getApplicationInstanceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CloseApplicationAction} returns this\n */\nproto.ubo.v1.CloseApplicationAction.prototype.setApplicationInstanceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainEvent}\n */\nproto.ubo.v1.MainEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainEvent;\n  return proto.ubo.v1.MainEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainEvent}\n */\nproto.ubo.v1.MainEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MainEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MainEvent} returns this\n */\nproto.ubo.v1.MainEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainEvent} returns this\n */\nproto.ubo.v1.MainEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InitEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InitEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InitEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InitEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InitEvent}\n */\nproto.ubo.v1.InitEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InitEvent;\n  return proto.ubo.v1.InitEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InitEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InitEvent}\n */\nproto.ubo.v1.InitEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InitEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InitEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InitEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InitEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InitEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InitEvent} returns this\n */\nproto.ubo.v1.InitEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InitEvent} returns this\n */\nproto.ubo.v1.InitEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InitEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuEvent}\n */\nproto.ubo.v1.MenuEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuEvent;\n  return proto.ubo.v1.MenuEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuEvent}\n */\nproto.ubo.v1.MenuEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuEvent} returns this\n */\nproto.ubo.v1.MenuEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuEvent} returns this\n */\nproto.ubo.v1.MenuEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuGoBackEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuGoBackEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuGoBackEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoBackEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuGoBackEvent}\n */\nproto.ubo.v1.MenuGoBackEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuGoBackEvent;\n  return proto.ubo.v1.MenuGoBackEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuGoBackEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuGoBackEvent}\n */\nproto.ubo.v1.MenuGoBackEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuGoBackEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuGoBackEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuGoBackEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoBackEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuGoBackEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuGoBackEvent} returns this\n */\nproto.ubo.v1.MenuGoBackEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuGoBackEvent} returns this\n */\nproto.ubo.v1.MenuGoBackEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuGoBackEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuGoHomeEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuGoHomeEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuGoHomeEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoHomeEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuGoHomeEvent}\n */\nproto.ubo.v1.MenuGoHomeEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuGoHomeEvent;\n  return proto.ubo.v1.MenuGoHomeEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuGoHomeEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuGoHomeEvent}\n */\nproto.ubo.v1.MenuGoHomeEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuGoHomeEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuGoHomeEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuGoHomeEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuGoHomeEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuGoHomeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuGoHomeEvent} returns this\n */\nproto.ubo.v1.MenuGoHomeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuGoHomeEvent} returns this\n */\nproto.ubo.v1.MenuGoHomeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuGoHomeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuChooseByIconEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuChooseByIconEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIconEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuChooseByIconEvent}\n */\nproto.ubo.v1.MenuChooseByIconEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuChooseByIconEvent;\n  return proto.ubo.v1.MenuChooseByIconEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuChooseByIconEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuChooseByIconEvent}\n */\nproto.ubo.v1.MenuChooseByIconEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuChooseByIconEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuChooseByIconEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIconEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIcon();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByIconEvent} returns this\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuChooseByIconEvent} returns this\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string icon = 2;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByIconEvent} returns this\n */\nproto.ubo.v1.MenuChooseByIconEvent.prototype.setIcon = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuChooseByLabelEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuChooseByLabelEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByLabelEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuChooseByLabelEvent}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuChooseByLabelEvent;\n  return proto.ubo.v1.MenuChooseByLabelEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuChooseByLabelEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuChooseByLabelEvent}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuChooseByLabelEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuChooseByLabelEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByLabelEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getLabel();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByLabelEvent} returns this\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuChooseByLabelEvent} returns this\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string label = 2;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByLabelEvent} returns this\n */\nproto.ubo.v1.MenuChooseByLabelEvent.prototype.setLabel = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuChooseByIndexEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuChooseByIndexEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIndexEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuChooseByIndexEvent}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuChooseByIndexEvent;\n  return proto.ubo.v1.MenuChooseByIndexEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuChooseByIndexEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuChooseByIndexEvent}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuChooseByIndexEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuChooseByIndexEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuChooseByIndexEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuChooseByIndexEvent} returns this\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuChooseByIndexEvent} returns this\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 index = 2;\n * @return {number}\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.MenuChooseByIndexEvent} returns this\n */\nproto.ubo.v1.MenuChooseByIndexEvent.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MenuScrollEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MenuScrollEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MenuScrollEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuScrollEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    direction: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MenuScrollEvent}\n */\nproto.ubo.v1.MenuScrollEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MenuScrollEvent;\n  return proto.ubo.v1.MenuScrollEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MenuScrollEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MenuScrollEvent}\n */\nproto.ubo.v1.MenuScrollEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.MenuScrollDirection} */ (reader.readEnum());\n      msg.setDirection(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MenuScrollEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MenuScrollEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MenuScrollEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MenuScrollEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDirection();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MenuScrollEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MenuScrollEvent} returns this\n */\nproto.ubo.v1.MenuScrollEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MenuScrollEvent} returns this\n */\nproto.ubo.v1.MenuScrollEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MenuScrollEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional MenuScrollDirection direction = 2;\n * @return {!proto.ubo.v1.MenuScrollDirection}\n */\nproto.ubo.v1.MenuScrollEvent.prototype.getDirection = function() {\n  return /** @type {!proto.ubo.v1.MenuScrollDirection} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.MenuScrollDirection} value\n * @return {!proto.ubo.v1.MenuScrollEvent} returns this\n */\nproto.ubo.v1.MenuScrollEvent.prototype.setDirection = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    applicationId: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.OpenApplicationEvent.InitializationArgs.toObject(includeInstance, f),\n    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationEvent}\n */\nproto.ubo.v1.OpenApplicationEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationEvent;\n  return proto.ubo.v1.OpenApplicationEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationEvent}\n */\nproto.ubo.v1.OpenApplicationEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setApplicationId(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.OpenApplicationEvent.InitializationArgs;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinaryFromReader);\n      msg.setInitializationArgs(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinaryFromReader);\n      msg.setInitializationKwargs(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getApplicationId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getInitializationArgs();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.OpenApplicationEvent.InitializationArgs.serializeBinaryToWriter\n    );\n  }\n  f = message.getInitializationKwargs();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationEvent.InitializationArgs.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.BasicType.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationArgs;\n  return proto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationEvent.InitializationArgs.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated BasicType items = 1;\n * @return {!Array<!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.BasicType>} value\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} returns this\n*/\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.BasicType=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.BasicType.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2;\n  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated BasicType items = 1;\n * @return {!Array<!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.BasicType>} value\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} returns this\n*/\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.BasicType=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase = {\n  INITIALIZATION_KWARGS_VALUE_NOT_SET: 0,\n  BASIC_TYPE: 1,\n  LIST: 2\n};\n\n/**\n * @return {proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.getInitializationKwargsValueCase = function() {\n  return /** @type {proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    basicType: (f = msg.getBasicType()) && proto.ubo.v1.BasicType.toObject(includeInstance, f),\n    list: (f = msg.getList()) && proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue;\n  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.setBasicType(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinaryFromReader);\n      msg.setList(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getBasicType();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n  f = message.getList();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional BasicType basic_type = 1;\n * @return {?proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.getBasicType = function() {\n  return /** @type{?proto.ubo.v1.BasicType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.BasicType|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this\n*/\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.setBasicType = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.clearBasicType = function() {\n  return this.setBasicType(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.hasBasicType = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional InitializationKwargsValue2 list = 2;\n * @return {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.getList = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this\n*/\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.setList = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.clearList = function() {\n  return this.setList(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.hasList = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.toObject) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict;\n  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinaryFromReader, \"\", new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue());\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.serializeBinaryToWriter);\n  }\n};\n\n\n/**\n * map<string, InitializationKwargsValue> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue>}\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string application_id = 2;\n * @return {string}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.getApplicationId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.setApplicationId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional InitializationArgs initialization_args = 3;\n * @return {?proto.ubo.v1.OpenApplicationEvent.InitializationArgs}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.getInitializationArgs = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationEvent.InitializationArgs} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent.InitializationArgs, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationEvent.InitializationArgs|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n*/\nproto.ubo.v1.OpenApplicationEvent.prototype.setInitializationArgs = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.clearInitializationArgs = function() {\n  return this.setInitializationArgs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.hasInitializationArgs = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional InitializationKwargsDict initialization_kwargs = 4;\n * @return {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.getInitializationKwargs = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict|undefined} value\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n*/\nproto.ubo.v1.OpenApplicationEvent.prototype.setInitializationKwargs = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.OpenApplicationEvent} returns this\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.clearInitializationKwargs = function() {\n  return this.setInitializationKwargs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.OpenApplicationEvent.prototype.hasInitializationKwargs = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CloseApplicationEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CloseApplicationEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CloseApplicationEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    applicationInstanceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CloseApplicationEvent}\n */\nproto.ubo.v1.CloseApplicationEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CloseApplicationEvent;\n  return proto.ubo.v1.CloseApplicationEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CloseApplicationEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CloseApplicationEvent}\n */\nproto.ubo.v1.CloseApplicationEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setApplicationInstanceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CloseApplicationEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CloseApplicationEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CloseApplicationEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getApplicationInstanceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CloseApplicationEvent} returns this\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CloseApplicationEvent} returns this\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string application_instance_id = 2;\n * @return {string}\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.getApplicationInstanceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CloseApplicationEvent} returns this\n */\nproto.ubo.v1.CloseApplicationEvent.prototype.setApplicationInstanceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PowerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PowerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PowerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PowerEvent}\n */\nproto.ubo.v1.PowerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PowerEvent;\n  return proto.ubo.v1.PowerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PowerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PowerEvent}\n */\nproto.ubo.v1.PowerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PowerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PowerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PowerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.PowerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PowerEvent} returns this\n */\nproto.ubo.v1.PowerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PowerEvent} returns this\n */\nproto.ubo.v1.PowerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PowerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PowerOffEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PowerOffEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PowerOffEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerOffEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PowerOffEvent}\n */\nproto.ubo.v1.PowerOffEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PowerOffEvent;\n  return proto.ubo.v1.PowerOffEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PowerOffEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PowerOffEvent}\n */\nproto.ubo.v1.PowerOffEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PowerOffEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PowerOffEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PowerOffEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PowerOffEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.PowerOffEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PowerOffEvent} returns this\n */\nproto.ubo.v1.PowerOffEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PowerOffEvent} returns this\n */\nproto.ubo.v1.PowerOffEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PowerOffEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RebootEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RebootEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RebootEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RebootEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RebootEvent}\n */\nproto.ubo.v1.RebootEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RebootEvent;\n  return proto.ubo.v1.RebootEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RebootEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RebootEvent}\n */\nproto.ubo.v1.RebootEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RebootEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RebootEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RebootEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RebootEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.RebootEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RebootEvent} returns this\n */\nproto.ubo.v1.RebootEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RebootEvent} returns this\n */\nproto.ubo.v1.RebootEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RebootEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ScreenshotEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ScreenshotEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ScreenshotEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ScreenshotEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ScreenshotEvent}\n */\nproto.ubo.v1.ScreenshotEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ScreenshotEvent;\n  return proto.ubo.v1.ScreenshotEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ScreenshotEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ScreenshotEvent}\n */\nproto.ubo.v1.ScreenshotEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ScreenshotEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ScreenshotEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ScreenshotEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ScreenshotEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ScreenshotEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ScreenshotEvent} returns this\n */\nproto.ubo.v1.ScreenshotEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ScreenshotEvent} returns this\n */\nproto.ubo.v1.ScreenshotEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ScreenshotEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SnapshotEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SnapshotEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SnapshotEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SnapshotEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SnapshotEvent}\n */\nproto.ubo.v1.SnapshotEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SnapshotEvent;\n  return proto.ubo.v1.SnapshotEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SnapshotEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SnapshotEvent}\n */\nproto.ubo.v1.SnapshotEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SnapshotEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SnapshotEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SnapshotEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SnapshotEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SnapshotEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SnapshotEvent} returns this\n */\nproto.ubo.v1.SnapshotEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SnapshotEvent} returns this\n */\nproto.ubo.v1.SnapshotEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SnapshotEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ToggleRecordingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ToggleRecordingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ToggleRecordingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ToggleRecordingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ToggleRecordingAction}\n */\nproto.ubo.v1.ToggleRecordingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ToggleRecordingAction;\n  return proto.ubo.v1.ToggleRecordingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ToggleRecordingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ToggleRecordingAction}\n */\nproto.ubo.v1.ToggleRecordingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ToggleRecordingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ToggleRecordingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ToggleRecordingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ToggleRecordingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ToggleRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ToggleRecordingAction} returns this\n */\nproto.ubo.v1.ToggleRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ToggleRecordingAction} returns this\n */\nproto.ubo.v1.ToggleRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ToggleRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.StoreRecordedSequenceEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.StoreRecordedSequenceEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    recordedSequenceList: jspb.Message.toObjectList(msg.getRecordedSequenceList(),\n    proto.ubo.v1.Action.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.StoreRecordedSequenceEvent}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.StoreRecordedSequenceEvent;\n  return proto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.StoreRecordedSequenceEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.StoreRecordedSequenceEvent}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Action;\n      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);\n      msg.addRecordedSequence(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.StoreRecordedSequenceEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.StoreRecordedSequenceEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getRecordedSequenceList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated Action recorded_sequence = 2;\n * @return {!Array<!proto.ubo.v1.Action>}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.getRecordedSequenceList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Action>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Action>} value\n * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this\n*/\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.setRecordedSequenceList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Action=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Action}\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.addRecordedSequence = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.Action, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this\n */\nproto.ubo.v1.StoreRecordedSequenceEvent.prototype.clearRecordedSequenceList = function() {\n  return this.setRecordedSequenceList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ReplayRecordedSequenceAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ReplayRecordedSequenceAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ReplayRecordedSequenceAction}\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ReplayRecordedSequenceAction;\n  return proto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ReplayRecordedSequenceAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ReplayRecordedSequenceAction}\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ReplayRecordedSequenceAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ReplayRecordedSequenceAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReplayRecordedSequenceAction} returns this\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ReplayRecordedSequenceAction} returns this\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ReplayRecordedSequenceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ReplayRecordedSequenceEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ReplayRecordedSequenceEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent}\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ReplayRecordedSequenceEvent;\n  return proto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ReplayRecordedSequenceEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent}\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ReplayRecordedSequenceEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ReplayRecordedSequenceEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent} returns this\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent} returns this\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ReplayRecordedSequenceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ReportReplayingDoneAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ReportReplayingDoneAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ReportReplayingDoneAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReportReplayingDoneAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ReportReplayingDoneAction}\n */\nproto.ubo.v1.ReportReplayingDoneAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ReportReplayingDoneAction;\n  return proto.ubo.v1.ReportReplayingDoneAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ReportReplayingDoneAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ReportReplayingDoneAction}\n */\nproto.ubo.v1.ReportReplayingDoneAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ReportReplayingDoneAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ReportReplayingDoneAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ReportReplayingDoneAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReportReplayingDoneAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ReportReplayingDoneAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReportReplayingDoneAction} returns this\n */\nproto.ubo.v1.ReportReplayingDoneAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ReportReplayingDoneAction} returns this\n */\nproto.ubo.v1.ReportReplayingDoneAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ReportReplayingDoneAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    menu: (f = msg.getMenu()) && proto.ubo.v1.Menu.toObject(includeInstance, f),\n    path: (f = msg.getPath()) && proto.ubo.v1.MainState.Path.toObject(includeInstance, f),\n    depth: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    isHeaderVisible: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    isFooterVisible: jspb.Message.getBooleanFieldWithDefault(msg, 6, false),\n    appsItemsPriorities: (f = msg.getAppsItemsPriorities()) && proto.ubo.v1.MainState.AppsItemsPrioritiesDict.toObject(includeInstance, f),\n    settingsItemsPriorities: (f = msg.getSettingsItemsPriorities()) && proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.toObject(includeInstance, f),\n    isRecording: jspb.Message.getBooleanFieldWithDefault(msg, 9, false),\n    isReplaying: jspb.Message.getBooleanFieldWithDefault(msg, 10, false),\n    recordedSequence: (f = msg.getRecordedSequence()) && proto.ubo.v1.MainState.RecordedSequence.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainState}\n */\nproto.ubo.v1.MainState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainState;\n  return proto.ubo.v1.MainState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainState}\n */\nproto.ubo.v1.MainState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Menu;\n      reader.readMessage(value,proto.ubo.v1.Menu.deserializeBinaryFromReader);\n      msg.setMenu(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.MainState.Path;\n      reader.readMessage(value,proto.ubo.v1.MainState.Path.deserializeBinaryFromReader);\n      msg.setPath(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setDepth(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsHeaderVisible(value);\n      break;\n    case 6:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsFooterVisible(value);\n      break;\n    case 7:\n      var value = new proto.ubo.v1.MainState.AppsItemsPrioritiesDict;\n      reader.readMessage(value,proto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinaryFromReader);\n      msg.setAppsItemsPriorities(value);\n      break;\n    case 8:\n      var value = new proto.ubo.v1.MainState.SettingsItemsPrioritiesDict;\n      reader.readMessage(value,proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinaryFromReader);\n      msg.setSettingsItemsPriorities(value);\n      break;\n    case 9:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsRecording(value);\n      break;\n    case 10:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsReplaying(value);\n      break;\n    case 11:\n      var value = new proto.ubo.v1.MainState.RecordedSequence;\n      reader.readMessage(value,proto.ubo.v1.MainState.RecordedSequence.deserializeBinaryFromReader);\n      msg.setRecordedSequence(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getMenu();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Menu.serializeBinaryToWriter\n    );\n  }\n  f = message.getPath();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.MainState.Path.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeBool(\n      6,\n      f\n    );\n  }\n  f = message.getAppsItemsPriorities();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.ubo.v1.MainState.AppsItemsPrioritiesDict.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsItemsPriorities();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeBool(\n      9,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeBool(\n      10,\n      f\n    );\n  }\n  f = message.getRecordedSequence();\n  if (f != null) {\n    writer.writeMessage(\n      11,\n      f,\n      proto.ubo.v1.MainState.RecordedSequence.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.MainState.Path.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainState.Path.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainState.Path.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainState.Path} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.Path.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainState.Path}\n */\nproto.ubo.v1.MainState.Path.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainState.Path;\n  return proto.ubo.v1.MainState.Path.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainState.Path} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainState.Path}\n */\nproto.ubo.v1.MainState.Path.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainState.Path.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainState.Path.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainState.Path} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.Path.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.MainState.Path.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.MainState.Path} returns this\n */\nproto.ubo.v1.MainState.Path.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.MainState.Path} returns this\n */\nproto.ubo.v1.MainState.Path.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.MainState.Path} returns this\n */\nproto.ubo.v1.MainState.Path.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainState.AppsItemsPrioritiesDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict}\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainState.AppsItemsPrioritiesDict;\n  return proto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict}\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readInt64, null, \"\", 0);\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainState.AppsItemsPrioritiesDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeInt64);\n  }\n};\n\n\n/**\n * map<string, int64> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,number>}\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,number>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} returns this\n */\nproto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict}\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainState.SettingsItemsPrioritiesDict;\n  return proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict}\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readInt64, null, \"\", 0);\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeInt64);\n  }\n};\n\n\n/**\n * map<string, int64> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,number>}\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,number>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} returns this\n */\nproto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.MainState.RecordedSequence.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.MainState.RecordedSequence.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.MainState.RecordedSequence.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.MainState.RecordedSequence} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.RecordedSequence.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Action.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.MainState.RecordedSequence}\n */\nproto.ubo.v1.MainState.RecordedSequence.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.MainState.RecordedSequence;\n  return proto.ubo.v1.MainState.RecordedSequence.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.MainState.RecordedSequence} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.MainState.RecordedSequence}\n */\nproto.ubo.v1.MainState.RecordedSequence.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.Action;\n      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.MainState.RecordedSequence.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.MainState.RecordedSequence.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.MainState.RecordedSequence} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.MainState.RecordedSequence.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated Action items = 1;\n * @return {!Array<!proto.ubo.v1.Action>}\n */\nproto.ubo.v1.MainState.RecordedSequence.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Action>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Action>} value\n * @return {!proto.ubo.v1.MainState.RecordedSequence} returns this\n*/\nproto.ubo.v1.MainState.RecordedSequence.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Action=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Action}\n */\nproto.ubo.v1.MainState.RecordedSequence.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Action, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.MainState.RecordedSequence} returns this\n */\nproto.ubo.v1.MainState.RecordedSequence.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.MainState.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Menu menu = 2;\n * @return {?proto.ubo.v1.Menu}\n */\nproto.ubo.v1.MainState.prototype.getMenu = function() {\n  return /** @type{?proto.ubo.v1.Menu} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Menu, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Menu|undefined} value\n * @return {!proto.ubo.v1.MainState} returns this\n*/\nproto.ubo.v1.MainState.prototype.setMenu = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearMenu = function() {\n  return this.setMenu(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasMenu = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional Path path = 3;\n * @return {?proto.ubo.v1.MainState.Path}\n */\nproto.ubo.v1.MainState.prototype.getPath = function() {\n  return /** @type{?proto.ubo.v1.MainState.Path} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.Path, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MainState.Path|undefined} value\n * @return {!proto.ubo.v1.MainState} returns this\n*/\nproto.ubo.v1.MainState.prototype.setPath = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearPath = function() {\n  return this.setPath(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasPath = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 depth = 4;\n * @return {number}\n */\nproto.ubo.v1.MainState.prototype.getDepth = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.setDepth = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearDepth = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasDepth = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_header_visible = 5;\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.getIsHeaderVisible = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.setIsHeaderVisible = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearIsHeaderVisible = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasIsHeaderVisible = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional bool is_footer_visible = 6;\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.getIsFooterVisible = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.setIsFooterVisible = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearIsFooterVisible = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasIsFooterVisible = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional AppsItemsPrioritiesDict apps_items_priorities = 7;\n * @return {?proto.ubo.v1.MainState.AppsItemsPrioritiesDict}\n */\nproto.ubo.v1.MainState.prototype.getAppsItemsPriorities = function() {\n  return /** @type{?proto.ubo.v1.MainState.AppsItemsPrioritiesDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.AppsItemsPrioritiesDict, 7));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MainState.AppsItemsPrioritiesDict|undefined} value\n * @return {!proto.ubo.v1.MainState} returns this\n*/\nproto.ubo.v1.MainState.prototype.setAppsItemsPriorities = function(value) {\n  return jspb.Message.setWrapperField(this, 7, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearAppsItemsPriorities = function() {\n  return this.setAppsItemsPriorities(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasAppsItemsPriorities = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional SettingsItemsPrioritiesDict settings_items_priorities = 8;\n * @return {?proto.ubo.v1.MainState.SettingsItemsPrioritiesDict}\n */\nproto.ubo.v1.MainState.prototype.getSettingsItemsPriorities = function() {\n  return /** @type{?proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.SettingsItemsPrioritiesDict, 8));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MainState.SettingsItemsPrioritiesDict|undefined} value\n * @return {!proto.ubo.v1.MainState} returns this\n*/\nproto.ubo.v1.MainState.prototype.setSettingsItemsPriorities = function(value) {\n  return jspb.Message.setWrapperField(this, 8, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearSettingsItemsPriorities = function() {\n  return this.setSettingsItemsPriorities(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasSettingsItemsPriorities = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional bool is_recording = 9;\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.getIsRecording = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.setIsRecording = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearIsRecording = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasIsRecording = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional bool is_replaying = 10;\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.getIsReplaying = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 10, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.setIsReplaying = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearIsReplaying = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasIsReplaying = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional RecordedSequence recorded_sequence = 11;\n * @return {?proto.ubo.v1.MainState.RecordedSequence}\n */\nproto.ubo.v1.MainState.prototype.getRecordedSequence = function() {\n  return /** @type{?proto.ubo.v1.MainState.RecordedSequence} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.RecordedSequence, 11));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MainState.RecordedSequence|undefined} value\n * @return {!proto.ubo.v1.MainState} returns this\n*/\nproto.ubo.v1.MainState.prototype.setRecordedSequence = function(value) {\n  return jspb.Message.setWrapperField(this, 11, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.MainState} returns this\n */\nproto.ubo.v1.MainState.prototype.clearRecordedSequence = function() {\n  return this.setRecordedSequence(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.MainState.prototype.hasRecordedSequence = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboDispatchItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboDispatchItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboDispatchItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboDispatchItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUboActions: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    storeAction: (f = msg.getStoreAction()) && proto.ubo.v1.UboDispatchItem.StoreAction.toObject(includeInstance, f),\n    key: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboDispatchItem}\n */\nproto.ubo.v1.UboDispatchItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboDispatchItem;\n  return proto.ubo.v1.UboDispatchItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboDispatchItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboDispatchItem}\n */\nproto.ubo.v1.UboDispatchItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUboActions(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.UboDispatchItem.StoreAction;\n      reader.readMessage(value,proto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinaryFromReader);\n      msg.setStoreAction(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 8:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 10:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboDispatchItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboDispatchItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboDispatchItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboDispatchItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getStoreAction();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.UboDispatchItem.StoreAction.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeBool(\n      8,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeFloat(\n      9,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeFloat(\n      10,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboDispatchItem.StoreAction2.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboDispatchItem.StoreAction2} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Action.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboDispatchItem.StoreAction2;\n  return proto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboDispatchItem.StoreAction2} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.Action;\n      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboDispatchItem.StoreAction2.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboDispatchItem.StoreAction2} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated Action items = 1;\n * @return {!Array<!proto.ubo.v1.Action>}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Action>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Action>} value\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2} returns this\n*/\nproto.ubo.v1.UboDispatchItem.StoreAction2.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Action=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Action}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Action, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2} returns this\n */\nproto.ubo.v1.UboDispatchItem.StoreAction2.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase = {\n  STORE_ACTION_NOT_SET: 0,\n  UBO_ACTION: 1,\n  LIST: 2\n};\n\n/**\n * @return {proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.getStoreActionCase = function() {\n  return /** @type {proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboDispatchItem.StoreAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboDispatchItem.StoreAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    uboAction: (f = msg.getUboAction()) && proto.ubo.v1.Action.toObject(includeInstance, f),\n    list: (f = msg.getList()) && proto.ubo.v1.UboDispatchItem.StoreAction2.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboDispatchItem.StoreAction;\n  return proto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboDispatchItem.StoreAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.Action;\n      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);\n      msg.setUboAction(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.UboDispatchItem.StoreAction2;\n      reader.readMessage(value,proto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinaryFromReader);\n      msg.setList(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboDispatchItem.StoreAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboDispatchItem.StoreAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getUboAction();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.Action.serializeBinaryToWriter\n    );\n  }\n  f = message.getList();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.UboDispatchItem.StoreAction2.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional Action ubo_action = 1;\n * @return {?proto.ubo.v1.Action}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.getUboAction = function() {\n  return /** @type{?proto.ubo.v1.Action} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Action, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Action|undefined} value\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this\n*/\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.setUboAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.clearUboAction = function() {\n  return this.setUboAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.hasUboAction = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional StoreAction2 list = 2;\n * @return {?proto.ubo.v1.UboDispatchItem.StoreAction2}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.getList = function() {\n  return /** @type{?proto.ubo.v1.UboDispatchItem.StoreAction2} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UboDispatchItem.StoreAction2, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UboDispatchItem.StoreAction2|undefined} value\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this\n*/\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.setList = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.clearList = function() {\n  return this.setList(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.StoreAction.prototype.hasList = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_ubo_actions = 1000;\n * @return {string}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotUboActions = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional StoreAction store_action = 2;\n * @return {?proto.ubo.v1.UboDispatchItem.StoreAction}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getStoreAction = function() {\n  return /** @type{?proto.ubo.v1.UboDispatchItem.StoreAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UboDispatchItem.StoreAction, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UboDispatchItem.StoreAction|undefined} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n*/\nproto.ubo.v1.UboDispatchItem.prototype.setStoreAction = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearStoreAction = function() {\n  return this.setStoreAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasStoreAction = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string key = 3;\n * @return {string}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string label = 4;\n * @return {string}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string color = 5;\n * @return {string}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string background_color = 6;\n * @return {string}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string icon = 7;\n * @return {string}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional bool is_short = 8;\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional float opacity = 9;\n * @return {number}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional float progress = 10;\n * @return {number}\n */\nproto.ubo.v1.UboDispatchItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboDispatchItem} returns this\n */\nproto.ubo.v1.UboDispatchItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboDispatchItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboApplicationItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboApplicationItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboApplicationItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUboActions: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    application: (f = msg.getApplication()) && proto.ubo.v1.UboApplicationItem.Application.toObject(includeInstance, f),\n    applicationId: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.UboApplicationItem.InitializationArgs.toObject(includeInstance, f),\n    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.toObject(includeInstance, f),\n    key: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 8, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 9, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 10, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 11, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 12, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 13, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboApplicationItem}\n */\nproto.ubo.v1.UboApplicationItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboApplicationItem;\n  return proto.ubo.v1.UboApplicationItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboApplicationItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboApplicationItem}\n */\nproto.ubo.v1.UboApplicationItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUboActions(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.UboApplicationItem.Application;\n      reader.readMessage(value,proto.ubo.v1.UboApplicationItem.Application.deserializeBinaryFromReader);\n      msg.setApplication(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setApplicationId(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.UboApplicationItem.InitializationArgs;\n      reader.readMessage(value,proto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinaryFromReader);\n      msg.setInitializationArgs(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.UboApplicationItem.InitializationKwargsDict;\n      reader.readMessage(value,proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader);\n      msg.setInitializationKwargs(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 8:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 9:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 10:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 11:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 12:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 13:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboApplicationItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboApplicationItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboApplicationItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getApplication();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.UboApplicationItem.Application.serializeBinaryToWriter\n    );\n  }\n  f = message.getApplicationId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getInitializationArgs();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.UboApplicationItem.InitializationArgs.serializeBinaryToWriter\n    );\n  }\n  f = message.getInitializationKwargs();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeString(\n      8,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeString(\n      9,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeString(\n      10,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeBool(\n      11,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 12));\n  if (f != null) {\n    writer.writeFloat(\n      12,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 13));\n  if (f != null) {\n    writer.writeFloat(\n      13,\n      f\n    );\n  }\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.UboApplicationItem.Application.oneofGroups_ = [[1]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.UboApplicationItem.Application.ApplicationCase = {\n  APPLICATION_NOT_SET: 0,\n  STRING: 1\n};\n\n/**\n * @return {proto.ubo.v1.UboApplicationItem.Application.ApplicationCase}\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.getApplicationCase = function() {\n  return /** @type {proto.ubo.v1.UboApplicationItem.Application.ApplicationCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboApplicationItem.Application.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboApplicationItem.Application} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.Application.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    string: jspb.Message.getFieldWithDefault(msg, 1, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboApplicationItem.Application}\n */\nproto.ubo.v1.UboApplicationItem.Application.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboApplicationItem.Application;\n  return proto.ubo.v1.UboApplicationItem.Application.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboApplicationItem.Application} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboApplicationItem.Application}\n */\nproto.ubo.v1.UboApplicationItem.Application.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setString(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboApplicationItem.Application.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboApplicationItem.Application} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.Application.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string string = 1;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.getString = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem.Application} returns this\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.setString = function(value) {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem.Application} returns this\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.clearString = function() {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.Application.prototype.hasString = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboApplicationItem.InitializationArgs.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboApplicationItem.InitializationArgs} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.BasicType.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs}\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboApplicationItem.InitializationArgs;\n  return proto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboApplicationItem.InitializationArgs} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs}\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboApplicationItem.InitializationArgs.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboApplicationItem.InitializationArgs} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated BasicType items = 1;\n * @return {!Array<!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.BasicType>} value\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs} returns this\n*/\nproto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.BasicType=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs} returns this\n */\nproto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.BasicType.toObject) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict}\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UboApplicationItem.InitializationKwargsDict;\n  return proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict}\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.BasicType.deserializeBinaryFromReader, \"\", new proto.ubo.v1.BasicType());\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.BasicType.serializeBinaryToWriter);\n  }\n};\n\n\n/**\n * map<string, BasicType> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.BasicType>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      proto.ubo.v1.BasicType));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} returns this\n */\nproto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_ubo_actions = 1000;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotUboActions = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Application application = 2;\n * @return {?proto.ubo.v1.UboApplicationItem.Application}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getApplication = function() {\n  return /** @type{?proto.ubo.v1.UboApplicationItem.Application} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UboApplicationItem.Application, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UboApplicationItem.Application|undefined} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n*/\nproto.ubo.v1.UboApplicationItem.prototype.setApplication = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearApplication = function() {\n  return this.setApplication(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasApplication = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string application_id = 3;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getApplicationId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setApplicationId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional InitializationArgs initialization_args = 4;\n * @return {?proto.ubo.v1.UboApplicationItem.InitializationArgs}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getInitializationArgs = function() {\n  return /** @type{?proto.ubo.v1.UboApplicationItem.InitializationArgs} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UboApplicationItem.InitializationArgs, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UboApplicationItem.InitializationArgs|undefined} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n*/\nproto.ubo.v1.UboApplicationItem.prototype.setInitializationArgs = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearInitializationArgs = function() {\n  return this.setInitializationArgs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasInitializationArgs = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional InitializationKwargsDict initialization_kwargs = 5;\n * @return {?proto.ubo.v1.UboApplicationItem.InitializationKwargsDict}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getInitializationKwargs = function() {\n  return /** @type{?proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UboApplicationItem.InitializationKwargsDict, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UboApplicationItem.InitializationKwargsDict|undefined} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n*/\nproto.ubo.v1.UboApplicationItem.prototype.setInitializationKwargs = function(value) {\n  return jspb.Message.setWrapperField(this, 5, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearInitializationKwargs = function() {\n  return this.setInitializationKwargs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasInitializationKwargs = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string key = 6;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string label = 7;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional string color = 8;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional string background_color = 9;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional string icon = 10;\n * @return {string}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional bool is_short = 11;\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional float opacity = 12;\n * @return {number}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 12, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 12, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 12, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional float progress = 13;\n * @return {number}\n */\nproto.ubo.v1.UboApplicationItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 13, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 13, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UboApplicationItem} returns this\n */\nproto.ubo.v1.UboApplicationItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 13, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UboApplicationItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 13) != null;\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.BasicTypeOptional.oneofGroups_ = [[1,2,3,4,5]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase = {\n  BASIC_TYPE_OPTIONAL_NOT_SET: 0,\n  BOOL: 1,\n  BYTES: 2,\n  FLOAT: 3,\n  INT64: 4,\n  STRING: 5\n};\n\n/**\n * @return {proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getBasicTypeOptionalCase = function() {\n  return /** @type {proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.BasicTypeOptional.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.BasicTypeOptional} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.BasicTypeOptional.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    bool: jspb.Message.getBooleanFieldWithDefault(msg, 1, false),\n    bytes: msg.getBytes_asB64(),\n    pb_float: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),\n    int64: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    string: jspb.Message.getFieldWithDefault(msg, 5, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.BasicTypeOptional}\n */\nproto.ubo.v1.BasicTypeOptional.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.BasicTypeOptional;\n  return proto.ubo.v1.BasicTypeOptional.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.BasicTypeOptional} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.BasicTypeOptional}\n */\nproto.ubo.v1.BasicTypeOptional.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setBool(value);\n      break;\n    case 2:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setBytes(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setFloat(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setInt64(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setString(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.BasicTypeOptional.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.BasicTypeOptional} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.BasicTypeOptional.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeBool(\n      1,\n      f\n    );\n  }\n  f = /** @type {!(string|Uint8Array)} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBytes(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeFloat(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional bool bool = 1;\n * @return {boolean}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getBool = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 1, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.setBool = function(value) {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.clearBool = function() {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.hasBool = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional bytes bytes = 2;\n * @return {string}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getBytes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * optional bytes bytes = 2;\n * This is a type-conversion wrapper around `getBytes()`\n * @return {string}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getBytes_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getBytes()));\n};\n\n\n/**\n * optional bytes bytes = 2;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getBytes()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getBytes_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getBytes()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.setBytes = function(value) {\n  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.clearBytes = function() {\n  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.hasBytes = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional float float = 3;\n * @return {number}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getFloat = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.setFloat = function(value) {\n  return jspb.Message.setOneofField(this, 3, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.clearFloat = function() {\n  return jspb.Message.setOneofField(this, 3, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.hasFloat = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 int64 = 4;\n * @return {number}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getInt64 = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.setInt64 = function(value) {\n  return jspb.Message.setOneofField(this, 4, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.clearInt64 = function() {\n  return jspb.Message.setOneofField(this, 4, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.hasInt64 = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string string = 5;\n * @return {string}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.getString = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.setString = function(value) {\n  return jspb.Message.setOneofField(this, 5, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.BasicTypeOptional} returns this\n */\nproto.ubo.v1.BasicTypeOptional.prototype.clearString = function() {\n  return jspb.Message.setOneofField(this, 5, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BasicTypeOptional.prototype.hasString = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.BasicType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.BasicType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.BasicType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.BasicType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    items: (f = msg.getItems()) && proto.ubo.v1.BasicTypeOptional.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.BasicType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.BasicType;\n  return proto.ubo.v1.BasicType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.BasicType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.BasicType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicTypeOptional;\n      reader.readMessage(value,proto.ubo.v1.BasicTypeOptional.deserializeBinaryFromReader);\n      msg.setItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.BasicType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.BasicType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.BasicType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.BasicType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItems();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicTypeOptional.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional BasicTypeOptional items = 1;\n * @return {?proto.ubo.v1.BasicTypeOptional}\n */\nproto.ubo.v1.BasicType.prototype.getItems = function() {\n  return /** @type{?proto.ubo.v1.BasicTypeOptional} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.BasicTypeOptional, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.BasicTypeOptional|undefined} value\n * @return {!proto.ubo.v1.BasicType} returns this\n*/\nproto.ubo.v1.BasicType.prototype.setItems = function(value) {\n  return jspb.Message.setWrapperField(this, 1, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.BasicType} returns this\n */\nproto.ubo.v1.BasicType.prototype.clearItems = function() {\n  return this.setItems(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.BasicType.prototype.hasItems = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputResult.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputResult.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputResult} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputResult.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    dataMap: (f = msg.getDataMap()) ? f.toObject(includeInstance, undefined) : [],\n    filesMap: (f = msg.getFilesMap()) ? f.toObject(includeInstance, undefined) : [],\n    method: jspb.Message.getFieldWithDefault(msg, 4, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputResult}\n */\nproto.ubo.v1.InputResult.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputResult;\n  return proto.ubo.v1.InputResult.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputResult} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputResult}\n */\nproto.ubo.v1.InputResult.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = msg.getDataMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, \"\", \"\");\n         });\n      break;\n    case 3:\n      var value = msg.getFilesMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readBytes, null, \"\", \"\");\n         });\n      break;\n    case 4:\n      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());\n      msg.setMethod(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputResult.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputResult.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputResult} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputResult.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDataMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(2, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);\n  }\n  f = message.getFilesMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(3, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeBytes);\n  }\n  f = message.getMethod();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputResult.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputResult} returns this\n */\nproto.ubo.v1.InputResult.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputResult} returns this\n */\nproto.ubo.v1.InputResult.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputResult.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * map<string, string> data = 2;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,string>}\n */\nproto.ubo.v1.InputResult.prototype.getDataMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,string>} */ (\n      jspb.Message.getMapField(this, 2, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.InputResult} returns this\n */\nproto.ubo.v1.InputResult.prototype.clearDataMap = function() {\n  this.getDataMap().clear();\n  return this;};\n\n\n/**\n * map<string, bytes> files = 3;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!(string|Uint8Array)>}\n */\nproto.ubo.v1.InputResult.prototype.getFilesMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!(string|Uint8Array)>} */ (\n      jspb.Message.getMapField(this, 3, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.InputResult} returns this\n */\nproto.ubo.v1.InputResult.prototype.clearFilesMap = function() {\n  this.getFilesMap().clear();\n  return this;};\n\n\n/**\n * optional InputMethod method = 4;\n * @return {!proto.ubo.v1.InputMethod}\n */\nproto.ubo.v1.InputResult.prototype.getMethod = function() {\n  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputMethod} value\n * @return {!proto.ubo.v1.InputResult} returns this\n */\nproto.ubo.v1.InputResult.prototype.setMethod = function(value) {\n  return jspb.Message.setProto3EnumField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputFieldDescription.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputFieldDescription.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputFieldDescription} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputFieldDescription.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    type: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    description: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    fileMimetype: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    pattern: jspb.Message.getFieldWithDefault(msg, 8, \"\"),\n    defaultValue: jspb.Message.getFieldWithDefault(msg, 9, \"\"),\n    options: (f = msg.getOptions()) && proto.ubo.v1.InputFieldDescription.Options.toObject(includeInstance, f),\n    required: jspb.Message.getBooleanFieldWithDefault(msg, 11, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputFieldDescription}\n */\nproto.ubo.v1.InputFieldDescription.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputFieldDescription;\n  return proto.ubo.v1.InputFieldDescription.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputFieldDescription} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputFieldDescription}\n */\nproto.ubo.v1.InputFieldDescription.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.ubo.v1.InputFieldType} */ (reader.readEnum());\n      msg.setType(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDescription(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setFileMimetype(value);\n      break;\n    case 8:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPattern(value);\n      break;\n    case 9:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDefaultValue(value);\n      break;\n    case 10:\n      var value = new proto.ubo.v1.InputFieldDescription.Options;\n      reader.readMessage(value,proto.ubo.v1.InputFieldDescription.Options.deserializeBinaryFromReader);\n      msg.setOptions(value);\n      break;\n    case 11:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setRequired(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputFieldDescription.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputFieldDescription.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputFieldDescription} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputFieldDescription.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getLabel();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getType();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeString(\n      8,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeString(\n      9,\n      f\n    );\n  }\n  f = message.getOptions();\n  if (f != null) {\n    writer.writeMessage(\n      10,\n      f,\n      proto.ubo.v1.InputFieldDescription.Options.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeBool(\n      11,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.InputFieldDescription.Options.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputFieldDescription.Options.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputFieldDescription.Options.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputFieldDescription.Options} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputFieldDescription.Options.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputFieldDescription.Options}\n */\nproto.ubo.v1.InputFieldDescription.Options.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputFieldDescription.Options;\n  return proto.ubo.v1.InputFieldDescription.Options.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputFieldDescription.Options} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputFieldDescription.Options}\n */\nproto.ubo.v1.InputFieldDescription.Options.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputFieldDescription.Options.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputFieldDescription.Options.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputFieldDescription.Options} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputFieldDescription.Options.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.InputFieldDescription.Options.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.InputFieldDescription.Options} returns this\n */\nproto.ubo.v1.InputFieldDescription.Options.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.InputFieldDescription.Options} returns this\n */\nproto.ubo.v1.InputFieldDescription.Options.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.InputFieldDescription.Options} returns this\n */\nproto.ubo.v1.InputFieldDescription.Options.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string name = 2;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string label = 3;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setLabel = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional InputFieldType type = 4;\n * @return {!proto.ubo.v1.InputFieldType}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getType = function() {\n  return /** @type {!proto.ubo.v1.InputFieldType} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputFieldType} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setType = function(value) {\n  return jspb.Message.setProto3EnumField(this, 4, value);\n};\n\n\n/**\n * optional string description = 5;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getDescription = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setDescription = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearDescription = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasDescription = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string title = 6;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setTitle = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearTitle = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasTitle = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string file_mimetype = 7;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getFileMimetype = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setFileMimetype = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearFileMimetype = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasFileMimetype = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional string pattern = 8;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getPattern = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setPattern = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearPattern = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasPattern = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional string default_value = 9;\n * @return {string}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getDefaultValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setDefaultValue = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearDefaultValue = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasDefaultValue = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional Options options = 10;\n * @return {?proto.ubo.v1.InputFieldDescription.Options}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getOptions = function() {\n  return /** @type{?proto.ubo.v1.InputFieldDescription.Options} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputFieldDescription.Options, 10));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputFieldDescription.Options|undefined} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n*/\nproto.ubo.v1.InputFieldDescription.prototype.setOptions = function(value) {\n  return jspb.Message.setWrapperField(this, 10, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearOptions = function() {\n  return this.setOptions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasOptions = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional bool required = 11;\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.getRequired = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.setRequired = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputFieldDescription} returns this\n */\nproto.ubo.v1.InputFieldDescription.prototype.clearRequired = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputFieldDescription.prototype.hasRequired = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputDescription.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputDescription.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputDescription} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputDescription.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    id: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    prompt: jspb.Message.getFieldWithDefault(msg, 5, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputDescription}\n */\nproto.ubo.v1.InputDescription.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputDescription;\n  return proto.ubo.v1.InputDescription.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputDescription} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputDescription}\n */\nproto.ubo.v1.InputDescription.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());\n      msg.setInputMethod(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPrompt(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputDescription.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputDescription.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputDescription} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputDescription.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getInputMethod();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional InputMethod input_method = 2;\n * @return {!proto.ubo.v1.InputMethod}\n */\nproto.ubo.v1.InputDescription.prototype.getInputMethod = function() {\n  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputMethod} value\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.setInputMethod = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional string id = 3;\n * @return {string}\n */\nproto.ubo.v1.InputDescription.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.clearId = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputDescription.prototype.hasId = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string title = 4;\n * @return {string}\n */\nproto.ubo.v1.InputDescription.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.setTitle = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.clearTitle = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputDescription.prototype.hasTitle = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string prompt = 5;\n * @return {string}\n */\nproto.ubo.v1.InputDescription.prototype.getPrompt = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.setPrompt = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputDescription} returns this\n */\nproto.ubo.v1.InputDescription.prototype.clearPrompt = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputDescription.prototype.hasPrompt = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WebUIInputDescription.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WebUIInputDescription} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIInputDescription.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    fields: (f = msg.getFields()) && proto.ubo.v1.WebUIInputDescription.Fields.toObject(includeInstance, f),\n    id: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    prompt: jspb.Message.getFieldWithDefault(msg, 6, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WebUIInputDescription}\n */\nproto.ubo.v1.WebUIInputDescription.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WebUIInputDescription;\n  return proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WebUIInputDescription} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WebUIInputDescription}\n */\nproto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());\n      msg.setInputMethod(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.WebUIInputDescription.Fields;\n      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.Fields.deserializeBinaryFromReader);\n      msg.setFields(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPrompt(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WebUIInputDescription} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getFields();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.WebUIInputDescription.Fields.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.WebUIInputDescription.Fields.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WebUIInputDescription.Fields.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WebUIInputDescription.Fields.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WebUIInputDescription.Fields} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIInputDescription.Fields.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.InputFieldDescription.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WebUIInputDescription.Fields}\n */\nproto.ubo.v1.WebUIInputDescription.Fields.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WebUIInputDescription.Fields;\n  return proto.ubo.v1.WebUIInputDescription.Fields.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WebUIInputDescription.Fields} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WebUIInputDescription.Fields}\n */\nproto.ubo.v1.WebUIInputDescription.Fields.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.InputFieldDescription;\n      reader.readMessage(value,proto.ubo.v1.InputFieldDescription.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WebUIInputDescription.Fields.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WebUIInputDescription.Fields.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WebUIInputDescription.Fields} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIInputDescription.Fields.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.InputFieldDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated InputFieldDescription items = 1;\n * @return {!Array<!proto.ubo.v1.InputFieldDescription>}\n */\nproto.ubo.v1.WebUIInputDescription.Fields.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.InputFieldDescription>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.InputFieldDescription, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.InputFieldDescription>} value\n * @return {!proto.ubo.v1.WebUIInputDescription.Fields} returns this\n*/\nproto.ubo.v1.WebUIInputDescription.Fields.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputFieldDescription=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.InputFieldDescription}\n */\nproto.ubo.v1.WebUIInputDescription.Fields.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.InputFieldDescription, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.WebUIInputDescription.Fields} returns this\n */\nproto.ubo.v1.WebUIInputDescription.Fields.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional InputMethod input_method = 2;\n * @return {!proto.ubo.v1.InputMethod}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.getInputMethod = function() {\n  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputMethod} value\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.setInputMethod = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.clearInputMethod = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.hasInputMethod = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional Fields fields = 3;\n * @return {?proto.ubo.v1.WebUIInputDescription.Fields}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.getFields = function() {\n  return /** @type{?proto.ubo.v1.WebUIInputDescription.Fields} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInputDescription.Fields, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WebUIInputDescription.Fields|undefined} value\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n*/\nproto.ubo.v1.WebUIInputDescription.prototype.setFields = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.clearFields = function() {\n  return this.setFields(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.hasFields = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string id = 4;\n * @return {string}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.clearId = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.hasId = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string title = 5;\n * @return {string}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.setTitle = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.clearTitle = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.hasTitle = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string prompt = 6;\n * @return {string}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.getPrompt = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.setPrompt = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIInputDescription} returns this\n */\nproto.ubo.v1.WebUIInputDescription.prototype.clearPrompt = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInputDescription.prototype.hasPrompt = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.QRCodeInputDescription.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.QRCodeInputDescription} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.QRCodeInputDescription.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    instructions: (f = msg.getInstructions()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),\n    pattern: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    prompt: jspb.Message.getFieldWithDefault(msg, 7, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.QRCodeInputDescription}\n */\nproto.ubo.v1.QRCodeInputDescription.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.QRCodeInputDescription;\n  return proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.QRCodeInputDescription} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.QRCodeInputDescription}\n */\nproto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());\n      msg.setInputMethod(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.ReadableInformation;\n      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);\n      msg.setInstructions(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPattern(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPrompt(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.QRCodeInputDescription} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getInstructions();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional InputMethod input_method = 2;\n * @return {!proto.ubo.v1.InputMethod}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getInputMethod = function() {\n  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputMethod} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.setInputMethod = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearInputMethod = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasInputMethod = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional ReadableInformation instructions = 3;\n * @return {?proto.ubo.v1.ReadableInformation}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getInstructions = function() {\n  return /** @type{?proto.ubo.v1.ReadableInformation} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReadableInformation|undefined} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n*/\nproto.ubo.v1.QRCodeInputDescription.prototype.setInstructions = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearInstructions = function() {\n  return this.setInstructions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasInstructions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string pattern = 4;\n * @return {string}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getPattern = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.setPattern = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearPattern = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasPattern = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string id = 5;\n * @return {string}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearId = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasId = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string title = 6;\n * @return {string}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.setTitle = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearTitle = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasTitle = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string prompt = 7;\n * @return {string}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.getPrompt = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.setPrompt = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.QRCodeInputDescription} returns this\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.clearPrompt = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.QRCodeInputDescription.prototype.hasPrompt = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PathInputDescription.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PathInputDescription.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PathInputDescription} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PathInputDescription.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    selectorConfig: (f = msg.getSelectorConfig()) && proto.ubo.v1.PathSelectorConfig.toObject(includeInstance, f),\n    id: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    prompt: jspb.Message.getFieldWithDefault(msg, 6, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PathInputDescription}\n */\nproto.ubo.v1.PathInputDescription.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PathInputDescription;\n  return proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PathInputDescription} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PathInputDescription}\n */\nproto.ubo.v1.PathInputDescription.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());\n      msg.setInputMethod(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.PathSelectorConfig;\n      reader.readMessage(value,proto.ubo.v1.PathSelectorConfig.deserializeBinaryFromReader);\n      msg.setSelectorConfig(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPrompt(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PathInputDescription.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PathInputDescription.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PathInputDescription} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PathInputDescription.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getSelectorConfig();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.PathSelectorConfig.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.PathInputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathInputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional InputMethod input_method = 2;\n * @return {!proto.ubo.v1.InputMethod}\n */\nproto.ubo.v1.PathInputDescription.prototype.getInputMethod = function() {\n  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.InputMethod} value\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.setInputMethod = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.clearInputMethod = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathInputDescription.prototype.hasInputMethod = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional PathSelectorConfig selector_config = 3;\n * @return {?proto.ubo.v1.PathSelectorConfig}\n */\nproto.ubo.v1.PathInputDescription.prototype.getSelectorConfig = function() {\n  return /** @type{?proto.ubo.v1.PathSelectorConfig} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PathSelectorConfig, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PathSelectorConfig|undefined} value\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n*/\nproto.ubo.v1.PathInputDescription.prototype.setSelectorConfig = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.clearSelectorConfig = function() {\n  return this.setSelectorConfig(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathInputDescription.prototype.hasSelectorConfig = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string id = 4;\n * @return {string}\n */\nproto.ubo.v1.PathInputDescription.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.clearId = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathInputDescription.prototype.hasId = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string title = 5;\n * @return {string}\n */\nproto.ubo.v1.PathInputDescription.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.setTitle = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.clearTitle = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathInputDescription.prototype.hasTitle = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string prompt = 6;\n * @return {string}\n */\nproto.ubo.v1.PathInputDescription.prototype.getPrompt = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.setPrompt = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathInputDescription} returns this\n */\nproto.ubo.v1.PathInputDescription.prototype.clearPrompt = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathInputDescription.prototype.hasPrompt = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputAction}\n */\nproto.ubo.v1.InputAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputAction;\n  return proto.ubo.v1.InputAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputAction}\n */\nproto.ubo.v1.InputAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputAction} returns this\n */\nproto.ubo.v1.InputAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputAction} returns this\n */\nproto.ubo.v1.InputAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputDemandAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputDemandAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputDemandAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputDemandAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    description: (f = msg.getDescription()) && proto.ubo.v1.AvailableInputDescription.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputDemandAction}\n */\nproto.ubo.v1.InputDemandAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputDemandAction;\n  return proto.ubo.v1.InputDemandAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputDemandAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputDemandAction}\n */\nproto.ubo.v1.InputDemandAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AvailableInputDescription;\n      reader.readMessage(value,proto.ubo.v1.AvailableInputDescription.deserializeBinaryFromReader);\n      msg.setDescription(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputDemandAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputDemandAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputDemandAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputDemandAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDescription();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AvailableInputDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputDemandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputDemandAction} returns this\n */\nproto.ubo.v1.InputDemandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputDemandAction} returns this\n */\nproto.ubo.v1.InputDemandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputDemandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AvailableInputDescription description = 2;\n * @return {?proto.ubo.v1.AvailableInputDescription}\n */\nproto.ubo.v1.InputDemandAction.prototype.getDescription = function() {\n  return /** @type{?proto.ubo.v1.AvailableInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AvailableInputDescription, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AvailableInputDescription|undefined} value\n * @return {!proto.ubo.v1.InputDemandAction} returns this\n*/\nproto.ubo.v1.InputDemandAction.prototype.setDescription = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.InputDemandAction} returns this\n */\nproto.ubo.v1.InputDemandAction.prototype.clearDescription = function() {\n  return this.setDescription(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputDemandAction.prototype.hasDescription = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputResolveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputResolveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputResolveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputResolveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputResolveAction}\n */\nproto.ubo.v1.InputResolveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputResolveAction;\n  return proto.ubo.v1.InputResolveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputResolveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputResolveAction}\n */\nproto.ubo.v1.InputResolveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputResolveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputResolveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputResolveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputResolveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputResolveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputResolveAction} returns this\n */\nproto.ubo.v1.InputResolveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputResolveAction} returns this\n */\nproto.ubo.v1.InputResolveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputResolveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.InputResolveAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputResolveAction} returns this\n */\nproto.ubo.v1.InputResolveAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputCancelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputCancelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputCancelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputCancelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputCancelAction}\n */\nproto.ubo.v1.InputCancelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputCancelAction;\n  return proto.ubo.v1.InputCancelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputCancelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputCancelAction}\n */\nproto.ubo.v1.InputCancelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputCancelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputCancelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputCancelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputCancelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputCancelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputCancelAction} returns this\n */\nproto.ubo.v1.InputCancelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputCancelAction} returns this\n */\nproto.ubo.v1.InputCancelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputCancelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.InputCancelAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputCancelAction} returns this\n */\nproto.ubo.v1.InputCancelAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputProvideAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputProvideAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputProvideAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputProvideAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    value: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    result: (f = msg.getResult()) && proto.ubo.v1.InputResult.toObject(includeInstance, f),\n    id: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputProvideAction}\n */\nproto.ubo.v1.InputProvideAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputProvideAction;\n  return proto.ubo.v1.InputProvideAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputProvideAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputProvideAction}\n */\nproto.ubo.v1.InputProvideAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setValue(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.InputResult;\n      reader.readMessage(value,proto.ubo.v1.InputResult.deserializeBinaryFromReader);\n      msg.setResult(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputProvideAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputProvideAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputProvideAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputProvideAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getValue();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getResult();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.InputResult.serializeBinaryToWriter\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputProvideAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputProvideAction} returns this\n */\nproto.ubo.v1.InputProvideAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputProvideAction} returns this\n */\nproto.ubo.v1.InputProvideAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputProvideAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string value = 2;\n * @return {string}\n */\nproto.ubo.v1.InputProvideAction.prototype.getValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputProvideAction} returns this\n */\nproto.ubo.v1.InputProvideAction.prototype.setValue = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional InputResult result = 3;\n * @return {?proto.ubo.v1.InputResult}\n */\nproto.ubo.v1.InputProvideAction.prototype.getResult = function() {\n  return /** @type{?proto.ubo.v1.InputResult} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResult, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputResult|undefined} value\n * @return {!proto.ubo.v1.InputProvideAction} returns this\n*/\nproto.ubo.v1.InputProvideAction.prototype.setResult = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.InputProvideAction} returns this\n */\nproto.ubo.v1.InputProvideAction.prototype.clearResult = function() {\n  return this.setResult(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputProvideAction.prototype.hasResult = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string id = 4;\n * @return {string}\n */\nproto.ubo.v1.InputProvideAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputProvideAction} returns this\n */\nproto.ubo.v1.InputProvideAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputResolveEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputResolveEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputResolveEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputResolveEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputResolveEvent}\n */\nproto.ubo.v1.InputResolveEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputResolveEvent;\n  return proto.ubo.v1.InputResolveEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputResolveEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputResolveEvent}\n */\nproto.ubo.v1.InputResolveEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputResolveEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputResolveEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputResolveEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputResolveEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputResolveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputResolveEvent} returns this\n */\nproto.ubo.v1.InputResolveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputResolveEvent} returns this\n */\nproto.ubo.v1.InputResolveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputResolveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.InputResolveEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputResolveEvent} returns this\n */\nproto.ubo.v1.InputResolveEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputCancelEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputCancelEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputCancelEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputCancelEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputCancelEvent}\n */\nproto.ubo.v1.InputCancelEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputCancelEvent;\n  return proto.ubo.v1.InputCancelEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputCancelEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputCancelEvent}\n */\nproto.ubo.v1.InputCancelEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputCancelEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputCancelEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputCancelEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputCancelEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputCancelEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputCancelEvent} returns this\n */\nproto.ubo.v1.InputCancelEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputCancelEvent} returns this\n */\nproto.ubo.v1.InputCancelEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputCancelEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.InputCancelEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputCancelEvent} returns this\n */\nproto.ubo.v1.InputCancelEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InputProvideEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InputProvideEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InputProvideEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputProvideEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    value: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    result: (f = msg.getResult()) && proto.ubo.v1.InputResult.toObject(includeInstance, f),\n    id: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InputProvideEvent}\n */\nproto.ubo.v1.InputProvideEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InputProvideEvent;\n  return proto.ubo.v1.InputProvideEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InputProvideEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InputProvideEvent}\n */\nproto.ubo.v1.InputProvideEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setValue(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.InputResult;\n      reader.readMessage(value,proto.ubo.v1.InputResult.deserializeBinaryFromReader);\n      msg.setResult(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InputProvideEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InputProvideEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InputProvideEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InputProvideEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getValue();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getResult();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.InputResult.serializeBinaryToWriter\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.InputProvideEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputProvideEvent} returns this\n */\nproto.ubo.v1.InputProvideEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InputProvideEvent} returns this\n */\nproto.ubo.v1.InputProvideEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputProvideEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string value = 2;\n * @return {string}\n */\nproto.ubo.v1.InputProvideEvent.prototype.getValue = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputProvideEvent} returns this\n */\nproto.ubo.v1.InputProvideEvent.prototype.setValue = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional InputResult result = 3;\n * @return {?proto.ubo.v1.InputResult}\n */\nproto.ubo.v1.InputProvideEvent.prototype.getResult = function() {\n  return /** @type{?proto.ubo.v1.InputResult} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResult, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputResult|undefined} value\n * @return {!proto.ubo.v1.InputProvideEvent} returns this\n*/\nproto.ubo.v1.InputProvideEvent.prototype.setResult = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.InputProvideEvent} returns this\n */\nproto.ubo.v1.InputProvideEvent.prototype.clearResult = function() {\n  return this.setResult(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InputProvideEvent.prototype.hasResult = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string id = 4;\n * @return {string}\n */\nproto.ubo.v1.InputProvideEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InputProvideEvent} returns this\n */\nproto.ubo.v1.InputProvideEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.AvailableInputDescription.oneofGroups_ = [[1,2,3]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase = {\n  AVAILABLE_INPUT_DESCRIPTION_NOT_SET: 0,\n  PATH_INPUT_DESCRIPTION: 1,\n  QR_CODE_INPUT_DESCRIPTION: 2,\n  WEB_UI_INPUT_DESCRIPTION: 3\n};\n\n/**\n * @return {proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.getAvailableInputDescriptionCase = function() {\n  return /** @type {proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AvailableInputDescription.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AvailableInputDescription} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AvailableInputDescription.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    pathInputDescription: (f = msg.getPathInputDescription()) && proto.ubo.v1.PathInputDescription.toObject(includeInstance, f),\n    qrCodeInputDescription: (f = msg.getQrCodeInputDescription()) && proto.ubo.v1.QRCodeInputDescription.toObject(includeInstance, f),\n    webUiInputDescription: (f = msg.getWebUiInputDescription()) && proto.ubo.v1.WebUIInputDescription.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AvailableInputDescription}\n */\nproto.ubo.v1.AvailableInputDescription.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AvailableInputDescription;\n  return proto.ubo.v1.AvailableInputDescription.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AvailableInputDescription} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AvailableInputDescription}\n */\nproto.ubo.v1.AvailableInputDescription.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.PathInputDescription;\n      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);\n      msg.setPathInputDescription(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.QRCodeInputDescription;\n      reader.readMessage(value,proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader);\n      msg.setQrCodeInputDescription(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.WebUIInputDescription;\n      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader);\n      msg.setWebUiInputDescription(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AvailableInputDescription.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AvailableInputDescription} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AvailableInputDescription.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getPathInputDescription();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter\n    );\n  }\n  f = message.getQrCodeInputDescription();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter\n    );\n  }\n  f = message.getWebUiInputDescription();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional PathInputDescription path_input_description = 1;\n * @return {?proto.ubo.v1.PathInputDescription}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.getPathInputDescription = function() {\n  return /** @type{?proto.ubo.v1.PathInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PathInputDescription, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PathInputDescription|undefined} value\n * @return {!proto.ubo.v1.AvailableInputDescription} returns this\n*/\nproto.ubo.v1.AvailableInputDescription.prototype.setPathInputDescription = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AvailableInputDescription} returns this\n */\nproto.ubo.v1.AvailableInputDescription.prototype.clearPathInputDescription = function() {\n  return this.setPathInputDescription(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.hasPathInputDescription = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional QRCodeInputDescription qr_code_input_description = 2;\n * @return {?proto.ubo.v1.QRCodeInputDescription}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.getQrCodeInputDescription = function() {\n  return /** @type{?proto.ubo.v1.QRCodeInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.QRCodeInputDescription, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.QRCodeInputDescription|undefined} value\n * @return {!proto.ubo.v1.AvailableInputDescription} returns this\n*/\nproto.ubo.v1.AvailableInputDescription.prototype.setQrCodeInputDescription = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AvailableInputDescription} returns this\n */\nproto.ubo.v1.AvailableInputDescription.prototype.clearQrCodeInputDescription = function() {\n  return this.setQrCodeInputDescription(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.hasQrCodeInputDescription = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional WebUIInputDescription web_ui_input_description = 3;\n * @return {?proto.ubo.v1.WebUIInputDescription}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.getWebUiInputDescription = function() {\n  return /** @type{?proto.ubo.v1.WebUIInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInputDescription, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WebUIInputDescription|undefined} value\n * @return {!proto.ubo.v1.AvailableInputDescription} returns this\n*/\nproto.ubo.v1.AvailableInputDescription.prototype.setWebUiInputDescription = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AvailableInputDescription} returns this\n */\nproto.ubo.v1.AvailableInputDescription.prototype.clearWebUiInputDescription = function() {\n  return this.setWebUiInputDescription(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AvailableInputDescription.prototype.hasWebUiInputDescription = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsAction}\n */\nproto.ubo.v1.SettingsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsAction;\n  return proto.ubo.v1.SettingsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsAction}\n */\nproto.ubo.v1.SettingsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsAction} returns this\n */\nproto.ubo.v1.SettingsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsAction} returns this\n */\nproto.ubo.v1.SettingsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsTogglePdbSignalAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsTogglePdbSignalAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction}\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsTogglePdbSignalAction;\n  return proto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsTogglePdbSignalAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction}\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsTogglePdbSignalAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsTogglePdbSignalAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction} returns this\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction} returns this\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsTogglePdbSignalAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsToggleVisualDebugAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsToggleVisualDebugAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction}\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsToggleVisualDebugAction;\n  return proto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsToggleVisualDebugAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction}\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsToggleVisualDebugAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsToggleVisualDebugAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction} returns this\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction} returns this\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsToggleVisualDebugAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsToggleBetaVersionsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsToggleBetaVersionsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction}\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsToggleBetaVersionsAction;\n  return proto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsToggleBetaVersionsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction}\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsToggleBetaVersionsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsToggleBetaVersionsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction} returns this\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction} returns this\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsSetServicesAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsSetServicesAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsSetServicesAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    servicesMap: (f = msg.getServicesMap()) ? f.toObject(includeInstance, proto.ubo.v1.ServiceState.toObject) : [],\n    gapDuration: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsSetServicesAction}\n */\nproto.ubo.v1.SettingsSetServicesAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsSetServicesAction;\n  return proto.ubo.v1.SettingsSetServicesAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsSetServicesAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsSetServicesAction}\n */\nproto.ubo.v1.SettingsSetServicesAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = msg.getServicesMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.ServiceState.deserializeBinaryFromReader, \"\", new proto.ubo.v1.ServiceState());\n         });\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setGapDuration(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsSetServicesAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsSetServicesAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsSetServicesAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServicesMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(2, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.ServiceState.serializeBinaryToWriter);\n  }\n  f = message.getGapDuration();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * map<string, ServiceState> services = 2;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.ServiceState>}\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.getServicesMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.ServiceState>} */ (\n      jspb.Message.getMapField(this, 2, opt_noLazyCreate,\n      proto.ubo.v1.ServiceState));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.clearServicesMap = function() {\n  this.getServicesMap().clear();\n  return this;};\n\n\n/**\n * optional float gap_duration = 3;\n * @return {number}\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.getGapDuration = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this\n */\nproto.ubo.v1.SettingsSetServicesAction.prototype.setGapDuration = function(value) {\n  return jspb.Message.setProto3FloatField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsServiceAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsServiceAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsServiceAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsServiceAction}\n */\nproto.ubo.v1.SettingsServiceAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsServiceAction;\n  return proto.ubo.v1.SettingsServiceAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsServiceAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsServiceAction}\n */\nproto.ubo.v1.SettingsServiceAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsServiceAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsServiceAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsServiceAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceAction} returns this\n */\nproto.ubo.v1.SettingsServiceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsServiceAction} returns this\n */\nproto.ubo.v1.SettingsServiceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service_id = 2;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceAction} returns this\n */\nproto.ubo.v1.SettingsServiceAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsStartServiceAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsStartServiceAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStartServiceAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsStartServiceAction}\n */\nproto.ubo.v1.SettingsStartServiceAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsStartServiceAction;\n  return proto.ubo.v1.SettingsStartServiceAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsStartServiceAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsStartServiceAction}\n */\nproto.ubo.v1.SettingsStartServiceAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsStartServiceAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsStartServiceAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStartServiceAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStartServiceAction} returns this\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsStartServiceAction} returns this\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service_id = 2;\n * @return {string}\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStartServiceAction} returns this\n */\nproto.ubo.v1.SettingsStartServiceAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsStopServiceAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsStopServiceAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStopServiceAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsStopServiceAction}\n */\nproto.ubo.v1.SettingsStopServiceAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsStopServiceAction;\n  return proto.ubo.v1.SettingsStopServiceAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsStopServiceAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsStopServiceAction}\n */\nproto.ubo.v1.SettingsStopServiceAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsStopServiceAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsStopServiceAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStopServiceAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStopServiceAction} returns this\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsStopServiceAction} returns this\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service_id = 2;\n * @return {string}\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStopServiceAction} returns this\n */\nproto.ubo.v1.SettingsStopServiceAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsServiceSetStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsServiceSetStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsServiceSetStatusAction}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsServiceSetStatusAction;\n  return proto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsServiceSetStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsServiceSetStatusAction}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsServiceSetStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsServiceSetStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsActive();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional string service_id = 3;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetStatusAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsServiceSetIsEnabledAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsServiceSetIsEnabledAction;\n  return proto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnabled(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsServiceSetIsEnabledAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsEnabled();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_enabled = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.getIsEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.setIsEnabled = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional string service_id = 3;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsServiceSetLogLevelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsServiceSetLogLevelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    logLevel: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsServiceSetLogLevelAction;\n  return proto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsServiceSetLogLevelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setLogLevel(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsServiceSetLogLevelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsServiceSetLogLevelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getLogLevel();\n  if (f !== 0) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 log_level = 2;\n * @return {number}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.getLogLevel = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.setLogLevel = function(value) {\n  return jspb.Message.setProto3IntField(this, 2, value);\n};\n\n\n/**\n * optional string service_id = 3;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsServiceSetShouldRestartAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    shouldAutoRestart: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsServiceSetShouldRestartAction;\n  return proto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShouldAutoRestart(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsServiceSetShouldRestartAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getShouldAutoRestart();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool should_auto_restart = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.getShouldAutoRestart = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.setShouldAutoRestart = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional string service_id = 3;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this\n */\nproto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsReportServiceErrorAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsReportServiceErrorAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    error: (f = msg.getError()) && proto.ubo.v1.ErrorReport.toObject(includeInstance, f),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsReportServiceErrorAction;\n  return proto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsReportServiceErrorAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.ErrorReport;\n      reader.readMessage(value,proto.ubo.v1.ErrorReport.deserializeBinaryFromReader);\n      msg.setError(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsReportServiceErrorAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsReportServiceErrorAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getError();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.ErrorReport.serializeBinaryToWriter\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional ErrorReport error = 2;\n * @return {?proto.ubo.v1.ErrorReport}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.getError = function() {\n  return /** @type{?proto.ubo.v1.ErrorReport} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ErrorReport, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ErrorReport|undefined} value\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this\n*/\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.setError = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.clearError = function() {\n  return this.setError(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.hasError = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string service_id = 3;\n * @return {string}\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this\n */\nproto.ubo.v1.SettingsReportServiceErrorAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsClearServiceErrorsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsClearServiceErrorsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsClearServiceErrorsAction;\n  return proto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsClearServiceErrorsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsClearServiceErrorsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsClearServiceErrorsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction} returns this\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction} returns this\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service_id = 2;\n * @return {string}\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction} returns this\n */\nproto.ubo.v1.SettingsClearServiceErrorsAction.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsEvent}\n */\nproto.ubo.v1.SettingsEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsEvent;\n  return proto.ubo.v1.SettingsEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsEvent}\n */\nproto.ubo.v1.SettingsEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsEvent} returns this\n */\nproto.ubo.v1.SettingsEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsEvent} returns this\n */\nproto.ubo.v1.SettingsEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsServiceEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsServiceEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsServiceEvent}\n */\nproto.ubo.v1.SettingsServiceEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsServiceEvent;\n  return proto.ubo.v1.SettingsServiceEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsServiceEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsServiceEvent}\n */\nproto.ubo.v1.SettingsServiceEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsServiceEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsServiceEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsServiceEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceEvent} returns this\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsServiceEvent} returns this\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service_id = 2;\n * @return {string}\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsServiceEvent} returns this\n */\nproto.ubo.v1.SettingsServiceEvent.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsStartServiceEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsStartServiceEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStartServiceEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    delay: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsStartServiceEvent}\n */\nproto.ubo.v1.SettingsStartServiceEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsStartServiceEvent;\n  return proto.ubo.v1.SettingsStartServiceEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsStartServiceEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsStartServiceEvent}\n */\nproto.ubo.v1.SettingsStartServiceEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setDelay(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsStartServiceEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsStartServiceEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStartServiceEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float delay = 2;\n * @return {number}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.getDelay = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.setDelay = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.clearDelay = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.hasDelay = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string service_id = 3;\n * @return {string}\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStartServiceEvent.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsStopServiceEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsStopServiceEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStopServiceEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsStopServiceEvent}\n */\nproto.ubo.v1.SettingsStopServiceEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsStopServiceEvent;\n  return proto.ubo.v1.SettingsStopServiceEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsStopServiceEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsStopServiceEvent}\n */\nproto.ubo.v1.SettingsStopServiceEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsStopServiceEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsStopServiceEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsStopServiceEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStopServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsStopServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string service_id = 2;\n * @return {string}\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsStopServiceEvent} returns this\n */\nproto.ubo.v1.SettingsStopServiceEvent.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ErrorReport.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ErrorReport.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ErrorReport} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ErrorReport.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    message: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ErrorReport}\n */\nproto.ubo.v1.ErrorReport.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ErrorReport;\n  return proto.ubo.v1.ErrorReport.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ErrorReport} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ErrorReport}\n */\nproto.ubo.v1.ErrorReport.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMessage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ErrorReport.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ErrorReport.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ErrorReport} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ErrorReport.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getMessage();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ErrorReport.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ErrorReport} returns this\n */\nproto.ubo.v1.ErrorReport.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ErrorReport} returns this\n */\nproto.ubo.v1.ErrorReport.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ErrorReport.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.ErrorReport.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.ErrorReport} returns this\n */\nproto.ubo.v1.ErrorReport.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional string message = 3;\n * @return {string}\n */\nproto.ubo.v1.ErrorReport.prototype.getMessage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ErrorReport} returns this\n */\nproto.ubo.v1.ErrorReport.prototype.setMessage = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ServiceState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ServiceState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ServiceState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ServiceState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    logLevel: jspb.Message.getFieldWithDefault(msg, 6, 0),\n    shouldAutoRestart: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),\n    errors: (f = msg.getErrors()) && proto.ubo.v1.ServiceState.Errors.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ServiceState}\n */\nproto.ubo.v1.ServiceState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ServiceState;\n  return proto.ubo.v1.ServiceState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ServiceState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ServiceState}\n */\nproto.ubo.v1.ServiceState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnabled(value);\n      break;\n    case 6:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setLogLevel(value);\n      break;\n    case 7:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShouldAutoRestart(value);\n      break;\n    case 8:\n      var value = new proto.ubo.v1.ServiceState.Errors;\n      reader.readMessage(value,proto.ubo.v1.ServiceState.Errors.deserializeBinaryFromReader);\n      msg.setErrors(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ServiceState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ServiceState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ServiceState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ServiceState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getLabel();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getIsActive();\n  if (f) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = message.getIsEnabled();\n  if (f) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = message.getLogLevel();\n  if (f !== 0) {\n    writer.writeInt64(\n      6,\n      f\n    );\n  }\n  f = message.getShouldAutoRestart();\n  if (f) {\n    writer.writeBool(\n      7,\n      f\n    );\n  }\n  f = message.getErrors();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.ubo.v1.ServiceState.Errors.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.ServiceState.Errors.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ServiceState.Errors.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ServiceState.Errors.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ServiceState.Errors} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ServiceState.Errors.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.ErrorReport.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ServiceState.Errors}\n */\nproto.ubo.v1.ServiceState.Errors.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ServiceState.Errors;\n  return proto.ubo.v1.ServiceState.Errors.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ServiceState.Errors} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ServiceState.Errors}\n */\nproto.ubo.v1.ServiceState.Errors.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.ErrorReport;\n      reader.readMessage(value,proto.ubo.v1.ErrorReport.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ServiceState.Errors.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ServiceState.Errors.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ServiceState.Errors} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ServiceState.Errors.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.ErrorReport.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated ErrorReport items = 1;\n * @return {!Array<!proto.ubo.v1.ErrorReport>}\n */\nproto.ubo.v1.ServiceState.Errors.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.ErrorReport>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.ErrorReport, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.ErrorReport>} value\n * @return {!proto.ubo.v1.ServiceState.Errors} returns this\n*/\nproto.ubo.v1.ServiceState.Errors.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.ErrorReport=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.ErrorReport}\n */\nproto.ubo.v1.ServiceState.Errors.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.ErrorReport, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.ServiceState.Errors} returns this\n */\nproto.ubo.v1.ServiceState.Errors.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.ServiceState.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ServiceState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.ServiceState.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string label = 3;\n * @return {string}\n */\nproto.ubo.v1.ServiceState.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setLabel = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional bool is_active = 4;\n * @return {boolean}\n */\nproto.ubo.v1.ServiceState.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setIsActive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 4, value);\n};\n\n\n/**\n * optional bool is_enabled = 5;\n * @return {boolean}\n */\nproto.ubo.v1.ServiceState.prototype.getIsEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setIsEnabled = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 5, value);\n};\n\n\n/**\n * optional int64 log_level = 6;\n * @return {number}\n */\nproto.ubo.v1.ServiceState.prototype.getLogLevel = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setLogLevel = function(value) {\n  return jspb.Message.setProto3IntField(this, 6, value);\n};\n\n\n/**\n * optional bool should_auto_restart = 7;\n * @return {boolean}\n */\nproto.ubo.v1.ServiceState.prototype.getShouldAutoRestart = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.setShouldAutoRestart = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 7, value);\n};\n\n\n/**\n * optional Errors errors = 8;\n * @return {?proto.ubo.v1.ServiceState.Errors}\n */\nproto.ubo.v1.ServiceState.prototype.getErrors = function() {\n  return /** @type{?proto.ubo.v1.ServiceState.Errors} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ServiceState.Errors, 8));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ServiceState.Errors|undefined} value\n * @return {!proto.ubo.v1.ServiceState} returns this\n*/\nproto.ubo.v1.ServiceState.prototype.setErrors = function(value) {\n  return jspb.Message.setWrapperField(this, 8, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.ServiceState} returns this\n */\nproto.ubo.v1.ServiceState.prototype.clearErrors = function() {\n  return this.setErrors(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ServiceState.prototype.hasErrors = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    pdbSignal: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    visualDebug: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    betaVersions: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    services: (f = msg.getServices()) && proto.ubo.v1.SettingsState.ServicesDict.toObject(includeInstance, f),\n    servicesStatus: jspb.Message.getFieldWithDefault(msg, 6, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsState}\n */\nproto.ubo.v1.SettingsState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsState;\n  return proto.ubo.v1.SettingsState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsState}\n */\nproto.ubo.v1.SettingsState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setPdbSignal(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setVisualDebug(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setBetaVersions(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.SettingsState.ServicesDict;\n      reader.readMessage(value,proto.ubo.v1.SettingsState.ServicesDict.deserializeBinaryFromReader);\n      msg.setServices(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.ubo.v1.ServicesStatus} */ (reader.readEnum());\n      msg.setServicesStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = message.getServices();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.SettingsState.ServicesDict.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {!proto.ubo.v1.ServicesStatus} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SettingsState.ServicesDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SettingsState.ServicesDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SettingsState.ServicesDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsState.ServicesDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.ServiceState.toObject) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SettingsState.ServicesDict}\n */\nproto.ubo.v1.SettingsState.ServicesDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SettingsState.ServicesDict;\n  return proto.ubo.v1.SettingsState.ServicesDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SettingsState.ServicesDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SettingsState.ServicesDict}\n */\nproto.ubo.v1.SettingsState.ServicesDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.ServiceState.deserializeBinaryFromReader, \"\", new proto.ubo.v1.ServiceState());\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SettingsState.ServicesDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SettingsState.ServicesDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SettingsState.ServicesDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SettingsState.ServicesDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.ServiceState.serializeBinaryToWriter);\n  }\n};\n\n\n/**\n * map<string, ServiceState> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.ServiceState>}\n */\nproto.ubo.v1.SettingsState.ServicesDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.ServiceState>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      proto.ubo.v1.ServiceState));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.SettingsState.ServicesDict} returns this\n */\nproto.ubo.v1.SettingsState.ServicesDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.SettingsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool pdb_signal = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.getPdbSignal = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.setPdbSignal = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.clearPdbSignal = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.hasPdbSignal = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool visual_debug = 3;\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.getVisualDebug = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.setVisualDebug = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.clearVisualDebug = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.hasVisualDebug = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool beta_versions = 4;\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.getBetaVersions = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.setBetaVersions = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.clearBetaVersions = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.hasBetaVersions = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional ServicesDict services = 5;\n * @return {?proto.ubo.v1.SettingsState.ServicesDict}\n */\nproto.ubo.v1.SettingsState.prototype.getServices = function() {\n  return /** @type{?proto.ubo.v1.SettingsState.ServicesDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsState.ServicesDict, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsState.ServicesDict|undefined} value\n * @return {!proto.ubo.v1.SettingsState} returns this\n*/\nproto.ubo.v1.SettingsState.prototype.setServices = function(value) {\n  return jspb.Message.setWrapperField(this, 5, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.clearServices = function() {\n  return this.setServices(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.hasServices = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional ServicesStatus services_status = 6;\n * @return {!proto.ubo.v1.ServicesStatus}\n */\nproto.ubo.v1.SettingsState.prototype.getServicesStatus = function() {\n  return /** @type {!proto.ubo.v1.ServicesStatus} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.ServicesStatus} value\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.setServicesStatus = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SettingsState} returns this\n */\nproto.ubo.v1.SettingsState.prototype.clearServicesStatus = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SettingsState.prototype.hasServicesStatus = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IconState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IconState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IconState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IconState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    symbol: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    priority: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    serviceId: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 6, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IconState}\n */\nproto.ubo.v1.IconState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IconState;\n  return proto.ubo.v1.IconState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IconState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IconState}\n */\nproto.ubo.v1.IconState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSymbol(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setPriority(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServiceId(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IconState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IconState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IconState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IconState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSymbol();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getPriority();\n  if (f !== 0) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = message.getServiceId();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.IconState.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IconState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string symbol = 2;\n * @return {string}\n */\nproto.ubo.v1.IconState.prototype.getSymbol = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.setSymbol = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string color = 3;\n * @return {string}\n */\nproto.ubo.v1.IconState.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.setColor = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional int64 priority = 4;\n * @return {number}\n */\nproto.ubo.v1.IconState.prototype.getPriority = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.setPriority = function(value) {\n  return jspb.Message.setProto3IntField(this, 4, value);\n};\n\n\n/**\n * optional string service_id = 5;\n * @return {string}\n */\nproto.ubo.v1.IconState.prototype.getServiceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.setServiceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n/**\n * optional string id = 6;\n * @return {string}\n */\nproto.ubo.v1.IconState.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IconState} returns this\n */\nproto.ubo.v1.IconState.prototype.clearId = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IconState.prototype.hasId = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.StatusIconsState.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.StatusIconsState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.StatusIconsState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.StatusIconsState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StatusIconsState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    iconsList: jspb.Message.toObjectList(msg.getIconsList(),\n    proto.ubo.v1.IconState.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.StatusIconsState}\n */\nproto.ubo.v1.StatusIconsState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.StatusIconsState;\n  return proto.ubo.v1.StatusIconsState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.StatusIconsState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.StatusIconsState}\n */\nproto.ubo.v1.StatusIconsState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.IconState;\n      reader.readMessage(value,proto.ubo.v1.IconState.deserializeBinaryFromReader);\n      msg.addIcons(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.StatusIconsState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.StatusIconsState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.StatusIconsState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StatusIconsState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIconsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.IconState.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsState} returns this\n */\nproto.ubo.v1.StatusIconsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsState} returns this\n */\nproto.ubo.v1.StatusIconsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated IconState icons = 2;\n * @return {!Array<!proto.ubo.v1.IconState>}\n */\nproto.ubo.v1.StatusIconsState.prototype.getIconsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.IconState>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.IconState, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.IconState>} value\n * @return {!proto.ubo.v1.StatusIconsState} returns this\n*/\nproto.ubo.v1.StatusIconsState.prototype.setIconsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.IconState=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.IconState}\n */\nproto.ubo.v1.StatusIconsState.prototype.addIcons = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.IconState, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.StatusIconsState} returns this\n */\nproto.ubo.v1.StatusIconsState.prototype.clearIconsList = function() {\n  return this.setIconsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.StatusIconsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.StatusIconsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.StatusIconsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StatusIconsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.StatusIconsAction}\n */\nproto.ubo.v1.StatusIconsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.StatusIconsAction;\n  return proto.ubo.v1.StatusIconsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.StatusIconsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.StatusIconsAction}\n */\nproto.ubo.v1.StatusIconsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.StatusIconsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.StatusIconsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.StatusIconsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StatusIconsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsAction} returns this\n */\nproto.ubo.v1.StatusIconsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsAction} returns this\n */\nproto.ubo.v1.StatusIconsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.StatusIconsRegisterAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.StatusIconsRegisterAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StatusIconsRegisterAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    priority: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    id: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    service: jspb.Message.getFieldWithDefault(msg, 6, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction}\n */\nproto.ubo.v1.StatusIconsRegisterAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.StatusIconsRegisterAction;\n  return proto.ubo.v1.StatusIconsRegisterAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.StatusIconsRegisterAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction}\n */\nproto.ubo.v1.StatusIconsRegisterAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setPriority(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setService(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.StatusIconsRegisterAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.StatusIconsRegisterAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.StatusIconsRegisterAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIcon();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string icon = 2;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.setIcon = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string color = 3;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 priority = 4;\n * @return {number}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.getPriority = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.setPriority = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.clearPriority = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.hasPriority = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string id = 5;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.clearId = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.hasId = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string service = 6;\n * @return {string}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.getService = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.setService = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.clearService = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.StatusIconsRegisterAction.prototype.hasService = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerAction}\n */\nproto.ubo.v1.UpdateManagerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerAction;\n  return proto.ubo.v1.UpdateManagerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerAction}\n */\nproto.ubo.v1.UpdateManagerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerAction} returns this\n */\nproto.ubo.v1.UpdateManagerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerAction} returns this\n */\nproto.ubo.v1.UpdateManagerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerSetVersionsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    flashNotification: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    currentVersion: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    baseImageVariant: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    latestVersion: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    recentVersions: (f = msg.getRecentVersions()) && proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerSetVersionsAction;\n  return proto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setFlashNotification(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setCurrentVersion(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBaseImageVariant(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLatestVersion(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinaryFromReader);\n      msg.setRecentVersions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerSetVersionsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getFlashNotification();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getCurrentVersion();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getBaseImageVariant();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getLatestVersion();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = message.getRecentVersions();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions;\n  return proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool flash_notification = 2;\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getFlashNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setFlashNotification = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional string current_version = 3;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getCurrentVersion = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setCurrentVersion = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional string base_image_variant = 4;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getBaseImageVariant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setBaseImageVariant = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n/**\n * optional string latest_version = 5;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getLatestVersion = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setLatestVersion = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n/**\n * optional RecentVersions recent_versions = 6;\n * @return {?proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getRecentVersions = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions|undefined} value\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n*/\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setRecentVersions = function(value) {\n  return jspb.Message.setWrapperField(this, 6, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.clearRecentVersions = function() {\n  return this.setRecentVersions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerSetVersionsAction.prototype.hasRecentVersions = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerRequestCheckAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerRequestCheckAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction}\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerRequestCheckAction;\n  return proto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerRequestCheckAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction}\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerRequestCheckAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerRequestCheckAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction} returns this\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction} returns this\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerRequestCheckAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerReportFailedCheckAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction}\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerReportFailedCheckAction;\n  return proto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction}\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerReportFailedCheckAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} returns this\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} returns this\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerRequestUpdateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerRequestUpdateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    version: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerRequestUpdateAction;\n  return proto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerRequestUpdateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setVersion(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerRequestUpdateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerRequestUpdateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getVersion();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction} returns this\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction} returns this\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string version = 2;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.getVersion = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction} returns this\n */\nproto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.setVersion = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerEvent}\n */\nproto.ubo.v1.UpdateManagerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerEvent;\n  return proto.ubo.v1.UpdateManagerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerEvent}\n */\nproto.ubo.v1.UpdateManagerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerEvent} returns this\n */\nproto.ubo.v1.UpdateManagerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerEvent} returns this\n */\nproto.ubo.v1.UpdateManagerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerCheckEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerCheckEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerCheckEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerCheckEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerCheckEvent}\n */\nproto.ubo.v1.UpdateManagerCheckEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerCheckEvent;\n  return proto.ubo.v1.UpdateManagerCheckEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerCheckEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerCheckEvent}\n */\nproto.ubo.v1.UpdateManagerCheckEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerCheckEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerCheckEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerCheckEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerCheckEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerCheckEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerCheckEvent} returns this\n */\nproto.ubo.v1.UpdateManagerCheckEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerCheckEvent} returns this\n */\nproto.ubo.v1.UpdateManagerCheckEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerCheckEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerUpdateEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerUpdateEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    version: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerUpdateEvent}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerUpdateEvent;\n  return proto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerUpdateEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerUpdateEvent}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setVersion(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerUpdateEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerUpdateEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string version = 2;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.getVersion = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.setVersion = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.clearVersion = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerUpdateEvent.prototype.hasVersion = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    currentVersion: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    baseImageVariant: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    latestVersion: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    updateStatus: jspb.Message.getFieldWithDefault(msg, 5, 0),\n    recentVersions: (f = msg.getRecentVersions()) && proto.ubo.v1.UpdateManagerState.RecentVersions.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerState}\n */\nproto.ubo.v1.UpdateManagerState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerState;\n  return proto.ubo.v1.UpdateManagerState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerState}\n */\nproto.ubo.v1.UpdateManagerState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setCurrentVersion(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBaseImageVariant(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLatestVersion(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.ubo.v1.UpdateStatus} */ (reader.readEnum());\n      msg.setUpdateStatus(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.UpdateManagerState.RecentVersions;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinaryFromReader);\n      msg.setRecentVersions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.UpdateStatus} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n  f = message.getRecentVersions();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.UpdateManagerState.RecentVersions.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UpdateManagerState.RecentVersions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UpdateManagerState.RecentVersions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions}\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UpdateManagerState.RecentVersions;\n  return proto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UpdateManagerState.RecentVersions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions}\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UpdateManagerState.RecentVersions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UpdateManagerState.RecentVersions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions} returns this\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions} returns this\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions} returns this\n */\nproto.ubo.v1.UpdateManagerState.RecentVersions.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerState.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string current_version = 2;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerState.prototype.getCurrentVersion = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.setCurrentVersion = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.clearCurrentVersion = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerState.prototype.hasCurrentVersion = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string base_image_variant = 3;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerState.prototype.getBaseImageVariant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.setBaseImageVariant = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.clearBaseImageVariant = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerState.prototype.hasBaseImageVariant = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string latest_version = 4;\n * @return {string}\n */\nproto.ubo.v1.UpdateManagerState.prototype.getLatestVersion = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.setLatestVersion = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.clearLatestVersion = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerState.prototype.hasLatestVersion = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional UpdateStatus update_status = 5;\n * @return {!proto.ubo.v1.UpdateStatus}\n */\nproto.ubo.v1.UpdateManagerState.prototype.getUpdateStatus = function() {\n  return /** @type {!proto.ubo.v1.UpdateStatus} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.UpdateStatus} value\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.setUpdateStatus = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.clearUpdateStatus = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerState.prototype.hasUpdateStatus = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional RecentVersions recent_versions = 6;\n * @return {?proto.ubo.v1.UpdateManagerState.RecentVersions}\n */\nproto.ubo.v1.UpdateManagerState.prototype.getRecentVersions = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerState.RecentVersions} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerState.RecentVersions, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerState.RecentVersions|undefined} value\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n*/\nproto.ubo.v1.UpdateManagerState.prototype.setRecentVersions = function(value) {\n  return jspb.Message.setWrapperField(this, 6, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UpdateManagerState} returns this\n */\nproto.ubo.v1.UpdateManagerState.prototype.clearRecentVersions = function() {\n  return this.setRecentVersions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UpdateManagerState.prototype.hasRecentVersions = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.McpServerMetadata.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.McpServerMetadata.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.McpServerMetadata} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.McpServerMetadata.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serverId: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    type: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    config: jspb.Message.getFieldWithDefault(msg, 5, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.McpServerMetadata}\n */\nproto.ubo.v1.McpServerMetadata.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.McpServerMetadata;\n  return proto.ubo.v1.McpServerMetadata.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.McpServerMetadata} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.McpServerMetadata}\n */\nproto.ubo.v1.McpServerMetadata.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServerId(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.ubo.v1.McpServerType} */ (reader.readEnum());\n      msg.setType(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setConfig(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.McpServerMetadata.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.McpServerMetadata.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.McpServerMetadata} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.McpServerMetadata.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServerId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getType();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n  f = message.getConfig();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.McpServerMetadata.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.McpServerMetadata} returns this\n */\nproto.ubo.v1.McpServerMetadata.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.McpServerMetadata} returns this\n */\nproto.ubo.v1.McpServerMetadata.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.McpServerMetadata.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string server_id = 2;\n * @return {string}\n */\nproto.ubo.v1.McpServerMetadata.prototype.getServerId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.McpServerMetadata} returns this\n */\nproto.ubo.v1.McpServerMetadata.prototype.setServerId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string name = 3;\n * @return {string}\n */\nproto.ubo.v1.McpServerMetadata.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.McpServerMetadata} returns this\n */\nproto.ubo.v1.McpServerMetadata.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional McpServerType type = 4;\n * @return {!proto.ubo.v1.McpServerType}\n */\nproto.ubo.v1.McpServerMetadata.prototype.getType = function() {\n  return /** @type {!proto.ubo.v1.McpServerType} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.McpServerType} value\n * @return {!proto.ubo.v1.McpServerMetadata} returns this\n */\nproto.ubo.v1.McpServerMetadata.prototype.setType = function(value) {\n  return jspb.Message.setProto3EnumField(this, 4, value);\n};\n\n\n/**\n * optional string config = 5;\n * @return {string}\n */\nproto.ubo.v1.McpServerMetadata.prototype.getConfig = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.McpServerMetadata} returns this\n */\nproto.ubo.v1.McpServerMetadata.prototype.setConfig = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantAction}\n */\nproto.ubo.v1.AssistantAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantAction;\n  return proto.ubo.v1.AssistantAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantAction}\n */\nproto.ubo.v1.AssistantAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAction} returns this\n */\nproto.ubo.v1.AssistantAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantAction} returns this\n */\nproto.ubo.v1.AssistantAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSetIsActiveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSetIsActiveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetIsActiveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSetIsActiveAction}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSetIsActiveAction;\n  return proto.ubo.v1.AssistantSetIsActiveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSetIsActiveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSetIsActiveAction}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSetIsActiveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSetIsActiveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetIsActiveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsActive();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetIsActiveAction} returns this\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSetIsActiveAction} returns this\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistantSetIsActiveAction} returns this\n */\nproto.ubo.v1.AssistantSetIsActiveAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSetSelectedSTTAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSetSelectedSTTAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sttName: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSetSelectedSTTAction;\n  return proto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSetSelectedSTTAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.AssistantSTTName} */ (reader.readEnum());\n      msg.setSttName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSetSelectedSTTAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSetSelectedSTTAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSttName();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AssistantSTTName stt_name = 2;\n * @return {!proto.ubo.v1.AssistantSTTName}\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.getSttName = function() {\n  return /** @type {!proto.ubo.v1.AssistantSTTName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantSTTName} value\n * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedSTTAction.prototype.setSttName = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSetSelectedLLMAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSetSelectedLLMAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    llmName: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSetSelectedLLMAction;\n  return proto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSetSelectedLLMAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.AssistantLLMName} */ (reader.readEnum());\n      msg.setLlmName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSetSelectedLLMAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSetSelectedLLMAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getLlmName();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AssistantLLMName llm_name = 2;\n * @return {!proto.ubo.v1.AssistantLLMName}\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.getLlmName = function() {\n  return /** @type {!proto.ubo.v1.AssistantLLMName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantLLMName} value\n * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedLLMAction.prototype.setLlmName = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSetSelectedTTSAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSetSelectedTTSAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    ttsName: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSetSelectedTTSAction;\n  return proto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSetSelectedTTSAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.AssistantTTSName} */ (reader.readEnum());\n      msg.setTtsName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSetSelectedTTSAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSetSelectedTTSAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTtsName();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AssistantTTSName tts_name = 2;\n * @return {!proto.ubo.v1.AssistantTTSName}\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.getTtsName = function() {\n  return /** @type {!proto.ubo.v1.AssistantTTSName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantTTSName} value\n * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedTTSAction.prototype.setTtsName = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    imageGeneratorName: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSetSelectedImageGeneratorAction;\n  return proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (reader.readEnum());\n      msg.setImageGeneratorName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImageGeneratorName();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AssistantImageGeneratorName image_generator_name = 2;\n * @return {!proto.ubo.v1.AssistantImageGeneratorName}\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.getImageGeneratorName = function() {\n  return /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantImageGeneratorName} value\n * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.setImageGeneratorName = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSetSelectedModelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSetSelectedModelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    model: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSetSelectedModelAction}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSetSelectedModelAction;\n  return proto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSetSelectedModelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSetSelectedModelAction}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setModel(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSetSelectedModelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSetSelectedModelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getModel();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetSelectedModelAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSetSelectedModelAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string model = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.getModel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSetSelectedModelAction} returns this\n */\nproto.ubo.v1.AssistantSetSelectedModelAction.prototype.setModel = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantDownloadOllamaModelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantDownloadOllamaModelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    model: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantDownloadOllamaModelAction;\n  return proto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantDownloadOllamaModelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setModel(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantDownloadOllamaModelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantDownloadOllamaModelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getModel();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction} returns this\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction} returns this\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string model = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.getModel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction} returns this\n */\nproto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.setModel = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistanceFrame.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistanceFrame.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistanceFrame} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceFrame.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),\n    id: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 5, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistanceFrame}\n */\nproto.ubo.v1.AssistanceFrame.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistanceFrame;\n  return proto.ubo.v1.AssistanceFrame.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistanceFrame} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistanceFrame}\n */\nproto.ubo.v1.AssistanceFrame.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsLastFrame(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistanceFrame.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistanceFrame.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistanceFrame} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceFrame.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsLastFrame();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      3,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistanceFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceFrame} returns this\n */\nproto.ubo.v1.AssistanceFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistanceFrame} returns this\n */\nproto.ubo.v1.AssistanceFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_last_frame = 2;\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceFrame.prototype.getIsLastFrame = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistanceFrame} returns this\n */\nproto.ubo.v1.AssistanceFrame.prototype.setIsLastFrame = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional float timestamp = 3;\n * @return {number}\n */\nproto.ubo.v1.AssistanceFrame.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceFrame} returns this\n */\nproto.ubo.v1.AssistanceFrame.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 3, value);\n};\n\n\n/**\n * optional string id = 4;\n * @return {string}\n */\nproto.ubo.v1.AssistanceFrame.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceFrame} returns this\n */\nproto.ubo.v1.AssistanceFrame.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n/**\n * optional int64 index = 5;\n * @return {number}\n */\nproto.ubo.v1.AssistanceFrame.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceFrame} returns this\n */\nproto.ubo.v1.AssistanceFrame.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistanceTextFrame.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistanceTextFrame} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceTextFrame.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    text: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),\n    id: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 6, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistanceTextFrame}\n */\nproto.ubo.v1.AssistanceTextFrame.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistanceTextFrame;\n  return proto.ubo.v1.AssistanceTextFrame.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistanceTextFrame} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistanceTextFrame}\n */\nproto.ubo.v1.AssistanceTextFrame.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setText(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsLastFrame(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 6:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistanceTextFrame.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistanceTextFrame} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceTextFrame.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getText();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getIsLastFrame();\n  if (f) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      4,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string text = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.getText = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.setText = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional bool is_last_frame = 3;\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.getIsLastFrame = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.setIsLastFrame = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 3, value);\n};\n\n\n/**\n * optional float timestamp = 4;\n * @return {number}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 4, value);\n};\n\n\n/**\n * optional string id = 5;\n * @return {string}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n/**\n * optional int64 index = 6;\n * @return {number}\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceTextFrame} returns this\n */\nproto.ubo.v1.AssistanceTextFrame.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 6, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistanceAudioFrame.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistanceAudioFrame} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceAudioFrame.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    audio: (f = msg.getAudio()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f),\n    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),\n    id: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 6, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistanceAudioFrame}\n */\nproto.ubo.v1.AssistanceAudioFrame.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistanceAudioFrame;\n  return proto.ubo.v1.AssistanceAudioFrame.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistanceAudioFrame} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistanceAudioFrame}\n */\nproto.ubo.v1.AssistanceAudioFrame.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setAudio(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsLastFrame(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 6:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistanceAudioFrame.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistanceAudioFrame} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceAudioFrame.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getAudio();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n  f = message.getIsLastFrame();\n  if (f) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      4,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AudioSample audio = 2;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.getAudio = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n*/\nproto.ubo.v1.AssistanceAudioFrame.prototype.setAudio = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.clearAudio = function() {\n  return this.setAudio(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.hasAudio = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_last_frame = 3;\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.getIsLastFrame = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.setIsLastFrame = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 3, value);\n};\n\n\n/**\n * optional float timestamp = 4;\n * @return {number}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 4, value);\n};\n\n\n/**\n * optional string id = 5;\n * @return {string}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n/**\n * optional int64 index = 6;\n * @return {number}\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this\n */\nproto.ubo.v1.AssistanceAudioFrame.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 6, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistanceImageFrame.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistanceImageFrame} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceImageFrame.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: msg.getImage_asB64(),\n    width: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    height: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    format: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    metadataMap: (f = msg.getMetadataMap()) ? f.toObject(includeInstance, undefined) : [],\n    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 0.0),\n    id: jspb.Message.getFieldWithDefault(msg, 9, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 10, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistanceImageFrame}\n */\nproto.ubo.v1.AssistanceImageFrame.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistanceImageFrame;\n  return proto.ubo.v1.AssistanceImageFrame.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistanceImageFrame} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistanceImageFrame}\n */\nproto.ubo.v1.AssistanceImageFrame.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setImage(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWidth(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setHeight(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setFormat(value);\n      break;\n    case 6:\n      var value = msg.getMetadataMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, \"\", \"\");\n         });\n      break;\n    case 7:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsLastFrame(value);\n      break;\n    case 8:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 9:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 10:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistanceImageFrame.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistanceImageFrame} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistanceImageFrame.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      2,\n      f\n    );\n  }\n  f = message.getWidth();\n  if (f !== 0) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getHeight();\n  if (f !== 0) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = message.getFormat();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = message.getMetadataMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(6, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);\n  }\n  f = message.getIsLastFrame();\n  if (f) {\n    writer.writeBool(\n      7,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      8,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      9,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      10,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bytes image = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * optional bytes image = 2;\n * This is a type-conversion wrapper around `getImage()`\n * @return {string}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getImage_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getImage()));\n};\n\n\n/**\n * optional bytes image = 2;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getImage()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getImage_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getImage()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setImage = function(value) {\n  return jspb.Message.setProto3BytesField(this, 2, value);\n};\n\n\n/**\n * optional int64 width = 3;\n * @return {number}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getWidth = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setWidth = function(value) {\n  return jspb.Message.setProto3IntField(this, 3, value);\n};\n\n\n/**\n * optional int64 height = 4;\n * @return {number}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getHeight = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setHeight = function(value) {\n  return jspb.Message.setProto3IntField(this, 4, value);\n};\n\n\n/**\n * optional string format = 5;\n * @return {string}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getFormat = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setFormat = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n/**\n * map<string, string> metadata = 6;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,string>}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getMetadataMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,string>} */ (\n      jspb.Message.getMapField(this, 6, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.clearMetadataMap = function() {\n  this.getMetadataMap().clear();\n  return this;};\n\n\n/**\n * optional bool is_last_frame = 7;\n * @return {boolean}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getIsLastFrame = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setIsLastFrame = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 7, value);\n};\n\n\n/**\n * optional float timestamp = 8;\n * @return {number}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 8, value);\n};\n\n\n/**\n * optional string id = 9;\n * @return {string}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 9, value);\n};\n\n\n/**\n * optional int64 index = 10;\n * @return {number}\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 10, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AssistanceImageFrame} returns this\n */\nproto.ubo.v1.AssistanceImageFrame.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 10, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantReportAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantReportAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantReportAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantReportAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sourceId: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    data: (f = msg.getData()) && proto.ubo.v1.AcceptableAssistanceFrame.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantReportAction}\n */\nproto.ubo.v1.AssistantReportAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantReportAction;\n  return proto.ubo.v1.AssistantReportAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantReportAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantReportAction}\n */\nproto.ubo.v1.AssistantReportAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSourceId(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AcceptableAssistanceFrame;\n      reader.readMessage(value,proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader);\n      msg.setData(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantReportAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantReportAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantReportAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantReportAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSourceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getData();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantReportAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantReportAction} returns this\n */\nproto.ubo.v1.AssistantReportAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantReportAction} returns this\n */\nproto.ubo.v1.AssistantReportAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantReportAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string source_id = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantReportAction.prototype.getSourceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantReportAction} returns this\n */\nproto.ubo.v1.AssistantReportAction.prototype.setSourceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional AcceptableAssistanceFrame data = 3;\n * @return {?proto.ubo.v1.AcceptableAssistanceFrame}\n */\nproto.ubo.v1.AssistantReportAction.prototype.getData = function() {\n  return /** @type{?proto.ubo.v1.AcceptableAssistanceFrame} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AcceptableAssistanceFrame, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AcceptableAssistanceFrame|undefined} value\n * @return {!proto.ubo.v1.AssistantReportAction} returns this\n*/\nproto.ubo.v1.AssistantReportAction.prototype.setData = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistantReportAction} returns this\n */\nproto.ubo.v1.AssistantReportAction.prototype.clearData = function() {\n  return this.setData(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantReportAction.prototype.hasData = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantStartListeningAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantStartListeningAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantStartListeningAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantStartListeningAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantStartListeningAction}\n */\nproto.ubo.v1.AssistantStartListeningAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantStartListeningAction;\n  return proto.ubo.v1.AssistantStartListeningAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantStartListeningAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantStartListeningAction}\n */\nproto.ubo.v1.AssistantStartListeningAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantStartListeningAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantStartListeningAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantStartListeningAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantStartListeningAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantStartListeningAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantStartListeningAction} returns this\n */\nproto.ubo.v1.AssistantStartListeningAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantStartListeningAction} returns this\n */\nproto.ubo.v1.AssistantStartListeningAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantStartListeningAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantStopListeningAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantStopListeningAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantStopListeningAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantStopListeningAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantStopListeningAction}\n */\nproto.ubo.v1.AssistantStopListeningAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantStopListeningAction;\n  return proto.ubo.v1.AssistantStopListeningAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantStopListeningAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantStopListeningAction}\n */\nproto.ubo.v1.AssistantStopListeningAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantStopListeningAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantStopListeningAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantStopListeningAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantStopListeningAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantStopListeningAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantStopListeningAction} returns this\n */\nproto.ubo.v1.AssistantStopListeningAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantStopListeningAction} returns this\n */\nproto.ubo.v1.AssistantStopListeningAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantStopListeningAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantToggleListeningAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantToggleListeningAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantToggleListeningAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantToggleListeningAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantToggleListeningAction}\n */\nproto.ubo.v1.AssistantToggleListeningAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantToggleListeningAction;\n  return proto.ubo.v1.AssistantToggleListeningAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantToggleListeningAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantToggleListeningAction}\n */\nproto.ubo.v1.AssistantToggleListeningAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantToggleListeningAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantToggleListeningAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantToggleListeningAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantToggleListeningAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantToggleListeningAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantToggleListeningAction} returns this\n */\nproto.ubo.v1.AssistantToggleListeningAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantToggleListeningAction} returns this\n */\nproto.ubo.v1.AssistantToggleListeningAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantToggleListeningAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantUpdateProvidersAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantUpdateProvidersAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantUpdateProvidersAction}\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantUpdateProvidersAction;\n  return proto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantUpdateProvidersAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantUpdateProvidersAction}\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantUpdateProvidersAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantUpdateProvidersAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantUpdateProvidersAction} returns this\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantUpdateProvidersAction} returns this\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantUpdateProvidersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantAddMcpServerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantAddMcpServerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantAddMcpServerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    type: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    config: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantAddMcpServerAction;\n  return proto.ubo.v1.AssistantAddMcpServerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantAddMcpServerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.McpServerType} */ (reader.readEnum());\n      msg.setType(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setConfig(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantAddMcpServerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantAddMcpServerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantAddMcpServerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getType();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = message.getConfig();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string name = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional McpServerType type = 3;\n * @return {!proto.ubo.v1.McpServerType}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.getType = function() {\n  return /** @type {!proto.ubo.v1.McpServerType} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.McpServerType} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.setType = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n/**\n * optional string config = 4;\n * @return {string}\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.getConfig = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerAction.prototype.setConfig = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantToggleMcpServerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantToggleMcpServerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serverId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantToggleMcpServerAction}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantToggleMcpServerAction;\n  return proto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantToggleMcpServerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantToggleMcpServerAction}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServerId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantToggleMcpServerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantToggleMcpServerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServerId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantToggleMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantToggleMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string server_id = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.getServerId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantToggleMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantToggleMcpServerAction.prototype.setServerId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantDeleteMcpServerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantDeleteMcpServerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serverId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantDeleteMcpServerAction;\n  return proto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantDeleteMcpServerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServerId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantDeleteMcpServerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantDeleteMcpServerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServerId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string server_id = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.getServerId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction} returns this\n */\nproto.ubo.v1.AssistantDeleteMcpServerAction.prototype.setServerId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantSyncMcpServersAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantSyncMcpServersAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantSyncMcpServersAction}\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantSyncMcpServersAction;\n  return proto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantSyncMcpServersAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantSyncMcpServersAction}\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantSyncMcpServersAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantSyncMcpServersAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantSyncMcpServersAction} returns this\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantSyncMcpServersAction} returns this\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantSyncMcpServersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantEvent}\n */\nproto.ubo.v1.AssistantEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantEvent;\n  return proto.ubo.v1.AssistantEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantEvent}\n */\nproto.ubo.v1.AssistantEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantEvent} returns this\n */\nproto.ubo.v1.AssistantEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantEvent} returns this\n */\nproto.ubo.v1.AssistantEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantDownloadOllamaModelEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    model: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantDownloadOllamaModelEvent;\n  return proto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setModel(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantDownloadOllamaModelEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getModel();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} returns this\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} returns this\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string model = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.getModel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} returns this\n */\nproto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.setModel = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantHandleReportEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantHandleReportEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantHandleReportEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sourceId: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    data: (f = msg.getData()) && proto.ubo.v1.AcceptableAssistanceFrame.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantHandleReportEvent}\n */\nproto.ubo.v1.AssistantHandleReportEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantHandleReportEvent;\n  return proto.ubo.v1.AssistantHandleReportEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantHandleReportEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantHandleReportEvent}\n */\nproto.ubo.v1.AssistantHandleReportEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSourceId(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AcceptableAssistanceFrame;\n      reader.readMessage(value,proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader);\n      msg.setData(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantHandleReportEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantHandleReportEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantHandleReportEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSourceId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getData();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string source_id = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.getSourceId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.setSourceId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional AcceptableAssistanceFrame data = 3;\n * @return {?proto.ubo.v1.AcceptableAssistanceFrame}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.getData = function() {\n  return /** @type{?proto.ubo.v1.AcceptableAssistanceFrame} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AcceptableAssistanceFrame, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AcceptableAssistanceFrame|undefined} value\n * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this\n*/\nproto.ubo.v1.AssistantHandleReportEvent.prototype.setData = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.clearData = function() {\n  return this.setData(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantHandleReportEvent.prototype.hasData = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantUpdateProvidersEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantUpdateProvidersEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent}\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantUpdateProvidersEvent;\n  return proto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantUpdateProvidersEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent}\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantUpdateProvidersEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantUpdateProvidersEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent} returns this\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent} returns this\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantUpdateProvidersEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantAddMcpServerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantAddMcpServerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    type: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    config: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantAddMcpServerEvent;\n  return proto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantAddMcpServerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.McpServerType} */ (reader.readEnum());\n      msg.setType(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setConfig(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantAddMcpServerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantAddMcpServerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getType();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = message.getConfig();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string name = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional McpServerType type = 3;\n * @return {!proto.ubo.v1.McpServerType}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.getType = function() {\n  return /** @type {!proto.ubo.v1.McpServerType} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.McpServerType} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.setType = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n/**\n * optional string config = 4;\n * @return {string}\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.getConfig = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantAddMcpServerEvent.prototype.setConfig = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantDeleteMcpServerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantDeleteMcpServerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    serverId: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantDeleteMcpServerEvent;\n  return proto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantDeleteMcpServerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setServerId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantDeleteMcpServerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantDeleteMcpServerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getServerId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string server_id = 2;\n * @return {string}\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.getServerId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent} returns this\n */\nproto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.setServerId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isListening: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isMicrophoneMute: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    selectedStt: jspb.Message.getFieldWithDefault(msg, 5, 0),\n    selectedLlm: jspb.Message.getFieldWithDefault(msg, 6, 0),\n    selectedModels: (f = msg.getSelectedModels()) && proto.ubo.v1.AssistantState.SelectedModelsDict.toObject(includeInstance, f),\n    selectedTts: jspb.Message.getFieldWithDefault(msg, 8, 0),\n    selectedImageGenerator: jspb.Message.getFieldWithDefault(msg, 9, 0),\n    mcpServers: (f = msg.getMcpServers()) && proto.ubo.v1.AssistantState.McpServersDict.toObject(includeInstance, f),\n    enabledMcpServers: (f = msg.getEnabledMcpServers()) && proto.ubo.v1.AssistantState.EnabledMcpServers.toObject(includeInstance, f),\n    enabledMcpServersWithMetadataJson: jspb.Message.getFieldWithDefault(msg, 12, \"\"),\n    providerSetupStatus: (f = msg.getProviderSetupStatus()) && proto.ubo.v1.AssistantState.ProviderSetupStatusDict.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantState}\n */\nproto.ubo.v1.AssistantState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantState;\n  return proto.ubo.v1.AssistantState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantState}\n */\nproto.ubo.v1.AssistantState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsListening(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsMicrophoneMute(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.ubo.v1.AssistantSTTName} */ (reader.readEnum());\n      msg.setSelectedStt(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.ubo.v1.AssistantLLMName} */ (reader.readEnum());\n      msg.setSelectedLlm(value);\n      break;\n    case 7:\n      var value = new proto.ubo.v1.AssistantState.SelectedModelsDict;\n      reader.readMessage(value,proto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinaryFromReader);\n      msg.setSelectedModels(value);\n      break;\n    case 8:\n      var value = /** @type {!proto.ubo.v1.AssistantTTSName} */ (reader.readEnum());\n      msg.setSelectedTts(value);\n      break;\n    case 9:\n      var value = /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (reader.readEnum());\n      msg.setSelectedImageGenerator(value);\n      break;\n    case 10:\n      var value = new proto.ubo.v1.AssistantState.McpServersDict;\n      reader.readMessage(value,proto.ubo.v1.AssistantState.McpServersDict.deserializeBinaryFromReader);\n      msg.setMcpServers(value);\n      break;\n    case 11:\n      var value = new proto.ubo.v1.AssistantState.EnabledMcpServers;\n      reader.readMessage(value,proto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinaryFromReader);\n      msg.setEnabledMcpServers(value);\n      break;\n    case 12:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setEnabledMcpServersWithMetadataJson(value);\n      break;\n    case 13:\n      var value = new proto.ubo.v1.AssistantState.ProviderSetupStatusDict;\n      reader.readMessage(value,proto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinaryFromReader);\n      msg.setProviderSetupStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.AssistantSTTName} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.AssistantLLMName} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n  f = message.getSelectedModels();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.ubo.v1.AssistantState.SelectedModelsDict.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {!proto.ubo.v1.AssistantTTSName} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeEnum(\n      8,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeEnum(\n      9,\n      f\n    );\n  }\n  f = message.getMcpServers();\n  if (f != null) {\n    writer.writeMessage(\n      10,\n      f,\n      proto.ubo.v1.AssistantState.McpServersDict.serializeBinaryToWriter\n    );\n  }\n  f = message.getEnabledMcpServers();\n  if (f != null) {\n    writer.writeMessage(\n      11,\n      f,\n      proto.ubo.v1.AssistantState.EnabledMcpServers.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 12));\n  if (f != null) {\n    writer.writeString(\n      12,\n      f\n    );\n  }\n  f = message.getProviderSetupStatus();\n  if (f != null) {\n    writer.writeMessage(\n      13,\n      f,\n      proto.ubo.v1.AssistantState.ProviderSetupStatusDict.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantState.SelectedModelsDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantState.SelectedModelsDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantState.SelectedModelsDict}\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantState.SelectedModelsDict;\n  return proto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantState.SelectedModelsDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantState.SelectedModelsDict}\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, \"\", \"\");\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantState.SelectedModelsDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantState.SelectedModelsDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);\n  }\n};\n\n\n/**\n * map<string, string> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,string>}\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,string>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.AssistantState.SelectedModelsDict} returns this\n */\nproto.ubo.v1.AssistantState.SelectedModelsDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantState.McpServersDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantState.McpServersDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantState.McpServersDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.McpServersDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.McpServerMetadata.toObject) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantState.McpServersDict}\n */\nproto.ubo.v1.AssistantState.McpServersDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantState.McpServersDict;\n  return proto.ubo.v1.AssistantState.McpServersDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantState.McpServersDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantState.McpServersDict}\n */\nproto.ubo.v1.AssistantState.McpServersDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.McpServerMetadata.deserializeBinaryFromReader, \"\", new proto.ubo.v1.McpServerMetadata());\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantState.McpServersDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantState.McpServersDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantState.McpServersDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.McpServersDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.McpServerMetadata.serializeBinaryToWriter);\n  }\n};\n\n\n/**\n * map<string, McpServerMetadata> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.McpServerMetadata>}\n */\nproto.ubo.v1.AssistantState.McpServersDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.McpServerMetadata>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      proto.ubo.v1.McpServerMetadata));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.AssistantState.McpServersDict} returns this\n */\nproto.ubo.v1.AssistantState.McpServersDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantState.EnabledMcpServers.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantState.EnabledMcpServers} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers}\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantState.EnabledMcpServers;\n  return proto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantState.EnabledMcpServers} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers}\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantState.EnabledMcpServers.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantState.EnabledMcpServers} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers} returns this\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers} returns this\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers} returns this\n */\nproto.ubo.v1.AssistantState.EnabledMcpServers.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AssistantState.ProviderSetupStatusDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict}\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AssistantState.ProviderSetupStatusDict;\n  return proto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict}\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readBool, null, \"\", false);\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AssistantState.ProviderSetupStatusDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeBool);\n  }\n};\n\n\n/**\n * map<string, bool> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,boolean>}\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,boolean>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} returns this\n */\nproto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;\n * @return {string}\n */\nproto.ubo.v1.AssistantState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_listening = 2;\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.getIsListening = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setIsListening = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearIsListening = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasIsListening = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_microphone_mute = 3;\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.getIsMicrophoneMute = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setIsMicrophoneMute = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearIsMicrophoneMute = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasIsMicrophoneMute = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool is_active = 4;\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional AssistantSTTName selected_stt = 5;\n * @return {!proto.ubo.v1.AssistantSTTName}\n */\nproto.ubo.v1.AssistantState.prototype.getSelectedStt = function() {\n  return /** @type {!proto.ubo.v1.AssistantSTTName} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantSTTName} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setSelectedStt = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearSelectedStt = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasSelectedStt = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional AssistantLLMName selected_llm = 6;\n * @return {!proto.ubo.v1.AssistantLLMName}\n */\nproto.ubo.v1.AssistantState.prototype.getSelectedLlm = function() {\n  return /** @type {!proto.ubo.v1.AssistantLLMName} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantLLMName} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setSelectedLlm = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearSelectedLlm = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasSelectedLlm = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional SelectedModelsDict selected_models = 7;\n * @return {?proto.ubo.v1.AssistantState.SelectedModelsDict}\n */\nproto.ubo.v1.AssistantState.prototype.getSelectedModels = function() {\n  return /** @type{?proto.ubo.v1.AssistantState.SelectedModelsDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.SelectedModelsDict, 7));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantState.SelectedModelsDict|undefined} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n*/\nproto.ubo.v1.AssistantState.prototype.setSelectedModels = function(value) {\n  return jspb.Message.setWrapperField(this, 7, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearSelectedModels = function() {\n  return this.setSelectedModels(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasSelectedModels = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional AssistantTTSName selected_tts = 8;\n * @return {!proto.ubo.v1.AssistantTTSName}\n */\nproto.ubo.v1.AssistantState.prototype.getSelectedTts = function() {\n  return /** @type {!proto.ubo.v1.AssistantTTSName} */ (jspb.Message.getFieldWithDefault(this, 8, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantTTSName} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setSelectedTts = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearSelectedTts = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasSelectedTts = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional AssistantImageGeneratorName selected_image_generator = 9;\n * @return {!proto.ubo.v1.AssistantImageGeneratorName}\n */\nproto.ubo.v1.AssistantState.prototype.getSelectedImageGenerator = function() {\n  return /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (jspb.Message.getFieldWithDefault(this, 9, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AssistantImageGeneratorName} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setSelectedImageGenerator = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearSelectedImageGenerator = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasSelectedImageGenerator = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional McpServersDict mcp_servers = 10;\n * @return {?proto.ubo.v1.AssistantState.McpServersDict}\n */\nproto.ubo.v1.AssistantState.prototype.getMcpServers = function() {\n  return /** @type{?proto.ubo.v1.AssistantState.McpServersDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.McpServersDict, 10));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantState.McpServersDict|undefined} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n*/\nproto.ubo.v1.AssistantState.prototype.setMcpServers = function(value) {\n  return jspb.Message.setWrapperField(this, 10, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearMcpServers = function() {\n  return this.setMcpServers(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasMcpServers = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional EnabledMcpServers enabled_mcp_servers = 11;\n * @return {?proto.ubo.v1.AssistantState.EnabledMcpServers}\n */\nproto.ubo.v1.AssistantState.prototype.getEnabledMcpServers = function() {\n  return /** @type{?proto.ubo.v1.AssistantState.EnabledMcpServers} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.EnabledMcpServers, 11));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantState.EnabledMcpServers|undefined} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n*/\nproto.ubo.v1.AssistantState.prototype.setEnabledMcpServers = function(value) {\n  return jspb.Message.setWrapperField(this, 11, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearEnabledMcpServers = function() {\n  return this.setEnabledMcpServers(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasEnabledMcpServers = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional string enabled_mcp_servers_with_metadata_json = 12;\n * @return {string}\n */\nproto.ubo.v1.AssistantState.prototype.getEnabledMcpServersWithMetadataJson = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.setEnabledMcpServersWithMetadataJson = function(value) {\n  return jspb.Message.setField(this, 12, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearEnabledMcpServersWithMetadataJson = function() {\n  return jspb.Message.setField(this, 12, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasEnabledMcpServersWithMetadataJson = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional ProviderSetupStatusDict provider_setup_status = 13;\n * @return {?proto.ubo.v1.AssistantState.ProviderSetupStatusDict}\n */\nproto.ubo.v1.AssistantState.prototype.getProviderSetupStatus = function() {\n  return /** @type{?proto.ubo.v1.AssistantState.ProviderSetupStatusDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.ProviderSetupStatusDict, 13));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantState.ProviderSetupStatusDict|undefined} value\n * @return {!proto.ubo.v1.AssistantState} returns this\n*/\nproto.ubo.v1.AssistantState.prototype.setProviderSetupStatus = function(value) {\n  return jspb.Message.setWrapperField(this, 13, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AssistantState} returns this\n */\nproto.ubo.v1.AssistantState.prototype.clearProviderSetupStatus = function() {\n  return this.setProviderSetupStatus(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AssistantState.prototype.hasProviderSetupStatus = function() {\n  return jspb.Message.getField(this, 13) != null;\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_ = [[1,2,3]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase = {\n  ACCEPTABLE_ASSISTANCE_FRAME_NOT_SET: 0,\n  ASSISTANCE_AUDIO_FRAME: 1,\n  ASSISTANCE_IMAGE_FRAME: 2,\n  ASSISTANCE_TEXT_FRAME: 3\n};\n\n/**\n * @return {proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.getAcceptableAssistanceFrameCase = function() {\n  return /** @type {proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AcceptableAssistanceFrame.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AcceptableAssistanceFrame} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AcceptableAssistanceFrame.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    assistanceAudioFrame: (f = msg.getAssistanceAudioFrame()) && proto.ubo.v1.AssistanceAudioFrame.toObject(includeInstance, f),\n    assistanceImageFrame: (f = msg.getAssistanceImageFrame()) && proto.ubo.v1.AssistanceImageFrame.toObject(includeInstance, f),\n    assistanceTextFrame: (f = msg.getAssistanceTextFrame()) && proto.ubo.v1.AssistanceTextFrame.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AcceptableAssistanceFrame;\n  return proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AcceptableAssistanceFrame} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.AssistanceAudioFrame;\n      reader.readMessage(value,proto.ubo.v1.AssistanceAudioFrame.deserializeBinaryFromReader);\n      msg.setAssistanceAudioFrame(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AssistanceImageFrame;\n      reader.readMessage(value,proto.ubo.v1.AssistanceImageFrame.deserializeBinaryFromReader);\n      msg.setAssistanceImageFrame(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AssistanceTextFrame;\n      reader.readMessage(value,proto.ubo.v1.AssistanceTextFrame.deserializeBinaryFromReader);\n      msg.setAssistanceTextFrame(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AcceptableAssistanceFrame} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getAssistanceAudioFrame();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.AssistanceAudioFrame.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistanceImageFrame();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AssistanceImageFrame.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistanceTextFrame();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AssistanceTextFrame.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional AssistanceAudioFrame assistance_audio_frame = 1;\n * @return {?proto.ubo.v1.AssistanceAudioFrame}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.getAssistanceAudioFrame = function() {\n  return /** @type{?proto.ubo.v1.AssistanceAudioFrame} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistanceAudioFrame, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistanceAudioFrame|undefined} value\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this\n*/\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.setAssistanceAudioFrame = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.clearAssistanceAudioFrame = function() {\n  return this.setAssistanceAudioFrame(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.hasAssistanceAudioFrame = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional AssistanceImageFrame assistance_image_frame = 2;\n * @return {?proto.ubo.v1.AssistanceImageFrame}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.getAssistanceImageFrame = function() {\n  return /** @type{?proto.ubo.v1.AssistanceImageFrame} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistanceImageFrame, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistanceImageFrame|undefined} value\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this\n*/\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.setAssistanceImageFrame = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.clearAssistanceImageFrame = function() {\n  return this.setAssistanceImageFrame(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.hasAssistanceImageFrame = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional AssistanceTextFrame assistance_text_frame = 3;\n * @return {?proto.ubo.v1.AssistanceTextFrame}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.getAssistanceTextFrame = function() {\n  return /** @type{?proto.ubo.v1.AssistanceTextFrame} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistanceTextFrame, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistanceTextFrame|undefined} value\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this\n*/\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.setAssistanceTextFrame = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.clearAssistanceTextFrame = function() {\n  return this.setAssistanceTextFrame(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AcceptableAssistanceFrame.prototype.hasAssistanceTextFrame = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioAction}\n */\nproto.ubo.v1.AudioAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioAction;\n  return proto.ubo.v1.AudioAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioAction}\n */\nproto.ubo.v1.AudioAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioAction} returns this\n */\nproto.ubo.v1.AudioAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioAction} returns this\n */\nproto.ubo.v1.AudioAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioInstallDriverAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioInstallDriverAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioInstallDriverAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioInstallDriverAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioInstallDriverAction}\n */\nproto.ubo.v1.AudioInstallDriverAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioInstallDriverAction;\n  return proto.ubo.v1.AudioInstallDriverAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioInstallDriverAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioInstallDriverAction}\n */\nproto.ubo.v1.AudioInstallDriverAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioInstallDriverAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioInstallDriverAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioInstallDriverAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioInstallDriverAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioInstallDriverAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioInstallDriverAction} returns this\n */\nproto.ubo.v1.AudioInstallDriverAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioInstallDriverAction} returns this\n */\nproto.ubo.v1.AudioInstallDriverAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioInstallDriverAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioSetVolumeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioSetVolumeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioSetVolumeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    volume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    device: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioSetVolumeAction}\n */\nproto.ubo.v1.AudioSetVolumeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioSetVolumeAction;\n  return proto.ubo.v1.AudioSetVolumeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioSetVolumeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioSetVolumeAction}\n */\nproto.ubo.v1.AudioSetVolumeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setVolume(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());\n      msg.setDevice(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioSetVolumeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioSetVolumeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioSetVolumeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getVolume();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getDevice();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float volume = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.getVolume = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.setVolume = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional AudioDevice device = 3;\n * @return {!proto.ubo.v1.AudioDevice}\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.getDevice = function() {\n  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AudioDevice} value\n * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this\n */\nproto.ubo.v1.AudioSetVolumeAction.prototype.setDevice = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioChangeVolumeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioChangeVolumeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioChangeVolumeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    amount: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    device: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioChangeVolumeAction}\n */\nproto.ubo.v1.AudioChangeVolumeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioChangeVolumeAction;\n  return proto.ubo.v1.AudioChangeVolumeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioChangeVolumeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioChangeVolumeAction}\n */\nproto.ubo.v1.AudioChangeVolumeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setAmount(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());\n      msg.setDevice(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioChangeVolumeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioChangeVolumeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioChangeVolumeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getAmount();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getDevice();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float amount = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.getAmount = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.setAmount = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional AudioDevice device = 3;\n * @return {!proto.ubo.v1.AudioDevice}\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.getDevice = function() {\n  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AudioDevice} value\n * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this\n */\nproto.ubo.v1.AudioChangeVolumeAction.prototype.setDevice = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioSetMuteStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioSetMuteStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioSetMuteStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isMute: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    device: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioSetMuteStatusAction}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioSetMuteStatusAction;\n  return proto.ubo.v1.AudioSetMuteStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioSetMuteStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioSetMuteStatusAction}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsMute(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());\n      msg.setDevice(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioSetMuteStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioSetMuteStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioSetMuteStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsMute();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getDevice();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_mute = 2;\n * @return {boolean}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.getIsMute = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.setIsMute = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional AudioDevice device = 3;\n * @return {!proto.ubo.v1.AudioDevice}\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.getDevice = function() {\n  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AudioDevice} value\n * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioSetMuteStatusAction.prototype.setDevice = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioToggleMuteStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioToggleMuteStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    device: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioToggleMuteStatusAction}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioToggleMuteStatusAction;\n  return proto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioToggleMuteStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioToggleMuteStatusAction}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());\n      msg.setDevice(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioToggleMuteStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioToggleMuteStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDevice();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioToggleMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioToggleMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AudioDevice device = 2;\n * @return {!proto.ubo.v1.AudioDevice}\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.getDevice = function() {\n  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.AudioDevice} value\n * @return {!proto.ubo.v1.AudioToggleMuteStatusAction} returns this\n */\nproto.ubo.v1.AudioToggleMuteStatusAction.prototype.setDevice = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayChimeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayChimeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayChimeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayChimeAction}\n */\nproto.ubo.v1.AudioPlayChimeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayChimeAction;\n  return proto.ubo.v1.AudioPlayChimeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayChimeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayChimeAction}\n */\nproto.ubo.v1.AudioPlayChimeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayChimeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayChimeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayChimeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayChimeAction} returns this\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayChimeAction} returns this\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string name = 2;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayChimeAction} returns this\n */\nproto.ubo.v1.AudioPlayChimeAction.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayAudioSampleAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayAudioSampleAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleAction}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayAudioSampleAction;\n  return proto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayAudioSampleAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleAction}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setSample(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayAudioSampleAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayAudioSampleAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSample();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AudioSample sample = 2;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.getSample = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this\n*/\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.setSample = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.clearSample = function() {\n  return this.setSample(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSampleAction.prototype.hasSample = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayAudioSequenceAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayAudioSequenceAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f),\n    id: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 4, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayAudioSequenceAction;\n  return proto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayAudioSequenceAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setSample(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayAudioSequenceAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayAudioSequenceAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSample();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional AudioSample sample = 2;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getSample = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this\n*/\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setSample = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.clearSample = function() {\n  return this.setSample(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.hasSample = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string id = 3;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional int64 index = 4;\n * @return {number}\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioSample.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioSample.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioSample} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioSample.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    data: msg.getData_asB64(),\n    channels: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    rate: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    width: jspb.Message.getFieldWithDefault(msg, 5, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioSample.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioSample;\n  return proto.ubo.v1.AudioSample.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioSample} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioSample.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setData(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setChannels(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setRate(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWidth(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioSample.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioSample.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioSample} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioSample.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getData_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      2,\n      f\n    );\n  }\n  f = message.getChannels();\n  if (f !== 0) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getRate();\n  if (f !== 0) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = message.getWidth();\n  if (f !== 0) {\n    writer.writeInt64(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioSample.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioSample} returns this\n */\nproto.ubo.v1.AudioSample.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioSample} returns this\n */\nproto.ubo.v1.AudioSample.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioSample.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bytes data = 2;\n * @return {string}\n */\nproto.ubo.v1.AudioSample.prototype.getData = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * optional bytes data = 2;\n * This is a type-conversion wrapper around `getData()`\n * @return {string}\n */\nproto.ubo.v1.AudioSample.prototype.getData_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getData()));\n};\n\n\n/**\n * optional bytes data = 2;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getData()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioSample.prototype.getData_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getData()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.AudioSample} returns this\n */\nproto.ubo.v1.AudioSample.prototype.setData = function(value) {\n  return jspb.Message.setProto3BytesField(this, 2, value);\n};\n\n\n/**\n * optional int64 channels = 3;\n * @return {number}\n */\nproto.ubo.v1.AudioSample.prototype.getChannels = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioSample} returns this\n */\nproto.ubo.v1.AudioSample.prototype.setChannels = function(value) {\n  return jspb.Message.setProto3IntField(this, 3, value);\n};\n\n\n/**\n * optional int64 rate = 4;\n * @return {number}\n */\nproto.ubo.v1.AudioSample.prototype.getRate = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioSample} returns this\n */\nproto.ubo.v1.AudioSample.prototype.setRate = function(value) {\n  return jspb.Message.setProto3IntField(this, 4, value);\n};\n\n\n/**\n * optional int64 width = 5;\n * @return {number}\n */\nproto.ubo.v1.AudioSample.prototype.getWidth = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioSample} returns this\n */\nproto.ubo.v1.AudioSample.prototype.setWidth = function(value) {\n  return jspb.Message.setProto3IntField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioReportSampleAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioReportSampleAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioReportSampleAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    sampleSpeechRecognition: msg.getSampleSpeechRecognition_asB64(),\n    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioReportSampleAction}\n */\nproto.ubo.v1.AudioReportSampleAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioReportSampleAction;\n  return proto.ubo.v1.AudioReportSampleAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioReportSampleAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioReportSampleAction}\n */\nproto.ubo.v1.AudioReportSampleAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setSampleSpeechRecognition(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setSample(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioReportSampleAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioReportSampleAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioReportSampleAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getSampleSpeechRecognition_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      3,\n      f\n    );\n  }\n  f = message.getSample();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioReportSampleAction} returns this\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioReportSampleAction} returns this\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioReportSampleAction} returns this\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional bytes sample_speech_recognition = 3;\n * @return {string}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.getSampleSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * optional bytes sample_speech_recognition = 3;\n * This is a type-conversion wrapper around `getSampleSpeechRecognition()`\n * @return {string}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.getSampleSpeechRecognition_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getSampleSpeechRecognition()));\n};\n\n\n/**\n * optional bytes sample_speech_recognition = 3;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getSampleSpeechRecognition()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.getSampleSpeechRecognition_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getSampleSpeechRecognition()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.AudioReportSampleAction} returns this\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.setSampleSpeechRecognition = function(value) {\n  return jspb.Message.setProto3BytesField(this, 3, value);\n};\n\n\n/**\n * optional AudioSample sample = 4;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.getSample = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioReportSampleAction} returns this\n*/\nproto.ubo.v1.AudioReportSampleAction.prototype.setSample = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioReportSampleAction} returns this\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.clearSample = function() {\n  return this.setSample(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioReportSampleAction.prototype.hasSample = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlaybackDoneAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlaybackDoneAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlaybackDoneAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlaybackDoneAction}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlaybackDoneAction;\n  return proto.ubo.v1.AudioPlaybackDoneAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlaybackDoneAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlaybackDoneAction}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlaybackDoneAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlaybackDoneAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlaybackDoneAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlaybackDoneAction} returns this\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlaybackDoneAction} returns this\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlaybackDoneAction} returns this\n */\nproto.ubo.v1.AudioPlaybackDoneAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioStartRecordingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioStartRecordingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioStartRecordingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioStartRecordingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioStartRecordingAction}\n */\nproto.ubo.v1.AudioStartRecordingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioStartRecordingAction;\n  return proto.ubo.v1.AudioStartRecordingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioStartRecordingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioStartRecordingAction}\n */\nproto.ubo.v1.AudioStartRecordingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioStartRecordingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioStartRecordingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioStartRecordingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioStartRecordingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioStartRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioStartRecordingAction} returns this\n */\nproto.ubo.v1.AudioStartRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioStartRecordingAction} returns this\n */\nproto.ubo.v1.AudioStartRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioStartRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioStopRecordingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioStopRecordingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioStopRecordingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioStopRecordingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioStopRecordingAction}\n */\nproto.ubo.v1.AudioStopRecordingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioStopRecordingAction;\n  return proto.ubo.v1.AudioStopRecordingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioStopRecordingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioStopRecordingAction}\n */\nproto.ubo.v1.AudioStopRecordingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioStopRecordingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioStopRecordingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioStopRecordingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioStopRecordingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioStopRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioStopRecordingAction} returns this\n */\nproto.ubo.v1.AudioStopRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioStopRecordingAction} returns this\n */\nproto.ubo.v1.AudioStopRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioStopRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioToggleRecordingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioToggleRecordingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioToggleRecordingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioToggleRecordingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioToggleRecordingAction}\n */\nproto.ubo.v1.AudioToggleRecordingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioToggleRecordingAction;\n  return proto.ubo.v1.AudioToggleRecordingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioToggleRecordingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioToggleRecordingAction}\n */\nproto.ubo.v1.AudioToggleRecordingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioToggleRecordingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioToggleRecordingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioToggleRecordingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioToggleRecordingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioToggleRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioToggleRecordingAction} returns this\n */\nproto.ubo.v1.AudioToggleRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioToggleRecordingAction} returns this\n */\nproto.ubo.v1.AudioToggleRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioToggleRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayRecordingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayRecordingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayRecordingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayRecordingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayRecordingAction}\n */\nproto.ubo.v1.AudioPlayRecordingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayRecordingAction;\n  return proto.ubo.v1.AudioPlayRecordingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayRecordingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayRecordingAction}\n */\nproto.ubo.v1.AudioPlayRecordingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayRecordingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayRecordingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayRecordingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayRecordingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayRecordingAction} returns this\n */\nproto.ubo.v1.AudioPlayRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayRecordingAction} returns this\n */\nproto.ubo.v1.AudioPlayRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioEvent}\n */\nproto.ubo.v1.AudioEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioEvent;\n  return proto.ubo.v1.AudioEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioEvent}\n */\nproto.ubo.v1.AudioEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioEvent} returns this\n */\nproto.ubo.v1.AudioEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioEvent} returns this\n */\nproto.ubo.v1.AudioEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioReportSampleEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioReportSampleEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioReportSampleEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    sampleSpeechRecognition: msg.getSampleSpeechRecognition_asB64(),\n    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioReportSampleEvent}\n */\nproto.ubo.v1.AudioReportSampleEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioReportSampleEvent;\n  return proto.ubo.v1.AudioReportSampleEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioReportSampleEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioReportSampleEvent}\n */\nproto.ubo.v1.AudioReportSampleEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setSampleSpeechRecognition(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setSample(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioReportSampleEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioReportSampleEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioReportSampleEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getSampleSpeechRecognition_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      3,\n      f\n    );\n  }\n  f = message.getSample();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional bytes sample_speech_recognition = 3;\n * @return {string}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.getSampleSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * optional bytes sample_speech_recognition = 3;\n * This is a type-conversion wrapper around `getSampleSpeechRecognition()`\n * @return {string}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.getSampleSpeechRecognition_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getSampleSpeechRecognition()));\n};\n\n\n/**\n * optional bytes sample_speech_recognition = 3;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getSampleSpeechRecognition()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.getSampleSpeechRecognition_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getSampleSpeechRecognition()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.setSampleSpeechRecognition = function(value) {\n  return jspb.Message.setProto3BytesField(this, 3, value);\n};\n\n\n/**\n * optional AudioSample sample = 4;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.getSample = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this\n*/\nproto.ubo.v1.AudioReportSampleEvent.prototype.setSample = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.clearSample = function() {\n  return this.setSample(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioReportSampleEvent.prototype.hasSample = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioInstallDriverEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioInstallDriverEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioInstallDriverEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioInstallDriverEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioInstallDriverEvent}\n */\nproto.ubo.v1.AudioInstallDriverEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioInstallDriverEvent;\n  return proto.ubo.v1.AudioInstallDriverEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioInstallDriverEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioInstallDriverEvent}\n */\nproto.ubo.v1.AudioInstallDriverEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioInstallDriverEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioInstallDriverEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioInstallDriverEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioInstallDriverEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioInstallDriverEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioInstallDriverEvent} returns this\n */\nproto.ubo.v1.AudioInstallDriverEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioInstallDriverEvent} returns this\n */\nproto.ubo.v1.AudioInstallDriverEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioInstallDriverEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayChimeEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayChimeEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayChimeEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayChimeEvent}\n */\nproto.ubo.v1.AudioPlayChimeEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayChimeEvent;\n  return proto.ubo.v1.AudioPlayChimeEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayChimeEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayChimeEvent}\n */\nproto.ubo.v1.AudioPlayChimeEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayChimeEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayChimeEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayChimeEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayChimeEvent} returns this\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayChimeEvent} returns this\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string name = 2;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayChimeEvent} returns this\n */\nproto.ubo.v1.AudioPlayChimeEvent.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayAudioSampleEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayAudioSampleEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    volume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayAudioSampleEvent;\n  return proto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayAudioSampleEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setVolume(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setSample(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayAudioSampleEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayAudioSampleEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getVolume();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getSample();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float volume = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.getVolume = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.setVolume = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional AudioSample sample = 3;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.getSample = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this\n*/\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.setSample = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.clearSample = function() {\n  return this.setSample(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSampleEvent.prototype.hasSample = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlayAudioSequenceEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlayAudioSequenceEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    volume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f),\n    id: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 5, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlayAudioSequenceEvent;\n  return proto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlayAudioSequenceEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setVolume(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setSample(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlayAudioSequenceEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlayAudioSequenceEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getVolume();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getSample();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float volume = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getVolume = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setVolume = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional AudioSample sample = 3;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getSample = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n*/\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setSample = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.clearSample = function() {\n  return this.setSample(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.hasSample = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string id = 4;\n * @return {string}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n/**\n * optional int64 index = 5;\n * @return {number}\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this\n */\nproto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioPlaybackDoneEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioPlaybackDoneEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioPlaybackDoneEvent}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioPlaybackDoneEvent;\n  return proto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioPlaybackDoneEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioPlaybackDoneEvent}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioPlaybackDoneEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioPlaybackDoneEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlaybackDoneEvent} returns this\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioPlaybackDoneEvent} returns this\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioPlaybackDoneEvent} returns this\n */\nproto.ubo.v1.AudioPlaybackDoneEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.AudioState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.AudioState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.AudioState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    playbackVolume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    isPlaybackMute: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    captureVolume: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),\n    isCaptureMute: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    isRecording: jspb.Message.getBooleanFieldWithDefault(msg, 6, false),\n    recording: (f = msg.getRecording()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.AudioState}\n */\nproto.ubo.v1.AudioState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.AudioState;\n  return proto.ubo.v1.AudioState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.AudioState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.AudioState}\n */\nproto.ubo.v1.AudioState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setPlaybackVolume(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsPlaybackMute(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setCaptureVolume(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsCaptureMute(value);\n      break;\n    case 6:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsRecording(value);\n      break;\n    case 7:\n      var value = new proto.ubo.v1.AudioSample;\n      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);\n      msg.setRecording(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.AudioState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.AudioState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.AudioState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.AudioState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeFloat(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeBool(\n      6,\n      f\n    );\n  }\n  f = message.getRecording();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.ubo.v1.AudioSample.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;\n * @return {string}\n */\nproto.ubo.v1.AudioState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float playback_volume = 2;\n * @return {number}\n */\nproto.ubo.v1.AudioState.prototype.getPlaybackVolume = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.setPlaybackVolume = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearPlaybackVolume = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasPlaybackVolume = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_playback_mute = 3;\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.getIsPlaybackMute = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.setIsPlaybackMute = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearIsPlaybackMute = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasIsPlaybackMute = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional float capture_volume = 4;\n * @return {number}\n */\nproto.ubo.v1.AudioState.prototype.getCaptureVolume = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.setCaptureVolume = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearCaptureVolume = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasCaptureVolume = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_capture_mute = 5;\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.getIsCaptureMute = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.setIsCaptureMute = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearIsCaptureMute = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasIsCaptureMute = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional bool is_recording = 6;\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.getIsRecording = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.setIsRecording = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearIsRecording = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasIsRecording = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional AudioSample recording = 7;\n * @return {?proto.ubo.v1.AudioSample}\n */\nproto.ubo.v1.AudioState.prototype.getRecording = function() {\n  return /** @type{?proto.ubo.v1.AudioSample} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 7));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSample|undefined} value\n * @return {!proto.ubo.v1.AudioState} returns this\n*/\nproto.ubo.v1.AudioState.prototype.setRecording = function(value) {\n  return jspb.Message.setWrapperField(this, 7, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.AudioState} returns this\n */\nproto.ubo.v1.AudioState.prototype.clearRecording = function() {\n  return this.setRecording(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.AudioState.prototype.hasRecording = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraAction}\n */\nproto.ubo.v1.CameraAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraAction;\n  return proto.ubo.v1.CameraAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraAction}\n */\nproto.ubo.v1.CameraAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraAction} returns this\n */\nproto.ubo.v1.CameraAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraAction} returns this\n */\nproto.ubo.v1.CameraAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraStartViewfinderAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraStartViewfinderAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraStartViewfinderAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    pattern: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraStartViewfinderAction}\n */\nproto.ubo.v1.CameraStartViewfinderAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraStartViewfinderAction;\n  return proto.ubo.v1.CameraStartViewfinderAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraStartViewfinderAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraStartViewfinderAction}\n */\nproto.ubo.v1.CameraStartViewfinderAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPattern(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraStartViewfinderAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraStartViewfinderAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraStartViewfinderAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string pattern = 2;\n * @return {string}\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.getPattern = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.setPattern = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.clearPattern = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraStartViewfinderAction.prototype.hasPattern = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.CameraReportBarcodeAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraReportBarcodeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraReportBarcodeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraReportBarcodeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    codesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraReportBarcodeAction}\n */\nproto.ubo.v1.CameraReportBarcodeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraReportBarcodeAction;\n  return proto.ubo.v1.CameraReportBarcodeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraReportBarcodeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraReportBarcodeAction}\n */\nproto.ubo.v1.CameraReportBarcodeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addCodes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraReportBarcodeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraReportBarcodeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraReportBarcodeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getCodesList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string codes = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.getCodesList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.setCodesList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.addCodes = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this\n */\nproto.ubo.v1.CameraReportBarcodeAction.prototype.clearCodesList = function() {\n  return this.setCodesList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraInstallDriverAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraInstallDriverAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraInstallDriverAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    make: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    model: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    variant: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraInstallDriverAction}\n */\nproto.ubo.v1.CameraInstallDriverAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraInstallDriverAction;\n  return proto.ubo.v1.CameraInstallDriverAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraInstallDriverAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraInstallDriverAction}\n */\nproto.ubo.v1.CameraInstallDriverAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMake(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setModel(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setVariant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraInstallDriverAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraInstallDriverAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraInstallDriverAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getMake();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getModel();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getVariant();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string make = 2;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.getMake = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.setMake = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string model = 3;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.getModel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.setModel = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional string variant = 4;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.getVariant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this\n */\nproto.ubo.v1.CameraInstallDriverAction.prototype.setVariant = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraEvent}\n */\nproto.ubo.v1.CameraEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraEvent;\n  return proto.ubo.v1.CameraEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraEvent}\n */\nproto.ubo.v1.CameraEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraEvent} returns this\n */\nproto.ubo.v1.CameraEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraEvent} returns this\n */\nproto.ubo.v1.CameraEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraInstallDriverEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraInstallDriverEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraInstallDriverEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    make: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    model: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    variant: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraInstallDriverEvent}\n */\nproto.ubo.v1.CameraInstallDriverEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraInstallDriverEvent;\n  return proto.ubo.v1.CameraInstallDriverEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraInstallDriverEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraInstallDriverEvent}\n */\nproto.ubo.v1.CameraInstallDriverEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMake(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setModel(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setVariant(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraInstallDriverEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraInstallDriverEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraInstallDriverEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getMake();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getModel();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getVariant();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string make = 2;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.getMake = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.setMake = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string model = 3;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.getModel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.setModel = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional string variant = 4;\n * @return {string}\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.getVariant = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this\n */\nproto.ubo.v1.CameraInstallDriverEvent.prototype.setVariant = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraRestoreDefaultAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraRestoreDefaultAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraRestoreDefaultAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraRestoreDefaultAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraRestoreDefaultAction}\n */\nproto.ubo.v1.CameraRestoreDefaultAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraRestoreDefaultAction;\n  return proto.ubo.v1.CameraRestoreDefaultAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraRestoreDefaultAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraRestoreDefaultAction}\n */\nproto.ubo.v1.CameraRestoreDefaultAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraRestoreDefaultAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraRestoreDefaultAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraRestoreDefaultAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraRestoreDefaultAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraRestoreDefaultAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraRestoreDefaultAction} returns this\n */\nproto.ubo.v1.CameraRestoreDefaultAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraRestoreDefaultAction} returns this\n */\nproto.ubo.v1.CameraRestoreDefaultAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraRestoreDefaultAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraRestoreDefaultEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraRestoreDefaultEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraRestoreDefaultEvent}\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraRestoreDefaultEvent;\n  return proto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraRestoreDefaultEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraRestoreDefaultEvent}\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraRestoreDefaultEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraRestoreDefaultEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraRestoreDefaultEvent} returns this\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraRestoreDefaultEvent} returns this\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraRestoreDefaultEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraStartViewfinderEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraStartViewfinderEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraStartViewfinderEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    pattern: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraStartViewfinderEvent}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraStartViewfinderEvent;\n  return proto.ubo.v1.CameraStartViewfinderEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraStartViewfinderEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraStartViewfinderEvent}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPattern(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraStartViewfinderEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraStartViewfinderEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraStartViewfinderEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string pattern = 2;\n * @return {string}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.getPattern = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.setPattern = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.clearPattern = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraStartViewfinderEvent.prototype.hasPattern = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraReportImageEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraReportImageEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraReportImageEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    data: msg.getData_asB64(),\n    width: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    height: jspb.Message.getFieldWithDefault(msg, 5, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraReportImageEvent}\n */\nproto.ubo.v1.CameraReportImageEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraReportImageEvent;\n  return proto.ubo.v1.CameraReportImageEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraReportImageEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraReportImageEvent}\n */\nproto.ubo.v1.CameraReportImageEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setData(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWidth(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setHeight(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraReportImageEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraReportImageEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraReportImageEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getData_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      3,\n      f\n    );\n  }\n  f = message.getWidth();\n  if (f !== 0) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = message.getHeight();\n  if (f !== 0) {\n    writer.writeInt64(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraReportImageEvent} returns this\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraReportImageEvent} returns this\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.CameraReportImageEvent} returns this\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional bytes data = 3;\n * @return {string}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getData = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * optional bytes data = 3;\n * This is a type-conversion wrapper around `getData()`\n * @return {string}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getData_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getData()));\n};\n\n\n/**\n * optional bytes data = 3;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getData()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getData_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getData()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.CameraReportImageEvent} returns this\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.setData = function(value) {\n  return jspb.Message.setProto3BytesField(this, 3, value);\n};\n\n\n/**\n * optional int64 width = 4;\n * @return {number}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getWidth = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.CameraReportImageEvent} returns this\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.setWidth = function(value) {\n  return jspb.Message.setProto3IntField(this, 4, value);\n};\n\n\n/**\n * optional int64 height = 5;\n * @return {number}\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.getHeight = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.CameraReportImageEvent} returns this\n */\nproto.ubo.v1.CameraReportImageEvent.prototype.setHeight = function(value) {\n  return jspb.Message.setProto3IntField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraStopViewfinderEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraStopViewfinderEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraStopViewfinderEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraStopViewfinderEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraStopViewfinderEvent}\n */\nproto.ubo.v1.CameraStopViewfinderEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraStopViewfinderEvent;\n  return proto.ubo.v1.CameraStopViewfinderEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraStopViewfinderEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraStopViewfinderEvent}\n */\nproto.ubo.v1.CameraStopViewfinderEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraStopViewfinderEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraStopViewfinderEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraStopViewfinderEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraStopViewfinderEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraStopViewfinderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraStopViewfinderEvent} returns this\n */\nproto.ubo.v1.CameraStopViewfinderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraStopViewfinderEvent} returns this\n */\nproto.ubo.v1.CameraStopViewfinderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraStopViewfinderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraSetIndexAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraSetIndexAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraSetIndexAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    index: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraSetIndexAction}\n */\nproto.ubo.v1.CameraSetIndexAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraSetIndexAction;\n  return proto.ubo.v1.CameraSetIndexAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraSetIndexAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraSetIndexAction}\n */\nproto.ubo.v1.CameraSetIndexAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraSetIndexAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraSetIndexAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraSetIndexAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIndex();\n  if (f !== 0) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraSetIndexAction} returns this\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraSetIndexAction} returns this\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 index = 2;\n * @return {number}\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.CameraSetIndexAction} returns this\n */\nproto.ubo.v1.CameraSetIndexAction.prototype.setIndex = function(value) {\n  return jspb.Message.setProto3IntField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraDetectAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraDetectAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraDetectAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraDetectAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraDetectAction}\n */\nproto.ubo.v1.CameraDetectAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraDetectAction;\n  return proto.ubo.v1.CameraDetectAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraDetectAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraDetectAction}\n */\nproto.ubo.v1.CameraDetectAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraDetectAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraDetectAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraDetectAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraDetectAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraDetectAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraDetectAction} returns this\n */\nproto.ubo.v1.CameraDetectAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraDetectAction} returns this\n */\nproto.ubo.v1.CameraDetectAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraDetectAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraSetAvailableCamerasAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraSetAvailableCamerasAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    availableCamerasList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraSetAvailableCamerasAction;\n  return proto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraSetAvailableCamerasAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addAvailableCameras(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraSetAvailableCamerasAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraSetAvailableCamerasAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getAvailableCamerasList();\n  if (f.length > 0) {\n    writer.writePackedInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated int64 available_cameras = 2;\n * @return {!Array<number>}\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.getAvailableCamerasList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.setAvailableCamerasList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.addAvailableCameras = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this\n */\nproto.ubo.v1.CameraSetAvailableCamerasAction.prototype.clearAvailableCamerasList = function() {\n  return this.setAvailableCamerasList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraDetectEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraDetectEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraDetectEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraDetectEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraDetectEvent}\n */\nproto.ubo.v1.CameraDetectEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraDetectEvent;\n  return proto.ubo.v1.CameraDetectEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraDetectEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraDetectEvent}\n */\nproto.ubo.v1.CameraDetectEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraDetectEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraDetectEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraDetectEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraDetectEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraDetectEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraDetectEvent} returns this\n */\nproto.ubo.v1.CameraDetectEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraDetectEvent} returns this\n */\nproto.ubo.v1.CameraDetectEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraDetectEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.CameraDetectedEvent.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraDetectedEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraDetectedEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraDetectedEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    availableCamerasList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraDetectedEvent}\n */\nproto.ubo.v1.CameraDetectedEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraDetectedEvent;\n  return proto.ubo.v1.CameraDetectedEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraDetectedEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraDetectedEvent}\n */\nproto.ubo.v1.CameraDetectedEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addAvailableCameras(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraDetectedEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraDetectedEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraDetectedEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getAvailableCamerasList();\n  if (f.length > 0) {\n    writer.writePackedInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraDetectedEvent} returns this\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraDetectedEvent} returns this\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated int64 available_cameras = 2;\n * @return {!Array<number>}\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.getAvailableCamerasList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.ubo.v1.CameraDetectedEvent} returns this\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.setAvailableCamerasList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.CameraDetectedEvent} returns this\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.addAvailableCameras = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.CameraDetectedEvent} returns this\n */\nproto.ubo.v1.CameraDetectedEvent.prototype.clearAvailableCamerasList = function() {\n  return this.setAvailableCamerasList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraReinitializeEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraReinitializeEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraReinitializeEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraReinitializeEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraReinitializeEvent}\n */\nproto.ubo.v1.CameraReinitializeEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraReinitializeEvent;\n  return proto.ubo.v1.CameraReinitializeEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraReinitializeEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraReinitializeEvent}\n */\nproto.ubo.v1.CameraReinitializeEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraReinitializeEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraReinitializeEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraReinitializeEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraReinitializeEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraReinitializeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraReinitializeEvent} returns this\n */\nproto.ubo.v1.CameraReinitializeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraReinitializeEvent} returns this\n */\nproto.ubo.v1.CameraReinitializeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraReinitializeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.CameraState.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    queueList: jspb.Message.toObjectList(msg.getQueueList(),\n    proto.ubo.v1.QRCodeInputDescription.toObject, includeInstance),\n    selectedCameraIndex: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    availableCameras: (f = msg.getAvailableCameras()) && proto.ubo.v1.CameraState.AvailableCameras.toObject(includeInstance, f),\n    cameraType: jspb.Message.getFieldWithDefault(msg, 5, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraState}\n */\nproto.ubo.v1.CameraState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraState;\n  return proto.ubo.v1.CameraState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraState}\n */\nproto.ubo.v1.CameraState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.QRCodeInputDescription;\n      reader.readMessage(value,proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader);\n      msg.addQueue(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setSelectedCameraIndex(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.CameraState.AvailableCameras;\n      reader.readMessage(value,proto.ubo.v1.CameraState.AvailableCameras.deserializeBinaryFromReader);\n      msg.setAvailableCameras(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.ubo.v1.CameraType} */ (reader.readEnum());\n      msg.setCameraType(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getQueueList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getAvailableCameras();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.CameraState.AvailableCameras.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {!proto.ubo.v1.CameraType} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.CameraState.AvailableCameras.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.CameraState.AvailableCameras.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.CameraState.AvailableCameras.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.CameraState.AvailableCameras} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraState.AvailableCameras.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.CameraState.AvailableCameras}\n */\nproto.ubo.v1.CameraState.AvailableCameras.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.CameraState.AvailableCameras;\n  return proto.ubo.v1.CameraState.AvailableCameras.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.CameraState.AvailableCameras} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.CameraState.AvailableCameras}\n */\nproto.ubo.v1.CameraState.AvailableCameras.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.CameraState.AvailableCameras.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.CameraState.AvailableCameras.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.CameraState.AvailableCameras} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.CameraState.AvailableCameras.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedInt64(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated int64 items = 1;\n * @return {!Array<number>}\n */\nproto.ubo.v1.CameraState.AvailableCameras.prototype.getItemsList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.ubo.v1.CameraState.AvailableCameras} returns this\n */\nproto.ubo.v1.CameraState.AvailableCameras.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.CameraState.AvailableCameras} returns this\n */\nproto.ubo.v1.CameraState.AvailableCameras.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.CameraState.AvailableCameras} returns this\n */\nproto.ubo.v1.CameraState.AvailableCameras.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;\n * @return {string}\n */\nproto.ubo.v1.CameraState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated QRCodeInputDescription queue = 2;\n * @return {!Array<!proto.ubo.v1.QRCodeInputDescription>}\n */\nproto.ubo.v1.CameraState.prototype.getQueueList = function() {\n  return /** @type{!Array<!proto.ubo.v1.QRCodeInputDescription>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.QRCodeInputDescription, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.QRCodeInputDescription>} value\n * @return {!proto.ubo.v1.CameraState} returns this\n*/\nproto.ubo.v1.CameraState.prototype.setQueueList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.QRCodeInputDescription=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.QRCodeInputDescription}\n */\nproto.ubo.v1.CameraState.prototype.addQueue = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.QRCodeInputDescription, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.clearQueueList = function() {\n  return this.setQueueList([]);\n};\n\n\n/**\n * optional int64 selected_camera_index = 3;\n * @return {number}\n */\nproto.ubo.v1.CameraState.prototype.getSelectedCameraIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.setSelectedCameraIndex = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.clearSelectedCameraIndex = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraState.prototype.hasSelectedCameraIndex = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional AvailableCameras available_cameras = 4;\n * @return {?proto.ubo.v1.CameraState.AvailableCameras}\n */\nproto.ubo.v1.CameraState.prototype.getAvailableCameras = function() {\n  return /** @type{?proto.ubo.v1.CameraState.AvailableCameras} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraState.AvailableCameras, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraState.AvailableCameras|undefined} value\n * @return {!proto.ubo.v1.CameraState} returns this\n*/\nproto.ubo.v1.CameraState.prototype.setAvailableCameras = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.clearAvailableCameras = function() {\n  return this.setAvailableCameras(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraState.prototype.hasAvailableCameras = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional CameraType camera_type = 5;\n * @return {!proto.ubo.v1.CameraType}\n */\nproto.ubo.v1.CameraState.prototype.getCameraType = function() {\n  return /** @type {!proto.ubo.v1.CameraType} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.CameraType} value\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.setCameraType = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.CameraState} returns this\n */\nproto.ubo.v1.CameraState.prototype.clearCameraType = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.CameraState.prototype.hasCameraType = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayAction}\n */\nproto.ubo.v1.DisplayAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayAction;\n  return proto.ubo.v1.DisplayAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayAction}\n */\nproto.ubo.v1.DisplayAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayAction} returns this\n */\nproto.ubo.v1.DisplayAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayAction} returns this\n */\nproto.ubo.v1.DisplayAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayEvent}\n */\nproto.ubo.v1.DisplayEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayEvent;\n  return proto.ubo.v1.DisplayEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayEvent}\n */\nproto.ubo.v1.DisplayEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayEvent} returns this\n */\nproto.ubo.v1.DisplayEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayEvent} returns this\n */\nproto.ubo.v1.DisplayEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayPauseAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayPauseAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayPauseAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayPauseAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayPauseAction}\n */\nproto.ubo.v1.DisplayPauseAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayPauseAction;\n  return proto.ubo.v1.DisplayPauseAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayPauseAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayPauseAction}\n */\nproto.ubo.v1.DisplayPauseAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayPauseAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayPauseAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayPauseAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayPauseAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayPauseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayPauseAction} returns this\n */\nproto.ubo.v1.DisplayPauseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayPauseAction} returns this\n */\nproto.ubo.v1.DisplayPauseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayPauseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayResumeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayResumeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayResumeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayResumeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayResumeAction}\n */\nproto.ubo.v1.DisplayResumeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayResumeAction;\n  return proto.ubo.v1.DisplayResumeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayResumeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayResumeAction}\n */\nproto.ubo.v1.DisplayResumeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayResumeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayResumeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayResumeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayResumeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayResumeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayResumeAction} returns this\n */\nproto.ubo.v1.DisplayResumeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayResumeAction} returns this\n */\nproto.ubo.v1.DisplayResumeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayResumeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayRedrawAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayRedrawAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayRedrawAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayRedrawAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayRedrawAction}\n */\nproto.ubo.v1.DisplayRedrawAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayRedrawAction;\n  return proto.ubo.v1.DisplayRedrawAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayRedrawAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayRedrawAction}\n */\nproto.ubo.v1.DisplayRedrawAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayRedrawAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayRedrawAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayRedrawAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayRedrawAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayRedrawAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayRedrawAction} returns this\n */\nproto.ubo.v1.DisplayRedrawAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayRedrawAction} returns this\n */\nproto.ubo.v1.DisplayRedrawAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayRedrawAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayBlankAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayBlankAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayBlankAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayBlankAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayBlankAction}\n */\nproto.ubo.v1.DisplayBlankAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayBlankAction;\n  return proto.ubo.v1.DisplayBlankAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayBlankAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayBlankAction}\n */\nproto.ubo.v1.DisplayBlankAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayBlankAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayBlankAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayBlankAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayBlankAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayBlankAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayBlankAction} returns this\n */\nproto.ubo.v1.DisplayBlankAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayBlankAction} returns this\n */\nproto.ubo.v1.DisplayBlankAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayBlankAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayUnblankAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayUnblankAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayUnblankAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayUnblankAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayUnblankAction}\n */\nproto.ubo.v1.DisplayUnblankAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayUnblankAction;\n  return proto.ubo.v1.DisplayUnblankAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayUnblankAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayUnblankAction}\n */\nproto.ubo.v1.DisplayUnblankAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayUnblankAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayUnblankAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayUnblankAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayUnblankAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayUnblankAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayUnblankAction} returns this\n */\nproto.ubo.v1.DisplayUnblankAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayUnblankAction} returns this\n */\nproto.ubo.v1.DisplayUnblankAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayUnblankAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayUpdateActivityAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayUpdateActivityAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayUpdateActivityAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayUpdateActivityAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayUpdateActivityAction}\n */\nproto.ubo.v1.DisplayUpdateActivityAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayUpdateActivityAction;\n  return proto.ubo.v1.DisplayUpdateActivityAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayUpdateActivityAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayUpdateActivityAction}\n */\nproto.ubo.v1.DisplayUpdateActivityAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayUpdateActivityAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayUpdateActivityAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayUpdateActivityAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayUpdateActivityAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayUpdateActivityAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayUpdateActivityAction} returns this\n */\nproto.ubo.v1.DisplayUpdateActivityAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayUpdateActivityAction} returns this\n */\nproto.ubo.v1.DisplayUpdateActivityAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayUpdateActivityAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplaySetBlankTimeoutAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplaySetBlankTimeoutAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timeout: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplaySetBlankTimeoutAction;\n  return proto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplaySetBlankTimeoutAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (reader.readEnum());\n      msg.setTimeout(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplaySetBlankTimeoutAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplaySetBlankTimeoutAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimeout();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction} returns this\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction} returns this\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional DisplayBlankTimeout timeout = 2;\n * @return {!proto.ubo.v1.DisplayBlankTimeout}\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.getTimeout = function() {\n  return /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.DisplayBlankTimeout} value\n * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction} returns this\n */\nproto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.setTimeout = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayRedrawEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayRedrawEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayRedrawEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayRedrawEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayRedrawEvent}\n */\nproto.ubo.v1.DisplayRedrawEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayRedrawEvent;\n  return proto.ubo.v1.DisplayRedrawEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayRedrawEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayRedrawEvent}\n */\nproto.ubo.v1.DisplayRedrawEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayRedrawEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayRedrawEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayRedrawEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayRedrawEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayRedrawEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayRedrawEvent} returns this\n */\nproto.ubo.v1.DisplayRedrawEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayRedrawEvent} returns this\n */\nproto.ubo.v1.DisplayRedrawEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayRedrawEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayBlankEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayBlankEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayBlankEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayBlankEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayBlankEvent}\n */\nproto.ubo.v1.DisplayBlankEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayBlankEvent;\n  return proto.ubo.v1.DisplayBlankEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayBlankEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayBlankEvent}\n */\nproto.ubo.v1.DisplayBlankEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayBlankEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayBlankEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayBlankEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayBlankEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayBlankEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayBlankEvent} returns this\n */\nproto.ubo.v1.DisplayBlankEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayBlankEvent} returns this\n */\nproto.ubo.v1.DisplayBlankEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayBlankEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayUnblankEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayUnblankEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayUnblankEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayUnblankEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayUnblankEvent}\n */\nproto.ubo.v1.DisplayUnblankEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayUnblankEvent;\n  return proto.ubo.v1.DisplayUnblankEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayUnblankEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayUnblankEvent}\n */\nproto.ubo.v1.DisplayUnblankEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayUnblankEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayUnblankEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayUnblankEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayUnblankEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayUnblankEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayUnblankEvent} returns this\n */\nproto.ubo.v1.DisplayUnblankEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayUnblankEvent} returns this\n */\nproto.ubo.v1.DisplayUnblankEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayUnblankEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.DisplayRenderEvent.repeatedFields_ = [4];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayRenderEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayRenderEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayRenderEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    data: msg.getData_asB64(),\n    rectangleList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f,\n    density: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayRenderEvent}\n */\nproto.ubo.v1.DisplayRenderEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayRenderEvent;\n  return proto.ubo.v1.DisplayRenderEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayRenderEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayRenderEvent}\n */\nproto.ubo.v1.DisplayRenderEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setData(value);\n      break;\n    case 4:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addRectangle(values[i]);\n      }\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setDensity(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayRenderEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayRenderEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayRenderEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getData_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      3,\n      f\n    );\n  }\n  f = message.getRectangleList();\n  if (f.length > 0) {\n    writer.writePackedInt64(\n      4,\n      f\n    );\n  }\n  f = message.getDensity();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional bytes data = 3;\n * @return {string}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getData = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * optional bytes data = 3;\n * This is a type-conversion wrapper around `getData()`\n * @return {string}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getData_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getData()));\n};\n\n\n/**\n * optional bytes data = 3;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getData()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getData_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getData()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.setData = function(value) {\n  return jspb.Message.setProto3BytesField(this, 3, value);\n};\n\n\n/**\n * repeated int64 rectangle = 4;\n * @return {!Array<number>}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getRectangleList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 4));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.setRectangleList = function(value) {\n  return jspb.Message.setField(this, 4, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.addRectangle = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.clearRectangleList = function() {\n  return this.setRectangleList([]);\n};\n\n\n/**\n * optional float density = 5;\n * @return {number}\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.getDensity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.DisplayRenderEvent} returns this\n */\nproto.ubo.v1.DisplayRenderEvent.prototype.setDensity = function(value) {\n  return jspb.Message.setProto3FloatField(this, 5, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.repeatedFields_ = [4];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayCompressedRenderEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayCompressedRenderEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    compressedData: msg.getCompressedData_asB64(),\n    rectangleList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f,\n    density: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayCompressedRenderEvent;\n  return proto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayCompressedRenderEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setCompressedData(value);\n      break;\n    case 4:\n      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addRectangle(values[i]);\n      }\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setDensity(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayCompressedRenderEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayCompressedRenderEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getCompressedData_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      3,\n      f\n    );\n  }\n  f = message.getRectangleList();\n  if (f.length > 0) {\n    writer.writePackedInt64(\n      4,\n      f\n    );\n  }\n  f = message.getDensity();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional bytes compressed_data = 3;\n * @return {string}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getCompressedData = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * optional bytes compressed_data = 3;\n * This is a type-conversion wrapper around `getCompressedData()`\n * @return {string}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getCompressedData_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getCompressedData()));\n};\n\n\n/**\n * optional bytes compressed_data = 3;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getCompressedData()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getCompressedData_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getCompressedData()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.setCompressedData = function(value) {\n  return jspb.Message.setProto3BytesField(this, 3, value);\n};\n\n\n/**\n * repeated int64 rectangle = 4;\n * @return {!Array<number>}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getRectangleList = function() {\n  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 4));\n};\n\n\n/**\n * @param {!Array<number>} value\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.setRectangleList = function(value) {\n  return jspb.Message.setField(this, 4, value || []);\n};\n\n\n/**\n * @param {number} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.addRectangle = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.clearRectangleList = function() {\n  return this.setRectangleList([]);\n};\n\n\n/**\n * optional float density = 5;\n * @return {number}\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.getDensity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this\n */\nproto.ubo.v1.DisplayCompressedRenderEvent.prototype.setDensity = function(value) {\n  return jspb.Message.setProto3FloatField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DisplayState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DisplayState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DisplayState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isPaused: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isBlanked: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    lastActivityTime: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),\n    selectedBlankTimeout: jspb.Message.getFieldWithDefault(msg, 5, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DisplayState}\n */\nproto.ubo.v1.DisplayState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DisplayState;\n  return proto.ubo.v1.DisplayState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DisplayState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DisplayState}\n */\nproto.ubo.v1.DisplayState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsPaused(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsBlanked(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setLastActivityTime(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (reader.readEnum());\n      msg.setSelectedBlankTimeout(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DisplayState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DisplayState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DisplayState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DisplayState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeFloat(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;\n * @return {string}\n */\nproto.ubo.v1.DisplayState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_paused = 2;\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.getIsPaused = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.setIsPaused = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.clearIsPaused = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.hasIsPaused = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_blanked = 3;\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.getIsBlanked = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.setIsBlanked = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.clearIsBlanked = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.hasIsBlanked = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional float last_activity_time = 4;\n * @return {number}\n */\nproto.ubo.v1.DisplayState.prototype.getLastActivityTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.setLastActivityTime = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.clearLastActivityTime = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.hasLastActivityTime = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional DisplayBlankTimeout selected_blank_timeout = 5;\n * @return {!proto.ubo.v1.DisplayBlankTimeout}\n */\nproto.ubo.v1.DisplayState.prototype.getSelectedBlankTimeout = function() {\n  return /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.DisplayBlankTimeout} value\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.setSelectedBlankTimeout = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DisplayState} returns this\n */\nproto.ubo.v1.DisplayState.prototype.clearSelectedBlankTimeout = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DisplayState.prototype.hasSelectedBlankTimeout = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerAction}\n */\nproto.ubo.v1.DockerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerAction;\n  return proto.ubo.v1.DockerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerAction}\n */\nproto.ubo.v1.DockerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerAction} returns this\n */\nproto.ubo.v1.DockerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerAction} returns this\n */\nproto.ubo.v1.DockerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerInstallAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerInstallAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerInstallAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerInstallAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerInstallAction}\n */\nproto.ubo.v1.DockerInstallAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerInstallAction;\n  return proto.ubo.v1.DockerInstallAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerInstallAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerInstallAction}\n */\nproto.ubo.v1.DockerInstallAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerInstallAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerInstallAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerInstallAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerInstallAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerInstallAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerInstallAction} returns this\n */\nproto.ubo.v1.DockerInstallAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerInstallAction} returns this\n */\nproto.ubo.v1.DockerInstallAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerInstallAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerStartAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerStartAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerStartAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStartAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerStartAction}\n */\nproto.ubo.v1.DockerStartAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerStartAction;\n  return proto.ubo.v1.DockerStartAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerStartAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerStartAction}\n */\nproto.ubo.v1.DockerStartAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerStartAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerStartAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerStartAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStartAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerStartAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStartAction} returns this\n */\nproto.ubo.v1.DockerStartAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerStartAction} returns this\n */\nproto.ubo.v1.DockerStartAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerStartAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerStopAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerStopAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerStopAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStopAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerStopAction}\n */\nproto.ubo.v1.DockerStopAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerStopAction;\n  return proto.ubo.v1.DockerStopAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerStopAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerStopAction}\n */\nproto.ubo.v1.DockerStopAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerStopAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerStopAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerStopAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStopAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerStopAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStopAction} returns this\n */\nproto.ubo.v1.DockerStopAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerStopAction} returns this\n */\nproto.ubo.v1.DockerStopAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerStopAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerSetStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerSetStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerSetStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    status: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerSetStatusAction}\n */\nproto.ubo.v1.DockerSetStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerSetStatusAction;\n  return proto.ubo.v1.DockerSetStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerSetStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerSetStatusAction}\n */\nproto.ubo.v1.DockerSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.DockerStatus} */ (reader.readEnum());\n      msg.setStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerSetStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerSetStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerSetStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getStatus();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerSetStatusAction} returns this\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerSetStatusAction} returns this\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional DockerStatus status = 2;\n * @return {!proto.ubo.v1.DockerStatus}\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.getStatus = function() {\n  return /** @type {!proto.ubo.v1.DockerStatus} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.DockerStatus} value\n * @return {!proto.ubo.v1.DockerSetStatusAction} returns this\n */\nproto.ubo.v1.DockerSetStatusAction.prototype.setStatus = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerStoreUsernameAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerStoreUsernameAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStoreUsernameAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    registry: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    username: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerStoreUsernameAction}\n */\nproto.ubo.v1.DockerStoreUsernameAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerStoreUsernameAction;\n  return proto.ubo.v1.DockerStoreUsernameAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerStoreUsernameAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerStoreUsernameAction}\n */\nproto.ubo.v1.DockerStoreUsernameAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setRegistry(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setUsername(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerStoreUsernameAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerStoreUsernameAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStoreUsernameAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getRegistry();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getUsername();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string registry = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.getRegistry = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.setRegistry = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string username = 3;\n * @return {string}\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.getUsername = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this\n */\nproto.ubo.v1.DockerStoreUsernameAction.prototype.setUsername = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerRemoveUsernameAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerRemoveUsernameAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerRemoveUsernameAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    registry: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerRemoveUsernameAction}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerRemoveUsernameAction;\n  return proto.ubo.v1.DockerRemoveUsernameAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerRemoveUsernameAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerRemoveUsernameAction}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setRegistry(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerRemoveUsernameAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerRemoveUsernameAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerRemoveUsernameAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getRegistry();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerRemoveUsernameAction} returns this\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerRemoveUsernameAction} returns this\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string registry = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.getRegistry = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerRemoveUsernameAction} returns this\n */\nproto.ubo.v1.DockerRemoveUsernameAction.prototype.setRegistry = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageAction}\n */\nproto.ubo.v1.DockerImageAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageAction;\n  return proto.ubo.v1.DockerImageAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageAction}\n */\nproto.ubo.v1.DockerImageAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageAction} returns this\n */\nproto.ubo.v1.DockerImageAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageAction} returns this\n */\nproto.ubo.v1.DockerImageAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageAction} returns this\n */\nproto.ubo.v1.DockerImageAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageSetStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageSetStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageSetStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    status: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    ports: (f = msg.getPorts()) && proto.ubo.v1.DockerImageSetStatusAction.Ports.toObject(includeInstance, f),\n    ip: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 5, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction}\n */\nproto.ubo.v1.DockerImageSetStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageSetStatusAction;\n  return proto.ubo.v1.DockerImageSetStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageSetStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction}\n */\nproto.ubo.v1.DockerImageSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.DockerItemStatus} */ (reader.readEnum());\n      msg.setStatus(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.DockerImageSetStatusAction.Ports;\n      reader.readMessage(value,proto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinaryFromReader);\n      msg.setPorts(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIp(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageSetStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageSetStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageSetStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getStatus();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getPorts();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.DockerImageSetStatusAction.Ports.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageSetStatusAction.Ports.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageSetStatusAction.Ports} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports}\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageSetStatusAction.Ports;\n  return proto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageSetStatusAction.Ports} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports}\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageSetStatusAction.Ports.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageSetStatusAction.Ports} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional DockerItemStatus status = 2;\n * @return {!proto.ubo.v1.DockerItemStatus}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.getStatus = function() {\n  return /** @type {!proto.ubo.v1.DockerItemStatus} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.DockerItemStatus} value\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.setStatus = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional Ports ports = 3;\n * @return {?proto.ubo.v1.DockerImageSetStatusAction.Ports}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.getPorts = function() {\n  return /** @type{?proto.ubo.v1.DockerImageSetStatusAction.Ports} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageSetStatusAction.Ports, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageSetStatusAction.Ports|undefined} value\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n*/\nproto.ubo.v1.DockerImageSetStatusAction.prototype.setPorts = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.clearPorts = function() {\n  return this.setPorts(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.hasPorts = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string ip = 4;\n * @return {string}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.getIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.setIp = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.clearIp = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.hasIp = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string image = 5;\n * @return {string}\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this\n */\nproto.ubo.v1.DockerImageSetStatusAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 5, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageSetDockerIdAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageSetDockerIdAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    dockerId: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageSetDockerIdAction}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageSetDockerIdAction;\n  return proto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageSetDockerIdAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageSetDockerIdAction}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDockerId(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageSetDockerIdAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageSetDockerIdAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDockerId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string docker_id = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.getDockerId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.setDockerId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string image = 3;\n * @return {string}\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this\n */\nproto.ubo.v1.DockerImageSetDockerIdAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageUpdateMetadataAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageUpdateMetadataAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    instructions: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageUpdateMetadataAction;\n  return proto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageUpdateMetadataAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setInstructions(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageUpdateMetadataAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageUpdateMetadataAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string instructions = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.getInstructions = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.setInstructions = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.clearInstructions = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.hasInstructions = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string image = 3;\n * @return {string}\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this\n */\nproto.ubo.v1.DockerImageUpdateMetadataAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageFetchAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageFetchAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageFetchAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageFetchAction}\n */\nproto.ubo.v1.DockerImageFetchAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageFetchAction;\n  return proto.ubo.v1.DockerImageFetchAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageFetchAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageFetchAction}\n */\nproto.ubo.v1.DockerImageFetchAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageFetchAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageFetchAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageFetchAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageFetchAction} returns this\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageFetchAction} returns this\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageFetchAction} returns this\n */\nproto.ubo.v1.DockerImageFetchAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRemoveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRemoveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRemoveAction}\n */\nproto.ubo.v1.DockerImageRemoveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRemoveAction;\n  return proto.ubo.v1.DockerImageRemoveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRemoveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRemoveAction}\n */\nproto.ubo.v1.DockerImageRemoveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRemoveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRemoveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveAction} returns this\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRemoveAction} returns this\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveAction} returns this\n */\nproto.ubo.v1.DockerImageRemoveAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRunAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRunAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRunAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRunAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRunAction}\n */\nproto.ubo.v1.DockerImageRunAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRunAction;\n  return proto.ubo.v1.DockerImageRunAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRunAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRunAction}\n */\nproto.ubo.v1.DockerImageRunAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRunAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRunAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRunAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRunAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRunAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRunAction} returns this\n */\nproto.ubo.v1.DockerImageRunAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRunAction} returns this\n */\nproto.ubo.v1.DockerImageRunAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRunAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRunAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRunAction} returns this\n */\nproto.ubo.v1.DockerImageRunAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageStopAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageStopAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageStopAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageStopAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageStopAction}\n */\nproto.ubo.v1.DockerImageStopAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageStopAction;\n  return proto.ubo.v1.DockerImageStopAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageStopAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageStopAction}\n */\nproto.ubo.v1.DockerImageStopAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageStopAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageStopAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageStopAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageStopAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageStopAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageStopAction} returns this\n */\nproto.ubo.v1.DockerImageStopAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageStopAction} returns this\n */\nproto.ubo.v1.DockerImageStopAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageStopAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageStopAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageStopAction} returns this\n */\nproto.ubo.v1.DockerImageStopAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageReleaseAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageReleaseAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageReleaseAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageReleaseAction}\n */\nproto.ubo.v1.DockerImageReleaseAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageReleaseAction;\n  return proto.ubo.v1.DockerImageReleaseAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageReleaseAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageReleaseAction}\n */\nproto.ubo.v1.DockerImageReleaseAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageReleaseAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageReleaseAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageReleaseAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageReleaseAction} returns this\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageReleaseAction} returns this\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageReleaseAction} returns this\n */\nproto.ubo.v1.DockerImageReleaseAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRemoveContainerAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRemoveContainerAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRemoveContainerAction}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRemoveContainerAction;\n  return proto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRemoveContainerAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRemoveContainerAction}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRemoveContainerAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRemoveContainerAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveContainerAction} returns this\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRemoveContainerAction} returns this\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveContainerAction} returns this\n */\nproto.ubo.v1.DockerImageRemoveContainerAction.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerEvent}\n */\nproto.ubo.v1.DockerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerEvent;\n  return proto.ubo.v1.DockerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerEvent}\n */\nproto.ubo.v1.DockerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerEvent} returns this\n */\nproto.ubo.v1.DockerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerEvent} returns this\n */\nproto.ubo.v1.DockerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerInstallEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerInstallEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerInstallEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerInstallEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerInstallEvent}\n */\nproto.ubo.v1.DockerInstallEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerInstallEvent;\n  return proto.ubo.v1.DockerInstallEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerInstallEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerInstallEvent}\n */\nproto.ubo.v1.DockerInstallEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerInstallEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerInstallEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerInstallEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerInstallEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerInstallEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerInstallEvent} returns this\n */\nproto.ubo.v1.DockerInstallEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerInstallEvent} returns this\n */\nproto.ubo.v1.DockerInstallEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerInstallEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerStartEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerStartEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerStartEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStartEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerStartEvent}\n */\nproto.ubo.v1.DockerStartEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerStartEvent;\n  return proto.ubo.v1.DockerStartEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerStartEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerStartEvent}\n */\nproto.ubo.v1.DockerStartEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerStartEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerStartEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerStartEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStartEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerStartEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStartEvent} returns this\n */\nproto.ubo.v1.DockerStartEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerStartEvent} returns this\n */\nproto.ubo.v1.DockerStartEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerStartEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerStopEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerStopEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerStopEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStopEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerStopEvent}\n */\nproto.ubo.v1.DockerStopEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerStopEvent;\n  return proto.ubo.v1.DockerStopEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerStopEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerStopEvent}\n */\nproto.ubo.v1.DockerStopEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerStopEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerStopEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerStopEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerStopEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerStopEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerStopEvent} returns this\n */\nproto.ubo.v1.DockerStopEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerStopEvent} returns this\n */\nproto.ubo.v1.DockerStopEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerStopEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerServiceState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerServiceState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerServiceState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerServiceState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    status: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    usernames: (f = msg.getUsernames()) && proto.ubo.v1.DockerServiceState.UsernamesDict.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerServiceState}\n */\nproto.ubo.v1.DockerServiceState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerServiceState;\n  return proto.ubo.v1.DockerServiceState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerServiceState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerServiceState}\n */\nproto.ubo.v1.DockerServiceState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.DockerStatus} */ (reader.readEnum());\n      msg.setStatus(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.DockerServiceState.UsernamesDict;\n      reader.readMessage(value,proto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinaryFromReader);\n      msg.setUsernames(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerServiceState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerServiceState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerServiceState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerServiceState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.DockerStatus} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getUsernames();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.DockerServiceState.UsernamesDict.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerServiceState.UsernamesDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerServiceState.UsernamesDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerServiceState.UsernamesDict}\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerServiceState.UsernamesDict;\n  return proto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerServiceState.UsernamesDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerServiceState.UsernamesDict}\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, \"\", \"\");\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerServiceState.UsernamesDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerServiceState.UsernamesDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);\n  }\n};\n\n\n/**\n * map<string, string> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,string>}\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,string>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      null));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.DockerServiceState.UsernamesDict} returns this\n */\nproto.ubo.v1.DockerServiceState.UsernamesDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerServiceState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerServiceState} returns this\n */\nproto.ubo.v1.DockerServiceState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerServiceState} returns this\n */\nproto.ubo.v1.DockerServiceState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerServiceState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional DockerStatus status = 2;\n * @return {!proto.ubo.v1.DockerStatus}\n */\nproto.ubo.v1.DockerServiceState.prototype.getStatus = function() {\n  return /** @type {!proto.ubo.v1.DockerStatus} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.DockerStatus} value\n * @return {!proto.ubo.v1.DockerServiceState} returns this\n */\nproto.ubo.v1.DockerServiceState.prototype.setStatus = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerServiceState} returns this\n */\nproto.ubo.v1.DockerServiceState.prototype.clearStatus = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerServiceState.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional UsernamesDict usernames = 3;\n * @return {?proto.ubo.v1.DockerServiceState.UsernamesDict}\n */\nproto.ubo.v1.DockerServiceState.prototype.getUsernames = function() {\n  return /** @type{?proto.ubo.v1.DockerServiceState.UsernamesDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerServiceState.UsernamesDict, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerServiceState.UsernamesDict|undefined} value\n * @return {!proto.ubo.v1.DockerServiceState} returns this\n*/\nproto.ubo.v1.DockerServiceState.prototype.setUsernames = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.DockerServiceState} returns this\n */\nproto.ubo.v1.DockerServiceState.prototype.clearUsernames = function() {\n  return this.setUsernames(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerServiceState.prototype.hasUsernames = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageEvent}\n */\nproto.ubo.v1.DockerImageEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageEvent;\n  return proto.ubo.v1.DockerImageEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageEvent}\n */\nproto.ubo.v1.DockerImageEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageEvent} returns this\n */\nproto.ubo.v1.DockerImageEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageEvent} returns this\n */\nproto.ubo.v1.DockerImageEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageEvent} returns this\n */\nproto.ubo.v1.DockerImageEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRegisterAppEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRegisterAppEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRegisterAppEvent}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRegisterAppEvent;\n  return proto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRegisterAppEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRegisterAppEvent}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRegisterAppEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRegisterAppEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRegisterAppEvent} returns this\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRegisterAppEvent} returns this\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRegisterAppEvent} returns this\n */\nproto.ubo.v1.DockerImageRegisterAppEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageFetchCompositionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageFetchCompositionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageFetchCompositionEvent;\n  return proto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageFetchCompositionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageFetchCompositionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageFetchCompositionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageFetchCompositionEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageFetchEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageFetchEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageFetchEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageFetchEvent}\n */\nproto.ubo.v1.DockerImageFetchEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageFetchEvent;\n  return proto.ubo.v1.DockerImageFetchEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageFetchEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageFetchEvent}\n */\nproto.ubo.v1.DockerImageFetchEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageFetchEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageFetchEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageFetchEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageFetchEvent} returns this\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageFetchEvent} returns this\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageFetchEvent} returns this\n */\nproto.ubo.v1.DockerImageFetchEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRemoveCompositionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRemoveCompositionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRemoveCompositionEvent;\n  return proto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRemoveCompositionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRemoveCompositionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRemoveCompositionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRemoveEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRemoveEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRemoveEvent}\n */\nproto.ubo.v1.DockerImageRemoveEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRemoveEvent;\n  return proto.ubo.v1.DockerImageRemoveEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRemoveEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRemoveEvent}\n */\nproto.ubo.v1.DockerImageRemoveEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRemoveEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRemoveEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRemoveEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRunCompositionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRunCompositionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRunCompositionEvent}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRunCompositionEvent;\n  return proto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRunCompositionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRunCompositionEvent}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRunCompositionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRunCompositionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRunCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRunCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRunCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageRunCompositionEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRunContainerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRunContainerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRunContainerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRunContainerEvent}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRunContainerEvent;\n  return proto.ubo.v1.DockerImageRunContainerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRunContainerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRunContainerEvent}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRunContainerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRunContainerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRunContainerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRunContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRunContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRunContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageRunContainerEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageStopCompositionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageStopCompositionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageStopCompositionEvent}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageStopCompositionEvent;\n  return proto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageStopCompositionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageStopCompositionEvent}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageStopCompositionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageStopCompositionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageStopCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageStopCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageStopCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageStopCompositionEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageStopContainerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageStopContainerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageStopContainerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageStopContainerEvent}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageStopContainerEvent;\n  return proto.ubo.v1.DockerImageStopContainerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageStopContainerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageStopContainerEvent}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageStopContainerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageStopContainerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageStopContainerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageStopContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageStopContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageStopContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageStopContainerEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageReleaseCompositionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageReleaseCompositionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageReleaseCompositionEvent;\n  return proto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageReleaseCompositionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageReleaseCompositionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageReleaseCompositionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent} returns this\n */\nproto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerImageRemoveContainerEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerImageRemoveContainerEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    image: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerImageRemoveContainerEvent;\n  return proto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerImageRemoveContainerEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setImage(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerImageRemoveContainerEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerImageRemoveContainerEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getImage();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string image = 2;\n * @return {string}\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.getImage = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent} returns this\n */\nproto.ubo.v1.DockerImageRemoveContainerEvent.prototype.setImage = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ImageState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ImageState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ImageState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ImageState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    instructions: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    status: jspb.Message.getFieldWithDefault(msg, 5, 0),\n    containerIp: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    dockerId: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    ports: (f = msg.getPorts()) && proto.ubo.v1.ImageState.Ports.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ImageState}\n */\nproto.ubo.v1.ImageState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ImageState;\n  return proto.ubo.v1.ImageState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ImageState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ImageState}\n */\nproto.ubo.v1.ImageState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setInstructions(value);\n      break;\n    case 5:\n      var value = /** @type {!proto.ubo.v1.DockerItemStatus} */ (reader.readEnum());\n      msg.setStatus(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setContainerIp(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDockerId(value);\n      break;\n    case 8:\n      var value = new proto.ubo.v1.ImageState.Ports;\n      reader.readMessage(value,proto.ubo.v1.ImageState.Ports.deserializeBinaryFromReader);\n      msg.setPorts(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ImageState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ImageState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ImageState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ImageState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getLabel();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.DockerItemStatus} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeEnum(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = message.getPorts();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.ubo.v1.ImageState.Ports.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.ImageState.Ports.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ImageState.Ports.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ImageState.Ports.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ImageState.Ports} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ImageState.Ports.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ImageState.Ports}\n */\nproto.ubo.v1.ImageState.Ports.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ImageState.Ports;\n  return proto.ubo.v1.ImageState.Ports.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ImageState.Ports} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ImageState.Ports}\n */\nproto.ubo.v1.ImageState.Ports.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ImageState.Ports.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ImageState.Ports.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ImageState.Ports} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ImageState.Ports.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.ImageState.Ports.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.ImageState.Ports} returns this\n */\nproto.ubo.v1.ImageState.Ports.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.ImageState.Ports} returns this\n */\nproto.ubo.v1.ImageState.Ports.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.ImageState.Ports} returns this\n */\nproto.ubo.v1.ImageState.Ports.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.ImageState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ImageState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.ImageState.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string label = 3;\n * @return {string}\n */\nproto.ubo.v1.ImageState.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setLabel = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional string instructions = 4;\n * @return {string}\n */\nproto.ubo.v1.ImageState.prototype.getInstructions = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setInstructions = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.clearInstructions = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ImageState.prototype.hasInstructions = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional DockerItemStatus status = 5;\n * @return {!proto.ubo.v1.DockerItemStatus}\n */\nproto.ubo.v1.ImageState.prototype.getStatus = function() {\n  return /** @type {!proto.ubo.v1.DockerItemStatus} */ (jspb.Message.getFieldWithDefault(this, 5, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.DockerItemStatus} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setStatus = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.clearStatus = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ImageState.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string container_ip = 6;\n * @return {string}\n */\nproto.ubo.v1.ImageState.prototype.getContainerIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setContainerIp = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.clearContainerIp = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ImageState.prototype.hasContainerIp = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string docker_id = 7;\n * @return {string}\n */\nproto.ubo.v1.ImageState.prototype.getDockerId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.setDockerId = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.clearDockerId = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ImageState.prototype.hasDockerId = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional Ports ports = 8;\n * @return {?proto.ubo.v1.ImageState.Ports}\n */\nproto.ubo.v1.ImageState.prototype.getPorts = function() {\n  return /** @type{?proto.ubo.v1.ImageState.Ports} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ImageState.Ports, 8));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ImageState.Ports|undefined} value\n * @return {!proto.ubo.v1.ImageState} returns this\n*/\nproto.ubo.v1.ImageState.prototype.setPorts = function(value) {\n  return jspb.Message.setWrapperField(this, 8, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.ImageState} returns this\n */\nproto.ubo.v1.ImageState.prototype.clearPorts = function() {\n  return this.setPorts(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ImageState.prototype.hasPorts = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.DockerState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.DockerState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.DockerState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    service: (f = msg.getService()) && proto.ubo.v1.DockerServiceState.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.DockerState}\n */\nproto.ubo.v1.DockerState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.DockerState;\n  return proto.ubo.v1.DockerState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.DockerState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.DockerState}\n */\nproto.ubo.v1.DockerState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.DockerServiceState;\n      reader.readMessage(value,proto.ubo.v1.DockerServiceState.deserializeBinaryFromReader);\n      msg.setService(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.DockerState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.DockerState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.DockerState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.DockerState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getService();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.DockerServiceState.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;\n * @return {string}\n */\nproto.ubo.v1.DockerState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.DockerState} returns this\n */\nproto.ubo.v1.DockerState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.DockerState} returns this\n */\nproto.ubo.v1.DockerState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional DockerServiceState service = 2;\n * @return {?proto.ubo.v1.DockerServiceState}\n */\nproto.ubo.v1.DockerState.prototype.getService = function() {\n  return /** @type{?proto.ubo.v1.DockerServiceState} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerServiceState, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerServiceState|undefined} value\n * @return {!proto.ubo.v1.DockerState} returns this\n*/\nproto.ubo.v1.DockerState.prototype.setService = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.DockerState} returns this\n */\nproto.ubo.v1.DockerState.prototype.clearService = function() {\n  return this.setService(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.DockerState.prototype.hasService = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PathSelectorConfig.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PathSelectorConfig} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PathSelectorConfig.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    initialPath: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    showHidden: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    acceptsFiles: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    acceptsDirectories: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    acceptableSuffixes: (f = msg.getAcceptableSuffixes()) && proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PathSelectorConfig}\n */\nproto.ubo.v1.PathSelectorConfig.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PathSelectorConfig;\n  return proto.ubo.v1.PathSelectorConfig.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PathSelectorConfig} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PathSelectorConfig}\n */\nproto.ubo.v1.PathSelectorConfig.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setInitialPath(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShowHidden(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setAcceptsFiles(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setAcceptsDirectories(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes;\n      reader.readMessage(value,proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinaryFromReader);\n      msg.setAcceptableSuffixes(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PathSelectorConfig.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PathSelectorConfig} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PathSelectorConfig.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = message.getAcceptableSuffixes();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes}\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes;\n  return proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes}\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} returns this\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} returns this\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} returns this\n */\nproto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string initial_path = 2;\n * @return {string}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.getInitialPath = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.setInitialPath = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.clearInitialPath = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.hasInitialPath = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool show_hidden = 3;\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.getShowHidden = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.setShowHidden = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.clearShowHidden = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.hasShowHidden = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool accepts_files = 4;\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.getAcceptsFiles = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.setAcceptsFiles = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.clearAcceptsFiles = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.hasAcceptsFiles = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool accepts_directories = 5;\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.getAcceptsDirectories = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.setAcceptsDirectories = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.clearAcceptsDirectories = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.hasAcceptsDirectories = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional AcceptableSuffixes acceptable_suffixes = 6;\n * @return {?proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.getAcceptableSuffixes = function() {\n  return /** @type{?proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes|undefined} value\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n*/\nproto.ubo.v1.PathSelectorConfig.prototype.setAcceptableSuffixes = function(value) {\n  return jspb.Message.setWrapperField(this, 6, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.PathSelectorConfig} returns this\n */\nproto.ubo.v1.PathSelectorConfig.prototype.clearAcceptableSuffixes = function() {\n  return this.setAcceptableSuffixes(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.PathSelectorConfig.prototype.hasAcceptableSuffixes = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemAction}\n */\nproto.ubo.v1.FileSystemAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemAction;\n  return proto.ubo.v1.FileSystemAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemAction}\n */\nproto.ubo.v1.FileSystemAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemAction} returns this\n */\nproto.ubo.v1.FileSystemAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemAction} returns this\n */\nproto.ubo.v1.FileSystemAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemReportSelectionAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemReportSelectionAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemReportSelectionAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    path: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemReportSelectionAction}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemReportSelectionAction;\n  return proto.ubo.v1.FileSystemReportSelectionAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemReportSelectionAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemReportSelectionAction}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPath(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemReportSelectionAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemReportSelectionAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemReportSelectionAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getPath();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemReportSelectionAction} returns this\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemReportSelectionAction} returns this\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string path = 2;\n * @return {string}\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.getPath = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemReportSelectionAction} returns this\n */\nproto.ubo.v1.FileSystemReportSelectionAction.prototype.setPath = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemCopyAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemCopyAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemCopyAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemCopyAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,\n    destination: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemCopyAction}\n */\nproto.ubo.v1.FileSystemCopyAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemCopyAction;\n  return proto.ubo.v1.FileSystemCopyAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemCopyAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemCopyAction}\n */\nproto.ubo.v1.FileSystemCopyAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addSources(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDestination(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemCopyAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemCopyAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemCopyAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSourcesList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n  f = message.getDestination();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemCopyAction} returns this\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemCopyAction} returns this\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string sources = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.getSourcesList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.FileSystemCopyAction} returns this\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.setSourcesList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.FileSystemCopyAction} returns this\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.addSources = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemCopyAction} returns this\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.clearSourcesList = function() {\n  return this.setSourcesList([]);\n};\n\n\n/**\n * optional string destination = 3;\n * @return {string}\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.getDestination = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemCopyAction} returns this\n */\nproto.ubo.v1.FileSystemCopyAction.prototype.setDestination = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemMoveAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemMoveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemMoveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemMoveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,\n    destination: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemMoveAction}\n */\nproto.ubo.v1.FileSystemMoveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemMoveAction;\n  return proto.ubo.v1.FileSystemMoveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemMoveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemMoveAction}\n */\nproto.ubo.v1.FileSystemMoveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addSources(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDestination(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemMoveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemMoveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemMoveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSourcesList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n  f = message.getDestination();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemMoveAction} returns this\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemMoveAction} returns this\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string sources = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.getSourcesList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.FileSystemMoveAction} returns this\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.setSourcesList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.FileSystemMoveAction} returns this\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.addSources = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemMoveAction} returns this\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.clearSourcesList = function() {\n  return this.setSourcesList([]);\n};\n\n\n/**\n * optional string destination = 3;\n * @return {string}\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.getDestination = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemMoveAction} returns this\n */\nproto.ubo.v1.FileSystemMoveAction.prototype.setDestination = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemRemoveAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemRemoveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemRemoveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemRemoveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    pathsList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemRemoveAction}\n */\nproto.ubo.v1.FileSystemRemoveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemRemoveAction;\n  return proto.ubo.v1.FileSystemRemoveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemRemoveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemRemoveAction}\n */\nproto.ubo.v1.FileSystemRemoveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addPaths(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemRemoveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemRemoveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemRemoveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getPathsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string paths = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.getPathsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.setPathsList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.addPaths = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this\n */\nproto.ubo.v1.FileSystemRemoveAction.prototype.clearPathsList = function() {\n  return this.setPathsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemEvent}\n */\nproto.ubo.v1.FileSystemEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemEvent;\n  return proto.ubo.v1.FileSystemEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemEvent}\n */\nproto.ubo.v1.FileSystemEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemEvent} returns this\n */\nproto.ubo.v1.FileSystemEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemEvent} returns this\n */\nproto.ubo.v1.FileSystemEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemSelectEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemSelectEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemSelectEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    description: (f = msg.getDescription()) && proto.ubo.v1.PathInputDescription.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemSelectEvent}\n */\nproto.ubo.v1.FileSystemSelectEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemSelectEvent;\n  return proto.ubo.v1.FileSystemSelectEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemSelectEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemSelectEvent}\n */\nproto.ubo.v1.FileSystemSelectEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.PathInputDescription;\n      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);\n      msg.setDescription(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemSelectEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemSelectEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemSelectEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDescription();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional PathInputDescription description = 2;\n * @return {?proto.ubo.v1.PathInputDescription}\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.getDescription = function() {\n  return /** @type{?proto.ubo.v1.PathInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PathInputDescription, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PathInputDescription|undefined} value\n * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this\n*/\nproto.ubo.v1.FileSystemSelectEvent.prototype.setDescription = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.clearDescription = function() {\n  return this.setDescription(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemSelectEvent.prototype.hasDescription = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemCopyEvent.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemCopyEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemCopyEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemCopyEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,\n    destination: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemCopyEvent}\n */\nproto.ubo.v1.FileSystemCopyEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemCopyEvent;\n  return proto.ubo.v1.FileSystemCopyEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemCopyEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemCopyEvent}\n */\nproto.ubo.v1.FileSystemCopyEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addSources(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDestination(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemCopyEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemCopyEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemCopyEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSourcesList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n  f = message.getDestination();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string sources = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.getSourcesList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.setSourcesList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.addSources = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.clearSourcesList = function() {\n  return this.setSourcesList([]);\n};\n\n\n/**\n * optional string destination = 3;\n * @return {string}\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.getDestination = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this\n */\nproto.ubo.v1.FileSystemCopyEvent.prototype.setDestination = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemMoveEvent.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemMoveEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemMoveEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemMoveEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,\n    destination: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemMoveEvent}\n */\nproto.ubo.v1.FileSystemMoveEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemMoveEvent;\n  return proto.ubo.v1.FileSystemMoveEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemMoveEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemMoveEvent}\n */\nproto.ubo.v1.FileSystemMoveEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addSources(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setDestination(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemMoveEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemMoveEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemMoveEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSourcesList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n  f = message.getDestination();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string sources = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.getSourcesList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.setSourcesList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.addSources = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.clearSourcesList = function() {\n  return this.setSourcesList([]);\n};\n\n\n/**\n * optional string destination = 3;\n * @return {string}\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.getDestination = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this\n */\nproto.ubo.v1.FileSystemMoveEvent.prototype.setDestination = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemRemoveEvent.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemRemoveEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemRemoveEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemRemoveEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    pathsList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemRemoveEvent}\n */\nproto.ubo.v1.FileSystemRemoveEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemRemoveEvent;\n  return proto.ubo.v1.FileSystemRemoveEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemRemoveEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemRemoveEvent}\n */\nproto.ubo.v1.FileSystemRemoveEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addPaths(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemRemoveEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemRemoveEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemRemoveEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getPathsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string paths = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.getPathsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.setPathsList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.addPaths = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this\n */\nproto.ubo.v1.FileSystemRemoveEvent.prototype.clearPathsList = function() {\n  return this.setPathsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.FileSystemState.repeatedFields_ = [3];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.FileSystemState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.FileSystemState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.FileSystemState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    currentInput: (f = msg.getCurrentInput()) && proto.ubo.v1.PathInputDescription.toObject(includeInstance, f),\n    queueList: jspb.Message.toObjectList(msg.getQueueList(),\n    proto.ubo.v1.PathInputDescription.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.FileSystemState}\n */\nproto.ubo.v1.FileSystemState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.FileSystemState;\n  return proto.ubo.v1.FileSystemState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.FileSystemState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.FileSystemState}\n */\nproto.ubo.v1.FileSystemState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.PathInputDescription;\n      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);\n      msg.setCurrentInput(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.PathInputDescription;\n      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);\n      msg.addQueue(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.FileSystemState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.FileSystemState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.FileSystemState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.FileSystemState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getCurrentInput();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter\n    );\n  }\n  f = message.getQueueList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      3,\n      f,\n      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;\n * @return {string}\n */\nproto.ubo.v1.FileSystemState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.FileSystemState} returns this\n */\nproto.ubo.v1.FileSystemState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.FileSystemState} returns this\n */\nproto.ubo.v1.FileSystemState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional PathInputDescription current_input = 2;\n * @return {?proto.ubo.v1.PathInputDescription}\n */\nproto.ubo.v1.FileSystemState.prototype.getCurrentInput = function() {\n  return /** @type{?proto.ubo.v1.PathInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PathInputDescription, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PathInputDescription|undefined} value\n * @return {!proto.ubo.v1.FileSystemState} returns this\n*/\nproto.ubo.v1.FileSystemState.prototype.setCurrentInput = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.FileSystemState} returns this\n */\nproto.ubo.v1.FileSystemState.prototype.clearCurrentInput = function() {\n  return this.setCurrentInput(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.FileSystemState.prototype.hasCurrentInput = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * repeated PathInputDescription queue = 3;\n * @return {!Array<!proto.ubo.v1.PathInputDescription>}\n */\nproto.ubo.v1.FileSystemState.prototype.getQueueList = function() {\n  return /** @type{!Array<!proto.ubo.v1.PathInputDescription>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.PathInputDescription, 3));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.PathInputDescription>} value\n * @return {!proto.ubo.v1.FileSystemState} returns this\n*/\nproto.ubo.v1.FileSystemState.prototype.setQueueList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 3, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.PathInputDescription=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.PathInputDescription}\n */\nproto.ubo.v1.FileSystemState.prototype.addQueue = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.PathInputDescription, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.FileSystemState} returns this\n */\nproto.ubo.v1.FileSystemState.prototype.clearQueueList = function() {\n  return this.setQueueList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredAction}\n */\nproto.ubo.v1.InfraredAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredAction;\n  return proto.ubo.v1.InfraredAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredAction}\n */\nproto.ubo.v1.InfraredAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredAction} returns this\n */\nproto.ubo.v1.InfraredAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredAction} returns this\n */\nproto.ubo.v1.InfraredAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredEvent}\n */\nproto.ubo.v1.InfraredEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredEvent;\n  return proto.ubo.v1.InfraredEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredEvent}\n */\nproto.ubo.v1.InfraredEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredEvent} returns this\n */\nproto.ubo.v1.InfraredEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredEvent} returns this\n */\nproto.ubo.v1.InfraredEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredHandleReceivedCodeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredHandleReceivedCodeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    protocol: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    scancode: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredHandleReceivedCodeAction;\n  return proto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredHandleReceivedCodeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setProtocol(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setScancode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredHandleReceivedCodeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredHandleReceivedCodeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getProtocol();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getScancode();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string protocol = 2;\n * @return {string}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.getProtocol = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.setProtocol = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string scancode = 3;\n * @return {string}\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.getScancode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this\n */\nproto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.setScancode = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredSendCodeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredSendCodeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSendCodeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    protocol: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    scancode: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredSendCodeAction}\n */\nproto.ubo.v1.InfraredSendCodeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredSendCodeAction;\n  return proto.ubo.v1.InfraredSendCodeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredSendCodeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredSendCodeAction}\n */\nproto.ubo.v1.InfraredSendCodeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setProtocol(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setScancode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredSendCodeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredSendCodeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSendCodeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getProtocol();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getScancode();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string protocol = 2;\n * @return {string}\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.getProtocol = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.setProtocol = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string scancode = 3;\n * @return {string}\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.getScancode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this\n */\nproto.ubo.v1.InfraredSendCodeAction.prototype.setScancode = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredSetShouldPropagateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredSetShouldPropagateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    shouldPropagate: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredSetShouldPropagateAction;\n  return proto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredSetShouldPropagateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShouldPropagate(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredSetShouldPropagateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredSetShouldPropagateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getShouldPropagate();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction} returns this\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction} returns this\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool should_propagate = 2;\n * @return {boolean}\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.getShouldPropagate = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction} returns this\n */\nproto.ubo.v1.InfraredSetShouldPropagateAction.prototype.setShouldPropagate = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredSetShouldReceiveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredSetShouldReceiveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    shouldReceive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredSetShouldReceiveAction;\n  return proto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredSetShouldReceiveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShouldReceive(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredSetShouldReceiveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredSetShouldReceiveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getShouldReceive();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction} returns this\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction} returns this\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool should_receive = 2;\n * @return {boolean}\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.getShouldReceive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction} returns this\n */\nproto.ubo.v1.InfraredSetShouldReceiveAction.prototype.setShouldReceive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredSendCodeEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredSendCodeEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSendCodeEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    protocol: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    scancode: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredSendCodeEvent}\n */\nproto.ubo.v1.InfraredSendCodeEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredSendCodeEvent;\n  return proto.ubo.v1.InfraredSendCodeEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredSendCodeEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredSendCodeEvent}\n */\nproto.ubo.v1.InfraredSendCodeEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setProtocol(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setScancode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredSendCodeEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredSendCodeEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredSendCodeEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getProtocol();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getScancode();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string protocol = 2;\n * @return {string}\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.getProtocol = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.setProtocol = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string scancode = 3;\n * @return {string}\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.getScancode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this\n */\nproto.ubo.v1.InfraredSendCodeEvent.prototype.setScancode = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.InfraredState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.InfraredState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.InfraredState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    shouldPropagateKeypadActions: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    shouldReceiveKeypadActions: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.InfraredState}\n */\nproto.ubo.v1.InfraredState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.InfraredState;\n  return proto.ubo.v1.InfraredState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.InfraredState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.InfraredState}\n */\nproto.ubo.v1.InfraredState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShouldPropagateKeypadActions(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShouldReceiveKeypadActions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.InfraredState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.InfraredState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.InfraredState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.InfraredState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;\n * @return {string}\n */\nproto.ubo.v1.InfraredState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.InfraredState} returns this\n */\nproto.ubo.v1.InfraredState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredState} returns this\n */\nproto.ubo.v1.InfraredState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool should_propagate_keypad_actions = 2;\n * @return {boolean}\n */\nproto.ubo.v1.InfraredState.prototype.getShouldPropagateKeypadActions = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.InfraredState} returns this\n */\nproto.ubo.v1.InfraredState.prototype.setShouldPropagateKeypadActions = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredState} returns this\n */\nproto.ubo.v1.InfraredState.prototype.clearShouldPropagateKeypadActions = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredState.prototype.hasShouldPropagateKeypadActions = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool should_receive_keypad_actions = 3;\n * @return {boolean}\n */\nproto.ubo.v1.InfraredState.prototype.getShouldReceiveKeypadActions = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.InfraredState} returns this\n */\nproto.ubo.v1.InfraredState.prototype.setShouldReceiveKeypadActions = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.InfraredState} returns this\n */\nproto.ubo.v1.InfraredState.prototype.clearShouldReceiveKeypadActions = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.InfraredState.prototype.hasShouldReceiveKeypadActions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IpAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IpAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IpAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IpAction}\n */\nproto.ubo.v1.IpAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IpAction;\n  return proto.ubo.v1.IpAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IpAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IpAction}\n */\nproto.ubo.v1.IpAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IpAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IpAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IpAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;\n * @return {string}\n */\nproto.ubo.v1.IpAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpAction} returns this\n */\nproto.ubo.v1.IpAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpAction} returns this\n */\nproto.ubo.v1.IpAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IpEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IpEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IpEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IpEvent}\n */\nproto.ubo.v1.IpEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IpEvent;\n  return proto.ubo.v1.IpEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IpEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IpEvent}\n */\nproto.ubo.v1.IpEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IpEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IpEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IpEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;\n * @return {string}\n */\nproto.ubo.v1.IpEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpEvent} returns this\n */\nproto.ubo.v1.IpEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpEvent} returns this\n */\nproto.ubo.v1.IpEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.IpUpdateInterfacesAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IpUpdateInterfacesAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IpUpdateInterfacesAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpUpdateInterfacesAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    interfacesList: jspb.Message.toObjectList(msg.getInterfacesList(),\n    proto.ubo.v1.IpNetworkInterface.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IpUpdateInterfacesAction}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IpUpdateInterfacesAction;\n  return proto.ubo.v1.IpUpdateInterfacesAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IpUpdateInterfacesAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IpUpdateInterfacesAction}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.IpNetworkInterface;\n      reader.readMessage(value,proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader);\n      msg.addInterfaces(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IpUpdateInterfacesAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IpUpdateInterfacesAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpUpdateInterfacesAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getInterfacesList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;\n * @return {string}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated IpNetworkInterface interfaces = 2;\n * @return {!Array<!proto.ubo.v1.IpNetworkInterface>}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.getInterfacesList = function() {\n  return /** @type{!Array<!proto.ubo.v1.IpNetworkInterface>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.IpNetworkInterface, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.IpNetworkInterface>} value\n * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this\n*/\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.setInterfacesList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.IpNetworkInterface=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.IpNetworkInterface}\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.addInterfaces = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.IpNetworkInterface, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this\n */\nproto.ubo.v1.IpUpdateInterfacesAction.prototype.clearInterfacesList = function() {\n  return this.setInterfacesList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IpSetIsConnectedAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IpSetIsConnectedAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpSetIsConnectedAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isConnected: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IpSetIsConnectedAction}\n */\nproto.ubo.v1.IpSetIsConnectedAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IpSetIsConnectedAction;\n  return proto.ubo.v1.IpSetIsConnectedAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IpSetIsConnectedAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IpSetIsConnectedAction}\n */\nproto.ubo.v1.IpSetIsConnectedAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsConnected(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IpSetIsConnectedAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IpSetIsConnectedAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpSetIsConnectedAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsConnected();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;\n * @return {string}\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpSetIsConnectedAction} returns this\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpSetIsConnectedAction} returns this\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_connected = 2;\n * @return {boolean}\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.getIsConnected = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.IpSetIsConnectedAction} returns this\n */\nproto.ubo.v1.IpSetIsConnectedAction.prototype.setIsConnected = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.IpNetworkInterface.repeatedFields_ = [3];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IpNetworkInterface.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IpNetworkInterface.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IpNetworkInterface} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpNetworkInterface.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    name: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    ipAddressesList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IpNetworkInterface}\n */\nproto.ubo.v1.IpNetworkInterface.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IpNetworkInterface;\n  return proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IpNetworkInterface} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IpNetworkInterface}\n */\nproto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addIpAddresses(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IpNetworkInterface.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IpNetworkInterface} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getName();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getIpAddressesList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;\n * @return {string}\n */\nproto.ubo.v1.IpNetworkInterface.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpNetworkInterface} returns this\n */\nproto.ubo.v1.IpNetworkInterface.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpNetworkInterface} returns this\n */\nproto.ubo.v1.IpNetworkInterface.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpNetworkInterface.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string name = 2;\n * @return {string}\n */\nproto.ubo.v1.IpNetworkInterface.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpNetworkInterface} returns this\n */\nproto.ubo.v1.IpNetworkInterface.prototype.setName = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * repeated string ip_addresses = 3;\n * @return {!Array<string>}\n */\nproto.ubo.v1.IpNetworkInterface.prototype.getIpAddressesList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 3));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.IpNetworkInterface} returns this\n */\nproto.ubo.v1.IpNetworkInterface.prototype.setIpAddressesList = function(value) {\n  return jspb.Message.setField(this, 3, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.IpNetworkInterface} returns this\n */\nproto.ubo.v1.IpNetworkInterface.prototype.addIpAddresses = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.IpNetworkInterface} returns this\n */\nproto.ubo.v1.IpNetworkInterface.prototype.clearIpAddressesList = function() {\n  return this.setIpAddressesList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.IpState.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.IpState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.IpState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.IpState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    interfacesList: jspb.Message.toObjectList(msg.getInterfacesList(),\n    proto.ubo.v1.IpNetworkInterface.toObject, includeInstance),\n    isConnected: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.IpState}\n */\nproto.ubo.v1.IpState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.IpState;\n  return proto.ubo.v1.IpState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.IpState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.IpState}\n */\nproto.ubo.v1.IpState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.IpNetworkInterface;\n      reader.readMessage(value,proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader);\n      msg.addInterfaces(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsConnected(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.IpState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.IpState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.IpState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.IpState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getInterfacesList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;\n * @return {string}\n */\nproto.ubo.v1.IpState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.IpState} returns this\n */\nproto.ubo.v1.IpState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpState} returns this\n */\nproto.ubo.v1.IpState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated IpNetworkInterface interfaces = 2;\n * @return {!Array<!proto.ubo.v1.IpNetworkInterface>}\n */\nproto.ubo.v1.IpState.prototype.getInterfacesList = function() {\n  return /** @type{!Array<!proto.ubo.v1.IpNetworkInterface>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.IpNetworkInterface, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.IpNetworkInterface>} value\n * @return {!proto.ubo.v1.IpState} returns this\n*/\nproto.ubo.v1.IpState.prototype.setInterfacesList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.IpNetworkInterface=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.IpNetworkInterface}\n */\nproto.ubo.v1.IpState.prototype.addInterfaces = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.IpNetworkInterface, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.IpState} returns this\n */\nproto.ubo.v1.IpState.prototype.clearInterfacesList = function() {\n  return this.setInterfacesList([]);\n};\n\n\n/**\n * optional bool is_connected = 3;\n * @return {boolean}\n */\nproto.ubo.v1.IpState.prototype.getIsConnected = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.IpState} returns this\n */\nproto.ubo.v1.IpState.prototype.setIsConnected = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.IpState} returns this\n */\nproto.ubo.v1.IpState.prototype.clearIsConnected = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.IpState.prototype.hasIsConnected = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadAction.PressedKeysSetType.toObject(includeInstance, f),\n    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadAction.HeldKeysSetType.toObject(includeInstance, f),\n    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadAction}\n */\nproto.ubo.v1.KeypadAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadAction;\n  return proto.ubo.v1.KeypadAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadAction}\n */\nproto.ubo.v1.KeypadAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.KeypadAction.PressedKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinaryFromReader);\n      msg.setPressedKeys(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.KeypadAction.HeldKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinaryFromReader);\n      msg.setHeldKeys(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTime(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getKey();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getPressedKeys();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.KeypadAction.PressedKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = message.getHeldKeys();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.KeypadAction.HeldKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadAction.PressedKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadAction.PressedKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadAction.PressedKeysSetType;\n  return proto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadAction.PressedKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadAction.PressedKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadAction.HeldKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadAction.HeldKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadAction.HeldKeysSetType;\n  return proto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadAction.HeldKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadAction.HeldKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;\n * @return {string}\n */\nproto.ubo.v1.KeypadAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Key key = 2;\n * @return {!proto.ubo.v1.Key}\n */\nproto.ubo.v1.KeypadAction.prototype.getKey = function() {\n  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.setKey = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional PressedKeysSetType pressed_keys = 3;\n * @return {?proto.ubo.v1.KeypadAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadAction.prototype.getPressedKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadAction.PressedKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadAction.PressedKeysSetType, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadAction.PressedKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadAction} returns this\n*/\nproto.ubo.v1.KeypadAction.prototype.setPressedKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.clearPressedKeys = function() {\n  return this.setPressedKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadAction.prototype.hasPressedKeys = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional HeldKeysSetType held_keys = 4;\n * @return {?proto.ubo.v1.KeypadAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadAction.prototype.getHeldKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadAction.HeldKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadAction.HeldKeysSetType, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadAction.HeldKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadAction} returns this\n*/\nproto.ubo.v1.KeypadAction.prototype.setHeldKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.clearHeldKeys = function() {\n  return this.setHeldKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadAction.prototype.hasHeldKeys = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional float time = 5;\n * @return {number}\n */\nproto.ubo.v1.KeypadAction.prototype.getTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.setTime = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadAction} returns this\n */\nproto.ubo.v1.KeypadAction.prototype.clearTime = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadAction.prototype.hasTime = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyPressAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyPressAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyPressAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.toObject(includeInstance, f),\n    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.toObject(includeInstance, f),\n    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyPressAction}\n */\nproto.ubo.v1.KeypadKeyPressAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyPressAction;\n  return proto.ubo.v1.KeypadKeyPressAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyPressAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyPressAction}\n */\nproto.ubo.v1.KeypadKeyPressAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinaryFromReader);\n      msg.setPressedKeys(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinaryFromReader);\n      msg.setHeldKeys(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTime(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyPressAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyPressAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyPressAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getKey();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getPressedKeys();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = message.getHeldKeys();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType;\n  return proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType;\n  return proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;\n * @return {string}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Key key = 2;\n * @return {!proto.ubo.v1.Key}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.getKey = function() {\n  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.setKey = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional PressedKeysSetType pressed_keys = 3;\n * @return {?proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.getPressedKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n*/\nproto.ubo.v1.KeypadKeyPressAction.prototype.setPressedKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.clearPressedKeys = function() {\n  return this.setPressedKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.hasPressedKeys = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional HeldKeysSetType held_keys = 4;\n * @return {?proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.getHeldKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n*/\nproto.ubo.v1.KeypadKeyPressAction.prototype.setHeldKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.clearHeldKeys = function() {\n  return this.setHeldKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.hasHeldKeys = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional float time = 5;\n * @return {number}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.getTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.setTime = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.clearTime = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyPressAction.prototype.hasTime = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyHoldAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyHoldAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyHoldAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.toObject(includeInstance, f),\n    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.toObject(includeInstance, f),\n    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction}\n */\nproto.ubo.v1.KeypadKeyHoldAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyHoldAction;\n  return proto.ubo.v1.KeypadKeyHoldAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyHoldAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction}\n */\nproto.ubo.v1.KeypadKeyHoldAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinaryFromReader);\n      msg.setPressedKeys(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinaryFromReader);\n      msg.setHeldKeys(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTime(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyHoldAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyHoldAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyHoldAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getKey();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getPressedKeys();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = message.getHeldKeys();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType;\n  return proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType;\n  return proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;\n * @return {string}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Key key = 2;\n * @return {!proto.ubo.v1.Key}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.getKey = function() {\n  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.setKey = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional PressedKeysSetType pressed_keys = 3;\n * @return {?proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.getPressedKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n*/\nproto.ubo.v1.KeypadKeyHoldAction.prototype.setPressedKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.clearPressedKeys = function() {\n  return this.setPressedKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.hasPressedKeys = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional HeldKeysSetType held_keys = 4;\n * @return {?proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.getHeldKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n*/\nproto.ubo.v1.KeypadKeyHoldAction.prototype.setHeldKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.clearHeldKeys = function() {\n  return this.setHeldKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.hasHeldKeys = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional float time = 5;\n * @return {number}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.getTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.setTime = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.clearTime = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyHoldAction.prototype.hasTime = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyUnholdAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyUnholdAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.toObject(includeInstance, f),\n    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.toObject(includeInstance, f),\n    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyUnholdAction;\n  return proto.ubo.v1.KeypadKeyUnholdAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinaryFromReader);\n      msg.setPressedKeys(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinaryFromReader);\n      msg.setHeldKeys(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTime(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyUnholdAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyUnholdAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getKey();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getPressedKeys();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = message.getHeldKeys();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType;\n  return proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType;\n  return proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;\n * @return {string}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Key key = 2;\n * @return {!proto.ubo.v1.Key}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.getKey = function() {\n  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.setKey = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional PressedKeysSetType pressed_keys = 3;\n * @return {?proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.getPressedKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n*/\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.setPressedKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.clearPressedKeys = function() {\n  return this.setPressedKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.hasPressedKeys = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional HeldKeysSetType held_keys = 4;\n * @return {?proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.getHeldKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n*/\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.setHeldKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.clearHeldKeys = function() {\n  return this.setHeldKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.hasHeldKeys = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional float time = 5;\n * @return {number}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.getTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.setTime = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.clearTime = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyUnholdAction.prototype.hasTime = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyReleaseAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyReleaseAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    key: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.toObject(includeInstance, f),\n    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.toObject(includeInstance, f),\n    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyReleaseAction;\n  return proto.ubo.v1.KeypadKeyReleaseAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());\n      msg.setKey(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinaryFromReader);\n      msg.setPressedKeys(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinaryFromReader);\n      msg.setHeldKeys(value);\n      break;\n    case 5:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTime(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyReleaseAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyReleaseAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getKey();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getPressedKeys();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = message.getHeldKeys();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeFloat(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType;\n  return proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType;\n  return proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);\n      for (var i = 0; i < values.length; i++) {\n        msg.addItems(values[i]);\n      }\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writePackedEnum(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated Key items = 1;\n * @return {!Array<!proto.ubo.v1.Key>}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.getItemsList = function() {\n  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Key>} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;\n * @return {string}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Key key = 2;\n * @return {!proto.ubo.v1.Key}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.getKey = function() {\n  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Key} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.setKey = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional PressedKeysSetType pressed_keys = 3;\n * @return {?proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.getPressedKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n*/\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.setPressedKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.clearPressedKeys = function() {\n  return this.setPressedKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.hasPressedKeys = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional HeldKeysSetType held_keys = 4;\n * @return {?proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.getHeldKeys = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType|undefined} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n*/\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.setHeldKeys = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.clearHeldKeys = function() {\n  return this.setHeldKeys(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.hasHeldKeys = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional float time = 5;\n * @return {number}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.getTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.setTime = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.clearTime = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadKeyReleaseAction.prototype.hasTime = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.KeypadState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.KeypadState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.KeypadState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    depth: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    isConsumed: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.KeypadState}\n */\nproto.ubo.v1.KeypadState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.KeypadState;\n  return proto.ubo.v1.KeypadState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.KeypadState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.KeypadState}\n */\nproto.ubo.v1.KeypadState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setDepth(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsConsumed(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.KeypadState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.KeypadState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.KeypadState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.KeypadState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;\n * @return {string}\n */\nproto.ubo.v1.KeypadState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.KeypadState} returns this\n */\nproto.ubo.v1.KeypadState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadState} returns this\n */\nproto.ubo.v1.KeypadState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 depth = 2;\n * @return {number}\n */\nproto.ubo.v1.KeypadState.prototype.getDepth = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.KeypadState} returns this\n */\nproto.ubo.v1.KeypadState.prototype.setDepth = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadState} returns this\n */\nproto.ubo.v1.KeypadState.prototype.clearDepth = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadState.prototype.hasDepth = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_consumed = 3;\n * @return {boolean}\n */\nproto.ubo.v1.KeypadState.prototype.getIsConsumed = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.KeypadState} returns this\n */\nproto.ubo.v1.KeypadState.prototype.setIsConsumed = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.KeypadState} returns this\n */\nproto.ubo.v1.KeypadState.prototype.clearIsConsumed = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.KeypadState.prototype.hasIsConsumed = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.LightDMAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.LightDMAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.LightDMAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.LightDMAction}\n */\nproto.ubo.v1.LightDMAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.LightDMAction;\n  return proto.ubo.v1.LightDMAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.LightDMAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.LightDMAction}\n */\nproto.ubo.v1.LightDMAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.LightDMAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.LightDMAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.LightDMAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;\n * @return {string}\n */\nproto.ubo.v1.LightDMAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.LightDMAction} returns this\n */\nproto.ubo.v1.LightDMAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMAction} returns this\n */\nproto.ubo.v1.LightDMAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.LightDMUpdateStateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.LightDMUpdateStateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMUpdateStateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    isInstalling: jspb.Message.getBooleanFieldWithDefault(msg, 5, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction}\n */\nproto.ubo.v1.LightDMUpdateStateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.LightDMUpdateStateAction;\n  return proto.ubo.v1.LightDMUpdateStateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.LightDMUpdateStateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction}\n */\nproto.ubo.v1.LightDMUpdateStateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnabled(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsInstalled(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsInstalling(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.LightDMUpdateStateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.LightDMUpdateStateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMUpdateStateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;\n * @return {string}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_enabled = 3;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.getIsEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.setIsEnabled = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsEnabled = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsEnabled = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool is_installed = 4;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.getIsInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.setIsInstalled = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsInstalled = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsInstalled = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_installing = 5;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.getIsInstalling = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.setIsInstalling = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsInstalling = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsInstalling = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.LightDMClearEnabledStateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.LightDMClearEnabledStateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.LightDMClearEnabledStateAction}\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.LightDMClearEnabledStateAction;\n  return proto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.LightDMClearEnabledStateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.LightDMClearEnabledStateAction}\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.LightDMClearEnabledStateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.LightDMClearEnabledStateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;\n * @return {string}\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.LightDMClearEnabledStateAction} returns this\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMClearEnabledStateAction} returns this\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMClearEnabledStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.LightDMState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.LightDMState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.LightDMState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    isInstalling: jspb.Message.getBooleanFieldWithDefault(msg, 5, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.LightDMState}\n */\nproto.ubo.v1.LightDMState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.LightDMState;\n  return proto.ubo.v1.LightDMState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.LightDMState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.LightDMState}\n */\nproto.ubo.v1.LightDMState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnabled(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsInstalled(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsInstalling(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.LightDMState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.LightDMState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.LightDMState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.LightDMState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;\n * @return {string}\n */\nproto.ubo.v1.LightDMState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_enabled = 3;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.getIsEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.setIsEnabled = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.clearIsEnabled = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.hasIsEnabled = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool is_installed = 4;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.getIsInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.setIsInstalled = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.clearIsInstalled = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.hasIsInstalled = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_installing = 5;\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.getIsInstalling = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.setIsInstalling = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.LightDMState} returns this\n */\nproto.ubo.v1.LightDMState.prototype.clearIsInstalling = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.LightDMState.prototype.hasIsInstalling = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationActionItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationActionItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationActionItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationActionItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    dismissNotification: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    closeNotification: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    key: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 8, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 9, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 11, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationActionItem}\n */\nproto.ubo.v1.NotificationActionItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationActionItem;\n  return proto.ubo.v1.NotificationActionItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationActionItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationActionItem}\n */\nproto.ubo.v1.NotificationActionItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDismissNotification(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setCloseNotification(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 8:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 9:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 10:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 11:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationActionItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationActionItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationActionItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationActionItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeString(\n      8,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeBool(\n      9,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeFloat(\n      10,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeFloat(\n      11,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string background_color = 2;\n * @return {string}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool dismiss_notification = 3;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getDismissNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setDismissNotification = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearDismissNotification = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasDismissNotification = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool close_notification = 4;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getCloseNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setCloseNotification = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearCloseNotification = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasCloseNotification = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string key = 5;\n * @return {string}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string label = 6;\n * @return {string}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string color = 7;\n * @return {string}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional string icon = 8;\n * @return {string}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional bool is_short = 9;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional float opacity = 10;\n * @return {number}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional float progress = 11;\n * @return {number}\n */\nproto.ubo.v1.NotificationActionItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 11, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationActionItem} returns this\n */\nproto.ubo.v1.NotificationActionItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationActionItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationApplicationItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationApplicationItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    dismissNotification: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    closeNotification: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    application: (f = msg.getApplication()) && proto.ubo.v1.NotificationApplicationItem.Application.toObject(includeInstance, f),\n    applicationId: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.NotificationApplicationItem.InitializationArgs.toObject(includeInstance, f),\n    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.toObject(includeInstance, f),\n    key: jspb.Message.getFieldWithDefault(msg, 9, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 10, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 11, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 12, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 13, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 14, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 15, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationApplicationItem}\n */\nproto.ubo.v1.NotificationApplicationItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationApplicationItem;\n  return proto.ubo.v1.NotificationApplicationItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationApplicationItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationApplicationItem}\n */\nproto.ubo.v1.NotificationApplicationItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDismissNotification(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setCloseNotification(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.NotificationApplicationItem.Application;\n      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.Application.deserializeBinaryFromReader);\n      msg.setApplication(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setApplicationId(value);\n      break;\n    case 7:\n      var value = new proto.ubo.v1.NotificationApplicationItem.InitializationArgs;\n      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinaryFromReader);\n      msg.setInitializationArgs(value);\n      break;\n    case 8:\n      var value = new proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict;\n      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader);\n      msg.setInitializationKwargs(value);\n      break;\n    case 9:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 10:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 11:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 12:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 13:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 14:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 15:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationApplicationItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationApplicationItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = message.getApplication();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.NotificationApplicationItem.Application.serializeBinaryToWriter\n    );\n  }\n  f = message.getApplicationId();\n  if (f.length > 0) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = message.getInitializationArgs();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.ubo.v1.NotificationApplicationItem.InitializationArgs.serializeBinaryToWriter\n    );\n  }\n  f = message.getInitializationKwargs();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeString(\n      9,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeString(\n      10,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeString(\n      11,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 12));\n  if (f != null) {\n    writer.writeString(\n      12,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 13));\n  if (f != null) {\n    writer.writeBool(\n      13,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 14));\n  if (f != null) {\n    writer.writeFloat(\n      14,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 15));\n  if (f != null) {\n    writer.writeFloat(\n      15,\n      f\n    );\n  }\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_ = [[1]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase = {\n  APPLICATION_NOT_SET: 0,\n  STRING: 1\n};\n\n/**\n * @return {proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.getApplicationCase = function() {\n  return /** @type {proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationApplicationItem.Application.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationApplicationItem.Application} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.Application.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    string: jspb.Message.getFieldWithDefault(msg, 1, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationApplicationItem.Application}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationApplicationItem.Application;\n  return proto.ubo.v1.NotificationApplicationItem.Application.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationApplicationItem.Application} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationApplicationItem.Application}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setString(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationApplicationItem.Application.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationApplicationItem.Application} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.Application.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string string = 1;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.getString = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem.Application} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.setString = function(value) {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem.Application} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.clearString = function() {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.Application.prototype.hasString = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationApplicationItem.InitializationArgs.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.BasicType.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationApplicationItem.InitializationArgs;\n  return proto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.BasicType;\n      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationApplicationItem.InitializationArgs.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.BasicType.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated BasicType items = 1;\n * @return {!Array<!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.BasicType>} value\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} returns this\n*/\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.BasicType=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.BasicType}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.BasicType.toObject) : []\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict;\n  return proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = msg.getItemsMap();\n      reader.readMessage(value, function(message, reader) {\n        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.BasicType.deserializeBinaryFromReader, \"\", new proto.ubo.v1.BasicType());\n         });\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsMap(true);\n  if (f && f.getLength() > 0) {\n    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.BasicType.serializeBinaryToWriter);\n  }\n};\n\n\n/**\n * map<string, BasicType> items = 1;\n * @param {boolean=} opt_noLazyCreate Do not create the map if\n * empty, instead returning `undefined`\n * @return {!jspb.Map<string,!proto.ubo.v1.BasicType>}\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {\n  return /** @type {!jspb.Map<string,!proto.ubo.v1.BasicType>} */ (\n      jspb.Message.getMapField(this, 1, opt_noLazyCreate,\n      proto.ubo.v1.BasicType));\n};\n\n\n/**\n * Clears values from the map. The map will be non-null.\n * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.clearItemsMap = function() {\n  this.getItemsMap().clear();\n  return this;};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string background_color = 2;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool dismiss_notification = 3;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getDismissNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setDismissNotification = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearDismissNotification = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasDismissNotification = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool close_notification = 4;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getCloseNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setCloseNotification = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearCloseNotification = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasCloseNotification = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional Application application = 5;\n * @return {?proto.ubo.v1.NotificationApplicationItem.Application}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getApplication = function() {\n  return /** @type{?proto.ubo.v1.NotificationApplicationItem.Application} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem.Application, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationApplicationItem.Application|undefined} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n*/\nproto.ubo.v1.NotificationApplicationItem.prototype.setApplication = function(value) {\n  return jspb.Message.setWrapperField(this, 5, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearApplication = function() {\n  return this.setApplication(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasApplication = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string application_id = 6;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getApplicationId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setApplicationId = function(value) {\n  return jspb.Message.setProto3StringField(this, 6, value);\n};\n\n\n/**\n * optional InitializationArgs initialization_args = 7;\n * @return {?proto.ubo.v1.NotificationApplicationItem.InitializationArgs}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getInitializationArgs = function() {\n  return /** @type{?proto.ubo.v1.NotificationApplicationItem.InitializationArgs} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem.InitializationArgs, 7));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationApplicationItem.InitializationArgs|undefined} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n*/\nproto.ubo.v1.NotificationApplicationItem.prototype.setInitializationArgs = function(value) {\n  return jspb.Message.setWrapperField(this, 7, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearInitializationArgs = function() {\n  return this.setInitializationArgs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasInitializationArgs = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional InitializationKwargsDict initialization_kwargs = 8;\n * @return {?proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getInitializationKwargs = function() {\n  return /** @type{?proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict, 8));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict|undefined} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n*/\nproto.ubo.v1.NotificationApplicationItem.prototype.setInitializationKwargs = function(value) {\n  return jspb.Message.setWrapperField(this, 8, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearInitializationKwargs = function() {\n  return this.setInitializationKwargs(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasInitializationKwargs = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional string key = 9;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional string label = 10;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional string color = 11;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 11, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional string icon = 12;\n * @return {string}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 12, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 12, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional bool is_short = 13;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 13, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 13, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 13, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 13) != null;\n};\n\n\n/**\n * optional float opacity = 14;\n * @return {number}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 14, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 14, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 14, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 14) != null;\n};\n\n\n/**\n * optional float progress = 15;\n * @return {number}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 15, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 15, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationApplicationItem} returns this\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 15, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationApplicationItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 15) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationDispatchItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationDispatchItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationDispatchItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    storeAction: (f = msg.getStoreAction()) && proto.ubo.v1.NotificationDispatchItem.StoreAction.toObject(includeInstance, f),\n    key: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    label: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, \"\"),\n    icon: jspb.Message.getFieldWithDefault(msg, 7, \"\"),\n    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),\n    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0),\n    dismissNotification: jspb.Message.getBooleanFieldWithDefault(msg, 11, false),\n    closeNotification: jspb.Message.getBooleanFieldWithDefault(msg, 12, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationDispatchItem}\n */\nproto.ubo.v1.NotificationDispatchItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationDispatchItem;\n  return proto.ubo.v1.NotificationDispatchItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationDispatchItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationDispatchItem}\n */\nproto.ubo.v1.NotificationDispatchItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.NotificationDispatchItem.StoreAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinaryFromReader);\n      msg.setStoreAction(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setKey(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setLabel(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 6:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setBackgroundColor(value);\n      break;\n    case 7:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 8:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsShort(value);\n      break;\n    case 9:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setOpacity(value);\n      break;\n    case 10:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    case 11:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDismissNotification(value);\n      break;\n    case 12:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setCloseNotification(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationDispatchItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationDispatchItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationDispatchItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getStoreAction();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.NotificationDispatchItem.StoreAction.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeString(\n      6,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeString(\n      7,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeBool(\n      8,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeFloat(\n      9,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeFloat(\n      10,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));\n  if (f != null) {\n    writer.writeBool(\n      11,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 12));\n  if (f != null) {\n    writer.writeBool(\n      12,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationDispatchItem.StoreAction2.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Action.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationDispatchItem.StoreAction2;\n  return proto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.Action;\n      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationDispatchItem.StoreAction2.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated Action items = 1;\n * @return {!Array<!proto.ubo.v1.Action>}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Action>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Action>} value\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} returns this\n*/\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Action=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Action}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Action, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase = {\n  STORE_ACTION_NOT_SET: 0,\n  UBO_ACTION: 1,\n  LIST: 2\n};\n\n/**\n * @return {proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.getStoreActionCase = function() {\n  return /** @type {proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationDispatchItem.StoreAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    uboAction: (f = msg.getUboAction()) && proto.ubo.v1.Action.toObject(includeInstance, f),\n    list: (f = msg.getList()) && proto.ubo.v1.NotificationDispatchItem.StoreAction2.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationDispatchItem.StoreAction;\n  return proto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.Action;\n      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);\n      msg.setUboAction(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.NotificationDispatchItem.StoreAction2;\n      reader.readMessage(value,proto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinaryFromReader);\n      msg.setList(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationDispatchItem.StoreAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getUboAction();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.Action.serializeBinaryToWriter\n    );\n  }\n  f = message.getList();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.NotificationDispatchItem.StoreAction2.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional Action ubo_action = 1;\n * @return {?proto.ubo.v1.Action}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.getUboAction = function() {\n  return /** @type{?proto.ubo.v1.Action} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Action, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Action|undefined} value\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this\n*/\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.setUboAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.clearUboAction = function() {\n  return this.setUboAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.hasUboAction = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional StoreAction2 list = 2;\n * @return {?proto.ubo.v1.NotificationDispatchItem.StoreAction2}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.getList = function() {\n  return /** @type{?proto.ubo.v1.NotificationDispatchItem.StoreAction2} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationDispatchItem.StoreAction2, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationDispatchItem.StoreAction2|undefined} value\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this\n*/\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.setList = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.clearList = function() {\n  return this.setList(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.hasList = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional StoreAction store_action = 2;\n * @return {?proto.ubo.v1.NotificationDispatchItem.StoreAction}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getStoreAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationDispatchItem.StoreAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationDispatchItem.StoreAction, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationDispatchItem.StoreAction|undefined} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n*/\nproto.ubo.v1.NotificationDispatchItem.prototype.setStoreAction = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearStoreAction = function() {\n  return this.setStoreAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasStoreAction = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string key = 3;\n * @return {string}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getKey = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setKey = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearKey = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasKey = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string label = 4;\n * @return {string}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getLabel = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setLabel = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearLabel = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasLabel = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string color = 5;\n * @return {string}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional string background_color = 6;\n * @return {string}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getBackgroundColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setBackgroundColor = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearBackgroundColor = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasBackgroundColor = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional string icon = 7;\n * @return {string}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional bool is_short = 8;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getIsShort = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setIsShort = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearIsShort = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasIsShort = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional float opacity = 9;\n * @return {number}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getOpacity = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setOpacity = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearOpacity = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasOpacity = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional float progress = 10;\n * @return {number}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional bool dismiss_notification = 11;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getDismissNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setDismissNotification = function(value) {\n  return jspb.Message.setField(this, 11, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearDismissNotification = function() {\n  return jspb.Message.setField(this, 11, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasDismissNotification = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional bool close_notification = 12;\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.getCloseNotification = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 12, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.setCloseNotification = function(value) {\n  return jspb.Message.setField(this, 12, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationDispatchItem} returns this\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.clearCloseNotification = function() {\n  return jspb.Message.setField(this, 12, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationDispatchItem.prototype.hasCloseNotification = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Notification.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Notification.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Notification} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    title: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    content: jspb.Message.getFieldWithDefault(msg, 4, \"\"),\n    extraInformation: (f = msg.getExtraInformation()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),\n    importance: jspb.Message.getFieldWithDefault(msg, 6, 0),\n    chime: jspb.Message.getFieldWithDefault(msg, 7, 0),\n    timestamp: jspb.Message.getFieldWithDefault(msg, 8, 0),\n    isRead: jspb.Message.getBooleanFieldWithDefault(msg, 9, false),\n    sender: jspb.Message.getFieldWithDefault(msg, 10, \"\"),\n    actions: (f = msg.getActions()) && proto.ubo.v1.Notification.Actions.toObject(includeInstance, f),\n    icon: jspb.Message.getFieldWithDefault(msg, 12, \"\"),\n    color: jspb.Message.getFieldWithDefault(msg, 13, \"\"),\n    expirationTimestamp: jspb.Message.getFieldWithDefault(msg, 14, 0),\n    displayType: jspb.Message.getFieldWithDefault(msg, 15, 0),\n    flashTime: jspb.Message.getFloatingPointFieldWithDefault(msg, 16, 0.0),\n    showDismissAction: jspb.Message.getBooleanFieldWithDefault(msg, 17, false),\n    dismissOnClose: jspb.Message.getBooleanFieldWithDefault(msg, 18, false),\n    onClose: (f = msg.getOnClose()) && proto.ubo.v1.Notification.OnClose.toObject(includeInstance, f),\n    blink: jspb.Message.getBooleanFieldWithDefault(msg, 20, false),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 21, 0.0),\n    progressWeight: jspb.Message.getFloatingPointFieldWithDefault(msg, 22, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Notification}\n */\nproto.ubo.v1.Notification.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Notification;\n  return proto.ubo.v1.Notification.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Notification} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Notification}\n */\nproto.ubo.v1.Notification.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setTitle(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setContent(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.ReadableInformation;\n      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);\n      msg.setExtraInformation(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.ubo.v1.Importance} */ (reader.readEnum());\n      msg.setImportance(value);\n      break;\n    case 7:\n      var value = /** @type {!proto.ubo.v1.Chime} */ (reader.readEnum());\n      msg.setChime(value);\n      break;\n    case 8:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setTimestamp(value);\n      break;\n    case 9:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsRead(value);\n      break;\n    case 10:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSender(value);\n      break;\n    case 11:\n      var value = new proto.ubo.v1.Notification.Actions;\n      reader.readMessage(value,proto.ubo.v1.Notification.Actions.deserializeBinaryFromReader);\n      msg.setActions(value);\n      break;\n    case 12:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setIcon(value);\n      break;\n    case 13:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setColor(value);\n      break;\n    case 14:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setExpirationTimestamp(value);\n      break;\n    case 15:\n      var value = /** @type {!proto.ubo.v1.NotificationDisplayType} */ (reader.readEnum());\n      msg.setDisplayType(value);\n      break;\n    case 16:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setFlashTime(value);\n      break;\n    case 17:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setShowDismissAction(value);\n      break;\n    case 18:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setDismissOnClose(value);\n      break;\n    case 19:\n      var value = new proto.ubo.v1.Notification.OnClose;\n      reader.readMessage(value,proto.ubo.v1.Notification.OnClose.deserializeBinaryFromReader);\n      msg.setOnClose(value);\n      break;\n    case 20:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setBlink(value);\n      break;\n    case 21:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    case 22:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgressWeight(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Notification.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Notification.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Notification} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getTitle();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getContent();\n  if (f.length > 0) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n  f = message.getExtraInformation();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {!proto.ubo.v1.Importance} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.Chime} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeEnum(\n      7,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 8));\n  if (f != null) {\n    writer.writeInt64(\n      8,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));\n  if (f != null) {\n    writer.writeBool(\n      9,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 10));\n  if (f != null) {\n    writer.writeString(\n      10,\n      f\n    );\n  }\n  f = message.getActions();\n  if (f != null) {\n    writer.writeMessage(\n      11,\n      f,\n      proto.ubo.v1.Notification.Actions.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 12));\n  if (f != null) {\n    writer.writeString(\n      12,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 13));\n  if (f != null) {\n    writer.writeString(\n      13,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 14));\n  if (f != null) {\n    writer.writeInt64(\n      14,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.NotificationDisplayType} */ (jspb.Message.getField(message, 15));\n  if (f != null) {\n    writer.writeEnum(\n      15,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 16));\n  if (f != null) {\n    writer.writeFloat(\n      16,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 17));\n  if (f != null) {\n    writer.writeBool(\n      17,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 18));\n  if (f != null) {\n    writer.writeBool(\n      18,\n      f\n    );\n  }\n  f = message.getOnClose();\n  if (f != null) {\n    writer.writeMessage(\n      19,\n      f,\n      proto.ubo.v1.Notification.OnClose.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 20));\n  if (f != null) {\n    writer.writeBool(\n      20,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 21));\n  if (f != null) {\n    writer.writeFloat(\n      21,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 22));\n  if (f != null) {\n    writer.writeFloat(\n      22,\n      f\n    );\n  }\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.Notification.ActionsItem.oneofGroups_ = [[1,2,3]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Notification.ActionsItem.ActionsItemCase = {\n  ACTIONS_ITEM_NOT_SET: 0,\n  NOTIFICATION_ACTION_ITEM: 1,\n  NOTIFICATION_APPLICATION_ITEM: 2,\n  NOTIFICATION_DISPATCH_ITEM: 3\n};\n\n/**\n * @return {proto.ubo.v1.Notification.ActionsItem.ActionsItemCase}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.getActionsItemCase = function() {\n  return /** @type {proto.ubo.v1.Notification.ActionsItem.ActionsItemCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Notification.ActionsItem.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Notification.ActionsItem} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.ActionsItem.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    notificationActionItem: (f = msg.getNotificationActionItem()) && proto.ubo.v1.NotificationActionItem.toObject(includeInstance, f),\n    notificationApplicationItem: (f = msg.getNotificationApplicationItem()) && proto.ubo.v1.NotificationApplicationItem.toObject(includeInstance, f),\n    notificationDispatchItem: (f = msg.getNotificationDispatchItem()) && proto.ubo.v1.NotificationDispatchItem.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Notification.ActionsItem}\n */\nproto.ubo.v1.Notification.ActionsItem.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Notification.ActionsItem;\n  return proto.ubo.v1.Notification.ActionsItem.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Notification.ActionsItem} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Notification.ActionsItem}\n */\nproto.ubo.v1.Notification.ActionsItem.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.NotificationActionItem;\n      reader.readMessage(value,proto.ubo.v1.NotificationActionItem.deserializeBinaryFromReader);\n      msg.setNotificationActionItem(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.NotificationApplicationItem;\n      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.deserializeBinaryFromReader);\n      msg.setNotificationApplicationItem(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.NotificationDispatchItem;\n      reader.readMessage(value,proto.ubo.v1.NotificationDispatchItem.deserializeBinaryFromReader);\n      msg.setNotificationDispatchItem(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Notification.ActionsItem.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Notification.ActionsItem} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.ActionsItem.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getNotificationActionItem();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.NotificationActionItem.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationApplicationItem();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.NotificationApplicationItem.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationDispatchItem();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.NotificationDispatchItem.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional NotificationActionItem notification_action_item = 1;\n * @return {?proto.ubo.v1.NotificationActionItem}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.getNotificationActionItem = function() {\n  return /** @type{?proto.ubo.v1.NotificationActionItem} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationActionItem, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationActionItem|undefined} value\n * @return {!proto.ubo.v1.Notification.ActionsItem} returns this\n*/\nproto.ubo.v1.Notification.ActionsItem.prototype.setNotificationActionItem = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Notification.ActionsItem} returns this\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.clearNotificationActionItem = function() {\n  return this.setNotificationActionItem(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.hasNotificationActionItem = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional NotificationApplicationItem notification_application_item = 2;\n * @return {?proto.ubo.v1.NotificationApplicationItem}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.getNotificationApplicationItem = function() {\n  return /** @type{?proto.ubo.v1.NotificationApplicationItem} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationApplicationItem|undefined} value\n * @return {!proto.ubo.v1.Notification.ActionsItem} returns this\n*/\nproto.ubo.v1.Notification.ActionsItem.prototype.setNotificationApplicationItem = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Notification.ActionsItem} returns this\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.clearNotificationApplicationItem = function() {\n  return this.setNotificationApplicationItem(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.hasNotificationApplicationItem = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional NotificationDispatchItem notification_dispatch_item = 3;\n * @return {?proto.ubo.v1.NotificationDispatchItem}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.getNotificationDispatchItem = function() {\n  return /** @type{?proto.ubo.v1.NotificationDispatchItem} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationDispatchItem, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationDispatchItem|undefined} value\n * @return {!proto.ubo.v1.Notification.ActionsItem} returns this\n*/\nproto.ubo.v1.Notification.ActionsItem.prototype.setNotificationDispatchItem = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Notification.ActionsItem} returns this\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.clearNotificationDispatchItem = function() {\n  return this.setNotificationDispatchItem(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.ActionsItem.prototype.hasNotificationDispatchItem = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.Notification.Actions.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Notification.Actions.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Notification.Actions.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Notification.Actions} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.Actions.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.Notification.ActionsItem.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Notification.Actions}\n */\nproto.ubo.v1.Notification.Actions.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Notification.Actions;\n  return proto.ubo.v1.Notification.Actions.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Notification.Actions} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Notification.Actions}\n */\nproto.ubo.v1.Notification.Actions.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.Notification.ActionsItem;\n      reader.readMessage(value,proto.ubo.v1.Notification.ActionsItem.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Notification.Actions.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Notification.Actions.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Notification.Actions} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.Actions.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.Notification.ActionsItem.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated ActionsItem items = 1;\n * @return {!Array<!proto.ubo.v1.Notification.ActionsItem>}\n */\nproto.ubo.v1.Notification.Actions.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Notification.ActionsItem>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Notification.ActionsItem, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Notification.ActionsItem>} value\n * @return {!proto.ubo.v1.Notification.Actions} returns this\n*/\nproto.ubo.v1.Notification.Actions.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Notification.ActionsItem=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Notification.ActionsItem}\n */\nproto.ubo.v1.Notification.Actions.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Notification.ActionsItem, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.Notification.Actions} returns this\n */\nproto.ubo.v1.Notification.Actions.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Notification.OnClose.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Notification.OnClose.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Notification.OnClose} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.OnClose.toObject = function(includeInstance, msg) {\n  var f, obj = {\n\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Notification.OnClose}\n */\nproto.ubo.v1.Notification.OnClose.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Notification.OnClose;\n  return proto.ubo.v1.Notification.OnClose.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Notification.OnClose} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Notification.OnClose}\n */\nproto.ubo.v1.Notification.OnClose.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Notification.OnClose.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Notification.OnClose.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Notification.OnClose} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Notification.OnClose.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setId = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearId = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasId = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string title = 3;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getTitle = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setTitle = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional string content = 4;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getContent = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setContent = function(value) {\n  return jspb.Message.setProto3StringField(this, 4, value);\n};\n\n\n/**\n * optional ReadableInformation extra_information = 5;\n * @return {?proto.ubo.v1.ReadableInformation}\n */\nproto.ubo.v1.Notification.prototype.getExtraInformation = function() {\n  return /** @type{?proto.ubo.v1.ReadableInformation} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReadableInformation|undefined} value\n * @return {!proto.ubo.v1.Notification} returns this\n*/\nproto.ubo.v1.Notification.prototype.setExtraInformation = function(value) {\n  return jspb.Message.setWrapperField(this, 5, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearExtraInformation = function() {\n  return this.setExtraInformation(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasExtraInformation = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional Importance importance = 6;\n * @return {!proto.ubo.v1.Importance}\n */\nproto.ubo.v1.Notification.prototype.getImportance = function() {\n  return /** @type {!proto.ubo.v1.Importance} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Importance} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setImportance = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearImportance = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasImportance = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional Chime chime = 7;\n * @return {!proto.ubo.v1.Chime}\n */\nproto.ubo.v1.Notification.prototype.getChime = function() {\n  return /** @type {!proto.ubo.v1.Chime} */ (jspb.Message.getFieldWithDefault(this, 7, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Chime} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setChime = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearChime = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasChime = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional int64 timestamp = 8;\n * @return {number}\n */\nproto.ubo.v1.Notification.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 8, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setTimestamp = function(value) {\n  return jspb.Message.setField(this, 8, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearTimestamp = function() {\n  return jspb.Message.setField(this, 8, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasTimestamp = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional bool is_read = 9;\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.getIsRead = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setIsRead = function(value) {\n  return jspb.Message.setField(this, 9, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearIsRead = function() {\n  return jspb.Message.setField(this, 9, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasIsRead = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional string sender = 10;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getSender = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setSender = function(value) {\n  return jspb.Message.setField(this, 10, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearSender = function() {\n  return jspb.Message.setField(this, 10, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasSender = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional Actions actions = 11;\n * @return {?proto.ubo.v1.Notification.Actions}\n */\nproto.ubo.v1.Notification.prototype.getActions = function() {\n  return /** @type{?proto.ubo.v1.Notification.Actions} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification.Actions, 11));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification.Actions|undefined} value\n * @return {!proto.ubo.v1.Notification} returns this\n*/\nproto.ubo.v1.Notification.prototype.setActions = function(value) {\n  return jspb.Message.setWrapperField(this, 11, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearActions = function() {\n  return this.setActions(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasActions = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional string icon = 12;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getIcon = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setIcon = function(value) {\n  return jspb.Message.setField(this, 12, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearIcon = function() {\n  return jspb.Message.setField(this, 12, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasIcon = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional string color = 13;\n * @return {string}\n */\nproto.ubo.v1.Notification.prototype.getColor = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 13, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setColor = function(value) {\n  return jspb.Message.setField(this, 13, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearColor = function() {\n  return jspb.Message.setField(this, 13, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 13) != null;\n};\n\n\n/**\n * optional int64 expiration_timestamp = 14;\n * @return {number}\n */\nproto.ubo.v1.Notification.prototype.getExpirationTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 14, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setExpirationTimestamp = function(value) {\n  return jspb.Message.setField(this, 14, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearExpirationTimestamp = function() {\n  return jspb.Message.setField(this, 14, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasExpirationTimestamp = function() {\n  return jspb.Message.getField(this, 14) != null;\n};\n\n\n/**\n * optional NotificationDisplayType display_type = 15;\n * @return {!proto.ubo.v1.NotificationDisplayType}\n */\nproto.ubo.v1.Notification.prototype.getDisplayType = function() {\n  return /** @type {!proto.ubo.v1.NotificationDisplayType} */ (jspb.Message.getFieldWithDefault(this, 15, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.NotificationDisplayType} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setDisplayType = function(value) {\n  return jspb.Message.setField(this, 15, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearDisplayType = function() {\n  return jspb.Message.setField(this, 15, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasDisplayType = function() {\n  return jspb.Message.getField(this, 15) != null;\n};\n\n\n/**\n * optional float flash_time = 16;\n * @return {number}\n */\nproto.ubo.v1.Notification.prototype.getFlashTime = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 16, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setFlashTime = function(value) {\n  return jspb.Message.setField(this, 16, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearFlashTime = function() {\n  return jspb.Message.setField(this, 16, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasFlashTime = function() {\n  return jspb.Message.getField(this, 16) != null;\n};\n\n\n/**\n * optional bool show_dismiss_action = 17;\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.getShowDismissAction = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 17, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setShowDismissAction = function(value) {\n  return jspb.Message.setField(this, 17, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearShowDismissAction = function() {\n  return jspb.Message.setField(this, 17, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasShowDismissAction = function() {\n  return jspb.Message.getField(this, 17) != null;\n};\n\n\n/**\n * optional bool dismiss_on_close = 18;\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.getDismissOnClose = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 18, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setDismissOnClose = function(value) {\n  return jspb.Message.setField(this, 18, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearDismissOnClose = function() {\n  return jspb.Message.setField(this, 18, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasDismissOnClose = function() {\n  return jspb.Message.getField(this, 18) != null;\n};\n\n\n/**\n * optional OnClose on_close = 19;\n * @return {?proto.ubo.v1.Notification.OnClose}\n */\nproto.ubo.v1.Notification.prototype.getOnClose = function() {\n  return /** @type{?proto.ubo.v1.Notification.OnClose} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification.OnClose, 19));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification.OnClose|undefined} value\n * @return {!proto.ubo.v1.Notification} returns this\n*/\nproto.ubo.v1.Notification.prototype.setOnClose = function(value) {\n  return jspb.Message.setWrapperField(this, 19, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearOnClose = function() {\n  return this.setOnClose(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasOnClose = function() {\n  return jspb.Message.getField(this, 19) != null;\n};\n\n\n/**\n * optional bool blink = 20;\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.getBlink = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 20, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setBlink = function(value) {\n  return jspb.Message.setField(this, 20, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearBlink = function() {\n  return jspb.Message.setField(this, 20, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasBlink = function() {\n  return jspb.Message.getField(this, 20) != null;\n};\n\n\n/**\n * optional float progress = 21;\n * @return {number}\n */\nproto.ubo.v1.Notification.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 21, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 21, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 21, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 21) != null;\n};\n\n\n/**\n * optional float progress_weight = 22;\n * @return {number}\n */\nproto.ubo.v1.Notification.prototype.getProgressWeight = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 22, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.setProgressWeight = function(value) {\n  return jspb.Message.setField(this, 22, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.Notification} returns this\n */\nproto.ubo.v1.Notification.prototype.clearProgressWeight = function() {\n  return jspb.Message.setField(this, 22, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Notification.prototype.hasProgressWeight = function() {\n  return jspb.Message.getField(this, 22) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsAction}\n */\nproto.ubo.v1.NotificationsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsAction;\n  return proto.ubo.v1.NotificationsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsAction}\n */\nproto.ubo.v1.NotificationsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsAction} returns this\n */\nproto.ubo.v1.NotificationsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsAction} returns this\n */\nproto.ubo.v1.NotificationsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsAddAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsAddAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsAddAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsAddAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsAddAction}\n */\nproto.ubo.v1.NotificationsAddAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsAddAction;\n  return proto.ubo.v1.NotificationsAddAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsAddAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsAddAction}\n */\nproto.ubo.v1.NotificationsAddAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Notification;\n      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);\n      msg.setNotification(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsAddAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsAddAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsAddAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsAddAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getNotification();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Notification.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsAddAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsAddAction} returns this\n */\nproto.ubo.v1.NotificationsAddAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsAddAction} returns this\n */\nproto.ubo.v1.NotificationsAddAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsAddAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Notification notification = 2;\n * @return {?proto.ubo.v1.Notification}\n */\nproto.ubo.v1.NotificationsAddAction.prototype.getNotification = function() {\n  return /** @type{?proto.ubo.v1.Notification} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification|undefined} value\n * @return {!proto.ubo.v1.NotificationsAddAction} returns this\n*/\nproto.ubo.v1.NotificationsAddAction.prototype.setNotification = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationsAddAction} returns this\n */\nproto.ubo.v1.NotificationsAddAction.prototype.clearNotification = function() {\n  return this.setNotification(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsAddAction.prototype.hasNotification = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsDisplayAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsDisplayAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsDisplayAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f),\n    index: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    count: jspb.Message.getFieldWithDefault(msg, 4, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsDisplayAction}\n */\nproto.ubo.v1.NotificationsDisplayAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsDisplayAction;\n  return proto.ubo.v1.NotificationsDisplayAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsDisplayAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsDisplayAction}\n */\nproto.ubo.v1.NotificationsDisplayAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Notification;\n      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);\n      msg.setNotification(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setCount(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsDisplayAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsDisplayAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsDisplayAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getNotification();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Notification.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Notification notification = 2;\n * @return {?proto.ubo.v1.Notification}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.getNotification = function() {\n  return /** @type{?proto.ubo.v1.Notification} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification|undefined} value\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n*/\nproto.ubo.v1.NotificationsDisplayAction.prototype.setNotification = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.clearNotification = function() {\n  return this.setNotification(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.hasNotification = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 index = 3;\n * @return {number}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.setIndex = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.clearIndex = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.hasIndex = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 count = 4;\n * @return {number}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.getCount = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.setCount = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.clearCount = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayAction.prototype.hasCount = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsClearAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsClearAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsClearAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsClearAction}\n */\nproto.ubo.v1.NotificationsClearAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsClearAction;\n  return proto.ubo.v1.NotificationsClearAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsClearAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsClearAction}\n */\nproto.ubo.v1.NotificationsClearAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Notification;\n      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);\n      msg.setNotification(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsClearAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsClearAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsClearAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getNotification();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Notification.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsClearAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsClearAction} returns this\n */\nproto.ubo.v1.NotificationsClearAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsClearAction} returns this\n */\nproto.ubo.v1.NotificationsClearAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsClearAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Notification notification = 2;\n * @return {?proto.ubo.v1.Notification}\n */\nproto.ubo.v1.NotificationsClearAction.prototype.getNotification = function() {\n  return /** @type{?proto.ubo.v1.Notification} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification|undefined} value\n * @return {!proto.ubo.v1.NotificationsClearAction} returns this\n*/\nproto.ubo.v1.NotificationsClearAction.prototype.setNotification = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationsClearAction} returns this\n */\nproto.ubo.v1.NotificationsClearAction.prototype.clearNotification = function() {\n  return this.setNotification(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsClearAction.prototype.hasNotification = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsClearByIdAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsClearByIdAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearByIdAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsClearByIdAction}\n */\nproto.ubo.v1.NotificationsClearByIdAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsClearByIdAction;\n  return proto.ubo.v1.NotificationsClearByIdAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsClearByIdAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsClearByIdAction}\n */\nproto.ubo.v1.NotificationsClearByIdAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsClearByIdAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsClearByIdAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearByIdAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsClearByIdAction} returns this\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsClearByIdAction} returns this\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsClearByIdAction} returns this\n */\nproto.ubo.v1.NotificationsClearByIdAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsClearAllAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsClearAllAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsClearAllAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearAllAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsClearAllAction}\n */\nproto.ubo.v1.NotificationsClearAllAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsClearAllAction;\n  return proto.ubo.v1.NotificationsClearAllAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsClearAllAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsClearAllAction}\n */\nproto.ubo.v1.NotificationsClearAllAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsClearAllAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsClearAllAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsClearAllAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearAllAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsClearAllAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsClearAllAction} returns this\n */\nproto.ubo.v1.NotificationsClearAllAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsClearAllAction} returns this\n */\nproto.ubo.v1.NotificationsClearAllAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsClearAllAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsEvent}\n */\nproto.ubo.v1.NotificationsEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsEvent;\n  return proto.ubo.v1.NotificationsEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsEvent}\n */\nproto.ubo.v1.NotificationsEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsEvent} returns this\n */\nproto.ubo.v1.NotificationsEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsEvent} returns this\n */\nproto.ubo.v1.NotificationsEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsClearEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsClearEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsClearEvent}\n */\nproto.ubo.v1.NotificationsClearEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsClearEvent;\n  return proto.ubo.v1.NotificationsClearEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsClearEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsClearEvent}\n */\nproto.ubo.v1.NotificationsClearEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Notification;\n      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);\n      msg.setNotification(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsClearEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsClearEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsClearEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getNotification();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Notification.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsClearEvent} returns this\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsClearEvent} returns this\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Notification notification = 2;\n * @return {?proto.ubo.v1.Notification}\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.getNotification = function() {\n  return /** @type{?proto.ubo.v1.Notification} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification|undefined} value\n * @return {!proto.ubo.v1.NotificationsClearEvent} returns this\n*/\nproto.ubo.v1.NotificationsClearEvent.prototype.setNotification = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationsClearEvent} returns this\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.clearNotification = function() {\n  return this.setNotification(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsClearEvent.prototype.hasNotification = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsDisplayEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsDisplayEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsDisplayEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f),\n    index: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    count: jspb.Message.getFieldWithDefault(msg, 4, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsDisplayEvent}\n */\nproto.ubo.v1.NotificationsDisplayEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsDisplayEvent;\n  return proto.ubo.v1.NotificationsDisplayEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsDisplayEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsDisplayEvent}\n */\nproto.ubo.v1.NotificationsDisplayEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Notification;\n      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);\n      msg.setNotification(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setIndex(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setCount(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsDisplayEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsDisplayEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsDisplayEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getNotification();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.Notification.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Notification notification = 2;\n * @return {?proto.ubo.v1.Notification}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.getNotification = function() {\n  return /** @type{?proto.ubo.v1.Notification} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.Notification|undefined} value\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n*/\nproto.ubo.v1.NotificationsDisplayEvent.prototype.setNotification = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.clearNotification = function() {\n  return this.setNotification(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.hasNotification = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 index = 3;\n * @return {number}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.getIndex = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.setIndex = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.clearIndex = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.hasIndex = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 count = 4;\n * @return {number}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.getCount = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.setCount = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.clearCount = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsDisplayEvent.prototype.hasCount = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.NotificationsState.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.NotificationsState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.NotificationsState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.NotificationsState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    notificationsList: jspb.Message.toObjectList(msg.getNotificationsList(),\n    proto.ubo.v1.Notification.toObject, includeInstance),\n    unreadCount: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.NotificationsState}\n */\nproto.ubo.v1.NotificationsState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.NotificationsState;\n  return proto.ubo.v1.NotificationsState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.NotificationsState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.NotificationsState}\n */\nproto.ubo.v1.NotificationsState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.Notification;\n      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);\n      msg.addNotifications(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setUnreadCount(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setProgress(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.NotificationsState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.NotificationsState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.NotificationsState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.NotificationsState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getNotificationsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.Notification.serializeBinaryToWriter\n    );\n  }\n  f = message.getUnreadCount();\n  if (f !== 0) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeFloat(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;\n * @return {string}\n */\nproto.ubo.v1.NotificationsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.NotificationsState} returns this\n */\nproto.ubo.v1.NotificationsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsState} returns this\n */\nproto.ubo.v1.NotificationsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated Notification notifications = 2;\n * @return {!Array<!proto.ubo.v1.Notification>}\n */\nproto.ubo.v1.NotificationsState.prototype.getNotificationsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.Notification>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Notification, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.Notification>} value\n * @return {!proto.ubo.v1.NotificationsState} returns this\n*/\nproto.ubo.v1.NotificationsState.prototype.setNotificationsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.Notification=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.Notification}\n */\nproto.ubo.v1.NotificationsState.prototype.addNotifications = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.Notification, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.NotificationsState} returns this\n */\nproto.ubo.v1.NotificationsState.prototype.clearNotificationsList = function() {\n  return this.setNotificationsList([]);\n};\n\n\n/**\n * optional int64 unread_count = 3;\n * @return {number}\n */\nproto.ubo.v1.NotificationsState.prototype.getUnreadCount = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationsState} returns this\n */\nproto.ubo.v1.NotificationsState.prototype.setUnreadCount = function(value) {\n  return jspb.Message.setProto3IntField(this, 3, value);\n};\n\n\n/**\n * optional float progress = 4;\n * @return {number}\n */\nproto.ubo.v1.NotificationsState.prototype.getProgress = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.NotificationsState} returns this\n */\nproto.ubo.v1.NotificationsState.prototype.setProgress = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.NotificationsState} returns this\n */\nproto.ubo.v1.NotificationsState.prototype.clearProgress = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.NotificationsState.prototype.hasProgress = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingAction}\n */\nproto.ubo.v1.RgbRingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingAction;\n  return proto.ubo.v1.RgbRingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingAction}\n */\nproto.ubo.v1.RgbRingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingAction} returns this\n */\nproto.ubo.v1.RgbRingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingAction} returns this\n */\nproto.ubo.v1.RgbRingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingEvent}\n */\nproto.ubo.v1.RgbRingEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingEvent;\n  return proto.ubo.v1.RgbRingEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingEvent}\n */\nproto.ubo.v1.RgbRingEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingEvent} returns this\n */\nproto.ubo.v1.RgbRingEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingEvent} returns this\n */\nproto.ubo.v1.RgbRingEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingSetIsBusyAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingSetIsBusyAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isBusy: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingSetIsBusyAction}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingSetIsBusyAction;\n  return proto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingSetIsBusyAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingSetIsBusyAction}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsBusy(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingSetIsBusyAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingSetIsBusyAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_busy = 2;\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.getIsBusy = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.setIsBusy = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.clearIsBusy = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetIsBusyAction.prototype.hasIsBusy = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingCommandAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingCommandAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingCommandAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingCommandAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingCommandAction}\n */\nproto.ubo.v1.RgbRingCommandAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingCommandAction;\n  return proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingCommandAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingCommandAction}\n */\nproto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingCommandAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingCommandAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingCommandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingCommandAction} returns this\n */\nproto.ubo.v1.RgbRingCommandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingCommandAction} returns this\n */\nproto.ubo.v1.RgbRingCommandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingCommandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingWaitableCommandAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingWaitableCommandAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    wait: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingWaitableCommandAction}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingWaitableCommandAction;\n  return proto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingWaitableCommandAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingWaitableCommandAction}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingWaitableCommandAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingWaitableCommandAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 wait = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingWaitableCommandAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingColorfulCommandAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingColorfulCommandAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingColorfulCommandAction}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingColorfulCommandAction;\n  return proto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingColorfulCommandAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingColorfulCommandAction}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingColorfulCommandAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingColorfulCommandAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional RgbColor color = 2;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this\n*/\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingColorfulCommandAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingSetEnabledAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingSetEnabledAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetEnabledAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    enabled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingSetEnabledAction}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingSetEnabledAction;\n  return proto.ubo.v1.RgbRingSetEnabledAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingSetEnabledAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingSetEnabledAction}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setEnabled(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingSetEnabledAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingSetEnabledAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetEnabledAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool enabled = 2;\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.getEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.setEnabled = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.clearEnabled = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetEnabledAction.prototype.hasEnabled = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingSetAllAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingSetAllAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetAllAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingSetAllAction}\n */\nproto.ubo.v1.RgbRingSetAllAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingSetAllAction;\n  return proto.ubo.v1.RgbRingSetAllAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingSetAllAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingSetAllAction}\n */\nproto.ubo.v1.RgbRingSetAllAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingSetAllAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingSetAllAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetAllAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional RgbColor color = 2;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this\n*/\nproto.ubo.v1.RgbRingSetAllAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetAllAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingSetBrightnessAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingSetBrightnessAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    brightness: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingSetBrightnessAction}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingSetBrightnessAction;\n  return proto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingSetBrightnessAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingSetBrightnessAction}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setBrightness(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingSetBrightnessAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingSetBrightnessAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float brightness = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.getBrightness = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.setBrightness = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.clearBrightness = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSetBrightnessAction.prototype.hasBrightness = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingBlankAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingBlankAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingBlankAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingBlankAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingBlankAction}\n */\nproto.ubo.v1.RgbRingBlankAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingBlankAction;\n  return proto.ubo.v1.RgbRingBlankAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingBlankAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingBlankAction}\n */\nproto.ubo.v1.RgbRingBlankAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingBlankAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingBlankAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingBlankAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingBlankAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingBlankAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingBlankAction} returns this\n */\nproto.ubo.v1.RgbRingBlankAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingBlankAction} returns this\n */\nproto.ubo.v1.RgbRingBlankAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingBlankAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingRainbowAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingRainbowAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingRainbowAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    wait: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    rounds: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingRainbowAction}\n */\nproto.ubo.v1.RgbRingRainbowAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingRainbowAction;\n  return proto.ubo.v1.RgbRingRainbowAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingRainbowAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingRainbowAction}\n */\nproto.ubo.v1.RgbRingRainbowAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setRounds(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingRainbowAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingRainbowAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingRainbowAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = message.getRounds();\n  if (f !== 0) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 wait = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 rounds = 3;\n * @return {number}\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.getRounds = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this\n */\nproto.ubo.v1.RgbRingRainbowAction.prototype.setRounds = function(value) {\n  return jspb.Message.setProto3IntField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingProgressWheelStepAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingProgressWheelStepAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingProgressWheelStepAction;\n  return proto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingProgressWheelStepAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingProgressWheelStepAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingProgressWheelStepAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional RgbColor color = 2;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this\n*/\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingProgressWheelStepAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingPulseAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingPulseAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingPulseAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    repetitions: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingPulseAction}\n */\nproto.ubo.v1.RgbRingPulseAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingPulseAction;\n  return proto.ubo.v1.RgbRingPulseAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingPulseAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingPulseAction}\n */\nproto.ubo.v1.RgbRingPulseAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setRepetitions(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingPulseAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingPulseAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingPulseAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 repetitions = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.getRepetitions = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.setRepetitions = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.clearRepetitions = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.hasRepetitions = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 wait = 3;\n * @return {number}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional RgbColor color = 4;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n*/\nproto.ubo.v1.RgbRingPulseAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingPulseAction} returns this\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingPulseAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingBlinkAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingBlinkAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingBlinkAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    repetitions: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingBlinkAction}\n */\nproto.ubo.v1.RgbRingBlinkAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingBlinkAction;\n  return proto.ubo.v1.RgbRingBlinkAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingBlinkAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingBlinkAction}\n */\nproto.ubo.v1.RgbRingBlinkAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setRepetitions(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingBlinkAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingBlinkAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingBlinkAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 repetitions = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.getRepetitions = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.setRepetitions = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.clearRepetitions = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.hasRepetitions = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 wait = 3;\n * @return {number}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional RgbColor color = 4;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n*/\nproto.ubo.v1.RgbRingBlinkAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingBlinkAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingSpinningWheelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingSpinningWheelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    length: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    repetitions: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    wait: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingSpinningWheelAction;\n  return proto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingSpinningWheelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setLength(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setRepetitions(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingSpinningWheelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingSpinningWheelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 length = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.getLength = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.setLength = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearLength = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasLength = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 repetitions = 3;\n * @return {number}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.getRepetitions = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.setRepetitions = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearRepetitions = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasRepetitions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 wait = 4;\n * @return {number}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional RgbColor color = 5;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n*/\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 5, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingProgressWheelAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingProgressWheelAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingProgressWheelAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    percentage: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingProgressWheelAction;\n  return proto.ubo.v1.RgbRingProgressWheelAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingProgressWheelAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setPercentage(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingProgressWheelAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingProgressWheelAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingProgressWheelAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float percentage = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.getPercentage = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.setPercentage = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.clearPercentage = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.hasPercentage = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional RgbColor color = 3;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this\n*/\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingProgressWheelAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingFillUptoAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingFillUptoAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingFillUptoAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    percentage: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingFillUptoAction}\n */\nproto.ubo.v1.RgbRingFillUptoAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingFillUptoAction;\n  return proto.ubo.v1.RgbRingFillUptoAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingFillUptoAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingFillUptoAction}\n */\nproto.ubo.v1.RgbRingFillUptoAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setPercentage(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingFillUptoAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingFillUptoAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingFillUptoAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float percentage = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.getPercentage = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.setPercentage = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.clearPercentage = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.hasPercentage = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 wait = 3;\n * @return {number}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional RgbColor color = 4;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n*/\nproto.ubo.v1.RgbRingFillUptoAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillUptoAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingFillDownfromAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingFillDownfromAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingFillDownfromAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    percentage: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingFillDownfromAction;\n  return proto.ubo.v1.RgbRingFillDownfromAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingFillDownfromAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setPercentage(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setWait(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.RgbColor;\n      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);\n      msg.setColor(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingFillDownfromAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingFillDownfromAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingFillDownfromAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n  f = message.getColor();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.RgbColor.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float percentage = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.getPercentage = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.setPercentage = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.clearPercentage = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.hasPercentage = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 wait = 3;\n * @return {number}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.getWait = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.setWait = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.clearWait = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.hasWait = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional RgbColor color = 4;\n * @return {?proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.getColor = function() {\n  return /** @type{?proto.ubo.v1.RgbColor} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbColor|undefined} value\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n*/\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.setColor = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.clearColor = function() {\n  return this.setColor(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingFillDownfromAction.prototype.hasColor = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.RgbRingSequenceAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingSequenceAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingSequenceAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSequenceAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sequenceList: jspb.Message.toObjectList(msg.getSequenceList(),\n    proto.ubo.v1.RgbRingCommandAction.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingSequenceAction}\n */\nproto.ubo.v1.RgbRingSequenceAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingSequenceAction;\n  return proto.ubo.v1.RgbRingSequenceAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingSequenceAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingSequenceAction}\n */\nproto.ubo.v1.RgbRingSequenceAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.RgbRingCommandAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader);\n      msg.addSequence(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingSequenceAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingSequenceAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingSequenceAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSequenceList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated RgbRingCommandAction sequence = 2;\n * @return {!Array<!proto.ubo.v1.RgbRingCommandAction>}\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.getSequenceList = function() {\n  return /** @type{!Array<!proto.ubo.v1.RgbRingCommandAction>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.RgbRingCommandAction, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.RgbRingCommandAction>} value\n * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this\n*/\nproto.ubo.v1.RgbRingSequenceAction.prototype.setSequenceList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.RgbRingCommandAction=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.RgbRingCommandAction}\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.addSequence = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.RgbRingCommandAction, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this\n */\nproto.ubo.v1.RgbRingSequenceAction.prototype.clearSequenceList = function() {\n  return this.setSequenceList([]);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.RgbRingCommandEvent.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingCommandEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingCommandEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingCommandEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    commandList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingCommandEvent}\n */\nproto.ubo.v1.RgbRingCommandEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingCommandEvent;\n  return proto.ubo.v1.RgbRingCommandEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingCommandEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingCommandEvent}\n */\nproto.ubo.v1.RgbRingCommandEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addCommand(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingCommandEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingCommandEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingCommandEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getCommandList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated string command = 2;\n * @return {!Array<string>}\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.getCommandList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.setCommandList = function(value) {\n  return jspb.Message.setField(this, 2, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.addCommand = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this\n */\nproto.ubo.v1.RgbRingCommandEvent.prototype.clearCommandList = function() {\n  return this.setCommandList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbRingState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbRingState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbRingState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isBusy: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbRingState}\n */\nproto.ubo.v1.RgbRingState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbRingState;\n  return proto.ubo.v1.RgbRingState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbRingState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbRingState}\n */\nproto.ubo.v1.RgbRingState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsBusy(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbRingState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbRingState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbRingState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbRingState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsBusy();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;\n * @return {string}\n */\nproto.ubo.v1.RgbRingState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RgbRingState} returns this\n */\nproto.ubo.v1.RgbRingState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbRingState} returns this\n */\nproto.ubo.v1.RgbRingState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_busy = 2;\n * @return {boolean}\n */\nproto.ubo.v1.RgbRingState.prototype.getIsBusy = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RgbRingState} returns this\n */\nproto.ubo.v1.RgbRingState.prototype.setIsBusy = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.RgbColorElement.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.RgbColorElement.RgbColorElementCase = {\n  RGB_COLOR_ELEMENT_NOT_SET: 0,\n  FLOAT: 1,\n  INT64: 2\n};\n\n/**\n * @return {proto.ubo.v1.RgbColorElement.RgbColorElementCase}\n */\nproto.ubo.v1.RgbColorElement.prototype.getRgbColorElementCase = function() {\n  return /** @type {proto.ubo.v1.RgbColorElement.RgbColorElementCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.RgbColorElement.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbColorElement.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbColorElement.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbColorElement} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbColorElement.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    pb_float: jspb.Message.getFloatingPointFieldWithDefault(msg, 1, 0.0),\n    int64: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbColorElement}\n */\nproto.ubo.v1.RgbColorElement.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbColorElement;\n  return proto.ubo.v1.RgbColorElement.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbColorElement} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbColorElement}\n */\nproto.ubo.v1.RgbColorElement.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setFloat(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setInt64(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbColorElement.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbColorElement.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbColorElement} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbColorElement.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {number} */ (jspb.Message.getField(message, 1));\n  if (f != null) {\n    writer.writeFloat(\n      1,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional float float = 1;\n * @return {number}\n */\nproto.ubo.v1.RgbColorElement.prototype.getFloat = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 1, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbColorElement} returns this\n */\nproto.ubo.v1.RgbColorElement.prototype.setFloat = function(value) {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.RgbColorElement.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbColorElement} returns this\n */\nproto.ubo.v1.RgbColorElement.prototype.clearFloat = function() {\n  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.RgbColorElement.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbColorElement.prototype.hasFloat = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional int64 int64 = 2;\n * @return {number}\n */\nproto.ubo.v1.RgbColorElement.prototype.getInt64 = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RgbColorElement} returns this\n */\nproto.ubo.v1.RgbColorElement.prototype.setInt64 = function(value) {\n  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.RgbColorElement.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RgbColorElement} returns this\n */\nproto.ubo.v1.RgbColorElement.prototype.clearInt64 = function() {\n  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.RgbColorElement.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RgbColorElement.prototype.hasInt64 = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.RgbColor.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RgbColor.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RgbColor.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RgbColor} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbColor.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.RgbColorElement.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbColor.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RgbColor;\n  return proto.ubo.v1.RgbColor.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RgbColor} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RgbColor}\n */\nproto.ubo.v1.RgbColor.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.RgbColorElement;\n      reader.readMessage(value,proto.ubo.v1.RgbColorElement.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RgbColor.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RgbColor.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RgbColor} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RgbColor.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.RgbColorElement.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated RgbColorElement items = 1;\n * @return {!Array<!proto.ubo.v1.RgbColorElement>}\n */\nproto.ubo.v1.RgbColor.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.RgbColorElement>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.RgbColorElement, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.RgbColorElement>} value\n * @return {!proto.ubo.v1.RgbColor} returns this\n*/\nproto.ubo.v1.RgbColor.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.RgbColorElement=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.RgbColorElement}\n */\nproto.ubo.v1.RgbColor.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.RgbColorElement, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.RgbColor} returns this\n */\nproto.ubo.v1.RgbColor.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectAction}\n */\nproto.ubo.v1.RPiConnectAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectAction;\n  return proto.ubo.v1.RPiConnectAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectAction}\n */\nproto.ubo.v1.RPiConnectAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectAction} returns this\n */\nproto.ubo.v1.RPiConnectAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectAction} returns this\n */\nproto.ubo.v1.RPiConnectAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectEvent}\n */\nproto.ubo.v1.RPiConnectEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectEvent;\n  return proto.ubo.v1.RPiConnectEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectEvent}\n */\nproto.ubo.v1.RPiConnectEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectEvent} returns this\n */\nproto.ubo.v1.RPiConnectEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectEvent} returns this\n */\nproto.ubo.v1.RPiConnectEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectStartDownloadingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectStartDownloadingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction}\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectStartDownloadingAction;\n  return proto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectStartDownloadingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction}\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectStartDownloadingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectStartDownloadingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction} returns this\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction} returns this\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectStartDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectDoneDownloadingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectDoneDownloadingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction}\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectDoneDownloadingAction;\n  return proto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectDoneDownloadingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction}\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectDoneDownloadingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectDoneDownloadingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction} returns this\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction} returns this\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectSetPendingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectSetPendingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectSetPendingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectSetPendingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectSetPendingAction}\n */\nproto.ubo.v1.RPiConnectSetPendingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectSetPendingAction;\n  return proto.ubo.v1.RPiConnectSetPendingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectSetPendingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectSetPendingAction}\n */\nproto.ubo.v1.RPiConnectSetPendingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectSetPendingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectSetPendingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectSetPendingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectSetPendingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectSetPendingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectSetPendingAction} returns this\n */\nproto.ubo.v1.RPiConnectSetPendingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectSetPendingAction} returns this\n */\nproto.ubo.v1.RPiConnectSetPendingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectSetPendingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectStatus.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectStatus} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectStatus.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    screenSharingSessions: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    remoteShellSessions: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectStatus}\n */\nproto.ubo.v1.RPiConnectStatus.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectStatus;\n  return proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectStatus} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectStatus}\n */\nproto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setScreenSharingSessions(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setRemoteShellSessions(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectStatus} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeInt64(\n      2,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeInt64(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectStatus} returns this\n */\nproto.ubo.v1.RPiConnectStatus.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectStatus} returns this\n */\nproto.ubo.v1.RPiConnectStatus.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional int64 screen_sharing_sessions = 2;\n * @return {number}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.getScreenSharingSessions = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RPiConnectStatus} returns this\n */\nproto.ubo.v1.RPiConnectStatus.prototype.setScreenSharingSessions = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectStatus} returns this\n */\nproto.ubo.v1.RPiConnectStatus.prototype.clearScreenSharingSessions = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.hasScreenSharingSessions = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional int64 remote_shell_sessions = 3;\n * @return {number}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.getRemoteShellSessions = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.RPiConnectStatus} returns this\n */\nproto.ubo.v1.RPiConnectStatus.prototype.setRemoteShellSessions = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectStatus} returns this\n */\nproto.ubo.v1.RPiConnectStatus.prototype.clearRemoteShellSessions = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectStatus.prototype.hasRemoteShellSessions = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectSetStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectSetStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectSetStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isSignedIn: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    status: (f = msg.getStatus()) && proto.ubo.v1.RPiConnectStatus.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectSetStatusAction;\n  return proto.ubo.v1.RPiConnectSetStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectSetStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsInstalled(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsSignedIn(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.RPiConnectStatus;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader);\n      msg.setStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectSetStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectSetStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectSetStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsInstalled();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = message.getStatus();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_installed = 2;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.getIsInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.setIsInstalled = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional bool is_signed_in = 3;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.getIsSignedIn = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.setIsSignedIn = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.clearIsSignedIn = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.hasIsSignedIn = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional RPiConnectStatus status = 4;\n * @return {?proto.ubo.v1.RPiConnectStatus}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.getStatus = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectStatus} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectStatus, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectStatus|undefined} value\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n*/\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.setStatus = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.clearStatus = function() {\n  return this.setStatus(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectSetStatusAction.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectLoginEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectLoginEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectLoginEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectLoginEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectLoginEvent}\n */\nproto.ubo.v1.RPiConnectLoginEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectLoginEvent;\n  return proto.ubo.v1.RPiConnectLoginEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectLoginEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectLoginEvent}\n */\nproto.ubo.v1.RPiConnectLoginEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectLoginEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectLoginEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectLoginEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectLoginEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectLoginEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectLoginEvent} returns this\n */\nproto.ubo.v1.RPiConnectLoginEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectLoginEvent} returns this\n */\nproto.ubo.v1.RPiConnectLoginEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectLoginEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectUpdateServiceStateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectUpdateServiceStateAction;\n  return proto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectUpdateServiceStateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.RPiConnectState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.RPiConnectState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.RPiConnectState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isDownloading: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    isSignedIn: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    status: (f = msg.getStatus()) && proto.ubo.v1.RPiConnectStatus.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.RPiConnectState}\n */\nproto.ubo.v1.RPiConnectState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.RPiConnectState;\n  return proto.ubo.v1.RPiConnectState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.RPiConnectState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.RPiConnectState}\n */\nproto.ubo.v1.RPiConnectState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsDownloading(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsInstalled(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsSignedIn(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.RPiConnectStatus;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader);\n      msg.setStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.RPiConnectState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.RPiConnectState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.RPiConnectState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.RPiConnectState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = message.getStatus();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;\n * @return {string}\n */\nproto.ubo.v1.RPiConnectState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_downloading = 2;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.getIsDownloading = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.setIsDownloading = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.clearIsDownloading = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.hasIsDownloading = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_active = 3;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool is_installed = 4;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.getIsInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.setIsInstalled = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.clearIsInstalled = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.hasIsInstalled = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_signed_in = 5;\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.getIsSignedIn = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.setIsSignedIn = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.clearIsSignedIn = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.hasIsSignedIn = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional RPiConnectStatus status = 6;\n * @return {?proto.ubo.v1.RPiConnectStatus}\n */\nproto.ubo.v1.RPiConnectState.prototype.getStatus = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectStatus} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectStatus, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectStatus|undefined} value\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n*/\nproto.ubo.v1.RPiConnectState.prototype.setStatus = function(value) {\n  return jspb.Message.setWrapperField(this, 6, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.RPiConnectState} returns this\n */\nproto.ubo.v1.RPiConnectState.prototype.clearStatus = function() {\n  return this.setStatus(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.RPiConnectState.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SensorsAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SensorsAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SensorsAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorsAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SensorsAction}\n */\nproto.ubo.v1.SensorsAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SensorsAction;\n  return proto.ubo.v1.SensorsAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SensorsAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SensorsAction}\n */\nproto.ubo.v1.SensorsAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SensorsAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SensorsAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SensorsAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorsAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;\n * @return {string}\n */\nproto.ubo.v1.SensorsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SensorsAction} returns this\n */\nproto.ubo.v1.SensorsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SensorsAction} returns this\n */\nproto.ubo.v1.SensorsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SensorsReportReadingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SensorsReportReadingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorsReportReadingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    sensor: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    reading: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),\n    timestamp: jspb.Message.getFieldWithDefault(msg, 4, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SensorsReportReadingAction}\n */\nproto.ubo.v1.SensorsReportReadingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SensorsReportReadingAction;\n  return proto.ubo.v1.SensorsReportReadingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SensorsReportReadingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SensorsReportReadingAction}\n */\nproto.ubo.v1.SensorsReportReadingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.Sensor} */ (reader.readEnum());\n      msg.setSensor(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setReading(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setTimestamp(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SensorsReportReadingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SensorsReportReadingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorsReportReadingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSensor();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getReading();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      3,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;\n * @return {string}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Sensor sensor = 2;\n * @return {!proto.ubo.v1.Sensor}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.getSensor = function() {\n  return /** @type {!proto.ubo.v1.Sensor} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.Sensor} value\n * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.setSensor = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional float reading = 3;\n * @return {number}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.getReading = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.setReading = function(value) {\n  return jspb.Message.setProto3FloatField(this, 3, value);\n};\n\n\n/**\n * optional int64 timestamp = 4;\n * @return {number}\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this\n */\nproto.ubo.v1.SensorsReportReadingAction.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3IntField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SensorState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SensorState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SensorState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    value: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SensorState}\n */\nproto.ubo.v1.SensorState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SensorState;\n  return proto.ubo.v1.SensorState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SensorState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SensorState}\n */\nproto.ubo.v1.SensorState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setValue(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SensorState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SensorState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SensorState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;\n * @return {string}\n */\nproto.ubo.v1.SensorState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SensorState} returns this\n */\nproto.ubo.v1.SensorState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SensorState} returns this\n */\nproto.ubo.v1.SensorState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float value = 2;\n * @return {number}\n */\nproto.ubo.v1.SensorState.prototype.getValue = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SensorState} returns this\n */\nproto.ubo.v1.SensorState.prototype.setValue = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SensorState} returns this\n */\nproto.ubo.v1.SensorState.prototype.clearValue = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorState.prototype.hasValue = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SensorsState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SensorsState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SensorsState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorsState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    temperature: (f = msg.getTemperature()) && proto.ubo.v1.SensorState.toObject(includeInstance, f),\n    light: (f = msg.getLight()) && proto.ubo.v1.SensorState.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SensorsState}\n */\nproto.ubo.v1.SensorsState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SensorsState;\n  return proto.ubo.v1.SensorsState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SensorsState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SensorsState}\n */\nproto.ubo.v1.SensorsState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.SensorState;\n      reader.readMessage(value,proto.ubo.v1.SensorState.deserializeBinaryFromReader);\n      msg.setTemperature(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.SensorState;\n      reader.readMessage(value,proto.ubo.v1.SensorState.deserializeBinaryFromReader);\n      msg.setLight(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SensorsState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SensorsState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SensorsState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SensorsState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTemperature();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.SensorState.serializeBinaryToWriter\n    );\n  }\n  f = message.getLight();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.SensorState.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;\n * @return {string}\n */\nproto.ubo.v1.SensorsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SensorsState} returns this\n */\nproto.ubo.v1.SensorsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SensorsState} returns this\n */\nproto.ubo.v1.SensorsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SensorState temperature = 2;\n * @return {?proto.ubo.v1.SensorState}\n */\nproto.ubo.v1.SensorsState.prototype.getTemperature = function() {\n  return /** @type{?proto.ubo.v1.SensorState} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorState, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SensorState|undefined} value\n * @return {!proto.ubo.v1.SensorsState} returns this\n*/\nproto.ubo.v1.SensorsState.prototype.setTemperature = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SensorsState} returns this\n */\nproto.ubo.v1.SensorsState.prototype.clearTemperature = function() {\n  return this.setTemperature(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorsState.prototype.hasTemperature = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional SensorState light = 3;\n * @return {?proto.ubo.v1.SensorState}\n */\nproto.ubo.v1.SensorsState.prototype.getLight = function() {\n  return /** @type{?proto.ubo.v1.SensorState} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorState, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SensorState|undefined} value\n * @return {!proto.ubo.v1.SensorsState} returns this\n*/\nproto.ubo.v1.SensorsState.prototype.setLight = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SensorsState} returns this\n */\nproto.ubo.v1.SensorsState.prototype.clearLight = function() {\n  return this.setLight(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SensorsState.prototype.hasLight = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionAction}\n */\nproto.ubo.v1.SpeechRecognitionAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionAction;\n  return proto.ubo.v1.SpeechRecognitionAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionAction}\n */\nproto.ubo.v1.SpeechRecognitionAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    engineName: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction;\n  return proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (reader.readEnum());\n      msg.setEngineName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SpeechRecognitionEngineName engine_name = 2;\n * @return {!proto.ubo.v1.SpeechRecognitionEngineName}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.getEngineName = function() {\n  return /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechRecognitionEngineName} value\n * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.setEngineName = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.clearEngineName = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.hasEngineName = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction;\n  return proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsActive();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction;\n  return proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsActive();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionIntent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionIntent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    phrase: (f = msg.getPhrase()) && proto.ubo.v1.SpeechRecognitionIntent.Phrase.toObject(includeInstance, f),\n    action: (f = msg.getAction()) && proto.ubo.v1.SpeechRecognitionIntent.Action.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent}\n */\nproto.ubo.v1.SpeechRecognitionIntent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionIntent;\n  return proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent}\n */\nproto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent.Phrase;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinaryFromReader);\n      msg.setPhrase(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader);\n      msg.setAction(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getPhrase();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.Phrase.serializeBinaryToWriter\n    );\n  }\n  f = message.getAction();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionIntent.Phrase1.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Phrase1;\n  return proto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = /** @type {string} */ (reader.readString());\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionIntent.Phrase1.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedString(\n      1,\n      f\n    );\n  }\n};\n\n\n/**\n * repeated string items = 1;\n * @return {!Array<string>}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.getItemsList = function() {\n  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));\n};\n\n\n/**\n * @param {!Array<string>} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.setItemsList = function(value) {\n  return jspb.Message.setField(this, 1, value || []);\n};\n\n\n/**\n * @param {string} value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.addItems = function(value, opt_index) {\n  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase = {\n  PHRASE_NOT_SET: 0,\n  LIST: 1,\n  STRING: 2\n};\n\n/**\n * @return {proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.getPhraseCase = function() {\n  return /** @type {proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionIntent.Phrase.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    list: (f = msg.getList()) && proto.ubo.v1.SpeechRecognitionIntent.Phrase1.toObject(includeInstance, f),\n    string: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Phrase;\n  return proto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent.Phrase1;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinaryFromReader);\n      msg.setList(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setString(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionIntent.Phrase.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getList();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.Phrase1.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional Phrase1 list = 1;\n * @return {?proto.ubo.v1.SpeechRecognitionIntent.Phrase1}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.getList = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Phrase1} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Phrase1, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionIntent.Phrase1|undefined} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this\n*/\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.setList = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.clearList = function() {\n  return this.setList(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.hasList = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional string string = 2;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.getString = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.setString = function(value) {\n  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.clearString = function() {\n  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0], undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.hasString = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionIntent.Action2.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action2} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.SpeechRecognitionIntent.Action.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Action2;\n  return proto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action2} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionIntent.Action2.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action2} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated Action items = 1;\n * @return {!Array<!proto.ubo.v1.SpeechRecognitionIntent.Action>}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.SpeechRecognitionIntent.Action>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.SpeechRecognitionIntent.Action>} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2} returns this\n*/\nproto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.SpeechRecognitionIntent.Action, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_ = [[1,2]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase = {\n  ACTION_NOT_SET: 0,\n  UBO_ACTION: 1,\n  LIST: 2\n};\n\n/**\n * @return {proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.getActionCase = function() {\n  return /** @type {proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionIntent.Action.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    uboAction: (f = msg.getUboAction()) && proto.ubo.v1.SpeechRecognitionIntent.Action.toObject(includeInstance, f),\n    list: (f = msg.getList()) && proto.ubo.v1.SpeechRecognitionIntent.Action2.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Action;\n  return proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader);\n      msg.setUboAction(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action2;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinaryFromReader);\n      msg.setList(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getUboAction();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter\n    );\n  }\n  f = message.getList();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.Action2.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional Action ubo_action = 1;\n * @return {?proto.ubo.v1.SpeechRecognitionIntent.Action}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.getUboAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Action} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionIntent.Action|undefined} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this\n*/\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.setUboAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.clearUboAction = function() {\n  return this.setUboAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.hasUboAction = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional Action2 list = 2;\n * @return {?proto.ubo.v1.SpeechRecognitionIntent.Action2}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.getList = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Action2} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action2, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionIntent.Action2|undefined} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this\n*/\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.setList = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.clearList = function() {\n  return this.setList(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.Action.prototype.hasList = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Phrase phrase = 2;\n * @return {?proto.ubo.v1.SpeechRecognitionIntent.Phrase}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.getPhrase = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Phrase} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Phrase, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionIntent.Phrase|undefined} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this\n*/\nproto.ubo.v1.SpeechRecognitionIntent.prototype.setPhrase = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.clearPhrase = function() {\n  return this.setPhrase(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.hasPhrase = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional Action action = 3;\n * @return {?proto.ubo.v1.SpeechRecognitionIntent.Action}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.getAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Action} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionIntent.Action|undefined} value\n * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this\n*/\nproto.ubo.v1.SpeechRecognitionIntent.prototype.setAction = function(value) {\n  return jspb.Message.setWrapperField(this, 3, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.clearAction = function() {\n  return this.setAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionIntent.prototype.hasAction = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    wakeWord: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction;\n  return proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setWakeWord(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getWakeWord();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string wake_word = 2;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.getWakeWord = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.setWakeWord = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    intent: (f = msg.getIntent()) && proto.ubo.v1.SpeechRecognitionIntent.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction;\n  return proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader);\n      msg.setIntent(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIntent();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SpeechRecognitionIntent intent = 2;\n * @return {?proto.ubo.v1.SpeechRecognitionIntent}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.getIntent = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionIntent|undefined} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this\n*/\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.setIntent = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.clearIntent = function() {\n  return this.setIntent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.hasIntent = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionReportSpeechAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    engineName: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    text: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    audio: msg.getAudio_asB64()\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionReportSpeechAction;\n  return proto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (reader.readEnum());\n      msg.setEngineName(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setText(value);\n      break;\n    case 4:\n      var value = /** @type {!Uint8Array} */ (reader.readBytes());\n      msg.setAudio(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionReportSpeechAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getEngineName();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getText();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = message.getAudio_asU8();\n  if (f.length > 0) {\n    writer.writeBytes(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SpeechRecognitionEngineName engine_name = 2;\n * @return {!proto.ubo.v1.SpeechRecognitionEngineName}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getEngineName = function() {\n  return /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechRecognitionEngineName} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setEngineName = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n/**\n * optional string text = 3;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getText = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setText = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n/**\n * optional bytes audio = 4;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getAudio = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * optional bytes audio = 4;\n * This is a type-conversion wrapper around `getAudio()`\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getAudio_asB64 = function() {\n  return /** @type {string} */ (jspb.Message.bytesAsB64(\n      this.getAudio()));\n};\n\n\n/**\n * optional bytes audio = 4;\n * Note that Uint8Array is not supported on all browsers.\n * @see http://caniuse.com/Uint8Array\n * This is a type-conversion wrapper around `getAudio()`\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getAudio_asU8 = function() {\n  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(\n      this.getAudio()));\n};\n\n\n/**\n * @param {!(string|Uint8Array)} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setAudio = function(value) {\n  return jspb.Message.setProto3BytesField(this, 4, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionEvent}\n */\nproto.ubo.v1.SpeechRecognitionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionEvent;\n  return proto.ubo.v1.SpeechRecognitionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionEvent}\n */\nproto.ubo.v1.SpeechRecognitionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionEvent} returns this\n */\nproto.ubo.v1.SpeechRecognitionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionEvent} returns this\n */\nproto.ubo.v1.SpeechRecognitionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionReportTextEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionReportTextEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),\n    text: jspb.Message.getFieldWithDefault(msg, 3, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionReportTextEvent;\n  return proto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionReportTextEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setTimestamp(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setText(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionReportTextEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionReportTextEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getTimestamp();\n  if (f !== 0.0) {\n    writer.writeFloat(\n      2,\n      f\n    );\n  }\n  f = message.getText();\n  if (f.length > 0) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional float timestamp = 2;\n * @return {number}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.getTimestamp = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.setTimestamp = function(value) {\n  return jspb.Message.setProto3FloatField(this, 2, value);\n};\n\n\n/**\n * optional string text = 3;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.getText = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this\n */\nproto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.setText = function(value) {\n  return jspb.Message.setProto3StringField(this, 3, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.SpeechRecognitionState.repeatedFields_ = [3];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechRecognitionState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechRecognitionState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    selectedEngine: jspb.Message.getFieldWithDefault(msg, 2, 0),\n    intentsList: jspb.Message.toObjectList(msg.getIntentsList(),\n    proto.ubo.v1.SpeechRecognitionIntent.toObject, includeInstance),\n    isIntentsActive: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    isAssistantActive: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    status: jspb.Message.getFieldWithDefault(msg, 6, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechRecognitionState}\n */\nproto.ubo.v1.SpeechRecognitionState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechRecognitionState;\n  return proto.ubo.v1.SpeechRecognitionState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechRecognitionState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechRecognitionState}\n */\nproto.ubo.v1.SpeechRecognitionState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (reader.readEnum());\n      msg.setSelectedEngine(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.SpeechRecognitionIntent;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader);\n      msg.addIntents(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsIntentsActive(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsAssistantActive(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.ubo.v1.SpeechRecognitionStatus} */ (reader.readEnum());\n      msg.setStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechRecognitionState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechRecognitionState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechRecognitionState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n  f = message.getIntentsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      3,\n      f,\n      proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.SpeechRecognitionStatus} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SpeechRecognitionEngineName selected_engine = 2;\n * @return {!proto.ubo.v1.SpeechRecognitionEngineName}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.getSelectedEngine = function() {\n  return /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechRecognitionEngineName} value\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.setSelectedEngine = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.clearSelectedEngine = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.hasSelectedEngine = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * repeated SpeechRecognitionIntent intents = 3;\n * @return {!Array<!proto.ubo.v1.SpeechRecognitionIntent>}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.getIntentsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.SpeechRecognitionIntent>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent, 3));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.SpeechRecognitionIntent>} value\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n*/\nproto.ubo.v1.SpeechRecognitionState.prototype.setIntentsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 3, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechRecognitionIntent=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.SpeechRecognitionIntent}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.addIntents = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.SpeechRecognitionIntent, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.clearIntentsList = function() {\n  return this.setIntentsList([]);\n};\n\n\n/**\n * optional bool is_intents_active = 4;\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.getIsIntentsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.setIsIntentsActive = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.clearIsIntentsActive = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.hasIsIntentsActive = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_assistant_active = 5;\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.getIsAssistantActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.setIsAssistantActive = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.clearIsAssistantActive = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.hasIsAssistantActive = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional SpeechRecognitionStatus status = 6;\n * @return {!proto.ubo.v1.SpeechRecognitionStatus}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.getStatus = function() {\n  return /** @type {!proto.ubo.v1.SpeechRecognitionStatus} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechRecognitionStatus} value\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.setStatus = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechRecognitionState} returns this\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.clearStatus = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechRecognitionState.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisAction}\n */\nproto.ubo.v1.SpeechSynthesisAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisAction;\n  return proto.ubo.v1.SpeechSynthesisAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisAction}\n */\nproto.ubo.v1.SpeechSynthesisAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisEvent}\n */\nproto.ubo.v1.SpeechSynthesisEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisEvent;\n  return proto.ubo.v1.SpeechSynthesisEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisEvent}\n */\nproto.ubo.v1.SpeechSynthesisEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isAccessKeySet: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus;\n  return proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsAccessKeySet(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsAccessKeySet();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} returns this\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} returns this\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_access_key_set = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.getIsAccessKeySet = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} returns this\n */\nproto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.setIsAccessKeySet = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    engineName: jspb.Message.getFieldWithDefault(msg, 2, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction;\n  return proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    case 2:\n      var value = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (reader.readEnum());\n      msg.setEngineName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getEngineName();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional SpeechSynthesisEngineName engine_name = 2;\n * @return {!proto.ubo.v1.SpeechSynthesisEngineName}\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.getEngineName = function() {\n  return /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechSynthesisEngineName} value\n * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.setEngineName = function(value) {\n  return jspb.Message.setProto3EnumField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.ReadableInformation.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.ReadableInformation.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.ReadableInformation} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReadableInformation.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    text: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    piperText: jspb.Message.getFieldWithDefault(msg, 3, \"\"),\n    picovoiceText: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.ReadableInformation}\n */\nproto.ubo.v1.ReadableInformation.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.ReadableInformation;\n  return proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.ReadableInformation} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.ReadableInformation}\n */\nproto.ubo.v1.ReadableInformation.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setText(value);\n      break;\n    case 3:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPiperText(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPicovoiceText(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.ReadableInformation.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.ReadableInformation.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.ReadableInformation} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.ReadableInformation.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getText();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeString(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.ReadableInformation.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ReadableInformation.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string text = 2;\n * @return {string}\n */\nproto.ubo.v1.ReadableInformation.prototype.getText = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.setText = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional string piper_text = 3;\n * @return {string}\n */\nproto.ubo.v1.ReadableInformation.prototype.getPiperText = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.setPiperText = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.clearPiperText = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ReadableInformation.prototype.hasPiperText = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional string picovoice_text = 4;\n * @return {string}\n */\nproto.ubo.v1.ReadableInformation.prototype.getPicovoiceText = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.setPicovoiceText = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.ReadableInformation} returns this\n */\nproto.ubo.v1.ReadableInformation.prototype.clearPicovoiceText = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.ReadableInformation.prototype.hasPicovoiceText = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisReadTextAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisReadTextAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    information: (f = msg.getInformation()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),\n    speechRate: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),\n    engine: jspb.Message.getFieldWithDefault(msg, 4, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisReadTextAction;\n  return proto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisReadTextAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.ReadableInformation;\n      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);\n      msg.setInformation(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setSpeechRate(value);\n      break;\n    case 4:\n      var value = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (reader.readEnum());\n      msg.setEngine(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisReadTextAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisReadTextAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getInformation();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeFloat(\n      3,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeEnum(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional ReadableInformation information = 2;\n * @return {?proto.ubo.v1.ReadableInformation}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getInformation = function() {\n  return /** @type{?proto.ubo.v1.ReadableInformation} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReadableInformation|undefined} value\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n*/\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setInformation = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearInformation = function() {\n  return this.setInformation(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasInformation = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional float speech_rate = 3;\n * @return {number}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getSpeechRate = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setSpeechRate = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearSpeechRate = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasSpeechRate = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional SpeechSynthesisEngineName engine = 4;\n * @return {!proto.ubo.v1.SpeechSynthesisEngineName}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getEngine = function() {\n  return /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechSynthesisEngineName} value\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setEngine = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearEngine = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasEngine = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    information: (f = msg.getInformation()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),\n    speechRate: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent;\n  return proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.ReadableInformation;\n      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);\n      msg.setInformation(value);\n      break;\n    case 3:\n      var value = /** @type {number} */ (reader.readFloat());\n      msg.setSpeechRate(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getInformation();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeFloat(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional ReadableInformation information = 2;\n * @return {?proto.ubo.v1.ReadableInformation}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.getInformation = function() {\n  return /** @type{?proto.ubo.v1.ReadableInformation} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReadableInformation|undefined} value\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this\n*/\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.setInformation = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.clearInformation = function() {\n  return this.setInformation(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.hasInformation = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional float speech_rate = 3;\n * @return {number}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.getSpeechRate = function() {\n  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.setSpeechRate = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.clearSpeechRate = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.hasSpeechRate = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SpeechSynthesisState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SpeechSynthesisState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isAccessKeySet: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    selectedEngine: jspb.Message.getFieldWithDefault(msg, 3, 0)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SpeechSynthesisState}\n */\nproto.ubo.v1.SpeechSynthesisState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SpeechSynthesisState;\n  return proto.ubo.v1.SpeechSynthesisState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SpeechSynthesisState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SpeechSynthesisState}\n */\nproto.ubo.v1.SpeechSynthesisState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsAccessKeySet(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (reader.readEnum());\n      msg.setSelectedEngine(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SpeechSynthesisState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SpeechSynthesisState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SpeechSynthesisState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;\n * @return {string}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SpeechSynthesisState} returns this\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisState} returns this\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_access_key_set = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.getIsAccessKeySet = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SpeechSynthesisState} returns this\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.setIsAccessKeySet = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisState} returns this\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.clearIsAccessKeySet = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.hasIsAccessKeySet = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional SpeechSynthesisEngineName selected_engine = 3;\n * @return {!proto.ubo.v1.SpeechSynthesisEngineName}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.getSelectedEngine = function() {\n  return /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.SpeechSynthesisEngineName} value\n * @return {!proto.ubo.v1.SpeechSynthesisState} returns this\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.setSelectedEngine = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SpeechSynthesisState} returns this\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.clearSelectedEngine = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SpeechSynthesisState.prototype.hasSelectedEngine = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SSHAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SSHAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SSHAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SSHAction}\n */\nproto.ubo.v1.SSHAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SSHAction;\n  return proto.ubo.v1.SSHAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SSHAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SSHAction}\n */\nproto.ubo.v1.SSHAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SSHAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SSHAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SSHAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;\n * @return {string}\n */\nproto.ubo.v1.SSHAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SSHAction} returns this\n */\nproto.ubo.v1.SSHAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHAction} returns this\n */\nproto.ubo.v1.SSHAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SSHUpdateStateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SSHUpdateStateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHUpdateStateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SSHUpdateStateAction}\n */\nproto.ubo.v1.SSHUpdateStateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SSHUpdateStateAction;\n  return proto.ubo.v1.SSHUpdateStateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SSHUpdateStateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SSHUpdateStateAction}\n */\nproto.ubo.v1.SSHUpdateStateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnabled(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SSHUpdateStateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SSHUpdateStateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHUpdateStateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;\n * @return {string}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_enabled = 3;\n * @return {boolean}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.getIsEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.setIsEnabled = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.clearIsEnabled = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHUpdateStateAction.prototype.hasIsEnabled = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SSHClearEnabledStateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SSHClearEnabledStateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SSHClearEnabledStateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHClearEnabledStateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SSHClearEnabledStateAction}\n */\nproto.ubo.v1.SSHClearEnabledStateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SSHClearEnabledStateAction;\n  return proto.ubo.v1.SSHClearEnabledStateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SSHClearEnabledStateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SSHClearEnabledStateAction}\n */\nproto.ubo.v1.SSHClearEnabledStateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SSHClearEnabledStateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SSHClearEnabledStateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SSHClearEnabledStateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHClearEnabledStateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;\n * @return {string}\n */\nproto.ubo.v1.SSHClearEnabledStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SSHClearEnabledStateAction} returns this\n */\nproto.ubo.v1.SSHClearEnabledStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHClearEnabledStateAction} returns this\n */\nproto.ubo.v1.SSHClearEnabledStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHClearEnabledStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.SSHState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.SSHState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.SSHState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.SSHState}\n */\nproto.ubo.v1.SSHState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.SSHState;\n  return proto.ubo.v1.SSHState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.SSHState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.SSHState}\n */\nproto.ubo.v1.SSHState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsActive(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsEnabled(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.SSHState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.SSHState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.SSHState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.SSHState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;\n * @return {string}\n */\nproto.ubo.v1.SSHState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.SSHState} returns this\n */\nproto.ubo.v1.SSHState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHState} returns this\n */\nproto.ubo.v1.SSHState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_active = 2;\n * @return {boolean}\n */\nproto.ubo.v1.SSHState.prototype.getIsActive = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SSHState} returns this\n */\nproto.ubo.v1.SSHState.prototype.setIsActive = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHState} returns this\n */\nproto.ubo.v1.SSHState.prototype.clearIsActive = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHState.prototype.hasIsActive = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_enabled = 3;\n * @return {boolean}\n */\nproto.ubo.v1.SSHState.prototype.getIsEnabled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.SSHState} returns this\n */\nproto.ubo.v1.SSHState.prototype.setIsEnabled = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.SSHState} returns this\n */\nproto.ubo.v1.SSHState.prototype.clearIsEnabled = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.SSHState.prototype.hasIsEnabled = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersAction}\n */\nproto.ubo.v1.UsersAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersAction;\n  return proto.ubo.v1.UsersAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersAction}\n */\nproto.ubo.v1.UsersAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersAction} returns this\n */\nproto.ubo.v1.UsersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersAction} returns this\n */\nproto.ubo.v1.UsersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.UsersSetUsersAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersSetUsersAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersSetUsersAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersSetUsersAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    usersList: jspb.Message.toObjectList(msg.getUsersList(),\n    proto.ubo.v1.UserState.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersSetUsersAction}\n */\nproto.ubo.v1.UsersSetUsersAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersSetUsersAction;\n  return proto.ubo.v1.UsersSetUsersAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersSetUsersAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersSetUsersAction}\n */\nproto.ubo.v1.UsersSetUsersAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.UserState;\n      reader.readMessage(value,proto.ubo.v1.UserState.deserializeBinaryFromReader);\n      msg.addUsers(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersSetUsersAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersSetUsersAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersSetUsersAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getUsersList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.UserState.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersSetUsersAction} returns this\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersSetUsersAction} returns this\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated UserState users = 2;\n * @return {!Array<!proto.ubo.v1.UserState>}\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.getUsersList = function() {\n  return /** @type{!Array<!proto.ubo.v1.UserState>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.UserState, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.UserState>} value\n * @return {!proto.ubo.v1.UsersSetUsersAction} returns this\n*/\nproto.ubo.v1.UsersSetUsersAction.prototype.setUsersList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.UserState=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.UserState}\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.addUsers = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.UserState, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.UsersSetUsersAction} returns this\n */\nproto.ubo.v1.UsersSetUsersAction.prototype.clearUsersList = function() {\n  return this.setUsersList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersCreateUserAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersCreateUserAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersCreateUserAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersCreateUserAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersCreateUserAction}\n */\nproto.ubo.v1.UsersCreateUserAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersCreateUserAction;\n  return proto.ubo.v1.UsersCreateUserAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersCreateUserAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersCreateUserAction}\n */\nproto.ubo.v1.UsersCreateUserAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersCreateUserAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersCreateUserAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersCreateUserAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersCreateUserAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersCreateUserAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersCreateUserAction} returns this\n */\nproto.ubo.v1.UsersCreateUserAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersCreateUserAction} returns this\n */\nproto.ubo.v1.UsersCreateUserAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersCreateUserAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersDeleteUserAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersDeleteUserAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersDeleteUserAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersDeleteUserAction}\n */\nproto.ubo.v1.UsersDeleteUserAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersDeleteUserAction;\n  return proto.ubo.v1.UsersDeleteUserAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersDeleteUserAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersDeleteUserAction}\n */\nproto.ubo.v1.UsersDeleteUserAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersDeleteUserAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersDeleteUserAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersDeleteUserAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersDeleteUserAction} returns this\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersDeleteUserAction} returns this\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersDeleteUserAction} returns this\n */\nproto.ubo.v1.UsersDeleteUserAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersResetPasswordAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersResetPasswordAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersResetPasswordAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersResetPasswordAction}\n */\nproto.ubo.v1.UsersResetPasswordAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersResetPasswordAction;\n  return proto.ubo.v1.UsersResetPasswordAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersResetPasswordAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersResetPasswordAction}\n */\nproto.ubo.v1.UsersResetPasswordAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersResetPasswordAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersResetPasswordAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersResetPasswordAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersResetPasswordAction} returns this\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersResetPasswordAction} returns this\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersResetPasswordAction} returns this\n */\nproto.ubo.v1.UsersResetPasswordAction.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersEvent}\n */\nproto.ubo.v1.UsersEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersEvent;\n  return proto.ubo.v1.UsersEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersEvent}\n */\nproto.ubo.v1.UsersEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersEvent} returns this\n */\nproto.ubo.v1.UsersEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersEvent} returns this\n */\nproto.ubo.v1.UsersEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersCreateUserEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersCreateUserEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersCreateUserEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersCreateUserEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersCreateUserEvent}\n */\nproto.ubo.v1.UsersCreateUserEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersCreateUserEvent;\n  return proto.ubo.v1.UsersCreateUserEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersCreateUserEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersCreateUserEvent}\n */\nproto.ubo.v1.UsersCreateUserEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersCreateUserEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersCreateUserEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersCreateUserEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersCreateUserEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersCreateUserEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersCreateUserEvent} returns this\n */\nproto.ubo.v1.UsersCreateUserEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersCreateUserEvent} returns this\n */\nproto.ubo.v1.UsersCreateUserEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersCreateUserEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersDeleteUserEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersDeleteUserEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersDeleteUserEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersDeleteUserEvent}\n */\nproto.ubo.v1.UsersDeleteUserEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersDeleteUserEvent;\n  return proto.ubo.v1.UsersDeleteUserEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersDeleteUserEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersDeleteUserEvent}\n */\nproto.ubo.v1.UsersDeleteUserEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersDeleteUserEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersDeleteUserEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersDeleteUserEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersDeleteUserEvent} returns this\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersDeleteUserEvent} returns this\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersDeleteUserEvent} returns this\n */\nproto.ubo.v1.UsersDeleteUserEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersResetPasswordEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersResetPasswordEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersResetPasswordEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersResetPasswordEvent}\n */\nproto.ubo.v1.UsersResetPasswordEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersResetPasswordEvent;\n  return proto.ubo.v1.UsersResetPasswordEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersResetPasswordEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersResetPasswordEvent}\n */\nproto.ubo.v1.UsersResetPasswordEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersResetPasswordEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersResetPasswordEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersResetPasswordEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersResetPasswordEvent} returns this\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersResetPasswordEvent} returns this\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersResetPasswordEvent} returns this\n */\nproto.ubo.v1.UsersResetPasswordEvent.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UserState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UserState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UserState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UserState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    id: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    isRemovable: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UserState}\n */\nproto.ubo.v1.UserState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UserState;\n  return proto.ubo.v1.UserState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UserState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UserState}\n */\nproto.ubo.v1.UserState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setId(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsRemovable(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UserState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UserState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UserState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UserState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getId();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = message.getIsRemovable();\n  if (f) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UserState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UserState} returns this\n */\nproto.ubo.v1.UserState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UserState} returns this\n */\nproto.ubo.v1.UserState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UserState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string id = 2;\n * @return {string}\n */\nproto.ubo.v1.UserState.prototype.getId = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UserState} returns this\n */\nproto.ubo.v1.UserState.prototype.setId = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional bool is_removable = 3;\n * @return {boolean}\n */\nproto.ubo.v1.UserState.prototype.getIsRemovable = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.UserState} returns this\n */\nproto.ubo.v1.UserState.prototype.setIsRemovable = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 3, value);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    users: (f = msg.getUsers()) && proto.ubo.v1.UsersState.Users.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersState}\n */\nproto.ubo.v1.UsersState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersState;\n  return proto.ubo.v1.UsersState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersState}\n */\nproto.ubo.v1.UsersState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.UsersState.Users;\n      reader.readMessage(value,proto.ubo.v1.UsersState.Users.deserializeBinaryFromReader);\n      msg.setUsers(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getUsers();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.UsersState.Users.serializeBinaryToWriter\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.UsersState.Users.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.UsersState.Users.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.UsersState.Users.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.UsersState.Users} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersState.Users.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.UserState.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.UsersState.Users}\n */\nproto.ubo.v1.UsersState.Users.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.UsersState.Users;\n  return proto.ubo.v1.UsersState.Users.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.UsersState.Users} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.UsersState.Users}\n */\nproto.ubo.v1.UsersState.Users.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.UserState;\n      reader.readMessage(value,proto.ubo.v1.UserState.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.UsersState.Users.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.UsersState.Users.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.UsersState.Users} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.UsersState.Users.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.UserState.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated UserState items = 1;\n * @return {!Array<!proto.ubo.v1.UserState>}\n */\nproto.ubo.v1.UsersState.Users.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.UserState>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.UserState, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.UserState>} value\n * @return {!proto.ubo.v1.UsersState.Users} returns this\n*/\nproto.ubo.v1.UsersState.Users.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.UserState=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.UserState}\n */\nproto.ubo.v1.UsersState.Users.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.UserState, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.UsersState.Users} returns this\n */\nproto.ubo.v1.UsersState.Users.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;\n * @return {string}\n */\nproto.ubo.v1.UsersState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.UsersState} returns this\n */\nproto.ubo.v1.UsersState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.UsersState} returns this\n */\nproto.ubo.v1.UsersState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Users users = 2;\n * @return {?proto.ubo.v1.UsersState.Users}\n */\nproto.ubo.v1.UsersState.prototype.getUsers = function() {\n  return /** @type{?proto.ubo.v1.UsersState.Users} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersState.Users, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersState.Users|undefined} value\n * @return {!proto.ubo.v1.UsersState} returns this\n*/\nproto.ubo.v1.UsersState.prototype.setUsers = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.UsersState} returns this\n */\nproto.ubo.v1.UsersState.prototype.clearUsers = function() {\n  return this.setUsers(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.UsersState.prototype.hasUsers = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeAction}\n */\nproto.ubo.v1.VSCodeAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeAction;\n  return proto.ubo.v1.VSCodeAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeAction}\n */\nproto.ubo.v1.VSCodeAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeAction} returns this\n */\nproto.ubo.v1.VSCodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeAction} returns this\n */\nproto.ubo.v1.VSCodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeEvent}\n */\nproto.ubo.v1.VSCodeEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeEvent;\n  return proto.ubo.v1.VSCodeEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeEvent}\n */\nproto.ubo.v1.VSCodeEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeEvent} returns this\n */\nproto.ubo.v1.VSCodeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeEvent} returns this\n */\nproto.ubo.v1.VSCodeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeStartDownloadingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeStartDownloadingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeStartDownloadingAction}\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeStartDownloadingAction;\n  return proto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeStartDownloadingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeStartDownloadingAction}\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeStartDownloadingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeStartDownloadingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeStartDownloadingAction} returns this\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeStartDownloadingAction} returns this\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeStartDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeDoneDownloadingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeDoneDownloadingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction}\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeDoneDownloadingAction;\n  return proto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeDoneDownloadingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction}\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeDoneDownloadingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeDoneDownloadingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction} returns this\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction} returns this\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeDoneDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeSetPendingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeSetPendingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeSetPendingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeSetPendingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeSetPendingAction}\n */\nproto.ubo.v1.VSCodeSetPendingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeSetPendingAction;\n  return proto.ubo.v1.VSCodeSetPendingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeSetPendingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeSetPendingAction}\n */\nproto.ubo.v1.VSCodeSetPendingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeSetPendingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeSetPendingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeSetPendingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeSetPendingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeSetPendingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeSetPendingAction} returns this\n */\nproto.ubo.v1.VSCodeSetPendingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeSetPendingAction} returns this\n */\nproto.ubo.v1.VSCodeSetPendingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeSetPendingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeStatus.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeStatus.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeStatus} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeStatus.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isServiceInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isRunning: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    name: jspb.Message.getFieldWithDefault(msg, 4, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeStatus}\n */\nproto.ubo.v1.VSCodeStatus.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeStatus;\n  return proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeStatus} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeStatus}\n */\nproto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsServiceInstalled(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsRunning(value);\n      break;\n    case 4:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setName(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeStatus.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeStatus} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeStatus.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsServiceInstalled();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getIsRunning();\n  if (f) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeString(\n      4,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeStatus.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeStatus} returns this\n */\nproto.ubo.v1.VSCodeStatus.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeStatus} returns this\n */\nproto.ubo.v1.VSCodeStatus.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeStatus.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_service_installed = 2;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeStatus.prototype.getIsServiceInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeStatus} returns this\n */\nproto.ubo.v1.VSCodeStatus.prototype.setIsServiceInstalled = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional bool is_running = 3;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeStatus.prototype.getIsRunning = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeStatus} returns this\n */\nproto.ubo.v1.VSCodeStatus.prototype.setIsRunning = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 3, value);\n};\n\n\n/**\n * optional string name = 4;\n * @return {string}\n */\nproto.ubo.v1.VSCodeStatus.prototype.getName = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeStatus} returns this\n */\nproto.ubo.v1.VSCodeStatus.prototype.setName = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeStatus} returns this\n */\nproto.ubo.v1.VSCodeStatus.prototype.clearName = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeStatus.prototype.hasName = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeSetStatusAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeSetStatusAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeSetStatusAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isBinaryInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isLoggedIn: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    status: (f = msg.getStatus()) && proto.ubo.v1.VSCodeStatus.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeSetStatusAction}\n */\nproto.ubo.v1.VSCodeSetStatusAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeSetStatusAction;\n  return proto.ubo.v1.VSCodeSetStatusAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeSetStatusAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeSetStatusAction}\n */\nproto.ubo.v1.VSCodeSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsBinaryInstalled(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsLoggedIn(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.VSCodeStatus;\n      reader.readMessage(value,proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader);\n      msg.setStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeSetStatusAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeSetStatusAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeSetStatusAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getIsBinaryInstalled();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = message.getIsLoggedIn();\n  if (f) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = message.getStatus();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_binary_installed = 2;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.getIsBinaryInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.setIsBinaryInstalled = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n/**\n * optional bool is_logged_in = 3;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.getIsLoggedIn = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.setIsLoggedIn = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 3, value);\n};\n\n\n/**\n * optional VSCodeStatus status = 4;\n * @return {?proto.ubo.v1.VSCodeStatus}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.getStatus = function() {\n  return /** @type{?proto.ubo.v1.VSCodeStatus} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeStatus, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeStatus|undefined} value\n * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this\n*/\nproto.ubo.v1.VSCodeSetStatusAction.prototype.setStatus = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.clearStatus = function() {\n  return this.setStatus(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeSetStatusAction.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeLoginEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeLoginEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeLoginEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeLoginEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeLoginEvent}\n */\nproto.ubo.v1.VSCodeLoginEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeLoginEvent;\n  return proto.ubo.v1.VSCodeLoginEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeLoginEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeLoginEvent}\n */\nproto.ubo.v1.VSCodeLoginEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeLoginEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeLoginEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeLoginEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeLoginEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeLoginEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeLoginEvent} returns this\n */\nproto.ubo.v1.VSCodeLoginEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeLoginEvent} returns this\n */\nproto.ubo.v1.VSCodeLoginEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeLoginEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeRestartEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeRestartEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeRestartEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeRestartEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeRestartEvent}\n */\nproto.ubo.v1.VSCodeRestartEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeRestartEvent;\n  return proto.ubo.v1.VSCodeRestartEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeRestartEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeRestartEvent}\n */\nproto.ubo.v1.VSCodeRestartEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeRestartEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeRestartEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeRestartEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeRestartEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeRestartEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeRestartEvent} returns this\n */\nproto.ubo.v1.VSCodeRestartEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeRestartEvent} returns this\n */\nproto.ubo.v1.VSCodeRestartEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeRestartEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.VSCodeState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.VSCodeState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.VSCodeState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    isPending: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),\n    isDownloading: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),\n    isBinaryInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),\n    isLoggedIn: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),\n    status: (f = msg.getStatus()) && proto.ubo.v1.VSCodeStatus.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.VSCodeState}\n */\nproto.ubo.v1.VSCodeState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.VSCodeState;\n  return proto.ubo.v1.VSCodeState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.VSCodeState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.VSCodeState}\n */\nproto.ubo.v1.VSCodeState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsPending(value);\n      break;\n    case 3:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsDownloading(value);\n      break;\n    case 4:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsBinaryInstalled(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setIsLoggedIn(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.VSCodeStatus;\n      reader.readMessage(value,proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader);\n      msg.setStatus(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.VSCodeState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.VSCodeState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.VSCodeState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.VSCodeState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeBool(\n      3,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeBool(\n      4,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n  f = message.getStatus();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;\n * @return {string}\n */\nproto.ubo.v1.VSCodeState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool is_pending = 2;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.getIsPending = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.setIsPending = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.clearIsPending = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.hasIsPending = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional bool is_downloading = 3;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.getIsDownloading = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.setIsDownloading = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.clearIsDownloading = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.hasIsDownloading = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional bool is_binary_installed = 4;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.getIsBinaryInstalled = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.setIsBinaryInstalled = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.clearIsBinaryInstalled = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.hasIsBinaryInstalled = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool is_logged_in = 5;\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.getIsLoggedIn = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.setIsLoggedIn = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.clearIsLoggedIn = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.hasIsLoggedIn = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional VSCodeStatus status = 6;\n * @return {?proto.ubo.v1.VSCodeStatus}\n */\nproto.ubo.v1.VSCodeState.prototype.getStatus = function() {\n  return /** @type{?proto.ubo.v1.VSCodeStatus} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeStatus, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeStatus|undefined} value\n * @return {!proto.ubo.v1.VSCodeState} returns this\n*/\nproto.ubo.v1.VSCodeState.prototype.setStatus = function(value) {\n  return jspb.Message.setWrapperField(this, 6, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.VSCodeState} returns this\n */\nproto.ubo.v1.VSCodeState.prototype.clearStatus = function() {\n  return this.setStatus(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.VSCodeState.prototype.hasStatus = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WebUIEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WebUIEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WebUIEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WebUIEvent}\n */\nproto.ubo.v1.WebUIEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WebUIEvent;\n  return proto.ubo.v1.WebUIEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WebUIEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WebUIEvent}\n */\nproto.ubo.v1.WebUIEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WebUIEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WebUIEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WebUIEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;\n * @return {string}\n */\nproto.ubo.v1.WebUIEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIEvent} returns this\n */\nproto.ubo.v1.WebUIEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIEvent} returns this\n */\nproto.ubo.v1.WebUIEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WebUIInitializeEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WebUIInitializeEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIInitializeEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    description: (f = msg.getDescription()) && proto.ubo.v1.WebUIInputDescription.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WebUIInitializeEvent}\n */\nproto.ubo.v1.WebUIInitializeEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WebUIInitializeEvent;\n  return proto.ubo.v1.WebUIInitializeEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WebUIInitializeEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WebUIInitializeEvent}\n */\nproto.ubo.v1.WebUIInitializeEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.WebUIInputDescription;\n      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader);\n      msg.setDescription(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WebUIInitializeEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WebUIInitializeEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIInitializeEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getDescription();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;\n * @return {string}\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional WebUIInputDescription description = 2;\n * @return {?proto.ubo.v1.WebUIInputDescription}\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.getDescription = function() {\n  return /** @type{?proto.ubo.v1.WebUIInputDescription} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInputDescription, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WebUIInputDescription|undefined} value\n * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this\n*/\nproto.ubo.v1.WebUIInitializeEvent.prototype.setDescription = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.clearDescription = function() {\n  return this.setDescription(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIInitializeEvent.prototype.hasDescription = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WebUIStopEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WebUIStopEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WebUIStopEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIStopEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WebUIStopEvent}\n */\nproto.ubo.v1.WebUIStopEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WebUIStopEvent;\n  return proto.ubo.v1.WebUIStopEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WebUIStopEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WebUIStopEvent}\n */\nproto.ubo.v1.WebUIStopEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WebUIStopEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WebUIStopEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WebUIStopEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIStopEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;\n * @return {string}\n */\nproto.ubo.v1.WebUIStopEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIStopEvent} returns this\n */\nproto.ubo.v1.WebUIStopEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIStopEvent} returns this\n */\nproto.ubo.v1.WebUIStopEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIStopEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.WebUIState.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WebUIState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WebUIState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WebUIState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    activeInputsList: jspb.Message.toObjectList(msg.getActiveInputsList(),\n    proto.ubo.v1.WebUIInputDescription.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WebUIState}\n */\nproto.ubo.v1.WebUIState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WebUIState;\n  return proto.ubo.v1.WebUIState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WebUIState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WebUIState}\n */\nproto.ubo.v1.WebUIState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.WebUIInputDescription;\n      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader);\n      msg.addActiveInputs(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WebUIState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WebUIState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WebUIState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WebUIState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getActiveInputsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;\n * @return {string}\n */\nproto.ubo.v1.WebUIState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WebUIState} returns this\n */\nproto.ubo.v1.WebUIState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WebUIState} returns this\n */\nproto.ubo.v1.WebUIState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WebUIState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated WebUIInputDescription active_inputs = 2;\n * @return {!Array<!proto.ubo.v1.WebUIInputDescription>}\n */\nproto.ubo.v1.WebUIState.prototype.getActiveInputsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.WebUIInputDescription>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.WebUIInputDescription, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.WebUIInputDescription>} value\n * @return {!proto.ubo.v1.WebUIState} returns this\n*/\nproto.ubo.v1.WebUIState.prototype.setActiveInputsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.WebUIInputDescription=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.WebUIInputDescription}\n */\nproto.ubo.v1.WebUIState.prototype.addActiveInputs = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.WebUIInputDescription, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.WebUIState} returns this\n */\nproto.ubo.v1.WebUIState.prototype.clearActiveInputsList = function() {\n  return this.setActiveInputsList([]);\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiConnection.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiConnection.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiConnection} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiConnection.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    ssid: jspb.Message.getFieldWithDefault(msg, 2, \"\"),\n    state: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    signalStrength: jspb.Message.getFieldWithDefault(msg, 4, 0),\n    password: jspb.Message.getFieldWithDefault(msg, 5, \"\"),\n    type: jspb.Message.getFieldWithDefault(msg, 6, 0),\n    hidden: jspb.Message.getBooleanFieldWithDefault(msg, 7, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiConnection}\n */\nproto.ubo.v1.WiFiConnection.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiConnection;\n  return proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiConnection} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiConnection}\n */\nproto.ubo.v1.WiFiConnection.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    case 2:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setSsid(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.ConnectionState} */ (reader.readEnum());\n      msg.setState(value);\n      break;\n    case 4:\n      var value = /** @type {number} */ (reader.readInt64());\n      msg.setSignalStrength(value);\n      break;\n    case 5:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setPassword(value);\n      break;\n    case 6:\n      var value = /** @type {!proto.ubo.v1.WiFiType} */ (reader.readEnum());\n      msg.setType(value);\n      break;\n    case 7:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setHidden(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiConnection.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiConnection.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiConnection} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiConnection.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getSsid();\n  if (f.length > 0) {\n    writer.writeString(\n      2,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.ConnectionState} */ (jspb.Message.getField(message, 3));\n  if (f != null) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = /** @type {number} */ (jspb.Message.getField(message, 4));\n  if (f != null) {\n    writer.writeInt64(\n      4,\n      f\n    );\n  }\n  f = /** @type {string} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeString(\n      5,\n      f\n    );\n  }\n  f = /** @type {!proto.ubo.v1.WiFiType} */ (jspb.Message.getField(message, 6));\n  if (f != null) {\n    writer.writeEnum(\n      6,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));\n  if (f != null) {\n    writer.writeBool(\n      7,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiConnection.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional string ssid = 2;\n * @return {string}\n */\nproto.ubo.v1.WiFiConnection.prototype.getSsid = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setSsid = function(value) {\n  return jspb.Message.setProto3StringField(this, 2, value);\n};\n\n\n/**\n * optional ConnectionState state = 3;\n * @return {!proto.ubo.v1.ConnectionState}\n */\nproto.ubo.v1.WiFiConnection.prototype.getState = function() {\n  return /** @type {!proto.ubo.v1.ConnectionState} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.ConnectionState} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setState = function(value) {\n  return jspb.Message.setField(this, 3, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.clearState = function() {\n  return jspb.Message.setField(this, 3, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.hasState = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional int64 signal_strength = 4;\n * @return {number}\n */\nproto.ubo.v1.WiFiConnection.prototype.getSignalStrength = function() {\n  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));\n};\n\n\n/**\n * @param {number} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setSignalStrength = function(value) {\n  return jspb.Message.setField(this, 4, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.clearSignalStrength = function() {\n  return jspb.Message.setField(this, 4, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.hasSignalStrength = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional string password = 5;\n * @return {string}\n */\nproto.ubo.v1.WiFiConnection.prototype.getPassword = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setPassword = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.clearPassword = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.hasPassword = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional WiFiType type = 6;\n * @return {!proto.ubo.v1.WiFiType}\n */\nproto.ubo.v1.WiFiConnection.prototype.getType = function() {\n  return /** @type {!proto.ubo.v1.WiFiType} */ (jspb.Message.getFieldWithDefault(this, 6, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.WiFiType} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setType = function(value) {\n  return jspb.Message.setField(this, 6, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.clearType = function() {\n  return jspb.Message.setField(this, 6, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.hasType = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional bool hidden = 7;\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.getHidden = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.setHidden = function(value) {\n  return jspb.Message.setField(this, 7, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiConnection} returns this\n */\nproto.ubo.v1.WiFiConnection.prototype.clearHidden = function() {\n  return jspb.Message.setField(this, 7, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiConnection.prototype.hasHidden = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiAction}\n */\nproto.ubo.v1.WiFiAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiAction;\n  return proto.ubo.v1.WiFiAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiAction}\n */\nproto.ubo.v1.WiFiAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiAction} returns this\n */\nproto.ubo.v1.WiFiAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiAction} returns this\n */\nproto.ubo.v1.WiFiAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiInputConnectionAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiInputConnectionAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiInputConnectionAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiInputConnectionAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiInputConnectionAction}\n */\nproto.ubo.v1.WiFiInputConnectionAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiInputConnectionAction;\n  return proto.ubo.v1.WiFiInputConnectionAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiInputConnectionAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiInputConnectionAction}\n */\nproto.ubo.v1.WiFiInputConnectionAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiInputConnectionAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiInputConnectionAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiInputConnectionAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiInputConnectionAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiInputConnectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiInputConnectionAction} returns this\n */\nproto.ubo.v1.WiFiInputConnectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiInputConnectionAction} returns this\n */\nproto.ubo.v1.WiFiInputConnectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiInputConnectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    hasVisitedOnboarding: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiSetHasVisitedOnboardingAction;\n  return proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setHasVisitedOnboarding(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getHasVisitedOnboarding();\n  if (f) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} returns this\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} returns this\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool has_visited_onboarding = 2;\n * @return {boolean}\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.getHasVisitedOnboarding = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} returns this\n */\nproto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.setHasVisitedOnboarding = function(value) {\n  return jspb.Message.setProto3BooleanField(this, 2, value);\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.WiFiUpdateAction.repeatedFields_ = [2];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiUpdateAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiUpdateAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiUpdateAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    connectionsList: jspb.Message.toObjectList(msg.getConnectionsList(),\n    proto.ubo.v1.WiFiConnection.toObject, includeInstance),\n    state: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    currentConnection: (f = msg.getCurrentConnection()) && proto.ubo.v1.WiFiConnection.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiUpdateAction}\n */\nproto.ubo.v1.WiFiUpdateAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiUpdateAction;\n  return proto.ubo.v1.WiFiUpdateAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiUpdateAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiUpdateAction}\n */\nproto.ubo.v1.WiFiUpdateAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.WiFiConnection;\n      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);\n      msg.addConnections(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.NetState} */ (reader.readEnum());\n      msg.setState(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.WiFiConnection;\n      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);\n      msg.setCurrentConnection(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiUpdateAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiUpdateAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiUpdateAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getConnectionsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      2,\n      f,\n      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter\n    );\n  }\n  f = message.getState();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = message.getCurrentConnection();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * repeated WiFiConnection connections = 2;\n * @return {!Array<!proto.ubo.v1.WiFiConnection>}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.getConnectionsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.WiFiConnection>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.WiFiConnection, 2));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.WiFiConnection>} value\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n*/\nproto.ubo.v1.WiFiUpdateAction.prototype.setConnectionsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 2, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.WiFiConnection=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.WiFiConnection}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.addConnections = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.WiFiConnection, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.clearConnectionsList = function() {\n  return this.setConnectionsList([]);\n};\n\n\n/**\n * optional NetState state = 3;\n * @return {!proto.ubo.v1.NetState}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.getState = function() {\n  return /** @type {!proto.ubo.v1.NetState} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.NetState} value\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.setState = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n/**\n * optional WiFiConnection current_connection = 4;\n * @return {?proto.ubo.v1.WiFiConnection}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.getCurrentConnection = function() {\n  return /** @type{?proto.ubo.v1.WiFiConnection} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiConnection, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiConnection|undefined} value\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n*/\nproto.ubo.v1.WiFiUpdateAction.prototype.setCurrentConnection = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.WiFiUpdateAction} returns this\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.clearCurrentConnection = function() {\n  return this.setCurrentConnection(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiUpdateAction.prototype.hasCurrentConnection = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiUpdateRequestAction.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiUpdateRequestAction} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiUpdateRequestAction.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    reset: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiUpdateRequestAction}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiUpdateRequestAction;\n  return proto.ubo.v1.WiFiUpdateRequestAction.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiUpdateRequestAction} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiUpdateRequestAction}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    case 2:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setReset(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiUpdateRequestAction.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiUpdateRequestAction} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiUpdateRequestAction.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));\n  if (f != null) {\n    writer.writeBool(\n      2,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional bool reset = 2;\n * @return {boolean}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.getReset = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.setReset = function(value) {\n  return jspb.Message.setField(this, 2, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.clearReset = function() {\n  return jspb.Message.setField(this, 2, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiUpdateRequestAction.prototype.hasReset = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiEvent}\n */\nproto.ubo.v1.WiFiEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiEvent;\n  return proto.ubo.v1.WiFiEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiEvent}\n */\nproto.ubo.v1.WiFiEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiEvent} returns this\n */\nproto.ubo.v1.WiFiEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiEvent} returns this\n */\nproto.ubo.v1.WiFiEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiInputConnectionEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiInputConnectionEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiInputConnectionEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiInputConnectionEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiInputConnectionEvent}\n */\nproto.ubo.v1.WiFiInputConnectionEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiInputConnectionEvent;\n  return proto.ubo.v1.WiFiInputConnectionEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiInputConnectionEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiInputConnectionEvent}\n */\nproto.ubo.v1.WiFiInputConnectionEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiInputConnectionEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiInputConnectionEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiInputConnectionEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiInputConnectionEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiInputConnectionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiInputConnectionEvent} returns this\n */\nproto.ubo.v1.WiFiInputConnectionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiInputConnectionEvent} returns this\n */\nproto.ubo.v1.WiFiInputConnectionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiInputConnectionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiUpdateRequestEvent.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiUpdateRequestEvent} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\")\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiUpdateRequestEvent}\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiUpdateRequestEvent;\n  return proto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiUpdateRequestEvent} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiUpdateRequestEvent}\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiUpdateRequestEvent.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiUpdateRequestEvent} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiUpdateRequestEvent} returns this\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiUpdateRequestEvent} returns this\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiUpdateRequestEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiState.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiState.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiState} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiState.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, \"\"),\n    connections: (f = msg.getConnections()) && proto.ubo.v1.WiFiState.Connections.toObject(includeInstance, f),\n    state: jspb.Message.getFieldWithDefault(msg, 3, 0),\n    currentConnection: (f = msg.getCurrentConnection()) && proto.ubo.v1.WiFiConnection.toObject(includeInstance, f),\n    hasVisitedOnboarding: jspb.Message.getBooleanFieldWithDefault(msg, 5, false)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiState}\n */\nproto.ubo.v1.WiFiState.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiState;\n  return proto.ubo.v1.WiFiState.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiState} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiState}\n */\nproto.ubo.v1.WiFiState.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1000:\n      var value = /** @type {string} */ (reader.readString());\n      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.WiFiState.Connections;\n      reader.readMessage(value,proto.ubo.v1.WiFiState.Connections.deserializeBinaryFromReader);\n      msg.setConnections(value);\n      break;\n    case 3:\n      var value = /** @type {!proto.ubo.v1.NetState} */ (reader.readEnum());\n      msg.setState(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.WiFiConnection;\n      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);\n      msg.setCurrentConnection(value);\n      break;\n    case 5:\n      var value = /** @type {boolean} */ (reader.readBool());\n      msg.setHasVisitedOnboarding(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiState.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiState.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiState} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiState.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = /** @type {string} */ (jspb.Message.getField(message, 1000));\n  if (f != null) {\n    writer.writeString(\n      1000,\n      f\n    );\n  }\n  f = message.getConnections();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.WiFiState.Connections.serializeBinaryToWriter\n    );\n  }\n  f = message.getState();\n  if (f !== 0.0) {\n    writer.writeEnum(\n      3,\n      f\n    );\n  }\n  f = message.getCurrentConnection();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter\n    );\n  }\n  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));\n  if (f != null) {\n    writer.writeBool(\n      5,\n      f\n    );\n  }\n};\n\n\n\n/**\n * List of repeated fields within this message type.\n * @private {!Array<number>}\n * @const\n */\nproto.ubo.v1.WiFiState.Connections.repeatedFields_ = [1];\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.WiFiState.Connections.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.WiFiState.Connections.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.WiFiState.Connections} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiState.Connections.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    itemsList: jspb.Message.toObjectList(msg.getItemsList(),\n    proto.ubo.v1.WiFiConnection.toObject, includeInstance)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.WiFiState.Connections}\n */\nproto.ubo.v1.WiFiState.Connections.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.WiFiState.Connections;\n  return proto.ubo.v1.WiFiState.Connections.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.WiFiState.Connections} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.WiFiState.Connections}\n */\nproto.ubo.v1.WiFiState.Connections.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.WiFiConnection;\n      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);\n      msg.addItems(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.WiFiState.Connections.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.WiFiState.Connections.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.WiFiState.Connections} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.WiFiState.Connections.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getItemsList();\n  if (f.length > 0) {\n    writer.writeRepeatedMessage(\n      1,\n      f,\n      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * repeated WiFiConnection items = 1;\n * @return {!Array<!proto.ubo.v1.WiFiConnection>}\n */\nproto.ubo.v1.WiFiState.Connections.prototype.getItemsList = function() {\n  return /** @type{!Array<!proto.ubo.v1.WiFiConnection>} */ (\n    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.WiFiConnection, 1));\n};\n\n\n/**\n * @param {!Array<!proto.ubo.v1.WiFiConnection>} value\n * @return {!proto.ubo.v1.WiFiState.Connections} returns this\n*/\nproto.ubo.v1.WiFiState.Connections.prototype.setItemsList = function(value) {\n  return jspb.Message.setRepeatedWrapperField(this, 1, value);\n};\n\n\n/**\n * @param {!proto.ubo.v1.WiFiConnection=} opt_value\n * @param {number=} opt_index\n * @return {!proto.ubo.v1.WiFiConnection}\n */\nproto.ubo.v1.WiFiState.Connections.prototype.addItems = function(opt_value, opt_index) {\n  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.WiFiConnection, opt_index);\n};\n\n\n/**\n * Clears the list making it empty but non-null.\n * @return {!proto.ubo.v1.WiFiState.Connections} returns this\n */\nproto.ubo.v1.WiFiState.Connections.prototype.clearItemsList = function() {\n  return this.setItemsList([]);\n};\n\n\n/**\n * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;\n * @return {string}\n */\nproto.ubo.v1.WiFiState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, \"\"));\n};\n\n\n/**\n * @param {string} value\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {\n  return jspb.Message.setField(this, 1000, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.setField(this, 1000, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {\n  return jspb.Message.getField(this, 1000) != null;\n};\n\n\n/**\n * optional Connections connections = 2;\n * @return {?proto.ubo.v1.WiFiState.Connections}\n */\nproto.ubo.v1.WiFiState.prototype.getConnections = function() {\n  return /** @type{?proto.ubo.v1.WiFiState.Connections} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiState.Connections, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiState.Connections|undefined} value\n * @return {!proto.ubo.v1.WiFiState} returns this\n*/\nproto.ubo.v1.WiFiState.prototype.setConnections = function(value) {\n  return jspb.Message.setWrapperField(this, 2, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.clearConnections = function() {\n  return this.setConnections(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiState.prototype.hasConnections = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional NetState state = 3;\n * @return {!proto.ubo.v1.NetState}\n */\nproto.ubo.v1.WiFiState.prototype.getState = function() {\n  return /** @type {!proto.ubo.v1.NetState} */ (jspb.Message.getFieldWithDefault(this, 3, 0));\n};\n\n\n/**\n * @param {!proto.ubo.v1.NetState} value\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.setState = function(value) {\n  return jspb.Message.setProto3EnumField(this, 3, value);\n};\n\n\n/**\n * optional WiFiConnection current_connection = 4;\n * @return {?proto.ubo.v1.WiFiConnection}\n */\nproto.ubo.v1.WiFiState.prototype.getCurrentConnection = function() {\n  return /** @type{?proto.ubo.v1.WiFiConnection} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiConnection, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiConnection|undefined} value\n * @return {!proto.ubo.v1.WiFiState} returns this\n*/\nproto.ubo.v1.WiFiState.prototype.setCurrentConnection = function(value) {\n  return jspb.Message.setWrapperField(this, 4, value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.clearCurrentConnection = function() {\n  return this.setCurrentConnection(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiState.prototype.hasCurrentConnection = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional bool has_visited_onboarding = 5;\n * @return {boolean}\n */\nproto.ubo.v1.WiFiState.prototype.getHasVisitedOnboarding = function() {\n  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));\n};\n\n\n/**\n * @param {boolean} value\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.setHasVisitedOnboarding = function(value) {\n  return jspb.Message.setField(this, 5, value);\n};\n\n\n/**\n * Clears the field making it undefined.\n * @return {!proto.ubo.v1.WiFiState} returns this\n */\nproto.ubo.v1.WiFiState.prototype.clearHasVisitedOnboarding = function() {\n  return jspb.Message.setField(this, 5, undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.WiFiState.prototype.hasHasVisitedOnboarding = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.Action.oneofGroups_ = [[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Action.ActionCase = {\n  ACTION_NOT_SET: 0,\n  ASSISTANT_ACTION: 1,\n  ASSISTANT_ADD_MCP_SERVER_ACTION: 2,\n  ASSISTANT_DELETE_MCP_SERVER_ACTION: 3,\n  ASSISTANT_DOWNLOAD_OLLAMA_MODEL_ACTION: 4,\n  ASSISTANT_REPORT_ACTION: 5,\n  ASSISTANT_SET_IS_ACTIVE_ACTION: 6,\n  ASSISTANT_SET_SELECTED_IMAGE_GENERATOR_ACTION: 7,\n  ASSISTANT_SET_SELECTED_LLM_ACTION: 8,\n  ASSISTANT_SET_SELECTED_MODEL_ACTION: 9,\n  ASSISTANT_SET_SELECTED_STT_ACTION: 10,\n  ASSISTANT_SET_SELECTED_TTS_ACTION: 11,\n  ASSISTANT_START_LISTENING_ACTION: 12,\n  ASSISTANT_STOP_LISTENING_ACTION: 13,\n  ASSISTANT_SYNC_MCP_SERVERS_ACTION: 14,\n  ASSISTANT_TOGGLE_LISTENING_ACTION: 15,\n  ASSISTANT_TOGGLE_MCP_SERVER_ACTION: 16,\n  ASSISTANT_UPDATE_PROVIDERS_ACTION: 17,\n  AUDIO_ACTION: 18,\n  AUDIO_CHANGE_VOLUME_ACTION: 19,\n  AUDIO_INSTALL_DRIVER_ACTION: 20,\n  AUDIO_PLAY_AUDIO_SAMPLE_ACTION: 21,\n  AUDIO_PLAY_AUDIO_SEQUENCE_ACTION: 22,\n  AUDIO_PLAY_CHIME_ACTION: 23,\n  AUDIO_PLAY_RECORDING_ACTION: 24,\n  AUDIO_PLAYBACK_DONE_ACTION: 25,\n  AUDIO_REPORT_SAMPLE_ACTION: 26,\n  AUDIO_SET_MUTE_STATUS_ACTION: 27,\n  AUDIO_SET_VOLUME_ACTION: 28,\n  AUDIO_START_RECORDING_ACTION: 29,\n  AUDIO_STOP_RECORDING_ACTION: 30,\n  AUDIO_TOGGLE_MUTE_STATUS_ACTION: 31,\n  AUDIO_TOGGLE_RECORDING_ACTION: 32,\n  CAMERA_ACTION: 33,\n  CAMERA_DETECT_ACTION: 34,\n  CAMERA_INSTALL_DRIVER_ACTION: 35,\n  CAMERA_REPORT_BARCODE_ACTION: 36,\n  CAMERA_RESTORE_DEFAULT_ACTION: 37,\n  CAMERA_SET_AVAILABLE_CAMERAS_ACTION: 38,\n  CAMERA_SET_INDEX_ACTION: 39,\n  CAMERA_START_VIEWFINDER_ACTION: 40,\n  CLOSE_APPLICATION_ACTION: 41,\n  DEREGISTER_REGULAR_APP_ACTION: 42,\n  DISPLAY_ACTION: 43,\n  DISPLAY_BLANK_ACTION: 44,\n  DISPLAY_PAUSE_ACTION: 45,\n  DISPLAY_REDRAW_ACTION: 46,\n  DISPLAY_RESUME_ACTION: 47,\n  DISPLAY_SET_BLANK_TIMEOUT_ACTION: 48,\n  DISPLAY_UNBLANK_ACTION: 49,\n  DISPLAY_UPDATE_ACTIVITY_ACTION: 50,\n  DOCKER_ACTION: 51,\n  DOCKER_IMAGE_ACTION: 52,\n  DOCKER_IMAGE_FETCH_ACTION: 53,\n  DOCKER_IMAGE_RELEASE_ACTION: 54,\n  DOCKER_IMAGE_REMOVE_ACTION: 55,\n  DOCKER_IMAGE_REMOVE_CONTAINER_ACTION: 56,\n  DOCKER_IMAGE_RUN_ACTION: 57,\n  DOCKER_IMAGE_SET_DOCKER_ID_ACTION: 58,\n  DOCKER_IMAGE_SET_STATUS_ACTION: 59,\n  DOCKER_IMAGE_STOP_ACTION: 60,\n  DOCKER_IMAGE_UPDATE_METADATA_ACTION: 61,\n  DOCKER_INSTALL_ACTION: 62,\n  DOCKER_REMOVE_USERNAME_ACTION: 63,\n  DOCKER_SET_STATUS_ACTION: 64,\n  DOCKER_START_ACTION: 65,\n  DOCKER_STOP_ACTION: 66,\n  DOCKER_STORE_USERNAME_ACTION: 67,\n  FILE_SYSTEM_ACTION: 68,\n  FILE_SYSTEM_COPY_ACTION: 69,\n  FILE_SYSTEM_MOVE_ACTION: 70,\n  FILE_SYSTEM_REMOVE_ACTION: 71,\n  FILE_SYSTEM_REPORT_SELECTION_ACTION: 72,\n  INFRARED_ACTION: 73,\n  INFRARED_HANDLE_RECEIVED_CODE_ACTION: 74,\n  INFRARED_SEND_CODE_ACTION: 75,\n  INFRARED_SET_SHOULD_PROPAGATE_ACTION: 76,\n  INFRARED_SET_SHOULD_RECEIVE_ACTION: 77,\n  INPUT_ACTION: 78,\n  INPUT_CANCEL_ACTION: 79,\n  INPUT_DEMAND_ACTION: 80,\n  INPUT_PROVIDE_ACTION: 81,\n  INPUT_RESOLVE_ACTION: 82,\n  IP_ACTION: 83,\n  IP_SET_IS_CONNECTED_ACTION: 84,\n  IP_UPDATE_INTERFACES_ACTION: 85,\n  KEYPAD_ACTION: 86,\n  KEYPAD_KEY_HOLD_ACTION: 87,\n  KEYPAD_KEY_PRESS_ACTION: 88,\n  KEYPAD_KEY_RELEASE_ACTION: 89,\n  KEYPAD_KEY_UNHOLD_ACTION: 90,\n  LIGHT_DM_ACTION: 91,\n  LIGHT_DM_CLEAR_ENABLED_STATE_ACTION: 92,\n  LIGHT_DM_UPDATE_STATE_ACTION: 93,\n  MAIN_ACTION: 94,\n  MENU_ACTION: 95,\n  MENU_CHOOSE_BY_ICON_ACTION: 96,\n  MENU_CHOOSE_BY_INDEX_ACTION: 97,\n  MENU_CHOOSE_BY_LABEL_ACTION: 98,\n  MENU_GO_BACK_ACTION: 99,\n  MENU_GO_HOME_ACTION: 100,\n  MENU_SCROLL_ACTION: 101,\n  NOTIFICATIONS_ACTION: 102,\n  NOTIFICATIONS_ADD_ACTION: 103,\n  NOTIFICATIONS_CLEAR_ACTION: 104,\n  NOTIFICATIONS_CLEAR_ALL_ACTION: 105,\n  NOTIFICATIONS_CLEAR_BY_ID_ACTION: 106,\n  NOTIFICATIONS_DISPLAY_ACTION: 107,\n  OPEN_APPLICATION_ACTION: 108,\n  POWER_ACTION: 109,\n  POWER_OFF_ACTION: 110,\n  R_PI_CONNECT_ACTION: 111,\n  R_PI_CONNECT_DONE_DOWNLOADING_ACTION: 112,\n  R_PI_CONNECT_SET_PENDING_ACTION: 113,\n  R_PI_CONNECT_SET_STATUS_ACTION: 114,\n  R_PI_CONNECT_START_DOWNLOADING_ACTION: 115,\n  R_PI_CONNECT_UPDATE_SERVICE_STATE_ACTION: 116,\n  REBOOT_ACTION: 117,\n  REGISTER_APP_ACTION: 118,\n  REGISTER_REGULAR_APP_ACTION: 119,\n  REGISTER_SETTING_APP_ACTION: 120,\n  REPLAY_RECORDED_SEQUENCE_ACTION: 121,\n  REPORT_REPLAYING_DONE_ACTION: 122,\n  RGB_RING_ACTION: 123,\n  RGB_RING_BLANK_ACTION: 124,\n  RGB_RING_BLINK_ACTION: 125,\n  RGB_RING_COLORFUL_COMMAND_ACTION: 126,\n  RGB_RING_COMMAND_ACTION: 127,\n  RGB_RING_FILL_DOWNFROM_ACTION: 128,\n  RGB_RING_FILL_UPTO_ACTION: 129,\n  RGB_RING_PROGRESS_WHEEL_ACTION: 130,\n  RGB_RING_PROGRESS_WHEEL_STEP_ACTION: 131,\n  RGB_RING_PULSE_ACTION: 132,\n  RGB_RING_RAINBOW_ACTION: 133,\n  RGB_RING_SEQUENCE_ACTION: 134,\n  RGB_RING_SET_ALL_ACTION: 135,\n  RGB_RING_SET_BRIGHTNESS_ACTION: 136,\n  RGB_RING_SET_ENABLED_ACTION: 137,\n  RGB_RING_SET_IS_BUSY_ACTION: 138,\n  RGB_RING_SPINNING_WHEEL_ACTION: 139,\n  RGB_RING_WAITABLE_COMMAND_ACTION: 140,\n  SSH_ACTION: 141,\n  SSH_CLEAR_ENABLED_STATE_ACTION: 142,\n  SSH_UPDATE_STATE_ACTION: 143,\n  SENSORS_ACTION: 144,\n  SENSORS_REPORT_READING_ACTION: 145,\n  SET_ARE_ENCLOSURES_VISIBLE_ACTION: 146,\n  SET_MENU_PATH_ACTION: 147,\n  SETTINGS_ACTION: 148,\n  SETTINGS_CLEAR_SERVICE_ERRORS_ACTION: 149,\n  SETTINGS_REPORT_SERVICE_ERROR_ACTION: 150,\n  SETTINGS_SERVICE_ACTION: 151,\n  SETTINGS_SERVICE_SET_IS_ENABLED_ACTION: 152,\n  SETTINGS_SERVICE_SET_LOG_LEVEL_ACTION: 153,\n  SETTINGS_SERVICE_SET_SHOULD_RESTART_ACTION: 154,\n  SETTINGS_SERVICE_SET_STATUS_ACTION: 155,\n  SETTINGS_SET_SERVICES_ACTION: 156,\n  SETTINGS_START_SERVICE_ACTION: 157,\n  SETTINGS_STOP_SERVICE_ACTION: 158,\n  SETTINGS_TOGGLE_BETA_VERSIONS_ACTION: 159,\n  SETTINGS_TOGGLE_PDB_SIGNAL_ACTION: 160,\n  SETTINGS_TOGGLE_VISUAL_DEBUG_ACTION: 161,\n  SPEECH_RECOGNITION_ACTION: 162,\n  SPEECH_RECOGNITION_REPORT_INTENT_DETECTION_ACTION: 163,\n  SPEECH_RECOGNITION_REPORT_SPEECH_ACTION: 164,\n  SPEECH_RECOGNITION_REPORT_WAKE_WORD_DETECTION_ACTION: 165,\n  SPEECH_RECOGNITION_SET_IS_ASSISTANT_ACTIVE_ACTION: 166,\n  SPEECH_RECOGNITION_SET_IS_INTENTS_ACTIVE_ACTION: 167,\n  SPEECH_RECOGNITION_SET_SELECTED_ENGINE_ACTION: 168,\n  SPEECH_SYNTHESIS_ACTION: 169,\n  SPEECH_SYNTHESIS_READ_TEXT_ACTION: 170,\n  SPEECH_SYNTHESIS_SET_SELECTED_ENGINE_ACTION: 171,\n  STATUS_ICONS_ACTION: 172,\n  STATUS_ICONS_REGISTER_ACTION: 173,\n  TOGGLE_RECORDING_ACTION: 174,\n  UPDATE_MANAGER_ACTION: 175,\n  UPDATE_MANAGER_REPORT_FAILED_CHECK_ACTION: 176,\n  UPDATE_MANAGER_REQUEST_CHECK_ACTION: 177,\n  UPDATE_MANAGER_REQUEST_UPDATE_ACTION: 178,\n  UPDATE_MANAGER_SET_VERSIONS_ACTION: 179,\n  USERS_ACTION: 180,\n  USERS_CREATE_USER_ACTION: 181,\n  USERS_DELETE_USER_ACTION: 182,\n  USERS_RESET_PASSWORD_ACTION: 183,\n  USERS_SET_USERS_ACTION: 184,\n  VS_CODE_ACTION: 185,\n  VS_CODE_DONE_DOWNLOADING_ACTION: 186,\n  VS_CODE_SET_PENDING_ACTION: 187,\n  VS_CODE_SET_STATUS_ACTION: 188,\n  VS_CODE_START_DOWNLOADING_ACTION: 189,\n  WI_FI_ACTION: 190,\n  WI_FI_INPUT_CONNECTION_ACTION: 191,\n  WI_FI_SET_HAS_VISITED_ONBOARDING_ACTION: 192,\n  WI_FI_UPDATE_ACTION: 193,\n  WI_FI_UPDATE_REQUEST_ACTION: 194\n};\n\n/**\n * @return {proto.ubo.v1.Action.ActionCase}\n */\nproto.ubo.v1.Action.prototype.getActionCase = function() {\n  return /** @type {proto.ubo.v1.Action.ActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Action.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Action.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Action.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Action} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Action.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    assistantAction: (f = msg.getAssistantAction()) && proto.ubo.v1.AssistantAction.toObject(includeInstance, f),\n    assistantAddMcpServerAction: (f = msg.getAssistantAddMcpServerAction()) && proto.ubo.v1.AssistantAddMcpServerAction.toObject(includeInstance, f),\n    assistantDeleteMcpServerAction: (f = msg.getAssistantDeleteMcpServerAction()) && proto.ubo.v1.AssistantDeleteMcpServerAction.toObject(includeInstance, f),\n    assistantDownloadOllamaModelAction: (f = msg.getAssistantDownloadOllamaModelAction()) && proto.ubo.v1.AssistantDownloadOllamaModelAction.toObject(includeInstance, f),\n    assistantReportAction: (f = msg.getAssistantReportAction()) && proto.ubo.v1.AssistantReportAction.toObject(includeInstance, f),\n    assistantSetIsActiveAction: (f = msg.getAssistantSetIsActiveAction()) && proto.ubo.v1.AssistantSetIsActiveAction.toObject(includeInstance, f),\n    assistantSetSelectedImageGeneratorAction: (f = msg.getAssistantSetSelectedImageGeneratorAction()) && proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.toObject(includeInstance, f),\n    assistantSetSelectedLlmAction: (f = msg.getAssistantSetSelectedLlmAction()) && proto.ubo.v1.AssistantSetSelectedLLMAction.toObject(includeInstance, f),\n    assistantSetSelectedModelAction: (f = msg.getAssistantSetSelectedModelAction()) && proto.ubo.v1.AssistantSetSelectedModelAction.toObject(includeInstance, f),\n    assistantSetSelectedSttAction: (f = msg.getAssistantSetSelectedSttAction()) && proto.ubo.v1.AssistantSetSelectedSTTAction.toObject(includeInstance, f),\n    assistantSetSelectedTtsAction: (f = msg.getAssistantSetSelectedTtsAction()) && proto.ubo.v1.AssistantSetSelectedTTSAction.toObject(includeInstance, f),\n    assistantStartListeningAction: (f = msg.getAssistantStartListeningAction()) && proto.ubo.v1.AssistantStartListeningAction.toObject(includeInstance, f),\n    assistantStopListeningAction: (f = msg.getAssistantStopListeningAction()) && proto.ubo.v1.AssistantStopListeningAction.toObject(includeInstance, f),\n    assistantSyncMcpServersAction: (f = msg.getAssistantSyncMcpServersAction()) && proto.ubo.v1.AssistantSyncMcpServersAction.toObject(includeInstance, f),\n    assistantToggleListeningAction: (f = msg.getAssistantToggleListeningAction()) && proto.ubo.v1.AssistantToggleListeningAction.toObject(includeInstance, f),\n    assistantToggleMcpServerAction: (f = msg.getAssistantToggleMcpServerAction()) && proto.ubo.v1.AssistantToggleMcpServerAction.toObject(includeInstance, f),\n    assistantUpdateProvidersAction: (f = msg.getAssistantUpdateProvidersAction()) && proto.ubo.v1.AssistantUpdateProvidersAction.toObject(includeInstance, f),\n    audioAction: (f = msg.getAudioAction()) && proto.ubo.v1.AudioAction.toObject(includeInstance, f),\n    audioChangeVolumeAction: (f = msg.getAudioChangeVolumeAction()) && proto.ubo.v1.AudioChangeVolumeAction.toObject(includeInstance, f),\n    audioInstallDriverAction: (f = msg.getAudioInstallDriverAction()) && proto.ubo.v1.AudioInstallDriverAction.toObject(includeInstance, f),\n    audioPlayAudioSampleAction: (f = msg.getAudioPlayAudioSampleAction()) && proto.ubo.v1.AudioPlayAudioSampleAction.toObject(includeInstance, f),\n    audioPlayAudioSequenceAction: (f = msg.getAudioPlayAudioSequenceAction()) && proto.ubo.v1.AudioPlayAudioSequenceAction.toObject(includeInstance, f),\n    audioPlayChimeAction: (f = msg.getAudioPlayChimeAction()) && proto.ubo.v1.AudioPlayChimeAction.toObject(includeInstance, f),\n    audioPlayRecordingAction: (f = msg.getAudioPlayRecordingAction()) && proto.ubo.v1.AudioPlayRecordingAction.toObject(includeInstance, f),\n    audioPlaybackDoneAction: (f = msg.getAudioPlaybackDoneAction()) && proto.ubo.v1.AudioPlaybackDoneAction.toObject(includeInstance, f),\n    audioReportSampleAction: (f = msg.getAudioReportSampleAction()) && proto.ubo.v1.AudioReportSampleAction.toObject(includeInstance, f),\n    audioSetMuteStatusAction: (f = msg.getAudioSetMuteStatusAction()) && proto.ubo.v1.AudioSetMuteStatusAction.toObject(includeInstance, f),\n    audioSetVolumeAction: (f = msg.getAudioSetVolumeAction()) && proto.ubo.v1.AudioSetVolumeAction.toObject(includeInstance, f),\n    audioStartRecordingAction: (f = msg.getAudioStartRecordingAction()) && proto.ubo.v1.AudioStartRecordingAction.toObject(includeInstance, f),\n    audioStopRecordingAction: (f = msg.getAudioStopRecordingAction()) && proto.ubo.v1.AudioStopRecordingAction.toObject(includeInstance, f),\n    audioToggleMuteStatusAction: (f = msg.getAudioToggleMuteStatusAction()) && proto.ubo.v1.AudioToggleMuteStatusAction.toObject(includeInstance, f),\n    audioToggleRecordingAction: (f = msg.getAudioToggleRecordingAction()) && proto.ubo.v1.AudioToggleRecordingAction.toObject(includeInstance, f),\n    cameraAction: (f = msg.getCameraAction()) && proto.ubo.v1.CameraAction.toObject(includeInstance, f),\n    cameraDetectAction: (f = msg.getCameraDetectAction()) && proto.ubo.v1.CameraDetectAction.toObject(includeInstance, f),\n    cameraInstallDriverAction: (f = msg.getCameraInstallDriverAction()) && proto.ubo.v1.CameraInstallDriverAction.toObject(includeInstance, f),\n    cameraReportBarcodeAction: (f = msg.getCameraReportBarcodeAction()) && proto.ubo.v1.CameraReportBarcodeAction.toObject(includeInstance, f),\n    cameraRestoreDefaultAction: (f = msg.getCameraRestoreDefaultAction()) && proto.ubo.v1.CameraRestoreDefaultAction.toObject(includeInstance, f),\n    cameraSetAvailableCamerasAction: (f = msg.getCameraSetAvailableCamerasAction()) && proto.ubo.v1.CameraSetAvailableCamerasAction.toObject(includeInstance, f),\n    cameraSetIndexAction: (f = msg.getCameraSetIndexAction()) && proto.ubo.v1.CameraSetIndexAction.toObject(includeInstance, f),\n    cameraStartViewfinderAction: (f = msg.getCameraStartViewfinderAction()) && proto.ubo.v1.CameraStartViewfinderAction.toObject(includeInstance, f),\n    closeApplicationAction: (f = msg.getCloseApplicationAction()) && proto.ubo.v1.CloseApplicationAction.toObject(includeInstance, f),\n    deregisterRegularAppAction: (f = msg.getDeregisterRegularAppAction()) && proto.ubo.v1.DeregisterRegularAppAction.toObject(includeInstance, f),\n    displayAction: (f = msg.getDisplayAction()) && proto.ubo.v1.DisplayAction.toObject(includeInstance, f),\n    displayBlankAction: (f = msg.getDisplayBlankAction()) && proto.ubo.v1.DisplayBlankAction.toObject(includeInstance, f),\n    displayPauseAction: (f = msg.getDisplayPauseAction()) && proto.ubo.v1.DisplayPauseAction.toObject(includeInstance, f),\n    displayRedrawAction: (f = msg.getDisplayRedrawAction()) && proto.ubo.v1.DisplayRedrawAction.toObject(includeInstance, f),\n    displayResumeAction: (f = msg.getDisplayResumeAction()) && proto.ubo.v1.DisplayResumeAction.toObject(includeInstance, f),\n    displaySetBlankTimeoutAction: (f = msg.getDisplaySetBlankTimeoutAction()) && proto.ubo.v1.DisplaySetBlankTimeoutAction.toObject(includeInstance, f),\n    displayUnblankAction: (f = msg.getDisplayUnblankAction()) && proto.ubo.v1.DisplayUnblankAction.toObject(includeInstance, f),\n    displayUpdateActivityAction: (f = msg.getDisplayUpdateActivityAction()) && proto.ubo.v1.DisplayUpdateActivityAction.toObject(includeInstance, f),\n    dockerAction: (f = msg.getDockerAction()) && proto.ubo.v1.DockerAction.toObject(includeInstance, f),\n    dockerImageAction: (f = msg.getDockerImageAction()) && proto.ubo.v1.DockerImageAction.toObject(includeInstance, f),\n    dockerImageFetchAction: (f = msg.getDockerImageFetchAction()) && proto.ubo.v1.DockerImageFetchAction.toObject(includeInstance, f),\n    dockerImageReleaseAction: (f = msg.getDockerImageReleaseAction()) && proto.ubo.v1.DockerImageReleaseAction.toObject(includeInstance, f),\n    dockerImageRemoveAction: (f = msg.getDockerImageRemoveAction()) && proto.ubo.v1.DockerImageRemoveAction.toObject(includeInstance, f),\n    dockerImageRemoveContainerAction: (f = msg.getDockerImageRemoveContainerAction()) && proto.ubo.v1.DockerImageRemoveContainerAction.toObject(includeInstance, f),\n    dockerImageRunAction: (f = msg.getDockerImageRunAction()) && proto.ubo.v1.DockerImageRunAction.toObject(includeInstance, f),\n    dockerImageSetDockerIdAction: (f = msg.getDockerImageSetDockerIdAction()) && proto.ubo.v1.DockerImageSetDockerIdAction.toObject(includeInstance, f),\n    dockerImageSetStatusAction: (f = msg.getDockerImageSetStatusAction()) && proto.ubo.v1.DockerImageSetStatusAction.toObject(includeInstance, f),\n    dockerImageStopAction: (f = msg.getDockerImageStopAction()) && proto.ubo.v1.DockerImageStopAction.toObject(includeInstance, f),\n    dockerImageUpdateMetadataAction: (f = msg.getDockerImageUpdateMetadataAction()) && proto.ubo.v1.DockerImageUpdateMetadataAction.toObject(includeInstance, f),\n    dockerInstallAction: (f = msg.getDockerInstallAction()) && proto.ubo.v1.DockerInstallAction.toObject(includeInstance, f),\n    dockerRemoveUsernameAction: (f = msg.getDockerRemoveUsernameAction()) && proto.ubo.v1.DockerRemoveUsernameAction.toObject(includeInstance, f),\n    dockerSetStatusAction: (f = msg.getDockerSetStatusAction()) && proto.ubo.v1.DockerSetStatusAction.toObject(includeInstance, f),\n    dockerStartAction: (f = msg.getDockerStartAction()) && proto.ubo.v1.DockerStartAction.toObject(includeInstance, f),\n    dockerStopAction: (f = msg.getDockerStopAction()) && proto.ubo.v1.DockerStopAction.toObject(includeInstance, f),\n    dockerStoreUsernameAction: (f = msg.getDockerStoreUsernameAction()) && proto.ubo.v1.DockerStoreUsernameAction.toObject(includeInstance, f),\n    fileSystemAction: (f = msg.getFileSystemAction()) && proto.ubo.v1.FileSystemAction.toObject(includeInstance, f),\n    fileSystemCopyAction: (f = msg.getFileSystemCopyAction()) && proto.ubo.v1.FileSystemCopyAction.toObject(includeInstance, f),\n    fileSystemMoveAction: (f = msg.getFileSystemMoveAction()) && proto.ubo.v1.FileSystemMoveAction.toObject(includeInstance, f),\n    fileSystemRemoveAction: (f = msg.getFileSystemRemoveAction()) && proto.ubo.v1.FileSystemRemoveAction.toObject(includeInstance, f),\n    fileSystemReportSelectionAction: (f = msg.getFileSystemReportSelectionAction()) && proto.ubo.v1.FileSystemReportSelectionAction.toObject(includeInstance, f),\n    infraredAction: (f = msg.getInfraredAction()) && proto.ubo.v1.InfraredAction.toObject(includeInstance, f),\n    infraredHandleReceivedCodeAction: (f = msg.getInfraredHandleReceivedCodeAction()) && proto.ubo.v1.InfraredHandleReceivedCodeAction.toObject(includeInstance, f),\n    infraredSendCodeAction: (f = msg.getInfraredSendCodeAction()) && proto.ubo.v1.InfraredSendCodeAction.toObject(includeInstance, f),\n    infraredSetShouldPropagateAction: (f = msg.getInfraredSetShouldPropagateAction()) && proto.ubo.v1.InfraredSetShouldPropagateAction.toObject(includeInstance, f),\n    infraredSetShouldReceiveAction: (f = msg.getInfraredSetShouldReceiveAction()) && proto.ubo.v1.InfraredSetShouldReceiveAction.toObject(includeInstance, f),\n    inputAction: (f = msg.getInputAction()) && proto.ubo.v1.InputAction.toObject(includeInstance, f),\n    inputCancelAction: (f = msg.getInputCancelAction()) && proto.ubo.v1.InputCancelAction.toObject(includeInstance, f),\n    inputDemandAction: (f = msg.getInputDemandAction()) && proto.ubo.v1.InputDemandAction.toObject(includeInstance, f),\n    inputProvideAction: (f = msg.getInputProvideAction()) && proto.ubo.v1.InputProvideAction.toObject(includeInstance, f),\n    inputResolveAction: (f = msg.getInputResolveAction()) && proto.ubo.v1.InputResolveAction.toObject(includeInstance, f),\n    ipAction: (f = msg.getIpAction()) && proto.ubo.v1.IpAction.toObject(includeInstance, f),\n    ipSetIsConnectedAction: (f = msg.getIpSetIsConnectedAction()) && proto.ubo.v1.IpSetIsConnectedAction.toObject(includeInstance, f),\n    ipUpdateInterfacesAction: (f = msg.getIpUpdateInterfacesAction()) && proto.ubo.v1.IpUpdateInterfacesAction.toObject(includeInstance, f),\n    keypadAction: (f = msg.getKeypadAction()) && proto.ubo.v1.KeypadAction.toObject(includeInstance, f),\n    keypadKeyHoldAction: (f = msg.getKeypadKeyHoldAction()) && proto.ubo.v1.KeypadKeyHoldAction.toObject(includeInstance, f),\n    keypadKeyPressAction: (f = msg.getKeypadKeyPressAction()) && proto.ubo.v1.KeypadKeyPressAction.toObject(includeInstance, f),\n    keypadKeyReleaseAction: (f = msg.getKeypadKeyReleaseAction()) && proto.ubo.v1.KeypadKeyReleaseAction.toObject(includeInstance, f),\n    keypadKeyUnholdAction: (f = msg.getKeypadKeyUnholdAction()) && proto.ubo.v1.KeypadKeyUnholdAction.toObject(includeInstance, f),\n    lightDmAction: (f = msg.getLightDmAction()) && proto.ubo.v1.LightDMAction.toObject(includeInstance, f),\n    lightDmClearEnabledStateAction: (f = msg.getLightDmClearEnabledStateAction()) && proto.ubo.v1.LightDMClearEnabledStateAction.toObject(includeInstance, f),\n    lightDmUpdateStateAction: (f = msg.getLightDmUpdateStateAction()) && proto.ubo.v1.LightDMUpdateStateAction.toObject(includeInstance, f),\n    mainAction: (f = msg.getMainAction()) && proto.ubo.v1.MainAction.toObject(includeInstance, f),\n    menuAction: (f = msg.getMenuAction()) && proto.ubo.v1.MenuAction.toObject(includeInstance, f),\n    menuChooseByIconAction: (f = msg.getMenuChooseByIconAction()) && proto.ubo.v1.MenuChooseByIconAction.toObject(includeInstance, f),\n    menuChooseByIndexAction: (f = msg.getMenuChooseByIndexAction()) && proto.ubo.v1.MenuChooseByIndexAction.toObject(includeInstance, f),\n    menuChooseByLabelAction: (f = msg.getMenuChooseByLabelAction()) && proto.ubo.v1.MenuChooseByLabelAction.toObject(includeInstance, f),\n    menuGoBackAction: (f = msg.getMenuGoBackAction()) && proto.ubo.v1.MenuGoBackAction.toObject(includeInstance, f),\n    menuGoHomeAction: (f = msg.getMenuGoHomeAction()) && proto.ubo.v1.MenuGoHomeAction.toObject(includeInstance, f),\n    menuScrollAction: (f = msg.getMenuScrollAction()) && proto.ubo.v1.MenuScrollAction.toObject(includeInstance, f),\n    notificationsAction: (f = msg.getNotificationsAction()) && proto.ubo.v1.NotificationsAction.toObject(includeInstance, f),\n    notificationsAddAction: (f = msg.getNotificationsAddAction()) && proto.ubo.v1.NotificationsAddAction.toObject(includeInstance, f),\n    notificationsClearAction: (f = msg.getNotificationsClearAction()) && proto.ubo.v1.NotificationsClearAction.toObject(includeInstance, f),\n    notificationsClearAllAction: (f = msg.getNotificationsClearAllAction()) && proto.ubo.v1.NotificationsClearAllAction.toObject(includeInstance, f),\n    notificationsClearByIdAction: (f = msg.getNotificationsClearByIdAction()) && proto.ubo.v1.NotificationsClearByIdAction.toObject(includeInstance, f),\n    notificationsDisplayAction: (f = msg.getNotificationsDisplayAction()) && proto.ubo.v1.NotificationsDisplayAction.toObject(includeInstance, f),\n    openApplicationAction: (f = msg.getOpenApplicationAction()) && proto.ubo.v1.OpenApplicationAction.toObject(includeInstance, f),\n    powerAction: (f = msg.getPowerAction()) && proto.ubo.v1.PowerAction.toObject(includeInstance, f),\n    powerOffAction: (f = msg.getPowerOffAction()) && proto.ubo.v1.PowerOffAction.toObject(includeInstance, f),\n    rPiConnectAction: (f = msg.getRPiConnectAction()) && proto.ubo.v1.RPiConnectAction.toObject(includeInstance, f),\n    rPiConnectDoneDownloadingAction: (f = msg.getRPiConnectDoneDownloadingAction()) && proto.ubo.v1.RPiConnectDoneDownloadingAction.toObject(includeInstance, f),\n    rPiConnectSetPendingAction: (f = msg.getRPiConnectSetPendingAction()) && proto.ubo.v1.RPiConnectSetPendingAction.toObject(includeInstance, f),\n    rPiConnectSetStatusAction: (f = msg.getRPiConnectSetStatusAction()) && proto.ubo.v1.RPiConnectSetStatusAction.toObject(includeInstance, f),\n    rPiConnectStartDownloadingAction: (f = msg.getRPiConnectStartDownloadingAction()) && proto.ubo.v1.RPiConnectStartDownloadingAction.toObject(includeInstance, f),\n    rPiConnectUpdateServiceStateAction: (f = msg.getRPiConnectUpdateServiceStateAction()) && proto.ubo.v1.RPiConnectUpdateServiceStateAction.toObject(includeInstance, f),\n    rebootAction: (f = msg.getRebootAction()) && proto.ubo.v1.RebootAction.toObject(includeInstance, f),\n    registerAppAction: (f = msg.getRegisterAppAction()) && proto.ubo.v1.RegisterAppAction.toObject(includeInstance, f),\n    registerRegularAppAction: (f = msg.getRegisterRegularAppAction()) && proto.ubo.v1.RegisterRegularAppAction.toObject(includeInstance, f),\n    registerSettingAppAction: (f = msg.getRegisterSettingAppAction()) && proto.ubo.v1.RegisterSettingAppAction.toObject(includeInstance, f),\n    replayRecordedSequenceAction: (f = msg.getReplayRecordedSequenceAction()) && proto.ubo.v1.ReplayRecordedSequenceAction.toObject(includeInstance, f),\n    reportReplayingDoneAction: (f = msg.getReportReplayingDoneAction()) && proto.ubo.v1.ReportReplayingDoneAction.toObject(includeInstance, f),\n    rgbRingAction: (f = msg.getRgbRingAction()) && proto.ubo.v1.RgbRingAction.toObject(includeInstance, f),\n    rgbRingBlankAction: (f = msg.getRgbRingBlankAction()) && proto.ubo.v1.RgbRingBlankAction.toObject(includeInstance, f),\n    rgbRingBlinkAction: (f = msg.getRgbRingBlinkAction()) && proto.ubo.v1.RgbRingBlinkAction.toObject(includeInstance, f),\n    rgbRingColorfulCommandAction: (f = msg.getRgbRingColorfulCommandAction()) && proto.ubo.v1.RgbRingColorfulCommandAction.toObject(includeInstance, f),\n    rgbRingCommandAction: (f = msg.getRgbRingCommandAction()) && proto.ubo.v1.RgbRingCommandAction.toObject(includeInstance, f),\n    rgbRingFillDownfromAction: (f = msg.getRgbRingFillDownfromAction()) && proto.ubo.v1.RgbRingFillDownfromAction.toObject(includeInstance, f),\n    rgbRingFillUptoAction: (f = msg.getRgbRingFillUptoAction()) && proto.ubo.v1.RgbRingFillUptoAction.toObject(includeInstance, f),\n    rgbRingProgressWheelAction: (f = msg.getRgbRingProgressWheelAction()) && proto.ubo.v1.RgbRingProgressWheelAction.toObject(includeInstance, f),\n    rgbRingProgressWheelStepAction: (f = msg.getRgbRingProgressWheelStepAction()) && proto.ubo.v1.RgbRingProgressWheelStepAction.toObject(includeInstance, f),\n    rgbRingPulseAction: (f = msg.getRgbRingPulseAction()) && proto.ubo.v1.RgbRingPulseAction.toObject(includeInstance, f),\n    rgbRingRainbowAction: (f = msg.getRgbRingRainbowAction()) && proto.ubo.v1.RgbRingRainbowAction.toObject(includeInstance, f),\n    rgbRingSequenceAction: (f = msg.getRgbRingSequenceAction()) && proto.ubo.v1.RgbRingSequenceAction.toObject(includeInstance, f),\n    rgbRingSetAllAction: (f = msg.getRgbRingSetAllAction()) && proto.ubo.v1.RgbRingSetAllAction.toObject(includeInstance, f),\n    rgbRingSetBrightnessAction: (f = msg.getRgbRingSetBrightnessAction()) && proto.ubo.v1.RgbRingSetBrightnessAction.toObject(includeInstance, f),\n    rgbRingSetEnabledAction: (f = msg.getRgbRingSetEnabledAction()) && proto.ubo.v1.RgbRingSetEnabledAction.toObject(includeInstance, f),\n    rgbRingSetIsBusyAction: (f = msg.getRgbRingSetIsBusyAction()) && proto.ubo.v1.RgbRingSetIsBusyAction.toObject(includeInstance, f),\n    rgbRingSpinningWheelAction: (f = msg.getRgbRingSpinningWheelAction()) && proto.ubo.v1.RgbRingSpinningWheelAction.toObject(includeInstance, f),\n    rgbRingWaitableCommandAction: (f = msg.getRgbRingWaitableCommandAction()) && proto.ubo.v1.RgbRingWaitableCommandAction.toObject(includeInstance, f),\n    sshAction: (f = msg.getSshAction()) && proto.ubo.v1.SSHAction.toObject(includeInstance, f),\n    sshClearEnabledStateAction: (f = msg.getSshClearEnabledStateAction()) && proto.ubo.v1.SSHClearEnabledStateAction.toObject(includeInstance, f),\n    sshUpdateStateAction: (f = msg.getSshUpdateStateAction()) && proto.ubo.v1.SSHUpdateStateAction.toObject(includeInstance, f),\n    sensorsAction: (f = msg.getSensorsAction()) && proto.ubo.v1.SensorsAction.toObject(includeInstance, f),\n    sensorsReportReadingAction: (f = msg.getSensorsReportReadingAction()) && proto.ubo.v1.SensorsReportReadingAction.toObject(includeInstance, f),\n    setAreEnclosuresVisibleAction: (f = msg.getSetAreEnclosuresVisibleAction()) && proto.ubo.v1.SetAreEnclosuresVisibleAction.toObject(includeInstance, f),\n    setMenuPathAction: (f = msg.getSetMenuPathAction()) && proto.ubo.v1.SetMenuPathAction.toObject(includeInstance, f),\n    settingsAction: (f = msg.getSettingsAction()) && proto.ubo.v1.SettingsAction.toObject(includeInstance, f),\n    settingsClearServiceErrorsAction: (f = msg.getSettingsClearServiceErrorsAction()) && proto.ubo.v1.SettingsClearServiceErrorsAction.toObject(includeInstance, f),\n    settingsReportServiceErrorAction: (f = msg.getSettingsReportServiceErrorAction()) && proto.ubo.v1.SettingsReportServiceErrorAction.toObject(includeInstance, f),\n    settingsServiceAction: (f = msg.getSettingsServiceAction()) && proto.ubo.v1.SettingsServiceAction.toObject(includeInstance, f),\n    settingsServiceSetIsEnabledAction: (f = msg.getSettingsServiceSetIsEnabledAction()) && proto.ubo.v1.SettingsServiceSetIsEnabledAction.toObject(includeInstance, f),\n    settingsServiceSetLogLevelAction: (f = msg.getSettingsServiceSetLogLevelAction()) && proto.ubo.v1.SettingsServiceSetLogLevelAction.toObject(includeInstance, f),\n    settingsServiceSetShouldRestartAction: (f = msg.getSettingsServiceSetShouldRestartAction()) && proto.ubo.v1.SettingsServiceSetShouldRestartAction.toObject(includeInstance, f),\n    settingsServiceSetStatusAction: (f = msg.getSettingsServiceSetStatusAction()) && proto.ubo.v1.SettingsServiceSetStatusAction.toObject(includeInstance, f),\n    settingsSetServicesAction: (f = msg.getSettingsSetServicesAction()) && proto.ubo.v1.SettingsSetServicesAction.toObject(includeInstance, f),\n    settingsStartServiceAction: (f = msg.getSettingsStartServiceAction()) && proto.ubo.v1.SettingsStartServiceAction.toObject(includeInstance, f),\n    settingsStopServiceAction: (f = msg.getSettingsStopServiceAction()) && proto.ubo.v1.SettingsStopServiceAction.toObject(includeInstance, f),\n    settingsToggleBetaVersionsAction: (f = msg.getSettingsToggleBetaVersionsAction()) && proto.ubo.v1.SettingsToggleBetaVersionsAction.toObject(includeInstance, f),\n    settingsTogglePdbSignalAction: (f = msg.getSettingsTogglePdbSignalAction()) && proto.ubo.v1.SettingsTogglePdbSignalAction.toObject(includeInstance, f),\n    settingsToggleVisualDebugAction: (f = msg.getSettingsToggleVisualDebugAction()) && proto.ubo.v1.SettingsToggleVisualDebugAction.toObject(includeInstance, f),\n    speechRecognitionAction: (f = msg.getSpeechRecognitionAction()) && proto.ubo.v1.SpeechRecognitionAction.toObject(includeInstance, f),\n    speechRecognitionReportIntentDetectionAction: (f = msg.getSpeechRecognitionReportIntentDetectionAction()) && proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.toObject(includeInstance, f),\n    speechRecognitionReportSpeechAction: (f = msg.getSpeechRecognitionReportSpeechAction()) && proto.ubo.v1.SpeechRecognitionReportSpeechAction.toObject(includeInstance, f),\n    speechRecognitionReportWakeWordDetectionAction: (f = msg.getSpeechRecognitionReportWakeWordDetectionAction()) && proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.toObject(includeInstance, f),\n    speechRecognitionSetIsAssistantActiveAction: (f = msg.getSpeechRecognitionSetIsAssistantActiveAction()) && proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.toObject(includeInstance, f),\n    speechRecognitionSetIsIntentsActiveAction: (f = msg.getSpeechRecognitionSetIsIntentsActiveAction()) && proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.toObject(includeInstance, f),\n    speechRecognitionSetSelectedEngineAction: (f = msg.getSpeechRecognitionSetSelectedEngineAction()) && proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.toObject(includeInstance, f),\n    speechSynthesisAction: (f = msg.getSpeechSynthesisAction()) && proto.ubo.v1.SpeechSynthesisAction.toObject(includeInstance, f),\n    speechSynthesisReadTextAction: (f = msg.getSpeechSynthesisReadTextAction()) && proto.ubo.v1.SpeechSynthesisReadTextAction.toObject(includeInstance, f),\n    speechSynthesisSetSelectedEngineAction: (f = msg.getSpeechSynthesisSetSelectedEngineAction()) && proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.toObject(includeInstance, f),\n    statusIconsAction: (f = msg.getStatusIconsAction()) && proto.ubo.v1.StatusIconsAction.toObject(includeInstance, f),\n    statusIconsRegisterAction: (f = msg.getStatusIconsRegisterAction()) && proto.ubo.v1.StatusIconsRegisterAction.toObject(includeInstance, f),\n    toggleRecordingAction: (f = msg.getToggleRecordingAction()) && proto.ubo.v1.ToggleRecordingAction.toObject(includeInstance, f),\n    updateManagerAction: (f = msg.getUpdateManagerAction()) && proto.ubo.v1.UpdateManagerAction.toObject(includeInstance, f),\n    updateManagerReportFailedCheckAction: (f = msg.getUpdateManagerReportFailedCheckAction()) && proto.ubo.v1.UpdateManagerReportFailedCheckAction.toObject(includeInstance, f),\n    updateManagerRequestCheckAction: (f = msg.getUpdateManagerRequestCheckAction()) && proto.ubo.v1.UpdateManagerRequestCheckAction.toObject(includeInstance, f),\n    updateManagerRequestUpdateAction: (f = msg.getUpdateManagerRequestUpdateAction()) && proto.ubo.v1.UpdateManagerRequestUpdateAction.toObject(includeInstance, f),\n    updateManagerSetVersionsAction: (f = msg.getUpdateManagerSetVersionsAction()) && proto.ubo.v1.UpdateManagerSetVersionsAction.toObject(includeInstance, f),\n    usersAction: (f = msg.getUsersAction()) && proto.ubo.v1.UsersAction.toObject(includeInstance, f),\n    usersCreateUserAction: (f = msg.getUsersCreateUserAction()) && proto.ubo.v1.UsersCreateUserAction.toObject(includeInstance, f),\n    usersDeleteUserAction: (f = msg.getUsersDeleteUserAction()) && proto.ubo.v1.UsersDeleteUserAction.toObject(includeInstance, f),\n    usersResetPasswordAction: (f = msg.getUsersResetPasswordAction()) && proto.ubo.v1.UsersResetPasswordAction.toObject(includeInstance, f),\n    usersSetUsersAction: (f = msg.getUsersSetUsersAction()) && proto.ubo.v1.UsersSetUsersAction.toObject(includeInstance, f),\n    vsCodeAction: (f = msg.getVsCodeAction()) && proto.ubo.v1.VSCodeAction.toObject(includeInstance, f),\n    vsCodeDoneDownloadingAction: (f = msg.getVsCodeDoneDownloadingAction()) && proto.ubo.v1.VSCodeDoneDownloadingAction.toObject(includeInstance, f),\n    vsCodeSetPendingAction: (f = msg.getVsCodeSetPendingAction()) && proto.ubo.v1.VSCodeSetPendingAction.toObject(includeInstance, f),\n    vsCodeSetStatusAction: (f = msg.getVsCodeSetStatusAction()) && proto.ubo.v1.VSCodeSetStatusAction.toObject(includeInstance, f),\n    vsCodeStartDownloadingAction: (f = msg.getVsCodeStartDownloadingAction()) && proto.ubo.v1.VSCodeStartDownloadingAction.toObject(includeInstance, f),\n    wiFiAction: (f = msg.getWiFiAction()) && proto.ubo.v1.WiFiAction.toObject(includeInstance, f),\n    wiFiInputConnectionAction: (f = msg.getWiFiInputConnectionAction()) && proto.ubo.v1.WiFiInputConnectionAction.toObject(includeInstance, f),\n    wiFiSetHasVisitedOnboardingAction: (f = msg.getWiFiSetHasVisitedOnboardingAction()) && proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.toObject(includeInstance, f),\n    wiFiUpdateAction: (f = msg.getWiFiUpdateAction()) && proto.ubo.v1.WiFiUpdateAction.toObject(includeInstance, f),\n    wiFiUpdateRequestAction: (f = msg.getWiFiUpdateRequestAction()) && proto.ubo.v1.WiFiUpdateRequestAction.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Action}\n */\nproto.ubo.v1.Action.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Action;\n  return proto.ubo.v1.Action.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Action} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Action}\n */\nproto.ubo.v1.Action.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.AssistantAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantAction.deserializeBinaryFromReader);\n      msg.setAssistantAction(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AssistantAddMcpServerAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantAddMcpServerAction.deserializeBinaryFromReader);\n      msg.setAssistantAddMcpServerAction(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AssistantDeleteMcpServerAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinaryFromReader);\n      msg.setAssistantDeleteMcpServerAction(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.AssistantDownloadOllamaModelAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinaryFromReader);\n      msg.setAssistantDownloadOllamaModelAction(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.AssistantReportAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantReportAction.deserializeBinaryFromReader);\n      msg.setAssistantReportAction(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.AssistantSetIsActiveAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSetIsActiveAction.deserializeBinaryFromReader);\n      msg.setAssistantSetIsActiveAction(value);\n      break;\n    case 7:\n      var value = new proto.ubo.v1.AssistantSetSelectedImageGeneratorAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinaryFromReader);\n      msg.setAssistantSetSelectedImageGeneratorAction(value);\n      break;\n    case 8:\n      var value = new proto.ubo.v1.AssistantSetSelectedLLMAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinaryFromReader);\n      msg.setAssistantSetSelectedLlmAction(value);\n      break;\n    case 9:\n      var value = new proto.ubo.v1.AssistantSetSelectedModelAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinaryFromReader);\n      msg.setAssistantSetSelectedModelAction(value);\n      break;\n    case 10:\n      var value = new proto.ubo.v1.AssistantSetSelectedSTTAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinaryFromReader);\n      msg.setAssistantSetSelectedSttAction(value);\n      break;\n    case 11:\n      var value = new proto.ubo.v1.AssistantSetSelectedTTSAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinaryFromReader);\n      msg.setAssistantSetSelectedTtsAction(value);\n      break;\n    case 12:\n      var value = new proto.ubo.v1.AssistantStartListeningAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantStartListeningAction.deserializeBinaryFromReader);\n      msg.setAssistantStartListeningAction(value);\n      break;\n    case 13:\n      var value = new proto.ubo.v1.AssistantStopListeningAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantStopListeningAction.deserializeBinaryFromReader);\n      msg.setAssistantStopListeningAction(value);\n      break;\n    case 14:\n      var value = new proto.ubo.v1.AssistantSyncMcpServersAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinaryFromReader);\n      msg.setAssistantSyncMcpServersAction(value);\n      break;\n    case 15:\n      var value = new proto.ubo.v1.AssistantToggleListeningAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantToggleListeningAction.deserializeBinaryFromReader);\n      msg.setAssistantToggleListeningAction(value);\n      break;\n    case 16:\n      var value = new proto.ubo.v1.AssistantToggleMcpServerAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinaryFromReader);\n      msg.setAssistantToggleMcpServerAction(value);\n      break;\n    case 17:\n      var value = new proto.ubo.v1.AssistantUpdateProvidersAction;\n      reader.readMessage(value,proto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinaryFromReader);\n      msg.setAssistantUpdateProvidersAction(value);\n      break;\n    case 18:\n      var value = new proto.ubo.v1.AudioAction;\n      reader.readMessage(value,proto.ubo.v1.AudioAction.deserializeBinaryFromReader);\n      msg.setAudioAction(value);\n      break;\n    case 19:\n      var value = new proto.ubo.v1.AudioChangeVolumeAction;\n      reader.readMessage(value,proto.ubo.v1.AudioChangeVolumeAction.deserializeBinaryFromReader);\n      msg.setAudioChangeVolumeAction(value);\n      break;\n    case 20:\n      var value = new proto.ubo.v1.AudioInstallDriverAction;\n      reader.readMessage(value,proto.ubo.v1.AudioInstallDriverAction.deserializeBinaryFromReader);\n      msg.setAudioInstallDriverAction(value);\n      break;\n    case 21:\n      var value = new proto.ubo.v1.AudioPlayAudioSampleAction;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinaryFromReader);\n      msg.setAudioPlayAudioSampleAction(value);\n      break;\n    case 22:\n      var value = new proto.ubo.v1.AudioPlayAudioSequenceAction;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinaryFromReader);\n      msg.setAudioPlayAudioSequenceAction(value);\n      break;\n    case 23:\n      var value = new proto.ubo.v1.AudioPlayChimeAction;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayChimeAction.deserializeBinaryFromReader);\n      msg.setAudioPlayChimeAction(value);\n      break;\n    case 24:\n      var value = new proto.ubo.v1.AudioPlayRecordingAction;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayRecordingAction.deserializeBinaryFromReader);\n      msg.setAudioPlayRecordingAction(value);\n      break;\n    case 25:\n      var value = new proto.ubo.v1.AudioPlaybackDoneAction;\n      reader.readMessage(value,proto.ubo.v1.AudioPlaybackDoneAction.deserializeBinaryFromReader);\n      msg.setAudioPlaybackDoneAction(value);\n      break;\n    case 26:\n      var value = new proto.ubo.v1.AudioReportSampleAction;\n      reader.readMessage(value,proto.ubo.v1.AudioReportSampleAction.deserializeBinaryFromReader);\n      msg.setAudioReportSampleAction(value);\n      break;\n    case 27:\n      var value = new proto.ubo.v1.AudioSetMuteStatusAction;\n      reader.readMessage(value,proto.ubo.v1.AudioSetMuteStatusAction.deserializeBinaryFromReader);\n      msg.setAudioSetMuteStatusAction(value);\n      break;\n    case 28:\n      var value = new proto.ubo.v1.AudioSetVolumeAction;\n      reader.readMessage(value,proto.ubo.v1.AudioSetVolumeAction.deserializeBinaryFromReader);\n      msg.setAudioSetVolumeAction(value);\n      break;\n    case 29:\n      var value = new proto.ubo.v1.AudioStartRecordingAction;\n      reader.readMessage(value,proto.ubo.v1.AudioStartRecordingAction.deserializeBinaryFromReader);\n      msg.setAudioStartRecordingAction(value);\n      break;\n    case 30:\n      var value = new proto.ubo.v1.AudioStopRecordingAction;\n      reader.readMessage(value,proto.ubo.v1.AudioStopRecordingAction.deserializeBinaryFromReader);\n      msg.setAudioStopRecordingAction(value);\n      break;\n    case 31:\n      var value = new proto.ubo.v1.AudioToggleMuteStatusAction;\n      reader.readMessage(value,proto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinaryFromReader);\n      msg.setAudioToggleMuteStatusAction(value);\n      break;\n    case 32:\n      var value = new proto.ubo.v1.AudioToggleRecordingAction;\n      reader.readMessage(value,proto.ubo.v1.AudioToggleRecordingAction.deserializeBinaryFromReader);\n      msg.setAudioToggleRecordingAction(value);\n      break;\n    case 33:\n      var value = new proto.ubo.v1.CameraAction;\n      reader.readMessage(value,proto.ubo.v1.CameraAction.deserializeBinaryFromReader);\n      msg.setCameraAction(value);\n      break;\n    case 34:\n      var value = new proto.ubo.v1.CameraDetectAction;\n      reader.readMessage(value,proto.ubo.v1.CameraDetectAction.deserializeBinaryFromReader);\n      msg.setCameraDetectAction(value);\n      break;\n    case 35:\n      var value = new proto.ubo.v1.CameraInstallDriverAction;\n      reader.readMessage(value,proto.ubo.v1.CameraInstallDriverAction.deserializeBinaryFromReader);\n      msg.setCameraInstallDriverAction(value);\n      break;\n    case 36:\n      var value = new proto.ubo.v1.CameraReportBarcodeAction;\n      reader.readMessage(value,proto.ubo.v1.CameraReportBarcodeAction.deserializeBinaryFromReader);\n      msg.setCameraReportBarcodeAction(value);\n      break;\n    case 37:\n      var value = new proto.ubo.v1.CameraRestoreDefaultAction;\n      reader.readMessage(value,proto.ubo.v1.CameraRestoreDefaultAction.deserializeBinaryFromReader);\n      msg.setCameraRestoreDefaultAction(value);\n      break;\n    case 38:\n      var value = new proto.ubo.v1.CameraSetAvailableCamerasAction;\n      reader.readMessage(value,proto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinaryFromReader);\n      msg.setCameraSetAvailableCamerasAction(value);\n      break;\n    case 39:\n      var value = new proto.ubo.v1.CameraSetIndexAction;\n      reader.readMessage(value,proto.ubo.v1.CameraSetIndexAction.deserializeBinaryFromReader);\n      msg.setCameraSetIndexAction(value);\n      break;\n    case 40:\n      var value = new proto.ubo.v1.CameraStartViewfinderAction;\n      reader.readMessage(value,proto.ubo.v1.CameraStartViewfinderAction.deserializeBinaryFromReader);\n      msg.setCameraStartViewfinderAction(value);\n      break;\n    case 41:\n      var value = new proto.ubo.v1.CloseApplicationAction;\n      reader.readMessage(value,proto.ubo.v1.CloseApplicationAction.deserializeBinaryFromReader);\n      msg.setCloseApplicationAction(value);\n      break;\n    case 42:\n      var value = new proto.ubo.v1.DeregisterRegularAppAction;\n      reader.readMessage(value,proto.ubo.v1.DeregisterRegularAppAction.deserializeBinaryFromReader);\n      msg.setDeregisterRegularAppAction(value);\n      break;\n    case 43:\n      var value = new proto.ubo.v1.DisplayAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayAction.deserializeBinaryFromReader);\n      msg.setDisplayAction(value);\n      break;\n    case 44:\n      var value = new proto.ubo.v1.DisplayBlankAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayBlankAction.deserializeBinaryFromReader);\n      msg.setDisplayBlankAction(value);\n      break;\n    case 45:\n      var value = new proto.ubo.v1.DisplayPauseAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayPauseAction.deserializeBinaryFromReader);\n      msg.setDisplayPauseAction(value);\n      break;\n    case 46:\n      var value = new proto.ubo.v1.DisplayRedrawAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayRedrawAction.deserializeBinaryFromReader);\n      msg.setDisplayRedrawAction(value);\n      break;\n    case 47:\n      var value = new proto.ubo.v1.DisplayResumeAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayResumeAction.deserializeBinaryFromReader);\n      msg.setDisplayResumeAction(value);\n      break;\n    case 48:\n      var value = new proto.ubo.v1.DisplaySetBlankTimeoutAction;\n      reader.readMessage(value,proto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinaryFromReader);\n      msg.setDisplaySetBlankTimeoutAction(value);\n      break;\n    case 49:\n      var value = new proto.ubo.v1.DisplayUnblankAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayUnblankAction.deserializeBinaryFromReader);\n      msg.setDisplayUnblankAction(value);\n      break;\n    case 50:\n      var value = new proto.ubo.v1.DisplayUpdateActivityAction;\n      reader.readMessage(value,proto.ubo.v1.DisplayUpdateActivityAction.deserializeBinaryFromReader);\n      msg.setDisplayUpdateActivityAction(value);\n      break;\n    case 51:\n      var value = new proto.ubo.v1.DockerAction;\n      reader.readMessage(value,proto.ubo.v1.DockerAction.deserializeBinaryFromReader);\n      msg.setDockerAction(value);\n      break;\n    case 52:\n      var value = new proto.ubo.v1.DockerImageAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageAction.deserializeBinaryFromReader);\n      msg.setDockerImageAction(value);\n      break;\n    case 53:\n      var value = new proto.ubo.v1.DockerImageFetchAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageFetchAction.deserializeBinaryFromReader);\n      msg.setDockerImageFetchAction(value);\n      break;\n    case 54:\n      var value = new proto.ubo.v1.DockerImageReleaseAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageReleaseAction.deserializeBinaryFromReader);\n      msg.setDockerImageReleaseAction(value);\n      break;\n    case 55:\n      var value = new proto.ubo.v1.DockerImageRemoveAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveAction.deserializeBinaryFromReader);\n      msg.setDockerImageRemoveAction(value);\n      break;\n    case 56:\n      var value = new proto.ubo.v1.DockerImageRemoveContainerAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinaryFromReader);\n      msg.setDockerImageRemoveContainerAction(value);\n      break;\n    case 57:\n      var value = new proto.ubo.v1.DockerImageRunAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRunAction.deserializeBinaryFromReader);\n      msg.setDockerImageRunAction(value);\n      break;\n    case 58:\n      var value = new proto.ubo.v1.DockerImageSetDockerIdAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinaryFromReader);\n      msg.setDockerImageSetDockerIdAction(value);\n      break;\n    case 59:\n      var value = new proto.ubo.v1.DockerImageSetStatusAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageSetStatusAction.deserializeBinaryFromReader);\n      msg.setDockerImageSetStatusAction(value);\n      break;\n    case 60:\n      var value = new proto.ubo.v1.DockerImageStopAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageStopAction.deserializeBinaryFromReader);\n      msg.setDockerImageStopAction(value);\n      break;\n    case 61:\n      var value = new proto.ubo.v1.DockerImageUpdateMetadataAction;\n      reader.readMessage(value,proto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinaryFromReader);\n      msg.setDockerImageUpdateMetadataAction(value);\n      break;\n    case 62:\n      var value = new proto.ubo.v1.DockerInstallAction;\n      reader.readMessage(value,proto.ubo.v1.DockerInstallAction.deserializeBinaryFromReader);\n      msg.setDockerInstallAction(value);\n      break;\n    case 63:\n      var value = new proto.ubo.v1.DockerRemoveUsernameAction;\n      reader.readMessage(value,proto.ubo.v1.DockerRemoveUsernameAction.deserializeBinaryFromReader);\n      msg.setDockerRemoveUsernameAction(value);\n      break;\n    case 64:\n      var value = new proto.ubo.v1.DockerSetStatusAction;\n      reader.readMessage(value,proto.ubo.v1.DockerSetStatusAction.deserializeBinaryFromReader);\n      msg.setDockerSetStatusAction(value);\n      break;\n    case 65:\n      var value = new proto.ubo.v1.DockerStartAction;\n      reader.readMessage(value,proto.ubo.v1.DockerStartAction.deserializeBinaryFromReader);\n      msg.setDockerStartAction(value);\n      break;\n    case 66:\n      var value = new proto.ubo.v1.DockerStopAction;\n      reader.readMessage(value,proto.ubo.v1.DockerStopAction.deserializeBinaryFromReader);\n      msg.setDockerStopAction(value);\n      break;\n    case 67:\n      var value = new proto.ubo.v1.DockerStoreUsernameAction;\n      reader.readMessage(value,proto.ubo.v1.DockerStoreUsernameAction.deserializeBinaryFromReader);\n      msg.setDockerStoreUsernameAction(value);\n      break;\n    case 68:\n      var value = new proto.ubo.v1.FileSystemAction;\n      reader.readMessage(value,proto.ubo.v1.FileSystemAction.deserializeBinaryFromReader);\n      msg.setFileSystemAction(value);\n      break;\n    case 69:\n      var value = new proto.ubo.v1.FileSystemCopyAction;\n      reader.readMessage(value,proto.ubo.v1.FileSystemCopyAction.deserializeBinaryFromReader);\n      msg.setFileSystemCopyAction(value);\n      break;\n    case 70:\n      var value = new proto.ubo.v1.FileSystemMoveAction;\n      reader.readMessage(value,proto.ubo.v1.FileSystemMoveAction.deserializeBinaryFromReader);\n      msg.setFileSystemMoveAction(value);\n      break;\n    case 71:\n      var value = new proto.ubo.v1.FileSystemRemoveAction;\n      reader.readMessage(value,proto.ubo.v1.FileSystemRemoveAction.deserializeBinaryFromReader);\n      msg.setFileSystemRemoveAction(value);\n      break;\n    case 72:\n      var value = new proto.ubo.v1.FileSystemReportSelectionAction;\n      reader.readMessage(value,proto.ubo.v1.FileSystemReportSelectionAction.deserializeBinaryFromReader);\n      msg.setFileSystemReportSelectionAction(value);\n      break;\n    case 73:\n      var value = new proto.ubo.v1.InfraredAction;\n      reader.readMessage(value,proto.ubo.v1.InfraredAction.deserializeBinaryFromReader);\n      msg.setInfraredAction(value);\n      break;\n    case 74:\n      var value = new proto.ubo.v1.InfraredHandleReceivedCodeAction;\n      reader.readMessage(value,proto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinaryFromReader);\n      msg.setInfraredHandleReceivedCodeAction(value);\n      break;\n    case 75:\n      var value = new proto.ubo.v1.InfraredSendCodeAction;\n      reader.readMessage(value,proto.ubo.v1.InfraredSendCodeAction.deserializeBinaryFromReader);\n      msg.setInfraredSendCodeAction(value);\n      break;\n    case 76:\n      var value = new proto.ubo.v1.InfraredSetShouldPropagateAction;\n      reader.readMessage(value,proto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinaryFromReader);\n      msg.setInfraredSetShouldPropagateAction(value);\n      break;\n    case 77:\n      var value = new proto.ubo.v1.InfraredSetShouldReceiveAction;\n      reader.readMessage(value,proto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinaryFromReader);\n      msg.setInfraredSetShouldReceiveAction(value);\n      break;\n    case 78:\n      var value = new proto.ubo.v1.InputAction;\n      reader.readMessage(value,proto.ubo.v1.InputAction.deserializeBinaryFromReader);\n      msg.setInputAction(value);\n      break;\n    case 79:\n      var value = new proto.ubo.v1.InputCancelAction;\n      reader.readMessage(value,proto.ubo.v1.InputCancelAction.deserializeBinaryFromReader);\n      msg.setInputCancelAction(value);\n      break;\n    case 80:\n      var value = new proto.ubo.v1.InputDemandAction;\n      reader.readMessage(value,proto.ubo.v1.InputDemandAction.deserializeBinaryFromReader);\n      msg.setInputDemandAction(value);\n      break;\n    case 81:\n      var value = new proto.ubo.v1.InputProvideAction;\n      reader.readMessage(value,proto.ubo.v1.InputProvideAction.deserializeBinaryFromReader);\n      msg.setInputProvideAction(value);\n      break;\n    case 82:\n      var value = new proto.ubo.v1.InputResolveAction;\n      reader.readMessage(value,proto.ubo.v1.InputResolveAction.deserializeBinaryFromReader);\n      msg.setInputResolveAction(value);\n      break;\n    case 83:\n      var value = new proto.ubo.v1.IpAction;\n      reader.readMessage(value,proto.ubo.v1.IpAction.deserializeBinaryFromReader);\n      msg.setIpAction(value);\n      break;\n    case 84:\n      var value = new proto.ubo.v1.IpSetIsConnectedAction;\n      reader.readMessage(value,proto.ubo.v1.IpSetIsConnectedAction.deserializeBinaryFromReader);\n      msg.setIpSetIsConnectedAction(value);\n      break;\n    case 85:\n      var value = new proto.ubo.v1.IpUpdateInterfacesAction;\n      reader.readMessage(value,proto.ubo.v1.IpUpdateInterfacesAction.deserializeBinaryFromReader);\n      msg.setIpUpdateInterfacesAction(value);\n      break;\n    case 86:\n      var value = new proto.ubo.v1.KeypadAction;\n      reader.readMessage(value,proto.ubo.v1.KeypadAction.deserializeBinaryFromReader);\n      msg.setKeypadAction(value);\n      break;\n    case 87:\n      var value = new proto.ubo.v1.KeypadKeyHoldAction;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyHoldAction.deserializeBinaryFromReader);\n      msg.setKeypadKeyHoldAction(value);\n      break;\n    case 88:\n      var value = new proto.ubo.v1.KeypadKeyPressAction;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyPressAction.deserializeBinaryFromReader);\n      msg.setKeypadKeyPressAction(value);\n      break;\n    case 89:\n      var value = new proto.ubo.v1.KeypadKeyReleaseAction;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyReleaseAction.deserializeBinaryFromReader);\n      msg.setKeypadKeyReleaseAction(value);\n      break;\n    case 90:\n      var value = new proto.ubo.v1.KeypadKeyUnholdAction;\n      reader.readMessage(value,proto.ubo.v1.KeypadKeyUnholdAction.deserializeBinaryFromReader);\n      msg.setKeypadKeyUnholdAction(value);\n      break;\n    case 91:\n      var value = new proto.ubo.v1.LightDMAction;\n      reader.readMessage(value,proto.ubo.v1.LightDMAction.deserializeBinaryFromReader);\n      msg.setLightDmAction(value);\n      break;\n    case 92:\n      var value = new proto.ubo.v1.LightDMClearEnabledStateAction;\n      reader.readMessage(value,proto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinaryFromReader);\n      msg.setLightDmClearEnabledStateAction(value);\n      break;\n    case 93:\n      var value = new proto.ubo.v1.LightDMUpdateStateAction;\n      reader.readMessage(value,proto.ubo.v1.LightDMUpdateStateAction.deserializeBinaryFromReader);\n      msg.setLightDmUpdateStateAction(value);\n      break;\n    case 94:\n      var value = new proto.ubo.v1.MainAction;\n      reader.readMessage(value,proto.ubo.v1.MainAction.deserializeBinaryFromReader);\n      msg.setMainAction(value);\n      break;\n    case 95:\n      var value = new proto.ubo.v1.MenuAction;\n      reader.readMessage(value,proto.ubo.v1.MenuAction.deserializeBinaryFromReader);\n      msg.setMenuAction(value);\n      break;\n    case 96:\n      var value = new proto.ubo.v1.MenuChooseByIconAction;\n      reader.readMessage(value,proto.ubo.v1.MenuChooseByIconAction.deserializeBinaryFromReader);\n      msg.setMenuChooseByIconAction(value);\n      break;\n    case 97:\n      var value = new proto.ubo.v1.MenuChooseByIndexAction;\n      reader.readMessage(value,proto.ubo.v1.MenuChooseByIndexAction.deserializeBinaryFromReader);\n      msg.setMenuChooseByIndexAction(value);\n      break;\n    case 98:\n      var value = new proto.ubo.v1.MenuChooseByLabelAction;\n      reader.readMessage(value,proto.ubo.v1.MenuChooseByLabelAction.deserializeBinaryFromReader);\n      msg.setMenuChooseByLabelAction(value);\n      break;\n    case 99:\n      var value = new proto.ubo.v1.MenuGoBackAction;\n      reader.readMessage(value,proto.ubo.v1.MenuGoBackAction.deserializeBinaryFromReader);\n      msg.setMenuGoBackAction(value);\n      break;\n    case 100:\n      var value = new proto.ubo.v1.MenuGoHomeAction;\n      reader.readMessage(value,proto.ubo.v1.MenuGoHomeAction.deserializeBinaryFromReader);\n      msg.setMenuGoHomeAction(value);\n      break;\n    case 101:\n      var value = new proto.ubo.v1.MenuScrollAction;\n      reader.readMessage(value,proto.ubo.v1.MenuScrollAction.deserializeBinaryFromReader);\n      msg.setMenuScrollAction(value);\n      break;\n    case 102:\n      var value = new proto.ubo.v1.NotificationsAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationsAction.deserializeBinaryFromReader);\n      msg.setNotificationsAction(value);\n      break;\n    case 103:\n      var value = new proto.ubo.v1.NotificationsAddAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationsAddAction.deserializeBinaryFromReader);\n      msg.setNotificationsAddAction(value);\n      break;\n    case 104:\n      var value = new proto.ubo.v1.NotificationsClearAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationsClearAction.deserializeBinaryFromReader);\n      msg.setNotificationsClearAction(value);\n      break;\n    case 105:\n      var value = new proto.ubo.v1.NotificationsClearAllAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationsClearAllAction.deserializeBinaryFromReader);\n      msg.setNotificationsClearAllAction(value);\n      break;\n    case 106:\n      var value = new proto.ubo.v1.NotificationsClearByIdAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationsClearByIdAction.deserializeBinaryFromReader);\n      msg.setNotificationsClearByIdAction(value);\n      break;\n    case 107:\n      var value = new proto.ubo.v1.NotificationsDisplayAction;\n      reader.readMessage(value,proto.ubo.v1.NotificationsDisplayAction.deserializeBinaryFromReader);\n      msg.setNotificationsDisplayAction(value);\n      break;\n    case 108:\n      var value = new proto.ubo.v1.OpenApplicationAction;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.deserializeBinaryFromReader);\n      msg.setOpenApplicationAction(value);\n      break;\n    case 109:\n      var value = new proto.ubo.v1.PowerAction;\n      reader.readMessage(value,proto.ubo.v1.PowerAction.deserializeBinaryFromReader);\n      msg.setPowerAction(value);\n      break;\n    case 110:\n      var value = new proto.ubo.v1.PowerOffAction;\n      reader.readMessage(value,proto.ubo.v1.PowerOffAction.deserializeBinaryFromReader);\n      msg.setPowerOffAction(value);\n      break;\n    case 111:\n      var value = new proto.ubo.v1.RPiConnectAction;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectAction.deserializeBinaryFromReader);\n      msg.setRPiConnectAction(value);\n      break;\n    case 112:\n      var value = new proto.ubo.v1.RPiConnectDoneDownloadingAction;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinaryFromReader);\n      msg.setRPiConnectDoneDownloadingAction(value);\n      break;\n    case 113:\n      var value = new proto.ubo.v1.RPiConnectSetPendingAction;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectSetPendingAction.deserializeBinaryFromReader);\n      msg.setRPiConnectSetPendingAction(value);\n      break;\n    case 114:\n      var value = new proto.ubo.v1.RPiConnectSetStatusAction;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectSetStatusAction.deserializeBinaryFromReader);\n      msg.setRPiConnectSetStatusAction(value);\n      break;\n    case 115:\n      var value = new proto.ubo.v1.RPiConnectStartDownloadingAction;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinaryFromReader);\n      msg.setRPiConnectStartDownloadingAction(value);\n      break;\n    case 116:\n      var value = new proto.ubo.v1.RPiConnectUpdateServiceStateAction;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinaryFromReader);\n      msg.setRPiConnectUpdateServiceStateAction(value);\n      break;\n    case 117:\n      var value = new proto.ubo.v1.RebootAction;\n      reader.readMessage(value,proto.ubo.v1.RebootAction.deserializeBinaryFromReader);\n      msg.setRebootAction(value);\n      break;\n    case 118:\n      var value = new proto.ubo.v1.RegisterAppAction;\n      reader.readMessage(value,proto.ubo.v1.RegisterAppAction.deserializeBinaryFromReader);\n      msg.setRegisterAppAction(value);\n      break;\n    case 119:\n      var value = new proto.ubo.v1.RegisterRegularAppAction;\n      reader.readMessage(value,proto.ubo.v1.RegisterRegularAppAction.deserializeBinaryFromReader);\n      msg.setRegisterRegularAppAction(value);\n      break;\n    case 120:\n      var value = new proto.ubo.v1.RegisterSettingAppAction;\n      reader.readMessage(value,proto.ubo.v1.RegisterSettingAppAction.deserializeBinaryFromReader);\n      msg.setRegisterSettingAppAction(value);\n      break;\n    case 121:\n      var value = new proto.ubo.v1.ReplayRecordedSequenceAction;\n      reader.readMessage(value,proto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinaryFromReader);\n      msg.setReplayRecordedSequenceAction(value);\n      break;\n    case 122:\n      var value = new proto.ubo.v1.ReportReplayingDoneAction;\n      reader.readMessage(value,proto.ubo.v1.ReportReplayingDoneAction.deserializeBinaryFromReader);\n      msg.setReportReplayingDoneAction(value);\n      break;\n    case 123:\n      var value = new proto.ubo.v1.RgbRingAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingAction.deserializeBinaryFromReader);\n      msg.setRgbRingAction(value);\n      break;\n    case 124:\n      var value = new proto.ubo.v1.RgbRingBlankAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingBlankAction.deserializeBinaryFromReader);\n      msg.setRgbRingBlankAction(value);\n      break;\n    case 125:\n      var value = new proto.ubo.v1.RgbRingBlinkAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingBlinkAction.deserializeBinaryFromReader);\n      msg.setRgbRingBlinkAction(value);\n      break;\n    case 126:\n      var value = new proto.ubo.v1.RgbRingColorfulCommandAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinaryFromReader);\n      msg.setRgbRingColorfulCommandAction(value);\n      break;\n    case 127:\n      var value = new proto.ubo.v1.RgbRingCommandAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader);\n      msg.setRgbRingCommandAction(value);\n      break;\n    case 128:\n      var value = new proto.ubo.v1.RgbRingFillDownfromAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingFillDownfromAction.deserializeBinaryFromReader);\n      msg.setRgbRingFillDownfromAction(value);\n      break;\n    case 129:\n      var value = new proto.ubo.v1.RgbRingFillUptoAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingFillUptoAction.deserializeBinaryFromReader);\n      msg.setRgbRingFillUptoAction(value);\n      break;\n    case 130:\n      var value = new proto.ubo.v1.RgbRingProgressWheelAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingProgressWheelAction.deserializeBinaryFromReader);\n      msg.setRgbRingProgressWheelAction(value);\n      break;\n    case 131:\n      var value = new proto.ubo.v1.RgbRingProgressWheelStepAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinaryFromReader);\n      msg.setRgbRingProgressWheelStepAction(value);\n      break;\n    case 132:\n      var value = new proto.ubo.v1.RgbRingPulseAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingPulseAction.deserializeBinaryFromReader);\n      msg.setRgbRingPulseAction(value);\n      break;\n    case 133:\n      var value = new proto.ubo.v1.RgbRingRainbowAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingRainbowAction.deserializeBinaryFromReader);\n      msg.setRgbRingRainbowAction(value);\n      break;\n    case 134:\n      var value = new proto.ubo.v1.RgbRingSequenceAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingSequenceAction.deserializeBinaryFromReader);\n      msg.setRgbRingSequenceAction(value);\n      break;\n    case 135:\n      var value = new proto.ubo.v1.RgbRingSetAllAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingSetAllAction.deserializeBinaryFromReader);\n      msg.setRgbRingSetAllAction(value);\n      break;\n    case 136:\n      var value = new proto.ubo.v1.RgbRingSetBrightnessAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinaryFromReader);\n      msg.setRgbRingSetBrightnessAction(value);\n      break;\n    case 137:\n      var value = new proto.ubo.v1.RgbRingSetEnabledAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingSetEnabledAction.deserializeBinaryFromReader);\n      msg.setRgbRingSetEnabledAction(value);\n      break;\n    case 138:\n      var value = new proto.ubo.v1.RgbRingSetIsBusyAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinaryFromReader);\n      msg.setRgbRingSetIsBusyAction(value);\n      break;\n    case 139:\n      var value = new proto.ubo.v1.RgbRingSpinningWheelAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinaryFromReader);\n      msg.setRgbRingSpinningWheelAction(value);\n      break;\n    case 140:\n      var value = new proto.ubo.v1.RgbRingWaitableCommandAction;\n      reader.readMessage(value,proto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinaryFromReader);\n      msg.setRgbRingWaitableCommandAction(value);\n      break;\n    case 141:\n      var value = new proto.ubo.v1.SSHAction;\n      reader.readMessage(value,proto.ubo.v1.SSHAction.deserializeBinaryFromReader);\n      msg.setSshAction(value);\n      break;\n    case 142:\n      var value = new proto.ubo.v1.SSHClearEnabledStateAction;\n      reader.readMessage(value,proto.ubo.v1.SSHClearEnabledStateAction.deserializeBinaryFromReader);\n      msg.setSshClearEnabledStateAction(value);\n      break;\n    case 143:\n      var value = new proto.ubo.v1.SSHUpdateStateAction;\n      reader.readMessage(value,proto.ubo.v1.SSHUpdateStateAction.deserializeBinaryFromReader);\n      msg.setSshUpdateStateAction(value);\n      break;\n    case 144:\n      var value = new proto.ubo.v1.SensorsAction;\n      reader.readMessage(value,proto.ubo.v1.SensorsAction.deserializeBinaryFromReader);\n      msg.setSensorsAction(value);\n      break;\n    case 145:\n      var value = new proto.ubo.v1.SensorsReportReadingAction;\n      reader.readMessage(value,proto.ubo.v1.SensorsReportReadingAction.deserializeBinaryFromReader);\n      msg.setSensorsReportReadingAction(value);\n      break;\n    case 146:\n      var value = new proto.ubo.v1.SetAreEnclosuresVisibleAction;\n      reader.readMessage(value,proto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinaryFromReader);\n      msg.setSetAreEnclosuresVisibleAction(value);\n      break;\n    case 147:\n      var value = new proto.ubo.v1.SetMenuPathAction;\n      reader.readMessage(value,proto.ubo.v1.SetMenuPathAction.deserializeBinaryFromReader);\n      msg.setSetMenuPathAction(value);\n      break;\n    case 148:\n      var value = new proto.ubo.v1.SettingsAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsAction.deserializeBinaryFromReader);\n      msg.setSettingsAction(value);\n      break;\n    case 149:\n      var value = new proto.ubo.v1.SettingsClearServiceErrorsAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinaryFromReader);\n      msg.setSettingsClearServiceErrorsAction(value);\n      break;\n    case 150:\n      var value = new proto.ubo.v1.SettingsReportServiceErrorAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinaryFromReader);\n      msg.setSettingsReportServiceErrorAction(value);\n      break;\n    case 151:\n      var value = new proto.ubo.v1.SettingsServiceAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsServiceAction.deserializeBinaryFromReader);\n      msg.setSettingsServiceAction(value);\n      break;\n    case 152:\n      var value = new proto.ubo.v1.SettingsServiceSetIsEnabledAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinaryFromReader);\n      msg.setSettingsServiceSetIsEnabledAction(value);\n      break;\n    case 153:\n      var value = new proto.ubo.v1.SettingsServiceSetLogLevelAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinaryFromReader);\n      msg.setSettingsServiceSetLogLevelAction(value);\n      break;\n    case 154:\n      var value = new proto.ubo.v1.SettingsServiceSetShouldRestartAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinaryFromReader);\n      msg.setSettingsServiceSetShouldRestartAction(value);\n      break;\n    case 155:\n      var value = new proto.ubo.v1.SettingsServiceSetStatusAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinaryFromReader);\n      msg.setSettingsServiceSetStatusAction(value);\n      break;\n    case 156:\n      var value = new proto.ubo.v1.SettingsSetServicesAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsSetServicesAction.deserializeBinaryFromReader);\n      msg.setSettingsSetServicesAction(value);\n      break;\n    case 157:\n      var value = new proto.ubo.v1.SettingsStartServiceAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsStartServiceAction.deserializeBinaryFromReader);\n      msg.setSettingsStartServiceAction(value);\n      break;\n    case 158:\n      var value = new proto.ubo.v1.SettingsStopServiceAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsStopServiceAction.deserializeBinaryFromReader);\n      msg.setSettingsStopServiceAction(value);\n      break;\n    case 159:\n      var value = new proto.ubo.v1.SettingsToggleBetaVersionsAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinaryFromReader);\n      msg.setSettingsToggleBetaVersionsAction(value);\n      break;\n    case 160:\n      var value = new proto.ubo.v1.SettingsTogglePdbSignalAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinaryFromReader);\n      msg.setSettingsTogglePdbSignalAction(value);\n      break;\n    case 161:\n      var value = new proto.ubo.v1.SettingsToggleVisualDebugAction;\n      reader.readMessage(value,proto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinaryFromReader);\n      msg.setSettingsToggleVisualDebugAction(value);\n      break;\n    case 162:\n      var value = new proto.ubo.v1.SpeechRecognitionAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionAction(value);\n      break;\n    case 163:\n      var value = new proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionReportIntentDetectionAction(value);\n      break;\n    case 164:\n      var value = new proto.ubo.v1.SpeechRecognitionReportSpeechAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionReportSpeechAction(value);\n      break;\n    case 165:\n      var value = new proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionReportWakeWordDetectionAction(value);\n      break;\n    case 166:\n      var value = new proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionSetIsAssistantActiveAction(value);\n      break;\n    case 167:\n      var value = new proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionSetIsIntentsActiveAction(value);\n      break;\n    case 168:\n      var value = new proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionSetSelectedEngineAction(value);\n      break;\n    case 169:\n      var value = new proto.ubo.v1.SpeechSynthesisAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisAction.deserializeBinaryFromReader);\n      msg.setSpeechSynthesisAction(value);\n      break;\n    case 170:\n      var value = new proto.ubo.v1.SpeechSynthesisReadTextAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinaryFromReader);\n      msg.setSpeechSynthesisReadTextAction(value);\n      break;\n    case 171:\n      var value = new proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction;\n      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinaryFromReader);\n      msg.setSpeechSynthesisSetSelectedEngineAction(value);\n      break;\n    case 172:\n      var value = new proto.ubo.v1.StatusIconsAction;\n      reader.readMessage(value,proto.ubo.v1.StatusIconsAction.deserializeBinaryFromReader);\n      msg.setStatusIconsAction(value);\n      break;\n    case 173:\n      var value = new proto.ubo.v1.StatusIconsRegisterAction;\n      reader.readMessage(value,proto.ubo.v1.StatusIconsRegisterAction.deserializeBinaryFromReader);\n      msg.setStatusIconsRegisterAction(value);\n      break;\n    case 174:\n      var value = new proto.ubo.v1.ToggleRecordingAction;\n      reader.readMessage(value,proto.ubo.v1.ToggleRecordingAction.deserializeBinaryFromReader);\n      msg.setToggleRecordingAction(value);\n      break;\n    case 175:\n      var value = new proto.ubo.v1.UpdateManagerAction;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerAction.deserializeBinaryFromReader);\n      msg.setUpdateManagerAction(value);\n      break;\n    case 176:\n      var value = new proto.ubo.v1.UpdateManagerReportFailedCheckAction;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinaryFromReader);\n      msg.setUpdateManagerReportFailedCheckAction(value);\n      break;\n    case 177:\n      var value = new proto.ubo.v1.UpdateManagerRequestCheckAction;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinaryFromReader);\n      msg.setUpdateManagerRequestCheckAction(value);\n      break;\n    case 178:\n      var value = new proto.ubo.v1.UpdateManagerRequestUpdateAction;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinaryFromReader);\n      msg.setUpdateManagerRequestUpdateAction(value);\n      break;\n    case 179:\n      var value = new proto.ubo.v1.UpdateManagerSetVersionsAction;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinaryFromReader);\n      msg.setUpdateManagerSetVersionsAction(value);\n      break;\n    case 180:\n      var value = new proto.ubo.v1.UsersAction;\n      reader.readMessage(value,proto.ubo.v1.UsersAction.deserializeBinaryFromReader);\n      msg.setUsersAction(value);\n      break;\n    case 181:\n      var value = new proto.ubo.v1.UsersCreateUserAction;\n      reader.readMessage(value,proto.ubo.v1.UsersCreateUserAction.deserializeBinaryFromReader);\n      msg.setUsersCreateUserAction(value);\n      break;\n    case 182:\n      var value = new proto.ubo.v1.UsersDeleteUserAction;\n      reader.readMessage(value,proto.ubo.v1.UsersDeleteUserAction.deserializeBinaryFromReader);\n      msg.setUsersDeleteUserAction(value);\n      break;\n    case 183:\n      var value = new proto.ubo.v1.UsersResetPasswordAction;\n      reader.readMessage(value,proto.ubo.v1.UsersResetPasswordAction.deserializeBinaryFromReader);\n      msg.setUsersResetPasswordAction(value);\n      break;\n    case 184:\n      var value = new proto.ubo.v1.UsersSetUsersAction;\n      reader.readMessage(value,proto.ubo.v1.UsersSetUsersAction.deserializeBinaryFromReader);\n      msg.setUsersSetUsersAction(value);\n      break;\n    case 185:\n      var value = new proto.ubo.v1.VSCodeAction;\n      reader.readMessage(value,proto.ubo.v1.VSCodeAction.deserializeBinaryFromReader);\n      msg.setVsCodeAction(value);\n      break;\n    case 186:\n      var value = new proto.ubo.v1.VSCodeDoneDownloadingAction;\n      reader.readMessage(value,proto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinaryFromReader);\n      msg.setVsCodeDoneDownloadingAction(value);\n      break;\n    case 187:\n      var value = new proto.ubo.v1.VSCodeSetPendingAction;\n      reader.readMessage(value,proto.ubo.v1.VSCodeSetPendingAction.deserializeBinaryFromReader);\n      msg.setVsCodeSetPendingAction(value);\n      break;\n    case 188:\n      var value = new proto.ubo.v1.VSCodeSetStatusAction;\n      reader.readMessage(value,proto.ubo.v1.VSCodeSetStatusAction.deserializeBinaryFromReader);\n      msg.setVsCodeSetStatusAction(value);\n      break;\n    case 189:\n      var value = new proto.ubo.v1.VSCodeStartDownloadingAction;\n      reader.readMessage(value,proto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinaryFromReader);\n      msg.setVsCodeStartDownloadingAction(value);\n      break;\n    case 190:\n      var value = new proto.ubo.v1.WiFiAction;\n      reader.readMessage(value,proto.ubo.v1.WiFiAction.deserializeBinaryFromReader);\n      msg.setWiFiAction(value);\n      break;\n    case 191:\n      var value = new proto.ubo.v1.WiFiInputConnectionAction;\n      reader.readMessage(value,proto.ubo.v1.WiFiInputConnectionAction.deserializeBinaryFromReader);\n      msg.setWiFiInputConnectionAction(value);\n      break;\n    case 192:\n      var value = new proto.ubo.v1.WiFiSetHasVisitedOnboardingAction;\n      reader.readMessage(value,proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinaryFromReader);\n      msg.setWiFiSetHasVisitedOnboardingAction(value);\n      break;\n    case 193:\n      var value = new proto.ubo.v1.WiFiUpdateAction;\n      reader.readMessage(value,proto.ubo.v1.WiFiUpdateAction.deserializeBinaryFromReader);\n      msg.setWiFiUpdateAction(value);\n      break;\n    case 194:\n      var value = new proto.ubo.v1.WiFiUpdateRequestAction;\n      reader.readMessage(value,proto.ubo.v1.WiFiUpdateRequestAction.deserializeBinaryFromReader);\n      msg.setWiFiUpdateRequestAction(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Action.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Action.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Action} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Action.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getAssistantAction();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.AssistantAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantAddMcpServerAction();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AssistantAddMcpServerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantDeleteMcpServerAction();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AssistantDeleteMcpServerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantDownloadOllamaModelAction();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.AssistantDownloadOllamaModelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantReportAction();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.AssistantReportAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSetIsActiveAction();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.AssistantSetIsActiveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSetSelectedImageGeneratorAction();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSetSelectedLlmAction();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.ubo.v1.AssistantSetSelectedLLMAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSetSelectedModelAction();\n  if (f != null) {\n    writer.writeMessage(\n      9,\n      f,\n      proto.ubo.v1.AssistantSetSelectedModelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSetSelectedSttAction();\n  if (f != null) {\n    writer.writeMessage(\n      10,\n      f,\n      proto.ubo.v1.AssistantSetSelectedSTTAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSetSelectedTtsAction();\n  if (f != null) {\n    writer.writeMessage(\n      11,\n      f,\n      proto.ubo.v1.AssistantSetSelectedTTSAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantStartListeningAction();\n  if (f != null) {\n    writer.writeMessage(\n      12,\n      f,\n      proto.ubo.v1.AssistantStartListeningAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantStopListeningAction();\n  if (f != null) {\n    writer.writeMessage(\n      13,\n      f,\n      proto.ubo.v1.AssistantStopListeningAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantSyncMcpServersAction();\n  if (f != null) {\n    writer.writeMessage(\n      14,\n      f,\n      proto.ubo.v1.AssistantSyncMcpServersAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantToggleListeningAction();\n  if (f != null) {\n    writer.writeMessage(\n      15,\n      f,\n      proto.ubo.v1.AssistantToggleListeningAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantToggleMcpServerAction();\n  if (f != null) {\n    writer.writeMessage(\n      16,\n      f,\n      proto.ubo.v1.AssistantToggleMcpServerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantUpdateProvidersAction();\n  if (f != null) {\n    writer.writeMessage(\n      17,\n      f,\n      proto.ubo.v1.AssistantUpdateProvidersAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioAction();\n  if (f != null) {\n    writer.writeMessage(\n      18,\n      f,\n      proto.ubo.v1.AudioAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioChangeVolumeAction();\n  if (f != null) {\n    writer.writeMessage(\n      19,\n      f,\n      proto.ubo.v1.AudioChangeVolumeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioInstallDriverAction();\n  if (f != null) {\n    writer.writeMessage(\n      20,\n      f,\n      proto.ubo.v1.AudioInstallDriverAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayAudioSampleAction();\n  if (f != null) {\n    writer.writeMessage(\n      21,\n      f,\n      proto.ubo.v1.AudioPlayAudioSampleAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayAudioSequenceAction();\n  if (f != null) {\n    writer.writeMessage(\n      22,\n      f,\n      proto.ubo.v1.AudioPlayAudioSequenceAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayChimeAction();\n  if (f != null) {\n    writer.writeMessage(\n      23,\n      f,\n      proto.ubo.v1.AudioPlayChimeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayRecordingAction();\n  if (f != null) {\n    writer.writeMessage(\n      24,\n      f,\n      proto.ubo.v1.AudioPlayRecordingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlaybackDoneAction();\n  if (f != null) {\n    writer.writeMessage(\n      25,\n      f,\n      proto.ubo.v1.AudioPlaybackDoneAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioReportSampleAction();\n  if (f != null) {\n    writer.writeMessage(\n      26,\n      f,\n      proto.ubo.v1.AudioReportSampleAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioSetMuteStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      27,\n      f,\n      proto.ubo.v1.AudioSetMuteStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioSetVolumeAction();\n  if (f != null) {\n    writer.writeMessage(\n      28,\n      f,\n      proto.ubo.v1.AudioSetVolumeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioStartRecordingAction();\n  if (f != null) {\n    writer.writeMessage(\n      29,\n      f,\n      proto.ubo.v1.AudioStartRecordingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioStopRecordingAction();\n  if (f != null) {\n    writer.writeMessage(\n      30,\n      f,\n      proto.ubo.v1.AudioStopRecordingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioToggleMuteStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      31,\n      f,\n      proto.ubo.v1.AudioToggleMuteStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioToggleRecordingAction();\n  if (f != null) {\n    writer.writeMessage(\n      32,\n      f,\n      proto.ubo.v1.AudioToggleRecordingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraAction();\n  if (f != null) {\n    writer.writeMessage(\n      33,\n      f,\n      proto.ubo.v1.CameraAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraDetectAction();\n  if (f != null) {\n    writer.writeMessage(\n      34,\n      f,\n      proto.ubo.v1.CameraDetectAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraInstallDriverAction();\n  if (f != null) {\n    writer.writeMessage(\n      35,\n      f,\n      proto.ubo.v1.CameraInstallDriverAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraReportBarcodeAction();\n  if (f != null) {\n    writer.writeMessage(\n      36,\n      f,\n      proto.ubo.v1.CameraReportBarcodeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraRestoreDefaultAction();\n  if (f != null) {\n    writer.writeMessage(\n      37,\n      f,\n      proto.ubo.v1.CameraRestoreDefaultAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraSetAvailableCamerasAction();\n  if (f != null) {\n    writer.writeMessage(\n      38,\n      f,\n      proto.ubo.v1.CameraSetAvailableCamerasAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraSetIndexAction();\n  if (f != null) {\n    writer.writeMessage(\n      39,\n      f,\n      proto.ubo.v1.CameraSetIndexAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraStartViewfinderAction();\n  if (f != null) {\n    writer.writeMessage(\n      40,\n      f,\n      proto.ubo.v1.CameraStartViewfinderAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getCloseApplicationAction();\n  if (f != null) {\n    writer.writeMessage(\n      41,\n      f,\n      proto.ubo.v1.CloseApplicationAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDeregisterRegularAppAction();\n  if (f != null) {\n    writer.writeMessage(\n      42,\n      f,\n      proto.ubo.v1.DeregisterRegularAppAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayAction();\n  if (f != null) {\n    writer.writeMessage(\n      43,\n      f,\n      proto.ubo.v1.DisplayAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayBlankAction();\n  if (f != null) {\n    writer.writeMessage(\n      44,\n      f,\n      proto.ubo.v1.DisplayBlankAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayPauseAction();\n  if (f != null) {\n    writer.writeMessage(\n      45,\n      f,\n      proto.ubo.v1.DisplayPauseAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayRedrawAction();\n  if (f != null) {\n    writer.writeMessage(\n      46,\n      f,\n      proto.ubo.v1.DisplayRedrawAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayResumeAction();\n  if (f != null) {\n    writer.writeMessage(\n      47,\n      f,\n      proto.ubo.v1.DisplayResumeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplaySetBlankTimeoutAction();\n  if (f != null) {\n    writer.writeMessage(\n      48,\n      f,\n      proto.ubo.v1.DisplaySetBlankTimeoutAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayUnblankAction();\n  if (f != null) {\n    writer.writeMessage(\n      49,\n      f,\n      proto.ubo.v1.DisplayUnblankAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayUpdateActivityAction();\n  if (f != null) {\n    writer.writeMessage(\n      50,\n      f,\n      proto.ubo.v1.DisplayUpdateActivityAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerAction();\n  if (f != null) {\n    writer.writeMessage(\n      51,\n      f,\n      proto.ubo.v1.DockerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageAction();\n  if (f != null) {\n    writer.writeMessage(\n      52,\n      f,\n      proto.ubo.v1.DockerImageAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageFetchAction();\n  if (f != null) {\n    writer.writeMessage(\n      53,\n      f,\n      proto.ubo.v1.DockerImageFetchAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageReleaseAction();\n  if (f != null) {\n    writer.writeMessage(\n      54,\n      f,\n      proto.ubo.v1.DockerImageReleaseAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRemoveAction();\n  if (f != null) {\n    writer.writeMessage(\n      55,\n      f,\n      proto.ubo.v1.DockerImageRemoveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRemoveContainerAction();\n  if (f != null) {\n    writer.writeMessage(\n      56,\n      f,\n      proto.ubo.v1.DockerImageRemoveContainerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRunAction();\n  if (f != null) {\n    writer.writeMessage(\n      57,\n      f,\n      proto.ubo.v1.DockerImageRunAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageSetDockerIdAction();\n  if (f != null) {\n    writer.writeMessage(\n      58,\n      f,\n      proto.ubo.v1.DockerImageSetDockerIdAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageSetStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      59,\n      f,\n      proto.ubo.v1.DockerImageSetStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageStopAction();\n  if (f != null) {\n    writer.writeMessage(\n      60,\n      f,\n      proto.ubo.v1.DockerImageStopAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageUpdateMetadataAction();\n  if (f != null) {\n    writer.writeMessage(\n      61,\n      f,\n      proto.ubo.v1.DockerImageUpdateMetadataAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerInstallAction();\n  if (f != null) {\n    writer.writeMessage(\n      62,\n      f,\n      proto.ubo.v1.DockerInstallAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerRemoveUsernameAction();\n  if (f != null) {\n    writer.writeMessage(\n      63,\n      f,\n      proto.ubo.v1.DockerRemoveUsernameAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerSetStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      64,\n      f,\n      proto.ubo.v1.DockerSetStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerStartAction();\n  if (f != null) {\n    writer.writeMessage(\n      65,\n      f,\n      proto.ubo.v1.DockerStartAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerStopAction();\n  if (f != null) {\n    writer.writeMessage(\n      66,\n      f,\n      proto.ubo.v1.DockerStopAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerStoreUsernameAction();\n  if (f != null) {\n    writer.writeMessage(\n      67,\n      f,\n      proto.ubo.v1.DockerStoreUsernameAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemAction();\n  if (f != null) {\n    writer.writeMessage(\n      68,\n      f,\n      proto.ubo.v1.FileSystemAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemCopyAction();\n  if (f != null) {\n    writer.writeMessage(\n      69,\n      f,\n      proto.ubo.v1.FileSystemCopyAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemMoveAction();\n  if (f != null) {\n    writer.writeMessage(\n      70,\n      f,\n      proto.ubo.v1.FileSystemMoveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemRemoveAction();\n  if (f != null) {\n    writer.writeMessage(\n      71,\n      f,\n      proto.ubo.v1.FileSystemRemoveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemReportSelectionAction();\n  if (f != null) {\n    writer.writeMessage(\n      72,\n      f,\n      proto.ubo.v1.FileSystemReportSelectionAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredAction();\n  if (f != null) {\n    writer.writeMessage(\n      73,\n      f,\n      proto.ubo.v1.InfraredAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredHandleReceivedCodeAction();\n  if (f != null) {\n    writer.writeMessage(\n      74,\n      f,\n      proto.ubo.v1.InfraredHandleReceivedCodeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredSendCodeAction();\n  if (f != null) {\n    writer.writeMessage(\n      75,\n      f,\n      proto.ubo.v1.InfraredSendCodeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredSetShouldPropagateAction();\n  if (f != null) {\n    writer.writeMessage(\n      76,\n      f,\n      proto.ubo.v1.InfraredSetShouldPropagateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredSetShouldReceiveAction();\n  if (f != null) {\n    writer.writeMessage(\n      77,\n      f,\n      proto.ubo.v1.InfraredSetShouldReceiveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputAction();\n  if (f != null) {\n    writer.writeMessage(\n      78,\n      f,\n      proto.ubo.v1.InputAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputCancelAction();\n  if (f != null) {\n    writer.writeMessage(\n      79,\n      f,\n      proto.ubo.v1.InputCancelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputDemandAction();\n  if (f != null) {\n    writer.writeMessage(\n      80,\n      f,\n      proto.ubo.v1.InputDemandAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputProvideAction();\n  if (f != null) {\n    writer.writeMessage(\n      81,\n      f,\n      proto.ubo.v1.InputProvideAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputResolveAction();\n  if (f != null) {\n    writer.writeMessage(\n      82,\n      f,\n      proto.ubo.v1.InputResolveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getIpAction();\n  if (f != null) {\n    writer.writeMessage(\n      83,\n      f,\n      proto.ubo.v1.IpAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getIpSetIsConnectedAction();\n  if (f != null) {\n    writer.writeMessage(\n      84,\n      f,\n      proto.ubo.v1.IpSetIsConnectedAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getIpUpdateInterfacesAction();\n  if (f != null) {\n    writer.writeMessage(\n      85,\n      f,\n      proto.ubo.v1.IpUpdateInterfacesAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getKeypadAction();\n  if (f != null) {\n    writer.writeMessage(\n      86,\n      f,\n      proto.ubo.v1.KeypadAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getKeypadKeyHoldAction();\n  if (f != null) {\n    writer.writeMessage(\n      87,\n      f,\n      proto.ubo.v1.KeypadKeyHoldAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getKeypadKeyPressAction();\n  if (f != null) {\n    writer.writeMessage(\n      88,\n      f,\n      proto.ubo.v1.KeypadKeyPressAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getKeypadKeyReleaseAction();\n  if (f != null) {\n    writer.writeMessage(\n      89,\n      f,\n      proto.ubo.v1.KeypadKeyReleaseAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getKeypadKeyUnholdAction();\n  if (f != null) {\n    writer.writeMessage(\n      90,\n      f,\n      proto.ubo.v1.KeypadKeyUnholdAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getLightDmAction();\n  if (f != null) {\n    writer.writeMessage(\n      91,\n      f,\n      proto.ubo.v1.LightDMAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getLightDmClearEnabledStateAction();\n  if (f != null) {\n    writer.writeMessage(\n      92,\n      f,\n      proto.ubo.v1.LightDMClearEnabledStateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getLightDmUpdateStateAction();\n  if (f != null) {\n    writer.writeMessage(\n      93,\n      f,\n      proto.ubo.v1.LightDMUpdateStateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMainAction();\n  if (f != null) {\n    writer.writeMessage(\n      94,\n      f,\n      proto.ubo.v1.MainAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuAction();\n  if (f != null) {\n    writer.writeMessage(\n      95,\n      f,\n      proto.ubo.v1.MenuAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuChooseByIconAction();\n  if (f != null) {\n    writer.writeMessage(\n      96,\n      f,\n      proto.ubo.v1.MenuChooseByIconAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuChooseByIndexAction();\n  if (f != null) {\n    writer.writeMessage(\n      97,\n      f,\n      proto.ubo.v1.MenuChooseByIndexAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuChooseByLabelAction();\n  if (f != null) {\n    writer.writeMessage(\n      98,\n      f,\n      proto.ubo.v1.MenuChooseByLabelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuGoBackAction();\n  if (f != null) {\n    writer.writeMessage(\n      99,\n      f,\n      proto.ubo.v1.MenuGoBackAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuGoHomeAction();\n  if (f != null) {\n    writer.writeMessage(\n      100,\n      f,\n      proto.ubo.v1.MenuGoHomeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuScrollAction();\n  if (f != null) {\n    writer.writeMessage(\n      101,\n      f,\n      proto.ubo.v1.MenuScrollAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsAction();\n  if (f != null) {\n    writer.writeMessage(\n      102,\n      f,\n      proto.ubo.v1.NotificationsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsAddAction();\n  if (f != null) {\n    writer.writeMessage(\n      103,\n      f,\n      proto.ubo.v1.NotificationsAddAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsClearAction();\n  if (f != null) {\n    writer.writeMessage(\n      104,\n      f,\n      proto.ubo.v1.NotificationsClearAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsClearAllAction();\n  if (f != null) {\n    writer.writeMessage(\n      105,\n      f,\n      proto.ubo.v1.NotificationsClearAllAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsClearByIdAction();\n  if (f != null) {\n    writer.writeMessage(\n      106,\n      f,\n      proto.ubo.v1.NotificationsClearByIdAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsDisplayAction();\n  if (f != null) {\n    writer.writeMessage(\n      107,\n      f,\n      proto.ubo.v1.NotificationsDisplayAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getOpenApplicationAction();\n  if (f != null) {\n    writer.writeMessage(\n      108,\n      f,\n      proto.ubo.v1.OpenApplicationAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getPowerAction();\n  if (f != null) {\n    writer.writeMessage(\n      109,\n      f,\n      proto.ubo.v1.PowerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getPowerOffAction();\n  if (f != null) {\n    writer.writeMessage(\n      110,\n      f,\n      proto.ubo.v1.PowerOffAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectAction();\n  if (f != null) {\n    writer.writeMessage(\n      111,\n      f,\n      proto.ubo.v1.RPiConnectAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectDoneDownloadingAction();\n  if (f != null) {\n    writer.writeMessage(\n      112,\n      f,\n      proto.ubo.v1.RPiConnectDoneDownloadingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectSetPendingAction();\n  if (f != null) {\n    writer.writeMessage(\n      113,\n      f,\n      proto.ubo.v1.RPiConnectSetPendingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectSetStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      114,\n      f,\n      proto.ubo.v1.RPiConnectSetStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectStartDownloadingAction();\n  if (f != null) {\n    writer.writeMessage(\n      115,\n      f,\n      proto.ubo.v1.RPiConnectStartDownloadingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectUpdateServiceStateAction();\n  if (f != null) {\n    writer.writeMessage(\n      116,\n      f,\n      proto.ubo.v1.RPiConnectUpdateServiceStateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRebootAction();\n  if (f != null) {\n    writer.writeMessage(\n      117,\n      f,\n      proto.ubo.v1.RebootAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRegisterAppAction();\n  if (f != null) {\n    writer.writeMessage(\n      118,\n      f,\n      proto.ubo.v1.RegisterAppAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRegisterRegularAppAction();\n  if (f != null) {\n    writer.writeMessage(\n      119,\n      f,\n      proto.ubo.v1.RegisterRegularAppAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRegisterSettingAppAction();\n  if (f != null) {\n    writer.writeMessage(\n      120,\n      f,\n      proto.ubo.v1.RegisterSettingAppAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getReplayRecordedSequenceAction();\n  if (f != null) {\n    writer.writeMessage(\n      121,\n      f,\n      proto.ubo.v1.ReplayRecordedSequenceAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getReportReplayingDoneAction();\n  if (f != null) {\n    writer.writeMessage(\n      122,\n      f,\n      proto.ubo.v1.ReportReplayingDoneAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingAction();\n  if (f != null) {\n    writer.writeMessage(\n      123,\n      f,\n      proto.ubo.v1.RgbRingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingBlankAction();\n  if (f != null) {\n    writer.writeMessage(\n      124,\n      f,\n      proto.ubo.v1.RgbRingBlankAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingBlinkAction();\n  if (f != null) {\n    writer.writeMessage(\n      125,\n      f,\n      proto.ubo.v1.RgbRingBlinkAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingColorfulCommandAction();\n  if (f != null) {\n    writer.writeMessage(\n      126,\n      f,\n      proto.ubo.v1.RgbRingColorfulCommandAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingCommandAction();\n  if (f != null) {\n    writer.writeMessage(\n      127,\n      f,\n      proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingFillDownfromAction();\n  if (f != null) {\n    writer.writeMessage(\n      128,\n      f,\n      proto.ubo.v1.RgbRingFillDownfromAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingFillUptoAction();\n  if (f != null) {\n    writer.writeMessage(\n      129,\n      f,\n      proto.ubo.v1.RgbRingFillUptoAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingProgressWheelAction();\n  if (f != null) {\n    writer.writeMessage(\n      130,\n      f,\n      proto.ubo.v1.RgbRingProgressWheelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingProgressWheelStepAction();\n  if (f != null) {\n    writer.writeMessage(\n      131,\n      f,\n      proto.ubo.v1.RgbRingProgressWheelStepAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingPulseAction();\n  if (f != null) {\n    writer.writeMessage(\n      132,\n      f,\n      proto.ubo.v1.RgbRingPulseAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingRainbowAction();\n  if (f != null) {\n    writer.writeMessage(\n      133,\n      f,\n      proto.ubo.v1.RgbRingRainbowAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingSequenceAction();\n  if (f != null) {\n    writer.writeMessage(\n      134,\n      f,\n      proto.ubo.v1.RgbRingSequenceAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingSetAllAction();\n  if (f != null) {\n    writer.writeMessage(\n      135,\n      f,\n      proto.ubo.v1.RgbRingSetAllAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingSetBrightnessAction();\n  if (f != null) {\n    writer.writeMessage(\n      136,\n      f,\n      proto.ubo.v1.RgbRingSetBrightnessAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingSetEnabledAction();\n  if (f != null) {\n    writer.writeMessage(\n      137,\n      f,\n      proto.ubo.v1.RgbRingSetEnabledAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingSetIsBusyAction();\n  if (f != null) {\n    writer.writeMessage(\n      138,\n      f,\n      proto.ubo.v1.RgbRingSetIsBusyAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingSpinningWheelAction();\n  if (f != null) {\n    writer.writeMessage(\n      139,\n      f,\n      proto.ubo.v1.RgbRingSpinningWheelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingWaitableCommandAction();\n  if (f != null) {\n    writer.writeMessage(\n      140,\n      f,\n      proto.ubo.v1.RgbRingWaitableCommandAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSshAction();\n  if (f != null) {\n    writer.writeMessage(\n      141,\n      f,\n      proto.ubo.v1.SSHAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSshClearEnabledStateAction();\n  if (f != null) {\n    writer.writeMessage(\n      142,\n      f,\n      proto.ubo.v1.SSHClearEnabledStateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSshUpdateStateAction();\n  if (f != null) {\n    writer.writeMessage(\n      143,\n      f,\n      proto.ubo.v1.SSHUpdateStateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSensorsAction();\n  if (f != null) {\n    writer.writeMessage(\n      144,\n      f,\n      proto.ubo.v1.SensorsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSensorsReportReadingAction();\n  if (f != null) {\n    writer.writeMessage(\n      145,\n      f,\n      proto.ubo.v1.SensorsReportReadingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSetAreEnclosuresVisibleAction();\n  if (f != null) {\n    writer.writeMessage(\n      146,\n      f,\n      proto.ubo.v1.SetAreEnclosuresVisibleAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSetMenuPathAction();\n  if (f != null) {\n    writer.writeMessage(\n      147,\n      f,\n      proto.ubo.v1.SetMenuPathAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsAction();\n  if (f != null) {\n    writer.writeMessage(\n      148,\n      f,\n      proto.ubo.v1.SettingsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsClearServiceErrorsAction();\n  if (f != null) {\n    writer.writeMessage(\n      149,\n      f,\n      proto.ubo.v1.SettingsClearServiceErrorsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsReportServiceErrorAction();\n  if (f != null) {\n    writer.writeMessage(\n      150,\n      f,\n      proto.ubo.v1.SettingsReportServiceErrorAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsServiceAction();\n  if (f != null) {\n    writer.writeMessage(\n      151,\n      f,\n      proto.ubo.v1.SettingsServiceAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsServiceSetIsEnabledAction();\n  if (f != null) {\n    writer.writeMessage(\n      152,\n      f,\n      proto.ubo.v1.SettingsServiceSetIsEnabledAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsServiceSetLogLevelAction();\n  if (f != null) {\n    writer.writeMessage(\n      153,\n      f,\n      proto.ubo.v1.SettingsServiceSetLogLevelAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsServiceSetShouldRestartAction();\n  if (f != null) {\n    writer.writeMessage(\n      154,\n      f,\n      proto.ubo.v1.SettingsServiceSetShouldRestartAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsServiceSetStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      155,\n      f,\n      proto.ubo.v1.SettingsServiceSetStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsSetServicesAction();\n  if (f != null) {\n    writer.writeMessage(\n      156,\n      f,\n      proto.ubo.v1.SettingsSetServicesAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsStartServiceAction();\n  if (f != null) {\n    writer.writeMessage(\n      157,\n      f,\n      proto.ubo.v1.SettingsStartServiceAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsStopServiceAction();\n  if (f != null) {\n    writer.writeMessage(\n      158,\n      f,\n      proto.ubo.v1.SettingsStopServiceAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsToggleBetaVersionsAction();\n  if (f != null) {\n    writer.writeMessage(\n      159,\n      f,\n      proto.ubo.v1.SettingsToggleBetaVersionsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsTogglePdbSignalAction();\n  if (f != null) {\n    writer.writeMessage(\n      160,\n      f,\n      proto.ubo.v1.SettingsTogglePdbSignalAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsToggleVisualDebugAction();\n  if (f != null) {\n    writer.writeMessage(\n      161,\n      f,\n      proto.ubo.v1.SettingsToggleVisualDebugAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionAction();\n  if (f != null) {\n    writer.writeMessage(\n      162,\n      f,\n      proto.ubo.v1.SpeechRecognitionAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionReportIntentDetectionAction();\n  if (f != null) {\n    writer.writeMessage(\n      163,\n      f,\n      proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionReportSpeechAction();\n  if (f != null) {\n    writer.writeMessage(\n      164,\n      f,\n      proto.ubo.v1.SpeechRecognitionReportSpeechAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionReportWakeWordDetectionAction();\n  if (f != null) {\n    writer.writeMessage(\n      165,\n      f,\n      proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionSetIsAssistantActiveAction();\n  if (f != null) {\n    writer.writeMessage(\n      166,\n      f,\n      proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionSetIsIntentsActiveAction();\n  if (f != null) {\n    writer.writeMessage(\n      167,\n      f,\n      proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionSetSelectedEngineAction();\n  if (f != null) {\n    writer.writeMessage(\n      168,\n      f,\n      proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechSynthesisAction();\n  if (f != null) {\n    writer.writeMessage(\n      169,\n      f,\n      proto.ubo.v1.SpeechSynthesisAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechSynthesisReadTextAction();\n  if (f != null) {\n    writer.writeMessage(\n      170,\n      f,\n      proto.ubo.v1.SpeechSynthesisReadTextAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechSynthesisSetSelectedEngineAction();\n  if (f != null) {\n    writer.writeMessage(\n      171,\n      f,\n      proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getStatusIconsAction();\n  if (f != null) {\n    writer.writeMessage(\n      172,\n      f,\n      proto.ubo.v1.StatusIconsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getStatusIconsRegisterAction();\n  if (f != null) {\n    writer.writeMessage(\n      173,\n      f,\n      proto.ubo.v1.StatusIconsRegisterAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getToggleRecordingAction();\n  if (f != null) {\n    writer.writeMessage(\n      174,\n      f,\n      proto.ubo.v1.ToggleRecordingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerAction();\n  if (f != null) {\n    writer.writeMessage(\n      175,\n      f,\n      proto.ubo.v1.UpdateManagerAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerReportFailedCheckAction();\n  if (f != null) {\n    writer.writeMessage(\n      176,\n      f,\n      proto.ubo.v1.UpdateManagerReportFailedCheckAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerRequestCheckAction();\n  if (f != null) {\n    writer.writeMessage(\n      177,\n      f,\n      proto.ubo.v1.UpdateManagerRequestCheckAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerRequestUpdateAction();\n  if (f != null) {\n    writer.writeMessage(\n      178,\n      f,\n      proto.ubo.v1.UpdateManagerRequestUpdateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerSetVersionsAction();\n  if (f != null) {\n    writer.writeMessage(\n      179,\n      f,\n      proto.ubo.v1.UpdateManagerSetVersionsAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersAction();\n  if (f != null) {\n    writer.writeMessage(\n      180,\n      f,\n      proto.ubo.v1.UsersAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersCreateUserAction();\n  if (f != null) {\n    writer.writeMessage(\n      181,\n      f,\n      proto.ubo.v1.UsersCreateUserAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersDeleteUserAction();\n  if (f != null) {\n    writer.writeMessage(\n      182,\n      f,\n      proto.ubo.v1.UsersDeleteUserAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersResetPasswordAction();\n  if (f != null) {\n    writer.writeMessage(\n      183,\n      f,\n      proto.ubo.v1.UsersResetPasswordAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersSetUsersAction();\n  if (f != null) {\n    writer.writeMessage(\n      184,\n      f,\n      proto.ubo.v1.UsersSetUsersAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeAction();\n  if (f != null) {\n    writer.writeMessage(\n      185,\n      f,\n      proto.ubo.v1.VSCodeAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeDoneDownloadingAction();\n  if (f != null) {\n    writer.writeMessage(\n      186,\n      f,\n      proto.ubo.v1.VSCodeDoneDownloadingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeSetPendingAction();\n  if (f != null) {\n    writer.writeMessage(\n      187,\n      f,\n      proto.ubo.v1.VSCodeSetPendingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeSetStatusAction();\n  if (f != null) {\n    writer.writeMessage(\n      188,\n      f,\n      proto.ubo.v1.VSCodeSetStatusAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeStartDownloadingAction();\n  if (f != null) {\n    writer.writeMessage(\n      189,\n      f,\n      proto.ubo.v1.VSCodeStartDownloadingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiAction();\n  if (f != null) {\n    writer.writeMessage(\n      190,\n      f,\n      proto.ubo.v1.WiFiAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiInputConnectionAction();\n  if (f != null) {\n    writer.writeMessage(\n      191,\n      f,\n      proto.ubo.v1.WiFiInputConnectionAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiSetHasVisitedOnboardingAction();\n  if (f != null) {\n    writer.writeMessage(\n      192,\n      f,\n      proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiUpdateAction();\n  if (f != null) {\n    writer.writeMessage(\n      193,\n      f,\n      proto.ubo.v1.WiFiUpdateAction.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiUpdateRequestAction();\n  if (f != null) {\n    writer.writeMessage(\n      194,\n      f,\n      proto.ubo.v1.WiFiUpdateRequestAction.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional AssistantAction assistant_action = 1;\n * @return {?proto.ubo.v1.AssistantAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantAction, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantAction = function() {\n  return this.setAssistantAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantAction = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional AssistantAddMcpServerAction assistant_add_mcp_server_action = 2;\n * @return {?proto.ubo.v1.AssistantAddMcpServerAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantAddMcpServerAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantAddMcpServerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantAddMcpServerAction, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantAddMcpServerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantAddMcpServerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantAddMcpServerAction = function() {\n  return this.setAssistantAddMcpServerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantAddMcpServerAction = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional AssistantDeleteMcpServerAction assistant_delete_mcp_server_action = 3;\n * @return {?proto.ubo.v1.AssistantDeleteMcpServerAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantDeleteMcpServerAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantDeleteMcpServerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDeleteMcpServerAction, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantDeleteMcpServerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantDeleteMcpServerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantDeleteMcpServerAction = function() {\n  return this.setAssistantDeleteMcpServerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantDeleteMcpServerAction = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional AssistantDownloadOllamaModelAction assistant_download_ollama_model_action = 4;\n * @return {?proto.ubo.v1.AssistantDownloadOllamaModelAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantDownloadOllamaModelAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantDownloadOllamaModelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDownloadOllamaModelAction, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantDownloadOllamaModelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantDownloadOllamaModelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 4, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantDownloadOllamaModelAction = function() {\n  return this.setAssistantDownloadOllamaModelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantDownloadOllamaModelAction = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional AssistantReportAction assistant_report_action = 5;\n * @return {?proto.ubo.v1.AssistantReportAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantReportAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantReportAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantReportAction, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantReportAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantReportAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 5, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantReportAction = function() {\n  return this.setAssistantReportAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantReportAction = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional AssistantSetIsActiveAction assistant_set_is_active_action = 6;\n * @return {?proto.ubo.v1.AssistantSetIsActiveAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSetIsActiveAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSetIsActiveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetIsActiveAction, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSetIsActiveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSetIsActiveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 6, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSetIsActiveAction = function() {\n  return this.setAssistantSetIsActiveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSetIsActiveAction = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional AssistantSetSelectedImageGeneratorAction assistant_set_selected_image_generator_action = 7;\n * @return {?proto.ubo.v1.AssistantSetSelectedImageGeneratorAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSetSelectedImageGeneratorAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedImageGeneratorAction, 7));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSetSelectedImageGeneratorAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSetSelectedImageGeneratorAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 7, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSetSelectedImageGeneratorAction = function() {\n  return this.setAssistantSetSelectedImageGeneratorAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSetSelectedImageGeneratorAction = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional AssistantSetSelectedLLMAction assistant_set_selected_llm_action = 8;\n * @return {?proto.ubo.v1.AssistantSetSelectedLLMAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSetSelectedLlmAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSetSelectedLLMAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedLLMAction, 8));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSetSelectedLLMAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSetSelectedLlmAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 8, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSetSelectedLlmAction = function() {\n  return this.setAssistantSetSelectedLlmAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSetSelectedLlmAction = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional AssistantSetSelectedModelAction assistant_set_selected_model_action = 9;\n * @return {?proto.ubo.v1.AssistantSetSelectedModelAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSetSelectedModelAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSetSelectedModelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedModelAction, 9));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSetSelectedModelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSetSelectedModelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 9, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSetSelectedModelAction = function() {\n  return this.setAssistantSetSelectedModelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSetSelectedModelAction = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional AssistantSetSelectedSTTAction assistant_set_selected_stt_action = 10;\n * @return {?proto.ubo.v1.AssistantSetSelectedSTTAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSetSelectedSttAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSetSelectedSTTAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedSTTAction, 10));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSetSelectedSTTAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSetSelectedSttAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 10, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSetSelectedSttAction = function() {\n  return this.setAssistantSetSelectedSttAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSetSelectedSttAction = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional AssistantSetSelectedTTSAction assistant_set_selected_tts_action = 11;\n * @return {?proto.ubo.v1.AssistantSetSelectedTTSAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSetSelectedTtsAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSetSelectedTTSAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedTTSAction, 11));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSetSelectedTTSAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSetSelectedTtsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 11, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSetSelectedTtsAction = function() {\n  return this.setAssistantSetSelectedTtsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSetSelectedTtsAction = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional AssistantStartListeningAction assistant_start_listening_action = 12;\n * @return {?proto.ubo.v1.AssistantStartListeningAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantStartListeningAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantStartListeningAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantStartListeningAction, 12));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantStartListeningAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantStartListeningAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 12, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantStartListeningAction = function() {\n  return this.setAssistantStartListeningAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantStartListeningAction = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional AssistantStopListeningAction assistant_stop_listening_action = 13;\n * @return {?proto.ubo.v1.AssistantStopListeningAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantStopListeningAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantStopListeningAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantStopListeningAction, 13));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantStopListeningAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantStopListeningAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 13, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantStopListeningAction = function() {\n  return this.setAssistantStopListeningAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantStopListeningAction = function() {\n  return jspb.Message.getField(this, 13) != null;\n};\n\n\n/**\n * optional AssistantSyncMcpServersAction assistant_sync_mcp_servers_action = 14;\n * @return {?proto.ubo.v1.AssistantSyncMcpServersAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantSyncMcpServersAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantSyncMcpServersAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSyncMcpServersAction, 14));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantSyncMcpServersAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantSyncMcpServersAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 14, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantSyncMcpServersAction = function() {\n  return this.setAssistantSyncMcpServersAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantSyncMcpServersAction = function() {\n  return jspb.Message.getField(this, 14) != null;\n};\n\n\n/**\n * optional AssistantToggleListeningAction assistant_toggle_listening_action = 15;\n * @return {?proto.ubo.v1.AssistantToggleListeningAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantToggleListeningAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantToggleListeningAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantToggleListeningAction, 15));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantToggleListeningAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantToggleListeningAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 15, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantToggleListeningAction = function() {\n  return this.setAssistantToggleListeningAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantToggleListeningAction = function() {\n  return jspb.Message.getField(this, 15) != null;\n};\n\n\n/**\n * optional AssistantToggleMcpServerAction assistant_toggle_mcp_server_action = 16;\n * @return {?proto.ubo.v1.AssistantToggleMcpServerAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantToggleMcpServerAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantToggleMcpServerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantToggleMcpServerAction, 16));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantToggleMcpServerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantToggleMcpServerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 16, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantToggleMcpServerAction = function() {\n  return this.setAssistantToggleMcpServerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantToggleMcpServerAction = function() {\n  return jspb.Message.getField(this, 16) != null;\n};\n\n\n/**\n * optional AssistantUpdateProvidersAction assistant_update_providers_action = 17;\n * @return {?proto.ubo.v1.AssistantUpdateProvidersAction}\n */\nproto.ubo.v1.Action.prototype.getAssistantUpdateProvidersAction = function() {\n  return /** @type{?proto.ubo.v1.AssistantUpdateProvidersAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantUpdateProvidersAction, 17));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantUpdateProvidersAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAssistantUpdateProvidersAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 17, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAssistantUpdateProvidersAction = function() {\n  return this.setAssistantUpdateProvidersAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAssistantUpdateProvidersAction = function() {\n  return jspb.Message.getField(this, 17) != null;\n};\n\n\n/**\n * optional AudioAction audio_action = 18;\n * @return {?proto.ubo.v1.AudioAction}\n */\nproto.ubo.v1.Action.prototype.getAudioAction = function() {\n  return /** @type{?proto.ubo.v1.AudioAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioAction, 18));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 18, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioAction = function() {\n  return this.setAudioAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioAction = function() {\n  return jspb.Message.getField(this, 18) != null;\n};\n\n\n/**\n * optional AudioChangeVolumeAction audio_change_volume_action = 19;\n * @return {?proto.ubo.v1.AudioChangeVolumeAction}\n */\nproto.ubo.v1.Action.prototype.getAudioChangeVolumeAction = function() {\n  return /** @type{?proto.ubo.v1.AudioChangeVolumeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioChangeVolumeAction, 19));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioChangeVolumeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioChangeVolumeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 19, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioChangeVolumeAction = function() {\n  return this.setAudioChangeVolumeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioChangeVolumeAction = function() {\n  return jspb.Message.getField(this, 19) != null;\n};\n\n\n/**\n * optional AudioInstallDriverAction audio_install_driver_action = 20;\n * @return {?proto.ubo.v1.AudioInstallDriverAction}\n */\nproto.ubo.v1.Action.prototype.getAudioInstallDriverAction = function() {\n  return /** @type{?proto.ubo.v1.AudioInstallDriverAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioInstallDriverAction, 20));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioInstallDriverAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioInstallDriverAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 20, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioInstallDriverAction = function() {\n  return this.setAudioInstallDriverAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioInstallDriverAction = function() {\n  return jspb.Message.getField(this, 20) != null;\n};\n\n\n/**\n * optional AudioPlayAudioSampleAction audio_play_audio_sample_action = 21;\n * @return {?proto.ubo.v1.AudioPlayAudioSampleAction}\n */\nproto.ubo.v1.Action.prototype.getAudioPlayAudioSampleAction = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayAudioSampleAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSampleAction, 21));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayAudioSampleAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioPlayAudioSampleAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 21, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioPlayAudioSampleAction = function() {\n  return this.setAudioPlayAudioSampleAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioPlayAudioSampleAction = function() {\n  return jspb.Message.getField(this, 21) != null;\n};\n\n\n/**\n * optional AudioPlayAudioSequenceAction audio_play_audio_sequence_action = 22;\n * @return {?proto.ubo.v1.AudioPlayAudioSequenceAction}\n */\nproto.ubo.v1.Action.prototype.getAudioPlayAudioSequenceAction = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayAudioSequenceAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSequenceAction, 22));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayAudioSequenceAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioPlayAudioSequenceAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 22, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioPlayAudioSequenceAction = function() {\n  return this.setAudioPlayAudioSequenceAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioPlayAudioSequenceAction = function() {\n  return jspb.Message.getField(this, 22) != null;\n};\n\n\n/**\n * optional AudioPlayChimeAction audio_play_chime_action = 23;\n * @return {?proto.ubo.v1.AudioPlayChimeAction}\n */\nproto.ubo.v1.Action.prototype.getAudioPlayChimeAction = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayChimeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayChimeAction, 23));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayChimeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioPlayChimeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 23, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioPlayChimeAction = function() {\n  return this.setAudioPlayChimeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioPlayChimeAction = function() {\n  return jspb.Message.getField(this, 23) != null;\n};\n\n\n/**\n * optional AudioPlayRecordingAction audio_play_recording_action = 24;\n * @return {?proto.ubo.v1.AudioPlayRecordingAction}\n */\nproto.ubo.v1.Action.prototype.getAudioPlayRecordingAction = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayRecordingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayRecordingAction, 24));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayRecordingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioPlayRecordingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 24, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioPlayRecordingAction = function() {\n  return this.setAudioPlayRecordingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioPlayRecordingAction = function() {\n  return jspb.Message.getField(this, 24) != null;\n};\n\n\n/**\n * optional AudioPlaybackDoneAction audio_playback_done_action = 25;\n * @return {?proto.ubo.v1.AudioPlaybackDoneAction}\n */\nproto.ubo.v1.Action.prototype.getAudioPlaybackDoneAction = function() {\n  return /** @type{?proto.ubo.v1.AudioPlaybackDoneAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlaybackDoneAction, 25));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlaybackDoneAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioPlaybackDoneAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 25, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioPlaybackDoneAction = function() {\n  return this.setAudioPlaybackDoneAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioPlaybackDoneAction = function() {\n  return jspb.Message.getField(this, 25) != null;\n};\n\n\n/**\n * optional AudioReportSampleAction audio_report_sample_action = 26;\n * @return {?proto.ubo.v1.AudioReportSampleAction}\n */\nproto.ubo.v1.Action.prototype.getAudioReportSampleAction = function() {\n  return /** @type{?proto.ubo.v1.AudioReportSampleAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioReportSampleAction, 26));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioReportSampleAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioReportSampleAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 26, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioReportSampleAction = function() {\n  return this.setAudioReportSampleAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioReportSampleAction = function() {\n  return jspb.Message.getField(this, 26) != null;\n};\n\n\n/**\n * optional AudioSetMuteStatusAction audio_set_mute_status_action = 27;\n * @return {?proto.ubo.v1.AudioSetMuteStatusAction}\n */\nproto.ubo.v1.Action.prototype.getAudioSetMuteStatusAction = function() {\n  return /** @type{?proto.ubo.v1.AudioSetMuteStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSetMuteStatusAction, 27));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSetMuteStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioSetMuteStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 27, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioSetMuteStatusAction = function() {\n  return this.setAudioSetMuteStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioSetMuteStatusAction = function() {\n  return jspb.Message.getField(this, 27) != null;\n};\n\n\n/**\n * optional AudioSetVolumeAction audio_set_volume_action = 28;\n * @return {?proto.ubo.v1.AudioSetVolumeAction}\n */\nproto.ubo.v1.Action.prototype.getAudioSetVolumeAction = function() {\n  return /** @type{?proto.ubo.v1.AudioSetVolumeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSetVolumeAction, 28));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioSetVolumeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioSetVolumeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 28, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioSetVolumeAction = function() {\n  return this.setAudioSetVolumeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioSetVolumeAction = function() {\n  return jspb.Message.getField(this, 28) != null;\n};\n\n\n/**\n * optional AudioStartRecordingAction audio_start_recording_action = 29;\n * @return {?proto.ubo.v1.AudioStartRecordingAction}\n */\nproto.ubo.v1.Action.prototype.getAudioStartRecordingAction = function() {\n  return /** @type{?proto.ubo.v1.AudioStartRecordingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioStartRecordingAction, 29));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioStartRecordingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioStartRecordingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 29, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioStartRecordingAction = function() {\n  return this.setAudioStartRecordingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioStartRecordingAction = function() {\n  return jspb.Message.getField(this, 29) != null;\n};\n\n\n/**\n * optional AudioStopRecordingAction audio_stop_recording_action = 30;\n * @return {?proto.ubo.v1.AudioStopRecordingAction}\n */\nproto.ubo.v1.Action.prototype.getAudioStopRecordingAction = function() {\n  return /** @type{?proto.ubo.v1.AudioStopRecordingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioStopRecordingAction, 30));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioStopRecordingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioStopRecordingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 30, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioStopRecordingAction = function() {\n  return this.setAudioStopRecordingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioStopRecordingAction = function() {\n  return jspb.Message.getField(this, 30) != null;\n};\n\n\n/**\n * optional AudioToggleMuteStatusAction audio_toggle_mute_status_action = 31;\n * @return {?proto.ubo.v1.AudioToggleMuteStatusAction}\n */\nproto.ubo.v1.Action.prototype.getAudioToggleMuteStatusAction = function() {\n  return /** @type{?proto.ubo.v1.AudioToggleMuteStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioToggleMuteStatusAction, 31));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioToggleMuteStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioToggleMuteStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 31, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioToggleMuteStatusAction = function() {\n  return this.setAudioToggleMuteStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioToggleMuteStatusAction = function() {\n  return jspb.Message.getField(this, 31) != null;\n};\n\n\n/**\n * optional AudioToggleRecordingAction audio_toggle_recording_action = 32;\n * @return {?proto.ubo.v1.AudioToggleRecordingAction}\n */\nproto.ubo.v1.Action.prototype.getAudioToggleRecordingAction = function() {\n  return /** @type{?proto.ubo.v1.AudioToggleRecordingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioToggleRecordingAction, 32));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioToggleRecordingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setAudioToggleRecordingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 32, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearAudioToggleRecordingAction = function() {\n  return this.setAudioToggleRecordingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasAudioToggleRecordingAction = function() {\n  return jspb.Message.getField(this, 32) != null;\n};\n\n\n/**\n * optional CameraAction camera_action = 33;\n * @return {?proto.ubo.v1.CameraAction}\n */\nproto.ubo.v1.Action.prototype.getCameraAction = function() {\n  return /** @type{?proto.ubo.v1.CameraAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraAction, 33));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 33, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraAction = function() {\n  return this.setCameraAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraAction = function() {\n  return jspb.Message.getField(this, 33) != null;\n};\n\n\n/**\n * optional CameraDetectAction camera_detect_action = 34;\n * @return {?proto.ubo.v1.CameraDetectAction}\n */\nproto.ubo.v1.Action.prototype.getCameraDetectAction = function() {\n  return /** @type{?proto.ubo.v1.CameraDetectAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraDetectAction, 34));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraDetectAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraDetectAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 34, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraDetectAction = function() {\n  return this.setCameraDetectAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraDetectAction = function() {\n  return jspb.Message.getField(this, 34) != null;\n};\n\n\n/**\n * optional CameraInstallDriverAction camera_install_driver_action = 35;\n * @return {?proto.ubo.v1.CameraInstallDriverAction}\n */\nproto.ubo.v1.Action.prototype.getCameraInstallDriverAction = function() {\n  return /** @type{?proto.ubo.v1.CameraInstallDriverAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraInstallDriverAction, 35));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraInstallDriverAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraInstallDriverAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 35, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraInstallDriverAction = function() {\n  return this.setCameraInstallDriverAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraInstallDriverAction = function() {\n  return jspb.Message.getField(this, 35) != null;\n};\n\n\n/**\n * optional CameraReportBarcodeAction camera_report_barcode_action = 36;\n * @return {?proto.ubo.v1.CameraReportBarcodeAction}\n */\nproto.ubo.v1.Action.prototype.getCameraReportBarcodeAction = function() {\n  return /** @type{?proto.ubo.v1.CameraReportBarcodeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraReportBarcodeAction, 36));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraReportBarcodeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraReportBarcodeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 36, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraReportBarcodeAction = function() {\n  return this.setCameraReportBarcodeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraReportBarcodeAction = function() {\n  return jspb.Message.getField(this, 36) != null;\n};\n\n\n/**\n * optional CameraRestoreDefaultAction camera_restore_default_action = 37;\n * @return {?proto.ubo.v1.CameraRestoreDefaultAction}\n */\nproto.ubo.v1.Action.prototype.getCameraRestoreDefaultAction = function() {\n  return /** @type{?proto.ubo.v1.CameraRestoreDefaultAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraRestoreDefaultAction, 37));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraRestoreDefaultAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraRestoreDefaultAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 37, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraRestoreDefaultAction = function() {\n  return this.setCameraRestoreDefaultAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraRestoreDefaultAction = function() {\n  return jspb.Message.getField(this, 37) != null;\n};\n\n\n/**\n * optional CameraSetAvailableCamerasAction camera_set_available_cameras_action = 38;\n * @return {?proto.ubo.v1.CameraSetAvailableCamerasAction}\n */\nproto.ubo.v1.Action.prototype.getCameraSetAvailableCamerasAction = function() {\n  return /** @type{?proto.ubo.v1.CameraSetAvailableCamerasAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraSetAvailableCamerasAction, 38));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraSetAvailableCamerasAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraSetAvailableCamerasAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 38, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraSetAvailableCamerasAction = function() {\n  return this.setCameraSetAvailableCamerasAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraSetAvailableCamerasAction = function() {\n  return jspb.Message.getField(this, 38) != null;\n};\n\n\n/**\n * optional CameraSetIndexAction camera_set_index_action = 39;\n * @return {?proto.ubo.v1.CameraSetIndexAction}\n */\nproto.ubo.v1.Action.prototype.getCameraSetIndexAction = function() {\n  return /** @type{?proto.ubo.v1.CameraSetIndexAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraSetIndexAction, 39));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraSetIndexAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraSetIndexAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 39, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraSetIndexAction = function() {\n  return this.setCameraSetIndexAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraSetIndexAction = function() {\n  return jspb.Message.getField(this, 39) != null;\n};\n\n\n/**\n * optional CameraStartViewfinderAction camera_start_viewfinder_action = 40;\n * @return {?proto.ubo.v1.CameraStartViewfinderAction}\n */\nproto.ubo.v1.Action.prototype.getCameraStartViewfinderAction = function() {\n  return /** @type{?proto.ubo.v1.CameraStartViewfinderAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraStartViewfinderAction, 40));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraStartViewfinderAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCameraStartViewfinderAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 40, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCameraStartViewfinderAction = function() {\n  return this.setCameraStartViewfinderAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCameraStartViewfinderAction = function() {\n  return jspb.Message.getField(this, 40) != null;\n};\n\n\n/**\n * optional CloseApplicationAction close_application_action = 41;\n * @return {?proto.ubo.v1.CloseApplicationAction}\n */\nproto.ubo.v1.Action.prototype.getCloseApplicationAction = function() {\n  return /** @type{?proto.ubo.v1.CloseApplicationAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CloseApplicationAction, 41));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CloseApplicationAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setCloseApplicationAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 41, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearCloseApplicationAction = function() {\n  return this.setCloseApplicationAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasCloseApplicationAction = function() {\n  return jspb.Message.getField(this, 41) != null;\n};\n\n\n/**\n * optional DeregisterRegularAppAction deregister_regular_app_action = 42;\n * @return {?proto.ubo.v1.DeregisterRegularAppAction}\n */\nproto.ubo.v1.Action.prototype.getDeregisterRegularAppAction = function() {\n  return /** @type{?proto.ubo.v1.DeregisterRegularAppAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DeregisterRegularAppAction, 42));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DeregisterRegularAppAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDeregisterRegularAppAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 42, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDeregisterRegularAppAction = function() {\n  return this.setDeregisterRegularAppAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDeregisterRegularAppAction = function() {\n  return jspb.Message.getField(this, 42) != null;\n};\n\n\n/**\n * optional DisplayAction display_action = 43;\n * @return {?proto.ubo.v1.DisplayAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayAction, 43));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 43, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayAction = function() {\n  return this.setDisplayAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayAction = function() {\n  return jspb.Message.getField(this, 43) != null;\n};\n\n\n/**\n * optional DisplayBlankAction display_blank_action = 44;\n * @return {?proto.ubo.v1.DisplayBlankAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayBlankAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayBlankAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayBlankAction, 44));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayBlankAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayBlankAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 44, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayBlankAction = function() {\n  return this.setDisplayBlankAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayBlankAction = function() {\n  return jspb.Message.getField(this, 44) != null;\n};\n\n\n/**\n * optional DisplayPauseAction display_pause_action = 45;\n * @return {?proto.ubo.v1.DisplayPauseAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayPauseAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayPauseAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayPauseAction, 45));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayPauseAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayPauseAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 45, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayPauseAction = function() {\n  return this.setDisplayPauseAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayPauseAction = function() {\n  return jspb.Message.getField(this, 45) != null;\n};\n\n\n/**\n * optional DisplayRedrawAction display_redraw_action = 46;\n * @return {?proto.ubo.v1.DisplayRedrawAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayRedrawAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayRedrawAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayRedrawAction, 46));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayRedrawAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayRedrawAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 46, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayRedrawAction = function() {\n  return this.setDisplayRedrawAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayRedrawAction = function() {\n  return jspb.Message.getField(this, 46) != null;\n};\n\n\n/**\n * optional DisplayResumeAction display_resume_action = 47;\n * @return {?proto.ubo.v1.DisplayResumeAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayResumeAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayResumeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayResumeAction, 47));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayResumeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayResumeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 47, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayResumeAction = function() {\n  return this.setDisplayResumeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayResumeAction = function() {\n  return jspb.Message.getField(this, 47) != null;\n};\n\n\n/**\n * optional DisplaySetBlankTimeoutAction display_set_blank_timeout_action = 48;\n * @return {?proto.ubo.v1.DisplaySetBlankTimeoutAction}\n */\nproto.ubo.v1.Action.prototype.getDisplaySetBlankTimeoutAction = function() {\n  return /** @type{?proto.ubo.v1.DisplaySetBlankTimeoutAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplaySetBlankTimeoutAction, 48));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplaySetBlankTimeoutAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplaySetBlankTimeoutAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 48, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplaySetBlankTimeoutAction = function() {\n  return this.setDisplaySetBlankTimeoutAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplaySetBlankTimeoutAction = function() {\n  return jspb.Message.getField(this, 48) != null;\n};\n\n\n/**\n * optional DisplayUnblankAction display_unblank_action = 49;\n * @return {?proto.ubo.v1.DisplayUnblankAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayUnblankAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayUnblankAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayUnblankAction, 49));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayUnblankAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayUnblankAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 49, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayUnblankAction = function() {\n  return this.setDisplayUnblankAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayUnblankAction = function() {\n  return jspb.Message.getField(this, 49) != null;\n};\n\n\n/**\n * optional DisplayUpdateActivityAction display_update_activity_action = 50;\n * @return {?proto.ubo.v1.DisplayUpdateActivityAction}\n */\nproto.ubo.v1.Action.prototype.getDisplayUpdateActivityAction = function() {\n  return /** @type{?proto.ubo.v1.DisplayUpdateActivityAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayUpdateActivityAction, 50));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayUpdateActivityAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDisplayUpdateActivityAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 50, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDisplayUpdateActivityAction = function() {\n  return this.setDisplayUpdateActivityAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDisplayUpdateActivityAction = function() {\n  return jspb.Message.getField(this, 50) != null;\n};\n\n\n/**\n * optional DockerAction docker_action = 51;\n * @return {?proto.ubo.v1.DockerAction}\n */\nproto.ubo.v1.Action.prototype.getDockerAction = function() {\n  return /** @type{?proto.ubo.v1.DockerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerAction, 51));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 51, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerAction = function() {\n  return this.setDockerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerAction = function() {\n  return jspb.Message.getField(this, 51) != null;\n};\n\n\n/**\n * optional DockerImageAction docker_image_action = 52;\n * @return {?proto.ubo.v1.DockerImageAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageAction, 52));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 52, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageAction = function() {\n  return this.setDockerImageAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageAction = function() {\n  return jspb.Message.getField(this, 52) != null;\n};\n\n\n/**\n * optional DockerImageFetchAction docker_image_fetch_action = 53;\n * @return {?proto.ubo.v1.DockerImageFetchAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageFetchAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageFetchAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageFetchAction, 53));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageFetchAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageFetchAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 53, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageFetchAction = function() {\n  return this.setDockerImageFetchAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageFetchAction = function() {\n  return jspb.Message.getField(this, 53) != null;\n};\n\n\n/**\n * optional DockerImageReleaseAction docker_image_release_action = 54;\n * @return {?proto.ubo.v1.DockerImageReleaseAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageReleaseAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageReleaseAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageReleaseAction, 54));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageReleaseAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageReleaseAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 54, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageReleaseAction = function() {\n  return this.setDockerImageReleaseAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageReleaseAction = function() {\n  return jspb.Message.getField(this, 54) != null;\n};\n\n\n/**\n * optional DockerImageRemoveAction docker_image_remove_action = 55;\n * @return {?proto.ubo.v1.DockerImageRemoveAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageRemoveAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRemoveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveAction, 55));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRemoveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageRemoveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 55, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageRemoveAction = function() {\n  return this.setDockerImageRemoveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageRemoveAction = function() {\n  return jspb.Message.getField(this, 55) != null;\n};\n\n\n/**\n * optional DockerImageRemoveContainerAction docker_image_remove_container_action = 56;\n * @return {?proto.ubo.v1.DockerImageRemoveContainerAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageRemoveContainerAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRemoveContainerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveContainerAction, 56));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRemoveContainerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageRemoveContainerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 56, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageRemoveContainerAction = function() {\n  return this.setDockerImageRemoveContainerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageRemoveContainerAction = function() {\n  return jspb.Message.getField(this, 56) != null;\n};\n\n\n/**\n * optional DockerImageRunAction docker_image_run_action = 57;\n * @return {?proto.ubo.v1.DockerImageRunAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageRunAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRunAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRunAction, 57));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRunAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageRunAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 57, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageRunAction = function() {\n  return this.setDockerImageRunAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageRunAction = function() {\n  return jspb.Message.getField(this, 57) != null;\n};\n\n\n/**\n * optional DockerImageSetDockerIdAction docker_image_set_docker_id_action = 58;\n * @return {?proto.ubo.v1.DockerImageSetDockerIdAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageSetDockerIdAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageSetDockerIdAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageSetDockerIdAction, 58));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageSetDockerIdAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageSetDockerIdAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 58, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageSetDockerIdAction = function() {\n  return this.setDockerImageSetDockerIdAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageSetDockerIdAction = function() {\n  return jspb.Message.getField(this, 58) != null;\n};\n\n\n/**\n * optional DockerImageSetStatusAction docker_image_set_status_action = 59;\n * @return {?proto.ubo.v1.DockerImageSetStatusAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageSetStatusAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageSetStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageSetStatusAction, 59));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageSetStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageSetStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 59, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageSetStatusAction = function() {\n  return this.setDockerImageSetStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageSetStatusAction = function() {\n  return jspb.Message.getField(this, 59) != null;\n};\n\n\n/**\n * optional DockerImageStopAction docker_image_stop_action = 60;\n * @return {?proto.ubo.v1.DockerImageStopAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageStopAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageStopAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageStopAction, 60));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageStopAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageStopAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 60, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageStopAction = function() {\n  return this.setDockerImageStopAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageStopAction = function() {\n  return jspb.Message.getField(this, 60) != null;\n};\n\n\n/**\n * optional DockerImageUpdateMetadataAction docker_image_update_metadata_action = 61;\n * @return {?proto.ubo.v1.DockerImageUpdateMetadataAction}\n */\nproto.ubo.v1.Action.prototype.getDockerImageUpdateMetadataAction = function() {\n  return /** @type{?proto.ubo.v1.DockerImageUpdateMetadataAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageUpdateMetadataAction, 61));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageUpdateMetadataAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerImageUpdateMetadataAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 61, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerImageUpdateMetadataAction = function() {\n  return this.setDockerImageUpdateMetadataAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerImageUpdateMetadataAction = function() {\n  return jspb.Message.getField(this, 61) != null;\n};\n\n\n/**\n * optional DockerInstallAction docker_install_action = 62;\n * @return {?proto.ubo.v1.DockerInstallAction}\n */\nproto.ubo.v1.Action.prototype.getDockerInstallAction = function() {\n  return /** @type{?proto.ubo.v1.DockerInstallAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerInstallAction, 62));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerInstallAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerInstallAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 62, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerInstallAction = function() {\n  return this.setDockerInstallAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerInstallAction = function() {\n  return jspb.Message.getField(this, 62) != null;\n};\n\n\n/**\n * optional DockerRemoveUsernameAction docker_remove_username_action = 63;\n * @return {?proto.ubo.v1.DockerRemoveUsernameAction}\n */\nproto.ubo.v1.Action.prototype.getDockerRemoveUsernameAction = function() {\n  return /** @type{?proto.ubo.v1.DockerRemoveUsernameAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerRemoveUsernameAction, 63));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerRemoveUsernameAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerRemoveUsernameAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 63, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerRemoveUsernameAction = function() {\n  return this.setDockerRemoveUsernameAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerRemoveUsernameAction = function() {\n  return jspb.Message.getField(this, 63) != null;\n};\n\n\n/**\n * optional DockerSetStatusAction docker_set_status_action = 64;\n * @return {?proto.ubo.v1.DockerSetStatusAction}\n */\nproto.ubo.v1.Action.prototype.getDockerSetStatusAction = function() {\n  return /** @type{?proto.ubo.v1.DockerSetStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerSetStatusAction, 64));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerSetStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerSetStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 64, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerSetStatusAction = function() {\n  return this.setDockerSetStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerSetStatusAction = function() {\n  return jspb.Message.getField(this, 64) != null;\n};\n\n\n/**\n * optional DockerStartAction docker_start_action = 65;\n * @return {?proto.ubo.v1.DockerStartAction}\n */\nproto.ubo.v1.Action.prototype.getDockerStartAction = function() {\n  return /** @type{?proto.ubo.v1.DockerStartAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStartAction, 65));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerStartAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerStartAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 65, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerStartAction = function() {\n  return this.setDockerStartAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerStartAction = function() {\n  return jspb.Message.getField(this, 65) != null;\n};\n\n\n/**\n * optional DockerStopAction docker_stop_action = 66;\n * @return {?proto.ubo.v1.DockerStopAction}\n */\nproto.ubo.v1.Action.prototype.getDockerStopAction = function() {\n  return /** @type{?proto.ubo.v1.DockerStopAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStopAction, 66));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerStopAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerStopAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 66, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerStopAction = function() {\n  return this.setDockerStopAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerStopAction = function() {\n  return jspb.Message.getField(this, 66) != null;\n};\n\n\n/**\n * optional DockerStoreUsernameAction docker_store_username_action = 67;\n * @return {?proto.ubo.v1.DockerStoreUsernameAction}\n */\nproto.ubo.v1.Action.prototype.getDockerStoreUsernameAction = function() {\n  return /** @type{?proto.ubo.v1.DockerStoreUsernameAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStoreUsernameAction, 67));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerStoreUsernameAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setDockerStoreUsernameAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 67, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearDockerStoreUsernameAction = function() {\n  return this.setDockerStoreUsernameAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasDockerStoreUsernameAction = function() {\n  return jspb.Message.getField(this, 67) != null;\n};\n\n\n/**\n * optional FileSystemAction file_system_action = 68;\n * @return {?proto.ubo.v1.FileSystemAction}\n */\nproto.ubo.v1.Action.prototype.getFileSystemAction = function() {\n  return /** @type{?proto.ubo.v1.FileSystemAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemAction, 68));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setFileSystemAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 68, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearFileSystemAction = function() {\n  return this.setFileSystemAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasFileSystemAction = function() {\n  return jspb.Message.getField(this, 68) != null;\n};\n\n\n/**\n * optional FileSystemCopyAction file_system_copy_action = 69;\n * @return {?proto.ubo.v1.FileSystemCopyAction}\n */\nproto.ubo.v1.Action.prototype.getFileSystemCopyAction = function() {\n  return /** @type{?proto.ubo.v1.FileSystemCopyAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemCopyAction, 69));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemCopyAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setFileSystemCopyAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 69, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearFileSystemCopyAction = function() {\n  return this.setFileSystemCopyAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasFileSystemCopyAction = function() {\n  return jspb.Message.getField(this, 69) != null;\n};\n\n\n/**\n * optional FileSystemMoveAction file_system_move_action = 70;\n * @return {?proto.ubo.v1.FileSystemMoveAction}\n */\nproto.ubo.v1.Action.prototype.getFileSystemMoveAction = function() {\n  return /** @type{?proto.ubo.v1.FileSystemMoveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemMoveAction, 70));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemMoveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setFileSystemMoveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 70, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearFileSystemMoveAction = function() {\n  return this.setFileSystemMoveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasFileSystemMoveAction = function() {\n  return jspb.Message.getField(this, 70) != null;\n};\n\n\n/**\n * optional FileSystemRemoveAction file_system_remove_action = 71;\n * @return {?proto.ubo.v1.FileSystemRemoveAction}\n */\nproto.ubo.v1.Action.prototype.getFileSystemRemoveAction = function() {\n  return /** @type{?proto.ubo.v1.FileSystemRemoveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemRemoveAction, 71));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemRemoveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setFileSystemRemoveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 71, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearFileSystemRemoveAction = function() {\n  return this.setFileSystemRemoveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasFileSystemRemoveAction = function() {\n  return jspb.Message.getField(this, 71) != null;\n};\n\n\n/**\n * optional FileSystemReportSelectionAction file_system_report_selection_action = 72;\n * @return {?proto.ubo.v1.FileSystemReportSelectionAction}\n */\nproto.ubo.v1.Action.prototype.getFileSystemReportSelectionAction = function() {\n  return /** @type{?proto.ubo.v1.FileSystemReportSelectionAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemReportSelectionAction, 72));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemReportSelectionAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setFileSystemReportSelectionAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 72, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearFileSystemReportSelectionAction = function() {\n  return this.setFileSystemReportSelectionAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasFileSystemReportSelectionAction = function() {\n  return jspb.Message.getField(this, 72) != null;\n};\n\n\n/**\n * optional InfraredAction infrared_action = 73;\n * @return {?proto.ubo.v1.InfraredAction}\n */\nproto.ubo.v1.Action.prototype.getInfraredAction = function() {\n  return /** @type{?proto.ubo.v1.InfraredAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredAction, 73));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInfraredAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 73, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInfraredAction = function() {\n  return this.setInfraredAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInfraredAction = function() {\n  return jspb.Message.getField(this, 73) != null;\n};\n\n\n/**\n * optional InfraredHandleReceivedCodeAction infrared_handle_received_code_action = 74;\n * @return {?proto.ubo.v1.InfraredHandleReceivedCodeAction}\n */\nproto.ubo.v1.Action.prototype.getInfraredHandleReceivedCodeAction = function() {\n  return /** @type{?proto.ubo.v1.InfraredHandleReceivedCodeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredHandleReceivedCodeAction, 74));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredHandleReceivedCodeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInfraredHandleReceivedCodeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 74, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInfraredHandleReceivedCodeAction = function() {\n  return this.setInfraredHandleReceivedCodeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInfraredHandleReceivedCodeAction = function() {\n  return jspb.Message.getField(this, 74) != null;\n};\n\n\n/**\n * optional InfraredSendCodeAction infrared_send_code_action = 75;\n * @return {?proto.ubo.v1.InfraredSendCodeAction}\n */\nproto.ubo.v1.Action.prototype.getInfraredSendCodeAction = function() {\n  return /** @type{?proto.ubo.v1.InfraredSendCodeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSendCodeAction, 75));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredSendCodeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInfraredSendCodeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 75, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInfraredSendCodeAction = function() {\n  return this.setInfraredSendCodeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInfraredSendCodeAction = function() {\n  return jspb.Message.getField(this, 75) != null;\n};\n\n\n/**\n * optional InfraredSetShouldPropagateAction infrared_set_should_propagate_action = 76;\n * @return {?proto.ubo.v1.InfraredSetShouldPropagateAction}\n */\nproto.ubo.v1.Action.prototype.getInfraredSetShouldPropagateAction = function() {\n  return /** @type{?proto.ubo.v1.InfraredSetShouldPropagateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSetShouldPropagateAction, 76));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredSetShouldPropagateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInfraredSetShouldPropagateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 76, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInfraredSetShouldPropagateAction = function() {\n  return this.setInfraredSetShouldPropagateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInfraredSetShouldPropagateAction = function() {\n  return jspb.Message.getField(this, 76) != null;\n};\n\n\n/**\n * optional InfraredSetShouldReceiveAction infrared_set_should_receive_action = 77;\n * @return {?proto.ubo.v1.InfraredSetShouldReceiveAction}\n */\nproto.ubo.v1.Action.prototype.getInfraredSetShouldReceiveAction = function() {\n  return /** @type{?proto.ubo.v1.InfraredSetShouldReceiveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSetShouldReceiveAction, 77));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredSetShouldReceiveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInfraredSetShouldReceiveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 77, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInfraredSetShouldReceiveAction = function() {\n  return this.setInfraredSetShouldReceiveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInfraredSetShouldReceiveAction = function() {\n  return jspb.Message.getField(this, 77) != null;\n};\n\n\n/**\n * optional InputAction input_action = 78;\n * @return {?proto.ubo.v1.InputAction}\n */\nproto.ubo.v1.Action.prototype.getInputAction = function() {\n  return /** @type{?proto.ubo.v1.InputAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputAction, 78));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInputAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 78, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInputAction = function() {\n  return this.setInputAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInputAction = function() {\n  return jspb.Message.getField(this, 78) != null;\n};\n\n\n/**\n * optional InputCancelAction input_cancel_action = 79;\n * @return {?proto.ubo.v1.InputCancelAction}\n */\nproto.ubo.v1.Action.prototype.getInputCancelAction = function() {\n  return /** @type{?proto.ubo.v1.InputCancelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputCancelAction, 79));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputCancelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInputCancelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 79, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInputCancelAction = function() {\n  return this.setInputCancelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInputCancelAction = function() {\n  return jspb.Message.getField(this, 79) != null;\n};\n\n\n/**\n * optional InputDemandAction input_demand_action = 80;\n * @return {?proto.ubo.v1.InputDemandAction}\n */\nproto.ubo.v1.Action.prototype.getInputDemandAction = function() {\n  return /** @type{?proto.ubo.v1.InputDemandAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputDemandAction, 80));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputDemandAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInputDemandAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 80, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInputDemandAction = function() {\n  return this.setInputDemandAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInputDemandAction = function() {\n  return jspb.Message.getField(this, 80) != null;\n};\n\n\n/**\n * optional InputProvideAction input_provide_action = 81;\n * @return {?proto.ubo.v1.InputProvideAction}\n */\nproto.ubo.v1.Action.prototype.getInputProvideAction = function() {\n  return /** @type{?proto.ubo.v1.InputProvideAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputProvideAction, 81));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputProvideAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInputProvideAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 81, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInputProvideAction = function() {\n  return this.setInputProvideAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInputProvideAction = function() {\n  return jspb.Message.getField(this, 81) != null;\n};\n\n\n/**\n * optional InputResolveAction input_resolve_action = 82;\n * @return {?proto.ubo.v1.InputResolveAction}\n */\nproto.ubo.v1.Action.prototype.getInputResolveAction = function() {\n  return /** @type{?proto.ubo.v1.InputResolveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResolveAction, 82));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputResolveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setInputResolveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 82, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearInputResolveAction = function() {\n  return this.setInputResolveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasInputResolveAction = function() {\n  return jspb.Message.getField(this, 82) != null;\n};\n\n\n/**\n * optional IpAction ip_action = 83;\n * @return {?proto.ubo.v1.IpAction}\n */\nproto.ubo.v1.Action.prototype.getIpAction = function() {\n  return /** @type{?proto.ubo.v1.IpAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.IpAction, 83));\n};\n\n\n/**\n * @param {?proto.ubo.v1.IpAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setIpAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 83, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearIpAction = function() {\n  return this.setIpAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasIpAction = function() {\n  return jspb.Message.getField(this, 83) != null;\n};\n\n\n/**\n * optional IpSetIsConnectedAction ip_set_is_connected_action = 84;\n * @return {?proto.ubo.v1.IpSetIsConnectedAction}\n */\nproto.ubo.v1.Action.prototype.getIpSetIsConnectedAction = function() {\n  return /** @type{?proto.ubo.v1.IpSetIsConnectedAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.IpSetIsConnectedAction, 84));\n};\n\n\n/**\n * @param {?proto.ubo.v1.IpSetIsConnectedAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setIpSetIsConnectedAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 84, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearIpSetIsConnectedAction = function() {\n  return this.setIpSetIsConnectedAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasIpSetIsConnectedAction = function() {\n  return jspb.Message.getField(this, 84) != null;\n};\n\n\n/**\n * optional IpUpdateInterfacesAction ip_update_interfaces_action = 85;\n * @return {?proto.ubo.v1.IpUpdateInterfacesAction}\n */\nproto.ubo.v1.Action.prototype.getIpUpdateInterfacesAction = function() {\n  return /** @type{?proto.ubo.v1.IpUpdateInterfacesAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.IpUpdateInterfacesAction, 85));\n};\n\n\n/**\n * @param {?proto.ubo.v1.IpUpdateInterfacesAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setIpUpdateInterfacesAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 85, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearIpUpdateInterfacesAction = function() {\n  return this.setIpUpdateInterfacesAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasIpUpdateInterfacesAction = function() {\n  return jspb.Message.getField(this, 85) != null;\n};\n\n\n/**\n * optional KeypadAction keypad_action = 86;\n * @return {?proto.ubo.v1.KeypadAction}\n */\nproto.ubo.v1.Action.prototype.getKeypadAction = function() {\n  return /** @type{?proto.ubo.v1.KeypadAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadAction, 86));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setKeypadAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 86, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearKeypadAction = function() {\n  return this.setKeypadAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasKeypadAction = function() {\n  return jspb.Message.getField(this, 86) != null;\n};\n\n\n/**\n * optional KeypadKeyHoldAction keypad_key_hold_action = 87;\n * @return {?proto.ubo.v1.KeypadKeyHoldAction}\n */\nproto.ubo.v1.Action.prototype.getKeypadKeyHoldAction = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyHoldAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyHoldAction, 87));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyHoldAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setKeypadKeyHoldAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 87, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearKeypadKeyHoldAction = function() {\n  return this.setKeypadKeyHoldAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasKeypadKeyHoldAction = function() {\n  return jspb.Message.getField(this, 87) != null;\n};\n\n\n/**\n * optional KeypadKeyPressAction keypad_key_press_action = 88;\n * @return {?proto.ubo.v1.KeypadKeyPressAction}\n */\nproto.ubo.v1.Action.prototype.getKeypadKeyPressAction = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyPressAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyPressAction, 88));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyPressAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setKeypadKeyPressAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 88, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearKeypadKeyPressAction = function() {\n  return this.setKeypadKeyPressAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasKeypadKeyPressAction = function() {\n  return jspb.Message.getField(this, 88) != null;\n};\n\n\n/**\n * optional KeypadKeyReleaseAction keypad_key_release_action = 89;\n * @return {?proto.ubo.v1.KeypadKeyReleaseAction}\n */\nproto.ubo.v1.Action.prototype.getKeypadKeyReleaseAction = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyReleaseAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyReleaseAction, 89));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyReleaseAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setKeypadKeyReleaseAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 89, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearKeypadKeyReleaseAction = function() {\n  return this.setKeypadKeyReleaseAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasKeypadKeyReleaseAction = function() {\n  return jspb.Message.getField(this, 89) != null;\n};\n\n\n/**\n * optional KeypadKeyUnholdAction keypad_key_unhold_action = 90;\n * @return {?proto.ubo.v1.KeypadKeyUnholdAction}\n */\nproto.ubo.v1.Action.prototype.getKeypadKeyUnholdAction = function() {\n  return /** @type{?proto.ubo.v1.KeypadKeyUnholdAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyUnholdAction, 90));\n};\n\n\n/**\n * @param {?proto.ubo.v1.KeypadKeyUnholdAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setKeypadKeyUnholdAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 90, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearKeypadKeyUnholdAction = function() {\n  return this.setKeypadKeyUnholdAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasKeypadKeyUnholdAction = function() {\n  return jspb.Message.getField(this, 90) != null;\n};\n\n\n/**\n * optional LightDMAction light_dm_action = 91;\n * @return {?proto.ubo.v1.LightDMAction}\n */\nproto.ubo.v1.Action.prototype.getLightDmAction = function() {\n  return /** @type{?proto.ubo.v1.LightDMAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.LightDMAction, 91));\n};\n\n\n/**\n * @param {?proto.ubo.v1.LightDMAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setLightDmAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 91, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearLightDmAction = function() {\n  return this.setLightDmAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasLightDmAction = function() {\n  return jspb.Message.getField(this, 91) != null;\n};\n\n\n/**\n * optional LightDMClearEnabledStateAction light_dm_clear_enabled_state_action = 92;\n * @return {?proto.ubo.v1.LightDMClearEnabledStateAction}\n */\nproto.ubo.v1.Action.prototype.getLightDmClearEnabledStateAction = function() {\n  return /** @type{?proto.ubo.v1.LightDMClearEnabledStateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.LightDMClearEnabledStateAction, 92));\n};\n\n\n/**\n * @param {?proto.ubo.v1.LightDMClearEnabledStateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setLightDmClearEnabledStateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 92, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearLightDmClearEnabledStateAction = function() {\n  return this.setLightDmClearEnabledStateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasLightDmClearEnabledStateAction = function() {\n  return jspb.Message.getField(this, 92) != null;\n};\n\n\n/**\n * optional LightDMUpdateStateAction light_dm_update_state_action = 93;\n * @return {?proto.ubo.v1.LightDMUpdateStateAction}\n */\nproto.ubo.v1.Action.prototype.getLightDmUpdateStateAction = function() {\n  return /** @type{?proto.ubo.v1.LightDMUpdateStateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.LightDMUpdateStateAction, 93));\n};\n\n\n/**\n * @param {?proto.ubo.v1.LightDMUpdateStateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setLightDmUpdateStateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 93, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearLightDmUpdateStateAction = function() {\n  return this.setLightDmUpdateStateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasLightDmUpdateStateAction = function() {\n  return jspb.Message.getField(this, 93) != null;\n};\n\n\n/**\n * optional MainAction main_action = 94;\n * @return {?proto.ubo.v1.MainAction}\n */\nproto.ubo.v1.Action.prototype.getMainAction = function() {\n  return /** @type{?proto.ubo.v1.MainAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MainAction, 94));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MainAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMainAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 94, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMainAction = function() {\n  return this.setMainAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMainAction = function() {\n  return jspb.Message.getField(this, 94) != null;\n};\n\n\n/**\n * optional MenuAction menu_action = 95;\n * @return {?proto.ubo.v1.MenuAction}\n */\nproto.ubo.v1.Action.prototype.getMenuAction = function() {\n  return /** @type{?proto.ubo.v1.MenuAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuAction, 95));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 95, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuAction = function() {\n  return this.setMenuAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuAction = function() {\n  return jspb.Message.getField(this, 95) != null;\n};\n\n\n/**\n * optional MenuChooseByIconAction menu_choose_by_icon_action = 96;\n * @return {?proto.ubo.v1.MenuChooseByIconAction}\n */\nproto.ubo.v1.Action.prototype.getMenuChooseByIconAction = function() {\n  return /** @type{?proto.ubo.v1.MenuChooseByIconAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIconAction, 96));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuChooseByIconAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuChooseByIconAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 96, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuChooseByIconAction = function() {\n  return this.setMenuChooseByIconAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuChooseByIconAction = function() {\n  return jspb.Message.getField(this, 96) != null;\n};\n\n\n/**\n * optional MenuChooseByIndexAction menu_choose_by_index_action = 97;\n * @return {?proto.ubo.v1.MenuChooseByIndexAction}\n */\nproto.ubo.v1.Action.prototype.getMenuChooseByIndexAction = function() {\n  return /** @type{?proto.ubo.v1.MenuChooseByIndexAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIndexAction, 97));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuChooseByIndexAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuChooseByIndexAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 97, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuChooseByIndexAction = function() {\n  return this.setMenuChooseByIndexAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuChooseByIndexAction = function() {\n  return jspb.Message.getField(this, 97) != null;\n};\n\n\n/**\n * optional MenuChooseByLabelAction menu_choose_by_label_action = 98;\n * @return {?proto.ubo.v1.MenuChooseByLabelAction}\n */\nproto.ubo.v1.Action.prototype.getMenuChooseByLabelAction = function() {\n  return /** @type{?proto.ubo.v1.MenuChooseByLabelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByLabelAction, 98));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuChooseByLabelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuChooseByLabelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 98, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuChooseByLabelAction = function() {\n  return this.setMenuChooseByLabelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuChooseByLabelAction = function() {\n  return jspb.Message.getField(this, 98) != null;\n};\n\n\n/**\n * optional MenuGoBackAction menu_go_back_action = 99;\n * @return {?proto.ubo.v1.MenuGoBackAction}\n */\nproto.ubo.v1.Action.prototype.getMenuGoBackAction = function() {\n  return /** @type{?proto.ubo.v1.MenuGoBackAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoBackAction, 99));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuGoBackAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuGoBackAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 99, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuGoBackAction = function() {\n  return this.setMenuGoBackAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuGoBackAction = function() {\n  return jspb.Message.getField(this, 99) != null;\n};\n\n\n/**\n * optional MenuGoHomeAction menu_go_home_action = 100;\n * @return {?proto.ubo.v1.MenuGoHomeAction}\n */\nproto.ubo.v1.Action.prototype.getMenuGoHomeAction = function() {\n  return /** @type{?proto.ubo.v1.MenuGoHomeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoHomeAction, 100));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuGoHomeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuGoHomeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 100, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuGoHomeAction = function() {\n  return this.setMenuGoHomeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuGoHomeAction = function() {\n  return jspb.Message.getField(this, 100) != null;\n};\n\n\n/**\n * optional MenuScrollAction menu_scroll_action = 101;\n * @return {?proto.ubo.v1.MenuScrollAction}\n */\nproto.ubo.v1.Action.prototype.getMenuScrollAction = function() {\n  return /** @type{?proto.ubo.v1.MenuScrollAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuScrollAction, 101));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuScrollAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setMenuScrollAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 101, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearMenuScrollAction = function() {\n  return this.setMenuScrollAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasMenuScrollAction = function() {\n  return jspb.Message.getField(this, 101) != null;\n};\n\n\n/**\n * optional NotificationsAction notifications_action = 102;\n * @return {?proto.ubo.v1.NotificationsAction}\n */\nproto.ubo.v1.Action.prototype.getNotificationsAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsAction, 102));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setNotificationsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 102, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearNotificationsAction = function() {\n  return this.setNotificationsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasNotificationsAction = function() {\n  return jspb.Message.getField(this, 102) != null;\n};\n\n\n/**\n * optional NotificationsAddAction notifications_add_action = 103;\n * @return {?proto.ubo.v1.NotificationsAddAction}\n */\nproto.ubo.v1.Action.prototype.getNotificationsAddAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationsAddAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsAddAction, 103));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsAddAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setNotificationsAddAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 103, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearNotificationsAddAction = function() {\n  return this.setNotificationsAddAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasNotificationsAddAction = function() {\n  return jspb.Message.getField(this, 103) != null;\n};\n\n\n/**\n * optional NotificationsClearAction notifications_clear_action = 104;\n * @return {?proto.ubo.v1.NotificationsClearAction}\n */\nproto.ubo.v1.Action.prototype.getNotificationsClearAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationsClearAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearAction, 104));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsClearAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setNotificationsClearAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 104, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearNotificationsClearAction = function() {\n  return this.setNotificationsClearAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasNotificationsClearAction = function() {\n  return jspb.Message.getField(this, 104) != null;\n};\n\n\n/**\n * optional NotificationsClearAllAction notifications_clear_all_action = 105;\n * @return {?proto.ubo.v1.NotificationsClearAllAction}\n */\nproto.ubo.v1.Action.prototype.getNotificationsClearAllAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationsClearAllAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearAllAction, 105));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsClearAllAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setNotificationsClearAllAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 105, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearNotificationsClearAllAction = function() {\n  return this.setNotificationsClearAllAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasNotificationsClearAllAction = function() {\n  return jspb.Message.getField(this, 105) != null;\n};\n\n\n/**\n * optional NotificationsClearByIdAction notifications_clear_by_id_action = 106;\n * @return {?proto.ubo.v1.NotificationsClearByIdAction}\n */\nproto.ubo.v1.Action.prototype.getNotificationsClearByIdAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationsClearByIdAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearByIdAction, 106));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsClearByIdAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setNotificationsClearByIdAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 106, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearNotificationsClearByIdAction = function() {\n  return this.setNotificationsClearByIdAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasNotificationsClearByIdAction = function() {\n  return jspb.Message.getField(this, 106) != null;\n};\n\n\n/**\n * optional NotificationsDisplayAction notifications_display_action = 107;\n * @return {?proto.ubo.v1.NotificationsDisplayAction}\n */\nproto.ubo.v1.Action.prototype.getNotificationsDisplayAction = function() {\n  return /** @type{?proto.ubo.v1.NotificationsDisplayAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsDisplayAction, 107));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsDisplayAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setNotificationsDisplayAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 107, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearNotificationsDisplayAction = function() {\n  return this.setNotificationsDisplayAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasNotificationsDisplayAction = function() {\n  return jspb.Message.getField(this, 107) != null;\n};\n\n\n/**\n * optional OpenApplicationAction open_application_action = 108;\n * @return {?proto.ubo.v1.OpenApplicationAction}\n */\nproto.ubo.v1.Action.prototype.getOpenApplicationAction = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction, 108));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setOpenApplicationAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 108, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearOpenApplicationAction = function() {\n  return this.setOpenApplicationAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasOpenApplicationAction = function() {\n  return jspb.Message.getField(this, 108) != null;\n};\n\n\n/**\n * optional PowerAction power_action = 109;\n * @return {?proto.ubo.v1.PowerAction}\n */\nproto.ubo.v1.Action.prototype.getPowerAction = function() {\n  return /** @type{?proto.ubo.v1.PowerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerAction, 109));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PowerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setPowerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 109, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearPowerAction = function() {\n  return this.setPowerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasPowerAction = function() {\n  return jspb.Message.getField(this, 109) != null;\n};\n\n\n/**\n * optional PowerOffAction power_off_action = 110;\n * @return {?proto.ubo.v1.PowerOffAction}\n */\nproto.ubo.v1.Action.prototype.getPowerOffAction = function() {\n  return /** @type{?proto.ubo.v1.PowerOffAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerOffAction, 110));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PowerOffAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setPowerOffAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 110, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearPowerOffAction = function() {\n  return this.setPowerOffAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasPowerOffAction = function() {\n  return jspb.Message.getField(this, 110) != null;\n};\n\n\n/**\n * optional RPiConnectAction r_pi_connect_action = 111;\n * @return {?proto.ubo.v1.RPiConnectAction}\n */\nproto.ubo.v1.Action.prototype.getRPiConnectAction = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectAction, 111));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRPiConnectAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 111, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRPiConnectAction = function() {\n  return this.setRPiConnectAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRPiConnectAction = function() {\n  return jspb.Message.getField(this, 111) != null;\n};\n\n\n/**\n * optional RPiConnectDoneDownloadingAction r_pi_connect_done_downloading_action = 112;\n * @return {?proto.ubo.v1.RPiConnectDoneDownloadingAction}\n */\nproto.ubo.v1.Action.prototype.getRPiConnectDoneDownloadingAction = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectDoneDownloadingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectDoneDownloadingAction, 112));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectDoneDownloadingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRPiConnectDoneDownloadingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 112, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRPiConnectDoneDownloadingAction = function() {\n  return this.setRPiConnectDoneDownloadingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRPiConnectDoneDownloadingAction = function() {\n  return jspb.Message.getField(this, 112) != null;\n};\n\n\n/**\n * optional RPiConnectSetPendingAction r_pi_connect_set_pending_action = 113;\n * @return {?proto.ubo.v1.RPiConnectSetPendingAction}\n */\nproto.ubo.v1.Action.prototype.getRPiConnectSetPendingAction = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectSetPendingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectSetPendingAction, 113));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectSetPendingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRPiConnectSetPendingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 113, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRPiConnectSetPendingAction = function() {\n  return this.setRPiConnectSetPendingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRPiConnectSetPendingAction = function() {\n  return jspb.Message.getField(this, 113) != null;\n};\n\n\n/**\n * optional RPiConnectSetStatusAction r_pi_connect_set_status_action = 114;\n * @return {?proto.ubo.v1.RPiConnectSetStatusAction}\n */\nproto.ubo.v1.Action.prototype.getRPiConnectSetStatusAction = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectSetStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectSetStatusAction, 114));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectSetStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRPiConnectSetStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 114, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRPiConnectSetStatusAction = function() {\n  return this.setRPiConnectSetStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRPiConnectSetStatusAction = function() {\n  return jspb.Message.getField(this, 114) != null;\n};\n\n\n/**\n * optional RPiConnectStartDownloadingAction r_pi_connect_start_downloading_action = 115;\n * @return {?proto.ubo.v1.RPiConnectStartDownloadingAction}\n */\nproto.ubo.v1.Action.prototype.getRPiConnectStartDownloadingAction = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectStartDownloadingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectStartDownloadingAction, 115));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectStartDownloadingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRPiConnectStartDownloadingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 115, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRPiConnectStartDownloadingAction = function() {\n  return this.setRPiConnectStartDownloadingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRPiConnectStartDownloadingAction = function() {\n  return jspb.Message.getField(this, 115) != null;\n};\n\n\n/**\n * optional RPiConnectUpdateServiceStateAction r_pi_connect_update_service_state_action = 116;\n * @return {?proto.ubo.v1.RPiConnectUpdateServiceStateAction}\n */\nproto.ubo.v1.Action.prototype.getRPiConnectUpdateServiceStateAction = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectUpdateServiceStateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectUpdateServiceStateAction, 116));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectUpdateServiceStateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRPiConnectUpdateServiceStateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 116, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRPiConnectUpdateServiceStateAction = function() {\n  return this.setRPiConnectUpdateServiceStateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRPiConnectUpdateServiceStateAction = function() {\n  return jspb.Message.getField(this, 116) != null;\n};\n\n\n/**\n * optional RebootAction reboot_action = 117;\n * @return {?proto.ubo.v1.RebootAction}\n */\nproto.ubo.v1.Action.prototype.getRebootAction = function() {\n  return /** @type{?proto.ubo.v1.RebootAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RebootAction, 117));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RebootAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRebootAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 117, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRebootAction = function() {\n  return this.setRebootAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRebootAction = function() {\n  return jspb.Message.getField(this, 117) != null;\n};\n\n\n/**\n * optional RegisterAppAction register_app_action = 118;\n * @return {?proto.ubo.v1.RegisterAppAction}\n */\nproto.ubo.v1.Action.prototype.getRegisterAppAction = function() {\n  return /** @type{?proto.ubo.v1.RegisterAppAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RegisterAppAction, 118));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RegisterAppAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRegisterAppAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 118, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRegisterAppAction = function() {\n  return this.setRegisterAppAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRegisterAppAction = function() {\n  return jspb.Message.getField(this, 118) != null;\n};\n\n\n/**\n * optional RegisterRegularAppAction register_regular_app_action = 119;\n * @return {?proto.ubo.v1.RegisterRegularAppAction}\n */\nproto.ubo.v1.Action.prototype.getRegisterRegularAppAction = function() {\n  return /** @type{?proto.ubo.v1.RegisterRegularAppAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RegisterRegularAppAction, 119));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RegisterRegularAppAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRegisterRegularAppAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 119, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRegisterRegularAppAction = function() {\n  return this.setRegisterRegularAppAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRegisterRegularAppAction = function() {\n  return jspb.Message.getField(this, 119) != null;\n};\n\n\n/**\n * optional RegisterSettingAppAction register_setting_app_action = 120;\n * @return {?proto.ubo.v1.RegisterSettingAppAction}\n */\nproto.ubo.v1.Action.prototype.getRegisterSettingAppAction = function() {\n  return /** @type{?proto.ubo.v1.RegisterSettingAppAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RegisterSettingAppAction, 120));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RegisterSettingAppAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRegisterSettingAppAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 120, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRegisterSettingAppAction = function() {\n  return this.setRegisterSettingAppAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRegisterSettingAppAction = function() {\n  return jspb.Message.getField(this, 120) != null;\n};\n\n\n/**\n * optional ReplayRecordedSequenceAction replay_recorded_sequence_action = 121;\n * @return {?proto.ubo.v1.ReplayRecordedSequenceAction}\n */\nproto.ubo.v1.Action.prototype.getReplayRecordedSequenceAction = function() {\n  return /** @type{?proto.ubo.v1.ReplayRecordedSequenceAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReplayRecordedSequenceAction, 121));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReplayRecordedSequenceAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setReplayRecordedSequenceAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 121, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearReplayRecordedSequenceAction = function() {\n  return this.setReplayRecordedSequenceAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasReplayRecordedSequenceAction = function() {\n  return jspb.Message.getField(this, 121) != null;\n};\n\n\n/**\n * optional ReportReplayingDoneAction report_replaying_done_action = 122;\n * @return {?proto.ubo.v1.ReportReplayingDoneAction}\n */\nproto.ubo.v1.Action.prototype.getReportReplayingDoneAction = function() {\n  return /** @type{?proto.ubo.v1.ReportReplayingDoneAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReportReplayingDoneAction, 122));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReportReplayingDoneAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setReportReplayingDoneAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 122, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearReportReplayingDoneAction = function() {\n  return this.setReportReplayingDoneAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasReportReplayingDoneAction = function() {\n  return jspb.Message.getField(this, 122) != null;\n};\n\n\n/**\n * optional RgbRingAction rgb_ring_action = 123;\n * @return {?proto.ubo.v1.RgbRingAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingAction, 123));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 123, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingAction = function() {\n  return this.setRgbRingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingAction = function() {\n  return jspb.Message.getField(this, 123) != null;\n};\n\n\n/**\n * optional RgbRingBlankAction rgb_ring_blank_action = 124;\n * @return {?proto.ubo.v1.RgbRingBlankAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingBlankAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingBlankAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingBlankAction, 124));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingBlankAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingBlankAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 124, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingBlankAction = function() {\n  return this.setRgbRingBlankAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingBlankAction = function() {\n  return jspb.Message.getField(this, 124) != null;\n};\n\n\n/**\n * optional RgbRingBlinkAction rgb_ring_blink_action = 125;\n * @return {?proto.ubo.v1.RgbRingBlinkAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingBlinkAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingBlinkAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingBlinkAction, 125));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingBlinkAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingBlinkAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 125, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingBlinkAction = function() {\n  return this.setRgbRingBlinkAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingBlinkAction = function() {\n  return jspb.Message.getField(this, 125) != null;\n};\n\n\n/**\n * optional RgbRingColorfulCommandAction rgb_ring_colorful_command_action = 126;\n * @return {?proto.ubo.v1.RgbRingColorfulCommandAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingColorfulCommandAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingColorfulCommandAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingColorfulCommandAction, 126));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingColorfulCommandAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingColorfulCommandAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 126, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingColorfulCommandAction = function() {\n  return this.setRgbRingColorfulCommandAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingColorfulCommandAction = function() {\n  return jspb.Message.getField(this, 126) != null;\n};\n\n\n/**\n * optional RgbRingCommandAction rgb_ring_command_action = 127;\n * @return {?proto.ubo.v1.RgbRingCommandAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingCommandAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingCommandAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingCommandAction, 127));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingCommandAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingCommandAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 127, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingCommandAction = function() {\n  return this.setRgbRingCommandAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingCommandAction = function() {\n  return jspb.Message.getField(this, 127) != null;\n};\n\n\n/**\n * optional RgbRingFillDownfromAction rgb_ring_fill_downfrom_action = 128;\n * @return {?proto.ubo.v1.RgbRingFillDownfromAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingFillDownfromAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingFillDownfromAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingFillDownfromAction, 128));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingFillDownfromAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingFillDownfromAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 128, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingFillDownfromAction = function() {\n  return this.setRgbRingFillDownfromAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingFillDownfromAction = function() {\n  return jspb.Message.getField(this, 128) != null;\n};\n\n\n/**\n * optional RgbRingFillUptoAction rgb_ring_fill_upto_action = 129;\n * @return {?proto.ubo.v1.RgbRingFillUptoAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingFillUptoAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingFillUptoAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingFillUptoAction, 129));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingFillUptoAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingFillUptoAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 129, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingFillUptoAction = function() {\n  return this.setRgbRingFillUptoAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingFillUptoAction = function() {\n  return jspb.Message.getField(this, 129) != null;\n};\n\n\n/**\n * optional RgbRingProgressWheelAction rgb_ring_progress_wheel_action = 130;\n * @return {?proto.ubo.v1.RgbRingProgressWheelAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingProgressWheelAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingProgressWheelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingProgressWheelAction, 130));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingProgressWheelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingProgressWheelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 130, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingProgressWheelAction = function() {\n  return this.setRgbRingProgressWheelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingProgressWheelAction = function() {\n  return jspb.Message.getField(this, 130) != null;\n};\n\n\n/**\n * optional RgbRingProgressWheelStepAction rgb_ring_progress_wheel_step_action = 131;\n * @return {?proto.ubo.v1.RgbRingProgressWheelStepAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingProgressWheelStepAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingProgressWheelStepAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingProgressWheelStepAction, 131));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingProgressWheelStepAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingProgressWheelStepAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 131, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingProgressWheelStepAction = function() {\n  return this.setRgbRingProgressWheelStepAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingProgressWheelStepAction = function() {\n  return jspb.Message.getField(this, 131) != null;\n};\n\n\n/**\n * optional RgbRingPulseAction rgb_ring_pulse_action = 132;\n * @return {?proto.ubo.v1.RgbRingPulseAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingPulseAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingPulseAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingPulseAction, 132));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingPulseAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingPulseAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 132, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingPulseAction = function() {\n  return this.setRgbRingPulseAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingPulseAction = function() {\n  return jspb.Message.getField(this, 132) != null;\n};\n\n\n/**\n * optional RgbRingRainbowAction rgb_ring_rainbow_action = 133;\n * @return {?proto.ubo.v1.RgbRingRainbowAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingRainbowAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingRainbowAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingRainbowAction, 133));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingRainbowAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingRainbowAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 133, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingRainbowAction = function() {\n  return this.setRgbRingRainbowAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingRainbowAction = function() {\n  return jspb.Message.getField(this, 133) != null;\n};\n\n\n/**\n * optional RgbRingSequenceAction rgb_ring_sequence_action = 134;\n * @return {?proto.ubo.v1.RgbRingSequenceAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingSequenceAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingSequenceAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSequenceAction, 134));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingSequenceAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingSequenceAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 134, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingSequenceAction = function() {\n  return this.setRgbRingSequenceAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingSequenceAction = function() {\n  return jspb.Message.getField(this, 134) != null;\n};\n\n\n/**\n * optional RgbRingSetAllAction rgb_ring_set_all_action = 135;\n * @return {?proto.ubo.v1.RgbRingSetAllAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingSetAllAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingSetAllAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetAllAction, 135));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingSetAllAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingSetAllAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 135, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingSetAllAction = function() {\n  return this.setRgbRingSetAllAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingSetAllAction = function() {\n  return jspb.Message.getField(this, 135) != null;\n};\n\n\n/**\n * optional RgbRingSetBrightnessAction rgb_ring_set_brightness_action = 136;\n * @return {?proto.ubo.v1.RgbRingSetBrightnessAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingSetBrightnessAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingSetBrightnessAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetBrightnessAction, 136));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingSetBrightnessAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingSetBrightnessAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 136, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingSetBrightnessAction = function() {\n  return this.setRgbRingSetBrightnessAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingSetBrightnessAction = function() {\n  return jspb.Message.getField(this, 136) != null;\n};\n\n\n/**\n * optional RgbRingSetEnabledAction rgb_ring_set_enabled_action = 137;\n * @return {?proto.ubo.v1.RgbRingSetEnabledAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingSetEnabledAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingSetEnabledAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetEnabledAction, 137));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingSetEnabledAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingSetEnabledAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 137, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingSetEnabledAction = function() {\n  return this.setRgbRingSetEnabledAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingSetEnabledAction = function() {\n  return jspb.Message.getField(this, 137) != null;\n};\n\n\n/**\n * optional RgbRingSetIsBusyAction rgb_ring_set_is_busy_action = 138;\n * @return {?proto.ubo.v1.RgbRingSetIsBusyAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingSetIsBusyAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingSetIsBusyAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetIsBusyAction, 138));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingSetIsBusyAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingSetIsBusyAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 138, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingSetIsBusyAction = function() {\n  return this.setRgbRingSetIsBusyAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingSetIsBusyAction = function() {\n  return jspb.Message.getField(this, 138) != null;\n};\n\n\n/**\n * optional RgbRingSpinningWheelAction rgb_ring_spinning_wheel_action = 139;\n * @return {?proto.ubo.v1.RgbRingSpinningWheelAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingSpinningWheelAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingSpinningWheelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSpinningWheelAction, 139));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingSpinningWheelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingSpinningWheelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 139, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingSpinningWheelAction = function() {\n  return this.setRgbRingSpinningWheelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingSpinningWheelAction = function() {\n  return jspb.Message.getField(this, 139) != null;\n};\n\n\n/**\n * optional RgbRingWaitableCommandAction rgb_ring_waitable_command_action = 140;\n * @return {?proto.ubo.v1.RgbRingWaitableCommandAction}\n */\nproto.ubo.v1.Action.prototype.getRgbRingWaitableCommandAction = function() {\n  return /** @type{?proto.ubo.v1.RgbRingWaitableCommandAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingWaitableCommandAction, 140));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingWaitableCommandAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setRgbRingWaitableCommandAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 140, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearRgbRingWaitableCommandAction = function() {\n  return this.setRgbRingWaitableCommandAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasRgbRingWaitableCommandAction = function() {\n  return jspb.Message.getField(this, 140) != null;\n};\n\n\n/**\n * optional SSHAction ssh_action = 141;\n * @return {?proto.ubo.v1.SSHAction}\n */\nproto.ubo.v1.Action.prototype.getSshAction = function() {\n  return /** @type{?proto.ubo.v1.SSHAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SSHAction, 141));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SSHAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSshAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 141, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSshAction = function() {\n  return this.setSshAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSshAction = function() {\n  return jspb.Message.getField(this, 141) != null;\n};\n\n\n/**\n * optional SSHClearEnabledStateAction ssh_clear_enabled_state_action = 142;\n * @return {?proto.ubo.v1.SSHClearEnabledStateAction}\n */\nproto.ubo.v1.Action.prototype.getSshClearEnabledStateAction = function() {\n  return /** @type{?proto.ubo.v1.SSHClearEnabledStateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SSHClearEnabledStateAction, 142));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SSHClearEnabledStateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSshClearEnabledStateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 142, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSshClearEnabledStateAction = function() {\n  return this.setSshClearEnabledStateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSshClearEnabledStateAction = function() {\n  return jspb.Message.getField(this, 142) != null;\n};\n\n\n/**\n * optional SSHUpdateStateAction ssh_update_state_action = 143;\n * @return {?proto.ubo.v1.SSHUpdateStateAction}\n */\nproto.ubo.v1.Action.prototype.getSshUpdateStateAction = function() {\n  return /** @type{?proto.ubo.v1.SSHUpdateStateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SSHUpdateStateAction, 143));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SSHUpdateStateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSshUpdateStateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 143, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSshUpdateStateAction = function() {\n  return this.setSshUpdateStateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSshUpdateStateAction = function() {\n  return jspb.Message.getField(this, 143) != null;\n};\n\n\n/**\n * optional SensorsAction sensors_action = 144;\n * @return {?proto.ubo.v1.SensorsAction}\n */\nproto.ubo.v1.Action.prototype.getSensorsAction = function() {\n  return /** @type{?proto.ubo.v1.SensorsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorsAction, 144));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SensorsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSensorsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 144, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSensorsAction = function() {\n  return this.setSensorsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSensorsAction = function() {\n  return jspb.Message.getField(this, 144) != null;\n};\n\n\n/**\n * optional SensorsReportReadingAction sensors_report_reading_action = 145;\n * @return {?proto.ubo.v1.SensorsReportReadingAction}\n */\nproto.ubo.v1.Action.prototype.getSensorsReportReadingAction = function() {\n  return /** @type{?proto.ubo.v1.SensorsReportReadingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorsReportReadingAction, 145));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SensorsReportReadingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSensorsReportReadingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 145, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSensorsReportReadingAction = function() {\n  return this.setSensorsReportReadingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSensorsReportReadingAction = function() {\n  return jspb.Message.getField(this, 145) != null;\n};\n\n\n/**\n * optional SetAreEnclosuresVisibleAction set_are_enclosures_visible_action = 146;\n * @return {?proto.ubo.v1.SetAreEnclosuresVisibleAction}\n */\nproto.ubo.v1.Action.prototype.getSetAreEnclosuresVisibleAction = function() {\n  return /** @type{?proto.ubo.v1.SetAreEnclosuresVisibleAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SetAreEnclosuresVisibleAction, 146));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SetAreEnclosuresVisibleAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSetAreEnclosuresVisibleAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 146, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSetAreEnclosuresVisibleAction = function() {\n  return this.setSetAreEnclosuresVisibleAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSetAreEnclosuresVisibleAction = function() {\n  return jspb.Message.getField(this, 146) != null;\n};\n\n\n/**\n * optional SetMenuPathAction set_menu_path_action = 147;\n * @return {?proto.ubo.v1.SetMenuPathAction}\n */\nproto.ubo.v1.Action.prototype.getSetMenuPathAction = function() {\n  return /** @type{?proto.ubo.v1.SetMenuPathAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SetMenuPathAction, 147));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SetMenuPathAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSetMenuPathAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 147, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSetMenuPathAction = function() {\n  return this.setSetMenuPathAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSetMenuPathAction = function() {\n  return jspb.Message.getField(this, 147) != null;\n};\n\n\n/**\n * optional SettingsAction settings_action = 148;\n * @return {?proto.ubo.v1.SettingsAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsAction, 148));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 148, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsAction = function() {\n  return this.setSettingsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsAction = function() {\n  return jspb.Message.getField(this, 148) != null;\n};\n\n\n/**\n * optional SettingsClearServiceErrorsAction settings_clear_service_errors_action = 149;\n * @return {?proto.ubo.v1.SettingsClearServiceErrorsAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsClearServiceErrorsAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsClearServiceErrorsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsClearServiceErrorsAction, 149));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsClearServiceErrorsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsClearServiceErrorsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 149, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsClearServiceErrorsAction = function() {\n  return this.setSettingsClearServiceErrorsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsClearServiceErrorsAction = function() {\n  return jspb.Message.getField(this, 149) != null;\n};\n\n\n/**\n * optional SettingsReportServiceErrorAction settings_report_service_error_action = 150;\n * @return {?proto.ubo.v1.SettingsReportServiceErrorAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsReportServiceErrorAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsReportServiceErrorAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsReportServiceErrorAction, 150));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsReportServiceErrorAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsReportServiceErrorAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 150, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsReportServiceErrorAction = function() {\n  return this.setSettingsReportServiceErrorAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsReportServiceErrorAction = function() {\n  return jspb.Message.getField(this, 150) != null;\n};\n\n\n/**\n * optional SettingsServiceAction settings_service_action = 151;\n * @return {?proto.ubo.v1.SettingsServiceAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsServiceAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsServiceAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceAction, 151));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsServiceAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsServiceAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 151, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsServiceAction = function() {\n  return this.setSettingsServiceAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsServiceAction = function() {\n  return jspb.Message.getField(this, 151) != null;\n};\n\n\n/**\n * optional SettingsServiceSetIsEnabledAction settings_service_set_is_enabled_action = 152;\n * @return {?proto.ubo.v1.SettingsServiceSetIsEnabledAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsServiceSetIsEnabledAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsServiceSetIsEnabledAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetIsEnabledAction, 152));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsServiceSetIsEnabledAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsServiceSetIsEnabledAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 152, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsServiceSetIsEnabledAction = function() {\n  return this.setSettingsServiceSetIsEnabledAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsServiceSetIsEnabledAction = function() {\n  return jspb.Message.getField(this, 152) != null;\n};\n\n\n/**\n * optional SettingsServiceSetLogLevelAction settings_service_set_log_level_action = 153;\n * @return {?proto.ubo.v1.SettingsServiceSetLogLevelAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsServiceSetLogLevelAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsServiceSetLogLevelAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetLogLevelAction, 153));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsServiceSetLogLevelAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsServiceSetLogLevelAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 153, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsServiceSetLogLevelAction = function() {\n  return this.setSettingsServiceSetLogLevelAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsServiceSetLogLevelAction = function() {\n  return jspb.Message.getField(this, 153) != null;\n};\n\n\n/**\n * optional SettingsServiceSetShouldRestartAction settings_service_set_should_restart_action = 154;\n * @return {?proto.ubo.v1.SettingsServiceSetShouldRestartAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsServiceSetShouldRestartAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsServiceSetShouldRestartAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetShouldRestartAction, 154));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsServiceSetShouldRestartAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsServiceSetShouldRestartAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 154, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsServiceSetShouldRestartAction = function() {\n  return this.setSettingsServiceSetShouldRestartAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsServiceSetShouldRestartAction = function() {\n  return jspb.Message.getField(this, 154) != null;\n};\n\n\n/**\n * optional SettingsServiceSetStatusAction settings_service_set_status_action = 155;\n * @return {?proto.ubo.v1.SettingsServiceSetStatusAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsServiceSetStatusAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsServiceSetStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetStatusAction, 155));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsServiceSetStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsServiceSetStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 155, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsServiceSetStatusAction = function() {\n  return this.setSettingsServiceSetStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsServiceSetStatusAction = function() {\n  return jspb.Message.getField(this, 155) != null;\n};\n\n\n/**\n * optional SettingsSetServicesAction settings_set_services_action = 156;\n * @return {?proto.ubo.v1.SettingsSetServicesAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsSetServicesAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsSetServicesAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsSetServicesAction, 156));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsSetServicesAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsSetServicesAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 156, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsSetServicesAction = function() {\n  return this.setSettingsSetServicesAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsSetServicesAction = function() {\n  return jspb.Message.getField(this, 156) != null;\n};\n\n\n/**\n * optional SettingsStartServiceAction settings_start_service_action = 157;\n * @return {?proto.ubo.v1.SettingsStartServiceAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsStartServiceAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsStartServiceAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStartServiceAction, 157));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsStartServiceAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsStartServiceAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 157, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsStartServiceAction = function() {\n  return this.setSettingsStartServiceAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsStartServiceAction = function() {\n  return jspb.Message.getField(this, 157) != null;\n};\n\n\n/**\n * optional SettingsStopServiceAction settings_stop_service_action = 158;\n * @return {?proto.ubo.v1.SettingsStopServiceAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsStopServiceAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsStopServiceAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStopServiceAction, 158));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsStopServiceAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsStopServiceAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 158, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsStopServiceAction = function() {\n  return this.setSettingsStopServiceAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsStopServiceAction = function() {\n  return jspb.Message.getField(this, 158) != null;\n};\n\n\n/**\n * optional SettingsToggleBetaVersionsAction settings_toggle_beta_versions_action = 159;\n * @return {?proto.ubo.v1.SettingsToggleBetaVersionsAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsToggleBetaVersionsAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsToggleBetaVersionsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsToggleBetaVersionsAction, 159));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsToggleBetaVersionsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsToggleBetaVersionsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 159, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsToggleBetaVersionsAction = function() {\n  return this.setSettingsToggleBetaVersionsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsToggleBetaVersionsAction = function() {\n  return jspb.Message.getField(this, 159) != null;\n};\n\n\n/**\n * optional SettingsTogglePdbSignalAction settings_toggle_pdb_signal_action = 160;\n * @return {?proto.ubo.v1.SettingsTogglePdbSignalAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsTogglePdbSignalAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsTogglePdbSignalAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsTogglePdbSignalAction, 160));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsTogglePdbSignalAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsTogglePdbSignalAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 160, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsTogglePdbSignalAction = function() {\n  return this.setSettingsTogglePdbSignalAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsTogglePdbSignalAction = function() {\n  return jspb.Message.getField(this, 160) != null;\n};\n\n\n/**\n * optional SettingsToggleVisualDebugAction settings_toggle_visual_debug_action = 161;\n * @return {?proto.ubo.v1.SettingsToggleVisualDebugAction}\n */\nproto.ubo.v1.Action.prototype.getSettingsToggleVisualDebugAction = function() {\n  return /** @type{?proto.ubo.v1.SettingsToggleVisualDebugAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsToggleVisualDebugAction, 161));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsToggleVisualDebugAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSettingsToggleVisualDebugAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 161, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSettingsToggleVisualDebugAction = function() {\n  return this.setSettingsToggleVisualDebugAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSettingsToggleVisualDebugAction = function() {\n  return jspb.Message.getField(this, 161) != null;\n};\n\n\n/**\n * optional SpeechRecognitionAction speech_recognition_action = 162;\n * @return {?proto.ubo.v1.SpeechRecognitionAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionAction, 162));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 162, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionAction = function() {\n  return this.setSpeechRecognitionAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionAction = function() {\n  return jspb.Message.getField(this, 162) != null;\n};\n\n\n/**\n * optional SpeechRecognitionReportIntentDetectionAction speech_recognition_report_intent_detection_action = 163;\n * @return {?proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionReportIntentDetectionAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction, 163));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionReportIntentDetectionAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 163, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionReportIntentDetectionAction = function() {\n  return this.setSpeechRecognitionReportIntentDetectionAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionReportIntentDetectionAction = function() {\n  return jspb.Message.getField(this, 163) != null;\n};\n\n\n/**\n * optional SpeechRecognitionReportSpeechAction speech_recognition_report_speech_action = 164;\n * @return {?proto.ubo.v1.SpeechRecognitionReportSpeechAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionReportSpeechAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionReportSpeechAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportSpeechAction, 164));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionReportSpeechAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionReportSpeechAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 164, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionReportSpeechAction = function() {\n  return this.setSpeechRecognitionReportSpeechAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionReportSpeechAction = function() {\n  return jspb.Message.getField(this, 164) != null;\n};\n\n\n/**\n * optional SpeechRecognitionReportWakeWordDetectionAction speech_recognition_report_wake_word_detection_action = 165;\n * @return {?proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionReportWakeWordDetectionAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction, 165));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionReportWakeWordDetectionAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 165, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionReportWakeWordDetectionAction = function() {\n  return this.setSpeechRecognitionReportWakeWordDetectionAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionReportWakeWordDetectionAction = function() {\n  return jspb.Message.getField(this, 165) != null;\n};\n\n\n/**\n * optional SpeechRecognitionSetIsAssistantActiveAction speech_recognition_set_is_assistant_active_action = 166;\n * @return {?proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionSetIsAssistantActiveAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction, 166));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionSetIsAssistantActiveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 166, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionSetIsAssistantActiveAction = function() {\n  return this.setSpeechRecognitionSetIsAssistantActiveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionSetIsAssistantActiveAction = function() {\n  return jspb.Message.getField(this, 166) != null;\n};\n\n\n/**\n * optional SpeechRecognitionSetIsIntentsActiveAction speech_recognition_set_is_intents_active_action = 167;\n * @return {?proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionSetIsIntentsActiveAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction, 167));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionSetIsIntentsActiveAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 167, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionSetIsIntentsActiveAction = function() {\n  return this.setSpeechRecognitionSetIsIntentsActiveAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionSetIsIntentsActiveAction = function() {\n  return jspb.Message.getField(this, 167) != null;\n};\n\n\n/**\n * optional SpeechRecognitionSetSelectedEngineAction speech_recognition_set_selected_engine_action = 168;\n * @return {?proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechRecognitionSetSelectedEngineAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction, 168));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechRecognitionSetSelectedEngineAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 168, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechRecognitionSetSelectedEngineAction = function() {\n  return this.setSpeechRecognitionSetSelectedEngineAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechRecognitionSetSelectedEngineAction = function() {\n  return jspb.Message.getField(this, 168) != null;\n};\n\n\n/**\n * optional SpeechSynthesisAction speech_synthesis_action = 169;\n * @return {?proto.ubo.v1.SpeechSynthesisAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechSynthesisAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechSynthesisAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisAction, 169));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechSynthesisAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechSynthesisAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 169, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechSynthesisAction = function() {\n  return this.setSpeechSynthesisAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechSynthesisAction = function() {\n  return jspb.Message.getField(this, 169) != null;\n};\n\n\n/**\n * optional SpeechSynthesisReadTextAction speech_synthesis_read_text_action = 170;\n * @return {?proto.ubo.v1.SpeechSynthesisReadTextAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechSynthesisReadTextAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechSynthesisReadTextAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisReadTextAction, 170));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechSynthesisReadTextAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechSynthesisReadTextAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 170, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechSynthesisReadTextAction = function() {\n  return this.setSpeechSynthesisReadTextAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechSynthesisReadTextAction = function() {\n  return jspb.Message.getField(this, 170) != null;\n};\n\n\n/**\n * optional SpeechSynthesisSetSelectedEngineAction speech_synthesis_set_selected_engine_action = 171;\n * @return {?proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction}\n */\nproto.ubo.v1.Action.prototype.getSpeechSynthesisSetSelectedEngineAction = function() {\n  return /** @type{?proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction, 171));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setSpeechSynthesisSetSelectedEngineAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 171, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearSpeechSynthesisSetSelectedEngineAction = function() {\n  return this.setSpeechSynthesisSetSelectedEngineAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasSpeechSynthesisSetSelectedEngineAction = function() {\n  return jspb.Message.getField(this, 171) != null;\n};\n\n\n/**\n * optional StatusIconsAction status_icons_action = 172;\n * @return {?proto.ubo.v1.StatusIconsAction}\n */\nproto.ubo.v1.Action.prototype.getStatusIconsAction = function() {\n  return /** @type{?proto.ubo.v1.StatusIconsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.StatusIconsAction, 172));\n};\n\n\n/**\n * @param {?proto.ubo.v1.StatusIconsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setStatusIconsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 172, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearStatusIconsAction = function() {\n  return this.setStatusIconsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasStatusIconsAction = function() {\n  return jspb.Message.getField(this, 172) != null;\n};\n\n\n/**\n * optional StatusIconsRegisterAction status_icons_register_action = 173;\n * @return {?proto.ubo.v1.StatusIconsRegisterAction}\n */\nproto.ubo.v1.Action.prototype.getStatusIconsRegisterAction = function() {\n  return /** @type{?proto.ubo.v1.StatusIconsRegisterAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.StatusIconsRegisterAction, 173));\n};\n\n\n/**\n * @param {?proto.ubo.v1.StatusIconsRegisterAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setStatusIconsRegisterAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 173, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearStatusIconsRegisterAction = function() {\n  return this.setStatusIconsRegisterAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasStatusIconsRegisterAction = function() {\n  return jspb.Message.getField(this, 173) != null;\n};\n\n\n/**\n * optional ToggleRecordingAction toggle_recording_action = 174;\n * @return {?proto.ubo.v1.ToggleRecordingAction}\n */\nproto.ubo.v1.Action.prototype.getToggleRecordingAction = function() {\n  return /** @type{?proto.ubo.v1.ToggleRecordingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ToggleRecordingAction, 174));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ToggleRecordingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setToggleRecordingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 174, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearToggleRecordingAction = function() {\n  return this.setToggleRecordingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasToggleRecordingAction = function() {\n  return jspb.Message.getField(this, 174) != null;\n};\n\n\n/**\n * optional UpdateManagerAction update_manager_action = 175;\n * @return {?proto.ubo.v1.UpdateManagerAction}\n */\nproto.ubo.v1.Action.prototype.getUpdateManagerAction = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerAction, 175));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUpdateManagerAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 175, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUpdateManagerAction = function() {\n  return this.setUpdateManagerAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUpdateManagerAction = function() {\n  return jspb.Message.getField(this, 175) != null;\n};\n\n\n/**\n * optional UpdateManagerReportFailedCheckAction update_manager_report_failed_check_action = 176;\n * @return {?proto.ubo.v1.UpdateManagerReportFailedCheckAction}\n */\nproto.ubo.v1.Action.prototype.getUpdateManagerReportFailedCheckAction = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerReportFailedCheckAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerReportFailedCheckAction, 176));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerReportFailedCheckAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUpdateManagerReportFailedCheckAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 176, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUpdateManagerReportFailedCheckAction = function() {\n  return this.setUpdateManagerReportFailedCheckAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUpdateManagerReportFailedCheckAction = function() {\n  return jspb.Message.getField(this, 176) != null;\n};\n\n\n/**\n * optional UpdateManagerRequestCheckAction update_manager_request_check_action = 177;\n * @return {?proto.ubo.v1.UpdateManagerRequestCheckAction}\n */\nproto.ubo.v1.Action.prototype.getUpdateManagerRequestCheckAction = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerRequestCheckAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerRequestCheckAction, 177));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerRequestCheckAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUpdateManagerRequestCheckAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 177, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUpdateManagerRequestCheckAction = function() {\n  return this.setUpdateManagerRequestCheckAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUpdateManagerRequestCheckAction = function() {\n  return jspb.Message.getField(this, 177) != null;\n};\n\n\n/**\n * optional UpdateManagerRequestUpdateAction update_manager_request_update_action = 178;\n * @return {?proto.ubo.v1.UpdateManagerRequestUpdateAction}\n */\nproto.ubo.v1.Action.prototype.getUpdateManagerRequestUpdateAction = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerRequestUpdateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerRequestUpdateAction, 178));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerRequestUpdateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUpdateManagerRequestUpdateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 178, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUpdateManagerRequestUpdateAction = function() {\n  return this.setUpdateManagerRequestUpdateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUpdateManagerRequestUpdateAction = function() {\n  return jspb.Message.getField(this, 178) != null;\n};\n\n\n/**\n * optional UpdateManagerSetVersionsAction update_manager_set_versions_action = 179;\n * @return {?proto.ubo.v1.UpdateManagerSetVersionsAction}\n */\nproto.ubo.v1.Action.prototype.getUpdateManagerSetVersionsAction = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerSetVersionsAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerSetVersionsAction, 179));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerSetVersionsAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUpdateManagerSetVersionsAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 179, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUpdateManagerSetVersionsAction = function() {\n  return this.setUpdateManagerSetVersionsAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUpdateManagerSetVersionsAction = function() {\n  return jspb.Message.getField(this, 179) != null;\n};\n\n\n/**\n * optional UsersAction users_action = 180;\n * @return {?proto.ubo.v1.UsersAction}\n */\nproto.ubo.v1.Action.prototype.getUsersAction = function() {\n  return /** @type{?proto.ubo.v1.UsersAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersAction, 180));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUsersAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 180, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUsersAction = function() {\n  return this.setUsersAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUsersAction = function() {\n  return jspb.Message.getField(this, 180) != null;\n};\n\n\n/**\n * optional UsersCreateUserAction users_create_user_action = 181;\n * @return {?proto.ubo.v1.UsersCreateUserAction}\n */\nproto.ubo.v1.Action.prototype.getUsersCreateUserAction = function() {\n  return /** @type{?proto.ubo.v1.UsersCreateUserAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersCreateUserAction, 181));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersCreateUserAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUsersCreateUserAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 181, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUsersCreateUserAction = function() {\n  return this.setUsersCreateUserAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUsersCreateUserAction = function() {\n  return jspb.Message.getField(this, 181) != null;\n};\n\n\n/**\n * optional UsersDeleteUserAction users_delete_user_action = 182;\n * @return {?proto.ubo.v1.UsersDeleteUserAction}\n */\nproto.ubo.v1.Action.prototype.getUsersDeleteUserAction = function() {\n  return /** @type{?proto.ubo.v1.UsersDeleteUserAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersDeleteUserAction, 182));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersDeleteUserAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUsersDeleteUserAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 182, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUsersDeleteUserAction = function() {\n  return this.setUsersDeleteUserAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUsersDeleteUserAction = function() {\n  return jspb.Message.getField(this, 182) != null;\n};\n\n\n/**\n * optional UsersResetPasswordAction users_reset_password_action = 183;\n * @return {?proto.ubo.v1.UsersResetPasswordAction}\n */\nproto.ubo.v1.Action.prototype.getUsersResetPasswordAction = function() {\n  return /** @type{?proto.ubo.v1.UsersResetPasswordAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersResetPasswordAction, 183));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersResetPasswordAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUsersResetPasswordAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 183, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUsersResetPasswordAction = function() {\n  return this.setUsersResetPasswordAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUsersResetPasswordAction = function() {\n  return jspb.Message.getField(this, 183) != null;\n};\n\n\n/**\n * optional UsersSetUsersAction users_set_users_action = 184;\n * @return {?proto.ubo.v1.UsersSetUsersAction}\n */\nproto.ubo.v1.Action.prototype.getUsersSetUsersAction = function() {\n  return /** @type{?proto.ubo.v1.UsersSetUsersAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersSetUsersAction, 184));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersSetUsersAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setUsersSetUsersAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 184, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearUsersSetUsersAction = function() {\n  return this.setUsersSetUsersAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasUsersSetUsersAction = function() {\n  return jspb.Message.getField(this, 184) != null;\n};\n\n\n/**\n * optional VSCodeAction vs_code_action = 185;\n * @return {?proto.ubo.v1.VSCodeAction}\n */\nproto.ubo.v1.Action.prototype.getVsCodeAction = function() {\n  return /** @type{?proto.ubo.v1.VSCodeAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeAction, 185));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setVsCodeAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 185, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearVsCodeAction = function() {\n  return this.setVsCodeAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasVsCodeAction = function() {\n  return jspb.Message.getField(this, 185) != null;\n};\n\n\n/**\n * optional VSCodeDoneDownloadingAction vs_code_done_downloading_action = 186;\n * @return {?proto.ubo.v1.VSCodeDoneDownloadingAction}\n */\nproto.ubo.v1.Action.prototype.getVsCodeDoneDownloadingAction = function() {\n  return /** @type{?proto.ubo.v1.VSCodeDoneDownloadingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeDoneDownloadingAction, 186));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeDoneDownloadingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setVsCodeDoneDownloadingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 186, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearVsCodeDoneDownloadingAction = function() {\n  return this.setVsCodeDoneDownloadingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasVsCodeDoneDownloadingAction = function() {\n  return jspb.Message.getField(this, 186) != null;\n};\n\n\n/**\n * optional VSCodeSetPendingAction vs_code_set_pending_action = 187;\n * @return {?proto.ubo.v1.VSCodeSetPendingAction}\n */\nproto.ubo.v1.Action.prototype.getVsCodeSetPendingAction = function() {\n  return /** @type{?proto.ubo.v1.VSCodeSetPendingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeSetPendingAction, 187));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeSetPendingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setVsCodeSetPendingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 187, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearVsCodeSetPendingAction = function() {\n  return this.setVsCodeSetPendingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasVsCodeSetPendingAction = function() {\n  return jspb.Message.getField(this, 187) != null;\n};\n\n\n/**\n * optional VSCodeSetStatusAction vs_code_set_status_action = 188;\n * @return {?proto.ubo.v1.VSCodeSetStatusAction}\n */\nproto.ubo.v1.Action.prototype.getVsCodeSetStatusAction = function() {\n  return /** @type{?proto.ubo.v1.VSCodeSetStatusAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeSetStatusAction, 188));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeSetStatusAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setVsCodeSetStatusAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 188, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearVsCodeSetStatusAction = function() {\n  return this.setVsCodeSetStatusAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasVsCodeSetStatusAction = function() {\n  return jspb.Message.getField(this, 188) != null;\n};\n\n\n/**\n * optional VSCodeStartDownloadingAction vs_code_start_downloading_action = 189;\n * @return {?proto.ubo.v1.VSCodeStartDownloadingAction}\n */\nproto.ubo.v1.Action.prototype.getVsCodeStartDownloadingAction = function() {\n  return /** @type{?proto.ubo.v1.VSCodeStartDownloadingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeStartDownloadingAction, 189));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeStartDownloadingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setVsCodeStartDownloadingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 189, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearVsCodeStartDownloadingAction = function() {\n  return this.setVsCodeStartDownloadingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasVsCodeStartDownloadingAction = function() {\n  return jspb.Message.getField(this, 189) != null;\n};\n\n\n/**\n * optional WiFiAction wi_fi_action = 190;\n * @return {?proto.ubo.v1.WiFiAction}\n */\nproto.ubo.v1.Action.prototype.getWiFiAction = function() {\n  return /** @type{?proto.ubo.v1.WiFiAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiAction, 190));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setWiFiAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 190, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearWiFiAction = function() {\n  return this.setWiFiAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasWiFiAction = function() {\n  return jspb.Message.getField(this, 190) != null;\n};\n\n\n/**\n * optional WiFiInputConnectionAction wi_fi_input_connection_action = 191;\n * @return {?proto.ubo.v1.WiFiInputConnectionAction}\n */\nproto.ubo.v1.Action.prototype.getWiFiInputConnectionAction = function() {\n  return /** @type{?proto.ubo.v1.WiFiInputConnectionAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiInputConnectionAction, 191));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiInputConnectionAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setWiFiInputConnectionAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 191, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearWiFiInputConnectionAction = function() {\n  return this.setWiFiInputConnectionAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasWiFiInputConnectionAction = function() {\n  return jspb.Message.getField(this, 191) != null;\n};\n\n\n/**\n * optional WiFiSetHasVisitedOnboardingAction wi_fi_set_has_visited_onboarding_action = 192;\n * @return {?proto.ubo.v1.WiFiSetHasVisitedOnboardingAction}\n */\nproto.ubo.v1.Action.prototype.getWiFiSetHasVisitedOnboardingAction = function() {\n  return /** @type{?proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiSetHasVisitedOnboardingAction, 192));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiSetHasVisitedOnboardingAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setWiFiSetHasVisitedOnboardingAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 192, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearWiFiSetHasVisitedOnboardingAction = function() {\n  return this.setWiFiSetHasVisitedOnboardingAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasWiFiSetHasVisitedOnboardingAction = function() {\n  return jspb.Message.getField(this, 192) != null;\n};\n\n\n/**\n * optional WiFiUpdateAction wi_fi_update_action = 193;\n * @return {?proto.ubo.v1.WiFiUpdateAction}\n */\nproto.ubo.v1.Action.prototype.getWiFiUpdateAction = function() {\n  return /** @type{?proto.ubo.v1.WiFiUpdateAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiUpdateAction, 193));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiUpdateAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setWiFiUpdateAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 193, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearWiFiUpdateAction = function() {\n  return this.setWiFiUpdateAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasWiFiUpdateAction = function() {\n  return jspb.Message.getField(this, 193) != null;\n};\n\n\n/**\n * optional WiFiUpdateRequestAction wi_fi_update_request_action = 194;\n * @return {?proto.ubo.v1.WiFiUpdateRequestAction}\n */\nproto.ubo.v1.Action.prototype.getWiFiUpdateRequestAction = function() {\n  return /** @type{?proto.ubo.v1.WiFiUpdateRequestAction} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiUpdateRequestAction, 194));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiUpdateRequestAction|undefined} value\n * @return {!proto.ubo.v1.Action} returns this\n*/\nproto.ubo.v1.Action.prototype.setWiFiUpdateRequestAction = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 194, proto.ubo.v1.Action.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Action} returns this\n */\nproto.ubo.v1.Action.prototype.clearWiFiUpdateRequestAction = function() {\n  return this.setWiFiUpdateRequestAction(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Action.prototype.hasWiFiUpdateRequestAction = function() {\n  return jspb.Message.getField(this, 194) != null;\n};\n\n\n\n/**\n * Oneof group definitions for this message. Each group defines the field\n * numbers belonging to that group. When of these fields' value is set, all\n * other fields in the group are cleared. During deserialization, if multiple\n * fields are encountered for a group, only the last value seen will be kept.\n * @private {!Array<!Array<number>>}\n * @const\n */\nproto.ubo.v1.Event.oneofGroups_ = [[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104]];\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Event.EventCase = {\n  EVENT_NOT_SET: 0,\n  ASSISTANT_ADD_MCP_SERVER_EVENT: 1,\n  ASSISTANT_DELETE_MCP_SERVER_EVENT: 2,\n  ASSISTANT_DOWNLOAD_OLLAMA_MODEL_EVENT: 3,\n  ASSISTANT_EVENT: 4,\n  ASSISTANT_HANDLE_REPORT_EVENT: 5,\n  ASSISTANT_UPDATE_PROVIDERS_EVENT: 6,\n  AUDIO_EVENT: 7,\n  AUDIO_INSTALL_DRIVER_EVENT: 8,\n  AUDIO_PLAY_AUDIO_SAMPLE_EVENT: 9,\n  AUDIO_PLAY_AUDIO_SEQUENCE_EVENT: 10,\n  AUDIO_PLAY_CHIME_EVENT: 11,\n  AUDIO_PLAYBACK_DONE_EVENT: 12,\n  AUDIO_REPORT_SAMPLE_EVENT: 13,\n  CAMERA_DETECT_EVENT: 14,\n  CAMERA_DETECTED_EVENT: 15,\n  CAMERA_EVENT: 16,\n  CAMERA_INSTALL_DRIVER_EVENT: 17,\n  CAMERA_REINITIALIZE_EVENT: 18,\n  CAMERA_REPORT_IMAGE_EVENT: 19,\n  CAMERA_RESTORE_DEFAULT_EVENT: 20,\n  CAMERA_START_VIEWFINDER_EVENT: 21,\n  CAMERA_STOP_VIEWFINDER_EVENT: 22,\n  CLOSE_APPLICATION_EVENT: 23,\n  DISPLAY_BLANK_EVENT: 24,\n  DISPLAY_COMPRESSED_RENDER_EVENT: 25,\n  DISPLAY_EVENT: 26,\n  DISPLAY_REDRAW_EVENT: 27,\n  DISPLAY_RENDER_EVENT: 28,\n  DISPLAY_UNBLANK_EVENT: 29,\n  DOCKER_EVENT: 30,\n  DOCKER_IMAGE_EVENT: 31,\n  DOCKER_IMAGE_FETCH_COMPOSITION_EVENT: 32,\n  DOCKER_IMAGE_FETCH_EVENT: 33,\n  DOCKER_IMAGE_REGISTER_APP_EVENT: 34,\n  DOCKER_IMAGE_RELEASE_COMPOSITION_EVENT: 35,\n  DOCKER_IMAGE_REMOVE_COMPOSITION_EVENT: 36,\n  DOCKER_IMAGE_REMOVE_CONTAINER_EVENT: 37,\n  DOCKER_IMAGE_REMOVE_EVENT: 38,\n  DOCKER_IMAGE_RUN_COMPOSITION_EVENT: 39,\n  DOCKER_IMAGE_RUN_CONTAINER_EVENT: 40,\n  DOCKER_IMAGE_STOP_COMPOSITION_EVENT: 41,\n  DOCKER_IMAGE_STOP_CONTAINER_EVENT: 42,\n  DOCKER_INSTALL_EVENT: 43,\n  DOCKER_START_EVENT: 44,\n  DOCKER_STOP_EVENT: 45,\n  FILE_SYSTEM_COPY_EVENT: 46,\n  FILE_SYSTEM_EVENT: 47,\n  FILE_SYSTEM_MOVE_EVENT: 48,\n  FILE_SYSTEM_REMOVE_EVENT: 49,\n  FILE_SYSTEM_SELECT_EVENT: 50,\n  INFRARED_EVENT: 51,\n  INFRARED_SEND_CODE_EVENT: 52,\n  INIT_EVENT: 53,\n  INPUT_CANCEL_EVENT: 54,\n  INPUT_PROVIDE_EVENT: 55,\n  INPUT_RESOLVE_EVENT: 56,\n  IP_EVENT: 57,\n  MAIN_EVENT: 58,\n  MENU_CHOOSE_BY_ICON_EVENT: 59,\n  MENU_CHOOSE_BY_INDEX_EVENT: 60,\n  MENU_CHOOSE_BY_LABEL_EVENT: 61,\n  MENU_EVENT: 62,\n  MENU_GO_BACK_EVENT: 63,\n  MENU_GO_HOME_EVENT: 64,\n  MENU_SCROLL_EVENT: 65,\n  NOTIFICATIONS_CLEAR_EVENT: 66,\n  NOTIFICATIONS_DISPLAY_EVENT: 67,\n  NOTIFICATIONS_EVENT: 68,\n  OPEN_APPLICATION_EVENT: 69,\n  POWER_EVENT: 70,\n  POWER_OFF_EVENT: 71,\n  R_PI_CONNECT_EVENT: 72,\n  R_PI_CONNECT_LOGIN_EVENT: 73,\n  REBOOT_EVENT: 74,\n  REPLAY_RECORDED_SEQUENCE_EVENT: 75,\n  RGB_RING_COMMAND_EVENT: 76,\n  RGB_RING_EVENT: 77,\n  SCREENSHOT_EVENT: 78,\n  SETTINGS_EVENT: 79,\n  SETTINGS_SERVICE_EVENT: 80,\n  SETTINGS_START_SERVICE_EVENT: 81,\n  SETTINGS_STOP_SERVICE_EVENT: 82,\n  SNAPSHOT_EVENT: 83,\n  SPEECH_RECOGNITION_EVENT: 84,\n  SPEECH_RECOGNITION_REPORT_TEXT_EVENT: 85,\n  SPEECH_SYNTHESIS_EVENT: 86,\n  SPEECH_SYNTHESIS_SYNTHESIZE_TEXT_EVENT: 87,\n  STORE_RECORDED_SEQUENCE_EVENT: 88,\n  UPDATE_MANAGER_CHECK_EVENT: 89,\n  UPDATE_MANAGER_EVENT: 90,\n  UPDATE_MANAGER_UPDATE_EVENT: 91,\n  USERS_CREATE_USER_EVENT: 92,\n  USERS_DELETE_USER_EVENT: 93,\n  USERS_EVENT: 94,\n  USERS_RESET_PASSWORD_EVENT: 95,\n  VS_CODE_EVENT: 96,\n  VS_CODE_LOGIN_EVENT: 97,\n  VS_CODE_RESTART_EVENT: 98,\n  WEB_UI_EVENT: 99,\n  WEB_UI_INITIALIZE_EVENT: 100,\n  WEB_UI_STOP_EVENT: 101,\n  WI_FI_EVENT: 102,\n  WI_FI_INPUT_CONNECTION_EVENT: 103,\n  WI_FI_UPDATE_REQUEST_EVENT: 104\n};\n\n/**\n * @return {proto.ubo.v1.Event.EventCase}\n */\nproto.ubo.v1.Event.prototype.getEventCase = function() {\n  return /** @type {proto.ubo.v1.Event.EventCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Event.oneofGroups_[0]));\n};\n\n\n\nif (jspb.Message.GENERATE_TO_OBJECT) {\n/**\n * Creates an object representation of this proto.\n * Field names that are reserved in JavaScript and will be renamed to pb_name.\n * Optional fields that are not set will be set to undefined.\n * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.\n * For the list of reserved names please see:\n *     net/proto2/compiler/js/internal/generator.cc#kKeyword.\n * @param {boolean=} opt_includeInstance Deprecated. whether to include the\n *     JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @return {!Object}\n */\nproto.ubo.v1.Event.prototype.toObject = function(opt_includeInstance) {\n  return proto.ubo.v1.Event.toObject(opt_includeInstance, this);\n};\n\n\n/**\n * Static version of the {@see toObject} method.\n * @param {boolean|undefined} includeInstance Deprecated. Whether to include\n *     the JSPB instance for transitional soy proto support:\n *     http://goto/soy-param-migration\n * @param {!proto.ubo.v1.Event} msg The msg instance to transform.\n * @return {!Object}\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Event.toObject = function(includeInstance, msg) {\n  var f, obj = {\n    assistantAddMcpServerEvent: (f = msg.getAssistantAddMcpServerEvent()) && proto.ubo.v1.AssistantAddMcpServerEvent.toObject(includeInstance, f),\n    assistantDeleteMcpServerEvent: (f = msg.getAssistantDeleteMcpServerEvent()) && proto.ubo.v1.AssistantDeleteMcpServerEvent.toObject(includeInstance, f),\n    assistantDownloadOllamaModelEvent: (f = msg.getAssistantDownloadOllamaModelEvent()) && proto.ubo.v1.AssistantDownloadOllamaModelEvent.toObject(includeInstance, f),\n    assistantEvent: (f = msg.getAssistantEvent()) && proto.ubo.v1.AssistantEvent.toObject(includeInstance, f),\n    assistantHandleReportEvent: (f = msg.getAssistantHandleReportEvent()) && proto.ubo.v1.AssistantHandleReportEvent.toObject(includeInstance, f),\n    assistantUpdateProvidersEvent: (f = msg.getAssistantUpdateProvidersEvent()) && proto.ubo.v1.AssistantUpdateProvidersEvent.toObject(includeInstance, f),\n    audioEvent: (f = msg.getAudioEvent()) && proto.ubo.v1.AudioEvent.toObject(includeInstance, f),\n    audioInstallDriverEvent: (f = msg.getAudioInstallDriverEvent()) && proto.ubo.v1.AudioInstallDriverEvent.toObject(includeInstance, f),\n    audioPlayAudioSampleEvent: (f = msg.getAudioPlayAudioSampleEvent()) && proto.ubo.v1.AudioPlayAudioSampleEvent.toObject(includeInstance, f),\n    audioPlayAudioSequenceEvent: (f = msg.getAudioPlayAudioSequenceEvent()) && proto.ubo.v1.AudioPlayAudioSequenceEvent.toObject(includeInstance, f),\n    audioPlayChimeEvent: (f = msg.getAudioPlayChimeEvent()) && proto.ubo.v1.AudioPlayChimeEvent.toObject(includeInstance, f),\n    audioPlaybackDoneEvent: (f = msg.getAudioPlaybackDoneEvent()) && proto.ubo.v1.AudioPlaybackDoneEvent.toObject(includeInstance, f),\n    audioReportSampleEvent: (f = msg.getAudioReportSampleEvent()) && proto.ubo.v1.AudioReportSampleEvent.toObject(includeInstance, f),\n    cameraDetectEvent: (f = msg.getCameraDetectEvent()) && proto.ubo.v1.CameraDetectEvent.toObject(includeInstance, f),\n    cameraDetectedEvent: (f = msg.getCameraDetectedEvent()) && proto.ubo.v1.CameraDetectedEvent.toObject(includeInstance, f),\n    cameraEvent: (f = msg.getCameraEvent()) && proto.ubo.v1.CameraEvent.toObject(includeInstance, f),\n    cameraInstallDriverEvent: (f = msg.getCameraInstallDriverEvent()) && proto.ubo.v1.CameraInstallDriverEvent.toObject(includeInstance, f),\n    cameraReinitializeEvent: (f = msg.getCameraReinitializeEvent()) && proto.ubo.v1.CameraReinitializeEvent.toObject(includeInstance, f),\n    cameraReportImageEvent: (f = msg.getCameraReportImageEvent()) && proto.ubo.v1.CameraReportImageEvent.toObject(includeInstance, f),\n    cameraRestoreDefaultEvent: (f = msg.getCameraRestoreDefaultEvent()) && proto.ubo.v1.CameraRestoreDefaultEvent.toObject(includeInstance, f),\n    cameraStartViewfinderEvent: (f = msg.getCameraStartViewfinderEvent()) && proto.ubo.v1.CameraStartViewfinderEvent.toObject(includeInstance, f),\n    cameraStopViewfinderEvent: (f = msg.getCameraStopViewfinderEvent()) && proto.ubo.v1.CameraStopViewfinderEvent.toObject(includeInstance, f),\n    closeApplicationEvent: (f = msg.getCloseApplicationEvent()) && proto.ubo.v1.CloseApplicationEvent.toObject(includeInstance, f),\n    displayBlankEvent: (f = msg.getDisplayBlankEvent()) && proto.ubo.v1.DisplayBlankEvent.toObject(includeInstance, f),\n    displayCompressedRenderEvent: (f = msg.getDisplayCompressedRenderEvent()) && proto.ubo.v1.DisplayCompressedRenderEvent.toObject(includeInstance, f),\n    displayEvent: (f = msg.getDisplayEvent()) && proto.ubo.v1.DisplayEvent.toObject(includeInstance, f),\n    displayRedrawEvent: (f = msg.getDisplayRedrawEvent()) && proto.ubo.v1.DisplayRedrawEvent.toObject(includeInstance, f),\n    displayRenderEvent: (f = msg.getDisplayRenderEvent()) && proto.ubo.v1.DisplayRenderEvent.toObject(includeInstance, f),\n    displayUnblankEvent: (f = msg.getDisplayUnblankEvent()) && proto.ubo.v1.DisplayUnblankEvent.toObject(includeInstance, f),\n    dockerEvent: (f = msg.getDockerEvent()) && proto.ubo.v1.DockerEvent.toObject(includeInstance, f),\n    dockerImageEvent: (f = msg.getDockerImageEvent()) && proto.ubo.v1.DockerImageEvent.toObject(includeInstance, f),\n    dockerImageFetchCompositionEvent: (f = msg.getDockerImageFetchCompositionEvent()) && proto.ubo.v1.DockerImageFetchCompositionEvent.toObject(includeInstance, f),\n    dockerImageFetchEvent: (f = msg.getDockerImageFetchEvent()) && proto.ubo.v1.DockerImageFetchEvent.toObject(includeInstance, f),\n    dockerImageRegisterAppEvent: (f = msg.getDockerImageRegisterAppEvent()) && proto.ubo.v1.DockerImageRegisterAppEvent.toObject(includeInstance, f),\n    dockerImageReleaseCompositionEvent: (f = msg.getDockerImageReleaseCompositionEvent()) && proto.ubo.v1.DockerImageReleaseCompositionEvent.toObject(includeInstance, f),\n    dockerImageRemoveCompositionEvent: (f = msg.getDockerImageRemoveCompositionEvent()) && proto.ubo.v1.DockerImageRemoveCompositionEvent.toObject(includeInstance, f),\n    dockerImageRemoveContainerEvent: (f = msg.getDockerImageRemoveContainerEvent()) && proto.ubo.v1.DockerImageRemoveContainerEvent.toObject(includeInstance, f),\n    dockerImageRemoveEvent: (f = msg.getDockerImageRemoveEvent()) && proto.ubo.v1.DockerImageRemoveEvent.toObject(includeInstance, f),\n    dockerImageRunCompositionEvent: (f = msg.getDockerImageRunCompositionEvent()) && proto.ubo.v1.DockerImageRunCompositionEvent.toObject(includeInstance, f),\n    dockerImageRunContainerEvent: (f = msg.getDockerImageRunContainerEvent()) && proto.ubo.v1.DockerImageRunContainerEvent.toObject(includeInstance, f),\n    dockerImageStopCompositionEvent: (f = msg.getDockerImageStopCompositionEvent()) && proto.ubo.v1.DockerImageStopCompositionEvent.toObject(includeInstance, f),\n    dockerImageStopContainerEvent: (f = msg.getDockerImageStopContainerEvent()) && proto.ubo.v1.DockerImageStopContainerEvent.toObject(includeInstance, f),\n    dockerInstallEvent: (f = msg.getDockerInstallEvent()) && proto.ubo.v1.DockerInstallEvent.toObject(includeInstance, f),\n    dockerStartEvent: (f = msg.getDockerStartEvent()) && proto.ubo.v1.DockerStartEvent.toObject(includeInstance, f),\n    dockerStopEvent: (f = msg.getDockerStopEvent()) && proto.ubo.v1.DockerStopEvent.toObject(includeInstance, f),\n    fileSystemCopyEvent: (f = msg.getFileSystemCopyEvent()) && proto.ubo.v1.FileSystemCopyEvent.toObject(includeInstance, f),\n    fileSystemEvent: (f = msg.getFileSystemEvent()) && proto.ubo.v1.FileSystemEvent.toObject(includeInstance, f),\n    fileSystemMoveEvent: (f = msg.getFileSystemMoveEvent()) && proto.ubo.v1.FileSystemMoveEvent.toObject(includeInstance, f),\n    fileSystemRemoveEvent: (f = msg.getFileSystemRemoveEvent()) && proto.ubo.v1.FileSystemRemoveEvent.toObject(includeInstance, f),\n    fileSystemSelectEvent: (f = msg.getFileSystemSelectEvent()) && proto.ubo.v1.FileSystemSelectEvent.toObject(includeInstance, f),\n    infraredEvent: (f = msg.getInfraredEvent()) && proto.ubo.v1.InfraredEvent.toObject(includeInstance, f),\n    infraredSendCodeEvent: (f = msg.getInfraredSendCodeEvent()) && proto.ubo.v1.InfraredSendCodeEvent.toObject(includeInstance, f),\n    initEvent: (f = msg.getInitEvent()) && proto.ubo.v1.InitEvent.toObject(includeInstance, f),\n    inputCancelEvent: (f = msg.getInputCancelEvent()) && proto.ubo.v1.InputCancelEvent.toObject(includeInstance, f),\n    inputProvideEvent: (f = msg.getInputProvideEvent()) && proto.ubo.v1.InputProvideEvent.toObject(includeInstance, f),\n    inputResolveEvent: (f = msg.getInputResolveEvent()) && proto.ubo.v1.InputResolveEvent.toObject(includeInstance, f),\n    ipEvent: (f = msg.getIpEvent()) && proto.ubo.v1.IpEvent.toObject(includeInstance, f),\n    mainEvent: (f = msg.getMainEvent()) && proto.ubo.v1.MainEvent.toObject(includeInstance, f),\n    menuChooseByIconEvent: (f = msg.getMenuChooseByIconEvent()) && proto.ubo.v1.MenuChooseByIconEvent.toObject(includeInstance, f),\n    menuChooseByIndexEvent: (f = msg.getMenuChooseByIndexEvent()) && proto.ubo.v1.MenuChooseByIndexEvent.toObject(includeInstance, f),\n    menuChooseByLabelEvent: (f = msg.getMenuChooseByLabelEvent()) && proto.ubo.v1.MenuChooseByLabelEvent.toObject(includeInstance, f),\n    menuEvent: (f = msg.getMenuEvent()) && proto.ubo.v1.MenuEvent.toObject(includeInstance, f),\n    menuGoBackEvent: (f = msg.getMenuGoBackEvent()) && proto.ubo.v1.MenuGoBackEvent.toObject(includeInstance, f),\n    menuGoHomeEvent: (f = msg.getMenuGoHomeEvent()) && proto.ubo.v1.MenuGoHomeEvent.toObject(includeInstance, f),\n    menuScrollEvent: (f = msg.getMenuScrollEvent()) && proto.ubo.v1.MenuScrollEvent.toObject(includeInstance, f),\n    notificationsClearEvent: (f = msg.getNotificationsClearEvent()) && proto.ubo.v1.NotificationsClearEvent.toObject(includeInstance, f),\n    notificationsDisplayEvent: (f = msg.getNotificationsDisplayEvent()) && proto.ubo.v1.NotificationsDisplayEvent.toObject(includeInstance, f),\n    notificationsEvent: (f = msg.getNotificationsEvent()) && proto.ubo.v1.NotificationsEvent.toObject(includeInstance, f),\n    openApplicationEvent: (f = msg.getOpenApplicationEvent()) && proto.ubo.v1.OpenApplicationEvent.toObject(includeInstance, f),\n    powerEvent: (f = msg.getPowerEvent()) && proto.ubo.v1.PowerEvent.toObject(includeInstance, f),\n    powerOffEvent: (f = msg.getPowerOffEvent()) && proto.ubo.v1.PowerOffEvent.toObject(includeInstance, f),\n    rPiConnectEvent: (f = msg.getRPiConnectEvent()) && proto.ubo.v1.RPiConnectEvent.toObject(includeInstance, f),\n    rPiConnectLoginEvent: (f = msg.getRPiConnectLoginEvent()) && proto.ubo.v1.RPiConnectLoginEvent.toObject(includeInstance, f),\n    rebootEvent: (f = msg.getRebootEvent()) && proto.ubo.v1.RebootEvent.toObject(includeInstance, f),\n    replayRecordedSequenceEvent: (f = msg.getReplayRecordedSequenceEvent()) && proto.ubo.v1.ReplayRecordedSequenceEvent.toObject(includeInstance, f),\n    rgbRingCommandEvent: (f = msg.getRgbRingCommandEvent()) && proto.ubo.v1.RgbRingCommandEvent.toObject(includeInstance, f),\n    rgbRingEvent: (f = msg.getRgbRingEvent()) && proto.ubo.v1.RgbRingEvent.toObject(includeInstance, f),\n    screenshotEvent: (f = msg.getScreenshotEvent()) && proto.ubo.v1.ScreenshotEvent.toObject(includeInstance, f),\n    settingsEvent: (f = msg.getSettingsEvent()) && proto.ubo.v1.SettingsEvent.toObject(includeInstance, f),\n    settingsServiceEvent: (f = msg.getSettingsServiceEvent()) && proto.ubo.v1.SettingsServiceEvent.toObject(includeInstance, f),\n    settingsStartServiceEvent: (f = msg.getSettingsStartServiceEvent()) && proto.ubo.v1.SettingsStartServiceEvent.toObject(includeInstance, f),\n    settingsStopServiceEvent: (f = msg.getSettingsStopServiceEvent()) && proto.ubo.v1.SettingsStopServiceEvent.toObject(includeInstance, f),\n    snapshotEvent: (f = msg.getSnapshotEvent()) && proto.ubo.v1.SnapshotEvent.toObject(includeInstance, f),\n    speechRecognitionEvent: (f = msg.getSpeechRecognitionEvent()) && proto.ubo.v1.SpeechRecognitionEvent.toObject(includeInstance, f),\n    speechRecognitionReportTextEvent: (f = msg.getSpeechRecognitionReportTextEvent()) && proto.ubo.v1.SpeechRecognitionReportTextEvent.toObject(includeInstance, f),\n    speechSynthesisEvent: (f = msg.getSpeechSynthesisEvent()) && proto.ubo.v1.SpeechSynthesisEvent.toObject(includeInstance, f),\n    speechSynthesisSynthesizeTextEvent: (f = msg.getSpeechSynthesisSynthesizeTextEvent()) && proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.toObject(includeInstance, f),\n    storeRecordedSequenceEvent: (f = msg.getStoreRecordedSequenceEvent()) && proto.ubo.v1.StoreRecordedSequenceEvent.toObject(includeInstance, f),\n    updateManagerCheckEvent: (f = msg.getUpdateManagerCheckEvent()) && proto.ubo.v1.UpdateManagerCheckEvent.toObject(includeInstance, f),\n    updateManagerEvent: (f = msg.getUpdateManagerEvent()) && proto.ubo.v1.UpdateManagerEvent.toObject(includeInstance, f),\n    updateManagerUpdateEvent: (f = msg.getUpdateManagerUpdateEvent()) && proto.ubo.v1.UpdateManagerUpdateEvent.toObject(includeInstance, f),\n    usersCreateUserEvent: (f = msg.getUsersCreateUserEvent()) && proto.ubo.v1.UsersCreateUserEvent.toObject(includeInstance, f),\n    usersDeleteUserEvent: (f = msg.getUsersDeleteUserEvent()) && proto.ubo.v1.UsersDeleteUserEvent.toObject(includeInstance, f),\n    usersEvent: (f = msg.getUsersEvent()) && proto.ubo.v1.UsersEvent.toObject(includeInstance, f),\n    usersResetPasswordEvent: (f = msg.getUsersResetPasswordEvent()) && proto.ubo.v1.UsersResetPasswordEvent.toObject(includeInstance, f),\n    vsCodeEvent: (f = msg.getVsCodeEvent()) && proto.ubo.v1.VSCodeEvent.toObject(includeInstance, f),\n    vsCodeLoginEvent: (f = msg.getVsCodeLoginEvent()) && proto.ubo.v1.VSCodeLoginEvent.toObject(includeInstance, f),\n    vsCodeRestartEvent: (f = msg.getVsCodeRestartEvent()) && proto.ubo.v1.VSCodeRestartEvent.toObject(includeInstance, f),\n    webUiEvent: (f = msg.getWebUiEvent()) && proto.ubo.v1.WebUIEvent.toObject(includeInstance, f),\n    webUiInitializeEvent: (f = msg.getWebUiInitializeEvent()) && proto.ubo.v1.WebUIInitializeEvent.toObject(includeInstance, f),\n    webUiStopEvent: (f = msg.getWebUiStopEvent()) && proto.ubo.v1.WebUIStopEvent.toObject(includeInstance, f),\n    wiFiEvent: (f = msg.getWiFiEvent()) && proto.ubo.v1.WiFiEvent.toObject(includeInstance, f),\n    wiFiInputConnectionEvent: (f = msg.getWiFiInputConnectionEvent()) && proto.ubo.v1.WiFiInputConnectionEvent.toObject(includeInstance, f),\n    wiFiUpdateRequestEvent: (f = msg.getWiFiUpdateRequestEvent()) && proto.ubo.v1.WiFiUpdateRequestEvent.toObject(includeInstance, f)\n  };\n\n  if (includeInstance) {\n    obj.$jspbMessageInstance = msg;\n  }\n  return obj;\n};\n}\n\n\n/**\n * Deserializes binary data (in protobuf wire format).\n * @param {jspb.ByteSource} bytes The bytes to deserialize.\n * @return {!proto.ubo.v1.Event}\n */\nproto.ubo.v1.Event.deserializeBinary = function(bytes) {\n  var reader = new jspb.BinaryReader(bytes);\n  var msg = new proto.ubo.v1.Event;\n  return proto.ubo.v1.Event.deserializeBinaryFromReader(msg, reader);\n};\n\n\n/**\n * Deserializes binary data (in protobuf wire format) from the\n * given reader into the given message object.\n * @param {!proto.ubo.v1.Event} msg The message object to deserialize into.\n * @param {!jspb.BinaryReader} reader The BinaryReader to use.\n * @return {!proto.ubo.v1.Event}\n */\nproto.ubo.v1.Event.deserializeBinaryFromReader = function(msg, reader) {\n  while (reader.nextField()) {\n    if (reader.isEndGroup()) {\n      break;\n    }\n    var field = reader.getFieldNumber();\n    switch (field) {\n    case 1:\n      var value = new proto.ubo.v1.AssistantAddMcpServerEvent;\n      reader.readMessage(value,proto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinaryFromReader);\n      msg.setAssistantAddMcpServerEvent(value);\n      break;\n    case 2:\n      var value = new proto.ubo.v1.AssistantDeleteMcpServerEvent;\n      reader.readMessage(value,proto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinaryFromReader);\n      msg.setAssistantDeleteMcpServerEvent(value);\n      break;\n    case 3:\n      var value = new proto.ubo.v1.AssistantDownloadOllamaModelEvent;\n      reader.readMessage(value,proto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinaryFromReader);\n      msg.setAssistantDownloadOllamaModelEvent(value);\n      break;\n    case 4:\n      var value = new proto.ubo.v1.AssistantEvent;\n      reader.readMessage(value,proto.ubo.v1.AssistantEvent.deserializeBinaryFromReader);\n      msg.setAssistantEvent(value);\n      break;\n    case 5:\n      var value = new proto.ubo.v1.AssistantHandleReportEvent;\n      reader.readMessage(value,proto.ubo.v1.AssistantHandleReportEvent.deserializeBinaryFromReader);\n      msg.setAssistantHandleReportEvent(value);\n      break;\n    case 6:\n      var value = new proto.ubo.v1.AssistantUpdateProvidersEvent;\n      reader.readMessage(value,proto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinaryFromReader);\n      msg.setAssistantUpdateProvidersEvent(value);\n      break;\n    case 7:\n      var value = new proto.ubo.v1.AudioEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioEvent.deserializeBinaryFromReader);\n      msg.setAudioEvent(value);\n      break;\n    case 8:\n      var value = new proto.ubo.v1.AudioInstallDriverEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioInstallDriverEvent.deserializeBinaryFromReader);\n      msg.setAudioInstallDriverEvent(value);\n      break;\n    case 9:\n      var value = new proto.ubo.v1.AudioPlayAudioSampleEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinaryFromReader);\n      msg.setAudioPlayAudioSampleEvent(value);\n      break;\n    case 10:\n      var value = new proto.ubo.v1.AudioPlayAudioSequenceEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinaryFromReader);\n      msg.setAudioPlayAudioSequenceEvent(value);\n      break;\n    case 11:\n      var value = new proto.ubo.v1.AudioPlayChimeEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioPlayChimeEvent.deserializeBinaryFromReader);\n      msg.setAudioPlayChimeEvent(value);\n      break;\n    case 12:\n      var value = new proto.ubo.v1.AudioPlaybackDoneEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinaryFromReader);\n      msg.setAudioPlaybackDoneEvent(value);\n      break;\n    case 13:\n      var value = new proto.ubo.v1.AudioReportSampleEvent;\n      reader.readMessage(value,proto.ubo.v1.AudioReportSampleEvent.deserializeBinaryFromReader);\n      msg.setAudioReportSampleEvent(value);\n      break;\n    case 14:\n      var value = new proto.ubo.v1.CameraDetectEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraDetectEvent.deserializeBinaryFromReader);\n      msg.setCameraDetectEvent(value);\n      break;\n    case 15:\n      var value = new proto.ubo.v1.CameraDetectedEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraDetectedEvent.deserializeBinaryFromReader);\n      msg.setCameraDetectedEvent(value);\n      break;\n    case 16:\n      var value = new proto.ubo.v1.CameraEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraEvent.deserializeBinaryFromReader);\n      msg.setCameraEvent(value);\n      break;\n    case 17:\n      var value = new proto.ubo.v1.CameraInstallDriverEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraInstallDriverEvent.deserializeBinaryFromReader);\n      msg.setCameraInstallDriverEvent(value);\n      break;\n    case 18:\n      var value = new proto.ubo.v1.CameraReinitializeEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraReinitializeEvent.deserializeBinaryFromReader);\n      msg.setCameraReinitializeEvent(value);\n      break;\n    case 19:\n      var value = new proto.ubo.v1.CameraReportImageEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraReportImageEvent.deserializeBinaryFromReader);\n      msg.setCameraReportImageEvent(value);\n      break;\n    case 20:\n      var value = new proto.ubo.v1.CameraRestoreDefaultEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinaryFromReader);\n      msg.setCameraRestoreDefaultEvent(value);\n      break;\n    case 21:\n      var value = new proto.ubo.v1.CameraStartViewfinderEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraStartViewfinderEvent.deserializeBinaryFromReader);\n      msg.setCameraStartViewfinderEvent(value);\n      break;\n    case 22:\n      var value = new proto.ubo.v1.CameraStopViewfinderEvent;\n      reader.readMessage(value,proto.ubo.v1.CameraStopViewfinderEvent.deserializeBinaryFromReader);\n      msg.setCameraStopViewfinderEvent(value);\n      break;\n    case 23:\n      var value = new proto.ubo.v1.CloseApplicationEvent;\n      reader.readMessage(value,proto.ubo.v1.CloseApplicationEvent.deserializeBinaryFromReader);\n      msg.setCloseApplicationEvent(value);\n      break;\n    case 24:\n      var value = new proto.ubo.v1.DisplayBlankEvent;\n      reader.readMessage(value,proto.ubo.v1.DisplayBlankEvent.deserializeBinaryFromReader);\n      msg.setDisplayBlankEvent(value);\n      break;\n    case 25:\n      var value = new proto.ubo.v1.DisplayCompressedRenderEvent;\n      reader.readMessage(value,proto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinaryFromReader);\n      msg.setDisplayCompressedRenderEvent(value);\n      break;\n    case 26:\n      var value = new proto.ubo.v1.DisplayEvent;\n      reader.readMessage(value,proto.ubo.v1.DisplayEvent.deserializeBinaryFromReader);\n      msg.setDisplayEvent(value);\n      break;\n    case 27:\n      var value = new proto.ubo.v1.DisplayRedrawEvent;\n      reader.readMessage(value,proto.ubo.v1.DisplayRedrawEvent.deserializeBinaryFromReader);\n      msg.setDisplayRedrawEvent(value);\n      break;\n    case 28:\n      var value = new proto.ubo.v1.DisplayRenderEvent;\n      reader.readMessage(value,proto.ubo.v1.DisplayRenderEvent.deserializeBinaryFromReader);\n      msg.setDisplayRenderEvent(value);\n      break;\n    case 29:\n      var value = new proto.ubo.v1.DisplayUnblankEvent;\n      reader.readMessage(value,proto.ubo.v1.DisplayUnblankEvent.deserializeBinaryFromReader);\n      msg.setDisplayUnblankEvent(value);\n      break;\n    case 30:\n      var value = new proto.ubo.v1.DockerEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerEvent.deserializeBinaryFromReader);\n      msg.setDockerEvent(value);\n      break;\n    case 31:\n      var value = new proto.ubo.v1.DockerImageEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageEvent.deserializeBinaryFromReader);\n      msg.setDockerImageEvent(value);\n      break;\n    case 32:\n      var value = new proto.ubo.v1.DockerImageFetchCompositionEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinaryFromReader);\n      msg.setDockerImageFetchCompositionEvent(value);\n      break;\n    case 33:\n      var value = new proto.ubo.v1.DockerImageFetchEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageFetchEvent.deserializeBinaryFromReader);\n      msg.setDockerImageFetchEvent(value);\n      break;\n    case 34:\n      var value = new proto.ubo.v1.DockerImageRegisterAppEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinaryFromReader);\n      msg.setDockerImageRegisterAppEvent(value);\n      break;\n    case 35:\n      var value = new proto.ubo.v1.DockerImageReleaseCompositionEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinaryFromReader);\n      msg.setDockerImageReleaseCompositionEvent(value);\n      break;\n    case 36:\n      var value = new proto.ubo.v1.DockerImageRemoveCompositionEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinaryFromReader);\n      msg.setDockerImageRemoveCompositionEvent(value);\n      break;\n    case 37:\n      var value = new proto.ubo.v1.DockerImageRemoveContainerEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinaryFromReader);\n      msg.setDockerImageRemoveContainerEvent(value);\n      break;\n    case 38:\n      var value = new proto.ubo.v1.DockerImageRemoveEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveEvent.deserializeBinaryFromReader);\n      msg.setDockerImageRemoveEvent(value);\n      break;\n    case 39:\n      var value = new proto.ubo.v1.DockerImageRunCompositionEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinaryFromReader);\n      msg.setDockerImageRunCompositionEvent(value);\n      break;\n    case 40:\n      var value = new proto.ubo.v1.DockerImageRunContainerEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageRunContainerEvent.deserializeBinaryFromReader);\n      msg.setDockerImageRunContainerEvent(value);\n      break;\n    case 41:\n      var value = new proto.ubo.v1.DockerImageStopCompositionEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinaryFromReader);\n      msg.setDockerImageStopCompositionEvent(value);\n      break;\n    case 42:\n      var value = new proto.ubo.v1.DockerImageStopContainerEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerImageStopContainerEvent.deserializeBinaryFromReader);\n      msg.setDockerImageStopContainerEvent(value);\n      break;\n    case 43:\n      var value = new proto.ubo.v1.DockerInstallEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerInstallEvent.deserializeBinaryFromReader);\n      msg.setDockerInstallEvent(value);\n      break;\n    case 44:\n      var value = new proto.ubo.v1.DockerStartEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerStartEvent.deserializeBinaryFromReader);\n      msg.setDockerStartEvent(value);\n      break;\n    case 45:\n      var value = new proto.ubo.v1.DockerStopEvent;\n      reader.readMessage(value,proto.ubo.v1.DockerStopEvent.deserializeBinaryFromReader);\n      msg.setDockerStopEvent(value);\n      break;\n    case 46:\n      var value = new proto.ubo.v1.FileSystemCopyEvent;\n      reader.readMessage(value,proto.ubo.v1.FileSystemCopyEvent.deserializeBinaryFromReader);\n      msg.setFileSystemCopyEvent(value);\n      break;\n    case 47:\n      var value = new proto.ubo.v1.FileSystemEvent;\n      reader.readMessage(value,proto.ubo.v1.FileSystemEvent.deserializeBinaryFromReader);\n      msg.setFileSystemEvent(value);\n      break;\n    case 48:\n      var value = new proto.ubo.v1.FileSystemMoveEvent;\n      reader.readMessage(value,proto.ubo.v1.FileSystemMoveEvent.deserializeBinaryFromReader);\n      msg.setFileSystemMoveEvent(value);\n      break;\n    case 49:\n      var value = new proto.ubo.v1.FileSystemRemoveEvent;\n      reader.readMessage(value,proto.ubo.v1.FileSystemRemoveEvent.deserializeBinaryFromReader);\n      msg.setFileSystemRemoveEvent(value);\n      break;\n    case 50:\n      var value = new proto.ubo.v1.FileSystemSelectEvent;\n      reader.readMessage(value,proto.ubo.v1.FileSystemSelectEvent.deserializeBinaryFromReader);\n      msg.setFileSystemSelectEvent(value);\n      break;\n    case 51:\n      var value = new proto.ubo.v1.InfraredEvent;\n      reader.readMessage(value,proto.ubo.v1.InfraredEvent.deserializeBinaryFromReader);\n      msg.setInfraredEvent(value);\n      break;\n    case 52:\n      var value = new proto.ubo.v1.InfraredSendCodeEvent;\n      reader.readMessage(value,proto.ubo.v1.InfraredSendCodeEvent.deserializeBinaryFromReader);\n      msg.setInfraredSendCodeEvent(value);\n      break;\n    case 53:\n      var value = new proto.ubo.v1.InitEvent;\n      reader.readMessage(value,proto.ubo.v1.InitEvent.deserializeBinaryFromReader);\n      msg.setInitEvent(value);\n      break;\n    case 54:\n      var value = new proto.ubo.v1.InputCancelEvent;\n      reader.readMessage(value,proto.ubo.v1.InputCancelEvent.deserializeBinaryFromReader);\n      msg.setInputCancelEvent(value);\n      break;\n    case 55:\n      var value = new proto.ubo.v1.InputProvideEvent;\n      reader.readMessage(value,proto.ubo.v1.InputProvideEvent.deserializeBinaryFromReader);\n      msg.setInputProvideEvent(value);\n      break;\n    case 56:\n      var value = new proto.ubo.v1.InputResolveEvent;\n      reader.readMessage(value,proto.ubo.v1.InputResolveEvent.deserializeBinaryFromReader);\n      msg.setInputResolveEvent(value);\n      break;\n    case 57:\n      var value = new proto.ubo.v1.IpEvent;\n      reader.readMessage(value,proto.ubo.v1.IpEvent.deserializeBinaryFromReader);\n      msg.setIpEvent(value);\n      break;\n    case 58:\n      var value = new proto.ubo.v1.MainEvent;\n      reader.readMessage(value,proto.ubo.v1.MainEvent.deserializeBinaryFromReader);\n      msg.setMainEvent(value);\n      break;\n    case 59:\n      var value = new proto.ubo.v1.MenuChooseByIconEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuChooseByIconEvent.deserializeBinaryFromReader);\n      msg.setMenuChooseByIconEvent(value);\n      break;\n    case 60:\n      var value = new proto.ubo.v1.MenuChooseByIndexEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuChooseByIndexEvent.deserializeBinaryFromReader);\n      msg.setMenuChooseByIndexEvent(value);\n      break;\n    case 61:\n      var value = new proto.ubo.v1.MenuChooseByLabelEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuChooseByLabelEvent.deserializeBinaryFromReader);\n      msg.setMenuChooseByLabelEvent(value);\n      break;\n    case 62:\n      var value = new proto.ubo.v1.MenuEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuEvent.deserializeBinaryFromReader);\n      msg.setMenuEvent(value);\n      break;\n    case 63:\n      var value = new proto.ubo.v1.MenuGoBackEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuGoBackEvent.deserializeBinaryFromReader);\n      msg.setMenuGoBackEvent(value);\n      break;\n    case 64:\n      var value = new proto.ubo.v1.MenuGoHomeEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuGoHomeEvent.deserializeBinaryFromReader);\n      msg.setMenuGoHomeEvent(value);\n      break;\n    case 65:\n      var value = new proto.ubo.v1.MenuScrollEvent;\n      reader.readMessage(value,proto.ubo.v1.MenuScrollEvent.deserializeBinaryFromReader);\n      msg.setMenuScrollEvent(value);\n      break;\n    case 66:\n      var value = new proto.ubo.v1.NotificationsClearEvent;\n      reader.readMessage(value,proto.ubo.v1.NotificationsClearEvent.deserializeBinaryFromReader);\n      msg.setNotificationsClearEvent(value);\n      break;\n    case 67:\n      var value = new proto.ubo.v1.NotificationsDisplayEvent;\n      reader.readMessage(value,proto.ubo.v1.NotificationsDisplayEvent.deserializeBinaryFromReader);\n      msg.setNotificationsDisplayEvent(value);\n      break;\n    case 68:\n      var value = new proto.ubo.v1.NotificationsEvent;\n      reader.readMessage(value,proto.ubo.v1.NotificationsEvent.deserializeBinaryFromReader);\n      msg.setNotificationsEvent(value);\n      break;\n    case 69:\n      var value = new proto.ubo.v1.OpenApplicationEvent;\n      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.deserializeBinaryFromReader);\n      msg.setOpenApplicationEvent(value);\n      break;\n    case 70:\n      var value = new proto.ubo.v1.PowerEvent;\n      reader.readMessage(value,proto.ubo.v1.PowerEvent.deserializeBinaryFromReader);\n      msg.setPowerEvent(value);\n      break;\n    case 71:\n      var value = new proto.ubo.v1.PowerOffEvent;\n      reader.readMessage(value,proto.ubo.v1.PowerOffEvent.deserializeBinaryFromReader);\n      msg.setPowerOffEvent(value);\n      break;\n    case 72:\n      var value = new proto.ubo.v1.RPiConnectEvent;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectEvent.deserializeBinaryFromReader);\n      msg.setRPiConnectEvent(value);\n      break;\n    case 73:\n      var value = new proto.ubo.v1.RPiConnectLoginEvent;\n      reader.readMessage(value,proto.ubo.v1.RPiConnectLoginEvent.deserializeBinaryFromReader);\n      msg.setRPiConnectLoginEvent(value);\n      break;\n    case 74:\n      var value = new proto.ubo.v1.RebootEvent;\n      reader.readMessage(value,proto.ubo.v1.RebootEvent.deserializeBinaryFromReader);\n      msg.setRebootEvent(value);\n      break;\n    case 75:\n      var value = new proto.ubo.v1.ReplayRecordedSequenceEvent;\n      reader.readMessage(value,proto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinaryFromReader);\n      msg.setReplayRecordedSequenceEvent(value);\n      break;\n    case 76:\n      var value = new proto.ubo.v1.RgbRingCommandEvent;\n      reader.readMessage(value,proto.ubo.v1.RgbRingCommandEvent.deserializeBinaryFromReader);\n      msg.setRgbRingCommandEvent(value);\n      break;\n    case 77:\n      var value = new proto.ubo.v1.RgbRingEvent;\n      reader.readMessage(value,proto.ubo.v1.RgbRingEvent.deserializeBinaryFromReader);\n      msg.setRgbRingEvent(value);\n      break;\n    case 78:\n      var value = new proto.ubo.v1.ScreenshotEvent;\n      reader.readMessage(value,proto.ubo.v1.ScreenshotEvent.deserializeBinaryFromReader);\n      msg.setScreenshotEvent(value);\n      break;\n    case 79:\n      var value = new proto.ubo.v1.SettingsEvent;\n      reader.readMessage(value,proto.ubo.v1.SettingsEvent.deserializeBinaryFromReader);\n      msg.setSettingsEvent(value);\n      break;\n    case 80:\n      var value = new proto.ubo.v1.SettingsServiceEvent;\n      reader.readMessage(value,proto.ubo.v1.SettingsServiceEvent.deserializeBinaryFromReader);\n      msg.setSettingsServiceEvent(value);\n      break;\n    case 81:\n      var value = new proto.ubo.v1.SettingsStartServiceEvent;\n      reader.readMessage(value,proto.ubo.v1.SettingsStartServiceEvent.deserializeBinaryFromReader);\n      msg.setSettingsStartServiceEvent(value);\n      break;\n    case 82:\n      var value = new proto.ubo.v1.SettingsStopServiceEvent;\n      reader.readMessage(value,proto.ubo.v1.SettingsStopServiceEvent.deserializeBinaryFromReader);\n      msg.setSettingsStopServiceEvent(value);\n      break;\n    case 83:\n      var value = new proto.ubo.v1.SnapshotEvent;\n      reader.readMessage(value,proto.ubo.v1.SnapshotEvent.deserializeBinaryFromReader);\n      msg.setSnapshotEvent(value);\n      break;\n    case 84:\n      var value = new proto.ubo.v1.SpeechRecognitionEvent;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionEvent.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionEvent(value);\n      break;\n    case 85:\n      var value = new proto.ubo.v1.SpeechRecognitionReportTextEvent;\n      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinaryFromReader);\n      msg.setSpeechRecognitionReportTextEvent(value);\n      break;\n    case 86:\n      var value = new proto.ubo.v1.SpeechSynthesisEvent;\n      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisEvent.deserializeBinaryFromReader);\n      msg.setSpeechSynthesisEvent(value);\n      break;\n    case 87:\n      var value = new proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent;\n      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinaryFromReader);\n      msg.setSpeechSynthesisSynthesizeTextEvent(value);\n      break;\n    case 88:\n      var value = new proto.ubo.v1.StoreRecordedSequenceEvent;\n      reader.readMessage(value,proto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinaryFromReader);\n      msg.setStoreRecordedSequenceEvent(value);\n      break;\n    case 89:\n      var value = new proto.ubo.v1.UpdateManagerCheckEvent;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerCheckEvent.deserializeBinaryFromReader);\n      msg.setUpdateManagerCheckEvent(value);\n      break;\n    case 90:\n      var value = new proto.ubo.v1.UpdateManagerEvent;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerEvent.deserializeBinaryFromReader);\n      msg.setUpdateManagerEvent(value);\n      break;\n    case 91:\n      var value = new proto.ubo.v1.UpdateManagerUpdateEvent;\n      reader.readMessage(value,proto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinaryFromReader);\n      msg.setUpdateManagerUpdateEvent(value);\n      break;\n    case 92:\n      var value = new proto.ubo.v1.UsersCreateUserEvent;\n      reader.readMessage(value,proto.ubo.v1.UsersCreateUserEvent.deserializeBinaryFromReader);\n      msg.setUsersCreateUserEvent(value);\n      break;\n    case 93:\n      var value = new proto.ubo.v1.UsersDeleteUserEvent;\n      reader.readMessage(value,proto.ubo.v1.UsersDeleteUserEvent.deserializeBinaryFromReader);\n      msg.setUsersDeleteUserEvent(value);\n      break;\n    case 94:\n      var value = new proto.ubo.v1.UsersEvent;\n      reader.readMessage(value,proto.ubo.v1.UsersEvent.deserializeBinaryFromReader);\n      msg.setUsersEvent(value);\n      break;\n    case 95:\n      var value = new proto.ubo.v1.UsersResetPasswordEvent;\n      reader.readMessage(value,proto.ubo.v1.UsersResetPasswordEvent.deserializeBinaryFromReader);\n      msg.setUsersResetPasswordEvent(value);\n      break;\n    case 96:\n      var value = new proto.ubo.v1.VSCodeEvent;\n      reader.readMessage(value,proto.ubo.v1.VSCodeEvent.deserializeBinaryFromReader);\n      msg.setVsCodeEvent(value);\n      break;\n    case 97:\n      var value = new proto.ubo.v1.VSCodeLoginEvent;\n      reader.readMessage(value,proto.ubo.v1.VSCodeLoginEvent.deserializeBinaryFromReader);\n      msg.setVsCodeLoginEvent(value);\n      break;\n    case 98:\n      var value = new proto.ubo.v1.VSCodeRestartEvent;\n      reader.readMessage(value,proto.ubo.v1.VSCodeRestartEvent.deserializeBinaryFromReader);\n      msg.setVsCodeRestartEvent(value);\n      break;\n    case 99:\n      var value = new proto.ubo.v1.WebUIEvent;\n      reader.readMessage(value,proto.ubo.v1.WebUIEvent.deserializeBinaryFromReader);\n      msg.setWebUiEvent(value);\n      break;\n    case 100:\n      var value = new proto.ubo.v1.WebUIInitializeEvent;\n      reader.readMessage(value,proto.ubo.v1.WebUIInitializeEvent.deserializeBinaryFromReader);\n      msg.setWebUiInitializeEvent(value);\n      break;\n    case 101:\n      var value = new proto.ubo.v1.WebUIStopEvent;\n      reader.readMessage(value,proto.ubo.v1.WebUIStopEvent.deserializeBinaryFromReader);\n      msg.setWebUiStopEvent(value);\n      break;\n    case 102:\n      var value = new proto.ubo.v1.WiFiEvent;\n      reader.readMessage(value,proto.ubo.v1.WiFiEvent.deserializeBinaryFromReader);\n      msg.setWiFiEvent(value);\n      break;\n    case 103:\n      var value = new proto.ubo.v1.WiFiInputConnectionEvent;\n      reader.readMessage(value,proto.ubo.v1.WiFiInputConnectionEvent.deserializeBinaryFromReader);\n      msg.setWiFiInputConnectionEvent(value);\n      break;\n    case 104:\n      var value = new proto.ubo.v1.WiFiUpdateRequestEvent;\n      reader.readMessage(value,proto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinaryFromReader);\n      msg.setWiFiUpdateRequestEvent(value);\n      break;\n    default:\n      reader.skipField();\n      break;\n    }\n  }\n  return msg;\n};\n\n\n/**\n * Serializes the message to binary data (in protobuf wire format).\n * @return {!Uint8Array}\n */\nproto.ubo.v1.Event.prototype.serializeBinary = function() {\n  var writer = new jspb.BinaryWriter();\n  proto.ubo.v1.Event.serializeBinaryToWriter(this, writer);\n  return writer.getResultBuffer();\n};\n\n\n/**\n * Serializes the given message to binary data (in protobuf wire\n * format), writing to the given BinaryWriter.\n * @param {!proto.ubo.v1.Event} message\n * @param {!jspb.BinaryWriter} writer\n * @suppress {unusedLocalVariables} f is only used for nested messages\n */\nproto.ubo.v1.Event.serializeBinaryToWriter = function(message, writer) {\n  var f = undefined;\n  f = message.getAssistantAddMcpServerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      1,\n      f,\n      proto.ubo.v1.AssistantAddMcpServerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantDeleteMcpServerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      2,\n      f,\n      proto.ubo.v1.AssistantDeleteMcpServerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantDownloadOllamaModelEvent();\n  if (f != null) {\n    writer.writeMessage(\n      3,\n      f,\n      proto.ubo.v1.AssistantDownloadOllamaModelEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantEvent();\n  if (f != null) {\n    writer.writeMessage(\n      4,\n      f,\n      proto.ubo.v1.AssistantEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantHandleReportEvent();\n  if (f != null) {\n    writer.writeMessage(\n      5,\n      f,\n      proto.ubo.v1.AssistantHandleReportEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAssistantUpdateProvidersEvent();\n  if (f != null) {\n    writer.writeMessage(\n      6,\n      f,\n      proto.ubo.v1.AssistantUpdateProvidersEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioEvent();\n  if (f != null) {\n    writer.writeMessage(\n      7,\n      f,\n      proto.ubo.v1.AudioEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioInstallDriverEvent();\n  if (f != null) {\n    writer.writeMessage(\n      8,\n      f,\n      proto.ubo.v1.AudioInstallDriverEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayAudioSampleEvent();\n  if (f != null) {\n    writer.writeMessage(\n      9,\n      f,\n      proto.ubo.v1.AudioPlayAudioSampleEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayAudioSequenceEvent();\n  if (f != null) {\n    writer.writeMessage(\n      10,\n      f,\n      proto.ubo.v1.AudioPlayAudioSequenceEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlayChimeEvent();\n  if (f != null) {\n    writer.writeMessage(\n      11,\n      f,\n      proto.ubo.v1.AudioPlayChimeEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioPlaybackDoneEvent();\n  if (f != null) {\n    writer.writeMessage(\n      12,\n      f,\n      proto.ubo.v1.AudioPlaybackDoneEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getAudioReportSampleEvent();\n  if (f != null) {\n    writer.writeMessage(\n      13,\n      f,\n      proto.ubo.v1.AudioReportSampleEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraDetectEvent();\n  if (f != null) {\n    writer.writeMessage(\n      14,\n      f,\n      proto.ubo.v1.CameraDetectEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraDetectedEvent();\n  if (f != null) {\n    writer.writeMessage(\n      15,\n      f,\n      proto.ubo.v1.CameraDetectedEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraEvent();\n  if (f != null) {\n    writer.writeMessage(\n      16,\n      f,\n      proto.ubo.v1.CameraEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraInstallDriverEvent();\n  if (f != null) {\n    writer.writeMessage(\n      17,\n      f,\n      proto.ubo.v1.CameraInstallDriverEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraReinitializeEvent();\n  if (f != null) {\n    writer.writeMessage(\n      18,\n      f,\n      proto.ubo.v1.CameraReinitializeEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraReportImageEvent();\n  if (f != null) {\n    writer.writeMessage(\n      19,\n      f,\n      proto.ubo.v1.CameraReportImageEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraRestoreDefaultEvent();\n  if (f != null) {\n    writer.writeMessage(\n      20,\n      f,\n      proto.ubo.v1.CameraRestoreDefaultEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraStartViewfinderEvent();\n  if (f != null) {\n    writer.writeMessage(\n      21,\n      f,\n      proto.ubo.v1.CameraStartViewfinderEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCameraStopViewfinderEvent();\n  if (f != null) {\n    writer.writeMessage(\n      22,\n      f,\n      proto.ubo.v1.CameraStopViewfinderEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getCloseApplicationEvent();\n  if (f != null) {\n    writer.writeMessage(\n      23,\n      f,\n      proto.ubo.v1.CloseApplicationEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayBlankEvent();\n  if (f != null) {\n    writer.writeMessage(\n      24,\n      f,\n      proto.ubo.v1.DisplayBlankEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayCompressedRenderEvent();\n  if (f != null) {\n    writer.writeMessage(\n      25,\n      f,\n      proto.ubo.v1.DisplayCompressedRenderEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayEvent();\n  if (f != null) {\n    writer.writeMessage(\n      26,\n      f,\n      proto.ubo.v1.DisplayEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayRedrawEvent();\n  if (f != null) {\n    writer.writeMessage(\n      27,\n      f,\n      proto.ubo.v1.DisplayRedrawEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayRenderEvent();\n  if (f != null) {\n    writer.writeMessage(\n      28,\n      f,\n      proto.ubo.v1.DisplayRenderEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDisplayUnblankEvent();\n  if (f != null) {\n    writer.writeMessage(\n      29,\n      f,\n      proto.ubo.v1.DisplayUnblankEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      30,\n      f,\n      proto.ubo.v1.DockerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageEvent();\n  if (f != null) {\n    writer.writeMessage(\n      31,\n      f,\n      proto.ubo.v1.DockerImageEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageFetchCompositionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      32,\n      f,\n      proto.ubo.v1.DockerImageFetchCompositionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageFetchEvent();\n  if (f != null) {\n    writer.writeMessage(\n      33,\n      f,\n      proto.ubo.v1.DockerImageFetchEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRegisterAppEvent();\n  if (f != null) {\n    writer.writeMessage(\n      34,\n      f,\n      proto.ubo.v1.DockerImageRegisterAppEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageReleaseCompositionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      35,\n      f,\n      proto.ubo.v1.DockerImageReleaseCompositionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRemoveCompositionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      36,\n      f,\n      proto.ubo.v1.DockerImageRemoveCompositionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRemoveContainerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      37,\n      f,\n      proto.ubo.v1.DockerImageRemoveContainerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRemoveEvent();\n  if (f != null) {\n    writer.writeMessage(\n      38,\n      f,\n      proto.ubo.v1.DockerImageRemoveEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRunCompositionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      39,\n      f,\n      proto.ubo.v1.DockerImageRunCompositionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageRunContainerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      40,\n      f,\n      proto.ubo.v1.DockerImageRunContainerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageStopCompositionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      41,\n      f,\n      proto.ubo.v1.DockerImageStopCompositionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerImageStopContainerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      42,\n      f,\n      proto.ubo.v1.DockerImageStopContainerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerInstallEvent();\n  if (f != null) {\n    writer.writeMessage(\n      43,\n      f,\n      proto.ubo.v1.DockerInstallEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerStartEvent();\n  if (f != null) {\n    writer.writeMessage(\n      44,\n      f,\n      proto.ubo.v1.DockerStartEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getDockerStopEvent();\n  if (f != null) {\n    writer.writeMessage(\n      45,\n      f,\n      proto.ubo.v1.DockerStopEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemCopyEvent();\n  if (f != null) {\n    writer.writeMessage(\n      46,\n      f,\n      proto.ubo.v1.FileSystemCopyEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemEvent();\n  if (f != null) {\n    writer.writeMessage(\n      47,\n      f,\n      proto.ubo.v1.FileSystemEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemMoveEvent();\n  if (f != null) {\n    writer.writeMessage(\n      48,\n      f,\n      proto.ubo.v1.FileSystemMoveEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemRemoveEvent();\n  if (f != null) {\n    writer.writeMessage(\n      49,\n      f,\n      proto.ubo.v1.FileSystemRemoveEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getFileSystemSelectEvent();\n  if (f != null) {\n    writer.writeMessage(\n      50,\n      f,\n      proto.ubo.v1.FileSystemSelectEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredEvent();\n  if (f != null) {\n    writer.writeMessage(\n      51,\n      f,\n      proto.ubo.v1.InfraredEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getInfraredSendCodeEvent();\n  if (f != null) {\n    writer.writeMessage(\n      52,\n      f,\n      proto.ubo.v1.InfraredSendCodeEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getInitEvent();\n  if (f != null) {\n    writer.writeMessage(\n      53,\n      f,\n      proto.ubo.v1.InitEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputCancelEvent();\n  if (f != null) {\n    writer.writeMessage(\n      54,\n      f,\n      proto.ubo.v1.InputCancelEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputProvideEvent();\n  if (f != null) {\n    writer.writeMessage(\n      55,\n      f,\n      proto.ubo.v1.InputProvideEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getInputResolveEvent();\n  if (f != null) {\n    writer.writeMessage(\n      56,\n      f,\n      proto.ubo.v1.InputResolveEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getIpEvent();\n  if (f != null) {\n    writer.writeMessage(\n      57,\n      f,\n      proto.ubo.v1.IpEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMainEvent();\n  if (f != null) {\n    writer.writeMessage(\n      58,\n      f,\n      proto.ubo.v1.MainEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuChooseByIconEvent();\n  if (f != null) {\n    writer.writeMessage(\n      59,\n      f,\n      proto.ubo.v1.MenuChooseByIconEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuChooseByIndexEvent();\n  if (f != null) {\n    writer.writeMessage(\n      60,\n      f,\n      proto.ubo.v1.MenuChooseByIndexEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuChooseByLabelEvent();\n  if (f != null) {\n    writer.writeMessage(\n      61,\n      f,\n      proto.ubo.v1.MenuChooseByLabelEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuEvent();\n  if (f != null) {\n    writer.writeMessage(\n      62,\n      f,\n      proto.ubo.v1.MenuEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuGoBackEvent();\n  if (f != null) {\n    writer.writeMessage(\n      63,\n      f,\n      proto.ubo.v1.MenuGoBackEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuGoHomeEvent();\n  if (f != null) {\n    writer.writeMessage(\n      64,\n      f,\n      proto.ubo.v1.MenuGoHomeEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getMenuScrollEvent();\n  if (f != null) {\n    writer.writeMessage(\n      65,\n      f,\n      proto.ubo.v1.MenuScrollEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsClearEvent();\n  if (f != null) {\n    writer.writeMessage(\n      66,\n      f,\n      proto.ubo.v1.NotificationsClearEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsDisplayEvent();\n  if (f != null) {\n    writer.writeMessage(\n      67,\n      f,\n      proto.ubo.v1.NotificationsDisplayEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getNotificationsEvent();\n  if (f != null) {\n    writer.writeMessage(\n      68,\n      f,\n      proto.ubo.v1.NotificationsEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getOpenApplicationEvent();\n  if (f != null) {\n    writer.writeMessage(\n      69,\n      f,\n      proto.ubo.v1.OpenApplicationEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getPowerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      70,\n      f,\n      proto.ubo.v1.PowerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getPowerOffEvent();\n  if (f != null) {\n    writer.writeMessage(\n      71,\n      f,\n      proto.ubo.v1.PowerOffEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectEvent();\n  if (f != null) {\n    writer.writeMessage(\n      72,\n      f,\n      proto.ubo.v1.RPiConnectEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getRPiConnectLoginEvent();\n  if (f != null) {\n    writer.writeMessage(\n      73,\n      f,\n      proto.ubo.v1.RPiConnectLoginEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getRebootEvent();\n  if (f != null) {\n    writer.writeMessage(\n      74,\n      f,\n      proto.ubo.v1.RebootEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getReplayRecordedSequenceEvent();\n  if (f != null) {\n    writer.writeMessage(\n      75,\n      f,\n      proto.ubo.v1.ReplayRecordedSequenceEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingCommandEvent();\n  if (f != null) {\n    writer.writeMessage(\n      76,\n      f,\n      proto.ubo.v1.RgbRingCommandEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getRgbRingEvent();\n  if (f != null) {\n    writer.writeMessage(\n      77,\n      f,\n      proto.ubo.v1.RgbRingEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getScreenshotEvent();\n  if (f != null) {\n    writer.writeMessage(\n      78,\n      f,\n      proto.ubo.v1.ScreenshotEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsEvent();\n  if (f != null) {\n    writer.writeMessage(\n      79,\n      f,\n      proto.ubo.v1.SettingsEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsServiceEvent();\n  if (f != null) {\n    writer.writeMessage(\n      80,\n      f,\n      proto.ubo.v1.SettingsServiceEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsStartServiceEvent();\n  if (f != null) {\n    writer.writeMessage(\n      81,\n      f,\n      proto.ubo.v1.SettingsStartServiceEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSettingsStopServiceEvent();\n  if (f != null) {\n    writer.writeMessage(\n      82,\n      f,\n      proto.ubo.v1.SettingsStopServiceEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSnapshotEvent();\n  if (f != null) {\n    writer.writeMessage(\n      83,\n      f,\n      proto.ubo.v1.SnapshotEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      84,\n      f,\n      proto.ubo.v1.SpeechRecognitionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechRecognitionReportTextEvent();\n  if (f != null) {\n    writer.writeMessage(\n      85,\n      f,\n      proto.ubo.v1.SpeechRecognitionReportTextEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechSynthesisEvent();\n  if (f != null) {\n    writer.writeMessage(\n      86,\n      f,\n      proto.ubo.v1.SpeechSynthesisEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getSpeechSynthesisSynthesizeTextEvent();\n  if (f != null) {\n    writer.writeMessage(\n      87,\n      f,\n      proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getStoreRecordedSequenceEvent();\n  if (f != null) {\n    writer.writeMessage(\n      88,\n      f,\n      proto.ubo.v1.StoreRecordedSequenceEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerCheckEvent();\n  if (f != null) {\n    writer.writeMessage(\n      89,\n      f,\n      proto.ubo.v1.UpdateManagerCheckEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerEvent();\n  if (f != null) {\n    writer.writeMessage(\n      90,\n      f,\n      proto.ubo.v1.UpdateManagerEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUpdateManagerUpdateEvent();\n  if (f != null) {\n    writer.writeMessage(\n      91,\n      f,\n      proto.ubo.v1.UpdateManagerUpdateEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersCreateUserEvent();\n  if (f != null) {\n    writer.writeMessage(\n      92,\n      f,\n      proto.ubo.v1.UsersCreateUserEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersDeleteUserEvent();\n  if (f != null) {\n    writer.writeMessage(\n      93,\n      f,\n      proto.ubo.v1.UsersDeleteUserEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersEvent();\n  if (f != null) {\n    writer.writeMessage(\n      94,\n      f,\n      proto.ubo.v1.UsersEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getUsersResetPasswordEvent();\n  if (f != null) {\n    writer.writeMessage(\n      95,\n      f,\n      proto.ubo.v1.UsersResetPasswordEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeEvent();\n  if (f != null) {\n    writer.writeMessage(\n      96,\n      f,\n      proto.ubo.v1.VSCodeEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeLoginEvent();\n  if (f != null) {\n    writer.writeMessage(\n      97,\n      f,\n      proto.ubo.v1.VSCodeLoginEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getVsCodeRestartEvent();\n  if (f != null) {\n    writer.writeMessage(\n      98,\n      f,\n      proto.ubo.v1.VSCodeRestartEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getWebUiEvent();\n  if (f != null) {\n    writer.writeMessage(\n      99,\n      f,\n      proto.ubo.v1.WebUIEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getWebUiInitializeEvent();\n  if (f != null) {\n    writer.writeMessage(\n      100,\n      f,\n      proto.ubo.v1.WebUIInitializeEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getWebUiStopEvent();\n  if (f != null) {\n    writer.writeMessage(\n      101,\n      f,\n      proto.ubo.v1.WebUIStopEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiEvent();\n  if (f != null) {\n    writer.writeMessage(\n      102,\n      f,\n      proto.ubo.v1.WiFiEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiInputConnectionEvent();\n  if (f != null) {\n    writer.writeMessage(\n      103,\n      f,\n      proto.ubo.v1.WiFiInputConnectionEvent.serializeBinaryToWriter\n    );\n  }\n  f = message.getWiFiUpdateRequestEvent();\n  if (f != null) {\n    writer.writeMessage(\n      104,\n      f,\n      proto.ubo.v1.WiFiUpdateRequestEvent.serializeBinaryToWriter\n    );\n  }\n};\n\n\n/**\n * optional AssistantAddMcpServerEvent assistant_add_mcp_server_event = 1;\n * @return {?proto.ubo.v1.AssistantAddMcpServerEvent}\n */\nproto.ubo.v1.Event.prototype.getAssistantAddMcpServerEvent = function() {\n  return /** @type{?proto.ubo.v1.AssistantAddMcpServerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantAddMcpServerEvent, 1));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantAddMcpServerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAssistantAddMcpServerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAssistantAddMcpServerEvent = function() {\n  return this.setAssistantAddMcpServerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAssistantAddMcpServerEvent = function() {\n  return jspb.Message.getField(this, 1) != null;\n};\n\n\n/**\n * optional AssistantDeleteMcpServerEvent assistant_delete_mcp_server_event = 2;\n * @return {?proto.ubo.v1.AssistantDeleteMcpServerEvent}\n */\nproto.ubo.v1.Event.prototype.getAssistantDeleteMcpServerEvent = function() {\n  return /** @type{?proto.ubo.v1.AssistantDeleteMcpServerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDeleteMcpServerEvent, 2));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantDeleteMcpServerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAssistantDeleteMcpServerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAssistantDeleteMcpServerEvent = function() {\n  return this.setAssistantDeleteMcpServerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAssistantDeleteMcpServerEvent = function() {\n  return jspb.Message.getField(this, 2) != null;\n};\n\n\n/**\n * optional AssistantDownloadOllamaModelEvent assistant_download_ollama_model_event = 3;\n * @return {?proto.ubo.v1.AssistantDownloadOllamaModelEvent}\n */\nproto.ubo.v1.Event.prototype.getAssistantDownloadOllamaModelEvent = function() {\n  return /** @type{?proto.ubo.v1.AssistantDownloadOllamaModelEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDownloadOllamaModelEvent, 3));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantDownloadOllamaModelEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAssistantDownloadOllamaModelEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAssistantDownloadOllamaModelEvent = function() {\n  return this.setAssistantDownloadOllamaModelEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAssistantDownloadOllamaModelEvent = function() {\n  return jspb.Message.getField(this, 3) != null;\n};\n\n\n/**\n * optional AssistantEvent assistant_event = 4;\n * @return {?proto.ubo.v1.AssistantEvent}\n */\nproto.ubo.v1.Event.prototype.getAssistantEvent = function() {\n  return /** @type{?proto.ubo.v1.AssistantEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantEvent, 4));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAssistantEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 4, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAssistantEvent = function() {\n  return this.setAssistantEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAssistantEvent = function() {\n  return jspb.Message.getField(this, 4) != null;\n};\n\n\n/**\n * optional AssistantHandleReportEvent assistant_handle_report_event = 5;\n * @return {?proto.ubo.v1.AssistantHandleReportEvent}\n */\nproto.ubo.v1.Event.prototype.getAssistantHandleReportEvent = function() {\n  return /** @type{?proto.ubo.v1.AssistantHandleReportEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantHandleReportEvent, 5));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantHandleReportEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAssistantHandleReportEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 5, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAssistantHandleReportEvent = function() {\n  return this.setAssistantHandleReportEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAssistantHandleReportEvent = function() {\n  return jspb.Message.getField(this, 5) != null;\n};\n\n\n/**\n * optional AssistantUpdateProvidersEvent assistant_update_providers_event = 6;\n * @return {?proto.ubo.v1.AssistantUpdateProvidersEvent}\n */\nproto.ubo.v1.Event.prototype.getAssistantUpdateProvidersEvent = function() {\n  return /** @type{?proto.ubo.v1.AssistantUpdateProvidersEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantUpdateProvidersEvent, 6));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AssistantUpdateProvidersEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAssistantUpdateProvidersEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 6, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAssistantUpdateProvidersEvent = function() {\n  return this.setAssistantUpdateProvidersEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAssistantUpdateProvidersEvent = function() {\n  return jspb.Message.getField(this, 6) != null;\n};\n\n\n/**\n * optional AudioEvent audio_event = 7;\n * @return {?proto.ubo.v1.AudioEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioEvent, 7));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 7, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioEvent = function() {\n  return this.setAudioEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioEvent = function() {\n  return jspb.Message.getField(this, 7) != null;\n};\n\n\n/**\n * optional AudioInstallDriverEvent audio_install_driver_event = 8;\n * @return {?proto.ubo.v1.AudioInstallDriverEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioInstallDriverEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioInstallDriverEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioInstallDriverEvent, 8));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioInstallDriverEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioInstallDriverEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 8, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioInstallDriverEvent = function() {\n  return this.setAudioInstallDriverEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioInstallDriverEvent = function() {\n  return jspb.Message.getField(this, 8) != null;\n};\n\n\n/**\n * optional AudioPlayAudioSampleEvent audio_play_audio_sample_event = 9;\n * @return {?proto.ubo.v1.AudioPlayAudioSampleEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioPlayAudioSampleEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayAudioSampleEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSampleEvent, 9));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayAudioSampleEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioPlayAudioSampleEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 9, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioPlayAudioSampleEvent = function() {\n  return this.setAudioPlayAudioSampleEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioPlayAudioSampleEvent = function() {\n  return jspb.Message.getField(this, 9) != null;\n};\n\n\n/**\n * optional AudioPlayAudioSequenceEvent audio_play_audio_sequence_event = 10;\n * @return {?proto.ubo.v1.AudioPlayAudioSequenceEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioPlayAudioSequenceEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayAudioSequenceEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSequenceEvent, 10));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayAudioSequenceEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioPlayAudioSequenceEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 10, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioPlayAudioSequenceEvent = function() {\n  return this.setAudioPlayAudioSequenceEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioPlayAudioSequenceEvent = function() {\n  return jspb.Message.getField(this, 10) != null;\n};\n\n\n/**\n * optional AudioPlayChimeEvent audio_play_chime_event = 11;\n * @return {?proto.ubo.v1.AudioPlayChimeEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioPlayChimeEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioPlayChimeEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayChimeEvent, 11));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlayChimeEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioPlayChimeEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 11, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioPlayChimeEvent = function() {\n  return this.setAudioPlayChimeEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioPlayChimeEvent = function() {\n  return jspb.Message.getField(this, 11) != null;\n};\n\n\n/**\n * optional AudioPlaybackDoneEvent audio_playback_done_event = 12;\n * @return {?proto.ubo.v1.AudioPlaybackDoneEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioPlaybackDoneEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioPlaybackDoneEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlaybackDoneEvent, 12));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioPlaybackDoneEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioPlaybackDoneEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 12, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioPlaybackDoneEvent = function() {\n  return this.setAudioPlaybackDoneEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioPlaybackDoneEvent = function() {\n  return jspb.Message.getField(this, 12) != null;\n};\n\n\n/**\n * optional AudioReportSampleEvent audio_report_sample_event = 13;\n * @return {?proto.ubo.v1.AudioReportSampleEvent}\n */\nproto.ubo.v1.Event.prototype.getAudioReportSampleEvent = function() {\n  return /** @type{?proto.ubo.v1.AudioReportSampleEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioReportSampleEvent, 13));\n};\n\n\n/**\n * @param {?proto.ubo.v1.AudioReportSampleEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setAudioReportSampleEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 13, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearAudioReportSampleEvent = function() {\n  return this.setAudioReportSampleEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasAudioReportSampleEvent = function() {\n  return jspb.Message.getField(this, 13) != null;\n};\n\n\n/**\n * optional CameraDetectEvent camera_detect_event = 14;\n * @return {?proto.ubo.v1.CameraDetectEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraDetectEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraDetectEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraDetectEvent, 14));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraDetectEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraDetectEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 14, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraDetectEvent = function() {\n  return this.setCameraDetectEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraDetectEvent = function() {\n  return jspb.Message.getField(this, 14) != null;\n};\n\n\n/**\n * optional CameraDetectedEvent camera_detected_event = 15;\n * @return {?proto.ubo.v1.CameraDetectedEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraDetectedEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraDetectedEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraDetectedEvent, 15));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraDetectedEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraDetectedEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 15, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraDetectedEvent = function() {\n  return this.setCameraDetectedEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraDetectedEvent = function() {\n  return jspb.Message.getField(this, 15) != null;\n};\n\n\n/**\n * optional CameraEvent camera_event = 16;\n * @return {?proto.ubo.v1.CameraEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraEvent, 16));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 16, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraEvent = function() {\n  return this.setCameraEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraEvent = function() {\n  return jspb.Message.getField(this, 16) != null;\n};\n\n\n/**\n * optional CameraInstallDriverEvent camera_install_driver_event = 17;\n * @return {?proto.ubo.v1.CameraInstallDriverEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraInstallDriverEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraInstallDriverEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraInstallDriverEvent, 17));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraInstallDriverEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraInstallDriverEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 17, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraInstallDriverEvent = function() {\n  return this.setCameraInstallDriverEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraInstallDriverEvent = function() {\n  return jspb.Message.getField(this, 17) != null;\n};\n\n\n/**\n * optional CameraReinitializeEvent camera_reinitialize_event = 18;\n * @return {?proto.ubo.v1.CameraReinitializeEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraReinitializeEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraReinitializeEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraReinitializeEvent, 18));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraReinitializeEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraReinitializeEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 18, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraReinitializeEvent = function() {\n  return this.setCameraReinitializeEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraReinitializeEvent = function() {\n  return jspb.Message.getField(this, 18) != null;\n};\n\n\n/**\n * optional CameraReportImageEvent camera_report_image_event = 19;\n * @return {?proto.ubo.v1.CameraReportImageEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraReportImageEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraReportImageEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraReportImageEvent, 19));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraReportImageEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraReportImageEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 19, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraReportImageEvent = function() {\n  return this.setCameraReportImageEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraReportImageEvent = function() {\n  return jspb.Message.getField(this, 19) != null;\n};\n\n\n/**\n * optional CameraRestoreDefaultEvent camera_restore_default_event = 20;\n * @return {?proto.ubo.v1.CameraRestoreDefaultEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraRestoreDefaultEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraRestoreDefaultEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraRestoreDefaultEvent, 20));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraRestoreDefaultEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraRestoreDefaultEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 20, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraRestoreDefaultEvent = function() {\n  return this.setCameraRestoreDefaultEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraRestoreDefaultEvent = function() {\n  return jspb.Message.getField(this, 20) != null;\n};\n\n\n/**\n * optional CameraStartViewfinderEvent camera_start_viewfinder_event = 21;\n * @return {?proto.ubo.v1.CameraStartViewfinderEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraStartViewfinderEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraStartViewfinderEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraStartViewfinderEvent, 21));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraStartViewfinderEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraStartViewfinderEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 21, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraStartViewfinderEvent = function() {\n  return this.setCameraStartViewfinderEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraStartViewfinderEvent = function() {\n  return jspb.Message.getField(this, 21) != null;\n};\n\n\n/**\n * optional CameraStopViewfinderEvent camera_stop_viewfinder_event = 22;\n * @return {?proto.ubo.v1.CameraStopViewfinderEvent}\n */\nproto.ubo.v1.Event.prototype.getCameraStopViewfinderEvent = function() {\n  return /** @type{?proto.ubo.v1.CameraStopViewfinderEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraStopViewfinderEvent, 22));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CameraStopViewfinderEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCameraStopViewfinderEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 22, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCameraStopViewfinderEvent = function() {\n  return this.setCameraStopViewfinderEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCameraStopViewfinderEvent = function() {\n  return jspb.Message.getField(this, 22) != null;\n};\n\n\n/**\n * optional CloseApplicationEvent close_application_event = 23;\n * @return {?proto.ubo.v1.CloseApplicationEvent}\n */\nproto.ubo.v1.Event.prototype.getCloseApplicationEvent = function() {\n  return /** @type{?proto.ubo.v1.CloseApplicationEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.CloseApplicationEvent, 23));\n};\n\n\n/**\n * @param {?proto.ubo.v1.CloseApplicationEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setCloseApplicationEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 23, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearCloseApplicationEvent = function() {\n  return this.setCloseApplicationEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasCloseApplicationEvent = function() {\n  return jspb.Message.getField(this, 23) != null;\n};\n\n\n/**\n * optional DisplayBlankEvent display_blank_event = 24;\n * @return {?proto.ubo.v1.DisplayBlankEvent}\n */\nproto.ubo.v1.Event.prototype.getDisplayBlankEvent = function() {\n  return /** @type{?proto.ubo.v1.DisplayBlankEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayBlankEvent, 24));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayBlankEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDisplayBlankEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 24, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDisplayBlankEvent = function() {\n  return this.setDisplayBlankEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDisplayBlankEvent = function() {\n  return jspb.Message.getField(this, 24) != null;\n};\n\n\n/**\n * optional DisplayCompressedRenderEvent display_compressed_render_event = 25;\n * @return {?proto.ubo.v1.DisplayCompressedRenderEvent}\n */\nproto.ubo.v1.Event.prototype.getDisplayCompressedRenderEvent = function() {\n  return /** @type{?proto.ubo.v1.DisplayCompressedRenderEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayCompressedRenderEvent, 25));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayCompressedRenderEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDisplayCompressedRenderEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 25, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDisplayCompressedRenderEvent = function() {\n  return this.setDisplayCompressedRenderEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDisplayCompressedRenderEvent = function() {\n  return jspb.Message.getField(this, 25) != null;\n};\n\n\n/**\n * optional DisplayEvent display_event = 26;\n * @return {?proto.ubo.v1.DisplayEvent}\n */\nproto.ubo.v1.Event.prototype.getDisplayEvent = function() {\n  return /** @type{?proto.ubo.v1.DisplayEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayEvent, 26));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDisplayEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 26, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDisplayEvent = function() {\n  return this.setDisplayEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDisplayEvent = function() {\n  return jspb.Message.getField(this, 26) != null;\n};\n\n\n/**\n * optional DisplayRedrawEvent display_redraw_event = 27;\n * @return {?proto.ubo.v1.DisplayRedrawEvent}\n */\nproto.ubo.v1.Event.prototype.getDisplayRedrawEvent = function() {\n  return /** @type{?proto.ubo.v1.DisplayRedrawEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayRedrawEvent, 27));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayRedrawEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDisplayRedrawEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 27, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDisplayRedrawEvent = function() {\n  return this.setDisplayRedrawEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDisplayRedrawEvent = function() {\n  return jspb.Message.getField(this, 27) != null;\n};\n\n\n/**\n * optional DisplayRenderEvent display_render_event = 28;\n * @return {?proto.ubo.v1.DisplayRenderEvent}\n */\nproto.ubo.v1.Event.prototype.getDisplayRenderEvent = function() {\n  return /** @type{?proto.ubo.v1.DisplayRenderEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayRenderEvent, 28));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayRenderEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDisplayRenderEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 28, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDisplayRenderEvent = function() {\n  return this.setDisplayRenderEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDisplayRenderEvent = function() {\n  return jspb.Message.getField(this, 28) != null;\n};\n\n\n/**\n * optional DisplayUnblankEvent display_unblank_event = 29;\n * @return {?proto.ubo.v1.DisplayUnblankEvent}\n */\nproto.ubo.v1.Event.prototype.getDisplayUnblankEvent = function() {\n  return /** @type{?proto.ubo.v1.DisplayUnblankEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayUnblankEvent, 29));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DisplayUnblankEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDisplayUnblankEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 29, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDisplayUnblankEvent = function() {\n  return this.setDisplayUnblankEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDisplayUnblankEvent = function() {\n  return jspb.Message.getField(this, 29) != null;\n};\n\n\n/**\n * optional DockerEvent docker_event = 30;\n * @return {?proto.ubo.v1.DockerEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerEvent, 30));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 30, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerEvent = function() {\n  return this.setDockerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerEvent = function() {\n  return jspb.Message.getField(this, 30) != null;\n};\n\n\n/**\n * optional DockerImageEvent docker_image_event = 31;\n * @return {?proto.ubo.v1.DockerImageEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageEvent, 31));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 31, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageEvent = function() {\n  return this.setDockerImageEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageEvent = function() {\n  return jspb.Message.getField(this, 31) != null;\n};\n\n\n/**\n * optional DockerImageFetchCompositionEvent docker_image_fetch_composition_event = 32;\n * @return {?proto.ubo.v1.DockerImageFetchCompositionEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageFetchCompositionEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageFetchCompositionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageFetchCompositionEvent, 32));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageFetchCompositionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageFetchCompositionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 32, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageFetchCompositionEvent = function() {\n  return this.setDockerImageFetchCompositionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageFetchCompositionEvent = function() {\n  return jspb.Message.getField(this, 32) != null;\n};\n\n\n/**\n * optional DockerImageFetchEvent docker_image_fetch_event = 33;\n * @return {?proto.ubo.v1.DockerImageFetchEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageFetchEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageFetchEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageFetchEvent, 33));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageFetchEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageFetchEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 33, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageFetchEvent = function() {\n  return this.setDockerImageFetchEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageFetchEvent = function() {\n  return jspb.Message.getField(this, 33) != null;\n};\n\n\n/**\n * optional DockerImageRegisterAppEvent docker_image_register_app_event = 34;\n * @return {?proto.ubo.v1.DockerImageRegisterAppEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageRegisterAppEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRegisterAppEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRegisterAppEvent, 34));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRegisterAppEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageRegisterAppEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 34, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageRegisterAppEvent = function() {\n  return this.setDockerImageRegisterAppEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageRegisterAppEvent = function() {\n  return jspb.Message.getField(this, 34) != null;\n};\n\n\n/**\n * optional DockerImageReleaseCompositionEvent docker_image_release_composition_event = 35;\n * @return {?proto.ubo.v1.DockerImageReleaseCompositionEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageReleaseCompositionEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageReleaseCompositionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageReleaseCompositionEvent, 35));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageReleaseCompositionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageReleaseCompositionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 35, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageReleaseCompositionEvent = function() {\n  return this.setDockerImageReleaseCompositionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageReleaseCompositionEvent = function() {\n  return jspb.Message.getField(this, 35) != null;\n};\n\n\n/**\n * optional DockerImageRemoveCompositionEvent docker_image_remove_composition_event = 36;\n * @return {?proto.ubo.v1.DockerImageRemoveCompositionEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageRemoveCompositionEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRemoveCompositionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveCompositionEvent, 36));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRemoveCompositionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageRemoveCompositionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 36, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageRemoveCompositionEvent = function() {\n  return this.setDockerImageRemoveCompositionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageRemoveCompositionEvent = function() {\n  return jspb.Message.getField(this, 36) != null;\n};\n\n\n/**\n * optional DockerImageRemoveContainerEvent docker_image_remove_container_event = 37;\n * @return {?proto.ubo.v1.DockerImageRemoveContainerEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageRemoveContainerEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRemoveContainerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveContainerEvent, 37));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRemoveContainerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageRemoveContainerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 37, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageRemoveContainerEvent = function() {\n  return this.setDockerImageRemoveContainerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageRemoveContainerEvent = function() {\n  return jspb.Message.getField(this, 37) != null;\n};\n\n\n/**\n * optional DockerImageRemoveEvent docker_image_remove_event = 38;\n * @return {?proto.ubo.v1.DockerImageRemoveEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageRemoveEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRemoveEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveEvent, 38));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRemoveEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageRemoveEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 38, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageRemoveEvent = function() {\n  return this.setDockerImageRemoveEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageRemoveEvent = function() {\n  return jspb.Message.getField(this, 38) != null;\n};\n\n\n/**\n * optional DockerImageRunCompositionEvent docker_image_run_composition_event = 39;\n * @return {?proto.ubo.v1.DockerImageRunCompositionEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageRunCompositionEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRunCompositionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRunCompositionEvent, 39));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRunCompositionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageRunCompositionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 39, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageRunCompositionEvent = function() {\n  return this.setDockerImageRunCompositionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageRunCompositionEvent = function() {\n  return jspb.Message.getField(this, 39) != null;\n};\n\n\n/**\n * optional DockerImageRunContainerEvent docker_image_run_container_event = 40;\n * @return {?proto.ubo.v1.DockerImageRunContainerEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageRunContainerEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageRunContainerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRunContainerEvent, 40));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageRunContainerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageRunContainerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 40, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageRunContainerEvent = function() {\n  return this.setDockerImageRunContainerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageRunContainerEvent = function() {\n  return jspb.Message.getField(this, 40) != null;\n};\n\n\n/**\n * optional DockerImageStopCompositionEvent docker_image_stop_composition_event = 41;\n * @return {?proto.ubo.v1.DockerImageStopCompositionEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageStopCompositionEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageStopCompositionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageStopCompositionEvent, 41));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageStopCompositionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageStopCompositionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 41, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageStopCompositionEvent = function() {\n  return this.setDockerImageStopCompositionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageStopCompositionEvent = function() {\n  return jspb.Message.getField(this, 41) != null;\n};\n\n\n/**\n * optional DockerImageStopContainerEvent docker_image_stop_container_event = 42;\n * @return {?proto.ubo.v1.DockerImageStopContainerEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerImageStopContainerEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerImageStopContainerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageStopContainerEvent, 42));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerImageStopContainerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerImageStopContainerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 42, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerImageStopContainerEvent = function() {\n  return this.setDockerImageStopContainerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerImageStopContainerEvent = function() {\n  return jspb.Message.getField(this, 42) != null;\n};\n\n\n/**\n * optional DockerInstallEvent docker_install_event = 43;\n * @return {?proto.ubo.v1.DockerInstallEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerInstallEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerInstallEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerInstallEvent, 43));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerInstallEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerInstallEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 43, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerInstallEvent = function() {\n  return this.setDockerInstallEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerInstallEvent = function() {\n  return jspb.Message.getField(this, 43) != null;\n};\n\n\n/**\n * optional DockerStartEvent docker_start_event = 44;\n * @return {?proto.ubo.v1.DockerStartEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerStartEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerStartEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStartEvent, 44));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerStartEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerStartEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 44, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerStartEvent = function() {\n  return this.setDockerStartEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerStartEvent = function() {\n  return jspb.Message.getField(this, 44) != null;\n};\n\n\n/**\n * optional DockerStopEvent docker_stop_event = 45;\n * @return {?proto.ubo.v1.DockerStopEvent}\n */\nproto.ubo.v1.Event.prototype.getDockerStopEvent = function() {\n  return /** @type{?proto.ubo.v1.DockerStopEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStopEvent, 45));\n};\n\n\n/**\n * @param {?proto.ubo.v1.DockerStopEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setDockerStopEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 45, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearDockerStopEvent = function() {\n  return this.setDockerStopEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasDockerStopEvent = function() {\n  return jspb.Message.getField(this, 45) != null;\n};\n\n\n/**\n * optional FileSystemCopyEvent file_system_copy_event = 46;\n * @return {?proto.ubo.v1.FileSystemCopyEvent}\n */\nproto.ubo.v1.Event.prototype.getFileSystemCopyEvent = function() {\n  return /** @type{?proto.ubo.v1.FileSystemCopyEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemCopyEvent, 46));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemCopyEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setFileSystemCopyEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 46, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearFileSystemCopyEvent = function() {\n  return this.setFileSystemCopyEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasFileSystemCopyEvent = function() {\n  return jspb.Message.getField(this, 46) != null;\n};\n\n\n/**\n * optional FileSystemEvent file_system_event = 47;\n * @return {?proto.ubo.v1.FileSystemEvent}\n */\nproto.ubo.v1.Event.prototype.getFileSystemEvent = function() {\n  return /** @type{?proto.ubo.v1.FileSystemEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemEvent, 47));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setFileSystemEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 47, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearFileSystemEvent = function() {\n  return this.setFileSystemEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasFileSystemEvent = function() {\n  return jspb.Message.getField(this, 47) != null;\n};\n\n\n/**\n * optional FileSystemMoveEvent file_system_move_event = 48;\n * @return {?proto.ubo.v1.FileSystemMoveEvent}\n */\nproto.ubo.v1.Event.prototype.getFileSystemMoveEvent = function() {\n  return /** @type{?proto.ubo.v1.FileSystemMoveEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemMoveEvent, 48));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemMoveEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setFileSystemMoveEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 48, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearFileSystemMoveEvent = function() {\n  return this.setFileSystemMoveEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasFileSystemMoveEvent = function() {\n  return jspb.Message.getField(this, 48) != null;\n};\n\n\n/**\n * optional FileSystemRemoveEvent file_system_remove_event = 49;\n * @return {?proto.ubo.v1.FileSystemRemoveEvent}\n */\nproto.ubo.v1.Event.prototype.getFileSystemRemoveEvent = function() {\n  return /** @type{?proto.ubo.v1.FileSystemRemoveEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemRemoveEvent, 49));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemRemoveEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setFileSystemRemoveEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 49, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearFileSystemRemoveEvent = function() {\n  return this.setFileSystemRemoveEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasFileSystemRemoveEvent = function() {\n  return jspb.Message.getField(this, 49) != null;\n};\n\n\n/**\n * optional FileSystemSelectEvent file_system_select_event = 50;\n * @return {?proto.ubo.v1.FileSystemSelectEvent}\n */\nproto.ubo.v1.Event.prototype.getFileSystemSelectEvent = function() {\n  return /** @type{?proto.ubo.v1.FileSystemSelectEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemSelectEvent, 50));\n};\n\n\n/**\n * @param {?proto.ubo.v1.FileSystemSelectEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setFileSystemSelectEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 50, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearFileSystemSelectEvent = function() {\n  return this.setFileSystemSelectEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasFileSystemSelectEvent = function() {\n  return jspb.Message.getField(this, 50) != null;\n};\n\n\n/**\n * optional InfraredEvent infrared_event = 51;\n * @return {?proto.ubo.v1.InfraredEvent}\n */\nproto.ubo.v1.Event.prototype.getInfraredEvent = function() {\n  return /** @type{?proto.ubo.v1.InfraredEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredEvent, 51));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setInfraredEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 51, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearInfraredEvent = function() {\n  return this.setInfraredEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasInfraredEvent = function() {\n  return jspb.Message.getField(this, 51) != null;\n};\n\n\n/**\n * optional InfraredSendCodeEvent infrared_send_code_event = 52;\n * @return {?proto.ubo.v1.InfraredSendCodeEvent}\n */\nproto.ubo.v1.Event.prototype.getInfraredSendCodeEvent = function() {\n  return /** @type{?proto.ubo.v1.InfraredSendCodeEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSendCodeEvent, 52));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InfraredSendCodeEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setInfraredSendCodeEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 52, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearInfraredSendCodeEvent = function() {\n  return this.setInfraredSendCodeEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasInfraredSendCodeEvent = function() {\n  return jspb.Message.getField(this, 52) != null;\n};\n\n\n/**\n * optional InitEvent init_event = 53;\n * @return {?proto.ubo.v1.InitEvent}\n */\nproto.ubo.v1.Event.prototype.getInitEvent = function() {\n  return /** @type{?proto.ubo.v1.InitEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InitEvent, 53));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InitEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setInitEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 53, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearInitEvent = function() {\n  return this.setInitEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasInitEvent = function() {\n  return jspb.Message.getField(this, 53) != null;\n};\n\n\n/**\n * optional InputCancelEvent input_cancel_event = 54;\n * @return {?proto.ubo.v1.InputCancelEvent}\n */\nproto.ubo.v1.Event.prototype.getInputCancelEvent = function() {\n  return /** @type{?proto.ubo.v1.InputCancelEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputCancelEvent, 54));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputCancelEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setInputCancelEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 54, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearInputCancelEvent = function() {\n  return this.setInputCancelEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasInputCancelEvent = function() {\n  return jspb.Message.getField(this, 54) != null;\n};\n\n\n/**\n * optional InputProvideEvent input_provide_event = 55;\n * @return {?proto.ubo.v1.InputProvideEvent}\n */\nproto.ubo.v1.Event.prototype.getInputProvideEvent = function() {\n  return /** @type{?proto.ubo.v1.InputProvideEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputProvideEvent, 55));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputProvideEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setInputProvideEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 55, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearInputProvideEvent = function() {\n  return this.setInputProvideEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasInputProvideEvent = function() {\n  return jspb.Message.getField(this, 55) != null;\n};\n\n\n/**\n * optional InputResolveEvent input_resolve_event = 56;\n * @return {?proto.ubo.v1.InputResolveEvent}\n */\nproto.ubo.v1.Event.prototype.getInputResolveEvent = function() {\n  return /** @type{?proto.ubo.v1.InputResolveEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResolveEvent, 56));\n};\n\n\n/**\n * @param {?proto.ubo.v1.InputResolveEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setInputResolveEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 56, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearInputResolveEvent = function() {\n  return this.setInputResolveEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasInputResolveEvent = function() {\n  return jspb.Message.getField(this, 56) != null;\n};\n\n\n/**\n * optional IpEvent ip_event = 57;\n * @return {?proto.ubo.v1.IpEvent}\n */\nproto.ubo.v1.Event.prototype.getIpEvent = function() {\n  return /** @type{?proto.ubo.v1.IpEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.IpEvent, 57));\n};\n\n\n/**\n * @param {?proto.ubo.v1.IpEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setIpEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 57, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearIpEvent = function() {\n  return this.setIpEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasIpEvent = function() {\n  return jspb.Message.getField(this, 57) != null;\n};\n\n\n/**\n * optional MainEvent main_event = 58;\n * @return {?proto.ubo.v1.MainEvent}\n */\nproto.ubo.v1.Event.prototype.getMainEvent = function() {\n  return /** @type{?proto.ubo.v1.MainEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MainEvent, 58));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MainEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMainEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 58, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMainEvent = function() {\n  return this.setMainEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMainEvent = function() {\n  return jspb.Message.getField(this, 58) != null;\n};\n\n\n/**\n * optional MenuChooseByIconEvent menu_choose_by_icon_event = 59;\n * @return {?proto.ubo.v1.MenuChooseByIconEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuChooseByIconEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuChooseByIconEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIconEvent, 59));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuChooseByIconEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuChooseByIconEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 59, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuChooseByIconEvent = function() {\n  return this.setMenuChooseByIconEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuChooseByIconEvent = function() {\n  return jspb.Message.getField(this, 59) != null;\n};\n\n\n/**\n * optional MenuChooseByIndexEvent menu_choose_by_index_event = 60;\n * @return {?proto.ubo.v1.MenuChooseByIndexEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuChooseByIndexEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuChooseByIndexEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIndexEvent, 60));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuChooseByIndexEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuChooseByIndexEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 60, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuChooseByIndexEvent = function() {\n  return this.setMenuChooseByIndexEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuChooseByIndexEvent = function() {\n  return jspb.Message.getField(this, 60) != null;\n};\n\n\n/**\n * optional MenuChooseByLabelEvent menu_choose_by_label_event = 61;\n * @return {?proto.ubo.v1.MenuChooseByLabelEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuChooseByLabelEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuChooseByLabelEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByLabelEvent, 61));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuChooseByLabelEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuChooseByLabelEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 61, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuChooseByLabelEvent = function() {\n  return this.setMenuChooseByLabelEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuChooseByLabelEvent = function() {\n  return jspb.Message.getField(this, 61) != null;\n};\n\n\n/**\n * optional MenuEvent menu_event = 62;\n * @return {?proto.ubo.v1.MenuEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuEvent, 62));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 62, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuEvent = function() {\n  return this.setMenuEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuEvent = function() {\n  return jspb.Message.getField(this, 62) != null;\n};\n\n\n/**\n * optional MenuGoBackEvent menu_go_back_event = 63;\n * @return {?proto.ubo.v1.MenuGoBackEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuGoBackEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuGoBackEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoBackEvent, 63));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuGoBackEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuGoBackEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 63, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuGoBackEvent = function() {\n  return this.setMenuGoBackEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuGoBackEvent = function() {\n  return jspb.Message.getField(this, 63) != null;\n};\n\n\n/**\n * optional MenuGoHomeEvent menu_go_home_event = 64;\n * @return {?proto.ubo.v1.MenuGoHomeEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuGoHomeEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuGoHomeEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoHomeEvent, 64));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuGoHomeEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuGoHomeEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 64, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuGoHomeEvent = function() {\n  return this.setMenuGoHomeEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuGoHomeEvent = function() {\n  return jspb.Message.getField(this, 64) != null;\n};\n\n\n/**\n * optional MenuScrollEvent menu_scroll_event = 65;\n * @return {?proto.ubo.v1.MenuScrollEvent}\n */\nproto.ubo.v1.Event.prototype.getMenuScrollEvent = function() {\n  return /** @type{?proto.ubo.v1.MenuScrollEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuScrollEvent, 65));\n};\n\n\n/**\n * @param {?proto.ubo.v1.MenuScrollEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setMenuScrollEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 65, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearMenuScrollEvent = function() {\n  return this.setMenuScrollEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasMenuScrollEvent = function() {\n  return jspb.Message.getField(this, 65) != null;\n};\n\n\n/**\n * optional NotificationsClearEvent notifications_clear_event = 66;\n * @return {?proto.ubo.v1.NotificationsClearEvent}\n */\nproto.ubo.v1.Event.prototype.getNotificationsClearEvent = function() {\n  return /** @type{?proto.ubo.v1.NotificationsClearEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearEvent, 66));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsClearEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setNotificationsClearEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 66, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearNotificationsClearEvent = function() {\n  return this.setNotificationsClearEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasNotificationsClearEvent = function() {\n  return jspb.Message.getField(this, 66) != null;\n};\n\n\n/**\n * optional NotificationsDisplayEvent notifications_display_event = 67;\n * @return {?proto.ubo.v1.NotificationsDisplayEvent}\n */\nproto.ubo.v1.Event.prototype.getNotificationsDisplayEvent = function() {\n  return /** @type{?proto.ubo.v1.NotificationsDisplayEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsDisplayEvent, 67));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsDisplayEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setNotificationsDisplayEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 67, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearNotificationsDisplayEvent = function() {\n  return this.setNotificationsDisplayEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasNotificationsDisplayEvent = function() {\n  return jspb.Message.getField(this, 67) != null;\n};\n\n\n/**\n * optional NotificationsEvent notifications_event = 68;\n * @return {?proto.ubo.v1.NotificationsEvent}\n */\nproto.ubo.v1.Event.prototype.getNotificationsEvent = function() {\n  return /** @type{?proto.ubo.v1.NotificationsEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsEvent, 68));\n};\n\n\n/**\n * @param {?proto.ubo.v1.NotificationsEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setNotificationsEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 68, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearNotificationsEvent = function() {\n  return this.setNotificationsEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasNotificationsEvent = function() {\n  return jspb.Message.getField(this, 68) != null;\n};\n\n\n/**\n * optional OpenApplicationEvent open_application_event = 69;\n * @return {?proto.ubo.v1.OpenApplicationEvent}\n */\nproto.ubo.v1.Event.prototype.getOpenApplicationEvent = function() {\n  return /** @type{?proto.ubo.v1.OpenApplicationEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent, 69));\n};\n\n\n/**\n * @param {?proto.ubo.v1.OpenApplicationEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setOpenApplicationEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 69, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearOpenApplicationEvent = function() {\n  return this.setOpenApplicationEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasOpenApplicationEvent = function() {\n  return jspb.Message.getField(this, 69) != null;\n};\n\n\n/**\n * optional PowerEvent power_event = 70;\n * @return {?proto.ubo.v1.PowerEvent}\n */\nproto.ubo.v1.Event.prototype.getPowerEvent = function() {\n  return /** @type{?proto.ubo.v1.PowerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerEvent, 70));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PowerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setPowerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 70, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearPowerEvent = function() {\n  return this.setPowerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasPowerEvent = function() {\n  return jspb.Message.getField(this, 70) != null;\n};\n\n\n/**\n * optional PowerOffEvent power_off_event = 71;\n * @return {?proto.ubo.v1.PowerOffEvent}\n */\nproto.ubo.v1.Event.prototype.getPowerOffEvent = function() {\n  return /** @type{?proto.ubo.v1.PowerOffEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerOffEvent, 71));\n};\n\n\n/**\n * @param {?proto.ubo.v1.PowerOffEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setPowerOffEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 71, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearPowerOffEvent = function() {\n  return this.setPowerOffEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasPowerOffEvent = function() {\n  return jspb.Message.getField(this, 71) != null;\n};\n\n\n/**\n * optional RPiConnectEvent r_pi_connect_event = 72;\n * @return {?proto.ubo.v1.RPiConnectEvent}\n */\nproto.ubo.v1.Event.prototype.getRPiConnectEvent = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectEvent, 72));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setRPiConnectEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 72, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearRPiConnectEvent = function() {\n  return this.setRPiConnectEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasRPiConnectEvent = function() {\n  return jspb.Message.getField(this, 72) != null;\n};\n\n\n/**\n * optional RPiConnectLoginEvent r_pi_connect_login_event = 73;\n * @return {?proto.ubo.v1.RPiConnectLoginEvent}\n */\nproto.ubo.v1.Event.prototype.getRPiConnectLoginEvent = function() {\n  return /** @type{?proto.ubo.v1.RPiConnectLoginEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectLoginEvent, 73));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RPiConnectLoginEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setRPiConnectLoginEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 73, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearRPiConnectLoginEvent = function() {\n  return this.setRPiConnectLoginEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasRPiConnectLoginEvent = function() {\n  return jspb.Message.getField(this, 73) != null;\n};\n\n\n/**\n * optional RebootEvent reboot_event = 74;\n * @return {?proto.ubo.v1.RebootEvent}\n */\nproto.ubo.v1.Event.prototype.getRebootEvent = function() {\n  return /** @type{?proto.ubo.v1.RebootEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RebootEvent, 74));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RebootEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setRebootEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 74, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearRebootEvent = function() {\n  return this.setRebootEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasRebootEvent = function() {\n  return jspb.Message.getField(this, 74) != null;\n};\n\n\n/**\n * optional ReplayRecordedSequenceEvent replay_recorded_sequence_event = 75;\n * @return {?proto.ubo.v1.ReplayRecordedSequenceEvent}\n */\nproto.ubo.v1.Event.prototype.getReplayRecordedSequenceEvent = function() {\n  return /** @type{?proto.ubo.v1.ReplayRecordedSequenceEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ReplayRecordedSequenceEvent, 75));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ReplayRecordedSequenceEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setReplayRecordedSequenceEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 75, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearReplayRecordedSequenceEvent = function() {\n  return this.setReplayRecordedSequenceEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasReplayRecordedSequenceEvent = function() {\n  return jspb.Message.getField(this, 75) != null;\n};\n\n\n/**\n * optional RgbRingCommandEvent rgb_ring_command_event = 76;\n * @return {?proto.ubo.v1.RgbRingCommandEvent}\n */\nproto.ubo.v1.Event.prototype.getRgbRingCommandEvent = function() {\n  return /** @type{?proto.ubo.v1.RgbRingCommandEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingCommandEvent, 76));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingCommandEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setRgbRingCommandEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 76, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearRgbRingCommandEvent = function() {\n  return this.setRgbRingCommandEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasRgbRingCommandEvent = function() {\n  return jspb.Message.getField(this, 76) != null;\n};\n\n\n/**\n * optional RgbRingEvent rgb_ring_event = 77;\n * @return {?proto.ubo.v1.RgbRingEvent}\n */\nproto.ubo.v1.Event.prototype.getRgbRingEvent = function() {\n  return /** @type{?proto.ubo.v1.RgbRingEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingEvent, 77));\n};\n\n\n/**\n * @param {?proto.ubo.v1.RgbRingEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setRgbRingEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 77, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearRgbRingEvent = function() {\n  return this.setRgbRingEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasRgbRingEvent = function() {\n  return jspb.Message.getField(this, 77) != null;\n};\n\n\n/**\n * optional ScreenshotEvent screenshot_event = 78;\n * @return {?proto.ubo.v1.ScreenshotEvent}\n */\nproto.ubo.v1.Event.prototype.getScreenshotEvent = function() {\n  return /** @type{?proto.ubo.v1.ScreenshotEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.ScreenshotEvent, 78));\n};\n\n\n/**\n * @param {?proto.ubo.v1.ScreenshotEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setScreenshotEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 78, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearScreenshotEvent = function() {\n  return this.setScreenshotEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasScreenshotEvent = function() {\n  return jspb.Message.getField(this, 78) != null;\n};\n\n\n/**\n * optional SettingsEvent settings_event = 79;\n * @return {?proto.ubo.v1.SettingsEvent}\n */\nproto.ubo.v1.Event.prototype.getSettingsEvent = function() {\n  return /** @type{?proto.ubo.v1.SettingsEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsEvent, 79));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSettingsEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 79, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSettingsEvent = function() {\n  return this.setSettingsEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSettingsEvent = function() {\n  return jspb.Message.getField(this, 79) != null;\n};\n\n\n/**\n * optional SettingsServiceEvent settings_service_event = 80;\n * @return {?proto.ubo.v1.SettingsServiceEvent}\n */\nproto.ubo.v1.Event.prototype.getSettingsServiceEvent = function() {\n  return /** @type{?proto.ubo.v1.SettingsServiceEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceEvent, 80));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsServiceEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSettingsServiceEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 80, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSettingsServiceEvent = function() {\n  return this.setSettingsServiceEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSettingsServiceEvent = function() {\n  return jspb.Message.getField(this, 80) != null;\n};\n\n\n/**\n * optional SettingsStartServiceEvent settings_start_service_event = 81;\n * @return {?proto.ubo.v1.SettingsStartServiceEvent}\n */\nproto.ubo.v1.Event.prototype.getSettingsStartServiceEvent = function() {\n  return /** @type{?proto.ubo.v1.SettingsStartServiceEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStartServiceEvent, 81));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsStartServiceEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSettingsStartServiceEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 81, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSettingsStartServiceEvent = function() {\n  return this.setSettingsStartServiceEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSettingsStartServiceEvent = function() {\n  return jspb.Message.getField(this, 81) != null;\n};\n\n\n/**\n * optional SettingsStopServiceEvent settings_stop_service_event = 82;\n * @return {?proto.ubo.v1.SettingsStopServiceEvent}\n */\nproto.ubo.v1.Event.prototype.getSettingsStopServiceEvent = function() {\n  return /** @type{?proto.ubo.v1.SettingsStopServiceEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStopServiceEvent, 82));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SettingsStopServiceEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSettingsStopServiceEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 82, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSettingsStopServiceEvent = function() {\n  return this.setSettingsStopServiceEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSettingsStopServiceEvent = function() {\n  return jspb.Message.getField(this, 82) != null;\n};\n\n\n/**\n * optional SnapshotEvent snapshot_event = 83;\n * @return {?proto.ubo.v1.SnapshotEvent}\n */\nproto.ubo.v1.Event.prototype.getSnapshotEvent = function() {\n  return /** @type{?proto.ubo.v1.SnapshotEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SnapshotEvent, 83));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SnapshotEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSnapshotEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 83, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSnapshotEvent = function() {\n  return this.setSnapshotEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSnapshotEvent = function() {\n  return jspb.Message.getField(this, 83) != null;\n};\n\n\n/**\n * optional SpeechRecognitionEvent speech_recognition_event = 84;\n * @return {?proto.ubo.v1.SpeechRecognitionEvent}\n */\nproto.ubo.v1.Event.prototype.getSpeechRecognitionEvent = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionEvent, 84));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSpeechRecognitionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 84, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSpeechRecognitionEvent = function() {\n  return this.setSpeechRecognitionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSpeechRecognitionEvent = function() {\n  return jspb.Message.getField(this, 84) != null;\n};\n\n\n/**\n * optional SpeechRecognitionReportTextEvent speech_recognition_report_text_event = 85;\n * @return {?proto.ubo.v1.SpeechRecognitionReportTextEvent}\n */\nproto.ubo.v1.Event.prototype.getSpeechRecognitionReportTextEvent = function() {\n  return /** @type{?proto.ubo.v1.SpeechRecognitionReportTextEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportTextEvent, 85));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechRecognitionReportTextEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSpeechRecognitionReportTextEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 85, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSpeechRecognitionReportTextEvent = function() {\n  return this.setSpeechRecognitionReportTextEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSpeechRecognitionReportTextEvent = function() {\n  return jspb.Message.getField(this, 85) != null;\n};\n\n\n/**\n * optional SpeechSynthesisEvent speech_synthesis_event = 86;\n * @return {?proto.ubo.v1.SpeechSynthesisEvent}\n */\nproto.ubo.v1.Event.prototype.getSpeechSynthesisEvent = function() {\n  return /** @type{?proto.ubo.v1.SpeechSynthesisEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisEvent, 86));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechSynthesisEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSpeechSynthesisEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 86, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSpeechSynthesisEvent = function() {\n  return this.setSpeechSynthesisEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSpeechSynthesisEvent = function() {\n  return jspb.Message.getField(this, 86) != null;\n};\n\n\n/**\n * optional SpeechSynthesisSynthesizeTextEvent speech_synthesis_synthesize_text_event = 87;\n * @return {?proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent}\n */\nproto.ubo.v1.Event.prototype.getSpeechSynthesisSynthesizeTextEvent = function() {\n  return /** @type{?proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent, 87));\n};\n\n\n/**\n * @param {?proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setSpeechSynthesisSynthesizeTextEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 87, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearSpeechSynthesisSynthesizeTextEvent = function() {\n  return this.setSpeechSynthesisSynthesizeTextEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasSpeechSynthesisSynthesizeTextEvent = function() {\n  return jspb.Message.getField(this, 87) != null;\n};\n\n\n/**\n * optional StoreRecordedSequenceEvent store_recorded_sequence_event = 88;\n * @return {?proto.ubo.v1.StoreRecordedSequenceEvent}\n */\nproto.ubo.v1.Event.prototype.getStoreRecordedSequenceEvent = function() {\n  return /** @type{?proto.ubo.v1.StoreRecordedSequenceEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.StoreRecordedSequenceEvent, 88));\n};\n\n\n/**\n * @param {?proto.ubo.v1.StoreRecordedSequenceEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setStoreRecordedSequenceEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 88, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearStoreRecordedSequenceEvent = function() {\n  return this.setStoreRecordedSequenceEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasStoreRecordedSequenceEvent = function() {\n  return jspb.Message.getField(this, 88) != null;\n};\n\n\n/**\n * optional UpdateManagerCheckEvent update_manager_check_event = 89;\n * @return {?proto.ubo.v1.UpdateManagerCheckEvent}\n */\nproto.ubo.v1.Event.prototype.getUpdateManagerCheckEvent = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerCheckEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerCheckEvent, 89));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerCheckEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUpdateManagerCheckEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 89, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUpdateManagerCheckEvent = function() {\n  return this.setUpdateManagerCheckEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUpdateManagerCheckEvent = function() {\n  return jspb.Message.getField(this, 89) != null;\n};\n\n\n/**\n * optional UpdateManagerEvent update_manager_event = 90;\n * @return {?proto.ubo.v1.UpdateManagerEvent}\n */\nproto.ubo.v1.Event.prototype.getUpdateManagerEvent = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerEvent, 90));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUpdateManagerEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 90, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUpdateManagerEvent = function() {\n  return this.setUpdateManagerEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUpdateManagerEvent = function() {\n  return jspb.Message.getField(this, 90) != null;\n};\n\n\n/**\n * optional UpdateManagerUpdateEvent update_manager_update_event = 91;\n * @return {?proto.ubo.v1.UpdateManagerUpdateEvent}\n */\nproto.ubo.v1.Event.prototype.getUpdateManagerUpdateEvent = function() {\n  return /** @type{?proto.ubo.v1.UpdateManagerUpdateEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerUpdateEvent, 91));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UpdateManagerUpdateEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUpdateManagerUpdateEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 91, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUpdateManagerUpdateEvent = function() {\n  return this.setUpdateManagerUpdateEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUpdateManagerUpdateEvent = function() {\n  return jspb.Message.getField(this, 91) != null;\n};\n\n\n/**\n * optional UsersCreateUserEvent users_create_user_event = 92;\n * @return {?proto.ubo.v1.UsersCreateUserEvent}\n */\nproto.ubo.v1.Event.prototype.getUsersCreateUserEvent = function() {\n  return /** @type{?proto.ubo.v1.UsersCreateUserEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersCreateUserEvent, 92));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersCreateUserEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUsersCreateUserEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 92, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUsersCreateUserEvent = function() {\n  return this.setUsersCreateUserEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUsersCreateUserEvent = function() {\n  return jspb.Message.getField(this, 92) != null;\n};\n\n\n/**\n * optional UsersDeleteUserEvent users_delete_user_event = 93;\n * @return {?proto.ubo.v1.UsersDeleteUserEvent}\n */\nproto.ubo.v1.Event.prototype.getUsersDeleteUserEvent = function() {\n  return /** @type{?proto.ubo.v1.UsersDeleteUserEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersDeleteUserEvent, 93));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersDeleteUserEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUsersDeleteUserEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 93, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUsersDeleteUserEvent = function() {\n  return this.setUsersDeleteUserEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUsersDeleteUserEvent = function() {\n  return jspb.Message.getField(this, 93) != null;\n};\n\n\n/**\n * optional UsersEvent users_event = 94;\n * @return {?proto.ubo.v1.UsersEvent}\n */\nproto.ubo.v1.Event.prototype.getUsersEvent = function() {\n  return /** @type{?proto.ubo.v1.UsersEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersEvent, 94));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUsersEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 94, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUsersEvent = function() {\n  return this.setUsersEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUsersEvent = function() {\n  return jspb.Message.getField(this, 94) != null;\n};\n\n\n/**\n * optional UsersResetPasswordEvent users_reset_password_event = 95;\n * @return {?proto.ubo.v1.UsersResetPasswordEvent}\n */\nproto.ubo.v1.Event.prototype.getUsersResetPasswordEvent = function() {\n  return /** @type{?proto.ubo.v1.UsersResetPasswordEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersResetPasswordEvent, 95));\n};\n\n\n/**\n * @param {?proto.ubo.v1.UsersResetPasswordEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setUsersResetPasswordEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 95, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearUsersResetPasswordEvent = function() {\n  return this.setUsersResetPasswordEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasUsersResetPasswordEvent = function() {\n  return jspb.Message.getField(this, 95) != null;\n};\n\n\n/**\n * optional VSCodeEvent vs_code_event = 96;\n * @return {?proto.ubo.v1.VSCodeEvent}\n */\nproto.ubo.v1.Event.prototype.getVsCodeEvent = function() {\n  return /** @type{?proto.ubo.v1.VSCodeEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeEvent, 96));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setVsCodeEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 96, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearVsCodeEvent = function() {\n  return this.setVsCodeEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasVsCodeEvent = function() {\n  return jspb.Message.getField(this, 96) != null;\n};\n\n\n/**\n * optional VSCodeLoginEvent vs_code_login_event = 97;\n * @return {?proto.ubo.v1.VSCodeLoginEvent}\n */\nproto.ubo.v1.Event.prototype.getVsCodeLoginEvent = function() {\n  return /** @type{?proto.ubo.v1.VSCodeLoginEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeLoginEvent, 97));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeLoginEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setVsCodeLoginEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 97, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearVsCodeLoginEvent = function() {\n  return this.setVsCodeLoginEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasVsCodeLoginEvent = function() {\n  return jspb.Message.getField(this, 97) != null;\n};\n\n\n/**\n * optional VSCodeRestartEvent vs_code_restart_event = 98;\n * @return {?proto.ubo.v1.VSCodeRestartEvent}\n */\nproto.ubo.v1.Event.prototype.getVsCodeRestartEvent = function() {\n  return /** @type{?proto.ubo.v1.VSCodeRestartEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeRestartEvent, 98));\n};\n\n\n/**\n * @param {?proto.ubo.v1.VSCodeRestartEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setVsCodeRestartEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 98, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearVsCodeRestartEvent = function() {\n  return this.setVsCodeRestartEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasVsCodeRestartEvent = function() {\n  return jspb.Message.getField(this, 98) != null;\n};\n\n\n/**\n * optional WebUIEvent web_ui_event = 99;\n * @return {?proto.ubo.v1.WebUIEvent}\n */\nproto.ubo.v1.Event.prototype.getWebUiEvent = function() {\n  return /** @type{?proto.ubo.v1.WebUIEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIEvent, 99));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WebUIEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setWebUiEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 99, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearWebUiEvent = function() {\n  return this.setWebUiEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasWebUiEvent = function() {\n  return jspb.Message.getField(this, 99) != null;\n};\n\n\n/**\n * optional WebUIInitializeEvent web_ui_initialize_event = 100;\n * @return {?proto.ubo.v1.WebUIInitializeEvent}\n */\nproto.ubo.v1.Event.prototype.getWebUiInitializeEvent = function() {\n  return /** @type{?proto.ubo.v1.WebUIInitializeEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInitializeEvent, 100));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WebUIInitializeEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setWebUiInitializeEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 100, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearWebUiInitializeEvent = function() {\n  return this.setWebUiInitializeEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasWebUiInitializeEvent = function() {\n  return jspb.Message.getField(this, 100) != null;\n};\n\n\n/**\n * optional WebUIStopEvent web_ui_stop_event = 101;\n * @return {?proto.ubo.v1.WebUIStopEvent}\n */\nproto.ubo.v1.Event.prototype.getWebUiStopEvent = function() {\n  return /** @type{?proto.ubo.v1.WebUIStopEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIStopEvent, 101));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WebUIStopEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setWebUiStopEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 101, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearWebUiStopEvent = function() {\n  return this.setWebUiStopEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasWebUiStopEvent = function() {\n  return jspb.Message.getField(this, 101) != null;\n};\n\n\n/**\n * optional WiFiEvent wi_fi_event = 102;\n * @return {?proto.ubo.v1.WiFiEvent}\n */\nproto.ubo.v1.Event.prototype.getWiFiEvent = function() {\n  return /** @type{?proto.ubo.v1.WiFiEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiEvent, 102));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setWiFiEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 102, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearWiFiEvent = function() {\n  return this.setWiFiEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasWiFiEvent = function() {\n  return jspb.Message.getField(this, 102) != null;\n};\n\n\n/**\n * optional WiFiInputConnectionEvent wi_fi_input_connection_event = 103;\n * @return {?proto.ubo.v1.WiFiInputConnectionEvent}\n */\nproto.ubo.v1.Event.prototype.getWiFiInputConnectionEvent = function() {\n  return /** @type{?proto.ubo.v1.WiFiInputConnectionEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiInputConnectionEvent, 103));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiInputConnectionEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setWiFiInputConnectionEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 103, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearWiFiInputConnectionEvent = function() {\n  return this.setWiFiInputConnectionEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasWiFiInputConnectionEvent = function() {\n  return jspb.Message.getField(this, 103) != null;\n};\n\n\n/**\n * optional WiFiUpdateRequestEvent wi_fi_update_request_event = 104;\n * @return {?proto.ubo.v1.WiFiUpdateRequestEvent}\n */\nproto.ubo.v1.Event.prototype.getWiFiUpdateRequestEvent = function() {\n  return /** @type{?proto.ubo.v1.WiFiUpdateRequestEvent} */ (\n    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiUpdateRequestEvent, 104));\n};\n\n\n/**\n * @param {?proto.ubo.v1.WiFiUpdateRequestEvent|undefined} value\n * @return {!proto.ubo.v1.Event} returns this\n*/\nproto.ubo.v1.Event.prototype.setWiFiUpdateRequestEvent = function(value) {\n  return jspb.Message.setOneofWrapperField(this, 104, proto.ubo.v1.Event.oneofGroups_[0], value);\n};\n\n\n/**\n * Clears the message field making it undefined.\n * @return {!proto.ubo.v1.Event} returns this\n */\nproto.ubo.v1.Event.prototype.clearWiFiUpdateRequestEvent = function() {\n  return this.setWiFiUpdateRequestEvent(undefined);\n};\n\n\n/**\n * Returns whether this field is set.\n * @return {boolean}\n */\nproto.ubo.v1.Event.prototype.hasWiFiUpdateRequestEvent = function() {\n  return jspb.Message.getField(this, 104) != null;\n};\n\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.SettingsCategory = {\n  SETTINGS_CATEGORY_UBO_APP_DOT_STORE_DOT_CORE_DOT_TYPES_UNSPECIFIED: 0,\n  SETTINGS_CATEGORY_NETWORK: 1,\n  SETTINGS_CATEGORY_REMOTE: 2,\n  SETTINGS_CATEGORY_SYSTEM: 3,\n  SETTINGS_CATEGORY_HARDWARE: 4,\n  SETTINGS_CATEGORY_ASSISTANT: 5,\n  SETTINGS_CATEGORY_DOCKER: 6,\n  SETTINGS_CATEGORY_ACCESSIBILITY: 7\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.MenuScrollDirection = {\n  MENU_SCROLL_DIRECTION_UBO_APP_DOT_STORE_DOT_CORE_DOT_TYPES_UNSPECIFIED: 0,\n  MENU_SCROLL_DIRECTION_UP: 1,\n  MENU_SCROLL_DIRECTION_DOWN: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.InputMethod = {\n  INPUT_METHOD_UBO_APP_DOT_STORE_DOT_INPUT_DOT_TYPES_UNSPECIFIED: 0,\n  INPUT_METHOD_CAMERA: 1,\n  INPUT_METHOD_WEB_DASHBOARD: 2,\n  INPUT_METHOD_PATH_SELECTOR: 3\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.InputFieldType = {\n  INPUT_FIELD_TYPE_UBO_APP_DOT_STORE_DOT_INPUT_DOT_TYPES_UNSPECIFIED: 0,\n  INPUT_FIELD_TYPE_LONG: 1,\n  INPUT_FIELD_TYPE_TEXT: 2,\n  INPUT_FIELD_TYPE_PASSWORD: 3,\n  INPUT_FIELD_TYPE_NUMBER: 4,\n  INPUT_FIELD_TYPE_CHECKBOX: 5,\n  INPUT_FIELD_TYPE_COLOR: 6,\n  INPUT_FIELD_TYPE_SELECT: 7,\n  INPUT_FIELD_TYPE_FILE: 8,\n  INPUT_FIELD_TYPE_DATE: 9,\n  INPUT_FIELD_TYPE_TIME: 10\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.ServicesStatus = {\n  SERVICES_STATUS_UBO_APP_DOT_STORE_DOT_SETTINGS_DOT_TYPES_UNSPECIFIED: 0,\n  SERVICES_STATUS_LOADING: 1,\n  SERVICES_STATUS_READY: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.UpdateStatus = {\n  UPDATE_STATUS_UBO_APP_DOT_STORE_DOT_UPDATE_MANAGER_DOT_TYPES_UNSPECIFIED: 0,\n  UPDATE_STATUS_CHECKING: 1,\n  UPDATE_STATUS_FAILED_TO_CHECK: 2,\n  UPDATE_STATUS_UP_TO_DATE: 3,\n  UPDATE_STATUS_OUTDATED: 4,\n  UPDATE_STATUS_UPDATING: 5\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AssistantSTTName = {\n  ASSISTANT_STT_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,\n  ASSISTANT_STT_NAME_VOSK: 1,\n  ASSISTANT_STT_NAME_GOOGLE_SEGMENTED: 2,\n  ASSISTANT_STT_NAME_GOOGLE: 3,\n  ASSISTANT_STT_NAME_OPENAI: 4,\n  ASSISTANT_STT_NAME_DEEPGRAM: 5,\n  ASSISTANT_STT_NAME_ASSEMBLYAI: 6\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AssistantLLMName = {\n  ASSISTANT_LLM_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,\n  ASSISTANT_LLM_NAME_OLLAMA: 1,\n  ASSISTANT_LLM_NAME_OLLAMA_ONPREM: 2,\n  ASSISTANT_LLM_NAME_GOOGLE: 3,\n  ASSISTANT_LLM_NAME_OPENAI: 4,\n  ASSISTANT_LLM_NAME_GROK: 5,\n  ASSISTANT_LLM_NAME_CEREBRAS: 6\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AssistantTTSName = {\n  ASSISTANT_TTS_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,\n  ASSISTANT_TTS_NAME_PIPER: 1,\n  ASSISTANT_TTS_NAME_GOOGLE: 2,\n  ASSISTANT_TTS_NAME_OPENAI: 3,\n  ASSISTANT_TTS_NAME_ELEVENLABS: 4,\n  ASSISTANT_TTS_NAME_RIME: 5\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AssistantImageGeneratorName = {\n  ASSISTANT_IMAGE_GENERATOR_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,\n  ASSISTANT_IMAGE_GENERATOR_NAME_GOOGLE: 1,\n  ASSISTANT_IMAGE_GENERATOR_NAME_OPENAI: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.McpServerType = {\n  MCP_SERVER_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,\n  MCP_SERVER_TYPE_STDIO: 1,\n  MCP_SERVER_TYPE_SSE: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.AudioDevice = {\n  AUDIO_DEVICE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_AUDIO_UNSPECIFIED: 0,\n  AUDIO_DEVICE_INPUT: 1,\n  AUDIO_DEVICE_OUTPUT: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.CameraType = {\n  CAMERA_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_CAMERA_UNSPECIFIED: 0,\n  CAMERA_TYPE_DEFAULT: 1,\n  CAMERA_TYPE_AUTOFOCUS: 2,\n  CAMERA_TYPE_FIXED_FOCUS: 3\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.DisplayBlankTimeout = {\n  DISPLAY_BLANK_TIMEOUT_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_DISPLAY_UNSPECIFIED: 0,\n  DISPLAY_BLANK_TIMEOUT_ONE_MINUTE: 1,\n  DISPLAY_BLANK_TIMEOUT_FIVE_MINUTES: 2,\n  DISPLAY_BLANK_TIMEOUT_TEN_MINUTES: 3,\n  DISPLAY_BLANK_TIMEOUT_THIRTY_MINUTES: 4,\n  DISPLAY_BLANK_TIMEOUT_ONE_HOUR: 5,\n  DISPLAY_BLANK_TIMEOUT_OFF: 6\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.DockerStatus = {\n  DOCKER_STATUS_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_DOCKER_UNSPECIFIED: 0,\n  DOCKER_STATUS_UNKNOWN: 1,\n  DOCKER_STATUS_NOT_INSTALLED: 2,\n  DOCKER_STATUS_INSTALLING: 3,\n  DOCKER_STATUS_NOT_RUNNING: 4,\n  DOCKER_STATUS_RUNNING: 5,\n  DOCKER_STATUS_ERROR: 6\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.DockerItemStatus = {\n  DOCKER_ITEM_STATUS_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_DOCKER_UNSPECIFIED: 0,\n  DOCKER_ITEM_STATUS_NOT_AVAILABLE: 1,\n  DOCKER_ITEM_STATUS_FETCHING: 2,\n  DOCKER_ITEM_STATUS_AVAILABLE: 3,\n  DOCKER_ITEM_STATUS_CREATED: 4,\n  DOCKER_ITEM_STATUS_RUNNING: 5,\n  DOCKER_ITEM_STATUS_ERROR: 6,\n  DOCKER_ITEM_STATUS_PROCESSING: 7\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.NetState = {\n  NET_STATE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ETHERNET_UNSPECIFIED: 0,\n  NET_STATE_CONNECTED: 1,\n  NET_STATE_DISCONNECTED: 2,\n  NET_STATE_PENDING: 3,\n  NET_STATE_NEEDS_ATTENTION: 4,\n  NET_STATE_UNKNOWN: 5\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Key = {\n  KEY_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_KEYPAD_UNSPECIFIED: 0,\n  KEY_BACK: 1,\n  KEY_HOME: 2,\n  KEY_UP: 3,\n  KEY_DOWN: 4,\n  KEY_L1: 5,\n  KEY_L2: 6,\n  KEY_L3: 7\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Importance = {\n  IMPORTANCE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_NOTIFICATIONS_UNSPECIFIED: 0,\n  IMPORTANCE_CRITICAL: 1,\n  IMPORTANCE_HIGH: 2,\n  IMPORTANCE_MEDIUM: 3,\n  IMPORTANCE_LOW: 4\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.NotificationDisplayType = {\n  NOTIFICATION_DISPLAY_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_NOTIFICATIONS_UNSPECIFIED: 0,\n  NOTIFICATION_DISPLAY_TYPE_NOT_SET: 1,\n  NOTIFICATION_DISPLAY_TYPE_BACKGROUND: 2,\n  NOTIFICATION_DISPLAY_TYPE_FLASH: 3,\n  NOTIFICATION_DISPLAY_TYPE_STICKY: 4\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Chime = {\n  CHIME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_NOTIFICATIONS_UNSPECIFIED: 0,\n  CHIME_ADD: 1,\n  CHIME_DONE: 2,\n  CHIME_FAILURE: 3,\n  CHIME_VOLUME_CHANGE: 4\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.Sensor = {\n  SENSOR_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SENSORS_UNSPECIFIED: 0,\n  SENSOR_TEMPERATURE: 1,\n  SENSOR_LIGHT: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.SpeechRecognitionStatus = {\n  SPEECH_RECOGNITION_STATUS_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SPEECH_RECOGNITION_UNSPECIFIED: 0,\n  SPEECH_RECOGNITION_STATUS_IDLE: 1,\n  SPEECH_RECOGNITION_STATUS_INTENTS_WAITING: 2,\n  SPEECH_RECOGNITION_STATUS_ASSISTANT_WAITING: 3\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.SpeechRecognitionEngineName = {\n  SPEECH_RECOGNITION_ENGINE_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SPEECH_RECOGNITION_UNSPECIFIED: 0,\n  SPEECH_RECOGNITION_ENGINE_NAME_VOSK: 1,\n  SPEECH_RECOGNITION_ENGINE_NAME_GOOGLE: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.SpeechSynthesisEngineName = {\n  SPEECH_SYNTHESIS_ENGINE_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SPEECH_SYNTHESIS_UNSPECIFIED: 0,\n  SPEECH_SYNTHESIS_ENGINE_NAME_PIPER: 1,\n  SPEECH_SYNTHESIS_ENGINE_NAME_PICOVOICE: 2\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.WiFiType = {\n  WI_FI_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_WIFI_UNSPECIFIED: 0,\n  WI_FI_TYPE_WEP: 1,\n  WI_FI_TYPE_WPA: 2,\n  WI_FI_TYPE_WPA2: 3,\n  WI_FI_TYPE_NOPASS: 4\n};\n\n/**\n * @enum {number}\n */\nproto.ubo.v1.ConnectionState = {\n  CONNECTION_STATE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_WIFI_UNSPECIFIED: 0,\n  CONNECTION_STATE_CONNECTED: 1,\n  CONNECTION_STATE_CONNECTING: 2,\n  CONNECTION_STATE_DISCONNECTED: 3,\n  CONNECTION_STATE_UNKNOWN: 4\n};\n\ngoog.object.extend(exports, proto.ubo.v1);\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/bindings/ubo/v1/ubo_pb.js?");

/***/ }),

/***/ "./src/client.tsx":
/*!************************!*\
  !*** ./src/client.tsx ***!
  \************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   Root: () => (/* binding */ Root),\n/* harmony export */   init: () => (/* binding */ init)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/AppBar/AppBar.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Container/Container.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Stack/Stack.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/styles/createTheme.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/styles/ThemeProvider.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/CssBaseline/CssBaseline.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var react_dom_client__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! react-dom/client */ \"./node_modules/react-dom/client.js\");\n/* harmony import */ var _fontsource_roboto_300_css__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @fontsource/roboto/300.css */ \"./node_modules/@fontsource/roboto/300.css\");\n/* harmony import */ var _fontsource_roboto_400_css__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @fontsource/roboto/400.css */ \"./node_modules/@fontsource/roboto/400.css\");\n/* harmony import */ var _fontsource_roboto_500_css__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @fontsource/roboto/500.css */ \"./node_modules/@fontsource/roboto/500.css\");\n/* harmony import */ var _fontsource_roboto_700_css__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @fontsource/roboto/700.css */ \"./node_modules/@fontsource/roboto/700.css\");\n/* harmony import */ var _bindings_store_v1_StoreServiceClientPb__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! ./bindings/store/v1/StoreServiceClientPb */ \"./src/bindings/store/v1/StoreServiceClientPb.ts\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! ./bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_8___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_8__);\n/* harmony import */ var _components_ThemeSwitch__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! ./components/ThemeSwitch */ \"./src/components/ThemeSwitch.tsx\");\n/* harmony import */ var _inputs__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! ./inputs */ \"./src/inputs.tsx\");\n/* harmony import */ var _main_view__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! ./main-view */ \"./src/main-view.tsx\");\n\n\n\n\n\n\n\n\n\n\n\n\n\nfunction Root({ state }) {\n    const [status, setStatus] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)();\n    const inputDescriptions = _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_8__.WebUIState.deserializeBinary(new Uint8Array(((status == null ? state : status.state).match(/.{1,2}/g) || []).map((byte) => parseInt(byte, 16)))).toObject().activeInputsList;\n    const store = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => new _bindings_store_v1_StoreServiceClientPb__WEBPACK_IMPORTED_MODULE_7__.StoreServiceClient(`${window.location.protocol}//${window.location.hostname}:${window.GRPC_ENVOY_LISTEN_PORT}`, null, null), []);\n    (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {\n        async function checkStatus() {\n            try {\n                const response = await fetch(\"/status\");\n                const data = await response.json();\n                setStatus(data);\n            }\n            catch {\n                setStatus(undefined);\n            }\n        }\n        checkStatus();\n        const interval = setInterval(checkStatus, 1000);\n        return () => clearInterval(interval);\n    }, []);\n    return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, { children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_12__[\"default\"], { position: \"static\", variant: \"outlined\", color: \"inherit\", children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_13__[\"default\"], { maxWidth: \"xs\", children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_ThemeSwitch__WEBPACK_IMPORTED_MODULE_9__.ThemeSwitch, {}) }) }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_13__[\"default\"], { maxWidth: \"xs\", component: _mui_material__WEBPACK_IMPORTED_MODULE_14__[\"default\"], spacing: 2, py: 2, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_main_view__WEBPACK_IMPORTED_MODULE_11__.MainView, { status: status, store: store }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_inputs__WEBPACK_IMPORTED_MODULE_10__.Inputs, { inputs: inputDescriptions, isGrpcConnected: status?.status === \"ok\" &&\n                            status?.docker === \"running\" &&\n                            status?.envoy === \"running\", store: store })] })] }));\n}\nconst theme = (0,_mui_material__WEBPACK_IMPORTED_MODULE_15__[\"default\"])({\n    colorSchemes: {\n        dark: true,\n    },\n});\nfunction init(state) {\n    const rootElement = document.getElementById(\"web-app-root\");\n    if (rootElement) {\n        const root = (0,react_dom_client__WEBPACK_IMPORTED_MODULE_2__.createRoot)(rootElement);\n        root.render((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_16__[\"default\"], { theme: theme, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_17__[\"default\"], {}), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Root, { state: state })] }));\n    }\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/client.tsx?");

/***/ }),

/***/ "./src/components/Status.tsx":
/*!***********************************!*\
  !*** ./src/components/Status.tsx ***!
  \***********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   Status: () => (/* binding */ Status)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Card/Card.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/CardContent/CardContent.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Alert/Alert.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/AlertTitle/AlertTitle.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/CardActions/CardActions.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Button/Button.js\");\n\n\nfunction Status({ message, severity = \"warning\", action }) {\n    return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_1__[\"default\"], { children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_2__[\"default\"], { children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_3__[\"default\"], { severity: severity, children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_4__[\"default\"], { children: message }) }) }), action && ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_5__[\"default\"], { children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_6__[\"default\"], { onClick: action.callback, children: action.label }) }))] }));\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/components/Status.tsx?");

/***/ }),

/***/ "./src/components/ThemeSwitch.tsx":
/*!****************************************!*\
  !*** ./src/components/ThemeSwitch.tsx ***!
  \****************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   ThemeSwitch: () => (/* binding */ ThemeSwitch)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_icons_material__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/icons-material */ \"./node_modules/@mui/icons-material/esm/LightMode.js\");\n/* harmony import */ var _mui_icons_material__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/icons-material */ \"./node_modules/@mui/icons-material/esm/SettingsBrightness.js\");\n/* harmony import */ var _mui_icons_material__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/icons-material */ \"./node_modules/@mui/icons-material/esm/DarkModeOutlined.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/styles/styled.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/ToggleButton/ToggleButton.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/styles/ThemeProviderWithVars.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/ToggleButtonGroup/ToggleButtonGroup.js\");\n\n\n\nconst IconToggleButton = (0,_mui_material__WEBPACK_IMPORTED_MODULE_1__[\"default\"])(_mui_material__WEBPACK_IMPORTED_MODULE_2__[\"default\"])({\n    display: \"flex\",\n    justifyContent: \"center\",\n    width: \"100%\",\n    \"& > *\": {\n        marginRight: \"8px\",\n    },\n});\nfunction ThemeSwitch() {\n    const { mode, setMode } = (0,_mui_material__WEBPACK_IMPORTED_MODULE_3__.useColorScheme)();\n    const handleChangeThemeMode = (_, paletteMode) => {\n        if (paletteMode === null) {\n            return;\n        }\n        setMode(paletteMode);\n    };\n    return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_4__[\"default\"], { exclusive: true, value: mode, color: \"primary\", onChange: handleChangeThemeMode, \"aria-labelledby\": \"settings-mode\", fullWidth: true, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(IconToggleButton, { value: \"light\", children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_icons_material__WEBPACK_IMPORTED_MODULE_5__[\"default\"], { fontSize: \"small\" }), \"Light\"] }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(IconToggleButton, { value: \"system\", children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_icons_material__WEBPACK_IMPORTED_MODULE_6__[\"default\"], { fontSize: \"small\" }), \"System\"] }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(IconToggleButton, { value: \"dark\", children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_icons_material__WEBPACK_IMPORTED_MODULE_7__[\"default\"], { fontSize: \"small\" }), \"Dark\"] })] }));\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/components/ThemeSwitch.tsx?");

/***/ }),

/***/ "./src/display/browser-event-handler.ts":
/*!**********************************************!*\
  !*** ./src/display/browser-event-handler.ts ***!
  \**********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   subscribeToBrowserEvents: () => (/* binding */ subscribeToBrowserEvents)\n/* harmony export */ });\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__);\n\nfunction subscribeToBrowserEvents({ onKeyUp, onKeyDown, }) {\n    const KEYS = {\n        \"1\": _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_L1,\n        \"2\": _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_L2,\n        \"3\": _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_L3,\n        Backspace: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_HOME,\n        ArrowLeft: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_BACK,\n        h: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_BACK,\n        ArrowUp: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_UP,\n        k: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_UP,\n        ArrowDown: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_DOWN,\n        j: _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.Key.KEY_DOWN,\n    };\n    function isValidKey(key) {\n        return key in KEYS;\n    }\n    function keyUpHandler({ target, key }) {\n        if (target &&\n            target instanceof HTMLElement &&\n            (target.closest(\"#web-app-root\") ||\n                target.querySelector(\"#web-app-root\")) &&\n            !(target instanceof HTMLInputElement) &&\n            !(target instanceof HTMLTextAreaElement) &&\n            !(target instanceof HTMLSelectElement) &&\n            isValidKey(key)) {\n            onKeyUp(KEYS[key]);\n        }\n    }\n    function keyDownHandler({ target, key }) {\n        if (target &&\n            target instanceof HTMLElement &&\n            (target.closest(\"#web-app-root\") ||\n                target.querySelector(\"#web-app-root\")) &&\n            !(target instanceof HTMLInputElement) &&\n            !(target instanceof HTMLTextAreaElement) &&\n            !(target instanceof HTMLSelectElement) &&\n            isValidKey(key)) {\n            onKeyDown(KEYS[key]);\n        }\n    }\n    document.addEventListener(\"keyup\", keyUpHandler);\n    document.addEventListener(\"keydown\", keyDownHandler);\n    return () => {\n        document.removeEventListener(\"keyup\", keyUpHandler);\n        document.removeEventListener(\"keydown\", keyDownHandler);\n    };\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/display/browser-event-handler.ts?");

/***/ }),

/***/ "./src/display/index.tsx":
/*!*******************************!*\
  !*** ./src/display/index.tsx ***!
  \*******************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   Display: () => (/* binding */ Display)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Box/Box.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _browser_event_handler__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./browser-event-handler */ \"./src/display/browser-event-handler.ts\");\n/* harmony import */ var _layout__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./layout */ \"./src/display/layout.tsx\");\n/* harmony import */ var _store_event_handler__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./store-event-handler */ \"./src/display/store-event-handler.ts\");\n/* harmony import */ var _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! ../bindings/store/v1/store_pb */ \"./src/bindings/store/v1/store_pb.js\");\n/* harmony import */ var _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(_bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_5__);\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! ../bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__);\n\n\n\n\n\n\n\n\nfunction Display({ store }) {\n    const [canvas, setCanvas] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(null);\n    (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {\n        if (!canvas)\n            return;\n        (0,_store_event_handler__WEBPACK_IMPORTED_MODULE_4__.subscribeToStoreEvents)(store, canvas);\n        return (0,_browser_event_handler__WEBPACK_IMPORTED_MODULE_2__.subscribeToBrowserEvents)({\n            onKeyDown: keyDownHandler,\n            onKeyUp: keyUpHandler,\n        });\n    }, [canvas]);\n    const pressedKeys = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.KeypadAction.PressedKeysSetType(), []);\n    function keyUpHandler(key) {\n        pressedKeys.setItemsList(pressedKeys.getItemsList().filter((item) => item !== key));\n        const keypadKeyReleaseAction = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.KeypadKeyReleaseAction();\n        keypadKeyReleaseAction.setKey(key);\n        keypadKeyReleaseAction.setPressedKeys(pressedKeys);\n        const action = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Action();\n        action.setKeypadKeyReleaseAction(keypadKeyReleaseAction);\n        const dispatchActionRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_5__.DispatchActionRequest();\n        dispatchActionRequest.setAction(action);\n        store.dispatchAction(dispatchActionRequest);\n    }\n    function keyDownHandler(key) {\n        pressedKeys.addItems(key);\n        const keypadKeyPressAction = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.KeypadKeyPressAction();\n        keypadKeyPressAction.setKey(key);\n        keypadKeyPressAction.setPressedKeys(pressedKeys);\n        const action = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Action();\n        action.setKeypadKeyPressAction(keypadKeyPressAction);\n        const dispatchActionRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_5__.DispatchActionRequest();\n        dispatchActionRequest.setAction(action);\n        store.dispatchAction(dispatchActionRequest);\n    }\n    const touchStartX = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(0);\n    const touchStartY = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(0);\n    const touchEndX = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(0);\n    const touchEndY = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(0);\n    const handleTouchStart = (event) => {\n        touchStartX.current = event.changedTouches[0].screenX;\n        touchStartY.current = event.changedTouches[0].screenY;\n    };\n    const handleTouchEnd = (event) => {\n        touchEndX.current = event.changedTouches[0].screenX;\n        touchEndY.current = event.changedTouches[0].screenY;\n        handleGesture();\n    };\n    const handleGesture = () => {\n        const deltaX = touchEndX.current - touchStartX.current;\n        const deltaY = touchEndY.current - touchStartY.current;\n        if (Math.abs(deltaX) > Math.abs(deltaY)) {\n            if (deltaX < -5) {\n                keyDownHandler(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Key.KEY_BACK);\n                keyUpHandler(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Key.KEY_BACK);\n            }\n        }\n        else {\n            if (deltaY > 5) {\n                keyDownHandler(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Key.KEY_UP);\n                keyUpHandler(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Key.KEY_UP);\n            }\n            else if (deltaY < -5) {\n                keyDownHandler(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Key.KEY_DOWN);\n                keyUpHandler(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_6__.Key.KEY_DOWN);\n            }\n        }\n    };\n    const [ref, setRef] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(null);\n    (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {\n        const preventDefault = (e) => e.preventDefault();\n        const disableBodyScroll = () => {\n            document.addEventListener(\"touchmove\", preventDefault, {\n                passive: false,\n            });\n        };\n        const enableBodyScroll = () => {\n            document.removeEventListener(\"touchmove\", preventDefault);\n        };\n        const element = ref;\n        if (element) {\n            element.addEventListener(\"touchstart\", disableBodyScroll);\n            element.addEventListener(\"touchend\", enableBodyScroll);\n        }\n        return () => {\n            if (element) {\n                element.removeEventListener(\"touchstart\", disableBodyScroll);\n                element.removeEventListener(\"touchend\", enableBodyScroll);\n            }\n            enableBodyScroll(); // Ensure scrolling is re-enabled on unmount\n        };\n    }, [ref]);\n    return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_7__[\"default\"], { ref: setRef, sx: {\n            position: \"relative\",\n            my: 1,\n            mx: \"auto\",\n            py: 2,\n            width: \"max-content\",\n        }, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_layout__WEBPACK_IMPORTED_MODULE_3__.Layout, { sx: { height: 290, mt: 2 }, onKeyDown: keyDownHandler, onKeyUp: keyUpHandler }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"canvas\", { style: {\n                    backgroundColor: \"black\",\n                    imageRendering: \"crisp-edges\",\n                    width: \"240px\",\n                    height: \"240px\",\n                    position: \"absolute\",\n                    top: \"16px\",\n                    left: \"50%\",\n                    transform: \"translateX(-50%)\",\n                }, ref: setCanvas, onTouchStart: handleTouchStart, onTouchEnd: handleTouchEnd })] }));\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/display/index.tsx?");

/***/ }),

/***/ "./src/display/layout.tsx":
/*!********************************!*\
  !*** ./src/display/layout.tsx ***!
  \********************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   Layout: () => (/* binding */ Layout)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/system/esm/styled/styled.js\");\n/* harmony import */ var _mui_system__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! @mui/system */ \"./node_modules/@mui/system/esm/styleFunctionSx/styleFunctionSx.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__);\n\n\n\nconst Svg = (0,_mui_system__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(\"svg\")(_mui_system__WEBPACK_IMPORTED_MODULE_3__[\"default\"]);\nconst LayoutIcon = (0,_mui_system__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(\"path\") `\n  fill: ${({ theme }) => theme.palette.background.paper};\n  pointer-events: none;\n`;\nconst LayoutButton = (0,_mui_system__WEBPACK_IMPORTED_MODULE_2__[\"default\"])(\"path\") `\n  fill: ${({ theme }) => theme.palette.text.disabled};\n  cursor: pointer;\n\n  &:active {\n    fill: ${({ theme }) => theme.palette.action.disabled};\n  }\n`;\nfunction Layout({ sx, onKeyDown, onKeyUp }) {\n    function handleMouseDown(key) {\n        return ({ target: originalTarget }) => {\n            const mouseUpHandler = ({ target }) => {\n                if (target === originalTarget) {\n                    onKeyDown(key);\n                    onKeyUp(key);\n                }\n                window.removeEventListener(\"mouseup\", mouseUpHandler);\n            };\n            window.addEventListener(\"mouseup\", mouseUpHandler);\n        };\n    }\n    return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Svg, { viewBox: \"0 0 96.043732 76.464592\", version: \"1.1\", xmlns: \"http://www.w3.org/2000/svg\", sx: sx, children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(\"g\", { transform: \"translate(-56.885418,-110.86042)\", strokeWidth: \"0.264583\", children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { id: \"L1\", fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 56.885418,123.6927 c 0,-2.41106 1.954556,-4.36562 4.365625,-4.36562 h 7.441406 c 0.338138,0 0.507233,0 0.647303,0.0307 0.504243,0.11063 0.898049,0.50443 1.008671,1.00868 0.03075,0.14007 0.03075,0.30916 0.03075,0.6473 v 5.35781 c 0,0.33814 0,0.50723 -0.03075,0.6473 -0.110622,0.50425 -0.504428,0.89805 -1.008671,1.00868 -0.14007,0.0307 -0.309165,0.0307 -0.647303,0.0307 h -7.441406 c -2.411069,0 -4.365625,-1.95456 -4.365625,-4.36563 z\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_L1) }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { id: \"L2\", fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 56.885418,138.77396 c 0,-2.41107 1.954556,-4.36562 4.365625,-4.36562 h 7.441406 c 0.338138,0 0.507233,0 0.647303,0.0307 0.504243,0.11062 0.898049,0.50443 1.008671,1.00867 0.03075,0.14007 0.03075,0.30917 0.03075,0.64731 v 5.35781 c 0,0.33813 0,0.5072 -0.03075,0.64743 -0.110622,0.50403 -0.504428,0.898 -1.008671,1.00859 -0.14007,0.0307 -0.309165,0.0307 -0.647303,0.0307 h -7.441406 c -2.411069,0 -4.365625,-1.95448 -4.365625,-4.36563 z\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_L2) }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { id: \"L3\", fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 56.885418,153.85522 c 0,-2.41115 1.954556,-4.36563 4.365625,-4.36563 h 7.441406 c 0.338138,0 0.507233,0 0.647303,0.0307 0.504243,0.1106 0.898049,0.50456 1.008671,1.0086 0.03075,0.14023 0.03075,0.30929 0.03075,0.64743 v 5.35781 c 0,0.33814 0,0.50721 -0.03075,0.64744 -0.110622,0.50403 -0.504428,0.898 -1.008671,1.00859 -0.14007,0.0307 -0.309165,0.0307 -0.647303,0.0307 h -7.441406 c -2.411069,0 -4.365625,-1.95447 -4.365625,-4.36562 z\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_L3) }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(\"g\", { id: \"Home\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_HOME), children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 137.31875,176.12678 c 0,3.28322 0,4.92496 -0.49425,6.23438 -0.7784,2.06296 -2.40665,3.6912 -4.4696,4.46961 -1.30942,0.49424 -2.95116,0.49424 -6.23438,0.49424 h -15.52469 c -0.74083,0 -1.11125,0 -1.39435,-0.1442 -0.24898,-0.12673 -0.45138,-0.32914 -0.57812,-0.57811 -0.1442,-0.28311 -0.1442,-0.65352 -0.1442,-1.39436 v -10.05416 c 0,-0.74084 0,-1.11125 0.1442,-1.39436 0.12674,-0.24897 0.32914,-0.45138 0.57812,-0.57811 0.2831,-0.1442 0.65352,-0.1442 1.39435,-0.1442 h 23.63364 c 0.90593,0 1.35864,0 1.72006,0.13626 0.56885,0.21484 1.01811,0.6641 1.23296,1.23296 0.13626,0.36142 0.13626,0.81412 0.13626,1.72005 z\" }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutIcon, { d: \"m 125.08918,180.04526 -0.38471,-0.38497 -1.53961,-1.53934 c -0.036,-0.036 -0.0849,-0.0564 -0.136,-0.0564 -0.0511,0 -0.10001,0.0204 -0.13599,0.0564 l -1.53935,1.53934 -0.38497,0.38497 c -0.0349,0.0362 -0.0545,0.0849 -0.054,0.13521 5.3e-4,0.0505 0.0206,0.0987 0.0564,0.1344 0.0357,0.0357 0.0839,0.0561 0.1344,0.0564 0.0505,5.3e-4 0.099,-0.0188 0.13547,-0.054 l 0.0564,-0.0563 v 1.54913 c 0,0.12939 0.0516,0.25321 0.14287,0.34476 0.0915,0.0915 0.21564,0.14287 0.34502,0.14314 h 2.48761 c 0.12938,-2.7e-4 0.25347,-0.0516 0.34502,-0.14314 0.0915,-0.0916 0.14287,-0.21537 0.14314,-0.34476 v -1.54913 l 0.0563,0.0563 c 0.0362,0.0352 0.0849,0.0545 0.13521,0.054 0.0505,-2.6e-4 0.0987,-0.0206 0.1344,-0.0564 0.0357,-0.0357 0.0561,-0.0839 0.0564,-0.1344 5.3e-4,-0.0503 -0.0188,-0.099 -0.054,-0.13521 z\", fillOpacity: \"0.8\" })] }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(\"g\", { id: \"Back\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_BACK), children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 73.81875,176.12678 c 0,3.28322 0,4.92496 0.494189,6.23438 0.778483,2.06296 2.406755,3.6912 4.469579,4.46961 1.309503,0.49424 2.951242,0.49424 6.234457,0.49424 h 15.855415 c 0.74084,0 1.11125,0 1.39436,-0.1442 0.24897,-0.12673 0.45138,-0.32914 0.57811,-0.57811 0.1442,-0.28311 0.1442,-0.65352 0.1442,-1.39436 v -10.05416 c 0,-0.74084 0,-1.11125 -0.1442,-1.39436 -0.12673,-0.24897 -0.32914,-0.45138 -0.57811,-0.57811 -0.28311,-0.1442 -0.65352,-0.1442 -1.39436,-0.1442 H 76.907945 c -0.905748,0 -1.358635,0 -1.719871,0.13626 -0.569066,0.21484 -1.018249,0.6641 -1.233011,1.23296 -0.136313,0.36142 -0.136313,0.81412 -0.136313,1.72005 z\" }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutIcon, { fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 87.233389,180.44875 -0.802216,-0.91466 h 3.113087 c 1.460235,0 1.540669,2.08624 0.240771,2.32701 -0.256911,0.032 -0.208756,0.48128 0.06403,0.43312 1.813454,-0.28866 1.717146,-3.20939 -0.3048,-3.20939 h -3.113088 l 0.802216,-0.91467 c 0.192617,-0.22463 -0.128323,-0.51356 -0.320939,-0.28892 l -1.139296,1.28375 c -0.06429,0.0804 -0.06429,0.20876 0,0.28893 l 1.139296,1.2999 c 0.176477,0.20849 0.529695,-0.0802 0.320939,-0.30507 z\", fillOpacity: \"0.8\" })] }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(\"g\", { id: \"Down\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_DOWN), children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 146.84374,165.62917 c -3.36101,0 -6.08542,-2.72442 -6.08542,-6.08542 v -16.40416 c 0,-0.74084 0,-1.11125 0.1442,-1.39436 0.12673,-0.24897 0.32914,-0.45138 0.57811,-0.57811 0.28311,-0.1442 0.65352,-0.1442 1.39436,-0.1442 h 7.9375 c 0.74083,0 1.11125,0 1.39435,0.1442 0.24897,0.12673 0.45138,0.32914 0.57811,0.57811 0.1442,0.28311 0.1442,0.65352 0.1442,1.39436 v 16.40416 c 0,3.361 -2.72441,6.08542 -6.08541,6.08542 z\" }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutIcon, { d: \"m 146.84797,154.2505 c 0.0717,-0.007 0.13943,-0.0357 0.1942,-0.0828 l 1.46579,-1.2565 c 0.0365,-0.0294 0.0669,-0.0656 0.0892,-0.10663 0.0222,-0.0413 0.0357,-0.0865 0.0402,-0.13308 0.004,-0.0466 -8e-4,-0.0937 -0.0148,-0.13812 -0.0143,-0.0447 -0.037,-0.086 -0.0675,-0.12144 -0.0304,-0.0357 -0.0677,-0.0648 -0.10954,-0.0855 -0.0421,-0.0209 -0.0876,-0.0331 -0.13441,-0.036 -0.0466,-0.003 -0.0934,0.004 -0.13758,0.019 -0.0442,0.0156 -0.0847,0.04 -0.11906,0.0714 l -1.23931,1.06257 -1.23904,-1.06257 c -0.0344,-0.0315 -0.0751,-0.0558 -0.11907,-0.0714 -0.0442,-0.0153 -0.091,-0.022 -0.13784,-0.019 -0.0466,0.003 -0.0923,0.0151 -0.13415,0.036 -0.0418,0.0206 -0.0791,0.0497 -0.10953,0.0855 -0.0304,0.0354 -0.0534,0.0767 -0.0675,0.12144 -0.0143,0.0444 -0.0193,0.0915 -0.0148,0.13812 0.004,0.0466 0.018,0.0918 0.0402,0.13308 0.0222,0.041 0.0524,0.0773 0.0889,0.10663 l 1.46605,1.2565 c 0.0355,0.0304 0.0767,0.0537 0.12145,0.0677 0.0445,0.0143 0.0915,0.0196 0.13811,0.0151 z\" })] }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(\"g\", { id: \"Up\", onMouseDown: handleMouseDown(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Key.KEY_UP), children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutButton, { fillRule: \"evenodd\", clipRule: \"evenodd\", d: \"m 146.84374,110.86042 c 3.361,0 6.08541,2.72452 6.08541,6.08542 V 133.35 c 0,0.74092 0,1.11136 -0.1442,1.39436 -0.12673,0.24892 -0.32914,0.4513 -0.57811,0.57811 -0.2831,0.1442 -0.65352,0.1442 -1.39435,0.1442 h -7.9375 c -0.74084,0 -1.11125,0 -1.39436,-0.1442 -0.24897,-0.12681 -0.45138,-0.32919 -0.57811,-0.57811 -0.1442,-0.283 -0.1442,-0.65344 -0.1442,-1.39436 v -16.40416 c 0,-3.36087 2.72441,-6.08542 6.08542,-6.08542 z\" }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LayoutIcon, { d: \"m 146.78262,122.23898 c -0.0717,0.007 -0.1397,0.0359 -0.19421,0.0829 l -1.46605,1.25653 c -0.0365,0.0293 -0.0667,0.0656 -0.0889,0.10676 -0.0222,0.0412 -0.036,0.0864 -0.0402,0.13301 -0.004,0.0466 5.3e-4,0.0936 0.0148,0.13816 0.014,0.0446 0.037,0.0859 0.0675,0.12145 0.0304,0.0355 0.0677,0.0646 0.10953,0.0855 0.0418,0.0209 0.0876,0.0331 0.13415,0.0359 0.0468,0.003 0.0937,-0.004 0.13784,-0.0193 0.0439,-0.0156 0.0847,-0.0399 0.11907,-0.0714 l 1.23904,-1.06238 1.23904,1.06238 c 0.0347,0.0316 0.0751,0.0559 0.11933,0.0714 0.0442,0.0156 0.091,0.0221 0.13758,0.0193 0.0468,-0.003 0.0923,-0.0151 0.13441,-0.0359 0.0418,-0.0209 0.0791,-0.0499 0.10954,-0.0855 0.0304,-0.0356 0.0532,-0.0768 0.0675,-0.12145 0.014,-0.0446 0.019,-0.0916 0.0148,-0.13816 -0.005,-0.0466 -0.018,-0.0918 -0.0402,-0.13301 -0.0222,-0.0412 -0.0526,-0.0775 -0.0892,-0.10676 l -1.46579,-1.25653 c -0.0354,-0.0305 -0.0767,-0.0536 -0.12144,-0.0678 -0.0444,-0.0142 -0.0915,-0.0194 -0.13811,-0.0151 z\" })] })] }) }));\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/display/layout.tsx?");

/***/ }),

/***/ "./src/display/store-event-handler.ts":
/*!********************************************!*\
  !*** ./src/display/store-event-handler.ts ***!
  \********************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   audioContext: () => (/* binding */ audioContext),\n/* harmony export */   subscribeToStoreEvents: () => (/* binding */ subscribeToStoreEvents)\n/* harmony export */ });\n/* harmony import */ var fflate__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! fflate */ \"./node_modules/fflate/esm/browser.js\");\n/* harmony import */ var _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ../bindings/store/v1/store_pb */ \"./src/bindings/store/v1/store_pb.js\");\n/* harmony import */ var _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_0__);\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ../bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__);\n\n\n\nfunction requestRedraw(store) {\n    const dispatchActionRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_0__.DispatchActionRequest();\n    const action = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Action();\n    dispatchActionRequest.setAction(action);\n    const displayRedrawAction = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.DisplayRedrawAction();\n    action.setDisplayRedrawAction(displayRedrawAction);\n    store.dispatchAction(dispatchActionRequest);\n}\nfunction subscribeToRenderEvents(store, canvas) {\n    const event = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Event();\n    event.setDisplayCompressedRenderEvent(new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.DisplayCompressedRenderEvent());\n    const subscribeEventRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_0__.SubscribeEventRequest();\n    subscribeEventRequest.setEvent(event);\n    const stream = store.subscribeEvent(subscribeEventRequest);\n    let context = canvas?.getContext(\"2d\");\n    stream.on(\"error\", () => setTimeout(() => subscribeToRenderEvents(store, canvas), 1000));\n    stream.on(\"data\", (response) => {\n        const renderEvent = response.getEvent()?.getDisplayCompressedRenderEvent();\n        if (!renderEvent || !canvas) {\n            return;\n        }\n        const compressedData = renderEvent.getCompressedData_asU8();\n        const rectangle = renderEvent.getRectangleList();\n        if (!compressedData || !rectangle) {\n            return;\n        }\n        const width = Math.round(240 * renderEvent.getDensity());\n        const height = Math.round(240 * renderEvent.getDensity());\n        if (width !== canvas.width || height !== canvas.height) {\n            canvas.width = width;\n            canvas.height = height;\n            context = canvas.getContext(\"2d\");\n        }\n        (0,fflate__WEBPACK_IMPORTED_MODULE_2__.inflate)(compressedData, (error, data) => {\n            if (error) {\n                console.error(error);\n                return;\n            }\n            if (context && data) {\n                const [y1, x1, y2, x2] = rectangle;\n                const [width, height] = [x2 - x1, y2 - y1];\n                context.putImageData(new ImageData(new Uint8ClampedArray(data), width, height), x1, y1, 0, 0, width, height);\n            }\n        });\n    });\n    requestRedraw(store);\n}\nconst audioContext = new AudioContext();\nfunction createWavFile(samples, sampleRate, numChannels, bitsPerSample) {\n    const header = new ArrayBuffer(44);\n    const view = new DataView(header);\n    /* Write WAV file header */\n    const blockAlign = (numChannels * bitsPerSample) / 8;\n    const byteRate = sampleRate * blockAlign;\n    const dataSize = samples.length;\n    // 'RIFF' chunk descriptor\n    writeString(view, 0, \"RIFF\");\n    view.setUint32(4, 36 + dataSize, true); // File size minus first 8 bytes\n    writeString(view, 8, \"WAVE\");\n    // 'fmt ' sub-chunk\n    writeString(view, 12, \"fmt \");\n    view.setUint32(16, 16, true); // SubChunk1Size for PCM\n    view.setUint16(20, 1, true); // AudioFormat (1 = PCM)\n    view.setUint16(22, numChannels, true);\n    view.setUint32(24, sampleRate, true);\n    view.setUint32(28, byteRate, true);\n    view.setUint16(32, blockAlign, true);\n    view.setUint16(34, bitsPerSample, true);\n    // 'data' sub-chunk\n    writeString(view, 36, \"data\");\n    view.setUint32(40, dataSize, true);\n    function writeString(view, offset, string) {\n        for (let i = 0; i < string.length; i++) {\n            view.setUint8(offset + i, string.charCodeAt(i));\n        }\n    }\n    const wavBuffer = new Uint8Array(header.byteLength + samples.length);\n    wavBuffer.set(new Uint8Array(header), 0);\n    wavBuffer.set(samples, header.byteLength);\n    return new Blob([wavBuffer], { type: \"audio/wav\" });\n}\nfunction subscribeToAudioEvents(store) {\n    const event = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.Event();\n    event.setAudioPlayAudioSampleEvent(new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_1__.AudioPlayAudioSampleEvent());\n    const subscribeEventRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_0__.SubscribeEventRequest();\n    subscribeEventRequest.setEvent(event);\n    const stream = store.subscribeEvent(subscribeEventRequest);\n    stream.on(\"error\", () => setTimeout(() => subscribeToAudioEvents(store), 1000));\n    stream.on(\"data\", async (response) => {\n        const audioEvent = response.getEvent()?.getAudioPlayAudioSampleEvent();\n        if (!audioEvent) {\n            return;\n        }\n        const audioSample = audioEvent.setSample().getSample();\n        if (!audioSample) {\n            return;\n        }\n        const data = audioSample.getData_asU8();\n        const rate = audioSample.getRate();\n        const width = audioSample.getWidth();\n        const channels = audioSample.getChannels();\n        const volume = audioEvent.getVolume();\n        const audioBlob = createWavFile(data, rate, channels, width * 8);\n        const arrayBuffer = await audioBlob.arrayBuffer();\n        const audioBuffer = await audioContext.decodeAudioData(arrayBuffer);\n        const source = audioContext.createBufferSource();\n        source.buffer = audioBuffer;\n        const gainNode = audioContext.createGain();\n        gainNode.gain.value = volume;\n        source.connect(gainNode);\n        gainNode.connect(audioContext.destination);\n        if (audioContext.state === \"suspended\") {\n            await audioContext.resume();\n        }\n        source.start(audioContext.currentTime + 0.1);\n    });\n}\nfunction subscribeToStoreEvents(store, canvas) {\n    subscribeToRenderEvents(store, canvas);\n    subscribeToAudioEvents(store);\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/display/store-event-handler.ts?");

/***/ }),

/***/ "./src/inputs.tsx":
/*!************************!*\
  !*** ./src/inputs.tsx ***!
  \************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   Inputs: () => (/* binding */ Inputs)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _mui_icons_material__WEBPACK_IMPORTED_MODULE_20__ = __webpack_require__(/*! @mui/icons-material */ \"./node_modules/@mui/icons-material/esm/Clear.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Dialog/Dialog.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/DialogTitle/DialogTitle.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/DialogContent/DialogContent.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Stack/Stack.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/FormControl/FormControl.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_10__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/InputLabel/InputLabel.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_11__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Select/Select.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_12__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/MenuItem/MenuItem.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_13__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/FormHelperText/FormHelperText.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_14__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/TextField/TextField.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_15__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/FormControlLabel/FormControlLabel.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_16__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Switch/Switch.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_17__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Button/Button.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_18__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Typography/Typography.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_19__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/IconButton/IconButton.js\");\n/* harmony import */ var _mui_material__WEBPACK_IMPORTED_MODULE_21__ = __webpack_require__(/*! @mui/material */ \"./node_modules/@mui/material/Divider/Divider.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react */ \"./node_modules/react/index.js\");\n/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);\n/* harmony import */ var _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./bindings/store/v1/store_pb */ \"./src/bindings/store/v1/store_pb.js\");\n/* harmony import */ var _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_2__);\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(/*! ./bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__);\n/* harmony import */ var _types__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(/*! ./types */ \"./src/types.ts\");\n\n\n\n\n\n\n\nfunction Inputs({ inputs, isGrpcConnected, store, }) {\n    const [files, setFiles] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)({});\n    async function handleSubmit(event) {\n        const form = event.currentTarget;\n        const formData = new FormData(form);\n        const id = formData.get(\"id\");\n        const input = inputs.find((input) => input.id === id);\n        if (input?.fields) {\n            for (const field of input.fields.itemsList) {\n                if (field.pattern) {\n                    if (field.type !== _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputFieldType.INPUT_FIELD_TYPE_FILE) {\n                        const value = formData.get(field.name);\n                        if (value && !new RegExp(`^${field.pattern}$`).test(value)) {\n                            alert(`The value for \"${field.label}\" does not match the required pattern!`);\n                            event.preventDefault();\n                            return;\n                        }\n                    }\n                }\n            }\n        }\n        if (!store || !isGrpcConnected) {\n            return;\n        }\n        event.preventDefault();\n        const value = (formData.get(\"value\") || \"\");\n        const action = event.nativeEvent.submitter?.value;\n        if (action === \"provide\") {\n            const inputResult = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputResult();\n            inputResult.setMethod(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputMethod.INPUT_METHOD_WEB_DASHBOARD);\n            const dataMap = inputResult.getDataMap();\n            const fileMap = inputResult.getFilesMap();\n            for (const [name, value] of formData.entries()) {\n                if (![\"id\", \"value\", \"action\"].includes(name)) {\n                    if (value instanceof File) {\n                        fileMap.set(name, await value.arrayBuffer());\n                    }\n                    else {\n                        dataMap.set(name, value);\n                    }\n                }\n            }\n            const inputProvideAction = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputProvideAction();\n            inputProvideAction.setId(id);\n            inputProvideAction.setValue(value);\n            inputProvideAction.setResult(inputResult);\n            const action = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.Action();\n            action.setInputProvideAction(inputProvideAction);\n            const dispatchActionRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_2__.DispatchActionRequest();\n            dispatchActionRequest.setAction(action);\n            await store.dispatchAction(dispatchActionRequest);\n        }\n        else if (action === \"cancel\") {\n            const inputCancelAction = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputCancelAction();\n            inputCancelAction.setId(id);\n            const action = new _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.Action();\n            action.setInputCancelAction(inputCancelAction);\n            const dispatchActionRequest = new _bindings_store_v1_store_pb__WEBPACK_IMPORTED_MODULE_2__.DispatchActionRequest();\n            dispatchActionRequest.setAction(action);\n            await store.dispatchAction(dispatchActionRequest);\n        }\n    }\n    return inputs.map((input, index) => {\n        const id = input.id;\n        if (!id) {\n            console.warn(\"Input description without ID:\", input);\n            return null;\n        }\n        return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_5__[\"default\"], { open: true, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_6__[\"default\"], { children: input.prompt }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_7__[\"default\"], { sx: { \"&&.MuiDialogContent-root\": { pt: 1 } }, children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_8__[\"default\"], { component: \"form\", autoComplete: \"off\", method: \"POST\", encType: \"multipart/form-data\", gap: 2, onSubmit: handleSubmit, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"input\", { name: \"id\", type: \"hidden\", value: id }), input.fields?.itemsList.length ? (input.fields.itemsList.map((field) => field.type === _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputFieldType.INPUT_FIELD_TYPE_SELECT ? ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_9__[\"default\"], { fullWidth: true, required: field.required, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_10__[\"default\"], { htmlFor: field.name, children: field.label }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_11__[\"default\"], { id: field.name, name: field.name, label: field.label, defaultValue: field.defaultValue || \"\", displayEmpty: true, children: field.options?.itemsList?.map((option) => ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_12__[\"default\"], { value: option, children: option }, option))) }), field.description && ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_13__[\"default\"], { children: field.description }))] }, field.name)) : field.type === _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputFieldType.INPUT_FIELD_TYPE_LONG ? ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_14__[\"default\"], { name: field.name, label: field.label, helperText: field.description, defaultValue: field.defaultValue || undefined, title: field.title || undefined, multiline: true, minRows: 4, slotProps: {\n                                    htmlInput: {\n                                        pattern: field.pattern || undefined,\n                                    },\n                                }, required: field.required, fullWidth: true }, field.name)) : field.type === _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputFieldType.INPUT_FIELD_TYPE_CHECKBOX ? ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_9__[\"default\"], { fullWidth: true, required: field.required, title: field.title || undefined, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_15__[\"default\"], { label: field.label, control: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_16__[\"default\"], { name: field.name, required: field.required, defaultValue: field.defaultValue || undefined }) }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_13__[\"default\"], { children: field.description })] }, field.name)) : field.type === _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_3__.InputFieldType.INPUT_FIELD_TYPE_FILE ? ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_9__[\"default\"], { fullWidth: true, required: field.required, title: field.title || undefined, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_10__[\"default\"], { htmlFor: field.name, children: field.label }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_8__[\"default\"], { direction: \"row\", spacing: 1, width: \"100%\", mt: 5, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_17__[\"default\"], { component: \"label\", variant: files[field.name] ? \"contained\" : \"outlined\", sx: { flexGrow: 1, textTransform: \"none\" }, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"input\", { type: \"file\", id: field.name, name: field.name, required: field.required, accept: field.fileMimetype || \"*\", onChange: async (event) => {\n                                                            if (event.target.files?.length) {\n                                                                const file = event.target.files[0];\n                                                                const content = await file.text();\n                                                                setFiles((files) => {\n                                                                    if (!field.pattern ||\n                                                                        new RegExp(field.pattern).test(content)) {\n                                                                        return {\n                                                                            ...files,\n                                                                            [id]: {\n                                                                                ...files[id],\n                                                                                [field.name]: file,\n                                                                            },\n                                                                        };\n                                                                    }\n                                                                    else {\n                                                                        alert(`The file \"${file.name}\" does not match the required pattern!`);\n                                                                        event.target.value = \"\";\n                                                                        const newFiles = files[id] || {};\n                                                                        delete newFiles[field.name];\n                                                                        return { ...files, [id]: newFiles };\n                                                                    }\n                                                                });\n                                                            }\n                                                            else {\n                                                                const newFiles = files[id] || {};\n                                                                delete newFiles[field.name];\n                                                                return { ...files, [id]: newFiles };\n                                                            }\n                                                        }, hidden: true }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_18__[\"default\"], { variant: \"body2\", children: files[id]?.[field.name]\n                                                            ? files[id][field.name].name\n                                                            : \"Select a file\" })] }), files[field.name] && ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_19__[\"default\"], { sx: { flexGrow: 0, flexShrink: 0 }, onClick: (event) => {\n                                                    const input = event.currentTarget.previousElementSibling?.querySelector(\"input\");\n                                                    input.value = \"\";\n                                                    setFiles((files) => {\n                                                        const newFiles = { ...files };\n                                                        delete newFiles[field.name];\n                                                        return newFiles;\n                                                    });\n                                                }, children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_icons_material__WEBPACK_IMPORTED_MODULE_20__[\"default\"], {}) }))] }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_13__[\"default\"], { children: field.description })] }, field.name)) : ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_14__[\"default\"], { type: _types__WEBPACK_IMPORTED_MODULE_4__.inputFieldTypes[field.type], name: field.name, label: field.label, helperText: field.description, defaultValue: field.defaultValue || undefined, title: field.title || undefined, required: field.required, fullWidth: true }, field.name)))) : ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_9__[\"default\"], { fullWidth: true, children: (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_14__[\"default\"], { type: \"text\", name: \"value\", fullWidth: true }) })), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(_mui_material__WEBPACK_IMPORTED_MODULE_8__[\"default\"], { direction: \"row\", spacing: 2, children: [(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_17__[\"default\"], { type: \"submit\", name: \"action\", value: \"provide\", variant: \"contained\", color: \"primary\", children: \"Provide\" }), (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_17__[\"default\"], { type: \"submit\", name: \"action\", value: \"cancel\", variant: \"outlined\", formNoValidate: true, children: \"Cancel\" })] }), index < inputs.length - 1 && (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_mui_material__WEBPACK_IMPORTED_MODULE_21__[\"default\"], {})] }) })] }, id));\n    });\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/inputs.tsx?");

/***/ }),

/***/ "./src/main-view.tsx":
/*!***************************!*\
  !*** ./src/main-view.tsx ***!
  \***************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   MainView: () => (/* binding */ MainView)\n/* harmony export */ });\n/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ \"./node_modules/react/jsx-runtime.js\");\n/* harmony import */ var _components_Status__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! ./components/Status */ \"./src/components/Status.tsx\");\n/* harmony import */ var _display__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(/*! ./display */ \"./src/display/index.tsx\");\n\n\n\nfunction request(action) {\n    fetch(\"/action/\", {\n        method: \"POST\",\n        headers: {\n            \"Content-Type\": \"application/json\",\n        },\n        body: JSON.stringify({ action }),\n    });\n}\nfunction MainView({ status, store, }) {\n    if (!status) {\n        return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { severity: \"info\", message: \"Waiting for the server to report its status\" }));\n    }\n    if (!store) {\n        return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { severity: \"info\", message: \"Waiting for the store to be initialized\" }));\n    }\n    if (status.status === \"ok\") {\n        if (status.docker === \"running\") {\n            if (status.envoy === \"running\") {\n                return (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_display__WEBPACK_IMPORTED_MODULE_2__.Display, { store: store });\n            }\n            else if (status.envoy === \"not downloaded\") {\n                return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Envoy image is not downloaded.\", action: {\n                        callback: () => request(\"download envoy\"),\n                        label: \"Download Envoy\",\n                    } }));\n            }\n            else if (status.envoy === \"not running\") {\n                return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Envoy container is not running.\", action: {\n                        callback: () => request(\"run envoy\"),\n                        label: \"Run Envoy\",\n                    } }));\n            }\n            else if (status.envoy === \"failed\") {\n                return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Failed to get envoy status.\", severity: \"error\", action: {\n                        callback: () => request(\"remove envoy\"),\n                        label: \"Stop and remove container\",\n                    } }));\n            }\n            else if (status.envoy === \"unknown\") {\n                return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Envoy status: Unknown\", severity: \"error\", action: {\n                        callback: () => request(\"remove envoy\"),\n                        label: \"Stop and remove container\",\n                    } }));\n            }\n        }\n        else if (status.docker === \"not ready\") {\n            return (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Docker not ready yet, please wait...\" });\n        }\n        else if (status.docker === \"not installed\") {\n            return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Docker is not installed.\", action: {\n                    callback: () => request(\"install docker\"),\n                    label: \"Install Docker\",\n                } }));\n        }\n        else if (status.docker === \"not running\") {\n            return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Docker service is not running.\", action: {\n                    callback: () => request(\"run docker\"),\n                    label: \"Run Docker Service\",\n                } }));\n        }\n        else if (status.docker === \"failed\") {\n            return ((0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Failed to get docker status.\", severity: \"error\", action: {\n                    callback: () => request(\"stop docker\"),\n                    label: \"Stop Docker Service\",\n                } }));\n        }\n        else if (status.docker === \"unknown\") {\n            (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_components_Status__WEBPACK_IMPORTED_MODULE_1__.Status, { message: \"Docker status: Unknown\", severity: \"error\", action: {\n                    callback: () => request(\"stop docker\"),\n                    label: \"Stop Docker Service\",\n                } });\n        }\n    }\n    else {\n        return (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(\"div\", { children: \"Server is not ready\" });\n    }\n}\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/main-view.tsx?");

/***/ }),

/***/ "./src/types.ts":
/*!**********************!*\
  !*** ./src/types.ts ***!
  \**********************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

eval("__webpack_require__.r(__webpack_exports__);\n/* harmony export */ __webpack_require__.d(__webpack_exports__, {\n/* harmony export */   inputFieldTypes: () => (/* binding */ inputFieldTypes)\n/* harmony export */ });\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! ./bindings/ubo/v1/ubo_pb */ \"./src/bindings/ubo/v1/ubo_pb.js\");\n/* harmony import */ var _bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__);\n\nconst inputFieldTypes = {\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_LONG]: \"long\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_TEXT]: \"text\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_PASSWORD]: \"password\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_NUMBER]: \"number\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_CHECKBOX]: \"checkbox\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_COLOR]: \"color\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_SELECT]: \"select\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_FILE]: \"file\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_DATE]: \"date\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_TIME]: \"time\",\n    [_bindings_ubo_v1_ubo_pb__WEBPACK_IMPORTED_MODULE_0__.InputFieldType.INPUT_FIELD_TYPE_UBO_APP_DOT_STORE_DOT_INPUT_DOT_TYPES_UNSPECIFIED]: \"unspecified\",\n};\n\n\n//# sourceURL=webpack://ubo-rpc-client-grpc-web/./src/types.ts?");

/***/ })

/******/ });
/************************************************************************/
/******/ // The module cache
/******/ var __webpack_module_cache__ = {};
/******/ 
/******/ // The require function
/******/ function __webpack_require__(moduleId) {
/******/ 	// Check if module is in cache
/******/ 	var cachedModule = __webpack_module_cache__[moduleId];
/******/ 	if (cachedModule !== undefined) {
/******/ 		return cachedModule.exports;
/******/ 	}
/******/ 	// Create a new module (and put it into the cache)
/******/ 	var module = __webpack_module_cache__[moduleId] = {
/******/ 		id: moduleId,
/******/ 		loaded: false,
/******/ 		exports: {}
/******/ 	};
/******/ 
/******/ 	// Execute the module function
/******/ 	__webpack_modules__[moduleId].call(module.exports, module, module.exports, __webpack_require__);
/******/ 
/******/ 	// Flag the module as loaded
/******/ 	module.loaded = true;
/******/ 
/******/ 	// Return the exports of the module
/******/ 	return module.exports;
/******/ }
/******/ 
/******/ // expose the modules object (__webpack_modules__)
/******/ __webpack_require__.m = __webpack_modules__;
/******/ 
/************************************************************************/
/******/ /* webpack/runtime/compat get default export */
/******/ (() => {
/******/ 	// getDefaultExport function for compatibility with non-harmony modules
/******/ 	__webpack_require__.n = (module) => {
/******/ 		var getter = module && module.__esModule ?
/******/ 			() => (module['default']) :
/******/ 			() => (module);
/******/ 		__webpack_require__.d(getter, { a: getter });
/******/ 		return getter;
/******/ 	};
/******/ })();
/******/ 
/******/ /* webpack/runtime/define property getters */
/******/ (() => {
/******/ 	// define getter functions for harmony exports
/******/ 	__webpack_require__.d = (exports, definition) => {
/******/ 		for(var key in definition) {
/******/ 			if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 				Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 			}
/******/ 		}
/******/ 	};
/******/ })();
/******/ 
/******/ /* webpack/runtime/global */
/******/ (() => {
/******/ 	__webpack_require__.g = (function() {
/******/ 		if (typeof globalThis === 'object') return globalThis;
/******/ 		try {
/******/ 			return this || new Function('return this')();
/******/ 		} catch (e) {
/******/ 			if (typeof window === 'object') return window;
/******/ 		}
/******/ 	})();
/******/ })();
/******/ 
/******/ /* webpack/runtime/hasOwnProperty shorthand */
/******/ (() => {
/******/ 	__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ })();
/******/ 
/******/ /* webpack/runtime/make namespace object */
/******/ (() => {
/******/ 	// define __esModule on exports
/******/ 	__webpack_require__.r = (exports) => {
/******/ 		if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
/******/ 			Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
/******/ 		}
/******/ 		Object.defineProperty(exports, '__esModule', { value: true });
/******/ 	};
/******/ })();
/******/ 
/******/ /* webpack/runtime/node module decorator */
/******/ (() => {
/******/ 	__webpack_require__.nmd = (module) => {
/******/ 		module.paths = [];
/******/ 		if (!module.children) module.children = [];
/******/ 		return module;
/******/ 	};
/******/ })();
/******/ 
/******/ /* webpack/runtime/publicPath */
/******/ (() => {
/******/ 	var scriptUrl;
/******/ 	if (typeof import.meta.url === "string") scriptUrl = import.meta.url
/******/ 	// When supporting browsers where an automatic publicPath is not supported you must specify an output.publicPath manually via configuration
/******/ 	// or pass an empty string ("") and set the __webpack_public_path__ variable from your code to use your own logic.
/******/ 	if (!scriptUrl) throw new Error("Automatic publicPath is not supported in this browser");
/******/ 	scriptUrl = scriptUrl.replace(/^blob:/, "").replace(/#.*$/, "").replace(/\?.*$/, "").replace(/\/[^\/]+$/, "/");
/******/ 	__webpack_require__.p = scriptUrl;
/******/ })();
/******/ 
/******/ /* webpack/runtime/import chunk loading */
/******/ (() => {
/******/ 	__webpack_require__.b = new URL("./", import.meta.url);
/******/ 	
/******/ 	// object to store loaded and loading chunks
/******/ 	// undefined = chunk not loaded, null = chunk preloaded/prefetched
/******/ 	// [resolve, Promise] = chunk loading, 0 = chunk loaded
/******/ 	var installedChunks = {
/******/ 		"main": 0
/******/ 	};
/******/ 	
/******/ 	// no install chunk
/******/ 	
/******/ 	// no chunk on demand loading
/******/ 	
/******/ 	// no prefetching
/******/ 	
/******/ 	// no preloaded
/******/ 	
/******/ 	// no external install chunk
/******/ 	
/******/ 	// no on chunks loaded
/******/ })();
/******/ 
/************************************************************************/
/******/ 
/******/ // startup
/******/ // Load entry module and return exports
/******/ // This entry module can't be inlined because the eval devtool is used.
/******/ var __webpack_exports__ = __webpack_require__("./src/client.tsx");
/******/ const __webpack_exports__Root = __webpack_exports__.Root;
/******/ const __webpack_exports__init = __webpack_exports__.init;
/******/ export { __webpack_exports__Root as Root, __webpack_exports__init as init };
/******/ 
